// Copyright (C) 2024 The Qt Company Ltd.
// Copyright © 2004-2023 Unicode, Inc.
// SPDX-License-Identifier: Unicode-3.0

#ifndef QTIMEZONELOCALE_DATA_P_H
#define QTIMEZONELOCALE_DATA_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not a normal header file.
// It defines data tables for internal use by QTimeZone's L10n code.
// It may change incompatibly between versions, or even be removed.
//
// We mean it.
//

// Only qtimezonelocale.cpp should #include this (after other things it needs),
// and even that only when feature icu is disabled.

QT_REQUIRE_CONFIG(timezone_locale);
#if QT_CONFIG(icu)
#  error "This file should only be needed (or seen) when ICU is not in use"
#endif

QT_BEGIN_NAMESPACE

#ifdef QT_CLDR_ZONE_DEBUG
#  define LOCALE_TAGS(lang, script, land) lang, script, land,
#else
#  define LOCALE_TAGS(lang, script, land)
#endif

namespace QtTimeZoneLocale {

/*
    Locale-specific data for timezone naming.
    https://www.unicode.org/reports/tr35/tr35-68/tr35-dates.html#Time_Zone_Names

    Available data, per locale, from LocaleScanner.timeZoneNames():

        formats: {
            'hour': ('+HH:mm', '-HH:mm'),
            'GMT': 'GMT%0',
            'region': ("%0 Time", std, dst),
            'fallback': "%1 (%0)" }
        zones: { ianaid: {
                'exemplarCity': name,
                'long': (gen, std, dst), 'short': (gen, std, dst) } }
        metazones: { meta: {'long': (gen, std, dst), 'short': (gen, std, dst) } }

    Mapped to C++ data-structures below (defined in QTZL_p.h) and used by
    QTZL.cpp, in conjunction with QTZP_data_p.h's locale-independent data
    conditioned on timezone_locale.
*/

// GENERATED PART STARTS HERE

/*
    This part of the file was generated on 2024-12-05 from the
    Common Locale Data Repository v46

    http://www.unicode.org/cldr/

    Do not edit this section: instead regenerate it using
    cldr2qlocalexml.py and qlocalexml2cpp.py on updated (or
    edited) CLDR data; see qtbase/util/locale_database/.
*/

// Sorted by locale index, then iana name
static inline constexpr LocaleZoneExemplar localeZoneExemplarTable[] = {
    // locInd, ianaInd, xcty{ind, sz}
    {    0,   14,       0,  6, }, // Africa/Asmara C/AnyScript/AnyTerritory
    {    0, 4435,       6,  8, }, // Africa/Sao_Tome
    {    0, 4252,      14,  8, }, // America/Asuncion
    {    0,  237,      22,  8, }, // America/Atikokan
    {    0, 7623,      30, 17, }, // America/Bahia_Banderas
    {    0, 3868,      47,  6, }, // America/Cancun
    {    0, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    0, 5723,      66,  7, }, // America/Curacao
    {    0,  481,      73, 13, }, // America/Indiana/Knox
    {    0, 7961,      86, 16, }, // America/Indiana/Marengo
    {    0, 7985,     102, 19, }, // America/Indiana/Petersburg
    {    0, 8012,     121, 18, }, // America/Indiana/Tell_City
    {    0, 8038,     139, 14, }, // America/Indiana/Vevay
    {    0, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    0, 8086,     171, 16, }, // America/Indiana/Winamac
    {    0, 8125,     187, 20, }, // America/Kentucky/Monticello
    {    0, 5932,     207, 22, }, // America/Lower_Princes
    {    0, 8220,     229,  6, }, // America/Merida
    {    0, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {    0, 8329,     255, 20, }, // America/North_Dakota/Center
    {    0, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {    0,  393,     298,  4, }, // America/Nuuk
    {    0, 6899,     302, 16, }, // America/Scoresbysund
    {    0, 8508,     318, 14, }, // America/St_Barthelemy
    {    0, 2061,     332, 10, }, // America/St_Johns
    {    0, 5863,     342,  9, }, // America/St_Kitts
    {    0, 5880,     351,  9, }, // America/St_Lucia
    {    0,  855,     360, 10, }, // America/St_Thomas
    {    0, 5913,     370, 11, }, // America/St_Vincent
    {    0, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {    0, 1259,     397, 16, }, // Asia/Ho_Chi_Minh
    {    0, 1185,     413,  9, }, // Asia/Kathmandu
    {    0,  992,     422,  7, }, // Asia/Kolkata
    {    0, 1211,     429,  5, }, // Asia/Macau
    {    0, 8708,     434,  8, }, // Asia/Qostanay
    {    0, 1235,     442,  6, }, // Asia/Yangon
    {    0, 1395,     448,  5, }, // Atlantic/Faroe
    {    0, 6821,     453, 10, }, // Atlantic/St_Helena
    {    0, 2527,     463,  4, }, // Europe/Kyiv
    {    0, 7211,     467,  7, }, // Indian/Reunion
    {    0, 3157,     474,  5, }, // Pacific/Chuuk
    {    0, 3096,     479,  7, }, // Pacific/Pohnpei
    {    1, 6788,     486,  6, }, // Africa/Abidjan Abkhazian/Cyrillic/Georgia
    {    1, 6760,     492,  5, }, // Africa/Accra
    {    1, 5059,     497, 11, }, // Africa/Addis_Ababa
    {    1, 7420,     508,  5, }, // Africa/Algiers
    {    1,   14,     513,  6, }, // Africa/Asmara
    {    1,   44,     519,  6, }, // Africa/Bamako
    {    1, 5257,     525,  5, }, // Africa/Bangui
    {    1, 6746,     530,  6, }, // Africa/Banjul
    {    1, 7435,     536,  5, }, // Africa/Bissau
    {    1, 4957,     541,  8, }, // Africa/Blantyre
    {    1, 5287,     549,  9, }, // Africa/Brazzaville
    {    1, 4922,     558,  9, }, // Africa/Bujumbura
    {    1, 2320,     567,  4, }, // Africa/Cairo
    {    1, 4117,     571, 10, }, // Africa/Casablanca
    {    1, 7449,     581,  5, }, // Africa/Ceuta
    {    1, 6773,     586,  7, }, // Africa/Conakry
    {    1, 6840,     593,  5, }, // Africa/Dakar
    {    1, 5130,     598, 12, }, // Africa/Dar_es_Salaam
    {    1, 5043,     610,  6, }, // Africa/Djibouti
    {    1, 5243,     616,  5, }, // Africa/Douala
    {    1, 5166,     621,  7, }, // Africa/El_Aaiun
    {    1, 6853,     628,  7, }, // Africa/Freetown
    {    1, 4906,     635,  8, }, // Africa/Gaborone
    {    1, 5015,     643,  6, }, // Africa/Harare
    {    1, 4479,     649, 12, }, // Africa/Johannesburg
    {    1, 4499,     661,  4, }, // Africa/Juba
    {    1, 5151,     665,  7, }, // Africa/Kampala
    {    1, 4524,     672,  6, }, // Africa/Khartoum
    {    1, 4973,     678,  6, }, // Africa/Kigali
    {    1, 5306,     684,  7, }, // Africa/Kinshasa
    {    1, 4744,     691,  5, }, // Africa/Lagos
    {    1, 5336,     696,  8, }, // Africa/Libreville
    {    1, 6869,     704,  4, }, // Africa/Lome
    {    1, 5211,     708,  6, }, // Africa/Luanda
    {    1, 4939,     714,  9, }, // Africa/Lubumbashi
    {    1, 5001,     723,  6, }, // Africa/Lusaka
    {    1, 5322,     729,  6, }, // Africa/Malabo
    {    1, 4987,     735,  6, }, // Africa/Maputo
    {    1, 5197,     741,  6, }, // Africa/Maseru
    {    1, 5182,     747,  7, }, // Africa/Mbabane
    {    1, 5113,     754,  8, }, // Africa/Mogadishu
    {    1, 7034,     762,  8, }, // Africa/Monrovia
    {    1, 3853,     770,  7, }, // Africa/Nairobi
    {    1, 5271,     777,  7, }, // Africa/Ndjamena
    {    1, 5354,     784,  6, }, // Africa/Niamey
    {    1, 6803,     790,  7, }, // Africa/Nouakchott
    {    1, 6727,     797,  8, }, // Africa/Ouagadougou
    {    1, 5225,     805, 10, }, // Africa/Porto-Novo
    {    1, 4435,     815,  8, }, // Africa/Sao_Tome
    {    1, 2866,     823,  7, }, // Africa/Tripoli
    {    1, 6554,     830,  5, }, // Africa/Tunis
    {    1, 4152,     835,  7, }, // Africa/Windhoek
    {    1,  132,     842,  4, }, // America/Adak
    {    1, 3346,     846,  7, }, // America/Anchorage
    {    1, 5607,     853,  7, }, // America/Anguilla
    {    1, 5624,     860,  7, }, // America/Antigua
    {    1, 4554,     867,  9, }, // America/Araguaina
    {    1,  166,     876, 12, }, // America/Argentina/Buenos_Aires
    {    1,   91,     888,  9, }, // America/Argentina/Catamarca
    {    1,  270,     897,  7, }, // America/Argentina/Cordoba
    {    1,  441,     904,  5, }, // America/Argentina/Jujuy
    {    1, 7462,     909,  8, }, // America/Argentina/La_Rioja
    {    1,  565,     917,  7, }, // America/Argentina/Mendoza
    {    1, 7489,     924, 11, }, // America/Argentina/Rio_Gallegos
    {    1, 7520,     935,  5, }, // America/Argentina/Salta
    {    1, 7544,     940,  8, }, // America/Argentina/San_Juan
    {    1, 5580,     948,  8, }, // America/Argentina/San_Luis
    {    1, 7571,     956,  7, }, // America/Argentina/Tucuman
    {    1, 7597,     963,  6, }, // America/Argentina/Ushuaia
    {    1, 5640,     969,  5, }, // America/Aruba
    {    1, 4252,     974,  7, }, // America/Asuncion
    {    1,  237,     981, 12, }, // America/Atikokan
    {    1, 3679,     993,  4, }, // America/Bahia
    {    1, 7623,     997, 16, }, // America/Bahia_Banderas
    {    1, 5654,    1013,  8, }, // America/Barbados
    {    1, 7646,    1021,  5, }, // America/Belem
    {    1, 5407,    1026,  5, }, // America/Belize
    {    1, 7660,    1031, 12, }, // America/Blanc-Sablon
    {    1, 7681,    1043,  9, }, // America/Boa_Vista
    {    1, 4361,    1052,  6, }, // America/Bogota
    {    1, 7699,    1058,  5, }, // America/Boise
    {    1, 7713,    1063, 12, }, // America/Cambridge_Bay
    {    1, 7735,    1075, 12, }, // America/Campo_Grande
    {    1, 3868,    1087,  6, }, // America/Cancun
    {    1, 4694,    1093,  7, }, // America/Caracas
    {    1, 4345,    1100,  7, }, // America/Cayenne
    {    1, 5496,    1107, 22, }, // America/Cayman
    {    1, 2260,    1129,  6, }, // America/Chicago
    {    1, 7756,    1135,  6, }, // America/Chihuahua
    {    1, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    1, 5422,    1141, 10, }, // America/Costa_Rica
    {    1, 7796,    1151,  7, }, // America/Creston
    {    1, 3791,    1158,  6, }, // America/Cuiaba
    {    1, 5723,    1164,  8, }, // America/Curacao
    {    1, 7812,    1172, 12, }, // America/Danmarkshavn
    {    1, 7833,    1184,  6, }, // America/Dawson
    {    1, 7848,    1190, 11, }, // America/Dawson_Creek
    {    1,  805,    1201,  6, }, // America/Denver
    {    1, 3465,    1207,  7, }, // America/Detroit
    {    1, 5739,    1214,  8, }, // America/Dominica
    {    1,  893,    1222,  8, }, // America/Edmonton
    {    1, 7869,    1230,  8, }, // America/Eirunepe
    {    1, 5441,    1238,  8, }, // America/El_Salvador
    {    1, 7886,    1246, 11, }, // America/Fort_Nelson
    {    1, 7906,    1257,  9, }, // America/Fortaleza
    {    1, 7924,    1266,  9, }, // America/Glace_Bay
    {    1, 6881,    1275,  7, }, // America/Goose_Bay
    {    1, 4612,    1282, 10, }, // America/Grand_Turk
    {    1, 5770,    1292,  7, }, // America/Grenada
    {    1, 5786,    1299,  9, }, // America/Guadeloupe
    {    1, 3760,    1308,  9, }, // America/Guatemala
    {    1, 4841,    1317,  8, }, // America/Guayaquil
    {    1, 6932,    1325,  6, }, // America/Guyana
    {    1, 1939,    1331,  8, }, // America/Halifax
    {    1, 2281,    1339,  6, }, // America/Havana
    {    1, 7942,    1345,  8, }, // America/Hermosillo
    {    1,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {    1,  481,    1365, 13, }, // America/Indiana/Knox
    {    1, 7961,    1378, 16, }, // America/Indiana/Marengo
    {    1, 7985,    1394, 19, }, // America/Indiana/Petersburg
    {    1, 8012,    1413,  9, }, // America/Indiana/Tell_City
    {    1, 8038,    1422, 14, }, // America/Indiana/Vevay
    {    1, 8060,    1436,  9, }, // America/Indiana/Vincennes
    {    1, 8086,    1445,  7, }, // America/Indiana/Winamac
    {    1, 8110,    1452,  6, }, // America/Inuvik
    {    1,  660,    1458,  7, }, // America/Iqaluit
    {    1, 2799,    1465,  7, }, // America/Jamaica
    {    1, 5380,    1472,  4, }, // America/Juneau
    {    1,  521,    1476,  8, }, // America/Kentucky/Louisville
    {    1, 8125,    1484, 20, }, // America/Kentucky/Monticello
    {    1, 5704,    1504, 10, }, // America/Kralendijk
    {    1, 4376,    1514,  6, }, // America/La_Paz
    {    1, 7169,    1520,  4, }, // America/Lima
    {    1, 3239,    1524, 11, }, // America/Los_Angeles
    {    1, 5932,    1535, 18, }, // America/Lower_Princes
    {    1, 8153,    1553,  6, }, // America/Maceio
    {    1, 8168,    1559,  7, }, // America/Managua
    {    1, 1908,    1566,  6, }, // America/Manaus
    {    1, 5897,    1572,  6, }, // America/Marigot
    {    1, 5805,    1578,  9, }, // America/Martinique
    {    1, 8184,    1587,  9, }, // America/Matamoros
    {    1, 2917,    1596,  8, }, // America/Mazatlan
    {    1, 8202,    1604,  8, }, // America/Menominee
    {    1, 8220,    1612,  6, }, // America/Merida
    {    1, 8235,    1618, 10, }, // America/Metlakatla
    {    1, 2949,    1628,  6, }, // America/Mexico_City
    {    1, 4391,    1634,  7, }, // America/Miquelon
    {    1, 8254,    1641,  7, }, // America/Moncton
    {    1, 8270,    1648,  9, }, // America/Monterrey
    {    1, 4098,    1657, 10, }, // America/Montevideo
    {    1, 5824,    1667, 10, }, // America/Montserrat
    {    1, 5481,    1677,  6, }, // America/Nassau
    {    1, 2379,    1683,  8, }, // America/New_York
    {    1, 8288,    1691,  3, }, // America/Nome
    {    1, 1850,    1694,  7, }, // America/Noronha
    {    1, 8301,    1701, 22, }, // America/North_Dakota/Beulah
    {    1, 8329,    1723, 22, }, // America/North_Dakota/Center
    {    1, 8357,    1745, 27, }, // America/North_Dakota/New_Salem
    {    1,  393,    1772,  4, }, // America/Nuuk
    {    1, 8388,    1776,  7, }, // America/Ojinaga
    {    1, 2356,    1783,  6, }, // America/Panama
    {    1, 6173,    1789, 10, }, // America/Paramaribo
    {    1, 2973,    1799,  6, }, // America/Phoenix
    {    1, 3945,    1805, 12, }, // America/Port-au-Prince
    {    1, 5954,    1817, 13, }, // America/Port_of_Spain
    {    1, 8404,    1830, 11, }, // America/Porto_Velho
    {    1, 5843,    1841, 11, }, // America/Puerto_Rico
    {    1, 4030,    1852, 12, }, // America/Punta_Arenas
    {    1, 8424,    1864, 12, }, // America/Rankin_Inlet
    {    1, 8445,    1876,  6, }, // America/Recife
    {    1, 1995,    1882,  7, }, // America/Regina
    {    1, 8460,    1889,  8, }, // America/Resolute
    {    1,  695,    1897, 10, }, // America/Rio_Branco
    {    1, 8477,    1907,  8, }, // America/Santarem
    {    1, 2201,    1915,  8, }, // America/Santiago
    {    1, 6111,    1923, 13, }, // America/Santo_Domingo
    {    1, 1878,    1936,  9, }, // America/Sao_Paulo
    {    1, 6899,    1945, 11, }, // America/Scoresbysund
    {    1, 8494,    1956,  5, }, // America/Sitka
    {    1, 8508,    1961, 13, }, // America/St_Barthelemy
    {    1, 2061,    1974,  9, }, // America/St_Johns
    {    1, 5863,    1983,  9, }, // America/St_Kitts
    {    1, 5880,    1992, 11, }, // America/St_Lucia
    {    1,  855,    2003, 10, }, // America/St_Thomas
    {    1, 5913,    2013, 12, }, // America/St_Vincent
    {    1, 8530,    2025, 13, }, // America/Swift_Current
    {    1, 5461,    2038, 11, }, // America/Tegucigalpa
    {    1, 5756,    2049,  4, }, // America/Thule
    {    1,  313,    2053,  7, }, // America/Tijuana
    {    1,  608,    2060,  7, }, // America/Toronto
    {    1, 5688,    2067,  7, }, // America/Tortola
    {    1, 2093,    2074,  8, }, // America/Vancouver
    {    1, 2144,    2082,  8, }, // America/Whitehorse
    {    1,  734,    2090,  8, }, // America/Winnipeg
    {    1, 8552,    2098,  7, }, // America/Yakutat
    {    1, 6001,    2105,  5, }, // Antarctica/Casey
    {    1, 6094,    2110,  6, }, // Antarctica/Davis
    {    1, 6133,    2116, 15, }, // Antarctica/DumontDUrville
    {    1, 8568,    2131,  8, }, // Antarctica/Macquarie
    {    1, 7066,    2139,  6, }, // Antarctica/Mawson
    {    1, 7113,    2145,  9, }, // Antarctica/McMurdo
    {    1, 8589,    2154,  6, }, // Antarctica/Palmer
    {    1, 7226,    2160,  6, }, // Antarctica/Rothera
    {    1, 7295,    2166,  5, }, // Antarctica/Syowa
    {    1, 8607,    2171,  5, }, // Antarctica/Troll
    {    1, 7374,    2176, 10, }, // Antarctica/Vostok
    {    1, 1429,    2186,  6, }, // Arctic/Longyearbyen
    {    1, 5570,    2192,  4, }, // Asia/Aden
    {    1, 5395,    2196,  6, }, // Asia/Almaty
    {    1, 3968,    2202,  5, }, // Asia/Amman
    {    1, 5511,    2207,  6, }, // Asia/Anadyr
    {    1, 5523,    2213,  5, }, // Asia/Aqtau
    {    1, 5368,    2218,  6, }, // Asia/Aqtobe
    {    1,  964,    2224,  7, }, // Asia/Ashgabat
    {    1, 8624,    2231,  6, }, // Asia/Atyrau
    {    1, 3607,    2237,  6, }, // Asia/Baghdad
    {    1, 5534,    2243,  7, }, // Asia/Bahrain
    {    1, 3653,    2250,  4, }, // Asia/Baku
    {    1, 4466,    2254,  7, }, // Asia/Bangkok
    {    1, 3571,    2261,  7, }, // Asia/Barnaul
    {    1, 4086,    2268,  6, }, // Asia/Beirut
    {    1, 3778,    2274,  6, }, // Asia/Bishkek
    {    1, 5989,    2280,  6, }, // Asia/Brunei
    {    1, 4601,    2286,  4, }, // Asia/Chita
    {    1, 4511,    2290,  7, }, // Asia/Colombo
    {    1, 4540,    2297,  6, }, // Asia/Damascus
    {    1, 1093,    2303,  5, }, // Asia/Dhaka
    {    1, 6192,    2308,  4, }, // Asia/Dili
    {    1, 3596,    2312,  5, }, // Asia/Dubai
    {    1, 6159,    2317,  7, }, // Asia/Dushanbe
    {    1, 8636,    2324,  9, }, // Asia/Famagusta
    {    1, 8651,    2333,  4, }, // Asia/Gaza
    {    1, 4795,    2337,  6, }, // Asia/Hebron
    {    1, 1259,    2343,  7, }, // Asia/Ho_Chi_Minh
    {    1, 2704,    2350,  7, }, // Asia/Hong_Kong
    {    1, 4771,    2357,  4, }, // Asia/Hovd
    {    1, 4184,    2361,  7, }, // Asia/Irkutsk
    {    1, 7006,    2368,  7, }, // Asia/Jakarta
    {    1, 6992,    2375,  8, }, // Asia/Jayapura
    {    1, 1290,    2383,  9, }, // Asia/Jerusalem
    {    1, 3560,    2392,  5, }, // Asia/Kabul
    {    1, 4316,    2397, 25, }, // Asia/Kamchatka
    {    1, 4239,    2422,  6, }, // Asia/Karachi
    {    1, 1185,    2428,  8, }, // Asia/Kathmandu
    {    1, 8661,    2436,  7, }, // Asia/Khandyga
    {    1,  992,    2443,  8, }, // Asia/Kolkata
    {    1, 4197,    2451, 11, }, // Asia/Krasnoyarsk
    {    1, 4859,    2462, 12, }, // Asia/Kuala_Lumpur
    {    1, 5976,    2474,  6, }, // Asia/Kuching
    {    1, 5547,    2480,  6, }, // Asia/Kuwait
    {    1, 1211,    2486,  5, }, // Asia/Macau
    {    1, 4017,    2491,  7, }, // Asia/Magadan
    {    1, 1349,    2498,  7, }, // Asia/Makassar
    {    1, 7182,    2505,  6, }, // Asia/Manila
    {    1, 6920,    2511,  6, }, // Asia/Muscat
    {    1, 2554,    2517,  7, }, // Asia/Nicosia
    {    1, 8675,    2524, 11, }, // Asia/Novokuznetsk
    {    1, 4135,    2535, 11, }, // Asia/Novosibirsk
    {    1, 4229,    2546,  4, }, // Asia/Omsk
    {    1, 7145,    2550,  6, }, // Asia/Oral
    {    1, 6961,    2556,  7, }, // Asia/Phnom_Penh
    {    1, 8693,    2563,  9, }, // Asia/Pontianak
    {    1, 4214,    2572,  7, }, // Asia/Pyongyang
    {    1, 5559,    2579,  5, }, // Asia/Qatar
    {    1, 8708,    2584,  8, }, // Asia/Qostanay
    {    1, 4269,    2592,  9, }, // Asia/Qyzylorda
    {    1, 3584,    2601,  7, }, // Asia/Riyadh
    {    1, 4408,    2608, 13, }, // Asia/Sakhalin
    {    1, 7245,    2621,  9, }, // Asia/Samarkand
    {    1, 3279,    2630,  4, }, // Asia/Seoul
    {    1, 1053,    2634,  6, }, // Asia/Shanghai
    {    1, 3300,    2640,  8, }, // Asia/Singapore
    {    1, 4297,    2648, 13, }, // Asia/Srednekolymsk
    {    1, 3263,    2661,  6, }, // Asia/Taipei
    {    1, 4781,    2667,  7, }, // Asia/Tashkent
    {    1, 3915,    2674,  4, }, // Asia/Tbilisi
    {    1, 2772,    2678,  7, }, // Asia/Tehran
    {    1, 1317,    2685,  7, }, // Asia/Thimphu
    {    1, 2821,    2692,  5, }, // Asia/Tokyo
    {    1, 4572,    2697,  5, }, // Asia/Tomsk
    {    1, 1021,    2702, 10, }, // Asia/Ulaanbaatar
    {    1, 1159,    2712,  6, }, // Asia/Urumqi
    {    1, 8722,    2718,  8, }, // Asia/Ust-Nera
    {    1, 6977,    2726,  7, }, // Asia/Vientiane
    {    1, 4710,    2733, 11, }, // Asia/Vladivostok
    {    1, 4828,    2744,  7, }, // Asia/Yakutsk
    {    1, 1235,    2751,  6, }, // Asia/Yangon
    {    1, 3883,    2757, 12, }, // Asia/Yekaterinburg
    {    1, 3747,    2769,  6, }, // Asia/Yerevan
    {    1, 3663,    2775, 16, }, // Atlantic/Azores
    {    1, 5671,    2791, 18, }, // Atlantic/Bermuda
    {    1, 6628,    2809, 17, }, // Atlantic/Canary
    {    1, 3727,    2826, 10, }, // Atlantic/Cape_Verde
    {    1, 1395,    2836, 17, }, // Atlantic/Faroe
    {    1, 8736,    2853,  7, }, // Atlantic/Madeira
    {    1, 2748,    2860,  9, }, // Atlantic/Reykjavik
    {    1, 7272,    2869, 16, }, // Atlantic/South_Georgia
    {    1, 6821,    2885, 26, }, // Atlantic/St_Helena
    {    1, 6644,    2911,  6, }, // Atlantic/Stanley
    {    1, 1670,    2917,  8, }, // Australia/Adelaide
    {    1, 1635,    2925,  7, }, // Australia/Brisbane
    {    1, 1799,    2932, 11, }, // Australia/Broken_Hill
    {    1, 1583,    2943,  6, }, // Australia/Darwin
    {    1, 3637,    2949,  5, }, // Australia/Eucla
    {    1, 1516,    2954,  6, }, // Australia/Hobart
    {    1, 8753,    2960,  8, }, // Australia/Lindeman
    {    1, 1547,    2968,  8, }, // Australia/Lord_Howe
    {    1, 1727,    2976,  7, }, // Australia/Melbourne
    {    1, 1762,    2983,  4, }, // Australia/Perth
    {    1, 1463,    2987,  6, }, // Australia/Sydney
    {    1, 6441,    2993,  9, }, // Europe/Amsterdam
    {    1, 6216,    3002,  7, }, // Europe/Andorra
    {    1, 3620,    3009,  8, }, // Europe/Astrakhan
    {    1, 2300,    3017,  4, }, // Europe/Athens
    {    1, 6336,    3021,  7, }, // Europe/Belgrade
    {    1, 4757,    3028,  6, }, // Europe/Berlin
    {    1, 6488,    3034, 10, }, // Europe/Bratislava
    {    1, 2167,    3044,  8, }, // Europe/Brussels
    {    1, 3928,    3052,  8, }, // Europe/Bucharest
    {    1, 3806,    3060,  8, }, // Europe/Budapest
    {    1, 8772,    3068, 23, }, // Europe/Busingen
    {    1, 2583,    3091,  7, }, // Europe/Chisinau
    {    1, 6289,    3098, 10, }, // Europe/Copenhagen
    {    1, 2338,    3108,  6, }, // Europe/Dublin
    {    1, 6307,    3114,  9, }, // Europe/Gibraltar
    {    1, 8788,    3123,  6, }, // Europe/Guernsey
    {    1, 6612,    3129,  8, }, // Europe/Helsinki
    {    1, 8804,    3137,  9, }, // Europe/Isle_of_Man
    {    1, 1130,    3146,  7, }, // Europe/Istanbul
    {    1, 8823,    3153,  5, }, // Europe/Jersey
    {    1, 3979,    3158, 11, }, // Europe/Kaliningrad
    {    1, 8929,    3169,  5, }, // Europe/Kirov
    {    1, 2527,    3174,  4, }, // Europe/Kyiv
    {    1, 3213,    3178,  8, }, // Europe/Lisbon
    {    1, 6506,    3186,  9, }, // Europe/Ljubljana
    {    1, 2501,    3195,  6, }, // Europe/London
    {    1, 6365,    3201, 11, }, // Europe/Luxembourg
    {    1, 4892,    3212,  6, }, // Europe/Madrid
    {    1, 6397,    3218,  5, }, // Europe/Malta
    {    1, 6582,    3223,  9, }, // Europe/Mariehamn
    {    1, 3693,    3232,  5, }, // Europe/Minsk
    {    1, 6410,    3237,  6, }, // Europe/Monaco
    {    1, 3537,    3243,  6, }, // Europe/Moscow
    {    1, 6458,    3249,  4, }, // Europe/Oslo
    {    1, 4284,    3253,  5, }, // Europe/Paris
    {    1, 6424,    3258,  9, }, // Europe/Podgorica
    {    1, 6275,    3267,  5, }, // Europe/Prague
    {    1, 8837,    3272,  4, }, // Europe/Riga
    {    1, 6324,    3276,  3, }, // Europe/Rome
    {    1, 4331,    3279,  6, }, // Europe/Samara
    {    1, 6470,    3285, 10, }, // Europe/San_Marino
    {    1, 6245,    3295,  7, }, // Europe/Sarajevo
    {    1, 4451,    3302,  7, }, // Europe/Saratov
    {    1, 8849,    3309, 10, }, // Europe/Simferopol
    {    1, 6383,    3319,  5, }, // Europe/Skopje
    {    1, 6599,    3324,  5, }, // Europe/Sofia
    {    1, 6523,    3329,  8, }, // Europe/Stockholm
    {    1, 8867,    3337,  6, }, // Europe/Tallinn
    {    1, 6202,    3343,  6, }, // Europe/Tirane
    {    1, 8882,    3349,  9, }, // Europe/Ulyanovsk
    {    1, 6352,    3358,  5, }, // Europe/Vaduz
    {    1, 6567,    3363,  7, }, // Europe/Vatican
    {    1, 6231,    3370,  4, }, // Europe/Vienna
    {    1, 8899,    3374,  7, }, // Europe/Vilnius
    {    1, 4727,    3381,  9, }, // Europe/Volgograd
    {    1, 3190,    3390,  7, }, // Europe/Warsaw
    {    1, 6261,    3397,  6, }, // Europe/Zagreb
    {    1, 6540,    3403,  6, }, // Europe/Zurich
    {    1, 5078,    3409, 12, }, // Indian/Antananarivo
    {    1, 6947,    3421,  5, }, // Indian/Chagos
    {    1, 6046,    3426, 12, }, // Indian/Christmas
    {    1, 6063,    3438, 17, }, // Indian/Cocos
    {    1, 5029,    3455,  5, }, // Indian/Comoro
    {    1, 6661,    3460,  8, }, // Indian/Kerguelen
    {    1, 7260,    3468,  3, }, // Indian/Mahe
    {    1, 7050,    3471,  6, }, // Indian/Maldives
    {    1, 4069,    3477,  7, }, // Indian/Mauritius
    {    1, 5098,    3484,  7, }, // Indian/Mayotte
    {    1, 7211,    3491,  7, }, // Indian/Reunion
    {    1, 4422,    3498,  4, }, // Pacific/Apia
    {    1,  932,    3502,  6, }, // Pacific/Auckland
    {    1, 3706,    3508,  8, }, // Pacific/Bougainville
    {    1, 3015,    3516,  5, }, // Pacific/Chatham
    {    1, 3157,    3521,  4, }, // Pacific/Chuuk
    {    1, 2237,    3525, 11, }, // Pacific/Easter
    {    1, 7360,    3536,  5, }, // Pacific/Efate
    {    1, 7327,    3541,  7, }, // Pacific/Fakaofo
    {    1, 3902,    3548,  4, }, // Pacific/Fiji
    {    1, 7343,    3552,  8, }, // Pacific/Funafuti
    {    1, 6678,    3560, 21, }, // Pacific/Galapagos
    {    1, 6696,    3581, 14, }, // Pacific/Gambier
    {    1, 3822,    3595, 11, }, // Pacific/Guadalcanal
    {    1, 6018,    3606,  4, }, // Pacific/Guam
    {    1, 3049,    3610,  6, }, // Pacific/Kanton
    {    1, 3998,    3616, 10, }, // Pacific/Kiritimati
    {    1, 7019,    3626,  6, }, // Pacific/Kosrae
    {    1, 2842,    3632,  9, }, // Pacific/Kwajalein
    {    1, 4877,    3641,  6, }, // Pacific/Majuro
    {    1, 4051,    3647, 18, }, // Pacific/Marquesas
    {    1, 8914,    3665, 15, }, // Pacific/Midway
    {    1, 7084,    3680,  5, }, // Pacific/Nauru
    {    1, 7132,    3685,  4, }, // Pacific/Niue
    {    1, 4168,    3689,  7, }, // Pacific/Norfolk
    {    1, 7098,    3696,  5, }, // Pacific/Noumea
    {    1, 3126,    3701,  9, }, // Pacific/Pago_Pago
    {    1, 7155,    3710,  5, }, // Pacific/Palau
    {    1, 7194,    3715,  7, }, // Pacific/Pitcairn
    {    1, 3096,    3722,  6, }, // Pacific/Pohnpei
    {    1, 4807,    3728, 11, }, // Pacific/Port_Moresby
    {    1, 6076,    3739,  9, }, // Pacific/Rarotonga
    {    1, 6031,    3748,  6, }, // Pacific/Saipan
    {    1, 7312,    3754,  5, }, // Pacific/Tahiti
    {    1, 6712,    3759,  6, }, // Pacific/Tarawa
    {    1, 4583,    3765,  9, }, // Pacific/Tongatapu
    {    1, 7392,    3774,  4, }, // Pacific/Wake
    {    1, 7405,    3778,  6, }, // Pacific/Wallis
    {    2,   14,       0,  6, }, // Africa/Asmara Afar/Latin/Ethiopia
    {    2, 4435,       6,  8, }, // Africa/Sao_Tome
    {    2, 4252,      14,  8, }, // America/Asuncion
    {    2,  237,      22,  8, }, // America/Atikokan
    {    2, 7623,      30, 17, }, // America/Bahia_Banderas
    {    2, 3868,      47,  6, }, // America/Cancun
    {    2, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    2, 5723,      66,  7, }, // America/Curacao
    {    2,  481,      73, 13, }, // America/Indiana/Knox
    {    2, 7961,      86, 16, }, // America/Indiana/Marengo
    {    2, 7985,     102, 19, }, // America/Indiana/Petersburg
    {    2, 8012,     121, 18, }, // America/Indiana/Tell_City
    {    2, 8038,     139, 14, }, // America/Indiana/Vevay
    {    2, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    2, 8086,     171, 16, }, // America/Indiana/Winamac
    {    2, 8125,     187, 20, }, // America/Kentucky/Monticello
    {    2, 5932,     207, 22, }, // America/Lower_Princes
    {    2, 8220,     229,  6, }, // America/Merida
    {    2, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {    2, 8329,     255, 20, }, // America/North_Dakota/Center
    {    2, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {    2,  393,     298,  4, }, // America/Nuuk
    {    2, 6899,     302, 16, }, // America/Scoresbysund
    {    2, 8508,     318, 14, }, // America/St_Barthelemy
    {    2, 2061,     332, 10, }, // America/St_Johns
    {    2, 5863,     342,  9, }, // America/St_Kitts
    {    2, 5880,     351,  9, }, // America/St_Lucia
    {    2,  855,     360, 10, }, // America/St_Thomas
    {    2, 5913,     370, 11, }, // America/St_Vincent
    {    2, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {    2, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {    2, 1185,     413,  9, }, // Asia/Kathmandu
    {    2,  992,     422,  7, }, // Asia/Kolkata
    {    2, 1211,     429,  5, }, // Asia/Macau
    {    2, 1235,     442,  6, }, // Asia/Yangon
    {    2, 1395,     448,  5, }, // Atlantic/Faroe
    {    2, 6821,     453, 10, }, // Atlantic/St_Helena
    {    2, 2527,     463,  4, }, // Europe/Kyiv
    {    2, 7211,     467,  7, }, // Indian/Reunion
    {    2, 3157,     474,  5, }, // Pacific/Chuuk
    {    2, 3096,     479,  7, }, // Pacific/Pohnpei
    {    5, 5059,    3784, 11, }, // Africa/Addis_Ababa Afrikaans/Latin/South Africa
    {    5,   14,       0,  6, }, // Africa/Asmara
    {    5, 2320,    3795,  5, }, // Africa/Cairo
    {    5, 5043,    3800,  8, }, // Africa/Djibouti
    {    5, 4524,    3808,  7, }, // Africa/Khartoum
    {    5, 5113,    3815, 10, }, // Africa/Mogadishu
    {    5, 4435,       6,  8, }, // Africa/Sao_Tome
    {    5, 4252,      14,  8, }, // America/Asuncion
    {    5,  237,      22,  8, }, // America/Atikokan
    {    5, 7623,    3825, 14, }, // America/Bahia_Banderas
    {    5, 7713,    3839, 13, }, // America/Cambridge_Bay
    {    5, 3868,      47,  6, }, // America/Cancun
    {    5, 5496,    3852,  7, }, // America/Cayman
    {    5, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    5, 5723,      66,  7, }, // America/Curacao
    {    5, 7924,    3859,  9, }, // America/Glace_Bay
    {    5, 6881,    3868,  9, }, // America/Goose_Bay
    {    5,  481,      73, 13, }, // America/Indiana/Knox
    {    5, 7961,      86, 16, }, // America/Indiana/Marengo
    {    5, 7985,     102, 19, }, // America/Indiana/Petersburg
    {    5, 8012,     121, 18, }, // America/Indiana/Tell_City
    {    5, 8038,     139, 14, }, // America/Indiana/Vevay
    {    5, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    5, 8086,     171, 16, }, // America/Indiana/Winamac
    {    5, 2799,    3877,  7, }, // America/Jamaica
    {    5, 8125,     187, 20, }, // America/Kentucky/Monticello
    {    5, 5932,     207, 22, }, // America/Lower_Princes
    {    5, 8220,     229,  6, }, // America/Merida
    {    5, 2949,    3884, 11, }, // America/Mexico_City
    {    5, 8301,    3895, 20, }, // America/North_Dakota/Beulah
    {    5, 8329,    3915, 20, }, // America/North_Dakota/Center
    {    5, 8357,    3935, 23, }, // America/North_Dakota/New_Salem
    {    5,  393,     298,  4, }, // America/Nuuk
    {    5, 6899,     302, 16, }, // America/Scoresbysund
    {    5, 8508,    3958, 17, }, // America/St_Barthelemy
    {    5, 2061,    3975, 11, }, // America/St_Johns
    {    5, 5863,    3986, 10, }, // America/St_Kitts
    {    5, 5880,    3996, 10, }, // America/St_Lucia
    {    5,  855,    4006, 11, }, // America/St_Thomas
    {    5, 5913,    4017, 12, }, // America/St_Vincent
    {    5, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {    5, 7374,    4029,  6, }, // Antarctica/Vostok
    {    5,  964,    4035,  9, }, // Asia/Ashgabat
    {    5, 3607,    4044,  6, }, // Asia/Baghdad
    {    5, 5534,    4050,  7, }, // Asia/Bahrain
    {    5, 3653,    4057,  5, }, // Asia/Baku
    {    5, 4086,    4062,  7, }, // Asia/Beirut
    {    5, 3778,    4069,  7, }, // Asia/Bishkek
    {    5, 5989,    4076,  7, }, // Asia/Brunei
    {    5, 4540,    4083,  8, }, // Asia/Damascus
    {    5, 3596,    4091,  6, }, // Asia/Dubai
    {    5, 1259,    4097, 16, }, // Asia/Ho_Chi_Minh
    {    5, 2704,    4113,  8, }, // Asia/Hong_Kong
    {    5, 4184,    4121,  8, }, // Asia/Irkutsk
    {    5, 7006,    4129,  8, }, // Asia/Jakarta
    {    5, 3560,    4137,  6, }, // Asia/Kabul
    {    5, 4316,    4143, 10, }, // Asia/Kamchatka
    {    5, 4239,    4153,  8, }, // Asia/Karachi
    {    5, 1185,    4161,  9, }, // Asia/Kathmandu
    {    5,  992,     422,  7, }, // Asia/Kolkata
    {    5, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {    5, 4859,    4181, 15, }, // Asia/Kuala_Lumpur
    {    5, 5547,    4196,  7, }, // Asia/Kuwait
    {    5, 1211,    4203,  5, }, // Asia/Macau
    {    5, 6920,    4208,  6, }, // Asia/Muscat
    {    5, 5559,    4214,  5, }, // Asia/Qatar
    {    5, 8708,     434,  8, }, // Asia/Qostanay
    {    5, 3584,    4219,  5, }, // Asia/Riyadh
    {    5, 3279,    4224,  5, }, // Asia/Seoul
    {    5, 3300,    4229,  9, }, // Asia/Singapore
    {    5, 2821,    4238,  5, }, // Asia/Tokyo
    {    5, 4710,    4243, 11, }, // Asia/Vladivostok
    {    5, 4828,    4254,  8, }, // Asia/Yakutsk
    {    5, 1235,     442,  6, }, // Asia/Yangon
    {    5, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {    5, 3663,    4275,  5, }, // Atlantic/Azores
    {    5, 6628,    4280,  7, }, // Atlantic/Canary
    {    5, 3727,    4287, 10, }, // Atlantic/Cape_Verde
    {    5, 1395,     448,  5, }, // Atlantic/Faroe
    {    5, 7272,    4297, 12, }, // Atlantic/South_Georgia
    {    5, 6821,    4309, 11, }, // Atlantic/St_Helena
    {    5, 2300,    4320,  6, }, // Europe/Athens
    {    5, 4757,    4326,  6, }, // Europe/Berlin
    {    5, 2167,    4332,  7, }, // Europe/Brussels
    {    5, 3928,    4339,  9, }, // Europe/Bucharest
    {    5, 3806,    4348,  9, }, // Europe/Budapest
    {    5, 6289,    4357, 10, }, // Europe/Copenhagen
    {    5, 8804,    4367, 10, }, // Europe/Isle_of_Man
    {    5, 2527,    4377,  4, }, // Europe/Kyiv
    {    5, 3213,    4381,  8, }, // Europe/Lisbon
    {    5, 2501,    4389,  6, }, // Europe/London
    {    5, 6365,    4395,  9, }, // Europe/Luxembourg
    {    5, 3537,    4404,  6, }, // Europe/Moscow
    {    5, 4284,    4410,  5, }, // Europe/Paris
    {    5, 6275,    4415,  5, }, // Europe/Prague
    {    5, 4451,    4420,  7, }, // Europe/Saratov
    {    5, 6567,    4427, 12, }, // Europe/Vatican
    {    5, 6231,    4439,  4, }, // Europe/Vienna
    {    5, 4727,    4443,  9, }, // Europe/Volgograd
    {    5, 3190,    4452,  7, }, // Europe/Warsaw
    {    5, 6540,    4459,  6, }, // Europe/Zurich
    {    5, 6063,    4465,  5, }, // Indian/Cocos
    {    5, 5029,    4470,  6, }, // Indian/Comoro
    {    5, 7050,    4476,  8, }, // Indian/Maldives
    {    5, 7211,     467,  7, }, // Indian/Reunion
    {    5, 3157,     474,  5, }, // Pacific/Chuuk
    {    5, 2237,    4484,  4, }, // Pacific/Easter
    {    5, 3902,    4488,  5, }, // Pacific/Fiji
    {    5, 2723,    4493,  8, }, // Pacific/Honolulu
    {    5, 3049,    4501,  9, }, // Pacific/Kanton
    {    5, 7098,    4510,  6, }, // Pacific/Noumea
    {    5, 3096,     479,  7, }, // Pacific/Pohnpei
    {    5, 7405,    4516,  8, }, // Pacific/Wallis
    {    7,   14,       0,  6, }, // Africa/Asmara Aghem/Latin/Cameroon
    {    7, 4435,       6,  8, }, // Africa/Sao_Tome
    {    7, 4252,      14,  8, }, // America/Asuncion
    {    7,  237,      22,  8, }, // America/Atikokan
    {    7, 7623,      30, 17, }, // America/Bahia_Banderas
    {    7, 3868,      47,  6, }, // America/Cancun
    {    7, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    7, 5723,      66,  7, }, // America/Curacao
    {    7,  481,      73, 13, }, // America/Indiana/Knox
    {    7, 7961,      86, 16, }, // America/Indiana/Marengo
    {    7, 7985,     102, 19, }, // America/Indiana/Petersburg
    {    7, 8012,     121, 18, }, // America/Indiana/Tell_City
    {    7, 8038,     139, 14, }, // America/Indiana/Vevay
    {    7, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    7, 8086,     171, 16, }, // America/Indiana/Winamac
    {    7, 8125,     187, 20, }, // America/Kentucky/Monticello
    {    7, 5932,     207, 22, }, // America/Lower_Princes
    {    7, 8220,     229,  6, }, // America/Merida
    {    7, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {    7, 8329,     255, 20, }, // America/North_Dakota/Center
    {    7, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {    7,  393,     298,  4, }, // America/Nuuk
    {    7, 6899,     302, 16, }, // America/Scoresbysund
    {    7, 8508,     318, 14, }, // America/St_Barthelemy
    {    7, 2061,     332, 10, }, // America/St_Johns
    {    7, 5863,     342,  9, }, // America/St_Kitts
    {    7, 5880,     351,  9, }, // America/St_Lucia
    {    7,  855,     360, 10, }, // America/St_Thomas
    {    7, 5913,     370, 11, }, // America/St_Vincent
    {    7, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {    7, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {    7, 1185,     413,  9, }, // Asia/Kathmandu
    {    7,  992,     422,  7, }, // Asia/Kolkata
    {    7, 1211,     429,  5, }, // Asia/Macau
    {    7, 1235,     442,  6, }, // Asia/Yangon
    {    7, 1395,     448,  5, }, // Atlantic/Faroe
    {    7, 6821,     453, 10, }, // Atlantic/St_Helena
    {    7, 2527,     463,  4, }, // Europe/Kyiv
    {    7, 7211,     467,  7, }, // Indian/Reunion
    {    7, 3157,     474,  5, }, // Pacific/Chuuk
    {    7, 3096,     479,  7, }, // Pacific/Pohnpei
    {    8, 6788,    4524,  7, }, // Africa/Abidjan Akan/Latin/Ghana
    {    8, 6760,    4531,  5, }, // Africa/Accra
    {    8, 7420,    4536,  7, }, // Africa/Algiers
    {    8,   14,       0,  6, }, // Africa/Asmara
    {    8, 6746,    4543,  7, }, // Africa/Banjul
    {    8, 7435,    4550,  5, }, // Africa/Bissau
    {    8, 4957,    4555,  7, }, // Africa/Blantyre
    {    8, 4922,    4562, 10, }, // Africa/Bujumbura
    {    8, 2320,    4572,  5, }, // Africa/Cairo
    {    8, 4117,    4577, 10, }, // Africa/Casablanca
    {    8, 7449,    4587,  5, }, // Africa/Ceuta
    {    8, 6773,    4592,  7, }, // Africa/Conakry
    {    8, 6840,    4599,  5, }, // Africa/Dakar
    {    8, 5043,    4604,  8, }, // Africa/Djibouti
    {    8, 4499,    4612,  5, }, // Africa/Juba
    {    8, 5306,    4617, 10, }, // Africa/Kinshasa
    {    8, 4744,    4627,  5, }, // Africa/Lagos
    {    8, 5271,    4632,  8, }, // Africa/Ndjamena
    {    8, 6727,    4640,  8, }, // Africa/Ouagadougou
    {    8, 4435,       6,  8, }, // Africa/Sao_Tome
    {    8, 3346,    4648,  9, }, // America/Anchorage
    {    8, 5607,    4657,  7, }, // America/Anguilla
    {    8,   91,    4664,  9, }, // America/Argentina/Catamarca
    {    8,  270,    4673,  7, }, // America/Argentina/Cordoba
    {    8,  441,    4680,  7, }, // America/Argentina/Jujuy
    {    8, 7462,    4687,  9, }, // America/Argentina/La_Rioja
    {    8,  565,    4696,  7, }, // America/Argentina/Mendoza
    {    8, 7489,    4703, 12, }, // America/Argentina/Rio_Gallegos
    {    8, 7544,    4715,  9, }, // America/Argentina/San_Juan
    {    8, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {    8, 4252,      14,  8, }, // America/Asuncion
    {    8,  237,      22,  8, }, // America/Atikokan
    {    8, 7623,      30, 17, }, // America/Bahia_Banderas
    {    8, 5654,    4731,  8, }, // America/Barbados
    {    8, 7646,    4739,  5, }, // America/Belem
    {    8, 5407,    4744,  6, }, // America/Belize
    {    8, 7660,    4750, 12, }, // America/Blanc-Sablon
    {    8, 7699,    4762,  5, }, // America/Boise
    {    8, 7713,    4767, 13, }, // America/Cambridge_Bay
    {    8, 7735,    4780, 12, }, // America/Campo_Grande
    {    8, 3868,      47,  6, }, // America/Cancun
    {    8, 4694,    4792,  7, }, // America/Caracas
    {    8, 4345,    4799,  7, }, // America/Cayenne
    {    8, 5496,    4806,  7, }, // America/Cayman
    {    8, 2260,    4813,  7, }, // America/Chicago
    {    8, 7756,    4820,  9, }, // America/Chihuahua
    {    8, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    8, 5422,    4829, 10, }, // America/Costa_Rica
    {    8, 7796,    4839,  7, }, // America/Creston
    {    8, 3791,    4846,  6, }, // America/Cuiaba
    {    8, 5723,    4852,  7, }, // America/Curacao
    {    8, 7833,    4859,  6, }, // America/Dawson
    {    8, 7848,    4865, 12, }, // America/Dawson_Creek
    {    8,  805,    4877,  5, }, // America/Denver
    {    8, 3465,    4882,  7, }, // America/Detroit
    {    8, 5739,    4889,  8, }, // America/Dominica
    {    8,  893,    4897,  8, }, // America/Edmonton
    {    8, 5441,    4905, 11, }, // America/El_Salvador
    {    8, 7886,    4916, 10, }, // America/Fort_Nelson
    {    8, 7906,    4926,  9, }, // America/Fortaleza
    {    8, 6881,    4935,  8, }, // America/Goose_Bay
    {    8, 4612,    4943,  9, }, // America/Grand_Turk
    {    8, 5786,    4952,  9, }, // America/Guadeloupe
    {    8, 3760,    4961, 10, }, // America/Guatemala
    {    8, 4841,    4971,  9, }, // America/Guayaquil
    {    8, 6932,    4980,  6, }, // America/Guyana
    {    8, 7942,    4986,  8, }, // America/Hermosillo
    {    8,  348,    4994, 12, }, // America/Indiana/Indianapolis
    {    8,  481,      73, 13, }, // America/Indiana/Knox
    {    8, 7961,      86, 16, }, // America/Indiana/Marengo
    {    8, 7985,    5006, 19, }, // America/Indiana/Petersburg
    {    8, 8012,    5025, 18, }, // America/Indiana/Tell_City
    {    8, 8038,     139, 14, }, // America/Indiana/Vevay
    {    8, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    8, 8086,    5043, 16, }, // America/Indiana/Winamac
    {    8,  660,    5059,  7, }, // America/Iqaluit
    {    8, 2799,    5066,  7, }, // America/Jamaica
    {    8, 8125,    5073, 18, }, // America/Kentucky/Monticello
    {    8, 5704,    5091, 10, }, // America/Kralendijk
    {    8, 3239,    5101, 12, }, // America/Los_Angeles
    {    8, 5932,    5113, 16, }, // America/Lower_Princes
    {    8, 8153,    5129,  6, }, // America/Maceio
    {    8, 5897,    5135,  7, }, // America/Marigot
    {    8, 5805,    5142,  9, }, // America/Martinique
    {    8, 8184,    5151, 10, }, // America/Matamoros
    {    8, 2917,    5161,  8, }, // America/Mazatlan
    {    8, 8202,    5169,  9, }, // America/Menominee
    {    8, 8220,     229,  6, }, // America/Merida
    {    8, 8235,    5178, 10, }, // America/Metlakatla
    {    8, 2949,    5188, 12, }, // America/Mexico_City
    {    8, 8254,    5200,  7, }, // America/Moncton
    {    8, 8270,    5207,  8, }, // America/Monterrey
    {    8, 5824,    5215,  9, }, // America/Montserrat
    {    8, 2379,    5224,  7, }, // America/New_York
    {    8, 8301,    5231, 17, }, // America/North_Dakota/Beulah
    {    8, 8329,    5248, 17, }, // America/North_Dakota/Center
    {    8, 8357,    5265, 21, }, // America/North_Dakota/New_Salem
    {    8,  393,     298,  4, }, // America/Nuuk
    {    8, 8388,    5286,  8, }, // America/Ojinaga
    {    8, 6173,    5294, 10, }, // America/Paramaribo
    {    8, 2973,    5304,  6, }, // America/Phoenix
    {    8, 5954,    5310, 11, }, // America/Port_of_Spain
    {    8, 8404,    5321, 11, }, // America/Porto_Velho
    {    8, 5843,    5332, 10, }, // America/Puerto_Rico
    {    8, 8445,    5342,  6, }, // America/Recife
    {    8, 1995,    5348,  7, }, // America/Regina
    {    8, 8460,    5355,  7, }, // America/Resolute
    {    8,  695,    5362, 10, }, // America/Rio_Branco
    {    8, 6899,    5372, 13, }, // America/Scoresbysund
    {    8, 8508,    5385, 13, }, // America/St_Barthelemy
    {    8, 2061,     332, 10, }, // America/St_Johns
    {    8, 5863,     342,  9, }, // America/St_Kitts
    {    8, 5880,     351,  9, }, // America/St_Lucia
    {    8,  855,     360, 10, }, // America/St_Thomas
    {    8, 5913,     370, 11, }, // America/St_Vincent
    {    8, 8530,    5398, 12, }, // America/Swift_Current
    {    8, 5461,    5410, 11, }, // America/Tegucigalpa
    {    8,  313,    5421,  8, }, // America/Tijuana
    {    8, 5688,    5429,  7, }, // America/Tortola
    {    8,  734,    5436,  7, }, // America/Winnipeg
    {    8, 6001,    5443,  4, }, // Antarctica/Casey
    {    8, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {    8, 8568,    5447,  7, }, // Antarctica/Macquarie
    {    8, 7066,    5454,  6, }, // Antarctica/Mawson
    {    8, 8589,    5460,  5, }, // Antarctica/Palmer
    {    8, 7226,    5465,  6, }, // Antarctica/Rothera
    {    8, 8607,    5471,  4, }, // Antarctica/Troll
    {    8, 7374,    5475,  6, }, // Antarctica/Vostok
    {    8, 5395,    5481,  6, }, // Asia/Almaty
    {    8, 3968,    5487,  4, }, // Asia/Amman
    {    8, 5523,    5491,  5, }, // Asia/Aqtau
    {    8, 5368,    5496,  6, }, // Asia/Aqtobe
    {    8, 4466,    5502,  6, }, // Asia/Bangkok
    {    8, 4086,    5508,  6, }, // Asia/Beirut
    {    8, 4601,    5514,  5, }, // Asia/Chita
    {    8, 4511,    5519,  7, }, // Asia/Colombo
    {    8, 4540,    5526,  9, }, // Asia/Damascus
    {    8, 1093,    4599,  4, }, // Asia/Dhaka
    {    8, 4795,    5535,  6, }, // Asia/Hebron
    {    8, 1259,    5541, 10, }, // Asia/Ho_Chi_Minh
    {    8, 7006,    5551,  8, }, // Asia/Jakarta
    {    8, 6992,    5559,  9, }, // Asia/Jayapura
    {    8, 1290,    5568,  9, }, // Asia/Jerusalem
    {    8, 4316,    5577,  9, }, // Asia/Kamchatka
    {    8, 4239,    5586,  7, }, // Asia/Karachi
    {    8, 1185,    5593,  8, }, // Asia/Kathmandu
    {    8,  992,    5601,  7, }, // Asia/Kolkata
    {    8, 5976,    5608,  6, }, // Asia/Kuching
    {    8, 1211,     429,  5, }, // Asia/Macau
    {    8, 1349,    5614,  6, }, // Asia/Makassar
    {    8, 6920,    4208,  6, }, // Asia/Muscat
    {    8, 2554,    5620,  7, }, // Asia/Nicosia
    {    8, 5559,    4214,  4, }, // Asia/Qatar
    {    8, 8708,     434,  8, }, // Asia/Qostanay
    {    8, 3300,    5627,  7, }, // Asia/Singapore
    {    8, 2772,    5634,  6, }, // Asia/Tehran
    {    8, 1317,    5640,  6, }, // Asia/Thimphu
    {    8, 1021,    5646,  9, }, // Asia/Ulaanbaatar
    {    8, 1159,    5655,  7, }, // Asia/Urumqi
    {    8, 6977,    5662, 10, }, // Asia/Vientiane
    {    8, 1235,     442,  6, }, // Asia/Yangon
    {    8, 3883,    5672, 12, }, // Asia/Yekaterinburg
    {    8, 5671,    5684,  6, }, // Atlantic/Bermuda
    {    8, 6628,    4280,  6, }, // Atlantic/Canary
    {    8, 3727,    5690,  7, }, // Atlantic/Cape_Verde
    {    8, 1395,     448,  5, }, // Atlantic/Faroe
    {    8, 2748,    5697, 10, }, // Atlantic/Reykjavik
    {    8, 7272,    5707, 16, }, // Atlantic/South_Georgia
    {    8, 6821,     453, 10, }, // Atlantic/St_Helena
    {    8, 6644,    5723,  6, }, // Atlantic/Stanley
    {    8, 1799,    5729, 11, }, // Australia/Broken_Hill
    {    8, 1583,    5740,  6, }, // Australia/Darwin
    {    8, 3637,    5746,  5, }, // Australia/Eucla
    {    8, 1516,    5751,  5, }, // Australia/Hobart
    {    8, 1547,    5756,  8, }, // Australia/Lord_Howe
    {    8, 1727,    5764,  6, }, // Australia/Melbourne
    {    8, 1762,    5770,  4, }, // Australia/Perth
    {    8, 1463,    5774,  5, }, // Australia/Sydney
    {    8, 6441,    5779,  8, }, // Europe/Amsterdam
    {    8, 6216,    5787,  6, }, // Europe/Andorra
    {    8, 2300,    5793,  5, }, // Europe/Athens
    {    8, 6336,    5798,  8, }, // Europe/Belgrade
    {    8, 4757,    5806,  6, }, // Europe/Berlin
    {    8, 2167,    5812,  8, }, // Europe/Brussels
    {    8, 3928,    5820,  9, }, // Europe/Bucharest
    {    8, 3806,    5829,  8, }, // Europe/Budapest
    {    8, 8772,    5837,  8, }, // Europe/Busingen
    {    8, 2583,    5845,  8, }, // Europe/Chisinau
    {    8, 6289,    5853, 11, }, // Europe/Copenhagen
    {    8, 2338,    5864,  6, }, // Europe/Dublin
    {    8, 6307,    5870,  8, }, // Europe/Gibraltar
    {    8, 6612,    5878,  8, }, // Europe/Helsinki
    {    8, 8823,    5886,  5, }, // Europe/Jersey
    {    8, 2527,     463,  4, }, // Europe/Kyiv
    {    8, 3213,    5891,  6, }, // Europe/Lisbon
    {    8, 6506,    5897, 10, }, // Europe/Ljubljana
    {    8, 2501,    5907,  6, }, // Europe/London
    {    8, 6365,    5913,  8, }, // Europe/Luxembourg
    {    8, 6397,    5921,  5, }, // Europe/Malta
    {    8, 6410,    5926,  6, }, // Europe/Monaco
    {    8, 3537,    5932,  5, }, // Europe/Moscow
    {    8, 6424,    5937,  9, }, // Europe/Podgorica
    {    8, 6324,    5946,  4, }, // Europe/Rome
    {    8, 6245,    5950,  9, }, // Europe/Sarajevo
    {    8, 6383,    5959,  7, }, // Europe/Skopje
    {    8, 6599,    5966,  5, }, // Europe/Sofia
    {    8, 6523,    5971,  7, }, // Europe/Stockholm
    {    8, 6567,    5978,  7, }, // Europe/Vatican
    {    8, 6231,    5985,  5, }, // Europe/Vienna
    {    8, 3190,    5990,  6, }, // Europe/Warsaw
    {    8, 6540,    5996,  7, }, // Europe/Zurich
    {    8, 6947,    6003,  6, }, // Indian/Chagos
    {    8, 6046,    6009,  7, }, // Indian/Christmas
    {    8, 6063,    6016,  6, }, // Indian/Cocos
    {    8, 5029,    6022,  6, }, // Indian/Comoro
    {    8, 6661,    6028,  8, }, // Indian/Kerguelen
    {    8, 4069,    6036, 10, }, // Indian/Mauritius
    {    8, 5098,    6046,  6, }, // Indian/Mayotte
    {    8, 7211,     467,  7, }, // Indian/Reunion
    {    8,  932,    6052,  7, }, // Pacific/Auckland
    {    8, 3015,    6059,  6, }, // Pacific/Chatham
    {    8, 3157,    6065,  5, }, // Pacific/Chuuk
    {    8, 2237,    6070,  5, }, // Pacific/Easter
    {    8, 3902,    6075,  5, }, // Pacific/Fiji
    {    8, 6678,    6080,  9, }, // Pacific/Galapagos
    {    8, 3822,    6089, 11, }, // Pacific/Guadalcanal
    {    8, 2842,    6100, 11, }, // Pacific/Kwajalein
    {    8, 4877,    6111,  7, }, // Pacific/Majuro
    {    8, 4051,    6118,  9, }, // Pacific/Marquesas
    {    8, 4168,    6127,  6, }, // Pacific/Norfolk
    {    8, 7194,    6133,  9, }, // Pacific/Pitcairn
    {    8, 3096,    6142,  6, }, // Pacific/Pohnpei
    {    8, 4807,    6148, 11, }, // Pacific/Port_Moresby
    {    9,   14,       0,  6, }, // Africa/Asmara Akoose/Latin/Cameroon
    {    9, 4435,       6,  8, }, // Africa/Sao_Tome
    {    9, 4252,      14,  8, }, // America/Asuncion
    {    9,  237,      22,  8, }, // America/Atikokan
    {    9, 7623,      30, 17, }, // America/Bahia_Banderas
    {    9, 3868,      47,  6, }, // America/Cancun
    {    9, 7774,      53, 13, }, // America/Ciudad_Juarez
    {    9, 5723,      66,  7, }, // America/Curacao
    {    9,  481,      73, 13, }, // America/Indiana/Knox
    {    9, 7961,      86, 16, }, // America/Indiana/Marengo
    {    9, 7985,     102, 19, }, // America/Indiana/Petersburg
    {    9, 8012,     121, 18, }, // America/Indiana/Tell_City
    {    9, 8038,     139, 14, }, // America/Indiana/Vevay
    {    9, 8060,     153, 18, }, // America/Indiana/Vincennes
    {    9, 8086,     171, 16, }, // America/Indiana/Winamac
    {    9, 8125,     187, 20, }, // America/Kentucky/Monticello
    {    9, 5932,     207, 22, }, // America/Lower_Princes
    {    9, 8220,     229,  6, }, // America/Merida
    {    9, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {    9, 8329,     255, 20, }, // America/North_Dakota/Center
    {    9, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {    9,  393,     298,  4, }, // America/Nuuk
    {    9, 6899,     302, 16, }, // America/Scoresbysund
    {    9, 8508,     318, 14, }, // America/St_Barthelemy
    {    9, 2061,     332, 10, }, // America/St_Johns
    {    9, 5863,     342,  9, }, // America/St_Kitts
    {    9, 5880,     351,  9, }, // America/St_Lucia
    {    9,  855,     360, 10, }, // America/St_Thomas
    {    9, 5913,     370, 11, }, // America/St_Vincent
    {    9, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {    9, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {    9, 1185,     413,  9, }, // Asia/Kathmandu
    {    9,  992,     422,  7, }, // Asia/Kolkata
    {    9, 1211,     429,  5, }, // Asia/Macau
    {    9, 1235,     442,  6, }, // Asia/Yangon
    {    9, 1395,     448,  5, }, // Atlantic/Faroe
    {    9, 6821,     453, 10, }, // Atlantic/St_Helena
    {    9, 2527,     463,  4, }, // Europe/Kyiv
    {    9, 7211,     467,  7, }, // Indian/Reunion
    {    9, 3157,     474,  5, }, // Pacific/Chuuk
    {    9, 3096,     479,  7, }, // Pacific/Pohnpei
    {   10, 6788,    6159,  7, }, // Africa/Abidjan Albanian/Latin/Albania
    {   10, 6760,    4531,  4, }, // Africa/Accra
    {   10, 5059,    6166, 10, }, // Africa/Addis_Ababa
    {   10, 7420,    6176,  6, }, // Africa/Algiers
    {   10,   14,    6182,  6, }, // Africa/Asmara
    {   10, 6746,    6188,  7, }, // Africa/Banjul
    {   10, 7435,    6195,  5, }, // Africa/Bissau
    {   10, 4957,    6200,  8, }, // Africa/Blantyre
    {   10, 5287,    6208, 10, }, // Africa/Brazzaville
    {   10, 4922,    6218, 10, }, // Africa/Bujumbura
    {   10, 2320,    6228,  5, }, // Africa/Cairo
    {   10, 4117,    6233, 10, }, // Africa/Casablanca
    {   10, 7449,    6243,  6, }, // Africa/Ceuta
    {   10, 6773,    6249,  7, }, // Africa/Conakry
    {   10, 5130,    6256, 12, }, // Africa/Dar_es_Salaam
    {   10, 5043,    6268,  7, }, // Africa/Djibouti
    {   10, 5166,    6275,  7, }, // Africa/El_Aaiun
    {   10, 6853,    6282,  7, }, // Africa/Freetown
    {   10, 4479,    6289, 11, }, // Africa/Johannesburg
    {   10, 4499,    6300,  5, }, // Africa/Juba
    {   10, 4524,    6305,  6, }, // Africa/Khartoum
    {   10, 5336,    6311,  9, }, // Africa/Libreville
    {   10, 5354,    6320,  6, }, // Africa/Niamey
    {   10, 6803,    6326,  7, }, // Africa/Nouakchott
    {   10, 6727,    6333,  8, }, // Africa/Ouagadougou
    {   10, 4435,    6341,  8, }, // Africa/Sao_Tome
    {   10, 6554,    6349,  5, }, // Africa/Tunis
    {   10, 4152,    6354,  8, }, // Africa/Windhoek
    {   10, 3346,    6362,  9, }, // America/Anchorage
    {   10, 5607,    6371,  7, }, // America/Anguilla
    {   10, 4554,    6378, 10, }, // America/Araguaina
    {   10,  166,    6388, 12, }, // America/Argentina/Buenos_Aires
    {   10,   91,    6400,  9, }, // America/Argentina/Catamarca
    {   10,  270,    6409,  7, }, // America/Argentina/Cordoba
    {   10,  441,    6416,  5, }, // America/Argentina/Jujuy
    {   10, 7462,    6421,  8, }, // America/Argentina/La_Rioja
    {   10, 7489,    6429, 11, }, // America/Argentina/Rio_Gallegos
    {   10, 7520,    6440,  5, }, // America/Argentina/Salta
    {   10, 7544,    6445,  8, }, // America/Argentina/San_Juan
    {   10, 5580,    6453,  9, }, // America/Argentina/San_Luis
    {   10, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {   10, 7597,    6462,  7, }, // America/Argentina/Ushuaia
    {   10, 5640,    6469,  5, }, // America/Aruba
    {   10, 4252,    6474,  8, }, // America/Asuncion
    {   10,  237,      22,  8, }, // America/Atikokan
    {   10, 7623,    6482, 14, }, // America/Bahia_Banderas
    {   10, 5407,    6496,  6, }, // America/Belize
    {   10, 7660,    6502, 12, }, // America/Blanc-Sablon
    {   10, 7681,    6514,  9, }, // America/Boa_Vista
    {   10, 4361,    6523,  6, }, // America/Bogota
    {   10, 7699,    6529,  5, }, // America/Boise
    {   10, 7713,    6534, 18, }, // America/Cambridge_Bay
    {   10, 7735,    6552, 12, }, // America/Campo_Grande
    {   10, 3868,    6564,  6, }, // America/Cancun
    {   10, 4694,    4792,  7, }, // America/Caracas
    {   10, 4345,    6570,  6, }, // America/Cayenne
    {   10, 5496,    6576,  6, }, // America/Cayman
    {   10, 2260,    6582,  6, }, // America/Chicago
    {   10, 7756,    6588,  7, }, // America/Chihuahua
    {   10, 7774,    6595, 13, }, // America/Ciudad_Juarez
    {   10, 5422,    6608, 10, }, // America/Costa_Rica
    {   10, 7796,    6618,  7, }, // America/Creston
    {   10, 3791,    6625,  6, }, // America/Cuiaba
    {   10, 5723,    6631,  7, }, // America/Curacao
    {   10, 7812,    6638, 13, }, // America/Danmarkshavn
    {   10, 7833,    6651,  6, }, // America/Dawson
    {   10, 7848,    6657, 16, }, // America/Dawson_Creek
    {   10, 5739,    6673,  8, }, // America/Dominica
    {   10, 7869,    6681,  7, }, // America/Eirunepe
    {   10, 5441,    6688,  8, }, // America/El_Salvador
    {   10, 7886,    6696, 11, }, // America/Fort_Nelson
    {   10, 7924,    6707, 14, }, // America/Glace_Bay
    {   10, 6881,    6721, 13, }, // America/Goose_Bay
    {   10, 4612,    6734, 12, }, // America/Grand_Turk
    {   10, 5770,    6746,  7, }, // America/Grenada
    {   10, 5786,    6753,  9, }, // America/Guadeloupe
    {   10, 3760,    6762,  9, }, // America/Guatemala
    {   10, 4841,    6771,  9, }, // America/Guayaquil
    {   10, 6932,    6780,  7, }, // America/Guyana
    {   10, 1939,    6787,  8, }, // America/Halifax
    {   10, 2281,    6795,  6, }, // America/Havana
    {   10, 7942,    6801,  9, }, // America/Hermosillo
    {   10,  481,    6810, 14, }, // America/Indiana/Knox
    {   10, 7961,      86, 16, }, // America/Indiana/Marengo
    {   10, 7985,    6824, 19, }, // America/Indiana/Petersburg
    {   10, 8012,    6843, 18, }, // America/Indiana/Tell_City
    {   10, 8038,    6861, 14, }, // America/Indiana/Vevay
    {   10, 8060,    6875, 17, }, // America/Indiana/Vincennes
    {   10, 8086,    6892, 16, }, // America/Indiana/Winamac
    {   10,  660,    5059,  7, }, // America/Iqaluit
    {   10, 2799,    6908,  8, }, // America/Jamaica
    {   10, 5380,    6916,  6, }, // America/Juneau
    {   10,  521,    6922,  8, }, // America/Kentucky/Louisville
    {   10, 8125,    6930, 18, }, // America/Kentucky/Monticello
    {   10, 7169,    6948,  4, }, // America/Lima
    {   10, 3239,    6952, 12, }, // America/Los_Angeles
    {   10, 5932,    6964, 21, }, // America/Lower_Princes
    {   10, 8153,    6985,  6, }, // America/Maceio
    {   10, 5805,    6991,  9, }, // America/Martinique
    {   10, 8202,    7000,  8, }, // America/Menominee
    {   10, 8220,    7008,  6, }, // America/Merida
    {   10, 2949,    7014, 17, }, // America/Mexico_City
    {   10, 4391,    7031,  7, }, // America/Miquelon
    {   10, 8254,    7038,  7, }, // America/Moncton
    {   10, 8270,    7045,  9, }, // America/Monterrey
    {   10, 5824,    7054,  9, }, // America/Montserrat
    {   10, 5481,    7063,  5, }, // America/Nassau
    {   10, 2379,    7068,  8, }, // America/New_York
    {   10, 1850,    7076,  7, }, // America/Noronha
    {   10, 8301,    7083, 22, }, // America/North_Dakota/Beulah
    {   10, 8329,    7105, 23, }, // America/North_Dakota/Center
    {   10, 8357,    7128, 26, }, // America/North_Dakota/New_Salem
    {   10,  393,    7154,  4, }, // America/Nuuk
    {   10, 8388,    7158,  7, }, // America/Ojinaga
    {   10, 2973,    7165,  6, }, // America/Phoenix
    {   10, 3945,    7171, 12, }, // America/Port-au-Prince
    {   10, 8404,    7183, 11, }, // America/Porto_Velho
    {   10, 5843,    7194, 10, }, // America/Puerto_Rico
    {   10, 4030,    7204, 12, }, // America/Punta_Arenas
    {   10, 8424,    7216, 11, }, // America/Rankin_Inlet
    {   10, 1995,    7227,  7, }, // America/Regina
    {   10,  695,    7234, 10, }, // America/Rio_Branco
    {   10, 6111,    7244, 13, }, // America/Santo_Domingo
    {   10, 1878,    7257,  9, }, // America/Sao_Paulo
    {   10, 6899,    7266, 12, }, // America/Scoresbysund
    {   10, 8508,    7278, 13, }, // America/St_Barthelemy
    {   10, 2061,    7291,  9, }, // America/St_Johns
    {   10, 5863,    7300,  9, }, // America/St_Kitts
    {   10, 5880,    7309, 10, }, // America/St_Lucia
    {   10,  855,    7319, 10, }, // America/St_Thomas
    {   10, 5913,    7329, 12, }, // America/St_Vincent
    {   10, 8530,    7341, 15, }, // America/Swift_Current
    {   10, 5461,    5410, 11, }, // America/Tegucigalpa
    {   10, 5756,    7356,  5, }, // America/Thule
    {   10,  313,    7361,  7, }, // America/Tijuana
    {   10, 5688,    7368,  7, }, // America/Tortola
    {   10, 2093,    7375,  8, }, // America/Vancouver
    {   10, 2144,    7383,  9, }, // America/Whitehorse
    {   10,  734,    7392,  7, }, // America/Winnipeg
    {   10, 8552,    7399,  7, }, // America/Yakutat
    {   10, 6001,    7406,  5, }, // Antarctica/Casey
    {   10, 6094,    7411,  6, }, // Antarctica/Davis
    {   10, 6133,    7417, 15, }, // Antarctica/DumontDUrville
    {   10, 8568,    7432,  7, }, // Antarctica/Macquarie
    {   10, 7066,    7439,  6, }, // Antarctica/Mawson
    {   10, 7113,    7445,  8, }, // Antarctica/McMurdo
    {   10, 7226,    7453,  7, }, // Antarctica/Rothera
    {   10, 7295,    7460,  5, }, // Antarctica/Syowa
    {   10, 1429,    7465, 11, }, // Arctic/Longyearbyen
    {   10, 5395,    7476,  6, }, // Asia/Almaty
    {   10, 3968,    5487,  4, }, // Asia/Amman
    {   10, 5511,    7482,  6, }, // Asia/Anadyr
    {   10, 5523,    5491,  5, }, // Asia/Aqtau
    {   10, 5368,    7488,  6, }, // Asia/Aqtobe
    {   10, 8624,    7494,  6, }, // Asia/Atyrau
    {   10, 3607,    4044,  6, }, // Asia/Baghdad
    {   10, 5534,    7500,  7, }, // Asia/Bahrain
    {   10, 4086,    7507,  6, }, // Asia/Beirut
    {   10, 4601,    7513,  4, }, // Asia/Chita
    {   10, 4511,    5519,  7, }, // Asia/Colombo
    {   10, 4540,    4083,  6, }, // Asia/Damascus
    {   10, 1093,    4599,  4, }, // Asia/Dhaka
    {   10, 8636,    7517,  9, }, // Asia/Famagusta
    {   10, 1259,    7526,  9, }, // Asia/Ho_Chi_Minh
    {   10, 2704,    7535,  9, }, // Asia/Hong_Kong
    {   10, 7006,    7544,  8, }, // Asia/Jakarta
    {   10, 6992,    7552,  9, }, // Asia/Jayapura
    {   10, 4316,    7561,  8, }, // Asia/Kamchatka
    {   10, 4239,    7569,  6, }, // Asia/Karachi
    {   10, 1185,    5593,  8, }, // Asia/Kathmandu
    {   10, 8661,    7575,  7, }, // Asia/Khandyga
    {   10,  992,    7582,  7, }, // Asia/Kolkata
    {   10, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {   10, 4859,    7589, 12, }, // Asia/Kuala_Lumpur
    {   10, 5976,    7601,  6, }, // Asia/Kuching
    {   10, 5547,    7607,  6, }, // Asia/Kuwait
    {   10, 1211,    7613,  5, }, // Asia/Macau
    {   10, 1349,    7618,  7, }, // Asia/Makassar
    {   10, 7182,    7625,  6, }, // Asia/Manila
    {   10, 6920,    4208,  6, }, // Asia/Muscat
    {   10, 2554,    7631,  7, }, // Asia/Nicosia
    {   10, 6961,    7638,  8, }, // Asia/Phnom_Penh
    {   10, 4214,    7646,  6, }, // Asia/Pyongyang
    {   10, 5559,    4214,  5, }, // Asia/Qatar
    {   10, 8708,    7652,  8, }, // Asia/Qostanay
    {   10, 4269,    7660,  9, }, // Asia/Qyzylorda
    {   10, 3584,    7669,  4, }, // Asia/Riyadh
    {   10, 4408,    7673,  7, }, // Asia/Sakhalin
    {   10, 3279,    7680,  4, }, // Asia/Seoul
    {   10, 1053,    7684,  7, }, // Asia/Shanghai
    {   10, 3300,    7691,  8, }, // Asia/Singapore
    {   10, 4297,    7699, 13, }, // Asia/Srednekolymsk
    {   10, 3263,    7712,  6, }, // Asia/Taipei
    {   10, 3915,    7718,  6, }, // Asia/Tbilisi
    {   10, 2772,    7724,  7, }, // Asia/Tehran
    {   10, 1317,    7731,  6, }, // Asia/Thimphu
    {   10, 2821,    4238,  5, }, // Asia/Tokyo
    {   10, 1021,    7737,  9, }, // Asia/Ulaanbaatar
    {   10, 6977,    7746,  9, }, // Asia/Vientiane
    {   10, 4828,    7755,  7, }, // Asia/Yakutsk
    {   10, 1235,    7762,  6, }, // Asia/Yangon
    {   10, 3883,    7768, 12, }, // Asia/Yekaterinburg
    {   10, 3747,    7780,  7, }, // Asia/Yerevan
    {   10, 3663,    7787,  5, }, // Atlantic/Azores
    {   10, 5671,    7792,  7, }, // Atlantic/Bermuda
    {   10, 6628,    4280,  6, }, // Atlantic/Canary
    {   10, 3727,    7799, 14, }, // Atlantic/Cape_Verde
    {   10, 1395,     448,  5, }, // Atlantic/Faroe
    {   10, 2748,    7813,  9, }, // Atlantic/Reykjavik
    {   10, 7272,    7822, 15, }, // Atlantic/South_Georgia
    {   10, 6821,    7837, 10, }, // Atlantic/St_Helena
    {   10, 6644,    7847,  6, }, // Atlantic/Stanley
    {   10, 1670,    7853,  8, }, // Australia/Adelaide
    {   10, 1635,    7861,  8, }, // Australia/Brisbane
    {   10, 1799,    7869, 11, }, // Australia/Broken_Hill
    {   10, 1583,    7880,  6, }, // Australia/Darwin
    {   10, 3637,    5746,  5, }, // Australia/Eucla
    {   10, 8753,    7886,  8, }, // Australia/Lindeman
    {   10, 1547,    7894,  9, }, // Australia/Lord_Howe
    {   10, 1727,    7903,  7, }, // Australia/Melbourne
    {   10, 1762,    7910,  5, }, // Australia/Perth
    {   10, 1463,    7915,  6, }, // Australia/Sydney
    {   10, 6216,    7921,  7, }, // Europe/Andorra
    {   10, 3620,    7928,  8, }, // Europe/Astrakhan
    {   10, 2300,    7936,  6, }, // Europe/Athens
    {   10, 6336,    7942,  7, }, // Europe/Belgrade
    {   10, 6488,    7949, 10, }, // Europe/Bratislava
    {   10, 2167,    7959,  7, }, // Europe/Brussels
    {   10, 3928,    7966,  9, }, // Europe/Bucharest
    {   10, 2583,    7975,  8, }, // Europe/Chisinau
    {   10, 6289,    4357, 10, }, // Europe/Copenhagen
    {   10, 6307,    7983, 10, }, // Europe/Gibraltar
    {   10, 8788,    7993,  7, }, // Europe/Guernsey
    {   10, 8804,    8000, 15, }, // Europe/Isle_of_Man
    {   10, 1130,    8015,  8, }, // Europe/Istanbul
    {   10, 8823,    8023,  7, }, // Europe/Jersey
    {   10, 2527,    8030,  4, }, // Europe/Kyiv
    {   10, 3213,    8034,  7, }, // Europe/Lisbon
    {   10, 6506,    8041,  7, }, // Europe/Ljubljana
    {   10, 2501,    8048,  6, }, // Europe/London
    {   10, 6365,    8054, 10, }, // Europe/Luxembourg
    {   10, 6397,    8064,  5, }, // Europe/Malta
    {   10, 6410,    5926,  6, }, // Europe/Monaco
    {   10, 3537,    8069,  5, }, // Europe/Moscow
    {   10, 6424,    8074,  9, }, // Europe/Podgorica
    {   10, 6275,    8083,  5, }, // Europe/Prague
    {   10, 8837,    8088,  4, }, // Europe/Riga
    {   10, 6324,    8092,  4, }, // Europe/Rome
    {   10, 6470,    8096, 10, }, // Europe/San_Marino
    {   10, 6245,    8106,  8, }, // Europe/Sarajevo
    {   10, 6383,    8114,  5, }, // Europe/Skopje
    {   10, 6599,    8119,  5, }, // Europe/Sofia
    {   10, 6523,    8124,  8, }, // Europe/Stockholm
    {   10, 8867,    8132,  5, }, // Europe/Tallinn
    {   10, 6202,    8137,  6, }, // Europe/Tirane
    {   10, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {   10, 6567,    5978,  7, }, // Europe/Vatican
    {   10, 6231,    8152,  5, }, // Europe/Vienna
    {   10, 3190,    8157,  8, }, // Europe/Warsaw
    {   10, 6540,    8165,  5, }, // Europe/Zurich
    {   10, 6947,    8170,  5, }, // Indian/Chagos
    {   10, 6046,    8175, 12, }, // Indian/Christmas
    {   10, 6063,    4465,  5, }, // Indian/Cocos
    {   10, 5029,    8187,  6, }, // Indian/Comoro
    {   10, 7050,    8193,  7, }, // Indian/Maldives
    {   10, 5098,    8200,  6, }, // Indian/Mayotte
    {   10, 7211,     467,  7, }, // Indian/Reunion
    {   10,  932,    8206,  6, }, // Pacific/Auckland
    {   10, 3706,    8212, 10, }, // Pacific/Bougainville
    {   10, 3015,    8222,  6, }, // Pacific/Chatham
    {   10, 3157,    8228,  3, }, // Pacific/Chuuk
    {   10, 2237,    8231,  6, }, // Pacific/Easter
    {   10, 3902,    8237,  5, }, // Pacific/Fiji
    {   10, 3822,    8242, 11, }, // Pacific/Guadalcanal
    {   10, 2723,    4493,  8, }, // Pacific/Honolulu
    {   10, 3049,    4501,  8, }, // Pacific/Kanton
    {   10, 3998,    8253,  9, }, // Pacific/Kiritimati
    {   10, 7019,    8262,  5, }, // Pacific/Kosrae
    {   10, 2842,    8267, 10, }, // Pacific/Kwajalein
    {   10, 4877,    8277,  6, }, // Pacific/Majuro
    {   10, 4051,    8283,  6, }, // Pacific/Marquesas
    {   10, 8914,    8289,  6, }, // Pacific/Midway
    {   10, 3126,    8295,  9, }, // Pacific/Pago_Pago
    {   10, 7194,    8304,  7, }, // Pacific/Pitcairn
    {   10, 3096,    8311,  7, }, // Pacific/Pohnpei
    {   10, 4807,    8318, 12, }, // Pacific/Port_Moresby
    {   10, 6076,    8330,  9, }, // Pacific/Rarotonga
    {   10, 6712,    8339,  6, }, // Pacific/Tarawa
    {   10, 7392,    8345,  4, }, // Pacific/Wake
    {   10, 7405,    8349,  6, }, // Pacific/Wallis
    {   13, 6788,    8355,  4, }, // Africa/Abidjan Amharic/Ethiopic/Ethiopia
    {   13, 6760,    8359,  3, }, // Africa/Accra
    {   13, 5059,    8362,  7, }, // Africa/Addis_Ababa
    {   13, 7420,    8369,  5, }, // Africa/Algiers
    {   13,   14,    8374,  4, }, // Africa/Asmara
    {   13,   44,    8378,  3, }, // Africa/Bamako
    {   13, 5257,    8381,  4, }, // Africa/Bangui
    {   13, 6746,    8385,  4, }, // Africa/Banjul
    {   13, 7435,    8389,  3, }, // Africa/Bissau
    {   13, 4957,    8392,  6, }, // Africa/Blantyre
    {   13, 5287,    8398,  5, }, // Africa/Brazzaville
    {   13, 4922,    8403,  5, }, // Africa/Bujumbura
    {   13, 2320,    8408,  3, }, // Africa/Cairo
    {   13, 4117,    8411,  6, }, // Africa/Casablanca
    {   13, 7449,    8417,  3, }, // Africa/Ceuta
    {   13, 6773,    8420,  4, }, // Africa/Conakry
    {   13, 6840,    8424,  3, }, // Africa/Dakar
    {   13, 5130,    8427,  6, }, // Africa/Dar_es_Salaam
    {   13, 5043,    8433,  3, }, // Africa/Djibouti
    {   13, 5243,    8436,  3, }, // Africa/Douala
    {   13, 5166,    8439,  7, }, // Africa/El_Aaiun
    {   13, 6853,    8446,  5, }, // Africa/Freetown
    {   13, 4906,    8451,  4, }, // Africa/Gaborone
    {   13, 5015,    8455,  3, }, // Africa/Harare
    {   13, 4479,    8458,  7, }, // Africa/Johannesburg
    {   13, 4499,    8465,  2, }, // Africa/Juba
    {   13, 5151,    8467,  4, }, // Africa/Kampala
    {   13, 4524,    8471,  4, }, // Africa/Khartoum
    {   13, 4973,    8475,  3, }, // Africa/Kigali
    {   13, 5306,    8478,  4, }, // Africa/Kinshasa
    {   13, 4744,    8482,  3, }, // Africa/Lagos
    {   13, 5336,    8485,  5, }, // Africa/Libreville
    {   13, 6869,    8490,  2, }, // Africa/Lome
    {   13, 5211,    8492,  4, }, // Africa/Luanda
    {   13, 4939,    8496,  5, }, // Africa/Lubumbashi
    {   13, 5001,    8501,  3, }, // Africa/Lusaka
    {   13, 5322,    8504,  3, }, // Africa/Malabo
    {   13, 4987,    8507,  3, }, // Africa/Maputo
    {   13, 5197,    8510,  3, }, // Africa/Maseru
    {   13, 5182,    8513,  4, }, // Africa/Mbabane
    {   13, 5113,    8517,  4, }, // Africa/Mogadishu
    {   13, 7034,    8521,  5, }, // Africa/Monrovia
    {   13, 3853,    8526,  4, }, // Africa/Nairobi
    {   13, 5271,    8530,  4, }, // Africa/Ndjamena
    {   13, 5354,    8534,  4, }, // Africa/Niamey
    {   13, 6803,    8538,  5, }, // Africa/Nouakchott
    {   13, 6727,    8543,  4, }, // Africa/Ouagadougou
    {   13, 5225,    8547,  8, }, // Africa/Porto-Novo
    {   13, 4435,    8555,  5, }, // Africa/Sao_Tome
    {   13, 2866,    8560,  4, }, // Africa/Tripoli
    {   13, 6554,    8564,  3, }, // Africa/Tunis
    {   13, 4152,    8567,  5, }, // Africa/Windhoek
    {   13,  132,    8572,  3, }, // America/Adak
    {   13, 3346,    8575,  5, }, // America/Anchorage
    {   13, 5607,    8580,  5, }, // America/Anguilla
    {   13, 5624,    8585,  4, }, // America/Antigua
    {   13, 4554,    8589,  5, }, // America/Araguaina
    {   13,  166,    8594,  8, }, // America/Argentina/Buenos_Aires
    {   13,   91,    8602,  5, }, // America/Argentina/Catamarca
    {   13,  270,    8607,  4, }, // America/Argentina/Cordoba
    {   13,  441,    8611,  3, }, // America/Argentina/Jujuy
    {   13, 7462,    8614,  5, }, // America/Argentina/La_Rioja
    {   13,  565,    8619,  4, }, // America/Argentina/Mendoza
    {   13, 7489,    8623,  7, }, // America/Argentina/Rio_Gallegos
    {   13, 7520,    8630,  3, }, // America/Argentina/Salta
    {   13, 7544,    8633,  6, }, // America/Argentina/San_Juan
    {   13, 5580,    8639,  6, }, // America/Argentina/San_Luis
    {   13, 7571,    8645,  4, }, // America/Argentina/Tucuman
    {   13, 7597,    8649,  4, }, // America/Argentina/Ushuaia
    {   13, 5640,    8653,  3, }, // America/Aruba
    {   13, 4252,    8656,  6, }, // America/Asuncion
    {   13,  237,    8662,  5, }, // America/Atikokan
    {   13, 3679,    8667,  3, }, // America/Bahia
    {   13, 7623,    8670,  9, }, // America/Bahia_Banderas
    {   13, 5654,    8679,  5, }, // America/Barbados
    {   13, 7646,    8684,  3, }, // America/Belem
    {   13, 5407,    8687,  3, }, // America/Belize
    {   13, 7660,    8690, 10, }, // America/Blanc-Sablon
    {   13, 7681,    8700,  6, }, // America/Boa_Vista
    {   13, 4361,    8706,  3, }, // America/Bogota
    {   13, 7699,    8709,  3, }, // America/Boise
    {   13, 7713,    8712,  8, }, // America/Cambridge_Bay
    {   13, 7735,    8720,  8, }, // America/Campo_Grande
    {   13, 3868,    8728,  4, }, // America/Cancun
    {   13, 4694,    8732,  4, }, // America/Caracas
    {   13, 4345,    8736,  4, }, // America/Cayenne
    {   13, 5496,    8740,  4, }, // America/Cayman
    {   13, 2260,    8744,  3, }, // America/Chicago
    {   13, 7756,    8747,  5, }, // America/Chihuahua
    {   13, 7774,    8752,  9, }, // America/Ciudad_Juarez
    {   13, 5422,    8761,  5, }, // America/Costa_Rica
    {   13, 7796,    8766,  5, }, // America/Creston
    {   13, 3791,    8771,  4, }, // America/Cuiaba
    {   13, 5723,    8775,  4, }, // America/Curacao
    {   13, 7812,    8779,  8, }, // America/Danmarkshavn
    {   13, 7833,    8787,  4, }, // America/Dawson
    {   13, 7848,    8791,  8, }, // America/Dawson_Creek
    {   13,  805,    8799,  4, }, // America/Denver
    {   13, 3465,    8803,  5, }, // America/Detroit
    {   13, 5739,    8808,  4, }, // America/Dominica
    {   13,  893,    8812,  6, }, // America/Edmonton
    {   13, 7869,    8818,  4, }, // America/Eirunepe
    {   13, 5441,    8822,  7, }, // America/El_Salvador
    {   13, 7886,    8829,  8, }, // America/Fort_Nelson
    {   13, 7906,    8837,  5, }, // America/Fortaleza
    {   13, 7924,    8842,  6, }, // America/Glace_Bay
    {   13, 6881,    8848,  5, }, // America/Goose_Bay
    {   13, 4612,    8853,  8, }, // America/Grand_Turk
    {   13, 5770,    8861,  4, }, // America/Grenada
    {   13, 5786,    8865,  4, }, // America/Guadeloupe
    {   13, 3760,    8869,  4, }, // America/Guatemala
    {   13, 4841,    8873,  5, }, // America/Guayaquil
    {   13, 6932,    8878,  3, }, // America/Guyana
    {   13, 1939,    8881,  5, }, // America/Halifax
    {   13, 2281,    8886,  3, }, // America/Havana
    {   13, 7942,    8889,  5, }, // America/Hermosillo
    {   13,  348,    8894,  8, }, // America/Indiana/Indianapolis
    {   13,  481,    8902, 10, }, // America/Indiana/Knox
    {   13, 7961,    8912, 11, }, // America/Indiana/Marengo
    {   13, 7985,    8923, 14, }, // America/Indiana/Petersburg
    {   13, 8012,    8937, 13, }, // America/Indiana/Tell_City
    {   13, 8038,    8950, 10, }, // America/Indiana/Vevay
    {   13, 8060,    8960, 12, }, // America/Indiana/Vincennes
    {   13, 8086,    8972, 11, }, // America/Indiana/Winamac
    {   13, 8110,    8983,  4, }, // America/Inuvik
    {   13,  660,    8987,  5, }, // America/Iqaluit
    {   13, 2799,    8992,  4, }, // America/Jamaica
    {   13, 5380,    8996,  3, }, // America/Juneau
    {   13,  521,    8999,  5, }, // America/Kentucky/Louisville
    {   13, 8125,    9004, 11, }, // America/Kentucky/Monticello
    {   13, 5704,    9015,  7, }, // America/Kralendijk
    {   13, 4376,    9022,  4, }, // America/La_Paz
    {   13, 7169,    9026,  2, }, // America/Lima
    {   13, 3239,    9028,  8, }, // America/Los_Angeles
    {   13, 5932,    9036, 12, }, // America/Lower_Princes
    {   13, 8153,    9048,  3, }, // America/Maceio
    {   13, 8168,    9051,  4, }, // America/Managua
    {   13, 1908,    9055,  4, }, // America/Manaus
    {   13, 5897,    9059,  4, }, // America/Marigot
    {   13, 5805,    9063,  5, }, // America/Martinique
    {   13, 8184,    9068,  5, }, // America/Matamoros
    {   13, 2917,    9073,  5, }, // America/Mazatlan
    {   13, 8202,    9078,  4, }, // America/Menominee
    {   13, 8220,    9082,  3, }, // America/Merida
    {   13, 8235,    9085,  6, }, // America/Metlakatla
    {   13, 2949,    9091,  8, }, // America/Mexico_City
    {   13, 4391,    9099,  5, }, // America/Miquelon
    {   13, 8254,    9104,  5, }, // America/Moncton
    {   13, 8270,    9109,  6, }, // America/Monterrey
    {   13, 4098,    9115,  6, }, // America/Montevideo
    {   13, 5824,    9121,  6, }, // America/Montserrat
    {   13, 5481,    9127,  3, }, // America/Nassau
    {   13, 2379,    9130,  5, }, // America/New_York
    {   13, 8288,    9135,  2, }, // America/Nome
    {   13, 1850,    9137,  3, }, // America/Noronha
    {   13, 8301,    9140, 13, }, // America/North_Dakota/Beulah
    {   13, 8329,    9153, 14, }, // America/North_Dakota/Center
    {   13, 8357,    9167, 16, }, // America/North_Dakota/New_Salem
    {   13,  393,    9183,  4, }, // America/Nuuk
    {   13, 8388,    9187,  4, }, // America/Ojinaga
    {   13, 2356,    9191,  3, }, // America/Panama
    {   13, 6173,    9194,  5, }, // America/Paramaribo
    {   13, 2973,    9199,  4, }, // America/Phoenix
    {   13, 3945,    9203, 10, }, // America/Port-au-Prince
    {   13, 5954,    9213,  9, }, // America/Port_of_Spain
    {   13, 8404,    9222,  7, }, // America/Porto_Velho
    {   13, 5843,    9229,  5, }, // America/Puerto_Rico
    {   13, 4030,    9234,  8, }, // America/Punta_Arenas
    {   13, 8424,    9242,  9, }, // America/Rankin_Inlet
    {   13, 8445,    9251,  3, }, // America/Recife
    {   13, 1995,    9254,  3, }, // America/Regina
    {   13, 8460,    9257,  5, }, // America/Resolute
    {   13,  695,    9262,  7, }, // America/Rio_Branco
    {   13, 8477,    9269,  5, }, // America/Santarem
    {   13, 2201,    9274,  5, }, // America/Santiago
    {   13, 6111,    9279,  8, }, // America/Santo_Domingo
    {   13, 1878,    9287,  5, }, // America/Sao_Paulo
    {   13, 6899,    9292,  9, }, // America/Scoresbysund
    {   13, 8494,    9301,  3, }, // America/Sitka
    {   13, 8508,    9304, 10, }, // America/St_Barthelemy
    {   13, 2061,    9314,  8, }, // America/St_Johns
    {   13, 5863,    9322,  7, }, // America/St_Kitts
    {   13, 5880,    9329,  8, }, // America/St_Lucia
    {   13,  855,    9337,  7, }, // America/St_Thomas
    {   13, 5913,    9344,  9, }, // America/St_Vincent
    {   13, 8530,    9353,  9, }, // America/Swift_Current
    {   13, 5461,    9362,  6, }, // America/Tegucigalpa
    {   13, 5756,    9368,  2, }, // America/Thule
    {   13,  313,    9370,  4, }, // America/Tijuana
    {   13,  608,    9374,  4, }, // America/Toronto
    {   13, 5688,    9378,  4, }, // America/Tortola
    {   13, 2093,    9382,  5, }, // America/Vancouver
    {   13, 2144,    9387,  6, }, // America/Whitehorse
    {   13,  734,    9393,  4, }, // America/Winnipeg
    {   13, 8552,    9397,  4, }, // America/Yakutat
    {   13, 6001,    9401,  3, }, // Antarctica/Casey
    {   13, 6094,    9404,  3, }, // Antarctica/Davis
    {   13, 6133,    9407, 10, }, // Antarctica/DumontDUrville
    {   13, 8568,    9417,  3, }, // Antarctica/Macquarie
    {   13, 7066,    9420,  4, }, // Antarctica/Mawson
    {   13, 7113,    9424,  5, }, // Antarctica/McMurdo
    {   13, 8589,    9429,  4, }, // Antarctica/Palmer
    {   13, 7226,    9433,  3, }, // Antarctica/Rothera
    {   13, 7295,    9436,  3, }, // Antarctica/Syowa
    {   13, 8607,    9439,  3, }, // Antarctica/Troll
    {   13, 7374,    9442,  4, }, // Antarctica/Vostok
    {   13, 1429,    9446,  8, }, // Arctic/Longyearbyen
    {   13, 5570,    9454,  3, }, // Asia/Aden
    {   13, 5395,    9457,  5, }, // Asia/Almaty
    {   13, 3968,    9462,  3, }, // Asia/Amman
    {   13, 5511,    9465,  5, }, // Asia/Anadyr
    {   13, 5523,    9470,  4, }, // Asia/Aqtau
    {   13, 5368,    9474,  4, }, // Asia/Aqtobe
    {   13,  964,    9478,  5, }, // Asia/Ashgabat
    {   13, 8624,    9483,  5, }, // Asia/Atyrau
    {   13, 3607,    9488,  4, }, // Asia/Baghdad
    {   13, 5534,    9492,  4, }, // Asia/Bahrain
    {   13, 3653,    8774,  2, }, // Asia/Baku
    {   13, 4466,    9496,  4, }, // Asia/Bangkok
    {   13, 3571,    9500,  5, }, // Asia/Barnaul
    {   13, 4086,    9505,  3, }, // Asia/Beirut
    {   13, 3778,    9508,  4, }, // Asia/Bishkek
    {   13, 5989,    9512,  4, }, // Asia/Brunei
    {   13, 4601,    9516,  2, }, // Asia/Chita
    {   13, 4511,    9518,  4, }, // Asia/Colombo
    {   13, 4540,    9522,  4, }, // Asia/Damascus
    {   13, 1093,    8424,  2, }, // Asia/Dhaka
    {   13, 6192,    9526,  2, }, // Asia/Dili
    {   13, 3596,    9528,  3, }, // Asia/Dubai
    {   13, 6159,    9531,  4, }, // Asia/Dushanbe
    {   13, 8636,    9535,  5, }, // Asia/Famagusta
    {   13, 8651,    9540,  2, }, // Asia/Gaza
    {   13, 4795,    9542,  4, }, // Asia/Hebron
    {   13, 1259,    9546, 11, }, // Asia/Ho_Chi_Minh
    {   13, 2704,    9557,  7, }, // Asia/Hong_Kong
    {   13, 4771,    9564,  3, }, // Asia/Hovd
    {   13, 4184,    9567,  6, }, // Asia/Irkutsk
    {   13, 7006,    9573,  4, }, // Asia/Jakarta
    {   13, 6992,    9577,  4, }, // Asia/Jayapura
    {   13, 1290,    9581,  6, }, // Asia/Jerusalem
    {   13, 3560,    9587,  3, }, // Asia/Kabul
    {   13, 4316,    9590,  5, }, // Asia/Kamchatka
    {   13, 4239,    9595,  3, }, // Asia/Karachi
    {   13, 1185,    9598,  5, }, // Asia/Kathmandu
    {   13, 8661,    9603,  4, }, // Asia/Khandyga
    {   13,  992,    9607,  4, }, // Asia/Kolkata
    {   13, 4197,    9611,  8, }, // Asia/Krasnoyarsk
    {   13, 4859,    9619,  8, }, // Asia/Kuala_Lumpur
    {   13, 5976,    9627,  4, }, // Asia/Kuching
    {   13, 5547,    9631,  3, }, // Asia/Kuwait
    {   13, 1211,    9634,  3, }, // Asia/Macau
    {   13, 4017,    9637,  4, }, // Asia/Magadan
    {   13, 1349,    9641,  4, }, // Asia/Makassar
    {   13, 7182,    9645,  3, }, // Asia/Manila
    {   13, 6920,    9648,  4, }, // Asia/Muscat
    {   13, 2554,    9652,  4, }, // Asia/Nicosia
    {   13, 8675,    9656,  7, }, // Asia/Novokuznetsk
    {   13, 4135,    9663,  7, }, // Asia/Novosibirsk
    {   13, 4229,    9670,  4, }, // Asia/Omsk
    {   13, 7145,    9674,  3, }, // Asia/Oral
    {   13, 6961,    9677,  7, }, // Asia/Phnom_Penh
    {   13, 8693,    9684,  6, }, // Asia/Pontianak
    {   13, 4214,    9690,  7, }, // Asia/Pyongyang
    {   13, 5559,    9697,  3, }, // Asia/Qatar
    {   13, 8708,    9700,  5, }, // Asia/Qostanay
    {   13, 4269,    9705,  6, }, // Asia/Qyzylorda
    {   13, 3584,    9711,  3, }, // Asia/Riyadh
    {   13, 4408,    9714,  5, }, // Asia/Sakhalin
    {   13, 7245,    9719,  6, }, // Asia/Samarkand
    {   13, 3279,    9725,  3, }, // Asia/Seoul
    {   13, 1053,    9728,  4, }, // Asia/Shanghai
    {   13, 3300,    9732,  5, }, // Asia/Singapore
    {   13, 4297,    9737, 10, }, // Asia/Srednekolymsk
    {   13, 3263,    9747,  4, }, // Asia/Taipei
    {   13, 4781,    9751,  5, }, // Asia/Tashkent
    {   13, 3915,    9756,  4, }, // Asia/Tbilisi
    {   13, 2772,    9760,  4, }, // Asia/Tehran
    {   13, 1317,    9764,  3, }, // Asia/Thimphu
    {   13, 2821,    9767,  3, }, // Asia/Tokyo
    {   13, 4572,    9770,  4, }, // Asia/Tomsk
    {   13, 1021,    9774,  8, }, // Asia/Ulaanbaatar
    {   13, 1159,    9782,  4, }, // Asia/Urumqi
    {   13, 8722,    9786,  6, }, // Asia/Ust-Nera
    {   13, 6977,    9792,  5, }, // Asia/Vientiane
    {   13, 4710,    9797,  7, }, // Asia/Vladivostok
    {   13, 4828,    9804,  5, }, // Asia/Yakutsk
    {   13, 1235,    9809,  4, }, // Asia/Yangon
    {   13, 3883,    9813,  8, }, // Asia/Yekaterinburg
    {   13, 3747,    9821,  4, }, // Asia/Yerevan
    {   13, 3663,    9825,  4, }, // Atlantic/Azores
    {   13, 5671,    9829,  4, }, // Atlantic/Bermuda
    {   13, 6628,    9833,  3, }, // Atlantic/Canary
    {   13, 3727,    9836,  6, }, // Atlantic/Cape_Verde
    {   13, 1395,    9842,  3, }, // Atlantic/Faroe
    {   13, 8736,    9845,  3, }, // Atlantic/Madeira
    {   13, 2748,    9848,  6, }, // Atlantic/Reykjavik
    {   13, 7272,    9854,  8, }, // Atlantic/South_Georgia
    {   13, 6821,    9862,  8, }, // Atlantic/St_Helena
    {   13, 6644,    9870,  5, }, // Atlantic/Stanley
    {   13, 1670,    9875,  5, }, // Australia/Adelaide
    {   13, 1635,    9880,  5, }, // Australia/Brisbane
    {   13, 1799,    9885,  7, }, // Australia/Broken_Hill
    {   13, 1583,    9892,  4, }, // Australia/Darwin
    {   13, 3637,    9896,  3, }, // Australia/Eucla
    {   13, 1516,    9899,  4, }, // Australia/Hobart
    {   13, 8753,    9903,  5, }, // Australia/Lindeman
    {   13, 1547,    9908,  6, }, // Australia/Lord_Howe
    {   13, 1727,    9914,  5, }, // Australia/Melbourne
    {   13, 1762,    9919,  3, }, // Australia/Perth
    {   13, 1463,    9922,  3, }, // Australia/Sydney
    {   13, 6441,    9925,  7, }, // Europe/Amsterdam
    {   13, 6216,    9932,  4, }, // Europe/Andorra
    {   13, 3620,    9936,  7, }, // Europe/Astrakhan
    {   13, 2300,    9943,  4, }, // Europe/Athens
    {   13, 6336,    9947,  5, }, // Europe/Belgrade
    {   13, 4757,    9952,  4, }, // Europe/Berlin
    {   13, 6488,    9956,  6, }, // Europe/Bratislava
    {   13, 2167,    9962,  5, }, // Europe/Brussels
    {   13, 3928,    9967,  5, }, // Europe/Bucharest
    {   13, 3806,    9972,  5, }, // Europe/Budapest
    {   13, 8772,    9977,  5, }, // Europe/Busingen
    {   13, 2583,    9982,  4, }, // Europe/Chisinau
    {   13, 6289,    9986,  6, }, // Europe/Copenhagen
    {   13, 2338,    9992,  4, }, // Europe/Dublin
    {   13, 6307,    9996,  6, }, // Europe/Gibraltar
    {   13, 8788,   10002,  4, }, // Europe/Guernsey
    {   13, 6612,   10006,  5, }, // Europe/Helsinki
    {   13, 8804,   10011, 10, }, // Europe/Isle_of_Man
    {   13, 1130,   10021,  6, }, // Europe/Istanbul
    {   13, 8823,   10027,  3, }, // Europe/Jersey
    {   13, 3979,   10030,  7, }, // Europe/Kaliningrad
    {   13, 8929,   10037,  3, }, // Europe/Kirov
    {   13, 2527,   10040,  3, }, // Europe/Kyiv
    {   13, 3213,   10043,  4, }, // Europe/Lisbon
    {   13, 6506,   10047,  6, }, // Europe/Ljubljana
    {   13, 2501,   10053,  4, }, // Europe/London
    {   13, 6365,   10057,  7, }, // Europe/Luxembourg
    {   13, 4892,   10064,  4, }, // Europe/Madrid
    {   13, 6397,   10068,  3, }, // Europe/Malta
    {   13, 6582,   10071,  5, }, // Europe/Mariehamn
    {   13, 3693,   10076,  4, }, // Europe/Minsk
    {   13, 6410,   10080,  3, }, // Europe/Monaco
    {   13, 3537,   10083,  3, }, // Europe/Moscow
    {   13, 6458,   10086,  3, }, // Europe/Oslo
    {   13, 4284,   10089,  3, }, // Europe/Paris
    {   13, 6424,   10092,  5, }, // Europe/Podgorica
    {   13, 6275,   10097,  3, }, // Europe/Prague
    {   13, 8837,   10100,  2, }, // Europe/Riga
    {   13, 6324,   10102,  2, }, // Europe/Rome
    {   13, 4331,   10104,  3, }, // Europe/Samara
    {   13, 6470,   10107,  6, }, // Europe/San_Marino
    {   13, 6245,   10113,  4, }, // Europe/Sarajevo
    {   13, 4451,   10117,  4, }, // Europe/Saratov
    {   13, 8849,   10121,  6, }, // Europe/Simferopol
    {   13, 6383,   10127,  4, }, // Europe/Skopje
    {   13, 6599,   10131,  3, }, // Europe/Sofia
    {   13, 6523,   10134,  6, }, // Europe/Stockholm
    {   13, 8867,   10140,  3, }, // Europe/Tallinn
    {   13, 6202,   10143,  3, }, // Europe/Tirane
    {   13, 8882,   10146,  7, }, // Europe/Ulyanovsk
    {   13, 6352,   10153,  3, }, // Europe/Vaduz
    {   13, 6567,   10156,  4, }, // Europe/Vatican
    {   13, 6231,   10160,  3, }, // Europe/Vienna
    {   13, 8899,   10163,  4, }, // Europe/Vilnius
    {   13, 4727,   10167,  5, }, // Europe/Volgograd
    {   13, 3190,   10172,  4, }, // Europe/Warsaw
    {   13, 6261,   10176,  4, }, // Europe/Zagreb
    {   13, 6540,   10180,  3, }, // Europe/Zurich
    {   13, 5078,   10183,  7, }, // Indian/Antananarivo
    {   13, 6947,   10190,  3, }, // Indian/Chagos
    {   13, 6046,   10193,  2, }, // Indian/Christmas
    {   13, 6063,   10195,  3, }, // Indian/Cocos
    {   13, 5029,   10198,  3, }, // Indian/Comoro
    {   13, 6661,   10201,  5, }, // Indian/Kerguelen
    {   13, 7260,   10206,  2, }, // Indian/Mahe
    {   13, 7050,   10208,  4, }, // Indian/Maldives
    {   13, 4069,   10212,  5, }, // Indian/Mauritius
    {   13, 5098,   10217,  3, }, // Indian/Mayotte
    {   13, 7211,   10220,  5, }, // Indian/Reunion
    {   13, 4422,   10225,  3, }, // Pacific/Apia
    {   13,  932,   10228,  5, }, // Pacific/Auckland
    {   13, 3706,   10233,  6, }, // Pacific/Bougainville
    {   13, 3015,   10239,  3, }, // Pacific/Chatham
    {   13, 3157,   10242,  2, }, // Pacific/Chuuk
    {   13, 2237,   10244,  3, }, // Pacific/Easter
    {   13, 7360,   10247,  3, }, // Pacific/Efate
    {   13, 7327,   10250,  4, }, // Pacific/Fakaofo
    {   13, 3902,   10254,  2, }, // Pacific/Fiji
    {   13, 7343,   10256,  4, }, // Pacific/Funafuti
    {   13, 6678,   10260,  5, }, // Pacific/Galapagos
    {   13, 6696,   10265,  5, }, // Pacific/Gambier
    {   13, 3822,   10270,  7, }, // Pacific/Guadalcanal
    {   13, 6018,   10277,  3, }, // Pacific/Guam
    {   13, 2723,   10280,  4, }, // Pacific/Honolulu
    {   13, 3049,   10284,  6, }, // Pacific/Kanton
    {   13, 3998,   10290,  5, }, // Pacific/Kiritimati
    {   13, 7019,   10295,  3, }, // Pacific/Kosrae
    {   13, 2842,   10298,  5, }, // Pacific/Kwajalein
    {   13, 4877,   10303,  3, }, // Pacific/Majuro
    {   13, 4051,   10306,  6, }, // Pacific/Marquesas
    {   13, 8914,   10312,  4, }, // Pacific/Midway
    {   13, 7084,   10316,  3, }, // Pacific/Nauru
    {   13, 7132,   10319,  2, }, // Pacific/Niue
    {   13, 4168,   10321,  5, }, // Pacific/Norfolk
    {   13, 7098,   10326,  4, }, // Pacific/Noumea
    {   13, 3126,   10330,  5, }, // Pacific/Pago_Pago
    {   13, 7155,   10335,  3, }, // Pacific/Palau
    {   13, 7194,   10338,  6, }, // Pacific/Pitcairn
    {   13, 3096,   10344,  4, }, // Pacific/Pohnpei
    {   13, 4807,   10348,  9, }, // Pacific/Port_Moresby
    {   13, 6076,   10357,  5, }, // Pacific/Rarotonga
    {   13, 6031,   10362,  4, }, // Pacific/Saipan
    {   13, 7312,   10366,  3, }, // Pacific/Tahiti
    {   13, 6712,   10369,  3, }, // Pacific/Tarawa
    {   13, 4583,   10372,  5, }, // Pacific/Tongatapu
    {   13, 7392,   10377,  2, }, // Pacific/Wake
    {   13, 7405,   10379,  3, }, // Pacific/Wallis
    {   14, 6788,   10382,  7, }, // Africa/Abidjan Arabic/Arabic/Egypt
    {   14, 6760,   10389,  4, }, // Africa/Accra
    {   14, 5059,   10393, 10, }, // Africa/Addis_Ababa
    {   14, 7420,   10403,  7, }, // Africa/Algiers
    {   14,   14,   10410,  5, }, // Africa/Asmara
    {   14,   44,   10415,  6, }, // Africa/Bamako
    {   14, 5257,   10421,  6, }, // Africa/Bangui
    {   14, 6746,   10427,  6, }, // Africa/Banjul
    {   14, 7435,   10433,  5, }, // Africa/Bissau
    {   14, 4957,   10438,  8, }, // Africa/Blantyre
    {   14, 5287,   10446,  8, }, // Africa/Brazzaville
    {   14, 4922,   10454,  9, }, // Africa/Bujumbura
    {   14, 2320,   10463,  7, }, // Africa/Cairo
    {   14, 4117,   10470, 13, }, // Africa/Casablanca
    {   14, 7449,   10483,  4, }, // Africa/Ceuta
    {   14, 6773,   10487,  7, }, // Africa/Conakry
    {   14, 6840,   10494,  5, }, // Africa/Dakar
    {   14, 5130,   10499, 10, }, // Africa/Dar_es_Salaam
    {   14, 5043,   10509,  6, }, // Africa/Djibouti
    {   14, 5243,   10515,  5, }, // Africa/Douala
    {   14, 5166,   10520,  6, }, // Africa/El_Aaiun
    {   14, 6853,   10526,  8, }, // Africa/Freetown
    {   14, 4906,   10534,  7, }, // Africa/Gaborone
    {   14, 5015,   10541,  5, }, // Africa/Harare
    {   14, 4479,   10546,  9, }, // Africa/Johannesburg
    {   14, 4499,   10555,  4, }, // Africa/Juba
    {   14, 5151,   10559,  7, }, // Africa/Kampala
    {   14, 4524,   10566,  7, }, // Africa/Khartoum
    {   14, 4973,   10573,  6, }, // Africa/Kigali
    {   14, 5306,   10579,  7, }, // Africa/Kinshasa
    {   14, 4744,   10586,  5, }, // Africa/Lagos
    {   14, 5336,   10591,  7, }, // Africa/Libreville
    {   14, 6869,   10598,  4, }, // Africa/Lome
    {   14, 5211,   10602,  6, }, // Africa/Luanda
    {   14, 4939,   10608,  8, }, // Africa/Lubumbashi
    {   14, 5001,   10616,  6, }, // Africa/Lusaka
    {   14, 5322,   10622,  6, }, // Africa/Malabo
    {   14, 4987,   10628,  6, }, // Africa/Maputo
    {   14, 5197,   10634,  6, }, // Africa/Maseru
    {   14, 5182,   10640,  7, }, // Africa/Mbabane
    {   14, 5113,   10647,  7, }, // Africa/Mogadishu
    {   14, 7034,   10654,  8, }, // Africa/Monrovia
    {   14, 3853,   10662,  6, }, // Africa/Nairobi
    {   14, 5271,   10668,  7, }, // Africa/Ndjamena
    {   14, 5354,   10675,  5, }, // Africa/Niamey
    {   14, 6803,   10680,  7, }, // Africa/Nouakchott
    {   14, 6727,   10687,  8, }, // Africa/Ouagadougou
    {   14, 5225,   10695, 10, }, // Africa/Porto-Novo
    {   14, 4435,   10705,  8, }, // Africa/Sao_Tome
    {   14, 2866,   10713,  6, }, // Africa/Tripoli
    {   14, 6554,   10719,  4, }, // Africa/Tunis
    {   14, 4152,   10723,  7, }, // Africa/Windhoek
    {   14,  132,   10730,  4, }, // America/Adak
    {   14, 3346,   10734,  7, }, // America/Anchorage
    {   14, 5607,   10741,  7, }, // America/Anguilla
    {   14, 5624,   10748,  7, }, // America/Antigua
    {   14, 4554,   10755,  9, }, // America/Araguaina
    {   14,  166,   10764, 11, }, // America/Argentina/Buenos_Aires
    {   14,   91,   10775,  9, }, // America/Argentina/Catamarca
    {   14,  270,   10784,  7, }, // America/Argentina/Cordoba
    {   14,  441,   10791,  4, }, // America/Argentina/Jujuy
    {   14, 7462,   10795,  8, }, // America/Argentina/La_Rioja
    {   14,  565,   10803,  7, }, // America/Argentina/Mendoza
    {   14, 7489,   10810, 11, }, // America/Argentina/Rio_Gallegos
    {   14, 7520,   10821,  5, }, // America/Argentina/Salta
    {   14, 7544,   10826,  8, }, // America/Argentina/San_Juan
    {   14, 5580,   10834,  8, }, // America/Argentina/San_Luis
    {   14, 7571,   10842,  6, }, // America/Argentina/Tucuman
    {   14, 7597,   10848,  4, }, // America/Argentina/Ushuaia
    {   14, 5640,   10852,  5, }, // America/Aruba
    {   14, 4252,   10857,  8, }, // America/Asuncion
    {   14,  237,   10865, 11, }, // America/Atikokan
    {   14, 3679,   10876,  5, }, // America/Bahia
    {   14, 7623,   10881, 14, }, // America/Bahia_Banderas
    {   14, 5654,   10895,  7, }, // America/Barbados
    {   14, 7646,   10902,  3, }, // America/Belem
    {   14, 5407,   10905,  4, }, // America/Belize
    {   14, 7660,   10909, 12, }, // America/Blanc-Sablon
    {   14, 7681,   10921,  9, }, // America/Boa_Vista
    {   14, 4361,   10930,  6, }, // America/Bogota
    {   14, 7699,   10936,  4, }, // America/Boise
    {   14, 7713,   10940, 12, }, // America/Cambridge_Bay
    {   14, 7735,   10952, 11, }, // America/Campo_Grande
    {   14, 3868,   10963,  6, }, // America/Cancun
    {   14, 4694,   10969,  7, }, // America/Caracas
    {   14, 4345,   10976,  5, }, // America/Cayenne
    {   14, 5496,   10981,  6, }, // America/Cayman
    {   14, 2260,   10987,  6, }, // America/Chicago
    {   14, 7756,   10993,  7, }, // America/Chihuahua
    {   14, 7774,   11000, 13, }, // America/Ciudad_Juarez
    {   14, 5422,   11013,  9, }, // America/Costa_Rica
    {   14, 7796,   11022,  7, }, // America/Creston
    {   14, 3791,   11029,  5, }, // America/Cuiaba
    {   14, 5723,   11034,  7, }, // America/Curacao
    {   14, 7812,   11041, 11, }, // America/Danmarkshavn
    {   14, 7833,   11052,  6, }, // America/Dawson
    {   14, 7848,   11058, 10, }, // America/Dawson_Creek
    {   14,  805,   11068,  4, }, // America/Denver
    {   14, 3465,   11072,  7, }, // America/Detroit
    {   14, 5739,   11079,  8, }, // America/Dominica
    {   14,  893,   11087,  9, }, // America/Edmonton
    {   14, 7869,   11096,  7, }, // America/Eirunepe
    {   14, 5441,   11103,  9, }, // America/El_Salvador
    {   14, 7886,   11112, 11, }, // America/Fort_Nelson
    {   14, 7906,   11123,  9, }, // America/Fortaleza
    {   14, 7924,   11132,  8, }, // America/Glace_Bay
    {   14, 6881,   11140,  7, }, // America/Goose_Bay
    {   14, 4612,   11147,  9, }, // America/Grand_Turk
    {   14, 5770,   11156,  7, }, // America/Grenada
    {   14, 5786,   11163,  7, }, // America/Guadeloupe
    {   14, 3760,   11170,  9, }, // America/Guatemala
    {   14, 4841,   11179,  9, }, // America/Guayaquil
    {   14, 6932,   11188,  5, }, // America/Guyana
    {   14, 1939,   11193,  8, }, // America/Halifax
    {   14, 2281,   11201,  6, }, // America/Havana
    {   14, 7942,   11207,  9, }, // America/Hermosillo
    {   14,  348,   11216, 12, }, // America/Indiana/Indianapolis
    {   14,  481,   11228,  5, }, // America/Indiana/Knox
    {   14, 7961,   11233,  6, }, // America/Indiana/Marengo
    {   14, 7985,   11239,  8, }, // America/Indiana/Petersburg
    {   14, 8012,   11247, 17, }, // America/Indiana/Tell_City
    {   14, 8038,   11264,  5, }, // America/Indiana/Vevay
    {   14, 8060,   11269,  7, }, // America/Indiana/Vincennes
    {   14, 8086,   11276,  7, }, // America/Indiana/Winamac
    {   14, 8110,   11283,  7, }, // America/Inuvik
    {   14,  660,   11290,  6, }, // America/Iqaluit
    {   14, 2799,   11296,  7, }, // America/Jamaica
    {   14, 5380,   11303,  4, }, // America/Juneau
    {   14,  521,   11307,  8, }, // America/Kentucky/Louisville
    {   14, 8125,   11315,  9, }, // America/Kentucky/Monticello
    {   14, 5704,   11324,  8, }, // America/Kralendijk
    {   14, 4376,   11332,  6, }, // America/La_Paz
    {   14, 7169,   11338,  4, }, // America/Lima
    {   14, 3239,   11342, 10, }, // America/Los_Angeles
    {   14, 5932,   11352, 16, }, // America/Lower_Princes
    {   14, 8153,   11368,  5, }, // America/Maceio
    {   14, 8168,   11373,  7, }, // America/Managua
    {   14, 1908,   11380,  6, }, // America/Manaus
    {   14, 5897,   11386,  7, }, // America/Marigot
    {   14, 5805,   11393, 10, }, // America/Martinique
    {   14, 8184,   11403,  9, }, // America/Matamoros
    {   14, 2917,   11412,  8, }, // America/Mazatlan
    {   14, 8202,   11420,  8, }, // America/Menominee
    {   14, 8220,   11428,  6, }, // America/Merida
    {   14, 8235,   11434, 10, }, // America/Metlakatla
    {   14, 2949,   11444, 11, }, // America/Mexico_City
    {   14, 4391,   11455,  7, }, // America/Miquelon
    {   14, 8254,   11462,  7, }, // America/Moncton
    {   14, 8270,   11469,  7, }, // America/Monterrey
    {   14, 4098,   11476,  9, }, // America/Montevideo
    {   14, 5824,   11485,  9, }, // America/Montserrat
    {   14, 5481,   11494,  4, }, // America/Nassau
    {   14, 2379,   11498,  7, }, // America/New_York
    {   14, 8288,   11422,  3, }, // America/Nome
    {   14, 1850,   11505,  7, }, // America/Noronha
    {   14, 8301,   11512, 22, }, // America/North_Dakota/Beulah
    {   14, 8329,   11534,  4, }, // America/North_Dakota/Center
    {   14, 8357,   11538,  9, }, // America/North_Dakota/New_Salem
    {   14,  393,   11547,  6, }, // America/Nuuk
    {   14, 8388,   11553,  8, }, // America/Ojinaga
    {   14, 2356,   11561,  4, }, // America/Panama
    {   14, 6173,   11565, 10, }, // America/Paramaribo
    {   14, 2973,   11575,  5, }, // America/Phoenix
    {   14, 3945,   11580, 12, }, // America/Port-au-Prince
    {   14, 5954,   11592, 13, }, // America/Port_of_Spain
    {   14, 8404,   11605, 10, }, // America/Porto_Velho
    {   14, 5843,   11615,  9, }, // America/Puerto_Rico
    {   14, 4030,   11624, 12, }, // America/Punta_Arenas
    {   14, 8424,   11636, 10, }, // America/Rankin_Inlet
    {   14, 8445,   11646,  5, }, // America/Recife
    {   14, 1995,   11651,  6, }, // America/Regina
    {   14, 8460,   11657,  7, }, // America/Resolute
    {   14,  695,   11664,  9, }, // America/Rio_Branco
    {   14, 8477,   11673,  8, }, // America/Santarem
    {   14, 2201,   11681,  8, }, // America/Santiago
    {   14, 6111,   11689, 13, }, // America/Santo_Domingo
    {   14, 1878,   11702,  9, }, // America/Sao_Paulo
    {   14, 6899,   11711, 10, }, // America/Scoresbysund
    {   14, 8494,   11721,  5, }, // America/Sitka
    {   14, 8508,   11726, 14, }, // America/St_Barthelemy
    {   14, 2061,   11740,  9, }, // America/St_Johns
    {   14, 5863,   11749,  9, }, // America/St_Kitts
    {   14, 5880,   11758, 10, }, // America/St_Lucia
    {   14,  855,   11768, 10, }, // America/St_Thomas
    {   14, 5913,   11778, 10, }, // America/St_Vincent
    {   14, 8530,   11788, 10, }, // America/Swift_Current
    {   14, 5461,   11798, 11, }, // America/Tegucigalpa
    {   14, 5756,   11809,  3, }, // America/Thule
    {   14,  313,   11812,  7, }, // America/Tijuana
    {   14,  608,   11819,  7, }, // America/Toronto
    {   14, 5688,   11826,  7, }, // America/Tortola
    {   14, 2093,   11833,  7, }, // America/Vancouver
    {   14, 2144,   11840,  9, }, // America/Whitehorse
    {   14,  734,   11849,  7, }, // America/Winnipeg
    {   14, 8552,   11856,  7, }, // America/Yakutat
    {   14, 6001,   11863,  5, }, // Antarctica/Casey
    {   14, 6094,   11868,  5, }, // Antarctica/Davis
    {   14, 6133,   11873, 16, }, // Antarctica/DumontDUrville
    {   14, 8568,   11889,  7, }, // Antarctica/Macquarie
    {   14, 7066,   11896,  6, }, // Antarctica/Mawson
    {   14, 7113,   11902,  9, }, // Antarctica/McMurdo
    {   14, 8589,   11911,  6, }, // Antarctica/Palmer
    {   14, 7226,   11917,  6, }, // Antarctica/Rothera
    {   14, 7295,   11923,  6, }, // Antarctica/Syowa
    {   14, 8607,   11929,  4, }, // Antarctica/Troll
    {   14, 7374,   11933,  6, }, // Antarctica/Vostok
    {   14, 1429,   11939,  9, }, // Arctic/Longyearbyen
    {   14, 5570,   11948,  3, }, // Asia/Aden
    {   14, 5395,   11951,  6, }, // Asia/Almaty
    {   14, 3968,   11957,  5, }, // Asia/Amman
    {   14, 5511,   11962,  5, }, // Asia/Anadyr
    {   14, 5523,   11967,  5, }, // Asia/Aqtau
    {   14, 5368,   11972,  5, }, // Asia/Aqtobe
    {   14,  964,   11977,  8, }, // Asia/Ashgabat
    {   14, 8624,   11985,  6, }, // Asia/Atyrau
    {   14, 3607,   11991,  5, }, // Asia/Baghdad
    {   14, 5534,   11996,  7, }, // Asia/Bahrain
    {   14, 3653,   11032,  4, }, // Asia/Baku
    {   14, 4466,   12003,  6, }, // Asia/Bangkok
    {   14, 3571,   12009,  7, }, // Asia/Barnaul
    {   14, 4086,   12016,  5, }, // Asia/Beirut
    {   14, 3778,   12021,  5, }, // Asia/Bishkek
    {   14, 5989,   12026,  6, }, // Asia/Brunei
    {   14, 4601,   12032,  5, }, // Asia/Chita
    {   14, 4511,   12037,  7, }, // Asia/Colombo
    {   14, 4540,   12044,  4, }, // Asia/Damascus
    {   14, 1093,   10962,  3, }, // Asia/Dhaka
    {   14, 6192,   12048,  4, }, // Asia/Dili
    {   14, 3596,   12052,  3, }, // Asia/Dubai
    {   14, 6159,   12055,  7, }, // Asia/Dushanbe
    {   14, 8636,   12062,  9, }, // Asia/Famagusta
    {   14, 8651,   12071,  3, }, // Asia/Gaza
    {   14, 4795,   12074, 21, }, // Asia/Hebron
    {   14, 1259,   12095, 16, }, // Asia/Ho_Chi_Minh
    {   14, 2704,   12111,  9, }, // Asia/Hong_Kong
    {   14, 4771,   12120,  4, }, // Asia/Hovd
    {   14, 4184,   12124,  8, }, // Asia/Irkutsk
    {   14, 7006,   12132,  6, }, // Asia/Jakarta
    {   14, 6992,   12138,  9, }, // Asia/Jayapura
    {   14, 1290,   12147,  5, }, // Asia/Jerusalem
    {   14, 3560,   12152,  5, }, // Asia/Kabul
    {   14, 4316,   12157,  9, }, // Asia/Kamchatka
    {   14, 4239,   12166,  6, }, // Asia/Karachi
    {   14, 1185,   12172,  8, }, // Asia/Kathmandu
    {   14, 8661,   12180,  7, }, // Asia/Khandyga
    {   14,  992,   12187,  6, }, // Asia/Kolkata
    {   14, 4197,   12193, 11, }, // Asia/Krasnoyarsk
    {   14, 4859,   12204, 12, }, // Asia/Kuala_Lumpur
    {   14, 5976,   12216,  6, }, // Asia/Kuching
    {   14, 5547,   12222,  6, }, // Asia/Kuwait
    {   14, 1211,   12228,  5, }, // Asia/Macau
    {   14, 4017,   12233,  5, }, // Asia/Magadan
    {   14, 1349,   12238,  6, }, // Asia/Makassar
    {   14, 7182,   12244,  6, }, // Asia/Manila
    {   14, 6920,   12250,  4, }, // Asia/Muscat
    {   14, 2554,   12254,  7, }, // Asia/Nicosia
    {   14, 8675,   12261, 11, }, // Asia/Novokuznetsk
    {   14, 4135,   12272, 10, }, // Asia/Novosibirsk
    {   14, 4229,   12282,  5, }, // Asia/Omsk
    {   14, 7145,   12287,  5, }, // Asia/Oral
    {   14, 6961,   12292,  8, }, // Asia/Phnom_Penh
    {   14, 8693,   12300,  8, }, // Asia/Pontianak
    {   14, 4214,   12308, 10, }, // Asia/Pyongyang
    {   14, 5559,   12318,  3, }, // Asia/Qatar
    {   14, 8708,   12321,  8, }, // Asia/Qostanay
    {   14, 4269,   12329,  9, }, // Asia/Qyzylorda
    {   14, 3584,   12338,  6, }, // Asia/Riyadh
    {   14, 4408,   12344,  6, }, // Asia/Sakhalin
    {   14, 7245,   12350,  6, }, // Asia/Samarkand
    {   14, 3279,   12356,  3, }, // Asia/Seoul
    {   14, 1053,   12359,  6, }, // Asia/Shanghai
    {   14, 3300,   12365,  8, }, // Asia/Singapore
    {   14, 4297,   12373, 12, }, // Asia/Srednekolymsk
    {   14, 3263,   12385,  6, }, // Asia/Taipei
    {   14, 4781,   12391,  5, }, // Asia/Tashkent
    {   14, 3915,   12396,  6, }, // Asia/Tbilisi
    {   14, 2772,   12402,  5, }, // Asia/Tehran
    {   14, 1317,   12407,  5, }, // Asia/Thimphu
    {   14, 2821,   12412,  5, }, // Asia/Tokyo
    {   14, 4572,   12417,  5, }, // Asia/Tomsk
    {   14, 1021,   12422,  9, }, // Asia/Ulaanbaatar
    {   14, 1159,   12431,  6, }, // Asia/Urumqi
    {   14, 8722,   12437,  9, }, // Asia/Ust-Nera
    {   14, 6977,   12446,  8, }, // Asia/Vientiane
    {   14, 4710,   12454, 10, }, // Asia/Vladivostok
    {   14, 4828,   12464,  6, }, // Asia/Yakutsk
    {   14, 1235,   12470,  6, }, // Asia/Yangon
    {   14, 3883,   12476, 10, }, // Asia/Yekaterinburg
    {   14, 3747,   12486,  6, }, // Asia/Yerevan
    {   14, 3663,   12492,  5, }, // Atlantic/Azores
    {   14, 5671,   12497,  6, }, // Atlantic/Bermuda
    {   14, 6628,   12503,  5, }, // Atlantic/Canary
    {   14, 3727,   12508, 12, }, // Atlantic/Cape_Verde
    {   14, 1395,   12520,  4, }, // Atlantic/Faroe
    {   14, 8736,   12524,  6, }, // Atlantic/Madeira
    {   14, 2748,   12530,  8, }, // Atlantic/Reykjavik
    {   14, 7272,   12538, 15, }, // Atlantic/South_Georgia
    {   14, 6821,   12553, 11, }, // Atlantic/St_Helena
    {   14, 6644,   12564,  7, }, // Atlantic/Stanley
    {   14, 1670,   12571,  6, }, // Australia/Adelaide
    {   14, 1635,   12577,  7, }, // Australia/Brisbane
    {   14, 1799,   12584,  9, }, // Australia/Broken_Hill
    {   14, 1583,   12593,  5, }, // Australia/Darwin
    {   14, 3637,   12598,  5, }, // Australia/Eucla
    {   14, 1516,   12603,  6, }, // Australia/Hobart
    {   14, 8753,   12609,  7, }, // Australia/Lindeman
    {   14, 1547,   12616,  8, }, // Australia/Lord_Howe
    {   14, 1727,   12624,  7, }, // Australia/Melbourne
    {   14, 1762,   12631,  4, }, // Australia/Perth
    {   14, 1463,   12635,  5, }, // Australia/Sydney
    {   14, 6441,   12640,  8, }, // Europe/Amsterdam
    {   14, 6216,   12648,  6, }, // Europe/Andorra
    {   14, 3620,   12654,  8, }, // Europe/Astrakhan
    {   14, 2300,   12662,  5, }, // Europe/Athens
    {   14, 6336,   12667,  6, }, // Europe/Belgrade
    {   14, 4757,   12673,  5, }, // Europe/Berlin
    {   14, 6488,   12678, 10, }, // Europe/Bratislava
    {   14, 2167,   12688,  6, }, // Europe/Brussels
    {   14, 3928,   12694,  7, }, // Europe/Bucharest
    {   14, 3806,   12701,  7, }, // Europe/Budapest
    {   14, 8772,   12708,  6, }, // Europe/Busingen
    {   14, 2583,   12714,  8, }, // Europe/Chisinau
    {   14, 6289,   12722,  8, }, // Europe/Copenhagen
    {   14, 2338,   12730,  4, }, // Europe/Dublin
    {   14, 6307,   12734,  8, }, // Europe/Gibraltar
    {   14, 8788,   12742,  6, }, // Europe/Guernsey
    {   14, 6612,   12748,  6, }, // Europe/Helsinki
    {   14, 8804,   12754,  9, }, // Europe/Isle_of_Man
    {   14, 1130,   12763,  7, }, // Europe/Istanbul
    {   14, 8823,   12770,  5, }, // Europe/Jersey
    {   14, 3979,   12775,  9, }, // Europe/Kaliningrad
    {   14, 8929,   12784,  5, }, // Europe/Kirov
    {   14, 2527,   12789,  4, }, // Europe/Kyiv
    {   14, 3213,   12793,  6, }, // Europe/Lisbon
    {   14, 6506,   12799,  9, }, // Europe/Ljubljana
    {   14, 2501,   12808,  4, }, // Europe/London
    {   14, 6365,   12812,  9, }, // Europe/Luxembourg
    {   14, 4892,   12821,  5, }, // Europe/Madrid
    {   14, 6397,   12826,  5, }, // Europe/Malta
    {   14, 6582,   12831,  8, }, // Europe/Mariehamn
    {   14, 3693,   12839,  5, }, // Europe/Minsk
    {   14, 6410,   12844,  6, }, // Europe/Monaco
    {   14, 3537,   12850,  5, }, // Europe/Moscow
    {   14, 6458,   12855,  5, }, // Europe/Oslo
    {   14, 4284,   12860,  5, }, // Europe/Paris
    {   14, 6424,   12865,  9, }, // Europe/Podgorica
    {   14, 6275,   12874,  4, }, // Europe/Prague
    {   14, 8837,   12878,  4, }, // Europe/Riga
    {   14, 6324,   12522,  4, }, // Europe/Rome
    {   14, 4331,   12882,  5, }, // Europe/Samara
    {   14, 6470,   12887, 10, }, // Europe/San_Marino
    {   14, 6245,   12897,  7, }, // Europe/Sarajevo
    {   14, 4451,   12904,  7, }, // Europe/Saratov
    {   14, 8849,   12911,  9, }, // Europe/Simferopol
    {   14, 6383,   12920,  5, }, // Europe/Skopje
    {   14, 6599,   12925,  5, }, // Europe/Sofia
    {   14, 6523,   12930,  8, }, // Europe/Stockholm
    {   14, 8867,   12938,  5, }, // Europe/Tallinn
    {   14, 6202,   12943,  6, }, // Europe/Tirane
    {   14, 8882,   12949, 10, }, // Europe/Ulyanovsk
    {   14, 6352,   12959,  5, }, // Europe/Vaduz
    {   14, 6567,   12964,  9, }, // Europe/Vatican
    {   14, 6231,   12973,  5, }, // Europe/Vienna
    {   14, 8899,   12978,  7, }, // Europe/Vilnius
    {   14, 4727,   12985,  8, }, // Europe/Volgograd
    {   14, 3190,   12993,  5, }, // Europe/Warsaw
    {   14, 6261,   12998,  4, }, // Europe/Zagreb
    {   14, 6540,   13002,  5, }, // Europe/Zurich
    {   14, 5078,   13007, 12, }, // Indian/Antananarivo
    {   14, 6947,   13019,  6, }, // Indian/Chagos
    {   14, 6046,   13025,  7, }, // Indian/Christmas
    {   14, 6063,   13032,  5, }, // Indian/Cocos
    {   14, 5029,   13037,  9, }, // Indian/Comoro
    {   14, 6661,   13046,  9, }, // Indian/Kerguelen
    {   14, 7260,   13055,  4, }, // Indian/Mahe
    {   14, 7050,   13059,  8, }, // Indian/Maldives
    {   14, 4069,   13067,  8, }, // Indian/Mauritius
    {   14, 5098,   13075,  5, }, // Indian/Mayotte
    {   14, 7211,   13080,  7, }, // Indian/Reunion
    {   14, 4422,   13087,  4, }, // Pacific/Apia
    {   14,  932,   13091,  7, }, // Pacific/Auckland
    {   14, 3706,   13098,  8, }, // Pacific/Bougainville
    {   14, 3015,   13106,  6, }, // Pacific/Chatham
    {   14, 3157,   11153,  3, }, // Pacific/Chuuk
    {   14, 2237,   13112,  4, }, // Pacific/Easter
    {   14, 7360,   13116,  5, }, // Pacific/Efate
    {   14, 7327,   13121,  7, }, // Pacific/Fakaofo
    {   14, 3902,   13128,  4, }, // Pacific/Fiji
    {   14, 7343,   13132,  8, }, // Pacific/Funafuti
    {   14, 6678,   13140,  8, }, // Pacific/Galapagos
    {   14, 6696,   13148,  6, }, // Pacific/Gambier
    {   14, 3822,   13154, 11, }, // Pacific/Guadalcanal
    {   14, 6018,   11377,  4, }, // Pacific/Guam
    {   14, 2723,   13165,  8, }, // Pacific/Honolulu
    {   14, 3049,   13173,  8, }, // Pacific/Kanton
    {   14, 3998,   13181, 11, }, // Pacific/Kiritimati
    {   14, 7019,   13192,  5, }, // Pacific/Kosrae
    {   14, 2842,   13197,  8, }, // Pacific/Kwajalein
    {   14, 4877,   13205,  6, }, // Pacific/Majuro
    {   14, 4051,   13211,  8, }, // Pacific/Marquesas
    {   14, 8914,   13219,  6, }, // Pacific/Midway
    {   14, 7084,   13225,  5, }, // Pacific/Nauru
    {   14, 7132,   11498,  4, }, // Pacific/Niue
    {   14, 4168,   13230,  7, }, // Pacific/Norfolk
    {   14, 7098,   13237,  5, }, // Pacific/Noumea
    {   14, 3126,   13242,  9, }, // Pacific/Pago_Pago
    {   14, 7155,   13251,  5, }, // Pacific/Palau
    {   14, 7194,   13256,  7, }, // Pacific/Pitcairn
    {   14, 3096,   13263,  5, }, // Pacific/Pohnpei
    {   14, 4807,   13268, 10, }, // Pacific/Port_Moresby
    {   14, 6076,   13278,  9, }, // Pacific/Rarotonga
    {   14, 6031,   13287,  6, }, // Pacific/Saipan
    {   14, 7312,   13293,  6, }, // Pacific/Tahiti
    {   14, 6712,   13299,  6, }, // Pacific/Tarawa
    {   14, 4583,   13305,  9, }, // Pacific/Tongatapu
    {   14, 7392,   10681,  3, }, // Pacific/Wake
    {   14, 7405,   13314,  5, }, // Pacific/Wallis
    {   42,   14,       0,  6, }, // Africa/Asmara Aragonese/Latin/Spain
    {   42, 4435,       6,  8, }, // Africa/Sao_Tome
    {   42, 4252,      14,  8, }, // America/Asuncion
    {   42,  237,      22,  8, }, // America/Atikokan
    {   42, 7623,      30, 17, }, // America/Bahia_Banderas
    {   42, 3868,      47,  6, }, // America/Cancun
    {   42, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   42, 5723,      66,  7, }, // America/Curacao
    {   42,  481,      73, 13, }, // America/Indiana/Knox
    {   42, 7961,      86, 16, }, // America/Indiana/Marengo
    {   42, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   42, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   42, 8038,     139, 14, }, // America/Indiana/Vevay
    {   42, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   42, 8086,     171, 16, }, // America/Indiana/Winamac
    {   42, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   42, 5932,     207, 22, }, // America/Lower_Princes
    {   42, 8220,     229,  6, }, // America/Merida
    {   42, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   42, 8329,     255, 20, }, // America/North_Dakota/Center
    {   42, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   42,  393,     298,  4, }, // America/Nuuk
    {   42, 6899,     302, 16, }, // America/Scoresbysund
    {   42, 8508,     318, 14, }, // America/St_Barthelemy
    {   42, 2061,     332, 10, }, // America/St_Johns
    {   42, 5863,     342,  9, }, // America/St_Kitts
    {   42, 5880,     351,  9, }, // America/St_Lucia
    {   42,  855,     360, 10, }, // America/St_Thomas
    {   42, 5913,     370, 11, }, // America/St_Vincent
    {   42, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   42, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   42, 1185,     413,  9, }, // Asia/Kathmandu
    {   42,  992,     422,  7, }, // Asia/Kolkata
    {   42, 1211,    4203,  5, }, // Asia/Macau
    {   42, 1235,     442,  6, }, // Asia/Yangon
    {   42, 1395,     448,  5, }, // Atlantic/Faroe
    {   42, 6821,     453, 10, }, // Atlantic/St_Helena
    {   42, 2527,     463,  4, }, // Europe/Kyiv
    {   42, 7211,     467,  7, }, // Indian/Reunion
    {   42, 3157,     474,  5, }, // Pacific/Chuuk
    {   42, 3096,     479,  7, }, // Pacific/Pohnpei
    {   43, 6788,   13319,  6, }, // Africa/Abidjan Armenian/Armenian/Armenia
    {   43, 6760,   13325,  4, }, // Africa/Accra
    {   43, 5059,   13329, 10, }, // Africa/Addis_Ababa
    {   43, 7420,   13339,  5, }, // Africa/Algiers
    {   43,   14,   13344,  6, }, // Africa/Asmara
    {   43,   44,   13350,  6, }, // Africa/Bamako
    {   43, 5257,   13356,  5, }, // Africa/Bangui
    {   43, 6746,   13361,  7, }, // Africa/Banjul
    {   43, 7435,   13368,  6, }, // Africa/Bissau
    {   43, 4957,   13374,  8, }, // Africa/Blantyre
    {   43, 5287,   13382,  8, }, // Africa/Brazzaville
    {   43, 4922,   13390, 12, }, // Africa/Bujumbura
    {   43, 2320,   13402,  6, }, // Africa/Cairo
    {   43, 4117,   13408, 10, }, // Africa/Casablanca
    {   43, 7449,   13418,  6, }, // Africa/Ceuta
    {   43, 6773,   13424,  7, }, // Africa/Conakry
    {   43, 6840,   13431,  5, }, // Africa/Dakar
    {   43, 5130,   13436, 12, }, // Africa/Dar_es_Salaam
    {   43, 5043,   13448,  7, }, // Africa/Djibouti
    {   43, 5243,   13455,  6, }, // Africa/Douala
    {   43, 5166,   13461,  8, }, // Africa/El_Aaiun
    {   43, 6853,   13469,  8, }, // Africa/Freetown
    {   43, 4906,   13477,  8, }, // Africa/Gaborone
    {   43, 5015,   13485,  6, }, // Africa/Harare
    {   43, 4479,   13491, 12, }, // Africa/Johannesburg
    {   43, 4499,   13503,  5, }, // Africa/Juba
    {   43, 5151,   13508,  7, }, // Africa/Kampala
    {   43, 4524,   13515,  7, }, // Africa/Khartoum
    {   43, 4973,   13522,  6, }, // Africa/Kigali
    {   43, 5306,   13528,  7, }, // Africa/Kinshasa
    {   43, 4744,   13535,  5, }, // Africa/Lagos
    {   43, 5336,   13540,  7, }, // Africa/Libreville
    {   43, 6869,   13547,  4, }, // Africa/Lome
    {   43, 5211,   13551,  7, }, // Africa/Luanda
    {   43, 4939,   13558, 11, }, // Africa/Lubumbashi
    {   43, 5001,   13569,  7, }, // Africa/Lusaka
    {   43, 5322,   13576,  6, }, // Africa/Malabo
    {   43, 4987,   13582,  8, }, // Africa/Maputo
    {   43, 5197,   13590,  7, }, // Africa/Maseru
    {   43, 5182,   13597,  7, }, // Africa/Mbabane
    {   43, 5113,   13604,  8, }, // Africa/Mogadishu
    {   43, 7034,   13612,  8, }, // Africa/Monrovia
    {   43, 3853,   13620,  7, }, // Africa/Nairobi
    {   43, 5271,   13627,  7, }, // Africa/Ndjamena
    {   43, 5354,   13634,  6, }, // Africa/Niamey
    {   43, 6803,   13640,  8, }, // Africa/Nouakchott
    {   43, 6727,   13648, 11, }, // Africa/Ouagadougou
    {   43, 5225,   13659, 10, }, // Africa/Porto-Novo
    {   43, 4435,   13669,  8, }, // Africa/Sao_Tome
    {   43, 2866,   13677,  7, }, // Africa/Tripoli
    {   43, 6554,   13684,  6, }, // Africa/Tunis
    {   43, 4152,   13690,  8, }, // Africa/Windhoek
    {   43,  132,   13698,  9, }, // America/Adak
    {   43, 3346,   13707,  7, }, // America/Anchorage
    {   43, 5607,   13714,  8, }, // America/Anguilla
    {   43, 5624,   13722,  8, }, // America/Antigua
    {   43, 4554,   13730, 10, }, // America/Araguaina
    {   43,  166,   13740, 13, }, // America/Argentina/Buenos_Aires
    {   43,   91,   13753,  9, }, // America/Argentina/Catamarca
    {   43,  270,   13762,  7, }, // America/Argentina/Cordoba
    {   43,  441,   13769,  7, }, // America/Argentina/Jujuy
    {   43, 7462,   13776,  8, }, // America/Argentina/La_Rioja
    {   43,  565,   13784,  7, }, // America/Argentina/Mendoza
    {   43, 7489,   13791, 12, }, // America/Argentina/Rio_Gallegos
    {   43, 7520,   13803,  5, }, // America/Argentina/Salta
    {   43, 7544,   13808,  9, }, // America/Argentina/San_Juan
    {   43, 5580,   13817,  9, }, // America/Argentina/San_Luis
    {   43, 7571,   13826,  9, }, // America/Argentina/Tucuman
    {   43, 7597,   13835,  8, }, // America/Argentina/Ushuaia
    {   43, 5640,   13843,  6, }, // America/Aruba
    {   43, 4252,   13849,  9, }, // America/Asuncion
    {   43,  237,   13858,  8, }, // America/Atikokan
    {   43, 3679,   13866,  5, }, // America/Bahia
    {   43, 7623,   13871, 14, }, // America/Bahia_Banderas
    {   43, 5654,   13885,  8, }, // America/Barbados
    {   43, 7646,   13893,  5, }, // America/Belem
    {   43, 5407,   13898,  5, }, // America/Belize
    {   43, 7660,   13903, 12, }, // America/Blanc-Sablon
    {   43, 7681,   13915,  9, }, // America/Boa_Vista
    {   43, 4361,   13924,  6, }, // America/Bogota
    {   43, 7699,   13930,  5, }, // America/Boise
    {   43, 7713,   13935, 11, }, // America/Cambridge_Bay
    {   43, 7735,   13946, 13, }, // America/Campo_Grande
    {   43, 3868,   13959,  7, }, // America/Cancun
    {   43, 4694,   13966,  7, }, // America/Caracas
    {   43, 4345,   13973,  5, }, // America/Cayenne
    {   43, 5496,   13978, 14, }, // America/Cayman
    {   43, 2260,   13992,  6, }, // America/Chicago
    {   43, 7756,   13998,  8, }, // America/Chihuahua
    {   43, 7774,   14006, 15, }, // America/Ciudad_Juarez
    {   43, 5422,   14021, 10, }, // America/Costa_Rica
    {   43, 7796,   14031,  7, }, // America/Creston
    {   43, 3791,   14038,  7, }, // America/Cuiaba
    {   43, 5723,   14045,  9, }, // America/Curacao
    {   43, 7812,   14054, 12, }, // America/Danmarkshavn
    {   43, 7833,   14066,  7, }, // America/Dawson
    {   43, 7848,   14073, 12, }, // America/Dawson_Creek
    {   43,  805,   14085,  6, }, // America/Denver
    {   43, 3465,   14091,  7, }, // America/Detroit
    {   43, 5739,   14098,  8, }, // America/Dominica
    {   43,  893,   14106,  8, }, // America/Edmonton
    {   43, 7869,   14114,  9, }, // America/Eirunepe
    {   43, 5441,   14123,  8, }, // America/El_Salvador
    {   43, 7886,   14131, 11, }, // America/Fort_Nelson
    {   43, 7906,   14142,  9, }, // America/Fortaleza
    {   43, 7924,   14151,  9, }, // America/Glace_Bay
    {   43, 6881,   14160,  8, }, // America/Goose_Bay
    {   43, 4612,   14168, 11, }, // America/Grand_Turk
    {   43, 5770,   14179,  7, }, // America/Grenada
    {   43, 5786,   14186, 10, }, // America/Guadeloupe
    {   43, 3760,   14196,  9, }, // America/Guatemala
    {   43, 4841,   14205,  9, }, // America/Guayaquil
    {   43, 6932,   14214,  6, }, // America/Guyana
    {   43, 1939,   14220,  8, }, // America/Halifax
    {   43, 2281,   14228,  6, }, // America/Havana
    {   43, 7942,   14234,  9, }, // America/Hermosillo
    {   43,  348,   14243, 12, }, // America/Indiana/Indianapolis
    {   43,  481,   14255, 13, }, // America/Indiana/Knox
    {   43, 7961,   14268, 16, }, // America/Indiana/Marengo
    {   43, 7985,   14284, 20, }, // America/Indiana/Petersburg
    {   43, 8012,   14304, 17, }, // America/Indiana/Tell_City
    {   43, 8038,   14321, 14, }, // America/Indiana/Vevay
    {   43, 8060,   14335, 16, }, // America/Indiana/Vincennes
    {   43, 8086,   14351, 16, }, // America/Indiana/Winamac
    {   43, 8110,   14367,  7, }, // America/Inuvik
    {   43,  660,   14374,  8, }, // America/Iqaluit
    {   43, 2799,   14382,  7, }, // America/Jamaica
    {   43, 5380,   14389,  5, }, // America/Juneau
    {   43,  521,   14394,  8, }, // America/Kentucky/Louisville
    {   43, 8125,   14402, 19, }, // America/Kentucky/Monticello
    {   43, 5704,   14421, 10, }, // America/Kralendijk
    {   43, 4376,   14431,  6, }, // America/La_Paz
    {   43, 7169,   14437,  4, }, // America/Lima
    {   43, 3239,   14441, 11, }, // America/Los_Angeles
    {   43, 5932,   14452, 22, }, // America/Lower_Princes
    {   43, 8153,   14474,  6, }, // America/Maceio
    {   43, 8168,   14480,  8, }, // America/Managua
    {   43, 1908,   14488,  7, }, // America/Manaus
    {   43, 5897,   14495,  6, }, // America/Marigot
    {   43, 5805,   14501,  9, }, // America/Martinique
    {   43, 8184,   14510,  9, }, // America/Matamoros
    {   43, 2917,   14519,  8, }, // America/Mazatlan
    {   43, 8202,   14527,  8, }, // America/Menominee
    {   43, 8220,   14535,  6, }, // America/Merida
    {   43, 8235,   14541, 10, }, // America/Metlakatla
    {   43, 2949,   14551,  6, }, // America/Mexico_City
    {   43, 4391,   14557,  7, }, // America/Miquelon
    {   43, 8254,   14564,  7, }, // America/Moncton
    {   43, 8270,   14571,  8, }, // America/Monterrey
    {   43, 4098,   14579, 10, }, // America/Montevideo
    {   43, 5824,   14589,  8, }, // America/Montserrat
    {   43, 5481,   14597,  6, }, // America/Nassau
    {   43, 2379,   14603,  9, }, // America/New_York
    {   43, 8288,   14612,  3, }, // America/Nome
    {   43, 1850,   14615,  7, }, // America/Noronha
    {   43, 8301,   14622, 25, }, // America/North_Dakota/Beulah
    {   43, 8329,   14647, 28, }, // America/North_Dakota/Center
    {   43, 8357,   14675, 30, }, // America/North_Dakota/New_Salem
    {   43,  393,   14705,  6, }, // America/Nuuk
    {   43, 8388,   14711,  7, }, // America/Ojinaga
    {   43, 2356,   14718,  6, }, // America/Panama
    {   43, 6173,   14724, 10, }, // America/Paramaribo
    {   43, 2973,   14734,  6, }, // America/Phoenix
    {   43, 3945,   14740, 12, }, // America/Port-au-Prince
    {   43, 5954,   14752, 13, }, // America/Port_of_Spain
    {   43, 8404,   14765, 13, }, // America/Porto_Velho
    {   43, 5843,   14778, 12, }, // America/Puerto_Rico
    {   43, 4030,   14790, 13, }, // America/Punta_Arenas
    {   43, 8424,   14803, 12, }, // America/Rankin_Inlet
    {   43, 8445,   14815,  6, }, // America/Recife
    {   43, 1995,   14821,  7, }, // America/Regina
    {   43, 8460,   14828,  9, }, // America/Resolute
    {   43,  695,   14837, 12, }, // America/Rio_Branco
    {   43, 8477,   14849,  8, }, // America/Santarem
    {   43, 2201,   14857,  8, }, // America/Santiago
    {   43, 6111,   14865, 13, }, // America/Santo_Domingo
    {   43, 1878,   14878, 11, }, // America/Sao_Paulo
    {   43, 6899,   14889, 11, }, // America/Scoresbysund
    {   43, 8494,   14900,  5, }, // America/Sitka
    {   43, 8508,   14905, 12, }, // America/St_Barthelemy
    {   43, 2061,   14917,  9, }, // America/St_Johns
    {   43, 5863,   14926,  9, }, // America/St_Kitts
    {   43, 5880,   14935, 12, }, // America/St_Lucia
    {   43,  855,   14947, 10, }, // America/St_Thomas
    {   43, 5913,   14957, 12, }, // America/St_Vincent
    {   43, 8530,   14969, 12, }, // America/Swift_Current
    {   43, 5461,   14981, 12, }, // America/Tegucigalpa
    {   43, 5756,   14993,  5, }, // America/Thule
    {   43,  313,   14998,  8, }, // America/Tijuana
    {   43,  608,   15006,  7, }, // America/Toronto
    {   43, 5688,   15013,  7, }, // America/Tortola
    {   43, 2093,   15020,  9, }, // America/Vancouver
    {   43, 2144,   15029,  9, }, // America/Whitehorse
    {   43,  734,   15038,  7, }, // America/Winnipeg
    {   43, 8552,   15045,  8, }, // America/Yakutat
    {   43, 6001,   15053,  5, }, // Antarctica/Casey
    {   43, 6094,   15058,  6, }, // Antarctica/Davis
    {   43, 6133,   15064, 17, }, // Antarctica/DumontDUrville
    {   43, 8568,   15081, 14, }, // Antarctica/Macquarie
    {   43, 7066,   15095,  7, }, // Antarctica/Mawson
    {   43, 7113,   15102,  9, }, // Antarctica/McMurdo
    {   43, 8589,   15111,  6, }, // Antarctica/Palmer
    {   43, 7226,   15117,  6, }, // Antarctica/Rothera
    {   43, 7295,   15123,  5, }, // Antarctica/Syowa
    {   43, 8607,   15128,  5, }, // Antarctica/Troll
    {   43, 7374,   15133,  6, }, // Antarctica/Vostok
    {   43, 1429,   15139,  7, }, // Arctic/Longyearbyen
    {   43, 5570,   15146,  4, }, // Asia/Aden
    {   43, 5395,   15150,  6, }, // Asia/Almaty
    {   43, 3968,   15156,  5, }, // Asia/Amman
    {   43, 5511,   15161,  6, }, // Asia/Anadyr
    {   43, 5523,   15167,  6, }, // Asia/Aqtau
    {   43, 5368,   15173,  6, }, // Asia/Aqtobe
    {   43,  964,   15179,  7, }, // Asia/Ashgabat
    {   43, 8624,   15186,  7, }, // Asia/Atyrau
    {   43, 3607,   15193,  6, }, // Asia/Baghdad
    {   43, 5534,   15199,  7, }, // Asia/Bahrain
    {   43, 3653,   15206,  5, }, // Asia/Baku
    {   43, 4466,   15211,  7, }, // Asia/Bangkok
    {   43, 3571,   15218,  8, }, // Asia/Barnaul
    {   43, 4086,   15226,  7, }, // Asia/Beirut
    {   43, 3778,   15233,  6, }, // Asia/Bishkek
    {   43, 5989,   15239,  7, }, // Asia/Brunei
    {   43, 4601,   15246,  4, }, // Asia/Chita
    {   43, 4511,   15250,  7, }, // Asia/Colombo
    {   43, 4540,   15257,  8, }, // Asia/Damascus
    {   43, 1093,   15265,  5, }, // Asia/Dhaka
    {   43, 6192,   15270,  4, }, // Asia/Dili
    {   43, 3596,   15274,  6, }, // Asia/Dubai
    {   43, 6159,   15280,  8, }, // Asia/Dushanbe
    {   43, 8636,   15288, 10, }, // Asia/Famagusta
    {   43, 8651,   15298,  4, }, // Asia/Gaza
    {   43, 4795,   15302,  6, }, // Asia/Hebron
    {   43, 1259,   15308,  7, }, // Asia/Ho_Chi_Minh
    {   43, 2704,   15315,  7, }, // Asia/Hong_Kong
    {   43, 4771,   15322,  4, }, // Asia/Hovd
    {   43, 4184,   15326,  8, }, // Asia/Irkutsk
    {   43, 7006,   15334,  7, }, // Asia/Jakarta
    {   43, 6992,   15341,  8, }, // Asia/Jayapura
    {   43, 1290,   15349,  9, }, // Asia/Jerusalem
    {   43, 3560,   15358,  6, }, // Asia/Kabul
    {   43, 4316,   15364,  8, }, // Asia/Kamchatka
    {   43, 4239,   15372,  6, }, // Asia/Karachi
    {   43, 1185,   15378,  9, }, // Asia/Kathmandu
    {   43, 8661,   15387,  7, }, // Asia/Khandyga
    {   43,  992,   15394,  8, }, // Asia/Kolkata
    {   43, 4197,   15402, 11, }, // Asia/Krasnoyarsk
    {   43, 4859,   15413, 15, }, // Asia/Kuala_Lumpur
    {   43, 5976,   15428,  7, }, // Asia/Kuching
    {   43, 5547,   15435,  7, }, // Asia/Kuwait
    {   43, 1211,   15442,  5, }, // Asia/Macau
    {   43, 4017,   15447,  7, }, // Asia/Magadan
    {   43, 1349,   15454,  7, }, // Asia/Makassar
    {   43, 7182,   15461,  6, }, // Asia/Manila
    {   43, 6920,   15467,  6, }, // Asia/Muscat
    {   43, 2554,   15473,  7, }, // Asia/Nicosia
    {   43, 8675,   15480, 12, }, // Asia/Novokuznetsk
    {   43, 4135,   15492, 11, }, // Asia/Novosibirsk
    {   43, 4229,   15503,  4, }, // Asia/Omsk
    {   43, 7145,   15507,  7, }, // Asia/Oral
    {   43, 6961,   15514,  7, }, // Asia/Phnom_Penh
    {   43, 8693,   15521,  9, }, // Asia/Pontianak
    {   43, 4214,   15530,  7, }, // Asia/Pyongyang
    {   43, 5559,   15537,  5, }, // Asia/Qatar
    {   43, 8708,   15542,  8, }, // Asia/Qostanay
    {   43, 4269,   15550,  9, }, // Asia/Qyzylorda
    {   43, 3584,   15559,  7, }, // Asia/Riyadh
    {   43, 4408,   15566,  7, }, // Asia/Sakhalin
    {   43, 7245,   15573,  9, }, // Asia/Samarkand
    {   43, 3279,   15582,  5, }, // Asia/Seoul
    {   43, 1053,   15587,  6, }, // Asia/Shanghai
    {   43, 3300,   15593,  9, }, // Asia/Singapore
    {   43, 4297,   15602, 13, }, // Asia/Srednekolymsk
    {   43, 3263,   15615,  6, }, // Asia/Taipei
    {   43, 4781,   15621,  7, }, // Asia/Tashkent
    {   43, 3915,   15628,  7, }, // Asia/Tbilisi
    {   43, 2772,   15635,  6, }, // Asia/Tehran
    {   43, 1317,   15641,  8, }, // Asia/Thimphu
    {   43, 2821,   15649,  5, }, // Asia/Tokyo
    {   43, 4572,   15654,  5, }, // Asia/Tomsk
    {   43, 1021,   15659, 11, }, // Asia/Ulaanbaatar
    {   43, 1159,   15670,  8, }, // Asia/Urumqi
    {   43, 8722,   15678,  9, }, // Asia/Ust-Nera
    {   43, 6977,   15687,  8, }, // Asia/Vientiane
    {   43, 4710,   15695, 11, }, // Asia/Vladivostok
    {   43, 4828,   15706,  8, }, // Asia/Yakutsk
    {   43, 1235,   15714,  7, }, // Asia/Yangon
    {   43, 3883,   15721, 13, }, // Asia/Yekaterinburg
    {   43, 3747,   15734,  5, }, // Asia/Yerevan
    {   43, 3663,   15739, 15, }, // Atlantic/Azores
    {   43, 5671,   15754, 10, }, // Atlantic/Bermuda
    {   43, 6628,   15764, 16, }, // Atlantic/Canary
    {   43, 3727,   15780, 10, }, // Atlantic/Cape_Verde
    {   43, 1395,   15790, 16, }, // Atlantic/Faroe
    {   43, 8736,   15806,  7, }, // Atlantic/Madeira
    {   43, 2748,   15813,  9, }, // Atlantic/Reykjavik
    {   43, 7272,   15822, 16, }, // Atlantic/South_Georgia
    {   43, 6821,   15838, 18, }, // Atlantic/St_Helena
    {   43, 6644,   15856,  6, }, // Atlantic/Stanley
    {   43, 1670,   15862,  8, }, // Australia/Adelaide
    {   43, 1635,   15870,  7, }, // Australia/Brisbane
    {   43, 1799,   15877, 10, }, // Australia/Broken_Hill
    {   43, 1583,   15887,  6, }, // Australia/Darwin
    {   43, 3637,   15893,  6, }, // Australia/Eucla
    {   43, 1516,   15899,  6, }, // Australia/Hobart
    {   43, 8753,   15905,  8, }, // Australia/Lindeman
    {   43, 1547,   15913, 14, }, // Australia/Lord_Howe
    {   43, 1727,   15927,  8, }, // Australia/Melbourne
    {   43, 1762,   15935,  4, }, // Australia/Perth
    {   43, 1463,   15939,  6, }, // Australia/Sydney
    {   43, 6441,   15945,  9, }, // Europe/Amsterdam
    {   43, 6216,   15954,  7, }, // Europe/Andorra
    {   43, 3620,   15961,  8, }, // Europe/Astrakhan
    {   43, 2300,   15969,  5, }, // Europe/Athens
    {   43, 6336,   15974,  7, }, // Europe/Belgrade
    {   43, 4757,   15981,  6, }, // Europe/Berlin
    {   43, 6488,   15987, 10, }, // Europe/Bratislava
    {   43, 2167,   15997,  8, }, // Europe/Brussels
    {   43, 3928,   16005,  9, }, // Europe/Bucharest
    {   43, 3806,   16014,  9, }, // Europe/Budapest
    {   43, 8772,   16023, 10, }, // Europe/Busingen
    {   43, 2583,   16033,  5, }, // Europe/Chisinau
    {   43, 6289,   16038, 10, }, // Europe/Copenhagen
    {   43, 2338,   16048,  7, }, // Europe/Dublin
    {   43, 6307,   16055,  9, }, // Europe/Gibraltar
    {   43, 8788,   16064,  6, }, // Europe/Guernsey
    {   43, 6612,   16070,  8, }, // Europe/Helsinki
    {   43, 8804,   16078,  8, }, // Europe/Isle_of_Man
    {   43, 1130,   16086,  8, }, // Europe/Istanbul
    {   43, 8823,   16094,  5, }, // Europe/Jersey
    {   43, 3979,   16099, 11, }, // Europe/Kaliningrad
    {   43, 8929,   16110,  5, }, // Europe/Kirov
    {   43, 2527,   16115,  3, }, // Europe/Kyiv
    {   43, 3213,   16118,  7, }, // Europe/Lisbon
    {   43, 6506,   16125, 10, }, // Europe/Ljubljana
    {   43, 2501,   16135,  6, }, // Europe/London
    {   43, 6365,   16141, 13, }, // Europe/Luxembourg
    {   43, 4892,   16154,  6, }, // Europe/Madrid
    {   43, 6397,   16160,  5, }, // Europe/Malta
    {   43, 6582,   16165,  9, }, // Europe/Mariehamn
    {   43, 3693,   16174,  5, }, // Europe/Minsk
    {   43, 6410,   16179,  6, }, // Europe/Monaco
    {   43, 3537,   16185,  6, }, // Europe/Moscow
    {   43, 6458,   16191,  4, }, // Europe/Oslo
    {   43, 4284,   16195,  5, }, // Europe/Paris
    {   43, 6424,   16200,  9, }, // Europe/Podgorica
    {   43, 6275,   16209,  5, }, // Europe/Prague
    {   43, 8837,   16214,  4, }, // Europe/Riga
    {   43, 6324,   16218,  4, }, // Europe/Rome
    {   43, 4331,   16222,  6, }, // Europe/Samara
    {   43, 6470,   16228, 10, }, // Europe/San_Marino
    {   43, 6245,   16238,  6, }, // Europe/Sarajevo
    {   43, 4451,   16244,  7, }, // Europe/Saratov
    {   43, 8849,   16251, 10, }, // Europe/Simferopol
    {   43, 6383,   16261,  6, }, // Europe/Skopje
    {   43, 6599,   16267,  5, }, // Europe/Sofia
    {   43, 6523,   16272,  8, }, // Europe/Stockholm
    {   43, 8867,   16280,  6, }, // Europe/Tallinn
    {   43, 6202,   16286,  6, }, // Europe/Tirane
    {   43, 8882,   16292, 10, }, // Europe/Ulyanovsk
    {   43, 6352,   16302,  6, }, // Europe/Vaduz
    {   43, 6567,   16308,  7, }, // Europe/Vatican
    {   43, 6231,   16315,  6, }, // Europe/Vienna
    {   43, 8899,   16321,  8, }, // Europe/Vilnius
    {   43, 4727,   16329,  9, }, // Europe/Volgograd
    {   43, 3190,   16338,  7, }, // Europe/Warsaw
    {   43, 6261,   16345,  6, }, // Europe/Zagreb
    {   43, 6540,   16351,  7, }, // Europe/Zurich
    {   43, 5078,   16358, 13, }, // Indian/Antananarivo
    {   43, 6947,   16371,  5, }, // Indian/Chagos
    {   43, 6046,   16376, 18, }, // Indian/Christmas
    {   43, 6063,   16394, 16, }, // Indian/Cocos
    {   43, 5029,   16410, 16, }, // Indian/Comoro
    {   43, 6661,   16426,  8, }, // Indian/Kerguelen
    {   43, 7260,   16434,  3, }, // Indian/Mahe
    {   43, 7050,   16437,  9, }, // Indian/Maldives
    {   43, 4069,   16446,  9, }, // Indian/Mauritius
    {   43, 5098,   16455,  5, }, // Indian/Mayotte
    {   43, 7211,   16460,  9, }, // Indian/Reunion
    {   43, 4422,   16469,  4, }, // Pacific/Apia
    {   43,  932,   16473,  6, }, // Pacific/Auckland
    {   43, 3706,   16479,  9, }, // Pacific/Bougainville
    {   43, 3015,   16488, 10, }, // Pacific/Chatham
    {   43, 3157,   16498,  6, }, // Pacific/Chuuk
    {   43, 2237,   16504, 10, }, // Pacific/Easter
    {   43, 7360,   16514,  5, }, // Pacific/Efate
    {   43, 7327,   16519,  7, }, // Pacific/Fakaofo
    {   43, 3902,   16526,  4, }, // Pacific/Fiji
    {   43, 7343,   16530, 10, }, // Pacific/Funafuti
    {   43, 6678,   16540, 20, }, // Pacific/Galapagos
    {   43, 6696,   16560, 14, }, // Pacific/Gambier
    {   43, 3822,   16574, 12, }, // Pacific/Guadalcanal
    {   43, 6018,   16586,  5, }, // Pacific/Guam
    {   43, 2723,   16591, 10, }, // Pacific/Honolulu
    {   43, 3049,   16601, 14, }, // Pacific/Kanton
    {   43, 3998,   16615, 10, }, // Pacific/Kiritimati
    {   43, 7019,   16625,  6, }, // Pacific/Kosrae
    {   43, 2842,   16631,  9, }, // Pacific/Kwajalein
    {   43, 4877,   16640,  7, }, // Pacific/Majuro
    {   43, 4051,   16647, 17, }, // Pacific/Marquesas
    {   43, 8914,   16664, 12, }, // Pacific/Midway
    {   43, 7084,   16676,  7, }, // Pacific/Nauru
    {   43, 7132,   16683,  5, }, // Pacific/Niue
    {   43, 4168,   16688,  7, }, // Pacific/Norfolk
    {   43, 7098,   16695,  6, }, // Pacific/Noumea
    {   43, 3126,   16701,  9, }, // Pacific/Pago_Pago
    {   43, 7155,   16710,  6, }, // Pacific/Palau
    {   43, 7194,   16716,  7, }, // Pacific/Pitcairn
    {   43, 3096,   16723,  6, }, // Pacific/Pohnpei
    {   43, 4807,   16729, 11, }, // Pacific/Port_Moresby
    {   43, 6076,   16740,  9, }, // Pacific/Rarotonga
    {   43, 6031,   16749,  6, }, // Pacific/Saipan
    {   43, 7312,   16755,  5, }, // Pacific/Tahiti
    {   43, 6712,   16760,  6, }, // Pacific/Tarawa
    {   43, 4583,   16766, 10, }, // Pacific/Tongatapu
    {   43, 7392,   16776, 10, }, // Pacific/Wake
    {   43, 7405,   16786,  6, }, // Pacific/Wallis
    {   44, 6788,   16792,  7, }, // Africa/Abidjan Assamese/Bangla/India
    {   44, 6760,   16799,  5, }, // Africa/Accra
    {   44, 5059,   16804, 12, }, // Africa/Addis_Ababa
    {   44, 7420,   16816, 10, }, // Africa/Algiers
    {   44,   14,   16826,  7, }, // Africa/Asmara
    {   44,   44,   16833,  6, }, // Africa/Bamako
    {   44, 5257,   16839,  6, }, // Africa/Bangui
    {   44, 6746,   16845,  7, }, // Africa/Banjul
    {   44, 7435,   16852,  5, }, // Africa/Bissau
    {   44, 4957,   16857, 12, }, // Africa/Blantyre
    {   44, 5287,   16869, 10, }, // Africa/Brazzaville
    {   44, 4922,   16879, 10, }, // Africa/Bujumbura
    {   44, 2320,   16889,  5, }, // Africa/Cairo
    {   44, 4117,   16894, 11, }, // Africa/Casablanca
    {   44, 7449,   16905,  5, }, // Africa/Ceuta
    {   44, 6773,   16910,  8, }, // Africa/Conakry
    {   44, 6840,   16918,  5, }, // Africa/Dakar
    {   44, 5130,   16923, 12, }, // Africa/Dar_es_Salaam
    {   44, 5043,   16935,  6, }, // Africa/Djibouti
    {   44, 5243,   16941,  6, }, // Africa/Douala
    {   44, 5166,   16947,  9, }, // Africa/El_Aaiun
    {   44, 6853,   16956,  8, }, // Africa/Freetown
    {   44, 4906,   16964,  7, }, // Africa/Gaborone
    {   44, 5015,   16971,  6, }, // Africa/Harare
    {   44, 4479,   16977, 12, }, // Africa/Johannesburg
    {   44, 4499,   16989,  4, }, // Africa/Juba
    {   44, 5151,   16993,  8, }, // Africa/Kampala
    {   44, 4524,   17001,  7, }, // Africa/Khartoum
    {   44, 4973,   17008,  6, }, // Africa/Kigali
    {   44, 5306,   17014,  7, }, // Africa/Kinshasa
    {   44, 4744,   17021,  5, }, // Africa/Lagos
    {   44, 5336,   17026, 10, }, // Africa/Libreville
    {   44, 6869,   17036,  3, }, // Africa/Lome
    {   44, 5211,   17039,  8, }, // Africa/Luanda
    {   44, 4939,   17047, 10, }, // Africa/Lubumbashi
    {   44, 5001,   17057,  6, }, // Africa/Lusaka
    {   44, 5322,   17063,  6, }, // Africa/Malabo
    {   44, 4987,   17069,  6, }, // Africa/Maputo
    {   44, 5197,   17075,  6, }, // Africa/Maseru
    {   44, 5182,   17081,  8, }, // Africa/Mbabane
    {   44, 5113,   17089,  8, }, // Africa/Mogadishu
    {   44, 7034,   17097, 10, }, // Africa/Monrovia
    {   44, 3853,   17107,  7, }, // Africa/Nairobi
    {   44, 5271,   17114,  9, }, // Africa/Ndjamena
    {   44, 5354,   17123,  7, }, // Africa/Niamey
    {   44, 6803,   17130,  7, }, // Africa/Nouakchott
    {   44, 6727,   17137,  7, }, // Africa/Ouagadougou
    {   44, 5225,   17144, 11, }, // Africa/Porto-Novo
    {   44, 4435,   17155,  8, }, // Africa/Sao_Tome
    {   44, 2866,   17163,  8, }, // Africa/Tripoli
    {   44, 6554,   17171,  5, }, // Africa/Tunis
    {   44, 4152,   17176,  8, }, // Africa/Windhoek
    {   44,  132,   17184,  4, }, // America/Adak
    {   44, 3346,   17188,  7, }, // America/Anchorage
    {   44, 5607,   17195,  7, }, // America/Anguilla
    {   44, 5624,   17202,  9, }, // America/Antigua
    {   44, 4554,   17211, 10, }, // America/Araguaina
    {   44,  166,   17221, 12, }, // America/Argentina/Buenos_Aires
    {   44,   91,   17233, 10, }, // America/Argentina/Catamarca
    {   44,  270,   17243,  8, }, // America/Argentina/Cordoba
    {   44,  441,   17251,  5, }, // America/Argentina/Jujuy
    {   44, 7462,   17256,  8, }, // America/Argentina/La_Rioja
    {   44,  565,   17264,  8, }, // America/Argentina/Mendoza
    {   44, 7489,   17272, 12, }, // America/Argentina/Rio_Gallegos
    {   44, 7520,   17284,  6, }, // America/Argentina/Salta
    {   44, 7544,   17290,  9, }, // America/Argentina/San_Juan
    {   44, 5580,   17299,  8, }, // America/Argentina/San_Luis
    {   44, 7571,   17307,  6, }, // America/Argentina/Tucuman
    {   44, 7597,   17313,  8, }, // America/Argentina/Ushuaia
    {   44, 5640,   17321,  5, }, // America/Aruba
    {   44, 4252,   17326,  9, }, // America/Asuncion
    {   44,  237,   17335,  8, }, // America/Atikokan
    {   44, 3679,   17343,  7, }, // America/Bahia
    {   44, 7623,   17350, 17, }, // America/Bahia_Banderas
    {   44, 5654,   17367,  9, }, // America/Barbados
    {   44, 7646,   17376,  5, }, // America/Belem
    {   44, 5407,   17381,  5, }, // America/Belize
    {   44, 7660,   17386, 13, }, // America/Blanc-Sablon
    {   44, 7681,   17399, 11, }, // America/Boa_Vista
    {   44, 4361,   17410,  6, }, // America/Bogota
    {   44, 7699,   17416,  5, }, // America/Boise
    {   44, 7713,   17421, 15, }, // America/Cambridge_Bay
    {   44, 7735,   17436, 15, }, // America/Campo_Grande
    {   44, 3868,   17451,  6, }, // America/Cancun
    {   44, 4694,   17457,  7, }, // America/Caracas
    {   44, 4345,   17464,  6, }, // America/Cayenne
    {   44, 5496,   17470,  6, }, // America/Cayman
    {   44, 2260,   17476,  6, }, // America/Chicago
    {   44, 7756,   17482,  9, }, // America/Chihuahua
    {   44, 7774,   17491, 12, }, // America/Ciudad_Juarez
    {   44, 5422,   17503, 11, }, // America/Costa_Rica
    {   44, 7796,   17514,  8, }, // America/Creston
    {   44, 3791,   17522,  6, }, // America/Cuiaba
    {   44, 5723,   17528,  7, }, // America/Curacao
    {   44, 7812,   17535, 12, }, // America/Danmarkshavn
    {   44, 7833,   17547,  5, }, // America/Dawson
    {   44, 7848,   17552, 11, }, // America/Dawson_Creek
    {   44,  805,   17563,  6, }, // America/Denver
    {   44, 3465,   17569,  8, }, // America/Detroit
    {   44, 5739,   17577,  7, }, // America/Dominica
    {   44,  893,   17584,  7, }, // America/Edmonton
    {   44, 7869,   17591,  7, }, // America/Eirunepe
    {   44, 5441,   17598, 11, }, // America/El_Salvador
    {   44, 7886,   17609, 11, }, // America/Fort_Nelson
    {   44, 7906,   17620, 10, }, // America/Fortaleza
    {   44, 7924,   17630, 12, }, // America/Glace_Bay
    {   44, 6881,   17642, 10, }, // America/Goose_Bay
    {   44, 4612,   17652, 13, }, // America/Grand_Turk
    {   44, 5770,   17665,  8, }, // America/Grenada
    {   44, 5786,   17673,  9, }, // America/Guadeloupe
    {   44, 3760,   17682, 10, }, // America/Guatemala
    {   44, 4841,   17692,  9, }, // America/Guayaquil
    {   44, 6932,   17701,  7, }, // America/Guyana
    {   44, 1939,   17708,  9, }, // America/Halifax
    {   44, 2281,   17717,  6, }, // America/Havana
    {   44, 7942,   17723, 12, }, // America/Hermosillo
    {   44,  348,   17735, 15, }, // America/Indiana/Indianapolis
    {   44,  481,   17750, 16, }, // America/Indiana/Knox
    {   44, 7961,   17766, 19, }, // America/Indiana/Marengo
    {   44, 7985,   17785, 24, }, // America/Indiana/Petersburg
    {   44, 8012,   17809, 20, }, // America/Indiana/Tell_City
    {   44, 8038,   17829, 16, }, // America/Indiana/Vevay
    {   44, 8060,   17845, 20, }, // America/Indiana/Vincennes
    {   44, 8086,   17865, 19, }, // America/Indiana/Winamac
    {   44, 8110,   17884,  6, }, // America/Inuvik
    {   44,  660,   17890,  7, }, // America/Iqaluit
    {   44, 2799,   17897,  7, }, // America/Jamaica
    {   44, 5380,   17904,  5, }, // America/Juneau
    {   44,  521,   17909,  8, }, // America/Kentucky/Louisville
    {   44, 8125,   17917, 19, }, // America/Kentucky/Monticello
    {   44, 5704,   17936, 13, }, // America/Kralendijk
    {   44, 4376,   17949,  6, }, // America/La_Paz
    {   44, 7169,   17955,  4, }, // America/Lima
    {   44, 3239,   17959, 11, }, // America/Los_Angeles
    {   44, 5932,   17970, 25, }, // America/Lower_Princes
    {   44, 8153,   17995,  5, }, // America/Maceio
    {   44, 8168,   18000,  8, }, // America/Managua
    {   44, 1908,   18008,  5, }, // America/Manaus
    {   44, 5897,   18013,  7, }, // America/Marigot
    {   44, 5805,   18020,  9, }, // America/Martinique
    {   44, 8184,   18029,  9, }, // America/Matamoros
    {   44, 2917,   18038, 10, }, // America/Mazatlan
    {   44, 8202,   18048,  8, }, // America/Menominee
    {   44, 8220,   18056,  6, }, // America/Merida
    {   44, 8235,   18062, 12, }, // America/Metlakatla
    {   44, 2949,   18074, 13, }, // America/Mexico_City
    {   44, 4391,   18087,  9, }, // America/Miquelon
    {   44, 8254,   18096,  7, }, // America/Moncton
    {   44, 8270,   18103,  9, }, // America/Monterrey
    {   44, 4098,   18112, 11, }, // America/Montevideo
    {   44, 5824,   18123,  9, }, // America/Montserrat
    {   44, 5481,   18132,  5, }, // America/Nassau
    {   44, 2379,   18137, 10, }, // America/New_York
    {   44, 8288,   18050,  3, }, // America/Nome
    {   44, 1850,   18147,  9, }, // America/Noronha
    {   44, 8301,   18156, 20, }, // America/North_Dakota/Beulah
    {   44, 8329,   18176, 21, }, // America/North_Dakota/Center
    {   44, 8357,   18197, 23, }, // America/North_Dakota/New_Salem
    {   44,  393,   18220,  3, }, // America/Nuuk
    {   44, 8388,   18223,  8, }, // America/Ojinaga
    {   44, 2356,   18231,  6, }, // America/Panama
    {   44, 6173,   18237, 10, }, // America/Paramaribo
    {   44, 2973,   18247,  7, }, // America/Phoenix
    {   44, 3945,   18254, 15, }, // America/Port-au-Prince
    {   44, 5954,   18269, 15, }, // America/Port_of_Spain
    {   44, 8404,   18284, 14, }, // America/Porto_Velho
    {   44, 5843,   18298, 13, }, // America/Puerto_Rico
    {   44, 4030,   18311, 13, }, // America/Punta_Arenas
    {   44, 8424,   18324, 12, }, // America/Rankin_Inlet
    {   44, 8445,   18336,  6, }, // America/Recife
    {   44, 1995,   18342,  6, }, // America/Regina
    {   44, 8460,   18348,  8, }, // America/Resolute
    {   44,  695,   18356, 12, }, // America/Rio_Branco
    {   44, 8477,   18368,  9, }, // America/Santarem
    {   44, 2201,   18377, 11, }, // America/Santiago
    {   44, 6111,   18388, 14, }, // America/Santo_Domingo
    {   44, 1878,   18402,  9, }, // America/Sao_Paulo
    {   44, 6899,   18411, 13, }, // America/Scoresbysund
    {   44, 8494,   18424,  8, }, // America/Sitka
    {   44, 8508,   18432, 16, }, // America/St_Barthelemy
    {   44, 2061,   18448, 11, }, // America/St_Johns
    {   44, 5863,   18459, 11, }, // America/St_Kitts
    {   44, 5880,   18470, 14, }, // America/St_Lucia
    {   44,  855,   18484, 11, }, // America/St_Thomas
    {   44, 5913,   18495, 15, }, // America/St_Vincent
    {   44, 8530,   18510, 14, }, // America/Swift_Current
    {   44, 5461,   18524, 12, }, // America/Tegucigalpa
    {   44, 5756,   18536,  6, }, // America/Thule
    {   44,  313,   18542,  8, }, // America/Tijuana
    {   44,  608,   18550,  8, }, // America/Toronto
    {   44, 5688,   18558,  8, }, // America/Tortola
    {   44, 2093,   18566,  8, }, // America/Vancouver
    {   44, 2144,   18574, 10, }, // America/Whitehorse
    {   44,  734,   18584,  7, }, // America/Winnipeg
    {   44, 8552,   18591,  8, }, // America/Yakutat
    {   44, 6001,   18599,  4, }, // Antarctica/Casey
    {   44, 6094,   18603,  5, }, // Antarctica/Davis
    {   44, 6133,   18608, 18, }, // Antarctica/DumontDUrville
    {   44, 8568,   18626, 11, }, // Antarctica/Macquarie
    {   44, 7066,   18637,  5, }, // Antarctica/Mawson
    {   44, 7113,   18642,  9, }, // Antarctica/McMurdo
    {   44, 8589,   18651,  4, }, // Antarctica/Palmer
    {   44, 7226,   18655,  6, }, // Antarctica/Rothera
    {   44, 7295,   18661,  6, }, // Antarctica/Syowa
    {   44, 8607,   18667,  5, }, // Antarctica/Troll
    {   44, 7374,   18672,  6, }, // Antarctica/Vostok
    {   44, 1429,   18678, 12, }, // Arctic/Longyearbyen
    {   44, 5570,   18690,  4, }, // Asia/Aden
    {   44, 5395,   18694,  6, }, // Asia/Almaty
    {   44, 3968,   18700,  6, }, // Asia/Amman
    {   44, 5511,   18706,  6, }, // Asia/Anadyr
    {   44, 5523,   18712, 10, }, // Asia/Aqtau
    {   44, 5368,   18722,  7, }, // Asia/Aqtobe
    {   44,  964,   18729, 11, }, // Asia/Ashgabat
    {   44, 8624,   18740,  5, }, // Asia/Atyrau
    {   44, 3607,   18745,  6, }, // Asia/Baghdad
    {   44, 5534,   18751,  7, }, // Asia/Bahrain
    {   44, 3653,   17526,  4, }, // Asia/Baku
    {   44, 4466,   18758,  5, }, // Asia/Bangkok
    {   44, 3571,   18763,  7, }, // Asia/Barnaul
    {   44, 4086,   18770,  6, }, // Asia/Beirut
    {   44, 3778,   18776, 10, }, // Asia/Bishkek
    {   44, 5989,   18786,  7, }, // Asia/Brunei
    {   44, 4601,   18793,  4, }, // Asia/Chita
    {   44, 4511,   18797,  6, }, // Asia/Colombo
    {   44, 4540,   18803,  9, }, // Asia/Damascus
    {   44, 1093,   18812,  4, }, // Asia/Dhaka
    {   44, 6192,   18816,  4, }, // Asia/Dili
    {   44, 3596,   18820,  5, }, // Asia/Dubai
    {   44, 6159,   18825,  7, }, // Asia/Dushanbe
    {   44, 8636,   18832, 10, }, // Asia/Famagusta
    {   44, 8651,   18842,  4, }, // Asia/Gaza
    {   44, 4795,   18846,  7, }, // Asia/Hebron
    {   44, 1259,   18853, 14, }, // Asia/Ho_Chi_Minh
    {   44, 2704,   18867,  5, }, // Asia/Hong_Kong
    {   44, 4771,   18872,  6, }, // Asia/Hovd
    {   44, 4184,   18878, 10, }, // Asia/Irkutsk
    {   44, 7006,   18888,  8, }, // Asia/Jakarta
    {   44, 6992,   18896,  9, }, // Asia/Jayapura
    {   44, 1290,   18905,  9, }, // Asia/Jerusalem
    {   44, 3560,   18914,  5, }, // Asia/Kabul
    {   44, 4316,   18919,  8, }, // Asia/Kamchatka
    {   44, 4239,   18927,  5, }, // Asia/Karachi
    {   44, 1185,   18932,  9, }, // Asia/Kathmandu
    {   44, 8661,   18941,  8, }, // Asia/Khandyga
    {   44,  992,   18949,  6, }, // Asia/Kolkata
    {   44, 4197,   18955, 15, }, // Asia/Krasnoyarsk
    {   44, 4859,   18970, 12, }, // Asia/Kuala_Lumpur
    {   44, 5976,   18982,  5, }, // Asia/Kuching
    {   44, 5547,   18987,  6, }, // Asia/Kuwait
    {   44, 1211,   18993,  5, }, // Asia/Macau
    {   44, 4017,   18998,  7, }, // Asia/Magadan
    {   44, 1349,   19005,  7, }, // Asia/Makassar
    {   44, 7182,   19012,  6, }, // Asia/Manila
    {   44, 6920,   19018,  7, }, // Asia/Muscat
    {   44, 2554,   19025,  9, }, // Asia/Nicosia
    {   44, 8675,   19034, 14, }, // Asia/Novokuznetsk
    {   44, 4135,   19048, 13, }, // Asia/Novosibirsk
    {   44, 4229,   19061,  5, }, // Asia/Omsk
    {   44, 7145,   19066,  5, }, // Asia/Oral
    {   44, 6961,   19071, 10, }, // Asia/Phnom_Penh
    {   44, 8693,   19081, 11, }, // Asia/Pontianak
    {   44, 4214,   19092,  8, }, // Asia/Pyongyang
    {   44, 5559,   19100,  5, }, // Asia/Qatar
    {   44, 8708,   19105,  8, }, // Asia/Qostanay
    {   44, 4269,   19113, 10, }, // Asia/Qyzylorda
    {   44, 3584,   19123,  6, }, // Asia/Riyadh
    {   44, 4408,   19129,  7, }, // Asia/Sakhalin
    {   44, 7245,   19136, 10, }, // Asia/Samarkand
    {   44, 3279,   19146,  4, }, // Asia/Seoul
    {   44, 1053,   19150,  6, }, // Asia/Shanghai
    {   44, 3300,   19156,  8, }, // Asia/Singapore
    {   44, 4297,   19164, 15, }, // Asia/Srednekolymsk
    {   44, 3263,   19179,  6, }, // Asia/Taipei
    {   44, 4781,   19185,  8, }, // Asia/Tashkent
    {   44, 3915,   19193,  7, }, // Asia/Tbilisi
    {   44, 2772,   19200,  6, }, // Asia/Tehran
    {   44, 1317,   19206,  6, }, // Asia/Thimphu
    {   44, 2821,   19212,  5, }, // Asia/Tokyo
    {   44, 4572,   19217,  8, }, // Asia/Tomsk
    {   44, 1021,   19225,  9, }, // Asia/Ulaanbaatar
    {   44, 1159,   19234,  8, }, // Asia/Urumqi
    {   44, 8722,   19242,  9, }, // Asia/Ust-Nera
    {   44, 6977,   19251, 13, }, // Asia/Vientiane
    {   44, 4710,   19264, 12, }, // Asia/Vladivostok
    {   44, 4828,   19276, 10, }, // Asia/Yakutsk
    {   44, 1235,   19286,  6, }, // Asia/Yangon
    {   44, 3883,   19292, 15, }, // Asia/Yekaterinburg
    {   44, 3747,   19307,  8, }, // Asia/Yerevan
    {   44, 3663,   19315,  6, }, // Atlantic/Azores
    {   44, 5671,   19321,  8, }, // Atlantic/Bermuda
    {   44, 6628,   19329,  6, }, // Atlantic/Canary
    {   44, 3727,   19335, 10, }, // Atlantic/Cape_Verde
    {   44, 1395,   19345,  4, }, // Atlantic/Faroe
    {   44, 8736,   19349,  6, }, // Atlantic/Madeira
    {   44, 2748,   19355,  9, }, // Atlantic/Reykjavik
    {   44, 7272,   19364, 15, }, // Atlantic/South_Georgia
    {   44, 6821,   19379, 13, }, // Atlantic/St_Helena
    {   44, 6644,   19392,  9, }, // Atlantic/Stanley
    {   44, 1670,   19401,  7, }, // Australia/Adelaide
    {   44, 1635,   19408,  8, }, // Australia/Brisbane
    {   44, 1799,   19416, 11, }, // Australia/Broken_Hill
    {   44, 1583,   19427,  6, }, // Australia/Darwin
    {   44, 3637,   19433,  6, }, // Australia/Eucla
    {   44, 1516,   19439,  7, }, // Australia/Hobart
    {   44, 8753,   19446,  8, }, // Australia/Lindeman
    {   44, 1547,   19454,  8, }, // Australia/Lord_Howe
    {   44, 1727,   19462,  8, }, // Australia/Melbourne
    {   44, 1762,   19470,  5, }, // Australia/Perth
    {   44, 1463,   19475,  5, }, // Australia/Sydney
    {   44, 6441,   19480, 10, }, // Europe/Amsterdam
    {   44, 6216,   19490,  7, }, // Europe/Andorra
    {   44, 3620,   19497, 10, }, // Europe/Astrakhan
    {   44, 2300,   19507,  6, }, // Europe/Athens
    {   44, 6336,   19513,  8, }, // Europe/Belgrade
    {   44, 4757,   19521,  7, }, // Europe/Berlin
    {   44, 6488,   19528, 11, }, // Europe/Bratislava
    {   44, 2167,   19539,  8, }, // Europe/Brussels
    {   44, 3928,   19547,  9, }, // Europe/Bucharest
    {   44, 3806,   19556,  9, }, // Europe/Budapest
    {   44, 8772,   19565,  8, }, // Europe/Busingen
    {   44, 2583,   19573,  7, }, // Europe/Chisinau
    {   44, 6289,   19580, 10, }, // Europe/Copenhagen
    {   44, 2338,   19590,  6, }, // Europe/Dublin
    {   44, 6307,   19596, 10, }, // Europe/Gibraltar
    {   44, 8788,   19606,  7, }, // Europe/Guernsey
    {   44, 6612,   19613,  8, }, // Europe/Helsinki
    {   44, 8804,   19621, 10, }, // Europe/Isle_of_Man
    {   44, 1130,   19631,  9, }, // Europe/Istanbul
    {   44, 8823,   19640,  6, }, // Europe/Jersey
    {   44, 3979,   19646, 12, }, // Europe/Kaliningrad
    {   44, 8929,   19658,  5, }, // Europe/Kirov
    {   44, 2527,   19663,  3, }, // Europe/Kyiv
    {   44, 3213,   19666,  5, }, // Europe/Lisbon
    {   44, 6506,   19671, 11, }, // Europe/Ljubljana
    {   44, 2501,   19682,  5, }, // Europe/London
    {   44, 6365,   19687, 12, }, // Europe/Luxembourg
    {   44, 4892,   19699,  7, }, // Europe/Madrid
    {   44, 6397,   19706,  6, }, // Europe/Malta
    {   44, 6582,   19712, 10, }, // Europe/Mariehamn
    {   44, 3693,   19722,  6, }, // Europe/Minsk
    {   44, 6410,   19728,  6, }, // Europe/Monaco
    {   44, 3537,   19734,  5, }, // Europe/Moscow
    {   44, 6458,   19739,  5, }, // Europe/Oslo
    {   44, 4284,   19744,  5, }, // Europe/Paris
    {   44, 6424,   19749,  9, }, // Europe/Podgorica
    {   44, 6275,   19758,  5, }, // Europe/Prague
    {   44, 8837,   19763,  4, }, // Europe/Riga
    {   44, 6324,   19347,  3, }, // Europe/Rome
    {   44, 4331,   19767,  6, }, // Europe/Samara
    {   44, 6470,   19773, 10, }, // Europe/San_Marino
    {   44, 6245,   19783,  8, }, // Europe/Sarajevo
    {   44, 4451,   19791,  7, }, // Europe/Saratov
    {   44, 8849,   19798, 11, }, // Europe/Simferopol
    {   44, 6383,   19809,  8, }, // Europe/Skopje
    {   44, 6599,   19817,  7, }, // Europe/Sofia
    {   44, 6523,   19824,  7, }, // Europe/Stockholm
    {   44, 8867,   19831,  5, }, // Europe/Tallinn
    {   44, 6202,   19836,  6, }, // Europe/Tirane
    {   44, 8882,   19842, 13, }, // Europe/Ulyanovsk
    {   44, 6352,   19855,  5, }, // Europe/Vaduz
    {   44, 6567,   19860,  7, }, // Europe/Vatican
    {   44, 6231,   19867,  7, }, // Europe/Vienna
    {   44, 8899,   19874,  9, }, // Europe/Vilnius
    {   44, 4727,   19883, 11, }, // Europe/Volgograd
    {   44, 3190,   19894,  6, }, // Europe/Warsaw
    {   44, 6261,   19900,  7, }, // Europe/Zagreb
    {   44, 6540,   19907,  5, }, // Europe/Zurich
    {   44, 5078,   19912, 13, }, // Indian/Antananarivo
    {   44, 6947,   19925,  5, }, // Indian/Chagos
    {   44, 6046,   19930, 10, }, // Indian/Christmas
    {   44, 6063,   19940,  5, }, // Indian/Cocos
    {   44, 5029,   19945,  6, }, // Indian/Comoro
    {   44, 6661,   19951,  8, }, // Indian/Kerguelen
    {   44, 7260,   19959,  4, }, // Indian/Mahe
    {   44, 7050,   19963,  8, }, // Indian/Maldives
    {   44, 4069,   19971,  6, }, // Indian/Mauritius
    {   44, 5098,   19977,  9, }, // Indian/Mayotte
    {   44, 7211,   19986,  9, }, // Indian/Reunion
    {   44, 4422,   19995,  6, }, // Pacific/Apia
    {   44,  932,   20001,  7, }, // Pacific/Auckland
    {   44, 3706,   20008, 10, }, // Pacific/Bougainville
    {   44, 3015,   20018,  5, }, // Pacific/Chatham
    {   44, 3157,   20023,  5, }, // Pacific/Chuuk
    {   44, 2237,   20028,  6, }, // Pacific/Easter
    {   44, 7360,   20034,  4, }, // Pacific/Efate
    {   44, 7327,   20038,  7, }, // Pacific/Fakaofo
    {   44, 3902,   20045,  4, }, // Pacific/Fiji
    {   44, 7343,   20049,  8, }, // Pacific/Funafuti
    {   44, 6678,   20057,  9, }, // Pacific/Galapagos
    {   44, 6696,   20066, 10, }, // Pacific/Gambier
    {   44, 3822,   20076, 11, }, // Pacific/Guadalcanal
    {   44, 6018,   18004,  5, }, // Pacific/Guam
    {   44, 3049,   20087,  9, }, // Pacific/Kanton
    {   44, 3998,   20096, 10, }, // Pacific/Kiritimati
    {   44, 7019,   20106,  8, }, // Pacific/Kosrae
    {   44, 2842,   20114,  9, }, // Pacific/Kwajalein
    {   44, 4877,   20123,  6, }, // Pacific/Majuro
    {   44, 4051,   20129,  8, }, // Pacific/Marquesas
    {   44, 8914,   20137,  5, }, // Pacific/Midway
    {   44, 7084,   20142,  4, }, // Pacific/Nauru
    {   44, 7132,   20146,  5, }, // Pacific/Niue
    {   44, 4168,   20151,  6, }, // Pacific/Norfolk
    {   44, 7098,   20157,  7, }, // Pacific/Noumea
    {   44, 3126,   20164,  9, }, // Pacific/Pago_Pago
    {   44, 7155,   20173,  5, }, // Pacific/Palau
    {   44, 7194,   20178,  9, }, // Pacific/Pitcairn
    {   44, 3096,   20187,  6, }, // Pacific/Pohnpei
    {   44, 4807,   20193, 15, }, // Pacific/Port_Moresby
    {   44, 6076,   20208, 10, }, // Pacific/Rarotonga
    {   44, 6031,   20218,  6, }, // Pacific/Saipan
    {   44, 7312,   20224,  6, }, // Pacific/Tahiti
    {   44, 6712,   20230,  6, }, // Pacific/Tarawa
    {   44, 4583,   20236,  8, }, // Pacific/Tongatapu
    {   44, 7392,   20244,  3, }, // Pacific/Wake
    {   44, 7405,   20247,  5, }, // Pacific/Wallis
    {   45, 5059,   20252, 10, }, // Africa/Addis_Ababa Asturian/Latin/Spain
    {   45, 7420,   20262,  5, }, // Africa/Algiers
    {   45,   14,       0,  6, }, // Africa/Asmara
    {   45, 2320,   20267,  8, }, // Africa/Cairo
    {   45, 5043,   20275,  6, }, // Africa/Djibouti
    {   45, 4479,   20281, 13, }, // Africa/Johannesburg
    {   45, 4524,   20294,  6, }, // Africa/Khartoum
    {   45, 6869,   20300,  4, }, // Africa/Lome
    {   45, 5113,   20304,  8, }, // Africa/Mogadishu
    {   45, 5271,   20312,  6, }, // Africa/Ndjamena
    {   45, 5354,   20318,  6, }, // Africa/Niamey
    {   45, 6803,   20324,  7, }, // Africa/Nouakchott
    {   45, 6727,   20331,  8, }, // Africa/Ouagadougou
    {   45, 4435,   20339, 10, }, // Africa/Sao_Tome
    {   45, 2866,   20349,  7, }, // Africa/Tripoli
    {   45, 6554,   20356,  5, }, // Africa/Tunis
    {   45, 5607,    4657,  7, }, // America/Anguilla
    {   45,  270,   20361,  7, }, // America/Argentina/Cordoba
    {   45, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {   45, 4252,      14,  8, }, // America/Asuncion
    {   45,  237,      22,  8, }, // America/Atikokan
    {   45, 3679,      30,  5, }, // America/Bahia
    {   45, 7623,   20375, 14, }, // America/Bahia_Banderas
    {   45, 4361,   20389,  6, }, // America/Bogota
    {   45, 3868,      47,  6, }, // America/Cancun
    {   45, 5496,   20395,  6, }, // America/Cayman
    {   45, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   45, 5723,      66,  7, }, // America/Curacao
    {   45, 5770,   20401,  7, }, // America/Grenada
    {   45, 5786,   20408,  9, }, // America/Guadeloupe
    {   45, 2281,   20417,  9, }, // America/Havana
    {   45,  481,      73, 13, }, // America/Indiana/Knox
    {   45, 7961,      86, 16, }, // America/Indiana/Marengo
    {   45, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   45, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   45, 8038,     139, 14, }, // America/Indiana/Vevay
    {   45, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   45, 8086,     171, 16, }, // America/Indiana/Winamac
    {   45, 2799,   20426,  7, }, // America/Jamaica
    {   45, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   45, 5932,     207, 22, }, // America/Lower_Princes
    {   45, 5805,   20433, 12, }, // America/Martinique
    {   45, 8220,     229,  6, }, // America/Merida
    {   45, 2949,   20445, 15, }, // America/Mexico_City
    {   45, 2379,   20460, 10, }, // America/New_York
    {   45, 8301,   20470, 24, }, // America/North_Dakota/Beulah
    {   45, 8329,   20494, 24, }, // America/North_Dakota/Center
    {   45, 8357,   20518, 29, }, // America/North_Dakota/New_Salem
    {   45,  393,     298,  4, }, // America/Nuuk
    {   45, 2356,   20547,  6, }, // America/Panama
    {   45, 3945,   20553, 15, }, // America/Port-au-Prince
    {   45, 5954,   20568, 13, }, // America/Port_of_Spain
    {   45, 5843,   20581, 11, }, // America/Puerto_Rico
    {   45, 6111,   20592, 13, }, // America/Santo_Domingo
    {   45, 6899,     302, 16, }, // America/Scoresbysund
    {   45, 8508,     318, 14, }, // America/St_Barthelemy
    {   45, 2061,     332, 10, }, // America/St_Johns
    {   45, 5863,     342,  9, }, // America/St_Kitts
    {   45, 5880,     351,  9, }, // America/St_Lucia
    {   45,  855,     360, 10, }, // America/St_Thomas
    {   45, 5913,     370, 11, }, // America/St_Vincent
    {   45, 5688,   20605,  7, }, // America/Tortola
    {   45, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   45, 3968,   20612,  4, }, // Asia/Amman
    {   45, 3607,   20616,  6, }, // Asia/Baghdad
    {   45, 5534,   20622,  7, }, // Asia/Bahrain
    {   45, 3653,   20629,  4, }, // Asia/Baku
    {   45, 5989,   20633,  6, }, // Asia/Brunei
    {   45, 4540,   20639,  7, }, // Asia/Damascus
    {   45, 6159,   20646,  7, }, // Asia/Dushanbe
    {   45, 4795,   20653,  6, }, // Asia/Hebron
    {   45, 1259,   20659, 20, }, // Asia/Ho_Chi_Minh
    {   45, 2704,   20679,  9, }, // Asia/Hong_Kong
    {   45, 7006,   20688,  7, }, // Asia/Jakarta
    {   45, 6992,   20695,  8, }, // Asia/Jayapura
    {   45, 1290,   20703,  9, }, // Asia/Jerusalem
    {   45, 1185,   20712,  8, }, // Asia/Kathmandu
    {   45,  992,   20720,  7, }, // Asia/Kolkata
    {   45, 1211,   20727,  5, }, // Asia/Macau
    {   45, 4017,   20732,  7, }, // Asia/Magadan
    {   45, 6920,   20739,  7, }, // Asia/Muscat
    {   45, 4214,   20746, 10, }, // Asia/Pyongyang
    {   45, 3584,   20756,  5, }, // Asia/Riyadh
    {   45, 4408,   20761,  7, }, // Asia/Sakhalin
    {   45, 7245,   20768, 10, }, // Asia/Samarkand
    {   45, 3279,   20778,  4, }, // Asia/Seoul
    {   45, 3300,   20782,  8, }, // Asia/Singapore
    {   45, 2772,   20790,  6, }, // Asia/Tehran
    {   45, 2821,   20796,  5, }, // Asia/Tokyo
    {   45, 1021,   20801, 10, }, // Asia/Ulaanbaatar
    {   45, 6977,   20811,  8, }, // Asia/Vientiane
    {   45, 1235,   20819,  6, }, // Asia/Yangon
    {   45, 3883,   20825, 14, }, // Asia/Yekaterinburg
    {   45, 3663,   20839, 10, }, // Atlantic/Azores
    {   45, 6628,   20849,  8, }, // Atlantic/Canary
    {   45, 3727,   20857, 10, }, // Atlantic/Cape_Verde
    {   45, 1395,   20867, 12, }, // Atlantic/Faroe
    {   45, 2748,   20879,  9, }, // Atlantic/Reykjavik
    {   45, 7272,   20888, 14, }, // Atlantic/South_Georgia
    {   45, 6821,   20902, 10, }, // Atlantic/St_Helena
    {   45, 1670,   20912,  8, }, // Australia/Adelaide
    {   45, 3620,   20920,  8, }, // Europe/Astrakhan
    {   45, 2300,   20928,  6, }, // Europe/Athens
    {   45, 6336,   20934,  7, }, // Europe/Belgrade
    {   45, 4757,   20941,  6, }, // Europe/Berlin
    {   45, 2167,   20947,  8, }, // Europe/Brussels
    {   45, 3928,   20955,  8, }, // Europe/Bucharest
    {   45, 6289,   20963, 10, }, // Europe/Copenhagen
    {   45, 2338,   20973,  6, }, // Europe/Dublin
    {   45, 6307,   20979,  9, }, // Europe/Gibraltar
    {   45, 6612,   20988,  8, }, // Europe/Helsinki
    {   45, 8804,   20996, 12, }, // Europe/Isle_of_Man
    {   45, 3979,   21008, 11, }, // Europe/Kaliningrad
    {   45, 2527,    8030,  4, }, // Europe/Kyiv
    {   45, 3213,   21019,  6, }, // Europe/Lisbon
    {   45, 6506,   21025,  9, }, // Europe/Ljubljana
    {   45, 2501,   21034,  7, }, // Europe/London
    {   45, 6365,   21041, 10, }, // Europe/Luxembourg
    {   45, 6410,   21051,  6, }, // Europe/Monaco
    {   45, 3537,   21057,  5, }, // Europe/Moscow
    {   45, 4284,   21062,  5, }, // Europe/Paris
    {   45, 6275,   21067,  5, }, // Europe/Prague
    {   45, 6324,    5946,  4, }, // Europe/Rome
    {   45, 6470,   21072,  9, }, // Europe/San_Marino
    {   45, 6599,   21081,  5, }, // Europe/Sofia
    {   45, 6523,   21086,  9, }, // Europe/Stockholm
    {   45, 6202,   21095,  6, }, // Europe/Tirane
    {   45, 6567,   21101, 11, }, // Europe/Vatican
    {   45, 6231,   21112,  5, }, // Europe/Vienna
    {   45, 4727,   21117,  9, }, // Europe/Volgograd
    {   45, 3190,   21126,  8, }, // Europe/Warsaw
    {   45, 4069,   21134,  8, }, // Indian/Mauritius
    {   45, 7211,   21142,  7, }, // Indian/Reunion
    {   45, 3157,     474,  5, }, // Pacific/Chuuk
    {   45, 2237,   21149,  6, }, // Pacific/Easter
    {   45, 3902,   21155,  4, }, // Pacific/Fiji
    {   45, 6678,   21159,  9, }, // Pacific/Galapagos
    {   45, 2723,    4493,  8, }, // Pacific/Honolulu
    {   45, 3049,    4501,  9, }, // Pacific/Kanton
    {   45, 3096,     479,  7, }, // Pacific/Pohnpei
    {   46,   14,       0,  6, }, // Africa/Asmara Asu/Latin/Tanzania
    {   46, 4435,       6,  8, }, // Africa/Sao_Tome
    {   46, 4252,      14,  8, }, // America/Asuncion
    {   46,  237,      22,  8, }, // America/Atikokan
    {   46, 7623,      30, 17, }, // America/Bahia_Banderas
    {   46, 3868,      47,  6, }, // America/Cancun
    {   46, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   46, 5723,      66,  7, }, // America/Curacao
    {   46,  481,      73, 13, }, // America/Indiana/Knox
    {   46, 7961,      86, 16, }, // America/Indiana/Marengo
    {   46, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   46, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   46, 8038,     139, 14, }, // America/Indiana/Vevay
    {   46, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   46, 8086,     171, 16, }, // America/Indiana/Winamac
    {   46, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   46, 5932,     207, 22, }, // America/Lower_Princes
    {   46, 8220,     229,  6, }, // America/Merida
    {   46, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   46, 8329,     255, 20, }, // America/North_Dakota/Center
    {   46, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   46,  393,     298,  4, }, // America/Nuuk
    {   46, 6899,     302, 16, }, // America/Scoresbysund
    {   46, 8508,     318, 14, }, // America/St_Barthelemy
    {   46, 2061,     332, 10, }, // America/St_Johns
    {   46, 5863,     342,  9, }, // America/St_Kitts
    {   46, 5880,     351,  9, }, // America/St_Lucia
    {   46,  855,     360, 10, }, // America/St_Thomas
    {   46, 5913,     370, 11, }, // America/St_Vincent
    {   46, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   46, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   46, 1185,     413,  9, }, // Asia/Kathmandu
    {   46,  992,     422,  7, }, // Asia/Kolkata
    {   46, 1211,     429,  5, }, // Asia/Macau
    {   46, 1235,     442,  6, }, // Asia/Yangon
    {   46, 1395,     448,  5, }, // Atlantic/Faroe
    {   46, 6821,     453, 10, }, // Atlantic/St_Helena
    {   46, 2527,     463,  4, }, // Europe/Kyiv
    {   46, 7211,     467,  7, }, // Indian/Reunion
    {   46, 3157,     474,  5, }, // Pacific/Chuuk
    {   46, 3096,     479,  7, }, // Pacific/Pohnpei
    {   47,   14,       0,  6, }, // Africa/Asmara Atsam/Latin/Nigeria
    {   47, 4435,       6,  8, }, // Africa/Sao_Tome
    {   47, 4252,      14,  8, }, // America/Asuncion
    {   47,  237,      22,  8, }, // America/Atikokan
    {   47, 7623,      30, 17, }, // America/Bahia_Banderas
    {   47, 3868,      47,  6, }, // America/Cancun
    {   47, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   47, 5723,      66,  7, }, // America/Curacao
    {   47,  481,      73, 13, }, // America/Indiana/Knox
    {   47, 7961,      86, 16, }, // America/Indiana/Marengo
    {   47, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   47, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   47, 8038,     139, 14, }, // America/Indiana/Vevay
    {   47, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   47, 8086,     171, 16, }, // America/Indiana/Winamac
    {   47, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   47, 5932,     207, 22, }, // America/Lower_Princes
    {   47, 8220,     229,  6, }, // America/Merida
    {   47, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   47, 8329,     255, 20, }, // America/North_Dakota/Center
    {   47, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   47,  393,     298,  4, }, // America/Nuuk
    {   47, 6899,     302, 16, }, // America/Scoresbysund
    {   47, 8508,     318, 14, }, // America/St_Barthelemy
    {   47, 2061,     332, 10, }, // America/St_Johns
    {   47, 5863,     342,  9, }, // America/St_Kitts
    {   47, 5880,     351,  9, }, // America/St_Lucia
    {   47,  855,     360, 10, }, // America/St_Thomas
    {   47, 5913,     370, 11, }, // America/St_Vincent
    {   47, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   47, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   47, 1185,     413,  9, }, // Asia/Kathmandu
    {   47,  992,     422,  7, }, // Asia/Kolkata
    {   47, 1211,     429,  5, }, // Asia/Macau
    {   47, 1235,     442,  6, }, // Asia/Yangon
    {   47, 1395,     448,  5, }, // Atlantic/Faroe
    {   47, 6821,     453, 10, }, // Atlantic/St_Helena
    {   47, 2527,     463,  4, }, // Europe/Kyiv
    {   47, 7211,     467,  7, }, // Indian/Reunion
    {   47, 3157,     474,  5, }, // Pacific/Chuuk
    {   47, 3096,     479,  7, }, // Pacific/Pohnpei
    {   48, 6788,   21168,  6, }, // Africa/Abidjan Azerbaijani/Latin/Azerbaijan
    {   48, 6760,   21174,  5, }, // Africa/Accra
    {   48, 5059,   21179, 11, }, // Africa/Addis_Ababa
    {   48, 7420,   21190,  8, }, // Africa/Algiers
    {   48,   14,   21198,  6, }, // Africa/Asmara
    {   48, 5257,   21204,  6, }, // Africa/Bangui
    {   48, 6746,   21210,  6, }, // Africa/Banjul
    {   48, 4957,    6200,  7, }, // Africa/Blantyre
    {   48, 5287,   21216,  9, }, // Africa/Brazzaville
    {   48, 2320,   21225,  6, }, // Africa/Cairo
    {   48, 4117,    4577, 10, }, // Africa/Casablanca
    {   48, 7449,   21231,  5, }, // Africa/Ceuta
    {   48, 6773,    6249,  7, }, // Africa/Conakry
    {   48, 5130,   21236, 12, }, // Africa/Dar_es_Salaam
    {   48, 5043,   21248,  6, }, // Africa/Djibouti
    {   48, 5243,   21254,  5, }, // Africa/Douala
    {   48, 5166,   21259,  7, }, // Africa/El_Aaiun
    {   48, 6853,    6282,  7, }, // Africa/Freetown
    {   48, 4906,   21266,  7, }, // Africa/Gaborone
    {   48, 4479,   21273, 11, }, // Africa/Johannesburg
    {   48, 4524,   21284,  6, }, // Africa/Khartoum
    {   48, 4973,   21290,  6, }, // Africa/Kigali
    {   48, 5306,   21296,  7, }, // Africa/Kinshasa
    {   48, 4744,   21303,  5, }, // Africa/Lagos
    {   48, 5336,    6311,  8, }, // Africa/Libreville
    {   48, 4939,   21308,  9, }, // Africa/Lubumbashi
    {   48, 5113,   21317,  8, }, // Africa/Mogadishu
    {   48, 5271,   21325,  7, }, // Africa/Ndjamena
    {   48, 6803,   21332,  8, }, // Africa/Nouakchott
    {   48, 6727,   21340,  7, }, // Africa/Ouagadougou
    {   48, 4435,   21347,  7, }, // Africa/Sao_Tome
    {   48, 4152,   21354,  7, }, // Africa/Windhoek
    {   48, 3346,   21361,  7, }, // America/Anchorage
    {   48, 5607,   21368,  7, }, // America/Anguilla
    {   48, 5624,   21375,  7, }, // America/Antigua
    {   48, 4554,   21382,  9, }, // America/Araguaina
    {   48,  166,   21391, 12, }, // America/Argentina/Buenos_Aires
    {   48,   91,    6400,  9, }, // America/Argentina/Catamarca
    {   48,  270,    6409,  7, }, // America/Argentina/Cordoba
    {   48, 7462,   21403,  5, }, // America/Argentina/La_Rioja
    {   48, 7489,   21408, 12, }, // America/Argentina/Rio_Gallegos
    {   48, 7544,   21420,  8, }, // America/Argentina/San_Juan
    {   48, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {   48, 7597,   21428,  6, }, // America/Argentina/Ushuaia
    {   48, 4252,    6474,  8, }, // America/Asuncion
    {   48,  237,      22,  8, }, // America/Atikokan
    {   48, 3679,   21434,  6, }, // America/Bahia
    {   48, 7623,    3825, 14, }, // America/Bahia_Banderas
    {   48, 5407,    6496,  5, }, // America/Belize
    {   48, 7660,    6502, 12, }, // America/Blanc-Sablon
    {   48, 4361,   21440,  6, }, // America/Bogota
    {   48, 7699,   21446,  5, }, // America/Boise
    {   48, 7713,   21451, 15, }, // America/Cambridge_Bay
    {   48, 7735,   21466, 12, }, // America/Campo_Grande
    {   48, 3868,    6564,  6, }, // America/Cancun
    {   48, 4694,    4792,  7, }, // America/Caracas
    {   48, 4345,   21478,  5, }, // America/Cayenne
    {   48, 5496,   21483,  6, }, // America/Cayman
    {   48, 2260,   21489,  6, }, // America/Chicago
    {   48, 7756,   21495,  8, }, // America/Chihuahua
    {   48, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   48, 5422,   21503, 10, }, // America/Costa_Rica
    {   48, 7796,    6618,  7, }, // America/Creston
    {   48, 3791,   21513,  6, }, // America/Cuiaba
    {   48, 5723,    6631,  7, }, // America/Curacao
    {   48, 7812,   21519, 11, }, // America/Danmarkshavn
    {   48, 7833,    6651,  6, }, // America/Dawson
    {   48, 7848,   21530, 11, }, // America/Dawson_Creek
    {   48, 3465,   21541,  7, }, // America/Detroit
    {   48, 5739,   21548,  8, }, // America/Dominica
    {   48,  893,   21556,  9, }, // America/Edmonton
    {   48, 7869,   21565,  7, }, // America/Eirunepe
    {   48, 5441,    6688,  8, }, // America/El_Salvador
    {   48, 7924,   21572, 13, }, // America/Glace_Bay
    {   48, 6881,   21585, 11, }, // America/Goose_Bay
    {   48, 4612,   21596, 10, }, // America/Grand_Turk
    {   48, 5770,   21606,  7, }, // America/Grenada
    {   48, 5786,   21613,  9, }, // America/Guadeloupe
    {   48, 3760,   21622,  9, }, // America/Guatemala
    {   48, 4841,   21631,  8, }, // America/Guayaquil
    {   48, 6932,   21639,  6, }, // America/Guyana
    {   48, 1939,    6787,  8, }, // America/Halifax
    {   48, 7942,   21645,  9, }, // America/Hermosillo
    {   48,  348,   21654, 12, }, // America/Indiana/Indianapolis
    {   48,  481,   21666,  4, }, // America/Indiana/Knox
    {   48, 7961,   21670,  7, }, // America/Indiana/Marengo
    {   48, 7985,   21677, 10, }, // America/Indiana/Petersburg
    {   48, 8012,     121,  4, }, // America/Indiana/Tell_City
    {   48, 8038,   21687,  5, }, // America/Indiana/Vevay
    {   48, 8060,   21692,  6, }, // America/Indiana/Vincennes
    {   48, 8086,   21698,  7, }, // America/Indiana/Winamac
    {   48, 8110,   21705,  6, }, // America/Inuvik
    {   48,  660,   21711,  7, }, // America/Iqaluit
    {   48, 2799,   21718,  7, }, // America/Jamaica
    {   48, 5380,   21725,  4, }, // America/Juneau
    {   48,  521,   21729,  7, }, // America/Kentucky/Louisville
    {   48, 8125,   21736, 10, }, // America/Kentucky/Monticello
    {   48, 5704,   21746,  9, }, // America/Kralendijk
    {   48, 4376,   21755,  6, }, // America/La_Paz
    {   48, 3239,   21761, 11, }, // America/Los_Angeles
    {   48, 5932,   21772, 20, }, // America/Lower_Princes
    {   48, 8153,   21792,  6, }, // America/Maceio
    {   48, 8168,   21798,  7, }, // America/Managua
    {   48, 5897,   21805,  7, }, // America/Marigot
    {   48, 5805,    5142,  8, }, // America/Martinique
    {   48, 2917,   21812,  8, }, // America/Mazatlan
    {   48, 8202,    7000,  8, }, // America/Menominee
    {   48, 8220,    7008,  6, }, // America/Merida
    {   48, 2949,   21820,  6, }, // America/Mexico_City
    {   48, 4391,    7031,  7, }, // America/Miquelon
    {   48, 8254,    7038,  7, }, // America/Moncton
    {   48, 8270,   21826,  8, }, // America/Monterrey
    {   48, 5824,   21834,  8, }, // America/Montserrat
    {   48, 2379,   21842,  8, }, // America/New_York
    {   48, 8288,   21850,  3, }, // America/Nome
    {   48, 1850,   21853,  7, }, // America/Noronha
    {   48, 8301,   21860, 21, }, // America/North_Dakota/Beulah
    {   48, 8329,   21881, 20, }, // America/North_Dakota/Center
    {   48, 8357,   21901,  9, }, // America/North_Dakota/New_Salem
    {   48,  393,     298,  4, }, // America/Nuuk
    {   48, 8388,   21910,  7, }, // America/Ojinaga
    {   48, 2973,    7165,  6, }, // America/Phoenix
    {   48, 3945,    7171, 12, }, // America/Port-au-Prince
    {   48, 5954,   21917, 12, }, // America/Port_of_Spain
    {   48, 8404,   21929, 11, }, // America/Porto_Velho
    {   48, 5843,   21940, 11, }, // America/Puerto_Rico
    {   48, 8424,   21951, 13, }, // America/Rankin_Inlet
    {   48, 8445,   21964,  5, }, // America/Recife
    {   48, 1995,   21969,  6, }, // America/Regina
    {   48, 8460,   21975,  8, }, // America/Resolute
    {   48,  695,    5362, 10, }, // America/Rio_Branco
    {   48, 2201,   21983,  8, }, // America/Santiago
    {   48, 6111,   21991, 13, }, // America/Santo_Domingo
    {   48, 1878,   22004,  9, }, // America/Sao_Paulo
    {   48, 6899,   22013, 12, }, // America/Scoresbysund
    {   48, 8508,   22025, 14, }, // America/St_Barthelemy
    {   48, 2061,   22039,  9, }, // America/St_Johns
    {   48, 5863,   22048,  8, }, // America/St_Kitts
    {   48, 5880,   22056, 10, }, // America/St_Lucia
    {   48,  855,   22066,  9, }, // America/St_Thomas
    {   48, 5913,   22075, 11, }, // America/St_Vincent
    {   48, 8530,   22086, 12, }, // America/Swift_Current
    {   48, 5461,   22098, 11, }, // America/Tegucigalpa
    {   48, 5756,   22109,  3, }, // America/Thule
    {   48,  313,   22112,  7, }, // America/Tijuana
    {   48, 2093,    7375,  8, }, // America/Vancouver
    {   48, 2144,   22119,  8, }, // America/Whitehorse
    {   48,  734,   22127,  8, }, // America/Winnipeg
    {   48, 6001,   22135,  5, }, // Antarctica/Casey
    {   48, 6094,   22140,  6, }, // Antarctica/Davis
    {   48, 6133,     381, 14, }, // Antarctica/DumontDUrville
    {   48, 8568,   22146,  7, }, // Antarctica/Macquarie
    {   48, 7066,   22153,  6, }, // Antarctica/Mawson
    {   48, 7113,   22159,  9, }, // Antarctica/McMurdo
    {   48, 7295,   22168,  5, }, // Antarctica/Syowa
    {   48, 1429,   22173,  7, }, // Arctic/Longyearbyen
    {   48, 5395,   22180,  6, }, // Asia/Almaty
    {   48, 5511,   22186,  6, }, // Asia/Anadyr
    {   48, 5523,    5491,  5, }, // Asia/Aqtau
    {   48,  964,   22192,  7, }, // Asia/Ashgabat
    {   48, 8624,   22199,  6, }, // Asia/Atyrau
    {   48, 3607,   22205,  6, }, // Asia/Baghdad
    {   48, 5534,   22211,  7, }, // Asia/Bahrain
    {   48, 3653,   22218,  4, }, // Asia/Baku
    {   48, 4466,   22222,  7, }, // Asia/Bangkok
    {   48, 4086,   22229,  6, }, // Asia/Beirut
    {   48, 3778,   22235,  6, }, // Asia/Bishkek
    {   48, 5989,   22241,  6, }, // Asia/Brunei
    {   48, 4601,    7513,  4, }, // Asia/Chita
    {   48, 4511,    5519,  7, }, // Asia/Colombo
    {   48, 4540,   22247,  6, }, // Asia/Damascus
    {   48, 1093,   22253,  5, }, // Asia/Dhaka
    {   48, 3596,   22258,  5, }, // Asia/Dubai
    {   48, 6159,   22263,  7, }, // Asia/Dushanbe
    {   48, 8636,   22270,  9, }, // Asia/Famagusta
    {   48, 8651,   22279,  4, }, // Asia/Gaza
    {   48, 1259,   22283,  9, }, // Asia/Ho_Chi_Minh
    {   48, 2704,   22292,  9, }, // Asia/Hong_Kong
    {   48, 4184,   22301,  7, }, // Asia/Irkutsk
    {   48, 7006,   22308,  7, }, // Asia/Jakarta
    {   48, 6992,   22315,  8, }, // Asia/Jayapura
    {   48, 1290,   22323,  9, }, // Asia/Jerusalem
    {   48, 3560,   22332,  5, }, // Asia/Kabul
    {   48, 4316,   22337,  8, }, // Asia/Kamchatka
    {   48, 4239,    7569,  6, }, // Asia/Karachi
    {   48, 1185,    5593,  8, }, // Asia/Kathmandu
    {   48, 8661,   22345,  7, }, // Asia/Khandyga
    {   48,  992,   22352,  7, }, // Asia/Kolkata
    {   48, 5976,   22359,  6, }, // Asia/Kuching
    {   48, 5547,   22365,  6, }, // Asia/Kuwait
    {   48, 1211,    7613,  5, }, // Asia/Macau
    {   48, 4017,   22371,  7, }, // Asia/Magadan
    {   48, 1349,    7618,  7, }, // Asia/Makassar
    {   48, 6920,    4208,  6, }, // Asia/Muscat
    {   48, 2554,   22378,  8, }, // Asia/Nicosia
    {   48, 6961,   22386,  8, }, // Asia/Phnom_Penh
    {   48, 4214,   22394,  7, }, // Asia/Pyongyang
    {   48, 4269,   22401,  9, }, // Asia/Qyzylorda
    {   48, 3584,   20756,  5, }, // Asia/Riyadh
    {   48, 4408,   22410,  7, }, // Asia/Sakhalin
    {   48, 7245,   22417,  9, }, // Asia/Samarkand
    {   48, 3279,    7680,  4, }, // Asia/Seoul
    {   48, 1053,   22426,  6, }, // Asia/Shanghai
    {   48, 3300,   22432,  8, }, // Asia/Singapore
    {   48, 4297,   22440, 13, }, // Asia/Srednekolymsk
    {   48, 3263,   22453,  6, }, // Asia/Taipei
    {   48, 4781,   22459,  7, }, // Asia/Tashkent
    {   48, 1021,   22466,  9, }, // Asia/Ulaanbaatar
    {   48, 1159,   22475,  6, }, // Asia/Urumqi
    {   48, 6977,   22481,  8, }, // Asia/Vientiane
    {   48, 1235,   22489,  6, }, // Asia/Yangon
    {   48, 3883,   22495, 13, }, // Asia/Yekaterinburg
    {   48, 3663,    7787,  4, }, // Atlantic/Azores
    {   48, 5671,   22508, 14, }, // Atlantic/Bermuda
    {   48, 6628,    4280,  5, }, // Atlantic/Canary
    {   48, 3727,   22522, 10, }, // Atlantic/Cape_Verde
    {   48, 1395,   22532,  5, }, // Atlantic/Faroe
    {   48, 8736,   22537,  7, }, // Atlantic/Madeira
    {   48, 2748,   22544,  9, }, // Atlantic/Reykjavik
    {   48, 7272,   22553, 14, }, // Atlantic/South_Georgia
    {   48, 6821,   22567, 15, }, // Atlantic/St_Helena
    {   48, 6644,    7847,  6, }, // Atlantic/Stanley
    {   48, 1670,   20912,  8, }, // Australia/Adelaide
    {   48, 1635,   22582,  8, }, // Australia/Brisbane
    {   48, 1583,    7880,  6, }, // Australia/Darwin
    {   48, 3637,   22590,  5, }, // Australia/Eucla
    {   48, 1547,   22595,  8, }, // Australia/Lord_Howe
    {   48, 1727,    7903,  7, }, // Australia/Melbourne
    {   48, 1762,   22603,  4, }, // Australia/Perth
    {   48, 1463,   22607,  6, }, // Australia/Sydney
    {   48, 3620,   22613,  9, }, // Europe/Astrakhan
    {   48, 2300,   22622,  5, }, // Europe/Athens
    {   48, 6336,   22627,  7, }, // Europe/Belgrade
    {   48, 2167,   22634,  7, }, // Europe/Brussels
    {   48, 3928,   22641,  8, }, // Europe/Bucharest
    {   48, 3806,   22649,  8, }, // Europe/Budapest
    {   48, 2583,   22657,  8, }, // Europe/Chisinau
    {   48, 6289,    4357, 10, }, // Europe/Copenhagen
    {   48, 6307,   22665, 12, }, // Europe/Gibraltar
    {   48, 8788,   22677,  7, }, // Europe/Guernsey
    {   48, 8804,   22684,  9, }, // Europe/Isle_of_Man
    {   48, 1130,   22693,  8, }, // Europe/Istanbul
    {   48, 8823,   22701,  5, }, // Europe/Jersey
    {   48, 3979,   22706, 11, }, // Europe/Kaliningrad
    {   48, 2527,   22717,  5, }, // Europe/Kyiv
    {   48, 3213,    4381,  8, }, // Europe/Lisbon
    {   48, 6506,   22722,  9, }, // Europe/Ljubljana
    {   48, 6365,   22731, 10, }, // Europe/Luxembourg
    {   48, 6582,   22741,  7, }, // Europe/Mariehamn
    {   48, 6410,    5926,  6, }, // Europe/Monaco
    {   48, 3537,   22748,  6, }, // Europe/Moscow
    {   48, 6424,   22754, 10, }, // Europe/Podgorica
    {   48, 6275,   22764,  5, }, // Europe/Prague
    {   48, 8837,   22769,  4, }, // Europe/Riga
    {   48, 6324,    5946,  4, }, // Europe/Rome
    {   48, 6245,   22773,  8, }, // Europe/Sarajevo
    {   48, 6383,   22781,  6, }, // Europe/Skopje
    {   48, 6599,   22787,  6, }, // Europe/Sofia
    {   48, 6523,    8124,  8, }, // Europe/Stockholm
    {   48, 8867,   22793,  6, }, // Europe/Tallinn
    {   48, 6202,   21095,  6, }, // Europe/Tirane
    {   48, 6352,   22799,  6, }, // Europe/Vaduz
    {   48, 6567,    5978,  7, }, // Europe/Vatican
    {   48, 6231,   22805,  5, }, // Europe/Vienna
    {   48, 8899,   22810,  6, }, // Europe/Vilnius
    {   48, 4727,   22816,  9, }, // Europe/Volgograd
    {   48, 3190,   22825,  7, }, // Europe/Warsaw
    {   48, 6261,   22832,  6, }, // Europe/Zagreb
    {   48, 6540,   22838,  5, }, // Europe/Zurich
    {   48, 6947,   22843,  5, }, // Indian/Chagos
    {   48, 6046,   22848,  5, }, // Indian/Christmas
    {   48, 6063,    4465,  5, }, // Indian/Cocos
    {   48, 5029,   22853,  6, }, // Indian/Comoro
    {   48, 6661,   22859,  8, }, // Indian/Kerguelen
    {   48, 7050,    8193,  6, }, // Indian/Maldives
    {   48, 4069,   22867,  7, }, // Indian/Mauritius
    {   48, 5098,    6046,  5, }, // Indian/Mayotte
    {   48, 7211,     467,  7, }, // Indian/Reunion
    {   48,  932,   22874,  6, }, // Pacific/Auckland
    {   48, 3706,   22880,  8, }, // Pacific/Bougainville
    {   48, 3015,   22888,  5, }, // Pacific/Chatham
    {   48, 3157,   22893,  4, }, // Pacific/Chuuk
    {   48, 2237,   22897,  5, }, // Pacific/Easter
    {   48, 3902,   22902,  4, }, // Pacific/Fiji
    {   48, 6678,   22906,  9, }, // Pacific/Galapagos
    {   48, 6696,   22915,  8, }, // Pacific/Gambier
    {   48, 3822,   22923, 11, }, // Pacific/Guadalcanal
    {   48, 6018,   22934,  4, }, // Pacific/Guam
    {   48, 2723,    4493,  8, }, // Pacific/Honolulu
    {   48, 3049,   22938,  9, }, // Pacific/Kanton
    {   48, 3998,   22947,  8, }, // Pacific/Kiritimati
    {   48, 7019,   22955,  7, }, // Pacific/Kosrae
    {   48, 2842,   22962,  9, }, // Pacific/Kwajalein
    {   48, 4877,   22971,  6, }, // Pacific/Majuro
    {   48, 4051,   22977,  8, }, // Pacific/Marquesas
    {   48, 8914,   22985,  6, }, // Pacific/Midway
    {   48, 3126,   22991,  9, }, // Pacific/Pago_Pago
    {   48, 7194,    8304,  7, }, // Pacific/Pitcairn
    {   48, 3096,     479,  7, }, // Pacific/Pohnpei
    {   48, 4807,   23000, 12, }, // Pacific/Port_Moresby
    {   48, 6076,   23012,  9, }, // Pacific/Rarotonga
    {   48, 6031,   23021,  6, }, // Pacific/Saipan
    {   48, 6712,   23027,  6, }, // Pacific/Tarawa
    {   48, 4583,   23033,  9, }, // Pacific/Tongatapu
    {   48, 7392,   23042,  4, }, // Pacific/Wake
    {   48, 7405,    8349,  6, }, // Pacific/Wallis
    {   49,   14,       0,  6, }, // Africa/Asmara Azerbaijani/Arabic/Iran
    {   49, 4435,       6,  8, }, // Africa/Sao_Tome
    {   49, 4252,      14,  8, }, // America/Asuncion
    {   49, 7623,      30, 17, }, // America/Bahia_Banderas
    {   49, 3868,      47,  6, }, // America/Cancun
    {   49, 5723,      66,  7, }, // America/Curacao
    {   49,  481,      73, 13, }, // America/Indiana/Knox
    {   49, 7961,      86, 16, }, // America/Indiana/Marengo
    {   49, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   49, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   49, 8038,     139, 14, }, // America/Indiana/Vevay
    {   49, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   49, 8086,     171, 16, }, // America/Indiana/Winamac
    {   49, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   49, 5932,     207, 22, }, // America/Lower_Princes
    {   49, 8220,     229,  6, }, // America/Merida
    {   49, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   49, 8329,     255, 20, }, // America/North_Dakota/Center
    {   49, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   49, 6899,     302, 16, }, // America/Scoresbysund
    {   49, 8508,     318, 14, }, // America/St_Barthelemy
    {   49, 2061,     332, 10, }, // America/St_Johns
    {   49, 5863,     342,  9, }, // America/St_Kitts
    {   49, 5880,     351,  9, }, // America/St_Lucia
    {   49,  855,     360, 10, }, // America/St_Thomas
    {   49, 5913,     370, 11, }, // America/St_Vincent
    {   49, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   49, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   49, 1185,     413,  9, }, // Asia/Kathmandu
    {   49,  992,     422,  7, }, // Asia/Kolkata
    {   49, 1211,     429,  5, }, // Asia/Macau
    {   49, 1235,     442,  6, }, // Asia/Yangon
    {   49, 1395,     448,  5, }, // Atlantic/Faroe
    {   49, 6821,     453, 10, }, // Atlantic/St_Helena
    {   49, 2527,     463,  4, }, // Europe/Kyiv
    {   49, 3157,     474,  5, }, // Pacific/Chuuk
    {   52,   14,       0,  6, }, // Africa/Asmara Azerbaijani/Cyrillic/Azerbaijan
    {   52, 4435,       6,  8, }, // Africa/Sao_Tome
    {   52, 4252,      14,  8, }, // America/Asuncion
    {   52, 7623,      30, 17, }, // America/Bahia_Banderas
    {   52, 3868,      47,  6, }, // America/Cancun
    {   52, 5723,      66,  7, }, // America/Curacao
    {   52,  481,      73, 13, }, // America/Indiana/Knox
    {   52, 7961,      86, 16, }, // America/Indiana/Marengo
    {   52, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   52, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   52, 8038,     139, 14, }, // America/Indiana/Vevay
    {   52, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   52, 8086,     171, 16, }, // America/Indiana/Winamac
    {   52, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   52, 5932,     207, 22, }, // America/Lower_Princes
    {   52, 8220,     229,  6, }, // America/Merida
    {   52, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   52, 8329,     255, 20, }, // America/North_Dakota/Center
    {   52, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   52, 6899,     302, 16, }, // America/Scoresbysund
    {   52, 8508,     318, 14, }, // America/St_Barthelemy
    {   52, 2061,     332, 10, }, // America/St_Johns
    {   52, 5863,     342,  9, }, // America/St_Kitts
    {   52, 5880,     351,  9, }, // America/St_Lucia
    {   52,  855,     360, 10, }, // America/St_Thomas
    {   52, 5913,     370, 11, }, // America/St_Vincent
    {   52, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   52, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   52, 1185,     413,  9, }, // Asia/Kathmandu
    {   52,  992,     422,  7, }, // Asia/Kolkata
    {   52, 1211,     429,  5, }, // Asia/Macau
    {   52, 1235,     442,  6, }, // Asia/Yangon
    {   52, 1395,     448,  5, }, // Atlantic/Faroe
    {   52, 6821,     453, 10, }, // Atlantic/St_Helena
    {   52, 2527,     463,  4, }, // Europe/Kyiv
    {   52, 3157,     474,  5, }, // Pacific/Chuuk
    {   53,   14,       0,  6, }, // Africa/Asmara Bafia/Latin/Cameroon
    {   53, 4435,       6,  8, }, // Africa/Sao_Tome
    {   53, 4252,      14,  8, }, // America/Asuncion
    {   53,  237,      22,  8, }, // America/Atikokan
    {   53, 7623,      30, 17, }, // America/Bahia_Banderas
    {   53, 3868,      47,  6, }, // America/Cancun
    {   53, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   53, 5723,      66,  7, }, // America/Curacao
    {   53,  481,      73, 13, }, // America/Indiana/Knox
    {   53, 7961,      86, 16, }, // America/Indiana/Marengo
    {   53, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   53, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   53, 8038,     139, 14, }, // America/Indiana/Vevay
    {   53, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   53, 8086,     171, 16, }, // America/Indiana/Winamac
    {   53, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   53, 5932,     207, 22, }, // America/Lower_Princes
    {   53, 8220,     229,  6, }, // America/Merida
    {   53, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   53, 8329,     255, 20, }, // America/North_Dakota/Center
    {   53, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   53,  393,     298,  4, }, // America/Nuuk
    {   53, 6899,     302, 16, }, // America/Scoresbysund
    {   53, 8508,     318, 14, }, // America/St_Barthelemy
    {   53, 2061,     332, 10, }, // America/St_Johns
    {   53, 5863,     342,  9, }, // America/St_Kitts
    {   53, 5880,     351,  9, }, // America/St_Lucia
    {   53,  855,     360, 10, }, // America/St_Thomas
    {   53, 5913,     370, 11, }, // America/St_Vincent
    {   53, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   53, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   53, 1185,     413,  9, }, // Asia/Kathmandu
    {   53,  992,     422,  7, }, // Asia/Kolkata
    {   53, 1211,     429,  5, }, // Asia/Macau
    {   53, 1235,     442,  6, }, // Asia/Yangon
    {   53, 1395,     448,  5, }, // Atlantic/Faroe
    {   53, 6821,     453, 10, }, // Atlantic/St_Helena
    {   53, 2527,     463,  4, }, // Europe/Kyiv
    {   53, 7211,     467,  7, }, // Indian/Reunion
    {   53, 3157,     474,  5, }, // Pacific/Chuuk
    {   53, 3096,     479,  7, }, // Pacific/Pohnpei
    {   54,   14,       0,  6, }, // Africa/Asmara Bambara/Latin/Mali
    {   54, 4435,       6,  8, }, // Africa/Sao_Tome
    {   54, 4252,      14,  8, }, // America/Asuncion
    {   54,  237,      22,  8, }, // America/Atikokan
    {   54, 7623,      30, 17, }, // America/Bahia_Banderas
    {   54, 3868,      47,  6, }, // America/Cancun
    {   54, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   54, 5723,      66,  7, }, // America/Curacao
    {   54,  481,      73, 13, }, // America/Indiana/Knox
    {   54, 7961,      86, 16, }, // America/Indiana/Marengo
    {   54, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   54, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   54, 8038,     139, 14, }, // America/Indiana/Vevay
    {   54, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   54, 8086,     171, 16, }, // America/Indiana/Winamac
    {   54, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   54, 5932,     207, 22, }, // America/Lower_Princes
    {   54, 8220,     229,  6, }, // America/Merida
    {   54, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   54, 8329,     255, 20, }, // America/North_Dakota/Center
    {   54, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   54,  393,     298,  4, }, // America/Nuuk
    {   54, 6899,     302, 16, }, // America/Scoresbysund
    {   54, 8508,     318, 14, }, // America/St_Barthelemy
    {   54, 2061,     332, 10, }, // America/St_Johns
    {   54, 5863,     342,  9, }, // America/St_Kitts
    {   54, 5880,     351,  9, }, // America/St_Lucia
    {   54,  855,     360, 10, }, // America/St_Thomas
    {   54, 5913,     370, 11, }, // America/St_Vincent
    {   54, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   54, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   54, 1185,     413,  9, }, // Asia/Kathmandu
    {   54,  992,     422,  7, }, // Asia/Kolkata
    {   54, 1211,     429,  5, }, // Asia/Macau
    {   54, 1235,     442,  6, }, // Asia/Yangon
    {   54, 1395,     448,  5, }, // Atlantic/Faroe
    {   54, 6821,     453, 10, }, // Atlantic/St_Helena
    {   54, 2527,     463,  4, }, // Europe/Kyiv
    {   54, 7211,     467,  7, }, // Indian/Reunion
    {   54, 3157,     474,  5, }, // Pacific/Chuuk
    {   54, 3096,     479,  7, }, // Pacific/Pohnpei
    {   56, 6788,   23046,  7, }, // Africa/Abidjan Bangla/Bangla/Bangladesh
    {   56, 6760,   23053,  5, }, // Africa/Accra
    {   56, 5059,   23058, 12, }, // Africa/Addis_Ababa
    {   56, 7420,   23070, 10, }, // Africa/Algiers
    {   56,   14,   23080, 11, }, // Africa/Asmara
    {   56,   44,   23091,  8, }, // Africa/Bamako
    {   56, 5257,   23099,  7, }, // Africa/Bangui
    {   56, 6746,   23106,  7, }, // Africa/Banjul
    {   56, 7435,   23113,  5, }, // Africa/Bissau
    {   56, 4957,   23118, 13, }, // Africa/Blantyre
    {   56, 5287,   23131, 10, }, // Africa/Brazzaville
    {   56, 4922,   23141, 10, }, // Africa/Bujumbura
    {   56, 2320,   23151,  6, }, // Africa/Cairo
    {   56, 4117,   23157, 12, }, // Africa/Casablanca
    {   56, 7449,   23169,  5, }, // Africa/Ceuta
    {   56, 6773,   23174,  7, }, // Africa/Conakry
    {   56, 6840,   23181,  5, }, // Africa/Dakar
    {   56, 5130,   23186, 12, }, // Africa/Dar_es_Salaam
    {   56, 5043,   16935,  6, }, // Africa/Djibouti
    {   56, 5243,   23198,  7, }, // Africa/Douala
    {   56, 5166,   23205,  8, }, // Africa/El_Aaiun
    {   56, 6853,   23213,  8, }, // Africa/Freetown
    {   56, 4906,   23221,  9, }, // Africa/Gaborone
    {   56, 5015,   23230,  6, }, // Africa/Harare
    {   56, 4479,   23236, 12, }, // Africa/Johannesburg
    {   56, 4499,   16989,  4, }, // Africa/Juba
    {   56, 5151,   23248,  7, }, // Africa/Kampala
    {   56, 4524,   23255,  7, }, // Africa/Khartoum
    {   56, 4973,   23262,  6, }, // Africa/Kigali
    {   56, 5306,   23268,  7, }, // Africa/Kinshasa
    {   56, 4744,   23275,  5, }, // Africa/Lagos
    {   56, 5336,   23280,  9, }, // Africa/Libreville
    {   56, 6869,   23289,  4, }, // Africa/Lome
    {   56, 5211,   23293,  9, }, // Africa/Luanda
    {   56, 4939,   23302, 10, }, // Africa/Lubumbashi
    {   56, 5001,   23312,  6, }, // Africa/Lusaka
    {   56, 5322,   17063,  6, }, // Africa/Malabo
    {   56, 4987,   23318,  6, }, // Africa/Maputo
    {   56, 5197,   23324,  7, }, // Africa/Maseru
    {   56, 5182,   23331,  7, }, // Africa/Mbabane
    {   56, 5113,   23338,  9, }, // Africa/Mogadishu
    {   56, 7034,   23347,  9, }, // Africa/Monrovia
    {   56, 3853,   23356,  7, }, // Africa/Nairobi
    {   56, 5271,   23363,  7, }, // Africa/Ndjamena
    {   56, 5354,   23370,  7, }, // Africa/Niamey
    {   56, 6803,   23377,  8, }, // Africa/Nouakchott
    {   56, 6727,   23385, 11, }, // Africa/Ouagadougou
    {   56, 5225,   23396, 11, }, // Africa/Porto-Novo
    {   56, 4435,   23407,  7, }, // Africa/Sao_Tome
    {   56, 2866,   23414,  8, }, // Africa/Tripoli
    {   56, 6554,   23422,  6, }, // Africa/Tunis
    {   56, 4152,   23428,  6, }, // Africa/Windhoek
    {   56,  132,   23434,  3, }, // America/Adak
    {   56, 3346,   23437,  7, }, // America/Anchorage
    {   56, 5607,   23444, 11, }, // America/Anguilla
    {   56, 5624,   23455, 13, }, // America/Antigua
    {   56, 4554,   23468, 11, }, // America/Araguaina
    {   56,  166,   23479, 16, }, // America/Argentina/Buenos_Aires
    {   56,   91,   23495, 12, }, // America/Argentina/Catamarca
    {   56,  270,   23507,  7, }, // America/Argentina/Cordoba
    {   56,  441,   17251,  5, }, // America/Argentina/Jujuy
    {   56, 7462,   23514,  8, }, // America/Argentina/La_Rioja
    {   56,  565,   23522,  8, }, // America/Argentina/Mendoza
    {   56, 7489,   23530, 12, }, // America/Argentina/Rio_Gallegos
    {   56, 7520,   23542,  7, }, // America/Argentina/Salta
    {   56, 7544,   23549, 10, }, // America/Argentina/San_Juan
    {   56, 5580,   23559,  8, }, // America/Argentina/San_Luis
    {   56, 7571,   23567,  7, }, // America/Argentina/Tucuman
    {   56, 7597,   23574, 10, }, // America/Argentina/Ushuaia
    {   56, 5640,   23584,  5, }, // America/Aruba
    {   56, 4252,   23589,  9, }, // America/Asuncion
    {   56,  237,   17335,  8, }, // America/Atikokan
    {   56, 3679,   17343,  7, }, // America/Bahia
    {   56, 7623,   23598, 17, }, // America/Bahia_Banderas
    {   56, 5654,   23615,  9, }, // America/Barbados
    {   56, 7646,   17376,  5, }, // America/Belem
    {   56, 5407,   17381,  5, }, // America/Belize
    {   56, 7660,   23624, 15, }, // America/Blanc-Sablon
    {   56, 7681,   23639, 12, }, // America/Boa_Vista
    {   56, 4361,   17410,  6, }, // America/Bogota
    {   56, 7699,   23651,  5, }, // America/Boise
    {   56, 7713,   23656, 11, }, // America/Cambridge_Bay
    {   56, 7735,   23667, 15, }, // America/Campo_Grande
    {   56, 3868,   23682,  8, }, // America/Cancun
    {   56, 4694,   23690,  9, }, // America/Caracas
    {   56, 4345,   23699,  5, }, // America/Cayenne
    {   56, 5496,   23704,  5, }, // America/Cayman
    {   56, 2260,   23709,  6, }, // America/Chicago
    {   56, 7756,   23715, 12, }, // America/Chihuahua
    {   56, 7774,   23727, 15, }, // America/Ciudad_Juarez
    {   56, 5422,   23742, 10, }, // America/Costa_Rica
    {   56, 7796,   23752,  9, }, // America/Creston
    {   56, 3791,   23761,  8, }, // America/Cuiaba
    {   56, 5723,   23769,  7, }, // America/Curacao
    {   56, 7812,   23776, 14, }, // America/Danmarkshavn
    {   56, 7833,   23790,  4, }, // America/Dawson
    {   56, 7848,   23794, 10, }, // America/Dawson_Creek
    {   56,  805,   23804,  6, }, // America/Denver
    {   56, 3465,   23810,  8, }, // America/Detroit
    {   56, 5739,   23818,  8, }, // America/Dominica
    {   56,  893,   23826,  8, }, // America/Edmonton
    {   56, 7869,   23834,  8, }, // America/Eirunepe
    {   56, 5441,   23842, 11, }, // America/El_Salvador
    {   56, 7886,   23853, 11, }, // America/Fort_Nelson
    {   56, 7906,   23864, 10, }, // America/Fortaleza
    {   56, 7924,   23874,  8, }, // America/Glace_Bay
    {   56, 6881,   23882,  6, }, // America/Goose_Bay
    {   56, 4612,   23888, 15, }, // America/Grand_Turk
    {   56, 5770,   23903,  8, }, // America/Grenada
    {   56, 5786,   23911, 10, }, // America/Guadeloupe
    {   56, 3760,   23921, 11, }, // America/Guatemala
    {   56, 4841,   23932, 12, }, // America/Guayaquil
    {   56, 6932,   17701,  7, }, // America/Guyana
    {   56, 1939,   23944, 13, }, // America/Halifax
    {   56, 2281,   17717,  6, }, // America/Havana
    {   56, 7942,   23957, 11, }, // America/Hermosillo
    {   56,  348,   23968, 15, }, // America/Indiana/Indianapolis
    {   56,  481,   23983, 16, }, // America/Indiana/Knox
    {   56, 7961,   23999, 20, }, // America/Indiana/Marengo
    {   56, 7985,   24019, 25, }, // America/Indiana/Petersburg
    {   56, 8012,   24044, 19, }, // America/Indiana/Tell_City
    {   56, 8038,   24063, 18, }, // America/Indiana/Vevay
    {   56, 8060,   24081, 22, }, // America/Indiana/Vincennes
    {   56, 8086,   24103, 21, }, // America/Indiana/Winamac
    {   56, 8110,   24124,  8, }, // America/Inuvik
    {   56,  660,   24132, 10, }, // America/Iqaluit
    {   56, 2799,   17897,  7, }, // America/Jamaica
    {   56, 5380,   24142,  4, }, // America/Juneau
    {   56,  521,   24146,  7, }, // America/Kentucky/Louisville
    {   56, 8125,   24153, 19, }, // America/Kentucky/Monticello
    {   56, 5704,   24172, 13, }, // America/Kralendijk
    {   56, 4376,   17949,  6, }, // America/La_Paz
    {   56, 7169,   17955,  4, }, // America/Lima
    {   56, 3239,   24185, 14, }, // America/Los_Angeles
    {   56, 5932,   24199, 27, }, // America/Lower_Princes
    {   56, 8153,   24226,  7, }, // America/Maceio
    {   56, 8168,   24233,  9, }, // America/Managua
    {   56, 1908,   24242,  6, }, // America/Manaus
    {   56, 5897,   24248,  6, }, // America/Marigot
    {   56, 5805,   24254,  8, }, // America/Martinique
    {   56, 8184,   24262,  8, }, // America/Matamoros
    {   56, 2917,   24270,  8, }, // America/Mazatlan
    {   56, 8202,   24278,  8, }, // America/Menominee
    {   56, 8220,   24286,  6, }, // America/Merida
    {   56, 8235,   24292, 10, }, // America/Metlakatla
    {   56, 2949,   24302, 13, }, // America/Mexico_City
    {   56, 4391,   24315,  6, }, // America/Miquelon
    {   56, 8254,   24321,  7, }, // America/Moncton
    {   56, 8270,   24328,  7, }, // America/Monterrey
    {   56, 4098,   24335, 10, }, // America/Montevideo
    {   56, 5824,   24345,  9, }, // America/Montserrat
    {   56, 5481,   24354,  5, }, // America/Nassau
    {   56, 2379,   24359,  9, }, // America/New_York
    {   56, 8288,   18050,  3, }, // America/Nome
    {   56, 1850,   24368,  8, }, // America/Noronha
    {   56, 8301,   24376, 19, }, // America/North_Dakota/Beulah
    {   56, 8329,   24395, 18, }, // America/North_Dakota/Center
    {   56, 8357,   24413, 23, }, // America/North_Dakota/New_Salem
    {   56,  393,   18220,  3, }, // America/Nuuk
    {   56, 8388,   24436,  7, }, // America/Ojinaga
    {   56, 2356,   18231,  6, }, // America/Panama
    {   56, 6173,   24443, 12, }, // America/Paramaribo
    {   56, 2973,   18247,  7, }, // America/Phoenix
    {   56, 3945,   24455, 16, }, // America/Port-au-Prince
    {   56, 5954,   24471, 14, }, // America/Port_of_Spain
    {   56, 8404,   24485, 14, }, // America/Porto_Velho
    {   56, 5843,   24499, 14, }, // America/Puerto_Rico
    {   56, 4030,   24513, 13, }, // America/Punta_Arenas
    {   56, 8424,   24526, 16, }, // America/Rankin_Inlet
    {   56, 8445,   24542,  6, }, // America/Recife
    {   56, 1995,   24548,  6, }, // America/Regina
    {   56, 8460,   24554,  7, }, // America/Resolute
    {   56,  695,   24561, 12, }, // America/Rio_Branco
    {   56, 8477,   24573,  8, }, // America/Santarem
    {   56, 2201,   24581, 11, }, // America/Santiago
    {   56, 6111,   24592, 16, }, // America/Santo_Domingo
    {   56, 1878,   24608,  9, }, // America/Sao_Paulo
    {   56, 6899,   24617, 16, }, // America/Scoresbysund
    {   56, 8494,   24633,  5, }, // America/Sitka
    {   56, 8508,   24638, 16, }, // America/St_Barthelemy
    {   56, 2061,   24654, 10, }, // America/St_Johns
    {   56, 5863,   24664, 11, }, // America/St_Kitts
    {   56, 5880,   24675, 14, }, // America/St_Lucia
    {   56,  855,   24689, 11, }, // America/St_Thomas
    {   56, 5913,   24700, 15, }, // America/St_Vincent
    {   56, 8530,   24715, 14, }, // America/Swift_Current
    {   56, 5461,   24729, 11, }, // America/Tegucigalpa
    {   56, 5756,   24740,  4, }, // America/Thule
    {   56,  313,   24744,  9, }, // America/Tijuana
    {   56,  608,   24753,  7, }, // America/Toronto
    {   56, 5688,   24760,  6, }, // America/Tortola
    {   56, 2093,   24766, 11, }, // America/Vancouver
    {   56, 2144,   24777, 11, }, // America/Whitehorse
    {   56,  734,   24788,  7, }, // America/Winnipeg
    {   56, 8552,   24795,  9, }, // America/Yakutat
    {   56, 6001,   24804,  5, }, // Antarctica/Casey
    {   56, 6094,   24809,  5, }, // Antarctica/Davis
    {   56, 6133,   24814, 15, }, // Antarctica/DumontDUrville
    {   56, 8568,   24829, 14, }, // Antarctica/Macquarie
    {   56, 7066,   24843,  4, }, // Antarctica/Mawson
    {   56, 7113,   24847, 11, }, // Antarctica/McMurdo
    {   56, 8589,   24858,  6, }, // Antarctica/Palmer
    {   56, 7226,   24864,  5, }, // Antarctica/Rothera
    {   56, 7295,   24869,  6, }, // Antarctica/Syowa
    {   56, 8607,   24875,  4, }, // Antarctica/Troll
    {   56, 7374,   24879,  6, }, // Antarctica/Vostok
    {   56, 1429,   24885, 14, }, // Arctic/Longyearbyen
    {   56, 5570,   24899,  5, }, // Asia/Aden
    {   56, 5395,   18694,  6, }, // Asia/Almaty
    {   56, 3968,   18700,  6, }, // Asia/Amman
    {   56, 5511,   24904,  9, }, // Asia/Anadyr
    {   56, 5523,   24913,  5, }, // Asia/Aqtau
    {   56, 5368,   24918,  6, }, // Asia/Aqtobe
    {   56,  964,   24924,  7, }, // Asia/Ashgabat
    {   56, 8624,   24931,  6, }, // Asia/Atyrau
    {   56, 3607,   18745,  6, }, // Asia/Baghdad
    {   56, 5534,   24937,  7, }, // Asia/Bahrain
    {   56, 3653,   17526,  4, }, // Asia/Baku
    {   56, 4466,   24944,  7, }, // Asia/Bangkok
    {   56, 3571,   24951,  7, }, // Asia/Barnaul
    {   56, 4086,   24958,  6, }, // Asia/Beirut
    {   56, 3778,   24964,  6, }, // Asia/Bishkek
    {   56, 5989,   24970,  7, }, // Asia/Brunei
    {   56, 4601,   24977,  4, }, // Asia/Chita
    {   56, 4511,   18797,  6, }, // Asia/Colombo
    {   56, 4540,   24981,  9, }, // Asia/Damascus
    {   56, 1093,   18812,  4, }, // Asia/Dhaka
    {   56, 6192,   24990,  4, }, // Asia/Dili
    {   56, 3596,   24994,  5, }, // Asia/Dubai
    {   56, 6159,   24999,  7, }, // Asia/Dushanbe
    {   56, 8636,   25006, 10, }, // Asia/Famagusta
    {   56, 8651,   18842,  4, }, // Asia/Gaza
    {   56, 4795,   25016,  7, }, // Asia/Hebron
    {   56, 1259,   25023, 13, }, // Asia/Ho_Chi_Minh
    {   56, 2704,   18867,  5, }, // Asia/Hong_Kong
    {   56, 4771,   25036,  5, }, // Asia/Hovd
    {   56, 4184,   25041,  8, }, // Asia/Irkutsk
    {   56, 7006,   25049,  8, }, // Asia/Jakarta
    {   56, 6992,   25057,  8, }, // Asia/Jayapura
    {   56, 1290,   25065,  9, }, // Asia/Jerusalem
    {   56, 3560,   18914,  5, }, // Asia/Kabul
    {   56, 4316,   25074,  8, }, // Asia/Kamchatka
    {   56, 4239,   25082,  5, }, // Asia/Karachi
    {   56, 1185,   25087,  9, }, // Asia/Kathmandu
    {   56, 8661,   25096, 10, }, // Asia/Khandyga
    {   56,  992,   25106,  7, }, // Asia/Kolkata
    {   56, 4197,   25113, 16, }, // Asia/Krasnoyarsk
    {   56, 4859,   25129, 13, }, // Asia/Kuala_Lumpur
    {   56, 5976,   18982,  5, }, // Asia/Kuching
    {   56, 5547,   25142,  6, }, // Asia/Kuwait
    {   56, 1211,   25148,  7, }, // Asia/Macau
    {   56, 4017,   25155,  9, }, // Asia/Magadan
    {   56, 1349,   25164,  8, }, // Asia/Makassar
    {   56, 7182,   25172,  8, }, // Asia/Manila
    {   56, 6920,   25180,  5, }, // Asia/Muscat
    {   56, 2554,   25185,  9, }, // Asia/Nicosia
    {   56, 8675,   25194, 12, }, // Asia/Novokuznetsk
    {   56, 4135,   25206, 12, }, // Asia/Novosibirsk
    {   56, 4229,   25218,  6, }, // Asia/Omsk
    {   56, 7145,   25224,  4, }, // Asia/Oral
    {   56, 6961,   25228,  6, }, // Asia/Phnom_Penh
    {   56, 8693,   25234, 13, }, // Asia/Pontianak
    {   56, 4214,   25247,  9, }, // Asia/Pyongyang
    {   56, 5559,   25256,  5, }, // Asia/Qatar
    {   56, 8708,   25261,  9, }, // Asia/Qostanay
    {   56, 4269,   25270,  9, }, // Asia/Qyzylorda
    {   56, 3584,   25279,  6, }, // Asia/Riyadh
    {   56, 4408,   25285,  7, }, // Asia/Sakhalin
    {   56, 7245,   25292,  7, }, // Asia/Samarkand
    {   56, 3279,   25299,  4, }, // Asia/Seoul
    {   56, 1053,   25303,  6, }, // Asia/Shanghai
    {   56, 3300,   25309,  9, }, // Asia/Singapore
    {   56, 4297,   25318, 16, }, // Asia/Srednekolymsk
    {   56, 3263,   25334,  6, }, // Asia/Taipei
    {   56, 4781,   25340,  7, }, // Asia/Tashkent
    {   56, 3915,   25347,  8, }, // Asia/Tbilisi
    {   56, 2772,   25355,  7, }, // Asia/Tehran
    {   56, 1317,   19206,  6, }, // Asia/Thimphu
    {   56, 2821,   25362,  5, }, // Asia/Tokyo
    {   56, 4572,   25367,  6, }, // Asia/Tomsk
    {   56, 1021,   25373,  9, }, // Asia/Ulaanbaatar
    {   56, 1159,   25382,  6, }, // Asia/Urumqi
    {   56, 8722,   25388, 10, }, // Asia/Ust-Nera
    {   56, 6977,   25398, 12, }, // Asia/Vientiane
    {   56, 4710,   25410, 12, }, // Asia/Vladivostok
    {   56, 4828,   25422, 10, }, // Asia/Yakutsk
    {   56, 1235,   25432,  7, }, // Asia/Yangon
    {   56, 3883,   25439, 16, }, // Asia/Yekaterinburg
    {   56, 3747,   25455,  9, }, // Asia/Yerevan
    {   56, 3663,   25464,  6, }, // Atlantic/Azores
    {   56, 5671,   25470,  7, }, // Atlantic/Bermuda
    {   56, 6628,   25477,  6, }, // Atlantic/Canary
    {   56, 3727,   25483,  9, }, // Atlantic/Cape_Verde
    {   56, 1395,   25492,  6, }, // Atlantic/Faroe
    {   56, 8736,   25498,  7, }, // Atlantic/Madeira
    {   56, 2748,   25505,  8, }, // Atlantic/Reykjavik
    {   56, 7272,   25513, 15, }, // Atlantic/South_Georgia
    {   56, 6821,   25528, 13, }, // Atlantic/St_Helena
    {   56, 6644,   25541,  7, }, // Atlantic/Stanley
    {   56, 1670,   25548, 10, }, // Australia/Adelaide
    {   56, 1635,   25558,  8, }, // Australia/Brisbane
    {   56, 1799,   25566, 11, }, // Australia/Broken_Hill
    {   56, 1583,   25577,  6, }, // Australia/Darwin
    {   56, 3637,   19433,  6, }, // Australia/Eucla
    {   56, 1516,   25583,  7, }, // Australia/Hobart
    {   56, 8753,   25590, 10, }, // Australia/Lindeman
    {   56, 1547,   25600,  8, }, // Australia/Lord_Howe
    {   56, 1727,   25608,  8, }, // Australia/Melbourne
    {   56, 1762,   25616,  5, }, // Australia/Perth
    {   56, 1463,   25621,  5, }, // Australia/Sydney
    {   56, 6441,   25626, 10, }, // Europe/Amsterdam
    {   56, 6216,   25636, 10, }, // Europe/Andorra
    {   56, 3620,   25646,  9, }, // Europe/Astrakhan
    {   56, 2300,   19507,  6, }, // Europe/Athens
    {   56, 6336,   25655,  8, }, // Europe/Belgrade
    {   56, 4757,   25663,  7, }, // Europe/Berlin
    {   56, 6488,   25670, 12, }, // Europe/Bratislava
    {   56, 2167,   25682,  8, }, // Europe/Brussels
    {   56, 3928,   25690,  9, }, // Europe/Bucharest
    {   56, 3806,   25699,  9, }, // Europe/Budapest
    {   56, 8772,   25708,  8, }, // Europe/Busingen
    {   56, 2583,   25716,  8, }, // Europe/Chisinau
    {   56, 6289,   25724, 10, }, // Europe/Copenhagen
    {   56, 2338,   19590,  6, }, // Europe/Dublin
    {   56, 6307,   25734, 11, }, // Europe/Gibraltar
    {   56, 8788,   25745, 10, }, // Europe/Guernsey
    {   56, 6612,   25755,  9, }, // Europe/Helsinki
    {   56, 8804,   25764, 12, }, // Europe/Isle_of_Man
    {   56, 1130,   19631,  9, }, // Europe/Istanbul
    {   56, 8823,   25776,  6, }, // Europe/Jersey
    {   56, 3979,   25782, 12, }, // Europe/Kaliningrad
    {   56, 8929,   25794,  5, }, // Europe/Kirov
    {   56, 2527,   25799,  6, }, // Europe/Kyiv
    {   56, 3213,   25805,  5, }, // Europe/Lisbon
    {   56, 6506,   25810, 10, }, // Europe/Ljubljana
    {   56, 2501,   25820,  5, }, // Europe/London
    {   56, 6365,   25825, 12, }, // Europe/Luxembourg
    {   56, 4892,   25837,  7, }, // Europe/Madrid
    {   56, 6397,   19706,  6, }, // Europe/Malta
    {   56, 6582,   25844, 11, }, // Europe/Mariehamn
    {   56, 3693,   25855,  6, }, // Europe/Minsk
    {   56, 6410,   19728,  6, }, // Europe/Monaco
    {   56, 3537,   19734,  5, }, // Europe/Moscow
    {   56, 6458,   25861,  4, }, // Europe/Oslo
    {   56, 4284,   25865,  7, }, // Europe/Paris
    {   56, 6424,   25872,  9, }, // Europe/Podgorica
    {   56, 6275,   25881,  5, }, // Europe/Prague
    {   56, 8837,   25886,  4, }, // Europe/Riga
    {   56, 6324,   25496,  3, }, // Europe/Rome
    {   56, 4331,   25890,  6, }, // Europe/Samara
    {   56, 6470,   25896, 10, }, // Europe/San_Marino
    {   56, 6245,   25906,  8, }, // Europe/Sarajevo
    {   56, 4451,   25914,  7, }, // Europe/Saratov
    {   56, 8849,   25921, 10, }, // Europe/Simferopol
    {   56, 6383,   25931,  7, }, // Europe/Skopje
    {   56, 6599,   25938,  7, }, // Europe/Sofia
    {   56, 6523,   25945,  7, }, // Europe/Stockholm
    {   56, 8867,   25952,  6, }, // Europe/Tallinn
    {   56, 6202,   25958,  6, }, // Europe/Tirane
    {   56, 8882,   25964, 12, }, // Europe/Ulyanovsk
    {   56, 6352,   19855,  5, }, // Europe/Vaduz
    {   56, 6567,   25976,  7, }, // Europe/Vatican
    {   56, 6231,   19867,  7, }, // Europe/Vienna
    {   56, 8899,   25983,  7, }, // Europe/Vilnius
    {   56, 4727,   25990, 11, }, // Europe/Volgograd
    {   56, 3190,   26001,  6, }, // Europe/Warsaw
    {   56, 6261,   26007,  7, }, // Europe/Zagreb
    {   56, 6540,   26014,  5, }, // Europe/Zurich
    {   56, 5078,   26019, 13, }, // Indian/Antananarivo
    {   56, 6947,   26032,  5, }, // Indian/Chagos
    {   56, 6046,   26037,  8, }, // Indian/Christmas
    {   56, 6063,   26045,  5, }, // Indian/Cocos
    {   56, 5029,   26050,  6, }, // Indian/Comoro
    {   56, 6661,   26056,  9, }, // Indian/Kerguelen
    {   56, 7260,   19959,  4, }, // Indian/Mahe
    {   56, 7050,   19963,  8, }, // Indian/Maldives
    {   56, 4069,   26065,  6, }, // Indian/Mauritius
    {   56, 5098,   26071,  7, }, // Indian/Mayotte
    {   56, 7211,   26078,  9, }, // Indian/Reunion
    {   56, 4422,   19995,  6, }, // Pacific/Apia
    {   56,  932,   26087,  9, }, // Pacific/Auckland
    {   56, 3706,   26096,  9, }, // Pacific/Bougainville
    {   56, 3015,   26105,  7, }, // Pacific/Chatham
    {   56, 3157,   20025,  3, }, // Pacific/Chuuk
    {   56, 2237,   26112,  6, }, // Pacific/Easter
    {   56, 7360,   26118,  5, }, // Pacific/Efate
    {   56, 7327,   26123,  9, }, // Pacific/Fakaofo
    {   56, 3902,   20045,  4, }, // Pacific/Fiji
    {   56, 7343,   20049,  8, }, // Pacific/Funafuti
    {   56, 6678,   26132, 13, }, // Pacific/Galapagos
    {   56, 6696,   26145, 10, }, // Pacific/Gambier
    {   56, 3822,   26155, 13, }, // Pacific/Guadalcanal
    {   56, 6018,   24237,  6, }, // Pacific/Guam
    {   56, 2723,   26168,  7, }, // Pacific/Honolulu
    {   56, 3049,   26175, 10, }, // Pacific/Kanton
    {   56, 3998,   26185, 10, }, // Pacific/Kiritimati
    {   56, 7019,   26195,  7, }, // Pacific/Kosrae
    {   56, 2842,   26202, 11, }, // Pacific/Kwajalein
    {   56, 4877,   26213,  6, }, // Pacific/Majuro
    {   56, 4051,   26219,  9, }, // Pacific/Marquesas
    {   56, 8914,   26228,  7, }, // Pacific/Midway
    {   56, 7084,   26235,  5, }, // Pacific/Nauru
    {   56, 7132,   26240,  6, }, // Pacific/Niue
    {   56, 4168,   26246,  4, }, // Pacific/Norfolk
    {   56, 7098,   26250,  7, }, // Pacific/Noumea
    {   56, 3126,   26257, 13, }, // Pacific/Pago_Pago
    {   56, 7155,   20173,  5, }, // Pacific/Palau
    {   56, 7194,   26270, 11, }, // Pacific/Pitcairn
    {   56, 3096,   26281,  6, }, // Pacific/Pohnpei
    {   56, 4807,   26287, 12, }, // Pacific/Port_Moresby
    {   56, 6076,   26299, 11, }, // Pacific/Rarotonga
    {   56, 6031,   26310,  6, }, // Pacific/Saipan
    {   56, 7312,   20224,  6, }, // Pacific/Tahiti
    {   56, 6712,   26316,  8, }, // Pacific/Tarawa
    {   56, 4583,   26324, 10, }, // Pacific/Tongatapu
    {   56, 7392,   26334,  5, }, // Pacific/Wake
    {   56, 7405,   26339,  4, }, // Pacific/Wallis
    {   58,   14,       0,  6, }, // Africa/Asmara Basaa/Latin/Cameroon
    {   58, 4435,       6,  8, }, // Africa/Sao_Tome
    {   58, 4252,      14,  8, }, // America/Asuncion
    {   58,  237,      22,  8, }, // America/Atikokan
    {   58, 7623,      30, 17, }, // America/Bahia_Banderas
    {   58, 3868,      47,  6, }, // America/Cancun
    {   58, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   58, 5723,      66,  7, }, // America/Curacao
    {   58,  481,      73, 13, }, // America/Indiana/Knox
    {   58, 7961,      86, 16, }, // America/Indiana/Marengo
    {   58, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   58, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   58, 8038,     139, 14, }, // America/Indiana/Vevay
    {   58, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   58, 8086,     171, 16, }, // America/Indiana/Winamac
    {   58, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   58, 5932,     207, 22, }, // America/Lower_Princes
    {   58, 8220,     229,  6, }, // America/Merida
    {   58, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   58, 8329,     255, 20, }, // America/North_Dakota/Center
    {   58, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   58,  393,     298,  4, }, // America/Nuuk
    {   58, 6899,     302, 16, }, // America/Scoresbysund
    {   58, 8508,     318, 14, }, // America/St_Barthelemy
    {   58, 2061,     332, 10, }, // America/St_Johns
    {   58, 5863,     342,  9, }, // America/St_Kitts
    {   58, 5880,     351,  9, }, // America/St_Lucia
    {   58,  855,     360, 10, }, // America/St_Thomas
    {   58, 5913,     370, 11, }, // America/St_Vincent
    {   58, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   58, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   58, 1185,     413,  9, }, // Asia/Kathmandu
    {   58,  992,     422,  7, }, // Asia/Kolkata
    {   58, 1211,     429,  5, }, // Asia/Macau
    {   58, 1235,     442,  6, }, // Asia/Yangon
    {   58, 1395,     448,  5, }, // Atlantic/Faroe
    {   58, 6821,     453, 10, }, // Atlantic/St_Helena
    {   58, 2527,     463,  4, }, // Europe/Kyiv
    {   58, 7211,     467,  7, }, // Indian/Reunion
    {   58, 3157,     474,  5, }, // Pacific/Chuuk
    {   58, 3096,     479,  7, }, // Pacific/Pohnpei
    {   59,   14,       0,  6, }, // Africa/Asmara Bashkir/Cyrillic/Russia
    {   59, 4435,       6,  8, }, // Africa/Sao_Tome
    {   59, 4252,      14,  8, }, // America/Asuncion
    {   59,  237,      22,  8, }, // America/Atikokan
    {   59, 7623,      30, 17, }, // America/Bahia_Banderas
    {   59, 3868,      47,  6, }, // America/Cancun
    {   59, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   59, 5723,      66,  7, }, // America/Curacao
    {   59,  481,      73, 13, }, // America/Indiana/Knox
    {   59, 7961,      86, 16, }, // America/Indiana/Marengo
    {   59, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   59, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   59, 8038,     139, 14, }, // America/Indiana/Vevay
    {   59, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   59, 8086,     171, 16, }, // America/Indiana/Winamac
    {   59, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   59, 5932,     207, 22, }, // America/Lower_Princes
    {   59, 8220,     229,  6, }, // America/Merida
    {   59, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   59, 8329,     255, 20, }, // America/North_Dakota/Center
    {   59, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   59,  393,     298,  4, }, // America/Nuuk
    {   59, 6899,     302, 16, }, // America/Scoresbysund
    {   59, 8508,     318, 14, }, // America/St_Barthelemy
    {   59, 2061,     332, 10, }, // America/St_Johns
    {   59, 5863,     342,  9, }, // America/St_Kitts
    {   59, 5880,     351,  9, }, // America/St_Lucia
    {   59,  855,     360, 10, }, // America/St_Thomas
    {   59, 5913,     370, 11, }, // America/St_Vincent
    {   59, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   59, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   59, 1185,     413,  9, }, // Asia/Kathmandu
    {   59,  992,     422,  7, }, // Asia/Kolkata
    {   59, 1211,     429,  5, }, // Asia/Macau
    {   59, 1235,     442,  6, }, // Asia/Yangon
    {   59, 1395,     448,  5, }, // Atlantic/Faroe
    {   59, 6821,     453, 10, }, // Atlantic/St_Helena
    {   59, 2527,     463,  4, }, // Europe/Kyiv
    {   59, 7211,     467,  7, }, // Indian/Reunion
    {   59, 3157,     474,  5, }, // Pacific/Chuuk
    {   59, 3096,     479,  7, }, // Pacific/Pohnpei
    {   60, 6760,   21174,  5, }, // Africa/Accra Basque/Latin/Spain
    {   60, 5059,    3784, 11, }, // Africa/Addis_Ababa
    {   60, 7420,   26343,  5, }, // Africa/Algiers
    {   60,   14,       0,  6, }, // Africa/Asmara
    {   60, 5257,   26348,  5, }, // Africa/Bangui
    {   60, 2320,    4572,  5, }, // Africa/Cairo
    {   60, 6773,   26353,  7, }, // Africa/Conakry
    {   60, 5043,   26360,  7, }, // Africa/Djibouti
    {   60, 5243,   21254,  5, }, // Africa/Douala
    {   60, 5166,   26367,  5, }, // Africa/El_Aaiun
    {   60, 4524,   26372,  7, }, // Africa/Khartoum
    {   60, 5113,   26379,  8, }, // Africa/Mogadishu
    {   60, 5271,   26387,  9, }, // Africa/Ndjamena
    {   60, 5354,   26396,  6, }, // Africa/Niamey
    {   60, 6803,   20324,  7, }, // Africa/Nouakchott
    {   60, 6727,   26402,  8, }, // Africa/Ouagadougou
    {   60, 5225,   26410, 10, }, // Africa/Porto-Novo
    {   60, 4435,       6,  8, }, // Africa/Sao_Tome
    {   60, 5607,   26420,  7, }, // America/Anguilla
    {   60, 4554,   26427,  9, }, // America/Araguaina
    {   60,  270,   20361,  7, }, // America/Argentina/Cordoba
    {   60, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {   60, 4252,      14,  8, }, // America/Asuncion
    {   60,  237,      22,  8, }, // America/Atikokan
    {   60, 7623,      30, 17, }, // America/Bahia_Banderas
    {   60, 3868,      47,  6, }, // America/Cancun
    {   60, 5496,   26436,  6, }, // America/Cayman
    {   60, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   60, 3791,   26442,  6, }, // America/Cuiaba
    {   60, 5723,      66,  7, }, // America/Curacao
    {   60, 5739,   21548,  8, }, // America/Dominica
    {   60, 7869,   26448,  8, }, // America/Eirunepe
    {   60, 5786,   20408,  9, }, // America/Guadeloupe
    {   60, 2281,   20420,  6, }, // America/Havana
    {   60,  481,      73, 13, }, // America/Indiana/Knox
    {   60, 7961,      86, 16, }, // America/Indiana/Marengo
    {   60, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   60, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   60, 8038,     139, 14, }, // America/Indiana/Vevay
    {   60, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   60, 8086,     171, 16, }, // America/Indiana/Winamac
    {   60, 2799,    3877,  7, }, // America/Jamaica
    {   60, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   60, 5932,     207, 22, }, // America/Lower_Princes
    {   60, 8153,   26456,  6, }, // America/Maceio
    {   60, 5805,   26462,  9, }, // America/Martinique
    {   60, 2917,   26471,  8, }, // America/Mazatlan
    {   60, 8220,     229,  6, }, // America/Merida
    {   60, 2949,   26479, 12, }, // America/Mexico_City
    {   60, 4391,   26491,  8, }, // America/Miquelon
    {   60, 8301,   26499, 19, }, // America/North_Dakota/Beulah
    {   60, 8329,   26518, 19, }, // America/North_Dakota/Center
    {   60, 8357,   26537, 22, }, // America/North_Dakota/New_Salem
    {   60,  393,     298,  4, }, // America/Nuuk
    {   60, 5954,   26559, 13, }, // America/Port_of_Spain
    {   60, 8477,   26572,  8, }, // America/Santarem
    {   60, 1878,   26580,  9, }, // America/Sao_Paulo
    {   60, 6899,     302, 16, }, // America/Scoresbysund
    {   60, 8508,   26589, 16, }, // America/St_Barthelemy
    {   60, 2061,   26605, 12, }, // America/St_Johns
    {   60, 5863,   26617, 11, }, // America/St_Kitts
    {   60, 5880,   26628, 11, }, // America/St_Lucia
    {   60,  855,   26639, 12, }, // America/St_Thomas
    {   60, 5913,   26651, 13, }, // America/St_Vincent
    {   60, 5756,   26664,  7, }, // America/Thule
    {   60, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   60, 5523,    5491,  5, }, // Asia/Aqtau
    {   60, 5368,    7488,  6, }, // Asia/Aqtobe
    {   60,  964,   26671,  7, }, // Asia/Ashgabat
    {   60, 3607,    4044,  6, }, // Asia/Baghdad
    {   60, 3778,   26678,  6, }, // Asia/Bishkek
    {   60, 4601,   26684,  5, }, // Asia/Chita
    {   60, 4511,    5519,  7, }, // Asia/Colombo
    {   60, 4540,   26689,  7, }, // Asia/Damascus
    {   60, 6159,   26696,  7, }, // Asia/Dushanbe
    {   60, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   60, 4771,   26703,  5, }, // Asia/Hovd
    {   60, 4316,   26708,  9, }, // Asia/Kamchatka
    {   60, 4239,   26717,  7, }, // Asia/Karachi
    {   60, 1185,    5593,  8, }, // Asia/Kathmandu
    {   60, 8661,   26724,  8, }, // Asia/Khandyga
    {   60,  992,   26732,  7, }, // Asia/Kolkata
    {   60, 4197,   26739, 11, }, // Asia/Krasnoyarsk
    {   60, 1211,    4203,  5, }, // Asia/Macau
    {   60, 6920,   26750,  6, }, // Asia/Muscat
    {   60, 2554,    5620,  7, }, // Asia/Nicosia
    {   60, 4214,   26756,  9, }, // Asia/Pyongyang
    {   60, 8708,     434,  8, }, // Asia/Qostanay
    {   60, 4269,   26765,  9, }, // Asia/Qyzylorda
    {   60, 3584,    7669,  4, }, // Asia/Riyadh
    {   60, 7245,   26774, 10, }, // Asia/Samarkand
    {   60, 3279,    7680,  4, }, // Asia/Seoul
    {   60, 3300,   20782,  8, }, // Asia/Singapore
    {   60, 4297,    7699, 13, }, // Asia/Srednekolymsk
    {   60, 4781,   26784,  7, }, // Asia/Tashkent
    {   60, 2772,    7724,  7, }, // Asia/Tehran
    {   60, 2821,    4238,  5, }, // Asia/Tokyo
    {   60, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {   60, 1159,   26801,  6, }, // Asia/Urumqi
    {   60, 6977,   26807,  8, }, // Asia/Vientiane
    {   60, 4828,    7755,  7, }, // Asia/Yakutsk
    {   60, 1235,     442,  6, }, // Asia/Yangon
    {   60, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {   60, 3747,   26815,  6, }, // Asia/Yerevan
    {   60, 3663,   26821,  7, }, // Atlantic/Azores
    {   60, 6628,   26828,  8, }, // Atlantic/Canary
    {   60, 3727,   26836, 10, }, // Atlantic/Cape_Verde
    {   60, 1395,     448,  5, }, // Atlantic/Faroe
    {   60, 7272,   26846, 19, }, // Atlantic/South_Georgia
    {   60, 6821,   26865, 12, }, // Atlantic/St_Helena
    {   60, 2300,   26877,  6, }, // Europe/Athens
    {   60, 6336,   26883,  7, }, // Europe/Belgrade
    {   60, 2167,   26890,  7, }, // Europe/Brussels
    {   60, 3928,   26897,  8, }, // Europe/Bucharest
    {   60, 8772,   26905,  8, }, // Europe/Busingen
    {   60, 6289,    4357,  9, }, // Europe/Copenhagen
    {   60, 8788,   26913,  9, }, // Europe/Guernsey
    {   60, 8804,   26922, 11, }, // Europe/Isle_of_Man
    {   60, 2527,    8030,  4, }, // Europe/Kyiv
    {   60, 3213,   21019,  6, }, // Europe/Lisbon
    {   60, 2501,   21034,  7, }, // Europe/London
    {   60, 6365,   26933, 10, }, // Europe/Luxembourg
    {   60, 4892,   26943,  6, }, // Europe/Madrid
    {   60, 6410,    5926,  6, }, // Europe/Monaco
    {   60, 3537,   26949,  5, }, // Europe/Moscow
    {   60, 6275,   21067,  5, }, // Europe/Prague
    {   60, 6324,   26954,  6, }, // Europe/Rome
    {   60, 6202,   21095,  6, }, // Europe/Tirane
    {   60, 8882,   26960,  9, }, // Europe/Ulyanovsk
    {   60, 6567,   26969, 14, }, // Europe/Vatican
    {   60, 6231,   21112,  5, }, // Europe/Vienna
    {   60, 3190,   21126,  8, }, // Europe/Warsaw
    {   60, 6540,    4459,  6, }, // Europe/Zurich
    {   60, 7260,   26983,  4, }, // Indian/Mahe
    {   60, 7050,   26987,  8, }, // Indian/Maldives
    {   60, 4069,   26995,  8, }, // Indian/Mauritius
    {   60, 7211,     467,  7, }, // Indian/Reunion
    {   60, 3157,     474,  5, }, // Pacific/Chuuk
    {   60, 2237,   27003, 13, }, // Pacific/Easter
    {   60, 7360,   27016,  5, }, // Pacific/Efate
    {   60, 6678,   27021, 10, }, // Pacific/Galapagos
    {   60, 2723,    4493,  8, }, // Pacific/Honolulu
    {   60, 3049,    4501,  9, }, // Pacific/Kanton
    {   60, 4051,   27031,  8, }, // Pacific/Marquesas
    {   60, 7098,    4510,  6, }, // Pacific/Noumea
    {   60, 3096,     479,  7, }, // Pacific/Pohnpei
    {   61, 6788,   27039,  7, }, // Africa/Abidjan Belarusian/Cyrillic/Belarus
    {   61, 6760,   27046,  4, }, // Africa/Accra
    {   61, 5059,   27050, 10, }, // Africa/Addis_Ababa
    {   61, 7420,   27060,  5, }, // Africa/Algiers
    {   61,   14,   27065,  6, }, // Africa/Asmara
    {   61,   44,     519,  6, }, // Africa/Bamako
    {   61, 5257,   27071,  5, }, // Africa/Bangui
    {   61, 6746,   27076,  7, }, // Africa/Banjul
    {   61, 7435,   27083,  5, }, // Africa/Bissau
    {   61, 4957,   27088,  8, }, // Africa/Blantyre
    {   61, 5287,   27096,  9, }, // Africa/Brazzaville
    {   61, 4922,     558,  9, }, // Africa/Bujumbura
    {   61, 2320,   27105,  4, }, // Africa/Cairo
    {   61, 4117,     571, 10, }, // Africa/Casablanca
    {   61, 7449,     581,  5, }, // Africa/Ceuta
    {   61, 6773,   27109,  7, }, // Africa/Conakry
    {   61, 6840,     593,  5, }, // Africa/Dakar
    {   61, 5130,   27116, 12, }, // Africa/Dar_es_Salaam
    {   61, 5043,   27128,  7, }, // Africa/Djibouti
    {   61, 5243,     616,  5, }, // Africa/Douala
    {   61, 5166,   27135,  7, }, // Africa/El_Aaiun
    {   61, 6853,   27142,  7, }, // Africa/Freetown
    {   61, 4906,   27149,  8, }, // Africa/Gaborone
    {   61, 5015,   27157,  6, }, // Africa/Harare
    {   61, 4479,   27163, 10, }, // Africa/Johannesburg
    {   61, 4499,   27173,  5, }, // Africa/Juba
    {   61, 5151,     665,  7, }, // Africa/Kampala
    {   61, 4524,     672,  6, }, // Africa/Khartoum
    {   61, 4973,   27178,  6, }, // Africa/Kigali
    {   61, 5306,   27184,  7, }, // Africa/Kinshasa
    {   61, 4744,   27191,  5, }, // Africa/Lagos
    {   61, 5336,   27196,  9, }, // Africa/Libreville
    {   61, 6869,   27205,  4, }, // Africa/Lome
    {   61, 5211,     708,  6, }, // Africa/Luanda
    {   61, 4939,   27209,  9, }, // Africa/Lubumbashi
    {   61, 5001,     723,  6, }, // Africa/Lusaka
    {   61, 5322,   27218,  6, }, // Africa/Malabo
    {   61, 4987,     735,  6, }, // Africa/Maputo
    {   61, 5197,     741,  6, }, // Africa/Maseru
    {   61, 5182,   27224,  7, }, // Africa/Mbabane
    {   61, 5113,   27231,  8, }, // Africa/Mogadishu
    {   61, 7034,   27239,  8, }, // Africa/Monrovia
    {   61, 3853,   27247,  7, }, // Africa/Nairobi
    {   61, 5271,   27254,  8, }, // Africa/Ndjamena
    {   61, 5354,   27262,  6, }, // Africa/Niamey
    {   61, 6803,     790,  7, }, // Africa/Nouakchott
    {   61, 6727,     797,  8, }, // Africa/Ouagadougou
    {   61, 5225,   27268, 10, }, // Africa/Porto-Novo
    {   61, 4435,   27278,  8, }, // Africa/Sao_Tome
    {   61, 2866,   27286,  7, }, // Africa/Tripoli
    {   61, 6554,   27293,  5, }, // Africa/Tunis
    {   61, 4152,   27298,  7, }, // Africa/Windhoek
    {   61,  132,     842,  4, }, // America/Adak
    {   61, 3346,   27305,  8, }, // America/Anchorage
    {   61, 5607,   27313,  7, }, // America/Anguilla
    {   61, 5624,   27320,  7, }, // America/Antigua
    {   61, 4554,   27327,  9, }, // America/Araguaina
    {   61,  166,   27336, 12, }, // America/Argentina/Buenos_Aires
    {   61,   91,     888,  9, }, // America/Argentina/Catamarca
    {   61,  270,   27348,  7, }, // America/Argentina/Cordoba
    {   61,  441,   27355,  5, }, // America/Argentina/Jujuy
    {   61, 7462,   27360,  8, }, // America/Argentina/La_Rioja
    {   61,  565,     917,  7, }, // America/Argentina/Mendoza
    {   61, 7489,   27368, 12, }, // America/Argentina/Rio_Gallegos
    {   61, 7520,   27380,  6, }, // America/Argentina/Salta
    {   61, 7544,     940,  8, }, // America/Argentina/San_Juan
    {   61, 5580,   27386,  8, }, // America/Argentina/San_Luis
    {   61, 7571,     956,  7, }, // America/Argentina/Tucuman
    {   61, 7597,   27394,  5, }, // America/Argentina/Ushuaia
    {   61, 5640,     969,  5, }, // America/Aruba
    {   61, 4252,   27399,  8, }, // America/Asuncion
    {   61,  237,   27407,  8, }, // America/Atikokan
    {   61, 3679,   27415,  4, }, // America/Bahia
    {   61, 7623,   27419, 16, }, // America/Bahia_Banderas
    {   61, 5654,   27435,  8, }, // America/Barbados
    {   61, 7646,    1021,  5, }, // America/Belem
    {   61, 5407,   27443,  5, }, // America/Belize
    {   61, 7660,    1031, 12, }, // America/Blanc-Sablon
    {   61, 7681,   27448,  9, }, // America/Boa_Vista
    {   61, 4361,   27457,  6, }, // America/Bogota
    {   61, 7699,   27463,  5, }, // America/Boise
    {   61, 7713,   27468, 12, }, // America/Cambridge_Bay
    {   61, 7735,   27480, 12, }, // America/Campo_Grande
    {   61, 3868,    1087,  6, }, // America/Cancun
    {   61, 4694,    1093,  7, }, // America/Caracas
    {   61, 4345,   27492,  5, }, // America/Cayenne
    {   61, 5496,   27497, 17, }, // America/Cayman
    {   61, 2260,   27514,  6, }, // America/Chicago
    {   61, 7756,   27520,  6, }, // America/Chihuahua
    {   61, 7774,   27526, 13, }, // America/Ciudad_Juarez
    {   61, 5422,   27539, 10, }, // America/Costa_Rica
    {   61, 7796,   27549,  7, }, // America/Creston
    {   61, 3791,   27556,  5, }, // America/Cuiaba
    {   61, 5723,   27561,  7, }, // America/Curacao
    {   61, 7812,   27568, 12, }, // America/Danmarkshavn
    {   61, 7833,   27580,  6, }, // America/Dawson
    {   61, 7848,   27586, 10, }, // America/Dawson_Creek
    {   61,  805,   27596,  6, }, // America/Denver
    {   61, 3465,   27602,  7, }, // America/Detroit
    {   61, 5739,   27609,  8, }, // America/Dominica
    {   61,  893,   27617,  8, }, // America/Edmonton
    {   61, 7869,   27625,  8, }, // America/Eirunepe
    {   61, 5441,   27633,  9, }, // America/El_Salvador
    {   61, 7886,   27642, 12, }, // America/Fort_Nelson
    {   61, 7906,   27654,  9, }, // America/Fortaleza
    {   61, 7924,   27663,  9, }, // America/Glace_Bay
    {   61, 6881,   27672,  7, }, // America/Goose_Bay
    {   61, 4612,   27679, 10, }, // America/Grand_Turk
    {   61, 5770,   27689,  7, }, // America/Grenada
    {   61, 5786,   27696,  9, }, // America/Guadeloupe
    {   61, 3760,   27705,  9, }, // America/Guatemala
    {   61, 4841,   27714,  8, }, // America/Guayaquil
    {   61, 6932,   27722,  5, }, // America/Guyana
    {   61, 1939,   27727,  8, }, // America/Halifax
    {   61, 2281,    1339,  6, }, // America/Havana
    {   61, 7942,   27735,  9, }, // America/Hermosillo
    {   61,  348,   27744, 12, }, // America/Indiana/Indianapolis
    {   61,  481,   27756, 13, }, // America/Indiana/Knox
    {   61, 7961,   27769, 16, }, // America/Indiana/Marengo
    {   61, 7985,   27785, 19, }, // America/Indiana/Petersburg
    {   61, 8012,   27804, 17, }, // America/Indiana/Tell_City
    {   61, 8038,   27821, 13, }, // America/Indiana/Vevay
    {   61, 8060,   27834, 16, }, // America/Indiana/Vincennes
    {   61, 8086,   27850, 16, }, // America/Indiana/Winamac
    {   61, 8110,   27866,  6, }, // America/Inuvik
    {   61,  660,   27872,  7, }, // America/Iqaluit
    {   61, 2799,   27879,  6, }, // America/Jamaica
    {   61, 5380,   27885,  5, }, // America/Juneau
    {   61,  521,   27890,  7, }, // America/Kentucky/Louisville
    {   61, 8125,   27897, 18, }, // America/Kentucky/Monticello
    {   61, 5704,   27915, 10, }, // America/Kralendijk
    {   61, 4376,    1514,  6, }, // America/La_Paz
    {   61, 7169,   27925,  4, }, // America/Lima
    {   61, 3239,   27929, 12, }, // America/Los_Angeles
    {   61, 5932,   27941, 20, }, // America/Lower_Princes
    {   61, 8153,   27961,  5, }, // America/Maceio
    {   61, 8168,    1559,  7, }, // America/Managua
    {   61, 1908,   27966,  6, }, // America/Manaus
    {   61, 5897,   27972,  6, }, // America/Marigot
    {   61, 5805,   27978,  9, }, // America/Martinique
    {   61, 8184,   27987,  9, }, // America/Matamoros
    {   61, 2917,    1596,  8, }, // America/Mazatlan
    {   61, 8202,   27996,  8, }, // America/Menominee
    {   61, 8220,   28004,  6, }, // America/Merida
    {   61, 8235,    1618, 10, }, // America/Metlakatla
    {   61, 2949,   28010,  6, }, // America/Mexico_City
    {   61, 4391,   28016,  7, }, // America/Miquelon
    {   61, 8254,   28023,  7, }, // America/Moncton
    {   61, 8270,   28030,  8, }, // America/Monterrey
    {   61, 4098,   28038, 10, }, // America/Montevideo
    {   61, 5824,   28048,  9, }, // America/Montserrat
    {   61, 5481,   28057,  5, }, // America/Nassau
    {   61, 2379,   28062,  7, }, // America/New_York
    {   61, 8288,    1691,  3, }, // America/Nome
    {   61, 1850,   28069,  7, }, // America/Noronha
    {   61, 8301,   28076, 23, }, // America/North_Dakota/Beulah
    {   61, 8329,   28099, 24, }, // America/North_Dakota/Center
    {   61, 8357,   28123, 28, }, // America/North_Dakota/New_Salem
    {   61,  393,   28151,  3, }, // America/Nuuk
    {   61, 8388,   28154,  7, }, // America/Ojinaga
    {   61, 2356,    1783,  6, }, // America/Panama
    {   61, 6173,   28161, 10, }, // America/Paramaribo
    {   61, 2973,   28171,  6, }, // America/Phoenix
    {   61, 3945,   28177, 12, }, // America/Port-au-Prince
    {   61, 5954,   28189, 13, }, // America/Port_of_Spain
    {   61, 8404,   28202, 11, }, // America/Porto_Velho
    {   61, 5843,   28213, 11, }, // America/Puerto_Rico
    {   61, 4030,   28224, 12, }, // America/Punta_Arenas
    {   61, 8424,   28236, 12, }, // America/Rankin_Inlet
    {   61, 8445,   28248,  6, }, // America/Recife
    {   61, 1995,   28254,  8, }, // America/Regina
    {   61, 8460,   28262,  8, }, // America/Resolute
    {   61,  695,   28270, 10, }, // America/Rio_Branco
    {   61, 8477,   28280,  8, }, // America/Santarem
    {   61, 2201,   28288,  8, }, // America/Santiago
    {   61, 6111,   28296, 13, }, // America/Santo_Domingo
    {   61, 1878,   28309,  9, }, // America/Sao_Paulo
    {   61, 6899,   28318, 13, }, // America/Scoresbysund
    {   61, 8494,   28331,  5, }, // America/Sitka
    {   61, 8508,   28336, 13, }, // America/St_Barthelemy
    {   61, 2061,   28349, 10, }, // America/St_Johns
    {   61, 5863,   28359,  9, }, // America/St_Kitts
    {   61, 5880,   28368, 10, }, // America/St_Lucia
    {   61,  855,    2003, 10, }, // America/St_Thomas
    {   61, 5913,   28378, 12, }, // America/St_Vincent
    {   61, 8530,   28390, 12, }, // America/Swift_Current
    {   61, 5461,   28402, 12, }, // America/Tegucigalpa
    {   61, 5756,    2049,  4, }, // America/Thule
    {   61,  313,   28414,  7, }, // America/Tijuana
    {   61,  608,   28421,  7, }, // America/Toronto
    {   61, 5688,   28428,  7, }, // America/Tortola
    {   61, 2093,    2074,  8, }, // America/Vancouver
    {   61, 2144,   28435,  8, }, // America/Whitehorse
    {   61,  734,   28443,  7, }, // America/Winnipeg
    {   61, 8552,   28450,  6, }, // America/Yakutat
    {   61, 6001,   28456,  5, }, // Antarctica/Casey
    {   61, 6094,   28461,  6, }, // Antarctica/Davis
    {   61, 6133,   28467, 15, }, // Antarctica/DumontDUrville
    {   61, 8568,   28482,  7, }, // Antarctica/Macquarie
    {   61, 7066,   28489,  6, }, // Antarctica/Mawson
    {   61, 7113,   28495,  9, }, // Antarctica/McMurdo
    {   61, 8589,    2154,  6, }, // Antarctica/Palmer
    {   61, 7226,   28504,  6, }, // Antarctica/Rothera
    {   61, 7295,   28510,  4, }, // Antarctica/Syowa
    {   61, 8607,    2171,  4, }, // Antarctica/Troll
    {   61, 7374,   28514,  6, }, // Antarctica/Vostok
    {   61, 1429,   28520,  7, }, // Arctic/Longyearbyen
    {   61, 5570,   28527,  4, }, // Asia/Aden
    {   61, 5395,    2196,  6, }, // Asia/Almaty
    {   61, 3968,   28531, 12, }, // Asia/Amman
    {   61, 5511,    2207,  6, }, // Asia/Anadyr
    {   61, 5523,    2213,  5, }, // Asia/Aqtau
    {   61, 5368,   28543,  6, }, // Asia/Aqtobe
    {   61,  964,   28549,  7, }, // Asia/Ashgabat
    {   61, 8624,    2231,  6, }, // Asia/Atyrau
    {   61, 3607,    2237,  6, }, // Asia/Baghdad
    {   61, 5534,   28556,  7, }, // Asia/Bahrain
    {   61, 3653,    2250,  4, }, // Asia/Baku
    {   61, 4466,    2254,  7, }, // Asia/Bangkok
    {   61, 3571,    2261,  7, }, // Asia/Barnaul
    {   61, 4086,   28563,  6, }, // Asia/Beirut
    {   61, 3778,   28569,  6, }, // Asia/Bishkek
    {   61, 5989,   28575,  6, }, // Asia/Brunei
    {   61, 4601,   28581,  4, }, // Asia/Chita
    {   61, 4511,   28585,  7, }, // Asia/Colombo
    {   61, 4540,    2297,  6, }, // Asia/Damascus
    {   61, 1093,     593,  4, }, // Asia/Dhaka
    {   61, 6192,   28592,  4, }, // Asia/Dili
    {   61, 3596,   28596,  5, }, // Asia/Dubai
    {   61, 6159,   28601,  7, }, // Asia/Dushanbe
    {   61, 8636,    2324,  9, }, // Asia/Famagusta
    {   61, 8651,    2333,  4, }, // Asia/Gaza
    {   61, 4795,   28608,  6, }, // Asia/Hebron
    {   61, 1259,   28614,  7, }, // Asia/Ho_Chi_Minh
    {   61, 2704,   28621,  7, }, // Asia/Hong_Kong
    {   61, 4771,   28628,  4, }, // Asia/Hovd
    {   61, 4184,   28632,  6, }, // Asia/Irkutsk
    {   61, 7006,   28638,  8, }, // Asia/Jakarta
    {   61, 6992,   28646,  8, }, // Asia/Jayapura
    {   61, 1290,   28654,  9, }, // Asia/Jerusalem
    {   61, 3560,    2392,  5, }, // Asia/Kabul
    {   61, 4316,    2411,  8, }, // Asia/Kamchatka
    {   61, 4239,   28663,  6, }, // Asia/Karachi
    {   61, 1185,    2428,  8, }, // Asia/Kathmandu
    {   61, 8661,    2436,  7, }, // Asia/Khandyga
    {   61,  992,   28669,  8, }, // Asia/Kolkata
    {   61, 4197,   28677, 10, }, // Asia/Krasnoyarsk
    {   61, 4859,    2462, 12, }, // Asia/Kuala_Lumpur
    {   61, 5976,   28687,  6, }, // Asia/Kuching
    {   61, 5547,   28693,  6, }, // Asia/Kuwait
    {   61, 1211,   28699,  5, }, // Asia/Macau
    {   61, 4017,    2491,  7, }, // Asia/Magadan
    {   61, 1349,    2498,  7, }, // Asia/Makassar
    {   61, 7182,   28704,  6, }, // Asia/Manila
    {   61, 6920,    2511,  6, }, // Asia/Muscat
    {   61, 2554,   28710,  7, }, // Asia/Nicosia
    {   61, 8675,   28717, 11, }, // Asia/Novokuznetsk
    {   61, 4135,   28728, 11, }, // Asia/Novosibirsk
    {   61, 4229,    2546,  4, }, // Asia/Omsk
    {   61, 7145,   28739,  7, }, // Asia/Oral
    {   61, 6961,   28746,  8, }, // Asia/Phnom_Penh
    {   61, 8693,   28754,  9, }, // Asia/Pontianak
    {   61, 4214,   28763,  7, }, // Asia/Pyongyang
    {   61, 5559,    2579,  5, }, // Asia/Qatar
    {   61, 8708,   28770,  8, }, // Asia/Qostanay
    {   61, 4269,   28778,  9, }, // Asia/Qyzylorda
    {   61, 3584,   28787,  7, }, // Asia/Riyadh
    {   61, 4408,   28794,  7, }, // Asia/Sakhalin
    {   61, 7245,    2621,  9, }, // Asia/Samarkand
    {   61, 3279,    2630,  4, }, // Asia/Seoul
    {   61, 1053,   28801,  6, }, // Asia/Shanghai
    {   61, 3300,   28807,  8, }, // Asia/Singapore
    {   61, 4297,   28815, 14, }, // Asia/Srednekolymsk
    {   61, 3263,   28829,  6, }, // Asia/Taipei
    {   61, 4781,    2667,  7, }, // Asia/Tashkent
    {   61, 3915,   28835,  7, }, // Asia/Tbilisi
    {   61, 2772,   28842,  7, }, // Asia/Tehran
    {   61, 1317,   28849,  7, }, // Asia/Thimphu
    {   61, 2821,   28856,  5, }, // Asia/Tokyo
    {   61, 4572,    2697,  5, }, // Asia/Tomsk
    {   61, 1021,   28861, 10, }, // Asia/Ulaanbaatar
    {   61, 1159,   28871,  6, }, // Asia/Urumqi
    {   61, 8722,   28877,  9, }, // Asia/Ust-Nera
    {   61, 6977,   28886,  8, }, // Asia/Vientiane
    {   61, 4710,   28894, 12, }, // Asia/Vladivostok
    {   61, 4828,   28906,  5, }, // Asia/Yakutsk
    {   61, 1235,   28911,  6, }, // Asia/Yangon
    {   61, 3883,   28917, 12, }, // Asia/Yekaterinburg
    {   61, 3747,   28929,  6, }, // Asia/Yerevan
    {   61, 3663,   28935, 16, }, // Atlantic/Azores
    {   61, 5671,   28951, 18, }, // Atlantic/Bermuda
    {   61, 6628,   28969, 17, }, // Atlantic/Canary
    {   61, 3727,   28986, 10, }, // Atlantic/Cape_Verde
    {   61, 1395,   28996, 17, }, // Atlantic/Faroe
    {   61, 8736,   29013,  7, }, // Atlantic/Madeira
    {   61, 2748,   29020,  9, }, // Atlantic/Reykjavik
    {   61, 7272,   29029, 17, }, // Atlantic/South_Georgia
    {   61, 6821,   29046, 20, }, // Atlantic/St_Helena
    {   61, 6644,   29066, 11, }, // Atlantic/Stanley
    {   61, 1670,   29077,  8, }, // Australia/Adelaide
    {   61, 1635,   29085,  7, }, // Australia/Brisbane
    {   61, 1799,   29092, 10, }, // Australia/Broken_Hill
    {   61, 1583,   29102,  6, }, // Australia/Darwin
    {   61, 3637,   29108,  4, }, // Australia/Eucla
    {   61, 1516,    2954,  6, }, // Australia/Hobart
    {   61, 8753,   29112,  8, }, // Australia/Lindeman
    {   61, 1547,    2968,  8, }, // Australia/Lord_Howe
    {   61, 1727,   29120,  8, }, // Australia/Melbourne
    {   61, 1762,    2983,  4, }, // Australia/Perth
    {   61, 1463,   29128,  6, }, // Australia/Sydney
    {   61, 6441,   29134,  9, }, // Europe/Amsterdam
    {   61, 6216,   29143,  6, }, // Europe/Andorra
    {   61, 3620,   29149,  9, }, // Europe/Astrakhan
    {   61, 2300,   29158,  5, }, // Europe/Athens
    {   61, 6336,   29163,  7, }, // Europe/Belgrade
    {   61, 4757,   29170,  6, }, // Europe/Berlin
    {   61, 6488,   29176, 10, }, // Europe/Bratislava
    {   61, 2167,   29186,  7, }, // Europe/Brussels
    {   61, 3928,   29193,  8, }, // Europe/Bucharest
    {   61, 3806,    3060,  8, }, // Europe/Budapest
    {   61, 8772,   29201,  8, }, // Europe/Busingen
    {   61, 2583,   29209,  7, }, // Europe/Chisinau
    {   61, 6289,   29216, 10, }, // Europe/Copenhagen
    {   61, 2338,   29226,  6, }, // Europe/Dublin
    {   61, 6307,   29232,  9, }, // Europe/Gibraltar
    {   61, 8788,   29241,  6, }, // Europe/Guernsey
    {   61, 6612,   29247,  9, }, // Europe/Helsinki
    {   61, 8804,   29256, 11, }, // Europe/Isle_of_Man
    {   61, 1130,    3146,  7, }, // Europe/Istanbul
    {   61, 8823,   29267,  6, }, // Europe/Jersey
    {   61, 3979,   29273, 11, }, // Europe/Kaliningrad
    {   61, 8929,   29284,  5, }, // Europe/Kirov
    {   61, 2527,   29289,  4, }, // Europe/Kyiv
    {   61, 3213,   29293,  7, }, // Europe/Lisbon
    {   61, 6506,   29300,  7, }, // Europe/Ljubljana
    {   61, 2501,   29307,  6, }, // Europe/London
    {   61, 6365,   29313, 10, }, // Europe/Luxembourg
    {   61, 4892,   29323,  6, }, // Europe/Madrid
    {   61, 6397,   29329,  6, }, // Europe/Malta
    {   61, 6582,   29335, 12, }, // Europe/Mariehamn
    {   61, 3693,   29347,  5, }, // Europe/Minsk
    {   61, 6410,   29352,  6, }, // Europe/Monaco
    {   61, 3537,   29358,  6, }, // Europe/Moscow
    {   61, 6458,   29364,  4, }, // Europe/Oslo
    {   61, 4284,   29368,  5, }, // Europe/Paris
    {   61, 6424,   29373,  9, }, // Europe/Podgorica
    {   61, 6275,    3267,  5, }, // Europe/Prague
    {   61, 8837,   29382,  4, }, // Europe/Riga
    {   61, 6324,   29386,  3, }, // Europe/Rome
    {   61, 4331,    3279,  6, }, // Europe/Samara
    {   61, 6470,   29389, 10, }, // Europe/San_Marino
    {   61, 6245,   29399,  7, }, // Europe/Sarajevo
    {   61, 4451,   29406,  7, }, // Europe/Saratov
    {   61, 8849,   29413, 11, }, // Europe/Simferopol
    {   61, 6383,   29424,  6, }, // Europe/Skopje
    {   61, 6599,   29430,  5, }, // Europe/Sofia
    {   61, 6523,   29435,  9, }, // Europe/Stockholm
    {   61, 8867,   29444,  5, }, // Europe/Tallinn
    {   61, 6202,   29449,  6, }, // Europe/Tirane
    {   61, 8882,   29455,  9, }, // Europe/Ulyanovsk
    {   61, 6352,    3358,  5, }, // Europe/Vaduz
    {   61, 6567,   29464,  7, }, // Europe/Vatican
    {   61, 6231,    3370,  4, }, // Europe/Vienna
    {   61, 8899,   29471,  7, }, // Europe/Vilnius
    {   61, 4727,   29478,  9, }, // Europe/Volgograd
    {   61, 3190,    3390,  7, }, // Europe/Warsaw
    {   61, 6261,   29487,  6, }, // Europe/Zagreb
    {   61, 6540,   29493,  5, }, // Europe/Zurich
    {   61, 5078,   29498, 12, }, // Indian/Antananarivo
    {   61, 6947,   29510,  5, }, // Indian/Chagos
    {   61, 6046,   29515, 13, }, // Indian/Christmas
    {   61, 6063,   29528, 17, }, // Indian/Cocos
    {   61, 5029,   29545, 17, }, // Indian/Comoro
    {   61, 6661,    3460,  8, }, // Indian/Kerguelen
    {   61, 7260,   29562,  3, }, // Indian/Mahe
    {   61, 7050,   29565,  8, }, // Indian/Maldives
    {   61, 4069,   29573,  8, }, // Indian/Mauritius
    {   61, 5098,   29581,  5, }, // Indian/Mayotte
    {   61, 7211,   29586,  7, }, // Indian/Reunion
    {   61, 4422,   29593,  4, }, // Pacific/Apia
    {   61,  932,    3502,  6, }, // Pacific/Auckland
    {   61, 3706,   29597,  9, }, // Pacific/Bougainville
    {   61, 3015,   29606,  5, }, // Pacific/Chatham
    {   61, 3157,    3521,  4, }, // Pacific/Chuuk
    {   61, 2237,   29611, 17, }, // Pacific/Easter
    {   61, 7360,   29628,  5, }, // Pacific/Efate
    {   61, 7327,   29633,  7, }, // Pacific/Fakaofo
    {   61, 3902,   29640,  5, }, // Pacific/Fiji
    {   61, 7343,   29645,  8, }, // Pacific/Funafuti
    {   61, 6678,   29653,  9, }, // Pacific/Galapagos
    {   61, 6696,   29662, 14, }, // Pacific/Gambier
    {   61, 3822,    3595, 11, }, // Pacific/Guadalcanal
    {   61, 6018,    3606,  4, }, // Pacific/Guam
    {   61, 2723,   29676,  8, }, // Pacific/Honolulu
    {   61, 3049,   29684,  9, }, // Pacific/Kanton
    {   61, 3998,   29693, 10, }, // Pacific/Kiritimati
    {   61, 7019,   29703,  6, }, // Pacific/Kosrae
    {   61, 2842,   29709, 10, }, // Pacific/Kwajalein
    {   61, 4877,   29719,  7, }, // Pacific/Majuro
    {   61, 4051,   29726, 18, }, // Pacific/Marquesas
    {   61, 8914,   29744,  6, }, // Pacific/Midway
    {   61, 7084,    3680,  5, }, // Pacific/Nauru
    {   61, 7132,   29750,  4, }, // Pacific/Niue
    {   61, 4168,   29754,  7, }, // Pacific/Norfolk
    {   61, 7098,    3696,  5, }, // Pacific/Noumea
    {   61, 3126,   29761,  9, }, // Pacific/Pago_Pago
    {   61, 7155,    3710,  5, }, // Pacific/Palau
    {   61, 7194,   29770,  7, }, // Pacific/Pitcairn
    {   61, 3096,   29777,  6, }, // Pacific/Pohnpei
    {   61, 4807,   29783, 11, }, // Pacific/Port_Moresby
    {   61, 6076,   29794,  9, }, // Pacific/Rarotonga
    {   61, 6031,   29803,  6, }, // Pacific/Saipan
    {   61, 7312,   29809,  5, }, // Pacific/Tahiti
    {   61, 6712,    3759,  6, }, // Pacific/Tarawa
    {   61, 4583,   29814,  9, }, // Pacific/Tongatapu
    {   61, 7392,   29823,  4, }, // Pacific/Wake
    {   61, 7405,   29827,  5, }, // Pacific/Wallis
    {   62,   14,       0,  6, }, // Africa/Asmara Bemba/Latin/Zambia
    {   62, 4435,       6,  8, }, // Africa/Sao_Tome
    {   62, 4252,      14,  8, }, // America/Asuncion
    {   62,  237,      22,  8, }, // America/Atikokan
    {   62, 7623,      30, 17, }, // America/Bahia_Banderas
    {   62, 3868,      47,  6, }, // America/Cancun
    {   62, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   62, 5723,      66,  7, }, // America/Curacao
    {   62,  481,      73, 13, }, // America/Indiana/Knox
    {   62, 7961,      86, 16, }, // America/Indiana/Marengo
    {   62, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   62, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   62, 8038,     139, 14, }, // America/Indiana/Vevay
    {   62, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   62, 8086,     171, 16, }, // America/Indiana/Winamac
    {   62, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   62, 5932,     207, 22, }, // America/Lower_Princes
    {   62, 8220,     229,  6, }, // America/Merida
    {   62, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   62, 8329,     255, 20, }, // America/North_Dakota/Center
    {   62, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   62,  393,     298,  4, }, // America/Nuuk
    {   62, 6899,     302, 16, }, // America/Scoresbysund
    {   62, 8508,     318, 14, }, // America/St_Barthelemy
    {   62, 2061,     332, 10, }, // America/St_Johns
    {   62, 5863,     342,  9, }, // America/St_Kitts
    {   62, 5880,     351,  9, }, // America/St_Lucia
    {   62,  855,     360, 10, }, // America/St_Thomas
    {   62, 5913,     370, 11, }, // America/St_Vincent
    {   62, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   62, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   62, 1185,     413,  9, }, // Asia/Kathmandu
    {   62,  992,     422,  7, }, // Asia/Kolkata
    {   62, 1211,     429,  5, }, // Asia/Macau
    {   62, 1235,     442,  6, }, // Asia/Yangon
    {   62, 1395,     448,  5, }, // Atlantic/Faroe
    {   62, 6821,     453, 10, }, // Atlantic/St_Helena
    {   62, 2527,     463,  4, }, // Europe/Kyiv
    {   62, 7211,     467,  7, }, // Indian/Reunion
    {   62, 3157,     474,  5, }, // Pacific/Chuuk
    {   62, 3096,     479,  7, }, // Pacific/Pohnpei
    {   63,   14,       0,  6, }, // Africa/Asmara Bena/Latin/Tanzania
    {   63, 4435,       6,  8, }, // Africa/Sao_Tome
    {   63, 4252,      14,  8, }, // America/Asuncion
    {   63,  237,      22,  8, }, // America/Atikokan
    {   63, 7623,      30, 17, }, // America/Bahia_Banderas
    {   63, 3868,      47,  6, }, // America/Cancun
    {   63, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   63, 5723,      66,  7, }, // America/Curacao
    {   63,  481,      73, 13, }, // America/Indiana/Knox
    {   63, 7961,      86, 16, }, // America/Indiana/Marengo
    {   63, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   63, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   63, 8038,     139, 14, }, // America/Indiana/Vevay
    {   63, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   63, 8086,     171, 16, }, // America/Indiana/Winamac
    {   63, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   63, 5932,     207, 22, }, // America/Lower_Princes
    {   63, 8220,     229,  6, }, // America/Merida
    {   63, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   63, 8329,     255, 20, }, // America/North_Dakota/Center
    {   63, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   63,  393,     298,  4, }, // America/Nuuk
    {   63, 6899,     302, 16, }, // America/Scoresbysund
    {   63, 8508,     318, 14, }, // America/St_Barthelemy
    {   63, 2061,     332, 10, }, // America/St_Johns
    {   63, 5863,     342,  9, }, // America/St_Kitts
    {   63, 5880,     351,  9, }, // America/St_Lucia
    {   63,  855,     360, 10, }, // America/St_Thomas
    {   63, 5913,     370, 11, }, // America/St_Vincent
    {   63, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   63, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   63, 1185,     413,  9, }, // Asia/Kathmandu
    {   63,  992,     422,  7, }, // Asia/Kolkata
    {   63, 1211,     429,  5, }, // Asia/Macau
    {   63, 1235,     442,  6, }, // Asia/Yangon
    {   63, 1395,     448,  5, }, // Atlantic/Faroe
    {   63, 6821,     453, 10, }, // Atlantic/St_Helena
    {   63, 2527,     463,  4, }, // Europe/Kyiv
    {   63, 7211,     467,  7, }, // Indian/Reunion
    {   63, 3157,     474,  5, }, // Pacific/Chuuk
    {   63, 3096,     479,  7, }, // Pacific/Pohnpei
    {   64,   14,       0,  6, }, // Africa/Asmara Bhojpuri/Devanagari/India
    {   64, 4435,       6,  8, }, // Africa/Sao_Tome
    {   64, 4252,      14,  8, }, // America/Asuncion
    {   64,  237,      22,  8, }, // America/Atikokan
    {   64, 7623,      30, 17, }, // America/Bahia_Banderas
    {   64, 3868,      47,  6, }, // America/Cancun
    {   64, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   64, 5723,      66,  7, }, // America/Curacao
    {   64,  481,      73, 13, }, // America/Indiana/Knox
    {   64, 7961,      86, 16, }, // America/Indiana/Marengo
    {   64, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   64, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   64, 8038,     139, 14, }, // America/Indiana/Vevay
    {   64, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   64, 8086,     171, 16, }, // America/Indiana/Winamac
    {   64, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   64, 5932,     207, 22, }, // America/Lower_Princes
    {   64, 8220,     229,  6, }, // America/Merida
    {   64, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   64, 8329,     255, 20, }, // America/North_Dakota/Center
    {   64, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   64,  393,     298,  4, }, // America/Nuuk
    {   64, 6899,     302, 16, }, // America/Scoresbysund
    {   64, 8508,     318, 14, }, // America/St_Barthelemy
    {   64, 2061,     332, 10, }, // America/St_Johns
    {   64, 5863,     342,  9, }, // America/St_Kitts
    {   64, 5880,     351,  9, }, // America/St_Lucia
    {   64,  855,     360, 10, }, // America/St_Thomas
    {   64, 5913,     370, 11, }, // America/St_Vincent
    {   64, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   64, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   64, 1185,     413,  9, }, // Asia/Kathmandu
    {   64,  992,     422,  7, }, // Asia/Kolkata
    {   64, 1211,     429,  5, }, // Asia/Macau
    {   64, 1235,     442,  6, }, // Asia/Yangon
    {   64, 1395,     448,  5, }, // Atlantic/Faroe
    {   64, 6821,     453, 10, }, // Atlantic/St_Helena
    {   64, 2527,     463,  4, }, // Europe/Kyiv
    {   64, 7211,     467,  7, }, // Indian/Reunion
    {   64, 3157,     474,  5, }, // Pacific/Chuuk
    {   64, 3096,     479,  7, }, // Pacific/Pohnpei
    {   65,   14,       0,  6, }, // Africa/Asmara Blin/Ethiopic/Eritrea
    {   65, 4435,       6,  8, }, // Africa/Sao_Tome
    {   65, 4252,      14,  8, }, // America/Asuncion
    {   65,  237,      22,  8, }, // America/Atikokan
    {   65, 7623,      30, 17, }, // America/Bahia_Banderas
    {   65, 3868,      47,  6, }, // America/Cancun
    {   65, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   65, 5723,      66,  7, }, // America/Curacao
    {   65,  481,      73, 13, }, // America/Indiana/Knox
    {   65, 7961,      86, 16, }, // America/Indiana/Marengo
    {   65, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   65, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   65, 8038,     139, 14, }, // America/Indiana/Vevay
    {   65, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   65, 8086,     171, 16, }, // America/Indiana/Winamac
    {   65, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   65, 5932,     207, 22, }, // America/Lower_Princes
    {   65, 8220,     229,  6, }, // America/Merida
    {   65, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   65, 8329,     255, 20, }, // America/North_Dakota/Center
    {   65, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   65,  393,     298,  4, }, // America/Nuuk
    {   65, 6899,     302, 16, }, // America/Scoresbysund
    {   65, 8508,     318, 14, }, // America/St_Barthelemy
    {   65, 2061,     332, 10, }, // America/St_Johns
    {   65, 5863,     342,  9, }, // America/St_Kitts
    {   65, 5880,     351,  9, }, // America/St_Lucia
    {   65,  855,     360, 10, }, // America/St_Thomas
    {   65, 5913,     370, 11, }, // America/St_Vincent
    {   65, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   65, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   65, 1185,     413,  9, }, // Asia/Kathmandu
    {   65,  992,     422,  7, }, // Asia/Kolkata
    {   65, 1211,     429,  5, }, // Asia/Macau
    {   65, 1235,     442,  6, }, // Asia/Yangon
    {   65, 1395,     448,  5, }, // Atlantic/Faroe
    {   65, 6821,     453, 10, }, // Atlantic/St_Helena
    {   65, 2527,     463,  4, }, // Europe/Kyiv
    {   65, 7211,     467,  7, }, // Indian/Reunion
    {   65, 3157,     474,  5, }, // Pacific/Chuuk
    {   65, 3096,     479,  7, }, // Pacific/Pohnpei
    {   66, 6788,   29832,  7, }, // Africa/Abidjan Bodo/Devanagari/India
    {   66, 6760,   29839,  5, }, // Africa/Accra
    {   66, 5059,   29844, 12, }, // Africa/Addis_Ababa
    {   66, 7420,   29856,  9, }, // Africa/Algiers
    {   66,   14,   29865,  7, }, // Africa/Asmara
    {   66,   44,   29872,  6, }, // Africa/Bamako
    {   66, 5257,   29878,  6, }, // Africa/Bangui
    {   66, 6746,   29884,  6, }, // Africa/Banjul
    {   66, 7435,   29890,  7, }, // Africa/Bissau
    {   66, 4957,   29897, 10, }, // Africa/Blantyre
    {   66, 5287,   29907, 11, }, // Africa/Brazzaville
    {   66, 4922,   29918, 10, }, // Africa/Bujumbura
    {   66, 2320,   29928,  5, }, // Africa/Cairo
    {   66, 4117,   29933, 11, }, // Africa/Casablanca
    {   66, 7449,   29944,  5, }, // Africa/Ceuta
    {   66, 6773,   29949,  7, }, // Africa/Conakry
    {   66, 6840,   29956,  5, }, // Africa/Dakar
    {   66, 5130,   29961, 11, }, // Africa/Dar_es_Salaam
    {   66, 5043,   29972,  6, }, // Africa/Djibouti
    {   66, 5243,   29978,  5, }, // Africa/Douala
    {   66, 5166,   29983,  7, }, // Africa/El_Aaiun
    {   66, 6853,   29990,  8, }, // Africa/Freetown
    {   66, 4906,   29998,  7, }, // Africa/Gaborone
    {   66, 5015,   30005,  6, }, // Africa/Harare
    {   66, 4479,   30011, 11, }, // Africa/Johannesburg
    {   66, 4499,   30022,  4, }, // Africa/Juba
    {   66, 5151,   30026,  8, }, // Africa/Kampala
    {   66, 4524,   30034,  7, }, // Africa/Khartoum
    {   66, 4973,   30041,  6, }, // Africa/Kigali
    {   66, 5306,   30047,  8, }, // Africa/Kinshasa
    {   66, 4744,   30055,  5, }, // Africa/Lagos
    {   66, 5336,   30060,  9, }, // Africa/Libreville
    {   66, 6869,   30069,  3, }, // Africa/Lome
    {   66, 5211,   30072,  7, }, // Africa/Luanda
    {   66, 4939,   30079, 10, }, // Africa/Lubumbashi
    {   66, 5001,   30089,  6, }, // Africa/Lusaka
    {   66, 5322,   30095,  6, }, // Africa/Malabo
    {   66, 4987,   30101,  6, }, // Africa/Maputo
    {   66, 5197,   30107,  6, }, // Africa/Maseru
    {   66, 5182,   30113,  8, }, // Africa/Mbabane
    {   66, 5113,   30121,  7, }, // Africa/Mogadishu
    {   66, 7034,   30128,  9, }, // Africa/Monrovia
    {   66, 3853,   30137,  7, }, // Africa/Nairobi
    {   66, 5271,   30144,  8, }, // Africa/Ndjamena
    {   66, 5354,   30152,  5, }, // Africa/Niamey
    {   66, 6803,   30157,  8, }, // Africa/Nouakchott
    {   66, 6727,   30165,  7, }, // Africa/Ouagadougou
    {   66, 5225,   30172,  9, }, // Africa/Porto-Novo
    {   66, 4435,   30181,  7, }, // Africa/Sao_Tome
    {   66, 2866,   30188,  8, }, // Africa/Tripoli
    {   66, 6554,   30196,  7, }, // Africa/Tunis
    {   66, 4152,   30203,  8, }, // Africa/Windhoek
    {   66,  132,   30211,  4, }, // America/Adak
    {   66, 3346,   30215,  6, }, // America/Anchorage
    {   66, 5607,   30221,  8, }, // America/Anguilla
    {   66, 5624,   30229,  9, }, // America/Antigua
    {   66, 4554,   30238,  9, }, // America/Araguaina
    {   66,  166,   30247, 11, }, // America/Argentina/Buenos_Aires
    {   66,   91,   30258, 10, }, // America/Argentina/Catamarca
    {   66,  270,   30268,  6, }, // America/Argentina/Cordoba
    {   66,  441,   30274,  5, }, // America/Argentina/Jujuy
    {   66, 7462,   30279,  8, }, // America/Argentina/La_Rioja
    {   66,  565,   30287,  8, }, // America/Argentina/Mendoza
    {   66, 7489,   30295, 10, }, // America/Argentina/Rio_Gallegos
    {   66, 7520,   30305,  6, }, // America/Argentina/Salta
    {   66, 7544,   30311,  9, }, // America/Argentina/San_Juan
    {   66, 5580,   30320,  8, }, // America/Argentina/San_Luis
    {   66, 7571,   30328,  7, }, // America/Argentina/Tucuman
    {   66, 7597,   30335,  8, }, // America/Argentina/Ushuaia
    {   66, 5640,   30343,  5, }, // America/Aruba
    {   66, 4252,   30348,  8, }, // America/Asuncion
    {   66,  237,   30356,  7, }, // America/Atikokan
    {   66, 3679,   30363,  6, }, // America/Bahia
    {   66, 7623,   30369, 16, }, // America/Bahia_Banderas
    {   66, 5654,   30385,  9, }, // America/Barbados
    {   66, 7646,   30394,  5, }, // America/Belem
    {   66, 5407,   30399,  5, }, // America/Belize
    {   66, 7660,   30404, 14, }, // America/Blanc-Sablon
    {   66, 7681,   30418,  9, }, // America/Boa_Vista
    {   66, 4361,   30427,  5, }, // America/Bogota
    {   66, 7699,   30432,  4, }, // America/Boise
    {   66, 7713,   30436, 15, }, // America/Cambridge_Bay
    {   66, 7735,   30451, 13, }, // America/Campo_Grande
    {   66, 3868,   30464,  6, }, // America/Cancun
    {   66, 4694,   30470,  7, }, // America/Caracas
    {   66, 4345,   30477,  5, }, // America/Cayenne
    {   66, 5496,   30482,  5, }, // America/Cayman
    {   66, 2260,   30487,  6, }, // America/Chicago
    {   66, 7756,   30493,  7, }, // America/Chihuahua
    {   66, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   66, 5422,   30500, 10, }, // America/Costa_Rica
    {   66, 7796,   30510,  8, }, // America/Creston
    {   66, 3791,   30518,  6, }, // America/Cuiaba
    {   66, 5723,   30524,  7, }, // America/Curacao
    {   66, 7812,   30531, 13, }, // America/Danmarkshavn
    {   66, 7833,   30544,  6, }, // America/Dawson
    {   66, 7848,   30550, 12, }, // America/Dawson_Creek
    {   66,  805,   30562,  6, }, // America/Denver
    {   66, 3465,   30568,  8, }, // America/Detroit
    {   66, 5739,   30576,  7, }, // America/Dominica
    {   66,  893,   30583,  7, }, // America/Edmonton
    {   66, 7869,   30590,  8, }, // America/Eirunepe
    {   66, 5441,   30598, 11, }, // America/El_Salvador
    {   66, 7886,   30609, 10, }, // America/Fort_Nelson
    {   66, 7906,   30619,  9, }, // America/Fortaleza
    {   66, 7924,   30628,  9, }, // America/Glace_Bay
    {   66, 6881,   30637,  7, }, // America/Goose_Bay
    {   66, 4612,   30644, 13, }, // America/Grand_Turk
    {   66, 5770,   30657,  8, }, // America/Grenada
    {   66, 5786,   30665,  9, }, // America/Guadeloupe
    {   66, 3760,   30674, 10, }, // America/Guatemala
    {   66, 4841,   30684,  8, }, // America/Guayaquil
    {   66, 6932,   30692,  6, }, // America/Guyana
    {   66, 1939,   30698,  9, }, // America/Halifax
    {   66, 2281,   30707,  6, }, // America/Havana
    {   66, 7942,   30713, 12, }, // America/Hermosillo
    {   66,  348,   30725, 13, }, // America/Indiana/Indianapolis
    {   66,  481,   30738, 15, }, // America/Indiana/Knox
    {   66, 7961,   30753, 17, }, // America/Indiana/Marengo
    {   66, 7985,   30770, 23, }, // America/Indiana/Petersburg
    {   66, 8012,   30793, 20, }, // America/Indiana/Tell_City
    {   66, 8038,   30813, 15, }, // America/Indiana/Vevay
    {   66, 8060,   30828, 21, }, // America/Indiana/Vincennes
    {   66, 8086,   30849, 18, }, // America/Indiana/Winamac
    {   66, 8110,   30867,  6, }, // America/Inuvik
    {   66,  660,   30873,  7, }, // America/Iqaluit
    {   66, 2799,   30880,  7, }, // America/Jamaica
    {   66, 5380,   30887,  4, }, // America/Juneau
    {   66,  521,   30891,  6, }, // America/Kentucky/Louisville
    {   66, 8125,   30897, 19, }, // America/Kentucky/Monticello
    {   66, 5704,   30916, 10, }, // America/Kralendijk
    {   66, 4376,   30926,  6, }, // America/La_Paz
    {   66, 7169,   30932,  4, }, // America/Lima
    {   66, 3239,   30936, 11, }, // America/Los_Angeles
    {   66, 5932,   30947, 24, }, // America/Lower_Princes
    {   66, 8153,   30971,  6, }, // America/Maceio
    {   66, 8168,   30977,  6, }, // America/Managua
    {   66, 1908,   30983,  5, }, // America/Manaus
    {   66, 5897,   30988,  6, }, // America/Marigot
    {   66, 5805,   30994,  9, }, // America/Martinique
    {   66, 8184,   31003,  7, }, // America/Matamoros
    {   66, 2917,   31010,  8, }, // America/Mazatlan
    {   66, 8202,   31018,  8, }, // America/Menominee
    {   66, 8220,   31026,  6, }, // America/Merida
    {   66, 8235,   31032, 12, }, // America/Metlakatla
    {   66, 2949,   31044, 13, }, // America/Mexico_City
    {   66, 4391,   31057,  9, }, // America/Miquelon
    {   66, 8254,   31066,  6, }, // America/Moncton
    {   66, 8270,   31072,  9, }, // America/Monterrey
    {   66, 4098,   31081, 10, }, // America/Montevideo
    {   66, 5824,   31091,  9, }, // America/Montserrat
    {   66, 5481,   31100,  7, }, // America/Nassau
    {   66, 2379,   31107,  8, }, // America/New_York
    {   66, 8288,   31020,  3, }, // America/Nome
    {   66, 1850,   31115,  7, }, // America/Noronha
    {   66, 8301,   31122, 19, }, // America/North_Dakota/Beulah
    {   66, 8329,   31141, 20, }, // America/North_Dakota/Center
    {   66, 8357,   31161, 22, }, // America/North_Dakota/New_Salem
    {   66,  393,   31183,  3, }, // America/Nuuk
    {   66, 8388,   31186,  7, }, // America/Ojinaga
    {   66, 2356,   31193,  6, }, // America/Panama
    {   66, 6173,   31199, 10, }, // America/Paramaribo
    {   66, 2973,   31209,  7, }, // America/Phoenix
    {   66, 3945,   31216, 15, }, // America/Port-au-Prince
    {   66, 5954,   31231, 13, }, // America/Port_of_Spain
    {   66, 8404,   31244, 12, }, // America/Porto_Velho
    {   66, 5843,   31256, 12, }, // America/Puerto_Rico
    {   66, 4030,   31268, 13, }, // America/Punta_Arenas
    {   66, 8424,   31281, 12, }, // America/Rankin_Inlet
    {   66, 8445,   31293,  6, }, // America/Recife
    {   66, 1995,   31299,  6, }, // America/Regina
    {   66, 8460,   31305,  8, }, // America/Resolute
    {   66,  695,   31313, 12, }, // America/Rio_Branco
    {   66, 8477,   31325,  9, }, // America/Santarem
    {   66, 2201,   31334,  9, }, // America/Santiago
    {   66, 6111,   31343, 12, }, // America/Santo_Domingo
    {   66, 1878,   31355,  9, }, // America/Sao_Paulo
    {   66, 6899,   31364, 14, }, // America/Scoresbysund
    {   66, 8494,   31378,  6, }, // America/Sitka
    {   66, 8508,   31384, 16, }, // America/St_Barthelemy
    {   66, 2061,   31400, 10, }, // America/St_Johns
    {   66, 5863,   31410, 11, }, // America/St_Kitts
    {   66, 5880,   31421, 12, }, // America/St_Lucia
    {   66,  855,   31433, 10, }, // America/St_Thomas
    {   66, 5913,   31443, 15, }, // America/St_Vincent
    {   66, 8530,   31458, 15, }, // America/Swift_Current
    {   66, 5461,   31473, 12, }, // America/Tegucigalpa
    {   66, 5756,   31485,  5, }, // America/Thule
    {   66,  313,   31490,  7, }, // America/Tijuana
    {   66,  608,   31497,  6, }, // America/Toronto
    {   66, 5688,   31503,  7, }, // America/Tortola
    {   66, 2093,   31510,  7, }, // America/Vancouver
    {   66, 2144,   31517, 10, }, // America/Whitehorse
    {   66,  734,   31527,  7, }, // America/Winnipeg
    {   66, 8552,   31534,  7, }, // America/Yakutat
    {   66, 6001,   31541,  4, }, // Antarctica/Casey
    {   66, 6094,   31545,  5, }, // Antarctica/Davis
    {   66, 6133,   31550, 15, }, // Antarctica/DumontDUrville
    {   66, 8568,   31565,  8, }, // Antarctica/Macquarie
    {   66, 7066,   31573,  4, }, // Antarctica/Mawson
    {   66, 7113,   31577,  9, }, // Antarctica/McMurdo
    {   66, 8589,   31586,  6, }, // Antarctica/Palmer
    {   66, 7226,   31592,  5, }, // Antarctica/Rothera
    {   66, 7295,   31597,  5, }, // Antarctica/Syowa
    {   66, 8607,   31602,  5, }, // Antarctica/Troll
    {   66, 7374,   31607,  6, }, // Antarctica/Vostok
    {   66, 1429,   31613, 10, }, // Arctic/Longyearbyen
    {   66, 5570,   31623,  4, }, // Asia/Aden
    {   66, 5395,   31627,  7, }, // Asia/Almaty
    {   66, 3968,   31634,  6, }, // Asia/Amman
    {   66, 5511,   31640,  6, }, // Asia/Anadyr
    {   66, 5523,   31646,  6, }, // Asia/Aqtau
    {   66, 5368,   31652,  7, }, // Asia/Aqtobe
    {   66,  964,   31659,  8, }, // Asia/Ashgabat
    {   66, 8624,   31667,  6, }, // Asia/Atyrau
    {   66, 3607,   31673,  6, }, // Asia/Baghdad
    {   66, 5534,   31679,  6, }, // Asia/Bahrain
    {   66, 3653,   30522,  4, }, // Asia/Baku
    {   66, 4466,   31685,  7, }, // Asia/Bangkok
    {   66, 3571,   31692,  7, }, // Asia/Barnaul
    {   66, 4086,   31699,  5, }, // Asia/Beirut
    {   66, 3778,   31704,  7, }, // Asia/Bishkek
    {   66, 5989,   31711,  6, }, // Asia/Brunei
    {   66, 4601,   31717,  4, }, // Asia/Chita
    {   66, 4511,   31721,  6, }, // Asia/Colombo
    {   66, 4540,   31727,  9, }, // Asia/Damascus
    {   66, 1093,   31736,  4, }, // Asia/Dhaka
    {   66, 6192,   31740,  4, }, // Asia/Dili
    {   66, 3596,   31744,  5, }, // Asia/Dubai
    {   66, 6159,   31749,  8, }, // Asia/Dushanbe
    {   66, 8636,   31757, 10, }, // Asia/Famagusta
    {   66, 8651,   31767,  4, }, // Asia/Gaza
    {   66, 4795,   31771,  7, }, // Asia/Hebron
    {   66, 1259,   31778,  6, }, // Asia/Ho_Chi_Minh
    {   66, 2704,   31784,  4, }, // Asia/Hong_Kong
    {   66, 4771,   31788,  5, }, // Asia/Hovd
    {   66, 4184,   31793, 10, }, // Asia/Irkutsk
    {   66, 7006,   31803,  8, }, // Asia/Jakarta
    {   66, 6992,   31811,  7, }, // Asia/Jayapura
    {   66, 1290,   31818,  9, }, // Asia/Jerusalem
    {   66, 3560,   31827,  5, }, // Asia/Kabul
    {   66, 4316,   31832,  8, }, // Asia/Kamchatka
    {   66, 4239,   31840,  5, }, // Asia/Karachi
    {   66, 1185,   31845,  9, }, // Asia/Kathmandu
    {   66, 8661,   31854,  8, }, // Asia/Khandyga
    {   66,  992,   31862,  6, }, // Asia/Kolkata
    {   66, 4197,   31868, 15, }, // Asia/Krasnoyarsk
    {   66, 4859,   31883, 13, }, // Asia/Kuala_Lumpur
    {   66, 5976,   31896,  6, }, // Asia/Kuching
    {   66, 5547,   31902,  5, }, // Asia/Kuwait
    {   66, 1211,   31907,  5, }, // Asia/Macau
    {   66, 4017,   31912,  7, }, // Asia/Magadan
    {   66, 1349,   31919,  9, }, // Asia/Makassar
    {   66, 7182,   31928,  5, }, // Asia/Manila
    {   66, 6920,   31933,  7, }, // Asia/Muscat
    {   66, 2554,   31940,  8, }, // Asia/Nicosia
    {   66, 8675,   31948, 14, }, // Asia/Novokuznetsk
    {   66, 4135,   31962, 12, }, // Asia/Novosibirsk
    {   66, 4229,   31974,  6, }, // Asia/Omsk
    {   66, 7145,   31980,  4, }, // Asia/Oral
    {   66, 6961,   31984,  7, }, // Asia/Phnom_Penh
    {   66, 8693,   31991,  9, }, // Asia/Pontianak
    {   66, 4214,   32000,  7, }, // Asia/Pyongyang
    {   66, 5559,   32007,  5, }, // Asia/Qatar
    {   66, 8708,   32012,  7, }, // Asia/Qostanay
    {   66, 4269,   32019,  9, }, // Asia/Qyzylorda
    {   66, 3584,   32028,  5, }, // Asia/Riyadh
    {   66, 4408,   32033,  7, }, // Asia/Sakhalin
    {   66, 7245,   32040,  8, }, // Asia/Samarkand
    {   66, 3279,   32048,  4, }, // Asia/Seoul
    {   66, 1053,   32052,  6, }, // Asia/Shanghai
    {   66, 3300,   32058,  8, }, // Asia/Singapore
    {   66, 4297,   32066, 16, }, // Asia/Srednekolymsk
    {   66, 3263,   32082,  6, }, // Asia/Taipei
    {   66, 4781,   32088,  8, }, // Asia/Tashkent
    {   66, 3915,   32096,  8, }, // Asia/Tbilisi
    {   66, 2772,   32104,  6, }, // Asia/Tehran
    {   66, 1317,   32110,  6, }, // Asia/Thimphu
    {   66, 2821,   32116,  6, }, // Asia/Tokyo
    {   66, 4572,   32122,  6, }, // Asia/Tomsk
    {   66, 1021,   32128,  9, }, // Asia/Ulaanbaatar
    {   66, 1159,   32137,  6, }, // Asia/Urumqi
    {   66, 8722,   32143,  9, }, // Asia/Ust-Nera
    {   66, 6977,   32152,  9, }, // Asia/Vientiane
    {   66, 4710,   32161, 12, }, // Asia/Vladivostok
    {   66, 4828,   32173,  9, }, // Asia/Yakutsk
    {   66, 1235,   32182,  5, }, // Asia/Yangon
    {   66, 3883,   32187, 15, }, // Asia/Yekaterinburg
    {   66, 3747,   32202,  7, }, // Asia/Yerevan
    {   66, 3663,   32209,  6, }, // Atlantic/Azores
    {   66, 5671,   32215,  8, }, // Atlantic/Bermuda
    {   66, 6628,   32223,  6, }, // Atlantic/Canary
    {   66, 3727,   32229, 10, }, // Atlantic/Cape_Verde
    {   66, 1395,   32239,  4, }, // Atlantic/Faroe
    {   66, 8736,   32243,  6, }, // Atlantic/Madeira
    {   66, 2748,   32249,  8, }, // Atlantic/Reykjavik
    {   66, 7272,   32257, 12, }, // Atlantic/South_Georgia
    {   66, 6821,   32269, 12, }, // Atlantic/St_Helena
    {   66, 6644,   32281,  7, }, // Atlantic/Stanley
    {   66, 1670,   32288,  6, }, // Australia/Adelaide
    {   66, 1635,   32294,  8, }, // Australia/Brisbane
    {   66, 1799,   32302, 11, }, // Australia/Broken_Hill
    {   66, 1583,   32313,  7, }, // Australia/Darwin
    {   66, 3637,   32320,  6, }, // Australia/Eucla
    {   66, 1516,   32326,  6, }, // Australia/Hobart
    {   66, 8753,   32332,  8, }, // Australia/Lindeman
    {   66, 1547,   32340,  9, }, // Australia/Lord_Howe
    {   66, 1727,   32349,  8, }, // Australia/Melbourne
    {   66, 1762,   32357,  5, }, // Australia/Perth
    {   66, 1463,   32362,  5, }, // Australia/Sydney
    {   66, 6441,   32367, 10, }, // Europe/Amsterdam
    {   66, 6216,   32377,  7, }, // Europe/Andorra
    {   66, 3620,   32384, 10, }, // Europe/Astrakhan
    {   66, 2300,   32394,  6, }, // Europe/Athens
    {   66, 6336,   32400,  8, }, // Europe/Belgrade
    {   66, 4757,   32408,  7, }, // Europe/Berlin
    {   66, 6488,   32415, 12, }, // Europe/Bratislava
    {   66, 2167,   32427, 11, }, // Europe/Brussels
    {   66, 3928,   32438,  9, }, // Europe/Bucharest
    {   66, 3806,   32447,  9, }, // Europe/Budapest
    {   66, 8772,   32456,  8, }, // Europe/Busingen
    {   66, 2583,   32464,  6, }, // Europe/Chisinau
    {   66, 6289,   32470,  9, }, // Europe/Copenhagen
    {   66, 2338,   32479,  6, }, // Europe/Dublin
    {   66, 6307,   32485, 11, }, // Europe/Gibraltar
    {   66, 8788,   32496,  7, }, // Europe/Guernsey
    {   66, 6612,   32503,  9, }, // Europe/Helsinki
    {   66, 8804,   32512, 11, }, // Europe/Isle_of_Man
    {   66, 1130,   32523,  9, }, // Europe/Istanbul
    {   66, 8823,   32532,  6, }, // Europe/Jersey
    {   66, 3979,   32538, 12, }, // Europe/Kaliningrad
    {   66, 8929,   32550,  5, }, // Europe/Kirov
    {   66, 2527,   32555,  3, }, // Europe/Kyiv
    {   66, 3213,   32558,  6, }, // Europe/Lisbon
    {   66, 6506,   32564, 10, }, // Europe/Ljubljana
    {   66, 2501,   32574,  5, }, // Europe/London
    {   66, 6365,   32579, 13, }, // Europe/Luxembourg
    {   66, 4892,   32592,  7, }, // Europe/Madrid
    {   66, 6397,   32599,  6, }, // Europe/Malta
    {   66, 6582,   32605,  7, }, // Europe/Mariehamn
    {   66, 3693,   32612,  7, }, // Europe/Minsk
    {   66, 6410,   32619,  5, }, // Europe/Monaco
    {   66, 3537,   32624,  5, }, // Europe/Moscow
    {   66, 6458,   32629,  5, }, // Europe/Oslo
    {   66, 4284,   32634,  5, }, // Europe/Paris
    {   66, 6424,   32639,  8, }, // Europe/Podgorica
    {   66, 6275,   32647,  5, }, // Europe/Prague
    {   66, 8837,   32652,  4, }, // Europe/Riga
    {   66, 6324,   32656,  3, }, // Europe/Rome
    {   66, 4331,   32659,  6, }, // Europe/Samara
    {   66, 6470,   32665, 10, }, // Europe/San_Marino
    {   66, 6245,   32675,  8, }, // Europe/Sarajevo
    {   66, 4451,   32683,  7, }, // Europe/Saratov
    {   66, 8849,   32690, 11, }, // Europe/Simferopol
    {   66, 6383,   32701,  8, }, // Europe/Skopje
    {   66, 6599,   32709,  5, }, // Europe/Sofia
    {   66, 6523,   32714,  8, }, // Europe/Stockholm
    {   66, 8867,   32722,  5, }, // Europe/Tallinn
    {   66, 6202,   32727,  5, }, // Europe/Tirane
    {   66, 8882,   32732, 12, }, // Europe/Ulyanovsk
    {   66, 6352,   32744,  5, }, // Europe/Vaduz
    {   66, 6567,   32749,  7, }, // Europe/Vatican
    {   66, 6231,   32756,  8, }, // Europe/Vienna
    {   66, 8899,   32764,  8, }, // Europe/Vilnius
    {   66, 4727,   32772, 10, }, // Europe/Volgograd
    {   66, 3190,   32782,  5, }, // Europe/Warsaw
    {   66, 6261,   32787,  7, }, // Europe/Zagreb
    {   66, 6540,   32794,  5, }, // Europe/Zurich
    {   66, 5078,   32799, 13, }, // Indian/Antananarivo
    {   66, 6947,   32812,  5, }, // Indian/Chagos
    {   66, 6046,   32817, 16, }, // Indian/Christmas
    {   66, 6063,   32833,  4, }, // Indian/Cocos
    {   66, 5029,   32837,  5, }, // Indian/Comoro
    {   66, 6661,   32842,  9, }, // Indian/Kerguelen
    {   66, 7260,   32851,  4, }, // Indian/Mahe
    {   66, 7050,   32855,  8, }, // Indian/Maldives
    {   66, 4069,   32863,  8, }, // Indian/Mauritius
    {   66, 5098,   32871,  8, }, // Indian/Mayotte
    {   66, 7211,   32879,  9, }, // Indian/Reunion
    {   66, 4422,   32888,  5, }, // Pacific/Apia
    {   66,  932,   32893,  7, }, // Pacific/Auckland
    {   66, 3706,   32900,  8, }, // Pacific/Bougainville
    {   66, 3015,   32908,  5, }, // Pacific/Chatham
    {   66, 3157,   32913,  3, }, // Pacific/Chuuk
    {   66, 2237,   32916,  9, }, // Pacific/Easter
    {   66, 7360,   32925,  5, }, // Pacific/Efate
    {   66, 7327,   32930,  7, }, // Pacific/Fakaofo
    {   66, 3902,   32937,  4, }, // Pacific/Fiji
    {   66, 7343,   32941,  8, }, // Pacific/Funafuti
    {   66, 6678,   32949,  9, }, // Pacific/Galapagos
    {   66, 6696,   32958,  9, }, // Pacific/Gambier
    {   66, 3822,   32967, 12, }, // Pacific/Guadalcanal
    {   66, 6018,   32979,  5, }, // Pacific/Guam
    {   66, 2723,   32984,  6, }, // Pacific/Honolulu
    {   66, 3049,   32990, 10, }, // Pacific/Kanton
    {   66, 3998,   33000, 10, }, // Pacific/Kiritimati
    {   66, 7019,   33010,  5, }, // Pacific/Kosrae
    {   66, 2842,   33015,  9, }, // Pacific/Kwajalein
    {   66, 4877,   33024,  6, }, // Pacific/Majuro
    {   66, 4051,   33030,  9, }, // Pacific/Marquesas
    {   66, 8914,   33039,  5, }, // Pacific/Midway
    {   66, 7084,   33044,  5, }, // Pacific/Nauru
    {   66, 7132,   33049,  4, }, // Pacific/Niue
    {   66, 4168,   33053,  5, }, // Pacific/Norfolk
    {   66, 7098,   33058,  6, }, // Pacific/Noumea
    {   66, 3126,   33064,  9, }, // Pacific/Pago_Pago
    {   66, 7155,   33073,  5, }, // Pacific/Palau
    {   66, 7194,   33078,  8, }, // Pacific/Pitcairn
    {   66, 3096,   33086,  5, }, // Pacific/Pohnpei
    {   66, 4807,   33091, 11, }, // Pacific/Port_Moresby
    {   66, 6076,   33102,  8, }, // Pacific/Rarotonga
    {   66, 6031,   33110,  6, }, // Pacific/Saipan
    {   66, 7312,   33116,  6, }, // Pacific/Tahiti
    {   66, 6712,   33122,  6, }, // Pacific/Tarawa
    {   66, 4583,   33128,  8, }, // Pacific/Tongatapu
    {   66, 7392,   33136,  3, }, // Pacific/Wake
    {   66, 7405,   33139,  5, }, // Pacific/Wallis
    {   67,   14,       0,  6, }, // Africa/Asmara Bosnian/Latin/Bosnia and Herzegovina
    {   67, 2320,    4572,  5, }, // Africa/Cairo
    {   67, 4117,   33144, 10, }, // Africa/Casablanca
    {   67, 5043,   33154,  7, }, // Africa/Djibouti
    {   67, 4524,    6305,  6, }, // Africa/Khartoum
    {   67, 5113,   33161,  7, }, // Africa/Mogadishu
    {   67, 4435,       6,  8, }, // Africa/Sao_Tome
    {   67, 5607,   33168,  7, }, // America/Anguilla
    {   67, 5624,   33175,  7, }, // America/Antigua
    {   67, 4252,      14,  8, }, // America/Asuncion
    {   67,  237,      22,  8, }, // America/Atikokan
    {   67, 7623,    3825, 14, }, // America/Bahia_Banderas
    {   67, 3868,   33182,  6, }, // America/Cancun
    {   67, 5496,    6576,  6, }, // America/Cayman
    {   67, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   67, 5422,   33188,  9, }, // America/Costa_Rica
    {   67, 5723,    6631,  7, }, // America/Curacao
    {   67, 5739,   21548,  8, }, // America/Dominica
    {   67, 5441,    6688,  8, }, // America/El_Salvador
    {   67, 5786,   33197,  9, }, // America/Guadeloupe
    {   67, 3760,   33206,  9, }, // America/Guatemala
    {   67,  481,      73, 13, }, // America/Indiana/Knox
    {   67, 7961,      86, 16, }, // America/Indiana/Marengo
    {   67, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   67, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   67, 8038,     139, 14, }, // America/Indiana/Vevay
    {   67, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   67, 8086,     171, 16, }, // America/Indiana/Winamac
    {   67, 2799,   33215,  7, }, // America/Jamaica
    {   67, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   67, 5932,     207, 22, }, // America/Lower_Princes
    {   67, 8220,    7008,  6, }, // America/Merida
    {   67, 8301,   33222, 23, }, // America/North_Dakota/Beulah
    {   67, 8329,   33245, 23, }, // America/North_Dakota/Center
    {   67, 8357,   33268, 26, }, // America/North_Dakota/New_Salem
    {   67,  393,     298,  4, }, // America/Nuuk
    {   67, 5843,   33294,  9, }, // America/Puerto_Rico
    {   67, 6899,     302, 16, }, // America/Scoresbysund
    {   67, 8508,     318, 14, }, // America/St_Barthelemy
    {   67, 2061,     332, 10, }, // America/St_Johns
    {   67, 5863,     342,  9, }, // America/St_Kitts
    {   67, 5880,     351,  9, }, // America/St_Lucia
    {   67,  855,     360, 10, }, // America/St_Thomas
    {   67, 5913,     370, 11, }, // America/St_Vincent
    {   67, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   67, 5395,    7476,  6, }, // Asia/Almaty
    {   67, 3968,    5487,  4, }, // Asia/Amman
    {   67, 5511,    7482,  6, }, // Asia/Anadyr
    {   67, 5523,    5491,  5, }, // Asia/Aqtau
    {   67, 5368,   33303,  7, }, // Asia/Aqtobe
    {   67,  964,   33310,  7, }, // Asia/Ashgabat
    {   67, 8624,   33317,  6, }, // Asia/Atyrau
    {   67, 3607,    4044,  6, }, // Asia/Baghdad
    {   67, 5534,    4050,  7, }, // Asia/Bahrain
    {   67, 4086,    7507,  6, }, // Asia/Beirut
    {   67, 3778,   33323,  6, }, // Asia/Bishkek
    {   67, 5989,   33329,  7, }, // Asia/Brunei
    {   67, 4511,    5519,  7, }, // Asia/Colombo
    {   67, 4540,    4083,  6, }, // Asia/Damascus
    {   67, 1093,    4599,  4, }, // Asia/Dhaka
    {   67, 6159,   33336,  7, }, // Asia/Dushanbe
    {   67, 1259,   33343,  9, }, // Asia/Ho_Chi_Minh
    {   67, 7006,   33352,  8, }, // Asia/Jakarta
    {   67, 6992,   33360,  9, }, // Asia/Jayapura
    {   67, 1290,   33369,  9, }, // Asia/Jerusalem
    {   67, 4316,   33378,  8, }, // Asia/Kamchatka
    {   67, 4239,   33386,  6, }, // Asia/Karachi
    {   67, 1185,    5593,  8, }, // Asia/Kathmandu
    {   67, 8661,   33392,  7, }, // Asia/Khandyga
    {   67,  992,     422,  7, }, // Asia/Kolkata
    {   67, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {   67, 5976,   33399,  6, }, // Asia/Kuching
    {   67, 5547,    7607,  6, }, // Asia/Kuwait
    {   67, 1211,   33405,  5, }, // Asia/Macau
    {   67, 1349,    7618,  7, }, // Asia/Makassar
    {   67, 6920,    4208,  6, }, // Asia/Muscat
    {   67, 2554,   33410,  8, }, // Asia/Nicosia
    {   67, 8675,   33418, 12, }, // Asia/Novokuznetsk
    {   67, 6961,   22386,  8, }, // Asia/Phnom_Penh
    {   67, 4214,   33430,  9, }, // Asia/Pyongyang
    {   67, 5559,    4214,  5, }, // Asia/Qatar
    {   67, 8708,    7652,  8, }, // Asia/Qostanay
    {   67, 4269,    7660,  9, }, // Asia/Qyzylorda
    {   67, 3584,   33439,  5, }, // Asia/Riyadh
    {   67, 4408,   33444,  7, }, // Asia/Sakhalin
    {   67, 3279,    7680,  4, }, // Asia/Seoul
    {   67, 1053,   33451,  6, }, // Asia/Shanghai
    {   67, 3300,   20782,  8, }, // Asia/Singapore
    {   67, 3263,    7712,  6, }, // Asia/Taipei
    {   67, 4781,   33457,  7, }, // Asia/Tashkent
    {   67, 2772,    7724,  7, }, // Asia/Tehran
    {   67, 2821,    4238,  5, }, // Asia/Tokyo
    {   67, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {   67, 1159,   33464,  6, }, // Asia/Urumqi
    {   67, 6977,   33470, 10, }, // Asia/Vientiane
    {   67, 4828,    7755,  7, }, // Asia/Yakutsk
    {   67, 1235,    7762,  6, }, // Asia/Yangon
    {   67, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {   67, 3747,    7780,  7, }, // Asia/Yerevan
    {   67, 3663,   33480,  5, }, // Atlantic/Azores
    {   67, 6628,    4280,  6, }, // Atlantic/Canary
    {   67, 3727,   22522, 10, }, // Atlantic/Cape_Verde
    {   67, 1395,     448,  5, }, // Atlantic/Faroe
    {   67, 2748,   33485,  9, }, // Atlantic/Reykjavik
    {   67, 6821,   33494, 12, }, // Atlantic/St_Helena
    {   67, 1547,   22595,  8, }, // Australia/Lord_Howe
    {   67, 1727,    7903,  7, }, // Australia/Melbourne
    {   67, 1762,   22603,  4, }, // Australia/Perth
    {   67, 1463,    7915,  6, }, // Australia/Sydney
    {   67, 6216,   33506,  6, }, // Europe/Andorra
    {   67, 3620,   33512,  8, }, // Europe/Astrakhan
    {   67, 2300,   33520,  5, }, // Europe/Athens
    {   67, 6336,    7942,  7, }, // Europe/Belgrade
    {   67, 2167,   33525,  6, }, // Europe/Brussels
    {   67, 3928,   33531,  8, }, // Europe/Bucharest
    {   67, 3806,   33539, 10, }, // Europe/Budapest
    {   67, 2583,   33549,  8, }, // Europe/Chisinau
    {   67, 6289,    4357, 10, }, // Europe/Copenhagen
    {   67, 2338,   33557,  6, }, // Europe/Dublin
    {   67, 8788,   33563,  6, }, // Europe/Guernsey
    {   67, 8804,   33569, 10, }, // Europe/Isle_of_Man
    {   67, 3979,   33579, 12, }, // Europe/Kaliningrad
    {   67, 2527,   33591,  5, }, // Europe/Kyiv
    {   67, 3213,   33596,  7, }, // Europe/Lisbon
    {   67, 6365,    8054, 10, }, // Europe/Luxembourg
    {   67, 6410,    5926,  6, }, // Europe/Monaco
    {   67, 3537,   22748,  6, }, // Europe/Moscow
    {   67, 4284,   33603,  5, }, // Europe/Paris
    {   67, 6275,    8083,  4, }, // Europe/Prague
    {   67, 6324,   33608,  3, }, // Europe/Rome
    {   67, 8849,   33611, 11, }, // Europe/Simferopol
    {   67, 6383,   33622,  7, }, // Europe/Skopje
    {   67, 6599,   33629,  6, }, // Europe/Sofia
    {   67, 6523,   33635,  8, }, // Europe/Stockholm
    {   67, 8867,    8132,  5, }, // Europe/Tallinn
    {   67, 6202,   21095,  6, }, // Europe/Tirane
    {   67, 6567,    5978,  7, }, // Europe/Vatican
    {   67, 6231,   33643,  3, }, // Europe/Vienna
    {   67, 3190,   33646,  7, }, // Europe/Warsaw
    {   67, 6540,   33653,  5, }, // Europe/Zurich
    {   67, 6046,   33658, 14, }, // Indian/Christmas
    {   67, 6063,   33672, 15, }, // Indian/Cocos
    {   67, 7050,   33687,  7, }, // Indian/Maldives
    {   67, 4069,   33694, 10, }, // Indian/Mauritius
    {   67, 7211,     467,  7, }, // Indian/Reunion
    {   67, 3157,     474,  5, }, // Pacific/Chuuk
    {   67, 3902,   33704,  5, }, // Pacific/Fiji
    {   67, 2723,    4493,  8, }, // Pacific/Honolulu
    {   67, 3049,    4501,  9, }, // Pacific/Kanton
    {   67, 7194,    8304,  7, }, // Pacific/Pitcairn
    {   67, 3096,     479,  7, }, // Pacific/Pohnpei
    {   67, 7405,   33709,  5, }, // Pacific/Wallis
    {   68, 6788,     486,  6, }, // Africa/Abidjan Bosnian/Cyrillic/Bosnia and Herzegovina
    {   68, 6760,   27046,  4, }, // Africa/Accra
    {   68, 5059,   33714, 10, }, // Africa/Addis_Ababa
    {   68, 7420,     508,  5, }, // Africa/Algiers
    {   68,   14,     513,  6, }, // Africa/Asmara
    {   68,   44,     519,  6, }, // Africa/Bamako
    {   68, 5257,   33724,  6, }, // Africa/Bangui
    {   68, 6746,     530,  6, }, // Africa/Banjul
    {   68, 7435,   33730,  5, }, // Africa/Bissau
    {   68, 4957,   33735,  7, }, // Africa/Blantyre
    {   68, 5287,   33742,  8, }, // Africa/Brazzaville
    {   68, 4922,   33750,  9, }, // Africa/Bujumbura
    {   68, 2320,   33759,  5, }, // Africa/Cairo
    {   68, 4117,   33764, 10, }, // Africa/Casablanca
    {   68, 7449,     581,  5, }, // Africa/Ceuta
    {   68, 6773,     586,  7, }, // Africa/Conakry
    {   68, 6840,     593,  5, }, // Africa/Dakar
    {   68, 5130,     598, 12, }, // Africa/Dar_es_Salaam
    {   68, 5043,     610,  6, }, // Africa/Djibouti
    {   68, 5243,     616,  5, }, // Africa/Douala
    {   68, 5166,   33774,  7, }, // Africa/El_Aaiun
    {   68, 6853,     628,  7, }, // Africa/Freetown
    {   68, 4906,     635,  7, }, // Africa/Gaborone
    {   68, 5015,   33781,  6, }, // Africa/Harare
    {   68, 4479,   33787, 11, }, // Africa/Johannesburg
    {   68, 4499,     661,  4, }, // Africa/Juba
    {   68, 5151,     665,  7, }, // Africa/Kampala
    {   68, 4524,   33798,  6, }, // Africa/Khartoum
    {   68, 4973,     678,  6, }, // Africa/Kigali
    {   68, 5306,     684,  7, }, // Africa/Kinshasa
    {   68, 4744,     691,  5, }, // Africa/Lagos
    {   68, 5336,     696,  8, }, // Africa/Libreville
    {   68, 6869,     704,  4, }, // Africa/Lome
    {   68, 5211,     708,  6, }, // Africa/Luanda
    {   68, 4939,   33804,  9, }, // Africa/Lubumbashi
    {   68, 5001,     723,  6, }, // Africa/Lusaka
    {   68, 5322,     729,  6, }, // Africa/Malabo
    {   68, 4987,   33813,  6, }, // Africa/Maputo
    {   68, 5197,     741,  6, }, // Africa/Maseru
    {   68, 5182,     747,  7, }, // Africa/Mbabane
    {   68, 5113,     754,  7, }, // Africa/Mogadishu
    {   68, 7034,   33819,  9, }, // Africa/Monrovia
    {   68, 3853,   33828,  7, }, // Africa/Nairobi
    {   68, 5271,     777,  7, }, // Africa/Ndjamena
    {   68, 5354,   33835,  7, }, // Africa/Niamey
    {   68, 6803,     790,  7, }, // Africa/Nouakchott
    {   68, 6727,     797,  8, }, // Africa/Ouagadougou
    {   68, 5225,   33842, 10, }, // Africa/Porto-Novo
    {   68, 4435,   33852,  8, }, // Africa/Sao_Tome
    {   68, 2866,     823,  7, }, // Africa/Tripoli
    {   68, 6554,     830,  5, }, // Africa/Tunis
    {   68, 4152,     835,  7, }, // Africa/Windhoek
    {   68,  132,     842,  4, }, // America/Adak
    {   68, 3346,   33860,  7, }, // America/Anchorage
    {   68, 5607,   33867,  7, }, // America/Anguilla
    {   68, 5624,   33874,  7, }, // America/Antigua
    {   68, 4554,   33881, 10, }, // America/Araguaina
    {   68,  166,   33891, 12, }, // America/Argentina/Buenos_Aires
    {   68,   91,     888,  9, }, // America/Argentina/Catamarca
    {   68,  270,   33903,  7, }, // America/Argentina/Cordoba
    {   68,  441,   33910,  5, }, // America/Argentina/Jujuy
    {   68, 7462,   33915,  8, }, // America/Argentina/La_Rioja
    {   68,  565,   33923,  7, }, // America/Argentina/Mendoza
    {   68, 7489,   33930, 11, }, // America/Argentina/Rio_Gallegos
    {   68, 7520,     935,  5, }, // America/Argentina/Salta
    {   68, 7544,   33941,  8, }, // America/Argentina/San_Juan
    {   68, 5580,   33949,  7, }, // America/Argentina/San_Luis
    {   68, 7571,     956,  7, }, // America/Argentina/Tucuman
    {   68, 7597,   33956,  7, }, // America/Argentina/Ushuaia
    {   68, 5640,     969,  5, }, // America/Aruba
    {   68, 4252,   33963,  8, }, // America/Asuncion
    {   68,  237,   33971,  8, }, // America/Atikokan
    {   68, 3679,   33979,  5, }, // America/Bahia
    {   68, 7623,   33984, 14, }, // America/Bahia_Banderas
    {   68, 5654,    1013,  8, }, // America/Barbados
    {   68, 7646,   33998,  5, }, // America/Belem
    {   68, 5407,   34003,  6, }, // America/Belize
    {   68, 7660,   34009, 13, }, // America/Blanc-Sablon
    {   68, 7681,   34022,  9, }, // America/Boa_Vista
    {   68, 4361,    1052,  6, }, // America/Bogota
    {   68, 7699,   34031,  5, }, // America/Boise
    {   68, 7713,   34036, 11, }, // America/Cambridge_Bay
    {   68, 7735,   34047, 12, }, // America/Campo_Grande
    {   68, 3868,    1087,  6, }, // America/Cancun
    {   68, 4694,    1093,  7, }, // America/Caracas
    {   68, 4345,   34059,  5, }, // America/Cayenne
    {   68, 5496,   34064, 16, }, // America/Cayman
    {   68, 2260,    1129,  6, }, // America/Chicago
    {   68, 7756,   34080,  6, }, // America/Chihuahua
    {   68, 5422,   34086,  9, }, // America/Costa_Rica
    {   68, 7796,    1151,  7, }, // America/Creston
    {   68, 3791,    1158,  6, }, // America/Cuiaba
    {   68, 5723,   34095,  7, }, // America/Curacao
    {   68, 7812,   34102, 13, }, // America/Danmarkshavn
    {   68, 7833,   34115,  5, }, // America/Dawson
    {   68, 7848,   34120, 10, }, // America/Dawson_Creek
    {   68,  805,    1201,  6, }, // America/Denver
    {   68, 3465,    1207,  7, }, // America/Detroit
    {   68, 5739,    1214,  8, }, // America/Dominica
    {   68,  893,    1222,  8, }, // America/Edmonton
    {   68, 7869,    1230,  8, }, // America/Eirunepe
    {   68, 5441,    1238,  8, }, // America/El_Salvador
    {   68, 7886,    1246, 11, }, // America/Fort_Nelson
    {   68, 7906,    1257,  9, }, // America/Fortaleza
    {   68, 7924,   34130,  9, }, // America/Glace_Bay
    {   68, 6881,   34139,  7, }, // America/Goose_Bay
    {   68, 4612,   34146, 10, }, // America/Grand_Turk
    {   68, 5770,    1292,  7, }, // America/Grenada
    {   68, 5786,   34156,  9, }, // America/Guadeloupe
    {   68, 3760,    1308,  9, }, // America/Guatemala
    {   68, 4841,   34165,  8, }, // America/Guayaquil
    {   68, 6932,   34173,  5, }, // America/Guyana
    {   68, 1939,   34178,  8, }, // America/Halifax
    {   68, 2281,   34186,  6, }, // America/Havana
    {   68, 7942,   34192,  9, }, // America/Hermosillo
    {   68,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {   68,  481,   34201, 15, }, // America/Indiana/Knox
    {   68, 7961,   34216, 17, }, // America/Indiana/Marengo
    {   68, 7985,   34233, 20, }, // America/Indiana/Petersburg
    {   68, 8012,   34253, 18, }, // America/Indiana/Tell_City
    {   68, 8038,   34271, 15, }, // America/Indiana/Vevay
    {   68, 8060,   34286, 18, }, // America/Indiana/Vincennes
    {   68, 8086,   34304, 17, }, // America/Indiana/Winamac
    {   68, 8110,    1452,  6, }, // America/Inuvik
    {   68,  660,   34321,  8, }, // America/Iqaluit
    {   68, 2799,   34329,  7, }, // America/Jamaica
    {   68, 5380,   34336,  4, }, // America/Juneau
    {   68,  521,    1476,  7, }, // America/Kentucky/Louisville
    {   68, 8125,   34340, 18, }, // America/Kentucky/Monticello
    {   68, 5704,   34358, 10, }, // America/Kralendijk
    {   68, 4376,   34368,  6, }, // America/La_Paz
    {   68, 7169,    1520,  4, }, // America/Lima
    {   68, 3239,   34374, 11, }, // America/Los_Angeles
    {   68, 5932,   34385, 19, }, // America/Lower_Princes
    {   68, 8153,   34404,  6, }, // America/Maceio
    {   68, 8168,   34410,  7, }, // America/Managua
    {   68, 1908,    1566,  6, }, // America/Manaus
    {   68, 5897,    1572,  6, }, // America/Marigot
    {   68, 5805,    1578,  8, }, // America/Martinique
    {   68, 8184,    1587,  9, }, // America/Matamoros
    {   68, 2917,   34417,  8, }, // America/Mazatlan
    {   68, 8202,    1604,  8, }, // America/Menominee
    {   68, 8220,    1612,  6, }, // America/Merida
    {   68, 8235,    1618, 10, }, // America/Metlakatla
    {   68, 2949,   34425, 12, }, // America/Mexico_City
    {   68, 4391,    1634,  7, }, // America/Miquelon
    {   68, 8254,    1641,  7, }, // America/Moncton
    {   68, 8270,   34437,  8, }, // America/Monterrey
    {   68, 4098,    1657, 10, }, // America/Montevideo
    {   68, 5824,   34445,  9, }, // America/Montserrat
    {   68, 5481,   34454,  5, }, // America/Nassau
    {   68, 2379,   34459,  6, }, // America/New_York
    {   68, 8288,    1691,  3, }, // America/Nome
    {   68, 1850,   34465,  6, }, // America/Noronha
    {   68, 8301,   34471, 22, }, // America/North_Dakota/Beulah
    {   68, 8329,   34493, 22, }, // America/North_Dakota/Center
    {   68, 8357,   34515, 26, }, // America/North_Dakota/New_Salem
    {   68,  393,   28151,  3, }, // America/Nuuk
    {   68, 8388,    1776,  7, }, // America/Ojinaga
    {   68, 2356,    1783,  6, }, // America/Panama
    {   68, 6173,    1789, 10, }, // America/Paramaribo
    {   68, 2973,   34541,  6, }, // America/Phoenix
    {   68, 3945,    1805, 12, }, // America/Port-au-Prince
    {   68, 5954,   34547, 13, }, // America/Port_of_Spain
    {   68, 8404,   34560, 10, }, // America/Porto_Velho
    {   68, 5843,   34570,  9, }, // America/Puerto_Rico
    {   68, 4030,   34579, 12, }, // America/Punta_Arenas
    {   68, 8424,   34591, 12, }, // America/Rankin_Inlet
    {   68, 8445,   34603,  6, }, // America/Recife
    {   68, 1995,   34609,  6, }, // America/Regina
    {   68, 8460,   34615,  7, }, // America/Resolute
    {   68,  695,   34622, 10, }, // America/Rio_Branco
    {   68, 8477,   34632,  8, }, // America/Santarem
    {   68, 2201,   34640,  9, }, // America/Santiago
    {   68, 6111,   34649, 13, }, // America/Santo_Domingo
    {   68, 1878,   34662,  9, }, // America/Sao_Paulo
    {   68, 6899,   34671, 12, }, // America/Scoresbysund
    {   68, 8494,    1956,  5, }, // America/Sitka
    {   68, 8508,   34683, 14, }, // America/St_Barthelemy
    {   68, 2061,   34697,  7, }, // America/St_Johns
    {   68, 5863,   34704,  9, }, // America/St_Kitts
    {   68, 5880,   34713, 10, }, // America/St_Lucia
    {   68,  855,   34723,  8, }, // America/St_Thomas
    {   68, 5913,   34731, 12, }, // America/St_Vincent
    {   68, 8530,   34743, 12, }, // America/Swift_Current
    {   68, 5461,    2038, 11, }, // America/Tegucigalpa
    {   68, 5756,    2049,  4, }, // America/Thule
    {   68,  313,    2053,  7, }, // America/Tijuana
    {   68,  608,    2060,  7, }, // America/Toronto
    {   68, 5688,    2067,  7, }, // America/Tortola
    {   68, 2093,    2074,  8, }, // America/Vancouver
    {   68, 2144,   34755,  8, }, // America/Whitehorse
    {   68,  734,   34763,  7, }, // America/Winnipeg
    {   68, 8552,   34770,  7, }, // America/Yakutat
    {   68, 6001,   34777,  5, }, // Antarctica/Casey
    {   68, 6094,   34782,  6, }, // Antarctica/Davis
    {   68, 6133,   34788, 13, }, // Antarctica/DumontDUrville
    {   68, 8568,   34801,  7, }, // Antarctica/Macquarie
    {   68, 7066,   34808,  5, }, // Antarctica/Mawson
    {   68, 7113,   34813,  8, }, // Antarctica/McMurdo
    {   68, 8589,    2154,  6, }, // Antarctica/Palmer
    {   68, 7226,    2160,  6, }, // Antarctica/Rothera
    {   68, 7295,   34821,  4, }, // Antarctica/Syowa
    {   68, 8607,    2171,  4, }, // Antarctica/Troll
    {   68, 7374,   34825,  6, }, // Antarctica/Vostok
    {   68, 1429,   34831, 11, }, // Arctic/Longyearbyen
    {   68, 5570,    2192,  4, }, // Asia/Aden
    {   68, 5395,   34842,  6, }, // Asia/Almaty
    {   68, 3968,   28531,  4, }, // Asia/Amman
    {   68, 5511,   34848,  6, }, // Asia/Anadyr
    {   68, 5523,    2213,  5, }, // Asia/Aqtau
    {   68, 5368,   34854,  7, }, // Asia/Aqtobe
    {   68,  964,    2224,  7, }, // Asia/Ashgabat
    {   68, 8624,   34861,  6, }, // Asia/Atyrau
    {   68, 3607,    2237,  6, }, // Asia/Baghdad
    {   68, 5534,    2243,  7, }, // Asia/Bahrain
    {   68, 3653,    2250,  4, }, // Asia/Baku
    {   68, 4466,   34867,  6, }, // Asia/Bangkok
    {   68, 3571,    2261,  7, }, // Asia/Barnaul
    {   68, 4086,   34873,  6, }, // Asia/Beirut
    {   68, 3778,    2274,  6, }, // Asia/Bishkek
    {   68, 5989,   34879,  7, }, // Asia/Brunei
    {   68, 4601,    2286,  4, }, // Asia/Chita
    {   68, 4511,    2290,  7, }, // Asia/Colombo
    {   68, 4540,    2297,  6, }, // Asia/Damascus
    {   68, 1093,     593,  4, }, // Asia/Dhaka
    {   68, 6192,    2308,  4, }, // Asia/Dili
    {   68, 3596,    2312,  5, }, // Asia/Dubai
    {   68, 6159,    2317,  7, }, // Asia/Dushanbe
    {   68, 8636,    2324,  9, }, // Asia/Famagusta
    {   68, 8651,    2333,  4, }, // Asia/Gaza
    {   68, 4795,   34886,  6, }, // Asia/Hebron
    {   68, 1259,   34892,  9, }, // Asia/Ho_Chi_Minh
    {   68, 2704,   34901,  9, }, // Asia/Hong_Kong
    {   68, 4771,    2357,  4, }, // Asia/Hovd
    {   68, 4184,   34910,  6, }, // Asia/Irkutsk
    {   68, 7006,    2368,  7, }, // Asia/Jakarta
    {   68, 6992,   34916,  8, }, // Asia/Jayapura
    {   68, 1290,   34924,  9, }, // Asia/Jerusalem
    {   68, 3560,    2392,  5, }, // Asia/Kabul
    {   68, 4316,    2411,  8, }, // Asia/Kamchatka
    {   68, 4239,    2422,  6, }, // Asia/Karachi
    {   68, 1185,    2428,  8, }, // Asia/Kathmandu
    {   68, 8661,   34933,  7, }, // Asia/Khandyga
    {   68,  992,   34940,  7, }, // Asia/Kolkata
    {   68, 4197,   34947, 11, }, // Asia/Krasnoyarsk
    {   68, 4859,   34958, 12, }, // Asia/Kuala_Lumpur
    {   68, 5976,    2474,  6, }, // Asia/Kuching
    {   68, 5547,   34970,  6, }, // Asia/Kuwait
    {   68, 1211,   34976,  5, }, // Asia/Macau
    {   68, 4017,    2491,  7, }, // Asia/Magadan
    {   68, 1349,    2498,  7, }, // Asia/Makassar
    {   68, 7182,    2505,  6, }, // Asia/Manila
    {   68, 6920,   34981,  6, }, // Asia/Muscat
    {   68, 2554,   34987,  8, }, // Asia/Nicosia
    {   68, 8675,   34995, 11, }, // Asia/Novokuznetsk
    {   68, 4135,    2535, 11, }, // Asia/Novosibirsk
    {   68, 4229,    2546,  4, }, // Asia/Omsk
    {   68, 7145,   35006,  4, }, // Asia/Oral
    {   68, 6961,   35010,  8, }, // Asia/Phnom_Penh
    {   68, 8693,    2563,  9, }, // Asia/Pontianak
    {   68, 4214,   35018,  9, }, // Asia/Pyongyang
    {   68, 5559,    2579,  5, }, // Asia/Qatar
    {   68, 8708,   35027,  8, }, // Asia/Qostanay
    {   68, 4269,   35035,  9, }, // Asia/Qyzylorda
    {   68, 3584,   35044,  5, }, // Asia/Riyadh
    {   68, 4408,    2614,  7, }, // Asia/Sakhalin
    {   68, 7245,    2621,  9, }, // Asia/Samarkand
    {   68, 3279,    2630,  4, }, // Asia/Seoul
    {   68, 1053,   35049,  6, }, // Asia/Shanghai
    {   68, 3300,    2640,  8, }, // Asia/Singapore
    {   68, 4297,   35055, 13, }, // Asia/Srednekolymsk
    {   68, 3263,   35068,  6, }, // Asia/Taipei
    {   68, 4781,    2667,  7, }, // Asia/Tashkent
    {   68, 3915,   35074,  7, }, // Asia/Tbilisi
    {   68, 2772,   35081,  7, }, // Asia/Tehran
    {   68, 1317,   35088,  5, }, // Asia/Thimphu
    {   68, 2821,    2692,  5, }, // Asia/Tokyo
    {   68, 4572,    2697,  5, }, // Asia/Tomsk
    {   68, 1021,   35093, 10, }, // Asia/Ulaanbaatar
    {   68, 1159,   35103,  7, }, // Asia/Urumqi
    {   68, 8722,    2718,  8, }, // Asia/Ust-Nera
    {   68, 6977,   35110, 10, }, // Asia/Vientiane
    {   68, 4710,    2733, 11, }, // Asia/Vladivostok
    {   68, 4828,   35120,  7, }, // Asia/Yakutsk
    {   68, 1235,   28911,  6, }, // Asia/Yangon
    {   68, 3883,   35127, 13, }, // Asia/Yekaterinburg
    {   68, 3747,   35140,  7, }, // Asia/Yerevan
    {   68, 3663,   35147,  5, }, // Atlantic/Azores
    {   68, 5671,   35152,  7, }, // Atlantic/Bermuda
    {   68, 6628,   35159, 15, }, // Atlantic/Canary
    {   68, 3727,   35174, 18, }, // Atlantic/Cape_Verde
    {   68, 1395,   35192, 13, }, // Atlantic/Faroe
    {   68, 8736,    2853,  7, }, // Atlantic/Madeira
    {   68, 2748,   35205,  9, }, // Atlantic/Reykjavik
    {   68, 7272,   35214, 13, }, // Atlantic/South_Georgia
    {   68, 6821,   35227, 10, }, // Atlantic/St_Helena
    {   68, 6644,    2911,  6, }, // Atlantic/Stanley
    {   68, 1670,    2917,  8, }, // Australia/Adelaide
    {   68, 1635,   35237,  8, }, // Australia/Brisbane
    {   68, 1799,   35245, 10, }, // Australia/Broken_Hill
    {   68, 1583,    2943,  6, }, // Australia/Darwin
    {   68, 3637,    2949,  5, }, // Australia/Eucla
    {   68, 1516,    2954,  6, }, // Australia/Hobart
    {   68, 8753,    2960,  8, }, // Australia/Lindeman
    {   68, 1547,   35255,  8, }, // Australia/Lord_Howe
    {   68, 1727,    2976,  7, }, // Australia/Melbourne
    {   68, 1762,    2983,  4, }, // Australia/Perth
    {   68, 1463,   35263,  6, }, // Australia/Sydney
    {   68, 6441,    2993,  9, }, // Europe/Amsterdam
    {   68, 6216,   29143,  6, }, // Europe/Andorra
    {   68, 3620,    3009,  8, }, // Europe/Astrakhan
    {   68, 2300,   35269,  5, }, // Europe/Athens
    {   68, 6336,   35274,  7, }, // Europe/Belgrade
    {   68, 4757,    3028,  6, }, // Europe/Berlin
    {   68, 6488,    3034, 10, }, // Europe/Bratislava
    {   68, 2167,   35281,  6, }, // Europe/Brussels
    {   68, 3928,   35287,  8, }, // Europe/Bucharest
    {   68, 3806,   35295, 10, }, // Europe/Budapest
    {   68, 8772,   35305,  8, }, // Europe/Busingen
    {   68, 2583,   35313,  7, }, // Europe/Chisinau
    {   68, 6289,   35320, 10, }, // Europe/Copenhagen
    {   68, 2338,   35330,  6, }, // Europe/Dublin
    {   68, 6307,    3114,  9, }, // Europe/Gibraltar
    {   68, 8788,   35336,  6, }, // Europe/Guernsey
    {   68, 6612,    3129,  8, }, // Europe/Helsinki
    {   68, 8804,   35342, 10, }, // Europe/Isle_of_Man
    {   68, 1130,   35352,  8, }, // Europe/Istanbul
    {   68, 8823,    3153,  5, }, // Europe/Jersey
    {   68, 3979,   35360, 11, }, // Europe/Kaliningrad
    {   68, 8929,    3169,  5, }, // Europe/Kirov
    {   68, 2527,   35371,  5, }, // Europe/Kyiv
    {   68, 3213,   35376,  7, }, // Europe/Lisbon
    {   68, 6506,   35383,  7, }, // Europe/Ljubljana
    {   68, 2501,    3195,  6, }, // Europe/London
    {   68, 6365,   35390, 10, }, // Europe/Luxembourg
    {   68, 4892,    3212,  6, }, // Europe/Madrid
    {   68, 6397,    3218,  5, }, // Europe/Malta
    {   68, 6582,   35400,  8, }, // Europe/Mariehamn
    {   68, 3693,    3232,  5, }, // Europe/Minsk
    {   68, 6410,    3237,  6, }, // Europe/Monaco
    {   68, 3537,    3243,  6, }, // Europe/Moscow
    {   68, 6458,    3249,  4, }, // Europe/Oslo
    {   68, 4284,   35408,  5, }, // Europe/Paris
    {   68, 6424,    3258,  9, }, // Europe/Podgorica
    {   68, 6275,    3267,  4, }, // Europe/Prague
    {   68, 8837,    3272,  4, }, // Europe/Riga
    {   68, 6324,    3276,  3, }, // Europe/Rome
    {   68, 4331,    3279,  6, }, // Europe/Samara
    {   68, 6470,   35413, 10, }, // Europe/San_Marino
    {   68, 6245,   35423,  8, }, // Europe/Sarajevo
    {   68, 4451,    3302,  7, }, // Europe/Saratov
    {   68, 8849,   35431, 10, }, // Europe/Simferopol
    {   68, 6383,   35441,  6, }, // Europe/Skopje
    {   68, 6599,   35447,  6, }, // Europe/Sofia
    {   68, 6523,   35453,  8, }, // Europe/Stockholm
    {   68, 8867,   35461,  5, }, // Europe/Tallinn
    {   68, 6202,    3343,  6, }, // Europe/Tirane
    {   68, 8882,   35466,  8, }, // Europe/Ulyanovsk
    {   68, 6352,   35474,  5, }, // Europe/Vaduz
    {   68, 6567,    3363,  7, }, // Europe/Vatican
    {   68, 6231,   35479,  3, }, // Europe/Vienna
    {   68, 8899,   35482,  6, }, // Europe/Vilnius
    {   68, 4727,    3381,  9, }, // Europe/Volgograd
    {   68, 3190,    3390,  7, }, // Europe/Warsaw
    {   68, 6261,    3397,  6, }, // Europe/Zagreb
    {   68, 6540,   35488,  5, }, // Europe/Zurich
    {   68, 5078,   35493, 12, }, // Indian/Antananarivo
    {   68, 6947,    3421,  5, }, // Indian/Chagos
    {   68, 6046,   35505,  5, }, // Indian/Christmas
    {   68, 6063,    3438,  5, }, // Indian/Cocos
    {   68, 5029,   35510,  6, }, // Indian/Comoro
    {   68, 6661,    3460,  8, }, // Indian/Kerguelen
    {   68, 7260,   35516,  4, }, // Indian/Mahe
    {   68, 7050,   35520,  7, }, // Indian/Maldives
    {   68, 4069,   35527, 10, }, // Indian/Mauritius
    {   68, 5098,   35537,  5, }, // Indian/Mayotte
    {   68, 7211,   35542,  7, }, // Indian/Reunion
    {   68, 4422,   35549,  5, }, // Pacific/Apia
    {   68,  932,   35554,  6, }, // Pacific/Auckland
    {   68, 3706,    3508,  8, }, // Pacific/Bougainville
    {   68, 3015,   35560,  6, }, // Pacific/Chatham
    {   68, 3157,    3521,  4, }, // Pacific/Chuuk
    {   68, 2237,   35566, 14, }, // Pacific/Easter
    {   68, 7360,    3536,  4, }, // Pacific/Efate
    {   68, 7327,    3541,  7, }, // Pacific/Fakaofo
    {   68, 3902,    3548,  4, }, // Pacific/Fiji
    {   68, 7343,    3552,  8, }, // Pacific/Funafuti
    {   68, 6678,    3560,  9, }, // Pacific/Galapagos
    {   68, 6696,   35580,  7, }, // Pacific/Gambier
    {   68, 3822,   35587, 11, }, // Pacific/Guadalcanal
    {   68, 6018,    3606,  4, }, // Pacific/Guam
    {   68, 2723,   35598,  8, }, // Pacific/Honolulu
    {   68, 3049,   35606,  9, }, // Pacific/Kanton
    {   68, 3998,    3616, 10, }, // Pacific/Kiritimati
    {   68, 7019,   35615,  5, }, // Pacific/Kosrae
    {   68, 2842,   35620, 10, }, // Pacific/Kwajalein
    {   68, 4877,   35630,  6, }, // Pacific/Majuro
    {   68, 4051,    3647,  6, }, // Pacific/Marquesas
    {   68, 8914,   35636,  6, }, // Pacific/Midway
    {   68, 7084,    3680,  5, }, // Pacific/Nauru
    {   68, 7132,    3685,  4, }, // Pacific/Niue
    {   68, 4168,    3689,  7, }, // Pacific/Norfolk
    {   68, 7098,    3696,  5, }, // Pacific/Noumea
    {   68, 3126,   35642,  9, }, // Pacific/Pago_Pago
    {   68, 7155,    3710,  5, }, // Pacific/Palau
    {   68, 7194,    3715,  7, }, // Pacific/Pitcairn
    {   68, 3096,   35651,  6, }, // Pacific/Pohnpei
    {   68, 4807,   35657, 11, }, // Pacific/Port_Moresby
    {   68, 6076,    3739,  9, }, // Pacific/Rarotonga
    {   68, 6031,   35668,  6, }, // Pacific/Saipan
    {   68, 7312,   35674,  6, }, // Pacific/Tahiti
    {   68, 6712,    3759,  6, }, // Pacific/Tarawa
    {   68, 4583,    3765,  9, }, // Pacific/Tongatapu
    {   68, 7392,   35680,  4, }, // Pacific/Wake
    {   68, 7405,   35684,  5, }, // Pacific/Wallis
    {   69, 5059,   35689, 10, }, // Africa/Addis_Ababa Breton/Latin/France
    {   69, 7420,   26343,  5, }, // Africa/Algiers
    {   69,   14,       0,  6, }, // Africa/Asmara
    {   69, 2320,   35699,  5, }, // Africa/Cairo
    {   69, 4117,   35704, 25, }, // Africa/Casablanca
    {   69, 5166,   35729,  7, }, // Africa/El_Aaiun
    {   69, 6869,   20300,  4, }, // Africa/Lome
    {   69, 5113,   26379,  8, }, // Africa/Mogadishu
    {   69, 5271,   35736,  9, }, // Africa/Ndjamena
    {   69, 4435,       6,  8, }, // Africa/Sao_Tome
    {   69, 2866,   35745, 19, }, // Africa/Tripoli
    {   69, 6554,    6349,  5, }, // Africa/Tunis
    {   69, 4252,      14,  8, }, // America/Asuncion
    {   69,  237,      22,  8, }, // America/Atikokan
    {   69, 7623,      30, 17, }, // America/Bahia_Banderas
    {   69, 7646,   35764,  5, }, // America/Belem
    {   69, 4361,   20389,  6, }, // America/Bogota
    {   69, 3868,      47,  6, }, // America/Cancun
    {   69, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   69, 5723,      66,  7, }, // America/Curacao
    {   69, 5441,    6688,  8, }, // America/El_Salvador
    {   69, 5786,   35769,  9, }, // America/Guadeloupe
    {   69, 2281,   20417,  9, }, // America/Havana
    {   69,  481,      73, 13, }, // America/Indiana/Knox
    {   69, 7961,      86, 16, }, // America/Indiana/Marengo
    {   69, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   69, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   69, 8038,     139, 14, }, // America/Indiana/Vevay
    {   69, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   69, 8086,     171, 16, }, // America/Indiana/Winamac
    {   69, 2799,    3877,  7, }, // America/Jamaica
    {   69, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   69, 5932,     207, 22, }, // America/Lower_Princes
    {   69, 5805,    5142,  8, }, // America/Martinique
    {   69, 8220,     229,  6, }, // America/Merida
    {   69, 2949,   35778, 12, }, // America/Mexico_City
    {   69, 4391,    7031,  7, }, // America/Miquelon
    {   69, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   69, 8329,     255, 20, }, // America/North_Dakota/Center
    {   69, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   69,  393,   35790, 14, }, // America/Nuuk
    {   69, 2356,   20547,  6, }, // America/Panama
    {   69, 1878,   26580,  9, }, // America/Sao_Paulo
    {   69, 6899,     302, 16, }, // America/Scoresbysund
    {   69, 8508,   35804, 16, }, // America/St_Barthelemy
    {   69, 2061,   35820, 12, }, // America/St_Johns
    {   69, 5863,   26617, 11, }, // America/St_Kitts
    {   69, 5880,   35832, 11, }, // America/St_Lucia
    {   69,  855,     360, 10, }, // America/St_Thomas
    {   69, 5913,   35843, 11, }, // America/St_Vincent
    {   69, 5756,   35854,  5, }, // America/Thule
    {   69, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   69, 5511,   35859,  7, }, // Asia/Anadyr
    {   69, 5534,    4050,  7, }, // Asia/Bahrain
    {   69, 3653,   35866,  5, }, // Asia/Baku
    {   69, 4086,   35871,  6, }, // Asia/Beirut
    {   69, 4601,   35877,  6, }, // Asia/Chita
    {   69, 4511,   35883,  7, }, // Asia/Colombo
    {   69, 4540,    4083,  6, }, // Asia/Damascus
    {   69, 1259,   35890, 15, }, // Asia/Ho_Chi_Minh
    {   69, 1290,   33369,  9, }, // Asia/Jerusalem
    {   69, 3560,   35905,  6, }, // Asia/Kabul
    {   69, 4316,   35911, 10, }, // Asia/Kamchatka
    {   69, 1185,     413,  9, }, // Asia/Kathmandu
    {   69,  992,   35921,  8, }, // Asia/Kolkata
    {   69, 5547,   35929,  6, }, // Asia/Kuwait
    {   69, 1211,    4203,  5, }, // Asia/Macau
    {   69, 6920,   35935,  6, }, // Asia/Muscat
    {   69, 2554,   35941,  8, }, // Asia/Nicosia
    {   69, 6961,   35949, 10, }, // Asia/Phnom_Penh
    {   69, 4214,   35959, 10, }, // Asia/Pyongyang
    {   69, 3300,   35969,  9, }, // Asia/Singapore
    {   69, 4781,   35978,  8, }, // Asia/Tashkent
    {   69, 1159,   26801,  6, }, // Asia/Urumqi
    {   69, 6977,   35986,  9, }, // Asia/Vientiane
    {   69, 1235,     442,  6, }, // Asia/Yangon
    {   69, 3883,   35995, 14, }, // Asia/Yekaterinburg
    {   69, 3663,   36009,  6, }, // Atlantic/Azores
    {   69, 5671,   36015,  8, }, // Atlantic/Bermuda
    {   69, 6628,   36023,  8, }, // Atlantic/Canary
    {   69, 3727,   36031,  8, }, // Atlantic/Cape_Verde
    {   69, 1395,   36039,  5, }, // Atlantic/Faroe
    {   69, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {   69, 7272,   36053, 13, }, // Atlantic/South_Georgia
    {   69, 6821,   36066, 12, }, // Atlantic/St_Helena
    {   69, 2300,    5793,  4, }, // Europe/Athens
    {   69, 6336,    7942,  7, }, // Europe/Belgrade
    {   69, 2167,   26890,  6, }, // Europe/Brussels
    {   69, 3928,   26897,  8, }, // Europe/Bucharest
    {   69, 6289,    4357, 10, }, // Europe/Copenhagen
    {   69, 2338,   36078,  6, }, // Europe/Dublin
    {   69, 6307,   36084,  9, }, // Europe/Gibraltar
    {   69, 8788,   36093,  9, }, // Europe/Guernsey
    {   69, 8804,   36102,  5, }, // Europe/Isle_of_Man
    {   69, 8823,   36107,  8, }, // Europe/Jersey
    {   69, 2527,    8030,  4, }, // Europe/Kyiv
    {   69, 3213,   21019,  6, }, // Europe/Lisbon
    {   69, 2501,   36115,  7, }, // Europe/London
    {   69, 6365,   36122, 11, }, // Europe/Luxembourg
    {   69, 6582,   36133,  9, }, // Europe/Mariehamn
    {   69, 3693,   36142,  5, }, // Europe/Minsk
    {   69, 3537,   36147,  6, }, // Europe/Moscow
    {   69, 4284,   33603,  5, }, // Europe/Paris
    {   69, 6275,   36153,  5, }, // Europe/Prague
    {   69, 6324,    5946,  4, }, // Europe/Rome
    {   69, 6202,    8137,  6, }, // Europe/Tirane
    {   69, 6567,    5978,  7, }, // Europe/Vatican
    {   69, 3190,   21126,  8, }, // Europe/Warsaw
    {   69, 6063,   36158,  5, }, // Indian/Cocos
    {   69, 5029,   36163,  7, }, // Indian/Comoro
    {   69, 6661,   36170,  9, }, // Indian/Kerguelen
    {   69, 7050,   36179,  8, }, // Indian/Maldives
    {   69, 4069,   36187,  5, }, // Indian/Mauritius
    {   69, 7211,   36192,  7, }, // Indian/Reunion
    {   69, 3157,     474,  5, }, // Pacific/Chuuk
    {   69, 2237,   36199,  9, }, // Pacific/Easter
    {   69, 3902,    4488,  5, }, // Pacific/Fiji
    {   69, 6678,   21159,  9, }, // Pacific/Galapagos
    {   69, 2723,    4493,  8, }, // Pacific/Honolulu
    {   69, 4051,   36208,  6, }, // Pacific/Marquesas
    {   69, 3096,     479,  7, }, // Pacific/Pohnpei
    {   70, 6788,   36214,  7, }, // Africa/Abidjan Bulgarian/Cyrillic/Bulgaria
    {   70, 6760,   27046,  4, }, // Africa/Accra
    {   70, 5059,   33714, 10, }, // Africa/Addis_Ababa
    {   70, 7420,     508,  5, }, // Africa/Algiers
    {   70,   14,   27065,  6, }, // Africa/Asmara
    {   70,   44,     519,  6, }, // Africa/Bamako
    {   70, 5257,   33724,  6, }, // Africa/Bangui
    {   70, 6746,   27076,  7, }, // Africa/Banjul
    {   70, 7435,     536,  5, }, // Africa/Bissau
    {   70, 4957,   36221,  9, }, // Africa/Blantyre
    {   70, 5287,   33742,  8, }, // Africa/Brazzaville
    {   70, 4922,     558,  9, }, // Africa/Bujumbura
    {   70, 2320,   36230,  5, }, // Africa/Cairo
    {   70, 4117,   33764, 10, }, // Africa/Casablanca
    {   70, 7449,     581,  5, }, // Africa/Ceuta
    {   70, 6773,     586,  7, }, // Africa/Conakry
    {   70, 6840,     593,  5, }, // Africa/Dakar
    {   70, 5130,   36235, 12, }, // Africa/Dar_es_Salaam
    {   70, 5043,   36247,  7, }, // Africa/Djibouti
    {   70, 5243,     616,  5, }, // Africa/Douala
    {   70, 5166,   36254,  6, }, // Africa/El_Aaiun
    {   70, 6853,   36260,  8, }, // Africa/Freetown
    {   70, 4906,     635,  8, }, // Africa/Gaborone
    {   70, 5015,   33781,  6, }, // Africa/Harare
    {   70, 4479,   36268, 11, }, // Africa/Johannesburg
    {   70, 4499,   27173,  5, }, // Africa/Juba
    {   70, 5151,     665,  7, }, // Africa/Kampala
    {   70, 4524,     672,  6, }, // Africa/Khartoum
    {   70, 4973,     678,  6, }, // Africa/Kigali
    {   70, 5306,     684,  7, }, // Africa/Kinshasa
    {   70, 4744,     691,  5, }, // Africa/Lagos
    {   70, 5336,     696,  8, }, // Africa/Libreville
    {   70, 6869,     704,  4, }, // Africa/Lome
    {   70, 5211,     708,  6, }, // Africa/Luanda
    {   70, 4939,     714,  9, }, // Africa/Lubumbashi
    {   70, 5001,     723,  6, }, // Africa/Lusaka
    {   70, 5322,     729,  6, }, // Africa/Malabo
    {   70, 4987,   33813,  6, }, // Africa/Maputo
    {   70, 5197,     741,  6, }, // Africa/Maseru
    {   70, 5182,     747,  7, }, // Africa/Mbabane
    {   70, 5113,   36279,  8, }, // Africa/Mogadishu
    {   70, 7034,   36287,  8, }, // Africa/Monrovia
    {   70, 3853,   36295,  7, }, // Africa/Nairobi
    {   70, 5271,   27254,  8, }, // Africa/Ndjamena
    {   70, 5354,   36302,  6, }, // Africa/Niamey
    {   70, 6803,     790,  7, }, // Africa/Nouakchott
    {   70, 6727,     797,  8, }, // Africa/Ouagadougou
    {   70, 5225,   33842, 10, }, // Africa/Porto-Novo
    {   70, 4435,   33852,  8, }, // Africa/Sao_Tome
    {   70, 2866,     823,  7, }, // Africa/Tripoli
    {   70, 6554,     830,  5, }, // Africa/Tunis
    {   70, 4152,     835,  7, }, // Africa/Windhoek
    {   70,  132,     842,  4, }, // America/Adak
    {   70, 3346,   36308,  8, }, // America/Anchorage
    {   70, 5607,   36316,  7, }, // America/Anguilla
    {   70, 5624,     860,  7, }, // America/Antigua
    {   70, 4554,   36323,  9, }, // America/Araguaina
    {   70,  166,   36332, 12, }, // America/Argentina/Buenos_Aires
    {   70,   91,     888,  9, }, // America/Argentina/Catamarca
    {   70,  270,   33903,  7, }, // America/Argentina/Cordoba
    {   70,  441,   36344,  5, }, // America/Argentina/Jujuy
    {   70, 7462,   33915,  8, }, // America/Argentina/La_Rioja
    {   70,  565,     917,  7, }, // America/Argentina/Mendoza
    {   70, 7489,   36349, 11, }, // America/Argentina/Rio_Gallegos
    {   70, 7520,     935,  5, }, // America/Argentina/Salta
    {   70, 7544,   33941,  8, }, // America/Argentina/San_Juan
    {   70, 5580,   36360,  8, }, // America/Argentina/San_Luis
    {   70, 7571,     956,  7, }, // America/Argentina/Tucuman
    {   70, 7597,   27394,  5, }, // America/Argentina/Ushuaia
    {   70, 5640,     969,  5, }, // America/Aruba
    {   70, 4252,   33963,  8, }, // America/Asuncion
    {   70,  237,   33971,  8, }, // America/Atikokan
    {   70, 3679,   36368,  4, }, // America/Bahia
    {   70, 7623,   36372, 16, }, // America/Bahia_Banderas
    {   70, 5654,    1013,  8, }, // America/Barbados
    {   70, 7646,   33998,  5, }, // America/Belem
    {   70, 5407,    1026,  5, }, // America/Belize
    {   70, 7660,   36388, 11, }, // America/Blanc-Sablon
    {   70, 7681,   34022,  9, }, // America/Boa_Vista
    {   70, 4361,    1052,  6, }, // America/Bogota
    {   70, 7699,   36399,  5, }, // America/Boise
    {   70, 7713,   36404, 13, }, // America/Cambridge_Bay
    {   70, 7735,   34047, 12, }, // America/Campo_Grande
    {   70, 3868,    1087,  6, }, // America/Cancun
    {   70, 4694,    1093,  7, }, // America/Caracas
    {   70, 4345,   36417,  5, }, // America/Cayenne
    {   70, 5496,   36422, 17, }, // America/Cayman
    {   70, 2260,    1129,  6, }, // America/Chicago
    {   70, 7756,    1135,  6, }, // America/Chihuahua
    {   70, 7774,   36439, 13, }, // America/Ciudad_Juarez
    {   70, 5422,   36452, 10, }, // America/Costa_Rica
    {   70, 7796,   36462,  7, }, // America/Creston
    {   70, 3791,   36469,  5, }, // America/Cuiaba
    {   70, 5723,   36474,  7, }, // America/Curacao
    {   70, 7812,   36481, 12, }, // America/Danmarkshavn
    {   70, 7833,   36493,  6, }, // America/Dawson
    {   70, 7848,   36499, 12, }, // America/Dawson_Creek
    {   70,  805,   36511,  6, }, // America/Denver
    {   70, 3465,   36517,  7, }, // America/Detroit
    {   70, 5739,    1214,  8, }, // America/Dominica
    {   70,  893,   36524,  8, }, // America/Edmonton
    {   70, 7869,   36532,  8, }, // America/Eirunepe
    {   70, 5441,    1238,  8, }, // America/El_Salvador
    {   70, 7886,   36540, 11, }, // America/Fort_Nelson
    {   70, 7906,    1257,  9, }, // America/Fortaleza
    {   70, 7924,   36551,  9, }, // America/Glace_Bay
    {   70, 6881,   36560,  7, }, // America/Goose_Bay
    {   70, 4612,   36567, 10, }, // America/Grand_Turk
    {   70, 5770,    1292,  7, }, // America/Grenada
    {   70, 5786,    1299,  9, }, // America/Guadeloupe
    {   70, 3760,    1308,  9, }, // America/Guatemala
    {   70, 4841,   36577,  7, }, // America/Guayaquil
    {   70, 6932,   27722,  5, }, // America/Guyana
    {   70, 1939,   34178,  8, }, // America/Halifax
    {   70, 2281,   34186,  6, }, // America/Havana
    {   70, 7942,   36584,  9, }, // America/Hermosillo
    {   70,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {   70,  481,    1365,  4, }, // America/Indiana/Knox
    {   70, 7961,    1378,  7, }, // America/Indiana/Marengo
    {   70, 7985,   36593, 10, }, // America/Indiana/Petersburg
    {   70, 8012,   34253,  8, }, // America/Indiana/Tell_City
    {   70, 8038,   36603,  4, }, // America/Indiana/Vevay
    {   70, 8060,   36607,  7, }, // America/Indiana/Vincennes
    {   70, 8086,    1445,  7, }, // America/Indiana/Winamac
    {   70, 8110,    1452,  6, }, // America/Inuvik
    {   70,  660,   34321,  8, }, // America/Iqaluit
    {   70, 2799,   27879,  6, }, // America/Jamaica
    {   70, 5380,   36614,  5, }, // America/Juneau
    {   70,  521,    1476,  7, }, // America/Kentucky/Louisville
    {   70, 8125,   34340,  9, }, // America/Kentucky/Monticello
    {   70, 5704,   36619, 10, }, // America/Kralendijk
    {   70, 4376,   36629,  6, }, // America/La_Paz
    {   70, 7169,    1520,  4, }, // America/Lima
    {   70, 3239,   36635, 12, }, // America/Los_Angeles
    {   70, 5932,   36647, 20, }, // America/Lower_Princes
    {   70, 8153,   36667,  6, }, // America/Maceio
    {   70, 8168,    1559,  7, }, // America/Managua
    {   70, 1908,    1566,  6, }, // America/Manaus
    {   70, 5897,    1572,  6, }, // America/Marigot
    {   70, 5805,    1578,  9, }, // America/Martinique
    {   70, 8184,    1587,  9, }, // America/Matamoros
    {   70, 2917,    1596,  8, }, // America/Mazatlan
    {   70, 8202,    1604,  8, }, // America/Menominee
    {   70, 8220,    1612,  6, }, // America/Merida
    {   70, 8235,    1618, 10, }, // America/Metlakatla
    {   70, 2949,   34425, 12, }, // America/Mexico_City
    {   70, 4391,    1634,  7, }, // America/Miquelon
    {   70, 8254,    1641,  7, }, // America/Moncton
    {   70, 8270,   36673,  8, }, // America/Monterrey
    {   70, 4098,    1657, 10, }, // America/Montevideo
    {   70, 5824,   34445,  9, }, // America/Montserrat
    {   70, 5481,   34454,  5, }, // America/Nassau
    {   70, 2379,   36681,  7, }, // America/New_York
    {   70, 8288,   36688,  4, }, // America/Nome
    {   70, 1850,   36692,  6, }, // America/Noronha
    {   70, 8301,   36698,  4, }, // America/North_Dakota/Beulah
    {   70, 8329,   36702,  6, }, // America/North_Dakota/Center
    {   70, 8357,   36708,  9, }, // America/North_Dakota/New_Salem
    {   70,  393,    1772,  4, }, // America/Nuuk
    {   70, 8388,    1776,  7, }, // America/Ojinaga
    {   70, 2356,    1783,  6, }, // America/Panama
    {   70, 6173,    1789, 10, }, // America/Paramaribo
    {   70, 2973,    1799,  6, }, // America/Phoenix
    {   70, 3945,    1805, 12, }, // America/Port-au-Prince
    {   70, 5954,   36717, 13, }, // America/Port_of_Spain
    {   70, 8404,   36730, 11, }, // America/Porto_Velho
    {   70, 5843,   36741, 11, }, // America/Puerto_Rico
    {   70, 4030,   34579, 12, }, // America/Punta_Arenas
    {   70, 8424,   34591, 12, }, // America/Rankin_Inlet
    {   70, 8445,   34603,  6, }, // America/Recife
    {   70, 1995,   36752,  8, }, // America/Regina
    {   70, 8460,   36760,  7, }, // America/Resolute
    {   70,  695,   34622, 10, }, // America/Rio_Branco
    {   70, 8477,   34632,  8, }, // America/Santarem
    {   70, 2201,   36767,  7, }, // America/Santiago
    {   70, 6111,   34649, 13, }, // America/Santo_Domingo
    {   70, 1878,   36774,  9, }, // America/Sao_Paulo
    {   70, 6899,   36783, 10, }, // America/Scoresbysund
    {   70, 8494,    1956,  5, }, // America/Sitka
    {   70, 8508,   36793, 13, }, // America/St_Barthelemy
    {   70, 2061,   36806, 11, }, // America/St_Johns
    {   70, 5863,   36817, 10, }, // America/St_Kitts
    {   70, 5880,   36827, 11, }, // America/St_Lucia
    {   70,  855,   36838, 11, }, // America/St_Thomas
    {   70, 5913,   36849, 13, }, // America/St_Vincent
    {   70, 8530,   36862, 12, }, // America/Swift_Current
    {   70, 5461,    2038, 11, }, // America/Tegucigalpa
    {   70, 5756,    2049,  4, }, // America/Thule
    {   70,  313,    2053,  7, }, // America/Tijuana
    {   70,  608,    2060,  7, }, // America/Toronto
    {   70, 5688,    2067,  7, }, // America/Tortola
    {   70, 2093,   36874,  8, }, // America/Vancouver
    {   70, 2144,   28435,  8, }, // America/Whitehorse
    {   70,  734,   36882,  7, }, // America/Winnipeg
    {   70, 8552,   28450,  6, }, // America/Yakutat
    {   70, 6001,   36889,  5, }, // Antarctica/Casey
    {   70, 6094,   36894,  6, }, // Antarctica/Davis
    {   70, 6133,   36900, 12, }, // Antarctica/DumontDUrville
    {   70, 8568,    2131,  8, }, // Antarctica/Macquarie
    {   70, 7066,   36912,  6, }, // Antarctica/Mawson
    {   70, 7113,   36918,  8, }, // Antarctica/McMurdo
    {   70, 8589,   36926,  6, }, // Antarctica/Palmer
    {   70, 7226,    2160,  6, }, // Antarctica/Rothera
    {   70, 7295,   34821,  4, }, // Antarctica/Syowa
    {   70, 8607,    2171,  4, }, // Antarctica/Troll
    {   70, 7374,   34825,  6, }, // Antarctica/Vostok
    {   70, 1429,   36932, 10, }, // Arctic/Longyearbyen
    {   70, 5570,    2192,  4, }, // Asia/Aden
    {   70, 5395,   34842,  6, }, // Asia/Almaty
    {   70, 3968,   28531,  4, }, // Asia/Amman
    {   70, 5511,   34848,  6, }, // Asia/Anadyr
    {   70, 5523,    2213,  5, }, // Asia/Aqtau
    {   70, 5368,    2218,  6, }, // Asia/Aqtobe
    {   70,  964,    2224,  7, }, // Asia/Ashgabat
    {   70, 8624,   36942,  6, }, // Asia/Atyrau
    {   70, 3607,    2237,  6, }, // Asia/Baghdad
    {   70, 5534,   36948,  7, }, // Asia/Bahrain
    {   70, 3653,    2250,  4, }, // Asia/Baku
    {   70, 4466,   34867,  6, }, // Asia/Bangkok
    {   70, 3571,    2261,  7, }, // Asia/Barnaul
    {   70, 4086,   28563,  6, }, // Asia/Beirut
    {   70, 3778,    2274,  6, }, // Asia/Bishkek
    {   70, 5989,   28575,  6, }, // Asia/Brunei
    {   70, 4601,    2286,  4, }, // Asia/Chita
    {   70, 4511,    2290,  7, }, // Asia/Colombo
    {   70, 4540,    2297,  6, }, // Asia/Damascus
    {   70, 1093,     593,  4, }, // Asia/Dhaka
    {   70, 6192,    2308,  4, }, // Asia/Dili
    {   70, 3596,   28596,  5, }, // Asia/Dubai
    {   70, 6159,    2317,  7, }, // Asia/Dushanbe
    {   70, 8636,    2324,  9, }, // Asia/Famagusta
    {   70, 8651,    2333,  4, }, // Asia/Gaza
    {   70, 4795,   34886,  6, }, // Asia/Hebron
    {   70, 1259,    2343,  7, }, // Asia/Ho_Chi_Minh
    {   70, 2704,   36955,  7, }, // Asia/Hong_Kong
    {   70, 4771,    2357,  4, }, // Asia/Hovd
    {   70, 4184,    2361,  7, }, // Asia/Irkutsk
    {   70, 7006,   28638,  8, }, // Asia/Jakarta
    {   70, 6992,   28646,  8, }, // Asia/Jayapura
    {   70, 1290,   36962,  9, }, // Asia/Jerusalem
    {   70, 3560,    2392,  5, }, // Asia/Kabul
    {   70, 4316,    2411,  8, }, // Asia/Kamchatka
    {   70, 4239,    2422,  6, }, // Asia/Karachi
    {   70, 1185,    2428,  8, }, // Asia/Kathmandu
    {   70, 8661,   34933,  7, }, // Asia/Khandyga
    {   70,  992,   36971,  7, }, // Asia/Kolkata
    {   70, 4197,   36978, 10, }, // Asia/Krasnoyarsk
    {   70, 4859,   34958, 12, }, // Asia/Kuala_Lumpur
    {   70, 5976,    2474,  5, }, // Asia/Kuching
    {   70, 5547,   28693,  6, }, // Asia/Kuwait
    {   70, 1211,    2486,  5, }, // Asia/Macau
    {   70, 4017,    2491,  7, }, // Asia/Magadan
    {   70, 1349,    2498,  7, }, // Asia/Makassar
    {   70, 7182,    2505,  6, }, // Asia/Manila
    {   70, 6920,   34981,  6, }, // Asia/Muscat
    {   70, 2554,   36988,  7, }, // Asia/Nicosia
    {   70, 8675,    2524, 11, }, // Asia/Novokuznetsk
    {   70, 4135,    2535, 11, }, // Asia/Novosibirsk
    {   70, 4229,    2546,  4, }, // Asia/Omsk
    {   70, 7145,   36995,  4, }, // Asia/Oral
    {   70, 6961,   35010,  8, }, // Asia/Phnom_Penh
    {   70, 8693,    2563,  9, }, // Asia/Pontianak
    {   70, 4214,   36999,  6, }, // Asia/Pyongyang
    {   70, 5559,    2579,  5, }, // Asia/Qatar
    {   70, 8708,   37005,  8, }, // Asia/Qostanay
    {   70, 4269,   37013,  9, }, // Asia/Qyzylorda
    {   70, 3584,   37022,  4, }, // Asia/Riyadh
    {   70, 4408,    2614,  7, }, // Asia/Sakhalin
    {   70, 7245,    2621,  9, }, // Asia/Samarkand
    {   70, 3279,    2630,  4, }, // Asia/Seoul
    {   70, 1053,   28801,  6, }, // Asia/Shanghai
    {   70, 3300,    2640,  8, }, // Asia/Singapore
    {   70, 4297,   37026, 13, }, // Asia/Srednekolymsk
    {   70, 3263,   37039,  5, }, // Asia/Taipei
    {   70, 4781,    2667,  7, }, // Asia/Tashkent
    {   70, 3915,   35074,  7, }, // Asia/Tbilisi
    {   70, 2772,   35081,  7, }, // Asia/Tehran
    {   70, 1317,    2685,  7, }, // Asia/Thimphu
    {   70, 2821,    2692,  5, }, // Asia/Tokyo
    {   70, 4572,    2697,  5, }, // Asia/Tomsk
    {   70, 1021,   35093, 10, }, // Asia/Ulaanbaatar
    {   70, 1159,    2712,  6, }, // Asia/Urumqi
    {   70, 8722,    2718,  8, }, // Asia/Ust-Nera
    {   70, 6977,   37044,  7, }, // Asia/Vientiane
    {   70, 4710,    2733, 11, }, // Asia/Vladivostok
    {   70, 4828,   37051,  6, }, // Asia/Yakutsk
    {   70, 1235,   28911,  6, }, // Asia/Yangon
    {   70, 3883,    2757, 12, }, // Asia/Yekaterinburg
    {   70, 3747,    2769,  6, }, // Asia/Yerevan
    {   70, 3663,   37057, 15, }, // Atlantic/Azores
    {   70, 5671,   37072, 17, }, // Atlantic/Bermuda
    {   70, 6628,   37089, 16, }, // Atlantic/Canary
    {   70, 3727,   37105, 10, }, // Atlantic/Cape_Verde
    {   70, 1395,   37115, 17, }, // Atlantic/Faroe
    {   70, 8736,   37132,  7, }, // Atlantic/Madeira
    {   70, 2748,   37139,  8, }, // Atlantic/Reykjavik
    {   70, 7272,   37147, 13, }, // Atlantic/South_Georgia
    {   70, 6821,   37160, 11, }, // Atlantic/St_Helena
    {   70, 6644,    2911,  6, }, // Atlantic/Stanley
    {   70, 1670,    2917,  8, }, // Australia/Adelaide
    {   70, 1635,   37171,  8, }, // Australia/Brisbane
    {   70, 1799,   37179, 11, }, // Australia/Broken_Hill
    {   70, 1583,    2943,  6, }, // Australia/Darwin
    {   70, 3637,   29108,  4, }, // Australia/Eucla
    {   70, 1516,    2954,  6, }, // Australia/Hobart
    {   70, 8753,    2960,  8, }, // Australia/Lindeman
    {   70, 1547,   37190,  8, }, // Australia/Lord_Howe
    {   70, 1727,   37198,  7, }, // Australia/Melbourne
    {   70, 1762,   37205,  4, }, // Australia/Perth
    {   70, 1463,   37209,  5, }, // Australia/Sydney
    {   70, 6441,    2993,  9, }, // Europe/Amsterdam
    {   70, 6216,   29143,  6, }, // Europe/Andorra
    {   70, 3620,    3009,  8, }, // Europe/Astrakhan
    {   70, 2300,   35269,  5, }, // Europe/Athens
    {   70, 6336,    3021,  7, }, // Europe/Belgrade
    {   70, 4757,    3028,  6, }, // Europe/Berlin
    {   70, 6488,    3034, 10, }, // Europe/Bratislava
    {   70, 2167,   37214,  7, }, // Europe/Brussels
    {   70, 3928,   37221,  7, }, // Europe/Bucharest
    {   70, 3806,   37228,  8, }, // Europe/Budapest
    {   70, 8772,   37236,  8, }, // Europe/Busingen
    {   70, 2583,    3091,  7, }, // Europe/Chisinau
    {   70, 6289,   35320, 10, }, // Europe/Copenhagen
    {   70, 2338,   37244,  6, }, // Europe/Dublin
    {   70, 6307,    3114,  9, }, // Europe/Gibraltar
    {   70, 8788,   37250,  6, }, // Europe/Guernsey
    {   70, 6612,   37256,  8, }, // Europe/Helsinki
    {   70, 8804,   37264, 10, }, // Europe/Isle_of_Man
    {   70, 1130,   35352,  8, }, // Europe/Istanbul
    {   70, 8823,   37274,  6, }, // Europe/Jersey
    {   70, 3979,    3158, 11, }, // Europe/Kaliningrad
    {   70, 8929,    3169,  5, }, // Europe/Kirov
    {   70, 2527,    3174,  4, }, // Europe/Kyiv
    {   70, 3213,   35376,  7, }, // Europe/Lisbon
    {   70, 6506,   29300,  7, }, // Europe/Ljubljana
    {   70, 2501,    3195,  6, }, // Europe/London
    {   70, 6365,   29313, 10, }, // Europe/Luxembourg
    {   70, 4892,    3212,  6, }, // Europe/Madrid
    {   70, 6397,    3218,  5, }, // Europe/Malta
    {   70, 6582,    3223,  9, }, // Europe/Mariehamn
    {   70, 3693,    3232,  5, }, // Europe/Minsk
    {   70, 6410,    3237,  6, }, // Europe/Monaco
    {   70, 3537,    3243,  6, }, // Europe/Moscow
    {   70, 6458,    3249,  4, }, // Europe/Oslo
    {   70, 4284,    3253,  5, }, // Europe/Paris
    {   70, 6424,    3258,  9, }, // Europe/Podgorica
    {   70, 6275,    3267,  5, }, // Europe/Prague
    {   70, 8837,    3272,  4, }, // Europe/Riga
    {   70, 6324,    3276,  3, }, // Europe/Rome
    {   70, 4331,    3279,  6, }, // Europe/Samara
    {   70, 6470,   35413, 10, }, // Europe/San_Marino
    {   70, 6245,    3295,  7, }, // Europe/Sarajevo
    {   70, 4451,    3302,  7, }, // Europe/Saratov
    {   70, 8849,    3309, 10, }, // Europe/Simferopol
    {   70, 6383,   37280,  6, }, // Europe/Skopje
    {   70, 6599,   37286,  5, }, // Europe/Sofia
    {   70, 6523,   35453,  8, }, // Europe/Stockholm
    {   70, 8867,   35461,  5, }, // Europe/Tallinn
    {   70, 6202,    3343,  6, }, // Europe/Tirane
    {   70, 8882,   37291,  8, }, // Europe/Ulyanovsk
    {   70, 6352,    3358,  5, }, // Europe/Vaduz
    {   70, 6567,    3363,  7, }, // Europe/Vatican
    {   70, 6231,   37299,  5, }, // Europe/Vienna
    {   70, 8899,   37304,  6, }, // Europe/Vilnius
    {   70, 4727,    3381,  9, }, // Europe/Volgograd
    {   70, 3190,    3390,  7, }, // Europe/Warsaw
    {   70, 6261,    3397,  6, }, // Europe/Zagreb
    {   70, 6540,   37310,  5, }, // Europe/Zurich
    {   70, 5078,   35493, 12, }, // Indian/Antananarivo
    {   70, 6947,    3421,  5, }, // Indian/Chagos
    {   70, 6046,   37315,  9, }, // Indian/Christmas
    {   70, 6063,   37324, 16, }, // Indian/Cocos
    {   70, 5029,   37340, 16, }, // Indian/Comoro
    {   70, 6661,    3460,  8, }, // Indian/Kerguelen
    {   70, 7260,    3468,  3, }, // Indian/Mahe
    {   70, 7050,   35520,  7, }, // Indian/Maldives
    {   70, 4069,   37356,  8, }, // Indian/Mauritius
    {   70, 5098,   37364,  5, }, // Indian/Mayotte
    {   70, 7211,   37369,  7, }, // Indian/Reunion
    {   70, 4422,   37376,  4, }, // Pacific/Apia
    {   70,  932,   35554,  6, }, // Pacific/Auckland
    {   70, 3706,    3508,  8, }, // Pacific/Bougainville
    {   70, 3015,   37380,  5, }, // Pacific/Chatham
    {   70, 3157,   37385,  4, }, // Pacific/Chuuk
    {   70, 2237,   37389, 18, }, // Pacific/Easter
    {   70, 7360,    3536,  5, }, // Pacific/Efate
    {   70, 7327,    3541,  7, }, // Pacific/Fakaofo
    {   70, 3902,   37407,  5, }, // Pacific/Fiji
    {   70, 7343,    3552,  8, }, // Pacific/Funafuti
    {   70, 6678,    3560,  9, }, // Pacific/Galapagos
    {   70, 6696,   37412,  6, }, // Pacific/Gambier
    {   70, 3822,    3595, 11, }, // Pacific/Guadalcanal
    {   70, 6018,    3606,  4, }, // Pacific/Guam
    {   70, 2723,   35598,  8, }, // Pacific/Honolulu
    {   70, 3049,   37418,  9, }, // Pacific/Kanton
    {   70, 3998,    3616, 10, }, // Pacific/Kiritimati
    {   70, 7019,   37427,  6, }, // Pacific/Kosrae
    {   70, 2842,   37433,  9, }, // Pacific/Kwajalein
    {   70, 4877,   37442,  7, }, // Pacific/Majuro
    {   70, 4051,   37449, 16, }, // Pacific/Marquesas
    {   70, 8914,   37465,  6, }, // Pacific/Midway
    {   70, 7084,    3680,  5, }, // Pacific/Nauru
    {   70, 7132,    3685,  4, }, // Pacific/Niue
    {   70, 4168,    3689,  7, }, // Pacific/Norfolk
    {   70, 7098,    3696,  5, }, // Pacific/Noumea
    {   70, 3126,   35642,  9, }, // Pacific/Pago_Pago
    {   70, 7155,    3710,  5, }, // Pacific/Palau
    {   70, 7194,    3715,  7, }, // Pacific/Pitcairn
    {   70, 3096,   37471,  6, }, // Pacific/Pohnpei
    {   70, 4807,   37477, 11, }, // Pacific/Port_Moresby
    {   70, 6076,    3739,  9, }, // Pacific/Rarotonga
    {   70, 6031,   29803,  6, }, // Pacific/Saipan
    {   70, 7312,    3754,  5, }, // Pacific/Tahiti
    {   70, 6712,   37488,  6, }, // Pacific/Tarawa
    {   70, 4583,    3765,  9, }, // Pacific/Tongatapu
    {   70, 7392,   37494,  4, }, // Pacific/Wake
    {   70, 7405,   37498,  5, }, // Pacific/Wallis
    {   71, 6788,   37503,  8, }, // Africa/Abidjan Burmese/Myanmar/Myanmar
    {   71, 6760,   37511,  6, }, // Africa/Accra
    {   71, 5059,   37517, 12, }, // Africa/Addis_Ababa
    {   71, 7420,   37529, 15, }, // Africa/Algiers
    {   71,   14,   37544, 10, }, // Africa/Asmara
    {   71,   44,   37554,  7, }, // Africa/Bamako
    {   71, 5257,   37561,  7, }, // Africa/Bangui
    {   71, 6746,   37568,  9, }, // Africa/Banjul
    {   71, 7435,   37577,  7, }, // Africa/Bissau
    {   71, 4957,   37584, 14, }, // Africa/Blantyre
    {   71, 5287,   37598, 11, }, // Africa/Brazzaville
    {   71, 4922,   37609, 12, }, // Africa/Bujumbura
    {   71, 2320,   37621,  8, }, // Africa/Cairo
    {   71, 4117,   37629, 10, }, // Africa/Casablanca
    {   71, 7449,   37639,  7, }, // Africa/Ceuta
    {   71, 6773,   37646,  9, }, // Africa/Conakry
    {   71, 6840,   37655,  6, }, // Africa/Dakar
    {   71, 5130,   37661, 12, }, // Africa/Dar_es_Salaam
    {   71, 5043,   37673,  7, }, // Africa/Djibouti
    {   71, 5243,   37680,  9, }, // Africa/Douala
    {   71, 5166,   37689, 12, }, // Africa/El_Aaiun
    {   71, 6853,   37701, 12, }, // Africa/Freetown
    {   71, 4906,   37713, 11, }, // Africa/Gaborone
    {   71, 5015,   37724,  9, }, // Africa/Harare
    {   71, 4479,   37733, 15, }, // Africa/Johannesburg
    {   71, 4499,   37748,  5, }, // Africa/Juba
    {   71, 5151,   37753,  7, }, // Africa/Kampala
    {   71, 4524,   37760,  8, }, // Africa/Khartoum
    {   71, 4973,   37768,  7, }, // Africa/Kigali
    {   71, 5306,   37775,  8, }, // Africa/Kinshasa
    {   71, 4744,   37783,  7, }, // Africa/Lagos
    {   71, 5336,   37790, 10, }, // Africa/Libreville
    {   71, 6869,   37800,  5, }, // Africa/Lome
    {   71, 5211,   37805,  7, }, // Africa/Luanda
    {   71, 4939,   37812, 12, }, // Africa/Lubumbashi
    {   71, 5001,   37824,  6, }, // Africa/Lusaka
    {   71, 5322,   37830,  7, }, // Africa/Malabo
    {   71, 4987,   37837,  8, }, // Africa/Maputo
    {   71, 5197,   37845,  7, }, // Africa/Maseru
    {   71, 5182,   37852,  8, }, // Africa/Mbabane
    {   71, 5113,   37860, 13, }, // Africa/Mogadishu
    {   71, 7034,   37873, 14, }, // Africa/Monrovia
    {   71, 3853,   37887, 10, }, // Africa/Nairobi
    {   71, 5271,   37897, 10, }, // Africa/Ndjamena
    {   71, 5354,   37907,  6, }, // Africa/Niamey
    {   71, 6803,   37913, 15, }, // Africa/Nouakchott
    {   71, 6727,   37928,  9, }, // Africa/Ouagadougou
    {   71, 5225,   37937, 13, }, // Africa/Porto-Novo
    {   71, 4435,   37950, 10, }, // Africa/Sao_Tome
    {   71, 2866,   37960,  8, }, // Africa/Tripoli
    {   71, 6554,   37968,  6, }, // Africa/Tunis
    {   71, 4152,   37974,  9, }, // Africa/Windhoek
    {   71,  132,   37983,  5, }, // America/Adak
    {   71, 3346,   37988, 11, }, // America/Anchorage
    {   71, 5607,   37999,  7, }, // America/Anguilla
    {   71, 5624,   38006,  8, }, // America/Antigua
    {   71, 4554,   38014, 11, }, // America/Araguaina
    {   71,  166,   38025, 13, }, // America/Argentina/Buenos_Aires
    {   71,   91,   38038,  9, }, // America/Argentina/Catamarca
    {   71,  270,   38047,  8, }, // America/Argentina/Cordoba
    {   71,  441,   38055,  6, }, // America/Argentina/Jujuy
    {   71, 7462,   38061, 10, }, // America/Argentina/La_Rioja
    {   71,  565,   38071,  8, }, // America/Argentina/Mendoza
    {   71, 7489,   38079, 15, }, // America/Argentina/Rio_Gallegos
    {   71, 7520,   38094,  6, }, // America/Argentina/Salta
    {   71, 7544,   38100,  8, }, // America/Argentina/San_Juan
    {   71, 5580,   38108, 10, }, // America/Argentina/San_Luis
    {   71, 7571,   38118,  7, }, // America/Argentina/Tucuman
    {   71, 7597,   38125,  8, }, // America/Argentina/Ushuaia
    {   71, 5640,   38133,  7, }, // America/Aruba
    {   71, 4252,   38140, 11, }, // America/Asuncion
    {   71,  237,   38151, 10, }, // America/Atikokan
    {   71, 3679,   38161,  6, }, // America/Bahia
    {   71, 7623,   38167, 16, }, // America/Bahia_Banderas
    {   71, 5654,   38183, 11, }, // America/Barbados
    {   71, 7646,   38194,  7, }, // America/Belem
    {   71, 5407,   38201,  5, }, // America/Belize
    {   71, 7660,   38206, 15, }, // America/Blanc-Sablon
    {   71, 7681,   38221, 11, }, // America/Boa_Vista
    {   71, 4361,   38232,  8, }, // America/Bogota
    {   71, 7699,   38240,  8, }, // America/Boise
    {   71, 7713,   38248, 16, }, // America/Cambridge_Bay
    {   71, 7735,   38264, 14, }, // America/Campo_Grande
    {   71, 3868,   38278,  6, }, // America/Cancun
    {   71, 4694,   38284,  8, }, // America/Caracas
    {   71, 4345,   38292,  7, }, // America/Cayenne
    {   71, 5496,   38299,  5, }, // America/Cayman
    {   71, 2260,   38304,  8, }, // America/Chicago
    {   71, 7756,   38312, 11, }, // America/Chihuahua
    {   71, 7774,   38323, 18, }, // America/Ciudad_Juarez
    {   71, 5422,   38341, 11, }, // America/Costa_Rica
    {   71, 7796,   38352,  9, }, // America/Creston
    {   71, 3791,   38361,  7, }, // America/Cuiaba
    {   71, 5723,   38368, 10, }, // America/Curacao
    {   71, 7812,   38378, 13, }, // America/Danmarkshavn
    {   71, 7833,   38391,  7, }, // America/Dawson
    {   71, 7848,   38398, 13, }, // America/Dawson_Creek
    {   71,  805,   38411,  7, }, // America/Denver
    {   71, 3465,   38418, 10, }, // America/Detroit
    {   71, 5739,   38428,  9, }, // America/Dominica
    {   71,  893,   38437, 12, }, // America/Edmonton
    {   71, 7869,   38449,  8, }, // America/Eirunepe
    {   71, 5441,   38457, 12, }, // America/El_Salvador
    {   71, 7886,   38469, 15, }, // America/Fort_Nelson
    {   71, 7906,   38484, 12, }, // America/Fortaleza
    {   71, 7924,   38496,  8, }, // America/Glace_Bay
    {   71, 6881,   38504,  8, }, // America/Goose_Bay
    {   71, 4612,   38512, 10, }, // America/Grand_Turk
    {   71, 5770,   38522,  7, }, // America/Grenada
    {   71, 5786,   38529,  9, }, // America/Guadeloupe
    {   71, 3760,   38538,  9, }, // America/Guatemala
    {   71, 4841,   38547, 11, }, // America/Guayaquil
    {   71, 6932,   38558,  9, }, // America/Guyana
    {   71, 1939,   38567,  8, }, // America/Halifax
    {   71, 2281,   38575,  6, }, // America/Havana
    {   71, 7942,   38581, 12, }, // America/Hermosillo
    {   71,  348,   38593, 19, }, // America/Indiana/Indianapolis
    {   71,  481,   38612, 22, }, // America/Indiana/Knox
    {   71, 7961,   38634, 21, }, // America/Indiana/Marengo
    {   71, 7985,   38655, 24, }, // America/Indiana/Petersburg
    {   71, 8012,   38679, 23, }, // America/Indiana/Tell_City
    {   71, 8038,   38702, 17, }, // America/Indiana/Vevay
    {   71, 8060,   38719, 22, }, // America/Indiana/Vincennes
    {   71, 8086,   38741, 22, }, // America/Indiana/Winamac
    {   71, 8110,   38763,  8, }, // America/Inuvik
    {   71,  660,   38771, 10, }, // America/Iqaluit
    {   71, 2799,   38781,  6, }, // America/Jamaica
    {   71, 5380,   38787,  9, }, // America/Juneau
    {   71,  521,   38796,  9, }, // America/Kentucky/Louisville
    {   71, 8125,   38805, 23, }, // America/Kentucky/Monticello
    {   71, 5704,   38828, 14, }, // America/Kralendijk
    {   71, 4376,   38842,  6, }, // America/La_Paz
    {   71, 7169,   38848,  4, }, // America/Lima
    {   71, 3239,   38852, 18, }, // America/Los_Angeles
    {   71, 5932,   38870, 18, }, // America/Lower_Princes
    {   71, 8153,   38888,  7, }, // America/Maceio
    {   71, 8168,   38895,  7, }, // America/Managua
    {   71, 1908,   38902,  9, }, // America/Manaus
    {   71, 5897,   38911,  8, }, // America/Marigot
    {   71, 5805,   38919,  8, }, // America/Martinique
    {   71, 8184,   38927, 12, }, // America/Matamoros
    {   71, 2917,   38939,  8, }, // America/Mazatlan
    {   71, 8202,   38947, 10, }, // America/Menominee
    {   71, 8220,   38957,  6, }, // America/Merida
    {   71, 8235,   38963, 12, }, // America/Metlakatla
    {   71, 2949,   38975, 16, }, // America/Mexico_City
    {   71, 4391,   38991,  9, }, // America/Miquelon
    {   71, 8254,   39000, 10, }, // America/Moncton
    {   71, 8270,   39010,  9, }, // America/Monterrey
    {   71, 4098,   39019, 13, }, // America/Montevideo
    {   71, 5824,   39032, 15, }, // America/Montserrat
    {   71, 5481,   39047,  7, }, // America/Nassau
    {   71, 2379,   39054,  9, }, // America/New_York
    {   71, 8288,   38949,  5, }, // America/Nome
    {   71, 1850,   39063,  8, }, // America/Noronha
    {   71, 8301,   39071, 20, }, // America/North_Dakota/Beulah
    {   71, 8329,   39091, 20, }, // America/North_Dakota/Center
    {   71, 8357,   39111, 25, }, // America/North_Dakota/New_Salem
    {   71,  393,   39136,  4, }, // America/Nuuk
    {   71, 8388,   39140,  8, }, // America/Ojinaga
    {   71, 2356,   39148,  7, }, // America/Panama
    {   71, 6173,   39155, 11, }, // America/Paramaribo
    {   71, 2973,   39166,  6, }, // America/Phoenix
    {   71, 3945,   39172, 18, }, // America/Port-au-Prince
    {   71, 5954,   39190, 19, }, // America/Port_of_Spain
    {   71, 8404,   39209, 17, }, // America/Porto_Velho
    {   71, 5843,   39226, 12, }, // America/Puerto_Rico
    {   71, 4030,   39238, 14, }, // America/Punta_Arenas
    {   71, 8424,   39252, 14, }, // America/Rankin_Inlet
    {   71, 8445,   39266,  8, }, // America/Recife
    {   71, 1995,   39274,  8, }, // America/Regina
    {   71, 8460,   39282, 13, }, // America/Resolute
    {   71,  695,   39295, 13, }, // America/Rio_Branco
    {   71, 8477,   39308,  8, }, // America/Santarem
    {   71, 2201,   39316, 10, }, // America/Santiago
    {   71, 6111,   39326, 17, }, // America/Santo_Domingo
    {   71, 1878,   39343, 10, }, // America/Sao_Paulo
    {   71, 6899,   39353, 19, }, // America/Scoresbysund
    {   71, 8494,   39372,  5, }, // America/Sitka
    {   71, 8508,   39377, 13, }, // America/St_Barthelemy
    {   71, 2061,   39390, 11, }, // America/St_Johns
    {   71, 5863,   39401,  9, }, // America/St_Kitts
    {   71, 5880,   39410, 11, }, // America/St_Lucia
    {   71,  855,   39421, 13, }, // America/St_Thomas
    {   71, 5913,   39434, 13, }, // America/St_Vincent
    {   71, 8530,   39447, 16, }, // America/Swift_Current
    {   71, 5461,   39463, 11, }, // America/Tegucigalpa
    {   71, 5756,   39474,  4, }, // America/Thule
    {   71,  313,   39478,  7, }, // America/Tijuana
    {   71,  608,   39485,  9, }, // America/Toronto
    {   71, 5688,   39494,  8, }, // America/Tortola
    {   71, 2093,   39502,  9, }, // America/Vancouver
    {   71, 2144,   39511, 14, }, // America/Whitehorse
    {   71,  734,   39525,  9, }, // America/Winnipeg
    {   71, 8552,   39534,  7, }, // America/Yakutat
    {   71, 6001,   39541,  4, }, // Antarctica/Casey
    {   71, 6094,   39545,  6, }, // Antarctica/Davis
    {   71, 6133,   39551, 19, }, // Antarctica/DumontDUrville
    {   71, 8568,   39570,  9, }, // Antarctica/Macquarie
    {   71, 7066,   39579,  8, }, // Antarctica/Mawson
    {   71, 7113,   39587,  9, }, // Antarctica/McMurdo
    {   71, 8589,   39596,  6, }, // Antarctica/Palmer
    {   71, 7226,   39602,  7, }, // Antarctica/Rothera
    {   71, 7295,   39609,  7, }, // Antarctica/Syowa
    {   71, 8607,   39616,  6, }, // Antarctica/Troll
    {   71, 7374,   39622, 10, }, // Antarctica/Vostok
    {   71, 1429,   39632, 13, }, // Arctic/Longyearbyen
    {   71, 5570,   39645,  5, }, // Asia/Aden
    {   71, 5395,   39650,  8, }, // Asia/Almaty
    {   71, 3968,   39658,  7, }, // Asia/Amman
    {   71, 5511,   39665,  9, }, // Asia/Anadyr
    {   71, 5523,   39674,  7, }, // Asia/Aqtau
    {   71, 5368,   39681,  9, }, // Asia/Aqtobe
    {   71,  964,   39690, 11, }, // Asia/Ashgabat
    {   71, 8624,   39701,  7, }, // Asia/Atyrau
    {   71, 3607,   39708,  7, }, // Asia/Baghdad
    {   71, 5534,   39715,  7, }, // Asia/Bahrain
    {   71, 3653,   39722,  4, }, // Asia/Baku
    {   71, 4466,   39726,  8, }, // Asia/Bangkok
    {   71, 3571,   39734,  7, }, // Asia/Barnaul
    {   71, 4086,   39741,  6, }, // Asia/Beirut
    {   71, 3778,   39747,  8, }, // Asia/Bishkek
    {   71, 5989,   39755,  9, }, // Asia/Brunei
    {   71, 4601,   39764,  5, }, // Asia/Chita
    {   71, 4511,   39769,  8, }, // Asia/Colombo
    {   71, 4540,   39777,  8, }, // Asia/Damascus
    {   71, 1093,   39785,  5, }, // Asia/Dhaka
    {   71, 6192,   39790,  5, }, // Asia/Dili
    {   71, 3596,   39795,  8, }, // Asia/Dubai
    {   71, 6159,   39803,  9, }, // Asia/Dushanbe
    {   71, 8636,   39812,  9, }, // Asia/Famagusta
    {   71, 8651,   39821,  4, }, // Asia/Gaza
    {   71, 4795,   39825,  7, }, // Asia/Hebron
    {   71, 1259,   39832, 16, }, // Asia/Ho_Chi_Minh
    {   71, 2704,   39848, 10, }, // Asia/Hong_Kong
    {   71, 4771,   39858,  6, }, // Asia/Hovd
    {   71, 4184,   39864,  8, }, // Asia/Irkutsk
    {   71, 7006,   39872,  6, }, // Asia/Jakarta
    {   71, 6992,   39878,  9, }, // Asia/Jayapura
    {   71, 1290,   39887,  9, }, // Asia/Jerusalem
    {   71, 3560,   39896,  7, }, // Asia/Kabul
    {   71, 4316,   39903,  9, }, // Asia/Kamchatka
    {   71, 4239,   39912,  6, }, // Asia/Karachi
    {   71, 1185,   39918,  9, }, // Asia/Kathmandu
    {   71, 8661,   39927, 10, }, // Asia/Khandyga
    {   71,  992,   39937, 11, }, // Asia/Kolkata
    {   71, 4197,   39948, 13, }, // Asia/Krasnoyarsk
    {   71, 4859,   39961, 10, }, // Asia/Kuala_Lumpur
    {   71, 5976,   39971,  6, }, // Asia/Kuching
    {   71, 5547,   39977,  6, }, // Asia/Kuwait
    {   71, 1211,   39983,  6, }, // Asia/Macau
    {   71, 4017,   39989,  8, }, // Asia/Magadan
    {   71, 1349,   39997,  8, }, // Asia/Makassar
    {   71, 7182,   40005,  5, }, // Asia/Manila
    {   71, 6920,   40010, 10, }, // Asia/Muscat
    {   71, 2554,   40020,  9, }, // Asia/Nicosia
    {   71, 8675,   40029, 15, }, // Asia/Novokuznetsk
    {   71, 4135,   40044, 16, }, // Asia/Novosibirsk
    {   71, 4229,   40060,  9, }, // Asia/Omsk
    {   71, 7145,   40069,  8, }, // Asia/Oral
    {   71, 6961,   40077,  8, }, // Asia/Phnom_Penh
    {   71, 8693,   40085, 14, }, // Asia/Pontianak
    {   71, 4214,   40099,  8, }, // Asia/Pyongyang
    {   71, 5559,   38038,  4, }, // Asia/Qatar
    {   71, 8708,   40107,  9, }, // Asia/Qostanay
    {   71, 4269,   40116,  9, }, // Asia/Qyzylorda
    {   71, 3584,   40125,  7, }, // Asia/Riyadh
    {   71, 4408,   40132,  7, }, // Asia/Sakhalin
    {   71, 7245,   40139, 12, }, // Asia/Samarkand
    {   71, 3279,   40151,  6, }, // Asia/Seoul
    {   71, 1053,   40157, 10, }, // Asia/Shanghai
    {   71, 3300,   40167,  8, }, // Asia/Singapore
    {   71, 4297,   40175, 16, }, // Asia/Srednekolymsk
    {   71, 3263,   40191,  7, }, // Asia/Taipei
    {   71, 4781,   40198, 10, }, // Asia/Tashkent
    {   71, 3915,   40208,  8, }, // Asia/Tbilisi
    {   71, 2772,   40216,  7, }, // Asia/Tehran
    {   71, 1317,   40223,  6, }, // Asia/Thimphu
    {   71, 2821,   40229,  7, }, // Asia/Tokyo
    {   71, 4572,   40236, 10, }, // Asia/Tomsk
    {   71, 1021,   40246,  9, }, // Asia/Ulaanbaatar
    {   71, 1159,   40255,  9, }, // Asia/Urumqi
    {   71, 8722,   40264, 11, }, // Asia/Ust-Nera
    {   71, 6977,   40275, 10, }, // Asia/Vientiane
    {   71, 4710,   40285, 16, }, // Asia/Vladivostok
    {   71, 4828,   40301,  9, }, // Asia/Yakutsk
    {   71, 1235,   40310,  7, }, // Asia/Yangon
    {   71, 3883,   40317, 16, }, // Asia/Yekaterinburg
    {   71, 3747,   40333, 10, }, // Asia/Yerevan
    {   71, 3663,   40343, 11, }, // Atlantic/Azores
    {   71, 5671,   40354,  7, }, // Atlantic/Bermuda
    {   71, 6628,   40361,  5, }, // Atlantic/Canary
    {   71, 3727,   40366, 11, }, // Atlantic/Cape_Verde
    {   71, 1395,   40377,  5, }, // Atlantic/Faroe
    {   71, 8736,   40382,  6, }, // Atlantic/Madeira
    {   71, 2748,   40388, 10, }, // Atlantic/Reykjavik
    {   71, 7272,   40398, 15, }, // Atlantic/South_Georgia
    {   71, 6821,   40413, 13, }, // Atlantic/St_Helena
    {   71, 6644,   40426,  6, }, // Atlantic/Stanley
    {   71, 1670,   40432, 11, }, // Australia/Adelaide
    {   71, 1635,   40443, 10, }, // Australia/Brisbane
    {   71, 1799,   40453, 12, }, // Australia/Broken_Hill
    {   71, 1583,   40465,  5, }, // Australia/Darwin
    {   71, 3637,   40470,  6, }, // Australia/Eucla
    {   71, 1516,   40476,  9, }, // Australia/Hobart
    {   71, 8753,   40485, 10, }, // Australia/Lindeman
    {   71, 1547,   40495, 12, }, // Australia/Lord_Howe
    {   71, 1727,   40507,  9, }, // Australia/Melbourne
    {   71, 1762,   40516,  5, }, // Australia/Perth
    {   71, 1463,   40521,  6, }, // Australia/Sydney
    {   71, 6441,   40527,  9, }, // Europe/Amsterdam
    {   71, 6216,   40536,  8, }, // Europe/Andorra
    {   71, 3620,   40544, 11, }, // Europe/Astrakhan
    {   71, 2300,   40555,  5, }, // Europe/Athens
    {   71, 6336,   40560,  9, }, // Europe/Belgrade
    {   71, 4757,   40569,  5, }, // Europe/Berlin
    {   71, 6488,   40574, 13, }, // Europe/Bratislava
    {   71, 2167,   40587,  8, }, // Europe/Brussels
    {   71, 3928,   40595,  9, }, // Europe/Bucharest
    {   71, 3806,   40604,  9, }, // Europe/Budapest
    {   71, 8772,   40613, 10, }, // Europe/Busingen
    {   71, 2583,   40623, 11, }, // Europe/Chisinau
    {   71, 6289,   40634, 11, }, // Europe/Copenhagen
    {   71, 2338,   40645,  7, }, // Europe/Dublin
    {   71, 6307,   40652, 12, }, // Europe/Gibraltar
    {   71, 8788,   40664,  7, }, // Europe/Guernsey
    {   71, 6612,   40671,  9, }, // Europe/Helsinki
    {   71, 8804,   40680, 10, }, // Europe/Isle_of_Man
    {   71, 1130,   40690, 11, }, // Europe/Istanbul
    {   71, 8823,   40701,  5, }, // Europe/Jersey
    {   71, 3979,   40706, 11, }, // Europe/Kaliningrad
    {   71, 8929,   40717,  7, }, // Europe/Kirov
    {   71, 2527,   40724,  8, }, // Europe/Kyiv
    {   71, 3213,   40732,  9, }, // Europe/Lisbon
    {   71, 6506,   40741, 11, }, // Europe/Ljubljana
    {   71, 2501,   40752,  6, }, // Europe/London
    {   71, 6365,   40758,  8, }, // Europe/Luxembourg
    {   71, 4892,   40766,  5, }, // Europe/Madrid
    {   71, 6397,   40771,  8, }, // Europe/Malta
    {   71, 6582,   40779, 12, }, // Europe/Mariehamn
    {   71, 3693,   40791,  7, }, // Europe/Minsk
    {   71, 6410,   40798,  8, }, // Europe/Monaco
    {   71, 3537,   40806,  8, }, // Europe/Moscow
    {   71, 6458,   40814,  8, }, // Europe/Oslo
    {   71, 4284,   40822,  5, }, // Europe/Paris
    {   71, 6424,   40827, 11, }, // Europe/Podgorica
    {   71, 6275,   40838,  6, }, // Europe/Prague
    {   71, 8837,   40844,  4, }, // Europe/Riga
    {   71, 6324,   40848,  4, }, // Europe/Rome
    {   71, 4331,   40852,  7, }, // Europe/Samara
    {   71, 6470,   40859, 10, }, // Europe/San_Marino
    {   71, 6245,   40869,  9, }, // Europe/Sarajevo
    {   71, 4451,   40878, 10, }, // Europe/Saratov
    {   71, 8849,   40888, 12, }, // Europe/Simferopol
    {   71, 6383,   40900,  9, }, // Europe/Skopje
    {   71, 6599,   40909,  7, }, // Europe/Sofia
    {   71, 6523,   40916, 10, }, // Europe/Stockholm
    {   71, 8867,   40926,  6, }, // Europe/Tallinn
    {   71, 6202,   40932,  6, }, // Europe/Tirane
    {   71, 8882,   40938, 15, }, // Europe/Ulyanovsk
    {   71, 6352,   40953,  7, }, // Europe/Vaduz
    {   71, 6567,   40960,  7, }, // Europe/Vatican
    {   71, 6231,   40967,  7, }, // Europe/Vienna
    {   71, 8899,   40974, 13, }, // Europe/Vilnius
    {   71, 4727,   40987, 12, }, // Europe/Volgograd
    {   71, 3190,   40999,  5, }, // Europe/Warsaw
    {   71, 6261,   41004, 10, }, // Europe/Zagreb
    {   71, 6540,   41014,  9, }, // Europe/Zurich
    {   71, 5078,   41023, 15, }, // Indian/Antananarivo
    {   71, 6947,   41038,  8, }, // Indian/Chagos
    {   71, 6046,   41046,  8, }, // Indian/Christmas
    {   71, 6063,   41054,  9, }, // Indian/Cocos
    {   71, 5029,   41063,  9, }, // Indian/Comoro
    {   71, 6661,   41072,  7, }, // Indian/Kerguelen
    {   71, 7260,   41079,  4, }, // Indian/Mahe
    {   71, 7050,   41083, 10, }, // Indian/Maldives
    {   71, 4069,   41093,  8, }, // Indian/Mauritius
    {   71, 5098,   41101,  7, }, // Indian/Mayotte
    {   71, 7211,   41108,  9, }, // Indian/Reunion
    {   71, 4422,   41117,  8, }, // Pacific/Apia
    {   71,  932,   41125,  8, }, // Pacific/Auckland
    {   71, 3706,   41133, 14, }, // Pacific/Bougainville
    {   71, 3015,   41147,  7, }, // Pacific/Chatham
    {   71, 3157,   41154,  5, }, // Pacific/Chuuk
    {   71, 2237,   41159,  5, }, // Pacific/Easter
    {   71, 7360,   41164,  6, }, // Pacific/Efate
    {   71, 7327,   41170, 10, }, // Pacific/Fakaofo
    {   71, 3902,   41180,  5, }, // Pacific/Fiji
    {   71, 7343,   41185,  8, }, // Pacific/Funafuti
    {   71, 6678,   41193, 11, }, // Pacific/Galapagos
    {   71, 6696,   41204,  7, }, // Pacific/Gambier
    {   71, 3822,   41211, 10, }, // Pacific/Guadalcanal
    {   71, 6018,   41221,  5, }, // Pacific/Guam
    {   71, 2723,   41226, 10, }, // Pacific/Honolulu
    {   71, 3049,   41236,  9, }, // Pacific/Kanton
    {   71, 3998,   41245,  8, }, // Pacific/Kiritimati
    {   71, 7019,   41253, 10, }, // Pacific/Kosrae
    {   71, 2842,   41263, 10, }, // Pacific/Kwajalein
    {   71, 4877,   41273,  9, }, // Pacific/Majuro
    {   71, 4051,   41282, 10, }, // Pacific/Marquesas
    {   71, 8914,   41292,  8, }, // Pacific/Midway
    {   71, 7084,   41300,  6, }, // Pacific/Nauru
    {   71, 7132,   41306,  6, }, // Pacific/Niue
    {   71, 4168,   41312, 10, }, // Pacific/Norfolk
    {   71, 7098,   41322,  7, }, // Pacific/Noumea
    {   71, 3126,   41329, 10, }, // Pacific/Pago_Pago
    {   71, 7155,   41339,  6, }, // Pacific/Palau
    {   71, 7194,   41345, 10, }, // Pacific/Pitcairn
    {   71, 3096,   41355,  9, }, // Pacific/Pohnpei
    {   71, 4807,   41364, 17, }, // Pacific/Port_Moresby
    {   71, 6076,   41381, 12, }, // Pacific/Rarotonga
    {   71, 6031,   41393,  8, }, // Pacific/Saipan
    {   71, 7312,   41401,  5, }, // Pacific/Tahiti
    {   71, 6712,   41406,  6, }, // Pacific/Tarawa
    {   71, 4583,   41412, 10, }, // Pacific/Tongatapu
    {   71, 7392,   41422,  6, }, // Pacific/Wake
    {   71, 7405,   41428,  7, }, // Pacific/Wallis
    {   72, 6788,   41435,  3, }, // Africa/Abidjan Cantonese/Traditional Han/Hong Kong
    {   72, 6760,   41438,  3, }, // Africa/Accra
    {   72, 5059,   41441,  6, }, // Africa/Addis_Ababa
    {   72, 7420,   41447,  4, }, // Africa/Algiers
    {   72,   14,   41451,  4, }, // Africa/Asmara
    {   72,   44,   41455,  3, }, // Africa/Bamako
    {   72, 5257,   41458,  2, }, // Africa/Bangui
    {   72, 6746,   41460,  2, }, // Africa/Banjul
    {   72, 7435,   41462,  2, }, // Africa/Bissau
    {   72, 4957,   41464,  4, }, // Africa/Blantyre
    {   72, 5287,   41468,  5, }, // Africa/Brazzaville
    {   72, 4922,   41473,  4, }, // Africa/Bujumbura
    {   72, 2320,   41477,  2, }, // Africa/Cairo
    {   72, 4117,   41479,  5, }, // Africa/Casablanca
    {   72, 7449,   41484,  2, }, // Africa/Ceuta
    {   72, 6773,   41486,  4, }, // Africa/Conakry
    {   72, 6840,   41490,  3, }, // Africa/Dakar
    {   72, 5130,   41493,  3, }, // Africa/Dar_es_Salaam
    {   72, 5043,   41496,  3, }, // Africa/Djibouti
    {   72, 5243,   41499,  3, }, // Africa/Douala
    {   72, 5166,   41502,  3, }, // Africa/El_Aaiun
    {   72, 6853,   41505,  3, }, // Africa/Freetown
    {   72, 4906,   41508,  4, }, // Africa/Gaborone
    {   72, 5015,   41512,  3, }, // Africa/Harare
    {   72, 4479,   41515,  5, }, // Africa/Johannesburg
    {   72, 4499,   41520,  2, }, // Africa/Juba
    {   72, 5151,   41522,  3, }, // Africa/Kampala
    {   72, 4524,   41525,  3, }, // Africa/Khartoum
    {   72, 4973,   41528,  3, }, // Africa/Kigali
    {   72, 5306,   41531,  3, }, // Africa/Kinshasa
    {   72, 4744,   41534,  3, }, // Africa/Lagos
    {   72, 5336,   41537,  3, }, // Africa/Libreville
    {   72, 6869,   41540,  2, }, // Africa/Lome
    {   72, 5211,   41542,  3, }, // Africa/Luanda
    {   72, 4939,   41545,  4, }, // Africa/Lubumbashi
    {   72, 5001,   41549,  3, }, // Africa/Lusaka
    {   72, 5322,   41552,  3, }, // Africa/Malabo
    {   72, 4987,   41555,  3, }, // Africa/Maputo
    {   72, 5197,   41558,  3, }, // Africa/Maseru
    {   72, 5182,   41561,  3, }, // Africa/Mbabane
    {   72, 5113,   41564,  4, }, // Africa/Mogadishu
    {   72, 7034,   41568,  4, }, // Africa/Monrovia
    {   72, 3853,   41572,  3, }, // Africa/Nairobi
    {   72, 5271,   41575,  4, }, // Africa/Ndjamena
    {   72, 5354,   41579,  3, }, // Africa/Niamey
    {   72, 6803,   41582,  3, }, // Africa/Nouakchott
    {   72, 6727,   41585,  4, }, // Africa/Ouagadougou
    {   72, 5225,   41589,  4, }, // Africa/Porto-Novo
    {   72, 4435,   41593,  3, }, // Africa/Sao_Tome
    {   72, 2866,   41596,  4, }, // Africa/Tripoli
    {   72, 6554,   41600,  3, }, // Africa/Tunis
    {   72, 4152,   41603,  4, }, // Africa/Windhoek
    {   72,  132,   41607,  3, }, // America/Adak
    {   72, 3346,   41610,  4, }, // America/Anchorage
    {   72, 5607,   41614,  3, }, // America/Anguilla
    {   72, 5624,   41617,  3, }, // America/Antigua
    {   72, 4554,   41620,  4, }, // America/Araguaina
    {   72,  166,   41624,  7, }, // America/Argentina/Buenos_Aires
    {   72,   91,   41631,  4, }, // America/Argentina/Catamarca
    {   72,  270,   41635,  3, }, // America/Argentina/Cordoba
    {   72,  441,   41638,  3, }, // America/Argentina/Jujuy
    {   72, 7462,   41641,  3, }, // America/Argentina/La_Rioja
    {   72,  565,   41644,  3, }, // America/Argentina/Mendoza
    {   72, 7489,   41647,  6, }, // America/Argentina/Rio_Gallegos
    {   72, 7520,   41653,  3, }, // America/Argentina/Salta
    {   72, 7544,   41656,  3, }, // America/Argentina/San_Juan
    {   72, 5580,   41659,  3, }, // America/Argentina/San_Luis
    {   72, 7571,   41662,  3, }, // America/Argentina/Tucuman
    {   72, 7597,   41665,  4, }, // America/Argentina/Ushuaia
    {   72, 5640,   41669,  3, }, // America/Aruba
    {   72, 4252,   41672,  3, }, // America/Asuncion
    {   72,  237,   41675,  4, }, // America/Atikokan
    {   72, 3679,   41679,  3, }, // America/Bahia
    {   72, 7623,   41682,  7, }, // America/Bahia_Banderas
    {   72, 5654,   41689,  3, }, // America/Barbados
    {   72, 7646,   41692,  2, }, // America/Belem
    {   72, 5407,   41694,  3, }, // America/Belize
    {   72, 7660,   41697,  5, }, // America/Blanc-Sablon
    {   72, 7681,   41702,  4, }, // America/Boa_Vista
    {   72, 4361,   41706,  3, }, // America/Bogota
    {   72, 7699,   41709,  2, }, // America/Boise
    {   72, 7713,   41711,  3, }, // America/Cambridge_Bay
    {   72, 7735,   41714,  3, }, // America/Campo_Grande
    {   72, 3868,   41717,  2, }, // America/Cancun
    {   72, 4694,   41719,  4, }, // America/Caracas
    {   72, 4345,   41723,  2, }, // America/Cayenne
    {   72, 5496,   41725,  4, }, // America/Cayman
    {   72, 2260,   41729,  3, }, // America/Chicago
    {   72, 7756,   41732,  3, }, // America/Chihuahua
    {   72, 7774,   41735,  4, }, // America/Ciudad_Juarez
    {   72, 5422,   41739,  5, }, // America/Costa_Rica
    {   72, 7796,   41744,  4, }, // America/Creston
    {   72, 3791,   41748,  3, }, // America/Cuiaba
    {   72, 5723,   41751,  3, }, // America/Curacao
    {   72, 7812,   41754,  4, }, // America/Danmarkshavn
    {   72, 7833,   41758,  2, }, // America/Dawson
    {   72, 7848,   41760,  5, }, // America/Dawson_Creek
    {   72,  805,   41765,  2, }, // America/Denver
    {   72, 3465,   41767,  3, }, // America/Detroit
    {   72, 5739,   41770,  4, }, // America/Dominica
    {   72,  893,   41774,  4, }, // America/Edmonton
    {   72, 7869,   41778,  4, }, // America/Eirunepe
    {   72, 5441,   41782,  4, }, // America/El_Salvador
    {   72, 7886,   41786,  4, }, // America/Fort_Nelson
    {   72, 7906,   41790,  4, }, // America/Fortaleza
    {   72, 7924,   41794,  4, }, // America/Glace_Bay
    {   72, 6881,   41798,  2, }, // America/Goose_Bay
    {   72, 4612,   41800,  4, }, // America/Grand_Turk
    {   72, 5770,   41804,  4, }, // America/Grenada
    {   72, 5786,   41808,  4, }, // America/Guadeloupe
    {   72, 3760,   41812,  4, }, // America/Guatemala
    {   72, 4841,   41816,  4, }, // America/Guayaquil
    {   72, 6932,   41820,  3, }, // America/Guyana
    {   72, 1939,   41823,  5, }, // America/Halifax
    {   72, 2281,   41828,  3, }, // America/Havana
    {   72, 7942,   41831,  4, }, // America/Hermosillo
    {   72,  348,   41835,  7, }, // America/Indiana/Indianapolis
    {   72,  481,   41842,  8, }, // America/Indiana/Knox
    {   72, 7961,   41850,  8, }, // America/Indiana/Marengo
    {   72, 7985,   41858,  8, }, // America/Indiana/Petersburg
    {   72, 8012,   41866,  8, }, // America/Indiana/Tell_City
    {   72, 8038,   41874,  7, }, // America/Indiana/Vevay
    {   72, 8060,   41881,  8, }, // America/Indiana/Vincennes
    {   72, 8086,   41889,  9, }, // America/Indiana/Winamac
    {   72, 8110,   41898,  4, }, // America/Inuvik
    {   72,  660,   41902,  3, }, // America/Iqaluit
    {   72, 2799,   41905,  3, }, // America/Jamaica
    {   72, 5380,   41908,  2, }, // America/Juneau
    {   72,  521,   41910,  5, }, // America/Kentucky/Louisville
    {   72, 8125,   41915,  8, }, // America/Kentucky/Monticello
    {   72, 5704,   41923,  5, }, // America/Kralendijk
    {   72, 4376,   41928,  3, }, // America/La_Paz
    {   72, 7169,   41931,  2, }, // America/Lima
    {   72, 3239,   41933,  3, }, // America/Los_Angeles
    {   72, 5932,   41936,  4, }, // America/Lower_Princes
    {   72, 8153,   41940,  3, }, // America/Maceio
    {   72, 8168,   41943,  3, }, // America/Managua
    {   72, 1908,   41946,  3, }, // America/Manaus
    {   72, 5897,   41949,  4, }, // America/Marigot
    {   72, 5805,   41953,  4, }, // America/Martinique
    {   72, 8184,   41957,  5, }, // America/Matamoros
    {   72, 2917,   41962,  4, }, // America/Mazatlan
    {   72, 8202,   41966,  4, }, // America/Menominee
    {   72, 8220,   41970,  3, }, // America/Merida
    {   72, 8235,   41973,  6, }, // America/Metlakatla
    {   72, 2949,   41979,  4, }, // America/Mexico_City
    {   72, 4391,   41983,  3, }, // America/Miquelon
    {   72, 8254,   41986,  3, }, // America/Moncton
    {   72, 8270,   41989,  3, }, // America/Monterrey
    {   72, 4098,   41992,  4, }, // America/Montevideo
    {   72, 5824,   41996,  3, }, // America/Montserrat
    {   72, 5481,   41999,  2, }, // America/Nassau
    {   72, 2379,   42001,  2, }, // America/New_York
    {   72, 8288,   42003,  2, }, // America/Nome
    {   72, 1850,   42005,  3, }, // America/Noronha
    {   72, 8301,   42008,  8, }, // America/North_Dakota/Beulah
    {   72, 8329,   42016,  7, }, // America/North_Dakota/Center
    {   72, 8357,   42023,  8, }, // America/North_Dakota/New_Salem
    {   72,  393,   42031,  2, }, // America/Nuuk
    {   72, 8388,   42033,  4, }, // America/Ojinaga
    {   72, 2356,   42037,  3, }, // America/Panama
    {   72, 6173,   42040,  5, }, // America/Paramaribo
    {   72, 2973,   42045,  3, }, // America/Phoenix
    {   72, 3945,   42048,  3, }, // America/Port-au-Prince
    {   72, 5954,   42051,  4, }, // America/Port_of_Spain
    {   72, 8404,   42055,  3, }, // America/Porto_Velho
    {   72, 5843,   42058,  4, }, // America/Puerto_Rico
    {   72, 4030,   42062,  6, }, // America/Punta_Arenas
    {   72, 8424,   42068,  3, }, // America/Rankin_Inlet
    {   72, 8445,   42071,  3, }, // America/Recife
    {   72, 1995,   42074,  3, }, // America/Regina
    {   72, 8460,   42077,  4, }, // America/Resolute
    {   72,  695,   42081,  4, }, // America/Rio_Branco
    {   72, 8477,   42085,  3, }, // America/Santarem
    {   72, 2201,   42088,  4, }, // America/Santiago
    {   72, 6111,   42092,  4, }, // America/Santo_Domingo
    {   72, 1878,   42096,  3, }, // America/Sao_Paulo
    {   72, 6899,   42099,  7, }, // America/Scoresbysund
    {   72, 8494,   42106,  3, }, // America/Sitka
    {   72, 8508,   42109,  6, }, // America/St_Barthelemy
    {   72, 2061,   42115,  3, }, // America/St_Johns
    {   72, 5863,   42118,  3, }, // America/St_Kitts
    {   72, 5880,   42121,  4, }, // America/St_Lucia
    {   72,  855,   42125,  4, }, // America/St_Thomas
    {   72, 5913,   42129,  3, }, // America/St_Vincent
    {   72, 8530,   42132,  7, }, // America/Swift_Current
    {   72, 5461,   42139,  5, }, // America/Tegucigalpa
    {   72, 5756,   42144,  2, }, // America/Thule
    {   72,  313,   42146,  3, }, // America/Tijuana
    {   72,  608,   42149,  3, }, // America/Toronto
    {   72, 5688,   42152,  4, }, // America/Tortola
    {   72, 2093,   42156,  3, }, // America/Vancouver
    {   72, 2144,   42159,  4, }, // America/Whitehorse
    {   72,  734,   42163,  3, }, // America/Winnipeg
    {   72, 8552,   42166,  3, }, // America/Yakutat
    {   72, 6001,   42169,  2, }, // Antarctica/Casey
    {   72, 6094,   42171,  3, }, // Antarctica/Davis
    {   72, 6133,   42174,  5, }, // Antarctica/DumontDUrville
    {   72, 8568,   42179,  3, }, // Antarctica/Macquarie
    {   72, 7066,   42182,  2, }, // Antarctica/Mawson
    {   72, 7113,   42184,  4, }, // Antarctica/McMurdo
    {   72, 8589,   42188,  2, }, // Antarctica/Palmer
    {   72, 7226,   42190,  3, }, // Antarctica/Rothera
    {   72, 7295,   42193,  4, }, // Antarctica/Syowa
    {   72, 8607,   42197,  2, }, // Antarctica/Troll
    {   72, 7374,   42199,  4, }, // Antarctica/Vostok
    {   72, 1429,   42203,  5, }, // Arctic/Longyearbyen
    {   72, 5570,   42208,  2, }, // Asia/Aden
    {   72, 5395,   42210,  4, }, // Asia/Almaty
    {   72, 3968,   42214,  2, }, // Asia/Amman
    {   72, 5511,   42216,  3, }, // Asia/Anadyr
    {   72, 5523,   42219,  3, }, // Asia/Aqtau
    {   72, 5368,   42222,  4, }, // Asia/Aqtobe
    {   72,  964,   42226,  5, }, // Asia/Ashgabat
    {   72, 8624,   42231,  3, }, // Asia/Atyrau
    {   72, 3607,   42234,  3, }, // Asia/Baghdad
    {   72, 5534,   42237,  2, }, // Asia/Bahrain
    {   72, 3653,   41750,  2, }, // Asia/Baku
    {   72, 4466,   42239,  2, }, // Asia/Bangkok
    {   72, 3571,   42241,  4, }, // Asia/Barnaul
    {   72, 4086,   42245,  3, }, // Asia/Beirut
    {   72, 3778,   42248,  4, }, // Asia/Bishkek
    {   72, 5989,   42252,  2, }, // Asia/Brunei
    {   72, 4601,   42254,  2, }, // Asia/Chita
    {   72, 4511,   42256,  3, }, // Asia/Colombo
    {   72, 4540,   42259,  4, }, // Asia/Damascus
    {   72, 1093,   42263,  2, }, // Asia/Dhaka
    {   72, 6192,   42265,  2, }, // Asia/Dili
    {   72, 3596,   42267,  2, }, // Asia/Dubai
    {   72, 6159,   42269,  3, }, // Asia/Dushanbe
    {   72, 8636,   42272,  5, }, // Asia/Famagusta
    {   72, 8651,   42277,  2, }, // Asia/Gaza
    {   72, 4795,   42279,  3, }, // Asia/Hebron
    {   72, 1259,   42282,  4, }, // Asia/Ho_Chi_Minh
    {   72, 2704,   42286, 14, }, // Asia/Hong_Kong
    {   72, 4771,   42300,  3, }, // Asia/Hovd
    {   72, 4184,   42303,  5, }, // Asia/Irkutsk
    {   72, 7006,   42308,  3, }, // Asia/Jakarta
    {   72, 6992,   42311,  4, }, // Asia/Jayapura
    {   72, 1290,   42315,  4, }, // Asia/Jerusalem
    {   72, 3560,   42319,  3, }, // Asia/Kabul
    {   72, 4316,   42322,  3, }, // Asia/Kamchatka
    {   72, 4239,   42325,  3, }, // Asia/Karachi
    {   72, 1185,   42328,  4, }, // Asia/Kathmandu
    {   72, 8661,   42332,  3, }, // Asia/Khandyga
    {   72,  992,   42335,  4, }, // Asia/Kolkata
    {   72, 4197,   42339,  8, }, // Asia/Krasnoyarsk
    {   72, 4859,   42347,  3, }, // Asia/Kuala_Lumpur
    {   72, 5976,   42350,  2, }, // Asia/Kuching
    {   72, 5547,   42352,  3, }, // Asia/Kuwait
    {   72, 1211,   42355, 14, }, // Asia/Macau
    {   72, 4017,   42369,  3, }, // Asia/Magadan
    {   72, 1349,   42372,  4, }, // Asia/Makassar
    {   72, 7182,   42376,  3, }, // Asia/Manila
    {   72, 6920,   42379,  4, }, // Asia/Muscat
    {   72, 2554,   42383,  4, }, // Asia/Nicosia
    {   72, 8675,   42387,  6, }, // Asia/Novokuznetsk
    {   72, 4135,   42393,  5, }, // Asia/Novosibirsk
    {   72, 4229,   42398,  4, }, // Asia/Omsk
    {   72, 7145,   42402,  3, }, // Asia/Oral
    {   72, 6961,   42405,  2, }, // Asia/Phnom_Penh
    {   72, 8693,   42407,  2, }, // Asia/Pontianak
    {   72, 4214,   42409,  2, }, // Asia/Pyongyang
    {   72, 5559,   42411,  2, }, // Asia/Qatar
    {   72, 8708,   42413,  4, }, // Asia/Qostanay
    {   72, 4269,   42417,  6, }, // Asia/Qyzylorda
    {   72, 3584,   42423,  3, }, // Asia/Riyadh
    {   72, 4408,   42426,  3, }, // Asia/Sakhalin
    {   72, 7245,   42429,  4, }, // Asia/Samarkand
    {   72, 3279,   42433,  2, }, // Asia/Seoul
    {   72, 1053,   42435,  2, }, // Asia/Shanghai
    {   72, 3300,   42437,  3, }, // Asia/Singapore
    {   72, 4297,   42440,  6, }, // Asia/Srednekolymsk
    {   72, 3263,   42446,  2, }, // Asia/Taipei
    {   72, 4781,   42448,  3, }, // Asia/Tashkent
    {   72, 3915,   42451,  4, }, // Asia/Tbilisi
    {   72, 2772,   42455,  3, }, // Asia/Tehran
    {   72, 1317,   42458,  2, }, // Asia/Thimphu
    {   72, 2821,   42460,  2, }, // Asia/Tokyo
    {   72, 4572,   42462,  4, }, // Asia/Tomsk
    {   72, 1021,   42466,  4, }, // Asia/Ulaanbaatar
    {   72, 1159,   42470,  4, }, // Asia/Urumqi
    {   72, 8722,   42474,  4, }, // Asia/Ust-Nera
    {   72, 6977,   42478,  2, }, // Asia/Vientiane
    {   72, 4710,   42480,  3, }, // Asia/Vladivostok
    {   72, 4828,   42483,  4, }, // Asia/Yakutsk
    {   72, 1235,   42487,  2, }, // Asia/Yangon
    {   72, 3883,   42489,  5, }, // Asia/Yekaterinburg
    {   72, 3747,   42494,  3, }, // Asia/Yerevan
    {   72, 3663,   42497,  5, }, // Atlantic/Azores
    {   72, 5671,   42502,  3, }, // Atlantic/Bermuda
    {   72, 6628,   42505,  3, }, // Atlantic/Canary
    {   72, 3727,   42508,  3, }, // Atlantic/Cape_Verde
    {   72, 1395,   42511,  4, }, // Atlantic/Faroe
    {   72, 8736,   42515,  5, }, // Atlantic/Madeira
    {   72, 2748,   42520,  5, }, // Atlantic/Reykjavik
    {   72, 7272,   42525,  4, }, // Atlantic/South_Georgia
    {   72, 6821,   42529,  5, }, // Atlantic/St_Helena
    {   72, 6644,   42534,  3, }, // Atlantic/Stanley
    {   72, 1670,   42537,  4, }, // Australia/Adelaide
    {   72, 1635,   42541,  4, }, // Australia/Brisbane
    {   72, 1799,   42545,  5, }, // Australia/Broken_Hill
    {   72, 1583,   42550,  3, }, // Australia/Darwin
    {   72, 3637,   42553,  3, }, // Australia/Eucla
    {   72, 1516,   42556,  3, }, // Australia/Hobart
    {   72, 8753,   42559,  3, }, // Australia/Lindeman
    {   72, 1547,   42562,  4, }, // Australia/Lord_Howe
    {   72, 1727,   42566,  3, }, // Australia/Melbourne
    {   72, 1762,   42569,  2, }, // Australia/Perth
    {   72, 1463,   42571,  2, }, // Australia/Sydney
    {   72, 6441,   42573,  5, }, // Europe/Amsterdam
    {   72, 6216,   42578,  3, }, // Europe/Andorra
    {   72, 3620,   42581,  5, }, // Europe/Astrakhan
    {   72, 2300,   42586,  2, }, // Europe/Athens
    {   72, 6336,   42588,  4, }, // Europe/Belgrade
    {   72, 4757,   42592,  2, }, // Europe/Berlin
    {   72, 6488,   42594,  6, }, // Europe/Bratislava
    {   72, 2167,   42600,  4, }, // Europe/Brussels
    {   72, 3928,   42604,  5, }, // Europe/Bucharest
    {   72, 3806,   42609,  4, }, // Europe/Budapest
    {   72, 8772,   42613,  3, }, // Europe/Busingen
    {   72, 2583,   42616,  3, }, // Europe/Chisinau
    {   72, 6289,   42619,  4, }, // Europe/Copenhagen
    {   72, 2338,   42623,  3, }, // Europe/Dublin
    {   72, 6307,   42626,  4, }, // Europe/Gibraltar
    {   72, 8788,   42630,  3, }, // Europe/Guernsey
    {   72, 6612,   42633,  4, }, // Europe/Helsinki
    {   72, 8804,   42637,  2, }, // Europe/Isle_of_Man
    {   72, 1130,   42639,  4, }, // Europe/Istanbul
    {   72, 8823,   42643,  3, }, // Europe/Jersey
    {   72, 3979,   42646,  5, }, // Europe/Kaliningrad
    {   72, 8929,   42651,  3, }, // Europe/Kirov
    {   72, 2527,   42654,  2, }, // Europe/Kyiv
    {   72, 3213,   42656,  3, }, // Europe/Lisbon
    {   72, 6506,   42659,  4, }, // Europe/Ljubljana
    {   72, 2501,   42663,  2, }, // Europe/London
    {   72, 6365,   42665,  3, }, // Europe/Luxembourg
    {   72, 4892,   42668,  3, }, // Europe/Madrid
    {   72, 6397,   42671,  3, }, // Europe/Malta
    {   72, 6582,   42674,  3, }, // Europe/Mariehamn
    {   72, 3693,   42677,  3, }, // Europe/Minsk
    {   72, 6410,   42680,  3, }, // Europe/Monaco
    {   72, 3537,   42683,  3, }, // Europe/Moscow
    {   72, 6458,   42686,  3, }, // Europe/Oslo
    {   72, 4284,   42689,  2, }, // Europe/Paris
    {   72, 6424,   42691,  4, }, // Europe/Podgorica
    {   72, 6275,   42695,  3, }, // Europe/Prague
    {   72, 8837,   42698,  2, }, // Europe/Riga
    {   72, 6324,   42700,  2, }, // Europe/Rome
    {   72, 4331,   42702,  3, }, // Europe/Samara
    {   72, 6470,   42705,  4, }, // Europe/San_Marino
    {   72, 6245,   42709,  4, }, // Europe/Sarajevo
    {   72, 4451,   42713,  4, }, // Europe/Saratov
    {   72, 8849,   42717,  4, }, // Europe/Simferopol
    {   72, 6383,   42721,  4, }, // Europe/Skopje
    {   72, 6599,   42725,  3, }, // Europe/Sofia
    {   72, 6523,   42728,  5, }, // Europe/Stockholm
    {   72, 8867,   42733,  2, }, // Europe/Tallinn
    {   72, 6202,   42735,  3, }, // Europe/Tirane
    {   72, 8882,   42738,  7, }, // Europe/Ulyanovsk
    {   72, 6352,   42745,  3, }, // Europe/Vaduz
    {   72, 6567,   42748,  3, }, // Europe/Vatican
    {   72, 6231,   42751,  3, }, // Europe/Vienna
    {   72, 8899,   42754,  4, }, // Europe/Vilnius
    {   72, 4727,   42758,  5, }, // Europe/Volgograd
    {   72, 3190,   42763,  2, }, // Europe/Warsaw
    {   72, 6261,   42765,  4, }, // Europe/Zagreb
    {   72, 6540,   42769,  3, }, // Europe/Zurich
    {   72, 5078,   42772,  6, }, // Indian/Antananarivo
    {   72, 6947,   42778,  3, }, // Indian/Chagos
    {   72, 6046,   42781,  3, }, // Indian/Christmas
    {   72, 6063,   42784,  5, }, // Indian/Cocos
    {   72, 5029,   42789,  5, }, // Indian/Comoro
    {   72, 6661,   42794,  5, }, // Indian/Kerguelen
    {   72, 7260,   42799,  3, }, // Indian/Mahe
    {   72, 7050,   42802,  4, }, // Indian/Maldives
    {   72, 4069,   42806,  4, }, // Indian/Mauritius
    {   72, 5098,   42810,  4, }, // Indian/Mayotte
    {   72, 7211,   42814,  4, }, // Indian/Reunion
    {   72, 4422,   42818,  3, }, // Pacific/Apia
    {   72,  932,   42821,  3, }, // Pacific/Auckland
    {   72, 3706,   42824,  4, }, // Pacific/Bougainville
    {   72, 3015,   42828,  2, }, // Pacific/Chatham
    {   72, 3157,   42830,  2, }, // Pacific/Chuuk
    {   72, 2237,   42832,  3, }, // Pacific/Easter
    {   72, 7360,   42835,  3, }, // Pacific/Efate
    {   72, 7327,   42838,  3, }, // Pacific/Fakaofo
    {   72, 3902,   42841,  2, }, // Pacific/Fiji
    {   72, 7343,   42843,  4, }, // Pacific/Funafuti
    {   72, 6678,   42847,  6, }, // Pacific/Galapagos
    {   72, 6696,   42853,  5, }, // Pacific/Gambier
    {   72, 3822,   42858,  6, }, // Pacific/Guadalcanal
    {   72, 6018,   42864,  2, }, // Pacific/Guam
    {   72, 2723,   42866,  3, }, // Pacific/Honolulu
    {   72, 3049,   42869,  5, }, // Pacific/Kanton
    {   72, 3998,   42874,  6, }, // Pacific/Kiritimati
    {   72, 7019,   42880,  3, }, // Pacific/Kosrae
    {   72, 2842,   42883,  4, }, // Pacific/Kwajalein
    {   72, 4877,   42887,  3, }, // Pacific/Majuro
    {   72, 4051,   42890,  5, }, // Pacific/Marquesas
    {   72, 8914,   42895,  3, }, // Pacific/Midway
    {   72, 7084,   42898,  2, }, // Pacific/Nauru
    {   72, 7132,   42900,  3, }, // Pacific/Niue
    {   72, 4168,   42903,  3, }, // Pacific/Norfolk
    {   72, 7098,   42906,  3, }, // Pacific/Noumea
    {   72, 3126,   42909,  4, }, // Pacific/Pago_Pago
    {   72, 7155,   42913,  2, }, // Pacific/Palau
    {   72, 7194,   42915,  5, }, // Pacific/Pitcairn
    {   72, 3096,   42920,  3, }, // Pacific/Pohnpei
    {   72, 4807,   42923,  4, }, // Pacific/Port_Moresby
    {   72, 6076,   42927,  4, }, // Pacific/Rarotonga
    {   72, 6031,   42931,  2, }, // Pacific/Saipan
    {   72, 7312,   42933,  3, }, // Pacific/Tahiti
    {   72, 6712,   42936,  3, }, // Pacific/Tarawa
    {   72, 4583,   42939,  5, }, // Pacific/Tongatapu
    {   72, 7392,   42944,  2, }, // Pacific/Wake
    {   72, 7405,   42946,  3, }, // Pacific/Wallis
    {   73, 6788,   42949,  3, }, // Africa/Abidjan Cantonese/Simplified Han/China
    {   73, 5059,   42952,  6, }, // Africa/Addis_Ababa
    {   73, 7420,   42958,  4, }, // Africa/Algiers
    {   73,   14,   42962,  4, }, // Africa/Asmara
    {   73,   44,   42966,  3, }, // Africa/Bamako
    {   73, 7435,   42969,  2, }, // Africa/Bissau
    {   73, 4957,   42971,  4, }, // Africa/Blantyre
    {   73, 5287,   42975,  5, }, // Africa/Brazzaville
    {   73, 2320,   42980,  2, }, // Africa/Cairo
    {   73, 4117,   42982,  5, }, // Africa/Casablanca
    {   73, 7449,   42987,  2, }, // Africa/Ceuta
    {   73, 6840,   42989,  3, }, // Africa/Dakar
    {   73, 5130,   42992,  3, }, // Africa/Dar_es_Salaam
    {   73, 4479,   42995,  5, }, // Africa/Johannesburg
    {   73, 5211,   43000,  3, }, // Africa/Luanda
    {   73, 4939,   43003,  4, }, // Africa/Lubumbashi
    {   73, 5322,   43007,  3, }, // Africa/Malabo
    {   73, 4987,   43010,  3, }, // Africa/Maputo
    {   73, 5197,   43013,  3, }, // Africa/Maseru
    {   73, 7034,   43016,  4, }, // Africa/Monrovia
    {   73, 5271,   43020,  4, }, // Africa/Ndjamena
    {   73, 5354,   43024,  3, }, // Africa/Niamey
    {   73, 6803,   43027,  3, }, // Africa/Nouakchott
    {   73, 5225,   43030,  4, }, // Africa/Porto-Novo
    {   73, 4435,   43034,  3, }, // Africa/Sao_Tome
    {   73, 4152,   43037,  4, }, // Africa/Windhoek
    {   73,  132,   43041,  3, }, // America/Adak
    {   73,  166,   43044,  7, }, // America/Argentina/Buenos_Aires
    {   73,   91,   43051,  4, }, // America/Argentina/Catamarca
    {   73,  270,   43055,  3, }, // America/Argentina/Cordoba
    {   73,  565,   43058,  3, }, // America/Argentina/Mendoza
    {   73, 7489,   43061,  6, }, // America/Argentina/Rio_Gallegos
    {   73, 7520,   43067,  3, }, // America/Argentina/Salta
    {   73, 7544,   43070,  3, }, // America/Argentina/San_Juan
    {   73, 5580,   43073,  3, }, // America/Argentina/San_Luis
    {   73, 7571,   43076,  3, }, // America/Argentina/Tucuman
    {   73, 7597,   43079,  4, }, // America/Argentina/Ushuaia
    {   73, 4252,   43083,  3, }, // America/Asuncion
    {   73, 7623,   43086,  7, }, // America/Bahia_Banderas
    {   73, 5654,   43093,  3, }, // America/Barbados
    {   73, 7646,   43096,  2, }, // America/Belem
    {   73, 5407,   43098,  3, }, // America/Belize
    {   73, 7660,   43101,  5, }, // America/Blanc-Sablon
    {   73, 7681,   43106,  4, }, // America/Boa_Vista
    {   73, 7713,   43110,  3, }, // America/Cambridge_Bay
    {   73, 7735,   43113,  3, }, // America/Campo_Grande
    {   73, 4345,   43116,  2, }, // America/Cayenne
    {   73, 5496,   43118,  4, }, // America/Cayman
    {   73, 7756,   43122,  3, }, // America/Chihuahua
    {   73, 7774,   43125,  4, }, // America/Ciudad_Juarez
    {   73, 7796,   43129,  4, }, // America/Creston
    {   73, 5723,   43133,  3, }, // America/Curacao
    {   73, 7812,   43136,  4, }, // America/Danmarkshavn
    {   73, 7869,   43140,  4, }, // America/Eirunepe
    {   73, 5441,   43144,  4, }, // America/El_Salvador
    {   73, 7886,   43148,  4, }, // America/Fort_Nelson
    {   73, 7924,   43152,  4, }, // America/Glace_Bay
    {   73, 6881,   43156,  2, }, // America/Goose_Bay
    {   73, 4612,   43158,  4, }, // America/Grand_Turk
    {   73, 5770,   43162,  4, }, // America/Grenada
    {   73, 3760,   43166,  4, }, // America/Guatemala
    {   73, 4841,   43170,  4, }, // America/Guayaquil
    {   73, 6932,   43174,  3, }, // America/Guyana
    {   73, 7942,   43177,  4, }, // America/Hermosillo
    {   73,  481,   43181,  8, }, // America/Indiana/Knox
    {   73, 7961,   43189,  8, }, // America/Indiana/Marengo
    {   73, 8012,   43197,  8, }, // America/Indiana/Tell_City
    {   73, 8038,   43205,  7, }, // America/Indiana/Vevay
    {   73, 8060,   43212,  8, }, // America/Indiana/Vincennes
    {   73, 8086,   43220,  9, }, // America/Indiana/Winamac
    {   73, 8110,   43229,  4, }, // America/Inuvik
    {   73, 2799,   43233,  3, }, // America/Jamaica
    {   73, 5380,   43236,  2, }, // America/Juneau
    {   73,  521,   43238,  5, }, // America/Kentucky/Louisville
    {   73, 8125,   43243,  8, }, // America/Kentucky/Monticello
    {   73, 5704,   43251,  5, }, // America/Kralendijk
    {   73, 7169,   43256,  2, }, // America/Lima
    {   73, 3239,   43258,  3, }, // America/Los_Angeles
    {   73, 5932,   43261,  4, }, // America/Lower_Princes
    {   73, 8153,   43265,  3, }, // America/Maceio
    {   73, 8168,   43268,  3, }, // America/Managua
    {   73, 1908,   43271,  3, }, // America/Manaus
    {   73, 5897,   43274,  4, }, // America/Marigot
    {   73, 5805,   43278,  4, }, // America/Martinique
    {   73, 8184,   43282,  5, }, // America/Matamoros
    {   73, 2917,   43287,  4, }, // America/Mazatlan
    {   73, 8202,   43291,  4, }, // America/Menominee
    {   73, 8220,   43295,  3, }, // America/Merida
    {   73, 4391,   43298,  3, }, // America/Miquelon
    {   73, 8254,   43301,  3, }, // America/Moncton
    {   73, 4098,   43304,  4, }, // America/Montevideo
    {   73, 5824,   43308,  3, }, // America/Montserrat
    {   73, 2379,   43311,  2, }, // America/New_York
    {   73, 8288,   43313,  2, }, // America/Nome
    {   73, 1850,   43315,  3, }, // America/Noronha
    {   73, 8301,   43318,  8, }, // America/North_Dakota/Beulah
    {   73, 8329,   43326,  7, }, // America/North_Dakota/Center
    {   73, 8357,   43333,  8, }, // America/North_Dakota/New_Salem
    {   73, 8388,   43341,  4, }, // America/Ojinaga
    {   73, 2356,   43345,  3, }, // America/Panama
    {   73, 6173,   43348,  5, }, // America/Paramaribo
    {   73, 2973,   43353,  3, }, // America/Phoenix
    {   73, 8404,   43356,  3, }, // America/Porto_Velho
    {   73, 4030,   43359,  6, }, // America/Punta_Arenas
    {   73, 8424,   43365,  3, }, // America/Rankin_Inlet
    {   73, 1995,   43368,  3, }, // America/Regina
    {   73, 8460,   43371,  4, }, // America/Resolute
    {   73,  695,   43375,  4, }, // America/Rio_Branco
    {   73, 8477,   43379,  3, }, // America/Santarem
    {   73, 2201,   43382,  4, }, // America/Santiago
    {   73, 6111,   43386,  4, }, // America/Santo_Domingo
    {   73, 1878,   43390,  3, }, // America/Sao_Paulo
    {   73, 6899,   43393,  7, }, // America/Scoresbysund
    {   73, 8494,   43400,  3, }, // America/Sitka
    {   73, 8508,   43403,  6, }, // America/St_Barthelemy
    {   73, 2061,   43409,  3, }, // America/St_Johns
    {   73, 5863,   43412,  3, }, // America/St_Kitts
    {   73, 5880,   43415,  4, }, // America/St_Lucia
    {   73,  855,   43419,  4, }, // America/St_Thomas
    {   73, 5913,   43423,  3, }, // America/St_Vincent
    {   73, 8530,   43426,  7, }, // America/Swift_Current
    {   73,  313,   43433,  3, }, // America/Tijuana
    {   73,  608,   43436,  3, }, // America/Toronto
    {   73, 5688,   43439,  4, }, // America/Tortola
    {   73, 2093,   43443,  3, }, // America/Vancouver
    {   73, 2144,   43446,  4, }, // America/Whitehorse
    {   73,  734,   43450,  3, }, // America/Winnipeg
    {   73, 8552,   43453,  3, }, // America/Yakutat
    {   73, 6001,   43456,  2, }, // Antarctica/Casey
    {   73, 6094,   43458,  3, }, // Antarctica/Davis
    {   73, 6133,   43461,  5, }, // Antarctica/DumontDUrville
    {   73, 8568,   43466,  3, }, // Antarctica/Macquarie
    {   73, 7113,   43469,  4, }, // Antarctica/McMurdo
    {   73, 8589,   43473,  2, }, // Antarctica/Palmer
    {   73, 7226,   43475,  3, }, // Antarctica/Rothera
    {   73, 8607,   43478,  2, }, // Antarctica/Troll
    {   73, 5570,   43480,  2, }, // Asia/Aden
    {   73, 5395,   43482,  4, }, // Asia/Almaty
    {   73, 8624,   43486,  3, }, // Asia/Atyrau
    {   73, 3607,   43489,  3, }, // Asia/Baghdad
    {   73, 3653,   43492,  2, }, // Asia/Baku
    {   73, 3571,   43494,  4, }, // Asia/Barnaul
    {   73, 4086,   43498,  3, }, // Asia/Beirut
    {   73, 3778,   43501,  4, }, // Asia/Bishkek
    {   73, 5989,   43505,  2, }, // Asia/Brunei
    {   73, 4511,   43507,  3, }, // Asia/Colombo
    {   73, 4540,   43510,  4, }, // Asia/Damascus
    {   73, 1093,   43514,  2, }, // Asia/Dhaka
    {   73, 6159,   43516,  3, }, // Asia/Dushanbe
    {   73, 8636,   43519,  5, }, // Asia/Famagusta
    {   73, 8651,   43524,  2, }, // Asia/Gaza
    {   73, 2704,   43526, 14, }, // Asia/Hong_Kong
    {   73, 4184,   43540,  5, }, // Asia/Irkutsk
    {   73, 7006,   43545,  3, }, // Asia/Jakarta
    {   73, 6992,   43548,  4, }, // Asia/Jayapura
    {   73, 3560,   43552,  3, }, // Asia/Kabul
    {   73, 1185,   43555,  4, }, // Asia/Kathmandu
    {   73,  992,   43559,  4, }, // Asia/Kolkata
    {   73, 4197,   43563,  8, }, // Asia/Krasnoyarsk
    {   73, 5976,   43571,  2, }, // Asia/Kuching
    {   73, 1211,   43573, 14, }, // Asia/Macau
    {   73, 4017,   43587,  3, }, // Asia/Magadan
    {   73, 1349,   43590,  4, }, // Asia/Makassar
    {   73, 7182,   43594,  3, }, // Asia/Manila
    {   73, 6920,   43597,  4, }, // Asia/Muscat
    {   73, 2554,   43601,  4, }, // Asia/Nicosia
    {   73, 8675,   43605,  6, }, // Asia/Novokuznetsk
    {   73, 4135,   43611,  5, }, // Asia/Novosibirsk
    {   73, 7145,   43616,  3, }, // Asia/Oral
    {   73, 6961,   43619,  2, }, // Asia/Phnom_Penh
    {   73, 5559,   43621,  2, }, // Asia/Qatar
    {   73, 4269,   43623,  6, }, // Asia/Qyzylorda
    {   73, 4408,   43629,  3, }, // Asia/Sakhalin
    {   73, 7245,   43632,  4, }, // Asia/Samarkand
    {   73, 3279,   43636,  2, }, // Asia/Seoul
    {   73, 2772,   43638,  3, }, // Asia/Tehran
    {   73, 2821,   43641,  2, }, // Asia/Tokyo
    {   73, 1021,   43643,  4, }, // Asia/Ulaanbaatar
    {   73, 1159,   43647,  4, }, // Asia/Urumqi
    {   73, 8722,   43651,  4, }, // Asia/Ust-Nera
    {   73, 4710,   43655,  3, }, // Asia/Vladivostok
    {   73, 4828,   43658,  4, }, // Asia/Yakutsk
    {   73, 3883,   43662,  5, }, // Asia/Yekaterinburg
    {   73, 3747,   43667,  3, }, // Asia/Yerevan
    {   73, 3663,   43670,  5, }, // Atlantic/Azores
    {   73, 5671,   43675,  3, }, // Atlantic/Bermuda
    {   73, 6628,   43678,  3, }, // Atlantic/Canary
    {   73, 3727,   43681,  3, }, // Atlantic/Cape_Verde
    {   73, 1395,   43684,  4, }, // Atlantic/Faroe
    {   73, 8736,   43688,  5, }, // Atlantic/Madeira
    {   73, 2748,   43693,  5, }, // Atlantic/Reykjavik
    {   73, 7272,   43698,  4, }, // Atlantic/South_Georgia
    {   73, 6821,   43702,  5, }, // Atlantic/St_Helena
    {   73, 1799,   43707,  5, }, // Australia/Broken_Hill
    {   73, 1583,   43712,  3, }, // Australia/Darwin
    {   73, 1547,   43715,  4, }, // Australia/Lord_Howe
    {   73, 1727,   43719,  3, }, // Australia/Melbourne
    {   73, 6216,   43722,  3, }, // Europe/Andorra
    {   73, 6336,   43725,  4, }, // Europe/Belgrade
    {   73, 2167,   43729,  4, }, // Europe/Brussels
    {   73, 3806,   43733,  4, }, // Europe/Budapest
    {   73, 2338,   42623,  3, }, // Europe/Dublin
    {   73, 6307,   43737,  4, }, // Europe/Gibraltar
    {   73, 8788,   43741,  3, }, // Europe/Guernsey
    {   73, 6612,   43744,  4, }, // Europe/Helsinki
    {   73, 8804,   43748,  2, }, // Europe/Isle_of_Man
    {   73, 8823,   43750,  3, }, // Europe/Jersey
    {   73, 3979,   43753,  5, }, // Europe/Kaliningrad
    {   73, 2527,   43758,  2, }, // Europe/Kyiv
    {   73, 6506,   43760,  4, }, // Europe/Ljubljana
    {   73, 2501,   43764,  2, }, // Europe/London
    {   73, 6365,   43766,  3, }, // Europe/Luxembourg
    {   73, 4892,   43769,  3, }, // Europe/Madrid
    {   73, 6397,   43772,  3, }, // Europe/Malta
    {   73, 6582,   43775,  3, }, // Europe/Mariehamn
    {   73, 6410,   43778,  3, }, // Europe/Monaco
    {   73, 6458,   43781,  3, }, // Europe/Oslo
    {   73, 6324,   43784,  2, }, // Europe/Rome
    {   73, 4331,   43786,  3, }, // Europe/Samara
    {   73, 6470,   43789,  4, }, // Europe/San_Marino
    {   73, 4451,   43793,  4, }, // Europe/Saratov
    {   73, 6599,   43797,  3, }, // Europe/Sofia
    {   73, 6523,   43800,  5, }, // Europe/Stockholm
    {   73, 8882,   43805,  7, }, // Europe/Ulyanovsk
    {   73, 6352,   43812,  3, }, // Europe/Vaduz
    {   73, 6567,   43815,  3, }, // Europe/Vatican
    {   73, 6231,   43818,  3, }, // Europe/Vienna
    {   73, 8899,   43821,  4, }, // Europe/Vilnius
    {   73, 4727,   43825,  5, }, // Europe/Volgograd
    {   73, 3190,   43830,  2, }, // Europe/Warsaw
    {   73, 6540,   43832,  3, }, // Europe/Zurich
    {   73, 6046,   43835,  3, }, // Indian/Christmas
    {   73, 6063,   43838,  5, }, // Indian/Cocos
    {   73, 5029,   43843,  5, }, // Indian/Comoro
    {   73, 6661,   43848,  5, }, // Indian/Kerguelen
    {   73, 7260,   43853,  3, }, // Indian/Mahe
    {   73, 7050,   43856,  4, }, // Indian/Maldives
    {   73, 5098,   43860,  4, }, // Indian/Mayotte
    {   73, 7211,   43864,  4, }, // Indian/Reunion
    {   73, 4422,   43868,  3, }, // Pacific/Apia
    {   73,  932,   43871,  3, }, // Pacific/Auckland
    {   73, 3706,   43874,  4, }, // Pacific/Bougainville
    {   73, 2237,   43878,  3, }, // Pacific/Easter
    {   73, 3902,   43881,  2, }, // Pacific/Fiji
    {   73, 6678,   43883,  6, }, // Pacific/Galapagos
    {   73, 6696,   43889,  5, }, // Pacific/Gambier
    {   73, 3822,   43894,  6, }, // Pacific/Guadalcanal
    {   73, 6018,   43900,  2, }, // Pacific/Guam
    {   73, 3049,   43902,  5, }, // Pacific/Kanton
    {   73, 3998,   43907,  6, }, // Pacific/Kiritimati
    {   73, 2842,   43913,  4, }, // Pacific/Kwajalein
    {   73, 4877,   43917,  3, }, // Pacific/Majuro
    {   73, 4051,   43920,  5, }, // Pacific/Marquesas
    {   73, 8914,   43925,  3, }, // Pacific/Midway
    {   73, 7084,   43928,  2, }, // Pacific/Nauru
    {   73, 7132,   43930,  3, }, // Pacific/Niue
    {   73, 4168,   43933,  3, }, // Pacific/Norfolk
    {   73, 7098,   43936,  3, }, // Pacific/Noumea
    {   73, 7194,   43939,  5, }, // Pacific/Pitcairn
    {   73, 3096,   43944,  3, }, // Pacific/Pohnpei
    {   73, 6076,   43947,  4, }, // Pacific/Rarotonga
    {   73, 4583,   43951,  5, }, // Pacific/Tongatapu
    {   75, 5059,    3784, 11, }, // Africa/Addis_Ababa Catalan/Latin/Spain
    {   75, 7420,   43956,  5, }, // Africa/Algiers
    {   75,   14,       0,  6, }, // Africa/Asmara
    {   75, 2320,   43961,  9, }, // Africa/Cairo
    {   75, 5166,   43970,  8, }, // Africa/El_Aaiun
    {   75, 4524,   26372,  7, }, // Africa/Khartoum
    {   75, 5113,   43978, 10, }, // Africa/Mogadishu
    {   75, 7034,   43988,  8, }, // Africa/Monrovia
    {   75, 5271,   43996,  9, }, // Africa/Ndjamena
    {   75, 4435,       6,  8, }, // Africa/Sao_Tome
    {   75, 2866,   20349,  7, }, // Africa/Tripoli
    {   75, 4554,   26427,  9, }, // America/Araguaina
    {   75,  270,   20361,  7, }, // America/Argentina/Cordoba
    {   75, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {   75, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {   75, 4252,      14,  8, }, // America/Asuncion
    {   75,  237,      22,  8, }, // America/Atikokan
    {   75, 7623,      30, 17, }, // America/Bahia_Banderas
    {   75, 7646,   35764,  5, }, // America/Belem
    {   75, 7660,   44017, 12, }, // America/Blanc-Sablon
    {   75, 4361,   44029,  6, }, // America/Bogota
    {   75, 3868,   33182,  6, }, // America/Cancun
    {   75, 4345,   44035,  6, }, // America/Cayenne
    {   75, 5496,   44041,  6, }, // America/Cayman
    {   75, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   75, 3791,   26442,  6, }, // America/Cuiaba
    {   75, 5723,      66,  7, }, // America/Curacao
    {   75, 7869,   26448,  8, }, // America/Eirunepe
    {   75,  481,      73, 13, }, // America/Indiana/Knox
    {   75, 7961,      86, 16, }, // America/Indiana/Marengo
    {   75, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   75, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   75, 8038,     139, 14, }, // America/Indiana/Vevay
    {   75, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   75, 8086,     171, 16, }, // America/Indiana/Winamac
    {   75, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   75, 5932,     207, 22, }, // America/Lower_Princes
    {   75, 8153,   26456,  6, }, // America/Maceio
    {   75, 5805,   20436,  9, }, // America/Martinique
    {   75, 2917,   26471,  8, }, // America/Mazatlan
    {   75, 8220,     229,  6, }, // America/Merida
    {   75, 2949,   44047, 15, }, // America/Mexico_City
    {   75, 2379,   44062,  9, }, // America/New_York
    {   75, 8301,   44071, 23, }, // America/North_Dakota/Beulah
    {   75, 8329,   44094, 23, }, // America/North_Dakota/Center
    {   75, 8357,   44117, 26, }, // America/North_Dakota/New_Salem
    {   75,  393,     298,  4, }, // America/Nuuk
    {   75, 2356,   44143,  6, }, // America/Panama
    {   75, 8477,   26572,  8, }, // America/Santarem
    {   75, 1878,   26580,  9, }, // America/Sao_Paulo
    {   75, 6899,   44149, 12, }, // America/Scoresbysund
    {   75, 8508,   35804, 16, }, // America/St_Barthelemy
    {   75, 2061,   26605, 12, }, // America/St_Johns
    {   75, 5863,   26617, 11, }, // America/St_Kitts
    {   75, 5880,   35832, 11, }, // America/St_Lucia
    {   75,  855,   44161, 12, }, // America/St_Thomas
    {   75, 5913,   26651, 13, }, // America/St_Vincent
    {   75, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   75, 5511,   44173,  6, }, // Asia/Anadyr
    {   75, 5523,   44179,  5, }, // Asia/Aqtau
    {   75, 5368,   44184,  6, }, // Asia/Aqtobe
    {   75,  964,   44190,  7, }, // Asia/Ashgabat
    {   75, 8624,   44197,  6, }, // Asia/Atyrau
    {   75, 3607,    4044,  6, }, // Asia/Baghdad
    {   75, 3653,   20629,  4, }, // Asia/Baku
    {   75, 3571,   44203,  7, }, // Asia/Barnaul
    {   75, 4601,   44210,  5, }, // Asia/Chita
    {   75, 4540,   20639,  6, }, // Asia/Damascus
    {   75, 6159,   44215,  8, }, // Asia/Dushanbe
    {   75, 1259,   44223, 11, }, // Asia/Ho_Chi_Minh
    {   75, 4771,   26703,  5, }, // Asia/Hovd
    {   75, 6992,   44234,  6, }, // Asia/Jayapura
    {   75, 4316,   26708,  9, }, // Asia/Kamchatka
    {   75, 1185,   20712,  8, }, // Asia/Kathmandu
    {   75,  992,   20720,  7, }, // Asia/Kolkata
    {   75, 4197,   26739, 11, }, // Asia/Krasnoyarsk
    {   75, 1211,    4203,  5, }, // Asia/Macau
    {   75, 6920,   35935,  6, }, // Asia/Muscat
    {   75, 2554,   44240,  7, }, // Asia/Nicosia
    {   75, 4135,   44247, 12, }, // Asia/Novosibirsk
    {   75, 8708,   44259,  8, }, // Asia/Qostanay
    {   75, 3584,    7669,  4, }, // Asia/Riyadh
    {   75, 4408,   44267,  8, }, // Asia/Sakhalin
    {   75, 7245,   20768, 10, }, // Asia/Samarkand
    {   75, 3279,   44275,  4, }, // Asia/Seoul
    {   75, 3300,   20782,  8, }, // Asia/Singapore
    {   75, 4297,    7699, 13, }, // Asia/Srednekolymsk
    {   75, 4781,   44279,  8, }, // Asia/Tashkent
    {   75, 2772,    7724,  7, }, // Asia/Tehran
    {   75, 2821,   44287,  6, }, // Asia/Tokyo
    {   75, 1159,   44293,  7, }, // Asia/Urumqi
    {   75, 8722,   44300,  9, }, // Asia/Ust-Nera
    {   75, 4828,   44309,  7, }, // Asia/Yakutsk
    {   75, 1235,     442,  6, }, // Asia/Yangon
    {   75, 3883,   44316, 13, }, // Asia/Yekaterinburg
    {   75, 3663,   44329,  6, }, // Atlantic/Azores
    {   75, 5671,   44335,  8, }, // Atlantic/Bermuda
    {   75, 6628,   44343, 14, }, // Atlantic/Canary
    {   75, 3727,   44357,  8, }, // Atlantic/Cape_Verde
    {   75, 1395,   44365, 11, }, // Atlantic/Faroe
    {   75, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {   75, 7272,   44376, 15, }, // Atlantic/South_Georgia
    {   75, 6821,   36066, 12, }, // Atlantic/St_Helena
    {   75, 3620,   44391,  8, }, // Europe/Astrakhan
    {   75, 2300,   20928,  6, }, // Europe/Athens
    {   75, 6336,   26883,  7, }, // Europe/Belgrade
    {   75, 4757,   20941,  6, }, // Europe/Berlin
    {   75, 2167,   44399, 11, }, // Europe/Brussels
    {   75, 3928,   20955,  8, }, // Europe/Bucharest
    {   75, 6289,   44410, 11, }, // Europe/Copenhagen
    {   75, 2338,   20973,  6, }, // Europe/Dublin
    {   75, 6612,   44421,  8, }, // Europe/Helsinki
    {   75, 8804,    4374,  3, }, // Europe/Isle_of_Man
    {   75, 8929,   44429,  5, }, // Europe/Kirov
    {   75, 2527,   44434,  4, }, // Europe/Kyiv
    {   75, 3213,   21019,  6, }, // Europe/Lisbon
    {   75, 2501,   21034,  7, }, // Europe/London
    {   75, 6365,    4395,  9, }, // Europe/Luxembourg
    {   75, 6410,   44438,  6, }, // Europe/Monaco
    {   75, 3537,   44444,  6, }, // Europe/Moscow
    {   75, 4284,   21062,  5, }, // Europe/Paris
    {   75, 6275,   21067,  5, }, // Europe/Prague
    {   75, 6324,    5946,  4, }, // Europe/Rome
    {   75, 4451,   44450,  7, }, // Europe/Saratov
    {   75, 8849,   44457, 10, }, // Europe/Simferopol
    {   75, 6523,   21086,  8, }, // Europe/Stockholm
    {   75, 6202,   21095,  6, }, // Europe/Tirane
    {   75, 8882,   44467,  9, }, // Europe/Ulyanovsk
    {   75, 6567,   44476,  6, }, // Europe/Vatican
    {   75, 6231,   21112,  5, }, // Europe/Vienna
    {   75, 8899,   44482,  7, }, // Europe/Vilnius
    {   75, 3190,   44489,  8, }, // Europe/Warsaw
    {   75, 6540,   44497,  5, }, // Europe/Zurich
    {   75, 4069,   21134,  7, }, // Indian/Mauritius
    {   75, 7211,   21142,  6, }, // Indian/Reunion
    {   75, 3157,     474,  5, }, // Pacific/Chuuk
    {   75, 2237,   44502, 14, }, // Pacific/Easter
    {   75, 6678,   21159,  9, }, // Pacific/Galapagos
    {   75, 2723,    4493,  8, }, // Pacific/Honolulu
    {   75, 3049,    4501,  9, }, // Pacific/Kanton
    {   75, 4051,   44516,  9, }, // Pacific/Marquesas
    {   75, 7098,    4510,  6, }, // Pacific/Noumea
    {   75, 3096,     479,  7, }, // Pacific/Pohnpei
    {   75, 7312,   44525,  6, }, // Pacific/Tahiti
    {   79,   14,       0,  6, }, // Africa/Asmara Cebuano/Latin/Philippines
    {   79, 4435,       6,  8, }, // Africa/Sao_Tome
    {   79, 4252,      14,  8, }, // America/Asuncion
    {   79,  237,      22,  8, }, // America/Atikokan
    {   79, 7623,      30, 17, }, // America/Bahia_Banderas
    {   79, 3868,      47,  6, }, // America/Cancun
    {   79, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   79, 5723,      66,  7, }, // America/Curacao
    {   79,  481,      73, 13, }, // America/Indiana/Knox
    {   79, 7961,      86, 16, }, // America/Indiana/Marengo
    {   79, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   79, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   79, 8038,     139, 14, }, // America/Indiana/Vevay
    {   79, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   79, 8086,     171, 16, }, // America/Indiana/Winamac
    {   79, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   79, 5932,     207, 22, }, // America/Lower_Princes
    {   79, 8220,     229,  6, }, // America/Merida
    {   79, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   79, 8329,     255, 20, }, // America/North_Dakota/Center
    {   79, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   79,  393,     298,  4, }, // America/Nuuk
    {   79, 6899,     302, 16, }, // America/Scoresbysund
    {   79, 8508,     318, 14, }, // America/St_Barthelemy
    {   79, 2061,     332, 10, }, // America/St_Johns
    {   79, 5863,     342,  9, }, // America/St_Kitts
    {   79, 5880,     351,  9, }, // America/St_Lucia
    {   79,  855,     360, 10, }, // America/St_Thomas
    {   79, 5913,     370, 11, }, // America/St_Vincent
    {   79, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   79, 1259,   44531, 22, }, // Asia/Ho_Chi_Minh
    {   79, 1185,     413,  9, }, // Asia/Kathmandu
    {   79,  992,     422,  7, }, // Asia/Kolkata
    {   79, 1211,     429,  5, }, // Asia/Macau
    {   79, 1235,     442,  6, }, // Asia/Yangon
    {   79, 1395,     448,  5, }, // Atlantic/Faroe
    {   79, 6821,     453, 10, }, // Atlantic/St_Helena
    {   79, 2527,     463,  4, }, // Europe/Kyiv
    {   79, 7211,     467,  7, }, // Indian/Reunion
    {   79, 3157,     474,  5, }, // Pacific/Chuuk
    {   79, 3049,    4501,  9, }, // Pacific/Kanton
    {   79, 3096,     479,  7, }, // Pacific/Pohnpei
    {   80,   14,       0,  6, }, // Africa/Asmara Central Atlas Tamazight/Latin/Morocco
    {   80, 4435,       6,  8, }, // Africa/Sao_Tome
    {   80, 4252,      14,  8, }, // America/Asuncion
    {   80,  237,      22,  8, }, // America/Atikokan
    {   80, 7623,      30, 17, }, // America/Bahia_Banderas
    {   80, 3868,      47,  6, }, // America/Cancun
    {   80, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   80, 5723,      66,  7, }, // America/Curacao
    {   80,  481,      73, 13, }, // America/Indiana/Knox
    {   80, 7961,      86, 16, }, // America/Indiana/Marengo
    {   80, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   80, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   80, 8038,     139, 14, }, // America/Indiana/Vevay
    {   80, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   80, 8086,     171, 16, }, // America/Indiana/Winamac
    {   80, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   80, 5932,     207, 22, }, // America/Lower_Princes
    {   80, 8220,     229,  6, }, // America/Merida
    {   80, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   80, 8329,     255, 20, }, // America/North_Dakota/Center
    {   80, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   80,  393,     298,  4, }, // America/Nuuk
    {   80, 6899,     302, 16, }, // America/Scoresbysund
    {   80, 8508,     318, 14, }, // America/St_Barthelemy
    {   80, 2061,     332, 10, }, // America/St_Johns
    {   80, 5863,     342,  9, }, // America/St_Kitts
    {   80, 5880,     351,  9, }, // America/St_Lucia
    {   80,  855,     360, 10, }, // America/St_Thomas
    {   80, 5913,     370, 11, }, // America/St_Vincent
    {   80, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   80, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   80, 1185,     413,  9, }, // Asia/Kathmandu
    {   80,  992,     422,  7, }, // Asia/Kolkata
    {   80, 1211,     429,  5, }, // Asia/Macau
    {   80, 1235,     442,  6, }, // Asia/Yangon
    {   80, 1395,     448,  5, }, // Atlantic/Faroe
    {   80, 6821,     453, 10, }, // Atlantic/St_Helena
    {   80, 2527,     463,  4, }, // Europe/Kyiv
    {   80, 7211,     467,  7, }, // Indian/Reunion
    {   80, 3157,     474,  5, }, // Pacific/Chuuk
    {   80, 3096,     479,  7, }, // Pacific/Pohnpei
    {   81,   14,       0,  6, }, // Africa/Asmara Central Kurdish/Arabic/Iraq
    {   81, 4435,       6,  8, }, // Africa/Sao_Tome
    {   81, 4252,      14,  8, }, // America/Asuncion
    {   81,  237,      22,  8, }, // America/Atikokan
    {   81, 7623,      30, 17, }, // America/Bahia_Banderas
    {   81, 3868,      47,  6, }, // America/Cancun
    {   81, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   81, 5723,      66,  7, }, // America/Curacao
    {   81,  481,      73, 13, }, // America/Indiana/Knox
    {   81, 7961,      86, 16, }, // America/Indiana/Marengo
    {   81, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   81, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   81, 8038,     139, 14, }, // America/Indiana/Vevay
    {   81, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   81, 8086,     171, 16, }, // America/Indiana/Winamac
    {   81, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   81, 5932,     207, 22, }, // America/Lower_Princes
    {   81, 8220,     229,  6, }, // America/Merida
    {   81, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   81, 8329,     255, 20, }, // America/North_Dakota/Center
    {   81, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   81,  393,     298,  4, }, // America/Nuuk
    {   81, 6899,     302, 16, }, // America/Scoresbysund
    {   81, 8508,     318, 14, }, // America/St_Barthelemy
    {   81, 2061,     332, 10, }, // America/St_Johns
    {   81, 5863,     342,  9, }, // America/St_Kitts
    {   81, 5880,     351,  9, }, // America/St_Lucia
    {   81,  855,     360, 10, }, // America/St_Thomas
    {   81, 5913,     370, 11, }, // America/St_Vincent
    {   81, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   81, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   81, 1185,     413,  9, }, // Asia/Kathmandu
    {   81,  992,     422,  7, }, // Asia/Kolkata
    {   81, 1211,     429,  5, }, // Asia/Macau
    {   81, 1235,     442,  6, }, // Asia/Yangon
    {   81, 1395,     448,  5, }, // Atlantic/Faroe
    {   81, 6821,     453, 10, }, // Atlantic/St_Helena
    {   81, 2527,     463,  4, }, // Europe/Kyiv
    {   81, 7211,     467,  7, }, // Indian/Reunion
    {   81, 3157,     474,  5, }, // Pacific/Chuuk
    {   81, 3096,     479,  7, }, // Pacific/Pohnpei
    {   83, 6788,   44553, 16, }, // Africa/Abidjan Chakma/Chakma/Bangladesh
    {   83, 6760,   44569,  8, }, // Africa/Accra
    {   83, 5059,   44577, 21, }, // Africa/Addis_Ababa
    {   83, 7420,   44598, 20, }, // Africa/Algiers
    {   83,   14,   44618, 16, }, // Africa/Asmara
    {   83,   44,   44634, 12, }, // Africa/Bamako
    {   83, 5257,   44646, 12, }, // Africa/Bangui
    {   83, 6746,   44658, 12, }, // Africa/Banjul
    {   83, 7435,   44670,  8, }, // Africa/Bissau
    {   83, 4957,   44678, 24, }, // Africa/Blantyre
    {   83, 5287,   44702, 18, }, // Africa/Brazzaville
    {   83, 4922,   44720, 24, }, // Africa/Bujumbura
    {   83, 2320,   44744, 10, }, // Africa/Cairo
    {   83, 4117,   44754, 14, }, // Africa/Casablanca
    {   83, 7449,   44768, 10, }, // Africa/Ceuta
    {   83, 6773,   44778, 14, }, // Africa/Conakry
    {   83, 6840,   44792,  8, }, // Africa/Dakar
    {   83, 5130,   44800, 24, }, // Africa/Dar_es_Salaam
    {   83, 5043,   44824, 12, }, // Africa/Djibouti
    {   83, 5243,   44836,  8, }, // Africa/Douala
    {   83, 5166,   44844, 27, }, // Africa/El_Aaiun
    {   83, 6853,   44871, 18, }, // Africa/Freetown
    {   83, 4906,   44889, 18, }, // Africa/Gaborone
    {   83, 5015,   44907,  8, }, // Africa/Harare
    {   83, 4479,   44915, 24, }, // Africa/Johannesburg
    {   83, 4499,   44939,  6, }, // Africa/Juba
    {   83, 5151,   44945, 10, }, // Africa/Kampala
    {   83, 4524,   44955, 14, }, // Africa/Khartoum
    {   83, 4973,   44969, 10, }, // Africa/Kigali
    {   83, 5306,   44979, 12, }, // Africa/Kinshasa
    {   83, 4744,   44991, 10, }, // Africa/Lagos
    {   83, 5336,   45001, 20, }, // Africa/Libreville
    {   83, 6869,   45021,  8, }, // Africa/Lome
    {   83, 5211,   45029, 12, }, // Africa/Luanda
    {   83, 4939,   45041, 18, }, // Africa/Lubumbashi
    {   83, 5001,   45059,  8, }, // Africa/Lusaka
    {   83, 5322,   45067,  8, }, // Africa/Malabo
    {   83, 4987,   45075, 10, }, // Africa/Maputo
    {   83, 5197,   45085, 14, }, // Africa/Maseru
    {   83, 5182,   45099, 16, }, // Africa/Mbabane
    {   83, 5113,   45115, 14, }, // Africa/Mogadishu
    {   83, 7034,   45129, 18, }, // Africa/Monrovia
    {   83, 3853,   45147, 12, }, // Africa/Nairobi
    {   83, 5271,   45159, 18, }, // Africa/Ndjamena
    {   83, 5354,   45177, 10, }, // Africa/Niamey
    {   83, 6803,   45187, 18, }, // Africa/Nouakchott
    {   83, 6727,   45205, 18, }, // Africa/Ouagadougou
    {   83, 5225,   45223, 21, }, // Africa/Porto-Novo
    {   83, 4435,   45244, 15, }, // Africa/Sao_Tome
    {   83, 2866,   45259, 16, }, // Africa/Tripoli
    {   83, 6554,   45275, 16, }, // Africa/Tunis
    {   83, 4152,   45291, 20, }, // Africa/Windhoek
    {   83,  132,   45311,  8, }, // America/Adak
    {   83, 3346,   45319, 20, }, // America/Anchorage
    {   83, 5607,   45339, 20, }, // America/Anguilla
    {   83, 5624,   45359, 22, }, // America/Antigua
    {   83, 4554,   45381, 14, }, // America/Araguaina
    {   83,  166,   45395, 31, }, // America/Argentina/Buenos_Aires
    {   83,   91,   45426, 16, }, // America/Argentina/Catamarca
    {   83,  270,   45442, 14, }, // America/Argentina/Cordoba
    {   83,  441,   45456, 12, }, // America/Argentina/Jujuy
    {   83, 7462,   45468, 13, }, // America/Argentina/La_Rioja
    {   83,  565,   45481, 14, }, // America/Argentina/Mendoza
    {   83, 7489,   45495, 23, }, // America/Argentina/Rio_Gallegos
    {   83, 7520,   45518, 12, }, // America/Argentina/Salta
    {   83, 7544,   45530, 17, }, // America/Argentina/San_Juan
    {   83, 5580,   45547, 17, }, // America/Argentina/San_Luis
    {   83, 7571,   45564, 14, }, // America/Argentina/Tucuman
    {   83, 7597,   45578, 12, }, // America/Argentina/Ushuaia
    {   83, 5640,   45590, 10, }, // America/Aruba
    {   83, 4252,   45600, 22, }, // America/Asuncion
    {   83,  237,   45622, 23, }, // America/Atikokan
    {   83, 3679,   45645,  8, }, // America/Bahia
    {   83, 7623,   45653, 25, }, // America/Bahia_Banderas
    {   83, 5654,   45678, 16, }, // America/Barbados
    {   83, 7646,   45694, 12, }, // America/Belem
    {   83, 5407,   45706, 12, }, // America/Belize
    {   83, 7660,   45718, 27, }, // America/Blanc-Sablon
    {   83, 7681,   45745, 15, }, // America/Boa_Vista
    {   83, 4361,   45760, 10, }, // America/Bogota
    {   83, 7699,   45770, 12, }, // America/Boise
    {   83, 7713,   45782, 25, }, // America/Cambridge_Bay
    {   83, 7735,   45807, 25, }, // America/Campo_Grande
    {   83, 3868,   45832, 18, }, // America/Cancun
    {   83, 4694,   45850, 14, }, // America/Caracas
    {   83, 4345,   45864, 10, }, // America/Cayenne
    {   83, 5496,   45874, 10, }, // America/Cayman
    {   83, 2260,   45884, 10, }, // America/Chicago
    {   83, 7756,   45894, 16, }, // America/Chihuahua
    {   83, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   83, 5422,   45910, 16, }, // America/Costa_Rica
    {   83, 7796,   45926, 18, }, // America/Creston
    {   83, 3791,   45944, 10, }, // America/Cuiaba
    {   83, 5723,   45954, 12, }, // America/Curacao
    {   83, 7812,   45966, 30, }, // America/Danmarkshavn
    {   83, 7833,   45996, 12, }, // America/Dawson
    {   83, 7848,   46008, 25, }, // America/Dawson_Creek
    {   83,  805,   46033, 14, }, // America/Denver
    {   83, 3465,   46047, 20, }, // America/Detroit
    {   83, 5739,   46067, 14, }, // America/Dominica
    {   83,  893,   46081, 24, }, // America/Edmonton
    {   83, 7869,   46105, 16, }, // America/Eirunepe
    {   83, 5441,   46121, 25, }, // America/El_Salvador
    {   83, 7886,   46146, 29, }, // America/Fort_Nelson
    {   83, 7906,   46175, 14, }, // America/Fortaleza
    {   83, 7924,   46189, 15, }, // America/Glace_Bay
    {   83, 6881,   46204, 13, }, // America/Goose_Bay
    {   83, 4612,   46217, 31, }, // America/Grand_Turk
    {   83, 5770,   46248, 12, }, // America/Grenada
    {   83, 5786,   46260, 16, }, // America/Guadeloupe
    {   83, 3760,   46276, 12, }, // America/Guatemala
    {   83, 4841,   46288, 20, }, // America/Guayaquil
    {   83, 6932,   46308,  6, }, // America/Guyana
    {   83, 1939,   46314, 24, }, // America/Halifax
    {   83, 2281,   46338,  6, }, // America/Havana
    {   83, 7942,   46344, 22, }, // America/Hermosillo
    {   83,  348,   46366, 28, }, // America/Indiana/Indianapolis
    {   83,  481,   46394, 30, }, // America/Indiana/Knox
    {   83, 7961,   46424, 32, }, // America/Indiana/Marengo
    {   83, 7985,   46456, 42, }, // America/Indiana/Petersburg
    {   83, 8012,   46498, 34, }, // America/Indiana/Tell_City
    {   83, 8038,   46532, 30, }, // America/Indiana/Vevay
    {   83, 8060,   46562, 42, }, // America/Indiana/Vincennes
    {   83, 8086,   46604, 38, }, // America/Indiana/Winamac
    {   83, 8110,   46642, 18, }, // America/Inuvik
    {   83,  660,   46660, 20, }, // America/Iqaluit
    {   83, 2799,   46680,  8, }, // America/Jamaica
    {   83, 5380,   46688,  8, }, // America/Juneau
    {   83,  521,   46696, 18, }, // America/Kentucky/Louisville
    {   83, 8125,   46714, 36, }, // America/Kentucky/Monticello
    {   83, 5704,   46750, 26, }, // America/Kralendijk
    {   83, 4376,   46776,  9, }, // America/La_Paz
    {   83, 7169,   46785,  6, }, // America/Lima
    {   83, 3239,   46791, 31, }, // America/Los_Angeles
    {   83, 5932,   46822, 48, }, // America/Lower_Princes
    {   83, 8153,   46870, 10, }, // America/Maceio
    {   83, 8168,   46880,  8, }, // America/Managua
    {   83, 1908,   46888, 12, }, // America/Manaus
    {   83, 5897,   46900, 10, }, // America/Marigot
    {   83, 5805,   46910, 18, }, // America/Martinique
    {   83, 8184,   46928, 16, }, // America/Matamoros
    {   83, 2917,   46944, 14, }, // America/Mazatlan
    {   83, 8202,   46958, 16, }, // America/Menominee
    {   83, 8220,   46974, 10, }, // America/Merida
    {   83, 8235,   46984, 16, }, // America/Metlakatla
    {   83, 2949,   47000, 29, }, // America/Mexico_City
    {   83, 4391,   47029, 16, }, // America/Miquelon
    {   83, 8254,   47045, 16, }, // America/Moncton
    {   83, 8270,   47061, 16, }, // America/Monterrey
    {   83, 4098,   47077, 24, }, // America/Montevideo
    {   83, 5824,   47101, 22, }, // America/Montserrat
    {   83, 5481,   47123,  8, }, // America/Nassau
    {   83, 2379,   47131, 24, }, // America/New_York
    {   83, 8288,   47155,  8, }, // America/Nome
    {   83, 1850,   47163, 14, }, // America/Noronha
    {   83, 8301,   47177, 37, }, // America/North_Dakota/Beulah
    {   83, 8329,   47214, 41, }, // America/North_Dakota/Center
    {   83, 8357,   47255, 46, }, // America/North_Dakota/New_Salem
    {   83,  393,   47301,  8, }, // America/Nuuk
    {   83, 8388,   47309, 14, }, // America/Ojinaga
    {   83, 2356,   47323,  6, }, // America/Panama
    {   83, 6173,   47329, 20, }, // America/Paramaribo
    {   83, 2973,   47349, 16, }, // America/Phoenix
    {   83, 3945,   47365, 34, }, // America/Port-au-Prince
    {   83, 5954,   47399, 34, }, // America/Port_of_Spain
    {   83, 8404,   47433, 25, }, // America/Porto_Velho
    {   83, 5843,   47458, 25, }, // America/Puerto_Rico
    {   83, 8424,   47483, 35, }, // America/Rankin_Inlet
    {   83, 8445,   47518, 12, }, // America/Recife
    {   83, 1995,   47530, 10, }, // America/Regina
    {   83, 8460,   47540, 16, }, // America/Resolute
    {   83,  695,   47556, 23, }, // America/Rio_Branco
    {   83, 8477,   47579, 20, }, // America/Santarem
    {   83, 2201,   47599, 16, }, // America/Santiago
    {   83, 6111,   47615, 29, }, // America/Santo_Domingo
    {   83, 1878,   47644, 17, }, // America/Sao_Paulo
    {   83, 6899,   47661, 36, }, // America/Scoresbysund
    {   83, 8494,   47697, 10, }, // America/Sitka
    {   83, 8508,   47707, 32, }, // America/St_Barthelemy
    {   83, 2061,   47739, 25, }, // America/St_Johns
    {   83, 5863,   47764, 26, }, // America/St_Kitts
    {   83, 5880,   47790, 24, }, // America/St_Lucia
    {   83,  855,   47814, 23, }, // America/St_Thomas
    {   83, 5913,   47837, 33, }, // America/St_Vincent
    {   83, 8530,   47870, 29, }, // America/Swift_Current
    {   83, 5461,   47899, 20, }, // America/Tegucigalpa
    {   83, 5756,   47919,  8, }, // America/Thule
    {   83,  313,   47927, 12, }, // America/Tijuana
    {   83,  608,   47939, 16, }, // America/Toronto
    {   83, 5688,   47955, 14, }, // America/Tortola
    {   83, 2093,   47969, 20, }, // America/Vancouver
    {   83, 2144,   47989, 22, }, // America/Whitehorse
    {   83,  734,   48011, 20, }, // America/Winnipeg
    {   83, 8552,   48031, 16, }, // America/Yakutat
    {   83, 6001,   48047, 12, }, // Antarctica/Casey
    {   83, 6094,   48059, 12, }, // Antarctica/Davis
    {   83, 6133,   48071, 37, }, // Antarctica/DumontDUrville
    {   83, 8568,   48108, 22, }, // Antarctica/Macquarie
    {   83, 7066,   48130, 12, }, // Antarctica/Mawson
    {   83, 7113,   48142, 22, }, // Antarctica/McMurdo
    {   83, 8589,   48164, 12, }, // Antarctica/Palmer
    {   83, 7226,   48176, 10, }, // Antarctica/Rothera
    {   83, 7295,   48186,  6, }, // Antarctica/Syowa
    {   83, 8607,   48192, 12, }, // Antarctica/Troll
    {   83, 7374,   48204, 16, }, // Antarctica/Vostok
    {   83, 1429,   48220, 28, }, // Arctic/Longyearbyen
    {   83, 5570,   48248, 14, }, // Asia/Aden
    {   83, 5395,   48262, 12, }, // Asia/Almaty
    {   83, 3968,   48274, 14, }, // Asia/Amman
    {   83, 5511,   48288, 16, }, // Asia/Anadyr
    {   83, 5523,   48304, 12, }, // Asia/Aqtau
    {   83, 5368,   48316, 14, }, // Asia/Aqtobe
    {   83,  964,   48330, 16, }, // Asia/Ashgabat
    {   83, 3607,   48346, 12, }, // Asia/Baghdad
    {   83, 5534,   48358, 14, }, // Asia/Bahrain
    {   83, 3653,   45952,  6, }, // Asia/Baku
    {   83, 4466,   48372, 16, }, // Asia/Bangkok
    {   83, 3571,   48388, 14, }, // Asia/Barnaul
    {   83, 4086,   48402, 14, }, // Asia/Beirut
    {   83, 3778,   48416, 16, }, // Asia/Bishkek
    {   83, 5989,   48432, 14, }, // Asia/Brunei
    {   83, 4601,   48446,  6, }, // Asia/Chita
    {   83, 4511,   48452, 22, }, // Asia/Colombo
    {   83, 4540,   48474, 14, }, // Asia/Damascus
    {   83, 1093,   48488,  4, }, // Asia/Dhaka
    {   83, 6192,   48492,  8, }, // Asia/Dili
    {   83, 3596,   48500,  8, }, // Asia/Dubai
    {   83, 6159,   48508, 14, }, // Asia/Dushanbe
    {   83, 8651,   48522,  4, }, // Asia/Gaza
    {   83, 4795,   48526, 16, }, // Asia/Hebron
    {   83, 1259,   48542, 31, }, // Asia/Ho_Chi_Minh
    {   83, 2704,   48573, 13, }, // Asia/Hong_Kong
    {   83, 4771,   48586, 12, }, // Asia/Hovd
    {   83, 4184,   48598, 24, }, // Asia/Irkutsk
    {   83, 7006,   48622, 10, }, // Asia/Jakarta
    {   83, 6992,   48632, 12, }, // Asia/Jayapura
    {   83, 1290,   48644, 18, }, // Asia/Jerusalem
    {   83, 3560,   48662, 10, }, // Asia/Kabul
    {   83, 4316,   48672, 14, }, // Asia/Kamchatka
    {   83, 4239,   48686, 10, }, // Asia/Karachi
    {   83, 1185,   48696, 18, }, // Asia/Kathmandu
    {   83, 8661,   48714, 14, }, // Asia/Khandyga
    {   83,  992,   48728, 12, }, // Asia/Kolkata
    {   83, 4197,   48740, 28, }, // Asia/Krasnoyarsk
    {   83, 4859,   48768, 22, }, // Asia/Kuala_Lumpur
    {   83, 5976,   48790, 10, }, // Asia/Kuching
    {   83, 5547,   48800, 12, }, // Asia/Kuwait
    {   83, 1211,   48812, 12, }, // Asia/Macau
    {   83, 4017,   48824, 14, }, // Asia/Magadan
    {   83, 1349,   48838, 14, }, // Asia/Makassar
    {   83, 7182,   48852, 12, }, // Asia/Manila
    {   83, 6920,   48864, 12, }, // Asia/Muscat
    {   83, 2554,   48876, 14, }, // Asia/Nicosia
    {   83, 8675,   48890, 32, }, // Asia/Novokuznetsk
    {   83, 4135,   48922, 28, }, // Asia/Novosibirsk
    {   83, 4229,   48950, 16, }, // Asia/Omsk
    {   83, 7145,   48966, 10, }, // Asia/Oral
    {   83, 6961,   48976, 17, }, // Asia/Phnom_Penh
    {   83, 8693,   48993, 24, }, // Asia/Pontianak
    {   83, 4214,   49017, 21, }, // Asia/Pyongyang
    {   83, 5559,   49038,  8, }, // Asia/Qatar
    {   83, 4269,   49046, 18, }, // Asia/Qyzylorda
    {   83, 3584,   49064, 10, }, // Asia/Riyadh
    {   83, 4408,   49074, 12, }, // Asia/Sakhalin
    {   83, 7245,   49086, 24, }, // Asia/Samarkand
    {   83, 3279,   49110, 12, }, // Asia/Seoul
    {   83, 1053,   49122,  8, }, // Asia/Shanghai
    {   83, 3300,   49130, 18, }, // Asia/Singapore
    {   83, 4297,   49148, 40, }, // Asia/Srednekolymsk
    {   83, 3263,   49188, 12, }, // Asia/Taipei
    {   83, 4781,   49200, 18, }, // Asia/Tashkent
    {   83, 3915,   49218, 16, }, // Asia/Tbilisi
    {   83, 2772,   49234, 14, }, // Asia/Tehran
    {   83, 1317,   49248, 12, }, // Asia/Thimphu
    {   83, 2821,   49260, 12, }, // Asia/Tokyo
    {   83, 4572,   49272, 16, }, // Asia/Tomsk
    {   83, 1021,   49288, 18, }, // Asia/Ulaanbaatar
    {   83, 1159,   49306, 16, }, // Asia/Urumqi
    {   83, 8722,   49322, 19, }, // Asia/Ust-Nera
    {   83, 6977,   49341, 24, }, // Asia/Vientiane
    {   83, 4710,   49365, 26, }, // Asia/Vladivostok
    {   83, 4828,   49391, 18, }, // Asia/Yakutsk
    {   83, 1235,   49409, 16, }, // Asia/Yangon
    {   83, 3883,   49425, 32, }, // Asia/Yekaterinburg
    {   83, 3747,   49457, 16, }, // Asia/Yerevan
    {   83, 3663,   49473, 14, }, // Atlantic/Azores
    {   83, 5671,   49487, 12, }, // Atlantic/Bermuda
    {   83, 6628,   49499, 10, }, // Atlantic/Canary
    {   83, 3727,   49509, 19, }, // Atlantic/Cape_Verde
    {   83, 1395,   49528, 10, }, // Atlantic/Faroe
    {   83, 8736,   49538, 10, }, // Atlantic/Madeira
    {   83, 2748,   49548, 18, }, // Atlantic/Reykjavik
    {   83, 7272,   49566, 27, }, // Atlantic/South_Georgia
    {   83, 6821,   49593, 23, }, // Atlantic/St_Helena
    {   83, 6644,   49616, 14, }, // Atlantic/Stanley
    {   83, 1670,   49630, 20, }, // Australia/Adelaide
    {   83, 1635,   49650, 20, }, // Australia/Brisbane
    {   83, 1799,   49670, 25, }, // Australia/Broken_Hill
    {   83, 1583,   49695, 18, }, // Australia/Darwin
    {   83, 3637,   49713, 14, }, // Australia/Eucla
    {   83, 1516,   49727, 14, }, // Australia/Hobart
    {   83, 8753,   49741, 22, }, // Australia/Lindeman
    {   83, 1547,   49763, 19, }, // Australia/Lord_Howe
    {   83, 1727,   49782, 18, }, // Australia/Melbourne
    {   83, 1762,   49800, 10, }, // Australia/Perth
    {   83, 1463,   49810, 12, }, // Australia/Sydney
    {   83, 6441,   49822, 22, }, // Europe/Amsterdam
    {   83, 6216,   49844, 16, }, // Europe/Andorra
    {   83, 3620,   49860, 18, }, // Europe/Astrakhan
    {   83, 2300,   49878, 16, }, // Europe/Athens
    {   83, 6336,   49894, 20, }, // Europe/Belgrade
    {   83, 4757,   49914, 14, }, // Europe/Berlin
    {   83, 6488,   49928, 18, }, // Europe/Bratislava
    {   83, 2167,   49946, 18, }, // Europe/Brussels
    {   83, 3928,   49964, 18, }, // Europe/Bucharest
    {   83, 3806,   49982, 18, }, // Europe/Budapest
    {   83, 8772,   50000, 20, }, // Europe/Busingen
    {   83, 2583,   50020, 14, }, // Europe/Chisinau
    {   83, 6289,   50034, 24, }, // Europe/Copenhagen
    {   83, 2338,   50058, 14, }, // Europe/Dublin
    {   83, 6307,   50072, 20, }, // Europe/Gibraltar
    {   83, 8788,   50092, 16, }, // Europe/Guernsey
    {   83, 6612,   50108, 18, }, // Europe/Helsinki
    {   83, 8804,   50126, 28, }, // Europe/Isle_of_Man
    {   83, 1130,   50154, 22, }, // Europe/Istanbul
    {   83, 8823,   50176, 10, }, // Europe/Jersey
    {   83, 3979,   50186, 24, }, // Europe/Kaliningrad
    {   83, 8929,   50210, 12, }, // Europe/Kirov
    {   83, 2527,   50222, 12, }, // Europe/Kyiv
    {   83, 3213,   50234, 16, }, // Europe/Lisbon
    {   83, 6506,   50250, 16, }, // Europe/Ljubljana
    {   83, 2501,   50266, 16, }, // Europe/London
    {   83, 6365,   50282, 26, }, // Europe/Luxembourg
    {   83, 4892,   50308, 14, }, // Europe/Madrid
    {   83, 6397,   50322, 10, }, // Europe/Malta
    {   83, 6582,   50332, 22, }, // Europe/Mariehamn
    {   83, 3693,   50354, 16, }, // Europe/Minsk
    {   83, 6410,   50370, 10, }, // Europe/Monaco
    {   83, 3537,   50380, 12, }, // Europe/Moscow
    {   83, 6458,   50392, 12, }, // Europe/Oslo
    {   83, 4284,   50404, 14, }, // Europe/Paris
    {   83, 6424,   50418, 18, }, // Europe/Podgorica
    {   83, 6275,   50436, 10, }, // Europe/Prague
    {   83, 8837,   46902,  6, }, // Europe/Riga
    {   83, 6324,   50446,  8, }, // Europe/Rome
    {   83, 4331,   50454,  6, }, // Europe/Samara
    {   83, 6470,   50460, 19, }, // Europe/San_Marino
    {   83, 6245,   50479, 12, }, // Europe/Sarajevo
    {   83, 8849,   50491, 24, }, // Europe/Simferopol
    {   83, 6383,   50515, 16, }, // Europe/Skopje
    {   83, 6599,   50531, 10, }, // Europe/Sofia
    {   83, 6523,   50541, 20, }, // Europe/Stockholm
    {   83, 8867,   50561, 10, }, // Europe/Tallinn
    {   83, 6202,   50571,  8, }, // Europe/Tirane
    {   83, 8882,   50579, 26, }, // Europe/Ulyanovsk
    {   83, 6352,   50605, 10, }, // Europe/Vaduz
    {   83, 6567,   50615, 12, }, // Europe/Vatican
    {   83, 6231,   49341, 10, }, // Europe/Vienna
    {   83, 8899,   50627, 18, }, // Europe/Vilnius
    {   83, 4727,   50645, 22, }, // Europe/Volgograd
    {   83, 3190,   50667, 10, }, // Europe/Warsaw
    {   83, 6261,   50677, 14, }, // Europe/Zagreb
    {   83, 6540,   50691, 12, }, // Europe/Zurich
    {   83, 5078,   50703, 20, }, // Indian/Antananarivo
    {   83, 6947,   50723, 10, }, // Indian/Chagos
    {   83, 6046,   50733, 33, }, // Indian/Christmas
    {   83, 6063,   50766, 12, }, // Indian/Cocos
    {   83, 5029,   50778, 12, }, // Indian/Comoro
    {   83, 6661,   50790, 18, }, // Indian/Kerguelen
    {   83, 7260,   50808,  6, }, // Indian/Mahe
    {   83, 7050,   50814, 14, }, // Indian/Maldives
    {   83, 4069,   50828, 14, }, // Indian/Mauritius
    {   83, 5098,   50842, 10, }, // Indian/Mayotte
    {   83, 7211,   50852, 24, }, // Indian/Reunion
    {   83, 4422,   50876,  8, }, // Pacific/Apia
    {   83,  932,   50884, 22, }, // Pacific/Auckland
    {   83, 3706,   50906, 20, }, // Pacific/Bougainville
    {   83, 3015,   50926, 12, }, // Pacific/Chatham
    {   83, 3157,   50938, 12, }, // Pacific/Chuuk
    {   83, 2237,   50950, 14, }, // Pacific/Easter
    {   83, 7360,   50964, 10, }, // Pacific/Efate
    {   83, 7327,   50974, 16, }, // Pacific/Fakaofo
    {   83, 3902,   50990,  8, }, // Pacific/Fiji
    {   83, 7343,   50998, 14, }, // Pacific/Funafuti
    {   83, 6678,   51012, 22, }, // Pacific/Galapagos
    {   83, 6696,   51034, 18, }, // Pacific/Gambier
    {   83, 3822,   51052, 20, }, // Pacific/Guadalcanal
    {   83, 6018,   51072,  8, }, // Pacific/Guam
    {   83, 2723,   51080, 16, }, // Pacific/Honolulu
    {   83, 3049,   51096, 20, }, // Pacific/Kanton
    {   83, 3998,   51116, 18, }, // Pacific/Kiritimati
    {   83, 7019,   51134, 14, }, // Pacific/Kosrae
    {   83, 2842,   51148, 16, }, // Pacific/Kwajalein
    {   83, 4877,   51164, 10, }, // Pacific/Majuro
    {   83, 4051,   51174, 16, }, // Pacific/Marquesas
    {   83, 8914,   51190, 16, }, // Pacific/Midway
    {   83, 7084,   51206, 10, }, // Pacific/Nauru
    {   83, 7132,   51216, 12, }, // Pacific/Niue
    {   83, 4168,   51228, 16, }, // Pacific/Norfolk
    {   83, 7098,   51244, 10, }, // Pacific/Noumea
    {   83, 3126,   51254, 21, }, // Pacific/Pago_Pago
    {   83, 7155,   51275,  8, }, // Pacific/Palau
    {   83, 7194,   51283, 22, }, // Pacific/Pitcairn
    {   83, 3096,   51305, 10, }, // Pacific/Pohnpei
    {   83, 4807,   51315, 29, }, // Pacific/Port_Moresby
    {   83, 6076,   51344, 14, }, // Pacific/Rarotonga
    {   83, 6031,   51358, 10, }, // Pacific/Saipan
    {   83, 7312,   51368, 10, }, // Pacific/Tahiti
    {   83, 6712,   51378,  6, }, // Pacific/Tarawa
    {   83, 4583,   51384, 14, }, // Pacific/Tongatapu
    {   83, 7392,   51398, 12, }, // Pacific/Wake
    {   83, 7405,   51410, 12, }, // Pacific/Wallis
    {   85, 6788,   36214,  7, }, // Africa/Abidjan Chechen/Cyrillic/Russia
    {   85, 6760,     492,  5, }, // Africa/Accra
    {   85, 5059,     497, 11, }, // Africa/Addis_Ababa
    {   85, 7420,     508,  5, }, // Africa/Algiers
    {   85,   14,     513,  6, }, // Africa/Asmara
    {   85,   44,     519,  6, }, // Africa/Bamako
    {   85, 5257,     525,  5, }, // Africa/Bangui
    {   85, 6746,     530,  6, }, // Africa/Banjul
    {   85, 7435,     536,  5, }, // Africa/Bissau
    {   85, 4957,   27088,  8, }, // Africa/Blantyre
    {   85, 5287,   51422, 10, }, // Africa/Brazzaville
    {   85, 4922,     558,  9, }, // Africa/Bujumbura
    {   85, 2320,   51432,  8, }, // Africa/Cairo
    {   85, 4117,     571, 10, }, // Africa/Casablanca
    {   85, 7449,     581,  5, }, // Africa/Ceuta
    {   85, 6773,     586,  7, }, // Africa/Conakry
    {   85, 6840,     593,  5, }, // Africa/Dakar
    {   85, 5130,   27116, 12, }, // Africa/Dar_es_Salaam
    {   85, 5043,   36247,  7, }, // Africa/Djibouti
    {   85, 5243,     616,  5, }, // Africa/Douala
    {   85, 5166,   27135,  7, }, // Africa/El_Aaiun
    {   85, 6853,     628,  7, }, // Africa/Freetown
    {   85, 4906,     635,  8, }, // Africa/Gaborone
    {   85, 5015,   33781,  6, }, // Africa/Harare
    {   85, 4479,   51440, 12, }, // Africa/Johannesburg
    {   85, 4499,   27173,  5, }, // Africa/Juba
    {   85, 5151,     665,  7, }, // Africa/Kampala
    {   85, 4524,     672,  6, }, // Africa/Khartoum
    {   85, 4973,     678,  6, }, // Africa/Kigali
    {   85, 5306,     684,  7, }, // Africa/Kinshasa
    {   85, 4744,     691,  5, }, // Africa/Lagos
    {   85, 5336,   51452,  9, }, // Africa/Libreville
    {   85, 6869,     704,  4, }, // Africa/Lome
    {   85, 5211,     708,  6, }, // Africa/Luanda
    {   85, 4939,     714,  9, }, // Africa/Lubumbashi
    {   85, 5001,     723,  6, }, // Africa/Lusaka
    {   85, 5322,     729,  6, }, // Africa/Malabo
    {   85, 4987,     735,  6, }, // Africa/Maputo
    {   85, 5197,     741,  6, }, // Africa/Maseru
    {   85, 5182,     747,  7, }, // Africa/Mbabane
    {   85, 5113,     754,  8, }, // Africa/Mogadishu
    {   85, 7034,     762,  7, }, // Africa/Monrovia
    {   85, 3853,   36295,  7, }, // Africa/Nairobi
    {   85, 5271,   27254,  8, }, // Africa/Ndjamena
    {   85, 5354,   36302,  6, }, // Africa/Niamey
    {   85, 6803,     790,  7, }, // Africa/Nouakchott
    {   85, 6727,     797,  8, }, // Africa/Ouagadougou
    {   85, 5225,     805, 10, }, // Africa/Porto-Novo
    {   85, 4435,     815,  8, }, // Africa/Sao_Tome
    {   85, 2866,     823,  7, }, // Africa/Tripoli
    {   85, 6554,     830,  5, }, // Africa/Tunis
    {   85, 4152,     835,  7, }, // Africa/Windhoek
    {   85,  132,     842,  4, }, // America/Adak
    {   85, 3346,   51461,  8, }, // America/Anchorage
    {   85, 5607,   51469,  7, }, // America/Anguilla
    {   85, 5624,     860,  7, }, // America/Antigua
    {   85, 4554,     867,  9, }, // America/Araguaina
    {   85,  166,   51476, 12, }, // America/Argentina/Buenos_Aires
    {   85,   91,     888,  9, }, // America/Argentina/Catamarca
    {   85,  270,     897,  7, }, // America/Argentina/Cordoba
    {   85,  441,   27355,  5, }, // America/Argentina/Jujuy
    {   85, 7462,     909,  8, }, // America/Argentina/La_Rioja
    {   85,  565,     917,  7, }, // America/Argentina/Mendoza
    {   85, 7489,   51488, 12, }, // America/Argentina/Rio_Gallegos
    {   85, 7520,   27380,  6, }, // America/Argentina/Salta
    {   85, 7544,     940,  8, }, // America/Argentina/San_Juan
    {   85, 5580,     948,  8, }, // America/Argentina/San_Luis
    {   85, 7571,     956,  7, }, // America/Argentina/Tucuman
    {   85, 7597,   27394,  5, }, // America/Argentina/Ushuaia
    {   85, 5640,     969,  5, }, // America/Aruba
    {   85, 4252,   51500,  8, }, // America/Asuncion
    {   85,  237,     981, 12, }, // America/Atikokan
    {   85, 3679,     993,  3, }, // America/Bahia
    {   85, 7623,   51508, 16, }, // America/Bahia_Banderas
    {   85, 5654,    1013,  8, }, // America/Barbados
    {   85, 7646,    1021,  5, }, // America/Belem
    {   85, 5407,    1026,  5, }, // America/Belize
    {   85, 7660,    1031, 12, }, // America/Blanc-Sablon
    {   85, 7681,    1043,  9, }, // America/Boa_Vista
    {   85, 4361,    1052,  6, }, // America/Bogota
    {   85, 7699,   51524,  5, }, // America/Boise
    {   85, 7713,   51529, 13, }, // America/Cambridge_Bay
    {   85, 7735,    1075, 12, }, // America/Campo_Grande
    {   85, 3868,    1087,  6, }, // America/Cancun
    {   85, 4694,    1093,  7, }, // America/Caracas
    {   85, 4345,   51542,  7, }, // America/Cayenne
    {   85, 5496,   27497,  6, }, // America/Cayman
    {   85, 2260,    1129,  6, }, // America/Chicago
    {   85, 7756,    1135,  6, }, // America/Chihuahua
    {   85, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   85, 5422,    1141, 10, }, // America/Costa_Rica
    {   85, 7796,    1151,  7, }, // America/Creston
    {   85, 3791,   27556,  5, }, // America/Cuiaba
    {   85, 5723,   36474,  7, }, // America/Curacao
    {   85, 7812,    1172, 12, }, // America/Danmarkshavn
    {   85, 7833,    1184,  6, }, // America/Dawson
    {   85, 7848,    1190, 11, }, // America/Dawson_Creek
    {   85,  805,    1201,  6, }, // America/Denver
    {   85, 3465,   36517,  7, }, // America/Detroit
    {   85, 5739,    1214,  8, }, // America/Dominica
    {   85,  893,   51549,  8, }, // America/Edmonton
    {   85, 7869,   51557,  8, }, // America/Eirunepe
    {   85, 5441,   27633,  9, }, // America/El_Salvador
    {   85, 7886,   51565, 12, }, // America/Fort_Nelson
    {   85, 7906,    1257,  9, }, // America/Fortaleza
    {   85, 7924,   51577,  9, }, // America/Glace_Bay
    {   85, 6881,   51586,  7, }, // America/Goose_Bay
    {   85, 4612,   34146, 10, }, // America/Grand_Turk
    {   85, 5770,    1292,  7, }, // America/Grenada
    {   85, 5786,    1299,  9, }, // America/Guadeloupe
    {   85, 3760,    1308,  9, }, // America/Guatemala
    {   85, 4841,   51593,  8, }, // America/Guayaquil
    {   85, 6932,   51601,  6, }, // America/Guyana
    {   85, 1939,   51607,  8, }, // America/Halifax
    {   85, 2281,    1339,  6, }, // America/Havana
    {   85, 7942,   51615,  9, }, // America/Hermosillo
    {   85,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {   85,  481,    1365,  4, }, // America/Indiana/Knox
    {   85, 7961,    1378,  7, }, // America/Indiana/Marengo
    {   85, 7985,   34233, 10, }, // America/Indiana/Petersburg
    {   85, 8012,    1413,  9, }, // America/Indiana/Tell_City
    {   85, 8038,   51624,  5, }, // America/Indiana/Vevay
    {   85, 8060,   36607,  7, }, // America/Indiana/Vincennes
    {   85, 8086,   34304,  7, }, // America/Indiana/Winamac
    {   85, 8110,    1452,  6, }, // America/Inuvik
    {   85,  660,    1458,  7, }, // America/Iqaluit
    {   85, 2799,   27879,  6, }, // America/Jamaica
    {   85, 5380,   36614,  5, }, // America/Juneau
    {   85,  521,    1476,  8, }, // America/Kentucky/Louisville
    {   85, 8125,    1484, 10, }, // America/Kentucky/Monticello
    {   85, 5704,   36619, 10, }, // America/Kralendijk
    {   85, 4376,    1514,  6, }, // America/La_Paz
    {   85, 7169,    1520,  4, }, // America/Lima
    {   85, 3239,   51629, 12, }, // America/Los_Angeles
    {   85, 5932,   51641, 21, }, // America/Lower_Princes
    {   85, 8153,   36667,  6, }, // America/Maceio
    {   85, 8168,    1559,  7, }, // America/Managua
    {   85, 1908,    1566,  6, }, // America/Manaus
    {   85, 5897,    1572,  6, }, // America/Marigot
    {   85, 5805,    1578,  9, }, // America/Martinique
    {   85, 8184,    1587,  9, }, // America/Matamoros
    {   85, 2917,    1596,  8, }, // America/Mazatlan
    {   85, 8202,    1604,  8, }, // America/Menominee
    {   85, 8220,    1612,  6, }, // America/Merida
    {   85, 8235,    1618, 10, }, // America/Metlakatla
    {   85, 2949,    1628,  6, }, // America/Mexico_City
    {   85, 4391,    1634,  7, }, // America/Miquelon
    {   85, 8254,    1641,  7, }, // America/Moncton
    {   85, 8270,   51662,  9, }, // America/Monterrey
    {   85, 4098,    1657, 10, }, // America/Montevideo
    {   85, 5824,   51671,  9, }, // America/Montserrat
    {   85, 5481,    1677,  6, }, // America/Nassau
    {   85, 2379,   51680,  8, }, // America/New_York
    {   85, 8288,    1691,  3, }, // America/Nome
    {   85, 1850,   51688,  7, }, // America/Noronha
    {   85, 8301,   51695, 24, }, // America/North_Dakota/Beulah
    {   85, 8329,   51719, 22, }, // America/North_Dakota/Center
    {   85, 8357,   51741,  9, }, // America/North_Dakota/New_Salem
    {   85,  393,   51750,  6, }, // America/Nuuk
    {   85, 8388,    1776,  7, }, // America/Ojinaga
    {   85, 2356,    1783,  6, }, // America/Panama
    {   85, 6173,    1789, 10, }, // America/Paramaribo
    {   85, 2973,    1799,  6, }, // America/Phoenix
    {   85, 3945,    1805, 12, }, // America/Port-au-Prince
    {   85, 5954,   28189, 13, }, // America/Port_of_Spain
    {   85, 8404,   28202, 11, }, // America/Porto_Velho
    {   85, 5843,   51756, 11, }, // America/Puerto_Rico
    {   85, 4030,    1852, 12, }, // America/Punta_Arenas
    {   85, 8424,    1864, 12, }, // America/Rankin_Inlet
    {   85, 8445,    1876,  6, }, // America/Recife
    {   85, 1995,   51767,  8, }, // America/Regina
    {   85, 8460,   36760,  7, }, // America/Resolute
    {   85,  695,    1897, 10, }, // America/Rio_Branco
    {   85, 8477,    1907,  8, }, // America/Santarem
    {   85, 2201,   51775,  8, }, // America/Santiago
    {   85, 6111,    1923, 13, }, // America/Santo_Domingo
    {   85, 1878,    1936,  9, }, // America/Sao_Paulo
    {   85, 6899,    1945, 11, }, // America/Scoresbysund
    {   85, 8494,    1956,  5, }, // America/Sitka
    {   85, 8508,   51783, 13, }, // America/St_Barthelemy
    {   85, 2061,   28349, 10, }, // America/St_Johns
    {   85, 5863,    1983,  9, }, // America/St_Kitts
    {   85, 5880,   51796,  9, }, // America/St_Lucia
    {   85,  855,    2003, 10, }, // America/St_Thomas
    {   85, 5913,    2013, 12, }, // America/St_Vincent
    {   85, 8530,   51805, 12, }, // America/Swift_Current
    {   85, 5461,   51817, 12, }, // America/Tegucigalpa
    {   85, 5756,    2049,  4, }, // America/Thule
    {   85,  313,    2053,  7, }, // America/Tijuana
    {   85,  608,    2060,  7, }, // America/Toronto
    {   85, 5688,    2067,  7, }, // America/Tortola
    {   85, 2093,    2074,  8, }, // America/Vancouver
    {   85, 2144,   28435,  8, }, // America/Whitehorse
    {   85,  734,    2090,  8, }, // America/Winnipeg
    {   85, 8552,   28450,  6, }, // America/Yakutat
    {   85, 6001,   36889,  5, }, // Antarctica/Casey
    {   85, 6094,   36894,  6, }, // Antarctica/Davis
    {   85, 6133,   51829, 14, }, // Antarctica/DumontDUrville
    {   85, 8568,    2131,  8, }, // Antarctica/Macquarie
    {   85, 7066,    2139,  6, }, // Antarctica/Mawson
    {   85, 7113,    2145,  9, }, // Antarctica/McMurdo
    {   85, 8589,    2154,  6, }, // Antarctica/Palmer
    {   85, 7226,    2160,  6, }, // Antarctica/Rothera
    {   85, 7295,   28510,  4, }, // Antarctica/Syowa
    {   85, 8607,   51843,  6, }, // Antarctica/Troll
    {   85, 7374,   34825,  6, }, // Antarctica/Vostok
    {   85, 1429,   51849,  7, }, // Arctic/Longyearbyen
    {   85, 5570,    2192,  4, }, // Asia/Aden
    {   85, 5395,   51856,  8, }, // Asia/Almaty
    {   85, 3968,   51864,  6, }, // Asia/Amman
    {   85, 5511,   51870,  7, }, // Asia/Anadyr
    {   85, 5523,    2213,  5, }, // Asia/Aqtau
    {   85, 5368,    2218,  6, }, // Asia/Aqtobe
    {   85,  964,    2224,  7, }, // Asia/Ashgabat
    {   85, 8624,   51877,  6, }, // Asia/Atyrau
    {   85, 3607,   51883,  7, }, // Asia/Baghdad
    {   85, 5534,   36948,  7, }, // Asia/Bahrain
    {   85, 3653,   51890,  5, }, // Asia/Baku
    {   85, 4466,    2254,  7, }, // Asia/Bangkok
    {   85, 3571,    2261,  7, }, // Asia/Barnaul
    {   85, 4086,   28563,  6, }, // Asia/Beirut
    {   85, 3778,    2274,  6, }, // Asia/Bishkek
    {   85, 5989,   28575,  6, }, // Asia/Brunei
    {   85, 4601,    2286,  4, }, // Asia/Chita
    {   85, 4511,    2290,  7, }, // Asia/Colombo
    {   85, 4540,   51895,  7, }, // Asia/Damascus
    {   85, 1093,    2303,  5, }, // Asia/Dhaka
    {   85, 6192,    2308,  4, }, // Asia/Dili
    {   85, 3596,   28596,  5, }, // Asia/Dubai
    {   85, 6159,    2317,  7, }, // Asia/Dushanbe
    {   85, 8636,    2324,  9, }, // Asia/Famagusta
    {   85, 8651,    2333,  4, }, // Asia/Gaza
    {   85, 4795,    2337,  6, }, // Asia/Hebron
    {   85, 1259,    2343,  7, }, // Asia/Ho_Chi_Minh
    {   85, 2704,   51902,  7, }, // Asia/Hong_Kong
    {   85, 4771,    2357,  4, }, // Asia/Hovd
    {   85, 4184,    2361,  7, }, // Asia/Irkutsk
    {   85, 7006,   28638,  8, }, // Asia/Jakarta
    {   85, 6992,   51909,  7, }, // Asia/Jayapura
    {   85, 1290,   51916, 11, }, // Asia/Jerusalem
    {   85, 3560,    2392,  5, }, // Asia/Kabul
    {   85, 4316,   51927, 23, }, // Asia/Kamchatka
    {   85, 4239,    2422,  6, }, // Asia/Karachi
    {   85, 1185,    2428,  8, }, // Asia/Kathmandu
    {   85, 8661,    2436,  7, }, // Asia/Khandyga
    {   85,  992,   51950,  9, }, // Asia/Kolkata
    {   85, 4197,   36978, 10, }, // Asia/Krasnoyarsk
    {   85, 4859,    2462, 12, }, // Asia/Kuala_Lumpur
    {   85, 5976,    2474,  6, }, // Asia/Kuching
    {   85, 5547,   28693,  6, }, // Asia/Kuwait
    {   85, 1211,    2486,  5, }, // Asia/Macau
    {   85, 4017,    2491,  7, }, // Asia/Magadan
    {   85, 1349,    2498,  7, }, // Asia/Makassar
    {   85, 7182,    2505,  6, }, // Asia/Manila
    {   85, 6920,    2511,  6, }, // Asia/Muscat
    {   85, 2554,    2517,  6, }, // Asia/Nicosia
    {   85, 8675,    2524, 11, }, // Asia/Novokuznetsk
    {   85, 4135,    2535, 11, }, // Asia/Novosibirsk
    {   85, 4229,    2546,  4, }, // Asia/Omsk
    {   85, 7145,   35006,  4, }, // Asia/Oral
    {   85, 6961,   51959,  8, }, // Asia/Phnom_Penh
    {   85, 8693,    2563,  9, }, // Asia/Pontianak
    {   85, 4214,   28763,  7, }, // Asia/Pyongyang
    {   85, 5559,    2579,  5, }, // Asia/Qatar
    {   85, 4269,    2592,  9, }, // Asia/Qyzylorda
    {   85, 3584,   51967,  7, }, // Asia/Riyadh
    {   85, 4408,   51974, 12, }, // Asia/Sakhalin
    {   85, 7245,    2621,  9, }, // Asia/Samarkand
    {   85, 3279,    2630,  4, }, // Asia/Seoul
    {   85, 1053,   28801,  6, }, // Asia/Shanghai
    {   85, 3300,    2640,  8, }, // Asia/Singapore
    {   85, 4297,    2648, 13, }, // Asia/Srednekolymsk
    {   85, 3263,   28829,  6, }, // Asia/Taipei
    {   85, 4781,    2667,  7, }, // Asia/Tashkent
    {   85, 3915,   35074,  7, }, // Asia/Tbilisi
    {   85, 2772,   51986,  7, }, // Asia/Tehran
    {   85, 1317,   35088,  5, }, // Asia/Thimphu
    {   85, 2821,    2692,  5, }, // Asia/Tokyo
    {   85, 4572,    2697,  5, }, // Asia/Tomsk
    {   85, 1021,    2702, 10, }, // Asia/Ulaanbaatar
    {   85, 1159,    2712,  6, }, // Asia/Urumqi
    {   85, 8722,   51993,  9, }, // Asia/Ust-Nera
    {   85, 6977,   52002,  8, }, // Asia/Vientiane
    {   85, 4710,    2733, 11, }, // Asia/Vladivostok
    {   85, 4828,   37051,  6, }, // Asia/Yakutsk
    {   85, 1235,   28911,  6, }, // Asia/Yangon
    {   85, 3883,    2757, 12, }, // Asia/Yekaterinburg
    {   85, 3747,    2769,  6, }, // Asia/Yerevan
    {   85, 3663,   52010, 16, }, // Atlantic/Azores
    {   85, 5671,   52026,  8, }, // Atlantic/Bermuda
    {   85, 6628,   52034, 17, }, // Atlantic/Canary
    {   85, 3727,    2826, 10, }, // Atlantic/Cape_Verde
    {   85, 1395,   52051, 18, }, // Atlantic/Faroe
    {   85, 8736,   52069, 13, }, // Atlantic/Madeira
    {   85, 2748,   52082,  9, }, // Atlantic/Reykjavik
    {   85, 7272,   52091, 14, }, // Atlantic/South_Georgia
    {   85, 6821,   52105, 19, }, // Atlantic/St_Helena
    {   85, 6644,   52124,  6, }, // Atlantic/Stanley
    {   85, 1670,    2917,  8, }, // Australia/Adelaide
    {   85, 1635,    2925,  7, }, // Australia/Brisbane
    {   85, 1799,    2932, 11, }, // Australia/Broken_Hill
    {   85, 1583,    2943,  6, }, // Australia/Darwin
    {   85, 3637,   29108,  4, }, // Australia/Eucla
    {   85, 1516,    2954,  6, }, // Australia/Hobart
    {   85, 8753,    2960,  8, }, // Australia/Lindeman
    {   85, 1547,   52130, 14, }, // Australia/Lord_Howe
    {   85, 1727,   29120,  8, }, // Australia/Melbourne
    {   85, 1762,    2983,  4, }, // Australia/Perth
    {   85, 1463,   52144,  6, }, // Australia/Sydney
    {   85, 6441,    2993,  9, }, // Europe/Amsterdam
    {   85, 6216,    3002,  7, }, // Europe/Andorra
    {   85, 3620,   52150,  9, }, // Europe/Astrakhan
    {   85, 2300,   52159,  5, }, // Europe/Athens
    {   85, 6336,    3021,  7, }, // Europe/Belgrade
    {   85, 4757,    3028,  6, }, // Europe/Berlin
    {   85, 6488,    3034, 10, }, // Europe/Bratislava
    {   85, 2167,   52164,  8, }, // Europe/Brussels
    {   85, 3928,    3052,  8, }, // Europe/Bucharest
    {   85, 3806,    3060,  8, }, // Europe/Budapest
    {   85, 8772,   52172, 19, }, // Europe/Busingen
    {   85, 2583,    3091,  7, }, // Europe/Chisinau
    {   85, 6289,   52191, 10, }, // Europe/Copenhagen
    {   85, 2338,    3108,  6, }, // Europe/Dublin
    {   85, 6307,    3114,  9, }, // Europe/Gibraltar
    {   85, 8788,    3123,  6, }, // Europe/Guernsey
    {   85, 6612,   52201,  9, }, // Europe/Helsinki
    {   85, 8804,   52210,  9, }, // Europe/Isle_of_Man
    {   85, 1130,    3146,  7, }, // Europe/Istanbul
    {   85, 8823,   52219,  6, }, // Europe/Jersey
    {   85, 3979,    3158, 11, }, // Europe/Kaliningrad
    {   85, 8929,    3169,  5, }, // Europe/Kirov
    {   85, 2527,    3174,  4, }, // Europe/Kyiv
    {   85, 3213,    3178,  8, }, // Europe/Lisbon
    {   85, 6506,   29300,  7, }, // Europe/Ljubljana
    {   85, 2501,    3195,  6, }, // Europe/London
    {   85, 6365,   29313, 10, }, // Europe/Luxembourg
    {   85, 4892,    3212,  6, }, // Europe/Madrid
    {   85, 6397,   29329,  6, }, // Europe/Malta
    {   85, 6582,    3223,  9, }, // Europe/Mariehamn
    {   85, 3693,    3232,  5, }, // Europe/Minsk
    {   85, 6410,    3237,  6, }, // Europe/Monaco
    {   85, 3537,    3243,  6, }, // Europe/Moscow
    {   85, 6458,    3249,  4, }, // Europe/Oslo
    {   85, 4284,    3253,  5, }, // Europe/Paris
    {   85, 6424,    3258,  9, }, // Europe/Podgorica
    {   85, 6275,    3267,  5, }, // Europe/Prague
    {   85, 8837,    3272,  4, }, // Europe/Riga
    {   85, 6324,    3276,  3, }, // Europe/Rome
    {   85, 4331,    3279,  6, }, // Europe/Samara
    {   85, 6470,    3285, 10, }, // Europe/San_Marino
    {   85, 6245,    3295,  7, }, // Europe/Sarajevo
    {   85, 4451,    3302,  7, }, // Europe/Saratov
    {   85, 8849,   52225, 11, }, // Europe/Simferopol
    {   85, 6383,   52236,  6, }, // Europe/Skopje
    {   85, 6599,    3324,  4, }, // Europe/Sofia
    {   85, 6523,   52242,  9, }, // Europe/Stockholm
    {   85, 8867,    3337,  6, }, // Europe/Tallinn
    {   85, 6202,    3343,  6, }, // Europe/Tirane
    {   85, 8882,   52251,  9, }, // Europe/Ulyanovsk
    {   85, 6352,    3358,  5, }, // Europe/Vaduz
    {   85, 6567,    3363,  7, }, // Europe/Vatican
    {   85, 6231,    3370,  4, }, // Europe/Vienna
    {   85, 8899,   52260,  7, }, // Europe/Vilnius
    {   85, 4727,    3381,  9, }, // Europe/Volgograd
    {   85, 3190,    3390,  7, }, // Europe/Warsaw
    {   85, 6261,    3397,  6, }, // Europe/Zagreb
    {   85, 6540,   37310,  5, }, // Europe/Zurich
    {   85, 5078,    3409, 12, }, // Indian/Antananarivo
    {   85, 6947,    3421,  5, }, // Indian/Chagos
    {   85, 6046,   52267, 31, }, // Indian/Christmas
    {   85, 6063,   52298, 16, }, // Indian/Cocos
    {   85, 5029,   52314, 18, }, // Indian/Comoro
    {   85, 6661,    3460,  8, }, // Indian/Kerguelen
    {   85, 7260,   29562,  3, }, // Indian/Mahe
    {   85, 7050,   52332,  9, }, // Indian/Maldives
    {   85, 4069,    3477,  7, }, // Indian/Mauritius
    {   85, 5098,   52341,  7, }, // Indian/Mayotte
    {   85, 7211,   52348,  7, }, // Indian/Reunion
    {   85, 4422,    3498,  3, }, // Pacific/Apia
    {   85,  932,    3502,  6, }, // Pacific/Auckland
    {   85, 3706,   52355, 15, }, // Pacific/Bougainville
    {   85, 3015,   52370, 11, }, // Pacific/Chatham
    {   85, 3157,   52381, 12, }, // Pacific/Chuuk
    {   85, 2237,   52393, 16, }, // Pacific/Easter
    {   85, 7360,   52409,  5, }, // Pacific/Efate
    {   85, 7327,    3541,  7, }, // Pacific/Fakaofo
    {   85, 3902,   37407,  5, }, // Pacific/Fiji
    {   85, 7343,    3552,  8, }, // Pacific/Funafuti
    {   85, 6678,   52414, 21, }, // Pacific/Galapagos
    {   85, 6696,   52435, 15, }, // Pacific/Gambier
    {   85, 3822,   35587, 11, }, // Pacific/Guadalcanal
    {   85, 6018,    3606,  4, }, // Pacific/Guam
    {   85, 2723,   52450,  8, }, // Pacific/Honolulu
    {   85, 3049,   52458, 16, }, // Pacific/Kanton
    {   85, 3998,    3616, 10, }, // Pacific/Kiritimati
    {   85, 7019,    3626,  6, }, // Pacific/Kosrae
    {   85, 2842,   29709, 10, }, // Pacific/Kwajalein
    {   85, 4877,   37442,  7, }, // Pacific/Majuro
    {   85, 4051,   52474, 16, }, // Pacific/Marquesas
    {   85, 8914,   52490, 14, }, // Pacific/Midway
    {   85, 7084,    3680,  5, }, // Pacific/Nauru
    {   85, 7132,   52504,  4, }, // Pacific/Niue
    {   85, 4168,    3689,  7, }, // Pacific/Norfolk
    {   85, 7098,    3696,  5, }, // Pacific/Noumea
    {   85, 3126,    3701,  9, }, // Pacific/Pago_Pago
    {   85, 7155,    3710,  5, }, // Pacific/Palau
    {   85, 7194,    3715,  7, }, // Pacific/Pitcairn
    {   85, 3096,   52508, 14, }, // Pacific/Pohnpei
    {   85, 4807,    3728, 11, }, // Pacific/Port_Moresby
    {   85, 6076,    3739,  9, }, // Pacific/Rarotonga
    {   85, 6031,   29803,  6, }, // Pacific/Saipan
    {   85, 7312,   52522, 13, }, // Pacific/Tahiti
    {   85, 6712,    3759,  6, }, // Pacific/Tarawa
    {   85, 4583,    3765,  9, }, // Pacific/Tongatapu
    {   85, 7392,   52535, 10, }, // Pacific/Wake
    {   85, 7405,    3778,  6, }, // Pacific/Wallis
    {   86, 6788,   52545,  4, }, // Africa/Abidjan Cherokee/Cherokee/United States
    {   86, 6760,   52549,  2, }, // Africa/Accra
    {   86, 5059,   52551,  7, }, // Africa/Addis_Ababa
    {   86, 7420,   52558,  4, }, // Africa/Algiers
    {   86,   14,   52562,  4, }, // Africa/Asmara
    {   86,   44,   52566,  3, }, // Africa/Bamako
    {   86, 5257,   52569,  4, }, // Africa/Bangui
    {   86, 6746,   52573,  4, }, // Africa/Banjul
    {   86, 7435,   52577,  3, }, // Africa/Bissau
    {   86, 4957,   52580,  3, }, // Africa/Blantyre
    {   86, 5287,   52583,  4, }, // Africa/Brazzaville
    {   86, 4922,   52587,  5, }, // Africa/Bujumbura
    {   86, 2320,   52592,  3, }, // Africa/Cairo
    {   86, 4117,   52595,  8, }, // Africa/Casablanca
    {   86, 7449,   52603,  2, }, // Africa/Ceuta
    {   86, 6773,   52605,  3, }, // Africa/Conakry
    {   86, 6840,   52608,  3, }, // Africa/Dakar
    {   86, 5130,   52611,  8, }, // Africa/Dar_es_Salaam
    {   86, 5043,   52619,  3, }, // Africa/Djibouti
    {   86, 5243,   52622,  4, }, // Africa/Douala
    {   86, 5166,   52626,  7, }, // Africa/El_Aaiun
    {   86, 6853,   52633,  7, }, // Africa/Freetown
    {   86, 4906,   52640,  4, }, // Africa/Gaborone
    {   86, 5015,   52644,  3, }, // Africa/Harare
    {   86, 4479,   52647,  6, }, // Africa/Johannesburg
    {   86, 4499,   52653,  2, }, // Africa/Juba
    {   86, 5151,   52655,  4, }, // Africa/Kampala
    {   86, 4524,   52659,  3, }, // Africa/Khartoum
    {   86, 4973,   52662,  3, }, // Africa/Kigali
    {   86, 5306,   52665,  4, }, // Africa/Kinshasa
    {   86, 4744,   52669,  3, }, // Africa/Lagos
    {   86, 5336,   52672,  4, }, // Africa/Libreville
    {   86, 6869,   52676,  2, }, // Africa/Lome
    {   86, 5211,   52678,  4, }, // Africa/Luanda
    {   86, 4939,   52682,  4, }, // Africa/Lubumbashi
    {   86, 5001,   52686,  3, }, // Africa/Lusaka
    {   86, 5322,   52689,  3, }, // Africa/Malabo
    {   86, 4987,   52692,  3, }, // Africa/Maputo
    {   86, 5197,   52695,  3, }, // Africa/Maseru
    {   86, 5182,   52698,  3, }, // Africa/Mbabane
    {   86, 5113,   52701,  4, }, // Africa/Mogadishu
    {   86, 7034,   52705,  5, }, // Africa/Monrovia
    {   86, 3853,   52710,  4, }, // Africa/Nairobi
    {   86, 5271,   52714,  4, }, // Africa/Ndjamena
    {   86, 5354,   52718,  3, }, // Africa/Niamey
    {   86, 6803,   52721,  4, }, // Africa/Nouakchott
    {   86, 6727,   52725,  5, }, // Africa/Ouagadougou
    {   86, 5225,   52730, 11, }, // Africa/Porto-Novo
    {   86, 4435,   52741,  4, }, // Africa/Sao_Tome
    {   86, 2866,   52745,  3, }, // Africa/Tripoli
    {   86, 6554,   52748,  3, }, // Africa/Tunis
    {   86, 4152,   52751,  5, }, // Africa/Windhoek
    {   86,  132,   52756,  3, }, // America/Adak
    {   86, 3346,   52759,  5, }, // America/Anchorage
    {   86, 5607,   52764,  4, }, // America/Anguilla
    {   86, 5624,   52768,  3, }, // America/Antigua
    {   86, 4554,   52771,  6, }, // America/Araguaina
    {   86,  166,   52777,  7, }, // America/Argentina/Buenos_Aires
    {   86,   91,   52784,  4, }, // America/Argentina/Catamarca
    {   86,  270,   52788,  3, }, // America/Argentina/Cordoba
    {   86,  441,   52791,  3, }, // America/Argentina/Jujuy
    {   86, 7462,   52794,  5, }, // America/Argentina/La_Rioja
    {   86,  565,   52799,  4, }, // America/Argentina/Mendoza
    {   86, 7489,   52803,  8, }, // America/Argentina/Rio_Gallegos
    {   86, 7520,   52811,  3, }, // America/Argentina/Salta
    {   86, 7544,   52814,  5, }, // America/Argentina/San_Juan
    {   86, 5580,   52819,  8, }, // America/Argentina/San_Luis
    {   86, 7571,   52827,  4, }, // America/Argentina/Tucuman
    {   86, 7597,   52831,  4, }, // America/Argentina/Ushuaia
    {   86, 5640,   52835,  3, }, // America/Aruba
    {   86, 4252,   52838,  5, }, // America/Asuncion
    {   86,  237,   52843,  5, }, // America/Atikokan
    {   86, 3679,   52848,  3, }, // America/Bahia
    {   86, 7623,   52851,  9, }, // America/Bahia_Banderas
    {   86, 5654,   52860,  4, }, // America/Barbados
    {   86, 7646,   52864,  3, }, // America/Belem
    {   86, 5407,   52867,  3, }, // America/Belize
    {   86, 7660,   52870,  6, }, // America/Blanc-Sablon
    {   86, 7681,   52876,  9, }, // America/Boa_Vista
    {   86, 4361,   52885,  3, }, // America/Bogota
    {   86, 7699,   52888,  3, }, // America/Boise
    {   86, 7713,   52891, 10, }, // America/Cambridge_Bay
    {   86, 7735,   52901,  7, }, // America/Campo_Grande
    {   86, 3868,   52908,  4, }, // America/Cancun
    {   86, 4694,   52912,  4, }, // America/Caracas
    {   86, 4345,   52916,  3, }, // America/Cayenne
    {   86, 5496,   52919,  4, }, // America/Cayman
    {   86, 2260,   52923,  3, }, // America/Chicago
    {   86, 7756,   52926,  3, }, // America/Chihuahua
    {   86, 7774,   52929,  7, }, // America/Ciudad_Juarez
    {   86, 5422,   52936,  5, }, // America/Costa_Rica
    {   86, 7796,   52941,  4, }, // America/Creston
    {   86, 3791,   52945,  3, }, // America/Cuiaba
    {   86, 5723,   52948,  4, }, // America/Curacao
    {   86, 7812,   52952,  7, }, // America/Danmarkshavn
    {   86, 7833,   52959,  3, }, // America/Dawson
    {   86, 7848,   52962,  7, }, // America/Dawson_Creek
    {   86,  805,   52969,  8, }, // America/Denver
    {   86, 3465,   52977,  3, }, // America/Detroit
    {   86, 5739,   52980,  4, }, // America/Dominica
    {   86,  893,   52984,  6, }, // America/Edmonton
    {   86, 7869,   52990,  4, }, // America/Eirunepe
    {   86, 5441,   52994,  8, }, // America/El_Salvador
    {   86, 7886,   53002,  8, }, // America/Fort_Nelson
    {   86, 7906,   53010,  4, }, // America/Fortaleza
    {   86, 7924,   53014,  8, }, // America/Glace_Bay
    {   86, 6881,   53022,  8, }, // America/Goose_Bay
    {   86, 4612,   53030,  8, }, // America/Grand_Turk
    {   86, 5770,   53038,  3, }, // America/Grenada
    {   86, 5786,   53041,  4, }, // America/Guadeloupe
    {   86, 3760,   53045,  4, }, // America/Guatemala
    {   86, 4841,   53049,  4, }, // America/Guayaquil
    {   86, 6932,   53053,  3, }, // America/Guyana
    {   86, 1939,   53056,  5, }, // America/Halifax
    {   86, 2281,   53061,  3, }, // America/Havana
    {   86, 7942,   53064,  4, }, // America/Hermosillo
    {   86,  348,   53068,  9, }, // America/Indiana/Indianapolis
    {   86,  481,   53077, 10, }, // America/Indiana/Knox
    {   86, 7961,   53087, 11, }, // America/Indiana/Marengo
    {   86, 7985,   53098, 11, }, // America/Indiana/Petersburg
    {   86, 8012,   53109, 13, }, // America/Indiana/Tell_City
    {   86, 8038,   53122,  9, }, // America/Indiana/Vevay
    {   86, 8060,   53131, 12, }, // America/Indiana/Vincennes
    {   86, 8086,   53143, 11, }, // America/Indiana/Winamac
    {   86, 8110,   53154,  4, }, // America/Inuvik
    {   86,  660,   53158,  5, }, // America/Iqaluit
    {   86, 2799,   53163,  4, }, // America/Jamaica
    {   86, 5380,   53167,  2, }, // America/Juneau
    {   86,  521,   53169,  4, }, // America/Kentucky/Louisville
    {   86, 8125,   53173, 11, }, // America/Kentucky/Monticello
    {   86, 5704,   53184,  4, }, // America/Kralendijk
    {   86, 4376,   53188,  2, }, // America/La_Paz
    {   86, 7169,   53172,  2, }, // America/Lima
    {   86, 3239,   53190, 12, }, // America/Los_Angeles
    {   86, 5932,   53202, 17, }, // America/Lower_Princes
    {   86, 8153,   53219,  3, }, // America/Maceio
    {   86, 8168,   53222,  3, }, // America/Managua
    {   86, 1908,   53225,  4, }, // America/Manaus
    {   86, 5897,   53229,  4, }, // America/Marigot
    {   86, 5805,   53233,  4, }, // America/Martinique
    {   86, 8184,   53237,  5, }, // America/Matamoros
    {   86, 2917,   53242,  4, }, // America/Mazatlan
    {   86, 8202,   53246,  4, }, // America/Menominee
    {   86, 8220,   53250,  3, }, // America/Merida
    {   86, 8235,   53253,  4, }, // America/Metlakatla
    {   86, 2949,   53257,  9, }, // America/Mexico_City
    {   86, 4391,   53266,  4, }, // America/Miquelon
    {   86, 8254,   53270,  4, }, // America/Moncton
    {   86, 8270,   53274,  3, }, // America/Monterrey
    {   86, 4098,   53277,  6, }, // America/Montevideo
    {   86, 5824,   53283,  6, }, // America/Montserrat
    {   86, 5481,   53289,  3, }, // America/Nassau
    {   86, 2379,   53292,  3, }, // America/New_York
    {   86, 8288,   53295,  2, }, // America/Nome
    {   86, 1850,   53297,  4, }, // America/Noronha
    {   86, 8301,   53301, 12, }, // America/North_Dakota/Beulah
    {   86, 8329,   53313, 12, }, // America/North_Dakota/Center
    {   86, 8357,   53325, 15, }, // America/North_Dakota/New_Salem
    {   86,  393,   53340,  2, }, // America/Nuuk
    {   86, 8388,   53342,  4, }, // America/Ojinaga
    {   86, 2356,   53346,  3, }, // America/Panama
    {   86, 6173,   53349,  5, }, // America/Paramaribo
    {   86, 2973,   53354,  6, }, // America/Phoenix
    {   86, 3945,   53360, 17, }, // America/Port-au-Prince
    {   86, 5954,   53377, 15, }, // America/Port_of_Spain
    {   86, 8404,   53392, 12, }, // America/Porto_Velho
    {   86, 5843,   53404,  7, }, // America/Puerto_Rico
    {   86, 4030,   53411,  7, }, // America/Punta_Arenas
    {   86, 8424,   53418,  9, }, // America/Rankin_Inlet
    {   86, 8445,   53427,  3, }, // America/Recife
    {   86, 1995,   53430,  3, }, // America/Regina
    {   86, 8460,   53433,  4, }, // America/Resolute
    {   86,  695,   53437,  7, }, // America/Rio_Branco
    {   86, 8477,   53444,  5, }, // America/Santarem
    {   86, 2201,   53449,  5, }, // America/Santiago
    {   86, 6111,   53454,  7, }, // America/Santo_Domingo
    {   86, 1878,   53461,  5, }, // America/Sao_Paulo
    {   86, 6899,   53466,  6, }, // America/Scoresbysund
    {   86, 8494,   53472,  3, }, // America/Sitka
    {   86, 8508,   53475, 10, }, // America/St_Barthelemy
    {   86, 2061,   53485, 11, }, // America/St_Johns
    {   86, 5863,   53496,  8, }, // America/St_Kitts
    {   86, 5880,   53504,  8, }, // America/St_Lucia
    {   86,  855,   53512,  8, }, // America/St_Thomas
    {   86, 5913,   53520,  9, }, // America/St_Vincent
    {   86, 8530,   53529,  8, }, // America/Swift_Current
    {   86, 5461,   53537,  6, }, // America/Tegucigalpa
    {   86, 5756,   53543,  2, }, // America/Thule
    {   86,  313,   53545,  4, }, // America/Tijuana
    {   86,  608,   53549,  4, }, // America/Toronto
    {   86, 5688,   53553,  3, }, // America/Tortola
    {   86, 2093,   53556,  5, }, // America/Vancouver
    {   86, 2144,   53561,  7, }, // America/Whitehorse
    {   86,  734,   53568,  4, }, // America/Winnipeg
    {   86, 8552,   53572,  4, }, // America/Yakutat
    {   86, 6001,   52906,  2, }, // Antarctica/Casey
    {   86, 6094,   53576,  3, }, // Antarctica/Davis
    {   86, 6133,   53579, 11, }, // Antarctica/DumontDUrville
    {   86, 8568,   53590,  3, }, // Antarctica/Macquarie
    {   86, 7066,   53593,  3, }, // Antarctica/Mawson
    {   86, 7113,   53596,  4, }, // Antarctica/McMurdo
    {   86, 8589,   53600,  4, }, // Antarctica/Palmer
    {   86, 7226,   53604,  3, }, // Antarctica/Rothera
    {   86, 7295,   53607,  3, }, // Antarctica/Syowa
    {   86, 8607,   53610,  2, }, // Antarctica/Troll
    {   86, 7374,   53612,  4, }, // Antarctica/Vostok
    {   86, 1429,   53616, 17, }, // Arctic/Longyearbyen
    {   86, 5570,   53633,  3, }, // Asia/Aden
    {   86, 5395,   53636,  4, }, // Asia/Almaty
    {   86, 3968,   53640,  3, }, // Asia/Amman
    {   86, 5511,   53643,  4, }, // Asia/Anadyr
    {   86, 5523,   53647,  4, }, // Asia/Aqtau
    {   86, 5368,   53651,  4, }, // Asia/Aqtobe
    {   86,  964,   53655,  5, }, // Asia/Ashgabat
    {   86, 8624,   53660,  4, }, // Asia/Atyrau
    {   86, 3607,   53664,  4, }, // Asia/Baghdad
    {   86, 5534,   53668,  5, }, // Asia/Bahrain
    {   86, 3653,   52947,  2, }, // Asia/Baku
    {   86, 4466,   53673,  5, }, // Asia/Bangkok
    {   86, 3571,   53678,  4, }, // Asia/Barnaul
    {   86, 4086,   53682,  3, }, // Asia/Beirut
    {   86, 3778,   53685,  4, }, // Asia/Bishkek
    {   86, 5989,   53689,  3, }, // Asia/Brunei
    {   86, 4601,   53692,  2, }, // Asia/Chita
    {   86, 4511,   53694,  4, }, // Asia/Colombo
    {   86, 4540,   53698,  5, }, // Asia/Damascus
    {   86, 1093,   52608,  2, }, // Asia/Dhaka
    {   86, 6192,   53645,  2, }, // Asia/Dili
    {   86, 3596,   53703,  3, }, // Asia/Dubai
    {   86, 6159,   53706,  4, }, // Asia/Dushanbe
    {   86, 8636,   53710,  5, }, // Asia/Famagusta
    {   86, 8651,   53715,  2, }, // Asia/Gaza
    {   86, 4795,   53717,  3, }, // Asia/Hebron
    {   86, 1259,   53720, 10, }, // Asia/Ho_Chi_Minh
    {   86, 2704,   53730,  7, }, // Asia/Hong_Kong
    {   86, 4771,   53737,  3, }, // Asia/Hovd
    {   86, 4184,   53740,  4, }, // Asia/Irkutsk
    {   86, 7006,   53744,  3, }, // Asia/Jakarta
    {   86, 6992,   53747,  4, }, // Asia/Jayapura
    {   86, 1290,   53751,  5, }, // Asia/Jerusalem
    {   86, 3560,   53756,  3, }, // Asia/Kabul
    {   86, 4316,   53759,  4, }, // Asia/Kamchatka
    {   86, 4239,   53763,  3, }, // Asia/Karachi
    {   86, 1185,   53766,  5, }, // Asia/Kathmandu
    {   86, 8661,   53771,  4, }, // Asia/Khandyga
    {   86,  992,   53775,  4, }, // Asia/Kolkata
    {   86, 4197,   53779,  6, }, // Asia/Krasnoyarsk
    {   86, 4859,   53785,  7, }, // Asia/Kuala_Lumpur
    {   86, 5976,   53792,  4, }, // Asia/Kuching
    {   86, 5547,   53796,  3, }, // Asia/Kuwait
    {   86, 1211,   53799,  3, }, // Asia/Macau
    {   86, 4017,   53802,  4, }, // Asia/Magadan
    {   86, 1349,   53806,  4, }, // Asia/Makassar
    {   86, 7182,   53810,  3, }, // Asia/Manila
    {   86, 6920,   53813,  4, }, // Asia/Muscat
    {   86, 2554,   53817,  4, }, // Asia/Nicosia
    {   86, 8675,   53821,  7, }, // Asia/Novokuznetsk
    {   86, 4135,   53828,  6, }, // Asia/Novosibirsk
    {   86, 4229,   53834,  4, }, // Asia/Omsk
    {   86, 7145,   53838,  3, }, // Asia/Oral
    {   86, 6961,   53841,  5, }, // Asia/Phnom_Penh
    {   86, 8693,   53846,  6, }, // Asia/Pontianak
    {   86, 4214,   53852,  7, }, // Asia/Pyongyang
    {   86, 5559,   53859,  3, }, // Asia/Qatar
    {   86, 8708,   53862,  4, }, // Asia/Qostanay
    {   86, 4269,   53866,  5, }, // Asia/Qyzylorda
    {   86, 3584,   53871,  3, }, // Asia/Riyadh
    {   86, 4408,   53874,  4, }, // Asia/Sakhalin
    {   86, 7245,   53878,  5, }, // Asia/Samarkand
    {   86, 3279,   53883,  2, }, // Asia/Seoul
    {   86, 1053,   53885,  5, }, // Asia/Shanghai
    {   86, 3300,   53890,  5, }, // Asia/Singapore
    {   86, 4297,   53895,  9, }, // Asia/Srednekolymsk
    {   86, 3263,   53904,  3, }, // Asia/Taipei
    {   86, 4781,   53907,  5, }, // Asia/Tashkent
    {   86, 3915,   53912,  4, }, // Asia/Tbilisi
    {   86, 2772,   53916,  3, }, // Asia/Tehran
    {   86, 1317,   53919,  3, }, // Asia/Thimphu
    {   86, 2821,   53922,  3, }, // Asia/Tokyo
    {   86, 4572,   53925,  4, }, // Asia/Tomsk
    {   86, 1021,   53929,  5, }, // Asia/Ulaanbaatar
    {   86, 1159,   53934,  4, }, // Asia/Urumqi
    {   86, 8722,   53938,  6, }, // Asia/Ust-Nera
    {   86, 6977,   53944,  6, }, // Asia/Vientiane
    {   86, 4710,   53950,  7, }, // Asia/Vladivostok
    {   86, 4828,   53957,  4, }, // Asia/Yakutsk
    {   86, 1235,   53961,  4, }, // Asia/Yangon
    {   86, 3883,   53965,  7, }, // Asia/Yekaterinburg
    {   86, 3747,   53972,  4, }, // Asia/Yerevan
    {   86, 3663,   53976,  4, }, // Atlantic/Azores
    {   86, 5671,   53980,  3, }, // Atlantic/Bermuda
    {   86, 6628,   53983,  3, }, // Atlantic/Canary
    {   86, 3727,   53986, 11, }, // Atlantic/Cape_Verde
    {   86, 1395,   53997,  2, }, // Atlantic/Faroe
    {   86, 8736,   53999,  3, }, // Atlantic/Madeira
    {   86, 2748,   54002,  4, }, // Atlantic/Reykjavik
    {   86, 7272,   54006,  9, }, // Atlantic/South_Georgia
    {   86, 6821,   54015,  8, }, // Atlantic/St_Helena
    {   86, 6644,   54023,  4, }, // Atlantic/Stanley
    {   86, 1670,   54027,  4, }, // Australia/Adelaide
    {   86, 1635,   54031,  4, }, // Australia/Brisbane
    {   86, 1799,   54035,  8, }, // Australia/Broken_Hill
    {   86, 1583,   54043,  3, }, // Australia/Darwin
    {   86, 3637,   54046,  2, }, // Australia/Eucla
    {   86, 1516,   54048,  3, }, // Australia/Hobart
    {   86, 8753,   54051,  5, }, // Australia/Lindeman
    {   86, 1547,   54056,  8, }, // Australia/Lord_Howe
    {   86, 1727,   54064,  4, }, // Australia/Melbourne
    {   86, 1762,   54068,  3, }, // Australia/Perth
    {   86, 1463,   54071,  3, }, // Australia/Sydney
    {   86, 6441,   54074,  6, }, // Europe/Amsterdam
    {   86, 6216,   54080,  4, }, // Europe/Andorra
    {   86, 3620,   54084,  4, }, // Europe/Astrakhan
    {   86, 2300,   54088,  4, }, // Europe/Athens
    {   86, 6336,   54092,  4, }, // Europe/Belgrade
    {   86, 4757,   54096,  3, }, // Europe/Berlin
    {   86, 6488,   54099,  5, }, // Europe/Bratislava
    {   86, 2167,   54104,  4, }, // Europe/Brussels
    {   86, 3928,   54108,  5, }, // Europe/Bucharest
    {   86, 3806,   54113,  5, }, // Europe/Budapest
    {   86, 8772,   54118,  5, }, // Europe/Busingen
    {   86, 2583,   54123,  4, }, // Europe/Chisinau
    {   86, 6289,   54127,  6, }, // Europe/Copenhagen
    {   86, 2338,   54133,  3, }, // Europe/Dublin
    {   86, 6307,   54136,  4, }, // Europe/Gibraltar
    {   86, 8788,   54140,  3, }, // Europe/Guernsey
    {   86, 6612,   54143,  5, }, // Europe/Helsinki
    {   86, 8804,   54148, 15, }, // Europe/Isle_of_Man
    {   86, 1130,   54163,  6, }, // Europe/Istanbul
    {   86, 8823,   54169,  3, }, // Europe/Jersey
    {   86, 3979,   54172,  5, }, // Europe/Kaliningrad
    {   86, 8929,   54177,  3, }, // Europe/Kirov
    {   86, 2527,   54180,  2, }, // Europe/Kyiv
    {   86, 3213,   54182,  4, }, // Europe/Lisbon
    {   86, 6506,   54186,  3, }, // Europe/Ljubljana
    {   86, 2501,   54189,  4, }, // Europe/London
    {   86, 6365,   54193,  5, }, // Europe/Luxembourg
    {   86, 4892,   54198,  3, }, // Europe/Madrid
    {   86, 6397,   54201,  3, }, // Europe/Malta
    {   86, 6582,   54204,  4, }, // Europe/Mariehamn
    {   86, 3693,   54208,  4, }, // Europe/Minsk
    {   86, 6410,   54212,  3, }, // Europe/Monaco
    {   86, 3537,   54215,  4, }, // Europe/Moscow
    {   86, 6458,   54219,  3, }, // Europe/Oslo
    {   86, 4284,   54222,  3, }, // Europe/Paris
    {   86, 6424,   54225,  4, }, // Europe/Podgorica
    {   86, 6275,   52698,  2, }, // Europe/Prague
    {   86, 8837,   53052,  2, }, // Europe/Riga
    {   86, 6324,   53695,  2, }, // Europe/Rome
    {   86, 4331,   54229,  3, }, // Europe/Samara
    {   86, 6470,   54232,  3, }, // Europe/San_Marino
    {   86, 6245,   54235,  4, }, // Europe/Sarajevo
    {   86, 4451,   54239,  4, }, // Europe/Saratov
    {   86, 8849,   54243,  6, }, // Europe/Simferopol
    {   86, 6383,   54249,  3, }, // Europe/Skopje
    {   86, 6599,   54252,  3, }, // Europe/Sofia
    {   86, 6523,   54255,  5, }, // Europe/Stockholm
    {   86, 8867,   54260,  3, }, // Europe/Tallinn
    {   86, 6202,   54263,  3, }, // Europe/Tirane
    {   86, 8882,   54266,  7, }, // Europe/Ulyanovsk
    {   86, 6352,   54273,  3, }, // Europe/Vaduz
    {   86, 6567,   54276,  5, }, // Europe/Vatican
    {   86, 6231,   54281,  3, }, // Europe/Vienna
    {   86, 8899,   54284,  5, }, // Europe/Vilnius
    {   86, 4727,   54289,  5, }, // Europe/Volgograd
    {   86, 3190,   54294,  8, }, // Europe/Warsaw
    {   86, 6261,   54302,  3, }, // Europe/Zagreb
    {   86, 6540,   54305,  3, }, // Europe/Zurich
    {   86, 5078,   54308,  7, }, // Indian/Antananarivo
    {   86, 6947,   54315,  3, }, // Indian/Chagos
    {   86, 6046,   54318,  7, }, // Indian/Christmas
    {   86, 6063,   54325,  3, }, // Indian/Cocos
    {   86, 5029,   54328,  3, }, // Indian/Comoro
    {   86, 6661,   54331,  6, }, // Indian/Kerguelen
    {   86, 7260,   54337,  2, }, // Indian/Mahe
    {   86, 7050,   54339,  5, }, // Indian/Maldives
    {   86, 4069,   54344,  5, }, // Indian/Mauritius
    {   86, 5098,   54349,  3, }, // Indian/Mayotte
    {   86, 7211,   54352,  5, }, // Indian/Reunion
    {   86, 4422,   54357,  3, }, // Pacific/Apia
    {   86,  932,   54360,  5, }, // Pacific/Auckland
    {   86, 3706,   54365,  5, }, // Pacific/Bougainville
    {   86, 3015,   54370,  3, }, // Pacific/Chatham
    {   86, 3157,   54373,  2, }, // Pacific/Chuuk
    {   86, 2237,   54375,  7, }, // Pacific/Easter
    {   86, 7360,   54382,  3, }, // Pacific/Efate
    {   86, 7327,   54385,  4, }, // Pacific/Fakaofo
    {   86, 3902,   54389,  2, }, // Pacific/Fiji
    {   86, 7343,   54391,  4, }, // Pacific/Funafuti
    {   86, 6678,   54395, 12, }, // Pacific/Galapagos
    {   86, 6696,   54407,  4, }, // Pacific/Gambier
    {   86, 3822,   54411,  6, }, // Pacific/Guadalcanal
    {   86, 6018,   54417,  2, }, // Pacific/Guam
    {   86, 2723,   54419,  4, }, // Pacific/Honolulu
    {   86, 3049,   54423,  4, }, // Pacific/Kanton
    {   86, 3998,   54427,  5, }, // Pacific/Kiritimati
    {   86, 7019,   54432,  3, }, // Pacific/Kosrae
    {   86, 2842,   54435,  5, }, // Pacific/Kwajalein
    {   86, 4877,   54440,  3, }, // Pacific/Majuro
    {   86, 4051,   54443,  5, }, // Pacific/Marquesas
    {   86, 8914,   54448,  5, }, // Pacific/Midway
    {   86, 7084,   54453,  3, }, // Pacific/Nauru
    {   86, 7132,   54045,  2, }, // Pacific/Niue
    {   86, 4168,   54456,  5, }, // Pacific/Norfolk
    {   86, 7098,   54461,  4, }, // Pacific/Noumea
    {   86, 3126,   54465,  5, }, // Pacific/Pago_Pago
    {   86, 7155,   54470,  4, }, // Pacific/Palau
    {   86, 7194,   54474,  5, }, // Pacific/Pitcairn
    {   86, 3096,   54479,  3, }, // Pacific/Pohnpei
    {   86, 4807,   54482, 13, }, // Pacific/Port_Moresby
    {   86, 6076,   54495,  5, }, // Pacific/Rarotonga
    {   86, 6031,   54500,  4, }, // Pacific/Saipan
    {   86, 7312,   54504,  3, }, // Pacific/Tahiti
    {   86, 6712,   54507,  3, }, // Pacific/Tarawa
    {   86, 4583,   54510,  5, }, // Pacific/Tongatapu
    {   86, 7392,   54515,  3, }, // Pacific/Wake
    {   86, 7405,   54518,  3, }, // Pacific/Wallis
    {   87,   14,       0,  6, }, // Africa/Asmara Chickasaw/Latin/United States
    {   87, 4435,       6,  8, }, // Africa/Sao_Tome
    {   87, 4252,      14,  8, }, // America/Asuncion
    {   87,  237,      22,  8, }, // America/Atikokan
    {   87, 7623,      30, 17, }, // America/Bahia_Banderas
    {   87, 3868,      47,  6, }, // America/Cancun
    {   87, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   87, 5723,      66,  7, }, // America/Curacao
    {   87,  481,      73, 13, }, // America/Indiana/Knox
    {   87, 7961,      86, 16, }, // America/Indiana/Marengo
    {   87, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   87, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   87, 8038,     139, 14, }, // America/Indiana/Vevay
    {   87, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   87, 8086,     171, 16, }, // America/Indiana/Winamac
    {   87, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   87, 5932,     207, 22, }, // America/Lower_Princes
    {   87, 8220,     229,  6, }, // America/Merida
    {   87, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   87, 8329,     255, 20, }, // America/North_Dakota/Center
    {   87, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   87,  393,     298,  4, }, // America/Nuuk
    {   87, 6899,     302, 16, }, // America/Scoresbysund
    {   87, 8508,     318, 14, }, // America/St_Barthelemy
    {   87, 2061,     332, 10, }, // America/St_Johns
    {   87, 5863,     342,  9, }, // America/St_Kitts
    {   87, 5880,     351,  9, }, // America/St_Lucia
    {   87,  855,     360, 10, }, // America/St_Thomas
    {   87, 5913,     370, 11, }, // America/St_Vincent
    {   87, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   87, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   87, 1185,     413,  9, }, // Asia/Kathmandu
    {   87,  992,     422,  7, }, // Asia/Kolkata
    {   87, 1211,     429,  5, }, // Asia/Macau
    {   87, 1235,     442,  6, }, // Asia/Yangon
    {   87, 1395,     448,  5, }, // Atlantic/Faroe
    {   87, 6821,     453, 10, }, // Atlantic/St_Helena
    {   87, 2527,     463,  4, }, // Europe/Kyiv
    {   87, 7211,     467,  7, }, // Indian/Reunion
    {   87, 3157,     474,  5, }, // Pacific/Chuuk
    {   87, 3096,     479,  7, }, // Pacific/Pohnpei
    {   88,   14,       0,  6, }, // Africa/Asmara Chiga/Latin/Uganda
    {   88, 4435,       6,  8, }, // Africa/Sao_Tome
    {   88, 4252,      14,  8, }, // America/Asuncion
    {   88,  237,      22,  8, }, // America/Atikokan
    {   88, 7623,      30, 17, }, // America/Bahia_Banderas
    {   88, 3868,      47,  6, }, // America/Cancun
    {   88, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   88, 5723,      66,  7, }, // America/Curacao
    {   88,  481,      73, 13, }, // America/Indiana/Knox
    {   88, 7961,      86, 16, }, // America/Indiana/Marengo
    {   88, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   88, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   88, 8038,     139, 14, }, // America/Indiana/Vevay
    {   88, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   88, 8086,     171, 16, }, // America/Indiana/Winamac
    {   88, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   88, 5932,     207, 22, }, // America/Lower_Princes
    {   88, 8220,     229,  6, }, // America/Merida
    {   88, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   88, 8329,     255, 20, }, // America/North_Dakota/Center
    {   88, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   88,  393,     298,  4, }, // America/Nuuk
    {   88, 6899,     302, 16, }, // America/Scoresbysund
    {   88, 8508,     318, 14, }, // America/St_Barthelemy
    {   88, 2061,     332, 10, }, // America/St_Johns
    {   88, 5863,     342,  9, }, // America/St_Kitts
    {   88, 5880,     351,  9, }, // America/St_Lucia
    {   88,  855,     360, 10, }, // America/St_Thomas
    {   88, 5913,     370, 11, }, // America/St_Vincent
    {   88, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   88, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   88, 1185,     413,  9, }, // Asia/Kathmandu
    {   88,  992,     422,  7, }, // Asia/Kolkata
    {   88, 1211,     429,  5, }, // Asia/Macau
    {   88, 1235,     442,  6, }, // Asia/Yangon
    {   88, 1395,     448,  5, }, // Atlantic/Faroe
    {   88, 6821,     453, 10, }, // Atlantic/St_Helena
    {   88, 2527,     463,  4, }, // Europe/Kyiv
    {   88, 7211,     467,  7, }, // Indian/Reunion
    {   88, 3157,     474,  5, }, // Pacific/Chuuk
    {   88, 3096,     479,  7, }, // Pacific/Pohnpei
    {   89, 6788,   42949,  3, }, // Africa/Abidjan Chinese/Simplified Han/China
    {   89, 6760,   41438,  3, }, // Africa/Accra
    {   89, 5059,   54521,  6, }, // Africa/Addis_Ababa
    {   89, 7420,   42958,  4, }, // Africa/Algiers
    {   89,   14,   54527,  4, }, // Africa/Asmara
    {   89,   44,   42966,  3, }, // Africa/Bamako
    {   89, 5257,   41458,  2, }, // Africa/Bangui
    {   89, 6746,   54531,  3, }, // Africa/Banjul
    {   89, 7435,   42969,  2, }, // Africa/Bissau
    {   89, 4957,   42971,  4, }, // Africa/Blantyre
    {   89, 5287,   42975,  5, }, // Africa/Brazzaville
    {   89, 4922,   54534,  4, }, // Africa/Bujumbura
    {   89, 2320,   42980,  2, }, // Africa/Cairo
    {   89, 4117,   42982,  5, }, // Africa/Casablanca
    {   89, 7449,   42987,  2, }, // Africa/Ceuta
    {   89, 6773,   54538,  4, }, // Africa/Conakry
    {   89, 6840,   42989,  3, }, // Africa/Dakar
    {   89, 5130,   54542,  6, }, // Africa/Dar_es_Salaam
    {   89, 5043,   54548,  3, }, // Africa/Djibouti
    {   89, 5243,   41499,  3, }, // Africa/Douala
    {   89, 5166,   41502,  3, }, // Africa/El_Aaiun
    {   89, 6853,   54551,  3, }, // Africa/Freetown
    {   89, 4906,   54554,  4, }, // Africa/Gaborone
    {   89, 5015,   41512,  3, }, // Africa/Harare
    {   89, 4479,   54558,  5, }, // Africa/Johannesburg
    {   89, 4499,   41520,  2, }, // Africa/Juba
    {   89, 5151,   41522,  3, }, // Africa/Kampala
    {   89, 4524,   41525,  3, }, // Africa/Khartoum
    {   89, 4973,   41528,  3, }, // Africa/Kigali
    {   89, 5306,   54563,  3, }, // Africa/Kinshasa
    {   89, 4744,   54566,  3, }, // Africa/Lagos
    {   89, 5336,   54569,  4, }, // Africa/Libreville
    {   89, 6869,   41540,  2, }, // Africa/Lome
    {   89, 5211,   43000,  3, }, // Africa/Luanda
    {   89, 4939,   43003,  4, }, // Africa/Lubumbashi
    {   89, 5001,   54573,  3, }, // Africa/Lusaka
    {   89, 5322,   43007,  3, }, // Africa/Malabo
    {   89, 4987,   43010,  3, }, // Africa/Maputo
    {   89, 5197,   54576,  3, }, // Africa/Maseru
    {   89, 5182,   54579,  4, }, // Africa/Mbabane
    {   89, 5113,   54583,  4, }, // Africa/Mogadishu
    {   89, 7034,   43016,  4, }, // Africa/Monrovia
    {   89, 3853,   54587,  3, }, // Africa/Nairobi
    {   89, 5271,   43020,  4, }, // Africa/Ndjamena
    {   89, 5354,   43024,  3, }, // Africa/Niamey
    {   89, 6803,   54590,  5, }, // Africa/Nouakchott
    {   89, 6727,   41585,  4, }, // Africa/Ouagadougou
    {   89, 5225,   54595,  4, }, // Africa/Porto-Novo
    {   89, 4435,   43034,  3, }, // Africa/Sao_Tome
    {   89, 2866,   41596,  4, }, // Africa/Tripoli
    {   89, 6554,   41600,  3, }, // Africa/Tunis
    {   89, 4152,   43037,  4, }, // Africa/Windhoek
    {   89,  132,   54599,  3, }, // America/Adak
    {   89, 3346,   54602,  4, }, // America/Anchorage
    {   89, 5607,   54606,  3, }, // America/Anguilla
    {   89, 5624,   54609,  3, }, // America/Antigua
    {   89, 4554,   54612,  5, }, // America/Araguaina
    {   89,  166,   43044,  7, }, // America/Argentina/Buenos_Aires
    {   89,   91,   43051,  4, }, // America/Argentina/Catamarca
    {   89,  270,   54617,  4, }, // America/Argentina/Cordoba
    {   89,  441,   41638,  3, }, // America/Argentina/Jujuy
    {   89, 7462,   54621,  4, }, // America/Argentina/La_Rioja
    {   89,  565,   43058,  3, }, // America/Argentina/Mendoza
    {   89, 7489,   43061,  6, }, // America/Argentina/Rio_Gallegos
    {   89, 7520,   43067,  3, }, // America/Argentina/Salta
    {   89, 7544,   43070,  3, }, // America/Argentina/San_Juan
    {   89, 5580,   54625,  4, }, // America/Argentina/San_Luis
    {   89, 7571,   54629,  3, }, // America/Argentina/Tucuman
    {   89, 7597,   43079,  4, }, // America/Argentina/Ushuaia
    {   89, 5640,   54632,  3, }, // America/Aruba
    {   89, 4252,   43083,  3, }, // America/Asuncion
    {   89,  237,   41675,  4, }, // America/Atikokan
    {   89, 3679,   43086,  3, }, // America/Bahia
    {   89, 7623,   43086,  7, }, // America/Bahia_Banderas
    {   89, 5654,   54635,  4, }, // America/Barbados
    {   89, 7646,   43096,  2, }, // America/Belem
    {   89, 5407,   54639,  3, }, // America/Belize
    {   89, 7660,   54642,  6, }, // America/Blanc-Sablon
    {   89, 7681,   54648,  5, }, // America/Boa_Vista
    {   89, 4361,   41706,  3, }, // America/Bogota
    {   89, 7699,   54653,  3, }, // America/Boise
    {   89, 7713,   43110,  3, }, // America/Cambridge_Bay
    {   89, 7735,   54656,  3, }, // America/Campo_Grande
    {   89, 3868,   41717,  2, }, // America/Cancun
    {   89, 4694,   54659,  4, }, // America/Caracas
    {   89, 4345,   54663,  2, }, // America/Cayenne
    {   89, 5496,   43118,  2, }, // America/Cayman
    {   89, 2260,   41729,  3, }, // America/Chicago
    {   89, 7756,   54665,  3, }, // America/Chihuahua
    {   89, 7774,   43125,  4, }, // America/Ciudad_Juarez
    {   89, 5422,   54668,  5, }, // America/Costa_Rica
    {   89, 7796,   43129,  4, }, // America/Creston
    {   89, 3791,   54673,  3, }, // America/Cuiaba
    {   89, 5723,   43133,  3, }, // America/Curacao
    {   89, 7812,   43136,  4, }, // America/Danmarkshavn
    {   89, 7833,   41758,  2, }, // America/Dawson
    {   89, 7848,   41760,  5, }, // America/Dawson_Creek
    {   89,  805,   41765,  2, }, // America/Denver
    {   89, 3465,   41767,  3, }, // America/Detroit
    {   89, 5739,   54676,  4, }, // America/Dominica
    {   89,  893,   54680,  4, }, // America/Edmonton
    {   89, 7869,   54684,  4, }, // America/Eirunepe
    {   89, 5441,   43144,  4, }, // America/El_Salvador
    {   89, 7886,   43148,  4, }, // America/Fort_Nelson
    {   89, 7906,   54688,  4, }, // America/Fortaleza
    {   89, 7924,   54692,  4, }, // America/Glace_Bay
    {   89, 6881,   54696,  3, }, // America/Goose_Bay
    {   89, 4612,   41800,  3, }, // America/Grand_Turk
    {   89, 5770,   54699,  4, }, // America/Grenada
    {   89, 5786,   54703,  4, }, // America/Guadeloupe
    {   89, 3760,   54707,  4, }, // America/Guatemala
    {   89, 4841,   43170,  4, }, // America/Guayaquil
    {   89, 6932,   54711,  3, }, // America/Guyana
    {   89, 1939,   54714,  5, }, // America/Halifax
    {   89, 2281,   41828,  3, }, // America/Havana
    {   89, 7942,   43177,  4, }, // America/Hermosillo
    {   89,  348,   54719,  7, }, // America/Indiana/Indianapolis
    {   89,  481,   54726,  8, }, // America/Indiana/Knox
    {   89, 7961,   54734,  8, }, // America/Indiana/Marengo
    {   89, 7985,   54742,  9, }, // America/Indiana/Petersburg
    {   89, 8012,   54751,  8, }, // America/Indiana/Tell_City
    {   89, 8038,   54759,  8, }, // America/Indiana/Vevay
    {   89, 8060,   54767,  8, }, // America/Indiana/Vincennes
    {   89, 8086,   54775,  9, }, // America/Indiana/Winamac
    {   89, 8110,   54784,  4, }, // America/Inuvik
    {   89,  660,   41902,  3, }, // America/Iqaluit
    {   89, 2799,   43233,  3, }, // America/Jamaica
    {   89, 5380,   43236,  2, }, // America/Juneau
    {   89,  521,   43238,  5, }, // America/Kentucky/Louisville
    {   89, 8125,   54788,  8, }, // America/Kentucky/Monticello
    {   89, 5704,   43251,  5, }, // America/Kralendijk
    {   89, 4376,   41928,  3, }, // America/La_Paz
    {   89, 7169,   43256,  2, }, // America/Lima
    {   89, 3239,   43258,  3, }, // America/Los_Angeles
    {   89, 5932,   43261,  4, }, // America/Lower_Princes
    {   89, 8153,   54796,  3, }, // America/Maceio
    {   89, 8168,   54799,  3, }, // America/Managua
    {   89, 1908,   54802,  3, }, // America/Manaus
    {   89, 5897,   43274,  4, }, // America/Marigot
    {   89, 5805,   54805,  4, }, // America/Martinique
    {   89, 8184,   43282,  5, }, // America/Matamoros
    {   89, 2917,   43287,  4, }, // America/Mazatlan
    {   89, 8202,   54809,  4, }, // America/Menominee
    {   89, 8220,   43295,  3, }, // America/Merida
    {   89, 8235,   41973,  6, }, // America/Metlakatla
    {   89, 2949,   54813,  4, }, // America/Mexico_City
    {   89, 4391,   54817,  3, }, // America/Miquelon
    {   89, 8254,   43301,  3, }, // America/Moncton
    {   89, 8270,   54820,  3, }, // America/Monterrey
    {   89, 4098,   54823,  5, }, // America/Montevideo
    {   89, 5824,   54828,  5, }, // America/Montserrat
    {   89, 5481,   54833,  2, }, // America/Nassau
    {   89, 2379,   43311,  2, }, // America/New_York
    {   89, 8288,   43313,  2, }, // America/Nome
    {   89, 1850,   54835,  4, }, // America/Noronha
    {   89, 8301,   54839,  8, }, // America/North_Dakota/Beulah
    {   89, 8329,   54847,  7, }, // America/North_Dakota/Center
    {   89, 8357,   54854,  9, }, // America/North_Dakota/New_Salem
    {   89,  393,   42031,  2, }, // America/Nuuk
    {   89, 8388,   43341,  4, }, // America/Ojinaga
    {   89, 2356,   43345,  3, }, // America/Panama
    {   89, 6173,   54863,  5, }, // America/Paramaribo
    {   89, 2973,   43353,  3, }, // America/Phoenix
    {   89, 3945,   42048,  3, }, // America/Port-au-Prince
    {   89, 5954,   42051,  4, }, // America/Port_of_Spain
    {   89, 8404,   54868,  4, }, // America/Porto_Velho
    {   89, 5843,   42058,  4, }, // America/Puerto_Rico
    {   89, 4030,   43359,  6, }, // America/Punta_Arenas
    {   89, 8424,   43365,  3, }, // America/Rankin_Inlet
    {   89, 8445,   54872,  3, }, // America/Recife
    {   89, 1995,   43368,  3, }, // America/Regina
    {   89, 8460,   54875,  4, }, // America/Resolute
    {   89,  695,   54879,  5, }, // America/Rio_Branco
    {   89, 8477,   43379,  3, }, // America/Santarem
    {   89, 2201,   54884,  4, }, // America/Santiago
    {   89, 6111,   54888,  4, }, // America/Santo_Domingo
    {   89, 1878,   43390,  3, }, // America/Sao_Paulo
    {   89, 6899,   54892,  7, }, // America/Scoresbysund
    {   89, 8494,   43400,  3, }, // America/Sitka
    {   89, 8508,   54899,  6, }, // America/St_Barthelemy
    {   89, 2061,   54905,  4, }, // America/St_Johns
    {   89, 5863,   43412,  3, }, // America/St_Kitts
    {   89, 5880,   54909,  4, }, // America/St_Lucia
    {   89,  855,   43419,  4, }, // America/St_Thomas
    {   89, 5913,   54913,  4, }, // America/St_Vincent
    {   89, 8530,   43426,  7, }, // America/Swift_Current
    {   89, 5461,   54917,  6, }, // America/Tegucigalpa
    {   89, 5756,   54923,  2, }, // America/Thule
    {   89,  313,   54925,  3, }, // America/Tijuana
    {   89,  608,   43436,  3, }, // America/Toronto
    {   89, 5688,   43439,  4, }, // America/Tortola
    {   89, 2093,   43443,  3, }, // America/Vancouver
    {   89, 2144,   43446,  4, }, // America/Whitehorse
    {   89,  734,   43450,  3, }, // America/Winnipeg
    {   89, 8552,   54928,  4, }, // America/Yakutat
    {   89, 6001,   54932,  2, }, // Antarctica/Casey
    {   89, 6094,   43458,  3, }, // Antarctica/Davis
    {   89, 6133,   54934,  6, }, // Antarctica/DumontDUrville
    {   89, 8568,   54940,  3, }, // Antarctica/Macquarie
    {   89, 7066,   42182,  2, }, // Antarctica/Mawson
    {   89, 7113,   43469,  4, }, // Antarctica/McMurdo
    {   89, 8589,   54943,  3, }, // Antarctica/Palmer
    {   89, 7226,   43475,  3, }, // Antarctica/Rothera
    {   89, 7295,   42193,  2, }, // Antarctica/Syowa
    {   89, 8607,   54946,  3, }, // Antarctica/Troll
    {   89, 7374,   42199,  4, }, // Antarctica/Vostok
    {   89, 1429,   54949,  4, }, // Arctic/Longyearbyen
    {   89, 5570,   43480,  2, }, // Asia/Aden
    {   89, 5395,   43482,  4, }, // Asia/Almaty
    {   89, 3968,   42214,  2, }, // Asia/Amman
    {   89, 5511,   54953,  4, }, // Asia/Anadyr
    {   89, 5523,   42219,  3, }, // Asia/Aqtau
    {   89, 5368,   54957,  4, }, // Asia/Aqtobe
    {   89,  964,   54961,  5, }, // Asia/Ashgabat
    {   89, 8624,   43486,  3, }, // Asia/Atyrau
    {   89, 3607,   43489,  3, }, // Asia/Baghdad
    {   89, 5534,   42237,  2, }, // Asia/Bahrain
    {   89, 3653,   43492,  2, }, // Asia/Baku
    {   89, 4466,   42239,  2, }, // Asia/Bangkok
    {   89, 3571,   43494,  4, }, // Asia/Barnaul
    {   89, 4086,   43498,  3, }, // Asia/Beirut
    {   89, 3778,   43501,  4, }, // Asia/Bishkek
    {   89, 5989,   54966,  2, }, // Asia/Brunei
    {   89, 4601,   42254,  2, }, // Asia/Chita
    {   89, 4511,   54968,  3, }, // Asia/Colombo
    {   89, 4540,   43510,  4, }, // Asia/Damascus
    {   89, 1093,   43514,  2, }, // Asia/Dhaka
    {   89, 6192,   42265,  2, }, // Asia/Dili
    {   89, 3596,   54971,  2, }, // Asia/Dubai
    {   89, 6159,   54973,  3, }, // Asia/Dushanbe
    {   89, 8636,   43519,  5, }, // Asia/Famagusta
    {   89, 8651,   54976,  2, }, // Asia/Gaza
    {   89, 4795,   54978,  3, }, // Asia/Hebron
    {   89, 1259,   42282,  4, }, // Asia/Ho_Chi_Minh
    {   89, 2704,   42293,  2, }, // Asia/Hong_Kong
    {   89, 4771,   42300,  3, }, // Asia/Hovd
    {   89, 4184,   54981,  5, }, // Asia/Irkutsk
    {   89, 7006,   43545,  3, }, // Asia/Jakarta
    {   89, 6992,   54986,  4, }, // Asia/Jayapura
    {   89, 1290,   42315,  4, }, // Asia/Jerusalem
    {   89, 3560,   43552,  3, }, // Asia/Kabul
    {   89, 4316,   42322,  3, }, // Asia/Kamchatka
    {   89, 4239,   54990,  3, }, // Asia/Karachi
    {   89, 1185,   43555,  4, }, // Asia/Kathmandu
    {   89, 8661,   54993,  3, }, // Asia/Khandyga
    {   89,  992,   43559,  4, }, // Asia/Kolkata
    {   89, 4197,   43563,  8, }, // Asia/Krasnoyarsk
    {   89, 4859,   42347,  3, }, // Asia/Kuala_Lumpur
    {   89, 5976,   43571,  2, }, // Asia/Kuching
    {   89, 5547,   42352,  3, }, // Asia/Kuwait
    {   89, 1211,   43580,  2, }, // Asia/Macau
    {   89, 4017,   43587,  3, }, // Asia/Magadan
    {   89, 1349,   54996,  3, }, // Asia/Makassar
    {   89, 7182,   43594,  3, }, // Asia/Manila
    {   89, 6920,   54999,  4, }, // Asia/Muscat
    {   89, 2554,   55003,  4, }, // Asia/Nicosia
    {   89, 8675,   43605,  6, }, // Asia/Novokuznetsk
    {   89, 4135,   43611,  5, }, // Asia/Novosibirsk
    {   89, 4229,   42398,  4, }, // Asia/Omsk
    {   89, 7145,   43616,  3, }, // Asia/Oral
    {   89, 6961,   43619,  2, }, // Asia/Phnom_Penh
    {   89, 8693,   42407,  2, }, // Asia/Pontianak
    {   89, 4214,   42409,  2, }, // Asia/Pyongyang
    {   89, 5559,   55007,  3, }, // Asia/Qatar
    {   89, 8708,   55010,  4, }, // Asia/Qostanay
    {   89, 4269,   55014,  5, }, // Asia/Qyzylorda
    {   89, 3584,   55019,  3, }, // Asia/Riyadh
    {   89, 4408,   55022,  3, }, // Asia/Sakhalin
    {   89, 7245,   43632,  4, }, // Asia/Samarkand
    {   89, 3279,   43636,  2, }, // Asia/Seoul
    {   89, 1053,   42435,  2, }, // Asia/Shanghai
    {   89, 3300,   42437,  3, }, // Asia/Singapore
    {   89, 4297,   42440,  6, }, // Asia/Srednekolymsk
    {   89, 3263,   42446,  2, }, // Asia/Taipei
    {   89, 4781,   42448,  3, }, // Asia/Tashkent
    {   89, 3915,   42451,  4, }, // Asia/Tbilisi
    {   89, 2772,   43638,  3, }, // Asia/Tehran
    {   89, 1317,   42458,  2, }, // Asia/Thimphu
    {   89, 2821,   43641,  2, }, // Asia/Tokyo
    {   89, 4572,   42462,  4, }, // Asia/Tomsk
    {   89, 1021,   43643,  4, }, // Asia/Ulaanbaatar
    {   89, 1159,   43647,  4, }, // Asia/Urumqi
    {   89, 8722,   43651,  4, }, // Asia/Ust-Nera
    {   89, 6977,   55025,  2, }, // Asia/Vientiane
    {   89, 4710,   55027,  7, }, // Asia/Vladivostok
    {   89, 4828,   55034,  4, }, // Asia/Yakutsk
    {   89, 1235,   42487,  2, }, // Asia/Yangon
    {   89, 3883,   55038,  5, }, // Asia/Yekaterinburg
    {   89, 3747,   55043,  3, }, // Asia/Yerevan
    {   89, 3663,   43670,  5, }, // Atlantic/Azores
    {   89, 5671,   55046,  3, }, // Atlantic/Bermuda
    {   89, 6628,   55049,  3, }, // Atlantic/Canary
    {   89, 3727,   55052,  3, }, // Atlantic/Cape_Verde
    {   89, 1395,   43684,  2, }, // Atlantic/Faroe
    {   89, 8736,   55055,  3, }, // Atlantic/Madeira
    {   89, 2748,   55058,  5, }, // Atlantic/Reykjavik
    {   89, 7272,   43698,  4, }, // Atlantic/South_Georgia
    {   89, 6821,   43702,  4, }, // Atlantic/St_Helena
    {   89, 6644,   55063,  3, }, // Atlantic/Stanley
    {   89, 1670,   55066,  4, }, // Australia/Adelaide
    {   89, 1635,   55070,  4, }, // Australia/Brisbane
    {   89, 1799,   43707,  5, }, // Australia/Broken_Hill
    {   89, 1583,   43712,  3, }, // Australia/Darwin
    {   89, 3637,   42553,  3, }, // Australia/Eucla
    {   89, 1516,   55074,  3, }, // Australia/Hobart
    {   89, 8753,   42559,  3, }, // Australia/Lindeman
    {   89, 1547,   43715,  4, }, // Australia/Lord_Howe
    {   89, 1727,   43719,  3, }, // Australia/Melbourne
    {   89, 1762,   55077,  2, }, // Australia/Perth
    {   89, 1463,   55079,  2, }, // Australia/Sydney
    {   89, 6441,   42573,  5, }, // Europe/Amsterdam
    {   89, 6216,   43722,  3, }, // Europe/Andorra
    {   89, 3620,   42581,  5, }, // Europe/Astrakhan
    {   89, 2300,   42586,  2, }, // Europe/Athens
    {   89, 6336,   55081,  5, }, // Europe/Belgrade
    {   89, 4757,   42592,  2, }, // Europe/Berlin
    {   89, 6488,   55086,  6, }, // Europe/Bratislava
    {   89, 2167,   43729,  4, }, // Europe/Brussels
    {   89, 3928,   42604,  5, }, // Europe/Bucharest
    {   89, 3806,   43733,  4, }, // Europe/Budapest
    {   89, 8772,   42613,  3, }, // Europe/Busingen
    {   89, 2583,   55092,  4, }, // Europe/Chisinau
    {   89, 6289,   42619,  4, }, // Europe/Copenhagen
    {   89, 2338,   42623,  3, }, // Europe/Dublin
    {   89, 6307,   43737,  4, }, // Europe/Gibraltar
    {   89, 8788,   55096,  3, }, // Europe/Guernsey
    {   89, 6612,   43744,  4, }, // Europe/Helsinki
    {   89, 8804,   55099,  3, }, // Europe/Isle_of_Man
    {   89, 1130,   55102,  5, }, // Europe/Istanbul
    {   89, 8823,   43750,  3, }, // Europe/Jersey
    {   89, 3979,   43753,  5, }, // Europe/Kaliningrad
    {   89, 8929,   42651,  3, }, // Europe/Kirov
    {   89, 2527,   43758,  2, }, // Europe/Kyiv
    {   89, 3213,   42656,  3, }, // Europe/Lisbon
    {   89, 6506,   55107,  5, }, // Europe/Ljubljana
    {   89, 2501,   43764,  2, }, // Europe/London
    {   89, 6365,   43766,  3, }, // Europe/Luxembourg
    {   89, 4892,   43769,  3, }, // Europe/Madrid
    {   89, 6397,   55112,  3, }, // Europe/Malta
    {   89, 6582,   43775,  3, }, // Europe/Mariehamn
    {   89, 3693,   42677,  3, }, // Europe/Minsk
    {   89, 6410,   43778,  3, }, // Europe/Monaco
    {   89, 3537,   42683,  3, }, // Europe/Moscow
    {   89, 6458,   43781,  3, }, // Europe/Oslo
    {   89, 4284,   42689,  2, }, // Europe/Paris
    {   89, 6424,   55115,  5, }, // Europe/Podgorica
    {   89, 6275,   42695,  3, }, // Europe/Prague
    {   89, 8837,   42698,  2, }, // Europe/Riga
    {   89, 6324,   43784,  2, }, // Europe/Rome
    {   89, 4331,   55120,  3, }, // Europe/Samara
    {   89, 6470,   55123,  4, }, // Europe/San_Marino
    {   89, 6245,   55127,  4, }, // Europe/Sarajevo
    {   89, 4451,   43793,  4, }, // Europe/Saratov
    {   89, 8849,   55131,  5, }, // Europe/Simferopol
    {   89, 6383,   55136,  4, }, // Europe/Skopje
    {   89, 6599,   55140,  3, }, // Europe/Sofia
    {   89, 6523,   43800,  5, }, // Europe/Stockholm
    {   89, 8867,   42733,  2, }, // Europe/Tallinn
    {   89, 6202,   42735,  3, }, // Europe/Tirane
    {   89, 8882,   43805,  7, }, // Europe/Ulyanovsk
    {   89, 6352,   55143,  3, }, // Europe/Vaduz
    {   89, 6567,   43815,  3, }, // Europe/Vatican
    {   89, 6231,   43818,  3, }, // Europe/Vienna
    {   89, 8899,   43821,  4, }, // Europe/Vilnius
    {   89, 4727,   43825,  5, }, // Europe/Volgograd
    {   89, 3190,   43830,  2, }, // Europe/Warsaw
    {   89, 6261,   55146,  4, }, // Europe/Zagreb
    {   89, 6540,   43832,  3, }, // Europe/Zurich
    {   89, 5078,   42772,  6, }, // Indian/Antananarivo
    {   89, 6947,   42778,  3, }, // Indian/Chagos
    {   89, 6046,   43835,  3, }, // Indian/Christmas
    {   89, 6063,   55150,  3, }, // Indian/Cocos
    {   89, 5029,   43843,  3, }, // Indian/Comoro
    {   89, 6661,   43848,  4, }, // Indian/Kerguelen
    {   89, 7260,   43853,  3, }, // Indian/Mahe
    {   89, 7050,   55153,  4, }, // Indian/Maldives
    {   89, 4069,   55157,  4, }, // Indian/Mauritius
    {   89, 5098,   43860,  3, }, // Indian/Mayotte
    {   89, 7211,   55161,  3, }, // Indian/Reunion
    {   89, 4422,   43868,  3, }, // Pacific/Apia
    {   89,  932,   43871,  3, }, // Pacific/Auckland
    {   89, 3706,   43874,  4, }, // Pacific/Bougainville
    {   89, 3015,   55164,  3, }, // Pacific/Chatham
    {   89, 3157,   55167,  5, }, // Pacific/Chuuk
    {   89, 2237,   55172,  4, }, // Pacific/Easter
    {   89, 7360,   42835,  3, }, // Pacific/Efate
    {   89, 7327,   42838,  3, }, // Pacific/Fakaofo
    {   89, 3902,   43881,  2, }, // Pacific/Fiji
    {   89, 7343,   55176,  4, }, // Pacific/Funafuti
    {   89, 6678,   55180,  4, }, // Pacific/Galapagos
    {   89, 6696,   43889,  3, }, // Pacific/Gambier
    {   89, 3822,   55184,  6, }, // Pacific/Guadalcanal
    {   89, 6018,   43900,  2, }, // Pacific/Guam
    {   89, 2723,   42866,  3, }, // Pacific/Honolulu
    {   89, 3049,   55190,  4, }, // Pacific/Kanton
    {   89, 3998,   43907,  6, }, // Pacific/Kiritimati
    {   89, 7019,   55194,  3, }, // Pacific/Kosrae
    {   89, 2842,   55197,  3, }, // Pacific/Kwajalein
    {   89, 4877,   55200,  3, }, // Pacific/Majuro
    {   89, 4051,   55203,  4, }, // Pacific/Marquesas
    {   89, 8914,   43925,  3, }, // Pacific/Midway
    {   89, 7084,   55207,  2, }, // Pacific/Nauru
    {   89, 7132,   43930,  2, }, // Pacific/Niue
    {   89, 4168,   43933,  3, }, // Pacific/Norfolk
    {   89, 7098,   55209,  3, }, // Pacific/Noumea
    {   89, 3126,   55212,  4, }, // Pacific/Pago_Pago
    {   89, 7155,   55216,  2, }, // Pacific/Palau
    {   89, 7194,   55218,  4, }, // Pacific/Pitcairn
    {   89, 3096,   55222,  4, }, // Pacific/Pohnpei
    {   89, 4807,   55226,  5, }, // Pacific/Port_Moresby
    {   89, 6076,   43947,  4, }, // Pacific/Rarotonga
    {   89, 6031,   42931,  2, }, // Pacific/Saipan
    {   89, 7312,   55231,  3, }, // Pacific/Tahiti
    {   89, 6712,   42936,  3, }, // Pacific/Tarawa
    {   89, 4583,   43951,  4, }, // Pacific/Tongatapu
    {   89, 7392,   42944,  2, }, // Pacific/Wake
    {   89, 7405,   42946,  3, }, // Pacific/Wallis
    {   94, 6899,   55234,  6, }, // America/Scoresbysund Chinese/Simplified Han/Singapore
    {   95, 6788,   55240,  3, }, // Africa/Abidjan Chinese/Traditional Han/Hong Kong
    {   95,   14,   55243,  4, }, // Africa/Asmara
    {   95, 5257,   55247,  2, }, // Africa/Bangui
    {   95, 6773,   55249,  4, }, // Africa/Conakry
    {   95, 5130,   55253,  6, }, // Africa/Dar_es_Salaam
    {   95, 4479,   55259,  5, }, // Africa/Johannesburg
    {   95, 5306,   55264,  3, }, // Africa/Kinshasa
    {   95, 5336,   55267,  3, }, // Africa/Libreville
    {   95, 5211,   55270,  3, }, // Africa/Luanda
    {   95, 5001,   55273,  3, }, // Africa/Lusaka
    {   95, 5197,   55276,  3, }, // Africa/Maseru
    {   95, 3853,   55279,  3, }, // Africa/Nairobi
    {   95, 4152,   55282,  4, }, // Africa/Windhoek
    {   95,  132,   55286,  3, }, // America/Adak
    {   95, 4554,   55289,  5, }, // America/Araguaina
    {   95,  270,   55294,  4, }, // America/Argentina/Cordoba
    {   95, 7462,   55298,  4, }, // America/Argentina/La_Rioja
    {   95, 5640,   55302,  3, }, // America/Aruba
    {   95, 4252,   55305,  3, }, // America/Asuncion
    {   95, 3679,   55308,  3, }, // America/Bahia
    {   95, 7623,   55311,  8, }, // America/Bahia_Banderas
    {   95, 5407,   55319,  3, }, // America/Belize
    {   95, 7681,   55322,  5, }, // America/Boa_Vista
    {   95, 7756,   55327,  3, }, // America/Chihuahua
    {   95, 7774,   41735,  4, }, // America/Ciudad_Juarez
    {   95, 5422,   55330,  5, }, // America/Costa_Rica
    {   95, 3791,   55335,  3, }, // America/Cuiaba
    {   95, 7848,   55338,  3, }, // America/Dawson_Creek
    {   95,  893,   55341,  3, }, // America/Edmonton
    {   95, 7906,   55344,  4, }, // America/Fortaleza
    {   95, 7924,   55348,  4, }, // America/Glace_Bay
    {   95, 5770,   55352,  4, }, // America/Grenada
    {   95, 5786,   55356,  4, }, // America/Guadeloupe
    {   95, 3760,   55360,  4, }, // America/Guatemala
    {   95, 6932,   55364,  3, }, // America/Guyana
    {   95, 2281,   55367,  3, }, // America/Havana
    {   95,  348,   55370,  7, }, // America/Indiana/Indianapolis
    {   95,  481,   55377,  8, }, // America/Indiana/Knox
    {   95, 7961,   55385,  8, }, // America/Indiana/Marengo
    {   95, 7985,   55393,  8, }, // America/Indiana/Petersburg
    {   95, 8012,   55401,  8, }, // America/Indiana/Tell_City
    {   95, 8038,   55409,  7, }, // America/Indiana/Vevay
    {   95, 8060,   55416,  8, }, // America/Indiana/Vincennes
    {   95, 8086,   55424,  9, }, // America/Indiana/Winamac
    {   95, 8110,   55433,  4, }, // America/Inuvik
    {   95,  521,   55437,  4, }, // America/Kentucky/Louisville
    {   95, 8153,   55441,  3, }, // America/Maceio
    {   95, 8168,   55444,  3, }, // America/Managua
    {   95, 5805,   55447,  4, }, // America/Martinique
    {   95, 8202,   55451,  4, }, // America/Menominee
    {   95, 4098,   55455,  5, }, // America/Montevideo
    {   95, 5824,   55460,  5, }, // America/Montserrat
    {   95, 5481,   55465,  2, }, // America/Nassau
    {   95, 1850,   55467,  4, }, // America/Noronha
    {   95, 8301,   55471,  8, }, // America/North_Dakota/Beulah
    {   95, 8329,   55479,  8, }, // America/North_Dakota/Center
    {   95, 8357,   55487,  9, }, // America/North_Dakota/New_Salem
    {   95, 8404,   55496,  3, }, // America/Porto_Velho
    {   95,  695,   55499,  5, }, // America/Rio_Branco
    {   95, 2201,   55504,  4, }, // America/Santiago
    {   95, 6111,   55508,  4, }, // America/Santo_Domingo
    {   95, 8508,   55512,  5, }, // America/St_Barthelemy
    {   95, 2061,   55517,  4, }, // America/St_Johns
    {   95, 5880,   55521,  4, }, // America/St_Lucia
    {   95, 5913,   55525,  4, }, // America/St_Vincent
    {   95, 5461,   55529,  6, }, // America/Tegucigalpa
    {   95, 5756,   55535,  2, }, // America/Thule
    {   95,  313,   55537,  3, }, // America/Tijuana
    {   95, 2144,   55540,  3, }, // America/Whitehorse
    {   95, 8552,   55543,  4, }, // America/Yakutat
    {   95, 6001,   55547,  3, }, // Antarctica/Casey
    {   95, 6094,   55550,  4, }, // Antarctica/Davis
    {   95, 6133,   55554,  6, }, // Antarctica/DumontDUrville
    {   95, 8568,   55560,  3, }, // Antarctica/Macquarie
    {   95, 7066,   55563,  3, }, // Antarctica/Mawson
    {   95, 7113,   55566,  5, }, // Antarctica/McMurdo
    {   95, 8589,   55571,  3, }, // Antarctica/Palmer
    {   95, 7226,   55574,  4, }, // Antarctica/Rothera
    {   95, 7295,   55578,  3, }, // Antarctica/Syowa
    {   95, 8607,   55581,  4, }, // Antarctica/Troll
    {   95, 1429,   55585,  4, }, // Arctic/Longyearbyen
    {   95, 5511,   55589,  4, }, // Asia/Anadyr
    {   95, 3778,   55593,  4, }, // Asia/Bishkek
    {   95, 4511,   55597,  3, }, // Asia/Colombo
    {   95, 6159,   55600,  3, }, // Asia/Dushanbe
    {   95, 4795,   55603,  3, }, // Asia/Hebron
    {   95, 4184,   55606,  5, }, // Asia/Irkutsk
    {   95, 6992,   55611,  4, }, // Asia/Jayapura
    {   95, 1349,   55615,  2, }, // Asia/Makassar
    {   95, 6920,   55617,  4, }, // Asia/Muscat
    {   95, 2554,   55621,  4, }, // Asia/Nicosia
    {   95, 5559,   55625,  3, }, // Asia/Qatar
    {   95, 4828,   55628,  4, }, // Asia/Yakutsk
    {   95, 3883,   55632,  5, }, // Asia/Yekaterinburg
    {   95, 8736,   55637,  4, }, // Atlantic/Madeira
    {   95, 6644,   55641,  3, }, // Atlantic/Stanley
    {   95, 1670,   55644,  4, }, // Australia/Adelaide
    {   95, 1635,   55648,  4, }, // Australia/Brisbane
    {   95, 1516,   55652,  3, }, // Australia/Hobart
    {   95, 6336,   55655,  5, }, // Europe/Belgrade
    {   95, 6488,   55660,  6, }, // Europe/Bratislava
    {   95, 2583,   55666,  4, }, // Europe/Chisinau
    {   95, 8788,   55670,  3, }, // Europe/Guernsey
    {   95, 8804,   55673,  3, }, // Europe/Isle_of_Man
    {   95, 6506,   55676,  5, }, // Europe/Ljubljana
    {   95, 6397,   55681,  3, }, // Europe/Malta
    {   95, 6245,   55684,  4, }, // Europe/Sarajevo
    {   95, 6352,   55688,  3, }, // Europe/Vaduz
    {   95, 6261,   55691,  4, }, // Europe/Zagreb
    {   95, 6947,   55695,  5, }, // Indian/Chagos
    {   95, 6063,   55700,  5, }, // Indian/Cocos
    {   95, 5029,   42789,  3, }, // Indian/Comoro
    {   95, 6661,   55705,  6, }, // Indian/Kerguelen
    {   95, 7050,   55711,  4, }, // Indian/Maldives
    {   95, 4069,   55715,  4, }, // Indian/Mauritius
    {   95, 5098,   42810,  3, }, // Indian/Mayotte
    {   95, 7211,   42814,  3, }, // Indian/Reunion
    {   95, 3706,   55719,  5, }, // Pacific/Bougainville
    {   95, 7327,   55724,  5, }, // Pacific/Fakaofo
    {   95, 6696,   42853,  3, }, // Pacific/Gambier
    {   95, 3822,   55729,  7, }, // Pacific/Guadalcanal
    {   95, 3049,   55736,  5, }, // Pacific/Kanton
    {   95, 7019,   55741,  3, }, // Pacific/Kosrae
    {   95, 2842,   55744,  5, }, // Pacific/Kwajalein
    {   95, 4877,   55749,  3, }, // Pacific/Majuro
    {   95, 4051,   55752,  6, }, // Pacific/Marquesas
    {   95, 7084,   55758,  2, }, // Pacific/Nauru
    {   95, 7194,   55760,  5, }, // Pacific/Pitcairn
    {   95, 4583,   55765,  5, }, // Pacific/Tongatapu
    {   95, 7392,   55770,  3, }, // Pacific/Wake
    {   96, 6788,   55240,  3, }, // Africa/Abidjan Chinese/Traditional Han/Macao
    {   96,   14,   55243,  4, }, // Africa/Asmara
    {   96, 5257,   55247,  2, }, // Africa/Bangui
    {   96, 6773,   55249,  4, }, // Africa/Conakry
    {   96, 5130,   55253,  6, }, // Africa/Dar_es_Salaam
    {   96, 4479,   55259,  5, }, // Africa/Johannesburg
    {   96, 5306,   55264,  3, }, // Africa/Kinshasa
    {   96, 5336,   55267,  3, }, // Africa/Libreville
    {   96, 5211,   55270,  3, }, // Africa/Luanda
    {   96, 5001,   55273,  3, }, // Africa/Lusaka
    {   96, 5197,   55276,  3, }, // Africa/Maseru
    {   96, 3853,   55279,  3, }, // Africa/Nairobi
    {   96, 4152,   55282,  4, }, // Africa/Windhoek
    {   96,  132,   55286,  3, }, // America/Adak
    {   96, 4554,   55289,  5, }, // America/Araguaina
    {   96,  270,   55294,  4, }, // America/Argentina/Cordoba
    {   96, 7462,   55298,  4, }, // America/Argentina/La_Rioja
    {   96, 5640,   55302,  3, }, // America/Aruba
    {   96, 4252,   55305,  3, }, // America/Asuncion
    {   96, 3679,   55308,  3, }, // America/Bahia
    {   96, 7623,   55311,  8, }, // America/Bahia_Banderas
    {   96, 5407,   55319,  3, }, // America/Belize
    {   96, 7681,   55322,  5, }, // America/Boa_Vista
    {   96, 7756,   55327,  3, }, // America/Chihuahua
    {   96, 7774,   41735,  4, }, // America/Ciudad_Juarez
    {   96, 5422,   55330,  5, }, // America/Costa_Rica
    {   96, 3791,   55335,  3, }, // America/Cuiaba
    {   96, 7848,   55338,  3, }, // America/Dawson_Creek
    {   96,  893,   55341,  3, }, // America/Edmonton
    {   96, 7906,   55344,  4, }, // America/Fortaleza
    {   96, 7924,   55348,  4, }, // America/Glace_Bay
    {   96, 5770,   55352,  4, }, // America/Grenada
    {   96, 5786,   55356,  4, }, // America/Guadeloupe
    {   96, 3760,   55360,  4, }, // America/Guatemala
    {   96, 6932,   55364,  3, }, // America/Guyana
    {   96, 2281,   55367,  3, }, // America/Havana
    {   96,  348,   55370,  7, }, // America/Indiana/Indianapolis
    {   96,  481,   55377,  8, }, // America/Indiana/Knox
    {   96, 7961,   55385,  8, }, // America/Indiana/Marengo
    {   96, 7985,   55393,  8, }, // America/Indiana/Petersburg
    {   96, 8012,   55401,  8, }, // America/Indiana/Tell_City
    {   96, 8038,   55409,  7, }, // America/Indiana/Vevay
    {   96, 8060,   55416,  8, }, // America/Indiana/Vincennes
    {   96, 8086,   55424,  9, }, // America/Indiana/Winamac
    {   96, 8110,   55433,  4, }, // America/Inuvik
    {   96,  521,   55437,  4, }, // America/Kentucky/Louisville
    {   96, 8153,   55441,  3, }, // America/Maceio
    {   96, 8168,   55444,  3, }, // America/Managua
    {   96, 5805,   55447,  4, }, // America/Martinique
    {   96, 8202,   55451,  4, }, // America/Menominee
    {   96, 4098,   55455,  5, }, // America/Montevideo
    {   96, 5824,   55460,  5, }, // America/Montserrat
    {   96, 5481,   55465,  2, }, // America/Nassau
    {   96, 1850,   55467,  4, }, // America/Noronha
    {   96, 8301,   55471,  8, }, // America/North_Dakota/Beulah
    {   96, 8329,   55479,  8, }, // America/North_Dakota/Center
    {   96, 8357,   55487,  9, }, // America/North_Dakota/New_Salem
    {   96, 8404,   55496,  3, }, // America/Porto_Velho
    {   96,  695,   55499,  5, }, // America/Rio_Branco
    {   96, 2201,   55504,  4, }, // America/Santiago
    {   96, 6111,   55508,  4, }, // America/Santo_Domingo
    {   96, 8508,   55512,  5, }, // America/St_Barthelemy
    {   96, 2061,   55517,  4, }, // America/St_Johns
    {   96, 5880,   55521,  4, }, // America/St_Lucia
    {   96, 5913,   55525,  4, }, // America/St_Vincent
    {   96, 5461,   55529,  6, }, // America/Tegucigalpa
    {   96, 5756,   55535,  2, }, // America/Thule
    {   96,  313,   55537,  3, }, // America/Tijuana
    {   96, 2144,   55540,  3, }, // America/Whitehorse
    {   96, 8552,   55543,  4, }, // America/Yakutat
    {   96, 6001,   55547,  3, }, // Antarctica/Casey
    {   96, 6094,   55550,  4, }, // Antarctica/Davis
    {   96, 6133,   55554,  6, }, // Antarctica/DumontDUrville
    {   96, 8568,   55560,  3, }, // Antarctica/Macquarie
    {   96, 7066,   55563,  3, }, // Antarctica/Mawson
    {   96, 7113,   55566,  5, }, // Antarctica/McMurdo
    {   96, 8589,   55571,  3, }, // Antarctica/Palmer
    {   96, 7226,   55574,  4, }, // Antarctica/Rothera
    {   96, 7295,   55578,  3, }, // Antarctica/Syowa
    {   96, 8607,   55581,  4, }, // Antarctica/Troll
    {   96, 1429,   55585,  4, }, // Arctic/Longyearbyen
    {   96, 5511,   55589,  4, }, // Asia/Anadyr
    {   96, 3778,   55593,  4, }, // Asia/Bishkek
    {   96, 4511,   55597,  3, }, // Asia/Colombo
    {   96, 6159,   55600,  3, }, // Asia/Dushanbe
    {   96, 4795,   55603,  3, }, // Asia/Hebron
    {   96, 4184,   55606,  5, }, // Asia/Irkutsk
    {   96, 6992,   55611,  4, }, // Asia/Jayapura
    {   96, 1349,   55615,  2, }, // Asia/Makassar
    {   96, 6920,   55617,  4, }, // Asia/Muscat
    {   96, 2554,   55621,  4, }, // Asia/Nicosia
    {   96, 5559,   55625,  3, }, // Asia/Qatar
    {   96, 4828,   55628,  4, }, // Asia/Yakutsk
    {   96, 3883,   55632,  5, }, // Asia/Yekaterinburg
    {   96, 8736,   55637,  4, }, // Atlantic/Madeira
    {   96, 6644,   55641,  3, }, // Atlantic/Stanley
    {   96, 1670,   55644,  4, }, // Australia/Adelaide
    {   96, 1635,   55648,  4, }, // Australia/Brisbane
    {   96, 1516,   55652,  3, }, // Australia/Hobart
    {   96, 6336,   55655,  5, }, // Europe/Belgrade
    {   96, 6488,   55660,  6, }, // Europe/Bratislava
    {   96, 2583,   55666,  4, }, // Europe/Chisinau
    {   96, 8788,   55670,  3, }, // Europe/Guernsey
    {   96, 8804,   55673,  3, }, // Europe/Isle_of_Man
    {   96, 6506,   55676,  5, }, // Europe/Ljubljana
    {   96, 6397,   55681,  3, }, // Europe/Malta
    {   96, 6245,   55684,  4, }, // Europe/Sarajevo
    {   96, 6352,   55688,  3, }, // Europe/Vaduz
    {   96, 6261,   55691,  4, }, // Europe/Zagreb
    {   96, 6947,   55695,  5, }, // Indian/Chagos
    {   96, 6063,   55700,  5, }, // Indian/Cocos
    {   96, 5029,   42789,  3, }, // Indian/Comoro
    {   96, 6661,   55705,  6, }, // Indian/Kerguelen
    {   96, 7050,   55711,  4, }, // Indian/Maldives
    {   96, 4069,   55715,  4, }, // Indian/Mauritius
    {   96, 5098,   42810,  3, }, // Indian/Mayotte
    {   96, 7211,   42814,  3, }, // Indian/Reunion
    {   96, 3706,   55719,  5, }, // Pacific/Bougainville
    {   96, 7327,   55724,  5, }, // Pacific/Fakaofo
    {   96, 6696,   42853,  3, }, // Pacific/Gambier
    {   96, 3822,   55729,  7, }, // Pacific/Guadalcanal
    {   96, 3049,   55736,  5, }, // Pacific/Kanton
    {   96, 7019,   55741,  3, }, // Pacific/Kosrae
    {   96, 2842,   55744,  5, }, // Pacific/Kwajalein
    {   96, 4877,   55749,  3, }, // Pacific/Majuro
    {   96, 4051,   55752,  6, }, // Pacific/Marquesas
    {   96, 7084,   55758,  2, }, // Pacific/Nauru
    {   96, 7194,   55760,  5, }, // Pacific/Pitcairn
    {   96, 4583,   55765,  5, }, // Pacific/Tongatapu
    {   96, 7392,   55770,  3, }, // Pacific/Wake
    {   98, 6788,   41435,  3, }, // Africa/Abidjan Chinese/Traditional Han/Taiwan
    {   98, 5059,   41441,  6, }, // Africa/Addis_Ababa
    {   98, 7420,   41447,  4, }, // Africa/Algiers
    {   98,   14,   41451,  4, }, // Africa/Asmara
    {   98,   44,   41455,  3, }, // Africa/Bamako
    {   98, 6746,   41460,  2, }, // Africa/Banjul
    {   98, 7435,   41462,  2, }, // Africa/Bissau
    {   98, 4957,   41464,  4, }, // Africa/Blantyre
    {   98, 5287,   41468,  5, }, // Africa/Brazzaville
    {   98, 4922,   41473,  4, }, // Africa/Bujumbura
    {   98, 2320,   41477,  2, }, // Africa/Cairo
    {   98, 4117,   41479,  5, }, // Africa/Casablanca
    {   98, 7449,   41484,  2, }, // Africa/Ceuta
    {   98, 6773,   41486,  4, }, // Africa/Conakry
    {   98, 6840,   41490,  3, }, // Africa/Dakar
    {   98, 5130,   41493,  3, }, // Africa/Dar_es_Salaam
    {   98, 5043,   41496,  3, }, // Africa/Djibouti
    {   98, 6853,   41505,  3, }, // Africa/Freetown
    {   98, 4906,   41508,  4, }, // Africa/Gaborone
    {   98, 4479,   41515,  5, }, // Africa/Johannesburg
    {   98, 5306,   41531,  3, }, // Africa/Kinshasa
    {   98, 4744,   41534,  3, }, // Africa/Lagos
    {   98, 5336,   41537,  3, }, // Africa/Libreville
    {   98, 5211,   41542,  3, }, // Africa/Luanda
    {   98, 4939,   41545,  4, }, // Africa/Lubumbashi
    {   98, 5001,   41549,  3, }, // Africa/Lusaka
    {   98, 5322,   41552,  3, }, // Africa/Malabo
    {   98, 4987,   41555,  3, }, // Africa/Maputo
    {   98, 5197,   41558,  3, }, // Africa/Maseru
    {   98, 5182,   41561,  3, }, // Africa/Mbabane
    {   98, 5113,   41564,  4, }, // Africa/Mogadishu
    {   98, 7034,   41568,  4, }, // Africa/Monrovia
    {   98, 3853,   41572,  3, }, // Africa/Nairobi
    {   98, 5271,   41575,  4, }, // Africa/Ndjamena
    {   98, 5354,   41579,  3, }, // Africa/Niamey
    {   98, 6803,   41582,  3, }, // Africa/Nouakchott
    {   98, 5225,   41589,  4, }, // Africa/Porto-Novo
    {   98, 4435,   41593,  3, }, // Africa/Sao_Tome
    {   98, 4152,   41603,  4, }, // Africa/Windhoek
    {   98,  132,   41607,  3, }, // America/Adak
    {   98, 3346,   41610,  4, }, // America/Anchorage
    {   98, 5607,   55773,  3, }, // America/Anguilla
    {   98, 5624,   41617,  3, }, // America/Antigua
    {   98, 4554,   41620,  4, }, // America/Araguaina
    {   98,  166,   41624,  7, }, // America/Argentina/Buenos_Aires
    {   98,   91,   41631,  4, }, // America/Argentina/Catamarca
    {   98,  270,   41635,  3, }, // America/Argentina/Cordoba
    {   98, 7462,   41641,  3, }, // America/Argentina/La_Rioja
    {   98,  565,   41644,  3, }, // America/Argentina/Mendoza
    {   98, 7489,   41647,  6, }, // America/Argentina/Rio_Gallegos
    {   98, 7520,   41653,  3, }, // America/Argentina/Salta
    {   98, 7544,   41656,  3, }, // America/Argentina/San_Juan
    {   98, 5580,   41659,  3, }, // America/Argentina/San_Luis
    {   98, 7571,   41662,  3, }, // America/Argentina/Tucuman
    {   98, 7597,   41665,  4, }, // America/Argentina/Ushuaia
    {   98, 5640,   55776,  5, }, // America/Aruba
    {   98, 4252,   41672,  3, }, // America/Asuncion
    {   98, 3679,   41679,  3, }, // America/Bahia
    {   98, 7623,   41682,  7, }, // America/Bahia_Banderas
    {   98, 5654,   41689,  3, }, // America/Barbados
    {   98, 7646,   41692,  2, }, // America/Belem
    {   98, 5407,   41694,  3, }, // America/Belize
    {   98, 7660,   41697,  5, }, // America/Blanc-Sablon
    {   98, 7681,   41702,  4, }, // America/Boa_Vista
    {   98, 7699,   41709,  2, }, // America/Boise
    {   98, 7713,   41711,  3, }, // America/Cambridge_Bay
    {   98, 7735,   41714,  3, }, // America/Campo_Grande
    {   98, 4694,   41719,  4, }, // America/Caracas
    {   98, 4345,   41723,  2, }, // America/Cayenne
    {   98, 5496,   41725,  4, }, // America/Cayman
    {   98, 7756,   41732,  3, }, // America/Chihuahua
    {   98, 7774,   41735,  3, }, // America/Ciudad_Juarez
    {   98, 5422,   41739,  5, }, // America/Costa_Rica
    {   98, 7796,   41744,  4, }, // America/Creston
    {   98, 3791,   41748,  3, }, // America/Cuiaba
    {   98, 5723,   41751,  3, }, // America/Curacao
    {   98, 7812,   41754,  4, }, // America/Danmarkshavn
    {   98, 5739,   55781,  4, }, // America/Dominica
    {   98,  893,   41774,  4, }, // America/Edmonton
    {   98, 7869,   41778,  4, }, // America/Eirunepe
    {   98, 5441,   41782,  4, }, // America/El_Salvador
    {   98, 7886,   41786,  4, }, // America/Fort_Nelson
    {   98, 7906,   41790,  4, }, // America/Fortaleza
    {   98, 7924,   41794,  4, }, // America/Glace_Bay
    {   98, 6881,   41798,  2, }, // America/Goose_Bay
    {   98, 4612,   41800,  4, }, // America/Grand_Turk
    {   98, 5770,   41804,  4, }, // America/Grenada
    {   98, 5786,   41808,  4, }, // America/Guadeloupe
    {   98, 3760,   41812,  4, }, // America/Guatemala
    {   98, 4841,   41816,  4, }, // America/Guayaquil
    {   98, 6932,   41820,  3, }, // America/Guyana
    {   98, 1939,   41823,  5, }, // America/Halifax
    {   98, 7942,   41831,  4, }, // America/Hermosillo
    {   98,  348,   41835,  7, }, // America/Indiana/Indianapolis
    {   98,  481,   41842,  8, }, // America/Indiana/Knox
    {   98, 7961,   41850,  8, }, // America/Indiana/Marengo
    {   98, 7985,   41858,  8, }, // America/Indiana/Petersburg
    {   98, 8012,   41866,  8, }, // America/Indiana/Tell_City
    {   98, 8038,   41874,  7, }, // America/Indiana/Vevay
    {   98, 8060,   41881,  8, }, // America/Indiana/Vincennes
    {   98, 8086,   41889,  9, }, // America/Indiana/Winamac
    {   98, 8110,   41898,  4, }, // America/Inuvik
    {   98, 2799,   41905,  3, }, // America/Jamaica
    {   98, 5380,   41908,  2, }, // America/Juneau
    {   98,  521,   41910,  5, }, // America/Kentucky/Louisville
    {   98, 8125,   41915,  8, }, // America/Kentucky/Monticello
    {   98, 5704,   41923,  5, }, // America/Kralendijk
    {   98, 7169,   41931,  2, }, // America/Lima
    {   98, 3239,   41933,  3, }, // America/Los_Angeles
    {   98, 5932,   41936,  4, }, // America/Lower_Princes
    {   98, 8153,   41940,  3, }, // America/Maceio
    {   98, 8168,   41943,  3, }, // America/Managua
    {   98, 1908,   41946,  3, }, // America/Manaus
    {   98, 5897,   41949,  4, }, // America/Marigot
    {   98, 5805,   41953,  4, }, // America/Martinique
    {   98, 8184,   41957,  5, }, // America/Matamoros
    {   98, 2917,   41962,  4, }, // America/Mazatlan
    {   98, 8202,   41966,  4, }, // America/Menominee
    {   98, 8220,   41970,  3, }, // America/Merida
    {   98, 2949,   41979,  4, }, // America/Mexico_City
    {   98, 4391,   41983,  3, }, // America/Miquelon
    {   98, 8254,   41986,  3, }, // America/Moncton
    {   98, 8270,   41989,  3, }, // America/Monterrey
    {   98, 4098,   41992,  4, }, // America/Montevideo
    {   98, 5824,   41996,  3, }, // America/Montserrat
    {   98, 5481,   41999,  2, }, // America/Nassau
    {   98, 2379,   42001,  2, }, // America/New_York
    {   98, 8288,   42003,  2, }, // America/Nome
    {   98, 1850,   42005,  3, }, // America/Noronha
    {   98, 8301,   42008,  8, }, // America/North_Dakota/Beulah
    {   98, 8329,   42016,  7, }, // America/North_Dakota/Center
    {   98, 8357,   42023,  8, }, // America/North_Dakota/New_Salem
    {   98, 8388,   42033,  4, }, // America/Ojinaga
    {   98, 2356,   42037,  3, }, // America/Panama
    {   98, 6173,   42040,  5, }, // America/Paramaribo
    {   98, 2973,   42045,  3, }, // America/Phoenix
    {   98, 8404,   42055,  3, }, // America/Porto_Velho
    {   98, 4030,   42062,  6, }, // America/Punta_Arenas
    {   98, 8424,   42068,  3, }, // America/Rankin_Inlet
    {   98, 8445,   42071,  3, }, // America/Recife
    {   98, 1995,   42074,  3, }, // America/Regina
    {   98, 8460,   42077,  4, }, // America/Resolute
    {   98,  695,   42081,  4, }, // America/Rio_Branco
    {   98, 8477,   42085,  3, }, // America/Santarem
    {   98, 2201,   42088,  4, }, // America/Santiago
    {   98, 6111,   42092,  4, }, // America/Santo_Domingo
    {   98, 1878,   42096,  3, }, // America/Sao_Paulo
    {   98, 6899,   42099,  7, }, // America/Scoresbysund
    {   98, 8494,   42106,  3, }, // America/Sitka
    {   98, 8508,   42109,  6, }, // America/St_Barthelemy
    {   98, 2061,   42115,  3, }, // America/St_Johns
    {   98, 5863,   42118,  3, }, // America/St_Kitts
    {   98, 5880,   42121,  4, }, // America/St_Lucia
    {   98,  855,   42125,  4, }, // America/St_Thomas
    {   98, 5913,   42129,  3, }, // America/St_Vincent
    {   98, 8530,   42132,  7, }, // America/Swift_Current
    {   98, 5461,   42139,  5, }, // America/Tegucigalpa
    {   98, 5756,   42144,  2, }, // America/Thule
    {   98,  313,   42146,  3, }, // America/Tijuana
    {   98,  608,   42149,  3, }, // America/Toronto
    {   98, 5688,   42152,  4, }, // America/Tortola
    {   98, 2093,   42156,  3, }, // America/Vancouver
    {   98, 2144,   42159,  4, }, // America/Whitehorse
    {   98,  734,   42163,  3, }, // America/Winnipeg
    {   98, 8552,   42166,  3, }, // America/Yakutat
    {   98, 6001,   42169,  2, }, // Antarctica/Casey
    {   98, 6094,   42171,  3, }, // Antarctica/Davis
    {   98, 6133,   42174,  5, }, // Antarctica/DumontDUrville
    {   98, 8568,   42179,  3, }, // Antarctica/Macquarie
    {   98, 7113,   42184,  4, }, // Antarctica/McMurdo
    {   98, 8589,   42188,  2, }, // Antarctica/Palmer
    {   98, 7226,   42190,  3, }, // Antarctica/Rothera
    {   98, 7295,   42193,  4, }, // Antarctica/Syowa
    {   98, 8607,   42197,  2, }, // Antarctica/Troll
    {   98, 1429,   42203,  5, }, // Arctic/Longyearbyen
    {   98, 5570,   42208,  2, }, // Asia/Aden
    {   98, 5395,   42210,  4, }, // Asia/Almaty
    {   98, 5511,   42216,  3, }, // Asia/Anadyr
    {   98, 5368,   42222,  4, }, // Asia/Aqtobe
    {   98,  964,   42226,  5, }, // Asia/Ashgabat
    {   98, 8624,   42231,  3, }, // Asia/Atyrau
    {   98, 3607,   42234,  3, }, // Asia/Baghdad
    {   98, 3653,   41750,  2, }, // Asia/Baku
    {   98, 3571,   42241,  4, }, // Asia/Barnaul
    {   98, 4086,   42245,  3, }, // Asia/Beirut
    {   98, 3778,   42248,  4, }, // Asia/Bishkek
    {   98, 5989,   42252,  2, }, // Asia/Brunei
    {   98, 4511,   42256,  3, }, // Asia/Colombo
    {   98, 4540,   42259,  4, }, // Asia/Damascus
    {   98, 1093,   42263,  2, }, // Asia/Dhaka
    {   98, 3596,   42267,  2, }, // Asia/Dubai
    {   98, 6159,   42269,  3, }, // Asia/Dushanbe
    {   98, 8636,   42272,  5, }, // Asia/Famagusta
    {   98, 8651,   42277,  2, }, // Asia/Gaza
    {   98, 4795,   42279,  3, }, // Asia/Hebron
    {   98, 4184,   42303,  5, }, // Asia/Irkutsk
    {   98, 7006,   42308,  3, }, // Asia/Jakarta
    {   98, 6992,   42311,  4, }, // Asia/Jayapura
    {   98, 3560,   42319,  3, }, // Asia/Kabul
    {   98, 4239,   42325,  3, }, // Asia/Karachi
    {   98, 1185,   42328,  4, }, // Asia/Kathmandu
    {   98, 8661,   42332,  3, }, // Asia/Khandyga
    {   98,  992,   42335,  4, }, // Asia/Kolkata
    {   98, 4197,   42339,  8, }, // Asia/Krasnoyarsk
    {   98, 5976,   42350,  2, }, // Asia/Kuching
    {   98, 1211,   42362,  2, }, // Asia/Macau
    {   98, 4017,   42369,  3, }, // Asia/Magadan
    {   98, 1349,   42372,  4, }, // Asia/Makassar
    {   98, 7182,   42376,  3, }, // Asia/Manila
    {   98, 6920,   42379,  4, }, // Asia/Muscat
    {   98, 2554,   42383,  4, }, // Asia/Nicosia
    {   98, 8675,   42387,  6, }, // Asia/Novokuznetsk
    {   98, 4135,   42393,  5, }, // Asia/Novosibirsk
    {   98, 7145,   42402,  3, }, // Asia/Oral
    {   98, 6961,   42405,  2, }, // Asia/Phnom_Penh
    {   98, 5559,   42411,  2, }, // Asia/Qatar
    {   98, 8708,   55785,  4, }, // Asia/Qostanay
    {   98, 4269,   42417,  6, }, // Asia/Qyzylorda
    {   98, 3584,   42423,  3, }, // Asia/Riyadh
    {   98, 4408,   42426,  3, }, // Asia/Sakhalin
    {   98, 7245,   42429,  4, }, // Asia/Samarkand
    {   98, 3279,   42433,  2, }, // Asia/Seoul
    {   98, 2772,   42455,  3, }, // Asia/Tehran
    {   98, 2821,   42460,  2, }, // Asia/Tokyo
    {   98, 1021,   42466,  4, }, // Asia/Ulaanbaatar
    {   98, 1159,   42470,  4, }, // Asia/Urumqi
    {   98, 8722,   42474,  4, }, // Asia/Ust-Nera
    {   98, 6977,   42478,  2, }, // Asia/Vientiane
    {   98, 4710,   42480,  3, }, // Asia/Vladivostok
    {   98, 4828,   42483,  4, }, // Asia/Yakutsk
    {   98, 3883,   42489,  5, }, // Asia/Yekaterinburg
    {   98, 3747,   42494,  3, }, // Asia/Yerevan
    {   98, 3663,   42497,  5, }, // Atlantic/Azores
    {   98, 5671,   42502,  3, }, // Atlantic/Bermuda
    {   98, 6628,   42505,  3, }, // Atlantic/Canary
    {   98, 3727,   42508,  3, }, // Atlantic/Cape_Verde
    {   98, 1395,   42511,  4, }, // Atlantic/Faroe
    {   98, 8736,   42515,  5, }, // Atlantic/Madeira
    {   98, 2748,   42520,  5, }, // Atlantic/Reykjavik
    {   98, 7272,   42525,  4, }, // Atlantic/South_Georgia
    {   98, 6821,   42529,  5, }, // Atlantic/St_Helena
    {   98, 6644,   42534,  3, }, // Atlantic/Stanley
    {   98, 1670,   42537,  4, }, // Australia/Adelaide
    {   98, 1635,   42541,  4, }, // Australia/Brisbane
    {   98, 1799,   42545,  5, }, // Australia/Broken_Hill
    {   98, 1583,   42550,  3, }, // Australia/Darwin
    {   98, 1516,   42556,  3, }, // Australia/Hobart
    {   98, 1547,   42562,  4, }, // Australia/Lord_Howe
    {   98, 1727,   42566,  3, }, // Australia/Melbourne
    {   98, 1762,   42569,  2, }, // Australia/Perth
    {   98, 1463,   42571,  2, }, // Australia/Sydney
    {   98, 6216,   42578,  3, }, // Europe/Andorra
    {   98, 6336,   42588,  4, }, // Europe/Belgrade
    {   98, 6488,   42594,  6, }, // Europe/Bratislava
    {   98, 2167,   42600,  4, }, // Europe/Brussels
    {   98, 3806,   42609,  4, }, // Europe/Budapest
    {   98, 2583,   55789,  3, }, // Europe/Chisinau
    {   98, 2338,   42623,  3, }, // Europe/Dublin
    {   98, 6307,   42626,  4, }, // Europe/Gibraltar
    {   98, 8788,   42630,  3, }, // Europe/Guernsey
    {   98, 6612,   42633,  4, }, // Europe/Helsinki
    {   98, 8804,   42637,  2, }, // Europe/Isle_of_Man
    {   98, 1130,   42639,  4, }, // Europe/Istanbul
    {   98, 8823,   42643,  3, }, // Europe/Jersey
    {   98, 3979,   42646,  5, }, // Europe/Kaliningrad
    {   98, 2527,   42654,  2, }, // Europe/Kyiv
    {   98, 6506,   42659,  4, }, // Europe/Ljubljana
    {   98, 2501,   42663,  2, }, // Europe/London
    {   98, 6365,   42665,  3, }, // Europe/Luxembourg
    {   98, 4892,   42668,  3, }, // Europe/Madrid
    {   98, 6397,   42671,  3, }, // Europe/Malta
    {   98, 6582,   42674,  3, }, // Europe/Mariehamn
    {   98, 6410,   42680,  3, }, // Europe/Monaco
    {   98, 6458,   42686,  3, }, // Europe/Oslo
    {   98, 6424,   42691,  4, }, // Europe/Podgorica
    {   98, 6324,   42700,  2, }, // Europe/Rome
    {   98, 4331,   42702,  3, }, // Europe/Samara
    {   98, 6470,   42705,  4, }, // Europe/San_Marino
    {   98, 6245,   42709,  4, }, // Europe/Sarajevo
    {   98, 4451,   42713,  4, }, // Europe/Saratov
    {   98, 8849,   42717,  4, }, // Europe/Simferopol
    {   98, 6383,   42721,  4, }, // Europe/Skopje
    {   98, 6599,   42725,  3, }, // Europe/Sofia
    {   98, 6523,   42728,  5, }, // Europe/Stockholm
    {   98, 8882,   42738,  7, }, // Europe/Ulyanovsk
    {   98, 6352,   42745,  3, }, // Europe/Vaduz
    {   98, 6567,   42748,  3, }, // Europe/Vatican
    {   98, 6231,   42751,  3, }, // Europe/Vienna
    {   98, 8899,   42754,  4, }, // Europe/Vilnius
    {   98, 4727,   42758,  5, }, // Europe/Volgograd
    {   98, 3190,   42763,  2, }, // Europe/Warsaw
    {   98, 6261,   42765,  4, }, // Europe/Zagreb
    {   98, 6540,   42769,  3, }, // Europe/Zurich
    {   98, 5078,   55792,  6, }, // Indian/Antananarivo
    {   98, 6046,   42781,  3, }, // Indian/Christmas
    {   98, 6063,   42784,  5, }, // Indian/Cocos
    {   98, 5029,   42789,  5, }, // Indian/Comoro
    {   98, 6661,   42794,  5, }, // Indian/Kerguelen
    {   98, 7260,   42799,  3, }, // Indian/Mahe
    {   98, 7050,   42802,  4, }, // Indian/Maldives
    {   98, 4069,   42806,  4, }, // Indian/Mauritius
    {   98, 5098,   42810,  4, }, // Indian/Mayotte
    {   98, 7211,   42814,  4, }, // Indian/Reunion
    {   98, 4422,   42818,  3, }, // Pacific/Apia
    {   98,  932,   42821,  3, }, // Pacific/Auckland
    {   98, 3706,   42824,  4, }, // Pacific/Bougainville
    {   98, 3015,   42828,  2, }, // Pacific/Chatham
    {   98, 3157,   42830,  2, }, // Pacific/Chuuk
    {   98, 2237,   42832,  3, }, // Pacific/Easter
    {   98, 3902,   42841,  2, }, // Pacific/Fiji
    {   98, 7343,   42843,  4, }, // Pacific/Funafuti
    {   98, 6678,   42847,  6, }, // Pacific/Galapagos
    {   98, 6696,   42853,  5, }, // Pacific/Gambier
    {   98, 3822,   42858,  6, }, // Pacific/Guadalcanal
    {   98, 6018,   42864,  2, }, // Pacific/Guam
    {   98, 3049,   42869,  5, }, // Pacific/Kanton
    {   98, 3998,   42874,  6, }, // Pacific/Kiritimati
    {   98, 7019,   42880,  3, }, // Pacific/Kosrae
    {   98, 2842,   42883,  4, }, // Pacific/Kwajalein
    {   98, 4877,   42887,  3, }, // Pacific/Majuro
    {   98, 4051,   42890,  5, }, // Pacific/Marquesas
    {   98, 8914,   42895,  3, }, // Pacific/Midway
    {   98, 7084,   42898,  2, }, // Pacific/Nauru
    {   98, 7132,   42900,  3, }, // Pacific/Niue
    {   98, 4168,   42903,  3, }, // Pacific/Norfolk
    {   98, 7098,   42906,  3, }, // Pacific/Noumea
    {   98, 3126,   42909,  4, }, // Pacific/Pago_Pago
    {   98, 7155,   42913,  2, }, // Pacific/Palau
    {   98, 7194,   42915,  5, }, // Pacific/Pitcairn
    {   98, 3096,   42920,  3, }, // Pacific/Pohnpei
    {   98, 4807,   42923,  4, }, // Pacific/Port_Moresby
    {   98, 6076,   42927,  4, }, // Pacific/Rarotonga
    {   98, 7312,   42933,  3, }, // Pacific/Tahiti
    {   98, 4583,   42939,  5, }, // Pacific/Tongatapu
    {   99,   14,       0,  6, }, // Africa/Asmara Church/Cyrillic/Russia
    {   99, 4435,       6,  8, }, // Africa/Sao_Tome
    {   99, 4252,      14,  8, }, // America/Asuncion
    {   99,  237,      22,  8, }, // America/Atikokan
    {   99, 7623,      30, 17, }, // America/Bahia_Banderas
    {   99, 3868,      47,  6, }, // America/Cancun
    {   99, 7774,      53, 13, }, // America/Ciudad_Juarez
    {   99, 5723,      66,  7, }, // America/Curacao
    {   99,  481,      73, 13, }, // America/Indiana/Knox
    {   99, 7961,      86, 16, }, // America/Indiana/Marengo
    {   99, 7985,     102, 19, }, // America/Indiana/Petersburg
    {   99, 8012,     121, 18, }, // America/Indiana/Tell_City
    {   99, 8038,     139, 14, }, // America/Indiana/Vevay
    {   99, 8060,     153, 18, }, // America/Indiana/Vincennes
    {   99, 8086,     171, 16, }, // America/Indiana/Winamac
    {   99, 8125,     187, 20, }, // America/Kentucky/Monticello
    {   99, 5932,     207, 22, }, // America/Lower_Princes
    {   99, 8220,     229,  6, }, // America/Merida
    {   99, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {   99, 8329,     255, 20, }, // America/North_Dakota/Center
    {   99, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {   99,  393,     298,  4, }, // America/Nuuk
    {   99, 6899,     302, 16, }, // America/Scoresbysund
    {   99, 8508,     318, 14, }, // America/St_Barthelemy
    {   99, 2061,     332, 10, }, // America/St_Johns
    {   99, 5863,     342,  9, }, // America/St_Kitts
    {   99, 5880,     351,  9, }, // America/St_Lucia
    {   99,  855,     360, 10, }, // America/St_Thomas
    {   99, 5913,     370, 11, }, // America/St_Vincent
    {   99, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {   99, 5395,   55798,  8, }, // Asia/Almaty
    {   99, 5511,   55806,  9, }, // Asia/Anadyr
    {   99, 5523,   55815,  7, }, // Asia/Aqtau
    {   99, 5368,   55822, 12, }, // Asia/Aqtobe
    {   99, 3778,   55834,  8, }, // Asia/Bishkek
    {   99, 4601,   55842,  5, }, // Asia/Chita
    {   99, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {   99, 4184,   55847, 10, }, // Asia/Irkutsk
    {   99, 4316,   55857, 27, }, // Asia/Kamchatka
    {   99, 1185,     413,  9, }, // Asia/Kathmandu
    {   99, 8661,   55884,  8, }, // Asia/Khandyga
    {   99,  992,     422,  7, }, // Asia/Kolkata
    {   99, 4197,   55892, 12, }, // Asia/Krasnoyarsk
    {   99, 1211,     429,  5, }, // Asia/Macau
    {   99, 4017,   55904,  9, }, // Asia/Magadan
    {   99, 8675,   55913, 13, }, // Asia/Novokuznetsk
    {   99, 4135,   55926, 13, }, // Asia/Novosibirsk
    {   99, 4229,   55939,  7, }, // Asia/Omsk
    {   99, 7145,   55946, 11, }, // Asia/Oral
    {   99, 4269,   55957, 10, }, // Asia/Qyzylorda
    {   99, 4408,   55967,  9, }, // Asia/Sakhalin
    {   99, 4297,   55976, 15, }, // Asia/Srednekolymsk
    {   99, 8722,   55991, 13, }, // Asia/Ust-Nera
    {   99, 4710,   56004, 13, }, // Asia/Vladivostok
    {   99, 4828,   56017,  9, }, // Asia/Yakutsk
    {   99, 1235,     442,  6, }, // Asia/Yangon
    {   99, 3883,   56026, 15, }, // Asia/Yekaterinburg
    {   99, 1395,     448,  5, }, // Atlantic/Faroe
    {   99, 6821,     453, 10, }, // Atlantic/St_Helena
    {   99, 3979,   56041, 13, }, // Europe/Kaliningrad
    {   99, 2527,   56054,  6, }, // Europe/Kyiv
    {   99, 3693,   56060,  7, }, // Europe/Minsk
    {   99, 3537,   56067,  7, }, // Europe/Moscow
    {   99, 4331,   56074,  7, }, // Europe/Samara
    {   99, 8849,   56081, 12, }, // Europe/Simferopol
    {   99, 4727,   56093, 11, }, // Europe/Volgograd
    {   99, 7211,     467,  7, }, // Indian/Reunion
    {   99, 3157,     474,  5, }, // Pacific/Chuuk
    {   99, 3096,     479,  7, }, // Pacific/Pohnpei
    {  100, 6788,   36214,  7, }, // Africa/Abidjan Chuvash/Cyrillic/Russia
    {  100, 6760,     492,  5, }, // Africa/Accra
    {  100, 5059,     497, 11, }, // Africa/Addis_Ababa
    {  100, 7420,     508,  5, }, // Africa/Algiers
    {  100,   14,   56104,  6, }, // Africa/Asmara
    {  100,   44,     519,  6, }, // Africa/Bamako
    {  100, 5257,     525,  5, }, // Africa/Bangui
    {  100, 6746,     530,  6, }, // Africa/Banjul
    {  100, 7435,     536,  5, }, // Africa/Bissau
    {  100, 4957,   27088,  8, }, // Africa/Blantyre
    {  100, 5287,   51422, 10, }, // Africa/Brazzaville
    {  100, 4922,     558,  9, }, // Africa/Bujumbura
    {  100, 2320,     567,  4, }, // Africa/Cairo
    {  100, 4117,     571, 10, }, // Africa/Casablanca
    {  100, 7449,     581,  5, }, // Africa/Ceuta
    {  100, 6773,     586,  7, }, // Africa/Conakry
    {  100, 6840,     593,  5, }, // Africa/Dakar
    {  100, 5130,   27116, 12, }, // Africa/Dar_es_Salaam
    {  100, 5043,   36247,  7, }, // Africa/Djibouti
    {  100, 5243,     616,  5, }, // Africa/Douala
    {  100, 5166,   27135,  7, }, // Africa/El_Aaiun
    {  100, 6853,     628,  7, }, // Africa/Freetown
    {  100, 4906,     635,  8, }, // Africa/Gaborone
    {  100, 5015,   33781,  6, }, // Africa/Harare
    {  100, 4479,   51440, 12, }, // Africa/Johannesburg
    {  100, 4499,   27173,  5, }, // Africa/Juba
    {  100, 5151,     665,  7, }, // Africa/Kampala
    {  100, 4524,     672,  6, }, // Africa/Khartoum
    {  100, 4973,     678,  6, }, // Africa/Kigali
    {  100, 5306,     684,  7, }, // Africa/Kinshasa
    {  100, 4744,     691,  5, }, // Africa/Lagos
    {  100, 5336,   51452,  9, }, // Africa/Libreville
    {  100, 6869,     704,  4, }, // Africa/Lome
    {  100, 5211,     708,  6, }, // Africa/Luanda
    {  100, 4939,     714,  9, }, // Africa/Lubumbashi
    {  100, 5001,     723,  6, }, // Africa/Lusaka
    {  100, 5322,     729,  6, }, // Africa/Malabo
    {  100, 4987,     735,  6, }, // Africa/Maputo
    {  100, 5197,     741,  6, }, // Africa/Maseru
    {  100, 5182,     747,  7, }, // Africa/Mbabane
    {  100, 5113,     754,  8, }, // Africa/Mogadishu
    {  100, 7034,     762,  7, }, // Africa/Monrovia
    {  100, 3853,   36295,  7, }, // Africa/Nairobi
    {  100, 5271,   27254,  8, }, // Africa/Ndjamena
    {  100, 5354,   36302,  6, }, // Africa/Niamey
    {  100, 6803,     790,  7, }, // Africa/Nouakchott
    {  100, 6727,     797,  8, }, // Africa/Ouagadougou
    {  100, 5225,     805, 10, }, // Africa/Porto-Novo
    {  100, 4435,   56110, 17, }, // Africa/Sao_Tome
    {  100, 2866,     823,  7, }, // Africa/Tripoli
    {  100, 6554,     830,  5, }, // Africa/Tunis
    {  100, 4152,     835,  7, }, // Africa/Windhoek
    {  100,  132,     842,  4, }, // America/Adak
    {  100, 3346,   51461,  8, }, // America/Anchorage
    {  100, 5607,   51469,  7, }, // America/Anguilla
    {  100, 5624,     860,  7, }, // America/Antigua
    {  100, 4554,     867,  9, }, // America/Araguaina
    {  100,  166,   51476, 12, }, // America/Argentina/Buenos_Aires
    {  100,   91,     888,  9, }, // America/Argentina/Catamarca
    {  100,  270,     897,  7, }, // America/Argentina/Cordoba
    {  100,  441,   27355,  5, }, // America/Argentina/Jujuy
    {  100, 7462,     909,  8, }, // America/Argentina/La_Rioja
    {  100,  565,     917,  7, }, // America/Argentina/Mendoza
    {  100, 7489,   51488, 12, }, // America/Argentina/Rio_Gallegos
    {  100, 7520,   27380,  6, }, // America/Argentina/Salta
    {  100, 7544,     940,  8, }, // America/Argentina/San_Juan
    {  100, 5580,   56127, 17, }, // America/Argentina/San_Luis
    {  100, 7571,     956,  7, }, // America/Argentina/Tucuman
    {  100, 7597,   27394,  5, }, // America/Argentina/Ushuaia
    {  100, 5640,     969,  5, }, // America/Aruba
    {  100, 4252,   51500,  8, }, // America/Asuncion
    {  100,  237,     981, 12, }, // America/Atikokan
    {  100, 3679,   36368,  4, }, // America/Bahia
    {  100, 7623,   51508, 16, }, // America/Bahia_Banderas
    {  100, 5654,    1013,  8, }, // America/Barbados
    {  100, 7646,    1021,  5, }, // America/Belem
    {  100, 5407,    1026,  5, }, // America/Belize
    {  100, 7660,    1031, 12, }, // America/Blanc-Sablon
    {  100, 7681,    1043,  9, }, // America/Boa_Vista
    {  100, 4361,    1052,  6, }, // America/Bogota
    {  100, 7699,   51524,  5, }, // America/Boise
    {  100, 7713,   51529, 13, }, // America/Cambridge_Bay
    {  100, 7735,    1075, 12, }, // America/Campo_Grande
    {  100, 3868,    1087,  6, }, // America/Cancun
    {  100, 4694,    1093,  7, }, // America/Caracas
    {  100, 4345,   51542,  7, }, // America/Cayenne
    {  100, 5496,   56144, 16, }, // America/Cayman
    {  100, 2260,    1129,  6, }, // America/Chicago
    {  100, 7756,    1135,  6, }, // America/Chihuahua
    {  100, 7774,   56160, 13, }, // America/Ciudad_Juarez
    {  100, 5422,    1141, 10, }, // America/Costa_Rica
    {  100, 7796,    1151,  7, }, // America/Creston
    {  100, 3791,   27556,  5, }, // America/Cuiaba
    {  100, 5723,   36474,  7, }, // America/Curacao
    {  100, 7812,    1172, 12, }, // America/Danmarkshavn
    {  100, 7833,    1184,  6, }, // America/Dawson
    {  100, 7848,    1190, 11, }, // America/Dawson_Creek
    {  100,  805,    1201,  6, }, // America/Denver
    {  100, 3465,   36517,  7, }, // America/Detroit
    {  100, 5739,    1214,  8, }, // America/Dominica
    {  100,  893,   51549,  8, }, // America/Edmonton
    {  100, 7869,   51557,  8, }, // America/Eirunepe
    {  100, 5441,   27633,  9, }, // America/El_Salvador
    {  100, 7886,   51565, 12, }, // America/Fort_Nelson
    {  100, 7906,    1257,  9, }, // America/Fortaleza
    {  100, 7924,   51577,  9, }, // America/Glace_Bay
    {  100, 6881,   51586,  7, }, // America/Goose_Bay
    {  100, 4612,    1282, 10, }, // America/Grand_Turk
    {  100, 5770,    1292,  7, }, // America/Grenada
    {  100, 5786,    1299,  9, }, // America/Guadeloupe
    {  100, 3760,    1308,  9, }, // America/Guatemala
    {  100, 4841,   51593,  8, }, // America/Guayaquil
    {  100, 6932,   51601,  6, }, // America/Guyana
    {  100, 1939,   51607,  8, }, // America/Halifax
    {  100, 2281,    1339,  6, }, // America/Havana
    {  100, 7942,   51615,  9, }, // America/Hermosillo
    {  100,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  100,  481,    1365, 13, }, // America/Indiana/Knox
    {  100, 7961,    1378, 16, }, // America/Indiana/Marengo
    {  100, 7985,    1394, 19, }, // America/Indiana/Petersburg
    {  100, 8012,   56173, 18, }, // America/Indiana/Tell_City
    {  100, 8038,   56191, 14, }, // America/Indiana/Vevay
    {  100, 8060,   56205, 18, }, // America/Indiana/Vincennes
    {  100, 8086,   56223, 16, }, // America/Indiana/Winamac
    {  100, 8110,    1452,  6, }, // America/Inuvik
    {  100,  660,    1458,  7, }, // America/Iqaluit
    {  100, 2799,   27879,  6, }, // America/Jamaica
    {  100, 5380,   36614,  5, }, // America/Juneau
    {  100,  521,    1476,  8, }, // America/Kentucky/Louisville
    {  100, 8125,    1484, 20, }, // America/Kentucky/Monticello
    {  100, 5704,   36619, 10, }, // America/Kralendijk
    {  100, 4376,    1514,  6, }, // America/La_Paz
    {  100, 7169,    1520,  4, }, // America/Lima
    {  100, 3239,   51629, 12, }, // America/Los_Angeles
    {  100, 5932,   56239, 18, }, // America/Lower_Princes
    {  100, 8153,   36667,  6, }, // America/Maceio
    {  100, 8168,    1559,  7, }, // America/Managua
    {  100, 1908,    1566,  6, }, // America/Manaus
    {  100, 5897,    1572,  6, }, // America/Marigot
    {  100, 5805,    1578,  9, }, // America/Martinique
    {  100, 8184,    1587,  9, }, // America/Matamoros
    {  100, 2917,    1596,  8, }, // America/Mazatlan
    {  100, 8202,    1604,  8, }, // America/Menominee
    {  100, 8220,    1612,  6, }, // America/Merida
    {  100, 8235,    1618, 10, }, // America/Metlakatla
    {  100, 2949,    1628,  6, }, // America/Mexico_City
    {  100, 4391,    1634,  7, }, // America/Miquelon
    {  100, 8254,    1641,  7, }, // America/Moncton
    {  100, 8270,   51662,  9, }, // America/Monterrey
    {  100, 4098,    1657, 10, }, // America/Montevideo
    {  100, 5824,    1667, 10, }, // America/Montserrat
    {  100, 5481,    1677,  6, }, // America/Nassau
    {  100, 2379,   51680,  8, }, // America/New_York
    {  100, 8288,    1691,  3, }, // America/Nome
    {  100, 1850,   56257,  7, }, // America/Noronha
    {  100, 8301,   56264, 20, }, // America/North_Dakota/Beulah
    {  100, 8329,   56284, 20, }, // America/North_Dakota/Center
    {  100, 8357,   56304, 25, }, // America/North_Dakota/New_Salem
    {  100,  393,    1772,  4, }, // America/Nuuk
    {  100, 8388,    1776,  7, }, // America/Ojinaga
    {  100, 2356,    1783,  6, }, // America/Panama
    {  100, 6173,    1789, 10, }, // America/Paramaribo
    {  100, 2973,    1799,  6, }, // America/Phoenix
    {  100, 3945,    1805, 12, }, // America/Port-au-Prince
    {  100, 5954,   28189, 13, }, // America/Port_of_Spain
    {  100, 8404,   28202, 11, }, // America/Porto_Velho
    {  100, 5843,   51756, 11, }, // America/Puerto_Rico
    {  100, 4030,    1852, 12, }, // America/Punta_Arenas
    {  100, 8424,    1864, 12, }, // America/Rankin_Inlet
    {  100, 8445,    1876,  6, }, // America/Recife
    {  100, 1995,   51767,  8, }, // America/Regina
    {  100, 8460,   56329,  8, }, // America/Resolute
    {  100,  695,    1897, 10, }, // America/Rio_Branco
    {  100, 8477,    1907,  8, }, // America/Santarem
    {  100, 2201,   51775,  8, }, // America/Santiago
    {  100, 6111,    1923, 13, }, // America/Santo_Domingo
    {  100, 1878,   56337, 19, }, // America/Sao_Paulo
    {  100, 6899,    1945, 11, }, // America/Scoresbysund
    {  100, 8494,    1956,  5, }, // America/Sitka
    {  100, 8508,    1961, 13, }, // America/St_Barthelemy
    {  100, 2061,   28349, 10, }, // America/St_Johns
    {  100, 5863,    1983,  9, }, // America/St_Kitts
    {  100, 5880,   56356, 10, }, // America/St_Lucia
    {  100,  855,    2003, 10, }, // America/St_Thomas
    {  100, 5913,    2013, 12, }, // America/St_Vincent
    {  100, 8530,    2025, 13, }, // America/Swift_Current
    {  100, 5461,   51817, 12, }, // America/Tegucigalpa
    {  100, 5756,    2049,  4, }, // America/Thule
    {  100,  313,    2053,  7, }, // America/Tijuana
    {  100,  608,    2060,  7, }, // America/Toronto
    {  100, 5688,    2067,  7, }, // America/Tortola
    {  100, 2093,    2074,  8, }, // America/Vancouver
    {  100, 2144,   28435,  8, }, // America/Whitehorse
    {  100,  734,    2090,  8, }, // America/Winnipeg
    {  100, 8552,   28450,  6, }, // America/Yakutat
    {  100, 6001,   36889,  5, }, // Antarctica/Casey
    {  100, 6094,   36894,  6, }, // Antarctica/Davis
    {  100, 6133,   56366, 29, }, // Antarctica/DumontDUrville
    {  100, 8568,    2131,  8, }, // Antarctica/Macquarie
    {  100, 7066,    2139,  6, }, // Antarctica/Mawson
    {  100, 7113,    2145,  9, }, // Antarctica/McMurdo
    {  100, 8589,    2154,  6, }, // Antarctica/Palmer
    {  100, 7226,    2160,  6, }, // Antarctica/Rothera
    {  100, 7295,   28510,  4, }, // Antarctica/Syowa
    {  100, 8607,   51843,  6, }, // Antarctica/Troll
    {  100, 7374,   34825,  6, }, // Antarctica/Vostok
    {  100, 1429,   51849,  7, }, // Arctic/Longyearbyen
    {  100, 5570,    2192,  4, }, // Asia/Aden
    {  100, 5395,    2196,  6, }, // Asia/Almaty
    {  100, 3968,    2202,  5, }, // Asia/Amman
    {  100, 5511,   51870,  7, }, // Asia/Anadyr
    {  100, 5523,    2213,  5, }, // Asia/Aqtau
    {  100, 5368,    2218,  6, }, // Asia/Aqtobe
    {  100,  964,    2224,  7, }, // Asia/Ashgabat
    {  100, 8624,    2231,  6, }, // Asia/Atyrau
    {  100, 3607,    2237,  6, }, // Asia/Baghdad
    {  100, 5534,   36948,  7, }, // Asia/Bahrain
    {  100, 3653,    2250,  4, }, // Asia/Baku
    {  100, 4466,    2254,  7, }, // Asia/Bangkok
    {  100, 3571,    2261,  7, }, // Asia/Barnaul
    {  100, 4086,   28563,  6, }, // Asia/Beirut
    {  100, 3778,    2274,  6, }, // Asia/Bishkek
    {  100, 5989,   28575,  6, }, // Asia/Brunei
    {  100, 4601,    2286,  4, }, // Asia/Chita
    {  100, 4511,    2290,  7, }, // Asia/Colombo
    {  100, 4540,    2297,  6, }, // Asia/Damascus
    {  100, 1093,    2303,  5, }, // Asia/Dhaka
    {  100, 6192,    2308,  4, }, // Asia/Dili
    {  100, 3596,   28596,  5, }, // Asia/Dubai
    {  100, 6159,    2317,  7, }, // Asia/Dushanbe
    {  100, 8636,    2324,  9, }, // Asia/Famagusta
    {  100, 8651,    2333,  4, }, // Asia/Gaza
    {  100, 4795,    2337,  6, }, // Asia/Hebron
    {  100, 1259,    2343,  7, }, // Asia/Ho_Chi_Minh
    {  100, 2704,   51902,  7, }, // Asia/Hong_Kong
    {  100, 4771,    2357,  4, }, // Asia/Hovd
    {  100, 4184,    2361,  7, }, // Asia/Irkutsk
    {  100, 7006,   28638,  8, }, // Asia/Jakarta
    {  100, 6992,   28646,  8, }, // Asia/Jayapura
    {  100, 1290,    2383,  9, }, // Asia/Jerusalem
    {  100, 3560,    2392,  5, }, // Asia/Kabul
    {  100, 4316,   51927, 23, }, // Asia/Kamchatka
    {  100, 4239,    2422,  6, }, // Asia/Karachi
    {  100, 1185,    2428,  8, }, // Asia/Kathmandu
    {  100, 8661,    2436,  7, }, // Asia/Khandyga
    {  100,  992,   51950,  9, }, // Asia/Kolkata
    {  100, 4197,   36978, 10, }, // Asia/Krasnoyarsk
    {  100, 4859,    2462, 12, }, // Asia/Kuala_Lumpur
    {  100, 5976,    2474,  6, }, // Asia/Kuching
    {  100, 5547,   28693,  6, }, // Asia/Kuwait
    {  100, 1211,    2486,  5, }, // Asia/Macau
    {  100, 4017,    2491,  7, }, // Asia/Magadan
    {  100, 1349,    2498,  7, }, // Asia/Makassar
    {  100, 7182,    2505,  6, }, // Asia/Manila
    {  100, 6920,    2511,  6, }, // Asia/Muscat
    {  100, 2554,    2517,  6, }, // Asia/Nicosia
    {  100, 8675,    2524, 11, }, // Asia/Novokuznetsk
    {  100, 4135,    2535, 11, }, // Asia/Novosibirsk
    {  100, 4229,    2546,  4, }, // Asia/Omsk
    {  100, 7145,   28739,  7, }, // Asia/Oral
    {  100, 6961,   51959,  8, }, // Asia/Phnom_Penh
    {  100, 8693,    2563,  9, }, // Asia/Pontianak
    {  100, 4214,   28763,  7, }, // Asia/Pyongyang
    {  100, 5559,    2579,  5, }, // Asia/Qatar
    {  100, 8708,   37005,  8, }, // Asia/Qostanay
    {  100, 4269,    2592,  9, }, // Asia/Qyzylorda
    {  100, 3584,   51967,  7, }, // Asia/Riyadh
    {  100, 4408,   56395, 14, }, // Asia/Sakhalin
    {  100, 7245,    2621,  9, }, // Asia/Samarkand
    {  100, 3279,    2630,  4, }, // Asia/Seoul
    {  100, 1053,   28801,  6, }, // Asia/Shanghai
    {  100, 3300,    2640,  8, }, // Asia/Singapore
    {  100, 4297,    2648, 13, }, // Asia/Srednekolymsk
    {  100, 3263,   28829,  6, }, // Asia/Taipei
    {  100, 4781,    2667,  7, }, // Asia/Tashkent
    {  100, 3915,   35074,  7, }, // Asia/Tbilisi
    {  100, 2772,    2678,  7, }, // Asia/Tehran
    {  100, 1317,    2685,  7, }, // Asia/Thimphu
    {  100, 2821,    2692,  5, }, // Asia/Tokyo
    {  100, 4572,    2697,  5, }, // Asia/Tomsk
    {  100, 1021,    2702, 10, }, // Asia/Ulaanbaatar
    {  100, 1159,    2712,  6, }, // Asia/Urumqi
    {  100, 8722,   51993,  9, }, // Asia/Ust-Nera
    {  100, 6977,   52002,  8, }, // Asia/Vientiane
    {  100, 4710,    2733, 11, }, // Asia/Vladivostok
    {  100, 4828,   37051,  6, }, // Asia/Yakutsk
    {  100, 1235,   56409,  5, }, // Asia/Yangon
    {  100, 3883,    2757, 12, }, // Asia/Yekaterinburg
    {  100, 3747,    2769,  6, }, // Asia/Yerevan
    {  100, 3663,   56414, 14, }, // Atlantic/Azores
    {  100, 5671,   56428, 16, }, // Atlantic/Bermuda
    {  100, 6628,   56444, 15, }, // Atlantic/Canary
    {  100, 3727,    2826, 10, }, // Atlantic/Cape_Verde
    {  100, 1395,   56459, 15, }, // Atlantic/Faroe
    {  100, 8736,   37132,  7, }, // Atlantic/Madeira
    {  100, 2748,   52082,  9, }, // Atlantic/Reykjavik
    {  100, 7272,   56474, 13, }, // Atlantic/South_Georgia
    {  100, 6821,   56487, 20, }, // Atlantic/St_Helena
    {  100, 6644,   52124,  6, }, // Atlantic/Stanley
    {  100, 1670,    2917,  8, }, // Australia/Adelaide
    {  100, 1635,    2925,  7, }, // Australia/Brisbane
    {  100, 1799,    2932, 11, }, // Australia/Broken_Hill
    {  100, 1583,    2943,  6, }, // Australia/Darwin
    {  100, 3637,   29108,  4, }, // Australia/Eucla
    {  100, 1516,    2954,  6, }, // Australia/Hobart
    {  100, 8753,    2960,  8, }, // Australia/Lindeman
    {  100, 1547,    2968,  8, }, // Australia/Lord_Howe
    {  100, 1727,   29120,  8, }, // Australia/Melbourne
    {  100, 1762,    2983,  4, }, // Australia/Perth
    {  100, 1463,   52144,  6, }, // Australia/Sydney
    {  100, 6441,    2993,  9, }, // Europe/Amsterdam
    {  100, 6216,    3002,  7, }, // Europe/Andorra
    {  100, 3620,   29149,  9, }, // Europe/Astrakhan
    {  100, 2300,   56507,  7, }, // Europe/Athens
    {  100, 6336,    3021,  7, }, // Europe/Belgrade
    {  100, 4757,    3028,  6, }, // Europe/Berlin
    {  100, 6488,    3034, 10, }, // Europe/Bratislava
    {  100, 2167,   52164,  8, }, // Europe/Brussels
    {  100, 3928,    3052,  8, }, // Europe/Bucharest
    {  100, 3806,    3060,  8, }, // Europe/Budapest
    {  100, 8772,   37236,  8, }, // Europe/Busingen
    {  100, 2583,    3091,  7, }, // Europe/Chisinau
    {  100, 6289,   52191, 10, }, // Europe/Copenhagen
    {  100, 2338,    3108,  6, }, // Europe/Dublin
    {  100, 6307,    3114,  9, }, // Europe/Gibraltar
    {  100, 8788,    3123,  6, }, // Europe/Guernsey
    {  100, 6612,   52201,  9, }, // Europe/Helsinki
    {  100, 8804,   56514, 10, }, // Europe/Isle_of_Man
    {  100, 1130,    3146,  7, }, // Europe/Istanbul
    {  100, 8823,   52219,  6, }, // Europe/Jersey
    {  100, 3979,    3158, 11, }, // Europe/Kaliningrad
    {  100, 8929,    3169,  5, }, // Europe/Kirov
    {  100, 2527,    3174,  4, }, // Europe/Kyiv
    {  100, 3213,    3178,  8, }, // Europe/Lisbon
    {  100, 6506,   29300,  7, }, // Europe/Ljubljana
    {  100, 2501,    3195,  6, }, // Europe/London
    {  100, 6365,   29313, 10, }, // Europe/Luxembourg
    {  100, 4892,    3212,  6, }, // Europe/Madrid
    {  100, 6397,   29329,  6, }, // Europe/Malta
    {  100, 6582,    3223,  9, }, // Europe/Mariehamn
    {  100, 3693,    3232,  5, }, // Europe/Minsk
    {  100, 6410,    3237,  6, }, // Europe/Monaco
    {  100, 3537,   56524,  6, }, // Europe/Moscow
    {  100, 6458,    3249,  4, }, // Europe/Oslo
    {  100, 4284,    3253,  5, }, // Europe/Paris
    {  100, 6424,    3258,  9, }, // Europe/Podgorica
    {  100, 6275,    3267,  5, }, // Europe/Prague
    {  100, 8837,    3272,  4, }, // Europe/Riga
    {  100, 6324,    3276,  3, }, // Europe/Rome
    {  100, 4331,    3279,  6, }, // Europe/Samara
    {  100, 6470,    3285, 10, }, // Europe/San_Marino
    {  100, 6245,    3295,  7, }, // Europe/Sarajevo
    {  100, 4451,    3302,  7, }, // Europe/Saratov
    {  100, 8849,   52225, 11, }, // Europe/Simferopol
    {  100, 6383,   52236,  6, }, // Europe/Skopje
    {  100, 6599,    3324,  4, }, // Europe/Sofia
    {  100, 6523,   52242,  9, }, // Europe/Stockholm
    {  100, 8867,    3337,  6, }, // Europe/Tallinn
    {  100, 6202,    3343,  6, }, // Europe/Tirane
    {  100, 8882,   52251,  9, }, // Europe/Ulyanovsk
    {  100, 6352,    3358,  5, }, // Europe/Vaduz
    {  100, 6567,    3363,  7, }, // Europe/Vatican
    {  100, 6231,    3370,  4, }, // Europe/Vienna
    {  100, 8899,   52260,  7, }, // Europe/Vilnius
    {  100, 4727,    3381,  9, }, // Europe/Volgograd
    {  100, 3190,    3390,  7, }, // Europe/Warsaw
    {  100, 6261,    3397,  6, }, // Europe/Zagreb
    {  100, 6540,   37310,  5, }, // Europe/Zurich
    {  100, 5078,    3409, 12, }, // Indian/Antananarivo
    {  100, 6947,    3421,  5, }, // Indian/Chagos
    {  100, 6046,   56530, 13, }, // Indian/Christmas
    {  100, 6063,   56543, 15, }, // Indian/Cocos
    {  100, 5029,   56558,  8, }, // Indian/Comoro
    {  100, 6661,    3460,  8, }, // Indian/Kerguelen
    {  100, 7260,   29562,  3, }, // Indian/Mahe
    {  100, 7050,   56566, 10, }, // Indian/Maldives
    {  100, 4069,   56576,  8, }, // Indian/Mauritius
    {  100, 5098,   56584,  7, }, // Indian/Mayotte
    {  100, 7211,   52348,  7, }, // Indian/Reunion
    {  100, 4422,    3498,  4, }, // Pacific/Apia
    {  100,  932,    3502,  6, }, // Pacific/Auckland
    {  100, 3706,   52355,  9, }, // Pacific/Bougainville
    {  100, 3015,    3516,  5, }, // Pacific/Chatham
    {  100, 3157,    3521,  4, }, // Pacific/Chuuk
    {  100, 2237,   56591, 13, }, // Pacific/Easter
    {  100, 7360,   52409,  5, }, // Pacific/Efate
    {  100, 7327,    3541,  7, }, // Pacific/Fakaofo
    {  100, 3902,   37407,  5, }, // Pacific/Fiji
    {  100, 7343,    3552,  8, }, // Pacific/Funafuti
    {  100, 6678,   56604, 19, }, // Pacific/Galapagos
    {  100, 6696,   56623, 16, }, // Pacific/Gambier
    {  100, 3822,    3595, 11, }, // Pacific/Guadalcanal
    {  100, 6018,    3606,  4, }, // Pacific/Guam
    {  100, 3049,    3610,  6, }, // Pacific/Kanton
    {  100, 3998,    3616, 10, }, // Pacific/Kiritimati
    {  100, 7019,    3626,  6, }, // Pacific/Kosrae
    {  100, 2842,   29709, 10, }, // Pacific/Kwajalein
    {  100, 4877,   37442,  7, }, // Pacific/Majuro
    {  100, 4051,   56639, 18, }, // Pacific/Marquesas
    {  100, 8914,   52490,  6, }, // Pacific/Midway
    {  100, 7084,    3680,  5, }, // Pacific/Nauru
    {  100, 7132,   52504,  4, }, // Pacific/Niue
    {  100, 4168,    3689,  7, }, // Pacific/Norfolk
    {  100, 7098,    3696,  5, }, // Pacific/Noumea
    {  100, 3126,    3701,  9, }, // Pacific/Pago_Pago
    {  100, 7155,    3710,  5, }, // Pacific/Palau
    {  100, 7194,   56657,  7, }, // Pacific/Pitcairn
    {  100, 3096,    3722,  6, }, // Pacific/Pohnpei
    {  100, 4807,    3728, 11, }, // Pacific/Port_Moresby
    {  100, 6076,    3739,  9, }, // Pacific/Rarotonga
    {  100, 6031,   29803,  6, }, // Pacific/Saipan
    {  100, 7312,    3754,  5, }, // Pacific/Tahiti
    {  100, 6712,    3759,  6, }, // Pacific/Tarawa
    {  100, 4583,    3765,  9, }, // Pacific/Tongatapu
    {  100, 7392,   29823,  4, }, // Pacific/Wake
    {  100, 7405,    3778,  6, }, // Pacific/Wallis
    {  101,   14,       0,  6, }, // Africa/Asmara Colognian/Latin/Germany
    {  101, 2320,   56664,  6, }, // Africa/Cairo
    {  101, 4744,   56670,  6, }, // Africa/Lagos
    {  101, 4435,       6,  8, }, // Africa/Sao_Tome
    {  101, 4152,   56676,  7, }, // Africa/Windhoek
    {  101, 4252,      14,  8, }, // America/Asuncion
    {  101,  237,      22,  8, }, // America/Atikokan
    {  101, 7623,      30, 17, }, // America/Bahia_Banderas
    {  101, 3868,      47,  6, }, // America/Cancun
    {  101, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  101, 5422,   56683, 10, }, // America/Costa_Rica
    {  101, 5723,      66,  7, }, // America/Curacao
    {  101, 4841,   56693,  9, }, // America/Guayaquil
    {  101,  481,   56702, 15, }, // America/Indiana/Knox
    {  101, 7961,   56717, 18, }, // America/Indiana/Marengo
    {  101, 7985,   56735, 21, }, // America/Indiana/Petersburg
    {  101, 8012,   56756, 20, }, // America/Indiana/Tell_City
    {  101, 8038,   56776, 16, }, // America/Indiana/Vevay
    {  101, 8060,   56792, 20, }, // America/Indiana/Vincennes
    {  101, 8086,   56812, 18, }, // America/Indiana/Winamac
    {  101, 8125,   56830, 22, }, // America/Kentucky/Monticello
    {  101, 5932,     207, 22, }, // America/Lower_Princes
    {  101, 8220,     229,  6, }, // America/Merida
    {  101, 2949,   56852, 14, }, // America/Mexico_City
    {  101, 8301,   56866, 21, }, // America/North_Dakota/Beulah
    {  101, 8329,   56887, 21, }, // America/North_Dakota/Center
    {  101, 8357,   56908, 24, }, // America/North_Dakota/New_Salem
    {  101,  393,     298,  4, }, // America/Nuuk
    {  101, 6899,     302, 16, }, // America/Scoresbysund
    {  101, 8508,   56932, 15, }, // America/St_Barthelemy
    {  101, 2061,   56947, 11, }, // America/St_Johns
    {  101, 5863,   56958, 10, }, // America/St_Kitts
    {  101, 5880,   56968, 16, }, // America/St_Lucia
    {  101,  855,   56984, 11, }, // America/St_Thomas
    {  101, 5913,   56995, 15, }, // America/St_Vincent
    {  101, 8552,    7399,  7, }, // America/Yakutat
    {  101, 6133,   57010, 27, }, // Antarctica/DumontDUrville
    {  101, 7295,   57037, 36, }, // Antarctica/Syowa
    {  101, 7374,    4029,  6, }, // Antarctica/Vostok
    {  101, 5368,   57073,  6, }, // Asia/Aqtobe
    {  101,  964,   57079,  9, }, // Asia/Ashgabat
    {  101, 3607,    4044,  6, }, // Asia/Baghdad
    {  101, 3778,   57088,  8, }, // Asia/Bishkek
    {  101, 4540,    4083,  8, }, // Asia/Damascus
    {  101, 6159,   57096,  9, }, // Asia/Dushanbe
    {  101, 8651,   57105,  5, }, // Asia/Gaza
    {  101, 1259,   57110, 30, }, // Asia/Ho_Chi_Minh
    {  101, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  101, 4316,   57140, 11, }, // Asia/Kamchatka
    {  101, 1185,     413,  9, }, // Asia/Kathmandu
    {  101,  992,     422,  7, }, // Asia/Kolkata
    {  101, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  101, 1211,     429,  5, }, // Asia/Macau
    {  101, 2554,   57151,  8, }, // Asia/Nicosia
    {  101, 6961,   57159,  9, }, // Asia/Phnom_Penh
    {  101, 4269,   57168,  9, }, // Asia/Qyzylorda
    {  101, 3584,   33439,  5, }, // Asia/Riyadh
    {  101, 1053,   57177,  9, }, // Asia/Shanghai
    {  101, 3300,   57186, 10, }, // Asia/Singapore
    {  101, 4781,   57196,  9, }, // Asia/Tashkent
    {  101, 3915,   57205,  6, }, // Asia/Tbilisi
    {  101, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  101, 1159,   57211, 11, }, // Asia/Urumqi
    {  101, 4710,    4243, 11, }, // Asia/Vladivostok
    {  101, 4828,    7755,  7, }, // Asia/Yakutsk
    {  101, 1235,   57222,  6, }, // Asia/Yangon
    {  101, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  101, 3747,   57228,  6, }, // Asia/Yerevan
    {  101, 3663,   57234,  6, }, // Atlantic/Azores
    {  101, 6628,   57240,  8, }, // Atlantic/Canary
    {  101, 3727,   57248,  9, }, // Atlantic/Cape_Verde
    {  101, 1395,   57257,  5, }, // Atlantic/Faroe
    {  101, 6821,   57262, 11, }, // Atlantic/St_Helena
    {  101, 2300,    4320,  5, }, // Europe/Athens
    {  101, 6336,   57273,  7, }, // Europe/Belgrade
    {  101, 2167,   22634,  7, }, // Europe/Brussels
    {  101, 3928,   26897,  8, }, // Europe/Bucharest
    {  101, 6289,   57280, 10, }, // Europe/Copenhagen
    {  101, 6307,   57290,  9, }, // Europe/Gibraltar
    {  101, 8788,   57299,  7, }, // Europe/Guernsey
    {  101, 8804,   57306,  9, }, // Europe/Isle_of_Man
    {  101, 1130,   57315,  8, }, // Europe/Istanbul
    {  101, 8823,   57323,  6, }, // Europe/Jersey
    {  101, 3979,   57329, 13, }, // Europe/Kaliningrad
    {  101, 2527,     463,  4, }, // Europe/Kyiv
    {  101, 3213,    4381,  8, }, // Europe/Lisbon
    {  101, 6365,   57342, 11, }, // Europe/Luxembourg
    {  101, 3537,   57353,  6, }, // Europe/Moscow
    {  101, 6275,    8083,  4, }, // Europe/Prague
    {  101, 8837,   57359,  5, }, // Europe/Riga
    {  101, 6324,    5946,  3, }, // Europe/Rome
    {  101, 6202,   57364,  7, }, // Europe/Tirane
    {  101, 6567,   57371, 12, }, // Europe/Vatican
    {  101, 6231,   57383,  4, }, // Europe/Vienna
    {  101, 4727,   57387,  9, }, // Europe/Volgograd
    {  101, 3190,   57396,  8, }, // Europe/Warsaw
    {  101, 6261,   57404,  6, }, // Europe/Zagreb
    {  101, 6540,   57410,  7, }, // Europe/Zurich
    {  101, 7050,   57417,  8, }, // Indian/Maldives
    {  101, 7211,     467,  7, }, // Indian/Reunion
    {  101, 3157,     474,  5, }, // Pacific/Chuuk
    {  101, 6678,   57425, 10, }, // Pacific/Galapagos
    {  101, 6018,   57435,  4, }, // Pacific/Guam
    {  101, 2723,   57439,  8, }, // Pacific/Honolulu
    {  101, 7194,    8304,  7, }, // Pacific/Pitcairn
    {  101, 3096,     479,  7, }, // Pacific/Pohnpei
    {  102,   14,       0,  6, }, // Africa/Asmara Cornish/Latin/United Kingdom
    {  102, 4435,       6,  8, }, // Africa/Sao_Tome
    {  102, 4252,      14,  8, }, // America/Asuncion
    {  102,  237,      22,  8, }, // America/Atikokan
    {  102, 7623,      30, 17, }, // America/Bahia_Banderas
    {  102, 3868,      47,  6, }, // America/Cancun
    {  102, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  102, 5723,      66,  7, }, // America/Curacao
    {  102,  481,      73, 13, }, // America/Indiana/Knox
    {  102, 7961,      86, 16, }, // America/Indiana/Marengo
    {  102, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  102, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  102, 8038,     139, 14, }, // America/Indiana/Vevay
    {  102, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  102, 8086,     171, 16, }, // America/Indiana/Winamac
    {  102, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  102, 5932,     207, 22, }, // America/Lower_Princes
    {  102, 8220,     229,  6, }, // America/Merida
    {  102, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  102, 8329,     255, 20, }, // America/North_Dakota/Center
    {  102, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  102,  393,     298,  4, }, // America/Nuuk
    {  102, 6899,     302, 16, }, // America/Scoresbysund
    {  102, 8508,     318, 14, }, // America/St_Barthelemy
    {  102, 2061,     332, 10, }, // America/St_Johns
    {  102, 5863,     342,  9, }, // America/St_Kitts
    {  102, 5880,     351,  9, }, // America/St_Lucia
    {  102,  855,     360, 10, }, // America/St_Thomas
    {  102, 5913,     370, 11, }, // America/St_Vincent
    {  102, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  102, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  102, 1185,     413,  9, }, // Asia/Kathmandu
    {  102,  992,     422,  7, }, // Asia/Kolkata
    {  102, 1211,     429,  5, }, // Asia/Macau
    {  102, 1235,     442,  6, }, // Asia/Yangon
    {  102, 1395,     448,  5, }, // Atlantic/Faroe
    {  102, 6821,     453, 10, }, // Atlantic/St_Helena
    {  102, 2527,     463,  4, }, // Europe/Kyiv
    {  102, 7211,     467,  7, }, // Indian/Reunion
    {  102, 3157,     474,  5, }, // Pacific/Chuuk
    {  102, 3096,     479,  7, }, // Pacific/Pohnpei
    {  103,   14,       0,  6, }, // Africa/Asmara Corsican/Latin/France
    {  103, 4435,       6,  8, }, // Africa/Sao_Tome
    {  103, 4252,      14,  8, }, // America/Asuncion
    {  103,  237,      22,  8, }, // America/Atikokan
    {  103, 7623,      30, 17, }, // America/Bahia_Banderas
    {  103, 3868,      47,  6, }, // America/Cancun
    {  103, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  103, 5723,      66,  7, }, // America/Curacao
    {  103,  481,      73, 13, }, // America/Indiana/Knox
    {  103, 7961,      86, 16, }, // America/Indiana/Marengo
    {  103, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  103, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  103, 8038,     139, 14, }, // America/Indiana/Vevay
    {  103, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  103, 8086,     171, 16, }, // America/Indiana/Winamac
    {  103, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  103, 5932,     207, 22, }, // America/Lower_Princes
    {  103, 8220,     229,  6, }, // America/Merida
    {  103, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  103, 8329,     255, 20, }, // America/North_Dakota/Center
    {  103, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  103,  393,     298,  4, }, // America/Nuuk
    {  103, 6899,     302, 16, }, // America/Scoresbysund
    {  103, 8508,     318, 14, }, // America/St_Barthelemy
    {  103, 2061,     332, 10, }, // America/St_Johns
    {  103, 5863,     342,  9, }, // America/St_Kitts
    {  103, 5880,     351,  9, }, // America/St_Lucia
    {  103,  855,     360, 10, }, // America/St_Thomas
    {  103, 5913,     370, 11, }, // America/St_Vincent
    {  103, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  103, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  103, 1185,     413,  9, }, // Asia/Kathmandu
    {  103,  992,     422,  7, }, // Asia/Kolkata
    {  103, 1211,     429,  5, }, // Asia/Macau
    {  103, 1235,     442,  6, }, // Asia/Yangon
    {  103, 1395,     448,  5, }, // Atlantic/Faroe
    {  103, 6821,     453, 10, }, // Atlantic/St_Helena
    {  103, 2527,     463,  4, }, // Europe/Kyiv
    {  103, 7211,     467,  7, }, // Indian/Reunion
    {  103, 3157,     474,  5, }, // Pacific/Chuuk
    {  103, 3096,     479,  7, }, // Pacific/Pohnpei
    {  104, 5059,    3784, 11, }, // Africa/Addis_Ababa Croatian/Latin/Croatia
    {  104, 7420,   57447,  5, }, // Africa/Algiers
    {  104,   14,       0,  6, }, // Africa/Asmara
    {  104, 2320,    4572,  5, }, // Africa/Cairo
    {  104, 5043,   33154,  7, }, // Africa/Djibouti
    {  104, 6869,   20300,  4, }, // Africa/Lome
    {  104, 4435,       6,  8, }, // Africa/Sao_Tome
    {  104, 5607,   33168,  7, }, // America/Anguilla
    {  104, 4252,      14,  8, }, // America/Asuncion
    {  104,  237,      22,  8, }, // America/Atikokan
    {  104, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  104, 3868,   33182,  6, }, // America/Cancun
    {  104, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  104, 5422,   33188,  9, }, // America/Costa_Rica
    {  104, 5723,      66,  7, }, // America/Curacao
    {  104, 5739,   21548,  8, }, // America/Dominica
    {  104, 5441,    6688,  8, }, // America/El_Salvador
    {  104, 5786,   20408,  9, }, // America/Guadeloupe
    {  104, 3760,   33206,  9, }, // America/Guatemala
    {  104, 6932,   57452,  7, }, // America/Guyana
    {  104,  481,      73, 13, }, // America/Indiana/Knox
    {  104, 7961,      86, 16, }, // America/Indiana/Marengo
    {  104, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  104, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  104, 8038,     139, 14, }, // America/Indiana/Vevay
    {  104, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  104, 8086,     171, 16, }, // America/Indiana/Winamac
    {  104, 2799,   33215,  7, }, // America/Jamaica
    {  104, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  104, 5932,     207, 22, }, // America/Lower_Princes
    {  104, 8220,    7008,  6, }, // America/Merida
    {  104, 2949,   57459, 16, }, // America/Mexico_City
    {  104, 8301,   33222, 23, }, // America/North_Dakota/Beulah
    {  104, 8329,   33245, 23, }, // America/North_Dakota/Center
    {  104, 8357,   33268, 26, }, // America/North_Dakota/New_Salem
    {  104,  393,     298,  4, }, // America/Nuuk
    {  104, 5843,   33294,  9, }, // America/Puerto_Rico
    {  104, 6899,     302, 16, }, // America/Scoresbysund
    {  104, 8508,   35804, 16, }, // America/St_Barthelemy
    {  104, 2061,     332, 10, }, // America/St_Johns
    {  104, 5863,     342,  9, }, // America/St_Kitts
    {  104, 5880,     351,  9, }, // America/St_Lucia
    {  104,  855,     360, 10, }, // America/St_Thomas
    {  104, 5913,     370, 11, }, // America/St_Vincent
    {  104, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  104, 5395,   57475,  8, }, // Asia/Almaty
    {  104, 5511,    7482,  6, }, // Asia/Anadyr
    {  104, 5523,    5491,  5, }, // Asia/Aqtau
    {  104, 5368,    7488,  6, }, // Asia/Aqtobe
    {  104,  964,   57483,  7, }, // Asia/Ashgabat
    {  104, 3607,    4044,  6, }, // Asia/Baghdad
    {  104, 5534,    4050,  7, }, // Asia/Bahrain
    {  104, 3778,   33323,  6, }, // Asia/Bishkek
    {  104, 5989,   33329,  6, }, // Asia/Brunei
    {  104, 4601,   57490,  4, }, // Asia/Chita
    {  104, 4540,    4083,  6, }, // Asia/Damascus
    {  104, 6159,   33336,  7, }, // Asia/Dushanbe
    {  104, 1259,   33343,  9, }, // Asia/Ho_Chi_Minh
    {  104, 7006,   33352,  8, }, // Asia/Jakarta
    {  104, 1290,   33369,  9, }, // Asia/Jerusalem
    {  104, 4316,   33378,  8, }, // Asia/Kamchatka
    {  104, 4239,   33386,  6, }, // Asia/Karachi
    {  104, 1185,    5593,  8, }, // Asia/Kathmandu
    {  104, 8661,   33392,  7, }, // Asia/Khandyga
    {  104,  992,     422,  7, }, // Asia/Kolkata
    {  104, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  104, 5547,    7607,  6, }, // Asia/Kuwait
    {  104, 1211,    7613,  5, }, // Asia/Macau
    {  104, 2554,   33410,  8, }, // Asia/Nicosia
    {  104, 8675,   33418, 12, }, // Asia/Novokuznetsk
    {  104, 4214,   33430,  9, }, // Asia/Pyongyang
    {  104, 5559,    4214,  5, }, // Asia/Qatar
    {  104, 8708,     434,  8, }, // Asia/Qostanay
    {  104, 4269,    7660,  9, }, // Asia/Qyzylorda
    {  104, 3584,   33439,  5, }, // Asia/Riyadh
    {  104, 4408,   33444,  7, }, // Asia/Sakhalin
    {  104, 3279,    7680,  4, }, // Asia/Seoul
    {  104, 1053,   33451,  6, }, // Asia/Shanghai
    {  104, 3300,   20782,  8, }, // Asia/Singapore
    {  104, 4297,    7699, 13, }, // Asia/Srednekolymsk
    {  104, 4781,   33457,  7, }, // Asia/Tashkent
    {  104, 2772,    7724,  7, }, // Asia/Tehran
    {  104, 2821,    4238,  5, }, // Asia/Tokyo
    {  104, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  104, 1159,   33464,  6, }, // Asia/Urumqi
    {  104, 4828,    7755,  7, }, // Asia/Yakutsk
    {  104, 1235,    7762,  6, }, // Asia/Yangon
    {  104, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  104, 3747,   26815,  6, }, // Asia/Yerevan
    {  104, 3663,   33480,  5, }, // Atlantic/Azores
    {  104, 5671,   57494,  7, }, // Atlantic/Bermuda
    {  104, 6628,    4280,  6, }, // Atlantic/Canary
    {  104, 1395,   57501, 14, }, // Atlantic/Faroe
    {  104, 7272,   57515, 14, }, // Atlantic/South_Georgia
    {  104, 6821,   33494, 12, }, // Atlantic/St_Helena
    {  104, 6216,   33506,  6, }, // Europe/Andorra
    {  104, 3620,   33512,  8, }, // Europe/Astrakhan
    {  104, 2300,   26877,  5, }, // Europe/Athens
    {  104, 6336,    7942,  7, }, // Europe/Belgrade
    {  104, 2167,   57529,  9, }, // Europe/Brussels
    {  104, 3928,   33531,  8, }, // Europe/Bucharest
    {  104, 3806,   33539, 10, }, // Europe/Budapest
    {  104, 2583,   33549,  8, }, // Europe/Chisinau
    {  104, 6289,    4357, 10, }, // Europe/Copenhagen
    {  104, 8804,   57538,  8, }, // Europe/Isle_of_Man
    {  104, 3979,   33579, 12, }, // Europe/Kaliningrad
    {  104, 2527,   33591,  5, }, // Europe/Kyiv
    {  104, 3213,   33596,  7, }, // Europe/Lisbon
    {  104, 6365,    8054, 10, }, // Europe/Luxembourg
    {  104, 6410,    5926,  6, }, // Europe/Monaco
    {  104, 3537,   22748,  6, }, // Europe/Moscow
    {  104, 4284,   33603,  5, }, // Europe/Paris
    {  104, 6275,    8083,  4, }, // Europe/Prague
    {  104, 6324,   33608,  3, }, // Europe/Rome
    {  104, 8849,   33611, 11, }, // Europe/Simferopol
    {  104, 6383,   33622,  7, }, // Europe/Skopje
    {  104, 6599,   33629,  6, }, // Europe/Sofia
    {  104, 6202,   21095,  6, }, // Europe/Tirane
    {  104, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  104, 6567,    5978,  7, }, // Europe/Vatican
    {  104, 6231,   33643,  3, }, // Europe/Vienna
    {  104, 3190,   33646,  7, }, // Europe/Warsaw
    {  104, 6540,    4459,  6, }, // Europe/Zurich
    {  104, 7050,   33687,  7, }, // Indian/Maldives
    {  104, 4069,   33694, 10, }, // Indian/Mauritius
    {  104, 7211,     467,  7, }, // Indian/Reunion
    {  104, 3157,     474,  5, }, // Pacific/Chuuk
    {  104, 3902,   33704,  5, }, // Pacific/Fiji
    {  104, 2723,    4493,  8, }, // Pacific/Honolulu
    {  104, 3049,    4501,  9, }, // Pacific/Kanton
    {  104, 4051,   57546, 18, }, // Pacific/Marquesas
    {  104, 3096,     479,  7, }, // Pacific/Pohnpei
    {  106, 6788,   57564,  7, }, // Africa/Abidjan Czech/Latin/Czechia
    {  106, 5059,    3784, 11, }, // Africa/Addis_Ababa
    {  106, 7420,   57571,  5, }, // Africa/Algiers
    {  106,   14,       0,  6, }, // Africa/Asmara
    {  106, 2320,   57576,  6, }, // Africa/Cairo
    {  106, 5043,   33154,  7, }, // Africa/Djibouti
    {  106, 4524,   57582,  7, }, // Africa/Khartoum
    {  106, 6869,   20300,  4, }, // Africa/Lome
    {  106, 5113,   57589,  8, }, // Africa/Mogadishu
    {  106, 5271,   57597,  8, }, // Africa/Ndjamena
    {  106, 6803,   57605,  8, }, // Africa/Nouakchott
    {  106, 4435,   57613, 11, }, // Africa/Sao_Tome
    {  106, 2866,   57624,  8, }, // Africa/Tripoli
    {  106,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  106, 4252,      14,  8, }, // America/Asuncion
    {  106,  237,      22,  8, }, // America/Atikokan
    {  106, 3679,      30,  5, }, // America/Bahia
    {  106, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  106, 7646,   35764,  5, }, // America/Belem
    {  106, 4361,   20389,  6, }, // America/Bogota
    {  106, 3868,      47,  6, }, // America/Cancun
    {  106, 5496,   57632, 17, }, // America/Cayman
    {  106, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  106, 5422,   33188,  9, }, // America/Costa_Rica
    {  106, 5723,      66,  7, }, // America/Curacao
    {  106, 5739,   21548,  8, }, // America/Dominica
    {  106, 5441,    6688,  8, }, // America/El_Salvador
    {  106,  481,      73, 13, }, // America/Indiana/Knox
    {  106, 7961,      86, 16, }, // America/Indiana/Marengo
    {  106, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  106, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  106, 8038,     139, 14, }, // America/Indiana/Vevay
    {  106, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  106, 8086,     171, 16, }, // America/Indiana/Winamac
    {  106, 2799,   33215,  7, }, // America/Jamaica
    {  106, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  106, 5932,     207, 22, }, // America/Lower_Princes
    {  106, 5805,    5142,  8, }, // America/Martinique
    {  106, 2917,   26471,  8, }, // America/Mazatlan
    {  106, 8220,    7008,  6, }, // America/Merida
    {  106, 2949,   57459, 16, }, // America/Mexico_City
    {  106, 8301,   57649, 22, }, // America/North_Dakota/Beulah
    {  106, 8329,   57671, 22, }, // America/North_Dakota/Center
    {  106, 8357,   57693, 25, }, // America/North_Dakota/New_Salem
    {  106,  393,     298,  4, }, // America/Nuuk
    {  106, 5843,   33294,  9, }, // America/Puerto_Rico
    {  106, 8477,   26572,  8, }, // America/Santarem
    {  106, 1878,   26580,  9, }, // America/Sao_Paulo
    {  106, 6899,     302, 16, }, // America/Scoresbysund
    {  106, 8508,   57718, 16, }, // America/St_Barthelemy
    {  106, 2061,     332, 10, }, // America/St_Johns
    {  106, 5863,   57734, 13, }, // America/St_Kitts
    {  106, 5880,   57747, 11, }, // America/St_Lucia
    {  106,  855,   57758, 21, }, // America/St_Thomas
    {  106, 5913,   57779, 13, }, // America/St_Vincent
    {  106, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  106, 3968,   57792,  5, }, // Asia/Amman
    {  106, 5523,    5491,  5, }, // Asia/Aqtau
    {  106, 5368,    7488,  6, }, // Asia/Aqtobe
    {  106,  964,   57797,  8, }, // Asia/Ashgabat
    {  106, 3607,   57805,  6, }, // Asia/Baghdad
    {  106, 5534,   57811,  7, }, // Asia/Bahrain
    {  106, 4086,   57818,  6, }, // Asia/Beirut
    {  106, 3778,   33323,  6, }, // Asia/Bishkek
    {  106, 5989,   33329,  6, }, // Asia/Brunei
    {  106, 4601,   57490,  4, }, // Asia/Chita
    {  106, 4511,    5519,  7, }, // Asia/Colombo
    {  106, 4540,   57824,  7, }, // Asia/Damascus
    {  106, 1093,   57831,  5, }, // Asia/Dhaka
    {  106, 3596,   57836,  5, }, // Asia/Dubai
    {  106, 6159,   33336,  7, }, // Asia/Dushanbe
    {  106, 1259,   57841, 18, }, // Asia/Ho_Chi_Minh
    {  106, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  106, 1290,   57859,  9, }, // Asia/Jerusalem
    {  106, 3560,   57868,  5, }, // Asia/Kabul
    {  106, 4316,   33378,  8, }, // Asia/Kamchatka
    {  106, 4239,   57873,  6, }, // Asia/Karachi
    {  106, 1185,   57879,  9, }, // Asia/Kathmandu
    {  106, 8661,   57888,  8, }, // Asia/Khandyga
    {  106,  992,   57896,  7, }, // Asia/Kolkata
    {  106, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  106, 5976,   33399,  6, }, // Asia/Kuching
    {  106, 5547,    7607,  6, }, // Asia/Kuwait
    {  106, 1211,     429,  5, }, // Asia/Macau
    {  106, 6920,   26750,  6, }, // Asia/Muscat
    {  106, 2554,   57903,  7, }, // Asia/Nicosia
    {  106, 8675,   57910, 11, }, // Asia/Novokuznetsk
    {  106, 7145,   57921,  6, }, // Asia/Oral
    {  106, 6961,   57927,  9, }, // Asia/Phnom_Penh
    {  106, 4214,   57936, 11, }, // Asia/Pyongyang
    {  106, 5559,    4214,  5, }, // Asia/Qatar
    {  106, 8708,    7652,  8, }, // Asia/Qostanay
    {  106, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  106, 3584,   57947,  5, }, // Asia/Riyadh
    {  106, 4408,   57952,  8, }, // Asia/Sakhalin
    {  106, 3279,   57960,  4, }, // Asia/Seoul
    {  106, 1053,   57964,  7, }, // Asia/Shanghai
    {  106, 3300,   20782,  8, }, // Asia/Singapore
    {  106, 4297,   57971, 13, }, // Asia/Srednekolymsk
    {  106, 3263,   57984,  9, }, // Asia/Taipei
    {  106, 4781,   33457,  7, }, // Asia/Tashkent
    {  106, 2772,   57993,  7, }, // Asia/Tehran
    {  106, 1317,   58000,  6, }, // Asia/Thimphu
    {  106, 2821,    4238,  5, }, // Asia/Tokyo
    {  106, 1021,   58006,  9, }, // Asia/Ulaanbaatar
    {  106, 1159,   33464,  6, }, // Asia/Urumqi
    {  106, 4828,    7755,  7, }, // Asia/Yakutsk
    {  106, 1235,   20819,  6, }, // Asia/Yangon
    {  106, 3883,   58015, 13, }, // Asia/Yekaterinburg
    {  106, 3747,    7780,  7, }, // Asia/Yerevan
    {  106, 3663,   58028, 15, }, // Atlantic/Azores
    {  106, 5671,   58043,  7, }, // Atlantic/Bermuda
    {  106, 6628,   58050, 16, }, // Atlantic/Canary
    {  106, 3727,   58066,  8, }, // Atlantic/Cape_Verde
    {  106, 1395,   58074, 15, }, // Atlantic/Faroe
    {  106, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  106, 7272,   58089, 13, }, // Atlantic/South_Georgia
    {  106, 6821,   58102, 12, }, // Atlantic/St_Helena
    {  106, 3620,   58114,  9, }, // Europe/Astrakhan
    {  106, 2300,   58123,  6, }, // Europe/Athens
    {  106, 6336,   58129,  8, }, // Europe/Belgrade
    {  106, 4757,   20941,  6, }, // Europe/Berlin
    {  106, 2167,   26890,  6, }, // Europe/Brussels
    {  106, 3928,   58137,  8, }, // Europe/Bucharest
    {  106, 3806,   58145,  8, }, // Europe/Budapest
    {  106, 2583,   58153,  7, }, // Europe/Chisinau
    {  106, 6289,   58160,  5, }, // Europe/Copenhagen
    {  106, 6612,   58165,  8, }, // Europe/Helsinki
    {  106, 8804,   58173, 10, }, // Europe/Isle_of_Man
    {  106, 2527,   58183,  5, }, // Europe/Kyiv
    {  106, 3213,   33596,  7, }, // Europe/Lisbon
    {  106, 6506,   58188,  6, }, // Europe/Ljubljana
    {  106, 2501,   58194,  6, }, // Europe/London
    {  106, 6365,   58200,  9, }, // Europe/Luxembourg
    {  106, 6410,    5926,  6, }, // Europe/Monaco
    {  106, 3537,   22748,  6, }, // Europe/Moscow
    {  106, 4284,   58209,  5, }, // Europe/Paris
    {  106, 6275,   36153,  5, }, // Europe/Prague
    {  106, 6324,   58214,  3, }, // Europe/Rome
    {  106, 6599,   58217,  5, }, // Europe/Sofia
    {  106, 6202,   21095,  6, }, // Europe/Tirane
    {  106, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  106, 6567,   58222,  7, }, // Europe/Vatican
    {  106, 6231,   58229,  5, }, // Europe/Vienna
    {  106, 3190,   33646,  7, }, // Europe/Warsaw
    {  106, 6261,   58234,  6, }, // Europe/Zagreb
    {  106, 6540,   58240,  6, }, // Europe/Zurich
    {  106, 6046,   58246, 14, }, // Indian/Christmas
    {  106, 6063,   58260, 16, }, // Indian/Cocos
    {  106, 5029,   58276,  6, }, // Indian/Comoro
    {  106, 6661,   58282, 20, }, // Indian/Kerguelen
    {  106, 7260,   26983,  4, }, // Indian/Mahe
    {  106, 7050,   58302,  8, }, // Indian/Maldives
    {  106, 4069,   58310,  9, }, // Indian/Mauritius
    {  106, 7211,     467,  7, }, // Indian/Reunion
    {  106, 3015,   58319, 18, }, // Pacific/Chatham
    {  106, 3157,   58337, 16, }, // Pacific/Chuuk
    {  106, 2237,   58353, 18, }, // Pacific/Easter
    {  106, 7360,   27016,  5, }, // Pacific/Efate
    {  106, 3902,   33704,  5, }, // Pacific/Fiji
    {  106, 6678,   58371,  8, }, // Pacific/Galapagos
    {  106, 6696,   58379, 18, }, // Pacific/Gambier
    {  106, 2723,    4493,  8, }, // Pacific/Honolulu
    {  106, 3049,    4501,  9, }, // Pacific/Kanton
    {  106, 4051,   58397,  7, }, // Pacific/Marquesas
    {  106, 7098,    4510,  6, }, // Pacific/Noumea
    {  106, 7194,   58404, 19, }, // Pacific/Pitcairn
    {  106, 3096,     479,  7, }, // Pacific/Pohnpei
    {  107, 5059,    3784, 11, }, // Africa/Addis_Ababa Danish/Latin/Denmark
    {  107, 7420,   58423,  6, }, // Africa/Algiers
    {  107,   14,       0,  6, }, // Africa/Asmara
    {  107, 4435,       6,  8, }, // Africa/Sao_Tome
    {  107, 4252,      14,  8, }, // America/Asuncion
    {  107,  237,      22,  8, }, // America/Atikokan
    {  107, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  107, 3868,   33182,  6, }, // America/Cancun
    {  107, 5496,   58429, 11, }, // America/Cayman
    {  107, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  107, 5723,      66,  7, }, // America/Curacao
    {  107,  481,      73, 13, }, // America/Indiana/Knox
    {  107, 7961,      86, 16, }, // America/Indiana/Marengo
    {  107, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  107, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  107, 8038,     139, 14, }, // America/Indiana/Vevay
    {  107, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  107, 8086,     171, 16, }, // America/Indiana/Winamac
    {  107, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  107, 5932,     207, 22, }, // America/Lower_Princes
    {  107, 8220,    7008,  6, }, // America/Merida
    {  107, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  107, 8329,     255, 20, }, // America/North_Dakota/Center
    {  107, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  107,  393,     298,  4, }, // America/Nuuk
    {  107, 1878,   26580,  9, }, // America/Sao_Paulo
    {  107, 6899,     302, 16, }, // America/Scoresbysund
    {  107, 8508,   26589, 16, }, // America/St_Barthelemy
    {  107, 2061,     332, 10, }, // America/St_Johns
    {  107, 5863,     342,  9, }, // America/St_Kitts
    {  107, 5880,     351,  9, }, // America/St_Lucia
    {  107,  855,     360, 10, }, // America/St_Thomas
    {  107, 5913,     370, 11, }, // America/St_Vincent
    {  107, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  107, 5523,    5491,  5, }, // Asia/Aqtau
    {  107, 5368,    7488,  6, }, // Asia/Aqtobe
    {  107,  964,   58440,  9, }, // Asia/Ashgabat
    {  107, 3607,    4044,  6, }, // Asia/Baghdad
    {  107, 3778,    4069,  7, }, // Asia/Bishkek
    {  107, 4540,    4083,  8, }, // Asia/Damascus
    {  107, 6159,   58449,  8, }, // Asia/Dushanbe
    {  107, 1259,     397, 16, }, // Asia/Ho_Chi_Minh
    {  107, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  107, 4316,   58457,  9, }, // Asia/Kamchatka
    {  107, 1185,    5593,  8, }, // Asia/Kathmandu
    {  107,  992,     422,  7, }, // Asia/Kolkata
    {  107, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  107, 1211,     429,  5, }, // Asia/Macau
    {  107, 8675,   58466, 12, }, // Asia/Novokuznetsk
    {  107, 8708,     434,  8, }, // Asia/Qostanay
    {  107, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  107, 4781,   58478,  8, }, // Asia/Tashkent
    {  107, 2772,    7724,  7, }, // Asia/Tehran
    {  107, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  107, 1159,   26801,  6, }, // Asia/Urumqi
    {  107, 4828,    7755,  7, }, // Asia/Yakutsk
    {  107, 1235,   58486,  7, }, // Asia/Yangon
    {  107, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  107, 3747,    7780,  7, }, // Asia/Yerevan
    {  107, 3663,   58493,  8, }, // Atlantic/Azores
    {  107, 6628,   58501, 16, }, // Atlantic/Canary
    {  107, 3727,   57248,  9, }, // Atlantic/Cape_Verde
    {  107, 1395,   58517,  8, }, // Atlantic/Faroe
    {  107, 6821,     453, 10, }, // Atlantic/St_Helena
    {  107, 2300,    4320,  5, }, // Europe/Athens
    {  107, 6336,    7942,  7, }, // Europe/Belgrade
    {  107, 2167,   57529,  9, }, // Europe/Brussels
    {  107, 3928,   26897,  8, }, // Europe/Bucharest
    {  107, 6289,   58525,  9, }, // Europe/Copenhagen
    {  107, 2527,    8030,  4, }, // Europe/Kyiv
    {  107, 3213,    4381,  8, }, // Europe/Lisbon
    {  107, 3537,   22748,  6, }, // Europe/Moscow
    {  107, 6275,    8083,  4, }, // Europe/Prague
    {  107, 6324,    5946,  3, }, // Europe/Rome
    {  107, 6202,   21095,  6, }, // Europe/Tirane
    {  107, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  107, 6567,   58534,  9, }, // Europe/Vatican
    {  107, 6231,   57383,  4, }, // Europe/Vienna
    {  107, 3190,   58543,  8, }, // Europe/Warsaw
    {  107, 6540,    4459,  6, }, // Europe/Zurich
    {  107, 6046,   58551,  7, }, // Indian/Christmas
    {  107, 5029,   58558,  9, }, // Indian/Comoro
    {  107, 7050,   58567, 10, }, // Indian/Maldives
    {  107, 7211,     467,  7, }, // Indian/Reunion
    {  107, 3157,     474,  5, }, // Pacific/Chuuk
    {  107, 2237,   58577,  8, }, // Pacific/Easter
    {  107, 2723,    4493,  8, }, // Pacific/Honolulu
    {  107, 3049,    4501,  9, }, // Pacific/Kanton
    {  107, 3096,     479,  7, }, // Pacific/Pohnpei
    {  109,   14,       0,  6, }, // Africa/Asmara Divehi/Thaana/Maldives
    {  109, 4435,       6,  8, }, // Africa/Sao_Tome
    {  109, 4252,      14,  8, }, // America/Asuncion
    {  109,  237,      22,  8, }, // America/Atikokan
    {  109, 7623,      30, 17, }, // America/Bahia_Banderas
    {  109, 3868,      47,  6, }, // America/Cancun
    {  109, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  109, 5723,      66,  7, }, // America/Curacao
    {  109,  481,      73, 13, }, // America/Indiana/Knox
    {  109, 7961,      86, 16, }, // America/Indiana/Marengo
    {  109, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  109, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  109, 8038,     139, 14, }, // America/Indiana/Vevay
    {  109, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  109, 8086,     171, 16, }, // America/Indiana/Winamac
    {  109, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  109, 5932,     207, 22, }, // America/Lower_Princes
    {  109, 8220,     229,  6, }, // America/Merida
    {  109, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  109, 8329,     255, 20, }, // America/North_Dakota/Center
    {  109, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  109,  393,     298,  4, }, // America/Nuuk
    {  109, 6899,     302, 16, }, // America/Scoresbysund
    {  109, 8508,     318, 14, }, // America/St_Barthelemy
    {  109, 2061,     332, 10, }, // America/St_Johns
    {  109, 5863,     342,  9, }, // America/St_Kitts
    {  109, 5880,     351,  9, }, // America/St_Lucia
    {  109,  855,     360, 10, }, // America/St_Thomas
    {  109, 5913,     370, 11, }, // America/St_Vincent
    {  109, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  109, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  109, 1185,     413,  9, }, // Asia/Kathmandu
    {  109,  992,     422,  7, }, // Asia/Kolkata
    {  109, 1211,     429,  5, }, // Asia/Macau
    {  109, 1235,     442,  6, }, // Asia/Yangon
    {  109, 1395,     448,  5, }, // Atlantic/Faroe
    {  109, 6821,     453, 10, }, // Atlantic/St_Helena
    {  109, 2527,     463,  4, }, // Europe/Kyiv
    {  109, 7211,     467,  7, }, // Indian/Reunion
    {  109, 3157,     474,  5, }, // Pacific/Chuuk
    {  109, 3096,     479,  7, }, // Pacific/Pohnpei
    {  110,   14,       0,  6, }, // Africa/Asmara Dogri/Devanagari/India
    {  110, 4435,       6,  8, }, // Africa/Sao_Tome
    {  110, 4252,      14,  8, }, // America/Asuncion
    {  110,  237,      22,  8, }, // America/Atikokan
    {  110, 7623,      30, 17, }, // America/Bahia_Banderas
    {  110, 3868,      47,  6, }, // America/Cancun
    {  110, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  110, 5723,      66,  7, }, // America/Curacao
    {  110,  481,      73, 13, }, // America/Indiana/Knox
    {  110, 7961,      86, 16, }, // America/Indiana/Marengo
    {  110, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  110, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  110, 8038,     139, 14, }, // America/Indiana/Vevay
    {  110, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  110, 8086,     171, 16, }, // America/Indiana/Winamac
    {  110, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  110, 5932,     207, 22, }, // America/Lower_Princes
    {  110, 8220,     229,  6, }, // America/Merida
    {  110, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  110, 8329,     255, 20, }, // America/North_Dakota/Center
    {  110, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  110,  393,     298,  4, }, // America/Nuuk
    {  110, 6899,     302, 16, }, // America/Scoresbysund
    {  110, 8508,     318, 14, }, // America/St_Barthelemy
    {  110, 2061,     332, 10, }, // America/St_Johns
    {  110, 5863,     342,  9, }, // America/St_Kitts
    {  110, 5880,     351,  9, }, // America/St_Lucia
    {  110,  855,     360, 10, }, // America/St_Thomas
    {  110, 5913,     370, 11, }, // America/St_Vincent
    {  110, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  110, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  110, 1185,     413,  9, }, // Asia/Kathmandu
    {  110,  992,     422,  7, }, // Asia/Kolkata
    {  110, 1211,     429,  5, }, // Asia/Macau
    {  110, 1235,     442,  6, }, // Asia/Yangon
    {  110, 1395,     448,  5, }, // Atlantic/Faroe
    {  110, 6821,     453, 10, }, // Atlantic/St_Helena
    {  110, 2527,     463,  4, }, // Europe/Kyiv
    {  110, 7211,     467,  7, }, // Indian/Reunion
    {  110, 3157,     474,  5, }, // Pacific/Chuuk
    {  110, 3096,     479,  7, }, // Pacific/Pohnpei
    {  111,   14,       0,  6, }, // Africa/Asmara Duala/Latin/Cameroon
    {  111, 4435,       6,  8, }, // Africa/Sao_Tome
    {  111, 4252,      14,  8, }, // America/Asuncion
    {  111,  237,      22,  8, }, // America/Atikokan
    {  111, 7623,      30, 17, }, // America/Bahia_Banderas
    {  111, 3868,      47,  6, }, // America/Cancun
    {  111, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  111, 5723,      66,  7, }, // America/Curacao
    {  111,  481,      73, 13, }, // America/Indiana/Knox
    {  111, 7961,      86, 16, }, // America/Indiana/Marengo
    {  111, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  111, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  111, 8038,     139, 14, }, // America/Indiana/Vevay
    {  111, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  111, 8086,     171, 16, }, // America/Indiana/Winamac
    {  111, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  111, 5932,     207, 22, }, // America/Lower_Princes
    {  111, 8220,     229,  6, }, // America/Merida
    {  111, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  111, 8329,     255, 20, }, // America/North_Dakota/Center
    {  111, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  111,  393,     298,  4, }, // America/Nuuk
    {  111, 6899,     302, 16, }, // America/Scoresbysund
    {  111, 8508,     318, 14, }, // America/St_Barthelemy
    {  111, 2061,     332, 10, }, // America/St_Johns
    {  111, 5863,     342,  9, }, // America/St_Kitts
    {  111, 5880,     351,  9, }, // America/St_Lucia
    {  111,  855,     360, 10, }, // America/St_Thomas
    {  111, 5913,     370, 11, }, // America/St_Vincent
    {  111, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  111, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  111, 1185,     413,  9, }, // Asia/Kathmandu
    {  111,  992,     422,  7, }, // Asia/Kolkata
    {  111, 1211,     429,  5, }, // Asia/Macau
    {  111, 1235,     442,  6, }, // Asia/Yangon
    {  111, 1395,     448,  5, }, // Atlantic/Faroe
    {  111, 6821,     453, 10, }, // Atlantic/St_Helena
    {  111, 2527,     463,  4, }, // Europe/Kyiv
    {  111, 7211,     467,  7, }, // Indian/Reunion
    {  111, 3157,     474,  5, }, // Pacific/Chuuk
    {  111, 3096,     479,  7, }, // Pacific/Pohnpei
    {  112, 5059,    3784, 11, }, // Africa/Addis_Ababa Dutch/Latin/Netherlands
    {  112,   14,       0,  6, }, // Africa/Asmara
    {  112, 2320,   58585,  5, }, // Africa/Cairo
    {  112, 4524,   58590,  8, }, // Africa/Khartoum
    {  112, 6869,   20300,  4, }, // Africa/Lome
    {  112, 4435,   58598,  8, }, // Africa/Sao_Tome
    {  112,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  112, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {  112, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  112, 4252,      14,  8, }, // America/Asuncion
    {  112,  237,      22,  8, }, // America/Atikokan
    {  112, 7623,      30, 17, }, // America/Bahia_Banderas
    {  112, 7646,   35764,  5, }, // America/Belem
    {  112, 3868,   33182,  6, }, // America/Cancun
    {  112, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  112, 3791,   26442,  6, }, // America/Cuiaba
    {  112, 5723,      66,  7, }, // America/Curacao
    {  112,  481,      73, 13, }, // America/Indiana/Knox
    {  112, 7961,      86, 16, }, // America/Indiana/Marengo
    {  112, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  112, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  112, 8038,     139, 14, }, // America/Indiana/Vevay
    {  112, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  112, 8086,     171, 16, }, // America/Indiana/Winamac
    {  112, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  112, 5932,   58606, 24, }, // America/Lower_Princes
    {  112, 8153,   26456,  6, }, // America/Maceio
    {  112, 2917,   26471,  8, }, // America/Mazatlan
    {  112, 8220,     229,  6, }, // America/Merida
    {  112, 2949,   58630, 11, }, // America/Mexico_City
    {  112, 8301,    3895, 20, }, // America/North_Dakota/Beulah
    {  112, 8329,    3915, 20, }, // America/North_Dakota/Center
    {  112, 8357,    3935, 23, }, // America/North_Dakota/New_Salem
    {  112,  393,     298,  4, }, // America/Nuuk
    {  112, 1878,   26580,  9, }, // America/Sao_Paulo
    {  112, 6899,     302, 16, }, // America/Scoresbysund
    {  112, 8508,   26589, 16, }, // America/St_Barthelemy
    {  112, 2061,   26605, 12, }, // America/St_Johns
    {  112, 5863,   26617, 11, }, // America/St_Kitts
    {  112, 5880,   35832, 11, }, // America/St_Lucia
    {  112,  855,   44161, 12, }, // America/St_Thomas
    {  112, 5913,   26651, 13, }, // America/St_Vincent
    {  112, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  112, 5395,   58641,  8, }, // Asia/Almaty
    {  112, 5368,   57073,  6, }, // Asia/Aqtobe
    {  112,  964,    4035,  9, }, // Asia/Ashgabat
    {  112, 8624,   58649,  6, }, // Asia/Atyrau
    {  112, 3607,    4044,  6, }, // Asia/Baghdad
    {  112, 5534,    4050,  7, }, // Asia/Bahrain
    {  112, 3653,    4057,  5, }, // Asia/Baku
    {  112, 4086,    4062,  7, }, // Asia/Beirut
    {  112, 3778,    4069,  7, }, // Asia/Bishkek
    {  112, 6159,   58655,  9, }, // Asia/Dushanbe
    {  112, 1259,   58664, 15, }, // Asia/Ho_Chi_Minh
    {  112, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  112, 4184,    4121,  8, }, // Asia/Irkutsk
    {  112, 1290,   33369,  9, }, // Asia/Jerusalem
    {  112, 4316,    4143, 10, }, // Asia/Kamchatka
    {  112, 1185,     413,  9, }, // Asia/Kathmandu
    {  112,  992,   35921,  8, }, // Asia/Kolkata
    {  112, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  112, 5547,    4196,  7, }, // Asia/Kuwait
    {  112, 1211,    4203,  5, }, // Asia/Macau
    {  112, 7182,   58679,  7, }, // Asia/Manila
    {  112, 3584,   20756,  5, }, // Asia/Riyadh
    {  112, 4408,   57952,  8, }, // Asia/Sakhalin
    {  112, 1053,   58686,  8, }, // Asia/Shanghai
    {  112, 4781,   58478,  8, }, // Asia/Tashkent
    {  112, 2772,    7724,  7, }, // Asia/Tehran
    {  112, 2821,    4238,  5, }, // Asia/Tokyo
    {  112, 4828,    4254,  8, }, // Asia/Yakutsk
    {  112, 1235,   58486,  7, }, // Asia/Yangon
    {  112, 3883,   58694, 15, }, // Asia/Yekaterinburg
    {  112, 3747,    7780,  7, }, // Asia/Yerevan
    {  112, 3663,   58709,  6, }, // Atlantic/Azores
    {  112, 6628,   58715, 19, }, // Atlantic/Canary
    {  112, 3727,   58734, 10, }, // Atlantic/Cape_Verde
    {  112, 1395,   58744,  7, }, // Atlantic/Faroe
    {  112, 7272,   58751, 12, }, // Atlantic/South_Georgia
    {  112, 6821,   58763, 11, }, // Atlantic/St_Helena
    {  112, 2300,    4320,  6, }, // Europe/Athens
    {  112, 6336,   58774,  8, }, // Europe/Belgrade
    {  112, 4757,   58782,  7, }, // Europe/Berlin
    {  112, 2167,    4332,  7, }, // Europe/Brussels
    {  112, 3928,    4339,  9, }, // Europe/Bucharest
    {  112, 3806,    4348,  9, }, // Europe/Budapest
    {  112, 6289,    4357, 10, }, // Europe/Copenhagen
    {  112, 1130,   58789,  9, }, // Europe/Istanbul
    {  112, 2527,    8030,  4, }, // Europe/Kyiv
    {  112, 3213,    4381,  8, }, // Europe/Lisbon
    {  112, 2501,    4389,  6, }, // Europe/London
    {  112, 6365,    4395,  9, }, // Europe/Luxembourg
    {  112, 3537,    4404,  6, }, // Europe/Moscow
    {  112, 4284,   58798,  6, }, // Europe/Paris
    {  112, 6275,    4415,  5, }, // Europe/Prague
    {  112, 6202,   21095,  6, }, // Europe/Tirane
    {  112, 6567,   58804, 12, }, // Europe/Vatican
    {  112, 6231,   58816,  5, }, // Europe/Vienna
    {  112, 4727,    4443,  9, }, // Europe/Volgograd
    {  112, 3190,   57396,  8, }, // Europe/Warsaw
    {  112, 6540,    4459,  6, }, // Europe/Zurich
    {  112, 6947,   58821, 14, }, // Indian/Chagos
    {  112, 6046,   58835, 15, }, // Indian/Christmas
    {  112, 6063,   58850, 13, }, // Indian/Cocos
    {  112, 7260,   26983,  4, }, // Indian/Mahe
    {  112, 7050,   58863,  8, }, // Indian/Maldives
    {  112, 7211,     467,  7, }, // Indian/Reunion
    {  112, 3157,     474,  5, }, // Pacific/Chuuk
    {  112, 2237,   58871, 10, }, // Pacific/Easter
    {  112, 6696,   58881, 12, }, // Pacific/Gambier
    {  112, 2723,    4493,  8, }, // Pacific/Honolulu
    {  112, 3049,    4501,  9, }, // Pacific/Kanton
    {  112, 4051,   58893, 17, }, // Pacific/Marquesas
    {  112, 7098,    4510,  6, }, // Pacific/Noumea
    {  112, 3096,     479,  7, }, // Pacific/Pohnpei
    {  119, 6760,   58910,  6, }, // Africa/Accra Dzongkha/Tibetan/Bhutan
    {  119, 5059,   58916, 14, }, // Africa/Addis_Ababa
    {  119, 7420,   58930,  8, }, // Africa/Algiers
    {  119,   14,   58938,  7, }, // Africa/Asmara
    {  119,   44,   58945,  7, }, // Africa/Bamako
    {  119, 5257,   58952,  6, }, // Africa/Bangui
    {  119, 6746,   58958,  8, }, // Africa/Banjul
    {  119, 7435,   58966,  8, }, // Africa/Bissau
    {  119, 5287,   58974,  9, }, // Africa/Brazzaville
    {  119, 4922,   58983, 13, }, // Africa/Bujumbura
    {  119, 2320,   58996,  7, }, // Africa/Cairo
    {  119, 6773,   59003,  8, }, // Africa/Conakry
    {  119, 6840,   59011,  5, }, // Africa/Dakar
    {  119, 5043,   59016,  9, }, // Africa/Djibouti
    {  119, 4906,   59025, 12, }, // Africa/Gaborone
    {  119, 5015,   59037,  7, }, // Africa/Harare
    {  119, 5151,   59044,  7, }, // Africa/Kampala
    {  119, 4524,   59051,  7, }, // Africa/Khartoum
    {  119, 4973,   59058,  8, }, // Africa/Kigali
    {  119, 5306,   59066,  7, }, // Africa/Kinshasa
    {  119, 5336,   59073, 10, }, // Africa/Libreville
    {  119, 6869,   59083,  5, }, // Africa/Lome
    {  119, 5211,   59088,  8, }, // Africa/Luanda
    {  119, 5001,   59096,  7, }, // Africa/Lusaka
    {  119, 5322,   59103,  7, }, // Africa/Malabo
    {  119, 4987,   59110,  7, }, // Africa/Maputo
    {  119, 5197,   59117,  6, }, // Africa/Maseru
    {  119, 5182,   59123, 12, }, // Africa/Mbabane
    {  119, 5113,   59135, 11, }, // Africa/Mogadishu
    {  119, 7034,   59146, 11, }, // Africa/Monrovia
    {  119, 3853,   59157, 10, }, // Africa/Nairobi
    {  119, 5354,   59167,  7, }, // Africa/Niamey
    {  119, 6803,   59174, 10, }, // Africa/Nouakchott
    {  119, 6727,   59184, 10, }, // Africa/Ouagadougou
    {  119, 4435,   59194,  9, }, // Africa/Sao_Tome
    {  119, 2866,   59203,  9, }, // Africa/Tripoli
    {  119, 6554,   59212,  7, }, // Africa/Tunis
    {  119, 4152,   59219,  8, }, // Africa/Windhoek
    {  119, 5624,   59227, 11, }, // America/Antigua
    {  119,  166,   59238, 18, }, // America/Argentina/Buenos_Aires
    {  119, 4252,   59256, 12, }, // America/Asuncion
    {  119,  237,   59268, 11, }, // America/Atikokan
    {  119, 7623,      30, 17, }, // America/Bahia_Banderas
    {  119, 5654,   59279, 10, }, // America/Barbados
    {  119, 5407,   59289,  7, }, // America/Belize
    {  119, 4361,   59296,  7, }, // America/Bogota
    {  119, 7713,   59303, 15, }, // America/Cambridge_Bay
    {  119, 3868,      47,  6, }, // America/Cancun
    {  119, 4694,   59318,  7, }, // America/Caracas
    {  119, 2260,   59325,  8, }, // America/Chicago
    {  119, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  119, 5422,   59333, 10, }, // America/Costa_Rica
    {  119, 5723,      66,  7, }, // America/Curacao
    {  119, 7833,   59343,  8, }, // America/Dawson
    {  119, 7848,   59351, 15, }, // America/Dawson_Creek
    {  119, 5739,   59366, 10, }, // America/Dominica
    {  119,  893,   59376, 12, }, // America/Edmonton
    {  119, 5441,   59388, 13, }, // America/El_Salvador
    {  119, 6881,   59401, 10, }, // America/Goose_Bay
    {  119, 3760,   59411,  9, }, // America/Guatemala
    {  119, 6932,   59420,  6, }, // America/Guyana
    {  119, 1939,   59426, 10, }, // America/Halifax
    {  119,  481,      73, 13, }, // America/Indiana/Knox
    {  119, 7961,      86, 16, }, // America/Indiana/Marengo
    {  119, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  119, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  119, 8038,     139, 14, }, // America/Indiana/Vevay
    {  119, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  119, 8086,     171, 16, }, // America/Indiana/Winamac
    {  119, 8110,   59436,  9, }, // America/Inuvik
    {  119, 2799,   59445,  8, }, // America/Jamaica
    {  119, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  119, 4376,   59453,  6, }, // America/La_Paz
    {  119, 7169,   59459,  4, }, // America/Lima
    {  119, 5932,     207, 22, }, // America/Lower_Princes
    {  119, 8220,     229,  6, }, // America/Merida
    {  119, 2949,   59463, 16, }, // America/Mexico_City
    {  119, 8254,   59479,  9, }, // America/Moncton
    {  119, 4098,   59488, 15, }, // America/Montevideo
    {  119, 5481,   59503,  5, }, // America/Nassau
    {  119, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  119, 8329,     255, 20, }, // America/North_Dakota/Center
    {  119, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  119,  393,     298,  4, }, // America/Nuuk
    {  119, 2356,   59508,  5, }, // America/Panama
    {  119, 6173,   59513, 12, }, // America/Paramaribo
    {  119, 8424,   59525, 16, }, // America/Rankin_Inlet
    {  119, 1995,   59541,  7, }, // America/Regina
    {  119, 8460,   59548, 11, }, // America/Resolute
    {  119, 2201,   59559, 12, }, // America/Santiago
    {  119, 6111,   59571, 18, }, // America/Santo_Domingo
    {  119, 6899,     302, 16, }, // America/Scoresbysund
    {  119, 8508,     318, 14, }, // America/St_Barthelemy
    {  119, 2061,   59589, 17, }, // America/St_Johns
    {  119, 5863,     342,  9, }, // America/St_Kitts
    {  119, 5880,     351,  9, }, // America/St_Lucia
    {  119,  855,     360, 10, }, // America/St_Thomas
    {  119, 5913,     370, 11, }, // America/St_Vincent
    {  119, 8530,   59606, 14, }, // America/Swift_Current
    {  119, 5461,   59620, 14, }, // America/Tegucigalpa
    {  119,  313,   59634,  8, }, // America/Tijuana
    {  119,  608,   59642,  9, }, // America/Toronto
    {  119, 8552,   59651,  7, }, // America/Yakutat
    {  119, 6094,   59658,  7, }, // Antarctica/Davis
    {  119, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  119, 7066,   59665,  8, }, // Antarctica/Mawson
    {  119, 8589,   59673,  6, }, // Antarctica/Palmer
    {  119, 7226,   59679,  8, }, // Antarctica/Rothera
    {  119, 7374,   59687,  8, }, // Antarctica/Vostok
    {  119, 5534,   59695,  7, }, // Asia/Bahrain
    {  119, 5989,   59702,  8, }, // Asia/Brunei
    {  119, 4511,   59710, 10, }, // Asia/Colombo
    {  119, 1093,   59011,  4, }, // Asia/Dhaka
    {  119, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  119, 7006,   59720,  7, }, // Asia/Jakarta
    {  119, 6992,   59727,  8, }, // Asia/Jayapura
    {  119, 3560,   59735,  6, }, // Asia/Kabul
    {  119, 4239,   59741,  6, }, // Asia/Karachi
    {  119, 1185,   59747, 10, }, // Asia/Kathmandu
    {  119,  992,   59757,  8, }, // Asia/Kolkata
    {  119, 4859,   59765, 12, }, // Asia/Kuala_Lumpur
    {  119, 1211,     429,  5, }, // Asia/Macau
    {  119, 8693,   59777, 11, }, // Asia/Pontianak
    {  119, 2772,   59788,  6, }, // Asia/Tehran
    {  119, 1317,   59794,  7, }, // Asia/Thimphu
    {  119, 6977,   59801, 13, }, // Asia/Vientiane
    {  119, 1235,   59814,  7, }, // Asia/Yangon
    {  119, 5671,   59821,  7, }, // Atlantic/Bermuda
    {  119, 3727,   59828,  7, }, // Atlantic/Cape_Verde
    {  119, 1395,   59835,  8, }, // Atlantic/Faroe
    {  119, 2748,   59843, 11, }, // Atlantic/Reykjavik
    {  119, 7272,   59854, 11, }, // Atlantic/South_Georgia
    {  119, 6821,     453, 10, }, // Atlantic/St_Helena
    {  119, 6441,   59865,  9, }, // Europe/Amsterdam
    {  119, 6216,   59874,  8, }, // Europe/Andorra
    {  119, 2300,   59882,  8, }, // Europe/Athens
    {  119, 4757,   59890,  7, }, // Europe/Berlin
    {  119, 2167,   59897,  7, }, // Europe/Brussels
    {  119, 3928,   59904, 10, }, // Europe/Bucharest
    {  119, 3806,   59914, 10, }, // Europe/Budapest
    {  119, 2583,   59924,  9, }, // Europe/Chisinau
    {  119, 6289,   59933, 13, }, // Europe/Copenhagen
    {  119, 2338,   59946,  6, }, // Europe/Dublin
    {  119, 6612,   59952, 10, }, // Europe/Helsinki
    {  119, 2527,     463,  4, }, // Europe/Kyiv
    {  119, 2501,   59962,  7, }, // Europe/London
    {  119, 6365,   59969,  9, }, // Europe/Luxembourg
    {  119, 4892,   59978,  8, }, // Europe/Madrid
    {  119, 3693,   59986,  5, }, // Europe/Minsk
    {  119, 6410,   59991,  7, }, // Europe/Monaco
    {  119, 3537,   59998,  7, }, // Europe/Moscow
    {  119, 6458,   60005,  6, }, // Europe/Oslo
    {  119, 4284,   60011,  6, }, // Europe/Paris
    {  119, 6275,   60017,  3, }, // Europe/Prague
    {  119, 8837,   60020,  5, }, // Europe/Riga
    {  119, 6245,   60025,  9, }, // Europe/Sarajevo
    {  119, 6599,   60034,  7, }, // Europe/Sofia
    {  119, 6523,   60041, 12, }, // Europe/Stockholm
    {  119, 8867,   60053,  7, }, // Europe/Tallinn
    {  119, 6202,   60060,  6, }, // Europe/Tirane
    {  119, 6352,   60066,  6, }, // Europe/Vaduz
    {  119, 6231,   60072,  7, }, // Europe/Vienna
    {  119, 8899,   60079, 10, }, // Europe/Vilnius
    {  119, 3190,   60089,  5, }, // Europe/Warsaw
    {  119, 6540,   60094,  6, }, // Europe/Zurich
    {  119, 5078,   60100, 15, }, // Indian/Antananarivo
    {  119, 6046,   60115, 10, }, // Indian/Christmas
    {  119, 6063,   60125,  7, }, // Indian/Cocos
    {  119, 5029,   60132,  8, }, // Indian/Comoro
    {  119, 6661,   60140, 10, }, // Indian/Kerguelen
    {  119, 7050,   60150,  8, }, // Indian/Maldives
    {  119, 4069,   60158,  9, }, // Indian/Mauritius
    {  119, 7211,     467,  7, }, // Indian/Reunion
    {  119, 4422,   60167,  6, }, // Pacific/Apia
    {  119, 3157,     474,  5, }, // Pacific/Chuuk
    {  119, 7327,   60173,  8, }, // Pacific/Fakaofo
    {  119, 7343,   60181, 10, }, // Pacific/Funafuti
    {  119, 6696,   60191,  8, }, // Pacific/Gambier
    {  119, 4877,   60199,  8, }, // Pacific/Majuro
    {  119, 8914,   60207,  6, }, // Pacific/Midway
    {  119, 7084,   60213,  6, }, // Pacific/Nauru
    {  119, 3126,   60219, 10, }, // Pacific/Pago_Pago
    {  119, 7155,   60229,  5, }, // Pacific/Palau
    {  119, 3096,     479,  7, }, // Pacific/Pohnpei
    {  119, 6076,   60234, 12, }, // Pacific/Rarotonga
    {  119, 6031,   60246,  8, }, // Pacific/Saipan
    {  119, 4583,   60254, 10, }, // Pacific/Tongatapu
    {  119, 7392,   60264,  3, }, // Pacific/Wake
    {  120,   14,       0,  6, }, // Africa/Asmara Embu/Latin/Kenya
    {  120, 4435,       6,  8, }, // Africa/Sao_Tome
    {  120, 4252,      14,  8, }, // America/Asuncion
    {  120,  237,      22,  8, }, // America/Atikokan
    {  120, 7623,      30, 17, }, // America/Bahia_Banderas
    {  120, 3868,      47,  6, }, // America/Cancun
    {  120, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  120, 5723,      66,  7, }, // America/Curacao
    {  120,  481,      73, 13, }, // America/Indiana/Knox
    {  120, 7961,      86, 16, }, // America/Indiana/Marengo
    {  120, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  120, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  120, 8038,     139, 14, }, // America/Indiana/Vevay
    {  120, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  120, 8086,     171, 16, }, // America/Indiana/Winamac
    {  120, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  120, 5932,     207, 22, }, // America/Lower_Princes
    {  120, 8220,     229,  6, }, // America/Merida
    {  120, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  120, 8329,     255, 20, }, // America/North_Dakota/Center
    {  120, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  120,  393,     298,  4, }, // America/Nuuk
    {  120, 6899,     302, 16, }, // America/Scoresbysund
    {  120, 8508,     318, 14, }, // America/St_Barthelemy
    {  120, 2061,     332, 10, }, // America/St_Johns
    {  120, 5863,     342,  9, }, // America/St_Kitts
    {  120, 5880,     351,  9, }, // America/St_Lucia
    {  120,  855,     360, 10, }, // America/St_Thomas
    {  120, 5913,     370, 11, }, // America/St_Vincent
    {  120, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  120, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  120, 1185,     413,  9, }, // Asia/Kathmandu
    {  120,  992,     422,  7, }, // Asia/Kolkata
    {  120, 1211,     429,  5, }, // Asia/Macau
    {  120, 1235,     442,  6, }, // Asia/Yangon
    {  120, 1395,     448,  5, }, // Atlantic/Faroe
    {  120, 6821,     453, 10, }, // Atlantic/St_Helena
    {  120, 2527,     463,  4, }, // Europe/Kyiv
    {  120, 7211,     467,  7, }, // Indian/Reunion
    {  120, 3157,     474,  5, }, // Pacific/Chuuk
    {  120, 3096,     479,  7, }, // Pacific/Pohnpei
    {  121,   14,       0,  6, }, // Africa/Asmara English/Latin/United States
    {  121, 4435,       6,  8, }, // Africa/Sao_Tome
    {  121, 4252,      14,  8, }, // America/Asuncion
    {  121,  237,      22,  8, }, // America/Atikokan
    {  121, 7623,      30, 17, }, // America/Bahia_Banderas
    {  121, 3868,      47,  6, }, // America/Cancun
    {  121, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  121, 5723,      66,  7, }, // America/Curacao
    {  121,  481,      73, 13, }, // America/Indiana/Knox
    {  121, 7961,      86, 16, }, // America/Indiana/Marengo
    {  121, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  121, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  121, 8038,     139, 14, }, // America/Indiana/Vevay
    {  121, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  121, 8086,     171, 16, }, // America/Indiana/Winamac
    {  121, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  121, 5932,     207, 22, }, // America/Lower_Princes
    {  121, 8220,     229,  6, }, // America/Merida
    {  121, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  121, 8329,     255, 20, }, // America/North_Dakota/Center
    {  121, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  121,  393,     298,  4, }, // America/Nuuk
    {  121, 6899,     302, 16, }, // America/Scoresbysund
    {  121, 8508,     318, 14, }, // America/St_Barthelemy
    {  121, 2061,     332, 10, }, // America/St_Johns
    {  121, 5863,     342,  9, }, // America/St_Kitts
    {  121, 5880,     351,  9, }, // America/St_Lucia
    {  121,  855,     360, 10, }, // America/St_Thomas
    {  121, 5913,     370, 11, }, // America/St_Vincent
    {  121, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  121, 1259,     397, 16, }, // Asia/Ho_Chi_Minh
    {  121, 1185,     413,  9, }, // Asia/Kathmandu
    {  121,  992,     422,  7, }, // Asia/Kolkata
    {  121, 1211,     429,  5, }, // Asia/Macau
    {  121, 8708,     434,  8, }, // Asia/Qostanay
    {  121, 1235,     442,  6, }, // Asia/Yangon
    {  121, 1395,     448,  5, }, // Atlantic/Faroe
    {  121, 6821,     453, 10, }, // Atlantic/St_Helena
    {  121, 2527,     463,  4, }, // Europe/Kyiv
    {  121, 7211,     467,  7, }, // Indian/Reunion
    {  121, 3157,     474,  5, }, // Pacific/Chuuk
    {  121, 3096,     479,  7, }, // Pacific/Pohnpei
    {  122,  132,   60267,  8, }, // America/Adak English/Deseret/United States
    {  122, 3346,   60275, 14, }, // America/Anchorage
    {  122, 7699,   60289, 10, }, // America/Boise
    {  122, 2260,   60299, 12, }, // America/Chicago
    {  122,  805,   60311, 12, }, // America/Denver
    {  122, 3465,   60323, 14, }, // America/Detroit
    {  122,  348,   60337, 24, }, // America/Indiana/Indianapolis
    {  122,  481,   60361, 24, }, // America/Indiana/Knox
    {  122, 7961,   60385, 30, }, // America/Indiana/Marengo
    {  122, 7985,   60415, 36, }, // America/Indiana/Petersburg
    {  122, 8012,   60451, 31, }, // America/Indiana/Tell_City
    {  122, 8038,   60482, 24, }, // America/Indiana/Vevay
    {  122, 8060,   60506, 30, }, // America/Indiana/Vincennes
    {  122, 8086,   60536, 30, }, // America/Indiana/Winamac
    {  122, 5380,   60566,  8, }, // America/Juneau
    {  122,  521,   60574, 14, }, // America/Kentucky/Louisville
    {  122, 8125,   60588, 34, }, // America/Kentucky/Monticello
    {  122, 3239,   60622, 19, }, // America/Los_Angeles
    {  122, 8202,   60641, 16, }, // America/Menominee
    {  122, 2379,   60657, 13, }, // America/New_York
    {  122, 8288,   60670,  6, }, // America/Nome
    {  122, 8329,   60676, 35, }, // America/North_Dakota/Center
    {  122, 8357,   60711, 38, }, // America/North_Dakota/New_Salem
    {  122, 2973,   60749, 12, }, // America/Phoenix
    {  122, 8552,   60761, 14, }, // America/Yakutat
    {  122, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  122, 2723,   60775, 16, }, // Pacific/Honolulu
    {  122, 8914,   60791, 10, }, // Pacific/Midway
    {  122, 7392,   60801,  6, }, // Pacific/Wake
    {  124, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Anguilla
    {  124, 2061,   60820,  9, }, // America/St_Johns
    {  124, 5863,   60829,  8, }, // America/St_Kitts
    {  124, 5880,   60837,  8, }, // America/St_Lucia
    {  124,  855,   60845,  9, }, // America/St_Thomas
    {  124, 5913,   60854, 10, }, // America/St_Vincent
    {  124, 5523,    5491,  5, }, // Asia/Aqtau
    {  124, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  124, 3049,    4501,  9, }, // Pacific/Kanton
    {  125, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Antigua and Barbuda
    {  125, 2061,   60820,  9, }, // America/St_Johns
    {  125, 5863,   60829,  8, }, // America/St_Kitts
    {  125, 5880,   60837,  8, }, // America/St_Lucia
    {  125,  855,   60845,  9, }, // America/St_Thomas
    {  125, 5913,   60854, 10, }, // America/St_Vincent
    {  125, 5523,    5491,  5, }, // Asia/Aqtau
    {  125, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  125, 3049,    4501,  9, }, // Pacific/Kanton
    {  126, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Australia
    {  126, 2061,   60820,  9, }, // America/St_Johns
    {  126, 5863,   60829,  8, }, // America/St_Kitts
    {  126, 5880,   60837,  8, }, // America/St_Lucia
    {  126,  855,   60845,  9, }, // America/St_Thomas
    {  126, 5913,   60854, 10, }, // America/St_Vincent
    {  126, 5523,    5491,  5, }, // Asia/Aqtau
    {  126, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  126, 3049,    4501,  9, }, // Pacific/Kanton
    {  127, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Austria
    {  127, 2061,   60820,  9, }, // America/St_Johns
    {  127, 5863,   60829,  8, }, // America/St_Kitts
    {  127, 5880,   60837,  8, }, // America/St_Lucia
    {  127,  855,   60845,  9, }, // America/St_Thomas
    {  127, 5913,   60854, 10, }, // America/St_Vincent
    {  127, 5523,    5491,  5, }, // Asia/Aqtau
    {  127, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  127, 3049,    4501,  9, }, // Pacific/Kanton
    {  128, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Bahamas
    {  128, 2061,   60820,  9, }, // America/St_Johns
    {  128, 5863,   60829,  8, }, // America/St_Kitts
    {  128, 5880,   60837,  8, }, // America/St_Lucia
    {  128,  855,   60845,  9, }, // America/St_Thomas
    {  128, 5913,   60854, 10, }, // America/St_Vincent
    {  128, 5523,    5491,  5, }, // Asia/Aqtau
    {  128, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  128, 3049,    4501,  9, }, // Pacific/Kanton
    {  129, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Barbados
    {  129, 2061,   60820,  9, }, // America/St_Johns
    {  129, 5863,   60829,  8, }, // America/St_Kitts
    {  129, 5880,   60837,  8, }, // America/St_Lucia
    {  129,  855,   60845,  9, }, // America/St_Thomas
    {  129, 5913,   60854, 10, }, // America/St_Vincent
    {  129, 5523,    5491,  5, }, // Asia/Aqtau
    {  129, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  129, 3049,    4501,  9, }, // Pacific/Kanton
    {  130, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Belgium
    {  130, 2061,   60820,  9, }, // America/St_Johns
    {  130, 5863,   60829,  8, }, // America/St_Kitts
    {  130, 5880,   60837,  8, }, // America/St_Lucia
    {  130,  855,   60845,  9, }, // America/St_Thomas
    {  130, 5913,   60854, 10, }, // America/St_Vincent
    {  130, 5523,    5491,  5, }, // Asia/Aqtau
    {  130, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  130, 3049,    4501,  9, }, // Pacific/Kanton
    {  131, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Belize
    {  131, 2061,   60820,  9, }, // America/St_Johns
    {  131, 5863,   60829,  8, }, // America/St_Kitts
    {  131, 5880,   60837,  8, }, // America/St_Lucia
    {  131,  855,   60845,  9, }, // America/St_Thomas
    {  131, 5913,   60854, 10, }, // America/St_Vincent
    {  131, 5523,    5491,  5, }, // Asia/Aqtau
    {  131, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  131, 3049,    4501,  9, }, // Pacific/Kanton
    {  132, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Bermuda
    {  132, 2061,   60820,  9, }, // America/St_Johns
    {  132, 5863,   60829,  8, }, // America/St_Kitts
    {  132, 5880,   60837,  8, }, // America/St_Lucia
    {  132,  855,   60845,  9, }, // America/St_Thomas
    {  132, 5913,   60854, 10, }, // America/St_Vincent
    {  132, 5523,    5491,  5, }, // Asia/Aqtau
    {  132, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  132, 3049,    4501,  9, }, // Pacific/Kanton
    {  133, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Botswana
    {  133, 2061,   60820,  9, }, // America/St_Johns
    {  133, 5863,   60829,  8, }, // America/St_Kitts
    {  133, 5880,   60837,  8, }, // America/St_Lucia
    {  133,  855,   60845,  9, }, // America/St_Thomas
    {  133, 5913,   60854, 10, }, // America/St_Vincent
    {  133, 5523,    5491,  5, }, // Asia/Aqtau
    {  133, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  133, 3049,    4501,  9, }, // Pacific/Kanton
    {  134, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/British Indian Ocean Territory
    {  134, 2061,   60820,  9, }, // America/St_Johns
    {  134, 5863,   60829,  8, }, // America/St_Kitts
    {  134, 5880,   60837,  8, }, // America/St_Lucia
    {  134,  855,   60845,  9, }, // America/St_Thomas
    {  134, 5913,   60854, 10, }, // America/St_Vincent
    {  134, 5523,    5491,  5, }, // Asia/Aqtau
    {  134, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  134, 3049,    4501,  9, }, // Pacific/Kanton
    {  135, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/British Virgin Islands
    {  135, 2061,   60820,  9, }, // America/St_Johns
    {  135, 5863,   60829,  8, }, // America/St_Kitts
    {  135, 5880,   60837,  8, }, // America/St_Lucia
    {  135,  855,   60845,  9, }, // America/St_Thomas
    {  135, 5913,   60854, 10, }, // America/St_Vincent
    {  135, 5523,    5491,  5, }, // Asia/Aqtau
    {  135, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  135, 3049,    4501,  9, }, // Pacific/Kanton
    {  137, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Cameroon
    {  137, 2061,   60820,  9, }, // America/St_Johns
    {  137, 5863,   60829,  8, }, // America/St_Kitts
    {  137, 5880,   60837,  8, }, // America/St_Lucia
    {  137,  855,   60845,  9, }, // America/St_Thomas
    {  137, 5913,   60854, 10, }, // America/St_Vincent
    {  137, 5523,    5491,  5, }, // Asia/Aqtau
    {  137, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  137, 3049,    4501,  9, }, // Pacific/Kanton
    {  138, 8508,   26589, 16, }, // America/St_Barthelemy English/Latin/Canada
    {  138, 2061,   26605, 12, }, // America/St_Johns
    {  138, 5863,   26617, 11, }, // America/St_Kitts
    {  138, 5880,   35832, 11, }, // America/St_Lucia
    {  138,  855,   44161, 12, }, // America/St_Thomas
    {  138, 5913,   26651, 13, }, // America/St_Vincent
    {  138, 1235,   58486,  7, }, // Asia/Yangon
    {  138, 6821,   36066, 12, }, // Atlantic/St_Helena
    {  139, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Cayman Islands
    {  139, 2061,   60820,  9, }, // America/St_Johns
    {  139, 5863,   60829,  8, }, // America/St_Kitts
    {  139, 5880,   60837,  8, }, // America/St_Lucia
    {  139,  855,   60845,  9, }, // America/St_Thomas
    {  139, 5913,   60854, 10, }, // America/St_Vincent
    {  139, 5523,    5491,  5, }, // Asia/Aqtau
    {  139, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  139, 3049,    4501,  9, }, // Pacific/Kanton
    {  140, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Christmas Island
    {  140, 2061,   60820,  9, }, // America/St_Johns
    {  140, 5863,   60829,  8, }, // America/St_Kitts
    {  140, 5880,   60837,  8, }, // America/St_Lucia
    {  140,  855,   60845,  9, }, // America/St_Thomas
    {  140, 5913,   60854, 10, }, // America/St_Vincent
    {  140, 5523,    5491,  5, }, // Asia/Aqtau
    {  140, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  140, 3049,    4501,  9, }, // Pacific/Kanton
    {  141, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Cocos Islands
    {  141, 2061,   60820,  9, }, // America/St_Johns
    {  141, 5863,   60829,  8, }, // America/St_Kitts
    {  141, 5880,   60837,  8, }, // America/St_Lucia
    {  141,  855,   60845,  9, }, // America/St_Thomas
    {  141, 5913,   60854, 10, }, // America/St_Vincent
    {  141, 5523,    5491,  5, }, // Asia/Aqtau
    {  141, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  141, 3049,    4501,  9, }, // Pacific/Kanton
    {  142, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Cook Islands
    {  142, 2061,   60820,  9, }, // America/St_Johns
    {  142, 5863,   60829,  8, }, // America/St_Kitts
    {  142, 5880,   60837,  8, }, // America/St_Lucia
    {  142,  855,   60845,  9, }, // America/St_Thomas
    {  142, 5913,   60854, 10, }, // America/St_Vincent
    {  142, 5523,    5491,  5, }, // Asia/Aqtau
    {  142, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  142, 3049,    4501,  9, }, // Pacific/Kanton
    {  143, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Cyprus
    {  143, 2061,   60820,  9, }, // America/St_Johns
    {  143, 5863,   60829,  8, }, // America/St_Kitts
    {  143, 5880,   60837,  8, }, // America/St_Lucia
    {  143,  855,   60845,  9, }, // America/St_Thomas
    {  143, 5913,   60854, 10, }, // America/St_Vincent
    {  143, 5523,    5491,  5, }, // Asia/Aqtau
    {  143, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  143, 3049,    4501,  9, }, // Pacific/Kanton
    {  144, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Denmark
    {  144, 2061,   60820,  9, }, // America/St_Johns
    {  144, 5863,   60829,  8, }, // America/St_Kitts
    {  144, 5880,   60837,  8, }, // America/St_Lucia
    {  144,  855,   60845,  9, }, // America/St_Thomas
    {  144, 5913,   60854, 10, }, // America/St_Vincent
    {  144, 5523,    5491,  5, }, // Asia/Aqtau
    {  144, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  144, 3049,    4501,  9, }, // Pacific/Kanton
    {  145, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Diego Garcia
    {  145, 2061,   60820,  9, }, // America/St_Johns
    {  145, 5863,   60829,  8, }, // America/St_Kitts
    {  145, 5880,   60837,  8, }, // America/St_Lucia
    {  145,  855,   60845,  9, }, // America/St_Thomas
    {  145, 5913,   60854, 10, }, // America/St_Vincent
    {  145, 5523,    5491,  5, }, // Asia/Aqtau
    {  145, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  145, 3049,    4501,  9, }, // Pacific/Kanton
    {  146, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Dominica
    {  146, 2061,   60820,  9, }, // America/St_Johns
    {  146, 5863,   60829,  8, }, // America/St_Kitts
    {  146, 5880,   60837,  8, }, // America/St_Lucia
    {  146,  855,   60845,  9, }, // America/St_Thomas
    {  146, 5913,   60854, 10, }, // America/St_Vincent
    {  146, 5523,    5491,  5, }, // Asia/Aqtau
    {  146, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  146, 3049,    4501,  9, }, // Pacific/Kanton
    {  147, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Eritrea
    {  147, 2061,   60820,  9, }, // America/St_Johns
    {  147, 5863,   60829,  8, }, // America/St_Kitts
    {  147, 5880,   60837,  8, }, // America/St_Lucia
    {  147,  855,   60845,  9, }, // America/St_Thomas
    {  147, 5913,   60854, 10, }, // America/St_Vincent
    {  147, 5523,    5491,  5, }, // Asia/Aqtau
    {  147, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  147, 3049,    4501,  9, }, // Pacific/Kanton
    {  148, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Eswatini
    {  148, 2061,   60820,  9, }, // America/St_Johns
    {  148, 5863,   60829,  8, }, // America/St_Kitts
    {  148, 5880,   60837,  8, }, // America/St_Lucia
    {  148,  855,   60845,  9, }, // America/St_Thomas
    {  148, 5913,   60854, 10, }, // America/St_Vincent
    {  148, 5523,    5491,  5, }, // Asia/Aqtau
    {  148, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  148, 3049,    4501,  9, }, // Pacific/Kanton
    {  149, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Europe
    {  149, 2061,   60820,  9, }, // America/St_Johns
    {  149, 5863,   60829,  8, }, // America/St_Kitts
    {  149, 5880,   60837,  8, }, // America/St_Lucia
    {  149,  855,   60845,  9, }, // America/St_Thomas
    {  149, 5913,   60854, 10, }, // America/St_Vincent
    {  149, 5523,    5491,  5, }, // Asia/Aqtau
    {  149, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  149, 3049,    4501,  9, }, // Pacific/Kanton
    {  150, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Falkland Islands
    {  150, 2061,   60820,  9, }, // America/St_Johns
    {  150, 5863,   60829,  8, }, // America/St_Kitts
    {  150, 5880,   60837,  8, }, // America/St_Lucia
    {  150,  855,   60845,  9, }, // America/St_Thomas
    {  150, 5913,   60854, 10, }, // America/St_Vincent
    {  150, 5523,    5491,  5, }, // Asia/Aqtau
    {  150, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  150, 3049,    4501,  9, }, // Pacific/Kanton
    {  151, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Fiji
    {  151, 2061,   60820,  9, }, // America/St_Johns
    {  151, 5863,   60829,  8, }, // America/St_Kitts
    {  151, 5880,   60837,  8, }, // America/St_Lucia
    {  151,  855,   60845,  9, }, // America/St_Thomas
    {  151, 5913,   60854, 10, }, // America/St_Vincent
    {  151, 5523,    5491,  5, }, // Asia/Aqtau
    {  151, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  151, 3049,    4501,  9, }, // Pacific/Kanton
    {  152, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Finland
    {  152, 2061,   60820,  9, }, // America/St_Johns
    {  152, 5863,   60829,  8, }, // America/St_Kitts
    {  152, 5880,   60837,  8, }, // America/St_Lucia
    {  152,  855,   60845,  9, }, // America/St_Thomas
    {  152, 5913,   60854, 10, }, // America/St_Vincent
    {  152, 5523,    5491,  5, }, // Asia/Aqtau
    {  152, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  152, 3049,    4501,  9, }, // Pacific/Kanton
    {  153, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Gambia
    {  153, 2061,   60820,  9, }, // America/St_Johns
    {  153, 5863,   60829,  8, }, // America/St_Kitts
    {  153, 5880,   60837,  8, }, // America/St_Lucia
    {  153,  855,   60845,  9, }, // America/St_Thomas
    {  153, 5913,   60854, 10, }, // America/St_Vincent
    {  153, 5523,    5491,  5, }, // Asia/Aqtau
    {  153, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  153, 3049,    4501,  9, }, // Pacific/Kanton
    {  154, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Germany
    {  154, 2061,   60820,  9, }, // America/St_Johns
    {  154, 5863,   60829,  8, }, // America/St_Kitts
    {  154, 5880,   60837,  8, }, // America/St_Lucia
    {  154,  855,   60845,  9, }, // America/St_Thomas
    {  154, 5913,   60854, 10, }, // America/St_Vincent
    {  154, 5523,    5491,  5, }, // Asia/Aqtau
    {  154, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  154, 3049,    4501,  9, }, // Pacific/Kanton
    {  155, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Ghana
    {  155, 2061,   60820,  9, }, // America/St_Johns
    {  155, 5863,   60829,  8, }, // America/St_Kitts
    {  155, 5880,   60837,  8, }, // America/St_Lucia
    {  155,  855,   60845,  9, }, // America/St_Thomas
    {  155, 5913,   60854, 10, }, // America/St_Vincent
    {  155, 5523,    5491,  5, }, // Asia/Aqtau
    {  155, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  155, 3049,    4501,  9, }, // Pacific/Kanton
    {  156, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Gibraltar
    {  156, 2061,   60820,  9, }, // America/St_Johns
    {  156, 5863,   60829,  8, }, // America/St_Kitts
    {  156, 5880,   60837,  8, }, // America/St_Lucia
    {  156,  855,   60845,  9, }, // America/St_Thomas
    {  156, 5913,   60854, 10, }, // America/St_Vincent
    {  156, 5523,    5491,  5, }, // Asia/Aqtau
    {  156, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  156, 3049,    4501,  9, }, // Pacific/Kanton
    {  157, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Grenada
    {  157, 2061,   60820,  9, }, // America/St_Johns
    {  157, 5863,   60829,  8, }, // America/St_Kitts
    {  157, 5880,   60837,  8, }, // America/St_Lucia
    {  157,  855,   60845,  9, }, // America/St_Thomas
    {  157, 5913,   60854, 10, }, // America/St_Vincent
    {  157, 5523,    5491,  5, }, // Asia/Aqtau
    {  157, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  157, 3049,    4501,  9, }, // Pacific/Kanton
    {  159, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Guernsey
    {  159, 2061,   60820,  9, }, // America/St_Johns
    {  159, 5863,   60829,  8, }, // America/St_Kitts
    {  159, 5880,   60837,  8, }, // America/St_Lucia
    {  159,  855,   60845,  9, }, // America/St_Thomas
    {  159, 5913,   60854, 10, }, // America/St_Vincent
    {  159, 5523,    5491,  5, }, // Asia/Aqtau
    {  159, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  159, 3049,    4501,  9, }, // Pacific/Kanton
    {  160, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Guyana
    {  160, 2061,   60820,  9, }, // America/St_Johns
    {  160, 5863,   60829,  8, }, // America/St_Kitts
    {  160, 5880,   60837,  8, }, // America/St_Lucia
    {  160,  855,   60845,  9, }, // America/St_Thomas
    {  160, 5913,   60854, 10, }, // America/St_Vincent
    {  160, 5523,    5491,  5, }, // Asia/Aqtau
    {  160, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  160, 3049,    4501,  9, }, // Pacific/Kanton
    {  161, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Hong Kong
    {  161, 2061,   60820,  9, }, // America/St_Johns
    {  161, 5863,   60829,  8, }, // America/St_Kitts
    {  161, 5880,   60837,  8, }, // America/St_Lucia
    {  161,  855,   60845,  9, }, // America/St_Thomas
    {  161, 5913,   60854, 10, }, // America/St_Vincent
    {  161, 5523,    5491,  5, }, // Asia/Aqtau
    {  161, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  161, 3049,    4501,  9, }, // Pacific/Kanton
    {  162, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/India
    {  162, 2061,   60820,  9, }, // America/St_Johns
    {  162, 5863,   60829,  8, }, // America/St_Kitts
    {  162, 5880,   60837,  8, }, // America/St_Lucia
    {  162,  855,   60845,  9, }, // America/St_Thomas
    {  162, 5913,   60854, 10, }, // America/St_Vincent
    {  162, 5523,    5491,  5, }, // Asia/Aqtau
    {  162, 4771,   26703,  5, }, // Asia/Hovd
    {  162, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  162, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  162, 3049,    4501,  9, }, // Pacific/Kanton
    {  163, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Indonesia
    {  163, 2061,   60820,  9, }, // America/St_Johns
    {  163, 5863,   60829,  8, }, // America/St_Kitts
    {  163, 5880,   60837,  8, }, // America/St_Lucia
    {  163,  855,   60845,  9, }, // America/St_Thomas
    {  163, 5913,   60854, 10, }, // America/St_Vincent
    {  163, 5523,    5491,  5, }, // Asia/Aqtau
    {  163, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  163, 3049,    4501,  9, }, // Pacific/Kanton
    {  164, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Ireland
    {  164, 2061,   60820,  9, }, // America/St_Johns
    {  164, 5863,   60829,  8, }, // America/St_Kitts
    {  164, 5880,   60837,  8, }, // America/St_Lucia
    {  164,  855,   60845,  9, }, // America/St_Thomas
    {  164, 5913,   60854, 10, }, // America/St_Vincent
    {  164, 5523,    5491,  5, }, // Asia/Aqtau
    {  164, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  164, 3049,    4501,  9, }, // Pacific/Kanton
    {  165, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Isle of Man
    {  165, 2061,   60820,  9, }, // America/St_Johns
    {  165, 5863,   60829,  8, }, // America/St_Kitts
    {  165, 5880,   60837,  8, }, // America/St_Lucia
    {  165,  855,   60845,  9, }, // America/St_Thomas
    {  165, 5913,   60854, 10, }, // America/St_Vincent
    {  165, 5523,    5491,  5, }, // Asia/Aqtau
    {  165, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  165, 3049,    4501,  9, }, // Pacific/Kanton
    {  166, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Israel
    {  166, 2061,   60820,  9, }, // America/St_Johns
    {  166, 5863,   60829,  8, }, // America/St_Kitts
    {  166, 5880,   60837,  8, }, // America/St_Lucia
    {  166,  855,   60845,  9, }, // America/St_Thomas
    {  166, 5913,   60854, 10, }, // America/St_Vincent
    {  166, 5523,    5491,  5, }, // Asia/Aqtau
    {  166, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  166, 3049,    4501,  9, }, // Pacific/Kanton
    {  167, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Jamaica
    {  167, 2061,   60820,  9, }, // America/St_Johns
    {  167, 5863,   60829,  8, }, // America/St_Kitts
    {  167, 5880,   60837,  8, }, // America/St_Lucia
    {  167,  855,   60845,  9, }, // America/St_Thomas
    {  167, 5913,   60854, 10, }, // America/St_Vincent
    {  167, 5523,    5491,  5, }, // Asia/Aqtau
    {  167, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  167, 3049,    4501,  9, }, // Pacific/Kanton
    {  168, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Jersey
    {  168, 2061,   60820,  9, }, // America/St_Johns
    {  168, 5863,   60829,  8, }, // America/St_Kitts
    {  168, 5880,   60837,  8, }, // America/St_Lucia
    {  168,  855,   60845,  9, }, // America/St_Thomas
    {  168, 5913,   60854, 10, }, // America/St_Vincent
    {  168, 5523,    5491,  5, }, // Asia/Aqtau
    {  168, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  168, 3049,    4501,  9, }, // Pacific/Kanton
    {  169, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Kenya
    {  169, 2061,   60820,  9, }, // America/St_Johns
    {  169, 5863,   60829,  8, }, // America/St_Kitts
    {  169, 5880,   60837,  8, }, // America/St_Lucia
    {  169,  855,   60845,  9, }, // America/St_Thomas
    {  169, 5913,   60854, 10, }, // America/St_Vincent
    {  169, 5523,    5491,  5, }, // Asia/Aqtau
    {  169, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  169, 3049,    4501,  9, }, // Pacific/Kanton
    {  170, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Kiribati
    {  170, 2061,   60820,  9, }, // America/St_Johns
    {  170, 5863,   60829,  8, }, // America/St_Kitts
    {  170, 5880,   60837,  8, }, // America/St_Lucia
    {  170,  855,   60845,  9, }, // America/St_Thomas
    {  170, 5913,   60854, 10, }, // America/St_Vincent
    {  170, 5523,    5491,  5, }, // Asia/Aqtau
    {  170, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  170, 3049,    4501,  9, }, // Pacific/Kanton
    {  171, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Lesotho
    {  171, 2061,   60820,  9, }, // America/St_Johns
    {  171, 5863,   60829,  8, }, // America/St_Kitts
    {  171, 5880,   60837,  8, }, // America/St_Lucia
    {  171,  855,   60845,  9, }, // America/St_Thomas
    {  171, 5913,   60854, 10, }, // America/St_Vincent
    {  171, 5523,    5491,  5, }, // Asia/Aqtau
    {  171, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  171, 3049,    4501,  9, }, // Pacific/Kanton
    {  172, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Liberia
    {  172, 2061,   60820,  9, }, // America/St_Johns
    {  172, 5863,   60829,  8, }, // America/St_Kitts
    {  172, 5880,   60837,  8, }, // America/St_Lucia
    {  172,  855,   60845,  9, }, // America/St_Thomas
    {  172, 5913,   60854, 10, }, // America/St_Vincent
    {  172, 5523,    5491,  5, }, // Asia/Aqtau
    {  172, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  172, 3049,    4501,  9, }, // Pacific/Kanton
    {  173, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Macao
    {  173, 2061,   60820,  9, }, // America/St_Johns
    {  173, 5863,   60829,  8, }, // America/St_Kitts
    {  173, 5880,   60837,  8, }, // America/St_Lucia
    {  173,  855,   60845,  9, }, // America/St_Thomas
    {  173, 5913,   60854, 10, }, // America/St_Vincent
    {  173, 5523,    5491,  5, }, // Asia/Aqtau
    {  173, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  173, 3049,    4501,  9, }, // Pacific/Kanton
    {  174, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Madagascar
    {  174, 2061,   60820,  9, }, // America/St_Johns
    {  174, 5863,   60829,  8, }, // America/St_Kitts
    {  174, 5880,   60837,  8, }, // America/St_Lucia
    {  174,  855,   60845,  9, }, // America/St_Thomas
    {  174, 5913,   60854, 10, }, // America/St_Vincent
    {  174, 5523,    5491,  5, }, // Asia/Aqtau
    {  174, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  174, 3049,    4501,  9, }, // Pacific/Kanton
    {  175, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Malawi
    {  175, 2061,   60820,  9, }, // America/St_Johns
    {  175, 5863,   60829,  8, }, // America/St_Kitts
    {  175, 5880,   60837,  8, }, // America/St_Lucia
    {  175,  855,   60845,  9, }, // America/St_Thomas
    {  175, 5913,   60854, 10, }, // America/St_Vincent
    {  175, 5523,    5491,  5, }, // Asia/Aqtau
    {  175, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  175, 3049,    4501,  9, }, // Pacific/Kanton
    {  176, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Malaysia
    {  176, 2061,   60820,  9, }, // America/St_Johns
    {  176, 5863,   60829,  8, }, // America/St_Kitts
    {  176, 5880,   60837,  8, }, // America/St_Lucia
    {  176,  855,   60845,  9, }, // America/St_Thomas
    {  176, 5913,   60854, 10, }, // America/St_Vincent
    {  176, 5523,    5491,  5, }, // Asia/Aqtau
    {  176, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  176, 3049,    4501,  9, }, // Pacific/Kanton
    {  177, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Maldives
    {  177, 2061,   60820,  9, }, // America/St_Johns
    {  177, 5863,   60829,  8, }, // America/St_Kitts
    {  177, 5880,   60837,  8, }, // America/St_Lucia
    {  177,  855,   60845,  9, }, // America/St_Thomas
    {  177, 5913,   60854, 10, }, // America/St_Vincent
    {  177, 5523,    5491,  5, }, // Asia/Aqtau
    {  177, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  177, 3049,    4501,  9, }, // Pacific/Kanton
    {  178, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Malta
    {  178, 2061,   60820,  9, }, // America/St_Johns
    {  178, 5863,   60829,  8, }, // America/St_Kitts
    {  178, 5880,   60837,  8, }, // America/St_Lucia
    {  178,  855,   60845,  9, }, // America/St_Thomas
    {  178, 5913,   60854, 10, }, // America/St_Vincent
    {  178, 5523,    5491,  5, }, // Asia/Aqtau
    {  178, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  178, 3049,    4501,  9, }, // Pacific/Kanton
    {  180, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Mauritius
    {  180, 2061,   60820,  9, }, // America/St_Johns
    {  180, 5863,   60829,  8, }, // America/St_Kitts
    {  180, 5880,   60837,  8, }, // America/St_Lucia
    {  180,  855,   60845,  9, }, // America/St_Thomas
    {  180, 5913,   60854, 10, }, // America/St_Vincent
    {  180, 5523,    5491,  5, }, // Asia/Aqtau
    {  180, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  180, 3049,    4501,  9, }, // Pacific/Kanton
    {  181, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Micronesia
    {  181, 2061,   60820,  9, }, // America/St_Johns
    {  181, 5863,   60829,  8, }, // America/St_Kitts
    {  181, 5880,   60837,  8, }, // America/St_Lucia
    {  181,  855,   60845,  9, }, // America/St_Thomas
    {  181, 5913,   60854, 10, }, // America/St_Vincent
    {  181, 5523,    5491,  5, }, // Asia/Aqtau
    {  181, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  181, 3049,    4501,  9, }, // Pacific/Kanton
    {  182, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Montserrat
    {  182, 2061,   60820,  9, }, // America/St_Johns
    {  182, 5863,   60829,  8, }, // America/St_Kitts
    {  182, 5880,   60837,  8, }, // America/St_Lucia
    {  182,  855,   60845,  9, }, // America/St_Thomas
    {  182, 5913,   60854, 10, }, // America/St_Vincent
    {  182, 5523,    5491,  5, }, // Asia/Aqtau
    {  182, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  182, 3049,    4501,  9, }, // Pacific/Kanton
    {  183, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Namibia
    {  183, 2061,   60820,  9, }, // America/St_Johns
    {  183, 5863,   60829,  8, }, // America/St_Kitts
    {  183, 5880,   60837,  8, }, // America/St_Lucia
    {  183,  855,   60845,  9, }, // America/St_Thomas
    {  183, 5913,   60854, 10, }, // America/St_Vincent
    {  183, 5523,    5491,  5, }, // Asia/Aqtau
    {  183, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  183, 3049,    4501,  9, }, // Pacific/Kanton
    {  184, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Nauru
    {  184, 2061,   60820,  9, }, // America/St_Johns
    {  184, 5863,   60829,  8, }, // America/St_Kitts
    {  184, 5880,   60837,  8, }, // America/St_Lucia
    {  184,  855,   60845,  9, }, // America/St_Thomas
    {  184, 5913,   60854, 10, }, // America/St_Vincent
    {  184, 5523,    5491,  5, }, // Asia/Aqtau
    {  184, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  184, 3049,    4501,  9, }, // Pacific/Kanton
    {  185, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Netherlands
    {  185, 2061,   60820,  9, }, // America/St_Johns
    {  185, 5863,   60829,  8, }, // America/St_Kitts
    {  185, 5880,   60837,  8, }, // America/St_Lucia
    {  185,  855,   60845,  9, }, // America/St_Thomas
    {  185, 5913,   60854, 10, }, // America/St_Vincent
    {  185, 5523,    5491,  5, }, // Asia/Aqtau
    {  185, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  185, 3049,    4501,  9, }, // Pacific/Kanton
    {  186, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/New Zealand
    {  186, 2061,   60820,  9, }, // America/St_Johns
    {  186, 5863,   60829,  8, }, // America/St_Kitts
    {  186, 5880,   60837,  8, }, // America/St_Lucia
    {  186,  855,   60845,  9, }, // America/St_Thomas
    {  186, 5913,   60854, 10, }, // America/St_Vincent
    {  186, 5523,    5491,  5, }, // Asia/Aqtau
    {  186, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  186, 3049,    4501,  9, }, // Pacific/Kanton
    {  187, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Nigeria
    {  187, 2061,   60820,  9, }, // America/St_Johns
    {  187, 5863,   60829,  8, }, // America/St_Kitts
    {  187, 5880,   60837,  8, }, // America/St_Lucia
    {  187,  855,   60845,  9, }, // America/St_Thomas
    {  187, 5913,   60854, 10, }, // America/St_Vincent
    {  187, 5523,    5491,  5, }, // Asia/Aqtau
    {  187, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  187, 3049,    4501,  9, }, // Pacific/Kanton
    {  188, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Niue
    {  188, 2061,   60820,  9, }, // America/St_Johns
    {  188, 5863,   60829,  8, }, // America/St_Kitts
    {  188, 5880,   60837,  8, }, // America/St_Lucia
    {  188,  855,   60845,  9, }, // America/St_Thomas
    {  188, 5913,   60854, 10, }, // America/St_Vincent
    {  188, 5523,    5491,  5, }, // Asia/Aqtau
    {  188, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  188, 3049,    4501,  9, }, // Pacific/Kanton
    {  189, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Norfolk Island
    {  189, 2061,   60820,  9, }, // America/St_Johns
    {  189, 5863,   60829,  8, }, // America/St_Kitts
    {  189, 5880,   60837,  8, }, // America/St_Lucia
    {  189,  855,   60845,  9, }, // America/St_Thomas
    {  189, 5913,   60854, 10, }, // America/St_Vincent
    {  189, 5523,    5491,  5, }, // Asia/Aqtau
    {  189, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  189, 3049,    4501,  9, }, // Pacific/Kanton
    {  191, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Pakistan
    {  191, 2061,   60820,  9, }, // America/St_Johns
    {  191, 5863,   60829,  8, }, // America/St_Kitts
    {  191, 5880,   60837,  8, }, // America/St_Lucia
    {  191,  855,   60845,  9, }, // America/St_Thomas
    {  191, 5913,   60854, 10, }, // America/St_Vincent
    {  191, 5523,    5491,  5, }, // Asia/Aqtau
    {  191, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  191, 3049,    4501,  9, }, // Pacific/Kanton
    {  192, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Palau
    {  192, 2061,   60820,  9, }, // America/St_Johns
    {  192, 5863,   60829,  8, }, // America/St_Kitts
    {  192, 5880,   60837,  8, }, // America/St_Lucia
    {  192,  855,   60845,  9, }, // America/St_Thomas
    {  192, 5913,   60854, 10, }, // America/St_Vincent
    {  192, 5523,    5491,  5, }, // Asia/Aqtau
    {  192, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  192, 3049,    4501,  9, }, // Pacific/Kanton
    {  193, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Papua New Guinea
    {  193, 2061,   60820,  9, }, // America/St_Johns
    {  193, 5863,   60829,  8, }, // America/St_Kitts
    {  193, 5880,   60837,  8, }, // America/St_Lucia
    {  193,  855,   60845,  9, }, // America/St_Thomas
    {  193, 5913,   60854, 10, }, // America/St_Vincent
    {  193, 5523,    5491,  5, }, // Asia/Aqtau
    {  193, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  193, 3049,    4501,  9, }, // Pacific/Kanton
    {  195, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Pitcairn
    {  195, 2061,   60820,  9, }, // America/St_Johns
    {  195, 5863,   60829,  8, }, // America/St_Kitts
    {  195, 5880,   60837,  8, }, // America/St_Lucia
    {  195,  855,   60845,  9, }, // America/St_Thomas
    {  195, 5913,   60854, 10, }, // America/St_Vincent
    {  195, 5523,    5491,  5, }, // Asia/Aqtau
    {  195, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  195, 3049,    4501,  9, }, // Pacific/Kanton
    {  197, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Rwanda
    {  197, 2061,   60820,  9, }, // America/St_Johns
    {  197, 5863,   60829,  8, }, // America/St_Kitts
    {  197, 5880,   60837,  8, }, // America/St_Lucia
    {  197,  855,   60845,  9, }, // America/St_Thomas
    {  197, 5913,   60854, 10, }, // America/St_Vincent
    {  197, 5523,    5491,  5, }, // Asia/Aqtau
    {  197, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  197, 3049,    4501,  9, }, // Pacific/Kanton
    {  198, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Saint Helena
    {  198, 2061,   60820,  9, }, // America/St_Johns
    {  198, 5863,   60829,  8, }, // America/St_Kitts
    {  198, 5880,   60837,  8, }, // America/St_Lucia
    {  198,  855,   60845,  9, }, // America/St_Thomas
    {  198, 5913,   60854, 10, }, // America/St_Vincent
    {  198, 5523,    5491,  5, }, // Asia/Aqtau
    {  198, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  198, 3049,    4501,  9, }, // Pacific/Kanton
    {  199, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Saint Kitts and Nevis
    {  199, 2061,   60820,  9, }, // America/St_Johns
    {  199, 5863,   60829,  8, }, // America/St_Kitts
    {  199, 5880,   60837,  8, }, // America/St_Lucia
    {  199,  855,   60845,  9, }, // America/St_Thomas
    {  199, 5913,   60854, 10, }, // America/St_Vincent
    {  199, 5523,    5491,  5, }, // Asia/Aqtau
    {  199, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  199, 3049,    4501,  9, }, // Pacific/Kanton
    {  200, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Saint Lucia
    {  200, 2061,   60820,  9, }, // America/St_Johns
    {  200, 5863,   60829,  8, }, // America/St_Kitts
    {  200, 5880,   60837,  8, }, // America/St_Lucia
    {  200,  855,   60845,  9, }, // America/St_Thomas
    {  200, 5913,   60854, 10, }, // America/St_Vincent
    {  200, 5523,    5491,  5, }, // Asia/Aqtau
    {  200, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  200, 3049,    4501,  9, }, // Pacific/Kanton
    {  201, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Saint Vincent and Grenadines
    {  201, 2061,   60820,  9, }, // America/St_Johns
    {  201, 5863,   60829,  8, }, // America/St_Kitts
    {  201, 5880,   60837,  8, }, // America/St_Lucia
    {  201,  855,   60845,  9, }, // America/St_Thomas
    {  201, 5913,   60854, 10, }, // America/St_Vincent
    {  201, 5523,    5491,  5, }, // Asia/Aqtau
    {  201, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  201, 3049,    4501,  9, }, // Pacific/Kanton
    {  202, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Samoa
    {  202, 2061,   60820,  9, }, // America/St_Johns
    {  202, 5863,   60829,  8, }, // America/St_Kitts
    {  202, 5880,   60837,  8, }, // America/St_Lucia
    {  202,  855,   60845,  9, }, // America/St_Thomas
    {  202, 5913,   60854, 10, }, // America/St_Vincent
    {  202, 5523,    5491,  5, }, // Asia/Aqtau
    {  202, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  202, 3049,    4501,  9, }, // Pacific/Kanton
    {  203, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Seychelles
    {  203, 2061,   60820,  9, }, // America/St_Johns
    {  203, 5863,   60829,  8, }, // America/St_Kitts
    {  203, 5880,   60837,  8, }, // America/St_Lucia
    {  203,  855,   60845,  9, }, // America/St_Thomas
    {  203, 5913,   60854, 10, }, // America/St_Vincent
    {  203, 5523,    5491,  5, }, // Asia/Aqtau
    {  203, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  203, 3049,    4501,  9, }, // Pacific/Kanton
    {  204, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Sierra Leone
    {  204, 2061,   60820,  9, }, // America/St_Johns
    {  204, 5863,   60829,  8, }, // America/St_Kitts
    {  204, 5880,   60837,  8, }, // America/St_Lucia
    {  204,  855,   60845,  9, }, // America/St_Thomas
    {  204, 5913,   60854, 10, }, // America/St_Vincent
    {  204, 5523,    5491,  5, }, // Asia/Aqtau
    {  204, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  204, 3049,    4501,  9, }, // Pacific/Kanton
    {  205, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Singapore
    {  205, 2061,   60820,  9, }, // America/St_Johns
    {  205, 5863,   60829,  8, }, // America/St_Kitts
    {  205, 5880,   60837,  8, }, // America/St_Lucia
    {  205,  855,   60845,  9, }, // America/St_Thomas
    {  205, 5913,   60854, 10, }, // America/St_Vincent
    {  205, 5523,    5491,  5, }, // Asia/Aqtau
    {  205, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  205, 3049,    4501,  9, }, // Pacific/Kanton
    {  206, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Sint Maarten
    {  206, 2061,   60820,  9, }, // America/St_Johns
    {  206, 5863,   60829,  8, }, // America/St_Kitts
    {  206, 5880,   60837,  8, }, // America/St_Lucia
    {  206,  855,   60845,  9, }, // America/St_Thomas
    {  206, 5913,   60854, 10, }, // America/St_Vincent
    {  206, 5523,    5491,  5, }, // Asia/Aqtau
    {  206, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  206, 3049,    4501,  9, }, // Pacific/Kanton
    {  207, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Slovenia
    {  207, 2061,   60820,  9, }, // America/St_Johns
    {  207, 5863,   60829,  8, }, // America/St_Kitts
    {  207, 5880,   60837,  8, }, // America/St_Lucia
    {  207,  855,   60845,  9, }, // America/St_Thomas
    {  207, 5913,   60854, 10, }, // America/St_Vincent
    {  207, 5523,    5491,  5, }, // Asia/Aqtau
    {  207, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  207, 3049,    4501,  9, }, // Pacific/Kanton
    {  208, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Solomon Islands
    {  208, 2061,   60820,  9, }, // America/St_Johns
    {  208, 5863,   60829,  8, }, // America/St_Kitts
    {  208, 5880,   60837,  8, }, // America/St_Lucia
    {  208,  855,   60845,  9, }, // America/St_Thomas
    {  208, 5913,   60854, 10, }, // America/St_Vincent
    {  208, 5523,    5491,  5, }, // Asia/Aqtau
    {  208, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  208, 3049,    4501,  9, }, // Pacific/Kanton
    {  209, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/South Africa
    {  209, 2061,   60820,  9, }, // America/St_Johns
    {  209, 5863,   60829,  8, }, // America/St_Kitts
    {  209, 5880,   60837,  8, }, // America/St_Lucia
    {  209,  855,   60845,  9, }, // America/St_Thomas
    {  209, 5913,   60854, 10, }, // America/St_Vincent
    {  209, 5523,    5491,  5, }, // Asia/Aqtau
    {  209, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  209, 3049,    4501,  9, }, // Pacific/Kanton
    {  210, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/South Sudan
    {  210, 2061,   60820,  9, }, // America/St_Johns
    {  210, 5863,   60829,  8, }, // America/St_Kitts
    {  210, 5880,   60837,  8, }, // America/St_Lucia
    {  210,  855,   60845,  9, }, // America/St_Thomas
    {  210, 5913,   60854, 10, }, // America/St_Vincent
    {  210, 5523,    5491,  5, }, // Asia/Aqtau
    {  210, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  210, 3049,    4501,  9, }, // Pacific/Kanton
    {  211, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Sudan
    {  211, 2061,   60820,  9, }, // America/St_Johns
    {  211, 5863,   60829,  8, }, // America/St_Kitts
    {  211, 5880,   60837,  8, }, // America/St_Lucia
    {  211,  855,   60845,  9, }, // America/St_Thomas
    {  211, 5913,   60854, 10, }, // America/St_Vincent
    {  211, 5523,    5491,  5, }, // Asia/Aqtau
    {  211, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  211, 3049,    4501,  9, }, // Pacific/Kanton
    {  212, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Sweden
    {  212, 2061,   60820,  9, }, // America/St_Johns
    {  212, 5863,   60829,  8, }, // America/St_Kitts
    {  212, 5880,   60837,  8, }, // America/St_Lucia
    {  212,  855,   60845,  9, }, // America/St_Thomas
    {  212, 5913,   60854, 10, }, // America/St_Vincent
    {  212, 5523,    5491,  5, }, // Asia/Aqtau
    {  212, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  212, 3049,    4501,  9, }, // Pacific/Kanton
    {  213, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Switzerland
    {  213, 2061,   60820,  9, }, // America/St_Johns
    {  213, 5863,   60829,  8, }, // America/St_Kitts
    {  213, 5880,   60837,  8, }, // America/St_Lucia
    {  213,  855,   60845,  9, }, // America/St_Thomas
    {  213, 5913,   60854, 10, }, // America/St_Vincent
    {  213, 5523,    5491,  5, }, // Asia/Aqtau
    {  213, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  213, 3049,    4501,  9, }, // Pacific/Kanton
    {  214, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Tanzania
    {  214, 2061,   60820,  9, }, // America/St_Johns
    {  214, 5863,   60829,  8, }, // America/St_Kitts
    {  214, 5880,   60837,  8, }, // America/St_Lucia
    {  214,  855,   60845,  9, }, // America/St_Thomas
    {  214, 5913,   60854, 10, }, // America/St_Vincent
    {  214, 5523,    5491,  5, }, // Asia/Aqtau
    {  214, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  214, 3049,    4501,  9, }, // Pacific/Kanton
    {  215, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Tokelau
    {  215, 2061,   60820,  9, }, // America/St_Johns
    {  215, 5863,   60829,  8, }, // America/St_Kitts
    {  215, 5880,   60837,  8, }, // America/St_Lucia
    {  215,  855,   60845,  9, }, // America/St_Thomas
    {  215, 5913,   60854, 10, }, // America/St_Vincent
    {  215, 5523,    5491,  5, }, // Asia/Aqtau
    {  215, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  215, 3049,    4501,  9, }, // Pacific/Kanton
    {  216, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Tonga
    {  216, 2061,   60820,  9, }, // America/St_Johns
    {  216, 5863,   60829,  8, }, // America/St_Kitts
    {  216, 5880,   60837,  8, }, // America/St_Lucia
    {  216,  855,   60845,  9, }, // America/St_Thomas
    {  216, 5913,   60854, 10, }, // America/St_Vincent
    {  216, 5523,    5491,  5, }, // Asia/Aqtau
    {  216, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  216, 3049,    4501,  9, }, // Pacific/Kanton
    {  217, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Trinidad and Tobago
    {  217, 2061,   60820,  9, }, // America/St_Johns
    {  217, 5863,   60829,  8, }, // America/St_Kitts
    {  217, 5880,   60837,  8, }, // America/St_Lucia
    {  217,  855,   60845,  9, }, // America/St_Thomas
    {  217, 5913,   60854, 10, }, // America/St_Vincent
    {  217, 5523,    5491,  5, }, // Asia/Aqtau
    {  217, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  217, 3049,    4501,  9, }, // Pacific/Kanton
    {  218, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Turks and Caicos Islands
    {  218, 2061,   60820,  9, }, // America/St_Johns
    {  218, 5863,   60829,  8, }, // America/St_Kitts
    {  218, 5880,   60837,  8, }, // America/St_Lucia
    {  218,  855,   60845,  9, }, // America/St_Thomas
    {  218, 5913,   60854, 10, }, // America/St_Vincent
    {  218, 5523,    5491,  5, }, // Asia/Aqtau
    {  218, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  218, 3049,    4501,  9, }, // Pacific/Kanton
    {  219, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Tuvalu
    {  219, 2061,   60820,  9, }, // America/St_Johns
    {  219, 5863,   60829,  8, }, // America/St_Kitts
    {  219, 5880,   60837,  8, }, // America/St_Lucia
    {  219,  855,   60845,  9, }, // America/St_Thomas
    {  219, 5913,   60854, 10, }, // America/St_Vincent
    {  219, 5523,    5491,  5, }, // Asia/Aqtau
    {  219, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  219, 3049,    4501,  9, }, // Pacific/Kanton
    {  220, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Uganda
    {  220, 2061,   60820,  9, }, // America/St_Johns
    {  220, 5863,   60829,  8, }, // America/St_Kitts
    {  220, 5880,   60837,  8, }, // America/St_Lucia
    {  220,  855,   60845,  9, }, // America/St_Thomas
    {  220, 5913,   60854, 10, }, // America/St_Vincent
    {  220, 5523,    5491,  5, }, // Asia/Aqtau
    {  220, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  220, 3049,    4501,  9, }, // Pacific/Kanton
    {  222, 7623,    3825, 14, }, // America/Bahia_Banderas English/Latin/United Kingdom
    {  222, 3868,   33182,  6, }, // America/Cancun
    {  222, 8220,    7008,  6, }, // America/Merida
    {  222, 8508,   60807, 13, }, // America/St_Barthelemy
    {  222, 2061,   60820,  9, }, // America/St_Johns
    {  222, 5863,   60829,  8, }, // America/St_Kitts
    {  222, 5880,   60837,  8, }, // America/St_Lucia
    {  222,  855,   60845,  9, }, // America/St_Thomas
    {  222, 5913,   60854, 10, }, // America/St_Vincent
    {  222, 5523,    5491,  5, }, // Asia/Aqtau
    {  222, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  222, 3049,    4501,  9, }, // Pacific/Kanton
    {  225, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Vanuatu
    {  225, 2061,   60820,  9, }, // America/St_Johns
    {  225, 5863,   60829,  8, }, // America/St_Kitts
    {  225, 5880,   60837,  8, }, // America/St_Lucia
    {  225,  855,   60845,  9, }, // America/St_Thomas
    {  225, 5913,   60854, 10, }, // America/St_Vincent
    {  225, 5523,    5491,  5, }, // Asia/Aqtau
    {  225, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  225, 3049,    4501,  9, }, // Pacific/Kanton
    {  226, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/world
    {  226, 2061,   60820,  9, }, // America/St_Johns
    {  226, 5863,   60829,  8, }, // America/St_Kitts
    {  226, 5880,   60837,  8, }, // America/St_Lucia
    {  226,  855,   60845,  9, }, // America/St_Thomas
    {  226, 5913,   60854, 10, }, // America/St_Vincent
    {  226, 5523,    5491,  5, }, // Asia/Aqtau
    {  226, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  226, 3049,    4501,  9, }, // Pacific/Kanton
    {  227, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Zambia
    {  227, 2061,   60820,  9, }, // America/St_Johns
    {  227, 5863,   60829,  8, }, // America/St_Kitts
    {  227, 5880,   60837,  8, }, // America/St_Lucia
    {  227,  855,   60845,  9, }, // America/St_Thomas
    {  227, 5913,   60854, 10, }, // America/St_Vincent
    {  227, 5523,    5491,  5, }, // Asia/Aqtau
    {  227, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  227, 3049,    4501,  9, }, // Pacific/Kanton
    {  228, 8508,   60807, 13, }, // America/St_Barthelemy English/Latin/Zimbabwe
    {  228, 2061,   60820,  9, }, // America/St_Johns
    {  228, 5863,   60829,  8, }, // America/St_Kitts
    {  228, 5880,   60837,  8, }, // America/St_Lucia
    {  228,  855,   60845,  9, }, // America/St_Thomas
    {  228, 5913,   60854, 10, }, // America/St_Vincent
    {  228, 5523,    5491,  5, }, // Asia/Aqtau
    {  228, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  228, 3049,    4501,  9, }, // Pacific/Kanton
    {  229, 1259,     397, 11, }, // Asia/Ho_Chi_Minh English/Shavian/United Kingdom
    {  230,   14,       0,  6, }, // Africa/Asmara Erzya/Cyrillic/Russia
    {  230, 4435,       6,  8, }, // Africa/Sao_Tome
    {  230, 4252,      14,  8, }, // America/Asuncion
    {  230,  237,      22,  8, }, // America/Atikokan
    {  230, 7623,      30, 17, }, // America/Bahia_Banderas
    {  230, 7699,   36399,  5, }, // America/Boise
    {  230, 3868,      47,  6, }, // America/Cancun
    {  230, 2260,   60873,  6, }, // America/Chicago
    {  230, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  230, 5723,      66,  7, }, // America/Curacao
    {  230,  805,    1201,  6, }, // America/Denver
    {  230, 5770,    1292,  7, }, // America/Grenada
    {  230,  481,      73, 13, }, // America/Indiana/Knox
    {  230, 7961,      86, 16, }, // America/Indiana/Marengo
    {  230, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  230, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  230, 8038,     139, 14, }, // America/Indiana/Vevay
    {  230, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  230, 8086,     171, 16, }, // America/Indiana/Winamac
    {  230, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  230, 5932,     207, 22, }, // America/Lower_Princes
    {  230, 8220,     229,  6, }, // America/Merida
    {  230, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  230, 8329,     255, 20, }, // America/North_Dakota/Center
    {  230, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  230,  393,     298,  4, }, // America/Nuuk
    {  230, 2356,    1783,  6, }, // America/Panama
    {  230, 6899,     302, 16, }, // America/Scoresbysund
    {  230, 8508,     318, 14, }, // America/St_Barthelemy
    {  230, 2061,     332, 10, }, // America/St_Johns
    {  230, 5863,     342,  9, }, // America/St_Kitts
    {  230, 5880,     351,  9, }, // America/St_Lucia
    {  230,  855,     360, 10, }, // America/St_Thomas
    {  230, 5913,     370, 11, }, // America/St_Vincent
    {  230, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  230, 1429,   60879, 11, }, // Arctic/Longyearbyen
    {  230, 3607,    2237,  6, }, // Asia/Baghdad
    {  230, 3653,    2250,  4, }, // Asia/Baku
    {  230, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  230, 2704,   51902,  7, }, // Asia/Hong_Kong
    {  230, 1185,     413,  9, }, // Asia/Kathmandu
    {  230,  992,     422,  7, }, // Asia/Kolkata
    {  230, 1211,     429,  5, }, // Asia/Macau
    {  230, 4229,   60890,  6, }, // Asia/Omsk
    {  230, 4408,    2614,  7, }, // Asia/Sakhalin
    {  230, 7245,    2621,  9, }, // Asia/Samarkand
    {  230, 3279,    2630,  4, }, // Asia/Seoul
    {  230, 1053,   28801,  6, }, // Asia/Shanghai
    {  230, 3263,   60896,  6, }, // Asia/Taipei
    {  230, 4781,    2667,  7, }, // Asia/Tashkent
    {  230, 2821,    2692,  5, }, // Asia/Tokyo
    {  230, 4572,   60902,  7, }, // Asia/Tomsk
    {  230, 1021,   60909, 10, }, // Asia/Ulaanbaatar
    {  230, 1235,     442,  6, }, // Asia/Yangon
    {  230, 5671,   35152,  7, }, // Atlantic/Bermuda
    {  230, 1395,     448,  5, }, // Atlantic/Faroe
    {  230, 8736,   37132,  7, }, // Atlantic/Madeira
    {  230, 2748,   52082,  9, }, // Atlantic/Reykjavik
    {  230, 6821,     453, 10, }, // Atlantic/St_Helena
    {  230, 6441,    2993,  9, }, // Europe/Amsterdam
    {  230, 6216,    3002,  7, }, // Europe/Andorra
    {  230, 2300,   60919,  5, }, // Europe/Athens
    {  230, 6336,    3021,  7, }, // Europe/Belgrade
    {  230, 4757,    3028,  6, }, // Europe/Berlin
    {  230, 6488,    3034, 10, }, // Europe/Bratislava
    {  230, 2167,   52164,  8, }, // Europe/Brussels
    {  230, 3928,    3052,  8, }, // Europe/Bucharest
    {  230, 3806,    3060,  8, }, // Europe/Budapest
    {  230, 6289,   52191, 10, }, // Europe/Copenhagen
    {  230, 2338,    3108,  6, }, // Europe/Dublin
    {  230, 6612,   52201,  9, }, // Europe/Helsinki
    {  230, 8929,    3169,  5, }, // Europe/Kirov
    {  230, 2527,    3174,  4, }, // Europe/Kyiv
    {  230, 3213,    3178,  8, }, // Europe/Lisbon
    {  230, 6506,   29300,  7, }, // Europe/Ljubljana
    {  230, 2501,    3195,  6, }, // Europe/London
    {  230, 6365,   29313, 10, }, // Europe/Luxembourg
    {  230, 4892,    3212,  6, }, // Europe/Madrid
    {  230, 6397,   29329,  6, }, // Europe/Malta
    {  230, 3693,   60924,  7, }, // Europe/Minsk
    {  230, 6410,    3237,  6, }, // Europe/Monaco
    {  230, 3537,   60931,  6, }, // Europe/Moscow
    {  230, 6458,    3249,  4, }, // Europe/Oslo
    {  230, 4284,    3253,  5, }, // Europe/Paris
    {  230, 6424,    3258,  9, }, // Europe/Podgorica
    {  230, 6275,    3267,  5, }, // Europe/Prague
    {  230, 8837,    3272,  4, }, // Europe/Riga
    {  230, 6324,    3276,  3, }, // Europe/Rome
    {  230, 4331,    3279,  6, }, // Europe/Samara
    {  230, 6470,    3285, 10, }, // Europe/San_Marino
    {  230, 6245,    3295,  7, }, // Europe/Sarajevo
    {  230, 4451,    3302,  7, }, // Europe/Saratov
    {  230, 8849,   52225, 11, }, // Europe/Simferopol
    {  230, 6383,   52236,  6, }, // Europe/Skopje
    {  230, 6599,   37286,  5, }, // Europe/Sofia
    {  230, 6523,   52242,  9, }, // Europe/Stockholm
    {  230, 8867,    3337,  6, }, // Europe/Tallinn
    {  230, 6202,    3343,  6, }, // Europe/Tirane
    {  230, 6352,    3358,  5, }, // Europe/Vaduz
    {  230, 6567,    3363,  7, }, // Europe/Vatican
    {  230, 6231,    3370,  4, }, // Europe/Vienna
    {  230, 8899,   52260,  7, }, // Europe/Vilnius
    {  230, 3190,    3390,  7, }, // Europe/Warsaw
    {  230, 6261,    3397,  6, }, // Europe/Zagreb
    {  230, 6540,   37310,  5, }, // Europe/Zurich
    {  230, 7211,     467,  7, }, // Indian/Reunion
    {  230, 3157,     474,  5, }, // Pacific/Chuuk
    {  230, 3096,     479,  7, }, // Pacific/Pohnpei
    {  231, 6788,   60937,  7, }, // Africa/Abidjan Esperanto/Latin/world
    {  231, 6760,   60944,  5, }, // Africa/Accra
    {  231, 5059,   60949,  9, }, // Africa/Addis_Ababa
    {  231, 7420,   60958,  6, }, // Africa/Algiers
    {  231,   14,   60964,  6, }, // Africa/Asmara
    {  231, 5257,   60970,  5, }, // Africa/Bangui
    {  231, 6746,   60975,  7, }, // Africa/Banjul
    {  231, 7435,   60982,  6, }, // Africa/Bissau
    {  231, 5287,   60988,  9, }, // Africa/Brazzaville
    {  231, 4922,   60997,  9, }, // Africa/Bujumbura
    {  231, 2320,    4572,  5, }, // Africa/Cairo
    {  231, 4117,   61006, 10, }, // Africa/Casablanca
    {  231, 7449,   61016,  5, }, // Africa/Ceuta
    {  231, 6773,   61021,  8, }, // Africa/Conakry
    {  231, 6840,   61029,  6, }, // Africa/Dakar
    {  231, 5130,   61035, 10, }, // Africa/Dar_es_Salaam
    {  231, 5043,   61045,  7, }, // Africa/Djibouti
    {  231, 5243,   61052,  6, }, // Africa/Douala
    {  231, 5166,   61058,  5, }, // Africa/El_Aaiun
    {  231, 6853,   61063,  8, }, // Africa/Freetown
    {  231, 4906,   61071,  8, }, // Africa/Gaborone
    {  231, 5015,   61079,  7, }, // Africa/Harare
    {  231, 4479,   61086, 12, }, // Africa/Johannesburg
    {  231, 4499,   61098,  5, }, // Africa/Juba
    {  231, 5151,   61103,  7, }, // Africa/Kampala
    {  231, 4524,   61110,  7, }, // Africa/Khartoum
    {  231, 4973,   61117,  6, }, // Africa/Kigali
    {  231, 5306,   61123,  7, }, // Africa/Kinshasa
    {  231, 4744,   61130,  6, }, // Africa/Lagos
    {  231, 5336,   61136,  9, }, // Africa/Libreville
    {  231, 6869,   61145,  5, }, // Africa/Lome
    {  231, 5211,   61150,  6, }, // Africa/Luanda
    {  231, 4939,   61156,  9, }, // Africa/Lubumbashi
    {  231, 5001,   61165,  6, }, // Africa/Lusaka
    {  231, 5197,   61171,  7, }, // Africa/Maseru
    {  231, 5182,   61178,  7, }, // Africa/Mbabane
    {  231, 5113,   61185,  8, }, // Africa/Mogadishu
    {  231, 7034,   61193,  8, }, // Africa/Monrovia
    {  231, 3853,   61201,  8, }, // Africa/Nairobi
    {  231, 5271,   61209,  8, }, // Africa/Ndjamena
    {  231, 5354,   61217,  7, }, // Africa/Niamey
    {  231, 6803,   61224,  8, }, // Africa/Nouakchott
    {  231, 6727,   61232,  9, }, // Africa/Ouagadougou
    {  231, 5225,   61241,  9, }, // Africa/Porto-Novo
    {  231, 4435,   61250,  8, }, // Africa/Sao_Tome
    {  231, 2866,   61258,  7, }, // Africa/Tripoli
    {  231, 6554,   61265,  6, }, // Africa/Tunis
    {  231, 4152,   61271,  8, }, // Africa/Windhoek
    {  231, 5607,   61279,  7, }, // America/Anguilla
    {  231, 5624,   61286,  7, }, // America/Antigua
    {  231, 4554,   26427,  9, }, // America/Araguaina
    {  231,  166,   61293,  7, }, // America/Argentina/Buenos_Aires
    {  231,  270,   61300,  7, }, // America/Argentina/Cordoba
    {  231, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {  231, 7520,   61307,  9, }, // America/Argentina/Salta
    {  231, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  231, 5640,   61316,  5, }, // America/Aruba
    {  231, 4252,   61321,  9, }, // America/Asuncion
    {  231,  237,      22,  8, }, // America/Atikokan
    {  231, 3679,   61330,  5, }, // America/Bahia
    {  231, 7623,      30, 17, }, // America/Bahia_Banderas
    {  231, 5654,   61335,  7, }, // America/Barbados
    {  231, 7646,   61342,  6, }, // America/Belem
    {  231, 5407,   61348,  6, }, // America/Belize
    {  231, 4361,   61354,  6, }, // America/Bogota
    {  231, 7713,   61360, 14, }, // America/Cambridge_Bay
    {  231, 3868,   61374,  7, }, // America/Cancun
    {  231, 4694,   61381,  8, }, // America/Caracas
    {  231, 4345,   61389,  6, }, // America/Cayenne
    {  231, 5496,   61395,  8, }, // America/Cayman
    {  231, 2260,   61403,  6, }, // America/Chicago
    {  231, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  231, 5422,   61409,  9, }, // America/Costa_Rica
    {  231, 3791,   26442,  6, }, // America/Cuiaba
    {  231, 5723,   61418,  7, }, // America/Curacao
    {  231,  805,   61425,  7, }, // America/Denver
    {  231, 3465,   61432,  8, }, // America/Detroit
    {  231, 5739,   61440,  8, }, // America/Dominica
    {  231,  893,   61448,  9, }, // America/Edmonton
    {  231, 7869,   26448,  8, }, // America/Eirunepe
    {  231, 5441,   61457,  9, }, // America/El_Salvador
    {  231, 7906,   61466,  9, }, // America/Fortaleza
    {  231, 4612,   61475, 12, }, // America/Grand_Turk
    {  231, 5770,   61487,  7, }, // America/Grenada
    {  231, 5786,   61494,  9, }, // America/Guadeloupe
    {  231, 3760,   61503,  9, }, // America/Guatemala
    {  231, 6932,   61512,  6, }, // America/Guyana
    {  231, 1939,   61518,  9, }, // America/Halifax
    {  231, 2281,   61527,  6, }, // America/Havana
    {  231,  348,   61533, 21, }, // America/Indiana/Indianapolis
    {  231,  481,   61554, 14, }, // America/Indiana/Knox
    {  231, 7961,   61568, 17, }, // America/Indiana/Marengo
    {  231, 7985,   61585, 20, }, // America/Indiana/Petersburg
    {  231, 8012,   61605, 19, }, // America/Indiana/Tell_City
    {  231, 8038,   61624, 15, }, // America/Indiana/Vevay
    {  231, 8060,   61639, 19, }, // America/Indiana/Vincennes
    {  231, 8086,   61658, 17, }, // America/Indiana/Winamac
    {  231,  660,   61675,  8, }, // America/Iqaluit
    {  231, 2799,   61683,  7, }, // America/Jamaica
    {  231, 8125,   61690, 20, }, // America/Kentucky/Monticello
    {  231, 7169,   61710,  4, }, // America/Lima
    {  231, 3239,   61714, 11, }, // America/Los_Angeles
    {  231, 5932,     207, 22, }, // America/Lower_Princes
    {  231, 8153,   26456,  6, }, // America/Maceio
    {  231, 8168,   61725,  7, }, // America/Managua
    {  231, 1908,   61732,  7, }, // America/Manaus
    {  231, 5897,   61739,  8, }, // America/Marigot
    {  231, 5805,   61747,  9, }, // America/Martinique
    {  231, 2917,   26471,  8, }, // America/Mazatlan
    {  231, 8220,   61756,  6, }, // America/Merida
    {  231, 2949,   61762, 10, }, // America/Mexico_City
    {  231, 4391,   61772,  8, }, // America/Miquelon
    {  231, 8254,   61780,  8, }, // America/Moncton
    {  231, 8270,   61788,  9, }, // America/Monterrey
    {  231, 5824,   61797,  9, }, // America/Montserrat
    {  231, 5481,   61806,  6, }, // America/Nassau
    {  231, 2379,   61812,  8, }, // America/New_York
    {  231, 1850,   61820, 19, }, // America/Noronha
    {  231, 8301,   61839, 20, }, // America/North_Dakota/Beulah
    {  231, 8329,   61859, 20, }, // America/North_Dakota/Center
    {  231, 8357,   61879, 23, }, // America/North_Dakota/New_Salem
    {  231,  393,   61902,  4, }, // America/Nuuk
    {  231, 2356,   61906,  6, }, // America/Panama
    {  231, 2973,   61912,  7, }, // America/Phoenix
    {  231, 3945,   61919, 11, }, // America/Port-au-Prince
    {  231, 5954,   61930, 10, }, // America/Port_of_Spain
    {  231, 5843,   61940, 10, }, // America/Puerto_Rico
    {  231, 8477,   26572,  8, }, // America/Santarem
    {  231, 6111,   61950, 14, }, // America/Santo_Domingo
    {  231, 1878,   61964,  8, }, // America/Sao_Paulo
    {  231, 6899,     302, 16, }, // America/Scoresbysund
    {  231, 8508,   61972, 17, }, // America/St_Barthelemy
    {  231, 2061,     332, 10, }, // America/St_Johns
    {  231, 5863,   61989, 17, }, // America/St_Kitts
    {  231, 5880,   62006, 12, }, // America/St_Lucia
    {  231,  855,   62018, 13, }, // America/St_Thomas
    {  231, 5913,   62031, 15, }, // America/St_Vincent
    {  231, 5461,   62046, 11, }, // America/Tegucigalpa
    {  231, 5756,   26664,  7, }, // America/Thule
    {  231, 2093,   62057,  9, }, // America/Vancouver
    {  231,  734,   62066,  8, }, // America/Winnipeg
    {  231, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  231, 8568,   62074, 14, }, // Antarctica/Macquarie
    {  231, 7295,   62088,  5, }, // Antarctica/Syowa
    {  231, 1429,   62093, 11, }, // Arctic/Longyearbyen
    {  231, 5570,   62104,  5, }, // Asia/Aden
    {  231, 5395,   62109,  6, }, // Asia/Almaty
    {  231, 3968,   62115,  5, }, // Asia/Amman
    {  231, 5511,    7482,  6, }, // Asia/Anadyr
    {  231, 5523,    5491,  5, }, // Asia/Aqtau
    {  231, 5368,    7488,  6, }, // Asia/Aqtobe
    {  231,  964,   62120,  8, }, // Asia/Ashgabat
    {  231, 8624,    7494,  6, }, // Asia/Atyrau
    {  231, 3607,   62128,  7, }, // Asia/Baghdad
    {  231, 5534,   62135,  7, }, // Asia/Bahrain
    {  231, 3653,   62142,  5, }, // Asia/Baku
    {  231, 4466,   62147,  7, }, // Asia/Bangkok
    {  231, 4086,   62154,  7, }, // Asia/Beirut
    {  231, 3778,   62161,  7, }, // Asia/Bishkek
    {  231, 5989,   62168,  7, }, // Asia/Brunei
    {  231, 4601,   62175,  4, }, // Asia/Chita
    {  231, 4511,    5519,  7, }, // Asia/Colombo
    {  231, 4540,   26689,  7, }, // Asia/Damascus
    {  231, 1093,     249,  4, }, // Asia/Dhaka
    {  231, 6192,   62179,  4, }, // Asia/Dili
    {  231, 3596,   62183,  9, }, // Asia/Dubai
    {  231, 6159,   62192,  8, }, // Asia/Dushanbe
    {  231, 8651,   62200,  5, }, // Asia/Gaza
    {  231, 4795,   62205,  7, }, // Asia/Hebron
    {  231, 1259,   62212,  8, }, // Asia/Ho_Chi_Minh
    {  231, 2704,   62220,  8, }, // Asia/Hong_Kong
    {  231, 4771,   62228,  4, }, // Asia/Hovd
    {  231, 4184,   62232,  8, }, // Asia/Irkutsk
    {  231, 7006,   62240,  7, }, // Asia/Jakarta
    {  231, 1290,   62247, 10, }, // Asia/Jerusalem
    {  231, 3560,   62257,  6, }, // Asia/Kabul
    {  231, 4316,   62263,  8, }, // Asia/Kamchatka
    {  231, 4239,   62271,  7, }, // Asia/Karachi
    {  231, 1185,   62278,  9, }, // Asia/Kathmandu
    {  231, 8661,   62287,  7, }, // Asia/Khandyga
    {  231,  992,   62294,  7, }, // Asia/Kolkata
    {  231, 4197,   62301, 12, }, // Asia/Krasnoyarsk
    {  231, 4859,   62313, 12, }, // Asia/Kuala_Lumpur
    {  231, 5547,   62325,  7, }, // Asia/Kuwait
    {  231, 1211,    7613,  5, }, // Asia/Macau
    {  231, 7182,   62332,  6, }, // Asia/Manila
    {  231, 6920,   62338,  7, }, // Asia/Muscat
    {  231, 2554,   62345,  7, }, // Asia/Nicosia
    {  231, 8675,   62352, 12, }, // Asia/Novokuznetsk
    {  231, 4135,   62364, 12, }, // Asia/Novosibirsk
    {  231, 4229,   62376,  5, }, // Asia/Omsk
    {  231, 7145,   62381,  5, }, // Asia/Oral
    {  231, 6961,   62386,  8, }, // Asia/Phnom_Penh
    {  231, 4214,   62394, 10, }, // Asia/Pyongyang
    {  231, 5559,   62404,  6, }, // Asia/Qatar
    {  231, 8708,    7652,  8, }, // Asia/Qostanay
    {  231, 4269,   62410,  9, }, // Asia/Qyzylorda
    {  231, 3584,   62419,  5, }, // Asia/Riyadh
    {  231, 4408,   62424,  8, }, // Asia/Sakhalin
    {  231, 7245,   62432, 10, }, // Asia/Samarkand
    {  231, 3279,   62442,  5, }, // Asia/Seoul
    {  231, 1053,   62447,  7, }, // Asia/Shanghai
    {  231, 3300,   62454,  9, }, // Asia/Singapore
    {  231, 4297,    7699, 13, }, // Asia/Srednekolymsk
    {  231, 3263,   62463,  6, }, // Asia/Taipei
    {  231, 4781,   62469,  8, }, // Asia/Tashkent
    {  231, 3915,   62477,  7, }, // Asia/Tbilisi
    {  231, 2772,   62484,  8, }, // Asia/Tehran
    {  231, 1317,   62492,  6, }, // Asia/Thimphu
    {  231, 2821,    4238,  5, }, // Asia/Tokyo
    {  231, 1021,   62498, 10, }, // Asia/Ulaanbaatar
    {  231, 1159,   62508,  7, }, // Asia/Urumqi
    {  231, 8722,   62515,  7, }, // Asia/Ust-Nera
    {  231, 6977,   62522,  9, }, // Asia/Vientiane
    {  231, 4710,   62531, 12, }, // Asia/Vladivostok
    {  231, 4828,   62543,  8, }, // Asia/Yakutsk
    {  231, 1235,   62551,  7, }, // Asia/Yangon
    {  231, 3883,   62558, 14, }, // Asia/Yekaterinburg
    {  231, 3747,   62572,  7, }, // Asia/Yerevan
    {  231, 3663,   62579,  6, }, // Atlantic/Azores
    {  231, 5671,   62585,  8, }, // Atlantic/Bermuda
    {  231, 6628,   62593,  8, }, // Atlantic/Canary
    {  231, 3727,   62601,  9, }, // Atlantic/Cape_Verde
    {  231, 1395,   62610,  6, }, // Atlantic/Faroe
    {  231, 8736,   62616,  7, }, // Atlantic/Madeira
    {  231, 2748,   62623, 10, }, // Atlantic/Reykjavik
    {  231, 7272,   62633, 11, }, // Atlantic/South_Georgia
    {  231, 6821,   62644, 13, }, // Atlantic/St_Helena
    {  231, 6644,   62657,  8, }, // Atlantic/Stanley
    {  231, 1670,   62665,  8, }, // Australia/Adelaide
    {  231, 1635,   62673,  8, }, // Australia/Brisbane
    {  231, 1583,   62681,  7, }, // Australia/Darwin
    {  231, 1516,   62688,  7, }, // Australia/Hobart
    {  231, 1727,   62695,  8, }, // Australia/Melbourne
    {  231, 1762,   62703,  5, }, // Australia/Perth
    {  231, 1463,   62708,  7, }, // Australia/Sydney
    {  231, 6441,   62715, 10, }, // Europe/Amsterdam
    {  231, 6216,   62725,  6, }, // Europe/Andorra
    {  231, 3620,   62731,  9, }, // Europe/Astrakhan
    {  231, 2300,   62740,  5, }, // Europe/Athens
    {  231, 6336,   62745,  8, }, // Europe/Belgrade
    {  231, 4757,   62753,  7, }, // Europe/Berlin
    {  231, 6488,   62760, 10, }, // Europe/Bratislava
    {  231, 2167,   62770,  7, }, // Europe/Brussels
    {  231, 3928,   62777,  9, }, // Europe/Bucharest
    {  231, 3806,   62786,  9, }, // Europe/Budapest
    {  231, 8772,   62795, 21, }, // Europe/Busingen
    {  231, 2583,   62816,  8, }, // Europe/Chisinau
    {  231, 6289,   62824,  9, }, // Europe/Copenhagen
    {  231, 2338,   62833,  7, }, // Europe/Dublin
    {  231, 6307,   62840, 10, }, // Europe/Gibraltar
    {  231, 8788,   62850,  9, }, // Europe/Guernsey
    {  231, 6612,   62859,  8, }, // Europe/Helsinki
    {  231, 8804,   62867, 10, }, // Europe/Isle_of_Man
    {  231, 1130,   62877,  9, }, // Europe/Istanbul
    {  231, 8823,   62886,  7, }, // Europe/Jersey
    {  231, 3979,   62893, 12, }, // Europe/Kaliningrad
    {  231, 2527,   62905,  5, }, // Europe/Kyiv
    {  231, 3213,   62910,  7, }, // Europe/Lisbon
    {  231, 6506,   62917,  9, }, // Europe/Ljubljana
    {  231, 2501,   62926,  7, }, // Europe/London
    {  231, 6365,   62933, 11, }, // Europe/Luxembourg
    {  231, 4892,   62944,  7, }, // Europe/Madrid
    {  231, 6397,   62951,  5, }, // Europe/Malta
    {  231, 3693,   62956,  6, }, // Europe/Minsk
    {  231, 6410,    5926,  6, }, // Europe/Monaco
    {  231, 3537,   62962,  6, }, // Europe/Moscow
    {  231, 4284,   62968,  6, }, // Europe/Paris
    {  231, 6424,   62974,  9, }, // Europe/Podgorica
    {  231, 6275,   62983,  5, }, // Europe/Prague
    {  231, 8837,   62988,  4, }, // Europe/Riga
    {  231, 6324,   62992,  4, }, // Europe/Rome
    {  231, 6470,   62996,  9, }, // Europe/San_Marino
    {  231, 8849,   63005, 11, }, // Europe/Simferopol
    {  231, 6383,   63016,  6, }, // Europe/Skopje
    {  231, 6599,   63022,  5, }, // Europe/Sofia
    {  231, 6523,   63027,  9, }, // Europe/Stockholm
    {  231, 8867,   63036,  6, }, // Europe/Tallinn
    {  231, 6202,   63042,  6, }, // Europe/Tirane
    {  231, 8882,   63048, 10, }, // Europe/Ulyanovsk
    {  231, 6352,   63058,  6, }, // Europe/Vaduz
    {  231, 6567,   26969,  8, }, // Europe/Vatican
    {  231, 6231,   63064,  5, }, // Europe/Vienna
    {  231, 8899,   63069,  5, }, // Europe/Vilnius
    {  231, 4727,   63074, 10, }, // Europe/Volgograd
    {  231, 3190,   63084,  8, }, // Europe/Warsaw
    {  231, 6261,   63092,  7, }, // Europe/Zagreb
    {  231, 6540,   63099,  6, }, // Europe/Zurich
    {  231, 6947,   63105,  7, }, // Indian/Chagos
    {  231, 6046,   63112, 15, }, // Indian/Christmas
    {  231, 6063,   63127, 12, }, // Indian/Cocos
    {  231, 5029,   63139,  7, }, // Indian/Comoro
    {  231, 6661,   63146, 10, }, // Indian/Kerguelen
    {  231, 7260,   26983,  4, }, // Indian/Mahe
    {  231, 7050,   63156,  8, }, // Indian/Maldives
    {  231, 4069,   63164,  8, }, // Indian/Mauritius
    {  231, 5098,   63172,  6, }, // Indian/Mayotte
    {  231, 7211,   63178,  6, }, // Indian/Reunion
    {  231, 4422,   63184,  4, }, // Pacific/Apia
    {  231,  932,   63188,  8, }, // Pacific/Auckland
    {  231, 3015,   63196,  7, }, // Pacific/Chatham
    {  231, 3157,   63203,  4, }, // Pacific/Chuuk
    {  231, 2237,   63207, 10, }, // Pacific/Easter
    {  231, 3902,   63217,  5, }, // Pacific/Fiji
    {  231, 7343,   63222,  9, }, // Pacific/Funafuti
    {  231, 6678,   63231,  9, }, // Pacific/Galapagos
    {  231, 6018,   63240,  5, }, // Pacific/Guam
    {  231, 4051,   63245, 13, }, // Pacific/Marquesas
    {  231, 8914,   63258, 13, }, // Pacific/Midway
    {  231, 7084,   63271,  5, }, // Pacific/Nauru
    {  231, 7132,   63276,  4, }, // Pacific/Niue
    {  231, 4168,   63280, 13, }, // Pacific/Norfolk
    {  231, 7098,   63293,  5, }, // Pacific/Noumea
    {  231, 3126,   63298,  8, }, // Pacific/Pago_Pago
    {  231, 7155,   63306,  6, }, // Pacific/Palau
    {  231, 7194,   63312, 15, }, // Pacific/Pitcairn
    {  231, 3096,    8311,  6, }, // Pacific/Pohnpei
    {  231, 6076,   63327,  9, }, // Pacific/Rarotonga
    {  231, 6031,   63336,  7, }, // Pacific/Saipan
    {  231, 7312,   63343,  7, }, // Pacific/Tahiti
    {  231, 6712,   63350,  6, }, // Pacific/Tarawa
    {  231, 7392,   63356, 10, }, // Pacific/Wake
    {  231, 7405,   63366,  6, }, // Pacific/Wallis
    {  232, 5059,    3784, 11, }, // Africa/Addis_Ababa Estonian/Latin/Estonia
    {  232, 7420,   63372,  6, }, // Africa/Algiers
    {  232,   14,       0,  6, }, // Africa/Asmara
    {  232, 2320,    4572,  5, }, // Africa/Cairo
    {  232, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  232, 4524,   63386,  6, }, // Africa/Khartoum
    {  232, 6869,   20300,  4, }, // Africa/Lome
    {  232, 5113,   26379,  8, }, // Africa/Mogadishu
    {  232, 5271,   43996,  9, }, // Africa/Ndjamena
    {  232, 4435,       6,  8, }, // Africa/Sao_Tome
    {  232, 4554,   26427,  9, }, // America/Araguaina
    {  232,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  232, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {  232, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  232, 4252,      14,  8, }, // America/Asuncion
    {  232,  237,      22,  8, }, // America/Atikokan
    {  232, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  232, 7646,   35764,  5, }, // America/Belem
    {  232, 4361,   20389,  6, }, // America/Bogota
    {  232, 3868,      47,  6, }, // America/Cancun
    {  232, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  232, 3791,   26442,  6, }, // America/Cuiaba
    {  232, 5723,      66,  7, }, // America/Curacao
    {  232, 7869,   26448,  8, }, // America/Eirunepe
    {  232, 2281,   63392,  7, }, // America/Havana
    {  232,  481,      73, 13, }, // America/Indiana/Knox
    {  232, 7961,      86, 16, }, // America/Indiana/Marengo
    {  232, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  232, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  232, 8038,     139, 14, }, // America/Indiana/Vevay
    {  232, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  232, 8086,     171, 16, }, // America/Indiana/Winamac
    {  232, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  232, 5932,     207, 22, }, // America/Lower_Princes
    {  232, 8153,   26456,  6, }, // America/Maceio
    {  232, 2917,   26471,  8, }, // America/Mazatlan
    {  232, 8220,     229,  6, }, // America/Merida
    {  232, 2949,   57469,  6, }, // America/Mexico_City
    {  232, 8301,   63399, 20, }, // America/North_Dakota/Beulah
    {  232, 8329,   63419, 20, }, // America/North_Dakota/Center
    {  232, 8357,   63439, 23, }, // America/North_Dakota/New_Salem
    {  232,  393,     298,  4, }, // America/Nuuk
    {  232, 8477,   26572,  8, }, // America/Santarem
    {  232, 1878,   26580,  9, }, // America/Sao_Paulo
    {  232, 6899,     302, 16, }, // America/Scoresbysund
    {  232, 8508,   26589, 16, }, // America/St_Barthelemy
    {  232, 2061,   26605, 12, }, // America/St_Johns
    {  232, 5863,   26617, 11, }, // America/St_Kitts
    {  232, 5880,   35832, 11, }, // America/St_Lucia
    {  232,  855,   44161, 12, }, // America/St_Thomas
    {  232, 5913,   26651, 13, }, // America/St_Vincent
    {  232, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  232, 5395,   63462,  6, }, // Asia/Almaty
    {  232, 5511,   63468,  6, }, // Asia/Anadyr
    {  232, 5523,    5491,  5, }, // Asia/Aqtau
    {  232, 5368,   63474,  6, }, // Asia/Aqtobe
    {  232,  964,   44190,  7, }, // Asia/Ashgabat
    {  232, 8624,   63480,  6, }, // Asia/Atyrau
    {  232, 3607,    4044,  6, }, // Asia/Baghdad
    {  232, 5534,    4050,  7, }, // Asia/Bahrain
    {  232, 3653,   63486,  5, }, // Asia/Baku
    {  232, 3778,   33323,  6, }, // Asia/Bishkek
    {  232, 4601,   63491,  5, }, // Asia/Chita
    {  232, 4540,    4083,  8, }, // Asia/Damascus
    {  232, 6159,   33336,  7, }, // Asia/Dushanbe
    {  232, 1259,   63496, 11, }, // Asia/Ho_Chi_Minh
    {  232, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  232, 1290,   63507, 11, }, // Asia/Jerusalem
    {  232, 4316,   63518,  9, }, // Asia/Kamchatka
    {  232, 1185,    5593,  8, }, // Asia/Kathmandu
    {  232, 8661,   63527,  7, }, // Asia/Khandyga
    {  232,  992,     422,  7, }, // Asia/Kolkata
    {  232, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  232, 5547,   63534,  6, }, // Asia/Kuwait
    {  232, 1211,    4203,  5, }, // Asia/Macau
    {  232, 6920,   63540,  6, }, // Asia/Muscat
    {  232, 2554,    5620,  7, }, // Asia/Nicosia
    {  232, 5559,    4214,  5, }, // Asia/Qatar
    {  232, 8708,    7652,  8, }, // Asia/Qostanay
    {  232, 4269,   63546,  9, }, // Asia/Qyzylorda
    {  232, 3584,   63555,  8, }, // Asia/Riyadh
    {  232, 4408,   63563,  8, }, // Asia/Sakhalin
    {  232, 3279,   57960,  4, }, // Asia/Seoul
    {  232, 3300,   20782,  8, }, // Asia/Singapore
    {  232, 4297,   63571, 13, }, // Asia/Srednekolymsk
    {  232, 4781,   33457,  7, }, // Asia/Tashkent
    {  232, 3915,   63584,  8, }, // Asia/Tbilisi
    {  232, 2772,    7724,  7, }, // Asia/Tehran
    {  232, 2821,   63592,  5, }, // Asia/Tokyo
    {  232, 1159,   26801,  6, }, // Asia/Urumqi
    {  232, 4828,    7755,  7, }, // Asia/Yakutsk
    {  232, 1235,     442,  6, }, // Asia/Yangon
    {  232, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  232, 3747,    7780,  7, }, // Asia/Yerevan
    {  232, 3663,   63597,  8, }, // Atlantic/Azores
    {  232, 6628,   63605, 14, }, // Atlantic/Canary
    {  232, 3727,   63619, 15, }, // Atlantic/Cape_Verde
    {  232, 1395,   63634, 12, }, // Atlantic/Faroe
    {  232, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  232, 7272,   63646, 13, }, // Atlantic/South_Georgia
    {  232, 6821,   36066, 12, }, // Atlantic/St_Helena
    {  232, 3620,   33512,  8, }, // Europe/Astrakhan
    {  232, 2300,   63659,  6, }, // Europe/Athens
    {  232, 6336,   26883,  7, }, // Europe/Belgrade
    {  232, 4757,   63665,  7, }, // Europe/Berlin
    {  232, 2167,   22634,  7, }, // Europe/Brussels
    {  232, 3928,   26897,  8, }, // Europe/Bucharest
    {  232, 8772,   26905,  8, }, // Europe/Busingen
    {  232, 2583,   63672,  8, }, // Europe/Chisinau
    {  232, 6289,   63680, 11, }, // Europe/Copenhagen
    {  232, 6612,   63691,  8, }, // Europe/Helsinki
    {  232, 8804,   63699,  9, }, // Europe/Isle_of_Man
    {  232, 1130,   22693,  8, }, // Europe/Istanbul
    {  232, 2527,   63708,  5, }, // Europe/Kyiv
    {  232, 3213,    4381,  8, }, // Europe/Lisbon
    {  232, 6582,   63713, 13, }, // Europe/Mariehamn
    {  232, 3537,   22748,  6, }, // Europe/Moscow
    {  232, 4284,   63726,  6, }, // Europe/Paris
    {  232, 6275,   36153,  5, }, // Europe/Prague
    {  232, 8837,   63732,  4, }, // Europe/Riga
    {  232, 6324,   63736,  5, }, // Europe/Rome
    {  232, 6202,   21095,  6, }, // Europe/Tirane
    {  232, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  232, 6567,    5978,  7, }, // Europe/Vatican
    {  232, 6231,   63741,  4, }, // Europe/Vienna
    {  232, 3190,   63745,  8, }, // Europe/Warsaw
    {  232, 6540,    4459,  6, }, // Europe/Zurich
    {  232, 6046,   63753,  9, }, // Indian/Christmas
    {  232, 6063,   63762, 12, }, // Indian/Cocos
    {  232, 7260,   26983,  4, }, // Indian/Mahe
    {  232, 7050,   63774,  9, }, // Indian/Maldives
    {  232, 7211,     467,  7, }, // Indian/Reunion
    {  232, 3157,     474,  5, }, // Pacific/Chuuk
    {  232, 2237,   63783, 13, }, // Pacific/Easter
    {  232, 3902,   33704,  5, }, // Pacific/Fiji
    {  232, 6678,   21159,  9, }, // Pacific/Galapagos
    {  232, 2723,    4493,  8, }, // Pacific/Honolulu
    {  232, 3049,    4501,  9, }, // Pacific/Kanton
    {  232, 4051,   63796, 12, }, // Pacific/Marquesas
    {  232, 7098,    4510,  6, }, // Pacific/Noumea
    {  232, 7155,   63808,  5, }, // Pacific/Palau
    {  232, 3096,     479,  7, }, // Pacific/Pohnpei
    {  233,   14,       0,  6, }, // Africa/Asmara Ewe/Latin/Ghana
    {  233, 4435,       6,  8, }, // Africa/Sao_Tome
    {  233, 4252,      14,  8, }, // America/Asuncion
    {  233,  237,      22,  8, }, // America/Atikokan
    {  233, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  233, 3868,   33182,  6, }, // America/Cancun
    {  233, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  233, 5723,      66,  7, }, // America/Curacao
    {  233,  481,      73, 13, }, // America/Indiana/Knox
    {  233, 7961,      86, 16, }, // America/Indiana/Marengo
    {  233, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  233, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  233, 8038,     139, 14, }, // America/Indiana/Vevay
    {  233, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  233, 8086,     171, 16, }, // America/Indiana/Winamac
    {  233, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  233, 5932,     207, 22, }, // America/Lower_Princes
    {  233, 8220,    7008,  6, }, // America/Merida
    {  233, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  233, 8329,     255, 20, }, // America/North_Dakota/Center
    {  233, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  233,  393,     298,  4, }, // America/Nuuk
    {  233, 6899,     302, 16, }, // America/Scoresbysund
    {  233, 8508,     318, 14, }, // America/St_Barthelemy
    {  233, 2061,     332, 10, }, // America/St_Johns
    {  233, 5863,     342,  9, }, // America/St_Kitts
    {  233, 5880,     351,  9, }, // America/St_Lucia
    {  233,  855,     360, 10, }, // America/St_Thomas
    {  233, 5913,     370, 11, }, // America/St_Vincent
    {  233, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  233, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  233, 1185,   63813, 24, }, // Asia/Kathmandu
    {  233,  992,     422,  7, }, // Asia/Kolkata
    {  233, 1211,    4203,  5, }, // Asia/Macau
    {  233, 1235,     442,  6, }, // Asia/Yangon
    {  233, 1395,     448,  5, }, // Atlantic/Faroe
    {  233, 6821,     453, 10, }, // Atlantic/St_Helena
    {  233, 2527,    8030,  4, }, // Europe/Kyiv
    {  233, 7211,     467,  7, }, // Indian/Reunion
    {  233, 3157,     474,  5, }, // Pacific/Chuuk
    {  233, 2723,    4493,  8, }, // Pacific/Honolulu
    {  233, 3049,    4501,  9, }, // Pacific/Kanton
    {  233, 3096,     479,  7, }, // Pacific/Pohnpei
    {  235,   14,       0,  6, }, // Africa/Asmara Ewondo/Latin/Cameroon
    {  235, 4435,       6,  8, }, // Africa/Sao_Tome
    {  235, 4252,      14,  8, }, // America/Asuncion
    {  235,  237,      22,  8, }, // America/Atikokan
    {  235, 7623,      30, 17, }, // America/Bahia_Banderas
    {  235, 3868,      47,  6, }, // America/Cancun
    {  235, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  235, 5723,      66,  7, }, // America/Curacao
    {  235,  481,      73, 13, }, // America/Indiana/Knox
    {  235, 7961,      86, 16, }, // America/Indiana/Marengo
    {  235, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  235, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  235, 8038,     139, 14, }, // America/Indiana/Vevay
    {  235, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  235, 8086,     171, 16, }, // America/Indiana/Winamac
    {  235, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  235, 5932,     207, 22, }, // America/Lower_Princes
    {  235, 8220,     229,  6, }, // America/Merida
    {  235, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  235, 8329,     255, 20, }, // America/North_Dakota/Center
    {  235, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  235,  393,     298,  4, }, // America/Nuuk
    {  235, 6899,     302, 16, }, // America/Scoresbysund
    {  235, 8508,     318, 14, }, // America/St_Barthelemy
    {  235, 2061,     332, 10, }, // America/St_Johns
    {  235, 5863,     342,  9, }, // America/St_Kitts
    {  235, 5880,     351,  9, }, // America/St_Lucia
    {  235,  855,     360, 10, }, // America/St_Thomas
    {  235, 5913,     370, 11, }, // America/St_Vincent
    {  235, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  235, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  235, 1185,     413,  9, }, // Asia/Kathmandu
    {  235,  992,     422,  7, }, // Asia/Kolkata
    {  235, 1211,     429,  5, }, // Asia/Macau
    {  235, 1235,     442,  6, }, // Asia/Yangon
    {  235, 1395,     448,  5, }, // Atlantic/Faroe
    {  235, 6821,     453, 10, }, // Atlantic/St_Helena
    {  235, 2527,     463,  4, }, // Europe/Kyiv
    {  235, 7211,     467,  7, }, // Indian/Reunion
    {  235, 3157,     474,  5, }, // Pacific/Chuuk
    {  235, 3096,     479,  7, }, // Pacific/Pohnpei
    {  236,   14,       0,  6, }, // Africa/Asmara Faroese/Latin/Faroe Islands
    {  236, 5043,   26360,  7, }, // Africa/Djibouti
    {  236, 4435,       6,  8, }, // Africa/Sao_Tome
    {  236, 4252,      14,  8, }, // America/Asuncion
    {  236,  237,      22,  8, }, // America/Atikokan
    {  236, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  236, 5407,   63837,  5, }, // America/Belize
    {  236, 3868,   33182,  6, }, // America/Cancun
    {  236, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  236, 5422,   21503, 10, }, // America/Costa_Rica
    {  236, 5723,      66,  7, }, // America/Curacao
    {  236,  481,      73, 13, }, // America/Indiana/Knox
    {  236, 7961,      86, 16, }, // America/Indiana/Marengo
    {  236, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  236, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  236, 8038,     139, 14, }, // America/Indiana/Vevay
    {  236, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  236, 8086,     171, 16, }, // America/Indiana/Winamac
    {  236, 2799,    3877,  7, }, // America/Jamaica
    {  236, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  236, 5932,     207, 22, }, // America/Lower_Princes
    {  236, 8220,    7008,  6, }, // America/Merida
    {  236, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  236, 8329,     255, 20, }, // America/North_Dakota/Center
    {  236, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  236,  393,     298,  4, }, // America/Nuuk
    {  236, 5843,   21940, 11, }, // America/Puerto_Rico
    {  236, 6899,     302, 16, }, // America/Scoresbysund
    {  236, 8508,     318, 14, }, // America/St_Barthelemy
    {  236, 2061,     332, 10, }, // America/St_Johns
    {  236, 5863,     342,  9, }, // America/St_Kitts
    {  236, 5880,     351,  9, }, // America/St_Lucia
    {  236,  855,     360, 10, }, // America/St_Thomas
    {  236, 5913,     370, 11, }, // America/St_Vincent
    {  236, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  236, 5534,   63842,  6, }, // Asia/Bahrain
    {  236, 8651,   63848,  4, }, // Asia/Gaza
    {  236, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  236, 1185,     413,  9, }, // Asia/Kathmandu
    {  236,  992,     422,  7, }, // Asia/Kolkata
    {  236, 5547,   63852,  6, }, // Asia/Kuwait
    {  236, 1211,    7613,  5, }, // Asia/Macau
    {  236, 8708,     434,  8, }, // Asia/Qostanay
    {  236, 3300,    7691,  8, }, // Asia/Singapore
    {  236, 2772,    7724,  7, }, // Asia/Tehran
    {  236, 1235,   58486,  7, }, // Asia/Yangon
    {  236, 3663,   63858,  9, }, // Atlantic/Azores
    {  236, 3727,   63867, 16, }, // Atlantic/Cape_Verde
    {  236, 1395,   63883,  7, }, // Atlantic/Faroe
    {  236, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  236, 7272,   63890, 19, }, // Atlantic/South_Georgia
    {  236, 6821,     453, 10, }, // Atlantic/St_Helena
    {  236, 2300,    5793,  4, }, // Europe/Athens
    {  236, 6336,    7942,  7, }, // Europe/Belgrade
    {  236, 2167,   57529,  9, }, // Europe/Brussels
    {  236, 3928,   26897,  8, }, // Europe/Bucharest
    {  236, 6289,   63909, 13, }, // Europe/Copenhagen
    {  236, 2527,    8030,  4, }, // Europe/Kyiv
    {  236, 3213,    4381,  8, }, // Europe/Lisbon
    {  236, 6365,   63922, 10, }, // Europe/Luxembourg
    {  236, 6410,    5926,  6, }, // Europe/Monaco
    {  236, 3537,   22748,  6, }, // Europe/Moscow
    {  236, 6275,    8083,  4, }, // Europe/Prague
    {  236, 6324,    5946,  3, }, // Europe/Rome
    {  236, 6523,   63932,  9, }, // Europe/Stockholm
    {  236, 6567,   63941,  9, }, // Europe/Vatican
    {  236, 6231,   57383,  4, }, // Europe/Vienna
    {  236, 3190,   63950,  8, }, // Europe/Warsaw
    {  236, 6540,    4459,  6, }, // Europe/Zurich
    {  236, 7050,   63958, 13, }, // Indian/Maldives
    {  236, 4069,   63971,  8, }, // Indian/Mauritius
    {  236, 7211,     467,  7, }, // Indian/Reunion
    {  236, 3157,     474,  5, }, // Pacific/Chuuk
    {  236, 2723,    4493,  8, }, // Pacific/Honolulu
    {  236, 3049,    4501,  9, }, // Pacific/Kanton
    {  236, 3096,     479,  7, }, // Pacific/Pohnpei
    {  238,   14,       0,  6, }, // Africa/Asmara Filipino/Latin/Philippines
    {  238, 4435,       6,  8, }, // Africa/Sao_Tome
    {  238, 4252,      14,  8, }, // America/Asuncion
    {  238,  237,      22,  8, }, // America/Atikokan
    {  238, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  238, 3868,   33182,  6, }, // America/Cancun
    {  238, 7774,   63979, 17, }, // America/Ciudad_Juarez
    {  238, 5723,      66,  7, }, // America/Curacao
    {  238,  481,      73, 13, }, // America/Indiana/Knox
    {  238, 7961,      86, 16, }, // America/Indiana/Marengo
    {  238, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  238, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  238, 8038,     139, 14, }, // America/Indiana/Vevay
    {  238, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  238, 8086,     171, 16, }, // America/Indiana/Winamac
    {  238, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  238, 5932,     207, 22, }, // America/Lower_Princes
    {  238, 8220,    7008,  6, }, // America/Merida
    {  238, 2949,   63996, 17, }, // America/Mexico_City
    {  238, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  238, 8329,     255, 20, }, // America/North_Dakota/Center
    {  238, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  238,  393,     298,  4, }, // America/Nuuk
    {  238, 5954,   64013, 18, }, // America/Port_of_Spain
    {  238, 8424,   64031, 25, }, // America/Rankin_Inlet
    {  238, 6899,     302, 16, }, // America/Scoresbysund
    {  238, 8508,     318, 14, }, // America/St_Barthelemy
    {  238, 2061,     332, 10, }, // America/St_Johns
    {  238, 5863,     342,  9, }, // America/St_Kitts
    {  238, 5880,     351,  9, }, // America/St_Lucia
    {  238,  855,     360, 10, }, // America/St_Thomas
    {  238, 5913,     370, 11, }, // America/St_Vincent
    {  238, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  238, 1259,   64056, 22, }, // Asia/Ho_Chi_Minh
    {  238, 1185,     413,  9, }, // Asia/Kathmandu
    {  238,  992,     422,  7, }, // Asia/Kolkata
    {  238, 1211,    4203,  5, }, // Asia/Macau
    {  238, 8708,     434,  8, }, // Asia/Qostanay
    {  238, 1235,     442,  6, }, // Asia/Yangon
    {  238, 1395,     448,  5, }, // Atlantic/Faroe
    {  238, 6821,     453, 10, }, // Atlantic/St_Helena
    {  238, 2527,    8030,  4, }, // Europe/Kyiv
    {  238, 7211,     467,  7, }, // Indian/Reunion
    {  238, 3157,     474,  5, }, // Pacific/Chuuk
    {  238, 2723,    4493,  8, }, // Pacific/Honolulu
    {  238, 3049,    4501,  9, }, // Pacific/Kanton
    {  238, 3096,     479,  7, }, // Pacific/Pohnpei
    {  239, 5059,    3784, 11, }, // Africa/Addis_Ababa Finnish/Latin/Finland
    {  239, 7420,   43956,  5, }, // Africa/Algiers
    {  239,   14,       0,  6, }, // Africa/Asmara
    {  239, 2320,    4572,  5, }, // Africa/Cairo
    {  239, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  239, 4524,   26372,  7, }, // Africa/Khartoum
    {  239, 6869,   20300,  4, }, // Africa/Lome
    {  239, 5271,   43996,  9, }, // Africa/Ndjamena
    {  239, 4435,       6,  8, }, // Africa/Sao_Tome
    {  239, 4554,   26427,  9, }, // America/Araguaina
    {  239,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  239, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  239, 4252,      14,  8, }, // America/Asuncion
    {  239,  237,      22,  8, }, // America/Atikokan
    {  239, 7623,      30, 17, }, // America/Bahia_Banderas
    {  239, 4361,   20389,  6, }, // America/Bogota
    {  239, 3868,      47,  6, }, // America/Cancun
    {  239, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  239, 3791,   26442,  6, }, // America/Cuiaba
    {  239, 5723,      66,  7, }, // America/Curacao
    {  239, 7869,   26448,  8, }, // America/Eirunepe
    {  239, 2281,   63392,  7, }, // America/Havana
    {  239,  481,      73, 13, }, // America/Indiana/Knox
    {  239, 7961,      86, 16, }, // America/Indiana/Marengo
    {  239, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  239, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  239, 8038,     139, 14, }, // America/Indiana/Vevay
    {  239, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  239, 8086,     171, 16, }, // America/Indiana/Winamac
    {  239, 2799,    3877,  7, }, // America/Jamaica
    {  239, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  239, 5932,     207, 22, }, // America/Lower_Princes
    {  239, 8153,   26456,  6, }, // America/Maceio
    {  239, 2917,   26471,  8, }, // America/Mazatlan
    {  239, 8220,     229,  6, }, // America/Merida
    {  239, 2949,   57459, 16, }, // America/Mexico_City
    {  239, 8301,   64078, 22, }, // America/North_Dakota/Beulah
    {  239, 8329,   64100, 22, }, // America/North_Dakota/Center
    {  239, 8357,   64122, 25, }, // America/North_Dakota/New_Salem
    {  239,  393,   35796,  7, }, // America/Nuuk
    {  239, 8477,   26572,  8, }, // America/Santarem
    {  239, 2201,   64147, 17, }, // America/Santiago
    {  239, 1878,   26580,  9, }, // America/Sao_Paulo
    {  239, 6899,   44149, 12, }, // America/Scoresbysund
    {  239, 8508,   26589, 16, }, // America/St_Barthelemy
    {  239, 2061,     332, 10, }, // America/St_Johns
    {  239, 5863,   26617, 11, }, // America/St_Kitts
    {  239, 5880,   35832, 11, }, // America/St_Lucia
    {  239,  855,   44161, 12, }, // America/St_Thomas
    {  239, 5913,   26651, 13, }, // America/St_Vincent
    {  239, 5756,   26664,  7, }, // America/Thule
    {  239, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  239, 8568,   64164, 15, }, // Antarctica/Macquarie
    {  239, 5523,   64179,  5, }, // Asia/Aqtau
    {  239, 5368,   57073,  6, }, // Asia/Aqtobe
    {  239,  964,   57483,  7, }, // Asia/Ashgabat
    {  239, 8624,   58649,  6, }, // Asia/Atyrau
    {  239, 3607,    4044,  6, }, // Asia/Baghdad
    {  239, 3778,   33323,  6, }, // Asia/Bishkek
    {  239, 4601,   63491,  5, }, // Asia/Chita
    {  239, 4540,   64184,  8, }, // Asia/Damascus
    {  239, 6159,   33336,  7, }, // Asia/Dushanbe
    {  239, 1259,   64192, 22, }, // Asia/Ho_Chi_Minh
    {  239, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  239, 4316,   63518,  9, }, // Asia/Kamchatka
    {  239, 1185,     413,  9, }, // Asia/Kathmandu
    {  239, 8661,   64214,  7, }, // Asia/Khandyga
    {  239,  992,   64221,  8, }, // Asia/Kolkata
    {  239, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  239, 1211,     429,  5, }, // Asia/Macau
    {  239, 6920,   35935,  6, }, // Asia/Muscat
    {  239, 2554,    5620,  7, }, // Asia/Nicosia
    {  239, 7145,   57921,  6, }, // Asia/Oral
    {  239, 4214,   33430,  9, }, // Asia/Pyongyang
    {  239, 8708,   64229,  8, }, // Asia/Qostanay
    {  239, 4269,   22401,  9, }, // Asia/Qyzylorda
    {  239, 3584,    7669,  4, }, // Asia/Riyadh
    {  239, 4408,   33444,  7, }, // Asia/Sakhalin
    {  239, 3279,   57960,  4, }, // Asia/Seoul
    {  239, 4781,   33457,  7, }, // Asia/Tashkent
    {  239, 2772,    7724,  7, }, // Asia/Tehran
    {  239, 2821,    4238,  5, }, // Asia/Tokyo
    {  239, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  239, 1159,   26801,  6, }, // Asia/Urumqi
    {  239, 4828,    7755,  7, }, // Asia/Yakutsk
    {  239, 1235,     442,  6, }, // Asia/Yangon
    {  239, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  239, 3747,    7780,  7, }, // Asia/Yerevan
    {  239, 3663,   64237,  6, }, // Atlantic/Azores
    {  239, 6628,   64243, 14, }, // Atlantic/Canary
    {  239, 3727,   57248,  9, }, // Atlantic/Cape_Verde
    {  239, 1395,   64257,  9, }, // Atlantic/Faroe
    {  239, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  239, 7272,   64266, 13, }, // Atlantic/South_Georgia
    {  239, 6821,   36066, 12, }, // Atlantic/St_Helena
    {  239, 3620,   33512,  8, }, // Europe/Astrakhan
    {  239, 2300,   63659,  6, }, // Europe/Athens
    {  239, 6336,   26883,  7, }, // Europe/Belgrade
    {  239, 4757,   64279,  8, }, // Europe/Berlin
    {  239, 2167,   64287,  7, }, // Europe/Brussels
    {  239, 3928,   26897,  8, }, // Europe/Bucharest
    {  239, 8772,   26905,  8, }, // Europe/Busingen
    {  239, 2583,   64294,  8, }, // Europe/Chisinau
    {  239, 6289,   64302, 12, }, // Europe/Copenhagen
    {  239, 8804,   64314,  8, }, // Europe/Isle_of_Man
    {  239, 2527,   64322,  5, }, // Europe/Kyiv
    {  239, 3213,    4381,  8, }, // Europe/Lisbon
    {  239, 2501,   64327,  6, }, // Europe/London
    {  239, 6365,    4395,  9, }, // Europe/Luxembourg
    {  239, 6582,   63713, 13, }, // Europe/Mariehamn
    {  239, 3537,   64333,  7, }, // Europe/Moscow
    {  239, 4284,   64340,  7, }, // Europe/Paris
    {  239, 6275,   36153,  5, }, // Europe/Prague
    {  239, 8837,   64347,  5, }, // Europe/Riga
    {  239, 6324,   63736,  5, }, // Europe/Rome
    {  239, 6523,   64352,  8, }, // Europe/Stockholm
    {  239, 8867,   64360,  8, }, // Europe/Tallinn
    {  239, 6202,   21095,  6, }, // Europe/Tirane
    {  239, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  239, 6567,   64368,  9, }, // Europe/Vatican
    {  239, 6231,   57383,  4, }, // Europe/Vienna
    {  239, 8899,   64377,  5, }, // Europe/Vilnius
    {  239, 3190,   64382,  7, }, // Europe/Warsaw
    {  239, 6540,    4459,  6, }, // Europe/Zurich
    {  239, 6046,   64389, 10, }, // Indian/Christmas
    {  239, 6063,   64399, 12, }, // Indian/Cocos
    {  239, 5029,   64411,  7, }, // Indian/Comoro
    {  239, 6661,   64418, 15, }, // Indian/Kerguelen
    {  239, 7260,   26983,  4, }, // Indian/Mahe
    {  239, 7050,   64433, 10, }, // Indian/Maldives
    {  239, 7211,     467,  7, }, // Indian/Reunion
    {  239, 3015,   64443, 13, }, // Pacific/Chatham
    {  239, 3157,     474,  5, }, // Pacific/Chuuk
    {  239, 2237,   64456, 13, }, // Pacific/Easter
    {  239, 3902,   33704,  5, }, // Pacific/Fiji
    {  239, 6696,   64469, 13, }, // Pacific/Gambier
    {  239, 2723,    4493,  8, }, // Pacific/Honolulu
    {  239, 3049,    4501,  9, }, // Pacific/Kanton
    {  239, 4051,   64482, 15, }, // Pacific/Marquesas
    {  239, 8914,   64497, 12, }, // Pacific/Midway
    {  239, 7098,    4510,  6, }, // Pacific/Noumea
    {  239, 3096,     479,  7, }, // Pacific/Pohnpei
    {  240, 5059,   64509, 11, }, // Africa/Addis_Ababa French/Latin/France
    {  240, 7420,   43956,  5, }, // Africa/Algiers
    {  240,   14,       0,  6, }, // Africa/Asmara
    {  240, 2320,   64520,  8, }, // Africa/Cairo
    {  240, 5166,   64528,  8, }, // Africa/El_Aaiun
    {  240, 6869,   20300,  4, }, // Africa/Lome
    {  240, 5113,   43978, 10, }, // Africa/Mogadishu
    {  240, 5271,   43996,  9, }, // Africa/Ndjamena
    {  240, 4435,       6,  8, }, // Africa/Sao_Tome
    {  240, 2866,   64536, 15, }, // Africa/Tripoli
    {  240, 4554,   26427,  9, }, // America/Araguaina
    {  240,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  240, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {  240, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  240, 7597,   64551,  7, }, // America/Argentina/Ushuaia
    {  240, 4252,      14,  8, }, // America/Asuncion
    {  240,  237,      22,  8, }, // America/Atikokan
    {  240, 7623,   64558, 17, }, // America/Bahia_Banderas
    {  240, 5654,   64575, 10, }, // America/Barbados
    {  240, 7646,   35764,  5, }, // America/Belem
    {  240, 3868,      47,  6, }, // America/Cancun
    {  240, 5496,   64585,  7, }, // America/Cayman
    {  240, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  240, 3791,   26442,  6, }, // America/Cuiaba
    {  240, 5723,      66,  7, }, // America/Curacao
    {  240, 3465,   64592,  7, }, // America/Detroit
    {  240, 5739,   64599,  9, }, // America/Dominica
    {  240, 7869,   26448,  8, }, // America/Eirunepe
    {  240, 5770,   64608,  7, }, // America/Grenada
    {  240, 2281,   64615,  9, }, // America/Havana
    {  240,  481,   64624, 14, }, // America/Indiana/Knox
    {  240, 7961,   64638, 17, }, // America/Indiana/Marengo
    {  240, 7985,   64655, 20, }, // America/Indiana/Petersburg
    {  240, 8012,   64675, 19, }, // America/Indiana/Tell_City
    {  240, 8038,   64694, 15, }, // America/Indiana/Vevay
    {  240, 8060,   64709, 19, }, // America/Indiana/Vincennes
    {  240, 8086,   64728, 17, }, // America/Indiana/Winamac
    {  240, 2799,   64745,  8, }, // America/Jamaica
    {  240, 8125,   64753, 21, }, // America/Kentucky/Monticello
    {  240, 5932,     207, 22, }, // America/Lower_Princes
    {  240, 8153,   26456,  6, }, // America/Maceio
    {  240, 1908,   64774,  6, }, // America/Manaus
    {  240, 2917,   26471,  8, }, // America/Mazatlan
    {  240, 8220,     229,  6, }, // America/Merida
    {  240, 2949,   58630,  6, }, // America/Mexico_City
    {  240, 8301,   64780, 23, }, // America/North_Dakota/Beulah
    {  240, 8329,   64803, 23, }, // America/North_Dakota/Center
    {  240, 8357,   64826, 26, }, // America/North_Dakota/New_Salem
    {  240,  393,     298,  4, }, // America/Nuuk
    {  240, 5954,   64852, 14, }, // America/Port_of_Spain
    {  240, 5843,   64866, 10, }, // America/Puerto_Rico
    {  240, 8477,   26572,  8, }, // America/Santarem
    {  240, 6111,   64876, 14, }, // America/Santo_Domingo
    {  240, 1878,   26580,  9, }, // America/Sao_Paulo
    {  240, 6899,     302, 16, }, // America/Scoresbysund
    {  240, 8508,   26589, 16, }, // America/St_Barthelemy
    {  240, 2061,   64890, 25, }, // America/St_Johns
    {  240, 5863,   64915, 16, }, // America/St_Kitts
    {  240, 5880,   64931, 12, }, // America/St_Lucia
    {  240,  855,   26639, 12, }, // America/St_Thomas
    {  240, 5913,   64943, 13, }, // America/St_Vincent
    {  240, 5756,   64956,  5, }, // America/Thule
    {  240, 6133,   57010, 16, }, // Antarctica/DumontDUrville
    {  240, 7295,   62088,  5, }, // Antarctica/Syowa
    {  240, 5395,   57475,  8, }, // Asia/Almaty
    {  240, 5523,   64961,  6, }, // Asia/Aqtau
    {  240, 5368,   64967, 11, }, // Asia/Aqtobe
    {  240,  964,   64978,  8, }, // Asia/Ashgabat
    {  240, 8624,   64986,  7, }, // Asia/Atyrau
    {  240, 3607,    4044,  6, }, // Asia/Baghdad
    {  240, 5534,   64993,  7, }, // Asia/Bahrain
    {  240, 3653,   35866,  5, }, // Asia/Baku
    {  240, 4086,   65000,  8, }, // Asia/Beirut
    {  240, 3778,   65008,  7, }, // Asia/Bishkek
    {  240, 4601,   35877,  6, }, // Asia/Chita
    {  240, 4540,    4083,  5, }, // Asia/Damascus
    {  240, 3596,   65015,  5, }, // Asia/Dubai
    {  240, 6159,   65020,  9, }, // Asia/Dushanbe
    {  240, 8636,   65029, 10, }, // Asia/Famagusta
    {  240, 4795,   65039,  6, }, // Asia/Hebron
    {  240, 1259,   65045, 17, }, // Asia/Ho_Chi_Minh
    {  240, 4184,   65062,  8, }, // Asia/Irkutsk
    {  240, 1290,   65070,  9, }, // Asia/Jerusalem
    {  240, 3560,   35905,  6, }, // Asia/Kabul
    {  240, 4316,   35911, 10, }, // Asia/Kamchatka
    {  240, 1185,   65079,  9, }, // Asia/Kathmandu
    {  240,  992,   35921,  8, }, // Asia/Kolkata
    {  240, 4197,   65088, 11, }, // Asia/Krasnoyarsk
    {  240, 5547,   65099,  6, }, // Asia/Kuwait
    {  240, 1211,     429,  5, }, // Asia/Macau
    {  240, 1349,   65105,  8, }, // Asia/Makassar
    {  240, 7182,   65113,  7, }, // Asia/Manila
    {  240, 6920,   20739,  7, }, // Asia/Muscat
    {  240, 2554,   65120,  7, }, // Asia/Nicosia
    {  240, 4135,   44247, 12, }, // Asia/Novosibirsk
    {  240, 7145,   65127,  7, }, // Asia/Oral
    {  240, 8708,   65134,  8, }, // Asia/Qostanay
    {  240, 4269,   65142,  9, }, // Asia/Qyzylorda
    {  240, 3584,   20756,  5, }, // Asia/Riyadh
    {  240, 4408,   65151,  9, }, // Asia/Sakhalin
    {  240, 7245,   65160, 10, }, // Asia/Samarkand
    {  240, 3279,   65170,  5, }, // Asia/Seoul
    {  240, 3300,   35969,  9, }, // Asia/Singapore
    {  240, 4781,   65175,  8, }, // Asia/Tashkent
    {  240, 3915,   65183,  8, }, // Asia/Tbilisi
    {  240, 2772,   65191,  7, }, // Asia/Tehran
    {  240, 1021,   65198, 11, }, // Asia/Ulaanbaatar
    {  240, 1159,   26801,  6, }, // Asia/Urumqi
    {  240, 4828,   65209,  8, }, // Asia/Yakutsk
    {  240, 1235,   65217,  7, }, // Asia/Yangon
    {  240, 3883,   65224, 13, }, // Asia/Yekaterinburg
    {  240, 3747,   26815,  6, }, // Asia/Yerevan
    {  240, 3663,   44329,  6, }, // Atlantic/Azores
    {  240, 5671,   44335,  8, }, // Atlantic/Bermuda
    {  240, 6628,   65237, 13, }, // Atlantic/Canary
    {  240, 3727,   65250,  8, }, // Atlantic/Cape_Verde
    {  240, 1395,   65258, 10, }, // Atlantic/Faroe
    {  240, 8736,   65268,  6, }, // Atlantic/Madeira
    {  240, 7272,   65274, 14, }, // Atlantic/South_Georgia
    {  240, 6821,   65288, 13, }, // Atlantic/St_Helena
    {  240, 1670,   65301,  8, }, // Australia/Adelaide
    {  240, 6216,   65309,  7, }, // Europe/Andorra
    {  240, 2300,   65316,  7, }, // Europe/Athens
    {  240, 2167,   57529,  9, }, // Europe/Brussels
    {  240, 3928,   20955,  8, }, // Europe/Bucharest
    {  240, 8772,   26905,  8, }, // Europe/Busingen
    {  240, 6289,   20963, 10, }, // Europe/Copenhagen
    {  240, 8788,   26913,  9, }, // Europe/Guernsey
    {  240, 8804,   65323, 10, }, // Europe/Isle_of_Man
    {  240, 2527,    8030,  4, }, // Europe/Kyiv
    {  240, 3213,   65333,  8, }, // Europe/Lisbon
    {  240, 2501,   21034,  7, }, // Europe/London
    {  240, 6397,   65341,  5, }, // Europe/Malta
    {  240, 3537,   44444,  6, }, // Europe/Moscow
    {  240, 6470,   65346, 11, }, // Europe/San_Marino
    {  240, 6202,   21095,  6, }, // Europe/Tirane
    {  240, 8882,   65357, 10, }, // Europe/Ulyanovsk
    {  240, 6567,   65367, 10, }, // Europe/Vatican
    {  240, 6231,   65377,  6, }, // Europe/Vienna
    {  240, 3190,   65383,  8, }, // Europe/Warsaw
    {  240, 5029,   65391,  7, }, // Indian/Comoro
    {  240, 7260,   26983,  4, }, // Indian/Mahe
    {  240, 4069,   65398,  7, }, // Indian/Mauritius
    {  240, 7211,   65405, 10, }, // Indian/Reunion
    {  240, 3157,     474,  5, }, // Pacific/Chuuk
    {  240, 2237,   65415, 13, }, // Pacific/Easter
    {  240, 7360,   27016,  5, }, // Pacific/Efate
    {  240, 3902,    4488,  5, }, // Pacific/Fiji
    {  240, 6678,   21159,  9, }, // Pacific/Galapagos
    {  240, 2723,    4493,  8, }, // Pacific/Honolulu
    {  240, 3049,    4501,  9, }, // Pacific/Kanton
    {  240, 4051,   65428,  9, }, // Pacific/Marquesas
    {  240, 7098,    4510,  6, }, // Pacific/Noumea
    {  240, 7155,   65437,  6, }, // Pacific/Palau
    {  240, 3096,     479,  7, }, // Pacific/Pohnpei
    {  247, 5271,   65443,  8, }, // Africa/Ndjamena French/Latin/Canada
    {  247, 2866,   65451, 15, }, // Africa/Tripoli
    {  247, 5654,   65466, 12, }, // America/Barbados
    {  247, 5496,   65478, 12, }, // America/Cayman
    {  247, 8301,   65490, 23, }, // America/North_Dakota/Beulah
    {  247, 8329,   65513, 23, }, // America/North_Dakota/Center
    {  247, 8357,   65536, 26, }, // America/North_Dakota/New_Salem
    {  247, 2061,     332, 10, }, // America/St_Johns
    {  247, 5863,   65562, 26, }, // America/St_Kitts
    {  247,  855,   44161, 12, }, // America/St_Thomas
    {  247, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  247, 1093,   65588,  5, }, // Asia/Dhaka
    {  247, 1317,   65593,  8, }, // Asia/Thimphu
    {  247, 6628,   65601, 13, }, // Atlantic/Canary
    {  247, 1395,   65614, 10, }, // Atlantic/Faroe
    {  247, 8804,   65624, 10, }, // Europe/Isle_of_Man
    {  247, 2501,   21034,  7, }, // Europe/London
    {  247, 6567,   21104,  7, }, // Europe/Vatican
    {  247, 2237,   65634, 13, }, // Pacific/Easter
    {  247, 7155,   65647,  5, }, // Pacific/Palau
    {  286,   14,       0,  6, }, // Africa/Asmara Friulian/Latin/Italy
    {  286, 4435,       6,  8, }, // Africa/Sao_Tome
    {  286, 4252,      14,  8, }, // America/Asuncion
    {  286,  237,      22,  8, }, // America/Atikokan
    {  286, 7623,      30, 17, }, // America/Bahia_Banderas
    {  286, 3868,      47,  6, }, // America/Cancun
    {  286, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  286, 5723,      66,  7, }, // America/Curacao
    {  286,  481,      73, 13, }, // America/Indiana/Knox
    {  286, 7961,      86, 16, }, // America/Indiana/Marengo
    {  286, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  286, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  286, 8038,     139, 14, }, // America/Indiana/Vevay
    {  286, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  286, 8086,     171, 16, }, // America/Indiana/Winamac
    {  286, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  286, 5932,     207, 22, }, // America/Lower_Princes
    {  286, 8220,     229,  6, }, // America/Merida
    {  286, 2379,   65652, 10, }, // America/New_York
    {  286, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  286, 8329,     255, 20, }, // America/North_Dakota/Center
    {  286, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  286,  393,     298,  4, }, // America/Nuuk
    {  286, 1878,   65662, 20, }, // America/Sao_Paulo
    {  286, 6899,     302, 16, }, // America/Scoresbysund
    {  286, 8508,     318, 14, }, // America/St_Barthelemy
    {  286, 2061,     332, 10, }, // America/St_Johns
    {  286, 5863,     342,  9, }, // America/St_Kitts
    {  286, 5880,     351,  9, }, // America/St_Lucia
    {  286,  855,     360, 10, }, // America/St_Thomas
    {  286, 5913,     370, 11, }, // America/St_Vincent
    {  286, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  286, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  286, 1185,     413,  9, }, // Asia/Kathmandu
    {  286,  992,     422,  7, }, // Asia/Kolkata
    {  286, 1211,     429,  5, }, // Asia/Macau
    {  286, 1235,     442,  6, }, // Asia/Yangon
    {  286, 3663,   65682,  6, }, // Atlantic/Azores
    {  286, 6628,   65688,  8, }, // Atlantic/Canary
    {  286, 1395,     448,  5, }, // Atlantic/Faroe
    {  286, 6821,     453, 10, }, // Atlantic/St_Helena
    {  286, 6336,   65696,  7, }, // Europe/Belgrade
    {  286, 2527,     463,  4, }, // Europe/Kyiv
    {  286, 3213,   65703,  7, }, // Europe/Lisbon
    {  286, 6506,   65710,  7, }, // Europe/Ljubljana
    {  286, 6365,   65717, 10, }, // Europe/Luxembourg
    {  286, 6397,   65341,  5, }, // Europe/Malta
    {  286, 3537,   65727,  6, }, // Europe/Moscow
    {  286, 6275,   65733,  6, }, // Europe/Prague
    {  286, 6470,   65739,  9, }, // Europe/San_Marino
    {  286, 6231,   65748,  5, }, // Europe/Vienna
    {  286, 7211,     467,  7, }, // Indian/Reunion
    {  286, 3157,     474,  5, }, // Pacific/Chuuk
    {  286, 3096,     479,  7, }, // Pacific/Pohnpei
    {  287,   14,       0,  6, }, // Africa/Asmara Fulah/Latin/Senegal
    {  287, 4435,       6,  8, }, // Africa/Sao_Tome
    {  287, 4252,      14,  8, }, // America/Asuncion
    {  287,  237,      22,  8, }, // America/Atikokan
    {  287, 7623,      30, 17, }, // America/Bahia_Banderas
    {  287, 3868,      47,  6, }, // America/Cancun
    {  287, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  287, 5723,      66,  7, }, // America/Curacao
    {  287,  481,      73, 13, }, // America/Indiana/Knox
    {  287, 7961,      86, 16, }, // America/Indiana/Marengo
    {  287, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  287, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  287, 8038,     139, 14, }, // America/Indiana/Vevay
    {  287, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  287, 8086,     171, 16, }, // America/Indiana/Winamac
    {  287, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  287, 5932,     207, 22, }, // America/Lower_Princes
    {  287, 8220,     229,  6, }, // America/Merida
    {  287, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  287, 8329,     255, 20, }, // America/North_Dakota/Center
    {  287, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  287,  393,     298,  4, }, // America/Nuuk
    {  287, 6899,     302, 16, }, // America/Scoresbysund
    {  287, 8508,     318, 14, }, // America/St_Barthelemy
    {  287, 2061,     332, 10, }, // America/St_Johns
    {  287, 5863,     342,  9, }, // America/St_Kitts
    {  287, 5880,     351,  9, }, // America/St_Lucia
    {  287,  855,     360, 10, }, // America/St_Thomas
    {  287, 5913,     370, 11, }, // America/St_Vincent
    {  287, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  287, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  287, 1185,     413,  9, }, // Asia/Kathmandu
    {  287,  992,     422,  7, }, // Asia/Kolkata
    {  287, 1211,     429,  5, }, // Asia/Macau
    {  287, 1235,     442,  6, }, // Asia/Yangon
    {  287, 1395,     448,  5, }, // Atlantic/Faroe
    {  287, 6821,     453, 10, }, // Atlantic/St_Helena
    {  287, 2527,     463,  4, }, // Europe/Kyiv
    {  287, 7211,     467,  7, }, // Indian/Reunion
    {  287, 3157,     474,  5, }, // Pacific/Chuuk
    {  287, 3096,     479,  7, }, // Pacific/Pohnpei
    {  293, 6788,   65753, 12, }, // Africa/Abidjan Fulah/Adlam/Guinea
    {  293, 6760,   65765, 10, }, // Africa/Accra
    {  293, 5059,   65775, 18, }, // Africa/Addis_Ababa
    {  293, 7420,   65793, 14, }, // Africa/Algiers
    {  293,   14,   65807, 12, }, // Africa/Asmara
    {  293,   44,   65819, 14, }, // Africa/Bamako
    {  293, 5257,   65833, 10, }, // Africa/Bangui
    {  293, 6746,   65843, 12, }, // Africa/Banjul
    {  293, 7435,   65855, 12, }, // Africa/Bissau
    {  293, 4957,   65867, 18, }, // Africa/Blantyre
    {  293, 5287,   65885, 16, }, // Africa/Brazzaville
    {  293, 4922,   65901, 18, }, // Africa/Bujumbura
    {  293, 2320,   65919, 10, }, // Africa/Cairo
    {  293, 4117,   65929, 26, }, // Africa/Casablanca
    {  293, 7449,   65955, 10, }, // Africa/Ceuta
    {  293, 6773,   65965, 18, }, // Africa/Conakry
    {  293, 6840,   65983, 12, }, // Africa/Dakar
    {  293, 5130,   65995, 25, }, // Africa/Dar_es_Salaam
    {  293, 5043,   66020, 14, }, // Africa/Djibouti
    {  293, 5243,   66034, 12, }, // Africa/Douala
    {  293, 5166,   66046, 14, }, // Africa/El_Aaiun
    {  293, 6853,   66060, 12, }, // Africa/Freetown
    {  293, 4906,   66072, 16, }, // Africa/Gaborone
    {  293, 5015,   66088, 12, }, // Africa/Harare
    {  293, 4479,   66100, 22, }, // Africa/Johannesburg
    {  293, 4499,   66122,  8, }, // Africa/Juba
    {  293, 5151,   66130, 14, }, // Africa/Kampala
    {  293, 4524,   66144, 14, }, // Africa/Khartoum
    {  293, 4973,   66158, 12, }, // Africa/Kigali
    {  293, 5306,   66170, 14, }, // Africa/Kinshasa
    {  293, 4744,   66184, 12, }, // Africa/Lagos
    {  293, 5336,   66196, 20, }, // Africa/Libreville
    {  293, 6869,   66216,  8, }, // Africa/Lome
    {  293, 5211,   66224, 16, }, // Africa/Luanda
    {  293, 4939,   66240, 18, }, // Africa/Lubumbashi
    {  293, 5001,   66258, 12, }, // Africa/Lusaka
    {  293, 5322,   66270, 14, }, // Africa/Malabo
    {  293, 4987,   66284, 12, }, // Africa/Maputo
    {  293, 5197,   66296, 12, }, // Africa/Maseru
    {  293, 5182,   66308, 18, }, // Africa/Mbabane
    {  293, 5113,   66326, 16, }, // Africa/Mogadishu
    {  293, 7034,   66342, 16, }, // Africa/Monrovia
    {  293, 3853,   66358, 14, }, // Africa/Nairobi
    {  293, 5271,   66372, 16, }, // Africa/Ndjamena
    {  293, 5354,   66388, 12, }, // Africa/Niamey
    {  293, 6803,   66400, 20, }, // Africa/Nouakchott
    {  293, 6727,   66420, 16, }, // Africa/Ouagadougou
    {  293, 5225,   66436, 21, }, // Africa/Porto-Novo
    {  293, 4435,   66457, 19, }, // Africa/Sao_Tome
    {  293, 2866,   66476, 14, }, // Africa/Tripoli
    {  293, 6554,   66490, 12, }, // Africa/Tunis
    {  293, 4152,   66502, 14, }, // Africa/Windhoek
    {  293,  132,   66516,  8, }, // America/Adak
    {  293, 3346,   66524, 18, }, // America/Anchorage
    {  293, 5607,   66542, 14, }, // America/Anguilla
    {  293, 5624,   66556, 12, }, // America/Antigua
    {  293, 4554,   66568, 18, }, // America/Araguaina
    {  293,  166,   66586, 29, }, // America/Argentina/Buenos_Aires
    {  293,   91,   66615, 20, }, // America/Argentina/Catamarca
    {  293,  270,   66635, 16, }, // America/Argentina/Cordoba
    {  293,  441,   66651, 10, }, // America/Argentina/Jujuy
    {  293, 7462,   66661, 17, }, // America/Argentina/La_Rioja
    {  293,  565,   66678, 16, }, // America/Argentina/Mendoza
    {  293, 7489,   66694, 21, }, // America/Argentina/Rio_Gallegos
    {  293, 7520,   66715, 10, }, // America/Argentina/Salta
    {  293, 7544,   66725, 19, }, // America/Argentina/San_Juan
    {  293, 5580,   66744, 19, }, // America/Argentina/San_Luis
    {  293, 7571,   66763, 16, }, // America/Argentina/Tucuman
    {  293, 7597,   66779, 14, }, // America/Argentina/Ushuaia
    {  293, 5640,   66793, 12, }, // America/Aruba
    {  293, 4252,   66805, 18, }, // America/Asuncion
    {  293,  237,   66823, 18, }, // America/Atikokan
    {  293, 3679,   66841, 12, }, // America/Bahia
    {  293, 7623,   66853, 33, }, // America/Bahia_Banderas
    {  293, 5654,   66886, 18, }, // America/Barbados
    {  293, 7646,   66904, 10, }, // America/Belem
    {  293, 5407,   66914, 12, }, // America/Belize
    {  293, 7660,   66926, 25, }, // America/Blanc-Sablon
    {  293, 7681,   66951, 19, }, // America/Boa_Vista
    {  293, 4361,   66970, 12, }, // America/Bogota
    {  293, 7699,   66982, 12, }, // America/Boise
    {  293, 7713,   66994, 25, }, // America/Cambridge_Bay
    {  293, 7735,   67019, 23, }, // America/Campo_Grande
    {  293, 3868,   67042, 14, }, // America/Cancun
    {  293, 4694,   67056, 14, }, // America/Caracas
    {  293, 4345,   67070, 12, }, // America/Cayenne
    {  293, 5496,   67082, 14, }, // America/Cayman
    {  293, 2260,   67096, 16, }, // America/Chicago
    {  293, 7756,   67112, 16, }, // America/Chihuahua
    {  293, 5422,   67128, 21, }, // America/Costa_Rica
    {  293, 7796,   67149, 14, }, // America/Creston
    {  293, 3791,   67163, 14, }, // America/Cuiaba
    {  293, 5723,   67177, 18, }, // America/Curacao
    {  293, 7812,   67195, 26, }, // America/Danmarkshavn
    {  293, 7833,   67221, 12, }, // America/Dawson
    {  293, 7848,   67233, 21, }, // America/Dawson_Creek
    {  293,  805,   67254, 12, }, // America/Denver
    {  293, 3465,   67266, 14, }, // America/Detroit
    {  293, 5739,   67280, 18, }, // America/Dominica
    {  293,  893,   67298, 16, }, // America/Edmonton
    {  293, 7869,   67314, 14, }, // America/Eirunepe
    {  293, 5441,   67328, 23, }, // America/El_Salvador
    {  293, 7886,   67351, 22, }, // America/Fort_Nelson
    {  293, 7906,   67373, 20, }, // America/Fortaleza
    {  293, 7924,   67393, 15, }, // America/Glace_Bay
    {  293, 6881,   67408, 15, }, // America/Goose_Bay
    {  293, 4612,   67423, 19, }, // America/Grand_Turk
    {  293, 5770,   67442, 16, }, // America/Grenada
    {  293, 5786,   67458, 24, }, // America/Guadeloupe
    {  293, 3760,   67482, 20, }, // America/Guatemala
    {  293, 4841,   67502, 14, }, // America/Guayaquil
    {  293, 6932,   67516, 14, }, // America/Guyana
    {  293, 1939,   67530, 18, }, // America/Halifax
    {  293, 2281,   67548, 14, }, // America/Havana
    {  293, 7942,   67562, 24, }, // America/Hermosillo
    {  293,  348,   67586, 28, }, // America/Indiana/Indianapolis
    {  293,  481,   67614, 28, }, // America/Indiana/Knox
    {  293, 7961,   67642, 34, }, // America/Indiana/Marengo
    {  293, 7985,   67676, 40, }, // America/Indiana/Petersburg
    {  293, 8012,   67716, 37, }, // America/Indiana/Tell_City
    {  293, 8038,   67753, 32, }, // America/Indiana/Vevay
    {  293, 8060,   67785, 34, }, // America/Indiana/Vincennes
    {  293, 8086,   67819, 34, }, // America/Indiana/Winamac
    {  293, 8110,   67853, 12, }, // America/Inuvik
    {  293,  660,   67865, 16, }, // America/Iqaluit
    {  293, 2799,   67881, 16, }, // America/Jamaica
    {  293, 5380,   67897, 12, }, // America/Juneau
    {  293,  521,   67909, 12, }, // America/Kentucky/Louisville
    {  293, 8125,   67921, 40, }, // America/Kentucky/Monticello
    {  293, 5704,   67961, 20, }, // America/Kralendijk
    {  293, 4376,   67981, 13, }, // America/La_Paz
    {  293, 7169,   67994, 10, }, // America/Lima
    {  293, 3239,   68004, 21, }, // America/Los_Angeles
    {  293, 5932,   68025, 36, }, // America/Lower_Princes
    {  293, 8153,   68061, 12, }, // America/Maceio
    {  293, 8168,   68073, 14, }, // America/Managua
    {  293, 1908,   68087, 12, }, // America/Manaus
    {  293, 5897,   68099, 14, }, // America/Marigot
    {  293, 5805,   68113, 16, }, // America/Martinique
    {  293, 8184,   68129, 20, }, // America/Matamoros
    {  293, 2917,   68149, 16, }, // America/Mazatlan
    {  293, 8202,   68165, 16, }, // America/Menominee
    {  293, 8220,   68181, 12, }, // America/Merida
    {  293, 8235,   68193, 20, }, // America/Metlakatla
    {  293, 2949,   68213, 27, }, // America/Mexico_City
    {  293, 4391,   68240, 14, }, // America/Miquelon
    {  293, 8254,   68254, 16, }, // America/Moncton
    {  293, 8270,   68270, 18, }, // America/Monterrey
    {  293, 4098,   68288, 24, }, // America/Montevideo
    {  293, 5824,   68312, 20, }, // America/Montserrat
    {  293, 5481,   68332, 10, }, // America/Nassau
    {  293, 2379,   68342, 15, }, // America/New_York
    {  293, 8288,   68357, 10, }, // America/Nome
    {  293, 1850,   68367, 14, }, // America/Noronha
    {  293, 8301,   68381, 33, }, // America/North_Dakota/Beulah
    {  293, 8329,   68414, 37, }, // America/North_Dakota/Center
    {  293, 8357,   68451, 42, }, // America/North_Dakota/New_Salem
    {  293,  393,   68493,  8, }, // America/Nuuk
    {  293, 8388,   68501, 14, }, // America/Ojinaga
    {  293, 2356,   68515, 16, }, // America/Panama
    {  293, 6173,   68531, 20, }, // America/Paramaribo
    {  293, 2973,   68551, 12, }, // America/Phoenix
    {  293, 3945,   68563, 22, }, // America/Port-au-Prince
    {  293, 5954,   68585, 26, }, // America/Port_of_Spain
    {  293, 8404,   68611, 23, }, // America/Porto_Velho
    {  293, 5843,   68634, 19, }, // America/Puerto_Rico
    {  293, 4030,   68653, 23, }, // America/Punta_Arenas
    {  293, 8424,   68676, 23, }, // America/Rankin_Inlet
    {  293, 8445,   68699, 12, }, // America/Recife
    {  293, 1995,   68711, 14, }, // America/Regina
    {  293, 8460,   68725, 16, }, // America/Resolute
    {  293,  695,   68741, 21, }, // America/Rio_Branco
    {  293, 8477,   68762, 18, }, // America/Santarem
    {  293, 2201,   68780, 20, }, // America/Santiago
    {  293, 6111,   68800, 27, }, // America/Santo_Domingo
    {  293, 1878,   68827, 19, }, // America/Sao_Paulo
    {  293, 6899,   68846, 30, }, // America/Scoresbysund
    {  293, 8494,   68876, 10, }, // America/Sitka
    {  293, 8508,   68886, 27, }, // America/St_Barthelemy
    {  293, 2061,   68913, 19, }, // America/St_Johns
    {  293, 5863,   68932, 21, }, // America/St_Kitts
    {  293, 5880,   68953, 19, }, // America/St_Lucia
    {  293,  855,   68972, 21, }, // America/St_Thomas
    {  293, 5913,   68993, 23, }, // America/St_Vincent
    {  293, 8530,   69016, 23, }, // America/Swift_Current
    {  293, 5461,   69039, 22, }, // America/Tegucigalpa
    {  293, 5756,   69061,  8, }, // America/Thule
    {  293,  313,   69069, 18, }, // America/Tijuana
    {  293,  608,   69087, 16, }, // America/Toronto
    {  293, 5688,   69103, 16, }, // America/Tortola
    {  293, 2093,   69119, 16, }, // America/Vancouver
    {  293, 2144,   69135, 18, }, // America/Whitehorse
    {  293,  734,   69153, 16, }, // America/Winnipeg
    {  293, 8552,   69169, 14, }, // America/Yakutat
    {  293, 6001,   69183, 10, }, // Antarctica/Casey
    {  293, 6094,   69193, 12, }, // Antarctica/Davis
    {  293, 6133,   69205, 27, }, // Antarctica/DumontDUrville
    {  293, 8568,   69232, 14, }, // Antarctica/Macquarie
    {  293, 7066,   69246, 12, }, // Antarctica/Mawson
    {  293, 7113,   69258, 18, }, // Antarctica/McMurdo
    {  293, 8589,   69276, 14, }, // Antarctica/Palmer
    {  293, 7226,   69290, 12, }, // Antarctica/Rothera
    {  293, 7295,   69302, 14, }, // Antarctica/Syowa
    {  293, 8607,   69316, 12, }, // Antarctica/Troll
    {  293, 7374,   69328, 12, }, // Antarctica/Vostok
    {  293, 1429,   69340, 24, }, // Arctic/Longyearbyen
    {  293, 5570,   69364,  8, }, // Asia/Aden
    {  293, 5395,   69372, 14, }, // Asia/Almaty
    {  293, 3968,   69386, 12, }, // Asia/Amman
    {  293, 5511,   69398, 14, }, // Asia/Anadyr
    {  293, 5523,   69412, 14, }, // Asia/Aqtau
    {  293, 5368,   69426, 14, }, // Asia/Aqtobe
    {  293,  964,   69440, 16, }, // Asia/Ashgabat
    {  293, 8624,   69456, 14, }, // Asia/Atyrau
    {  293, 3607,   69470, 16, }, // Asia/Baghdad
    {  293, 5534,   69486, 16, }, // Asia/Bahrain
    {  293, 3653,   69502, 10, }, // Asia/Baku
    {  293, 4466,   69512, 16, }, // Asia/Bangkok
    {  293, 3571,   69528, 16, }, // Asia/Barnaul
    {  293, 4086,   69544, 16, }, // Asia/Beirut
    {  293, 3778,   69560, 16, }, // Asia/Bishkek
    {  293, 5989,   69576, 12, }, // Asia/Brunei
    {  293, 4601,   69588, 10, }, // Asia/Chita
    {  293, 4511,   69598, 14, }, // Asia/Colombo
    {  293, 4540,   69612, 14, }, // Asia/Damascus
    {  293, 1093,   65983, 10, }, // Asia/Dhaka
    {  293, 6192,   69626, 10, }, // Asia/Dili
    {  293, 3596,   69636, 10, }, // Asia/Dubai
    {  293, 6159,   69646, 14, }, // Asia/Dushanbe
    {  293, 8636,   69660, 18, }, // Asia/Famagusta
    {  293, 8651,   69678, 10, }, // Asia/Gaza
    {  293, 4795,   69688, 14, }, // Asia/Hebron
    {  293, 1259,   69702, 31, }, // Asia/Ho_Chi_Minh
    {  293, 2704,   69733, 17, }, // Asia/Hong_Kong
    {  293, 4771,   69750, 10, }, // Asia/Hovd
    {  293, 4184,   69760, 18, }, // Asia/Irkutsk
    {  293, 7006,   69778, 16, }, // Asia/Jakarta
    {  293, 6992,   69794, 16, }, // Asia/Jayapura
    {  293, 1290,   69810, 12, }, // Asia/Jerusalem
    {  293, 3560,   69822, 10, }, // Asia/Kabul
    {  293, 4316,   69832, 18, }, // Asia/Kamchatka
    {  293, 4239,   69850, 14, }, // Asia/Karachi
    {  293, 1185,   69864, 18, }, // Asia/Kathmandu
    {  293, 8661,   69882, 14, }, // Asia/Khandyga
    {  293,  992,   69896, 14, }, // Asia/Kolkata
    {  293, 4197,   69910, 28, }, // Asia/Krasnoyarsk
    {  293, 4859,   69938, 27, }, // Asia/Kuala_Lumpur
    {  293, 5976,   69965, 10, }, // Asia/Kuching
    {  293, 5547,   69975, 14, }, // Asia/Kuwait
    {  293, 1211,   69989, 12, }, // Asia/Macau
    {  293, 4017,   70001, 14, }, // Asia/Magadan
    {  293, 1349,   70015, 16, }, // Asia/Makassar
    {  293, 7182,   70031, 12, }, // Asia/Manila
    {  293, 6920,   70043, 14, }, // Asia/Muscat
    {  293, 2554,   70057, 16, }, // Asia/Nicosia
    {  293, 8675,   70073, 28, }, // Asia/Novokuznetsk
    {  293, 4135,   70101, 24, }, // Asia/Novosibirsk
    {  293, 4229,   70125, 12, }, // Asia/Omsk
    {  293, 7145,   70137,  8, }, // Asia/Oral
    {  293, 6961,   70145, 17, }, // Asia/Phnom_Penh
    {  293, 8693,   70162, 20, }, // Asia/Pontianak
    {  293, 4214,   70182, 16, }, // Asia/Pyongyang
    {  293, 5559,   70198, 10, }, // Asia/Qatar
    {  293, 8708,   70208, 16, }, // Asia/Qostanay
    {  293, 4269,   70224, 18, }, // Asia/Qyzylorda
    {  293, 3584,   70242, 12, }, // Asia/Riyadh
    {  293, 4408,   70254, 16, }, // Asia/Sakhalin
    {  293, 7245,   70270, 20, }, // Asia/Samarkand
    {  293, 3279,   70290, 10, }, // Asia/Seoul
    {  293, 1053,   70300, 12, }, // Asia/Shanghai
    {  293, 3300,   70312, 18, }, // Asia/Singapore
    {  293, 4297,   70330, 24, }, // Asia/Srednekolymsk
    {  293, 3263,   70354, 12, }, // Asia/Taipei
    {  293, 4781,   70366, 16, }, // Asia/Tashkent
    {  293, 3915,   70382, 18, }, // Asia/Tbilisi
    {  293, 2772,   70400, 16, }, // Asia/Tehran
    {  293, 1317,   70416, 10, }, // Asia/Thimphu
    {  293, 2821,   70426, 12, }, // Asia/Tokyo
    {  293, 4572,   70438, 14, }, // Asia/Tomsk
    {  293, 1021,   70452, 18, }, // Asia/Ulaanbaatar
    {  293, 1159,   70470, 12, }, // Asia/Urumqi
    {  293, 8722,   70482, 17, }, // Asia/Ust-Nera
    {  293, 6977,   70499, 18, }, // Asia/Vientiane
    {  293, 4710,   70517, 28, }, // Asia/Vladivostok
    {  293, 4828,   70545, 18, }, // Asia/Yakutsk
    {  293, 1235,   70563, 14, }, // Asia/Yangon
    {  293, 3883,   70577, 28, }, // Asia/Yekaterinburg
    {  293, 3747,   70605, 16, }, // Asia/Yerevan
    {  293, 3663,   70621, 14, }, // Atlantic/Azores
    {  293, 5671,   70635, 16, }, // Atlantic/Bermuda
    {  293, 6628,   70651, 12, }, // Atlantic/Canary
    {  293, 3727,   70663, 21, }, // Atlantic/Cape_Verde
    {  293, 1395,   70684, 10, }, // Atlantic/Faroe
    {  293, 8736,   70694, 14, }, // Atlantic/Madeira
    {  293, 2748,   70708, 18, }, // Atlantic/Reykjavik
    {  293, 7272,   70726, 25, }, // Atlantic/South_Georgia
    {  293, 6821,   70751, 23, }, // Atlantic/St_Helena
    {  293, 6644,   70774, 16, }, // Atlantic/Stanley
    {  293, 1670,   70790, 16, }, // Australia/Adelaide
    {  293, 1635,   70806, 20, }, // Australia/Brisbane
    {  293, 1799,   70826, 21, }, // Australia/Broken_Hill
    {  293, 1583,   70847, 14, }, // Australia/Darwin
    {  293, 3637,   70861, 12, }, // Australia/Eucla
    {  293, 1516,   70873, 16, }, // Australia/Hobart
    {  293, 8753,   70889, 20, }, // Australia/Lindeman
    {  293, 1547,   70909, 19, }, // Australia/Lord_Howe
    {  293, 1727,   70928, 18, }, // Australia/Melbourne
    {  293, 1762,   70946, 10, }, // Australia/Perth
    {  293, 1463,   70956, 12, }, // Australia/Sydney
    {  293, 6441,   70968, 22, }, // Europe/Amsterdam
    {  293, 6216,   70990, 14, }, // Europe/Andorra
    {  293, 3620,   71004, 18, }, // Europe/Astrakhan
    {  293, 2300,   71022, 12, }, // Europe/Athens
    {  293, 6336,   71034, 18, }, // Europe/Belgrade
    {  293, 4757,   71052, 12, }, // Europe/Berlin
    {  293, 6488,   71064, 22, }, // Europe/Bratislava
    {  293, 2167,   71086, 16, }, // Europe/Brussels
    {  293, 3928,   71102, 18, }, // Europe/Bucharest
    {  293, 3806,   71120, 16, }, // Europe/Budapest
    {  293, 8772,   71136, 16, }, // Europe/Busingen
    {  293, 2583,   71152, 18, }, // Europe/Chisinau
    {  293, 6289,   71170, 18, }, // Europe/Copenhagen
    {  293, 2338,   71188, 14, }, // Europe/Dublin
    {  293, 6307,   71202, 20, }, // Europe/Gibraltar
    {  293, 8788,   71222, 16, }, // Europe/Guernsey
    {  293, 6612,   71238, 16, }, // Europe/Helsinki
    {  293, 8804,   71254, 23, }, // Europe/Isle_of_Man
    {  293, 1130,   71277, 16, }, // Europe/Istanbul
    {  293, 8823,   71293, 12, }, // Europe/Jersey
    {  293, 3979,   71305, 18, }, // Europe/Kaliningrad
    {  293, 8929,   71323, 14, }, // Europe/Kirov
    {  293, 2527,   71337, 12, }, // Europe/Kyiv
    {  293, 3213,   71349, 14, }, // Europe/Lisbon
    {  293, 6506,   71363, 20, }, // Europe/Ljubljana
    {  293, 2501,   71383, 12, }, // Europe/London
    {  293, 6365,   71395, 22, }, // Europe/Luxembourg
    {  293, 4892,   71417, 14, }, // Europe/Madrid
    {  293, 6397,   71431, 10, }, // Europe/Malta
    {  293, 6582,   71441, 26, }, // Europe/Mariehamn
    {  293, 3693,   71467, 14, }, // Europe/Minsk
    {  293, 6410,   71481, 14, }, // Europe/Monaco
    {  293, 3537,   71495, 10, }, // Europe/Moscow
    {  293, 6458,   71505, 10, }, // Europe/Oslo
    {  293, 4284,   71515,  8, }, // Europe/Paris
    {  293, 6424,   71523, 20, }, // Europe/Podgorica
    {  293, 6275,   71543, 14, }, // Europe/Prague
    {  293, 8837,   71557,  8, }, // Europe/Riga
    {  293, 6324,   71565, 10, }, // Europe/Rome
    {  293, 4331,   71575, 12, }, // Europe/Samara
    {  293, 6470,   71587, 19, }, // Europe/San_Marino
    {  293, 6245,   71606, 18, }, // Europe/Sarajevo
    {  293, 4451,   71624, 16, }, // Europe/Saratov
    {  293, 8849,   71640, 20, }, // Europe/Simferopol
    {  293, 6383,   71660, 18, }, // Europe/Skopje
    {  293, 6599,   71678, 12, }, // Europe/Sofia
    {  293, 6523,   71690, 18, }, // Europe/Stockholm
    {  293, 8867,   71708, 14, }, // Europe/Tallinn
    {  293, 6202,   71722, 12, }, // Europe/Tirane
    {  293, 8882,   71734, 22, }, // Europe/Ulyanovsk
    {  293, 6352,   71756, 14, }, // Europe/Vaduz
    {  293, 6567,   71770, 14, }, // Europe/Vatican
    {  293, 6231,   71784, 14, }, // Europe/Vienna
    {  293, 8899,   71798, 14, }, // Europe/Vilnius
    {  293, 4727,   71812, 20, }, // Europe/Volgograd
    {  293, 3190,   71832, 10, }, // Europe/Warsaw
    {  293, 6261,   71842, 14, }, // Europe/Zagreb
    {  293, 6540,   71856, 10, }, // Europe/Zurich
    {  293, 5078,   71866, 28, }, // Indian/Antananarivo
    {  293, 6947,   71894, 10, }, // Indian/Chagos
    {  293, 6046,   71904, 18, }, // Indian/Christmas
    {  293, 6063,   71922, 12, }, // Indian/Cocos
    {  293, 5029,   71934, 14, }, // Indian/Comoro
    {  293, 6661,   71948, 16, }, // Indian/Kerguelen
    {  293, 7260,   71964, 10, }, // Indian/Mahe
    {  293, 7050,   71974, 18, }, // Indian/Maldives
    {  293, 4069,   71992, 14, }, // Indian/Mauritius
    {  293, 5098,   72006, 14, }, // Indian/Mayotte
    {  293, 7211,   72020, 16, }, // Indian/Reunion
    {  293, 4422,   72036, 12, }, // Pacific/Apia
    {  293,  932,   72048, 14, }, // Pacific/Auckland
    {  293, 3706,   72062, 18, }, // Pacific/Bougainville
    {  293, 3015,   72080, 12, }, // Pacific/Chatham
    {  293, 3157,   72092, 10, }, // Pacific/Chuuk
    {  293, 2237,   72102, 10, }, // Pacific/Easter
    {  293, 7360,   72112, 10, }, // Pacific/Efate
    {  293, 7327,   72122, 14, }, // Pacific/Fakaofo
    {  293, 3902,   72136, 10, }, // Pacific/Fiji
    {  293, 7343,   72146, 18, }, // Pacific/Funafuti
    {  293, 6678,   72164, 18, }, // Pacific/Galapagos
    {  293, 6696,   72182, 16, }, // Pacific/Gambier
    {  293, 3822,   72198, 24, }, // Pacific/Guadalcanal
    {  293, 6018,   72222, 12, }, // Pacific/Guam
    {  293, 3049,   72234, 20, }, // Pacific/Kanton
    {  293, 3998,   72254, 20, }, // Pacific/Kiritimati
    {  293, 7019,   72274, 12, }, // Pacific/Kosrae
    {  293, 2842,   72286, 20, }, // Pacific/Kwajalein
    {  293, 4877,   72306, 14, }, // Pacific/Majuro
    {  293, 4051,   72320, 18, }, // Pacific/Marquesas
    {  293, 8914,   72338, 12, }, // Pacific/Midway
    {  293, 7084,   72350, 10, }, // Pacific/Nauru
    {  293, 7132,   72360, 10, }, // Pacific/Niue
    {  293, 4168,   72370, 16, }, // Pacific/Norfolk
    {  293, 7098,   72386, 12, }, // Pacific/Noumea
    {  293, 3126,   72398, 17, }, // Pacific/Pago_Pago
    {  293, 7155,   72415, 10, }, // Pacific/Palau
    {  293, 7194,   72425, 18, }, // Pacific/Pitcairn
    {  293, 3096,   72443, 12, }, // Pacific/Pohnpei
    {  293, 4807,   72455, 25, }, // Pacific/Port_Moresby
    {  293, 6076,   72480, 18, }, // Pacific/Rarotonga
    {  293, 6031,   72498, 14, }, // Pacific/Saipan
    {  293, 7312,   72512, 14, }, // Pacific/Tahiti
    {  293, 6712,   72526, 14, }, // Pacific/Tarawa
    {  293, 4583,   72540, 20, }, // Pacific/Tongatapu
    {  293, 7392,   72560, 10, }, // Pacific/Wake
    {  293, 7405,   72570, 14, }, // Pacific/Wallis
    {  311, 5059,   72584, 11, }, // Africa/Addis_Ababa Gaelic/Latin/United Kingdom
    {  311,   14,   72595,  6, }, // Africa/Asmara
    {  311, 5130,   72601, 12, }, // Africa/Dar_es_Salaam
    {  311, 5043,   72613, 10, }, // Africa/Djibouti
    {  311, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  311, 4479,   72623, 11, }, // Africa/Johannesburg
    {  311, 6869,   20300,  4, }, // Africa/Lome
    {  311, 5271,   72634,  9, }, // Africa/Ndjamena
    {  311, 4435,       6,  8, }, // Africa/Sao_Tome
    {  311, 5607,   72643,  9, }, // America/Anguilla
    {  311, 5624,   72652,  8, }, // America/Antigua
    {  311, 4554,   26427,  9, }, // America/Araguaina
    {  311,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  311, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {  311, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  311, 5640,   72660,  5, }, // America/Aruba
    {  311, 4252,      14,  8, }, // America/Asuncion
    {  311,  237,      22,  8, }, // America/Atikokan
    {  311, 7623,      30, 17, }, // America/Bahia_Banderas
    {  311, 7646,   35764,  5, }, // America/Belem
    {  311, 5407,   72665, 10, }, // America/Belize
    {  311, 4361,   20389,  6, }, // America/Bogota
    {  311, 3868,      47,  6, }, // America/Cancun
    {  311, 5496,   72675,  7, }, // America/Cayman
    {  311, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  311, 5422,   72682, 11, }, // America/Costa_Rica
    {  311, 3791,   26442,  6, }, // America/Cuiaba
    {  311, 5723,      66,  7, }, // America/Curacao
    {  311, 5739,   72693, 10, }, // America/Dominica
    {  311, 7869,   26448,  8, }, // America/Eirunepe
    {  311, 5441,   72703, 12, }, // America/El_Salvador
    {  311, 7924,   72715,  9, }, // America/Glace_Bay
    {  311, 4612,   72724, 12, }, // America/Grand_Turk
    {  311, 5770,   72736,  8, }, // America/Grenada
    {  311, 5786,   20408,  8, }, // America/Guadeloupe
    {  311, 3760,   72744,  9, }, // America/Guatemala
    {  311, 6932,   72753,  9, }, // America/Guyana
    {  311,  481,      73, 13, }, // America/Indiana/Knox
    {  311, 7961,      86, 16, }, // America/Indiana/Marengo
    {  311, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  311, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  311, 8038,     139, 14, }, // America/Indiana/Vevay
    {  311, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  311, 8086,     171, 16, }, // America/Indiana/Winamac
    {  311, 8110,   72762,  7, }, // America/Inuvik
    {  311, 2799,   72769,  8, }, // America/Jamaica
    {  311, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  311, 5932,     207, 22, }, // America/Lower_Princes
    {  311, 8153,   26456,  6, }, // America/Maceio
    {  311, 5805,   72777,  9, }, // America/Martinique
    {  311, 2917,   26471,  8, }, // America/Mazatlan
    {  311, 8220,     229,  6, }, // America/Merida
    {  311, 2949,   72786, 17, }, // America/Mexico_City
    {  311, 5824,   72803,  9, }, // America/Montserrat
    {  311, 2379,   72812, 13, }, // America/New_York
    {  311, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  311, 8329,     255, 20, }, // America/North_Dakota/Center
    {  311, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  311,  393,     298,  4, }, // America/Nuuk
    {  311, 5954,   72825, 15, }, // America/Port_of_Spain
    {  311, 5843,   72840, 11, }, // America/Puerto_Rico
    {  311, 8424,   72851, 11, }, // America/Rankin_Inlet
    {  311, 8460,   72862, 10, }, // America/Resolute
    {  311, 8477,   26572,  8, }, // America/Santarem
    {  311, 1878,   26580,  9, }, // America/Sao_Paulo
    {  311, 6899,     302, 16, }, // America/Scoresbysund
    {  311, 8508,   35804, 16, }, // America/St_Barthelemy
    {  311, 2061,     332, 10, }, // America/St_Johns
    {  311, 5863,   72872, 14, }, // America/St_Kitts
    {  311, 5880,   72886, 12, }, // America/St_Lucia
    {  311,  855,     360, 10, }, // America/St_Thomas
    {  311, 5913,   72898, 15, }, // America/St_Vincent
    {  311, 5756,   26664,  7, }, // America/Thule
    {  311, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  311, 8568,   72913, 17, }, // Antarctica/Macquarie
    {  311, 3968,   72930,  5, }, // Asia/Amman
    {  311, 5368,   57073,  6, }, // Asia/Aqtobe
    {  311,  964,   44190,  7, }, // Asia/Ashgabat
    {  311, 3607,   72935,  7, }, // Asia/Baghdad
    {  311, 5534,   72942,  8, }, // Asia/Bahrain
    {  311, 3778,   33323,  6, }, // Asia/Bishkek
    {  311, 5989,   72950,  8, }, // Asia/Brunei
    {  311, 6159,   33336,  7, }, // Asia/Dushanbe
    {  311, 8651,   72958,  4, }, // Asia/Gaza
    {  311, 1259,   72962, 19, }, // Asia/Ho_Chi_Minh
    {  311, 4771,   26703,  5, }, // Asia/Hovd
    {  311, 1290,   72981,  9, }, // Asia/Jerusalem
    {  311, 4239,   72990,  6, }, // Asia/Karachi
    {  311, 1185,     413,  9, }, // Asia/Kathmandu
    {  311,  992,     422,  7, }, // Asia/Kolkata
    {  311, 5547,   72996,  8, }, // Asia/Kuwait
    {  311, 1211,   73004,  7, }, // Asia/Macau
    {  311, 4214,   73011, 10, }, // Asia/Pyongyang
    {  311, 5559,   73021,  5, }, // Asia/Qatar
    {  311, 8708,   73026,  8, }, // Asia/Qostanay
    {  311, 4269,   22401,  9, }, // Asia/Qyzylorda
    {  311, 3300,   73034,  9, }, // Asia/Singapore
    {  311, 4781,   73043,  7, }, // Asia/Tashkent
    {  311, 3915,   73050,  8, }, // Asia/Tbilisi
    {  311, 2772,   73058,  6, }, // Asia/Tehran
    {  311, 2821,   63592,  5, }, // Asia/Tokyo
    {  311, 1159,   26801,  6, }, // Asia/Urumqi
    {  311, 6977,   73064, 10, }, // Asia/Vientiane
    {  311, 1235,    7762,  6, }, // Asia/Yangon
    {  311, 3663,   73074, 13, }, // Atlantic/Azores
    {  311, 5671,   73087,  8, }, // Atlantic/Bermuda
    {  311, 6628,   73095, 22, }, // Atlantic/Canary
    {  311, 3727,   73117, 13, }, // Atlantic/Cape_Verde
    {  311, 1395,   73130,  4, }, // Atlantic/Faroe
    {  311, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  311, 7272,   73134, 14, }, // Atlantic/South_Georgia
    {  311, 6821,   73148, 19, }, // Atlantic/St_Helena
    {  311, 1463,    5774,  5, }, // Australia/Sydney
    {  311, 2300,   73167,  9, }, // Europe/Athens
    {  311, 2167,   73176, 12, }, // Europe/Brussels
    {  311, 2583,   63672,  8, }, // Europe/Chisinau
    {  311, 6289,   73188,  6, }, // Europe/Copenhagen
    {  311, 2338,   73194, 17, }, // Europe/Dublin
    {  311, 6307,   73211, 10, }, // Europe/Gibraltar
    {  311, 8788,   73221, 10, }, // Europe/Guernsey
    {  311, 8804,   73231, 15, }, // Europe/Isle_of_Man
    {  311, 8823,   73246,  9, }, // Europe/Jersey
    {  311, 2527,    8030,  4, }, // Europe/Kyiv
    {  311, 2501,   73255, 20, }, // Europe/London
    {  311, 6365,   73275, 10, }, // Europe/Luxembourg
    {  311, 3537,   73285,  5, }, // Europe/Moscow
    {  311, 4284,   73290,  5, }, // Europe/Paris
    {  311, 6275,   73295,  4, }, // Europe/Prague
    {  311, 8837,   73299,  4, }, // Europe/Riga
    {  311, 6324,   73303,  8, }, // Europe/Rome
    {  311, 6599,   22787,  6, }, // Europe/Sofia
    {  311, 6202,    8137,  6, }, // Europe/Tirane
    {  311, 6567,   73311, 11, }, // Europe/Vatican
    {  311, 6540,    4459,  6, }, // Europe/Zurich
    {  311, 6046,   73322,  7, }, // Indian/Christmas
    {  311, 6661,   36170,  9, }, // Indian/Kerguelen
    {  311, 7260,   26983,  4, }, // Indian/Mahe
    {  311, 7050,   73329, 24, }, // Indian/Maldives
    {  311, 4069,   73353, 24, }, // Indian/Mauritius
    {  311, 7211,     467,  7, }, // Indian/Reunion
    {  311, 3157,     474,  5, }, // Pacific/Chuuk
    {  311, 2237,   73377,  8, }, // Pacific/Easter
    {  311, 3902,   73385,  4, }, // Pacific/Fiji
    {  311, 6678,   21159,  9, }, // Pacific/Galapagos
    {  311, 6696,   73389,  9, }, // Pacific/Gambier
    {  311, 2723,    4493,  8, }, // Pacific/Honolulu
    {  311, 3049,    4501,  9, }, // Pacific/Kanton
    {  311, 4051,   73398, 20, }, // Pacific/Marquesas
    {  311, 7084,   73418,  6, }, // Pacific/Nauru
    {  311, 7155,   73424,  6, }, // Pacific/Palau
    {  311, 7194,   73430, 14, }, // Pacific/Pitcairn
    {  311, 3096,     479,  7, }, // Pacific/Pohnpei
    {  311, 7405,   73444,  6, }, // Pacific/Wallis
    {  312, 6760,    4707,  2, }, // Africa/Accra Ga/Latin/Ghana
    {  312,   14,       0,  6, }, // Africa/Asmara
    {  312, 2320,    4572,  5, }, // Africa/Cairo
    {  312, 4117,    4577, 10, }, // Africa/Casablanca
    {  312, 6773,   26353,  7, }, // Africa/Conakry
    {  312, 6803,   73450, 10, }, // Africa/Nouakchott
    {  312, 4435,       6,  8, }, // Africa/Sao_Tome
    {  312, 3346,   73460,  8, }, // America/Anchorage
    {  312, 4252,      14,  8, }, // America/Asuncion
    {  312,  237,      22,  8, }, // America/Atikokan
    {  312, 7623,      30, 17, }, // America/Bahia_Banderas
    {  312, 7660,    6502, 12, }, // America/Blanc-Sablon
    {  312, 7713,   73468, 25, }, // America/Cambridge_Bay
    {  312, 3868,    6564,  6, }, // America/Cancun
    {  312, 5496,   21483,  6, }, // America/Cayman
    {  312, 2260,   73493,  7, }, // America/Chicago
    {  312, 7756,   73500,  9, }, // America/Chihuahua
    {  312, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  312, 5422,   21503, 10, }, // America/Costa_Rica
    {  312, 7796,    6618,  7, }, // America/Creston
    {  312, 5723,    6631,  7, }, // America/Curacao
    {  312, 7848,   73509, 21, }, // America/Dawson_Creek
    {  312, 5739,   21548,  8, }, // America/Dominica
    {  312, 7886,   73530, 10, }, // America/Fort_Nelson
    {  312, 7924,   73540, 21, }, // America/Glace_Bay
    {  312, 6881,   73561, 21, }, // America/Goose_Bay
    {  312, 4612,   73582, 10, }, // America/Grand_Turk
    {  312, 1939,   73592,  7, }, // America/Halifax
    {  312,  481,   73599, 13, }, // America/Indiana/Knox
    {  312, 7961,      86, 16, }, // America/Indiana/Marengo
    {  312, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  312, 8012,   73612, 19, }, // America/Indiana/Tell_City
    {  312, 8038,     139, 14, }, // America/Indiana/Vevay
    {  312, 8060,   73631, 18, }, // America/Indiana/Vincennes
    {  312, 8086,    5043, 16, }, // America/Indiana/Winamac
    {  312,  660,    5059,  7, }, // America/Iqaluit
    {  312, 2799,    3877,  7, }, // America/Jamaica
    {  312, 8125,   73649, 19, }, // America/Kentucky/Monticello
    {  312, 5932,   73668, 26, }, // America/Lower_Princes
    {  312, 5805,    5142,  8, }, // America/Martinique
    {  312, 8220,     229,  6, }, // America/Merida
    {  312, 2949,   73694, 10, }, // America/Mexico_City
    {  312, 4391,    7031,  7, }, // America/Miquelon
    {  312, 8254,    7038,  7, }, // America/Moncton
    {  312, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  312, 8329,   73704, 23, }, // America/North_Dakota/Center
    {  312, 8357,   73727, 26, }, // America/North_Dakota/New_Salem
    {  312,  393,     298,  4, }, // America/Nuuk
    {  312, 3945,   73753, 21, }, // America/Port-au-Prince
    {  312, 5954,   73774, 18, }, // America/Port_of_Spain
    {  312, 5843,   21940, 11, }, // America/Puerto_Rico
    {  312, 8424,   73792, 15, }, // America/Rankin_Inlet
    {  312, 6899,   73807, 15, }, // America/Scoresbysund
    {  312, 8508,     318, 14, }, // America/St_Barthelemy
    {  312, 2061,     332, 10, }, // America/St_Johns
    {  312, 5863,     342,  9, }, // America/St_Kitts
    {  312, 5880,   73822,  9, }, // America/St_Lucia
    {  312,  855,     360, 10, }, // America/St_Thomas
    {  312, 5913,   73831, 11, }, // America/St_Vincent
    {  312, 8530,   73842, 12, }, // America/Swift_Current
    {  312, 5461,    5410, 11, }, // America/Tegucigalpa
    {  312, 2093,   73854,  9, }, // America/Vancouver
    {  312, 2144,   73863, 10, }, // America/Whitehorse
    {  312, 6001,   73873,  5, }, // Antarctica/Casey
    {  312, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  312, 8568,   73878,  8, }, // Antarctica/Macquarie
    {  312, 7113,   73886,  8, }, // Antarctica/McMurdo
    {  312, 5523,    5491,  5, }, // Asia/Aqtau
    {  312, 5368,    7488,  6, }, // Asia/Aqtobe
    {  312, 4511,    5519,  7, }, // Asia/Colombo
    {  312, 4540,   26689,  7, }, // Asia/Damascus
    {  312, 1259,   73894, 20, }, // Asia/Ho_Chi_Minh
    {  312, 1290,    5568,  9, }, // Asia/Jerusalem
    {  312, 4239,   73914,  7, }, // Asia/Karachi
    {  312, 1185,     413,  9, }, // Asia/Kathmandu
    {  312,  992,     422,  7, }, // Asia/Kolkata
    {  312, 5976,   73921,  7, }, // Asia/Kuching
    {  312, 1211,    7613,  5, }, // Asia/Macau
    {  312, 6920,    4208,  6, }, // Asia/Muscat
    {  312, 2554,    5620,  7, }, // Asia/Nicosia
    {  312, 5559,    4214,  4, }, // Asia/Qatar
    {  312, 8708,     434,  8, }, // Asia/Qostanay
    {  312, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  312, 1159,   73928,  6, }, // Asia/Urumqi
    {  312, 1235,     442,  6, }, // Asia/Yangon
    {  312, 3727,   22522, 10, }, // Atlantic/Cape_Verde
    {  312, 1395,     448,  5, }, // Atlantic/Faroe
    {  312, 6821,     453, 10, }, // Atlantic/St_Helena
    {  312, 1799,   73934, 15, }, // Australia/Broken_Hill
    {  312, 3637,   22590,  5, }, // Australia/Eucla
    {  312, 1547,   73949, 11, }, // Australia/Lord_Howe
    {  312, 2527,     463,  4, }, // Europe/Kyiv
    {  312, 6947,   73960,  6, }, // Indian/Chagos
    {  312, 6046,   73966,  7, }, // Indian/Christmas
    {  312, 6063,    4465,  5, }, // Indian/Cocos
    {  312, 5029,   22853,  6, }, // Indian/Comoro
    {  312, 7211,     467,  7, }, // Indian/Reunion
    {  312,  932,   73973,  6, }, // Pacific/Auckland
    {  312, 3015,   73979,  7, }, // Pacific/Chatham
    {  312, 3157,   73986,  5, }, // Pacific/Chuuk
    {  312, 3822,    8242, 11, }, // Pacific/Guadalcanal
    {  312, 4051,   73991,  9, }, // Pacific/Marquesas
    {  312, 7194,    6133,  8, }, // Pacific/Pitcairn
    {  312, 3096,     479,  7, }, // Pacific/Pohnpei
    {  313, 6760,   74000,  4, }, // Africa/Accra Galician/Latin/Spain
    {  313, 5059,   20252, 10, }, // Africa/Addis_Ababa
    {  313, 7420,   74004,  5, }, // Africa/Algiers
    {  313,   14,       0,  6, }, // Africa/Asmara
    {  313,   44,   74009,  6, }, // Africa/Bamako
    {  313, 2320,   74015,  7, }, // Africa/Cairo
    {  313, 5043,   26360,  7, }, // Africa/Djibouti
    {  313, 5166,   74022,  6, }, // Africa/El_Aaiun
    {  313, 4479,   74028, 12, }, // Africa/Johannesburg
    {  313, 4524,   74040,  7, }, // Africa/Khartoum
    {  313, 6869,   20300,  4, }, // Africa/Lome
    {  313, 5001,   74047,  6, }, // Africa/Lusaka
    {  313, 5113,   74053,  8, }, // Africa/Mogadishu
    {  313, 5271,   43996,  9, }, // Africa/Ndjamena
    {  313, 6727,   26402,  8, }, // Africa/Ouagadougou
    {  313, 4435,   74061,  8, }, // Africa/Sao_Tome
    {  313, 2866,   20349,  7, }, // Africa/Tripoli
    {  313, 6554,   74069,  5, }, // Africa/Tunis
    {  313, 5607,    4657,  7, }, // America/Anguilla
    {  313, 4554,   26427,  9, }, // America/Araguaina
    {  313,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  313, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {  313, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  313, 4252,      14,  8, }, // America/Asuncion
    {  313,  237,      22,  8, }, // America/Atikokan
    {  313, 3679,   74074,  4, }, // America/Bahia
    {  313, 7623,      30, 17, }, // America/Bahia_Banderas
    {  313, 7646,   35764,  5, }, // America/Belem
    {  313, 4361,   20389,  6, }, // America/Bogota
    {  313, 3868,      47,  6, }, // America/Cancun
    {  313, 4345,   44035,  6, }, // America/Cayenne
    {  313, 5496,   74078, 12, }, // America/Cayman
    {  313, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  313, 3791,   26442,  6, }, // America/Cuiaba
    {  313, 5723,      66,  7, }, // America/Curacao
    {  313, 7869,   26448,  8, }, // America/Eirunepe
    {  313, 5441,   74090, 10, }, // America/El_Salvador
    {  313, 5770,   20401,  7, }, // America/Grenada
    {  313, 5786,   20408,  9, }, // America/Guadeloupe
    {  313, 6932,   74100,  6, }, // America/Guyana
    {  313, 2281,   74106,  8, }, // America/Havana
    {  313,  348,   74114, 12, }, // America/Indiana/Indianapolis
    {  313,  481,      73, 13, }, // America/Indiana/Knox
    {  313, 7961,      86, 16, }, // America/Indiana/Marengo
    {  313, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  313, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  313, 8038,     139, 14, }, // America/Indiana/Vevay
    {  313, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  313, 8086,     171, 16, }, // America/Indiana/Winamac
    {  313, 2799,   20426,  7, }, // America/Jamaica
    {  313, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  313, 4376,   74126,  5, }, // America/La_Paz
    {  313, 3239,   74131, 10, }, // America/Los_Angeles
    {  313, 5932,     207, 22, }, // America/Lower_Princes
    {  313, 8153,   26456,  6, }, // America/Maceio
    {  313, 5805,   20436,  9, }, // America/Martinique
    {  313, 2917,   26471,  8, }, // America/Mazatlan
    {  313, 8220,     229,  6, }, // America/Merida
    {  313, 2949,   74141, 16, }, // America/Mexico_City
    {  313, 2379,   44062,  9, }, // America/New_York
    {  313, 8301,   74157, 23, }, // America/North_Dakota/Beulah
    {  313, 8329,   74180, 23, }, // America/North_Dakota/Center
    {  313, 8357,   74203, 26, }, // America/North_Dakota/New_Salem
    {  313,  393,     298,  4, }, // America/Nuuk
    {  313, 2356,   20547,  6, }, // America/Panama
    {  313, 3945,   74229, 14, }, // America/Port-au-Prince
    {  313, 5954,   74243, 12, }, // America/Port_of_Spain
    {  313, 5843,   64866, 10, }, // America/Puerto_Rico
    {  313,  695,   74255, 10, }, // America/Rio_Branco
    {  313, 8477,   26572,  8, }, // America/Santarem
    {  313, 1878,   26580,  9, }, // America/Sao_Paulo
    {  313, 6899,     302, 16, }, // America/Scoresbysund
    {  313, 8508,   35804, 16, }, // America/St_Barthelemy
    {  313, 2061,   26605, 12, }, // America/St_Johns
    {  313, 5863,   26617, 11, }, // America/St_Kitts
    {  313, 5880,   74265, 11, }, // America/St_Lucia
    {  313,  855,   44161, 12, }, // America/St_Thomas
    {  313, 5913,   74276, 11, }, // America/St_Vincent
    {  313, 5688,   20605,  7, }, // America/Tortola
    {  313, 6133,   57010, 16, }, // Antarctica/DumontDUrville
    {  313, 7295,   62088,  5, }, // Antarctica/Syowa
    {  313, 5570,   74287,  4, }, // Asia/Aden
    {  313, 5395,    7476,  6, }, // Asia/Almaty
    {  313, 3968,   20612,  4, }, // Asia/Amman
    {  313, 5523,    5491,  5, }, // Asia/Aqtau
    {  313, 5368,    7488,  6, }, // Asia/Aqtobe
    {  313,  964,   74291,  9, }, // Asia/Ashgabat
    {  313, 3607,    4044,  6, }, // Asia/Baghdad
    {  313, 3653,   74300,  4, }, // Asia/Baku
    {  313, 4601,   74304,  5, }, // Asia/Chita
    {  313, 4540,   74309,  7, }, // Asia/Damascus
    {  313, 4795,   20653,  6, }, // Asia/Hebron
    {  313, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  313, 7006,   74316,  7, }, // Asia/Jakarta
    {  313, 1290,   20703,  9, }, // Asia/Jerusalem
    {  313, 3560,   74323,  5, }, // Asia/Kabul
    {  313, 1185,   20712,  8, }, // Asia/Kathmandu
    {  313, 8661,   57888,  8, }, // Asia/Khandyga
    {  313,  992,   20720,  7, }, // Asia/Kolkata
    {  313, 1211,    4203,  5, }, // Asia/Macau
    {  313, 6920,   20739,  7, }, // Asia/Muscat
    {  313, 8708,   44259,  8, }, // Asia/Qostanay
    {  313, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  313, 3584,    7669,  4, }, // Asia/Riyadh
    {  313, 4408,   44267,  8, }, // Asia/Sakhalin
    {  313, 7245,   20768, 10, }, // Asia/Samarkand
    {  313, 3279,   20778,  4, }, // Asia/Seoul
    {  313, 3300,   20782,  8, }, // Asia/Singapore
    {  313, 4297,    7699, 13, }, // Asia/Srednekolymsk
    {  313, 2772,   57993,  7, }, // Asia/Tehran
    {  313, 1159,   26801,  6, }, // Asia/Urumqi
    {  313, 4828,   44309,  7, }, // Asia/Yakutsk
    {  313, 1235,     442,  6, }, // Asia/Yangon
    {  313, 3883,   74328, 13, }, // Asia/Yekaterinburg
    {  313, 3747,   74341,  7, }, // Asia/Yerevan
    {  313, 5671,   74348, 14, }, // Atlantic/Bermuda
    {  313, 6628,   74362, 14, }, // Atlantic/Canary
    {  313, 3727,   26836, 10, }, // Atlantic/Cape_Verde
    {  313, 1395,   20874,  5, }, // Atlantic/Faroe
    {  313, 2748,   74376, 10, }, // Atlantic/Reykjavik
    {  313, 7272,   74386, 14, }, // Atlantic/South_Georgia
    {  313, 6821,   26865, 12, }, // Atlantic/St_Helena
    {  313, 1670,   20912,  8, }, // Australia/Adelaide
    {  313, 1463,   22607,  6, }, // Australia/Sydney
    {  313, 6441,   74400,  9, }, // Europe/Amsterdam
    {  313, 3620,   74409,  8, }, // Europe/Astrakhan
    {  313, 2300,   26877,  6, }, // Europe/Athens
    {  313, 6336,   58774,  8, }, // Europe/Belgrade
    {  313, 4757,   20941,  6, }, // Europe/Berlin
    {  313, 2167,   74417,  8, }, // Europe/Brussels
    {  313, 3928,   20955,  8, }, // Europe/Bucharest
    {  313, 6289,   20963, 10, }, // Europe/Copenhagen
    {  313, 2338,   20973,  6, }, // Europe/Dublin
    {  313, 6307,   20979,  9, }, // Europe/Gibraltar
    {  313, 6612,   74425,  9, }, // Europe/Helsinki
    {  313, 8804,   74434, 11, }, // Europe/Isle_of_Man
    {  313, 3979,   62893, 12, }, // Europe/Kaliningrad
    {  313, 2527,   44434,  4, }, // Europe/Kyiv
    {  313, 3213,   21019,  6, }, // Europe/Lisbon
    {  313, 6506,   21025,  9, }, // Europe/Ljubljana
    {  313, 2501,   21034,  7, }, // Europe/London
    {  313, 6365,   74445, 10, }, // Europe/Luxembourg
    {  313, 6410,   74455,  6, }, // Europe/Monaco
    {  313, 3537,   74461,  7, }, // Europe/Moscow
    {  313, 4284,   21062,  5, }, // Europe/Paris
    {  313, 6275,   21067,  5, }, // Europe/Prague
    {  313, 6324,    5946,  4, }, // Europe/Rome
    {  313, 6245,   74468,  8, }, // Europe/Sarajevo
    {  313, 8849,   74476, 10, }, // Europe/Simferopol
    {  313, 6599,   21081,  5, }, // Europe/Sofia
    {  313, 6523,   74486,  9, }, // Europe/Stockholm
    {  313, 6202,   21095,  6, }, // Europe/Tirane
    {  313, 8882,   26960,  9, }, // Europe/Ulyanovsk
    {  313, 6567,   74495,  8, }, // Europe/Vatican
    {  313, 6231,   21112,  5, }, // Europe/Vienna
    {  313, 4727,   63074, 10, }, // Europe/Volgograd
    {  313, 3190,   21126,  8, }, // Europe/Warsaw
    {  313, 6540,    4459,  6, }, // Europe/Zurich
    {  313, 6046,   74503, 14, }, // Indian/Christmas
    {  313, 5029,   65391,  7, }, // Indian/Comoro
    {  313, 7260,   26983,  4, }, // Indian/Mahe
    {  313, 7050,   74517,  8, }, // Indian/Maldives
    {  313, 4069,   74525,  8, }, // Indian/Mauritius
    {  313, 7211,   21142,  7, }, // Indian/Reunion
    {  313, 3157,     474,  5, }, // Pacific/Chuuk
    {  313, 2237,   74533, 14, }, // Pacific/Easter
    {  313, 3902,   21155,  4, }, // Pacific/Fiji
    {  313, 6678,   74547, 15, }, // Pacific/Galapagos
    {  313, 2723,   74562,  8, }, // Pacific/Honolulu
    {  313, 3049,    4501,  9, }, // Pacific/Kanton
    {  313, 3096,     479,  7, }, // Pacific/Pohnpei
    {  313, 7312,   44525,  6, }, // Pacific/Tahiti
    {  314,   14,       0,  6, }, // Africa/Asmara Ganda/Latin/Uganda
    {  314, 4435,       6,  8, }, // Africa/Sao_Tome
    {  314, 4252,      14,  8, }, // America/Asuncion
    {  314,  237,      22,  8, }, // America/Atikokan
    {  314, 7623,      30, 17, }, // America/Bahia_Banderas
    {  314, 3868,      47,  6, }, // America/Cancun
    {  314, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  314, 5723,      66,  7, }, // America/Curacao
    {  314,  481,      73, 13, }, // America/Indiana/Knox
    {  314, 7961,      86, 16, }, // America/Indiana/Marengo
    {  314, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  314, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  314, 8038,     139, 14, }, // America/Indiana/Vevay
    {  314, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  314, 8086,     171, 16, }, // America/Indiana/Winamac
    {  314, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  314, 5932,     207, 22, }, // America/Lower_Princes
    {  314, 8220,     229,  6, }, // America/Merida
    {  314, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  314, 8329,     255, 20, }, // America/North_Dakota/Center
    {  314, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  314,  393,     298,  4, }, // America/Nuuk
    {  314, 6899,     302, 16, }, // America/Scoresbysund
    {  314, 8508,     318, 14, }, // America/St_Barthelemy
    {  314, 2061,     332, 10, }, // America/St_Johns
    {  314, 5863,     342,  9, }, // America/St_Kitts
    {  314, 5880,     351,  9, }, // America/St_Lucia
    {  314,  855,     360, 10, }, // America/St_Thomas
    {  314, 5913,     370, 11, }, // America/St_Vincent
    {  314, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  314, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  314, 1185,     413,  9, }, // Asia/Kathmandu
    {  314,  992,     422,  7, }, // Asia/Kolkata
    {  314, 1211,     429,  5, }, // Asia/Macau
    {  314, 1235,     442,  6, }, // Asia/Yangon
    {  314, 1395,     448,  5, }, // Atlantic/Faroe
    {  314, 6821,     453, 10, }, // Atlantic/St_Helena
    {  314, 2527,     463,  4, }, // Europe/Kyiv
    {  314, 7211,     467,  7, }, // Indian/Reunion
    {  314, 3157,     474,  5, }, // Pacific/Chuuk
    {  314, 3096,     479,  7, }, // Pacific/Pohnpei
    {  315,   14,       0,  6, }, // Africa/Asmara Geez/Ethiopic/Ethiopia
    {  315, 4435,       6,  8, }, // Africa/Sao_Tome
    {  315, 4252,      14,  8, }, // America/Asuncion
    {  315,  237,      22,  8, }, // America/Atikokan
    {  315, 7623,      30, 17, }, // America/Bahia_Banderas
    {  315, 3868,      47,  6, }, // America/Cancun
    {  315, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  315, 5723,      66,  7, }, // America/Curacao
    {  315,  481,      73, 13, }, // America/Indiana/Knox
    {  315, 7961,      86, 16, }, // America/Indiana/Marengo
    {  315, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  315, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  315, 8038,     139, 14, }, // America/Indiana/Vevay
    {  315, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  315, 8086,     171, 16, }, // America/Indiana/Winamac
    {  315, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  315, 5932,     207, 22, }, // America/Lower_Princes
    {  315, 8220,     229,  6, }, // America/Merida
    {  315, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  315, 8329,     255, 20, }, // America/North_Dakota/Center
    {  315, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  315,  393,     298,  4, }, // America/Nuuk
    {  315, 6899,     302, 16, }, // America/Scoresbysund
    {  315, 8508,     318, 14, }, // America/St_Barthelemy
    {  315, 2061,     332, 10, }, // America/St_Johns
    {  315, 5863,     342,  9, }, // America/St_Kitts
    {  315, 5880,     351,  9, }, // America/St_Lucia
    {  315,  855,     360, 10, }, // America/St_Thomas
    {  315, 5913,     370, 11, }, // America/St_Vincent
    {  315, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  315, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  315, 1185,     413,  9, }, // Asia/Kathmandu
    {  315,  992,     422,  7, }, // Asia/Kolkata
    {  315, 1211,     429,  5, }, // Asia/Macau
    {  315, 1235,     442,  6, }, // Asia/Yangon
    {  315, 1395,     448,  5, }, // Atlantic/Faroe
    {  315, 6821,     453, 10, }, // Atlantic/St_Helena
    {  315, 2527,     463,  4, }, // Europe/Kyiv
    {  315, 7211,     467,  7, }, // Indian/Reunion
    {  315, 3157,     474,  5, }, // Pacific/Chuuk
    {  315, 3096,     479,  7, }, // Pacific/Pohnpei
    {  317, 6788,   74570,  7, }, // Africa/Abidjan Georgian/Georgian/Georgia
    {  317, 6760,   74577,  4, }, // Africa/Accra
    {  317, 5059,   74581, 10, }, // Africa/Addis_Ababa
    {  317, 7420,   74591,  6, }, // Africa/Algiers
    {  317,   14,   74597,  6, }, // Africa/Asmara
    {  317,   44,   74603,  6, }, // Africa/Bamako
    {  317, 5257,   74609,  5, }, // Africa/Bangui
    {  317, 6746,   74614,  7, }, // Africa/Banjul
    {  317, 7435,   74621,  5, }, // Africa/Bissau
    {  317, 4957,   74626,  8, }, // Africa/Blantyre
    {  317, 5287,   74634,  9, }, // Africa/Brazzaville
    {  317, 4922,   74643,  9, }, // Africa/Bujumbura
    {  317, 2320,   74652,  5, }, // Africa/Cairo
    {  317, 4117,   74657, 10, }, // Africa/Casablanca
    {  317, 7449,   74667,  5, }, // Africa/Ceuta
    {  317, 6773,   74672,  7, }, // Africa/Conakry
    {  317, 6840,   74679,  6, }, // Africa/Dakar
    {  317, 5130,   74685, 13, }, // Africa/Dar_es_Salaam
    {  317, 5043,   74698,  6, }, // Africa/Djibouti
    {  317, 5243,   74704,  5, }, // Africa/Douala
    {  317, 5166,   74709,  9, }, // Africa/El_Aaiun
    {  317, 6853,   74718,  8, }, // Africa/Freetown
    {  317, 4906,   74726,  8, }, // Africa/Gaborone
    {  317, 5015,   74734,  6, }, // Africa/Harare
    {  317, 4479,   74740, 12, }, // Africa/Johannesburg
    {  317, 4499,   74752,  4, }, // Africa/Juba
    {  317, 5151,   74756,  7, }, // Africa/Kampala
    {  317, 4524,   74763,  7, }, // Africa/Khartoum
    {  317, 4973,   74770,  6, }, // Africa/Kigali
    {  317, 5306,   74776,  7, }, // Africa/Kinshasa
    {  317, 4744,   74783,  6, }, // Africa/Lagos
    {  317, 5336,   74789,  9, }, // Africa/Libreville
    {  317, 6869,   74798,  4, }, // Africa/Lome
    {  317, 5211,   74802,  6, }, // Africa/Luanda
    {  317, 4939,   74808,  9, }, // Africa/Lubumbashi
    {  317, 5001,   74817,  6, }, // Africa/Lusaka
    {  317, 5322,   74823,  6, }, // Africa/Malabo
    {  317, 4987,   74829,  6, }, // Africa/Maputo
    {  317, 5197,   74835,  6, }, // Africa/Maseru
    {  317, 5182,   74841,  7, }, // Africa/Mbabane
    {  317, 5113,   74848,  8, }, // Africa/Mogadishu
    {  317, 7034,   74856,  8, }, // Africa/Monrovia
    {  317, 3853,   74864,  7, }, // Africa/Nairobi
    {  317, 5271,   74871,  7, }, // Africa/Ndjamena
    {  317, 5354,   74878,  6, }, // Africa/Niamey
    {  317, 6803,   74884,  8, }, // Africa/Nouakchott
    {  317, 6727,   74892,  8, }, // Africa/Ouagadougou
    {  317, 5225,   74900, 10, }, // Africa/Porto-Novo
    {  317, 4435,   74910,  8, }, // Africa/Sao_Tome
    {  317, 2866,   74918,  7, }, // Africa/Tripoli
    {  317, 6554,   74925,  6, }, // Africa/Tunis
    {  317, 4152,   74931,  8, }, // Africa/Windhoek
    {  317,  132,   74939,  5, }, // America/Adak
    {  317, 3346,   74944,  8, }, // America/Anchorage
    {  317, 5607,   74952,  7, }, // America/Anguilla
    {  317, 5624,   74959,  7, }, // America/Antigua
    {  317, 4554,   74966,  9, }, // America/Araguaina
    {  317,  166,   74975, 13, }, // America/Argentina/Buenos_Aires
    {  317,   91,   74988,  9, }, // America/Argentina/Catamarca
    {  317,  270,   74997,  7, }, // America/Argentina/Cordoba
    {  317,  441,   75004,  5, }, // America/Argentina/Jujuy
    {  317, 7462,   75009,  8, }, // America/Argentina/La_Rioja
    {  317,  565,   75017,  7, }, // America/Argentina/Mendoza
    {  317, 7489,   75024, 12, }, // America/Argentina/Rio_Gallegos
    {  317, 7520,   75036,  5, }, // America/Argentina/Salta
    {  317, 7544,   75041,  9, }, // America/Argentina/San_Juan
    {  317, 5580,   75050,  9, }, // America/Argentina/San_Luis
    {  317, 7571,   75059,  8, }, // America/Argentina/Tucuman
    {  317, 7597,   75067,  6, }, // America/Argentina/Ushuaia
    {  317, 5640,   75073,  5, }, // America/Aruba
    {  317, 4252,   75078,  9, }, // America/Asuncion
    {  317,  237,   75087,  9, }, // America/Atikokan
    {  317, 3679,   75096,  4, }, // America/Bahia
    {  317, 7623,   75100, 15, }, // America/Bahia_Banderas
    {  317, 5654,   75115,  9, }, // America/Barbados
    {  317, 7646,   75124,  6, }, // America/Belem
    {  317, 5407,   75130,  6, }, // America/Belize
    {  317, 7660,   75136, 12, }, // America/Blanc-Sablon
    {  317, 7681,   75148,  9, }, // America/Boa_Vista
    {  317, 4361,   75157,  6, }, // America/Bogota
    {  317, 7699,   75163,  5, }, // America/Boise
    {  317, 7713,   75168, 11, }, // America/Cambridge_Bay
    {  317, 7735,   75179, 12, }, // America/Campo_Grande
    {  317, 3868,   75191,  7, }, // America/Cancun
    {  317, 4694,   75198,  8, }, // America/Caracas
    {  317, 4345,   75206,  6, }, // America/Cayenne
    {  317, 5496,   75212,  7, }, // America/Cayman
    {  317, 2260,   75219,  6, }, // America/Chicago
    {  317, 7756,   75225,  8, }, // America/Chihuahua
    {  317, 7774,   75233, 14, }, // America/Ciudad_Juarez
    {  317, 5422,   75247, 10, }, // America/Costa_Rica
    {  317, 7796,   75257,  8, }, // America/Creston
    {  317, 3791,   75265,  6, }, // America/Cuiaba
    {  317, 5723,   75271,  8, }, // America/Curacao
    {  317, 7812,   75279, 12, }, // America/Danmarkshavn
    {  317, 7833,   75291,  7, }, // America/Dawson
    {  317, 7848,   75298, 12, }, // America/Dawson_Creek
    {  317,  805,   75310,  7, }, // America/Denver
    {  317, 3465,   75317,  8, }, // America/Detroit
    {  317, 5739,   75325,  8, }, // America/Dominica
    {  317,  893,   75333,  9, }, // America/Edmonton
    {  317, 7869,   75342,  8, }, // America/Eirunepe
    {  317, 5441,   75350,  9, }, // America/El_Salvador
    {  317, 7886,   75359, 12, }, // America/Fort_Nelson
    {  317, 7906,   75371,  9, }, // America/Fortaleza
    {  317, 7924,   75380,  9, }, // America/Glace_Bay
    {  317, 6881,   75389,  7, }, // America/Goose_Bay
    {  317, 4612,   75396, 11, }, // America/Grand_Turk
    {  317, 5770,   75407,  7, }, // America/Grenada
    {  317, 5786,   75414,  9, }, // America/Guadeloupe
    {  317, 3760,   75423,  9, }, // America/Guatemala
    {  317, 4841,   75432,  9, }, // America/Guayaquil
    {  317, 6932,   75441,  6, }, // America/Guyana
    {  317, 1939,   75447,  9, }, // America/Halifax
    {  317, 2281,   75456,  6, }, // America/Havana
    {  317, 7942,   75462,  9, }, // America/Hermosillo
    {  317,  348,   75471, 13, }, // America/Indiana/Indianapolis
    {  317,  481,   75484, 14, }, // America/Indiana/Knox
    {  317, 7961,   75498, 16, }, // America/Indiana/Marengo
    {  317, 7985,   75514, 20, }, // America/Indiana/Petersburg
    {  317, 8012,   75534, 17, }, // America/Indiana/Tell_City
    {  317, 8038,   75551, 14, }, // America/Indiana/Vevay
    {  317, 8060,   75565, 16, }, // America/Indiana/Vincennes
    {  317, 8086,   75581, 17, }, // America/Indiana/Winamac
    {  317, 8110,   75598,  7, }, // America/Inuvik
    {  317,  660,   75605,  8, }, // America/Iqaluit
    {  317, 2799,   75613,  7, }, // America/Jamaica
    {  317, 5380,   75620,  4, }, // America/Juneau
    {  317,  521,   75624,  8, }, // America/Kentucky/Louisville
    {  317, 8125,   75632, 18, }, // America/Kentucky/Monticello
    {  317, 5704,   75650, 11, }, // America/Kralendijk
    {  317, 4376,   75661,  7, }, // America/La_Paz
    {  317, 7169,   75668,  4, }, // America/Lima
    {  317, 3239,   75672, 12, }, // America/Los_Angeles
    {  317, 5932,   75684, 19, }, // America/Lower_Princes
    {  317, 8153,   75703,  6, }, // America/Maceio
    {  317, 8168,   75709,  7, }, // America/Managua
    {  317, 1908,   75716,  7, }, // America/Manaus
    {  317, 5897,   75723,  6, }, // America/Marigot
    {  317, 5805,   75729,  9, }, // America/Martinique
    {  317, 8184,   75738, 10, }, // America/Matamoros
    {  317, 2917,   75748,  9, }, // America/Mazatlan
    {  317, 8202,   75757,  8, }, // America/Menominee
    {  317, 8220,   75765,  6, }, // America/Merida
    {  317, 8235,   75771, 10, }, // America/Metlakatla
    {  317, 2949,   75781,  6, }, // America/Mexico_City
    {  317, 4391,   75787,  8, }, // America/Miquelon
    {  317, 8254,   75795,  8, }, // America/Moncton
    {  317, 8270,   75803,  8, }, // America/Monterrey
    {  317, 4098,   75811, 10, }, // America/Montevideo
    {  317, 5824,   75821,  9, }, // America/Montserrat
    {  317, 5481,   75830,  5, }, // America/Nassau
    {  317, 2379,   75835,  9, }, // America/New_York
    {  317, 8288,   75759,  4, }, // America/Nome
    {  317, 1850,   75844,  7, }, // America/Noronha
    {  317, 8301,   75851, 23, }, // America/North_Dakota/Beulah
    {  317, 8329,   75874, 24, }, // America/North_Dakota/Center
    {  317, 8357,   75898, 28, }, // America/North_Dakota/New_Salem
    {  317,  393,   75926,  7, }, // America/Nuuk
    {  317, 8388,   75933,  7, }, // America/Ojinaga
    {  317, 2356,   75940,  6, }, // America/Panama
    {  317, 6173,   75946, 10, }, // America/Paramaribo
    {  317, 2973,   75956,  7, }, // America/Phoenix
    {  317, 3945,   75963, 13, }, // America/Port-au-Prince
    {  317, 5954,   75976, 14, }, // America/Port_of_Spain
    {  317, 8404,   75990, 11, }, // America/Porto_Velho
    {  317, 5843,   76001, 11, }, // America/Puerto_Rico
    {  317, 4030,   76012, 13, }, // America/Punta_Arenas
    {  317, 8424,   76025, 13, }, // America/Rankin_Inlet
    {  317, 8445,   76038,  6, }, // America/Recife
    {  317, 1995,   76044,  6, }, // America/Regina
    {  317, 8460,   76050,  8, }, // America/Resolute
    {  317,  695,   76058, 10, }, // America/Rio_Branco
    {  317, 8477,   76068,  9, }, // America/Santarem
    {  317, 2201,   76077,  8, }, // America/Santiago
    {  317, 6111,   76085, 13, }, // America/Santo_Domingo
    {  317, 1878,   76098,  9, }, // America/Sao_Paulo
    {  317, 6899,   76107, 12, }, // America/Scoresbysund
    {  317, 8494,   76119,  5, }, // America/Sitka
    {  317, 8508,   76124, 13, }, // America/St_Barthelemy
    {  317, 2061,   76137, 10, }, // America/St_Johns
    {  317, 5863,   76147, 10, }, // America/St_Kitts
    {  317, 5880,   76157, 10, }, // America/St_Lucia
    {  317,  855,   76167, 10, }, // America/St_Thomas
    {  317, 5913,   76177, 12, }, // America/St_Vincent
    {  317, 8530,   76189, 13, }, // America/Swift_Current
    {  317, 5461,   76202, 11, }, // America/Tegucigalpa
    {  317, 5756,   76213,  4, }, // America/Thule
    {  317,  313,   76217,  7, }, // America/Tijuana
    {  317,  608,   76224,  7, }, // America/Toronto
    {  317, 5688,   76231,  7, }, // America/Tortola
    {  317, 2093,   76238,  9, }, // America/Vancouver
    {  317, 2144,   76247,  9, }, // America/Whitehorse
    {  317,  734,   76256,  8, }, // America/Winnipeg
    {  317, 8552,   76264,  8, }, // America/Yakutat
    {  317, 6001,   76272,  5, }, // Antarctica/Casey
    {  317, 6094,   76277,  6, }, // Antarctica/Davis
    {  317, 6133,   76283, 15, }, // Antarctica/DumontDUrville
    {  317, 8568,   76298,  7, }, // Antarctica/Macquarie
    {  317, 7066,   76305,  7, }, // Antarctica/Mawson
    {  317, 7113,   76312,  8, }, // Antarctica/McMurdo
    {  317, 8589,   76320,  7, }, // Antarctica/Palmer
    {  317, 7226,   76327,  6, }, // Antarctica/Rothera
    {  317, 7295,   76333,  5, }, // Antarctica/Syowa
    {  317, 8607,   76338,  5, }, // Antarctica/Troll
    {  317, 7374,   76343,  7, }, // Antarctica/Vostok
    {  317, 1429,   76350, 11, }, // Arctic/Longyearbyen
    {  317, 5570,   76361,  5, }, // Asia/Aden
    {  317, 5395,   76366,  6, }, // Asia/Almaty
    {  317, 3968,   76372,  5, }, // Asia/Amman
    {  317, 5511,   76377,  7, }, // Asia/Anadyr
    {  317, 5523,   76384,  5, }, // Asia/Aqtau
    {  317, 5368,   76389,  6, }, // Asia/Aqtobe
    {  317,  964,   76395,  8, }, // Asia/Ashgabat
    {  317, 8624,   76403,  6, }, // Asia/Atyrau
    {  317, 3607,   76409,  7, }, // Asia/Baghdad
    {  317, 5534,   76416,  8, }, // Asia/Bahrain
    {  317, 3653,   76424,  4, }, // Asia/Baku
    {  317, 4466,   76428,  8, }, // Asia/Bangkok
    {  317, 3571,   76436,  8, }, // Asia/Barnaul
    {  317, 4086,   76444,  7, }, // Asia/Beirut
    {  317, 3778,   76451,  7, }, // Asia/Bishkek
    {  317, 5989,   76458,  6, }, // Asia/Brunei
    {  317, 4601,   76464,  4, }, // Asia/Chita
    {  317, 4511,   76468,  7, }, // Asia/Colombo
    {  317, 4540,   76475,  7, }, // Asia/Damascus
    {  317, 1093,   74679,  4, }, // Asia/Dhaka
    {  317, 6192,   76482,  4, }, // Asia/Dili
    {  317, 3596,   76486,  5, }, // Asia/Dubai
    {  317, 6159,   76491,  7, }, // Asia/Dushanbe
    {  317, 8636,   76498,  9, }, // Asia/Famagusta
    {  317, 8651,   76507,  4, }, // Asia/Gaza
    {  317, 4795,   76511,  7, }, // Asia/Hebron
    {  317, 1259,   76518,  8, }, // Asia/Ho_Chi_Minh
    {  317, 2704,   76526,  8, }, // Asia/Hong_Kong
    {  317, 4771,   76534,  5, }, // Asia/Hovd
    {  317, 4184,   76539,  8, }, // Asia/Irkutsk
    {  317, 7006,   76547,  7, }, // Asia/Jakarta
    {  317, 6992,   76554,  8, }, // Asia/Jayapura
    {  317, 1290,   76562, 10, }, // Asia/Jerusalem
    {  317, 3560,   76572,  6, }, // Asia/Kabul
    {  317, 4316,   76578,  8, }, // Asia/Kamchatka
    {  317, 4239,   76586,  6, }, // Asia/Karachi
    {  317, 1185,   76592,  8, }, // Asia/Kathmandu
    {  317, 8661,   76600,  7, }, // Asia/Khandyga
    {  317,  992,   76607,  7, }, // Asia/Kolkata
    {  317, 4197,   76614, 12, }, // Asia/Krasnoyarsk
    {  317, 4859,   76626, 13, }, // Asia/Kuala_Lumpur
    {  317, 5976,   76639,  7, }, // Asia/Kuching
    {  317, 5547,   76646,  7, }, // Asia/Kuwait
    {  317, 1211,   76653,  5, }, // Asia/Macau
    {  317, 4017,   76658,  8, }, // Asia/Magadan
    {  317, 1349,   76666,  8, }, // Asia/Makassar
    {  317, 7182,   76674,  6, }, // Asia/Manila
    {  317, 6920,   76680,  7, }, // Asia/Muscat
    {  317, 2554,   76687,  7, }, // Asia/Nicosia
    {  317, 8675,   76694, 12, }, // Asia/Novokuznetsk
    {  317, 4135,   76706, 12, }, // Asia/Novosibirsk
    {  317, 4229,   76718,  5, }, // Asia/Omsk
    {  317, 7145,   76723,  5, }, // Asia/Oral
    {  317, 6961,   76728,  8, }, // Asia/Phnom_Penh
    {  317, 8693,   76736, 10, }, // Asia/Pontianak
    {  317, 4214,   76746,  8, }, // Asia/Pyongyang
    {  317, 5559,   76754,  6, }, // Asia/Qatar
    {  317, 8708,   76760,  8, }, // Asia/Qostanay
    {  317, 4269,   76768,  9, }, // Asia/Qyzylorda
    {  317, 3584,   76777,  8, }, // Asia/Riyadh
    {  317, 4408,   76785,  8, }, // Asia/Sakhalin
    {  317, 7245,   76793, 10, }, // Asia/Samarkand
    {  317, 3279,   76803,  5, }, // Asia/Seoul
    {  317, 1053,   76808,  6, }, // Asia/Shanghai
    {  317, 3300,   76814,  9, }, // Asia/Singapore
    {  317, 4297,   76823, 14, }, // Asia/Srednekolymsk
    {  317, 3263,   76837,  6, }, // Asia/Taipei
    {  317, 4781,   76843,  8, }, // Asia/Tashkent
    {  317, 3915,   76851,  7, }, // Asia/Tbilisi
    {  317, 2772,   76858,  7, }, // Asia/Tehran
    {  317, 1317,   76865,  7, }, // Asia/Thimphu
    {  317, 2821,   76872,  5, }, // Asia/Tokyo
    {  317, 4572,   76877,  6, }, // Asia/Tomsk
    {  317, 1021,   76883, 10, }, // Asia/Ulaanbaatar
    {  317, 1159,   76893,  6, }, // Asia/Urumqi
    {  317, 8722,   76899,  8, }, // Asia/Ust-Nera
    {  317, 6977,   76907,  9, }, // Asia/Vientiane
    {  317, 4710,   76916, 12, }, // Asia/Vladivostok
    {  317, 4828,   76928,  8, }, // Asia/Yakutsk
    {  317, 1235,   76936,  7, }, // Asia/Yangon
    {  317, 3883,   76943, 13, }, // Asia/Yekaterinburg
    {  317, 3747,   76956,  7, }, // Asia/Yerevan
    {  317, 3663,   76963, 16, }, // Atlantic/Azores
    {  317, 5671,   76979,  7, }, // Atlantic/Bermuda
    {  317, 6628,   76986,  6, }, // Atlantic/Canary
    {  317, 3727,   76992, 10, }, // Atlantic/Cape_Verde
    {  317, 1395,   77002, 17, }, // Atlantic/Faroe
    {  317, 8736,   77019,  7, }, // Atlantic/Madeira
    {  317, 2748,   77026, 10, }, // Atlantic/Reykjavik
    {  317, 7272,   77036, 14, }, // Atlantic/South_Georgia
    {  317, 6821,   77050, 18, }, // Atlantic/St_Helena
    {  317, 6644,   77068,  6, }, // Atlantic/Stanley
    {  317, 1670,   77074,  8, }, // Australia/Adelaide
    {  317, 1635,   77082,  9, }, // Australia/Brisbane
    {  317, 1799,   77091, 12, }, // Australia/Broken_Hill
    {  317, 1583,   77103,  7, }, // Australia/Darwin
    {  317, 3637,   77110,  5, }, // Australia/Eucla
    {  317, 1516,   77115,  7, }, // Australia/Hobart
    {  317, 8753,   77122,  8, }, // Australia/Lindeman
    {  317, 1547,   77130,  9, }, // Australia/Lord_Howe
    {  317, 1727,   77139,  8, }, // Australia/Melbourne
    {  317, 1762,   77147,  5, }, // Australia/Perth
    {  317, 1463,   77152,  6, }, // Australia/Sydney
    {  317, 6441,   77158, 10, }, // Europe/Amsterdam
    {  317, 6216,   77168,  6, }, // Europe/Andorra
    {  317, 3620,   77174,  9, }, // Europe/Astrakhan
    {  317, 2300,   77183,  5, }, // Europe/Athens
    {  317, 6336,   77188,  8, }, // Europe/Belgrade
    {  317, 4757,   77196,  7, }, // Europe/Berlin
    {  317, 6488,   77203, 10, }, // Europe/Bratislava
    {  317, 2167,   77213,  8, }, // Europe/Brussels
    {  317, 3928,   77221,  9, }, // Europe/Bucharest
    {  317, 3806,   77230,  9, }, // Europe/Budapest
    {  317, 8772,   77239,  9, }, // Europe/Busingen
    {  317, 2583,   77248,  9, }, // Europe/Chisinau
    {  317, 6289,   77257, 11, }, // Europe/Copenhagen
    {  317, 2338,   77268,  7, }, // Europe/Dublin
    {  317, 6307,   77275, 10, }, // Europe/Gibraltar
    {  317, 8788,   77285,  6, }, // Europe/Guernsey
    {  317, 6612,   77291,  8, }, // Europe/Helsinki
    {  317, 8804,   77299, 13, }, // Europe/Isle_of_Man
    {  317, 1130,   77312,  8, }, // Europe/Istanbul
    {  317, 8823,   77320,  5, }, // Europe/Jersey
    {  317, 3979,   77325, 12, }, // Europe/Kaliningrad
    {  317, 8929,   77337,  6, }, // Europe/Kirov
    {  317, 2527,   77343,  5, }, // Europe/Kyiv
    {  317, 3213,   77348,  8, }, // Europe/Lisbon
    {  317, 6506,   77356,  9, }, // Europe/Ljubljana
    {  317, 2501,   77365,  7, }, // Europe/London
    {  317, 6365,   77372, 11, }, // Europe/Luxembourg
    {  317, 4892,   77383,  7, }, // Europe/Madrid
    {  317, 6397,   77390,  5, }, // Europe/Malta
    {  317, 6582,   77395,  9, }, // Europe/Mariehamn
    {  317, 3693,   77404,  6, }, // Europe/Minsk
    {  317, 6410,   77410,  6, }, // Europe/Monaco
    {  317, 3537,   77416,  7, }, // Europe/Moscow
    {  317, 6458,   77423,  4, }, // Europe/Oslo
    {  317, 4284,   77427,  6, }, // Europe/Paris
    {  317, 6424,   77433,  9, }, // Europe/Podgorica
    {  317, 6275,   77442,  5, }, // Europe/Prague
    {  317, 8837,   77447,  4, }, // Europe/Riga
    {  317, 6324,   77451,  4, }, // Europe/Rome
    {  317, 4331,   77455,  6, }, // Europe/Samara
    {  317, 6470,   77461, 10, }, // Europe/San_Marino
    {  317, 6245,   77471,  7, }, // Europe/Sarajevo
    {  317, 4451,   77478,  8, }, // Europe/Saratov
    {  317, 8849,   77486, 11, }, // Europe/Simferopol
    {  317, 6383,   77497,  6, }, // Europe/Skopje
    {  317, 6599,   77503,  5, }, // Europe/Sofia
    {  317, 6523,   77508,  9, }, // Europe/Stockholm
    {  317, 8867,   77517,  6, }, // Europe/Tallinn
    {  317, 6202,   77523,  6, }, // Europe/Tirane
    {  317, 8882,   77529, 10, }, // Europe/Ulyanovsk
    {  317, 6352,   77539,  6, }, // Europe/Vaduz
    {  317, 6567,   77545,  8, }, // Europe/Vatican
    {  317, 6231,   77553,  4, }, // Europe/Vienna
    {  317, 8899,   77557,  8, }, // Europe/Vilnius
    {  317, 4727,   77565, 10, }, // Europe/Volgograd
    {  317, 3190,   77575,  7, }, // Europe/Warsaw
    {  317, 6261,   77582,  7, }, // Europe/Zagreb
    {  317, 6540,   77589,  7, }, // Europe/Zurich
    {  317, 5078,   77596, 12, }, // Indian/Antananarivo
    {  317, 6947,   77608,  6, }, // Indian/Chagos
    {  317, 6046,   77614, 13, }, // Indian/Christmas
    {  317, 6063,   77627,  6, }, // Indian/Cocos
    {  317, 5029,   77633,  6, }, // Indian/Comoro
    {  317, 6661,   77639,  9, }, // Indian/Kerguelen
    {  317, 7260,   77648,  4, }, // Indian/Mahe
    {  317, 7050,   77652,  9, }, // Indian/Maldives
    {  317, 4069,   77661,  7, }, // Indian/Mauritius
    {  317, 5098,   77668,  6, }, // Indian/Mayotte
    {  317, 7211,   77674,  9, }, // Indian/Reunion
    {  317, 4422,   77683,  4, }, // Pacific/Apia
    {  317,  932,   77687,  7, }, // Pacific/Auckland
    {  317, 3706,   77694,  9, }, // Pacific/Bougainville
    {  317, 3015,   77703,  6, }, // Pacific/Chatham
    {  317, 3157,   77709,  5, }, // Pacific/Chuuk
    {  317, 2237,   77714,  6, }, // Pacific/Easter
    {  317, 7360,   77720,  5, }, // Pacific/Efate
    {  317, 7327,   77725,  7, }, // Pacific/Fakaofo
    {  317, 3902,   77732,  4, }, // Pacific/Fiji
    {  317, 7343,   77736,  8, }, // Pacific/Funafuti
    {  317, 6678,   77744, 10, }, // Pacific/Galapagos
    {  317, 6696,   77754,  8, }, // Pacific/Gambier
    {  317, 3822,   77762, 12, }, // Pacific/Guadalcanal
    {  317, 6018,   77774,  5, }, // Pacific/Guam
    {  317, 2723,   77779,  8, }, // Pacific/Honolulu
    {  317, 3049,   77787,  9, }, // Pacific/Kanton
    {  317, 3998,   77796, 10, }, // Pacific/Kiritimati
    {  317, 7019,   77806,  6, }, // Pacific/Kosrae
    {  317, 2842,   77812, 10, }, // Pacific/Kwajalein
    {  317, 4877,   77822,  6, }, // Pacific/Majuro
    {  317, 4051,   77828,  9, }, // Pacific/Marquesas
    {  317, 8914,   77837,  6, }, // Pacific/Midway
    {  317, 7084,   77843,  5, }, // Pacific/Nauru
    {  317, 7132,   77848,  4, }, // Pacific/Niue
    {  317, 4168,   77852,  8, }, // Pacific/Norfolk
    {  317, 7098,   77860,  5, }, // Pacific/Noumea
    {  317, 3126,   77865,  9, }, // Pacific/Pago_Pago
    {  317, 7155,   77874,  5, }, // Pacific/Palau
    {  317, 7194,   77879,  8, }, // Pacific/Pitcairn
    {  317, 3096,   77887,  6, }, // Pacific/Pohnpei
    {  317, 4807,   77893, 11, }, // Pacific/Port_Moresby
    {  317, 6076,   77904,  9, }, // Pacific/Rarotonga
    {  317, 6031,   77913,  7, }, // Pacific/Saipan
    {  317, 7312,   77920,  5, }, // Pacific/Tahiti
    {  317, 6712,   77925,  6, }, // Pacific/Tarawa
    {  317, 4583,   77931,  9, }, // Pacific/Tongatapu
    {  317, 7392,   77940,  5, }, // Pacific/Wake
    {  317, 7405,   77945,  6, }, // Pacific/Wallis
    {  318, 5059,    3784, 11, }, // Africa/Addis_Ababa German/Latin/Germany
    {  318, 7420,   58423,  6, }, // Africa/Algiers
    {  318,   14,       0,  6, }, // Africa/Asmara
    {  318, 2320,    4572,  5, }, // Africa/Cairo
    {  318, 5130,   77951, 10, }, // Africa/Dar_es_Salaam
    {  318, 5043,   77961,  9, }, // Africa/Djibouti
    {  318, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  318, 4524,   26372,  7, }, // Africa/Khartoum
    {  318, 6869,   20300,  4, }, // Africa/Lome
    {  318, 5113,   77970, 10, }, // Africa/Mogadishu
    {  318, 5271,   43996,  9, }, // Africa/Ndjamena
    {  318, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  318, 4435,       6,  8, }, // Africa/Sao_Tome
    {  318, 2866,   57624,  8, }, // Africa/Tripoli
    {  318,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  318, 4252,      14,  8, }, // America/Asuncion
    {  318,  237,      22,  8, }, // America/Atikokan
    {  318, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  318, 4361,   20389,  6, }, // America/Bogota
    {  318, 3868,      47,  6, }, // America/Cancun
    {  318, 5496,   77980, 12, }, // America/Cayman
    {  318, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  318, 5723,      66,  7, }, // America/Curacao
    {  318, 2281,   63392,  7, }, // America/Havana
    {  318,  481,      73, 13, }, // America/Indiana/Knox
    {  318, 7961,      86, 16, }, // America/Indiana/Marengo
    {  318, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  318, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  318, 8038,     139, 14, }, // America/Indiana/Vevay
    {  318, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  318, 8086,     171, 16, }, // America/Indiana/Winamac
    {  318, 2799,    3877,  7, }, // America/Jamaica
    {  318, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  318, 5932,     207, 22, }, // America/Lower_Princes
    {  318, 8220,    7008,  6, }, // America/Merida
    {  318, 2949,   77992, 12, }, // America/Mexico_City
    {  318, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  318, 8329,     255, 20, }, // America/North_Dakota/Center
    {  318, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  318,  393,     298,  4, }, // America/Nuuk
    {  318, 1878,   26580,  9, }, // America/Sao_Paulo
    {  318, 6899,     302, 16, }, // America/Scoresbysund
    {  318, 8508,   26589, 16, }, // America/St_Barthelemy
    {  318, 2061,     332, 10, }, // America/St_Johns
    {  318, 5863,     342,  9, }, // America/St_Kitts
    {  318, 5880,     351,  9, }, // America/St_Lucia
    {  318,  855,     360, 10, }, // America/St_Thomas
    {  318, 5913,     370, 11, }, // America/St_Vincent
    {  318, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  318, 7374,    4029,  6, }, // Antarctica/Vostok
    {  318, 5368,    7488,  6, }, // Asia/Aqtobe
    {  318,  964,   44190,  7, }, // Asia/Ashgabat
    {  318, 3607,    4044,  6, }, // Asia/Baghdad
    {  318, 3778,   57088,  8, }, // Asia/Bishkek
    {  318, 5989,   78004, 17, }, // Asia/Brunei
    {  318, 4601,   78021,  7, }, // Asia/Chita
    {  318, 4540,    4083,  8, }, // Asia/Damascus
    {  318, 6159,   57096,  9, }, // Asia/Dushanbe
    {  318, 1259,   78028, 17, }, // Asia/Ho_Chi_Minh
    {  318, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  318, 4771,   78045,  5, }, // Asia/Hovd
    {  318, 4316,   78050, 11, }, // Asia/Kamchatka
    {  318, 4239,   78061,  9, }, // Asia/Karachi
    {  318, 1185,     413,  9, }, // Asia/Kathmandu
    {  318, 8661,   57888,  8, }, // Asia/Khandyga
    {  318,  992,   64221,  8, }, // Asia/Kolkata
    {  318, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  318, 1211,    4203,  5, }, // Asia/Macau
    {  318, 6920,   26750,  6, }, // Asia/Muscat
    {  318, 2554,    5620,  7, }, // Asia/Nicosia
    {  318, 8675,   78070, 12, }, // Asia/Novokuznetsk
    {  318, 4135,   78082, 11, }, // Asia/Novosibirsk
    {  318, 4214,   78093,  9, }, // Asia/Pyongyang
    {  318, 5559,    4214,  5, }, // Asia/Qatar
    {  318, 8708,   44259,  8, }, // Asia/Qostanay
    {  318, 4269,   57168,  9, }, // Asia/Qyzylorda
    {  318, 3584,    7669,  4, }, // Asia/Riyadh
    {  318, 4408,   57952,  8, }, // Asia/Sakhalin
    {  318, 3300,   20782,  8, }, // Asia/Singapore
    {  318, 3263,   78102,  6, }, // Asia/Taipei
    {  318, 4781,   57196,  9, }, // Asia/Tashkent
    {  318, 3915,   78108,  6, }, // Asia/Tbilisi
    {  318, 2772,    7724,  7, }, // Asia/Tehran
    {  318, 2821,    4238,  5, }, // Asia/Tokyo
    {  318, 1159,   26801,  6, }, // Asia/Urumqi
    {  318, 4710,    4243, 11, }, // Asia/Vladivostok
    {  318, 4828,    7755,  7, }, // Asia/Yakutsk
    {  318, 1235,    7762,  6, }, // Asia/Yangon
    {  318, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  318, 3747,   57228,  6, }, // Asia/Yerevan
    {  318, 3663,   58709,  6, }, // Atlantic/Azores
    {  318, 6628,   78114,  7, }, // Atlantic/Canary
    {  318, 3727,   26836, 10, }, // Atlantic/Cape_Verde
    {  318, 1395,   78121,  6, }, // Atlantic/Faroe
    {  318, 2748,   78127, 11, }, // Atlantic/Reykjavik
    {  318, 7272,   78138, 11, }, // Atlantic/South_Georgia
    {  318, 6821,     453, 10, }, // Atlantic/St_Helena
    {  318, 3620,   78149,  9, }, // Europe/Astrakhan
    {  318, 2300,    4320,  5, }, // Europe/Athens
    {  318, 6336,   26883,  7, }, // Europe/Belgrade
    {  318, 2167,   22634,  7, }, // Europe/Brussels
    {  318, 3928,   26897,  8, }, // Europe/Bucharest
    {  318, 8772,   26905,  8, }, // Europe/Busingen
    {  318, 6289,    4357, 10, }, // Europe/Copenhagen
    {  318, 8929,   78158,  5, }, // Europe/Kirov
    {  318, 2527,   78163,  4, }, // Europe/Kyiv
    {  318, 3213,    4381,  8, }, // Europe/Lisbon
    {  318, 6365,    4395,  9, }, // Europe/Luxembourg
    {  318, 3537,   57353,  6, }, // Europe/Moscow
    {  318, 6275,    8083,  4, }, // Europe/Prague
    {  318, 6324,    5946,  3, }, // Europe/Rome
    {  318, 4451,   78167,  7, }, // Europe/Saratov
    {  318, 6202,   21095,  6, }, // Europe/Tirane
    {  318, 8882,   78174,  9, }, // Europe/Ulyanovsk
    {  318, 6567,    5978,  7, }, // Europe/Vatican
    {  318, 6231,   57383,  4, }, // Europe/Vienna
    {  318, 4727,    4443,  9, }, // Europe/Volgograd
    {  318, 3190,   57396,  8, }, // Europe/Warsaw
    {  318, 6540,    4459,  6, }, // Europe/Zurich
    {  318, 6046,   78183, 15, }, // Indian/Christmas
    {  318, 5029,   78198,  7, }, // Indian/Comoro
    {  318, 7050,   78205,  9, }, // Indian/Maldives
    {  318, 7211,     467,  7, }, // Indian/Reunion
    {  318, 3157,     474,  5, }, // Pacific/Chuuk
    {  318, 2237,   78214, 10, }, // Pacific/Easter
    {  318, 3902,   78224,  7, }, // Pacific/Fiji
    {  318, 2723,    4493,  8, }, // Pacific/Honolulu
    {  318, 3049,    4501,  9, }, // Pacific/Kanton
    {  318, 3096,     479,  7, }, // Pacific/Pohnpei
    {  324, 5989,   78004,  6, }, // Asia/Brunei German/Latin/Switzerland
    {  324, 1211,     429,  5, }, // Asia/Macau
    {  324, 4451,   78231,  7, }, // Europe/Saratov
    {  325, 6788,   78238,  8, }, // Africa/Abidjan Greek/Greek/Greece
    {  325, 6760,   78246,  4, }, // Africa/Accra
    {  325, 5059,   78250, 13, }, // Africa/Addis_Ababa
    {  325, 7420,   78263,  6, }, // Africa/Algiers
    {  325,   14,   78269,  6, }, // Africa/Asmara
    {  325,   44,   78275,  7, }, // Africa/Bamako
    {  325, 5257,   78282,  7, }, // Africa/Bangui
    {  325, 6746,   78289,  8, }, // Africa/Banjul
    {  325, 7435,   78297,  7, }, // Africa/Bissau
    {  325, 4957,   78304,  9, }, // Africa/Blantyre
    {  325, 5287,   78313,  9, }, // Africa/Brazzaville
    {  325, 4922,   78322, 13, }, // Africa/Bujumbura
    {  325, 2320,   78335,  5, }, // Africa/Cairo
    {  325, 4117,   78340, 11, }, // Africa/Casablanca
    {  325, 7449,   78351,  6, }, // Africa/Ceuta
    {  325, 6773,   78357,  7, }, // Africa/Conakry
    {  325, 6840,   78364,  6, }, // Africa/Dakar
    {  325, 5130,   78370, 13, }, // Africa/Dar_es_Salaam
    {  325, 5043,   78383,  9, }, // Africa/Djibouti
    {  325, 5243,   78392,  7, }, // Africa/Douala
    {  325, 5166,   78399,  9, }, // Africa/El_Aaiun
    {  325, 6853,   78408,  8, }, // Africa/Freetown
    {  325, 4906,   78416, 10, }, // Africa/Gaborone
    {  325, 5015,   78426,  6, }, // Africa/Harare
    {  325, 4479,   78432, 15, }, // Africa/Johannesburg
    {  325, 4499,   78447,  7, }, // Africa/Juba
    {  325, 5151,   78454,  7, }, // Africa/Kampala
    {  325, 4524,   78461,  7, }, // Africa/Khartoum
    {  325, 4973,   78468,  7, }, // Africa/Kigali
    {  325, 5306,   78475,  7, }, // Africa/Kinshasa
    {  325, 4744,   78482,  6, }, // Africa/Lagos
    {  325, 5336,   78488,  9, }, // Africa/Libreville
    {  325, 6869,   78497,  4, }, // Africa/Lome
    {  325, 5211,   78501,  7, }, // Africa/Luanda
    {  325, 4939,   78508, 12, }, // Africa/Lubumbashi
    {  325, 5001,   78520,  7, }, // Africa/Lusaka
    {  325, 5322,   78527,  7, }, // Africa/Malabo
    {  325, 4987,   78534,  7, }, // Africa/Maputo
    {  325, 5197,   78541,  7, }, // Africa/Maseru
    {  325, 5182,   78548,  8, }, // Africa/Mbabane
    {  325, 5113,   78556, 11, }, // Africa/Mogadishu
    {  325, 7034,   78567,  8, }, // Africa/Monrovia
    {  325, 3853,   78575,  8, }, // Africa/Nairobi
    {  325, 5271,   78583,  8, }, // Africa/Ndjamena
    {  325, 5354,   78591,  6, }, // Africa/Niamey
    {  325, 6803,   78597,  8, }, // Africa/Nouakchott
    {  325, 6727,   78605, 14, }, // Africa/Ouagadougou
    {  325, 5225,   78619, 10, }, // Africa/Porto-Novo
    {  325, 4435,   78629,  8, }, // Africa/Sao_Tome
    {  325, 2866,   78637,  7, }, // Africa/Tripoli
    {  325, 6554,   78644,  6, }, // Africa/Tunis
    {  325, 4152,   78650,  8, }, // Africa/Windhoek
    {  325,  132,   78658,  5, }, // America/Adak
    {  325, 3346,   78663,  8, }, // America/Anchorage
    {  325, 5607,   78671,  9, }, // America/Anguilla
    {  325, 5624,   78680,  9, }, // America/Antigua
    {  325, 4554,   78689, 11, }, // America/Araguaina
    {  325,  166,   78700, 14, }, // America/Argentina/Buenos_Aires
    {  325,   91,   78714,  9, }, // America/Argentina/Catamarca
    {  325,  270,   78723,  7, }, // America/Argentina/Cordoba
    {  325,  441,   78730,  7, }, // America/Argentina/Jujuy
    {  325, 7462,   78737,  8, }, // America/Argentina/La_Rioja
    {  325,  565,   78745,  7, }, // America/Argentina/Mendoza
    {  325, 7489,   78752, 14, }, // America/Argentina/Rio_Gallegos
    {  325, 7520,   78766,  5, }, // America/Argentina/Salta
    {  325, 7544,   78771,  9, }, // America/Argentina/San_Juan
    {  325, 5580,   78780,  9, }, // America/Argentina/San_Luis
    {  325, 7571,   78789,  9, }, // America/Argentina/Tucuman
    {  325, 7597,   78798,  8, }, // America/Argentina/Ushuaia
    {  325, 5640,   78806,  7, }, // America/Aruba
    {  325, 4252,   78813,  9, }, // America/Asuncion
    {  325,  237,   78822,  8, }, // America/Atikokan
    {  325, 3679,   78830,  5, }, // America/Bahia
    {  325, 7623,   78835, 19, }, // America/Bahia_Banderas
    {  325, 5654,   78854, 12, }, // America/Barbados
    {  325, 7646,   78866,  6, }, // America/Belem
    {  325, 5407,   78872,  6, }, // America/Belize
    {  325, 7660,   78878, 13, }, // America/Blanc-Sablon
    {  325, 7681,   78891, 10, }, // America/Boa_Vista
    {  325, 4361,   78901,  8, }, // America/Bogota
    {  325, 7699,   78909,  6, }, // America/Boise
    {  325, 7713,   78915, 13, }, // America/Cambridge_Bay
    {  325, 7735,   78928, 13, }, // America/Campo_Grande
    {  325, 3868,   78941,  7, }, // America/Cancun
    {  325, 4694,   78948,  7, }, // America/Caracas
    {  325, 4345,   78955,  6, }, // America/Cayenne
    {  325, 5496,   78961,  6, }, // America/Cayman
    {  325, 2260,   78967,  6, }, // America/Chicago
    {  325, 7756,   78973,  9, }, // America/Chihuahua
    {  325, 7774,   78982, 15, }, // America/Ciudad_Juarez
    {  325, 5422,   78997, 10, }, // America/Costa_Rica
    {  325, 7796,   79007,  7, }, // America/Creston
    {  325, 3791,   79014,  8, }, // America/Cuiaba
    {  325, 5723,   79022,  8, }, // America/Curacao
    {  325, 7812,   79030, 12, }, // America/Danmarkshavn
    {  325, 7833,   79042,  6, }, // America/Dawson
    {  325, 7848,   79048, 11, }, // America/Dawson_Creek
    {  325,  805,   79059,  7, }, // America/Denver
    {  325, 3465,   79066,  8, }, // America/Detroit
    {  325, 5739,   79074,  9, }, // America/Dominica
    {  325,  893,   79083,  9, }, // America/Edmonton
    {  325, 7869,   79092,  9, }, // America/Eirunepe
    {  325, 5441,   79101, 11, }, // America/El_Salvador
    {  325, 7886,   79112, 11, }, // America/Fort_Nelson
    {  325, 7906,   79123,  9, }, // America/Fortaleza
    {  325, 7924,   79132, 11, }, // America/Glace_Bay
    {  325, 6881,   79143, 10, }, // America/Goose_Bay
    {  325, 4612,   79153, 12, }, // America/Grand_Turk
    {  325, 5770,   79165,  7, }, // America/Grenada
    {  325, 5786,   79172, 11, }, // America/Guadeloupe
    {  325, 3760,   79183, 10, }, // America/Guatemala
    {  325, 4841,   79193, 11, }, // America/Guayaquil
    {  325, 6932,   79204,  7, }, // America/Guyana
    {  325, 1939,   79211,  7, }, // America/Halifax
    {  325, 2281,   79218,  5, }, // America/Havana
    {  325, 7942,   79223,  9, }, // America/Hermosillo
    {  325,  348,   79232, 12, }, // America/Indiana/Indianapolis
    {  325,  481,   79244, 12, }, // America/Indiana/Knox
    {  325, 7961,   79256, 17, }, // America/Indiana/Marengo
    {  325, 7985,   79273, 21, }, // America/Indiana/Petersburg
    {  325, 8012,   79294, 17, }, // America/Indiana/Tell_City
    {  325, 8038,   79311, 14, }, // America/Indiana/Vevay
    {  325, 8060,   79325, 15, }, // America/Indiana/Vincennes
    {  325, 8086,   79340, 18, }, // America/Indiana/Winamac
    {  325, 8110,   79358,  7, }, // America/Inuvik
    {  325,  660,   79365,  8, }, // America/Iqaluit
    {  325, 2799,   79373,  8, }, // America/Jamaica
    {  325, 5380,   79381,  6, }, // America/Juneau
    {  325,  521,   79387,  7, }, // America/Kentucky/Louisville
    {  325, 8125,   79394, 19, }, // America/Kentucky/Monticello
    {  325, 5704,   79413,  9, }, // America/Kralendijk
    {  325, 4376,   79422,  6, }, // America/La_Paz
    {  325, 7169,   79428,  4, }, // America/Lima
    {  325, 3239,   79432, 12, }, // America/Los_Angeles
    {  325, 5932,     207, 22, }, // America/Lower_Princes
    {  325, 8153,   79444,  6, }, // America/Maceio
    {  325, 8168,   79450,  9, }, // America/Managua
    {  325, 1908,   79459,  7, }, // America/Manaus
    {  325, 5897,   79466,  7, }, // America/Marigot
    {  325, 5805,   79473,  9, }, // America/Martinique
    {  325, 8184,   79482,  9, }, // America/Matamoros
    {  325, 2917,   79491,  8, }, // America/Mazatlan
    {  325, 8202,   79499,  8, }, // America/Menominee
    {  325, 8220,   79507,  6, }, // America/Merida
    {  325, 8235,   79513, 10, }, // America/Metlakatla
    {  325, 2949,   79523, 16, }, // America/Mexico_City
    {  325, 4391,   79539,  7, }, // America/Miquelon
    {  325, 8254,   79546,  7, }, // America/Moncton
    {  325, 8270,   79553,  8, }, // America/Monterrey
    {  325, 4098,   79561, 10, }, // America/Montevideo
    {  325, 5824,   79571,  8, }, // America/Montserrat
    {  325, 5481,   79579,  6, }, // America/Nassau
    {  325, 2379,   79585,  9, }, // America/New_York
    {  325, 8288,   79594,  4, }, // America/Nome
    {  325, 1850,   79598,  7, }, // America/Noronha
    {  325, 8301,   79605, 23, }, // America/North_Dakota/Beulah
    {  325, 8329,   79628, 22, }, // America/North_Dakota/Center
    {  325, 8357,   79650, 27, }, // America/North_Dakota/New_Salem
    {  325,  393,   79677,  6, }, // America/Nuuk
    {  325, 8388,   79683,  8, }, // America/Ojinaga
    {  325, 2356,   79691,  7, }, // America/Panama
    {  325, 6173,   79698, 11, }, // America/Paramaribo
    {  325, 2973,   79709,  6, }, // America/Phoenix
    {  325, 3945,   79715, 12, }, // America/Port-au-Prince
    {  325, 5954,   79727, 13, }, // America/Port_of_Spain
    {  325, 8404,   79740, 11, }, // America/Porto_Velho
    {  325, 5843,   79751, 12, }, // America/Puerto_Rico
    {  325, 4030,   79763, 13, }, // America/Punta_Arenas
    {  325, 8424,   79776, 12, }, // America/Rankin_Inlet
    {  325, 8445,   79788,  6, }, // America/Recife
    {  325, 1995,   79794,  8, }, // America/Regina
    {  325, 8460,   79802,  8, }, // America/Resolute
    {  325,  695,   79810, 11, }, // America/Rio_Branco
    {  325, 8477,   79821,  8, }, // America/Santarem
    {  325, 2201,   79829,  9, }, // America/Santiago
    {  325, 6111,   79838, 15, }, // America/Santo_Domingo
    {  325, 1878,   79853,  9, }, // America/Sao_Paulo
    {  325, 6899,   79862, 14, }, // America/Scoresbysund
    {  325, 8494,   79876,  5, }, // America/Sitka
    {  325, 8508,   79881, 18, }, // America/St_Barthelemy
    {  325, 2061,   79899,  9, }, // America/St_Johns
    {  325, 5863,   79908,  8, }, // America/St_Kitts
    {  325, 5880,   79916, 11, }, // America/St_Lucia
    {  325,  855,   79927, 11, }, // America/St_Thomas
    {  325, 5913,   79938, 15, }, // America/St_Vincent
    {  325, 8530,   79953, 13, }, // America/Swift_Current
    {  325, 5461,   79966, 14, }, // America/Tegucigalpa
    {  325, 5756,   79980,  5, }, // America/Thule
    {  325,  313,   79985,  8, }, // America/Tijuana
    {  325,  608,   79993,  7, }, // America/Toronto
    {  325, 5688,   80000,  7, }, // America/Tortola
    {  325, 2093,   80007,  9, }, // America/Vancouver
    {  325, 2144,   80016, 10, }, // America/Whitehorse
    {  325,  734,   80026, 10, }, // America/Winnipeg
    {  325, 8552,   80036,  9, }, // America/Yakutat
    {  325, 6001,   80045,  5, }, // Antarctica/Casey
    {  325, 6094,   80050,  7, }, // Antarctica/Davis
    {  325, 6133,   80057, 17, }, // Antarctica/DumontDUrville
    {  325, 8568,   80074,  8, }, // Antarctica/Macquarie
    {  325, 7066,   80082,  5, }, // Antarctica/Mawson
    {  325, 7113,   80087,  9, }, // Antarctica/McMurdo
    {  325, 8589,   80096,  6, }, // Antarctica/Palmer
    {  325, 7226,   80102,  6, }, // Antarctica/Rothera
    {  325, 7295,   80108,  5, }, // Antarctica/Syowa
    {  325, 8607,   80113,  4, }, // Antarctica/Troll
    {  325, 7374,   80117,  6, }, // Antarctica/Vostok
    {  325, 1429,   80123, 14, }, // Arctic/Longyearbyen
    {  325, 5570,   80137,  5, }, // Asia/Aden
    {  325, 5395,   80142,  6, }, // Asia/Almaty
    {  325, 3968,   80148,  5, }, // Asia/Amman
    {  325, 5511,   80153,  7, }, // Asia/Anadyr
    {  325, 5523,   80160,  6, }, // Asia/Aqtau
    {  325, 5368,   80166,  7, }, // Asia/Aqtobe
    {  325,  964,   80173,  8, }, // Asia/Ashgabat
    {  325, 8624,   80181,  7, }, // Asia/Atyrau
    {  325, 3607,   80188,  7, }, // Asia/Baghdad
    {  325, 5534,   80195,  8, }, // Asia/Bahrain
    {  325, 3653,   80203,  6, }, // Asia/Baku
    {  325, 4466,   80209,  8, }, // Asia/Bangkok
    {  325, 3571,   80217,  9, }, // Asia/Barnaul
    {  325, 4086,   80226,  7, }, // Asia/Beirut
    {  325, 3778,   80233,  7, }, // Asia/Bishkek
    {  325, 5989,   80240,  8, }, // Asia/Brunei
    {  325, 4601,   80248,  5, }, // Asia/Chita
    {  325, 4511,   80253,  7, }, // Asia/Colombo
    {  325, 4540,   80260,  8, }, // Asia/Damascus
    {  325, 1093,   80268,  5, }, // Asia/Dhaka
    {  325, 6192,   80273,  5, }, // Asia/Dili
    {  325, 3596,   80278,  8, }, // Asia/Dubai
    {  325, 6159,   80286, 10, }, // Asia/Dushanbe
    {  325, 8636,   80296, 10, }, // Asia/Famagusta
    {  325, 8651,   80306,  4, }, // Asia/Gaza
    {  325, 4795,   80310,  7, }, // Asia/Hebron
    {  325, 1259,   80317, 16, }, // Asia/Ho_Chi_Minh
    {  325, 2704,   80333, 11, }, // Asia/Hong_Kong
    {  325, 4771,   80344,  5, }, // Asia/Hovd
    {  325, 4184,   80349,  8, }, // Asia/Irkutsk
    {  325, 7006,   80357,  8, }, // Asia/Jakarta
    {  325, 6992,   80365, 11, }, // Asia/Jayapura
    {  325, 1290,   80376, 10, }, // Asia/Jerusalem
    {  325, 3560,   80386,  7, }, // Asia/Kabul
    {  325, 4316,   80393,  9, }, // Asia/Kamchatka
    {  325, 4239,   80402,  7, }, // Asia/Karachi
    {  325, 1185,   80409,  9, }, // Asia/Kathmandu
    {  325, 8661,   80418,  8, }, // Asia/Khandyga
    {  325,  992,   80426,  8, }, // Asia/Kolkata
    {  325, 4197,   80434, 12, }, // Asia/Krasnoyarsk
    {  325, 4859,   80446, 15, }, // Asia/Kuala_Lumpur
    {  325, 5976,   80461,  9, }, // Asia/Kuching
    {  325, 5547,   80470,  7, }, // Asia/Kuwait
    {  325, 1211,   80477,  5, }, // Asia/Macau
    {  325, 4017,   80482,  9, }, // Asia/Magadan
    {  325, 1349,   80491,  7, }, // Asia/Makassar
    {  325, 7182,   80498,  6, }, // Asia/Manila
    {  325, 6920,   80504,  6, }, // Asia/Muscat
    {  325, 2554,   80510,  8, }, // Asia/Nicosia
    {  325, 8675,   80518, 13, }, // Asia/Novokuznetsk
    {  325, 4135,   80531, 12, }, // Asia/Novosibirsk
    {  325, 4229,   80543,  4, }, // Asia/Omsk
    {  325, 7145,   80547,  4, }, // Asia/Oral
    {  325, 6961,   80551,  9, }, // Asia/Phnom_Penh
    {  325, 8693,   80560,  9, }, // Asia/Pontianak
    {  325, 4214,   80569, 12, }, // Asia/Pyongyang
    {  325, 5559,   80581,  5, }, // Asia/Qatar
    {  325, 8708,   80586,  8, }, // Asia/Qostanay
    {  325, 4269,   80594, 10, }, // Asia/Qyzylorda
    {  325, 3584,   80604,  5, }, // Asia/Riyadh
    {  325, 4408,   80609,  8, }, // Asia/Sakhalin
    {  325, 7245,   80617, 10, }, // Asia/Samarkand
    {  325, 3279,   80627,  5, }, // Asia/Seoul
    {  325, 1053,   80632,  7, }, // Asia/Shanghai
    {  325, 3300,   80639, 10, }, // Asia/Singapore
    {  325, 4297,   80649, 14, }, // Asia/Srednekolymsk
    {  325, 3263,   80663,  6, }, // Asia/Taipei
    {  325, 4781,   80669,  8, }, // Asia/Tashkent
    {  325, 3915,   80677,  7, }, // Asia/Tbilisi
    {  325, 2772,   80684,  8, }, // Asia/Tehran
    {  325, 1317,   80692,  6, }, // Asia/Thimphu
    {  325, 2821,   80698,  5, }, // Asia/Tokyo
    {  325, 4572,   80703,  5, }, // Asia/Tomsk
    {  325, 1021,   80708, 12, }, // Asia/Ulaanbaatar
    {  325, 1159,   80720,  9, }, // Asia/Urumqi
    {  325, 8722,   80729,  9, }, // Asia/Ust-Nera
    {  325, 6977,   80738,  8, }, // Asia/Vientiane
    {  325, 4710,   80746, 11, }, // Asia/Vladivostok
    {  325, 4828,   80757,  9, }, // Asia/Yakutsk
    {  325, 1235,   80766,  8, }, // Asia/Yangon
    {  325, 3883,   80774, 15, }, // Asia/Yekaterinburg
    {  325, 3747,   80789,  6, }, // Asia/Yerevan
    {  325, 3663,   80795,  6, }, // Atlantic/Azores
    {  325, 5671,   80801,  9, }, // Atlantic/Bermuda
    {  325, 6628,   80810,  7, }, // Atlantic/Canary
    {  325, 3727,   80817, 17, }, // Atlantic/Cape_Verde
    {  325, 1395,   80834,  6, }, // Atlantic/Faroe
    {  325, 8736,   80840,  6, }, // Atlantic/Madeira
    {  325, 2748,   80846,  9, }, // Atlantic/Reykjavik
    {  325, 7272,   80855, 13, }, // Atlantic/South_Georgia
    {  325, 6821,   80868,  9, }, // Atlantic/St_Helena
    {  325, 6644,   80877,  7, }, // Atlantic/Stanley
    {  325, 1670,   80884,  8, }, // Australia/Adelaide
    {  325, 1635,   80892, 10, }, // Australia/Brisbane
    {  325, 1799,   80902, 11, }, // Australia/Broken_Hill
    {  325, 1583,   80913,  9, }, // Australia/Darwin
    {  325, 3637,   80922,  7, }, // Australia/Eucla
    {  325, 1516,   80929,  7, }, // Australia/Hobart
    {  325, 8753,   80936,  8, }, // Australia/Lindeman
    {  325, 1547,   80944, 10, }, // Australia/Lord_Howe
    {  325, 1727,   80954,  9, }, // Australia/Melbourne
    {  325, 1762,   80963,  4, }, // Australia/Perth
    {  325, 1463,   80967,  6, }, // Australia/Sydney
    {  325, 6441,   80973, 10, }, // Europe/Amsterdam
    {  325, 6216,   80983,  6, }, // Europe/Andorra
    {  325, 3620,   80989,  8, }, // Europe/Astrakhan
    {  325, 2300,   80997,  5, }, // Europe/Athens
    {  325, 6336,   81002,  9, }, // Europe/Belgrade
    {  325, 4757,   81011,  8, }, // Europe/Berlin
    {  325, 6488,   81019, 11, }, // Europe/Bratislava
    {  325, 2167,   81030,  9, }, // Europe/Brussels
    {  325, 3928,   81039, 11, }, // Europe/Bucharest
    {  325, 3806,   81050, 10, }, // Europe/Budapest
    {  325, 8772,   81060, 10, }, // Europe/Busingen
    {  325, 2583,   81070,  8, }, // Europe/Chisinau
    {  325, 6289,   81078,  9, }, // Europe/Copenhagen
    {  325, 2338,   81087,  8, }, // Europe/Dublin
    {  325, 6307,   81095,  9, }, // Europe/Gibraltar
    {  325, 8788,   81104,  7, }, // Europe/Guernsey
    {  325, 6612,   81111,  7, }, // Europe/Helsinki
    {  325, 8804,   81118, 13, }, // Europe/Isle_of_Man
    {  325, 1130,   81131, 16, }, // Europe/Istanbul
    {  325, 8823,   81147,  7, }, // Europe/Jersey
    {  325, 3979,   81154, 13, }, // Europe/Kaliningrad
    {  325, 8929,   81167,  5, }, // Europe/Kirov
    {  325, 2527,   81172,  5, }, // Europe/Kyiv
    {  325, 3213,   81177,  8, }, // Europe/Lisbon
    {  325, 6506,   81185, 11, }, // Europe/Ljubljana
    {  325, 2501,   81196,  7, }, // Europe/London
    {  325, 6365,   81203, 12, }, // Europe/Luxembourg
    {  325, 4892,   81215,  7, }, // Europe/Madrid
    {  325, 6397,   81222,  5, }, // Europe/Malta
    {  325, 6582,   81227,  9, }, // Europe/Mariehamn
    {  325, 3693,   81236,  5, }, // Europe/Minsk
    {  325, 6410,   81241,  6, }, // Europe/Monaco
    {  325, 3537,   81247,  5, }, // Europe/Moscow
    {  325, 6458,   81252,  4, }, // Europe/Oslo
    {  325, 4284,   81256,  6, }, // Europe/Paris
    {  325, 6424,   81262, 12, }, // Europe/Podgorica
    {  325, 6275,   81274,  5, }, // Europe/Prague
    {  325, 8837,   81279,  4, }, // Europe/Riga
    {  325, 6324,   81283,  4, }, // Europe/Rome
    {  325, 4331,   81287,  6, }, // Europe/Samara
    {  325, 6470,   81293, 13, }, // Europe/San_Marino
    {  325, 6245,   81306,  8, }, // Europe/Sarajevo
    {  325, 4451,   81314,  7, }, // Europe/Saratov
    {  325, 8849,   81321, 11, }, // Europe/Simferopol
    {  325, 6383,   81332,  6, }, // Europe/Skopje
    {  325, 6599,   81338,  5, }, // Europe/Sofia
    {  325, 6523,   81343,  9, }, // Europe/Stockholm
    {  325, 8867,   81352,  5, }, // Europe/Tallinn
    {  325, 6202,   81357,  6, }, // Europe/Tirane
    {  325, 8882,   81363, 10, }, // Europe/Ulyanovsk
    {  325, 6352,   81373,  7, }, // Europe/Vaduz
    {  325, 6567,   81380,  8, }, // Europe/Vatican
    {  325, 6231,   81388,  6, }, // Europe/Vienna
    {  325, 8899,   81394,  8, }, // Europe/Vilnius
    {  325, 4727,   81402, 11, }, // Europe/Volgograd
    {  325, 3190,   81413,  8, }, // Europe/Warsaw
    {  325, 6261,   81421,  8, }, // Europe/Zagreb
    {  325, 6540,   81429,  6, }, // Europe/Zurich
    {  325, 5078,   81435, 12, }, // Indian/Antananarivo
    {  325, 6947,   81447,  7, }, // Indian/Chagos
    {  325, 6046,   81454, 19, }, // Indian/Christmas
    {  325, 6063,   81473,  5, }, // Indian/Cocos
    {  325, 5029,   81478,  6, }, // Indian/Comoro
    {  325, 6661,   81484,  9, }, // Indian/Kerguelen
    {  325, 7260,   81493,  4, }, // Indian/Mahe
    {  325, 7050,   81497,  8, }, // Indian/Maldives
    {  325, 4069,   81505,  9, }, // Indian/Mauritius
    {  325, 5098,   81514,  6, }, // Indian/Mayotte
    {  325, 7211,   81520,  7, }, // Indian/Reunion
    {  325, 4422,   81527,  4, }, // Pacific/Apia
    {  325,  932,   81531,  6, }, // Pacific/Auckland
    {  325, 3706,   81537, 11, }, // Pacific/Bougainville
    {  325, 3015,   81548,  6, }, // Pacific/Chatham
    {  325, 3157,   81554,  5, }, // Pacific/Chuuk
    {  325, 2237,   81559, 11, }, // Pacific/Easter
    {  325, 7360,   81570,  5, }, // Pacific/Efate
    {  325, 7327,   81575,  7, }, // Pacific/Fakaofo
    {  325, 3902,   81582,  5, }, // Pacific/Fiji
    {  325, 7343,   81587, 10, }, // Pacific/Funafuti
    {  325, 6678,   81597, 11, }, // Pacific/Galapagos
    {  325, 6696,   81608,  8, }, // Pacific/Gambier
    {  325, 3822,   81616, 14, }, // Pacific/Guadalcanal
    {  325, 6018,   81630,  6, }, // Pacific/Guam
    {  325, 2723,   81636, 10, }, // Pacific/Honolulu
    {  325, 3049,   81646, 10, }, // Pacific/Kanton
    {  325, 3998,   81656, 10, }, // Pacific/Kiritimati
    {  325, 7019,   81666,  6, }, // Pacific/Kosrae
    {  325, 2842,   81672, 11, }, // Pacific/Kwajalein
    {  325, 4877,   81683,  8, }, // Pacific/Majuro
    {  325, 4051,   81691,  8, }, // Pacific/Marquesas
    {  325, 8914,   81699,  9, }, // Pacific/Midway
    {  325, 7084,   81708,  7, }, // Pacific/Nauru
    {  325, 7132,   81715,  5, }, // Pacific/Niue
    {  325, 4168,   81720,  7, }, // Pacific/Norfolk
    {  325, 7098,   81727,  6, }, // Pacific/Noumea
    {  325, 3126,   81733, 11, }, // Pacific/Pago_Pago
    {  325, 7155,   81744,  6, }, // Pacific/Palau
    {  325, 7194,   81750,  7, }, // Pacific/Pitcairn
    {  325, 3096,   81757,  6, }, // Pacific/Pohnpei
    {  325, 4807,   81763, 13, }, // Pacific/Port_Moresby
    {  325, 6076,   81776, 10, }, // Pacific/Rarotonga
    {  325, 6031,   81786,  6, }, // Pacific/Saipan
    {  325, 7312,   81792,  5, }, // Pacific/Tahiti
    {  325, 6712,   81797,  7, }, // Pacific/Tarawa
    {  325, 4583,   81804, 11, }, // Pacific/Tongatapu
    {  325, 7392,   81815,  6, }, // Pacific/Wake
    {  325, 7405,   81821,  7, }, // Pacific/Wallis
    {  327,   14,       0,  6, }, // Africa/Asmara Guarani/Latin/Paraguay
    {  327, 4435,       6,  8, }, // Africa/Sao_Tome
    {  327, 4252,      14,  8, }, // America/Asuncion
    {  327,  237,      22,  8, }, // America/Atikokan
    {  327, 7623,      30, 17, }, // America/Bahia_Banderas
    {  327, 3868,      47,  6, }, // America/Cancun
    {  327, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  327, 5723,      66,  7, }, // America/Curacao
    {  327,  481,      73, 13, }, // America/Indiana/Knox
    {  327, 7961,      86, 16, }, // America/Indiana/Marengo
    {  327, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  327, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  327, 8038,     139, 14, }, // America/Indiana/Vevay
    {  327, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  327, 8086,     171, 16, }, // America/Indiana/Winamac
    {  327, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  327, 5932,     207, 22, }, // America/Lower_Princes
    {  327, 8220,     229,  6, }, // America/Merida
    {  327, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  327, 8329,     255, 20, }, // America/North_Dakota/Center
    {  327, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  327,  393,     298,  4, }, // America/Nuuk
    {  327, 6899,     302, 16, }, // America/Scoresbysund
    {  327, 8508,     318, 14, }, // America/St_Barthelemy
    {  327, 2061,     332, 10, }, // America/St_Johns
    {  327, 5863,     342,  9, }, // America/St_Kitts
    {  327, 5880,     351,  9, }, // America/St_Lucia
    {  327,  855,     360, 10, }, // America/St_Thomas
    {  327, 5913,     370, 11, }, // America/St_Vincent
    {  327, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  327, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  327, 1185,     413,  9, }, // Asia/Kathmandu
    {  327,  992,     422,  7, }, // Asia/Kolkata
    {  327, 1211,     429,  5, }, // Asia/Macau
    {  327, 1235,     442,  6, }, // Asia/Yangon
    {  327, 1395,     448,  5, }, // Atlantic/Faroe
    {  327, 6821,     453, 10, }, // Atlantic/St_Helena
    {  327, 2527,     463,  4, }, // Europe/Kyiv
    {  327, 7211,     467,  7, }, // Indian/Reunion
    {  327, 3157,     474,  5, }, // Pacific/Chuuk
    {  327, 3096,     479,  7, }, // Pacific/Pohnpei
    {  328, 6788,   81828,  8, }, // Africa/Abidjan Gujarati/Gujarati/India
    {  328, 6760,   81836,  4, }, // Africa/Accra
    {  328, 5059,   81840, 10, }, // Africa/Addis_Ababa
    {  328, 7420,   81850,  9, }, // Africa/Algiers
    {  328,   14,   81859,  7, }, // Africa/Asmara
    {  328,   44,   81866,  5, }, // Africa/Bamako
    {  328, 5257,   81871,  6, }, // Africa/Bangui
    {  328, 6746,   81877,  6, }, // Africa/Banjul
    {  328, 7435,   81883,  5, }, // Africa/Bissau
    {  328, 4957,   81888,  8, }, // Africa/Blantyre
    {  328, 5287,   81896, 10, }, // Africa/Brazzaville
    {  328, 4922,   81906,  9, }, // Africa/Bujumbura
    {  328, 2320,   81915,  4, }, // Africa/Cairo
    {  328, 4117,   81919, 11, }, // Africa/Casablanca
    {  328, 7449,   81930,  6, }, // Africa/Ceuta
    {  328, 6773,   81936,  8, }, // Africa/Conakry
    {  328, 6840,   81944,  4, }, // Africa/Dakar
    {  328, 5130,   81948, 11, }, // Africa/Dar_es_Salaam
    {  328, 5043,   81959,  6, }, // Africa/Djibouti
    {  328, 5243,   81965,  5, }, // Africa/Douala
    {  328, 5166,   81970,  6, }, // Africa/El_Aaiun
    {  328, 6853,   81976,  8, }, // Africa/Freetown
    {  328, 4906,   81984,  7, }, // Africa/Gaborone
    {  328, 5015,   81991,  5, }, // Africa/Harare
    {  328, 4479,   81996, 11, }, // Africa/Johannesburg
    {  328, 4499,   82007,  4, }, // Africa/Juba
    {  328, 5151,   82011,  7, }, // Africa/Kampala
    {  328, 4524,   82018,  7, }, // Africa/Khartoum
    {  328, 4973,   82025,  6, }, // Africa/Kigali
    {  328, 5306,   82031,  7, }, // Africa/Kinshasa
    {  328, 4744,   82038,  5, }, // Africa/Lagos
    {  328, 5336,   82043, 10, }, // Africa/Libreville
    {  328, 6869,   82053,  3, }, // Africa/Lome
    {  328, 5211,   82056,  9, }, // Africa/Luanda
    {  328, 4939,   82065, 10, }, // Africa/Lubumbashi
    {  328, 5001,   82075,  6, }, // Africa/Lusaka
    {  328, 5322,   82081,  5, }, // Africa/Malabo
    {  328, 4987,   82086,  6, }, // Africa/Maputo
    {  328, 5197,   82092,  5, }, // Africa/Maseru
    {  328, 5182,   82097,  7, }, // Africa/Mbabane
    {  328, 5113,   82104,  8, }, // Africa/Mogadishu
    {  328, 7034,   82112,  9, }, // Africa/Monrovia
    {  328, 3853,   82121,  6, }, // Africa/Nairobi
    {  328, 5271,   82127,  7, }, // Africa/Ndjamena
    {  328, 5354,   82134,  6, }, // Africa/Niamey
    {  328, 6803,   82140,  6, }, // Africa/Nouakchott
    {  328, 6727,   82146,  8, }, // Africa/Ouagadougou
    {  328, 5225,   82154, 11, }, // Africa/Porto-Novo
    {  328, 4435,   82165,  7, }, // Africa/Sao_Tome
    {  328, 2866,   82172,  8, }, // Africa/Tripoli
    {  328, 6554,   82180,  7, }, // Africa/Tunis
    {  328, 4152,   82187,  7, }, // Africa/Windhoek
    {  328,  132,   82194,  3, }, // America/Adak
    {  328, 3346,   82197,  8, }, // America/Anchorage
    {  328, 5607,   82205,  8, }, // America/Anguilla
    {  328, 5624,   82213,  8, }, // America/Antigua
    {  328, 4554,   82221,  8, }, // America/Araguaina
    {  328,  166,   82229, 12, }, // America/Argentina/Buenos_Aires
    {  328,   91,   82241, 10, }, // America/Argentina/Catamarca
    {  328,  270,   82251,  8, }, // America/Argentina/Cordoba
    {  328,  441,   82259,  5, }, // America/Argentina/Jujuy
    {  328, 7462,   82264,  8, }, // America/Argentina/La_Rioja
    {  328,  565,   82272,  8, }, // America/Argentina/Mendoza
    {  328, 7489,   82280, 11, }, // America/Argentina/Rio_Gallegos
    {  328, 7520,   82291,  6, }, // America/Argentina/Salta
    {  328, 7544,   82297,  8, }, // America/Argentina/San_Juan
    {  328, 5580,   82305,  8, }, // America/Argentina/San_Luis
    {  328, 7571,   82313,  7, }, // America/Argentina/Tucuman
    {  328, 7597,   82320,  5, }, // America/Argentina/Ushuaia
    {  328, 5640,   82325,  5, }, // America/Aruba
    {  328, 4252,   82330,  8, }, // America/Asuncion
    {  328,  237,   82338,  8, }, // America/Atikokan
    {  328, 3679,   82346,  6, }, // America/Bahia
    {  328, 7623,   82352, 14, }, // America/Bahia_Banderas
    {  328, 5654,   82366,  8, }, // America/Barbados
    {  328, 7646,   82374,  5, }, // America/Belem
    {  328, 5407,   82379,  5, }, // America/Belize
    {  328, 7660,   82384, 13, }, // America/Blanc-Sablon
    {  328, 7681,   82397, 10, }, // America/Boa_Vista
    {  328, 4361,   82407,  6, }, // America/Bogota
    {  328, 7699,   82413,  4, }, // America/Boise
    {  328, 7713,   82417, 12, }, // America/Cambridge_Bay
    {  328, 7735,   82429, 13, }, // America/Campo_Grande
    {  328, 3868,   82442,  7, }, // America/Cancun
    {  328, 4694,   82449,  6, }, // America/Caracas
    {  328, 4345,   82455,  6, }, // America/Cayenne
    {  328, 5496,   82461,  5, }, // America/Cayman
    {  328, 2260,   82466,  6, }, // America/Chicago
    {  328, 7756,   82472,  8, }, // America/Chihuahua
    {  328, 7774,   82480, 14, }, // America/Ciudad_Juarez
    {  328, 5422,   82494, 11, }, // America/Costa_Rica
    {  328, 7796,   82505,  8, }, // America/Creston
    {  328, 3791,   82513,  7, }, // America/Cuiaba
    {  328, 5723,   82520,  7, }, // America/Curacao
    {  328, 7812,   82527, 11, }, // America/Danmarkshavn
    {  328, 7833,   82538,  4, }, // America/Dawson
    {  328, 7848,   82542, 10, }, // America/Dawson_Creek
    {  328,  805,   82552,  6, }, // America/Denver
    {  328, 3465,   82558,  8, }, // America/Detroit
    {  328, 5739,   82566,  8, }, // America/Dominica
    {  328,  893,   82574,  8, }, // America/Edmonton
    {  328, 7869,   82582,  6, }, // America/Eirunepe
    {  328, 5441,   82588, 12, }, // America/El_Salvador
    {  328, 7886,   82600, 12, }, // America/Fort_Nelson
    {  328, 7906,   82612, 10, }, // America/Fortaleza
    {  328, 7924,   82622,  8, }, // America/Glace_Bay
    {  328, 6881,   82630,  6, }, // America/Goose_Bay
    {  328, 4612,   82636, 12, }, // America/Grand_Turk
    {  328, 5770,   82648,  8, }, // America/Grenada
    {  328, 5786,   82656,  9, }, // America/Guadeloupe
    {  328, 3760,   82665, 10, }, // America/Guatemala
    {  328, 4841,   82675,  9, }, // America/Guayaquil
    {  328, 6932,   82684,  5, }, // America/Guyana
    {  328, 1939,   82689,  9, }, // America/Halifax
    {  328, 2281,   82698,  5, }, // America/Havana
    {  328, 7942,   82703,  8, }, // America/Hermosillo
    {  328,  348,   82711, 14, }, // America/Indiana/Indianapolis
    {  328,  481,   82725, 16, }, // America/Indiana/Knox
    {  328, 7961,   82741, 18, }, // America/Indiana/Marengo
    {  328, 7985,   82759, 21, }, // America/Indiana/Petersburg
    {  328, 8012,   82780, 19, }, // America/Indiana/Tell_City
    {  328, 8038,   82799, 15, }, // America/Indiana/Vevay
    {  328, 8060,   82814, 20, }, // America/Indiana/Vincennes
    {  328, 8086,   82834, 18, }, // America/Indiana/Winamac
    {  328, 8110,   82852,  6, }, // America/Inuvik
    {  328,  660,   82858,  7, }, // America/Iqaluit
    {  328, 2799,   82865,  5, }, // America/Jamaica
    {  328, 5380,   82870,  5, }, // America/Juneau
    {  328,  521,   82875,  6, }, // America/Kentucky/Louisville
    {  328, 8125,   82881, 20, }, // America/Kentucky/Monticello
    {  328, 5704,   82901, 11, }, // America/Kralendijk
    {  328, 4376,   82912,  6, }, // America/La_Paz
    {  328, 7169,   82918,  4, }, // America/Lima
    {  328, 3239,   82922, 11, }, // America/Los_Angeles
    {  328, 5932,   82933, 24, }, // America/Lower_Princes
    {  328, 8153,   82957,  5, }, // America/Maceio
    {  328, 8168,   82962,  7, }, // America/Managua
    {  328, 1908,   82969,  4, }, // America/Manaus
    {  328, 5897,   82973,  7, }, // America/Marigot
    {  328, 5805,   82980,  9, }, // America/Martinique
    {  328, 8184,   82989,  9, }, // America/Matamoros
    {  328, 2917,   82998,  7, }, // America/Mazatlan
    {  328, 8202,   83005,  8, }, // America/Menominee
    {  328, 8220,   83013,  6, }, // America/Merida
    {  328, 8235,   83019, 12, }, // America/Metlakatla
    {  328, 2949,   83031, 13, }, // America/Mexico_City
    {  328, 4391,   83044,  9, }, // America/Miquelon
    {  328, 8254,   83053,  7, }, // America/Moncton
    {  328, 8270,   83060,  8, }, // America/Monterrey
    {  328, 4098,   83068, 10, }, // America/Montevideo
    {  328, 5824,   83078,  9, }, // America/Montserrat
    {  328, 5481,   83087,  5, }, // America/Nassau
    {  328, 2379,   83092,  9, }, // America/New_York
    {  328, 8288,   83007,  3, }, // America/Nome
    {  328, 1850,   83101,  8, }, // America/Noronha
    {  328, 8301,   83109, 21, }, // America/North_Dakota/Beulah
    {  328, 8329,   83130, 20, }, // America/North_Dakota/Center
    {  328, 8357,   83150, 23, }, // America/North_Dakota/New_Salem
    {  328,  393,   83173,  3, }, // America/Nuuk
    {  328, 8388,   83176,  7, }, // America/Ojinaga
    {  328, 2356,   83183,  5, }, // America/Panama
    {  328, 6173,   83188, 10, }, // America/Paramaribo
    {  328, 2973,   83198,  7, }, // America/Phoenix
    {  328, 3945,   83205, 15, }, // America/Port-au-Prince
    {  328, 5954,   83220, 14, }, // America/Port_of_Spain
    {  328, 8404,   83234, 13, }, // America/Porto_Velho
    {  328, 5843,   83247, 14, }, // America/Puerto_Rico
    {  328, 4030,   83261, 13, }, // America/Punta_Arenas
    {  328, 8424,   83274, 13, }, // America/Rankin_Inlet
    {  328, 8445,   83287,  5, }, // America/Recife
    {  328, 1995,   83292,  6, }, // America/Regina
    {  328, 8460,   83298,  7, }, // America/Resolute
    {  328,  695,   83305, 12, }, // America/Rio_Branco
    {  328, 8477,   83317,  9, }, // America/Santarem
    {  328, 2201,   83326,  9, }, // America/Santiago
    {  328, 6111,   83335, 13, }, // America/Santo_Domingo
    {  328, 1878,   83348,  9, }, // America/Sao_Paulo
    {  328, 6899,   83357, 14, }, // America/Scoresbysund
    {  328, 8494,   83371,  6, }, // America/Sitka
    {  328, 8508,   83377, 15, }, // America/St_Barthelemy
    {  328, 2061,   83392, 12, }, // America/St_Johns
    {  328, 5863,   83404, 10, }, // America/St_Kitts
    {  328, 5880,   83414, 11, }, // America/St_Lucia
    {  328,  855,   83425,  9, }, // America/St_Thomas
    {  328, 5913,   83434, 14, }, // America/St_Vincent
    {  328, 8530,   83448, 12, }, // America/Swift_Current
    {  328, 5461,   83460, 11, }, // America/Tegucigalpa
    {  328, 5756,   83471,  4, }, // America/Thule
    {  328,  313,   83475,  7, }, // America/Tijuana
    {  328,  608,   83482,  7, }, // America/Toronto
    {  328, 5688,   83489,  8, }, // America/Tortola
    {  328, 2093,   83497,  8, }, // America/Vancouver
    {  328, 2144,   83505, 11, }, // America/Whitehorse
    {  328,  734,   83516,  9, }, // America/Winnipeg
    {  328, 8552,   83525,  5, }, // America/Yakutat
    {  328, 6001,   83530,  4, }, // Antarctica/Casey
    {  328, 6094,   83534,  5, }, // Antarctica/Davis
    {  328, 6133,   83539, 17, }, // Antarctica/DumontDUrville
    {  328, 8568,   83556,  8, }, // Antarctica/Macquarie
    {  328, 7066,   83564,  4, }, // Antarctica/Mawson
    {  328, 7113,   83568,  8, }, // Antarctica/McMurdo
    {  328, 8589,   83576,  6, }, // Antarctica/Palmer
    {  328, 7226,   83582,  6, }, // Antarctica/Rothera
    {  328, 7295,   83588,  6, }, // Antarctica/Syowa
    {  328, 8607,   83594,  5, }, // Antarctica/Troll
    {  328, 7374,   83599,  7, }, // Antarctica/Vostok
    {  328, 1429,   83606, 12, }, // Arctic/Longyearbyen
    {  328, 5570,   83618,  4, }, // Asia/Aden
    {  328, 5395,   83622,  7, }, // Asia/Almaty
    {  328, 3968,   83629,  6, }, // Asia/Amman
    {  328, 5511,   83635,  5, }, // Asia/Anadyr
    {  328, 5523,   83640,  5, }, // Asia/Aqtau
    {  328, 5368,   83645,  6, }, // Asia/Aqtobe
    {  328,  964,   83651,  6, }, // Asia/Ashgabat
    {  328, 8624,   83657,  7, }, // Asia/Atyrau
    {  328, 3607,   83664,  5, }, // Asia/Baghdad
    {  328, 5534,   83669,  6, }, // Asia/Bahrain
    {  328, 3653,   82518,  4, }, // Asia/Baku
    {  328, 4466,   83675,  7, }, // Asia/Bangkok
    {  328, 3571,   83682,  6, }, // Asia/Barnaul
    {  328, 4086,   83688,  5, }, // Asia/Beirut
    {  328, 3778,   83693,  6, }, // Asia/Bishkek
    {  328, 5989,   83699,  7, }, // Asia/Brunei
    {  328, 4601,   83706,  4, }, // Asia/Chita
    {  328, 4511,   83710,  6, }, // Asia/Colombo
    {  328, 4540,   83716,  7, }, // Asia/Damascus
    {  328, 1093,   83723,  4, }, // Asia/Dhaka
    {  328, 6192,   83727,  4, }, // Asia/Dili
    {  328, 3596,   83731,  4, }, // Asia/Dubai
    {  328, 6159,   83735,  8, }, // Asia/Dushanbe
    {  328, 8636,   83743, 10, }, // Asia/Famagusta
    {  328, 8651,   83753,  4, }, // Asia/Gaza
    {  328, 4795,   83757,  7, }, // Asia/Hebron
    {  328, 1259,   83764, 14, }, // Asia/Ho_Chi_Minh
    {  328, 2704,   83778,  8, }, // Asia/Hong_Kong
    {  328, 4771,   83786,  5, }, // Asia/Hovd
    {  328, 4184,   83791, 10, }, // Asia/Irkutsk
    {  328, 7006,   83801,  7, }, // Asia/Jakarta
    {  328, 6992,   83808,  7, }, // Asia/Jayapura
    {  328, 1290,   83815,  8, }, // Asia/Jerusalem
    {  328, 3560,   83823,  5, }, // Asia/Kabul
    {  328, 4316,   83828,  8, }, // Asia/Kamchatka
    {  328, 4239,   83836,  5, }, // Asia/Karachi
    {  328, 1185,   83841,  7, }, // Asia/Kathmandu
    {  328, 8661,   83848,  6, }, // Asia/Khandyga
    {  328,  992,   83854,  7, }, // Asia/Kolkata
    {  328, 4197,   83861, 15, }, // Asia/Krasnoyarsk
    {  328, 4859,   83876, 13, }, // Asia/Kuala_Lumpur
    {  328, 5976,   83889,  6, }, // Asia/Kuching
    {  328, 5547,   83895,  5, }, // Asia/Kuwait
    {  328, 1211,   83900,  4, }, // Asia/Macau
    {  328, 4017,   83904,  6, }, // Asia/Magadan
    {  328, 1349,   83910,  7, }, // Asia/Makassar
    {  328, 7182,   83917,  5, }, // Asia/Manila
    {  328, 6920,   83922,  5, }, // Asia/Muscat
    {  328, 2554,   83927,  8, }, // Asia/Nicosia
    {  328, 8675,   83935, 15, }, // Asia/Novokuznetsk
    {  328, 4135,   83950, 15, }, // Asia/Novosibirsk
    {  328, 4229,   83965,  6, }, // Asia/Omsk
    {  328, 7145,   83971,  3, }, // Asia/Oral
    {  328, 6961,   83974, 11, }, // Asia/Phnom_Penh
    {  328, 8693,   83985, 10, }, // Asia/Pontianak
    {  328, 4214,   83995, 10, }, // Asia/Pyongyang
    {  328, 5559,   84005,  4, }, // Asia/Qatar
    {  328, 8708,   84009,  8, }, // Asia/Qostanay
    {  328, 4269,   84017, 10, }, // Asia/Qyzylorda
    {  328, 3584,   84027,  5, }, // Asia/Riyadh
    {  328, 4408,   84032,  6, }, // Asia/Sakhalin
    {  328, 7245,   84038,  7, }, // Asia/Samarkand
    {  328, 3279,   84045,  4, }, // Asia/Seoul
    {  328, 1053,   84049,  6, }, // Asia/Shanghai
    {  328, 3300,   84055,  8, }, // Asia/Singapore
    {  328, 4297,   84063, 17, }, // Asia/Srednekolymsk
    {  328, 3263,   84080,  6, }, // Asia/Taipei
    {  328, 4781,   84086,  8, }, // Asia/Tashkent
    {  328, 3915,   84094,  6, }, // Asia/Tbilisi
    {  328, 2772,   84100,  6, }, // Asia/Tehran
    {  328, 1317,   84106,  6, }, // Asia/Thimphu
    {  328, 2821,   84112,  6, }, // Asia/Tokyo
    {  328, 4572,   84118,  6, }, // Asia/Tomsk
    {  328, 1021,   84124,  8, }, // Asia/Ulaanbaatar
    {  328, 1159,   84132,  7, }, // Asia/Urumqi
    {  328, 8722,   84139,  9, }, // Asia/Ust-Nera
    {  328, 6977,   84148, 10, }, // Asia/Vientiane
    {  328, 4710,   84158, 13, }, // Asia/Vladivostok
    {  328, 4828,   84171,  7, }, // Asia/Yakutsk
    {  328, 1235,   84178,  5, }, // Asia/Yangon
    {  328, 3883,   84183, 13, }, // Asia/Yekaterinburg
    {  328, 3747,   84196,  7, }, // Asia/Yerevan
    {  328, 3663,   84203,  6, }, // Atlantic/Azores
    {  328, 5671,   84209,  7, }, // Atlantic/Bermuda
    {  328, 6628,   84216,  6, }, // Atlantic/Canary
    {  328, 3727,   84222,  9, }, // Atlantic/Cape_Verde
    {  328, 1395,   84231,  4, }, // Atlantic/Faroe
    {  328, 8736,   84235,  5, }, // Atlantic/Madeira
    {  328, 2748,   84240,  8, }, // Atlantic/Reykjavik
    {  328, 7272,   84248, 17, }, // Atlantic/South_Georgia
    {  328, 6821,   84265, 11, }, // Atlantic/St_Helena
    {  328, 6644,   84276,  7, }, // Atlantic/Stanley
    {  328, 1670,   84283,  7, }, // Australia/Adelaide
    {  328, 1635,   84290,  8, }, // Australia/Brisbane
    {  328, 1799,   84298, 10, }, // Australia/Broken_Hill
    {  328, 1583,   84308,  7, }, // Australia/Darwin
    {  328, 3637,   84315,  5, }, // Australia/Eucla
    {  328, 1516,   84320,  7, }, // Australia/Hobart
    {  328, 8753,   84327,  9, }, // Australia/Lindeman
    {  328, 1547,   84336,  9, }, // Australia/Lord_Howe
    {  328, 1727,   84345,  8, }, // Australia/Melbourne
    {  328, 1762,   84353,  4, }, // Australia/Perth
    {  328, 1463,   84357,  5, }, // Australia/Sydney
    {  328, 6441,   84362, 10, }, // Europe/Amsterdam
    {  328, 6216,   84372,  6, }, // Europe/Andorra
    {  328, 3620,   84378,  8, }, // Europe/Astrakhan
    {  328, 2300,   84386,  5, }, // Europe/Athens
    {  328, 6336,   84391,  8, }, // Europe/Belgrade
    {  328, 4757,   84399,  6, }, // Europe/Berlin
    {  328, 6488,   84405, 11, }, // Europe/Bratislava
    {  328, 2167,   84416,  8, }, // Europe/Brussels
    {  328, 3928,   84424,  8, }, // Europe/Bucharest
    {  328, 3806,   84432,  9, }, // Europe/Budapest
    {  328, 8772,   84441,  8, }, // Europe/Busingen
    {  328, 2583,   84449,  7, }, // Europe/Chisinau
    {  328, 6289,   84456,  8, }, // Europe/Copenhagen
    {  328, 2338,   84464,  5, }, // Europe/Dublin
    {  328, 6307,   84469, 10, }, // Europe/Gibraltar
    {  328, 8788,   84479,  6, }, // Europe/Guernsey
    {  328, 6612,   84485,  8, }, // Europe/Helsinki
    {  328, 8804,   84493, 10, }, // Europe/Isle_of_Man
    {  328, 1130,   84503,  9, }, // Europe/Istanbul
    {  328, 8823,   84512,  5, }, // Europe/Jersey
    {  328, 3979,   84517, 12, }, // Europe/Kaliningrad
    {  328, 8929,   84529,  5, }, // Europe/Kirov
    {  328, 2527,   84534,  3, }, // Europe/Kyiv
    {  328, 3213,   84537,  6, }, // Europe/Lisbon
    {  328, 6506,   84543, 10, }, // Europe/Ljubljana
    {  328, 2501,   84553,  4, }, // Europe/London
    {  328, 6365,   84557,  9, }, // Europe/Luxembourg
    {  328, 4892,   84566,  7, }, // Europe/Madrid
    {  328, 6397,   84573,  6, }, // Europe/Malta
    {  328, 6582,   84579,  7, }, // Europe/Mariehamn
    {  328, 3693,   84586,  6, }, // Europe/Minsk
    {  328, 6410,   84592,  6, }, // Europe/Monaco
    {  328, 3537,   84598,  6, }, // Europe/Moscow
    {  328, 6458,   84604,  5, }, // Europe/Oslo
    {  328, 4284,   84609,  5, }, // Europe/Paris
    {  328, 6424,   84614,  9, }, // Europe/Podgorica
    {  328, 6275,   84623,  5, }, // Europe/Prague
    {  328, 8837,   84628,  4, }, // Europe/Riga
    {  328, 6324,   83366,  3, }, // Europe/Rome
    {  328, 4331,   84632,  5, }, // Europe/Samara
    {  328, 6470,   84637, 10, }, // Europe/San_Marino
    {  328, 6245,   84647,  8, }, // Europe/Sarajevo
    {  328, 4451,   84655,  7, }, // Europe/Saratov
    {  328, 8849,   84662, 11, }, // Europe/Simferopol
    {  328, 6383,   84673,  7, }, // Europe/Skopje
    {  328, 6599,   84680,  6, }, // Europe/Sofia
    {  328, 6523,   84686,  8, }, // Europe/Stockholm
    {  328, 8867,   84694,  6, }, // Europe/Tallinn
    {  328, 6202,   84700,  6, }, // Europe/Tirane
    {  328, 8882,   84706, 10, }, // Europe/Ulyanovsk
    {  328, 6352,   84716,  5, }, // Europe/Vaduz
    {  328, 6567,   84721,  6, }, // Europe/Vatican
    {  328, 6231,   84727,  6, }, // Europe/Vienna
    {  328, 8899,   84733,  8, }, // Europe/Vilnius
    {  328, 4727,   84741, 11, }, // Europe/Volgograd
    {  328, 3190,   84752,  5, }, // Europe/Warsaw
    {  328, 6261,   84757,  6, }, // Europe/Zagreb
    {  328, 6540,   84763,  5, }, // Europe/Zurich
    {  328, 5078,   84768, 12, }, // Indian/Antananarivo
    {  328, 6947,   84780,  5, }, // Indian/Chagos
    {  328, 6046,   84785,  7, }, // Indian/Christmas
    {  328, 6063,   84792,  5, }, // Indian/Cocos
    {  328, 5029,   84797,  6, }, // Indian/Comoro
    {  328, 6661,   84803, 10, }, // Indian/Kerguelen
    {  328, 7260,   84813,  4, }, // Indian/Mahe
    {  328, 7050,   84817,  8, }, // Indian/Maldives
    {  328, 4069,   84825,  8, }, // Indian/Mauritius
    {  328, 5098,   84833,  5, }, // Indian/Mayotte
    {  328, 7211,   84838,  8, }, // Indian/Reunion
    {  328, 4422,   84846,  5, }, // Pacific/Apia
    {  328,  932,   84851,  7, }, // Pacific/Auckland
    {  328, 3706,   84858, 10, }, // Pacific/Bougainville
    {  328, 3015,   84868,  3, }, // Pacific/Chatham
    {  328, 3157,   84871,  3, }, // Pacific/Chuuk
    {  328, 2237,   84874,  5, }, // Pacific/Easter
    {  328, 7360,   84879,  4, }, // Pacific/Efate
    {  328, 7327,   84883,  7, }, // Pacific/Fakaofo
    {  328, 3902,   84890,  4, }, // Pacific/Fiji
    {  328, 7343,   84894,  8, }, // Pacific/Funafuti
    {  328, 6678,   84902,  9, }, // Pacific/Galapagos
    {  328, 6696,   84911,  8, }, // Pacific/Gambier
    {  328, 3822,   84919,  8, }, // Pacific/Guadalcanal
    {  328, 6018,   84927,  5, }, // Pacific/Guam
    {  328, 2723,   84932,  8, }, // Pacific/Honolulu
    {  328, 3049,   84940,  8, }, // Pacific/Kanton
    {  328, 3998,   84948, 10, }, // Pacific/Kiritimati
    {  328, 7019,   84958,  5, }, // Pacific/Kosrae
    {  328, 2842,   84963,  9, }, // Pacific/Kwajalein
    {  328, 4877,   84972,  6, }, // Pacific/Majuro
    {  328, 4051,   84978,  9, }, // Pacific/Marquesas
    {  328, 8914,   84987,  5, }, // Pacific/Midway
    {  328, 7084,   84992,  4, }, // Pacific/Nauru
    {  328, 7132,   84996,  4, }, // Pacific/Niue
    {  328, 4168,   85000,  6, }, // Pacific/Norfolk
    {  328, 7098,   85006,  5, }, // Pacific/Noumea
    {  328, 3126,   85011,  9, }, // Pacific/Pago_Pago
    {  328, 7155,   85020,  4, }, // Pacific/Palau
    {  328, 7194,   85024,  8, }, // Pacific/Pitcairn
    {  328, 3096,   85032,  6, }, // Pacific/Pohnpei
    {  328, 4807,   85038, 14, }, // Pacific/Port_Moresby
    {  328, 6076,   85052,  9, }, // Pacific/Rarotonga
    {  328, 6031,   85061,  5, }, // Pacific/Saipan
    {  328, 7312,   85066,  6, }, // Pacific/Tahiti
    {  328, 6712,   85072,  6, }, // Pacific/Tarawa
    {  328, 4583,   85078,  9, }, // Pacific/Tongatapu
    {  328, 7392,   85087,  3, }, // Pacific/Wake
    {  328, 7405,   85090,  5, }, // Pacific/Wallis
    {  329,   14,       0,  6, }, // Africa/Asmara Gusii/Latin/Kenya
    {  329, 4435,       6,  8, }, // Africa/Sao_Tome
    {  329, 4252,      14,  8, }, // America/Asuncion
    {  329,  237,      22,  8, }, // America/Atikokan
    {  329, 7623,      30, 17, }, // America/Bahia_Banderas
    {  329, 3868,      47,  6, }, // America/Cancun
    {  329, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  329, 5723,      66,  7, }, // America/Curacao
    {  329,  481,      73, 13, }, // America/Indiana/Knox
    {  329, 7961,      86, 16, }, // America/Indiana/Marengo
    {  329, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  329, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  329, 8038,     139, 14, }, // America/Indiana/Vevay
    {  329, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  329, 8086,     171, 16, }, // America/Indiana/Winamac
    {  329, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  329, 5932,     207, 22, }, // America/Lower_Princes
    {  329, 8220,     229,  6, }, // America/Merida
    {  329, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  329, 8329,     255, 20, }, // America/North_Dakota/Center
    {  329, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  329,  393,     298,  4, }, // America/Nuuk
    {  329, 6899,     302, 16, }, // America/Scoresbysund
    {  329, 8508,     318, 14, }, // America/St_Barthelemy
    {  329, 2061,     332, 10, }, // America/St_Johns
    {  329, 5863,     342,  9, }, // America/St_Kitts
    {  329, 5880,     351,  9, }, // America/St_Lucia
    {  329,  855,     360, 10, }, // America/St_Thomas
    {  329, 5913,     370, 11, }, // America/St_Vincent
    {  329, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  329, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  329, 1185,     413,  9, }, // Asia/Kathmandu
    {  329,  992,     422,  7, }, // Asia/Kolkata
    {  329, 1211,     429,  5, }, // Asia/Macau
    {  329, 1235,     442,  6, }, // Asia/Yangon
    {  329, 1395,     448,  5, }, // Atlantic/Faroe
    {  329, 6821,     453, 10, }, // Atlantic/St_Helena
    {  329, 2527,     463,  4, }, // Europe/Kyiv
    {  329, 7211,     467,  7, }, // Indian/Reunion
    {  329, 3157,     474,  5, }, // Pacific/Chuuk
    {  329, 3096,     479,  7, }, // Pacific/Pohnpei
    {  330,   14,       0,  6, }, // Africa/Asmara Hausa/Latin/Nigeria
    {  330, 4435,       6,  8, }, // Africa/Sao_Tome
    {  330, 4252,      14,  8, }, // America/Asuncion
    {  330,  237,      22,  8, }, // America/Atikokan
    {  330, 7623,      30, 17, }, // America/Bahia_Banderas
    {  330, 3868,      47,  6, }, // America/Cancun
    {  330, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  330, 5723,      66,  7, }, // America/Curacao
    {  330,  481,      73, 13, }, // America/Indiana/Knox
    {  330, 7961,      86, 16, }, // America/Indiana/Marengo
    {  330, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  330, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  330, 8038,     139, 14, }, // America/Indiana/Vevay
    {  330, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  330, 8086,     171, 16, }, // America/Indiana/Winamac
    {  330, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  330, 5932,     207, 22, }, // America/Lower_Princes
    {  330, 8220,     229,  6, }, // America/Merida
    {  330, 8301,   85095, 23, }, // America/North_Dakota/Beulah
    {  330, 8329,   85118, 23, }, // America/North_Dakota/Center
    {  330, 8357,   85141, 26, }, // America/North_Dakota/New_Salem
    {  330,  393,     298,  4, }, // America/Nuuk
    {  330, 6899,     302, 16, }, // America/Scoresbysund
    {  330, 8508,     318, 14, }, // America/St_Barthelemy
    {  330, 2061,     332, 10, }, // America/St_Johns
    {  330, 5863,     342,  9, }, // America/St_Kitts
    {  330, 5880,     351,  9, }, // America/St_Lucia
    {  330,  855,     360, 10, }, // America/St_Thomas
    {  330, 5913,     370, 11, }, // America/St_Vincent
    {  330, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  330, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  330, 1185,     413,  9, }, // Asia/Kathmandu
    {  330,  992,     422,  7, }, // Asia/Kolkata
    {  330, 1211,     429,  5, }, // Asia/Macau
    {  330, 1235,     442,  6, }, // Asia/Yangon
    {  330, 1395,     448,  5, }, // Atlantic/Faroe
    {  330, 6821,     453, 10, }, // Atlantic/St_Helena
    {  330, 2527,     463,  4, }, // Europe/Kyiv
    {  330, 7211,     467,  7, }, // Indian/Reunion
    {  330, 3157,     474,  5, }, // Pacific/Chuuk
    {  330, 3096,     479,  7, }, // Pacific/Pohnpei
    {  331, 8301,     235, 20, }, // America/North_Dakota/Beulah Hausa/Arabic/Nigeria
    {  331, 8329,     255, 20, }, // America/North_Dakota/Center
    {  331, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  332, 8301,     235, 20, }, // America/North_Dakota/Beulah Hausa/Arabic/Sudan
    {  332, 8329,     255, 20, }, // America/North_Dakota/Center
    {  332, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  335,   14,       0,  6, }, // Africa/Asmara Hawaiian/Latin/United States
    {  335, 4435,       6,  8, }, // Africa/Sao_Tome
    {  335, 4252,      14,  8, }, // America/Asuncion
    {  335,  237,      22,  8, }, // America/Atikokan
    {  335, 7623,      30, 17, }, // America/Bahia_Banderas
    {  335, 3868,      47,  6, }, // America/Cancun
    {  335, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  335, 5723,      66,  7, }, // America/Curacao
    {  335,  481,      73, 13, }, // America/Indiana/Knox
    {  335, 7961,      86, 16, }, // America/Indiana/Marengo
    {  335, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  335, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  335, 8038,     139, 14, }, // America/Indiana/Vevay
    {  335, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  335, 8086,     171, 16, }, // America/Indiana/Winamac
    {  335, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  335, 5932,     207, 22, }, // America/Lower_Princes
    {  335, 8220,     229,  6, }, // America/Merida
    {  335, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  335, 8329,     255, 20, }, // America/North_Dakota/Center
    {  335, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  335,  393,     298,  4, }, // America/Nuuk
    {  335, 6899,     302, 16, }, // America/Scoresbysund
    {  335, 8508,     318, 14, }, // America/St_Barthelemy
    {  335, 2061,     332, 10, }, // America/St_Johns
    {  335, 5863,     342,  9, }, // America/St_Kitts
    {  335, 5880,     351,  9, }, // America/St_Lucia
    {  335,  855,     360, 10, }, // America/St_Thomas
    {  335, 5913,     370, 11, }, // America/St_Vincent
    {  335, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  335, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  335, 1185,     413,  9, }, // Asia/Kathmandu
    {  335,  992,     422,  7, }, // Asia/Kolkata
    {  335, 1211,     429,  5, }, // Asia/Macau
    {  335, 1235,     442,  6, }, // Asia/Yangon
    {  335, 1395,     448,  5, }, // Atlantic/Faroe
    {  335, 6821,     453, 10, }, // Atlantic/St_Helena
    {  335, 2527,     463,  4, }, // Europe/Kyiv
    {  335, 7211,     467,  7, }, // Indian/Reunion
    {  335, 3157,     474,  5, }, // Pacific/Chuuk
    {  335, 3096,     479,  7, }, // Pacific/Pohnpei
    {  336, 6788,   85167,  7, }, // Africa/Abidjan Hebrew/Hebrew/Israel
    {  336, 6760,   85174,  4, }, // Africa/Accra
    {  336, 5059,   85178,  9, }, // Africa/Addis_Ababa
    {  336, 7420,   85187,  6, }, // Africa/Algiers
    {  336,   14,   85193,  5, }, // Africa/Asmara
    {  336,   44,   85198,  5, }, // Africa/Bamako
    {  336, 5257,   85203,  6, }, // Africa/Bangui
    {  336, 6746,   85209,  6, }, // Africa/Banjul
    {  336, 7435,   85215,  5, }, // Africa/Bissau
    {  336, 4957,   85220,  6, }, // Africa/Blantyre
    {  336, 5287,   85226,  7, }, // Africa/Brazzaville
    {  336, 4922,   85233, 10, }, // Africa/Bujumbura
    {  336, 2320,   85243,  4, }, // Africa/Cairo
    {  336, 4117,   85247,  7, }, // Africa/Casablanca
    {  336, 7449,   85254,  5, }, // Africa/Ceuta
    {  336, 6773,   85259,  7, }, // Africa/Conakry
    {  336, 6840,   85266,  4, }, // Africa/Dakar
    {  336, 5130,   85270, 10, }, // Africa/Dar_es_Salaam
    {  336, 5043,   85280,  7, }, // Africa/Djibouti
    {  336, 5243,   85287,  5, }, // Africa/Douala
    {  336, 5166,   85292,  7, }, // Africa/El_Aaiun
    {  336, 6853,   85299,  7, }, // Africa/Freetown
    {  336, 4906,   85306,  7, }, // Africa/Gaborone
    {  336, 5015,   85313,  5, }, // Africa/Harare
    {  336, 4479,   85318,  9, }, // Africa/Johannesburg
    {  336, 4499,   85327,  5, }, // Africa/Juba
    {  336, 5151,   85332,  6, }, // Africa/Kampala
    {  336, 4524,   85338,  5, }, // Africa/Khartoum
    {  336, 4973,   85343,  5, }, // Africa/Kigali
    {  336, 5306,   85348,  6, }, // Africa/Kinshasa
    {  336, 4744,   85354,  5, }, // Africa/Lagos
    {  336, 5336,   85359,  8, }, // Africa/Libreville
    {  336, 6869,   85367,  4, }, // Africa/Lome
    {  336, 5211,   85371,  6, }, // Africa/Luanda
    {  336, 4939,   85377,  9, }, // Africa/Lubumbashi
    {  336, 5001,   85386,  5, }, // Africa/Lusaka
    {  336, 5322,   85391,  4, }, // Africa/Malabo
    {  336, 4987,   85395,  6, }, // Africa/Maputo
    {  336, 5197,   85401,  4, }, // Africa/Maseru
    {  336, 5182,   85405,  5, }, // Africa/Mbabane
    {  336, 5113,   85410,  7, }, // Africa/Mogadishu
    {  336, 7034,   85417,  8, }, // Africa/Monrovia
    {  336, 3853,   85425,  7, }, // Africa/Nairobi
    {  336, 5271,   85432,  6, }, // Africa/Ndjamena
    {  336, 5354,   85438,  6, }, // Africa/Niamey
    {  336, 6803,   85444,  8, }, // Africa/Nouakchott
    {  336, 6727,   85452,  9, }, // Africa/Ouagadougou
    {  336, 5225,   85461, 10, }, // Africa/Porto-Novo
    {  336, 4435,   85471,  8, }, // Africa/Sao_Tome
    {  336, 2866,   85479,  7, }, // Africa/Tripoli
    {  336, 6554,   85486,  5, }, // Africa/Tunis
    {  336, 4152,   85491,  7, }, // Africa/Windhoek
    {  336,  132,   85498,  4, }, // America/Adak
    {  336, 3346,   85502,  7, }, // America/Anchorage
    {  336, 5607,   85509,  8, }, // America/Anguilla
    {  336, 5624,   85517,  8, }, // America/Antigua
    {  336, 4554,   85525,  9, }, // America/Araguaina
    {  336,  166,   85534, 12, }, // America/Argentina/Buenos_Aires
    {  336,   91,   85546,  6, }, // America/Argentina/Catamarca
    {  336,  270,   85552,  7, }, // America/Argentina/Cordoba
    {  336,  441,   85559,  5, }, // America/Argentina/Jujuy
    {  336, 7462,   85564,  8, }, // America/Argentina/La_Rioja
    {  336,  565,   85572,  6, }, // America/Argentina/Mendoza
    {  336, 7489,   85578, 11, }, // America/Argentina/Rio_Gallegos
    {  336, 7520,   85589,  4, }, // America/Argentina/Salta
    {  336, 7544,   85593,  7, }, // America/Argentina/San_Juan
    {  336, 5580,   85600,  8, }, // America/Argentina/San_Luis
    {  336, 7571,   85608,  6, }, // America/Argentina/Tucuman
    {  336, 7597,   85614,  8, }, // America/Argentina/Ushuaia
    {  336, 5640,   85622,  5, }, // America/Aruba
    {  336, 4252,   85627,  8, }, // America/Asuncion
    {  336,  237,   85635,  7, }, // America/Atikokan
    {  336, 3679,   85642,  5, }, // America/Bahia
    {  336, 7623,   85647, 11, }, // America/Bahia_Banderas
    {  336, 5654,   85658,  6, }, // America/Barbados
    {  336, 7646,   85664,  3, }, // America/Belem
    {  336, 5407,   85667,  4, }, // America/Belize
    {  336, 7660,   85671, 10, }, // America/Blanc-Sablon
    {  336, 7681,   85681, 10, }, // America/Boa_Vista
    {  336, 4361,   85691,  6, }, // America/Bogota
    {  336, 7699,   85697,  5, }, // America/Boise
    {  336, 7713,   85702, 14, }, // America/Cambridge_Bay
    {  336, 7735,   85716, 10, }, // America/Campo_Grande
    {  336, 3868,   85726,  5, }, // America/Cancun
    {  336, 4694,   85731,  5, }, // America/Caracas
    {  336, 4345,   85736,  5, }, // America/Cayenne
    {  336, 5496,   85741,  5, }, // America/Cayman
    {  336, 2260,   85746,  5, }, // America/Chicago
    {  336, 7756,   85751,  9, }, // America/Chihuahua
    {  336, 7774,   85760, 11, }, // America/Ciudad_Juarez
    {  336, 5422,   85771, 10, }, // America/Costa_Rica
    {  336, 7796,   85781,  6, }, // America/Creston
    {  336, 3791,   85787,  6, }, // America/Cuiaba
    {  336, 5723,   85793,  7, }, // America/Curacao
    {  336, 7812,   85800, 10, }, // America/Danmarkshavn
    {  336, 7833,   85810,  5, }, // America/Dawson
    {  336, 7848,   85815, 10, }, // America/Dawson_Creek
    {  336,  805,   85825,  5, }, // America/Denver
    {  336, 3465,   85830,  6, }, // America/Detroit
    {  336, 5739,   85836,  8, }, // America/Dominica
    {  336,  893,   85844,  8, }, // America/Edmonton
    {  336, 7869,   85852,  7, }, // America/Eirunepe
    {  336, 5441,   85859,  9, }, // America/El_Salvador
    {  336, 7886,   85868, 10, }, // America/Fort_Nelson
    {  336, 7906,   85878,  8, }, // America/Fortaleza
    {  336, 7924,   85886,  9, }, // America/Glace_Bay
    {  336, 6881,   85895,  7, }, // America/Goose_Bay
    {  336, 4612,   85902,  9, }, // America/Grand_Turk
    {  336, 5770,   85721,  5, }, // America/Grenada
    {  336, 5786,   85911,  7, }, // America/Guadeloupe
    {  336, 3760,   85918,  7, }, // America/Guatemala
    {  336, 4841,   85925,  8, }, // America/Guayaquil
    {  336, 6932,   85933,  5, }, // America/Guyana
    {  336, 1939,   85938,  6, }, // America/Halifax
    {  336, 2281,   85944,  6, }, // America/Havana
    {  336, 7942,   85950,  7, }, // America/Hermosillo
    {  336,  348,   85957, 12, }, // America/Indiana/Indianapolis
    {  336,  481,   85969, 14, }, // America/Indiana/Knox
    {  336, 7961,   85983, 15, }, // America/Indiana/Marengo
    {  336, 7985,   85998, 19, }, // America/Indiana/Petersburg
    {  336, 8012,   86017, 17, }, // America/Indiana/Tell_City
    {  336, 8038,   86034, 17, }, // America/Indiana/Vevay
    {  336, 8060,   86051, 16, }, // America/Indiana/Vincennes
    {  336, 8086,   86067, 16, }, // America/Indiana/Winamac
    {  336, 8110,   86083,  7, }, // America/Inuvik
    {  336,  660,   86090,  8, }, // America/Iqaluit
    {  336, 2799,   86098,  7, }, // America/Jamaica
    {  336, 5380,   86105,  5, }, // America/Juneau
    {  336,  521,   86110,  8, }, // America/Kentucky/Louisville
    {  336, 8125,   86118, 17, }, // America/Kentucky/Monticello
    {  336, 5704,   86135,  8, }, // America/Kralendijk
    {  336, 4376,   86143,  6, }, // America/La_Paz
    {  336, 7169,   86149,  4, }, // America/Lima
    {  336, 3239,   86153, 10, }, // America/Los_Angeles
    {  336, 5932,   86163, 20, }, // America/Lower_Princes
    {  336, 8153,   86183,  6, }, // America/Maceio
    {  336, 8168,   86189,  6, }, // America/Managua
    {  336, 1908,   86195,  5, }, // America/Manaus
    {  336, 5897,   86200,  5, }, // America/Marigot
    {  336, 5805,   86205,  7, }, // America/Martinique
    {  336, 8184,   86212,  7, }, // America/Matamoros
    {  336, 2917,   86219,  5, }, // America/Mazatlan
    {  336, 8202,   86224,  7, }, // America/Menominee
    {  336, 8220,   86231,  5, }, // America/Merida
    {  336, 8235,   86236,  7, }, // America/Metlakatla
    {  336, 2949,   86243, 11, }, // America/Mexico_City
    {  336, 4391,   86254,  6, }, // America/Miquelon
    {  336, 8254,   86260,  7, }, // America/Moncton
    {  336, 8270,   86267,  7, }, // America/Monterrey
    {  336, 4098,   86274, 10, }, // America/Montevideo
    {  336, 5824,   86284,  7, }, // America/Montserrat
    {  336, 5481,   86291,  4, }, // America/Nassau
    {  336, 2379,   86295,  8, }, // America/New_York
    {  336, 8288,   86303,  3, }, // America/Nome
    {  336, 1850,   86306,  7, }, // America/Noronha
    {  336, 8301,   86313, 20, }, // America/North_Dakota/Beulah
    {  336, 8329,   86333, 19, }, // America/North_Dakota/Center
    {  336, 8357,   86352, 24, }, // America/North_Dakota/New_Salem
    {  336,  393,   86376,  5, }, // America/Nuuk
    {  336, 8388,   86381,  9, }, // America/Ojinaga
    {  336, 2356,   86390,  4, }, // America/Panama
    {  336, 6173,   86394,  7, }, // America/Paramaribo
    {  336, 2973,   86401,  6, }, // America/Phoenix
    {  336, 3945,   86407, 13, }, // America/Port-au-Prince
    {  336, 5954,   86420, 14, }, // America/Port_of_Spain
    {  336, 8404,   86434, 10, }, // America/Porto_Velho
    {  336, 5843,   86444, 11, }, // America/Puerto_Rico
    {  336, 4030,   86455, 10, }, // America/Punta_Arenas
    {  336, 8424,   86465, 11, }, // America/Rankin_Inlet
    {  336, 8445,   86476,  5, }, // America/Recife
    {  336, 1995,   86481,  6, }, // America/Regina
    {  336, 8460,   86487,  6, }, // America/Resolute
    {  336,  695,   86493,  9, }, // America/Rio_Branco
    {  336, 8477,   86502,  5, }, // America/Santarem
    {  336, 2201,   86507,  7, }, // America/Santiago
    {  336, 6111,   86514, 12, }, // America/Santo_Domingo
    {  336, 1878,   86526,  9, }, // America/Sao_Paulo
    {  336, 6899,   86535, 11, }, // America/Scoresbysund
    {  336, 8494,   86546,  5, }, // America/Sitka
    {  336, 8508,   86551, 10, }, // America/St_Barthelemy
    {  336, 2061,   86561,  9, }, // America/St_Johns
    {  336, 5863,   86570,  8, }, // America/St_Kitts
    {  336, 5880,   86578,  9, }, // America/St_Lucia
    {  336,  855,   86587,  9, }, // America/St_Thomas
    {  336, 5913,   86596, 10, }, // America/St_Vincent
    {  336, 8530,   86606, 11, }, // America/Swift_Current
    {  336, 5461,   86617,  9, }, // America/Tegucigalpa
    {  336, 5756,   86626,  4, }, // America/Thule
    {  336,  313,   86630,  7, }, // America/Tijuana
    {  336,  608,   86637,  7, }, // America/Toronto
    {  336, 5688,   86644,  7, }, // America/Tortola
    {  336, 2093,   86651,  6, }, // America/Vancouver
    {  336, 2144,   86657,  9, }, // America/Whitehorse
    {  336,  734,   86666,  7, }, // America/Winnipeg
    {  336, 8552,   86673,  6, }, // America/Yakutat
    {  336, 6001,   86679,  5, }, // Antarctica/Casey
    {  336, 6094,   86684,  6, }, // Antarctica/Davis
    {  336, 6133,   86690, 15, }, // Antarctica/DumontDUrville
    {  336, 8568,   86705,  7, }, // Antarctica/Macquarie
    {  336, 7066,   86712,  5, }, // Antarctica/Mawson
    {  336, 7113,   86717,  7, }, // Antarctica/McMurdo
    {  336, 8589,   86724,  5, }, // Antarctica/Palmer
    {  336, 7226,   86729,  6, }, // Antarctica/Rothera
    {  336, 7295,   86735,  6, }, // Antarctica/Syowa
    {  336, 8607,   86741,  4, }, // Antarctica/Troll
    {  336, 7374,   86745,  6, }, // Antarctica/Vostok
    {  336, 1429,   86751, 10, }, // Arctic/Longyearbyen
    {  336, 5570,   86761,  3, }, // Asia/Aden
    {  336, 5395,   86764,  6, }, // Asia/Almaty
    {  336, 3968,   86770,  4, }, // Asia/Amman
    {  336, 5511,   86774,  5, }, // Asia/Anadyr
    {  336, 5523,   86779,  5, }, // Asia/Aqtau
    {  336, 5368,   86784,  6, }, // Asia/Aqtobe
    {  336,  964,   86790,  5, }, // Asia/Ashgabat
    {  336, 8624,   86795,  6, }, // Asia/Atyrau
    {  336, 3607,   86801,  4, }, // Asia/Baghdad
    {  336, 5534,   86805,  6, }, // Asia/Bahrain
    {  336, 3653,   86811,  4, }, // Asia/Baku
    {  336, 4466,   86815,  6, }, // Asia/Bangkok
    {  336, 3571,   86821,  6, }, // Asia/Barnaul
    {  336, 4086,   86827,  6, }, // Asia/Beirut
    {  336, 3778,   86833,  5, }, // Asia/Bishkek
    {  336, 5989,   86838,  6, }, // Asia/Brunei
    {  336, 4601,   86844,  5, }, // Asia/Chita
    {  336, 4511,   86849,  7, }, // Asia/Colombo
    {  336, 4540,   86856,  4, }, // Asia/Damascus
    {  336, 1093,   86860,  4, }, // Asia/Dhaka
    {  336, 6192,   86864,  4, }, // Asia/Dili
    {  336, 3596,   86868,  5, }, // Asia/Dubai
    {  336, 6159,   86873,  6, }, // Asia/Dushanbe
    {  336, 8636,   86879,  7, }, // Asia/Famagusta
    {  336, 8651,   86886,  3, }, // Asia/Gaza
    {  336, 4795,   86889,  5, }, // Asia/Hebron
    {  336, 1259,   86894, 15, }, // Asia/Ho_Chi_Minh
    {  336, 2704,   86909,  9, }, // Asia/Hong_Kong
    {  336, 4771,   86918,  4, }, // Asia/Hovd
    {  336, 4184,   86922,  8, }, // Asia/Irkutsk
    {  336, 7006,   86930,  6, }, // Asia/Jakarta
    {  336, 6992,   86936,  9, }, // Asia/Jayapura
    {  336, 1290,   86945,  7, }, // Asia/Jerusalem
    {  336, 3560,   86952,  5, }, // Asia/Kabul
    {  336, 4316,   86957,  7, }, // Asia/Kamchatka
    {  336, 4239,   86964,  6, }, // Asia/Karachi
    {  336, 1185,   86970,  6, }, // Asia/Kathmandu
    {  336, 8661,   86976,  6, }, // Asia/Khandyga
    {  336,  992,   86982,  6, }, // Asia/Kolkata
    {  336, 4197,   86988, 10, }, // Asia/Krasnoyarsk
    {  336, 4859,   86998, 12, }, // Asia/Kuala_Lumpur
    {  336, 5976,   87010,  7, }, // Asia/Kuching
    {  336, 5547,   87017,  5, }, // Asia/Kuwait
    {  336, 1211,   87022,  4, }, // Asia/Macau
    {  336, 4017,   87026,  4, }, // Asia/Magadan
    {  336, 1349,   87030,  7, }, // Asia/Makassar
    {  336, 7182,   87037,  5, }, // Asia/Manila
    {  336, 6920,   87042,  5, }, // Asia/Muscat
    {  336, 2554,   87047,  7, }, // Asia/Nicosia
    {  336, 8675,   87054, 11, }, // Asia/Novokuznetsk
    {  336, 4135,   87065, 11, }, // Asia/Novosibirsk
    {  336, 4229,   87076,  5, }, // Asia/Omsk
    {  336, 7145,   87081,  4, }, // Asia/Oral
    {  336, 6961,   87085,  7, }, // Asia/Phnom_Penh
    {  336, 8693,   87092,  8, }, // Asia/Pontianak
    {  336, 4214,   87100,  9, }, // Asia/Pyongyang
    {  336, 5559,   87109,  4, }, // Asia/Qatar
    {  336, 8708,   87113,  7, }, // Asia/Qostanay
    {  336, 4269,   87120,  9, }, // Asia/Qyzylorda
    {  336, 3584,   87129,  4, }, // Asia/Riyadh
    {  336, 4408,   87133,  5, }, // Asia/Sakhalin
    {  336, 7245,   87138,  6, }, // Asia/Samarkand
    {  336, 3279,   87144,  5, }, // Asia/Seoul
    {  336, 1053,   87149,  5, }, // Asia/Shanghai
    {  336, 3300,   87154,  7, }, // Asia/Singapore
    {  336, 4297,   87161, 13, }, // Asia/Srednekolymsk
    {  336, 3263,   87174,  6, }, // Asia/Taipei
    {  336, 4781,   87180,  5, }, // Asia/Tashkent
    {  336, 3915,   87185,  7, }, // Asia/Tbilisi
    {  336, 2772,   87192,  4, }, // Asia/Tehran
    {  336, 1317,   87196,  7, }, // Asia/Thimphu
    {  336, 2821,   87203,  5, }, // Asia/Tokyo
    {  336, 4572,   87208,  5, }, // Asia/Tomsk
    {  336, 1021,   87213, 11, }, // Asia/Ulaanbaatar
    {  336, 1159,   87224,  7, }, // Asia/Urumqi
    {  336, 8722,   87231,  8, }, // Asia/Ust-Nera
    {  336, 6977,   87239,  8, }, // Asia/Vientiane
    {  336, 4710,   87247, 10, }, // Asia/Vladivostok
    {  336, 4828,   87257,  6, }, // Asia/Yakutsk
    {  336, 1235,   87263,  5, }, // Asia/Yangon
    {  336, 3883,   87268, 10, }, // Asia/Yekaterinburg
    {  336, 3747,   87278,  6, }, // Asia/Yerevan
    {  336, 3663,   87284, 14, }, // Atlantic/Azores
    {  336, 5671,   87298,  6, }, // Atlantic/Bermuda
    {  336, 6628,   87304, 13, }, // Atlantic/Canary
    {  336, 3727,   87317,  7, }, // Atlantic/Cape_Verde
    {  336, 1395,   87324,  4, }, // Atlantic/Faroe
    {  336, 8736,   87328,  6, }, // Atlantic/Madeira
    {  336, 2748,   87334, 10, }, // Atlantic/Reykjavik
    {  336, 7272,   87344, 13, }, // Atlantic/South_Georgia
    {  336, 6821,   87357,  8, }, // Atlantic/St_Helena
    {  336, 6644,   87365,  5, }, // Atlantic/Stanley
    {  336, 1670,   87370,  6, }, // Australia/Adelaide
    {  336, 1635,   87376,  8, }, // Australia/Brisbane
    {  336, 1799,   87384,  9, }, // Australia/Broken_Hill
    {  336, 1583,   87393,  6, }, // Australia/Darwin
    {  336, 3637,   87399,  5, }, // Australia/Eucla
    {  336, 1516,   87404,  5, }, // Australia/Hobart
    {  336, 8753,   87409,  6, }, // Australia/Lindeman
    {  336, 1547,   87415, 12, }, // Australia/Lord_Howe
    {  336, 1727,   87427,  6, }, // Australia/Melbourne
    {  336, 1762,   87433,  4, }, // Australia/Perth
    {  336, 1463,   87437,  5, }, // Australia/Sydney
    {  336, 6441,   87442,  7, }, // Europe/Amsterdam
    {  336, 6216,   87449,  6, }, // Europe/Andorra
    {  336, 3620,   87455,  6, }, // Europe/Astrakhan
    {  336, 2300,   87461,  5, }, // Europe/Athens
    {  336, 6336,   87466,  5, }, // Europe/Belgrade
    {  336, 4757,   87471,  5, }, // Europe/Berlin
    {  336, 6488,   87476,  8, }, // Europe/Bratislava
    {  336, 2167,   87484,  5, }, // Europe/Brussels
    {  336, 3928,   87489,  6, }, // Europe/Bucharest
    {  336, 3806,   87495,  6, }, // Europe/Budapest
    {  336, 8772,   87501,  7, }, // Europe/Busingen
    {  336, 2583,   87508,  6, }, // Europe/Chisinau
    {  336, 6289,   87514,  7, }, // Europe/Copenhagen
    {  336, 2338,   87521,  5, }, // Europe/Dublin
    {  336, 6307,   87526,  7, }, // Europe/Gibraltar
    {  336, 8788,   87533,  5, }, // Europe/Guernsey
    {  336, 6612,   87538,  7, }, // Europe/Helsinki
    {  336, 8804,   87545,  7, }, // Europe/Isle_of_Man
    {  336, 1130,   87552,  8, }, // Europe/Istanbul
    {  336, 8823,   87560,  5, }, // Europe/Jersey
    {  336, 3979,   87565,  9, }, // Europe/Kaliningrad
    {  336, 8929,   87574,  5, }, // Europe/Kirov
    {  336, 2527,   87579,  4, }, // Europe/Kyiv
    {  336, 3213,   87583,  6, }, // Europe/Lisbon
    {  336, 6506,   87589,  8, }, // Europe/Ljubljana
    {  336, 2501,   87597,  6, }, // Europe/London
    {  336, 6365,   87603,  9, }, // Europe/Luxembourg
    {  336, 4892,   87612,  5, }, // Europe/Madrid
    {  336, 6397,   87617,  4, }, // Europe/Malta
    {  336, 6582,   87621,  8, }, // Europe/Mariehamn
    {  336, 3693,   87629,  5, }, // Europe/Minsk
    {  336, 6410,   87634,  5, }, // Europe/Monaco
    {  336, 3537,   87639,  6, }, // Europe/Moscow
    {  336, 6458,   87645,  5, }, // Europe/Oslo
    {  336, 4284,   87650,  4, }, // Europe/Paris
    {  336, 6424,   87654,  9, }, // Europe/Podgorica
    {  336, 6275,   87663,  4, }, // Europe/Prague
    {  336, 8837,   87667,  4, }, // Europe/Riga
    {  336, 6324,   87671,  4, }, // Europe/Rome
    {  336, 4331,   85194,  4, }, // Europe/Samara
    {  336, 6470,   87675,  8, }, // Europe/San_Marino
    {  336, 6245,   87683,  6, }, // Europe/Sarajevo
    {  336, 4451,   87689,  6, }, // Europe/Saratov
    {  336, 8849,   87695,  9, }, // Europe/Simferopol
    {  336, 6383,   87704,  6, }, // Europe/Skopje
    {  336, 6599,   87710,  5, }, // Europe/Sofia
    {  336, 6523,   87715,  8, }, // Europe/Stockholm
    {  336, 8867,   87723,  5, }, // Europe/Tallinn
    {  336, 6202,   87728,  5, }, // Europe/Tirane
    {  336, 8882,   87733, 10, }, // Europe/Ulyanovsk
    {  336, 6352,   87743,  5, }, // Europe/Vaduz
    {  336, 6567,   87748,  7, }, // Europe/Vatican
    {  336, 6231,   87755,  4, }, // Europe/Vienna
    {  336, 8899,   87759,  5, }, // Europe/Vilnius
    {  336, 4727,   87764,  8, }, // Europe/Volgograd
    {  336, 3190,   87772,  4, }, // Europe/Warsaw
    {  336, 6261,   87776,  5, }, // Europe/Zagreb
    {  336, 6540,   87781,  5, }, // Europe/Zurich
    {  336, 5078,   87786,  9, }, // Indian/Antananarivo
    {  336, 6947,   87795,  6, }, // Indian/Chagos
    {  336, 6046,   87801, 11, }, // Indian/Christmas
    {  336, 6063,   87812,  5, }, // Indian/Cocos
    {  336, 5029,   87817,  6, }, // Indian/Comoro
    {  336, 6661,   87823,  7, }, // Indian/Kerguelen
    {  336, 7260,   87830,  3, }, // Indian/Mahe
    {  336, 7050,   87833, 15, }, // Indian/Maldives
    {  336, 4069,   87848,  9, }, // Indian/Mauritius
    {  336, 5098,   87857,  5, }, // Indian/Mayotte
    {  336, 7211,   87862,  7, }, // Indian/Reunion
    {  336, 4422,   87869,  4, }, // Pacific/Apia
    {  336,  932,   87873,  6, }, // Pacific/Auckland
    {  336, 3706,   87879,  8, }, // Pacific/Bougainville
    {  336, 3015,   87887,  7, }, // Pacific/Chatham
    {  336, 3157,   87894,  4, }, // Pacific/Chuuk
    {  336, 2237,   87898,  8, }, // Pacific/Easter
    {  336, 7360,   87906,  4, }, // Pacific/Efate
    {  336, 7327,   87910,  6, }, // Pacific/Fakaofo
    {  336, 3902,   87916,  5, }, // Pacific/Fiji
    {  336, 7343,   87921,  7, }, // Pacific/Funafuti
    {  336, 6678,   87928,  7, }, // Pacific/Galapagos
    {  336, 6696,   87935, 10, }, // Pacific/Gambier
    {  336, 3822,   87945,  9, }, // Pacific/Guadalcanal
    {  336, 6018,   87954,  4, }, // Pacific/Guam
    {  336, 2723,   87958,  8, }, // Pacific/Honolulu
    {  336, 3049,   87966,  8, }, // Pacific/Kanton
    {  336, 3998,   87974, 10, }, // Pacific/Kiritimati
    {  336, 7019,   87984,  5, }, // Pacific/Kosrae
    {  336, 2842,   87989, 10, }, // Pacific/Kwajalein
    {  336, 4877,   87999,  7, }, // Pacific/Majuro
    {  336, 4051,   88006,  9, }, // Pacific/Marquesas
    {  336, 8914,   88015,  7, }, // Pacific/Midway
    {  336, 7084,   88022,  5, }, // Pacific/Nauru
    {  336, 7132,   88027,  5, }, // Pacific/Niue
    {  336, 4168,   88032,  6, }, // Pacific/Norfolk
    {  336, 7098,   88038,  5, }, // Pacific/Noumea
    {  336, 3126,   88043,  9, }, // Pacific/Pago_Pago
    {  336, 7155,   88052,  4, }, // Pacific/Palau
    {  336, 7194,   88056,  6, }, // Pacific/Pitcairn
    {  336, 3096,   88062,  6, }, // Pacific/Pohnpei
    {  336, 4807,   88068, 11, }, // Pacific/Port_Moresby
    {  336, 6076,   88079,  9, }, // Pacific/Rarotonga
    {  336, 6031,   88088,  6, }, // Pacific/Saipan
    {  336, 7312,   88094,  5, }, // Pacific/Tahiti
    {  336, 6712,   88099,  7, }, // Pacific/Tarawa
    {  336, 4583,   88106,  8, }, // Pacific/Tongatapu
    {  336, 7392,   88114,  4, }, // Pacific/Wake
    {  336, 7405,   88118,  5, }, // Pacific/Wallis
    {  337, 6788,   29832,  7, }, // Africa/Abidjan Hindi/Devanagari/India
    {  337, 6760,   88123,  5, }, // Africa/Accra
    {  337, 5059,   88128, 10, }, // Africa/Addis_Ababa
    {  337, 7420,   88138,  9, }, // Africa/Algiers
    {  337,   14,   88147,  7, }, // Africa/Asmara
    {  337,   44,   88154,  6, }, // Africa/Bamako
    {  337, 5257,   88160,  6, }, // Africa/Bangui
    {  337, 6746,   88166,  6, }, // Africa/Banjul
    {  337, 7435,   88172,  5, }, // Africa/Bissau
    {  337, 4957,   88177,  9, }, // Africa/Blantyre
    {  337, 5287,   88186, 11, }, // Africa/Brazzaville
    {  337, 4922,   88197,  9, }, // Africa/Bujumbura
    {  337, 2320,   88206,  5, }, // Africa/Cairo
    {  337, 4117,   29933, 11, }, // Africa/Casablanca
    {  337, 7449,   88211,  5, }, // Africa/Ceuta
    {  337, 6773,   88216,  8, }, // Africa/Conakry
    {  337, 6840,   88224,  4, }, // Africa/Dakar
    {  337, 5130,   88228, 11, }, // Africa/Dar_es_Salaam
    {  337, 5043,   88239,  6, }, // Africa/Djibouti
    {  337, 5243,   88245,  5, }, // Africa/Douala
    {  337, 5166,   88250,  8, }, // Africa/El_Aaiun
    {  337, 6853,   88258,  9, }, // Africa/Freetown
    {  337, 4906,   88267,  7, }, // Africa/Gaborone
    {  337, 5015,   88274,  5, }, // Africa/Harare
    {  337, 4479,   88279, 10, }, // Africa/Johannesburg
    {  337, 4499,   30022,  4, }, // Africa/Juba
    {  337, 5151,   88289,  6, }, // Africa/Kampala
    {  337, 4524,   30034,  7, }, // Africa/Khartoum
    {  337, 4973,   88295,  6, }, // Africa/Kigali
    {  337, 5306,   88301,  7, }, // Africa/Kinshasa
    {  337, 4744,   88308,  5, }, // Africa/Lagos
    {  337, 5336,   88313, 10, }, // Africa/Libreville
    {  337, 6869,   88323,  3, }, // Africa/Lome
    {  337, 5211,   88326,  6, }, // Africa/Luanda
    {  337, 4939,   88332,  9, }, // Africa/Lubumbashi
    {  337, 5001,   30089,  6, }, // Africa/Lusaka
    {  337, 5322,   88341,  5, }, // Africa/Malabo
    {  337, 4987,   88346,  6, }, // Africa/Maputo
    {  337, 5197,   30107,  6, }, // Africa/Maseru
    {  337, 5182,   88352,  7, }, // Africa/Mbabane
    {  337, 5113,   88359,  8, }, // Africa/Mogadishu
    {  337, 7034,   88367,  9, }, // Africa/Monrovia
    {  337, 3853,   88376,  6, }, // Africa/Nairobi
    {  337, 5271,   88382, 10, }, // Africa/Ndjamena
    {  337, 5354,   88392,  6, }, // Africa/Niamey
    {  337, 6803,   88398,  7, }, // Africa/Nouakchott
    {  337, 6727,   88405,  7, }, // Africa/Ouagadougou
    {  337, 5225,   88412, 11, }, // Africa/Porto-Novo
    {  337, 4435,   88423,  7, }, // Africa/Sao_Tome
    {  337, 2866,   88430,  8, }, // Africa/Tripoli
    {  337, 6554,   30196,  7, }, // Africa/Tunis
    {  337, 4152,   88438,  7, }, // Africa/Windhoek
    {  337,  132,   88445,  3, }, // America/Adak
    {  337, 3346,   30215,  6, }, // America/Anchorage
    {  337, 5607,   88448,  8, }, // America/Anguilla
    {  337, 5624,   88456,  7, }, // America/Antigua
    {  337, 4554,   88463,  9, }, // America/Araguaina
    {  337,  166,   88472, 11, }, // America/Argentina/Buenos_Aires
    {  337,   91,   30258, 10, }, // America/Argentina/Catamarca
    {  337,  270,   88483,  8, }, // America/Argentina/Cordoba
    {  337,  441,   88491,  5, }, // America/Argentina/Jujuy
    {  337, 7462,   88496,  8, }, // America/Argentina/La_Rioja
    {  337,  565,   88504,  8, }, // America/Argentina/Mendoza
    {  337, 7489,   88512, 12, }, // America/Argentina/Rio_Gallegos
    {  337, 7520,   30305,  6, }, // America/Argentina/Salta
    {  337, 7544,   88524,  9, }, // America/Argentina/San_Juan
    {  337, 5580,   88533,  7, }, // America/Argentina/San_Luis
    {  337, 7571,   88540,  6, }, // America/Argentina/Tucuman
    {  337, 7597,   88546,  7, }, // America/Argentina/Ushuaia
    {  337, 5640,   88553,  5, }, // America/Aruba
    {  337, 4252,   88558,  8, }, // America/Asuncion
    {  337,  237,   88566,  8, }, // America/Atikokan
    {  337, 3679,   88574,  5, }, // America/Bahia
    {  337, 7623,   88579, 15, }, // America/Bahia_Banderas
    {  337, 5654,   88594,  8, }, // America/Barbados
    {  337, 7646,   30394,  5, }, // America/Belem
    {  337, 5407,   88602,  6, }, // America/Belize
    {  337, 7660,   88608, 13, }, // America/Blanc-Sablon
    {  337, 7681,   88621, 10, }, // America/Boa_Vista
    {  337, 4361,   88631,  6, }, // America/Bogota
    {  337, 7699,   88637,  5, }, // America/Boise
    {  337, 7713,   88642, 15, }, // America/Cambridge_Bay
    {  337, 7735,   88657, 13, }, // America/Campo_Grande
    {  337, 3868,   88670,  6, }, // America/Cancun
    {  337, 4694,   88676,  6, }, // America/Caracas
    {  337, 4345,   30477,  5, }, // America/Cayenne
    {  337, 5496,   30482,  5, }, // America/Cayman
    {  337, 2260,   88682,  6, }, // America/Chicago
    {  337, 7756,   88688,  8, }, // America/Chihuahua
    {  337, 7774,   88696, 16, }, // America/Ciudad_Juarez
    {  337, 5422,   88712, 11, }, // America/Costa_Rica
    {  337, 7796,   30510,  8, }, // America/Creston
    {  337, 3791,   88723,  7, }, // America/Cuiaba
    {  337, 5723,   88730,  7, }, // America/Curacao
    {  337, 7812,   88737, 11, }, // America/Danmarkshavn
    {  337, 7833,   88748,  4, }, // America/Dawson
    {  337, 7848,   88752, 10, }, // America/Dawson_Creek
    {  337,  805,   88762,  5, }, // America/Denver
    {  337, 3465,   88767,  8, }, // America/Detroit
    {  337, 5739,   88775,  8, }, // America/Dominica
    {  337,  893,   88783,  6, }, // America/Edmonton
    {  337, 7869,   30591,  7, }, // America/Eirunepe
    {  337, 5441,   88789, 11, }, // America/El_Salvador
    {  337, 7886,   88800, 13, }, // America/Fort_Nelson
    {  337, 7906,   88813, 12, }, // America/Fortaleza
    {  337, 7924,   88825, 11, }, // America/Glace_Bay
    {  337, 6881,   88836,  9, }, // America/Goose_Bay
    {  337, 4612,   88845, 11, }, // America/Grand_Turk
    {  337, 5770,   30657,  8, }, // America/Grenada
    {  337, 5786,   88856,  9, }, // America/Guadeloupe
    {  337, 3760,   30674, 10, }, // America/Guatemala
    {  337, 4841,   88865,  9, }, // America/Guayaquil
    {  337, 6932,   88874,  5, }, // America/Guyana
    {  337, 1939,   88879, 10, }, // America/Halifax
    {  337, 2281,   88889,  5, }, // America/Havana
    {  337, 7942,   88894, 11, }, // America/Hermosillo
    {  337,  348,   88905, 13, }, // America/Indiana/Indianapolis
    {  337,  481,   88918, 15, }, // America/Indiana/Knox
    {  337, 7961,   88933, 17, }, // America/Indiana/Marengo
    {  337, 7985,   88950, 20, }, // America/Indiana/Petersburg
    {  337, 8012,   88970, 18, }, // America/Indiana/Tell_City
    {  337, 8038,   88988, 14, }, // America/Indiana/Vevay
    {  337, 8060,   89002, 18, }, // America/Indiana/Vincennes
    {  337, 8086,   89020, 17, }, // America/Indiana/Winamac
    {  337, 8110,   30867,  6, }, // America/Inuvik
    {  337,  660,   89037,  7, }, // America/Iqaluit
    {  337, 2799,   89044,  5, }, // America/Jamaica
    {  337, 5380,   89049,  7, }, // America/Juneau
    {  337,  521,   89056,  8, }, // America/Kentucky/Louisville
    {  337, 8125,   89064, 17, }, // America/Kentucky/Monticello
    {  337, 5704,   89081, 12, }, // America/Kralendijk
    {  337, 4376,   89093,  7, }, // America/La_Paz
    {  337, 7169,   30932,  4, }, // America/Lima
    {  337, 3239,   89100, 11, }, // America/Los_Angeles
    {  337, 5932,   89111, 22, }, // America/Lower_Princes
    {  337, 8153,   89133,  5, }, // America/Maceio
    {  337, 8168,   89138,  7, }, // America/Managua
    {  337, 1908,   89145,  4, }, // America/Manaus
    {  337, 5897,   89149,  7, }, // America/Marigot
    {  337, 5805,   89156,  9, }, // America/Martinique
    {  337, 8184,   89165,  9, }, // America/Matamoros
    {  337, 2917,   89174,  9, }, // America/Mazatlan
    {  337, 8202,   89183,  8, }, // America/Menominee
    {  337, 8220,   31026,  6, }, // America/Merida
    {  337, 8235,   89191, 12, }, // America/Metlakatla
    {  337, 2949,   89203, 13, }, // America/Mexico_City
    {  337, 4391,   89216,  7, }, // America/Miquelon
    {  337, 8254,   89223,  6, }, // America/Moncton
    {  337, 8270,   89229,  9, }, // America/Monterrey
    {  337, 4098,   89238, 11, }, // America/Montevideo
    {  337, 5824,   89249,  9, }, // America/Montserrat
    {  337, 5481,   89258,  5, }, // America/Nassau
    {  337, 2379,   89263,  9, }, // America/New_York
    {  337, 8288,   89185,  3, }, // America/Nome
    {  337, 1850,   89272,  8, }, // America/Noronha
    {  337, 8301,   89280, 20, }, // America/North_Dakota/Beulah
    {  337, 8329,   89300, 19, }, // America/North_Dakota/Center
    {  337, 8357,   89319, 24, }, // America/North_Dakota/New_Salem
    {  337,  393,   89343,  3, }, // America/Nuuk
    {  337, 8388,   89346,  9, }, // America/Ojinaga
    {  337, 2356,   89355,  5, }, // America/Panama
    {  337, 6173,   89360, 10, }, // America/Paramaribo
    {  337, 2973,   89370,  8, }, // America/Phoenix
    {  337, 3945,   89378, 14, }, // America/Port-au-Prince
    {  337, 5954,   89392, 15, }, // America/Port_of_Spain
    {  337, 8404,   89407, 13, }, // America/Porto_Velho
    {  337, 5843,   89420, 11, }, // America/Puerto_Rico
    {  337, 4030,   89431, 12, }, // America/Punta_Arenas
    {  337, 8424,   89443, 12, }, // America/Rankin_Inlet
    {  337, 8445,   89455,  7, }, // America/Recife
    {  337, 1995,   31299,  6, }, // America/Regina
    {  337, 8460,   89462,  9, }, // America/Resolute
    {  337,  695,   89471, 12, }, // America/Rio_Branco
    {  337, 8477,   89483,  8, }, // America/Santarem
    {  337, 2201,   89491,  9, }, // America/Santiago
    {  337, 6111,   89500, 13, }, // America/Santo_Domingo
    {  337, 1878,   89513,  9, }, // America/Sao_Paulo
    {  337, 6899,   89522, 13, }, // America/Scoresbysund
    {  337, 8494,   89535,  6, }, // America/Sitka
    {  337, 8508,   89541, 15, }, // America/St_Barthelemy
    {  337, 2061,   89556,  9, }, // America/St_Johns
    {  337, 5863,   89565, 10, }, // America/St_Kitts
    {  337, 5880,   89575, 11, }, // America/St_Lucia
    {  337,  855,   89586,  9, }, // America/St_Thomas
    {  337, 5913,   89595, 12, }, // America/St_Vincent
    {  337, 8530,   89607, 13, }, // America/Swift_Current
    {  337, 5461,   89620, 11, }, // America/Tegucigalpa
    {  337, 5756,   89631,  6, }, // America/Thule
    {  337,  313,   89637,  7, }, // America/Tijuana
    {  337,  608,   89644,  6, }, // America/Toronto
    {  337, 5688,   89650,  8, }, // America/Tortola
    {  337, 2093,   89658,  7, }, // America/Vancouver
    {  337, 2144,   89665, 11, }, // America/Whitehorse
    {  337,  734,   89676,  7, }, // America/Winnipeg
    {  337, 8552,   89683,  7, }, // America/Yakutat
    {  337, 6001,   31541,  4, }, // Antarctica/Casey
    {  337, 6094,   89690,  5, }, // Antarctica/Davis
    {  337, 6133,   89695, 19, }, // Antarctica/DumontDUrville
    {  337, 8568,   89714,  7, }, // Antarctica/Macquarie
    {  337, 7066,   89721,  4, }, // Antarctica/Mawson
    {  337, 7113,   89725,  9, }, // Antarctica/McMurdo
    {  337, 8589,   89734,  4, }, // Antarctica/Palmer
    {  337, 7226,   89738,  6, }, // Antarctica/Rothera
    {  337, 7295,   89744,  6, }, // Antarctica/Syowa
    {  337, 8607,   89750,  5, }, // Antarctica/Troll
    {  337, 7374,   89755,  7, }, // Antarctica/Vostok
    {  337, 1429,   89762, 13, }, // Arctic/Longyearbyen
    {  337, 5570,   89775,  4, }, // Asia/Aden
    {  337, 5395,   89779,  7, }, // Asia/Almaty
    {  337, 3968,   89786,  6, }, // Asia/Amman
    {  337, 5511,   89792,  6, }, // Asia/Anadyr
    {  337, 5523,   31646,  6, }, // Asia/Aqtau
    {  337, 5368,   89798,  6, }, // Asia/Aqtobe
    {  337,  964,   31659,  8, }, // Asia/Ashgabat
    {  337, 8624,   89804,  5, }, // Asia/Atyrau
    {  337, 3607,   89809,  5, }, // Asia/Baghdad
    {  337, 5534,   89814,  5, }, // Asia/Bahrain
    {  337, 3653,   88728,  4, }, // Asia/Baku
    {  337, 4466,   89819,  6, }, // Asia/Bangkok
    {  337, 3571,   89825,  6, }, // Asia/Barnaul
    {  337, 4086,   89831,  5, }, // Asia/Beirut
    {  337, 3778,   31704,  7, }, // Asia/Bishkek
    {  337, 5989,   89836,  7, }, // Asia/Brunei
    {  337, 4601,   89843,  6, }, // Asia/Chita
    {  337, 4511,   89849,  6, }, // Asia/Colombo
    {  337, 4540,   89855,  7, }, // Asia/Damascus
    {  337, 1093,   89862,  4, }, // Asia/Dhaka
    {  337, 6192,   89866,  4, }, // Asia/Dili
    {  337, 3596,   89870,  4, }, // Asia/Dubai
    {  337, 6159,   89874,  7, }, // Asia/Dushanbe
    {  337, 8636,   89881, 11, }, // Asia/Famagusta
    {  337, 8651,   89892,  5, }, // Asia/Gaza
    {  337, 4795,   89897,  7, }, // Asia/Hebron
    {  337, 1259,   89904, 16, }, // Asia/Ho_Chi_Minh
    {  337, 2704,   89920,  9, }, // Asia/Hong_Kong
    {  337, 4771,   89929,  5, }, // Asia/Hovd
    {  337, 4184,   89934, 10, }, // Asia/Irkutsk
    {  337, 7006,   89944,  7, }, // Asia/Jakarta
    {  337, 6992,   31811,  7, }, // Asia/Jayapura
    {  337, 1290,   89951,  6, }, // Asia/Jerusalem
    {  337, 3560,   89957,  5, }, // Asia/Kabul
    {  337, 4316,   89962,  7, }, // Asia/Kamchatka
    {  337, 4239,   31840,  5, }, // Asia/Karachi
    {  337, 1185,   89969,  8, }, // Asia/Kathmandu
    {  337, 8661,   89977,  7, }, // Asia/Khandyga
    {  337,  992,   89984,  7, }, // Asia/Kolkata
    {  337, 4197,   89991, 15, }, // Asia/Krasnoyarsk
    {  337, 4859,   90006, 10, }, // Asia/Kuala_Lumpur
    {  337, 5976,   31896,  6, }, // Asia/Kuching
    {  337, 5547,   31902,  5, }, // Asia/Kuwait
    {  337, 1211,   90016,  4, }, // Asia/Macau
    {  337, 4017,   31912,  7, }, // Asia/Magadan
    {  337, 1349,   90020,  6, }, // Asia/Makassar
    {  337, 7182,   31928,  5, }, // Asia/Manila
    {  337, 6920,   90026,  5, }, // Asia/Muscat
    {  337, 2554,   90031,  8, }, // Asia/Nicosia
    {  337, 8675,   90039, 16, }, // Asia/Novokuznetsk
    {  337, 4135,   90055, 13, }, // Asia/Novosibirsk
    {  337, 4229,   90068,  6, }, // Asia/Omsk
    {  337, 7145,   90074,  3, }, // Asia/Oral
    {  337, 6961,   90077,  9, }, // Asia/Phnom_Penh
    {  337, 8693,   90086,  9, }, // Asia/Pontianak
    {  337, 4214,   90095, 10, }, // Asia/Pyongyang
    {  337, 5559,   90105,  3, }, // Asia/Qatar
    {  337, 8708,   90108,  8, }, // Asia/Qostanay
    {  337, 4269,   90116, 11, }, // Asia/Qyzylorda
    {  337, 3584,   90127,  5, }, // Asia/Riyadh
    {  337, 4408,   90132,  6, }, // Asia/Sakhalin
    {  337, 7245,   90138,  6, }, // Asia/Samarkand
    {  337, 3279,   90144,  4, }, // Asia/Seoul
    {  337, 1053,   90148,  5, }, // Asia/Shanghai
    {  337, 3300,   32058,  8, }, // Asia/Singapore
    {  337, 4297,   90153, 16, }, // Asia/Srednekolymsk
    {  337, 3263,   90169,  6, }, // Asia/Taipei
    {  337, 4781,   90175,  6, }, // Asia/Tashkent
    {  337, 3915,   90181,  7, }, // Asia/Tbilisi
    {  337, 2772,   32104,  6, }, // Asia/Tehran
    {  337, 1317,   90188,  5, }, // Asia/Thimphu
    {  337, 2821,   90193,  6, }, // Asia/Tokyo
    {  337, 4572,   90199,  7, }, // Asia/Tomsk
    {  337, 1021,   90206,  8, }, // Asia/Ulaanbaatar
    {  337, 1159,   90214,  7, }, // Asia/Urumqi
    {  337, 8722,   90221, 10, }, // Asia/Ust-Nera
    {  337, 6977,   90231,  9, }, // Asia/Vientiane
    {  337, 4710,   90240, 13, }, // Asia/Vladivostok
    {  337, 4828,   32173,  9, }, // Asia/Yakutsk
    {  337, 1235,   32182,  5, }, // Asia/Yangon
    {  337, 3883,   90253, 13, }, // Asia/Yekaterinburg
    {  337, 3747,   90266,  7, }, // Asia/Yerevan
    {  337, 3663,   90273,  7, }, // Atlantic/Azores
    {  337, 5671,   90280,  6, }, // Atlantic/Bermuda
    {  337, 6628,   90286,  6, }, // Atlantic/Canary
    {  337, 3727,   90292,  8, }, // Atlantic/Cape_Verde
    {  337, 1395,   90300,  5, }, // Atlantic/Faroe
    {  337, 8736,   90305,  5, }, // Atlantic/Madeira
    {  337, 2748,   90310,  9, }, // Atlantic/Reykjavik
    {  337, 7272,   90319, 15, }, // Atlantic/South_Georgia
    {  337, 6821,   90334, 11, }, // Atlantic/St_Helena
    {  337, 6644,   90345,  7, }, // Atlantic/Stanley
    {  337, 1670,   90352,  6, }, // Australia/Adelaide
    {  337, 1635,   90358,  8, }, // Australia/Brisbane
    {  337, 1799,   90366, 10, }, // Australia/Broken_Hill
    {  337, 1583,   90376,  7, }, // Australia/Darwin
    {  337, 3637,   32320,  6, }, // Australia/Eucla
    {  337, 1516,   90383,  7, }, // Australia/Hobart
    {  337, 8753,   90390,  8, }, // Australia/Lindeman
    {  337, 1547,   90398, 10, }, // Australia/Lord_Howe
    {  337, 1727,   90408,  8, }, // Australia/Melbourne
    {  337, 1762,   90416,  4, }, // Australia/Perth
    {  337, 1463,   32362,  5, }, // Australia/Sydney
    {  337, 6441,   90420, 10, }, // Europe/Amsterdam
    {  337, 6216,   90430,  6, }, // Europe/Andorra
    {  337, 3620,   90436, 10, }, // Europe/Astrakhan
    {  337, 2300,   90446,  5, }, // Europe/Athens
    {  337, 6336,   32400,  8, }, // Europe/Belgrade
    {  337, 4757,   90451,  6, }, // Europe/Berlin
    {  337, 6488,   90457, 12, }, // Europe/Bratislava
    {  337, 2167,   90469,  9, }, // Europe/Brussels
    {  337, 3928,   90478, 10, }, // Europe/Bucharest
    {  337, 3806,   90488,  9, }, // Europe/Budapest
    {  337, 8772,   90497, 10, }, // Europe/Busingen
    {  337, 2583,   90507,  7, }, // Europe/Chisinau
    {  337, 6289,   90514,  9, }, // Europe/Copenhagen
    {  337, 2338,   90523,  5, }, // Europe/Dublin
    {  337, 6307,   90528, 10, }, // Europe/Gibraltar
    {  337, 8788,   90538,  6, }, // Europe/Guernsey
    {  337, 6612,   90544,  8, }, // Europe/Helsinki
    {  337, 8804,   90552, 11, }, // Europe/Isle_of_Man
    {  337, 1130,   90563,  9, }, // Europe/Istanbul
    {  337, 8823,   90572,  5, }, // Europe/Jersey
    {  337, 3979,   90577, 12, }, // Europe/Kaliningrad
    {  337, 8929,   90589,  5, }, // Europe/Kirov
    {  337, 2527,   90594,  3, }, // Europe/Kyiv
    {  337, 3213,   32558,  6, }, // Europe/Lisbon
    {  337, 6506,   90597, 11, }, // Europe/Ljubljana
    {  337, 2501,   90608,  4, }, // Europe/London
    {  337, 6365,   90612, 10, }, // Europe/Luxembourg
    {  337, 4892,   90622,  7, }, // Europe/Madrid
    {  337, 6397,   32599,  6, }, // Europe/Malta
    {  337, 6582,   90629,  9, }, // Europe/Mariehamn
    {  337, 3693,   90638,  6, }, // Europe/Minsk
    {  337, 6410,   90644,  6, }, // Europe/Monaco
    {  337, 3537,   90650,  6, }, // Europe/Moscow
    {  337, 6458,   90656,  5, }, // Europe/Oslo
    {  337, 4284,   32634,  5, }, // Europe/Paris
    {  337, 6424,   90661, 10, }, // Europe/Podgorica
    {  337, 6275,   90671,  5, }, // Europe/Prague
    {  337, 8837,   90676,  4, }, // Europe/Riga
    {  337, 6324,   90303,  3, }, // Europe/Rome
    {  337, 4331,   90680,  5, }, // Europe/Samara
    {  337, 6470,   90685, 10, }, // Europe/San_Marino
    {  337, 6245,   90695,  8, }, // Europe/Sarajevo
    {  337, 4451,   90703,  7, }, // Europe/Saratov
    {  337, 8849,   90710, 12, }, // Europe/Simferopol
    {  337, 6383,   90722,  8, }, // Europe/Skopje
    {  337, 6599,   90730,  7, }, // Europe/Sofia
    {  337, 6523,   90737,  8, }, // Europe/Stockholm
    {  337, 8867,   90745,  5, }, // Europe/Tallinn
    {  337, 6202,   90750,  6, }, // Europe/Tirane
    {  337, 8882,   90756, 12, }, // Europe/Ulyanovsk
    {  337, 6352,   90768,  6, }, // Europe/Vaduz
    {  337, 6567,   90774,  6, }, // Europe/Vatican
    {  337, 6231,   90780,  5, }, // Europe/Vienna
    {  337, 8899,   90785,  8, }, // Europe/Vilnius
    {  337, 4727,   90793, 11, }, // Europe/Volgograd
    {  337, 3190,   90804,  5, }, // Europe/Warsaw
    {  337, 6261,   90809,  8, }, // Europe/Zagreb
    {  337, 6540,   90817,  9, }, // Europe/Zurich
    {  337, 5078,   90826, 11, }, // Indian/Antananarivo
    {  337, 6947,   90837,  5, }, // Indian/Chagos
    {  337, 6046,   90842,  7, }, // Indian/Christmas
    {  337, 6063,   90849,  5, }, // Indian/Cocos
    {  337, 5029,   90854,  6, }, // Indian/Comoro
    {  337, 6661,   90860,  7, }, // Indian/Kerguelen
    {  337, 7260,   32851,  4, }, // Indian/Mahe
    {  337, 7050,   90867,  6, }, // Indian/Maldives
    {  337, 4069,   90873,  6, }, // Indian/Mauritius
    {  337, 5098,   90879,  8, }, // Indian/Mayotte
    {  337, 7211,   90887,  8, }, // Indian/Reunion
    {  337, 4422,   90895,  5, }, // Pacific/Apia
    {  337,  932,   90900,  6, }, // Pacific/Auckland
    {  337, 3706,   90906,  8, }, // Pacific/Bougainville
    {  337, 3015,   90914,  4, }, // Pacific/Chatham
    {  337, 3157,   90918,  2, }, // Pacific/Chuuk
    {  337, 2237,   90920,  5, }, // Pacific/Easter
    {  337, 7360,   90925,  5, }, // Pacific/Efate
    {  337, 7327,   90930,  9, }, // Pacific/Fakaofo
    {  337, 3902,   90939,  5, }, // Pacific/Fiji
    {  337, 7343,   90944, 12, }, // Pacific/Funafuti
    {  337, 6678,   90956,  9, }, // Pacific/Galapagos
    {  337, 6696,   90965,  7, }, // Pacific/Gambier
    {  337, 3822,   90972, 10, }, // Pacific/Guadalcanal
    {  337, 6018,   30980,  4, }, // Pacific/Guam
    {  337, 2723,   90982,  8, }, // Pacific/Honolulu
    {  337, 3049,   90990,  7, }, // Pacific/Kanton
    {  337, 3998,   33000, 10, }, // Pacific/Kiritimati
    {  337, 7019,   90997,  6, }, // Pacific/Kosrae
    {  337, 2842,   91003, 10, }, // Pacific/Kwajalein
    {  337, 4877,   91013,  6, }, // Pacific/Majuro
    {  337, 4051,   91019,  8, }, // Pacific/Marquesas
    {  337, 8914,   91027,  5, }, // Pacific/Midway
    {  337, 7084,   91032,  4, }, // Pacific/Nauru
    {  337, 7132,   91036,  4, }, // Pacific/Niue
    {  337, 4168,   91040,  7, }, // Pacific/Norfolk
    {  337, 7098,   91047,  6, }, // Pacific/Noumea
    {  337, 3126,   91053,  9, }, // Pacific/Pago_Pago
    {  337, 7155,   91062,  4, }, // Pacific/Palau
    {  337, 7194,   33078,  8, }, // Pacific/Pitcairn
    {  337, 3096,   91066,  6, }, // Pacific/Pohnpei
    {  337, 4807,   91072, 14, }, // Pacific/Port_Moresby
    {  337, 6076,   91086,  9, }, // Pacific/Rarotonga
    {  337, 6031,   91095,  6, }, // Pacific/Saipan
    {  337, 7312,   91101,  6, }, // Pacific/Tahiti
    {  337, 6712,   91107,  6, }, // Pacific/Tarawa
    {  337, 4583,   91113,  9, }, // Pacific/Tongatapu
    {  337, 7392,   33136,  3, }, // Pacific/Wake
    {  337, 7405,   33139,  5, }, // Pacific/Wallis
    {  338,   14,   91122,  6, }, // Africa/Asmara Hindi/Latin/India
    {  338, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  338, 4435,   91128,  8, }, // Africa/Sao_Tome
    {  338, 4252,   61321,  8, }, // America/Asuncion
    {  338,  237,   91136, 13, }, // America/Atikokan
    {  338, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  338, 7660,   44017, 12, }, // America/Blanc-Sablon
    {  338, 3868,   33182,  6, }, // America/Cancun
    {  338, 7774,   91149, 13, }, // America/Ciudad_Juarez
    {  338, 5723,   91162,  7, }, // America/Curacao
    {  338,  481,      73, 13, }, // America/Indiana/Knox
    {  338, 7961,      86, 16, }, // America/Indiana/Marengo
    {  338, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  338, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  338, 8038,     139, 14, }, // America/Indiana/Vevay
    {  338, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  338, 8086,     171, 16, }, // America/Indiana/Winamac
    {  338, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  338, 5932,     207, 22, }, // America/Lower_Princes
    {  338, 8220,    7008,  6, }, // America/Merida
    {  338, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  338, 8329,     255, 20, }, // America/North_Dakota/Center
    {  338, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  338,  393,     298,  4, }, // America/Nuuk
    {  338, 6899,     302, 16, }, // America/Scoresbysund
    {  338, 8508,   91169, 13, }, // America/St_Barthelemy
    {  338, 2061,   60820,  9, }, // America/St_Johns
    {  338, 5863,   60829,  8, }, // America/St_Kitts
    {  338, 5880,   60837,  8, }, // America/St_Lucia
    {  338,  855,   60845,  9, }, // America/St_Thomas
    {  338, 5913,   60854, 10, }, // America/St_Vincent
    {  338, 6133,   91182, 14, }, // Antarctica/DumontDUrville
    {  338, 5523,   91196,  5, }, // Asia/Aqtau
    {  338, 1259,   91201,  6, }, // Asia/Ho_Chi_Minh
    {  338, 4771,   26703,  5, }, // Asia/Hovd
    {  338, 1185,     413,  9, }, // Asia/Kathmandu
    {  338,  992,     422,  7, }, // Asia/Kolkata
    {  338, 1211,    4203,  5, }, // Asia/Macau
    {  338, 8708,   91207,  8, }, // Asia/Qostanay
    {  338, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  338, 1235,     442,  6, }, // Asia/Yangon
    {  338, 1395,   91215,  6, }, // Atlantic/Faroe
    {  338, 6821,   60864,  9, }, // Atlantic/St_Helena
    {  338, 2527,     463,  4, }, // Europe/Kyiv
    {  338, 7211,   91221,  7, }, // Indian/Reunion
    {  338, 3157,   91228,  4, }, // Pacific/Chuuk
    {  338, 2723,    4493,  8, }, // Pacific/Honolulu
    {  338, 3049,    4501,  9, }, // Pacific/Kanton
    {  338, 3096,    8311,  6, }, // Pacific/Pohnpei
    {  339, 5059,   91232, 12, }, // Africa/Addis_Ababa Hungarian/Latin/Hungary
    {  339, 7420,   91244,  5, }, // Africa/Algiers
    {  339,   14,   91249,  7, }, // Africa/Asmara
    {  339, 2320,   91256,  5, }, // Africa/Cairo
    {  339, 5130,   91261, 13, }, // Africa/Dar_es_Salaam
    {  339, 5043,   91274,  8, }, // Africa/Djibouti
    {  339, 5166,   91282,  7, }, // Africa/El_Aaiun
    {  339, 4524,   91289,  6, }, // Africa/Khartoum
    {  339, 5322,   91295,  6, }, // Africa/Malabo
    {  339, 4435,       6,  8, }, // Africa/Sao_Tome
    {  339, 6554,   91301,  6, }, // Africa/Tunis
    {  339, 4554,   26427,  9, }, // America/Araguaina
    {  339,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  339, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {  339, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  339, 4252,      14,  8, }, // America/Asuncion
    {  339,  237,      22,  8, }, // America/Atikokan
    {  339, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  339, 7646,   35764,  5, }, // America/Belem
    {  339, 4361,   20389,  6, }, // America/Bogota
    {  339, 3868,      47,  6, }, // America/Cancun
    {  339, 5496,   91307, 15, }, // America/Cayman
    {  339, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  339, 3791,   26442,  6, }, // America/Cuiaba
    {  339, 5723,      66,  7, }, // America/Curacao
    {  339, 5739,   21548,  8, }, // America/Dominica
    {  339, 7869,   26448,  8, }, // America/Eirunepe
    {  339, 5441,    6688,  8, }, // America/El_Salvador
    {  339, 2281,   63392,  7, }, // America/Havana
    {  339,  481,      73, 13, }, // America/Indiana/Knox
    {  339, 7961,      86, 16, }, // America/Indiana/Marengo
    {  339, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  339, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  339, 8038,     139, 14, }, // America/Indiana/Vevay
    {  339, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  339, 8086,     171, 16, }, // America/Indiana/Winamac
    {  339, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  339, 5932,     207, 22, }, // America/Lower_Princes
    {  339, 8153,   26456,  6, }, // America/Maceio
    {  339, 2917,   26471,  8, }, // America/Mazatlan
    {  339, 8220,     229,  6, }, // America/Merida
    {  339, 2949,   91322, 11, }, // America/Mexico_City
    {  339, 8301,   91333, 20, }, // America/North_Dakota/Beulah
    {  339, 8329,   91353, 20, }, // America/North_Dakota/Center
    {  339, 8357,   91373, 23, }, // America/North_Dakota/New_Salem
    {  339,  393,     298,  4, }, // America/Nuuk
    {  339,  695,   74255, 10, }, // America/Rio_Branco
    {  339, 1878,   26580,  9, }, // America/Sao_Paulo
    {  339, 6899,     302, 16, }, // America/Scoresbysund
    {  339, 8508,   26589, 16, }, // America/St_Barthelemy
    {  339, 2061,     332, 10, }, // America/St_Johns
    {  339, 5863,     342,  9, }, // America/St_Kitts
    {  339, 5880,     351,  9, }, // America/St_Lucia
    {  339,  855,     360, 10, }, // America/St_Thomas
    {  339, 5913,     370, 11, }, // America/St_Vincent
    {  339, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  339, 7374,   91396,  7, }, // Antarctica/Vostok
    {  339, 5570,   91403,  4, }, // Asia/Aden
    {  339, 5395,   58641,  8, }, // Asia/Almaty
    {  339, 3968,   57792,  5, }, // Asia/Amman
    {  339, 5511,    7482,  6, }, // Asia/Anadyr
    {  339, 5523,    5491,  5, }, // Asia/Aqtau
    {  339, 5368,   63474,  6, }, // Asia/Aqtobe
    {  339,  964,   26671,  7, }, // Asia/Ashgabat
    {  339, 8624,    7494,  6, }, // Asia/Atyrau
    {  339, 3607,    4044,  6, }, // Asia/Baghdad
    {  339, 5534,    4050,  7, }, // Asia/Bahrain
    {  339, 4086,   57818,  6, }, // Asia/Beirut
    {  339, 3778,   91407,  6, }, // Asia/Bishkek
    {  339, 4601,   91413,  5, }, // Asia/Chita
    {  339, 4540,   91418, 10, }, // Asia/Damascus
    {  339, 1093,   91428,  5, }, // Asia/Dhaka
    {  339, 3596,   57836,  5, }, // Asia/Dubai
    {  339, 6159,   91433,  7, }, // Asia/Dushanbe
    {  339, 8651,   91440,  4, }, // Asia/Gaza
    {  339, 1259,   91444, 16, }, // Asia/Ho_Chi_Minh
    {  339, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  339, 4184,   91460,  8, }, // Asia/Irkutsk
    {  339, 1290,   91468, 10, }, // Asia/Jerusalem
    {  339, 4316,   91478,  9, }, // Asia/Kamchatka
    {  339, 4239,   91487,  7, }, // Asia/Karachi
    {  339, 1185,    5593,  8, }, // Asia/Kathmandu
    {  339, 8661,   33392,  7, }, // Asia/Khandyga
    {  339,  992,   64221,  8, }, // Asia/Kolkata
    {  339, 4197,   91494, 13, }, // Asia/Krasnoyarsk
    {  339, 5976,   91507,  7, }, // Asia/Kuching
    {  339, 5547,   63852,  6, }, // Asia/Kuwait
    {  339, 1211,   91514,  5, }, // Asia/Macau
    {  339, 4017,   20732,  7, }, // Asia/Magadan
    {  339, 1349,    7618,  7, }, // Asia/Makassar
    {  339, 6920,   91519,  7, }, // Asia/Muscat
    {  339, 8675,   91526, 12, }, // Asia/Novokuznetsk
    {  339, 4135,   91538, 13, }, // Asia/Novosibirsk
    {  339, 4229,   91551,  5, }, // Asia/Omsk
    {  339, 4214,   91556,  7, }, // Asia/Pyongyang
    {  339, 5559,    4214,  5, }, // Asia/Qatar
    {  339, 8708,   91563,  9, }, // Asia/Qostanay
    {  339, 4269,    7660,  9, }, // Asia/Qyzylorda
    {  339, 3584,   57947,  5, }, // Asia/Riyadh
    {  339, 4408,   91572,  8, }, // Asia/Sakhalin
    {  339, 7245,   91580, 10, }, // Asia/Samarkand
    {  339, 3279,   91590,  5, }, // Asia/Seoul
    {  339, 1053,   91595,  7, }, // Asia/Shanghai
    {  339, 3300,   91602,  9, }, // Asia/Singapore
    {  339, 4297,   91611, 15, }, // Asia/Srednekolymsk
    {  339, 3263,    7712,  6, }, // Asia/Taipei
    {  339, 4781,   91626,  7, }, // Asia/Tashkent
    {  339, 3915,   91633,  8, }, // Asia/Tbilisi
    {  339, 2772,   57993,  7, }, // Asia/Tehran
    {  339, 1317,   91641,  5, }, // Asia/Thimphu
    {  339, 2821,   91646,  5, }, // Asia/Tokyo
    {  339, 4572,   91651,  6, }, // Asia/Tomsk
    {  339, 1021,   91657,  9, }, // Asia/Ulaanbaatar
    {  339, 1159,   91666,  7, }, // Asia/Urumqi
    {  339, 8722,   91673, 11, }, // Asia/Ust-Nera
    {  339, 6977,   20811,  8, }, // Asia/Vientiane
    {  339, 4710,   91684, 13, }, // Asia/Vladivostok
    {  339, 4828,   91697,  8, }, // Asia/Yakutsk
    {  339, 1235,     442,  6, }, // Asia/Yangon
    {  339, 3883,   91705, 14, }, // Asia/Yekaterinburg
    {  339, 3747,   91719,  7, }, // Asia/Yerevan
    {  339, 3663,   91726, 14, }, // Atlantic/Azores
    {  339, 6628,   91740, 15, }, // Atlantic/Canary
    {  339, 3727,   91755, 18, }, // Atlantic/Cape_Verde
    {  339, 1395,   91773,  6, }, // Atlantic/Faroe
    {  339, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  339, 7272,   91779, 12, }, // Atlantic/South_Georgia
    {  339, 6821,   91791, 11, }, // Atlantic/St_Helena
    {  339, 6441,   91802, 10, }, // Europe/Amsterdam
    {  339, 3620,   91812,  9, }, // Europe/Astrakhan
    {  339, 2300,   58123,  5, }, // Europe/Athens
    {  339, 6336,   91821,  7, }, // Europe/Belgrade
    {  339, 6488,   91828,  7, }, // Europe/Bratislava
    {  339, 2167,   91835,  8, }, // Europe/Brussels
    {  339, 3928,   26897,  8, }, // Europe/Bucharest
    {  339, 8772,   26905,  8, }, // Europe/Busingen
    {  339, 6289,   91843, 10, }, // Europe/Copenhagen
    {  339, 6307,   91853,  9, }, // Europe/Gibraltar
    {  339, 8804,   91862, 10, }, // Europe/Isle_of_Man
    {  339, 1130,   91872,  9, }, // Europe/Istanbul
    {  339, 3979,   91881, 12, }, // Europe/Kaliningrad
    {  339, 2527,   33591,  5, }, // Europe/Kyiv
    {  339, 3213,   91893,  9, }, // Europe/Lisbon
    {  339, 6365,    4395,  9, }, // Europe/Luxembourg
    {  339, 6397,   91902,  5, }, // Europe/Malta
    {  339, 3693,   91907,  6, }, // Europe/Minsk
    {  339, 3537,   91913,  7, }, // Europe/Moscow
    {  339, 4284,   91920,  6, }, // Europe/Paris
    {  339, 6275,   91926,  5, }, // Europe/Prague
    {  339, 6324,   91931,  4, }, // Europe/Rome
    {  339, 4331,   91935,  7, }, // Europe/Samara
    {  339, 6245,   91942,  9, }, // Europe/Sarajevo
    {  339, 4451,   91951,  8, }, // Europe/Saratov
    {  339, 8849,   91959, 11, }, // Europe/Simferopol
    {  339, 6383,   91970,  7, }, // Europe/Skopje
    {  339, 6599,   91977,  6, }, // Europe/Sofia
    {  339, 8867,   22793,  6, }, // Europe/Tallinn
    {  339, 6202,   21095,  6, }, // Europe/Tirane
    {  339, 8882,   91983, 10, }, // Europe/Ulyanovsk
    {  339, 6567,   58222,  7, }, // Europe/Vatican
    {  339, 6231,   91993,  4, }, // Europe/Vienna
    {  339, 4727,   91997,  9, }, // Europe/Volgograd
    {  339, 3190,   92006,  5, }, // Europe/Warsaw
    {  339, 6261,   92011,  6, }, // Europe/Zagreb
    {  339, 6540,    4459,  6, }, // Europe/Zurich
    {  339, 6046,   92017, 16, }, // Indian/Christmas
    {  339, 6063,   92033, 13, }, // Indian/Cocos
    {  339, 5029,   92046,  6, }, // Indian/Comoro
    {  339, 7050,   92052, 15, }, // Indian/Maldives
    {  339, 7211,     467,  7, }, // Indian/Reunion
    {  339, 3015,   92067, 16, }, // Pacific/Chatham
    {  339, 3157,   91228,  4, }, // Pacific/Chuuk
    {  339, 2237,   92083, 15, }, // Pacific/Easter
    {  339, 3902,   92098,  6, }, // Pacific/Fiji
    {  339, 6678,   92104, 18, }, // Pacific/Galapagos
    {  339, 6696,   92122, 16, }, // Pacific/Gambier
    {  339, 2723,    4493,  8, }, // Pacific/Honolulu
    {  339, 3049,    4501,  9, }, // Pacific/Kanton
    {  339, 3998,   92138, 17, }, // Pacific/Kiritimati
    {  339, 7019,   92155, 15, }, // Pacific/Kosrae
    {  339, 2842,   92170, 16, }, // Pacific/Kwajalein
    {  339, 4877,   92186, 13, }, // Pacific/Majuro
    {  339, 4051,   92199, 18, }, // Pacific/Marquesas
    {  339, 8914,   92217, 15, }, // Pacific/Midway
    {  339, 7194,   92232, 17, }, // Pacific/Pitcairn
    {  339, 3096,   92249, 15, }, // Pacific/Pohnpei
    {  339, 7392,   92264, 11, }, // Pacific/Wake
    {  340, 7420,   92275, 11, }, // Africa/Algiers Icelandic/Latin/Iceland
    {  340,   14,       0,  6, }, // Africa/Asmara
    {  340, 7435,   92286,  5, }, // Africa/Bissau
    {  340, 2320,   92291,  5, }, // Africa/Cairo
    {  340, 5043,   92296,  7, }, // Africa/Djibouti
    {  340, 4479,   92303, 14, }, // Africa/Johannesburg
    {  340, 4435,   92317,  8, }, // Africa/Sao_Tome
    {  340, 2866,   92325,  7, }, // Africa/Tripoli
    {  340, 6554,   92332,  9, }, // Africa/Tunis
    {  340, 5607,   92341,  8, }, // America/Anguilla
    {  340, 5624,   92349,  7, }, // America/Antigua
    {  340, 5640,   92356,  5, }, // America/Aruba
    {  340, 4252,      14,  8, }, // America/Asuncion
    {  340,  237,      22,  8, }, // America/Atikokan
    {  340, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  340, 5407,   92361,  5, }, // America/Belize
    {  340, 3868,   92366,  6, }, // America/Cancun
    {  340, 5496,   92372, 12, }, // America/Cayman
    {  340, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  340, 5422,   92384,  9, }, // America/Costa_Rica
    {  340, 5723,      66,  7, }, // America/Curacao
    {  340, 5739,   92393,  8, }, // America/Dominica
    {  340, 5786,   92401,  8, }, // America/Guadeloupe
    {  340, 3760,   33206,  9, }, // America/Guatemala
    {  340, 6932,   92409,  7, }, // America/Guyana
    {  340,  481,      73, 13, }, // America/Indiana/Knox
    {  340, 7961,      86, 16, }, // America/Indiana/Marengo
    {  340, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  340, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  340, 8038,     139, 14, }, // America/Indiana/Vevay
    {  340, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  340, 8086,     171, 16, }, // America/Indiana/Winamac
    {  340, 2799,   92416,  7, }, // America/Jamaica
    {  340, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  340, 5932,     207, 22, }, // America/Lower_Princes
    {  340, 5805,   92423,  8, }, // America/Martinique
    {  340, 8220,    7008,  6, }, // America/Merida
    {  340, 2949,   92431, 10, }, // America/Mexico_City
    {  340, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  340, 8329,     255, 20, }, // America/North_Dakota/Center
    {  340, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  340,  393,     298,  4, }, // America/Nuuk
    {  340, 5843,   92441, 11, }, // America/Puerto_Rico
    {  340, 6899,     302, 16, }, // America/Scoresbysund
    {  340, 8508,   92452, 25, }, // America/St_Barthelemy
    {  340, 2061,     332, 10, }, // America/St_Johns
    {  340, 5863,   92477, 12, }, // America/St_Kitts
    {  340, 5880,   92489, 12, }, // America/St_Lucia
    {  340,  855,   92501, 13, }, // America/St_Thomas
    {  340, 5913,   92514, 14, }, // America/St_Vincent
    {  340, 5688,   92528,  7, }, // America/Tortola
    {  340, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  340, 3607,    4044,  6, }, // Asia/Baghdad
    {  340, 5534,   63842,  6, }, // Asia/Bahrain
    {  340, 3653,   20629,  4, }, // Asia/Baku
    {  340, 4086,   92535,  6, }, // Asia/Beirut
    {  340, 5989,   92541,  6, }, // Asia/Brunei
    {  340, 4511,   92547,  7, }, // Asia/Colombo
    {  340, 4540,    4083,  8, }, // Asia/Damascus
    {  340, 1093,   91428,  5, }, // Asia/Dhaka
    {  340, 1259,   92554, 16, }, // Asia/Ho_Chi_Minh
    {  340, 7006,    4129,  8, }, // Asia/Jakarta
    {  340, 1290,   92570,  9, }, // Asia/Jerusalem
    {  340, 3560,   92579,  5, }, // Asia/Kabul
    {  340, 4316,    4143, 10, }, // Asia/Kamchatka
    {  340, 1185,   20712,  8, }, // Asia/Kathmandu
    {  340,  992,   92584,  8, }, // Asia/Kolkata
    {  340, 4859,   92592, 12, }, // Asia/Kuala_Lumpur
    {  340, 5547,   92604,  6, }, // Asia/Kuwait
    {  340, 1211,   91514,  5, }, // Asia/Macau
    {  340, 2554,   92610,  7, }, // Asia/Nicosia
    {  340, 4214,   33430,  9, }, // Asia/Pyongyang
    {  340, 5559,    4214,  5, }, // Asia/Qatar
    {  340, 8708,     434,  8, }, // Asia/Qostanay
    {  340, 3584,   92617,  5, }, // Asia/Riyadh
    {  340, 3279,   20778,  4, }, // Asia/Seoul
    {  340, 1053,   92622,  7, }, // Asia/Shanghai
    {  340, 3300,   92629,  8, }, // Asia/Singapore
    {  340, 2772,    7724,  7, }, // Asia/Tehran
    {  340, 2821,   92637,  5, }, // Asia/Tokyo
    {  340, 1021,   92642, 10, }, // Asia/Ulaanbaatar
    {  340, 1235,   20819,  6, }, // Asia/Yangon
    {  340, 3747,    7780,  7, }, // Asia/Yerevan
    {  340, 3663,   92652,  9, }, // Atlantic/Azores
    {  340, 5671,   92661,  7, }, // Atlantic/Bermuda
    {  340, 6628,   92668, 11, }, // Atlantic/Canary
    {  340, 3727,   92679, 14, }, // Atlantic/Cape_Verde
    {  340, 1395,   92693,  8, }, // Atlantic/Faroe
    {  340, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  340, 7272,   92701, 13, }, // Atlantic/South_Georgia
    {  340, 6821,   92714, 13, }, // Atlantic/St_Helena
    {  340, 2300,   92727,  5, }, // Europe/Athens
    {  340, 6336,   26883,  7, }, // Europe/Belgrade
    {  340, 4757,   20941,  6, }, // Europe/Berlin
    {  340, 2167,    4332,  7, }, // Europe/Brussels
    {  340, 3928,   92732,  8, }, // Europe/Bucharest
    {  340, 3806,   92740,  8, }, // Europe/Budapest
    {  340, 6289,   92748, 13, }, // Europe/Copenhagen
    {  340, 6307,   92761,  9, }, // Europe/Gibraltar
    {  340, 8804,   92770,  3, }, // Europe/Isle_of_Man
    {  340, 1130,   92773,  8, }, // Europe/Istanbul
    {  340, 2527,   92781, 10, }, // Europe/Kyiv
    {  340, 3213,    4381,  8, }, // Europe/Lisbon
    {  340, 2501,   92791,  8, }, // Europe/London
    {  340, 6365,   92799,  9, }, // Europe/Luxembourg
    {  340, 4892,   92808,  6, }, // Europe/Madrid
    {  340, 6582,   92814,  9, }, // Europe/Mariehamn
    {  340, 6410,   92823,  6, }, // Europe/Monaco
    {  340, 3537,   22748,  6, }, // Europe/Moscow
    {  340, 6458,   92829,  4, }, // Europe/Oslo
    {  340, 4284,   21062,  5, }, // Europe/Paris
    {  340, 6275,    8083,  4, }, // Europe/Prague
    {  340, 6324,   91931,  3, }, // Europe/Rome
    {  340, 6470,   92833, 10, }, // Europe/San_Marino
    {  340, 6245,   92843,  8, }, // Europe/Sarajevo
    {  340, 6599,   92851,  5, }, // Europe/Sofia
    {  340, 6523,   92856, 11, }, // Europe/Stockholm
    {  340, 6202,   92867,  6, }, // Europe/Tirane
    {  340, 6567,   92873,  9, }, // Europe/Vatican
    {  340, 6231,   92882,  3, }, // Europe/Vienna
    {  340, 8899,   92885,  7, }, // Europe/Vilnius
    {  340, 3190,   92892,  6, }, // Europe/Warsaw
    {  340, 6046,   92898,  6, }, // Indian/Christmas
    {  340, 6063,   92904, 10, }, // Indian/Cocos
    {  340, 7050,   92914, 11, }, // Indian/Maldives
    {  340, 4069,   92925,  8, }, // Indian/Mauritius
    {  340, 7211,     467,  7, }, // Indian/Reunion
    {  340, 3157,     474,  5, }, // Pacific/Chuuk
    {  340, 2237,   92933,  9, }, // Pacific/Easter
    {  340, 3902,   92942,  5, }, // Pacific/Fiji
    {  340, 6018,   63240,  4, }, // Pacific/Guam
    {  340, 2723,    4493,  8, }, // Pacific/Honolulu
    {  340, 3049,    4501,  9, }, // Pacific/Kanton
    {  340, 4051,   92947, 15, }, // Pacific/Marquesas
    {  340, 7084,   92962,  4, }, // Pacific/Nauru
    {  340, 7155,   92966,  4, }, // Pacific/Palau
    {  340, 3096,     479,  7, }, // Pacific/Pohnpei
    {  340, 7312,   92970,  6, }, // Pacific/Tahiti
    {  341,   14,       0,  6, }, // Africa/Asmara Ido/Latin/world
    {  341, 4435,       6,  8, }, // Africa/Sao_Tome
    {  341, 4252,      14,  8, }, // America/Asuncion
    {  341,  237,      22,  8, }, // America/Atikokan
    {  341, 7623,      30, 17, }, // America/Bahia_Banderas
    {  341, 3868,      47,  6, }, // America/Cancun
    {  341, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  341, 5723,      66,  7, }, // America/Curacao
    {  341,  481,      73, 13, }, // America/Indiana/Knox
    {  341, 7961,      86, 16, }, // America/Indiana/Marengo
    {  341, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  341, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  341, 8038,     139, 14, }, // America/Indiana/Vevay
    {  341, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  341, 8086,     171, 16, }, // America/Indiana/Winamac
    {  341, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  341, 5932,     207, 22, }, // America/Lower_Princes
    {  341, 8220,     229,  6, }, // America/Merida
    {  341, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  341, 8329,     255, 20, }, // America/North_Dakota/Center
    {  341, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  341,  393,     298,  4, }, // America/Nuuk
    {  341, 6899,     302, 16, }, // America/Scoresbysund
    {  341, 8508,     318, 14, }, // America/St_Barthelemy
    {  341, 2061,     332, 10, }, // America/St_Johns
    {  341, 5863,     342,  9, }, // America/St_Kitts
    {  341, 5880,     351,  9, }, // America/St_Lucia
    {  341,  855,     360, 10, }, // America/St_Thomas
    {  341, 5913,     370, 11, }, // America/St_Vincent
    {  341, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  341, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  341, 1185,     413,  9, }, // Asia/Kathmandu
    {  341,  992,     422,  7, }, // Asia/Kolkata
    {  341, 1211,     429,  5, }, // Asia/Macau
    {  341, 1235,     442,  6, }, // Asia/Yangon
    {  341, 1395,     448,  5, }, // Atlantic/Faroe
    {  341, 6821,     453, 10, }, // Atlantic/St_Helena
    {  341, 2527,     463,  4, }, // Europe/Kyiv
    {  341, 7211,     467,  7, }, // Indian/Reunion
    {  341, 3157,     474,  5, }, // Pacific/Chuuk
    {  341, 3096,     479,  7, }, // Pacific/Pohnpei
    {  342,   14,       0,  6, }, // Africa/Asmara Igbo/Latin/Nigeria
    {  342, 4435,       6,  8, }, // Africa/Sao_Tome
    {  342, 4252,      14,  8, }, // America/Asuncion
    {  342,  237,      22,  8, }, // America/Atikokan
    {  342, 7623,      30, 17, }, // America/Bahia_Banderas
    {  342, 3868,      47,  6, }, // America/Cancun
    {  342, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  342, 5723,      66,  7, }, // America/Curacao
    {  342,  481,      73, 13, }, // America/Indiana/Knox
    {  342, 7961,      86, 16, }, // America/Indiana/Marengo
    {  342, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  342, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  342, 8038,     139, 14, }, // America/Indiana/Vevay
    {  342, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  342, 8086,     171, 16, }, // America/Indiana/Winamac
    {  342, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  342, 5932,     207, 22, }, // America/Lower_Princes
    {  342, 8220,     229,  6, }, // America/Merida
    {  342, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  342, 8329,     255, 20, }, // America/North_Dakota/Center
    {  342, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  342,  393,     298,  4, }, // America/Nuuk
    {  342, 6899,     302, 16, }, // America/Scoresbysund
    {  342, 8508,     318, 14, }, // America/St_Barthelemy
    {  342, 2061,     332, 10, }, // America/St_Johns
    {  342, 5863,     342,  9, }, // America/St_Kitts
    {  342, 5880,     351,  9, }, // America/St_Lucia
    {  342,  855,     360, 10, }, // America/St_Thomas
    {  342, 5913,     370, 11, }, // America/St_Vincent
    {  342, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  342, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  342, 1185,     413,  9, }, // Asia/Kathmandu
    {  342,  992,     422,  7, }, // Asia/Kolkata
    {  342, 1211,     429,  5, }, // Asia/Macau
    {  342, 1235,     442,  6, }, // Asia/Yangon
    {  342, 1395,     448,  5, }, // Atlantic/Faroe
    {  342, 6821,     453, 10, }, // Atlantic/St_Helena
    {  342, 2527,     463,  4, }, // Europe/Kyiv
    {  342, 7211,     467,  7, }, // Indian/Reunion
    {  342, 3157,     474,  5, }, // Pacific/Chuuk
    {  342, 3049,    4501,  9, }, // Pacific/Kanton
    {  342, 3096,     479,  7, }, // Pacific/Pohnpei
    {  343,   14,       0,  6, }, // Africa/Asmara Inari Sami/Latin/Finland
    {  343, 4435,       6,  8, }, // Africa/Sao_Tome
    {  343, 4252,      14,  8, }, // America/Asuncion
    {  343,  237,      22,  8, }, // America/Atikokan
    {  343, 7623,      30, 17, }, // America/Bahia_Banderas
    {  343, 3868,      47,  6, }, // America/Cancun
    {  343, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  343, 5723,      66,  7, }, // America/Curacao
    {  343,  481,      73, 13, }, // America/Indiana/Knox
    {  343, 7961,      86, 16, }, // America/Indiana/Marengo
    {  343, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  343, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  343, 8038,     139, 14, }, // America/Indiana/Vevay
    {  343, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  343, 8086,     171, 16, }, // America/Indiana/Winamac
    {  343, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  343, 5932,     207, 22, }, // America/Lower_Princes
    {  343, 8220,     229,  6, }, // America/Merida
    {  343, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  343, 8329,     255, 20, }, // America/North_Dakota/Center
    {  343, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  343,  393,     298,  4, }, // America/Nuuk
    {  343, 6899,     302, 16, }, // America/Scoresbysund
    {  343, 8508,     318, 14, }, // America/St_Barthelemy
    {  343, 2061,     332, 10, }, // America/St_Johns
    {  343, 5863,     342,  9, }, // America/St_Kitts
    {  343, 5880,     351,  9, }, // America/St_Lucia
    {  343,  855,     360, 10, }, // America/St_Thomas
    {  343, 5913,     370, 11, }, // America/St_Vincent
    {  343, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  343, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  343, 1185,     413,  9, }, // Asia/Kathmandu
    {  343,  992,     422,  7, }, // Asia/Kolkata
    {  343, 1211,     429,  5, }, // Asia/Macau
    {  343, 1235,     442,  6, }, // Asia/Yangon
    {  343, 1395,     448,  5, }, // Atlantic/Faroe
    {  343, 6821,     453, 10, }, // Atlantic/St_Helena
    {  343, 2527,     463,  4, }, // Europe/Kyiv
    {  343, 7211,     467,  7, }, // Indian/Reunion
    {  343, 3157,     474,  5, }, // Pacific/Chuuk
    {  343, 3096,     479,  7, }, // Pacific/Pohnpei
    {  344, 7420,   92976,  5, }, // Africa/Algiers Indonesian/Latin/Indonesia
    {  344,   14,       0,  6, }, // Africa/Asmara
    {  344, 2320,    4572,  5, }, // Africa/Cairo
    {  344, 4435,       6,  8, }, // Africa/Sao_Tome
    {  344, 5607,    4657,  7, }, // America/Anguilla
    {  344, 4252,      14,  8, }, // America/Asuncion
    {  344,  237,      22,  8, }, // America/Atikokan
    {  344, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  344, 3868,   33182,  6, }, // America/Cancun
    {  344, 7774,   91149, 13, }, // America/Ciudad_Juarez
    {  344, 5422,   21503, 10, }, // America/Costa_Rica
    {  344, 5723,      66,  7, }, // America/Curacao
    {  344, 5739,   21548,  8, }, // America/Dominica
    {  344,  481,      73, 13, }, // America/Indiana/Knox
    {  344, 7961,      86, 16, }, // America/Indiana/Marengo
    {  344, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  344, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  344, 8038,     139, 14, }, // America/Indiana/Vevay
    {  344, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  344, 8086,     171, 16, }, // America/Indiana/Winamac
    {  344, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  344, 5932,     207, 22, }, // America/Lower_Princes
    {  344, 5805,    5142,  8, }, // America/Martinique
    {  344, 8220,    7008,  6, }, // America/Merida
    {  344, 8301,   92981, 20, }, // America/North_Dakota/Beulah
    {  344, 8329,   93001, 20, }, // America/North_Dakota/Center
    {  344, 8357,   93021, 23, }, // America/North_Dakota/New_Salem
    {  344,  393,     298,  4, }, // America/Nuuk
    {  344, 6899,     302, 16, }, // America/Scoresbysund
    {  344, 8508,     318, 14, }, // America/St_Barthelemy
    {  344, 2061,     332, 10, }, // America/St_Johns
    {  344, 5863,     342,  9, }, // America/St_Kitts
    {  344, 5880,     351,  9, }, // America/St_Lucia
    {  344,  855,     360, 10, }, // America/St_Thomas
    {  344, 5913,     370, 11, }, // America/St_Vincent
    {  344, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  344, 5523,    5491,  5, }, // Asia/Aqtau
    {  344, 5368,    7488,  6, }, // Asia/Aqtobe
    {  344, 4511,    5519,  7, }, // Asia/Colombo
    {  344, 4540,    4083,  8, }, // Asia/Damascus
    {  344, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  344, 1290,    5568,  9, }, // Asia/Jerusalem
    {  344, 1185,     413,  9, }, // Asia/Kathmandu
    {  344,  992,     422,  7, }, // Asia/Kolkata
    {  344, 1211,   33405,  5, }, // Asia/Macau
    {  344, 6920,    4208,  6, }, // Asia/Muscat
    {  344, 2554,    5620,  7, }, // Asia/Nicosia
    {  344, 8708,     434,  8, }, // Asia/Qostanay
    {  344, 3300,   93044,  9, }, // Asia/Singapore
    {  344, 2772,    7724,  7, }, // Asia/Tehran
    {  344, 1235,   58486,  7, }, // Asia/Yangon
    {  344, 3727,   93053, 13, }, // Atlantic/Cape_Verde
    {  344, 1395,     448,  5, }, // Atlantic/Faroe
    {  344, 7272,   93066, 15, }, // Atlantic/South_Georgia
    {  344, 6821,     453, 10, }, // Atlantic/St_Helena
    {  344, 2300,   93081,  6, }, // Europe/Athens
    {  344, 6336,    7942,  7, }, // Europe/Belgrade
    {  344, 2583,   93087,  8, }, // Europe/Chisinau
    {  344, 6289,    4357, 10, }, // Europe/Copenhagen
    {  344, 8804,   93095,  9, }, // Europe/Isle_of_Man
    {  344, 2527,    8030,  4, }, // Europe/Kyiv
    {  344, 6365,    8054, 10, }, // Europe/Luxembourg
    {  344, 6410,    5926,  6, }, // Europe/Monaco
    {  344, 3537,   93104,  6, }, // Europe/Moscow
    {  344, 6275,   36153,  5, }, // Europe/Prague
    {  344, 6324,    5946,  4, }, // Europe/Rome
    {  344, 6567,    5978,  7, }, // Europe/Vatican
    {  344, 6231,     171,  4, }, // Europe/Vienna
    {  344, 3190,   93110,  7, }, // Europe/Warsaw
    {  344, 5029,   22853,  6, }, // Indian/Comoro
    {  344, 7050,   93117,  8, }, // Indian/Maldives
    {  344, 7211,     467,  7, }, // Indian/Reunion
    {  344, 3157,     474,  5, }, // Pacific/Chuuk
    {  344, 3822,    8242, 11, }, // Pacific/Guadalcanal
    {  344, 2723,    4493,  8, }, // Pacific/Honolulu
    {  344, 3049,    4501,  9, }, // Pacific/Kanton
    {  344, 3096,     479,  7, }, // Pacific/Pohnpei
    {  345,   14,       0,  6, }, // Africa/Asmara Interlingua/Latin/world
    {  345, 5043,   26360,  7, }, // Africa/Djibouti
    {  345, 4435,       6,  8, }, // Africa/Sao_Tome
    {  345, 4252,      14,  8, }, // America/Asuncion
    {  345,  237,      22,  8, }, // America/Atikokan
    {  345, 7623,   64558, 17, }, // America/Bahia_Banderas
    {  345, 3868,   33182,  6, }, // America/Cancun
    {  345, 5496,   44041,  6, }, // America/Cayman
    {  345, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  345, 5723,      66,  7, }, // America/Curacao
    {  345,  481,      73, 13, }, // America/Indiana/Knox
    {  345, 7961,      86, 16, }, // America/Indiana/Marengo
    {  345, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  345, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  345, 8038,     139, 14, }, // America/Indiana/Vevay
    {  345, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  345, 8086,     171, 16, }, // America/Indiana/Winamac
    {  345, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  345, 5932,     207, 22, }, // America/Lower_Princes
    {  345, 5805,   20436,  9, }, // America/Martinique
    {  345, 8220,    7008,  6, }, // America/Merida
    {  345, 2949,   93125, 16, }, // America/Mexico_City
    {  345, 2379,   93141,  9, }, // America/New_York
    {  345, 8301,   44071, 23, }, // America/North_Dakota/Beulah
    {  345, 8329,   44094, 23, }, // America/North_Dakota/Center
    {  345, 8357,   44117, 26, }, // America/North_Dakota/New_Salem
    {  345,  393,     298,  4, }, // America/Nuuk
    {  345, 5843,   64866, 10, }, // America/Puerto_Rico
    {  345, 6899,     302, 16, }, // America/Scoresbysund
    {  345, 8508,   93150, 18, }, // America/St_Barthelemy
    {  345, 2061,   93168, 28, }, // America/St_Johns
    {  345, 5863,   93196, 19, }, // America/St_Kitts
    {  345, 5880,   93215, 12, }, // America/St_Lucia
    {  345,  855,   93227, 13, }, // America/St_Thomas
    {  345, 5913,   93240, 15, }, // America/St_Vincent
    {  345, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  345, 5534,    4050,  7, }, // Asia/Bahrain
    {  345, 4540,   74309,  7, }, // Asia/Damascus
    {  345, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  345, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  345, 1185,     413,  9, }, // Asia/Kathmandu
    {  345, 8661,   57888,  8, }, // Asia/Khandyga
    {  345,  992,   35921,  8, }, // Asia/Kolkata
    {  345, 1211,     429,  5, }, // Asia/Macau
    {  345, 4408,   57952,  8, }, // Asia/Sakhalin
    {  345, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  345, 1235,     442,  6, }, // Asia/Yangon
    {  345, 3883,    7768, 12, }, // Asia/Yekaterinburg
    {  345, 6628,   74368,  8, }, // Atlantic/Canary
    {  345, 3727,   93255, 10, }, // Atlantic/Cape_Verde
    {  345, 1395,   20874,  5, }, // Atlantic/Faroe
    {  345, 6821,     453, 10, }, // Atlantic/St_Helena
    {  345, 2300,   93265,  7, }, // Europe/Athens
    {  345, 6336,   58774,  8, }, // Europe/Belgrade
    {  345, 2167,   57529,  9, }, // Europe/Brussels
    {  345, 3928,   20955,  8, }, // Europe/Bucharest
    {  345, 8772,   26905,  8, }, // Europe/Busingen
    {  345, 8804,   93272, 13, }, // Europe/Isle_of_Man
    {  345, 2527,    8030,  4, }, // Europe/Kyiv
    {  345, 3213,   93285,  7, }, // Europe/Lisbon
    {  345, 6365,    4395,  9, }, // Europe/Luxembourg
    {  345, 3537,   74461,  7, }, // Europe/Moscow
    {  345, 6275,   21067,  5, }, // Europe/Prague
    {  345, 6324,    5946,  4, }, // Europe/Rome
    {  345, 6202,   21095,  6, }, // Europe/Tirane
    {  345, 6567,   74495,  8, }, // Europe/Vatican
    {  345, 3190,   21126,  8, }, // Europe/Warsaw
    {  345, 6046,   93292,  5, }, // Indian/Christmas
    {  345, 7050,   74517,  8, }, // Indian/Maldives
    {  345, 4069,   93297,  8, }, // Indian/Mauritius
    {  345, 5098,   93305,  7, }, // Indian/Mayotte
    {  345, 7211,     467,  7, }, // Indian/Reunion
    {  345, 3157,     474,  5, }, // Pacific/Chuuk
    {  345, 2237,   93312,  6, }, // Pacific/Easter
    {  345, 6678,   21159,  9, }, // Pacific/Galapagos
    {  345, 3096,     479,  7, }, // Pacific/Pohnpei
    {  346, 5059,   64509, 11, }, // Africa/Addis_Ababa Interlingue/Latin/Estonia
    {  346, 7420,   43956,  5, }, // Africa/Algiers
    {  346,   14,       0,  6, }, // Africa/Asmara
    {  346, 5130,   93318, 13, }, // Africa/Dar_es_Salaam
    {  346, 5043,   26360,  7, }, // Africa/Djibouti
    {  346, 5166,   93331,  8, }, // Africa/El_Aaiun
    {  346, 4524,   26372,  7, }, // Africa/Khartoum
    {  346, 6727,   26402,  8, }, // Africa/Ouagadougou
    {  346, 4435,   93339,  8, }, // Africa/Sao_Tome
    {  346,  166,   93347, 12, }, // America/Argentina/Buenos_Aires
    {  346, 7462,   93359,  8, }, // America/Argentina/La_Rioja
    {  346, 7489,   93367, 12, }, // America/Argentina/Rio_Gallegos
    {  346, 7544,   93379,  9, }, // America/Argentina/San_Juan
    {  346, 5580,   93388,  9, }, // America/Argentina/San_Luis
    {  346, 4252,      14,  8, }, // America/Asuncion
    {  346,  237,      22,  8, }, // America/Atikokan
    {  346, 7623,   93397, 15, }, // America/Bahia_Banderas
    {  346, 7681,    6514,  9, }, // America/Boa_Vista
    {  346, 7713,   93412, 13, }, // America/Cambridge_Bay
    {  346, 7735,   93425, 12, }, // America/Campo_Grande
    {  346, 3868,      47,  6, }, // America/Cancun
    {  346, 7774,   93437, 13, }, // America/Ciudad_Juarez
    {  346, 5422,   93450, 10, }, // America/Costa_Rica
    {  346, 5723,      66,  7, }, // America/Curacao
    {  346, 7848,   93460, 12, }, // America/Dawson_Creek
    {  346, 5441,   93472, 11, }, // America/El_Salvador
    {  346, 7886,    6696, 11, }, // America/Fort_Nelson
    {  346, 7924,   93483,  9, }, // America/Glace_Bay
    {  346, 6881,   93492,  9, }, // America/Goose_Bay
    {  346, 4612,   93501, 10, }, // America/Grand_Turk
    {  346, 5786,   93511,  9, }, // America/Guadeloupe
    {  346,  481,      73, 13, }, // America/Indiana/Knox
    {  346, 7961,      86, 16, }, // America/Indiana/Marengo
    {  346, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  346, 8012,   93520, 18, }, // America/Indiana/Tell_City
    {  346, 8038,     139, 14, }, // America/Indiana/Vevay
    {  346, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  346, 8086,     171, 16, }, // America/Indiana/Winamac
    {  346, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  346, 4376,   93538,  6, }, // America/La_Paz
    {  346, 3239,   93544, 11, }, // America/Los_Angeles
    {  346, 5932,   93555, 22, }, // America/Lower_Princes
    {  346, 5805,   20436,  9, }, // America/Martinique
    {  346, 8220,     229,  6, }, // America/Merida
    {  346, 2949,   93577, 12, }, // America/Mexico_City
    {  346, 8301,   93589, 19, }, // America/North_Dakota/Beulah
    {  346, 8329,   93608, 19, }, // America/North_Dakota/Center
    {  346, 8357,   93627, 22, }, // America/North_Dakota/New_Salem
    {  346,  393,     298,  4, }, // America/Nuuk
    {  346, 5954,   26559, 13, }, // America/Port_of_Spain
    {  346, 8404,    7183, 11, }, // America/Porto_Velho
    {  346, 5843,   93649, 10, }, // America/Puerto_Rico
    {  346, 4030,    7204, 12, }, // America/Punta_Arenas
    {  346, 8424,   93659, 12, }, // America/Rankin_Inlet
    {  346,  695,   93671, 10, }, // America/Rio_Branco
    {  346, 6111,   93681, 11, }, // America/Santo_Domingo
    {  346, 1878,   93692,  9, }, // America/Sao_Paulo
    {  346, 6899,     302, 16, }, // America/Scoresbysund
    {  346, 8508,   93701, 15, }, // America/St_Barthelemy
    {  346, 2061,   93716, 10, }, // America/St_Johns
    {  346, 5863,   93726,  9, }, // America/St_Kitts
    {  346, 5880,   93735,  9, }, // America/St_Lucia
    {  346,  855,   93744, 10, }, // America/St_Thomas
    {  346, 5913,   93754, 11, }, // America/St_Vincent
    {  346, 8530,   93765, 13, }, // America/Swift_Current
    {  346, 6133,   57010, 16, }, // Antarctica/DumontDUrville
    {  346, 7295,   62088,  5, }, // Antarctica/Syowa
    {  346, 5395,    7476,  6, }, // Asia/Almaty
    {  346, 5523,    5491,  5, }, // Asia/Aqtau
    {  346, 5368,    7488,  6, }, // Asia/Aqtobe
    {  346, 8624,    7494,  6, }, // Asia/Atyrau
    {  346, 4540,   74309,  7, }, // Asia/Damascus
    {  346, 1259,   78028, 11, }, // Asia/Ho_Chi_Minh
    {  346, 2704,    7535,  9, }, // Asia/Hong_Kong
    {  346, 1185,     413,  9, }, // Asia/Kathmandu
    {  346,  992,     422,  7, }, // Asia/Kolkata
    {  346, 4859,    7589, 12, }, // Asia/Kuala_Lumpur
    {  346, 1211,     429,  5, }, // Asia/Macau
    {  346, 6961,   93778, 10, }, // Asia/Phnom_Penh
    {  346, 8708,     434,  8, }, // Asia/Qostanay
    {  346, 4269,    7660,  9, }, // Asia/Qyzylorda
    {  346, 3300,   20782,  8, }, // Asia/Singapore
    {  346, 2772,    7724,  7, }, // Asia/Tehran
    {  346, 2821,    4238,  5, }, // Asia/Tokyo
    {  346, 1235,     442,  6, }, // Asia/Yangon
    {  346, 6628,   74368,  7, }, // Atlantic/Canary
    {  346, 3727,   93788,  9, }, // Atlantic/Cape_Verde
    {  346, 1395,   20874,  5, }, // Atlantic/Faroe
    {  346, 7272,   93797, 11, }, // Atlantic/South_Georgia
    {  346, 6821,   93808, 11, }, // Atlantic/St_Helena
    {  346, 1799,   93819, 11, }, // Australia/Broken_Hill
    {  346, 1547,   93830, 15, }, // Australia/Lord_Howe
    {  346, 1762,   22603,  4, }, // Australia/Perth
    {  346, 2300,    5793,  4, }, // Europe/Athens
    {  346, 6336,   26883,  7, }, // Europe/Belgrade
    {  346, 2167,   57529,  9, }, // Europe/Brussels
    {  346, 3928,   93845,  9, }, // Europe/Bucharest
    {  346, 6289,   58525,  9, }, // Europe/Copenhagen
    {  346, 8804,   93854, 12, }, // Europe/Isle_of_Man
    {  346, 2527,    8030,  4, }, // Europe/Kyiv
    {  346, 3213,   21019,  6, }, // Europe/Lisbon
    {  346, 6365,    4395,  9, }, // Europe/Luxembourg
    {  346, 6410,   74455,  6, }, // Europe/Monaco
    {  346, 3537,   93866,  6, }, // Europe/Moscow
    {  346, 6275,   36153,  5, }, // Europe/Prague
    {  346, 6324,    5946,  4, }, // Europe/Rome
    {  346, 6470,    8096, 10, }, // Europe/San_Marino
    {  346, 6202,   21095,  6, }, // Europe/Tirane
    {  346, 3190,   58543,  8, }, // Europe/Warsaw
    {  346, 6540,    4459,  6, }, // Europe/Zurich
    {  346, 4069,   74525,  8, }, // Indian/Mauritius
    {  346, 7211,     467,  7, }, // Indian/Reunion
    {  346, 3157,     474,  5, }, // Pacific/Chuuk
    {  346, 2237,   93872,  5, }, // Pacific/Easter
    {  346, 3902,    4488,  5, }, // Pacific/Fiji
    {  346, 3126,    8295,  9, }, // Pacific/Pago_Pago
    {  346, 3096,     479,  7, }, // Pacific/Pohnpei
    {  346, 4807,   93877, 12, }, // Pacific/Port_Moresby
    {  347,   14,       0,  6, }, // Africa/Asmara Inuktitut/Canadian Aboriginal/Canada
    {  347, 4435,       6,  8, }, // Africa/Sao_Tome
    {  347, 4252,      14,  8, }, // America/Asuncion
    {  347,  237,      22,  8, }, // America/Atikokan
    {  347, 7623,      30, 17, }, // America/Bahia_Banderas
    {  347, 3868,      47,  6, }, // America/Cancun
    {  347, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  347, 5723,      66,  7, }, // America/Curacao
    {  347,  481,      73, 13, }, // America/Indiana/Knox
    {  347, 7961,      86, 16, }, // America/Indiana/Marengo
    {  347, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  347, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  347, 8038,     139, 14, }, // America/Indiana/Vevay
    {  347, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  347, 8086,     171, 16, }, // America/Indiana/Winamac
    {  347, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  347, 5932,     207, 22, }, // America/Lower_Princes
    {  347, 8220,     229,  6, }, // America/Merida
    {  347, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  347, 8329,     255, 20, }, // America/North_Dakota/Center
    {  347, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  347,  393,     298,  4, }, // America/Nuuk
    {  347, 6899,     302, 16, }, // America/Scoresbysund
    {  347, 8508,     318, 14, }, // America/St_Barthelemy
    {  347, 2061,     332, 10, }, // America/St_Johns
    {  347, 5863,     342,  9, }, // America/St_Kitts
    {  347, 5880,     351,  9, }, // America/St_Lucia
    {  347,  855,     360, 10, }, // America/St_Thomas
    {  347, 5913,     370, 11, }, // America/St_Vincent
    {  347, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  347, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  347, 1185,     413,  9, }, // Asia/Kathmandu
    {  347,  992,     422,  7, }, // Asia/Kolkata
    {  347, 1211,     429,  5, }, // Asia/Macau
    {  347, 1235,     442,  6, }, // Asia/Yangon
    {  347, 1395,     448,  5, }, // Atlantic/Faroe
    {  347, 6821,     453, 10, }, // Atlantic/St_Helena
    {  347, 2527,     463,  4, }, // Europe/Kyiv
    {  347, 7211,     467,  7, }, // Indian/Reunion
    {  347, 3157,     474,  5, }, // Pacific/Chuuk
    {  347, 3096,     479,  7, }, // Pacific/Pohnpei
    {  349, 7420,   93889, 20, }, // Africa/Algiers Irish/Latin/Ireland
    {  349,   14,       0,  6, }, // Africa/Asmara
    {  349, 2320,   93909,  6, }, // Africa/Cairo
    {  349, 6773,   93915,  8, }, // Africa/Conakry
    {  349, 6840,   93923,  5, }, // Africa/Dakar
    {  349, 5130,   93928,  9, }, // Africa/Dar_es_Salaam
    {  349, 5166,   93937,  6, }, // Africa/El_Aaiun
    {  349, 4524,   93943,  6, }, // Africa/Khartoum
    {  349, 6869,   20300,  4, }, // Africa/Lome
    {  349, 4987,   93949,  6, }, // Africa/Maputo
    {  349, 5113,   93955, 10, }, // Africa/Mogadishu
    {  349, 6803,   93965,  7, }, // Africa/Nouakchott
    {  349, 4435,       6,  8, }, // Africa/Sao_Tome
    {  349, 2866,   93972,  7, }, // Africa/Tripoli
    {  349, 6554,   93979,  6, }, // Africa/Tunis
    {  349, 5607,   93985,  7, }, // America/Anguilla
    {  349,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  349, 5640,   92356,  5, }, // America/Aruba
    {  349, 4252,      14,  8, }, // America/Asuncion
    {  349,  237,      22,  8, }, // America/Atikokan
    {  349, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  349, 5654,   93992,  8, }, // America/Barbados
    {  349, 7646,   35764,  5, }, // America/Belem
    {  349, 5407,   94000, 10, }, // America/Belize
    {  349, 4361,   94010,  6, }, // America/Bogota
    {  349, 3868,      47,  6, }, // America/Cancun
    {  349, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  349, 5422,   94016, 10, }, // America/Costa_Rica
    {  349, 5723,   94026,  6, }, // America/Curacao
    {  349, 5739,   72693,  9, }, // America/Dominica
    {  349, 5441,   94032, 13, }, // America/El_Salvador
    {  349, 5770,   94045,  8, }, // America/Grenada
    {  349, 5786,   94053,  9, }, // America/Guadeloupe
    {  349, 3760,   72744,  9, }, // America/Guatemala
    {  349, 6932,   94062,  9, }, // America/Guyana
    {  349,  481,      73, 13, }, // America/Indiana/Knox
    {  349, 7961,      86, 16, }, // America/Indiana/Marengo
    {  349, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  349, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  349, 8038,     139, 14, }, // America/Indiana/Vevay
    {  349, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  349, 8086,     171, 16, }, // America/Indiana/Winamac
    {  349, 2799,   94071,  7, }, // America/Jamaica
    {  349, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  349, 7169,   94078,  5, }, // America/Lima
    {  349, 5932,     207, 22, }, // America/Lower_Princes
    {  349, 8220,    7008,  6, }, // America/Merida
    {  349, 2949,   94083, 18, }, // America/Mexico_City
    {  349, 5824,   72803,  9, }, // America/Montserrat
    {  349, 2379,   94101, 11, }, // America/New_York
    {  349, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  349, 8329,     255, 20, }, // America/North_Dakota/Center
    {  349, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  349,  393,     298,  4, }, // America/Nuuk
    {  349, 5843,   94112, 10, }, // America/Puerto_Rico
    {  349, 1878,   26580,  9, }, // America/Sao_Paulo
    {  349, 6899,     302, 16, }, // America/Scoresbysund
    {  349, 8508,   35804, 16, }, // America/St_Barthelemy
    {  349, 2061,     332, 10, }, // America/St_Johns
    {  349, 5863,   94122, 13, }, // America/St_Kitts
    {  349, 5880,   35832, 11, }, // America/St_Lucia
    {  349,  855,     360, 10, }, // America/St_Thomas
    {  349, 5913,   94135, 12, }, // America/St_Vincent
    {  349, 5756,   94147, 10, }, // America/Thule
    {  349, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  349, 8568,   94157, 10, }, // Antarctica/Macquarie
    {  349, 5570,   94167,  5, }, // Asia/Aden
    {  349, 3607,    4044,  6, }, // Asia/Baghdad
    {  349, 5534,   94172,  7, }, // Asia/Bahrain
    {  349, 3653,   94179,  4, }, // Asia/Baku
    {  349, 4086,   94183,  7, }, // Asia/Beirut
    {  349, 5989,   94190,  6, }, // Asia/Brunei
    {  349, 4540,   94196, 10, }, // Asia/Damascus
    {  349, 4795,   94206,  7, }, // Asia/Hebron
    {  349, 1259,   72962, 19, }, // Asia/Ho_Chi_Minh
    {  349, 2704,   94213,  9, }, // Asia/Hong_Kong
    {  349, 7006,   94222,  7, }, // Asia/Jakarta
    {  349, 1290,   94229, 11, }, // Asia/Jerusalem
    {  349, 3560,   94240,  5, }, // Asia/Kabul
    {  349, 1185,     413,  9, }, // Asia/Kathmandu
    {  349,  992,   94245,  7, }, // Asia/Kolkata
    {  349, 5547,   94252,  5, }, // Asia/Kuwait
    {  349, 1211,     429,  5, }, // Asia/Macau
    {  349, 1349,   94257,  7, }, // Asia/Makassar
    {  349, 7182,   94264,  7, }, // Asia/Manila
    {  349, 2554,   94271, 10, }, // Asia/Nicosia
    {  349, 5559,   73021,  5, }, // Asia/Qatar
    {  349, 8708,     434,  8, }, // Asia/Qostanay
    {  349, 3279,   94281,  3, }, // Asia/Seoul
    {  349, 1053,   94284,  9, }, // Asia/Shanghai
    {  349, 3300,   94293,  9, }, // Asia/Singapore
    {  349, 2821,   94302,  6, }, // Asia/Tokyo
    {  349, 4828,   94308,  7, }, // Asia/Yakutsk
    {  349, 1235,   20819,  6, }, // Asia/Yangon
    {  349, 3747,   94315,  8, }, // Asia/Yerevan
    {  349, 3663,   94323,  9, }, // Atlantic/Azores
    {  349, 5671,   94332,  9, }, // Atlantic/Bermuda
    {  349, 6628,   94341, 22, }, // Atlantic/Canary
    {  349, 3727,   94363, 10, }, // Atlantic/Cape_Verde
    {  349, 1395,   94373, 13, }, // Atlantic/Faroe
    {  349, 8736,   94386,  8, }, // Atlantic/Madeira
    {  349, 2748,   94394,  8, }, // Atlantic/Reykjavik
    {  349, 7272,   94402, 18, }, // Atlantic/South_Georgia
    {  349, 6821,   94420, 10, }, // Atlantic/St_Helena
    {  349, 6441,   94430,  9, }, // Europe/Amsterdam
    {  349, 6216,   94439,  6, }, // Europe/Andorra
    {  349, 3620,   94445, 12, }, // Europe/Astrakhan
    {  349, 2300,   94457,  9, }, // Europe/Athens
    {  349, 6336,   94466,  8, }, // Europe/Belgrade
    {  349, 4757,   94474,  7, }, // Europe/Berlin
    {  349, 6488,   94481, 14, }, // Europe/Bratislava
    {  349, 2167,   94495, 12, }, // Europe/Brussels
    {  349, 3928,   94507,  9, }, // Europe/Bucharest
    {  349, 3806,   94516, 10, }, // Europe/Budapest
    {  349, 2583,   94526,  7, }, // Europe/Chisinau
    {  349, 6289,   94533, 10, }, // Europe/Copenhagen
    {  349, 2338,   94543, 17, }, // Europe/Dublin
    {  349, 6307,   94560, 10, }, // Europe/Gibraltar
    {  349, 8788,   94570,  7, }, // Europe/Guernsey
    {  349, 6612,   94577,  9, }, // Europe/Helsinki
    {  349, 8804,   94586, 14, }, // Europe/Isle_of_Man
    {  349, 1130,   94600,  9, }, // Europe/Istanbul
    {  349, 8823,   94609,  6, }, // Europe/Jersey
    {  349, 2527,   94615,  3, }, // Europe/Kyiv
    {  349, 3213,   94618,  8, }, // Europe/Lisbon
    {  349, 6506,   94626, 10, }, // Europe/Ljubljana
    {  349, 2501,   94636,  7, }, // Europe/London
    {  349, 6365,   94643, 10, }, // Europe/Luxembourg
    {  349, 4892,   94653,  7, }, // Europe/Madrid
    {  349, 6397,   91902,  5, }, // Europe/Malta
    {  349, 3693,   94660,  6, }, // Europe/Minsk
    {  349, 6410,   94666,  6, }, // Europe/Monaco
    {  349, 3537,   94672,  5, }, // Europe/Moscow
    {  349, 6458,   92829,  4, }, // Europe/Oslo
    {  349, 4284,   94677,  5, }, // Europe/Paris
    {  349, 6424,   94682, 11, }, // Europe/Podgorica
    {  349, 6275,   91926,  4, }, // Europe/Prague
    {  349, 8837,   94693,  4, }, // Europe/Riga
    {  349, 6324,   94697,  8, }, // Europe/Rome
    {  349, 6470,   94705, 11, }, // Europe/San_Marino
    {  349, 6245,   94716,  8, }, // Europe/Sarajevo
    {  349, 6383,   94724,  6, }, // Europe/Skopje
    {  349, 6599,   94730,  6, }, // Europe/Sofia
    {  349, 6523,   94736,  7, }, // Europe/Stockholm
    {  349, 8867,   94743,  8, }, // Europe/Tallinn
    {  349, 6202,   94751,  7, }, // Europe/Tirane
    {  349, 6352,   94758,  5, }, // Europe/Vaduz
    {  349, 6567,   94763, 11, }, // Europe/Vatican
    {  349, 6231,   92882,  3, }, // Europe/Vienna
    {  349, 8899,   94774,  7, }, // Europe/Vilnius
    {  349, 3190,   94781,  5, }, // Europe/Warsaw
    {  349, 6261,   94786,  6, }, // Europe/Zagreb
    {  349, 6540,    4459,  6, }, // Europe/Zurich
    {  349, 5078,   94792, 14, }, // Indian/Antananarivo
    {  349, 6046,   94806, 16, }, // Indian/Christmas
    {  349, 6063,   94822, 12, }, // Indian/Cocos
    {  349, 5029,   94834, 15, }, // Indian/Comoro
    {  349, 7050,   94849, 17, }, // Indian/Maldives
    {  349, 4069,   94866, 14, }, // Indian/Mauritius
    {  349, 7211,   65405, 10, }, // Indian/Reunion
    {  349, 3157,     474,  5, }, // Pacific/Chuuk
    {  349, 3902,   94880,  5, }, // Pacific/Fiji
    {  349, 2723,    4493,  8, }, // Pacific/Honolulu
    {  349, 3049,    4501,  9, }, // Pacific/Kanton
    {  349, 4051,   94885, 23, }, // Pacific/Marquesas
    {  349, 8914,   94908, 14, }, // Pacific/Midway
    {  349, 7084,   92962,  4, }, // Pacific/Nauru
    {  349, 3096,     479,  7, }, // Pacific/Pohnpei
    {  349, 7312,   94922,  7, }, // Pacific/Tahiti
    {  349, 7405,   94929,  6, }, // Pacific/Wallis
    {  351, 5059,    3784, 11, }, // Africa/Addis_Ababa Italian/Latin/Italy
    {  351, 7420,   94935,  6, }, // Africa/Algiers
    {  351,   14,       0,  6, }, // Africa/Asmara
    {  351, 2320,   94941,  8, }, // Africa/Cairo
    {  351, 5043,   94949,  6, }, // Africa/Djibouti
    {  351, 5166,   94955,  7, }, // Africa/El_Aaiun
    {  351, 4499,   94962,  5, }, // Africa/Juba
    {  351, 4524,   26372,  7, }, // Africa/Khartoum
    {  351, 6869,   20300,  4, }, // Africa/Lome
    {  351, 5113,   43978, 10, }, // Africa/Mogadishu
    {  351, 5271,   43996,  9, }, // Africa/Ndjamena
    {  351, 4435,       6,  8, }, // Africa/Sao_Tome
    {  351, 6554,   94967,  6, }, // Africa/Tunis
    {  351, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  351, 4252,      14,  8, }, // America/Asuncion
    {  351,  237,      22,  8, }, // America/Atikokan
    {  351, 7623,      30, 17, }, // America/Bahia_Banderas
    {  351, 4361,   20389,  6, }, // America/Bogota
    {  351, 3868,      47,  6, }, // America/Cancun
    {  351, 4345,   94973,  7, }, // America/Cayenne
    {  351, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  351, 5723,      66,  7, }, // America/Curacao
    {  351, 5786,   94980,  9, }, // America/Guadeloupe
    {  351, 2281,   94989,  7, }, // America/Havana
    {  351,  481,      73, 13, }, // America/Indiana/Knox
    {  351, 7961,      86, 16, }, // America/Indiana/Marengo
    {  351, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  351, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  351, 8038,     139, 14, }, // America/Indiana/Vevay
    {  351, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  351, 8086,     171, 16, }, // America/Indiana/Winamac
    {  351, 2799,   94996,  8, }, // America/Jamaica
    {  351, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  351, 5932,     207, 22, }, // America/Lower_Princes
    {  351, 5805,   20436,  9, }, // America/Martinique
    {  351, 8220,     229,  6, }, // America/Merida
    {  351, 2949,   95004, 17, }, // America/Mexico_City
    {  351, 8301,   95021, 23, }, // America/North_Dakota/Beulah
    {  351, 8329,   95044, 23, }, // America/North_Dakota/Center
    {  351, 8357,   95067, 26, }, // America/North_Dakota/New_Salem
    {  351,  393,     298,  4, }, // America/Nuuk
    {  351, 5843,   95093,  9, }, // America/Puerto_Rico
    {  351, 8477,   26572,  8, }, // America/Santarem
    {  351, 1878,   95102,  9, }, // America/Sao_Paulo
    {  351, 6899,     302, 16, }, // America/Scoresbysund
    {  351, 8508,   26589, 16, }, // America/St_Barthelemy
    {  351, 2061,     332, 10, }, // America/St_Johns
    {  351, 5863,     342,  9, }, // America/St_Kitts
    {  351, 5880,   95111, 11, }, // America/St_Lucia
    {  351,  855,   44161, 12, }, // America/St_Thomas
    {  351, 5913,   26651, 13, }, // America/St_Vincent
    {  351, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  351, 5511,   95122,  7, }, // Asia/Anadyr
    {  351, 5368,   57073,  6, }, // Asia/Aqtobe
    {  351, 5534,    4050,  7, }, // Asia/Bahrain
    {  351, 4601,   57490,  4, }, // Asia/Chita
    {  351, 4540,   74309,  7, }, // Asia/Damascus
    {  351, 1093,   65588,  5, }, // Asia/Dhaka
    {  351, 8636,   95129,  9, }, // Asia/Famagusta
    {  351, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  351, 7006,   95138,  8, }, // Asia/Jakarta
    {  351, 1290,   95146, 11, }, // Asia/Jerusalem
    {  351, 1185,     413,  9, }, // Asia/Kathmandu
    {  351, 8661,   57888,  8, }, // Asia/Khandyga
    {  351,  992,   35921,  8, }, // Asia/Kolkata
    {  351, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  351, 1211,     429,  5, }, // Asia/Macau
    {  351, 6920,   20739,  7, }, // Asia/Muscat
    {  351, 8675,   62352, 11, }, // Asia/Novokuznetsk
    {  351, 3584,   20756,  5, }, // Asia/Riyadh
    {  351, 4408,   57952,  8, }, // Asia/Sakhalin
    {  351, 7245,   20768, 10, }, // Asia/Samarkand
    {  351, 3279,    7680,  4, }, // Asia/Seoul
    {  351, 2772,    7724,  7, }, // Asia/Tehran
    {  351, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  351, 8722,   44300,  9, }, // Asia/Ust-Nera
    {  351, 4828,    7755,  7, }, // Asia/Yakutsk
    {  351, 1235,   58486,  7, }, // Asia/Yangon
    {  351, 3883,    7768, 12, }, // Asia/Yekaterinburg
    {  351, 3663,   95157,  7, }, // Atlantic/Azores
    {  351, 6628,   20849,  7, }, // Atlantic/Canary
    {  351, 3727,   93255, 10, }, // Atlantic/Cape_Verde
    {  351, 1395,   95164, 13, }, // Atlantic/Faroe
    {  351, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  351, 7272,   95177, 15, }, // Atlantic/South_Georgia
    {  351, 6821,   95192, 10, }, // Atlantic/St_Helena
    {  351, 2300,    5793,  5, }, // Europe/Athens
    {  351, 6336,   58774,  8, }, // Europe/Belgrade
    {  351, 4757,   62753,  7, }, // Europe/Berlin
    {  351, 2167,   57529,  9, }, // Europe/Brussels
    {  351, 3928,   20955,  8, }, // Europe/Bucharest
    {  351, 8772,   26905,  8, }, // Europe/Busingen
    {  351, 6289,   95202, 10, }, // Europe/Copenhagen
    {  351, 2338,   62833,  7, }, // Europe/Dublin
    {  351, 6307,   95212, 10, }, // Europe/Gibraltar
    {  351, 8804,   95222, 12, }, // Europe/Isle_of_Man
    {  351, 2527,    8030,  4, }, // Europe/Kyiv
    {  351, 3213,   93285,  7, }, // Europe/Lisbon
    {  351, 6506,   95234,  7, }, // Europe/Ljubljana
    {  351, 2501,   95241,  6, }, // Europe/London
    {  351, 6365,   95247, 11, }, // Europe/Luxembourg
    {  351, 3537,   95258,  5, }, // Europe/Moscow
    {  351, 4284,   95263,  6, }, // Europe/Paris
    {  351, 6275,   21067,  5, }, // Europe/Prague
    {  351, 6324,    5946,  4, }, // Europe/Rome
    {  351, 8849,   95269, 11, }, // Europe/Simferopol
    {  351, 6523,   95280,  9, }, // Europe/Stockholm
    {  351, 6202,   21095,  6, }, // Europe/Tirane
    {  351, 6567,   95289, 18, }, // Europe/Vatican
    {  351, 3190,   95307,  8, }, // Europe/Warsaw
    {  351, 6261,   95315,  8, }, // Europe/Zagreb
    {  351, 6540,   95323,  6, }, // Europe/Zurich
    {  351, 5029,    4470,  6, }, // Indian/Comoro
    {  351, 7050,    8193,  7, }, // Indian/Maldives
    {  351, 7211,   95329, 11, }, // Indian/Reunion
    {  351, 3157,     474,  5, }, // Pacific/Chuuk
    {  351, 2237,   44510,  6, }, // Pacific/Easter
    {  351, 3902,   95340,  4, }, // Pacific/Fiji
    {  351, 2723,    4493,  8, }, // Pacific/Honolulu
    {  351, 3049,    4501,  9, }, // Pacific/Kanton
    {  351, 4051,   95344,  8, }, // Pacific/Marquesas
    {  351, 3096,     479,  7, }, // Pacific/Pohnpei
    {  355, 6788,   95352,  5, }, // Africa/Abidjan Japanese/Japanese/Japan
    {  355, 6760,   95357,  3, }, // Africa/Accra
    {  355, 5059,   95360,  6, }, // Africa/Addis_Ababa
    {  355, 7420,   95366,  4, }, // Africa/Algiers
    {  355,   14,   95370,  4, }, // Africa/Asmara
    {  355,   44,   95374,  3, }, // Africa/Bamako
    {  355, 5257,   95377,  3, }, // Africa/Bangui
    {  355, 6746,   95380,  6, }, // Africa/Banjul
    {  355, 7435,   95386,  3, }, // Africa/Bissau
    {  355, 4957,   95389,  6, }, // Africa/Blantyre
    {  355, 5287,   95395,  5, }, // Africa/Brazzaville
    {  355, 4922,   95400,  6, }, // Africa/Bujumbura
    {  355, 2320,   95406,  3, }, // Africa/Cairo
    {  355, 4117,   95409,  6, }, // Africa/Casablanca
    {  355, 7449,   95415,  3, }, // Africa/Ceuta
    {  355, 6773,   95418,  4, }, // Africa/Conakry
    {  355, 6840,   95422,  4, }, // Africa/Dakar
    {  355, 5130,   95426,  8, }, // Africa/Dar_es_Salaam
    {  355, 5043,   95434,  3, }, // Africa/Djibouti
    {  355, 5243,   95437,  4, }, // Africa/Douala
    {  355, 5166,   95441,  4, }, // Africa/El_Aaiun
    {  355, 6853,   95445,  6, }, // Africa/Freetown
    {  355, 4906,   95451,  5, }, // Africa/Gaborone
    {  355, 5015,   95456,  3, }, // Africa/Harare
    {  355, 4479,   95459,  7, }, // Africa/Johannesburg
    {  355, 4499,   95466,  3, }, // Africa/Juba
    {  355, 5151,   95469,  4, }, // Africa/Kampala
    {  355, 4524,   95473,  5, }, // Africa/Khartoum
    {  355, 4973,   95478,  3, }, // Africa/Kigali
    {  355, 5306,   95481,  5, }, // Africa/Kinshasa
    {  355, 4744,   95486,  3, }, // Africa/Lagos
    {  355, 5336,   95489,  7, }, // Africa/Libreville
    {  355, 6869,   95496,  2, }, // Africa/Lome
    {  355, 5211,   95498,  4, }, // Africa/Luanda
    {  355, 4939,   95502,  5, }, // Africa/Lubumbashi
    {  355, 5001,   95507,  3, }, // Africa/Lusaka
    {  355, 5322,   95510,  3, }, // Africa/Malabo
    {  355, 4987,   95513,  3, }, // Africa/Maputo
    {  355, 5197,   95516,  3, }, // Africa/Maseru
    {  355, 5182,   95519,  5, }, // Africa/Mbabane
    {  355, 5113,   95524,  6, }, // Africa/Mogadishu
    {  355, 7034,   95530,  5, }, // Africa/Monrovia
    {  355, 3853,   95535,  4, }, // Africa/Nairobi
    {  355, 5271,   95539,  5, }, // Africa/Ndjamena
    {  355, 5354,   95544,  3, }, // Africa/Niamey
    {  355, 6803,   95547,  7, }, // Africa/Nouakchott
    {  355, 6727,   95554,  6, }, // Africa/Ouagadougou
    {  355, 5225,   95560,  5, }, // Africa/Porto-Novo
    {  355, 4435,   95565,  4, }, // Africa/Sao_Tome
    {  355, 2866,   95569,  4, }, // Africa/Tripoli
    {  355, 6554,   95573,  4, }, // Africa/Tunis
    {  355, 4152,   95577,  7, }, // Africa/Windhoek
    {  355,  132,   95584,  4, }, // America/Adak
    {  355, 3346,   95588,  6, }, // America/Anchorage
    {  355, 5607,   95594,  4, }, // America/Anguilla
    {  355, 5624,   95598,  6, }, // America/Antigua
    {  355, 4554,   95604,  6, }, // America/Araguaina
    {  355,  166,   95610,  8, }, // America/Argentina/Buenos_Aires
    {  355,   91,   95618,  5, }, // America/Argentina/Catamarca
    {  355,  270,   95623,  4, }, // America/Argentina/Cordoba
    {  355,  441,   95627,  3, }, // America/Argentina/Jujuy
    {  355, 7462,   95630,  4, }, // America/Argentina/La_Rioja
    {  355,  565,   95634,  5, }, // America/Argentina/Mendoza
    {  355, 7489,   95639,  7, }, // America/Argentina/Rio_Gallegos
    {  355, 7520,   95646,  3, }, // America/Argentina/Salta
    {  355, 7544,   95649,  5, }, // America/Argentina/San_Juan
    {  355, 5580,   95654,  5, }, // America/Argentina/San_Luis
    {  355, 7571,   95659,  5, }, // America/Argentina/Tucuman
    {  355, 7597,   95664,  6, }, // America/Argentina/Ushuaia
    {  355, 5640,   95670,  3, }, // America/Aruba
    {  355, 4252,   95673,  6, }, // America/Asuncion
    {  355,  237,   95679,  6, }, // America/Atikokan
    {  355, 3679,   95685,  4, }, // America/Bahia
    {  355, 7623,   95689,  7, }, // America/Bahia_Banderas
    {  355, 5654,   95696,  5, }, // America/Barbados
    {  355, 7646,   95701,  3, }, // America/Belem
    {  355, 5407,   95704,  4, }, // America/Belize
    {  355, 7660,   95708,  8, }, // America/Blanc-Sablon
    {  355, 7681,   95716,  5, }, // America/Boa_Vista
    {  355, 4361,   95721,  3, }, // America/Bogota
    {  355, 7699,   95724,  3, }, // America/Boise
    {  355, 7713,   95727,  8, }, // America/Cambridge_Bay
    {  355, 7735,   95735,  7, }, // America/Campo_Grande
    {  355, 3868,   95742,  4, }, // America/Cancun
    {  355, 4694,   95746,  4, }, // America/Caracas
    {  355, 4345,   95750,  5, }, // America/Cayenne
    {  355, 5496,   95755,  4, }, // America/Cayman
    {  355, 2260,   95759,  3, }, // America/Chicago
    {  355, 7756,   95762,  3, }, // America/Chihuahua
    {  355, 7774,   95765,  9, }, // America/Ciudad_Juarez
    {  355, 5422,   95774,  5, }, // America/Costa_Rica
    {  355, 7796,   95779,  5, }, // America/Creston
    {  355, 3791,   95784,  4, }, // America/Cuiaba
    {  355, 5723,   95788,  5, }, // America/Curacao
    {  355, 7812,   95793,  9, }, // America/Danmarkshavn
    {  355, 7833,   95802,  4, }, // America/Dawson
    {  355, 7848,   95806,  8, }, // America/Dawson_Creek
    {  355,  805,   95814,  4, }, // America/Denver
    {  355, 3465,   95818,  5, }, // America/Detroit
    {  355, 5739,   95823,  4, }, // America/Dominica
    {  355,  893,   95827,  6, }, // America/Edmonton
    {  355, 7869,   95833,  5, }, // America/Eirunepe
    {  355, 5441,   95838,  7, }, // America/El_Salvador
    {  355, 7886,   95845,  8, }, // America/Fort_Nelson
    {  355, 7906,   95853,  6, }, // America/Fortaleza
    {  355, 7924,   95859,  6, }, // America/Glace_Bay
    {  355, 6881,   95865,  5, }, // America/Goose_Bay
    {  355, 4612,   95870,  7, }, // America/Grand_Turk
    {  355, 5770,   95877,  4, }, // America/Grenada
    {  355, 5786,   95881,  6, }, // America/Guadeloupe
    {  355, 3760,   95887,  5, }, // America/Guatemala
    {  355, 4841,   95892,  5, }, // America/Guayaquil
    {  355, 6932,   95897,  4, }, // America/Guyana
    {  355, 1939,   95901,  6, }, // America/Halifax
    {  355, 2281,   95907,  3, }, // America/Havana
    {  355, 7942,   95910,  5, }, // America/Hermosillo
    {  355,  348,   95915,  9, }, // America/Indiana/Indianapolis
    {  355,  481,   95924, 11, }, // America/Indiana/Knox
    {  355, 7961,   95935, 11, }, // America/Indiana/Marengo
    {  355, 7985,   95946, 15, }, // America/Indiana/Petersburg
    {  355, 8012,   95961, 12, }, // America/Indiana/Tell_City
    {  355, 8038,   95973, 10, }, // America/Indiana/Vevay
    {  355, 8060,   95983, 12, }, // America/Indiana/Vincennes
    {  355, 8086,   95995, 13, }, // America/Indiana/Winamac
    {  355, 8110,   96008,  5, }, // America/Inuvik
    {  355,  660,   96013,  6, }, // America/Iqaluit
    {  355, 2799,   96019,  5, }, // America/Jamaica
    {  355, 5380,   96024,  4, }, // America/Juneau
    {  355,  521,   96028,  4, }, // America/Kentucky/Louisville
    {  355, 8125,   96032, 14, }, // America/Kentucky/Monticello
    {  355, 5704,   96046,  7, }, // America/Kralendijk
    {  355, 4376,   96053,  3, }, // America/La_Paz
    {  355, 7169,   96056,  2, }, // America/Lima
    {  355, 3239,   96058,  6, }, // America/Los_Angeles
    {  355, 5932,   96064, 17, }, // America/Lower_Princes
    {  355, 8153,   96081,  4, }, // America/Maceio
    {  355, 8168,   96085,  4, }, // America/Managua
    {  355, 1908,   96089,  4, }, // America/Manaus
    {  355, 5897,   96093,  3, }, // America/Marigot
    {  355, 5805,   96096,  7, }, // America/Martinique
    {  355, 8184,   96103,  5, }, // America/Matamoros
    {  355, 2917,   96108,  5, }, // America/Mazatlan
    {  355, 8202,   96113,  5, }, // America/Menominee
    {  355, 8220,   96118,  3, }, // America/Merida
    {  355, 8235,   96121,  6, }, // America/Metlakatla
    {  355, 2949,   96127,  8, }, // America/Mexico_City
    {  355, 4391,   96135,  5, }, // America/Miquelon
    {  355, 8254,   96140,  5, }, // America/Moncton
    {  355, 8270,   96145,  5, }, // America/Monterrey
    {  355, 4098,   96150,  6, }, // America/Montevideo
    {  355, 5824,   96156,  6, }, // America/Montserrat
    {  355, 5481,   96162,  4, }, // America/Nassau
    {  355, 2379,   96166,  6, }, // America/New_York
    {  355, 8288,   96172,  3, }, // America/Nome
    {  355, 1850,   96175,  5, }, // America/Noronha
    {  355, 8301,   96180, 12, }, // America/North_Dakota/Beulah
    {  355, 8329,   96192, 11, }, // America/North_Dakota/Center
    {  355, 8357,   96203, 14, }, // America/North_Dakota/New_Salem
    {  355,  393,   96217,  3, }, // America/Nuuk
    {  355, 8388,   96220,  4, }, // America/Ojinaga
    {  355, 2356,   96224,  3, }, // America/Panama
    {  355, 6173,   96227,  5, }, // America/Paramaribo
    {  355, 2973,   96232,  6, }, // America/Phoenix
    {  355, 3945,   96238,  8, }, // America/Port-au-Prince
    {  355, 5954,   96246,  9, }, // America/Port_of_Spain
    {  355, 8404,   96255,  7, }, // America/Porto_Velho
    {  355, 5843,   96262,  6, }, // America/Puerto_Rico
    {  355, 4030,   96268,  7, }, // America/Punta_Arenas
    {  355, 8424,   96275,  9, }, // America/Rankin_Inlet
    {  355, 8445,   96284,  4, }, // America/Recife
    {  355, 1995,   96288,  5, }, // America/Regina
    {  355, 8460,   96293,  6, }, // America/Resolute
    {  355,  695,   96299,  6, }, // America/Rio_Branco
    {  355, 8477,   96305,  5, }, // America/Santarem
    {  355, 2201,   96310,  5, }, // America/Santiago
    {  355, 6111,   96315,  7, }, // America/Santo_Domingo
    {  355, 1878,   96322,  5, }, // America/Sao_Paulo
    {  355, 6899,   96327,  9, }, // America/Scoresbysund
    {  355, 8494,   96336,  3, }, // America/Sitka
    {  355, 8508,   96339,  9, }, // America/St_Barthelemy
    {  355, 2061,   96348,  7, }, // America/St_Johns
    {  355, 5863,   96355, 10, }, // America/St_Kitts
    {  355, 5880,   96365,  6, }, // America/St_Lucia
    {  355,  855,   96371,  7, }, // America/St_Thomas
    {  355, 5913,   96378,  8, }, // America/St_Vincent
    {  355, 8530,   96386,  9, }, // America/Swift_Current
    {  355, 5461,   96395,  6, }, // America/Tegucigalpa
    {  355, 5756,   96401,  4, }, // America/Thule
    {  355,  313,   96405,  5, }, // America/Tijuana
    {  355,  608,   96410,  4, }, // America/Toronto
    {  355, 5688,   96414,  5, }, // America/Tortola
    {  355, 2093,   96419,  6, }, // America/Vancouver
    {  355, 2144,   96425,  7, }, // America/Whitehorse
    {  355,  734,   96432,  5, }, // America/Winnipeg
    {  355, 8552,   96437,  5, }, // America/Yakutat
    {  355, 6001,   96442,  6, }, // Antarctica/Casey
    {  355, 6094,   96448,  6, }, // Antarctica/Davis
    {  355, 6133,   96454, 13, }, // Antarctica/DumontDUrville
    {  355, 8568,   96467,  5, }, // Antarctica/Macquarie
    {  355, 7066,   96472,  6, }, // Antarctica/Mawson
    {  355, 7113,   96478,  7, }, // Antarctica/McMurdo
    {  355, 8589,   96485,  6, }, // Antarctica/Palmer
    {  355, 7226,   96491,  5, }, // Antarctica/Rothera
    {  355, 7295,   42193,  4, }, // Antarctica/Syowa
    {  355, 8607,   96496,  5, }, // Antarctica/Troll
    {  355, 7374,   96501,  7, }, // Antarctica/Vostok
    {  355, 1429,   96508, 10, }, // Arctic/Longyearbyen
    {  355, 5570,   96518,  3, }, // Asia/Aden
    {  355, 5395,   96521,  5, }, // Asia/Almaty
    {  355, 3968,   96526,  4, }, // Asia/Amman
    {  355, 5511,   96530,  5, }, // Asia/Anadyr
    {  355, 5523,   96535,  4, }, // Asia/Aqtau
    {  355, 5368,   96539,  4, }, // Asia/Aqtobe
    {  355,  964,   96543,  6, }, // Asia/Ashgabat
    {  355, 8624,   96549,  5, }, // Asia/Atyrau
    {  355, 3607,   96554,  5, }, // Asia/Baghdad
    {  355, 5534,   96559,  5, }, // Asia/Bahrain
    {  355, 3653,   96564,  3, }, // Asia/Baku
    {  355, 4466,   96567,  4, }, // Asia/Bangkok
    {  355, 3571,   96571,  5, }, // Asia/Barnaul
    {  355, 4086,   96576,  5, }, // Asia/Beirut
    {  355, 3778,   96581,  5, }, // Asia/Bishkek
    {  355, 5989,   96586,  4, }, // Asia/Brunei
    {  355, 4601,   96590,  2, }, // Asia/Chita
    {  355, 4511,   96592,  4, }, // Asia/Colombo
    {  355, 4540,   96596,  5, }, // Asia/Damascus
    {  355, 1093,   96601,  3, }, // Asia/Dhaka
    {  355, 6192,   96532,  3, }, // Asia/Dili
    {  355, 3596,   96604,  3, }, // Asia/Dubai
    {  355, 6159,   96607,  6, }, // Asia/Dushanbe
    {  355, 8636,   96613,  6, }, // Asia/Famagusta
    {  355, 8651,   96619,  2, }, // Asia/Gaza
    {  355, 4795,   96621,  4, }, // Asia/Hebron
    {  355, 1259,   96625,  5, }, // Asia/Ho_Chi_Minh
    {  355, 2704,   42293,  2, }, // Asia/Hong_Kong
    {  355, 4771,   96630,  3, }, // Asia/Hovd
    {  355, 4184,   96633,  6, }, // Asia/Irkutsk
    {  355, 7006,   96639,  5, }, // Asia/Jakarta
    {  355, 6992,   96644,  5, }, // Asia/Jayapura
    {  355, 1290,   96649,  5, }, // Asia/Jerusalem
    {  355, 3560,   96654,  4, }, // Asia/Kabul
    {  355, 4316,   96658,  6, }, // Asia/Kamchatka
    {  355, 4239,   96664,  3, }, // Asia/Karachi
    {  355, 1185,   96667,  5, }, // Asia/Kathmandu
    {  355, 8661,   96672,  6, }, // Asia/Khandyga
    {  355,  992,   96678,  4, }, // Asia/Kolkata
    {  355, 4197,   96682,  8, }, // Asia/Krasnoyarsk
    {  355, 4859,   96690,  8, }, // Asia/Kuala_Lumpur
    {  355, 5976,   96698,  3, }, // Asia/Kuching
    {  355, 5547,   96701,  5, }, // Asia/Kuwait
    {  355, 1211,   96706,  3, }, // Asia/Macau
    {  355, 4017,   96709,  4, }, // Asia/Magadan
    {  355, 1349,   96713,  5, }, // Asia/Makassar
    {  355, 7182,   96718,  3, }, // Asia/Manila
    {  355, 6920,   96721,  5, }, // Asia/Muscat
    {  355, 2554,   96726,  4, }, // Asia/Nicosia
    {  355, 8675,   96730,  8, }, // Asia/Novokuznetsk
    {  355, 4135,   96738,  8, }, // Asia/Novosibirsk
    {  355, 4229,   96746,  4, }, // Asia/Omsk
    {  355, 7145,   96750,  3, }, // Asia/Oral
    {  355, 6961,   96753,  5, }, // Asia/Phnom_Penh
    {  355, 8693,   96758,  8, }, // Asia/Pontianak
    {  355, 4214,   96766,  2, }, // Asia/Pyongyang
    {  355, 5559,   96768,  4, }, // Asia/Qatar
    {  355, 8708,   96772,  5, }, // Asia/Qostanay
    {  355, 4269,   96777,  5, }, // Asia/Qyzylorda
    {  355, 3584,   96782,  3, }, // Asia/Riyadh
    {  355, 4408,   96785,  4, }, // Asia/Sakhalin
    {  355, 7245,   96789,  6, }, // Asia/Samarkand
    {  355, 3279,   96795,  3, }, // Asia/Seoul
    {  355, 1053,   42435,  2, }, // Asia/Shanghai
    {  355, 3300,   96798,  6, }, // Asia/Singapore
    {  355, 4297,   96804,  9, }, // Asia/Srednekolymsk
    {  355, 3263,   42446,  2, }, // Asia/Taipei
    {  355, 4781,   96813,  5, }, // Asia/Tashkent
    {  355, 3915,   96818,  4, }, // Asia/Tbilisi
    {  355, 2772,   96822,  4, }, // Asia/Tehran
    {  355, 1317,   96826,  5, }, // Asia/Thimphu
    {  355, 2821,   42460,  2, }, // Asia/Tokyo
    {  355, 4572,   96831,  4, }, // Asia/Tomsk
    {  355, 1021,   96835,  7, }, // Asia/Ulaanbaatar
    {  355, 1159,   96842,  4, }, // Asia/Urumqi
    {  355, 8722,   96846,  5, }, // Asia/Ust-Nera
    {  355, 6977,   96851,  6, }, // Asia/Vientiane
    {  355, 4710,   96857,  7, }, // Asia/Vladivostok
    {  355, 4828,   96864,  5, }, // Asia/Yakutsk
    {  355, 1235,   96869,  4, }, // Asia/Yangon
    {  355, 3883,   96873,  8, }, // Asia/Yekaterinburg
    {  355, 3747,   96881,  4, }, // Asia/Yerevan
    {  355, 3663,   96885,  4, }, // Atlantic/Azores
    {  355, 5671,   96889,  5, }, // Atlantic/Bermuda
    {  355, 6628,   96894,  4, }, // Atlantic/Canary
    {  355, 3727,   96898,  6, }, // Atlantic/Cape_Verde
    {  355, 1395,   96904,  4, }, // Atlantic/Faroe
    {  355, 8736,   96908,  4, }, // Atlantic/Madeira
    {  355, 2748,   96912,  6, }, // Atlantic/Reykjavik
    {  355, 7272,   96918,  8, }, // Atlantic/South_Georgia
    {  355, 6821,   96926,  6, }, // Atlantic/St_Helena
    {  355, 6644,   96932,  5, }, // Atlantic/Stanley
    {  355, 1670,   96937,  5, }, // Australia/Adelaide
    {  355, 1635,   96942,  5, }, // Australia/Brisbane
    {  355, 1799,   96947,  7, }, // Australia/Broken_Hill
    {  355, 1583,   96954,  5, }, // Australia/Darwin
    {  355, 3637,   96959,  4, }, // Australia/Eucla
    {  355, 1516,   96963,  4, }, // Australia/Hobart
    {  355, 8753,   96967,  5, }, // Australia/Lindeman
    {  355, 1547,   96972,  5, }, // Australia/Lord_Howe
    {  355, 1727,   96977,  5, }, // Australia/Melbourne
    {  355, 1762,   96982,  3, }, // Australia/Perth
    {  355, 1463,   96985,  4, }, // Australia/Sydney
    {  355, 6441,   96989,  7, }, // Europe/Amsterdam
    {  355, 6216,   96996,  4, }, // Europe/Andorra
    {  355, 3620,   97000,  6, }, // Europe/Astrakhan
    {  355, 2300,   97006,  3, }, // Europe/Athens
    {  355, 6336,   97009,  6, }, // Europe/Belgrade
    {  355, 4757,   97015,  4, }, // Europe/Berlin
    {  355, 6488,   97019,  6, }, // Europe/Bratislava
    {  355, 2167,   97025,  6, }, // Europe/Brussels
    {  355, 3928,   97031,  5, }, // Europe/Bucharest
    {  355, 3806,   97036,  5, }, // Europe/Budapest
    {  355, 8772,   97041,  7, }, // Europe/Busingen
    {  355, 2583,   97048,  4, }, // Europe/Chisinau
    {  355, 6289,   97052,  7, }, // Europe/Copenhagen
    {  355, 2338,   97059,  4, }, // Europe/Dublin
    {  355, 6307,   97063,  6, }, // Europe/Gibraltar
    {  355, 8788,   97069,  5, }, // Europe/Guernsey
    {  355, 6612,   97074,  5, }, // Europe/Helsinki
    {  355, 8804,   97079,  3, }, // Europe/Isle_of_Man
    {  355, 1130,   97082,  7, }, // Europe/Istanbul
    {  355, 8823,   97089,  5, }, // Europe/Jersey
    {  355, 3979,   97094,  9, }, // Europe/Kaliningrad
    {  355, 8929,   97103,  4, }, // Europe/Kirov
    {  355, 2527,   97107,  3, }, // Europe/Kyiv
    {  355, 3213,   97110,  4, }, // Europe/Lisbon
    {  355, 6506,   97114,  6, }, // Europe/Ljubljana
    {  355, 2501,   97120,  4, }, // Europe/London
    {  355, 6365,   97124,  7, }, // Europe/Luxembourg
    {  355, 4892,   97131,  5, }, // Europe/Madrid
    {  355, 6397,   97136,  3, }, // Europe/Malta
    {  355, 6582,   97139,  6, }, // Europe/Mariehamn
    {  355, 3693,   97145,  4, }, // Europe/Minsk
    {  355, 6410,   97149,  3, }, // Europe/Monaco
    {  355, 3537,   97152,  4, }, // Europe/Moscow
    {  355, 6458,   97156,  3, }, // Europe/Oslo
    {  355, 4284,   97159,  2, }, // Europe/Paris
    {  355, 6424,   97161,  6, }, // Europe/Podgorica
    {  355, 6275,   97167,  3, }, // Europe/Prague
    {  355, 8837,   97170,  2, }, // Europe/Riga
    {  355, 6324,   96906,  3, }, // Europe/Rome
    {  355, 4331,   97172,  3, }, // Europe/Samara
    {  355, 6470,   97175,  5, }, // Europe/San_Marino
    {  355, 6245,   97180,  4, }, // Europe/Sarajevo
    {  355, 4451,   97184,  4, }, // Europe/Saratov
    {  355, 8849,   97188,  7, }, // Europe/Simferopol
    {  355, 6383,   97195,  4, }, // Europe/Skopje
    {  355, 6599,   97199,  4, }, // Europe/Sofia
    {  355, 6523,   97203,  7, }, // Europe/Stockholm
    {  355, 8867,   97210,  3, }, // Europe/Tallinn
    {  355, 6202,   97213,  4, }, // Europe/Tirane
    {  355, 8882,   97217,  7, }, // Europe/Ulyanovsk
    {  355, 6352,   97224,  6, }, // Europe/Vaduz
    {  355, 6567,   97230,  4, }, // Europe/Vatican
    {  355, 6231,   97234,  4, }, // Europe/Vienna
    {  355, 8899,   97238,  6, }, // Europe/Vilnius
    {  355, 4727,   97244,  7, }, // Europe/Volgograd
    {  355, 3190,   97251,  5, }, // Europe/Warsaw
    {  355, 6261,   97256,  4, }, // Europe/Zagreb
    {  355, 6540,   97260,  6, }, // Europe/Zurich
    {  355, 5078,   97266,  7, }, // Indian/Antananarivo
    {  355, 6947,   97273,  4, }, // Indian/Chagos
    {  355, 6046,   97277,  6, }, // Indian/Christmas
    {  355, 6063,   97283,  5, }, // Indian/Cocos
    {  355, 5029,   97288,  3, }, // Indian/Comoro
    {  355, 6661,   97291,  7, }, // Indian/Kerguelen
    {  355, 7260,   97298,  2, }, // Indian/Mahe
    {  355, 7050,   97300,  5, }, // Indian/Maldives
    {  355, 4069,   97305,  6, }, // Indian/Mauritius
    {  355, 5098,   97311,  4, }, // Indian/Mayotte
    {  355, 7211,   97315,  5, }, // Indian/Reunion
    {  355, 4422,   97320,  3, }, // Pacific/Apia
    {  355,  932,   97323,  6, }, // Pacific/Auckland
    {  355, 3706,   97329,  6, }, // Pacific/Bougainville
    {  355, 3015,   97335,  4, }, // Pacific/Chatham
    {  355, 3157,   97339,  4, }, // Pacific/Chuuk
    {  355, 2237,   97343,  6, }, // Pacific/Easter
    {  355, 7360,   97349,  5, }, // Pacific/Efate
    {  355, 7327,   97354,  6, }, // Pacific/Fakaofo
    {  355, 3902,   97360,  4, }, // Pacific/Fiji
    {  355, 7343,   97364,  5, }, // Pacific/Funafuti
    {  355, 6678,   97369,  5, }, // Pacific/Galapagos
    {  355, 6696,   97374,  6, }, // Pacific/Gambier
    {  355, 3822,   97380,  6, }, // Pacific/Guadalcanal
    {  355, 6018,   97386,  3, }, // Pacific/Guam
    {  355, 2723,   97389,  4, }, // Pacific/Honolulu
    {  355, 3049,   97393,  8, }, // Pacific/Kanton
    {  355, 3998,   97401,  8, }, // Pacific/Kiritimati
    {  355, 7019,   97409,  4, }, // Pacific/Kosrae
    {  355, 2842,   97413,  5, }, // Pacific/Kwajalein
    {  355, 4877,   97418,  4, }, // Pacific/Majuro
    {  355, 4051,   97422,  5, }, // Pacific/Marquesas
    {  355, 8914,   97427,  7, }, // Pacific/Midway
    {  355, 7084,   96573,  3, }, // Pacific/Nauru
    {  355, 7132,   97434,  3, }, // Pacific/Niue
    {  355, 4168,   97437,  7, }, // Pacific/Norfolk
    {  355, 7098,   97444,  3, }, // Pacific/Noumea
    {  355, 3126,   97447,  4, }, // Pacific/Pago_Pago
    {  355, 7155,   97451,  3, }, // Pacific/Palau
    {  355, 7194,   97454,  7, }, // Pacific/Pitcairn
    {  355, 3096,   97461,  5, }, // Pacific/Pohnpei
    {  355, 4807,   97466,  8, }, // Pacific/Port_Moresby
    {  355, 6076,   97474,  5, }, // Pacific/Rarotonga
    {  355, 6031,   97479,  4, }, // Pacific/Saipan
    {  355, 7312,   97483,  3, }, // Pacific/Tahiti
    {  355, 6712,   97486,  3, }, // Pacific/Tarawa
    {  355, 4583,   97489,  5, }, // Pacific/Tongatapu
    {  355, 7392,   97494,  5, }, // Pacific/Wake
    {  355, 7405,   97499,  6, }, // Pacific/Wallis
    {  356,   14,       0,  6, }, // Africa/Asmara Javanese/Latin/Indonesia
    {  356, 2320,    4572,  5, }, // Africa/Cairo
    {  356, 4117,    4577, 10, }, // Africa/Casablanca
    {  356, 6773,    6249,  7, }, // Africa/Conakry
    {  356, 4435,       6,  8, }, // Africa/Sao_Tome
    {  356,   91,    6400,  9, }, // America/Argentina/Catamarca
    {  356,  270,    6409,  7, }, // America/Argentina/Cordoba
    {  356,  565,   97505,  7, }, // America/Argentina/Mendoza
    {  356, 4252,      14,  8, }, // America/Asuncion
    {  356,  237,      22,  8, }, // America/Atikokan
    {  356, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  356, 5407,   97512,  6, }, // America/Belize
    {  356, 7713,   97518, 15, }, // America/Cambridge_Bay
    {  356, 7735,    4780, 12, }, // America/Campo_Grande
    {  356, 3868,   33182,  6, }, // America/Cancun
    {  356, 4694,    4792,  7, }, // America/Caracas
    {  356, 4345,   97533,  7, }, // America/Cayenne
    {  356, 5496,   44041,  6, }, // America/Cayman
    {  356, 7774,   97540, 13, }, // America/Ciudad_Juarez
    {  356, 5422,   21503, 10, }, // America/Costa_Rica
    {  356, 3791,    4846,  6, }, // America/Cuiaba
    {  356, 5723,      66,  7, }, // America/Curacao
    {  356, 5739,   21548,  8, }, // America/Dominica
    {  356, 7886,   97553, 14, }, // America/Fort_Nelson
    {  356, 7924,   97567, 11, }, // America/Glace_Bay
    {  356, 6881,   97578, 11, }, // America/Goose_Bay
    {  356, 1939,    6787,  7, }, // America/Halifax
    {  356,  481,   64624, 14, }, // America/Indiana/Knox
    {  356, 7961,   64638, 17, }, // America/Indiana/Marengo
    {  356, 7985,   64655, 20, }, // America/Indiana/Petersburg
    {  356, 8012,   64675, 19, }, // America/Indiana/Tell_City
    {  356, 8038,   64694, 15, }, // America/Indiana/Vevay
    {  356, 8060,   64709, 19, }, // America/Indiana/Vincennes
    {  356, 8086,   64728, 17, }, // America/Indiana/Winamac
    {  356, 8125,   64753, 21, }, // America/Kentucky/Monticello
    {  356, 5932,     207, 22, }, // America/Lower_Princes
    {  356, 5805,    5142,  8, }, // America/Martinique
    {  356, 8220,    7008,  6, }, // America/Merida
    {  356, 2949,   97589, 13, }, // America/Mexico_City
    {  356, 8301,   97602, 19, }, // America/North_Dakota/Beulah
    {  356, 8329,   97621, 19, }, // America/North_Dakota/Center
    {  356, 8357,   97640, 24, }, // America/North_Dakota/New_Salem
    {  356,  393,     298,  4, }, // America/Nuuk
    {  356, 5954,   97664, 17, }, // America/Port_of_Spain
    {  356, 5843,   21940, 11, }, // America/Puerto_Rico
    {  356, 6899,     302, 16, }, // America/Scoresbysund
    {  356, 8508,   97681, 16, }, // America/St_Barthelemy
    {  356, 2061,   97697, 10, }, // America/St_Johns
    {  356, 5863,   97707, 11, }, // America/St_Kitts
    {  356, 5880,   95111, 11, }, // America/St_Lucia
    {  356,  855,   97718, 12, }, // America/St_Thomas
    {  356, 5913,   97730, 13, }, // America/St_Vincent
    {  356, 8530,   97743, 11, }, // America/Swift_Current
    {  356, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  356, 4511,    5519,  7, }, // Asia/Colombo
    {  356, 4540,    4083,  8, }, // Asia/Damascus
    {  356, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  356, 1290,    5568,  9, }, // Asia/Jerusalem
    {  356, 1185,     413,  9, }, // Asia/Kathmandu
    {  356,  992,   26732,  7, }, // Asia/Kolkata
    {  356, 1211,    4203,  5, }, // Asia/Macau
    {  356, 1349,    7618,  7, }, // Asia/Makassar
    {  356, 8708,   64229,  8, }, // Asia/Qostanay
    {  356, 3300,   93044,  9, }, // Asia/Singapore
    {  356, 2772,    7724,  7, }, // Asia/Tehran
    {  356, 1235,     442,  6, }, // Asia/Yangon
    {  356, 6628,    4280,  6, }, // Atlantic/Canary
    {  356, 3727,   22522, 10, }, // Atlantic/Cape_Verde
    {  356, 1395,     448,  5, }, // Atlantic/Faroe
    {  356, 7272,   97754, 13, }, // Atlantic/South_Georgia
    {  356, 6821,   26865, 12, }, // Atlantic/St_Helena
    {  356, 2300,   93081,  6, }, // Europe/Athens
    {  356, 6289,    4357, 10, }, // Europe/Copenhagen
    {  356, 8804,   97767,  8, }, // Europe/Isle_of_Man
    {  356, 2527,    8030,  4, }, // Europe/Kyiv
    {  356, 6365,   97775, 10, }, // Europe/Luxembourg
    {  356, 6410,    5926,  6, }, // Europe/Monaco
    {  356, 6324,    5946,  4, }, // Europe/Rome
    {  356, 6567,    5978,  7, }, // Europe/Vatican
    {  356, 6947,   97785,  6, }, // Indian/Chagos
    {  356, 6046,   93292,  5, }, // Indian/Christmas
    {  356, 5029,   22853,  6, }, // Indian/Comoro
    {  356, 7050,   93117,  8, }, // Indian/Maldives
    {  356, 7211,     467,  7, }, // Indian/Reunion
    {  356, 3157,     474,  5, }, // Pacific/Chuuk
    {  356, 2237,   97791,  6, }, // Pacific/Easter
    {  356, 3049,    4501,  9, }, // Pacific/Kanton
    {  356, 3096,     479,  7, }, // Pacific/Pohnpei
    {  356, 4807,   97797, 17, }, // Pacific/Port_Moresby
    {  357,   14,       0,  6, }, // Africa/Asmara Jju/Latin/Nigeria
    {  357, 4435,       6,  8, }, // Africa/Sao_Tome
    {  357, 4252,      14,  8, }, // America/Asuncion
    {  357,  237,      22,  8, }, // America/Atikokan
    {  357, 7623,      30, 17, }, // America/Bahia_Banderas
    {  357, 3868,      47,  6, }, // America/Cancun
    {  357, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  357, 5723,      66,  7, }, // America/Curacao
    {  357,  481,      73, 13, }, // America/Indiana/Knox
    {  357, 7961,      86, 16, }, // America/Indiana/Marengo
    {  357, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  357, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  357, 8038,     139, 14, }, // America/Indiana/Vevay
    {  357, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  357, 8086,     171, 16, }, // America/Indiana/Winamac
    {  357, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  357, 5932,     207, 22, }, // America/Lower_Princes
    {  357, 8220,     229,  6, }, // America/Merida
    {  357, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  357, 8329,     255, 20, }, // America/North_Dakota/Center
    {  357, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  357,  393,     298,  4, }, // America/Nuuk
    {  357, 6899,     302, 16, }, // America/Scoresbysund
    {  357, 8508,     318, 14, }, // America/St_Barthelemy
    {  357, 2061,     332, 10, }, // America/St_Johns
    {  357, 5863,     342,  9, }, // America/St_Kitts
    {  357, 5880,     351,  9, }, // America/St_Lucia
    {  357,  855,     360, 10, }, // America/St_Thomas
    {  357, 5913,     370, 11, }, // America/St_Vincent
    {  357, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  357, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  357, 1185,     413,  9, }, // Asia/Kathmandu
    {  357,  992,     422,  7, }, // Asia/Kolkata
    {  357, 1211,     429,  5, }, // Asia/Macau
    {  357, 1235,     442,  6, }, // Asia/Yangon
    {  357, 1395,     448,  5, }, // Atlantic/Faroe
    {  357, 6821,     453, 10, }, // Atlantic/St_Helena
    {  357, 2527,     463,  4, }, // Europe/Kyiv
    {  357, 7211,     467,  7, }, // Indian/Reunion
    {  357, 3157,     474,  5, }, // Pacific/Chuuk
    {  357, 3096,     479,  7, }, // Pacific/Pohnpei
    {  358,   14,       0,  6, }, // Africa/Asmara Jola-Fonyi/Latin/Senegal
    {  358, 4435,       6,  8, }, // Africa/Sao_Tome
    {  358, 4252,      14,  8, }, // America/Asuncion
    {  358,  237,      22,  8, }, // America/Atikokan
    {  358, 7623,      30, 17, }, // America/Bahia_Banderas
    {  358, 3868,      47,  6, }, // America/Cancun
    {  358, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  358, 5723,      66,  7, }, // America/Curacao
    {  358,  481,      73, 13, }, // America/Indiana/Knox
    {  358, 7961,      86, 16, }, // America/Indiana/Marengo
    {  358, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  358, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  358, 8038,     139, 14, }, // America/Indiana/Vevay
    {  358, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  358, 8086,     171, 16, }, // America/Indiana/Winamac
    {  358, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  358, 5932,     207, 22, }, // America/Lower_Princes
    {  358, 8220,     229,  6, }, // America/Merida
    {  358, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  358, 8329,     255, 20, }, // America/North_Dakota/Center
    {  358, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  358,  393,     298,  4, }, // America/Nuuk
    {  358, 6899,     302, 16, }, // America/Scoresbysund
    {  358, 8508,     318, 14, }, // America/St_Barthelemy
    {  358, 2061,     332, 10, }, // America/St_Johns
    {  358, 5863,     342,  9, }, // America/St_Kitts
    {  358, 5880,     351,  9, }, // America/St_Lucia
    {  358,  855,     360, 10, }, // America/St_Thomas
    {  358, 5913,     370, 11, }, // America/St_Vincent
    {  358, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  358, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  358, 1185,     413,  9, }, // Asia/Kathmandu
    {  358,  992,     422,  7, }, // Asia/Kolkata
    {  358, 1211,     429,  5, }, // Asia/Macau
    {  358, 1235,     442,  6, }, // Asia/Yangon
    {  358, 1395,     448,  5, }, // Atlantic/Faroe
    {  358, 6821,     453, 10, }, // Atlantic/St_Helena
    {  358, 2527,     463,  4, }, // Europe/Kyiv
    {  358, 7211,     467,  7, }, // Indian/Reunion
    {  358, 3157,     474,  5, }, // Pacific/Chuuk
    {  358, 3096,     479,  7, }, // Pacific/Pohnpei
    {  359,   14,       0,  6, }, // Africa/Asmara Kabuverdianu/Latin/Cape Verde
    {  359, 4435,       6,  8, }, // Africa/Sao_Tome
    {  359,  132,   97814,  5, }, // America/Adak
    {  359, 3346,   97819,  8, }, // America/Anchorage
    {  359, 5607,   97827,  6, }, // America/Anguilla
    {  359, 4252,      14,  8, }, // America/Asuncion
    {  359,  237,      22,  8, }, // America/Atikokan
    {  359, 7623,   97833, 16, }, // America/Bahia_Banderas
    {  359, 5654,   97849,  8, }, // America/Barbados
    {  359, 5407,   97857,  6, }, // America/Belize
    {  359, 7660,    6502, 12, }, // America/Blanc-Sablon
    {  359, 7699,    6529,  4, }, // America/Boise
    {  359, 3868,    6564,  6, }, // America/Cancun
    {  359, 5496,   26436,  6, }, // America/Cayman
    {  359, 2260,   97863,  6, }, // America/Chicago
    {  359, 7756,   97869,  6, }, // America/Chihuahua
    {  359, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  359, 5422,   21503, 10, }, // America/Costa_Rica
    {  359, 7796,   97875,  7, }, // America/Creston
    {  359, 5723,   97882,  7, }, // America/Curacao
    {  359, 5739,   21548,  8, }, // America/Dominica
    {  359, 5770,   20401,  7, }, // America/Grenada
    {  359, 5786,   97889,  9, }, // America/Guadeloupe
    {  359, 3760,   97898,  9, }, // America/Guatemala
    {  359, 2281,   94991,  5, }, // America/Havana
    {  359,  348,   74114, 12, }, // America/Indiana/Indianapolis
    {  359,  481,      73, 13, }, // America/Indiana/Knox
    {  359, 7961,      86, 16, }, // America/Indiana/Marengo
    {  359, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  359, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  359, 8038,     139, 14, }, // America/Indiana/Vevay
    {  359, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  359, 8086,     171, 16, }, // America/Indiana/Winamac
    {  359, 2799,    3877,  7, }, // America/Jamaica
    {  359, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  359, 5932,     207, 22, }, // America/Lower_Princes
    {  359, 8168,   97907,  7, }, // America/Managua
    {  359, 5805,   26462,  9, }, // America/Martinique
    {  359, 8220,     229,  6, }, // America/Merida
    {  359, 2949,   97914, 16, }, // America/Mexico_City
    {  359, 4391,    7031,  7, }, // America/Miquelon
    {  359, 2379,   97930, 10, }, // America/New_York
    {  359, 8301,   97940, 23, }, // America/North_Dakota/Beulah
    {  359, 8329,   97963, 23, }, // America/North_Dakota/Center
    {  359, 8357,   97986, 26, }, // America/North_Dakota/New_Salem
    {  359,  393,     298,  4, }, // America/Nuuk
    {  359, 2356,   20547,  6, }, // America/Panama
    {  359, 5954,   98012, 15, }, // America/Port_of_Spain
    {  359, 5843,   98027, 10, }, // America/Puerto_Rico
    {  359, 6111,   98037, 13, }, // America/Santo_Domingo
    {  359, 6899,     302, 16, }, // America/Scoresbysund
    {  359, 8508,   98050, 14, }, // America/St_Barthelemy
    {  359, 2061,     332, 10, }, // America/St_Johns
    {  359, 5863,   98064, 13, }, // America/St_Kitts
    {  359, 5880,   98077, 11, }, // America/St_Lucia
    {  359,  855,     360, 10, }, // America/St_Thomas
    {  359, 5913,   98088, 11, }, // America/St_Vincent
    {  359,  608,   98099,  7, }, // America/Toronto
    {  359, 2093,    7375,  8, }, // America/Vancouver
    {  359,  734,   98106,  7, }, // America/Winnipeg
    {  359, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  359, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  359, 1185,     413,  9, }, // Asia/Kathmandu
    {  359,  992,     422,  7, }, // Asia/Kolkata
    {  359, 1211,     429,  5, }, // Asia/Macau
    {  359, 1235,     442,  6, }, // Asia/Yangon
    {  359, 1395,     448,  5, }, // Atlantic/Faroe
    {  359, 6821,     453, 10, }, // Atlantic/St_Helena
    {  359, 2527,     463,  4, }, // Europe/Kyiv
    {  359, 7211,     467,  7, }, // Indian/Reunion
    {  359, 3157,     474,  5, }, // Pacific/Chuuk
    {  359, 3096,     479,  7, }, // Pacific/Pohnpei
    {  360, 6788,   98113,  6, }, // Africa/Abidjan Kabyle/Latin/Algeria
    {  360, 6760,   98119,  4, }, // Africa/Accra
    {  360, 5059,   98123, 10, }, // Africa/Addis_Ababa
    {  360, 7420,   98133,  8, }, // Africa/Algiers
    {  360,   14,       0,  6, }, // Africa/Asmara
    {  360,   44,   98141,  6, }, // Africa/Bamako
    {  360, 5257,   26348,  5, }, // Africa/Bangui
    {  360, 7435,    4550,  5, }, // Africa/Bissau
    {  360, 4957,    6200,  7, }, // Africa/Blantyre
    {  360, 5287,   98147,  8, }, // Africa/Brazzaville
    {  360, 2320,   98155,  7, }, // Africa/Cairo
    {  360, 4117,   98162, 10, }, // Africa/Casablanca
    {  360, 7449,   98172,  5, }, // Africa/Ceuta
    {  360, 6773,   98177,  7, }, // Africa/Conakry
    {  360, 5130,   98184,  9, }, // Africa/Dar_es_Salaam
    {  360, 5043,   98193,  6, }, // Africa/Djibouti
    {  360, 5243,   98199,  5, }, // Africa/Douala
    {  360, 5166,   98204,  6, }, // Africa/El_Aaiun
    {  360, 6853,   98210,  7, }, // Africa/Freetown
    {  360, 4906,   98217,  7, }, // Africa/Gaborone
    {  360, 5015,   98224,  6, }, // Africa/Harare
    {  360, 4479,   98230, 11, }, // Africa/Johannesburg
    {  360, 4524,   98241,  6, }, // Africa/Khartoum
    {  360, 5306,   98247,  7, }, // Africa/Kinshasa
    {  360, 4744,   98254,  5, }, // Africa/Lagos
    {  360, 5336,   98259,  8, }, // Africa/Libreville
    {  360, 6869,   98267,  4, }, // Africa/Lome
    {  360, 5211,   98271,  6, }, // Africa/Luanda
    {  360, 4939,   98277,  9, }, // Africa/Lubumbashi
    {  360, 5001,   98286,  6, }, // Africa/Lusaka
    {  360, 5322,   98292,  6, }, // Africa/Malabo
    {  360, 4987,   98298,  6, }, // Africa/Maputo
    {  360, 5197,   98304,  6, }, // Africa/Maseru
    {  360, 5182,   61178,  6, }, // Africa/Mbabane
    {  360, 5113,   98310,  8, }, // Africa/Mogadishu
    {  360, 7034,   98318,  8, }, // Africa/Monrovia
    {  360, 3853,   98326,  7, }, // Africa/Nairobi
    {  360, 5271,   98333,  7, }, // Africa/Ndjamena
    {  360, 5354,   98340,  6, }, // Africa/Niamey
    {  360, 6803,   98346,  7, }, // Africa/Nouakchott
    {  360, 6727,    4640,  8, }, // Africa/Ouagadougou
    {  360, 5225,   98353, 10, }, // Africa/Porto-Novo
    {  360, 4435,   98363,  8, }, // Africa/Sao_Tome
    {  360, 2866,   98371,  7, }, // Africa/Tripoli
    {  360, 6554,   74069,  5, }, // Africa/Tunis
    {  360, 4152,   98378,  7, }, // Africa/Windhoek
    {  360, 3346,   98385,  7, }, // America/Anchorage
    {  360, 5607,   97827,  6, }, // America/Anguilla
    {  360, 5624,   98392,  6, }, // America/Antigua
    {  360,   91,    6400,  9, }, // America/Argentina/Catamarca
    {  360,  270,   98398,  7, }, // America/Argentina/Cordoba
    {  360,  565,   98405,  7, }, // America/Argentina/Mendoza
    {  360, 7544,   98412,  8, }, // America/Argentina/San_Juan
    {  360, 5580,   98420,  8, }, // America/Argentina/San_Luis
    {  360, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {  360, 7597,   98428,  6, }, // America/Argentina/Ushuaia
    {  360, 4252,   98434,  8, }, // America/Asuncion
    {  360,  237,   98442,  8, }, // America/Atikokan
    {  360, 3679,   98450,  5, }, // America/Bahia
    {  360, 7623,   98455, 14, }, // America/Bahia_Banderas
    {  360, 5654,   97849,  8, }, // America/Barbados
    {  360, 5407,   98469,  5, }, // America/Belize
    {  360, 7660,   98474, 11, }, // America/Blanc-Sablon
    {  360, 4361,   98485,  6, }, // America/Bogota
    {  360, 7699,   98491,  4, }, // America/Boise
    {  360, 7713,   98495, 11, }, // America/Cambridge_Bay
    {  360, 7735,   98506, 12, }, // America/Campo_Grande
    {  360, 3868,    6564,  6, }, // America/Cancun
    {  360, 4694,    4792,  7, }, // America/Caracas
    {  360, 4345,   98518,  5, }, // America/Cayenne
    {  360, 5496,   21483,  6, }, // America/Cayman
    {  360, 2260,   98523,  6, }, // America/Chicago
    {  360, 7756,   98529,  6, }, // America/Chihuahua
    {  360, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  360, 5422,   98535, 10, }, // America/Costa_Rica
    {  360, 7796,   98545,  7, }, // America/Creston
    {  360, 5723,   98552,  7, }, // America/Curacao
    {  360, 7812,   98559, 12, }, // America/Danmarkshavn
    {  360, 7833,   98571,  6, }, // America/Dawson
    {  360, 7848,   98577, 11, }, // America/Dawson_Creek
    {  360, 3465,   98588,  6, }, // America/Detroit
    {  360, 5739,   98594,  7, }, // America/Dominica
    {  360,  893,   98601,  8, }, // America/Edmonton
    {  360, 5441,   98609,  8, }, // America/El_Salvador
    {  360, 7886,   98617, 11, }, // America/Fort_Nelson
    {  360, 7906,   98628,  9, }, // America/Fortaleza
    {  360, 5770,   98637,  7, }, // America/Grenada
    {  360, 5786,   98644,  9, }, // America/Guadeloupe
    {  360, 3760,   98653,  9, }, // America/Guatemala
    {  360, 4841,   98662,  7, }, // America/Guayaquil
    {  360, 6932,   98669,  5, }, // America/Guyana
    {  360,  481,      73, 13, }, // America/Indiana/Knox
    {  360, 7961,      86, 16, }, // America/Indiana/Marengo
    {  360, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  360, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  360, 8038,     139, 14, }, // America/Indiana/Vevay
    {  360, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  360, 8086,     171, 16, }, // America/Indiana/Winamac
    {  360, 2799,   98674,  7, }, // America/Jamaica
    {  360, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  360, 5932,     207, 22, }, // America/Lower_Princes
    {  360, 8168,   98681,  6, }, // America/Managua
    {  360, 5897,   98687,  7, }, // America/Marigot
    {  360, 5805,    5142,  8, }, // America/Martinique
    {  360, 8220,    7008,  6, }, // America/Merida
    {  360, 1850,   98694,  7, }, // America/Noronha
    {  360, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  360, 8329,     255, 20, }, // America/North_Dakota/Center
    {  360, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  360,  393,   61902,  3, }, // America/Nuuk
    {  360, 6173,   98701, 10, }, // America/Paramaribo
    {  360, 8404,   98711, 11, }, // America/Porto_Velho
    {  360, 8445,   98722,  5, }, // America/Recife
    {  360,  695,   98727, 10, }, // America/Rio_Branco
    {  360, 2201,   98737,  8, }, // America/Santiago
    {  360, 1878,   98745,  9, }, // America/Sao_Paulo
    {  360, 6899,     302, 16, }, // America/Scoresbysund
    {  360, 8508,     318, 14, }, // America/St_Barthelemy
    {  360, 2061,     332, 10, }, // America/St_Johns
    {  360, 5863,     342,  9, }, // America/St_Kitts
    {  360, 5880,     351,  9, }, // America/St_Lucia
    {  360,  855,     360, 10, }, // America/St_Thomas
    {  360, 5913,     370, 11, }, // America/St_Vincent
    {  360, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  360, 8568,   98754,  6, }, // Antarctica/Macquarie
    {  360, 7374,   98760,  6, }, // Antarctica/Vostok
    {  360, 1429,   98766, 11, }, // Arctic/Longyearbyen
    {  360, 5570,   98777,  5, }, // Asia/Aden
    {  360, 5395,    7476,  6, }, // Asia/Almaty
    {  360, 3968,   98782,  6, }, // Asia/Amman
    {  360, 5511,    7482,  6, }, // Asia/Anadyr
    {  360, 3607,   98788,  6, }, // Asia/Baghdad
    {  360, 5534,   98794,  6, }, // Asia/Bahrain
    {  360, 4466,   98800,  7, }, // Asia/Bangkok
    {  360, 4086,   35871,  6, }, // Asia/Beirut
    {  360, 5989,   98807,  6, }, // Asia/Brunei
    {  360, 4511,   98813,  7, }, // Asia/Colombo
    {  360, 4540,    4083,  5, }, // Asia/Damascus
    {  360, 3596,   22258,  5, }, // Asia/Dubai
    {  360, 8651,   98820,  5, }, // Asia/Gaza
    {  360, 4795,   98825,  6, }, // Asia/Hebron
    {  360, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  360, 2704,   98831,  9, }, // Asia/Hong_Kong
    {  360, 4771,   98840,  4, }, // Asia/Hovd
    {  360, 1290,   98844,  9, }, // Asia/Jerusalem
    {  360, 4239,   98853,  6, }, // Asia/Karachi
    {  360, 1185,    5593,  8, }, // Asia/Kathmandu
    {  360,  992,   98859,  7, }, // Asia/Kolkata
    {  360, 4859,   98866, 12, }, // Asia/Kuala_Lumpur
    {  360, 5547,   98878,  4, }, // Asia/Kuwait
    {  360, 1211,   98882,  5, }, // Asia/Macau
    {  360, 6920,   98887,  6, }, // Asia/Muscat
    {  360, 2554,   98893,  7, }, // Asia/Nicosia
    {  360, 4214,   98900,  9, }, // Asia/Pyongyang
    {  360, 5559,   98909,  5, }, // Asia/Qatar
    {  360, 8708,   98914,  8, }, // Asia/Qostanay
    {  360, 3584,   98922,  5, }, // Asia/Riyadh
    {  360, 3279,   98927,  5, }, // Asia/Seoul
    {  360, 1053,   98932,  6, }, // Asia/Shanghai
    {  360, 3300,   98938,  8, }, // Asia/Singapore
    {  360, 3263,   98946,  6, }, // Asia/Taipei
    {  360, 2821,   98952,  5, }, // Asia/Tokyo
    {  360, 1021,    7737,  9, }, // Asia/Ulaanbaatar
    {  360, 1235,   58486,  7, }, // Asia/Yangon
    {  360, 3747,   98957,  6, }, // Asia/Yerevan
    {  360, 3663,   98963,  6, }, // Atlantic/Azores
    {  360, 5671,   98969,  6, }, // Atlantic/Bermuda
    {  360, 6628,   98975, 17, }, // Atlantic/Canary
    {  360, 3727,   98992,  7, }, // Atlantic/Cape_Verde
    {  360, 1395,   98999,  5, }, // Atlantic/Faroe
    {  360, 8736,   99004,  6, }, // Atlantic/Madeira
    {  360, 7272,   99010, 15, }, // Atlantic/South_Georgia
    {  360, 6821,   99025, 10, }, // Atlantic/St_Helena
    {  360, 1670,   99035,  8, }, // Australia/Adelaide
    {  360, 1635,   62673,  7, }, // Australia/Brisbane
    {  360, 1727,   99043,  7, }, // Australia/Melbourne
    {  360, 1463,    5774,  5, }, // Australia/Sydney
    {  360, 6441,   99050, 10, }, // Europe/Amsterdam
    {  360, 6216,   99060,  6, }, // Europe/Andorra
    {  360, 2300,   33520,  5, }, // Europe/Athens
    {  360, 6336,   99066,  7, }, // Europe/Belgrade
    {  360, 2167,    7959,  7, }, // Europe/Brussels
    {  360, 3928,   22641,  8, }, // Europe/Bucharest
    {  360, 6289,   99073, 10, }, // Europe/Copenhagen
    {  360, 2338,   99083,  6, }, // Europe/Dublin
    {  360, 6307,   99089,  9, }, // Europe/Gibraltar
    {  360, 8788,   99098,  8, }, // Europe/Guernsey
    {  360, 6612,   99106,  8, }, // Europe/Helsinki
    {  360, 8804,   99114, 13, }, // Europe/Isle_of_Man
    {  360, 1130,   99127,  8, }, // Europe/Istanbul
    {  360, 8823,   99135,  6, }, // Europe/Jersey
    {  360, 2527,   99141,  4, }, // Europe/Kyiv
    {  360, 3213,   99145,  6, }, // Europe/Lisbon
    {  360, 2501,   99151,  6, }, // Europe/London
    {  360, 6365,    8054, 10, }, // Europe/Luxembourg
    {  360, 6582,   99157,  8, }, // Europe/Mariehamn
    {  360, 6410,   99165,  6, }, // Europe/Monaco
    {  360, 3537,   99171,  5, }, // Europe/Moscow
    {  360, 6458,   99176,  4, }, // Europe/Oslo
    {  360, 6275,    8083,  4, }, // Europe/Prague
    {  360, 6324,   99180,  3, }, // Europe/Rome
    {  360, 6470,   99183, 10, }, // Europe/San_Marino
    {  360, 6245,   99193,  8, }, // Europe/Sarajevo
    {  360, 6599,   99201,  5, }, // Europe/Sofia
    {  360, 6523,   99206,  8, }, // Europe/Stockholm
    {  360, 8867,   22793,  6, }, // Europe/Tallinn
    {  360, 6202,    8137,  5, }, // Europe/Tirane
    {  360, 6567,    5978,  7, }, // Europe/Vatican
    {  360, 6231,   99214,  6, }, // Europe/Vienna
    {  360, 8899,   99220,  7, }, // Europe/Vilnius
    {  360, 6540,   99227,  5, }, // Europe/Zurich
    {  360, 5078,   99232, 12, }, // Indian/Antananarivo
    {  360, 6947,   99244,  5, }, // Indian/Chagos
    {  360, 6046,   73966,  7, }, // Indian/Christmas
    {  360, 6063,   99249,  4, }, // Indian/Cocos
    {  360, 5029,   99253,  5, }, // Indian/Comoro
    {  360, 6661,   99258,  7, }, // Indian/Kerguelen
    {  360, 7260,   99265,  4, }, // Indian/Mahe
    {  360, 7050,    8193,  6, }, // Indian/Maldives
    {  360, 4069,   99269,  5, }, // Indian/Mauritius
    {  360, 5098,   99274,  5, }, // Indian/Mayotte
    {  360, 7211,   99279,  8, }, // Indian/Reunion
    {  360, 4422,   99287,  4, }, // Pacific/Apia
    {  360,  932,   99291,  6, }, // Pacific/Auckland
    {  360, 3015,   99297,  6, }, // Pacific/Chatham
    {  360, 3157,     474,  5, }, // Pacific/Chuuk
    {  360, 6678,   99303,  9, }, // Pacific/Galapagos
    {  360, 6696,   99312,  7, }, // Pacific/Gambier
    {  360, 2723,    4493,  8, }, // Pacific/Honolulu
    {  360, 3049,    4501,  9, }, // Pacific/Kanton
    {  360, 4051,   36208,  6, }, // Pacific/Marquesas
    {  360, 7132,   99319,  4, }, // Pacific/Niue
    {  360, 4168,   99323,  7, }, // Pacific/Norfolk
    {  360, 3096,     479,  7, }, // Pacific/Pohnpei
    {  361,   14,       0,  6, }, // Africa/Asmara Kako/Latin/Cameroon
    {  361, 4435,       6,  8, }, // Africa/Sao_Tome
    {  361, 4252,      14,  8, }, // America/Asuncion
    {  361,  237,      22,  8, }, // America/Atikokan
    {  361, 7623,      30, 17, }, // America/Bahia_Banderas
    {  361, 3868,      47,  6, }, // America/Cancun
    {  361, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  361, 5723,      66,  7, }, // America/Curacao
    {  361,  481,      73, 13, }, // America/Indiana/Knox
    {  361, 7961,      86, 16, }, // America/Indiana/Marengo
    {  361, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  361, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  361, 8038,     139, 14, }, // America/Indiana/Vevay
    {  361, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  361, 8086,     171, 16, }, // America/Indiana/Winamac
    {  361, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  361, 5932,     207, 22, }, // America/Lower_Princes
    {  361, 8220,     229,  6, }, // America/Merida
    {  361, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  361, 8329,     255, 20, }, // America/North_Dakota/Center
    {  361, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  361,  393,     298,  4, }, // America/Nuuk
    {  361, 6899,     302, 16, }, // America/Scoresbysund
    {  361, 8508,     318, 14, }, // America/St_Barthelemy
    {  361, 2061,     332, 10, }, // America/St_Johns
    {  361, 5863,     342,  9, }, // America/St_Kitts
    {  361, 5880,     351,  9, }, // America/St_Lucia
    {  361,  855,     360, 10, }, // America/St_Thomas
    {  361, 5913,     370, 11, }, // America/St_Vincent
    {  361, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  361, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  361, 1185,     413,  9, }, // Asia/Kathmandu
    {  361,  992,     422,  7, }, // Asia/Kolkata
    {  361, 1211,     429,  5, }, // Asia/Macau
    {  361, 1235,     442,  6, }, // Asia/Yangon
    {  361, 1395,     448,  5, }, // Atlantic/Faroe
    {  361, 6821,     453, 10, }, // Atlantic/St_Helena
    {  361, 2527,     463,  4, }, // Europe/Kyiv
    {  361, 7211,     467,  7, }, // Indian/Reunion
    {  361, 3157,     474,  5, }, // Pacific/Chuuk
    {  361, 3096,     479,  7, }, // Pacific/Pohnpei
    {  362,   14,       0,  6, }, // Africa/Asmara Kalaallisut/Latin/Greenland
    {  362, 4435,       6,  8, }, // Africa/Sao_Tome
    {  362, 4252,      14,  8, }, // America/Asuncion
    {  362,  237,      22,  8, }, // America/Atikokan
    {  362, 7623,      30, 17, }, // America/Bahia_Banderas
    {  362, 3868,      47,  6, }, // America/Cancun
    {  362, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  362, 5723,      66,  7, }, // America/Curacao
    {  362,  481,      73, 13, }, // America/Indiana/Knox
    {  362, 7961,      86, 16, }, // America/Indiana/Marengo
    {  362, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  362, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  362, 8038,     139, 14, }, // America/Indiana/Vevay
    {  362, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  362, 8086,     171, 16, }, // America/Indiana/Winamac
    {  362, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  362, 5932,     207, 22, }, // America/Lower_Princes
    {  362, 8220,     229,  6, }, // America/Merida
    {  362, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  362, 8329,     255, 20, }, // America/North_Dakota/Center
    {  362, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  362,  393,     298,  4, }, // America/Nuuk
    {  362, 6899,     302, 16, }, // America/Scoresbysund
    {  362, 8508,     318, 14, }, // America/St_Barthelemy
    {  362, 2061,     332, 10, }, // America/St_Johns
    {  362, 5863,     342,  9, }, // America/St_Kitts
    {  362, 5880,     351,  9, }, // America/St_Lucia
    {  362,  855,     360, 10, }, // America/St_Thomas
    {  362, 5913,     370, 11, }, // America/St_Vincent
    {  362, 5756,   26664,  7, }, // America/Thule
    {  362, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  362, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  362, 1185,     413,  9, }, // Asia/Kathmandu
    {  362,  992,     422,  7, }, // Asia/Kolkata
    {  362, 1211,     429,  5, }, // Asia/Macau
    {  362, 1235,     442,  6, }, // Asia/Yangon
    {  362, 1395,     448,  5, }, // Atlantic/Faroe
    {  362, 6821,     453, 10, }, // Atlantic/St_Helena
    {  362, 2527,     463,  4, }, // Europe/Kyiv
    {  362, 7211,     467,  7, }, // Indian/Reunion
    {  362, 3157,     474,  5, }, // Pacific/Chuuk
    {  362, 3096,     479,  7, }, // Pacific/Pohnpei
    {  363,   14,       0,  6, }, // Africa/Asmara Kalenjin/Latin/Kenya
    {  363, 4435,       6,  8, }, // Africa/Sao_Tome
    {  363, 4252,      14,  8, }, // America/Asuncion
    {  363,  237,      22,  8, }, // America/Atikokan
    {  363, 7623,      30, 17, }, // America/Bahia_Banderas
    {  363, 3868,      47,  6, }, // America/Cancun
    {  363, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  363, 5723,      66,  7, }, // America/Curacao
    {  363,  481,      73, 13, }, // America/Indiana/Knox
    {  363, 7961,      86, 16, }, // America/Indiana/Marengo
    {  363, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  363, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  363, 8038,     139, 14, }, // America/Indiana/Vevay
    {  363, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  363, 8086,     171, 16, }, // America/Indiana/Winamac
    {  363, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  363, 5932,     207, 22, }, // America/Lower_Princes
    {  363, 8220,     229,  6, }, // America/Merida
    {  363, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  363, 8329,     255, 20, }, // America/North_Dakota/Center
    {  363, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  363,  393,     298,  4, }, // America/Nuuk
    {  363, 6899,     302, 16, }, // America/Scoresbysund
    {  363, 8508,     318, 14, }, // America/St_Barthelemy
    {  363, 2061,     332, 10, }, // America/St_Johns
    {  363, 5863,     342,  9, }, // America/St_Kitts
    {  363, 5880,     351,  9, }, // America/St_Lucia
    {  363,  855,     360, 10, }, // America/St_Thomas
    {  363, 5913,     370, 11, }, // America/St_Vincent
    {  363, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  363, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  363, 1185,     413,  9, }, // Asia/Kathmandu
    {  363,  992,     422,  7, }, // Asia/Kolkata
    {  363, 1211,     429,  5, }, // Asia/Macau
    {  363, 1235,     442,  6, }, // Asia/Yangon
    {  363, 1395,     448,  5, }, // Atlantic/Faroe
    {  363, 6821,     453, 10, }, // Atlantic/St_Helena
    {  363, 2527,     463,  4, }, // Europe/Kyiv
    {  363, 7211,     467,  7, }, // Indian/Reunion
    {  363, 3157,     474,  5, }, // Pacific/Chuuk
    {  363, 3096,     479,  7, }, // Pacific/Pohnpei
    {  364,   14,       0,  6, }, // Africa/Asmara Kamba/Latin/Kenya
    {  364, 4435,       6,  8, }, // Africa/Sao_Tome
    {  364, 4252,      14,  8, }, // America/Asuncion
    {  364,  237,      22,  8, }, // America/Atikokan
    {  364, 7623,      30, 17, }, // America/Bahia_Banderas
    {  364, 3868,      47,  6, }, // America/Cancun
    {  364, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  364, 5723,      66,  7, }, // America/Curacao
    {  364,  481,      73, 13, }, // America/Indiana/Knox
    {  364, 7961,      86, 16, }, // America/Indiana/Marengo
    {  364, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  364, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  364, 8038,     139, 14, }, // America/Indiana/Vevay
    {  364, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  364, 8086,     171, 16, }, // America/Indiana/Winamac
    {  364, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  364, 5932,     207, 22, }, // America/Lower_Princes
    {  364, 8220,     229,  6, }, // America/Merida
    {  364, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  364, 8329,     255, 20, }, // America/North_Dakota/Center
    {  364, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  364,  393,     298,  4, }, // America/Nuuk
    {  364, 6899,     302, 16, }, // America/Scoresbysund
    {  364, 8508,     318, 14, }, // America/St_Barthelemy
    {  364, 2061,     332, 10, }, // America/St_Johns
    {  364, 5863,     342,  9, }, // America/St_Kitts
    {  364, 5880,     351,  9, }, // America/St_Lucia
    {  364,  855,     360, 10, }, // America/St_Thomas
    {  364, 5913,     370, 11, }, // America/St_Vincent
    {  364, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  364, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  364, 1185,     413,  9, }, // Asia/Kathmandu
    {  364,  992,     422,  7, }, // Asia/Kolkata
    {  364, 1211,     429,  5, }, // Asia/Macau
    {  364, 1235,     442,  6, }, // Asia/Yangon
    {  364, 1395,     448,  5, }, // Atlantic/Faroe
    {  364, 6821,     453, 10, }, // Atlantic/St_Helena
    {  364, 2527,     463,  4, }, // Europe/Kyiv
    {  364, 7211,     467,  7, }, // Indian/Reunion
    {  364, 3157,     474,  5, }, // Pacific/Chuuk
    {  364, 3096,     479,  7, }, // Pacific/Pohnpei
    {  365, 6788,   99330,  9, }, // Africa/Abidjan Kannada/Kannada/India
    {  365, 6760,   99339,  5, }, // Africa/Accra
    {  365, 5059,   99344, 11, }, // Africa/Addis_Ababa
    {  365, 7420,   99355,  9, }, // Africa/Algiers
    {  365,   14,   99364,  7, }, // Africa/Asmara
    {  365,   44,   99371,  5, }, // Africa/Bamako
    {  365, 5257,   99376,  5, }, // Africa/Bangui
    {  365, 6746,   99381,  6, }, // Africa/Banjul
    {  365, 7435,   99387,  8, }, // Africa/Bissau
    {  365, 4957,   99395, 11, }, // Africa/Blantyre
    {  365, 5287,   99406, 10, }, // Africa/Brazzaville
    {  365, 4922,   99416,  9, }, // Africa/Bujumbura
    {  365, 2320,   99425,  4, }, // Africa/Cairo
    {  365, 4117,   99429, 13, }, // Africa/Casablanca
    {  365, 7449,   99442,  6, }, // Africa/Ceuta
    {  365, 6773,   99448,  8, }, // Africa/Conakry
    {  365, 6840,   99456,  5, }, // Africa/Dakar
    {  365, 5130,   99461, 14, }, // Africa/Dar_es_Salaam
    {  365, 5043,   99475,  6, }, // Africa/Djibouti
    {  365, 5243,   99481,  4, }, // Africa/Douala
    {  365, 5166,   99485,  7, }, // Africa/El_Aaiun
    {  365, 6853,   99492,  8, }, // Africa/Freetown
    {  365, 4906,   99500,  9, }, // Africa/Gaborone
    {  365, 5015,   99509,  5, }, // Africa/Harare
    {  365, 4479,   99514, 14, }, // Africa/Johannesburg
    {  365, 4499,   99528,  4, }, // Africa/Juba
    {  365, 5151,   99532,  6, }, // Africa/Kampala
    {  365, 4524,   99538,  7, }, // Africa/Khartoum
    {  365, 4973,   99545,  6, }, // Africa/Kigali
    {  365, 5306,   99551,  8, }, // Africa/Kinshasa
    {  365, 4744,   99559,  6, }, // Africa/Lagos
    {  365, 5336,   99565, 10, }, // Africa/Libreville
    {  365, 6869,   99575,  4, }, // Africa/Lome
    {  365, 5211,   99579,  7, }, // Africa/Luanda
    {  365, 4939,   99586,  9, }, // Africa/Lubumbashi
    {  365, 5001,   99595,  6, }, // Africa/Lusaka
    {  365, 5322,   99601,  5, }, // Africa/Malabo
    {  365, 4987,   99606,  8, }, // Africa/Maputo
    {  365, 5197,   99614,  5, }, // Africa/Maseru
    {  365, 5182,   99619,  7, }, // Africa/Mbabane
    {  365, 5113,   99626,  8, }, // Africa/Mogadishu
    {  365, 7034,   99634, 10, }, // Africa/Monrovia
    {  365, 3853,   99644,  6, }, // Africa/Nairobi
    {  365, 5271,   99650,  7, }, // Africa/Ndjamena
    {  365, 5354,   99657,  6, }, // Africa/Niamey
    {  365, 6803,   99663, 11, }, // Africa/Nouakchott
    {  365, 6727,   99674,  7, }, // Africa/Ouagadougou
    {  365, 5225,   99681, 11, }, // Africa/Porto-Novo
    {  365, 4435,   99692,  9, }, // Africa/Sao_Tome
    {  365, 2866,   99701,  8, }, // Africa/Tripoli
    {  365, 6554,   99709,  6, }, // Africa/Tunis
    {  365, 4152,   99715,  9, }, // Africa/Windhoek
    {  365,  132,   99724,  6, }, // America/Adak
    {  365, 3346,   99730,  7, }, // America/Anchorage
    {  365, 5607,   99737, 10, }, // America/Anguilla
    {  365, 5624,   99747,  8, }, // America/Antigua
    {  365, 4554,   99755,  9, }, // America/Araguaina
    {  365,  166,   99764, 13, }, // America/Argentina/Buenos_Aires
    {  365,   91,   99777,  7, }, // America/Argentina/Catamarca
    {  365,  270,   99784,  8, }, // America/Argentina/Cordoba
    {  365,  441,   99792,  4, }, // America/Argentina/Jujuy
    {  365, 7462,   99796,  9, }, // America/Argentina/La_Rioja
    {  365,  565,   99805,  7, }, // America/Argentina/Mendoza
    {  365, 7489,   99812, 14, }, // America/Argentina/Rio_Gallegos
    {  365, 7520,   99826,  6, }, // America/Argentina/Salta
    {  365, 7544,   99832, 13, }, // America/Argentina/San_Juan
    {  365, 5580,   99845, 13, }, // America/Argentina/San_Luis
    {  365, 7571,   99858,  8, }, // America/Argentina/Tucuman
    {  365, 7597,   99866,  6, }, // America/Argentina/Ushuaia
    {  365, 5640,   99872,  5, }, // America/Aruba
    {  365, 4252,   99877, 11, }, // America/Asuncion
    {  365,  237,   99888,  9, }, // America/Atikokan
    {  365, 3679,   99897,  5, }, // America/Bahia
    {  365, 7623,   99902, 15, }, // America/Bahia_Banderas
    {  365, 5654,   99917,  9, }, // America/Barbados
    {  365, 7646,   99926,  5, }, // America/Belem
    {  365, 5407,   99931,  6, }, // America/Belize
    {  365, 7660,   99937, 15, }, // America/Blanc-Sablon
    {  365, 7681,   99952, 10, }, // America/Boa_Vista
    {  365, 4361,   99962,  5, }, // America/Bogota
    {  365, 7699,   99967,  6, }, // America/Boise
    {  365, 7713,   99973, 16, }, // America/Cambridge_Bay
    {  365, 7735,   99989, 13, }, // America/Campo_Grande
    {  365, 3868,  100002,  8, }, // America/Cancun
    {  365, 4694,  100010,  9, }, // America/Caracas
    {  365, 4345,  100019,  6, }, // America/Cayenne
    {  365, 5496,  100025,  5, }, // America/Cayman
    {  365, 2260,  100030,  6, }, // America/Chicago
    {  365, 7756,  100036,  5, }, // America/Chihuahua
    {  365, 7774,  100041, 15, }, // America/Ciudad_Juarez
    {  365, 5422,  100056, 11, }, // America/Costa_Rica
    {  365, 7796,  100067,  9, }, // America/Creston
    {  365, 3791,  100076,  7, }, // America/Cuiaba
    {  365, 5723,  100083,  8, }, // America/Curacao
    {  365, 7812,  100091, 17, }, // America/Danmarkshavn
    {  365, 7833,  100108,  5, }, // America/Dawson
    {  365, 7848,  100113, 12, }, // America/Dawson_Creek
    {  365,  805,  100125,  7, }, // America/Denver
    {  365, 3465,  100132, 10, }, // America/Detroit
    {  365, 5739,  100142,  8, }, // America/Dominica
    {  365,  893,  100150,  9, }, // America/Edmonton
    {  365, 7869,  100159,  7, }, // America/Eirunepe
    {  365, 5441,  100166, 13, }, // America/El_Salvador
    {  365, 7886,  100179, 14, }, // America/Fort_Nelson
    {  365, 7906,  100193,  9, }, // America/Fortaleza
    {  365, 7924,  100202,  9, }, // America/Glace_Bay
    {  365, 6881,  100211,  7, }, // America/Goose_Bay
    {  365, 4612,  100218, 15, }, // America/Grand_Turk
    {  365, 5770,  100233,  7, }, // America/Grenada
    {  365, 5786,  100240, 10, }, // America/Guadeloupe
    {  365, 3760,  100250, 10, }, // America/Guatemala
    {  365, 4841,  100260, 10, }, // America/Guayaquil
    {  365, 6932,  100270,  5, }, // America/Guyana
    {  365, 1939,  100275, 12, }, // America/Halifax
    {  365, 2281,  100287,  5, }, // America/Havana
    {  365, 7942,  100292, 11, }, // America/Hermosillo
    {  365,  348,  100303, 14, }, // America/Indiana/Indianapolis
    {  365,  481,  100317, 16, }, // America/Indiana/Knox
    {  365, 7961,  100333, 16, }, // America/Indiana/Marengo
    {  365, 7985,  100349, 23, }, // America/Indiana/Petersburg
    {  365, 8012,  100372, 19, }, // America/Indiana/Tell_City
    {  365, 8038,  100391, 16, }, // America/Indiana/Vevay
    {  365, 8060,  100407, 23, }, // America/Indiana/Vincennes
    {  365, 8086,  100430, 20, }, // America/Indiana/Winamac
    {  365, 8110,  100450,  7, }, // America/Inuvik
    {  365,  660,  100457,  9, }, // America/Iqaluit
    {  365, 2799,  100466,  5, }, // America/Jamaica
    {  365, 5380,  100471,  6, }, // America/Juneau
    {  365,  521,  100477, 13, }, // America/Kentucky/Louisville
    {  365, 8125,  100490, 20, }, // America/Kentucky/Monticello
    {  365, 5704,  100510, 13, }, // America/Kralendijk
    {  365, 4376,  100523,  7, }, // America/La_Paz
    {  365, 7169,  100530,  4, }, // America/Lima
    {  365, 3239,  100534, 12, }, // America/Los_Angeles
    {  365, 5932,  100546, 25, }, // America/Lower_Princes
    {  365, 8153,  100571,  6, }, // America/Maceio
    {  365, 8168,  100577,  7, }, // America/Managua
    {  365, 1908,  100584,  7, }, // America/Manaus
    {  365, 5897,  100591,  5, }, // America/Marigot
    {  365, 5805,  100596, 10, }, // America/Martinique
    {  365, 8184,  100606,  8, }, // America/Matamoros
    {  365, 2917,  100614,  8, }, // America/Mazatlan
    {  365, 8202,  100622,  8, }, // America/Menominee
    {  365, 8220,  100630,  6, }, // America/Merida
    {  365, 8235,  100636, 12, }, // America/Metlakatla
    {  365, 2949,  100648, 12, }, // America/Mexico_City
    {  365, 4391,  100660,  8, }, // America/Miquelon
    {  365, 8254,  100668,  8, }, // America/Moncton
    {  365, 8270,  100676,  9, }, // America/Monterrey
    {  365, 4098,  100685, 11, }, // America/Montevideo
    {  365, 5824,  100696, 12, }, // America/Montserrat
    {  365, 5481,  100708,  7, }, // America/Nassau
    {  365, 2379,  100715, 10, }, // America/New_York
    {  365, 8288,  100725,  4, }, // America/Nome
    {  365, 1850,  100729,  8, }, // America/Noronha
    {  365, 8301,  100737, 18, }, // America/North_Dakota/Beulah
    {  365, 8329,  100755, 18, }, // America/North_Dakota/Center
    {  365, 8357,  100773, 21, }, // America/North_Dakota/New_Salem
    {  365,  393,  100794,  4, }, // America/Nuuk
    {  365, 8388,  100798,  7, }, // America/Ojinaga
    {  365, 2356,  100805,  5, }, // America/Panama
    {  365, 6173,  100810,  8, }, // America/Paramaribo
    {  365, 2973,  100818,  8, }, // America/Phoenix
    {  365, 3945,  100826, 17, }, // America/Port-au-Prince
    {  365, 5954,  100843, 17, }, // America/Port_of_Spain
    {  365, 8404,  100860, 13, }, // America/Porto_Velho
    {  365, 5843,  100873, 13, }, // America/Puerto_Rico
    {  365, 4030,  100886, 13, }, // America/Punta_Arenas
    {  365, 8424,  100899, 16, }, // America/Rankin_Inlet
    {  365, 8445,  100915,  6, }, // America/Recife
    {  365, 1995,  100921,  6, }, // America/Regina
    {  365, 8460,  100927, 10, }, // America/Resolute
    {  365,  695,  100937, 14, }, // America/Rio_Branco
    {  365, 8477,  100951,  8, }, // America/Santarem
    {  365, 2201,  100959, 11, }, // America/Santiago
    {  365, 6111,  100970, 15, }, // America/Santo_Domingo
    {  365, 1878,  100985,  9, }, // America/Sao_Paulo
    {  365, 6899,  100994, 19, }, // America/Scoresbysund
    {  365, 8494,  101013,  6, }, // America/Sitka
    {  365, 8508,  101019, 16, }, // America/St_Barthelemy
    {  365, 2061,  101035, 12, }, // America/St_Johns
    {  365, 5863,  101047, 12, }, // America/St_Kitts
    {  365, 5880,  101059, 12, }, // America/St_Lucia
    {  365,  855,  101071, 11, }, // America/St_Thomas
    {  365, 5913,  101082, 15, }, // America/St_Vincent
    {  365, 8530,  101097, 15, }, // America/Swift_Current
    {  365, 5461,  101112, 11, }, // America/Tegucigalpa
    {  365, 5756,  101123,  4, }, // America/Thule
    {  365,  313,  101127,  8, }, // America/Tijuana
    {  365,  608,  101135,  7, }, // America/Toronto
    {  365, 5688,  101142,  8, }, // America/Tortola
    {  365, 2093,  101150, 11, }, // America/Vancouver
    {  365, 2144,  101161, 11, }, // America/Whitehorse
    {  365,  734,  101172, 10, }, // America/Winnipeg
    {  365, 8552,  101182,  8, }, // America/Yakutat
    {  365, 6001,  101190,  4, }, // Antarctica/Casey
    {  365, 6094,  101194,  6, }, // Antarctica/Davis
    {  365, 6133,  101200, 17, }, // Antarctica/DumontDUrville
    {  365, 8568,  101217, 10, }, // Antarctica/Macquarie
    {  365, 7066,  101227,  7, }, // Antarctica/Mawson
    {  365, 7113,  101234, 13, }, // Antarctica/McMurdo
    {  365, 8589,  101247,  7, }, // Antarctica/Palmer
    {  365, 7226,  101254,  6, }, // Antarctica/Rothera
    {  365, 7295,  101260,  6, }, // Antarctica/Syowa
    {  365, 8607,  101266,  6, }, // Antarctica/Troll
    {  365, 7374,  101272,  8, }, // Antarctica/Vostok
    {  365, 1429,  101280, 13, }, // Arctic/Longyearbyen
    {  365, 5570,  101293,  4, }, // Asia/Aden
    {  365, 5395,  101297,  7, }, // Asia/Almaty
    {  365, 3968,  101304,  7, }, // Asia/Amman
    {  365, 5511,  101311,  7, }, // Asia/Anadyr
    {  365, 5523,  101318,  7, }, // Asia/Aqtau
    {  365, 5368,  101325,  7, }, // Asia/Aqtobe
    {  365,  964,  101332,  8, }, // Asia/Ashgabat
    {  365, 8624,  101340,  7, }, // Asia/Atyrau
    {  365, 3607,  101347,  8, }, // Asia/Baghdad
    {  365, 5534,  101355,  7, }, // Asia/Bahrain
    {  365, 3653,  101362,  3, }, // Asia/Baku
    {  365, 4466,  101365,  9, }, // Asia/Bangkok
    {  365, 3571,  101374,  8, }, // Asia/Barnaul
    {  365, 4086,  101382,  5, }, // Asia/Beirut
    {  365, 3778,  101387,  8, }, // Asia/Bishkek
    {  365, 5989,  101395,  6, }, // Asia/Brunei
    {  365, 4601,  101401,  3, }, // Asia/Chita
    {  365, 4511,  101404,  6, }, // Asia/Colombo
    {  365, 4540,  101410,  8, }, // Asia/Damascus
    {  365, 1093,  101418,  4, }, // Asia/Dhaka
    {  365, 6192,  101422,  4, }, // Asia/Dili
    {  365, 3596,  101426,  4, }, // Asia/Dubai
    {  365, 6159,  101430,  7, }, // Asia/Dushanbe
    {  365, 8636,  101437,  9, }, // Asia/Famagusta
    {  365, 8651,  101446,  4, }, // Asia/Gaza
    {  365, 4795,  101450,  8, }, // Asia/Hebron
    {  365, 1259,  101458, 15, }, // Asia/Ho_Chi_Minh
    {  365, 2704,  101473, 11, }, // Asia/Hong_Kong
    {  365, 4771,  101484,  6, }, // Asia/Hovd
    {  365, 4184,  101490, 10, }, // Asia/Irkutsk
    {  365, 7006,  101500,  7, }, // Asia/Jakarta
    {  365, 6992,  101507,  5, }, // Asia/Jayapura
    {  365, 1290,  101512,  9, }, // Asia/Jerusalem
    {  365, 3560,  101521,  6, }, // Asia/Kabul
    {  365, 4316,  101527,  7, }, // Asia/Kamchatka
    {  365, 4239,  101534,  5, }, // Asia/Karachi
    {  365, 1185,  101539,  7, }, // Asia/Kathmandu
    {  365, 8661,  101546,  8, }, // Asia/Khandyga
    {  365,  992,  101554,  9, }, // Asia/Kolkata
    {  365, 4197,  101563, 16, }, // Asia/Krasnoyarsk
    {  365, 4859,  101579, 10, }, // Asia/Kuala_Lumpur
    {  365, 5976,  101589,  7, }, // Asia/Kuching
    {  365, 5547,  101596,  6, }, // Asia/Kuwait
    {  365, 1211,  101602,  5, }, // Asia/Macau
    {  365, 4017,  101607,  6, }, // Asia/Magadan
    {  365, 1349,  101613,  9, }, // Asia/Makassar
    {  365, 7182,  101622,  5, }, // Asia/Manila
    {  365, 6920,  101627,  6, }, // Asia/Muscat
    {  365, 2554,  101633,  8, }, // Asia/Nicosia
    {  365, 8675,  101641,  8, }, // Asia/Novokuznetsk
    {  365, 4135,  101649, 12, }, // Asia/Novosibirsk
    {  365, 4229,  101661,  7, }, // Asia/Omsk
    {  365, 7145,  101668,  4, }, // Asia/Oral
    {  365, 6961,  101672,  9, }, // Asia/Phnom_Penh
    {  365, 8693,  101681, 10, }, // Asia/Pontianak
    {  365, 4214,  101691, 13, }, // Asia/Pyongyang
    {  365, 5559,  101704,  5, }, // Asia/Qatar
    {  365, 8708,  101709,  8, }, // Asia/Qostanay
    {  365, 4269,  101717,  9, }, // Asia/Qyzylorda
    {  365, 3584,  101726,  6, }, // Asia/Riyadh
    {  365, 4408,  101732,  7, }, // Asia/Sakhalin
    {  365, 7245,  101739,  6, }, // Asia/Samarkand
    {  365, 3279,  101745,  6, }, // Asia/Seoul
    {  365, 1053,  101751,  5, }, // Asia/Shanghai
    {  365, 3300,  101756,  8, }, // Asia/Singapore
    {  365, 4297,  101764, 16, }, // Asia/Srednekolymsk
    {  365, 3263,  101780,  4, }, // Asia/Taipei
    {  365, 4781,  101784,  9, }, // Asia/Tashkent
    {  365, 3915,  101793,  8, }, // Asia/Tbilisi
    {  365, 2772,  101801,  8, }, // Asia/Tehran
    {  365, 1317,  101809,  5, }, // Asia/Thimphu
    {  365, 2821,  101814,  6, }, // Asia/Tokyo
    {  365, 4572,  101820,  9, }, // Asia/Tomsk
    {  365, 1021,  101829, 11, }, // Asia/Ulaanbaatar
    {  365, 1159,  101840,  8, }, // Asia/Urumqi
    {  365, 8722,  101848, 10, }, // Asia/Ust-Nera
    {  365, 6977,  101858, 11, }, // Asia/Vientiane
    {  365, 4710,  101869, 15, }, // Asia/Vladivostok
    {  365, 4828,  101884,  9, }, // Asia/Yakutsk
    {  365, 1235,  101893,  7, }, // Asia/Yangon
    {  365, 3883,  101900, 16, }, // Asia/Yekaterinburg
    {  365, 3747,  101916,  8, }, // Asia/Yerevan
    {  365, 3663,  101924,  7, }, // Atlantic/Azores
    {  365, 5671,  101931,  7, }, // Atlantic/Bermuda
    {  365, 6628,  101938,  7, }, // Atlantic/Canary
    {  365, 3727,  101945, 11, }, // Atlantic/Cape_Verde
    {  365, 1395,  101956,  6, }, // Atlantic/Faroe
    {  365, 8736,  101962,  6, }, // Atlantic/Madeira
    {  365, 2748,  101968, 10, }, // Atlantic/Reykjavik
    {  365, 7272,  101978, 15, }, // Atlantic/South_Georgia
    {  365, 6821,  101993, 12, }, // Atlantic/St_Helena
    {  365, 6644,  102005, 10, }, // Atlantic/Stanley
    {  365, 1670,  102015,  7, }, // Australia/Adelaide
    {  365, 1635,  102022, 11, }, // Australia/Brisbane
    {  365, 1799,  102033, 12, }, // Australia/Broken_Hill
    {  365, 1583,  102045,  8, }, // Australia/Darwin
    {  365, 3637,  102053,  6, }, // Australia/Eucla
    {  365, 1516,  102059,  9, }, // Australia/Hobart
    {  365, 8753,  102068,  9, }, // Australia/Lindeman
    {  365, 1547,  102077, 11, }, // Australia/Lord_Howe
    {  365, 1727,  102088,  9, }, // Australia/Melbourne
    {  365, 1762,  102097,  5, }, // Australia/Perth
    {  365, 1463,  102102,  6, }, // Australia/Sydney
    {  365, 6441,  102108, 16, }, // Europe/Amsterdam
    {  365, 6216,  102124,  6, }, // Europe/Andorra
    {  365, 3620,  102130, 11, }, // Europe/Astrakhan
    {  365, 2300,  102141,  7, }, // Europe/Athens
    {  365, 6336,  102148, 11, }, // Europe/Belgrade
    {  365, 4757,  102159,  7, }, // Europe/Berlin
    {  365, 6488,  102166, 11, }, // Europe/Bratislava
    {  365, 2167,  102177, 11, }, // Europe/Brussels
    {  365, 3928,  102188, 10, }, // Europe/Bucharest
    {  365, 3806,  102198, 10, }, // Europe/Budapest
    {  365, 8772,  102208,  9, }, // Europe/Busingen
    {  365, 2583,  102217,  8, }, // Europe/Chisinau
    {  365, 6289,  102225, 12, }, // Europe/Copenhagen
    {  365, 2338,  102237,  8, }, // Europe/Dublin
    {  365, 6307,  102245, 11, }, // Europe/Gibraltar
    {  365, 8788,  102256, 11, }, // Europe/Guernsey
    {  365, 6612,  102267,  9, }, // Europe/Helsinki
    {  365, 8804,  102276, 14, }, // Europe/Isle_of_Man
    {  365, 1130,  102290, 12, }, // Europe/Istanbul
    {  365, 8823,  102302,  6, }, // Europe/Jersey
    {  365, 3979,  102308, 14, }, // Europe/Kaliningrad
    {  365, 8929,  102322,  6, }, // Europe/Kirov
    {  365, 2527,  102328,  4, }, // Europe/Kyiv
    {  365, 3213,  102332,  7, }, // Europe/Lisbon
    {  365, 6506,  102339, 12, }, // Europe/Ljubljana
    {  365, 2501,  102351,  5, }, // Europe/London
    {  365, 6365,  102356, 11, }, // Europe/Luxembourg
    {  365, 4892,  102367, 10, }, // Europe/Madrid
    {  365, 6397,  102377,  6, }, // Europe/Malta
    {  365, 6582,  102383,  8, }, // Europe/Mariehamn
    {  365, 3693,  102391,  9, }, // Europe/Minsk
    {  365, 6410,  102400,  6, }, // Europe/Monaco
    {  365, 3537,  102406,  6, }, // Europe/Moscow
    {  365, 6458,  102412,  5, }, // Europe/Oslo
    {  365, 4284,  102417,  8, }, // Europe/Paris
    {  365, 6424,  102425,  9, }, // Europe/Podgorica
    {  365, 6275,  102434,  8, }, // Europe/Prague
    {  365, 8837,  102442,  4, }, // Europe/Riga
    {  365, 6324,  102446,  4, }, // Europe/Rome
    {  365, 4331,  101739,  3, }, // Europe/Samara
    {  365, 6470,  102450, 12, }, // Europe/San_Marino
    {  365, 6245,  102462,  6, }, // Europe/Sarajevo
    {  365, 4451,  102468,  7, }, // Europe/Saratov
    {  365, 8849,  102475, 12, }, // Europe/Simferopol
    {  365, 6383,  102487,  8, }, // Europe/Skopje
    {  365, 6599,  102495,  6, }, // Europe/Sofia
    {  365, 6523,  102501, 11, }, // Europe/Stockholm
    {  365, 8867,  102512,  8, }, // Europe/Tallinn
    {  365, 6202,  102520,  6, }, // Europe/Tirane
    {  365, 8882,  102526, 12, }, // Europe/Ulyanovsk
    {  365, 6352,  102538,  5, }, // Europe/Vaduz
    {  365, 6567,  102543,  9, }, // Europe/Vatican
    {  365, 6231,  102552,  8, }, // Europe/Vienna
    {  365, 8899,  102560,  9, }, // Europe/Vilnius
    {  365, 4727,  102569, 12, }, // Europe/Volgograd
    {  365, 3190,  102581,  6, }, // Europe/Warsaw
    {  365, 6261,  102587,  9, }, // Europe/Zagreb
    {  365, 6540,  102596,  8, }, // Europe/Zurich
    {  365, 5078,  102604, 11, }, // Indian/Antananarivo
    {  365, 6947,  102615,  6, }, // Indian/Chagos
    {  365, 6046,  102621,  9, }, // Indian/Christmas
    {  365, 6063,  102630,  6, }, // Indian/Cocos
    {  365, 5029,  102636,  6, }, // Indian/Comoro
    {  365, 6661,  102642, 12, }, // Indian/Kerguelen
    {  365, 7260,  102654,  4, }, // Indian/Mahe
    {  365, 7050,  102658, 10, }, // Indian/Maldives
    {  365, 4069,  102668,  7, }, // Indian/Mauritius
    {  365, 5098,  102675,  7, }, // Indian/Mayotte
    {  365, 7211,  102682,  9, }, // Indian/Reunion
    {  365, 4422,  102691,  5, }, // Pacific/Apia
    {  365,  932,  102696, 11, }, // Pacific/Auckland
    {  365, 3706,  102707, 13, }, // Pacific/Bougainville
    {  365, 3015,  102720,  5, }, // Pacific/Chatham
    {  365, 3157,  102725,  4, }, // Pacific/Chuuk
    {  365, 2237,  102729,  6, }, // Pacific/Easter
    {  365, 7360,  102735,  5, }, // Pacific/Efate
    {  365, 7327,  102740,  5, }, // Pacific/Fakaofo
    {  365, 3902,  102745,  4, }, // Pacific/Fiji
    {  365, 7343,  102749,  8, }, // Pacific/Funafuti
    {  365, 6678,  102757,  8, }, // Pacific/Galapagos
    {  365, 6696,  102765, 10, }, // Pacific/Gambier
    {  365, 3822,  102775, 13, }, // Pacific/Guadalcanal
    {  365, 6018,  102788,  6, }, // Pacific/Guam
    {  365, 2723,  102794,  7, }, // Pacific/Honolulu
    {  365, 3049,  102801,  8, }, // Pacific/Kanton
    {  365, 3998,  102809,  9, }, // Pacific/Kiritimati
    {  365, 7019,  102818,  6, }, // Pacific/Kosrae
    {  365, 2842,  102824,  9, }, // Pacific/Kwajalein
    {  365, 4877,  102833,  5, }, // Pacific/Majuro
    {  365, 4051,  102838, 11, }, // Pacific/Marquesas
    {  365, 8914,  102849,  7, }, // Pacific/Midway
    {  365, 7084,  102856,  4, }, // Pacific/Nauru
    {  365, 7132,  102860,  4, }, // Pacific/Niue
    {  365, 4168,  102864,  8, }, // Pacific/Norfolk
    {  365, 7098,  102872,  8, }, // Pacific/Noumea
    {  365, 3126,  102880,  9, }, // Pacific/Pago_Pago
    {  365, 7155,  102889,  6, }, // Pacific/Palau
    {  365, 7194,  102895, 11, }, // Pacific/Pitcairn
    {  365, 3096,  102906,  7, }, // Pacific/Pohnpei
    {  365, 4807,  102913, 16, }, // Pacific/Port_Moresby
    {  365, 6076,  102929,  8, }, // Pacific/Rarotonga
    {  365, 6031,  102937,  5, }, // Pacific/Saipan
    {  365, 7312,  102942,  5, }, // Pacific/Tahiti
    {  365, 6712,  102947,  5, }, // Pacific/Tarawa
    {  365, 4583,  102952,  9, }, // Pacific/Tongatapu
    {  365, 7392,  102961,  4, }, // Pacific/Wake
    {  365, 7405,  102965,  8, }, // Pacific/Wallis
    {  366, 6788,  102973,  8, }, // Africa/Abidjan Kashmiri/Arabic/India
    {  366, 6760,  102981,  5, }, // Africa/Accra
    {  366, 5059,  102986, 10, }, // Africa/Addis_Ababa
    {  366, 7420,  102996, 10, }, // Africa/Algiers
    {  366,   14,  103006,  7, }, // Africa/Asmara
    {  366,   44,  103013,  5, }, // Africa/Bamako
    {  366, 5257,  103018,  8, }, // Africa/Bangui
    {  366, 6746,  103026,  7, }, // Africa/Banjul
    {  366, 7435,  103033,  6, }, // Africa/Bissau
    {  366, 4957,  103039,  8, }, // Africa/Blantyre
    {  366, 5287,  103047,  8, }, // Africa/Brazzaville
    {  366, 4922,  103055,  9, }, // Africa/Bujumbura
    {  366, 2320,  103064,  5, }, // Africa/Cairo
    {  366, 4117,  103069, 10, }, // Africa/Casablanca
    {  366, 7449,  103079,  6, }, // Africa/Ceuta
    {  366, 6773,  103085,  7, }, // Africa/Conakry
    {  366, 6840,  103092,  5, }, // Africa/Dakar
    {  366, 5130,  103097,  9, }, // Africa/Dar_es_Salaam
    {  366, 5043,  103106,  8, }, // Africa/Djibouti
    {  366, 5243,  103114,  5, }, // Africa/Douala
    {  366, 5166,  103119,  7, }, // Africa/El_Aaiun
    {  366, 6853,  103126,  9, }, // Africa/Freetown
    {  366, 4906,  103135,  7, }, // Africa/Gaborone
    {  366, 5015,  103142,  7, }, // Africa/Harare
    {  366, 4479,  103149,  9, }, // Africa/Johannesburg
    {  366, 4499,   10555,  4, }, // Africa/Juba
    {  366, 5151,  103158,  7, }, // Africa/Kampala
    {  366, 4524,  103165,  7, }, // Africa/Khartoum
    {  366, 4973,  103172,  6, }, // Africa/Kigali
    {  366, 5306,  103178,  7, }, // Africa/Kinshasa
    {  366, 4744,  103185,  5, }, // Africa/Lagos
    {  366, 5336,  103190,  8, }, // Africa/Libreville
    {  366, 6869,   10598,  3, }, // Africa/Lome
    {  366, 5211,  103198,  7, }, // Africa/Luanda
    {  366, 4939,  103205,  9, }, // Africa/Lubumbashi
    {  366, 5001,  103214,  6, }, // Africa/Lusaka
    {  366, 5322,   10622,  6, }, // Africa/Malabo
    {  366, 4987,  103220,  7, }, // Africa/Maputo
    {  366, 5197,  103227,  7, }, // Africa/Maseru
    {  366, 5182,  103234,  5, }, // Africa/Mbabane
    {  366, 5113,  103239,  9, }, // Africa/Mogadishu
    {  366, 7034,  103248,  9, }, // Africa/Monrovia
    {  366, 3853,  103257,  6, }, // Africa/Nairobi
    {  366, 5271,  103263,  7, }, // Africa/Ndjamena
    {  366, 5354,  103270,  6, }, // Africa/Niamey
    {  366, 6803,  103276,  9, }, // Africa/Nouakchott
    {  366, 6727,  103285,  8, }, // Africa/Ouagadougou
    {  366, 5225,  103293, 10, }, // Africa/Porto-Novo
    {  366, 4435,  103303,  7, }, // Africa/Sao_Tome
    {  366, 2866,  103310,  6, }, // Africa/Tripoli
    {  366, 6554,  103316,  6, }, // Africa/Tunis
    {  366, 4152,  103322,  7, }, // Africa/Windhoek
    {  366,  132,  103329,  3, }, // America/Adak
    {  366, 3346,  103332,  8, }, // America/Anchorage
    {  366, 5607,  103340,  8, }, // America/Anguilla
    {  366, 5624,  103348,  9, }, // America/Antigua
    {  366, 4554,  103357, 10, }, // America/Araguaina
    {  366,  166,  103367, 13, }, // America/Argentina/Buenos_Aires
    {  366,   91,  103380,  9, }, // America/Argentina/Catamarca
    {  366,  270,  103389,  7, }, // America/Argentina/Cordoba
    {  366,  441,  103396,  6, }, // America/Argentina/Jujuy
    {  366, 7462,  103402,  9, }, // America/Argentina/La_Rioja
    {  366,  565,  103411,  7, }, // America/Argentina/Mendoza
    {  366, 7489,  103418, 12, }, // America/Argentina/Rio_Gallegos
    {  366, 7520,  103430,  5, }, // America/Argentina/Salta
    {  366, 7544,  103435,  9, }, // America/Argentina/San_Juan
    {  366, 5580,  103444, 10, }, // America/Argentina/San_Luis
    {  366, 7571,  103454,  8, }, // America/Argentina/Tucuman
    {  366, 7597,  103462, 10, }, // America/Argentina/Ushuaia
    {  366, 5640,  103472,  7, }, // America/Aruba
    {  366, 4252,  103479, 10, }, // America/Asuncion
    {  366,  237,  103489,  9, }, // America/Atikokan
    {  366, 3679,  103498,  6, }, // America/Bahia
    {  366, 7623,  103504, 12, }, // America/Bahia_Banderas
    {  366, 5654,  103516,  8, }, // America/Barbados
    {  366, 7646,  103524,  5, }, // America/Belem
    {  366, 5407,  103529,  6, }, // America/Belize
    {  366, 7660,  103535, 12, }, // America/Blanc-Sablon
    {  366, 7681,  103547,  9, }, // America/Boa_Vista
    {  366, 4361,  103556,  6, }, // America/Bogota
    {  366, 7699,  103562,  5, }, // America/Boise
    {  366, 7713,  103567, 13, }, // America/Cambridge_Bay
    {  366, 7735,  103580, 11, }, // America/Campo_Grande
    {  366, 3868,  103591,  6, }, // America/Cancun
    {  366, 4694,  103597,  7, }, // America/Caracas
    {  366, 4345,  103604,  5, }, // America/Cayenne
    {  366, 5496,  103609,  5, }, // America/Cayman
    {  366, 2260,  103614,  6, }, // America/Chicago
    {  366, 7756,  103620, 11, }, // America/Chihuahua
    {  366, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  366, 5422,  103631, 11, }, // America/Costa_Rica
    {  366, 7796,  103642,  6, }, // America/Creston
    {  366, 3791,  103648,  7, }, // America/Cuiaba
    {  366, 5723,  103655, 10, }, // America/Curacao
    {  366, 7812,  103665, 11, }, // America/Danmarkshavn
    {  366, 7833,  103676,  5, }, // America/Dawson
    {  366, 7848,  103681, 12, }, // America/Dawson_Creek
    {  366,  805,  103693,  6, }, // America/Denver
    {  366, 3465,  103699,  8, }, // America/Detroit
    {  366, 5739,  103707,  8, }, // America/Dominica
    {  366,  893,  103715,  9, }, // America/Edmonton
    {  366, 7869,  103724,  9, }, // America/Eirunepe
    {  366, 5441,  103733, 12, }, // America/El_Salvador
    {  366, 7886,  103745, 10, }, // America/Fort_Nelson
    {  366, 7906,  103755,  9, }, // America/Fortaleza
    {  366, 7924,  103764, 10, }, // America/Glace_Bay
    {  366, 6881,  103774, 10, }, // America/Goose_Bay
    {  366, 4612,  103784, 10, }, // America/Grand_Turk
    {  366, 5770,  103794,  7, }, // America/Grenada
    {  366, 5786,  103801,  6, }, // America/Guadeloupe
    {  366, 3760,  103807,  9, }, // America/Guatemala
    {  366, 4841,  103816, 10, }, // America/Guayaquil
    {  366, 6932,  103826,  6, }, // America/Guyana
    {  366, 1939,  103832,  8, }, // America/Halifax
    {  366, 2281,  103840,  5, }, // America/Havana
    {  366, 7942,  103845,  8, }, // America/Hermosillo
    {  366,  348,  103853, 13, }, // America/Indiana/Indianapolis
    {  366,  481,  103866,  4, }, // America/Indiana/Knox
    {  366, 7961,  103870, 15, }, // America/Indiana/Marengo
    {  366, 7985,  103885, 18, }, // America/Indiana/Petersburg
    {  366, 8012,  103903, 17, }, // America/Indiana/Tell_City
    {  366, 8038,  103920, 14, }, // America/Indiana/Vevay
    {  366, 8060,  103934, 17, }, // America/Indiana/Vincennes
    {  366, 8086,  103951, 16, }, // America/Indiana/Winamac
    {  366, 8110,  103967,  8, }, // America/Inuvik
    {  366,  660,  103975, 11, }, // America/Iqaluit
    {  366, 2799,  103986,  7, }, // America/Jamaica
    {  366, 5380,  103993,  5, }, // America/Juneau
    {  366,  521,  103998, 10, }, // America/Kentucky/Louisville
    {  366, 8125,  104008, 18, }, // America/Kentucky/Monticello
    {  366, 5704,  104026,  9, }, // America/Kralendijk
    {  366, 4376,  104035,  6, }, // America/La_Paz
    {  366, 7169,  104041,  4, }, // America/Lima
    {  366, 3239,  104045, 11, }, // America/Los_Angeles
    {  366, 5932,  104056, 16, }, // America/Lower_Princes
    {  366, 8153,  104072,  7, }, // America/Maceio
    {  366, 8168,  104079,  7, }, // America/Managua
    {  366, 1908,  104086,  5, }, // America/Manaus
    {  366, 5897,  104091,  7, }, // America/Marigot
    {  366, 5805,  104098,  8, }, // America/Martinique
    {  366, 8184,  104106,  9, }, // America/Matamoros
    {  366, 2917,  104115,  8, }, // America/Mazatlan
    {  366, 8202,  104123,  8, }, // America/Menominee
    {  366, 8220,  104131,  6, }, // America/Merida
    {  366, 8235,  104137, 11, }, // America/Metlakatla
    {  366, 2949,  104148, 12, }, // America/Mexico_City
    {  366, 4391,  104160,  8, }, // America/Miquelon
    {  366, 8254,  104168,  7, }, // America/Moncton
    {  366, 8270,  104175,  6, }, // America/Monterrey
    {  366, 4098,  104181, 10, }, // America/Montevideo
    {  366, 5824,  104191,  8, }, // America/Montserrat
    {  366, 5481,  104199,  5, }, // America/Nassau
    {  366, 2379,  104204,  8, }, // America/New_York
    {  366, 8288,   11422,  3, }, // America/Nome
    {  366, 1850,  104212,  7, }, // America/Noronha
    {  366, 8301,  104219, 21, }, // America/North_Dakota/Beulah
    {  366, 8329,  104240, 19, }, // America/North_Dakota/Center
    {  366, 8357,  104259, 24, }, // America/North_Dakota/New_Salem
    {  366,  393,  104283,  4, }, // America/Nuuk
    {  366, 8388,  104287,  9, }, // America/Ojinaga
    {  366, 2356,  104296,  6, }, // America/Panama
    {  366, 6173,  104302, 10, }, // America/Paramaribo
    {  366, 2973,  104312,  7, }, // America/Phoenix
    {  366, 3945,  104319, 13, }, // America/Port-au-Prince
    {  366, 5954,  104332, 11, }, // America/Port_of_Spain
    {  366, 8404,  104343, 10, }, // America/Porto_Velho
    {  366, 5843,  104353, 10, }, // America/Puerto_Rico
    {  366, 4030,  104363, 12, }, // America/Punta_Arenas
    {  366, 8424,  104375, 13, }, // America/Rankin_Inlet
    {  366, 8445,  104388,  5, }, // America/Recife
    {  366, 1995,  104393,  7, }, // America/Regina
    {  366, 8460,  104400,  9, }, // America/Resolute
    {  366,  695,  104409, 11, }, // America/Rio_Branco
    {  366, 8477,  104420,  8, }, // America/Santarem
    {  366, 2201,  104428,  8, }, // America/Santiago
    {  366, 6111,  104436, 13, }, // America/Santo_Domingo
    {  366, 1878,  104449,  8, }, // America/Sao_Paulo
    {  366, 6899,  104457, 13, }, // America/Scoresbysund
    {  366, 8494,  104470,  5, }, // America/Sitka
    {  366, 8508,  104475, 14, }, // America/St_Barthelemy
    {  366, 2061,  104489,  9, }, // America/St_Johns
    {  366, 5863,  104498,  9, }, // America/St_Kitts
    {  366, 5880,  104507, 12, }, // America/St_Lucia
    {  366,  855,  104519, 11, }, // America/St_Thomas
    {  366, 5913,  104530, 12, }, // America/St_Vincent
    {  366, 8530,  104542, 13, }, // America/Swift_Current
    {  366, 5461,  104555, 12, }, // America/Tegucigalpa
    {  366, 5756,  104567,  7, }, // America/Thule
    {  366,  313,  104574,  8, }, // America/Tijuana
    {  366,  608,  104582,  7, }, // America/Toronto
    {  366, 5688,  104589,  7, }, // America/Tortola
    {  366, 2093,  104596,  8, }, // America/Vancouver
    {  366, 2144,  104604, 11, }, // America/Whitehorse
    {  366,  734,  104615,  7, }, // America/Winnipeg
    {  366, 8552,  104622,  7, }, // America/Yakutat
    {  366, 6001,  104629,  4, }, // Antarctica/Casey
    {  366, 6094,  104633,  5, }, // Antarctica/Davis
    {  366, 6133,  104638, 15, }, // Antarctica/DumontDUrville
    {  366, 7066,  104653,  5, }, // Antarctica/Mawson
    {  366, 7113,  104658,  8, }, // Antarctica/McMurdo
    {  366, 8589,  104666,  5, }, // Antarctica/Palmer
    {  366, 7226,  104671,  7, }, // Antarctica/Rothera
    {  366, 7295,  104678,  5, }, // Antarctica/Syowa
    {  366, 7374,  104683,  6, }, // Antarctica/Vostok
    {  366, 1429,  104689, 14, }, // Arctic/Longyearbyen
    {  366, 5570,  104703,  5, }, // Asia/Aden
    {  366, 5395,  104708,  7, }, // Asia/Almaty
    {  366, 3968,  104715,  5, }, // Asia/Amman
    {  366, 5511,  104720,  7, }, // Asia/Anadyr
    {  366, 5523,  104727,  4, }, // Asia/Aqtau
    {  366, 5368,  104731,  6, }, // Asia/Aqtobe
    {  366,  964,  104737,  8, }, // Asia/Ashgabat
    {  366, 8624,  104745,  5, }, // Asia/Atyrau
    {  366, 3607,   11991,  5, }, // Asia/Baghdad
    {  366, 5534,  104750,  7, }, // Asia/Bahrain
    {  366, 3653,  104757,  5, }, // Asia/Baku
    {  366, 4466,  104762,  8, }, // Asia/Bangkok
    {  366, 3571,  104770,  5, }, // Asia/Barnaul
    {  366, 4086,  104775,  4, }, // Asia/Beirut
    {  366, 3778,  104779,  6, }, // Asia/Bishkek
    {  366, 5989,  104785,  7, }, // Asia/Brunei
    {  366, 4601,  104792,  4, }, // Asia/Chita
    {  366, 4511,  104796,  7, }, // Asia/Colombo
    {  366, 4540,  104803,  8, }, // Asia/Damascus
    {  366, 1093,  104811,  5, }, // Asia/Dhaka
    {  366, 6192,  104816,  4, }, // Asia/Dili
    {  366, 3596,  104820,  5, }, // Asia/Dubai
    {  366, 6159,  104825,  8, }, // Asia/Dushanbe
    {  366, 8636,  104833, 10, }, // Asia/Famagusta
    {  366, 8651,  104843,  3, }, // Asia/Gaza
    {  366, 4795,  104846,  6, }, // Asia/Hebron
    {  366, 1259,  104852,  5, }, // Asia/Ho_Chi_Minh
    {  366, 2704,  104857,  9, }, // Asia/Hong_Kong
    {  366, 4771,  104866,  4, }, // Asia/Hovd
    {  366, 4184,  104870,  7, }, // Asia/Irkutsk
    {  366, 7006,  104877,  6, }, // Asia/Jakarta
    {  366, 6992,  104883,  9, }, // Asia/Jayapura
    {  366, 1290,  104892,  6, }, // Asia/Jerusalem
    {  366, 3560,  104898,  5, }, // Asia/Kabul
    {  366, 4316,  104903,  8, }, // Asia/Kamchatka
    {  366, 4239,  104911,  6, }, // Asia/Karachi
    {  366, 1185,  104917, 10, }, // Asia/Kathmandu
    {  366, 8661,  104927,  8, }, // Asia/Khandyga
    {  366,  992,  104935,  8, }, // Asia/Kolkata
    {  366, 4197,  104943, 11, }, // Asia/Krasnoyarsk
    {  366, 4859,  104954, 11, }, // Asia/Kuala_Lumpur
    {  366, 5976,  104965,  6, }, // Asia/Kuching
    {  366, 5547,  104971,  5, }, // Asia/Kuwait
    {  366, 1211,  104976,  3, }, // Asia/Macau
    {  366, 4017,  104979,  7, }, // Asia/Magadan
    {  366, 1349,  104986,  7, }, // Asia/Makassar
    {  366, 7182,  104993,  7, }, // Asia/Manila
    {  366, 6920,   12250,  4, }, // Asia/Muscat
    {  366, 2554,  105000,  8, }, // Asia/Nicosia
    {  366, 8675,  105008, 12, }, // Asia/Novokuznetsk
    {  366, 4135,  105020, 11, }, // Asia/Novosibirsk
    {  366, 4229,  105031,  5, }, // Asia/Omsk
    {  366, 7145,  105036,  5, }, // Asia/Oral
    {  366, 6961,  105041,  9, }, // Asia/Phnom_Penh
    {  366, 8693,  105050,  9, }, // Asia/Pontianak
    {  366, 4214,  105059,  9, }, // Asia/Pyongyang
    {  366, 5559,   12318,  3, }, // Asia/Qatar
    {  366, 8708,  105068,  7, }, // Asia/Qostanay
    {  366, 4269,  105075,  8, }, // Asia/Qyzylorda
    {  366, 3584,  105083,  4, }, // Asia/Riyadh
    {  366, 4408,  105087,  8, }, // Asia/Sakhalin
    {  366, 7245,  105095,  9, }, // Asia/Samarkand
    {  366, 3279,  105104,  4, }, // Asia/Seoul
    {  366, 1053,  105108,  5, }, // Asia/Shanghai
    {  366, 3300,  105113,  8, }, // Asia/Singapore
    {  366, 4297,  105121, 13, }, // Asia/Srednekolymsk
    {  366, 3263,  105134,  6, }, // Asia/Taipei
    {  366, 4781,  105140,  7, }, // Asia/Tashkent
    {  366, 3915,  105147,  6, }, // Asia/Tbilisi
    {  366, 2772,  105153,  6, }, // Asia/Tehran
    {  366, 1317,  105159,  7, }, // Asia/Thimphu
    {  366, 2821,  105166,  5, }, // Asia/Tokyo
    {  366, 4572,  105171,  5, }, // Asia/Tomsk
    {  366, 1021,  105176,  9, }, // Asia/Ulaanbaatar
    {  366, 1159,  105185,  7, }, // Asia/Urumqi
    {  366, 8722,  105192,  9, }, // Asia/Ust-Nera
    {  366, 6977,  105201, 10, }, // Asia/Vientiane
    {  366, 4710,  105211, 10, }, // Asia/Vladivostok
    {  366, 4828,  105221,  5, }, // Asia/Yakutsk
    {  366, 1235,  105226,  6, }, // Asia/Yangon
    {  366, 3883,  105232, 13, }, // Asia/Yekaterinburg
    {  366, 3747,  105245,  7, }, // Asia/Yerevan
    {  366, 3663,  105252,  7, }, // Atlantic/Azores
    {  366, 5671,  105259,  7, }, // Atlantic/Bermuda
    {  366, 6628,  105266,  6, }, // Atlantic/Canary
    {  366, 3727,  105272,  8, }, // Atlantic/Cape_Verde
    {  366, 1395,  105280,  4, }, // Atlantic/Faroe
    {  366, 8736,  105284,  7, }, // Atlantic/Madeira
    {  366, 2748,  105291,  8, }, // Atlantic/Reykjavik
    {  366, 7272,  105299, 12, }, // Atlantic/South_Georgia
    {  366, 6821,  105311, 11, }, // Atlantic/St_Helena
    {  366, 6644,  105322,  6, }, // Atlantic/Stanley
    {  366, 1670,  105328,  7, }, // Australia/Adelaide
    {  366, 1635,  105335,  6, }, // Australia/Brisbane
    {  366, 1799,  105341, 10, }, // Australia/Broken_Hill
    {  366, 1583,  105351,  6, }, // Australia/Darwin
    {  366, 3637,  105357,  6, }, // Australia/Eucla
    {  366, 1516,  105363,  5, }, // Australia/Hobart
    {  366, 8753,  105368,  8, }, // Australia/Lindeman
    {  366, 1547,  105376,  9, }, // Australia/Lord_Howe
    {  366, 1727,  105385,  8, }, // Australia/Melbourne
    {  366, 1762,  105393,  5, }, // Australia/Perth
    {  366, 1463,  105398,  5, }, // Australia/Sydney
    {  366, 6441,  105403, 10, }, // Europe/Amsterdam
    {  366, 6216,  105413,  7, }, // Europe/Andorra
    {  366, 3620,  105420, 11, }, // Europe/Astrakhan
    {  366, 2300,  105431,  7, }, // Europe/Athens
    {  366, 6336,  105438,  7, }, // Europe/Belgrade
    {  366, 4757,  105445,  6, }, // Europe/Berlin
    {  366, 6488,  105451, 10, }, // Europe/Bratislava
    {  366, 2167,  105461,  6, }, // Europe/Brussels
    {  366, 3928,  105467,  8, }, // Europe/Bucharest
    {  366, 3806,  105475,  8, }, // Europe/Budapest
    {  366, 8772,  105483,  8, }, // Europe/Busingen
    {  366, 2583,  105491,  7, }, // Europe/Chisinau
    {  366, 6289,  105498,  9, }, // Europe/Copenhagen
    {  366, 2338,  105507,  6, }, // Europe/Dublin
    {  366, 6307,  105513,  9, }, // Europe/Gibraltar
    {  366, 8788,  105522,  7, }, // Europe/Guernsey
    {  366, 6612,  105529,  8, }, // Europe/Helsinki
    {  366, 8804,  105537, 10, }, // Europe/Isle_of_Man
    {  366, 1130,  105547,  9, }, // Europe/Istanbul
    {  366, 8823,  105556,  4, }, // Europe/Jersey
    {  366, 3979,  105560, 12, }, // Europe/Kaliningrad
    {  366, 8929,  105572,  4, }, // Europe/Kirov
    {  366, 2527,  105576,  4, }, // Europe/Kyiv
    {  366, 3213,  105580,  6, }, // Europe/Lisbon
    {  366, 6506,  105586, 13, }, // Europe/Ljubljana
    {  366, 2501,  105599,  6, }, // Europe/London
    {  366, 6365,  105605, 10, }, // Europe/Luxembourg
    {  366, 4892,  105615,  6, }, // Europe/Madrid
    {  366, 6397,  105621,  5, }, // Europe/Malta
    {  366, 6582,  105626,  8, }, // Europe/Mariehamn
    {  366, 3693,  105634,  5, }, // Europe/Minsk
    {  366, 6410,  105639,  6, }, // Europe/Monaco
    {  366, 3537,  105645,  5, }, // Europe/Moscow
    {  366, 6458,  105650,  5, }, // Europe/Oslo
    {  366, 4284,  105655,  5, }, // Europe/Paris
    {  366, 6424,  105660,  9, }, // Europe/Podgorica
    {  366, 6275,  105669,  4, }, // Europe/Prague
    {  366, 8837,  105673,  4, }, // Europe/Riga
    {  366, 6324,   10638,  3, }, // Europe/Rome
    {  366, 4331,  105677,  6, }, // Europe/Samara
    {  366, 6470,  105683,  9, }, // Europe/San_Marino
    {  366, 6245,  105692,  9, }, // Europe/Sarajevo
    {  366, 4451,  105701,  6, }, // Europe/Saratov
    {  366, 8849,  105707, 10, }, // Europe/Simferopol
    {  366, 6383,  105717,  7, }, // Europe/Skopje
    {  366, 6599,  105724,  6, }, // Europe/Sofia
    {  366, 6523,  105730,  9, }, // Europe/Stockholm
    {  366, 8867,  105739,  5, }, // Europe/Tallinn
    {  366, 6202,  105744,  5, }, // Europe/Tirane
    {  366, 8882,  105749, 10, }, // Europe/Ulyanovsk
    {  366, 6352,  105759,  5, }, // Europe/Vaduz
    {  366, 6567,  105764,  6, }, // Europe/Vatican
    {  366, 6231,  105770,  7, }, // Europe/Vienna
    {  366, 8899,  105777,  9, }, // Europe/Vilnius
    {  366, 4727,  105786,  9, }, // Europe/Volgograd
    {  366, 3190,  105795,  5, }, // Europe/Warsaw
    {  366, 6261,  105800,  5, }, // Europe/Zagreb
    {  366, 6540,  105805,  7, }, // Europe/Zurich
    {  366, 5078,  105812, 11, }, // Indian/Antananarivo
    {  366, 6947,  105823,  5, }, // Indian/Chagos
    {  366, 6046,  105828,  7, }, // Indian/Christmas
    {  366, 6063,  105835,  5, }, // Indian/Cocos
    {  366, 5029,  105840,  6, }, // Indian/Comoro
    {  366, 6661,  105846,  8, }, // Indian/Kerguelen
    {  366, 7260,  105854,  5, }, // Indian/Mahe
    {  366, 7050,  105859,  8, }, // Indian/Maldives
    {  366, 4069,  105867,  7, }, // Indian/Mauritius
    {  366, 5098,  105874,  4, }, // Indian/Mayotte
    {  366, 7211,  105878,  9, }, // Indian/Reunion
    {  366, 4422,  105887,  5, }, // Pacific/Apia
    {  366,  932,  105892,  6, }, // Pacific/Auckland
    {  366, 3015,  105898,  6, }, // Pacific/Chatham
    {  366, 3157,  105904,  3, }, // Pacific/Chuuk
    {  366, 2237,  105907,  7, }, // Pacific/Easter
    {  366, 7360,  105914,  7, }, // Pacific/Efate
    {  366, 7327,  105921,  7, }, // Pacific/Fakaofo
    {  366, 3902,  105928,  4, }, // Pacific/Fiji
    {  366, 7343,  105932,  9, }, // Pacific/Funafuti
    {  366, 6678,  105941,  8, }, // Pacific/Galapagos
    {  366, 6696,  105949,  8, }, // Pacific/Gambier
    {  366, 3822,  105957, 12, }, // Pacific/Guadalcanal
    {  366, 6018,  104083,  4, }, // Pacific/Guam
    {  366, 2723,  105969,  9, }, // Pacific/Honolulu
    {  366, 3049,  105978,  9, }, // Pacific/Kanton
    {  366, 3998,  105987, 10, }, // Pacific/Kiritimati
    {  366, 7019,  105997,  6, }, // Pacific/Kosrae
    {  366, 2842,  106003,  7, }, // Pacific/Kwajalein
    {  366, 4877,  106010,  7, }, // Pacific/Majuro
    {  366, 4051,  106017,  8, }, // Pacific/Marquesas
    {  366, 8914,  106025,  6, }, // Pacific/Midway
    {  366, 7084,  106031,  6, }, // Pacific/Nauru
    {  366, 7132,  104204,  3, }, // Pacific/Niue
    {  366, 4168,  106037,  6, }, // Pacific/Norfolk
    {  366, 7098,  106043,  5, }, // Pacific/Noumea
    {  366, 3126,  106048,  9, }, // Pacific/Pago_Pago
    {  366, 7155,  106057,  5, }, // Pacific/Palau
    {  366, 7194,  106062,  8, }, // Pacific/Pitcairn
    {  366, 3096,  106070,  6, }, // Pacific/Pohnpei
    {  366, 4807,  106076, 10, }, // Pacific/Port_Moresby
    {  366, 6076,  106086,  9, }, // Pacific/Rarotonga
    {  366, 6031,  106095,  6, }, // Pacific/Saipan
    {  366, 7312,  106101,  6, }, // Pacific/Tahiti
    {  366, 6712,  106107,  6, }, // Pacific/Tarawa
    {  366, 4583,  106113, 10, }, // Pacific/Tongatapu
    {  366, 7392,  106123,  3, }, // Pacific/Wake
    {  366, 7405,  106126,  5, }, // Pacific/Wallis
    {  367,   14,       0,  6, }, // Africa/Asmara Kashmiri/Devanagari/India
    {  367, 4435,       6,  8, }, // Africa/Sao_Tome
    {  367, 4252,      14,  8, }, // America/Asuncion
    {  367,  237,      22,  8, }, // America/Atikokan
    {  367, 7623,      30, 17, }, // America/Bahia_Banderas
    {  367, 3868,      47,  6, }, // America/Cancun
    {  367, 5723,      66,  7, }, // America/Curacao
    {  367,  481,      73, 13, }, // America/Indiana/Knox
    {  367, 7961,      86, 16, }, // America/Indiana/Marengo
    {  367, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  367, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  367, 8038,     139, 14, }, // America/Indiana/Vevay
    {  367, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  367, 8086,     171, 16, }, // America/Indiana/Winamac
    {  367, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  367, 5932,     207, 22, }, // America/Lower_Princes
    {  367, 8220,     229,  6, }, // America/Merida
    {  367, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  367, 8329,     255, 20, }, // America/North_Dakota/Center
    {  367, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  367,  393,     298,  4, }, // America/Nuuk
    {  367, 6899,     302, 16, }, // America/Scoresbysund
    {  367, 8508,     318, 14, }, // America/St_Barthelemy
    {  367, 2061,     332, 10, }, // America/St_Johns
    {  367, 5863,     342,  9, }, // America/St_Kitts
    {  367, 5880,     351,  9, }, // America/St_Lucia
    {  367,  855,     360, 10, }, // America/St_Thomas
    {  367, 5913,     370, 11, }, // America/St_Vincent
    {  367, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  367, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  367, 1185,     413,  9, }, // Asia/Kathmandu
    {  367,  992,     422,  7, }, // Asia/Kolkata
    {  367, 1211,     429,  5, }, // Asia/Macau
    {  367, 1235,     442,  6, }, // Asia/Yangon
    {  367, 1395,     448,  5, }, // Atlantic/Faroe
    {  367, 6821,     453, 10, }, // Atlantic/St_Helena
    {  367, 2527,     463,  4, }, // Europe/Kyiv
    {  367, 7211,     467,  7, }, // Indian/Reunion
    {  367, 3157,     474,  5, }, // Pacific/Chuuk
    {  367, 3096,     479,  7, }, // Pacific/Pohnpei
    {  368, 6788,   36214,  7, }, // Africa/Abidjan Kazakh/Cyrillic/Kazakhstan
    {  368, 6760,     492,  5, }, // Africa/Accra
    {  368, 5059,     497, 11, }, // Africa/Addis_Ababa
    {  368, 7420,     508,  5, }, // Africa/Algiers
    {  368,   14,   27065,  6, }, // Africa/Asmara
    {  368,   44,     519,  6, }, // Africa/Bamako
    {  368, 5257,     525,  5, }, // Africa/Bangui
    {  368, 6746,     530,  6, }, // Africa/Banjul
    {  368, 7435,     536,  5, }, // Africa/Bissau
    {  368, 4957,   27088,  8, }, // Africa/Blantyre
    {  368, 5287,   51422, 10, }, // Africa/Brazzaville
    {  368, 4922,     558,  9, }, // Africa/Bujumbura
    {  368, 2320,     567,  4, }, // Africa/Cairo
    {  368, 4117,     571, 10, }, // Africa/Casablanca
    {  368, 7449,     581,  5, }, // Africa/Ceuta
    {  368, 6773,     586,  7, }, // Africa/Conakry
    {  368, 6840,     593,  5, }, // Africa/Dakar
    {  368, 5130,   27116, 12, }, // Africa/Dar_es_Salaam
    {  368, 5043,   36247,  7, }, // Africa/Djibouti
    {  368, 5243,     616,  5, }, // Africa/Douala
    {  368, 5166,   27135,  7, }, // Africa/El_Aaiun
    {  368, 6853,     628,  7, }, // Africa/Freetown
    {  368, 4906,     635,  8, }, // Africa/Gaborone
    {  368, 5015,   33781,  6, }, // Africa/Harare
    {  368, 4479,   51440, 12, }, // Africa/Johannesburg
    {  368, 4499,   27173,  5, }, // Africa/Juba
    {  368, 5151,     665,  7, }, // Africa/Kampala
    {  368, 4524,     672,  6, }, // Africa/Khartoum
    {  368, 4973,     678,  6, }, // Africa/Kigali
    {  368, 5306,     684,  7, }, // Africa/Kinshasa
    {  368, 4744,     691,  5, }, // Africa/Lagos
    {  368, 5336,   51452,  9, }, // Africa/Libreville
    {  368, 6869,     704,  4, }, // Africa/Lome
    {  368, 5211,     708,  6, }, // Africa/Luanda
    {  368, 4939,     714,  9, }, // Africa/Lubumbashi
    {  368, 5001,     723,  6, }, // Africa/Lusaka
    {  368, 5322,     729,  6, }, // Africa/Malabo
    {  368, 4987,     735,  6, }, // Africa/Maputo
    {  368, 5197,     741,  6, }, // Africa/Maseru
    {  368, 5182,     747,  7, }, // Africa/Mbabane
    {  368, 5113,     754,  8, }, // Africa/Mogadishu
    {  368, 7034,   36287,  8, }, // Africa/Monrovia
    {  368, 3853,   36295,  7, }, // Africa/Nairobi
    {  368, 5271,   27254,  8, }, // Africa/Ndjamena
    {  368, 5354,   36302,  6, }, // Africa/Niamey
    {  368, 6803,     790,  7, }, // Africa/Nouakchott
    {  368, 6727,     797,  8, }, // Africa/Ouagadougou
    {  368, 5225,     805, 10, }, // Africa/Porto-Novo
    {  368, 4435,     815,  8, }, // Africa/Sao_Tome
    {  368, 2866,     823,  7, }, // Africa/Tripoli
    {  368, 6554,     830,  5, }, // Africa/Tunis
    {  368, 4152,     835,  7, }, // Africa/Windhoek
    {  368,  132,     842,  4, }, // America/Adak
    {  368, 3346,   51461,  8, }, // America/Anchorage
    {  368, 5607,   51469,  7, }, // America/Anguilla
    {  368, 5624,     860,  7, }, // America/Antigua
    {  368, 4554,     867,  9, }, // America/Araguaina
    {  368,  166,   51476, 12, }, // America/Argentina/Buenos_Aires
    {  368,   91,     888,  9, }, // America/Argentina/Catamarca
    {  368,  270,     897,  7, }, // America/Argentina/Cordoba
    {  368,  441,   27355,  5, }, // America/Argentina/Jujuy
    {  368, 7462,     909,  8, }, // America/Argentina/La_Rioja
    {  368,  565,     917,  7, }, // America/Argentina/Mendoza
    {  368, 7489,   51488, 12, }, // America/Argentina/Rio_Gallegos
    {  368, 7520,   27380,  6, }, // America/Argentina/Salta
    {  368, 7544,     940,  8, }, // America/Argentina/San_Juan
    {  368, 5580,     948,  8, }, // America/Argentina/San_Luis
    {  368, 7571,     956,  7, }, // America/Argentina/Tucuman
    {  368, 7597,  106131,  6, }, // America/Argentina/Ushuaia
    {  368, 5640,     969,  5, }, // America/Aruba
    {  368, 4252,   51500,  8, }, // America/Asuncion
    {  368,  237,   33971,  8, }, // America/Atikokan
    {  368, 3679,   36368,  4, }, // America/Bahia
    {  368, 7623,   51508, 16, }, // America/Bahia_Banderas
    {  368, 5654,    1013,  8, }, // America/Barbados
    {  368, 7646,    1021,  5, }, // America/Belem
    {  368, 5407,    1026,  5, }, // America/Belize
    {  368, 7660,    1031, 12, }, // America/Blanc-Sablon
    {  368, 7681,    1043,  9, }, // America/Boa_Vista
    {  368, 4361,    1052,  6, }, // America/Bogota
    {  368, 7699,   51524,  5, }, // America/Boise
    {  368, 7713,  106137, 12, }, // America/Cambridge_Bay
    {  368, 7735,    1075, 12, }, // America/Campo_Grande
    {  368, 3868,    1087,  6, }, // America/Cancun
    {  368, 4694,    1093,  7, }, // America/Caracas
    {  368, 4345,   51542,  7, }, // America/Cayenne
    {  368, 5496,  106149, 15, }, // America/Cayman
    {  368, 2260,    1129,  6, }, // America/Chicago
    {  368, 7756,    1135,  6, }, // America/Chihuahua
    {  368, 7774,   56160, 13, }, // America/Ciudad_Juarez
    {  368, 5422,    1141, 10, }, // America/Costa_Rica
    {  368, 7796,    1151,  7, }, // America/Creston
    {  368, 3791,   27556,  5, }, // America/Cuiaba
    {  368, 5723,   36474,  7, }, // America/Curacao
    {  368, 7812,   36481, 12, }, // America/Danmarkshavn
    {  368, 7833,    1184,  6, }, // America/Dawson
    {  368, 7848,  106164, 10, }, // America/Dawson_Creek
    {  368,  805,    1201,  6, }, // America/Denver
    {  368, 3465,   36517,  7, }, // America/Detroit
    {  368, 5739,    1214,  8, }, // America/Dominica
    {  368,  893,   51549,  8, }, // America/Edmonton
    {  368, 7869,   51557,  8, }, // America/Eirunepe
    {  368, 5441,   27633,  9, }, // America/El_Salvador
    {  368, 7886,  106174, 12, }, // America/Fort_Nelson
    {  368, 7906,    1257,  9, }, // America/Fortaleza
    {  368, 7924,   51577,  9, }, // America/Glace_Bay
    {  368, 6881,   51586,  7, }, // America/Goose_Bay
    {  368, 4612,    1282, 10, }, // America/Grand_Turk
    {  368, 5770,    1292,  7, }, // America/Grenada
    {  368, 5786,    1299,  9, }, // America/Guadeloupe
    {  368, 3760,    1308,  9, }, // America/Guatemala
    {  368, 4841,   51593,  8, }, // America/Guayaquil
    {  368, 6932,   51601,  6, }, // America/Guyana
    {  368, 1939,   51607,  8, }, // America/Halifax
    {  368, 2281,    1339,  6, }, // America/Havana
    {  368, 7942,   51615,  9, }, // America/Hermosillo
    {  368,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  368,  481,    1365, 13, }, // America/Indiana/Knox
    {  368, 7961,    1378, 16, }, // America/Indiana/Marengo
    {  368, 7985,    1394, 19, }, // America/Indiana/Petersburg
    {  368, 8012,   56173, 18, }, // America/Indiana/Tell_City
    {  368, 8038,  106186, 14, }, // America/Indiana/Vevay
    {  368, 8060,   56205, 18, }, // America/Indiana/Vincennes
    {  368, 8086,   56223, 16, }, // America/Indiana/Winamac
    {  368, 8110,    1452,  6, }, // America/Inuvik
    {  368,  660,    1458,  7, }, // America/Iqaluit
    {  368, 2799,   27879,  6, }, // America/Jamaica
    {  368, 5380,   36614,  5, }, // America/Juneau
    {  368,  521,    1476,  8, }, // America/Kentucky/Louisville
    {  368, 8125,    1484, 20, }, // America/Kentucky/Monticello
    {  368, 5704,   36619, 10, }, // America/Kralendijk
    {  368, 4376,    1514,  6, }, // America/La_Paz
    {  368, 7169,    1520,  4, }, // America/Lima
    {  368, 3239,   51629, 12, }, // America/Los_Angeles
    {  368, 5932,   56239, 18, }, // America/Lower_Princes
    {  368, 8153,   36667,  6, }, // America/Maceio
    {  368, 8168,    1559,  7, }, // America/Managua
    {  368, 1908,    1566,  6, }, // America/Manaus
    {  368, 5897,    1572,  6, }, // America/Marigot
    {  368, 5805,    1578,  9, }, // America/Martinique
    {  368, 8184,    1587,  9, }, // America/Matamoros
    {  368, 2917,    1596,  8, }, // America/Mazatlan
    {  368, 8202,    1604,  8, }, // America/Menominee
    {  368, 8220,    1612,  6, }, // America/Merida
    {  368, 8235,    1618, 10, }, // America/Metlakatla
    {  368, 2949,    1628,  6, }, // America/Mexico_City
    {  368, 4391,    1634,  7, }, // America/Miquelon
    {  368, 8254,    1641,  7, }, // America/Moncton
    {  368, 8270,   51662,  9, }, // America/Monterrey
    {  368, 4098,    1657, 10, }, // America/Montevideo
    {  368, 5824,    1667, 10, }, // America/Montserrat
    {  368, 5481,    1677,  6, }, // America/Nassau
    {  368, 2379,   51680,  8, }, // America/New_York
    {  368, 8288,    1691,  3, }, // America/Nome
    {  368, 1850,   56257,  7, }, // America/Noronha
    {  368, 8301,  106200, 23, }, // America/North_Dakota/Beulah
    {  368, 8329,  106223, 24, }, // America/North_Dakota/Center
    {  368, 8357,  106247, 28, }, // America/North_Dakota/New_Salem
    {  368,  393,    1772,  4, }, // America/Nuuk
    {  368, 8388,    1776,  7, }, // America/Ojinaga
    {  368, 2356,    1783,  6, }, // America/Panama
    {  368, 6173,    1789, 10, }, // America/Paramaribo
    {  368, 2973,    1799,  6, }, // America/Phoenix
    {  368, 3945,    1805, 12, }, // America/Port-au-Prince
    {  368, 5954,   28189, 13, }, // America/Port_of_Spain
    {  368, 8404,   28202, 11, }, // America/Porto_Velho
    {  368, 5843,   51756, 11, }, // America/Puerto_Rico
    {  368, 4030,    1852, 12, }, // America/Punta_Arenas
    {  368, 8424,    1864, 12, }, // America/Rankin_Inlet
    {  368, 8445,    1876,  6, }, // America/Recife
    {  368, 1995,   51767,  8, }, // America/Regina
    {  368, 8460,   56329,  8, }, // America/Resolute
    {  368,  695,    1897, 10, }, // America/Rio_Branco
    {  368, 8477,    1907,  8, }, // America/Santarem
    {  368, 2201,   51775,  8, }, // America/Santiago
    {  368, 6111,    1923, 13, }, // America/Santo_Domingo
    {  368, 1878,    1936,  9, }, // America/Sao_Paulo
    {  368, 6899,  106275, 16, }, // America/Scoresbysund
    {  368, 8494,    1956,  5, }, // America/Sitka
    {  368, 8508,    1961, 13, }, // America/St_Barthelemy
    {  368, 2061,   28349, 10, }, // America/St_Johns
    {  368, 5863,    1983,  9, }, // America/St_Kitts
    {  368, 5880,   56356, 10, }, // America/St_Lucia
    {  368,  855,    2003, 10, }, // America/St_Thomas
    {  368, 5913,    2013, 12, }, // America/St_Vincent
    {  368, 8530,  106291, 13, }, // America/Swift_Current
    {  368, 5461,   51817, 12, }, // America/Tegucigalpa
    {  368, 5756,    2049,  4, }, // America/Thule
    {  368,  313,    2053,  7, }, // America/Tijuana
    {  368,  608,    2060,  7, }, // America/Toronto
    {  368, 5688,    2067,  7, }, // America/Tortola
    {  368, 2093,    2074,  8, }, // America/Vancouver
    {  368, 2144,   28435,  8, }, // America/Whitehorse
    {  368,  734,    2090,  8, }, // America/Winnipeg
    {  368, 8552,   28450,  6, }, // America/Yakutat
    {  368, 6001,   36889,  5, }, // Antarctica/Casey
    {  368, 6094,  106304,  6, }, // Antarctica/Davis
    {  368, 6133,   51829, 14, }, // Antarctica/DumontDUrville
    {  368, 8568,    2131,  8, }, // Antarctica/Macquarie
    {  368, 7066,    2139,  6, }, // Antarctica/Mawson
    {  368, 7113,    2145,  9, }, // Antarctica/McMurdo
    {  368, 8589,    2154,  6, }, // Antarctica/Palmer
    {  368, 7226,    2160,  6, }, // Antarctica/Rothera
    {  368, 7295,   28510,  4, }, // Antarctica/Syowa
    {  368, 8607,   51843,  6, }, // Antarctica/Troll
    {  368, 7374,   34825,  6, }, // Antarctica/Vostok
    {  368, 1429,   51849,  7, }, // Arctic/Longyearbyen
    {  368, 5570,    2192,  4, }, // Asia/Aden
    {  368, 5395,    2196,  6, }, // Asia/Almaty
    {  368, 3968,    2202,  5, }, // Asia/Amman
    {  368, 5511,    2207,  6, }, // Asia/Anadyr
    {  368, 5523,  106310,  5, }, // Asia/Aqtau
    {  368, 5368,  106315,  6, }, // Asia/Aqtobe
    {  368,  964,    2224,  7, }, // Asia/Ashgabat
    {  368, 8624,    2231,  6, }, // Asia/Atyrau
    {  368, 3607,  106321,  6, }, // Asia/Baghdad
    {  368, 5534,   36948,  7, }, // Asia/Bahrain
    {  368, 3653,    2250,  4, }, // Asia/Baku
    {  368, 4466,    2254,  7, }, // Asia/Bangkok
    {  368, 3571,  106327,  8, }, // Asia/Barnaul
    {  368, 4086,   28563,  6, }, // Asia/Beirut
    {  368, 3778,   28569,  6, }, // Asia/Bishkek
    {  368, 5989,   28575,  6, }, // Asia/Brunei
    {  368, 4601,    2286,  4, }, // Asia/Chita
    {  368, 4511,    2290,  7, }, // Asia/Colombo
    {  368, 4540,    2297,  6, }, // Asia/Damascus
    {  368, 1093,    2303,  5, }, // Asia/Dhaka
    {  368, 6192,    2308,  4, }, // Asia/Dili
    {  368, 3596,   28596,  5, }, // Asia/Dubai
    {  368, 6159,    2317,  7, }, // Asia/Dushanbe
    {  368, 8636,    2324,  9, }, // Asia/Famagusta
    {  368, 8651,    2333,  4, }, // Asia/Gaza
    {  368, 4795,    2337,  6, }, // Asia/Hebron
    {  368, 1259,    2343,  7, }, // Asia/Ho_Chi_Minh
    {  368, 2704,   51902,  7, }, // Asia/Hong_Kong
    {  368, 4771,    2357,  4, }, // Asia/Hovd
    {  368, 4184,    2361,  7, }, // Asia/Irkutsk
    {  368, 7006,   28638,  8, }, // Asia/Jakarta
    {  368, 6992,   28646,  8, }, // Asia/Jayapura
    {  368, 1290,    2383,  9, }, // Asia/Jerusalem
    {  368, 3560,    2392,  5, }, // Asia/Kabul
    {  368, 4316,    2411,  8, }, // Asia/Kamchatka
    {  368, 4239,    2422,  6, }, // Asia/Karachi
    {  368, 1185,    2428,  8, }, // Asia/Kathmandu
    {  368, 8661,    2436,  7, }, // Asia/Khandyga
    {  368,  992,   51950,  9, }, // Asia/Kolkata
    {  368, 4197,   36978, 10, }, // Asia/Krasnoyarsk
    {  368, 4859,    2462, 12, }, // Asia/Kuala_Lumpur
    {  368, 5976,    2474,  6, }, // Asia/Kuching
    {  368, 5547,   28693,  6, }, // Asia/Kuwait
    {  368, 1211,    2486,  5, }, // Asia/Macau
    {  368, 4017,    2491,  7, }, // Asia/Magadan
    {  368, 1349,    2498,  7, }, // Asia/Makassar
    {  368, 7182,    2505,  6, }, // Asia/Manila
    {  368, 6920,    2511,  6, }, // Asia/Muscat
    {  368, 2554,  106335,  7, }, // Asia/Nicosia
    {  368, 8675,    2524, 11, }, // Asia/Novokuznetsk
    {  368, 4135,  106342,  9, }, // Asia/Novosibirsk
    {  368, 4229,  106351,  4, }, // Asia/Omsk
    {  368, 7145,   35006,  4, }, // Asia/Oral
    {  368, 6961,   51959,  8, }, // Asia/Phnom_Penh
    {  368, 8693,    2563,  9, }, // Asia/Pontianak
    {  368, 4214,   28763,  7, }, // Asia/Pyongyang
    {  368, 5559,    2579,  5, }, // Asia/Qatar
    {  368, 8708,  106355,  8, }, // Asia/Qostanay
    {  368, 4269,  106363,  9, }, // Asia/Qyzylorda
    {  368, 3584,   51967,  7, }, // Asia/Riyadh
    {  368, 4408,    2614,  7, }, // Asia/Sakhalin
    {  368, 7245,  106372,  9, }, // Asia/Samarkand
    {  368, 3279,    2630,  4, }, // Asia/Seoul
    {  368, 1053,   28801,  6, }, // Asia/Shanghai
    {  368, 3300,    2640,  8, }, // Asia/Singapore
    {  368, 4297,    2648, 13, }, // Asia/Srednekolymsk
    {  368, 3263,   28829,  6, }, // Asia/Taipei
    {  368, 4781,    2667,  7, }, // Asia/Tashkent
    {  368, 3915,   35074,  7, }, // Asia/Tbilisi
    {  368, 2772,    2678,  7, }, // Asia/Tehran
    {  368, 1317,    2685,  7, }, // Asia/Thimphu
    {  368, 2821,    2692,  5, }, // Asia/Tokyo
    {  368, 4572,    2697,  5, }, // Asia/Tomsk
    {  368, 1021,  106381,  9, }, // Asia/Ulaanbaatar
    {  368, 1159,  106390,  6, }, // Asia/Urumqi
    {  368, 8722,   51993,  9, }, // Asia/Ust-Nera
    {  368, 6977,   52002,  8, }, // Asia/Vientiane
    {  368, 4710,    2733, 11, }, // Asia/Vladivostok
    {  368, 4828,   37051,  6, }, // Asia/Yakutsk
    {  368, 1235,   56409,  5, }, // Asia/Yangon
    {  368, 3883,    2757, 12, }, // Asia/Yekaterinburg
    {  368, 3747,    2769,  6, }, // Asia/Yerevan
    {  368, 3663,  106396, 13, }, // Atlantic/Azores
    {  368, 5671,  106409, 15, }, // Atlantic/Bermuda
    {  368, 6628,  106424, 14, }, // Atlantic/Canary
    {  368, 3727,    2826, 10, }, // Atlantic/Cape_Verde
    {  368, 1395,  106438, 14, }, // Atlantic/Faroe
    {  368, 8736,   37132,  7, }, // Atlantic/Madeira
    {  368, 2748,   52082,  9, }, // Atlantic/Reykjavik
    {  368, 7272,  106452, 16, }, // Atlantic/South_Georgia
    {  368, 6821,  106468, 17, }, // Atlantic/St_Helena
    {  368, 6644,   52124,  6, }, // Atlantic/Stanley
    {  368, 1670,    2917,  8, }, // Australia/Adelaide
    {  368, 1635,    2925,  7, }, // Australia/Brisbane
    {  368, 1799,    2932, 11, }, // Australia/Broken_Hill
    {  368, 1583,    2943,  6, }, // Australia/Darwin
    {  368, 3637,   29108,  4, }, // Australia/Eucla
    {  368, 1516,    2954,  6, }, // Australia/Hobart
    {  368, 8753,    2960,  8, }, // Australia/Lindeman
    {  368, 1547,    2968,  8, }, // Australia/Lord_Howe
    {  368, 1727,   29120,  8, }, // Australia/Melbourne
    {  368, 1762,    2983,  4, }, // Australia/Perth
    {  368, 1463,   52144,  6, }, // Australia/Sydney
    {  368, 6441,    2993,  9, }, // Europe/Amsterdam
    {  368, 6216,    3002,  7, }, // Europe/Andorra
    {  368, 3620,    3009,  8, }, // Europe/Astrakhan
    {  368, 2300,   52159,  5, }, // Europe/Athens
    {  368, 6336,    3021,  7, }, // Europe/Belgrade
    {  368, 4757,    3028,  6, }, // Europe/Berlin
    {  368, 6488,    3034, 10, }, // Europe/Bratislava
    {  368, 2167,   52164,  8, }, // Europe/Brussels
    {  368, 3928,    3052,  8, }, // Europe/Bucharest
    {  368, 3806,    3060,  8, }, // Europe/Budapest
    {  368, 8772,   52172, 19, }, // Europe/Busingen
    {  368, 2583,    3091,  7, }, // Europe/Chisinau
    {  368, 6289,   52191, 10, }, // Europe/Copenhagen
    {  368, 2338,    3108,  6, }, // Europe/Dublin
    {  368, 6307,    3114,  9, }, // Europe/Gibraltar
    {  368, 8788,    3123,  6, }, // Europe/Guernsey
    {  368, 6612,   52201,  9, }, // Europe/Helsinki
    {  368, 8804,  106485,  9, }, // Europe/Isle_of_Man
    {  368, 1130,  106494,  7, }, // Europe/Istanbul
    {  368, 8823,   52219,  6, }, // Europe/Jersey
    {  368, 3979,    3158, 11, }, // Europe/Kaliningrad
    {  368, 8929,    3169,  5, }, // Europe/Kirov
    {  368, 2527,    3174,  4, }, // Europe/Kyiv
    {  368, 3213,    3178,  8, }, // Europe/Lisbon
    {  368, 6506,   29300,  7, }, // Europe/Ljubljana
    {  368, 2501,    3195,  6, }, // Europe/London
    {  368, 6365,   29313, 10, }, // Europe/Luxembourg
    {  368, 4892,    3212,  6, }, // Europe/Madrid
    {  368, 6397,   29329,  6, }, // Europe/Malta
    {  368, 6582,    3223,  9, }, // Europe/Mariehamn
    {  368, 3693,    3232,  5, }, // Europe/Minsk
    {  368, 6410,    3237,  6, }, // Europe/Monaco
    {  368, 3537,  106501,  6, }, // Europe/Moscow
    {  368, 6458,    3249,  4, }, // Europe/Oslo
    {  368, 4284,    3253,  5, }, // Europe/Paris
    {  368, 6424,    3258,  9, }, // Europe/Podgorica
    {  368, 6275,    3267,  5, }, // Europe/Prague
    {  368, 8837,    3272,  4, }, // Europe/Riga
    {  368, 6324,    3276,  3, }, // Europe/Rome
    {  368, 4331,    3279,  6, }, // Europe/Samara
    {  368, 6470,    3285, 10, }, // Europe/San_Marino
    {  368, 6245,    3295,  7, }, // Europe/Sarajevo
    {  368, 4451,    3302,  7, }, // Europe/Saratov
    {  368, 8849,   52225, 11, }, // Europe/Simferopol
    {  368, 6383,   52236,  6, }, // Europe/Skopje
    {  368, 6599,   37286,  5, }, // Europe/Sofia
    {  368, 6523,   52242,  9, }, // Europe/Stockholm
    {  368, 8867,    3337,  6, }, // Europe/Tallinn
    {  368, 6202,    3343,  6, }, // Europe/Tirane
    {  368, 8882,   52251,  9, }, // Europe/Ulyanovsk
    {  368, 6352,    3358,  5, }, // Europe/Vaduz
    {  368, 6567,    3363,  7, }, // Europe/Vatican
    {  368, 6231,    3370,  4, }, // Europe/Vienna
    {  368, 8899,   52260,  7, }, // Europe/Vilnius
    {  368, 4727,    3381,  9, }, // Europe/Volgograd
    {  368, 3190,    3390,  7, }, // Europe/Warsaw
    {  368, 6261,    3397,  6, }, // Europe/Zagreb
    {  368, 6540,   37310,  5, }, // Europe/Zurich
    {  368, 5078,    3409, 12, }, // Indian/Antananarivo
    {  368, 6947,    3421,  5, }, // Indian/Chagos
    {  368, 6046,  106507, 15, }, // Indian/Christmas
    {  368, 6063,  106522, 14, }, // Indian/Cocos
    {  368, 5029,  106536, 14, }, // Indian/Comoro
    {  368, 6661,    3460,  8, }, // Indian/Kerguelen
    {  368, 7260,   29562,  3, }, // Indian/Mahe
    {  368, 7050,  106550, 16, }, // Indian/Maldives
    {  368, 4069,   56576,  8, }, // Indian/Mauritius
    {  368, 5098,   56584,  7, }, // Indian/Mayotte
    {  368, 7211,   52348,  7, }, // Indian/Reunion
    {  368, 4422,    3498,  4, }, // Pacific/Apia
    {  368,  932,    3502,  6, }, // Pacific/Auckland
    {  368, 3706,   52355,  9, }, // Pacific/Bougainville
    {  368, 3015,    3516,  5, }, // Pacific/Chatham
    {  368, 3157,    3521,  4, }, // Pacific/Chuuk
    {  368, 2237,  106566, 11, }, // Pacific/Easter
    {  368, 7360,   52409,  5, }, // Pacific/Efate
    {  368, 7327,    3541,  7, }, // Pacific/Fakaofo
    {  368, 3902,   37407,  5, }, // Pacific/Fiji
    {  368, 7343,    3552,  8, }, // Pacific/Funafuti
    {  368, 6678,    3560,  9, }, // Pacific/Galapagos
    {  368, 6696,   52435,  6, }, // Pacific/Gambier
    {  368, 3822,    3595, 11, }, // Pacific/Guadalcanal
    {  368, 6018,    3606,  4, }, // Pacific/Guam
    {  368, 2723,   52450,  8, }, // Pacific/Honolulu
    {  368, 3049,   52458,  9, }, // Pacific/Kanton
    {  368, 3998,    3616, 10, }, // Pacific/Kiritimati
    {  368, 7019,  106577,  6, }, // Pacific/Kosrae
    {  368, 2842,   29709, 10, }, // Pacific/Kwajalein
    {  368, 4877,   37442,  7, }, // Pacific/Majuro
    {  368, 4051,  106583, 15, }, // Pacific/Marquesas
    {  368, 8914,   52490,  6, }, // Pacific/Midway
    {  368, 7084,    3680,  5, }, // Pacific/Nauru
    {  368, 7132,   52504,  4, }, // Pacific/Niue
    {  368, 4168,    3689,  7, }, // Pacific/Norfolk
    {  368, 7098,    3696,  5, }, // Pacific/Noumea
    {  368, 3126,    3701,  9, }, // Pacific/Pago_Pago
    {  368, 7155,    3710,  5, }, // Pacific/Palau
    {  368, 7194,   56657,  7, }, // Pacific/Pitcairn
    {  368, 3096,   35651,  6, }, // Pacific/Pohnpei
    {  368, 4807,    3728, 11, }, // Pacific/Port_Moresby
    {  368, 6076,    3739,  9, }, // Pacific/Rarotonga
    {  368, 6031,   29803,  6, }, // Pacific/Saipan
    {  368, 7312,    3754,  5, }, // Pacific/Tahiti
    {  368, 6712,    3759,  6, }, // Pacific/Tarawa
    {  368, 4583,    3765,  9, }, // Pacific/Tongatapu
    {  368, 7392,   29823,  4, }, // Pacific/Wake
    {  368, 7405,    3778,  6, }, // Pacific/Wallis
    {  369, 6788,  106598,  7, }, // Africa/Abidjan Kazakh/Arabic/China
    {  369, 6760,  106605,  5, }, // Africa/Accra
    {  369, 5059,  106610, 11, }, // Africa/Addis_Ababa
    {  369, 7420,  106621,  5, }, // Africa/Algiers
    {  369,   14,  106626,  6, }, // Africa/Asmara
    {  369,   44,   10415,  6, }, // Africa/Bamako
    {  369, 5257,  106632,  5, }, // Africa/Bangui
    {  369, 6746,  106637,  6, }, // Africa/Banjul
    {  369, 7435,  106643,  5, }, // Africa/Bissau
    {  369, 4957,   10438,  8, }, // Africa/Blantyre
    {  369, 5287,  106648,  9, }, // Africa/Brazzaville
    {  369, 4922,  106657,  9, }, // Africa/Bujumbura
    {  369, 2320,  106666,  4, }, // Africa/Cairo
    {  369, 4117,  106670, 10, }, // Africa/Casablanca
    {  369, 7449,  106680,  5, }, // Africa/Ceuta
    {  369, 6773,   10487,  7, }, // Africa/Conakry
    {  369, 6840,   10494,  5, }, // Africa/Dakar
    {  369, 5130,  106685, 12, }, // Africa/Dar_es_Salaam
    {  369, 5043,  106697,  6, }, // Africa/Djibouti
    {  369, 5243,  106703,  5, }, // Africa/Douala
    {  369, 5166,  106708,  7, }, // Africa/El_Aaiun
    {  369, 6853,  106715,  7, }, // Africa/Freetown
    {  369, 4906,  106722,  8, }, // Africa/Gaborone
    {  369, 5015,  106730,  6, }, // Africa/Harare
    {  369, 4479,  106736, 12, }, // Africa/Johannesburg
    {  369, 4499,  106748,  4, }, // Africa/Juba
    {  369, 5151,  106752,  7, }, // Africa/Kampala
    {  369, 4524,  106759,  6, }, // Africa/Khartoum
    {  369, 4973,  106765,  6, }, // Africa/Kigali
    {  369, 5306,   10579,  7, }, // Africa/Kinshasa
    {  369, 4744,  106771,  5, }, // Africa/Lagos
    {  369, 5336,  106776,  8, }, // Africa/Libreville
    {  369, 6869,  106784,  4, }, // Africa/Lome
    {  369, 5211,  106788,  6, }, // Africa/Luanda
    {  369, 4939,  106794,  9, }, // Africa/Lubumbashi
    {  369, 5001,  106803,  6, }, // Africa/Lusaka
    {  369, 5322,   10622,  6, }, // Africa/Malabo
    {  369, 4987,  106809,  6, }, // Africa/Maputo
    {  369, 5197,  106815,  6, }, // Africa/Maseru
    {  369, 5182,  106821,  7, }, // Africa/Mbabane
    {  369, 5113,  106828,  8, }, // Africa/Mogadishu
    {  369, 7034,  106836,  8, }, // Africa/Monrovia
    {  369, 3853,  106844,  7, }, // Africa/Nairobi
    {  369, 5271,  106851,  8, }, // Africa/Ndjamena
    {  369, 5354,  106859,  6, }, // Africa/Niamey
    {  369, 6803,  106865,  7, }, // Africa/Nouakchott
    {  369, 6727,  106872,  8, }, // Africa/Ouagadougou
    {  369, 5225,  106880, 10, }, // Africa/Porto-Novo
    {  369, 4435,  106890,  8, }, // Africa/Sao_Tome
    {  369, 2866,  106898,  7, }, // Africa/Tripoli
    {  369, 6554,  106905,  5, }, // Africa/Tunis
    {  369, 4152,  106910,  7, }, // Africa/Windhoek
    {  369,  132,  106917,  4, }, // America/Adak
    {  369, 3346,  106921,  7, }, // America/Anchorage
    {  369, 5607,  106928,  7, }, // America/Anguilla
    {  369, 5624,  106935,  7, }, // America/Antigua
    {  369, 4554,  106942,  9, }, // America/Araguaina
    {  369,  166,  106951, 12, }, // America/Argentina/Buenos_Aires
    {  369,   91,   10775,  9, }, // America/Argentina/Catamarca
    {  369,  270,   10784,  7, }, // America/Argentina/Cordoba
    {  369,  441,  106963,  5, }, // America/Argentina/Jujuy
    {  369, 7462,  106968,  8, }, // America/Argentina/La_Rioja
    {  369,  565,  106976,  7, }, // America/Argentina/Mendoza
    {  369, 7489,  106983, 11, }, // America/Argentina/Rio_Gallegos
    {  369, 7520,  106994,  5, }, // America/Argentina/Salta
    {  369, 7544,  106999,  8, }, // America/Argentina/San_Juan
    {  369, 5580,  107007,  8, }, // America/Argentina/San_Luis
    {  369, 7571,  107015,  7, }, // America/Argentina/Tucuman
    {  369, 7597,  107022,  6, }, // America/Argentina/Ushuaia
    {  369, 5640,  107028,  5, }, // America/Aruba
    {  369, 4252,  107033,  8, }, // America/Asuncion
    {  369,  237,  107041,  8, }, // America/Atikokan
    {  369, 3679,  107049,  4, }, // America/Bahia
    {  369, 7623,  107053, 16, }, // America/Bahia_Banderas
    {  369, 5654,  107069,  8, }, // America/Barbados
    {  369, 7646,  107077,  5, }, // America/Belem
    {  369, 5407,  107082,  5, }, // America/Belize
    {  369, 7660,   10909, 12, }, // America/Blanc-Sablon
    {  369, 7681,  107087,  9, }, // America/Boa_Vista
    {  369, 4361,  107096,  6, }, // America/Bogota
    {  369, 7699,  107102,  5, }, // America/Boise
    {  369, 7713,  107107, 11, }, // America/Cambridge_Bay
    {  369, 7735,  107118, 12, }, // America/Campo_Grande
    {  369, 3868,  107130,  6, }, // America/Cancun
    {  369, 4694,   10969,  7, }, // America/Caracas
    {  369, 4345,  107136,  7, }, // America/Cayenne
    {  369, 5496,  107143, 15, }, // America/Cayman
    {  369, 2260,  107158,  6, }, // America/Chicago
    {  369, 7756,  107164,  6, }, // America/Chihuahua
    {  369, 7774,  107170, 13, }, // America/Ciudad_Juarez
    {  369, 5422,  107183, 10, }, // America/Costa_Rica
    {  369, 7796,  107193,  7, }, // America/Creston
    {  369, 3791,  107200,  6, }, // America/Cuiaba
    {  369, 5723,  107206,  8, }, // America/Curacao
    {  369, 7812,  107214, 12, }, // America/Danmarkshavn
    {  369, 7833,  107226,  6, }, // America/Dawson
    {  369, 7848,  107232, 11, }, // America/Dawson_Creek
    {  369,  805,  107243,  6, }, // America/Denver
    {  369, 3465,  107249,  7, }, // America/Detroit
    {  369, 5739,   11079,  8, }, // America/Dominica
    {  369,  893,  107256,  8, }, // America/Edmonton
    {  369, 7869,  107264,  8, }, // America/Eirunepe
    {  369, 5441,  107272,  8, }, // America/El_Salvador
    {  369, 7886,  107280, 11, }, // America/Fort_Nelson
    {  369, 7906,  107291,  9, }, // America/Fortaleza
    {  369, 7924,  107300,  9, }, // America/Glace_Bay
    {  369, 6881,  107309,  7, }, // America/Goose_Bay
    {  369, 4612,  107316, 10, }, // America/Grand_Turk
    {  369, 5770,  107326,  7, }, // America/Grenada
    {  369, 5786,  107333,  9, }, // America/Guadeloupe
    {  369, 3760,  107342,  9, }, // America/Guatemala
    {  369, 4841,  107351,  8, }, // America/Guayaquil
    {  369, 6932,  107359,  6, }, // America/Guyana
    {  369, 1939,  107365,  8, }, // America/Halifax
    {  369, 2281,  107373,  6, }, // America/Havana
    {  369, 7942,  107379,  8, }, // America/Hermosillo
    {  369,  348,  107387, 12, }, // America/Indiana/Indianapolis
    {  369,  481,  107399, 13, }, // America/Indiana/Knox
    {  369, 7961,  107412, 16, }, // America/Indiana/Marengo
    {  369, 7985,  107428, 19, }, // America/Indiana/Petersburg
    {  369, 8012,  107447, 18, }, // America/Indiana/Tell_City
    {  369, 8038,  107465, 14, }, // America/Indiana/Vevay
    {  369, 8060,  107479, 18, }, // America/Indiana/Vincennes
    {  369, 8086,  107497, 16, }, // America/Indiana/Winamac
    {  369, 8110,  107513,  6, }, // America/Inuvik
    {  369,  660,  107519,  7, }, // America/Iqaluit
    {  369, 2799,  107526,  7, }, // America/Jamaica
    {  369, 5380,  107533,  4, }, // America/Juneau
    {  369,  521,  107537,  7, }, // America/Kentucky/Louisville
    {  369, 8125,  107544, 20, }, // America/Kentucky/Monticello
    {  369, 5704,  107564, 10, }, // America/Kralendijk
    {  369, 4376,  107574,  6, }, // America/La_Paz
    {  369, 7169,   11338,  4, }, // America/Lima
    {  369, 3239,  107580, 11, }, // America/Los_Angeles
    {  369, 5932,  107591, 18, }, // America/Lower_Princes
    {  369, 8153,  107609,  6, }, // America/Maceio
    {  369, 8168,  107615,  7, }, // America/Managua
    {  369, 1908,  107622,  6, }, // America/Manaus
    {  369, 5897,  107628,  6, }, // America/Marigot
    {  369, 5805,  107634,  9, }, // America/Martinique
    {  369, 8184,   11403,  9, }, // America/Matamoros
    {  369, 2917,  107643,  8, }, // America/Mazatlan
    {  369, 8202,  107651,  8, }, // America/Menominee
    {  369, 8220,  107659,  6, }, // America/Merida
    {  369, 8235,  107665, 10, }, // America/Metlakatla
    {  369, 2949,  107675,  6, }, // America/Mexico_City
    {  369, 4391,  107681,  7, }, // America/Miquelon
    {  369, 8254,  107688,  7, }, // America/Moncton
    {  369, 8270,  107695,  9, }, // America/Monterrey
    {  369, 4098,  107704, 10, }, // America/Montevideo
    {  369, 5824,  107714, 10, }, // America/Montserrat
    {  369, 5481,  107724,  6, }, // America/Nassau
    {  369, 2379,  107730,  8, }, // America/New_York
    {  369, 8288,   11422,  3, }, // America/Nome
    {  369, 1850,  107738,  7, }, // America/Noronha
    {  369, 8301,  107745, 23, }, // America/North_Dakota/Beulah
    {  369, 8329,  107768, 24, }, // America/North_Dakota/Center
    {  369, 8357,  107792, 28, }, // America/North_Dakota/New_Salem
    {  369,  393,  107820,  4, }, // America/Nuuk
    {  369, 8388,  107824,  7, }, // America/Ojinaga
    {  369, 2356,  107831,  6, }, // America/Panama
    {  369, 6173,  107837, 10, }, // America/Paramaribo
    {  369, 2973,  107847,  6, }, // America/Phoenix
    {  369, 3945,  107853, 13, }, // America/Port-au-Prince
    {  369, 5954,  107866, 13, }, // America/Port_of_Spain
    {  369, 8404,  107879, 11, }, // America/Porto_Velho
    {  369, 5843,  107890, 11, }, // America/Puerto_Rico
    {  369, 4030,  107901, 12, }, // America/Punta_Arenas
    {  369, 8424,  107913, 12, }, // America/Rankin_Inlet
    {  369, 8445,  107925,  6, }, // America/Recife
    {  369, 1995,  107931,  7, }, // America/Regina
    {  369, 8460,  107938,  8, }, // America/Resolute
    {  369,  695,  107946, 10, }, // America/Rio_Branco
    {  369, 8477,  107956,  8, }, // America/Santarem
    {  369, 2201,  107964,  8, }, // America/Santiago
    {  369, 6111,  107972, 13, }, // America/Santo_Domingo
    {  369, 1878,  107985,  9, }, // America/Sao_Paulo
    {  369, 6899,  107994, 16, }, // America/Scoresbysund
    {  369, 8494,   11721,  5, }, // America/Sitka
    {  369, 8508,  108010, 13, }, // America/St_Barthelemy
    {  369, 2061,  108023,  9, }, // America/St_Johns
    {  369, 5863,  108032,  9, }, // America/St_Kitts
    {  369, 5880,  108041, 11, }, // America/St_Lucia
    {  369,  855,  108052, 10, }, // America/St_Thomas
    {  369, 5913,  108062, 12, }, // America/St_Vincent
    {  369, 8530,  108074, 13, }, // America/Swift_Current
    {  369, 5461,  108087, 11, }, // America/Tegucigalpa
    {  369, 5756,  108098,  4, }, // America/Thule
    {  369,  313,  108102,  7, }, // America/Tijuana
    {  369,  608,   11819,  7, }, // America/Toronto
    {  369, 5688,   11826,  7, }, // America/Tortola
    {  369, 2093,  108109,  8, }, // America/Vancouver
    {  369, 2144,  108117,  8, }, // America/Whitehorse
    {  369,  734,  108125,  8, }, // America/Winnipeg
    {  369, 8552,  108133,  7, }, // America/Yakutat
    {  369, 6001,  108140,  5, }, // Antarctica/Casey
    {  369, 6094,  108145,  6, }, // Antarctica/Davis
    {  369, 6133,  108151, 14, }, // Antarctica/DumontDUrville
    {  369, 8568,  108165,  8, }, // Antarctica/Macquarie
    {  369, 7066,  108173,  6, }, // Antarctica/Mawson
    {  369, 7113,  108179,  9, }, // Antarctica/McMurdo
    {  369, 8589,  108188,  6, }, // Antarctica/Palmer
    {  369, 7226,  108194,  6, }, // Antarctica/Rothera
    {  369, 7295,  108200,  4, }, // Antarctica/Syowa
    {  369, 8607,   11929,  4, }, // Antarctica/Troll
    {  369, 7374,  108204,  6, }, // Antarctica/Vostok
    {  369, 1429,  108210,  6, }, // Arctic/Longyearbyen
    {  369, 5570,  108216,  4, }, // Asia/Aden
    {  369, 5395,  108220,  6, }, // Asia/Almaty
    {  369, 3968,  108226,  5, }, // Asia/Amman
    {  369, 5511,  108231,  6, }, // Asia/Anadyr
    {  369, 5523,  108237,  5, }, // Asia/Aqtau
    {  369, 5368,  108242,  6, }, // Asia/Aqtobe
    {  369,  964,  108248,  7, }, // Asia/Ashgabat
    {  369, 8624,  108255,  6, }, // Asia/Atyrau
    {  369, 3607,  108261,  6, }, // Asia/Baghdad
    {  369, 5534,  108267,  7, }, // Asia/Bahrain
    {  369, 3653,  108274,  4, }, // Asia/Baku
    {  369, 4466,  108278,  6, }, // Asia/Bangkok
    {  369, 3571,  108284,  8, }, // Asia/Barnaul
    {  369, 4086,  108292,  6, }, // Asia/Beirut
    {  369, 3778,  108298,  6, }, // Asia/Bishkek
    {  369, 5989,  108304,  6, }, // Asia/Brunei
    {  369, 4601,  108310,  4, }, // Asia/Chita
    {  369, 4511,   12037,  7, }, // Asia/Colombo
    {  369, 4540,  108314,  6, }, // Asia/Damascus
    {  369, 1093,  108320,  5, }, // Asia/Dhaka
    {  369, 6192,   12048,  4, }, // Asia/Dili
    {  369, 3596,  108325,  5, }, // Asia/Dubai
    {  369, 6159,  108330,  7, }, // Asia/Dushanbe
    {  369, 8636,  108337,  9, }, // Asia/Famagusta
    {  369, 8651,  108346,  4, }, // Asia/Gaza
    {  369, 4795,  108350,  6, }, // Asia/Hebron
    {  369, 1259,  108356,  7, }, // Asia/Ho_Chi_Minh
    {  369, 2704,  108363,  6, }, // Asia/Hong_Kong
    {  369, 4771,  108369,  4, }, // Asia/Hovd
    {  369, 4184,  108373,  7, }, // Asia/Irkutsk
    {  369, 7006,  108380,  7, }, // Asia/Jakarta
    {  369, 6992,  108387,  8, }, // Asia/Jayapura
    {  369, 1290,  108395,  9, }, // Asia/Jerusalem
    {  369, 3560,  108404,  5, }, // Asia/Kabul
    {  369, 4316,  108409,  8, }, // Asia/Kamchatka
    {  369, 4239,  108417,  6, }, // Asia/Karachi
    {  369, 1185,  108423,  8, }, // Asia/Kathmandu
    {  369, 8661,  108431,  7, }, // Asia/Khandyga
    {  369,  992,  108438,  7, }, // Asia/Kolkata
    {  369, 4197,   12193, 11, }, // Asia/Krasnoyarsk
    {  369, 4859,  108445, 12, }, // Asia/Kuala_Lumpur
    {  369, 5976,  108457,  5, }, // Asia/Kuching
    {  369, 5547,  108462,  6, }, // Asia/Kuwait
    {  369, 1211,   12228,  5, }, // Asia/Macau
    {  369, 4017,  108468,  7, }, // Asia/Magadan
    {  369, 1349,  108475,  7, }, // Asia/Makassar
    {  369, 7182,   12244,  6, }, // Asia/Manila
    {  369, 6920,  108482,  6, }, // Asia/Muscat
    {  369, 2554,  108488,  7, }, // Asia/Nicosia
    {  369, 8675,  108495, 12, }, // Asia/Novokuznetsk
    {  369, 4135,  108507,  9, }, // Asia/Novosibirsk
    {  369, 4229,  108516,  4, }, // Asia/Omsk
    {  369, 7145,   10866,  4, }, // Asia/Oral
    {  369, 6961,  108520,  7, }, // Asia/Phnom_Penh
    {  369, 8693,  108527,  9, }, // Asia/Pontianak
    {  369, 4214,  108536,  7, }, // Asia/Pyongyang
    {  369, 5559,  108543,  5, }, // Asia/Qatar
    {  369, 8708,   12321,  8, }, // Asia/Qostanay
    {  369, 4269,  108548,  9, }, // Asia/Qyzylorda
    {  369, 3584,  108557,  7, }, // Asia/Riyadh
    {  369, 4408,  108564,  7, }, // Asia/Sakhalin
    {  369, 7245,  108571,  9, }, // Asia/Samarkand
    {  369, 3279,  108580,  4, }, // Asia/Seoul
    {  369, 1053,  108584,  6, }, // Asia/Shanghai
    {  369, 3300,  108590,  8, }, // Asia/Singapore
    {  369, 4297,  108598, 13, }, // Asia/Srednekolymsk
    {  369, 3263,  108611,  6, }, // Asia/Taipei
    {  369, 4781,  108617,  7, }, // Asia/Tashkent
    {  369, 3915,  108624,  7, }, // Asia/Tbilisi
    {  369, 2772,  108631,  7, }, // Asia/Tehran
    {  369, 1317,  108638,  7, }, // Asia/Thimphu
    {  369, 2821,  108645,  5, }, // Asia/Tokyo
    {  369, 4572,  108650,  5, }, // Asia/Tomsk
    {  369, 1021,  108655,  9, }, // Asia/Ulaanbaatar
    {  369, 1159,  108664,  7, }, // Asia/Urumqi
    {  369, 8722,  108671,  8, }, // Asia/Ust-Nera
    {  369, 6977,  108679,  7, }, // Asia/Vientiane
    {  369, 4710,  108686, 11, }, // Asia/Vladivostok
    {  369, 4828,  108697,  7, }, // Asia/Yakutsk
    {  369, 1235,  108704,  6, }, // Asia/Yangon
    {  369, 3883,  108710, 12, }, // Asia/Yekaterinburg
    {  369, 3747,  108722,  7, }, // Asia/Yerevan
    {  369, 3663,  108729, 13, }, // Atlantic/Azores
    {  369, 5671,  108742, 15, }, // Atlantic/Bermuda
    {  369, 6628,  108757, 14, }, // Atlantic/Canary
    {  369, 3727,  108771, 10, }, // Atlantic/Cape_Verde
    {  369, 1395,  108781, 14, }, // Atlantic/Faroe
    {  369, 8736,  108795,  7, }, // Atlantic/Madeira
    {  369, 2748,  108802,  9, }, // Atlantic/Reykjavik
    {  369, 7272,  108811, 16, }, // Atlantic/South_Georgia
    {  369, 6821,  108827, 17, }, // Atlantic/St_Helena
    {  369, 6644,  108844,  6, }, // Atlantic/Stanley
    {  369, 1670,  108850,  8, }, // Australia/Adelaide
    {  369, 1635,  108858,  7, }, // Australia/Brisbane
    {  369, 1799,  108865, 10, }, // Australia/Broken_Hill
    {  369, 1583,  108875,  6, }, // Australia/Darwin
    {  369, 3637,  108881,  5, }, // Australia/Eucla
    {  369, 1516,  108886,  6, }, // Australia/Hobart
    {  369, 8753,  108892,  8, }, // Australia/Lindeman
    {  369, 1547,  108900,  8, }, // Australia/Lord_Howe
    {  369, 1727,  108908,  7, }, // Australia/Melbourne
    {  369, 1762,  108915,  4, }, // Australia/Perth
    {  369, 1463,  108919,  6, }, // Australia/Sydney
    {  369, 6441,  108925,  9, }, // Europe/Amsterdam
    {  369, 6216,  108934,  7, }, // Europe/Andorra
    {  369, 3620,  108941,  8, }, // Europe/Astrakhan
    {  369, 2300,  108949,  5, }, // Europe/Athens
    {  369, 6336,  108954,  7, }, // Europe/Belgrade
    {  369, 4757,  108961,  6, }, // Europe/Berlin
    {  369, 6488,  108967, 10, }, // Europe/Bratislava
    {  369, 2167,  108977,  7, }, // Europe/Brussels
    {  369, 3928,  108984,  8, }, // Europe/Bucharest
    {  369, 3806,  108992,  8, }, // Europe/Budapest
    {  369, 8772,  109000,  9, }, // Europe/Busingen
    {  369, 2583,  109009,  7, }, // Europe/Chisinau
    {  369, 6289,  109016, 10, }, // Europe/Copenhagen
    {  369, 2338,  109026,  6, }, // Europe/Dublin
    {  369, 6307,  109032,  9, }, // Europe/Gibraltar
    {  369, 8788,  109041,  6, }, // Europe/Guernsey
    {  369, 6612,  109047,  8, }, // Europe/Helsinki
    {  369, 8804,  109055,  9, }, // Europe/Isle_of_Man
    {  369, 1130,  109064,  7, }, // Europe/Istanbul
    {  369, 8823,  109071,  6, }, // Europe/Jersey
    {  369, 3979,  109077, 11, }, // Europe/Kaliningrad
    {  369, 8929,  109088,  5, }, // Europe/Kirov
    {  369, 2527,  109093,  4, }, // Europe/Kyiv
    {  369, 3213,  109097,  7, }, // Europe/Lisbon
    {  369, 6506,  109104,  9, }, // Europe/Ljubljana
    {  369, 2501,  109113,  6, }, // Europe/London
    {  369, 6365,  109119, 11, }, // Europe/Luxembourg
    {  369, 4892,  109130,  6, }, // Europe/Madrid
    {  369, 6397,  109136,  5, }, // Europe/Malta
    {  369, 6582,  109141,  9, }, // Europe/Mariehamn
    {  369, 3693,   12839,  5, }, // Europe/Minsk
    {  369, 6410,   12844,  6, }, // Europe/Monaco
    {  369, 3537,  109150,  6, }, // Europe/Moscow
    {  369, 6458,   12856,  4, }, // Europe/Oslo
    {  369, 4284,  109156,  5, }, // Europe/Paris
    {  369, 6424,  109161,  9, }, // Europe/Podgorica
    {  369, 6275,  109170,  5, }, // Europe/Prague
    {  369, 8837,  109175,  4, }, // Europe/Riga
    {  369, 6324,   11474,  3, }, // Europe/Rome
    {  369, 4331,  109179,  6, }, // Europe/Samara
    {  369, 6470,  109185, 10, }, // Europe/San_Marino
    {  369, 6245,  109195,  7, }, // Europe/Sarajevo
    {  369, 4451,  109202,  7, }, // Europe/Saratov
    {  369, 8849,  109209,  8, }, // Europe/Simferopol
    {  369, 6383,  109217,  6, }, // Europe/Skopje
    {  369, 6599,  109223,  5, }, // Europe/Sofia
    {  369, 6523,  109228,  9, }, // Europe/Stockholm
    {  369, 8867,  109237,  6, }, // Europe/Tallinn
    {  369, 6202,   12943,  6, }, // Europe/Tirane
    {  369, 8882,  109243,  9, }, // Europe/Ulyanovsk
    {  369, 6352,  109252,  5, }, // Europe/Vaduz
    {  369, 6567,  109257,  7, }, // Europe/Vatican
    {  369, 6231,  109264,  4, }, // Europe/Vienna
    {  369, 8899,  109268,  6, }, // Europe/Vilnius
    {  369, 4727,  109274,  9, }, // Europe/Volgograd
    {  369, 3190,  109283,  7, }, // Europe/Warsaw
    {  369, 6261,  109290,  6, }, // Europe/Zagreb
    {  369, 6540,  109296,  6, }, // Europe/Zurich
    {  369, 5078,  109302, 12, }, // Indian/Antananarivo
    {  369, 6947,  105823,  5, }, // Indian/Chagos
    {  369, 6046,  109314, 17, }, // Indian/Christmas
    {  369, 6063,  109331, 14, }, // Indian/Cocos
    {  369, 5029,  109345, 14, }, // Indian/Comoro
    {  369, 6661,  109359,  8, }, // Indian/Kerguelen
    {  369, 7260,  109367,  3, }, // Indian/Mahe
    {  369, 7050,  109370, 15, }, // Indian/Maldives
    {  369, 4069,  109385,  7, }, // Indian/Mauritius
    {  369, 5098,  109392,  7, }, // Indian/Mayotte
    {  369, 7211,  109399,  8, }, // Indian/Reunion
    {  369, 4422,  109407,  4, }, // Pacific/Apia
    {  369,  932,  109411,  6, }, // Pacific/Auckland
    {  369, 3706,  109417,  8, }, // Pacific/Bougainville
    {  369, 3015,  109425,  5, }, // Pacific/Chatham
    {  369, 3157,  109430,  4, }, // Pacific/Chuuk
    {  369, 2237,  109434, 11, }, // Pacific/Easter
    {  369, 7360,  109445,  5, }, // Pacific/Efate
    {  369, 7327,   13121,  7, }, // Pacific/Fakaofo
    {  369, 3902,   13128,  4, }, // Pacific/Fiji
    {  369, 7343,  109450,  8, }, // Pacific/Funafuti
    {  369, 6678,  109458,  9, }, // Pacific/Galapagos
    {  369, 6696,  109467,  6, }, // Pacific/Gambier
    {  369, 3822,  109473, 11, }, // Pacific/Guadalcanal
    {  369, 6018,  107619,  4, }, // Pacific/Guam
    {  369, 3049,  109484,  6, }, // Pacific/Kanton
    {  369, 3998,  109490, 10, }, // Pacific/Kiritimati
    {  369, 7019,  109500,  6, }, // Pacific/Kosrae
    {  369, 2842,  109506, 10, }, // Pacific/Kwajalein
    {  369, 4877,  109516,  6, }, // Pacific/Majuro
    {  369, 4051,  109522, 15, }, // Pacific/Marquesas
    {  369, 8914,  109537,  6, }, // Pacific/Midway
    {  369, 7084,  109543,  5, }, // Pacific/Nauru
    {  369, 7132,  109548,  4, }, // Pacific/Niue
    {  369, 4168,   13230,  7, }, // Pacific/Norfolk
    {  369, 7098,  109552,  5, }, // Pacific/Noumea
    {  369, 3126,  109557,  9, }, // Pacific/Pago_Pago
    {  369, 7155,  109566,  5, }, // Pacific/Palau
    {  369, 7194,  109571,  7, }, // Pacific/Pitcairn
    {  369, 3096,  109578,  6, }, // Pacific/Pohnpei
    {  369, 4807,  109584, 11, }, // Pacific/Port_Moresby
    {  369, 6076,  109595,  9, }, // Pacific/Rarotonga
    {  369, 6031,  109604,  6, }, // Pacific/Saipan
    {  369, 7312,  109610,  5, }, // Pacific/Tahiti
    {  369, 6712,  109615,  6, }, // Pacific/Tarawa
    {  369, 4583,  109621,  9, }, // Pacific/Tongatapu
    {  369, 7392,  109630,  4, }, // Pacific/Wake
    {  369, 7405,  109634,  6, }, // Pacific/Wallis
    {  370,   14,       0,  6, }, // Africa/Asmara Kenyang/Latin/Cameroon
    {  370, 4435,       6,  8, }, // Africa/Sao_Tome
    {  370, 4252,      14,  8, }, // America/Asuncion
    {  370,  237,      22,  8, }, // America/Atikokan
    {  370, 7623,      30, 17, }, // America/Bahia_Banderas
    {  370, 3868,      47,  6, }, // America/Cancun
    {  370, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  370, 5723,      66,  7, }, // America/Curacao
    {  370,  481,      73, 13, }, // America/Indiana/Knox
    {  370, 7961,      86, 16, }, // America/Indiana/Marengo
    {  370, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  370, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  370, 8038,     139, 14, }, // America/Indiana/Vevay
    {  370, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  370, 8086,     171, 16, }, // America/Indiana/Winamac
    {  370, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  370, 5932,     207, 22, }, // America/Lower_Princes
    {  370, 8220,     229,  6, }, // America/Merida
    {  370, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  370, 8329,     255, 20, }, // America/North_Dakota/Center
    {  370, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  370,  393,     298,  4, }, // America/Nuuk
    {  370, 6899,     302, 16, }, // America/Scoresbysund
    {  370, 8508,     318, 14, }, // America/St_Barthelemy
    {  370, 2061,     332, 10, }, // America/St_Johns
    {  370, 5863,     342,  9, }, // America/St_Kitts
    {  370, 5880,     351,  9, }, // America/St_Lucia
    {  370,  855,     360, 10, }, // America/St_Thomas
    {  370, 5913,     370, 11, }, // America/St_Vincent
    {  370, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  370, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  370, 1185,     413,  9, }, // Asia/Kathmandu
    {  370,  992,     422,  7, }, // Asia/Kolkata
    {  370, 1211,     429,  5, }, // Asia/Macau
    {  370, 1235,     442,  6, }, // Asia/Yangon
    {  370, 1395,     448,  5, }, // Atlantic/Faroe
    {  370, 6821,     453, 10, }, // Atlantic/St_Helena
    {  370, 2527,     463,  4, }, // Europe/Kyiv
    {  370, 7211,     467,  7, }, // Indian/Reunion
    {  370, 3157,     474,  5, }, // Pacific/Chuuk
    {  370, 3096,     479,  7, }, // Pacific/Pohnpei
    {  371, 6788,  109640, 10, }, // Africa/Abidjan Khmer/Khmer/Cambodia
    {  371, 6760,  109650,  6, }, // Africa/Accra
    {  371, 5059,  109656, 11, }, // Africa/Addis_Ababa
    {  371, 7420,  109667,  8, }, // Africa/Algiers
    {  371,   14,  109675,  9, }, // Africa/Asmara
    {  371,   44,  109684,  7, }, // Africa/Bamako
    {  371, 5257,  109691,  7, }, // Africa/Bangui
    {  371, 6746,  109698,  9, }, // Africa/Banjul
    {  371, 7435,  109707,  7, }, // Africa/Bissau
    {  371, 4957,  109714, 11, }, // Africa/Blantyre
    {  371, 5287,  109725, 11, }, // Africa/Brazzaville
    {  371, 4922,  109736, 13, }, // Africa/Bujumbura
    {  371, 2320,  109749,  3, }, // Africa/Cairo
    {  371, 4117,  109752, 12, }, // Africa/Casablanca
    {  371, 7449,  109764,  4, }, // Africa/Ceuta
    {  371, 6773,  109768,  8, }, // Africa/Conakry
    {  371, 6840,  109776,  4, }, // Africa/Dakar
    {  371, 5130,  109780, 10, }, // Africa/Dar_es_Salaam
    {  371, 5043,  109790,  7, }, // Africa/Djibouti
    {  371, 5243,  109797,  7, }, // Africa/Douala
    {  371, 5166,  109804, 11, }, // Africa/El_Aaiun
    {  371, 6853,  109815,  9, }, // Africa/Freetown
    {  371, 4906,  109824, 10, }, // Africa/Gaborone
    {  371, 5015,  109834,  8, }, // Africa/Harare
    {  371, 4479,  109842, 13, }, // Africa/Johannesburg
    {  371, 4499,  109855,  4, }, // Africa/Juba
    {  371, 5151,  109859,  7, }, // Africa/Kampala
    {  371, 4524,  109866,  5, }, // Africa/Khartoum
    {  371, 4973,  109871,  8, }, // Africa/Kigali
    {  371, 5306,  109879,  9, }, // Africa/Kinshasa
    {  371, 4744,  109888,  7, }, // Africa/Lagos
    {  371, 5336,  109895,  9, }, // Africa/Libreville
    {  371, 6869,  109904,  5, }, // Africa/Lome
    {  371, 5211,  109909,  7, }, // Africa/Luanda
    {  371, 4939,  109916, 11, }, // Africa/Lubumbashi
    {  371, 5001,  109927,  6, }, // Africa/Lusaka
    {  371, 5322,  109933,  7, }, // Africa/Malabo
    {  371, 4987,  109940,  8, }, // Africa/Maputo
    {  371, 5197,  109948,  8, }, // Africa/Maseru
    {  371, 5182,  109956,  5, }, // Africa/Mbabane
    {  371, 5113,  109961, 13, }, // Africa/Mogadishu
    {  371, 7034,  109974, 12, }, // Africa/Monrovia
    {  371, 3853,  109986,  8, }, // Africa/Nairobi
    {  371, 5271,  109994,  8, }, // Africa/Ndjamena
    {  371, 5354,  110002,  8, }, // Africa/Niamey
    {  371, 6803,  110010,  6, }, // Africa/Nouakchott
    {  371, 6727,  110016, 12, }, // Africa/Ouagadougou
    {  371, 5225,  110028, 10, }, // Africa/Porto-Novo
    {  371, 4435,  110038,  7, }, // Africa/Sao_Tome
    {  371, 2866,  110045,  9, }, // Africa/Tripoli
    {  371, 6554,  110054,  6, }, // Africa/Tunis
    {  371, 4152,  110060,  6, }, // Africa/Windhoek
    {  371,  132,  110066,  5, }, // America/Adak
    {  371, 3346,  110071,  9, }, // America/Anchorage
    {  371, 5607,  110080,  9, }, // America/Anguilla
    {  371, 5624,  110089,  9, }, // America/Antigua
    {  371, 4554,  110098, 12, }, // America/Araguaina
    {  371,  166,  110110,  9, }, // America/Argentina/Buenos_Aires
    {  371,   91,  110119,  9, }, // America/Argentina/Catamarca
    {  371,  270,  110128,  6, }, // America/Argentina/Cordoba
    {  371,  441,  110134,  5, }, // America/Argentina/Jujuy
    {  371, 7462,  110139,  9, }, // America/Argentina/La_Rioja
    {  371,  565,  110148, 10, }, // America/Argentina/Mendoza
    {  371, 7489,  110158, 16, }, // America/Argentina/Rio_Gallegos
    {  371, 7520,  110174,  6, }, // America/Argentina/Salta
    {  371, 7544,  110180,  9, }, // America/Argentina/San_Juan
    {  371, 5580,  110189, 10, }, // America/Argentina/San_Luis
    {  371, 7571,  110199,  8, }, // America/Argentina/Tucuman
    {  371, 7597,  110207,  8, }, // America/Argentina/Ushuaia
    {  371, 5640,  110215,  7, }, // America/Aruba
    {  371, 4252,  110222, 10, }, // America/Asuncion
    {  371,  237,  110232,  9, }, // America/Atikokan
    {  371, 3679,  110241,  4, }, // America/Bahia
    {  371, 7623,  110245, 13, }, // America/Bahia_Banderas
    {  371, 5654,  110258,  7, }, // America/Barbados
    {  371, 7646,  110265,  5, }, // America/Belem
    {  371, 5407,  110270,  4, }, // America/Belize
    {  371, 7660,  110274, 13, }, // America/Blanc-Sablon
    {  371, 7681,  110287,  9, }, // America/Boa_Vista
    {  371, 4361,  110296,  8, }, // America/Bogota
    {  371, 7699,  110304,  6, }, // America/Boise
    {  371, 7713,  110310, 10, }, // America/Cambridge_Bay
    {  371, 7735,  110320, 14, }, // America/Campo_Grande
    {  371, 3868,  110334,  6, }, // America/Cancun
    {  371, 4694,  110340,  8, }, // America/Caracas
    {  371, 4345,  110348,  6, }, // America/Cayenne
    {  371, 5496,  110354,  5, }, // America/Cayman
    {  371, 2260,  110359,  9, }, // America/Chicago
    {  371, 7756,  110368,  6, }, // America/Chihuahua
    {  371, 7774,  110374, 19, }, // America/Ciudad_Juarez
    {  371, 5422,  110393, 10, }, // America/Costa_Rica
    {  371, 7796,  110403,  9, }, // America/Creston
    {  371, 3791,  110412,  7, }, // America/Cuiaba
    {  371, 5723,  110419,  7, }, // America/Curacao
    {  371, 7812,  110426, 14, }, // America/Danmarkshavn
    {  371, 7833,  110440,  6, }, // America/Dawson
    {  371, 7848,  110446, 12, }, // America/Dawson_Creek
    {  371,  805,  110458,  5, }, // America/Denver
    {  371, 3465,  110463, 10, }, // America/Detroit
    {  371, 5739,  110473,  7, }, // America/Dominica
    {  371,  893,  110480, 10, }, // America/Edmonton
    {  371, 7869,  110490, 11, }, // America/Eirunepe
    {  371, 5441,  110501, 13, }, // America/El_Salvador
    {  371, 7886,  110514, 11, }, // America/Fort_Nelson
    {  371, 7906,  110525, 14, }, // America/Fortaleza
    {  371, 7924,  110539,  8, }, // America/Glace_Bay
    {  371, 6881,  110547,  6, }, // America/Goose_Bay
    {  371, 4612,  110553, 11, }, // America/Grand_Turk
    {  371, 5770,  110564,  9, }, // America/Grenada
    {  371, 5786,  110573, 11, }, // America/Guadeloupe
    {  371, 3760,  110584, 11, }, // America/Guatemala
    {  371, 4841,  110595, 10, }, // America/Guayaquil
    {  371, 6932,  110605,  8, }, // America/Guyana
    {  371, 1939,  110613,  9, }, // America/Halifax
    {  371, 2281,  110622,  8, }, // America/Havana
    {  371, 7942,  110630, 11, }, // America/Hermosillo
    {  371,  348,  110641, 13, }, // America/Indiana/Indianapolis
    {  371,  481,  110654, 11, }, // America/Indiana/Knox
    {  371, 7961,  110665, 16, }, // America/Indiana/Marengo
    {  371, 7985,  110681, 17, }, // America/Indiana/Petersburg
    {  371, 8012,  110698, 18, }, // America/Indiana/Tell_City
    {  371, 8038,  110716, 16, }, // America/Indiana/Vevay
    {  371, 8060,  110732, 16, }, // America/Indiana/Vincennes
    {  371, 8086,  110748, 17, }, // America/Indiana/Winamac
    {  371, 8110,  110765,  8, }, // America/Inuvik
    {  371,  660,  110773, 11, }, // America/Iqaluit
    {  371, 2799,  110784, 11, }, // America/Jamaica
    {  371, 5380,  110795,  6, }, // America/Juneau
    {  371,  521,  110801,  8, }, // America/Kentucky/Louisville
    {  371, 8125,  110809, 17, }, // America/Kentucky/Monticello
    {  371, 5704,  110826, 10, }, // America/Kralendijk
    {  371, 4376,  110836,  6, }, // America/La_Paz
    {  371, 7169,  110842,  5, }, // America/Lima
    {  371, 3239,  110847, 12, }, // America/Los_Angeles
    {  371, 5932,  110859, 19, }, // America/Lower_Princes
    {  371, 8153,  110878,  8, }, // America/Maceio
    {  371, 8168,  110886,  9, }, // America/Managua
    {  371, 1908,  110895,  9, }, // America/Manaus
    {  371, 5897,  110904, 10, }, // America/Marigot
    {  371, 5805,  110914,  8, }, // America/Martinique
    {  371, 8184,  110922, 12, }, // America/Matamoros
    {  371, 2917,  110934, 11, }, // America/Mazatlan
    {  371, 8202,  110945,  9, }, // America/Menominee
    {  371, 8220,  110954,  7, }, // America/Merida
    {  371, 8235,  110961, 13, }, // America/Metlakatla
    {  371, 2949,  110974, 16, }, // America/Mexico_City
    {  371, 4391,  110990,  9, }, // America/Miquelon
    {  371, 8254,  110999,  7, }, // America/Moncton
    {  371, 8270,  111006,  9, }, // America/Monterrey
    {  371, 4098,  111015, 12, }, // America/Montevideo
    {  371, 5824,  111027, 12, }, // America/Montserrat
    {  371, 5481,  111039,  6, }, // America/Nassau
    {  371, 2379,  111045,  6, }, // America/New_York
    {  371, 8288,  110948,  3, }, // America/Nome
    {  371, 1850,  111051,  9, }, // America/Noronha
    {  371, 8301,  111060, 19, }, // America/North_Dakota/Beulah
    {  371, 8329,  111079, 20, }, // America/North_Dakota/Center
    {  371, 8357,  111099, 24, }, // America/North_Dakota/New_Salem
    {  371,  393,  111123,  3, }, // America/Nuuk
    {  371, 8388,  111126, 10, }, // America/Ojinaga
    {  371, 2356,  111136,  8, }, // America/Panama
    {  371, 6173,  111144, 13, }, // America/Paramaribo
    {  371, 2973,  111157,  5, }, // America/Phoenix
    {  371, 3945,  111162, 12, }, // America/Port-au-Prince
    {  371, 5954,  111174, 11, }, // America/Port_of_Spain
    {  371, 8404,  111185, 11, }, // America/Porto_Velho
    {  371, 5843,  111196,  9, }, // America/Puerto_Rico
    {  371, 4030,  111205, 13, }, // America/Punta_Arenas
    {  371, 8424,  111218, 16, }, // America/Rankin_Inlet
    {  371, 8445,  111234, 10, }, // America/Recife
    {  371, 1995,  111244,  9, }, // America/Regina
    {  371, 8460,  111253, 10, }, // America/Resolute
    {  371,  695,  111263, 12, }, // America/Rio_Branco
    {  371, 8477,  111275,  9, }, // America/Santarem
    {  371, 2201,  111284, 12, }, // America/Santiago
    {  371, 6111,  111296, 11, }, // America/Santo_Domingo
    {  371, 1878,  111307,  8, }, // America/Sao_Paulo
    {  371, 6899,  111315, 13, }, // America/Scoresbysund
    {  371, 8494,  111328,  6, }, // America/Sitka
    {  371, 8508,  111334, 14, }, // America/St_Barthelemy
    {  371, 2061,  111348,  7, }, // America/St_Johns
    {  371, 5863,  111355,  8, }, // America/St_Kitts
    {  371, 5880,  111363,  9, }, // America/St_Lucia
    {  371,  855,  111372, 12, }, // America/St_Thomas
    {  371, 5913,  111384, 11, }, // America/St_Vincent
    {  371, 8530,  111395, 11, }, // America/Swift_Current
    {  371, 5461,  111406, 18, }, // America/Tegucigalpa
    {  371, 5756,  111424,  4, }, // America/Thule
    {  371,  313,  111428, 10, }, // America/Tijuana
    {  371,  608,  111438,  8, }, // America/Toronto
    {  371, 5688,  111446,  6, }, // America/Tortola
    {  371, 2093,  111452,  9, }, // America/Vancouver
    {  371, 2144,  111461,  6, }, // America/Whitehorse
    {  371,  734,  111467,  7, }, // America/Winnipeg
    {  371, 8552,  111474,  8, }, // America/Yakutat
    {  371, 6001,  111482,  4, }, // Antarctica/Casey
    {  371, 6094,  111486,  5, }, // Antarctica/Davis
    {  371, 6133,  111491, 16, }, // Antarctica/DumontDUrville
    {  371, 8568,  111507, 10, }, // Antarctica/Macquarie
    {  371, 7066,  111517,  7, }, // Antarctica/Mawson
    {  371, 7113,  111524, 10, }, // Antarctica/McMurdo
    {  371, 8589,  111534,  7, }, // Antarctica/Palmer
    {  371, 7226,  111541,  8, }, // Antarctica/Rothera
    {  371, 7295,  111549,  9, }, // Antarctica/Syowa
    {  371, 8607,  111558,  5, }, // Antarctica/Troll
    {  371, 7374,  111563,  8, }, // Antarctica/Vostok
    {  371, 1429,  111571, 14, }, // Arctic/Longyearbyen
    {  371, 5570,  111585,  5, }, // Asia/Aden
    {  371, 5395,  111590,  9, }, // Asia/Almaty
    {  371, 3968,  111599,  7, }, // Asia/Amman
    {  371, 5511,  111606,  6, }, // Asia/Anadyr
    {  371, 5523,  111612,  5, }, // Asia/Aqtau
    {  371, 5368,  111617,  8, }, // Asia/Aqtobe
    {  371,  964,  111625, 10, }, // Asia/Ashgabat
    {  371, 8624,  111635,  7, }, // Asia/Atyrau
    {  371, 3607,  111642,  6, }, // Asia/Baghdad
    {  371, 5534,  111648,  6, }, // Asia/Bahrain
    {  371, 3653,  111654,  4, }, // Asia/Baku
    {  371, 4466,  111658,  5, }, // Asia/Bangkok
    {  371, 3571,  111663,  5, }, // Asia/Barnaul
    {  371, 4086,  111668,  5, }, // Asia/Beirut
    {  371, 3778,  111673,  8, }, // Asia/Bishkek
    {  371, 5989,  111681,  7, }, // Asia/Brunei
    {  371, 4601,  111688,  4, }, // Asia/Chita
    {  371, 4511,  111692,  7, }, // Asia/Colombo
    {  371, 4540,  111699,  6, }, // Asia/Damascus
    {  371, 1093,  111705,  6, }, // Asia/Dhaka
    {  371, 6192,  111711,  4, }, // Asia/Dili
    {  371, 3596,  111715,  5, }, // Asia/Dubai
    {  371, 6159,  111720, 10, }, // Asia/Dushanbe
    {  371, 8636,  111730, 16, }, // Asia/Famagusta
    {  371, 8651,  109752,  4, }, // Asia/Gaza
    {  371, 4795,  111746,  6, }, // Asia/Hebron
    {  371, 1259,  111752,  7, }, // Asia/Ho_Chi_Minh
    {  371, 2704,  111759,  6, }, // Asia/Hong_Kong
    {  371, 4771,  111765,  3, }, // Asia/Hovd
    {  371, 4184,  111768, 11, }, // Asia/Irkutsk
    {  371, 7006,  111779,  9, }, // Asia/Jakarta
    {  371, 6992,  111788, 10, }, // Asia/Jayapura
    {  371, 1290,  111798, 13, }, // Asia/Jerusalem
    {  371, 3560,  111811,  6, }, // Asia/Kabul
    {  371, 4316,  111817,  8, }, // Asia/Kamchatka
    {  371, 4239,  111825,  7, }, // Asia/Karachi
    {  371, 1185,  111832, 11, }, // Asia/Kathmandu
    {  371, 8661,  111843, 10, }, // Asia/Khandyga
    {  371,  992,  111853,  7, }, // Asia/Kolkata
    {  371, 4197,  111860, 11, }, // Asia/Krasnoyarsk
    {  371, 4859,  111871, 10, }, // Asia/Kuala_Lumpur
    {  371, 5976,  111881,  6, }, // Asia/Kuching
    {  371, 5547,  111887,  6, }, // Asia/Kuwait
    {  371, 1211,  111893,  6, }, // Asia/Macau
    {  371, 4017,  111899, 10, }, // Asia/Magadan
    {  371, 1349,  111909,  9, }, // Asia/Makassar
    {  371, 7182,  111918,  6, }, // Asia/Manila
    {  371, 6920,  111924,  8, }, // Asia/Muscat
    {  371, 2554,  111932,  7, }, // Asia/Nicosia
    {  371, 8675,  111939, 16, }, // Asia/Novokuznetsk
    {  371, 4135,  111955, 12, }, // Asia/Novosibirsk
    {  371, 4229,  111967,  8, }, // Asia/Omsk
    {  371, 7145,  111975,  7, }, // Asia/Oral
    {  371, 6961,  111982,  7, }, // Asia/Phnom_Penh
    {  371, 8693,  111989, 12, }, // Asia/Pontianak
    {  371, 4214,  112001,  9, }, // Asia/Pyongyang
    {  371, 5559,  110119,  4, }, // Asia/Qatar
    {  371, 8708,  112010,  8, }, // Asia/Qostanay
    {  371, 4269,  112018, 11, }, // Asia/Qyzylorda
    {  371, 3584,  112029,  6, }, // Asia/Riyadh
    {  371, 4408,  112035,  7, }, // Asia/Sakhalin
    {  371, 7245,  112042,  9, }, // Asia/Samarkand
    {  371, 3279,  112051,  6, }, // Asia/Seoul
    {  371, 1053,  112057,  5, }, // Asia/Shanghai
    {  371, 3300,  112062,  9, }, // Asia/Singapore
    {  371, 4297,  112071, 12, }, // Asia/Srednekolymsk
    {  371, 3263,  112083,  5, }, // Asia/Taipei
    {  371, 4781,  112088,  7, }, // Asia/Tashkent
    {  371, 3915,  112095,  8, }, // Asia/Tbilisi
    {  371, 2772,  112103,  8, }, // Asia/Tehran
    {  371, 1317,  112111,  5, }, // Asia/Thimphu
    {  371, 2821,  112116,  6, }, // Asia/Tokyo
    {  371, 4572,  112122,  7, }, // Asia/Tomsk
    {  371, 1021,  112129, 11, }, // Asia/Ulaanbaatar
    {  371, 1159,  112140, 10, }, // Asia/Urumqi
    {  371, 8722,  112150, 10, }, // Asia/Ust-Nera
    {  371, 6977,  112160,  9, }, // Asia/Vientiane
    {  371, 4710,  112169, 15, }, // Asia/Vladivostok
    {  371, 4828,  112184, 11, }, // Asia/Yakutsk
    {  371, 1235,  112195,  9, }, // Asia/Yangon
    {  371, 3883,  112204, 14, }, // Asia/Yekaterinburg
    {  371, 3747,  112218, 11, }, // Asia/Yerevan
    {  371, 3663,  112229,  8, }, // Atlantic/Azores
    {  371, 5671,  112237,  8, }, // Atlantic/Bermuda
    {  371, 6628,  112245,  6, }, // Atlantic/Canary
    {  371, 3727,  112251,  7, }, // Atlantic/Cape_Verde
    {  371, 1395,  112258,  7, }, // Atlantic/Faroe
    {  371, 8736,  112265,  8, }, // Atlantic/Madeira
    {  371, 2748,  112273, 10, }, // Atlantic/Reykjavik
    {  371, 7272,  112283, 19, }, // Atlantic/South_Georgia
    {  371, 6821,  112302, 10, }, // Atlantic/St_Helena
    {  371, 6644,  112312,  7, }, // Atlantic/Stanley
    {  371, 1670,  112319,  7, }, // Australia/Adelaide
    {  371, 1635,  112326,  9, }, // Australia/Brisbane
    {  371, 1799,  112335, 11, }, // Australia/Broken_Hill
    {  371, 1583,  112346,  5, }, // Australia/Darwin
    {  371, 3637,  112351,  8, }, // Australia/Eucla
    {  371, 1516,  112359,  5, }, // Australia/Hobart
    {  371, 8753,  112364,  9, }, // Australia/Lindeman
    {  371, 1547,  112373,  7, }, // Australia/Lord_Howe
    {  371, 1727,  112380,  7, }, // Australia/Melbourne
    {  371, 1762,  112387,  3, }, // Australia/Perth
    {  371, 1463,  112390,  6, }, // Australia/Sydney
    {  371, 6441,  112396, 10, }, // Europe/Amsterdam
    {  371, 6216,  112406,  8, }, // Europe/Andorra
    {  371, 3620,  112414, 12, }, // Europe/Astrakhan
    {  371, 2300,  112426,  5, }, // Europe/Athens
    {  371, 6336,  112431,  8, }, // Europe/Belgrade
    {  371, 4757,  112439,  7, }, // Europe/Berlin
    {  371, 6488,  112446, 13, }, // Europe/Bratislava
    {  371, 2167,  112459,  8, }, // Europe/Brussels
    {  371, 3928,  112467, 10, }, // Europe/Bucharest
    {  371, 3806,  112477, 10, }, // Europe/Budapest
    {  371, 8772,  112487, 12, }, // Europe/Busingen
    {  371, 2583,  112499,  7, }, // Europe/Chisinau
    {  371, 6289,  112506,  9, }, // Europe/Copenhagen
    {  371, 2338,  112515,  9, }, // Europe/Dublin
    {  371, 6307,  112524, 13, }, // Europe/Gibraltar
    {  371, 8788,  112537,  8, }, // Europe/Guernsey
    {  371, 6612,  112545,  9, }, // Europe/Helsinki
    {  371, 8804,  112554, 12, }, // Europe/Isle_of_Man
    {  371, 1130,  112566, 12, }, // Europe/Istanbul
    {  371, 8823,  112578,  5, }, // Europe/Jersey
    {  371, 3979,  112583, 14, }, // Europe/Kaliningrad
    {  371, 8929,  112597,  6, }, // Europe/Kirov
    {  371, 2527,  112603,  3, }, // Europe/Kyiv
    {  371, 3213,  112606,  6, }, // Europe/Lisbon
    {  371, 6506,  112612,  9, }, // Europe/Ljubljana
    {  371, 2501,  112621,  5, }, // Europe/London
    {  371, 6365,  112626,  7, }, // Europe/Luxembourg
    {  371, 4892,  112633,  8, }, // Europe/Madrid
    {  371, 6397,  112641,  7, }, // Europe/Malta
    {  371, 6582,  112648,  8, }, // Europe/Mariehamn
    {  371, 3693,  112656,  8, }, // Europe/Minsk
    {  371, 6410,  112664,  7, }, // Europe/Monaco
    {  371, 3537,  112671,  6, }, // Europe/Moscow
    {  371, 6458,  112677,  6, }, // Europe/Oslo
    {  371, 4284,  112683,  6, }, // Europe/Paris
    {  371, 6424,  112689, 12, }, // Europe/Podgorica
    {  371, 6275,  112701,  5, }, // Europe/Prague
    {  371, 8837,  112706,  6, }, // Europe/Riga
    {  371, 6324,  112262,  4, }, // Europe/Rome
    {  371, 4331,  112712,  8, }, // Europe/Samara
    {  371, 6470,  112720, 12, }, // Europe/San_Marino
    {  371, 6245,  112732, 11, }, // Europe/Sarajevo
    {  371, 4451,  112743,  8, }, // Europe/Saratov
    {  371, 8849,  112751, 15, }, // Europe/Simferopol
    {  371, 6383,  112766,  7, }, // Europe/Skopje
    {  371, 6599,  112773, 10, }, // Europe/Sofia
    {  371, 6523,  112783,  8, }, // Europe/Stockholm
    {  371, 8867,  112791,  5, }, // Europe/Tallinn
    {  371, 6202,  112796,  7, }, // Europe/Tirane
    {  371, 8882,  112803, 14, }, // Europe/Ulyanovsk
    {  371, 6352,  112817,  6, }, // Europe/Vaduz
    {  371, 6567,  112823,  8, }, // Europe/Vatican
    {  371, 6231,  112831,  5, }, // Europe/Vienna
    {  371, 8899,  112836,  8, }, // Europe/Vilnius
    {  371, 4727,  112844, 15, }, // Europe/Volgograd
    {  371, 3190,  112859,  7, }, // Europe/Warsaw
    {  371, 6261,  112866,  7, }, // Europe/Zagreb
    {  371, 6540,  112873,  8, }, // Europe/Zurich
    {  371, 5078,  112881, 12, }, // Indian/Antananarivo
    {  371, 6947,  110362,  6, }, // Indian/Chagos
    {  371, 6046,  112893,  9, }, // Indian/Christmas
    {  371, 6063,  112902,  5, }, // Indian/Cocos
    {  371, 5029,  112907,  5, }, // Indian/Comoro
    {  371, 6661,  112912, 10, }, // Indian/Kerguelen
    {  371, 7260,  112922,  5, }, // Indian/Mahe
    {  371, 7050,  112927,  8, }, // Indian/Maldives
    {  371, 4069,  112935,  6, }, // Indian/Mauritius
    {  371, 5098,  112941,  6, }, // Indian/Mayotte
    {  371, 7211,  112947,  9, }, // Indian/Reunion
    {  371, 4422,  112956,  4, }, // Pacific/Apia
    {  371,  932,  112960,  5, }, // Pacific/Auckland
    {  371, 3706,  112965, 10, }, // Pacific/Bougainville
    {  371, 3015,  112975,  5, }, // Pacific/Chatham
    {  371, 3157,  112980,  5, }, // Pacific/Chuuk
    {  371, 2237,  112985,  8, }, // Pacific/Easter
    {  371, 7360,  112993,  9, }, // Pacific/Efate
    {  371, 7327,  113002, 10, }, // Pacific/Fakaofo
    {  371, 3902,  113012,  6, }, // Pacific/Fiji
    {  371, 7343,  113018, 14, }, // Pacific/Funafuti
    {  371, 6678,  113032, 14, }, // Pacific/Galapagos
    {  371, 6696,  113046,  7, }, // Pacific/Gambier
    {  371, 3822,  113053, 14, }, // Pacific/Guadalcanal
    {  371, 6018,  113046,  5, }, // Pacific/Guam
    {  371, 2723,  113067,  8, }, // Pacific/Honolulu
    {  371, 3049,  113075, 11, }, // Pacific/Kanton
    {  371, 3998,  113086, 11, }, // Pacific/Kiritimati
    {  371, 7019,  113097,  6, }, // Pacific/Kosrae
    {  371, 2842,  113103, 11, }, // Pacific/Kwajalein
    {  371, 4877,  113114, 10, }, // Pacific/Majuro
    {  371, 4051,  113124, 10, }, // Pacific/Marquesas
    {  371, 8914,  113134,  6, }, // Pacific/Midway
    {  371, 7084,  113140,  4, }, // Pacific/Nauru
    {  371, 7132,  113144,  5, }, // Pacific/Niue
    {  371, 4168,  113149,  8, }, // Pacific/Norfolk
    {  371, 7098,  113157,  4, }, // Pacific/Noumea
    {  371, 3126,  113161, 15, }, // Pacific/Pago_Pago
    {  371, 7155,  113176,  5, }, // Pacific/Palau
    {  371, 7194,  113181,  9, }, // Pacific/Pitcairn
    {  371, 3096,  113190,  6, }, // Pacific/Pohnpei
    {  371, 4807,  113196, 12, }, // Pacific/Port_Moresby
    {  371, 6076,  113208, 13, }, // Pacific/Rarotonga
    {  371, 6031,  113221,  6, }, // Pacific/Saipan
    {  371, 7312,  113227,  6, }, // Pacific/Tahiti
    {  371, 6712,  113233,  8, }, // Pacific/Tarawa
    {  371, 4583,  113241, 11, }, // Pacific/Tongatapu
    {  371, 7392,  113252,  5, }, // Pacific/Wake
    {  371, 7405,  113257,  6, }, // Pacific/Wallis
    {  372,   14,       0,  6, }, // Africa/Asmara Kiche/Latin/Guatemala
    {  372, 4435,       6,  8, }, // Africa/Sao_Tome
    {  372, 4252,      14,  8, }, // America/Asuncion
    {  372,  237,      22,  8, }, // America/Atikokan
    {  372, 7623,      30, 17, }, // America/Bahia_Banderas
    {  372, 3868,      47,  6, }, // America/Cancun
    {  372, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  372, 5723,      66,  7, }, // America/Curacao
    {  372,  481,      73, 13, }, // America/Indiana/Knox
    {  372, 7961,      86, 16, }, // America/Indiana/Marengo
    {  372, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  372, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  372, 8038,     139, 14, }, // America/Indiana/Vevay
    {  372, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  372, 8086,     171, 16, }, // America/Indiana/Winamac
    {  372, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  372, 5932,     207, 22, }, // America/Lower_Princes
    {  372, 8220,     229,  6, }, // America/Merida
    {  372, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  372, 8329,     255, 20, }, // America/North_Dakota/Center
    {  372, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  372,  393,     298,  4, }, // America/Nuuk
    {  372, 6899,     302, 16, }, // America/Scoresbysund
    {  372, 8508,     318, 14, }, // America/St_Barthelemy
    {  372, 2061,     332, 10, }, // America/St_Johns
    {  372, 5863,     342,  9, }, // America/St_Kitts
    {  372, 5880,     351,  9, }, // America/St_Lucia
    {  372,  855,     360, 10, }, // America/St_Thomas
    {  372, 5913,     370, 11, }, // America/St_Vincent
    {  372, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  372, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  372, 1185,     413,  9, }, // Asia/Kathmandu
    {  372,  992,     422,  7, }, // Asia/Kolkata
    {  372, 1211,     429,  5, }, // Asia/Macau
    {  372, 1235,     442,  6, }, // Asia/Yangon
    {  372, 1395,     448,  5, }, // Atlantic/Faroe
    {  372, 6821,     453, 10, }, // Atlantic/St_Helena
    {  372, 2527,     463,  4, }, // Europe/Kyiv
    {  372, 7211,     467,  7, }, // Indian/Reunion
    {  372, 3157,     474,  5, }, // Pacific/Chuuk
    {  372, 3096,     479,  7, }, // Pacific/Pohnpei
    {  373,   14,       0,  6, }, // Africa/Asmara Kikuyu/Latin/Kenya
    {  373, 4435,       6,  8, }, // Africa/Sao_Tome
    {  373, 4252,      14,  8, }, // America/Asuncion
    {  373,  237,      22,  8, }, // America/Atikokan
    {  373, 7623,      30, 17, }, // America/Bahia_Banderas
    {  373, 3868,      47,  6, }, // America/Cancun
    {  373, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  373, 5723,      66,  7, }, // America/Curacao
    {  373,  481,      73, 13, }, // America/Indiana/Knox
    {  373, 7961,      86, 16, }, // America/Indiana/Marengo
    {  373, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  373, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  373, 8038,     139, 14, }, // America/Indiana/Vevay
    {  373, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  373, 8086,     171, 16, }, // America/Indiana/Winamac
    {  373, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  373, 5932,     207, 22, }, // America/Lower_Princes
    {  373, 8220,     229,  6, }, // America/Merida
    {  373, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  373, 8329,     255, 20, }, // America/North_Dakota/Center
    {  373, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  373,  393,     298,  4, }, // America/Nuuk
    {  373, 6899,     302, 16, }, // America/Scoresbysund
    {  373, 8508,     318, 14, }, // America/St_Barthelemy
    {  373, 2061,     332, 10, }, // America/St_Johns
    {  373, 5863,     342,  9, }, // America/St_Kitts
    {  373, 5880,     351,  9, }, // America/St_Lucia
    {  373,  855,     360, 10, }, // America/St_Thomas
    {  373, 5913,     370, 11, }, // America/St_Vincent
    {  373, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  373, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  373, 1185,     413,  9, }, // Asia/Kathmandu
    {  373,  992,     422,  7, }, // Asia/Kolkata
    {  373, 1211,     429,  5, }, // Asia/Macau
    {  373, 1235,     442,  6, }, // Asia/Yangon
    {  373, 1395,     448,  5, }, // Atlantic/Faroe
    {  373, 6821,     453, 10, }, // Atlantic/St_Helena
    {  373, 2527,     463,  4, }, // Europe/Kyiv
    {  373, 7211,     467,  7, }, // Indian/Reunion
    {  373, 3157,     474,  5, }, // Pacific/Chuuk
    {  373, 3096,     479,  7, }, // Pacific/Pohnpei
    {  374,   14,       0,  6, }, // Africa/Asmara Kinyarwanda/Latin/Rwanda
    {  374, 4435,       6,  8, }, // Africa/Sao_Tome
    {  374, 4252,      14,  8, }, // America/Asuncion
    {  374,  237,      22,  8, }, // America/Atikokan
    {  374, 7623,      30, 17, }, // America/Bahia_Banderas
    {  374, 3868,      47,  6, }, // America/Cancun
    {  374, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  374, 5723,      66,  7, }, // America/Curacao
    {  374,  481,      73, 13, }, // America/Indiana/Knox
    {  374, 7961,      86, 16, }, // America/Indiana/Marengo
    {  374, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  374, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  374, 8038,     139, 14, }, // America/Indiana/Vevay
    {  374, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  374, 8086,     171, 16, }, // America/Indiana/Winamac
    {  374, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  374, 5932,     207, 22, }, // America/Lower_Princes
    {  374, 8220,     229,  6, }, // America/Merida
    {  374, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  374, 8329,     255, 20, }, // America/North_Dakota/Center
    {  374, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  374,  393,     298,  4, }, // America/Nuuk
    {  374, 6899,     302, 16, }, // America/Scoresbysund
    {  374, 8508,     318, 14, }, // America/St_Barthelemy
    {  374, 2061,     332, 10, }, // America/St_Johns
    {  374, 5863,     342,  9, }, // America/St_Kitts
    {  374, 5880,     351,  9, }, // America/St_Lucia
    {  374,  855,     360, 10, }, // America/St_Thomas
    {  374, 5913,     370, 11, }, // America/St_Vincent
    {  374, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  374, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  374, 1185,     413,  9, }, // Asia/Kathmandu
    {  374,  992,     422,  7, }, // Asia/Kolkata
    {  374, 1211,     429,  5, }, // Asia/Macau
    {  374, 1235,     442,  6, }, // Asia/Yangon
    {  374, 1395,     448,  5, }, // Atlantic/Faroe
    {  374, 6821,     453, 10, }, // Atlantic/St_Helena
    {  374, 2527,     463,  4, }, // Europe/Kyiv
    {  374, 7211,     467,  7, }, // Indian/Reunion
    {  374, 3157,     474,  5, }, // Pacific/Chuuk
    {  374, 3096,     479,  7, }, // Pacific/Pohnpei
    {  375, 6788,   29832,  7, }, // Africa/Abidjan Konkani/Devanagari/India
    {  375, 6760,  113263,  4, }, // Africa/Accra
    {  375, 5059,  113267, 10, }, // Africa/Addis_Ababa
    {  375, 7420,  113277, 10, }, // Africa/Algiers
    {  375,   14,   88147,  7, }, // Africa/Asmara
    {  375,   44,   88154,  6, }, // Africa/Bamako
    {  375, 5257,  113287,  5, }, // Africa/Bangui
    {  375, 6746,  113292,  6, }, // Africa/Banjul
    {  375, 7435,  113298,  5, }, // Africa/Bissau
    {  375, 4957,  113303,  9, }, // Africa/Blantyre
    {  375, 5287,  113312, 10, }, // Africa/Brazzaville
    {  375, 4922,  113322,  9, }, // Africa/Bujumbura
    {  375, 2320,   88206,  5, }, // Africa/Cairo
    {  375, 4117,   29933, 11, }, // Africa/Casablanca
    {  375, 7449,   88211,  5, }, // Africa/Ceuta
    {  375, 6773,  113331,  8, }, // Africa/Conakry
    {  375, 6840,   88224,  4, }, // Africa/Dakar
    {  375, 5130,  113339, 10, }, // Africa/Dar_es_Salaam
    {  375, 5043,   88239,  6, }, // Africa/Djibouti
    {  375, 5243,  113349,  5, }, // Africa/Douala
    {  375, 5166,  113354,  7, }, // Africa/El_Aaiun
    {  375, 6853,  113361,  8, }, // Africa/Freetown
    {  375, 4906,   88267,  7, }, // Africa/Gaborone
    {  375, 5015,   88274,  5, }, // Africa/Harare
    {  375, 4479,  113369, 11, }, // Africa/Johannesburg
    {  375, 4499,   30022,  4, }, // Africa/Juba
    {  375, 5151,   30026,  8, }, // Africa/Kampala
    {  375, 4524,  113380,  7, }, // Africa/Khartoum
    {  375, 4973,   88295,  6, }, // Africa/Kigali
    {  375, 5306,  113387,  7, }, // Africa/Kinshasa
    {  375, 4744,   88308,  5, }, // Africa/Lagos
    {  375, 5336,   88313, 10, }, // Africa/Libreville
    {  375, 6869,   88323,  3, }, // Africa/Lome
    {  375, 5211,   88326,  6, }, // Africa/Luanda
    {  375, 4939,   88332,  9, }, // Africa/Lubumbashi
    {  375, 5001,   30089,  6, }, // Africa/Lusaka
    {  375, 5322,   88341,  5, }, // Africa/Malabo
    {  375, 4987,   88346,  6, }, // Africa/Maputo
    {  375, 5197,  113394,  6, }, // Africa/Maseru
    {  375, 5182,   88352,  7, }, // Africa/Mbabane
    {  375, 5113,  113400,  8, }, // Africa/Mogadishu
    {  375, 7034,   88367,  9, }, // Africa/Monrovia
    {  375, 3853,  113408,  7, }, // Africa/Nairobi
    {  375, 5271,  113415,  8, }, // Africa/Ndjamena
    {  375, 5354,   88392,  6, }, // Africa/Niamey
    {  375, 6803,  113423,  7, }, // Africa/Nouakchott
    {  375, 6727,  113430,  7, }, // Africa/Ouagadougou
    {  375, 5225,   88412, 11, }, // Africa/Porto-Novo
    {  375, 4435,   88423,  7, }, // Africa/Sao_Tome
    {  375, 2866,   88430,  8, }, // Africa/Tripoli
    {  375, 6554,  113437,  5, }, // Africa/Tunis
    {  375, 4152,   88438,  7, }, // Africa/Windhoek
    {  375,  132,  113442,  4, }, // America/Adak
    {  375, 3346,  113446,  5, }, // America/Anchorage
    {  375, 5607,  113451,  6, }, // America/Anguilla
    {  375, 5624,  113457,  7, }, // America/Antigua
    {  375, 4554,  113464,  8, }, // America/Araguaina
    {  375,  166,  113472, 12, }, // America/Argentina/Buenos_Aires
    {  375,   91,  113484,  9, }, // America/Argentina/Catamarca
    {  375,  270,  113493,  7, }, // America/Argentina/Cordoba
    {  375,  441,  113500,  5, }, // America/Argentina/Jujuy
    {  375, 7462,  113505,  9, }, // America/Argentina/La_Rioja
    {  375,  565,  113514,  8, }, // America/Argentina/Mendoza
    {  375, 7489,  113522, 11, }, // America/Argentina/Rio_Gallegos
    {  375, 7520,   30305,  6, }, // America/Argentina/Salta
    {  375, 7544,  113533,  8, }, // America/Argentina/San_Juan
    {  375, 5580,   30320,  8, }, // America/Argentina/San_Luis
    {  375, 7571,  113541,  7, }, // America/Argentina/Tucuman
    {  375, 7597,   88546,  7, }, // America/Argentina/Ushuaia
    {  375, 5640,  113548,  5, }, // America/Aruba
    {  375, 4252,  113553,  7, }, // America/Asuncion
    {  375,  237,  113560,  8, }, // America/Atikokan
    {  375, 3679,   88574,  5, }, // America/Bahia
    {  375, 7623,  113568, 12, }, // America/Bahia_Banderas
    {  375, 5654,   88594,  8, }, // America/Barbados
    {  375, 7646,   30394,  5, }, // America/Belem
    {  375, 5407,  113580,  5, }, // America/Belize
    {  375, 7660,  113585, 14, }, // America/Blanc-Sablon
    {  375, 7681,  113599, 10, }, // America/Boa_Vista
    {  375, 4361,   88631,  6, }, // America/Bogota
    {  375, 7699,  113609,  4, }, // America/Boise
    {  375, 7713,  113613, 11, }, // America/Cambridge_Bay
    {  375, 7735,  113624, 13, }, // America/Campo_Grande
    {  375, 3868,  113637,  6, }, // America/Cancun
    {  375, 4694,  113643,  5, }, // America/Caracas
    {  375, 4345,  113648,  4, }, // America/Cayenne
    {  375, 5496,   30482,  5, }, // America/Cayman
    {  375, 2260,   88682,  6, }, // America/Chicago
    {  375, 7756,   88688,  8, }, // America/Chihuahua
    {  375, 7774,  113652, 13, }, // America/Ciudad_Juarez
    {  375, 5422,  113665, 11, }, // America/Costa_Rica
    {  375, 7796,  113676,  9, }, // America/Creston
    {  375, 3791,  113685,  6, }, // America/Cuiaba
    {  375, 5723,   88730,  7, }, // America/Curacao
    {  375, 7812,  113691, 12, }, // America/Danmarkshavn
    {  375, 7833,  113703,  4, }, // America/Dawson
    {  375, 7848,  113707, 10, }, // America/Dawson_Creek
    {  375,  805,  113717,  7, }, // America/Denver
    {  375, 3465,  113724,  8, }, // America/Detroit
    {  375, 5739,  113732,  8, }, // America/Dominica
    {  375,  893,  113740,  7, }, // America/Edmonton
    {  375, 7869,   30591,  7, }, // America/Eirunepe
    {  375, 5441,  113747, 12, }, // America/El_Salvador
    {  375, 7886,  113759, 12, }, // America/Fort_Nelson
    {  375, 7906,  113771, 10, }, // America/Fortaleza
    {  375, 7924,   30628,  8, }, // America/Glace_Bay
    {  375, 6881,  113781,  6, }, // America/Goose_Bay
    {  375, 4612,  113787, 11, }, // America/Grand_Turk
    {  375, 5770,  113798,  8, }, // America/Grenada
    {  375, 5786,   88856,  9, }, // America/Guadeloupe
    {  375, 3760,   30674, 10, }, // America/Guatemala
    {  375, 4841,  113806,  9, }, // America/Guayaquil
    {  375, 6932,   30692,  6, }, // America/Guyana
    {  375, 1939,  113815,  9, }, // America/Halifax
    {  375, 2281,   88889,  5, }, // America/Havana
    {  375, 7942,  113824,  9, }, // America/Hermosillo
    {  375,  348,  113833, 13, }, // America/Indiana/Indianapolis
    {  375,  481,  113846, 15, }, // America/Indiana/Knox
    {  375, 7961,   88933, 17, }, // America/Indiana/Marengo
    {  375, 7985,  113861, 20, }, // America/Indiana/Petersburg
    {  375, 8012,   88970, 18, }, // America/Indiana/Tell_City
    {  375, 8038,  113881, 14, }, // America/Indiana/Vevay
    {  375, 8060,  113895, 17, }, // America/Indiana/Vincennes
    {  375, 8086,  113912, 17, }, // America/Indiana/Winamac
    {  375, 8110,  113929,  6, }, // America/Inuvik
    {  375,  660,  113935,  8, }, // America/Iqaluit
    {  375, 2799,   89044,  5, }, // America/Jamaica
    {  375, 5380,  113943,  4, }, // America/Juneau
    {  375,  521,  113947,  8, }, // America/Kentucky/Louisville
    {  375, 8125,  113955, 19, }, // America/Kentucky/Monticello
    {  375, 5704,  113974, 10, }, // America/Kralendijk
    {  375, 4376,  113984,  6, }, // America/La_Paz
    {  375, 7169,  113990,  4, }, // America/Lima
    {  375, 3239,  113994, 10, }, // America/Los_Angeles
    {  375, 5932,  114004, 22, }, // America/Lower_Princes
    {  375, 8153,  114026,  6, }, // America/Maceio
    {  375, 8168,  114032,  8, }, // America/Managua
    {  375, 1908,   89145,  4, }, // America/Manaus
    {  375, 5897,  114040,  7, }, // America/Marigot
    {  375, 5805,   89156,  9, }, // America/Martinique
    {  375, 8184,   89165,  9, }, // America/Matamoros
    {  375, 2917,  114047,  9, }, // America/Mazatlan
    {  375, 8202,   89183,  8, }, // America/Menominee
    {  375, 8220,   31026,  6, }, // America/Merida
    {  375, 8235,  114056, 12, }, // America/Metlakatla
    {  375, 2949,  114068, 12, }, // America/Mexico_City
    {  375, 4391,  114080,  9, }, // America/Miquelon
    {  375, 8254,  114089,  8, }, // America/Moncton
    {  375, 8270,  114097,  7, }, // America/Monterrey
    {  375, 4098,  114104, 13, }, // America/Montevideo
    {  375, 5824,  114117, 10, }, // America/Montserrat
    {  375, 5481,  114127,  4, }, // America/Nassau
    {  375, 2379,  114131, 10, }, // America/New_York
    {  375, 8288,  114141,  4, }, // America/Nome
    {  375, 1850,   89272,  8, }, // America/Noronha
    {  375, 8301,  114145, 23, }, // America/North_Dakota/Beulah
    {  375, 8329,  114168, 18, }, // America/North_Dakota/Center
    {  375, 8357,  114186, 24, }, // America/North_Dakota/New_Salem
    {  375,  393,   31183,  3, }, // America/Nuuk
    {  375, 8388,  114210,  7, }, // America/Ojinaga
    {  375, 2356,   89355,  5, }, // America/Panama
    {  375, 6173,  114217,  9, }, // America/Paramaribo
    {  375, 2973,  114226,  7, }, // America/Phoenix
    {  375, 3945,  114233, 15, }, // America/Port-au-Prince
    {  375, 5954,  114248, 14, }, // America/Port_of_Spain
    {  375, 8404,   89407, 13, }, // America/Porto_Velho
    {  375, 5843,  114262, 11, }, // America/Puerto_Rico
    {  375, 4030,  114273, 12, }, // America/Punta_Arenas
    {  375, 8424,  114285, 11, }, // America/Rankin_Inlet
    {  375, 8445,  114296,  5, }, // America/Recife
    {  375, 1995,  114301,  6, }, // America/Regina
    {  375, 8460,  114307,  9, }, // America/Resolute
    {  375,  695,   89471, 12, }, // America/Rio_Branco
    {  375, 8477,  114316,  8, }, // America/Santarem
    {  375, 2201,  114324,  8, }, // America/Santiago
    {  375, 6111,  114332, 13, }, // America/Santo_Domingo
    {  375, 1878,  114345,  9, }, // America/Sao_Paulo
    {  375, 6899,  114354, 13, }, // America/Scoresbysund
    {  375, 8494,   89535,  6, }, // America/Sitka
    {  375, 8508,   89541, 15, }, // America/St_Barthelemy
    {  375, 2061,  114367, 10, }, // America/St_Johns
    {  375, 5863,   89565, 10, }, // America/St_Kitts
    {  375, 5880,  114377, 11, }, // America/St_Lucia
    {  375,  855,   89586,  9, }, // America/St_Thomas
    {  375, 5913,  114388, 13, }, // America/St_Vincent
    {  375, 8530,  114401, 12, }, // America/Swift_Current
    {  375, 5461,  114413, 11, }, // America/Tegucigalpa
    {  375, 5756,  114424,  4, }, // America/Thule
    {  375,  313,   89637,  7, }, // America/Tijuana
    {  375,  608,  114428,  6, }, // America/Toronto
    {  375, 5688,  114434,  7, }, // America/Tortola
    {  375, 2093,  114441, 10, }, // America/Vancouver
    {  375, 2144,  114451, 11, }, // America/Whitehorse
    {  375,  734,   89676,  7, }, // America/Winnipeg
    {  375, 8552,  114462,  7, }, // America/Yakutat
    {  375, 6001,   31541,  4, }, // Antarctica/Casey
    {  375, 6094,  114469,  7, }, // Antarctica/Davis
    {  375, 6133,  114476, 14, }, // Antarctica/DumontDUrville
    {  375, 8568,  114490,  8, }, // Antarctica/Macquarie
    {  375, 7066,   89721,  4, }, // Antarctica/Mawson
    {  375, 7113,  114498,  9, }, // Antarctica/McMurdo
    {  375, 8589,  114507,  7, }, // Antarctica/Palmer
    {  375, 7226,   89738,  6, }, // Antarctica/Rothera
    {  375, 7295,   89744,  6, }, // Antarctica/Syowa
    {  375, 8607,   89750,  5, }, // Antarctica/Troll
    {  375, 7374,   89755,  7, }, // Antarctica/Vostok
    {  375, 1429,  114514, 13, }, // Arctic/Longyearbyen
    {  375, 5570,  114527,  3, }, // Asia/Aden
    {  375, 5395,   89779,  7, }, // Asia/Almaty
    {  375, 3968,   89786,  6, }, // Asia/Amman
    {  375, 5511,  114530,  6, }, // Asia/Anadyr
    {  375, 5523,  114536,  5, }, // Asia/Aqtau
    {  375, 5368,  114541,  7, }, // Asia/Aqtobe
    {  375,  964,  114548,  7, }, // Asia/Ashgabat
    {  375, 8624,  114555,  7, }, // Asia/Atyrau
    {  375, 3607,   89809,  5, }, // Asia/Baghdad
    {  375, 5534,  114562,  6, }, // Asia/Bahrain
    {  375, 3653,   30522,  4, }, // Asia/Baku
    {  375, 4466,  114568,  5, }, // Asia/Bangkok
    {  375, 3571,  114573,  5, }, // Asia/Barnaul
    {  375, 4086,  114578,  5, }, // Asia/Beirut
    {  375, 3778,   31704,  7, }, // Asia/Bishkek
    {  375, 5989,   89836,  7, }, // Asia/Brunei
    {  375, 4601,  114583,  4, }, // Asia/Chita
    {  375, 4511,   89849,  6, }, // Asia/Colombo
    {  375, 4540,  114587,  6, }, // Asia/Damascus
    {  375, 1093,   89862,  4, }, // Asia/Dhaka
    {  375, 6192,  114593,  4, }, // Asia/Dili
    {  375, 3596,  114597,  4, }, // Asia/Dubai
    {  375, 6159,   31749,  8, }, // Asia/Dushanbe
    {  375, 8636,  114601,  9, }, // Asia/Famagusta
    {  375, 8651,  114610,  4, }, // Asia/Gaza
    {  375, 4795,  114614,  7, }, // Asia/Hebron
    {  375, 1259,   89904, 11, }, // Asia/Ho_Chi_Minh
    {  375, 2704,   89920,  9, }, // Asia/Hong_Kong
    {  375, 4771,  114621,  6, }, // Asia/Hovd
    {  375, 4184,  114627,  9, }, // Asia/Irkutsk
    {  375, 7006,   89944,  7, }, // Asia/Jakarta
    {  375, 6992,  114636,  8, }, // Asia/Jayapura
    {  375, 1290,  114644,  9, }, // Asia/Jerusalem
    {  375, 3560,   89957,  5, }, // Asia/Kabul
    {  375, 4316,  114653,  9, }, // Asia/Kamchatka
    {  375, 4239,   31840,  5, }, // Asia/Karachi
    {  375, 1185,   89969,  8, }, // Asia/Kathmandu
    {  375, 8661,  114662,  8, }, // Asia/Khandyga
    {  375,  992,   89984,  7, }, // Asia/Kolkata
    {  375, 4197,   89991, 15, }, // Asia/Krasnoyarsk
    {  375, 4859,  114670, 11, }, // Asia/Kuala_Lumpur
    {  375, 5976,  114681,  6, }, // Asia/Kuching
    {  375, 5547,  114687,  5, }, // Asia/Kuwait
    {  375, 1211,  114692,  4, }, // Asia/Macau
    {  375, 4017,  114696,  6, }, // Asia/Magadan
    {  375, 1349,  114702,  8, }, // Asia/Makassar
    {  375, 7182,  114710,  5, }, // Asia/Manila
    {  375, 6920,   90026,  5, }, // Asia/Muscat
    {  375, 2554,   90031,  8, }, // Asia/Nicosia
    {  375, 8675,  114715, 14, }, // Asia/Novokuznetsk
    {  375, 4135,   90055, 13, }, // Asia/Novosibirsk
    {  375, 4229,   90068,  6, }, // Asia/Omsk
    {  375, 7145,   90074,  3, }, // Asia/Oral
    {  375, 6961,  114729, 11, }, // Asia/Phnom_Penh
    {  375, 8693,  114740, 10, }, // Asia/Pontianak
    {  375, 4214,   90095, 10, }, // Asia/Pyongyang
    {  375, 5559,  114750,  4, }, // Asia/Qatar
    {  375, 8708,  114754,  9, }, // Asia/Qostanay
    {  375, 4269,  114763, 12, }, // Asia/Qyzylorda
    {  375, 3584,   32028,  5, }, // Asia/Riyadh
    {  375, 4408,  114775,  5, }, // Asia/Sakhalin
    {  375, 7245,   90138,  6, }, // Asia/Samarkand
    {  375, 3279,   89464,  3, }, // Asia/Seoul
    {  375, 1053,  114780,  6, }, // Asia/Shanghai
    {  375, 3300,  114786,  8, }, // Asia/Singapore
    {  375, 4297,  114794, 16, }, // Asia/Srednekolymsk
    {  375, 3263,  114810,  5, }, // Asia/Taipei
    {  375, 4781,  114815,  7, }, // Asia/Tashkent
    {  375, 3915,  114822,  7, }, // Asia/Tbilisi
    {  375, 2772,   32104,  6, }, // Asia/Tehran
    {  375, 1317,  114829,  6, }, // Asia/Thimphu
    {  375, 2821,  114835,  6, }, // Asia/Tokyo
    {  375, 4572,  114841,  6, }, // Asia/Tomsk
    {  375, 1021,  114847,  8, }, // Asia/Ulaanbaatar
    {  375, 1159,  114855,  7, }, // Asia/Urumqi
    {  375, 8722,  114862,  9, }, // Asia/Ust-Nera
    {  375, 6977,  114871, 11, }, // Asia/Vientiane
    {  375, 4710,   90240, 13, }, // Asia/Vladivostok
    {  375, 4828,  114882,  8, }, // Asia/Yakutsk
    {  375, 1235,  114890,  6, }, // Asia/Yangon
    {  375, 3883,   90253, 13, }, // Asia/Yekaterinburg
    {  375, 3747,   90266,  7, }, // Asia/Yerevan
    {  375, 3663,  114896,  6, }, // Atlantic/Azores
    {  375, 5671,  114902,  7, }, // Atlantic/Bermuda
    {  375, 6628,  114909,  5, }, // Atlantic/Canary
    {  375, 3727,  114914,  9, }, // Atlantic/Cape_Verde
    {  375, 1395,  114923,  4, }, // Atlantic/Faroe
    {  375, 8736,  114927,  5, }, // Atlantic/Madeira
    {  375, 2748,  114932,  9, }, // Atlantic/Reykjavik
    {  375, 7272,   90319, 15, }, // Atlantic/South_Georgia
    {  375, 6821,  114941, 11, }, // Atlantic/St_Helena
    {  375, 6644,  114952,  8, }, // Atlantic/Stanley
    {  375, 1670,   90352,  6, }, // Australia/Adelaide
    {  375, 1635,  114960,  9, }, // Australia/Brisbane
    {  375, 1799,   90366, 10, }, // Australia/Broken_Hill
    {  375, 1583,   90376,  7, }, // Australia/Darwin
    {  375, 3637,  114969,  5, }, // Australia/Eucla
    {  375, 1516,   90383,  7, }, // Australia/Hobart
    {  375, 8753,  114974,  7, }, // Australia/Lindeman
    {  375, 1547,   90398, 10, }, // Australia/Lord_Howe
    {  375, 1727,  114981,  7, }, // Australia/Melbourne
    {  375, 1762,   90416,  4, }, // Australia/Perth
    {  375, 1463,   32362,  5, }, // Australia/Sydney
    {  375, 6441,  114988, 10, }, // Europe/Amsterdam
    {  375, 6216,   90430,  6, }, // Europe/Andorra
    {  375, 3620,  114998,  8, }, // Europe/Astrakhan
    {  375, 2300,  115006,  6, }, // Europe/Athens
    {  375, 6336,   32400,  8, }, // Europe/Belgrade
    {  375, 4757,   90451,  6, }, // Europe/Berlin
    {  375, 6488,  115012, 14, }, // Europe/Bratislava
    {  375, 2167,  115026,  9, }, // Europe/Brussels
    {  375, 3928,  115035,  9, }, // Europe/Bucharest
    {  375, 3806,   90488,  9, }, // Europe/Budapest
    {  375, 8772,  115044,  8, }, // Europe/Busingen
    {  375, 2583,  115052,  7, }, // Europe/Chisinau
    {  375, 6289,  115059,  8, }, // Europe/Copenhagen
    {  375, 2338,  115067,  6, }, // Europe/Dublin
    {  375, 6307,   90528, 10, }, // Europe/Gibraltar
    {  375, 8788,   90538,  6, }, // Europe/Guernsey
    {  375, 6612,   90544,  8, }, // Europe/Helsinki
    {  375, 8804,  115073, 11, }, // Europe/Isle_of_Man
    {  375, 1130,  115084,  8, }, // Europe/Istanbul
    {  375, 8823,   90572,  5, }, // Europe/Jersey
    {  375, 3979,  115092, 12, }, // Europe/Kaliningrad
    {  375, 8929,   90589,  5, }, // Europe/Kirov
    {  375, 2527,   90594,  3, }, // Europe/Kyiv
    {  375, 3213,   32558,  6, }, // Europe/Lisbon
    {  375, 6506,  115104,  9, }, // Europe/Ljubljana
    {  375, 2501,  115113,  4, }, // Europe/London
    {  375, 6365,  115117, 10, }, // Europe/Luxembourg
    {  375, 4892,  115127,  7, }, // Europe/Madrid
    {  375, 6397,   32599,  6, }, // Europe/Malta
    {  375, 6582,  115134,  9, }, // Europe/Mariehamn
    {  375, 3693,   90638,  6, }, // Europe/Minsk
    {  375, 6410,  115143,  6, }, // Europe/Monaco
    {  375, 3537,   90650,  6, }, // Europe/Moscow
    {  375, 6458,   90656,  5, }, // Europe/Oslo
    {  375, 4284,  115149,  5, }, // Europe/Paris
    {  375, 6424,  115154,  9, }, // Europe/Podgorica
    {  375, 6275,   90671,  5, }, // Europe/Prague
    {  375, 8837,   90676,  4, }, // Europe/Riga
    {  375, 6324,   90303,  3, }, // Europe/Rome
    {  375, 4331,   90680,  5, }, // Europe/Samara
    {  375, 6470,  115163, 10, }, // Europe/San_Marino
    {  375, 6245,  115173,  8, }, // Europe/Sarajevo
    {  375, 4451,  115181,  7, }, // Europe/Saratov
    {  375, 8849,  115188,  9, }, // Europe/Simferopol
    {  375, 6383,   90722,  8, }, // Europe/Skopje
    {  375, 6599,  115197,  6, }, // Europe/Sofia
    {  375, 6523,   90737,  8, }, // Europe/Stockholm
    {  375, 8867,  115203,  5, }, // Europe/Tallinn
    {  375, 6202,  115208,  6, }, // Europe/Tirane
    {  375, 8882,   90756, 12, }, // Europe/Ulyanovsk
    {  375, 6352,  115214,  4, }, // Europe/Vaduz
    {  375, 6567,  115218,  6, }, // Europe/Vatican
    {  375, 6231,  115224,  6, }, // Europe/Vienna
    {  375, 8899,   90785,  8, }, // Europe/Vilnius
    {  375, 4727,  115230, 13, }, // Europe/Volgograd
    {  375, 3190,  115243,  5, }, // Europe/Warsaw
    {  375, 6261,  115248,  7, }, // Europe/Zagreb
    {  375, 6540,  115255,  5, }, // Europe/Zurich
    {  375, 5078,  115260, 12, }, // Indian/Antananarivo
    {  375, 6947,  115272,  5, }, // Indian/Chagos
    {  375, 6046,   90842,  7, }, // Indian/Christmas
    {  375, 6063,   90849,  5, }, // Indian/Cocos
    {  375, 5029,   90854,  6, }, // Indian/Comoro
    {  375, 6661,  115277,  8, }, // Indian/Kerguelen
    {  375, 7260,   32851,  4, }, // Indian/Mahe
    {  375, 7050,   90867,  6, }, // Indian/Maldives
    {  375, 4069,  115285,  6, }, // Indian/Mauritius
    {  375, 5098,  115291,  5, }, // Indian/Mayotte
    {  375, 7211,  115296,  8, }, // Indian/Reunion
    {  375, 4422,  115304,  5, }, // Pacific/Apia
    {  375,  932,  115309,  5, }, // Pacific/Auckland
    {  375, 3706,  115314,  9, }, // Pacific/Bougainville
    {  375, 3015,  115323,  4, }, // Pacific/Chatham
    {  375, 3157,  115327,  3, }, // Pacific/Chuuk
    {  375, 2237,   90920,  5, }, // Pacific/Easter
    {  375, 7360,  115330,  4, }, // Pacific/Efate
    {  375, 7327,  115334,  7, }, // Pacific/Fakaofo
    {  375, 3902,   32937,  4, }, // Pacific/Fiji
    {  375, 7343,  115341,  8, }, // Pacific/Funafuti
    {  375, 6678,  115349,  9, }, // Pacific/Galapagos
    {  375, 6696,  115358,  8, }, // Pacific/Gambier
    {  375, 3822,  115366, 12, }, // Pacific/Guadalcanal
    {  375, 6018,   30980,  4, }, // Pacific/Guam
    {  375, 3049,  115378,  7, }, // Pacific/Kanton
    {  375, 3998,  115385, 10, }, // Pacific/Kiritimati
    {  375, 7019,  115395,  5, }, // Pacific/Kosrae
    {  375, 2842,  115400,  8, }, // Pacific/Kwajalein
    {  375, 4877,   91013,  6, }, // Pacific/Majuro
    {  375, 4051,  115408,  8, }, // Pacific/Marquesas
    {  375, 8914,   91027,  5, }, // Pacific/Midway
    {  375, 7084,  115416,  5, }, // Pacific/Nauru
    {  375, 7132,   91036,  4, }, // Pacific/Niue
    {  375, 4168,  115421,  6, }, // Pacific/Norfolk
    {  375, 7098,  115427,  5, }, // Pacific/Noumea
    {  375, 3126,   91053,  9, }, // Pacific/Pago_Pago
    {  375, 7155,   91062,  4, }, // Pacific/Palau
    {  375, 7194,  115432,  7, }, // Pacific/Pitcairn
    {  375, 3096,  115439,  8, }, // Pacific/Pohnpei
    {  375, 4807,  115447, 14, }, // Pacific/Port_Moresby
    {  375, 6076,   91086,  9, }, // Pacific/Rarotonga
    {  375, 6031,   91095,  6, }, // Pacific/Saipan
    {  375, 7312,   91101,  6, }, // Pacific/Tahiti
    {  375, 6712,  115461,  5, }, // Pacific/Tarawa
    {  375, 4583,   91113,  9, }, // Pacific/Tongatapu
    {  375, 7392,  115466,  3, }, // Pacific/Wake
    {  375, 7405,   33139,  5, }, // Pacific/Wallis
    {  376,   14,       0,  6, }, // Africa/Asmara Konkani/Latin/India
    {  376, 4435,       6,  8, }, // Africa/Sao_Tome
    {  376, 4252,      14,  8, }, // America/Asuncion
    {  376,  237,      22,  8, }, // America/Atikokan
    {  376, 7623,      30, 17, }, // America/Bahia_Banderas
    {  376, 3868,      47,  6, }, // America/Cancun
    {  376, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  376, 5723,      66,  7, }, // America/Curacao
    {  376,  481,      73, 13, }, // America/Indiana/Knox
    {  376, 7961,      86, 16, }, // America/Indiana/Marengo
    {  376, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  376, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  376, 8038,     139, 14, }, // America/Indiana/Vevay
    {  376, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  376, 8086,     171, 16, }, // America/Indiana/Winamac
    {  376, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  376, 5932,     207, 22, }, // America/Lower_Princes
    {  376, 8220,     229,  6, }, // America/Merida
    {  376, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  376, 8329,     255, 20, }, // America/North_Dakota/Center
    {  376, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  376,  393,     298,  4, }, // America/Nuuk
    {  376, 6899,     302, 16, }, // America/Scoresbysund
    {  376, 8508,     318, 14, }, // America/St_Barthelemy
    {  376, 2061,     332, 10, }, // America/St_Johns
    {  376, 5863,     342,  9, }, // America/St_Kitts
    {  376, 5880,     351,  9, }, // America/St_Lucia
    {  376,  855,     360, 10, }, // America/St_Thomas
    {  376, 5913,     370, 11, }, // America/St_Vincent
    {  376, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  376, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  376, 1185,     413,  9, }, // Asia/Kathmandu
    {  376,  992,     422,  7, }, // Asia/Kolkata
    {  376, 1211,     429,  5, }, // Asia/Macau
    {  376, 1235,     442,  6, }, // Asia/Yangon
    {  376, 1395,     448,  5, }, // Atlantic/Faroe
    {  376, 6821,     453, 10, }, // Atlantic/St_Helena
    {  376, 2527,     463,  4, }, // Europe/Kyiv
    {  376, 7211,     467,  7, }, // Indian/Reunion
    {  376, 3157,     474,  5, }, // Pacific/Chuuk
    {  376, 3096,     479,  7, }, // Pacific/Pohnpei
    {  377, 6788,  115469,  3, }, // Africa/Abidjan Korean/Korean/South Korea
    {  377, 6760,  115472,  3, }, // Africa/Accra
    {  377, 5059,  115475,  6, }, // Africa/Addis_Ababa
    {  377, 7420,  115481,  2, }, // Africa/Algiers
    {  377,   14,  115483,  4, }, // Africa/Asmara
    {  377,   44,  115487,  3, }, // Africa/Bamako
    {  377, 5257,  115490,  2, }, // Africa/Bangui
    {  377, 6746,  115492,  2, }, // Africa/Banjul
    {  377, 7435,  115494,  3, }, // Africa/Bissau
    {  377, 4957,  115497,  5, }, // Africa/Blantyre
    {  377, 5287,  115502,  4, }, // Africa/Brazzaville
    {  377, 4922,  115506,  4, }, // Africa/Bujumbura
    {  377, 2320,  115510,  3, }, // Africa/Cairo
    {  377, 4117,  115513,  5, }, // Africa/Casablanca
    {  377, 7449,  115518,  3, }, // Africa/Ceuta
    {  377, 6773,  115521,  4, }, // Africa/Conakry
    {  377, 6840,  115525,  3, }, // Africa/Dakar
    {  377, 5130,  115528,  6, }, // Africa/Dar_es_Salaam
    {  377, 5043,  115534,  3, }, // Africa/Djibouti
    {  377, 5243,  115537,  3, }, // Africa/Douala
    {  377, 5166,  115540,  4, }, // Africa/El_Aaiun
    {  377, 6853,  115544,  4, }, // Africa/Freetown
    {  377, 4906,  115548,  4, }, // Africa/Gaborone
    {  377, 5015,  115552,  3, }, // Africa/Harare
    {  377, 4479,  115555,  6, }, // Africa/Johannesburg
    {  377, 4499,  115561,  2, }, // Africa/Juba
    {  377, 5151,  115563,  3, }, // Africa/Kampala
    {  377, 4524,  115566,  3, }, // Africa/Khartoum
    {  377, 4973,  115569,  3, }, // Africa/Kigali
    {  377, 5306,  115572,  3, }, // Africa/Kinshasa
    {  377, 4744,  115575,  3, }, // Africa/Lagos
    {  377, 5336,  115578,  4, }, // Africa/Libreville
    {  377, 6869,  115582,  2, }, // Africa/Lome
    {  377, 5211,  115584,  3, }, // Africa/Luanda
    {  377, 4939,  115587,  4, }, // Africa/Lubumbashi
    {  377, 5001,  115591,  3, }, // Africa/Lusaka
    {  377, 5322,  115594,  3, }, // Africa/Malabo
    {  377, 4987,  115597,  3, }, // Africa/Maputo
    {  377, 5197,  115600,  3, }, // Africa/Maseru
    {  377, 5182,  115603,  4, }, // Africa/Mbabane
    {  377, 5113,  115607,  4, }, // Africa/Mogadishu
    {  377, 7034,  115611,  4, }, // Africa/Monrovia
    {  377, 3853,  115615,  4, }, // Africa/Nairobi
    {  377, 5271,  115619,  4, }, // Africa/Ndjamena
    {  377, 5354,  115623,  3, }, // Africa/Niamey
    {  377, 6803,  115626,  4, }, // Africa/Nouakchott
    {  377, 6727,  115630,  4, }, // Africa/Ouagadougou
    {  377, 5225,  115634,  5, }, // Africa/Porto-Novo
    {  377, 4435,  115639,  3, }, // Africa/Sao_Tome
    {  377, 2866,  115642,  4, }, // Africa/Tripoli
    {  377, 6554,  115646,  3, }, // Africa/Tunis
    {  377, 4152,  115649,  4, }, // Africa/Windhoek
    {  377,  132,  115653,  3, }, // America/Adak
    {  377, 3346,  115656,  4, }, // America/Anchorage
    {  377, 5607,  115660,  3, }, // America/Anguilla
    {  377, 5624,  115663,  3, }, // America/Antigua
    {  377, 4554,  115666,  5, }, // America/Araguaina
    {  377,  166,  115671,  9, }, // America/Argentina/Buenos_Aires
    {  377,   91,  115680,  5, }, // America/Argentina/Catamarca
    {  377,  270,  115685,  4, }, // America/Argentina/Cordoba
    {  377,  441,  115689,  3, }, // America/Argentina/Jujuy
    {  377, 7462,  115692,  5, }, // America/Argentina/La_Rioja
    {  377,  565,  115697,  3, }, // America/Argentina/Mendoza
    {  377, 7489,  115700,  7, }, // America/Argentina/Rio_Gallegos
    {  377, 7520,  115707,  2, }, // America/Argentina/Salta
    {  377, 7544,  115709,  3, }, // America/Argentina/San_Juan
    {  377, 5580,  115712,  4, }, // America/Argentina/San_Luis
    {  377, 7571,  115716,  3, }, // America/Argentina/Tucuman
    {  377, 7597,  115719,  5, }, // America/Argentina/Ushuaia
    {  377, 5640,  115724,  3, }, // America/Aruba
    {  377, 4252,  115727,  4, }, // America/Asuncion
    {  377,  237,  115731,  4, }, // America/Atikokan
    {  377, 3679,  115735,  3, }, // America/Bahia
    {  377, 7623,  115738,  8, }, // America/Bahia_Banderas
    {  377, 5654,  115746,  5, }, // America/Barbados
    {  377, 7646,  115751,  2, }, // America/Belem
    {  377, 5407,  115753,  3, }, // America/Belize
    {  377, 7660,  115756,  6, }, // America/Blanc-Sablon
    {  377, 7681,  115762,  6, }, // America/Boa_Vista
    {  377, 4361,  115768,  3, }, // America/Bogota
    {  377, 7699,  115771,  3, }, // America/Boise
    {  377, 7713,  115774,  8, }, // America/Cambridge_Bay
    {  377, 7735,  115782,  6, }, // America/Campo_Grande
    {  377, 3868,  115788,  2, }, // America/Cancun
    {  377, 4694,  115790,  4, }, // America/Caracas
    {  377, 4345,  115794,  2, }, // America/Cayenne
    {  377, 5496,  115796,  3, }, // America/Cayman
    {  377, 2260,  115799,  3, }, // America/Chicago
    {  377, 7756,  115802,  3, }, // America/Chihuahua
    {  377, 7774,  115805,  8, }, // America/Ciudad_Juarez
    {  377, 5422,  115813,  5, }, // America/Costa_Rica
    {  377, 7796,  115818,  4, }, // America/Creston
    {  377, 3791,  115822,  4, }, // America/Cuiaba
    {  377, 5723,  115826,  3, }, // America/Curacao
    {  377, 7812,  115829,  5, }, // America/Danmarkshavn
    {  377, 7833,  115834,  2, }, // America/Dawson
    {  377, 7848,  115836,  4, }, // America/Dawson_Creek
    {  377,  805,  115840,  2, }, // America/Denver
    {  377, 3465,  115842,  5, }, // America/Detroit
    {  377, 5739,  115847,  4, }, // America/Dominica
    {  377,  893,  115851,  4, }, // America/Edmonton
    {  377, 7869,  115855,  5, }, // America/Eirunepe
    {  377, 5441,  115860,  5, }, // America/El_Salvador
    {  377, 7886,  115865,  5, }, // America/Fort_Nelson
    {  377, 7906,  115870,  5, }, // America/Fortaleza
    {  377, 7924,  115875,  5, }, // America/Glace_Bay
    {  377, 6881,  115880,  4, }, // America/Goose_Bay
    {  377, 4612,  115884,  6, }, // America/Grand_Turk
    {  377, 5770,  115890,  4, }, // America/Grenada
    {  377, 5786,  115894,  4, }, // America/Guadeloupe
    {  377, 3760,  115898,  4, }, // America/Guatemala
    {  377, 4841,  115902,  3, }, // America/Guayaquil
    {  377, 6932,  115905,  4, }, // America/Guyana
    {  377, 1939,  115909,  4, }, // America/Halifax
    {  377, 2281,  115913,  3, }, // America/Havana
    {  377, 7942,  115916,  5, }, // America/Hermosillo
    {  377,  348,  115921,  7, }, // America/Indiana/Indianapolis
    {  377,  481,  115928,  9, }, // America/Indiana/Knox
    {  377, 7961,  115937, 10, }, // America/Indiana/Marengo
    {  377, 7985,  115947, 11, }, // America/Indiana/Petersburg
    {  377, 8012,  115958, 10, }, // America/Indiana/Tell_City
    {  377, 8038,  115968,  9, }, // America/Indiana/Vevay
    {  377, 8060,  115977, 10, }, // America/Indiana/Vincennes
    {  377, 8086,  115987, 10, }, // America/Indiana/Winamac
    {  377, 8110,  115997,  3, }, // America/Inuvik
    {  377,  660,  116000,  5, }, // America/Iqaluit
    {  377, 2799,  116005,  4, }, // America/Jamaica
    {  377, 5380,  116009,  2, }, // America/Juneau
    {  377,  521,  116011,  3, }, // America/Kentucky/Louisville
    {  377, 8125,  116014, 10, }, // America/Kentucky/Monticello
    {  377, 5704,  116024,  6, }, // America/Kralendijk
    {  377, 4376,  116030,  3, }, // America/La_Paz
    {  377, 7169,  116033,  2, }, // America/Lima
    {  377, 3239,  116035,  6, }, // America/Los_Angeles
    {  377, 5932,  116041,  9, }, // America/Lower_Princes
    {  377, 8153,  116050,  4, }, // America/Maceio
    {  377, 8168,  116054,  3, }, // America/Managua
    {  377, 1908,  116057,  4, }, // America/Manaus
    {  377, 5897,  116061,  3, }, // America/Marigot
    {  377, 5805,  116064,  4, }, // America/Martinique
    {  377, 8184,  116068,  5, }, // America/Matamoros
    {  377, 2917,  116073,  4, }, // America/Mazatlan
    {  377, 8202,  116077,  4, }, // America/Menominee
    {  377, 8220,  116081,  3, }, // America/Merida
    {  377, 8235,  116084,  6, }, // America/Metlakatla
    {  377, 2949,  116090,  6, }, // America/Mexico_City
    {  377, 4391,  116096,  3, }, // America/Miquelon
    {  377, 8254,  116099,  2, }, // America/Moncton
    {  377, 8270,  116101,  4, }, // America/Monterrey
    {  377, 4098,  116105,  5, }, // America/Montevideo
    {  377, 5824,  116110,  4, }, // America/Montserrat
    {  377, 5481,  116114,  2, }, // America/Nassau
    {  377, 2379,  116116,  2, }, // America/New_York
    {  377, 8288,  116118,  1, }, // America/Nome
    {  377, 1850,  116119,  3, }, // America/Noronha
    {  377, 8301,  116122, 10, }, // America/North_Dakota/Beulah
    {  377, 8329,  116132,  9, }, // America/North_Dakota/Center
    {  377, 8357,  116141, 11, }, // America/North_Dakota/New_Salem
    {  377,  393,  116152,  4, }, // America/Nuuk
    {  377, 8388,  116156,  4, }, // America/Ojinaga
    {  377, 2356,  116160,  3, }, // America/Panama
    {  377, 6173,  116163,  5, }, // America/Paramaribo
    {  377, 2973,  116168,  3, }, // America/Phoenix
    {  377, 3945,  116171,  6, }, // America/Port-au-Prince
    {  377, 5954,  116177,  7, }, // America/Port_of_Spain
    {  377, 8404,  116184,  5, }, // America/Porto_Velho
    {  377, 5843,  116189,  6, }, // America/Puerto_Rico
    {  377, 4030,  116195,  6, }, // America/Punta_Arenas
    {  377, 8424,  116201,  5, }, // America/Rankin_Inlet
    {  377, 8445,  116206,  3, }, // America/Recife
    {  377, 1995,  116209,  4, }, // America/Regina
    {  377, 8460,  116213,  4, }, // America/Resolute
    {  377,  695,  116217,  6, }, // America/Rio_Branco
    {  377, 8477,  116223,  3, }, // America/Santarem
    {  377, 2201,  116226,  4, }, // America/Santiago
    {  377, 6111,  116230,  5, }, // America/Santo_Domingo
    {  377, 1878,  116235,  4, }, // America/Sao_Paulo
    {  377, 6899,  116239,  8, }, // America/Scoresbysund
    {  377, 8494,  116247,  2, }, // America/Sitka
    {  377, 8508,  116249,  6, }, // America/St_Barthelemy
    {  377, 2061,  116255,  5, }, // America/St_Johns
    {  377, 5863,  116260,  5, }, // America/St_Kitts
    {  377, 5880,  116265,  6, }, // America/St_Lucia
    {  377,  855,  116271,  6, }, // America/St_Thomas
    {  377, 5913,  116277,  6, }, // America/St_Vincent
    {  377, 8530,  116283,  7, }, // America/Swift_Current
    {  377, 5461,  116290,  5, }, // America/Tegucigalpa
    {  377, 5756,  116295,  2, }, // America/Thule
    {  377,  313,  116297,  4, }, // America/Tijuana
    {  377,  608,  116301,  3, }, // America/Toronto
    {  377, 5688,  116304,  4, }, // America/Tortola
    {  377, 2093,  116308,  3, }, // America/Vancouver
    {  377, 2144,  116311,  5, }, // America/Whitehorse
    {  377,  734,  116316,  3, }, // America/Winnipeg
    {  377, 8552,  116319,  4, }, // America/Yakutat
    {  377, 6001,  116323,  3, }, // Antarctica/Casey
    {  377, 6094,  116326,  4, }, // Antarctica/Davis
    {  377, 6133,  116330,  6, }, // Antarctica/DumontDUrville
    {  377, 8568,  116336,  3, }, // Antarctica/Macquarie
    {  377, 7066,  116339,  2, }, // Antarctica/Mawson
    {  377, 7113,  116341,  3, }, // Antarctica/McMurdo
    {  377, 8589,  116344,  2, }, // Antarctica/Palmer
    {  377, 7226,  116346,  3, }, // Antarctica/Rothera
    {  377, 7295,  116349,  2, }, // Antarctica/Syowa
    {  377, 8607,  116351,  2, }, // Antarctica/Troll
    {  377, 7374,  116353,  4, }, // Antarctica/Vostok
    {  377, 1429,  116357,  5, }, // Arctic/Longyearbyen
    {  377, 5570,  116362,  2, }, // Asia/Aden
    {  377, 5395,  116364,  3, }, // Asia/Almaty
    {  377, 3968,  116367,  2, }, // Asia/Amman
    {  377, 5511,  116369,  4, }, // Asia/Anadyr
    {  377, 5523,  116373,  4, }, // Asia/Aqtau
    {  377, 5368,  116377,  3, }, // Asia/Aqtobe
    {  377,  964,  116380,  5, }, // Asia/Ashgabat
    {  377, 8624,  116385,  4, }, // Asia/Atyrau
    {  377, 3607,  116389,  4, }, // Asia/Baghdad
    {  377, 5534,  116393,  3, }, // Asia/Bahrain
    {  377, 3653,  116396,  2, }, // Asia/Baku
    {  377, 4466,  116398,  2, }, // Asia/Bangkok
    {  377, 3571,  116400,  4, }, // Asia/Barnaul
    {  377, 4086,  116404,  4, }, // Asia/Beirut
    {  377, 3778,  116408,  4, }, // Asia/Bishkek
    {  377, 5989,  116412,  4, }, // Asia/Brunei
    {  377, 4601,  116416,  2, }, // Asia/Chita
    {  377, 4511,  116418,  3, }, // Asia/Colombo
    {  377, 4540,  116421,  5, }, // Asia/Damascus
    {  377, 1093,  115525,  2, }, // Asia/Dhaka
    {  377, 6192,  116426,  2, }, // Asia/Dili
    {  377, 3596,  116428,  3, }, // Asia/Dubai
    {  377, 6159,  116431,  3, }, // Asia/Dushanbe
    {  377, 8636,  116434,  5, }, // Asia/Famagusta
    {  377, 8651,  116439,  2, }, // Asia/Gaza
    {  377, 4795,  116441,  3, }, // Asia/Hebron
    {  377, 1259,  116444,  3, }, // Asia/Ho_Chi_Minh
    {  377, 2704,  116447,  2, }, // Asia/Hong_Kong
    {  377, 4771,  116449,  3, }, // Asia/Hovd
    {  377, 4184,  116452,  5, }, // Asia/Irkutsk
    {  377, 7006,  116457,  4, }, // Asia/Jakarta
    {  377, 6992,  116461,  4, }, // Asia/Jayapura
    {  377, 1290,  116465,  4, }, // Asia/Jerusalem
    {  377, 3560,  116469,  2, }, // Asia/Kabul
    {  377, 4316,  116471,  3, }, // Asia/Kamchatka
    {  377, 4239,  116474,  3, }, // Asia/Karachi
    {  377, 1185,  116477,  4, }, // Asia/Kathmandu
    {  377, 8661,  116481,  3, }, // Asia/Khandyga
    {  377,  992,  116484,  3, }, // Asia/Kolkata
    {  377, 4197,  116487,  8, }, // Asia/Krasnoyarsk
    {  377, 4859,  116495,  6, }, // Asia/Kuala_Lumpur
    {  377, 5976,  116501,  2, }, // Asia/Kuching
    {  377, 5547,  116503,  4, }, // Asia/Kuwait
    {  377, 1211,  116507,  3, }, // Asia/Macau
    {  377, 4017,  116510,  3, }, // Asia/Magadan
    {  377, 1349,  116513,  4, }, // Asia/Makassar
    {  377, 7182,  116517,  3, }, // Asia/Manila
    {  377, 6920,  116520,  4, }, // Asia/Muscat
    {  377, 2554,  116524,  4, }, // Asia/Nicosia
    {  377, 8675,  116528,  7, }, // Asia/Novokuznetsk
    {  377, 4135,  116535,  7, }, // Asia/Novosibirsk
    {  377, 4229,  116542,  3, }, // Asia/Omsk
    {  377, 7145,  116545,  2, }, // Asia/Oral
    {  377, 6961,  116547,  3, }, // Asia/Phnom_Penh
    {  377, 8693,  116550,  5, }, // Asia/Pontianak
    {  377, 4214,  116555,  2, }, // Asia/Pyongyang
    {  377, 5559,  116557,  3, }, // Asia/Qatar
    {  377, 8708,  116560,  5, }, // Asia/Qostanay
    {  377, 4269,  116565,  5, }, // Asia/Qyzylorda
    {  377, 3584,  116570,  3, }, // Asia/Riyadh
    {  377, 4408,  116573,  3, }, // Asia/Sakhalin
    {  377, 7245,  116576,  5, }, // Asia/Samarkand
    {  377, 3279,  116581,  2, }, // Asia/Seoul
    {  377, 1053,  116583,  3, }, // Asia/Shanghai
    {  377, 3300,  116586,  4, }, // Asia/Singapore
    {  377, 4297,  116590,  8, }, // Asia/Srednekolymsk
    {  377, 3263,  116598,  4, }, // Asia/Taipei
    {  377, 4781,  116602,  4, }, // Asia/Tashkent
    {  377, 3915,  116606,  4, }, // Asia/Tbilisi
    {  377, 2772,  116610,  3, }, // Asia/Tehran
    {  377, 1317,  116613,  2, }, // Asia/Thimphu
    {  377, 2821,  116615,  2, }, // Asia/Tokyo
    {  377, 4572,  116617,  3, }, // Asia/Tomsk
    {  377, 1021,  116620,  5, }, // Asia/Ulaanbaatar
    {  377, 1159,  116625,  4, }, // Asia/Urumqi
    {  377, 8722,  116629,  5, }, // Asia/Ust-Nera
    {  377, 6977,  116634,  4, }, // Asia/Vientiane
    {  377, 4710,  116638,  7, }, // Asia/Vladivostok
    {  377, 4828,  116645,  4, }, // Asia/Yakutsk
    {  377, 1235,  116649,  2, }, // Asia/Yangon
    {  377, 3883,  116651,  7, }, // Asia/Yekaterinburg
    {  377, 3747,  116658,  3, }, // Asia/Yerevan
    {  377, 3663,  116661,  4, }, // Atlantic/Azores
    {  377, 5671,  116665,  3, }, // Atlantic/Bermuda
    {  377, 6628,  116668,  7, }, // Atlantic/Canary
    {  377, 3727,  116675,  6, }, // Atlantic/Cape_Verde
    {  377, 1395,  116681,  5, }, // Atlantic/Faroe
    {  377, 8736,  116686,  4, }, // Atlantic/Madeira
    {  377, 2748,  116690,  5, }, // Atlantic/Reykjavik
    {  377, 7272,  116695,  6, }, // Atlantic/South_Georgia
    {  377, 6821,  116701,  7, }, // Atlantic/St_Helena
    {  377, 6644,  116708,  3, }, // Atlantic/Stanley
    {  377, 1670,  116711,  5, }, // Australia/Adelaide
    {  377, 1635,  116716,  5, }, // Australia/Brisbane
    {  377, 1799,  116721,  4, }, // Australia/Broken_Hill
    {  377, 1583,  116725,  2, }, // Australia/Darwin
    {  377, 3637,  116727,  3, }, // Australia/Eucla
    {  377, 1516,  116730,  3, }, // Australia/Hobart
    {  377, 8753,  116733,  3, }, // Australia/Lindeman
    {  377, 1547,  116736,  5, }, // Australia/Lord_Howe
    {  377, 1727,  116741,  3, }, // Australia/Melbourne
    {  377, 1762,  116744,  2, }, // Australia/Perth
    {  377, 1463,  116746,  3, }, // Australia/Sydney
    {  377, 6441,  116749,  5, }, // Europe/Amsterdam
    {  377, 6216,  116754,  3, }, // Europe/Andorra
    {  377, 3620,  116757,  5, }, // Europe/Astrakhan
    {  377, 2300,  116762,  3, }, // Europe/Athens
    {  377, 6336,  116765,  5, }, // Europe/Belgrade
    {  377, 4757,  116770,  3, }, // Europe/Berlin
    {  377, 6488,  116773,  6, }, // Europe/Bratislava
    {  377, 2167,  116779,  3, }, // Europe/Brussels
    {  377, 3928,  116782,  5, }, // Europe/Bucharest
    {  377, 3806,  116787,  5, }, // Europe/Budapest
    {  377, 8772,  116792,  3, }, // Europe/Busingen
    {  377, 2583,  116795,  4, }, // Europe/Chisinau
    {  377, 6289,  116799,  4, }, // Europe/Copenhagen
    {  377, 2338,  116803,  3, }, // Europe/Dublin
    {  377, 6307,  116806,  4, }, // Europe/Gibraltar
    {  377, 8788,  116810,  2, }, // Europe/Guernsey
    {  377, 6612,  116812,  3, }, // Europe/Helsinki
    {  377, 8804,  116815,  2, }, // Europe/Isle_of_Man
    {  377, 1130,  116817,  4, }, // Europe/Istanbul
    {  377, 8823,  116821,  2, }, // Europe/Jersey
    {  377, 3979,  116823,  6, }, // Europe/Kaliningrad
    {  377, 8929,  116829,  3, }, // Europe/Kirov
    {  377, 2527,  116832,  3, }, // Europe/Kyiv
    {  377, 3213,  116835,  3, }, // Europe/Lisbon
    {  377, 6506,  116838,  4, }, // Europe/Ljubljana
    {  377, 2501,  116842,  2, }, // Europe/London
    {  377, 6365,  116844,  5, }, // Europe/Luxembourg
    {  377, 4892,  116849,  4, }, // Europe/Madrid
    {  377, 6397,  116853,  2, }, // Europe/Malta
    {  377, 6582,  116855,  4, }, // Europe/Mariehamn
    {  377, 3693,  116859,  3, }, // Europe/Minsk
    {  377, 6410,  116862,  3, }, // Europe/Monaco
    {  377, 3537,  116865,  4, }, // Europe/Moscow
    {  377, 6458,  116869,  3, }, // Europe/Oslo
    {  377, 4284,  116872,  2, }, // Europe/Paris
    {  377, 6424,  116874,  5, }, // Europe/Podgorica
    {  377, 6275,  116879,  3, }, // Europe/Prague
    {  377, 8837,  116882,  2, }, // Europe/Riga
    {  377, 6324,  116884,  2, }, // Europe/Rome
    {  377, 4331,  116886,  3, }, // Europe/Samara
    {  377, 6470,  116889,  4, }, // Europe/San_Marino
    {  377, 6245,  116893,  4, }, // Europe/Sarajevo
    {  377, 4451,  116897,  4, }, // Europe/Saratov
    {  377, 8849,  116901,  4, }, // Europe/Simferopol
    {  377, 6383,  116905,  3, }, // Europe/Skopje
    {  377, 6599,  116908,  3, }, // Europe/Sofia
    {  377, 6523,  116911,  4, }, // Europe/Stockholm
    {  377, 8867,  116915,  2, }, // Europe/Tallinn
    {  377, 6202,  116917,  3, }, // Europe/Tirane
    {  377, 8882,  116920,  7, }, // Europe/Ulyanovsk
    {  377, 6352,  116927,  3, }, // Europe/Vaduz
    {  377, 6567,  116930,  3, }, // Europe/Vatican
    {  377, 6231,  116933,  3, }, // Europe/Vienna
    {  377, 8899,  116936,  4, }, // Europe/Vilnius
    {  377, 4727,  116940,  5, }, // Europe/Volgograd
    {  377, 3190,  116945,  4, }, // Europe/Warsaw
    {  377, 6261,  116949,  4, }, // Europe/Zagreb
    {  377, 6540,  116953,  3, }, // Europe/Zurich
    {  377, 5078,  116956,  6, }, // Indian/Antananarivo
    {  377, 6947,  116962,  3, }, // Indian/Chagos
    {  377, 6046,  116965,  5, }, // Indian/Christmas
    {  377, 6063,  116970,  3, }, // Indian/Cocos
    {  377, 5029,  116973,  3, }, // Indian/Comoro
    {  377, 6661,  116976,  4, }, // Indian/Kerguelen
    {  377, 7260,  116980,  2, }, // Indian/Mahe
    {  377, 7050,  116982,  3, }, // Indian/Maldives
    {  377, 4069,  116985,  4, }, // Indian/Mauritius
    {  377, 5098,  116989,  3, }, // Indian/Mayotte
    {  377, 7211,  116992,  4, }, // Indian/Reunion
    {  377, 4422,  116996,  3, }, // Pacific/Apia
    {  377,  932,  116999,  4, }, // Pacific/Auckland
    {  377, 3706,  117003,  3, }, // Pacific/Bougainville
    {  377, 3015,  117006,  2, }, // Pacific/Chatham
    {  377, 3157,  117008,  3, }, // Pacific/Chuuk
    {  377, 2237,  117011,  5, }, // Pacific/Easter
    {  377, 7360,  117016,  3, }, // Pacific/Efate
    {  377, 7327,  117019,  4, }, // Pacific/Fakaofo
    {  377, 3902,  117023,  2, }, // Pacific/Fiji
    {  377, 7343,  117025,  4, }, // Pacific/Funafuti
    {  377, 6678,  117029,  5, }, // Pacific/Galapagos
    {  377, 6696,  117034,  3, }, // Pacific/Gambier
    {  377, 3822,  117037,  4, }, // Pacific/Guadalcanal
    {  377, 6018,  117041,  1, }, // Pacific/Guam
    {  377, 2723,  117042,  4, }, // Pacific/Honolulu
    {  377, 3049,  117046,  4, }, // Pacific/Kanton
    {  377, 3998,  117050,  5, }, // Pacific/Kiritimati
    {  377, 7019,  117055,  3, }, // Pacific/Kosrae
    {  377, 2842,  117058,  3, }, // Pacific/Kwajalein
    {  377, 4877,  117061,  3, }, // Pacific/Majuro
    {  377, 4051,  117064,  4, }, // Pacific/Marquesas
    {  377, 8914,  117068,  4, }, // Pacific/Midway
    {  377, 7084,  117072,  3, }, // Pacific/Nauru
    {  377, 7132,  117075,  3, }, // Pacific/Niue
    {  377, 4168,  117078,  2, }, // Pacific/Norfolk
    {  377, 7098,  117080,  3, }, // Pacific/Noumea
    {  377, 3126,  117083,  4, }, // Pacific/Pago_Pago
    {  377, 7155,  117087,  3, }, // Pacific/Palau
    {  377, 7194,  117090,  3, }, // Pacific/Pitcairn
    {  377, 3096,  117093,  3, }, // Pacific/Pohnpei
    {  377, 4807,  117096,  6, }, // Pacific/Port_Moresby
    {  377, 6076,  117102,  4, }, // Pacific/Rarotonga
    {  377, 6031,  117106,  3, }, // Pacific/Saipan
    {  377, 7312,  117109,  3, }, // Pacific/Tahiti
    {  377, 6712,  117112,  3, }, // Pacific/Tarawa
    {  377, 4583,  117115,  4, }, // Pacific/Tongatapu
    {  377, 7392,  117119,  3, }, // Pacific/Wake
    {  377, 7405,  117122,  3, }, // Pacific/Wallis
    {  380,   14,       0,  6, }, // Africa/Asmara Koyraboro Senni/Latin/Mali
    {  380, 4435,       6,  8, }, // Africa/Sao_Tome
    {  380, 4252,      14,  8, }, // America/Asuncion
    {  380,  237,      22,  8, }, // America/Atikokan
    {  380, 7623,      30, 17, }, // America/Bahia_Banderas
    {  380, 3868,      47,  6, }, // America/Cancun
    {  380, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  380, 5723,      66,  7, }, // America/Curacao
    {  380,  481,      73, 13, }, // America/Indiana/Knox
    {  380, 7961,      86, 16, }, // America/Indiana/Marengo
    {  380, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  380, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  380, 8038,     139, 14, }, // America/Indiana/Vevay
    {  380, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  380, 8086,     171, 16, }, // America/Indiana/Winamac
    {  380, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  380, 5932,     207, 22, }, // America/Lower_Princes
    {  380, 8220,     229,  6, }, // America/Merida
    {  380, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  380, 8329,     255, 20, }, // America/North_Dakota/Center
    {  380, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  380,  393,     298,  4, }, // America/Nuuk
    {  380, 6899,     302, 16, }, // America/Scoresbysund
    {  380, 8508,     318, 14, }, // America/St_Barthelemy
    {  380, 2061,     332, 10, }, // America/St_Johns
    {  380, 5863,     342,  9, }, // America/St_Kitts
    {  380, 5880,     351,  9, }, // America/St_Lucia
    {  380,  855,     360, 10, }, // America/St_Thomas
    {  380, 5913,     370, 11, }, // America/St_Vincent
    {  380, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  380, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  380, 1185,     413,  9, }, // Asia/Kathmandu
    {  380,  992,     422,  7, }, // Asia/Kolkata
    {  380, 1211,     429,  5, }, // Asia/Macau
    {  380, 1235,     442,  6, }, // Asia/Yangon
    {  380, 1395,     448,  5, }, // Atlantic/Faroe
    {  380, 6821,     453, 10, }, // Atlantic/St_Helena
    {  380, 2527,     463,  4, }, // Europe/Kyiv
    {  380, 7211,     467,  7, }, // Indian/Reunion
    {  380, 3157,     474,  5, }, // Pacific/Chuuk
    {  380, 3096,     479,  7, }, // Pacific/Pohnpei
    {  381,   14,       0,  6, }, // Africa/Asmara Koyra Chiini/Latin/Mali
    {  381, 4435,       6,  8, }, // Africa/Sao_Tome
    {  381, 4252,      14,  8, }, // America/Asuncion
    {  381,  237,      22,  8, }, // America/Atikokan
    {  381, 7623,      30, 17, }, // America/Bahia_Banderas
    {  381, 3868,      47,  6, }, // America/Cancun
    {  381, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  381, 5723,      66,  7, }, // America/Curacao
    {  381,  481,      73, 13, }, // America/Indiana/Knox
    {  381, 7961,      86, 16, }, // America/Indiana/Marengo
    {  381, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  381, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  381, 8038,     139, 14, }, // America/Indiana/Vevay
    {  381, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  381, 8086,     171, 16, }, // America/Indiana/Winamac
    {  381, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  381, 5932,     207, 22, }, // America/Lower_Princes
    {  381, 8220,     229,  6, }, // America/Merida
    {  381, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  381, 8329,     255, 20, }, // America/North_Dakota/Center
    {  381, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  381,  393,     298,  4, }, // America/Nuuk
    {  381, 6899,     302, 16, }, // America/Scoresbysund
    {  381, 8508,     318, 14, }, // America/St_Barthelemy
    {  381, 2061,     332, 10, }, // America/St_Johns
    {  381, 5863,     342,  9, }, // America/St_Kitts
    {  381, 5880,     351,  9, }, // America/St_Lucia
    {  381,  855,     360, 10, }, // America/St_Thomas
    {  381, 5913,     370, 11, }, // America/St_Vincent
    {  381, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  381, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  381, 1185,     413,  9, }, // Asia/Kathmandu
    {  381,  992,     422,  7, }, // Asia/Kolkata
    {  381, 1211,     429,  5, }, // Asia/Macau
    {  381, 1235,     442,  6, }, // Asia/Yangon
    {  381, 1395,     448,  5, }, // Atlantic/Faroe
    {  381, 6821,     453, 10, }, // Atlantic/St_Helena
    {  381, 2527,     463,  4, }, // Europe/Kyiv
    {  381, 7211,     467,  7, }, // Indian/Reunion
    {  381, 3157,     474,  5, }, // Pacific/Chuuk
    {  381, 3096,     479,  7, }, // Pacific/Pohnpei
    {  382,   14,       0,  6, }, // Africa/Asmara Kpelle/Latin/Liberia
    {  382, 4435,       6,  8, }, // Africa/Sao_Tome
    {  382, 4252,      14,  8, }, // America/Asuncion
    {  382,  237,      22,  8, }, // America/Atikokan
    {  382, 7623,      30, 17, }, // America/Bahia_Banderas
    {  382, 3868,      47,  6, }, // America/Cancun
    {  382, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  382, 5723,      66,  7, }, // America/Curacao
    {  382,  481,      73, 13, }, // America/Indiana/Knox
    {  382, 7961,      86, 16, }, // America/Indiana/Marengo
    {  382, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  382, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  382, 8038,     139, 14, }, // America/Indiana/Vevay
    {  382, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  382, 8086,     171, 16, }, // America/Indiana/Winamac
    {  382, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  382, 5932,     207, 22, }, // America/Lower_Princes
    {  382, 8220,     229,  6, }, // America/Merida
    {  382, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  382, 8329,     255, 20, }, // America/North_Dakota/Center
    {  382, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  382,  393,     298,  4, }, // America/Nuuk
    {  382, 6899,     302, 16, }, // America/Scoresbysund
    {  382, 8508,     318, 14, }, // America/St_Barthelemy
    {  382, 2061,     332, 10, }, // America/St_Johns
    {  382, 5863,     342,  9, }, // America/St_Kitts
    {  382, 5880,     351,  9, }, // America/St_Lucia
    {  382,  855,     360, 10, }, // America/St_Thomas
    {  382, 5913,     370, 11, }, // America/St_Vincent
    {  382, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  382, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  382, 1185,     413,  9, }, // Asia/Kathmandu
    {  382,  992,     422,  7, }, // Asia/Kolkata
    {  382, 1211,     429,  5, }, // Asia/Macau
    {  382, 1235,     442,  6, }, // Asia/Yangon
    {  382, 1395,     448,  5, }, // Atlantic/Faroe
    {  382, 6821,     453, 10, }, // Atlantic/St_Helena
    {  382, 2527,     463,  4, }, // Europe/Kyiv
    {  382, 7211,     467,  7, }, // Indian/Reunion
    {  382, 3157,     474,  5, }, // Pacific/Chuuk
    {  382, 3096,     479,  7, }, // Pacific/Pohnpei
    {  384, 6760,    4531,  4, }, // Africa/Accra Kurdish/Latin/Turkey
    {  384, 7420,  117125,  7, }, // Africa/Algiers
    {  384,   14,       0,  6, }, // Africa/Asmara
    {  384, 2320,  117132,  6, }, // Africa/Cairo
    {  384, 4117,   33144, 10, }, // Africa/Casablanca
    {  384, 7449,  117138,  5, }, // Africa/Ceuta
    {  384, 6773,  117143,  7, }, // Africa/Conakry
    {  384, 5130,  117150,  9, }, // Africa/Dar_es_Salaam
    {  384, 5043,  117159,  6, }, // Africa/Djibouti
    {  384, 4524,  117165,  6, }, // Africa/Khartoum
    {  384, 4973,  117171,  6, }, // Africa/Kigali
    {  384, 5306,  117177,  7, }, // Africa/Kinshasa
    {  384, 5336,  117184,  8, }, // Africa/Libreville
    {  384, 5197,  117192,  6, }, // Africa/Maseru
    {  384, 5113,  117198,  8, }, // Africa/Mogadishu
    {  384, 3853,  117206,  7, }, // Africa/Nairobi
    {  384, 4435,       6,  8, }, // Africa/Sao_Tome
    {  384, 2866,  117213,  7, }, // Africa/Tripoli
    {  384, 6554,  117220,  5, }, // Africa/Tunis
    {  384, 5640,  117225,  5, }, // America/Aruba
    {  384, 4252,      14,  8, }, // America/Asuncion
    {  384,  237,      22,  8, }, // America/Atikokan
    {  384, 7623,  117230, 14, }, // America/Bahia_Banderas
    {  384, 5407,  117244,  6, }, // America/Belize
    {  384, 3868,  117250,  6, }, // America/Cancun
    {  384, 7774,  117256, 13, }, // America/Ciudad_Juarez
    {  384, 5422,  117269, 10, }, // America/Costa_Rica
    {  384, 5723,      66,  7, }, // America/Curacao
    {  384, 5739,  117279,  8, }, // America/Dominica
    {  384,  481,      73, 13, }, // America/Indiana/Knox
    {  384, 7961,      86, 16, }, // America/Indiana/Marengo
    {  384, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  384, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  384, 8038,     139, 14, }, // America/Indiana/Vevay
    {  384, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  384, 8086,     171, 16, }, // America/Indiana/Winamac
    {  384, 2799,  117287,  7, }, // America/Jamaica
    {  384, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  384, 5932,     207, 22, }, // America/Lower_Princes
    {  384, 8220,  117294,  6, }, // America/Merida
    {  384, 8301,  117300, 22, }, // America/North_Dakota/Beulah
    {  384, 8329,  117322, 22, }, // America/North_Dakota/Center
    {  384, 8357,  117344, 25, }, // America/North_Dakota/New_Salem
    {  384,  393,     298,  4, }, // America/Nuuk
    {  384, 5843,  117369, 10, }, // America/Puerto_Rico
    {  384, 6899,     302, 16, }, // America/Scoresbysund
    {  384, 8508,  117379, 16, }, // America/St_Barthelemy
    {  384, 2061,     332, 10, }, // America/St_Johns
    {  384, 5863,     342,  9, }, // America/St_Kitts
    {  384, 5880,     351,  9, }, // America/St_Lucia
    {  384,  855,     360, 10, }, // America/St_Thomas
    {  384, 5913,     370, 11, }, // America/St_Vincent
    {  384, 6094,  117395,  5, }, // Antarctica/Davis
    {  384, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  384, 5395,  117400,  6, }, // Asia/Almaty
    {  384, 3968,  117406,  4, }, // Asia/Amman
    {  384, 5511,    7482,  6, }, // Asia/Anadyr
    {  384, 5523,   64179,  5, }, // Asia/Aqtau
    {  384,  964,  117410,  7, }, // Asia/Ashgabat
    {  384, 8624,  117417,  6, }, // Asia/Atyrau
    {  384, 3607,  117423,  5, }, // Asia/Baghdad
    {  384, 5534,  117428,  7, }, // Asia/Bahrain
    {  384, 3653,  117435,  4, }, // Asia/Baku
    {  384, 4086,  117439,  6, }, // Asia/Beirut
    {  384, 3778,  117445,  6, }, // Asia/Bishkek
    {  384, 5989,  117451,  6, }, // Asia/Brunei
    {  384, 4601,  117457,  4, }, // Asia/Chita
    {  384, 4511,    5519,  7, }, // Asia/Colombo
    {  384, 4540,  117461,  3, }, // Asia/Damascus
    {  384, 1093,    4599,  4, }, // Asia/Dhaka
    {  384, 6192,  117464,  4, }, // Asia/Dili
    {  384, 3596,  117468,  5, }, // Asia/Dubai
    {  384, 6159,  117473,  7, }, // Asia/Dushanbe
    {  384, 8651,  117480,  4, }, // Asia/Gaza
    {  384, 1259,  117484, 19, }, // Asia/Ho_Chi_Minh
    {  384, 7006,   22308,  7, }, // Asia/Jakarta
    {  384, 6992,  117503,  8, }, // Asia/Jayapura
    {  384, 1290,  117511,  4, }, // Asia/Jerusalem
    {  384, 3560,  117515,  5, }, // Asia/Kabul
    {  384, 4316,   22337,  8, }, // Asia/Kamchatka
    {  384, 4239,  117520,  6, }, // Asia/Karachi
    {  384, 1185,  117526,  8, }, // Asia/Kathmandu
    {  384, 8661,  117534,  7, }, // Asia/Khandyga
    {  384,  992,     422,  7, }, // Asia/Kolkata
    {  384, 4859,  117541, 12, }, // Asia/Kuala_Lumpur
    {  384, 5976,  117553,  6, }, // Asia/Kuching
    {  384, 5547,  117559,  6, }, // Asia/Kuwait
    {  384, 1211,    7613,  5, }, // Asia/Macau
    {  384, 1349,    7618,  7, }, // Asia/Makassar
    {  384, 7182,  117565,  6, }, // Asia/Manila
    {  384, 6920,    4208,  6, }, // Asia/Muscat
    {  384, 2554,  117571,  7, }, // Asia/Nicosia
    {  384, 5559,  117578,  5, }, // Asia/Qatar
    {  384, 4269,  117583,  9, }, // Asia/Qyzylorda
    {  384, 3584,   20756,  5, }, // Asia/Riyadh
    {  384, 4408,  117592,  7, }, // Asia/Sakhalin
    {  384, 7245,  117599,  9, }, // Asia/Samarkand
    {  384, 3279,  117608,  4, }, // Asia/Seoul
    {  384, 1053,  117612,  7, }, // Asia/Shanghai
    {  384, 3300,  117619,  8, }, // Asia/Singapore
    {  384, 3263,  117627,  6, }, // Asia/Taipei
    {  384, 4781,  117633,  7, }, // Asia/Tashkent
    {  384, 3915,  117640,  6, }, // Asia/Tbilisi
    {  384, 1021,  117646,  9, }, // Asia/Ulaanbaatar
    {  384, 1159,  117655,  6, }, // Asia/Urumqi
    {  384, 8722,  117661,  8, }, // Asia/Ust-Nera
    {  384, 1235,     442,  6, }, // Asia/Yangon
    {  384, 3747,  117669,  5, }, // Asia/Yerevan
    {  384, 3663,  117674, 13, }, // Atlantic/Azores
    {  384, 5671,  117687,  7, }, // Atlantic/Bermuda
    {  384, 6628,  117694, 17, }, // Atlantic/Canary
    {  384, 3727,   57248,  9, }, // Atlantic/Cape_Verde
    {  384, 1395,     448,  5, }, // Atlantic/Faroe
    {  384, 7272,  117711, 15, }, // Atlantic/South_Georgia
    {  384, 6821,     453, 10, }, // Atlantic/St_Helena
    {  384, 1463,  117726,  6, }, // Australia/Sydney
    {  384, 6216,   33506,  6, }, // Europe/Andorra
    {  384, 3620,  117732,  8, }, // Europe/Astrakhan
    {  384, 2300,  117740,  5, }, // Europe/Athens
    {  384, 6336,   26883,  7, }, // Europe/Belgrade
    {  384, 4757,  117745,  6, }, // Europe/Berlin
    {  384, 2167,    7959,  7, }, // Europe/Brussels
    {  384, 3928,  117751,  6, }, // Europe/Bucharest
    {  384, 3806,  117757,  8, }, // Europe/Budapest
    {  384, 8772,  117765,  8, }, // Europe/Busingen
    {  384, 2583,  117773,  7, }, // Europe/Chisinau
    {  384, 6289,    4357,  8, }, // Europe/Copenhagen
    {  384, 2338,  117780,  6, }, // Europe/Dublin
    {  384, 6307,  117786, 11, }, // Europe/Gibraltar
    {  384, 6612,  117797,  8, }, // Europe/Helsinki
    {  384, 8804,  117805, 11, }, // Europe/Isle_of_Man
    {  384, 1130,  117816,  7, }, // Europe/Istanbul
    {  384, 3979,  117823, 11, }, // Europe/Kaliningrad
    {  384, 8929,  117834,  5, }, // Europe/Kirov
    {  384, 2527,  117839,  4, }, // Europe/Kyiv
    {  384, 3213,  117843,  6, }, // Europe/Lisbon
    {  384, 2501,   95241,  6, }, // Europe/London
    {  384, 6365,  117849, 10, }, // Europe/Luxembourg
    {  384, 4892,  117859,  6, }, // Europe/Madrid
    {  384, 3693,  117865,  5, }, // Europe/Minsk
    {  384, 6410,    5926,  6, }, // Europe/Monaco
    {  384, 3537,   64333,  7, }, // Europe/Moscow
    {  384, 4284,  117870,  5, }, // Europe/Paris
    {  384, 6424,  117875,  9, }, // Europe/Podgorica
    {  384, 6275,    8083,  4, }, // Europe/Prague
    {  384, 8837,  117884,  4, }, // Europe/Riga
    {  384, 6324,    5946,  4, }, // Europe/Rome
    {  384, 6470,  117888, 10, }, // Europe/San_Marino
    {  384, 6245,  117898, 10, }, // Europe/Sarajevo
    {  384, 6383,  117908,  5, }, // Europe/Skopje
    {  384, 6599,   99201,  5, }, // Europe/Sofia
    {  384, 6523,    8124,  8, }, // Europe/Stockholm
    {  384, 8867,  117913,  5, }, // Europe/Tallinn
    {  384, 6202,  117918,  5, }, // Europe/Tirane
    {  384, 6567,  117923,  7, }, // Europe/Vatican
    {  384, 6231,  117930,  6, }, // Europe/Vienna
    {  384, 8899,  117936,  6, }, // Europe/Vilnius
    {  384, 3190,  117942,  7, }, // Europe/Warsaw
    {  384, 6540,  117949,  5, }, // Europe/Zurich
    {  384, 5029,    8187,  5, }, // Indian/Comoro
    {  384, 7050,  117954,  6, }, // Indian/Maldives
    {  384, 7211,     467,  7, }, // Indian/Reunion
    {  384, 3157,     474,  5, }, // Pacific/Chuuk
    {  384, 3902,  117960,  4, }, // Pacific/Fiji
    {  384, 3096,     479,  7, }, // Pacific/Pohnpei
    {  384, 7312,  117964,  6, }, // Pacific/Tahiti
    {  385,   14,       0,  6, }, // Africa/Asmara Kwasio/Latin/Cameroon
    {  385, 4435,       6,  8, }, // Africa/Sao_Tome
    {  385, 4252,      14,  8, }, // America/Asuncion
    {  385,  237,      22,  8, }, // America/Atikokan
    {  385, 7623,      30, 17, }, // America/Bahia_Banderas
    {  385, 3868,      47,  6, }, // America/Cancun
    {  385, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  385, 5723,      66,  7, }, // America/Curacao
    {  385,  481,      73, 13, }, // America/Indiana/Knox
    {  385, 7961,      86, 16, }, // America/Indiana/Marengo
    {  385, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  385, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  385, 8038,     139, 14, }, // America/Indiana/Vevay
    {  385, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  385, 8086,     171, 16, }, // America/Indiana/Winamac
    {  385, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  385, 5932,     207, 22, }, // America/Lower_Princes
    {  385, 8220,     229,  6, }, // America/Merida
    {  385, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  385, 8329,     255, 20, }, // America/North_Dakota/Center
    {  385, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  385,  393,     298,  4, }, // America/Nuuk
    {  385, 6899,     302, 16, }, // America/Scoresbysund
    {  385, 8508,     318, 14, }, // America/St_Barthelemy
    {  385, 2061,     332, 10, }, // America/St_Johns
    {  385, 5863,     342,  9, }, // America/St_Kitts
    {  385, 5880,     351,  9, }, // America/St_Lucia
    {  385,  855,     360, 10, }, // America/St_Thomas
    {  385, 5913,     370, 11, }, // America/St_Vincent
    {  385, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  385, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  385, 1185,     413,  9, }, // Asia/Kathmandu
    {  385,  992,     422,  7, }, // Asia/Kolkata
    {  385, 1211,     429,  5, }, // Asia/Macau
    {  385, 1235,     442,  6, }, // Asia/Yangon
    {  385, 1395,     448,  5, }, // Atlantic/Faroe
    {  385, 6821,     453, 10, }, // Atlantic/St_Helena
    {  385, 2527,     463,  4, }, // Europe/Kyiv
    {  385, 7211,     467,  7, }, // Indian/Reunion
    {  385, 3157,     474,  5, }, // Pacific/Chuuk
    {  385, 3096,     479,  7, }, // Pacific/Pohnpei
    {  386, 6788,   36214,  7, }, // Africa/Abidjan Kyrgyz/Cyrillic/Kyrgyzstan
    {  386, 6760,     492,  5, }, // Africa/Accra
    {  386, 5059,     497, 11, }, // Africa/Addis_Ababa
    {  386, 7420,     508,  5, }, // Africa/Algiers
    {  386,   14,   27065,  6, }, // Africa/Asmara
    {  386,   44,     519,  6, }, // Africa/Bamako
    {  386, 5257,  117970,  4, }, // Africa/Bangui
    {  386, 6746,     530,  6, }, // Africa/Banjul
    {  386, 7435,     536,  5, }, // Africa/Bissau
    {  386, 4957,   27088,  8, }, // Africa/Blantyre
    {  386, 5287,  117974, 10, }, // Africa/Brazzaville
    {  386, 4922,     558,  9, }, // Africa/Bujumbura
    {  386, 2320,     567,  4, }, // Africa/Cairo
    {  386, 4117,     571, 10, }, // Africa/Casablanca
    {  386, 7449,     581,  5, }, // Africa/Ceuta
    {  386, 6773,     586,  7, }, // Africa/Conakry
    {  386, 6840,     593,  5, }, // Africa/Dakar
    {  386, 5130,  117984, 13, }, // Africa/Dar_es_Salaam
    {  386, 5043,  117997,  6, }, // Africa/Djibouti
    {  386, 5243,     616,  5, }, // Africa/Douala
    {  386, 5166,  118003,  7, }, // Africa/El_Aaiun
    {  386, 6853,     628,  7, }, // Africa/Freetown
    {  386, 4906,     635,  8, }, // Africa/Gaborone
    {  386, 5015,   33781,  6, }, // Africa/Harare
    {  386, 4479,   51440, 12, }, // Africa/Johannesburg
    {  386, 4499,  118010,  4, }, // Africa/Juba
    {  386, 5151,     665,  7, }, // Africa/Kampala
    {  386, 4524,   33798,  6, }, // Africa/Khartoum
    {  386, 4973,     678,  6, }, // Africa/Kigali
    {  386, 5306,     684,  7, }, // Africa/Kinshasa
    {  386, 4744,     691,  5, }, // Africa/Lagos
    {  386, 5336,  118014,  9, }, // Africa/Libreville
    {  386, 6869,     704,  4, }, // Africa/Lome
    {  386, 5211,     708,  6, }, // Africa/Luanda
    {  386, 4939,     714,  9, }, // Africa/Lubumbashi
    {  386, 5001,     723,  6, }, // Africa/Lusaka
    {  386, 5322,     729,  6, }, // Africa/Malabo
    {  386, 4987,   33813,  6, }, // Africa/Maputo
    {  386, 5197,     741,  6, }, // Africa/Maseru
    {  386, 5182,     747,  7, }, // Africa/Mbabane
    {  386, 5113,   36279,  8, }, // Africa/Mogadishu
    {  386, 7034,     762,  8, }, // Africa/Monrovia
    {  386, 3853,   36295,  7, }, // Africa/Nairobi
    {  386, 5271,  118023,  7, }, // Africa/Ndjamena
    {  386, 5354,  118030,  5, }, // Africa/Niamey
    {  386, 6803,     790,  7, }, // Africa/Nouakchott
    {  386, 6727,     797,  8, }, // Africa/Ouagadougou
    {  386, 5225,     805, 10, }, // Africa/Porto-Novo
    {  386, 4435,   33852,  8, }, // Africa/Sao_Tome
    {  386, 2866,     823,  7, }, // Africa/Tripoli
    {  386, 6554,     830,  5, }, // Africa/Tunis
    {  386, 4152,  118035,  7, }, // Africa/Windhoek
    {  386,  132,     842,  4, }, // America/Adak
    {  386, 3346,  118042,  7, }, // America/Anchorage
    {  386, 5607,   36316,  7, }, // America/Anguilla
    {  386, 5624,     860,  7, }, // America/Antigua
    {  386, 4554,     867,  9, }, // America/Araguaina
    {  386,  166,   51476, 12, }, // America/Argentina/Buenos_Aires
    {  386,   91,     888,  9, }, // America/Argentina/Catamarca
    {  386,  270,     897,  7, }, // America/Argentina/Cordoba
    {  386,  441,   27355,  5, }, // America/Argentina/Jujuy
    {  386, 7462,     909,  8, }, // America/Argentina/La_Rioja
    {  386,  565,   33923,  7, }, // America/Argentina/Mendoza
    {  386, 7489,  118049, 12, }, // America/Argentina/Rio_Gallegos
    {  386, 7520,     935,  5, }, // America/Argentina/Salta
    {  386, 7544,   33941,  8, }, // America/Argentina/San_Juan
    {  386, 5580,   33949,  7, }, // America/Argentina/San_Luis
    {  386, 7571,     956,  7, }, // America/Argentina/Tucuman
    {  386, 7597,  118061,  6, }, // America/Argentina/Ushuaia
    {  386, 5640,     969,  5, }, // America/Aruba
    {  386, 4252,   33963,  8, }, // America/Asuncion
    {  386,  237,   33971,  8, }, // America/Atikokan
    {  386, 3679,  118067,  5, }, // America/Bahia
    {  386, 7623,  118072, 14, }, // America/Bahia_Banderas
    {  386, 5654,    1013,  8, }, // America/Barbados
    {  386, 7646,   33998,  5, }, // America/Belem
    {  386, 5407,    1026,  5, }, // America/Belize
    {  386, 7660,    1031, 12, }, // America/Blanc-Sablon
    {  386, 7681,   34022,  9, }, // America/Boa_Vista
    {  386, 4361,    1052,  6, }, // America/Bogota
    {  386, 7699,   51524,  5, }, // America/Boise
    {  386, 7713,  118086, 11, }, // America/Cambridge_Bay
    {  386, 7735,   34047, 12, }, // America/Campo_Grande
    {  386, 3868,    1087,  6, }, // America/Cancun
    {  386, 4694,    1093,  7, }, // America/Caracas
    {  386, 4345,   36417,  5, }, // America/Cayenne
    {  386, 5496,   27497,  6, }, // America/Cayman
    {  386, 2260,    1129,  6, }, // America/Chicago
    {  386, 7756,  118097,  8, }, // America/Chihuahua
    {  386, 7774,   56160, 13, }, // America/Ciudad_Juarez
    {  386, 5422,    1141, 10, }, // America/Costa_Rica
    {  386, 7796,    1151,  7, }, // America/Creston
    {  386, 3791,  118105,  6, }, // America/Cuiaba
    {  386, 5723,   36474,  7, }, // America/Curacao
    {  386, 7812,   36481, 12, }, // America/Danmarkshavn
    {  386, 7833,    1184,  6, }, // America/Dawson
    {  386, 7848,  118111, 11, }, // America/Dawson_Creek
    {  386,  805,    1201,  6, }, // America/Denver
    {  386, 3465,   36517,  7, }, // America/Detroit
    {  386, 5739,    1214,  8, }, // America/Dominica
    {  386,  893,   51549,  8, }, // America/Edmonton
    {  386, 7869,   51557,  8, }, // America/Eirunepe
    {  386, 5441,  118122, 11, }, // America/El_Salvador
    {  386, 7886,   51565, 12, }, // America/Fort_Nelson
    {  386, 7906,    1257,  9, }, // America/Fortaleza
    {  386, 7924,   36551,  9, }, // America/Glace_Bay
    {  386, 6881,   36560,  7, }, // America/Goose_Bay
    {  386, 4612,  118133, 10, }, // America/Grand_Turk
    {  386, 5770,    1292,  7, }, // America/Grenada
    {  386, 5786,  118143,  9, }, // America/Guadeloupe
    {  386, 3760,    1308,  9, }, // America/Guatemala
    {  386, 4841,  118152,  8, }, // America/Guayaquil
    {  386, 6932,  118160,  6, }, // America/Guyana
    {  386, 1939,   51607,  8, }, // America/Halifax
    {  386, 2281,    1339,  6, }, // America/Havana
    {  386, 7942,  118166,  9, }, // America/Hermosillo
    {  386,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  386,  481,    1365, 13, }, // America/Indiana/Knox
    {  386, 7961,    1378, 16, }, // America/Indiana/Marengo
    {  386, 7985,  118175, 19, }, // America/Indiana/Petersburg
    {  386, 8012,  118194, 18, }, // America/Indiana/Tell_City
    {  386, 8038,  106186, 14, }, // America/Indiana/Vevay
    {  386, 8060,  118212, 15, }, // America/Indiana/Vincennes
    {  386, 8086,   56223, 16, }, // America/Indiana/Winamac
    {  386, 8110,    1452,  6, }, // America/Inuvik
    {  386,  660,    1458,  7, }, // America/Iqaluit
    {  386, 2799,   27879,  6, }, // America/Jamaica
    {  386, 5380,   34336,  4, }, // America/Juneau
    {  386,  521,  118227,  8, }, // America/Kentucky/Louisville
    {  386, 8125,  118235, 20, }, // America/Kentucky/Monticello
    {  386, 5704,   36619, 10, }, // America/Kralendijk
    {  386, 4376,   36629,  6, }, // America/La_Paz
    {  386, 7169,    1520,  4, }, // America/Lima
    {  386, 3239,  118255, 11, }, // America/Los_Angeles
    {  386, 5932,  118266, 19, }, // America/Lower_Princes
    {  386, 8153,   36667,  6, }, // America/Maceio
    {  386, 8168,    1559,  7, }, // America/Managua
    {  386, 1908,    1566,  6, }, // America/Manaus
    {  386, 5897,    1572,  6, }, // America/Marigot
    {  386, 5805,    1578,  8, }, // America/Martinique
    {  386, 8184,    1587,  9, }, // America/Matamoros
    {  386, 2917,   34417,  8, }, // America/Mazatlan
    {  386, 8202,    1604,  8, }, // America/Menominee
    {  386, 8220,    1612,  6, }, // America/Merida
    {  386, 8235,    1618, 10, }, // America/Metlakatla
    {  386, 2949,  118285, 12, }, // America/Mexico_City
    {  386, 4391,    1634,  7, }, // America/Miquelon
    {  386, 8254,    1641,  7, }, // America/Moncton
    {  386, 8270,   36673,  8, }, // America/Monterrey
    {  386, 4098,    1657, 10, }, // America/Montevideo
    {  386, 5824,  118297,  8, }, // America/Montserrat
    {  386, 5481,    1677,  6, }, // America/Nassau
    {  386, 2379,   51680,  8, }, // America/New_York
    {  386, 8288,    1691,  3, }, // America/Nome
    {  386, 1850,    1694,  7, }, // America/Noronha
    {  386, 8301,  118305, 20, }, // America/North_Dakota/Beulah
    {  386, 8329,  118325, 24, }, // America/North_Dakota/Center
    {  386, 8357,  118349, 24, }, // America/North_Dakota/New_Salem
    {  386,  393,    1772,  4, }, // America/Nuuk
    {  386, 8388,    1776,  7, }, // America/Ojinaga
    {  386, 2356,    1783,  6, }, // America/Panama
    {  386, 6173,    1789, 10, }, // America/Paramaribo
    {  386, 2973,   34541,  6, }, // America/Phoenix
    {  386, 3945,    1805, 12, }, // America/Port-au-Prince
    {  386, 5954,  118373, 13, }, // America/Port_of_Spain
    {  386, 8404,  118386, 11, }, // America/Porto_Velho
    {  386, 5843,   51756, 11, }, // America/Puerto_Rico
    {  386, 4030,    1852, 12, }, // America/Punta_Arenas
    {  386, 8424,  118397, 12, }, // America/Rankin_Inlet
    {  386, 8445,    1876,  6, }, // America/Recife
    {  386, 1995,   34609,  6, }, // America/Regina
    {  386, 8460,  118409,  7, }, // America/Resolute
    {  386,  695,   34622, 10, }, // America/Rio_Branco
    {  386, 8477,   34632,  8, }, // America/Santarem
    {  386, 2201,    1915,  8, }, // America/Santiago
    {  386, 6111,   34649, 13, }, // America/Santo_Domingo
    {  386, 1878,   36774,  9, }, // America/Sao_Paulo
    {  386, 6899,  118416, 16, }, // America/Scoresbysund
    {  386, 8494,    1956,  5, }, // America/Sitka
    {  386, 8508,  118432, 14, }, // America/St_Barthelemy
    {  386, 2061,  118446,  9, }, // America/St_Johns
    {  386, 5863,    1983,  9, }, // America/St_Kitts
    {  386, 5880,  118455, 11, }, // America/St_Lucia
    {  386,  855,    2003, 10, }, // America/St_Thomas
    {  386, 5913,    2013, 12, }, // America/St_Vincent
    {  386, 8530,  118466, 13, }, // America/Swift_Current
    {  386, 5461,   51817, 12, }, // America/Tegucigalpa
    {  386, 5756,    2049,  4, }, // America/Thule
    {  386,  313,    2053,  7, }, // America/Tijuana
    {  386,  608,    2060,  7, }, // America/Toronto
    {  386, 5688,    2067,  7, }, // America/Tortola
    {  386, 2093,    2074,  8, }, // America/Vancouver
    {  386, 2144,   28435,  8, }, // America/Whitehorse
    {  386,  734,  118479,  8, }, // America/Winnipeg
    {  386, 8552,   28450,  6, }, // America/Yakutat
    {  386, 6001,   36889,  5, }, // Antarctica/Casey
    {  386, 6094,  118487,  5, }, // Antarctica/Davis
    {  386, 6133,  118492, 13, }, // Antarctica/DumontDUrville
    {  386, 8568,    2131,  8, }, // Antarctica/Macquarie
    {  386, 7066,    2139,  6, }, // Antarctica/Mawson
    {  386, 7113,  118505,  8, }, // Antarctica/McMurdo
    {  386, 8589,    2154,  6, }, // Antarctica/Palmer
    {  386, 7226,    2160,  6, }, // Antarctica/Rothera
    {  386, 7295,  118513,  4, }, // Antarctica/Syowa
    {  386, 8607,    2171,  5, }, // Antarctica/Troll
    {  386, 7374,   34825,  6, }, // Antarctica/Vostok
    {  386, 1429,  118517, 11, }, // Arctic/Longyearbyen
    {  386, 5570,    2192,  4, }, // Asia/Aden
    {  386, 5395,    2196,  6, }, // Asia/Almaty
    {  386, 3968,    2202,  5, }, // Asia/Amman
    {  386, 5511,    2207,  6, }, // Asia/Anadyr
    {  386, 5523,    2213,  5, }, // Asia/Aqtau
    {  386, 5368,    2218,  6, }, // Asia/Aqtobe
    {  386,  964,    2224,  7, }, // Asia/Ashgabat
    {  386, 8624,    2231,  6, }, // Asia/Atyrau
    {  386, 3607,    2237,  6, }, // Asia/Baghdad
    {  386, 5534,   36948,  7, }, // Asia/Bahrain
    {  386, 3653,    2250,  4, }, // Asia/Baku
    {  386, 4466,    2254,  7, }, // Asia/Bangkok
    {  386, 3571,    2261,  7, }, // Asia/Barnaul
    {  386, 4086,   28563,  6, }, // Asia/Beirut
    {  386, 3778,    2274,  6, }, // Asia/Bishkek
    {  386, 5989,   28575,  6, }, // Asia/Brunei
    {  386, 4601,    2286,  4, }, // Asia/Chita
    {  386, 4511,    2290,  7, }, // Asia/Colombo
    {  386, 4540,    2297,  6, }, // Asia/Damascus
    {  386, 1093,    2303,  5, }, // Asia/Dhaka
    {  386, 6192,    2308,  4, }, // Asia/Dili
    {  386, 3596,   28596,  5, }, // Asia/Dubai
    {  386, 6159,    2317,  7, }, // Asia/Dushanbe
    {  386, 8636,    2324,  9, }, // Asia/Famagusta
    {  386, 8651,    2333,  4, }, // Asia/Gaza
    {  386, 4795,   34886,  6, }, // Asia/Hebron
    {  386, 1259,   34892,  9, }, // Asia/Ho_Chi_Minh
    {  386, 2704,   51902,  7, }, // Asia/Hong_Kong
    {  386, 4771,    2357,  4, }, // Asia/Hovd
    {  386, 4184,    2361,  7, }, // Asia/Irkutsk
    {  386, 7006,  118528,  7, }, // Asia/Jakarta
    {  386, 6992,  118535,  8, }, // Asia/Jayapura
    {  386, 1290,    2383,  9, }, // Asia/Jerusalem
    {  386, 3560,    2392,  5, }, // Asia/Kabul
    {  386, 4316,    2411,  8, }, // Asia/Kamchatka
    {  386, 4239,    2422,  6, }, // Asia/Karachi
    {  386, 1185,    2428,  8, }, // Asia/Kathmandu
    {  386, 8661,  118543,  7, }, // Asia/Khandyga
    {  386,  992,   51950,  9, }, // Asia/Kolkata
    {  386, 4197,   36978, 10, }, // Asia/Krasnoyarsk
    {  386, 4859,   34958, 12, }, // Asia/Kuala_Lumpur
    {  386, 5976,  118550,  5, }, // Asia/Kuching
    {  386, 5547,   28693,  6, }, // Asia/Kuwait
    {  386, 1211,   34976,  5, }, // Asia/Macau
    {  386, 4017,    2491,  7, }, // Asia/Magadan
    {  386, 1349,  118555,  8, }, // Asia/Makassar
    {  386, 7182,    2505,  6, }, // Asia/Manila
    {  386, 6920,   34981,  6, }, // Asia/Muscat
    {  386, 2554,  106335,  7, }, // Asia/Nicosia
    {  386, 8675,    2524, 11, }, // Asia/Novokuznetsk
    {  386, 4135,    2535, 11, }, // Asia/Novosibirsk
    {  386, 4229,    2546,  4, }, // Asia/Omsk
    {  386, 7145,   35006,  4, }, // Asia/Oral
    {  386, 6961,   51959,  8, }, // Asia/Phnom_Penh
    {  386, 8693,    2563,  9, }, // Asia/Pontianak
    {  386, 4214,   28763,  7, }, // Asia/Pyongyang
    {  386, 5559,    2579,  5, }, // Asia/Qatar
    {  386, 8708,   37005,  8, }, // Asia/Qostanay
    {  386, 4269,    2592,  9, }, // Asia/Qyzylorda
    {  386, 3584,  118563,  5, }, // Asia/Riyadh
    {  386, 4408,    2614,  7, }, // Asia/Sakhalin
    {  386, 7245,    2621,  9, }, // Asia/Samarkand
    {  386, 3279,    2630,  4, }, // Asia/Seoul
    {  386, 1053,   28801,  6, }, // Asia/Shanghai
    {  386, 3300,    2640,  8, }, // Asia/Singapore
    {  386, 4297,    2648, 13, }, // Asia/Srednekolymsk
    {  386, 3263,   60896,  6, }, // Asia/Taipei
    {  386, 4781,    2667,  7, }, // Asia/Tashkent
    {  386, 3915,   35074,  7, }, // Asia/Tbilisi
    {  386, 2772,    2678,  7, }, // Asia/Tehran
    {  386, 1317,   35088,  5, }, // Asia/Thimphu
    {  386, 2821,    2692,  5, }, // Asia/Tokyo
    {  386, 4572,    2697,  5, }, // Asia/Tomsk
    {  386, 1021,    2702, 10, }, // Asia/Ulaanbaatar
    {  386, 1159,  118568,  6, }, // Asia/Urumqi
    {  386, 8722,   51993,  9, }, // Asia/Ust-Nera
    {  386, 6977,  118574,  8, }, // Asia/Vientiane
    {  386, 4710,    2733, 11, }, // Asia/Vladivostok
    {  386, 4828,   37051,  6, }, // Asia/Yakutsk
    {  386, 1235,   28911,  6, }, // Asia/Yangon
    {  386, 3883,    2757, 12, }, // Asia/Yekaterinburg
    {  386, 3747,    2769,  6, }, // Asia/Yerevan
    {  386, 3663,    2775,  4, }, // Atlantic/Azores
    {  386, 5671,   35152,  7, }, // Atlantic/Bermuda
    {  386, 6628,  118582,  6, }, // Atlantic/Canary
    {  386, 3727,  118588, 10, }, // Atlantic/Cape_Verde
    {  386, 1395,    2836,  5, }, // Atlantic/Faroe
    {  386, 8736,   37132,  7, }, // Atlantic/Madeira
    {  386, 2748,   52082,  9, }, // Atlantic/Reykjavik
    {  386, 7272,  118598, 13, }, // Atlantic/South_Georgia
    {  386, 6821,  118611, 10, }, // Atlantic/St_Helena
    {  386, 6644,   52124,  6, }, // Atlantic/Stanley
    {  386, 1670,    2917,  8, }, // Australia/Adelaide
    {  386, 1635,    2925,  7, }, // Australia/Brisbane
    {  386, 1799,  118621, 11, }, // Australia/Broken_Hill
    {  386, 1583,    2943,  6, }, // Australia/Darwin
    {  386, 3637,   29108,  4, }, // Australia/Eucla
    {  386, 1516,    2954,  6, }, // Australia/Hobart
    {  386, 8753,    2960,  8, }, // Australia/Lindeman
    {  386, 1547,   37190,  8, }, // Australia/Lord_Howe
    {  386, 1727,   29120,  8, }, // Australia/Melbourne
    {  386, 1762,    2983,  4, }, // Australia/Perth
    {  386, 1463,   52144,  6, }, // Australia/Sydney
    {  386, 6441,    2993,  9, }, // Europe/Amsterdam
    {  386, 6216,    3002,  7, }, // Europe/Andorra
    {  386, 3620,   29149,  9, }, // Europe/Astrakhan
    {  386, 2300,   60919,  5, }, // Europe/Athens
    {  386, 6336,    3021,  7, }, // Europe/Belgrade
    {  386, 4757,    3028,  6, }, // Europe/Berlin
    {  386, 6488,    3034, 10, }, // Europe/Bratislava
    {  386, 2167,   52164,  8, }, // Europe/Brussels
    {  386, 3928,    3052,  8, }, // Europe/Bucharest
    {  386, 3806,    3060,  8, }, // Europe/Budapest
    {  386, 8772,  118632,  8, }, // Europe/Busingen
    {  386, 2583,    3091,  7, }, // Europe/Chisinau
    {  386, 6289,   52191, 10, }, // Europe/Copenhagen
    {  386, 2338,    3108,  6, }, // Europe/Dublin
    {  386, 6307,    3114,  9, }, // Europe/Gibraltar
    {  386, 8788,    3123,  6, }, // Europe/Guernsey
    {  386, 6612,   52201,  9, }, // Europe/Helsinki
    {  386, 8804,  106485,  9, }, // Europe/Isle_of_Man
    {  386, 1130,    3146,  7, }, // Europe/Istanbul
    {  386, 8823,  118640,  5, }, // Europe/Jersey
    {  386, 3979,    3158, 11, }, // Europe/Kaliningrad
    {  386, 8929,    3169,  5, }, // Europe/Kirov
    {  386, 2527,    3174,  4, }, // Europe/Kyiv
    {  386, 3213,    3178,  8, }, // Europe/Lisbon
    {  386, 6506,   29300,  7, }, // Europe/Ljubljana
    {  386, 2501,    3195,  6, }, // Europe/London
    {  386, 6365,   29313, 10, }, // Europe/Luxembourg
    {  386, 4892,    3212,  6, }, // Europe/Madrid
    {  386, 6397,   29329,  6, }, // Europe/Malta
    {  386, 6582,    3223,  9, }, // Europe/Mariehamn
    {  386, 3693,    3232,  5, }, // Europe/Minsk
    {  386, 6410,    3237,  6, }, // Europe/Monaco
    {  386, 3537,    3243,  6, }, // Europe/Moscow
    {  386, 6458,    3249,  4, }, // Europe/Oslo
    {  386, 4284,    3253,  5, }, // Europe/Paris
    {  386, 6424,    3258,  9, }, // Europe/Podgorica
    {  386, 6275,    3267,  5, }, // Europe/Prague
    {  386, 8837,    3272,  4, }, // Europe/Riga
    {  386, 6324,    3276,  3, }, // Europe/Rome
    {  386, 4331,    3279,  6, }, // Europe/Samara
    {  386, 6470,   35413, 10, }, // Europe/San_Marino
    {  386, 6245,    3295,  7, }, // Europe/Sarajevo
    {  386, 4451,    3302,  7, }, // Europe/Saratov
    {  386, 8849,   52225, 11, }, // Europe/Simferopol
    {  386, 6383,  118645,  6, }, // Europe/Skopje
    {  386, 6599,   37286,  5, }, // Europe/Sofia
    {  386, 6523,   52242,  9, }, // Europe/Stockholm
    {  386, 8867,    3337,  6, }, // Europe/Tallinn
    {  386, 6202,    3343,  6, }, // Europe/Tirane
    {  386, 8882,   52251,  9, }, // Europe/Ulyanovsk
    {  386, 6352,  118651,  5, }, // Europe/Vaduz
    {  386, 6567,    3363,  7, }, // Europe/Vatican
    {  386, 6231,    3370,  4, }, // Europe/Vienna
    {  386, 8899,   52260,  7, }, // Europe/Vilnius
    {  386, 4727,    3381,  9, }, // Europe/Volgograd
    {  386, 3190,    3390,  7, }, // Europe/Warsaw
    {  386, 6261,    3397,  6, }, // Europe/Zagreb
    {  386, 6540,   37310,  5, }, // Europe/Zurich
    {  386, 5078,   35493, 12, }, // Indian/Antananarivo
    {  386, 6947,    3421,  5, }, // Indian/Chagos
    {  386, 6046,  118656,  7, }, // Indian/Christmas
    {  386, 6063,    3438,  5, }, // Indian/Cocos
    {  386, 5029,   35510,  6, }, // Indian/Comoro
    {  386, 6661,    3460,  8, }, // Indian/Kerguelen
    {  386, 7260,   29562,  3, }, // Indian/Mahe
    {  386, 7050,  106550, 16, }, // Indian/Maldives
    {  386, 4069,   56576,  8, }, // Indian/Mauritius
    {  386, 5098,   56584,  6, }, // Indian/Mayotte
    {  386, 7211,   37369,  7, }, // Indian/Reunion
    {  386, 4422,    3498,  4, }, // Pacific/Apia
    {  386,  932,  118663,  6, }, // Pacific/Auckland
    {  386, 3706,   52355,  9, }, // Pacific/Bougainville
    {  386, 3015,    3516,  5, }, // Pacific/Chatham
    {  386, 3157,  118669,  4, }, // Pacific/Chuuk
    {  386, 2237,  118673,  5, }, // Pacific/Easter
    {  386, 7360,   29628,  4, }, // Pacific/Efate
    {  386, 7327,    3541,  7, }, // Pacific/Fakaofo
    {  386, 3902,  118678,  4, }, // Pacific/Fiji
    {  386, 7343,    3552,  8, }, // Pacific/Funafuti
    {  386, 6678,    3560,  9, }, // Pacific/Galapagos
    {  386, 6696,   37412,  6, }, // Pacific/Gambier
    {  386, 3822,    3595, 11, }, // Pacific/Guadalcanal
    {  386, 6018,    3606,  4, }, // Pacific/Guam
    {  386, 2723,   52450,  8, }, // Pacific/Honolulu
    {  386, 3049,   52458,  9, }, // Pacific/Kanton
    {  386, 3998,    3616, 10, }, // Pacific/Kiritimati
    {  386, 7019,  118682,  6, }, // Pacific/Kosrae
    {  386, 2842,  118688,  9, }, // Pacific/Kwajalein
    {  386, 4877,  118697,  6, }, // Pacific/Majuro
    {  386, 4051,  118703,  8, }, // Pacific/Marquesas
    {  386, 8914,   52490,  6, }, // Pacific/Midway
    {  386, 7084,    3680,  5, }, // Pacific/Nauru
    {  386, 7132,   52504,  4, }, // Pacific/Niue
    {  386, 4168,    3689,  7, }, // Pacific/Norfolk
    {  386, 7098,    3696,  5, }, // Pacific/Noumea
    {  386, 3126,   35642,  9, }, // Pacific/Pago_Pago
    {  386, 7155,    3710,  5, }, // Pacific/Palau
    {  386, 7194,   56657,  7, }, // Pacific/Pitcairn
    {  386, 3096,   37471,  6, }, // Pacific/Pohnpei
    {  386, 4807,  118711, 12, }, // Pacific/Port_Moresby
    {  386, 6076,  118723,  9, }, // Pacific/Rarotonga
    {  386, 6031,   29803,  6, }, // Pacific/Saipan
    {  386, 7312,    3754,  5, }, // Pacific/Tahiti
    {  386, 6712,    3759,  6, }, // Pacific/Tarawa
    {  386, 4583,    3765,  9, }, // Pacific/Tongatapu
    {  386, 7392,   37494,  4, }, // Pacific/Wake
    {  386, 7405,   37498,  5, }, // Pacific/Wallis
    {  387,   14,       0,  6, }, // Africa/Asmara Lakota/Latin/United States
    {  387, 4435,       6,  8, }, // Africa/Sao_Tome
    {  387, 4252,      14,  8, }, // America/Asuncion
    {  387,  237,      22,  8, }, // America/Atikokan
    {  387, 7623,      30, 17, }, // America/Bahia_Banderas
    {  387, 3868,      47,  6, }, // America/Cancun
    {  387, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  387, 5723,      66,  7, }, // America/Curacao
    {  387,  481,      73, 13, }, // America/Indiana/Knox
    {  387, 7961,      86, 16, }, // America/Indiana/Marengo
    {  387, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  387, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  387, 8038,     139, 14, }, // America/Indiana/Vevay
    {  387, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  387, 8086,     171, 16, }, // America/Indiana/Winamac
    {  387, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  387, 5932,     207, 22, }, // America/Lower_Princes
    {  387, 8220,     229,  6, }, // America/Merida
    {  387, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  387, 8329,     255, 20, }, // America/North_Dakota/Center
    {  387, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  387,  393,     298,  4, }, // America/Nuuk
    {  387, 6899,     302, 16, }, // America/Scoresbysund
    {  387, 8508,     318, 14, }, // America/St_Barthelemy
    {  387, 2061,     332, 10, }, // America/St_Johns
    {  387, 5863,     342,  9, }, // America/St_Kitts
    {  387, 5880,     351,  9, }, // America/St_Lucia
    {  387,  855,     360, 10, }, // America/St_Thomas
    {  387, 5913,     370, 11, }, // America/St_Vincent
    {  387, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  387, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  387, 1185,     413,  9, }, // Asia/Kathmandu
    {  387,  992,     422,  7, }, // Asia/Kolkata
    {  387, 1211,     429,  5, }, // Asia/Macau
    {  387, 1235,     442,  6, }, // Asia/Yangon
    {  387, 1395,     448,  5, }, // Atlantic/Faroe
    {  387, 6821,     453, 10, }, // Atlantic/St_Helena
    {  387, 2527,     463,  4, }, // Europe/Kyiv
    {  387, 7211,     467,  7, }, // Indian/Reunion
    {  387, 3157,     474,  5, }, // Pacific/Chuuk
    {  387, 3096,     479,  7, }, // Pacific/Pohnpei
    {  388,   14,       0,  6, }, // Africa/Asmara Langi/Latin/Tanzania
    {  388, 4435,       6,  8, }, // Africa/Sao_Tome
    {  388, 4252,      14,  8, }, // America/Asuncion
    {  388,  237,      22,  8, }, // America/Atikokan
    {  388, 7623,      30, 17, }, // America/Bahia_Banderas
    {  388, 3868,      47,  6, }, // America/Cancun
    {  388, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  388, 5723,      66,  7, }, // America/Curacao
    {  388,  481,      73, 13, }, // America/Indiana/Knox
    {  388, 7961,      86, 16, }, // America/Indiana/Marengo
    {  388, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  388, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  388, 8038,     139, 14, }, // America/Indiana/Vevay
    {  388, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  388, 8086,     171, 16, }, // America/Indiana/Winamac
    {  388, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  388, 5932,     207, 22, }, // America/Lower_Princes
    {  388, 8220,     229,  6, }, // America/Merida
    {  388, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  388, 8329,     255, 20, }, // America/North_Dakota/Center
    {  388, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  388,  393,     298,  4, }, // America/Nuuk
    {  388, 6899,     302, 16, }, // America/Scoresbysund
    {  388, 8508,     318, 14, }, // America/St_Barthelemy
    {  388, 2061,     332, 10, }, // America/St_Johns
    {  388, 5863,     342,  9, }, // America/St_Kitts
    {  388, 5880,     351,  9, }, // America/St_Lucia
    {  388,  855,     360, 10, }, // America/St_Thomas
    {  388, 5913,     370, 11, }, // America/St_Vincent
    {  388, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  388, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  388, 1185,     413,  9, }, // Asia/Kathmandu
    {  388,  992,     422,  7, }, // Asia/Kolkata
    {  388, 1211,     429,  5, }, // Asia/Macau
    {  388, 1235,     442,  6, }, // Asia/Yangon
    {  388, 1395,     448,  5, }, // Atlantic/Faroe
    {  388, 6821,     453, 10, }, // Atlantic/St_Helena
    {  388, 2527,     463,  4, }, // Europe/Kyiv
    {  388, 7211,     467,  7, }, // Indian/Reunion
    {  388, 3157,     474,  5, }, // Pacific/Chuuk
    {  388, 3096,     479,  7, }, // Pacific/Pohnpei
    {  389, 6788,  118732,  8, }, // Africa/Abidjan Lao/Lao/Laos
    {  389, 6760,  118740,  6, }, // Africa/Accra
    {  389, 5059,  118746, 13, }, // Africa/Addis_Ababa
    {  389, 7420,  118759,  6, }, // Africa/Algiers
    {  389,   14,  118765,  7, }, // Africa/Asmara
    {  389,   44,  118772,  6, }, // Africa/Bamako
    {  389, 5257,  118778,  6, }, // Africa/Bangui
    {  389, 6746,  118784,  5, }, // Africa/Banjul
    {  389, 7435,  118789,  7, }, // Africa/Bissau
    {  389, 4957,  118796,  8, }, // Africa/Blantyre
    {  389, 5287,  118804,  8, }, // Africa/Brazzaville
    {  389, 4922,  118812,  9, }, // Africa/Bujumbura
    {  389, 2320,  118821,  4, }, // Africa/Cairo
    {  389, 4117,  118825, 10, }, // Africa/Casablanca
    {  389, 7449,  118835,  5, }, // Africa/Ceuta
    {  389, 6773,  118840,  7, }, // Africa/Conakry
    {  389, 6840,  118847,  4, }, // Africa/Dakar
    {  389, 5130,  118851, 10, }, // Africa/Dar_es_Salaam
    {  389, 5043,  118861,  6, }, // Africa/Djibouti
    {  389, 5243,  118867,  6, }, // Africa/Douala
    {  389, 5166,  118873,  9, }, // Africa/El_Aaiun
    {  389, 6853,  118882,  6, }, // Africa/Freetown
    {  389, 4906,  118888,  7, }, // Africa/Gaborone
    {  389, 5015,  118895,  6, }, // Africa/Harare
    {  389, 4479,  118901, 13, }, // Africa/Johannesburg
    {  389, 4499,  118914,  4, }, // Africa/Juba
    {  389, 5151,  118918,  6, }, // Africa/Kampala
    {  389, 4524,  118924,  5, }, // Africa/Khartoum
    {  389, 4973,  118929,  6, }, // Africa/Kigali
    {  389, 5306,  118935,  7, }, // Africa/Kinshasa
    {  389, 4744,  118942,  5, }, // Africa/Lagos
    {  389, 5336,  118947,  9, }, // Africa/Libreville
    {  389, 6869,  118956,  4, }, // Africa/Lome
    {  389, 5211,  118960,  5, }, // Africa/Luanda
    {  389, 4939,  118965,  8, }, // Africa/Lubumbashi
    {  389, 5001,  118973,  6, }, // Africa/Lusaka
    {  389, 5322,  118979,  6, }, // Africa/Malabo
    {  389, 4987,  118985,  6, }, // Africa/Maputo
    {  389, 5197,  118991,  6, }, // Africa/Maseru
    {  389, 5182,  118997,  8, }, // Africa/Mbabane
    {  389, 5113,  119005,  8, }, // Africa/Mogadishu
    {  389, 7034,  119013,  8, }, // Africa/Monrovia
    {  389, 3853,  119021,  6, }, // Africa/Nairobi
    {  389, 5271,  119027, 10, }, // Africa/Ndjamena
    {  389, 5354,  119037,  6, }, // Africa/Niamey
    {  389, 6803,  119043,  8, }, // Africa/Nouakchott
    {  389, 6727,  119051,  8, }, // Africa/Ouagadougou
    {  389, 5225,  119059,  9, }, // Africa/Porto-Novo
    {  389, 4435,  119068,  7, }, // Africa/Sao_Tome
    {  389, 2866,  119075,  7, }, // Africa/Tripoli
    {  389, 6554,  119082,  5, }, // Africa/Tunis
    {  389, 4152,  119087,  6, }, // Africa/Windhoek
    {  389,  132,  119093,  5, }, // America/Adak
    {  389, 3346,  119098,  9, }, // America/Anchorage
    {  389, 5607,  119107,  8, }, // America/Anguilla
    {  389, 5624,  119115,  8, }, // America/Antigua
    {  389, 4554,  119123,  9, }, // America/Araguaina
    {  389,  166,  119132, 12, }, // America/Argentina/Buenos_Aires
    {  389,   91,  119144,  8, }, // America/Argentina/Catamarca
    {  389,  270,  119152,  6, }, // America/Argentina/Cordoba
    {  389,  441,  119158,  5, }, // America/Argentina/Jujuy
    {  389, 7462,  119163,  9, }, // America/Argentina/La_Rioja
    {  389,  565,  119172,  9, }, // America/Argentina/Mendoza
    {  389, 7489,  119181, 13, }, // America/Argentina/Rio_Gallegos
    {  389, 7520,  119194,  5, }, // America/Argentina/Salta
    {  389, 7544,  119199,  6, }, // America/Argentina/San_Juan
    {  389, 5580,  119205,  8, }, // America/Argentina/San_Luis
    {  389, 7571,  119213,  7, }, // America/Argentina/Tucuman
    {  389, 7597,  119220,  7, }, // America/Argentina/Ushuaia
    {  389, 5640,  119227,  6, }, // America/Aruba
    {  389, 4252,  119233, 10, }, // America/Asuncion
    {  389,  237,  119243,  9, }, // America/Atikokan
    {  389, 3679,  119252,  5, }, // America/Bahia
    {  389, 7623,  119257, 15, }, // America/Bahia_Banderas
    {  389, 5654,  119272,  7, }, // America/Barbados
    {  389, 7646,  119279,  5, }, // America/Belem
    {  389, 5407,  119284,  5, }, // America/Belize
    {  389, 7660,  119289, 11, }, // America/Blanc-Sablon
    {  389, 7681,  119300,  9, }, // America/Boa_Vista
    {  389, 4361,  119309,  6, }, // America/Bogota
    {  389, 7699,  119315,  5, }, // America/Boise
    {  389, 7713,  119320, 10, }, // America/Cambridge_Bay
    {  389, 7735,  119330, 10, }, // America/Campo_Grande
    {  389, 3868,  119340,  6, }, // America/Cancun
    {  389, 4694,  119346,  7, }, // America/Caracas
    {  389, 4345,  119353,  5, }, // America/Cayenne
    {  389, 5496,  119358,  5, }, // America/Cayman
    {  389, 2260,  119363,  6, }, // America/Chicago
    {  389, 7756,  119369,  6, }, // America/Chihuahua
    {  389, 7774,  119375, 14, }, // America/Ciudad_Juarez
    {  389, 5422,  119389,  9, }, // America/Costa_Rica
    {  389, 7796,  119398,  7, }, // America/Creston
    {  389, 3791,  119405,  6, }, // America/Cuiaba
    {  389, 5723,  119411,  8, }, // America/Curacao
    {  389, 7812,  119419,  9, }, // America/Danmarkshavn
    {  389, 7833,  119428,  5, }, // America/Dawson
    {  389, 7848,  119433, 10, }, // America/Dawson_Creek
    {  389,  805,  119443,  7, }, // America/Denver
    {  389, 3465,  119450,  6, }, // America/Detroit
    {  389, 5739,  119456,  8, }, // America/Dominica
    {  389,  893,  119464,  9, }, // America/Edmonton
    {  389, 7869,  119473,  8, }, // America/Eirunepe
    {  389, 5441,  119481, 10, }, // America/El_Salvador
    {  389, 7886,  119491, 10, }, // America/Fort_Nelson
    {  389, 7906,  119501,  8, }, // America/Fortaleza
    {  389, 7924,  119509,  6, }, // America/Glace_Bay
    {  389, 6881,  119515,  5, }, // America/Goose_Bay
    {  389, 4612,  119520,  9, }, // America/Grand_Turk
    {  389, 5770,  119529,  7, }, // America/Grenada
    {  389, 5786,  119536,  8, }, // America/Guadeloupe
    {  389, 3760,  119544,  9, }, // America/Guatemala
    {  389, 4841,  119553,  8, }, // America/Guayaquil
    {  389, 6932,  119561,  6, }, // America/Guyana
    {  389, 1939,  119567,  8, }, // America/Halifax
    {  389, 2281,  119575,  6, }, // America/Havana
    {  389, 7942,  119581,  8, }, // America/Hermosillo
    {  389,  348,  119589, 14, }, // America/Indiana/Indianapolis
    {  389,  481,  119603, 14, }, // America/Indiana/Knox
    {  389, 7961,  119617, 17, }, // America/Indiana/Marengo
    {  389, 7985,  119634, 20, }, // America/Indiana/Petersburg
    {  389, 8012,  119654, 18, }, // America/Indiana/Tell_City
    {  389, 8038,  119672, 14, }, // America/Indiana/Vevay
    {  389, 8060,  119686, 19, }, // America/Indiana/Vincennes
    {  389, 8086,  119705, 18, }, // America/Indiana/Winamac
    {  389, 8110,  119723,  7, }, // America/Inuvik
    {  389,  660,  119730,  8, }, // America/Iqaluit
    {  389, 2799,  119738,  6, }, // America/Jamaica
    {  389, 5380,  119744,  4, }, // America/Juneau
    {  389,  521,  119748,  7, }, // America/Kentucky/Louisville
    {  389, 8125,  119755, 20, }, // America/Kentucky/Monticello
    {  389, 5704,  119775,  9, }, // America/Kralendijk
    {  389, 4376,  119784,  5, }, // America/La_Paz
    {  389, 7169,  119789,  4, }, // America/Lima
    {  389, 3239,  119793, 11, }, // America/Los_Angeles
    {  389, 5932,  119804, 19, }, // America/Lower_Princes
    {  389, 8153,  119823,  6, }, // America/Maceio
    {  389, 8168,  119829,  7, }, // America/Managua
    {  389, 1908,  119836,  8, }, // America/Manaus
    {  389, 5897,  119844,  7, }, // America/Marigot
    {  389, 5805,  119851,  7, }, // America/Martinique
    {  389, 8184,  119858,  9, }, // America/Matamoros
    {  389, 2917,  119867,  8, }, // America/Mazatlan
    {  389, 8202,  119875,  8, }, // America/Menominee
    {  389, 8220,  119883,  7, }, // America/Merida
    {  389, 8235,  119890, 11, }, // America/Metlakatla
    {  389, 2949,  119901, 13, }, // America/Mexico_City
    {  389, 4391,  119914,  8, }, // America/Miquelon
    {  389, 8254,  119922,  6, }, // America/Moncton
    {  389, 8270,  119928,  9, }, // America/Monterrey
    {  389, 4098,  119937, 11, }, // America/Montevideo
    {  389, 5824,  119948,  9, }, // America/Montserrat
    {  389, 5481,  119957,  5, }, // America/Nassau
    {  389, 2379,  119962,  6, }, // America/New_York
    {  389, 8288,  119968,  3, }, // America/Nome
    {  389, 1850,  119971,  7, }, // America/Noronha
    {  389, 8301,  119978, 17, }, // America/North_Dakota/Beulah
    {  389, 8329,  119995, 19, }, // America/North_Dakota/Center
    {  389, 8357,  120014, 21, }, // America/North_Dakota/New_Salem
    {  389,  393,  120035,  3, }, // America/Nuuk
    {  389, 8388,  120038,  8, }, // America/Ojinaga
    {  389, 2356,  120046,  6, }, // America/Panama
    {  389, 6173,  120052, 10, }, // America/Paramaribo
    {  389, 2973,  120062,  5, }, // America/Phoenix
    {  389, 3945,  120067,  8, }, // America/Port-au-Prince
    {  389, 5954,  120075, 11, }, // America/Port_of_Spain
    {  389, 8404,  120086, 10, }, // America/Porto_Velho
    {  389, 5843,  120096,  9, }, // America/Puerto_Rico
    {  389, 4030,  120105, 14, }, // America/Punta_Arenas
    {  389, 8424,  120119, 14, }, // America/Rankin_Inlet
    {  389, 8445,  120133,  6, }, // America/Recife
    {  389, 1995,  120139,  6, }, // America/Regina
    {  389, 8460,  120145,  7, }, // America/Resolute
    {  389,  695,  120152, 11, }, // America/Rio_Branco
    {  389, 8477,  120163,  8, }, // America/Santarem
    {  389, 2201,  120171,  9, }, // America/Santiago
    {  389, 6111,  120180, 12, }, // America/Santo_Domingo
    {  389, 1878,  120192, 11, }, // America/Sao_Paulo
    {  389, 6899,  120203, 12, }, // America/Scoresbysund
    {  389, 8494,  120215,  5, }, // America/Sitka
    {  389, 8508,  120220, 11, }, // America/St_Barthelemy
    {  389, 2061,  120231,  7, }, // America/St_Johns
    {  389, 5863,  120238,  9, }, // America/St_Kitts
    {  389, 5880,  120247,  9, }, // America/St_Lucia
    {  389,  855,  120256,  9, }, // America/St_Thomas
    {  389, 5913,  120265, 10, }, // America/St_Vincent
    {  389, 8530,  120275, 12, }, // America/Swift_Current
    {  389, 5461,  120287, 11, }, // America/Tegucigalpa
    {  389, 5756,  120298,  4, }, // America/Thule
    {  389,  313,  120302,  7, }, // America/Tijuana
    {  389,  608,  120309,  7, }, // America/Toronto
    {  389, 5688,  120316,  6, }, // America/Tortola
    {  389, 2093,  120322,  8, }, // America/Vancouver
    {  389, 2144,  120330,  7, }, // America/Whitehorse
    {  389,  734,  120337,  8, }, // America/Winnipeg
    {  389, 8552,  120345,  7, }, // America/Yakutat
    {  389, 6001,  120352,  4, }, // Antarctica/Casey
    {  389, 6094,  120356,  5, }, // Antarctica/Davis
    {  389, 6133,  120361, 14, }, // Antarctica/DumontDUrville
    {  389, 8568,  120375,  9, }, // Antarctica/Macquarie
    {  389, 7066,  120384,  7, }, // Antarctica/Mawson
    {  389, 7113,  120391,  9, }, // Antarctica/McMurdo
    {  389, 8589,  120400,  6, }, // Antarctica/Palmer
    {  389, 7226,  120406,  6, }, // Antarctica/Rothera
    {  389, 7295,  120412,  5, }, // Antarctica/Syowa
    {  389, 8607,  120417,  5, }, // Antarctica/Troll
    {  389, 7374,  120422,  7, }, // Antarctica/Vostok
    {  389, 1429,  120429,  9, }, // Arctic/Longyearbyen
    {  389, 5570,  120438,  6, }, // Asia/Aden
    {  389, 5395,  120444,  6, }, // Asia/Almaty
    {  389, 3968,  120450,  5, }, // Asia/Amman
    {  389, 5511,  120455,  6, }, // Asia/Anadyr
    {  389, 5523,  120461,  7, }, // Asia/Aqtau
    {  389, 5368,  120468,  7, }, // Asia/Aqtobe
    {  389,  964,  120475, 10, }, // Asia/Ashgabat
    {  389, 8624,  120485,  9, }, // Asia/Atyrau
    {  389, 3607,  120494,  6, }, // Asia/Baghdad
    {  389, 5534,  120500,  5, }, // Asia/Bahrain
    {  389, 3653,  120505,  4, }, // Asia/Baku
    {  389, 4466,  120509,  6, }, // Asia/Bangkok
    {  389, 3571,  120515,  5, }, // Asia/Barnaul
    {  389, 4086,  120520,  5, }, // Asia/Beirut
    {  389, 3778,  120525,  8, }, // Asia/Bishkek
    {  389, 5989,  120533,  5, }, // Asia/Brunei
    {  389, 4601,  120538,  5, }, // Asia/Chita
    {  389, 4511,  120543,  6, }, // Asia/Colombo
    {  389, 4540,  120549,  8, }, // Asia/Damascus
    {  389, 1093,  120557,  6, }, // Asia/Dhaka
    {  389, 6192,  120563,  4, }, // Asia/Dili
    {  389, 3596,  120567,  4, }, // Asia/Dubai
    {  389, 6159,  120571,  7, }, // Asia/Dushanbe
    {  389, 8636,  120578,  9, }, // Asia/Famagusta
    {  389, 8651,  120587,  4, }, // Asia/Gaza
    {  389, 4795,  120591,  6, }, // Asia/Hebron
    {  389, 1259,  120597,  7, }, // Asia/Ho_Chi_Minh
    {  389, 2704,  120604,  7, }, // Asia/Hong_Kong
    {  389, 4771,  120611,  3, }, // Asia/Hovd
    {  389, 4184,  120614,  7, }, // Asia/Irkutsk
    {  389, 7006,  120621,  6, }, // Asia/Jakarta
    {  389, 6992,  120627,  8, }, // Asia/Jayapura
    {  389, 1290,  120635,  9, }, // Asia/Jerusalem
    {  389, 3560,  120644,  4, }, // Asia/Kabul
    {  389, 4316,  120648,  8, }, // Asia/Kamchatka
    {  389, 4239,  120656,  6, }, // Asia/Karachi
    {  389, 1185,  120662,  8, }, // Asia/Kathmandu
    {  389, 8661,  120670,  7, }, // Asia/Khandyga
    {  389,  992,  120677,  7, }, // Asia/Kolkata
    {  389, 4197,  120684, 11, }, // Asia/Krasnoyarsk
    {  389, 4859,  120695, 10, }, // Asia/Kuala_Lumpur
    {  389, 5976,  120705,  5, }, // Asia/Kuching
    {  389, 5547,  120710,  5, }, // Asia/Kuwait
    {  389, 1211,  120715,  6, }, // Asia/Macau
    {  389, 4017,  120721,  7, }, // Asia/Magadan
    {  389, 1349,  120728,  7, }, // Asia/Makassar
    {  389, 7182,  120735,  6, }, // Asia/Manila
    {  389, 6920,  120741,  6, }, // Asia/Muscat
    {  389, 2554,  120747,  7, }, // Asia/Nicosia
    {  389, 8675,  120754, 11, }, // Asia/Novokuznetsk
    {  389, 4135,  120765, 11, }, // Asia/Novosibirsk
    {  389, 4229,  120776,  6, }, // Asia/Omsk
    {  389, 7145,  120782,  4, }, // Asia/Oral
    {  389, 6961,  120786,  9, }, // Asia/Phnom_Penh
    {  389, 8693,  120795,  9, }, // Asia/Pontianak
    {  389, 4214,  120804,  6, }, // Asia/Pyongyang
    {  389, 5559,  120810,  6, }, // Asia/Qatar
    {  389, 8708,  120816,  7, }, // Asia/Qostanay
    {  389, 4269,  120823,  8, }, // Asia/Qyzylorda
    {  389, 3584,  120831,  5, }, // Asia/Riyadh
    {  389, 4408,  120836,  7, }, // Asia/Sakhalin
    {  389, 7245,  120843,  7, }, // Asia/Samarkand
    {  389, 3279,  120850,  4, }, // Asia/Seoul
    {  389, 1053,  120854,  7, }, // Asia/Shanghai
    {  389, 3300,  120861,  7, }, // Asia/Singapore
    {  389, 4297,  120868, 13, }, // Asia/Srednekolymsk
    {  389, 3263,  120881,  4, }, // Asia/Taipei
    {  389, 4781,  120885,  8, }, // Asia/Tashkent
    {  389, 3915,  120893,  8, }, // Asia/Tbilisi
    {  389, 2772,  120901,  5, }, // Asia/Tehran
    {  389, 1317,  120906,  5, }, // Asia/Thimphu
    {  389, 2821,  120911,  5, }, // Asia/Tokyo
    {  389, 4572,  120916,  6, }, // Asia/Tomsk
    {  389, 1021,  120922, 11, }, // Asia/Ulaanbaatar
    {  389, 1159,  120933,  7, }, // Asia/Urumqi
    {  389, 8722,  120940,  8, }, // Asia/Ust-Nera
    {  389, 6977,  120948,  6, }, // Asia/Vientiane
    {  389, 4710,  120954, 12, }, // Asia/Vladivostok
    {  389, 4828,  120966,  7, }, // Asia/Yakutsk
    {  389, 1235,  120973,  6, }, // Asia/Yangon
    {  389, 3883,  120979, 14, }, // Asia/Yekaterinburg
    {  389, 3747,  120993,  7, }, // Asia/Yerevan
    {  389, 3663,  121000, 10, }, // Atlantic/Azores
    {  389, 5671,  121010,  8, }, // Atlantic/Bermuda
    {  389, 6628,  121018,  6, }, // Atlantic/Canary
    {  389, 3727,  121024,  7, }, // Atlantic/Cape_Verde
    {  389, 1395,  121031,  4, }, // Atlantic/Faroe
    {  389, 8736,  121035,  6, }, // Atlantic/Madeira
    {  389, 2748,  121041,  8, }, // Atlantic/Reykjavik
    {  389, 7272,  121049,  9, }, // Atlantic/South_Georgia
    {  389, 6821,  121058, 10, }, // Atlantic/St_Helena
    {  389, 6644,  121068,  7, }, // Atlantic/Stanley
    {  389, 1670,  121075,  7, }, // Australia/Adelaide
    {  389, 1635,  121082,  7, }, // Australia/Brisbane
    {  389, 1799,  121089, 11, }, // Australia/Broken_Hill
    {  389, 1583,  121100,  5, }, // Australia/Darwin
    {  389, 3637,  121105,  5, }, // Australia/Eucla
    {  389, 1516,  121110,  5, }, // Australia/Hobart
    {  389, 8753,  121115,  8, }, // Australia/Lindeman
    {  389, 1547,  121123,  7, }, // Australia/Lord_Howe
    {  389, 1727,  121130,  7, }, // Australia/Melbourne
    {  389, 1762,  121137,  4, }, // Australia/Perth
    {  389, 1463,  121141,  5, }, // Australia/Sydney
    {  389, 6441,  121146,  9, }, // Europe/Amsterdam
    {  389, 6216,  121155,  7, }, // Europe/Andorra
    {  389, 3620,  121162,  9, }, // Europe/Astrakhan
    {  389, 2300,  121171,  7, }, // Europe/Athens
    {  389, 6336,  121178,  7, }, // Europe/Belgrade
    {  389, 4757,  121185,  6, }, // Europe/Berlin
    {  389, 6488,  121191, 10, }, // Europe/Bratislava
    {  389, 2167,  121201,  7, }, // Europe/Brussels
    {  389, 3928,  121208,  9, }, // Europe/Bucharest
    {  389, 3806,  121217,  9, }, // Europe/Budapest
    {  389, 8772,  121226,  9, }, // Europe/Busingen
    {  389, 2583,  121235,  8, }, // Europe/Chisinau
    {  389, 6289,  121243, 10, }, // Europe/Copenhagen
    {  389, 2338,  121253,  7, }, // Europe/Dublin
    {  389, 6307,  121260,  8, }, // Europe/Gibraltar
    {  389, 8788,  121268,  6, }, // Europe/Guernsey
    {  389, 6612,  121274,  8, }, // Europe/Helsinki
    {  389, 8804,  121282,  7, }, // Europe/Isle_of_Man
    {  389, 1130,  121289, 10, }, // Europe/Istanbul
    {  389, 8823,  121299,  5, }, // Europe/Jersey
    {  389, 3979,  121304, 11, }, // Europe/Kaliningrad
    {  389, 8929,  121315,  5, }, // Europe/Kirov
    {  389, 2527,  121320,  3, }, // Europe/Kyiv
    {  389, 3213,  121323,  6, }, // Europe/Lisbon
    {  389, 6506,  121329,  9, }, // Europe/Ljubljana
    {  389, 2501,  121338,  6, }, // Europe/London
    {  389, 6365,  121344, 11, }, // Europe/Luxembourg
    {  389, 4892,  121355,  6, }, // Europe/Madrid
    {  389, 6397,  121361,  5, }, // Europe/Malta
    {  389, 6582,  121366,  9, }, // Europe/Mariehamn
    {  389, 3693,  121375,  6, }, // Europe/Minsk
    {  389, 6410,  121381,  6, }, // Europe/Monaco
    {  389, 3537,  121387,  5, }, // Europe/Moscow
    {  389, 6458,  121392,  5, }, // Europe/Oslo
    {  389, 4284,  121397,  5, }, // Europe/Paris
    {  389, 6424,  121402,  9, }, // Europe/Podgorica
    {  389, 6275,  121411,  4, }, // Europe/Prague
    {  389, 8837,  119394,  4, }, // Europe/Riga
    {  389, 6324,  121033,  3, }, // Europe/Rome
    {  389, 4331,  121415,  6, }, // Europe/Samara
    {  389, 6470,  121421,  9, }, // Europe/San_Marino
    {  389, 6245,  121430,  8, }, // Europe/Sarajevo
    {  389, 4451,  121438,  7, }, // Europe/Saratov
    {  389, 8849,  121445,  9, }, // Europe/Simferopol
    {  389, 6383,  121454,  7, }, // Europe/Skopje
    {  389, 6599,  121461,  5, }, // Europe/Sofia
    {  389, 6523,  121466,  8, }, // Europe/Stockholm
    {  389, 8867,  121474,  8, }, // Europe/Tallinn
    {  389, 6202,  121482,  5, }, // Europe/Tirane
    {  389, 8882,  121487, 12, }, // Europe/Ulyanovsk
    {  389, 6352,  121499,  5, }, // Europe/Vaduz
    {  389, 6567,  121504,  7, }, // Europe/Vatican
    {  389, 6231,  121511,  5, }, // Europe/Vienna
    {  389, 8899,  121516,  8, }, // Europe/Vilnius
    {  389, 4727,  121524,  9, }, // Europe/Volgograd
    {  389, 3190,  121533,  4, }, // Europe/Warsaw
    {  389, 6261,  121537,  6, }, // Europe/Zagreb
    {  389, 6540,  121543,  5, }, // Europe/Zurich
    {  389, 5078,  121548, 13, }, // Indian/Antananarivo
    {  389, 6947,  121561,  5, }, // Indian/Chagos
    {  389, 6046,  121566,  9, }, // Indian/Christmas
    {  389, 6063,  121575,  5, }, // Indian/Cocos
    {  389, 5029,  121580,  6, }, // Indian/Comoro
    {  389, 6661,  121586,  8, }, // Indian/Kerguelen
    {  389, 7260,  121594,  4, }, // Indian/Mahe
    {  389, 7050,  121598,  6, }, // Indian/Maldives
    {  389, 4069,  121604,  9, }, // Indian/Mauritius
    {  389, 5098,  121613,  8, }, // Indian/Mayotte
    {  389, 7211,  121621,  9, }, // Indian/Reunion
    {  389, 4422,  121630,  6, }, // Pacific/Apia
    {  389,  932,  121636,  6, }, // Pacific/Auckland
    {  389, 3706,  121642, 19, }, // Pacific/Bougainville
    {  389, 3015,  121661,  5, }, // Pacific/Chatham
    {  389, 3157,  121666,  3, }, // Pacific/Chuuk
    {  389, 2237,  121669,  6, }, // Pacific/Easter
    {  389, 7360,  121675,  6, }, // Pacific/Efate
    {  389, 7327,  121681,  7, }, // Pacific/Fakaofo
    {  389, 3902,  121688,  4, }, // Pacific/Fiji
    {  389, 7343,  121692,  8, }, // Pacific/Funafuti
    {  389, 6678,  121700,  9, }, // Pacific/Galapagos
    {  389, 6696,  121709,  6, }, // Pacific/Gambier
    {  389, 3822,  121715, 11, }, // Pacific/Guadalcanal
    {  389, 6018,  121726,  3, }, // Pacific/Guam
    {  389, 2723,  121729,  8, }, // Pacific/Honolulu
    {  389, 3049,  121737, 12, }, // Pacific/Kanton
    {  389, 3998,  121749, 10, }, // Pacific/Kiritimati
    {  389, 7019,  121759,  5, }, // Pacific/Kosrae
    {  389, 2842,  121764,  8, }, // Pacific/Kwajalein
    {  389, 4877,  121772,  6, }, // Pacific/Majuro
    {  389, 4051,  121778,  8, }, // Pacific/Marquesas
    {  389, 8914,  121786,  5, }, // Pacific/Midway
    {  389, 7084,  121791,  6, }, // Pacific/Nauru
    {  389, 7132,  121797,  6, }, // Pacific/Niue
    {  389, 4168,  121803,  7, }, // Pacific/Norfolk
    {  389, 7098,  121810,  6, }, // Pacific/Noumea
    {  389, 3126,  121816,  8, }, // Pacific/Pago_Pago
    {  389, 7155,  121824,  6, }, // Pacific/Palau
    {  389, 7194,  121830,  6, }, // Pacific/Pitcairn
    {  389, 3096,  121836,  7, }, // Pacific/Pohnpei
    {  389, 4807,  121843, 11, }, // Pacific/Port_Moresby
    {  389, 6076,  121854,  9, }, // Pacific/Rarotonga
    {  389, 6031,  121863,  5, }, // Pacific/Saipan
    {  389, 7312,  121868,  6, }, // Pacific/Tahiti
    {  389, 6712,  121874,  6, }, // Pacific/Tarawa
    {  389, 4583,  121880,  9, }, // Pacific/Tongatapu
    {  389, 7392,  121889,  3, }, // Pacific/Wake
    {  389, 7405,  121892,  5, }, // Pacific/Wallis
    {  390,   14,       0,  6, }, // Africa/Asmara Latin/Latin/Vatican City
    {  390, 4435,       6,  8, }, // Africa/Sao_Tome
    {  390, 4252,      14,  8, }, // America/Asuncion
    {  390,  237,      22,  8, }, // America/Atikokan
    {  390, 7623,      30, 17, }, // America/Bahia_Banderas
    {  390, 3868,      47,  6, }, // America/Cancun
    {  390, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  390, 5723,      66,  7, }, // America/Curacao
    {  390,  481,      73, 13, }, // America/Indiana/Knox
    {  390, 7961,      86, 16, }, // America/Indiana/Marengo
    {  390, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  390, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  390, 8038,     139, 14, }, // America/Indiana/Vevay
    {  390, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  390, 8086,     171, 16, }, // America/Indiana/Winamac
    {  390, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  390, 5932,     207, 22, }, // America/Lower_Princes
    {  390, 8220,     229,  6, }, // America/Merida
    {  390, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  390, 8329,     255, 20, }, // America/North_Dakota/Center
    {  390, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  390,  393,     298,  4, }, // America/Nuuk
    {  390, 6899,     302, 16, }, // America/Scoresbysund
    {  390, 8508,     318, 14, }, // America/St_Barthelemy
    {  390, 2061,     332, 10, }, // America/St_Johns
    {  390, 5863,     342,  9, }, // America/St_Kitts
    {  390, 5880,     351,  9, }, // America/St_Lucia
    {  390,  855,     360, 10, }, // America/St_Thomas
    {  390, 5913,     370, 11, }, // America/St_Vincent
    {  390, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  390, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  390, 1185,     413,  9, }, // Asia/Kathmandu
    {  390,  992,     422,  7, }, // Asia/Kolkata
    {  390, 1211,     429,  5, }, // Asia/Macau
    {  390, 1235,     442,  6, }, // Asia/Yangon
    {  390, 1395,     448,  5, }, // Atlantic/Faroe
    {  390, 6821,     453, 10, }, // Atlantic/St_Helena
    {  390, 2527,     463,  4, }, // Europe/Kyiv
    {  390, 7211,     467,  7, }, // Indian/Reunion
    {  390, 3157,     474,  5, }, // Pacific/Chuuk
    {  390, 3096,     479,  7, }, // Pacific/Pohnpei
    {  391, 6788,  121897,  8, }, // Africa/Abidjan Latvian/Latin/Latvia
    {  391, 6760,    4531,  4, }, // Africa/Accra
    {  391, 5059,  121905,  9, }, // Africa/Addis_Ababa
    {  391, 7420,  121914,  6, }, // Africa/Algiers
    {  391,   14,       0,  6, }, // Africa/Asmara
    {  391, 5257,   26348,  5, }, // Africa/Bangui
    {  391, 6746,  121920,  8, }, // Africa/Banjul
    {  391, 7435,  121928,  6, }, // Africa/Bissau
    {  391, 4957,  121934,  9, }, // Africa/Blantyre
    {  391, 5287,  121943,  9, }, // Africa/Brazzaville
    {  391, 4922,  121952,  9, }, // Africa/Bujumbura
    {  391, 2320,  121961,  5, }, // Africa/Cairo
    {  391, 4117,    4577, 10, }, // Africa/Casablanca
    {  391, 7449,   21231,  5, }, // Africa/Ceuta
    {  391, 6773,    6249,  7, }, // Africa/Conakry
    {  391, 6840,  121966,  6, }, // Africa/Dakar
    {  391, 5130,  121972, 10, }, // Africa/Dar_es_Salaam
    {  391, 5043,  121982,  9, }, // Africa/Djibouti
    {  391, 5243,   21254,  5, }, // Africa/Douala
    {  391, 5166,  121991,  5, }, // Africa/El_Aaiun
    {  391, 6853,  121996,  8, }, // Africa/Freetown
    {  391, 4479,  122004, 13, }, // Africa/Johannesburg
    {  391, 4499,  122017,  5, }, // Africa/Juba
    {  391, 4524,  122022,  7, }, // Africa/Khartoum
    {  391, 5306,  122029,  7, }, // Africa/Kinshasa
    {  391, 4744,  122036,  6, }, // Africa/Lagos
    {  391, 5336,  122042,  9, }, // Africa/Libreville
    {  391, 4939,  122051,  9, }, // Africa/Lubumbashi
    {  391, 4987,  122060,  6, }, // Africa/Maputo
    {  391, 5113,  122066,  8, }, // Africa/Mogadishu
    {  391, 7034,  122074,  9, }, // Africa/Monrovia
    {  391, 5271,   57597,  8, }, // Africa/Ndjamena
    {  391, 5354,  122083,  7, }, // Africa/Niamey
    {  391, 6803,  122090,  8, }, // Africa/Nouakchott
    {  391, 6727,   61232,  8, }, // Africa/Ouagadougou
    {  391, 5225,   61241,  9, }, // Africa/Porto-Novo
    {  391, 4435,   61250,  7, }, // Africa/Sao_Tome
    {  391, 2866,  122098,  7, }, // Africa/Tripoli
    {  391, 6554,  122105,  6, }, // Africa/Tunis
    {  391, 4152,  122111,  8, }, // Africa/Windhoek
    {  391,  132,  122119,  5, }, // America/Adak
    {  391, 3346,  122124,  8, }, // America/Anchorage
    {  391, 5607,  122132,  7, }, // America/Anguilla
    {  391, 5624,   33175,  7, }, // America/Antigua
    {  391, 4554,  122139,  9, }, // America/Araguaina
    {  391,  166,  122148, 12, }, // America/Argentina/Buenos_Aires
    {  391,   91,    6400,  9, }, // America/Argentina/Catamarca
    {  391,  270,  122160,  7, }, // America/Argentina/Cordoba
    {  391,  441,  122167,  6, }, // America/Argentina/Jujuy
    {  391, 7462,  122173,  7, }, // America/Argentina/La_Rioja
    {  391,  565,   97505,  7, }, // America/Argentina/Mendoza
    {  391, 7489,  122180, 12, }, // America/Argentina/Rio_Gallegos
    {  391, 7544,  122192,  8, }, // America/Argentina/San_Juan
    {  391, 5580,  122200,  8, }, // America/Argentina/San_Luis
    {  391, 7571,  122208,  8, }, // America/Argentina/Tucuman
    {  391, 7597,  122216,  6, }, // America/Argentina/Ushuaia
    {  391, 4252,  122222,  9, }, // America/Asuncion
    {  391,  237,  122231,  9, }, // America/Atikokan
    {  391, 3679,  122240,  5, }, // America/Bahia
    {  391, 7623,  122245, 13, }, // America/Bahia_Banderas
    {  391, 5654,  122258,  9, }, // America/Barbados
    {  391, 7646,  122267,  6, }, // America/Belem
    {  391, 5407,  122273,  6, }, // America/Belize
    {  391, 7660,  122279, 11, }, // America/Blanc-Sablon
    {  391, 7681,  122290,  8, }, // America/Boa_Vista
    {  391, 7699,  122298, 11, }, // America/Boise
    {  391, 7713,  122309, 12, }, // America/Cambridge_Bay
    {  391, 7735,  122321, 11, }, // America/Campo_Grande
    {  391, 3868,  122332,  7, }, // America/Cancun
    {  391, 4694,  122339,  8, }, // America/Caracas
    {  391, 4345,  122347,  7, }, // America/Cayenne
    {  391, 5496,  122354, 13, }, // America/Cayman
    {  391, 2260,  122367,  6, }, // America/Chicago
    {  391, 7756,  122373,  6, }, // America/Chihuahua
    {  391, 7774,  122379,  7, }, // America/Ciudad_Juarez
    {  391, 5422,   33188,  9, }, // America/Costa_Rica
    {  391, 7796,  122386,  8, }, // America/Creston
    {  391, 3791,    6625,  6, }, // America/Cuiaba
    {  391, 5723,  122394,  7, }, // America/Curacao
    {  391, 7812,  122401, 12, }, // America/Danmarkshavn
    {  391, 7833,  122413,  7, }, // America/Dawson
    {  391, 7848,  122420, 11, }, // America/Dawson_Creek
    {  391,  805,  122431,  7, }, // America/Denver
    {  391, 3465,  122438,  8, }, // America/Detroit
    {  391, 5739,   21548,  8, }, // America/Dominica
    {  391,  893,  122446,  9, }, // America/Edmonton
    {  391, 5441,  122455,  9, }, // America/El_Salvador
    {  391, 7886,  122464, 11, }, // America/Fort_Nelson
    {  391, 7924,  122475,  9, }, // America/Glace_Bay
    {  391, 6881,  122484,  7, }, // America/Goose_Bay
    {  391, 4612,  122491, 11, }, // America/Grand_Turk
    {  391, 5770,  122502,  7, }, // America/Grenada
    {  391, 5786,  122509,  9, }, // America/Guadeloupe
    {  391, 3760,   33206,  9, }, // America/Guatemala
    {  391, 4841,  122518,  9, }, // America/Guayaquil
    {  391, 6932,  122527,  6, }, // America/Guyana
    {  391, 1939,  122533,  9, }, // America/Halifax
    {  391, 7942,  122542,  9, }, // America/Hermosillo
    {  391,  348,  122551, 13, }, // America/Indiana/Indianapolis
    {  391,  481,  122564, 14, }, // America/Indiana/Knox
    {  391, 7961,  122578, 16, }, // America/Indiana/Marengo
    {  391, 7985,  122594, 20, }, // America/Indiana/Petersburg
    {  391, 8012,  122614, 18, }, // America/Indiana/Tell_City
    {  391, 8038,  122632, 15, }, // America/Indiana/Vevay
    {  391, 8060,  122647, 17, }, // America/Indiana/Vincennes
    {  391, 8086,  122664, 17, }, // America/Indiana/Winamac
    {  391, 8110,  122681,  7, }, // America/Inuvik
    {  391,  660,  122688,  8, }, // America/Iqaluit
    {  391, 2799,    3877,  7, }, // America/Jamaica
    {  391, 5380,  122696,  5, }, // America/Juneau
    {  391,  521,  122701,  7, }, // America/Kentucky/Louisville
    {  391, 8125,  122708, 18, }, // America/Kentucky/Monticello
    {  391, 5704,  122726, 11, }, // America/Kralendijk
    {  391, 4376,  122737,  6, }, // America/La_Paz
    {  391, 3239,  122743, 12, }, // America/Los_Angeles
    {  391, 5932,  122755, 19, }, // America/Lower_Princes
    {  391, 8153,  122774,  6, }, // America/Maceio
    {  391, 8168,  122780,  7, }, // America/Managua
    {  391, 1908,  122787,  7, }, // America/Manaus
    {  391, 5897,  122794,  8, }, // America/Marigot
    {  391, 5805,   26462,  9, }, // America/Martinique
    {  391, 8184,  122802, 10, }, // America/Matamoros
    {  391, 2917,  122812,  9, }, // America/Mazatlan
    {  391, 8202,  122821,  8, }, // America/Menominee
    {  391, 8220,    7008,  6, }, // America/Merida
    {  391, 2949,  122829,  6, }, // America/Mexico_City
    {  391, 4391,  122835,  8, }, // America/Miquelon
    {  391, 8254,  122843,  8, }, // America/Moncton
    {  391, 8270,  122851, 10, }, // America/Monterrey
    {  391, 5824,  122861, 11, }, // America/Montserrat
    {  391, 5481,  122872,  4, }, // America/Nassau
    {  391, 2379,  122876,  7, }, // America/New_York
    {  391, 8288,  122883,  4, }, // America/Nome
    {  391, 1850,  122887,  6, }, // America/Noronha
    {  391, 8301,  122893, 19, }, // America/North_Dakota/Beulah
    {  391, 8329,  122912, 21, }, // America/North_Dakota/Center
    {  391, 8357,  122933, 23, }, // America/North_Dakota/New_Salem
    {  391,  393,  122956,  4, }, // America/Nuuk
    {  391, 8388,    7158,  7, }, // America/Ojinaga
    {  391, 2973,  122960,  7, }, // America/Phoenix
    {  391, 3945,  122967, 11, }, // America/Port-au-Prince
    {  391, 5954,  122978, 12, }, // America/Port_of_Spain
    {  391, 8404,  122990,  9, }, // America/Porto_Velho
    {  391, 5843,   61940, 10, }, // America/Puerto_Rico
    {  391, 4030,  122999, 12, }, // America/Punta_Arenas
    {  391, 8424,  123011, 12, }, // America/Rankin_Inlet
    {  391, 8445,  123023,  6, }, // America/Recife
    {  391, 1995,  123029,  8, }, // America/Regina
    {  391, 8460,  123037,  8, }, // America/Resolute
    {  391,  695,  123045,  9, }, // America/Rio_Branco
    {  391, 8477,  123054,  9, }, // America/Santarem
    {  391, 2201,  123063,  8, }, // America/Santiago
    {  391, 6111,  123071, 12, }, // America/Santo_Domingo
    {  391, 1878,  123083,  8, }, // America/Sao_Paulo
    {  391, 6899,  123091, 13, }, // America/Scoresbysund
    {  391, 8508,  123104, 11, }, // America/St_Barthelemy
    {  391, 2061,  123115, 10, }, // America/St_Johns
    {  391, 5863,  123125,  9, }, // America/St_Kitts
    {  391, 5880,  123134, 10, }, // America/St_Lucia
    {  391,  855,  123144,  9, }, // America/St_Thomas
    {  391, 5913,  123153, 12, }, // America/St_Vincent
    {  391, 8530,  123165, 12, }, // America/Swift_Current
    {  391, 5461,    5410, 11, }, // America/Tegucigalpa
    {  391, 5756,  123177,  4, }, // America/Thule
    {  391,  313,    7361,  7, }, // America/Tijuana
    {  391, 2093,  123181,  9, }, // America/Vancouver
    {  391, 2144,  123190,  9, }, // America/Whitehorse
    {  391,  734,  123199,  8, }, // America/Winnipeg
    {  391, 8552,  123207,  8, }, // America/Yakutat
    {  391, 6001,  123215,  5, }, // Antarctica/Casey
    {  391, 6094,  123220,  7, }, // Antarctica/Davis
    {  391, 6133,  123227, 12, }, // Antarctica/DumontDUrville
    {  391, 8568,  123239,  7, }, // Antarctica/Macquarie
    {  391, 7066,  123246,  6, }, // Antarctica/Mawson
    {  391, 7113,  123252,  8, }, // Antarctica/McMurdo
    {  391, 8589,  123260,  7, }, // Antarctica/Palmer
    {  391, 7226,    5465,  6, }, // Antarctica/Rothera
    {  391, 7295,  123267,  4, }, // Antarctica/Syowa
    {  391, 8607,  123271,  6, }, // Antarctica/Troll
    {  391, 7374,  123277,  7, }, // Antarctica/Vostok
    {  391, 1429,  123284, 12, }, // Arctic/Longyearbyen
    {  391, 5570,  123296,  5, }, // Asia/Aden
    {  391, 5395,    7476,  6, }, // Asia/Almaty
    {  391, 3968,  123301,  6, }, // Asia/Amman
    {  391, 5511,  123307,  7, }, // Asia/Anadyr
    {  391, 5523,    5491,  5, }, // Asia/Aqtau
    {  391, 5368,  123314,  6, }, // Asia/Aqtobe
    {  391,  964,  123320,  8, }, // Asia/Ashgabat
    {  391, 8624,    7494,  6, }, // Asia/Atyrau
    {  391, 3607,  123328,  7, }, // Asia/Baghdad
    {  391, 5534,  123335,  8, }, // Asia/Bahrain
    {  391, 4466,  123343,  8, }, // Asia/Bangkok
    {  391, 3571,  123351,  8, }, // Asia/Barnaul
    {  391, 4086,  123359,  7, }, // Asia/Beirut
    {  391, 3778,  123366,  7, }, // Asia/Bishkek
    {  391, 5989,  123373,  7, }, // Asia/Brunei
    {  391, 4601,   57490,  4, }, // Asia/Chita
    {  391, 4511,    5519,  7, }, // Asia/Colombo
    {  391, 4540,  123380,  7, }, // Asia/Damascus
    {  391, 1093,    4599,  4, }, // Asia/Dhaka
    {  391, 3596,  123387,  7, }, // Asia/Dubai
    {  391, 6159,   33336,  7, }, // Asia/Dushanbe
    {  391, 4795,  123394,  7, }, // Asia/Hebron
    {  391, 1259,  123401,  8, }, // Asia/Ho_Chi_Minh
    {  391, 2704,  123409,  8, }, // Asia/Hong_Kong
    {  391, 4771,  123417,  5, }, // Asia/Hovd
    {  391, 4184,  123422,  8, }, // Asia/Irkutsk
    {  391, 7006,   33352,  8, }, // Asia/Jakarta
    {  391, 6992,   33360,  9, }, // Asia/Jayapura
    {  391, 1290,  123430, 10, }, // Asia/Jerusalem
    {  391, 3560,  123440,  6, }, // Asia/Kabul
    {  391, 4316,   33378,  8, }, // Asia/Kamchatka
    {  391, 4239,  123446,  6, }, // Asia/Karachi
    {  391, 1185,    5593,  8, }, // Asia/Kathmandu
    {  391, 8661,   33392,  7, }, // Asia/Khandyga
    {  391,  992,  123452,  7, }, // Asia/Kolkata
    {  391, 4197,  123459, 12, }, // Asia/Krasnoyarsk
    {  391, 4859,  123471, 12, }, // Asia/Kuala_Lumpur
    {  391, 5976,  123483,  6, }, // Asia/Kuching
    {  391, 5547,  123489,  7, }, // Asia/Kuwait
    {  391, 1211,    7613,  5, }, // Asia/Macau
    {  391, 4017,  123496,  8, }, // Asia/Magadan
    {  391, 1349,  123504,  8, }, // Asia/Makassar
    {  391, 6920,  123512,  7, }, // Asia/Muscat
    {  391, 2554,   57151,  8, }, // Asia/Nicosia
    {  391, 8675,  123519, 12, }, // Asia/Novokuznetsk
    {  391, 4135,  123531, 12, }, // Asia/Novosibirsk
    {  391, 4229,  123543,  5, }, // Asia/Omsk
    {  391, 7145,  123548,  5, }, // Asia/Oral
    {  391, 6961,  123553,  8, }, // Asia/Phnom_Penh
    {  391, 8693,  123561, 10, }, // Asia/Pontianak
    {  391, 4214,  123571,  8, }, // Asia/Pyongyang
    {  391, 5559,  123579,  6, }, // Asia/Qatar
    {  391, 8708,  123585,  9, }, // Asia/Qostanay
    {  391, 4269,    7660,  9, }, // Asia/Qyzylorda
    {  391, 3584,  123594,  6, }, // Asia/Riyadh
    {  391, 4408,  123600,  8, }, // Asia/Sakhalin
    {  391, 7245,   26774, 10, }, // Asia/Samarkand
    {  391, 3279,  123608,  5, }, // Asia/Seoul
    {  391, 1053,  123613,  7, }, // Asia/Shanghai
    {  391, 3300,  123620,  9, }, // Asia/Singapore
    {  391, 4297,  123629, 14, }, // Asia/Srednekolymsk
    {  391, 3263,  123643,  6, }, // Asia/Taipei
    {  391, 4781,  123649,  8, }, // Asia/Tashkent
    {  391, 2772,  123657,  8, }, // Asia/Tehran
    {  391, 2821,  123665,  6, }, // Asia/Tokyo
    {  391, 4572,  123671,  6, }, // Asia/Tomsk
    {  391, 1021,  123677, 10, }, // Asia/Ulaanbaatar
    {  391, 1159,   33464,  6, }, // Asia/Urumqi
    {  391, 8722,  123687,  8, }, // Asia/Ust-Nera
    {  391, 6977,  123695,  8, }, // Asia/Vientiane
    {  391, 4710,  123703, 12, }, // Asia/Vladivostok
    {  391, 4828,  123715,  8, }, // Asia/Yakutsk
    {  391, 1235,  123723,  7, }, // Asia/Yangon
    {  391, 3883,  123730, 14, }, // Asia/Yekaterinburg
    {  391, 3747,  123744,  7, }, // Asia/Yerevan
    {  391, 3663,  123751, 11, }, // Atlantic/Azores
    {  391, 6628,  123762, 14, }, // Atlantic/Canary
    {  391, 3727,  123776,  9, }, // Atlantic/Cape_Verde
    {  391, 1395,  123785, 10, }, // Atlantic/Faroe
    {  391, 2748,  123795, 10, }, // Atlantic/Reykjavik
    {  391, 7272,  123805, 16, }, // Atlantic/South_Georgia
    {  391, 6821,  123821, 15, }, // Atlantic/St_Helena
    {  391, 6644,    7847,  6, }, // Atlantic/Stanley
    {  391, 1670,   20912,  8, }, // Australia/Adelaide
    {  391, 1635,  123836,  8, }, // Australia/Brisbane
    {  391, 1799,  123844, 10, }, // Australia/Broken_Hill
    {  391, 1583,  123854,  7, }, // Australia/Darwin
    {  391, 3637,  123861,  5, }, // Australia/Eucla
    {  391, 1516,  123866,  7, }, // Australia/Hobart
    {  391, 8753,  123873,  9, }, // Australia/Lindeman
    {  391, 1547,  123882,  8, }, // Australia/Lord_Howe
    {  391, 1727,  123890,  8, }, // Australia/Melbourne
    {  391, 1762,  123898,  5, }, // Australia/Perth
    {  391, 1463,  123903,  7, }, // Australia/Sydney
    {  391, 6441,  123910, 10, }, // Europe/Amsterdam
    {  391, 6216,   33506,  6, }, // Europe/Andorra
    {  391, 3620,  123920,  9, }, // Europe/Astrakhan
    {  391, 2300,  123929,  6, }, // Europe/Athens
    {  391, 6336,  123935,  8, }, // Europe/Belgrade
    {  391, 4757,  123943,  7, }, // Europe/Berlin
    {  391, 2167,  123950,  7, }, // Europe/Brussels
    {  391, 3928,  123957,  9, }, // Europe/Bucharest
    {  391, 3806,  123966,  9, }, // Europe/Budapest
    {  391, 8772,  123975,  9, }, // Europe/Busingen
    {  391, 2583,  123984,  8, }, // Europe/Chisinau
    {  391, 6289,  123992, 11, }, // Europe/Copenhagen
    {  391, 2338,  124003,  7, }, // Europe/Dublin
    {  391, 6307,  124010, 10, }, // Europe/Gibraltar
    {  391, 8788,  124020,  8, }, // Europe/Guernsey
    {  391, 8804,  124028, 10, }, // Europe/Isle_of_Man
    {  391, 1130,  124038,  8, }, // Europe/Istanbul
    {  391, 8823,  124046,  8, }, // Europe/Jersey
    {  391, 3979,  124054, 12, }, // Europe/Kaliningrad
    {  391, 8929,  124066,  6, }, // Europe/Kirov
    {  391, 2527,  124072,  6, }, // Europe/Kyiv
    {  391, 3213,  124078,  8, }, // Europe/Lisbon
    {  391, 6506,  124086,  7, }, // Europe/Ljubljana
    {  391, 2501,  124093,  7, }, // Europe/London
    {  391, 6365,  124100, 11, }, // Europe/Luxembourg
    {  391, 4892,  124111,  7, }, // Europe/Madrid
    {  391, 6582,  124118, 10, }, // Europe/Mariehamn
    {  391, 3693,  124128,  6, }, // Europe/Minsk
    {  391, 6410,    5926,  6, }, // Europe/Monaco
    {  391, 3537,  124134,  7, }, // Europe/Moscow
    {  391, 4284,  124141,  6, }, // Europe/Paris
    {  391, 6275,  124147,  5, }, // Europe/Prague
    {  391, 8837,   73299,  4, }, // Europe/Riga
    {  391, 6324,    5946,  4, }, // Europe/Rome
    {  391, 6470,  124152,  9, }, // Europe/San_Marino
    {  391, 6245,  124161,  8, }, // Europe/Sarajevo
    {  391, 4451,  124169,  8, }, // Europe/Saratov
    {  391, 8849,  124177, 11, }, // Europe/Simferopol
    {  391, 6599,   33629,  6, }, // Europe/Sofia
    {  391, 6523,  124188,  9, }, // Europe/Stockholm
    {  391, 8867,  124197,  7, }, // Europe/Tallinn
    {  391, 6202,  124204,  6, }, // Europe/Tirane
    {  391, 8882,  124210, 10, }, // Europe/Ulyanovsk
    {  391, 6352,  124220,  6, }, // Europe/Vaduz
    {  391, 6567,  124226,  8, }, // Europe/Vatican
    {  391, 6231,  124234,  4, }, // Europe/Vienna
    {  391, 8899,  124238,  5, }, // Europe/Vilnius
    {  391, 4727,  124243, 10, }, // Europe/Volgograd
    {  391, 3190,   33646,  7, }, // Europe/Warsaw
    {  391, 6261,  124253,  7, }, // Europe/Zagreb
    {  391, 6540,  124260,  6, }, // Europe/Zurich
    {  391, 5078,   99232, 12, }, // Indian/Antananarivo
    {  391, 6947,  124266,  6, }, // Indian/Chagos
    {  391, 6046,  124272, 15, }, // Indian/Christmas
    {  391, 6063,  124287, 21, }, // Indian/Cocos
    {  391, 5029,  124308,  7, }, // Indian/Comoro
    {  391, 6661,  124315, 14, }, // Indian/Kerguelen
    {  391, 7260,  124329,  3, }, // Indian/Mahe
    {  391, 7050,  124332,  9, }, // Indian/Maldives
    {  391, 4069,  124341,  9, }, // Indian/Mauritius
    {  391, 5098,  124350,  6, }, // Indian/Mayotte
    {  391, 7211,  124356,  8, }, // Indian/Reunion
    {  391, 4422,  124364,  5, }, // Pacific/Apia
    {  391,  932,  124369,  7, }, // Pacific/Auckland
    {  391, 3706,  124376, 14, }, // Pacific/Bougainville
    {  391, 3015,  124390,  6, }, // Pacific/Chatham
    {  391, 3157,  124396,  4, }, // Pacific/Chuuk
    {  391, 2237,  124400, 14, }, // Pacific/Easter
    {  391, 3902,   33704,  5, }, // Pacific/Fiji
    {  391, 6678,  124414, 14, }, // Pacific/Galapagos
    {  391, 6696,  124428, 12, }, // Pacific/Gambier
    {  391, 3822,  124440, 12, }, // Pacific/Guadalcanal
    {  391, 6018,  124452,  5, }, // Pacific/Guam
    {  391, 2723,    4493,  8, }, // Pacific/Honolulu
    {  391, 3049,  124457, 11, }, // Pacific/Kanton
    {  391, 3998,  124468, 10, }, // Pacific/Kiritimati
    {  391, 7019,  124478,  7, }, // Pacific/Kosrae
    {  391, 2842,  124485, 11, }, // Pacific/Kwajalein
    {  391, 4877,  124496,  7, }, // Pacific/Majuro
    {  391, 4051,  124503, 13, }, // Pacific/Marquesas
    {  391, 8914,  124516,  7, }, // Pacific/Midway
    {  391, 4168,  124523,  8, }, // Pacific/Norfolk
    {  391, 7098,  124531,  5, }, // Pacific/Noumea
    {  391, 3126,   63298,  8, }, // Pacific/Pago_Pago
    {  391, 7194,  124536,  8, }, // Pacific/Pitcairn
    {  391, 3096,  124544,  7, }, // Pacific/Pohnpei
    {  391, 4807,  124551, 10, }, // Pacific/Port_Moresby
    {  391, 6031,  124561,  7, }, // Pacific/Saipan
    {  391, 7312,  124568,  5, }, // Pacific/Tahiti
    {  391, 6712,   23027,  6, }, // Pacific/Tarawa
    {  391, 7392,  124573, 10, }, // Pacific/Wake
    {  391, 7405,  124583,  6, }, // Pacific/Wallis
    {  392,   14,       0,  6, }, // Africa/Asmara Lingala/Latin/Congo - Kinshasa
    {  392, 4435,       6,  8, }, // Africa/Sao_Tome
    {  392, 4252,      14,  8, }, // America/Asuncion
    {  392,  237,      22,  8, }, // America/Atikokan
    {  392, 7623,      30, 17, }, // America/Bahia_Banderas
    {  392, 3868,      47,  6, }, // America/Cancun
    {  392, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  392, 5723,      66,  7, }, // America/Curacao
    {  392,  481,      73, 13, }, // America/Indiana/Knox
    {  392, 7961,      86, 16, }, // America/Indiana/Marengo
    {  392, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  392, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  392, 8038,     139, 14, }, // America/Indiana/Vevay
    {  392, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  392, 8086,     171, 16, }, // America/Indiana/Winamac
    {  392, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  392, 5932,     207, 22, }, // America/Lower_Princes
    {  392, 8220,     229,  6, }, // America/Merida
    {  392, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  392, 8329,     255, 20, }, // America/North_Dakota/Center
    {  392, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  392,  393,     298,  4, }, // America/Nuuk
    {  392, 6899,     302, 16, }, // America/Scoresbysund
    {  392, 8508,     318, 14, }, // America/St_Barthelemy
    {  392, 2061,     332, 10, }, // America/St_Johns
    {  392, 5863,     342,  9, }, // America/St_Kitts
    {  392, 5880,     351,  9, }, // America/St_Lucia
    {  392,  855,     360, 10, }, // America/St_Thomas
    {  392, 5913,     370, 11, }, // America/St_Vincent
    {  392, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  392, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  392, 1185,     413,  9, }, // Asia/Kathmandu
    {  392,  992,     422,  7, }, // Asia/Kolkata
    {  392, 1211,     429,  5, }, // Asia/Macau
    {  392, 1235,     442,  6, }, // Asia/Yangon
    {  392, 1395,     448,  5, }, // Atlantic/Faroe
    {  392, 6821,     453, 10, }, // Atlantic/St_Helena
    {  392, 2527,     463,  4, }, // Europe/Kyiv
    {  392, 7211,     467,  7, }, // Indian/Reunion
    {  392, 3157,     474,  5, }, // Pacific/Chuuk
    {  392, 3096,     479,  7, }, // Pacific/Pohnpei
    {  396, 6788,  124589,  9, }, // Africa/Abidjan Lithuanian/Latin/Lithuania
    {  396, 6760,    4531,  4, }, // Africa/Accra
    {  396, 5059,   35689, 10, }, // Africa/Addis_Ababa
    {  396, 7420,  124598,  7, }, // Africa/Algiers
    {  396,   14,       0,  6, }, // Africa/Asmara
    {  396,   44,  124605,  7, }, // Africa/Bamako
    {  396, 5257,  124612,  6, }, // Africa/Bangui
    {  396, 6746,  124618,  9, }, // Africa/Banjul
    {  396, 7435,    6195,  5, }, // Africa/Bissau
    {  396, 4957,  124627, 10, }, // Africa/Blantyre
    {  396, 5287,  124637, 10, }, // Africa/Brazzaville
    {  396, 4922,  124647,  9, }, // Africa/Bujumbura
    {  396, 2320,  124656,  6, }, // Africa/Cairo
    {  396, 4117,    4577, 10, }, // Africa/Casablanca
    {  396, 7449,   21231,  5, }, // Africa/Ceuta
    {  396, 6773,  124662,  8, }, // Africa/Conakry
    {  396, 6840,  124670,  7, }, // Africa/Dakar
    {  396, 5130,  124677, 14, }, // Africa/Dar_es_Salaam
    {  396, 5043,  124691,  8, }, // Africa/Djibouti
    {  396, 5243,   21254,  5, }, // Africa/Douala
    {  396, 5166,  124699,  9, }, // Africa/El_Aaiun
    {  396, 6853,  124708,  9, }, // Africa/Freetown
    {  396, 4906,  124717,  9, }, // Africa/Gaborone
    {  396, 5015,  124726,  6, }, // Africa/Harare
    {  396, 4479,  124732, 13, }, // Africa/Johannesburg
    {  396, 4499,  124745,  5, }, // Africa/Juba
    {  396, 4524,  124750,  9, }, // Africa/Khartoum
    {  396, 4973,  124759,  7, }, // Africa/Kigali
    {  396, 5306,  122029,  7, }, // Africa/Kinshasa
    {  396, 4744,  124766,  7, }, // Africa/Lagos
    {  396, 5336,  124773, 10, }, // Africa/Libreville
    {  396, 6869,  124783,  4, }, // Africa/Lome
    {  396, 4939,  124787, 10, }, // Africa/Lubumbashi
    {  396, 5322,  124797,  7, }, // Africa/Malabo
    {  396, 4987,  124804,  7, }, // Africa/Maputo
    {  396, 5182,  124811,  7, }, // Africa/Mbabane
    {  396, 5113,  124818,  9, }, // Africa/Mogadishu
    {  396, 7034,  122074,  9, }, // Africa/Monrovia
    {  396, 3853,  124827,  8, }, // Africa/Nairobi
    {  396, 5271,   57597,  8, }, // Africa/Ndjamena
    {  396, 5354,  124835,  8, }, // Africa/Niamey
    {  396, 6803,  124843,  9, }, // Africa/Nouakchott
    {  396, 6727,   61232,  8, }, // Africa/Ouagadougou
    {  396, 5225,  124852, 11, }, // Africa/Porto-Novo
    {  396, 4435,  124863,  8, }, // Africa/Sao_Tome
    {  396, 2866,   57624,  8, }, // Africa/Tripoli
    {  396, 6554,  124871,  7, }, // Africa/Tunis
    {  396, 4152,  124878,  9, }, // Africa/Windhoek
    {  396,  132,  124887,  7, }, // America/Adak
    {  396, 3346,  124894, 10, }, // America/Anchorage
    {  396, 5607,  124904,  8, }, // America/Anguilla
    {  396, 5624,   33175,  7, }, // America/Antigua
    {  396, 4554,  124912, 10, }, // America/Araguaina
    {  396,  166,  124922, 12, }, // America/Argentina/Buenos_Aires
    {  396,   91,    6400,  9, }, // America/Argentina/Catamarca
    {  396,  270,    6409,  7, }, // America/Argentina/Cordoba
    {  396,  441,  124934,  9, }, // America/Argentina/Jujuy
    {  396, 7462,  124943,  9, }, // America/Argentina/La_Rioja
    {  396,  565,   97505,  7, }, // America/Argentina/Mendoza
    {  396, 7489,  124952, 14, }, // America/Argentina/Rio_Gallegos
    {  396, 7520,  124966,  6, }, // America/Argentina/Salta
    {  396, 7544,  124972, 11, }, // America/Argentina/San_Juan
    {  396, 5580,  124983, 11, }, // America/Argentina/San_Luis
    {  396, 7571,  124994,  9, }, // America/Argentina/Tucuman
    {  396, 7597,  122216,  6, }, // America/Argentina/Ushuaia
    {  396, 4252,  125003, 10, }, // America/Asuncion
    {  396,  237,  125013, 10, }, // America/Atikokan
    {  396, 3679,  122240,  5, }, // America/Bahia
    {  396, 7623,  125023, 16, }, // America/Bahia_Banderas
    {  396, 5654,  125039, 10, }, // America/Barbados
    {  396, 7646,  125049,  7, }, // America/Belem
    {  396, 5407,  125056,  7, }, // America/Belize
    {  396, 7660,  125063, 14, }, // America/Blanc-Sablon
    {  396, 7681,  125077,  9, }, // America/Boa_Vista
    {  396, 7699,  122298,  6, }, // America/Boise
    {  396, 7713,  125086, 15, }, // America/Cambridge_Bay
    {  396, 7735,  125101, 12, }, // America/Campo_Grande
    {  396, 3868,  125113,  8, }, // America/Cancun
    {  396, 4694,  125121,  9, }, // America/Caracas
    {  396, 4345,  125130,  7, }, // America/Cayenne
    {  396, 5496,  125137,  8, }, // America/Cayman
    {  396, 2260,  125145,  6, }, // America/Chicago
    {  396, 7756,  125151,  8, }, // America/Chihuahua
    {  396, 7774,  125159, 16, }, // America/Ciudad_Juarez
    {  396, 5422,   21503, 10, }, // America/Costa_Rica
    {  396, 7796,  125175,  9, }, // America/Creston
    {  396, 3791,    6625,  6, }, // America/Cuiaba
    {  396, 5723,  125184,  8, }, // America/Curacao
    {  396, 7812,  125192, 14, }, // America/Danmarkshavn
    {  396, 7833,  125206,  7, }, // America/Dawson
    {  396, 7848,  125213, 12, }, // America/Dawson_Creek
    {  396,  805,  125225,  8, }, // America/Denver
    {  396, 3465,  125233,  9, }, // America/Detroit
    {  396, 5739,   21548,  8, }, // America/Dominica
    {  396,  893,  125242, 10, }, // America/Edmonton
    {  396, 7869,  125252,  8, }, // America/Eirunepe
    {  396, 5441,  125260, 10, }, // America/El_Salvador
    {  396, 7886,  125270, 13, }, // America/Fort_Nelson
    {  396, 7924,  125283, 11, }, // America/Glace_Bay
    {  396, 6881,  125294,  9, }, // America/Goose_Bay
    {  396, 4612,  125303, 12, }, // America/Grand_Turk
    {  396, 5786,  125315,  9, }, // America/Guadeloupe
    {  396, 3760,   33206,  9, }, // America/Guatemala
    {  396, 4841,  125324, 10, }, // America/Guayaquil
    {  396, 6932,   57452,  7, }, // America/Guyana
    {  396, 1939,  125334, 10, }, // America/Halifax
    {  396, 7942,  125344, 11, }, // America/Hermosillo
    {  396,  481,  125355, 15, }, // America/Indiana/Knox
    {  396, 7961,  125370, 17, }, // America/Indiana/Marengo
    {  396, 7985,  125387, 21, }, // America/Indiana/Petersburg
    {  396, 8012,  125408, 18, }, // America/Indiana/Tell_City
    {  396, 8038,  125426, 14, }, // America/Indiana/Vevay
    {  396, 8060,  125440, 17, }, // America/Indiana/Vincennes
    {  396, 8086,  125457, 18, }, // America/Indiana/Winamac
    {  396, 8110,  125475,  8, }, // America/Inuvik
    {  396,  660,  125483,  9, }, // America/Iqaluit
    {  396, 2799,    3877,  7, }, // America/Jamaica
    {  396, 5380,  125492,  6, }, // America/Juneau
    {  396,  521,  125498,  9, }, // America/Kentucky/Louisville
    {  396, 8125,  125507, 20, }, // America/Kentucky/Monticello
    {  396, 5704,  125527, 12, }, // America/Kralendijk
    {  396, 4376,  125539,  8, }, // America/La_Paz
    {  396, 3239,  125547, 12, }, // America/Los_Angeles
    {  396, 5932,  125559, 27, }, // America/Lower_Princes
    {  396, 8153,  122774,  6, }, // America/Maceio
    {  396, 8168,  122780,  7, }, // America/Managua
    {  396, 1908,  125586,  8, }, // America/Manaus
    {  396, 5897,   61739,  6, }, // America/Marigot
    {  396, 5805,   26462,  9, }, // America/Martinique
    {  396, 8184,  125594, 11, }, // America/Matamoros
    {  396, 2917,  125605, 10, }, // America/Mazatlan
    {  396, 8202,  125615,  9, }, // America/Menominee
    {  396, 8220,    7008,  6, }, // America/Merida
    {  396, 2949,  125624,  8, }, // America/Mexico_City
    {  396, 4391,  125632,  9, }, // America/Miquelon
    {  396, 8254,  125641,  9, }, // America/Moncton
    {  396, 8270,  125650, 10, }, // America/Monterrey
    {  396, 4098,  125660, 12, }, // America/Montevideo
    {  396, 5824,  125672, 11, }, // America/Montserrat
    {  396, 5481,  125683,  5, }, // America/Nassau
    {  396, 2379,  125688,  9, }, // America/New_York
    {  396, 8288,  125697,  5, }, // America/Nome
    {  396, 1850,    7076,  7, }, // America/Noronha
    {  396, 8301,  125702, 21, }, // America/North_Dakota/Beulah
    {  396, 8329,  125723, 24, }, // America/North_Dakota/Center
    {  396, 8357,  125747, 28, }, // America/North_Dakota/New_Salem
    {  396,  393,  125775,  5, }, // America/Nuuk
    {  396, 8388,  125780,  8, }, // America/Ojinaga
    {  396, 6173,  125788, 11, }, // America/Paramaribo
    {  396, 2973,  125799,  8, }, // America/Phoenix
    {  396, 3945,  125807, 14, }, // America/Port-au-Prince
    {  396, 5954,  125821, 15, }, // America/Port_of_Spain
    {  396, 8404,  125836, 12, }, // America/Porto_Velho
    {  396, 5843,  125848, 12, }, // America/Puerto_Rico
    {  396, 4030,  125860, 14, }, // America/Punta_Arenas
    {  396, 8424,  125874, 14, }, // America/Rankin_Inlet
    {  396, 8445,  125888,  6, }, // America/Recife
    {  396, 1995,  125894,  7, }, // America/Regina
    {  396, 8460,  125901,  9, }, // America/Resolute
    {  396,  695,  125910, 11, }, // America/Rio_Branco
    {  396, 8477,  125921, 10, }, // America/Santarem
    {  396, 2201,  125931,  9, }, // America/Santiago
    {  396, 6111,  125940, 14, }, // America/Santo_Domingo
    {  396, 1878,  125954, 10, }, // America/Sao_Paulo
    {  396, 6899,  125964, 14, }, // America/Scoresbysund
    {  396, 8508,  125978, 13, }, // America/St_Barthelemy
    {  396, 2061,  125991, 12, }, // America/St_Johns
    {  396, 5863,  126003, 11, }, // America/St_Kitts
    {  396, 5880,  126014, 11, }, // America/St_Lucia
    {  396,  855,  126025, 12, }, // America/St_Thomas
    {  396, 5913,  126037, 14, }, // America/St_Vincent
    {  396, 8530,  126051, 14, }, // America/Swift_Current
    {  396, 5461,    5410, 11, }, // America/Tegucigalpa
    {  396, 5756,  126065,  7, }, // America/Thule
    {  396,  313,  126072,  8, }, // America/Tijuana
    {  396,  608,  126080,  8, }, // America/Toronto
    {  396, 2093,  126088, 10, }, // America/Vancouver
    {  396, 2144,  126098, 10, }, // America/Whitehorse
    {  396,  734,  126108,  9, }, // America/Winnipeg
    {  396, 8552,  126117,  9, }, // America/Yakutat
    {  396, 6001,  126126,  6, }, // Antarctica/Casey
    {  396, 6094,  126132,  8, }, // Antarctica/Davis
    {  396, 6133,  126140, 18, }, // Antarctica/DumontDUrville
    {  396, 8568,  126158,  8, }, // Antarctica/Macquarie
    {  396, 7066,  126166,  7, }, // Antarctica/Mawson
    {  396, 7113,  126173,  9, }, // Antarctica/McMurdo
    {  396, 8589,  126182,  8, }, // Antarctica/Palmer
    {  396, 7226,    5465,  6, }, // Antarctica/Rothera
    {  396, 7295,  126190,  5, }, // Antarctica/Syowa
    {  396, 8607,  126195,  6, }, // Antarctica/Troll
    {  396, 7374,  126201,  8, }, // Antarctica/Vostok
    {  396, 1429,  126209, 13, }, // Arctic/Longyearbyen
    {  396, 5570,  126222,  6, }, // Asia/Aden
    {  396, 5395,   57475,  8, }, // Asia/Almaty
    {  396, 3968,  126228,  6, }, // Asia/Amman
    {  396, 5511,  126234,  8, }, // Asia/Anadyr
    {  396, 5523,    5491,  5, }, // Asia/Aqtau
    {  396, 5368,  126242, 12, }, // Asia/Aqtobe
    {  396,  964,  126254, 10, }, // Asia/Ashgabat
    {  396, 3607,  126264,  8, }, // Asia/Baghdad
    {  396, 5534,  126272,  9, }, // Asia/Bahrain
    {  396, 4466,  126281,  8, }, // Asia/Bangkok
    {  396, 3571,  126289,  9, }, // Asia/Barnaul
    {  396, 4086,  126298,  8, }, // Asia/Beirut
    {  396, 3778,  126306,  8, }, // Asia/Bishkek
    {  396, 5989,  126314,  8, }, // Asia/Brunei
    {  396, 4601,   57490,  4, }, // Asia/Chita
    {  396, 4511,  126322,  8, }, // Asia/Colombo
    {  396, 4540,  126330,  8, }, // Asia/Damascus
    {  396, 1093,    4599,  4, }, // Asia/Dhaka
    {  396, 6192,  126338,  5, }, // Asia/Dili
    {  396, 3596,  126343,  7, }, // Asia/Dubai
    {  396, 6159,  126350,  7, }, // Asia/Dushanbe
    {  396, 8651,  126357, 12, }, // Asia/Gaza
    {  396, 4795,  126369,  8, }, // Asia/Hebron
    {  396, 1259,  126377,  9, }, // Asia/Ho_Chi_Minh
    {  396, 2704,  126386,  9, }, // Asia/Hong_Kong
    {  396, 4771,  126395,  6, }, // Asia/Hovd
    {  396, 4184,  126401,  9, }, // Asia/Irkutsk
    {  396, 7006,   33352,  8, }, // Asia/Jakarta
    {  396, 6992,   33360,  9, }, // Asia/Jayapura
    {  396, 1290,  126410,  8, }, // Asia/Jerusalem
    {  396, 3560,  126418,  7, }, // Asia/Kabul
    {  396, 4316,  126425,  9, }, // Asia/Kamchatka
    {  396, 4239,  126434,  7, }, // Asia/Karachi
    {  396, 1185,    5593,  8, }, // Asia/Kathmandu
    {  396, 8661,   57888,  8, }, // Asia/Khandyga
    {  396,  992,     422,  7, }, // Asia/Kolkata
    {  396, 4197,  126441, 13, }, // Asia/Krasnoyarsk
    {  396, 4859,  126454, 14, }, // Asia/Kuala_Lumpur
    {  396, 5976,  126468,  8, }, // Asia/Kuching
    {  396, 5547,  126476,  8, }, // Asia/Kuwait
    {  396, 1211,    7613,  5, }, // Asia/Macau
    {  396, 4017,  126484,  9, }, // Asia/Magadan
    {  396, 1349,  126493,  9, }, // Asia/Makassar
    {  396, 6920,  126502,  8, }, // Asia/Muscat
    {  396, 2554,   57151,  8, }, // Asia/Nicosia
    {  396, 8675,  126510, 13, }, // Asia/Novokuznetsk
    {  396, 4135,  126523, 13, }, // Asia/Novosibirsk
    {  396, 4229,  126536,  6, }, // Asia/Omsk
    {  396, 7145,  126542,  8, }, // Asia/Oral
    {  396, 6961,  126550,  9, }, // Asia/Phnom_Penh
    {  396, 8693,  126559, 11, }, // Asia/Pontianak
    {  396, 4214,  126570, 10, }, // Asia/Pyongyang
    {  396, 5559,  126580,  7, }, // Asia/Qatar
    {  396, 8708,  126587, 10, }, // Asia/Qostanay
    {  396, 4269,  126597,  9, }, // Asia/Qyzylorda
    {  396, 3584,  126606,  7, }, // Asia/Riyadh
    {  396, 4408,  126613, 10, }, // Asia/Sakhalin
    {  396, 7245,  126623, 11, }, // Asia/Samarkand
    {  396, 3279,  126634,  6, }, // Asia/Seoul
    {  396, 1053,  126640,  9, }, // Asia/Shanghai
    {  396, 3300,  126649, 10, }, // Asia/Singapore
    {  396, 4297,  126659, 15, }, // Asia/Srednekolymsk
    {  396, 3263,  126674,  8, }, // Asia/Taipei
    {  396, 4781,  126682,  9, }, // Asia/Tashkent
    {  396, 3915,  126691,  8, }, // Asia/Tbilisi
    {  396, 2772,  126699,  9, }, // Asia/Tehran
    {  396, 1317,   91641,  5, }, // Asia/Thimphu
    {  396, 2821,  126708,  7, }, // Asia/Tokyo
    {  396, 4572,  126715,  7, }, // Asia/Tomsk
    {  396, 1021,  126722, 12, }, // Asia/Ulaanbaatar
    {  396, 1159,   33464,  6, }, // Asia/Urumqi
    {  396, 8722,  126734,  8, }, // Asia/Ust-Nera
    {  396, 6977,  126742, 10, }, // Asia/Vientiane
    {  396, 4710,  126752, 13, }, // Asia/Vladivostok
    {  396, 4828,  126765,  9, }, // Asia/Yakutsk
    {  396, 1235,  126774,  8, }, // Asia/Yangon
    {  396, 3883,  126782, 15, }, // Asia/Yekaterinburg
    {  396, 3747,  126797,  9, }, // Asia/Yerevan
    {  396, 3663,  126806,  6, }, // Atlantic/Azores
    {  396, 6628,  126812, 12, }, // Atlantic/Canary
    {  396, 3727,  126824, 16, }, // Atlantic/Cape_Verde
    {  396, 1395,  126840,  7, }, // Atlantic/Faroe
    {  396, 2748,  126847, 11, }, // Atlantic/Reykjavik
    {  396, 7272,  126858, 15, }, // Atlantic/South_Georgia
    {  396, 6821,  126873, 11, }, // Atlantic/St_Helena
    {  396, 6644,  126884,  7, }, // Atlantic/Stanley
    {  396, 1670,  126891,  8, }, // Australia/Adelaide
    {  396, 1635,  126899,  9, }, // Australia/Brisbane
    {  396, 1799,  126908, 12, }, // Australia/Broken_Hill
    {  396, 1583,  126920,  8, }, // Australia/Darwin
    {  396, 3637,  123861,  5, }, // Australia/Eucla
    {  396, 1516,  126928,  8, }, // Australia/Hobart
    {  396, 8753,  126936, 10, }, // Australia/Lindeman
    {  396, 1547,  126946, 14, }, // Australia/Lord_Howe
    {  396, 1727,  126960,  9, }, // Australia/Melbourne
    {  396, 1762,  126969,  6, }, // Australia/Perth
    {  396, 1463,  126975,  8, }, // Australia/Sydney
    {  396, 6441,  126983, 11, }, // Europe/Amsterdam
    {  396, 6216,   33506,  6, }, // Europe/Andorra
    {  396, 3620,  126994, 10, }, // Europe/Astrakhan
    {  396, 2300,  127004,  6, }, // Europe/Athens
    {  396, 6336,  127010,  9, }, // Europe/Belgrade
    {  396, 4757,  127019,  8, }, // Europe/Berlin
    {  396, 2167,  127027,  9, }, // Europe/Brussels
    {  396, 3928,  127036, 10, }, // Europe/Bucharest
    {  396, 3806,  127046, 10, }, // Europe/Budapest
    {  396, 8772,  127056, 11, }, // Europe/Busingen
    {  396, 2583,  127067, 10, }, // Europe/Chisinau
    {  396, 6289,  127077,  9, }, // Europe/Copenhagen
    {  396, 2338,  127086,  8, }, // Europe/Dublin
    {  396, 6307,  127094, 11, }, // Europe/Gibraltar
    {  396, 8788,  127105,  7, }, // Europe/Guernsey
    {  396, 6612,  127112,  9, }, // Europe/Helsinki
    {  396, 8804,  127121,  9, }, // Europe/Isle_of_Man
    {  396, 1130,  127130,  9, }, // Europe/Istanbul
    {  396, 8823,  127139,  7, }, // Europe/Jersey
    {  396, 3979,  127146, 13, }, // Europe/Kaliningrad
    {  396, 8929,  127159,  7, }, // Europe/Kirov
    {  396, 2527,  127166,  7, }, // Europe/Kyiv
    {  396, 3213,  124078,  8, }, // Europe/Lisbon
    {  396, 6506,   21025,  9, }, // Europe/Ljubljana
    {  396, 2501,  127173,  8, }, // Europe/London
    {  396, 6365,  127181, 13, }, // Europe/Luxembourg
    {  396, 4892,  127194,  8, }, // Europe/Madrid
    {  396, 6582,  127202, 12, }, // Europe/Mariehamn
    {  396, 3693,  127214,  7, }, // Europe/Minsk
    {  396, 6410,  127221,  7, }, // Europe/Monaco
    {  396, 3537,  127228,  6, }, // Europe/Moscow
    {  396, 6458,  127234,  5, }, // Europe/Oslo
    {  396, 4284,  127239,  8, }, // Europe/Paris
    {  396, 6275,   36153,  5, }, // Europe/Prague
    {  396, 8837,  127247,  4, }, // Europe/Riga
    {  396, 6324,    5946,  4, }, // Europe/Rome
    {  396, 6470,  127251, 11, }, // Europe/San_Marino
    {  396, 6245,  127262,  9, }, // Europe/Sarajevo
    {  396, 4451,  127271,  9, }, // Europe/Saratov
    {  396, 8849,  127280, 12, }, // Europe/Simferopol
    {  396, 6383,  127292,  6, }, // Europe/Skopje
    {  396, 6599,   33629,  6, }, // Europe/Sofia
    {  396, 6523,  127298, 10, }, // Europe/Stockholm
    {  396, 8867,  127308,  7, }, // Europe/Tallinn
    {  396, 6202,   21095,  6, }, // Europe/Tirane
    {  396, 8882,  127315, 11, }, // Europe/Ulyanovsk
    {  396, 6352,  127326,  7, }, // Europe/Vaduz
    {  396, 6567,  127333,  9, }, // Europe/Vatican
    {  396, 6231,   21112,  5, }, // Europe/Vienna
    {  396, 4727,  127342, 11, }, // Europe/Volgograd
    {  396, 3190,  127353,  7, }, // Europe/Warsaw
    {  396, 6261,  127360,  8, }, // Europe/Zagreb
    {  396, 6540,  127368,  9, }, // Europe/Zurich
    {  396, 5078,  127377, 13, }, // Indian/Antananarivo
    {  396, 6947,  127390,  7, }, // Indian/Chagos
    {  396, 6046,  127397, 11, }, // Indian/Christmas
    {  396, 6063,  127408, 11, }, // Indian/Cocos
    {  396, 5029,  124308,  7, }, // Indian/Comoro
    {  396, 6661,  127419, 10, }, // Indian/Kerguelen
    {  396, 7260,  127429,  4, }, // Indian/Mahe
    {  396, 7050,  127433,  8, }, // Indian/Maldives
    {  396, 4069,   33694, 10, }, // Indian/Mauritius
    {  396, 5098,  127441,  7, }, // Indian/Mayotte
    {  396, 7211,  127448,  9, }, // Indian/Reunion
    {  396, 4422,  124364,  5, }, // Pacific/Apia
    {  396,  932,  127457,  8, }, // Pacific/Auckland
    {  396, 3706,  127465, 10, }, // Pacific/Bougainville
    {  396, 3015,  127475,  7, }, // Pacific/Chatham
    {  396, 3157,  127482,  5, }, // Pacific/Chuuk
    {  396, 2237,  127487, 11, }, // Pacific/Easter
    {  396, 7360,  127498,  5, }, // Pacific/Efate
    {  396, 7327,  127503,  8, }, // Pacific/Fakaofo
    {  396, 3902,  127511,  6, }, // Pacific/Fiji
    {  396, 7343,  127517,  9, }, // Pacific/Funafuti
    {  396, 6678,  127526,  9, }, // Pacific/Galapagos
    {  396, 6696,  127535,  8, }, // Pacific/Gambier
    {  396, 3822,  127543, 13, }, // Pacific/Guadalcanal
    {  396, 6018,  127556,  6, }, // Pacific/Guam
    {  396, 2723,    4493,  8, }, // Pacific/Honolulu
    {  396, 3049,  127562, 10, }, // Pacific/Kanton
    {  396, 3998,  127572, 11, }, // Pacific/Kiritimati
    {  396, 7019,  127583,  7, }, // Pacific/Kosrae
    {  396, 2842,  127590, 12, }, // Pacific/Kwajalein
    {  396, 4877,  127602,  8, }, // Pacific/Majuro
    {  396, 4051,  127610, 13, }, // Pacific/Marquesas
    {  396, 8914,  127623,  8, }, // Pacific/Midway
    {  396, 7132,  127631,  5, }, // Pacific/Niue
    {  396, 4168,  127636,  9, }, // Pacific/Norfolk
    {  396, 7098,  127645,  6, }, // Pacific/Noumea
    {  396, 3126,  127651, 10, }, // Pacific/Pago_Pago
    {  396, 7194,  127661, 13, }, // Pacific/Pitcairn
    {  396, 3096,  127674,  6, }, // Pacific/Pohnpei
    {  396, 4807,  127680, 12, }, // Pacific/Port_Moresby
    {  396, 6031,  127692,  8, }, // Pacific/Saipan
    {  396, 7312,  127700,  6, }, // Pacific/Tahiti
    {  396, 6712,   23027,  6, }, // Pacific/Tarawa
    {  396, 7392,  127706, 10, }, // Pacific/Wake
    {  396, 7405,  127716,  7, }, // Pacific/Wallis
    {  397,   14,       0,  6, }, // Africa/Asmara Lojban/Latin/world
    {  397, 4435,       6,  8, }, // Africa/Sao_Tome
    {  397, 4252,      14,  8, }, // America/Asuncion
    {  397,  237,      22,  8, }, // America/Atikokan
    {  397, 7623,      30, 17, }, // America/Bahia_Banderas
    {  397, 3868,      47,  6, }, // America/Cancun
    {  397, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  397, 5723,      66,  7, }, // America/Curacao
    {  397,  481,      73, 13, }, // America/Indiana/Knox
    {  397, 7961,      86, 16, }, // America/Indiana/Marengo
    {  397, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  397, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  397, 8038,     139, 14, }, // America/Indiana/Vevay
    {  397, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  397, 8086,     171, 16, }, // America/Indiana/Winamac
    {  397, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  397, 5932,     207, 22, }, // America/Lower_Princes
    {  397, 8220,     229,  6, }, // America/Merida
    {  397, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  397, 8329,     255, 20, }, // America/North_Dakota/Center
    {  397, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  397,  393,     298,  4, }, // America/Nuuk
    {  397, 6899,     302, 16, }, // America/Scoresbysund
    {  397, 8508,     318, 14, }, // America/St_Barthelemy
    {  397, 2061,     332, 10, }, // America/St_Johns
    {  397, 5863,     342,  9, }, // America/St_Kitts
    {  397, 5880,     351,  9, }, // America/St_Lucia
    {  397,  855,     360, 10, }, // America/St_Thomas
    {  397, 5913,     370, 11, }, // America/St_Vincent
    {  397, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  397, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  397, 1185,     413,  9, }, // Asia/Kathmandu
    {  397,  992,     422,  7, }, // Asia/Kolkata
    {  397, 1211,     429,  5, }, // Asia/Macau
    {  397, 1235,     442,  6, }, // Asia/Yangon
    {  397, 1395,     448,  5, }, // Atlantic/Faroe
    {  397, 6821,     453, 10, }, // Atlantic/St_Helena
    {  397, 2527,     463,  4, }, // Europe/Kyiv
    {  397, 7211,     467,  7, }, // Indian/Reunion
    {  397, 3157,     474,  5, }, // Pacific/Chuuk
    {  397, 3096,     479,  7, }, // Pacific/Pohnpei
    {  398, 6760,   21174,  5, }, // Africa/Accra Lower Sorbian/Latin/Germany
    {  398, 5059,    3784, 11, }, // Africa/Addis_Ababa
    {  398, 7420,   58423,  6, }, // Africa/Algiers
    {  398,   14,       0,  6, }, // Africa/Asmara
    {  398, 2320,    4572,  5, }, // Africa/Cairo
    {  398, 5130,   77951, 10, }, // Africa/Dar_es_Salaam
    {  398, 5043,   33154,  7, }, // Africa/Djibouti
    {  398, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  398, 4524,   26372,  7, }, // Africa/Khartoum
    {  398, 6869,   20300,  4, }, // Africa/Lome
    {  398, 5113,   57589,  8, }, // Africa/Mogadishu
    {  398, 6727,    4640,  8, }, // Africa/Ouagadougou
    {  398, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  398, 4435,       6,  8, }, // Africa/Sao_Tome
    {  398, 2866,   57624,  8, }, // Africa/Tripoli
    {  398, 4252,      14,  8, }, // America/Asuncion
    {  398,  237,      22,  8, }, // America/Atikokan
    {  398, 7623,      30, 17, }, // America/Bahia_Banderas
    {  398, 4361,   20389,  6, }, // America/Bogota
    {  398, 3868,      47,  6, }, // America/Cancun
    {  398, 5496,  127723, 15, }, // America/Cayman
    {  398, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  398, 5723,      66,  7, }, // America/Curacao
    {  398, 2281,   63392,  7, }, // America/Havana
    {  398,  481,      73, 13, }, // America/Indiana/Knox
    {  398, 7961,      86, 16, }, // America/Indiana/Marengo
    {  398, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  398, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  398, 8038,     139, 14, }, // America/Indiana/Vevay
    {  398, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  398, 8086,     171, 16, }, // America/Indiana/Winamac
    {  398, 2799,    3877,  7, }, // America/Jamaica
    {  398, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  398, 5932,     207, 22, }, // America/Lower_Princes
    {  398, 8220,     229,  6, }, // America/Merida
    {  398, 2949,  127738, 12, }, // America/Mexico_City
    {  398, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  398, 8329,     255, 20, }, // America/North_Dakota/Center
    {  398, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  398,  393,     298,  4, }, // America/Nuuk
    {  398, 5954,   26559, 13, }, // America/Port_of_Spain
    {  398, 1878,   26580,  9, }, // America/Sao_Paulo
    {  398, 6899,     302, 16, }, // America/Scoresbysund
    {  398, 8508,  127750, 13, }, // America/St_Barthelemy
    {  398, 2061,     332, 10, }, // America/St_Johns
    {  398, 5863,     342,  9, }, // America/St_Kitts
    {  398, 5880,     351,  9, }, // America/St_Lucia
    {  398,  855,     360, 10, }, // America/St_Thomas
    {  398, 5913,     370, 11, }, // America/St_Vincent
    {  398, 6133,  127763, 16, }, // Antarctica/DumontDUrville
    {  398, 7374,    4029,  6, }, // Antarctica/Vostok
    {  398, 5368,    7488,  6, }, // Asia/Aqtobe
    {  398, 3607,    4044,  6, }, // Asia/Baghdad
    {  398, 3778,   33323,  6, }, // Asia/Bishkek
    {  398, 4540,    4083,  8, }, // Asia/Damascus
    {  398, 6159,   33336,  7, }, // Asia/Dushanbe
    {  398, 1259,  127779, 17, }, // Asia/Ho_Chi_Minh
    {  398, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  398, 6992,  127796, 11, }, // Asia/Jayapura
    {  398, 4316,   33378,  8, }, // Asia/Kamchatka
    {  398, 4239,   33386,  6, }, // Asia/Karachi
    {  398, 1185,     413,  9, }, // Asia/Kathmandu
    {  398,  992,   64221,  8, }, // Asia/Kolkata
    {  398, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  398, 1211,     429,  5, }, // Asia/Macau
    {  398, 6920,    4208,  6, }, // Asia/Muscat
    {  398, 2554,    5620,  7, }, // Asia/Nicosia
    {  398, 8675,  127807, 13, }, // Asia/Novokuznetsk
    {  398, 4135,   78082, 11, }, // Asia/Novosibirsk
    {  398, 4214,   78093,  9, }, // Asia/Pyongyang
    {  398, 5559,    4214,  5, }, // Asia/Qatar
    {  398, 3584,    7669,  4, }, // Asia/Riyadh
    {  398, 4408,   57952,  8, }, // Asia/Sakhalin
    {  398, 3300,   20782,  8, }, // Asia/Singapore
    {  398, 3263,   78102,  6, }, // Asia/Taipei
    {  398, 4781,   33457,  7, }, // Asia/Tashkent
    {  398, 2772,    7724,  7, }, // Asia/Tehran
    {  398, 2821,    4238,  5, }, // Asia/Tokyo
    {  398, 4710,    4243, 11, }, // Asia/Vladivostok
    {  398, 4828,    7755,  7, }, // Asia/Yakutsk
    {  398, 1235,    7762,  6, }, // Asia/Yangon
    {  398, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  398, 3747,  127820,  7, }, // Asia/Yerevan
    {  398, 3663,  127827,  5, }, // Atlantic/Azores
    {  398, 5671,   58043,  7, }, // Atlantic/Bermuda
    {  398, 6628,  117702,  6, }, // Atlantic/Canary
    {  398, 3727,   57248,  9, }, // Atlantic/Cape_Verde
    {  398, 1395,  127832,  6, }, // Atlantic/Faroe
    {  398, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  398, 7272,  127838, 22, }, // Atlantic/South_Georgia
    {  398, 6821,     453, 10, }, // Atlantic/St_Helena
    {  398, 2300,    4320,  5, }, // Europe/Athens
    {  398, 6336,  127860,  8, }, // Europe/Belgrade
    {  398, 2167,   22634,  7, }, // Europe/Brussels
    {  398, 3928,   26897,  8, }, // Europe/Bucharest
    {  398, 8772,   26905,  8, }, // Europe/Busingen
    {  398, 2583,  127868,  7, }, // Europe/Chisinau
    {  398, 6289,    4357, 10, }, // Europe/Copenhagen
    {  398, 2527,   78163,  4, }, // Europe/Kyiv
    {  398, 3213,    4381,  8, }, // Europe/Lisbon
    {  398, 6365,    4395,  9, }, // Europe/Luxembourg
    {  398, 3537,   93104,  6, }, // Europe/Moscow
    {  398, 6275,   21067,  5, }, // Europe/Prague
    {  398, 6324,    5946,  3, }, // Europe/Rome
    {  398, 6202,   21095,  6, }, // Europe/Tirane
    {  398, 6567,    5978,  7, }, // Europe/Vatican
    {  398, 6231,   57383,  4, }, // Europe/Vienna
    {  398, 8899,  127875,  5, }, // Europe/Vilnius
    {  398, 4727,    4443,  9, }, // Europe/Volgograd
    {  398, 3190,  127880,  7, }, // Europe/Warsaw
    {  398, 6540,    4459,  6, }, // Europe/Zurich
    {  398, 6046,  127887, 12, }, // Indian/Christmas
    {  398, 5029,   58276,  6, }, // Indian/Comoro
    {  398, 7050,  127899,  8, }, // Indian/Maldives
    {  398, 7211,     467,  7, }, // Indian/Reunion
    {  398, 3157,     474,  5, }, // Pacific/Chuuk
    {  398, 2237,  127907, 13, }, // Pacific/Easter
    {  398, 3902,   33704,  5, }, // Pacific/Fiji
    {  398, 3049,    4501,  9, }, // Pacific/Kanton
    {  398, 3096,     479,  7, }, // Pacific/Pohnpei
    {  399,   14,       0,  6, }, // Africa/Asmara Low German/Latin/Germany
    {  399, 4435,       6,  8, }, // Africa/Sao_Tome
    {  399, 4252,      14,  8, }, // America/Asuncion
    {  399,  237,      22,  8, }, // America/Atikokan
    {  399, 7623,      30, 17, }, // America/Bahia_Banderas
    {  399, 3868,      47,  6, }, // America/Cancun
    {  399, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  399, 5723,      66,  7, }, // America/Curacao
    {  399,  481,      73, 13, }, // America/Indiana/Knox
    {  399, 7961,      86, 16, }, // America/Indiana/Marengo
    {  399, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  399, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  399, 8038,     139, 14, }, // America/Indiana/Vevay
    {  399, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  399, 8086,     171, 16, }, // America/Indiana/Winamac
    {  399, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  399, 5932,     207, 22, }, // America/Lower_Princes
    {  399, 8220,     229,  6, }, // America/Merida
    {  399, 2949,   77992, 12, }, // America/Mexico_City
    {  399, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  399, 8329,    3915, 20, }, // America/North_Dakota/Center
    {  399, 8357,    3935, 23, }, // America/North_Dakota/New_Salem
    {  399,  393,     298,  4, }, // America/Nuuk
    {  399, 6899,     302, 16, }, // America/Scoresbysund
    {  399, 8508,     318, 14, }, // America/St_Barthelemy
    {  399, 2061,     332, 10, }, // America/St_Johns
    {  399, 5863,     342,  9, }, // America/St_Kitts
    {  399, 5880,     351,  9, }, // America/St_Lucia
    {  399,  855,     360, 10, }, // America/St_Thomas
    {  399, 5913,     370, 11, }, // America/St_Vincent
    {  399, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  399, 7374,    4029,  6, }, // Antarctica/Vostok
    {  399, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  399, 4316,  127920, 10, }, // Asia/Kamchatka
    {  399, 1185,     413,  9, }, // Asia/Kathmandu
    {  399,  992,     422,  7, }, // Asia/Kolkata
    {  399, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  399, 1211,     429,  5, }, // Asia/Macau
    {  399, 4135,   78082, 11, }, // Asia/Novosibirsk
    {  399, 4408,   57952,  8, }, // Asia/Sakhalin
    {  399, 1021,   22466,  9, }, // Asia/Ulaanbaatar
    {  399, 4710,    4243, 11, }, // Asia/Vladivostok
    {  399, 4828,    7755,  7, }, // Asia/Yakutsk
    {  399, 1235,     442,  6, }, // Asia/Yangon
    {  399, 3883,  127930, 13, }, // Asia/Yekaterinburg
    {  399, 3663,   58709,  6, }, // Atlantic/Azores
    {  399, 6628,   78114,  7, }, // Atlantic/Canary
    {  399, 1395,     448,  5, }, // Atlantic/Faroe
    {  399, 6821,     453, 10, }, // Atlantic/St_Helena
    {  399, 2527,     463,  4, }, // Europe/Kyiv
    {  399, 3537,   57353,  6, }, // Europe/Moscow
    {  399, 4727,    4443,  9, }, // Europe/Volgograd
    {  399, 7211,     467,  7, }, // Indian/Reunion
    {  399, 3157,     474,  5, }, // Pacific/Chuuk
    {  399, 3096,     479,  7, }, // Pacific/Pohnpei
    {  401,   14,       0,  6, }, // Africa/Asmara Luba-Katanga/Latin/Congo - Kinshasa
    {  401, 4435,       6,  8, }, // Africa/Sao_Tome
    {  401, 4252,      14,  8, }, // America/Asuncion
    {  401,  237,      22,  8, }, // America/Atikokan
    {  401, 7623,      30, 17, }, // America/Bahia_Banderas
    {  401, 3868,      47,  6, }, // America/Cancun
    {  401, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  401, 5723,      66,  7, }, // America/Curacao
    {  401,  481,      73, 13, }, // America/Indiana/Knox
    {  401, 7961,      86, 16, }, // America/Indiana/Marengo
    {  401, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  401, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  401, 8038,     139, 14, }, // America/Indiana/Vevay
    {  401, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  401, 8086,     171, 16, }, // America/Indiana/Winamac
    {  401, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  401, 5932,     207, 22, }, // America/Lower_Princes
    {  401, 8220,     229,  6, }, // America/Merida
    {  401, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  401, 8329,     255, 20, }, // America/North_Dakota/Center
    {  401, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  401,  393,     298,  4, }, // America/Nuuk
    {  401, 6899,     302, 16, }, // America/Scoresbysund
    {  401, 8508,     318, 14, }, // America/St_Barthelemy
    {  401, 2061,     332, 10, }, // America/St_Johns
    {  401, 5863,     342,  9, }, // America/St_Kitts
    {  401, 5880,     351,  9, }, // America/St_Lucia
    {  401,  855,     360, 10, }, // America/St_Thomas
    {  401, 5913,     370, 11, }, // America/St_Vincent
    {  401, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  401, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  401, 1185,     413,  9, }, // Asia/Kathmandu
    {  401,  992,     422,  7, }, // Asia/Kolkata
    {  401, 1211,     429,  5, }, // Asia/Macau
    {  401, 1235,     442,  6, }, // Asia/Yangon
    {  401, 1395,     448,  5, }, // Atlantic/Faroe
    {  401, 6821,     453, 10, }, // Atlantic/St_Helena
    {  401, 2527,     463,  4, }, // Europe/Kyiv
    {  401, 7211,     467,  7, }, // Indian/Reunion
    {  401, 3157,     474,  5, }, // Pacific/Chuuk
    {  401, 3096,     479,  7, }, // Pacific/Pohnpei
    {  402,   14,       0,  6, }, // Africa/Asmara Lule Sami/Latin/Sweden
    {  402, 4435,       6,  8, }, // Africa/Sao_Tome
    {  402, 4252,      14,  8, }, // America/Asuncion
    {  402,  237,      22,  8, }, // America/Atikokan
    {  402, 7623,      30, 17, }, // America/Bahia_Banderas
    {  402, 3868,      47,  6, }, // America/Cancun
    {  402, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  402, 5723,      66,  7, }, // America/Curacao
    {  402,  481,      73, 13, }, // America/Indiana/Knox
    {  402, 7961,      86, 16, }, // America/Indiana/Marengo
    {  402, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  402, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  402, 8038,     139, 14, }, // America/Indiana/Vevay
    {  402, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  402, 8086,     171, 16, }, // America/Indiana/Winamac
    {  402, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  402, 5932,     207, 22, }, // America/Lower_Princes
    {  402, 8220,     229,  6, }, // America/Merida
    {  402, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  402, 8329,     255, 20, }, // America/North_Dakota/Center
    {  402, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  402,  393,     298,  4, }, // America/Nuuk
    {  402, 6899,     302, 16, }, // America/Scoresbysund
    {  402, 8508,     318, 14, }, // America/St_Barthelemy
    {  402, 2061,     332, 10, }, // America/St_Johns
    {  402, 5863,     342,  9, }, // America/St_Kitts
    {  402, 5880,     351,  9, }, // America/St_Lucia
    {  402,  855,     360, 10, }, // America/St_Thomas
    {  402, 5913,     370, 11, }, // America/St_Vincent
    {  402, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  402, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  402, 1185,     413,  9, }, // Asia/Kathmandu
    {  402,  992,     422,  7, }, // Asia/Kolkata
    {  402, 1211,     429,  5, }, // Asia/Macau
    {  402, 1235,     442,  6, }, // Asia/Yangon
    {  402, 1395,     448,  5, }, // Atlantic/Faroe
    {  402, 6821,     453, 10, }, // Atlantic/St_Helena
    {  402, 2527,     463,  4, }, // Europe/Kyiv
    {  402, 7211,     467,  7, }, // Indian/Reunion
    {  402, 3157,     474,  5, }, // Pacific/Chuuk
    {  402, 3096,     479,  7, }, // Pacific/Pohnpei
    {  404,   14,       0,  6, }, // Africa/Asmara Luo/Latin/Kenya
    {  404, 4435,       6,  8, }, // Africa/Sao_Tome
    {  404, 4252,      14,  8, }, // America/Asuncion
    {  404,  237,      22,  8, }, // America/Atikokan
    {  404, 7623,      30, 17, }, // America/Bahia_Banderas
    {  404, 3868,      47,  6, }, // America/Cancun
    {  404, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  404, 5723,      66,  7, }, // America/Curacao
    {  404,  481,      73, 13, }, // America/Indiana/Knox
    {  404, 7961,      86, 16, }, // America/Indiana/Marengo
    {  404, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  404, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  404, 8038,     139, 14, }, // America/Indiana/Vevay
    {  404, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  404, 8086,     171, 16, }, // America/Indiana/Winamac
    {  404, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  404, 5932,     207, 22, }, // America/Lower_Princes
    {  404, 8220,     229,  6, }, // America/Merida
    {  404, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  404, 8329,     255, 20, }, // America/North_Dakota/Center
    {  404, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  404,  393,     298,  4, }, // America/Nuuk
    {  404, 6899,     302, 16, }, // America/Scoresbysund
    {  404, 8508,     318, 14, }, // America/St_Barthelemy
    {  404, 2061,     332, 10, }, // America/St_Johns
    {  404, 5863,     342,  9, }, // America/St_Kitts
    {  404, 5880,     351,  9, }, // America/St_Lucia
    {  404,  855,     360, 10, }, // America/St_Thomas
    {  404, 5913,     370, 11, }, // America/St_Vincent
    {  404, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  404, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  404, 1185,     413,  9, }, // Asia/Kathmandu
    {  404,  992,     422,  7, }, // Asia/Kolkata
    {  404, 1211,     429,  5, }, // Asia/Macau
    {  404, 1235,     442,  6, }, // Asia/Yangon
    {  404, 1395,     448,  5, }, // Atlantic/Faroe
    {  404, 6821,     453, 10, }, // Atlantic/St_Helena
    {  404, 2527,     463,  4, }, // Europe/Kyiv
    {  404, 7211,     467,  7, }, // Indian/Reunion
    {  404, 3157,     474,  5, }, // Pacific/Chuuk
    {  404, 3096,     479,  7, }, // Pacific/Pohnpei
    {  405, 5059,    3784, 11, }, // Africa/Addis_Ababa Luxembourgish/Latin/Luxembourg
    {  405, 7420,   43956,  5, }, // Africa/Algiers
    {  405,   14,       0,  6, }, // Africa/Asmara
    {  405, 2320,    4572,  5, }, // Africa/Cairo
    {  405, 5043,   77961,  9, }, // Africa/Djibouti
    {  405, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  405, 4524,   26372,  7, }, // Africa/Khartoum
    {  405, 5113,   77970, 10, }, // Africa/Mogadishu
    {  405, 6727,    4640,  8, }, // Africa/Ouagadougou
    {  405, 4435,       6,  8, }, // Africa/Sao_Tome
    {  405, 4252,      14,  8, }, // America/Asuncion
    {  405,  237,      22,  8, }, // America/Atikokan
    {  405, 7623,      30, 17, }, // America/Bahia_Banderas
    {  405, 3868,      47,  6, }, // America/Cancun
    {  405, 5496,  127943, 13, }, // America/Cayman
    {  405, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  405, 5723,      66,  7, }, // America/Curacao
    {  405, 5441,    6688,  8, }, // America/El_Salvador
    {  405, 2281,   63392,  7, }, // America/Havana
    {  405,  481,      73, 13, }, // America/Indiana/Knox
    {  405, 7961,      86, 16, }, // America/Indiana/Marengo
    {  405, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  405, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  405, 8038,     139, 14, }, // America/Indiana/Vevay
    {  405, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  405, 8086,     171, 16, }, // America/Indiana/Winamac
    {  405, 2799,    3877,  7, }, // America/Jamaica
    {  405, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  405, 5932,     207, 22, }, // America/Lower_Princes
    {  405, 8220,     229,  6, }, // America/Merida
    {  405, 2949,   77992, 11, }, // America/Mexico_City
    {  405, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  405, 8329,     255, 20, }, // America/North_Dakota/Center
    {  405, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  405,  393,     298,  4, }, // America/Nuuk
    {  405, 5954,   26559, 13, }, // America/Port_of_Spain
    {  405, 6899,     302, 16, }, // America/Scoresbysund
    {  405, 8508,   26589, 16, }, // America/St_Barthelemy
    {  405, 2061,     332, 10, }, // America/St_Johns
    {  405, 5863,     342,  9, }, // America/St_Kitts
    {  405, 5880,     351,  9, }, // America/St_Lucia
    {  405,  855,     360, 10, }, // America/St_Thomas
    {  405, 5913,     370, 11, }, // America/St_Vincent
    {  405, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  405, 7374,    4029,  6, }, // Antarctica/Vostok
    {  405, 5368,   57073,  6, }, // Asia/Aqtobe
    {  405, 3607,    4044,  6, }, // Asia/Baghdad
    {  405, 3778,   57088,  8, }, // Asia/Bishkek
    {  405, 4540,    4083,  8, }, // Asia/Damascus
    {  405, 6159,   57096,  9, }, // Asia/Dushanbe
    {  405, 1259,   78028, 16, }, // Asia/Ho_Chi_Minh
    {  405, 6992,  127796, 11, }, // Asia/Jayapura
    {  405, 4316,   78050, 11, }, // Asia/Kamchatka
    {  405, 1185,     413,  9, }, // Asia/Kathmandu
    {  405,  992,   64221,  8, }, // Asia/Kolkata
    {  405, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  405, 1211,    4203,  5, }, // Asia/Macau
    {  405, 6920,    4208,  6, }, // Asia/Muscat
    {  405, 2554,    5620,  7, }, // Asia/Nicosia
    {  405, 4135,   78082, 11, }, // Asia/Novosibirsk
    {  405, 4214,   78093,  9, }, // Asia/Pyongyang
    {  405, 5559,    4214,  5, }, // Asia/Qatar
    {  405, 3584,    7669,  4, }, // Asia/Riyadh
    {  405, 4408,   57952,  8, }, // Asia/Sakhalin
    {  405, 3300,   20782,  8, }, // Asia/Singapore
    {  405, 3263,   78102,  6, }, // Asia/Taipei
    {  405, 4781,   57196,  9, }, // Asia/Tashkent
    {  405, 3915,   78108,  6, }, // Asia/Tbilisi
    {  405, 2772,    7724,  7, }, // Asia/Tehran
    {  405, 2821,    4238,  5, }, // Asia/Tokyo
    {  405, 4710,    4243, 11, }, // Asia/Vladivostok
    {  405, 4828,    7755,  7, }, // Asia/Yakutsk
    {  405, 1235,     442,  6, }, // Asia/Yangon
    {  405, 3883,  127956, 14, }, // Asia/Yekaterinburg
    {  405, 3747,  127970,  6, }, // Asia/Yerevan
    {  405, 3663,   58709,  6, }, // Atlantic/Azores
    {  405, 5671,   74354,  8, }, // Atlantic/Bermuda
    {  405, 6628,  127976, 17, }, // Atlantic/Canary
    {  405, 3727,   57248,  9, }, // Atlantic/Cape_Verde
    {  405, 1395,   78121,  6, }, // Atlantic/Faroe
    {  405, 7272,   78138, 11, }, // Atlantic/South_Georgia
    {  405, 6821,     453, 10, }, // Atlantic/St_Helena
    {  405, 2300,    4320,  5, }, // Europe/Athens
    {  405, 6336,   26883,  7, }, // Europe/Belgrade
    {  405, 2167,  127993,  8, }, // Europe/Brussels
    {  405, 3928,   26897,  8, }, // Europe/Bucharest
    {  405, 2583,  128001,  9, }, // Europe/Chisinau
    {  405, 6289,    4357, 10, }, // Europe/Copenhagen
    {  405, 2527,   78163,  4, }, // Europe/Kyiv
    {  405, 3213,    4381,  8, }, // Europe/Lisbon
    {  405, 6365,  128010, 10, }, // Europe/Luxembourg
    {  405, 3537,   57353,  6, }, // Europe/Moscow
    {  405, 6275,    8083,  4, }, // Europe/Prague
    {  405, 6324,  128020,  4, }, // Europe/Rome
    {  405, 6202,   21095,  6, }, // Europe/Tirane
    {  405, 6567,    5978,  7, }, // Europe/Vatican
    {  405, 6231,   57383,  4, }, // Europe/Vienna
    {  405, 8899,  127875,  5, }, // Europe/Vilnius
    {  405, 4727,    4443,  9, }, // Europe/Volgograd
    {  405, 3190,   57396,  8, }, // Europe/Warsaw
    {  405, 6540,  128024,  6, }, // Europe/Zurich
    {  405, 6046,  128030, 17, }, // Indian/Christmas
    {  405, 5029,   78198,  7, }, // Indian/Comoro
    {  405, 7050,   58863,  8, }, // Indian/Maldives
    {  405, 7211,     467,  7, }, // Indian/Reunion
    {  405, 3157,     474,  5, }, // Pacific/Chuuk
    {  405, 2237,  128047, 13, }, // Pacific/Easter
    {  405, 3902,   78224,  7, }, // Pacific/Fiji
    {  405, 3096,     479,  7, }, // Pacific/Pohnpei
    {  406,   14,       0,  6, }, // Africa/Asmara Luyia/Latin/Kenya
    {  406, 4435,       6,  8, }, // Africa/Sao_Tome
    {  406, 4252,      14,  8, }, // America/Asuncion
    {  406,  237,      22,  8, }, // America/Atikokan
    {  406, 7623,      30, 17, }, // America/Bahia_Banderas
    {  406, 3868,      47,  6, }, // America/Cancun
    {  406, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  406, 5723,      66,  7, }, // America/Curacao
    {  406,  481,      73, 13, }, // America/Indiana/Knox
    {  406, 7961,      86, 16, }, // America/Indiana/Marengo
    {  406, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  406, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  406, 8038,     139, 14, }, // America/Indiana/Vevay
    {  406, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  406, 8086,     171, 16, }, // America/Indiana/Winamac
    {  406, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  406, 5932,     207, 22, }, // America/Lower_Princes
    {  406, 8220,     229,  6, }, // America/Merida
    {  406, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  406, 8329,     255, 20, }, // America/North_Dakota/Center
    {  406, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  406,  393,     298,  4, }, // America/Nuuk
    {  406, 6899,     302, 16, }, // America/Scoresbysund
    {  406, 8508,     318, 14, }, // America/St_Barthelemy
    {  406, 2061,     332, 10, }, // America/St_Johns
    {  406, 5863,     342,  9, }, // America/St_Kitts
    {  406, 5880,     351,  9, }, // America/St_Lucia
    {  406,  855,     360, 10, }, // America/St_Thomas
    {  406, 5913,     370, 11, }, // America/St_Vincent
    {  406, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  406, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  406, 1185,     413,  9, }, // Asia/Kathmandu
    {  406,  992,     422,  7, }, // Asia/Kolkata
    {  406, 1211,     429,  5, }, // Asia/Macau
    {  406, 1235,     442,  6, }, // Asia/Yangon
    {  406, 1395,     448,  5, }, // Atlantic/Faroe
    {  406, 6821,     453, 10, }, // Atlantic/St_Helena
    {  406, 2527,     463,  4, }, // Europe/Kyiv
    {  406, 7211,     467,  7, }, // Indian/Reunion
    {  406, 3157,     474,  5, }, // Pacific/Chuuk
    {  406, 3096,     479,  7, }, // Pacific/Pohnpei
    {  407, 6788,     486,  6, }, // Africa/Abidjan Macedonian/Cyrillic/Macedonia
    {  407, 6760,   27046,  4, }, // Africa/Accra
    {  407, 5059,   33714, 10, }, // Africa/Addis_Ababa
    {  407, 7420,     508,  5, }, // Africa/Algiers
    {  407,   14,   27065,  6, }, // Africa/Asmara
    {  407,   44,     519,  6, }, // Africa/Bamako
    {  407, 5257,     525,  5, }, // Africa/Bangui
    {  407, 6746,  128060,  6, }, // Africa/Banjul
    {  407, 7435,     536,  5, }, // Africa/Bissau
    {  407, 4957,  128066,  8, }, // Africa/Blantyre
    {  407, 5287,   33742,  8, }, // Africa/Brazzaville
    {  407, 4922,   33750,  9, }, // Africa/Bujumbura
    {  407, 2320,   33759,  5, }, // Africa/Cairo
    {  407, 4117,   33764, 10, }, // Africa/Casablanca
    {  407, 7449,     581,  5, }, // Africa/Ceuta
    {  407, 6773,     586,  7, }, // Africa/Conakry
    {  407, 6840,     593,  5, }, // Africa/Dakar
    {  407, 5130,   36235, 12, }, // Africa/Dar_es_Salaam
    {  407, 5043,     610,  6, }, // Africa/Djibouti
    {  407, 5243,     616,  5, }, // Africa/Douala
    {  407, 5166,   33774,  7, }, // Africa/El_Aaiun
    {  407, 6853,     628,  7, }, // Africa/Freetown
    {  407, 4906,     635,  8, }, // Africa/Gaborone
    {  407, 5015,   33781,  6, }, // Africa/Harare
    {  407, 4479,   33787, 11, }, // Africa/Johannesburg
    {  407, 4499,     661,  4, }, // Africa/Juba
    {  407, 5151,     665,  7, }, // Africa/Kampala
    {  407, 4524,   33798,  6, }, // Africa/Khartoum
    {  407, 4973,     678,  6, }, // Africa/Kigali
    {  407, 5306,     684,  7, }, // Africa/Kinshasa
    {  407, 4744,     691,  5, }, // Africa/Lagos
    {  407, 5336,  128074,  7, }, // Africa/Libreville
    {  407, 6869,     704,  4, }, // Africa/Lome
    {  407, 5211,     708,  6, }, // Africa/Luanda
    {  407, 4939,     714,  9, }, // Africa/Lubumbashi
    {  407, 5001,     723,  6, }, // Africa/Lusaka
    {  407, 5322,     729,  6, }, // Africa/Malabo
    {  407, 4987,   33813,  6, }, // Africa/Maputo
    {  407, 5197,     741,  6, }, // Africa/Maseru
    {  407, 5182,     747,  7, }, // Africa/Mbabane
    {  407, 5113,   36279,  8, }, // Africa/Mogadishu
    {  407, 7034,   33819,  9, }, // Africa/Monrovia
    {  407, 3853,   33828,  7, }, // Africa/Nairobi
    {  407, 5271,     777,  7, }, // Africa/Ndjamena
    {  407, 5354,   33835,  7, }, // Africa/Niamey
    {  407, 6803,     790,  7, }, // Africa/Nouakchott
    {  407, 6727,     797,  8, }, // Africa/Ouagadougou
    {  407, 5225,   33842, 10, }, // Africa/Porto-Novo
    {  407, 4435,  128081,  9, }, // Africa/Sao_Tome
    {  407, 2866,     823,  7, }, // Africa/Tripoli
    {  407, 6554,     830,  5, }, // Africa/Tunis
    {  407, 4152,     835,  7, }, // Africa/Windhoek
    {  407,  132,     842,  4, }, // America/Adak
    {  407, 3346,   33860,  7, }, // America/Anchorage
    {  407, 5607,   33867,  7, }, // America/Anguilla
    {  407, 5624,  128090,  6, }, // America/Antigua
    {  407, 4554,   33881, 10, }, // America/Araguaina
    {  407,  166,   33891, 12, }, // America/Argentina/Buenos_Aires
    {  407,   91,     888,  9, }, // America/Argentina/Catamarca
    {  407,  270,   33903,  7, }, // America/Argentina/Cordoba
    {  407,  441,   33910,  5, }, // America/Argentina/Jujuy
    {  407, 7462,   33915,  8, }, // America/Argentina/La_Rioja
    {  407,  565,   33923,  7, }, // America/Argentina/Mendoza
    {  407, 7489,   36349, 11, }, // America/Argentina/Rio_Gallegos
    {  407, 7520,     935,  5, }, // America/Argentina/Salta
    {  407, 7544,   33941,  8, }, // America/Argentina/San_Juan
    {  407, 5580,   36360,  8, }, // America/Argentina/San_Luis
    {  407, 7571,     956,  7, }, // America/Argentina/Tucuman
    {  407, 7597,  128096,  6, }, // America/Argentina/Ushuaia
    {  407, 5640,     969,  5, }, // America/Aruba
    {  407, 4252,   33963,  8, }, // America/Asuncion
    {  407,  237,  128102, 12, }, // America/Atikokan
    {  407, 3679,  128114,  5, }, // America/Bahia
    {  407, 7623,  128119, 18, }, // America/Bahia_Banderas
    {  407, 5654,    1013,  8, }, // America/Barbados
    {  407, 7646,   33998,  5, }, // America/Belem
    {  407, 5407,   34003,  6, }, // America/Belize
    {  407, 7660,   34009, 13, }, // America/Blanc-Sablon
    {  407, 7681,   34022,  9, }, // America/Boa_Vista
    {  407, 4361,    1052,  6, }, // America/Bogota
    {  407, 7699,   34031,  5, }, // America/Boise
    {  407, 7713,   34036, 11, }, // America/Cambridge_Bay
    {  407, 7735,   34047, 12, }, // America/Campo_Grande
    {  407, 3868,    1087,  6, }, // America/Cancun
    {  407, 4694,    1093,  7, }, // America/Caracas
    {  407, 4345,  128137,  6, }, // America/Cayenne
    {  407, 5496,  128143, 17, }, // America/Cayman
    {  407, 2260,    1129,  6, }, // America/Chicago
    {  407, 7756,   34080,  6, }, // America/Chihuahua
    {  407, 7774,  128160, 13, }, // America/Ciudad_Juarez
    {  407, 5422,   34086,  9, }, // America/Costa_Rica
    {  407, 7796,    1151,  7, }, // America/Creston
    {  407, 3791,  128173,  6, }, // America/Cuiaba
    {  407, 5723,   34095,  7, }, // America/Curacao
    {  407, 7812,  128179, 10, }, // America/Danmarkshavn
    {  407, 7833,   34115,  5, }, // America/Dawson
    {  407, 7848,   34120, 10, }, // America/Dawson_Creek
    {  407,  805,    1201,  6, }, // America/Denver
    {  407, 3465,    1207,  7, }, // America/Detroit
    {  407, 5739,    1214,  8, }, // America/Dominica
    {  407,  893,    1222,  8, }, // America/Edmonton
    {  407, 7869,    1230,  8, }, // America/Eirunepe
    {  407, 5441,  128189, 11, }, // America/El_Salvador
    {  407, 7886,    1246, 11, }, // America/Fort_Nelson
    {  407, 7906,    1257,  9, }, // America/Fortaleza
    {  407, 7924,   34130,  9, }, // America/Glace_Bay
    {  407, 6881,   34139,  7, }, // America/Goose_Bay
    {  407, 4612,   34146, 10, }, // America/Grand_Turk
    {  407, 5770,    1292,  7, }, // America/Grenada
    {  407, 5786,   34156,  9, }, // America/Guadeloupe
    {  407, 3760,    1308,  9, }, // America/Guatemala
    {  407, 4841,   34165,  8, }, // America/Guayaquil
    {  407, 6932,  128200,  7, }, // America/Guyana
    {  407, 1939,   34178,  8, }, // America/Halifax
    {  407, 2281,   34186,  6, }, // America/Havana
    {  407, 7942,  128207,  8, }, // America/Hermosillo
    {  407,  348,  128215, 13, }, // America/Indiana/Indianapolis
    {  407,  481,  128228, 14, }, // America/Indiana/Knox
    {  407, 7961,   34216, 17, }, // America/Indiana/Marengo
    {  407, 7985,  128242, 20, }, // America/Indiana/Petersburg
    {  407, 8012,   34253, 18, }, // America/Indiana/Tell_City
    {  407, 8038,  128262, 15, }, // America/Indiana/Vevay
    {  407, 8060,  128277, 16, }, // America/Indiana/Vincennes
    {  407, 8086,   34304, 17, }, // America/Indiana/Winamac
    {  407, 8110,    1452,  6, }, // America/Inuvik
    {  407,  660,    1458,  7, }, // America/Iqaluit
    {  407, 2799,   34329,  7, }, // America/Jamaica
    {  407, 5380,    1472,  4, }, // America/Juneau
    {  407,  521,  118227,  7, }, // America/Kentucky/Louisville
    {  407, 8125,   34340, 18, }, // America/Kentucky/Monticello
    {  407, 5704,  128293, 10, }, // America/Kralendijk
    {  407, 4376,   34368,  6, }, // America/La_Paz
    {  407, 7169,    1520,  4, }, // America/Lima
    {  407, 3239,  128303, 11, }, // America/Los_Angeles
    {  407, 5932,  128314, 18, }, // America/Lower_Princes
    {  407, 8153,   34404,  6, }, // America/Maceio
    {  407, 8168,   34410,  7, }, // America/Managua
    {  407, 1908,    1566,  6, }, // America/Manaus
    {  407, 5897,    1572,  6, }, // America/Marigot
    {  407, 5805,    1578,  8, }, // America/Martinique
    {  407, 8184,    1587,  9, }, // America/Matamoros
    {  407, 2917,   34417,  8, }, // America/Mazatlan
    {  407, 8202,    1604,  8, }, // America/Menominee
    {  407, 8220,    1612,  6, }, // America/Merida
    {  407, 8235,    1618, 10, }, // America/Metlakatla
    {  407, 2949,   34425, 12, }, // America/Mexico_City
    {  407, 4391,  128332,  7, }, // America/Miquelon
    {  407, 8254,    1641,  7, }, // America/Moncton
    {  407, 8270,   34437,  8, }, // America/Monterrey
    {  407, 4098,    1657, 10, }, // America/Montevideo
    {  407, 5824,   34445,  9, }, // America/Montserrat
    {  407, 5481,   34454,  5, }, // America/Nassau
    {  407, 2379,   34459,  6, }, // America/New_York
    {  407, 8288,    1691,  3, }, // America/Nome
    {  407, 1850,   34465,  6, }, // America/Noronha
    {  407, 8301,  128339, 21, }, // America/North_Dakota/Beulah
    {  407, 8329,   34493, 22, }, // America/North_Dakota/Center
    {  407, 8357,  128360, 24, }, // America/North_Dakota/New_Salem
    {  407,  393,   28151,  3, }, // America/Nuuk
    {  407, 8388,    1776,  7, }, // America/Ojinaga
    {  407, 2356,    1783,  6, }, // America/Panama
    {  407, 6173,    1789, 10, }, // America/Paramaribo
    {  407, 2973,   34541,  6, }, // America/Phoenix
    {  407, 3945,  128384, 12, }, // America/Port-au-Prince
    {  407, 5954,  128396, 13, }, // America/Port_of_Spain
    {  407, 8404,   34560, 10, }, // America/Porto_Velho
    {  407, 5843,   34570,  9, }, // America/Puerto_Rico
    {  407, 4030,   34579, 12, }, // America/Punta_Arenas
    {  407, 8424,  128409, 12, }, // America/Rankin_Inlet
    {  407, 8445,    1876,  6, }, // America/Recife
    {  407, 1995,  128421,  7, }, // America/Regina
    {  407, 8460,  118409,  7, }, // America/Resolute
    {  407,  695,   34622, 10, }, // America/Rio_Branco
    {  407, 8477,   34632,  8, }, // America/Santarem
    {  407, 2201,   34640,  9, }, // America/Santiago
    {  407, 6111,   34649, 13, }, // America/Santo_Domingo
    {  407, 1878,   34662,  9, }, // America/Sao_Paulo
    {  407, 6899,   34671, 12, }, // America/Scoresbysund
    {  407, 8494,    1956,  5, }, // America/Sitka
    {  407, 8508,  128428, 15, }, // America/St_Barthelemy
    {  407, 2061,  128443,  9, }, // America/St_Johns
    {  407, 5863,  128452, 10, }, // America/St_Kitts
    {  407, 5880,  128462, 11, }, // America/St_Lucia
    {  407,  855,  128473, 10, }, // America/St_Thomas
    {  407, 5913,   34731, 12, }, // America/St_Vincent
    {  407, 8530,   34743, 12, }, // America/Swift_Current
    {  407, 5461,    2038, 11, }, // America/Tegucigalpa
    {  407, 5756,    2049,  4, }, // America/Thule
    {  407,  313,    2053,  7, }, // America/Tijuana
    {  407,  608,    2060,  7, }, // America/Toronto
    {  407, 5688,    2067,  7, }, // America/Tortola
    {  407, 2093,    2074,  8, }, // America/Vancouver
    {  407, 2144,   34755,  8, }, // America/Whitehorse
    {  407,  734,   34763,  7, }, // America/Winnipeg
    {  407, 8552,   34770,  7, }, // America/Yakutat
    {  407, 6001,  128483,  5, }, // Antarctica/Casey
    {  407, 6094,   34782,  6, }, // Antarctica/Davis
    {  407, 6133,  128488, 12, }, // Antarctica/DumontDUrville
    {  407, 8568,  128500,  7, }, // Antarctica/Macquarie
    {  407, 7066,   34808,  5, }, // Antarctica/Mawson
    {  407, 7113,   34813,  8, }, // Antarctica/McMurdo
    {  407, 8589,    2154,  6, }, // Antarctica/Palmer
    {  407, 7226,    2160,  6, }, // Antarctica/Rothera
    {  407, 7295,  128507,  6, }, // Antarctica/Syowa
    {  407, 8607,    2171,  4, }, // Antarctica/Troll
    {  407, 7374,   34825,  6, }, // Antarctica/Vostok
    {  407, 1429,  128513, 12, }, // Arctic/Longyearbyen
    {  407, 5570,    2192,  4, }, // Asia/Aden
    {  407, 5395,   34842,  6, }, // Asia/Almaty
    {  407, 3968,   28531,  4, }, // Asia/Amman
    {  407, 5511,   34848,  6, }, // Asia/Anadyr
    {  407, 5523,    2213,  5, }, // Asia/Aqtau
    {  407, 5368,    2218,  6, }, // Asia/Aqtobe
    {  407,  964,  128525,  7, }, // Asia/Ashgabat
    {  407, 8624,   51877,  6, }, // Asia/Atyrau
    {  407, 3607,    2237,  6, }, // Asia/Baghdad
    {  407, 5534,    2243,  7, }, // Asia/Bahrain
    {  407, 3653,    2250,  4, }, // Asia/Baku
    {  407, 4466,    2254,  7, }, // Asia/Bangkok
    {  407, 3571,    2261,  7, }, // Asia/Barnaul
    {  407, 4086,   34873,  6, }, // Asia/Beirut
    {  407, 3778,    2274,  6, }, // Asia/Bishkek
    {  407, 5989,   34879,  6, }, // Asia/Brunei
    {  407, 4601,    2286,  4, }, // Asia/Chita
    {  407, 4511,    2290,  7, }, // Asia/Colombo
    {  407, 4540,    2297,  6, }, // Asia/Damascus
    {  407, 1093,     593,  4, }, // Asia/Dhaka
    {  407, 6192,    2308,  4, }, // Asia/Dili
    {  407, 3596,    2312,  5, }, // Asia/Dubai
    {  407, 6159,    2317,  7, }, // Asia/Dushanbe
    {  407, 8636,    2324,  9, }, // Asia/Famagusta
    {  407, 8651,    2333,  4, }, // Asia/Gaza
    {  407, 4795,   34886,  6, }, // Asia/Hebron
    {  407, 1259,   34892,  9, }, // Asia/Ho_Chi_Minh
    {  407, 2704,   34901,  9, }, // Asia/Hong_Kong
    {  407, 4771,    2357,  4, }, // Asia/Hovd
    {  407, 4184,    2361,  7, }, // Asia/Irkutsk
    {  407, 7006,    2368,  7, }, // Asia/Jakarta
    {  407, 6992,   34916,  8, }, // Asia/Jayapura
    {  407, 1290,  128532,  8, }, // Asia/Jerusalem
    {  407, 3560,    2392,  5, }, // Asia/Kabul
    {  407, 4316,    2411,  8, }, // Asia/Kamchatka
    {  407, 4239,    2422,  6, }, // Asia/Karachi
    {  407, 1185,    2428,  8, }, // Asia/Kathmandu
    {  407, 8661,   34933,  7, }, // Asia/Khandyga
    {  407,  992,   34940,  7, }, // Asia/Kolkata
    {  407, 4197,   34947, 11, }, // Asia/Krasnoyarsk
    {  407, 4859,   34958, 12, }, // Asia/Kuala_Lumpur
    {  407, 5976,    2474,  6, }, // Asia/Kuching
    {  407, 5547,   34970,  6, }, // Asia/Kuwait
    {  407, 1211,    2486,  5, }, // Asia/Macau
    {  407, 4017,    2491,  7, }, // Asia/Magadan
    {  407, 1349,    2498,  7, }, // Asia/Makassar
    {  407, 7182,    2505,  6, }, // Asia/Manila
    {  407, 6920,   34981,  6, }, // Asia/Muscat
    {  407, 2554,   34987,  8, }, // Asia/Nicosia
    {  407, 8675,   34995, 11, }, // Asia/Novokuznetsk
    {  407, 4135,    2535, 11, }, // Asia/Novosibirsk
    {  407, 4229,    2546,  4, }, // Asia/Omsk
    {  407, 7145,   35006,  4, }, // Asia/Oral
    {  407, 6961,   35010,  8, }, // Asia/Phnom_Penh
    {  407, 8693,  128540, 10, }, // Asia/Pontianak
    {  407, 4214,   35018,  9, }, // Asia/Pyongyang
    {  407, 5559,    2579,  5, }, // Asia/Qatar
    {  407, 8708,   35027,  8, }, // Asia/Qostanay
    {  407, 4269,   35035,  9, }, // Asia/Qyzylorda
    {  407, 3584,   35044,  5, }, // Asia/Riyadh
    {  407, 4408,  128550,  7, }, // Asia/Sakhalin
    {  407, 7245,    2621,  9, }, // Asia/Samarkand
    {  407, 3279,    2630,  4, }, // Asia/Seoul
    {  407, 1053,   35049,  6, }, // Asia/Shanghai
    {  407, 3300,    2640,  8, }, // Asia/Singapore
    {  407, 4297,   37026, 13, }, // Asia/Srednekolymsk
    {  407, 3263,   35068,  6, }, // Asia/Taipei
    {  407, 4781,    2667,  7, }, // Asia/Tashkent
    {  407, 3915,   35074,  7, }, // Asia/Tbilisi
    {  407, 2772,   35081,  7, }, // Asia/Tehran
    {  407, 1317,   35088,  5, }, // Asia/Thimphu
    {  407, 2821,    2692,  5, }, // Asia/Tokyo
    {  407, 4572,    2697,  5, }, // Asia/Tomsk
    {  407, 1021,   35093, 10, }, // Asia/Ulaanbaatar
    {  407, 1159,    2712,  6, }, // Asia/Urumqi
    {  407, 8722,    2718,  8, }, // Asia/Ust-Nera
    {  407, 6977,  128557,  9, }, // Asia/Vientiane
    {  407, 4710,    2733, 11, }, // Asia/Vladivostok
    {  407, 4828,   35120,  7, }, // Asia/Yakutsk
    {  407, 1235,   28911,  6, }, // Asia/Yangon
    {  407, 3883,    2757, 12, }, // Asia/Yekaterinburg
    {  407, 3747,    2769,  6, }, // Asia/Yerevan
    {  407, 3663,  128566, 15, }, // Atlantic/Azores
    {  407, 5671,  128581,  7, }, // Atlantic/Bermuda
    {  407, 6628,  128588, 16, }, // Atlantic/Canary
    {  407, 3727,   37105, 10, }, // Atlantic/Cape_Verde
    {  407, 1395,  128604, 14, }, // Atlantic/Faroe
    {  407, 8736,    2853,  7, }, // Atlantic/Madeira
    {  407, 2748,   35205,  9, }, // Atlantic/Reykjavik
    {  407, 7272,   35214, 13, }, // Atlantic/South_Georgia
    {  407, 6821,   37160, 11, }, // Atlantic/St_Helena
    {  407, 6644,    2911,  6, }, // Atlantic/Stanley
    {  407, 1670,    2917,  8, }, // Australia/Adelaide
    {  407, 1635,   35237,  8, }, // Australia/Brisbane
    {  407, 1799,   35245, 10, }, // Australia/Broken_Hill
    {  407, 1583,    2943,  6, }, // Australia/Darwin
    {  407, 3637,  128618,  5, }, // Australia/Eucla
    {  407, 1516,    2954,  6, }, // Australia/Hobart
    {  407, 8753,    2960,  8, }, // Australia/Lindeman
    {  407, 1547,   37190,  8, }, // Australia/Lord_Howe
    {  407, 1727,    2976,  7, }, // Australia/Melbourne
    {  407, 1762,    2983,  4, }, // Australia/Perth
    {  407, 1463,   35263,  6, }, // Australia/Sydney
    {  407, 6441,    2993,  9, }, // Europe/Amsterdam
    {  407, 6216,   29143,  6, }, // Europe/Andorra
    {  407, 3620,    3009,  8, }, // Europe/Astrakhan
    {  407, 2300,   35269,  5, }, // Europe/Athens
    {  407, 6336,    3021,  7, }, // Europe/Belgrade
    {  407, 4757,    3028,  6, }, // Europe/Berlin
    {  407, 6488,    3034, 10, }, // Europe/Bratislava
    {  407, 2167,   35281,  6, }, // Europe/Brussels
    {  407, 3928,   35287,  8, }, // Europe/Bucharest
    {  407, 3806,   35295, 10, }, // Europe/Budapest
    {  407, 8772,  128623,  8, }, // Europe/Busingen
    {  407, 2583,  128631,  7, }, // Europe/Chisinau
    {  407, 6289,   35320, 10, }, // Europe/Copenhagen
    {  407, 2338,   35330,  6, }, // Europe/Dublin
    {  407, 6307,    3114,  9, }, // Europe/Gibraltar
    {  407, 8788,   35336,  6, }, // Europe/Guernsey
    {  407, 6612,    3129,  8, }, // Europe/Helsinki
    {  407, 8804,  128638, 10, }, // Europe/Isle_of_Man
    {  407, 1130,   35352,  8, }, // Europe/Istanbul
    {  407, 8823,  128648,  5, }, // Europe/Jersey
    {  407, 3979,    3158, 11, }, // Europe/Kaliningrad
    {  407, 8929,    3169,  5, }, // Europe/Kirov
    {  407, 2527,    3174,  4, }, // Europe/Kyiv
    {  407, 3213,   35376,  7, }, // Europe/Lisbon
    {  407, 6506,   35383,  7, }, // Europe/Ljubljana
    {  407, 2501,    3195,  6, }, // Europe/London
    {  407, 6365,   35390, 10, }, // Europe/Luxembourg
    {  407, 4892,    3212,  6, }, // Europe/Madrid
    {  407, 6397,    3218,  5, }, // Europe/Malta
    {  407, 6582,  128653, 11, }, // Europe/Mariehamn
    {  407, 3693,    3232,  5, }, // Europe/Minsk
    {  407, 6410,    3237,  6, }, // Europe/Monaco
    {  407, 3537,    3243,  6, }, // Europe/Moscow
    {  407, 6458,    3249,  4, }, // Europe/Oslo
    {  407, 4284,   35408,  5, }, // Europe/Paris
    {  407, 6424,    3258,  9, }, // Europe/Podgorica
    {  407, 6275,    3267,  5, }, // Europe/Prague
    {  407, 8837,    3272,  4, }, // Europe/Riga
    {  407, 6324,    3276,  3, }, // Europe/Rome
    {  407, 4331,    3279,  6, }, // Europe/Samara
    {  407, 6470,   35413, 10, }, // Europe/San_Marino
    {  407, 6245,    3295,  7, }, // Europe/Sarajevo
    {  407, 4451,    3302,  7, }, // Europe/Saratov
    {  407, 8849,    3309, 10, }, // Europe/Simferopol
    {  407, 6383,  128664,  6, }, // Europe/Skopje
    {  407, 6599,   35447,  6, }, // Europe/Sofia
    {  407, 6523,   35453,  8, }, // Europe/Stockholm
    {  407, 8867,   35461,  5, }, // Europe/Tallinn
    {  407, 6202,    3343,  6, }, // Europe/Tirane
    {  407, 8882,  128670,  9, }, // Europe/Ulyanovsk
    {  407, 6352,    3358,  5, }, // Europe/Vaduz
    {  407, 6567,    3363,  7, }, // Europe/Vatican
    {  407, 6231,   37299,  5, }, // Europe/Vienna
    {  407, 8899,  128679,  6, }, // Europe/Vilnius
    {  407, 4727,    3381,  9, }, // Europe/Volgograd
    {  407, 3190,    3390,  7, }, // Europe/Warsaw
    {  407, 6261,    3397,  6, }, // Europe/Zagreb
    {  407, 6540,   35488,  5, }, // Europe/Zurich
    {  407, 5078,   35493, 12, }, // Indian/Antananarivo
    {  407, 6947,    3421,  5, }, // Indian/Chagos
    {  407, 6046,  128685, 14, }, // Indian/Christmas
    {  407, 6063,  128699, 16, }, // Indian/Cocos
    {  407, 5029,   52314,  6, }, // Indian/Comoro
    {  407, 6661,    3460,  8, }, // Indian/Kerguelen
    {  407, 7260,   35516,  4, }, // Indian/Mahe
    {  407, 7050,   35520,  7, }, // Indian/Maldives
    {  407, 4069,  128715,  9, }, // Indian/Mauritius
    {  407, 5098,   35537,  5, }, // Indian/Mayotte
    {  407, 7211,  128724,  8, }, // Indian/Reunion
    {  407, 4422,   35549,  5, }, // Pacific/Apia
    {  407,  932,    3502,  6, }, // Pacific/Auckland
    {  407, 3706,  128732,  8, }, // Pacific/Bougainville
    {  407, 3015,   37380,  5, }, // Pacific/Chatham
    {  407, 3157,  128740,  3, }, // Pacific/Chuuk
    {  407, 2237,  128743, 18, }, // Pacific/Easter
    {  407, 7360,    3536,  5, }, // Pacific/Efate
    {  407, 7327,    3541,  7, }, // Pacific/Fakaofo
    {  407, 3902,    3548,  4, }, // Pacific/Fiji
    {  407, 7343,    3552,  8, }, // Pacific/Funafuti
    {  407, 6678,    3560,  9, }, // Pacific/Galapagos
    {  407, 6696,   35580,  7, }, // Pacific/Gambier
    {  407, 3822,   35587, 11, }, // Pacific/Guadalcanal
    {  407, 6018,  128761,  4, }, // Pacific/Guam
    {  407, 2723,   35598,  8, }, // Pacific/Honolulu
    {  407, 3049,   35606,  9, }, // Pacific/Kanton
    {  407, 3998,    3616, 10, }, // Pacific/Kiritimati
    {  407, 7019,    3626,  6, }, // Pacific/Kosrae
    {  407, 2842,  128765,  9, }, // Pacific/Kwajalein
    {  407, 4877,    3641,  6, }, // Pacific/Majuro
    {  407, 4051,  128774, 16, }, // Pacific/Marquesas
    {  407, 8914,   35636,  6, }, // Pacific/Midway
    {  407, 7084,    3680,  5, }, // Pacific/Nauru
    {  407, 7132,    3685,  4, }, // Pacific/Niue
    {  407, 4168,    3689,  7, }, // Pacific/Norfolk
    {  407, 7098,    3696,  5, }, // Pacific/Noumea
    {  407, 3126,   35642,  9, }, // Pacific/Pago_Pago
    {  407, 7155,    3710,  5, }, // Pacific/Palau
    {  407, 7194,  128790, 18, }, // Pacific/Pitcairn
    {  407, 3096,  128808,  6, }, // Pacific/Pohnpei
    {  407, 4807,   37477, 11, }, // Pacific/Port_Moresby
    {  407, 6076,    3739,  9, }, // Pacific/Rarotonga
    {  407, 6031,   35668,  6, }, // Pacific/Saipan
    {  407, 7312,   35674,  6, }, // Pacific/Tahiti
    {  407, 6712,    3759,  6, }, // Pacific/Tarawa
    {  407, 4583,    3765,  9, }, // Pacific/Tongatapu
    {  407, 7392,   35680,  4, }, // Pacific/Wake
    {  407, 7405,   35684,  5, }, // Pacific/Wallis
    {  408,   14,       0,  6, }, // Africa/Asmara Machame/Latin/Tanzania
    {  408, 4435,       6,  8, }, // Africa/Sao_Tome
    {  408, 4252,      14,  8, }, // America/Asuncion
    {  408,  237,      22,  8, }, // America/Atikokan
    {  408, 7623,      30, 17, }, // America/Bahia_Banderas
    {  408, 3868,      47,  6, }, // America/Cancun
    {  408, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  408, 5723,      66,  7, }, // America/Curacao
    {  408,  481,      73, 13, }, // America/Indiana/Knox
    {  408, 7961,      86, 16, }, // America/Indiana/Marengo
    {  408, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  408, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  408, 8038,     139, 14, }, // America/Indiana/Vevay
    {  408, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  408, 8086,     171, 16, }, // America/Indiana/Winamac
    {  408, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  408, 5932,     207, 22, }, // America/Lower_Princes
    {  408, 8220,     229,  6, }, // America/Merida
    {  408, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  408, 8329,     255, 20, }, // America/North_Dakota/Center
    {  408, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  408,  393,     298,  4, }, // America/Nuuk
    {  408, 6899,     302, 16, }, // America/Scoresbysund
    {  408, 8508,     318, 14, }, // America/St_Barthelemy
    {  408, 2061,     332, 10, }, // America/St_Johns
    {  408, 5863,     342,  9, }, // America/St_Kitts
    {  408, 5880,     351,  9, }, // America/St_Lucia
    {  408,  855,     360, 10, }, // America/St_Thomas
    {  408, 5913,     370, 11, }, // America/St_Vincent
    {  408, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  408, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  408, 1185,     413,  9, }, // Asia/Kathmandu
    {  408,  992,     422,  7, }, // Asia/Kolkata
    {  408, 1211,     429,  5, }, // Asia/Macau
    {  408, 1235,     442,  6, }, // Asia/Yangon
    {  408, 1395,     448,  5, }, // Atlantic/Faroe
    {  408, 6821,     453, 10, }, // Atlantic/St_Helena
    {  408, 2527,     463,  4, }, // Europe/Kyiv
    {  408, 7211,     467,  7, }, // Indian/Reunion
    {  408, 3157,     474,  5, }, // Pacific/Chuuk
    {  408, 3096,     479,  7, }, // Pacific/Pohnpei
    {  409, 6788,  128814,  6, }, // Africa/Abidjan Maithili/Devanagari/India
    {  409, 6760,  128820,  4, }, // Africa/Accra
    {  409, 5059,  128824, 10, }, // Africa/Addis_Ababa
    {  409, 7420,   88138,  9, }, // Africa/Algiers
    {  409,   14,  128834,  6, }, // Africa/Asmara
    {  409,   44,  128840,  5, }, // Africa/Bamako
    {  409, 5257,  128845,  5, }, // Africa/Bangui
    {  409, 6746,  128850,  5, }, // Africa/Banjul
    {  409, 7435,   88172,  5, }, // Africa/Bissau
    {  409, 4957,  128855,  9, }, // Africa/Blantyre
    {  409, 5287,  128864,  9, }, // Africa/Brazzaville
    {  409, 4922,  113322,  9, }, // Africa/Bujumbura
    {  409, 2320,  128873,  6, }, // Africa/Cairo
    {  409, 4117,   29933, 11, }, // Africa/Casablanca
    {  409, 7449,  128879,  5, }, // Africa/Ceuta
    {  409, 6773,   88216,  8, }, // Africa/Conakry
    {  409, 6840,   88224,  4, }, // Africa/Dakar
    {  409, 5130,  128884,  9, }, // Africa/Dar_es_Salaam
    {  409, 5043,   88239,  6, }, // Africa/Djibouti
    {  409, 5243,  128893,  5, }, // Africa/Douala
    {  409, 5166,  128898,  7, }, // Africa/El_Aaiun
    {  409, 6853,   29990,  8, }, // Africa/Freetown
    {  409, 4906,  128905,  7, }, // Africa/Gaborone
    {  409, 5015,   88274,  5, }, // Africa/Harare
    {  409, 4479,  128912, 11, }, // Africa/Johannesburg
    {  409, 4499,   30022,  4, }, // Africa/Juba
    {  409, 5151,  128923,  6, }, // Africa/Kampala
    {  409, 4524,  128929,  6, }, // Africa/Khartoum
    {  409, 4973,   88295,  6, }, // Africa/Kigali
    {  409, 5306,   88301,  7, }, // Africa/Kinshasa
    {  409, 4744,   88308,  5, }, // Africa/Lagos
    {  409, 5336,   88313,  9, }, // Africa/Libreville
    {  409, 6869,   88323,  3, }, // Africa/Lome
    {  409, 5211,   88326,  6, }, // Africa/Luanda
    {  409, 4939,   88332,  9, }, // Africa/Lubumbashi
    {  409, 5001,   30089,  6, }, // Africa/Lusaka
    {  409, 5322,   88341,  5, }, // Africa/Malabo
    {  409, 4987,  128935,  5, }, // Africa/Maputo
    {  409, 5197,  128940,  5, }, // Africa/Maseru
    {  409, 5182,  128945,  9, }, // Africa/Mbabane
    {  409, 5113,  113400,  8, }, // Africa/Mogadishu
    {  409, 7034,   88367,  9, }, // Africa/Monrovia
    {  409, 3853,   88376,  6, }, // Africa/Nairobi
    {  409, 5271,  128954,  8, }, // Africa/Ndjamena
    {  409, 5354,  128962,  7, }, // Africa/Niamey
    {  409, 6803,  128969,  6, }, // Africa/Nouakchott
    {  409, 6727,  128975,  7, }, // Africa/Ouagadougou
    {  409, 5225,   88412, 11, }, // Africa/Porto-Novo
    {  409, 4435,   88423,  7, }, // Africa/Sao_Tome
    {  409, 2866,   88430,  8, }, // Africa/Tripoli
    {  409, 6554,   30196,  7, }, // Africa/Tunis
    {  409, 4152,  128982,  7, }, // Africa/Windhoek
    {  409,  132,  128989,  3, }, // America/Adak
    {  409, 3346,  128992,  7, }, // America/Anchorage
    {  409, 5607,   88448,  8, }, // America/Anguilla
    {  409, 5624,  128999,  7, }, // America/Antigua
    {  409, 4554,  129006,  8, }, // America/Araguaina
    {  409,  166,  113472, 12, }, // America/Argentina/Buenos_Aires
    {  409,   91,   30258, 10, }, // America/Argentina/Catamarca
    {  409,  270,   88483,  8, }, // America/Argentina/Cordoba
    {  409,  441,  129014,  5, }, // America/Argentina/Jujuy
    {  409, 7462,  113505,  9, }, // America/Argentina/La_Rioja
    {  409,  565,  129019,  7, }, // America/Argentina/Mendoza
    {  409, 7489,  129026, 10, }, // America/Argentina/Rio_Gallegos
    {  409, 7520,  129036,  5, }, // America/Argentina/Salta
    {  409, 7544,  129041,  7, }, // America/Argentina/San_Juan
    {  409, 5580,  129048,  7, }, // America/Argentina/San_Luis
    {  409, 7571,   30328,  7, }, // America/Argentina/Tucuman
    {  409, 7597,  129055,  7, }, // America/Argentina/Ushuaia
    {  409, 5640,   88553,  5, }, // America/Aruba
    {  409, 4252,  129062,  8, }, // America/Asuncion
    {  409,  237,  129070,  7, }, // America/Atikokan
    {  409, 3679,   88574,  5, }, // America/Bahia
    {  409, 7623,  129077, 13, }, // America/Bahia_Banderas
    {  409, 5654,  129090,  7, }, // America/Barbados
    {  409, 7646,  129097,  5, }, // America/Belem
    {  409, 5407,   30399,  5, }, // America/Belize
    {  409, 7660,  129102, 12, }, // America/Blanc-Sablon
    {  409, 7681,  129114, 10, }, // America/Boa_Vista
    {  409, 4361,   88631,  6, }, // America/Bogota
    {  409, 7699,  129124,  5, }, // America/Boise
    {  409, 7713,  129129, 12, }, // America/Cambridge_Bay
    {  409, 7735,  129141, 14, }, // America/Campo_Grande
    {  409, 3868,  113637,  6, }, // America/Cancun
    {  409, 4694,   30470,  7, }, // America/Caracas
    {  409, 4345,   30477,  5, }, // America/Cayenne
    {  409, 5496,   30482,  5, }, // America/Cayman
    {  409, 2260,   88682,  6, }, // America/Chicago
    {  409, 7756,   88688,  8, }, // America/Chihuahua
    {  409, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  409, 5422,   88712, 11, }, // America/Costa_Rica
    {  409, 7796,   30510,  8, }, // America/Creston
    {  409, 3791,  129155,  6, }, // America/Cuiaba
    {  409, 5723,  129161,  9, }, // America/Curacao
    {  409, 7812,  129170,  7, }, // America/Danmarkshavn
    {  409, 7833,  129177,  5, }, // America/Dawson
    {  409, 7848,  129182, 11, }, // America/Dawson_Creek
    {  409,  805,  129193,  5, }, // America/Denver
    {  409, 3465,  129198,  8, }, // America/Detroit
    {  409, 5739,   88775,  8, }, // America/Dominica
    {  409,  893,  129206,  8, }, // America/Edmonton
    {  409, 7869,  129214,  7, }, // America/Eirunepe
    {  409, 5441,   88789, 11, }, // America/El_Salvador
    {  409, 7886,  129221, 11, }, // America/Fort_Nelson
    {  409, 7906,  129232, 10, }, // America/Fortaleza
    {  409, 7924,   30628,  8, }, // America/Glace_Bay
    {  409, 6881,   30637,  6, }, // America/Goose_Bay
    {  409, 4612,   88845, 11, }, // America/Grand_Turk
    {  409, 5770,   30657,  8, }, // America/Grenada
    {  409, 5786,  129242,  9, }, // America/Guadeloupe
    {  409, 3760,   30674, 10, }, // America/Guatemala
    {  409, 4841,  129251, 10, }, // America/Guayaquil
    {  409, 6932,   30692,  6, }, // America/Guyana
    {  409, 1939,   30698,  9, }, // America/Halifax
    {  409, 2281,   88889,  5, }, // America/Havana
    {  409, 7942,  129261,  8, }, // America/Hermosillo
    {  409,  348,   88905, 13, }, // America/Indiana/Indianapolis
    {  409,  481,  129269, 15, }, // America/Indiana/Knox
    {  409, 7961,  129284, 17, }, // America/Indiana/Marengo
    {  409, 7985,   88950, 20, }, // America/Indiana/Petersburg
    {  409, 8012,   88970, 18, }, // America/Indiana/Tell_City
    {  409, 8038,  129301, 14, }, // America/Indiana/Vevay
    {  409, 8060,  129315, 17, }, // America/Indiana/Vincennes
    {  409, 8086,  129332, 17, }, // America/Indiana/Winamac
    {  409, 8110,  129349,  6, }, // America/Inuvik
    {  409,  660,  129355,  7, }, // America/Iqaluit
    {  409, 2799,   89044,  5, }, // America/Jamaica
    {  409, 5380,  129362,  5, }, // America/Juneau
    {  409,  521,   89056,  8, }, // America/Kentucky/Louisville
    {  409, 8125,  129367, 18, }, // America/Kentucky/Monticello
    {  409, 5704,  129385, 10, }, // America/Kralendijk
    {  409, 4376,   30926,  6, }, // America/La_Paz
    {  409, 7169,   30932,  4, }, // America/Lima
    {  409, 3239,  129395, 11, }, // America/Los_Angeles
    {  409, 5932,  129406, 20, }, // America/Lower_Princes
    {  409, 8153,  129426,  5, }, // America/Maceio
    {  409, 8168,   30977,  6, }, // America/Managua
    {  409, 1908,   89145,  4, }, // America/Manaus
    {  409, 5897,  129431,  7, }, // America/Marigot
    {  409, 5805,   89156,  9, }, // America/Martinique
    {  409, 8184,   89165,  9, }, // America/Matamoros
    {  409, 2917,  129438,  7, }, // America/Mazatlan
    {  409, 8202,  129445,  8, }, // America/Menominee
    {  409, 8220,  129453,  6, }, // America/Merida
    {  409, 8235,  129459, 10, }, // America/Metlakatla
    {  409, 2949,  129469, 13, }, // America/Mexico_City
    {  409, 4391,  129482,  9, }, // America/Miquelon
    {  409, 8254,  129491,  8, }, // America/Moncton
    {  409, 8270,  129499,  8, }, // America/Monterrey
    {  409, 4098,  129507, 11, }, // America/Montevideo
    {  409, 5824,   89249,  9, }, // America/Montserrat
    {  409, 5481,  129518,  4, }, // America/Nassau
    {  409, 2379,  129522,  9, }, // America/New_York
    {  409, 8288,   89185,  3, }, // America/Nome
    {  409, 1850,   89272,  8, }, // America/Noronha
    {  409, 8301,  129531, 19, }, // America/North_Dakota/Beulah
    {  409, 8329,  129550, 18, }, // America/North_Dakota/Center
    {  409, 8357,  129568, 22, }, // America/North_Dakota/New_Salem
    {  409,  393,   89343,  3, }, // America/Nuuk
    {  409, 8388,  114210,  7, }, // America/Ojinaga
    {  409, 2356,   89355,  5, }, // America/Panama
    {  409, 6173,   89360, 10, }, // America/Paramaribo
    {  409, 2973,  114226,  7, }, // America/Phoenix
    {  409, 3945,  129590, 15, }, // America/Port-au-Prince
    {  409, 5954,  129605, 14, }, // America/Port_of_Spain
    {  409, 8404,  129619, 13, }, // America/Porto_Velho
    {  409, 5843,   89420, 11, }, // America/Puerto_Rico
    {  409, 4030,  129632, 12, }, // America/Punta_Arenas
    {  409, 8424,   89443, 12, }, // America/Rankin_Inlet
    {  409, 8445,  129644,  5, }, // America/Recife
    {  409, 1995,  129649,  6, }, // America/Regina
    {  409, 8460,  129655,  7, }, // America/Resolute
    {  409,  695,  129662, 12, }, // America/Rio_Branco
    {  409, 8477,  114316,  8, }, // America/Santarem
    {  409, 2201,   89491,  9, }, // America/Santiago
    {  409, 6111,  129674, 13, }, // America/Santo_Domingo
    {  409, 1878,   89513,  9, }, // America/Sao_Paulo
    {  409, 6899,  129687, 13, }, // America/Scoresbysund
    {  409, 8494,  129700,  5, }, // America/Sitka
    {  409, 8508,  129705, 13, }, // America/St_Barthelemy
    {  409, 2061,  129718, 10, }, // America/St_Johns
    {  409, 5863,   89565, 10, }, // America/St_Kitts
    {  409, 5880,  129728, 11, }, // America/St_Lucia
    {  409,  855,  129739,  9, }, // America/St_Thomas
    {  409, 5913,   89595, 12, }, // America/St_Vincent
    {  409, 8530,  129748, 13, }, // America/Swift_Current
    {  409, 5461,   31473, 12, }, // America/Tegucigalpa
    {  409, 5756,  129761,  4, }, // America/Thule
    {  409,  313,   89637,  7, }, // America/Tijuana
    {  409,  608,   89644,  6, }, // America/Toronto
    {  409, 5688,   89650,  8, }, // America/Tortola
    {  409, 2093,  129765,  7, }, // America/Vancouver
    {  409, 2144,  129772,  9, }, // America/Whitehorse
    {  409,  734,  129781,  7, }, // America/Winnipeg
    {  409, 8552,  129788,  6, }, // America/Yakutat
    {  409, 6001,   31541,  4, }, // Antarctica/Casey
    {  409, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  409, 8568,  129794,  8, }, // Antarctica/Macquarie
    {  409, 1429,  129802, 11, }, // Arctic/Longyearbyen
    {  409, 5570,  129813,  3, }, // Asia/Aden
    {  409, 5395,  129816,  7, }, // Asia/Almaty
    {  409, 3968,   89786,  6, }, // Asia/Amman
    {  409, 5511,  129823,  6, }, // Asia/Anadyr
    {  409, 5523,   31646,  6, }, // Asia/Aqtau
    {  409, 5368,   89798,  6, }, // Asia/Aqtobe
    {  409,  964,   31659,  8, }, // Asia/Ashgabat
    {  409, 8624,  129829,  6, }, // Asia/Atyrau
    {  409, 3607,   89809,  5, }, // Asia/Baghdad
    {  409, 5534,   89814,  5, }, // Asia/Bahrain
    {  409, 3653,   30522,  4, }, // Asia/Baku
    {  409, 4466,  129835,  6, }, // Asia/Bangkok
    {  409, 3571,   89825,  6, }, // Asia/Barnaul
    {  409, 4086,  114578,  5, }, // Asia/Beirut
    {  409, 3778,   31704,  7, }, // Asia/Bishkek
    {  409, 5989,   89836,  7, }, // Asia/Brunei
    {  409, 4601,  129841,  4, }, // Asia/Chita
    {  409, 4511,  129845,  7, }, // Asia/Colombo
    {  409, 4540,  129852,  6, }, // Asia/Damascus
    {  409, 1093,   89862,  4, }, // Asia/Dhaka
    {  409, 6192,   31740,  4, }, // Asia/Dili
    {  409, 3596,   89870,  4, }, // Asia/Dubai
    {  409, 6159,   89874,  7, }, // Asia/Dushanbe
    {  409, 8636,   31757, 10, }, // Asia/Famagusta
    {  409, 8651,   31767,  4, }, // Asia/Gaza
    {  409, 4795,  129858,  7, }, // Asia/Hebron
    {  409, 1259,   89904, 16, }, // Asia/Ho_Chi_Minh
    {  409, 2704,  129865,  9, }, // Asia/Hong_Kong
    {  409, 4771,  129874,  5, }, // Asia/Hovd
    {  409, 4184,  129879,  7, }, // Asia/Irkutsk
    {  409, 7006,   89944,  7, }, // Asia/Jakarta
    {  409, 6992,   31811,  7, }, // Asia/Jayapura
    {  409, 1290,  129886,  8, }, // Asia/Jerusalem
    {  409, 3560,   89957,  5, }, // Asia/Kabul
    {  409, 4316,  129894,  6, }, // Asia/Kamchatka
    {  409, 4239,   31840,  5, }, // Asia/Karachi
    {  409, 1185,   89969,  8, }, // Asia/Kathmandu
    {  409, 8661,  129900,  8, }, // Asia/Khandyga
    {  409,  992,   89984,  7, }, // Asia/Kolkata
    {  409, 4197,  129908, 15, }, // Asia/Krasnoyarsk
    {  409, 4859,  129923, 11, }, // Asia/Kuala_Lumpur
    {  409, 5976,  129934,  6, }, // Asia/Kuching
    {  409, 5547,   31902,  5, }, // Asia/Kuwait
    {  409, 1211,   90016,  4, }, // Asia/Macau
    {  409, 4017,  114696,  6, }, // Asia/Magadan
    {  409, 1349,  129940,  6, }, // Asia/Makassar
    {  409, 7182,   31928,  5, }, // Asia/Manila
    {  409, 6920,   90026,  5, }, // Asia/Muscat
    {  409, 2554,  129946,  8, }, // Asia/Nicosia
    {  409, 8675,  129954, 12, }, // Asia/Novokuznetsk
    {  409, 4135,  129966, 12, }, // Asia/Novosibirsk
    {  409, 4229,  129978,  5, }, // Asia/Omsk
    {  409, 7145,  129983,  4, }, // Asia/Oral
    {  409, 6961,  114731,  9, }, // Asia/Phnom_Penh
    {  409, 8693,  129987,  9, }, // Asia/Pontianak
    {  409, 4214,   90095, 10, }, // Asia/Pyongyang
    {  409, 5559,   90105,  3, }, // Asia/Qatar
    {  409, 8708,  129996,  8, }, // Asia/Qostanay
    {  409, 4269,  130004, 10, }, // Asia/Qyzylorda
    {  409, 3584,   90127,  5, }, // Asia/Riyadh
    {  409, 4408,  130014,  6, }, // Asia/Sakhalin
    {  409, 7245,   90138,  6, }, // Asia/Samarkand
    {  409, 3279,  130020,  5, }, // Asia/Seoul
    {  409, 1053,   32052,  6, }, // Asia/Shanghai
    {  409, 3300,   32058,  8, }, // Asia/Singapore
    {  409, 4297,  130025, 16, }, // Asia/Srednekolymsk
    {  409, 3263,  130041,  6, }, // Asia/Taipei
    {  409, 4781,   90175,  6, }, // Asia/Tashkent
    {  409, 3915,  130047,  7, }, // Asia/Tbilisi
    {  409, 2772,   32104,  6, }, // Asia/Tehran
    {  409, 1317,  130054,  6, }, // Asia/Thimphu
    {  409, 2821,   90193,  6, }, // Asia/Tokyo
    {  409, 4572,  130060,  5, }, // Asia/Tomsk
    {  409, 1021,  130065,  7, }, // Asia/Ulaanbaatar
    {  409, 1159,  130072,  6, }, // Asia/Urumqi
    {  409, 8722,  130078,  9, }, // Asia/Ust-Nera
    {  409, 6977,  130087,  9, }, // Asia/Vientiane
    {  409, 4710,  130096, 13, }, // Asia/Vladivostok
    {  409, 4828,  130109,  9, }, // Asia/Yakutsk
    {  409, 1235,  130118,  6, }, // Asia/Yangon
    {  409, 3883,  130124, 13, }, // Asia/Yekaterinburg
    {  409, 3747,  130137,  6, }, // Asia/Yerevan
    {  409, 3663,  130143,  6, }, // Atlantic/Azores
    {  409, 5671,   90280,  6, }, // Atlantic/Bermuda
    {  409, 6628,   32223,  6, }, // Atlantic/Canary
    {  409, 3727,  130149,  8, }, // Atlantic/Cape_Verde
    {  409, 1395,  130157,  4, }, // Atlantic/Faroe
    {  409, 8736,  130161,  5, }, // Atlantic/Madeira
    {  409, 2748,  130166,  8, }, // Atlantic/Reykjavik
    {  409, 7272,  130174, 15, }, // Atlantic/South_Georgia
    {  409, 6821,   90334, 11, }, // Atlantic/St_Helena
    {  409, 6644,   32281,  7, }, // Atlantic/Stanley
    {  409, 1670,   90352,  6, }, // Australia/Adelaide
    {  409, 1635,  130189,  8, }, // Australia/Brisbane
    {  409, 1799,  130197, 11, }, // Australia/Broken_Hill
    {  409, 1583,  130208,  7, }, // Australia/Darwin
    {  409, 3637,  130215,  6, }, // Australia/Eucla
    {  409, 1516,  130221,  6, }, // Australia/Hobart
    {  409, 8753,  130227,  7, }, // Australia/Lindeman
    {  409, 1547,  130234, 10, }, // Australia/Lord_Howe
    {  409, 1727,   90408,  8, }, // Australia/Melbourne
    {  409, 1762,   90416,  4, }, // Australia/Perth
    {  409, 1463,   32362,  5, }, // Australia/Sydney
    {  409, 6441,  130244,  9, }, // Europe/Amsterdam
    {  409, 6216,  130253,  6, }, // Europe/Andorra
    {  409, 3620,  130259, 10, }, // Europe/Astrakhan
    {  409, 2300,   90446,  5, }, // Europe/Athens
    {  409, 6336,   32400,  8, }, // Europe/Belgrade
    {  409, 4757,   90451,  6, }, // Europe/Berlin
    {  409, 6488,  130269, 11, }, // Europe/Bratislava
    {  409, 2167,  130280,  8, }, // Europe/Brussels
    {  409, 3928,  115035,  9, }, // Europe/Bucharest
    {  409, 3806,   90488,  9, }, // Europe/Budapest
    {  409, 8772,  130288,  7, }, // Europe/Busingen
    {  409, 2583,  130295,  7, }, // Europe/Chisinau
    {  409, 6289,  130302, 10, }, // Europe/Copenhagen
    {  409, 2338,   90523,  5, }, // Europe/Dublin
    {  409, 6307,   90528, 10, }, // Europe/Gibraltar
    {  409, 8788,  130312,  6, }, // Europe/Guernsey
    {  409, 6612,  130318,  8, }, // Europe/Helsinki
    {  409, 8804,  130326, 11, }, // Europe/Isle_of_Man
    {  409, 1130,  130337, 10, }, // Europe/Istanbul
    {  409, 8823,   90572,  5, }, // Europe/Jersey
    {  409, 3979,  130347, 11, }, // Europe/Kaliningrad
    {  409, 8929,   90589,  5, }, // Europe/Kirov
    {  409, 2527,   90594,  3, }, // Europe/Kyiv
    {  409, 3213,  130358,  5, }, // Europe/Lisbon
    {  409, 6506,  130363,  9, }, // Europe/Ljubljana
    {  409, 2501,   90608,  4, }, // Europe/London
    {  409, 6365,  130372,  9, }, // Europe/Luxembourg
    {  409, 4892,  130381,  7, }, // Europe/Madrid
    {  409, 6397,  130388,  5, }, // Europe/Malta
    {  409, 6582,  130393,  8, }, // Europe/Mariehamn
    {  409, 3693,  130401,  6, }, // Europe/Minsk
    {  409, 6410,  130407,  6, }, // Europe/Monaco
    {  409, 3537,  130413,  6, }, // Europe/Moscow
    {  409, 6458,   90656,  5, }, // Europe/Oslo
    {  409, 4284,   32634,  5, }, // Europe/Paris
    {  409, 6424,  130419,  9, }, // Europe/Podgorica
    {  409, 6275,   90671,  5, }, // Europe/Prague
    {  409, 8837,   32652,  4, }, // Europe/Riga
    {  409, 6324,   90303,  3, }, // Europe/Rome
    {  409, 4331,   90680,  5, }, // Europe/Samara
    {  409, 6470,  130428,  9, }, // Europe/San_Marino
    {  409, 6245,  130437,  7, }, // Europe/Sarajevo
    {  409, 4451,  130444,  7, }, // Europe/Saratov
    {  409, 8849,  130451, 11, }, // Europe/Simferopol
    {  409, 6383,  130462,  7, }, // Europe/Skopje
    {  409, 6599,  115197,  6, }, // Europe/Sofia
    {  409, 6523,  130469,  8, }, // Europe/Stockholm
    {  409, 8867,  130477,  5, }, // Europe/Tallinn
    {  409, 6202,  130482,  6, }, // Europe/Tirane
    {  409, 8882,  130488, 12, }, // Europe/Ulyanovsk
    {  409, 6352,  130500,  5, }, // Europe/Vaduz
    {  409, 6567,  130505,  6, }, // Europe/Vatican
    {  409, 6231,  130511,  5, }, // Europe/Vienna
    {  409, 8899,  130516,  7, }, // Europe/Vilnius
    {  409, 4727,  130523, 10, }, // Europe/Volgograd
    {  409, 3190,  130533,  5, }, // Europe/Warsaw
    {  409, 6261,  130538,  5, }, // Europe/Zagreb
    {  409, 6540,  130543,  7, }, // Europe/Zurich
    {  409, 5078,  130550, 12, }, // Indian/Antananarivo
    {  409, 6947,  130562,  4, }, // Indian/Chagos
    {  409, 6046,   90842,  7, }, // Indian/Christmas
    {  409, 6063,   90849,  5, }, // Indian/Cocos
    {  409, 5029,   90854,  6, }, // Indian/Comoro
    {  409, 6661,  130566,  9, }, // Indian/Kerguelen
    {  409, 7260,   32851,  4, }, // Indian/Mahe
    {  409, 7050,   90867,  6, }, // Indian/Maldives
    {  409, 4069,  130575,  6, }, // Indian/Mauritius
    {  409, 5098,  130581,  5, }, // Indian/Mayotte
    {  409, 7211,  130586,  8, }, // Indian/Reunion
    {  409,  932,  130594,  6, }, // Pacific/Auckland
    {  409, 3015,  130600,  4, }, // Pacific/Chatham
    {  409, 3157,     474,  5, }, // Pacific/Chuuk
    {  409, 2237,   90920,  5, }, // Pacific/Easter
    {  409, 6678,  130604,  9, }, // Pacific/Galapagos
    {  409, 4168,  130613,  6, }, // Pacific/Norfolk
    {  409, 3096,     479,  7, }, // Pacific/Pohnpei
    {  410,   14,       0,  6, }, // Africa/Asmara Makhuwa-Meetto/Latin/Mozambique
    {  410, 4435,       6,  8, }, // Africa/Sao_Tome
    {  410, 4252,      14,  8, }, // America/Asuncion
    {  410,  237,      22,  8, }, // America/Atikokan
    {  410, 7623,      30, 17, }, // America/Bahia_Banderas
    {  410, 3868,      47,  6, }, // America/Cancun
    {  410, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  410, 5723,      66,  7, }, // America/Curacao
    {  410,  481,      73, 13, }, // America/Indiana/Knox
    {  410, 7961,      86, 16, }, // America/Indiana/Marengo
    {  410, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  410, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  410, 8038,     139, 14, }, // America/Indiana/Vevay
    {  410, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  410, 8086,     171, 16, }, // America/Indiana/Winamac
    {  410, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  410, 5932,     207, 22, }, // America/Lower_Princes
    {  410, 8220,     229,  6, }, // America/Merida
    {  410, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  410, 8329,     255, 20, }, // America/North_Dakota/Center
    {  410, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  410,  393,     298,  4, }, // America/Nuuk
    {  410, 6899,     302, 16, }, // America/Scoresbysund
    {  410, 8508,     318, 14, }, // America/St_Barthelemy
    {  410, 2061,     332, 10, }, // America/St_Johns
    {  410, 5863,     342,  9, }, // America/St_Kitts
    {  410, 5880,     351,  9, }, // America/St_Lucia
    {  410,  855,     360, 10, }, // America/St_Thomas
    {  410, 5913,     370, 11, }, // America/St_Vincent
    {  410, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  410, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  410, 1185,     413,  9, }, // Asia/Kathmandu
    {  410,  992,     422,  7, }, // Asia/Kolkata
    {  410, 1211,     429,  5, }, // Asia/Macau
    {  410, 1235,     442,  6, }, // Asia/Yangon
    {  410, 1395,     448,  5, }, // Atlantic/Faroe
    {  410, 6821,     453, 10, }, // Atlantic/St_Helena
    {  410, 2527,     463,  4, }, // Europe/Kyiv
    {  410, 7211,     467,  7, }, // Indian/Reunion
    {  410, 3157,     474,  5, }, // Pacific/Chuuk
    {  410, 3096,     479,  7, }, // Pacific/Pohnpei
    {  411,   14,       0,  6, }, // Africa/Asmara Makonde/Latin/Tanzania
    {  411, 4435,       6,  8, }, // Africa/Sao_Tome
    {  411, 4252,      14,  8, }, // America/Asuncion
    {  411,  237,      22,  8, }, // America/Atikokan
    {  411, 7623,      30, 17, }, // America/Bahia_Banderas
    {  411, 3868,      47,  6, }, // America/Cancun
    {  411, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  411, 5723,      66,  7, }, // America/Curacao
    {  411,  481,      73, 13, }, // America/Indiana/Knox
    {  411, 7961,      86, 16, }, // America/Indiana/Marengo
    {  411, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  411, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  411, 8038,     139, 14, }, // America/Indiana/Vevay
    {  411, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  411, 8086,     171, 16, }, // America/Indiana/Winamac
    {  411, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  411, 5932,     207, 22, }, // America/Lower_Princes
    {  411, 8220,     229,  6, }, // America/Merida
    {  411, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  411, 8329,     255, 20, }, // America/North_Dakota/Center
    {  411, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  411,  393,     298,  4, }, // America/Nuuk
    {  411, 6899,     302, 16, }, // America/Scoresbysund
    {  411, 8508,     318, 14, }, // America/St_Barthelemy
    {  411, 2061,     332, 10, }, // America/St_Johns
    {  411, 5863,     342,  9, }, // America/St_Kitts
    {  411, 5880,     351,  9, }, // America/St_Lucia
    {  411,  855,     360, 10, }, // America/St_Thomas
    {  411, 5913,     370, 11, }, // America/St_Vincent
    {  411, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  411, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  411, 1185,     413,  9, }, // Asia/Kathmandu
    {  411,  992,     422,  7, }, // Asia/Kolkata
    {  411, 1211,     429,  5, }, // Asia/Macau
    {  411, 1235,     442,  6, }, // Asia/Yangon
    {  411, 1395,     448,  5, }, // Atlantic/Faroe
    {  411, 6821,     453, 10, }, // Atlantic/St_Helena
    {  411, 2527,     463,  4, }, // Europe/Kyiv
    {  411, 7211,     467,  7, }, // Indian/Reunion
    {  411, 3157,     474,  5, }, // Pacific/Chuuk
    {  411, 3096,     479,  7, }, // Pacific/Pohnpei
    {  412,   14,       0,  6, }, // Africa/Asmara Malagasy/Latin/Madagascar
    {  412, 4435,       6,  8, }, // Africa/Sao_Tome
    {  412, 4252,      14,  8, }, // America/Asuncion
    {  412,  237,      22,  8, }, // America/Atikokan
    {  412, 7623,      30, 17, }, // America/Bahia_Banderas
    {  412, 3868,      47,  6, }, // America/Cancun
    {  412, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  412, 5723,      66,  7, }, // America/Curacao
    {  412,  481,      73, 13, }, // America/Indiana/Knox
    {  412, 7961,      86, 16, }, // America/Indiana/Marengo
    {  412, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  412, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  412, 8038,     139, 14, }, // America/Indiana/Vevay
    {  412, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  412, 8086,     171, 16, }, // America/Indiana/Winamac
    {  412, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  412, 5932,     207, 22, }, // America/Lower_Princes
    {  412, 8220,     229,  6, }, // America/Merida
    {  412, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  412, 8329,     255, 20, }, // America/North_Dakota/Center
    {  412, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  412,  393,     298,  4, }, // America/Nuuk
    {  412, 6899,     302, 16, }, // America/Scoresbysund
    {  412, 8508,     318, 14, }, // America/St_Barthelemy
    {  412, 2061,     332, 10, }, // America/St_Johns
    {  412, 5863,     342,  9, }, // America/St_Kitts
    {  412, 5880,     351,  9, }, // America/St_Lucia
    {  412,  855,     360, 10, }, // America/St_Thomas
    {  412, 5913,     370, 11, }, // America/St_Vincent
    {  412, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  412, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  412, 1185,     413,  9, }, // Asia/Kathmandu
    {  412,  992,     422,  7, }, // Asia/Kolkata
    {  412, 1211,     429,  5, }, // Asia/Macau
    {  412, 1235,     442,  6, }, // Asia/Yangon
    {  412, 1395,     448,  5, }, // Atlantic/Faroe
    {  412, 6821,     453, 10, }, // Atlantic/St_Helena
    {  412, 2527,     463,  4, }, // Europe/Kyiv
    {  412, 7211,     467,  7, }, // Indian/Reunion
    {  412, 3157,     474,  5, }, // Pacific/Chuuk
    {  412, 3096,     479,  7, }, // Pacific/Pohnpei
    {  413, 6788,  130619, 10, }, // Africa/Abidjan Malayalam/Malayalam/India
    {  413, 6760,  130629,  4, }, // Africa/Accra
    {  413, 5059,  130633, 10, }, // Africa/Addis_Ababa
    {  413, 7420,  130643, 11, }, // Africa/Algiers
    {  413,   14,  130654,  7, }, // Africa/Asmara
    {  413,   44,  130661,  5, }, // Africa/Bamako
    {  413, 5257,  130666,  6, }, // Africa/Bangui
    {  413, 6746,  130672,  6, }, // Africa/Banjul
    {  413, 7435,  130678,  8, }, // Africa/Bissau
    {  413, 4957,  130686, 11, }, // Africa/Blantyre
    {  413, 5287,  130697,  8, }, // Africa/Brazzaville
    {  413, 4922,  130705,  9, }, // Africa/Bujumbura
    {  413, 2320,  130714,  7, }, // Africa/Cairo
    {  413, 4117,  130721, 10, }, // Africa/Casablanca
    {  413, 7449,  130731,  7, }, // Africa/Ceuta
    {  413, 6773,  130738,  8, }, // Africa/Conakry
    {  413, 6840,  130746,  7, }, // Africa/Dakar
    {  413, 5130,  130753, 12, }, // Africa/Dar_es_Salaam
    {  413, 5043,  130765, 10, }, // Africa/Djibouti
    {  413, 5243,  130775,  3, }, // Africa/Douala
    {  413, 5166,  130778,  9, }, // Africa/El_Aaiun
    {  413, 6853,  130787,  7, }, // Africa/Freetown
    {  413, 4906,  130794,  6, }, // Africa/Gaborone
    {  413, 5015,  130800,  5, }, // Africa/Harare
    {  413, 4479,  130805, 14, }, // Africa/Johannesburg
    {  413, 4499,  130819,  3, }, // Africa/Juba
    {  413, 5151,  130822,  6, }, // Africa/Kampala
    {  413, 4524,  130828,  7, }, // Africa/Khartoum
    {  413, 4973,  130835,  6, }, // Africa/Kigali
    {  413, 5306,  130841,  6, }, // Africa/Kinshasa
    {  413, 4744,  130847,  6, }, // Africa/Lagos
    {  413, 5336,  130853, 11, }, // Africa/Libreville
    {  413, 6869,  130864,  3, }, // Africa/Lome
    {  413, 5211,  130867,  6, }, // Africa/Luanda
    {  413, 4939,  130873,  9, }, // Africa/Lubumbashi
    {  413, 5001,  130882,  5, }, // Africa/Lusaka
    {  413, 5322,  130887,  5, }, // Africa/Malabo
    {  413, 4987,  130892, 10, }, // Africa/Maputo
    {  413, 5197,  130902,  5, }, // Africa/Maseru
    {  413, 5182,  130907, 10, }, // Africa/Mbabane
    {  413, 5113,  130917,  8, }, // Africa/Mogadishu
    {  413, 7034,  130925,  8, }, // Africa/Monrovia
    {  413, 3853,  130933,  8, }, // Africa/Nairobi
    {  413, 5271,  130941,  4, }, // Africa/Ndjamena
    {  413, 5354,  130945,  6, }, // Africa/Niamey
    {  413, 6803,  130951, 13, }, // Africa/Nouakchott
    {  413, 6727,  130964,  7, }, // Africa/Ouagadougou
    {  413, 5225,  130971, 13, }, // Africa/Porto-Novo
    {  413, 4435,  130984,  9, }, // Africa/Sao_Tome
    {  413, 2866,  130993,  8, }, // Africa/Tripoli
    {  413, 6554,  131001,  8, }, // Africa/Tunis
    {  413, 4152,  131009, 10, }, // Africa/Windhoek
    {  413,  132,  131019,  5, }, // America/Adak
    {  413, 3346,  131024,  8, }, // America/Anchorage
    {  413, 5607,  131032,  9, }, // America/Anguilla
    {  413, 5624,  131041,  7, }, // America/Antigua
    {  413, 4554,  131048,  7, }, // America/Araguaina
    {  413,  166,  131055, 13, }, // America/Argentina/Buenos_Aires
    {  413,   91,  131068, 14, }, // America/Argentina/Catamarca
    {  413,  270,  131082,  6, }, // America/Argentina/Cordoba
    {  413,  441,  131088,  6, }, // America/Argentina/Jujuy
    {  413, 7462,  131094,  8, }, // America/Argentina/La_Rioja
    {  413,  565,  131102,  6, }, // America/Argentina/Mendoza
    {  413, 7489,  131108, 14, }, // America/Argentina/Rio_Gallegos
    {  413, 7520,  131122,  6, }, // America/Argentina/Salta
    {  413, 7544,  131128,  9, }, // America/Argentina/San_Juan
    {  413, 5580,  131137, 10, }, // America/Argentina/San_Luis
    {  413, 7571,  131147,  9, }, // America/Argentina/Tucuman
    {  413, 7597,  131156,  4, }, // America/Argentina/Ushuaia
    {  413, 5640,  131160,  4, }, // America/Aruba
    {  413, 4252,  131164,  9, }, // America/Asuncion
    {  413,  237,  131173, 11, }, // America/Atikokan
    {  413, 3679,  131184,  4, }, // America/Bahia
    {  413, 7623,  131188, 12, }, // America/Bahia_Banderas
    {  413, 5654,  131200,  8, }, // America/Barbados
    {  413, 7646,  131208,  4, }, // America/Belem
    {  413, 5407,  131212,  6, }, // America/Belize
    {  413, 7660,  131218, 16, }, // America/Blanc-Sablon
    {  413, 7681,  131234, 11, }, // America/Boa_Vista
    {  413, 4361,  131245,  7, }, // America/Bogota
    {  413, 7699,  131252,  6, }, // America/Boise
    {  413, 7713,  131258, 15, }, // America/Cambridge_Bay
    {  413, 7735,  131273, 16, }, // America/Campo_Grande
    {  413, 3868,  131289,  6, }, // America/Cancun
    {  413, 4694,  131295,  8, }, // America/Caracas
    {  413, 4345,  131303,  5, }, // America/Cayenne
    {  413, 5496,  131308,  5, }, // America/Cayman
    {  413, 2260,  131313,  8, }, // America/Chicago
    {  413, 7756,  131321,  7, }, // America/Chihuahua
    {  413, 7774,  131328, 16, }, // America/Ciudad_Juarez
    {  413, 5422,  131344, 13, }, // America/Costa_Rica
    {  413, 7796,  131357, 10, }, // America/Creston
    {  413, 3791,  131367,  6, }, // America/Cuiaba
    {  413, 5723,  131373, 10, }, // America/Curacao
    {  413, 7812,  131383, 15, }, // America/Danmarkshavn
    {  413, 7833,  131398,  6, }, // America/Dawson
    {  413, 7848,  131404, 15, }, // America/Dawson_Creek
    {  413,  805,  131419,  6, }, // America/Denver
    {  413, 3465,  131425, 12, }, // America/Detroit
    {  413, 5739,  131437,  9, }, // America/Dominica
    {  413,  893,  131446,  8, }, // America/Edmonton
    {  413, 7869,  131454, 10, }, // America/Eirunepe
    {  413, 5441,  131464, 11, }, // America/El_Salvador
    {  413, 7886,  131475, 13, }, // America/Fort_Nelson
    {  413, 7906,  131488,  9, }, // America/Fortaleza
    {  413, 7924,  131497, 11, }, // America/Glace_Bay
    {  413, 6881,  131508,  7, }, // America/Goose_Bay
    {  413, 4612,  131515, 14, }, // America/Grand_Turk
    {  413, 5770,  131529,  6, }, // America/Grenada
    {  413, 5786,  131535, 11, }, // America/Guadeloupe
    {  413, 3760,  131546, 11, }, // America/Guatemala
    {  413, 4841,  131557, 10, }, // America/Guayaquil
    {  413, 6932,  131567,  4, }, // America/Guyana
    {  413, 1939,  131571, 10, }, // America/Halifax
    {  413, 2281,  131581,  4, }, // America/Havana
    {  413, 7942,  131585, 11, }, // America/Hermosillo
    {  413,  348,  131596, 14, }, // America/Indiana/Indianapolis
    {  413,  481,  131610, 15, }, // America/Indiana/Knox
    {  413, 7961,  131625, 16, }, // America/Indiana/Marengo
    {  413, 7985,  131641, 23, }, // America/Indiana/Petersburg
    {  413, 8012,  131664, 21, }, // America/Indiana/Tell_City
    {  413, 8038,  131685, 15, }, // America/Indiana/Vevay
    {  413, 8060,  131700, 17, }, // America/Indiana/Vincennes
    {  413, 8086,  131717, 17, }, // America/Indiana/Winamac
    {  413, 8110,  131734,  7, }, // America/Inuvik
    {  413,  660,  131741,  7, }, // America/Iqaluit
    {  413, 2799,  131748,  6, }, // America/Jamaica
    {  413, 5380,  131754,  4, }, // America/Juneau
    {  413,  521,  131758, 13, }, // America/Kentucky/Louisville
    {  413, 8125,  131771, 23, }, // America/Kentucky/Monticello
    {  413, 5704,  131794, 13, }, // America/Kralendijk
    {  413, 4376,  131807,  7, }, // America/La_Paz
    {  413, 7169,  131814,  3, }, // America/Lima
    {  413, 3239,  131817, 14, }, // America/Los_Angeles
    {  413, 5932,  131831, 23, }, // America/Lower_Princes
    {  413, 8153,  131854,  6, }, // America/Maceio
    {  413, 8168,  131860,  6, }, // America/Managua
    {  413, 1908,  131866,  5, }, // America/Manaus
    {  413, 5897,  131871,  6, }, // America/Marigot
    {  413, 5805,  131877, 11, }, // America/Martinique
    {  413, 8184,  131888, 11, }, // America/Matamoros
    {  413, 2917,  131899, 10, }, // America/Mazatlan
    {  413, 8202,  131909,  8, }, // America/Menominee
    {  413, 8220,  131917,  5, }, // America/Merida
    {  413, 8235,  131922, 12, }, // America/Metlakatla
    {  413, 2949,  131934, 17, }, // America/Mexico_City
    {  413, 4391,  131951,  7, }, // America/Miquelon
    {  413, 8254,  131958,  8, }, // America/Moncton
    {  413, 8270,  131966,  8, }, // America/Monterrey
    {  413, 4098,  131974, 13, }, // America/Montevideo
    {  413, 5824,  131987, 12, }, // America/Montserrat
    {  413, 5481,  131999,  4, }, // America/Nassau
    {  413, 2379,  132003, 11, }, // America/New_York
    {  413, 8288,  132014,  3, }, // America/Nome
    {  413, 1850,  132017,  5, }, // America/Noronha
    {  413, 8301,  132022, 23, }, // America/North_Dakota/Beulah
    {  413, 8329,  132045, 24, }, // America/North_Dakota/Center
    {  413, 8357,  132069, 30, }, // America/North_Dakota/New_Salem
    {  413,  393,  132099,  6, }, // America/Nuuk
    {  413, 8388,  132105,  5, }, // America/Ojinaga
    {  413, 2356,  132110,  4, }, // America/Panama
    {  413, 6173,  132114,  9, }, // America/Paramaribo
    {  413, 2973,  132123,  8, }, // America/Phoenix
    {  413, 3945,  132131, 14, }, // America/Port-au-Prince
    {  413, 5954,  132145, 22, }, // America/Port_of_Spain
    {  413, 8404,  132167, 14, }, // America/Porto_Velho
    {  413, 5843,  132181, 16, }, // America/Puerto_Rico
    {  413, 4030,  132197, 12, }, // America/Punta_Arenas
    {  413, 8424,  132209, 16, }, // America/Rankin_Inlet
    {  413, 8445,  132225,  6, }, // America/Recife
    {  413, 1995,  132231,  5, }, // America/Regina
    {  413, 8460,  132236, 11, }, // America/Resolute
    {  413,  695,  132247, 13, }, // America/Rio_Branco
    {  413, 8477,  132260,  8, }, // America/Santarem
    {  413, 2201,  132268, 10, }, // America/Santiago
    {  413, 6111,  132278, 14, }, // America/Santo_Domingo
    {  413, 1878,  132292,  8, }, // America/Sao_Paulo
    {  413, 6899,  132300, 21, }, // America/Scoresbysund
    {  413, 8494,  132321,  9, }, // America/Sitka
    {  413, 8508,  132330, 16, }, // America/St_Barthelemy
    {  413, 2061,  132346, 12, }, // America/St_Johns
    {  413, 5863,  132358, 15, }, // America/St_Kitts
    {  413, 5880,  132373, 12, }, // America/St_Lucia
    {  413,  855,  132385, 12, }, // America/St_Thomas
    {  413, 5913,  132397, 16, }, // America/St_Vincent
    {  413, 8530,  132413, 18, }, // America/Swift_Current
    {  413, 5461,  132431, 10, }, // America/Tegucigalpa
    {  413, 5756,  132441,  4, }, // America/Thule
    {  413,  313,  132445,  7, }, // America/Tijuana
    {  413,  608,  132452,  7, }, // America/Toronto
    {  413, 5688,  132459,  9, }, // America/Tortola
    {  413, 2093,  132468,  8, }, // America/Vancouver
    {  413, 2144,  132476, 12, }, // America/Whitehorse
    {  413,  734,  132488, 10, }, // America/Winnipeg
    {  413, 8552,  132498, 12, }, // America/Yakutat
    {  413, 6001,  132328,  4, }, // Antarctica/Casey
    {  413, 6094,  132510,  9, }, // Antarctica/Davis
    {  413, 6133,  132519, 19, }, // Antarctica/DumontDUrville
    {  413, 8568,  132538,  7, }, // Antarctica/Macquarie
    {  413, 7066,  132545,  6, }, // Antarctica/Mawson
    {  413, 7113,  132551,  8, }, // Antarctica/McMurdo
    {  413, 8589,  132559,  4, }, // Antarctica/Palmer
    {  413, 7226,  132563,  5, }, // Antarctica/Rothera
    {  413, 7295,  132568,  5, }, // Antarctica/Syowa
    {  413, 8607,  132573,  5, }, // Antarctica/Troll
    {  413, 7374,  132578, 10, }, // Antarctica/Vostok
    {  413, 1429,  132588, 11, }, // Arctic/Longyearbyen
    {  413, 5570,  132599,  4, }, // Asia/Aden
    {  413, 5395,  132603,  8, }, // Asia/Almaty
    {  413, 3968,  132611,  7, }, // Asia/Amman
    {  413, 5511,  132618,  6, }, // Asia/Anadyr
    {  413, 5523,  132624,  5, }, // Asia/Aqtau
    {  413, 5368,  132629,  8, }, // Asia/Aqtobe
    {  413,  964,  132637, 11, }, // Asia/Ashgabat
    {  413, 8624,  132648,  7, }, // Asia/Atyrau
    {  413, 3607,  132655,  9, }, // Asia/Baghdad
    {  413, 5534,  132664,  6, }, // Asia/Bahrain
    {  413, 3653,  132670,  6, }, // Asia/Baku
    {  413, 4466,  132676, 10, }, // Asia/Bangkok
    {  413, 3571,  132686,  5, }, // Asia/Barnaul
    {  413, 4086,  132691, 11, }, // Asia/Beirut
    {  413, 3778,  132702,  9, }, // Asia/Bishkek
    {  413, 5989,  132711,  6, }, // Asia/Brunei
    {  413, 4601,  132717,  5, }, // Asia/Chita
    {  413, 4511,  132722,  7, }, // Asia/Colombo
    {  413, 4540,  132729,  8, }, // Asia/Damascus
    {  413, 1093,  132737,  5, }, // Asia/Dhaka
    {  413, 6192,  132742,  4, }, // Asia/Dili
    {  413, 3596,  132746,  6, }, // Asia/Dubai
    {  413, 6159,  132752,  7, }, // Asia/Dushanbe
    {  413, 8636,  132759, 11, }, // Asia/Famagusta
    {  413, 8651,  132770,  3, }, // Asia/Gaza
    {  413, 4795,  132773,  8, }, // Asia/Hebron
    {  413, 1259,  132781, 16, }, // Asia/Ho_Chi_Minh
    {  413, 2704,  132797,  9, }, // Asia/Hong_Kong
    {  413, 4771,  132806,  4, }, // Asia/Hovd
    {  413, 4184,  132810,  7, }, // Asia/Irkutsk
    {  413, 7006,  132817,  9, }, // Asia/Jakarta
    {  413, 6992,  132826,  5, }, // Asia/Jayapura
    {  413, 1290,  132831,  8, }, // Asia/Jerusalem
    {  413, 3560,  132839,  5, }, // Asia/Kabul
    {  413, 4316,  132844,  7, }, // Asia/Kamchatka
    {  413, 4239,  132851,  7, }, // Asia/Karachi
    {  413, 1185,  132858, 10, }, // Asia/Kathmandu
    {  413, 8661,  132868,  6, }, // Asia/Khandyga
    {  413,  992,  132874, 10, }, // Asia/Kolkata
    {  413, 4197,  132884, 14, }, // Asia/Krasnoyarsk
    {  413, 4859,  132898, 13, }, // Asia/Kuala_Lumpur
    {  413, 5976,  132911,  6, }, // Asia/Kuching
    {  413, 5547,  132917,  8, }, // Asia/Kuwait
    {  413, 1211,  132925,  7, }, // Asia/Macau
    {  413, 4017,  132932,  5, }, // Asia/Magadan
    {  413, 1349,  132937,  6, }, // Asia/Makassar
    {  413, 7182,  132943,  4, }, // Asia/Manila
    {  413, 6920,  132947,  8, }, // Asia/Muscat
    {  413, 2554,  132955,  9, }, // Asia/Nicosia
    {  413, 8675,  132964, 14, }, // Asia/Novokuznetsk
    {  413, 4135,  132978, 13, }, // Asia/Novosibirsk
    {  413, 4229,  132991,  6, }, // Asia/Omsk
    {  413, 7145,  132997,  3, }, // Asia/Oral
    {  413, 6961,  133000,  9, }, // Asia/Phnom_Penh
    {  413, 8693,  133009, 11, }, // Asia/Pontianak
    {  413, 4214,  133020, 13, }, // Asia/Pyongyang
    {  413, 5559,  133033,  5, }, // Asia/Qatar
    {  413, 8708,  133038,  9, }, // Asia/Qostanay
    {  413, 4269,  133047,  8, }, // Asia/Qyzylorda
    {  413, 3584,  133055,  6, }, // Asia/Riyadh
    {  413, 4408,  133061,  6, }, // Asia/Sakhalin
    {  413, 7245,  133067, 10, }, // Asia/Samarkand
    {  413, 3279,  133077,  3, }, // Asia/Seoul
    {  413, 1053,  133080,  9, }, // Asia/Shanghai
    {  413, 3300,  133089,  9, }, // Asia/Singapore
    {  413, 4297,  133098, 18, }, // Asia/Srednekolymsk
    {  413, 3263,  133116,  9, }, // Asia/Taipei
    {  413, 4781,  133125, 12, }, // Asia/Tashkent
    {  413, 3915,  133137,  8, }, // Asia/Tbilisi
    {  413, 2772,  133145,  9, }, // Asia/Tehran
    {  413, 1317,  133154,  5, }, // Asia/Thimphu
    {  413, 2821,  133159,  8, }, // Asia/Tokyo
    {  413, 4572,  133167,  7, }, // Asia/Tomsk
    {  413, 1021,  133174, 10, }, // Asia/Ulaanbaatar
    {  413, 1159,  133184,  6, }, // Asia/Urumqi
    {  413, 8722,  133190,  8, }, // Asia/Ust-Nera
    {  413, 6977,  133198,  9, }, // Asia/Vientiane
    {  413, 4710,  133207, 16, }, // Asia/Vladivostok
    {  413, 4828,  133223, 14, }, // Asia/Yakutsk
    {  413, 1235,  133237,  7, }, // Asia/Yangon
    {  413, 3883,  133244, 15, }, // Asia/Yekaterinburg
    {  413, 3747,  133259,  6, }, // Asia/Yerevan
    {  413, 3663,  133265,  6, }, // Atlantic/Azores
    {  413, 5671,  133271,  5, }, // Atlantic/Bermuda
    {  413, 6628,  133276,  7, }, // Atlantic/Canary
    {  413, 3727,  133283, 10, }, // Atlantic/Cape_Verde
    {  413, 1395,  133293,  4, }, // Atlantic/Faroe
    {  413, 8736,  133297,  6, }, // Atlantic/Madeira
    {  413, 2748,  133303, 11, }, // Atlantic/Reykjavik
    {  413, 7272,  133314, 13, }, // Atlantic/South_Georgia
    {  413, 6821,  133327, 12, }, // Atlantic/St_Helena
    {  413, 6644,  133339, 10, }, // Atlantic/Stanley
    {  413, 1670,  133349, 10, }, // Australia/Adelaide
    {  413, 1635,  133359, 12, }, // Australia/Brisbane
    {  413, 1799,  133371, 12, }, // Australia/Broken_Hill
    {  413, 1583,  133383,  6, }, // Australia/Darwin
    {  413, 3637,  133389,  5, }, // Australia/Eucla
    {  413, 1516,  133394,  9, }, // Australia/Hobart
    {  413, 8753,  133403,  8, }, // Australia/Lindeman
    {  413, 1547,  133411,  9, }, // Australia/Lord_Howe
    {  413, 1727,  133420,  5, }, // Australia/Melbourne
    {  413, 1762,  133425,  7, }, // Australia/Perth
    {  413, 1463,  133432,  6, }, // Australia/Sydney
    {  413, 6441,  133438, 13, }, // Europe/Amsterdam
    {  413, 6216,  133451,  6, }, // Europe/Andorra
    {  413, 3620,  133457, 10, }, // Europe/Astrakhan
    {  413, 2300,  133467,  6, }, // Europe/Athens
    {  413, 6336,  133473,  9, }, // Europe/Belgrade
    {  413, 4757,  133482,  8, }, // Europe/Berlin
    {  413, 6488,  133490, 14, }, // Europe/Bratislava
    {  413, 2167,  133504, 10, }, // Europe/Brussels
    {  413, 3928,  133514, 14, }, // Europe/Bucharest
    {  413, 3806,  133528, 12, }, // Europe/Budapest
    {  413, 8772,  133540,  7, }, // Europe/Busingen
    {  413, 2583,  133547,  8, }, // Europe/Chisinau
    {  413, 6289,  133555, 12, }, // Europe/Copenhagen
    {  413, 2338,  133567,  6, }, // Europe/Dublin
    {  413, 6307,  133573, 11, }, // Europe/Gibraltar
    {  413, 8788,  133584,  5, }, // Europe/Guernsey
    {  413, 6612,  133589, 10, }, // Europe/Helsinki
    {  413, 8804,  133599, 12, }, // Europe/Isle_of_Man
    {  413, 1130,  133611, 11, }, // Europe/Istanbul
    {  413, 8823,  133622,  7, }, // Europe/Jersey
    {  413, 3979,  133629, 12, }, // Europe/Kaliningrad
    {  413, 8929,  133641,  4, }, // Europe/Kirov
    {  413, 2527,  133645,  4, }, // Europe/Kyiv
    {  413, 3213,  133649,  8, }, // Europe/Lisbon
    {  413, 6506,  133657, 10, }, // Europe/Ljubljana
    {  413, 2501,  133667,  6, }, // Europe/London
    {  413, 6365,  133673, 10, }, // Europe/Luxembourg
    {  413, 4892,  133683,  8, }, // Europe/Madrid
    {  413, 6397,  133691,  6, }, // Europe/Malta
    {  413, 6582,  133697,  9, }, // Europe/Mariehamn
    {  413, 3693,  133706,  8, }, // Europe/Minsk
    {  413, 6410,  133714,  8, }, // Europe/Monaco
    {  413, 3537,  133722,  6, }, // Europe/Moscow
    {  413, 6458,  133728,  5, }, // Europe/Oslo
    {  413, 4284,  133733,  6, }, // Europe/Paris
    {  413, 6424,  133739, 10, }, // Europe/Podgorica
    {  413, 6275,  133749,  6, }, // Europe/Prague
    {  413, 8837,  131043,  3, }, // Europe/Riga
    {  413, 6324,  133755,  3, }, // Europe/Rome
    {  413, 4331,  133758,  4, }, // Europe/Samara
    {  413, 6470,  133762, 10, }, // Europe/San_Marino
    {  413, 6245,  133772,  6, }, // Europe/Sarajevo
    {  413, 4451,  133778,  9, }, // Europe/Saratov
    {  413, 8849,  133787, 10, }, // Europe/Simferopol
    {  413, 6383,  133797, 10, }, // Europe/Skopje
    {  413, 6599,  133807,  5, }, // Europe/Sofia
    {  413, 6523,  133812, 13, }, // Europe/Stockholm
    {  413, 8867,  133825,  6, }, // Europe/Tallinn
    {  413, 6202,  133831,  6, }, // Europe/Tirane
    {  413, 8882,  133837, 17, }, // Europe/Ulyanovsk
    {  413, 6352,  133854,  6, }, // Europe/Vaduz
    {  413, 6567,  133860, 10, }, // Europe/Vatican
    {  413, 6231,  133870,  6, }, // Europe/Vienna
    {  413, 8899,  133876,  9, }, // Europe/Vilnius
    {  413, 4727,  133885, 11, }, // Europe/Volgograd
    {  413, 3190,  133896,  5, }, // Europe/Warsaw
    {  413, 6261,  133901,  8, }, // Europe/Zagreb
    {  413, 6540,  133909,  8, }, // Europe/Zurich
    {  413, 5078,  133917, 10, }, // Indian/Antananarivo
    {  413, 6947,  133927,  6, }, // Indian/Chagos
    {  413, 6046,  133933, 11, }, // Indian/Christmas
    {  413, 6063,  133944,  8, }, // Indian/Cocos
    {  413, 5029,  133952,  6, }, // Indian/Comoro
    {  413, 6661,  133958,  8, }, // Indian/Kerguelen
    {  413, 7260,  133966,  4, }, // Indian/Mahe
    {  413, 7050,  133970, 10, }, // Indian/Maldives
    {  413, 4069,  133980,  9, }, // Indian/Mauritius
    {  413, 5098,  133989,  7, }, // Indian/Mayotte
    {  413, 7211,  133996,  8, }, // Indian/Reunion
    {  413, 4422,  134004,  4, }, // Pacific/Apia
    {  413,  932,  134008, 12, }, // Pacific/Auckland
    {  413, 3706,  134020, 10, }, // Pacific/Bougainville
    {  413, 3015,  134030,  6, }, // Pacific/Chatham
    {  413, 3157,  134036,  3, }, // Pacific/Chuuk
    {  413, 2237,  134039,  7, }, // Pacific/Easter
    {  413, 7360,  134046,  5, }, // Pacific/Efate
    {  413, 7327,  134051,  9, }, // Pacific/Fakaofo
    {  413, 3902,  134060,  4, }, // Pacific/Fiji
    {  413, 7343,  134064, 10, }, // Pacific/Funafuti
    {  413, 6678,  134074, 11, }, // Pacific/Galapagos
    {  413, 6696,  134085,  8, }, // Pacific/Gambier
    {  413, 3822,  134093, 10, }, // Pacific/Guadalcanal
    {  413, 6018,  134103,  5, }, // Pacific/Guam
    {  413, 2723,  134108,  7, }, // Pacific/Honolulu
    {  413, 3049,  134115,  6, }, // Pacific/Kanton
    {  413, 3998,  134121, 14, }, // Pacific/Kiritimati
    {  413, 7019,  134135,  6, }, // Pacific/Kosrae
    {  413, 2842,  134141, 10, }, // Pacific/Kwajalein
    {  413, 4877,  134151,  5, }, // Pacific/Majuro
    {  413, 4051,  134156, 11, }, // Pacific/Marquesas
    {  413, 8914,  134167,  7, }, // Pacific/Midway
    {  413, 7084,  134174,  4, }, // Pacific/Nauru
    {  413, 7132,  134178,  4, }, // Pacific/Niue
    {  413, 4168,  134182, 10, }, // Pacific/Norfolk
    {  413, 7098,  134192,  5, }, // Pacific/Noumea
    {  413, 3126,  134197,  9, }, // Pacific/Pago_Pago
    {  413, 7155,  134206,  5, }, // Pacific/Palau
    {  413, 7194,  134211, 11, }, // Pacific/Pitcairn
    {  413, 3096,  134222,  5, }, // Pacific/Pohnpei
    {  413, 4807,  134227, 17, }, // Pacific/Port_Moresby
    {  413, 6076,  134244,  8, }, // Pacific/Rarotonga
    {  413, 6031,  134252, 11, }, // Pacific/Saipan
    {  413, 7312,  134263,  6, }, // Pacific/Tahiti
    {  413, 6712,  134269,  4, }, // Pacific/Tarawa
    {  413, 4583,  134273,  9, }, // Pacific/Tongatapu
    {  413, 7392,  134282,  6, }, // Pacific/Wake
    {  413, 7405,  134288,  8, }, // Pacific/Wallis
    {  414,   14,       0,  6, }, // Africa/Asmara Malay/Latin/Malaysia
    {  414, 2320,  134296,  7, }, // Africa/Cairo
    {  414, 4435,       6,  8, }, // Africa/Sao_Tome
    {  414, 4252,      14,  8, }, // America/Asuncion
    {  414,  237,      22,  8, }, // America/Atikokan
    {  414, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  414, 7713,   97518, 15, }, // America/Cambridge_Bay
    {  414, 3868,   33182,  6, }, // America/Cancun
    {  414, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  414, 5723,      66,  7, }, // America/Curacao
    {  414, 7924,   97567, 11, }, // America/Glace_Bay
    {  414, 6881,   97578, 11, }, // America/Goose_Bay
    {  414,  481,      73, 13, }, // America/Indiana/Knox
    {  414, 7961,      86, 16, }, // America/Indiana/Marengo
    {  414, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  414, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  414, 8038,     139, 14, }, // America/Indiana/Vevay
    {  414, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  414, 8086,     171, 16, }, // America/Indiana/Winamac
    {  414, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  414, 5932,     207, 22, }, // America/Lower_Princes
    {  414, 8220,    7008,  6, }, // America/Merida
    {  414, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  414, 8329,     255, 20, }, // America/North_Dakota/Center
    {  414, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  414,  393,     298,  4, }, // America/Nuuk
    {  414, 6899,     302, 16, }, // America/Scoresbysund
    {  414, 8508,  117379, 16, }, // America/St_Barthelemy
    {  414, 2061,     332, 10, }, // America/St_Johns
    {  414, 5863,     342,  9, }, // America/St_Kitts
    {  414, 5880,     351,  9, }, // America/St_Lucia
    {  414,  855,     360, 10, }, // America/St_Thomas
    {  414, 5913,     370, 11, }, // America/St_Vincent
    {  414, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  414, 4540,  134303,  7, }, // Asia/Damascus
    {  414, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  414, 1290,  134310, 14, }, // Asia/Jerusalem
    {  414, 1185,     413,  9, }, // Asia/Kathmandu
    {  414,  992,     422,  7, }, // Asia/Kolkata
    {  414, 1211,    4203,  5, }, // Asia/Macau
    {  414, 8708,     434,  8, }, // Asia/Qostanay
    {  414, 3300,   93044,  9, }, // Asia/Singapore
    {  414, 1235,     442,  6, }, // Asia/Yangon
    {  414, 1395,     448,  5, }, // Atlantic/Faroe
    {  414, 6821,     453, 10, }, // Atlantic/St_Helena
    {  414, 2527,    8030,  4, }, // Europe/Kyiv
    {  414, 7211,     467,  7, }, // Indian/Reunion
    {  414, 3157,     474,  5, }, // Pacific/Chuuk
    {  414, 2723,    4493,  8, }, // Pacific/Honolulu
    {  414, 3049,    4501,  9, }, // Pacific/Kanton
    {  414, 3096,     479,  7, }, // Pacific/Pohnpei
    {  415, 7623,      30, 17, }, // America/Bahia_Banderas Malay/Arabic/Brunei
    {  415, 3868,      47,  6, }, // America/Cancun
    {  415, 8220,     229,  6, }, // America/Merida
    {  415, 8508,     318, 14, }, // America/St_Barthelemy
    {  415, 1211,     429,  5, }, // Asia/Macau
    {  415, 3300,  134324,  8, }, // Asia/Singapore
    {  415, 2527,     463,  4, }, // Europe/Kyiv
    {  416, 7623,      30, 17, }, // America/Bahia_Banderas Malay/Arabic/Malaysia
    {  416, 3868,      47,  6, }, // America/Cancun
    {  416, 8220,     229,  6, }, // America/Merida
    {  416, 8508,     318, 14, }, // America/St_Barthelemy
    {  416, 1211,     429,  5, }, // Asia/Macau
    {  416, 3300,  134324,  8, }, // Asia/Singapore
    {  416, 2527,     463,  4, }, // Europe/Kyiv
    {  420, 7420,  134332,  8, }, // Africa/Algiers Maltese/Latin/Malta
    {  420,   14,       0,  6, }, // Africa/Asmara
    {  420, 4435,       6,  8, }, // Africa/Sao_Tome
    {  420, 4252,      14,  8, }, // America/Asuncion
    {  420,  237,      22,  8, }, // America/Atikokan
    {  420, 7623,      30, 17, }, // America/Bahia_Banderas
    {  420, 3868,      47,  6, }, // America/Cancun
    {  420, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  420, 5723,      66,  7, }, // America/Curacao
    {  420, 3760,  134340, 12, }, // America/Guatemala
    {  420,  481,      73, 13, }, // America/Indiana/Knox
    {  420, 7961,      86, 16, }, // America/Indiana/Marengo
    {  420, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  420, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  420, 8038,     139, 14, }, // America/Indiana/Vevay
    {  420, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  420, 8086,     171, 16, }, // America/Indiana/Winamac
    {  420, 2799,  134352, 10, }, // America/Jamaica
    {  420, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  420, 5932,     207, 22, }, // America/Lower_Princes
    {  420, 8220,     229,  6, }, // America/Merida
    {  420, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  420, 8329,     255, 20, }, // America/North_Dakota/Center
    {  420, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  420,  393,     298,  4, }, // America/Nuuk
    {  420, 6899,     302, 16, }, // America/Scoresbysund
    {  420, 8508,     318, 14, }, // America/St_Barthelemy
    {  420, 2061,     332, 10, }, // America/St_Johns
    {  420, 5863,     342,  9, }, // America/St_Kitts
    {  420, 5880,     351,  9, }, // America/St_Lucia
    {  420,  855,     360, 10, }, // America/St_Thomas
    {  420, 5913,     370, 11, }, // America/St_Vincent
    {  420, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  420, 4086,    7507,  6, }, // Asia/Beirut
    {  420, 4540,    4083,  7, }, // Asia/Damascus
    {  420, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  420, 1290,  134362, 10, }, // Asia/Jerusalem
    {  420, 1185,     413,  9, }, // Asia/Kathmandu
    {  420,  992,     422,  7, }, // Asia/Kolkata
    {  420, 5547,  134372, 18, }, // Asia/Kuwait
    {  420, 1211,    4203,  5, }, // Asia/Macau
    {  420, 2554,   57151,  8, }, // Asia/Nicosia
    {  420, 1235,   58486,  7, }, // Asia/Yangon
    {  420, 1395,     448,  5, }, // Atlantic/Faroe
    {  420, 7272,  134390, 24, }, // Atlantic/South_Georgia
    {  420, 6821,     453, 10, }, // Atlantic/St_Helena
    {  420, 2300,  134414,  5, }, // Europe/Athens
    {  420, 6336,   26883,  7, }, // Europe/Belgrade
    {  420, 2167,  134419,  8, }, // Europe/Brussels
    {  420, 6307,  134427,  7, }, // Europe/Gibraltar
    {  420, 2527,     463,  4, }, // Europe/Kyiv
    {  420, 3213,  134434,  7, }, // Europe/Lisbon
    {  420, 2501,   95241,  6, }, // Europe/London
    {  420, 6365,  134441, 14, }, // Europe/Luxembourg
    {  420, 6397,  134455,  8, }, // Europe/Malta
    {  420, 3537,   57353,  5, }, // Europe/Moscow
    {  420, 4284,  134463,  6, }, // Europe/Paris
    {  420, 6275,   21067,  5, }, // Europe/Prague
    {  420, 6324,  134469,  4, }, // Europe/Rome
    {  420, 6599,   33629,  6, }, // Europe/Sofia
    {  420, 6523,  134473,  9, }, // Europe/Stockholm
    {  420, 6202,   21095,  6, }, // Europe/Tirane
    {  420, 6567,  134482, 19, }, // Europe/Vatican
    {  420, 6231,  134501,  6, }, // Europe/Vienna
    {  420, 3190,  134507,  8, }, // Europe/Warsaw
    {  420, 7050,  134515, 10, }, // Indian/Maldives
    {  420, 7211,     467,  7, }, // Indian/Reunion
    {  420, 3157,     474,  5, }, // Pacific/Chuuk
    {  420, 2723,    4493,  8, }, // Pacific/Honolulu
    {  420, 3049,    4501,  9, }, // Pacific/Kanton
    {  420, 3096,     479,  7, }, // Pacific/Pohnpei
    {  421,   14,       0,  6, }, // Africa/Asmara Manipuri/Bangla/India
    {  421, 4435,       6,  8, }, // Africa/Sao_Tome
    {  421, 4252,      14,  8, }, // America/Asuncion
    {  421,  237,      22,  8, }, // America/Atikokan
    {  421, 7623,      30, 17, }, // America/Bahia_Banderas
    {  421, 3868,      47,  6, }, // America/Cancun
    {  421, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  421, 5723,      66,  7, }, // America/Curacao
    {  421,  481,      73, 13, }, // America/Indiana/Knox
    {  421, 7961,      86, 16, }, // America/Indiana/Marengo
    {  421, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  421, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  421, 8038,     139, 14, }, // America/Indiana/Vevay
    {  421, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  421, 8086,     171, 16, }, // America/Indiana/Winamac
    {  421, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  421, 5932,     207, 22, }, // America/Lower_Princes
    {  421, 8220,     229,  6, }, // America/Merida
    {  421, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  421, 8329,     255, 20, }, // America/North_Dakota/Center
    {  421, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  421,  393,     298,  4, }, // America/Nuuk
    {  421, 6899,     302, 16, }, // America/Scoresbysund
    {  421, 8508,     318, 14, }, // America/St_Barthelemy
    {  421, 2061,     332, 10, }, // America/St_Johns
    {  421, 5863,     342,  9, }, // America/St_Kitts
    {  421, 5880,     351,  9, }, // America/St_Lucia
    {  421,  855,     360, 10, }, // America/St_Thomas
    {  421, 5913,     370, 11, }, // America/St_Vincent
    {  421, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  421, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  421, 1185,     413,  9, }, // Asia/Kathmandu
    {  421,  992,     422,  7, }, // Asia/Kolkata
    {  421, 1211,     429,  5, }, // Asia/Macau
    {  421, 1235,     442,  6, }, // Asia/Yangon
    {  421, 1395,     448,  5, }, // Atlantic/Faroe
    {  421, 6821,     453, 10, }, // Atlantic/St_Helena
    {  421, 2527,     463,  4, }, // Europe/Kyiv
    {  421, 7211,     467,  7, }, // Indian/Reunion
    {  421, 3157,     474,  5, }, // Pacific/Chuuk
    {  421, 3096,     479,  7, }, // Pacific/Pohnpei
    {  423,   14,       0,  6, }, // Africa/Asmara Manx/Latin/Isle of Man
    {  423, 4435,       6,  8, }, // Africa/Sao_Tome
    {  423, 4252,      14,  8, }, // America/Asuncion
    {  423,  237,      22,  8, }, // America/Atikokan
    {  423, 7623,      30, 17, }, // America/Bahia_Banderas
    {  423, 3868,      47,  6, }, // America/Cancun
    {  423, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  423, 5723,      66,  7, }, // America/Curacao
    {  423,  481,      73, 13, }, // America/Indiana/Knox
    {  423, 7961,      86, 16, }, // America/Indiana/Marengo
    {  423, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  423, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  423, 8038,     139, 14, }, // America/Indiana/Vevay
    {  423, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  423, 8086,     171, 16, }, // America/Indiana/Winamac
    {  423, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  423, 5932,     207, 22, }, // America/Lower_Princes
    {  423, 8220,     229,  6, }, // America/Merida
    {  423, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  423, 8329,     255, 20, }, // America/North_Dakota/Center
    {  423, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  423,  393,     298,  4, }, // America/Nuuk
    {  423, 6899,     302, 16, }, // America/Scoresbysund
    {  423, 8508,     318, 14, }, // America/St_Barthelemy
    {  423, 2061,     332, 10, }, // America/St_Johns
    {  423, 5863,     342,  9, }, // America/St_Kitts
    {  423, 5880,     351,  9, }, // America/St_Lucia
    {  423,  855,     360, 10, }, // America/St_Thomas
    {  423, 5913,     370, 11, }, // America/St_Vincent
    {  423, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  423, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  423, 1185,     413,  9, }, // Asia/Kathmandu
    {  423,  992,     422,  7, }, // Asia/Kolkata
    {  423, 1211,     429,  5, }, // Asia/Macau
    {  423, 1235,     442,  6, }, // Asia/Yangon
    {  423, 1395,     448,  5, }, // Atlantic/Faroe
    {  423, 6821,     453, 10, }, // Atlantic/St_Helena
    {  423, 2527,     463,  4, }, // Europe/Kyiv
    {  423, 7211,     467,  7, }, // Indian/Reunion
    {  423, 3157,     474,  5, }, // Pacific/Chuuk
    {  423, 3096,     479,  7, }, // Pacific/Pohnpei
    {  424,   14,       0,  6, }, // Africa/Asmara Maori/Latin/New Zealand
    {  424, 2320,    4572,  5, }, // Africa/Cairo
    {  424, 3853,  134525,  8, }, // Africa/Nairobi
    {  424, 4435,       6,  8, }, // Africa/Sao_Tome
    {  424, 2866,  134533,  8, }, // Africa/Tripoli
    {  424, 6554,  134541,  6, }, // Africa/Tunis
    {  424, 5624,  134547, 12, }, // America/Antigua
    {  424, 4252,      14,  8, }, // America/Asuncion
    {  424,  237,      22,  8, }, // America/Atikokan
    {  424, 7623,      30, 17, }, // America/Bahia_Banderas
    {  424, 5654,  134559,  8, }, // America/Barbados
    {  424, 5407,  134567,  6, }, // America/Belize
    {  424, 7713,  134573, 12, }, // America/Cambridge_Bay
    {  424, 3868,      47,  6, }, // America/Cancun
    {  424, 5496,  134585,  6, }, // America/Cayman
    {  424, 2260,  134591,  6, }, // America/Chicago
    {  424, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  424, 5422,  134597,  9, }, // America/Costa_Rica
    {  424, 5723,      66,  7, }, // America/Curacao
    {  424, 5739,  134606,  8, }, // America/Dominica
    {  424, 5441,  134614,  8, }, // America/El_Salvador
    {  424, 6881,  134622,  9, }, // America/Goose_Bay
    {  424, 4612,  134631, 10, }, // America/Grand_Turk
    {  424, 5770,  134641,  8, }, // America/Grenada
    {  424, 3760,  134649,  9, }, // America/Guatemala
    {  424, 2281,  134658,  7, }, // America/Havana
    {  424,  481,      73, 13, }, // America/Indiana/Knox
    {  424, 7961,      86, 16, }, // America/Indiana/Marengo
    {  424, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  424, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  424, 8038,     139, 14, }, // America/Indiana/Vevay
    {  424, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  424, 8086,     171, 16, }, // America/Indiana/Winamac
    {  424, 2799,  134665,  7, }, // America/Jamaica
    {  424, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  424, 3239,  134672, 11, }, // America/Los_Angeles
    {  424, 5932,     207, 22, }, // America/Lower_Princes
    {  424, 5805,  134683,  8, }, // America/Martinique
    {  424, 8220,     229,  6, }, // America/Merida
    {  424, 2949,  134691, 15, }, // America/Mexico_City
    {  424, 2379,  134706, 12, }, // America/New_York
    {  424, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  424, 8329,     255, 20, }, // America/North_Dakota/Center
    {  424, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  424,  393,     298,  4, }, // America/Nuuk
    {  424, 5843,  134718,  9, }, // America/Puerto_Rico
    {  424, 6899,     302, 16, }, // America/Scoresbysund
    {  424, 8508,     318, 14, }, // America/St_Barthelemy
    {  424, 2061,  134727,  9, }, // America/St_Johns
    {  424, 5863,     342,  9, }, // America/St_Kitts
    {  424, 5880,  134736, 11, }, // America/St_Lucia
    {  424,  855,  134747, 11, }, // America/St_Thomas
    {  424, 5913,  134758, 11, }, // America/St_Vincent
    {  424,  608,  134769,  6, }, // America/Toronto
    {  424, 2093,  134775, 16, }, // America/Vancouver
    {  424, 6094,  134791,  6, }, // Antarctica/Davis
    {  424, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  424, 8568,  134797,  7, }, // Antarctica/Macquarie
    {  424, 3607,  134804,  8, }, // Asia/Baghdad
    {  424, 5534,  134812,  7, }, // Asia/Bahrain
    {  424, 4466,  134819,  9, }, // Asia/Bangkok
    {  424, 5989,  134828,  7, }, // Asia/Brunei
    {  424, 3596,  134835,  5, }, // Asia/Dubai
    {  424, 8651,  134840,  4, }, // Asia/Gaza
    {  424, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  424, 2704,  134844,  8, }, // Asia/Hong_Kong
    {  424, 7006,  134852,  7, }, // Asia/Jakarta
    {  424, 1290,  134859, 10, }, // Asia/Jerusalem
    {  424, 1185,  134869,  8, }, // Asia/Kathmandu
    {  424,  992,     422,  7, }, // Asia/Kolkata
    {  424, 4859,  134877, 10, }, // Asia/Kuala_Lumpur
    {  424, 5547,  134887,  7, }, // Asia/Kuwait
    {  424, 1211,   33405,  5, }, // Asia/Macau
    {  424, 7182,  134894,  6, }, // Asia/Manila
    {  424, 6961,  134900, 11, }, // Asia/Phnom_Penh
    {  424, 5559,  134911,  4, }, // Asia/Qatar
    {  424, 3584,  134915,  5, }, // Asia/Riyadh
    {  424, 3279,  134920,  5, }, // Asia/Seoul
    {  424, 1053,  134925,  8, }, // Asia/Shanghai
    {  424, 3300,  134933,  8, }, // Asia/Singapore
    {  424, 2772,  134941,  6, }, // Asia/Tehran
    {  424, 2821,  134947,  5, }, // Asia/Tokyo
    {  424, 1235,     442,  6, }, // Asia/Yangon
    {  424, 5671,  134952,  6, }, // Atlantic/Bermuda
    {  424, 3727,  134958, 17, }, // Atlantic/Cape_Verde
    {  424, 1395,     448,  5, }, // Atlantic/Faroe
    {  424, 7272,  134975, 17, }, // Atlantic/South_Georgia
    {  424, 6821,  134992, 11, }, // Atlantic/St_Helena
    {  424, 1670,  135003,  8, }, // Australia/Adelaide
    {  424, 1635,  135011,  8, }, // Australia/Brisbane
    {  424, 1583,  135019,  6, }, // Australia/Darwin
    {  424, 1516,  135025,  9, }, // Australia/Hobart
    {  424, 1727,  135034,  9, }, // Australia/Melbourne
    {  424, 1762,  135043,  5, }, // Australia/Perth
    {  424, 1463,  135048,  9, }, // Australia/Sydney
    {  424, 6441,  135057, 11, }, // Europe/Amsterdam
    {  424, 6216,  135068,  7, }, // Europe/Andorra
    {  424, 2300,  135075,  5, }, // Europe/Athens
    {  424, 4757,  135080,  7, }, // Europe/Berlin
    {  424, 2167,  135087,  6, }, // Europe/Brussels
    {  424, 3806,  135093,  8, }, // Europe/Budapest
    {  424, 6289,  135101, 11, }, // Europe/Copenhagen
    {  424, 2338,  135112,  8, }, // Europe/Dublin
    {  424, 6612,  135120,  6, }, // Europe/Helsinki
    {  424, 8804,  135126, 15, }, // Europe/Isle_of_Man
    {  424, 1130,  135141,  7, }, // Europe/Istanbul
    {  424, 2527,     463,  4, }, // Europe/Kyiv
    {  424, 3213,  135148,  6, }, // Europe/Lisbon
    {  424, 2501,  135154,  6, }, // Europe/London
    {  424, 6365,  135160,  7, }, // Europe/Luxembourg
    {  424, 4892,  135167,  6, }, // Europe/Madrid
    {  424, 6397,  135173,  6, }, // Europe/Malta
    {  424, 6410,  135179,  6, }, // Europe/Monaco
    {  424, 3537,  135185,  7, }, // Europe/Moscow
    {  424, 6458,  135192,  5, }, // Europe/Oslo
    {  424, 4284,  124141,  4, }, // Europe/Paris
    {  424, 6275,  135197,  6, }, // Europe/Prague
    {  424, 6324,  135203,  4, }, // Europe/Rome
    {  424, 6470,  135207, 11, }, // Europe/San_Marino
    {  424, 6523,  135218,  7, }, // Europe/Stockholm
    {  424, 6567,  135225, 14, }, // Europe/Vatican
    {  424, 6231,  135239,  6, }, // Europe/Vienna
    {  424, 6540,  135245,  6, }, // Europe/Zurich
    {  424, 6046,  135251, 10, }, // Indian/Christmas
    {  424, 6063,  135261,  6, }, // Indian/Cocos
    {  424, 5029,   22853,  6, }, // Indian/Comoro
    {  424, 7050,  135267,  8, }, // Indian/Maldives
    {  424, 4069,  135275,  6, }, // Indian/Mauritius
    {  424, 7211,     467,  7, }, // Indian/Reunion
    {  424, 4422,  135281,  4, }, // Pacific/Apia
    {  424,  932,  135285, 15, }, // Pacific/Auckland
    {  424, 3015,  135300,  6, }, // Pacific/Chatham
    {  424, 3157,     474,  5, }, // Pacific/Chuuk
    {  424, 3902,  135306,  5, }, // Pacific/Fiji
    {  424, 6018,  135311,  5, }, // Pacific/Guam
    {  424, 4168,  135316,  6, }, // Pacific/Norfolk
    {  424, 7098,  135322,  5, }, // Pacific/Noumea
    {  424, 3126,  135327, 11, }, // Pacific/Pago_Pago
    {  424, 7155,  135338,  5, }, // Pacific/Palau
    {  424, 3096,     479,  7, }, // Pacific/Pohnpei
    {  424, 4807,  135343, 12, }, // Pacific/Port_Moresby
    {  424, 7405,  135355,  6, }, // Pacific/Wallis
    {  425,   14,       0,  6, }, // Africa/Asmara Mapuche/Latin/Chile
    {  425, 4435,       6,  8, }, // Africa/Sao_Tome
    {  425, 4252,      14,  8, }, // America/Asuncion
    {  425,  237,      22,  8, }, // America/Atikokan
    {  425, 7623,      30, 17, }, // America/Bahia_Banderas
    {  425, 3868,      47,  6, }, // America/Cancun
    {  425, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  425, 5723,      66,  7, }, // America/Curacao
    {  425,  481,      73, 13, }, // America/Indiana/Knox
    {  425, 7961,      86, 16, }, // America/Indiana/Marengo
    {  425, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  425, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  425, 8038,     139, 14, }, // America/Indiana/Vevay
    {  425, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  425, 8086,     171, 16, }, // America/Indiana/Winamac
    {  425, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  425, 5932,     207, 22, }, // America/Lower_Princes
    {  425, 8220,     229,  6, }, // America/Merida
    {  425, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  425, 8329,     255, 20, }, // America/North_Dakota/Center
    {  425, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  425,  393,     298,  4, }, // America/Nuuk
    {  425, 6899,     302, 16, }, // America/Scoresbysund
    {  425, 8508,     318, 14, }, // America/St_Barthelemy
    {  425, 2061,     332, 10, }, // America/St_Johns
    {  425, 5863,     342,  9, }, // America/St_Kitts
    {  425, 5880,     351,  9, }, // America/St_Lucia
    {  425,  855,     360, 10, }, // America/St_Thomas
    {  425, 5913,     370, 11, }, // America/St_Vincent
    {  425, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  425, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  425, 1185,     413,  9, }, // Asia/Kathmandu
    {  425,  992,     422,  7, }, // Asia/Kolkata
    {  425, 1211,     429,  5, }, // Asia/Macau
    {  425, 1235,     442,  6, }, // Asia/Yangon
    {  425, 1395,     448,  5, }, // Atlantic/Faroe
    {  425, 6821,     453, 10, }, // Atlantic/St_Helena
    {  425, 2527,     463,  4, }, // Europe/Kyiv
    {  425, 7211,     467,  7, }, // Indian/Reunion
    {  425, 3157,     474,  5, }, // Pacific/Chuuk
    {  425, 3096,     479,  7, }, // Pacific/Pohnpei
    {  426, 6788,  135361,  8, }, // Africa/Abidjan Marathi/Devanagari/India
    {  426, 6760,  135369,  7, }, // Africa/Accra
    {  426, 5059,  135376, 10, }, // Africa/Addis_Ababa
    {  426, 7420,  135386,  9, }, // Africa/Algiers
    {  426,   14,  135395,  7, }, // Africa/Asmara
    {  426,   44,  135402,  5, }, // Africa/Bamako
    {  426, 5257,  135407,  6, }, // Africa/Bangui
    {  426, 6746,  128850,  5, }, // Africa/Banjul
    {  426, 7435,  113298,  5, }, // Africa/Bissau
    {  426, 4957,  135413,  8, }, // Africa/Blantyre
    {  426, 5287,  135421, 12, }, // Africa/Brazzaville
    {  426, 4922,  135433,  9, }, // Africa/Bujumbura
    {  426, 2320,  135442,  4, }, // Africa/Cairo
    {  426, 4117,  135446, 12, }, // Africa/Casablanca
    {  426, 7449,   88211,  5, }, // Africa/Ceuta
    {  426, 6773,  135458,  8, }, // Africa/Conakry
    {  426, 6840,  135466,  3, }, // Africa/Dakar
    {  426, 5130,  135469, 10, }, // Africa/Dar_es_Salaam
    {  426, 5043,   29972,  6, }, // Africa/Djibouti
    {  426, 5243,  135479,  6, }, // Africa/Douala
    {  426, 5166,  135485,  6, }, // Africa/El_Aaiun
    {  426, 6853,   29990,  8, }, // Africa/Freetown
    {  426, 4906,   88267,  7, }, // Africa/Gaborone
    {  426, 5015,   88274,  5, }, // Africa/Harare
    {  426, 4479,  113369, 11, }, // Africa/Johannesburg
    {  426, 4499,   30022,  4, }, // Africa/Juba
    {  426, 5151,   88289,  6, }, // Africa/Kampala
    {  426, 4524,  135491,  7, }, // Africa/Khartoum
    {  426, 4973,  135498,  6, }, // Africa/Kigali
    {  426, 5306,   30047,  8, }, // Africa/Kinshasa
    {  426, 4744,   88308,  5, }, // Africa/Lagos
    {  426, 5336,  135504,  9, }, // Africa/Libreville
    {  426, 6869,   88323,  3, }, // Africa/Lome
    {  426, 5211,   88326,  6, }, // Africa/Luanda
    {  426, 4939,  135513,  8, }, // Africa/Lubumbashi
    {  426, 5001,   30089,  6, }, // Africa/Lusaka
    {  426, 5322,   88341,  5, }, // Africa/Malabo
    {  426, 4987,   88346,  6, }, // Africa/Maputo
    {  426, 5197,  135521,  5, }, // Africa/Maseru
    {  426, 5182,  135526,  8, }, // Africa/Mbabane
    {  426, 5113,   88359,  8, }, // Africa/Mogadishu
    {  426, 7034,  135534, 11, }, // Africa/Monrovia
    {  426, 3853,   88376,  6, }, // Africa/Nairobi
    {  426, 5271,  135545,  8, }, // Africa/Ndjamena
    {  426, 5354,  128962,  6, }, // Africa/Niamey
    {  426, 6803,  135553,  8, }, // Africa/Nouakchott
    {  426, 6727,  135561,  7, }, // Africa/Ouagadougou
    {  426, 5225,  135568, 13, }, // Africa/Porto-Novo
    {  426, 4435,  135581,  8, }, // Africa/Sao_Tome
    {  426, 2866,   88430,  8, }, // Africa/Tripoli
    {  426, 6554,  135589,  6, }, // Africa/Tunis
    {  426, 4152,  135595,  8, }, // Africa/Windhoek
    {  426,  132,  135603,  4, }, // America/Adak
    {  426, 3346,  135607,  6, }, // America/Anchorage
    {  426, 5607,  135613,  8, }, // America/Anguilla
    {  426, 5624,  135621,  8, }, // America/Antigua
    {  426, 4554,  135629,  8, }, // America/Araguaina
    {  426,  166,  135637, 13, }, // America/Argentina/Buenos_Aires
    {  426,   91,  135650, 10, }, // America/Argentina/Catamarca
    {  426,  270,  135660,  8, }, // America/Argentina/Cordoba
    {  426,  441,  129014,  5, }, // America/Argentina/Jujuy
    {  426, 7462,  113505,  9, }, // America/Argentina/La_Rioja
    {  426,  565,  135668,  7, }, // America/Argentina/Mendoza
    {  426, 7489,  135675, 12, }, // America/Argentina/Rio_Gallegos
    {  426, 7520,  135687,  6, }, // America/Argentina/Salta
    {  426, 7544,  113533,  8, }, // America/Argentina/San_Juan
    {  426, 5580,  135693, 10, }, // America/Argentina/San_Luis
    {  426, 7571,   30328,  7, }, // America/Argentina/Tucuman
    {  426, 7597,  135703,  8, }, // America/Argentina/Ushuaia
    {  426, 5640,  113548,  5, }, // America/Aruba
    {  426, 4252,  135711, 10, }, // America/Asuncion
    {  426,  237,  135721,  8, }, // America/Atikokan
    {  426, 3679,   88574,  5, }, // America/Bahia
    {  426, 7623,  135729, 15, }, // America/Bahia_Banderas
    {  426, 5654,  135744,  9, }, // America/Barbados
    {  426, 7646,   30394,  5, }, // America/Belem
    {  426, 5407,  135753,  6, }, // America/Belize
    {  426, 7660,  135759, 13, }, // America/Blanc-Sablon
    {  426, 7681,  135772, 12, }, // America/Boa_Vista
    {  426, 4361,   88631,  6, }, // America/Bogota
    {  426, 7699,  135784,  5, }, // America/Boise
    {  426, 7713,  135789, 15, }, // America/Cambridge_Bay
    {  426, 7735,  135804, 13, }, // America/Campo_Grande
    {  426, 3868,  113637,  6, }, // America/Cancun
    {  426, 4694,  135817,  6, }, // America/Caracas
    {  426, 4345,  135823,  6, }, // America/Cayenne
    {  426, 5496,  135829,  4, }, // America/Cayman
    {  426, 2260,   88682,  6, }, // America/Chicago
    {  426, 7756,   88688,  8, }, // America/Chihuahua
    {  426, 7774,  135833, 13, }, // America/Ciudad_Juarez
    {  426, 5422,   88712, 11, }, // America/Costa_Rica
    {  426, 7796,   30510,  8, }, // America/Creston
    {  426, 3791,  129155,  6, }, // America/Cuiaba
    {  426, 5723,  135846,  9, }, // America/Curacao
    {  426, 7812,  113691, 12, }, // America/Danmarkshavn
    {  426, 7833,   88748,  4, }, // America/Dawson
    {  426, 7848,   88752, 10, }, // America/Dawson_Creek
    {  426,  805,  135855,  8, }, // America/Denver
    {  426, 3465,  135863,  8, }, // America/Detroit
    {  426, 5739,   88775,  8, }, // America/Dominica
    {  426,  893,  135871,  7, }, // America/Edmonton
    {  426, 7869,  135878,  8, }, // America/Eirunepe
    {  426, 5441,  113747, 12, }, // America/El_Salvador
    {  426, 7886,  113759, 12, }, // America/Fort_Nelson
    {  426, 7906,  113771, 10, }, // America/Fortaleza
    {  426, 7924,  135886, 12, }, // America/Glace_Bay
    {  426, 6881,  135898, 10, }, // America/Goose_Bay
    {  426, 4612,  135908, 10, }, // America/Grand_Turk
    {  426, 5770,  135918,  8, }, // America/Grenada
    {  426, 5786,  135926, 11, }, // America/Guadeloupe
    {  426, 3760,   30674, 10, }, // America/Guatemala
    {  426, 4841,  135937,  8, }, // America/Guayaquil
    {  426, 6932,   88874,  5, }, // America/Guyana
    {  426, 1939,  135945,  9, }, // America/Halifax
    {  426, 2281,   88889,  5, }, // America/Havana
    {  426, 7942,  113824,  9, }, // America/Hermosillo
    {  426,  348,   88905, 13, }, // America/Indiana/Indianapolis
    {  426,  481,  113846, 15, }, // America/Indiana/Knox
    {  426, 7961,   88933, 17, }, // America/Indiana/Marengo
    {  426, 7985,   88950, 20, }, // America/Indiana/Petersburg
    {  426, 8012,   88970, 18, }, // America/Indiana/Tell_City
    {  426, 8038,  135954, 14, }, // America/Indiana/Vevay
    {  426, 8060,  113895, 17, }, // America/Indiana/Vincennes
    {  426, 8086,  135968, 16, }, // America/Indiana/Winamac
    {  426, 8110,  113929,  6, }, // America/Inuvik
    {  426,  660,  135984,  7, }, // America/Iqaluit
    {  426, 2799,   89044,  5, }, // America/Jamaica
    {  426, 5380,  135991,  6, }, // America/Juneau
    {  426,  521,  135997, 11, }, // America/Kentucky/Louisville
    {  426, 8125,  136008, 18, }, // America/Kentucky/Monticello
    {  426, 5704,  136026, 12, }, // America/Kralendijk
    {  426, 4376,  113984,  6, }, // America/La_Paz
    {  426, 7169,   30932,  4, }, // America/Lima
    {  426, 3239,  136038, 11, }, // America/Los_Angeles
    {  426, 5932,  136049, 22, }, // America/Lower_Princes
    {  426, 8153,  136071,  6, }, // America/Maceio
    {  426, 8168,   89138,  7, }, // America/Managua
    {  426, 1908,   89145,  4, }, // America/Manaus
    {  426, 5897,  136077,  7, }, // America/Marigot
    {  426, 5805,   89156,  9, }, // America/Martinique
    {  426, 8184,   89165,  9, }, // America/Matamoros
    {  426, 2917,  136084,  8, }, // America/Mazatlan
    {  426, 8202,   89183,  8, }, // America/Menominee
    {  426, 8220,   31026,  6, }, // America/Merida
    {  426, 8235,  136092,  9, }, // America/Metlakatla
    {  426, 2949,   89203, 13, }, // America/Mexico_City
    {  426, 4391,  114080,  9, }, // America/Miquelon
    {  426, 8254,  136101,  6, }, // America/Moncton
    {  426, 8270,  136107,  8, }, // America/Monterrey
    {  426, 4098,  136115, 14, }, // America/Montevideo
    {  426, 5824,  136129, 11, }, // America/Montserrat
    {  426, 5481,  114127,  4, }, // America/Nassau
    {  426, 2379,   89263,  9, }, // America/New_York
    {  426, 8288,   89185,  3, }, // America/Nome
    {  426, 1850,   89272,  8, }, // America/Noronha
    {  426, 8301,  136140, 20, }, // America/North_Dakota/Beulah
    {  426, 8329,  136160, 17, }, // America/North_Dakota/Center
    {  426, 8357,  136177, 23, }, // America/North_Dakota/New_Salem
    {  426,  393,   31183,  3, }, // America/Nuuk
    {  426, 8388,  114210,  7, }, // America/Ojinaga
    {  426, 2356,   89355,  5, }, // America/Panama
    {  426, 6173,  136200,  8, }, // America/Paramaribo
    {  426, 2973,  136208,  7, }, // America/Phoenix
    {  426, 3945,  136215, 14, }, // America/Port-au-Prince
    {  426, 5954,  114248, 14, }, // America/Port_of_Spain
    {  426, 8404,   89407, 13, }, // America/Porto_Velho
    {  426, 5843,  136229, 14, }, // America/Puerto_Rico
    {  426, 4030,  136243, 12, }, // America/Punta_Arenas
    {  426, 8424,  136255, 12, }, // America/Rankin_Inlet
    {  426, 8445,   31293,  6, }, // America/Recife
    {  426, 1995,   31299,  6, }, // America/Regina
    {  426, 8460,  136267,  9, }, // America/Resolute
    {  426,  695,   89471, 12, }, // America/Rio_Branco
    {  426, 8477,  136276,  7, }, // America/Santarem
    {  426, 2201,  136283,  9, }, // America/Santiago
    {  426, 6111,  136292, 14, }, // America/Santo_Domingo
    {  426, 1878,  114345,  9, }, // America/Sao_Paulo
    {  426, 6899,  114354, 13, }, // America/Scoresbysund
    {  426, 8494,  136306,  5, }, // America/Sitka
    {  426, 8508,   89541, 15, }, // America/St_Barthelemy
    {  426, 2061,  114367, 10, }, // America/St_Johns
    {  426, 5863,   89565, 10, }, // America/St_Kitts
    {  426, 5880,  136311, 11, }, // America/St_Lucia
    {  426,  855,   89586,  9, }, // America/St_Thomas
    {  426, 5913,  114388, 13, }, // America/St_Vincent
    {  426, 8530,  114401, 12, }, // America/Swift_Current
    {  426, 5461,  136322, 12, }, // America/Tegucigalpa
    {  426, 5756,  114424,  4, }, // America/Thule
    {  426,  313,   89637,  7, }, // America/Tijuana
    {  426,  608,  136334,  7, }, // America/Toronto
    {  426, 5688,   89650,  8, }, // America/Tortola
    {  426, 2093,  136341, 11, }, // America/Vancouver
    {  426, 2144,  136352, 11, }, // America/Whitehorse
    {  426,  734,   89676,  7, }, // America/Winnipeg
    {  426, 8552,  136363,  6, }, // America/Yakutat
    {  426, 6001,  136369,  4, }, // Antarctica/Casey
    {  426, 6094,  136373,  7, }, // Antarctica/Davis
    {  426, 6133,  136380, 19, }, // Antarctica/DumontDUrville
    {  426, 8568,  136399,  8, }, // Antarctica/Macquarie
    {  426, 7066,   89721,  4, }, // Antarctica/Mawson
    {  426, 7113,  136407,  9, }, // Antarctica/McMurdo
    {  426, 8589,  136416,  5, }, // Antarctica/Palmer
    {  426, 7226,   89738,  6, }, // Antarctica/Rothera
    {  426, 7295,   89744,  6, }, // Antarctica/Syowa
    {  426, 8607,   89750,  5, }, // Antarctica/Troll
    {  426, 7374,  136421,  7, }, // Antarctica/Vostok
    {  426, 1429,  136428, 12, }, // Arctic/Longyearbyen
    {  426, 5570,   31623,  4, }, // Asia/Aden
    {  426, 5395,   89779,  7, }, // Asia/Almaty
    {  426, 3968,   89786,  6, }, // Asia/Amman
    {  426, 5511,  136440,  7, }, // Asia/Anadyr
    {  426, 5523,  136447,  7, }, // Asia/Aqtau
    {  426, 5368,  136454,  9, }, // Asia/Aqtobe
    {  426,  964,   31659,  8, }, // Asia/Ashgabat
    {  426, 8624,  136463,  5, }, // Asia/Atyrau
    {  426, 3607,   89809,  5, }, // Asia/Baghdad
    {  426, 5534,  136468,  6, }, // Asia/Bahrain
    {  426, 3653,   88728,  4, }, // Asia/Baku
    {  426, 4466,  114568,  5, }, // Asia/Bangkok
    {  426, 3571,  136474,  6, }, // Asia/Barnaul
    {  426, 4086,   31699,  5, }, // Asia/Beirut
    {  426, 3778,   31704,  7, }, // Asia/Bishkek
    {  426, 5989,  136480,  7, }, // Asia/Brunei
    {  426, 4601,  129841,  4, }, // Asia/Chita
    {  426, 4511,   89849,  6, }, // Asia/Colombo
    {  426, 4540,   89855,  7, }, // Asia/Damascus
    {  426, 1093,   89862,  4, }, // Asia/Dhaka
    {  426, 6192,   89866,  4, }, // Asia/Dili
    {  426, 3596,   89870,  4, }, // Asia/Dubai
    {  426, 6159,   31749,  8, }, // Asia/Dushanbe
    {  426, 8636,  136487,  9, }, // Asia/Famagusta
    {  426, 8651,  114610,  4, }, // Asia/Gaza
    {  426, 4795,  114614,  7, }, // Asia/Hebron
    {  426, 1259,  136496, 15, }, // Asia/Ho_Chi_Minh
    {  426, 2704,  136511,  8, }, // Asia/Hong_Kong
    {  426, 4771,   89929,  5, }, // Asia/Hovd
    {  426, 4184,   31793, 10, }, // Asia/Irkutsk
    {  426, 7006,   89944,  7, }, // Asia/Jakarta
    {  426, 6992,   31811,  7, }, // Asia/Jayapura
    {  426, 1290,  136519,  8, }, // Asia/Jerusalem
    {  426, 3560,   31827,  5, }, // Asia/Kabul
    {  426, 4316,  136527,  8, }, // Asia/Kamchatka
    {  426, 4239,   31840,  5, }, // Asia/Karachi
    {  426, 1185,   89969,  8, }, // Asia/Kathmandu
    {  426, 8661,  136535,  6, }, // Asia/Khandyga
    {  426,  992,   89984,  7, }, // Asia/Kolkata
    {  426, 4197,   89991, 15, }, // Asia/Krasnoyarsk
    {  426, 4859,  136541, 11, }, // Asia/Kuala_Lumpur
    {  426, 5976,  129934,  6, }, // Asia/Kuching
    {  426, 5547,  114687,  5, }, // Asia/Kuwait
    {  426, 1211,   90016,  4, }, // Asia/Macau
    {  426, 4017,  136552,  6, }, // Asia/Magadan
    {  426, 1349,  136558,  7, }, // Asia/Makassar
    {  426, 7182,  114710,  5, }, // Asia/Manila
    {  426, 6920,   90165,  5, }, // Asia/Muscat
    {  426, 2554,   90031,  8, }, // Asia/Nicosia
    {  426, 8675,  136565, 15, }, // Asia/Novokuznetsk
    {  426, 4135,   90055, 13, }, // Asia/Novosibirsk
    {  426, 4229,   90068,  6, }, // Asia/Omsk
    {  426, 7145,   90074,  3, }, // Asia/Oral
    {  426, 6961,  136580, 11, }, // Asia/Phnom_Penh
    {  426, 8693,  136591, 10, }, // Asia/Pontianak
    {  426, 4214,   90095, 10, }, // Asia/Pyongyang
    {  426, 5559,  114750,  4, }, // Asia/Qatar
    {  426, 8708,   90108,  8, }, // Asia/Qostanay
    {  426, 4269,  136601, 10, }, // Asia/Qyzylorda
    {  426, 3584,   32028,  5, }, // Asia/Riyadh
    {  426, 4408,   32033,  7, }, // Asia/Sakhalin
    {  426, 7245,   90138,  6, }, // Asia/Samarkand
    {  426, 3279,  136611,  4, }, // Asia/Seoul
    {  426, 1053,  114780,  6, }, // Asia/Shanghai
    {  426, 3300,  114786,  8, }, // Asia/Singapore
    {  426, 4297,  136615, 16, }, // Asia/Srednekolymsk
    {  426, 3263,  114810,  5, }, // Asia/Taipei
    {  426, 4781,  114815,  7, }, // Asia/Tashkent
    {  426, 3915,   32098,  6, }, // Asia/Tbilisi
    {  426, 2772,   32104,  6, }, // Asia/Tehran
    {  426, 1317,  136631,  5, }, // Asia/Thimphu
    {  426, 2821,  136636,  6, }, // Asia/Tokyo
    {  426, 4572,  136642,  5, }, // Asia/Tomsk
    {  426, 1021,   90206,  8, }, // Asia/Ulaanbaatar
    {  426, 1159,  114855,  7, }, // Asia/Urumqi
    {  426, 8722,  130078,  9, }, // Asia/Ust-Nera
    {  426, 6977,  136647, 11, }, // Asia/Vientiane
    {  426, 4710,  136658, 13, }, // Asia/Vladivostok
    {  426, 4828,  114882,  8, }, // Asia/Yakutsk
    {  426, 1235,   32182,  5, }, // Asia/Yangon
    {  426, 3883,  136671, 13, }, // Asia/Yekaterinburg
    {  426, 3747,  136684,  6, }, // Asia/Yerevan
    {  426, 3663,  114896,  6, }, // Atlantic/Azores
    {  426, 5671,  114902,  7, }, // Atlantic/Bermuda
    {  426, 6628,  114909,  5, }, // Atlantic/Canary
    {  426, 3727,  136690, 11, }, // Atlantic/Cape_Verde
    {  426, 1395,  136701,  4, }, // Atlantic/Faroe
    {  426, 8736,  136705,  6, }, // Atlantic/Madeira
    {  426, 2748,  136711, 10, }, // Atlantic/Reykjavik
    {  426, 7272,   90319, 15, }, // Atlantic/South_Georgia
    {  426, 6821,  136721, 12, }, // Atlantic/St_Helena
    {  426, 6644,  136733,  7, }, // Atlantic/Stanley
    {  426, 1670,  136740,  5, }, // Australia/Adelaide
    {  426, 1635,  114960,  9, }, // Australia/Brisbane
    {  426, 1799,   90366, 10, }, // Australia/Broken_Hill
    {  426, 1583,   90376,  7, }, // Australia/Darwin
    {  426, 3637,  114969,  5, }, // Australia/Eucla
    {  426, 1516,   90383,  7, }, // Australia/Hobart
    {  426, 8753,  136745,  7, }, // Australia/Lindeman
    {  426, 1547,   90398, 10, }, // Australia/Lord_Howe
    {  426, 1727,   90408,  8, }, // Australia/Melbourne
    {  426, 1762,   90416,  4, }, // Australia/Perth
    {  426, 1463,   32362,  5, }, // Australia/Sydney
    {  426, 6441,  136752, 11, }, // Europe/Amsterdam
    {  426, 6216,  136763,  6, }, // Europe/Andorra
    {  426, 3620,   32384, 10, }, // Europe/Astrakhan
    {  426, 2300,  115006,  6, }, // Europe/Athens
    {  426, 6336,   32400,  8, }, // Europe/Belgrade
    {  426, 4757,   90451,  6, }, // Europe/Berlin
    {  426, 6488,  136769, 14, }, // Europe/Bratislava
    {  426, 2167,  115026,  9, }, // Europe/Brussels
    {  426, 3928,  115035,  9, }, // Europe/Bucharest
    {  426, 3806,   90488,  9, }, // Europe/Budapest
    {  426, 8772,  115044,  8, }, // Europe/Busingen
    {  426, 2583,  136783,  6, }, // Europe/Chisinau
    {  426, 6289,   90514,  9, }, // Europe/Copenhagen
    {  426, 2338,  115067,  6, }, // Europe/Dublin
    {  426, 6307,   90528, 10, }, // Europe/Gibraltar
    {  426, 8788,  136789,  9, }, // Europe/Guernsey
    {  426, 6612,   90544,  8, }, // Europe/Helsinki
    {  426, 8804,  136798, 10, }, // Europe/Isle_of_Man
    {  426, 1130,  115084,  8, }, // Europe/Istanbul
    {  426, 8823,   90572,  5, }, // Europe/Jersey
    {  426, 3979,  136808, 11, }, // Europe/Kaliningrad
    {  426, 8929,   90589,  5, }, // Europe/Kirov
    {  426, 2527,   90594,  3, }, // Europe/Kyiv
    {  426, 3213,   32558,  6, }, // Europe/Lisbon
    {  426, 6506,  136819, 10, }, // Europe/Ljubljana
    {  426, 2501,  115113,  4, }, // Europe/London
    {  426, 6365,  136829, 10, }, // Europe/Luxembourg
    {  426, 4892,   32592,  7, }, // Europe/Madrid
    {  426, 6397,   32599,  6, }, // Europe/Malta
    {  426, 6582,  136839, 10, }, // Europe/Mariehamn
    {  426, 3693,   32612,  7, }, // Europe/Minsk
    {  426, 6410,  115143,  6, }, // Europe/Monaco
    {  426, 3537,   90650,  6, }, // Europe/Moscow
    {  426, 6458,  136849,  5, }, // Europe/Oslo
    {  426, 4284,  115149,  5, }, // Europe/Paris
    {  426, 6424,  115154,  9, }, // Europe/Podgorica
    {  426, 6275,   90671,  5, }, // Europe/Prague
    {  426, 8837,   32652,  4, }, // Europe/Riga
    {  426, 6324,   90303,  3, }, // Europe/Rome
    {  426, 4331,   90680,  5, }, // Europe/Samara
    {  426, 6470,  136854,  9, }, // Europe/San_Marino
    {  426, 6245,  136863, 10, }, // Europe/Sarajevo
    {  426, 4451,   90703,  7, }, // Europe/Saratov
    {  426, 8849,  136873, 10, }, // Europe/Simferopol
    {  426, 6383,  136883,  8, }, // Europe/Skopje
    {  426, 6599,  115197,  6, }, // Europe/Sofia
    {  426, 6523,   90737,  8, }, // Europe/Stockholm
    {  426, 8867,  136891,  5, }, // Europe/Tallinn
    {  426, 6202,  130482,  6, }, // Europe/Tirane
    {  426, 8882,   90756, 12, }, // Europe/Ulyanovsk
    {  426, 6352,  136896,  4, }, // Europe/Vaduz
    {  426, 6567,  136900,  8, }, // Europe/Vatican
    {  426, 6231,  136908,  9, }, // Europe/Vienna
    {  426, 8899,  136917,  7, }, // Europe/Vilnius
    {  426, 4727,  115230, 13, }, // Europe/Volgograd
    {  426, 3190,  136924,  6, }, // Europe/Warsaw
    {  426, 6261,  136930,  7, }, // Europe/Zagreb
    {  426, 6540,  136937,  5, }, // Europe/Zurich
    {  426, 5078,  115260, 12, }, // Indian/Antananarivo
    {  426, 6947,  115272,  5, }, // Indian/Chagos
    {  426, 6046,  136942,  7, }, // Indian/Christmas
    {  426, 6063,   90849,  5, }, // Indian/Cocos
    {  426, 5029,   90854,  6, }, // Indian/Comoro
    {  426, 6661,  136949,  9, }, // Indian/Kerguelen
    {  426, 7260,   32851,  4, }, // Indian/Mahe
    {  426, 7050,   90867,  6, }, // Indian/Maldives
    {  426, 4069,  115285,  6, }, // Indian/Mauritius
    {  426, 5098,  136958,  8, }, // Indian/Mayotte
    {  426, 7211,  115296,  8, }, // Indian/Reunion
    {  426, 4422,  115304,  5, }, // Pacific/Apia
    {  426,  932,  136966,  5, }, // Pacific/Auckland
    {  426, 3706,  136971, 10, }, // Pacific/Bougainville
    {  426, 3015,  136981,  6, }, // Pacific/Chatham
    {  426, 3157,   32913,  3, }, // Pacific/Chuuk
    {  426, 2237,   90920,  5, }, // Pacific/Easter
    {  426, 7360,  115330,  4, }, // Pacific/Efate
    {  426, 7327,  115334,  7, }, // Pacific/Fakaofo
    {  426, 3902,   32937,  4, }, // Pacific/Fiji
    {  426, 7343,  136987,  8, }, // Pacific/Funafuti
    {  426, 6678,  136995,  9, }, // Pacific/Galapagos
    {  426, 6696,  137004,  8, }, // Pacific/Gambier
    {  426, 3822,   90972, 10, }, // Pacific/Guadalcanal
    {  426, 6018,   30980,  4, }, // Pacific/Guam
    {  426, 2723,  137012,  8, }, // Pacific/Honolulu
    {  426, 3049,   90990,  7, }, // Pacific/Kanton
    {  426, 3998,  115385, 10, }, // Pacific/Kiritimati
    {  426, 7019,  137020,  5, }, // Pacific/Kosrae
    {  426, 2842,  137025, 10, }, // Pacific/Kwajalein
    {  426, 4877,  137035,  5, }, // Pacific/Majuro
    {  426, 4051,  137040, 10, }, // Pacific/Marquesas
    {  426, 8914,   91027,  5, }, // Pacific/Midway
    {  426, 7084,  137050,  4, }, // Pacific/Nauru
    {  426, 7132,  137054,  5, }, // Pacific/Niue
    {  426, 4168,  115421,  6, }, // Pacific/Norfolk
    {  426, 7098,  137059,  5, }, // Pacific/Noumea
    {  426, 3126,  137064,  9, }, // Pacific/Pago_Pago
    {  426, 7155,   91062,  4, }, // Pacific/Palau
    {  426, 7194,  137073,  8, }, // Pacific/Pitcairn
    {  426, 3096,   91066,  6, }, // Pacific/Pohnpei
    {  426, 4807,  137081, 14, }, // Pacific/Port_Moresby
    {  426, 6076,   91086,  9, }, // Pacific/Rarotonga
    {  426, 6031,  137095,  5, }, // Pacific/Saipan
    {  426, 7312,   91101,  6, }, // Pacific/Tahiti
    {  426, 6712,   33122,  6, }, // Pacific/Tarawa
    {  426, 4583,   91113,  9, }, // Pacific/Tongatapu
    {  426, 7392,   33136,  3, }, // Pacific/Wake
    {  426, 7405,   33139,  5, }, // Pacific/Wallis
    {  427,   14,       0,  6, }, // Africa/Asmara Masai/Latin/Kenya
    {  427, 4435,       6,  8, }, // Africa/Sao_Tome
    {  427, 4252,      14,  8, }, // America/Asuncion
    {  427,  237,      22,  8, }, // America/Atikokan
    {  427, 7623,      30, 17, }, // America/Bahia_Banderas
    {  427, 3868,      47,  6, }, // America/Cancun
    {  427, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  427, 5723,      66,  7, }, // America/Curacao
    {  427,  481,      73, 13, }, // America/Indiana/Knox
    {  427, 7961,      86, 16, }, // America/Indiana/Marengo
    {  427, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  427, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  427, 8038,     139, 14, }, // America/Indiana/Vevay
    {  427, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  427, 8086,     171, 16, }, // America/Indiana/Winamac
    {  427, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  427, 5932,     207, 22, }, // America/Lower_Princes
    {  427, 8220,     229,  6, }, // America/Merida
    {  427, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  427, 8329,     255, 20, }, // America/North_Dakota/Center
    {  427, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  427,  393,     298,  4, }, // America/Nuuk
    {  427, 6899,     302, 16, }, // America/Scoresbysund
    {  427, 8508,     318, 14, }, // America/St_Barthelemy
    {  427, 2061,     332, 10, }, // America/St_Johns
    {  427, 5863,     342,  9, }, // America/St_Kitts
    {  427, 5880,     351,  9, }, // America/St_Lucia
    {  427,  855,     360, 10, }, // America/St_Thomas
    {  427, 5913,     370, 11, }, // America/St_Vincent
    {  427, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  427, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  427, 1185,     413,  9, }, // Asia/Kathmandu
    {  427,  992,     422,  7, }, // Asia/Kolkata
    {  427, 1211,     429,  5, }, // Asia/Macau
    {  427, 1235,     442,  6, }, // Asia/Yangon
    {  427, 1395,     448,  5, }, // Atlantic/Faroe
    {  427, 6821,     453, 10, }, // Atlantic/St_Helena
    {  427, 2527,     463,  4, }, // Europe/Kyiv
    {  427, 7211,     467,  7, }, // Indian/Reunion
    {  427, 3157,     474,  5, }, // Pacific/Chuuk
    {  427, 3096,     479,  7, }, // Pacific/Pohnpei
    {  429,   14,       0,  6, }, // Africa/Asmara Mazanderani/Arabic/Iran
    {  429, 4435,       6,  8, }, // Africa/Sao_Tome
    {  429, 4252,      14,  8, }, // America/Asuncion
    {  429,  237,      22,  8, }, // America/Atikokan
    {  429, 7623,      30, 17, }, // America/Bahia_Banderas
    {  429, 3868,      47,  6, }, // America/Cancun
    {  429, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  429, 5723,      66,  7, }, // America/Curacao
    {  429,  481,      73, 13, }, // America/Indiana/Knox
    {  429, 7961,      86, 16, }, // America/Indiana/Marengo
    {  429, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  429, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  429, 8038,     139, 14, }, // America/Indiana/Vevay
    {  429, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  429, 8086,     171, 16, }, // America/Indiana/Winamac
    {  429, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  429, 5932,     207, 22, }, // America/Lower_Princes
    {  429, 8220,     229,  6, }, // America/Merida
    {  429, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  429, 8329,     255, 20, }, // America/North_Dakota/Center
    {  429, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  429,  393,     298,  4, }, // America/Nuuk
    {  429, 6899,     302, 16, }, // America/Scoresbysund
    {  429, 8508,     318, 14, }, // America/St_Barthelemy
    {  429, 2061,     332, 10, }, // America/St_Johns
    {  429, 5863,     342,  9, }, // America/St_Kitts
    {  429, 5880,     351,  9, }, // America/St_Lucia
    {  429,  855,     360, 10, }, // America/St_Thomas
    {  429, 5913,     370, 11, }, // America/St_Vincent
    {  429, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  429, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  429, 1185,     413,  9, }, // Asia/Kathmandu
    {  429,  992,     422,  7, }, // Asia/Kolkata
    {  429, 1211,     429,  5, }, // Asia/Macau
    {  429, 1235,     442,  6, }, // Asia/Yangon
    {  429, 1395,     448,  5, }, // Atlantic/Faroe
    {  429, 6821,     453, 10, }, // Atlantic/St_Helena
    {  429, 2527,     463,  4, }, // Europe/Kyiv
    {  429, 7211,     467,  7, }, // Indian/Reunion
    {  429, 3157,     474,  5, }, // Pacific/Chuuk
    {  429, 3096,     479,  7, }, // Pacific/Pohnpei
    {  430,   14,       0,  6, }, // Africa/Asmara Meru/Latin/Kenya
    {  430, 4435,       6,  8, }, // Africa/Sao_Tome
    {  430, 4252,      14,  8, }, // America/Asuncion
    {  430,  237,      22,  8, }, // America/Atikokan
    {  430, 7623,      30, 17, }, // America/Bahia_Banderas
    {  430, 3868,      47,  6, }, // America/Cancun
    {  430, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  430, 5723,      66,  7, }, // America/Curacao
    {  430,  481,      73, 13, }, // America/Indiana/Knox
    {  430, 7961,      86, 16, }, // America/Indiana/Marengo
    {  430, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  430, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  430, 8038,     139, 14, }, // America/Indiana/Vevay
    {  430, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  430, 8086,     171, 16, }, // America/Indiana/Winamac
    {  430, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  430, 5932,     207, 22, }, // America/Lower_Princes
    {  430, 8220,     229,  6, }, // America/Merida
    {  430, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  430, 8329,     255, 20, }, // America/North_Dakota/Center
    {  430, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  430,  393,     298,  4, }, // America/Nuuk
    {  430, 6899,     302, 16, }, // America/Scoresbysund
    {  430, 8508,     318, 14, }, // America/St_Barthelemy
    {  430, 2061,     332, 10, }, // America/St_Johns
    {  430, 5863,     342,  9, }, // America/St_Kitts
    {  430, 5880,     351,  9, }, // America/St_Lucia
    {  430,  855,     360, 10, }, // America/St_Thomas
    {  430, 5913,     370, 11, }, // America/St_Vincent
    {  430, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  430, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  430, 1185,     413,  9, }, // Asia/Kathmandu
    {  430,  992,     422,  7, }, // Asia/Kolkata
    {  430, 1211,     429,  5, }, // Asia/Macau
    {  430, 1235,     442,  6, }, // Asia/Yangon
    {  430, 1395,     448,  5, }, // Atlantic/Faroe
    {  430, 6821,     453, 10, }, // Atlantic/St_Helena
    {  430, 2527,     463,  4, }, // Europe/Kyiv
    {  430, 7211,     467,  7, }, // Indian/Reunion
    {  430, 3157,     474,  5, }, // Pacific/Chuuk
    {  430, 3096,     479,  7, }, // Pacific/Pohnpei
    {  431,   14,       0,  6, }, // Africa/Asmara Meta/Latin/Cameroon
    {  431, 4435,       6,  8, }, // Africa/Sao_Tome
    {  431, 4252,      14,  8, }, // America/Asuncion
    {  431,  237,      22,  8, }, // America/Atikokan
    {  431, 7623,      30, 17, }, // America/Bahia_Banderas
    {  431, 3868,      47,  6, }, // America/Cancun
    {  431, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  431, 5723,      66,  7, }, // America/Curacao
    {  431,  481,      73, 13, }, // America/Indiana/Knox
    {  431, 7961,      86, 16, }, // America/Indiana/Marengo
    {  431, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  431, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  431, 8038,     139, 14, }, // America/Indiana/Vevay
    {  431, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  431, 8086,     171, 16, }, // America/Indiana/Winamac
    {  431, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  431, 5932,     207, 22, }, // America/Lower_Princes
    {  431, 8220,     229,  6, }, // America/Merida
    {  431, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  431, 8329,     255, 20, }, // America/North_Dakota/Center
    {  431, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  431,  393,     298,  4, }, // America/Nuuk
    {  431, 6899,     302, 16, }, // America/Scoresbysund
    {  431, 8508,     318, 14, }, // America/St_Barthelemy
    {  431, 2061,     332, 10, }, // America/St_Johns
    {  431, 5863,     342,  9, }, // America/St_Kitts
    {  431, 5880,     351,  9, }, // America/St_Lucia
    {  431,  855,     360, 10, }, // America/St_Thomas
    {  431, 5913,     370, 11, }, // America/St_Vincent
    {  431, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  431, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  431, 1185,     413,  9, }, // Asia/Kathmandu
    {  431,  992,     422,  7, }, // Asia/Kolkata
    {  431, 1211,     429,  5, }, // Asia/Macau
    {  431, 1235,     442,  6, }, // Asia/Yangon
    {  431, 1395,     448,  5, }, // Atlantic/Faroe
    {  431, 6821,     453, 10, }, // Atlantic/St_Helena
    {  431, 2527,     463,  4, }, // Europe/Kyiv
    {  431, 7211,     467,  7, }, // Indian/Reunion
    {  431, 3157,     474,  5, }, // Pacific/Chuuk
    {  431, 3096,     479,  7, }, // Pacific/Pohnpei
    {  432,   14,       0,  6, }, // Africa/Asmara Mohawk/Latin/Canada
    {  432, 4435,       6,  8, }, // Africa/Sao_Tome
    {  432, 4252,      14,  8, }, // America/Asuncion
    {  432,  237,      22,  8, }, // America/Atikokan
    {  432, 7623,      30, 17, }, // America/Bahia_Banderas
    {  432, 3868,      47,  6, }, // America/Cancun
    {  432, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  432, 5723,      66,  7, }, // America/Curacao
    {  432,  481,      73, 13, }, // America/Indiana/Knox
    {  432, 7961,      86, 16, }, // America/Indiana/Marengo
    {  432, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  432, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  432, 8038,     139, 14, }, // America/Indiana/Vevay
    {  432, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  432, 8086,     171, 16, }, // America/Indiana/Winamac
    {  432, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  432, 5932,     207, 22, }, // America/Lower_Princes
    {  432, 8220,     229,  6, }, // America/Merida
    {  432, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  432, 8329,     255, 20, }, // America/North_Dakota/Center
    {  432, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  432,  393,     298,  4, }, // America/Nuuk
    {  432, 6899,     302, 16, }, // America/Scoresbysund
    {  432, 8508,     318, 14, }, // America/St_Barthelemy
    {  432, 2061,     332, 10, }, // America/St_Johns
    {  432, 5863,     342,  9, }, // America/St_Kitts
    {  432, 5880,     351,  9, }, // America/St_Lucia
    {  432,  855,     360, 10, }, // America/St_Thomas
    {  432, 5913,     370, 11, }, // America/St_Vincent
    {  432, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  432, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  432, 1185,     413,  9, }, // Asia/Kathmandu
    {  432,  992,     422,  7, }, // Asia/Kolkata
    {  432, 1211,     429,  5, }, // Asia/Macau
    {  432, 1235,     442,  6, }, // Asia/Yangon
    {  432, 1395,     448,  5, }, // Atlantic/Faroe
    {  432, 6821,     453, 10, }, // Atlantic/St_Helena
    {  432, 2527,     463,  4, }, // Europe/Kyiv
    {  432, 7211,     467,  7, }, // Indian/Reunion
    {  432, 3157,     474,  5, }, // Pacific/Chuuk
    {  432, 3096,     479,  7, }, // Pacific/Pohnpei
    {  433, 6788,  137100,  6, }, // Africa/Abidjan Mongolian/Cyrillic/Mongolia
    {  433, 6760,     492,  5, }, // Africa/Accra
    {  433, 5059,  137106, 11, }, // Africa/Addis_Ababa
    {  433, 7420,     508,  5, }, // Africa/Algiers
    {  433,   14,   27065,  6, }, // Africa/Asmara
    {  433,   44,     519,  6, }, // Africa/Bamako
    {  433, 5257,  137117,  6, }, // Africa/Bangui
    {  433, 6746,     530,  6, }, // Africa/Banjul
    {  433, 7435,  137123,  6, }, // Africa/Bissau
    {  433, 4957,   27088,  8, }, // Africa/Blantyre
    {  433, 5287,   51422, 10, }, // Africa/Brazzaville
    {  433, 4922,     558,  9, }, // Africa/Bujumbura
    {  433, 2320,     567,  4, }, // Africa/Cairo
    {  433, 4117,     571, 10, }, // Africa/Casablanca
    {  433, 7449,     581,  5, }, // Africa/Ceuta
    {  433, 6773,     586,  7, }, // Africa/Conakry
    {  433, 6840,     593,  5, }, // Africa/Dakar
    {  433, 5130,   27116, 12, }, // Africa/Dar_es_Salaam
    {  433, 5043,  117997,  6, }, // Africa/Djibouti
    {  433, 5243,     616,  5, }, // Africa/Douala
    {  433, 5166,   27135,  7, }, // Africa/El_Aaiun
    {  433, 6853,     628,  7, }, // Africa/Freetown
    {  433, 4906,     635,  8, }, // Africa/Gaborone
    {  433, 5015,   33781,  6, }, // Africa/Harare
    {  433, 4479,   51440, 12, }, // Africa/Johannesburg
    {  433, 4499,  118010,  4, }, // Africa/Juba
    {  433, 5151,     665,  7, }, // Africa/Kampala
    {  433, 4524,     672,  6, }, // Africa/Khartoum
    {  433, 4973,     678,  6, }, // Africa/Kigali
    {  433, 5306,     684,  7, }, // Africa/Kinshasa
    {  433, 4744,     691,  5, }, // Africa/Lagos
    {  433, 5336,   51452,  9, }, // Africa/Libreville
    {  433, 6869,     704,  4, }, // Africa/Lome
    {  433, 5211,     708,  6, }, // Africa/Luanda
    {  433, 4939,     714,  9, }, // Africa/Lubumbashi
    {  433, 5001,     723,  6, }, // Africa/Lusaka
    {  433, 5322,     729,  6, }, // Africa/Malabo
    {  433, 4987,   33813,  6, }, // Africa/Maputo
    {  433, 5197,     741,  6, }, // Africa/Maseru
    {  433, 5182,     747,  7, }, // Africa/Mbabane
    {  433, 5113,   36279,  8, }, // Africa/Mogadishu
    {  433, 7034,     762,  8, }, // Africa/Monrovia
    {  433, 3853,   36295,  7, }, // Africa/Nairobi
    {  433, 5271,  118023,  7, }, // Africa/Ndjamena
    {  433, 5354,   36302,  6, }, // Africa/Niamey
    {  433, 6803,     790,  7, }, // Africa/Nouakchott
    {  433, 6727,     797,  8, }, // Africa/Ouagadougou
    {  433, 5225,     805, 10, }, // Africa/Porto-Novo
    {  433, 4435,     815,  8, }, // Africa/Sao_Tome
    {  433, 2866,     823,  7, }, // Africa/Tripoli
    {  433, 6554,     830,  5, }, // Africa/Tunis
    {  433, 4152,     835,  7, }, // Africa/Windhoek
    {  433,  132,     842,  4, }, // America/Adak
    {  433, 3346,  137129,  7, }, // America/Anchorage
    {  433, 5607,   51469,  7, }, // America/Anguilla
    {  433, 5624,     860,  7, }, // America/Antigua
    {  433, 4554,  137136,  9, }, // America/Araguaina
    {  433,  166,   51476, 12, }, // America/Argentina/Buenos_Aires
    {  433,   91,     888,  9, }, // America/Argentina/Catamarca
    {  433,  270,     897,  7, }, // America/Argentina/Cordoba
    {  433,  441,   27355,  5, }, // America/Argentina/Jujuy
    {  433, 7462,     909,  8, }, // America/Argentina/La_Rioja
    {  433,  565,   33923,  7, }, // America/Argentina/Mendoza
    {  433, 7489,   51488, 12, }, // America/Argentina/Rio_Gallegos
    {  433, 7520,     935,  5, }, // America/Argentina/Salta
    {  433, 7544,     940,  8, }, // America/Argentina/San_Juan
    {  433, 5580,   33949,  7, }, // America/Argentina/San_Luis
    {  433, 7571,     956,  7, }, // America/Argentina/Tucuman
    {  433, 7597,  106131,  6, }, // America/Argentina/Ushuaia
    {  433, 5640,     969,  5, }, // America/Aruba
    {  433, 4252,   51500,  8, }, // America/Asuncion
    {  433,  237,   33971,  8, }, // America/Atikokan
    {  433, 3679,  137145,  4, }, // America/Bahia
    {  433, 7623,  137149, 14, }, // America/Bahia_Banderas
    {  433, 5654,    1013,  8, }, // America/Barbados
    {  433, 7646,   33998,  5, }, // America/Belem
    {  433, 5407,    1026,  5, }, // America/Belize
    {  433, 7660,   36388, 11, }, // America/Blanc-Sablon
    {  433, 7681,    1043,  9, }, // America/Boa_Vista
    {  433, 4361,    1052,  6, }, // America/Bogota
    {  433, 7699,    1058,  5, }, // America/Boise
    {  433, 7713,  137163, 16, }, // America/Cambridge_Bay
    {  433, 7735,  137179, 12, }, // America/Campo_Grande
    {  433, 3868,    1087,  6, }, // America/Cancun
    {  433, 4694,    1093,  7, }, // America/Caracas
    {  433, 4345,   51542,  7, }, // America/Cayenne
    {  433, 5496,   27497,  6, }, // America/Cayman
    {  433, 2260,    1129,  6, }, // America/Chicago
    {  433, 7756,  118097,  8, }, // America/Chihuahua
    {  433, 7774,  137191, 13, }, // America/Ciudad_Juarez
    {  433, 5422,    1141, 10, }, // America/Costa_Rica
    {  433, 7796,    1151,  7, }, // America/Creston
    {  433, 3791,   27556,  5, }, // America/Cuiaba
    {  433, 5723,   36474,  7, }, // America/Curacao
    {  433, 7812,  137204, 11, }, // America/Danmarkshavn
    {  433, 7833,    1184,  6, }, // America/Dawson
    {  433, 7848,  137215, 11, }, // America/Dawson_Creek
    {  433,  805,    1201,  6, }, // America/Denver
    {  433, 3465,   36517,  7, }, // America/Detroit
    {  433, 5739,    1214,  7, }, // America/Dominica
    {  433,  893,   51549,  8, }, // America/Edmonton
    {  433, 7869,   51557,  8, }, // America/Eirunepe
    {  433, 5441,  137226, 13, }, // America/El_Salvador
    {  433, 7886,  137239, 12, }, // America/Fort_Nelson
    {  433, 7906,    1257,  9, }, // America/Fortaleza
    {  433, 7924,  137251, 11, }, // America/Glace_Bay
    {  433, 6881,  137262, 10, }, // America/Goose_Bay
    {  433, 4612,   34146, 10, }, // America/Grand_Turk
    {  433, 5770,    1292,  7, }, // America/Grenada
    {  433, 5786,    1299,  8, }, // America/Guadeloupe
    {  433, 3760,    1308,  8, }, // America/Guatemala
    {  433, 4841,   51593,  8, }, // America/Guayaquil
    {  433, 6932,   51601,  6, }, // America/Guyana
    {  433, 1939,   34178,  8, }, // America/Halifax
    {  433, 2281,   34186,  6, }, // America/Havana
    {  433, 7942,  137272, 10, }, // America/Hermosillo
    {  433,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  433,  481,    1365, 13, }, // America/Indiana/Knox
    {  433, 7961,    1378, 16, }, // America/Indiana/Marengo
    {  433, 7985,  118175, 19, }, // America/Indiana/Petersburg
    {  433, 8012,  137282, 17, }, // America/Indiana/Tell_City
    {  433, 8038,  106186, 14, }, // America/Indiana/Vevay
    {  433, 8060,  137299, 17, }, // America/Indiana/Vincennes
    {  433, 8086,  137316, 16, }, // America/Indiana/Winamac
    {  433, 8110,    1452,  6, }, // America/Inuvik
    {  433,  660,    1458,  7, }, // America/Iqaluit
    {  433, 2799,   27879,  6, }, // America/Jamaica
    {  433, 5380,   34336,  4, }, // America/Juneau
    {  433,  521,    1476,  8, }, // America/Kentucky/Louisville
    {  433, 8125,  137332, 19, }, // America/Kentucky/Monticello
    {  433, 5704,  137351,  9, }, // America/Kralendijk
    {  433, 4376,   34368,  6, }, // America/La_Paz
    {  433, 7169,    1520,  4, }, // America/Lima
    {  433, 3239,  118255, 11, }, // America/Los_Angeles
    {  433, 5932,  137360, 21, }, // America/Lower_Princes
    {  433, 8153,   36667,  6, }, // America/Maceio
    {  433, 8168,    1559,  7, }, // America/Managua
    {  433, 1908,    1566,  6, }, // America/Manaus
    {  433, 5897,  137381,  7, }, // America/Marigot
    {  433, 5805,    1578,  8, }, // America/Martinique
    {  433, 8184,    1587,  9, }, // America/Matamoros
    {  433, 2917,   34417,  8, }, // America/Mazatlan
    {  433, 8202,    1604,  8, }, // America/Menominee
    {  433, 8220,    1612,  6, }, // America/Merida
    {  433, 8235,    1618, 10, }, // America/Metlakatla
    {  433, 2949,    1628,  6, }, // America/Mexico_City
    {  433, 4391,    1634,  7, }, // America/Miquelon
    {  433, 8254,    1641,  7, }, // America/Moncton
    {  433, 8270,   36673,  8, }, // America/Monterrey
    {  433, 4098,    1657, 10, }, // America/Montevideo
    {  433, 5824,    1667, 10, }, // America/Montserrat
    {  433, 5481,    1677,  6, }, // America/Nassau
    {  433, 2379,   51680,  8, }, // America/New_York
    {  433, 8288,    1691,  3, }, // America/Nome
    {  433, 1850,  137388,  6, }, // America/Noronha
    {  433, 8301,  137394, 17, }, // America/North_Dakota/Beulah
    {  433, 8329,  137411, 16, }, // America/North_Dakota/Center
    {  433, 8357,  137427, 22, }, // America/North_Dakota/New_Salem
    {  433,  393,  137449,  4, }, // America/Nuuk
    {  433, 8388,  137453,  7, }, // America/Ojinaga
    {  433, 2356,    1783,  6, }, // America/Panama
    {  433, 6173,    1789, 10, }, // America/Paramaribo
    {  433, 2973,   34541,  6, }, // America/Phoenix
    {  433, 3945,  137460, 12, }, // America/Port-au-Prince
    {  433, 5954,  137472, 13, }, // America/Port_of_Spain
    {  433, 8404,  137485, 11, }, // America/Porto_Velho
    {  433, 5843,   51756, 11, }, // America/Puerto_Rico
    {  433, 4030,   34579, 11, }, // America/Punta_Arenas
    {  433, 8424,  118397, 12, }, // America/Rankin_Inlet
    {  433, 8445,    1876,  6, }, // America/Recife
    {  433, 1995,   34609,  6, }, // America/Regina
    {  433, 8460,   36760,  7, }, // America/Resolute
    {  433,  695,  137496, 10, }, // America/Rio_Branco
    {  433, 8477,   34632,  8, }, // America/Santarem
    {  433, 2201,   51775,  8, }, // America/Santiago
    {  433, 6111,   34649, 13, }, // America/Santo_Domingo
    {  433, 1878,    1936,  9, }, // America/Sao_Paulo
    {  433, 6899,    1945, 11, }, // America/Scoresbysund
    {  433, 8494,    1956,  5, }, // America/Sitka
    {  433, 8508,  137506, 14, }, // America/St_Barthelemy
    {  433, 2061,  137520,  9, }, // America/St_Johns
    {  433, 5863,  137529, 10, }, // America/St_Kitts
    {  433, 5880,  137539, 10, }, // America/St_Lucia
    {  433,  855,  128473, 10, }, // America/St_Thomas
    {  433, 5913,   34731, 12, }, // America/St_Vincent
    {  433, 8530,  118466, 13, }, // America/Swift_Current
    {  433, 5461,   51817, 12, }, // America/Tegucigalpa
    {  433, 5756,    2049,  4, }, // America/Thule
    {  433,  313,    2053,  7, }, // America/Tijuana
    {  433,  608,    2060,  7, }, // America/Toronto
    {  433, 5688,    2067,  7, }, // America/Tortola
    {  433, 2093,    2074,  8, }, // America/Vancouver
    {  433, 2144,  137549,  8, }, // America/Whitehorse
    {  433,  734,    2090,  8, }, // America/Winnipeg
    {  433, 8552,   28450,  6, }, // America/Yakutat
    {  433, 6001,  137557,  5, }, // Antarctica/Casey
    {  433, 6094,  118487,  5, }, // Antarctica/Davis
    {  433, 6133,  137562, 14, }, // Antarctica/DumontDUrville
    {  433, 8568,    2131,  8, }, // Antarctica/Macquarie
    {  433, 7066,    2139,  6, }, // Antarctica/Mawson
    {  433, 7113,  137576,  9, }, // Antarctica/McMurdo
    {  433, 8589,    2154,  6, }, // Antarctica/Palmer
    {  433, 7226,    2160,  6, }, // Antarctica/Rothera
    {  433, 7295,   28510,  4, }, // Antarctica/Syowa
    {  433, 8607,    2171,  5, }, // Antarctica/Troll
    {  433, 7374,   34825,  6, }, // Antarctica/Vostok
    {  433, 1429,  137585, 11, }, // Arctic/Longyearbyen
    {  433, 5570,    2192,  4, }, // Asia/Aden
    {  433, 5395,    2196,  6, }, // Asia/Almaty
    {  433, 3968,    2202,  5, }, // Asia/Amman
    {  433, 5511,    2207,  6, }, // Asia/Anadyr
    {  433, 5523,    2213,  5, }, // Asia/Aqtau
    {  433, 5368,  137596,  6, }, // Asia/Aqtobe
    {  433,  964,    2224,  7, }, // Asia/Ashgabat
    {  433, 8624,    2231,  6, }, // Asia/Atyrau
    {  433, 3607,    2237,  6, }, // Asia/Baghdad
    {  433, 5534,   36948,  7, }, // Asia/Bahrain
    {  433, 3653,    2250,  4, }, // Asia/Baku
    {  433, 4466,    2254,  7, }, // Asia/Bangkok
    {  433, 3571,    2261,  7, }, // Asia/Barnaul
    {  433, 4086,   28563,  6, }, // Asia/Beirut
    {  433, 3778,    2274,  6, }, // Asia/Bishkek
    {  433, 5989,   28575,  6, }, // Asia/Brunei
    {  433, 4601,    2286,  4, }, // Asia/Chita
    {  433, 4511,    2290,  7, }, // Asia/Colombo
    {  433, 4540,    2297,  6, }, // Asia/Damascus
    {  433, 1093,     593,  4, }, // Asia/Dhaka
    {  433, 6192,    2308,  4, }, // Asia/Dili
    {  433, 3596,   28596,  5, }, // Asia/Dubai
    {  433, 6159,    2317,  7, }, // Asia/Dushanbe
    {  433, 8636,    2324,  9, }, // Asia/Famagusta
    {  433, 8651,    2333,  4, }, // Asia/Gaza
    {  433, 4795,   34886,  6, }, // Asia/Hebron
    {  433, 1259,  137602, 13, }, // Asia/Ho_Chi_Minh
    {  433, 2704,   34901,  9, }, // Asia/Hong_Kong
    {  433, 4771,    2357,  4, }, // Asia/Hovd
    {  433, 4184,  137615,  5, }, // Asia/Irkutsk
    {  433, 7006,  118528,  7, }, // Asia/Jakarta
    {  433, 6992,  137620,  6, }, // Asia/Jayapura
    {  433, 1290,   36962,  9, }, // Asia/Jerusalem
    {  433, 3560,    2392,  5, }, // Asia/Kabul
    {  433, 4316,    2411,  8, }, // Asia/Kamchatka
    {  433, 4239,    2422,  6, }, // Asia/Karachi
    {  433, 1185,    2428,  8, }, // Asia/Kathmandu
    {  433, 8661,  118543,  7, }, // Asia/Khandyga
    {  433,  992,   51950,  9, }, // Asia/Kolkata
    {  433, 4197,   36978, 10, }, // Asia/Krasnoyarsk
    {  433, 4859,   34958, 12, }, // Asia/Kuala_Lumpur
    {  433, 5976,    2474,  5, }, // Asia/Kuching
    {  433, 5547,   28693,  6, }, // Asia/Kuwait
    {  433, 1211,    2486,  5, }, // Asia/Macau
    {  433, 4017,    2491,  7, }, // Asia/Magadan
    {  433, 1349,  118555,  8, }, // Asia/Makassar
    {  433, 7182,    2505,  6, }, // Asia/Manila
    {  433, 6920,   34981,  6, }, // Asia/Muscat
    {  433, 2554,  106335,  7, }, // Asia/Nicosia
    {  433, 8675,    2524, 11, }, // Asia/Novokuznetsk
    {  433, 4135,    2535, 11, }, // Asia/Novosibirsk
    {  433, 4229,    2546,  4, }, // Asia/Omsk
    {  433, 7145,   35006,  4, }, // Asia/Oral
    {  433, 6961,   51959,  8, }, // Asia/Phnom_Penh
    {  433, 8693,    2563,  9, }, // Asia/Pontianak
    {  433, 4214,  137626,  6, }, // Asia/Pyongyang
    {  433, 5559,    2579,  5, }, // Asia/Qatar
    {  433, 8708,   37005,  8, }, // Asia/Qostanay
    {  433, 4269,    2592,  8, }, // Asia/Qyzylorda
    {  433, 3584,   37022,  4, }, // Asia/Riyadh
    {  433, 4408,    2614,  7, }, // Asia/Sakhalin
    {  433, 7245,    2621,  9, }, // Asia/Samarkand
    {  433, 3279,  137632,  4, }, // Asia/Seoul
    {  433, 1053,   28801,  6, }, // Asia/Shanghai
    {  433, 3300,    2640,  8, }, // Asia/Singapore
    {  433, 4297,    2648, 13, }, // Asia/Srednekolymsk
    {  433, 3263,   60896,  6, }, // Asia/Taipei
    {  433, 4781,    2667,  7, }, // Asia/Tashkent
    {  433, 3915,   35074,  7, }, // Asia/Tbilisi
    {  433, 2772,    2678,  7, }, // Asia/Tehran
    {  433, 1317,    2685,  7, }, // Asia/Thimphu
    {  433, 2821,    2692,  5, }, // Asia/Tokyo
    {  433, 4572,    2697,  5, }, // Asia/Tomsk
    {  433, 1021,  137636, 11, }, // Asia/Ulaanbaatar
    {  433, 1159,  137647,  6, }, // Asia/Urumqi
    {  433, 8722,    2718,  8, }, // Asia/Ust-Nera
    {  433, 6977,   52002,  8, }, // Asia/Vientiane
    {  433, 4710,    2733, 11, }, // Asia/Vladivostok
    {  433, 4828,   37051,  6, }, // Asia/Yakutsk
    {  433, 1235,   28911,  6, }, // Asia/Yangon
    {  433, 3883,    2757, 12, }, // Asia/Yekaterinburg
    {  433, 3747,    2769,  6, }, // Asia/Yerevan
    {  433, 3663,    2775,  4, }, // Atlantic/Azores
    {  433, 5671,   35152,  7, }, // Atlantic/Bermuda
    {  433, 6628,  118582,  6, }, // Atlantic/Canary
    {  433, 3727,    2826, 10, }, // Atlantic/Cape_Verde
    {  433, 1395,    2836,  5, }, // Atlantic/Faroe
    {  433, 8736,   37132,  7, }, // Atlantic/Madeira
    {  433, 2748,   52082,  9, }, // Atlantic/Reykjavik
    {  433, 7272,  137653, 12, }, // Atlantic/South_Georgia
    {  433, 6821,  137665, 11, }, // Atlantic/St_Helena
    {  433, 6644,   52124,  6, }, // Atlantic/Stanley
    {  433, 1670,    2917,  8, }, // Australia/Adelaide
    {  433, 1635,    2925,  7, }, // Australia/Brisbane
    {  433, 1799,  137676, 11, }, // Australia/Broken_Hill
    {  433, 1583,    2943,  6, }, // Australia/Darwin
    {  433, 3637,  137687,  5, }, // Australia/Eucla
    {  433, 1516,    2954,  6, }, // Australia/Hobart
    {  433, 8753,  137692,  8, }, // Australia/Lindeman
    {  433, 1547,  137700,  8, }, // Australia/Lord_Howe
    {  433, 1727,   29120,  8, }, // Australia/Melbourne
    {  433, 1762,  137708,  4, }, // Australia/Perth
    {  433, 1463,   52144,  6, }, // Australia/Sydney
    {  433, 6441,    2993,  9, }, // Europe/Amsterdam
    {  433, 6216,    3002,  7, }, // Europe/Andorra
    {  433, 3620,   29149,  9, }, // Europe/Astrakhan
    {  433, 2300,    3017,  4, }, // Europe/Athens
    {  433, 6336,    3021,  7, }, // Europe/Belgrade
    {  433, 4757,    3028,  6, }, // Europe/Berlin
    {  433, 6488,    3034,  9, }, // Europe/Bratislava
    {  433, 2167,   52164,  8, }, // Europe/Brussels
    {  433, 3928,    3052,  8, }, // Europe/Bucharest
    {  433, 3806,    3060,  8, }, // Europe/Budapest
    {  433, 8772,   35305,  8, }, // Europe/Busingen
    {  433, 2583,  137712,  7, }, // Europe/Chisinau
    {  433, 6289,   35320, 10, }, // Europe/Copenhagen
    {  433, 2338,    3108,  6, }, // Europe/Dublin
    {  433, 6307,    3114,  9, }, // Europe/Gibraltar
    {  433, 8788,    3123,  6, }, // Europe/Guernsey
    {  433, 6612,   52201,  8, }, // Europe/Helsinki
    {  433, 8804,  137719,  8, }, // Europe/Isle_of_Man
    {  433, 1130,   35352,  8, }, // Europe/Istanbul
    {  433, 8823,  118640,  5, }, // Europe/Jersey
    {  433, 3979,    3158, 11, }, // Europe/Kaliningrad
    {  433, 8929,    3169,  5, }, // Europe/Kirov
    {  433, 2527,    3174,  4, }, // Europe/Kyiv
    {  433, 3213,  137727,  6, }, // Europe/Lisbon
    {  433, 6506,   29300,  7, }, // Europe/Ljubljana
    {  433, 2501,    3195,  6, }, // Europe/London
    {  433, 6365,   29313, 10, }, // Europe/Luxembourg
    {  433, 4892,    3212,  6, }, // Europe/Madrid
    {  433, 6397,   29329,  6, }, // Europe/Malta
    {  433, 6582,   35400,  8, }, // Europe/Mariehamn
    {  433, 3693,    3232,  5, }, // Europe/Minsk
    {  433, 6410,    3237,  6, }, // Europe/Monaco
    {  433, 3537,    3243,  6, }, // Europe/Moscow
    {  433, 6458,    3249,  4, }, // Europe/Oslo
    {  433, 4284,  137733,  5, }, // Europe/Paris
    {  433, 6424,    3258,  9, }, // Europe/Podgorica
    {  433, 6275,    3267,  5, }, // Europe/Prague
    {  433, 8837,    3272,  4, }, // Europe/Riga
    {  433, 6324,  137738,  3, }, // Europe/Rome
    {  433, 4331,    3279,  6, }, // Europe/Samara
    {  433, 6470,    3285, 10, }, // Europe/San_Marino
    {  433, 6245,    3295,  7, }, // Europe/Sarajevo
    {  433, 4451,    3302,  7, }, // Europe/Saratov
    {  433, 8849,   52225, 11, }, // Europe/Simferopol
    {  433, 6383,   52236,  6, }, // Europe/Skopje
    {  433, 6599,    3324,  4, }, // Europe/Sofia
    {  433, 6523,   35453,  8, }, // Europe/Stockholm
    {  433, 8867,    3337,  6, }, // Europe/Tallinn
    {  433, 6202,    3343,  6, }, // Europe/Tirane
    {  433, 8882,   52251,  9, }, // Europe/Ulyanovsk
    {  433, 6352,   35474,  5, }, // Europe/Vaduz
    {  433, 6567,    3363,  7, }, // Europe/Vatican
    {  433, 6231,    3370,  4, }, // Europe/Vienna
    {  433, 8899,   52260,  7, }, // Europe/Vilnius
    {  433, 4727,    3381,  9, }, // Europe/Volgograd
    {  433, 3190,    3390,  6, }, // Europe/Warsaw
    {  433, 6261,    3397,  6, }, // Europe/Zagreb
    {  433, 6540,   37310,  5, }, // Europe/Zurich
    {  433, 5078,   35493, 12, }, // Indian/Antananarivo
    {  433, 6947,    3421,  5, }, // Indian/Chagos
    {  433, 6046,  118656,  7, }, // Indian/Christmas
    {  433, 6063,    3438,  5, }, // Indian/Cocos
    {  433, 5029,   35510,  6, }, // Indian/Comoro
    {  433, 6661,    3460,  8, }, // Indian/Kerguelen
    {  433, 7260,   35516,  4, }, // Indian/Mahe
    {  433, 7050,   52332,  7, }, // Indian/Maldives
    {  433, 4069,    3477,  7, }, // Indian/Mauritius
    {  433, 5098,   56584,  7, }, // Indian/Mayotte
    {  433, 7211,   37369,  7, }, // Indian/Reunion
    {  433, 4422,    3498,  4, }, // Pacific/Apia
    {  433,  932,  137741,  7, }, // Pacific/Auckland
    {  433, 3706,   52355,  9, }, // Pacific/Bougainville
    {  433, 3015,    3516,  5, }, // Pacific/Chatham
    {  433, 3157,  137748,  4, }, // Pacific/Chuuk
    {  433, 2237,  118673,  5, }, // Pacific/Easter
    {  433, 7360,   52409,  5, }, // Pacific/Efate
    {  433, 7327,    3541,  7, }, // Pacific/Fakaofo
    {  433, 3902,  118678,  4, }, // Pacific/Fiji
    {  433, 7343,    3552,  8, }, // Pacific/Funafuti
    {  433, 6678,    3560,  9, }, // Pacific/Galapagos
    {  433, 6696,  137752,  7, }, // Pacific/Gambier
    {  433, 3822,    3595, 11, }, // Pacific/Guadalcanal
    {  433, 6018,    3606,  4, }, // Pacific/Guam
    {  433, 2723,   35598,  8, }, // Pacific/Honolulu
    {  433, 3049,   52458,  9, }, // Pacific/Kanton
    {  433, 3998,    3616, 10, }, // Pacific/Kiritimati
    {  433, 7019,  137759,  5, }, // Pacific/Kosrae
    {  433, 2842,  137764,  9, }, // Pacific/Kwajalein
    {  433, 4877,  118697,  6, }, // Pacific/Majuro
    {  433, 4051,  118703,  6, }, // Pacific/Marquesas
    {  433, 8914,   52490,  6, }, // Pacific/Midway
    {  433, 7084,    3680,  5, }, // Pacific/Nauru
    {  433, 7132,   52504,  4, }, // Pacific/Niue
    {  433, 4168,    3689,  7, }, // Pacific/Norfolk
    {  433, 7098,    3696,  5, }, // Pacific/Noumea
    {  433, 3126,   35642,  9, }, // Pacific/Pago_Pago
    {  433, 7155,    3710,  5, }, // Pacific/Palau
    {  433, 7194,   56657,  7, }, // Pacific/Pitcairn
    {  433, 3096,   37471,  6, }, // Pacific/Pohnpei
    {  433, 4807,    3728, 11, }, // Pacific/Port_Moresby
    {  433, 6076,    3739,  9, }, // Pacific/Rarotonga
    {  433, 6031,   29803,  6, }, // Pacific/Saipan
    {  433, 7312,    3754,  5, }, // Pacific/Tahiti
    {  433, 6712,    3759,  6, }, // Pacific/Tarawa
    {  433, 4583,    3765,  9, }, // Pacific/Tongatapu
    {  433, 7392,  137773,  3, }, // Pacific/Wake
    {  433, 7405,    3778,  6, }, // Pacific/Wallis
    {  434,   14,       0,  6, }, // Africa/Asmara Mongolian/Mongolian/China
    {  434, 4435,       6,  8, }, // Africa/Sao_Tome
    {  434, 4252,      14,  8, }, // America/Asuncion
    {  434,  237,      22,  8, }, // America/Atikokan
    {  434, 7623,      30, 17, }, // America/Bahia_Banderas
    {  434, 3868,      47,  6, }, // America/Cancun
    {  434, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  434, 5723,      66,  7, }, // America/Curacao
    {  434,  481,      73, 13, }, // America/Indiana/Knox
    {  434, 7961,      86, 16, }, // America/Indiana/Marengo
    {  434, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  434, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  434, 8038,     139, 14, }, // America/Indiana/Vevay
    {  434, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  434, 8086,     171, 16, }, // America/Indiana/Winamac
    {  434, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  434, 5932,     207, 22, }, // America/Lower_Princes
    {  434, 8220,     229,  6, }, // America/Merida
    {  434, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  434, 8329,     255, 20, }, // America/North_Dakota/Center
    {  434, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  434,  393,     298,  4, }, // America/Nuuk
    {  434, 6899,     302, 16, }, // America/Scoresbysund
    {  434, 8508,     318, 14, }, // America/St_Barthelemy
    {  434, 2061,     332, 10, }, // America/St_Johns
    {  434, 5863,     342,  9, }, // America/St_Kitts
    {  434, 5880,     351,  9, }, // America/St_Lucia
    {  434,  855,     360, 10, }, // America/St_Thomas
    {  434, 5913,     370, 11, }, // America/St_Vincent
    {  434, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  434, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  434, 1185,     413,  9, }, // Asia/Kathmandu
    {  434,  992,     422,  7, }, // Asia/Kolkata
    {  434, 1211,     429,  5, }, // Asia/Macau
    {  434, 1235,     442,  6, }, // Asia/Yangon
    {  434, 1395,     448,  5, }, // Atlantic/Faroe
    {  434, 6821,     453, 10, }, // Atlantic/St_Helena
    {  434, 2527,     463,  4, }, // Europe/Kyiv
    {  434, 7211,     467,  7, }, // Indian/Reunion
    {  434, 3157,     474,  5, }, // Pacific/Chuuk
    {  434, 3096,     479,  7, }, // Pacific/Pohnpei
    {  435, 4771,  137776,  5, }, // Asia/Hovd Mongolian/Mongolian/Mongolia
    {  435, 1021,  137781, 13, }, // Asia/Ulaanbaatar
    {  436,   14,       0,  6, }, // Africa/Asmara Morisyen/Latin/Mauritius
    {  436, 4435,       6,  8, }, // Africa/Sao_Tome
    {  436, 4252,      14,  8, }, // America/Asuncion
    {  436,  237,      22,  8, }, // America/Atikokan
    {  436, 7623,      30, 17, }, // America/Bahia_Banderas
    {  436, 3868,      47,  6, }, // America/Cancun
    {  436, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  436, 5723,      66,  7, }, // America/Curacao
    {  436,  481,      73, 13, }, // America/Indiana/Knox
    {  436, 7961,      86, 16, }, // America/Indiana/Marengo
    {  436, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  436, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  436, 8038,     139, 14, }, // America/Indiana/Vevay
    {  436, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  436, 8086,     171, 16, }, // America/Indiana/Winamac
    {  436, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  436, 5932,     207, 22, }, // America/Lower_Princes
    {  436, 8220,     229,  6, }, // America/Merida
    {  436, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  436, 8329,     255, 20, }, // America/North_Dakota/Center
    {  436, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  436,  393,     298,  4, }, // America/Nuuk
    {  436, 6899,     302, 16, }, // America/Scoresbysund
    {  436, 8508,     318, 14, }, // America/St_Barthelemy
    {  436, 2061,     332, 10, }, // America/St_Johns
    {  436, 5863,     342,  9, }, // America/St_Kitts
    {  436, 5880,     351,  9, }, // America/St_Lucia
    {  436,  855,     360, 10, }, // America/St_Thomas
    {  436, 5913,     370, 11, }, // America/St_Vincent
    {  436, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  436, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  436, 1185,     413,  9, }, // Asia/Kathmandu
    {  436,  992,     422,  7, }, // Asia/Kolkata
    {  436, 1211,     429,  5, }, // Asia/Macau
    {  436, 1235,     442,  6, }, // Asia/Yangon
    {  436, 1395,     448,  5, }, // Atlantic/Faroe
    {  436, 6821,     453, 10, }, // Atlantic/St_Helena
    {  436, 2527,     463,  4, }, // Europe/Kyiv
    {  436, 7211,     467,  7, }, // Indian/Reunion
    {  436, 3157,     474,  5, }, // Pacific/Chuuk
    {  436, 3096,     479,  7, }, // Pacific/Pohnpei
    {  437,   14,       0,  6, }, // Africa/Asmara Mundang/Latin/Cameroon
    {  437, 4435,       6,  8, }, // Africa/Sao_Tome
    {  437, 4252,      14,  8, }, // America/Asuncion
    {  437,  237,      22,  8, }, // America/Atikokan
    {  437, 7623,      30, 17, }, // America/Bahia_Banderas
    {  437, 3868,      47,  6, }, // America/Cancun
    {  437, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  437, 5723,      66,  7, }, // America/Curacao
    {  437,  481,      73, 13, }, // America/Indiana/Knox
    {  437, 7961,      86, 16, }, // America/Indiana/Marengo
    {  437, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  437, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  437, 8038,     139, 14, }, // America/Indiana/Vevay
    {  437, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  437, 8086,     171, 16, }, // America/Indiana/Winamac
    {  437, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  437, 5932,     207, 22, }, // America/Lower_Princes
    {  437, 8220,     229,  6, }, // America/Merida
    {  437, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  437, 8329,     255, 20, }, // America/North_Dakota/Center
    {  437, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  437,  393,     298,  4, }, // America/Nuuk
    {  437, 6899,     302, 16, }, // America/Scoresbysund
    {  437, 8508,     318, 14, }, // America/St_Barthelemy
    {  437, 2061,     332, 10, }, // America/St_Johns
    {  437, 5863,     342,  9, }, // America/St_Kitts
    {  437, 5880,     351,  9, }, // America/St_Lucia
    {  437,  855,     360, 10, }, // America/St_Thomas
    {  437, 5913,     370, 11, }, // America/St_Vincent
    {  437, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  437, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  437, 1185,     413,  9, }, // Asia/Kathmandu
    {  437,  992,     422,  7, }, // Asia/Kolkata
    {  437, 1211,     429,  5, }, // Asia/Macau
    {  437, 1235,     442,  6, }, // Asia/Yangon
    {  437, 1395,     448,  5, }, // Atlantic/Faroe
    {  437, 6821,     453, 10, }, // Atlantic/St_Helena
    {  437, 2527,     463,  4, }, // Europe/Kyiv
    {  437, 7211,     467,  7, }, // Indian/Reunion
    {  437, 3157,     474,  5, }, // Pacific/Chuuk
    {  437, 3096,     479,  7, }, // Pacific/Pohnpei
    {  438,   14,       0,  6, }, // Africa/Asmara Muscogee/Latin/United States
    {  438, 4435,       6,  8, }, // Africa/Sao_Tome
    {  438, 4252,      14,  8, }, // America/Asuncion
    {  438,  237,      22,  8, }, // America/Atikokan
    {  438, 7623,      30, 17, }, // America/Bahia_Banderas
    {  438, 3868,      47,  6, }, // America/Cancun
    {  438, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  438, 5723,      66,  7, }, // America/Curacao
    {  438,  481,      73, 13, }, // America/Indiana/Knox
    {  438, 7961,      86, 16, }, // America/Indiana/Marengo
    {  438, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  438, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  438, 8038,     139, 14, }, // America/Indiana/Vevay
    {  438, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  438, 8086,     171, 16, }, // America/Indiana/Winamac
    {  438, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  438, 5932,     207, 22, }, // America/Lower_Princes
    {  438, 8220,     229,  6, }, // America/Merida
    {  438, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  438, 8329,     255, 20, }, // America/North_Dakota/Center
    {  438, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  438,  393,     298,  4, }, // America/Nuuk
    {  438, 6899,     302, 16, }, // America/Scoresbysund
    {  438, 8508,     318, 14, }, // America/St_Barthelemy
    {  438, 2061,     332, 10, }, // America/St_Johns
    {  438, 5863,     342,  9, }, // America/St_Kitts
    {  438, 5880,     351,  9, }, // America/St_Lucia
    {  438,  855,     360, 10, }, // America/St_Thomas
    {  438, 5913,     370, 11, }, // America/St_Vincent
    {  438, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  438, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  438, 1185,     413,  9, }, // Asia/Kathmandu
    {  438,  992,     422,  7, }, // Asia/Kolkata
    {  438, 1211,     429,  5, }, // Asia/Macau
    {  438, 1235,     442,  6, }, // Asia/Yangon
    {  438, 1395,     448,  5, }, // Atlantic/Faroe
    {  438, 6821,     453, 10, }, // Atlantic/St_Helena
    {  438, 2527,     463,  4, }, // Europe/Kyiv
    {  438, 7211,     467,  7, }, // Indian/Reunion
    {  438, 3157,     474,  5, }, // Pacific/Chuuk
    {  438, 3096,     479,  7, }, // Pacific/Pohnpei
    {  439,   14,       0,  6, }, // Africa/Asmara Nama/Latin/Namibia
    {  439, 4435,       6,  8, }, // Africa/Sao_Tome
    {  439, 4252,      14,  8, }, // America/Asuncion
    {  439,  237,      22,  8, }, // America/Atikokan
    {  439, 7623,      30, 17, }, // America/Bahia_Banderas
    {  439, 3868,      47,  6, }, // America/Cancun
    {  439, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  439, 5723,      66,  7, }, // America/Curacao
    {  439,  481,      73, 13, }, // America/Indiana/Knox
    {  439, 7961,      86, 16, }, // America/Indiana/Marengo
    {  439, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  439, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  439, 8038,     139, 14, }, // America/Indiana/Vevay
    {  439, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  439, 8086,     171, 16, }, // America/Indiana/Winamac
    {  439, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  439, 5932,     207, 22, }, // America/Lower_Princes
    {  439, 8220,     229,  6, }, // America/Merida
    {  439, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  439, 8329,     255, 20, }, // America/North_Dakota/Center
    {  439, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  439,  393,     298,  4, }, // America/Nuuk
    {  439, 6899,     302, 16, }, // America/Scoresbysund
    {  439, 8508,     318, 14, }, // America/St_Barthelemy
    {  439, 2061,     332, 10, }, // America/St_Johns
    {  439, 5863,     342,  9, }, // America/St_Kitts
    {  439, 5880,     351,  9, }, // America/St_Lucia
    {  439,  855,     360, 10, }, // America/St_Thomas
    {  439, 5913,     370, 11, }, // America/St_Vincent
    {  439, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  439, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  439, 1185,     413,  9, }, // Asia/Kathmandu
    {  439,  992,     422,  7, }, // Asia/Kolkata
    {  439, 1211,     429,  5, }, // Asia/Macau
    {  439, 1235,     442,  6, }, // Asia/Yangon
    {  439, 1395,     448,  5, }, // Atlantic/Faroe
    {  439, 6821,     453, 10, }, // Atlantic/St_Helena
    {  439, 2527,     463,  4, }, // Europe/Kyiv
    {  439, 7211,     467,  7, }, // Indian/Reunion
    {  439, 3157,     474,  5, }, // Pacific/Chuuk
    {  439, 3096,     479,  7, }, // Pacific/Pohnpei
    {  440,   14,       0,  6, }, // Africa/Asmara Navajo/Latin/United States
    {  440, 4435,       6,  8, }, // Africa/Sao_Tome
    {  440, 4252,      14,  8, }, // America/Asuncion
    {  440,  237,      22,  8, }, // America/Atikokan
    {  440, 7623,      30, 17, }, // America/Bahia_Banderas
    {  440, 3868,      47,  6, }, // America/Cancun
    {  440, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  440, 5723,      66,  7, }, // America/Curacao
    {  440,  481,      73, 13, }, // America/Indiana/Knox
    {  440, 7961,      86, 16, }, // America/Indiana/Marengo
    {  440, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  440, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  440, 8038,     139, 14, }, // America/Indiana/Vevay
    {  440, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  440, 8086,     171, 16, }, // America/Indiana/Winamac
    {  440, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  440, 5932,     207, 22, }, // America/Lower_Princes
    {  440, 8220,     229,  6, }, // America/Merida
    {  440, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  440, 8329,     255, 20, }, // America/North_Dakota/Center
    {  440, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  440,  393,     298,  4, }, // America/Nuuk
    {  440, 6899,     302, 16, }, // America/Scoresbysund
    {  440, 8508,     318, 14, }, // America/St_Barthelemy
    {  440, 2061,     332, 10, }, // America/St_Johns
    {  440, 5863,     342,  9, }, // America/St_Kitts
    {  440, 5880,     351,  9, }, // America/St_Lucia
    {  440,  855,     360, 10, }, // America/St_Thomas
    {  440, 5913,     370, 11, }, // America/St_Vincent
    {  440, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  440, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  440, 1185,     413,  9, }, // Asia/Kathmandu
    {  440,  992,     422,  7, }, // Asia/Kolkata
    {  440, 1211,     429,  5, }, // Asia/Macau
    {  440, 1235,     442,  6, }, // Asia/Yangon
    {  440, 1395,     448,  5, }, // Atlantic/Faroe
    {  440, 6821,     453, 10, }, // Atlantic/St_Helena
    {  440, 2527,     463,  4, }, // Europe/Kyiv
    {  440, 7211,     467,  7, }, // Indian/Reunion
    {  440, 3157,     474,  5, }, // Pacific/Chuuk
    {  440, 3096,     479,  7, }, // Pacific/Pohnpei
    {  441, 6788,  137794,  8, }, // Africa/Abidjan Nepali/Devanagari/Nepal
    {  441, 6760,  137802,  5, }, // Africa/Accra
    {  441, 5059,  137807, 12, }, // Africa/Addis_Ababa
    {  441, 7420,  137819,  9, }, // Africa/Algiers
    {  441,   14,   29865,  7, }, // Africa/Asmara
    {  441,   44,  137828,  6, }, // Africa/Bamako
    {  441, 5257,  137834,  8, }, // Africa/Bangui
    {  441, 6746,   29884,  6, }, // Africa/Banjul
    {  441, 7435,  113298,  5, }, // Africa/Bissau
    {  441, 4957,  137842, 10, }, // Africa/Blantyre
    {  441, 5287,  137852, 12, }, // Africa/Brazzaville
    {  441, 4922,   29918, 10, }, // Africa/Bujumbura
    {  441, 2320,  137864,  5, }, // Africa/Cairo
    {  441, 4117,  137869, 12, }, // Africa/Casablanca
    {  441, 7449,   88211,  5, }, // Africa/Ceuta
    {  441, 6773,   88216,  8, }, // Africa/Conakry
    {  441, 6840,   29956,  5, }, // Africa/Dakar
    {  441, 5130,  137881, 12, }, // Africa/Dar_es_Salaam
    {  441, 5043,  137893,  7, }, // Africa/Djibouti
    {  441, 5243,  137900,  6, }, // Africa/Douala
    {  441, 5166,  137906,  9, }, // Africa/El_Aaiun
    {  441, 6853,  137915,  8, }, // Africa/Freetown
    {  441, 4906,  137923,  7, }, // Africa/Gaborone
    {  441, 5015,   88274,  5, }, // Africa/Harare
    {  441, 4479,  137930, 11, }, // Africa/Johannesburg
    {  441, 4499,   30022,  4, }, // Africa/Juba
    {  441, 5151,   30026,  8, }, // Africa/Kampala
    {  441, 4524,  137941,  7, }, // Africa/Khartoum
    {  441, 4973,   88295,  6, }, // Africa/Kigali
    {  441, 5306,   30047,  8, }, // Africa/Kinshasa
    {  441, 4744,   88308,  5, }, // Africa/Lagos
    {  441, 5336,  137948, 12, }, // Africa/Libreville
    {  441, 6869,   88323,  3, }, // Africa/Lome
    {  441, 5211,  137960,  8, }, // Africa/Luanda
    {  441, 4939,  137968, 10, }, // Africa/Lubumbashi
    {  441, 5001,   30089,  6, }, // Africa/Lusaka
    {  441, 5322,  137978,  6, }, // Africa/Malabo
    {  441, 4987,  137984,  6, }, // Africa/Maputo
    {  441, 5197,   30107,  6, }, // Africa/Maseru
    {  441, 5182,   88352,  7, }, // Africa/Mbabane
    {  441, 5113,  137990,  8, }, // Africa/Mogadishu
    {  441, 7034,  137998, 10, }, // Africa/Monrovia
    {  441, 3853,  138008,  7, }, // Africa/Nairobi
    {  441, 5271,  138015, 10, }, // Africa/Ndjamena
    {  441, 5354,  138025,  5, }, // Africa/Niamey
    {  441, 6803,  138030,  8, }, // Africa/Nouakchott
    {  441, 6727,  138038,  8, }, // Africa/Ouagadougou
    {  441, 5225,  138046, 11, }, // Africa/Porto-Novo
    {  441, 4435,  138057,  8, }, // Africa/Sao_Tome
    {  441, 2866,   88430,  8, }, // Africa/Tripoli
    {  441, 6554,  113437,  5, }, // Africa/Tunis
    {  441, 4152,  138065,  9, }, // Africa/Windhoek
    {  441,  132,  138074,  4, }, // America/Adak
    {  441, 3346,  138078,  8, }, // America/Anchorage
    {  441, 5607,  138086, 11, }, // America/Anguilla
    {  441, 5624,   30229,  9, }, // America/Antigua
    {  441, 4554,  138097,  9, }, // America/Araguaina
    {  441,  166,  138106, 12, }, // America/Argentina/Buenos_Aires
    {  441,   91,  138118, 10, }, // America/Argentina/Catamarca
    {  441,  270,  138128,  7, }, // America/Argentina/Cordoba
    {  441,  441,   30274,  5, }, // America/Argentina/Jujuy
    {  441, 7462,  113505,  9, }, // America/Argentina/La_Rioja
    {  441,  565,  138135,  8, }, // America/Argentina/Mendoza
    {  441, 7489,  138143, 14, }, // America/Argentina/Rio_Gallegos
    {  441, 7520,   30305,  6, }, // America/Argentina/Salta
    {  441, 7544,  138157,  9, }, // America/Argentina/San_Juan
    {  441, 5580,  138166,  8, }, // America/Argentina/San_Luis
    {  441, 7571,   30328,  7, }, // America/Argentina/Tucuman
    {  441, 7597,  138174,  6, }, // America/Argentina/Ushuaia
    {  441, 5640,  113548,  5, }, // America/Aruba
    {  441, 4252,  138180,  8, }, // America/Asuncion
    {  441,  237,  138188,  8, }, // America/Atikokan
    {  441, 3679,   30363,  6, }, // America/Bahia
    {  441, 7623,  138196, 16, }, // America/Bahia_Banderas
    {  441, 5654,  135744,  9, }, // America/Barbados
    {  441, 7646,   30394,  5, }, // America/Belem
    {  441, 5407,   30399,  5, }, // America/Belize
    {  441, 7660,  138212, 15, }, // America/Blanc-Sablon
    {  441, 7681,  138227, 10, }, // America/Boa_Vista
    {  441, 4361,   88631,  6, }, // America/Bogota
    {  441, 7699,  138237,  4, }, // America/Boise
    {  441, 7713,  138241, 14, }, // America/Cambridge_Bay
    {  441, 7735,  138255, 17, }, // America/Campo_Grande
    {  441, 3868,  113637,  6, }, // America/Cancun
    {  441, 4694,   30470,  7, }, // America/Caracas
    {  441, 4345,  138272,  8, }, // America/Cayenne
    {  441, 5496,  138280,  7, }, // America/Cayman
    {  441, 2260,   88682,  6, }, // America/Chicago
    {  441, 7756,  138287, 10, }, // America/Chihuahua
    {  441, 7774,  138297, 10, }, // America/Ciudad_Juarez
    {  441, 5422,  138307, 11, }, // America/Costa_Rica
    {  441, 7796,   30510,  8, }, // America/Creston
    {  441, 3791,  138318,  8, }, // America/Cuiaba
    {  441, 5723,   88730,  7, }, // America/Curacao
    {  441, 7812,  138326, 13, }, // America/Danmarkshavn
    {  441, 7833,  138339,  3, }, // America/Dawson
    {  441, 7848,  138342,  9, }, // America/Dawson_Creek
    {  441,  805,  138351,  6, }, // America/Denver
    {  441, 3465,  138357,  8, }, // America/Detroit
    {  441, 5739,   88775,  8, }, // America/Dominica
    {  441,  893,  138365,  9, }, // America/Edmonton
    {  441, 7869,  138374,  7, }, // America/Eirunepe
    {  441, 5441,  138381, 13, }, // America/El_Salvador
    {  441, 7886,  113759, 12, }, // America/Fort_Nelson
    {  441, 7906,  129232, 10, }, // America/Fortaleza
    {  441, 7924,   30628,  8, }, // America/Glace_Bay
    {  441, 6881,  138394,  6, }, // America/Goose_Bay
    {  441, 4612,  138400, 12, }, // America/Grand_Turk
    {  441, 5770,   30657,  8, }, // America/Grenada
    {  441, 5786,  138412,  9, }, // America/Guadeloupe
    {  441, 3760,   30674, 10, }, // America/Guatemala
    {  441, 4841,  138421,  9, }, // America/Guayaquil
    {  441, 6932,   30692,  6, }, // America/Guyana
    {  441, 1939,  138430, 13, }, // America/Halifax
    {  441, 2281,  138443,  5, }, // America/Havana
    {  441, 7942,  138448, 12, }, // America/Hermosillo
    {  441,  348,  138460, 14, }, // America/Indiana/Indianapolis
    {  441,  481,  138474, 15, }, // America/Indiana/Knox
    {  441, 7961,  138489, 19, }, // America/Indiana/Marengo
    {  441, 7985,  138508, 21, }, // America/Indiana/Petersburg
    {  441, 8012,  138529, 19, }, // America/Indiana/Tell_City
    {  441, 8038,  138548, 15, }, // America/Indiana/Vevay
    {  441, 8060,  138563,  9, }, // America/Indiana/Vincennes
    {  441, 8086,  138572, 18, }, // America/Indiana/Winamac
    {  441, 8110,  138590,  6, }, // America/Inuvik
    {  441,  660,  138596,  9, }, // America/Iqaluit
    {  441, 2799,  138605,  6, }, // America/Jamaica
    {  441, 5380,  138611,  5, }, // America/Juneau
    {  441,  521,  138616, 10, }, // America/Kentucky/Louisville
    {  441, 8125,  138626, 20, }, // America/Kentucky/Monticello
    {  441, 5704,  138646, 11, }, // America/Kralendijk
    {  441, 4376,   30926,  6, }, // America/La_Paz
    {  441, 7169,  113990,  4, }, // America/Lima
    {  441, 3239,  138657, 11, }, // America/Los_Angeles
    {  441, 5932,  138668, 23, }, // America/Lower_Princes
    {  441, 8153,  138691,  6, }, // America/Maceio
    {  441, 8168,  138697,  8, }, // America/Managua
    {  441, 1908,  138705,  6, }, // America/Manaus
    {  441, 5897,  138711,  8, }, // America/Marigot
    {  441, 5805,   89156,  9, }, // America/Martinique
    {  441, 8184,  138719, 12, }, // America/Matamoros
    {  441, 2917,  138731,  9, }, // America/Mazatlan
    {  441, 8202,   89183,  8, }, // America/Menominee
    {  441, 8220,   31026,  6, }, // America/Merida
    {  441, 8235,  138740, 11, }, // America/Metlakatla
    {  441, 2949,   89203, 13, }, // America/Mexico_City
    {  441, 4391,  138751,  8, }, // America/Miquelon
    {  441, 8254,  138759,  7, }, // America/Moncton
    {  441, 8270,  138766, 10, }, // America/Monterrey
    {  441, 4098,  138776, 12, }, // America/Montevideo
    {  441, 5824,  138788, 11, }, // America/Montserrat
    {  441, 5481,   31100,  7, }, // America/Nassau
    {  441, 2379,  138799,  9, }, // America/New_York
    {  441, 8288,   89185,  3, }, // America/Nome
    {  441, 1850,   89272,  8, }, // America/Noronha
    {  441, 8301,  138808, 19, }, // America/North_Dakota/Beulah
    {  441, 8329,  138827, 21, }, // America/North_Dakota/Center
    {  441, 8357,  138848, 24, }, // America/North_Dakota/New_Salem
    {  441,  393,   31183,  3, }, // America/Nuuk
    {  441, 8388,  114210,  7, }, // America/Ojinaga
    {  441, 2356,   31193,  6, }, // America/Panama
    {  441, 6173,  138872, 10, }, // America/Paramaribo
    {  441, 2973,  114226,  7, }, // America/Phoenix
    {  441, 3945,  138882, 16, }, // America/Port-au-Prince
    {  441, 5954,  138898, 15, }, // America/Port_of_Spain
    {  441, 8404,  138913, 13, }, // America/Porto_Velho
    {  441, 5843,  138926, 13, }, // America/Puerto_Rico
    {  441, 4030,  138939, 13, }, // America/Punta_Arenas
    {  441, 8424,  138952, 14, }, // America/Rankin_Inlet
    {  441, 8445,  138966,  6, }, // America/Recife
    {  441, 1995,   31299,  6, }, // America/Regina
    {  441, 8460,  138972,  7, }, // America/Resolute
    {  441,  695,  138979, 13, }, // America/Rio_Branco
    {  441, 8477,  138992,  9, }, // America/Santarem
    {  441, 2201,  139001,  9, }, // America/Santiago
    {  441, 6111,  139010, 15, }, // America/Santo_Domingo
    {  441, 1878,   89513,  9, }, // America/Sao_Paulo
    {  441, 6899,  139025, 17, }, // America/Scoresbysund
    {  441, 8494,   89535,  6, }, // America/Sitka
    {  441, 8508,  139042, 16, }, // America/St_Barthelemy
    {  441, 2061,  139058, 11, }, // America/St_Johns
    {  441, 5863,  139069, 12, }, // America/St_Kitts
    {  441, 5880,  139081, 12, }, // America/St_Lucia
    {  441,  855,  139093, 10, }, // America/St_Thomas
    {  441, 5913,  139103, 15, }, // America/St_Vincent
    {  441, 8530,  139118, 14, }, // America/Swift_Current
    {  441, 5461,  136322, 12, }, // America/Tegucigalpa
    {  441, 5756,  114424,  4, }, // America/Thule
    {  441,  313,   89637,  7, }, // America/Tijuana
    {  441,  608,  139132,  8, }, // America/Toronto
    {  441, 5688,  139140,  8, }, // America/Tortola
    {  441, 2093,  139148,  9, }, // America/Vancouver
    {  441, 2144,  139157, 10, }, // America/Whitehorse
    {  441,  734,  139167,  9, }, // America/Winnipeg
    {  441, 8552,  114462,  7, }, // America/Yakutat
    {  441, 6001,  139176,  4, }, // Antarctica/Casey
    {  441, 6094,   31545,  5, }, // Antarctica/Davis
    {  441, 6133,  139180, 20, }, // Antarctica/DumontDUrville
    {  441, 8568,  139200,  8, }, // Antarctica/Macquarie
    {  441, 7066,  139208,  5, }, // Antarctica/Mawson
    {  441, 7113,  139213,  9, }, // Antarctica/McMurdo
    {  441, 8589,  114507,  7, }, // Antarctica/Palmer
    {  441, 7226,   89738,  6, }, // Antarctica/Rothera
    {  441, 7295,  139222,  4, }, // Antarctica/Syowa
    {  441, 8607,   89750,  5, }, // Antarctica/Troll
    {  441, 7374,  139226,  7, }, // Antarctica/Vostok
    {  441, 1429,  139233, 11, }, // Arctic/Longyearbyen
    {  441, 5570,   31623,  4, }, // Asia/Aden
    {  441, 5395,   31627,  7, }, // Asia/Almaty
    {  441, 3968,   31634,  6, }, // Asia/Amman
    {  441, 5511,  139244,  7, }, // Asia/Anadyr
    {  441, 5523,  139251,  6, }, // Asia/Aqtau
    {  441, 5368,  139257,  6, }, // Asia/Aqtobe
    {  441,  964,  139263,  8, }, // Asia/Ashgabat
    {  441, 8624,  139271,  6, }, // Asia/Atyrau
    {  441, 3607,   89809,  5, }, // Asia/Baghdad
    {  441, 5534,  139277,  6, }, // Asia/Bahrain
    {  441, 3653,   88728,  4, }, // Asia/Baku
    {  441, 4466,  139283,  5, }, // Asia/Bangkok
    {  441, 3571,  114573,  5, }, // Asia/Barnaul
    {  441, 4086,  139288,  6, }, // Asia/Beirut
    {  441, 3778,  139294,  7, }, // Asia/Bishkek
    {  441, 5989,  139301,  7, }, // Asia/Brunei
    {  441, 4601,  129841,  4, }, // Asia/Chita
    {  441, 4511,  129845,  7, }, // Asia/Colombo
    {  441, 4540,  139308,  9, }, // Asia/Damascus
    {  441, 1093,   89862,  4, }, // Asia/Dhaka
    {  441, 6192,  139317,  6, }, // Asia/Dili
    {  441, 3596,  139323,  5, }, // Asia/Dubai
    {  441, 6159,  139328,  9, }, // Asia/Dushanbe
    {  441, 8636,   31757, 10, }, // Asia/Famagusta
    {  441, 8651,   31767,  4, }, // Asia/Gaza
    {  441, 4795,  139337,  7, }, // Asia/Hebron
    {  441, 1259,  139344, 15, }, // Asia/Ho_Chi_Minh
    {  441, 2704,  139359,  4, }, // Asia/Hong_Kong
    {  441, 4771,  139363,  5, }, // Asia/Hovd
    {  441, 4184,   89934, 10, }, // Asia/Irkutsk
    {  441, 7006,   31803,  8, }, // Asia/Jakarta
    {  441, 6992,   31811,  7, }, // Asia/Jayapura
    {  441, 1290,  136519,  8, }, // Asia/Jerusalem
    {  441, 3560,   89957,  5, }, // Asia/Kabul
    {  441, 4316,  139368,  8, }, // Asia/Kamchatka
    {  441, 4239,   31840,  5, }, // Asia/Karachi
    {  441, 1185,  139376, 10, }, // Asia/Kathmandu
    {  441, 8661,  139386,  8, }, // Asia/Khandyga
    {  441,  992,   89984,  7, }, // Asia/Kolkata
    {  441, 4197,   89991, 15, }, // Asia/Krasnoyarsk
    {  441, 4859,  139394, 11, }, // Asia/Kuala_Lumpur
    {  441, 5976,  139405,  5, }, // Asia/Kuching
    {  441, 5547,  114687,  5, }, // Asia/Kuwait
    {  441, 1211,  139410,  4, }, // Asia/Macau
    {  441, 4017,  139414,  7, }, // Asia/Magadan
    {  441, 1349,   31919,  9, }, // Asia/Makassar
    {  441, 7182,  114710,  5, }, // Asia/Manila
    {  441, 6920,  139421,  8, }, // Asia/Muscat
    {  441, 2554,   90031,  8, }, // Asia/Nicosia
    {  441, 8675,  139429, 12, }, // Asia/Novokuznetsk
    {  441, 4135,  139441, 13, }, // Asia/Novosibirsk
    {  441, 4229,   90068,  6, }, // Asia/Omsk
    {  441, 7145,   90074,  3, }, // Asia/Oral
    {  441, 6961,  139454,  9, }, // Asia/Phnom_Penh
    {  441, 8693,  139463, 10, }, // Asia/Pontianak
    {  441, 4214,  139473,  8, }, // Asia/Pyongyang
    {  441, 5559,  114750,  4, }, // Asia/Qatar
    {  441, 8708,  139481,  7, }, // Asia/Qostanay
    {  441, 4269,  130004, 10, }, // Asia/Qyzylorda
    {  441, 3584,   90127,  5, }, // Asia/Riyadh
    {  441, 4408,   32033,  7, }, // Asia/Sakhalin
    {  441, 7245,  139488,  8, }, // Asia/Samarkand
    {  441, 3279,   90144,  4, }, // Asia/Seoul
    {  441, 1053,  139496,  7, }, // Asia/Shanghai
    {  441, 3300,   32058,  8, }, // Asia/Singapore
    {  441, 4297,  139503, 17, }, // Asia/Srednekolymsk
    {  441, 3263,   32082,  5, }, // Asia/Taipei
    {  441, 4781,  139520,  9, }, // Asia/Tashkent
    {  441, 3915,   32096,  8, }, // Asia/Tbilisi
    {  441, 2772,  139529,  7, }, // Asia/Tehran
    {  441, 1317,  139536,  6, }, // Asia/Thimphu
    {  441, 2821,  136636,  6, }, // Asia/Tokyo
    {  441, 4572,  139542,  7, }, // Asia/Tomsk
    {  441, 1021,  139549,  8, }, // Asia/Ulaanbaatar
    {  441, 1159,   90214,  7, }, // Asia/Urumqi
    {  441, 8722,  139557,  9, }, // Asia/Ust-Nera
    {  441, 6977,  139566,  8, }, // Asia/Vientiane
    {  441, 4710,  139574, 13, }, // Asia/Vladivostok
    {  441, 4828,  139587,  9, }, // Asia/Yakutsk
    {  441, 1235,  139596,  7, }, // Asia/Yangon
    {  441, 3883,  139603, 11, }, // Asia/Yekaterinburg
    {  441, 3747,   32202,  7, }, // Asia/Yerevan
    {  441, 3663,  139614,  6, }, // Atlantic/Azores
    {  441, 5671,  114902,  7, }, // Atlantic/Bermuda
    {  441, 6628,  139620,  8, }, // Atlantic/Canary
    {  441, 3727,  139628,  9, }, // Atlantic/Cape_Verde
    {  441, 1395,  139637,  5, }, // Atlantic/Faroe
    {  441, 8736,  139642,  7, }, // Atlantic/Madeira
    {  441, 2748,  139649,  9, }, // Atlantic/Reykjavik
    {  441, 7272,  139658, 14, }, // Atlantic/South_Georgia
    {  441, 6821,  139672, 12, }, // Atlantic/St_Helena
    {  441, 6644,  139684, 10, }, // Atlantic/Stanley
    {  441, 1670,  139694,  6, }, // Australia/Adelaide
    {  441, 1635,  114960,  9, }, // Australia/Brisbane
    {  441, 1799,   90366, 10, }, // Australia/Broken_Hill
    {  441, 1583,   90376,  7, }, // Australia/Darwin
    {  441, 3637,  139700,  7, }, // Australia/Eucla
    {  441, 1516,  139707,  7, }, // Australia/Hobart
    {  441, 8753,  139714, 11, }, // Australia/Lindeman
    {  441, 1547,  139725,  9, }, // Australia/Lord_Howe
    {  441, 1727,  139734,  8, }, // Australia/Melbourne
    {  441, 1762,   90416,  4, }, // Australia/Perth
    {  441, 1463,  139742,  6, }, // Australia/Sydney
    {  441, 6441,  139748, 11, }, // Europe/Amsterdam
    {  441, 6216,  139759,  9, }, // Europe/Andorra
    {  441, 3620,  139768,  9, }, // Europe/Astrakhan
    {  441, 2300,   32394,  6, }, // Europe/Athens
    {  441, 6336,   32400,  8, }, // Europe/Belgrade
    {  441, 4757,   90451,  6, }, // Europe/Berlin
    {  441, 6488,  139777, 12, }, // Europe/Bratislava
    {  441, 2167,  130280,  8, }, // Europe/Brussels
    {  441, 3928,  139789,  9, }, // Europe/Bucharest
    {  441, 3806,   90488,  9, }, // Europe/Budapest
    {  441, 8772,  139798,  9, }, // Europe/Busingen
    {  441, 2583,  139807,  7, }, // Europe/Chisinau
    {  441, 6289,  139814, 11, }, // Europe/Copenhagen
    {  441, 2338,  115067,  6, }, // Europe/Dublin
    {  441, 6307,   32485, 11, }, // Europe/Gibraltar
    {  441, 8788,  139825,  8, }, // Europe/Guernsey
    {  441, 6612,  139833, 10, }, // Europe/Helsinki
    {  441, 8804,  139843, 13, }, // Europe/Isle_of_Man
    {  441, 1130,  139856,  9, }, // Europe/Istanbul
    {  441, 8823,   90572,  5, }, // Europe/Jersey
    {  441, 3979,  139865, 12, }, // Europe/Kaliningrad
    {  441, 8929,  139877,  5, }, // Europe/Kirov
    {  441, 2527,  139882,  3, }, // Europe/Kyiv
    {  441, 3213,  139885,  7, }, // Europe/Lisbon
    {  441, 6506,  139892,  9, }, // Europe/Ljubljana
    {  441, 2501,  139901,  5, }, // Europe/London
    {  441, 6365,  139906, 11, }, // Europe/Luxembourg
    {  441, 4892,  139917,  9, }, // Europe/Madrid
    {  441, 6397,   32599,  6, }, // Europe/Malta
    {  441, 6582,  139926, 13, }, // Europe/Mariehamn
    {  441, 3693,   32612,  7, }, // Europe/Minsk
    {  441, 6410,   90644,  6, }, // Europe/Monaco
    {  441, 3537,  139939,  5, }, // Europe/Moscow
    {  441, 6458,   90656,  5, }, // Europe/Oslo
    {  441, 4284,   32634,  5, }, // Europe/Paris
    {  441, 6424,  139944,  9, }, // Europe/Podgorica
    {  441, 6275,   90671,  5, }, // Europe/Prague
    {  441, 8837,   32652,  4, }, // Europe/Riga
    {  441, 6324,   90303,  3, }, // Europe/Rome
    {  441, 4331,   32659,  6, }, // Europe/Samara
    {  441, 6470,  139953, 10, }, // Europe/San_Marino
    {  441, 6245,  139963,  8, }, // Europe/Sarajevo
    {  441, 4451,  139971,  7, }, // Europe/Saratov
    {  441, 8849,  130451, 11, }, // Europe/Simferopol
    {  441, 6383,  139978,  7, }, // Europe/Skopje
    {  441, 6599,  115197,  6, }, // Europe/Sofia
    {  441, 6523,  139985,  9, }, // Europe/Stockholm
    {  441, 8867,  139994,  7, }, // Europe/Tallinn
    {  441, 6202,  130482,  6, }, // Europe/Tirane
    {  441, 8882,  140001, 12, }, // Europe/Ulyanovsk
    {  441, 6352,   32744,  5, }, // Europe/Vaduz
    {  441, 6567,  140013,  8, }, // Europe/Vatican
    {  441, 6231,  140021,  5, }, // Europe/Vienna
    {  441, 8899,  140026,  8, }, // Europe/Vilnius
    {  441, 4727,  140034, 11, }, // Europe/Volgograd
    {  441, 3190,  140045,  5, }, // Europe/Warsaw
    {  441, 6261,   32787,  7, }, // Europe/Zagreb
    {  441, 6540,  140050,  5, }, // Europe/Zurich
    {  441, 5078,  140055, 11, }, // Indian/Antananarivo
    {  441, 6947,  115272,  5, }, // Indian/Chagos
    {  441, 6046,  140066,  9, }, // Indian/Christmas
    {  441, 6063,   90849,  5, }, // Indian/Cocos
    {  441, 5029,   90854,  6, }, // Indian/Comoro
    {  441, 6661,  140075, 10, }, // Indian/Kerguelen
    {  441, 7260,   32851,  4, }, // Indian/Mahe
    {  441, 7050,  140085,  9, }, // Indian/Maldives
    {  441, 4069,  140094,  8, }, // Indian/Mauritius
    {  441, 5098,  136958,  8, }, // Indian/Mayotte
    {  441, 7211,  115296,  8, }, // Indian/Reunion
    {  441, 4422,  115304,  5, }, // Pacific/Apia
    {  441,  932,  140102,  9, }, // Pacific/Auckland
    {  441, 3706,  140111, 11, }, // Pacific/Bougainville
    {  441, 3015,   32908,  5, }, // Pacific/Chatham
    {  441, 3157,   32913,  3, }, // Pacific/Chuuk
    {  441, 2237,  140122,  5, }, // Pacific/Easter
    {  441, 7360,  140127,  5, }, // Pacific/Efate
    {  441, 7327,  115334,  7, }, // Pacific/Fakaofo
    {  441, 3902,   32937,  4, }, // Pacific/Fiji
    {  441, 7343,  140132,  8, }, // Pacific/Funafuti
    {  441, 6678,  140140,  8, }, // Pacific/Galapagos
    {  441, 6696,  140148, 10, }, // Pacific/Gambier
    {  441, 3822,  140158, 10, }, // Pacific/Guadalcanal
    {  441, 6018,   32979,  5, }, // Pacific/Guam
    {  441, 2723,   90982,  8, }, // Pacific/Honolulu
    {  441, 3049,  140168,  8, }, // Pacific/Kanton
    {  441, 3998,  140176, 10, }, // Pacific/Kiritimati
    {  441, 7019,  140186,  7, }, // Pacific/Kosrae
    {  441, 2842,  140193, 10, }, // Pacific/Kwajalein
    {  441, 4877,   91013,  6, }, // Pacific/Majuro
    {  441, 4051,   33030,  9, }, // Pacific/Marquesas
    {  441, 8914,   91027,  5, }, // Pacific/Midway
    {  441, 7084,  140203,  5, }, // Pacific/Nauru
    {  441, 7132,  140208,  4, }, // Pacific/Niue
    {  441, 4168,  140212,  7, }, // Pacific/Norfolk
    {  441, 7098,  140219,  6, }, // Pacific/Noumea
    {  441, 3126,   91053,  9, }, // Pacific/Pago_Pago
    {  441, 7155,  140225,  4, }, // Pacific/Palau
    {  441, 7194,  140229,  8, }, // Pacific/Pitcairn
    {  441, 3096,  140237,  6, }, // Pacific/Pohnpei
    {  441, 4807,   91072, 14, }, // Pacific/Port_Moresby
    {  441, 6076,  140243,  8, }, // Pacific/Rarotonga
    {  441, 6031,  140251,  5, }, // Pacific/Saipan
    {  441, 7312,   91101,  6, }, // Pacific/Tahiti
    {  441, 6712,  140256,  4, }, // Pacific/Tarawa
    {  441, 4583,  140260,  7, }, // Pacific/Tongatapu
    {  441, 7392,   33136,  3, }, // Pacific/Wake
    {  441, 7405,   33139,  5, }, // Pacific/Wallis
    {  443,   14,       0,  6, }, // Africa/Asmara Ngiemboon/Latin/Cameroon
    {  443, 4435,       6,  8, }, // Africa/Sao_Tome
    {  443, 4252,      14,  8, }, // America/Asuncion
    {  443,  237,      22,  8, }, // America/Atikokan
    {  443, 7623,      30, 17, }, // America/Bahia_Banderas
    {  443, 3868,      47,  6, }, // America/Cancun
    {  443, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  443, 5723,      66,  7, }, // America/Curacao
    {  443,  481,      73, 13, }, // America/Indiana/Knox
    {  443, 7961,      86, 16, }, // America/Indiana/Marengo
    {  443, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  443, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  443, 8038,     139, 14, }, // America/Indiana/Vevay
    {  443, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  443, 8086,     171, 16, }, // America/Indiana/Winamac
    {  443, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  443, 5932,     207, 22, }, // America/Lower_Princes
    {  443, 8220,     229,  6, }, // America/Merida
    {  443, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  443, 8329,     255, 20, }, // America/North_Dakota/Center
    {  443, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  443,  393,     298,  4, }, // America/Nuuk
    {  443, 6899,     302, 16, }, // America/Scoresbysund
    {  443, 8508,     318, 14, }, // America/St_Barthelemy
    {  443, 2061,     332, 10, }, // America/St_Johns
    {  443, 5863,     342,  9, }, // America/St_Kitts
    {  443, 5880,     351,  9, }, // America/St_Lucia
    {  443,  855,     360, 10, }, // America/St_Thomas
    {  443, 5913,     370, 11, }, // America/St_Vincent
    {  443, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  443, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  443, 1185,     413,  9, }, // Asia/Kathmandu
    {  443,  992,     422,  7, }, // Asia/Kolkata
    {  443, 1211,     429,  5, }, // Asia/Macau
    {  443, 1235,     442,  6, }, // Asia/Yangon
    {  443, 1395,     448,  5, }, // Atlantic/Faroe
    {  443, 6821,     453, 10, }, // Atlantic/St_Helena
    {  443, 2527,     463,  4, }, // Europe/Kyiv
    {  443, 7211,     467,  7, }, // Indian/Reunion
    {  443, 3157,     474,  5, }, // Pacific/Chuuk
    {  443, 3096,     479,  7, }, // Pacific/Pohnpei
    {  444,   14,       0,  6, }, // Africa/Asmara Ngomba/Latin/Cameroon
    {  444, 4435,       6,  8, }, // Africa/Sao_Tome
    {  444, 4252,      14,  8, }, // America/Asuncion
    {  444,  237,      22,  8, }, // America/Atikokan
    {  444, 7623,      30, 17, }, // America/Bahia_Banderas
    {  444, 3868,      47,  6, }, // America/Cancun
    {  444, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  444, 5723,      66,  7, }, // America/Curacao
    {  444,  481,      73, 13, }, // America/Indiana/Knox
    {  444, 7961,      86, 16, }, // America/Indiana/Marengo
    {  444, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  444, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  444, 8038,     139, 14, }, // America/Indiana/Vevay
    {  444, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  444, 8086,     171, 16, }, // America/Indiana/Winamac
    {  444, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  444, 5932,     207, 22, }, // America/Lower_Princes
    {  444, 8220,     229,  6, }, // America/Merida
    {  444, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  444, 8329,     255, 20, }, // America/North_Dakota/Center
    {  444, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  444,  393,     298,  4, }, // America/Nuuk
    {  444, 6899,     302, 16, }, // America/Scoresbysund
    {  444, 8508,     318, 14, }, // America/St_Barthelemy
    {  444, 2061,     332, 10, }, // America/St_Johns
    {  444, 5863,     342,  9, }, // America/St_Kitts
    {  444, 5880,     351,  9, }, // America/St_Lucia
    {  444,  855,     360, 10, }, // America/St_Thomas
    {  444, 5913,     370, 11, }, // America/St_Vincent
    {  444, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  444, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  444, 1185,     413,  9, }, // Asia/Kathmandu
    {  444,  992,     422,  7, }, // Asia/Kolkata
    {  444, 1211,     429,  5, }, // Asia/Macau
    {  444, 1235,     442,  6, }, // Asia/Yangon
    {  444, 1395,     448,  5, }, // Atlantic/Faroe
    {  444, 6821,     453, 10, }, // Atlantic/St_Helena
    {  444, 2527,     463,  4, }, // Europe/Kyiv
    {  444, 7211,     467,  7, }, // Indian/Reunion
    {  444, 3157,     474,  5, }, // Pacific/Chuuk
    {  444, 3096,     479,  7, }, // Pacific/Pohnpei
    {  445, 6788,  140267,  6, }, // Africa/Abidjan Nigerian Pidgin/Latin/Nigeria
    {  445, 6760,  140273,  4, }, // Africa/Accra
    {  445, 5059,  140277,  9, }, // Africa/Addis_Ababa
    {  445, 7420,  140286,  6, }, // Africa/Algiers
    {  445,   14,  140292,  6, }, // Africa/Asmara
    {  445,   44,  140298,  6, }, // Africa/Bamako
    {  445, 5257,  140304,  6, }, // Africa/Bangui
    {  445, 7435,    6195,  5, }, // Africa/Bissau
    {  445, 4957,  140310,  8, }, // Africa/Blantyre
    {  445, 5287,  140318,  8, }, // Africa/Brazzaville
    {  445, 4922,  140326,  9, }, // Africa/Bujumbura
    {  445, 2320,  140335,  5, }, // Africa/Cairo
    {  445, 4117,  140340, 10, }, // Africa/Casablanca
    {  445, 7449,  140350,  5, }, // Africa/Ceuta
    {  445, 6773,  140355,  7, }, // Africa/Conakry
    {  445, 6840,  140362,  5, }, // Africa/Dakar
    {  445, 5130,  140367, 13, }, // Africa/Dar_es_Salaam
    {  445, 5043,  140380,  6, }, // Africa/Djibouti
    {  445, 5243,  140386,  5, }, // Africa/Douala
    {  445, 5166,  140391,  7, }, // Africa/El_Aaiun
    {  445, 6853,  140398,  7, }, // Africa/Freetown
    {  445, 4906,  140405,  8, }, // Africa/Gaborone
    {  445, 5015,  140413,  6, }, // Africa/Harare
    {  445, 4479,  140419, 10, }, // Africa/Johannesburg
    {  445, 4499,  140429,  4, }, // Africa/Juba
    {  445, 5151,  140433,  7, }, // Africa/Kampala
    {  445, 4524,    6305,  6, }, // Africa/Khartoum
    {  445, 4973,  140440,  6, }, // Africa/Kigali
    {  445, 5306,  140446,  9, }, // Africa/Kinshasa
    {  445, 4744,  140455,  5, }, // Africa/Lagos
    {  445, 5336,  140460,  8, }, // Africa/Libreville
    {  445, 6869,  140468,  4, }, // Africa/Lome
    {  445, 5211,  140472,  6, }, // Africa/Luanda
    {  445, 4939,  140478, 10, }, // Africa/Lubumbashi
    {  445, 5001,  140488,  6, }, // Africa/Lusaka
    {  445, 5322,  140494,  6, }, // Africa/Malabo
    {  445, 4987,  140500,  6, }, // Africa/Maputo
    {  445, 5197,  140506,  7, }, // Africa/Maseru
    {  445, 5182,  140513,  7, }, // Africa/Mbabane
    {  445, 5113,  140520,  9, }, // Africa/Mogadishu
    {  445, 7034,  140529,  8, }, // Africa/Monrovia
    {  445, 3853,  140537,  7, }, // Africa/Nairobi
    {  445, 5271,  140544,  8, }, // Africa/Ndjamena
    {  445, 5354,  140552,  5, }, // Africa/Niamey
    {  445, 6803,  140557,  9, }, // Africa/Nouakchott
    {  445, 6727,  140566,  9, }, // Africa/Ouagadougou
    {  445, 5225,  140575,  9, }, // Africa/Porto-Novo
    {  445, 4435,  140584,  8, }, // Africa/Sao_Tome
    {  445, 2866,  140592,  8, }, // Africa/Tripoli
    {  445, 6554,   92332,  5, }, // Africa/Tunis
    {  445, 4152,  140600,  7, }, // Africa/Windhoek
    {  445,  132,  140607,  4, }, // America/Adak
    {  445, 3346,  140611,  8, }, // America/Anchorage
    {  445, 5607,  140619,  7, }, // America/Anguilla
    {  445, 5624,  140626,  8, }, // America/Antigua
    {  445, 4554,  140634, 10, }, // America/Araguaina
    {  445,  166,  140644, 12, }, // America/Argentina/Buenos_Aires
    {  445,   91,  140656,  8, }, // America/Argentina/Catamarca
    {  445,  270,  140664,  7, }, // America/Argentina/Cordoba
    {  445,  441,    6416,  5, }, // America/Argentina/Jujuy
    {  445, 7462,  140671,  9, }, // America/Argentina/La_Rioja
    {  445,  565,  140680,  7, }, // America/Argentina/Mendoza
    {  445, 7489,  140687, 12, }, // America/Argentina/Rio_Gallegos
    {  445, 7520,  140699,  5, }, // America/Argentina/Salta
    {  445, 7544,  140704,  8, }, // America/Argentina/San_Juan
    {  445, 5580,  140712,  8, }, // America/Argentina/San_Luis
    {  445, 7571,  140720,  7, }, // America/Argentina/Tucuman
    {  445, 7597,  140727,  6, }, // America/Argentina/Ushuaia
    {  445, 5640,   92356,  5, }, // America/Aruba
    {  445, 4252,  140733,  8, }, // America/Asuncion
    {  445,  237,  140741,  8, }, // America/Atikokan
    {  445, 3679,      30,  5, }, // America/Bahia
    {  445, 7623,  140749, 15, }, // America/Bahia_Banderas
    {  445, 5654,  140764,  8, }, // America/Barbados
    {  445, 7646,  140772,  5, }, // America/Belem
    {  445, 5407,  140777,  5, }, // America/Belize
    {  445, 7660,  140782, 12, }, // America/Blanc-Sablon
    {  445, 7681,  140794,  9, }, // America/Boa_Vista
    {  445, 7699,  140803,  5, }, // America/Boise
    {  445, 7713,  140808, 10, }, // America/Cambridge_Bay
    {  445, 7735,  140818, 12, }, // America/Campo_Grande
    {  445, 3868,    6564,  6, }, // America/Cancun
    {  445, 4694,  140830,  7, }, // America/Caracas
    {  445, 4345,  140837,  5, }, // America/Cayenne
    {  445, 5496,  140842,  5, }, // America/Cayman
    {  445, 2260,  140847,  7, }, // America/Chicago
    {  445, 7756,  140854,  9, }, // America/Chihuahua
    {  445, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  445, 5422,  140863, 10, }, // America/Costa_Rica
    {  445, 7796,  140873,  8, }, // America/Creston
    {  445, 3791,  140881,  7, }, // America/Cuiaba
    {  445, 5723,  140888,  8, }, // America/Curacao
    {  445, 7812,  140896, 12, }, // America/Danmarkshavn
    {  445, 7833,  140908,  6, }, // America/Dawson
    {  445, 7848,  140914, 11, }, // America/Dawson_Creek
    {  445,  805,  140925,  6, }, // America/Denver
    {  445, 3465,  140931,  7, }, // America/Detroit
    {  445, 5739,  140938,  8, }, // America/Dominica
    {  445,  893,  140946, 10, }, // America/Edmonton
    {  445, 7869,  140956,  8, }, // America/Eirunepe
    {  445, 5441,  140964, 10, }, // America/El_Salvador
    {  445, 7886,  140974, 11, }, // America/Fort_Nelson
    {  445, 7906,  140985,  9, }, // America/Fortaleza
    {  445, 7924,  140994,  7, }, // America/Glace_Bay
    {  445, 6881,  141001,  6, }, // America/Goose_Bay
    {  445, 4612,  141007,  9, }, // America/Grand_Turk
    {  445, 5770,  141016,  7, }, // America/Grenada
    {  445, 5786,  141023,  9, }, // America/Guadeloupe
    {  445, 3760,  141032,  9, }, // America/Guatemala
    {  445, 4841,  141041,  8, }, // America/Guayaquil
    {  445, 6932,  141049,  6, }, // America/Guyana
    {  445, 1939,  141055,  8, }, // America/Halifax
    {  445, 2281,  141063,  6, }, // America/Havana
    {  445, 7942,  141069,  9, }, // America/Hermosillo
    {  445,  348,  141078, 13, }, // America/Indiana/Indianapolis
    {  445,  481,  141091, 13, }, // America/Indiana/Knox
    {  445, 7961,  141104, 17, }, // America/Indiana/Marengo
    {  445, 7985,  141121, 17, }, // America/Indiana/Petersburg
    {  445, 8012,  141138, 18, }, // America/Indiana/Tell_City
    {  445, 8038,  141156, 14, }, // America/Indiana/Vevay
    {  445, 8060,  141170, 15, }, // America/Indiana/Vincennes
    {  445, 8086,  141185, 16, }, // America/Indiana/Winamac
    {  445, 8110,  141201,  6, }, // America/Inuvik
    {  445,  660,  141207,  7, }, // America/Iqaluit
    {  445, 2799,   92416,  7, }, // America/Jamaica
    {  445, 5380,  141214,  4, }, // America/Juneau
    {  445,  521,  141218,  6, }, // America/Kentucky/Louisville
    {  445, 8125,  141224, 22, }, // America/Kentucky/Monticello
    {  445, 5704,  141246, 11, }, // America/Kralendijk
    {  445, 7169,  141257,  4, }, // America/Lima
    {  445, 3239,  141261, 12, }, // America/Los_Angeles
    {  445, 5932,  141273, 20, }, // America/Lower_Princes
    {  445, 8153,  141293,  7, }, // America/Maceio
    {  445, 8168,   97907,  7, }, // America/Managua
    {  445, 1908,  141300,  6, }, // America/Manaus
    {  445, 5897,  141306,  7, }, // America/Marigot
    {  445, 5805,  141313,  7, }, // America/Martinique
    {  445, 8184,  141320,  9, }, // America/Matamoros
    {  445, 2917,  141329,  8, }, // America/Mazatlan
    {  445, 8202,  141337, 10, }, // America/Menominee
    {  445, 8220,  141347,  7, }, // America/Merida
    {  445, 8235,  141354, 10, }, // America/Metlakatla
    {  445, 2949,  141364, 13, }, // America/Mexico_City
    {  445, 4391,  141377,  7, }, // America/Miquelon
    {  445, 8254,  141384,  8, }, // America/Moncton
    {  445, 8270,  141392,  8, }, // America/Monterrey
    {  445, 4098,  141400, 10, }, // America/Montevideo
    {  445, 5824,  141410,  9, }, // America/Montserrat
    {  445, 5481,  141419,  6, }, // America/Nassau
    {  445, 2379,  141425,  7, }, // America/New_York
    {  445, 8288,  141432,  4, }, // America/Nome
    {  445, 1850,  141436,  7, }, // America/Noronha
    {  445, 8301,  141443, 18, }, // America/North_Dakota/Beulah
    {  445, 8329,  141461, 19, }, // America/North_Dakota/Center
    {  445, 8357,  141480, 22, }, // America/North_Dakota/New_Salem
    {  445,  393,     298,  4, }, // America/Nuuk
    {  445, 8388,  141502,  8, }, // America/Ojinaga
    {  445, 2356,  141510,  6, }, // America/Panama
    {  445, 6173,  141516, 10, }, // America/Paramaribo
    {  445, 2973,  141526,  6, }, // America/Phoenix
    {  445, 3945,  141532, 11, }, // America/Port-au-Prince
    {  445, 5954,  141543, 12, }, // America/Port_of_Spain
    {  445, 8404,  141555, 11, }, // America/Porto_Velho
    {  445, 5843,  141566, 12, }, // America/Puerto_Rico
    {  445, 4030,  141578, 13, }, // America/Punta_Arenas
    {  445, 8424,  141591, 12, }, // America/Rankin_Inlet
    {  445, 8445,  141603,  6, }, // America/Recife
    {  445, 1995,  141609,  6, }, // America/Regina
    {  445, 8460,  141615,  8, }, // America/Resolute
    {  445,  695,  141623, 10, }, // America/Rio_Branco
    {  445, 8477,  141633,  8, }, // America/Santarem
    {  445, 2201,  141641,  8, }, // America/Santiago
    {  445, 6111,  141649, 13, }, // America/Santo_Domingo
    {  445, 1878,  141662,  9, }, // America/Sao_Paulo
    {  445, 6899,  141671, 11, }, // America/Scoresbysund
    {  445, 8508,  141682, 14, }, // America/St_Barthelemy
    {  445, 2061,  141696,  8, }, // America/St_Johns
    {  445, 5863,  126003,  9, }, // America/St_Kitts
    {  445, 5880,  141704, 11, }, // America/St_Lucia
    {  445,  855,  141715, 10, }, // America/St_Thomas
    {  445, 5913,  141725, 12, }, // America/St_Vincent
    {  445, 8530,  141737, 13, }, // America/Swift_Current
    {  445, 5461,  141750, 11, }, // America/Tegucigalpa
    {  445, 5756,  141761,  4, }, // America/Thule
    {  445,  313,  141765,  8, }, // America/Tijuana
    {  445,  608,  141773,  7, }, // America/Toronto
    {  445, 5688,  141780,  6, }, // America/Tortola
    {  445, 2093,  141786,  7, }, // America/Vancouver
    {  445, 2144,  141793,  7, }, // America/Whitehorse
    {  445,  734,  141800,  7, }, // America/Winnipeg
    {  445, 8552,  141807,  7, }, // America/Yakutat
    {  445, 6001,  141814,  4, }, // Antarctica/Casey
    {  445, 6094,  141818,  5, }, // Antarctica/Davis
    {  445, 6133,  141823, 14, }, // Antarctica/DumontDUrville
    {  445, 8568,  141837,  7, }, // Antarctica/Macquarie
    {  445, 7066,  141844,  6, }, // Antarctica/Mawson
    {  445, 7113,  141850,  7, }, // Antarctica/McMurdo
    {  445, 8589,  141857,  4, }, // Antarctica/Palmer
    {  445, 7226,  141861,  7, }, // Antarctica/Rothera
    {  445, 7295,  141868,  5, }, // Antarctica/Syowa
    {  445, 8607,  123271,  4, }, // Antarctica/Troll
    {  445, 7374,  141873,  7, }, // Antarctica/Vostok
    {  445, 1429,  141880, 12, }, // Arctic/Longyearbyen
    {  445, 5570,  141892,  4, }, // Asia/Aden
    {  445, 5395,  141896,  6, }, // Asia/Almaty
    {  445, 3968,    5487,  4, }, // Asia/Amman
    {  445, 5511,  141902,  7, }, // Asia/Anadyr
    {  445, 5523,  141909,  5, }, // Asia/Aqtau
    {  445, 5368,  141914,  6, }, // Asia/Aqtobe
    {  445,  964,  141920,  8, }, // Asia/Ashgabat
    {  445, 8624,  141928,  6, }, // Asia/Atyrau
    {  445, 3607,  141934,  6, }, // Asia/Baghdad
    {  445, 5534,  141940,  6, }, // Asia/Bahrain
    {  445, 3653,  141946,  4, }, // Asia/Baku
    {  445, 4466,  141950,  6, }, // Asia/Bangkok
    {  445, 3571,  141956,  7, }, // Asia/Barnaul
    {  445, 4086,  141963,  5, }, // Asia/Beirut
    {  445, 3778,  141968,  7, }, // Asia/Bishkek
    {  445, 5989,  141975,  6, }, // Asia/Brunei
    {  445, 4601,  141981,  5, }, // Asia/Chita
    {  445, 4511,  141986,  7, }, // Asia/Colombo
    {  445, 4540,  141993,  8, }, // Asia/Damascus
    {  445, 1093,  142001,  4, }, // Asia/Dhaka
    {  445, 6192,  142005,  4, }, // Asia/Dili
    {  445, 6159,  142009,  8, }, // Asia/Dushanbe
    {  445, 8636,  142017,  7, }, // Asia/Famagusta
    {  445, 8651,   91440,  4, }, // Asia/Gaza
    {  445, 4795,  142024,  7, }, // Asia/Hebron
    {  445, 1259,  142031, 15, }, // Asia/Ho_Chi_Minh
    {  445, 2704,  142046,  9, }, // Asia/Hong_Kong
    {  445, 4184,  142055,  7, }, // Asia/Irkutsk
    {  445, 7006,  142062,  6, }, // Asia/Jakarta
    {  445, 6992,  142068,  8, }, // Asia/Jayapura
    {  445, 1290,  142076,  9, }, // Asia/Jerusalem
    {  445, 4316,  142085,  9, }, // Asia/Kamchatka
    {  445, 4239,  142094,  7, }, // Asia/Karachi
    {  445, 1185,  142101,  8, }, // Asia/Kathmandu
    {  445, 8661,  142109,  7, }, // Asia/Khandyga
    {  445,  992,  142116,  7, }, // Asia/Kolkata
    {  445, 4197,  142123, 10, }, // Asia/Krasnoyarsk
    {  445, 4859,  142133, 11, }, // Asia/Kuala_Lumpur
    {  445, 5547,  142144,  5, }, // Asia/Kuwait
    {  445, 1211,  142149,  5, }, // Asia/Macau
    {  445, 4017,  142154,  7, }, // Asia/Magadan
    {  445, 1349,  142161,  6, }, // Asia/Makassar
    {  445, 7182,  142167,  6, }, // Asia/Manila
    {  445, 6920,  142173,  6, }, // Asia/Muscat
    {  445, 2554,    5620,  7, }, // Asia/Nicosia
    {  445, 8675,  142179, 12, }, // Asia/Novokuznetsk
    {  445, 4135,  142191, 10, }, // Asia/Novosibirsk
    {  445, 4229,  142201,  4, }, // Asia/Omsk
    {  445, 7145,  142205,  5, }, // Asia/Oral
    {  445, 6961,  142210,  9, }, // Asia/Phnom_Penh
    {  445, 8693,  142219,  9, }, // Asia/Pontianak
    {  445, 4214,  142228,  9, }, // Asia/Pyongyang
    {  445, 5559,  142237,  4, }, // Asia/Qatar
    {  445, 8708,  142241,  8, }, // Asia/Qostanay
    {  445, 4269,  142249,  9, }, // Asia/Qyzylorda
    {  445, 3584,  142258,  5, }, // Asia/Riyadh
    {  445, 4408,  142263,  8, }, // Asia/Sakhalin
    {  445, 7245,  142271,  8, }, // Asia/Samarkand
    {  445, 3279,  142279,  3, }, // Asia/Seoul
    {  445, 1053,  142282,  8, }, // Asia/Shanghai
    {  445, 3300,  142290,  7, }, // Asia/Singapore
    {  445, 4297,  142297, 14, }, // Asia/Srednekolymsk
    {  445, 3263,  142311,  6, }, // Asia/Taipei
    {  445, 4781,  142317,  8, }, // Asia/Tashkent
    {  445, 3915,  142325,  7, }, // Asia/Tbilisi
    {  445, 2772,  142332,  5, }, // Asia/Tehran
    {  445, 1317,  142337,  5, }, // Asia/Thimphu
    {  445, 2821,  142342,  5, }, // Asia/Tokyo
    {  445, 1021,  142347,  8, }, // Asia/Ulaanbaatar
    {  445, 1159,  142355,  8, }, // Asia/Urumqi
    {  445, 8722,  142363,  9, }, // Asia/Ust-Nera
    {  445, 6977,  142372,  9, }, // Asia/Vientiane
    {  445, 4710,  142381, 11, }, // Asia/Vladivostok
    {  445, 4828,  142392,  7, }, // Asia/Yakutsk
    {  445, 1235,  142399,  6, }, // Asia/Yangon
    {  445, 3883,  142405, 12, }, // Asia/Yekaterinburg
    {  445, 3747,  142417,  8, }, // Asia/Yerevan
    {  445, 3663,  142425,  4, }, // Atlantic/Azores
    {  445, 5671,  142429,  7, }, // Atlantic/Bermuda
    {  445, 6628,  142436,  6, }, // Atlantic/Canary
    {  445, 3727,  142442,  7, }, // Atlantic/Cape_Verde
    {  445, 1395,  142449,  6, }, // Atlantic/Faroe
    {  445, 8736,  142455,  6, }, // Atlantic/Madeira
    {  445, 2748,  142461,  9, }, // Atlantic/Reykjavik
    {  445, 7272,  142470, 11, }, // Atlantic/South_Georgia
    {  445, 6821,  142481, 12, }, // Atlantic/St_Helena
    {  445, 6644,  142493,  6, }, // Atlantic/Stanley
    {  445, 1670,  142499,  6, }, // Australia/Adelaide
    {  445, 1635,  142505,  7, }, // Australia/Brisbane
    {  445, 1799,  142512, 10, }, // Australia/Broken_Hill
    {  445, 1583,  142522,  6, }, // Australia/Darwin
    {  445, 3637,  142528,  5, }, // Australia/Eucla
    {  445, 1516,  142533,  5, }, // Australia/Hobart
    {  445, 8753,  142538,  8, }, // Australia/Lindeman
    {  445, 1547,  142546,  7, }, // Australia/Lord_Howe
    {  445, 1727,  142553,  7, }, // Australia/Melbourne
    {  445, 1762,  142560,  4, }, // Australia/Perth
    {  445, 1463,  142564,  5, }, // Australia/Sydney
    {  445, 6441,  142569,  8, }, // Europe/Amsterdam
    {  445, 6216,  142577,  7, }, // Europe/Andorra
    {  445, 3620,  142584,  8, }, // Europe/Astrakhan
    {  445, 2300,  142592,  5, }, // Europe/Athens
    {  445, 6336,  142597,  8, }, // Europe/Belgrade
    {  445, 4757,  142605,  5, }, // Europe/Berlin
    {  445, 6488,  142610, 10, }, // Europe/Bratislava
    {  445, 2167,  142620,  7, }, // Europe/Brussels
    {  445, 3928,  142627,  8, }, // Europe/Bucharest
    {  445, 3806,  142635,  8, }, // Europe/Budapest
    {  445, 8772,  142643,  8, }, // Europe/Busingen
    {  445, 2583,  142651,  8, }, // Europe/Chisinau
    {  445, 6289,  142659, 10, }, // Europe/Copenhagen
    {  445, 2338,  142669,  7, }, // Europe/Dublin
    {  445, 6307,  142676,  9, }, // Europe/Gibraltar
    {  445, 8788,  142685,  6, }, // Europe/Guernsey
    {  445, 6612,  142691,  8, }, // Europe/Helsinki
    {  445, 8804,  142699, 10, }, // Europe/Isle_of_Man
    {  445, 1130,  142709,  8, }, // Europe/Istanbul
    {  445, 8823,  142717,  5, }, // Europe/Jersey
    {  445, 3979,  142722, 11, }, // Europe/Kaliningrad
    {  445, 8929,  142733,  5, }, // Europe/Kirov
    {  445, 2527,  142738,  4, }, // Europe/Kyiv
    {  445, 3213,  142742,  6, }, // Europe/Lisbon
    {  445, 6506,  142748,  8, }, // Europe/Ljubljana
    {  445, 2501,  142756,  7, }, // Europe/London
    {  445, 6365,  142763, 11, }, // Europe/Luxembourg
    {  445, 6397,  142774,  6, }, // Europe/Malta
    {  445, 6582,  142780,  9, }, // Europe/Mariehamn
    {  445, 6410,  142789,  7, }, // Europe/Monaco
    {  445, 3537,  142796,  6, }, // Europe/Moscow
    {  445, 6458,  142802,  5, }, // Europe/Oslo
    {  445, 4284,  142807,  5, }, // Europe/Paris
    {  445, 6424,  142812,  9, }, // Europe/Podgorica
    {  445, 6275,    8083,  4, }, // Europe/Prague
    {  445, 8837,  142821,  4, }, // Europe/Riga
    {  445, 6324,    5946,  3, }, // Europe/Rome
    {  445, 4331,  142825,  6, }, // Europe/Samara
    {  445, 6470,  142831, 10, }, // Europe/San_Marino
    {  445, 6245,  142841,  9, }, // Europe/Sarajevo
    {  445, 4451,  142850,  7, }, // Europe/Saratov
    {  445, 8849,  142857, 10, }, // Europe/Simferopol
    {  445, 6383,  142867,  7, }, // Europe/Skopje
    {  445, 6599,   21081,  5, }, // Europe/Sofia
    {  445, 6523,  142874,  8, }, // Europe/Stockholm
    {  445, 8867,  142882,  5, }, // Europe/Tallinn
    {  445, 6202,  142887,  6, }, // Europe/Tirane
    {  445, 8882,  142893,  9, }, // Europe/Ulyanovsk
    {  445, 6567,  142902,  7, }, // Europe/Vatican
    {  445, 6231,  142909,  6, }, // Europe/Vienna
    {  445, 8899,   44482,  7, }, // Europe/Vilnius
    {  445, 4727,  142915,  9, }, // Europe/Volgograd
    {  445, 3190,  142924,  5, }, // Europe/Warsaw
    {  445, 6261,  142929,  6, }, // Europe/Zagreb
    {  445, 6540,  142935,  5, }, // Europe/Zurich
    {  445, 5078,  142940, 12, }, // Indian/Antananarivo
    {  445, 6947,  142952,  6, }, // Indian/Chagos
    {  445, 6046,  142958,  7, }, // Indian/Christmas
    {  445, 6063,   92904,  5, }, // Indian/Cocos
    {  445, 5029,  142965,  9, }, // Indian/Comoro
    {  445, 6661,  142974,  9, }, // Indian/Kerguelen
    {  445, 7260,  142983,  5, }, // Indian/Mahe
    {  445, 7050,  142988,  8, }, // Indian/Maldives
    {  445, 4069,  142996,  8, }, // Indian/Mauritius
    {  445, 5098,  143004,  5, }, // Indian/Mayotte
    {  445, 7211,  143009,  8, }, // Indian/Reunion
    {  445, 4422,  143017,  4, }, // Pacific/Apia
    {  445,  932,  143021,  7, }, // Pacific/Auckland
    {  445, 3706,  143028, 10, }, // Pacific/Bougainville
    {  445, 3015,  143038,  6, }, // Pacific/Chatham
    {  445, 3157,  143044,  4, }, // Pacific/Chuuk
    {  445, 2237,  143048,  4, }, // Pacific/Easter
    {  445, 7360,  143052,  5, }, // Pacific/Efate
    {  445, 7327,  143057,  7, }, // Pacific/Fakaofo
    {  445, 3902,  143064,  4, }, // Pacific/Fiji
    {  445, 7343,  143068,  8, }, // Pacific/Funafuti
    {  445, 6678,  143076,  9, }, // Pacific/Galapagos
    {  445, 6696,  143085,  7, }, // Pacific/Gambier
    {  445, 3822,  143092, 11, }, // Pacific/Guadalcanal
    {  445, 3049,  143103, 10, }, // Pacific/Kanton
    {  445, 3998,  143113,  9, }, // Pacific/Kiritimati
    {  445, 7019,  143122,  7, }, // Pacific/Kosrae
    {  445, 2842,  143129,  8, }, // Pacific/Kwajalein
    {  445, 4877,  143137,  6, }, // Pacific/Majuro
    {  445, 4051,  143143,  9, }, // Pacific/Marquesas
    {  445, 8914,  143152,  5, }, // Pacific/Midway
    {  445, 7084,  143157,  5, }, // Pacific/Nauru
    {  445, 7132,  141425,  3, }, // Pacific/Niue
    {  445, 4168,  143162,  9, }, // Pacific/Norfolk
    {  445, 7098,  143171,  6, }, // Pacific/Noumea
    {  445, 3126,  143177,  9, }, // Pacific/Pago_Pago
    {  445, 7155,  143186,  5, }, // Pacific/Palau
    {  445, 7194,  143191,  6, }, // Pacific/Pitcairn
    {  445, 3096,  143197,  7, }, // Pacific/Pohnpei
    {  445, 4807,  143204, 12, }, // Pacific/Port_Moresby
    {  445, 6076,  143216,  9, }, // Pacific/Rarotonga
    {  445, 7312,  143225,  6, }, // Pacific/Tahiti
    {  445, 6712,  143231,  6, }, // Pacific/Tarawa
    {  445, 4583,  143237,  9, }, // Pacific/Tongatapu
    {  445, 7392,  143246,  3, }, // Pacific/Wake
    {  445, 7405,  143249,  4, }, // Pacific/Wallis
    {  446, 6788,  143253,  7, }, // Africa/Abidjan Nko/Nko/Guinea
    {  446, 6760,  143260,  5, }, // Africa/Accra
    {  446, 5059,  143265, 14, }, // Africa/Addis_Ababa
    {  446, 7420,  143279,  6, }, // Africa/Algiers
    {  446,   14,  143285,  7, }, // Africa/Asmara
    {  446,   44,  143292,  7, }, // Africa/Bamako
    {  446, 5257,  143299,  7, }, // Africa/Bangui
    {  446, 6746,  143306,  6, }, // Africa/Banjul
    {  446, 7435,  143312,  7, }, // Africa/Bissau
    {  446, 4957,  143319,  9, }, // Africa/Blantyre
    {  446, 5287,  143328,  9, }, // Africa/Brazzaville
    {  446, 4922,  143337, 11, }, // Africa/Bujumbura
    {  446, 2320,  143348,  4, }, // Africa/Cairo
    {  446, 4117,  143352, 11, }, // Africa/Casablanca
    {  446, 7449,  143363,  7, }, // Africa/Ceuta
    {  446, 6773,  143370,  8, }, // Africa/Conakry
    {  446, 6840,  143378,  9, }, // Africa/Dakar
    {  446, 5130,  143387, 12, }, // Africa/Dar_es_Salaam
    {  446, 5043,  143399,  7, }, // Africa/Djibouti
    {  446, 5243,  143406,  6, }, // Africa/Douala
    {  446, 5166,  143412,  6, }, // Africa/El_Aaiun
    {  446, 6853,  143418,  7, }, // Africa/Freetown
    {  446, 4906,  143425,  8, }, // Africa/Gaborone
    {  446, 5015,  143433,  6, }, // Africa/Harare
    {  446, 4479,  143439, 13, }, // Africa/Johannesburg
    {  446, 4499,  143452,  5, }, // Africa/Juba
    {  446, 5151,  143457,  7, }, // Africa/Kampala
    {  446, 4524,  143464,  7, }, // Africa/Khartoum
    {  446, 4973,  143471,  8, }, // Africa/Kigali
    {  446, 5306,  143479,  8, }, // Africa/Kinshasa
    {  446, 4744,  143487,  6, }, // Africa/Lagos
    {  446, 5336,  143493,  9, }, // Africa/Libreville
    {  446, 6869,  143502,  5, }, // Africa/Lome
    {  446, 5211,  143507,  8, }, // Africa/Luanda
    {  446, 4939,  143515, 10, }, // Africa/Lubumbashi
    {  446, 5001,  143525,  6, }, // Africa/Lusaka
    {  446, 5322,  143531,  6, }, // Africa/Malabo
    {  446, 4987,  143537,  7, }, // Africa/Maputo
    {  446, 5197,  143544,  7, }, // Africa/Maseru
    {  446, 5182,  143551,  6, }, // Africa/Mbabane
    {  446, 5113,  143557, 10, }, // Africa/Mogadishu
    {  446, 7034,  143567, 11, }, // Africa/Monrovia
    {  446, 3853,  143578,  8, }, // Africa/Nairobi
    {  446, 5271,  143586,  8, }, // Africa/Ndjamena
    {  446, 5354,  143594,  5, }, // Africa/Niamey
    {  446, 6803,  143599,  9, }, // Africa/Nouakchott
    {  446, 6727,  143608, 10, }, // Africa/Ouagadougou
    {  446, 5225,  143618, 13, }, // Africa/Porto-Novo
    {  446, 4435,  143631,  9, }, // Africa/Sao_Tome
    {  446, 2866,  143640, 16, }, // Africa/Tripoli
    {  446, 6554,  143656,  5, }, // Africa/Tunis
    {  446, 4152,  143661,  6, }, // Africa/Windhoek
    {  446,  132,  143667,  4, }, // America/Adak
    {  446, 3346,  143671,  8, }, // America/Anchorage
    {  446, 5607,  143679,  8, }, // America/Anguilla
    {  446, 5624,  143687,  8, }, // America/Antigua
    {  446, 4554,  143695, 22, }, // America/Araguaina
    {  446,  166,  143717,  8, }, // America/Argentina/Buenos_Aires
    {  446,   91,  143725, 10, }, // America/Argentina/Catamarca
    {  446,  270,  143735,  9, }, // America/Argentina/Cordoba
    {  446,  441,  143744,  7, }, // America/Argentina/Jujuy
    {  446, 7462,  143751, 10, }, // America/Argentina/La_Rioja
    {  446,  565,  143761,  8, }, // America/Argentina/Mendoza
    {  446, 7489,  143769, 16, }, // America/Argentina/Rio_Gallegos
    {  446, 7520,  143785,  7, }, // America/Argentina/Salta
    {  446, 7544,  143792, 11, }, // America/Argentina/San_Juan
    {  446, 5580,  143803, 10, }, // America/Argentina/San_Luis
    {  446, 7571,  143813,  7, }, // America/Argentina/Tucuman
    {  446, 7597,  143820,  7, }, // America/Argentina/Ushuaia
    {  446, 5640,  143827,  6, }, // America/Aruba
    {  446, 4252,  143833, 10, }, // America/Asuncion
    {  446,  237,  143843,  9, }, // America/Atikokan
    {  446, 3679,  143852,  7, }, // America/Bahia
    {  446, 7623,  143859, 19, }, // America/Bahia_Banderas
    {  446, 5654,  143878,  6, }, // America/Barbados
    {  446, 7646,  143884,  4, }, // America/Belem
    {  446, 5407,  143888,  9, }, // America/Belize
    {  446, 7660,  143897, 15, }, // America/Blanc-Sablon
    {  446, 7681,  143912, 14, }, // America/Boa_Vista
    {  446, 4361,  143926,  7, }, // America/Bogota
    {  446, 7699,  143933,  5, }, // America/Boise
    {  446, 7713,  143938, 12, }, // America/Cambridge_Bay
    {  446, 7735,  143950, 14, }, // America/Campo_Grande
    {  446, 3868,  143964,  7, }, // America/Cancun
    {  446, 4694,  143971,  6, }, // America/Caracas
    {  446, 4345,  143977,  7, }, // America/Cayenne
    {  446, 5496,  143984,  8, }, // America/Cayman
    {  446, 2260,  143992,  8, }, // America/Chicago
    {  446, 7756,  144000,  9, }, // America/Chihuahua
    {  446, 7774,  144009, 15, }, // America/Ciudad_Juarez
    {  446, 5422,  144024, 11, }, // America/Costa_Rica
    {  446, 7796,  144035,  9, }, // America/Creston
    {  446, 3791,  144044,  8, }, // America/Cuiaba
    {  446, 5723,  144052,  8, }, // America/Curacao
    {  446, 7812,  144060, 14, }, // America/Danmarkshavn
    {  446, 7833,  144074,  8, }, // America/Dawson
    {  446, 7848,  144082, 13, }, // America/Dawson_Creek
    {  446,  805,  144095,  7, }, // America/Denver
    {  446, 3465,  144102,  7, }, // America/Detroit
    {  446, 5739,  144109,  7, }, // America/Dominica
    {  446,  893,  144116, 10, }, // America/Edmonton
    {  446, 7869,  144126,  6, }, // America/Eirunepe
    {  446, 5441,  144132, 10, }, // America/El_Salvador
    {  446, 7886,  144142, 13, }, // America/Fort_Nelson
    {  446, 7906,  144155, 12, }, // America/Fortaleza
    {  446, 7924,  144167, 10, }, // America/Glace_Bay
    {  446, 6881,  144177, 10, }, // America/Goose_Bay
    {  446, 4612,  144187, 12, }, // America/Grand_Turk
    {  446, 5770,  144199,  8, }, // America/Grenada
    {  446, 5786,  144207, 10, }, // America/Guadeloupe
    {  446, 3760,  144217, 13, }, // America/Guatemala
    {  446, 4841,  144230,  9, }, // America/Guayaquil
    {  446, 6932,  144239,  7, }, // America/Guyana
    {  446, 1939,  144246,  8, }, // America/Halifax
    {  446, 2281,  144254,  7, }, // America/Havana
    {  446, 7942,  144261, 11, }, // America/Hermosillo
    {  446,  348,  144272, 13, }, // America/Indiana/Indianapolis
    {  446,  481,  144285, 17, }, // America/Indiana/Knox
    {  446, 7961,  144302, 20, }, // America/Indiana/Marengo
    {  446, 7985,  144322, 23, }, // America/Indiana/Petersburg
    {  446, 8012,  144345, 19, }, // America/Indiana/Tell_City
    {  446, 8038,  144364, 18, }, // America/Indiana/Vevay
    {  446, 8060,  144382, 19, }, // America/Indiana/Vincennes
    {  446, 8086,  144401, 17, }, // America/Indiana/Winamac
    {  446, 8110,  144418,  7, }, // America/Inuvik
    {  446,  660,  144425,  8, }, // America/Iqaluit
    {  446, 2799,  144433,  8, }, // America/Jamaica
    {  446, 5380,  144441,  7, }, // America/Juneau
    {  446,  521,  144448,  9, }, // America/Kentucky/Louisville
    {  446, 8125,  144457, 21, }, // America/Kentucky/Monticello
    {  446, 5704,  144478, 11, }, // America/Kralendijk
    {  446, 4376,  144489,  7, }, // America/La_Paz
    {  446, 7169,  144496,  5, }, // America/Lima
    {  446, 3239,  144501, 12, }, // America/Los_Angeles
    {  446, 5932,  144513, 19, }, // America/Lower_Princes
    {  446, 8153,  144532,  7, }, // America/Maceio
    {  446, 8168,  144539,  9, }, // America/Managua
    {  446, 1908,  144548,  6, }, // America/Manaus
    {  446, 5897,  144554,  8, }, // America/Marigot
    {  446, 5805,  144562,  8, }, // America/Martinique
    {  446, 8184,  144570,  7, }, // America/Matamoros
    {  446, 2917,  144577, 10, }, // America/Mazatlan
    {  446, 8202,  144587,  8, }, // America/Menominee
    {  446, 8220,  144595,  7, }, // America/Merida
    {  446, 8235,  144602, 12, }, // America/Metlakatla
    {  446, 2949,  144614,  8, }, // America/Mexico_City
    {  446, 4391,  144622,  8, }, // America/Miquelon
    {  446, 8254,  144630,  9, }, // America/Moncton
    {  446, 8270,  144639,  8, }, // America/Monterrey
    {  446, 4098,  144647, 13, }, // America/Montevideo
    {  446, 5824,  144660,  9, }, // America/Montserrat
    {  446, 5481,  144669,  7, }, // America/Nassau
    {  446, 2379,  144676,  8, }, // America/New_York
    {  446, 8288,  144684,  5, }, // America/Nome
    {  446, 1850,  144689,  9, }, // America/Noronha
    {  446, 8301,  144698, 24, }, // America/North_Dakota/Beulah
    {  446, 8329,  144722, 24, }, // America/North_Dakota/Center
    {  446, 8357,  144746, 26, }, // America/North_Dakota/New_Salem
    {  446,  393,  144772,  4, }, // America/Nuuk
    {  446, 8388,  144776,  8, }, // America/Ojinaga
    {  446, 2356,  144784,  6, }, // America/Panama
    {  446, 6173,  144790, 10, }, // America/Paramaribo
    {  446, 2973,  144800,  7, }, // America/Phoenix
    {  446, 3945,  144807, 13, }, // America/Port-au-Prince
    {  446, 5954,  144820, 12, }, // America/Port_of_Spain
    {  446, 8404,  144832, 14, }, // America/Porto_Velho
    {  446, 5843,  144846, 12, }, // America/Puerto_Rico
    {  446, 4030,  144858, 13, }, // America/Punta_Arenas
    {  446, 8424,  144871, 13, }, // America/Rankin_Inlet
    {  446, 8445,  144884,  5, }, // America/Recife
    {  446, 1995,  144889,  8, }, // America/Regina
    {  446, 8460,  144897,  8, }, // America/Resolute
    {  446,  695,  144905, 14, }, // America/Rio_Branco
    {  446, 8477,  144919,  8, }, // America/Santarem
    {  446, 2201,  144927, 11, }, // America/Santiago
    {  446, 6111,  144938, 13, }, // America/Santo_Domingo
    {  446, 1878,  144951,  9, }, // America/Sao_Paulo
    {  446, 6899,  144960, 16, }, // America/Scoresbysund
    {  446, 8494,  144976,  7, }, // America/Sitka
    {  446, 8508,  144983, 15, }, // America/St_Barthelemy
    {  446, 2061,  144998, 20, }, // America/St_Johns
    {  446, 5863,  145018, 12, }, // America/St_Kitts
    {  446, 5880,  145030,  8, }, // America/St_Lucia
    {  446,  855,  145038,  9, }, // America/St_Thomas
    {  446, 5913,  145047, 12, }, // America/St_Vincent
    {  446, 8530,  145059, 11, }, // America/Swift_Current
    {  446, 5461,  145070, 15, }, // America/Tegucigalpa
    {  446, 5756,  145085,  5, }, // America/Thule
    {  446,  313,  145090,  8, }, // America/Tijuana
    {  446,  608,  145098,  8, }, // America/Toronto
    {  446, 5688,  145106,  9, }, // America/Tortola
    {  446, 2093,  145115, 10, }, // America/Vancouver
    {  446, 2144,  145125,  8, }, // America/Whitehorse
    {  446,  734,  145133,  9, }, // America/Winnipeg
    {  446, 8552,  145142,  7, }, // America/Yakutat
    {  446, 6001,  145149, 16, }, // Antarctica/Casey
    {  446, 6094,  145165,  6, }, // Antarctica/Davis
    {  446, 6133,  145171, 13, }, // Antarctica/DumontDUrville
    {  446, 8568,  145184,  6, }, // Antarctica/Macquarie
    {  446, 7066,  145190,  8, }, // Antarctica/Mawson
    {  446, 7113,  145198, 10, }, // Antarctica/McMurdo
    {  446, 8589,  145208,  7, }, // Antarctica/Palmer
    {  446, 7226,  145215,  7, }, // Antarctica/Rothera
    {  446, 7295,  145222,  5, }, // Antarctica/Syowa
    {  446, 8607,  145227,  4, }, // Antarctica/Troll
    {  446, 7374,  145231,  8, }, // Antarctica/Vostok
    {  446, 1429,  145239, 17, }, // Arctic/Longyearbyen
    {  446, 5570,  145256,  5, }, // Asia/Aden
    {  446, 5395,  145261, 10, }, // Asia/Almaty
    {  446, 3968,  145271,  6, }, // Asia/Amman
    {  446, 5511,  145277,  6, }, // Asia/Anadyr
    {  446, 5523,  145283,  8, }, // Asia/Aqtau
    {  446, 5368,  145291, 10, }, // Asia/Aqtobe
    {  446,  964,  145301,  9, }, // Asia/Ashgabat
    {  446, 8624,  145310,  7, }, // Asia/Atyrau
    {  446, 3607,  145317,  8, }, // Asia/Baghdad
    {  446, 5534,  145325,  7, }, // Asia/Bahrain
    {  446, 3653,  145332,  5, }, // Asia/Baku
    {  446, 4466,  145337,  6, }, // Asia/Bangkok
    {  446, 3571,  145343,  7, }, // Asia/Barnaul
    {  446, 4086,  145350,  7, }, // Asia/Beirut
    {  446, 3778,  145357,  5, }, // Asia/Bishkek
    {  446, 5989,  145362,  8, }, // Asia/Brunei
    {  446, 4601,  145370,  5, }, // Asia/Chita
    {  446, 4511,  145375,  8, }, // Asia/Colombo
    {  446, 4540,  145383,  4, }, // Asia/Damascus
    {  446, 1093,  145387,  4, }, // Asia/Dhaka
    {  446, 6192,  145391,  4, }, // Asia/Dili
    {  446, 3596,  145395,  8, }, // Asia/Dubai
    {  446, 6159,  145403,  8, }, // Asia/Dushanbe
    {  446, 8636,  145411, 11, }, // Asia/Famagusta
    {  446, 8651,  145422,  7, }, // Asia/Gaza
    {  446, 4795,  145429,  8, }, // Asia/Hebron
    {  446, 1259,  145437, 14, }, // Asia/Ho_Chi_Minh
    {  446, 2704,  145451, 11, }, // Asia/Hong_Kong
    {  446, 4771,  145462,  5, }, // Asia/Hovd
    {  446, 4184,  145467, 11, }, // Asia/Irkutsk
    {  446, 7006,  145478,  8, }, // Asia/Jakarta
    {  446, 6992,  145486,  8, }, // Asia/Jayapura
    {  446, 1290,  145494, 11, }, // Asia/Jerusalem
    {  446, 3560,  145505,  5, }, // Asia/Kabul
    {  446, 4316,  145510, 12, }, // Asia/Kamchatka
    {  446, 4239,  145522,  6, }, // Asia/Karachi
    {  446, 1185,  145528, 10, }, // Asia/Kathmandu
    {  446, 8661,  145538, 10, }, // Asia/Khandyga
    {  446,  992,  145548,  9, }, // Asia/Kolkata
    {  446, 4197,  145557, 17, }, // Asia/Krasnoyarsk
    {  446, 4859,  145574, 12, }, // Asia/Kuala_Lumpur
    {  446, 5976,  145586,  7, }, // Asia/Kuching
    {  446, 5547,  145593,  7, }, // Asia/Kuwait
    {  446, 1211,  145600,  6, }, // Asia/Macau
    {  446, 4017,  145606,  8, }, // Asia/Magadan
    {  446, 1349,  145614,  6, }, // Asia/Makassar
    {  446, 7182,  145620,  5, }, // Asia/Manila
    {  446, 6920,  145625,  7, }, // Asia/Muscat
    {  446, 2554,  145632,  9, }, // Asia/Nicosia
    {  446, 8675,  145641, 16, }, // Asia/Novokuznetsk
    {  446, 4135,  145657, 14, }, // Asia/Novosibirsk
    {  446, 4229,  145671,  5, }, // Asia/Omsk
    {  446, 7145,  145676,  4, }, // Asia/Oral
    {  446, 6961,  145680, 11, }, // Asia/Phnom_Penh
    {  446, 8693,  145691, 10, }, // Asia/Pontianak
    {  446, 4214,  145701, 12, }, // Asia/Pyongyang
    {  446, 5559,  145713,  4, }, // Asia/Qatar
    {  446, 8708,  145717,  8, }, // Asia/Qostanay
    {  446, 4269,  145725, 11, }, // Asia/Qyzylorda
    {  446, 3584,  145736,  5, }, // Asia/Riyadh
    {  446, 4408,  145741,  6, }, // Asia/Sakhalin
    {  446, 7245,  145747,  9, }, // Asia/Samarkand
    {  446, 3279,  145756,  4, }, // Asia/Seoul
    {  446, 1053,  145760,  9, }, // Asia/Shanghai
    {  446, 3300,  145769,  9, }, // Asia/Singapore
    {  446, 4297,  145778, 17, }, // Asia/Srednekolymsk
    {  446, 3263,  145795,  7, }, // Asia/Taipei
    {  446, 4781,  145802,  9, }, // Asia/Tashkent
    {  446, 3915,  145811,  7, }, // Asia/Tbilisi
    {  446, 2772,  145818,  7, }, // Asia/Tehran
    {  446, 1317,  145825,  6, }, // Asia/Thimphu
    {  446, 2821,  145831,  7, }, // Asia/Tokyo
    {  446, 4572,  145838,  6, }, // Asia/Tomsk
    {  446, 1021,  145844,  9, }, // Asia/Ulaanbaatar
    {  446, 1159,  145853,  7, }, // Asia/Urumqi
    {  446, 8722,  145860,  9, }, // Asia/Ust-Nera
    {  446, 6977,  145869, 10, }, // Asia/Vientiane
    {  446, 4710,  145879, 15, }, // Asia/Vladivostok
    {  446, 4828,  145894,  8, }, // Asia/Yakutsk
    {  446, 1235,  145902,  8, }, // Asia/Yangon
    {  446, 3883,  145910, 14, }, // Asia/Yekaterinburg
    {  446, 3747,  145924,  8, }, // Asia/Yerevan
    {  446, 3663,  145932,  7, }, // Atlantic/Azores
    {  446, 5671,  145939,  9, }, // Atlantic/Bermuda
    {  446, 6628,  145948,  6, }, // Atlantic/Canary
    {  446, 3727,  145954, 14, }, // Atlantic/Cape_Verde
    {  446, 1395,  145968,  6, }, // Atlantic/Faroe
    {  446, 8736,  145974,  5, }, // Atlantic/Madeira
    {  446, 2748,  145979, 12, }, // Atlantic/Reykjavik
    {  446, 7272,  145991, 18, }, // Atlantic/South_Georgia
    {  446, 6821,  146009, 10, }, // Atlantic/St_Helena
    {  446, 6644,  146019,  8, }, // Atlantic/Stanley
    {  446, 1670,  146027,  7, }, // Australia/Adelaide
    {  446, 1635,  146034, 10, }, // Australia/Brisbane
    {  446, 1799,  146044, 12, }, // Australia/Broken_Hill
    {  446, 1583,  146056,  8, }, // Australia/Darwin
    {  446, 3637,  146064,  6, }, // Australia/Eucla
    {  446, 1516,  146070,  6, }, // Australia/Hobart
    {  446, 8753,  146076,  9, }, // Australia/Lindeman
    {  446, 1547,  146085, 10, }, // Australia/Lord_Howe
    {  446, 1727,  146095,  8, }, // Australia/Melbourne
    {  446, 1762,  146103,  4, }, // Australia/Perth
    {  446, 1463,  146107,  7, }, // Australia/Sydney
    {  446, 6441,  146114, 11, }, // Europe/Amsterdam
    {  446, 6216,  146125,  7, }, // Europe/Andorra
    {  446, 3620,  146132, 12, }, // Europe/Astrakhan
    {  446, 2300,  146144,  4, }, // Europe/Athens
    {  446, 6336,  146148,  9, }, // Europe/Belgrade
    {  446, 4757,  146157,  8, }, // Europe/Berlin
    {  446, 6488,  146165, 13, }, // Europe/Bratislava
    {  446, 2167,  146178,  9, }, // Europe/Brussels
    {  446, 3928,  146187,  7, }, // Europe/Bucharest
    {  446, 3806,  146194,  8, }, // Europe/Budapest
    {  446, 8772,  146202, 10, }, // Europe/Busingen
    {  446, 2583,  146212,  6, }, // Europe/Chisinau
    {  446, 6289,  146218,  8, }, // Europe/Copenhagen
    {  446, 2338,  146226,  7, }, // Europe/Dublin
    {  446, 6307,  146233, 11, }, // Europe/Gibraltar
    {  446, 8788,  146244,  9, }, // Europe/Guernsey
    {  446, 6612,  146253, 10, }, // Europe/Helsinki
    {  446, 8804,  146263,  8, }, // Europe/Isle_of_Man
    {  446, 1130,  146271,  9, }, // Europe/Istanbul
    {  446, 8823,  146280,  7, }, // Europe/Jersey
    {  446, 3979,  146287, 12, }, // Europe/Kaliningrad
    {  446, 8929,  146299,  6, }, // Europe/Kirov
    {  446, 2527,  146305,  6, }, // Europe/Kyiv
    {  446, 3213,  146311,  8, }, // Europe/Lisbon
    {  446, 6506,  146319, 10, }, // Europe/Ljubljana
    {  446, 2501,  146329,  7, }, // Europe/London
    {  446, 6365,  146336, 12, }, // Europe/Luxembourg
    {  446, 4892,  146348,  6, }, // Europe/Madrid
    {  446, 6397,  146354,  5, }, // Europe/Malta
    {  446, 6582,  146359,  7, }, // Europe/Mariehamn
    {  446, 3693,  146366,  5, }, // Europe/Minsk
    {  446, 6410,  146371,  7, }, // Europe/Monaco
    {  446, 3537,  146378,  7, }, // Europe/Moscow
    {  446, 6458,  146385,  6, }, // Europe/Oslo
    {  446, 4284,  146391,  6, }, // Europe/Paris
    {  446, 6424,  146397, 11, }, // Europe/Podgorica
    {  446, 6275,  146408,  5, }, // Europe/Prague
    {  446, 8837,  146413,  6, }, // Europe/Riga
    {  446, 6324,  146419,  3, }, // Europe/Rome
    {  446, 4331,  146422,  6, }, // Europe/Samara
    {  446, 6470,  146428, 11, }, // Europe/San_Marino
    {  446, 6245,  146439, 10, }, // Europe/Sarajevo
    {  446, 4451,  146449,  7, }, // Europe/Saratov
    {  446, 8849,  146456,  9, }, // Europe/Simferopol
    {  446, 6383,  146465,  9, }, // Europe/Skopje
    {  446, 6599,  146474,  7, }, // Europe/Sofia
    {  446, 6523,  146481,  8, }, // Europe/Stockholm
    {  446, 8867,  146489,  6, }, // Europe/Tallinn
    {  446, 6202,  146495,  6, }, // Europe/Tirane
    {  446, 8882,  146501, 12, }, // Europe/Ulyanovsk
    {  446, 6352,  146513,  8, }, // Europe/Vaduz
    {  446, 6567,  146521,  9, }, // Europe/Vatican
    {  446, 6231,  146530,  6, }, // Europe/Vienna
    {  446, 8899,  146536, 10, }, // Europe/Vilnius
    {  446, 4727,  146546, 12, }, // Europe/Volgograd
    {  446, 3190,  146558, 11, }, // Europe/Warsaw
    {  446, 6261,  146569,  8, }, // Europe/Zagreb
    {  446, 6540,  146577,  6, }, // Europe/Zurich
    {  446, 5078,  146583, 13, }, // Indian/Antananarivo
    {  446, 6947,  146596,  6, }, // Indian/Chagos
    {  446, 6046,  146602,  8, }, // Indian/Christmas
    {  446, 6063,  146610,  5, }, // Indian/Cocos
    {  446, 5029,  146615,  6, }, // Indian/Comoro
    {  446, 6661,  146621, 11, }, // Indian/Kerguelen
    {  446, 7260,  146632,  5, }, // Indian/Mahe
    {  446, 7050,  146637,  8, }, // Indian/Maldives
    {  446, 4069,  146645,  5, }, // Indian/Mauritius
    {  446, 5098,  146650,  5, }, // Indian/Mayotte
    {  446, 7211,  146655,  8, }, // Indian/Reunion
    {  446, 4422,  146663,  6, }, // Pacific/Apia
    {  446,  932,  146669,  7, }, // Pacific/Auckland
    {  446, 3706,  146676, 10, }, // Pacific/Bougainville
    {  446, 3015,  146686,  4, }, // Pacific/Chatham
    {  446, 3157,  146690,  4, }, // Pacific/Chuuk
    {  446, 2237,  146694,  6, }, // Pacific/Easter
    {  446, 7360,  146700,  6, }, // Pacific/Efate
    {  446, 7327,  146706,  7, }, // Pacific/Fakaofo
    {  446, 3902,  146713,  4, }, // Pacific/Fiji
    {  446, 7343,  146717,  9, }, // Pacific/Funafuti
    {  446, 6678,  146726, 10, }, // Pacific/Galapagos
    {  446, 6696,  146736,  9, }, // Pacific/Gambier
    {  446, 3822,  146745, 12, }, // Pacific/Guadalcanal
    {  446, 6018,  146757,  7, }, // Pacific/Guam
    {  446, 3049,  146764,  7, }, // Pacific/Kanton
    {  446, 3998,  146771, 10, }, // Pacific/Kiritimati
    {  446, 7019,  146781,  7, }, // Pacific/Kosrae
    {  446, 2842,  146788, 11, }, // Pacific/Kwajalein
    {  446, 4877,  146799,  7, }, // Pacific/Majuro
    {  446, 4051,  146806,  6, }, // Pacific/Marquesas
    {  446, 8914,  146812,  7, }, // Pacific/Midway
    {  446, 7084,  146819,  6, }, // Pacific/Nauru
    {  446, 7132,  146825,  6, }, // Pacific/Niue
    {  446, 4168,  146831,  8, }, // Pacific/Norfolk
    {  446, 7098,  146839,  7, }, // Pacific/Noumea
    {  446, 3126,  146846, 13, }, // Pacific/Pago_Pago
    {  446, 7155,  146859,  5, }, // Pacific/Palau
    {  446, 7194,  146864,  7, }, // Pacific/Pitcairn
    {  446, 3096,  146871,  8, }, // Pacific/Pohnpei
    {  446, 4807,  146879, 16, }, // Pacific/Port_Moresby
    {  446, 6076,  146895, 11, }, // Pacific/Rarotonga
    {  446, 6031,  146906,  7, }, // Pacific/Saipan
    {  446, 7312,  146913,  6, }, // Pacific/Tahiti
    {  446, 6712,  146919,  6, }, // Pacific/Tarawa
    {  446, 4583,  146925, 10, }, // Pacific/Tongatapu
    {  446, 7392,  146935,  5, }, // Pacific/Wake
    {  446, 7405,  146940,  6, }, // Pacific/Wallis
    {  447,   14,       0,  6, }, // Africa/Asmara Northern Luri/Arabic/Iran
    {  447, 4435,       6,  8, }, // Africa/Sao_Tome
    {  447, 4252,      14,  8, }, // America/Asuncion
    {  447,  237,      22,  8, }, // America/Atikokan
    {  447, 7623,      30, 17, }, // America/Bahia_Banderas
    {  447, 3868,      47,  6, }, // America/Cancun
    {  447, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  447, 5723,      66,  7, }, // America/Curacao
    {  447,  481,      73, 13, }, // America/Indiana/Knox
    {  447, 7961,      86, 16, }, // America/Indiana/Marengo
    {  447, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  447, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  447, 8038,     139, 14, }, // America/Indiana/Vevay
    {  447, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  447, 8086,     171, 16, }, // America/Indiana/Winamac
    {  447, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  447, 5932,     207, 22, }, // America/Lower_Princes
    {  447, 8220,     229,  6, }, // America/Merida
    {  447, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  447, 8329,     255, 20, }, // America/North_Dakota/Center
    {  447, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  447,  393,     298,  4, }, // America/Nuuk
    {  447, 6899,     302, 16, }, // America/Scoresbysund
    {  447, 8508,     318, 14, }, // America/St_Barthelemy
    {  447, 2061,     332, 10, }, // America/St_Johns
    {  447, 5863,     342,  9, }, // America/St_Kitts
    {  447, 5880,     351,  9, }, // America/St_Lucia
    {  447,  855,     360, 10, }, // America/St_Thomas
    {  447, 5913,     370, 11, }, // America/St_Vincent
    {  447, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  447, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  447, 1185,     413,  9, }, // Asia/Kathmandu
    {  447,  992,     422,  7, }, // Asia/Kolkata
    {  447, 1211,     429,  5, }, // Asia/Macau
    {  447, 1235,     442,  6, }, // Asia/Yangon
    {  447, 1395,     448,  5, }, // Atlantic/Faroe
    {  447, 6821,     453, 10, }, // Atlantic/St_Helena
    {  447, 2527,     463,  4, }, // Europe/Kyiv
    {  447, 7211,     467,  7, }, // Indian/Reunion
    {  447, 3157,     474,  5, }, // Pacific/Chuuk
    {  447, 3096,     479,  7, }, // Pacific/Pohnpei
    {  449,   14,       0,  6, }, // Africa/Asmara Northern Sami/Latin/Norway
    {  449, 4435,       6,  8, }, // Africa/Sao_Tome
    {  449, 4252,      14,  8, }, // America/Asuncion
    {  449,  237,      22,  8, }, // America/Atikokan
    {  449, 7623,      30, 17, }, // America/Bahia_Banderas
    {  449, 3868,      47,  6, }, // America/Cancun
    {  449, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  449, 5723,      66,  7, }, // America/Curacao
    {  449,  481,      73, 13, }, // America/Indiana/Knox
    {  449, 7961,      86, 16, }, // America/Indiana/Marengo
    {  449, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  449, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  449, 8038,     139, 14, }, // America/Indiana/Vevay
    {  449, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  449, 8086,     171, 16, }, // America/Indiana/Winamac
    {  449, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  449, 5932,     207, 22, }, // America/Lower_Princes
    {  449, 8220,     229,  6, }, // America/Merida
    {  449, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  449, 8329,     255, 20, }, // America/North_Dakota/Center
    {  449, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  449,  393,     298,  4, }, // America/Nuuk
    {  449, 1878,   26580,  9, }, // America/Sao_Paulo
    {  449, 6899,     302, 16, }, // America/Scoresbysund
    {  449, 8508,   35804, 16, }, // America/St_Barthelemy
    {  449, 2061,     332, 10, }, // America/St_Johns
    {  449, 5863,     342,  9, }, // America/St_Kitts
    {  449, 5880,     351,  9, }, // America/St_Lucia
    {  449,  855,     360, 10, }, // America/St_Thomas
    {  449, 5913,     370, 11, }, // America/St_Vincent
    {  449, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  449, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  449, 1185,     413,  9, }, // Asia/Kathmandu
    {  449,  992,     422,  7, }, // Asia/Kolkata
    {  449, 1211,     429,  5, }, // Asia/Macau
    {  449, 1235,     442,  6, }, // Asia/Yangon
    {  449, 1395,     448,  5, }, // Atlantic/Faroe
    {  449, 6821,     453, 10, }, // Atlantic/St_Helena
    {  449, 2527,     463,  4, }, // Europe/Kyiv
    {  449, 7211,     467,  7, }, // Indian/Reunion
    {  449, 3157,     474,  5, }, // Pacific/Chuuk
    {  449, 3096,     479,  7, }, // Pacific/Pohnpei
    {  450, 2320,    4572,  5, }, // Africa/Cairo Northern Sami/Latin/Finland
    {  450, 7812,  146946, 15, }, // America/Danmarkshavn
    {  450, 8301,  146961, 20, }, // America/North_Dakota/Beulah
    {  450, 8329,  146981, 22, }, // America/North_Dakota/Center
    {  450, 8357,  147003, 23, }, // America/North_Dakota/New_Salem
    {  450, 4540,   64184,  8, }, // Asia/Damascus
    {  450, 4184,  147026,  6, }, // Asia/Irkutsk
    {  450, 4316,   33378,  8, }, // Asia/Kamchatka
    {  450, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  450, 8675,  147032, 11, }, // Asia/Novokuznetsk
    {  450, 4408,   33444,  7, }, // Asia/Sakhalin
    {  450, 2772,    7724,  7, }, // Asia/Tehran
    {  450, 1021,  147043, 10, }, // Asia/Ulaanbaatar
    {  450, 4828,  147053,  6, }, // Asia/Yakutsk
    {  450, 1235,   58486,  7, }, // Asia/Yangon
    {  450, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  450, 3747,    7780,  7, }, // Asia/Yerevan
    {  450, 3663,  147059,  6, }, // Atlantic/Azores
    {  450, 6628,  147065,  7, }, // Atlantic/Canary
    {  450, 3727,   57248,  9, }, // Atlantic/Cape_Verde
    {  450, 7272,  147072, 13, }, // Atlantic/South_Georgia
    {  450, 2300,   93081,  6, }, // Europe/Athens
    {  450, 6336,   26883,  7, }, // Europe/Belgrade
    {  450, 2167,   22634,  7, }, // Europe/Brussels
    {  450, 3928,   26897,  8, }, // Europe/Bucharest
    {  450, 6289,   58525,  9, }, // Europe/Copenhagen
    {  450, 6612,  147085,  7, }, // Europe/Helsinki
    {  450, 8804,  147092,  8, }, // Europe/Isle_of_Man
    {  450, 3213,   21019,  6, }, // Europe/Lisbon
    {  450, 6365,   26933,  9, }, // Europe/Luxembourg
    {  450, 3537,   22748,  6, }, // Europe/Moscow
    {  450, 6275,   36153,  5, }, // Europe/Prague
    {  450, 6324,    5946,  4, }, // Europe/Rome
    {  450, 6202,   21095,  6, }, // Europe/Tirane
    {  450, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  450, 6231,   57383,  4, }, // Europe/Vienna
    {  450, 3190,   93110,  7, }, // Europe/Warsaw
    {  450, 6540,    4459,  6, }, // Europe/Zurich
    {  450, 6046,  147100, 11, }, // Indian/Christmas
    {  450, 6063,    4465,  5, }, // Indian/Cocos
    {  450, 7050,  147111, 11, }, // Indian/Maldives
    {  450, 4051,  147122, 11, }, // Pacific/Marquesas
    {  452,   14,       0,  6, }, // Africa/Asmara Northern Sotho/Latin/South Africa
    {  452, 4435,       6,  8, }, // Africa/Sao_Tome
    {  452, 4252,      14,  8, }, // America/Asuncion
    {  452,  237,      22,  8, }, // America/Atikokan
    {  452, 7623,      30, 17, }, // America/Bahia_Banderas
    {  452, 3868,      47,  6, }, // America/Cancun
    {  452, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  452, 5723,      66,  7, }, // America/Curacao
    {  452,  481,      73, 13, }, // America/Indiana/Knox
    {  452, 7961,      86, 16, }, // America/Indiana/Marengo
    {  452, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  452, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  452, 8038,     139, 14, }, // America/Indiana/Vevay
    {  452, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  452, 8086,     171, 16, }, // America/Indiana/Winamac
    {  452, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  452, 5932,     207, 22, }, // America/Lower_Princes
    {  452, 8220,     229,  6, }, // America/Merida
    {  452, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  452, 8329,     255, 20, }, // America/North_Dakota/Center
    {  452, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  452,  393,     298,  4, }, // America/Nuuk
    {  452, 6899,     302, 16, }, // America/Scoresbysund
    {  452, 8508,     318, 14, }, // America/St_Barthelemy
    {  452, 2061,     332, 10, }, // America/St_Johns
    {  452, 5863,     342,  9, }, // America/St_Kitts
    {  452, 5880,     351,  9, }, // America/St_Lucia
    {  452,  855,     360, 10, }, // America/St_Thomas
    {  452, 5913,     370, 11, }, // America/St_Vincent
    {  452, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  452, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  452, 1185,     413,  9, }, // Asia/Kathmandu
    {  452,  992,     422,  7, }, // Asia/Kolkata
    {  452, 1211,     429,  5, }, // Asia/Macau
    {  452, 1235,     442,  6, }, // Asia/Yangon
    {  452, 1395,     448,  5, }, // Atlantic/Faroe
    {  452, 6821,     453, 10, }, // Atlantic/St_Helena
    {  452, 2527,     463,  4, }, // Europe/Kyiv
    {  452, 7211,     467,  7, }, // Indian/Reunion
    {  452, 3157,     474,  5, }, // Pacific/Chuuk
    {  452, 3096,     479,  7, }, // Pacific/Pohnpei
    {  453,   14,       0,  6, }, // Africa/Asmara North Ndebele/Latin/Zimbabwe
    {  453, 4435,       6,  8, }, // Africa/Sao_Tome
    {  453, 4252,      14,  8, }, // America/Asuncion
    {  453,  237,      22,  8, }, // America/Atikokan
    {  453, 7623,      30, 17, }, // America/Bahia_Banderas
    {  453, 3868,      47,  6, }, // America/Cancun
    {  453, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  453, 5723,      66,  7, }, // America/Curacao
    {  453,  481,      73, 13, }, // America/Indiana/Knox
    {  453, 7961,      86, 16, }, // America/Indiana/Marengo
    {  453, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  453, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  453, 8038,     139, 14, }, // America/Indiana/Vevay
    {  453, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  453, 8086,     171, 16, }, // America/Indiana/Winamac
    {  453, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  453, 5932,     207, 22, }, // America/Lower_Princes
    {  453, 8220,     229,  6, }, // America/Merida
    {  453, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  453, 8329,     255, 20, }, // America/North_Dakota/Center
    {  453, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  453,  393,     298,  4, }, // America/Nuuk
    {  453, 6899,     302, 16, }, // America/Scoresbysund
    {  453, 8508,     318, 14, }, // America/St_Barthelemy
    {  453, 2061,     332, 10, }, // America/St_Johns
    {  453, 5863,     342,  9, }, // America/St_Kitts
    {  453, 5880,     351,  9, }, // America/St_Lucia
    {  453,  855,     360, 10, }, // America/St_Thomas
    {  453, 5913,     370, 11, }, // America/St_Vincent
    {  453, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  453, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  453, 1185,     413,  9, }, // Asia/Kathmandu
    {  453,  992,     422,  7, }, // Asia/Kolkata
    {  453, 1211,     429,  5, }, // Asia/Macau
    {  453, 1235,     442,  6, }, // Asia/Yangon
    {  453, 1395,     448,  5, }, // Atlantic/Faroe
    {  453, 6821,     453, 10, }, // Atlantic/St_Helena
    {  453, 2527,     463,  4, }, // Europe/Kyiv
    {  453, 7211,     467,  7, }, // Indian/Reunion
    {  453, 3157,     474,  5, }, // Pacific/Chuuk
    {  453, 3096,     479,  7, }, // Pacific/Pohnpei
    {  454, 5059,    3784, 11, }, // Africa/Addis_Ababa Norwegian Bokmal/Latin/Norway
    {  454, 7420,   43956,  5, }, // Africa/Algiers
    {  454,   14,       0,  6, }, // Africa/Asmara
    {  454, 2320,    4572,  5, }, // Africa/Cairo
    {  454, 5130,   93318, 13, }, // Africa/Dar_es_Salaam
    {  454, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  454, 6869,   20300,  4, }, // Africa/Lome
    {  454, 4435,       6,  8, }, // Africa/Sao_Tome
    {  454, 4554,   26427,  9, }, // America/Araguaina
    {  454,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  454, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  454, 4252,      14,  8, }, // America/Asuncion
    {  454,  237,      22,  8, }, // America/Atikokan
    {  454, 7623,   20375, 14, }, // America/Bahia_Banderas
    {  454, 7646,   35764,  5, }, // America/Belem
    {  454, 4361,   20389,  6, }, // America/Bogota
    {  454, 3868,      47,  6, }, // America/Cancun
    {  454, 5496,  147133, 11, }, // America/Cayman
    {  454, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  454, 3791,   26442,  6, }, // America/Cuiaba
    {  454, 5723,      66,  7, }, // America/Curacao
    {  454,  481,      73, 13, }, // America/Indiana/Knox
    {  454, 7961,      86, 16, }, // America/Indiana/Marengo
    {  454, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  454, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  454, 8038,     139, 14, }, // America/Indiana/Vevay
    {  454, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  454, 8086,     171, 16, }, // America/Indiana/Winamac
    {  454, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  454, 5932,     207, 22, }, // America/Lower_Princes
    {  454, 8153,   26456,  6, }, // America/Maceio
    {  454, 8220,     229,  6, }, // America/Merida
    {  454, 2949,  147144,  9, }, // America/Mexico_City
    {  454, 8301,   93589, 19, }, // America/North_Dakota/Beulah
    {  454, 8329,   93608, 19, }, // America/North_Dakota/Center
    {  454, 8357,   93627, 22, }, // America/North_Dakota/New_Salem
    {  454,  393,     298,  4, }, // America/Nuuk
    {  454, 8477,   26572,  8, }, // America/Santarem
    {  454, 1878,   26580,  9, }, // America/Sao_Paulo
    {  454, 6899,     302, 16, }, // America/Scoresbysund
    {  454, 8508,   26589, 16, }, // America/St_Barthelemy
    {  454, 2061,     332, 10, }, // America/St_Johns
    {  454, 5863,     342,  9, }, // America/St_Kitts
    {  454, 5880,     351,  9, }, // America/St_Lucia
    {  454,  855,     360, 10, }, // America/St_Thomas
    {  454, 5913,     370, 11, }, // America/St_Vincent
    {  454, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  454, 5523,    5491,  5, }, // Asia/Aqtau
    {  454, 5368,   57073,  6, }, // Asia/Aqtobe
    {  454,  964,   58440,  9, }, // Asia/Ashgabat
    {  454, 3607,    4044,  6, }, // Asia/Baghdad
    {  454, 3778,    4069,  7, }, // Asia/Bishkek
    {  454, 4601,  147153,  6, }, // Asia/Chita
    {  454, 4540,    4083,  8, }, // Asia/Damascus
    {  454, 6159,   58449,  8, }, // Asia/Dushanbe
    {  454, 1259,  147159, 16, }, // Asia/Ho_Chi_Minh
    {  454, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  454, 6992,  147175,  8, }, // Asia/Jayapura
    {  454, 4316,    4143, 10, }, // Asia/Kamchatka
    {  454, 1185,    5593,  8, }, // Asia/Kathmandu
    {  454,  992,     422,  7, }, // Asia/Kolkata
    {  454, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  454, 1211,     429,  5, }, // Asia/Macau
    {  454, 6920,    4208,  6, }, // Asia/Muscat
    {  454, 2554,    5620,  7, }, // Asia/Nicosia
    {  454, 8708,    7652,  8, }, // Asia/Qostanay
    {  454, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  454, 4781,   58478,  8, }, // Asia/Tashkent
    {  454, 2772,    7724,  7, }, // Asia/Tehran
    {  454, 1317,    7731,  6, }, // Asia/Thimphu
    {  454, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  454, 1159,   26801,  6, }, // Asia/Urumqi
    {  454, 4828,    7755,  7, }, // Asia/Yakutsk
    {  454, 1235,     442,  6, }, // Asia/Yangon
    {  454, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  454, 3747,    7780,  7, }, // Asia/Yerevan
    {  454, 3663,  147183,  7, }, // Atlantic/Azores
    {  454, 6628,  147190, 11, }, // Atlantic/Canary
    {  454, 3727,  147201, 10, }, // Atlantic/Cape_Verde
    {  454, 1395,  147211,  8, }, // Atlantic/Faroe
    {  454, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  454, 7272,  147219, 11, }, // Atlantic/South_Georgia
    {  454, 6821,     453, 10, }, // Atlantic/St_Helena
    {  454, 2300,    4320,  5, }, // Europe/Athens
    {  454, 6336,    7942,  7, }, // Europe/Belgrade
    {  454, 2167,    4332,  7, }, // Europe/Brussels
    {  454, 3928,  147230,  9, }, // Europe/Bucharest
    {  454, 8772,   26905,  8, }, // Europe/Busingen
    {  454, 2583,   63672,  8, }, // Europe/Chisinau
    {  454, 6289,   58525,  9, }, // Europe/Copenhagen
    {  454, 6612,  147239, 11, }, // Europe/Helsinki
    {  454, 8804,    4374,  3, }, // Europe/Isle_of_Man
    {  454, 2527,     463,  4, }, // Europe/Kyiv
    {  454, 3213,   21019,  6, }, // Europe/Lisbon
    {  454, 6365,    4395,  9, }, // Europe/Luxembourg
    {  454, 3537,   22748,  6, }, // Europe/Moscow
    {  454, 6275,   36153,  5, }, // Europe/Prague
    {  454, 6324,    5946,  4, }, // Europe/Rome
    {  454, 6202,   21095,  6, }, // Europe/Tirane
    {  454, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  454, 6567,  147250, 13, }, // Europe/Vatican
    {  454, 6231,   57383,  4, }, // Europe/Vienna
    {  454, 3190,   58543,  8, }, // Europe/Warsaw
    {  454, 6540,    4459,  6, }, // Europe/Zurich
    {  454, 6046,  147263, 12, }, // Indian/Christmas
    {  454, 6063,  147275, 10, }, // Indian/Cocos
    {  454, 5029,  147285,  8, }, // Indian/Comoro
    {  454, 7260,   26983,  4, }, // Indian/Mahe
    {  454, 7050,  147293,  9, }, // Indian/Maldives
    {  454, 7211,     467,  7, }, // Indian/Reunion
    {  454, 3157,     474,  5, }, // Pacific/Chuuk
    {  454, 2237,  147302,  8, }, // Pacific/Easter
    {  454, 6678,  147310, 14, }, // Pacific/Galapagos
    {  454, 2723,    4493,  8, }, // Pacific/Honolulu
    {  454, 3049,    4501,  9, }, // Pacific/Kanton
    {  454, 4168,  147324, 10, }, // Pacific/Norfolk
    {  454, 7098,    4510,  6, }, // Pacific/Noumea
    {  454, 3096,     479,  7, }, // Pacific/Pohnpei
    {  456, 5059,    3784, 11, }, // Africa/Addis_Ababa Norwegian Nynorsk/Latin/Norway
    {  456, 7420,   43956,  5, }, // Africa/Algiers
    {  456,   14,       0,  6, }, // Africa/Asmara
    {  456, 2320,    4572,  5, }, // Africa/Cairo
    {  456, 5130,   93318, 13, }, // Africa/Dar_es_Salaam
    {  456, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  456, 6869,   20300,  4, }, // Africa/Lome
    {  456, 4435,       6,  8, }, // Africa/Sao_Tome
    {  456, 4554,   26427,  9, }, // America/Araguaina
    {  456,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  456, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  456, 4252,      14,  8, }, // America/Asuncion
    {  456,  237,      22,  8, }, // America/Atikokan
    {  456, 7623,   20375, 14, }, // America/Bahia_Banderas
    {  456, 7646,   35764,  5, }, // America/Belem
    {  456, 4361,   20389,  6, }, // America/Bogota
    {  456, 3868,      47,  6, }, // America/Cancun
    {  456, 5496,  147334, 11, }, // America/Cayman
    {  456, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  456, 3791,  147345,  6, }, // America/Cuiaba
    {  456, 5723,      66,  7, }, // America/Curacao
    {  456,  481,      73, 13, }, // America/Indiana/Knox
    {  456, 7961,      86, 16, }, // America/Indiana/Marengo
    {  456, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  456, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  456, 8038,     139, 14, }, // America/Indiana/Vevay
    {  456, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  456, 8086,     171, 16, }, // America/Indiana/Winamac
    {  456, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  456, 5932,     207, 22, }, // America/Lower_Princes
    {  456, 8153,   26456,  6, }, // America/Maceio
    {  456, 8220,    7008,  6, }, // America/Merida
    {  456, 2949,  147144,  9, }, // America/Mexico_City
    {  456, 8301,   93589, 19, }, // America/North_Dakota/Beulah
    {  456, 8329,   93608, 19, }, // America/North_Dakota/Center
    {  456, 8357,   93627, 22, }, // America/North_Dakota/New_Salem
    {  456,  393,     298,  4, }, // America/Nuuk
    {  456, 8477,   26572,  8, }, // America/Santarem
    {  456, 1878,   26580,  9, }, // America/Sao_Paulo
    {  456, 6899,     302, 16, }, // America/Scoresbysund
    {  456, 8508,   26589, 16, }, // America/St_Barthelemy
    {  456, 2061,     332, 10, }, // America/St_Johns
    {  456, 5863,     342,  9, }, // America/St_Kitts
    {  456, 5880,     351,  9, }, // America/St_Lucia
    {  456,  855,     360, 10, }, // America/St_Thomas
    {  456, 5913,     370, 11, }, // America/St_Vincent
    {  456, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  456, 5523,    5491,  5, }, // Asia/Aqtau
    {  456, 5368,    7488,  6, }, // Asia/Aqtobe
    {  456,  964,  147351,  8, }, // Asia/Ashgabat
    {  456, 3607,    4044,  6, }, // Asia/Baghdad
    {  456, 3778,    4069,  7, }, // Asia/Bishkek
    {  456, 4601,  147153,  6, }, // Asia/Chita
    {  456, 4540,    4083,  8, }, // Asia/Damascus
    {  456, 6159,   58449,  8, }, // Asia/Dushanbe
    {  456, 1259,  147159, 16, }, // Asia/Ho_Chi_Minh
    {  456, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  456, 4771,   26703,  5, }, // Asia/Hovd
    {  456, 6992,  147175,  8, }, // Asia/Jayapura
    {  456, 4316,    4143, 10, }, // Asia/Kamchatka
    {  456, 1185,    5593,  8, }, // Asia/Kathmandu
    {  456,  992,     422,  7, }, // Asia/Kolkata
    {  456, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  456, 1211,     429,  5, }, // Asia/Macau
    {  456, 6920,    4208,  6, }, // Asia/Muscat
    {  456, 2554,    5620,  7, }, // Asia/Nicosia
    {  456, 8708,    7652,  8, }, // Asia/Qostanay
    {  456, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  456, 4781,   58478,  8, }, // Asia/Tashkent
    {  456, 2772,    7724,  7, }, // Asia/Tehran
    {  456, 1317,    7731,  6, }, // Asia/Thimphu
    {  456, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  456, 1159,   26801,  6, }, // Asia/Urumqi
    {  456, 4828,    7755,  7, }, // Asia/Yakutsk
    {  456, 1235,  147359,  6, }, // Asia/Yangon
    {  456, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  456, 3747,    7780,  7, }, // Asia/Yerevan
    {  456, 3663,  147365,  7, }, // Atlantic/Azores
    {  456, 6628,  147372, 11, }, // Atlantic/Canary
    {  456, 3727,  147201, 10, }, // Atlantic/Cape_Verde
    {  456, 1395,  147383,  8, }, // Atlantic/Faroe
    {  456, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  456, 7272,  147219, 11, }, // Atlantic/South_Georgia
    {  456, 6821,     453, 10, }, // Atlantic/St_Helena
    {  456, 2300,    4320,  5, }, // Europe/Athens
    {  456, 6336,    7942,  7, }, // Europe/Belgrade
    {  456, 2167,    4332,  7, }, // Europe/Brussels
    {  456, 3928,  147230,  9, }, // Europe/Bucharest
    {  456, 8772,   26905,  8, }, // Europe/Busingen
    {  456, 2583,   63672,  8, }, // Europe/Chisinau
    {  456, 6289,   58525,  9, }, // Europe/Copenhagen
    {  456, 6612,  147239, 11, }, // Europe/Helsinki
    {  456, 8804,    4374,  3, }, // Europe/Isle_of_Man
    {  456, 2527,     463,  4, }, // Europe/Kyiv
    {  456, 3213,   21019,  6, }, // Europe/Lisbon
    {  456, 6365,    4395,  9, }, // Europe/Luxembourg
    {  456, 3537,   22748,  6, }, // Europe/Moscow
    {  456, 6275,   36153,  5, }, // Europe/Prague
    {  456, 6324,    5946,  4, }, // Europe/Rome
    {  456, 6202,   21095,  6, }, // Europe/Tirane
    {  456, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  456, 6567,  147250, 13, }, // Europe/Vatican
    {  456, 6231,   57383,  4, }, // Europe/Vienna
    {  456, 3190,   58543,  8, }, // Europe/Warsaw
    {  456, 6540,    4459,  6, }, // Europe/Zurich
    {  456, 6046,  147263, 12, }, // Indian/Christmas
    {  456, 6063,  147391, 10, }, // Indian/Cocos
    {  456, 5029,  147401,  8, }, // Indian/Comoro
    {  456, 7260,   26983,  4, }, // Indian/Mahe
    {  456, 7050,  147409,  9, }, // Indian/Maldives
    {  456, 7211,     467,  7, }, // Indian/Reunion
    {  456, 3157,     474,  5, }, // Pacific/Chuuk
    {  456, 2237,  147302,  8, }, // Pacific/Easter
    {  456, 6678,  147418, 14, }, // Pacific/Galapagos
    {  456, 2723,    4493,  8, }, // Pacific/Honolulu
    {  456, 3049,    4501,  9, }, // Pacific/Kanton
    {  456, 4168,  147324, 10, }, // Pacific/Norfolk
    {  456, 7098,    4510,  6, }, // Pacific/Noumea
    {  456, 3096,     479,  7, }, // Pacific/Pohnpei
    {  457,   14,       0,  6, }, // Africa/Asmara Nuer/Latin/South Sudan
    {  457, 4435,       6,  8, }, // Africa/Sao_Tome
    {  457, 4252,      14,  8, }, // America/Asuncion
    {  457,  237,      22,  8, }, // America/Atikokan
    {  457, 7623,      30, 17, }, // America/Bahia_Banderas
    {  457, 3868,      47,  6, }, // America/Cancun
    {  457, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  457, 5723,      66,  7, }, // America/Curacao
    {  457,  481,      73, 13, }, // America/Indiana/Knox
    {  457, 7961,      86, 16, }, // America/Indiana/Marengo
    {  457, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  457, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  457, 8038,     139, 14, }, // America/Indiana/Vevay
    {  457, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  457, 8086,     171, 16, }, // America/Indiana/Winamac
    {  457, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  457, 5932,     207, 22, }, // America/Lower_Princes
    {  457, 8220,     229,  6, }, // America/Merida
    {  457, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  457, 8329,     255, 20, }, // America/North_Dakota/Center
    {  457, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  457,  393,     298,  4, }, // America/Nuuk
    {  457, 6899,     302, 16, }, // America/Scoresbysund
    {  457, 8508,     318, 14, }, // America/St_Barthelemy
    {  457, 2061,     332, 10, }, // America/St_Johns
    {  457, 5863,     342,  9, }, // America/St_Kitts
    {  457, 5880,     351,  9, }, // America/St_Lucia
    {  457,  855,     360, 10, }, // America/St_Thomas
    {  457, 5913,     370, 11, }, // America/St_Vincent
    {  457, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  457, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  457, 1185,     413,  9, }, // Asia/Kathmandu
    {  457,  992,     422,  7, }, // Asia/Kolkata
    {  457, 1211,     429,  5, }, // Asia/Macau
    {  457, 1235,     442,  6, }, // Asia/Yangon
    {  457, 1395,     448,  5, }, // Atlantic/Faroe
    {  457, 6821,     453, 10, }, // Atlantic/St_Helena
    {  457, 2527,     463,  4, }, // Europe/Kyiv
    {  457, 7211,     467,  7, }, // Indian/Reunion
    {  457, 3157,     474,  5, }, // Pacific/Chuuk
    {  457, 3096,     479,  7, }, // Pacific/Pohnpei
    {  458,   14,       0,  6, }, // Africa/Asmara Nyanja/Latin/Malawi
    {  458, 4435,       6,  8, }, // Africa/Sao_Tome
    {  458, 4252,      14,  8, }, // America/Asuncion
    {  458,  237,      22,  8, }, // America/Atikokan
    {  458, 7623,      30, 17, }, // America/Bahia_Banderas
    {  458, 3868,      47,  6, }, // America/Cancun
    {  458, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  458, 5723,      66,  7, }, // America/Curacao
    {  458,  481,      73, 13, }, // America/Indiana/Knox
    {  458, 7961,      86, 16, }, // America/Indiana/Marengo
    {  458, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  458, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  458, 8038,     139, 14, }, // America/Indiana/Vevay
    {  458, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  458, 8086,     171, 16, }, // America/Indiana/Winamac
    {  458, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  458, 5932,     207, 22, }, // America/Lower_Princes
    {  458, 8220,     229,  6, }, // America/Merida
    {  458, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  458, 8329,     255, 20, }, // America/North_Dakota/Center
    {  458, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  458,  393,     298,  4, }, // America/Nuuk
    {  458, 6899,     302, 16, }, // America/Scoresbysund
    {  458, 8508,     318, 14, }, // America/St_Barthelemy
    {  458, 2061,     332, 10, }, // America/St_Johns
    {  458, 5863,     342,  9, }, // America/St_Kitts
    {  458, 5880,     351,  9, }, // America/St_Lucia
    {  458,  855,     360, 10, }, // America/St_Thomas
    {  458, 5913,     370, 11, }, // America/St_Vincent
    {  458, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  458, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  458, 1185,     413,  9, }, // Asia/Kathmandu
    {  458,  992,     422,  7, }, // Asia/Kolkata
    {  458, 1211,     429,  5, }, // Asia/Macau
    {  458, 1235,     442,  6, }, // Asia/Yangon
    {  458, 1395,     448,  5, }, // Atlantic/Faroe
    {  458, 6821,     453, 10, }, // Atlantic/St_Helena
    {  458, 2527,     463,  4, }, // Europe/Kyiv
    {  458, 7211,     467,  7, }, // Indian/Reunion
    {  458, 3157,     474,  5, }, // Pacific/Chuuk
    {  458, 3096,     479,  7, }, // Pacific/Pohnpei
    {  459,   14,       0,  6, }, // Africa/Asmara Nyankole/Latin/Uganda
    {  459, 4435,       6,  8, }, // Africa/Sao_Tome
    {  459, 4252,      14,  8, }, // America/Asuncion
    {  459,  237,      22,  8, }, // America/Atikokan
    {  459, 7623,      30, 17, }, // America/Bahia_Banderas
    {  459, 3868,      47,  6, }, // America/Cancun
    {  459, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  459, 5723,      66,  7, }, // America/Curacao
    {  459,  481,      73, 13, }, // America/Indiana/Knox
    {  459, 7961,      86, 16, }, // America/Indiana/Marengo
    {  459, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  459, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  459, 8038,     139, 14, }, // America/Indiana/Vevay
    {  459, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  459, 8086,     171, 16, }, // America/Indiana/Winamac
    {  459, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  459, 5932,     207, 22, }, // America/Lower_Princes
    {  459, 8220,     229,  6, }, // America/Merida
    {  459, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  459, 8329,     255, 20, }, // America/North_Dakota/Center
    {  459, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  459,  393,     298,  4, }, // America/Nuuk
    {  459, 6899,     302, 16, }, // America/Scoresbysund
    {  459, 8508,     318, 14, }, // America/St_Barthelemy
    {  459, 2061,     332, 10, }, // America/St_Johns
    {  459, 5863,     342,  9, }, // America/St_Kitts
    {  459, 5880,     351,  9, }, // America/St_Lucia
    {  459,  855,     360, 10, }, // America/St_Thomas
    {  459, 5913,     370, 11, }, // America/St_Vincent
    {  459, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  459, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  459, 1185,     413,  9, }, // Asia/Kathmandu
    {  459,  992,     422,  7, }, // Asia/Kolkata
    {  459, 1211,     429,  5, }, // Asia/Macau
    {  459, 1235,     442,  6, }, // Asia/Yangon
    {  459, 1395,     448,  5, }, // Atlantic/Faroe
    {  459, 6821,     453, 10, }, // Atlantic/St_Helena
    {  459, 2527,     463,  4, }, // Europe/Kyiv
    {  459, 7211,     467,  7, }, // Indian/Reunion
    {  459, 3157,     474,  5, }, // Pacific/Chuuk
    {  459, 3096,     479,  7, }, // Pacific/Pohnpei
    {  460, 7420,  147432,  7, }, // Africa/Algiers Occitan/Latin/France
    {  460,   14,       0,  6, }, // Africa/Asmara
    {  460, 5043,  147439,  6, }, // Africa/Djibouti
    {  460, 5243,  147445,  5, }, // Africa/Douala
    {  460, 4435,       6,  8, }, // Africa/Sao_Tome
    {  460, 4252,      14,  8, }, // America/Asuncion
    {  460,  237,      22,  8, }, // America/Atikokan
    {  460, 7623,      30, 17, }, // America/Bahia_Banderas
    {  460, 3868,      47,  6, }, // America/Cancun
    {  460, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  460, 5723,      66,  7, }, // America/Curacao
    {  460,  481,      73, 13, }, // America/Indiana/Knox
    {  460, 7961,      86, 16, }, // America/Indiana/Marengo
    {  460, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  460, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  460, 8038,     139, 14, }, // America/Indiana/Vevay
    {  460, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  460, 8086,     171, 16, }, // America/Indiana/Winamac
    {  460, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  460, 5932,     207, 22, }, // America/Lower_Princes
    {  460, 8220,     229,  6, }, // America/Merida
    {  460, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  460, 8329,     255, 20, }, // America/North_Dakota/Center
    {  460, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  460,  393,     298,  4, }, // America/Nuuk
    {  460, 6899,     302, 16, }, // America/Scoresbysund
    {  460, 8508,     318, 14, }, // America/St_Barthelemy
    {  460, 2061,     332, 10, }, // America/St_Johns
    {  460, 5863,     342,  9, }, // America/St_Kitts
    {  460, 5880,     351,  9, }, // America/St_Lucia
    {  460,  855,     360, 10, }, // America/St_Thomas
    {  460, 5913,     370, 11, }, // America/St_Vincent
    {  460, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  460, 4466,  147450,  7, }, // Asia/Bangkok
    {  460, 1093,   65588,  5, }, // Asia/Dhaka
    {  460, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  460, 3560,  147457,  5, }, // Asia/Kabul
    {  460, 1185,     413,  9, }, // Asia/Kathmandu
    {  460,  992,   20720,  7, }, // Asia/Kolkata
    {  460, 1211,     429,  5, }, // Asia/Macau
    {  460, 4408,  147462,  7, }, // Asia/Sakhalin
    {  460, 3300,    7691,  8, }, // Asia/Singapore
    {  460, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  460, 1235,     442,  6, }, // Asia/Yangon
    {  460, 3883,  147469, 12, }, // Asia/Yekaterinburg
    {  460, 3663,  147481,  6, }, // Atlantic/Azores
    {  460, 6628,   44349,  8, }, // Atlantic/Canary
    {  460, 3727,   44357,  8, }, // Atlantic/Cape_Verde
    {  460, 1395,     448,  5, }, // Atlantic/Faroe
    {  460, 6821,     453, 10, }, // Atlantic/St_Helena
    {  460, 1670,  147487,  8, }, // Australia/Adelaide
    {  460, 6216,  147495,  7, }, // Europe/Andorra
    {  460, 2300,   20928,  6, }, // Europe/Athens
    {  460, 6336,   26883,  7, }, // Europe/Belgrade
    {  460, 2167,  147502,  9, }, // Europe/Brussels
    {  460, 3928,  147511,  8, }, // Europe/Bucharest
    {  460, 3806,  147519,  8, }, // Europe/Budapest
    {  460, 2527,     463,  4, }, // Europe/Kyiv
    {  460, 3213,   93285,  7, }, // Europe/Lisbon
    {  460, 2501,   21034,  7, }, // Europe/London
    {  460, 6365,  147527,  9, }, // Europe/Luxembourg
    {  460, 6410,  147536,  7, }, // Europe/Monaco
    {  460, 3537,  147543,  6, }, // Europe/Moscow
    {  460, 4284,   21062,  5, }, // Europe/Paris
    {  460, 6275,   21067,  5, }, // Europe/Prague
    {  460, 6324,    5946,  4, }, // Europe/Rome
    {  460, 7050,  147549,  8, }, // Indian/Maldives
    {  460, 7211,     467,  7, }, // Indian/Reunion
    {  460, 3157,     474,  5, }, // Pacific/Chuuk
    {  460, 3096,     479,  7, }, // Pacific/Pohnpei
    {  461, 6788,  147557,  6, }, // Africa/Abidjan Occitan/Latin/Spain
    {  461, 6760,   74000,  4, }, // Africa/Accra
    {  461, 5059,   20252, 10, }, // Africa/Addis_Ababa
    {  461, 7420,  147563,  5, }, // Africa/Algiers
    {  461, 7435,    6195,  5, }, // Africa/Bissau
    {  461, 2320,  147568,  9, }, // Africa/Cairo
    {  461, 5043,  147577,  6, }, // Africa/Djibouti
    {  461, 4524,  147583,  6, }, // Africa/Khartoum
    {  461, 6869,   20300,  4, }, // Africa/Lome
    {  461, 5113,   43978, 10, }, // Africa/Mogadishu
    {  461, 5271,  147589,  6, }, // Africa/Ndjamena
    {  461, 6803,  147595,  8, }, // Africa/Nouakchott
    {  461, 6727,   20331,  8, }, // Africa/Ouagadougou
    {  461, 5225,   61241,  9, }, // Africa/Porto-Novo
    {  461, 4435,  147603,  9, }, // Africa/Sao_Tome
    {  461, 2866,   20349,  7, }, // Africa/Tripoli
    {  461, 6554,   20356,  5, }, // Africa/Tunis
    {  461, 4554,   26427,  9, }, // America/Araguaina
    {  461,  270,  147612,  6, }, // America/Argentina/Cordoba
    {  461, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {  461, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  461, 7623,   64558, 17, }, // America/Bahia_Banderas
    {  461, 7646,  147618,  5, }, // America/Belem
    {  461, 5407,  147623,  6, }, // America/Belize
    {  461, 4361,   20389,  6, }, // America/Bogota
    {  461, 4345,  147629,  6, }, // America/Cayenne
    {  461, 5496,   44041,  6, }, // America/Cayman
    {  461, 5422,  147635, 10, }, // America/Costa_Rica
    {  461, 3791,   26442,  6, }, // America/Cuiaba
    {  461, 7869,   26448,  8, }, // America/Eirunepe
    {  461, 4612,  147645, 10, }, // America/Grand_Turk
    {  461, 5770,   20401,  7, }, // America/Grenada
    {  461, 5786,   20408,  9, }, // America/Guadeloupe
    {  461, 2281,  147655, 10, }, // America/Havana
    {  461, 8153,   26456,  6, }, // America/Maceio
    {  461, 1908,  147665,  8, }, // America/Manaus
    {  461, 5805,   20436,  9, }, // America/Martinique
    {  461, 2949,  147673, 15, }, // America/Mexico_City
    {  461, 2379,  147688,  9, }, // America/New_York
    {  461, 8301,  147697, 24, }, // America/North_Dakota/Beulah
    {  461, 8329,  147721, 24, }, // America/North_Dakota/Center
    {  461, 8357,  147745, 27, }, // America/North_Dakota/New_Salem
    {  461, 2356,   44143,  6, }, // America/Panama
    {  461, 3945,  147772, 15, }, // America/Port-au-Prince
    {  461, 5954,  147787, 14, }, // America/Port_of_Spain
    {  461, 8477,   26572,  8, }, // America/Santarem
    {  461, 2201,   64147, 17, }, // America/Santiago
    {  461, 1878,  147801,  9, }, // America/Sao_Paulo
    {  461, 8508,  147810, 14, }, // America/St_Barthelemy
    {  461, 2061,  147824, 22, }, // America/St_Johns
    {  461, 5863,  147846, 14, }, // America/St_Kitts
    {  461, 5880,   74265, 11, }, // America/St_Lucia
    {  461, 5913,  147860, 11, }, // America/St_Vincent
    {  461, 5688,  147871,  7, }, // America/Tortola
    {  461, 5511,  147878,  6, }, // Asia/Anadyr
    {  461, 5523,    5491,  5, }, // Asia/Aqtau
    {  461, 5368,    7488,  6, }, // Asia/Aqtobe
    {  461,  964,  147884,  7, }, // Asia/Ashgabat
    {  461, 3607,    4044,  6, }, // Asia/Baghdad
    {  461, 5534,  147891,  6, }, // Asia/Bahrain
    {  461, 3653,   20629,  4, }, // Asia/Baku
    {  461, 3571,  147897,  7, }, // Asia/Barnaul
    {  461, 4601,   74304,  5, }, // Asia/Chita
    {  461, 4540,   74309,  7, }, // Asia/Damascus
    {  461, 6159,  147904,  7, }, // Asia/Dushanbe
    {  461, 1259,  147911, 18, }, // Asia/Ho_Chi_Minh
    {  461, 1290,  147929,  9, }, // Asia/Jerusalem
    {  461, 1185,   20712,  8, }, // Asia/Kathmandu
    {  461, 1349,    7618,  7, }, // Asia/Makassar
    {  461, 6920,   20739,  7, }, // Asia/Muscat
    {  461, 5559,   73021,  5, }, // Asia/Qatar
    {  461, 8708,   64229,  8, }, // Asia/Qostanay
    {  461, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  461, 3584,    7669,  4, }, // Asia/Riyadh
    {  461, 4408,  147938,  7, }, // Asia/Sakhalin
    {  461, 7245,   20768, 10, }, // Asia/Samarkand
    {  461, 3279,  147945,  4, }, // Asia/Seoul
    {  461, 4297,    7699, 13, }, // Asia/Srednekolymsk
    {  461, 4781,   91626,  7, }, // Asia/Tashkent
    {  461, 3915,  147949,  7, }, // Asia/Tbilisi
    {  461, 2772,    7724,  7, }, // Asia/Tehran
    {  461, 1317,   62492,  5, }, // Asia/Thimphu
    {  461, 2821,    4238,  5, }, // Asia/Tokyo
    {  461, 1159,   26801,  6, }, // Asia/Urumqi
    {  461, 3747,  147956,  6, }, // Asia/Yerevan
    {  461, 5671,   44335,  8, }, // Atlantic/Bermuda
    {  461, 3727,   26836, 10, }, // Atlantic/Cape_Verde
    {  461, 1395,  147962, 11, }, // Atlantic/Faroe
    {  461, 2748,   20879,  9, }, // Atlantic/Reykjavik
    {  461, 7272,  147973, 16, }, // Atlantic/South_Georgia
    {  461, 6821,  147989, 11, }, // Atlantic/St_Helena
    {  461, 1463,  148000,  6, }, // Australia/Sydney
    {  461, 3620,   44391,  8, }, // Europe/Astrakhan
    {  461, 3928,   20955,  8, }, // Europe/Bucharest
    {  461, 8804,  148006, 11, }, // Europe/Isle_of_Man
    {  461, 1130,  148017,  8, }, // Europe/Istanbul
    {  461, 8929,   44429,  5, }, // Europe/Kirov
    {  461, 2527,    8030,  4, }, // Europe/Kyiv
    {  461, 3213,   21019,  6, }, // Europe/Lisbon
    {  461, 6506,   21025,  9, }, // Europe/Ljubljana
    {  461, 2501,  148025,  7, }, // Europe/London
    {  461, 6470,  148032, 11, }, // Europe/San_Marino
    {  461, 4451,  148043,  7, }, // Europe/Saratov
    {  461, 8849,   44457, 10, }, // Europe/Simferopol
    {  461, 6383,  148050,  6, }, // Europe/Skopje
    {  461, 6599,   21081,  5, }, // Europe/Sofia
    {  461, 6523,   21086,  8, }, // Europe/Stockholm
    {  461, 8867,   22793,  6, }, // Europe/Tallinn
    {  461, 6202,   21095,  6, }, // Europe/Tirane
    {  461, 8882,  148056,  9, }, // Europe/Ulyanovsk
    {  461, 6567,  148065, 11, }, // Europe/Vatican
    {  461, 6231,   21112,  5, }, // Europe/Vienna
    {  461, 8899,   64377,  5, }, // Europe/Vilnius
    {  461, 3190,   44489,  8, }, // Europe/Warsaw
    {  461, 6540,  148076,  6, }, // Europe/Zurich
    {  461, 6046,  148082,  5, }, // Indian/Christmas
    {  461, 5029,   65391,  7, }, // Indian/Comoro
    {  461, 7260,   26983,  4, }, // Indian/Mahe
    {  461, 4069,   21134,  7, }, // Indian/Mauritius
    {  461, 2237,  148087, 14, }, // Pacific/Easter
    {  461, 3902,  148101,  4, }, // Pacific/Fiji
    {  461, 6678,   21159,  9, }, // Pacific/Galapagos
    {  461, 4051,   44516,  9, }, // Pacific/Marquesas
    {  461, 7155,   65437,  6, }, // Pacific/Palau
    {  461, 7312,   44525,  6, }, // Pacific/Tahiti
    {  462, 6788,  148105,  7, }, // Africa/Abidjan Odia/Odia/India
    {  462, 6760,  148112,  5, }, // Africa/Accra
    {  462, 5059,  148117, 12, }, // Africa/Addis_Ababa
    {  462, 7420,  148129,  8, }, // Africa/Algiers
    {  462,   14,  148137,  5, }, // Africa/Asmara
    {  462,   44,  148142,  6, }, // Africa/Bamako
    {  462, 5257,  148148,  7, }, // Africa/Bangui
    {  462, 6746,  148155,  7, }, // Africa/Banjul
    {  462, 7435,  148162,  5, }, // Africa/Bissau
    {  462, 4957,  148167, 13, }, // Africa/Blantyre
    {  462, 5287,  148180, 12, }, // Africa/Brazzaville
    {  462, 4922,  148192, 10, }, // Africa/Bujumbura
    {  462, 2320,  148202,  5, }, // Africa/Cairo
    {  462, 4117,  148207, 12, }, // Africa/Casablanca
    {  462, 7449,  148219,  5, }, // Africa/Ceuta
    {  462, 6773,  148224,  8, }, // Africa/Conakry
    {  462, 6840,  148232,  4, }, // Africa/Dakar
    {  462, 5130,  148236, 15, }, // Africa/Dar_es_Salaam
    {  462, 5043,  148251,  6, }, // Africa/Djibouti
    {  462, 5243,  148257,  4, }, // Africa/Douala
    {  462, 5166,  148261,  9, }, // Africa/El_Aaiun
    {  462, 6853,  148270, 10, }, // Africa/Freetown
    {  462, 4906,  148280,  9, }, // Africa/Gaborone
    {  462, 5015,  148289,  5, }, // Africa/Harare
    {  462, 4479,  148294, 11, }, // Africa/Johannesburg
    {  462, 4499,  148305,  4, }, // Africa/Juba
    {  462, 5151,  148309,  8, }, // Africa/Kampala
    {  462, 4524,  148317,  7, }, // Africa/Khartoum
    {  462, 4973,  148324,  6, }, // Africa/Kigali
    {  462, 5306,  148330,  8, }, // Africa/Kinshasa
    {  462, 4744,  148338,  7, }, // Africa/Lagos
    {  462, 5336,  148345, 12, }, // Africa/Libreville
    {  462, 6869,  148357,  5, }, // Africa/Lome
    {  462, 5211,  148362,  7, }, // Africa/Luanda
    {  462, 4939,  148369, 10, }, // Africa/Lubumbashi
    {  462, 5001,  148379,  6, }, // Africa/Lusaka
    {  462, 5322,  148385,  6, }, // Africa/Malabo
    {  462, 4987,  148391,  6, }, // Africa/Maputo
    {  462, 5197,  148397,  6, }, // Africa/Maseru
    {  462, 5182,  148403,  7, }, // Africa/Mbabane
    {  462, 5113,  148410,  8, }, // Africa/Mogadishu
    {  462, 7034,  148418,  7, }, // Africa/Monrovia
    {  462, 3853,  148425,  7, }, // Africa/Nairobi
    {  462, 5271,  148432,  6, }, // Africa/Ndjamena
    {  462, 5354,  148438,  5, }, // Africa/Niamey
    {  462, 6803,  148443,  7, }, // Africa/Nouakchott
    {  462, 6727,  148450,  8, }, // Africa/Ouagadougou
    {  462, 5225,  148458, 11, }, // Africa/Porto-Novo
    {  462, 4435,  148469,  8, }, // Africa/Sao_Tome
    {  462, 2866,  148477,  8, }, // Africa/Tripoli
    {  462, 6554,  148485,  7, }, // Africa/Tunis
    {  462, 4152,  148492, 10, }, // Africa/Windhoek
    {  462,  132,  148502,  5, }, // America/Adak
    {  462, 3346,  148507,  8, }, // America/Anchorage
    {  462, 5607,  148515,  8, }, // America/Anguilla
    {  462, 5624,  148523,  8, }, // America/Antigua
    {  462, 4554,  148531,  8, }, // America/Araguaina
    {  462,  166,  148539, 14, }, // America/Argentina/Buenos_Aires
    {  462,   91,  148553, 10, }, // America/Argentina/Catamarca
    {  462,  270,  148563,  8, }, // America/Argentina/Cordoba
    {  462,  441,  148571,  5, }, // America/Argentina/Jujuy
    {  462, 7462,  148576,  8, }, // America/Argentina/La_Rioja
    {  462,  565,  148584,  8, }, // America/Argentina/Mendoza
    {  462, 7489,  148592, 14, }, // America/Argentina/Rio_Gallegos
    {  462, 7520,  148606,  5, }, // America/Argentina/Salta
    {  462, 7544,  148611, 10, }, // America/Argentina/San_Juan
    {  462, 5580,  148621, 12, }, // America/Argentina/San_Luis
    {  462, 7571,  148633,  9, }, // America/Argentina/Tucuman
    {  462, 7597,  148642,  5, }, // America/Argentina/Ushuaia
    {  462, 5640,  148647,  5, }, // America/Aruba
    {  462, 4252,  148652,  9, }, // America/Asuncion
    {  462,  237,  148661,  9, }, // America/Atikokan
    {  462, 3679,  148670,  5, }, // America/Bahia
    {  462, 7623,  148675, 18, }, // America/Bahia_Banderas
    {  462, 5654,  148693, 10, }, // America/Barbados
    {  462, 7646,  148703,  5, }, // America/Belem
    {  462, 5407,  148708,  6, }, // America/Belize
    {  462, 7660,  148714, 15, }, // America/Blanc-Sablon
    {  462, 7681,  148729, 10, }, // America/Boa_Vista
    {  462, 4361,  148739,  6, }, // America/Bogota
    {  462, 7699,  148745,  5, }, // America/Boise
    {  462, 7713,  148750, 13, }, // America/Cambridge_Bay
    {  462, 7735,  148763, 15, }, // America/Campo_Grande
    {  462, 3868,  148778,  7, }, // America/Cancun
    {  462, 4694,  148785,  8, }, // America/Caracas
    {  462, 4345,  148793,  8, }, // America/Cayenne
    {  462, 5496,  148801,  9, }, // America/Cayman
    {  462, 2260,  148810,  6, }, // America/Chicago
    {  462, 7756,  148816,  8, }, // America/Chihuahua
    {  462, 7774,  148824, 15, }, // America/Ciudad_Juarez
    {  462, 5422,  148839, 11, }, // America/Costa_Rica
    {  462, 7796,  148850, 10, }, // America/Creston
    {  462, 3791,  148860,  5, }, // America/Cuiaba
    {  462, 5723,  148865,  6, }, // America/Curacao
    {  462, 7812,  148871, 13, }, // America/Danmarkshavn
    {  462, 7833,  148884,  5, }, // America/Dawson
    {  462, 7848,  148889, 12, }, // America/Dawson_Creek
    {  462,  805,  148901,  7, }, // America/Denver
    {  462, 3465,  148908,  9, }, // America/Detroit
    {  462, 5739,  148917,  8, }, // America/Dominica
    {  462,  893,  148925,  9, }, // America/Edmonton
    {  462, 7869,  148934,  7, }, // America/Eirunepe
    {  462, 5441,  148941, 13, }, // America/El_Salvador
    {  462, 7886,  148954, 13, }, // America/Fort_Nelson
    {  462, 7906,  148967, 10, }, // America/Fortaleza
    {  462, 7924,  148977,  9, }, // America/Glace_Bay
    {  462, 6881,  148986,  7, }, // America/Goose_Bay
    {  462, 4612,  148993, 14, }, // America/Grand_Turk
    {  462, 5770,  149007,  8, }, // America/Grenada
    {  462, 5786,  149015, 10, }, // America/Guadeloupe
    {  462, 3760,  149025,  9, }, // America/Guatemala
    {  462, 4841,  149034,  9, }, // America/Guayaquil
    {  462, 6932,  149043,  5, }, // America/Guyana
    {  462, 1939,  149048, 12, }, // America/Halifax
    {  462, 2281,  149060,  6, }, // America/Havana
    {  462, 7942,  149066,  9, }, // America/Hermosillo
    {  462,  348,  149075, 14, }, // America/Indiana/Indianapolis
    {  462,  481,  149089, 17, }, // America/Indiana/Knox
    {  462, 7961,  149106, 17, }, // America/Indiana/Marengo
    {  462, 7985,  149123, 22, }, // America/Indiana/Petersburg
    {  462, 8012,  149145, 19, }, // America/Indiana/Tell_City
    {  462, 8038,  149164, 15, }, // America/Indiana/Vevay
    {  462, 8060,  149179, 21, }, // America/Indiana/Vincennes
    {  462, 8086,  149200, 18, }, // America/Indiana/Winamac
    {  462, 8110,  149218,  7, }, // America/Inuvik
    {  462,  660,  149225, 10, }, // America/Iqaluit
    {  462, 2799,  149235,  7, }, // America/Jamaica
    {  462, 5380,  149242,  6, }, // America/Juneau
    {  462,  521,  149248,  9, }, // America/Kentucky/Louisville
    {  462, 8125,  149257, 22, }, // America/Kentucky/Monticello
    {  462, 5704,  149279, 12, }, // America/Kralendijk
    {  462, 4376,  149291,  8, }, // America/La_Paz
    {  462, 7169,  149299,  4, }, // America/Lima
    {  462, 3239,  149303, 13, }, // America/Los_Angeles
    {  462, 5932,  149316, 23, }, // America/Lower_Princes
    {  462, 8153,  149339,  5, }, // America/Maceio
    {  462, 8168,  149344,  7, }, // America/Managua
    {  462, 1908,  149351,  8, }, // America/Manaus
    {  462, 5897,  149359,  8, }, // America/Marigot
    {  462, 5805,  149367, 12, }, // America/Martinique
    {  462, 8184,  149379, 10, }, // America/Matamoros
    {  462, 2917,  149389,  9, }, // America/Mazatlan
    {  462, 8202,  149398,  8, }, // America/Menominee
    {  462, 8220,  149406,  6, }, // America/Merida
    {  462, 8235,  149412, 10, }, // America/Metlakatla
    {  462, 2949,  149422, 13, }, // America/Mexico_City
    {  462, 4391,  149435, 10, }, // America/Miquelon
    {  462, 8254,  149445,  8, }, // America/Moncton
    {  462, 8270,  149453,  8, }, // America/Monterrey
    {  462, 4098,  149461, 10, }, // America/Montevideo
    {  462, 5824,  149471, 11, }, // America/Montserrat
    {  462, 5481,  149482,  5, }, // America/Nassau
    {  462, 2379,  149487, 11, }, // America/New_York
    {  462, 8288,  149498,  4, }, // America/Nome
    {  462, 1850,  149502,  8, }, // America/Noronha
    {  462, 8301,  149510, 20, }, // America/North_Dakota/Beulah
    {  462, 8329,  149530, 21, }, // America/North_Dakota/Center
    {  462, 8357,  149551, 25, }, // America/North_Dakota/New_Salem
    {  462,  393,  149576,  5, }, // America/Nuuk
    {  462, 8388,  149581,  7, }, // America/Ojinaga
    {  462, 2356,  149588,  6, }, // America/Panama
    {  462, 6173,  149594, 10, }, // America/Paramaribo
    {  462, 2973,  149604,  8, }, // America/Phoenix
    {  462, 3945,  149612, 18, }, // America/Port-au-Prince
    {  462, 5954,  149630, 17, }, // America/Port_of_Spain
    {  462, 8404,  149647, 11, }, // America/Porto_Velho
    {  462, 5843,  149658, 12, }, // America/Puerto_Rico
    {  462, 4030,  149670, 15, }, // America/Punta_Arenas
    {  462, 8424,  149685, 14, }, // America/Rankin_Inlet
    {  462, 8445,  149699,  6, }, // America/Recife
    {  462, 1995,  149705,  6, }, // America/Regina
    {  462, 8460,  149711,  8, }, // America/Resolute
    {  462,  695,  149719, 12, }, // America/Rio_Branco
    {  462, 8477,  149731, 11, }, // America/Santarem
    {  462, 2201,  149742,  9, }, // America/Santiago
    {  462, 6111,  149751, 15, }, // America/Santo_Domingo
    {  462, 1878,  149766,  9, }, // America/Sao_Paulo
    {  462, 6899,  149775, 17, }, // America/Scoresbysund
    {  462, 8494,  149792,  5, }, // America/Sitka
    {  462, 8508,  149797, 18, }, // America/St_Barthelemy
    {  462, 2061,  149815, 12, }, // America/St_Johns
    {  462, 5863,  149827, 13, }, // America/St_Kitts
    {  462, 5880,  149840, 13, }, // America/St_Lucia
    {  462,  855,  149853, 14, }, // America/St_Thomas
    {  462, 5913,  149867, 17, }, // America/St_Vincent
    {  462, 8530,  149884, 15, }, // America/Swift_Current
    {  462, 5461,  149899, 11, }, // America/Tegucigalpa
    {  462, 5756,  149910,  4, }, // America/Thule
    {  462,  313,  149914,  7, }, // America/Tijuana
    {  462,  608,  149921,  8, }, // America/Toronto
    {  462, 5688,  149929,  7, }, // America/Tortola
    {  462, 2093,  149936,  9, }, // America/Vancouver
    {  462, 2144,  149945, 11, }, // America/Whitehorse
    {  462,  734,  149956,  8, }, // America/Winnipeg
    {  462, 8552,  149964,  8, }, // America/Yakutat
    {  462, 6001,  148217,  4, }, // Antarctica/Casey
    {  462, 6094,  149972,  7, }, // Antarctica/Davis
    {  462, 6133,  149979, 19, }, // Antarctica/DumontDUrville
    {  462, 8568,  149998,  8, }, // Antarctica/Macquarie
    {  462, 7066,  150006,  6, }, // Antarctica/Mawson
    {  462, 7113,  150012, 13, }, // Antarctica/McMurdo
    {  462, 8589,  150025,  7, }, // Antarctica/Palmer
    {  462, 7226,  150032,  6, }, // Antarctica/Rothera
    {  462, 7295,  150038,  4, }, // Antarctica/Syowa
    {  462, 8607,  150042,  6, }, // Antarctica/Troll
    {  462, 7374,  150048,  9, }, // Antarctica/Vostok
    {  462, 1429,  150057, 13, }, // Arctic/Longyearbyen
    {  462, 5570,  150070,  6, }, // Asia/Aden
    {  462, 5395,  150076,  6, }, // Asia/Almaty
    {  462, 3968,  150082,  6, }, // Asia/Amman
    {  462, 5511,  150088,  8, }, // Asia/Anadyr
    {  462, 5523,  150096,  5, }, // Asia/Aqtau
    {  462, 5368,  150101,  7, }, // Asia/Aqtobe
    {  462,  964,  150108, 11, }, // Asia/Ashgabat
    {  462, 8624,  150119,  6, }, // Asia/Atyrau
    {  462, 3607,  150125,  8, }, // Asia/Baghdad
    {  462, 5534,  150133,  8, }, // Asia/Bahrain
    {  462, 3653,  148863,  4, }, // Asia/Baku
    {  462, 4466,  150141, 11, }, // Asia/Bangkok
    {  462, 3571,  150152,  8, }, // Asia/Barnaul
    {  462, 4086,  150160,  7, }, // Asia/Beirut
    {  462, 3778,  150167,  8, }, // Asia/Bishkek
    {  462, 5989,  150175,  7, }, // Asia/Brunei
    {  462, 4601,  150182,  4, }, // Asia/Chita
    {  462, 4511,  150186,  6, }, // Asia/Colombo
    {  462, 4540,  150192,  9, }, // Asia/Damascus
    {  462, 1093,  150201,  4, }, // Asia/Dhaka
    {  462, 6192,  150205,  6, }, // Asia/Dili
    {  462, 3596,  150211,  5, }, // Asia/Dubai
    {  462, 6159,  150216,  7, }, // Asia/Dushanbe
    {  462, 8636,  150223, 10, }, // Asia/Famagusta
    {  462, 8651,  150233,  4, }, // Asia/Gaza
    {  462, 4795,  150237,  8, }, // Asia/Hebron
    {  462, 1259,  150245, 16, }, // Asia/Ho_Chi_Minh
    {  462, 2704,  150261,  5, }, // Asia/Hong_Kong
    {  462, 4771,  150266,  6, }, // Asia/Hovd
    {  462, 4184,  150272,  8, }, // Asia/Irkutsk
    {  462, 7006,  150280,  9, }, // Asia/Jakarta
    {  462, 6992,  150289,  6, }, // Asia/Jayapura
    {  462, 1290,  150295,  8, }, // Asia/Jerusalem
    {  462, 3560,  150303,  5, }, // Asia/Kabul
    {  462, 4316,  150308,  8, }, // Asia/Kamchatka
    {  462, 4239,  150316,  5, }, // Asia/Karachi
    {  462, 1185,  150321,  9, }, // Asia/Kathmandu
    {  462, 8661,  150330,  9, }, // Asia/Khandyga
    {  462,  992,  150339,  7, }, // Asia/Kolkata
    {  462, 4197,  150346, 13, }, // Asia/Krasnoyarsk
    {  462, 4859,  150359, 12, }, // Asia/Kuala_Lumpur
    {  462, 5976,  150371,  5, }, // Asia/Kuching
    {  462, 5547,  150376,  4, }, // Asia/Kuwait
    {  462, 1211,  150380,  5, }, // Asia/Macau
    {  462, 4017,  150385,  8, }, // Asia/Magadan
    {  462, 1349,  150393,  9, }, // Asia/Makassar
    {  462, 7182,  150402,  6, }, // Asia/Manila
    {  462, 6920,  150408,  8, }, // Asia/Muscat
    {  462, 2554,  150416,  7, }, // Asia/Nicosia
    {  462, 8675,  150423, 13, }, // Asia/Novokuznetsk
    {  462, 4135,  150436, 12, }, // Asia/Novosibirsk
    {  462, 4229,  150448,  5, }, // Asia/Omsk
    {  462, 7145,  150453,  6, }, // Asia/Oral
    {  462, 6961,  150459, 11, }, // Asia/Phnom_Penh
    {  462, 8693,  150470, 12, }, // Asia/Pontianak
    {  462, 4214,  150482, 11, }, // Asia/Pyongyang
    {  462, 5559,  150493,  5, }, // Asia/Qatar
    {  462, 8708,  150498,  7, }, // Asia/Qostanay
    {  462, 4269,  150505, 10, }, // Asia/Qyzylorda
    {  462, 3584,  150515,  4, }, // Asia/Riyadh
    {  462, 4408,  150519,  7, }, // Asia/Sakhalin
    {  462, 7245,  150526,  7, }, // Asia/Samarkand
    {  462, 3279,  150533,  4, }, // Asia/Seoul
    {  462, 1053,  150537,  6, }, // Asia/Shanghai
    {  462, 3300,  150543, 11, }, // Asia/Singapore
    {  462, 4297,  150554, 15, }, // Asia/Srednekolymsk
    {  462, 3263,  150569,  6, }, // Asia/Taipei
    {  462, 4781,  150575,  8, }, // Asia/Tashkent
    {  462, 3915,  150583,  8, }, // Asia/Tbilisi
    {  462, 2772,  150591,  8, }, // Asia/Tehran
    {  462, 1317,  150599,  6, }, // Asia/Thimphu
    {  462, 2821,  150605,  5, }, // Asia/Tokyo
    {  462, 4572,  150610,  6, }, // Asia/Tomsk
    {  462, 1021,  150616, 12, }, // Asia/Ulaanbaatar
    {  462, 1159,  150628,  6, }, // Asia/Urumqi
    {  462, 8722,  150634, 10, }, // Asia/Ust-Nera
    {  462, 6977,  150644, 11, }, // Asia/Vientiane
    {  462, 4710,  150655, 14, }, // Asia/Vladivostok
    {  462, 4828,  150669,  8, }, // Asia/Yakutsk
    {  462, 1235,  150677,  9, }, // Asia/Yangon
    {  462, 3883,  150686, 14, }, // Asia/Yekaterinburg
    {  462, 3747,  150700,  8, }, // Asia/Yerevan
    {  462, 3663,  150708,  7, }, // Atlantic/Azores
    {  462, 5671,  150715,  7, }, // Atlantic/Bermuda
    {  462, 6628,  150722,  6, }, // Atlantic/Canary
    {  462, 3727,  150728, 11, }, // Atlantic/Cape_Verde
    {  462, 1395,  150739,  5, }, // Atlantic/Faroe
    {  462, 8736,  150744,  7, }, // Atlantic/Madeira
    {  462, 2748,  150751, 10, }, // Atlantic/Reykjavik
    {  462, 7272,  150761, 13, }, // Atlantic/South_Georgia
    {  462, 6821,  150774, 14, }, // Atlantic/St_Helena
    {  462, 6644,  150788,  7, }, // Atlantic/Stanley
    {  462, 1670,  150795,  8, }, // Australia/Adelaide
    {  462, 1635,  150803, 10, }, // Australia/Brisbane
    {  462, 1799,  150813, 15, }, // Australia/Broken_Hill
    {  462, 1583,  150828,  8, }, // Australia/Darwin
    {  462, 3637,  150836,  5, }, // Australia/Eucla
    {  462, 1516,  150841,  9, }, // Australia/Hobart
    {  462, 8753,  150850, 13, }, // Australia/Lindeman
    {  462, 1547,  150863,  9, }, // Australia/Lord_Howe
    {  462, 1727,  150872, 10, }, // Australia/Melbourne
    {  462, 1762,  150882,  6, }, // Australia/Perth
    {  462, 1463,  150888,  5, }, // Australia/Sydney
    {  462, 6441,  150893, 10, }, // Europe/Amsterdam
    {  462, 6216,  150903,  6, }, // Europe/Andorra
    {  462, 3620,  150909, 11, }, // Europe/Astrakhan
    {  462, 2300,  150920,  6, }, // Europe/Athens
    {  462, 6336,  150926, 10, }, // Europe/Belgrade
    {  462, 4757,  150936,  7, }, // Europe/Berlin
    {  462, 6488,  150943, 11, }, // Europe/Bratislava
    {  462, 2167,  150954,  8, }, // Europe/Brussels
    {  462, 3928,  150962, 10, }, // Europe/Bucharest
    {  462, 3806,  150972, 10, }, // Europe/Budapest
    {  462, 8772,  150982,  9, }, // Europe/Busingen
    {  462, 2583,  150991,  6, }, // Europe/Chisinau
    {  462, 6289,  150997, 11, }, // Europe/Copenhagen
    {  462, 2338,  151008,  6, }, // Europe/Dublin
    {  462, 6307,  151014, 11, }, // Europe/Gibraltar
    {  462, 8788,  151025,  7, }, // Europe/Guernsey
    {  462, 6612,  151032,  8, }, // Europe/Helsinki
    {  462, 8804,  151040, 13, }, // Europe/Isle_of_Man
    {  462, 1130,  151053,  9, }, // Europe/Istanbul
    {  462, 8823,  151062,  5, }, // Europe/Jersey
    {  462, 3979,  151067, 13, }, // Europe/Kaliningrad
    {  462, 8929,  151080,  5, }, // Europe/Kirov
    {  462, 2527,  151085,  5, }, // Europe/Kyiv
    {  462, 3213,  151090,  7, }, // Europe/Lisbon
    {  462, 6506,  151097, 10, }, // Europe/Ljubljana
    {  462, 2501,  151107,  6, }, // Europe/London
    {  462, 6365,  151113,  9, }, // Europe/Luxembourg
    {  462, 4892,  151122,  8, }, // Europe/Madrid
    {  462, 6397,  151130,  5, }, // Europe/Malta
    {  462, 6582,  151135, 10, }, // Europe/Mariehamn
    {  462, 3693,  151145,  6, }, // Europe/Minsk
    {  462, 6410,  151151,  6, }, // Europe/Monaco
    {  462, 3537,  151157,  5, }, // Europe/Moscow
    {  462, 6458,  151162,  4, }, // Europe/Oslo
    {  462, 4284,  151166,  6, }, // Europe/Paris
    {  462, 6424,  151172,  9, }, // Europe/Podgorica
    {  462, 6275,  151181,  6, }, // Europe/Prague
    {  462, 8837,  151187,  4, }, // Europe/Riga
    {  462, 6324,  151191,  4, }, // Europe/Rome
    {  462, 4331,  151195,  6, }, // Europe/Samara
    {  462, 6470,  151201, 11, }, // Europe/San_Marino
    {  462, 6245,  151212,  8, }, // Europe/Sarajevo
    {  462, 4451,  151220,  8, }, // Europe/Saratov
    {  462, 8849,  151228, 11, }, // Europe/Simferopol
    {  462, 6383,  151239,  7, }, // Europe/Skopje
    {  462, 6599,  151246,  5, }, // Europe/Sofia
    {  462, 6523,  151251,  9, }, // Europe/Stockholm
    {  462, 8867,  151260,  7, }, // Europe/Tallinn
    {  462, 6202,  151267,  6, }, // Europe/Tirane
    {  462, 8882,  151273, 11, }, // Europe/Ulyanovsk
    {  462, 6352,  151284,  5, }, // Europe/Vaduz
    {  462, 6567,  151289,  8, }, // Europe/Vatican
    {  462, 6231,  151297,  5, }, // Europe/Vienna
    {  462, 8899,  151302,  8, }, // Europe/Vilnius
    {  462, 4727,  151310, 11, }, // Europe/Volgograd
    {  462, 3190,  151321,  6, }, // Europe/Warsaw
    {  462, 6261,  151327,  8, }, // Europe/Zagreb
    {  462, 6540,  151335,  6, }, // Europe/Zurich
    {  462, 5078,  151341, 13, }, // Indian/Antananarivo
    {  462, 6947,  151354,  7, }, // Indian/Chagos
    {  462, 6046,  151361, 10, }, // Indian/Christmas
    {  462, 6063,  151371,  7, }, // Indian/Cocos
    {  462, 5029,  151378,  6, }, // Indian/Comoro
    {  462, 6661,  151384,  9, }, // Indian/Kerguelen
    {  462, 7260,  151393,  4, }, // Indian/Mahe
    {  462, 7050,  151397,  8, }, // Indian/Maldives
    {  462, 4069,  151405,  7, }, // Indian/Mauritius
    {  462, 5098,  151412,  6, }, // Indian/Mayotte
    {  462, 7211,  151418, 10, }, // Indian/Reunion
    {  462, 4422,  151428,  4, }, // Pacific/Apia
    {  462,  932,  151432,  7, }, // Pacific/Auckland
    {  462, 3706,  151439, 13, }, // Pacific/Bougainville
    {  462, 3015,  151452,  7, }, // Pacific/Chatham
    {  462, 3157,  151459,  5, }, // Pacific/Chuuk
    {  462, 2237,  151464,  7, }, // Pacific/Easter
    {  462, 7360,  151471,  6, }, // Pacific/Efate
    {  462, 7327,  151477,  7, }, // Pacific/Fakaofo
    {  462, 3902,  151484,  4, }, // Pacific/Fiji
    {  462, 7343,  151488,  8, }, // Pacific/Funafuti
    {  462, 6678,  151496,  9, }, // Pacific/Galapagos
    {  462, 6696,  151505, 10, }, // Pacific/Gambier
    {  462, 3822,  151515, 11, }, // Pacific/Guadalcanal
    {  462, 6018,  149348,  4, }, // Pacific/Guam
    {  462, 2723,  151526,  8, }, // Pacific/Honolulu
    {  462, 3049,  151534, 10, }, // Pacific/Kanton
    {  462, 3998,  151544, 10, }, // Pacific/Kiritimati
    {  462, 7019,  151554,  6, }, // Pacific/Kosrae
    {  462, 2842,  151560, 11, }, // Pacific/Kwajalein
    {  462, 4877,  151571,  6, }, // Pacific/Majuro
    {  462, 4051,  151577, 13, }, // Pacific/Marquesas
    {  462, 8914,  151590,  7, }, // Pacific/Midway
    {  462, 7084,  151597,  5, }, // Pacific/Nauru
    {  462, 7132,  151602,  4, }, // Pacific/Niue
    {  462, 4168,  151606,  7, }, // Pacific/Norfolk
    {  462, 7098,  151613,  5, }, // Pacific/Noumea
    {  462, 3126,  151618,  9, }, // Pacific/Pago_Pago
    {  462, 7155,  151627,  5, }, // Pacific/Palau
    {  462, 7194,  151632, 10, }, // Pacific/Pitcairn
    {  462, 3096,  151642,  7, }, // Pacific/Pohnpei
    {  462, 4807,  151649, 15, }, // Pacific/Port_Moresby
    {  462, 6076,  151664, 10, }, // Pacific/Rarotonga
    {  462, 6031,  151674,  7, }, // Pacific/Saipan
    {  462, 7312,  151681,  6, }, // Pacific/Tahiti
    {  462, 6712,  151687,  5, }, // Pacific/Tarawa
    {  462, 4583,  151692, 10, }, // Pacific/Tongatapu
    {  462, 7392,  151702,  4, }, // Pacific/Wake
    {  462, 7405,  151706,  7, }, // Pacific/Wallis
    {  463,   14,       0,  6, }, // Africa/Asmara Oromo/Latin/Ethiopia
    {  463, 4435,       6,  8, }, // Africa/Sao_Tome
    {  463, 4252,      14,  8, }, // America/Asuncion
    {  463,  237,      22,  8, }, // America/Atikokan
    {  463, 7623,      30, 17, }, // America/Bahia_Banderas
    {  463, 3868,      47,  6, }, // America/Cancun
    {  463, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  463, 5723,      66,  7, }, // America/Curacao
    {  463,  481,      73, 13, }, // America/Indiana/Knox
    {  463, 7961,      86, 16, }, // America/Indiana/Marengo
    {  463, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  463, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  463, 8038,     139, 14, }, // America/Indiana/Vevay
    {  463, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  463, 8086,     171, 16, }, // America/Indiana/Winamac
    {  463, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  463, 5932,     207, 22, }, // America/Lower_Princes
    {  463, 8220,     229,  6, }, // America/Merida
    {  463, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  463, 8329,     255, 20, }, // America/North_Dakota/Center
    {  463, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  463,  393,     298,  4, }, // America/Nuuk
    {  463, 6899,     302, 16, }, // America/Scoresbysund
    {  463, 8508,     318, 14, }, // America/St_Barthelemy
    {  463, 2061,     332, 10, }, // America/St_Johns
    {  463, 5863,     342,  9, }, // America/St_Kitts
    {  463, 5880,     351,  9, }, // America/St_Lucia
    {  463,  855,     360, 10, }, // America/St_Thomas
    {  463, 5913,     370, 11, }, // America/St_Vincent
    {  463, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  463, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  463, 1185,     413,  9, }, // Asia/Kathmandu
    {  463,  992,     422,  7, }, // Asia/Kolkata
    {  463, 1211,     429,  5, }, // Asia/Macau
    {  463, 1235,     442,  6, }, // Asia/Yangon
    {  463, 1395,     448,  5, }, // Atlantic/Faroe
    {  463, 6821,     453, 10, }, // Atlantic/St_Helena
    {  463, 2527,     463,  4, }, // Europe/Kyiv
    {  463, 7211,     467,  7, }, // Indian/Reunion
    {  463, 3157,     474,  5, }, // Pacific/Chuuk
    {  463, 3096,     479,  7, }, // Pacific/Pohnpei
    {  465,   14,       0,  6, }, // Africa/Asmara Osage/Osage/United States
    {  465, 4435,       6,  8, }, // Africa/Sao_Tome
    {  465, 4252,      14,  8, }, // America/Asuncion
    {  465,  237,      22,  8, }, // America/Atikokan
    {  465, 7623,      30, 17, }, // America/Bahia_Banderas
    {  465, 3868,      47,  6, }, // America/Cancun
    {  465, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  465, 5723,      66,  7, }, // America/Curacao
    {  465,  481,      73, 13, }, // America/Indiana/Knox
    {  465, 7961,      86, 16, }, // America/Indiana/Marengo
    {  465, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  465, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  465, 8038,     139, 14, }, // America/Indiana/Vevay
    {  465, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  465, 8086,     171, 16, }, // America/Indiana/Winamac
    {  465, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  465, 5932,     207, 22, }, // America/Lower_Princes
    {  465, 8220,     229,  6, }, // America/Merida
    {  465, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  465, 8329,     255, 20, }, // America/North_Dakota/Center
    {  465, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  465,  393,     298,  4, }, // America/Nuuk
    {  465, 6899,     302, 16, }, // America/Scoresbysund
    {  465, 8508,     318, 14, }, // America/St_Barthelemy
    {  465, 2061,     332, 10, }, // America/St_Johns
    {  465, 5863,     342,  9, }, // America/St_Kitts
    {  465, 5880,     351,  9, }, // America/St_Lucia
    {  465,  855,     360, 10, }, // America/St_Thomas
    {  465, 5913,     370, 11, }, // America/St_Vincent
    {  465, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  465, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  465, 1185,     413,  9, }, // Asia/Kathmandu
    {  465,  992,     422,  7, }, // Asia/Kolkata
    {  465, 1211,     429,  5, }, // Asia/Macau
    {  465, 1235,     442,  6, }, // Asia/Yangon
    {  465, 1395,     448,  5, }, // Atlantic/Faroe
    {  465, 6821,     453, 10, }, // Atlantic/St_Helena
    {  465, 2527,     463,  4, }, // Europe/Kyiv
    {  465, 7211,     467,  7, }, // Indian/Reunion
    {  465, 3157,     474,  5, }, // Pacific/Chuuk
    {  465, 3096,     479,  7, }, // Pacific/Pohnpei
    {  467,   14,       0,  6, }, // Africa/Asmara Ossetic/Cyrillic/Russia
    {  467, 4435,       6,  8, }, // Africa/Sao_Tome
    {  467, 4252,      14,  8, }, // America/Asuncion
    {  467,  237,      22,  8, }, // America/Atikokan
    {  467, 7623,      30, 17, }, // America/Bahia_Banderas
    {  467, 3868,      47,  6, }, // America/Cancun
    {  467, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  467, 5723,      66,  7, }, // America/Curacao
    {  467,  481,      73, 13, }, // America/Indiana/Knox
    {  467, 7961,      86, 16, }, // America/Indiana/Marengo
    {  467, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  467, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  467, 8038,     139, 14, }, // America/Indiana/Vevay
    {  467, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  467, 8086,     171, 16, }, // America/Indiana/Winamac
    {  467, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  467, 5932,     207, 22, }, // America/Lower_Princes
    {  467, 8220,     229,  6, }, // America/Merida
    {  467, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  467, 8329,     255, 20, }, // America/North_Dakota/Center
    {  467, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  467,  393,     298,  4, }, // America/Nuuk
    {  467, 6899,     302, 16, }, // America/Scoresbysund
    {  467, 8508,     318, 14, }, // America/St_Barthelemy
    {  467, 2061,     332, 10, }, // America/St_Johns
    {  467, 5863,     342,  9, }, // America/St_Kitts
    {  467, 5880,     351,  9, }, // America/St_Lucia
    {  467,  855,     360, 10, }, // America/St_Thomas
    {  467, 5913,     370, 11, }, // America/St_Vincent
    {  467, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  467, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  467, 1185,     413,  9, }, // Asia/Kathmandu
    {  467,  992,     422,  7, }, // Asia/Kolkata
    {  467, 1211,     429,  5, }, // Asia/Macau
    {  467, 3915,   35074,  6, }, // Asia/Tbilisi
    {  467, 1235,     442,  6, }, // Asia/Yangon
    {  467, 1395,     448,  5, }, // Atlantic/Faroe
    {  467, 6821,     453, 10, }, // Atlantic/St_Helena
    {  467, 2527,     463,  4, }, // Europe/Kyiv
    {  467, 3693,    3232,  5, }, // Europe/Minsk
    {  467, 3537,  151713,  6, }, // Europe/Moscow
    {  467, 7211,     467,  7, }, // Indian/Reunion
    {  467, 3157,     474,  5, }, // Pacific/Chuuk
    {  467, 3096,     479,  7, }, // Pacific/Pohnpei
    {  468,   14,       0,  6, }, // Africa/Asmara Papiamento/Latin/Curacao
    {  468, 4435,       6,  8, }, // Africa/Sao_Tome
    {  468, 4252,      14,  8, }, // America/Asuncion
    {  468,  237,      22,  8, }, // America/Atikokan
    {  468, 7623,      30, 17, }, // America/Bahia_Banderas
    {  468, 3868,      47,  6, }, // America/Cancun
    {  468, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  468, 5723,  151719,  6, }, // America/Curacao
    {  468,  481,      73, 13, }, // America/Indiana/Knox
    {  468, 7961,      86, 16, }, // America/Indiana/Marengo
    {  468, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  468, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  468, 8038,     139, 14, }, // America/Indiana/Vevay
    {  468, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  468, 8086,     171, 16, }, // America/Indiana/Winamac
    {  468, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  468, 5932,     207, 22, }, // America/Lower_Princes
    {  468, 8220,     229,  6, }, // America/Merida
    {  468, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  468, 8329,     255, 20, }, // America/North_Dakota/Center
    {  468, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  468,  393,     298,  4, }, // America/Nuuk
    {  468, 2356,   20547,  6, }, // America/Panama
    {  468, 6111,  151725, 19, }, // America/Santo_Domingo
    {  468, 6899,     302, 16, }, // America/Scoresbysund
    {  468, 8508,     318, 14, }, // America/St_Barthelemy
    {  468, 2061,    3975, 11, }, // America/St_Johns
    {  468, 5863,    3986, 10, }, // America/St_Kitts
    {  468, 5880,    3996, 10, }, // America/St_Lucia
    {  468,  855,    4006, 11, }, // America/St_Thomas
    {  468, 5913,    4017, 12, }, // America/St_Vincent
    {  468, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  468, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  468, 1290,  151744,  9, }, // Asia/Jerusalem
    {  468, 1185,     413,  9, }, // Asia/Kathmandu
    {  468,  992,     422,  7, }, // Asia/Kolkata
    {  468, 1211,     429,  5, }, // Asia/Macau
    {  468, 1235,     442,  6, }, // Asia/Yangon
    {  468, 3663,  151753,  6, }, // Atlantic/Azores
    {  468, 3727,  151759, 10, }, // Atlantic/Cape_Verde
    {  468, 1395,     448,  5, }, // Atlantic/Faroe
    {  468, 6821,     453, 10, }, // Atlantic/St_Helena
    {  468, 2300,  151769,  6, }, // Europe/Athens
    {  468, 2527,     463,  4, }, // Europe/Kyiv
    {  468, 2501,  151775,  6, }, // Europe/London
    {  468, 6324,    5946,  4, }, // Europe/Rome
    {  468, 7211,     467,  7, }, // Indian/Reunion
    {  468, 3157,     474,  5, }, // Pacific/Chuuk
    {  468, 3096,     479,  7, }, // Pacific/Pohnpei
    {  470, 6788,  151781,  6, }, // Africa/Abidjan Pashto/Arabic/Afghanistan
    {  470, 6760,  151787,  4, }, // Africa/Accra
    {  470, 5059,  151791, 10, }, // Africa/Addis_Ababa
    {  470, 7420,  151801,  7, }, // Africa/Algiers
    {  470,   14,  151808,  6, }, // Africa/Asmara
    {  470,   44,  151814,  6, }, // Africa/Bamako
    {  470, 5257,  151820,  6, }, // Africa/Bangui
    {  470, 6746,   10427,  6, }, // Africa/Banjul
    {  470, 7435,  151826,  4, }, // Africa/Bissau
    {  470, 4957,  151830,  7, }, // Africa/Blantyre
    {  470, 5287,  151837,  8, }, // Africa/Brazzaville
    {  470, 4922,  151845,  7, }, // Africa/Bujumbura
    {  470, 2320,  151852,  5, }, // Africa/Cairo
    {  470, 4117,  151857, 10, }, // Africa/Casablanca
    {  470, 7449,  151867,  5, }, // Africa/Ceuta
    {  470, 6773,  151872,  6, }, // Africa/Conakry
    {  470, 6840,  151878,  5, }, // Africa/Dakar
    {  470, 5130,   10499, 10, }, // Africa/Dar_es_Salaam
    {  470, 5043,  151883,  5, }, // Africa/Djibouti
    {  470, 5243,   10515,  5, }, // Africa/Douala
    {  470, 5166,  151888,  5, }, // Africa/El_Aaiun
    {  470, 6853,  151893,  6, }, // Africa/Freetown
    {  470, 4906,  151899,  6, }, // Africa/Gaborone
    {  470, 5015,  151905,  5, }, // Africa/Harare
    {  470, 4479,  151910,  9, }, // Africa/Johannesburg
    {  470, 4499,   10555,  4, }, // Africa/Juba
    {  470, 5151,  151919,  6, }, // Africa/Kampala
    {  470, 4524,  151925,  5, }, // Africa/Khartoum
    {  470, 4973,  151930,  6, }, // Africa/Kigali
    {  470, 5306,  151936,  7, }, // Africa/Kinshasa
    {  470, 4744,  106771,  5, }, // Africa/Lagos
    {  470, 5336,  151943,  6, }, // Africa/Libreville
    {  470, 6869,  151949,  4, }, // Africa/Lome
    {  470, 5211,  151953,  5, }, // Africa/Luanda
    {  470, 4939,  151958,  7, }, // Africa/Lubumbashi
    {  470, 5001,  103214,  6, }, // Africa/Lusaka
    {  470, 5322,   10622,  6, }, // Africa/Malabo
    {  470, 4987,  151965,  6, }, // Africa/Maputo
    {  470, 5197,  151971,  5, }, // Africa/Maseru
    {  470, 5182,  151976,  7, }, // Africa/Mbabane
    {  470, 5113,  151983,  7, }, // Africa/Mogadishu
    {  470, 7034,  151990,  8, }, // Africa/Monrovia
    {  470, 3853,  106844,  7, }, // Africa/Nairobi
    {  470, 5271,  151998,  7, }, // Africa/Ndjamena
    {  470, 5354,  152005,  4, }, // Africa/Niamey
    {  470, 6803,  152009,  6, }, // Africa/Nouakchott
    {  470, 6727,  152015,  8, }, // Africa/Ouagadougou
    {  470, 5225,  152023, 11, }, // Africa/Porto-Novo
    {  470, 4435,  152034,  7, }, // Africa/Sao_Tome
    {  470, 2866,  106898,  7, }, // Africa/Tripoli
    {  470, 6554,   10719,  4, }, // Africa/Tunis
    {  470, 4152,  152041,  6, }, // Africa/Windhoek
    {  470,  132,  152047,  4, }, // America/Adak
    {  470, 3346,  152051,  7, }, // America/Anchorage
    {  470, 5607,  152058,  6, }, // America/Anguilla
    {  470, 5624,  152064,  5, }, // America/Antigua
    {  470, 4554,  152069,  7, }, // America/Araguaina
    {  470,  166,  152076, 11, }, // America/Argentina/Buenos_Aires
    {  470,   91,  152087,  8, }, // America/Argentina/Catamarca
    {  470,  270,  152095,  7, }, // America/Argentina/Cordoba
    {  470,  441,  152102,  5, }, // America/Argentina/Jujuy
    {  470, 7462,  152107,  7, }, // America/Argentina/La_Rioja
    {  470,  565,  152114,  7, }, // America/Argentina/Mendoza
    {  470, 7489,  152121, 11, }, // America/Argentina/Rio_Gallegos
    {  470, 7520,  152132,  5, }, // America/Argentina/Salta
    {  470, 7544,  152137,  8, }, // America/Argentina/San_Juan
    {  470, 5580,  152145,  8, }, // America/Argentina/San_Luis
    {  470, 7571,  152153,  6, }, // America/Argentina/Tucuman
    {  470, 7597,  152159,  7, }, // America/Argentina/Ushuaia
    {  470, 5640,  152166,  5, }, // America/Aruba
    {  470, 4252,  152171,  8, }, // America/Asuncion
    {  470,  237,  152179,  7, }, // America/Atikokan
    {  470, 3679,  152186,  4, }, // America/Bahia
    {  470, 7623,  152190, 12, }, // America/Bahia_Banderas
    {  470, 5654,  107069,  8, }, // America/Barbados
    {  470, 7646,   10902,  3, }, // America/Belem
    {  470, 5407,  152202,  4, }, // America/Belize
    {  470, 7660,  152206, 12, }, // America/Blanc-Sablon
    {  470, 7681,  152218,  9, }, // America/Boa_Vista
    {  470, 4361,  152227,  5, }, // America/Bogota
    {  470, 7699,  152232,  3, }, // America/Boise
    {  470, 7713,  152235,  9, }, // America/Cambridge_Bay
    {  470, 7735,  152244,  9, }, // America/Campo_Grande
    {  470, 3868,  152253,  5, }, // America/Cancun
    {  470, 4694,  152258,  7, }, // America/Caracas
    {  470, 4345,  152265,  4, }, // America/Cayenne
    {  470, 5496,  152269,  4, }, // America/Cayman
    {  470, 2260,  152273,  6, }, // America/Chicago
    {  470, 7756,  152279,  7, }, // America/Chihuahua
    {  470, 7774,  152286, 13, }, // America/Ciudad_Juarez
    {  470, 5422,  152299,  9, }, // America/Costa_Rica
    {  470, 7796,  152308,  6, }, // America/Creston
    {  470, 3791,  152314,  6, }, // America/Cuiaba
    {  470, 5723,  152320,  7, }, // America/Curacao
    {  470, 7812,  152327,  9, }, // America/Danmarkshavn
    {  470, 7833,   11058,  5, }, // America/Dawson
    {  470, 7848,  152336,  9, }, // America/Dawson_Creek
    {  470,  805,  152345,  4, }, // America/Denver
    {  470, 3465,  152349,  7, }, // America/Detroit
    {  470, 5739,  152356,  8, }, // America/Dominica
    {  470,  893,  152364,  8, }, // America/Edmonton
    {  470, 7869,  152372,  7, }, // America/Eirunepe
    {  470, 5441,  152379, 11, }, // America/El_Salvador
    {  470, 7886,  152390, 10, }, // America/Fort_Nelson
    {  470, 7906,  152400,  8, }, // America/Fortaleza
    {  470, 7924,  152408,  8, }, // America/Glace_Bay
    {  470, 6881,  152416,  6, }, // America/Goose_Bay
    {  470, 4612,  152422,  7, }, // America/Grand_Turk
    {  470, 5770,  152429,  6, }, // America/Grenada
    {  470, 5786,  152435,  6, }, // America/Guadeloupe
    {  470, 3760,  152441,  8, }, // America/Guatemala
    {  470, 4841,  152449,  6, }, // America/Guayaquil
    {  470, 6932,  152455,  5, }, // America/Guyana
    {  470, 1939,  152460,  6, }, // America/Halifax
    {  470, 2281,  152466,  5, }, // America/Havana
    {  470, 7942,  152471,  8, }, // America/Hermosillo
    {  470,  348,  152479, 13, }, // America/Indiana/Indianapolis
    {  470,  481,  152492, 12, }, // America/Indiana/Knox
    {  470, 7961,  152504, 15, }, // America/Indiana/Marengo
    {  470, 7985,  152519, 16, }, // America/Indiana/Petersburg
    {  470, 8012,  152535, 15, }, // America/Indiana/Tell_City
    {  470, 8038,  152550, 13, }, // America/Indiana/Vevay
    {  470, 8060,  152563, 16, }, // America/Indiana/Vincennes
    {  470, 8086,  152579, 14, }, // America/Indiana/Winamac
    {  470, 8110,  152593,  4, }, // America/Inuvik
    {  470,  660,  152597,  5, }, // America/Iqaluit
    {  470, 2799,  152602,  6, }, // America/Jamaica
    {  470, 5380,  152608,  4, }, // America/Juneau
    {  470,  521,  152612,  8, }, // America/Kentucky/Louisville
    {  470, 8125,  152620, 17, }, // America/Kentucky/Monticello
    {  470, 5704,  152637,  8, }, // America/Kralendijk
    {  470, 4376,  104035,  6, }, // America/La_Paz
    {  470, 7169,  152645,  4, }, // America/Lima
    {  470, 3239,  152649, 10, }, // America/Los_Angeles
    {  470, 5932,  152659, 16, }, // America/Lower_Princes
    {  470, 8153,  152675,  5, }, // America/Maceio
    {  470, 8168,  152680,  5, }, // America/Managua
    {  470, 1908,  152685,  5, }, // America/Manaus
    {  470, 5897,  152690,  7, }, // America/Marigot
    {  470, 5805,  152697,  8, }, // America/Martinique
    {  470, 8184,  152705,  9, }, // America/Matamoros
    {  470, 2917,  152714,  7, }, // America/Mazatlan
    {  470, 8202,  152721,  7, }, // America/Menominee
    {  470, 8220,  152728,  6, }, // America/Merida
    {  470, 8235,  152734, 10, }, // America/Metlakatla
    {  470, 2949,  152744, 10, }, // America/Mexico_City
    {  470, 4391,  152754,  8, }, // America/Miquelon
    {  470, 8254,  152762,  7, }, // America/Moncton
    {  470, 8270,  152769,  6, }, // America/Monterrey
    {  470, 4098,  152775,  9, }, // America/Montevideo
    {  470, 5824,  152784,  9, }, // America/Montserrat
    {  470, 5481,  152793,  5, }, // America/Nassau
    {  470, 2379,  152798,  7, }, // America/New_York
    {  470, 8288,   11422,  3, }, // America/Nome
    {  470, 1850,  152805,  7, }, // America/Noronha
    {  470, 8301,  152812, 19, }, // America/North_Dakota/Beulah
    {  470, 8329,  152831, 18, }, // America/North_Dakota/Center
    {  470, 8357,  152849, 22, }, // America/North_Dakota/New_Salem
    {  470,  393,  152871,  4, }, // America/Nuuk
    {  470, 8388,  152875,  6, }, // America/Ojinaga
    {  470, 2356,  107831,  6, }, // America/Panama
    {  470, 6173,  152881,  9, }, // America/Paramaribo
    {  470, 2973,  152890,  5, }, // America/Phoenix
    {  470, 3945,  152895, 15, }, // America/Port-au-Prince
    {  470, 5954,  152910, 14, }, // America/Port_of_Spain
    {  470, 8404,  152924, 11, }, // America/Porto_Velho
    {  470, 5843,  152935, 10, }, // America/Puerto_Rico
    {  470, 4030,  152945, 11, }, // America/Punta_Arenas
    {  470, 8424,  152956, 10, }, // America/Rankin_Inlet
    {  470, 8445,  152966,  6, }, // America/Recife
    {  470, 1995,  152972,  6, }, // America/Regina
    {  470, 8460,  152978,  7, }, // America/Resolute
    {  470,  695,  152985, 10, }, // America/Rio_Branco
    {  470, 8477,  152995,  6, }, // America/Santarem
    {  470, 2201,  153001,  7, }, // America/Santiago
    {  470, 6111,  153008, 12, }, // America/Santo_Domingo
    {  470, 1878,  153020,  9, }, // America/Sao_Paulo
    {  470, 6899,  153029, 11, }, // America/Scoresbysund
    {  470, 8494,  153040,  5, }, // America/Sitka
    {  470, 8508,  153045, 13, }, // America/St_Barthelemy
    {  470, 2061,  153058,  9, }, // America/St_Johns
    {  470, 5863,  153067,  8, }, // America/St_Kitts
    {  470, 5880,  153075, 10, }, // America/St_Lucia
    {  470,  855,  153085, 10, }, // America/St_Thomas
    {  470, 5913,  153095, 10, }, // America/St_Vincent
    {  470, 8530,  153105, 10, }, // America/Swift_Current
    {  470, 5461,  153115,  9, }, // America/Tegucigalpa
    {  470, 5756,   11829,  3, }, // America/Thule
    {  470,  313,  153124,  6, }, // America/Tijuana
    {  470,  608,  153130,  6, }, // America/Toronto
    {  470, 5688,  153136,  7, }, // America/Tortola
    {  470, 2093,  153143,  7, }, // America/Vancouver
    {  470, 2144,  153150,  9, }, // America/Whitehorse
    {  470,  734,  153159,  6, }, // America/Winnipeg
    {  470, 8552,  153165,  6, }, // America/Yakutat
    {  470, 6001,  153171,  4, }, // Antarctica/Casey
    {  470, 6094,  153175,  4, }, // Antarctica/Davis
    {  470, 6133,  153179, 14, }, // Antarctica/DumontDUrville
    {  470, 8568,  153193,  6, }, // Antarctica/Macquarie
    {  470, 7066,  153199,  5, }, // Antarctica/Mawson
    {  470, 7113,  153204,  6, }, // Antarctica/McMurdo
    {  470, 8589,  153210,  5, }, // Antarctica/Palmer
    {  470, 7226,  153215,  5, }, // Antarctica/Rothera
    {  470, 7295,  153220,  4, }, // Antarctica/Syowa
    {  470, 8607,  153224,  4, }, // Antarctica/Troll
    {  470, 7374,  153228,  6, }, // Antarctica/Vostok
    {  470, 1429,  153234,  9, }, // Arctic/Longyearbyen
    {  470, 5570,  153243,  3, }, // Asia/Aden
    {  470, 5395,  153246,  6, }, // Asia/Almaty
    {  470, 3968,  108226,  5, }, // Asia/Amman
    {  470, 5511,  153252,  6, }, // Asia/Anadyr
    {  470, 5523,  153258,  5, }, // Asia/Aqtau
    {  470, 5368,  153263,  5, }, // Asia/Aqtobe
    {  470,  964,  153268,  8, }, // Asia/Ashgabat
    {  470, 8624,  153276,  6, }, // Asia/Atyrau
    {  470, 3607,   11991,  5, }, // Asia/Baghdad
    {  470, 5534,  153282,  5, }, // Asia/Bahrain
    {  470, 3653,  152318,  4, }, // Asia/Baku
    {  470, 4466,  153287,  5, }, // Asia/Bangkok
    {  470, 3571,  104770,  5, }, // Asia/Barnaul
    {  470, 4086,  153292,  5, }, // Asia/Beirut
    {  470, 3778,  153297,  5, }, // Asia/Bishkek
    {  470, 5989,  153302,  6, }, // Asia/Brunei
    {  470, 4601,  108310,  4, }, // Asia/Chita
    {  470, 4511,  153308,  6, }, // Asia/Colombo
    {  470, 4540,   12044,  4, }, // Asia/Damascus
    {  470, 1093,  153314,  5, }, // Asia/Dhaka
    {  470, 6192,  153319,  3, }, // Asia/Dili
    {  470, 3596,  153322,  4, }, // Asia/Dubai
    {  470, 6159,  153326,  6, }, // Asia/Dushanbe
    {  470, 8636,  153332,  8, }, // Asia/Famagusta
    {  470, 8651,  153340,  3, }, // Asia/Gaza
    {  470, 4795,  153343,  5, }, // Asia/Hebron
    {  470, 1259,  153348, 12, }, // Asia/Ho_Chi_Minh
    {  470, 2704,  153360,  9, }, // Asia/Hong_Kong
    {  470, 4771,  153369,  4, }, // Asia/Hovd
    {  470, 4184,  153373,  7, }, // Asia/Irkutsk
    {  470, 7006,  104877,  6, }, // Asia/Jakarta
    {  470, 6992,  153380,  8, }, // Asia/Jayapura
    {  470, 1290,  153388,  6, }, // Asia/Jerusalem
    {  470, 3560,  153394,  4, }, // Asia/Kabul
    {  470, 4316,  153398,  8, }, // Asia/Kamchatka
    {  470, 4239,  153406,  5, }, // Asia/Karachi
    {  470, 1185,  153411,  6, }, // Asia/Kathmandu
    {  470, 8661,  153417,  6, }, // Asia/Khandyga
    {  470,  992,  153423,  6, }, // Asia/Kolkata
    {  470, 4197,  153429, 11, }, // Asia/Krasnoyarsk
    {  470, 4859,  153440,  9, }, // Asia/Kuala_Lumpur
    {  470, 5976,  153449,  5, }, // Asia/Kuching
    {  470, 5547,  153454,  4, }, // Asia/Kuwait
    {  470, 1211,  153458,  4, }, // Asia/Macau
    {  470, 4017,  153462,  5, }, // Asia/Magadan
    {  470, 1349,  153467,  6, }, // Asia/Makassar
    {  470, 7182,  153473,  5, }, // Asia/Manila
    {  470, 6920,   12250,  4, }, // Asia/Muscat
    {  470, 2554,  153478,  7, }, // Asia/Nicosia
    {  470, 8675,  153485, 10, }, // Asia/Novokuznetsk
    {  470, 4135,  153495,  9, }, // Asia/Novosibirsk
    {  470, 4229,  105031,  5, }, // Asia/Omsk
    {  470, 7145,  153504,  4, }, // Asia/Oral
    {  470, 6961,  153508,  7, }, // Asia/Phnom_Penh
    {  470, 8693,  153515,  8, }, // Asia/Pontianak
    {  470, 4214,  153523, 10, }, // Asia/Pyongyang
    {  470, 5559,   12318,  3, }, // Asia/Qatar
    {  470, 8708,  153533,  7, }, // Asia/Qostanay
    {  470, 4269,  153540, 10, }, // Asia/Qyzylorda
    {  470, 3584,   12340,  4, }, // Asia/Riyadh
    {  470, 4408,  153550,  6, }, // Asia/Sakhalin
    {  470, 7245,   12350,  6, }, // Asia/Samarkand
    {  470, 3279,  105104,  4, }, // Asia/Seoul
    {  470, 1053,  153556,  7, }, // Asia/Shanghai
    {  470, 3300,  153563,  8, }, // Asia/Singapore
    {  470, 4297,  153571, 13, }, // Asia/Srednekolymsk
    {  470, 3263,  153584,  5, }, // Asia/Taipei
    {  470, 4781,  153589,  6, }, // Asia/Tashkent
    {  470, 3915,  153595,  6, }, // Asia/Tbilisi
    {  470, 2772,  153601,  5, }, // Asia/Tehran
    {  470, 1317,  153606,  6, }, // Asia/Thimphu
    {  470, 2821,  153612,  5, }, // Asia/Tokyo
    {  470, 4572,   10632,  5, }, // Asia/Tomsk
    {  470, 1021,  153617, 10, }, // Asia/Ulaanbaatar
    {  470, 1159,  153627,  7, }, // Asia/Urumqi
    {  470, 8722,  153634,  8, }, // Asia/Ust-Nera
    {  470, 6977,  153642,  6, }, // Asia/Vientiane
    {  470, 4710,  153648, 10, }, // Asia/Vladivostok
    {  470, 4828,  153658,  7, }, // Asia/Yakutsk
    {  470, 1235,  153665,  6, }, // Asia/Yangon
    {  470, 3883,  153671, 10, }, // Asia/Yekaterinburg
    {  470, 3747,  153681,  6, }, // Asia/Yerevan
    {  470, 3663,  153687,  6, }, // Atlantic/Azores
    {  470, 5671,   12497,  6, }, // Atlantic/Bermuda
    {  470, 6628,  153693,  5, }, // Atlantic/Canary
    {  470, 3727,  153698,  7, }, // Atlantic/Cape_Verde
    {  470, 1395,   12520,  4, }, // Atlantic/Faroe
    {  470, 8736,  153705,  5, }, // Atlantic/Madeira
    {  470, 2748,  153710,  8, }, // Atlantic/Reykjavik
    {  470, 7272,  153718, 12, }, // Atlantic/South_Georgia
    {  470, 6821,  153730, 11, }, // Atlantic/St_Helena
    {  470, 6644,  153741,  5, }, // Atlantic/Stanley
    {  470, 1670,  153746,  7, }, // Australia/Adelaide
    {  470, 1635,  153753,  6, }, // Australia/Brisbane
    {  470, 1799,  153759,  8, }, // Australia/Broken_Hill
    {  470, 1583,  153767,  5, }, // Australia/Darwin
    {  470, 3637,  153772,  6, }, // Australia/Eucla
    {  470, 1516,  153778,  6, }, // Australia/Hobart
    {  470, 8753,  153784,  8, }, // Australia/Lindeman
    {  470, 1547,  153792,  8, }, // Australia/Lord_Howe
    {  470, 1727,  153800,  7, }, // Australia/Melbourne
    {  470, 1762,  153807,  3, }, // Australia/Perth
    {  470, 1463,  153810,  4, }, // Australia/Sydney
    {  470, 6441,  153814,  8, }, // Europe/Amsterdam
    {  470, 6216,  153822,  6, }, // Europe/Andorra
    {  470, 3620,  153828,  9, }, // Europe/Astrakhan
    {  470, 2300,  153837,  5, }, // Europe/Athens
    {  470, 6336,  153842,  5, }, // Europe/Belgrade
    {  470, 4757,  153847,  4, }, // Europe/Berlin
    {  470, 6488,  153851,  9, }, // Europe/Bratislava
    {  470, 2167,  153860,  6, }, // Europe/Brussels
    {  470, 3928,  153866,  6, }, // Europe/Bucharest
    {  470, 3806,  153872,  6, }, // Europe/Budapest
    {  470, 8772,  153878,  7, }, // Europe/Busingen
    {  470, 2583,  153885,  6, }, // Europe/Chisinau
    {  470, 6289,  153891,  9, }, // Europe/Copenhagen
    {  470, 2338,  153900,  4, }, // Europe/Dublin
    {  470, 6307,  153904, 10, }, // Europe/Gibraltar
    {  470, 8788,  153914,  5, }, // Europe/Guernsey
    {  470, 6612,  153919,  7, }, // Europe/Helsinki
    {  470, 8804,  153926, 12, }, // Europe/Isle_of_Man
    {  470, 1130,  153938,  7, }, // Europe/Istanbul
    {  470, 8823,  105556,  4, }, // Europe/Jersey
    {  470, 3979,  153945, 10, }, // Europe/Kaliningrad
    {  470, 8929,  153955,  5, }, // Europe/Kirov
    {  470, 2527,  153960,  3, }, // Europe/Kyiv
    {  470, 3213,  153963,  6, }, // Europe/Lisbon
    {  470, 6506,  153969,  7, }, // Europe/Ljubljana
    {  470, 2501,   12808,  4, }, // Europe/London
    {  470, 6365,  153976, 10, }, // Europe/Luxembourg
    {  470, 4892,  153986,  6, }, // Europe/Madrid
    {  470, 6397,  153992,  5, }, // Europe/Malta
    {  470, 6582,  153997,  7, }, // Europe/Mariehamn
    {  470, 3693,  154004,  4, }, // Europe/Minsk
    {  470, 6410,  105639,  6, }, // Europe/Monaco
    {  470, 3537,  105645,  5, }, // Europe/Moscow
    {  470, 6458,  105650,  5, }, // Europe/Oslo
    {  470, 4284,  154008,  5, }, // Europe/Paris
    {  470, 6424,  154013,  9, }, // Europe/Podgorica
    {  470, 6275,  154022,  4, }, // Europe/Prague
    {  470, 8837,  152692,  4, }, // Europe/Riga
    {  470, 6324,   10638,  3, }, // Europe/Rome
    {  470, 4331,  106627,  5, }, // Europe/Samara
    {  470, 6470,  154026, 10, }, // Europe/San_Marino
    {  470, 6245,  154036,  8, }, // Europe/Sarajevo
    {  470, 4451,  154044,  5, }, // Europe/Saratov
    {  470, 8849,  154049,  9, }, // Europe/Simferopol
    {  470, 6383,  154058,  5, }, // Europe/Skopje
    {  470, 6599,  154063,  5, }, // Europe/Sofia
    {  470, 6523,  154068,  7, }, // Europe/Stockholm
    {  470, 8867,   12938,  5, }, // Europe/Tallinn
    {  470, 6202,   12943,  5, }, // Europe/Tirane
    {  470, 8882,  154075,  8, }, // Europe/Ulyanovsk
    {  470, 6352,  154083,  5, }, // Europe/Vaduz
    {  470, 6567,  154088,  7, }, // Europe/Vatican
    {  470, 6231,  154095,  5, }, // Europe/Vienna
    {  470, 8899,  154100,  7, }, // Europe/Vilnius
    {  470, 4727,  154107,  9, }, // Europe/Volgograd
    {  470, 3190,  105795,  5, }, // Europe/Warsaw
    {  470, 6261,  154116,  4, }, // Europe/Zagreb
    {  470, 6540,  154120,  4, }, // Europe/Zurich
    {  470, 5078,  154124, 11, }, // Indian/Antananarivo
    {  470, 6947,  154135,  5, }, // Indian/Chagos
    {  470, 6046,  154140,  6, }, // Indian/Christmas
    {  470, 6063,  154146,  5, }, // Indian/Cocos
    {  470, 5029,  105840,  6, }, // Indian/Comoro
    {  470, 6661,  154151,  7, }, // Indian/Kerguelen
    {  470, 7260,   13055,  4, }, // Indian/Mahe
    {  470, 7050,  154158,  6, }, // Indian/Maldives
    {  470, 4069,  154164,  7, }, // Indian/Mauritius
    {  470, 5098,  154171,  5, }, // Indian/Mayotte
    {  470, 7211,  154176,  8, }, // Indian/Reunion
    {  470, 4422,  154184,  4, }, // Pacific/Apia
    {  470,  932,  154188,  5, }, // Pacific/Auckland
    {  470, 3706,  154193,  8, }, // Pacific/Bougainville
    {  470, 3015,  154201,  5, }, // Pacific/Chatham
    {  470, 3157,  154206,  3, }, // Pacific/Chuuk
    {  470, 2237,  154209,  5, }, // Pacific/Easter
    {  470, 7360,  154214,  4, }, // Pacific/Efate
    {  470, 7327,  154218,  6, }, // Pacific/Fakaofo
    {  470, 3902,  154224,  3, }, // Pacific/Fiji
    {  470, 7343,   13132,  8, }, // Pacific/Funafuti
    {  470, 6678,  154227,  7, }, // Pacific/Galapagos
    {  470, 6696,  154234,  6, }, // Pacific/Gambier
    {  470, 3822,  154240, 10, }, // Pacific/Guadalcanal
    {  470, 6018,  152682,  4, }, // Pacific/Guam
    {  470, 2723,  154250,  8, }, // Pacific/Honolulu
    {  470, 3049,  154258,  7, }, // Pacific/Kanton
    {  470, 3998,  154265,  9, }, // Pacific/Kiritimati
    {  470, 7019,  154274,  6, }, // Pacific/Kosrae
    {  470, 2842,  154280,  7, }, // Pacific/Kwajalein
    {  470, 4877,  154287,  5, }, // Pacific/Majuro
    {  470, 4051,  154292,  8, }, // Pacific/Marquesas
    {  470, 8914,  154300,  5, }, // Pacific/Midway
    {  470, 7084,  154305,  5, }, // Pacific/Nauru
    {  470, 7132,  154310,  4, }, // Pacific/Niue
    {  470, 4168,  154314,  5, }, // Pacific/Norfolk
    {  470, 7098,   13237,  5, }, // Pacific/Noumea
    {  470, 3126,  154319,  9, }, // Pacific/Pago_Pago
    {  470, 7155,  154328,  4, }, // Pacific/Palau
    {  470, 7194,  154332,  8, }, // Pacific/Pitcairn
    {  470, 3096,  154340,  5, }, // Pacific/Pohnpei
    {  470, 4807,  154345, 11, }, // Pacific/Port_Moresby
    {  470, 6076,  154356,  9, }, // Pacific/Rarotonga
    {  470, 6031,  154365,  5, }, // Pacific/Saipan
    {  470, 7312,  154370,  5, }, // Pacific/Tahiti
    {  470, 6712,   13299,  6, }, // Pacific/Tarawa
    {  470, 4583,  154375,  9, }, // Pacific/Tongatapu
    {  470, 7392,  106123,  3, }, // Pacific/Wake
    {  470, 7405,  154384,  4, }, // Pacific/Wallis
    {  471, 5015,  154388,  5, }, // Africa/Harare Pashto/Arabic/Pakistan
    {  471, 5932,  154393, 25, }, // America/Lower_Princes
    {  472, 6788,  154418,  6, }, // Africa/Abidjan Persian/Arabic/Iran
    {  472, 6760,  151787,  4, }, // Africa/Accra
    {  472, 5059,  154424, 10, }, // Africa/Addis_Ababa
    {  472, 7420,  154434,  7, }, // Africa/Algiers
    {  472,   14,  154441,  5, }, // Africa/Asmara
    {  472,   44,  154446,  6, }, // Africa/Bamako
    {  472, 5257,  154452,  5, }, // Africa/Bangui
    {  472, 6746,   10427,  6, }, // Africa/Banjul
    {  472, 7435,  154457,  6, }, // Africa/Bissau
    {  472, 4957,  154463,  8, }, // Africa/Blantyre
    {  472, 5287,  154471,  7, }, // Africa/Brazzaville
    {  472, 4922,   10454,  9, }, // Africa/Bujumbura
    {  472, 2320,  151852,  5, }, // Africa/Cairo
    {  472, 4117,  154478, 10, }, // Africa/Casablanca
    {  472, 7449,  154488,  4, }, // Africa/Ceuta
    {  472, 6773,  103085,  7, }, // Africa/Conakry
    {  472, 6840,  154492,  5, }, // Africa/Dakar
    {  472, 5130,  103097,  9, }, // Africa/Dar_es_Salaam
    {  472, 5043,  154497,  6, }, // Africa/Djibouti
    {  472, 5243,  154503,  5, }, // Africa/Douala
    {  472, 5166,  154508,  6, }, // Africa/El_Aaiun
    {  472, 6853,  154514,  8, }, // Africa/Freetown
    {  472, 4906,  106722,  7, }, // Africa/Gaborone
    {  472, 5015,  154522,  5, }, // Africa/Harare
    {  472, 4479,  154527, 10, }, // Africa/Johannesburg
    {  472, 4499,   10555,  4, }, // Africa/Juba
    {  472, 5151,  154537,  7, }, // Africa/Kampala
    {  472, 4524,  154544,  6, }, // Africa/Khartoum
    {  472, 4973,  154550,  6, }, // Africa/Kigali
    {  472, 5306,  154556,  7, }, // Africa/Kinshasa
    {  472, 4744,  106771,  5, }, // Africa/Lagos
    {  472, 5336,  154563,  7, }, // Africa/Libreville
    {  472, 6869,  154570,  4, }, // Africa/Lome
    {  472, 5211,   10602,  6, }, // Africa/Luanda
    {  472, 4939,  154574,  9, }, // Africa/Lubumbashi
    {  472, 5001,  154583,  6, }, // Africa/Lusaka
    {  472, 5322,   10622,  6, }, // Africa/Malabo
    {  472, 4987,  154589,  6, }, // Africa/Maputo
    {  472, 5197,  154595,  5, }, // Africa/Maseru
    {  472, 5182,  154600,  7, }, // Africa/Mbabane
    {  472, 5113,  154607,  8, }, // Africa/Mogadishu
    {  472, 7034,  154615,  8, }, // Africa/Monrovia
    {  472, 3853,  154623,  7, }, // Africa/Nairobi
    {  472, 5271,  154630,  7, }, // Africa/Ndjamena
    {  472, 5354,  154637,  5, }, // Africa/Niamey
    {  472, 6803,  154642,  7, }, // Africa/Nouakchott
    {  472, 6727,  154649,  9, }, // Africa/Ouagadougou
    {  472, 5225,  154658, 10, }, // Africa/Porto-Novo
    {  472, 4435,  154668,  8, }, // Africa/Sao_Tome
    {  472, 2866,   10713,  6, }, // Africa/Tripoli
    {  472, 6554,   10719,  4, }, // Africa/Tunis
    {  472, 4152,  154676,  7, }, // Africa/Windhoek
    {  472,  132,  154683,  4, }, // America/Adak
    {  472, 3346,  154687,  7, }, // America/Anchorage
    {  472, 5607,  154694,  8, }, // America/Anguilla
    {  472, 5624,  154702,  7, }, // America/Antigua
    {  472, 4554,  154709,  9, }, // America/Araguaina
    {  472,  166,  154718, 11, }, // America/Argentina/Buenos_Aires
    {  472,   91,  154729,  9, }, // America/Argentina/Catamarca
    {  472,  270,  154738,  7, }, // America/Argentina/Cordoba
    {  472,  441,  154745,  5, }, // America/Argentina/Jujuy
    {  472, 7462,  154750,  7, }, // America/Argentina/La_Rioja
    {  472,  565,  154757,  6, }, // America/Argentina/Mendoza
    {  472, 7489,  154763,  9, }, // America/Argentina/Rio_Gallegos
    {  472, 7520,  106994,  5, }, // America/Argentina/Salta
    {  472, 7544,  154772,  7, }, // America/Argentina/San_Juan
    {  472, 5580,  154779,  8, }, // America/Argentina/San_Luis
    {  472, 7571,  154787,  6, }, // America/Argentina/Tucuman
    {  472, 7597,  152159,  7, }, // America/Argentina/Ushuaia
    {  472, 5640,  154793,  5, }, // America/Aruba
    {  472, 4252,  154798,  8, }, // America/Asuncion
    {  472,  237,  154806,  7, }, // America/Atikokan
    {  472, 3679,  154813,  4, }, // America/Bahia
    {  472, 7623,  154817, 13, }, // America/Bahia_Banderas
    {  472, 5654,  107069,  8, }, // America/Barbados
    {  472, 7646,   10902,  3, }, // America/Belem
    {  472, 5407,  152202,  4, }, // America/Belize
    {  472, 7660,  154830, 11, }, // America/Blanc-Sablon
    {  472, 7681,  154841,  9, }, // America/Boa_Vista
    {  472, 4361,  107096,  6, }, // America/Bogota
    {  472, 7699,  154850,  5, }, // America/Boise
    {  472, 7713,  154855,  9, }, // America/Cambridge_Bay
    {  472, 7735,  154864, 11, }, // America/Campo_Grande
    {  472, 3868,  154875,  6, }, // America/Cancun
    {  472, 4694,  152258,  7, }, // America/Caracas
    {  472, 4345,  152050,  4, }, // America/Cayenne
    {  472, 5496,  152269,  4, }, // America/Cayman
    {  472, 2260,  154881,  6, }, // America/Chicago
    {  472, 7756,  154887,  6, }, // America/Chihuahua
    {  472, 7774,  154893, 12, }, // America/Ciudad_Juarez
    {  472, 5422,  154905,  9, }, // America/Costa_Rica
    {  472, 7796,  154914,  6, }, // America/Creston
    {  472, 3791,  154920,  6, }, // America/Cuiaba
    {  472, 5723,  154926,  8, }, // America/Curacao
    {  472, 7812,  154934, 13, }, // America/Danmarkshavn
    {  472, 7833,   11058,  5, }, // America/Dawson
    {  472, 7848,  154947, 10, }, // America/Dawson_Creek
    {  472,  805,  154957,  4, }, // America/Denver
    {  472, 3465,  154961,  7, }, // America/Detroit
    {  472, 5739,  152356,  8, }, // America/Dominica
    {  472,  893,  154968,  8, }, // America/Edmonton
    {  472, 7869,  154976,  7, }, // America/Eirunepe
    {  472, 5441,  154983, 10, }, // America/El_Salvador
    {  472, 7886,  154993, 10, }, // America/Fort_Nelson
    {  472, 7906,  155003,  8, }, // America/Fortaleza
    {  472, 7924,  155011,  7, }, // America/Glace_Bay
    {  472, 6881,  155018,  6, }, // America/Goose_Bay
    {  472, 4612,  155024, 10, }, // America/Grand_Turk
    {  472, 5770,  155034,  6, }, // America/Grenada
    {  472, 5786,  155040,  7, }, // America/Guadeloupe
    {  472, 3760,  155047,  8, }, // America/Guatemala
    {  472, 4841,  155055,  8, }, // America/Guayaquil
    {  472, 6932,  155063,  5, }, // America/Guyana
    {  472, 1939,  155068,  6, }, // America/Halifax
    {  472, 2281,  155074,  6, }, // America/Havana
    {  472, 7942,  155080,  7, }, // America/Hermosillo
    {  472,  348,  155087, 13, }, // America/Indiana/Indianapolis
    {  472,  481,  155100, 14, }, // America/Indiana/Knox
    {  472, 7961,  155114, 16, }, // America/Indiana/Marengo
    {  472, 7985,  155130, 18, }, // America/Indiana/Petersburg
    {  472, 8012,  155148, 17, }, // America/Indiana/Tell_City
    {  472, 8038,  155165, 14, }, // America/Indiana/Vevay
    {  472, 8060,  155179, 15, }, // America/Indiana/Vincennes
    {  472, 8086,  155194, 17, }, // America/Indiana/Winamac
    {  472, 8110,  155211,  7, }, // America/Inuvik
    {  472,  660,  155218,  7, }, // America/Iqaluit
    {  472, 2799,  155225,  8, }, // America/Jamaica
    {  472, 5380,  152608,  4, }, // America/Juneau
    {  472,  521,  155233,  8, }, // America/Kentucky/Louisville
    {  472, 8125,  155241, 16, }, // America/Kentucky/Monticello
    {  472, 5704,  155257,  8, }, // America/Kralendijk
    {  472, 4376,  155265,  5, }, // America/La_Paz
    {  472, 7169,  152645,  4, }, // America/Lima
    {  472, 3239,  155270,  9, }, // America/Los_Angeles
    {  472, 5932,  155279, 23, }, // America/Lower_Princes
    {  472, 8153,  155302,  5, }, // America/Maceio
    {  472, 8168,  155307,  7, }, // America/Managua
    {  472, 1908,   11380,  6, }, // America/Manaus
    {  472, 5897,  155314,  7, }, // America/Marigot
    {  472, 5805,  155321,  8, }, // America/Martinique
    {  472, 8184,   11403,  9, }, // America/Matamoros
    {  472, 2917,  107643,  8, }, // America/Mazatlan
    {  472, 8202,  155329,  7, }, // America/Menominee
    {  472, 8220,  155336,  5, }, // America/Merida
    {  472, 8235,  155341, 10, }, // America/Metlakatla
    {  472, 2949,  155351, 10, }, // America/Mexico_City
    {  472, 4391,  155361,  6, }, // America/Miquelon
    {  472, 8254,  155367,  7, }, // America/Moncton
    {  472, 8270,  155374,  6, }, // America/Monterrey
    {  472, 4098,  155380, 11, }, // America/Montevideo
    {  472, 5824,  155391,  8, }, // America/Montserrat
    {  472, 5481,  155399,  6, }, // America/Nassau
    {  472, 2379,  155405,  7, }, // America/New_York
    {  472, 8288,   11422,  3, }, // America/Nome
    {  472, 1850,  155412,  7, }, // America/Noronha
    {  472, 8301,  155419, 20, }, // America/North_Dakota/Beulah
    {  472, 8329,  155439, 19, }, // America/North_Dakota/Center
    {  472, 8357,  155458, 22, }, // America/North_Dakota/New_Salem
    {  472,  393,  152871,  4, }, // America/Nuuk
    {  472, 8388,  155480,  8, }, // America/Ojinaga
    {  472, 2356,  107831,  6, }, // America/Panama
    {  472, 6173,  155488, 10, }, // America/Paramaribo
    {  472, 2973,  152890,  5, }, // America/Phoenix
    {  472, 3945,  155498,  9, }, // America/Port-au-Prince
    {  472, 5954,  155507, 12, }, // America/Port_of_Spain
    {  472, 8404,  155519,  9, }, // America/Porto_Velho
    {  472, 5843,  155528,  9, }, // America/Puerto_Rico
    {  472, 4030,  155537, 11, }, // America/Punta_Arenas
    {  472, 8424,  155548, 11, }, // America/Rankin_Inlet
    {  472, 8445,  155559,  6, }, // America/Recife
    {  472, 1995,  155565,  6, }, // America/Regina
    {  472, 8460,  155571,  6, }, // America/Resolute
    {  472,  695,  155577,  9, }, // America/Rio_Branco
    {  472, 8477,  155586,  6, }, // America/Santarem
    {  472, 2201,  155592,  8, }, // America/Santiago
    {  472, 6111,  155600, 13, }, // America/Santo_Domingo
    {  472, 1878,  155613, 10, }, // America/Sao_Paulo
    {  472, 6899,  155623, 12, }, // America/Scoresbysund
    {  472, 8494,  155635,  5, }, // America/Sitka
    {  472, 8508,  155640, 11, }, // America/St_Barthelemy
    {  472, 2061,  155651,  7, }, // America/St_Johns
    {  472, 5863,  155658,  8, }, // America/St_Kitts
    {  472, 5880,  155666,  9, }, // America/St_Lucia
    {  472,  855,  155675,  9, }, // America/St_Thomas
    {  472, 5913,  155684, 10, }, // America/St_Vincent
    {  472, 8530,  155694, 11, }, // America/Swift_Current
    {  472, 5461,  155705, 10, }, // America/Tegucigalpa
    {  472, 5756,   11829,  3, }, // America/Thule
    {  472,  313,  155715,  7, }, // America/Tijuana
    {  472,  608,  155722,  6, }, // America/Toronto
    {  472, 5688,   11826,  7, }, // America/Tortola
    {  472, 2093,  155728,  6, }, // America/Vancouver
    {  472, 2144,  155734,  9, }, // America/Whitehorse
    {  472,  734,  155743,  6, }, // America/Winnipeg
    {  472, 8552,  155749,  7, }, // America/Yakutat
    {  472, 6001,  104629,  4, }, // Antarctica/Casey
    {  472, 6094,  155756,  5, }, // Antarctica/Davis
    {  472, 6133,  155761, 12, }, // Antarctica/DumontDUrville
    {  472, 8568,  155773,  6, }, // Antarctica/Macquarie
    {  472, 7066,   11896,  6, }, // Antarctica/Mawson
    {  472, 7113,  155779,  8, }, // Antarctica/McMurdo
    {  472, 8589,  153210,  5, }, // Antarctica/Palmer
    {  472, 7226,  155787,  5, }, // Antarctica/Rothera
    {  472, 7295,  155792,  4, }, // Antarctica/Syowa
    {  472, 8607,   11929,  4, }, // Antarctica/Troll
    {  472, 7374,  104684,  5, }, // Antarctica/Vostok
    {  472, 1429,  155796, 10, }, // Arctic/Longyearbyen
    {  472, 5570,   11948,  3, }, // Asia/Aden
    {  472, 5395,  155806,  6, }, // Asia/Almaty
    {  472, 3968,  155812,  5, }, // Asia/Amman
    {  472, 5511,  155817,  6, }, // Asia/Anadyr
    {  472, 5523,  155823,  5, }, // Asia/Aqtau
    {  472, 5368,  155828,  6, }, // Asia/Aqtobe
    {  472,  964,  155834,  8, }, // Asia/Ashgabat
    {  472, 8624,  155842,  6, }, // Asia/Atyrau
    {  472, 3607,   11991,  5, }, // Asia/Baghdad
    {  472, 5534,  153282,  5, }, // Asia/Bahrain
    {  472, 3653,  152318,  4, }, // Asia/Baku
    {  472, 4466,  155848,  6, }, // Asia/Bangkok
    {  472, 3571,  155854,  8, }, // Asia/Barnaul
    {  472, 4086,  153292,  5, }, // Asia/Beirut
    {  472, 3778,  155862,  5, }, // Asia/Bishkek
    {  472, 5989,  155867,  6, }, // Asia/Brunei
    {  472, 4601,  155873,  4, }, // Asia/Chita
    {  472, 4511,  155877,  5, }, // Asia/Colombo
    {  472, 4540,   12044,  4, }, // Asia/Damascus
    {  472, 1093,  152048,  4, }, // Asia/Dhaka
    {  472, 6192,  155882,  4, }, // Asia/Dili
    {  472, 3596,  155886,  3, }, // Asia/Dubai
    {  472, 6159,  155889,  6, }, // Asia/Dushanbe
    {  472, 8636,  155895,  9, }, // Asia/Famagusta
    {  472, 8651,  153340,  3, }, // Asia/Gaza
    {  472, 4795,  155904,  6, }, // Asia/Hebron
    {  472, 1259,  155910, 13, }, // Asia/Ho_Chi_Minh
    {  472, 2704,  155923,  7, }, // Asia/Hong_Kong
    {  472, 4771,  155930,  4, }, // Asia/Hovd
    {  472, 4184,  155934,  8, }, // Asia/Irkutsk
    {  472, 7006,  155942,  7, }, // Asia/Jakarta
    {  472, 6992,  155949,  8, }, // Asia/Jayapura
    {  472, 1290,  155957,  7, }, // Asia/Jerusalem
    {  472, 3560,  153394,  4, }, // Asia/Kabul
    {  472, 4316,  153398,  8, }, // Asia/Kamchatka
    {  472, 4239,  155964,  5, }, // Asia/Karachi
    {  472, 1185,  155969,  8, }, // Asia/Kathmandu
    {  472, 8661,  155977,  7, }, // Asia/Khandyga
    {  472,  992,  155984,  5, }, // Asia/Kolkata
    {  472, 4197,  155989, 11, }, // Asia/Krasnoyarsk
    {  472, 4859,  156000, 11, }, // Asia/Kuala_Lumpur
    {  472, 5976,  156011,  6, }, // Asia/Kuching
    {  472, 5547,  156017,  4, }, // Asia/Kuwait
    {  472, 1211,  156021,  6, }, // Asia/Macau
    {  472, 4017,  108468,  7, }, // Asia/Magadan
    {  472, 1349,  156027,  7, }, // Asia/Makassar
    {  472, 7182,  156034,  5, }, // Asia/Manila
    {  472, 6920,   12250,  4, }, // Asia/Muscat
    {  472, 2554,  156039,  7, }, // Asia/Nicosia
    {  472, 8675,  156046, 11, }, // Asia/Novokuznetsk
    {  472, 4135,  156057, 11, }, // Asia/Novosibirsk
    {  472, 4229,  105031,  5, }, // Asia/Omsk
    {  472, 7145,  156068,  5, }, // Asia/Oral
    {  472, 6961,  156073,  7, }, // Asia/Phnom_Penh
    {  472, 8693,  156080,  9, }, // Asia/Pontianak
    {  472, 4214,  156089, 10, }, // Asia/Pyongyang
    {  472, 5559,   12318,  3, }, // Asia/Qatar
    {  472, 8708,  156099,  8, }, // Asia/Qostanay
    {  472, 4269,  156107, 11, }, // Asia/Qyzylorda
    {  472, 3584,  105083,  4, }, // Asia/Riyadh
    {  472, 4408,  156118,  7, }, // Asia/Sakhalin
    {  472, 7245,   12350,  6, }, // Asia/Samarkand
    {  472, 3279,  156125,  4, }, // Asia/Seoul
    {  472, 1053,  156129,  7, }, // Asia/Shanghai
    {  472, 3300,  156136,  7, }, // Asia/Singapore
    {  472, 4297,  156143, 12, }, // Asia/Srednekolymsk
    {  472, 3263,  156155,  5, }, // Asia/Taipei
    {  472, 4781,  156160,  6, }, // Asia/Tashkent
    {  472, 3915,  156166,  5, }, // Asia/Tbilisi
    {  472, 2772,  153601,  5, }, // Asia/Tehran
    {  472, 1317,  156171,  5, }, // Asia/Thimphu
    {  472, 2821,  156176,  5, }, // Asia/Tokyo
    {  472, 4572,  156181,  5, }, // Asia/Tomsk
    {  472, 1021,  156186, 11, }, // Asia/Ulaanbaatar
    {  472, 1159,  156197,  6, }, // Asia/Urumqi
    {  472, 8722,  156203,  8, }, // Asia/Ust-Nera
    {  472, 6977,  156211,  7, }, // Asia/Vientiane
    {  472, 4710,  156218, 11, }, // Asia/Vladivostok
    {  472, 4828,  156229,  7, }, // Asia/Yakutsk
    {  472, 1235,  156236,  6, }, // Asia/Yangon
    {  472, 3883,  156242, 10, }, // Asia/Yekaterinburg
    {  472, 3747,  156252,  6, }, // Asia/Yerevan
    {  472, 3663,  156258,  4, }, // Atlantic/Azores
    {  472, 5671,   12497,  6, }, // Atlantic/Bermuda
    {  472, 6628,  156262,  5, }, // Atlantic/Canary
    {  472, 3727,  156267,  7, }, // Atlantic/Cape_Verde
    {  472, 1395,   12520,  4, }, // Atlantic/Faroe
    {  472, 8736,  156274,  6, }, // Atlantic/Madeira
    {  472, 2748,  156280,  8, }, // Atlantic/Reykjavik
    {  472, 7272,  156288, 13, }, // Atlantic/South_Georgia
    {  472, 6821,  156301,  8, }, // Atlantic/St_Helena
    {  472, 6644,  156309,  7, }, // Atlantic/Stanley
    {  472, 1670,  156316,  6, }, // Australia/Adelaide
    {  472, 1635,  156322,  7, }, // Australia/Brisbane
    {  472, 1799,  156329,  9, }, // Australia/Broken_Hill
    {  472, 1583,  156338,  6, }, // Australia/Darwin
    {  472, 3637,  156344,  5, }, // Australia/Eucla
    {  472, 1516,   12603,  6, }, // Australia/Hobart
    {  472, 8753,  156349,  6, }, // Australia/Lindeman
    {  472, 1547,  156355,  6, }, // Australia/Lord_Howe
    {  472, 1727,  156361,  6, }, // Australia/Melbourne
    {  472, 1762,  153807,  3, }, // Australia/Perth
    {  472, 1463,  156367,  5, }, // Australia/Sydney
    {  472, 6441,  156372,  8, }, // Europe/Amsterdam
    {  472, 6216,  156380,  6, }, // Europe/Andorra
    {  472, 3620,  156386,  8, }, // Europe/Astrakhan
    {  472, 2300,  156394,  3, }, // Europe/Athens
    {  472, 6336,  156397,  6, }, // Europe/Belgrade
    {  472, 4757,  156403,  5, }, // Europe/Berlin
    {  472, 6488,  156408, 10, }, // Europe/Bratislava
    {  472, 2167,  156418,  6, }, // Europe/Brussels
    {  472, 3928,  153866,  6, }, // Europe/Bucharest
    {  472, 3806,  156424,  7, }, // Europe/Budapest
    {  472, 8772,  156431,  6, }, // Europe/Busingen
    {  472, 2583,  156437,  7, }, // Europe/Chisinau
    {  472, 6289,  156444,  6, }, // Europe/Copenhagen
    {  472, 2338,  156450,  6, }, // Europe/Dublin
    {  472, 6307,  156456, 10, }, // Europe/Gibraltar
    {  472, 8788,  156466,  5, }, // Europe/Guernsey
    {  472, 6612,  156471,  7, }, // Europe/Helsinki
    {  472, 8804,  156478,  9, }, // Europe/Isle_of_Man
    {  472, 1130,  156487,  8, }, // Europe/Istanbul
    {  472, 8823,  156495,  4, }, // Europe/Jersey
    {  472, 3979,  156499, 11, }, // Europe/Kaliningrad
    {  472, 8929,  153955,  5, }, // Europe/Kirov
    {  472, 2527,  153960,  3, }, // Europe/Kyiv
    {  472, 3213,  153963,  6, }, // Europe/Lisbon
    {  472, 6506,  156510,  9, }, // Europe/Ljubljana
    {  472, 2501,   12808,  4, }, // Europe/London
    {  472, 6365,  156519, 10, }, // Europe/Luxembourg
    {  472, 4892,  156529,  6, }, // Europe/Madrid
    {  472, 6397,  109136,  4, }, // Europe/Malta
    {  472, 6582,  156535, 10, }, // Europe/Mariehamn
    {  472, 3693,  156545,  5, }, // Europe/Minsk
    {  472, 6410,  105639,  6, }, // Europe/Monaco
    {  472, 3537,  154144,  4, }, // Europe/Moscow
    {  472, 6458,  107578,  4, }, // Europe/Oslo
    {  472, 4284,  154008,  5, }, // Europe/Paris
    {  472, 6424,  156550, 10, }, // Europe/Podgorica
    {  472, 6275,  105669,  4, }, // Europe/Prague
    {  472, 8837,  155316,  4, }, // Europe/Riga
    {  472, 6324,   11209,  2, }, // Europe/Rome
    {  472, 4331,  109179,  6, }, // Europe/Samara
    {  472, 6470,  156560, 10, }, // Europe/San_Marino
    {  472, 6245,  156570,  7, }, // Europe/Sarajevo
    {  472, 4451,   12904,  7, }, // Europe/Saratov
    {  472, 8849,  156577,  8, }, // Europe/Simferopol
    {  472, 6383,  156585,  7, }, // Europe/Skopje
    {  472, 6599,  156592,  5, }, // Europe/Sofia
    {  472, 6523,  156597,  7, }, // Europe/Stockholm
    {  472, 8867,  156604,  5, }, // Europe/Tallinn
    {  472, 6202,  156609,  6, }, // Europe/Tirane
    {  472, 8882,  156615, 10, }, // Europe/Ulyanovsk
    {  472, 6352,  156625,  6, }, // Europe/Vaduz
    {  472, 6567,  156631,  7, }, // Europe/Vatican
    {  472, 6231,  103363,  3, }, // Europe/Vienna
    {  472, 8899,  156638,  7, }, // Europe/Vilnius
    {  472, 4727,  156645,  8, }, // Europe/Volgograd
    {  472, 3190,  156653,  4, }, // Europe/Warsaw
    {  472, 6261,  156657,  5, }, // Europe/Zagreb
    {  472, 6540,  156662,  5, }, // Europe/Zurich
    {  472, 5078,  156667, 12, }, // Indian/Antananarivo
    {  472, 6947,  156679,  5, }, // Indian/Chagos
    {  472, 6046,  156684,  6, }, // Indian/Christmas
    {  472, 6063,  105835,  5, }, // Indian/Cocos
    {  472, 5029,  105840,  6, }, // Indian/Comoro
    {  472, 6661,  156690,  6, }, // Indian/Kerguelen
    {  472, 7260,  156696,  4, }, // Indian/Mahe
    {  472, 7050,  156700,  6, }, // Indian/Maldives
    {  472, 4069,  156706,  5, }, // Indian/Mauritius
    {  472, 5098,  156711,  5, }, // Indian/Mayotte
    {  472, 7211,  156716,  7, }, // Indian/Reunion
    {  472, 4422,  156723,  4, }, // Pacific/Apia
    {  472,  932,  156727,  6, }, // Pacific/Auckland
    {  472, 3706,  156733,  7, }, // Pacific/Bougainville
    {  472, 3015,  156740,  6, }, // Pacific/Chatham
    {  472, 3157,  154206,  3, }, // Pacific/Chuuk
    {  472, 2237,  156746,  5, }, // Pacific/Easter
    {  472, 7360,  156751,  5, }, // Pacific/Efate
    {  472, 7327,  156756,  8, }, // Pacific/Fakaofo
    {  472, 3902,  156764,  4, }, // Pacific/Fiji
    {  472, 7343,  156768,  8, }, // Pacific/Funafuti
    {  472, 6678,  109458,  9, }, // Pacific/Galapagos
    {  472, 6696,  156776,  6, }, // Pacific/Gambier
    {  472, 3822,  156782, 12, }, // Pacific/Guadalcanal
    {  472, 6018,  104083,  4, }, // Pacific/Guam
    {  472, 2723,   13165,  8, }, // Pacific/Honolulu
    {  472, 3049,  156794,  7, }, // Pacific/Kanton
    {  472, 3998,  156801,  9, }, // Pacific/Kiritimati
    {  472, 7019,  156810,  6, }, // Pacific/Kosrae
    {  472, 2842,  156816,  8, }, // Pacific/Kwajalein
    {  472, 4877,   13205,  6, }, // Pacific/Majuro
    {  472, 4051,  156824,  7, }, // Pacific/Marquesas
    {  472, 8914,  156831,  5, }, // Pacific/Midway
    {  472, 7084,  156836,  6, }, // Pacific/Nauru
    {  472, 7132,  156842,  5, }, // Pacific/Niue
    {  472, 4168,  156847,  7, }, // Pacific/Norfolk
    {  472, 7098,  156854,  5, }, // Pacific/Noumea
    {  472, 3126,  156859,  8, }, // Pacific/Pago_Pago
    {  472, 7155,  156867,  6, }, // Pacific/Palau
    {  472, 7194,  156873,  7, }, // Pacific/Pitcairn
    {  472, 3096,  156880,  5, }, // Pacific/Pohnpei
    {  472, 4807,  156885, 11, }, // Pacific/Port_Moresby
    {  472, 6076,  109595,  9, }, // Pacific/Rarotonga
    {  472, 6031,  156896,  6, }, // Pacific/Saipan
    {  472, 7312,  156902,  6, }, // Pacific/Tahiti
    {  472, 6712,   13299,  6, }, // Pacific/Tarawa
    {  472, 4583,  156908,  9, }, // Pacific/Tongatapu
    {  472, 7392,  106123,  3, }, // Pacific/Wake
    {  472, 7405,  156917,  5, }, // Pacific/Wallis
    {  474, 6788,  156922,  7, }, // Africa/Abidjan Polish/Latin/Poland
    {  474, 6760,    4531,  4, }, // Africa/Accra
    {  474, 5059,    3784, 11, }, // Africa/Addis_Ababa
    {  474, 7420,   58423,  6, }, // Africa/Algiers
    {  474,   14,       0,  6, }, // Africa/Asmara
    {  474, 5257,   26348,  5, }, // Africa/Bangui
    {  474, 6746,  156929,  7, }, // Africa/Banjul
    {  474, 4922,  156936,  9, }, // Africa/Bujumbura
    {  474, 2320,    4572,  4, }, // Africa/Cairo
    {  474, 6773,   26353,  7, }, // Africa/Conakry
    {  474, 5043,  156945,  7, }, // Africa/Djibouti
    {  474, 5243,   21254,  5, }, // Africa/Douala
    {  474, 5166,  156952,  7, }, // Africa/El_Aaiun
    {  474, 4499,  156959,  5, }, // Africa/Juba
    {  474, 4524,  124750,  7, }, // Africa/Khartoum
    {  474, 5306,  156964,  8, }, // Africa/Kinshasa
    {  474, 6869,   20300,  4, }, // Africa/Lome
    {  474, 5113,  156972,  9, }, // Africa/Mogadishu
    {  474, 5271,  156981,  8, }, // Africa/Ndjamena
    {  474, 6803,  156989,  9, }, // Africa/Nouakchott
    {  474, 6727,    4640,  8, }, // Africa/Ouagadougou
    {  474, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  474, 4435,       6,  8, }, // Africa/Sao_Tome
    {  474, 2866,  156998,  8, }, // Africa/Tripoli
    {  474, 4152,   56676,  7, }, // Africa/Windhoek
    {  474,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  474, 4252,      14,  8, }, // America/Asuncion
    {  474,  237,      22,  8, }, // America/Atikokan
    {  474, 3679,    6688,  8, }, // America/Bahia
    {  474, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  474, 7646,   35764,  5, }, // America/Belem
    {  474, 3868,      47,  6, }, // America/Cancun
    {  474, 4345,  122347,  7, }, // America/Cayenne
    {  474, 5496,  157006,  7, }, // America/Cayman
    {  474, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  474, 5422,  157013,  9, }, // America/Costa_Rica
    {  474, 3791,   26442,  6, }, // America/Cuiaba
    {  474, 5723,      66,  7, }, // America/Curacao
    {  474, 5739,   21548,  8, }, // America/Dominica
    {  474, 5441,  157022,  8, }, // America/El_Salvador
    {  474, 5786,  157030,  9, }, // America/Guadeloupe
    {  474, 3760,  134343,  9, }, // America/Guatemala
    {  474, 6932,  157039,  6, }, // America/Guyana
    {  474, 2281,  157045,  6, }, // America/Havana
    {  474,  481,      73, 13, }, // America/Indiana/Knox
    {  474, 7961,      86, 16, }, // America/Indiana/Marengo
    {  474, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  474, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  474, 8038,     139, 14, }, // America/Indiana/Vevay
    {  474, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  474, 8086,     171, 16, }, // America/Indiana/Winamac
    {  474, 2799,   33215,  7, }, // America/Jamaica
    {  474, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  474, 5932,     207, 22, }, // America/Lower_Princes
    {  474, 8153,   26456,  6, }, // America/Maceio
    {  474, 5805,  157051,  9, }, // America/Martinique
    {  474, 8220,    7008,  6, }, // America/Merida
    {  474, 2949,  157060, 15, }, // America/Mexico_City
    {  474, 2379,  157075,  9, }, // America/New_York
    {  474, 8301,  157084, 23, }, // America/North_Dakota/Beulah
    {  474, 8329,  157107, 23, }, // America/North_Dakota/Center
    {  474, 8357,  157130, 26, }, // America/North_Dakota/New_Salem
    {  474,  393,     298,  4, }, // America/Nuuk
    {  474, 5954,   26559, 13, }, // America/Port_of_Spain
    {  474, 5843,  157156,  9, }, // America/Puerto_Rico
    {  474, 6899,     302, 16, }, // America/Scoresbysund
    {  474, 8508,   26589, 16, }, // America/St_Barthelemy
    {  474, 2061,     332, 10, }, // America/St_Johns
    {  474, 5863,   26617, 11, }, // America/St_Kitts
    {  474, 5880,   35832, 11, }, // America/St_Lucia
    {  474,  855,   44161, 12, }, // America/St_Thomas
    {  474, 5913,   26651, 13, }, // America/St_Vincent
    {  474, 5756,   26664,  7, }, // America/Thule
    {  474, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  474, 7374,    4029,  6, }, // Antarctica/Vostok
    {  474, 5395,  157165,  6, }, // Asia/Almaty
    {  474, 5523,    5491,  5, }, // Asia/Aqtau
    {  474, 5368,  157171, 10, }, // Asia/Aqtobe
    {  474,  964,  157181,  9, }, // Asia/Ashgabat
    {  474, 3607,    4044,  6, }, // Asia/Baghdad
    {  474, 5534,   57811,  7, }, // Asia/Bahrain
    {  474, 3571,  157190,  7, }, // Asia/Barnaul
    {  474, 4086,    7507,  6, }, // Asia/Beirut
    {  474, 3778,  157197,  7, }, // Asia/Bishkek
    {  474, 4601,  157204,  5, }, // Asia/Chita
    {  474, 4511,    5519,  7, }, // Asia/Colombo
    {  474, 4540,  157209,  8, }, // Asia/Damascus
    {  474, 3596,   57836,  5, }, // Asia/Dubai
    {  474, 6159,  157217,  8, }, // Asia/Dushanbe
    {  474, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  474, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  474, 4771,  157225,  5, }, // Asia/Hovd
    {  474, 4184,  147026,  6, }, // Asia/Irkutsk
    {  474, 7006,  157230,  8, }, // Asia/Jakarta
    {  474, 1290,  157238, 10, }, // Asia/Jerusalem
    {  474, 4316,  157248,  9, }, // Asia/Kamchatka
    {  474, 4239,  157257,  7, }, // Asia/Karachi
    {  474, 1185,    5593,  8, }, // Asia/Kathmandu
    {  474, 8661,   57888,  8, }, // Asia/Khandyga
    {  474,  992,   26732,  7, }, // Asia/Kolkata
    {  474, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  474, 5547,  157264,  6, }, // Asia/Kuwait
    {  474, 1211,   33405,  5, }, // Asia/Macau
    {  474, 6920,   26750,  6, }, // Asia/Muscat
    {  474, 2554,  157270,  7, }, // Asia/Nicosia
    {  474, 8675,  157277, 12, }, // Asia/Novokuznetsk
    {  474, 4135,  157289, 11, }, // Asia/Novosibirsk
    {  474, 7145,   57921,  6, }, // Asia/Oral
    {  474, 4214,   33430,  9, }, // Asia/Pyongyang
    {  474, 5559,    4214,  5, }, // Asia/Qatar
    {  474, 8708,  157300,  8, }, // Asia/Qostanay
    {  474, 4269,  157308,  9, }, // Asia/Qyzylorda
    {  474, 3584,   33439,  5, }, // Asia/Riyadh
    {  474, 4408,   57952,  8, }, // Asia/Sakhalin
    {  474, 7245,   26774, 10, }, // Asia/Samarkand
    {  474, 3279,    7680,  4, }, // Asia/Seoul
    {  474, 1053,  157317,  8, }, // Asia/Shanghai
    {  474, 3300,   20782,  8, }, // Asia/Singapore
    {  474, 4297,  157325, 15, }, // Asia/Srednekolymsk
    {  474, 3263,    7712,  6, }, // Asia/Taipei
    {  474, 4781,  157340,  9, }, // Asia/Tashkent
    {  474, 2772,    7724,  7, }, // Asia/Tehran
    {  474, 2821,    4238,  5, }, // Asia/Tokyo
    {  474, 1021,  157349, 10, }, // Asia/Ulaanbaatar
    {  474, 1159,  157359,  7, }, // Asia/Urumqi
    {  474, 8722,  157366,  9, }, // Asia/Ust-Nera
    {  474, 6977,  157375,  8, }, // Asia/Vientiane
    {  474, 4710,  157383, 11, }, // Asia/Vladivostok
    {  474, 4828,  147053,  6, }, // Asia/Yakutsk
    {  474, 1235,    7762,  6, }, // Asia/Yangon
    {  474, 3883,  157394, 13, }, // Asia/Yekaterinburg
    {  474, 3747,  157407,  6, }, // Asia/Yerevan
    {  474, 3663,  157413,  5, }, // Atlantic/Azores
    {  474, 5671,   58043,  7, }, // Atlantic/Bermuda
    {  474, 6628,  157418, 17, }, // Atlantic/Canary
    {  474, 3727,  157435, 29, }, // Atlantic/Cape_Verde
    {  474, 1395,  157464, 11, }, // Atlantic/Faroe
    {  474, 8736,  157475,  6, }, // Atlantic/Madeira
    {  474, 7272,  157481, 18, }, // Atlantic/South_Georgia
    {  474, 6821,  157499, 13, }, // Atlantic/St_Helena
    {  474, 6216,   33506,  6, }, // Europe/Andorra
    {  474, 3620,  157512,  9, }, // Europe/Astrakhan
    {  474, 2300,  157521,  5, }, // Europe/Athens
    {  474, 6336,   26883,  7, }, // Europe/Belgrade
    {  474, 6488,  157526, 10, }, // Europe/Bratislava
    {  474, 2167,  157536,  8, }, // Europe/Brussels
    {  474, 3928,  157544,  9, }, // Europe/Bucharest
    {  474, 3806,  157553,  9, }, // Europe/Budapest
    {  474, 8772,   62795, 21, }, // Europe/Busingen
    {  474, 2583,  157562,  9, }, // Europe/Chisinau
    {  474, 6289,  127077,  9, }, // Europe/Copenhagen
    {  474, 8804,  157571,  9, }, // Europe/Isle_of_Man
    {  474, 1130,  157580,  7, }, // Europe/Istanbul
    {  474, 8929,   78158,  5, }, // Europe/Kirov
    {  474, 2527,  157587,  5, }, // Europe/Kyiv
    {  474, 3213,  157592,  7, }, // Europe/Lisbon
    {  474, 6506,  157599,  7, }, // Europe/Ljubljana
    {  474, 2501,  157606,  6, }, // Europe/London
    {  474, 6365,    8054, 10, }, // Europe/Luxembourg
    {  474, 4892,  157612,  6, }, // Europe/Madrid
    {  474, 6582,   63713, 13, }, // Europe/Mariehamn
    {  474, 3693,  157618,  5, }, // Europe/Minsk
    {  474, 6410,    5926,  6, }, // Europe/Monaco
    {  474, 3537,   93104,  6, }, // Europe/Moscow
    {  474, 4284,  157623,  5, }, // Europe/Paris
    {  474, 6275,   21067,  5, }, // Europe/Prague
    {  474, 8837,  127247,  4, }, // Europe/Riga
    {  474, 6324,  157628,  4, }, // Europe/Rome
    {  474, 6245,  157632,  8, }, // Europe/Sarajevo
    {  474, 4451,  157640,  7, }, // Europe/Saratov
    {  474, 8849,  157647, 10, }, // Europe/Simferopol
    {  474, 6523,  157657,  9, }, // Europe/Stockholm
    {  474, 8867,   22793,  6, }, // Europe/Tallinn
    {  474, 6202,   21095,  6, }, // Europe/Tirane
    {  474, 8882,   78174,  9, }, // Europe/Ulyanovsk
    {  474, 6567,  157666,  7, }, // Europe/Vatican
    {  474, 6231,  157673,  6, }, // Europe/Vienna
    {  474, 8899,  157679,  5, }, // Europe/Vilnius
    {  474, 4727,  157684,  9, }, // Europe/Volgograd
    {  474, 3190,   58543,  8, }, // Europe/Warsaw
    {  474, 6261,  157693,  7, }, // Europe/Zagreb
    {  474, 6540,  157700,  6, }, // Europe/Zurich
    {  474, 5078,  157706, 12, }, // Indian/Antananarivo
    {  474, 6947,  157718,  6, }, // Indian/Chagos
    {  474, 6046,  157724, 23, }, // Indian/Christmas
    {  474, 6063,  157747, 14, }, // Indian/Cocos
    {  474, 5029,   58276,  6, }, // Indian/Comoro
    {  474, 6661,  157761, 16, }, // Indian/Kerguelen
    {  474, 7260,   26983,  4, }, // Indian/Mahe
    {  474, 7050,  127899,  8, }, // Indian/Maldives
    {  474, 5098,  157777,  7, }, // Indian/Mayotte
    {  474, 7211,     467,  7, }, // Indian/Reunion
    {  474, 3706,  157784, 20, }, // Pacific/Bougainville
    {  474, 3157,     474,  5, }, // Pacific/Chuuk
    {  474, 2237,  157804, 17, }, // Pacific/Easter
    {  474, 3902,  157821,  5, }, // Pacific/Fiji
    {  474, 6696,  157826, 14, }, // Pacific/Gambier
    {  474, 2723,    4493,  8, }, // Pacific/Honolulu
    {  474, 3049,    4501,  9, }, // Pacific/Kanton
    {  474, 4051,  157840,  7, }, // Pacific/Marquesas
    {  474, 7098,  124531,  5, }, // Pacific/Noumea
    {  474, 3096,     479,  7, }, // Pacific/Pohnpei
    {  475, 6760,   74000,  4, }, // Africa/Accra Portuguese/Latin/Brazil
    {  475, 5059,   35689, 10, }, // Africa/Addis_Ababa
    {  475, 7420,  147563,  5, }, // Africa/Algiers
    {  475,   14,       0,  6, }, // Africa/Asmara
    {  475, 6773,  157847,  7, }, // Africa/Conakry
    {  475, 5043,   26360,  7, }, // Africa/Djibouti
    {  475, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  475, 4479,  157854, 11, }, // Africa/Johannesburg
    {  475, 4524,  157865,  6, }, // Africa/Khartoum
    {  475, 6869,   20300,  4, }, // Africa/Lome
    {  475, 5113,  157871, 10, }, // Africa/Mogadishu
    {  475, 7034,  140529,  8, }, // Africa/Monrovia
    {  475, 3853,  157881,  7, }, // Africa/Nairobi
    {  475, 5271,   43996,  9, }, // Africa/Ndjamena
    {  475, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  475, 4435,       6,  8, }, // Africa/Sao_Tome
    {  475, 2866,   20349,  7, }, // Africa/Tripoli
    {  475, 6554,   92332,  5, }, // Africa/Tunis
    {  475, 5607,    4657,  7, }, // America/Anguilla
    {  475, 5624,  157888,  7, }, // America/Antigua
    {  475, 4554,   26427,  9, }, // America/Araguaina
    {  475,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  475, 7571,  157895,  6, }, // America/Argentina/Tucuman
    {  475, 4252,  157901,  8, }, // America/Asuncion
    {  475,  237,      22,  8, }, // America/Atikokan
    {  475, 7623,   64558, 17, }, // America/Bahia_Banderas
    {  475, 7646,   35764,  5, }, // America/Belem
    {  475, 4361,   20389,  6, }, // America/Bogota
    {  475, 3868,      47,  6, }, // America/Cancun
    {  475, 4345,   44035,  6, }, // America/Cayenne
    {  475, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  475, 3791,   26442,  6, }, // America/Cuiaba
    {  475, 5723,      66,  7, }, // America/Curacao
    {  475, 7869,   26448,  8, }, // America/Eirunepe
    {  475, 5770,   20401,  7, }, // America/Grenada
    {  475, 5786,   20408,  9, }, // America/Guadeloupe
    {  475, 4841,  157909,  9, }, // America/Guayaquil
    {  475, 6932,  157918,  6, }, // America/Guyana
    {  475,  348,   74114, 12, }, // America/Indiana/Indianapolis
    {  475,  481,      73, 13, }, // America/Indiana/Knox
    {  475, 7961,      86, 16, }, // America/Indiana/Marengo
    {  475, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  475, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  475, 8038,     139, 14, }, // America/Indiana/Vevay
    {  475, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  475, 8086,     171, 16, }, // America/Indiana/Winamac
    {  475, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  475, 5932,     207, 22, }, // America/Lower_Princes
    {  475, 8153,   26456,  6, }, // America/Maceio
    {  475, 8168,   97907,  7, }, // America/Managua
    {  475, 5805,   20436,  9, }, // America/Martinique
    {  475, 8220,     229,  6, }, // America/Merida
    {  475, 2949,  157924, 16, }, // America/Mexico_City
    {  475, 4098,  157940, 10, }, // America/Montevideo
    {  475, 2379,   44062,  9, }, // America/New_York
    {  475, 1850,   61820, 19, }, // America/Noronha
    {  475, 8301,  157950, 23, }, // America/North_Dakota/Beulah
    {  475, 8329,  157973, 23, }, // America/North_Dakota/Center
    {  475, 8357,  157996, 26, }, // America/North_Dakota/New_Salem
    {  475,  393,     298,  4, }, // America/Nuuk
    {  475, 2356,   20547,  6, }, // America/Panama
    {  475, 3945,   74229, 14, }, // America/Port-au-Prince
    {  475, 5843,   64866, 10, }, // America/Puerto_Rico
    {  475, 8477,   26572,  8, }, // America/Santarem
    {  475, 1878,   26580,  9, }, // America/Sao_Paulo
    {  475, 6899,     302, 16, }, // America/Scoresbysund
    {  475, 8508,  158022, 14, }, // America/St_Barthelemy
    {  475, 2061,   26605, 12, }, // America/St_Johns
    {  475, 5863,  158036, 13, }, // America/St_Kitts
    {  475, 5880,  158049, 11, }, // America/St_Lucia
    {  475,  855,   44161, 12, }, // America/St_Thomas
    {  475, 5913,  158060, 11, }, // America/St_Vincent
    {  475, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  475, 5570,   91403,  4, }, // Asia/Aden
    {  475, 3968,  158071,  3, }, // Asia/Amman
    {  475, 5523,    5491,  5, }, // Asia/Aqtau
    {  475, 5368,    7488,  6, }, // Asia/Aqtobe
    {  475,  964,  158074,  8, }, // Asia/Ashgabat
    {  475, 3607,   57805,  5, }, // Asia/Baghdad
    {  475, 5534,    4050,  7, }, // Asia/Bahrain
    {  475, 4086,  158082,  7, }, // Asia/Beirut
    {  475, 4540,   74309,  7, }, // Asia/Damascus
    {  475, 1093,   65588,  5, }, // Asia/Dhaka
    {  475, 6159,  158089,  8, }, // Asia/Dushanbe
    {  475, 1259,  158097, 21, }, // Asia/Ho_Chi_Minh
    {  475, 7006,  158118,  7, }, // Asia/Jakarta
    {  475, 1290,  158125,  9, }, // Asia/Jerusalem
    {  475, 3560,   74323,  5, }, // Asia/Kabul
    {  475, 1185,    5593,  8, }, // Asia/Kathmandu
    {  475,  992,  158134,  7, }, // Asia/Kolkata
    {  475, 1211,    4203,  5, }, // Asia/Macau
    {  475, 6920,   20739,  7, }, // Asia/Muscat
    {  475, 2554,  158141,  7, }, // Asia/Nicosia
    {  475, 5559,   73021,  5, }, // Asia/Qatar
    {  475, 3584,  158148,  5, }, // Asia/Riyadh
    {  475, 4408,  158153,  8, }, // Asia/Sakhalin
    {  475, 7245,   20768, 10, }, // Asia/Samarkand
    {  475, 3279,    7680,  4, }, // Asia/Seoul
    {  475, 1053,  158161,  6, }, // Asia/Shanghai
    {  475, 3300,   93044,  9, }, // Asia/Singapore
    {  475, 2772,  158167,  5, }, // Asia/Tehran
    {  475, 2821,  158172,  6, }, // Asia/Tokyo
    {  475, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  475, 1235,  158178,  6, }, // Asia/Yangon
    {  475, 3883,  158184, 13, }, // Asia/Yekaterinburg
    {  475, 3663,   44329,  6, }, // Atlantic/Azores
    {  475, 5671,   74354,  8, }, // Atlantic/Bermuda
    {  475, 6628,  158197,  8, }, // Atlantic/Canary
    {  475, 3727,   26836, 10, }, // Atlantic/Cape_Verde
    {  475, 1395,  158205, 11, }, // Atlantic/Faroe
    {  475, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  475, 7272,  158216, 14, }, // Atlantic/South_Georgia
    {  475, 6821,   26865, 12, }, // Atlantic/St_Helena
    {  475, 6441,  158230,  8, }, // Europe/Amsterdam
    {  475, 3620,  158238,  7, }, // Europe/Astrakhan
    {  475, 2300,   26877,  6, }, // Europe/Athens
    {  475, 6336,   58774,  8, }, // Europe/Belgrade
    {  475, 4757,  158245,  6, }, // Europe/Berlin
    {  475, 2167,   74417,  8, }, // Europe/Brussels
    {  475, 3928,  158251,  9, }, // Europe/Bucharest
    {  475, 3806,  158260,  9, }, // Europe/Budapest
    {  475, 8772,   26905,  8, }, // Europe/Busingen
    {  475, 6289,   20963, 10, }, // Europe/Copenhagen
    {  475, 6612,  158269,  9, }, // Europe/Helsinki
    {  475, 8804,  158278, 11, }, // Europe/Isle_of_Man
    {  475, 1130,   57315,  8, }, // Europe/Istanbul
    {  475, 3979,   62893, 12, }, // Europe/Kaliningrad
    {  475, 2527,    8030,  4, }, // Europe/Kyiv
    {  475, 3213,   21019,  6, }, // Europe/Lisbon
    {  475, 6506,   21025,  9, }, // Europe/Ljubljana
    {  475, 2501,   21034,  7, }, // Europe/London
    {  475, 6365,   74445, 10, }, // Europe/Luxembourg
    {  475, 4892,   26943,  5, }, // Europe/Madrid
    {  475, 6410,  158289,  6, }, // Europe/Monaco
    {  475, 3537,   44444,  6, }, // Europe/Moscow
    {  475, 6275,   21067,  5, }, // Europe/Prague
    {  475, 6324,    5946,  4, }, // Europe/Rome
    {  475, 6599,  158295,  5, }, // Europe/Sofia
    {  475, 6523,   74486,  9, }, // Europe/Stockholm
    {  475, 6202,   21095,  6, }, // Europe/Tirane
    {  475, 8882,   26960,  9, }, // Europe/Ulyanovsk
    {  475, 6567,   74495,  8, }, // Europe/Vatican
    {  475, 6231,   21112,  5, }, // Europe/Vienna
    {  475, 4727,   63074, 10, }, // Europe/Volgograd
    {  475, 3190,  158300,  8, }, // Europe/Warsaw
    {  475, 6540,  158308,  7, }, // Europe/Zurich
    {  475, 5029,   65391,  7, }, // Indian/Comoro
    {  475, 7260,   26983,  4, }, // Indian/Mahe
    {  475, 7050,   74517,  8, }, // Indian/Maldives
    {  475, 4069,  158315,  8, }, // Indian/Mauritius
    {  475, 7211,  158323,  7, }, // Indian/Reunion
    {  475, 3015,  158330,  7, }, // Pacific/Chatham
    {  475, 3157,     474,  5, }, // Pacific/Chuuk
    {  475, 2237,  158337, 14, }, // Pacific/Easter
    {  475, 7360,   27016,  5, }, // Pacific/Efate
    {  475, 6678,   21159,  9, }, // Pacific/Galapagos
    {  475, 2723,    4493,  8, }, // Pacific/Honolulu
    {  475, 3049,    4501,  9, }, // Pacific/Kanton
    {  475, 7098,    4510,  6, }, // Pacific/Noumea
    {  475, 3096,     479,  7, }, // Pacific/Pohnpei
    {  475, 7312,  124568,  5, }, // Pacific/Tahiti
    {  475, 6712,  158351,  6, }, // Pacific/Tarawa
    {  476, 5059,  158357, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Angola
    {  476,   44,   74009,  6, }, // Africa/Bamako
    {  476, 6840,  158367,  5, }, // Africa/Dakar
    {  476, 5043,  158372,  6, }, // Africa/Djibouti
    {  476, 5151,  158378,  7, }, // Africa/Kampala
    {  476, 5001,   74047,  6, }, // Africa/Lusaka
    {  476, 3853,  124827,  7, }, // Africa/Nairobi
    {  476, 5271,   65443,  8, }, // Africa/Ndjamena
    {  476, 5354,   26396,  6, }, // Africa/Niamey
    {  476, 5225,  158385, 10, }, // Africa/Porto-Novo
    {  476, 2866,   35756,  7, }, // Africa/Tripoli
    {  476, 6554,   74069,  5, }, // Africa/Tunis
    {  476, 4554,  158395,  9, }, // America/Araguaina
    {  476, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  476, 3679,   74074,  4, }, // America/Bahia
    {  476, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  476, 3868,   33182,  6, }, // America/Cancun
    {  476, 5496,  158404,  6, }, // America/Cayman
    {  476, 5723,  158410,  7, }, // America/Curacao
    {  476, 5739,  158417,  8, }, // America/Dominica
    {  476, 5441,    6688,  8, }, // America/El_Salvador
    {  476, 4098,  158425, 10, }, // America/Montevideo
    {  476, 5824,  158435, 10, }, // America/Montserrat
    {  476, 2379,  158445, 11, }, // America/New_York
    {  476, 3945,  158456, 14, }, // America/Port-au-Prince
    {  476, 5954,  158470, 16, }, // America/Port_of_Spain
    {  476, 2061,     332, 10, }, // America/St_Johns
    {  476,  855,     360, 10, }, // America/St_Thomas
    {  476, 5570,  158486,  4, }, // Asia/Aden
    {  476, 5523,   91196,  5, }, // Asia/Aqtau
    {  476, 5368,  158490,  6, }, // Asia/Aqtobe
    {  476, 3607,  158496,  7, }, // Asia/Baghdad
    {  476, 5534,  158503,  5, }, // Asia/Bahrain
    {  476, 4466,  158508, 11, }, // Asia/Bangkok
    {  476, 1093,  158367,  4, }, // Asia/Dhaka
    {  476, 4239,  158519,  7, }, // Asia/Karachi
    {  476, 1185,  158526,  8, }, // Asia/Kathmandu
    {  476, 5547,   35929,  6, }, // Asia/Kuwait
    {  476, 1349,   65105,  8, }, // Asia/Makassar
    {  476, 8708,     434,  8, }, // Asia/Qostanay
    {  476, 3263,  158534,  5, }, // Asia/Taipei
    {  476, 2772,  158539,  6, }, // Asia/Tehran
    {  476, 1317,    5640,  6, }, // Asia/Thimphu
    {  476, 1235,     442,  6, }, // Asia/Yangon
    {  476, 3747,   26815,  6, }, // Asia/Yerevan
    {  476, 1395,  158211,  5, }, // Atlantic/Faroe
    {  476, 2748,  158545, 12, }, // Atlantic/Reykjavik
    {  476, 1547,  158557, 17, }, // Australia/Lord_Howe
    {  476, 6441,  158574, 10, }, // Europe/Amsterdam
    {  476, 8772,  158584,  8, }, // Europe/Busingen
    {  476, 6289,  158592,  9, }, // Europe/Copenhagen
    {  476, 6612,  158601, 10, }, // Europe/Helsinki
    {  476, 3979,  158611, 12, }, // Europe/Kaliningrad
    {  476, 2501,   21034,  7, }, // Europe/London
    {  476, 4892,   62944,  6, }, // Europe/Madrid
    {  476, 6410,   74455,  6, }, // Europe/Monaco
    {  476, 3537,  158623,  7, }, // Europe/Moscow
    {  476, 6470,  158630, 11, }, // Europe/San_Marino
    {  476, 8867,  158641,  5, }, // Europe/Tallinn
    {  476, 6046,  158646, 13, }, // Indian/Christmas
    {  476, 6063,  158659, 11, }, // Indian/Cocos
    {  476, 7260,  158670,  4, }, // Indian/Mahe
    {  476, 4069,  158674,  8, }, // Indian/Mauritius
    {  476, 3015,   58319,  7, }, // Pacific/Chatham
    {  476, 2237,  158682, 14, }, // Pacific/Easter
    {  476, 7360,  158696,  5, }, // Pacific/Efate
    {  476, 7194,  158701, 14, }, // Pacific/Pitcairn
    {  476, 6712,  158715,  6, }, // Pacific/Tarawa
    {  477, 5059,  158357, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Cape Verde
    {  477,   44,   74009,  6, }, // Africa/Bamako
    {  477, 6840,  158367,  5, }, // Africa/Dakar
    {  477, 5043,  158372,  6, }, // Africa/Djibouti
    {  477, 5151,  158378,  7, }, // Africa/Kampala
    {  477, 5001,   74047,  6, }, // Africa/Lusaka
    {  477, 3853,  124827,  7, }, // Africa/Nairobi
    {  477, 5271,   65443,  8, }, // Africa/Ndjamena
    {  477, 5354,   26396,  6, }, // Africa/Niamey
    {  477, 5225,  158385, 10, }, // Africa/Porto-Novo
    {  477, 2866,   35756,  7, }, // Africa/Tripoli
    {  477, 6554,   74069,  5, }, // Africa/Tunis
    {  477, 4554,  158395,  9, }, // America/Araguaina
    {  477, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  477, 3679,   74074,  4, }, // America/Bahia
    {  477, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  477, 3868,   33182,  6, }, // America/Cancun
    {  477, 5496,  158404,  6, }, // America/Cayman
    {  477, 5723,  158410,  7, }, // America/Curacao
    {  477, 5739,  158417,  8, }, // America/Dominica
    {  477, 5441,    6688,  8, }, // America/El_Salvador
    {  477, 4098,  158425, 10, }, // America/Montevideo
    {  477, 5824,  158435, 10, }, // America/Montserrat
    {  477, 2379,  158445, 11, }, // America/New_York
    {  477, 3945,  158456, 14, }, // America/Port-au-Prince
    {  477, 5954,  158470, 16, }, // America/Port_of_Spain
    {  477, 2061,     332, 10, }, // America/St_Johns
    {  477,  855,     360, 10, }, // America/St_Thomas
    {  477, 5570,  158486,  4, }, // Asia/Aden
    {  477, 5523,   91196,  5, }, // Asia/Aqtau
    {  477, 5368,  158490,  6, }, // Asia/Aqtobe
    {  477, 3607,  158496,  7, }, // Asia/Baghdad
    {  477, 5534,  158503,  5, }, // Asia/Bahrain
    {  477, 4466,  158508, 11, }, // Asia/Bangkok
    {  477, 1093,  158367,  4, }, // Asia/Dhaka
    {  477, 4239,  158519,  7, }, // Asia/Karachi
    {  477, 1185,  158526,  8, }, // Asia/Kathmandu
    {  477, 5547,   35929,  6, }, // Asia/Kuwait
    {  477, 1349,   65105,  8, }, // Asia/Makassar
    {  477, 8708,     434,  8, }, // Asia/Qostanay
    {  477, 3263,  158534,  5, }, // Asia/Taipei
    {  477, 2772,  158539,  6, }, // Asia/Tehran
    {  477, 1317,    5640,  6, }, // Asia/Thimphu
    {  477, 1235,     442,  6, }, // Asia/Yangon
    {  477, 3747,   26815,  6, }, // Asia/Yerevan
    {  477, 1395,  158211,  5, }, // Atlantic/Faroe
    {  477, 2748,  158545, 12, }, // Atlantic/Reykjavik
    {  477, 1547,  158557, 17, }, // Australia/Lord_Howe
    {  477, 6441,  158574, 10, }, // Europe/Amsterdam
    {  477, 8772,  158584,  8, }, // Europe/Busingen
    {  477, 6289,  158592,  9, }, // Europe/Copenhagen
    {  477, 6612,  158601, 10, }, // Europe/Helsinki
    {  477, 3979,  158611, 12, }, // Europe/Kaliningrad
    {  477, 2501,   21034,  7, }, // Europe/London
    {  477, 4892,   62944,  6, }, // Europe/Madrid
    {  477, 6410,   74455,  6, }, // Europe/Monaco
    {  477, 3537,  158623,  7, }, // Europe/Moscow
    {  477, 6470,  158630, 11, }, // Europe/San_Marino
    {  477, 8867,  158641,  5, }, // Europe/Tallinn
    {  477, 6046,  158646, 13, }, // Indian/Christmas
    {  477, 6063,  158659, 11, }, // Indian/Cocos
    {  477, 7260,  158670,  4, }, // Indian/Mahe
    {  477, 4069,  158674,  8, }, // Indian/Mauritius
    {  477, 3015,   58319,  7, }, // Pacific/Chatham
    {  477, 2237,  158682, 14, }, // Pacific/Easter
    {  477, 7360,  158696,  5, }, // Pacific/Efate
    {  477, 7194,  158701, 14, }, // Pacific/Pitcairn
    {  477, 6712,  158715,  6, }, // Pacific/Tarawa
    {  478, 5059,  158357, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Equatorial Guinea
    {  478,   44,   74009,  6, }, // Africa/Bamako
    {  478, 6840,  158367,  5, }, // Africa/Dakar
    {  478, 5043,  158372,  6, }, // Africa/Djibouti
    {  478, 5151,  158378,  7, }, // Africa/Kampala
    {  478, 5001,   74047,  6, }, // Africa/Lusaka
    {  478, 3853,  124827,  7, }, // Africa/Nairobi
    {  478, 5271,   65443,  8, }, // Africa/Ndjamena
    {  478, 5354,   26396,  6, }, // Africa/Niamey
    {  478, 5225,  158385, 10, }, // Africa/Porto-Novo
    {  478, 2866,   35756,  7, }, // Africa/Tripoli
    {  478, 6554,   74069,  5, }, // Africa/Tunis
    {  478, 4554,  158395,  9, }, // America/Araguaina
    {  478, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  478, 3679,   74074,  4, }, // America/Bahia
    {  478, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  478, 3868,   33182,  6, }, // America/Cancun
    {  478, 5496,  158404,  6, }, // America/Cayman
    {  478, 5723,  158410,  7, }, // America/Curacao
    {  478, 5739,  158417,  8, }, // America/Dominica
    {  478, 5441,    6688,  8, }, // America/El_Salvador
    {  478, 4098,  158425, 10, }, // America/Montevideo
    {  478, 5824,  158435, 10, }, // America/Montserrat
    {  478, 2379,  158445, 11, }, // America/New_York
    {  478, 3945,  158456, 14, }, // America/Port-au-Prince
    {  478, 5954,  158470, 16, }, // America/Port_of_Spain
    {  478, 2061,     332, 10, }, // America/St_Johns
    {  478,  855,     360, 10, }, // America/St_Thomas
    {  478, 5570,  158486,  4, }, // Asia/Aden
    {  478, 5523,   91196,  5, }, // Asia/Aqtau
    {  478, 5368,  158490,  6, }, // Asia/Aqtobe
    {  478, 3607,  158496,  7, }, // Asia/Baghdad
    {  478, 5534,  158503,  5, }, // Asia/Bahrain
    {  478, 4466,  158508, 11, }, // Asia/Bangkok
    {  478, 1093,  158367,  4, }, // Asia/Dhaka
    {  478, 4239,  158519,  7, }, // Asia/Karachi
    {  478, 1185,  158526,  8, }, // Asia/Kathmandu
    {  478, 5547,   35929,  6, }, // Asia/Kuwait
    {  478, 1349,   65105,  8, }, // Asia/Makassar
    {  478, 8708,     434,  8, }, // Asia/Qostanay
    {  478, 3263,  158534,  5, }, // Asia/Taipei
    {  478, 2772,  158539,  6, }, // Asia/Tehran
    {  478, 1317,    5640,  6, }, // Asia/Thimphu
    {  478, 1235,     442,  6, }, // Asia/Yangon
    {  478, 3747,   26815,  6, }, // Asia/Yerevan
    {  478, 1395,  158211,  5, }, // Atlantic/Faroe
    {  478, 2748,  158545, 12, }, // Atlantic/Reykjavik
    {  478, 1547,  158557, 17, }, // Australia/Lord_Howe
    {  478, 6441,  158574, 10, }, // Europe/Amsterdam
    {  478, 8772,  158584,  8, }, // Europe/Busingen
    {  478, 6289,  158592,  9, }, // Europe/Copenhagen
    {  478, 6612,  158601, 10, }, // Europe/Helsinki
    {  478, 3979,  158611, 12, }, // Europe/Kaliningrad
    {  478, 2501,   21034,  7, }, // Europe/London
    {  478, 4892,   62944,  6, }, // Europe/Madrid
    {  478, 6410,   74455,  6, }, // Europe/Monaco
    {  478, 3537,  158623,  7, }, // Europe/Moscow
    {  478, 6470,  158630, 11, }, // Europe/San_Marino
    {  478, 8867,  158641,  5, }, // Europe/Tallinn
    {  478, 6046,  158646, 13, }, // Indian/Christmas
    {  478, 6063,  158659, 11, }, // Indian/Cocos
    {  478, 7260,  158670,  4, }, // Indian/Mahe
    {  478, 4069,  158674,  8, }, // Indian/Mauritius
    {  478, 3015,   58319,  7, }, // Pacific/Chatham
    {  478, 2237,  158682, 14, }, // Pacific/Easter
    {  478, 7360,  158696,  5, }, // Pacific/Efate
    {  478, 7194,  158701, 14, }, // Pacific/Pitcairn
    {  478, 6712,  158715,  6, }, // Pacific/Tarawa
    {  479, 5059,  158357, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Guinea-Bissau
    {  479,   44,   74009,  6, }, // Africa/Bamako
    {  479, 6840,  158367,  5, }, // Africa/Dakar
    {  479, 5043,  158372,  6, }, // Africa/Djibouti
    {  479, 5151,  158378,  7, }, // Africa/Kampala
    {  479, 5001,   74047,  6, }, // Africa/Lusaka
    {  479, 3853,  124827,  7, }, // Africa/Nairobi
    {  479, 5271,   65443,  8, }, // Africa/Ndjamena
    {  479, 5354,   26396,  6, }, // Africa/Niamey
    {  479, 5225,  158385, 10, }, // Africa/Porto-Novo
    {  479, 2866,   35756,  7, }, // Africa/Tripoli
    {  479, 6554,   74069,  5, }, // Africa/Tunis
    {  479, 4554,  158395,  9, }, // America/Araguaina
    {  479, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  479, 3679,   74074,  4, }, // America/Bahia
    {  479, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  479, 3868,   33182,  6, }, // America/Cancun
    {  479, 5496,  158404,  6, }, // America/Cayman
    {  479, 5723,  158410,  7, }, // America/Curacao
    {  479, 5739,  158417,  8, }, // America/Dominica
    {  479, 5441,    6688,  8, }, // America/El_Salvador
    {  479, 4098,  158425, 10, }, // America/Montevideo
    {  479, 5824,  158435, 10, }, // America/Montserrat
    {  479, 2379,  158445, 11, }, // America/New_York
    {  479, 3945,  158456, 14, }, // America/Port-au-Prince
    {  479, 5954,  158470, 16, }, // America/Port_of_Spain
    {  479, 2061,     332, 10, }, // America/St_Johns
    {  479,  855,     360, 10, }, // America/St_Thomas
    {  479, 5570,  158486,  4, }, // Asia/Aden
    {  479, 5523,   91196,  5, }, // Asia/Aqtau
    {  479, 5368,  158490,  6, }, // Asia/Aqtobe
    {  479, 3607,  158496,  7, }, // Asia/Baghdad
    {  479, 5534,  158503,  5, }, // Asia/Bahrain
    {  479, 4466,  158508, 11, }, // Asia/Bangkok
    {  479, 1093,  158367,  4, }, // Asia/Dhaka
    {  479, 4239,  158519,  7, }, // Asia/Karachi
    {  479, 1185,  158526,  8, }, // Asia/Kathmandu
    {  479, 5547,   35929,  6, }, // Asia/Kuwait
    {  479, 1349,   65105,  8, }, // Asia/Makassar
    {  479, 8708,     434,  8, }, // Asia/Qostanay
    {  479, 3263,  158534,  5, }, // Asia/Taipei
    {  479, 2772,  158539,  6, }, // Asia/Tehran
    {  479, 1317,    5640,  6, }, // Asia/Thimphu
    {  479, 1235,     442,  6, }, // Asia/Yangon
    {  479, 3747,   26815,  6, }, // Asia/Yerevan
    {  479, 1395,  158211,  5, }, // Atlantic/Faroe
    {  479, 2748,  158545, 12, }, // Atlantic/Reykjavik
    {  479, 1547,  158557, 17, }, // Australia/Lord_Howe
    {  479, 6441,  158574, 10, }, // Europe/Amsterdam
    {  479, 8772,  158584,  8, }, // Europe/Busingen
    {  479, 6289,  158592,  9, }, // Europe/Copenhagen
    {  479, 6612,  158601, 10, }, // Europe/Helsinki
    {  479, 3979,  158611, 12, }, // Europe/Kaliningrad
    {  479, 2501,   21034,  7, }, // Europe/London
    {  479, 4892,   62944,  6, }, // Europe/Madrid
    {  479, 6410,   74455,  6, }, // Europe/Monaco
    {  479, 3537,  158623,  7, }, // Europe/Moscow
    {  479, 6470,  158630, 11, }, // Europe/San_Marino
    {  479, 8867,  158641,  5, }, // Europe/Tallinn
    {  479, 6046,  158646, 13, }, // Indian/Christmas
    {  479, 6063,  158659, 11, }, // Indian/Cocos
    {  479, 7260,  158670,  4, }, // Indian/Mahe
    {  479, 4069,  158674,  8, }, // Indian/Mauritius
    {  479, 3015,   58319,  7, }, // Pacific/Chatham
    {  479, 2237,  158682, 14, }, // Pacific/Easter
    {  479, 7360,  158696,  5, }, // Pacific/Efate
    {  479, 7194,  158701, 14, }, // Pacific/Pitcairn
    {  479, 6712,  158715,  6, }, // Pacific/Tarawa
    {  480, 5059,  158357, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Luxembourg
    {  480,   44,   74009,  6, }, // Africa/Bamako
    {  480, 6840,  158367,  5, }, // Africa/Dakar
    {  480, 5043,  158372,  6, }, // Africa/Djibouti
    {  480, 5151,  158378,  7, }, // Africa/Kampala
    {  480, 5001,   74047,  6, }, // Africa/Lusaka
    {  480, 3853,  124827,  7, }, // Africa/Nairobi
    {  480, 5271,   65443,  8, }, // Africa/Ndjamena
    {  480, 5354,   26396,  6, }, // Africa/Niamey
    {  480, 5225,  158385, 10, }, // Africa/Porto-Novo
    {  480, 2866,   35756,  7, }, // Africa/Tripoli
    {  480, 6554,   74069,  5, }, // Africa/Tunis
    {  480, 4554,  158395,  9, }, // America/Araguaina
    {  480, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  480, 3679,   74074,  4, }, // America/Bahia
    {  480, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  480, 3868,   33182,  6, }, // America/Cancun
    {  480, 5496,  158404,  6, }, // America/Cayman
    {  480, 5723,  158410,  7, }, // America/Curacao
    {  480, 5739,  158417,  8, }, // America/Dominica
    {  480, 5441,    6688,  8, }, // America/El_Salvador
    {  480, 4098,  158425, 10, }, // America/Montevideo
    {  480, 5824,  158435, 10, }, // America/Montserrat
    {  480, 2379,  158445, 11, }, // America/New_York
    {  480, 3945,  158456, 14, }, // America/Port-au-Prince
    {  480, 5954,  158470, 16, }, // America/Port_of_Spain
    {  480, 2061,     332, 10, }, // America/St_Johns
    {  480,  855,     360, 10, }, // America/St_Thomas
    {  480, 5570,  158486,  4, }, // Asia/Aden
    {  480, 5523,   91196,  5, }, // Asia/Aqtau
    {  480, 5368,  158490,  6, }, // Asia/Aqtobe
    {  480, 3607,  158496,  7, }, // Asia/Baghdad
    {  480, 5534,  158503,  5, }, // Asia/Bahrain
    {  480, 4466,  158508, 11, }, // Asia/Bangkok
    {  480, 1093,  158367,  4, }, // Asia/Dhaka
    {  480, 4239,  158519,  7, }, // Asia/Karachi
    {  480, 1185,  158526,  8, }, // Asia/Kathmandu
    {  480, 5547,   35929,  6, }, // Asia/Kuwait
    {  480, 1349,   65105,  8, }, // Asia/Makassar
    {  480, 8708,     434,  8, }, // Asia/Qostanay
    {  480, 3263,  158534,  5, }, // Asia/Taipei
    {  480, 2772,  158539,  6, }, // Asia/Tehran
    {  480, 1317,    5640,  6, }, // Asia/Thimphu
    {  480, 1235,     442,  6, }, // Asia/Yangon
    {  480, 3747,   26815,  6, }, // Asia/Yerevan
    {  480, 1395,  158211,  5, }, // Atlantic/Faroe
    {  480, 2748,  158545, 12, }, // Atlantic/Reykjavik
    {  480, 1547,  158557, 17, }, // Australia/Lord_Howe
    {  480, 6441,  158574, 10, }, // Europe/Amsterdam
    {  480, 8772,  158584,  8, }, // Europe/Busingen
    {  480, 6289,  158592,  9, }, // Europe/Copenhagen
    {  480, 6612,  158601, 10, }, // Europe/Helsinki
    {  480, 3979,  158611, 12, }, // Europe/Kaliningrad
    {  480, 2501,   21034,  7, }, // Europe/London
    {  480, 4892,   62944,  6, }, // Europe/Madrid
    {  480, 6410,   74455,  6, }, // Europe/Monaco
    {  480, 3537,  158623,  7, }, // Europe/Moscow
    {  480, 6470,  158630, 11, }, // Europe/San_Marino
    {  480, 8867,  158641,  5, }, // Europe/Tallinn
    {  480, 6046,  158646, 13, }, // Indian/Christmas
    {  480, 6063,  158659, 11, }, // Indian/Cocos
    {  480, 7260,  158670,  4, }, // Indian/Mahe
    {  480, 4069,  158674,  8, }, // Indian/Mauritius
    {  480, 3015,   58319,  7, }, // Pacific/Chatham
    {  480, 2237,  158682, 14, }, // Pacific/Easter
    {  480, 7360,  158696,  5, }, // Pacific/Efate
    {  480, 7194,  158701, 14, }, // Pacific/Pitcairn
    {  480, 6712,  158715,  6, }, // Pacific/Tarawa
    {  481, 5059,  158357, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Macao
    {  481,   44,   74009,  6, }, // Africa/Bamako
    {  481, 6840,  158367,  5, }, // Africa/Dakar
    {  481, 5043,  158372,  6, }, // Africa/Djibouti
    {  481, 5151,  158378,  7, }, // Africa/Kampala
    {  481, 5001,   74047,  6, }, // Africa/Lusaka
    {  481, 3853,  124827,  7, }, // Africa/Nairobi
    {  481, 5271,   65443,  8, }, // Africa/Ndjamena
    {  481, 5354,   26396,  6, }, // Africa/Niamey
    {  481, 5225,  158385, 10, }, // Africa/Porto-Novo
    {  481, 2866,   35756,  7, }, // Africa/Tripoli
    {  481, 6554,   74069,  5, }, // Africa/Tunis
    {  481, 4554,  158395,  9, }, // America/Araguaina
    {  481, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  481, 3679,   74074,  4, }, // America/Bahia
    {  481, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  481, 3868,   33182,  6, }, // America/Cancun
    {  481, 5496,  158404,  6, }, // America/Cayman
    {  481, 5723,  158410,  7, }, // America/Curacao
    {  481, 5739,  158417,  8, }, // America/Dominica
    {  481, 5441,    6688,  8, }, // America/El_Salvador
    {  481, 4098,  158425, 10, }, // America/Montevideo
    {  481, 5824,  158435, 10, }, // America/Montserrat
    {  481, 2379,  158445, 11, }, // America/New_York
    {  481, 3945,  158456, 14, }, // America/Port-au-Prince
    {  481, 5954,  158470, 16, }, // America/Port_of_Spain
    {  481, 2061,     332, 10, }, // America/St_Johns
    {  481,  855,     360, 10, }, // America/St_Thomas
    {  481, 5570,  158486,  4, }, // Asia/Aden
    {  481, 5523,   91196,  5, }, // Asia/Aqtau
    {  481, 5368,  158490,  6, }, // Asia/Aqtobe
    {  481, 3607,  158496,  7, }, // Asia/Baghdad
    {  481, 5534,  158503,  5, }, // Asia/Bahrain
    {  481, 4466,  158508, 11, }, // Asia/Bangkok
    {  481, 1093,  158367,  4, }, // Asia/Dhaka
    {  481, 4239,  158519,  7, }, // Asia/Karachi
    {  481, 1185,  158526,  8, }, // Asia/Kathmandu
    {  481, 5547,   35929,  6, }, // Asia/Kuwait
    {  481, 1349,   65105,  8, }, // Asia/Makassar
    {  481, 8708,     434,  8, }, // Asia/Qostanay
    {  481, 3263,  158534,  5, }, // Asia/Taipei
    {  481, 2772,  158539,  6, }, // Asia/Tehran
    {  481, 1317,    5640,  6, }, // Asia/Thimphu
    {  481, 1235,     442,  6, }, // Asia/Yangon
    {  481, 3747,   26815,  6, }, // Asia/Yerevan
    {  481, 1395,  158211,  5, }, // Atlantic/Faroe
    {  481, 2748,  158545, 12, }, // Atlantic/Reykjavik
    {  481, 1547,  158557, 17, }, // Australia/Lord_Howe
    {  481, 6441,  158574, 10, }, // Europe/Amsterdam
    {  481, 8772,  158584,  8, }, // Europe/Busingen
    {  481, 6289,  158592,  9, }, // Europe/Copenhagen
    {  481, 6612,  158601, 10, }, // Europe/Helsinki
    {  481, 3979,  158611, 12, }, // Europe/Kaliningrad
    {  481, 2501,   21034,  7, }, // Europe/London
    {  481, 4892,   62944,  6, }, // Europe/Madrid
    {  481, 6410,   74455,  6, }, // Europe/Monaco
    {  481, 3537,  158623,  7, }, // Europe/Moscow
    {  481, 6470,  158630, 11, }, // Europe/San_Marino
    {  481, 8867,  158641,  5, }, // Europe/Tallinn
    {  481, 6046,  158646, 13, }, // Indian/Christmas
    {  481, 6063,  158659, 11, }, // Indian/Cocos
    {  481, 7260,  158670,  4, }, // Indian/Mahe
    {  481, 4069,  158674,  8, }, // Indian/Mauritius
    {  481, 3015,   58319,  7, }, // Pacific/Chatham
    {  481, 2237,  158682, 14, }, // Pacific/Easter
    {  481, 7360,  158696,  5, }, // Pacific/Efate
    {  481, 7194,  158701, 14, }, // Pacific/Pitcairn
    {  481, 6712,  158715,  6, }, // Pacific/Tarawa
    {  482, 5059,  158357, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Mozambique
    {  482,   44,   74009,  6, }, // Africa/Bamako
    {  482, 6840,  158367,  5, }, // Africa/Dakar
    {  482, 5043,  158372,  6, }, // Africa/Djibouti
    {  482, 5151,  158378,  7, }, // Africa/Kampala
    {  482, 5001,   74047,  6, }, // Africa/Lusaka
    {  482, 3853,  124827,  7, }, // Africa/Nairobi
    {  482, 5271,   65443,  8, }, // Africa/Ndjamena
    {  482, 5354,   26396,  6, }, // Africa/Niamey
    {  482, 5225,  158385, 10, }, // Africa/Porto-Novo
    {  482, 2866,   35756,  7, }, // Africa/Tripoli
    {  482, 6554,   74069,  5, }, // Africa/Tunis
    {  482, 4554,  158395,  9, }, // America/Araguaina
    {  482, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  482, 3679,   74074,  4, }, // America/Bahia
    {  482, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  482, 3868,   33182,  6, }, // America/Cancun
    {  482, 5496,  158404,  6, }, // America/Cayman
    {  482, 5723,  158410,  7, }, // America/Curacao
    {  482, 5739,  158417,  8, }, // America/Dominica
    {  482, 5441,    6688,  8, }, // America/El_Salvador
    {  482, 4098,  158425, 10, }, // America/Montevideo
    {  482, 5824,  158435, 10, }, // America/Montserrat
    {  482, 2379,  158445, 11, }, // America/New_York
    {  482, 3945,  158456, 14, }, // America/Port-au-Prince
    {  482, 5954,  158470, 16, }, // America/Port_of_Spain
    {  482, 2061,     332, 10, }, // America/St_Johns
    {  482,  855,     360, 10, }, // America/St_Thomas
    {  482, 5570,  158486,  4, }, // Asia/Aden
    {  482, 5523,   91196,  5, }, // Asia/Aqtau
    {  482, 5368,  158490,  6, }, // Asia/Aqtobe
    {  482, 3607,  158496,  7, }, // Asia/Baghdad
    {  482, 5534,  158503,  5, }, // Asia/Bahrain
    {  482, 4466,  158508, 11, }, // Asia/Bangkok
    {  482, 1093,  158367,  4, }, // Asia/Dhaka
    {  482, 4239,  158519,  7, }, // Asia/Karachi
    {  482, 1185,  158526,  8, }, // Asia/Kathmandu
    {  482, 5547,   35929,  6, }, // Asia/Kuwait
    {  482, 1349,   65105,  8, }, // Asia/Makassar
    {  482, 8708,     434,  8, }, // Asia/Qostanay
    {  482, 3263,  158534,  5, }, // Asia/Taipei
    {  482, 2772,  158539,  6, }, // Asia/Tehran
    {  482, 1317,    5640,  6, }, // Asia/Thimphu
    {  482, 1235,     442,  6, }, // Asia/Yangon
    {  482, 3747,   26815,  6, }, // Asia/Yerevan
    {  482, 1395,  158211,  5, }, // Atlantic/Faroe
    {  482, 2748,  158545, 12, }, // Atlantic/Reykjavik
    {  482, 1547,  158557, 17, }, // Australia/Lord_Howe
    {  482, 6441,  158574, 10, }, // Europe/Amsterdam
    {  482, 8772,  158584,  8, }, // Europe/Busingen
    {  482, 6289,  158592,  9, }, // Europe/Copenhagen
    {  482, 6612,  158601, 10, }, // Europe/Helsinki
    {  482, 3979,  158611, 12, }, // Europe/Kaliningrad
    {  482, 2501,   21034,  7, }, // Europe/London
    {  482, 4892,   62944,  6, }, // Europe/Madrid
    {  482, 6410,   74455,  6, }, // Europe/Monaco
    {  482, 3537,  158623,  7, }, // Europe/Moscow
    {  482, 6470,  158630, 11, }, // Europe/San_Marino
    {  482, 8867,  158641,  5, }, // Europe/Tallinn
    {  482, 6046,  158646, 13, }, // Indian/Christmas
    {  482, 6063,  158659, 11, }, // Indian/Cocos
    {  482, 7260,  158670,  4, }, // Indian/Mahe
    {  482, 4069,  158674,  8, }, // Indian/Mauritius
    {  482, 3015,   58319,  7, }, // Pacific/Chatham
    {  482, 2237,  158682, 14, }, // Pacific/Easter
    {  482, 7360,  158696,  5, }, // Pacific/Efate
    {  482, 7194,  158701, 14, }, // Pacific/Pitcairn
    {  482, 6712,  158715,  6, }, // Pacific/Tarawa
    {  483, 5059,  158357, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Portugal
    {  483,   44,   74009,  6, }, // Africa/Bamako
    {  483, 6840,  158367,  5, }, // Africa/Dakar
    {  483, 5043,  158372,  6, }, // Africa/Djibouti
    {  483, 5151,  158378,  7, }, // Africa/Kampala
    {  483, 5001,   74047,  6, }, // Africa/Lusaka
    {  483, 3853,  124827,  7, }, // Africa/Nairobi
    {  483, 5271,   65443,  8, }, // Africa/Ndjamena
    {  483, 5354,   26396,  6, }, // Africa/Niamey
    {  483, 5225,  158385, 10, }, // Africa/Porto-Novo
    {  483, 2866,   35756,  7, }, // Africa/Tripoli
    {  483, 6554,   74069,  5, }, // Africa/Tunis
    {  483, 4554,  158395,  9, }, // America/Araguaina
    {  483, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  483, 3679,   74074,  4, }, // America/Bahia
    {  483, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  483, 3868,   33182,  6, }, // America/Cancun
    {  483, 5496,  158404,  6, }, // America/Cayman
    {  483, 5723,  158410,  7, }, // America/Curacao
    {  483, 5739,  158417,  8, }, // America/Dominica
    {  483, 5441,    6688,  8, }, // America/El_Salvador
    {  483, 4098,  158425, 10, }, // America/Montevideo
    {  483, 5824,  158435, 10, }, // America/Montserrat
    {  483, 2379,  158445, 11, }, // America/New_York
    {  483, 3945,  158456, 14, }, // America/Port-au-Prince
    {  483, 5954,  158470, 16, }, // America/Port_of_Spain
    {  483, 2061,     332, 10, }, // America/St_Johns
    {  483,  855,     360, 10, }, // America/St_Thomas
    {  483, 5570,  158486,  4, }, // Asia/Aden
    {  483, 5523,   91196,  5, }, // Asia/Aqtau
    {  483, 5368,  158490,  6, }, // Asia/Aqtobe
    {  483, 3607,  158496,  7, }, // Asia/Baghdad
    {  483, 5534,  158503,  5, }, // Asia/Bahrain
    {  483, 4466,  158508, 11, }, // Asia/Bangkok
    {  483, 1093,  158367,  4, }, // Asia/Dhaka
    {  483, 4239,  158519,  7, }, // Asia/Karachi
    {  483, 1185,  158526,  8, }, // Asia/Kathmandu
    {  483, 5547,   35929,  6, }, // Asia/Kuwait
    {  483, 1349,   65105,  8, }, // Asia/Makassar
    {  483, 8708,     434,  8, }, // Asia/Qostanay
    {  483, 3263,  158534,  5, }, // Asia/Taipei
    {  483, 2772,  158539,  6, }, // Asia/Tehran
    {  483, 1317,    5640,  6, }, // Asia/Thimphu
    {  483, 1235,     442,  6, }, // Asia/Yangon
    {  483, 3747,   26815,  6, }, // Asia/Yerevan
    {  483, 1395,  158211,  5, }, // Atlantic/Faroe
    {  483, 2748,  158545, 12, }, // Atlantic/Reykjavik
    {  483, 1547,  158557, 17, }, // Australia/Lord_Howe
    {  483, 6441,  158574, 10, }, // Europe/Amsterdam
    {  483, 8772,  158584,  8, }, // Europe/Busingen
    {  483, 6289,  158592,  9, }, // Europe/Copenhagen
    {  483, 6612,  158601, 10, }, // Europe/Helsinki
    {  483, 3979,  158611, 12, }, // Europe/Kaliningrad
    {  483, 2501,   21034,  7, }, // Europe/London
    {  483, 4892,   62944,  6, }, // Europe/Madrid
    {  483, 6410,   74455,  6, }, // Europe/Monaco
    {  483, 3537,  158623,  7, }, // Europe/Moscow
    {  483, 6470,  158630, 11, }, // Europe/San_Marino
    {  483, 8867,  158641,  5, }, // Europe/Tallinn
    {  483, 6046,  158646, 13, }, // Indian/Christmas
    {  483, 6063,  158659, 11, }, // Indian/Cocos
    {  483, 7260,  158670,  4, }, // Indian/Mahe
    {  483, 4069,  158674,  8, }, // Indian/Mauritius
    {  483, 3015,   58319,  7, }, // Pacific/Chatham
    {  483, 2237,  158682, 14, }, // Pacific/Easter
    {  483, 7360,  158696,  5, }, // Pacific/Efate
    {  483, 7194,  158701, 14, }, // Pacific/Pitcairn
    {  483, 6712,  158715,  6, }, // Pacific/Tarawa
    {  484, 5059,  158357, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Sao Tome and Principe
    {  484,   44,   74009,  6, }, // Africa/Bamako
    {  484, 6840,  158367,  5, }, // Africa/Dakar
    {  484, 5043,  158372,  6, }, // Africa/Djibouti
    {  484, 5151,  158378,  7, }, // Africa/Kampala
    {  484, 5001,   74047,  6, }, // Africa/Lusaka
    {  484, 3853,  124827,  7, }, // Africa/Nairobi
    {  484, 5271,   65443,  8, }, // Africa/Ndjamena
    {  484, 5354,   26396,  6, }, // Africa/Niamey
    {  484, 5225,  158385, 10, }, // Africa/Porto-Novo
    {  484, 2866,   35756,  7, }, // Africa/Tripoli
    {  484, 6554,   74069,  5, }, // Africa/Tunis
    {  484, 4554,  158395,  9, }, // America/Araguaina
    {  484, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  484, 3679,   74074,  4, }, // America/Bahia
    {  484, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  484, 3868,   33182,  6, }, // America/Cancun
    {  484, 5496,  158404,  6, }, // America/Cayman
    {  484, 5723,  158410,  7, }, // America/Curacao
    {  484, 5739,  158417,  8, }, // America/Dominica
    {  484, 5441,    6688,  8, }, // America/El_Salvador
    {  484, 4098,  158425, 10, }, // America/Montevideo
    {  484, 5824,  158435, 10, }, // America/Montserrat
    {  484, 2379,  158445, 11, }, // America/New_York
    {  484, 3945,  158456, 14, }, // America/Port-au-Prince
    {  484, 5954,  158470, 16, }, // America/Port_of_Spain
    {  484, 2061,     332, 10, }, // America/St_Johns
    {  484,  855,     360, 10, }, // America/St_Thomas
    {  484, 5570,  158486,  4, }, // Asia/Aden
    {  484, 5523,   91196,  5, }, // Asia/Aqtau
    {  484, 5368,  158490,  6, }, // Asia/Aqtobe
    {  484, 3607,  158496,  7, }, // Asia/Baghdad
    {  484, 5534,  158503,  5, }, // Asia/Bahrain
    {  484, 4466,  158508, 11, }, // Asia/Bangkok
    {  484, 1093,  158367,  4, }, // Asia/Dhaka
    {  484, 4239,  158519,  7, }, // Asia/Karachi
    {  484, 1185,  158526,  8, }, // Asia/Kathmandu
    {  484, 5547,   35929,  6, }, // Asia/Kuwait
    {  484, 1349,   65105,  8, }, // Asia/Makassar
    {  484, 8708,     434,  8, }, // Asia/Qostanay
    {  484, 3263,  158534,  5, }, // Asia/Taipei
    {  484, 2772,  158539,  6, }, // Asia/Tehran
    {  484, 1317,    5640,  6, }, // Asia/Thimphu
    {  484, 1235,     442,  6, }, // Asia/Yangon
    {  484, 3747,   26815,  6, }, // Asia/Yerevan
    {  484, 1395,  158211,  5, }, // Atlantic/Faroe
    {  484, 2748,  158545, 12, }, // Atlantic/Reykjavik
    {  484, 1547,  158557, 17, }, // Australia/Lord_Howe
    {  484, 6441,  158574, 10, }, // Europe/Amsterdam
    {  484, 8772,  158584,  8, }, // Europe/Busingen
    {  484, 6289,  158592,  9, }, // Europe/Copenhagen
    {  484, 6612,  158601, 10, }, // Europe/Helsinki
    {  484, 3979,  158611, 12, }, // Europe/Kaliningrad
    {  484, 2501,   21034,  7, }, // Europe/London
    {  484, 4892,   62944,  6, }, // Europe/Madrid
    {  484, 6410,   74455,  6, }, // Europe/Monaco
    {  484, 3537,  158623,  7, }, // Europe/Moscow
    {  484, 6470,  158630, 11, }, // Europe/San_Marino
    {  484, 8867,  158641,  5, }, // Europe/Tallinn
    {  484, 6046,  158646, 13, }, // Indian/Christmas
    {  484, 6063,  158659, 11, }, // Indian/Cocos
    {  484, 7260,  158670,  4, }, // Indian/Mahe
    {  484, 4069,  158674,  8, }, // Indian/Mauritius
    {  484, 3015,   58319,  7, }, // Pacific/Chatham
    {  484, 2237,  158682, 14, }, // Pacific/Easter
    {  484, 7360,  158696,  5, }, // Pacific/Efate
    {  484, 7194,  158701, 14, }, // Pacific/Pitcairn
    {  484, 6712,  158715,  6, }, // Pacific/Tarawa
    {  485, 5059,  158357, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Switzerland
    {  485,   44,   74009,  6, }, // Africa/Bamako
    {  485, 6840,  158367,  5, }, // Africa/Dakar
    {  485, 5043,  158372,  6, }, // Africa/Djibouti
    {  485, 5151,  158378,  7, }, // Africa/Kampala
    {  485, 5001,   74047,  6, }, // Africa/Lusaka
    {  485, 3853,  124827,  7, }, // Africa/Nairobi
    {  485, 5271,   65443,  8, }, // Africa/Ndjamena
    {  485, 5354,   26396,  6, }, // Africa/Niamey
    {  485, 5225,  158385, 10, }, // Africa/Porto-Novo
    {  485, 2866,   35756,  7, }, // Africa/Tripoli
    {  485, 6554,   74069,  5, }, // Africa/Tunis
    {  485, 4554,  158395,  9, }, // America/Araguaina
    {  485, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  485, 3679,   74074,  4, }, // America/Bahia
    {  485, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  485, 3868,   33182,  6, }, // America/Cancun
    {  485, 5496,  158404,  6, }, // America/Cayman
    {  485, 5723,  158410,  7, }, // America/Curacao
    {  485, 5739,  158417,  8, }, // America/Dominica
    {  485, 5441,    6688,  8, }, // America/El_Salvador
    {  485, 4098,  158425, 10, }, // America/Montevideo
    {  485, 5824,  158435, 10, }, // America/Montserrat
    {  485, 2379,  158445, 11, }, // America/New_York
    {  485, 3945,  158456, 14, }, // America/Port-au-Prince
    {  485, 5954,  158470, 16, }, // America/Port_of_Spain
    {  485, 2061,     332, 10, }, // America/St_Johns
    {  485,  855,     360, 10, }, // America/St_Thomas
    {  485, 5570,  158486,  4, }, // Asia/Aden
    {  485, 5523,   91196,  5, }, // Asia/Aqtau
    {  485, 5368,  158490,  6, }, // Asia/Aqtobe
    {  485, 3607,  158496,  7, }, // Asia/Baghdad
    {  485, 5534,  158503,  5, }, // Asia/Bahrain
    {  485, 4466,  158508, 11, }, // Asia/Bangkok
    {  485, 1093,  158367,  4, }, // Asia/Dhaka
    {  485, 4239,  158519,  7, }, // Asia/Karachi
    {  485, 1185,  158526,  8, }, // Asia/Kathmandu
    {  485, 5547,   35929,  6, }, // Asia/Kuwait
    {  485, 1349,   65105,  8, }, // Asia/Makassar
    {  485, 8708,     434,  8, }, // Asia/Qostanay
    {  485, 3263,  158534,  5, }, // Asia/Taipei
    {  485, 2772,  158539,  6, }, // Asia/Tehran
    {  485, 1317,    5640,  6, }, // Asia/Thimphu
    {  485, 1235,     442,  6, }, // Asia/Yangon
    {  485, 3747,   26815,  6, }, // Asia/Yerevan
    {  485, 1395,  158211,  5, }, // Atlantic/Faroe
    {  485, 2748,  158545, 12, }, // Atlantic/Reykjavik
    {  485, 1547,  158557, 17, }, // Australia/Lord_Howe
    {  485, 6441,  158574, 10, }, // Europe/Amsterdam
    {  485, 8772,  158584,  8, }, // Europe/Busingen
    {  485, 6289,  158592,  9, }, // Europe/Copenhagen
    {  485, 6612,  158601, 10, }, // Europe/Helsinki
    {  485, 3979,  158611, 12, }, // Europe/Kaliningrad
    {  485, 2501,   21034,  7, }, // Europe/London
    {  485, 4892,   62944,  6, }, // Europe/Madrid
    {  485, 6410,   74455,  6, }, // Europe/Monaco
    {  485, 3537,  158623,  7, }, // Europe/Moscow
    {  485, 6470,  158630, 11, }, // Europe/San_Marino
    {  485, 8867,  158641,  5, }, // Europe/Tallinn
    {  485, 6046,  158646, 13, }, // Indian/Christmas
    {  485, 6063,  158659, 11, }, // Indian/Cocos
    {  485, 7260,  158670,  4, }, // Indian/Mahe
    {  485, 4069,  158674,  8, }, // Indian/Mauritius
    {  485, 3015,   58319,  7, }, // Pacific/Chatham
    {  485, 2237,  158682, 14, }, // Pacific/Easter
    {  485, 7360,  158696,  5, }, // Pacific/Efate
    {  485, 7194,  158701, 14, }, // Pacific/Pitcairn
    {  485, 6712,  158715,  6, }, // Pacific/Tarawa
    {  486, 5059,  158357, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Timor-Leste
    {  486,   44,   74009,  6, }, // Africa/Bamako
    {  486, 6840,  158367,  5, }, // Africa/Dakar
    {  486, 5043,  158372,  6, }, // Africa/Djibouti
    {  486, 5151,  158378,  7, }, // Africa/Kampala
    {  486, 5001,   74047,  6, }, // Africa/Lusaka
    {  486, 3853,  124827,  7, }, // Africa/Nairobi
    {  486, 5271,   65443,  8, }, // Africa/Ndjamena
    {  486, 5354,   26396,  6, }, // Africa/Niamey
    {  486, 5225,  158385, 10, }, // Africa/Porto-Novo
    {  486, 2866,   35756,  7, }, // Africa/Tripoli
    {  486, 6554,   74069,  5, }, // Africa/Tunis
    {  486, 4554,  158395,  9, }, // America/Araguaina
    {  486, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  486, 3679,   74074,  4, }, // America/Bahia
    {  486, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  486, 3868,   33182,  6, }, // America/Cancun
    {  486, 5496,  158404,  6, }, // America/Cayman
    {  486, 5723,  158410,  7, }, // America/Curacao
    {  486, 5739,  158417,  8, }, // America/Dominica
    {  486, 5441,    6688,  8, }, // America/El_Salvador
    {  486, 4098,  158425, 10, }, // America/Montevideo
    {  486, 5824,  158435, 10, }, // America/Montserrat
    {  486, 2379,  158445, 11, }, // America/New_York
    {  486, 3945,  158456, 14, }, // America/Port-au-Prince
    {  486, 5954,  158470, 16, }, // America/Port_of_Spain
    {  486, 2061,     332, 10, }, // America/St_Johns
    {  486,  855,     360, 10, }, // America/St_Thomas
    {  486, 5570,  158486,  4, }, // Asia/Aden
    {  486, 5523,   91196,  5, }, // Asia/Aqtau
    {  486, 5368,  158490,  6, }, // Asia/Aqtobe
    {  486, 3607,  158496,  7, }, // Asia/Baghdad
    {  486, 5534,  158503,  5, }, // Asia/Bahrain
    {  486, 4466,  158508, 11, }, // Asia/Bangkok
    {  486, 1093,  158367,  4, }, // Asia/Dhaka
    {  486, 4239,  158519,  7, }, // Asia/Karachi
    {  486, 1185,  158526,  8, }, // Asia/Kathmandu
    {  486, 5547,   35929,  6, }, // Asia/Kuwait
    {  486, 1349,   65105,  8, }, // Asia/Makassar
    {  486, 8708,     434,  8, }, // Asia/Qostanay
    {  486, 3263,  158534,  5, }, // Asia/Taipei
    {  486, 2772,  158539,  6, }, // Asia/Tehran
    {  486, 1317,    5640,  6, }, // Asia/Thimphu
    {  486, 1235,     442,  6, }, // Asia/Yangon
    {  486, 3747,   26815,  6, }, // Asia/Yerevan
    {  486, 1395,  158211,  5, }, // Atlantic/Faroe
    {  486, 2748,  158545, 12, }, // Atlantic/Reykjavik
    {  486, 1547,  158557, 17, }, // Australia/Lord_Howe
    {  486, 6441,  158574, 10, }, // Europe/Amsterdam
    {  486, 8772,  158584,  8, }, // Europe/Busingen
    {  486, 6289,  158592,  9, }, // Europe/Copenhagen
    {  486, 6612,  158601, 10, }, // Europe/Helsinki
    {  486, 3979,  158611, 12, }, // Europe/Kaliningrad
    {  486, 2501,   21034,  7, }, // Europe/London
    {  486, 4892,   62944,  6, }, // Europe/Madrid
    {  486, 6410,   74455,  6, }, // Europe/Monaco
    {  486, 3537,  158623,  7, }, // Europe/Moscow
    {  486, 6470,  158630, 11, }, // Europe/San_Marino
    {  486, 8867,  158641,  5, }, // Europe/Tallinn
    {  486, 6046,  158646, 13, }, // Indian/Christmas
    {  486, 6063,  158659, 11, }, // Indian/Cocos
    {  486, 7260,  158670,  4, }, // Indian/Mahe
    {  486, 4069,  158674,  8, }, // Indian/Mauritius
    {  486, 3015,   58319,  7, }, // Pacific/Chatham
    {  486, 2237,  158682, 14, }, // Pacific/Easter
    {  486, 7360,  158696,  5, }, // Pacific/Efate
    {  486, 7194,  158701, 14, }, // Pacific/Pitcairn
    {  486, 6712,  158715,  6, }, // Pacific/Tarawa
    {  487,   14,       0,  6, }, // Africa/Asmara Prussian/Latin/Poland
    {  487, 4435,       6,  8, }, // Africa/Sao_Tome
    {  487, 4252,      14,  8, }, // America/Asuncion
    {  487,  237,      22,  8, }, // America/Atikokan
    {  487, 7623,      30, 17, }, // America/Bahia_Banderas
    {  487, 3868,      47,  6, }, // America/Cancun
    {  487, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  487, 5723,      66,  7, }, // America/Curacao
    {  487,  481,      73, 13, }, // America/Indiana/Knox
    {  487, 7961,      86, 16, }, // America/Indiana/Marengo
    {  487, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  487, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  487, 8038,     139, 14, }, // America/Indiana/Vevay
    {  487, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  487, 8086,     171, 16, }, // America/Indiana/Winamac
    {  487, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  487, 5932,     207, 22, }, // America/Lower_Princes
    {  487, 8220,     229,  6, }, // America/Merida
    {  487, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  487, 8329,     255, 20, }, // America/North_Dakota/Center
    {  487, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  487,  393,     298,  4, }, // America/Nuuk
    {  487, 6899,     302, 16, }, // America/Scoresbysund
    {  487, 8508,     318, 14, }, // America/St_Barthelemy
    {  487, 2061,     332, 10, }, // America/St_Johns
    {  487, 5863,     342,  9, }, // America/St_Kitts
    {  487, 5880,     351,  9, }, // America/St_Lucia
    {  487,  855,     360, 10, }, // America/St_Thomas
    {  487, 5913,     370, 11, }, // America/St_Vincent
    {  487, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  487, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  487, 1185,     413,  9, }, // Asia/Kathmandu
    {  487,  992,     422,  7, }, // Asia/Kolkata
    {  487, 1211,     429,  5, }, // Asia/Macau
    {  487, 1235,     442,  6, }, // Asia/Yangon
    {  487, 1395,     448,  5, }, // Atlantic/Faroe
    {  487, 6821,     453, 10, }, // Atlantic/St_Helena
    {  487, 2527,     463,  4, }, // Europe/Kyiv
    {  487, 7211,     467,  7, }, // Indian/Reunion
    {  487, 3157,     474,  5, }, // Pacific/Chuuk
    {  487, 3096,     479,  7, }, // Pacific/Pohnpei
    {  488, 6788,  158721,  7, }, // Africa/Abidjan Punjabi/Gurmukhi/India
    {  488, 6760,  158728,  5, }, // Africa/Accra
    {  488, 5059,  158733, 10, }, // Africa/Addis_Ababa
    {  488, 7420,  158743,  7, }, // Africa/Algiers
    {  488,   14,  158750,  6, }, // Africa/Asmara
    {  488,   44,  158756,  5, }, // Africa/Bamako
    {  488, 5257,  158761,  6, }, // Africa/Bangui
    {  488, 6746,  158767,  6, }, // Africa/Banjul
    {  488, 7435,  158773,  5, }, // Africa/Bissau
    {  488, 4957,  158778,  7, }, // Africa/Blantyre
    {  488, 5287,  158785, 11, }, // Africa/Brazzaville
    {  488, 4922,  158796,  9, }, // Africa/Bujumbura
    {  488, 2320,  158805,  4, }, // Africa/Cairo
    {  488, 4117,  158809, 10, }, // Africa/Casablanca
    {  488, 7449,  158819,  5, }, // Africa/Ceuta
    {  488, 6773,  158824,  7, }, // Africa/Conakry
    {  488, 6840,  158831,  4, }, // Africa/Dakar
    {  488, 5130,  158835, 11, }, // Africa/Dar_es_Salaam
    {  488, 5043,  158846,  6, }, // Africa/Djibouti
    {  488, 5243,  158852,  5, }, // Africa/Douala
    {  488, 5166,  158857,  7, }, // Africa/El_Aaiun
    {  488, 6853,  158864,  7, }, // Africa/Freetown
    {  488, 4906,  158871,  7, }, // Africa/Gaborone
    {  488, 5015,  158878,  5, }, // Africa/Harare
    {  488, 4479,  158883,  8, }, // Africa/Johannesburg
    {  488, 4499,  158891,  4, }, // Africa/Juba
    {  488, 5151,  158895,  6, }, // Africa/Kampala
    {  488, 4524,  158901,  6, }, // Africa/Khartoum
    {  488, 4973,  158907,  6, }, // Africa/Kigali
    {  488, 5306,  158913,  8, }, // Africa/Kinshasa
    {  488, 4744,  158921,  5, }, // Africa/Lagos
    {  488, 5336,  158926,  8, }, // Africa/Libreville
    {  488, 6869,  158934,  3, }, // Africa/Lome
    {  488, 5211,  158937,  6, }, // Africa/Luanda
    {  488, 4939,  158943, 10, }, // Africa/Lubumbashi
    {  488, 5001,  158953,  6, }, // Africa/Lusaka
    {  488, 5322,  158959,  6, }, // Africa/Malabo
    {  488, 4987,  158965,  6, }, // Africa/Maputo
    {  488, 5197,  158971,  5, }, // Africa/Maseru
    {  488, 5182,  158976,  8, }, // Africa/Mbabane
    {  488, 5113,  158984,  9, }, // Africa/Mogadishu
    {  488, 7034,  158993,  8, }, // Africa/Monrovia
    {  488, 3853,  159001,  6, }, // Africa/Nairobi
    {  488, 5271,  159007,  8, }, // Africa/Ndjamena
    {  488, 5354,  159015,  5, }, // Africa/Niamey
    {  488, 6803,  159020,  8, }, // Africa/Nouakchott
    {  488, 6727,  159028,  8, }, // Africa/Ouagadougou
    {  488, 5225,  159036, 10, }, // Africa/Porto-Novo
    {  488, 4435,  159046,  7, }, // Africa/Sao_Tome
    {  488, 2866,  159053,  8, }, // Africa/Tripoli
    {  488, 6554,  159061,  5, }, // Africa/Tunis
    {  488, 4152,  159066,  8, }, // Africa/Windhoek
    {  488,  132,  159074,  3, }, // America/Adak
    {  488, 3346,  159077,  6, }, // America/Anchorage
    {  488, 5607,  159083,  7, }, // America/Anguilla
    {  488, 5624,  159090,  7, }, // America/Antigua
    {  488, 4554,  159097,  8, }, // America/Araguaina
    {  488,  166,  159105, 10, }, // America/Argentina/Buenos_Aires
    {  488,   91,  159115,  9, }, // America/Argentina/Catamarca
    {  488,  270,  159124,  7, }, // America/Argentina/Cordoba
    {  488,  441,  159131,  5, }, // America/Argentina/Jujuy
    {  488, 7462,  159136,  8, }, // America/Argentina/La_Rioja
    {  488,  565,  159144,  8, }, // America/Argentina/Mendoza
    {  488, 7489,  159152, 11, }, // America/Argentina/Rio_Gallegos
    {  488, 7520,  159163,  5, }, // America/Argentina/Salta
    {  488, 7544,  159168,  8, }, // America/Argentina/San_Juan
    {  488, 5580,  159176,  8, }, // America/Argentina/San_Luis
    {  488, 7571,  159184,  6, }, // America/Argentina/Tucuman
    {  488, 7597,  159190,  7, }, // America/Argentina/Ushuaia
    {  488, 5640,  159197,  5, }, // America/Aruba
    {  488, 4252,  159202,  8, }, // America/Asuncion
    {  488,  237,  159210,  7, }, // America/Atikokan
    {  488, 3679,  159217,  5, }, // America/Bahia
    {  488, 7623,  159222, 13, }, // America/Bahia_Banderas
    {  488, 5654,  159235,  8, }, // America/Barbados
    {  488, 7646,  159243,  5, }, // America/Belem
    {  488, 5407,  159248,  6, }, // America/Belize
    {  488, 7660,  159254, 12, }, // America/Blanc-Sablon
    {  488, 7681,  159266,  9, }, // America/Boa_Vista
    {  488, 4361,  159275,  6, }, // America/Bogota
    {  488, 7699,  159281,  4, }, // America/Boise
    {  488, 7713,  159285, 11, }, // America/Cambridge_Bay
    {  488, 7735,  159296, 13, }, // America/Campo_Grande
    {  488, 3868,  159309,  6, }, // America/Cancun
    {  488, 4694,  159315,  7, }, // America/Caracas
    {  488, 4345,  159322,  6, }, // America/Cayenne
    {  488, 5496,  159328,  5, }, // America/Cayman
    {  488, 2260,  159333,  7, }, // America/Chicago
    {  488, 7756,  159340,  8, }, // America/Chihuahua
    {  488, 7774,  159348, 14, }, // America/Ciudad_Juarez
    {  488, 5422,  159362, 10, }, // America/Costa_Rica
    {  488, 7796,  159372,  7, }, // America/Creston
    {  488, 3791,  159379,  6, }, // America/Cuiaba
    {  488, 5723,  159385,  7, }, // America/Curacao
    {  488, 7812,  159392, 11, }, // America/Danmarkshavn
    {  488, 7833,  159403,  5, }, // America/Dawson
    {  488, 7848,  159408, 11, }, // America/Dawson_Creek
    {  488,  805,  159419,  5, }, // America/Denver
    {  488, 3465,  159424,  7, }, // America/Detroit
    {  488, 5739,  159431,  8, }, // America/Dominica
    {  488,  893,  159439,  7, }, // America/Edmonton
    {  488, 7869,  159446,  8, }, // America/Eirunepe
    {  488, 5441,  159454, 10, }, // America/El_Salvador
    {  488, 7886,  159464, 10, }, // America/Fort_Nelson
    {  488, 7906,  159474, 10, }, // America/Fortaleza
    {  488, 7924,  159484,  7, }, // America/Glace_Bay
    {  488, 6881,  159491,  7, }, // America/Goose_Bay
    {  488, 4612,  159498, 10, }, // America/Grand_Turk
    {  488, 5770,  159508,  8, }, // America/Grenada
    {  488, 5786,  159516,  8, }, // America/Guadeloupe
    {  488, 3760,  159524,  9, }, // America/Guatemala
    {  488, 4841,  159533,  8, }, // America/Guayaquil
    {  488, 6932,  159541,  5, }, // America/Guyana
    {  488, 1939,  159546,  8, }, // America/Halifax
    {  488, 2281,  159554,  5, }, // America/Havana
    {  488, 7942,  159559,  9, }, // America/Hermosillo
    {  488,  348,  159568, 12, }, // America/Indiana/Indianapolis
    {  488,  481,  159580, 13, }, // America/Indiana/Knox
    {  488, 7961,  159593, 16, }, // America/Indiana/Marengo
    {  488, 7985,  159609, 17, }, // America/Indiana/Petersburg
    {  488, 8012,  159626, 17, }, // America/Indiana/Tell_City
    {  488, 8038,  159643, 13, }, // America/Indiana/Vevay
    {  488, 8060,  159656, 17, }, // America/Indiana/Vincennes
    {  488, 8086,  159673, 15, }, // America/Indiana/Winamac
    {  488, 8110,  159688,  6, }, // America/Inuvik
    {  488,  660,  159694,  7, }, // America/Iqaluit
    {  488, 2799,  159701,  6, }, // America/Jamaica
    {  488, 5380,  159707,  5, }, // America/Juneau
    {  488,  521,  159712,  8, }, // America/Kentucky/Louisville
    {  488, 8125,  159720, 17, }, // America/Kentucky/Monticello
    {  488, 5704,  159737, 11, }, // America/Kralendijk
    {  488, 4376,  159748,  7, }, // America/La_Paz
    {  488, 7169,  159755,  4, }, // America/Lima
    {  488, 3239,  159759,  9, }, // America/Los_Angeles
    {  488, 5932,  159768, 20, }, // America/Lower_Princes
    {  488, 8153,  159788,  5, }, // America/Maceio
    {  488, 8168,  159793,  6, }, // America/Managua
    {  488, 1908,  159799,  4, }, // America/Manaus
    {  488, 5897,  159803,  7, }, // America/Marigot
    {  488, 5805,  159810,  8, }, // America/Martinique
    {  488, 8184,  159818,  8, }, // America/Matamoros
    {  488, 2917,  159826,  8, }, // America/Mazatlan
    {  488, 8202,  159834,  8, }, // America/Menominee
    {  488, 8220,  159842,  6, }, // America/Merida
    {  488, 8235,  159848, 10, }, // America/Metlakatla
    {  488, 2949,  159858, 12, }, // America/Mexico_City
    {  488, 4391,  159870,  6, }, // America/Miquelon
    {  488, 8254,  159876,  6, }, // America/Moncton
    {  488, 8270,  159882,  7, }, // America/Monterrey
    {  488, 4098,  159889, 10, }, // America/Montevideo
    {  488, 5824,  159899,  9, }, // America/Montserrat
    {  488, 5481,  159908,  5, }, // America/Nassau
    {  488, 2379,  159913,  8, }, // America/New_York
    {  488, 8288,  159836,  3, }, // America/Nome
    {  488, 1850,  159921,  7, }, // America/Noronha
    {  488, 8301,  159928, 18, }, // America/North_Dakota/Beulah
    {  488, 8329,  159946, 18, }, // America/North_Dakota/Center
    {  488, 8357,  159964, 21, }, // America/North_Dakota/New_Salem
    {  488,  393,  159985,  3, }, // America/Nuuk
    {  488, 8388,  159988,  7, }, // America/Ojinaga
    {  488, 2356,  159995,  5, }, // America/Panama
    {  488, 6173,  160000,  9, }, // America/Paramaribo
    {  488, 2973,  160009,  6, }, // America/Phoenix
    {  488, 3945,  160015, 13, }, // America/Port-au-Prince
    {  488, 5954,  160028, 12, }, // America/Port_of_Spain
    {  488, 8404,  160040, 12, }, // America/Porto_Velho
    {  488, 5843,  160052, 11, }, // America/Puerto_Rico
    {  488, 4030,  160063, 10, }, // America/Punta_Arenas
    {  488, 8424,  160073, 12, }, // America/Rankin_Inlet
    {  488, 8445,  160085,  6, }, // America/Recife
    {  488, 1995,  160091,  6, }, // America/Regina
    {  488, 8460,  160097,  9, }, // America/Resolute
    {  488,  695,  160106, 11, }, // America/Rio_Branco
    {  488, 8477,  160117,  7, }, // America/Santarem
    {  488, 2201,  160124,  8, }, // America/Santiago
    {  488, 6111,  160132, 13, }, // America/Santo_Domingo
    {  488, 1878,  160145,  8, }, // America/Sao_Paulo
    {  488, 6899,  160153, 13, }, // America/Scoresbysund
    {  488, 8494,  160166,  5, }, // America/Sitka
    {  488, 8508,  160171, 14, }, // America/St_Barthelemy
    {  488, 2061,  160185, 10, }, // America/St_Johns
    {  488, 5863,  160195,  9, }, // America/St_Kitts
    {  488, 5880,  160204, 10, }, // America/St_Lucia
    {  488,  855,  160214,  9, }, // America/St_Thomas
    {  488, 5913,  160223, 12, }, // America/St_Vincent
    {  488, 8530,  160235, 10, }, // America/Swift_Current
    {  488, 5461,  160245, 10, }, // America/Tegucigalpa
    {  488, 5756,  160255,  4, }, // America/Thule
    {  488,  313,  160259,  7, }, // America/Tijuana
    {  488,  608,  160266,  7, }, // America/Toronto
    {  488, 5688,  160273,  7, }, // America/Tortola
    {  488, 2093,  160280,  7, }, // America/Vancouver
    {  488, 2144,  160287,  8, }, // America/Whitehorse
    {  488,  734,  160295,  7, }, // America/Winnipeg
    {  488, 8552,  160302,  5, }, // America/Yakutat
    {  488, 6001,  160169,  4, }, // Antarctica/Casey
    {  488, 6094,  160307,  5, }, // Antarctica/Davis
    {  488, 6133,  160312, 16, }, // Antarctica/DumontDUrville
    {  488, 8568,  160328,  8, }, // Antarctica/Macquarie
    {  488, 7066,  160336,  4, }, // Antarctica/Mawson
    {  488, 7113,  160340,  8, }, // Antarctica/McMurdo
    {  488, 8589,  160348,  4, }, // Antarctica/Palmer
    {  488, 7226,  160352,  6, }, // Antarctica/Rothera
    {  488, 7295,  160358,  5, }, // Antarctica/Syowa
    {  488, 8607,  160363,  4, }, // Antarctica/Troll
    {  488, 7374,  160367,  6, }, // Antarctica/Vostok
    {  488, 1429,  160373, 11, }, // Arctic/Longyearbyen
    {  488, 5570,  160384,  4, }, // Asia/Aden
    {  488, 5395,  160388,  6, }, // Asia/Almaty
    {  488, 3968,  160394,  4, }, // Asia/Amman
    {  488, 5511,  160398,  7, }, // Asia/Anadyr
    {  488, 5523,  160405,  4, }, // Asia/Aqtau
    {  488, 5368,  160409,  6, }, // Asia/Aqtobe
    {  488,  964,  160415,  8, }, // Asia/Ashgabat
    {  488, 8624,  160423,  6, }, // Asia/Atyrau
    {  488, 3607,  160429,  5, }, // Asia/Baghdad
    {  488, 5534,  160434,  6, }, // Asia/Bahrain
    {  488, 3653,  160440,  4, }, // Asia/Baku
    {  488, 4466,  160444,  6, }, // Asia/Bangkok
    {  488, 3571,  160450,  5, }, // Asia/Barnaul
    {  488, 4086,  160455,  5, }, // Asia/Beirut
    {  488, 3778,  160460,  7, }, // Asia/Bishkek
    {  488, 5989,  160467,  6, }, // Asia/Brunei
    {  488, 4601,  160473,  4, }, // Asia/Chita
    {  488, 4511,  160477,  6, }, // Asia/Colombo
    {  488, 4540,  160483,  6, }, // Asia/Damascus
    {  488, 1093,  160489,  4, }, // Asia/Dhaka
    {  488, 6192,  160493,  4, }, // Asia/Dili
    {  488, 3596,  160497,  4, }, // Asia/Dubai
    {  488, 6159,  160501,  8, }, // Asia/Dushanbe
    {  488, 8636,  160509,  9, }, // Asia/Famagusta
    {  488, 8651,  160518,  5, }, // Asia/Gaza
    {  488, 4795,  160523,  6, }, // Asia/Hebron
    {  488, 1259,  160529, 16, }, // Asia/Ho_Chi_Minh
    {  488, 2704,  160545,  9, }, // Asia/Hong_Kong
    {  488, 4771,  160554,  4, }, // Asia/Hovd
    {  488, 4184,  160558,  7, }, // Asia/Irkutsk
    {  488, 7006,  160565,  6, }, // Asia/Jakarta
    {  488, 6992,  160571,  7, }, // Asia/Jayapura
    {  488, 1290,  160578,  7, }, // Asia/Jerusalem
    {  488, 3560,  160585,  5, }, // Asia/Kabul
    {  488, 4316,  160590,  6, }, // Asia/Kamchatka
    {  488, 4239,  160596,  5, }, // Asia/Karachi
    {  488, 1185,  160601,  8, }, // Asia/Kathmandu
    {  488, 8661,  160609,  7, }, // Asia/Khandyga
    {  488,  992,  160616,  7, }, // Asia/Kolkata
    {  488, 4197,  160623,  9, }, // Asia/Krasnoyarsk
    {  488, 4859,  160632, 10, }, // Asia/Kuala_Lumpur
    {  488, 5976,  160642,  6, }, // Asia/Kuching
    {  488, 5547,  160648,  5, }, // Asia/Kuwait
    {  488, 1211,  160653,  4, }, // Asia/Macau
    {  488, 4017,  160657,  6, }, // Asia/Magadan
    {  488, 1349,  160663,  5, }, // Asia/Makassar
    {  488, 7182,  160668,  5, }, // Asia/Manila
    {  488, 6920,  160673,  4, }, // Asia/Muscat
    {  488, 2554,  160677,  7, }, // Asia/Nicosia
    {  488, 8675,  160684, 13, }, // Asia/Novokuznetsk
    {  488, 4135,  160697, 11, }, // Asia/Novosibirsk
    {  488, 4229,  160708,  4, }, // Asia/Omsk
    {  488, 7145,  160712,  3, }, // Asia/Oral
    {  488, 6961,  160715,  9, }, // Asia/Phnom_Penh
    {  488, 8693,  160724,  9, }, // Asia/Pontianak
    {  488, 4214,  160733,  9, }, // Asia/Pyongyang
    {  488, 5559,  160742,  3, }, // Asia/Qatar
    {  488, 8708,  160745,  8, }, // Asia/Qostanay
    {  488, 4269,  160753, 10, }, // Asia/Qyzylorda
    {  488, 3584,  160763,  4, }, // Asia/Riyadh
    {  488, 4408,  160767,  5, }, // Asia/Sakhalin
    {  488, 7245,  160772,  6, }, // Asia/Samarkand
    {  488, 3279,  160778,  4, }, // Asia/Seoul
    {  488, 1053,  160782,  6, }, // Asia/Shanghai
    {  488, 3300,  160788,  8, }, // Asia/Singapore
    {  488, 4297,  160796, 12, }, // Asia/Srednekolymsk
    {  488, 3263,  160808,  4, }, // Asia/Taipei
    {  488, 4781,  160812,  7, }, // Asia/Tashkent
    {  488, 3915,  160819,  7, }, // Asia/Tbilisi
    {  488, 2772,  160826,  6, }, // Asia/Tehran
    {  488, 1317,  160832,  5, }, // Asia/Thimphu
    {  488, 2821,  160837,  5, }, // Asia/Tokyo
    {  488, 4572,  160842,  5, }, // Asia/Tomsk
    {  488, 1021,  160847,  8, }, // Asia/Ulaanbaatar
    {  488, 1159,  160855,  6, }, // Asia/Urumqi
    {  488, 8722,  160861,  8, }, // Asia/Ust-Nera
    {  488, 6977,  160869,  9, }, // Asia/Vientiane
    {  488, 4710,  160878, 10, }, // Asia/Vladivostok
    {  488, 4828,  160888,  6, }, // Asia/Yakutsk
    {  488, 1235,  160894,  5, }, // Asia/Yangon
    {  488, 3883,  160899, 10, }, // Asia/Yekaterinburg
    {  488, 3747,  160909,  6, }, // Asia/Yerevan
    {  488, 3663,  160915,  6, }, // Atlantic/Azores
    {  488, 5671,  160921,  6, }, // Atlantic/Bermuda
    {  488, 6628,  160927,  6, }, // Atlantic/Canary
    {  488, 3727,  160933,  7, }, // Atlantic/Cape_Verde
    {  488, 1395,  160940,  4, }, // Atlantic/Faroe
    {  488, 8736,  160944,  6, }, // Atlantic/Madeira
    {  488, 2748,  160950,  8, }, // Atlantic/Reykjavik
    {  488, 7272,  160958, 12, }, // Atlantic/South_Georgia
    {  488, 6821,  160970, 11, }, // Atlantic/St_Helena
    {  488, 6644,  160981,  6, }, // Atlantic/Stanley
    {  488, 1670,  160987,  6, }, // Australia/Adelaide
    {  488, 1635,  160993,  8, }, // Australia/Brisbane
    {  488, 1799,  161001, 10, }, // Australia/Broken_Hill
    {  488, 1583,  161011,  6, }, // Australia/Darwin
    {  488, 3637,  161017,  5, }, // Australia/Eucla
    {  488, 1516,  161022,  6, }, // Australia/Hobart
    {  488, 8753,  161028,  7, }, // Australia/Lindeman
    {  488, 1547,  161035,  9, }, // Australia/Lord_Howe
    {  488, 1727,  161044,  7, }, // Australia/Melbourne
    {  488, 1762,  161051,  3, }, // Australia/Perth
    {  488, 1463,  161054,  5, }, // Australia/Sydney
    {  488, 6441,  161059,  7, }, // Europe/Amsterdam
    {  488, 6216,  161066,  6, }, // Europe/Andorra
    {  488, 3620,  161072,  9, }, // Europe/Astrakhan
    {  488, 2300,  161081,  4, }, // Europe/Athens
    {  488, 6336,  161085,  8, }, // Europe/Belgrade
    {  488, 4757,  161093,  5, }, // Europe/Berlin
    {  488, 6488,  161098, 11, }, // Europe/Bratislava
    {  488, 2167,  161109,  6, }, // Europe/Brussels
    {  488, 3928,  161115,  8, }, // Europe/Bucharest
    {  488, 3806,  161123,  8, }, // Europe/Budapest
    {  488, 8772,  161131,  8, }, // Europe/Busingen
    {  488, 2583,  161139,  6, }, // Europe/Chisinau
    {  488, 6289,  161145,  8, }, // Europe/Copenhagen
    {  488, 2338,  161153,  5, }, // Europe/Dublin
    {  488, 6307,  161158,  8, }, // Europe/Gibraltar
    {  488, 8788,  161166,  5, }, // Europe/Guernsey
    {  488, 6612,  161171,  8, }, // Europe/Helsinki
    {  488, 8804,  161179, 10, }, // Europe/Isle_of_Man
    {  488, 1130,  161189,  8, }, // Europe/Istanbul
    {  488, 8823,  161197,  4, }, // Europe/Jersey
    {  488, 3979,  161201, 11, }, // Europe/Kaliningrad
    {  488, 8929,  161212,  5, }, // Europe/Kirov
    {  488, 2527,  161217,  3, }, // Europe/Kyiv
    {  488, 3213,  161220,  5, }, // Europe/Lisbon
    {  488, 6506,  161225,  9, }, // Europe/Ljubljana
    {  488, 2501,  161234,  4, }, // Europe/London
    {  488, 6365,  161238,  8, }, // Europe/Luxembourg
    {  488, 4892,  161246,  7, }, // Europe/Madrid
    {  488, 6397,  161253,  5, }, // Europe/Malta
    {  488, 6582,  161258, 10, }, // Europe/Mariehamn
    {  488, 3693,  161268,  5, }, // Europe/Minsk
    {  488, 6410,  161273,  6, }, // Europe/Monaco
    {  488, 3537,  161279,  5, }, // Europe/Moscow
    {  488, 6458,  161284,  4, }, // Europe/Oslo
    {  488, 4284,  161288,  5, }, // Europe/Paris
    {  488, 6424,  161293,  9, }, // Europe/Podgorica
    {  488, 6275,  161302,  5, }, // Europe/Prague
    {  488, 8837,  161307,  4, }, // Europe/Riga
    {  488, 6324,  160942,  3, }, // Europe/Rome
    {  488, 4331,  158751,  5, }, // Europe/Samara
    {  488, 6470,  161311,  9, }, // Europe/San_Marino
    {  488, 6245,  161320,  8, }, // Europe/Sarajevo
    {  488, 4451,  161328,  7, }, // Europe/Saratov
    {  488, 8849,  161335,  9, }, // Europe/Simferopol
    {  488, 6383,  161344,  6, }, // Europe/Skopje
    {  488, 6599,  161350,  5, }, // Europe/Sofia
    {  488, 6523,  161355,  7, }, // Europe/Stockholm
    {  488, 8867,  161362,  5, }, // Europe/Tallinn
    {  488, 6202,  161367,  6, }, // Europe/Tirane
    {  488, 8882,  161373,  9, }, // Europe/Ulyanovsk
    {  488, 6352,  161382,  6, }, // Europe/Vaduz
    {  488, 6567,  161388,  6, }, // Europe/Vatican
    {  488, 6231,  161394,  5, }, // Europe/Vienna
    {  488, 8899,  161399,  7, }, // Europe/Vilnius
    {  488, 4727,  161406, 10, }, // Europe/Volgograd
    {  488, 3190,  161416,  6, }, // Europe/Warsaw
    {  488, 6261,  161422,  6, }, // Europe/Zagreb
    {  488, 6540,  161428,  6, }, // Europe/Zurich
    {  488, 5078,  161434, 11, }, // Indian/Antananarivo
    {  488, 6947,  161445,  5, }, // Indian/Chagos
    {  488, 6046,  161450,  7, }, // Indian/Christmas
    {  488, 6063,  161457,  6, }, // Indian/Cocos
    {  488, 5029,  161463,  6, }, // Indian/Comoro
    {  488, 6661,  161469,  8, }, // Indian/Kerguelen
    {  488, 7260,  161477,  4, }, // Indian/Mahe
    {  488, 7050,  161481,  6, }, // Indian/Maldives
    {  488, 4069,  161487,  7, }, // Indian/Mauritius
    {  488, 5098,  161494,  6, }, // Indian/Mayotte
    {  488, 7211,  161500,  8, }, // Indian/Reunion
    {  488, 4422,  161508,  4, }, // Pacific/Apia
    {  488,  932,  161512,  6, }, // Pacific/Auckland
    {  488, 3706,  161518,  8, }, // Pacific/Bougainville
    {  488, 3015,  161526,  4, }, // Pacific/Chatham
    {  488, 3157,  161530,  3, }, // Pacific/Chuuk
    {  488, 2237,  161533,  4, }, // Pacific/Easter
    {  488, 7360,  161537,  4, }, // Pacific/Efate
    {  488, 7327,  161541,  6, }, // Pacific/Fakaofo
    {  488, 3902,  161547,  5, }, // Pacific/Fiji
    {  488, 7343,  161552,  8, }, // Pacific/Funafuti
    {  488, 6678,  161560,  7, }, // Pacific/Galapagos
    {  488, 6696,  161567,  7, }, // Pacific/Gambier
    {  488, 3822,  161574,  9, }, // Pacific/Guadalcanal
    {  488, 6018,  159796,  4, }, // Pacific/Guam
    {  488, 2723,  161583,  8, }, // Pacific/Honolulu
    {  488, 3049,  161591,  7, }, // Pacific/Kanton
    {  488, 3998,  161598,  9, }, // Pacific/Kiritimati
    {  488, 7019,  161607,  7, }, // Pacific/Kosrae
    {  488, 2842,  161614,  8, }, // Pacific/Kwajalein
    {  488, 4877,  161622,  6, }, // Pacific/Majuro
    {  488, 4051,  161628,  8, }, // Pacific/Marquesas
    {  488, 8914,  161636,  5, }, // Pacific/Midway
    {  488, 7084,  161641,  5, }, // Pacific/Nauru
    {  488, 7132,  161646,  4, }, // Pacific/Niue
    {  488, 4168,  161650,  6, }, // Pacific/Norfolk
    {  488, 7098,  161656,  5, }, // Pacific/Noumea
    {  488, 3126,  161661,  9, }, // Pacific/Pago_Pago
    {  488, 7155,  161670,  4, }, // Pacific/Palau
    {  488, 7194,  161674,  7, }, // Pacific/Pitcairn
    {  488, 3096,  161681,  7, }, // Pacific/Pohnpei
    {  488, 4807,  161688, 12, }, // Pacific/Port_Moresby
    {  488, 6076,  161700,  9, }, // Pacific/Rarotonga
    {  488, 6031,  161709,  5, }, // Pacific/Saipan
    {  488, 7312,  161714,  5, }, // Pacific/Tahiti
    {  488, 6712,  161719,  6, }, // Pacific/Tarawa
    {  488, 4583,  161725,  9, }, // Pacific/Tongatapu
    {  488, 7392,  161734,  3, }, // Pacific/Wake
    {  488, 7405,  161737,  5, }, // Pacific/Wallis
    {  489,   14,       0,  6, }, // Africa/Asmara Punjabi/Arabic/Pakistan
    {  489, 4435,       6,  8, }, // Africa/Sao_Tome
    {  489, 4252,      14,  8, }, // America/Asuncion
    {  489,  237,      22,  8, }, // America/Atikokan
    {  489, 7623,      30, 17, }, // America/Bahia_Banderas
    {  489, 3868,      47,  6, }, // America/Cancun
    {  489, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  489, 5723,      66,  7, }, // America/Curacao
    {  489,  481,      73, 13, }, // America/Indiana/Knox
    {  489, 7961,      86, 16, }, // America/Indiana/Marengo
    {  489, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  489, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  489, 8038,     139, 14, }, // America/Indiana/Vevay
    {  489, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  489, 8086,     171, 16, }, // America/Indiana/Winamac
    {  489, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  489, 5932,     207, 22, }, // America/Lower_Princes
    {  489, 8220,     229,  6, }, // America/Merida
    {  489, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  489, 8329,     255, 20, }, // America/North_Dakota/Center
    {  489, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  489,  393,     298,  4, }, // America/Nuuk
    {  489, 6899,     302, 16, }, // America/Scoresbysund
    {  489, 8508,     318, 14, }, // America/St_Barthelemy
    {  489, 2061,     332, 10, }, // America/St_Johns
    {  489, 5863,     342,  9, }, // America/St_Kitts
    {  489, 5880,     351,  9, }, // America/St_Lucia
    {  489,  855,     360, 10, }, // America/St_Thomas
    {  489, 5913,     370, 11, }, // America/St_Vincent
    {  489, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  489, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  489, 1185,     413,  9, }, // Asia/Kathmandu
    {  489,  992,     422,  7, }, // Asia/Kolkata
    {  489, 1211,     429,  5, }, // Asia/Macau
    {  489, 1235,     442,  6, }, // Asia/Yangon
    {  489, 1395,     448,  5, }, // Atlantic/Faroe
    {  489, 6821,     453, 10, }, // Atlantic/St_Helena
    {  489, 2527,     463,  4, }, // Europe/Kyiv
    {  489, 7211,     467,  7, }, // Indian/Reunion
    {  489, 3157,     474,  5, }, // Pacific/Chuuk
    {  489, 3096,     479,  7, }, // Pacific/Pohnpei
    {  490, 6788,  161742,  6, }, // Africa/Abidjan Quechua/Latin/Peru
    {  490, 6760,   74000,  4, }, // Africa/Accra
    {  490, 5059,   20252, 10, }, // Africa/Addis_Ababa
    {  490, 7420,  147563,  5, }, // Africa/Algiers
    {  490,   14,       0,  6, }, // Africa/Asmara
    {  490, 2320,  161748,  8, }, // Africa/Cairo
    {  490, 5043,  147577,  6, }, // Africa/Djibouti
    {  490, 5243,   21254,  5, }, // Africa/Douala
    {  490, 4479,  161756, 13, }, // Africa/Johannesburg
    {  490, 4524,  147583,  6, }, // Africa/Khartoum
    {  490, 5113,   43978, 10, }, // Africa/Mogadishu
    {  490, 4435,       6,  8, }, // Africa/Sao_Tome
    {  490, 6554,  161769,  5, }, // Africa/Tunis
    {  490, 5607,    4657,  7, }, // America/Anguilla
    {  490, 4252,      14,  8, }, // America/Asuncion
    {  490,  237,      22,  8, }, // America/Atikokan
    {  490, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  490, 5407,  147623,  6, }, // America/Belize
    {  490, 3868,   33182,  6, }, // America/Cancun
    {  490, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  490, 5723,  161774,  7, }, // America/Curacao
    {  490, 5770,   20401,  7, }, // America/Grenada
    {  490, 5786,   20408,  9, }, // America/Guadeloupe
    {  490, 2281,   20417,  9, }, // America/Havana
    {  490,  481,      73, 13, }, // America/Indiana/Knox
    {  490, 7961,      86, 16, }, // America/Indiana/Marengo
    {  490, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  490, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  490, 8038,     139, 14, }, // America/Indiana/Vevay
    {  490, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  490, 8086,     171, 16, }, // America/Indiana/Winamac
    {  490, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  490, 5932,     207, 22, }, // America/Lower_Princes
    {  490, 5805,   20436,  9, }, // America/Martinique
    {  490, 8220,    7008,  6, }, // America/Merida
    {  490, 2949,  161781, 16, }, // America/Mexico_City
    {  490, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  490, 8329,     255, 20, }, // America/North_Dakota/Center
    {  490, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  490,  393,     298,  4, }, // America/Nuuk
    {  490, 3945,  147772, 15, }, // America/Port-au-Prince
    {  490, 5954,  161797, 13, }, // America/Port_of_Spain
    {  490, 6899,     302, 16, }, // America/Scoresbysund
    {  490, 8508,  161810, 13, }, // America/St_Barthelemy
    {  490, 2061,  161823, 21, }, // America/St_Johns
    {  490, 5863,  161844, 13, }, // America/St_Kitts
    {  490, 5880,   95111, 11, }, // America/St_Lucia
    {  490,  855,  161857, 11, }, // America/St_Thomas
    {  490, 5913,   74276, 11, }, // America/St_Vincent
    {  490, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  490, 3968,   20612,  4, }, // Asia/Amman
    {  490, 5523,    5491,  5, }, // Asia/Aqtau
    {  490, 5368,    7488,  6, }, // Asia/Aqtobe
    {  490,  964,  147884,  7, }, // Asia/Ashgabat
    {  490, 3607,    4044,  6, }, // Asia/Baghdad
    {  490, 5534,  147891,  6, }, // Asia/Bahrain
    {  490, 4540,   74309,  7, }, // Asia/Damascus
    {  490, 6159,  147904,  7, }, // Asia/Dushanbe
    {  490, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  490, 7006,  161868,  7, }, // Asia/Jakarta
    {  490, 1185,   20712,  8, }, // Asia/Kathmandu
    {  490,  992,     422,  7, }, // Asia/Kolkata
    {  490, 1211,     429,  5, }, // Asia/Macau
    {  490, 1349,   94257,  7, }, // Asia/Makassar
    {  490, 6920,   20739,  7, }, // Asia/Muscat
    {  490, 4214,  161875,  8, }, // Asia/Pyongyang
    {  490, 5559,   73021,  5, }, // Asia/Qatar
    {  490, 8708,  161883,  8, }, // Asia/Qostanay
    {  490, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  490, 3584,    7669,  4, }, // Asia/Riyadh
    {  490, 7245,   20768, 10, }, // Asia/Samarkand
    {  490, 3279,   20778,  4, }, // Asia/Seoul
    {  490, 1053,  161891,  7, }, // Asia/Shanghai
    {  490, 3300,   20782,  8, }, // Asia/Singapore
    {  490, 4781,   91626,  7, }, // Asia/Tashkent
    {  490, 3915,   78108,  6, }, // Asia/Tbilisi
    {  490, 2772,   57993,  7, }, // Asia/Tehran
    {  490, 2821,    4238,  5, }, // Asia/Tokyo
    {  490, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  490, 1235,   20819,  6, }, // Asia/Yangon
    {  490, 3883,   74328, 13, }, // Asia/Yekaterinburg
    {  490, 3747,  147956,  6, }, // Asia/Yerevan
    {  490, 6628,   74368,  8, }, // Atlantic/Canary
    {  490, 3727,   26836, 10, }, // Atlantic/Cape_Verde
    {  490, 1395,     448,  5, }, // Atlantic/Faroe
    {  490, 7272,  161898, 15, }, // Atlantic/South_Georgia
    {  490, 6821,  147989, 11, }, // Atlantic/St_Helena
    {  490, 1670,   20912,  8, }, // Australia/Adelaide
    {  490, 1463,   22607,  6, }, // Australia/Sydney
    {  490, 2167,  161913,  8, }, // Europe/Brussels
    {  490, 3928,   20955,  8, }, // Europe/Bucharest
    {  490, 6289,   20963, 10, }, // Europe/Copenhagen
    {  490, 8804,  148006, 11, }, // Europe/Isle_of_Man
    {  490, 1130,  148017,  8, }, // Europe/Istanbul
    {  490, 3979,   62893, 12, }, // Europe/Kaliningrad
    {  490, 2527,    8030,  4, }, // Europe/Kyiv
    {  490, 3213,   21019,  6, }, // Europe/Lisbon
    {  490, 6506,   21025,  9, }, // Europe/Ljubljana
    {  490, 2501,   21034,  7, }, // Europe/London
    {  490, 6365,   74445, 10, }, // Europe/Luxembourg
    {  490, 3537,   21057,  5, }, // Europe/Moscow
    {  490, 6275,   21067,  5, }, // Europe/Prague
    {  490, 6324,    5946,  4, }, // Europe/Rome
    {  490, 6523,   74486,  9, }, // Europe/Stockholm
    {  490, 6202,   21095,  6, }, // Europe/Tirane
    {  490, 6567,  161921, 11, }, // Europe/Vatican
    {  490, 6231,   21112,  5, }, // Europe/Vienna
    {  490, 4727,   63074, 10, }, // Europe/Volgograd
    {  490, 3190,   21126,  8, }, // Europe/Warsaw
    {  490, 7050,   74517,  8, }, // Indian/Maldives
    {  490, 7211,     467,  7, }, // Indian/Reunion
    {  490, 3157,     474,  5, }, // Pacific/Chuuk
    {  490, 2723,    4493,  8, }, // Pacific/Honolulu
    {  490, 3049,    4501,  9, }, // Pacific/Kanton
    {  490, 3096,     479,  7, }, // Pacific/Pohnpei
    {  493, 5059,    3784, 11, }, // Africa/Addis_Ababa Romanian/Latin/Romania
    {  493, 7420,   43956,  5, }, // Africa/Algiers
    {  493,   14,       0,  6, }, // Africa/Asmara
    {  493, 5113,   43978, 10, }, // Africa/Mogadishu
    {  493, 5271,   43996,  9, }, // Africa/Ndjamena
    {  493, 4435,   58598,  8, }, // Africa/Sao_Tome
    {  493, 4252,      14,  8, }, // America/Asuncion
    {  493,  237,      22,  8, }, // America/Atikokan
    {  493, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  493, 3868,   33182,  6, }, // America/Cancun
    {  493, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  493, 5723,      66,  7, }, // America/Curacao
    {  493, 5786,   93511,  9, }, // America/Guadeloupe
    {  493,  481,      73, 13, }, // America/Indiana/Knox
    {  493, 7961,      86, 16, }, // America/Indiana/Marengo
    {  493, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  493, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  493, 8038,     139, 14, }, // America/Indiana/Vevay
    {  493, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  493, 8086,     171, 16, }, // America/Indiana/Winamac
    {  493, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  493, 5932,     207, 22, }, // America/Lower_Princes
    {  493, 5805,   20436,  9, }, // America/Martinique
    {  493, 8220,    7008,  6, }, // America/Merida
    {  493, 2949,  161781, 16, }, // America/Mexico_City
    {  493, 8301,  161932, 22, }, // America/North_Dakota/Beulah
    {  493, 8329,  161954, 22, }, // America/North_Dakota/Center
    {  493, 8357,  161976, 25, }, // America/North_Dakota/New_Salem
    {  493,  393,     298,  4, }, // America/Nuuk
    {  493, 6899,     302, 16, }, // America/Scoresbysund
    {  493, 8508,   35804, 16, }, // America/St_Barthelemy
    {  493, 2061,     332, 10, }, // America/St_Johns
    {  493, 5863,     342,  9, }, // America/St_Kitts
    {  493, 5880,     351,  9, }, // America/St_Lucia
    {  493,  855,     360, 10, }, // America/St_Thomas
    {  493, 5913,     370, 11, }, // America/St_Vincent
    {  493, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  493, 7295,   62088,  5, }, // Antarctica/Syowa
    {  493, 5395,  117400,  6, }, // Asia/Almaty
    {  493, 5511,    7482,  6, }, // Asia/Anadyr
    {  493, 5523,    5491,  5, }, // Asia/Aqtau
    {  493, 5368,    7488,  6, }, // Asia/Aqtobe
    {  493,  964,  162001,  7, }, // Asia/Ashgabat
    {  493, 8624,  162008,  6, }, // Asia/Atyrau
    {  493, 3607,    4044,  6, }, // Asia/Baghdad
    {  493, 3778,  162014,  6, }, // Asia/Bishkek
    {  493, 4601,   93125,  4, }, // Asia/Chita
    {  493, 4540,   20639,  6, }, // Asia/Damascus
    {  493, 1093,   65588,  5, }, // Asia/Dhaka
    {  493, 6159,  162020,  7, }, // Asia/Dushanbe
    {  493, 1259,  162027,  9, }, // Asia/Ho_Chi_Minh
    {  493, 4184,  162036,  6, }, // Asia/Irkutsk
    {  493, 1290,  162042,  9, }, // Asia/Jerusalem
    {  493, 4316,  162051,  9, }, // Asia/Kamchatka
    {  493, 1185,     413,  9, }, // Asia/Kathmandu
    {  493,  992,   35921,  8, }, // Asia/Kolkata
    {  493, 4197,   26739, 11, }, // Asia/Krasnoyarsk
    {  493, 5547,  162060,  6, }, // Asia/Kuwait
    {  493, 1211,     429,  5, }, // Asia/Macau
    {  493, 8675,  162066, 11, }, // Asia/Novokuznetsk
    {  493, 7145,   57921,  6, }, // Asia/Oral
    {  493, 4214,  162077,  7, }, // Asia/Pyongyang
    {  493, 8708,     434,  8, }, // Asia/Qostanay
    {  493, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  493, 3584,    7669,  4, }, // Asia/Riyadh
    {  493, 4408,   33444,  7, }, // Asia/Sakhalin
    {  493, 4297,    7699, 13, }, // Asia/Srednekolymsk
    {  493, 4781,  162084,  7, }, // Asia/Tashkent
    {  493, 2772,    7724,  7, }, // Asia/Tehran
    {  493, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  493, 4828,  162091,  6, }, // Asia/Yakutsk
    {  493, 1235,     442,  6, }, // Asia/Yangon
    {  493, 3883,    7768, 12, }, // Asia/Yekaterinburg
    {  493, 3747,   26815,  6, }, // Asia/Yerevan
    {  493, 3663,    7787,  5, }, // Atlantic/Azores
    {  493, 6628,  162097,  6, }, // Atlantic/Canary
    {  493, 3727,  162103, 11, }, // Atlantic/Cape_Verde
    {  493, 1395,   20874,  5, }, // Atlantic/Faroe
    {  493, 7272,  162114, 14, }, // Atlantic/South_Georgia
    {  493, 6821,  162128,  9, }, // Atlantic/St_Helena
    {  493, 3620,   33512,  8, }, // Europe/Astrakhan
    {  493, 2300,   26877,  5, }, // Europe/Athens
    {  493, 6336,   26883,  7, }, // Europe/Belgrade
    {  493, 2167,   57529,  9, }, // Europe/Brussels
    {  493, 3928,  147230,  9, }, // Europe/Bucharest
    {  493, 3806,  162137,  9, }, // Europe/Budapest
    {  493, 2583,   63672,  8, }, // Europe/Chisinau
    {  493, 6289,  158592,  9, }, // Europe/Copenhagen
    {  493, 8804,  162146, 10, }, // Europe/Isle_of_Man
    {  493, 2527,    8030,  4, }, // Europe/Kyiv
    {  493, 3213,  124078,  8, }, // Europe/Lisbon
    {  493, 2501,   95241,  6, }, // Europe/London
    {  493, 6365,    4395,  9, }, // Europe/Luxembourg
    {  493, 3537,   74461,  7, }, // Europe/Moscow
    {  493, 6424,  162156,  9, }, // Europe/Podgorica
    {  493, 6275,   21067,  5, }, // Europe/Prague
    {  493, 6324,    5946,  4, }, // Europe/Rome
    {  493, 6202,   21095,  6, }, // Europe/Tirane
    {  493, 8882,   26960,  9, }, // Europe/Ulyanovsk
    {  493, 6231,   21112,  5, }, // Europe/Vienna
    {  493, 3190,  162165,  8, }, // Europe/Warsaw
    {  493, 6540,    4459,  6, }, // Europe/Zurich
    {  493, 5029,    4470,  6, }, // Indian/Comoro
    {  493, 7050,    8193,  7, }, // Indian/Maldives
    {  493, 7211,     467,  7, }, // Indian/Reunion
    {  493, 3157,     474,  5, }, // Pacific/Chuuk
    {  493, 2237,  162173, 15, }, // Pacific/Easter
    {  493, 2723,    4493,  8, }, // Pacific/Honolulu
    {  493, 3049,    4501,  9, }, // Pacific/Kanton
    {  493, 4051,  162188,  8, }, // Pacific/Marquesas
    {  493, 7194,  162196, 15, }, // Pacific/Pitcairn
    {  493, 3096,     479,  7, }, // Pacific/Pohnpei
    {  495, 5059,    3784, 11, }, // Africa/Addis_Ababa Romansh/Latin/Switzerland
    {  495, 7420,   58423,  6, }, // Africa/Algiers
    {  495,   14,       0,  6, }, // Africa/Asmara
    {  495, 5130,   77951, 10, }, // Africa/Dar_es_Salaam
    {  495, 5043,   77961,  9, }, // Africa/Djibouti
    {  495, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  495, 4524,   26372,  7, }, // Africa/Khartoum
    {  495, 6869,   20300,  4, }, // Africa/Lome
    {  495, 5113,   77970, 10, }, // Africa/Mogadishu
    {  495, 4435,       6,  8, }, // Africa/Sao_Tome
    {  495, 3346,  162211,  6, }, // America/Anchorage
    {  495, 5607,  162217, 10, }, // America/Anguilla
    {  495, 4554,   26427,  9, }, // America/Araguaina
    {  495,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  495,  441,  162227, 21, }, // America/Argentina/Jujuy
    {  495, 7544,  162248, 20, }, // America/Argentina/San_Juan
    {  495, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  495, 7597,   64551,  7, }, // America/Argentina/Ushuaia
    {  495, 5640,  162268, 10, }, // America/Aruba
    {  495, 4252,      14,  8, }, // America/Asuncion
    {  495,  237,      22,  8, }, // America/Atikokan
    {  495, 7623,      30, 17, }, // America/Bahia_Banderas
    {  495, 7646,   35764,  5, }, // America/Belem
    {  495, 5407,  162278,  8, }, // America/Belize
    {  495, 3868,      47,  6, }, // America/Cancun
    {  495, 5496,  162286, 13, }, // America/Cayman
    {  495, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  495, 3791,   26442,  6, }, // America/Cuiaba
    {  495, 5723,      66,  7, }, // America/Curacao
    {  495, 7869,   26448,  8, }, // America/Eirunepe
    {  495, 5441,    6688,  8, }, // America/El_Salvador
    {  495, 4612,  162299, 13, }, // America/Grand_Turk
    {  495, 5786,  162312, 11, }, // America/Guadeloupe
    {  495,  481,      73,  4, }, // America/Indiana/Knox
    {  495, 7961,      86,  7, }, // America/Indiana/Marengo
    {  495, 7985,     102, 10, }, // America/Indiana/Petersburg
    {  495, 8012,     121,  9, }, // America/Indiana/Tell_City
    {  495, 8038,     139,  5, }, // America/Indiana/Vevay
    {  495, 8060,     153,  9, }, // America/Indiana/Vincennes
    {  495, 8086,     171,  7, }, // America/Indiana/Winamac
    {  495, 2799,   94996,  8, }, // America/Jamaica
    {  495, 8125,     187, 10, }, // America/Kentucky/Monticello
    {  495, 5932,     207, 22, }, // America/Lower_Princes
    {  495, 2917,   26471,  8, }, // America/Mazatlan
    {  495, 8220,     229,  6, }, // America/Merida
    {  495, 2949,  162323, 15, }, // America/Mexico_City
    {  495, 4391,  162338, 12, }, // America/Miquelon
    {  495, 5824,  162350,  6, }, // America/Montserrat
    {  495, 1850,   61820, 19, }, // America/Noronha
    {  495, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  495, 8329,  162356, 22, }, // America/North_Dakota/Center
    {  495, 8357,  162378, 24, }, // America/North_Dakota/New_Salem
    {  495,  393,   35796,  7, }, // America/Nuuk
    {  495, 1878,   26580,  9, }, // America/Sao_Paulo
    {  495, 6899,     302, 16, }, // America/Scoresbysund
    {  495, 8508,     318, 14, }, // America/St_Barthelemy
    {  495, 2061,   26605, 12, }, // America/St_Johns
    {  495, 5863,   26617, 11, }, // America/St_Kitts
    {  495, 5880,   95111, 11, }, // America/St_Lucia
    {  495,  855,   44161, 12, }, // America/St_Thomas
    {  495, 5913,   26651, 13, }, // America/St_Vincent
    {  495, 5688,  162402,  9, }, // America/Tortola
    {  495, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  495, 7113,  162411,  9, }, // Antarctica/McMurdo
    {  495, 7295,   62088,  5, }, // Antarctica/Syowa
    {  495, 5368,   57073,  6, }, // Asia/Aqtobe
    {  495,  964,   44190,  7, }, // Asia/Ashgabat
    {  495, 3607,    4044,  6, }, // Asia/Baghdad
    {  495, 3778,   57088,  8, }, // Asia/Bishkek
    {  495, 5989,  162420, 19, }, // Asia/Brunei
    {  495, 6159,   57096,  9, }, // Asia/Dushanbe
    {  495, 1259,  162439, 20, }, // Asia/Ho_Chi_Minh
    {  495, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  495, 4316,   78050, 11, }, // Asia/Kamchatka
    {  495, 4239,   78061,  9, }, // Asia/Karachi
    {  495, 1185,     413,  9, }, // Asia/Kathmandu
    {  495,  992,     422,  7, }, // Asia/Kolkata
    {  495, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  495, 1211,     429,  5, }, // Asia/Macau
    {  495, 1349,   65105,  8, }, // Asia/Makassar
    {  495, 6920,   20739,  7, }, // Asia/Muscat
    {  495, 4135,   78082, 11, }, // Asia/Novosibirsk
    {  495, 3584,   20756,  5, }, // Asia/Riyadh
    {  495, 4408,   57952,  8, }, // Asia/Sakhalin
    {  495, 7245,   20768, 10, }, // Asia/Samarkand
    {  495, 3279,    7680,  4, }, // Asia/Seoul
    {  495, 3300,   20782,  8, }, // Asia/Singapore
    {  495, 4781,   57196,  9, }, // Asia/Tashkent
    {  495, 2772,    7724,  7, }, // Asia/Tehran
    {  495, 2821,    4238,  5, }, // Asia/Tokyo
    {  495, 1159,   26801,  6, }, // Asia/Urumqi
    {  495, 4828,    7755,  7, }, // Asia/Yakutsk
    {  495, 1235,     442,  6, }, // Asia/Yangon
    {  495, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  495, 3747,    7780,  7, }, // Asia/Yerevan
    {  495, 3663,  162459,  6, }, // Atlantic/Azores
    {  495, 5671,   74354,  8, }, // Atlantic/Bermuda
    {  495, 6628,  162465, 15, }, // Atlantic/Canary
    {  495, 3727,   44357,  8, }, // Atlantic/Cape_Verde
    {  495, 1395,  162480, 12, }, // Atlantic/Faroe
    {  495, 7272,  162492, 15, }, // Atlantic/South_Georgia
    {  495, 6821,  162507, 11, }, // Atlantic/St_Helena
    {  495, 6644,  162518, 12, }, // Atlantic/Stanley
    {  495, 2300,    4320,  5, }, // Europe/Athens
    {  495, 2167,   57529,  9, }, // Europe/Brussels
    {  495, 3928,   20955,  8, }, // Europe/Bucharest
    {  495, 8788,  162530, 16, }, // Europe/Guernsey
    {  495, 8804,  162546,  7, }, // Europe/Isle_of_Man
    {  495, 8823,  162553, 12, }, // Europe/Jersey
    {  495, 2527,     463,  4, }, // Europe/Kyiv
    {  495, 3213,    4381,  8, }, // Europe/Lisbon
    {  495, 2501,   95241,  6, }, // Europe/London
    {  495, 6365,    4395,  9, }, // Europe/Luxembourg
    {  495, 3537,  162565,  6, }, // Europe/Moscow
    {  495, 6324,    5946,  4, }, // Europe/Rome
    {  495, 6202,   21095,  6, }, // Europe/Tirane
    {  495, 3190,   21126,  8, }, // Europe/Warsaw
    {  495, 6540,  162571,  6, }, // Europe/Zurich
    {  495, 6046,  162577, 16, }, // Indian/Christmas
    {  495, 6063,  162593, 11, }, // Indian/Cocos
    {  495, 5029,  162604,  7, }, // Indian/Comoro
    {  495, 7050,   74517,  8, }, // Indian/Maldives
    {  495, 7211,     467,  7, }, // Indian/Reunion
    {  495, 3157,  162611,  4, }, // Pacific/Chuuk
    {  495, 2237,  162615, 14, }, // Pacific/Easter
    {  495, 3902,   78224,  7, }, // Pacific/Fiji
    {  495, 6678,   21159,  9, }, // Pacific/Galapagos
    {  495, 6696,  162629,  7, }, // Pacific/Gambier
    {  495, 3822,  162636,  7, }, // Pacific/Guadalcanal
    {  495, 6018,  162643,  7, }, // Pacific/Guam
    {  495, 7019,  162650,  5, }, // Pacific/Kosrae
    {  495, 7084,  162655,  5, }, // Pacific/Nauru
    {  495, 7132,  162660,  5, }, // Pacific/Niue
    {  495, 4168,  162665,  8, }, // Pacific/Norfolk
    {  495, 7098,    4510,  6, }, // Pacific/Noumea
    {  495, 7155,  162673,  7, }, // Pacific/Palau
    {  495, 3096,  162680,  7, }, // Pacific/Pohnpei
    {  495, 6712,  162687, 12, }, // Pacific/Tarawa
    {  495, 4583,  162699, 10, }, // Pacific/Tongatapu
    {  495, 7405,  162709,  8, }, // Pacific/Wallis
    {  496,   14,       0,  6, }, // Africa/Asmara Rombo/Latin/Tanzania
    {  496, 4435,       6,  8, }, // Africa/Sao_Tome
    {  496, 4252,      14,  8, }, // America/Asuncion
    {  496,  237,      22,  8, }, // America/Atikokan
    {  496, 7623,      30, 17, }, // America/Bahia_Banderas
    {  496, 3868,      47,  6, }, // America/Cancun
    {  496, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  496, 5723,      66,  7, }, // America/Curacao
    {  496,  481,      73, 13, }, // America/Indiana/Knox
    {  496, 7961,      86, 16, }, // America/Indiana/Marengo
    {  496, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  496, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  496, 8038,     139, 14, }, // America/Indiana/Vevay
    {  496, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  496, 8086,     171, 16, }, // America/Indiana/Winamac
    {  496, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  496, 5932,     207, 22, }, // America/Lower_Princes
    {  496, 8220,     229,  6, }, // America/Merida
    {  496, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  496, 8329,     255, 20, }, // America/North_Dakota/Center
    {  496, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  496,  393,     298,  4, }, // America/Nuuk
    {  496, 6899,     302, 16, }, // America/Scoresbysund
    {  496, 8508,     318, 14, }, // America/St_Barthelemy
    {  496, 2061,     332, 10, }, // America/St_Johns
    {  496, 5863,     342,  9, }, // America/St_Kitts
    {  496, 5880,     351,  9, }, // America/St_Lucia
    {  496,  855,     360, 10, }, // America/St_Thomas
    {  496, 5913,     370, 11, }, // America/St_Vincent
    {  496, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  496, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  496, 1185,     413,  9, }, // Asia/Kathmandu
    {  496,  992,     422,  7, }, // Asia/Kolkata
    {  496, 1211,     429,  5, }, // Asia/Macau
    {  496, 1235,     442,  6, }, // Asia/Yangon
    {  496, 1395,     448,  5, }, // Atlantic/Faroe
    {  496, 6821,     453, 10, }, // Atlantic/St_Helena
    {  496, 2527,     463,  4, }, // Europe/Kyiv
    {  496, 7211,     467,  7, }, // Indian/Reunion
    {  496, 3157,     474,  5, }, // Pacific/Chuuk
    {  496, 3096,     479,  7, }, // Pacific/Pohnpei
    {  497,   14,       0,  6, }, // Africa/Asmara Rundi/Latin/Burundi
    {  497, 4435,       6,  8, }, // Africa/Sao_Tome
    {  497, 4252,      14,  8, }, // America/Asuncion
    {  497,  237,      22,  8, }, // America/Atikokan
    {  497, 7623,      30, 17, }, // America/Bahia_Banderas
    {  497, 3868,      47,  6, }, // America/Cancun
    {  497, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  497, 5723,      66,  7, }, // America/Curacao
    {  497,  481,      73, 13, }, // America/Indiana/Knox
    {  497, 7961,      86, 16, }, // America/Indiana/Marengo
    {  497, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  497, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  497, 8038,     139, 14, }, // America/Indiana/Vevay
    {  497, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  497, 8086,     171, 16, }, // America/Indiana/Winamac
    {  497, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  497, 5932,     207, 22, }, // America/Lower_Princes
    {  497, 8220,     229,  6, }, // America/Merida
    {  497, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  497, 8329,     255, 20, }, // America/North_Dakota/Center
    {  497, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  497,  393,     298,  4, }, // America/Nuuk
    {  497, 6899,     302, 16, }, // America/Scoresbysund
    {  497, 8508,     318, 14, }, // America/St_Barthelemy
    {  497, 2061,     332, 10, }, // America/St_Johns
    {  497, 5863,     342,  9, }, // America/St_Kitts
    {  497, 5880,     351,  9, }, // America/St_Lucia
    {  497,  855,     360, 10, }, // America/St_Thomas
    {  497, 5913,     370, 11, }, // America/St_Vincent
    {  497, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  497, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  497, 1185,     413,  9, }, // Asia/Kathmandu
    {  497,  992,     422,  7, }, // Asia/Kolkata
    {  497, 1211,     429,  5, }, // Asia/Macau
    {  497, 1235,     442,  6, }, // Asia/Yangon
    {  497, 1395,     448,  5, }, // Atlantic/Faroe
    {  497, 6821,     453, 10, }, // Atlantic/St_Helena
    {  497, 2527,     463,  4, }, // Europe/Kyiv
    {  497, 7211,     467,  7, }, // Indian/Reunion
    {  497, 3157,     474,  5, }, // Pacific/Chuuk
    {  497, 3096,     479,  7, }, // Pacific/Pohnpei
    {  498, 6788,   36214,  7, }, // Africa/Abidjan Russian/Cyrillic/Russia
    {  498, 6760,     492,  5, }, // Africa/Accra
    {  498, 5059,     497, 11, }, // Africa/Addis_Ababa
    {  498, 7420,     508,  5, }, // Africa/Algiers
    {  498,   14,   56104,  6, }, // Africa/Asmara
    {  498,   44,     519,  6, }, // Africa/Bamako
    {  498, 5257,     525,  5, }, // Africa/Bangui
    {  498, 6746,     530,  6, }, // Africa/Banjul
    {  498, 7435,     536,  5, }, // Africa/Bissau
    {  498, 4957,   27088,  8, }, // Africa/Blantyre
    {  498, 5287,   51422, 10, }, // Africa/Brazzaville
    {  498, 4922,     558,  9, }, // Africa/Bujumbura
    {  498, 2320,     567,  4, }, // Africa/Cairo
    {  498, 4117,     571, 10, }, // Africa/Casablanca
    {  498, 7449,     581,  5, }, // Africa/Ceuta
    {  498, 6773,     586,  7, }, // Africa/Conakry
    {  498, 6840,     593,  5, }, // Africa/Dakar
    {  498, 5130,   27116, 12, }, // Africa/Dar_es_Salaam
    {  498, 5043,   36247,  7, }, // Africa/Djibouti
    {  498, 5243,     616,  5, }, // Africa/Douala
    {  498, 5166,   27135,  7, }, // Africa/El_Aaiun
    {  498, 6853,     628,  7, }, // Africa/Freetown
    {  498, 4906,     635,  8, }, // Africa/Gaborone
    {  498, 5015,   33781,  6, }, // Africa/Harare
    {  498, 4479,   51440, 12, }, // Africa/Johannesburg
    {  498, 4499,   27173,  5, }, // Africa/Juba
    {  498, 5151,     665,  7, }, // Africa/Kampala
    {  498, 4524,     672,  6, }, // Africa/Khartoum
    {  498, 4973,     678,  6, }, // Africa/Kigali
    {  498, 5306,     684,  7, }, // Africa/Kinshasa
    {  498, 4744,     691,  5, }, // Africa/Lagos
    {  498, 5336,   51452,  9, }, // Africa/Libreville
    {  498, 6869,     704,  4, }, // Africa/Lome
    {  498, 5211,     708,  6, }, // Africa/Luanda
    {  498, 4939,     714,  9, }, // Africa/Lubumbashi
    {  498, 5001,     723,  6, }, // Africa/Lusaka
    {  498, 5322,     729,  6, }, // Africa/Malabo
    {  498, 4987,     735,  6, }, // Africa/Maputo
    {  498, 5197,     741,  6, }, // Africa/Maseru
    {  498, 5182,     747,  7, }, // Africa/Mbabane
    {  498, 5113,     754,  8, }, // Africa/Mogadishu
    {  498, 7034,   36287,  8, }, // Africa/Monrovia
    {  498, 3853,   36295,  7, }, // Africa/Nairobi
    {  498, 5271,   27254,  8, }, // Africa/Ndjamena
    {  498, 5354,   36302,  6, }, // Africa/Niamey
    {  498, 6803,     790,  7, }, // Africa/Nouakchott
    {  498, 6727,     797,  8, }, // Africa/Ouagadougou
    {  498, 5225,     805, 10, }, // Africa/Porto-Novo
    {  498, 4435,     815,  8, }, // Africa/Sao_Tome
    {  498, 2866,     823,  7, }, // Africa/Tripoli
    {  498, 6554,     830,  5, }, // Africa/Tunis
    {  498, 4152,     835,  7, }, // Africa/Windhoek
    {  498,  132,     842,  4, }, // America/Adak
    {  498, 3346,   51461,  8, }, // America/Anchorage
    {  498, 5607,   51469,  7, }, // America/Anguilla
    {  498, 5624,     860,  7, }, // America/Antigua
    {  498, 4554,     867,  9, }, // America/Araguaina
    {  498,  166,   51476, 12, }, // America/Argentina/Buenos_Aires
    {  498,   91,     888,  9, }, // America/Argentina/Catamarca
    {  498,  270,     897,  7, }, // America/Argentina/Cordoba
    {  498,  441,   27355,  5, }, // America/Argentina/Jujuy
    {  498, 7462,     909,  8, }, // America/Argentina/La_Rioja
    {  498,  565,     917,  7, }, // America/Argentina/Mendoza
    {  498, 7489,   51488, 12, }, // America/Argentina/Rio_Gallegos
    {  498, 7520,   27380,  6, }, // America/Argentina/Salta
    {  498, 7544,     940,  8, }, // America/Argentina/San_Juan
    {  498, 5580,     948,  8, }, // America/Argentina/San_Luis
    {  498, 7571,     956,  7, }, // America/Argentina/Tucuman
    {  498, 7597,   27394,  5, }, // America/Argentina/Ushuaia
    {  498, 5640,     969,  5, }, // America/Aruba
    {  498, 4252,   51500,  8, }, // America/Asuncion
    {  498,  237,     981, 12, }, // America/Atikokan
    {  498, 3679,   36368,  4, }, // America/Bahia
    {  498, 7623,   51508, 16, }, // America/Bahia_Banderas
    {  498, 5654,    1013,  8, }, // America/Barbados
    {  498, 7646,    1021,  5, }, // America/Belem
    {  498, 5407,    1026,  5, }, // America/Belize
    {  498, 7660,    1031, 12, }, // America/Blanc-Sablon
    {  498, 7681,    1043,  9, }, // America/Boa_Vista
    {  498, 4361,    1052,  6, }, // America/Bogota
    {  498, 7699,   51524,  5, }, // America/Boise
    {  498, 7713,   51529, 13, }, // America/Cambridge_Bay
    {  498, 7735,    1075, 12, }, // America/Campo_Grande
    {  498, 3868,    1087,  6, }, // America/Cancun
    {  498, 4694,    1093,  7, }, // America/Caracas
    {  498, 4345,   51542,  7, }, // America/Cayenne
    {  498, 5496,  162717, 14, }, // America/Cayman
    {  498, 2260,    1129,  6, }, // America/Chicago
    {  498, 7756,    1135,  6, }, // America/Chihuahua
    {  498, 7774,   56160, 13, }, // America/Ciudad_Juarez
    {  498, 5422,    1141, 10, }, // America/Costa_Rica
    {  498, 7796,    1151,  7, }, // America/Creston
    {  498, 3791,   27556,  5, }, // America/Cuiaba
    {  498, 5723,   36474,  7, }, // America/Curacao
    {  498, 7812,    1172, 12, }, // America/Danmarkshavn
    {  498, 7833,    1184,  6, }, // America/Dawson
    {  498, 7848,    1190, 11, }, // America/Dawson_Creek
    {  498,  805,    1201,  6, }, // America/Denver
    {  498, 3465,   36517,  7, }, // America/Detroit
    {  498, 5739,    1214,  8, }, // America/Dominica
    {  498,  893,   51549,  8, }, // America/Edmonton
    {  498, 7869,   51557,  8, }, // America/Eirunepe
    {  498, 5441,   27633,  9, }, // America/El_Salvador
    {  498, 7886,   51565, 12, }, // America/Fort_Nelson
    {  498, 7906,    1257,  9, }, // America/Fortaleza
    {  498, 7924,   51577,  9, }, // America/Glace_Bay
    {  498, 6881,   51586,  7, }, // America/Goose_Bay
    {  498, 4612,    1282, 10, }, // America/Grand_Turk
    {  498, 5770,    1292,  7, }, // America/Grenada
    {  498, 5786,    1299,  9, }, // America/Guadeloupe
    {  498, 3760,    1308,  9, }, // America/Guatemala
    {  498, 4841,   51593,  8, }, // America/Guayaquil
    {  498, 6932,   51601,  6, }, // America/Guyana
    {  498, 1939,   51607,  8, }, // America/Halifax
    {  498, 2281,    1339,  6, }, // America/Havana
    {  498, 7942,   51615,  9, }, // America/Hermosillo
    {  498,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  498,  481,    1365, 13, }, // America/Indiana/Knox
    {  498, 7961,    1378, 16, }, // America/Indiana/Marengo
    {  498, 7985,    1394, 19, }, // America/Indiana/Petersburg
    {  498, 8012,    1413,  9, }, // America/Indiana/Tell_City
    {  498, 8038,   56191, 14, }, // America/Indiana/Vevay
    {  498, 8060,    1436,  9, }, // America/Indiana/Vincennes
    {  498, 8086,    1445,  7, }, // America/Indiana/Winamac
    {  498, 8110,    1452,  6, }, // America/Inuvik
    {  498,  660,    1458,  7, }, // America/Iqaluit
    {  498, 2799,   27879,  6, }, // America/Jamaica
    {  498, 5380,   36614,  5, }, // America/Juneau
    {  498,  521,    1476,  8, }, // America/Kentucky/Louisville
    {  498, 8125,    1484, 20, }, // America/Kentucky/Monticello
    {  498, 5704,   36619, 10, }, // America/Kralendijk
    {  498, 4376,    1514,  6, }, // America/La_Paz
    {  498, 7169,    1520,  4, }, // America/Lima
    {  498, 3239,   51629, 12, }, // America/Los_Angeles
    {  498, 5932,   56239, 18, }, // America/Lower_Princes
    {  498, 8153,   36667,  6, }, // America/Maceio
    {  498, 8168,    1559,  7, }, // America/Managua
    {  498, 1908,    1566,  6, }, // America/Manaus
    {  498, 5897,    1572,  6, }, // America/Marigot
    {  498, 5805,    1578,  9, }, // America/Martinique
    {  498, 8184,    1587,  9, }, // America/Matamoros
    {  498, 2917,    1596,  8, }, // America/Mazatlan
    {  498, 8202,    1604,  8, }, // America/Menominee
    {  498, 8220,    1612,  6, }, // America/Merida
    {  498, 8235,    1618, 10, }, // America/Metlakatla
    {  498, 2949,    1628,  6, }, // America/Mexico_City
    {  498, 4391,    1634,  7, }, // America/Miquelon
    {  498, 8254,    1641,  7, }, // America/Moncton
    {  498, 8270,   51662,  9, }, // America/Monterrey
    {  498, 4098,    1657, 10, }, // America/Montevideo
    {  498, 5824,    1667, 10, }, // America/Montserrat
    {  498, 5481,    1677,  6, }, // America/Nassau
    {  498, 2379,   51680,  8, }, // America/New_York
    {  498, 8288,    1691,  3, }, // America/Nome
    {  498, 1850,   56257,  7, }, // America/Noronha
    {  498, 8301,  162731, 22, }, // America/North_Dakota/Beulah
    {  498, 8329,  162753, 22, }, // America/North_Dakota/Center
    {  498, 8357,  162775, 27, }, // America/North_Dakota/New_Salem
    {  498,  393,    1772,  4, }, // America/Nuuk
    {  498, 8388,    1776,  7, }, // America/Ojinaga
    {  498, 2356,    1783,  6, }, // America/Panama
    {  498, 6173,    1789, 10, }, // America/Paramaribo
    {  498, 2973,    1799,  6, }, // America/Phoenix
    {  498, 3945,    1805, 12, }, // America/Port-au-Prince
    {  498, 5954,   28189, 13, }, // America/Port_of_Spain
    {  498, 8404,   28202, 11, }, // America/Porto_Velho
    {  498, 5843,   51756, 11, }, // America/Puerto_Rico
    {  498, 4030,    1852, 12, }, // America/Punta_Arenas
    {  498, 8424,    1864, 12, }, // America/Rankin_Inlet
    {  498, 8445,    1876,  6, }, // America/Recife
    {  498, 1995,   51767,  8, }, // America/Regina
    {  498, 8460,   56329,  8, }, // America/Resolute
    {  498,  695,    1897, 10, }, // America/Rio_Branco
    {  498, 8477,    1907,  8, }, // America/Santarem
    {  498, 2201,   51775,  8, }, // America/Santiago
    {  498, 6111,    1923, 13, }, // America/Santo_Domingo
    {  498, 1878,    1936,  9, }, // America/Sao_Paulo
    {  498, 6899,    1945, 11, }, // America/Scoresbysund
    {  498, 8494,    1956,  5, }, // America/Sitka
    {  498, 8508,    1961, 13, }, // America/St_Barthelemy
    {  498, 2061,   28349, 10, }, // America/St_Johns
    {  498, 5863,    1983,  9, }, // America/St_Kitts
    {  498, 5880,   56356, 10, }, // America/St_Lucia
    {  498,  855,    2003, 10, }, // America/St_Thomas
    {  498, 5913,    2013, 12, }, // America/St_Vincent
    {  498, 8530,    2025, 13, }, // America/Swift_Current
    {  498, 5461,   51817, 12, }, // America/Tegucigalpa
    {  498, 5756,    2049,  4, }, // America/Thule
    {  498,  313,    2053,  7, }, // America/Tijuana
    {  498,  608,    2060,  7, }, // America/Toronto
    {  498, 5688,    2067,  7, }, // America/Tortola
    {  498, 2093,    2074,  8, }, // America/Vancouver
    {  498, 2144,   28435,  8, }, // America/Whitehorse
    {  498,  734,    2090,  8, }, // America/Winnipeg
    {  498, 8552,   28450,  6, }, // America/Yakutat
    {  498, 6001,   36889,  5, }, // Antarctica/Casey
    {  498, 6094,   36894,  6, }, // Antarctica/Davis
    {  498, 6133,   51829, 14, }, // Antarctica/DumontDUrville
    {  498, 8568,    2131,  8, }, // Antarctica/Macquarie
    {  498, 7066,    2139,  6, }, // Antarctica/Mawson
    {  498, 7113,    2145,  9, }, // Antarctica/McMurdo
    {  498, 8589,    2154,  6, }, // Antarctica/Palmer
    {  498, 7226,    2160,  6, }, // Antarctica/Rothera
    {  498, 7295,   28510,  4, }, // Antarctica/Syowa
    {  498, 8607,   51843,  6, }, // Antarctica/Troll
    {  498, 7374,   34825,  6, }, // Antarctica/Vostok
    {  498, 1429,   51849,  7, }, // Arctic/Longyearbyen
    {  498, 5570,    2192,  4, }, // Asia/Aden
    {  498, 5395,    2196,  6, }, // Asia/Almaty
    {  498, 3968,    2202,  5, }, // Asia/Amman
    {  498, 5511,   51870,  7, }, // Asia/Anadyr
    {  498, 5523,    2213,  5, }, // Asia/Aqtau
    {  498, 5368,    2218,  6, }, // Asia/Aqtobe
    {  498,  964,    2224,  7, }, // Asia/Ashgabat
    {  498, 8624,    2231,  6, }, // Asia/Atyrau
    {  498, 3607,    2237,  6, }, // Asia/Baghdad
    {  498, 5534,   36948,  7, }, // Asia/Bahrain
    {  498, 3653,    2250,  4, }, // Asia/Baku
    {  498, 4466,    2254,  7, }, // Asia/Bangkok
    {  498, 3571,    2261,  7, }, // Asia/Barnaul
    {  498, 4086,   28563,  6, }, // Asia/Beirut
    {  498, 3778,    2274,  6, }, // Asia/Bishkek
    {  498, 5989,   28575,  6, }, // Asia/Brunei
    {  498, 4601,    2286,  4, }, // Asia/Chita
    {  498, 4511,    2290,  7, }, // Asia/Colombo
    {  498, 4540,    2297,  6, }, // Asia/Damascus
    {  498, 1093,    2303,  5, }, // Asia/Dhaka
    {  498, 6192,    2308,  4, }, // Asia/Dili
    {  498, 3596,   28596,  5, }, // Asia/Dubai
    {  498, 6159,    2317,  7, }, // Asia/Dushanbe
    {  498, 8636,    2324,  9, }, // Asia/Famagusta
    {  498, 8651,    2333,  4, }, // Asia/Gaza
    {  498, 4795,    2337,  6, }, // Asia/Hebron
    {  498, 1259,    2343,  7, }, // Asia/Ho_Chi_Minh
    {  498, 2704,   51902,  7, }, // Asia/Hong_Kong
    {  498, 4771,    2357,  4, }, // Asia/Hovd
    {  498, 4184,    2361,  7, }, // Asia/Irkutsk
    {  498, 7006,   28638,  8, }, // Asia/Jakarta
    {  498, 6992,   28646,  8, }, // Asia/Jayapura
    {  498, 1290,    2383,  9, }, // Asia/Jerusalem
    {  498, 3560,    2392,  5, }, // Asia/Kabul
    {  498, 4316,  162802, 24, }, // Asia/Kamchatka
    {  498, 4239,    2422,  6, }, // Asia/Karachi
    {  498, 1185,    2428,  8, }, // Asia/Kathmandu
    {  498, 8661,    2436,  7, }, // Asia/Khandyga
    {  498,  992,   51950,  9, }, // Asia/Kolkata
    {  498, 4197,   36978, 10, }, // Asia/Krasnoyarsk
    {  498, 4859,    2462, 12, }, // Asia/Kuala_Lumpur
    {  498, 5976,    2474,  6, }, // Asia/Kuching
    {  498, 5547,   28693,  6, }, // Asia/Kuwait
    {  498, 1211,    2486,  5, }, // Asia/Macau
    {  498, 4017,    2491,  7, }, // Asia/Magadan
    {  498, 1349,    2498,  7, }, // Asia/Makassar
    {  498, 7182,    2505,  6, }, // Asia/Manila
    {  498, 6920,    2511,  6, }, // Asia/Muscat
    {  498, 2554,  106335,  7, }, // Asia/Nicosia
    {  498, 8675,    2524, 11, }, // Asia/Novokuznetsk
    {  498, 4135,    2535, 11, }, // Asia/Novosibirsk
    {  498, 4229,    2546,  4, }, // Asia/Omsk
    {  498, 7145,   28739,  7, }, // Asia/Oral
    {  498, 6961,   51959,  8, }, // Asia/Phnom_Penh
    {  498, 8693,    2563,  9, }, // Asia/Pontianak
    {  498, 4214,   28763,  7, }, // Asia/Pyongyang
    {  498, 5559,    2579,  5, }, // Asia/Qatar
    {  498, 8708,   37005,  8, }, // Asia/Qostanay
    {  498, 4269,    2592,  9, }, // Asia/Qyzylorda
    {  498, 3584,   51967,  7, }, // Asia/Riyadh
    {  498, 4408,  162826, 11, }, // Asia/Sakhalin
    {  498, 7245,    2621,  9, }, // Asia/Samarkand
    {  498, 3279,    2630,  4, }, // Asia/Seoul
    {  498, 1053,   28801,  6, }, // Asia/Shanghai
    {  498, 3300,    2640,  8, }, // Asia/Singapore
    {  498, 4297,    2648, 13, }, // Asia/Srednekolymsk
    {  498, 3263,   28829,  6, }, // Asia/Taipei
    {  498, 4781,    2667,  7, }, // Asia/Tashkent
    {  498, 3915,   35074,  7, }, // Asia/Tbilisi
    {  498, 2772,    2678,  7, }, // Asia/Tehran
    {  498, 1317,    2685,  7, }, // Asia/Thimphu
    {  498, 2821,    2692,  5, }, // Asia/Tokyo
    {  498, 4572,    2697,  5, }, // Asia/Tomsk
    {  498, 1021,    2702, 10, }, // Asia/Ulaanbaatar
    {  498, 1159,    2712,  6, }, // Asia/Urumqi
    {  498, 8722,   51993,  9, }, // Asia/Ust-Nera
    {  498, 6977,   52002,  8, }, // Asia/Vientiane
    {  498, 4710,    2733, 11, }, // Asia/Vladivostok
    {  498, 4828,   37051,  6, }, // Asia/Yakutsk
    {  498, 1235,   56409,  5, }, // Asia/Yangon
    {  498, 3883,    2757, 12, }, // Asia/Yekaterinburg
    {  498, 3747,    2769,  6, }, // Asia/Yerevan
    {  498, 3663,  162837, 13, }, // Atlantic/Azores
    {  498, 5671,  162850, 15, }, // Atlantic/Bermuda
    {  498, 6628,  162865, 14, }, // Atlantic/Canary
    {  498, 3727,    2826, 10, }, // Atlantic/Cape_Verde
    {  498, 1395,  162879, 14, }, // Atlantic/Faroe
    {  498, 8736,   37132,  7, }, // Atlantic/Madeira
    {  498, 2748,   52082,  9, }, // Atlantic/Reykjavik
    {  498, 7272,  162893, 13, }, // Atlantic/South_Georgia
    {  498, 6821,  162906, 16, }, // Atlantic/St_Helena
    {  498, 6644,   52124,  6, }, // Atlantic/Stanley
    {  498, 1670,    2917,  8, }, // Australia/Adelaide
    {  498, 1635,    2925,  7, }, // Australia/Brisbane
    {  498, 1799,    2932, 11, }, // Australia/Broken_Hill
    {  498, 1583,    2943,  6, }, // Australia/Darwin
    {  498, 3637,   29108,  4, }, // Australia/Eucla
    {  498, 1516,    2954,  6, }, // Australia/Hobart
    {  498, 8753,    2960,  8, }, // Australia/Lindeman
    {  498, 1547,    2968,  8, }, // Australia/Lord_Howe
    {  498, 1727,   29120,  8, }, // Australia/Melbourne
    {  498, 1762,    2983,  4, }, // Australia/Perth
    {  498, 1463,   52144,  6, }, // Australia/Sydney
    {  498, 6441,    2993,  9, }, // Europe/Amsterdam
    {  498, 6216,    3002,  7, }, // Europe/Andorra
    {  498, 3620,   29149,  9, }, // Europe/Astrakhan
    {  498, 2300,   60919,  5, }, // Europe/Athens
    {  498, 6336,    3021,  7, }, // Europe/Belgrade
    {  498, 4757,    3028,  6, }, // Europe/Berlin
    {  498, 6488,    3034, 10, }, // Europe/Bratislava
    {  498, 2167,   52164,  8, }, // Europe/Brussels
    {  498, 3928,    3052,  8, }, // Europe/Bucharest
    {  498, 3806,    3060,  8, }, // Europe/Budapest
    {  498, 8772,  162922, 25, }, // Europe/Busingen
    {  498, 2583,    3091,  7, }, // Europe/Chisinau
    {  498, 6289,   52191, 10, }, // Europe/Copenhagen
    {  498, 2338,    3108,  6, }, // Europe/Dublin
    {  498, 6307,    3114,  9, }, // Europe/Gibraltar
    {  498, 8788,    3123,  6, }, // Europe/Guernsey
    {  498, 6612,   52201,  9, }, // Europe/Helsinki
    {  498, 8804,  162947,  7, }, // Europe/Isle_of_Man
    {  498, 1130,    3146,  7, }, // Europe/Istanbul
    {  498, 8823,   52219,  6, }, // Europe/Jersey
    {  498, 3979,    3158, 11, }, // Europe/Kaliningrad
    {  498, 8929,    3169,  5, }, // Europe/Kirov
    {  498, 2527,    3174,  4, }, // Europe/Kyiv
    {  498, 3213,    3178,  8, }, // Europe/Lisbon
    {  498, 6506,   29300,  7, }, // Europe/Ljubljana
    {  498, 2501,    3195,  6, }, // Europe/London
    {  498, 6365,   29313, 10, }, // Europe/Luxembourg
    {  498, 4892,    3212,  6, }, // Europe/Madrid
    {  498, 6397,   29329,  6, }, // Europe/Malta
    {  498, 6582,    3223,  9, }, // Europe/Mariehamn
    {  498, 3693,    3232,  5, }, // Europe/Minsk
    {  498, 6410,    3237,  6, }, // Europe/Monaco
    {  498, 3537,    3243,  6, }, // Europe/Moscow
    {  498, 6458,    3249,  4, }, // Europe/Oslo
    {  498, 4284,    3253,  5, }, // Europe/Paris
    {  498, 6424,    3258,  9, }, // Europe/Podgorica
    {  498, 6275,    3267,  5, }, // Europe/Prague
    {  498, 8837,    3272,  4, }, // Europe/Riga
    {  498, 6324,    3276,  3, }, // Europe/Rome
    {  498, 4331,    3279,  6, }, // Europe/Samara
    {  498, 6470,    3285, 10, }, // Europe/San_Marino
    {  498, 6245,    3295,  7, }, // Europe/Sarajevo
    {  498, 4451,    3302,  7, }, // Europe/Saratov
    {  498, 8849,   52225, 11, }, // Europe/Simferopol
    {  498, 6383,   52236,  6, }, // Europe/Skopje
    {  498, 6599,   37286,  5, }, // Europe/Sofia
    {  498, 6523,   52242,  9, }, // Europe/Stockholm
    {  498, 8867,    3337,  6, }, // Europe/Tallinn
    {  498, 6202,    3343,  6, }, // Europe/Tirane
    {  498, 8882,   52251,  9, }, // Europe/Ulyanovsk
    {  498, 6352,    3358,  5, }, // Europe/Vaduz
    {  498, 6567,    3363,  7, }, // Europe/Vatican
    {  498, 6231,    3370,  4, }, // Europe/Vienna
    {  498, 8899,   52260,  7, }, // Europe/Vilnius
    {  498, 4727,    3381,  9, }, // Europe/Volgograd
    {  498, 3190,    3390,  7, }, // Europe/Warsaw
    {  498, 6261,    3397,  6, }, // Europe/Zagreb
    {  498, 6540,   37310,  5, }, // Europe/Zurich
    {  498, 5078,    3409, 12, }, // Indian/Antananarivo
    {  498, 6947,    3421,  5, }, // Indian/Chagos
    {  498, 6046,  162954, 13, }, // Indian/Christmas
    {  498, 6063,  162967, 14, }, // Indian/Cocos
    {  498, 5029,  162981,  6, }, // Indian/Comoro
    {  498, 6661,    3460,  8, }, // Indian/Kerguelen
    {  498, 7260,   29562,  3, }, // Indian/Mahe
    {  498, 7050,  162987,  8, }, // Indian/Maldives
    {  498, 4069,   56576,  8, }, // Indian/Mauritius
    {  498, 5098,   56584,  7, }, // Indian/Mayotte
    {  498, 7211,   52348,  7, }, // Indian/Reunion
    {  498, 4422,    3498,  4, }, // Pacific/Apia
    {  498,  932,    3502,  6, }, // Pacific/Auckland
    {  498, 3706,   52355,  9, }, // Pacific/Bougainville
    {  498, 3015,    3516,  5, }, // Pacific/Chatham
    {  498, 3157,    3521,  4, }, // Pacific/Chuuk
    {  498, 2237,  162995,  9, }, // Pacific/Easter
    {  498, 7360,   52409,  5, }, // Pacific/Efate
    {  498, 7327,    3541,  7, }, // Pacific/Fakaofo
    {  498, 3902,   37407,  5, }, // Pacific/Fiji
    {  498, 7343,    3552,  8, }, // Pacific/Funafuti
    {  498, 6678,  163004, 18, }, // Pacific/Galapagos
    {  498, 6696,  163022, 11, }, // Pacific/Gambier
    {  498, 3822,    3595, 11, }, // Pacific/Guadalcanal
    {  498, 6018,    3606,  4, }, // Pacific/Guam
    {  498, 2723,   52450,  8, }, // Pacific/Honolulu
    {  498, 3049,  163033, 13, }, // Pacific/Kanton
    {  498, 3998,    3616, 10, }, // Pacific/Kiritimati
    {  498, 7019,    3626,  6, }, // Pacific/Kosrae
    {  498, 2842,   29709, 10, }, // Pacific/Kwajalein
    {  498, 4877,   37442,  7, }, // Pacific/Majuro
    {  498, 4051,  163046, 15, }, // Pacific/Marquesas
    {  498, 8914,  163061, 11, }, // Pacific/Midway
    {  498, 7084,    3680,  5, }, // Pacific/Nauru
    {  498, 7132,   52504,  4, }, // Pacific/Niue
    {  498, 4168,    3689,  7, }, // Pacific/Norfolk
    {  498, 7098,    3696,  5, }, // Pacific/Noumea
    {  498, 3126,    3701,  9, }, // Pacific/Pago_Pago
    {  498, 7155,    3710,  5, }, // Pacific/Palau
    {  498, 7194,   56657,  7, }, // Pacific/Pitcairn
    {  498, 3096,    3722,  6, }, // Pacific/Pohnpei
    {  498, 4807,    3728, 11, }, // Pacific/Port_Moresby
    {  498, 6076,    3739,  9, }, // Pacific/Rarotonga
    {  498, 6031,   29803,  6, }, // Pacific/Saipan
    {  498, 7312,    3754,  5, }, // Pacific/Tahiti
    {  498, 6712,    3759,  6, }, // Pacific/Tarawa
    {  498, 4583,    3765,  9, }, // Pacific/Tongatapu
    {  498, 7392,   29823,  4, }, // Pacific/Wake
    {  498, 7405,    3778,  6, }, // Pacific/Wallis
    {  504,   14,       0,  6, }, // Africa/Asmara Rwa/Latin/Tanzania
    {  504, 4435,       6,  8, }, // Africa/Sao_Tome
    {  504, 4252,      14,  8, }, // America/Asuncion
    {  504,  237,      22,  8, }, // America/Atikokan
    {  504, 7623,      30, 17, }, // America/Bahia_Banderas
    {  504, 3868,      47,  6, }, // America/Cancun
    {  504, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  504, 5723,      66,  7, }, // America/Curacao
    {  504,  481,      73, 13, }, // America/Indiana/Knox
    {  504, 7961,      86, 16, }, // America/Indiana/Marengo
    {  504, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  504, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  504, 8038,     139, 14, }, // America/Indiana/Vevay
    {  504, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  504, 8086,     171, 16, }, // America/Indiana/Winamac
    {  504, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  504, 5932,     207, 22, }, // America/Lower_Princes
    {  504, 8220,     229,  6, }, // America/Merida
    {  504, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  504, 8329,     255, 20, }, // America/North_Dakota/Center
    {  504, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  504,  393,     298,  4, }, // America/Nuuk
    {  504, 6899,     302, 16, }, // America/Scoresbysund
    {  504, 8508,     318, 14, }, // America/St_Barthelemy
    {  504, 2061,     332, 10, }, // America/St_Johns
    {  504, 5863,     342,  9, }, // America/St_Kitts
    {  504, 5880,     351,  9, }, // America/St_Lucia
    {  504,  855,     360, 10, }, // America/St_Thomas
    {  504, 5913,     370, 11, }, // America/St_Vincent
    {  504, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  504, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  504, 1185,     413,  9, }, // Asia/Kathmandu
    {  504,  992,     422,  7, }, // Asia/Kolkata
    {  504, 1211,     429,  5, }, // Asia/Macau
    {  504, 1235,     442,  6, }, // Asia/Yangon
    {  504, 1395,     448,  5, }, // Atlantic/Faroe
    {  504, 6821,     453, 10, }, // Atlantic/St_Helena
    {  504, 2527,     463,  4, }, // Europe/Kyiv
    {  504, 7211,     467,  7, }, // Indian/Reunion
    {  504, 3157,     474,  5, }, // Pacific/Chuuk
    {  504, 3096,     479,  7, }, // Pacific/Pohnpei
    {  505,   14,       0,  6, }, // Africa/Asmara Saho/Latin/Eritrea
    {  505, 4435,       6,  8, }, // Africa/Sao_Tome
    {  505, 4252,      14,  8, }, // America/Asuncion
    {  505,  237,      22,  8, }, // America/Atikokan
    {  505, 7623,      30, 17, }, // America/Bahia_Banderas
    {  505, 3868,      47,  6, }, // America/Cancun
    {  505, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  505, 5723,      66,  7, }, // America/Curacao
    {  505,  481,      73, 13, }, // America/Indiana/Knox
    {  505, 7961,      86, 16, }, // America/Indiana/Marengo
    {  505, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  505, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  505, 8038,     139, 14, }, // America/Indiana/Vevay
    {  505, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  505, 8086,     171, 16, }, // America/Indiana/Winamac
    {  505, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  505, 5932,     207, 22, }, // America/Lower_Princes
    {  505, 8220,     229,  6, }, // America/Merida
    {  505, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  505, 8329,     255, 20, }, // America/North_Dakota/Center
    {  505, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  505,  393,     298,  4, }, // America/Nuuk
    {  505, 6899,     302, 16, }, // America/Scoresbysund
    {  505, 8508,     318, 14, }, // America/St_Barthelemy
    {  505, 2061,     332, 10, }, // America/St_Johns
    {  505, 5863,     342,  9, }, // America/St_Kitts
    {  505, 5880,     351,  9, }, // America/St_Lucia
    {  505,  855,     360, 10, }, // America/St_Thomas
    {  505, 5913,     370, 11, }, // America/St_Vincent
    {  505, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  505, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  505, 1185,     413,  9, }, // Asia/Kathmandu
    {  505,  992,     422,  7, }, // Asia/Kolkata
    {  505, 1211,     429,  5, }, // Asia/Macau
    {  505, 1235,     442,  6, }, // Asia/Yangon
    {  505, 1395,     448,  5, }, // Atlantic/Faroe
    {  505, 6821,     453, 10, }, // Atlantic/St_Helena
    {  505, 2527,     463,  4, }, // Europe/Kyiv
    {  505, 7211,     467,  7, }, // Indian/Reunion
    {  505, 3157,     474,  5, }, // Pacific/Chuuk
    {  505, 3096,     479,  7, }, // Pacific/Pohnpei
    {  506,   14,       0,  6, }, // Africa/Asmara Sakha/Cyrillic/Russia
    {  506, 4435,       6,  8, }, // Africa/Sao_Tome
    {  506, 4252,      14,  8, }, // America/Asuncion
    {  506,  237,      22,  8, }, // America/Atikokan
    {  506, 7623,      30, 17, }, // America/Bahia_Banderas
    {  506, 3868,      47,  6, }, // America/Cancun
    {  506, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  506, 5723,      66,  7, }, // America/Curacao
    {  506,  481,      73, 13, }, // America/Indiana/Knox
    {  506, 7961,      86, 16, }, // America/Indiana/Marengo
    {  506, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  506, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  506, 8038,     139, 14, }, // America/Indiana/Vevay
    {  506, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  506, 8086,     171, 16, }, // America/Indiana/Winamac
    {  506, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  506, 5932,     207, 22, }, // America/Lower_Princes
    {  506, 8220,     229,  6, }, // America/Merida
    {  506, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  506, 8329,     255, 20, }, // America/North_Dakota/Center
    {  506, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  506,  393,     298,  4, }, // America/Nuuk
    {  506, 6899,     302, 16, }, // America/Scoresbysund
    {  506, 8508,     318, 14, }, // America/St_Barthelemy
    {  506, 2061,     332, 10, }, // America/St_Johns
    {  506, 5863,     342,  9, }, // America/St_Kitts
    {  506, 5880,     351,  9, }, // America/St_Lucia
    {  506,  855,     360, 10, }, // America/St_Thomas
    {  506, 5913,     370, 11, }, // America/St_Vincent
    {  506, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  506, 5395,    2196,  6, }, // Asia/Almaty
    {  506, 5511,  163072,  7, }, // Asia/Anadyr
    {  506,  964,  163079,  8, }, // Asia/Ashgabat
    {  506, 3607,  163087,  7, }, // Asia/Baghdad
    {  506, 3653,  163094,  5, }, // Asia/Baku
    {  506, 3571,    2261,  7, }, // Asia/Barnaul
    {  506, 4601,  163099,  5, }, // Asia/Chita
    {  506, 4511,    2290,  7, }, // Asia/Colombo
    {  506, 4540,  163104,  6, }, // Asia/Damascus
    {  506, 3596,  163110,  6, }, // Asia/Dubai
    {  506, 1259,  163116, 10, }, // Asia/Ho_Chi_Minh
    {  506, 4184,  163126,  9, }, // Asia/Irkutsk
    {  506, 3560,  163135,  6, }, // Asia/Kabul
    {  506, 4316,  163141,  9, }, // Asia/Kamchatka
    {  506, 1185,     413,  9, }, // Asia/Kathmandu
    {  506, 8661,  163150,  8, }, // Asia/Khandyga
    {  506,  992,     422,  7, }, // Asia/Kolkata
    {  506, 4197,  163158, 12, }, // Asia/Krasnoyarsk
    {  506, 1211,     429,  5, }, // Asia/Macau
    {  506, 4017,  163170,  8, }, // Asia/Magadan
    {  506, 4135,  163178, 13, }, // Asia/Novosibirsk
    {  506, 4229,  163191,  6, }, // Asia/Omsk
    {  506, 4408,  163197,  8, }, // Asia/Sakhalin
    {  506, 7245,  163205,  9, }, // Asia/Samarkand
    {  506, 4297,  163214, 11, }, // Asia/Srednekolymsk
    {  506, 1021,  163225, 11, }, // Asia/Ulaanbaatar
    {  506, 1159,  163236,  6, }, // Asia/Urumqi
    {  506, 8722,  163242,  9, }, // Asia/Ust-Nera
    {  506, 4828,  163251, 10, }, // Asia/Yakutsk
    {  506, 1235,     442,  6, }, // Asia/Yangon
    {  506, 3883,  163261, 12, }, // Asia/Yekaterinburg
    {  506, 1395,     448,  5, }, // Atlantic/Faroe
    {  506, 6821,     453, 10, }, // Atlantic/St_Helena
    {  506, 3620,  163273,  9, }, // Europe/Astrakhan
    {  506, 1130,  163282,  8, }, // Europe/Istanbul
    {  506, 3979,    3158, 11, }, // Europe/Kaliningrad
    {  506, 2527,     463,  4, }, // Europe/Kyiv
    {  506, 3537,  163290,  7, }, // Europe/Moscow
    {  506, 4331,  163297,  7, }, // Europe/Samara
    {  506, 8849,   52225, 11, }, // Europe/Simferopol
    {  506, 8882,  163304, 11, }, // Europe/Ulyanovsk
    {  506, 7211,     467,  7, }, // Indian/Reunion
    {  506, 3157,     474,  5, }, // Pacific/Chuuk
    {  506, 3096,     479,  7, }, // Pacific/Pohnpei
    {  507,   14,       0,  6, }, // Africa/Asmara Samburu/Latin/Kenya
    {  507, 4435,       6,  8, }, // Africa/Sao_Tome
    {  507, 4252,      14,  8, }, // America/Asuncion
    {  507,  237,      22,  8, }, // America/Atikokan
    {  507, 7623,      30, 17, }, // America/Bahia_Banderas
    {  507, 3868,      47,  6, }, // America/Cancun
    {  507, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  507, 5723,      66,  7, }, // America/Curacao
    {  507,  481,      73, 13, }, // America/Indiana/Knox
    {  507, 7961,      86, 16, }, // America/Indiana/Marengo
    {  507, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  507, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  507, 8038,     139, 14, }, // America/Indiana/Vevay
    {  507, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  507, 8086,     171, 16, }, // America/Indiana/Winamac
    {  507, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  507, 5932,     207, 22, }, // America/Lower_Princes
    {  507, 8220,     229,  6, }, // America/Merida
    {  507, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  507, 8329,     255, 20, }, // America/North_Dakota/Center
    {  507, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  507,  393,     298,  4, }, // America/Nuuk
    {  507, 6899,     302, 16, }, // America/Scoresbysund
    {  507, 8508,     318, 14, }, // America/St_Barthelemy
    {  507, 2061,     332, 10, }, // America/St_Johns
    {  507, 5863,     342,  9, }, // America/St_Kitts
    {  507, 5880,     351,  9, }, // America/St_Lucia
    {  507,  855,     360, 10, }, // America/St_Thomas
    {  507, 5913,     370, 11, }, // America/St_Vincent
    {  507, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  507, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  507, 1185,     413,  9, }, // Asia/Kathmandu
    {  507,  992,     422,  7, }, // Asia/Kolkata
    {  507, 1211,     429,  5, }, // Asia/Macau
    {  507, 1235,     442,  6, }, // Asia/Yangon
    {  507, 1395,     448,  5, }, // Atlantic/Faroe
    {  507, 6821,     453, 10, }, // Atlantic/St_Helena
    {  507, 2527,     463,  4, }, // Europe/Kyiv
    {  507, 7211,     467,  7, }, // Indian/Reunion
    {  507, 3157,     474,  5, }, // Pacific/Chuuk
    {  507, 3096,     479,  7, }, // Pacific/Pohnpei
    {  508,   14,       0,  6, }, // Africa/Asmara Sango/Latin/Central African Republic
    {  508, 4435,       6,  8, }, // Africa/Sao_Tome
    {  508, 4252,      14,  8, }, // America/Asuncion
    {  508,  237,      22,  8, }, // America/Atikokan
    {  508, 7623,      30, 17, }, // America/Bahia_Banderas
    {  508, 3868,      47,  6, }, // America/Cancun
    {  508, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  508, 5723,      66,  7, }, // America/Curacao
    {  508,  481,      73, 13, }, // America/Indiana/Knox
    {  508, 7961,      86, 16, }, // America/Indiana/Marengo
    {  508, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  508, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  508, 8038,     139, 14, }, // America/Indiana/Vevay
    {  508, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  508, 8086,     171, 16, }, // America/Indiana/Winamac
    {  508, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  508, 5932,     207, 22, }, // America/Lower_Princes
    {  508, 8220,     229,  6, }, // America/Merida
    {  508, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  508, 8329,     255, 20, }, // America/North_Dakota/Center
    {  508, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  508,  393,     298,  4, }, // America/Nuuk
    {  508, 6899,     302, 16, }, // America/Scoresbysund
    {  508, 8508,     318, 14, }, // America/St_Barthelemy
    {  508, 2061,     332, 10, }, // America/St_Johns
    {  508, 5863,     342,  9, }, // America/St_Kitts
    {  508, 5880,     351,  9, }, // America/St_Lucia
    {  508,  855,     360, 10, }, // America/St_Thomas
    {  508, 5913,     370, 11, }, // America/St_Vincent
    {  508, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  508, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  508, 1185,     413,  9, }, // Asia/Kathmandu
    {  508,  992,     422,  7, }, // Asia/Kolkata
    {  508, 1211,     429,  5, }, // Asia/Macau
    {  508, 1235,     442,  6, }, // Asia/Yangon
    {  508, 1395,     448,  5, }, // Atlantic/Faroe
    {  508, 6821,     453, 10, }, // Atlantic/St_Helena
    {  508, 2527,     463,  4, }, // Europe/Kyiv
    {  508, 7211,     467,  7, }, // Indian/Reunion
    {  508, 3157,     474,  5, }, // Pacific/Chuuk
    {  508, 3096,     479,  7, }, // Pacific/Pohnpei
    {  509,   14,       0,  6, }, // Africa/Asmara Sangu/Latin/Tanzania
    {  509, 4435,       6,  8, }, // Africa/Sao_Tome
    {  509, 4252,      14,  8, }, // America/Asuncion
    {  509,  237,      22,  8, }, // America/Atikokan
    {  509, 7623,      30, 17, }, // America/Bahia_Banderas
    {  509, 3868,      47,  6, }, // America/Cancun
    {  509, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  509, 5723,      66,  7, }, // America/Curacao
    {  509,  481,      73, 13, }, // America/Indiana/Knox
    {  509, 7961,      86, 16, }, // America/Indiana/Marengo
    {  509, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  509, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  509, 8038,     139, 14, }, // America/Indiana/Vevay
    {  509, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  509, 8086,     171, 16, }, // America/Indiana/Winamac
    {  509, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  509, 5932,     207, 22, }, // America/Lower_Princes
    {  509, 8220,     229,  6, }, // America/Merida
    {  509, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  509, 8329,     255, 20, }, // America/North_Dakota/Center
    {  509, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  509,  393,     298,  4, }, // America/Nuuk
    {  509, 6899,     302, 16, }, // America/Scoresbysund
    {  509, 8508,     318, 14, }, // America/St_Barthelemy
    {  509, 2061,     332, 10, }, // America/St_Johns
    {  509, 5863,     342,  9, }, // America/St_Kitts
    {  509, 5880,     351,  9, }, // America/St_Lucia
    {  509,  855,     360, 10, }, // America/St_Thomas
    {  509, 5913,     370, 11, }, // America/St_Vincent
    {  509, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  509, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  509, 1185,     413,  9, }, // Asia/Kathmandu
    {  509,  992,     422,  7, }, // Asia/Kolkata
    {  509, 1211,     429,  5, }, // Asia/Macau
    {  509, 1235,     442,  6, }, // Asia/Yangon
    {  509, 1395,     448,  5, }, // Atlantic/Faroe
    {  509, 6821,     453, 10, }, // Atlantic/St_Helena
    {  509, 2527,     463,  4, }, // Europe/Kyiv
    {  509, 7211,     467,  7, }, // Indian/Reunion
    {  509, 3157,     474,  5, }, // Pacific/Chuuk
    {  509, 3096,     479,  7, }, // Pacific/Pohnpei
    {  510,   14,       0,  6, }, // Africa/Asmara Sanskrit/Devanagari/India
    {  510, 4435,       6,  8, }, // Africa/Sao_Tome
    {  510, 4252,      14,  8, }, // America/Asuncion
    {  510,  237,      22,  8, }, // America/Atikokan
    {  510, 7623,      30, 17, }, // America/Bahia_Banderas
    {  510, 3868,      47,  6, }, // America/Cancun
    {  510, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  510, 5723,      66,  7, }, // America/Curacao
    {  510,  481,      73, 13, }, // America/Indiana/Knox
    {  510, 7961,      86, 16, }, // America/Indiana/Marengo
    {  510, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  510, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  510, 8038,     139, 14, }, // America/Indiana/Vevay
    {  510, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  510, 8086,     171, 16, }, // America/Indiana/Winamac
    {  510, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  510, 5932,     207, 22, }, // America/Lower_Princes
    {  510, 8220,     229,  6, }, // America/Merida
    {  510, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  510, 8329,     255, 20, }, // America/North_Dakota/Center
    {  510, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  510,  393,     298,  4, }, // America/Nuuk
    {  510, 6899,     302, 16, }, // America/Scoresbysund
    {  510, 8508,     318, 14, }, // America/St_Barthelemy
    {  510, 2061,     332, 10, }, // America/St_Johns
    {  510, 5863,     342,  9, }, // America/St_Kitts
    {  510, 5880,     351,  9, }, // America/St_Lucia
    {  510,  855,     360, 10, }, // America/St_Thomas
    {  510, 5913,     370, 11, }, // America/St_Vincent
    {  510, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  510, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  510, 1185,     413,  9, }, // Asia/Kathmandu
    {  510,  992,     422,  7, }, // Asia/Kolkata
    {  510, 1211,     429,  5, }, // Asia/Macau
    {  510, 1235,     442,  6, }, // Asia/Yangon
    {  510, 1395,     448,  5, }, // Atlantic/Faroe
    {  510, 6821,     453, 10, }, // Atlantic/St_Helena
    {  510, 2527,     463,  4, }, // Europe/Kyiv
    {  510, 7211,     467,  7, }, // Indian/Reunion
    {  510, 3157,     474,  5, }, // Pacific/Chuuk
    {  510, 3096,     479,  7, }, // Pacific/Pohnpei
    {  511, 6788,  163315,  8, }, // Africa/Abidjan Santali/Ol Chiki/India
    {  511,   14,       0,  6, }, // Africa/Asmara
    {  511, 4435,       6,  8, }, // Africa/Sao_Tome
    {  511,  132,  163323,  4, }, // America/Adak
    {  511, 3346,  163327,  9, }, // America/Anchorage
    {  511, 5607,  163336,  7, }, // America/Anguilla
    {  511, 5624,  163343,  8, }, // America/Antigua
    {  511, 5640,  163351,  5, }, // America/Aruba
    {  511, 4252,      14,  8, }, // America/Asuncion
    {  511,  237,  163356,  8, }, // America/Atikokan
    {  511, 7623,  163364, 15, }, // America/Bahia_Banderas
    {  511, 5654,  163379,  8, }, // America/Barbados
    {  511, 5407,  163387,  6, }, // America/Belize
    {  511, 7660,  163393, 12, }, // America/Blanc-Sablon
    {  511, 7699,  163405,  5, }, // America/Boise
    {  511, 7713,  163410, 12, }, // America/Cambridge_Bay
    {  511, 3868,  163422,  6, }, // America/Cancun
    {  511, 5496,  163428,  5, }, // America/Cayman
    {  511, 2260,  163433,  6, }, // America/Chicago
    {  511, 7756,  163439, 11, }, // America/Chihuahua
    {  511, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  511, 5422,  163450, 10, }, // America/Costa_Rica
    {  511, 7796,  163460,  7, }, // America/Creston
    {  511, 5723,  163467,  8, }, // America/Curacao
    {  511, 7812,  163475, 11, }, // America/Danmarkshavn
    {  511, 7833,  163486,  6, }, // America/Dawson
    {  511, 7848,  163492, 11, }, // America/Dawson_Creek
    {  511,  805,  163503,  6, }, // America/Denver
    {  511, 3465,  163509,  8, }, // America/Detroit
    {  511, 5739,  163517,  8, }, // America/Dominica
    {  511,  893,  163525,  8, }, // America/Edmonton
    {  511, 5441,  163533, 11, }, // America/El_Salvador
    {  511, 7886,  163544, 11, }, // America/Fort_Nelson
    {  511, 7924,  163555,  7, }, // America/Glace_Bay
    {  511, 6881,  163562,  6, }, // America/Goose_Bay
    {  511, 4612,  163568, 11, }, // America/Grand_Turk
    {  511, 5770,  163579,  8, }, // America/Grenada
    {  511, 5786,  163587,  9, }, // America/Guadeloupe
    {  511, 3760,  163596, 10, }, // America/Guatemala
    {  511, 1939,  163606,  9, }, // America/Halifax
    {  511, 2281,  163615,  6, }, // America/Havana
    {  511, 7942,  163621, 10, }, // America/Hermosillo
    {  511,  348,  163631, 13, }, // America/Indiana/Indianapolis
    {  511,  481,  163644, 14, }, // America/Indiana/Knox
    {  511, 7961,  163658, 17, }, // America/Indiana/Marengo
    {  511, 7985,  163675, 21, }, // America/Indiana/Petersburg
    {  511, 8012,  163696, 18, }, // America/Indiana/Tell_City
    {  511, 8038,  163714, 17, }, // America/Indiana/Vevay
    {  511, 8060,  163731, 18, }, // America/Indiana/Vincennes
    {  511, 8086,  163749, 17, }, // America/Indiana/Winamac
    {  511, 8110,  163766,  6, }, // America/Inuvik
    {  511,  660,  163772,  8, }, // America/Iqaluit
    {  511, 2799,  163780,  7, }, // America/Jamaica
    {  511, 5380,  163787,  7, }, // America/Juneau
    {  511,  521,  163794,  9, }, // America/Kentucky/Louisville
    {  511, 8125,  163803, 18, }, // America/Kentucky/Monticello
    {  511, 5704,  163821, 12, }, // America/Kralendijk
    {  511, 3239,  163833, 11, }, // America/Los_Angeles
    {  511, 5932,  163844, 20, }, // America/Lower_Princes
    {  511, 8168,  163864,  8, }, // America/Managua
    {  511, 5897,  163872,  7, }, // America/Marigot
    {  511, 5805,  163879,  7, }, // America/Martinique
    {  511, 8184,  163886,  9, }, // America/Matamoros
    {  511, 2917,  163895,  8, }, // America/Mazatlan
    {  511, 8202,  163903,  8, }, // America/Menominee
    {  511, 8220,  163911,  6, }, // America/Merida
    {  511, 8235,  163917, 10, }, // America/Metlakatla
    {  511, 2949,  163927, 13, }, // America/Mexico_City
    {  511, 4391,  163940,  8, }, // America/Miquelon
    {  511, 8254,  163948,  7, }, // America/Moncton
    {  511, 8270,  163955,  8, }, // America/Monterrey
    {  511, 5824,  163963,  9, }, // America/Montserrat
    {  511, 5481,  163972,  5, }, // America/Nassau
    {  511, 2379,  163977,  8, }, // America/New_York
    {  511, 8288,  163985,  4, }, // America/Nome
    {  511, 8301,  163989, 19, }, // America/North_Dakota/Beulah
    {  511, 8329,  164008, 19, }, // America/North_Dakota/Center
    {  511, 8357,  164027, 22, }, // America/North_Dakota/New_Salem
    {  511,  393,  164049,  3, }, // America/Nuuk
    {  511, 8388,  164052,  7, }, // America/Ojinaga
    {  511, 2356,  164059,  6, }, // America/Panama
    {  511, 2973,  164065,  7, }, // America/Phoenix
    {  511, 3945,  164072, 13, }, // America/Port-au-Prince
    {  511, 5954,  164085, 13, }, // America/Port_of_Spain
    {  511, 5843,  164098, 11, }, // America/Puerto_Rico
    {  511, 8424,  164109, 12, }, // America/Rankin_Inlet
    {  511, 1995,  164121,  8, }, // America/Regina
    {  511, 8460,  164129,  8, }, // America/Resolute
    {  511, 6111,  164137, 13, }, // America/Santo_Domingo
    {  511, 6899,  164150, 11, }, // America/Scoresbysund
    {  511, 8494,  164161,  5, }, // America/Sitka
    {  511, 8508,  164166, 15, }, // America/St_Barthelemy
    {  511, 2061,  164181, 10, }, // America/St_Johns
    {  511, 5863,  164191, 10, }, // America/St_Kitts
    {  511, 5880,  164201, 12, }, // America/St_Lucia
    {  511,  855,  164213, 12, }, // America/St_Thomas
    {  511, 5913,  164225, 12, }, // America/St_Vincent
    {  511, 8530,  164237, 13, }, // America/Swift_Current
    {  511, 5461,  164250, 11, }, // America/Tegucigalpa
    {  511, 5756,  164261,  5, }, // America/Thule
    {  511,  313,  164266,  8, }, // America/Tijuana
    {  511,  608,  164274,  7, }, // America/Toronto
    {  511, 5688,  164281,  7, }, // America/Tortola
    {  511, 2093,  164288,  9, }, // America/Vancouver
    {  511, 2144,  164297,  9, }, // America/Whitehorse
    {  511,  734,  164306,  8, }, // America/Winnipeg
    {  511, 8552,  164314,  7, }, // America/Yakutat
    {  511, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  511, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  511, 1185,     413,  9, }, // Asia/Kathmandu
    {  511,  992,     422,  7, }, // Asia/Kolkata
    {  511, 1211,     429,  5, }, // Asia/Macau
    {  511, 1235,     442,  6, }, // Asia/Yangon
    {  511, 5671,  164321,  9, }, // Atlantic/Bermuda
    {  511, 1395,     448,  5, }, // Atlantic/Faroe
    {  511, 6821,     453, 10, }, // Atlantic/St_Helena
    {  511, 2527,     463,  4, }, // Europe/Kyiv
    {  511, 7211,     467,  7, }, // Indian/Reunion
    {  511, 3157,     474,  5, }, // Pacific/Chuuk
    {  511, 3096,     479,  7, }, // Pacific/Pohnpei
    {  512,  237,      22,  8, }, // America/Atikokan Santali/Devanagari/India
    {  512, 7623,      30, 17, }, // America/Bahia_Banderas
    {  512, 3868,      47,  6, }, // America/Cancun
    {  512, 5723,      66,  7, }, // America/Curacao
    {  512,  481,      73, 13, }, // America/Indiana/Knox
    {  512, 7961,      86, 16, }, // America/Indiana/Marengo
    {  512, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  512, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  512, 8038,     139, 14, }, // America/Indiana/Vevay
    {  512, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  512, 8086,     171, 16, }, // America/Indiana/Winamac
    {  512, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  512, 5932,     207, 22, }, // America/Lower_Princes
    {  512, 8220,     229,  6, }, // America/Merida
    {  512, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  512, 8329,     255, 20, }, // America/North_Dakota/Center
    {  512, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  512,  393,     298,  4, }, // America/Nuuk
    {  512, 6899,     302, 16, }, // America/Scoresbysund
    {  512, 8508,     318, 14, }, // America/St_Barthelemy
    {  512, 2061,     332, 10, }, // America/St_Johns
    {  512, 5863,     342,  9, }, // America/St_Kitts
    {  512, 5880,     351,  9, }, // America/St_Lucia
    {  512,  855,     360, 10, }, // America/St_Thomas
    {  512, 5913,     370, 11, }, // America/St_Vincent
    {  513, 5059,    3784, 11, }, // Africa/Addis_Ababa Sardinian/Latin/Italy
    {  513, 7420,   94935,  6, }, // Africa/Algiers
    {  513,   14,       0,  6, }, // Africa/Asmara
    {  513, 2320,  164330,  8, }, // Africa/Cairo
    {  513, 5113,  164338, 10, }, // Africa/Mogadishu
    {  513, 7034,   43988,  8, }, // Africa/Monrovia
    {  513, 5225,  164348,  9, }, // Africa/Porto-Novo
    {  513, 4435,       6,  8, }, // Africa/Sao_Tome
    {  513, 2866,  164357,  7, }, // Africa/Tripoli
    {  513, 6554,  164364,  6, }, // Africa/Tunis
    {  513, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  513, 4252,      14,  8, }, // America/Asuncion
    {  513,  237,      22,  8, }, // America/Atikokan
    {  513, 7623,      30, 17, }, // America/Bahia_Banderas
    {  513, 3868,      47,  6, }, // America/Cancun
    {  513, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  513, 5723,      66,  7, }, // America/Curacao
    {  513, 2281,  164370,  7, }, // America/Havana
    {  513,  481,      73, 13, }, // America/Indiana/Knox
    {  513, 7961,      86, 16, }, // America/Indiana/Marengo
    {  513, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  513, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  513, 8038,     139, 14, }, // America/Indiana/Vevay
    {  513, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  513, 8086,     171, 16, }, // America/Indiana/Winamac
    {  513, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  513, 5932,     207, 22, }, // America/Lower_Princes
    {  513, 5805,   20436,  9, }, // America/Martinique
    {  513, 8220,     229,  6, }, // America/Merida
    {  513, 2949,  164377, 21, }, // America/Mexico_City
    {  513, 2379,  164398,  8, }, // America/New_York
    {  513, 8301,  164406, 25, }, // America/North_Dakota/Beulah
    {  513, 8329,  164431, 25, }, // America/North_Dakota/Center
    {  513, 8357,  164456, 28, }, // America/North_Dakota/New_Salem
    {  513,  393,     298,  4, }, // America/Nuuk
    {  513, 2356,  164484,  6, }, // America/Panama
    {  513, 5954,  164490, 16, }, // America/Port_of_Spain
    {  513, 6899,     302, 16, }, // America/Scoresbysund
    {  513, 8508,  164506, 16, }, // America/St_Barthelemy
    {  513, 2061,  164522, 13, }, // America/St_Johns
    {  513, 5863,  164535, 14, }, // America/St_Kitts
    {  513, 5880,  164549, 12, }, // America/St_Lucia
    {  513,  855,  164561, 13, }, // America/St_Thomas
    {  513, 5913,  164574, 14, }, // America/St_Vincent
    {  513, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  513, 5523,    5491,  5, }, // Asia/Aqtau
    {  513, 5368,    7488,  6, }, // Asia/Aqtobe
    {  513, 5534,    4050,  7, }, // Asia/Bahrain
    {  513, 4601,   57490,  4, }, // Asia/Chita
    {  513, 4540,   20639,  7, }, // Asia/Damascus
    {  513, 1093,  158367,  4, }, // Asia/Dhaka
    {  513, 1259,  164588, 22, }, // Asia/Ho_Chi_Minh
    {  513, 7006,   95138,  8, }, // Asia/Jakarta
    {  513, 1290,   95146, 11, }, // Asia/Jerusalem
    {  513, 1185,     413,  9, }, // Asia/Kathmandu
    {  513,  992,   20720,  7, }, // Asia/Kolkata
    {  513, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  513, 1211,     429,  5, }, // Asia/Macau
    {  513, 6920,   20739,  7, }, // Asia/Muscat
    {  513, 5559,  164610,  5, }, // Asia/Qatar
    {  513, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  513, 3584,  164615,  5, }, // Asia/Riyadh
    {  513, 7245,   20768, 10, }, // Asia/Samarkand
    {  513, 3279,  164620,  4, }, // Asia/Seoul
    {  513, 2772,  164624,  7, }, // Asia/Tehran
    {  513, 1021,  164631, 10, }, // Asia/Ulaanbaatar
    {  513, 1235,     442,  6, }, // Asia/Yangon
    {  513, 3663,  164641,  7, }, // Atlantic/Azores
    {  513, 6628,  164648, 15, }, // Atlantic/Canary
    {  513, 3727,  164663, 10, }, // Atlantic/Cape_Verde
    {  513, 1395,  164673, 14, }, // Atlantic/Faroe
    {  513, 7272,  164687, 17, }, // Atlantic/South_Georgia
    {  513, 6821,  164704, 10, }, // Atlantic/St_Helena
    {  513, 2300,    5793,  5, }, // Europe/Athens
    {  513, 6336,  164714,  8, }, // Europe/Belgrade
    {  513, 4757,  164722,  7, }, // Europe/Berlin
    {  513, 2167,   57529,  9, }, // Europe/Brussels
    {  513, 3928,  164729,  8, }, // Europe/Bucharest
    {  513, 3806,  164737,  8, }, // Europe/Budapest
    {  513, 8772,   26905,  8, }, // Europe/Busingen
    {  513, 2338,  164745,  7, }, // Europe/Dublin
    {  513, 6307,   95212, 10, }, // Europe/Gibraltar
    {  513, 8804,  164752, 12, }, // Europe/Isle_of_Man
    {  513, 1130,  164764,  8, }, // Europe/Istanbul
    {  513, 2527,     463,  4, }, // Europe/Kyiv
    {  513, 3213,   93285,  7, }, // Europe/Lisbon
    {  513, 2501,   95241,  6, }, // Europe/London
    {  513, 6365,  134444, 11, }, // Europe/Luxembourg
    {  513, 6410,  164772,  6, }, // Europe/Monaco
    {  513, 3537,   95258,  5, }, // Europe/Moscow
    {  513, 4284,   95263,  6, }, // Europe/Paris
    {  513, 6275,   21067,  5, }, // Europe/Prague
    {  513, 6324,    5946,  4, }, // Europe/Rome
    {  513, 6470,  164778, 12, }, // Europe/San_Marino
    {  513, 6523,  164790,  9, }, // Europe/Stockholm
    {  513, 6202,   21095,  6, }, // Europe/Tirane
    {  513, 6567,  164799, 22, }, // Europe/Vatican
    {  513, 3190,  164821,  8, }, // Europe/Warsaw
    {  513, 6261,  164829,  8, }, // Europe/Zagreb
    {  513, 6540,  164837,  6, }, // Europe/Zurich
    {  513, 6046,  164843, 10, }, // Indian/Christmas
    {  513, 7050,   74517,  8, }, // Indian/Maldives
    {  513, 4069,  164853, 10, }, // Indian/Mauritius
    {  513, 5098,  164863,  6, }, // Indian/Mayotte
    {  513, 7211,  164869,  8, }, // Indian/Reunion
    {  513, 3157,     474,  5, }, // Pacific/Chuuk
    {  513, 6018,  164877,  4, }, // Pacific/Guam
    {  513, 3049,    4501,  9, }, // Pacific/Kanton
    {  513, 4051,  164881,  9, }, // Pacific/Marquesas
    {  513, 3096,     479,  7, }, // Pacific/Pohnpei
    {  514,   14,       0,  6, }, // Africa/Asmara Sena/Latin/Mozambique
    {  514, 4435,       6,  8, }, // Africa/Sao_Tome
    {  514, 4252,      14,  8, }, // America/Asuncion
    {  514,  237,      22,  8, }, // America/Atikokan
    {  514, 7623,      30, 17, }, // America/Bahia_Banderas
    {  514, 3868,      47,  6, }, // America/Cancun
    {  514, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  514, 5723,      66,  7, }, // America/Curacao
    {  514,  481,      73, 13, }, // America/Indiana/Knox
    {  514, 7961,      86, 16, }, // America/Indiana/Marengo
    {  514, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  514, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  514, 8038,     139, 14, }, // America/Indiana/Vevay
    {  514, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  514, 8086,     171, 16, }, // America/Indiana/Winamac
    {  514, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  514, 5932,     207, 22, }, // America/Lower_Princes
    {  514, 8220,     229,  6, }, // America/Merida
    {  514, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  514, 8329,     255, 20, }, // America/North_Dakota/Center
    {  514, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  514,  393,     298,  4, }, // America/Nuuk
    {  514, 6899,     302, 16, }, // America/Scoresbysund
    {  514, 8508,     318, 14, }, // America/St_Barthelemy
    {  514, 2061,     332, 10, }, // America/St_Johns
    {  514, 5863,     342,  9, }, // America/St_Kitts
    {  514, 5880,     351,  9, }, // America/St_Lucia
    {  514,  855,     360, 10, }, // America/St_Thomas
    {  514, 5913,     370, 11, }, // America/St_Vincent
    {  514, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  514, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  514, 1185,     413,  9, }, // Asia/Kathmandu
    {  514,  992,     422,  7, }, // Asia/Kolkata
    {  514, 1211,     429,  5, }, // Asia/Macau
    {  514, 1235,     442,  6, }, // Asia/Yangon
    {  514, 1395,     448,  5, }, // Atlantic/Faroe
    {  514, 6821,     453, 10, }, // Atlantic/St_Helena
    {  514, 2527,     463,  4, }, // Europe/Kyiv
    {  514, 7211,     467,  7, }, // Indian/Reunion
    {  514, 3157,     474,  5, }, // Pacific/Chuuk
    {  514, 3096,     479,  7, }, // Pacific/Pohnpei
    {  515, 6788,     486,  6, }, // Africa/Abidjan Serbian/Cyrillic/Serbia
    {  515, 6760,   27046,  4, }, // Africa/Accra
    {  515, 5059,   33714, 10, }, // Africa/Addis_Ababa
    {  515, 7420,     508,  5, }, // Africa/Algiers
    {  515,   14,     513,  6, }, // Africa/Asmara
    {  515,   44,     519,  6, }, // Africa/Bamako
    {  515, 5257,   33724,  6, }, // Africa/Bangui
    {  515, 6746,     530,  6, }, // Africa/Banjul
    {  515, 7435,   33730,  5, }, // Africa/Bissau
    {  515, 4957,   33735,  7, }, // Africa/Blantyre
    {  515, 5287,   33742,  8, }, // Africa/Brazzaville
    {  515, 4922,   33750,  9, }, // Africa/Bujumbura
    {  515, 2320,   33759,  5, }, // Africa/Cairo
    {  515, 4117,   33764, 10, }, // Africa/Casablanca
    {  515, 7449,     581,  5, }, // Africa/Ceuta
    {  515, 6773,     586,  7, }, // Africa/Conakry
    {  515, 6840,     593,  5, }, // Africa/Dakar
    {  515, 5130,     598, 12, }, // Africa/Dar_es_Salaam
    {  515, 5043,     610,  6, }, // Africa/Djibouti
    {  515, 5243,     616,  5, }, // Africa/Douala
    {  515, 5166,   33774,  7, }, // Africa/El_Aaiun
    {  515, 6853,     628,  7, }, // Africa/Freetown
    {  515, 4906,     635,  7, }, // Africa/Gaborone
    {  515, 5015,   33781,  6, }, // Africa/Harare
    {  515, 4479,   33787, 11, }, // Africa/Johannesburg
    {  515, 4499,     661,  4, }, // Africa/Juba
    {  515, 5151,     665,  7, }, // Africa/Kampala
    {  515, 4524,   33798,  6, }, // Africa/Khartoum
    {  515, 4973,     678,  6, }, // Africa/Kigali
    {  515, 5306,     684,  7, }, // Africa/Kinshasa
    {  515, 4744,     691,  5, }, // Africa/Lagos
    {  515, 5336,     696,  8, }, // Africa/Libreville
    {  515, 6869,     704,  4, }, // Africa/Lome
    {  515, 5211,     708,  6, }, // Africa/Luanda
    {  515, 4939,     714,  9, }, // Africa/Lubumbashi
    {  515, 5001,     723,  6, }, // Africa/Lusaka
    {  515, 5322,     729,  6, }, // Africa/Malabo
    {  515, 4987,   33813,  6, }, // Africa/Maputo
    {  515, 5197,     741,  6, }, // Africa/Maseru
    {  515, 5182,     747,  7, }, // Africa/Mbabane
    {  515, 5113,     754,  7, }, // Africa/Mogadishu
    {  515, 7034,   33819,  9, }, // Africa/Monrovia
    {  515, 3853,   33828,  7, }, // Africa/Nairobi
    {  515, 5271,     777,  7, }, // Africa/Ndjamena
    {  515, 5354,   33835,  7, }, // Africa/Niamey
    {  515, 6803,     790,  7, }, // Africa/Nouakchott
    {  515, 6727,     797,  8, }, // Africa/Ouagadougou
    {  515, 5225,   33842, 10, }, // Africa/Porto-Novo
    {  515, 4435,   33852,  8, }, // Africa/Sao_Tome
    {  515, 2866,     823,  7, }, // Africa/Tripoli
    {  515, 6554,     830,  5, }, // Africa/Tunis
    {  515, 4152,     835,  7, }, // Africa/Windhoek
    {  515,  132,     842,  4, }, // America/Adak
    {  515, 3346,   33860,  7, }, // America/Anchorage
    {  515, 5607,   33867,  7, }, // America/Anguilla
    {  515, 5624,   33874,  7, }, // America/Antigua
    {  515, 4554,   33881, 10, }, // America/Araguaina
    {  515,  166,  164890, 12, }, // America/Argentina/Buenos_Aires
    {  515,   91,     888,  9, }, // America/Argentina/Catamarca
    {  515,  270,   33903,  7, }, // America/Argentina/Cordoba
    {  515,  441,     904,  5, }, // America/Argentina/Jujuy
    {  515, 7462,   33915,  8, }, // America/Argentina/La_Rioja
    {  515,  565,     917,  7, }, // America/Argentina/Mendoza
    {  515, 7489,   33930, 11, }, // America/Argentina/Rio_Gallegos
    {  515, 7520,     935,  5, }, // America/Argentina/Salta
    {  515, 7544,   33941,  8, }, // America/Argentina/San_Juan
    {  515, 5580,   33949,  7, }, // America/Argentina/San_Luis
    {  515, 7571,     956,  7, }, // America/Argentina/Tucuman
    {  515, 7597,   33956,  7, }, // America/Argentina/Ushuaia
    {  515, 5640,     969,  5, }, // America/Aruba
    {  515, 4252,   33963,  8, }, // America/Asuncion
    {  515,  237,  164902, 12, }, // America/Atikokan
    {  515, 3679,   33979,  5, }, // America/Bahia
    {  515, 7623,   33984, 14, }, // America/Bahia_Banderas
    {  515, 5654,    1013,  8, }, // America/Barbados
    {  515, 7646,   33998,  5, }, // America/Belem
    {  515, 5407,   34003,  6, }, // America/Belize
    {  515, 7660,   34009, 13, }, // America/Blanc-Sablon
    {  515, 7681,   34022,  9, }, // America/Boa_Vista
    {  515, 4361,    1052,  6, }, // America/Bogota
    {  515, 7699,   34031,  5, }, // America/Boise
    {  515, 7713,   34036, 11, }, // America/Cambridge_Bay
    {  515, 7735,   34047, 12, }, // America/Campo_Grande
    {  515, 3868,    1087,  6, }, // America/Cancun
    {  515, 4694,    1093,  7, }, // America/Caracas
    {  515, 4345,   34059,  5, }, // America/Cayenne
    {  515, 5496,  164914, 16, }, // America/Cayman
    {  515, 2260,    1129,  6, }, // America/Chicago
    {  515, 7756,  118097,  8, }, // America/Chihuahua
    {  515, 7774,  128160, 13, }, // America/Ciudad_Juarez
    {  515, 5422,   34086,  9, }, // America/Costa_Rica
    {  515, 7796,    1151,  7, }, // America/Creston
    {  515, 3791,    1158,  6, }, // America/Cuiaba
    {  515, 5723,   34095,  7, }, // America/Curacao
    {  515, 7812,   34102, 13, }, // America/Danmarkshavn
    {  515, 7833,   34115,  5, }, // America/Dawson
    {  515, 7848,   34120, 10, }, // America/Dawson_Creek
    {  515,  805,    1201,  6, }, // America/Denver
    {  515, 3465,    1207,  7, }, // America/Detroit
    {  515, 5739,    1214,  8, }, // America/Dominica
    {  515,  893,    1222,  8, }, // America/Edmonton
    {  515, 7869,    1230,  8, }, // America/Eirunepe
    {  515, 5441,    1238,  8, }, // America/El_Salvador
    {  515, 7886,    1246, 11, }, // America/Fort_Nelson
    {  515, 7906,    1257,  9, }, // America/Fortaleza
    {  515, 7924,   34130,  9, }, // America/Glace_Bay
    {  515, 6881,   34139,  7, }, // America/Goose_Bay
    {  515, 4612,   34146, 10, }, // America/Grand_Turk
    {  515, 5770,    1292,  7, }, // America/Grenada
    {  515, 5786,   34156,  9, }, // America/Guadeloupe
    {  515, 3760,    1308,  9, }, // America/Guatemala
    {  515, 4841,   34165,  8, }, // America/Guayaquil
    {  515, 6932,  128200,  7, }, // America/Guyana
    {  515, 1939,   34178,  8, }, // America/Halifax
    {  515, 2281,   34186,  6, }, // America/Havana
    {  515, 7942,   34192,  9, }, // America/Hermosillo
    {  515,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  515,  481,  128228, 14, }, // America/Indiana/Knox
    {  515, 7961,   34216, 17, }, // America/Indiana/Marengo
    {  515, 7985,  128242, 20, }, // America/Indiana/Petersburg
    {  515, 8012,   34253, 18, }, // America/Indiana/Tell_City
    {  515, 8038,   34271, 15, }, // America/Indiana/Vevay
    {  515, 8060,   34286, 18, }, // America/Indiana/Vincennes
    {  515, 8086,   34304, 17, }, // America/Indiana/Winamac
    {  515, 8110,    1452,  6, }, // America/Inuvik
    {  515,  660,   34321,  8, }, // America/Iqaluit
    {  515, 2799,   34329,  7, }, // America/Jamaica
    {  515, 5380,   34336,  4, }, // America/Juneau
    {  515,  521,  164930,  7, }, // America/Kentucky/Louisville
    {  515, 8125,   34340, 18, }, // America/Kentucky/Monticello
    {  515, 5704,  128293, 10, }, // America/Kralendijk
    {  515, 4376,   34368,  6, }, // America/La_Paz
    {  515, 7169,    1520,  4, }, // America/Lima
    {  515, 3239,   34374, 11, }, // America/Los_Angeles
    {  515, 5932,  164937, 21, }, // America/Lower_Princes
    {  515, 8153,   34404,  6, }, // America/Maceio
    {  515, 8168,   34410,  7, }, // America/Managua
    {  515, 1908,    1566,  6, }, // America/Manaus
    {  515, 5897,    1572,  6, }, // America/Marigot
    {  515, 5805,    1578,  8, }, // America/Martinique
    {  515, 8184,    1587,  9, }, // America/Matamoros
    {  515, 2917,   34417,  8, }, // America/Mazatlan
    {  515, 8202,    1604,  8, }, // America/Menominee
    {  515, 8220,    1612,  6, }, // America/Merida
    {  515, 8235,    1618, 10, }, // America/Metlakatla
    {  515, 2949,   34425, 12, }, // America/Mexico_City
    {  515, 4391,    1634,  7, }, // America/Miquelon
    {  515, 8254,    1641,  7, }, // America/Moncton
    {  515, 8270,   34437,  8, }, // America/Monterrey
    {  515, 4098,    1657, 10, }, // America/Montevideo
    {  515, 5824,   34445,  9, }, // America/Montserrat
    {  515, 5481,   34454,  5, }, // America/Nassau
    {  515, 2379,   34459,  6, }, // America/New_York
    {  515, 8288,    1691,  3, }, // America/Nome
    {  515, 1850,   34465,  6, }, // America/Noronha
    {  515, 8301,   34471, 22, }, // America/North_Dakota/Beulah
    {  515, 8329,   34493, 22, }, // America/North_Dakota/Center
    {  515, 8357,   34515, 26, }, // America/North_Dakota/New_Salem
    {  515,  393,  164958,  6, }, // America/Nuuk
    {  515, 8388,    1776,  7, }, // America/Ojinaga
    {  515, 2356,    1783,  6, }, // America/Panama
    {  515, 6173,    1789, 10, }, // America/Paramaribo
    {  515, 2973,    1799,  6, }, // America/Phoenix
    {  515, 3945,  128384, 12, }, // America/Port-au-Prince
    {  515, 5954,   34547, 13, }, // America/Port_of_Spain
    {  515, 8404,   34560, 10, }, // America/Porto_Velho
    {  515, 5843,  164964, 10, }, // America/Puerto_Rico
    {  515, 4030,   34579, 12, }, // America/Punta_Arenas
    {  515, 8424,   34591, 12, }, // America/Rankin_Inlet
    {  515, 8445,   34603,  6, }, // America/Recife
    {  515, 1995,   34609,  6, }, // America/Regina
    {  515, 8460,   34615,  7, }, // America/Resolute
    {  515,  695,   34622, 10, }, // America/Rio_Branco
    {  515, 8477,   34632,  8, }, // America/Santarem
    {  515, 2201,  164974,  8, }, // America/Santiago
    {  515, 6111,   34649, 13, }, // America/Santo_Domingo
    {  515, 1878,   34662,  9, }, // America/Sao_Paulo
    {  515, 6899,  164982, 12, }, // America/Scoresbysund
    {  515, 8494,    1956,  5, }, // America/Sitka
    {  515, 8508,   34683, 14, }, // America/St_Barthelemy
    {  515, 2061,   34697,  7, }, // America/St_Johns
    {  515, 5863,   34704,  9, }, // America/St_Kitts
    {  515, 5880,   34713, 10, }, // America/St_Lucia
    {  515,  855,   34723,  8, }, // America/St_Thomas
    {  515, 5913,   34731, 12, }, // America/St_Vincent
    {  515, 8530,   34743, 12, }, // America/Swift_Current
    {  515, 5461,    2038, 11, }, // America/Tegucigalpa
    {  515, 5756,    2049,  3, }, // America/Thule
    {  515,  313,    2053,  7, }, // America/Tijuana
    {  515,  608,    2060,  7, }, // America/Toronto
    {  515, 5688,    2067,  7, }, // America/Tortola
    {  515, 2093,    2074,  8, }, // America/Vancouver
    {  515, 2144,   34755,  8, }, // America/Whitehorse
    {  515,  734,   34763,  7, }, // America/Winnipeg
    {  515, 8552,   34770,  7, }, // America/Yakutat
    {  515, 6001,  128483,  5, }, // Antarctica/Casey
    {  515, 6094,   34782,  6, }, // Antarctica/Davis
    {  515, 6133,   34788, 13, }, // Antarctica/DumontDUrville
    {  515, 8568,   34801,  7, }, // Antarctica/Macquarie
    {  515, 7066,   34808,  5, }, // Antarctica/Mawson
    {  515, 7113,   34813,  8, }, // Antarctica/McMurdo
    {  515, 8589,    2154,  6, }, // Antarctica/Palmer
    {  515, 7226,    2160,  6, }, // Antarctica/Rothera
    {  515, 7295,   34821,  4, }, // Antarctica/Syowa
    {  515, 8607,    2171,  4, }, // Antarctica/Troll
    {  515, 7374,   34825,  6, }, // Antarctica/Vostok
    {  515, 1429,   34831, 11, }, // Arctic/Longyearbyen
    {  515, 5570,    2192,  4, }, // Asia/Aden
    {  515, 5395,   34842,  6, }, // Asia/Almaty
    {  515, 3968,   28531,  4, }, // Asia/Amman
    {  515, 5511,   34848,  6, }, // Asia/Anadyr
    {  515, 5523,    2213,  5, }, // Asia/Aqtau
    {  515, 5368,   34854,  7, }, // Asia/Aqtobe
    {  515,  964,    2224,  7, }, // Asia/Ashgabat
    {  515, 8624,   51877,  6, }, // Asia/Atyrau
    {  515, 3607,    2237,  6, }, // Asia/Baghdad
    {  515, 5534,    2243,  7, }, // Asia/Bahrain
    {  515, 3653,    2250,  4, }, // Asia/Baku
    {  515, 4466,    2254,  7, }, // Asia/Bangkok
    {  515, 3571,    2261,  7, }, // Asia/Barnaul
    {  515, 4086,   34873,  6, }, // Asia/Beirut
    {  515, 3778,    2274,  6, }, // Asia/Bishkek
    {  515, 5989,   34879,  6, }, // Asia/Brunei
    {  515, 4601,    2286,  4, }, // Asia/Chita
    {  515, 4511,    2290,  7, }, // Asia/Colombo
    {  515, 4540,    2297,  6, }, // Asia/Damascus
    {  515, 1093,     593,  4, }, // Asia/Dhaka
    {  515, 6192,    2308,  4, }, // Asia/Dili
    {  515, 3596,    2312,  5, }, // Asia/Dubai
    {  515, 6159,    2317,  7, }, // Asia/Dushanbe
    {  515, 8636,    2324,  9, }, // Asia/Famagusta
    {  515, 8651,    2333,  4, }, // Asia/Gaza
    {  515, 4795,   34886,  6, }, // Asia/Hebron
    {  515, 1259,   34892,  9, }, // Asia/Ho_Chi_Minh
    {  515, 2704,  164994,  8, }, // Asia/Hong_Kong
    {  515, 4771,    2357,  4, }, // Asia/Hovd
    {  515, 4184,   34910,  6, }, // Asia/Irkutsk
    {  515, 7006,    2368,  7, }, // Asia/Jakarta
    {  515, 6992,   34916,  8, }, // Asia/Jayapura
    {  515, 1290,   34924,  9, }, // Asia/Jerusalem
    {  515, 3560,    2392,  5, }, // Asia/Kabul
    {  515, 4316,    2411,  8, }, // Asia/Kamchatka
    {  515, 4239,    2422,  6, }, // Asia/Karachi
    {  515, 1185,    2428,  8, }, // Asia/Kathmandu
    {  515, 8661,   34933,  7, }, // Asia/Khandyga
    {  515,  992,   34940,  7, }, // Asia/Kolkata
    {  515, 4197,   34947, 11, }, // Asia/Krasnoyarsk
    {  515, 4859,   34958, 12, }, // Asia/Kuala_Lumpur
    {  515, 5976,    2474,  6, }, // Asia/Kuching
    {  515, 5547,   34970,  6, }, // Asia/Kuwait
    {  515, 1211,    2486,  5, }, // Asia/Macau
    {  515, 4017,    2491,  7, }, // Asia/Magadan
    {  515, 1349,    2498,  7, }, // Asia/Makassar
    {  515, 7182,    2505,  6, }, // Asia/Manila
    {  515, 6920,   34981,  6, }, // Asia/Muscat
    {  515, 2554,   34987,  8, }, // Asia/Nicosia
    {  515, 8675,   34995, 11, }, // Asia/Novokuznetsk
    {  515, 4135,    2535, 11, }, // Asia/Novosibirsk
    {  515, 4229,    2546,  4, }, // Asia/Omsk
    {  515, 7145,   35006,  4, }, // Asia/Oral
    {  515, 6961,   35010,  8, }, // Asia/Phnom_Penh
    {  515, 8693,  128540, 10, }, // Asia/Pontianak
    {  515, 4214,   35018,  9, }, // Asia/Pyongyang
    {  515, 5559,    2579,  5, }, // Asia/Qatar
    {  515, 8708,   35027,  8, }, // Asia/Qostanay
    {  515, 4269,   35035,  9, }, // Asia/Qyzylorda
    {  515, 3584,   35044,  5, }, // Asia/Riyadh
    {  515, 4408,    2614,  7, }, // Asia/Sakhalin
    {  515, 7245,    2621,  9, }, // Asia/Samarkand
    {  515, 3279,    2630,  4, }, // Asia/Seoul
    {  515, 1053,   35049,  6, }, // Asia/Shanghai
    {  515, 3300,    2640,  8, }, // Asia/Singapore
    {  515, 4297,   35055, 13, }, // Asia/Srednekolymsk
    {  515, 3263,   35068,  6, }, // Asia/Taipei
    {  515, 4781,    2667,  7, }, // Asia/Tashkent
    {  515, 3915,   35074,  7, }, // Asia/Tbilisi
    {  515, 2772,   35081,  7, }, // Asia/Tehran
    {  515, 1317,   35088,  5, }, // Asia/Thimphu
    {  515, 2821,    2692,  5, }, // Asia/Tokyo
    {  515, 4572,    2697,  5, }, // Asia/Tomsk
    {  515, 1021,   35093, 10, }, // Asia/Ulaanbaatar
    {  515, 1159,  165002,  6, }, // Asia/Urumqi
    {  515, 8722,    2718,  8, }, // Asia/Ust-Nera
    {  515, 6977,   35110, 10, }, // Asia/Vientiane
    {  515, 4710,    2733, 11, }, // Asia/Vladivostok
    {  515, 4828,   35120,  7, }, // Asia/Yakutsk
    {  515, 1235,   28911,  6, }, // Asia/Yangon
    {  515, 3883,   35127, 13, }, // Asia/Yekaterinburg
    {  515, 3747,   35140,  7, }, // Asia/Yerevan
    {  515, 3663,   35147,  5, }, // Atlantic/Azores
    {  515, 5671,   35152,  7, }, // Atlantic/Bermuda
    {  515, 6628,   35159, 15, }, // Atlantic/Canary
    {  515, 3727,   35174, 18, }, // Atlantic/Cape_Verde
    {  515, 1395,  165008, 13, }, // Atlantic/Faroe
    {  515, 8736,    2853,  7, }, // Atlantic/Madeira
    {  515, 2748,   35205,  9, }, // Atlantic/Reykjavik
    {  515, 7272,   35214, 13, }, // Atlantic/South_Georgia
    {  515, 6821,  165021, 12, }, // Atlantic/St_Helena
    {  515, 6644,    2911,  6, }, // Atlantic/Stanley
    {  515, 1670,  165033,  7, }, // Australia/Adelaide
    {  515, 1635,   35237,  8, }, // Australia/Brisbane
    {  515, 1799,   35245, 10, }, // Australia/Broken_Hill
    {  515, 1583,    2943,  6, }, // Australia/Darwin
    {  515, 3637,    2949,  5, }, // Australia/Eucla
    {  515, 1516,    2954,  6, }, // Australia/Hobart
    {  515, 8753,    2960,  8, }, // Australia/Lindeman
    {  515, 1547,   37190,  8, }, // Australia/Lord_Howe
    {  515, 1727,    2976,  7, }, // Australia/Melbourne
    {  515, 1762,    2983,  4, }, // Australia/Perth
    {  515, 1463,   35263,  6, }, // Australia/Sydney
    {  515, 6441,    2993,  9, }, // Europe/Amsterdam
    {  515, 6216,   29143,  6, }, // Europe/Andorra
    {  515, 3620,  165040,  8, }, // Europe/Astrakhan
    {  515, 2300,   35269,  5, }, // Europe/Athens
    {  515, 6336,   35274,  7, }, // Europe/Belgrade
    {  515, 4757,    3028,  6, }, // Europe/Berlin
    {  515, 6488,    3034, 10, }, // Europe/Bratislava
    {  515, 2167,   35281,  6, }, // Europe/Brussels
    {  515, 3928,   35287,  8, }, // Europe/Bucharest
    {  515, 3806,   35295, 10, }, // Europe/Budapest
    {  515, 8772,  165048,  8, }, // Europe/Busingen
    {  515, 2583,   35313,  7, }, // Europe/Chisinau
    {  515, 6289,   35320, 10, }, // Europe/Copenhagen
    {  515, 2338,   35330,  6, }, // Europe/Dublin
    {  515, 6307,    3114,  9, }, // Europe/Gibraltar
    {  515, 8788,   35336,  6, }, // Europe/Guernsey
    {  515, 6612,    3129,  8, }, // Europe/Helsinki
    {  515, 8804,  165056, 10, }, // Europe/Isle_of_Man
    {  515, 1130,   35352,  8, }, // Europe/Istanbul
    {  515, 8823,  128648,  5, }, // Europe/Jersey
    {  515, 3979,   35360, 11, }, // Europe/Kaliningrad
    {  515, 8929,    3169,  5, }, // Europe/Kirov
    {  515, 2527,   35371,  5, }, // Europe/Kyiv
    {  515, 3213,   35376,  7, }, // Europe/Lisbon
    {  515, 6506,   35383,  7, }, // Europe/Ljubljana
    {  515, 2501,    3195,  6, }, // Europe/London
    {  515, 6365,   35390, 10, }, // Europe/Luxembourg
    {  515, 4892,    3212,  6, }, // Europe/Madrid
    {  515, 6397,    3218,  5, }, // Europe/Malta
    {  515, 6582,   35400,  8, }, // Europe/Mariehamn
    {  515, 3693,    3232,  5, }, // Europe/Minsk
    {  515, 6410,    3237,  6, }, // Europe/Monaco
    {  515, 3537,    3243,  6, }, // Europe/Moscow
    {  515, 6458,    3249,  4, }, // Europe/Oslo
    {  515, 4284,   35408,  5, }, // Europe/Paris
    {  515, 6424,    3258,  9, }, // Europe/Podgorica
    {  515, 6275,    3267,  4, }, // Europe/Prague
    {  515, 8837,    3272,  4, }, // Europe/Riga
    {  515, 6324,    3276,  3, }, // Europe/Rome
    {  515, 4331,    3279,  6, }, // Europe/Samara
    {  515, 6470,   35413, 10, }, // Europe/San_Marino
    {  515, 6245,   35423,  8, }, // Europe/Sarajevo
    {  515, 4451,    3302,  7, }, // Europe/Saratov
    {  515, 8849,   35431, 10, }, // Europe/Simferopol
    {  515, 6383,   35441,  6, }, // Europe/Skopje
    {  515, 6599,   35447,  6, }, // Europe/Sofia
    {  515, 6523,   35453,  8, }, // Europe/Stockholm
    {  515, 8867,   35461,  5, }, // Europe/Tallinn
    {  515, 6202,    3343,  6, }, // Europe/Tirane
    {  515, 8882,   35466,  8, }, // Europe/Ulyanovsk
    {  515, 6352,   35474,  5, }, // Europe/Vaduz
    {  515, 6567,    3363,  7, }, // Europe/Vatican
    {  515, 6231,   35479,  3, }, // Europe/Vienna
    {  515, 8899,  165066,  6, }, // Europe/Vilnius
    {  515, 4727,    3381,  9, }, // Europe/Volgograd
    {  515, 3190,    3390,  7, }, // Europe/Warsaw
    {  515, 6261,    3397,  6, }, // Europe/Zagreb
    {  515, 6540,   35488,  5, }, // Europe/Zurich
    {  515, 5078,   35493, 12, }, // Indian/Antananarivo
    {  515, 6947,    3421,  5, }, // Indian/Chagos
    {  515, 6046,   35505,  5, }, // Indian/Christmas
    {  515, 6063,    3438,  5, }, // Indian/Cocos
    {  515, 5029,   35510,  6, }, // Indian/Comoro
    {  515, 6661,    3460,  8, }, // Indian/Kerguelen
    {  515, 7260,   35516,  4, }, // Indian/Mahe
    {  515, 7050,   35520,  7, }, // Indian/Maldives
    {  515, 4069,   35527, 10, }, // Indian/Mauritius
    {  515, 5098,   35537,  5, }, // Indian/Mayotte
    {  515, 7211,   35542,  7, }, // Indian/Reunion
    {  515, 4422,   35549,  5, }, // Pacific/Apia
    {  515,  932,   35554,  6, }, // Pacific/Auckland
    {  515, 3706,  128732,  8, }, // Pacific/Bougainville
    {  515, 3015,   37380,  5, }, // Pacific/Chatham
    {  515, 3157,    3521,  4, }, // Pacific/Chuuk
    {  515, 2237,   35566, 14, }, // Pacific/Easter
    {  515, 7360,    3536,  4, }, // Pacific/Efate
    {  515, 7327,    3541,  7, }, // Pacific/Fakaofo
    {  515, 3902,    3548,  4, }, // Pacific/Fiji
    {  515, 7343,    3552,  8, }, // Pacific/Funafuti
    {  515, 6678,    3560,  9, }, // Pacific/Galapagos
    {  515, 6696,   35580,  7, }, // Pacific/Gambier
    {  515, 3822,   35587, 11, }, // Pacific/Guadalcanal
    {  515, 6018,    3606,  4, }, // Pacific/Guam
    {  515, 2723,   35598,  8, }, // Pacific/Honolulu
    {  515, 3049,   35606,  9, }, // Pacific/Kanton
    {  515, 3998,    3616, 10, }, // Pacific/Kiritimati
    {  515, 7019,   35615,  5, }, // Pacific/Kosrae
    {  515, 2842,   35620, 10, }, // Pacific/Kwajalein
    {  515, 4877,   35630,  6, }, // Pacific/Majuro
    {  515, 4051,    3647,  6, }, // Pacific/Marquesas
    {  515, 8914,   35636,  6, }, // Pacific/Midway
    {  515, 7084,    3680,  5, }, // Pacific/Nauru
    {  515, 7132,    3685,  4, }, // Pacific/Niue
    {  515, 4168,    3689,  7, }, // Pacific/Norfolk
    {  515, 7098,    3696,  5, }, // Pacific/Noumea
    {  515, 3126,   35642,  9, }, // Pacific/Pago_Pago
    {  515, 7155,    3710,  5, }, // Pacific/Palau
    {  515, 7194,    3715,  7, }, // Pacific/Pitcairn
    {  515, 3096,   35651,  6, }, // Pacific/Pohnpei
    {  515, 4807,   35657, 11, }, // Pacific/Port_Moresby
    {  515, 6076,    3739,  9, }, // Pacific/Rarotonga
    {  515, 6031,   35668,  6, }, // Pacific/Saipan
    {  515, 7312,   35674,  6, }, // Pacific/Tahiti
    {  515, 6712,    3759,  6, }, // Pacific/Tarawa
    {  515, 4583,    3765,  9, }, // Pacific/Tongatapu
    {  515, 7392,   35680,  4, }, // Pacific/Wake
    {  515, 7405,   35684,  5, }, // Pacific/Wallis
    {  516,  348,  128215, 13, }, // America/Indiana/Indianapolis Serbian/Cyrillic/Bosnia and Herzegovina
    {  516, 8038,  165072, 14, }, // America/Indiana/Vevay
    {  516, 8060,  165086, 17, }, // America/Indiana/Vincennes
    {  516,  521,  164930,  6, }, // America/Kentucky/Louisville
    {  516, 8301,  165103, 22, }, // America/North_Dakota/Beulah
    {  516, 8329,  165125, 23, }, // America/North_Dakota/Center
    {  516, 8357,  165148, 27, }, // America/North_Dakota/New_Salem
    {  516, 3945,    1805, 12, }, // America/Port-au-Prince
    {  516, 5954,  128396, 13, }, // America/Port_of_Spain
    {  516, 5843,   34570,  9, }, // America/Puerto_Rico
    {  516, 1995,  128421,  7, }, // America/Regina
    {  516, 8460,  118409,  7, }, // America/Resolute
    {  516, 6899,   34671, 12, }, // America/Scoresbysund
    {  516, 8508,   36793, 13, }, // America/St_Barthelemy
    {  516, 2061,  128443,  9, }, // America/St_Johns
    {  516,  855,  128473, 10, }, // America/St_Thomas
    {  516, 8530,  165175, 12, }, // America/Swift_Current
    {  516, 6133,  165187, 13, }, // Antarctica/DumontDUrville
    {  516, 8568,  128500,  7, }, // Antarctica/Macquarie
    {  516, 1429,  165200,  7, }, // Arctic/Longyearbyen
    {  516, 2338,   35330,  6, }, // Europe/Dublin
    {  516, 2501,    3195,  6, }, // Europe/London
    {  516, 7132,  165207,  5, }, // Pacific/Niue
    {  519, 6788,   57564,  7, }, // Africa/Abidjan Serbian/Latin/Bosnia and Herzegovina
    {  519, 6760,    4531,  4, }, // Africa/Accra
    {  519, 5059,   35689, 10, }, // Africa/Addis_Ababa
    {  519, 7420,   57447,  5, }, // Africa/Algiers
    {  519,   14,   91122,  6, }, // Africa/Asmara
    {  519,   44,  165212,  6, }, // Africa/Bamako
    {  519, 5257,  165218,  6, }, // Africa/Bangui
    {  519, 6746,  165224,  6, }, // Africa/Banjul
    {  519, 7435,  165230,  5, }, // Africa/Bissau
    {  519, 4957,    6200,  7, }, // Africa/Blantyre
    {  519, 5287,    6208,  8, }, // Africa/Brazzaville
    {  519, 4922,  165235, 10, }, // Africa/Bujumbura
    {  519, 2320,    4572,  5, }, // Africa/Cairo
    {  519, 4117,   33144, 10, }, // Africa/Casablanca
    {  519, 7449,   21231,  5, }, // Africa/Ceuta
    {  519, 6773,    6249,  7, }, // Africa/Conakry
    {  519, 6840,   61029,  5, }, // Africa/Dakar
    {  519, 5130,    6256, 12, }, // Africa/Dar_es_Salaam
    {  519, 5043,   33154,  7, }, // Africa/Djibouti
    {  519, 5243,   21254,  5, }, // Africa/Douala
    {  519, 5166,    6275,  7, }, // Africa/El_Aaiun
    {  519, 6853,    6282,  7, }, // Africa/Freetown
    {  519, 4906,   61071,  7, }, // Africa/Gaborone
    {  519, 5015,   61079,  6, }, // Africa/Harare
    {  519, 4479,    6289, 11, }, // Africa/Johannesburg
    {  519, 4499,  124745,  5, }, // Africa/Juba
    {  519, 5151,  165245,  7, }, // Africa/Kampala
    {  519, 4524,    6305,  6, }, // Africa/Khartoum
    {  519, 4973,  124759,  6, }, // Africa/Kigali
    {  519, 5306,  122029,  7, }, // Africa/Kinshasa
    {  519, 4744,   61130,  5, }, // Africa/Lagos
    {  519, 5336,    6311,  8, }, // Africa/Libreville
    {  519, 6869,   61145,  4, }, // Africa/Lome
    {  519, 5211,  165252,  6, }, // Africa/Luanda
    {  519, 4939,  122051,  9, }, // Africa/Lubumbashi
    {  519, 5001,  165258,  6, }, // Africa/Lusaka
    {  519, 5322,  165264,  6, }, // Africa/Malabo
    {  519, 4987,  165270,  6, }, // Africa/Maputo
    {  519, 5197,   61171,  6, }, // Africa/Maseru
    {  519, 5182,  165276,  7, }, // Africa/Mbabane
    {  519, 5113,   33161,  7, }, // Africa/Mogadishu
    {  519, 7034,  122074,  9, }, // Africa/Monrovia
    {  519, 3853,   61201,  7, }, // Africa/Nairobi
    {  519, 5271,   57597,  8, }, // Africa/Ndjamena
    {  519, 5354,  165283,  7, }, // Africa/Niamey
    {  519, 6803,   57605,  7, }, // Africa/Nouakchott
    {  519, 6727,   26402,  8, }, // Africa/Ouagadougou
    {  519, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  519, 4435,   91128,  8, }, // Africa/Sao_Tome
    {  519, 2866,   35756,  7, }, // Africa/Tripoli
    {  519, 6554,   91301,  5, }, // Africa/Tunis
    {  519, 4152,   21354,  7, }, // Africa/Windhoek
    {  519,  132,   97814,  4, }, // America/Adak
    {  519, 3346,  165290,  8, }, // America/Anchorage
    {  519, 5607,   33168,  7, }, // America/Anguilla
    {  519, 5624,   33175,  7, }, // America/Antigua
    {  519, 4554,  124912, 10, }, // America/Araguaina
    {  519,  166,  165298, 12, }, // America/Argentina/Buenos_Aires
    {  519,   91,    6400,  9, }, // America/Argentina/Catamarca
    {  519,  270,    6409,  7, }, // America/Argentina/Cordoba
    {  519,  441,  165310,  5, }, // America/Argentina/Jujuy
    {  519, 7462,    6421,  8, }, // America/Argentina/La_Rioja
    {  519,  565,   97505,  7, }, // America/Argentina/Mendoza
    {  519, 7489,  124952, 12, }, // America/Argentina/Rio_Gallegos
    {  519, 7520,   61307,  5, }, // America/Argentina/Salta
    {  519, 7544,  165315,  8, }, // America/Argentina/San_Juan
    {  519, 5580,  165323,  7, }, // America/Argentina/San_Luis
    {  519, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {  519, 7597,  165330,  7, }, // America/Argentina/Ushuaia
    {  519, 5640,  165337,  5, }, // America/Aruba
    {  519, 4252,    6474,  8, }, // America/Asuncion
    {  519,  237,  165342, 12, }, // America/Atikokan
    {  519, 3679,  122240,  5, }, // America/Bahia
    {  519, 7623,  165354, 14, }, // America/Bahia_Banderas
    {  519, 5654,  122258,  8, }, // America/Barbados
    {  519, 7646,   61342,  5, }, // America/Belem
    {  519, 5407,  165368,  6, }, // America/Belize
    {  519, 7660,  165374, 13, }, // America/Blanc-Sablon
    {  519, 7681,  165387,  9, }, // America/Boa_Vista
    {  519, 4361,  165396,  6, }, // America/Bogota
    {  519, 7699,  165402,  5, }, // America/Boise
    {  519, 7713,  165407, 12, }, // America/Cambridge_Bay
    {  519, 7735,    4780, 12, }, // America/Campo_Grande
    {  519, 3868,    6564,  6, }, // America/Cancun
    {  519, 4694,    4792,  7, }, // America/Caracas
    {  519, 4345,    6570,  5, }, // America/Cayenne
    {  519, 5496,  165419, 16, }, // America/Cayman
    {  519, 2260,  165435,  6, }, // America/Chicago
    {  519, 7756,  125151,  8, }, // America/Chihuahua
    {  519, 7774,  165441, 13, }, // America/Ciudad_Juarez
    {  519, 5422,   33188,  9, }, // America/Costa_Rica
    {  519, 7796,    6618,  7, }, // America/Creston
    {  519, 3791,    4846,  6, }, // America/Cuiaba
    {  519, 5723,    6631,  7, }, // America/Curacao
    {  519, 7812,  165454, 13, }, // America/Danmarkshavn
    {  519, 7833,  125206,  5, }, // America/Dawson
    {  519, 7848,  125213, 10, }, // America/Dawson_Creek
    {  519,  805,   61425,  6, }, // America/Denver
    {  519, 3465,  122438,  7, }, // America/Detroit
    {  519, 5739,   21548,  8, }, // America/Dominica
    {  519,  893,   61448,  8, }, // America/Edmonton
    {  519, 7869,  165467,  8, }, // America/Eirunepe
    {  519, 5441,    6688,  8, }, // America/El_Salvador
    {  519, 7886,  125270, 11, }, // America/Fort_Nelson
    {  519, 7906,  165475,  9, }, // America/Fortaleza
    {  519, 7924,  165484,  9, }, // America/Glace_Bay
    {  519, 6881,  165493,  7, }, // America/Goose_Bay
    {  519, 4612,  165500, 10, }, // America/Grand_Turk
    {  519, 5770,  165510,  7, }, // America/Grenada
    {  519, 5786,   33197,  9, }, // America/Guadeloupe
    {  519, 3760,   33206,  9, }, // America/Guatemala
    {  519, 4841,  122518,  8, }, // America/Guayaquil
    {  519, 6932,   57452,  7, }, // America/Guyana
    {  519, 1939,    6787,  8, }, // America/Halifax
    {  519, 2281,  147659,  6, }, // America/Havana
    {  519, 7942,  165517, 10, }, // America/Hermosillo
    {  519,  348,  165527, 13, }, // America/Indiana/Indianapolis
    {  519,  481,  165540, 14, }, // America/Indiana/Knox
    {  519, 7961,  165554, 17, }, // America/Indiana/Marengo
    {  519, 7985,  165571, 20, }, // America/Indiana/Petersburg
    {  519, 8012,  165591, 18, }, // America/Indiana/Tell_City
    {  519, 8038,  165609, 14, }, // America/Indiana/Vevay
    {  519, 8060,  165623, 17, }, // America/Indiana/Vincennes
    {  519, 8086,  165640, 17, }, // America/Indiana/Winamac
    {  519, 8110,  122681,  6, }, // America/Inuvik
    {  519,  660,  165657,  8, }, // America/Iqaluit
    {  519, 2799,   33215,  7, }, // America/Jamaica
    {  519, 5380,  165665,  4, }, // America/Juneau
    {  519,  521,  165669,  6, }, // America/Kentucky/Louisville
    {  519, 8125,  165675, 18, }, // America/Kentucky/Monticello
    {  519, 5704,  165693, 10, }, // America/Kralendijk
    {  519, 4376,  165703,  6, }, // America/La_Paz
    {  519, 7169,   21923,  4, }, // America/Lima
    {  519, 3239,  165709, 11, }, // America/Los_Angeles
    {  519, 5932,  165720, 21, }, // America/Lower_Princes
    {  519, 8153,  122774,  6, }, // America/Maceio
    {  519, 8168,  122780,  7, }, // America/Managua
    {  519, 1908,  122787,  6, }, // America/Manaus
    {  519, 5897,   61739,  6, }, // America/Marigot
    {  519, 5805,    5142,  8, }, // America/Martinique
    {  519, 8184,  122802,  9, }, // America/Matamoros
    {  519, 2917,  165741,  8, }, // America/Mazatlan
    {  519, 8202,    7000,  8, }, // America/Menominee
    {  519, 8220,    7008,  6, }, // America/Merida
    {  519, 8235,  165749, 10, }, // America/Metlakatla
    {  519, 2949,  165759, 12, }, // America/Mexico_City
    {  519, 4391,    7031,  7, }, // America/Miquelon
    {  519, 8254,    7038,  7, }, // America/Moncton
    {  519, 8270,   61788,  8, }, // America/Monterrey
    {  519, 4098,  165771, 10, }, // America/Montevideo
    {  519, 5824,    7054,  9, }, // America/Montserrat
    {  519, 5481,  125683,  5, }, // America/Nassau
    {  519, 2379,  165781,  7, }, // America/New_York
    {  519, 8288,   21850,  3, }, // America/Nome
    {  519, 1850,    7076,  7, }, // America/Noronha
    {  519, 8301,  165788, 22, }, // America/North_Dakota/Beulah
    {  519, 8329,  165810, 23, }, // America/North_Dakota/Center
    {  519, 8357,  165833, 27, }, // America/North_Dakota/New_Salem
    {  519,  393,  165860,  6, }, // America/Nuuk
    {  519, 8388,    7158,  7, }, // America/Ojinaga
    {  519, 2356,  165866,  6, }, // America/Panama
    {  519, 6173,  165872, 10, }, // America/Paramaribo
    {  519, 2973,  125799,  6, }, // America/Phoenix
    {  519, 3945,  165882, 12, }, // America/Port-au-Prince
    {  519, 5954,  165894, 13, }, // America/Port_of_Spain
    {  519, 8404,  165907, 11, }, // America/Porto_Velho
    {  519, 5843,   33294,  9, }, // America/Puerto_Rico
    {  519, 4030,  125860, 12, }, // America/Punta_Arenas
    {  519, 8424,  125874, 12, }, // America/Rankin_Inlet
    {  519, 8445,  165918,  6, }, // America/Recife
    {  519, 1995,  165924,  8, }, // America/Regina
    {  519, 8460,  165932,  7, }, // America/Resolute
    {  519,  695,    5362, 10, }, // America/Rio_Branco
    {  519, 8477,  165939,  8, }, // America/Santarem
    {  519, 2201,  123063,  8, }, // America/Santiago
    {  519, 6111,  165947, 13, }, // America/Santo_Domingo
    {  519, 1878,  165960,  9, }, // America/Sao_Paulo
    {  519, 6899,  123091, 12, }, // America/Scoresbysund
    {  519, 8494,  165969,  5, }, // America/Sitka
    {  519, 8508,  125978, 13, }, // America/St_Barthelemy
    {  519, 2061,  125991, 10, }, // America/St_Johns
    {  519, 5863,  126003,  9, }, // America/St_Kitts
    {  519, 5880,  165974, 10, }, // America/St_Lucia
    {  519,  855,  126025, 10, }, // America/St_Thomas
    {  519, 5913,  165984, 12, }, // America/St_Vincent
    {  519, 8530,  126051, 12, }, // America/Swift_Current
    {  519, 5461,    5410, 11, }, // America/Tegucigalpa
    {  519, 5756,   22109,  3, }, // America/Thule
    {  519,  313,    7361,  7, }, // America/Tijuana
    {  519,  608,  165996,  7, }, // America/Toronto
    {  519, 5688,  166003,  7, }, // America/Tortola
    {  519, 2093,    7375,  8, }, // America/Vancouver
    {  519, 2144,  166010,  8, }, // America/Whitehorse
    {  519,  734,   62066,  7, }, // America/Winnipeg
    {  519, 8552,    7399,  7, }, // America/Yakutat
    {  519, 6001,    7406,  5, }, // Antarctica/Casey
    {  519, 6094,    7411,  6, }, // Antarctica/Davis
    {  519, 6133,  166018, 13, }, // Antarctica/DumontDUrville
    {  519, 8568,  123239,  7, }, // Antarctica/Macquarie
    {  519, 7066,  123246,  5, }, // Antarctica/Mawson
    {  519, 7113,  166031,  8, }, // Antarctica/McMurdo
    {  519, 8589,  126182,  6, }, // Antarctica/Palmer
    {  519, 7226,    5465,  6, }, // Antarctica/Rothera
    {  519, 7295,  123267,  4, }, // Antarctica/Syowa
    {  519, 8607,  123271,  4, }, // Antarctica/Troll
    {  519, 7374,  123277,  6, }, // Antarctica/Vostok
    {  519, 1429,  166039,  7, }, // Arctic/Longyearbyen
    {  519, 5570,   62104,  4, }, // Asia/Aden
    {  519, 5395,    7476,  6, }, // Asia/Almaty
    {  519, 3968,    5487,  4, }, // Asia/Amman
    {  519, 5511,    7482,  6, }, // Asia/Anadyr
    {  519, 5523,    5491,  5, }, // Asia/Aqtau
    {  519, 5368,   33303,  7, }, // Asia/Aqtobe
    {  519,  964,   33310,  7, }, // Asia/Ashgabat
    {  519, 8624,    7494,  6, }, // Asia/Atyrau
    {  519, 3607,    4044,  6, }, // Asia/Baghdad
    {  519, 5534,    4050,  7, }, // Asia/Bahrain
    {  519, 3653,   62142,  4, }, // Asia/Baku
    {  519, 4466,  123343,  7, }, // Asia/Bangkok
    {  519, 3571,  123351,  7, }, // Asia/Barnaul
    {  519, 4086,    7507,  6, }, // Asia/Beirut
    {  519, 3778,   33323,  6, }, // Asia/Bishkek
    {  519, 5989,   33329,  6, }, // Asia/Brunei
    {  519, 4601,   57490,  4, }, // Asia/Chita
    {  519, 4511,    5519,  7, }, // Asia/Colombo
    {  519, 4540,    4083,  6, }, // Asia/Damascus
    {  519, 1093,    4599,  4, }, // Asia/Dhaka
    {  519, 6192,  126338,  4, }, // Asia/Dili
    {  519, 3596,  123387,  5, }, // Asia/Dubai
    {  519, 6159,   33336,  7, }, // Asia/Dushanbe
    {  519, 8636,  166046,  9, }, // Asia/Famagusta
    {  519, 8651,   62200,  4, }, // Asia/Gaza
    {  519, 4795,   62205,  6, }, // Asia/Hebron
    {  519, 1259,   33343,  9, }, // Asia/Ho_Chi_Minh
    {  519, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  519, 4771,  123417,  4, }, // Asia/Hovd
    {  519, 4184,  147026,  6, }, // Asia/Irkutsk
    {  519, 7006,   33352,  8, }, // Asia/Jakarta
    {  519, 6992,   33360,  9, }, // Asia/Jayapura
    {  519, 1290,  166055,  9, }, // Asia/Jerusalem
    {  519, 3560,   62257,  5, }, // Asia/Kabul
    {  519, 4316,   33378,  8, }, // Asia/Kamchatka
    {  519, 4239,   33386,  6, }, // Asia/Karachi
    {  519, 1185,    5593,  8, }, // Asia/Kathmandu
    {  519, 8661,   33392,  7, }, // Asia/Khandyga
    {  519,  992,   26732,  7, }, // Asia/Kolkata
    {  519, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  519, 4859,  166064, 12, }, // Asia/Kuala_Lumpur
    {  519, 5976,   33399,  6, }, // Asia/Kuching
    {  519, 5547,    7607,  6, }, // Asia/Kuwait
    {  519, 1211,    7613,  5, }, // Asia/Macau
    {  519, 4017,  123496,  7, }, // Asia/Magadan
    {  519, 1349,    7618,  7, }, // Asia/Makassar
    {  519, 7182,  166076,  6, }, // Asia/Manila
    {  519, 6920,    4208,  6, }, // Asia/Muscat
    {  519, 2554,   33410,  8, }, // Asia/Nicosia
    {  519, 8675,   33418, 12, }, // Asia/Novokuznetsk
    {  519, 4135,   62364, 11, }, // Asia/Novosibirsk
    {  519, 4229,   62376,  4, }, // Asia/Omsk
    {  519, 7145,   62381,  4, }, // Asia/Oral
    {  519, 6961,   22386,  8, }, // Asia/Phnom_Penh
    {  519, 8693,  166082, 10, }, // Asia/Pontianak
    {  519, 4214,   33430,  9, }, // Asia/Pyongyang
    {  519, 5559,    4214,  5, }, // Asia/Qatar
    {  519, 8708,    7652,  8, }, // Asia/Qostanay
    {  519, 4269,    7660,  9, }, // Asia/Qyzylorda
    {  519, 3584,   33439,  5, }, // Asia/Riyadh
    {  519, 4408,   33444,  7, }, // Asia/Sakhalin
    {  519, 7245,   26774,  9, }, // Asia/Samarkand
    {  519, 3279,    7680,  4, }, // Asia/Seoul
    {  519, 1053,   33451,  6, }, // Asia/Shanghai
    {  519, 3300,   20782,  8, }, // Asia/Singapore
    {  519, 4297,  166092, 14, }, // Asia/Srednekolymsk
    {  519, 3263,    7712,  6, }, // Asia/Taipei
    {  519, 4781,   33457,  7, }, // Asia/Tashkent
    {  519, 3915,  126691,  7, }, // Asia/Tbilisi
    {  519, 2772,    7724,  7, }, // Asia/Tehran
    {  519, 1317,   91641,  5, }, // Asia/Thimphu
    {  519, 2821,    4238,  5, }, // Asia/Tokyo
    {  519, 4572,  123671,  5, }, // Asia/Tomsk
    {  519, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  519, 1159,  166106,  6, }, // Asia/Urumqi
    {  519, 8722,  166112,  8, }, // Asia/Ust-Nera
    {  519, 6977,   33470, 10, }, // Asia/Vientiane
    {  519, 4710,   62531, 11, }, // Asia/Vladivostok
    {  519, 4828,    7755,  7, }, // Asia/Yakutsk
    {  519, 1235,    7762,  6, }, // Asia/Yangon
    {  519, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  519, 3747,    7780,  7, }, // Asia/Yerevan
    {  519, 3663,   33480,  5, }, // Atlantic/Azores
    {  519, 5671,   74354,  7, }, // Atlantic/Bermuda
    {  519, 6628,  166120, 15, }, // Atlantic/Canary
    {  519, 3727,  166135, 18, }, // Atlantic/Cape_Verde
    {  519, 1395,  166153, 13, }, // Atlantic/Faroe
    {  519, 8736,  166166,  7, }, // Atlantic/Madeira
    {  519, 2748,   33485,  9, }, // Atlantic/Reykjavik
    {  519, 7272,  166173, 15, }, // Atlantic/South_Georgia
    {  519, 6821,  166188, 12, }, // Atlantic/St_Helena
    {  519, 6644,    7847,  6, }, // Atlantic/Stanley
    {  519, 1670,  166200,  7, }, // Australia/Adelaide
    {  519, 1635,  166207,  8, }, // Australia/Brisbane
    {  519, 1799,  126908, 10, }, // Australia/Broken_Hill
    {  519, 1583,    7880,  6, }, // Australia/Darwin
    {  519, 3637,  166215,  5, }, // Australia/Eucla
    {  519, 1516,   62688,  6, }, // Australia/Hobart
    {  519, 8753,  123873,  8, }, // Australia/Lindeman
    {  519, 1547,   22595,  8, }, // Australia/Lord_Howe
    {  519, 1727,    7903,  7, }, // Australia/Melbourne
    {  519, 1762,   22603,  4, }, // Australia/Perth
    {  519, 1463,    7915,  6, }, // Australia/Sydney
    {  519, 6441,   62715,  9, }, // Europe/Amsterdam
    {  519, 6216,   33506,  6, }, // Europe/Andorra
    {  519, 3620,    7928,  8, }, // Europe/Astrakhan
    {  519, 2300,   33520,  5, }, // Europe/Athens
    {  519, 6336,    7942,  7, }, // Europe/Belgrade
    {  519, 4757,   62753,  6, }, // Europe/Berlin
    {  519, 6488,  166220, 10, }, // Europe/Bratislava
    {  519, 2167,   33525,  6, }, // Europe/Brussels
    {  519, 3928,   33531,  8, }, // Europe/Bucharest
    {  519, 3806,   33539, 10, }, // Europe/Budapest
    {  519, 8772,  166230,  8, }, // Europe/Busingen
    {  519, 2583,   33549,  8, }, // Europe/Chisinau
    {  519, 6289,    4357, 10, }, // Europe/Copenhagen
    {  519, 2338,   33557,  6, }, // Europe/Dublin
    {  519, 6307,  127094,  9, }, // Europe/Gibraltar
    {  519, 8788,   33563,  6, }, // Europe/Guernsey
    {  519, 6612,  127112,  8, }, // Europe/Helsinki
    {  519, 8804,   33569, 10, }, // Europe/Isle_of_Man
    {  519, 1130,   62877,  8, }, // Europe/Istanbul
    {  519, 8823,  166238,  6, }, // Europe/Jersey
    {  519, 3979,   33579, 12, }, // Europe/Kaliningrad
    {  519, 8929,  124066,  5, }, // Europe/Kirov
    {  519, 2527,   33591,  5, }, // Europe/Kyiv
    {  519, 3213,   33596,  7, }, // Europe/Lisbon
    {  519, 6506,  166244,  9, }, // Europe/Ljubljana
    {  519, 2501,   62926,  6, }, // Europe/London
    {  519, 6365,    8054, 10, }, // Europe/Luxembourg
    {  519, 4892,   62944,  6, }, // Europe/Madrid
    {  519, 6397,  166253,  5, }, // Europe/Malta
    {  519, 6582,   99157,  8, }, // Europe/Mariehamn
    {  519, 3693,   62956,  5, }, // Europe/Minsk
    {  519, 6410,    5926,  6, }, // Europe/Monaco
    {  519, 3537,   22748,  6, }, // Europe/Moscow
    {  519, 6458,  166258,  4, }, // Europe/Oslo
    {  519, 4284,   33603,  5, }, // Europe/Paris
    {  519, 6424,  166262,  9, }, // Europe/Podgorica
    {  519, 6275,    8083,  4, }, // Europe/Prague
    {  519, 8837,  166271,  4, }, // Europe/Riga
    {  519, 6324,   33608,  3, }, // Europe/Rome
    {  519, 4331,  166275,  6, }, // Europe/Samara
    {  519, 6470,  166281, 10, }, // Europe/San_Marino
    {  519, 6245,  166291,  8, }, // Europe/Sarajevo
    {  519, 4451,   78231,  7, }, // Europe/Saratov
    {  519, 8849,   33611, 11, }, // Europe/Simferopol
    {  519, 6383,   33622,  7, }, // Europe/Skopje
    {  519, 6599,   33629,  6, }, // Europe/Sofia
    {  519, 6523,    8124,  8, }, // Europe/Stockholm
    {  519, 8867,    8132,  5, }, // Europe/Tallinn
    {  519, 6202,   21095,  6, }, // Europe/Tirane
    {  519, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  519, 6352,   63058,  5, }, // Europe/Vaduz
    {  519, 6567,    5978,  7, }, // Europe/Vatican
    {  519, 6231,   33643,  3, }, // Europe/Vienna
    {  519, 8899,  166299,  7, }, // Europe/Vilnius
    {  519, 4727,   63074,  9, }, // Europe/Volgograd
    {  519, 3190,   33646,  7, }, // Europe/Warsaw
    {  519, 6261,   63092,  6, }, // Europe/Zagreb
    {  519, 6540,   33653,  5, }, // Europe/Zurich
    {  519, 5078,  166306, 12, }, // Indian/Antananarivo
    {  519, 6947,  124266,  5, }, // Indian/Chagos
    {  519, 6046,   33658,  5, }, // Indian/Christmas
    {  519, 6063,    4465,  5, }, // Indian/Cocos
    {  519, 5029,   22853,  6, }, // Indian/Comoro
    {  519, 6661,   22859,  8, }, // Indian/Kerguelen
    {  519, 7260,  158670,  4, }, // Indian/Mahe
    {  519, 7050,   33687,  7, }, // Indian/Maldives
    {  519, 4069,   33694, 10, }, // Indian/Mauritius
    {  519, 5098,    8200,  5, }, // Indian/Mayotte
    {  519, 7211,   91221,  7, }, // Indian/Reunion
    {  519, 4422,  124364,  5, }, // Pacific/Apia
    {  519,  932,    8206,  6, }, // Pacific/Auckland
    {  519, 3706,  166318,  8, }, // Pacific/Bougainville
    {  519, 3015,  127475,  5, }, // Pacific/Chatham
    {  519, 3157,   91228,  4, }, // Pacific/Chuuk
    {  519, 2237,  166326, 15, }, // Pacific/Easter
    {  519, 7360,  127498,  4, }, // Pacific/Efate
    {  519, 7327,  166341,  7, }, // Pacific/Fakaofo
    {  519, 3902,   33704,  5, }, // Pacific/Fiji
    {  519, 7343,   63222,  8, }, // Pacific/Funafuti
    {  519, 6678,   92104,  9, }, // Pacific/Galapagos
    {  519, 6696,  166348,  7, }, // Pacific/Gambier
    {  519, 3822,  124440, 11, }, // Pacific/Guadalcanal
    {  519, 6018,  124452,  4, }, // Pacific/Guam
    {  519, 2723,    4493,  8, }, // Pacific/Honolulu
    {  519, 3049,  124457,  9, }, // Pacific/Kanton
    {  519, 3998,   92138, 10, }, // Pacific/Kiritimati
    {  519, 7019,  166355,  5, }, // Pacific/Kosrae
    {  519, 2842,  166360, 11, }, // Pacific/Kwajalein
    {  519, 4877,   92186,  6, }, // Pacific/Majuro
    {  519, 4051,   36208,  6, }, // Pacific/Marquesas
    {  519, 8914,   63258,  6, }, // Pacific/Midway
    {  519, 7084,  166371,  5, }, // Pacific/Nauru
    {  519, 7132,  166376,  5, }, // Pacific/Niue
    {  519, 4168,   63280,  7, }, // Pacific/Norfolk
    {  519, 7098,  124531,  5, }, // Pacific/Noumea
    {  519, 3126,  166381,  9, }, // Pacific/Pago_Pago
    {  519, 7155,   65647,  5, }, // Pacific/Palau
    {  519, 7194,    8304,  7, }, // Pacific/Pitcairn
    {  519, 3096,    8311,  6, }, // Pacific/Pohnpei
    {  519, 4807,  166390, 11, }, // Pacific/Port_Moresby
    {  519, 6076,  166401,  9, }, // Pacific/Rarotonga
    {  519, 6031,  166410,  6, }, // Pacific/Saipan
    {  519, 7312,   63343,  6, }, // Pacific/Tahiti
    {  519, 6712,   23027,  6, }, // Pacific/Tarawa
    {  519, 4583,  166416,  9, }, // Pacific/Tongatapu
    {  519, 7392,   63356,  4, }, // Pacific/Wake
    {  519, 7405,   33709,  5, }, // Pacific/Wallis
    {  520, 6788,   57564,  7, }, // Africa/Abidjan Serbian/Latin/Kosovo
    {  520, 6760,    4531,  4, }, // Africa/Accra
    {  520, 5059,   35689, 10, }, // Africa/Addis_Ababa
    {  520, 7420,   57447,  5, }, // Africa/Algiers
    {  520,   14,   91122,  6, }, // Africa/Asmara
    {  520,   44,  165212,  6, }, // Africa/Bamako
    {  520, 5257,  165218,  6, }, // Africa/Bangui
    {  520, 6746,  165224,  6, }, // Africa/Banjul
    {  520, 7435,  165230,  5, }, // Africa/Bissau
    {  520, 4957,    6200,  7, }, // Africa/Blantyre
    {  520, 5287,    6208,  8, }, // Africa/Brazzaville
    {  520, 4922,  165235, 10, }, // Africa/Bujumbura
    {  520, 2320,    4572,  5, }, // Africa/Cairo
    {  520, 4117,   33144, 10, }, // Africa/Casablanca
    {  520, 7449,   21231,  5, }, // Africa/Ceuta
    {  520, 6773,    6249,  7, }, // Africa/Conakry
    {  520, 6840,   61029,  5, }, // Africa/Dakar
    {  520, 5130,    6256, 12, }, // Africa/Dar_es_Salaam
    {  520, 5043,   33154,  7, }, // Africa/Djibouti
    {  520, 5243,   21254,  5, }, // Africa/Douala
    {  520, 5166,    6275,  7, }, // Africa/El_Aaiun
    {  520, 6853,    6282,  7, }, // Africa/Freetown
    {  520, 4906,   61071,  7, }, // Africa/Gaborone
    {  520, 5015,   61079,  6, }, // Africa/Harare
    {  520, 4479,    6289, 11, }, // Africa/Johannesburg
    {  520, 4499,  124745,  5, }, // Africa/Juba
    {  520, 5151,  165245,  7, }, // Africa/Kampala
    {  520, 4524,    6305,  6, }, // Africa/Khartoum
    {  520, 4973,  124759,  6, }, // Africa/Kigali
    {  520, 5306,  122029,  7, }, // Africa/Kinshasa
    {  520, 4744,   61130,  5, }, // Africa/Lagos
    {  520, 5336,    6311,  8, }, // Africa/Libreville
    {  520, 6869,   61145,  4, }, // Africa/Lome
    {  520, 5211,  165252,  6, }, // Africa/Luanda
    {  520, 4939,  122051,  9, }, // Africa/Lubumbashi
    {  520, 5001,  165258,  6, }, // Africa/Lusaka
    {  520, 5322,  165264,  6, }, // Africa/Malabo
    {  520, 4987,  165270,  6, }, // Africa/Maputo
    {  520, 5197,   61171,  6, }, // Africa/Maseru
    {  520, 5182,  165276,  7, }, // Africa/Mbabane
    {  520, 5113,   33161,  7, }, // Africa/Mogadishu
    {  520, 7034,  122074,  9, }, // Africa/Monrovia
    {  520, 3853,   61201,  7, }, // Africa/Nairobi
    {  520, 5271,   57597,  8, }, // Africa/Ndjamena
    {  520, 5354,  165283,  7, }, // Africa/Niamey
    {  520, 6803,   57605,  7, }, // Africa/Nouakchott
    {  520, 6727,   26402,  8, }, // Africa/Ouagadougou
    {  520, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  520, 4435,   91128,  8, }, // Africa/Sao_Tome
    {  520, 2866,   35756,  7, }, // Africa/Tripoli
    {  520, 6554,   91301,  5, }, // Africa/Tunis
    {  520, 4152,   21354,  7, }, // Africa/Windhoek
    {  520,  132,   97814,  4, }, // America/Adak
    {  520, 3346,  165290,  8, }, // America/Anchorage
    {  520, 5607,   33168,  7, }, // America/Anguilla
    {  520, 5624,   33175,  7, }, // America/Antigua
    {  520, 4554,  124912, 10, }, // America/Araguaina
    {  520,  166,  165298, 12, }, // America/Argentina/Buenos_Aires
    {  520,   91,    6400,  9, }, // America/Argentina/Catamarca
    {  520,  270,    6409,  7, }, // America/Argentina/Cordoba
    {  520,  441,  165310,  5, }, // America/Argentina/Jujuy
    {  520, 7462,    6421,  8, }, // America/Argentina/La_Rioja
    {  520,  565,   97505,  7, }, // America/Argentina/Mendoza
    {  520, 7489,  124952, 12, }, // America/Argentina/Rio_Gallegos
    {  520, 7520,   61307,  5, }, // America/Argentina/Salta
    {  520, 7544,  165315,  8, }, // America/Argentina/San_Juan
    {  520, 5580,  165323,  7, }, // America/Argentina/San_Luis
    {  520, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {  520, 7597,  165330,  7, }, // America/Argentina/Ushuaia
    {  520, 5640,  165337,  5, }, // America/Aruba
    {  520, 4252,    6474,  8, }, // America/Asuncion
    {  520,  237,  165342, 12, }, // America/Atikokan
    {  520, 3679,  122240,  5, }, // America/Bahia
    {  520, 7623,  165354, 14, }, // America/Bahia_Banderas
    {  520, 5654,  122258,  8, }, // America/Barbados
    {  520, 7646,   61342,  5, }, // America/Belem
    {  520, 5407,  165368,  6, }, // America/Belize
    {  520, 7660,  165374, 13, }, // America/Blanc-Sablon
    {  520, 7681,  165387,  9, }, // America/Boa_Vista
    {  520, 4361,  165396,  6, }, // America/Bogota
    {  520, 7699,  165402,  5, }, // America/Boise
    {  520, 7713,  165407, 12, }, // America/Cambridge_Bay
    {  520, 7735,    4780, 12, }, // America/Campo_Grande
    {  520, 3868,    6564,  6, }, // America/Cancun
    {  520, 4694,    4792,  7, }, // America/Caracas
    {  520, 4345,    6570,  5, }, // America/Cayenne
    {  520, 5496,  165419, 16, }, // America/Cayman
    {  520, 2260,  165435,  6, }, // America/Chicago
    {  520, 7756,  125151,  8, }, // America/Chihuahua
    {  520, 7774,  165441, 13, }, // America/Ciudad_Juarez
    {  520, 5422,   33188,  9, }, // America/Costa_Rica
    {  520, 7796,    6618,  7, }, // America/Creston
    {  520, 3791,    4846,  6, }, // America/Cuiaba
    {  520, 5723,    6631,  7, }, // America/Curacao
    {  520, 7812,  165454, 13, }, // America/Danmarkshavn
    {  520, 7833,  125206,  5, }, // America/Dawson
    {  520, 7848,  125213, 10, }, // America/Dawson_Creek
    {  520,  805,   61425,  6, }, // America/Denver
    {  520, 3465,  122438,  7, }, // America/Detroit
    {  520, 5739,   21548,  8, }, // America/Dominica
    {  520,  893,   61448,  8, }, // America/Edmonton
    {  520, 7869,  165467,  8, }, // America/Eirunepe
    {  520, 5441,    6688,  8, }, // America/El_Salvador
    {  520, 7886,  125270, 11, }, // America/Fort_Nelson
    {  520, 7906,  165475,  9, }, // America/Fortaleza
    {  520, 7924,  165484,  9, }, // America/Glace_Bay
    {  520, 6881,  165493,  7, }, // America/Goose_Bay
    {  520, 4612,  165500, 10, }, // America/Grand_Turk
    {  520, 5770,  165510,  7, }, // America/Grenada
    {  520, 5786,   33197,  9, }, // America/Guadeloupe
    {  520, 3760,   33206,  9, }, // America/Guatemala
    {  520, 4841,  122518,  8, }, // America/Guayaquil
    {  520, 6932,   57452,  7, }, // America/Guyana
    {  520, 1939,    6787,  8, }, // America/Halifax
    {  520, 2281,  147659,  6, }, // America/Havana
    {  520, 7942,  165517, 10, }, // America/Hermosillo
    {  520,  348,  122551, 12, }, // America/Indiana/Indianapolis
    {  520,  481,  165540, 14, }, // America/Indiana/Knox
    {  520, 7961,  165554, 17, }, // America/Indiana/Marengo
    {  520, 7985,  165571, 20, }, // America/Indiana/Petersburg
    {  520, 8012,  165591, 18, }, // America/Indiana/Tell_City
    {  520, 8038,  166425, 15, }, // America/Indiana/Vevay
    {  520, 8060,  166440, 18, }, // America/Indiana/Vincennes
    {  520, 8086,  165640, 17, }, // America/Indiana/Winamac
    {  520, 8110,  122681,  6, }, // America/Inuvik
    {  520,  660,  165657,  8, }, // America/Iqaluit
    {  520, 2799,   33215,  7, }, // America/Jamaica
    {  520, 5380,  165665,  4, }, // America/Juneau
    {  520,  521,  166458,  7, }, // America/Kentucky/Louisville
    {  520, 8125,  165675, 18, }, // America/Kentucky/Monticello
    {  520, 5704,  165693, 10, }, // America/Kralendijk
    {  520, 4376,  165703,  6, }, // America/La_Paz
    {  520, 7169,   21923,  4, }, // America/Lima
    {  520, 3239,  165709, 11, }, // America/Los_Angeles
    {  520, 5932,  165720, 21, }, // America/Lower_Princes
    {  520, 8153,  122774,  6, }, // America/Maceio
    {  520, 8168,  122780,  7, }, // America/Managua
    {  520, 1908,  122787,  6, }, // America/Manaus
    {  520, 5897,   61739,  6, }, // America/Marigot
    {  520, 5805,    5142,  8, }, // America/Martinique
    {  520, 8184,  122802,  9, }, // America/Matamoros
    {  520, 2917,  165741,  8, }, // America/Mazatlan
    {  520, 8202,    7000,  8, }, // America/Menominee
    {  520, 8220,    7008,  6, }, // America/Merida
    {  520, 8235,  165749, 10, }, // America/Metlakatla
    {  520, 2949,  165759, 12, }, // America/Mexico_City
    {  520, 4391,    7031,  7, }, // America/Miquelon
    {  520, 8254,    7038,  7, }, // America/Moncton
    {  520, 8270,   61788,  8, }, // America/Monterrey
    {  520, 4098,  165771, 10, }, // America/Montevideo
    {  520, 5824,    7054,  9, }, // America/Montserrat
    {  520, 5481,  125683,  5, }, // America/Nassau
    {  520, 2379,  165781,  7, }, // America/New_York
    {  520, 8288,   21850,  3, }, // America/Nome
    {  520, 1850,    7076,  7, }, // America/Noronha
    {  520, 8301,  166465, 22, }, // America/North_Dakota/Beulah
    {  520, 8329,  166487, 22, }, // America/North_Dakota/Center
    {  520, 8357,  166509, 26, }, // America/North_Dakota/New_Salem
    {  520,  393,  165860,  6, }, // America/Nuuk
    {  520, 8388,    7158,  7, }, // America/Ojinaga
    {  520, 2356,  165866,  6, }, // America/Panama
    {  520, 6173,  165872, 10, }, // America/Paramaribo
    {  520, 2973,  125799,  6, }, // America/Phoenix
    {  520, 3945,  125807, 12, }, // America/Port-au-Prince
    {  520, 5954,  166535, 13, }, // America/Port_of_Spain
    {  520, 8404,  165907, 11, }, // America/Porto_Velho
    {  520, 5843,  166548, 10, }, // America/Puerto_Rico
    {  520, 4030,  125860, 12, }, // America/Punta_Arenas
    {  520, 8424,  125874, 12, }, // America/Rankin_Inlet
    {  520, 8445,  165918,  6, }, // America/Recife
    {  520, 1995,  166558,  6, }, // America/Regina
    {  520, 8460,  125901,  7, }, // America/Resolute
    {  520,  695,    5362, 10, }, // America/Rio_Branco
    {  520, 8477,  165939,  8, }, // America/Santarem
    {  520, 2201,  123063,  8, }, // America/Santiago
    {  520, 6111,  165947, 13, }, // America/Santo_Domingo
    {  520, 1878,  165960,  9, }, // America/Sao_Paulo
    {  520, 6899,  166564, 12, }, // America/Scoresbysund
    {  520, 8494,  165969,  5, }, // America/Sitka
    {  520, 8508,  166576, 14, }, // America/St_Barthelemy
    {  520, 2061,  166590,  8, }, // America/St_Johns
    {  520, 5863,  126003,  9, }, // America/St_Kitts
    {  520, 5880,  165974, 10, }, // America/St_Lucia
    {  520,  855,  166598,  8, }, // America/St_Thomas
    {  520, 5913,  165984, 12, }, // America/St_Vincent
    {  520, 8530,   22086, 12, }, // America/Swift_Current
    {  520, 5461,    5410, 11, }, // America/Tegucigalpa
    {  520, 5756,   22109,  3, }, // America/Thule
    {  520,  313,    7361,  7, }, // America/Tijuana
    {  520,  608,  165996,  7, }, // America/Toronto
    {  520, 5688,  166003,  7, }, // America/Tortola
    {  520, 2093,    7375,  8, }, // America/Vancouver
    {  520, 2144,  166010,  8, }, // America/Whitehorse
    {  520,  734,   62066,  7, }, // America/Winnipeg
    {  520, 8552,    7399,  7, }, // America/Yakutat
    {  520, 6001,    7406,  5, }, // Antarctica/Casey
    {  520, 6094,    7411,  6, }, // Antarctica/Davis
    {  520, 6133,  166606, 13, }, // Antarctica/DumontDUrville
    {  520, 8568,  166619,  7, }, // Antarctica/Macquarie
    {  520, 7066,  123246,  5, }, // Antarctica/Mawson
    {  520, 7113,  166031,  8, }, // Antarctica/McMurdo
    {  520, 8589,  126182,  6, }, // Antarctica/Palmer
    {  520, 7226,    5465,  6, }, // Antarctica/Rothera
    {  520, 7295,  123267,  4, }, // Antarctica/Syowa
    {  520, 8607,  123271,  4, }, // Antarctica/Troll
    {  520, 7374,  123277,  6, }, // Antarctica/Vostok
    {  520, 1429,  166626, 11, }, // Arctic/Longyearbyen
    {  520, 5570,   62104,  4, }, // Asia/Aden
    {  520, 5395,    7476,  6, }, // Asia/Almaty
    {  520, 3968,    5487,  4, }, // Asia/Amman
    {  520, 5511,    7482,  6, }, // Asia/Anadyr
    {  520, 5523,    5491,  5, }, // Asia/Aqtau
    {  520, 5368,   33303,  7, }, // Asia/Aqtobe
    {  520,  964,   33310,  7, }, // Asia/Ashgabat
    {  520, 8624,    7494,  6, }, // Asia/Atyrau
    {  520, 3607,    4044,  6, }, // Asia/Baghdad
    {  520, 5534,    4050,  7, }, // Asia/Bahrain
    {  520, 3653,   62142,  4, }, // Asia/Baku
    {  520, 4466,  123343,  7, }, // Asia/Bangkok
    {  520, 3571,  123351,  7, }, // Asia/Barnaul
    {  520, 4086,    7507,  6, }, // Asia/Beirut
    {  520, 3778,   33323,  6, }, // Asia/Bishkek
    {  520, 5989,   33329,  6, }, // Asia/Brunei
    {  520, 4601,   57490,  4, }, // Asia/Chita
    {  520, 4511,    5519,  7, }, // Asia/Colombo
    {  520, 4540,    4083,  6, }, // Asia/Damascus
    {  520, 1093,    4599,  4, }, // Asia/Dhaka
    {  520, 6192,  126338,  4, }, // Asia/Dili
    {  520, 3596,  123387,  5, }, // Asia/Dubai
    {  520, 6159,   33336,  7, }, // Asia/Dushanbe
    {  520, 8636,  166046,  9, }, // Asia/Famagusta
    {  520, 8651,   62200,  4, }, // Asia/Gaza
    {  520, 4795,   62205,  6, }, // Asia/Hebron
    {  520, 1259,   33343,  9, }, // Asia/Ho_Chi_Minh
    {  520, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  520, 4771,  123417,  4, }, // Asia/Hovd
    {  520, 4184,  147026,  6, }, // Asia/Irkutsk
    {  520, 7006,   33352,  8, }, // Asia/Jakarta
    {  520, 6992,   33360,  9, }, // Asia/Jayapura
    {  520, 1290,  166055,  9, }, // Asia/Jerusalem
    {  520, 3560,   62257,  5, }, // Asia/Kabul
    {  520, 4316,   33378,  8, }, // Asia/Kamchatka
    {  520, 4239,   33386,  6, }, // Asia/Karachi
    {  520, 1185,    5593,  8, }, // Asia/Kathmandu
    {  520, 8661,   33392,  7, }, // Asia/Khandyga
    {  520,  992,   26732,  7, }, // Asia/Kolkata
    {  520, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  520, 4859,  166064, 12, }, // Asia/Kuala_Lumpur
    {  520, 5976,   33399,  6, }, // Asia/Kuching
    {  520, 5547,    7607,  6, }, // Asia/Kuwait
    {  520, 1211,    7613,  5, }, // Asia/Macau
    {  520, 4017,  123496,  7, }, // Asia/Magadan
    {  520, 1349,    7618,  7, }, // Asia/Makassar
    {  520, 7182,  166076,  6, }, // Asia/Manila
    {  520, 6920,    4208,  6, }, // Asia/Muscat
    {  520, 2554,   33410,  8, }, // Asia/Nicosia
    {  520, 8675,   33418, 12, }, // Asia/Novokuznetsk
    {  520, 4135,   62364, 11, }, // Asia/Novosibirsk
    {  520, 4229,   62376,  4, }, // Asia/Omsk
    {  520, 7145,   62381,  4, }, // Asia/Oral
    {  520, 6961,   22386,  8, }, // Asia/Phnom_Penh
    {  520, 8693,  166082, 10, }, // Asia/Pontianak
    {  520, 4214,   33430,  9, }, // Asia/Pyongyang
    {  520, 5559,    4214,  5, }, // Asia/Qatar
    {  520, 8708,    7652,  8, }, // Asia/Qostanay
    {  520, 4269,    7660,  9, }, // Asia/Qyzylorda
    {  520, 3584,   33439,  5, }, // Asia/Riyadh
    {  520, 4408,   33444,  7, }, // Asia/Sakhalin
    {  520, 7245,   26774,  9, }, // Asia/Samarkand
    {  520, 3279,    7680,  4, }, // Asia/Seoul
    {  520, 1053,   33451,  6, }, // Asia/Shanghai
    {  520, 3300,   20782,  8, }, // Asia/Singapore
    {  520, 4297,  166092, 14, }, // Asia/Srednekolymsk
    {  520, 3263,    7712,  6, }, // Asia/Taipei
    {  520, 4781,   33457,  7, }, // Asia/Tashkent
    {  520, 3915,  126691,  7, }, // Asia/Tbilisi
    {  520, 2772,    7724,  7, }, // Asia/Tehran
    {  520, 1317,   91641,  5, }, // Asia/Thimphu
    {  520, 2821,    4238,  5, }, // Asia/Tokyo
    {  520, 4572,  123671,  5, }, // Asia/Tomsk
    {  520, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  520, 1159,  166106,  6, }, // Asia/Urumqi
    {  520, 8722,  166112,  8, }, // Asia/Ust-Nera
    {  520, 6977,   33470, 10, }, // Asia/Vientiane
    {  520, 4710,   62531, 11, }, // Asia/Vladivostok
    {  520, 4828,    7755,  7, }, // Asia/Yakutsk
    {  520, 1235,    7762,  6, }, // Asia/Yangon
    {  520, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  520, 3747,    7780,  7, }, // Asia/Yerevan
    {  520, 3663,   33480,  5, }, // Atlantic/Azores
    {  520, 5671,   74354,  7, }, // Atlantic/Bermuda
    {  520, 6628,  166120, 15, }, // Atlantic/Canary
    {  520, 3727,  166135, 18, }, // Atlantic/Cape_Verde
    {  520, 1395,  166153, 13, }, // Atlantic/Faroe
    {  520, 8736,  166166,  7, }, // Atlantic/Madeira
    {  520, 2748,   33485,  9, }, // Atlantic/Reykjavik
    {  520, 7272,  166173, 15, }, // Atlantic/South_Georgia
    {  520, 6821,  166188, 12, }, // Atlantic/St_Helena
    {  520, 6644,    7847,  6, }, // Atlantic/Stanley
    {  520, 1670,  166200,  7, }, // Australia/Adelaide
    {  520, 1635,  166207,  8, }, // Australia/Brisbane
    {  520, 1799,  126908, 10, }, // Australia/Broken_Hill
    {  520, 1583,    7880,  6, }, // Australia/Darwin
    {  520, 3637,  166215,  5, }, // Australia/Eucla
    {  520, 1516,   62688,  6, }, // Australia/Hobart
    {  520, 8753,  123873,  8, }, // Australia/Lindeman
    {  520, 1547,   22595,  8, }, // Australia/Lord_Howe
    {  520, 1727,    7903,  7, }, // Australia/Melbourne
    {  520, 1762,   22603,  4, }, // Australia/Perth
    {  520, 1463,    7915,  6, }, // Australia/Sydney
    {  520, 6441,   62715,  9, }, // Europe/Amsterdam
    {  520, 6216,   33506,  6, }, // Europe/Andorra
    {  520, 3620,    7928,  8, }, // Europe/Astrakhan
    {  520, 2300,   33520,  5, }, // Europe/Athens
    {  520, 6336,    7942,  7, }, // Europe/Belgrade
    {  520, 4757,   62753,  6, }, // Europe/Berlin
    {  520, 6488,  166220, 10, }, // Europe/Bratislava
    {  520, 2167,   33525,  6, }, // Europe/Brussels
    {  520, 3928,   33531,  8, }, // Europe/Bucharest
    {  520, 3806,   33539, 10, }, // Europe/Budapest
    {  520, 8772,  166230,  8, }, // Europe/Busingen
    {  520, 2583,   33549,  8, }, // Europe/Chisinau
    {  520, 6289,    4357, 10, }, // Europe/Copenhagen
    {  520, 2338,   33557,  6, }, // Europe/Dublin
    {  520, 6307,  127094,  9, }, // Europe/Gibraltar
    {  520, 8788,   33563,  6, }, // Europe/Guernsey
    {  520, 6612,  127112,  8, }, // Europe/Helsinki
    {  520, 8804,   33569, 10, }, // Europe/Isle_of_Man
    {  520, 1130,   62877,  8, }, // Europe/Istanbul
    {  520, 8823,  166238,  6, }, // Europe/Jersey
    {  520, 3979,   33579, 12, }, // Europe/Kaliningrad
    {  520, 8929,  124066,  5, }, // Europe/Kirov
    {  520, 2527,   33591,  5, }, // Europe/Kyiv
    {  520, 3213,   33596,  7, }, // Europe/Lisbon
    {  520, 6506,  166244,  9, }, // Europe/Ljubljana
    {  520, 2501,   62926,  6, }, // Europe/London
    {  520, 6365,    8054, 10, }, // Europe/Luxembourg
    {  520, 4892,   62944,  6, }, // Europe/Madrid
    {  520, 6397,  166253,  5, }, // Europe/Malta
    {  520, 6582,   99157,  8, }, // Europe/Mariehamn
    {  520, 3693,   62956,  5, }, // Europe/Minsk
    {  520, 6410,    5926,  6, }, // Europe/Monaco
    {  520, 3537,   22748,  6, }, // Europe/Moscow
    {  520, 6458,  166258,  4, }, // Europe/Oslo
    {  520, 4284,   33603,  5, }, // Europe/Paris
    {  520, 6424,  166262,  9, }, // Europe/Podgorica
    {  520, 6275,    8083,  4, }, // Europe/Prague
    {  520, 8837,  166271,  4, }, // Europe/Riga
    {  520, 6324,   33608,  3, }, // Europe/Rome
    {  520, 4331,  166275,  6, }, // Europe/Samara
    {  520, 6470,  166281, 10, }, // Europe/San_Marino
    {  520, 6245,  166291,  8, }, // Europe/Sarajevo
    {  520, 4451,   78231,  7, }, // Europe/Saratov
    {  520, 8849,   33611, 11, }, // Europe/Simferopol
    {  520, 6383,   33622,  7, }, // Europe/Skopje
    {  520, 6599,   33629,  6, }, // Europe/Sofia
    {  520, 6523,    8124,  8, }, // Europe/Stockholm
    {  520, 8867,    8132,  5, }, // Europe/Tallinn
    {  520, 6202,   21095,  6, }, // Europe/Tirane
    {  520, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  520, 6352,   63058,  5, }, // Europe/Vaduz
    {  520, 6567,    5978,  7, }, // Europe/Vatican
    {  520, 6231,   33643,  3, }, // Europe/Vienna
    {  520, 8899,  166299,  7, }, // Europe/Vilnius
    {  520, 4727,   63074,  9, }, // Europe/Volgograd
    {  520, 3190,   33646,  7, }, // Europe/Warsaw
    {  520, 6261,   63092,  6, }, // Europe/Zagreb
    {  520, 6540,   33653,  5, }, // Europe/Zurich
    {  520, 5078,  166306, 12, }, // Indian/Antananarivo
    {  520, 6947,  124266,  5, }, // Indian/Chagos
    {  520, 6046,   33658,  5, }, // Indian/Christmas
    {  520, 6063,    4465,  5, }, // Indian/Cocos
    {  520, 5029,   22853,  6, }, // Indian/Comoro
    {  520, 6661,   22859,  8, }, // Indian/Kerguelen
    {  520, 7260,  158670,  4, }, // Indian/Mahe
    {  520, 7050,   33687,  7, }, // Indian/Maldives
    {  520, 4069,   33694, 10, }, // Indian/Mauritius
    {  520, 5098,    8200,  5, }, // Indian/Mayotte
    {  520, 7211,   91221,  7, }, // Indian/Reunion
    {  520, 4422,  124364,  5, }, // Pacific/Apia
    {  520,  932,    8206,  6, }, // Pacific/Auckland
    {  520, 3706,  166318,  8, }, // Pacific/Bougainville
    {  520, 3015,  127475,  5, }, // Pacific/Chatham
    {  520, 3157,   91228,  4, }, // Pacific/Chuuk
    {  520, 2237,  166326, 15, }, // Pacific/Easter
    {  520, 7360,  127498,  4, }, // Pacific/Efate
    {  520, 7327,  166341,  7, }, // Pacific/Fakaofo
    {  520, 3902,   33704,  5, }, // Pacific/Fiji
    {  520, 7343,   63222,  8, }, // Pacific/Funafuti
    {  520, 6678,   92104,  9, }, // Pacific/Galapagos
    {  520, 6696,  166348,  7, }, // Pacific/Gambier
    {  520, 3822,  124440, 11, }, // Pacific/Guadalcanal
    {  520, 6018,  124452,  4, }, // Pacific/Guam
    {  520, 2723,    4493,  8, }, // Pacific/Honolulu
    {  520, 3049,  124457,  9, }, // Pacific/Kanton
    {  520, 3998,   92138, 10, }, // Pacific/Kiritimati
    {  520, 7019,  166355,  5, }, // Pacific/Kosrae
    {  520, 2842,  166360, 11, }, // Pacific/Kwajalein
    {  520, 4877,   92186,  6, }, // Pacific/Majuro
    {  520, 4051,   36208,  6, }, // Pacific/Marquesas
    {  520, 8914,   63258,  6, }, // Pacific/Midway
    {  520, 7084,  166371,  5, }, // Pacific/Nauru
    {  520, 7132,  166637,  4, }, // Pacific/Niue
    {  520, 4168,   63280,  7, }, // Pacific/Norfolk
    {  520, 7098,  124531,  5, }, // Pacific/Noumea
    {  520, 3126,  166381,  9, }, // Pacific/Pago_Pago
    {  520, 7155,   65647,  5, }, // Pacific/Palau
    {  520, 7194,    8304,  7, }, // Pacific/Pitcairn
    {  520, 3096,    8311,  6, }, // Pacific/Pohnpei
    {  520, 4807,  166390, 11, }, // Pacific/Port_Moresby
    {  520, 6076,  166401,  9, }, // Pacific/Rarotonga
    {  520, 6031,  166410,  6, }, // Pacific/Saipan
    {  520, 7312,   63343,  6, }, // Pacific/Tahiti
    {  520, 6712,   23027,  6, }, // Pacific/Tarawa
    {  520, 4583,  166416,  9, }, // Pacific/Tongatapu
    {  520, 7392,   63356,  4, }, // Pacific/Wake
    {  520, 7405,   33709,  5, }, // Pacific/Wallis
    {  521, 6788,   57564,  7, }, // Africa/Abidjan Serbian/Latin/Montenegro
    {  521, 6760,    4531,  4, }, // Africa/Accra
    {  521, 5059,   35689, 10, }, // Africa/Addis_Ababa
    {  521, 7420,   57447,  5, }, // Africa/Algiers
    {  521,   14,   91122,  6, }, // Africa/Asmara
    {  521,   44,  165212,  6, }, // Africa/Bamako
    {  521, 5257,  165218,  6, }, // Africa/Bangui
    {  521, 6746,  165224,  6, }, // Africa/Banjul
    {  521, 7435,  165230,  5, }, // Africa/Bissau
    {  521, 4957,    6200,  7, }, // Africa/Blantyre
    {  521, 5287,    6208,  8, }, // Africa/Brazzaville
    {  521, 4922,  165235, 10, }, // Africa/Bujumbura
    {  521, 2320,    4572,  5, }, // Africa/Cairo
    {  521, 4117,   33144, 10, }, // Africa/Casablanca
    {  521, 7449,   21231,  5, }, // Africa/Ceuta
    {  521, 6773,    6249,  7, }, // Africa/Conakry
    {  521, 6840,   61029,  5, }, // Africa/Dakar
    {  521, 5130,    6256, 12, }, // Africa/Dar_es_Salaam
    {  521, 5043,   33154,  7, }, // Africa/Djibouti
    {  521, 5243,   21254,  5, }, // Africa/Douala
    {  521, 5166,    6275,  7, }, // Africa/El_Aaiun
    {  521, 6853,    6282,  7, }, // Africa/Freetown
    {  521, 4906,   61071,  7, }, // Africa/Gaborone
    {  521, 5015,   61079,  6, }, // Africa/Harare
    {  521, 4479,    6289, 11, }, // Africa/Johannesburg
    {  521, 4499,  124745,  5, }, // Africa/Juba
    {  521, 5151,  165245,  7, }, // Africa/Kampala
    {  521, 4524,    6305,  6, }, // Africa/Khartoum
    {  521, 4973,  124759,  6, }, // Africa/Kigali
    {  521, 5306,  122029,  7, }, // Africa/Kinshasa
    {  521, 4744,   61130,  5, }, // Africa/Lagos
    {  521, 5336,    6311,  8, }, // Africa/Libreville
    {  521, 6869,   61145,  4, }, // Africa/Lome
    {  521, 5211,  165252,  6, }, // Africa/Luanda
    {  521, 4939,  122051,  9, }, // Africa/Lubumbashi
    {  521, 5001,  165258,  6, }, // Africa/Lusaka
    {  521, 5322,  165264,  6, }, // Africa/Malabo
    {  521, 4987,  165270,  6, }, // Africa/Maputo
    {  521, 5197,   61171,  6, }, // Africa/Maseru
    {  521, 5182,  165276,  7, }, // Africa/Mbabane
    {  521, 5113,   33161,  7, }, // Africa/Mogadishu
    {  521, 7034,  122074,  9, }, // Africa/Monrovia
    {  521, 3853,   61201,  7, }, // Africa/Nairobi
    {  521, 5271,   57597,  8, }, // Africa/Ndjamena
    {  521, 5354,  165283,  7, }, // Africa/Niamey
    {  521, 6803,   57605,  7, }, // Africa/Nouakchott
    {  521, 6727,   26402,  8, }, // Africa/Ouagadougou
    {  521, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  521, 4435,   91128,  8, }, // Africa/Sao_Tome
    {  521, 2866,   35756,  7, }, // Africa/Tripoli
    {  521, 6554,   91301,  5, }, // Africa/Tunis
    {  521, 4152,   21354,  7, }, // Africa/Windhoek
    {  521,  132,   97814,  4, }, // America/Adak
    {  521, 3346,  165290,  8, }, // America/Anchorage
    {  521, 5607,   33168,  7, }, // America/Anguilla
    {  521, 5624,   33175,  7, }, // America/Antigua
    {  521, 4554,  124912, 10, }, // America/Araguaina
    {  521,  166,  165298, 12, }, // America/Argentina/Buenos_Aires
    {  521,   91,    6400,  9, }, // America/Argentina/Catamarca
    {  521,  270,    6409,  7, }, // America/Argentina/Cordoba
    {  521,  441,  165310,  5, }, // America/Argentina/Jujuy
    {  521, 7462,    6421,  8, }, // America/Argentina/La_Rioja
    {  521,  565,   97505,  7, }, // America/Argentina/Mendoza
    {  521, 7489,  124952, 12, }, // America/Argentina/Rio_Gallegos
    {  521, 7520,   61307,  5, }, // America/Argentina/Salta
    {  521, 7544,  165315,  8, }, // America/Argentina/San_Juan
    {  521, 5580,  165323,  7, }, // America/Argentina/San_Luis
    {  521, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {  521, 7597,  165330,  7, }, // America/Argentina/Ushuaia
    {  521, 5640,  165337,  5, }, // America/Aruba
    {  521, 4252,    6474,  8, }, // America/Asuncion
    {  521,  237,  165342, 12, }, // America/Atikokan
    {  521, 3679,  122240,  5, }, // America/Bahia
    {  521, 7623,  165354, 14, }, // America/Bahia_Banderas
    {  521, 5654,  122258,  8, }, // America/Barbados
    {  521, 7646,   61342,  5, }, // America/Belem
    {  521, 5407,  165368,  6, }, // America/Belize
    {  521, 7660,  165374, 13, }, // America/Blanc-Sablon
    {  521, 7681,  165387,  9, }, // America/Boa_Vista
    {  521, 4361,  165396,  6, }, // America/Bogota
    {  521, 7699,  165402,  5, }, // America/Boise
    {  521, 7713,  165407, 12, }, // America/Cambridge_Bay
    {  521, 7735,    4780, 12, }, // America/Campo_Grande
    {  521, 3868,    6564,  6, }, // America/Cancun
    {  521, 4694,    4792,  7, }, // America/Caracas
    {  521, 4345,    6570,  5, }, // America/Cayenne
    {  521, 5496,  165419, 16, }, // America/Cayman
    {  521, 2260,  165435,  6, }, // America/Chicago
    {  521, 7756,  125151,  8, }, // America/Chihuahua
    {  521, 7774,  165441, 13, }, // America/Ciudad_Juarez
    {  521, 5422,   33188,  9, }, // America/Costa_Rica
    {  521, 7796,    6618,  7, }, // America/Creston
    {  521, 3791,    4846,  6, }, // America/Cuiaba
    {  521, 5723,    6631,  7, }, // America/Curacao
    {  521, 7812,  165454, 13, }, // America/Danmarkshavn
    {  521, 7833,  125206,  5, }, // America/Dawson
    {  521, 7848,  125213, 10, }, // America/Dawson_Creek
    {  521,  805,   61425,  6, }, // America/Denver
    {  521, 3465,  122438,  7, }, // America/Detroit
    {  521, 5739,   21548,  8, }, // America/Dominica
    {  521,  893,   61448,  8, }, // America/Edmonton
    {  521, 7869,  165467,  8, }, // America/Eirunepe
    {  521, 5441,    6688,  8, }, // America/El_Salvador
    {  521, 7886,  125270, 11, }, // America/Fort_Nelson
    {  521, 7906,  165475,  9, }, // America/Fortaleza
    {  521, 7924,  165484,  9, }, // America/Glace_Bay
    {  521, 6881,  165493,  7, }, // America/Goose_Bay
    {  521, 4612,  165500, 10, }, // America/Grand_Turk
    {  521, 5770,  165510,  7, }, // America/Grenada
    {  521, 5786,   33197,  9, }, // America/Guadeloupe
    {  521, 3760,   33206,  9, }, // America/Guatemala
    {  521, 4841,  122518,  8, }, // America/Guayaquil
    {  521, 6932,   57452,  7, }, // America/Guyana
    {  521, 1939,    6787,  8, }, // America/Halifax
    {  521, 2281,  147659,  6, }, // America/Havana
    {  521, 7942,  165517, 10, }, // America/Hermosillo
    {  521,  348,  122551, 12, }, // America/Indiana/Indianapolis
    {  521,  481,  165540, 14, }, // America/Indiana/Knox
    {  521, 7961,  165554, 17, }, // America/Indiana/Marengo
    {  521, 7985,  165571, 20, }, // America/Indiana/Petersburg
    {  521, 8012,  165591, 18, }, // America/Indiana/Tell_City
    {  521, 8038,  166425, 15, }, // America/Indiana/Vevay
    {  521, 8060,  166440, 18, }, // America/Indiana/Vincennes
    {  521, 8086,  165640, 17, }, // America/Indiana/Winamac
    {  521, 8110,  122681,  6, }, // America/Inuvik
    {  521,  660,  165657,  8, }, // America/Iqaluit
    {  521, 2799,   33215,  7, }, // America/Jamaica
    {  521, 5380,  165665,  4, }, // America/Juneau
    {  521,  521,  166458,  7, }, // America/Kentucky/Louisville
    {  521, 8125,  165675, 18, }, // America/Kentucky/Monticello
    {  521, 5704,  165693, 10, }, // America/Kralendijk
    {  521, 4376,  165703,  6, }, // America/La_Paz
    {  521, 7169,   21923,  4, }, // America/Lima
    {  521, 3239,  165709, 11, }, // America/Los_Angeles
    {  521, 5932,  165720, 21, }, // America/Lower_Princes
    {  521, 8153,  122774,  6, }, // America/Maceio
    {  521, 8168,  122780,  7, }, // America/Managua
    {  521, 1908,  122787,  6, }, // America/Manaus
    {  521, 5897,   61739,  6, }, // America/Marigot
    {  521, 5805,    5142,  8, }, // America/Martinique
    {  521, 8184,  122802,  9, }, // America/Matamoros
    {  521, 2917,  165741,  8, }, // America/Mazatlan
    {  521, 8202,    7000,  8, }, // America/Menominee
    {  521, 8220,    7008,  6, }, // America/Merida
    {  521, 8235,  165749, 10, }, // America/Metlakatla
    {  521, 2949,  165759, 12, }, // America/Mexico_City
    {  521, 4391,    7031,  7, }, // America/Miquelon
    {  521, 8254,    7038,  7, }, // America/Moncton
    {  521, 8270,   61788,  8, }, // America/Monterrey
    {  521, 4098,  165771, 10, }, // America/Montevideo
    {  521, 5824,    7054,  9, }, // America/Montserrat
    {  521, 5481,  125683,  5, }, // America/Nassau
    {  521, 2379,  165781,  7, }, // America/New_York
    {  521, 8288,   21850,  3, }, // America/Nome
    {  521, 1850,    7076,  7, }, // America/Noronha
    {  521, 8301,  166465, 22, }, // America/North_Dakota/Beulah
    {  521, 8329,  166487, 22, }, // America/North_Dakota/Center
    {  521, 8357,  166509, 26, }, // America/North_Dakota/New_Salem
    {  521,  393,  165860,  6, }, // America/Nuuk
    {  521, 8388,    7158,  7, }, // America/Ojinaga
    {  521, 2356,  165866,  6, }, // America/Panama
    {  521, 6173,  165872, 10, }, // America/Paramaribo
    {  521, 2973,  125799,  6, }, // America/Phoenix
    {  521, 3945,  125807, 12, }, // America/Port-au-Prince
    {  521, 5954,  166535, 13, }, // America/Port_of_Spain
    {  521, 8404,  165907, 11, }, // America/Porto_Velho
    {  521, 5843,  166548, 10, }, // America/Puerto_Rico
    {  521, 4030,  125860, 12, }, // America/Punta_Arenas
    {  521, 8424,  125874, 12, }, // America/Rankin_Inlet
    {  521, 8445,  165918,  6, }, // America/Recife
    {  521, 1995,  166558,  6, }, // America/Regina
    {  521, 8460,  125901,  7, }, // America/Resolute
    {  521,  695,    5362, 10, }, // America/Rio_Branco
    {  521, 8477,  165939,  8, }, // America/Santarem
    {  521, 2201,  123063,  8, }, // America/Santiago
    {  521, 6111,  165947, 13, }, // America/Santo_Domingo
    {  521, 1878,  165960,  9, }, // America/Sao_Paulo
    {  521, 6899,  166564, 12, }, // America/Scoresbysund
    {  521, 8494,  165969,  5, }, // America/Sitka
    {  521, 8508,  166576, 14, }, // America/St_Barthelemy
    {  521, 2061,  166590,  8, }, // America/St_Johns
    {  521, 5863,  126003,  9, }, // America/St_Kitts
    {  521, 5880,  165974, 10, }, // America/St_Lucia
    {  521,  855,  166598,  8, }, // America/St_Thomas
    {  521, 5913,  165984, 12, }, // America/St_Vincent
    {  521, 8530,   22086, 12, }, // America/Swift_Current
    {  521, 5461,    5410, 11, }, // America/Tegucigalpa
    {  521, 5756,   22109,  3, }, // America/Thule
    {  521,  313,    7361,  7, }, // America/Tijuana
    {  521,  608,  165996,  7, }, // America/Toronto
    {  521, 5688,  166003,  7, }, // America/Tortola
    {  521, 2093,    7375,  8, }, // America/Vancouver
    {  521, 2144,  166010,  8, }, // America/Whitehorse
    {  521,  734,   62066,  7, }, // America/Winnipeg
    {  521, 8552,    7399,  7, }, // America/Yakutat
    {  521, 6001,    7406,  5, }, // Antarctica/Casey
    {  521, 6094,    7411,  6, }, // Antarctica/Davis
    {  521, 6133,  166606, 13, }, // Antarctica/DumontDUrville
    {  521, 8568,  166619,  7, }, // Antarctica/Macquarie
    {  521, 7066,  123246,  5, }, // Antarctica/Mawson
    {  521, 7113,  166031,  8, }, // Antarctica/McMurdo
    {  521, 8589,  126182,  6, }, // Antarctica/Palmer
    {  521, 7226,    5465,  6, }, // Antarctica/Rothera
    {  521, 7295,  123267,  4, }, // Antarctica/Syowa
    {  521, 8607,  123271,  4, }, // Antarctica/Troll
    {  521, 7374,  123277,  6, }, // Antarctica/Vostok
    {  521, 1429,  166626, 11, }, // Arctic/Longyearbyen
    {  521, 5570,   62104,  4, }, // Asia/Aden
    {  521, 5395,    7476,  6, }, // Asia/Almaty
    {  521, 3968,    5487,  4, }, // Asia/Amman
    {  521, 5511,    7482,  6, }, // Asia/Anadyr
    {  521, 5523,    5491,  5, }, // Asia/Aqtau
    {  521, 5368,   33303,  7, }, // Asia/Aqtobe
    {  521,  964,   33310,  7, }, // Asia/Ashgabat
    {  521, 8624,    7494,  6, }, // Asia/Atyrau
    {  521, 3607,    4044,  6, }, // Asia/Baghdad
    {  521, 5534,    4050,  7, }, // Asia/Bahrain
    {  521, 3653,   62142,  4, }, // Asia/Baku
    {  521, 4466,  123343,  7, }, // Asia/Bangkok
    {  521, 3571,  123351,  7, }, // Asia/Barnaul
    {  521, 4086,    7507,  6, }, // Asia/Beirut
    {  521, 3778,   33323,  6, }, // Asia/Bishkek
    {  521, 5989,   33329,  6, }, // Asia/Brunei
    {  521, 4601,   57490,  4, }, // Asia/Chita
    {  521, 4511,    5519,  7, }, // Asia/Colombo
    {  521, 4540,    4083,  6, }, // Asia/Damascus
    {  521, 1093,    4599,  4, }, // Asia/Dhaka
    {  521, 6192,  126338,  4, }, // Asia/Dili
    {  521, 3596,  123387,  5, }, // Asia/Dubai
    {  521, 6159,   33336,  7, }, // Asia/Dushanbe
    {  521, 8636,  166046,  9, }, // Asia/Famagusta
    {  521, 8651,   62200,  4, }, // Asia/Gaza
    {  521, 4795,   62205,  6, }, // Asia/Hebron
    {  521, 1259,   33343,  9, }, // Asia/Ho_Chi_Minh
    {  521, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  521, 4771,  123417,  4, }, // Asia/Hovd
    {  521, 4184,  147026,  6, }, // Asia/Irkutsk
    {  521, 7006,   33352,  8, }, // Asia/Jakarta
    {  521, 6992,   33360,  9, }, // Asia/Jayapura
    {  521, 1290,  166055,  9, }, // Asia/Jerusalem
    {  521, 3560,   62257,  5, }, // Asia/Kabul
    {  521, 4316,   33378,  8, }, // Asia/Kamchatka
    {  521, 4239,   33386,  6, }, // Asia/Karachi
    {  521, 1185,    5593,  8, }, // Asia/Kathmandu
    {  521, 8661,   33392,  7, }, // Asia/Khandyga
    {  521,  992,   26732,  7, }, // Asia/Kolkata
    {  521, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  521, 4859,  166064, 12, }, // Asia/Kuala_Lumpur
    {  521, 5976,   33399,  6, }, // Asia/Kuching
    {  521, 5547,    7607,  6, }, // Asia/Kuwait
    {  521, 1211,    7613,  5, }, // Asia/Macau
    {  521, 4017,  123496,  7, }, // Asia/Magadan
    {  521, 1349,    7618,  7, }, // Asia/Makassar
    {  521, 7182,  166076,  6, }, // Asia/Manila
    {  521, 6920,    4208,  6, }, // Asia/Muscat
    {  521, 2554,   33410,  8, }, // Asia/Nicosia
    {  521, 8675,   33418, 12, }, // Asia/Novokuznetsk
    {  521, 4135,   62364, 11, }, // Asia/Novosibirsk
    {  521, 4229,   62376,  4, }, // Asia/Omsk
    {  521, 7145,   62381,  4, }, // Asia/Oral
    {  521, 6961,   22386,  8, }, // Asia/Phnom_Penh
    {  521, 8693,  166082, 10, }, // Asia/Pontianak
    {  521, 4214,   33430,  9, }, // Asia/Pyongyang
    {  521, 5559,    4214,  5, }, // Asia/Qatar
    {  521, 8708,    7652,  8, }, // Asia/Qostanay
    {  521, 4269,    7660,  9, }, // Asia/Qyzylorda
    {  521, 3584,   33439,  5, }, // Asia/Riyadh
    {  521, 4408,   33444,  7, }, // Asia/Sakhalin
    {  521, 7245,   26774,  9, }, // Asia/Samarkand
    {  521, 3279,    7680,  4, }, // Asia/Seoul
    {  521, 1053,   33451,  6, }, // Asia/Shanghai
    {  521, 3300,   20782,  8, }, // Asia/Singapore
    {  521, 4297,  166092, 14, }, // Asia/Srednekolymsk
    {  521, 3263,    7712,  6, }, // Asia/Taipei
    {  521, 4781,   33457,  7, }, // Asia/Tashkent
    {  521, 3915,  126691,  7, }, // Asia/Tbilisi
    {  521, 2772,    7724,  7, }, // Asia/Tehran
    {  521, 1317,   91641,  5, }, // Asia/Thimphu
    {  521, 2821,    4238,  5, }, // Asia/Tokyo
    {  521, 4572,  123671,  5, }, // Asia/Tomsk
    {  521, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  521, 1159,  166106,  6, }, // Asia/Urumqi
    {  521, 8722,  166112,  8, }, // Asia/Ust-Nera
    {  521, 6977,   33470, 10, }, // Asia/Vientiane
    {  521, 4710,   62531, 11, }, // Asia/Vladivostok
    {  521, 4828,    7755,  7, }, // Asia/Yakutsk
    {  521, 1235,    7762,  6, }, // Asia/Yangon
    {  521, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  521, 3747,    7780,  7, }, // Asia/Yerevan
    {  521, 3663,   33480,  5, }, // Atlantic/Azores
    {  521, 5671,   74354,  7, }, // Atlantic/Bermuda
    {  521, 6628,  166120, 15, }, // Atlantic/Canary
    {  521, 3727,  166135, 18, }, // Atlantic/Cape_Verde
    {  521, 1395,  166153, 13, }, // Atlantic/Faroe
    {  521, 8736,  166166,  7, }, // Atlantic/Madeira
    {  521, 2748,   33485,  9, }, // Atlantic/Reykjavik
    {  521, 7272,  166173, 15, }, // Atlantic/South_Georgia
    {  521, 6821,  166188, 12, }, // Atlantic/St_Helena
    {  521, 6644,    7847,  6, }, // Atlantic/Stanley
    {  521, 1670,  166200,  7, }, // Australia/Adelaide
    {  521, 1635,  166207,  8, }, // Australia/Brisbane
    {  521, 1799,  126908, 10, }, // Australia/Broken_Hill
    {  521, 1583,    7880,  6, }, // Australia/Darwin
    {  521, 3637,  166215,  5, }, // Australia/Eucla
    {  521, 1516,   62688,  6, }, // Australia/Hobart
    {  521, 8753,  123873,  8, }, // Australia/Lindeman
    {  521, 1547,   22595,  8, }, // Australia/Lord_Howe
    {  521, 1727,    7903,  7, }, // Australia/Melbourne
    {  521, 1762,   22603,  4, }, // Australia/Perth
    {  521, 1463,    7915,  6, }, // Australia/Sydney
    {  521, 6441,   62715,  9, }, // Europe/Amsterdam
    {  521, 6216,   33506,  6, }, // Europe/Andorra
    {  521, 3620,    7928,  8, }, // Europe/Astrakhan
    {  521, 2300,   33520,  5, }, // Europe/Athens
    {  521, 6336,    7942,  7, }, // Europe/Belgrade
    {  521, 4757,   62753,  6, }, // Europe/Berlin
    {  521, 6488,  166220, 10, }, // Europe/Bratislava
    {  521, 2167,   33525,  6, }, // Europe/Brussels
    {  521, 3928,   33531,  8, }, // Europe/Bucharest
    {  521, 3806,   33539, 10, }, // Europe/Budapest
    {  521, 8772,  166230,  8, }, // Europe/Busingen
    {  521, 2583,   33549,  8, }, // Europe/Chisinau
    {  521, 6289,    4357, 10, }, // Europe/Copenhagen
    {  521, 2338,   33557,  6, }, // Europe/Dublin
    {  521, 6307,  127094,  9, }, // Europe/Gibraltar
    {  521, 8788,   33563,  6, }, // Europe/Guernsey
    {  521, 6612,  127112,  8, }, // Europe/Helsinki
    {  521, 8804,   33569, 10, }, // Europe/Isle_of_Man
    {  521, 1130,   62877,  8, }, // Europe/Istanbul
    {  521, 8823,  166238,  6, }, // Europe/Jersey
    {  521, 3979,   33579, 12, }, // Europe/Kaliningrad
    {  521, 8929,  124066,  5, }, // Europe/Kirov
    {  521, 2527,   33591,  5, }, // Europe/Kyiv
    {  521, 3213,   33596,  7, }, // Europe/Lisbon
    {  521, 6506,  166244,  9, }, // Europe/Ljubljana
    {  521, 2501,   62926,  6, }, // Europe/London
    {  521, 6365,    8054, 10, }, // Europe/Luxembourg
    {  521, 4892,   62944,  6, }, // Europe/Madrid
    {  521, 6397,  166253,  5, }, // Europe/Malta
    {  521, 6582,   99157,  8, }, // Europe/Mariehamn
    {  521, 3693,   62956,  5, }, // Europe/Minsk
    {  521, 6410,    5926,  6, }, // Europe/Monaco
    {  521, 3537,   22748,  6, }, // Europe/Moscow
    {  521, 6458,  166258,  4, }, // Europe/Oslo
    {  521, 4284,   33603,  5, }, // Europe/Paris
    {  521, 6424,  166262,  9, }, // Europe/Podgorica
    {  521, 6275,    8083,  4, }, // Europe/Prague
    {  521, 8837,  166271,  4, }, // Europe/Riga
    {  521, 6324,   33608,  3, }, // Europe/Rome
    {  521, 4331,  166275,  6, }, // Europe/Samara
    {  521, 6470,  166281, 10, }, // Europe/San_Marino
    {  521, 6245,  166291,  8, }, // Europe/Sarajevo
    {  521, 4451,   78231,  7, }, // Europe/Saratov
    {  521, 8849,   33611, 11, }, // Europe/Simferopol
    {  521, 6383,   33622,  7, }, // Europe/Skopje
    {  521, 6599,   33629,  6, }, // Europe/Sofia
    {  521, 6523,    8124,  8, }, // Europe/Stockholm
    {  521, 8867,    8132,  5, }, // Europe/Tallinn
    {  521, 6202,   21095,  6, }, // Europe/Tirane
    {  521, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  521, 6352,   63058,  5, }, // Europe/Vaduz
    {  521, 6567,    5978,  7, }, // Europe/Vatican
    {  521, 6231,   33643,  3, }, // Europe/Vienna
    {  521, 8899,  166299,  7, }, // Europe/Vilnius
    {  521, 4727,   63074,  9, }, // Europe/Volgograd
    {  521, 3190,   33646,  7, }, // Europe/Warsaw
    {  521, 6261,   63092,  6, }, // Europe/Zagreb
    {  521, 6540,   33653,  5, }, // Europe/Zurich
    {  521, 5078,  166306, 12, }, // Indian/Antananarivo
    {  521, 6947,  124266,  5, }, // Indian/Chagos
    {  521, 6046,   33658,  5, }, // Indian/Christmas
    {  521, 6063,    4465,  5, }, // Indian/Cocos
    {  521, 5029,   22853,  6, }, // Indian/Comoro
    {  521, 6661,   22859,  8, }, // Indian/Kerguelen
    {  521, 7260,  158670,  4, }, // Indian/Mahe
    {  521, 7050,   33687,  7, }, // Indian/Maldives
    {  521, 4069,   33694, 10, }, // Indian/Mauritius
    {  521, 5098,    8200,  5, }, // Indian/Mayotte
    {  521, 7211,   91221,  7, }, // Indian/Reunion
    {  521, 4422,  124364,  5, }, // Pacific/Apia
    {  521,  932,    8206,  6, }, // Pacific/Auckland
    {  521, 3706,  166318,  8, }, // Pacific/Bougainville
    {  521, 3015,  127475,  5, }, // Pacific/Chatham
    {  521, 3157,   91228,  4, }, // Pacific/Chuuk
    {  521, 2237,  166326, 15, }, // Pacific/Easter
    {  521, 7360,  127498,  4, }, // Pacific/Efate
    {  521, 7327,  166341,  7, }, // Pacific/Fakaofo
    {  521, 3902,   33704,  5, }, // Pacific/Fiji
    {  521, 7343,   63222,  8, }, // Pacific/Funafuti
    {  521, 6678,   92104,  9, }, // Pacific/Galapagos
    {  521, 6696,  166348,  7, }, // Pacific/Gambier
    {  521, 3822,  124440, 11, }, // Pacific/Guadalcanal
    {  521, 6018,  124452,  4, }, // Pacific/Guam
    {  521, 2723,    4493,  8, }, // Pacific/Honolulu
    {  521, 3049,  124457,  9, }, // Pacific/Kanton
    {  521, 3998,   92138, 10, }, // Pacific/Kiritimati
    {  521, 7019,  166355,  5, }, // Pacific/Kosrae
    {  521, 2842,  166360, 11, }, // Pacific/Kwajalein
    {  521, 4877,   92186,  6, }, // Pacific/Majuro
    {  521, 4051,   36208,  6, }, // Pacific/Marquesas
    {  521, 8914,   63258,  6, }, // Pacific/Midway
    {  521, 7084,  166371,  5, }, // Pacific/Nauru
    {  521, 7132,  166637,  4, }, // Pacific/Niue
    {  521, 4168,   63280,  7, }, // Pacific/Norfolk
    {  521, 7098,  124531,  5, }, // Pacific/Noumea
    {  521, 3126,  166381,  9, }, // Pacific/Pago_Pago
    {  521, 7155,   65647,  5, }, // Pacific/Palau
    {  521, 7194,    8304,  7, }, // Pacific/Pitcairn
    {  521, 3096,    8311,  6, }, // Pacific/Pohnpei
    {  521, 4807,  166390, 11, }, // Pacific/Port_Moresby
    {  521, 6076,  166401,  9, }, // Pacific/Rarotonga
    {  521, 6031,  166410,  6, }, // Pacific/Saipan
    {  521, 7312,   63343,  6, }, // Pacific/Tahiti
    {  521, 6712,   23027,  6, }, // Pacific/Tarawa
    {  521, 4583,  166416,  9, }, // Pacific/Tongatapu
    {  521, 7392,   63356,  4, }, // Pacific/Wake
    {  521, 7405,   33709,  5, }, // Pacific/Wallis
    {  522, 6788,   57564,  7, }, // Africa/Abidjan Serbian/Latin/Serbia
    {  522, 6760,    4531,  4, }, // Africa/Accra
    {  522, 5059,   35689, 10, }, // Africa/Addis_Ababa
    {  522, 7420,   57447,  5, }, // Africa/Algiers
    {  522,   14,   91122,  6, }, // Africa/Asmara
    {  522,   44,  165212,  6, }, // Africa/Bamako
    {  522, 5257,  165218,  6, }, // Africa/Bangui
    {  522, 6746,  165224,  6, }, // Africa/Banjul
    {  522, 7435,  165230,  5, }, // Africa/Bissau
    {  522, 4957,    6200,  7, }, // Africa/Blantyre
    {  522, 5287,    6208,  8, }, // Africa/Brazzaville
    {  522, 4922,  165235, 10, }, // Africa/Bujumbura
    {  522, 2320,    4572,  5, }, // Africa/Cairo
    {  522, 4117,   33144, 10, }, // Africa/Casablanca
    {  522, 7449,   21231,  5, }, // Africa/Ceuta
    {  522, 6773,    6249,  7, }, // Africa/Conakry
    {  522, 6840,   61029,  5, }, // Africa/Dakar
    {  522, 5130,    6256, 12, }, // Africa/Dar_es_Salaam
    {  522, 5043,   33154,  7, }, // Africa/Djibouti
    {  522, 5243,   21254,  5, }, // Africa/Douala
    {  522, 5166,    6275,  7, }, // Africa/El_Aaiun
    {  522, 6853,    6282,  7, }, // Africa/Freetown
    {  522, 4906,   61071,  7, }, // Africa/Gaborone
    {  522, 5015,   61079,  6, }, // Africa/Harare
    {  522, 4479,    6289, 11, }, // Africa/Johannesburg
    {  522, 4499,  124745,  5, }, // Africa/Juba
    {  522, 5151,  165245,  7, }, // Africa/Kampala
    {  522, 4524,    6305,  6, }, // Africa/Khartoum
    {  522, 4973,  124759,  6, }, // Africa/Kigali
    {  522, 5306,  122029,  7, }, // Africa/Kinshasa
    {  522, 4744,   61130,  5, }, // Africa/Lagos
    {  522, 5336,    6311,  8, }, // Africa/Libreville
    {  522, 6869,   61145,  4, }, // Africa/Lome
    {  522, 5211,  165252,  6, }, // Africa/Luanda
    {  522, 4939,  122051,  9, }, // Africa/Lubumbashi
    {  522, 5001,  165258,  6, }, // Africa/Lusaka
    {  522, 5322,  165264,  6, }, // Africa/Malabo
    {  522, 4987,  165270,  6, }, // Africa/Maputo
    {  522, 5197,   61171,  6, }, // Africa/Maseru
    {  522, 5182,  165276,  7, }, // Africa/Mbabane
    {  522, 5113,   33161,  7, }, // Africa/Mogadishu
    {  522, 7034,  122074,  9, }, // Africa/Monrovia
    {  522, 3853,   61201,  7, }, // Africa/Nairobi
    {  522, 5271,   57597,  8, }, // Africa/Ndjamena
    {  522, 5354,  165283,  7, }, // Africa/Niamey
    {  522, 6803,   57605,  7, }, // Africa/Nouakchott
    {  522, 6727,   26402,  8, }, // Africa/Ouagadougou
    {  522, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  522, 4435,   91128,  8, }, // Africa/Sao_Tome
    {  522, 2866,   35756,  7, }, // Africa/Tripoli
    {  522, 6554,   91301,  5, }, // Africa/Tunis
    {  522, 4152,   21354,  7, }, // Africa/Windhoek
    {  522,  132,   97814,  4, }, // America/Adak
    {  522, 3346,  165290,  8, }, // America/Anchorage
    {  522, 5607,   33168,  7, }, // America/Anguilla
    {  522, 5624,   33175,  7, }, // America/Antigua
    {  522, 4554,  124912, 10, }, // America/Araguaina
    {  522,  166,  165298, 12, }, // America/Argentina/Buenos_Aires
    {  522,   91,    6400,  9, }, // America/Argentina/Catamarca
    {  522,  270,    6409,  7, }, // America/Argentina/Cordoba
    {  522,  441,  165310,  5, }, // America/Argentina/Jujuy
    {  522, 7462,    6421,  8, }, // America/Argentina/La_Rioja
    {  522,  565,   97505,  7, }, // America/Argentina/Mendoza
    {  522, 7489,  124952, 12, }, // America/Argentina/Rio_Gallegos
    {  522, 7520,   61307,  5, }, // America/Argentina/Salta
    {  522, 7544,  165315,  8, }, // America/Argentina/San_Juan
    {  522, 5580,  165323,  7, }, // America/Argentina/San_Luis
    {  522, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {  522, 7597,  165330,  7, }, // America/Argentina/Ushuaia
    {  522, 5640,  165337,  5, }, // America/Aruba
    {  522, 4252,    6474,  8, }, // America/Asuncion
    {  522,  237,  165342, 12, }, // America/Atikokan
    {  522, 3679,  122240,  5, }, // America/Bahia
    {  522, 7623,  165354, 14, }, // America/Bahia_Banderas
    {  522, 5654,  122258,  8, }, // America/Barbados
    {  522, 7646,   61342,  5, }, // America/Belem
    {  522, 5407,  165368,  6, }, // America/Belize
    {  522, 7660,  165374, 13, }, // America/Blanc-Sablon
    {  522, 7681,  165387,  9, }, // America/Boa_Vista
    {  522, 4361,  165396,  6, }, // America/Bogota
    {  522, 7699,  165402,  5, }, // America/Boise
    {  522, 7713,  165407, 12, }, // America/Cambridge_Bay
    {  522, 7735,    4780, 12, }, // America/Campo_Grande
    {  522, 3868,    6564,  6, }, // America/Cancun
    {  522, 4694,    4792,  7, }, // America/Caracas
    {  522, 4345,    6570,  5, }, // America/Cayenne
    {  522, 5496,  165419, 16, }, // America/Cayman
    {  522, 2260,  165435,  6, }, // America/Chicago
    {  522, 7756,  125151,  8, }, // America/Chihuahua
    {  522, 7774,  165441, 13, }, // America/Ciudad_Juarez
    {  522, 5422,   33188,  9, }, // America/Costa_Rica
    {  522, 7796,    6618,  7, }, // America/Creston
    {  522, 3791,    4846,  6, }, // America/Cuiaba
    {  522, 5723,    6631,  7, }, // America/Curacao
    {  522, 7812,  165454, 13, }, // America/Danmarkshavn
    {  522, 7833,  125206,  5, }, // America/Dawson
    {  522, 7848,  125213, 10, }, // America/Dawson_Creek
    {  522,  805,   61425,  6, }, // America/Denver
    {  522, 3465,  122438,  7, }, // America/Detroit
    {  522, 5739,   21548,  8, }, // America/Dominica
    {  522,  893,   61448,  8, }, // America/Edmonton
    {  522, 7869,  165467,  8, }, // America/Eirunepe
    {  522, 5441,    6688,  8, }, // America/El_Salvador
    {  522, 7886,  125270, 11, }, // America/Fort_Nelson
    {  522, 7906,  165475,  9, }, // America/Fortaleza
    {  522, 7924,  165484,  9, }, // America/Glace_Bay
    {  522, 6881,  165493,  7, }, // America/Goose_Bay
    {  522, 4612,  165500, 10, }, // America/Grand_Turk
    {  522, 5770,  165510,  7, }, // America/Grenada
    {  522, 5786,   33197,  9, }, // America/Guadeloupe
    {  522, 3760,   33206,  9, }, // America/Guatemala
    {  522, 4841,  122518,  8, }, // America/Guayaquil
    {  522, 6932,   57452,  7, }, // America/Guyana
    {  522, 1939,    6787,  8, }, // America/Halifax
    {  522, 2281,  147659,  6, }, // America/Havana
    {  522, 7942,  165517, 10, }, // America/Hermosillo
    {  522,  348,  122551, 12, }, // America/Indiana/Indianapolis
    {  522,  481,  165540, 14, }, // America/Indiana/Knox
    {  522, 7961,  165554, 17, }, // America/Indiana/Marengo
    {  522, 7985,  165571, 20, }, // America/Indiana/Petersburg
    {  522, 8012,  165591, 18, }, // America/Indiana/Tell_City
    {  522, 8038,  166425, 15, }, // America/Indiana/Vevay
    {  522, 8060,  166440, 18, }, // America/Indiana/Vincennes
    {  522, 8086,  165640, 17, }, // America/Indiana/Winamac
    {  522, 8110,  122681,  6, }, // America/Inuvik
    {  522,  660,  165657,  8, }, // America/Iqaluit
    {  522, 2799,   33215,  7, }, // America/Jamaica
    {  522, 5380,  165665,  4, }, // America/Juneau
    {  522,  521,  166458,  7, }, // America/Kentucky/Louisville
    {  522, 8125,  165675, 18, }, // America/Kentucky/Monticello
    {  522, 5704,  165693, 10, }, // America/Kralendijk
    {  522, 4376,  165703,  6, }, // America/La_Paz
    {  522, 7169,   21923,  4, }, // America/Lima
    {  522, 3239,  165709, 11, }, // America/Los_Angeles
    {  522, 5932,  165720, 21, }, // America/Lower_Princes
    {  522, 8153,  122774,  6, }, // America/Maceio
    {  522, 8168,  122780,  7, }, // America/Managua
    {  522, 1908,  122787,  6, }, // America/Manaus
    {  522, 5897,   61739,  6, }, // America/Marigot
    {  522, 5805,    5142,  8, }, // America/Martinique
    {  522, 8184,  122802,  9, }, // America/Matamoros
    {  522, 2917,  165741,  8, }, // America/Mazatlan
    {  522, 8202,    7000,  8, }, // America/Menominee
    {  522, 8220,    7008,  6, }, // America/Merida
    {  522, 8235,  165749, 10, }, // America/Metlakatla
    {  522, 2949,  165759, 12, }, // America/Mexico_City
    {  522, 4391,    7031,  7, }, // America/Miquelon
    {  522, 8254,    7038,  7, }, // America/Moncton
    {  522, 8270,   61788,  8, }, // America/Monterrey
    {  522, 4098,  165771, 10, }, // America/Montevideo
    {  522, 5824,    7054,  9, }, // America/Montserrat
    {  522, 5481,  125683,  5, }, // America/Nassau
    {  522, 2379,  165781,  7, }, // America/New_York
    {  522, 8288,   21850,  3, }, // America/Nome
    {  522, 1850,    7076,  7, }, // America/Noronha
    {  522, 8301,  166465, 22, }, // America/North_Dakota/Beulah
    {  522, 8329,  166487, 22, }, // America/North_Dakota/Center
    {  522, 8357,  166509, 26, }, // America/North_Dakota/New_Salem
    {  522,  393,  165860,  6, }, // America/Nuuk
    {  522, 8388,    7158,  7, }, // America/Ojinaga
    {  522, 2356,  165866,  6, }, // America/Panama
    {  522, 6173,  165872, 10, }, // America/Paramaribo
    {  522, 2973,  125799,  6, }, // America/Phoenix
    {  522, 3945,  125807, 12, }, // America/Port-au-Prince
    {  522, 5954,  166535, 13, }, // America/Port_of_Spain
    {  522, 8404,  165907, 11, }, // America/Porto_Velho
    {  522, 5843,  166548, 10, }, // America/Puerto_Rico
    {  522, 4030,  125860, 12, }, // America/Punta_Arenas
    {  522, 8424,  125874, 12, }, // America/Rankin_Inlet
    {  522, 8445,  165918,  6, }, // America/Recife
    {  522, 1995,  166558,  6, }, // America/Regina
    {  522, 8460,  125901,  7, }, // America/Resolute
    {  522,  695,    5362, 10, }, // America/Rio_Branco
    {  522, 8477,  165939,  8, }, // America/Santarem
    {  522, 2201,  123063,  8, }, // America/Santiago
    {  522, 6111,  165947, 13, }, // America/Santo_Domingo
    {  522, 1878,  165960,  9, }, // America/Sao_Paulo
    {  522, 6899,  166564, 12, }, // America/Scoresbysund
    {  522, 8494,  165969,  5, }, // America/Sitka
    {  522, 8508,  166576, 14, }, // America/St_Barthelemy
    {  522, 2061,  166590,  8, }, // America/St_Johns
    {  522, 5863,  126003,  9, }, // America/St_Kitts
    {  522, 5880,  165974, 10, }, // America/St_Lucia
    {  522,  855,  166598,  8, }, // America/St_Thomas
    {  522, 5913,  165984, 12, }, // America/St_Vincent
    {  522, 8530,   22086, 12, }, // America/Swift_Current
    {  522, 5461,    5410, 11, }, // America/Tegucigalpa
    {  522, 5756,   22109,  3, }, // America/Thule
    {  522,  313,    7361,  7, }, // America/Tijuana
    {  522,  608,  165996,  7, }, // America/Toronto
    {  522, 5688,  166003,  7, }, // America/Tortola
    {  522, 2093,    7375,  8, }, // America/Vancouver
    {  522, 2144,  166010,  8, }, // America/Whitehorse
    {  522,  734,   62066,  7, }, // America/Winnipeg
    {  522, 8552,    7399,  7, }, // America/Yakutat
    {  522, 6001,    7406,  5, }, // Antarctica/Casey
    {  522, 6094,    7411,  6, }, // Antarctica/Davis
    {  522, 6133,  166606, 13, }, // Antarctica/DumontDUrville
    {  522, 8568,  166619,  7, }, // Antarctica/Macquarie
    {  522, 7066,  123246,  5, }, // Antarctica/Mawson
    {  522, 7113,  166031,  8, }, // Antarctica/McMurdo
    {  522, 8589,  126182,  6, }, // Antarctica/Palmer
    {  522, 7226,    5465,  6, }, // Antarctica/Rothera
    {  522, 7295,  123267,  4, }, // Antarctica/Syowa
    {  522, 8607,  123271,  4, }, // Antarctica/Troll
    {  522, 7374,  123277,  6, }, // Antarctica/Vostok
    {  522, 1429,  166626, 11, }, // Arctic/Longyearbyen
    {  522, 5570,   62104,  4, }, // Asia/Aden
    {  522, 5395,    7476,  6, }, // Asia/Almaty
    {  522, 3968,    5487,  4, }, // Asia/Amman
    {  522, 5511,    7482,  6, }, // Asia/Anadyr
    {  522, 5523,    5491,  5, }, // Asia/Aqtau
    {  522, 5368,   33303,  7, }, // Asia/Aqtobe
    {  522,  964,   33310,  7, }, // Asia/Ashgabat
    {  522, 8624,    7494,  6, }, // Asia/Atyrau
    {  522, 3607,    4044,  6, }, // Asia/Baghdad
    {  522, 5534,    4050,  7, }, // Asia/Bahrain
    {  522, 3653,   62142,  4, }, // Asia/Baku
    {  522, 4466,  123343,  7, }, // Asia/Bangkok
    {  522, 3571,  123351,  7, }, // Asia/Barnaul
    {  522, 4086,    7507,  6, }, // Asia/Beirut
    {  522, 3778,   33323,  6, }, // Asia/Bishkek
    {  522, 5989,   33329,  6, }, // Asia/Brunei
    {  522, 4601,   57490,  4, }, // Asia/Chita
    {  522, 4511,    5519,  7, }, // Asia/Colombo
    {  522, 4540,    4083,  6, }, // Asia/Damascus
    {  522, 1093,    4599,  4, }, // Asia/Dhaka
    {  522, 6192,  126338,  4, }, // Asia/Dili
    {  522, 3596,  123387,  5, }, // Asia/Dubai
    {  522, 6159,   33336,  7, }, // Asia/Dushanbe
    {  522, 8636,  166046,  9, }, // Asia/Famagusta
    {  522, 8651,   62200,  4, }, // Asia/Gaza
    {  522, 4795,   62205,  6, }, // Asia/Hebron
    {  522, 1259,   33343,  9, }, // Asia/Ho_Chi_Minh
    {  522, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  522, 4771,  123417,  4, }, // Asia/Hovd
    {  522, 4184,  147026,  6, }, // Asia/Irkutsk
    {  522, 7006,   33352,  8, }, // Asia/Jakarta
    {  522, 6992,   33360,  9, }, // Asia/Jayapura
    {  522, 1290,  166055,  9, }, // Asia/Jerusalem
    {  522, 3560,   62257,  5, }, // Asia/Kabul
    {  522, 4316,   33378,  8, }, // Asia/Kamchatka
    {  522, 4239,   33386,  6, }, // Asia/Karachi
    {  522, 1185,    5593,  8, }, // Asia/Kathmandu
    {  522, 8661,   33392,  7, }, // Asia/Khandyga
    {  522,  992,   26732,  7, }, // Asia/Kolkata
    {  522, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  522, 4859,  166064, 12, }, // Asia/Kuala_Lumpur
    {  522, 5976,   33399,  6, }, // Asia/Kuching
    {  522, 5547,    7607,  6, }, // Asia/Kuwait
    {  522, 1211,    7613,  5, }, // Asia/Macau
    {  522, 4017,  123496,  7, }, // Asia/Magadan
    {  522, 1349,    7618,  7, }, // Asia/Makassar
    {  522, 7182,  166076,  6, }, // Asia/Manila
    {  522, 6920,    4208,  6, }, // Asia/Muscat
    {  522, 2554,   33410,  8, }, // Asia/Nicosia
    {  522, 8675,   33418, 12, }, // Asia/Novokuznetsk
    {  522, 4135,   62364, 11, }, // Asia/Novosibirsk
    {  522, 4229,   62376,  4, }, // Asia/Omsk
    {  522, 7145,   62381,  4, }, // Asia/Oral
    {  522, 6961,   22386,  8, }, // Asia/Phnom_Penh
    {  522, 8693,  166082, 10, }, // Asia/Pontianak
    {  522, 4214,   33430,  9, }, // Asia/Pyongyang
    {  522, 5559,    4214,  5, }, // Asia/Qatar
    {  522, 8708,    7652,  8, }, // Asia/Qostanay
    {  522, 4269,    7660,  9, }, // Asia/Qyzylorda
    {  522, 3584,   33439,  5, }, // Asia/Riyadh
    {  522, 4408,   33444,  7, }, // Asia/Sakhalin
    {  522, 7245,   26774,  9, }, // Asia/Samarkand
    {  522, 3279,    7680,  4, }, // Asia/Seoul
    {  522, 1053,   33451,  6, }, // Asia/Shanghai
    {  522, 3300,   20782,  8, }, // Asia/Singapore
    {  522, 4297,  166092, 14, }, // Asia/Srednekolymsk
    {  522, 3263,    7712,  6, }, // Asia/Taipei
    {  522, 4781,   33457,  7, }, // Asia/Tashkent
    {  522, 3915,  126691,  7, }, // Asia/Tbilisi
    {  522, 2772,    7724,  7, }, // Asia/Tehran
    {  522, 1317,   91641,  5, }, // Asia/Thimphu
    {  522, 2821,    4238,  5, }, // Asia/Tokyo
    {  522, 4572,  123671,  5, }, // Asia/Tomsk
    {  522, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  522, 1159,  166106,  6, }, // Asia/Urumqi
    {  522, 8722,  166112,  8, }, // Asia/Ust-Nera
    {  522, 6977,   33470, 10, }, // Asia/Vientiane
    {  522, 4710,   62531, 11, }, // Asia/Vladivostok
    {  522, 4828,    7755,  7, }, // Asia/Yakutsk
    {  522, 1235,    7762,  6, }, // Asia/Yangon
    {  522, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  522, 3747,    7780,  7, }, // Asia/Yerevan
    {  522, 3663,   33480,  5, }, // Atlantic/Azores
    {  522, 5671,   74354,  7, }, // Atlantic/Bermuda
    {  522, 6628,  166120, 15, }, // Atlantic/Canary
    {  522, 3727,  166135, 18, }, // Atlantic/Cape_Verde
    {  522, 1395,  166153, 13, }, // Atlantic/Faroe
    {  522, 8736,  166166,  7, }, // Atlantic/Madeira
    {  522, 2748,   33485,  9, }, // Atlantic/Reykjavik
    {  522, 7272,  166173, 15, }, // Atlantic/South_Georgia
    {  522, 6821,  166188, 12, }, // Atlantic/St_Helena
    {  522, 6644,    7847,  6, }, // Atlantic/Stanley
    {  522, 1670,  166200,  7, }, // Australia/Adelaide
    {  522, 1635,  166207,  8, }, // Australia/Brisbane
    {  522, 1799,  126908, 10, }, // Australia/Broken_Hill
    {  522, 1583,    7880,  6, }, // Australia/Darwin
    {  522, 3637,  166215,  5, }, // Australia/Eucla
    {  522, 1516,   62688,  6, }, // Australia/Hobart
    {  522, 8753,  123873,  8, }, // Australia/Lindeman
    {  522, 1547,   22595,  8, }, // Australia/Lord_Howe
    {  522, 1727,    7903,  7, }, // Australia/Melbourne
    {  522, 1762,   22603,  4, }, // Australia/Perth
    {  522, 1463,    7915,  6, }, // Australia/Sydney
    {  522, 6441,   62715,  9, }, // Europe/Amsterdam
    {  522, 6216,   33506,  6, }, // Europe/Andorra
    {  522, 3620,    7928,  8, }, // Europe/Astrakhan
    {  522, 2300,   33520,  5, }, // Europe/Athens
    {  522, 6336,    7942,  7, }, // Europe/Belgrade
    {  522, 4757,   62753,  6, }, // Europe/Berlin
    {  522, 6488,  166220, 10, }, // Europe/Bratislava
    {  522, 2167,   33525,  6, }, // Europe/Brussels
    {  522, 3928,   33531,  8, }, // Europe/Bucharest
    {  522, 3806,   33539, 10, }, // Europe/Budapest
    {  522, 8772,  166230,  8, }, // Europe/Busingen
    {  522, 2583,   33549,  8, }, // Europe/Chisinau
    {  522, 6289,    4357, 10, }, // Europe/Copenhagen
    {  522, 2338,   33557,  6, }, // Europe/Dublin
    {  522, 6307,  127094,  9, }, // Europe/Gibraltar
    {  522, 8788,   33563,  6, }, // Europe/Guernsey
    {  522, 6612,  127112,  8, }, // Europe/Helsinki
    {  522, 8804,   33569, 10, }, // Europe/Isle_of_Man
    {  522, 1130,   62877,  8, }, // Europe/Istanbul
    {  522, 8823,  166238,  6, }, // Europe/Jersey
    {  522, 3979,   33579, 12, }, // Europe/Kaliningrad
    {  522, 8929,  124066,  5, }, // Europe/Kirov
    {  522, 2527,   33591,  5, }, // Europe/Kyiv
    {  522, 3213,   33596,  7, }, // Europe/Lisbon
    {  522, 6506,  166244,  9, }, // Europe/Ljubljana
    {  522, 2501,   62926,  6, }, // Europe/London
    {  522, 6365,    8054, 10, }, // Europe/Luxembourg
    {  522, 4892,   62944,  6, }, // Europe/Madrid
    {  522, 6397,  166253,  5, }, // Europe/Malta
    {  522, 6582,   99157,  8, }, // Europe/Mariehamn
    {  522, 3693,   62956,  5, }, // Europe/Minsk
    {  522, 6410,    5926,  6, }, // Europe/Monaco
    {  522, 3537,   22748,  6, }, // Europe/Moscow
    {  522, 6458,  166258,  4, }, // Europe/Oslo
    {  522, 4284,   33603,  5, }, // Europe/Paris
    {  522, 6424,  166262,  9, }, // Europe/Podgorica
    {  522, 6275,    8083,  4, }, // Europe/Prague
    {  522, 8837,  166271,  4, }, // Europe/Riga
    {  522, 6324,   33608,  3, }, // Europe/Rome
    {  522, 4331,  166275,  6, }, // Europe/Samara
    {  522, 6470,  166281, 10, }, // Europe/San_Marino
    {  522, 6245,  166291,  8, }, // Europe/Sarajevo
    {  522, 4451,   78231,  7, }, // Europe/Saratov
    {  522, 8849,   33611, 11, }, // Europe/Simferopol
    {  522, 6383,   33622,  7, }, // Europe/Skopje
    {  522, 6599,   33629,  6, }, // Europe/Sofia
    {  522, 6523,    8124,  8, }, // Europe/Stockholm
    {  522, 8867,    8132,  5, }, // Europe/Tallinn
    {  522, 6202,   21095,  6, }, // Europe/Tirane
    {  522, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  522, 6352,   63058,  5, }, // Europe/Vaduz
    {  522, 6567,    5978,  7, }, // Europe/Vatican
    {  522, 6231,   33643,  3, }, // Europe/Vienna
    {  522, 8899,  166299,  7, }, // Europe/Vilnius
    {  522, 4727,   63074,  9, }, // Europe/Volgograd
    {  522, 3190,   33646,  7, }, // Europe/Warsaw
    {  522, 6261,   63092,  6, }, // Europe/Zagreb
    {  522, 6540,   33653,  5, }, // Europe/Zurich
    {  522, 5078,  166306, 12, }, // Indian/Antananarivo
    {  522, 6947,  124266,  5, }, // Indian/Chagos
    {  522, 6046,   33658,  5, }, // Indian/Christmas
    {  522, 6063,    4465,  5, }, // Indian/Cocos
    {  522, 5029,   22853,  6, }, // Indian/Comoro
    {  522, 6661,   22859,  8, }, // Indian/Kerguelen
    {  522, 7260,  158670,  4, }, // Indian/Mahe
    {  522, 7050,   33687,  7, }, // Indian/Maldives
    {  522, 4069,   33694, 10, }, // Indian/Mauritius
    {  522, 5098,    8200,  5, }, // Indian/Mayotte
    {  522, 7211,   91221,  7, }, // Indian/Reunion
    {  522, 4422,  124364,  5, }, // Pacific/Apia
    {  522,  932,    8206,  6, }, // Pacific/Auckland
    {  522, 3706,  166318,  8, }, // Pacific/Bougainville
    {  522, 3015,  127475,  5, }, // Pacific/Chatham
    {  522, 3157,   91228,  4, }, // Pacific/Chuuk
    {  522, 2237,  166326, 15, }, // Pacific/Easter
    {  522, 7360,  127498,  4, }, // Pacific/Efate
    {  522, 7327,  166341,  7, }, // Pacific/Fakaofo
    {  522, 3902,   33704,  5, }, // Pacific/Fiji
    {  522, 7343,   63222,  8, }, // Pacific/Funafuti
    {  522, 6678,   92104,  9, }, // Pacific/Galapagos
    {  522, 6696,  166348,  7, }, // Pacific/Gambier
    {  522, 3822,  124440, 11, }, // Pacific/Guadalcanal
    {  522, 6018,  124452,  4, }, // Pacific/Guam
    {  522, 2723,    4493,  8, }, // Pacific/Honolulu
    {  522, 3049,  124457,  9, }, // Pacific/Kanton
    {  522, 3998,   92138, 10, }, // Pacific/Kiritimati
    {  522, 7019,  166355,  5, }, // Pacific/Kosrae
    {  522, 2842,  166360, 11, }, // Pacific/Kwajalein
    {  522, 4877,   92186,  6, }, // Pacific/Majuro
    {  522, 4051,   36208,  6, }, // Pacific/Marquesas
    {  522, 8914,   63258,  6, }, // Pacific/Midway
    {  522, 7084,  166371,  5, }, // Pacific/Nauru
    {  522, 7132,  166637,  4, }, // Pacific/Niue
    {  522, 4168,   63280,  7, }, // Pacific/Norfolk
    {  522, 7098,  124531,  5, }, // Pacific/Noumea
    {  522, 3126,  166381,  9, }, // Pacific/Pago_Pago
    {  522, 7155,   65647,  5, }, // Pacific/Palau
    {  522, 7194,    8304,  7, }, // Pacific/Pitcairn
    {  522, 3096,    8311,  6, }, // Pacific/Pohnpei
    {  522, 4807,  166390, 11, }, // Pacific/Port_Moresby
    {  522, 6076,  166401,  9, }, // Pacific/Rarotonga
    {  522, 6031,  166410,  6, }, // Pacific/Saipan
    {  522, 7312,   63343,  6, }, // Pacific/Tahiti
    {  522, 6712,   23027,  6, }, // Pacific/Tarawa
    {  522, 4583,  166416,  9, }, // Pacific/Tongatapu
    {  522, 7392,   63356,  4, }, // Pacific/Wake
    {  522, 7405,   33709,  5, }, // Pacific/Wallis
    {  523,   14,       0,  6, }, // Africa/Asmara Shambala/Latin/Tanzania
    {  523, 4435,       6,  8, }, // Africa/Sao_Tome
    {  523, 4252,      14,  8, }, // America/Asuncion
    {  523,  237,      22,  8, }, // America/Atikokan
    {  523, 7623,      30, 17, }, // America/Bahia_Banderas
    {  523, 3868,      47,  6, }, // America/Cancun
    {  523, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  523, 5723,      66,  7, }, // America/Curacao
    {  523,  481,      73, 13, }, // America/Indiana/Knox
    {  523, 7961,      86, 16, }, // America/Indiana/Marengo
    {  523, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  523, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  523, 8038,     139, 14, }, // America/Indiana/Vevay
    {  523, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  523, 8086,     171, 16, }, // America/Indiana/Winamac
    {  523, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  523, 5932,     207, 22, }, // America/Lower_Princes
    {  523, 8220,     229,  6, }, // America/Merida
    {  523, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  523, 8329,     255, 20, }, // America/North_Dakota/Center
    {  523, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  523,  393,     298,  4, }, // America/Nuuk
    {  523, 6899,     302, 16, }, // America/Scoresbysund
    {  523, 8508,     318, 14, }, // America/St_Barthelemy
    {  523, 2061,     332, 10, }, // America/St_Johns
    {  523, 5863,     342,  9, }, // America/St_Kitts
    {  523, 5880,     351,  9, }, // America/St_Lucia
    {  523,  855,     360, 10, }, // America/St_Thomas
    {  523, 5913,     370, 11, }, // America/St_Vincent
    {  523, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  523, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  523, 1185,     413,  9, }, // Asia/Kathmandu
    {  523,  992,     422,  7, }, // Asia/Kolkata
    {  523, 1211,     429,  5, }, // Asia/Macau
    {  523, 1235,     442,  6, }, // Asia/Yangon
    {  523, 1395,     448,  5, }, // Atlantic/Faroe
    {  523, 6821,     453, 10, }, // Atlantic/St_Helena
    {  523, 2527,     463,  4, }, // Europe/Kyiv
    {  523, 7211,     467,  7, }, // Indian/Reunion
    {  523, 3157,     474,  5, }, // Pacific/Chuuk
    {  523, 3096,     479,  7, }, // Pacific/Pohnpei
    {  524,   14,       0,  6, }, // Africa/Asmara Shona/Latin/Zimbabwe
    {  524, 4435,       6,  8, }, // Africa/Sao_Tome
    {  524, 4252,      14,  8, }, // America/Asuncion
    {  524,  237,      22,  8, }, // America/Atikokan
    {  524, 7623,      30, 17, }, // America/Bahia_Banderas
    {  524, 3868,      47,  6, }, // America/Cancun
    {  524, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  524, 5723,      66,  7, }, // America/Curacao
    {  524,  481,      73, 13, }, // America/Indiana/Knox
    {  524, 7961,      86, 16, }, // America/Indiana/Marengo
    {  524, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  524, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  524, 8038,     139, 14, }, // America/Indiana/Vevay
    {  524, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  524, 8086,     171, 16, }, // America/Indiana/Winamac
    {  524, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  524, 5932,     207, 22, }, // America/Lower_Princes
    {  524, 8220,     229,  6, }, // America/Merida
    {  524, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  524, 8329,     255, 20, }, // America/North_Dakota/Center
    {  524, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  524,  393,     298,  4, }, // America/Nuuk
    {  524, 6899,     302, 16, }, // America/Scoresbysund
    {  524, 8508,     318, 14, }, // America/St_Barthelemy
    {  524, 2061,     332, 10, }, // America/St_Johns
    {  524, 5863,     342,  9, }, // America/St_Kitts
    {  524, 5880,     351,  9, }, // America/St_Lucia
    {  524,  855,     360, 10, }, // America/St_Thomas
    {  524, 5913,     370, 11, }, // America/St_Vincent
    {  524, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  524, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  524, 1185,     413,  9, }, // Asia/Kathmandu
    {  524,  992,     422,  7, }, // Asia/Kolkata
    {  524, 1211,     429,  5, }, // Asia/Macau
    {  524, 1235,     442,  6, }, // Asia/Yangon
    {  524, 1395,     448,  5, }, // Atlantic/Faroe
    {  524, 6821,     453, 10, }, // Atlantic/St_Helena
    {  524, 2527,     463,  4, }, // Europe/Kyiv
    {  524, 7211,     467,  7, }, // Indian/Reunion
    {  524, 3157,     474,  5, }, // Pacific/Chuuk
    {  524, 3096,     479,  7, }, // Pacific/Pohnpei
    {  525,   14,       0,  6, }, // Africa/Asmara Sichuan Yi/Yi/China
    {  525, 4435,       6,  8, }, // Africa/Sao_Tome
    {  525, 4252,      14,  8, }, // America/Asuncion
    {  525,  237,      22,  8, }, // America/Atikokan
    {  525, 7623,      30, 17, }, // America/Bahia_Banderas
    {  525, 3868,      47,  6, }, // America/Cancun
    {  525, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  525, 5723,      66,  7, }, // America/Curacao
    {  525,  481,      73, 13, }, // America/Indiana/Knox
    {  525, 7961,      86, 16, }, // America/Indiana/Marengo
    {  525, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  525, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  525, 8038,     139, 14, }, // America/Indiana/Vevay
    {  525, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  525, 8086,     171, 16, }, // America/Indiana/Winamac
    {  525, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  525, 5932,     207, 22, }, // America/Lower_Princes
    {  525, 8220,     229,  6, }, // America/Merida
    {  525, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  525, 8329,     255, 20, }, // America/North_Dakota/Center
    {  525, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  525,  393,     298,  4, }, // America/Nuuk
    {  525, 6899,     302, 16, }, // America/Scoresbysund
    {  525, 8508,     318, 14, }, // America/St_Barthelemy
    {  525, 2061,     332, 10, }, // America/St_Johns
    {  525, 5863,     342,  9, }, // America/St_Kitts
    {  525, 5880,     351,  9, }, // America/St_Lucia
    {  525,  855,     360, 10, }, // America/St_Thomas
    {  525, 5913,     370, 11, }, // America/St_Vincent
    {  525, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  525, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  525, 1185,     413,  9, }, // Asia/Kathmandu
    {  525,  992,     422,  7, }, // Asia/Kolkata
    {  525, 1211,     429,  5, }, // Asia/Macau
    {  525, 1235,     442,  6, }, // Asia/Yangon
    {  525, 1395,     448,  5, }, // Atlantic/Faroe
    {  525, 6821,     453, 10, }, // Atlantic/St_Helena
    {  525, 2527,     463,  4, }, // Europe/Kyiv
    {  525, 7211,     467,  7, }, // Indian/Reunion
    {  525, 3157,     474,  5, }, // Pacific/Chuuk
    {  525, 3096,     479,  7, }, // Pacific/Pohnpei
    {  526, 7420,   94935,  6, }, // Africa/Algiers Sicilian/Latin/Italy
    {  526,   14,       0,  6, }, // Africa/Asmara
    {  526, 2320,   20270,  5, }, // Africa/Cairo
    {  526, 4117,  166641, 11, }, // Africa/Casablanca
    {  526, 7449,  166652,  5, }, // Africa/Ceuta
    {  526, 5043,  166657,  7, }, // Africa/Djibouti
    {  526, 5113,  166664, 10, }, // Africa/Mogadishu
    {  526, 3853,  166674,  8, }, // Africa/Nairobi
    {  526, 4435,       6,  8, }, // Africa/Sao_Tome
    {  526, 2866,  166682,  7, }, // Africa/Tripoli
    {  526, 6554,  164364,  6, }, // Africa/Tunis
    {  526,  270,  166689,  7, }, // America/Argentina/Cordoba
    {  526, 4252,      14,  8, }, // America/Asuncion
    {  526,  237,      22,  8, }, // America/Atikokan
    {  526, 7623,      30, 17, }, // America/Bahia_Banderas
    {  526, 5407,  166696,  6, }, // America/Belize
    {  526, 7735,  166702, 11, }, // America/Campo_Grande
    {  526, 3868,      47,  6, }, // America/Cancun
    {  526, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  526, 5422,  166713, 10, }, // America/Costa_Rica
    {  526, 5723,      66,  7, }, // America/Curacao
    {  526, 3465,  166723,  7, }, // America/Detroit
    {  526, 5739,  166730,  8, }, // America/Dominica
    {  526, 5786,   94980,  9, }, // America/Guadeloupe
    {  526, 3760,   97898,  9, }, // America/Guatemala
    {  526, 6932,  157918,  6, }, // America/Guyana
    {  526,  481,      73, 13, }, // America/Indiana/Knox
    {  526, 7961,      86, 16, }, // America/Indiana/Marengo
    {  526, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  526, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  526, 8038,     139, 14, }, // America/Indiana/Vevay
    {  526, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  526, 8086,     171, 16, }, // America/Indiana/Winamac
    {  526, 2799,   94996,  8, }, // America/Jamaica
    {  526, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  526, 5932,     207, 22, }, // America/Lower_Princes
    {  526, 5805,   20436,  9, }, // America/Martinique
    {  526, 8220,     229,  6, }, // America/Merida
    {  526, 2949,  166738, 15, }, // America/Mexico_City
    {  526, 8270,  166753,  9, }, // America/Monterrey
    {  526, 5824,  166762, 10, }, // America/Montserrat
    {  526, 2379,  166772, 10, }, // America/New_York
    {  526, 8301,  166782, 28, }, // America/North_Dakota/Beulah
    {  526, 8329,  166810, 28, }, // America/North_Dakota/Center
    {  526, 8357,  166838, 31, }, // America/North_Dakota/New_Salem
    {  526,  393,     298,  4, }, // America/Nuuk
    {  526, 2356,  164484,  6, }, // America/Panama
    {  526, 5954,  166869, 15, }, // America/Port_of_Spain
    {  526, 5843,  166884, 10, }, // America/Puerto_Rico
    {  526, 1995,  166894,  7, }, // America/Regina
    {  526, 6111,   98037, 13, }, // America/Santo_Domingo
    {  526, 1878,  166901,  9, }, // America/Sao_Paulo
    {  526, 6899,     302, 16, }, // America/Scoresbysund
    {  526, 8508,     318, 14, }, // America/St_Barthelemy
    {  526, 2061,     332, 10, }, // America/St_Johns
    {  526, 5863,     342,  9, }, // America/St_Kitts
    {  526, 5880,  166910, 11, }, // America/St_Lucia
    {  526,  855,     360, 10, }, // America/St_Thomas
    {  526, 5913,     370, 11, }, // America/St_Vincent
    {  526,  608,   98099,  7, }, // America/Toronto
    {  526, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  526, 3968,  166921,  5, }, // Asia/Amman
    {  526, 4511,  166926,  7, }, // Asia/Colombo
    {  526, 4540,   20639,  7, }, // Asia/Damascus
    {  526, 8651,  166933,  5, }, // Asia/Gaza
    {  526, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  526, 7006,   95138,  8, }, // Asia/Jakarta
    {  526, 1290,  166938, 11, }, // Asia/Jerusalem
    {  526, 1185,     413,  9, }, // Asia/Kathmandu
    {  526,  992,  166949,  7, }, // Asia/Kolkata
    {  526, 1211,    4203,  5, }, // Asia/Macau
    {  526, 6920,  166956,  7, }, // Asia/Muscat
    {  526, 2554,  166963,  7, }, // Asia/Nicosia
    {  526, 7245,  166970, 10, }, // Asia/Samarkand
    {  526, 3279,  166980,  4, }, // Asia/Seoul
    {  526, 3300,  166984,  9, }, // Asia/Singapore
    {  526, 4710,  166993, 11, }, // Asia/Vladivostok
    {  526, 1235,     442,  6, }, // Asia/Yangon
    {  526, 3883,  167004, 14, }, // Asia/Yekaterinburg
    {  526, 3663,  167018,  7, }, // Atlantic/Azores
    {  526, 5671,  167025,  7, }, // Atlantic/Bermuda
    {  526, 6628,   20849,  6, }, // Atlantic/Canary
    {  526, 3727,  167032, 10, }, // Atlantic/Cape_Verde
    {  526, 1395,     448,  5, }, // Atlantic/Faroe
    {  526, 7272,  167042, 19, }, // Atlantic/South_Georgia
    {  526, 6821,  167061, 10, }, // Atlantic/St_Helena
    {  526, 1670,  167071,  8, }, // Australia/Adelaide
    {  526, 2300,  134414,  5, }, // Europe/Athens
    {  526, 6336,  164714,  8, }, // Europe/Belgrade
    {  526, 4757,  167079,  7, }, // Europe/Berlin
    {  526, 2167,  167086,  8, }, // Europe/Brussels
    {  526, 2338,  167094,  8, }, // Europe/Dublin
    {  526, 6307,  167102, 11, }, // Europe/Gibraltar
    {  526, 6612,   44421,  8, }, // Europe/Helsinki
    {  526, 8804,  167113, 12, }, // Europe/Isle_of_Man
    {  526, 1130,  164764,  8, }, // Europe/Istanbul
    {  526, 2527,    8030,  4, }, // Europe/Kyiv
    {  526, 3213,   93285,  7, }, // Europe/Lisbon
    {  526, 2501,  167125,  7, }, // Europe/London
    {  526, 6365,  167132, 11, }, // Europe/Luxembourg
    {  526, 6397,  167143,  5, }, // Europe/Malta
    {  526, 6410,  164772,  6, }, // Europe/Monaco
    {  526, 3537,   95258,  5, }, // Europe/Moscow
    {  526, 4284,  167148,  7, }, // Europe/Paris
    {  526, 6275,   21067,  5, }, // Europe/Prague
    {  526, 6324,    5946,  4, }, // Europe/Rome
    {  526, 6470,   99183, 10, }, // Europe/San_Marino
    {  526, 6245,  167155,  8, }, // Europe/Sarajevo
    {  526, 6523,  167163,  9, }, // Europe/Stockholm
    {  526, 6202,   21095,  6, }, // Europe/Tirane
    {  526, 6567,   21104,  8, }, // Europe/Vatican
    {  526, 4727,  167172,  9, }, // Europe/Volgograd
    {  526, 3190,   95307,  8, }, // Europe/Warsaw
    {  526, 6261,  167181,  9, }, // Europe/Zagreb
    {  526, 6540,  164837,  6, }, // Europe/Zurich
    {  526, 6046,  167190,  6, }, // Indian/Christmas
    {  526, 7050,  167196,  7, }, // Indian/Maldives
    {  526, 7211,     467,  7, }, // Indian/Reunion
    {  526, 3157,     474,  5, }, // Pacific/Chuuk
    {  526, 3902,   95340,  4, }, // Pacific/Fiji
    {  526, 6678,  167203,  9, }, // Pacific/Galapagos
    {  526, 4051,   95344,  8, }, // Pacific/Marquesas
    {  526, 3096,     479,  7, }, // Pacific/Pohnpei
    {  527,   14,       0,  6, }, // Africa/Asmara Sidamo/Latin/Ethiopia
    {  527, 4435,       6,  8, }, // Africa/Sao_Tome
    {  527, 4252,      14,  8, }, // America/Asuncion
    {  527,  237,      22,  8, }, // America/Atikokan
    {  527, 7623,      30, 17, }, // America/Bahia_Banderas
    {  527, 3868,      47,  6, }, // America/Cancun
    {  527, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  527, 5723,      66,  7, }, // America/Curacao
    {  527,  481,      73, 13, }, // America/Indiana/Knox
    {  527, 7961,      86, 16, }, // America/Indiana/Marengo
    {  527, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  527, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  527, 8038,     139, 14, }, // America/Indiana/Vevay
    {  527, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  527, 8086,     171, 16, }, // America/Indiana/Winamac
    {  527, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  527, 5932,     207, 22, }, // America/Lower_Princes
    {  527, 8220,     229,  6, }, // America/Merida
    {  527, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  527, 8329,     255, 20, }, // America/North_Dakota/Center
    {  527, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  527,  393,     298,  4, }, // America/Nuuk
    {  527, 6899,     302, 16, }, // America/Scoresbysund
    {  527, 8508,     318, 14, }, // America/St_Barthelemy
    {  527, 2061,     332, 10, }, // America/St_Johns
    {  527, 5863,     342,  9, }, // America/St_Kitts
    {  527, 5880,     351,  9, }, // America/St_Lucia
    {  527,  855,     360, 10, }, // America/St_Thomas
    {  527, 5913,     370, 11, }, // America/St_Vincent
    {  527, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  527, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  527, 1185,     413,  9, }, // Asia/Kathmandu
    {  527,  992,     422,  7, }, // Asia/Kolkata
    {  527, 1211,     429,  5, }, // Asia/Macau
    {  527, 1235,     442,  6, }, // Asia/Yangon
    {  527, 1395,     448,  5, }, // Atlantic/Faroe
    {  527, 6821,     453, 10, }, // Atlantic/St_Helena
    {  527, 2527,     463,  4, }, // Europe/Kyiv
    {  527, 7211,     467,  7, }, // Indian/Reunion
    {  527, 3157,     474,  5, }, // Pacific/Chuuk
    {  527, 3096,     479,  7, }, // Pacific/Pohnpei
    {  528, 6788,  156922,  7, }, // Africa/Abidjan Silesian/Latin/Poland
    {  528, 6760,    4531,  4, }, // Africa/Accra
    {  528, 5059,    3784, 11, }, // Africa/Addis_Ababa
    {  528, 7420,   43956,  5, }, // Africa/Algiers
    {  528,   14,       0,  6, }, // Africa/Asmara
    {  528, 5257,   26348,  5, }, // Africa/Bangui
    {  528, 6746,  156929,  7, }, // Africa/Banjul
    {  528, 4922,  156936,  9, }, // Africa/Bujumbura
    {  528, 2320,    4572,  4, }, // Africa/Cairo
    {  528, 6773,  167212,  7, }, // Africa/Conakry
    {  528, 5043,  156945,  7, }, // Africa/Djibouti
    {  528, 5243,   21254,  5, }, // Africa/Douala
    {  528, 5166,  156952,  7, }, // Africa/El_Aaiun
    {  528, 4499,  156959,  5, }, // Africa/Juba
    {  528, 4524,  124750,  7, }, // Africa/Khartoum
    {  528, 5306,  156964,  8, }, // Africa/Kinshasa
    {  528, 6869,   20300,  4, }, // Africa/Lome
    {  528, 5113,  156972,  9, }, // Africa/Mogadishu
    {  528, 5271,  156981,  8, }, // Africa/Ndjamena
    {  528, 6803,  156989,  9, }, // Africa/Nouakchott
    {  528, 6727,    4640,  8, }, // Africa/Ouagadougou
    {  528, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  528, 4435,       6,  8, }, // Africa/Sao_Tome
    {  528, 2866,  156998,  8, }, // Africa/Tripoli
    {  528, 4152,   56676,  7, }, // Africa/Windhoek
    {  528,  565,  167219,  7, }, // America/Argentina/Mendoza
    {  528, 4252,  167226,  8, }, // America/Asuncion
    {  528,  237,      22,  8, }, // America/Atikokan
    {  528, 3679,    6688,  8, }, // America/Bahia
    {  528, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  528, 7646,   35764,  5, }, // America/Belem
    {  528, 3868,      47,  6, }, // America/Cancun
    {  528, 4345,  167234,  7, }, // America/Cayenne
    {  528, 5496,  157006,  7, }, // America/Cayman
    {  528, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  528, 5422,  157013,  9, }, // America/Costa_Rica
    {  528, 3791,   26442,  6, }, // America/Cuiaba
    {  528, 5723,      66,  7, }, // America/Curacao
    {  528, 5739,  167241,  8, }, // America/Dominica
    {  528, 5441,  157022,  8, }, // America/El_Salvador
    {  528, 5770,  167249,  7, }, // America/Grenada
    {  528, 5786,  157030,  9, }, // America/Guadeloupe
    {  528, 3760,  167256,  9, }, // America/Guatemala
    {  528, 6932,  157039,  6, }, // America/Guyana
    {  528, 2281,  157045,  6, }, // America/Havana
    {  528,  481,      73, 13, }, // America/Indiana/Knox
    {  528, 7961,      86, 16, }, // America/Indiana/Marengo
    {  528, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  528, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  528, 8038,     139, 14, }, // America/Indiana/Vevay
    {  528, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  528, 8086,     171, 16, }, // America/Indiana/Winamac
    {  528, 2799,   33215,  7, }, // America/Jamaica
    {  528, 8125,     187, 10, }, // America/Kentucky/Monticello
    {  528, 5932,     207, 22, }, // America/Lower_Princes
    {  528, 8153,  167265,  6, }, // America/Maceio
    {  528, 5805,  157051,  9, }, // America/Martinique
    {  528, 8202,  167271,  9, }, // America/Menominee
    {  528, 8220,    7008,  6, }, // America/Merida
    {  528, 2949,  157060, 15, }, // America/Mexico_City
    {  528, 5824,  167280, 10, }, // America/Montserrat
    {  528, 2379,  157075,  9, }, // America/New_York
    {  528, 8301,  167290, 23, }, // America/North_Dakota/Beulah
    {  528, 8329,  167313, 23, }, // America/North_Dakota/Center
    {  528, 8357,  167336, 26, }, // America/North_Dakota/New_Salem
    {  528,  393,     298,  4, }, // America/Nuuk
    {  528, 5954,   26559, 13, }, // America/Port_of_Spain
    {  528, 5843,  157156,  9, }, // America/Puerto_Rico
    {  528, 6899,     302, 16, }, // America/Scoresbysund
    {  528, 8508,  167362, 16, }, // America/St_Barthelemy
    {  528, 2061,     332, 10, }, // America/St_Johns
    {  528, 5863,   26617, 11, }, // America/St_Kitts
    {  528, 5880,   35832, 11, }, // America/St_Lucia
    {  528,  855,   44161, 12, }, // America/St_Thomas
    {  528, 5913,   26651, 13, }, // America/St_Vincent
    {  528, 5756,   26664,  7, }, // America/Thule
    {  528, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  528, 7374,    4029,  6, }, // Antarctica/Vostok
    {  528, 1429,  167378, 12, }, // Arctic/Longyearbyen
    {  528, 5570,  167390,  4, }, // Asia/Aden
    {  528, 5395,  157165,  6, }, // Asia/Almaty
    {  528, 5523,    5491,  5, }, // Asia/Aqtau
    {  528, 5368,  157171, 10, }, // Asia/Aqtobe
    {  528,  964,  157181,  9, }, // Asia/Ashgabat
    {  528, 3607,    4044,  6, }, // Asia/Baghdad
    {  528, 5534,   57811,  7, }, // Asia/Bahrain
    {  528, 3571,  157190,  7, }, // Asia/Barnaul
    {  528, 4086,    7507,  6, }, // Asia/Beirut
    {  528, 3778,  157197,  7, }, // Asia/Bishkek
    {  528, 4601,  157204,  5, }, // Asia/Chita
    {  528, 4511,  167394,  7, }, // Asia/Colombo
    {  528, 4540,  157209,  8, }, // Asia/Damascus
    {  528, 3596,   57836,  5, }, // Asia/Dubai
    {  528, 6159,  157217,  8, }, // Asia/Dushanbe
    {  528, 4795,  167401,  6, }, // Asia/Hebron
    {  528, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  528, 2704,  167407,  8, }, // Asia/Hong_Kong
    {  528, 4771,  157225,  5, }, // Asia/Hovd
    {  528, 4184,  147026,  6, }, // Asia/Irkutsk
    {  528, 7006,  157230,  8, }, // Asia/Jakarta
    {  528, 1290,  157238, 10, }, // Asia/Jerusalem
    {  528, 4316,  157248,  9, }, // Asia/Kamchatka
    {  528, 4239,  157257,  7, }, // Asia/Karachi
    {  528, 1185,    5593,  8, }, // Asia/Kathmandu
    {  528, 8661,   57888,  8, }, // Asia/Khandyga
    {  528,  992,   26732,  7, }, // Asia/Kolkata
    {  528, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  528, 5547,  157264,  6, }, // Asia/Kuwait
    {  528, 1211,   33405,  5, }, // Asia/Macau
    {  528, 6920,   26750,  6, }, // Asia/Muscat
    {  528, 2554,  157270,  7, }, // Asia/Nicosia
    {  528, 8675,  157277, 12, }, // Asia/Novokuznetsk
    {  528, 4135,  157289, 11, }, // Asia/Novosibirsk
    {  528, 4229,  167415,  4, }, // Asia/Omsk
    {  528, 7145,  167419,  4, }, // Asia/Oral
    {  528, 4214,  167423,  9, }, // Asia/Pyongyang
    {  528, 5559,    4214,  5, }, // Asia/Qatar
    {  528, 8708,  157300,  8, }, // Asia/Qostanay
    {  528, 4269,  157308,  9, }, // Asia/Qyzylorda
    {  528, 3584,   33439,  5, }, // Asia/Riyadh
    {  528, 4408,   57952,  8, }, // Asia/Sakhalin
    {  528, 7245,   26774, 10, }, // Asia/Samarkand
    {  528, 3279,    7680,  4, }, // Asia/Seoul
    {  528, 1053,  157317,  8, }, // Asia/Shanghai
    {  528, 3300,   20782,  8, }, // Asia/Singapore
    {  528, 4297,  157325, 15, }, // Asia/Srednekolymsk
    {  528, 3263,    7712,  6, }, // Asia/Taipei
    {  528, 4781,  167432,  8, }, // Asia/Tashkent
    {  528, 2772,    7724,  7, }, // Asia/Tehran
    {  528, 2821,    4238,  5, }, // Asia/Tokyo
    {  528, 4572,  167440,  5, }, // Asia/Tomsk
    {  528, 1021,  157349, 10, }, // Asia/Ulaanbaatar
    {  528, 1159,  157359,  7, }, // Asia/Urumqi
    {  528, 8722,  157366,  9, }, // Asia/Ust-Nera
    {  528, 6977,  167445,  8, }, // Asia/Vientiane
    {  528, 4710,  157383, 11, }, // Asia/Vladivostok
    {  528, 4828,  147053,  6, }, // Asia/Yakutsk
    {  528, 1235,    7762,  6, }, // Asia/Yangon
    {  528, 3883,  157394, 13, }, // Asia/Yekaterinburg
    {  528, 3747,  157407,  6, }, // Asia/Yerevan
    {  528, 3663,  157413,  5, }, // Atlantic/Azores
    {  528, 5671,   58043,  7, }, // Atlantic/Bermuda
    {  528, 6628,  167453, 16, }, // Atlantic/Canary
    {  528, 3727,  167469, 30, }, // Atlantic/Cape_Verde
    {  528, 1395,  167499, 11, }, // Atlantic/Faroe
    {  528, 8736,  157475,  6, }, // Atlantic/Madeira
    {  528, 7272,  167510, 18, }, // Atlantic/South_Georgia
    {  528, 6821,  167528, 14, }, // Atlantic/St_Helena
    {  528, 6216,   33506,  6, }, // Europe/Andorra
    {  528, 3620,  157512,  9, }, // Europe/Astrakhan
    {  528, 2300,  167542,  5, }, // Europe/Athens
    {  528, 6336,   26883,  7, }, // Europe/Belgrade
    {  528, 6488,  157526, 10, }, // Europe/Bratislava
    {  528, 2167,  157536,  8, }, // Europe/Brussels
    {  528, 3928,  157544,  9, }, // Europe/Bucharest
    {  528, 3806,  157553,  9, }, // Europe/Budapest
    {  528, 8772,   62795, 21, }, // Europe/Busingen
    {  528, 2583,  167547,  9, }, // Europe/Chisinau
    {  528, 6289,  167556,  9, }, // Europe/Copenhagen
    {  528, 8804,  157571,  9, }, // Europe/Isle_of_Man
    {  528, 1130,  157580,  7, }, // Europe/Istanbul
    {  528, 8929,   78158,  5, }, // Europe/Kirov
    {  528, 2527,  167565,  5, }, // Europe/Kyiv
    {  528, 3213,  167570,  7, }, // Europe/Lisbon
    {  528, 6506,  157599,  7, }, // Europe/Ljubljana
    {  528, 2501,  167577,  6, }, // Europe/London
    {  528, 6365,  167583, 10, }, // Europe/Luxembourg
    {  528, 4892,  157612,  6, }, // Europe/Madrid
    {  528, 6582,   63713, 13, }, // Europe/Mariehamn
    {  528, 3693,  157618,  5, }, // Europe/Minsk
    {  528, 6410,  167593,  6, }, // Europe/Monaco
    {  528, 3537,   93104,  6, }, // Europe/Moscow
    {  528, 6458,  167599,  4, }, // Europe/Oslo
    {  528, 4284,  157623,  5, }, // Europe/Paris
    {  528, 6275,   21067,  5, }, // Europe/Prague
    {  528, 8837,  127247,  4, }, // Europe/Riga
    {  528, 6324,  157628,  4, }, // Europe/Rome
    {  528, 6245,  157632,  8, }, // Europe/Sarajevo
    {  528, 4451,  167603,  7, }, // Europe/Saratov
    {  528, 8849,  157647, 10, }, // Europe/Simferopol
    {  528, 6523,  157657,  9, }, // Europe/Stockholm
    {  528, 8867,   22793,  6, }, // Europe/Tallinn
    {  528, 6202,   21095,  6, }, // Europe/Tirane
    {  528, 8882,   78174,  9, }, // Europe/Ulyanovsk
    {  528, 6567,  157666,  7, }, // Europe/Vatican
    {  528, 6231,  167610,  6, }, // Europe/Vienna
    {  528, 8899,  157679,  5, }, // Europe/Vilnius
    {  528, 4727,  157684,  9, }, // Europe/Volgograd
    {  528, 3190,   58543,  8, }, // Europe/Warsaw
    {  528, 6261,  157693,  7, }, // Europe/Zagreb
    {  528, 6540,  157700,  6, }, // Europe/Zurich
    {  528, 5078,  157706, 12, }, // Indian/Antananarivo
    {  528, 6947,  157718,  6, }, // Indian/Chagos
    {  528, 6046,  167616, 12, }, // Indian/Christmas
    {  528, 6063,  157747, 14, }, // Indian/Cocos
    {  528, 5029,  167628,  6, }, // Indian/Comoro
    {  528, 6661,  167634, 18, }, // Indian/Kerguelen
    {  528, 7260,   26983,  4, }, // Indian/Mahe
    {  528, 7050,  127899,  8, }, // Indian/Maldives
    {  528, 5098,  157777,  7, }, // Indian/Mayotte
    {  528, 7211,     467,  7, }, // Indian/Reunion
    {  528, 3706,  167652, 22, }, // Pacific/Bougainville
    {  528, 3157,     474,  5, }, // Pacific/Chuuk
    {  528, 2237,  167674, 17, }, // Pacific/Easter
    {  528, 3902,  157821,  5, }, // Pacific/Fiji
    {  528, 6696,  157826, 14, }, // Pacific/Gambier
    {  528, 3049,    4501,  9, }, // Pacific/Kanton
    {  528, 4051,  157840,  7, }, // Pacific/Marquesas
    {  528, 7098,  124531,  5, }, // Pacific/Noumea
    {  528, 3096,     479,  7, }, // Pacific/Pohnpei
    {  529, 6788,  167691,  7, }, // Africa/Abidjan Sindhi/Arabic/Pakistan
    {  529, 6760,  167698,  5, }, // Africa/Accra
    {  529, 5059,  167703, 10, }, // Africa/Addis_Ababa
    {  529, 7420,  167713,  6, }, // Africa/Algiers
    {  529,   14,  167719,  6, }, // Africa/Asmara
    {  529,   44,  167725,  6, }, // Africa/Bamako
    {  529, 5257,  167731,  4, }, // Africa/Bangui
    {  529, 6746,  167735,  6, }, // Africa/Banjul
    {  529, 7435,  167741,  5, }, // Africa/Bissau
    {  529, 4957,  167746,  7, }, // Africa/Blantyre
    {  529, 5287,  151837,  8, }, // Africa/Brazzaville
    {  529, 4922,  167753,  8, }, // Africa/Bujumbura
    {  529, 2320,  167761,  5, }, // Africa/Cairo
    {  529, 4117,  167766, 10, }, // Africa/Casablanca
    {  529, 7449,  167776,  5, }, // Africa/Ceuta
    {  529, 6773,  167781,  7, }, // Africa/Conakry
    {  529, 6840,  167788,  5, }, // Africa/Dakar
    {  529, 5130,  103097,  9, }, // Africa/Dar_es_Salaam
    {  529, 5043,  151883,  5, }, // Africa/Djibouti
    {  529, 5243,   10515,  5, }, // Africa/Douala
    {  529, 5166,  167793,  7, }, // Africa/El_Aaiun
    {  529, 6853,  167800,  9, }, // Africa/Freetown
    {  529, 4906,  167809,  7, }, // Africa/Gaborone
    {  529, 5015,   10541,  5, }, // Africa/Harare
    {  529, 4479,  167816,  9, }, // Africa/Johannesburg
    {  529, 4499,   10555,  4, }, // Africa/Juba
    {  529, 5151,  167825,  6, }, // Africa/Kampala
    {  529, 4524,   10568,  5, }, // Africa/Khartoum
    {  529, 4973,  167831,  6, }, // Africa/Kigali
    {  529, 5306,  167837,  6, }, // Africa/Kinshasa
    {  529, 4744,  106771,  5, }, // Africa/Lagos
    {  529, 5336,  167843,  8, }, // Africa/Libreville
    {  529, 6869,   10598,  3, }, // Africa/Lome
    {  529, 5211,  167851,  6, }, // Africa/Luanda
    {  529, 4939,  167857,  8, }, // Africa/Lubumbashi
    {  529, 5001,  167865,  6, }, // Africa/Lusaka
    {  529, 5322,  167871,  5, }, // Africa/Malabo
    {  529, 4987,  154589,  6, }, // Africa/Maputo
    {  529, 5197,   10634,  6, }, // Africa/Maseru
    {  529, 5182,  167876,  6, }, // Africa/Mbabane
    {  529, 5113,  167882,  8, }, // Africa/Mogadishu
    {  529, 7034,  154615,  8, }, // Africa/Monrovia
    {  529, 3853,   10662,  6, }, // Africa/Nairobi
    {  529, 5271,   10668,  7, }, // Africa/Ndjamena
    {  529, 5354,   10675,  5, }, // Africa/Niamey
    {  529, 6803,  167890,  7, }, // Africa/Nouakchott
    {  529, 6727,  167897, 12, }, // Africa/Ouagadougou
    {  529, 5225,  167909,  9, }, // Africa/Porto-Novo
    {  529, 4435,  167918,  8, }, // Africa/Sao_Tome
    {  529, 2866,  167926,  6, }, // Africa/Tripoli
    {  529, 6554,  167932,  5, }, // Africa/Tunis
    {  529, 4152,  167937,  6, }, // Africa/Windhoek
    {  529,  132,  167943,  3, }, // America/Adak
    {  529, 3346,  167946,  7, }, // America/Anchorage
    {  529, 5607,  167953,  7, }, // America/Anguilla
    {  529, 5624,  167960,  8, }, // America/Antigua
    {  529, 4554,  167968,  9, }, // America/Araguaina
    {  529,  166,  167977, 12, }, // America/Argentina/Buenos_Aires
    {  529,   91,  167989, 10, }, // America/Argentina/Catamarca
    {  529,  270,  167999,  7, }, // America/Argentina/Cordoba
    {  529,  441,  168006,  6, }, // America/Argentina/Jujuy
    {  529, 7462,  168012,  9, }, // America/Argentina/La_Rioja
    {  529,  565,  168021,  7, }, // America/Argentina/Mendoza
    {  529, 7489,  168028, 11, }, // America/Argentina/Rio_Gallegos
    {  529, 7520,  168039,  5, }, // America/Argentina/Salta
    {  529, 7544,  168044,  8, }, // America/Argentina/San_Juan
    {  529, 5580,  168052,  9, }, // America/Argentina/San_Luis
    {  529, 7571,  168061,  7, }, // America/Argentina/Tucuman
    {  529, 7597,  168068,  8, }, // America/Argentina/Ushuaia
    {  529, 5640,  154793,  5, }, // America/Aruba
    {  529, 4252,  168076,  8, }, // America/Asuncion
    {  529,  237,  168084,  7, }, // America/Atikokan
    {  529, 3679,  168091,  5, }, // America/Bahia
    {  529, 7623,  168096, 13, }, // America/Bahia_Banderas
    {  529, 5654,  168109,  8, }, // America/Barbados
    {  529, 7646,  168117,  4, }, // America/Belem
    {  529, 5407,  168121,  5, }, // America/Belize
    {  529, 7660,  168126, 11, }, // America/Blanc-Sablon
    {  529, 7681,  168137,  9, }, // America/Boa_Vista
    {  529, 4361,  168146,  5, }, // America/Bogota
    {  529, 7699,  168151,  4, }, // America/Boise
    {  529, 7713,  168155,  9, }, // America/Cambridge_Bay
    {  529, 7735,  168164, 12, }, // America/Campo_Grande
    {  529, 3868,  168176,  6, }, // America/Cancun
    {  529, 4694,  168182,  5, }, // America/Caracas
    {  529, 4345,  168187,  5, }, // America/Cayenne
    {  529, 5496,  168192,  5, }, // America/Cayman
    {  529, 2260,  168197,  5, }, // America/Chicago
    {  529, 7756,  168202, 10, }, // America/Chihuahua
    {  529, 7774,  168212, 11, }, // America/Ciudad_Juarez
    {  529, 5422,  168223, 10, }, // America/Costa_Rica
    {  529, 7796,  168233,  6, }, // America/Creston
    {  529, 3791,  168239,  7, }, // America/Cuiaba
    {  529, 5723,  168246,  9, }, // America/Curacao
    {  529, 7812,  168255, 11, }, // America/Danmarkshavn
    {  529, 7833,  168266,  5, }, // America/Dawson
    {  529, 7848,  168271, 10, }, // America/Dawson_Creek
    {  529,  805,  168281,  5, }, // America/Denver
    {  529, 3465,  168286,  8, }, // America/Detroit
    {  529, 5739,  168294,  8, }, // America/Dominica
    {  529,  893,  168302,  8, }, // America/Edmonton
    {  529, 7869,  168310,  7, }, // America/Eirunepe
    {  529, 5441,  168317, 11, }, // America/El_Salvador
    {  529, 7886,  168328, 10, }, // America/Fort_Nelson
    {  529, 7906,  168338,  9, }, // America/Fortaleza
    {  529, 7924,  168347,  7, }, // America/Glace_Bay
    {  529, 6881,  152416,  6, }, // America/Goose_Bay
    {  529, 4612,  168354,  9, }, // America/Grand_Turk
    {  529, 5770,  168363,  7, }, // America/Grenada
    {  529, 5786,  168370,  8, }, // America/Guadeloupe
    {  529, 3760,  168378, 10, }, // America/Guatemala
    {  529, 4841,  168388,  8, }, // America/Guayaquil
    {  529, 6932,  168396,  5, }, // America/Guyana
    {  529, 1939,  168401,  9, }, // America/Halifax
    {  529, 2281,  152466,  5, }, // America/Havana
    {  529, 7942,  168410,  7, }, // America/Hermosillo
    {  529,  348,  168417, 12, }, // America/Indiana/Indianapolis
    {  529,  481,  168429, 13, }, // America/Indiana/Knox
    {  529, 7961,  168442, 15, }, // America/Indiana/Marengo
    {  529, 7985,  168457, 17, }, // America/Indiana/Petersburg
    {  529, 8012,  168474, 16, }, // America/Indiana/Tell_City
    {  529, 8038,  168490, 13, }, // America/Indiana/Vevay
    {  529, 8060,  168503, 16, }, // America/Indiana/Vincennes
    {  529, 8086,  168519, 16, }, // America/Indiana/Winamac
    {  529, 8110,  168535,  5, }, // America/Inuvik
    {  529,  660,  168540,  7, }, // America/Iqaluit
    {  529, 2799,  168547,  6, }, // America/Jamaica
    {  529, 5380,  168553,  5, }, // America/Juneau
    {  529,  521,  168558,  8, }, // America/Kentucky/Louisville
    {  529, 8125,  168566, 17, }, // America/Kentucky/Monticello
    {  529, 5704,  168583,  8, }, // America/Kralendijk
    {  529, 4376,  168591,  5, }, // America/La_Paz
    {  529, 7169,   11338,  4, }, // America/Lima
    {  529, 3239,  168596, 10, }, // America/Los_Angeles
    {  529, 5932,  168606, 17, }, // America/Lower_Princes
    {  529, 8153,  168623,  6, }, // America/Maceio
    {  529, 8168,  155307,  7, }, // America/Managua
    {  529, 1908,  168629,  6, }, // America/Manaus
    {  529, 5897,  168635,  7, }, // America/Marigot
    {  529, 5805,  168642,  7, }, // America/Martinique
    {  529, 8184,  168649,  8, }, // America/Matamoros
    {  529, 2917,  168657,  7, }, // America/Mazatlan
    {  529, 8202,  168664,  8, }, // America/Menominee
    {  529, 8220,  168672,  6, }, // America/Merida
    {  529, 8235,  168678, 11, }, // America/Metlakatla
    {  529, 2949,  168689, 11, }, // America/Mexico_City
    {  529, 4391,  168700,  9, }, // America/Miquelon
    {  529, 8254,  168709,  6, }, // America/Moncton
    {  529, 8270,  168715,  7, }, // America/Monterrey
    {  529, 4098,  168722, 11, }, // America/Montevideo
    {  529, 5824,  168733,  8, }, // America/Montserrat
    {  529, 5481,  155399,  6, }, // America/Nassau
    {  529, 2379,  168741,  7, }, // America/New_York
    {  529, 8288,   11422,  3, }, // America/Nome
    {  529, 1850,  168748,  8, }, // America/Noronha
    {  529, 8301,  168756, 17, }, // America/North_Dakota/Beulah
    {  529, 8329,  168773, 16, }, // America/North_Dakota/Center
    {  529, 8357,  168789, 19, }, // America/North_Dakota/New_Salem
    {  529,  393,  168808,  4, }, // America/Nuuk
    {  529, 8388,  168812,  8, }, // America/Ojinaga
    {  529, 2356,  168820,  5, }, // America/Panama
    {  529, 6173,  168825, 10, }, // America/Paramaribo
    {  529, 2973,  168835,  6, }, // America/Phoenix
    {  529, 3945,  168841, 12, }, // America/Port-au-Prince
    {  529, 5954,  168853, 12, }, // America/Port_of_Spain
    {  529, 8404,  168865, 11, }, // America/Porto_Velho
    {  529, 5843,  168876, 10, }, // America/Puerto_Rico
    {  529, 4030,  168886, 11, }, // America/Punta_Arenas
    {  529, 8424,  168897, 11, }, // America/Rankin_Inlet
    {  529, 8445,  168908,  6, }, // America/Recife
    {  529, 1995,  168914,  6, }, // America/Regina
    {  529, 8460,  168920,  8, }, // America/Resolute
    {  529,  695,  168928, 10, }, // America/Rio_Branco
    {  529, 8477,  168938,  7, }, // America/Santarem
    {  529, 2201,  168945,  8, }, // America/Santiago
    {  529, 6111,  168953, 13, }, // America/Santo_Domingo
    {  529, 1878,  168966,  9, }, // America/Sao_Paulo
    {  529, 6899,  168975, 12, }, // America/Scoresbysund
    {  529, 8494,  168987,  4, }, // America/Sitka
    {  529, 8508,  168991, 12, }, // America/St_Barthelemy
    {  529, 2061,  169003,  9, }, // America/St_Johns
    {  529, 5863,  169012,  8, }, // America/St_Kitts
    {  529, 5880,  169020, 10, }, // America/St_Lucia
    {  529,  855,  169030,  9, }, // America/St_Thomas
    {  529, 5913,  169039, 11, }, // America/St_Vincent
    {  529, 8530,  169050, 10, }, // America/Swift_Current
    {  529, 5461,  169060, 10, }, // America/Tegucigalpa
    {  529, 5756,  169070,  4, }, // America/Thule
    {  529,  313,  169074,  7, }, // America/Tijuana
    {  529,  608,  169081,  6, }, // America/Toronto
    {  529, 5688,  169087,  7, }, // America/Tortola
    {  529, 2093,  169094,  7, }, // America/Vancouver
    {  529, 2144,  169101, 11, }, // America/Whitehorse
    {  529,  734,  169112,  7, }, // America/Winnipeg
    {  529, 8552,  169119,  7, }, // America/Yakutat
    {  529, 6001,  169126,  4, }, // Antarctica/Casey
    {  529, 6094,  169130,  4, }, // Antarctica/Davis
    {  529, 6133,  169134, 13, }, // Antarctica/DumontDUrville
    {  529, 8568,  169147,  7, }, // Antarctica/Macquarie
    {  529, 7066,  169154,  4, }, // Antarctica/Mawson
    {  529, 7113,  169158,  8, }, // Antarctica/McMurdo
    {  529, 8589,  169166,  4, }, // Antarctica/Palmer
    {  529, 7226,  169170,  5, }, // Antarctica/Rothera
    {  529, 7295,  169175,  5, }, // Antarctica/Syowa
    {  529, 8607,  169180,  4, }, // Antarctica/Troll
    {  529, 7374,  169184,  6, }, // Antarctica/Vostok
    {  529, 1429,  169190, 11, }, // Arctic/Longyearbyen
    {  529, 5570,   11948,  3, }, // Asia/Aden
    {  529, 5395,  169201,  6, }, // Asia/Almaty
    {  529, 3968,   11379,  4, }, // Asia/Amman
    {  529, 5511,  169207,  5, }, // Asia/Anadyr
    {  529, 5523,  169212,  6, }, // Asia/Aqtau
    {  529, 5368,  169218,  6, }, // Asia/Aqtobe
    {  529,  964,  169224,  7, }, // Asia/Ashgabat
    {  529, 8624,  169231,  7, }, // Asia/Atyrau
    {  529, 3607,   11991,  5, }, // Asia/Baghdad
    {  529, 5534,   11998,  5, }, // Asia/Bahrain
    {  529, 3653,  168004,  4, }, // Asia/Baku
    {  529, 4466,  169238,  6, }, // Asia/Bangkok
    {  529, 3571,  169244,  4, }, // Asia/Barnaul
    {  529, 4086,   12016,  5, }, // Asia/Beirut
    {  529, 3778,  169248,  5, }, // Asia/Bishkek
    {  529, 5989,  169253,  7, }, // Asia/Brunei
    {  529, 4601,  108310,  4, }, // Asia/Chita
    {  529, 4511,  169260,  6, }, // Asia/Colombo
    {  529, 4540,   12044,  4, }, // Asia/Damascus
    {  529, 1093,  169266,  4, }, // Asia/Dhaka
    {  529, 6192,  153319,  3, }, // Asia/Dili
    {  529, 3596,  169270,  4, }, // Asia/Dubai
    {  529, 6159,   12055,  7, }, // Asia/Dushanbe
    {  529, 8636,  155895,  9, }, // Asia/Famagusta
    {  529, 8651,  153340,  3, }, // Asia/Gaza
    {  529, 4795,   12074,  6, }, // Asia/Hebron
    {  529, 1259,  169274,  7, }, // Asia/Ho_Chi_Minh
    {  529, 2704,  169281,  9, }, // Asia/Hong_Kong
    {  529, 4771,  169290,  4, }, // Asia/Hovd
    {  529, 4184,  169294,  6, }, // Asia/Irkutsk
    {  529, 7006,  169300,  6, }, // Asia/Jakarta
    {  529, 6992,  169306,  7, }, // Asia/Jayapura
    {  529, 1290,  169313,  7, }, // Asia/Jerusalem
    {  529, 3560,  169320,  4, }, // Asia/Kabul
    {  529, 4316,  169324,  7, }, // Asia/Kamchatka
    {  529, 4239,  169331,  5, }, // Asia/Karachi
    {  529, 1185,  169336,  6, }, // Asia/Kathmandu
    {  529, 8661,  169342,  7, }, // Asia/Khandyga
    {  529,  992,  169349,  5, }, // Asia/Kolkata
    {  529, 4197,  169354, 11, }, // Asia/Krasnoyarsk
    {  529, 4859,  169365, 10, }, // Asia/Kuala_Lumpur
    {  529, 5976,  169375,  5, }, // Asia/Kuching
    {  529, 5547,  169380,  4, }, // Asia/Kuwait
    {  529, 1211,  169384,  5, }, // Asia/Macau
    {  529, 4017,  169389,  6, }, // Asia/Magadan
    {  529, 1349,  169395,  6, }, // Asia/Makassar
    {  529, 7182,  169401,  5, }, // Asia/Manila
    {  529, 6920,   12250,  4, }, // Asia/Muscat
    {  529, 2554,  169406,  7, }, // Asia/Nicosia
    {  529, 8675,  169413, 10, }, // Asia/Novokuznetsk
    {  529, 4135,  169423, 11, }, // Asia/Novosibirsk
    {  529, 4229,  169434,  5, }, // Asia/Omsk
    {  529, 7145,  169439,  5, }, // Asia/Oral
    {  529, 6961,  169444,  9, }, // Asia/Phnom_Penh
    {  529, 8693,  169453,  8, }, // Asia/Pontianak
    {  529, 4214,  169461, 10, }, // Asia/Pyongyang
    {  529, 5559,   12318,  3, }, // Asia/Qatar
    {  529, 8708,  169471,  7, }, // Asia/Qostanay
    {  529, 4269,  169478,  8, }, // Asia/Qyzylorda
    {  529, 3584,   12340,  4, }, // Asia/Riyadh
    {  529, 4408,  169486,  6, }, // Asia/Sakhalin
    {  529, 7245,   12350,  6, }, // Asia/Samarkand
    {  529, 3279,  169492,  4, }, // Asia/Seoul
    {  529, 1053,  169496,  7, }, // Asia/Shanghai
    {  529, 3300,  156136,  7, }, // Asia/Singapore
    {  529, 4297,  169503, 12, }, // Asia/Srednekolymsk
    {  529, 3263,  169515,  5, }, // Asia/Taipei
    {  529, 4781,  169520,  6, }, // Asia/Tashkent
    {  529, 3915,   12396,  6, }, // Asia/Tbilisi
    {  529, 2772,  153601,  5, }, // Asia/Tehran
    {  529, 1317,  169526,  4, }, // Asia/Thimphu
    {  529, 2821,  169530,  5, }, // Asia/Tokyo
    {  529, 4572,  169535,  4, }, // Asia/Tomsk
    {  529, 1021,  169539, 10, }, // Asia/Ulaanbaatar
    {  529, 1159,  169549,  6, }, // Asia/Urumqi
    {  529, 8722,  169555,  8, }, // Asia/Ust-Nera
    {  529, 6977,  169563,  7, }, // Asia/Vientiane
    {  529, 4710,  169570, 10, }, // Asia/Vladivostok
    {  529, 4828,  169580,  6, }, // Asia/Yakutsk
    {  529, 1235,  169586,  5, }, // Asia/Yangon
    {  529, 3883,  169591,  9, }, // Asia/Yekaterinburg
    {  529, 3747,  153681,  6, }, // Asia/Yerevan
    {  529, 3663,  169600,  5, }, // Atlantic/Azores
    {  529, 5671,   12497,  6, }, // Atlantic/Bermuda
    {  529, 6628,  169605,  5, }, // Atlantic/Canary
    {  529, 3727,  169610,  8, }, // Atlantic/Cape_Verde
    {  529, 1395,  169618,  4, }, // Atlantic/Faroe
    {  529, 8736,  169622,  6, }, // Atlantic/Madeira
    {  529, 2748,  169628,  6, }, // Atlantic/Reykjavik
    {  529, 7272,  169634, 10, }, // Atlantic/South_Georgia
    {  529, 6821,  169644, 11, }, // Atlantic/St_Helena
    {  529, 6644,  169655,  7, }, // Atlantic/Stanley
    {  529, 1670,  169662,  7, }, // Australia/Adelaide
    {  529, 1635,  169669,  6, }, // Australia/Brisbane
    {  529, 1799,  169675,  8, }, // Australia/Broken_Hill
    {  529, 1583,  169683,  5, }, // Australia/Darwin
    {  529, 3637,  169688,  5, }, // Australia/Eucla
    {  529, 1516,  169693,  6, }, // Australia/Hobart
    {  529, 8753,  169699,  6, }, // Australia/Lindeman
    {  529, 1547,  169705,  9, }, // Australia/Lord_Howe
    {  529, 1727,   12624,  7, }, // Australia/Melbourne
    {  529, 1762,  169714,  3, }, // Australia/Perth
    {  529, 1463,  169717,  4, }, // Australia/Sydney
    {  529, 6441,  169721,  9, }, // Europe/Amsterdam
    {  529, 6216,  153822,  6, }, // Europe/Andorra
    {  529, 3620,  156386,  8, }, // Europe/Astrakhan
    {  529, 2300,  169730,  5, }, // Europe/Athens
    {  529, 6336,   12667,  6, }, // Europe/Belgrade
    {  529, 4757,  153847,  4, }, // Europe/Berlin
    {  529, 6488,  169735,  9, }, // Europe/Bratislava
    {  529, 2167,  169744,  5, }, // Europe/Brussels
    {  529, 3928,  169749,  7, }, // Europe/Bucharest
    {  529, 3806,  169756,  7, }, // Europe/Budapest
    {  529, 8772,  169763,  6, }, // Europe/Busingen
    {  529, 2583,  169769,  7, }, // Europe/Chisinau
    {  529, 6289,  169776,  9, }, // Europe/Copenhagen
    {  529, 2338,  169785,  4, }, // Europe/Dublin
    {  529, 6307,  169789,  7, }, // Europe/Gibraltar
    {  529, 8788,  169796,  5, }, // Europe/Guernsey
    {  529, 6612,  169801,  7, }, // Europe/Helsinki
    {  529, 8804,  169808, 12, }, // Europe/Isle_of_Man
    {  529, 1130,  153938,  7, }, // Europe/Istanbul
    {  529, 8823,  169820,  4, }, // Europe/Jersey
    {  529, 3979,  169824, 10, }, // Europe/Kaliningrad
    {  529, 8929,  169834,  5, }, // Europe/Kirov
    {  529, 2527,  169839,  4, }, // Europe/Kyiv
    {  529, 3213,  169843,  4, }, // Europe/Lisbon
    {  529, 6506,  169847,  7, }, // Europe/Ljubljana
    {  529, 2501,  169854,  4, }, // Europe/London
    {  529, 6365,  169858,  7, }, // Europe/Luxembourg
    {  529, 4892,  169865,  5, }, // Europe/Madrid
    {  529, 6397,  169870,  5, }, // Europe/Malta
    {  529, 6582,  153997,  7, }, // Europe/Mariehamn
    {  529, 3693,  169875,  4, }, // Europe/Minsk
    {  529, 6410,  169879,  6, }, // Europe/Monaco
    {  529, 3537,  169885,  5, }, // Europe/Moscow
    {  529, 6458,  105650,  5, }, // Europe/Oslo
    {  529, 4284,  169890,  4, }, // Europe/Paris
    {  529, 6424,  169894,  9, }, // Europe/Podgorica
    {  529, 6275,  105669,  4, }, // Europe/Prague
    {  529, 8837,  156250,  3, }, // Europe/Riga
    {  529, 6324,   10638,  3, }, // Europe/Rome
    {  529, 4331,  106627,  5, }, // Europe/Samara
    {  529, 6470,  169903,  9, }, // Europe/San_Marino
    {  529, 6245,  169912,  7, }, // Europe/Sarajevo
    {  529, 4451,  169919,  5, }, // Europe/Saratov
    {  529, 8849,  169924,  8, }, // Europe/Simferopol
    {  529, 6383,  169932,  6, }, // Europe/Skopje
    {  529, 6599,  109223,  5, }, // Europe/Sofia
    {  529, 6523,  169938,  9, }, // Europe/Stockholm
    {  529, 8867,  169947,  4, }, // Europe/Tallinn
    {  529, 6202,  169951,  5, }, // Europe/Tirane
    {  529, 8882,  169956,  8, }, // Europe/Ulyanovsk
    {  529, 6352,  169964,  4, }, // Europe/Vaduz
    {  529, 6567,  169968,  6, }, // Europe/Vatican
    {  529, 6231,   11276,  4, }, // Europe/Vienna
    {  529, 8899,  169974,  6, }, // Europe/Vilnius
    {  529, 4727,  169980,  9, }, // Europe/Volgograd
    {  529, 3190,  105795,  5, }, // Europe/Warsaw
    {  529, 6261,   12998,  4, }, // Europe/Zagreb
    {  529, 6540,   13002,  5, }, // Europe/Zurich
    {  529, 5078,  169989, 12, }, // Indian/Antananarivo
    {  529, 6947,  105823,  5, }, // Indian/Chagos
    {  529, 6046,  170001,  5, }, // Indian/Christmas
    {  529, 6063,  170006,  5, }, // Indian/Cocos
    {  529, 5029,  170011,  6, }, // Indian/Comoro
    {  529, 6661,  170017,  8, }, // Indian/Kerguelen
    {  529, 7260,   13055,  4, }, // Indian/Mahe
    {  529, 7050,  154158,  6, }, // Indian/Maldives
    {  529, 4069,  170025,  6, }, // Indian/Mauritius
    {  529, 5098,  170031,  5, }, // Indian/Mayotte
    {  529, 7211,  154176,  8, }, // Indian/Reunion
    {  529, 4422,  109407,  4, }, // Pacific/Apia
    {  529,  932,  170036,  6, }, // Pacific/Auckland
    {  529, 3706,  170042, 11, }, // Pacific/Bougainville
    {  529, 3015,  170053,  4, }, // Pacific/Chatham
    {  529, 3157,  170057,  4, }, // Pacific/Chuuk
    {  529, 2237,  170061,  5, }, // Pacific/Easter
    {  529, 7360,  170066,  5, }, // Pacific/Efate
    {  529, 7327,  170071,  7, }, // Pacific/Fakaofo
    {  529, 3902,  154224,  3, }, // Pacific/Fiji
    {  529, 7343,  170078,  7, }, // Pacific/Funafuti
    {  529, 6678,  170085,  9, }, // Pacific/Galapagos
    {  529, 6696,  170094,  7, }, // Pacific/Gambier
    {  529, 3822,  170101, 11, }, // Pacific/Guadalcanal
    {  529, 6018,  104083,  4, }, // Pacific/Guam
    {  529, 2723,  170112,  9, }, // Pacific/Honolulu
    {  529, 3049,  170121,  8, }, // Pacific/Kanton
    {  529, 3998,  170129,  7, }, // Pacific/Kiritimati
    {  529, 7019,  170136,  7, }, // Pacific/Kosrae
    {  529, 2842,  170143,  7, }, // Pacific/Kwajalein
    {  529, 4877,  154287,  5, }, // Pacific/Majuro
    {  529, 4051,  170150,  7, }, // Pacific/Marquesas
    {  529, 8914,  170157,  4, }, // Pacific/Midway
    {  529, 7084,  170161,  5, }, // Pacific/Nauru
    {  529, 7132,  170166,  5, }, // Pacific/Niue
    {  529, 4168,  170171,  7, }, // Pacific/Norfolk
    {  529, 7098,  170178,  7, }, // Pacific/Noumea
    {  529, 3126,  170185,  9, }, // Pacific/Pago_Pago
    {  529, 7155,  170194,  5, }, // Pacific/Palau
    {  529, 7194,  170199,  6, }, // Pacific/Pitcairn
    {  529, 3096,  170205,  7, }, // Pacific/Pohnpei
    {  529, 4807,  170212, 11, }, // Pacific/Port_Moresby
    {  529, 6076,  170223,  9, }, // Pacific/Rarotonga
    {  529, 6031,  170232,  4, }, // Pacific/Saipan
    {  529, 7312,  170236,  5, }, // Pacific/Tahiti
    {  529, 6712,  170241,  5, }, // Pacific/Tarawa
    {  529, 4583,  170246,  9, }, // Pacific/Tongatapu
    {  529, 7392,  170255,  3, }, // Pacific/Wake
    {  529, 7405,  170258,  4, }, // Pacific/Wallis
    {  530,   14,       0,  6, }, // Africa/Asmara Sindhi/Devanagari/India
    {  530, 4435,       6,  8, }, // Africa/Sao_Tome
    {  530, 4252,      14,  8, }, // America/Asuncion
    {  530,  237,      22,  8, }, // America/Atikokan
    {  530, 7623,      30, 17, }, // America/Bahia_Banderas
    {  530, 3868,      47,  6, }, // America/Cancun
    {  530, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  530, 5723,      66,  7, }, // America/Curacao
    {  530,  481,      73, 13, }, // America/Indiana/Knox
    {  530, 7961,      86, 16, }, // America/Indiana/Marengo
    {  530, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  530, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  530, 8038,     139, 14, }, // America/Indiana/Vevay
    {  530, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  530, 8086,     171, 16, }, // America/Indiana/Winamac
    {  530, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  530, 5932,     207, 22, }, // America/Lower_Princes
    {  530, 8220,     229,  6, }, // America/Merida
    {  530, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  530, 8329,     255, 20, }, // America/North_Dakota/Center
    {  530, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  530,  393,     298,  4, }, // America/Nuuk
    {  530, 6899,     302, 16, }, // America/Scoresbysund
    {  530, 8508,     318, 14, }, // America/St_Barthelemy
    {  530, 2061,     332, 10, }, // America/St_Johns
    {  530, 5863,     342,  9, }, // America/St_Kitts
    {  530, 5880,     351,  9, }, // America/St_Lucia
    {  530,  855,     360, 10, }, // America/St_Thomas
    {  530, 5913,     370, 11, }, // America/St_Vincent
    {  530, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  530, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  530, 1185,     413,  9, }, // Asia/Kathmandu
    {  530,  992,     422,  7, }, // Asia/Kolkata
    {  530, 1211,     429,  5, }, // Asia/Macau
    {  530, 1235,     442,  6, }, // Asia/Yangon
    {  530, 1395,     448,  5, }, // Atlantic/Faroe
    {  530, 6821,     453, 10, }, // Atlantic/St_Helena
    {  530, 2527,     463,  4, }, // Europe/Kyiv
    {  530, 7211,     467,  7, }, // Indian/Reunion
    {  530, 3157,     474,  5, }, // Pacific/Chuuk
    {  530, 3096,     479,  7, }, // Pacific/Pohnpei
    {  531, 6788,  170262,  7, }, // Africa/Abidjan Sinhala/Sinhala/Sri Lanka
    {  531, 6760,  170269,  5, }, // Africa/Accra
    {  531, 5059,  170274, 11, }, // Africa/Addis_Ababa
    {  531, 7420,  170285, 10, }, // Africa/Algiers
    {  531,   14,  170295,  7, }, // Africa/Asmara
    {  531,   44,  170302,  4, }, // Africa/Bamako
    {  531, 5257,  170306,  7, }, // Africa/Bangui
    {  531, 6746,  170313,  7, }, // Africa/Banjul
    {  531, 7435,  170320,  4, }, // Africa/Bissau
    {  531, 4957,  170324,  9, }, // Africa/Blantyre
    {  531, 5287,  170333, 10, }, // Africa/Brazzaville
    {  531, 4922,  170343, 10, }, // Africa/Bujumbura
    {  531, 2320,  170353,  5, }, // Africa/Cairo
    {  531, 4117,  170358, 10, }, // Africa/Casablanca
    {  531, 7449,  170368,  6, }, // Africa/Ceuta
    {  531, 6773,  170374,  8, }, // Africa/Conakry
    {  531, 6840,  170382,  4, }, // Africa/Dakar
    {  531, 5130,  170386, 14, }, // Africa/Dar_es_Salaam
    {  531, 5043,  170400,  6, }, // Africa/Djibouti
    {  531, 5243,  170406,  5, }, // Africa/Douala
    {  531, 5166,  170411,  9, }, // Africa/El_Aaiun
    {  531, 6853,  170420, 10, }, // Africa/Freetown
    {  531, 4906,  170430,  7, }, // Africa/Gaborone
    {  531, 5015,  170437,  5, }, // Africa/Harare
    {  531, 4479,  170442, 13, }, // Africa/Johannesburg
    {  531, 4499,  170455,  4, }, // Africa/Juba
    {  531, 5151,  170459,  7, }, // Africa/Kampala
    {  531, 4524,  170466,  8, }, // Africa/Khartoum
    {  531, 4973,  170474,  6, }, // Africa/Kigali
    {  531, 5306,  170480,  8, }, // Africa/Kinshasa
    {  531, 4744,  170488,  6, }, // Africa/Lagos
    {  531, 5336,  170494,  9, }, // Africa/Libreville
    {  531, 6869,  170503,  4, }, // Africa/Lome
    {  531, 5211,  170507,  7, }, // Africa/Luanda
    {  531, 4939,  170514, 10, }, // Africa/Lubumbashi
    {  531, 5001,  170524,  6, }, // Africa/Lusaka
    {  531, 5322,  170530,  4, }, // Africa/Malabo
    {  531, 4987,  170534,  5, }, // Africa/Maputo
    {  531, 5197,  170539,  5, }, // Africa/Maseru
    {  531, 5182,  170544,  7, }, // Africa/Mbabane
    {  531, 5113,  170551,  7, }, // Africa/Mogadishu
    {  531, 7034,  170558, 10, }, // Africa/Monrovia
    {  531, 3853,  170568,  7, }, // Africa/Nairobi
    {  531, 5271,  170575,  7, }, // Africa/Ndjamena
    {  531, 5354,  170582,  6, }, // Africa/Niamey
    {  531, 6803,  170588,  9, }, // Africa/Nouakchott
    {  531, 6727,  170597,  7, }, // Africa/Ouagadougou
    {  531, 5225,  170604, 11, }, // Africa/Porto-Novo
    {  531, 4435,  170615,  8, }, // Africa/Sao_Tome
    {  531, 2866,  170623,  9, }, // Africa/Tripoli
    {  531, 6554,  170632,  8, }, // Africa/Tunis
    {  531, 4152,  170640, 11, }, // Africa/Windhoek
    {  531,  132,  170651,  4, }, // America/Adak
    {  531, 3346,  170655,  8, }, // America/Anchorage
    {  531, 5607,  170663, 10, }, // America/Anguilla
    {  531, 5624,  170673,  9, }, // America/Antigua
    {  531, 4554,  170682,  9, }, // America/Araguaina
    {  531,  166,  170691, 14, }, // America/Argentina/Buenos_Aires
    {  531,   91,  170705,  7, }, // America/Argentina/Catamarca
    {  531,  270,  170712,  8, }, // America/Argentina/Cordoba
    {  531,  441,  170720,  6, }, // America/Argentina/Jujuy
    {  531, 7462,  170726,  9, }, // America/Argentina/La_Rioja
    {  531,  565,  170735,  8, }, // America/Argentina/Mendoza
    {  531, 7489,  170743, 12, }, // America/Argentina/Rio_Gallegos
    {  531, 7520,  170755,  6, }, // America/Argentina/Salta
    {  531, 7544,  170761, 11, }, // America/Argentina/San_Juan
    {  531, 5580,  170772, 11, }, // America/Argentina/San_Luis
    {  531, 7571,  170783,  7, }, // America/Argentina/Tucuman
    {  531, 7597,  170790,  8, }, // America/Argentina/Ushuaia
    {  531, 5640,  170798,  5, }, // America/Aruba
    {  531, 4252,  170803, 11, }, // America/Asuncion
    {  531,  237,  170814,  8, }, // America/Atikokan
    {  531, 3679,  170822,  5, }, // America/Bahia
    {  531, 7623,  170827, 15, }, // America/Bahia_Banderas
    {  531, 5654,  170842,  7, }, // America/Barbados
    {  531, 7646,  170849,  6, }, // America/Belem
    {  531, 5407,  170855,  6, }, // America/Belize
    {  531, 7660,  170861, 16, }, // America/Blanc-Sablon
    {  531, 7681,  170877, 10, }, // America/Boa_Vista
    {  531, 4361,  170887,  6, }, // America/Bogota
    {  531, 7699,  170893,  6, }, // America/Boise
    {  531, 7713,  170899, 17, }, // America/Cambridge_Bay
    {  531, 7735,  170916, 15, }, // America/Campo_Grande
    {  531, 3868,  170931,  8, }, // America/Cancun
    {  531, 4694,  170939,  5, }, // America/Caracas
    {  531, 4345,  170944,  6, }, // America/Cayenne
    {  531, 5496,  170950,  5, }, // America/Cayman
    {  531, 2260,  170955,  6, }, // America/Chicago
    {  531, 7756,  170961,  8, }, // America/Chihuahua
    {  531, 7774,  170969, 17, }, // America/Ciudad_Juarez
    {  531, 5422,  170986,  9, }, // America/Costa_Rica
    {  531, 7796,  170995, 10, }, // America/Creston
    {  531, 3791,  171005,  7, }, // America/Cuiaba
    {  531, 5723,  171012,  7, }, // America/Curacao
    {  531, 7812,  171019, 15, }, // America/Danmarkshavn
    {  531, 7833,  171034,  5, }, // America/Dawson
    {  531, 7848,  171039, 13, }, // America/Dawson_Creek
    {  531,  805,  171052,  7, }, // America/Denver
    {  531, 3465,  171059,  8, }, // America/Detroit
    {  531, 5739,  171067,  8, }, // America/Dominica
    {  531,  893,  171075,  9, }, // America/Edmonton
    {  531, 7869,  171084,  9, }, // America/Eirunepe
    {  531, 5441,  171093, 12, }, // America/El_Salvador
    {  531, 7886,  171105, 12, }, // America/Fort_Nelson
    {  531, 7906,  171117,  9, }, // America/Fortaleza
    {  531, 7924,  171126, 12, }, // America/Glace_Bay
    {  531, 6881,  171138, 10, }, // America/Goose_Bay
    {  531, 4612,  171148, 14, }, // America/Grand_Turk
    {  531, 5770,  171162,  8, }, // America/Grenada
    {  531, 5786,  171170, 10, }, // America/Guadeloupe
    {  531, 3760,  171180,  8, }, // America/Guatemala
    {  531, 4841,  171188, 11, }, // America/Guayaquil
    {  531, 6932,  171199,  5, }, // America/Guyana
    {  531, 1939,  171204,  9, }, // America/Halifax
    {  531, 2281,  171213,  5, }, // America/Havana
    {  531, 7942,  171218,  8, }, // America/Hermosillo
    {  531,  348,  171226, 14, }, // America/Indiana/Indianapolis
    {  531,  481,  171240, 17, }, // America/Indiana/Knox
    {  531, 7961,  171257, 18, }, // America/Indiana/Marengo
    {  531, 7985,  171275, 23, }, // America/Indiana/Petersburg
    {  531, 8012,  171298, 20, }, // America/Indiana/Tell_City
    {  531, 8038,  171318, 15, }, // America/Indiana/Vevay
    {  531, 8060,  171333, 19, }, // America/Indiana/Vincennes
    {  531, 8086,  171352, 18, }, // America/Indiana/Winamac
    {  531, 8110,  171370,  7, }, // America/Inuvik
    {  531,  660,  171377,  9, }, // America/Iqaluit
    {  531, 2799,  171386,  8, }, // America/Jamaica
    {  531, 5380,  171394,  4, }, // America/Juneau
    {  531,  521,  171398, 10, }, // America/Kentucky/Louisville
    {  531, 8125,  171408, 19, }, // America/Kentucky/Monticello
    {  531, 5704,  171427, 14, }, // America/Kralendijk
    {  531, 4376,  171441,  6, }, // America/La_Paz
    {  531, 7169,  171447,  4, }, // America/Lima
    {  531, 3239,  171451, 13, }, // America/Los_Angeles
    {  531, 5932,  171464, 24, }, // America/Lower_Princes
    {  531, 8153,  171488,  7, }, // America/Maceio
    {  531, 8168,  171495,  5, }, // America/Managua
    {  531, 1908,  171500,  6, }, // America/Manaus
    {  531, 5897,  171506,  8, }, // America/Marigot
    {  531, 5805,  171514,  8, }, // America/Martinique
    {  531, 8184,  171522,  8, }, // America/Matamoros
    {  531, 2917,  171530,  8, }, // America/Mazatlan
    {  531, 8202,  171538,  8, }, // America/Menominee
    {  531, 8220,  171546,  6, }, // America/Merida
    {  531, 8235,  171552, 10, }, // America/Metlakatla
    {  531, 2949,  171562, 13, }, // America/Mexico_City
    {  531, 4391,  171575,  9, }, // America/Miquelon
    {  531, 8254,  171584,  9, }, // America/Moncton
    {  531, 8270,  171593,  8, }, // America/Monterrey
    {  531, 4098,  171601, 12, }, // America/Montevideo
    {  531, 5824,  171613, 12, }, // America/Montserrat
    {  531, 5481,  171625,  5, }, // America/Nassau
    {  531, 2379,  171630,  8, }, // America/New_York
    {  531, 8288,  171638,  4, }, // America/Nome
    {  531, 1850,  171642,  8, }, // America/Noronha
    {  531, 8301,  171650, 19, }, // America/North_Dakota/Beulah
    {  531, 8329,  171669, 19, }, // America/North_Dakota/Center
    {  531, 8357,  171688, 22, }, // America/North_Dakota/New_Salem
    {  531,  393,  171710,  4, }, // America/Nuuk
    {  531, 8388,  171714,  6, }, // America/Ojinaga
    {  531, 2356,  171720,  5, }, // America/Panama
    {  531, 6173,  171725,  9, }, // America/Paramaribo
    {  531, 2973,  171734,  8, }, // America/Phoenix
    {  531, 3945,  171742, 18, }, // America/Port-au-Prince
    {  531, 5954,  171760, 19, }, // America/Port_of_Spain
    {  531, 8404,  171779, 13, }, // America/Porto_Velho
    {  531, 5843,  171792, 10, }, // America/Puerto_Rico
    {  531, 4030,  171802, 14, }, // America/Punta_Arenas
    {  531, 8424,  171816, 16, }, // America/Rankin_Inlet
    {  531, 8445,  171832,  6, }, // America/Recife
    {  531, 1995,  171838,  6, }, // America/Regina
    {  531, 8460,  171844,  8, }, // America/Resolute
    {  531,  695,  171852, 13, }, // America/Rio_Branco
    {  531, 8477,  171865,  8, }, // America/Santarem
    {  531, 2201,  171873,  9, }, // America/Santiago
    {  531, 6111,  171882, 15, }, // America/Santo_Domingo
    {  531, 1878,  171897,  9, }, // America/Sao_Paulo
    {  531, 6899,  171906, 13, }, // America/Scoresbysund
    {  531, 8494,  171919,  6, }, // America/Sitka
    {  531, 8508,  171925, 14, }, // America/St_Barthelemy
    {  531, 2061,  171939, 12, }, // America/St_Johns
    {  531, 5863,  171951, 12, }, // America/St_Kitts
    {  531, 5880,  171963, 12, }, // America/St_Lucia
    {  531,  855,  171975, 11, }, // America/St_Thomas
    {  531, 5913,  171986, 15, }, // America/St_Vincent
    {  531, 8530,  172001, 15, }, // America/Swift_Current
    {  531, 5461,  172016, 11, }, // America/Tegucigalpa
    {  531, 5756,  172027,  4, }, // America/Thule
    {  531,  313,  172031,  7, }, // America/Tijuana
    {  531,  608,  172038,  7, }, // America/Toronto
    {  531, 5688,  172045,  8, }, // America/Tortola
    {  531, 2093,  172053,  9, }, // America/Vancouver
    {  531, 2144,  172062, 11, }, // America/Whitehorse
    {  531,  734,  172073,  8, }, // America/Winnipeg
    {  531, 8552,  172081,  6, }, // America/Yakutat
    {  531, 6001,  172087,  4, }, // Antarctica/Casey
    {  531, 6094,  172091,  6, }, // Antarctica/Davis
    {  531, 6133,  172097, 18, }, // Antarctica/DumontDUrville
    {  531, 8568,  172115,  8, }, // Antarctica/Macquarie
    {  531, 7066,  172123,  6, }, // Antarctica/Mawson
    {  531, 7113,  172129, 10, }, // Antarctica/McMurdo
    {  531, 8589,  172139,  7, }, // Antarctica/Palmer
    {  531, 7226,  172146,  6, }, // Antarctica/Rothera
    {  531, 7295,  172152,  6, }, // Antarctica/Syowa
    {  531, 8607,  172158,  7, }, // Antarctica/Troll
    {  531, 7374,  172165,  8, }, // Antarctica/Vostok
    {  531, 1429,  172173, 16, }, // Arctic/Longyearbyen
    {  531, 5570,  172189,  5, }, // Asia/Aden
    {  531, 5395,  172194,  6, }, // Asia/Almaty
    {  531, 3968,  172200,  7, }, // Asia/Amman
    {  531, 5511,  172207,  9, }, // Asia/Anadyr
    {  531, 5523,  172216,  5, }, // Asia/Aqtau
    {  531, 5368,  172221,  7, }, // Asia/Aqtobe
    {  531,  964,  172228,  8, }, // Asia/Ashgabat
    {  531, 8624,  172236,  6, }, // Asia/Atyrau
    {  531, 3607,  172242,  8, }, // Asia/Baghdad
    {  531, 5534,  172250,  6, }, // Asia/Bahrain
    {  531, 3653,  171010,  4, }, // Asia/Baku
    {  531, 4466,  172256,  7, }, // Asia/Bangkok
    {  531, 3571,  172263,  7, }, // Asia/Barnaul
    {  531, 4086,  172270,  6, }, // Asia/Beirut
    {  531, 3778,  172276,  8, }, // Asia/Bishkek
    {  531, 5989,  172284,  6, }, // Asia/Brunei
    {  531, 4601,  172290,  4, }, // Asia/Chita
    {  531, 4511,  172294,  4, }, // Asia/Colombo
    {  531, 4540,  172298,  8, }, // Asia/Damascus
    {  531, 1093,  172306,  3, }, // Asia/Dhaka
    {  531, 6192,  172309,  4, }, // Asia/Dili
    {  531, 3596,  172313,  6, }, // Asia/Dubai
    {  531, 6159,  172319,  7, }, // Asia/Dushanbe
    {  531, 8636,  172326,  9, }, // Asia/Famagusta
    {  531, 8651,  172335,  4, }, // Asia/Gaza
    {  531, 4795,  172339,  8, }, // Asia/Hebron
    {  531, 1259,  172347, 13, }, // Asia/Ho_Chi_Minh
    {  531, 2704,  172360,  6, }, // Asia/Hong_Kong
    {  531, 4771,  172366,  6, }, // Asia/Hovd
    {  531, 4184,  172372, 10, }, // Asia/Irkutsk
    {  531, 7006,  172382,  6, }, // Asia/Jakarta
    {  531, 6992,  172388,  5, }, // Asia/Jayapura
    {  531, 1290,  172393,  7, }, // Asia/Jerusalem
    {  531, 3560,  172400,  6, }, // Asia/Kabul
    {  531, 4316,  172406,  8, }, // Asia/Kamchatka
    {  531, 4239,  172414,  8, }, // Asia/Karachi
    {  531, 1185,  172422,  8, }, // Asia/Kathmandu
    {  531, 8661,  172430,  7, }, // Asia/Khandyga
    {  531,  992,  172437,  6, }, // Asia/Kolkata
    {  531, 4197,  172443, 16, }, // Asia/Krasnoyarsk
    {  531, 4859,  172459, 13, }, // Asia/Kuala_Lumpur
    {  531, 5976,  172472,  5, }, // Asia/Kuching
    {  531, 5547,  172477,  6, }, // Asia/Kuwait
    {  531, 1211,  172483,  6, }, // Asia/Macau
    {  531, 4017,  172489,  5, }, // Asia/Magadan
    {  531, 1349,  172494,  9, }, // Asia/Makassar
    {  531, 7182,  172503,  6, }, // Asia/Manila
    {  531, 6920,  172509,  6, }, // Asia/Muscat
    {  531, 2554,  172515,  8, }, // Asia/Nicosia
    {  531, 8675,  172523, 16, }, // Asia/Novokuznetsk
    {  531, 4135,  172539, 13, }, // Asia/Novosibirsk
    {  531, 4229,  172552,  7, }, // Asia/Omsk
    {  531, 7145,  172559,  5, }, // Asia/Oral
    {  531, 6961,  172564,  9, }, // Asia/Phnom_Penh
    {  531, 8693,  172573, 10, }, // Asia/Pontianak
    {  531, 4214,  172583, 14, }, // Asia/Pyongyang
    {  531, 5559,  172597,  5, }, // Asia/Qatar
    {  531, 8708,  172602,  8, }, // Asia/Qostanay
    {  531, 4269,  172610, 12, }, // Asia/Qyzylorda
    {  531, 3584,  172622,  6, }, // Asia/Riyadh
    {  531, 4408,  172628,  8, }, // Asia/Sakhalin
    {  531, 7245,  172636, 11, }, // Asia/Samarkand
    {  531, 3279,  172647,  4, }, // Asia/Seoul
    {  531, 1053,  172651,  7, }, // Asia/Shanghai
    {  531, 3300,  172658, 11, }, // Asia/Singapore
    {  531, 4297,  172669, 18, }, // Asia/Srednekolymsk
    {  531, 3263,  172687,  6, }, // Asia/Taipei
    {  531, 4781,  172693,  9, }, // Asia/Tashkent
    {  531, 3915,  172702,  8, }, // Asia/Tbilisi
    {  531, 2772,  172710,  8, }, // Asia/Tehran
    {  531, 1317,  172718,  6, }, // Asia/Thimphu
    {  531, 2821,  172724,  6, }, // Asia/Tokyo
    {  531, 4572,  172730,  8, }, // Asia/Tomsk
    {  531, 1021,  172738, 10, }, // Asia/Ulaanbaatar
    {  531, 1159,  172748,  7, }, // Asia/Urumqi
    {  531, 8722,  172755, 10, }, // Asia/Ust-Nera
    {  531, 6977,  172765, 11, }, // Asia/Vientiane
    {  531, 4710,  172776, 13, }, // Asia/Vladivostok
    {  531, 4828,  172789,  9, }, // Asia/Yakutsk
    {  531, 1235,  172798,  7, }, // Asia/Yangon
    {  531, 3883,  172805, 13, }, // Asia/Yekaterinburg
    {  531, 3747,  172818,  7, }, // Asia/Yerevan
    {  531, 3663,  172825,  5, }, // Atlantic/Azores
    {  531, 5671,  172830,  9, }, // Atlantic/Bermuda
    {  531, 6628,  172839,  5, }, // Atlantic/Canary
    {  531, 3727,  172844, 10, }, // Atlantic/Cape_Verde
    {  531, 1395,  172854,  4, }, // Atlantic/Faroe
    {  531, 8736,  172858,  7, }, // Atlantic/Madeira
    {  531, 2748,  172865, 12, }, // Atlantic/Reykjavik
    {  531, 7272,  172877, 15, }, // Atlantic/South_Georgia
    {  531, 6821,  172892, 10, }, // Atlantic/St_Helena
    {  531, 6644,  172902,  8, }, // Atlantic/Stanley
    {  531, 1670,  172910,  7, }, // Australia/Adelaide
    {  531, 1635,  172917, 11, }, // Australia/Brisbane
    {  531, 1799,  172928, 13, }, // Australia/Broken_Hill
    {  531, 1583,  172941,  6, }, // Australia/Darwin
    {  531, 3637,  172947,  7, }, // Australia/Eucla
    {  531, 1516,  172954,  8, }, // Australia/Hobart
    {  531, 8753,  172962,  9, }, // Australia/Lindeman
    {  531, 1547,  172971, 11, }, // Australia/Lord_Howe
    {  531, 1727,  172982, 10, }, // Australia/Melbourne
    {  531, 1762,  172992,  5, }, // Australia/Perth
    {  531, 1463,  172997,  6, }, // Australia/Sydney
    {  531, 6441,  173003, 12, }, // Europe/Amsterdam
    {  531, 6216,  173015,  8, }, // Europe/Andorra
    {  531, 3620,  173023, 11, }, // Europe/Astrakhan
    {  531, 2300,  173034,  7, }, // Europe/Athens
    {  531, 6336,  173041, 13, }, // Europe/Belgrade
    {  531, 4757,  173054,  7, }, // Europe/Berlin
    {  531, 6488,  173061, 11, }, // Europe/Bratislava
    {  531, 2167,  173072,  9, }, // Europe/Brussels
    {  531, 3928,  173081, 10, }, // Europe/Bucharest
    {  531, 3806,  173091, 10, }, // Europe/Budapest
    {  531, 8772,  173101, 10, }, // Europe/Busingen
    {  531, 2583,  173111,  6, }, // Europe/Chisinau
    {  531, 6289,  173117, 10, }, // Europe/Copenhagen
    {  531, 2338,  173127,  7, }, // Europe/Dublin
    {  531, 6307,  173134, 10, }, // Europe/Gibraltar
    {  531, 8788,  173144,  7, }, // Europe/Guernsey
    {  531, 6612,  173151, 10, }, // Europe/Helsinki
    {  531, 8804,  173161, 14, }, // Europe/Isle_of_Man
    {  531, 1130,  173175, 11, }, // Europe/Istanbul
    {  531, 8823,  173186,  5, }, // Europe/Jersey
    {  531, 3979,  173191, 14, }, // Europe/Kaliningrad
    {  531, 8929,  173205,  6, }, // Europe/Kirov
    {  531, 2527,  173211,  4, }, // Europe/Kyiv
    {  531, 3213,  173215,  7, }, // Europe/Lisbon
    {  531, 6506,  173222, 11, }, // Europe/Ljubljana
    {  531, 2501,  173233,  6, }, // Europe/London
    {  531, 6365,  173239, 12, }, // Europe/Luxembourg
    {  531, 4892,  173251,  8, }, // Europe/Madrid
    {  531, 6397,  173259,  6, }, // Europe/Malta
    {  531, 6582,  173265, 10, }, // Europe/Mariehamn
    {  531, 3693,  173275,  8, }, // Europe/Minsk
    {  531, 6410,  173283,  6, }, // Europe/Monaco
    {  531, 3537,  173289,  7, }, // Europe/Moscow
    {  531, 6458,  173296,  5, }, // Europe/Oslo
    {  531, 4284,  173301,  6, }, // Europe/Paris
    {  531, 6424,  173307, 10, }, // Europe/Podgorica
    {  531, 6275,  173317,  7, }, // Europe/Prague
    {  531, 8837,  173324,  4, }, // Europe/Riga
    {  531, 6324,  173328,  4, }, // Europe/Rome
    {  531, 4331,  173332,  4, }, // Europe/Samara
    {  531, 6470,  173336, 11, }, // Europe/San_Marino
    {  531, 6245,  173347,  7, }, // Europe/Sarajevo
    {  531, 4451,  173354,  6, }, // Europe/Saratov
    {  531, 8849,  173360, 12, }, // Europe/Simferopol
    {  531, 6383,  173372,  8, }, // Europe/Skopje
    {  531, 6599,  173380,  6, }, // Europe/Sofia
    {  531, 6523,  173386, 10, }, // Europe/Stockholm
    {  531, 8867,  173396,  5, }, // Europe/Tallinn
    {  531, 6202,  173401,  6, }, // Europe/Tirane
    {  531, 8882,  173407, 12, }, // Europe/Ulyanovsk
    {  531, 6352,  173419,  5, }, // Europe/Vaduz
    {  531, 6567,  173424,  8, }, // Europe/Vatican
    {  531, 6231,  173432,  5, }, // Europe/Vienna
    {  531, 8899,  173437, 10, }, // Europe/Vilnius
    {  531, 4727,  173447, 13, }, // Europe/Volgograd
    {  531, 3190,  173460,  5, }, // Europe/Warsaw
    {  531, 6261,  173465,  7, }, // Europe/Zagreb
    {  531, 6540,  173472,  6, }, // Europe/Zurich
    {  531, 5078,  173478, 11, }, // Indian/Antananarivo
    {  531, 6947,  173489,  6, }, // Indian/Chagos
    {  531, 6046,  173495, 10, }, // Indian/Christmas
    {  531, 6063,  173505,  6, }, // Indian/Cocos
    {  531, 5029,  173511,  6, }, // Indian/Comoro
    {  531, 6661,  173517, 11, }, // Indian/Kerguelen
    {  531, 7260,  173528,  4, }, // Indian/Mahe
    {  531, 7050,  173532, 10, }, // Indian/Maldives
    {  531, 4069,  173542,  7, }, // Indian/Mauritius
    {  531, 5098,  173549,  5, }, // Indian/Mayotte
    {  531, 7211,  173554,  9, }, // Indian/Reunion
    {  531, 4422,  173563,  5, }, // Pacific/Apia
    {  531,  932,  173568,  8, }, // Pacific/Auckland
    {  531, 3706,  173576,  9, }, // Pacific/Bougainville
    {  531, 3015,  173585,  5, }, // Pacific/Chatham
    {  531, 3157,  173590,  4, }, // Pacific/Chuuk
    {  531, 2237,  173594,  6, }, // Pacific/Easter
    {  531, 7360,  173600,  5, }, // Pacific/Efate
    {  531, 7327,  173605,  7, }, // Pacific/Fakaofo
    {  531, 3902,  173612,  4, }, // Pacific/Fiji
    {  531, 7343,  173616,  7, }, // Pacific/Funafuti
    {  531, 6678,  173623,  7, }, // Pacific/Galapagos
    {  531, 6696,  173630,  9, }, // Pacific/Gambier
    {  531, 3822,  173639, 12, }, // Pacific/Guadalcanal
    {  531, 6018,  173651,  6, }, // Pacific/Guam
    {  531, 2723,  173657,  8, }, // Pacific/Honolulu
    {  531, 3049,  173665,  9, }, // Pacific/Kanton
    {  531, 3998,  173674,  7, }, // Pacific/Kiritimati
    {  531, 7019,  173681,  6, }, // Pacific/Kosrae
    {  531, 2842,  173687, 11, }, // Pacific/Kwajalein
    {  531, 4877,  173698,  5, }, // Pacific/Majuro
    {  531, 4051,  173703, 10, }, // Pacific/Marquesas
    {  531, 8914,  173713,  6, }, // Pacific/Midway
    {  531, 7084,  173719,  5, }, // Pacific/Nauru
    {  531, 7132,  173441,  4, }, // Pacific/Niue
    {  531, 4168,  173724,  6, }, // Pacific/Norfolk
    {  531, 7098,  173730,  6, }, // Pacific/Noumea
    {  531, 3126,  173736,  7, }, // Pacific/Pago_Pago
    {  531, 7155,  173743,  5, }, // Pacific/Palau
    {  531, 7194,  173748, 10, }, // Pacific/Pitcairn
    {  531, 3096,  173758,  8, }, // Pacific/Pohnpei
    {  531, 4807,  173766, 15, }, // Pacific/Port_Moresby
    {  531, 6076,  173781,  9, }, // Pacific/Rarotonga
    {  531, 6031,  173790,  7, }, // Pacific/Saipan
    {  531, 7312,  173797,  5, }, // Pacific/Tahiti
    {  531, 6712,  173802,  5, }, // Pacific/Tarawa
    {  531, 4583,  173807,  9, }, // Pacific/Tongatapu
    {  531, 7392,  173816,  4, }, // Pacific/Wake
    {  531, 7405,  173820,  6, }, // Pacific/Wallis
    {  532,   14,       0,  6, }, // Africa/Asmara Skolt Sami/Latin/Finland
    {  532, 4435,       6,  8, }, // Africa/Sao_Tome
    {  532, 4252,      14,  8, }, // America/Asuncion
    {  532,  237,      22,  8, }, // America/Atikokan
    {  532, 7623,      30, 17, }, // America/Bahia_Banderas
    {  532, 3868,      47,  6, }, // America/Cancun
    {  532, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  532, 5723,      66,  7, }, // America/Curacao
    {  532,  481,      73, 13, }, // America/Indiana/Knox
    {  532, 7961,      86, 16, }, // America/Indiana/Marengo
    {  532, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  532, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  532, 8038,     139, 14, }, // America/Indiana/Vevay
    {  532, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  532, 8086,     171, 16, }, // America/Indiana/Winamac
    {  532, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  532, 5932,     207, 22, }, // America/Lower_Princes
    {  532, 8220,     229,  6, }, // America/Merida
    {  532, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  532, 8329,     255, 20, }, // America/North_Dakota/Center
    {  532, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  532,  393,     298,  4, }, // America/Nuuk
    {  532, 6899,     302, 16, }, // America/Scoresbysund
    {  532, 8508,     318, 14, }, // America/St_Barthelemy
    {  532, 2061,     332, 10, }, // America/St_Johns
    {  532, 5863,     342,  9, }, // America/St_Kitts
    {  532, 5880,     351,  9, }, // America/St_Lucia
    {  532,  855,     360, 10, }, // America/St_Thomas
    {  532, 5913,     370, 11, }, // America/St_Vincent
    {  532, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  532, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  532, 1185,     413,  9, }, // Asia/Kathmandu
    {  532,  992,     422,  7, }, // Asia/Kolkata
    {  532, 1211,     429,  5, }, // Asia/Macau
    {  532, 1235,     442,  6, }, // Asia/Yangon
    {  532, 1395,     448,  5, }, // Atlantic/Faroe
    {  532, 6821,     453, 10, }, // Atlantic/St_Helena
    {  532, 2527,     463,  4, }, // Europe/Kyiv
    {  532, 7211,     467,  7, }, // Indian/Reunion
    {  532, 3157,     474,  5, }, // Pacific/Chuuk
    {  532, 3096,     479,  7, }, // Pacific/Pohnpei
    {  533, 6760,   21174,  5, }, // Africa/Accra Slovak/Latin/Slovakia
    {  533, 5059,    3784, 11, }, // Africa/Addis_Ababa
    {  533, 7420,   57571,  5, }, // Africa/Algiers
    {  533,   14,       0,  6, }, // Africa/Asmara
    {  533, 2320,   57576,  6, }, // Africa/Cairo
    {  533, 6773,   26353,  7, }, // Africa/Conakry
    {  533, 5043,   33154,  7, }, // Africa/Djibouti
    {  533, 5166,  173826,  8, }, // Africa/El_Aaiun
    {  533, 4524,   57582,  7, }, // Africa/Khartoum
    {  533, 6869,   20300,  4, }, // Africa/Lome
    {  533, 5113,  173834,  8, }, // Africa/Mogadishu
    {  533, 5271,   43996,  9, }, // Africa/Ndjamena
    {  533, 4435,  173842, 11, }, // Africa/Sao_Tome
    {  533, 2866,   57624,  8, }, // Africa/Tripoli
    {  533,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  533, 4252,      14,  8, }, // America/Asuncion
    {  533,  237,      22,  8, }, // America/Atikokan
    {  533, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  533, 7646,   35764,  5, }, // America/Belem
    {  533, 4361,   20389,  6, }, // America/Bogota
    {  533, 3868,      47,  6, }, // America/Cancun
    {  533, 5496,  173853, 16, }, // America/Cayman
    {  533, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  533, 5422,   33188,  9, }, // America/Costa_Rica
    {  533, 3791,   26442,  6, }, // America/Cuiaba
    {  533, 5723,      66,  7, }, // America/Curacao
    {  533, 5739,   21548,  8, }, // America/Dominica
    {  533, 7869,   26448,  8, }, // America/Eirunepe
    {  533, 5441,  173869,  8, }, // America/El_Salvador
    {  533,  481,      73, 13, }, // America/Indiana/Knox
    {  533, 7961,      86, 16, }, // America/Indiana/Marengo
    {  533, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  533, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  533, 8038,     139, 14, }, // America/Indiana/Vevay
    {  533, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  533, 8086,     171, 16, }, // America/Indiana/Winamac
    {  533, 2799,   33215,  7, }, // America/Jamaica
    {  533, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  533, 5932,     207, 22, }, // America/Lower_Princes
    {  533, 8153,   26456,  6, }, // America/Maceio
    {  533, 5805,    5142,  8, }, // America/Martinique
    {  533, 2917,   26471,  8, }, // America/Mazatlan
    {  533, 8220,     229,  6, }, // America/Merida
    {  533, 2949,   57469,  6, }, // America/Mexico_City
    {  533, 8301,  173877, 22, }, // America/North_Dakota/Beulah
    {  533, 8329,  173899, 22, }, // America/North_Dakota/Center
    {  533, 8357,  173921, 25, }, // America/North_Dakota/New_Salem
    {  533,  393,     298,  4, }, // America/Nuuk
    {  533, 5843,   33294,  9, }, // America/Puerto_Rico
    {  533, 8477,   26572,  8, }, // America/Santarem
    {  533, 1878,   26580,  9, }, // America/Sao_Paulo
    {  533, 6899,     302, 16, }, // America/Scoresbysund
    {  533, 8508,  173946, 16, }, // America/St_Barthelemy
    {  533, 2061,     332, 10, }, // America/St_Johns
    {  533, 5863,     342,  9, }, // America/St_Kitts
    {  533, 5880,  173962, 11, }, // America/St_Lucia
    {  533,  855,  173973,  9, }, // America/St_Thomas
    {  533, 5913,  173982, 11, }, // America/St_Vincent
    {  533, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  533, 7295,  173993,  4, }, // Antarctica/Syowa
    {  533, 5395,  173997,  8, }, // Asia/Almaty
    {  533, 3968,   57792,  5, }, // Asia/Amman
    {  533, 5523,    5491,  5, }, // Asia/Aqtau
    {  533, 5368,    7488,  6, }, // Asia/Aqtobe
    {  533,  964,   57797,  8, }, // Asia/Ashgabat
    {  533, 3607,    4044,  6, }, // Asia/Baghdad
    {  533, 5534,   57811,  7, }, // Asia/Bahrain
    {  533, 4086,   57818,  6, }, // Asia/Beirut
    {  533, 3778,   33323,  6, }, // Asia/Bishkek
    {  533, 5989,   33329,  6, }, // Asia/Brunei
    {  533, 4601,   57490,  4, }, // Asia/Chita
    {  533, 4511,    5519,  7, }, // Asia/Colombo
    {  533, 4540,    4083,  6, }, // Asia/Damascus
    {  533, 1093,   57831,  5, }, // Asia/Dhaka
    {  533, 3596,   57836,  5, }, // Asia/Dubai
    {  533, 6159,   33336,  7, }, // Asia/Dushanbe
    {  533, 1259,  174005, 16, }, // Asia/Ho_Chi_Minh
    {  533, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  533, 4771,  174021,  5, }, // Asia/Hovd
    {  533, 1290,   33369,  9, }, // Asia/Jerusalem
    {  533, 3560,   57868,  5, }, // Asia/Kabul
    {  533, 4316,   33378,  8, }, // Asia/Kamchatka
    {  533, 4239,  174026,  6, }, // Asia/Karachi
    {  533, 1185,  174032,  9, }, // Asia/Kathmandu
    {  533, 8661,   57888,  8, }, // Asia/Khandyga
    {  533,  992,   57896,  7, }, // Asia/Kolkata
    {  533, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  533, 5976,   33399,  6, }, // Asia/Kuching
    {  533, 5547,    7607,  6, }, // Asia/Kuwait
    {  533, 1211,     429,  5, }, // Asia/Macau
    {  533, 6920,   26750,  6, }, // Asia/Muscat
    {  533, 2554,  174041,  7, }, // Asia/Nicosia
    {  533, 8675,   62352, 11, }, // Asia/Novokuznetsk
    {  533, 7145,  174048,  6, }, // Asia/Oral
    {  533, 6961,  174054, 10, }, // Asia/Phnom_Penh
    {  533, 4214,   57936, 11, }, // Asia/Pyongyang
    {  533, 5559,    4214,  5, }, // Asia/Qatar
    {  533, 8708,    7652,  8, }, // Asia/Qostanay
    {  533, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  533, 3584,   57947,  5, }, // Asia/Riyadh
    {  533, 4408,   57952,  8, }, // Asia/Sakhalin
    {  533, 3279,   57960,  4, }, // Asia/Seoul
    {  533, 1053,   57964,  7, }, // Asia/Shanghai
    {  533, 3300,   20782,  8, }, // Asia/Singapore
    {  533, 3263,   57984,  9, }, // Asia/Taipei
    {  533, 4781,   33457,  7, }, // Asia/Tashkent
    {  533, 2772,   57993,  7, }, // Asia/Tehran
    {  533, 2821,    4238,  5, }, // Asia/Tokyo
    {  533, 1021,  174064,  9, }, // Asia/Ulaanbaatar
    {  533, 1159,   33464,  6, }, // Asia/Urumqi
    {  533, 8722,  174073,  8, }, // Asia/Ust-Nera
    {  533, 6977,   26807,  8, }, // Asia/Vientiane
    {  533, 4828,    7755,  7, }, // Asia/Yakutsk
    {  533, 1235,   20819,  6, }, // Asia/Yangon
    {  533, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  533, 3747,    7780,  7, }, // Asia/Yerevan
    {  533, 3663,  157413,  5, }, // Atlantic/Azores
    {  533, 5671,   58043,  7, }, // Atlantic/Bermuda
    {  533, 6628,  174081, 16, }, // Atlantic/Canary
    {  533, 3727,   58066,  8, }, // Atlantic/Cape_Verde
    {  533, 1395,   58074, 15, }, // Atlantic/Faroe
    {  533, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  533, 7272,  174097, 13, }, // Atlantic/South_Georgia
    {  533, 6821,  174110, 12, }, // Atlantic/St_Helena
    {  533, 3620,  174122,  9, }, // Europe/Astrakhan
    {  533, 2300,  174131,  5, }, // Europe/Athens
    {  533, 6336,  174136,  8, }, // Europe/Belgrade
    {  533, 4757,   20941,  6, }, // Europe/Berlin
    {  533, 2167,   26890,  6, }, // Europe/Brussels
    {  533, 3928,   58137,  8, }, // Europe/Bucharest
    {  533, 3806,   58145,  8, }, // Europe/Budapest
    {  533, 8772,   26905,  8, }, // Europe/Busingen
    {  533, 2583,  174144,  7, }, // Europe/Chisinau
    {  533, 6289,   58160,  5, }, // Europe/Copenhagen
    {  533, 6307,   91853,  9, }, // Europe/Gibraltar
    {  533, 8804,   58173, 10, }, // Europe/Isle_of_Man
    {  533, 2527,   58183,  5, }, // Europe/Kyiv
    {  533, 3213,   33596,  7, }, // Europe/Lisbon
    {  533, 6506,  174151,  7, }, // Europe/Ljubljana
    {  533, 2501,   58194,  6, }, // Europe/London
    {  533, 6365,  174158, 11, }, // Europe/Luxembourg
    {  533, 6410,    5926,  6, }, // Europe/Monaco
    {  533, 3537,   22748,  6, }, // Europe/Moscow
    {  533, 4284,  174169,  5, }, // Europe/Paris
    {  533, 6275,   36153,  5, }, // Europe/Prague
    {  533, 6324,  174174,  3, }, // Europe/Rome
    {  533, 6470,  142831, 10, }, // Europe/San_Marino
    {  533, 6523,   33635,  8, }, // Europe/Stockholm
    {  533, 6202,   21095,  6, }, // Europe/Tirane
    {  533, 8882,  174177,  9, }, // Europe/Ulyanovsk
    {  533, 6567,   58222,  7, }, // Europe/Vatican
    {  533, 6231,  174186,  6, }, // Europe/Vienna
    {  533, 3190,   33646,  7, }, // Europe/Warsaw
    {  533, 6261,  174192,  6, }, // Europe/Zagreb
    {  533, 6540,    4459,  6, }, // Europe/Zurich
    {  533, 6046,  174198, 15, }, // Indian/Christmas
    {  533, 6063,   58260, 16, }, // Indian/Cocos
    {  533, 5029,   58276,  6, }, // Indian/Comoro
    {  533, 6661,  174213, 10, }, // Indian/Kerguelen
    {  533, 7260,   26983,  4, }, // Indian/Mahe
    {  533, 7050,  174223,  7, }, // Indian/Maldives
    {  533, 4069,  174230,  9, }, // Indian/Mauritius
    {  533, 7211,     467,  7, }, // Indian/Reunion
    {  533, 3157,     474,  5, }, // Pacific/Chuuk
    {  533, 2237,  174239, 17, }, // Pacific/Easter
    {  533, 3902,   33704,  5, }, // Pacific/Fiji
    {  533, 6678,   58371,  8, }, // Pacific/Galapagos
    {  533, 2723,    4493,  8, }, // Pacific/Honolulu
    {  533, 3049,    4501,  9, }, // Pacific/Kanton
    {  533, 4051,   58397,  7, }, // Pacific/Marquesas
    {  533, 7098,    4510,  6, }, // Pacific/Noumea
    {  533, 3096,     479,  7, }, // Pacific/Pohnpei
    {  534, 6788,   57564,  7, }, // Africa/Abidjan Slovenian/Latin/Slovenia
    {  534, 6760,    4531,  4, }, // Africa/Accra
    {  534, 5059,   35689, 10, }, // Africa/Addis_Ababa
    {  534, 7420,   57447,  5, }, // Africa/Algiers
    {  534,   14,       0,  6, }, // Africa/Asmara
    {  534, 2320,    4572,  5, }, // Africa/Cairo
    {  534, 5043,   33154,  7, }, // Africa/Djibouti
    {  534, 4524,    6305,  6, }, // Africa/Khartoum
    {  534, 5306,  122029,  7, }, // Africa/Kinshasa
    {  534, 4939,  122051,  9, }, // Africa/Lubumbashi
    {  534, 5113,   57589,  8, }, // Africa/Mogadishu
    {  534, 4435,       6,  8, }, // Africa/Sao_Tome
    {  534, 5607,   33168,  7, }, // America/Anguilla
    {  534, 4252,      14,  8, }, // America/Asuncion
    {  534,  237,      22,  8, }, // America/Atikokan
    {  534, 7623,   64558, 17, }, // America/Bahia_Banderas
    {  534, 3868,   33182,  6, }, // America/Cancun
    {  534, 5496,    6576,  6, }, // America/Cayman
    {  534, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  534, 5422,   33188,  9, }, // America/Costa_Rica
    {  534, 5723,      66,  7, }, // America/Curacao
    {  534, 5739,   21548,  8, }, // America/Dominica
    {  534, 5441,    6688,  8, }, // America/El_Salvador
    {  534, 3760,   33206,  9, }, // America/Guatemala
    {  534, 6932,   57452,  7, }, // America/Guyana
    {  534,  481,      73, 13, }, // America/Indiana/Knox
    {  534, 7961,      86, 16, }, // America/Indiana/Marengo
    {  534, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  534, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  534, 8038,     139, 14, }, // America/Indiana/Vevay
    {  534, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  534, 8086,     171, 16, }, // America/Indiana/Winamac
    {  534, 2799,   33215,  7, }, // America/Jamaica
    {  534, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  534, 5932,     207, 22, }, // America/Lower_Princes
    {  534, 5805,    5142,  8, }, // America/Martinique
    {  534, 8220,    7008,  6, }, // America/Merida
    {  534, 2949,  161781, 16, }, // America/Mexico_City
    {  534, 8301,  174256, 22, }, // America/North_Dakota/Beulah
    {  534, 8329,  174278, 22, }, // America/North_Dakota/Center
    {  534, 8357,  174300, 25, }, // America/North_Dakota/New_Salem
    {  534,  393,     298,  4, }, // America/Nuuk
    {  534, 5843,   33294,  9, }, // America/Puerto_Rico
    {  534, 6899,     302, 16, }, // America/Scoresbysund
    {  534, 8508,  166576, 14, }, // America/St_Barthelemy
    {  534, 2061,     332, 10, }, // America/St_Johns
    {  534, 5863,  174325, 11, }, // America/St_Kitts
    {  534, 5880,  165974, 10, }, // America/St_Lucia
    {  534,  855,  174336,  9, }, // America/St_Thomas
    {  534, 5913,  174345, 13, }, // America/St_Vincent
    {  534, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  534, 5395,    7476,  6, }, // Asia/Almaty
    {  534, 3968,    5487,  4, }, // Asia/Amman
    {  534, 5511,    7482,  6, }, // Asia/Anadyr
    {  534, 5523,    5491,  5, }, // Asia/Aqtau
    {  534, 5368,    7488,  6, }, // Asia/Aqtobe
    {  534,  964,   33310,  7, }, // Asia/Ashgabat
    {  534, 3607,    4044,  6, }, // Asia/Baghdad
    {  534, 5534,   57811,  7, }, // Asia/Bahrain
    {  534, 4086,    7507,  6, }, // Asia/Beirut
    {  534, 3778,   33323,  6, }, // Asia/Bishkek
    {  534, 5989,   33329,  6, }, // Asia/Brunei
    {  534, 4601,   57490,  4, }, // Asia/Chita
    {  534, 4511,    5519,  7, }, // Asia/Colombo
    {  534, 4540,    4083,  6, }, // Asia/Damascus
    {  534, 1093,    4599,  4, }, // Asia/Dhaka
    {  534, 3596,   57836,  5, }, // Asia/Dubai
    {  534, 6159,   33336,  7, }, // Asia/Dushanbe
    {  534, 1259,  174358,  8, }, // Asia/Ho_Chi_Minh
    {  534, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  534, 7006,   33352,  8, }, // Asia/Jakarta
    {  534, 1290,   33369,  9, }, // Asia/Jerusalem
    {  534, 4316,   33378,  8, }, // Asia/Kamchatka
    {  534, 4239,   33386,  6, }, // Asia/Karachi
    {  534, 1185,    5593,  8, }, // Asia/Kathmandu
    {  534, 8661,   33392,  7, }, // Asia/Khandyga
    {  534,  992,   26732,  7, }, // Asia/Kolkata
    {  534, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  534, 5547,    7607,  6, }, // Asia/Kuwait
    {  534, 1211,     429,  5, }, // Asia/Macau
    {  534, 1349,    7618,  7, }, // Asia/Makassar
    {  534, 6920,  174366,  6, }, // Asia/Muscat
    {  534, 2554,   33410,  8, }, // Asia/Nicosia
    {  534, 7145,   57921,  6, }, // Asia/Oral
    {  534, 4214,   33430,  9, }, // Asia/Pyongyang
    {  534, 5559,    4214,  5, }, // Asia/Qatar
    {  534, 8708,    7652,  8, }, // Asia/Qostanay
    {  534, 4269,  174372,  8, }, // Asia/Qyzylorda
    {  534, 3584,   33439,  5, }, // Asia/Riyadh
    {  534, 4408,   33444,  7, }, // Asia/Sakhalin
    {  534, 3279,    7680,  4, }, // Asia/Seoul
    {  534, 1053,   57964,  7, }, // Asia/Shanghai
    {  534, 3300,   20782,  8, }, // Asia/Singapore
    {  534, 4297,    7699, 13, }, // Asia/Srednekolymsk
    {  534, 3263,    7712,  6, }, // Asia/Taipei
    {  534, 4781,   33457,  7, }, // Asia/Tashkent
    {  534, 2772,    7724,  7, }, // Asia/Tehran
    {  534, 1317,   91641,  5, }, // Asia/Thimphu
    {  534, 2821,    4238,  5, }, // Asia/Tokyo
    {  534, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  534, 1159,   33464,  6, }, // Asia/Urumqi
    {  534, 4828,    7755,  7, }, // Asia/Yakutsk
    {  534, 1235,    7762,  6, }, // Asia/Yangon
    {  534, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  534, 3747,   26815,  6, }, // Asia/Yerevan
    {  534, 3663,   33480,  5, }, // Atlantic/Azores
    {  534, 5671,   57494,  7, }, // Atlantic/Bermuda
    {  534, 6628,  174380, 14, }, // Atlantic/Canary
    {  534, 3727,  174394, 17, }, // Atlantic/Cape_Verde
    {  534, 1395,  174411, 12, }, // Atlantic/Faroe
    {  534, 7272,  174423, 13, }, // Atlantic/South_Georgia
    {  534, 6821,  174436, 10, }, // Atlantic/St_Helena
    {  534, 6216,   33506,  6, }, // Europe/Andorra
    {  534, 3620,   33512,  8, }, // Europe/Astrakhan
    {  534, 2300,    5793,  5, }, // Europe/Athens
    {  534, 6336,    7942,  7, }, // Europe/Belgrade
    {  534, 2167,  174446,  7, }, // Europe/Brussels
    {  534, 3928,  127036,  9, }, // Europe/Bucharest
    {  534, 3806,   33539, 10, }, // Europe/Budapest
    {  534, 2583,   33549,  8, }, // Europe/Chisinau
    {  534, 6289,  174453,  9, }, // Europe/Copenhagen
    {  534, 8804,   57538,  8, }, // Europe/Isle_of_Man
    {  534, 2527,   33591,  5, }, // Europe/Kyiv
    {  534, 3213,  157592,  7, }, // Europe/Lisbon
    {  534, 6365,    8054, 10, }, // Europe/Luxembourg
    {  534, 6410,    5926,  6, }, // Europe/Monaco
    {  534, 3537,   22748,  6, }, // Europe/Moscow
    {  534, 4284,   33603,  5, }, // Europe/Paris
    {  534, 6275,   21067,  5, }, // Europe/Prague
    {  534, 6324,   33608,  3, }, // Europe/Rome
    {  534, 6599,   33629,  6, }, // Europe/Sofia
    {  534, 8867,    8132,  5, }, // Europe/Tallinn
    {  534, 6202,   21095,  6, }, // Europe/Tirane
    {  534, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  534, 6567,    5978,  7, }, // Europe/Vatican
    {  534, 6231,  174462,  5, }, // Europe/Vienna
    {  534, 8899,   64377,  5, }, // Europe/Vilnius
    {  534, 3190,   33646,  7, }, // Europe/Warsaw
    {  534, 6540,    4459,  6, }, // Europe/Zurich
    {  534, 6046,  174467, 12, }, // Indian/Christmas
    {  534, 6063,  174479, 14, }, // Indian/Cocos
    {  534, 5029,   64411,  6, }, // Indian/Comoro
    {  534, 7050,   33687,  7, }, // Indian/Maldives
    {  534, 7211,     467,  7, }, // Indian/Reunion
    {  534, 3157,     474,  5, }, // Pacific/Chuuk
    {  534, 2237,  174493, 16, }, // Pacific/Easter
    {  534, 3902,   33704,  5, }, // Pacific/Fiji
    {  534, 2723,    4493,  8, }, // Pacific/Honolulu
    {  534, 3049,    4501,  9, }, // Pacific/Kanton
    {  534, 3096,     479,  7, }, // Pacific/Pohnpei
    {  535,   14,       0,  6, }, // Africa/Asmara Soga/Latin/Uganda
    {  535, 4435,       6,  8, }, // Africa/Sao_Tome
    {  535, 4252,      14,  8, }, // America/Asuncion
    {  535,  237,      22,  8, }, // America/Atikokan
    {  535, 7623,      30, 17, }, // America/Bahia_Banderas
    {  535, 3868,      47,  6, }, // America/Cancun
    {  535, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  535, 5723,      66,  7, }, // America/Curacao
    {  535,  481,      73, 13, }, // America/Indiana/Knox
    {  535, 7961,      86, 16, }, // America/Indiana/Marengo
    {  535, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  535, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  535, 8038,     139, 14, }, // America/Indiana/Vevay
    {  535, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  535, 8086,     171, 16, }, // America/Indiana/Winamac
    {  535, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  535, 5932,     207, 22, }, // America/Lower_Princes
    {  535, 8220,     229,  6, }, // America/Merida
    {  535, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  535, 8329,     255, 20, }, // America/North_Dakota/Center
    {  535, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  535,  393,     298,  4, }, // America/Nuuk
    {  535, 6899,     302, 16, }, // America/Scoresbysund
    {  535, 8508,     318, 14, }, // America/St_Barthelemy
    {  535, 2061,     332, 10, }, // America/St_Johns
    {  535, 5863,     342,  9, }, // America/St_Kitts
    {  535, 5880,     351,  9, }, // America/St_Lucia
    {  535,  855,     360, 10, }, // America/St_Thomas
    {  535, 5913,     370, 11, }, // America/St_Vincent
    {  535, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  535, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  535, 1185,     413,  9, }, // Asia/Kathmandu
    {  535,  992,     422,  7, }, // Asia/Kolkata
    {  535, 1211,     429,  5, }, // Asia/Macau
    {  535, 1235,     442,  6, }, // Asia/Yangon
    {  535, 1395,     448,  5, }, // Atlantic/Faroe
    {  535, 6821,     453, 10, }, // Atlantic/St_Helena
    {  535, 2527,     463,  4, }, // Europe/Kyiv
    {  535, 7211,     467,  7, }, // Indian/Reunion
    {  535, 3157,     474,  5, }, // Pacific/Chuuk
    {  535, 3096,     479,  7, }, // Pacific/Pohnpei
    {  536, 6788,  174509,  8, }, // Africa/Abidjan Somali/Latin/Somalia
    {  536, 6760,    4531,  4, }, // Africa/Accra
    {  536, 7420,  174517,  8, }, // Africa/Algiers
    {  536,   14,       0,  6, }, // Africa/Asmara
    {  536,   44,  174525,  7, }, // Africa/Bamako
    {  536, 5257,  174532,  5, }, // Africa/Bangui
    {  536, 7435,  174537,  6, }, // Africa/Bissau
    {  536, 4957,  174543,  9, }, // Africa/Blantyre
    {  536, 5287,  174552,  9, }, // Africa/Brazzaville
    {  536, 2320,  174561,  7, }, // Africa/Cairo
    {  536, 4117,  174568, 12, }, // Africa/Casablanca
    {  536, 7449,   21231,  5, }, // Africa/Ceuta
    {  536, 6773,  174580,  7, }, // Africa/Conakry
    {  536, 5130,  174587, 10, }, // Africa/Dar_es_Salaam
    {  536, 5043,  174597,  7, }, // Africa/Djibouti
    {  536, 5243,  174604,  7, }, // Africa/Douala
    {  536, 5166,  174611,  9, }, // Africa/El_Aaiun
    {  536, 6853,  174620,  9, }, // Africa/Freetown
    {  536, 4906,  174629,  7, }, // Africa/Gaborone
    {  536, 5015,  174636,  7, }, // Africa/Harare
    {  536, 4479,  174643, 10, }, // Africa/Johannesburg
    {  536, 5151,  174653,  8, }, // Africa/Kampala
    {  536, 4524,  174661,  8, }, // Africa/Khartoum
    {  536, 4744,  174669,  7, }, // Africa/Lagos
    {  536, 5336,  174676,  8, }, // Africa/Libreville
    {  536, 6869,  174684,  4, }, // Africa/Lome
    {  536, 5211,  174688,  8, }, // Africa/Luanda
    {  536, 4939,  174696, 11, }, // Africa/Lubumbashi
    {  536, 5001,  174707,  7, }, // Africa/Lusaka
    {  536, 4987,  174714,  7, }, // Africa/Maputo
    {  536, 5197,  174721,  7, }, // Africa/Maseru
    {  536, 5182,  174728,  9, }, // Africa/Mbabane
    {  536, 5113,   26379,  8, }, // Africa/Mogadishu
    {  536, 7034,  174737,  9, }, // Africa/Monrovia
    {  536, 3853,  174746,  8, }, // Africa/Nairobi
    {  536, 5271,  174754,  8, }, // Africa/Ndjamena
    {  536, 5354,  165283,  6, }, // Africa/Niamey
    {  536, 6803,  174762,  7, }, // Africa/Nouakchott
    {  536, 6727,    4640,  8, }, // Africa/Ouagadougou
    {  536, 5225,  174769, 12, }, // Africa/Porto-Novo
    {  536, 4435,  174781,  8, }, // Africa/Sao_Tome
    {  536, 2866,  174789,  9, }, // Africa/Tripoli
    {  536, 6554,  174798,  6, }, // Africa/Tunis
    {  536, 4152,  174804,  8, }, // Africa/Windhoek
    {  536, 3346,  174812,  8, }, // America/Anchorage
    {  536, 5607,  174820,  8, }, // America/Anguilla
    {  536, 5624,  174828,  8, }, // America/Antigua
    {  536, 4554,  174836, 10, }, // America/Araguaina
    {  536,  166,  174846, 13, }, // America/Argentina/Buenos_Aires
    {  536,   91,  174859, 10, }, // America/Argentina/Catamarca
    {  536,  270,  174869,  8, }, // America/Argentina/Cordoba
    {  536, 7462,  174877,  9, }, // America/Argentina/La_Rioja
    {  536,  565,  174886,  9, }, // America/Argentina/Mendoza
    {  536, 7489,  174895, 12, }, // America/Argentina/Rio_Gallegos
    {  536, 7544,  174907, 10, }, // America/Argentina/San_Juan
    {  536, 5580,  174917,  9, }, // America/Argentina/San_Luis
    {  536, 7571,  174926,  9, }, // America/Argentina/Tucuman
    {  536, 7597,  174935,  7, }, // America/Argentina/Ushuaia
    {  536, 4252,  174942,  9, }, // America/Asuncion
    {  536,  237,  174951,  9, }, // America/Atikokan
    {  536, 3679,  174960,  7, }, // America/Bahia
    {  536, 7623,  174967, 15, }, // America/Bahia_Banderas
    {  536, 5407,  174982,  7, }, // America/Belize
    {  536, 7660,  174989, 13, }, // America/Blanc-Sablon
    {  536, 7681,  175002,  9, }, // America/Boa_Vista
    {  536, 7699,   21446,  5, }, // America/Boise
    {  536, 7713,  175011, 14, }, // America/Cambridge_Bay
    {  536, 7735,  175025, 14, }, // America/Campo_Grande
    {  536, 3868,  175039,  8, }, // America/Cancun
    {  536, 4694,  175047,  8, }, // America/Caracas
    {  536, 4345,  175055,  6, }, // America/Cayenne
    {  536, 5496,  175061,  7, }, // America/Cayman
    {  536, 2260,  175068,  7, }, // America/Chicago
    {  536, 7756,  175075,  8, }, // America/Chihuahua
    {  536, 7774,  175083, 16, }, // America/Ciudad_Juarez
    {  536, 5422,  175099, 11, }, // America/Costa_Rica
    {  536, 7796,  175110,  9, }, // America/Creston
    {  536, 3791,  175119,  7, }, // America/Cuiaba
    {  536, 5723,  175126,  8, }, // America/Curacao
    {  536, 7812,  175134, 13, }, // America/Danmarkshavn
    {  536, 7833,  175147,  6, }, // America/Dawson
    {  536, 7848,  175153, 13, }, // America/Dawson_Creek
    {  536,  805,  175166,  6, }, // America/Denver
    {  536, 3465,  175172,  8, }, // America/Detroit
    {  536, 5739,  175180,  9, }, // America/Dominica
    {  536, 7869,  175189,  8, }, // America/Eirunepe
    {  536, 5441,  175197, 12, }, // America/El_Salvador
    {  536, 7886,  175209, 11, }, // America/Fort_Nelson
    {  536, 7906,  175220,  9, }, // America/Fortaleza
    {  536, 7924,  175229, 11, }, // America/Glace_Bay
    {  536, 6881,  175240,  9, }, // America/Goose_Bay
    {  536, 4612,  175249, 11, }, // America/Grand_Turk
    {  536, 5770,  175260,  9, }, // America/Grenada
    {  536, 5786,  175269, 10, }, // America/Guadeloupe
    {  536, 3760,  175279, 11, }, // America/Guatemala
    {  536, 4841,  175290, 11, }, // America/Guayaquil
    {  536, 6932,  175301,  7, }, // America/Guyana
    {  536, 1939,  175308,  9, }, // America/Halifax
    {  536, 2281,  175317,  7, }, // America/Havana
    {  536, 7942,  175324,  9, }, // America/Hermosillo
    {  536,  348,  175333, 14, }, // America/Indiana/Indianapolis
    {  536,  481,  175347, 16, }, // America/Indiana/Knox
    {  536, 7961,  175363, 19, }, // America/Indiana/Marengo
    {  536, 7985,  175382, 20, }, // America/Indiana/Petersburg
    {  536, 8012,  175402, 19, }, // America/Indiana/Tell_City
    {  536, 8038,  175421, 18, }, // America/Indiana/Vevay
    {  536, 8060,  175439, 20, }, // America/Indiana/Vincennes
    {  536, 8086,  175459, 19, }, // America/Indiana/Winamac
    {  536, 8110,  175478,  7, }, // America/Inuvik
    {  536,  660,  175485,  8, }, // America/Iqaluit
    {  536, 2799,   98674,  7, }, // America/Jamaica
    {  536, 5380,  175493,  7, }, // America/Juneau
    {  536,  521,  175500,  8, }, // America/Kentucky/Louisville
    {  536, 8125,  175508, 19, }, // America/Kentucky/Monticello
    {  536, 5704,  175527, 11, }, // America/Kralendijk
    {  536, 4376,  175538,  8, }, // America/La_Paz
    {  536, 7169,  175546,  5, }, // America/Lima
    {  536, 3239,  175551, 12, }, // America/Los_Angeles
    {  536, 5932,  175563, 22, }, // America/Lower_Princes
    {  536, 8153,  175585,  8, }, // America/Maceio
    {  536, 8168,  175593,  9, }, // America/Managua
    {  536, 1908,  175602,  8, }, // America/Manaus
    {  536, 5897,  175610,  8, }, // America/Marigot
    {  536, 5805,  175618, 14, }, // America/Martinique
    {  536, 2917,  175632,  9, }, // America/Mazatlan
    {  536, 8202,  175641, 10, }, // America/Menominee
    {  536, 8220,  175651,  7, }, // America/Merida
    {  536, 8235,  175658, 11, }, // America/Metlakatla
    {  536, 2949,  165759, 12, }, // America/Mexico_City
    {  536, 4391,  175669,  9, }, // America/Miquelon
    {  536, 8254,  175678,  9, }, // America/Moncton
    {  536, 8270,  175687, 10, }, // America/Monterrey
    {  536, 4098,  175697, 13, }, // America/Montevideo
    {  536, 5824,  175710, 11, }, // America/Montserrat
    {  536, 5481,  175721,  6, }, // America/Nassau
    {  536, 2379,  175727, 10, }, // America/New_York
    {  536, 8288,  141432,  4, }, // America/Nome
    {  536, 8301,  175737, 27, }, // America/North_Dakota/Beulah
    {  536, 8329,  175764, 28, }, // America/North_Dakota/Center
    {  536, 8357,  175792, 31, }, // America/North_Dakota/New_Salem
    {  536,  393,     298,  4, }, // America/Nuuk
    {  536, 8388,  175823,  8, }, // America/Ojinaga
    {  536, 2356,  175831,  7, }, // America/Panama
    {  536, 2973,  175838,  8, }, // America/Phoenix
    {  536, 3945,  175846, 16, }, // America/Port-au-Prince
    {  536, 5954,  175862, 15, }, // America/Port_of_Spain
    {  536, 5843,  175877, 12, }, // America/Puerto_Rico
    {  536, 4030,  175889, 13, }, // America/Punta_Arenas
    {  536, 8424,  175902, 14, }, // America/Rankin_Inlet
    {  536, 8445,  175916,  6, }, // America/Recife
    {  536, 1995,  175922,  7, }, // America/Regina
    {  536, 8460,  175929,  8, }, // America/Resolute
    {  536,  695,  175937, 13, }, // America/Rio_Branco
    {  536, 8477,  175950,  9, }, // America/Santarem
    {  536, 2201,  175959, 10, }, // America/Santiago
    {  536, 6111,  175969, 14, }, // America/Santo_Domingo
    {  536, 1878,  175983, 10, }, // America/Sao_Paulo
    {  536, 6899,  175993, 13, }, // America/Scoresbysund
    {  536, 8494,  176006,  6, }, // America/Sitka
    {  536, 8508,  176012, 14, }, // America/St_Barthelemy
    {  536, 2061,  176026,  8, }, // America/St_Johns
    {  536, 5863,  176034,  9, }, // America/St_Kitts
    {  536, 5880,  176043, 10, }, // America/St_Lucia
    {  536,  855,  176053, 10, }, // America/St_Thomas
    {  536, 5913,  176063, 11, }, // America/St_Vincent
    {  536, 8530,  176074, 13, }, // America/Swift_Current
    {  536, 5461,  176087, 11, }, // America/Tegucigalpa
    {  536, 5756,  176098,  4, }, // America/Thule
    {  536,  313,  176102,  9, }, // America/Tijuana
    {  536, 5688,  176111,  8, }, // America/Tortola
    {  536, 2093,  176119,  9, }, // America/Vancouver
    {  536, 2144,  176128,  9, }, // America/Whitehorse
    {  536,  734,  176137,  7, }, // America/Winnipeg
    {  536, 8552,  176144,  8, }, // America/Yakutat
    {  536, 6001,  176152,  6, }, // Antarctica/Casey
    {  536, 6094,  176158,  5, }, // Antarctica/Davis
    {  536, 6133,  176163, 15, }, // Antarctica/DumontDUrville
    {  536, 8568,  176178, 10, }, // Antarctica/Macquarie
    {  536, 7113,   73886,  8, }, // Antarctica/McMurdo
    {  536, 8589,  176188,  6, }, // Antarctica/Palmer
    {  536, 7226,    5465,  6, }, // Antarctica/Rothera
    {  536, 7295,  176194,  6, }, // Antarctica/Syowa
    {  536, 8607,  176200,  6, }, // Antarctica/Troll
    {  536, 7374,  176206,  6, }, // Antarctica/Vostok
    {  536, 1429,  176212, 11, }, // Arctic/Longyearbyen
    {  536, 5570,  176223,  5, }, // Asia/Aden
    {  536, 5395,    7476,  6, }, // Asia/Almaty
    {  536, 3968,  176228,  6, }, // Asia/Amman
    {  536, 5511,  176234,  7, }, // Asia/Anadyr
    {  536, 5523,  176241,  5, }, // Asia/Aqtau
    {  536, 8624,  176246,  7, }, // Asia/Atyrau
    {  536, 3607,  176253,  7, }, // Asia/Baghdad
    {  536, 5534,  176260,  7, }, // Asia/Bahrain
    {  536, 4466,  176267,  8, }, // Asia/Bangkok
    {  536, 3571,  176275,  9, }, // Asia/Barnaul
    {  536, 4086,  176284,  7, }, // Asia/Beirut
    {  536, 5989,  176291,  7, }, // Asia/Brunei
    {  536, 4601,  176298,  5, }, // Asia/Chita
    {  536, 4511,    5519,  7, }, // Asia/Colombo
    {  536, 4540,  176303,  8, }, // Asia/Damascus
    {  536, 1093,  176311,  6, }, // Asia/Dhaka
    {  536, 3596,   22258,  5, }, // Asia/Dubai
    {  536, 8651,  176317,  4, }, // Asia/Gaza
    {  536, 1259,  176321, 15, }, // Asia/Ho_Chi_Minh
    {  536, 2704,  176336, 11, }, // Asia/Hong_Kong
    {  536, 4771,  176347,  5, }, // Asia/Hovd
    {  536, 4184,  176352,  8, }, // Asia/Irkutsk
    {  536, 7006,  176360,  8, }, // Asia/Jakarta
    {  536, 6992,  176368,  8, }, // Asia/Jayapura
    {  536, 1290,  176376, 11, }, // Asia/Jerusalem
    {  536, 3560,  176387,  7, }, // Asia/Kabul
    {  536, 4316,  176394,  8, }, // Asia/Kamchatka
    {  536, 4239,  176402,  7, }, // Asia/Karachi
    {  536, 1185,    5593,  8, }, // Asia/Kathmandu
    {  536, 8661,  176409,  9, }, // Asia/Khandyga
    {  536,  992,  176418,  8, }, // Asia/Kolkata
    {  536, 4197,  176426, 13, }, // Asia/Krasnoyarsk
    {  536, 4859,  176439, 13, }, // Asia/Kuala_Lumpur
    {  536, 5976,  176452,  6, }, // Asia/Kuching
    {  536, 5547,  117559,  6, }, // Asia/Kuwait
    {  536, 1211,  176458,  5, }, // Asia/Macau
    {  536, 4017,  176463,  7, }, // Asia/Magadan
    {  536, 1349,    7618,  7, }, // Asia/Makassar
    {  536, 7182,  176470,  7, }, // Asia/Manila
    {  536, 6920,    4208,  6, }, // Asia/Muscat
    {  536, 2554,   22378,  8, }, // Asia/Nicosia
    {  536, 8675,  176477, 13, }, // Asia/Novokuznetsk
    {  536, 4135,  176490, 12, }, // Asia/Novosibirsk
    {  536, 6961,  176502,  9, }, // Asia/Phnom_Penh
    {  536, 8693,  176511, 10, }, // Asia/Pontianak
    {  536, 4214,  176521, 10, }, // Asia/Pyongyang
    {  536, 5559,  176531,  6, }, // Asia/Qatar
    {  536, 8708,     434,  8, }, // Asia/Qostanay
    {  536, 4269,  176537,  9, }, // Asia/Qyzylorda
    {  536, 3584,  176546,  6, }, // Asia/Riyadh
    {  536, 7245,  176552,  9, }, // Asia/Samarkand
    {  536, 3279,   57960,  4, }, // Asia/Seoul
    {  536, 1053,  176561,  9, }, // Asia/Shanghai
    {  536, 3300,  176570,  9, }, // Asia/Singapore
    {  536, 4297,  176579, 16, }, // Asia/Srednekolymsk
    {  536, 3263,  176595,  6, }, // Asia/Taipei
    {  536, 4781,  176601,  9, }, // Asia/Tashkent
    {  536, 3915,  176610,  8, }, // Asia/Tbilisi
    {  536, 2772,  176618,  7, }, // Asia/Tehran
    {  536, 1317,   62492,  5, }, // Asia/Thimphu
    {  536, 6977,  176625, 12, }, // Asia/Vientiane
    {  536, 4710,  176637, 12, }, // Asia/Vladivostok
    {  536, 4828,  176144,  5, }, // Asia/Yakutsk
    {  536, 1235,     442,  6, }, // Asia/Yangon
    {  536, 3883,  176649, 13, }, // Asia/Yekaterinburg
    {  536, 3747,  176662,  7, }, // Asia/Yerevan
    {  536, 3663,  176669,  6, }, // Atlantic/Azores
    {  536, 5671,  176675,  8, }, // Atlantic/Bermuda
    {  536, 6628,    4280,  6, }, // Atlantic/Canary
    {  536, 3727,  176683, 11, }, // Atlantic/Cape_Verde
    {  536, 1395,  176694,  5, }, // Atlantic/Faroe
    {  536, 8736,   99004,  6, }, // Atlantic/Madeira
    {  536, 2748,  176699,  9, }, // Atlantic/Reykjavik
    {  536, 7272,  176708, 13, }, // Atlantic/South_Georgia
    {  536, 6821,     453, 10, }, // Atlantic/St_Helena
    {  536, 6644,  176721,  9, }, // Atlantic/Stanley
    {  536, 1670,  176730,  8, }, // Australia/Adelaide
    {  536, 1635,  176738,  8, }, // Australia/Brisbane
    {  536, 1799,  176746, 11, }, // Australia/Broken_Hill
    {  536, 3637,   22590,  5, }, // Australia/Eucla
    {  536, 1516,  176757,  7, }, // Australia/Hobart
    {  536, 1547,  176764,  7, }, // Australia/Lord_Howe
    {  536, 1727,  176771,  7, }, // Australia/Melbourne
    {  536, 1762,  176778,  4, }, // Australia/Perth
    {  536, 1463,   22607,  6, }, // Australia/Sydney
    {  536, 6441,  176782, 10, }, // Europe/Amsterdam
    {  536, 6216,  176792,  7, }, // Europe/Andorra
    {  536, 3620,  176799, 11, }, // Europe/Astrakhan
    {  536, 2300,  176810,  5, }, // Europe/Athens
    {  536, 6336,  176815, 10, }, // Europe/Belgrade
    {  536, 4757,  176825,  7, }, // Europe/Berlin
    {  536, 6488,  176832, 11, }, // Europe/Bratislava
    {  536, 2167,  176843,  9, }, // Europe/Brussels
    {  536, 3928,  176852,  8, }, // Europe/Bucharest
    {  536, 3806,  176860,  8, }, // Europe/Budapest
    {  536, 8772,  176868,  9, }, // Europe/Busingen
    {  536, 2583,  176877,  8, }, // Europe/Chisinau
    {  536, 6289,  176885, 11, }, // Europe/Copenhagen
    {  536, 2338,  176896,  8, }, // Europe/Dublin
    {  536, 8788,  176904,  8, }, // Europe/Guernsey
    {  536, 6612,  176912,  9, }, // Europe/Helsinki
    {  536, 8804,  176921, 12, }, // Europe/Isle_of_Man
    {  536, 1130,  176933,  9, }, // Europe/Istanbul
    {  536, 8823,  176942,  7, }, // Europe/Jersey
    {  536, 8929,  176949,  6, }, // Europe/Kirov
    {  536, 2527,  176955,  6, }, // Europe/Kyiv
    {  536, 6506,  176961,  9, }, // Europe/Ljubljana
    {  536, 2501,  176970,  6, }, // Europe/London
    {  536, 6365,  176976, 10, }, // Europe/Luxembourg
    {  536, 4892,  176986,  7, }, // Europe/Madrid
    {  536, 6582,  176993,  9, }, // Europe/Mariehamn
    {  536, 3693,  177002,  6, }, // Europe/Minsk
    {  536, 6410,    5926,  6, }, // Europe/Monaco
    {  536, 3537,  177008,  6, }, // Europe/Moscow
    {  536, 4284,  177014,  7, }, // Europe/Paris
    {  536, 6424,  177021,  9, }, // Europe/Podgorica
    {  536, 6275,  177030,  6, }, // Europe/Prague
    {  536, 8837,   57359,  5, }, // Europe/Riga
    {  536, 6324,   63736,  5, }, // Europe/Rome
    {  536, 6470,  177036, 11, }, // Europe/San_Marino
    {  536, 6245,  177047,  9, }, // Europe/Sarajevo
    {  536, 4451,  177056,  8, }, // Europe/Saratov
    {  536, 8849,  177064, 10, }, // Europe/Simferopol
    {  536, 6383,  177074,  8, }, // Europe/Skopje
    {  536, 6599,   22787,  6, }, // Europe/Sofia
    {  536, 6523,  177082,  9, }, // Europe/Stockholm
    {  536, 8867,  177091,  6, }, // Europe/Tallinn
    {  536, 6202,  177097,  7, }, // Europe/Tirane
    {  536, 8882,  177104, 10, }, // Europe/Ulyanovsk
    {  536, 6352,  177114,  6, }, // Europe/Vaduz
    {  536, 6567,  177120,  8, }, // Europe/Vatican
    {  536, 6231,  177128,  7, }, // Europe/Vienna
    {  536, 8899,  177135,  8, }, // Europe/Vilnius
    {  536, 4727,  177143,  9, }, // Europe/Volgograd
    {  536, 6261,  177152,  6, }, // Europe/Zagreb
    {  536, 6540,  177158,  7, }, // Europe/Zurich
    {  536, 5078,  177165, 12, }, // Indian/Antananarivo
    {  536, 6947,  177177,  5, }, // Indian/Chagos
    {  536, 6046,  177182,  9, }, // Indian/Christmas
    {  536, 6063,    4465,  5, }, // Indian/Cocos
    {  536, 5029,   22853,  6, }, // Indian/Comoro
    {  536, 6661,  177191,  8, }, // Indian/Kerguelen
    {  536, 7050,  177199,  8, }, // Indian/Maldives
    {  536, 4069,  177207, 11, }, // Indian/Mauritius
    {  536, 5098,  177218,  7, }, // Indian/Mayotte
    {  536, 7211,  177225, 10, }, // Indian/Reunion
    {  536, 4422,  147557,  5, }, // Pacific/Apia
    {  536,  932,  177235,  7, }, // Pacific/Auckland
    {  536, 3706,  177242, 10, }, // Pacific/Bougainville
    {  536, 3015,  177252,  5, }, // Pacific/Chatham
    {  536, 3157,  177257,  4, }, // Pacific/Chuuk
    {  536, 2237,  177261,  6, }, // Pacific/Easter
    {  536, 7327,  177267,  6, }, // Pacific/Fakaofo
    {  536, 6678,  177273,  9, }, // Pacific/Galapagos
    {  536, 6696,  177282,  7, }, // Pacific/Gambier
    {  536, 3822,  177289, 11, }, // Pacific/Guadalcanal
    {  536, 6018,  177300,  5, }, // Pacific/Guam
    {  536, 3049,  127562,  9, }, // Pacific/Kanton
    {  536, 3998,  177305, 11, }, // Pacific/Kiritimati
    {  536, 7019,  177316,  6, }, // Pacific/Kosrae
    {  536, 2842,  177322, 10, }, // Pacific/Kwajalein
    {  536, 4877,  177332,  5, }, // Pacific/Majuro
    {  536, 4051,  177337, 10, }, // Pacific/Marquesas
    {  536, 7084,  177347,  6, }, // Pacific/Nauru
    {  536, 7132,  175727,  5, }, // Pacific/Niue
    {  536, 4168,  177353,  7, }, // Pacific/Norfolk
    {  536, 7098,  177360,  7, }, // Pacific/Noumea
    {  536, 3126,  177367,  9, }, // Pacific/Pago_Pago
    {  536, 7155,  177376,  5, }, // Pacific/Palau
    {  536, 7194,  177381,  8, }, // Pacific/Pitcairn
    {  536, 3096,  177389,  6, }, // Pacific/Pohnpei
    {  536, 4807,  177395, 13, }, // Pacific/Port_Moresby
    {  536, 6031,  177408,  6, }, // Pacific/Saipan
    {  536, 4583,  177414,  9, }, // Pacific/Tongatapu
    {  536, 7405,  177423,  5, }, // Pacific/Wallis
    {  540,   14,       0,  6, }, // Africa/Asmara Southern Kurdish/Arabic/Iran
    {  540, 4435,       6,  8, }, // Africa/Sao_Tome
    {  540, 4252,      14,  8, }, // America/Asuncion
    {  540,  237,      22,  8, }, // America/Atikokan
    {  540, 7623,      30, 17, }, // America/Bahia_Banderas
    {  540, 3868,      47,  6, }, // America/Cancun
    {  540, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  540, 5723,      66,  7, }, // America/Curacao
    {  540,  481,      73, 13, }, // America/Indiana/Knox
    {  540, 7961,      86, 16, }, // America/Indiana/Marengo
    {  540, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  540, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  540, 8038,     139, 14, }, // America/Indiana/Vevay
    {  540, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  540, 8086,     171, 16, }, // America/Indiana/Winamac
    {  540, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  540, 5932,     207, 22, }, // America/Lower_Princes
    {  540, 8220,     229,  6, }, // America/Merida
    {  540, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  540, 8329,     255, 20, }, // America/North_Dakota/Center
    {  540, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  540,  393,     298,  4, }, // America/Nuuk
    {  540, 6899,     302, 16, }, // America/Scoresbysund
    {  540, 8508,     318, 14, }, // America/St_Barthelemy
    {  540, 2061,     332, 10, }, // America/St_Johns
    {  540, 5863,     342,  9, }, // America/St_Kitts
    {  540, 5880,     351,  9, }, // America/St_Lucia
    {  540,  855,     360, 10, }, // America/St_Thomas
    {  540, 5913,     370, 11, }, // America/St_Vincent
    {  540, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  540, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  540, 1185,     413,  9, }, // Asia/Kathmandu
    {  540,  992,     422,  7, }, // Asia/Kolkata
    {  540, 1211,     429,  5, }, // Asia/Macau
    {  540, 1235,     442,  6, }, // Asia/Yangon
    {  540, 1395,     448,  5, }, // Atlantic/Faroe
    {  540, 6821,     453, 10, }, // Atlantic/St_Helena
    {  540, 2527,     463,  4, }, // Europe/Kyiv
    {  540, 7211,     467,  7, }, // Indian/Reunion
    {  540, 3157,     474,  5, }, // Pacific/Chuuk
    {  540, 3096,     479,  7, }, // Pacific/Pohnpei
    {  542,   14,       0,  6, }, // Africa/Asmara Southern Sami/Latin/Sweden
    {  542, 4435,       6,  8, }, // Africa/Sao_Tome
    {  542, 4252,      14,  8, }, // America/Asuncion
    {  542,  237,      22,  8, }, // America/Atikokan
    {  542, 7623,      30, 17, }, // America/Bahia_Banderas
    {  542, 3868,      47,  6, }, // America/Cancun
    {  542, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  542, 5723,      66,  7, }, // America/Curacao
    {  542,  481,      73, 13, }, // America/Indiana/Knox
    {  542, 7961,      86, 16, }, // America/Indiana/Marengo
    {  542, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  542, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  542, 8038,     139, 14, }, // America/Indiana/Vevay
    {  542, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  542, 8086,     171, 16, }, // America/Indiana/Winamac
    {  542, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  542, 5932,     207, 22, }, // America/Lower_Princes
    {  542, 8220,     229,  6, }, // America/Merida
    {  542, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  542, 8329,     255, 20, }, // America/North_Dakota/Center
    {  542, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  542,  393,     298,  4, }, // America/Nuuk
    {  542, 6899,     302, 16, }, // America/Scoresbysund
    {  542, 8508,     318, 14, }, // America/St_Barthelemy
    {  542, 2061,     332, 10, }, // America/St_Johns
    {  542, 5863,     342,  9, }, // America/St_Kitts
    {  542, 5880,     351,  9, }, // America/St_Lucia
    {  542,  855,     360, 10, }, // America/St_Thomas
    {  542, 5913,     370, 11, }, // America/St_Vincent
    {  542, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  542, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  542, 1185,     413,  9, }, // Asia/Kathmandu
    {  542,  992,     422,  7, }, // Asia/Kolkata
    {  542, 1211,     429,  5, }, // Asia/Macau
    {  542, 1235,     442,  6, }, // Asia/Yangon
    {  542, 1395,     448,  5, }, // Atlantic/Faroe
    {  542, 6821,     453, 10, }, // Atlantic/St_Helena
    {  542, 2527,     463,  4, }, // Europe/Kyiv
    {  542, 7211,     467,  7, }, // Indian/Reunion
    {  542, 3157,     474,  5, }, // Pacific/Chuuk
    {  542, 3096,     479,  7, }, // Pacific/Pohnpei
    {  544,   14,       0,  6, }, // Africa/Asmara Southern Sotho/Latin/South Africa
    {  544, 4435,       6,  8, }, // Africa/Sao_Tome
    {  544, 4252,      14,  8, }, // America/Asuncion
    {  544,  237,      22,  8, }, // America/Atikokan
    {  544, 7623,      30, 17, }, // America/Bahia_Banderas
    {  544, 3868,      47,  6, }, // America/Cancun
    {  544, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  544, 5723,      66,  7, }, // America/Curacao
    {  544,  481,      73, 13, }, // America/Indiana/Knox
    {  544, 7961,      86, 16, }, // America/Indiana/Marengo
    {  544, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  544, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  544, 8038,     139, 14, }, // America/Indiana/Vevay
    {  544, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  544, 8086,     171, 16, }, // America/Indiana/Winamac
    {  544, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  544, 5932,     207, 22, }, // America/Lower_Princes
    {  544, 8220,     229,  6, }, // America/Merida
    {  544, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  544, 8329,     255, 20, }, // America/North_Dakota/Center
    {  544, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  544,  393,     298,  4, }, // America/Nuuk
    {  544, 6899,     302, 16, }, // America/Scoresbysund
    {  544, 8508,     318, 14, }, // America/St_Barthelemy
    {  544, 2061,     332, 10, }, // America/St_Johns
    {  544, 5863,     342,  9, }, // America/St_Kitts
    {  544, 5880,     351,  9, }, // America/St_Lucia
    {  544,  855,     360, 10, }, // America/St_Thomas
    {  544, 5913,     370, 11, }, // America/St_Vincent
    {  544, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  544, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  544, 1185,     413,  9, }, // Asia/Kathmandu
    {  544,  992,     422,  7, }, // Asia/Kolkata
    {  544, 1211,     429,  5, }, // Asia/Macau
    {  544, 1235,     442,  6, }, // Asia/Yangon
    {  544, 1395,     448,  5, }, // Atlantic/Faroe
    {  544, 6821,     453, 10, }, // Atlantic/St_Helena
    {  544, 2527,     463,  4, }, // Europe/Kyiv
    {  544, 7211,     467,  7, }, // Indian/Reunion
    {  544, 3157,     474,  5, }, // Pacific/Chuuk
    {  544, 3096,     479,  7, }, // Pacific/Pohnpei
    {  546,   14,       0,  6, }, // Africa/Asmara South Ndebele/Latin/South Africa
    {  546, 4435,       6,  8, }, // Africa/Sao_Tome
    {  546, 4252,      14,  8, }, // America/Asuncion
    {  546,  237,      22,  8, }, // America/Atikokan
    {  546, 7623,      30, 17, }, // America/Bahia_Banderas
    {  546, 3868,      47,  6, }, // America/Cancun
    {  546, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  546, 5723,      66,  7, }, // America/Curacao
    {  546,  481,      73, 13, }, // America/Indiana/Knox
    {  546, 7961,      86, 16, }, // America/Indiana/Marengo
    {  546, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  546, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  546, 8038,     139, 14, }, // America/Indiana/Vevay
    {  546, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  546, 8086,     171, 16, }, // America/Indiana/Winamac
    {  546, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  546, 5932,     207, 22, }, // America/Lower_Princes
    {  546, 8220,     229,  6, }, // America/Merida
    {  546, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  546, 8329,     255, 20, }, // America/North_Dakota/Center
    {  546, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  546,  393,     298,  4, }, // America/Nuuk
    {  546, 6899,     302, 16, }, // America/Scoresbysund
    {  546, 8508,     318, 14, }, // America/St_Barthelemy
    {  546, 2061,     332, 10, }, // America/St_Johns
    {  546, 5863,     342,  9, }, // America/St_Kitts
    {  546, 5880,     351,  9, }, // America/St_Lucia
    {  546,  855,     360, 10, }, // America/St_Thomas
    {  546, 5913,     370, 11, }, // America/St_Vincent
    {  546, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  546, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  546, 1185,     413,  9, }, // Asia/Kathmandu
    {  546,  992,     422,  7, }, // Asia/Kolkata
    {  546, 1211,     429,  5, }, // Asia/Macau
    {  546, 1235,     442,  6, }, // Asia/Yangon
    {  546, 1395,     448,  5, }, // Atlantic/Faroe
    {  546, 6821,     453, 10, }, // Atlantic/St_Helena
    {  546, 2527,     463,  4, }, // Europe/Kyiv
    {  546, 7211,     467,  7, }, // Indian/Reunion
    {  546, 3157,     474,  5, }, // Pacific/Chuuk
    {  546, 3096,     479,  7, }, // Pacific/Pohnpei
    {  547, 6788,  161742,  6, }, // Africa/Abidjan Spanish/Latin/Spain
    {  547, 6760,   74000,  4, }, // Africa/Accra
    {  547, 5059,   20252, 10, }, // Africa/Addis_Ababa
    {  547, 7420,  147563,  5, }, // Africa/Algiers
    {  547,   14,       0,  6, }, // Africa/Asmara
    {  547, 7435,  177428,  5, }, // Africa/Bissau
    {  547, 2320,  161748,  8, }, // Africa/Cairo
    {  547, 6773,  174580,  7, }, // Africa/Conakry
    {  547, 5130,  177433, 12, }, // Africa/Dar_es_Salaam
    {  547, 5043,  147577,  6, }, // Africa/Djibouti
    {  547, 5243,   21254,  5, }, // Africa/Douala
    {  547, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  547, 4479,  161756, 13, }, // Africa/Johannesburg
    {  547, 4524,  147583,  6, }, // Africa/Khartoum
    {  547, 6869,   20300,  4, }, // Africa/Lome
    {  547, 5113,   43978, 10, }, // Africa/Mogadishu
    {  547, 5271,  147589,  6, }, // Africa/Ndjamena
    {  547, 6803,  147595,  8, }, // Africa/Nouakchott
    {  547, 6727,   20331,  8, }, // Africa/Ouagadougou
    {  547, 5225,   61241,  9, }, // Africa/Porto-Novo
    {  547, 4435,  177445, 10, }, // Africa/Sao_Tome
    {  547, 2866,   20349,  7, }, // Africa/Tripoli
    {  547, 6554,   20356,  5, }, // Africa/Tunis
    {  547, 5607,    4657,  7, }, // America/Anguilla
    {  547, 4554,   26427,  9, }, // America/Araguaina
    {  547,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  547, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {  547, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  547, 4252,      14,  8, }, // America/Asuncion
    {  547,  237,      22,  8, }, // America/Atikokan
    {  547, 3679,      30,  5, }, // America/Bahia
    {  547, 7623,      30, 17, }, // America/Bahia_Banderas
    {  547, 7646,  147618,  5, }, // America/Belem
    {  547, 5407,  147623,  6, }, // America/Belize
    {  547, 4361,   20389,  6, }, // America/Bogota
    {  547, 3868,      47,  6, }, // America/Cancun
    {  547, 4345,  147629,  6, }, // America/Cayenne
    {  547, 5496,   20395,  6, }, // America/Cayman
    {  547, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  547, 3791,   26442,  6, }, // America/Cuiaba
    {  547, 5723,  161774,  7, }, // America/Curacao
    {  547, 7869,   26448,  8, }, // America/Eirunepe
    {  547, 4612,  147645, 10, }, // America/Grand_Turk
    {  547, 5770,   20401,  7, }, // America/Grenada
    {  547, 5786,   20408,  9, }, // America/Guadeloupe
    {  547, 2281,   20417,  9, }, // America/Havana
    {  547,  348,   74114, 12, }, // America/Indiana/Indianapolis
    {  547,  481,      73, 13, }, // America/Indiana/Knox
    {  547, 7961,      86, 16, }, // America/Indiana/Marengo
    {  547, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  547, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  547, 8038,     139, 14, }, // America/Indiana/Vevay
    {  547, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  547, 8086,     171, 16, }, // America/Indiana/Winamac
    {  547, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  547, 3239,  177455, 11, }, // America/Los_Angeles
    {  547, 5932,     207, 22, }, // America/Lower_Princes
    {  547, 8153,   26456,  6, }, // America/Maceio
    {  547, 1908,   64774,  6, }, // America/Manaus
    {  547, 5805,   20436,  9, }, // America/Martinique
    {  547, 2917,   26471,  8, }, // America/Mazatlan
    {  547, 8220,     229,  6, }, // America/Merida
    {  547, 2949,   57459, 16, }, // America/Mexico_City
    {  547, 4391,  177466,  8, }, // America/Miquelon
    {  547, 2379,   20460, 10, }, // America/New_York
    {  547, 8301,   20470, 24, }, // America/North_Dakota/Beulah
    {  547, 8329,   20494, 24, }, // America/North_Dakota/Center
    {  547, 8357,  177474, 27, }, // America/North_Dakota/New_Salem
    {  547,  393,     298,  4, }, // America/Nuuk
    {  547, 2356,   20547,  6, }, // America/Panama
    {  547, 3945,  147772, 15, }, // America/Port-au-Prince
    {  547, 5954,  161797, 13, }, // America/Port_of_Spain
    {  547,  695,   74255, 10, }, // America/Rio_Branco
    {  547, 8477,   26572,  8, }, // America/Santarem
    {  547, 2201,   64147, 17, }, // America/Santiago
    {  547, 1878,   26580,  9, }, // America/Sao_Paulo
    {  547, 6899,     302, 16, }, // America/Scoresbysund
    {  547, 8508,  161810, 13, }, // America/St_Barthelemy
    {  547, 2061,  161823, 21, }, // America/St_Johns
    {  547, 5863,  177501, 13, }, // America/St_Kitts
    {  547, 5880,   74265, 11, }, // America/St_Lucia
    {  547,  855,     360, 10, }, // America/St_Thomas
    {  547, 5913,   74276, 11, }, // America/St_Vincent
    {  547, 5688,   20605,  7, }, // America/Tortola
    {  547, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  547, 5570,   74287,  4, }, // Asia/Aden
    {  547, 3968,   57792,  5, }, // Asia/Amman
    {  547, 5511,  147878,  6, }, // Asia/Anadyr
    {  547, 5523,    5491,  5, }, // Asia/Aqtau
    {  547, 5368,    7488,  6, }, // Asia/Aqtobe
    {  547,  964,  147884,  7, }, // Asia/Ashgabat
    {  547, 3607,    4044,  6, }, // Asia/Baghdad
    {  547, 5534,  147891,  6, }, // Asia/Bahrain
    {  547, 3653,   20629,  4, }, // Asia/Baku
    {  547, 3571,  147897,  7, }, // Asia/Barnaul
    {  547, 5989,   20633,  6, }, // Asia/Brunei
    {  547, 4601,   74304,  5, }, // Asia/Chita
    {  547, 4540,   74309,  7, }, // Asia/Damascus
    {  547, 1093,  158367,  4, }, // Asia/Dhaka
    {  547, 3596,  177514,  5, }, // Asia/Dubai
    {  547, 6159,  147904,  7, }, // Asia/Dushanbe
    {  547, 4795,   20653,  6, }, // Asia/Hebron
    {  547, 1259,  177519, 18, }, // Asia/Ho_Chi_Minh
    {  547, 7006,  161868,  7, }, // Asia/Jakarta
    {  547, 1290,  177537,  9, }, // Asia/Jerusalem
    {  547, 1185,   20712,  8, }, // Asia/Kathmandu
    {  547, 8661,   26724,  8, }, // Asia/Khandyga
    {  547,  992,   20720,  7, }, // Asia/Kolkata
    {  547, 1211,     429,  5, }, // Asia/Macau
    {  547, 4017,   20732,  7, }, // Asia/Magadan
    {  547, 1349,    7618,  7, }, // Asia/Makassar
    {  547, 6920,   20739,  7, }, // Asia/Muscat
    {  547, 5559,   73021,  5, }, // Asia/Qatar
    {  547, 8708,  161883,  8, }, // Asia/Qostanay
    {  547, 4269,   26765,  9, }, // Asia/Qyzylorda
    {  547, 3584,    7669,  4, }, // Asia/Riyadh
    {  547, 4408,  147938,  7, }, // Asia/Sakhalin
    {  547, 7245,   20768, 10, }, // Asia/Samarkand
    {  547, 3279,   20778,  4, }, // Asia/Seoul
    {  547, 1053,  177546,  8, }, // Asia/Shanghai
    {  547, 3300,   20782,  8, }, // Asia/Singapore
    {  547, 4297,    7699, 13, }, // Asia/Srednekolymsk
    {  547, 3263,  177554,  6, }, // Asia/Taipei
    {  547, 4781,   91626,  7, }, // Asia/Tashkent
    {  547, 3915,   78108,  6, }, // Asia/Tbilisi
    {  547, 2772,   57993,  7, }, // Asia/Tehran
    {  547, 1317,   62492,  5, }, // Asia/Thimphu
    {  547, 2821,    4238,  5, }, // Asia/Tokyo
    {  547, 1021,  177560, 10, }, // Asia/Ulaanbaatar
    {  547, 1159,   26801,  6, }, // Asia/Urumqi
    {  547, 6977,   20811,  8, }, // Asia/Vientiane
    {  547, 1235,  177570, 15, }, // Asia/Yangon
    {  547, 3883,  177585, 13, }, // Asia/Yekaterinburg
    {  547, 3747,  147956,  6, }, // Asia/Yerevan
    {  547, 5671,   74354,  8, }, // Atlantic/Bermuda
    {  547, 6628,   74368,  8, }, // Atlantic/Canary
    {  547, 3727,   26836, 10, }, // Atlantic/Cape_Verde
    {  547, 1395,  177598, 11, }, // Atlantic/Faroe
    {  547, 2748,   20879,  9, }, // Atlantic/Reykjavik
    {  547, 7272,  161898, 15, }, // Atlantic/South_Georgia
    {  547, 6821,  147989, 11, }, // Atlantic/St_Helena
    {  547, 1670,   20912,  8, }, // Australia/Adelaide
    {  547, 1463,  148000,  6, }, // Australia/Sydney
    {  547, 6441,   74400,  9, }, // Europe/Amsterdam
    {  547, 3620,   20920,  8, }, // Europe/Astrakhan
    {  547, 2300,   26877,  6, }, // Europe/Athens
    {  547, 6336,   58774,  8, }, // Europe/Belgrade
    {  547, 4757,   20941,  6, }, // Europe/Berlin
    {  547, 2167,  161913,  8, }, // Europe/Brussels
    {  547, 3928,   20955,  8, }, // Europe/Bucharest
    {  547, 2583,  142651,  8, }, // Europe/Chisinau
    {  547, 6289,   20963, 10, }, // Europe/Copenhagen
    {  547, 2338,   20973,  6, }, // Europe/Dublin
    {  547, 8788,   26913,  9, }, // Europe/Guernsey
    {  547, 8804,  148006, 11, }, // Europe/Isle_of_Man
    {  547, 1130,  148017,  8, }, // Europe/Istanbul
    {  547, 3979,   62893, 12, }, // Europe/Kaliningrad
    {  547, 8929,   44429,  5, }, // Europe/Kirov
    {  547, 2527,    8030,  4, }, // Europe/Kyiv
    {  547, 3213,   21019,  6, }, // Europe/Lisbon
    {  547, 6506,   21025,  9, }, // Europe/Ljubljana
    {  547, 2501,   21034,  7, }, // Europe/London
    {  547, 6365,   74445, 10, }, // Europe/Luxembourg
    {  547, 6410,   74455,  6, }, // Europe/Monaco
    {  547, 3537,   21057,  5, }, // Europe/Moscow
    {  547, 4284,   21062,  5, }, // Europe/Paris
    {  547, 6275,   21067,  5, }, // Europe/Prague
    {  547, 6324,    5946,  4, }, // Europe/Rome
    {  547, 4451,  148043,  7, }, // Europe/Saratov
    {  547, 8849,   74476, 10, }, // Europe/Simferopol
    {  547, 6383,  148050,  6, }, // Europe/Skopje
    {  547, 6599,   21081,  5, }, // Europe/Sofia
    {  547, 6523,   74486,  9, }, // Europe/Stockholm
    {  547, 8867,   22793,  6, }, // Europe/Tallinn
    {  547, 6202,   21095,  6, }, // Europe/Tirane
    {  547, 8882,  148056,  9, }, // Europe/Ulyanovsk
    {  547, 6567,  161921, 11, }, // Europe/Vatican
    {  547, 6231,   21112,  5, }, // Europe/Vienna
    {  547, 8899,   64377,  5, }, // Europe/Vilnius
    {  547, 4727,   63074, 10, }, // Europe/Volgograd
    {  547, 3190,   21126,  8, }, // Europe/Warsaw
    {  547, 6540,  148076,  6, }, // Europe/Zurich
    {  547, 6046,  177609,  7, }, // Indian/Christmas
    {  547, 5029,  162604,  7, }, // Indian/Comoro
    {  547, 7260,   26983,  4, }, // Indian/Mahe
    {  547, 7050,   74517,  8, }, // Indian/Maldives
    {  547, 4069,   74525,  8, }, // Indian/Mauritius
    {  547, 7211,   21142,  7, }, // Indian/Reunion
    {  547, 3157,     474,  5, }, // Pacific/Chuuk
    {  547, 2237,  177616, 14, }, // Pacific/Easter
    {  547, 3902,  148101,  4, }, // Pacific/Fiji
    {  547, 6678,   21159,  9, }, // Pacific/Galapagos
    {  547, 2723,   74562,  8, }, // Pacific/Honolulu
    {  547, 3049,    4501,  9, }, // Pacific/Kanton
    {  547, 7098,  124531,  5, }, // Pacific/Noumea
    {  547, 7155,   65437,  6, }, // Pacific/Palau
    {  547, 3096,     479,  7, }, // Pacific/Pohnpei
    {  547, 6031,  177630,  6, }, // Pacific/Saipan
    {  547, 7312,   44525,  6, }, // Pacific/Tahiti
    {  548, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Argentina
    {  548, 7886,  177643, 13, }, // America/Fort_Nelson
    {  548, 5481,  177656,  5, }, // America/Nassau
    {  548, 2201,   64147,  8, }, // America/Santiago
    {  548,  855,  177661, 11, }, // America/St_Thomas
    {  548, 8661,  177672,  8, }, // Asia/Khandyga
    {  548, 6628,  177680, 14, }, // Atlantic/Canary
    {  548, 8772,   26905,  8, }, // Europe/Busingen
    {  548, 2338,   20973,  6, }, // Europe/Dublin
    {  548, 8788,  177694,  8, }, // Europe/Guernsey
    {  548, 7392,  177702,  9, }, // Pacific/Wake
    {  549, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Belize
    {  549, 7886,  177643, 13, }, // America/Fort_Nelson
    {  549, 5481,  177656,  5, }, // America/Nassau
    {  549, 2201,   64147,  8, }, // America/Santiago
    {  549,  855,  177661, 11, }, // America/St_Thomas
    {  549, 8661,  177672,  8, }, // Asia/Khandyga
    {  549, 6628,  177680, 14, }, // Atlantic/Canary
    {  549, 8772,   26905,  8, }, // Europe/Busingen
    {  549, 2338,   20973,  6, }, // Europe/Dublin
    {  549, 8788,  177694,  8, }, // Europe/Guernsey
    {  549, 7392,  177702,  9, }, // Pacific/Wake
    {  550, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Bolivia
    {  550, 7886,  177643, 13, }, // America/Fort_Nelson
    {  550, 5481,  177656,  5, }, // America/Nassau
    {  550, 2201,   64147,  8, }, // America/Santiago
    {  550,  855,  177661, 11, }, // America/St_Thomas
    {  550, 8661,  177672,  8, }, // Asia/Khandyga
    {  550, 6628,  177680, 14, }, // Atlantic/Canary
    {  550, 8772,   26905,  8, }, // Europe/Busingen
    {  550, 2338,   20973,  6, }, // Europe/Dublin
    {  550, 8788,  177694,  8, }, // Europe/Guernsey
    {  550, 7392,  177702,  9, }, // Pacific/Wake
    {  551, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Brazil
    {  551, 7886,  177643, 13, }, // America/Fort_Nelson
    {  551, 5481,  177656,  5, }, // America/Nassau
    {  551, 2201,   64147,  8, }, // America/Santiago
    {  551,  855,  177661, 11, }, // America/St_Thomas
    {  551, 8661,  177672,  8, }, // Asia/Khandyga
    {  551, 6628,  177680, 14, }, // Atlantic/Canary
    {  551, 8772,   26905,  8, }, // Europe/Busingen
    {  551, 2338,   20973,  6, }, // Europe/Dublin
    {  551, 8788,  177694,  8, }, // Europe/Guernsey
    {  551, 7392,  177702,  9, }, // Pacific/Wake
    {  554, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Chile
    {  554, 7886,  177643, 13, }, // America/Fort_Nelson
    {  554, 5481,  177656,  5, }, // America/Nassau
    {  554, 2201,   64147,  8, }, // America/Santiago
    {  554,  855,  177661, 11, }, // America/St_Thomas
    {  554, 8661,  177672,  8, }, // Asia/Khandyga
    {  554, 6628,  177680, 14, }, // Atlantic/Canary
    {  554, 8772,   26905,  8, }, // Europe/Busingen
    {  554, 2338,   20973,  6, }, // Europe/Dublin
    {  554, 8788,  177694,  8, }, // Europe/Guernsey
    {  554, 7392,  177702,  9, }, // Pacific/Wake
    {  555, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Colombia
    {  555, 7886,  177643, 13, }, // America/Fort_Nelson
    {  555, 5481,  177656,  5, }, // America/Nassau
    {  555, 2201,   64147,  8, }, // America/Santiago
    {  555,  855,  177661, 11, }, // America/St_Thomas
    {  555, 8661,  177672,  8, }, // Asia/Khandyga
    {  555, 6628,  177680, 14, }, // Atlantic/Canary
    {  555, 8772,   26905,  8, }, // Europe/Busingen
    {  555, 2338,   20973,  6, }, // Europe/Dublin
    {  555, 8788,  177694,  8, }, // Europe/Guernsey
    {  555, 7392,  177702,  9, }, // Pacific/Wake
    {  556, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Costa Rica
    {  556, 7886,  177643, 13, }, // America/Fort_Nelson
    {  556, 5481,  177656,  5, }, // America/Nassau
    {  556, 2201,   64147,  8, }, // America/Santiago
    {  556,  855,  177661, 11, }, // America/St_Thomas
    {  556, 8661,  177672,  8, }, // Asia/Khandyga
    {  556, 6628,  177680, 14, }, // Atlantic/Canary
    {  556, 8772,   26905,  8, }, // Europe/Busingen
    {  556, 2338,   20973,  6, }, // Europe/Dublin
    {  556, 8788,  177694,  8, }, // Europe/Guernsey
    {  556, 7392,  177702,  9, }, // Pacific/Wake
    {  557, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Cuba
    {  557, 7886,  177643, 13, }, // America/Fort_Nelson
    {  557, 5481,  177656,  5, }, // America/Nassau
    {  557, 2201,   64147,  8, }, // America/Santiago
    {  557,  855,  177661, 11, }, // America/St_Thomas
    {  557, 8661,  177672,  8, }, // Asia/Khandyga
    {  557, 6628,  177680, 14, }, // Atlantic/Canary
    {  557, 8772,   26905,  8, }, // Europe/Busingen
    {  557, 2338,   20973,  6, }, // Europe/Dublin
    {  557, 8788,  177694,  8, }, // Europe/Guernsey
    {  557, 7392,  177702,  9, }, // Pacific/Wake
    {  558, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Dominican Republic
    {  558, 7886,  177643, 13, }, // America/Fort_Nelson
    {  558, 5481,  177656,  5, }, // America/Nassau
    {  558, 2201,   64147,  8, }, // America/Santiago
    {  558,  855,  177661, 11, }, // America/St_Thomas
    {  558, 8661,  177672,  8, }, // Asia/Khandyga
    {  558, 6628,  177680, 14, }, // Atlantic/Canary
    {  558, 8772,   26905,  8, }, // Europe/Busingen
    {  558, 2338,   20973,  6, }, // Europe/Dublin
    {  558, 8788,  177694,  8, }, // Europe/Guernsey
    {  558, 7392,  177702,  9, }, // Pacific/Wake
    {  559, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Ecuador
    {  559, 7886,  177643, 13, }, // America/Fort_Nelson
    {  559, 5481,  177656,  5, }, // America/Nassau
    {  559, 2201,   64147,  8, }, // America/Santiago
    {  559,  855,  177661, 11, }, // America/St_Thomas
    {  559, 8661,  177672,  8, }, // Asia/Khandyga
    {  559, 6628,  177680, 14, }, // Atlantic/Canary
    {  559, 8772,   26905,  8, }, // Europe/Busingen
    {  559, 2338,   20973,  6, }, // Europe/Dublin
    {  559, 8788,  177694,  8, }, // Europe/Guernsey
    {  559, 7392,  177702,  9, }, // Pacific/Wake
    {  560, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/El Salvador
    {  560, 7886,  177643, 13, }, // America/Fort_Nelson
    {  560, 5481,  177656,  5, }, // America/Nassau
    {  560, 2201,   64147,  8, }, // America/Santiago
    {  560,  855,  177661, 11, }, // America/St_Thomas
    {  560, 8661,  177672,  8, }, // Asia/Khandyga
    {  560, 6628,  177680, 14, }, // Atlantic/Canary
    {  560, 8772,   26905,  8, }, // Europe/Busingen
    {  560, 2338,   20973,  6, }, // Europe/Dublin
    {  560, 8788,  177694,  8, }, // Europe/Guernsey
    {  560, 7392,  177702,  9, }, // Pacific/Wake
    {  562, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Guatemala
    {  562, 7886,  177643, 13, }, // America/Fort_Nelson
    {  562, 5481,  177656,  5, }, // America/Nassau
    {  562, 2201,   64147,  8, }, // America/Santiago
    {  562,  855,  177661, 11, }, // America/St_Thomas
    {  562, 8661,  177672,  8, }, // Asia/Khandyga
    {  562, 6628,  177680, 14, }, // Atlantic/Canary
    {  562, 8772,   26905,  8, }, // Europe/Busingen
    {  562, 2338,   20973,  6, }, // Europe/Dublin
    {  562, 8788,  177694,  8, }, // Europe/Guernsey
    {  562, 7392,  177702,  9, }, // Pacific/Wake
    {  563, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Honduras
    {  563, 7886,  177643, 13, }, // America/Fort_Nelson
    {  563, 5481,  177656,  5, }, // America/Nassau
    {  563, 2201,   64147,  8, }, // America/Santiago
    {  563,  855,  177661, 11, }, // America/St_Thomas
    {  563, 8661,  177672,  8, }, // Asia/Khandyga
    {  563, 6628,  177680, 14, }, // Atlantic/Canary
    {  563, 8772,   26905,  8, }, // Europe/Busingen
    {  563, 2338,   20973,  6, }, // Europe/Dublin
    {  563, 8788,  177694,  8, }, // Europe/Guernsey
    {  563, 7392,  177702,  9, }, // Pacific/Wake
    {  564, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Latin America
    {  564, 7886,  177643, 13, }, // America/Fort_Nelson
    {  564, 5481,  177656,  5, }, // America/Nassau
    {  564, 2201,   64147,  8, }, // America/Santiago
    {  564,  855,  177661, 11, }, // America/St_Thomas
    {  564, 8661,  177672,  8, }, // Asia/Khandyga
    {  564, 6628,  177680, 14, }, // Atlantic/Canary
    {  564, 8772,   26905,  8, }, // Europe/Busingen
    {  564, 2338,   20973,  6, }, // Europe/Dublin
    {  564, 8788,  177694,  8, }, // Europe/Guernsey
    {  564, 7392,  177702,  9, }, // Pacific/Wake
    {  565, 4922,  177711,  9, }, // Africa/Bujumbura Spanish/Latin/Mexico
    {  565, 5130,   91261, 13, }, // Africa/Dar_es_Salaam
    {  565, 7886,  125270, 11, }, // America/Fort_Nelson
    {  565, 5481,  177656,  5, }, // America/Nassau
    {  565,  695,  177720, 10, }, // America/Rio_Branco
    {  565, 2201,   64147,  8, }, // America/Santiago
    {  565, 5395,  177730,  6, }, // Asia/Almaty
    {  565, 5368,  177736,  6, }, // Asia/Aqtobe
    {  565, 8624,    7494,  6, }, // Asia/Atyrau
    {  565, 8661,  177672,  8, }, // Asia/Khandyga
    {  565, 6628,  177680, 14, }, // Atlantic/Canary
    {  565, 8772,   26905,  8, }, // Europe/Busingen
    {  565, 2338,   20973,  6, }, // Europe/Dublin
    {  565, 8788,  177694,  8, }, // Europe/Guernsey
    {  565, 2723,    4493,  8, }, // Pacific/Honolulu
    {  565, 7392,   92264,  4, }, // Pacific/Wake
    {  566, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Nicaragua
    {  566, 7886,  177643, 13, }, // America/Fort_Nelson
    {  566, 5481,  177656,  5, }, // America/Nassau
    {  566, 2201,   64147,  8, }, // America/Santiago
    {  566,  855,  177661, 11, }, // America/St_Thomas
    {  566, 8661,  177672,  8, }, // Asia/Khandyga
    {  566, 6628,  177680, 14, }, // Atlantic/Canary
    {  566, 8772,   26905,  8, }, // Europe/Busingen
    {  566, 2338,   20973,  6, }, // Europe/Dublin
    {  566, 8788,  177694,  8, }, // Europe/Guernsey
    {  566, 7392,  177702,  9, }, // Pacific/Wake
    {  567, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Panama
    {  567, 7886,  177643, 13, }, // America/Fort_Nelson
    {  567, 5481,  177656,  5, }, // America/Nassau
    {  567, 2201,   64147,  8, }, // America/Santiago
    {  567,  855,  177661, 11, }, // America/St_Thomas
    {  567, 8661,  177672,  8, }, // Asia/Khandyga
    {  567, 6628,  177680, 14, }, // Atlantic/Canary
    {  567, 8772,   26905,  8, }, // Europe/Busingen
    {  567, 2338,   20973,  6, }, // Europe/Dublin
    {  567, 8788,  177694,  8, }, // Europe/Guernsey
    {  567, 7392,  177702,  9, }, // Pacific/Wake
    {  568, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Paraguay
    {  568, 7886,  177643, 13, }, // America/Fort_Nelson
    {  568, 5481,  177656,  5, }, // America/Nassau
    {  568, 2201,   64147,  8, }, // America/Santiago
    {  568,  855,  177661, 11, }, // America/St_Thomas
    {  568, 8661,  177672,  8, }, // Asia/Khandyga
    {  568, 6628,  177680, 14, }, // Atlantic/Canary
    {  568, 8772,   26905,  8, }, // Europe/Busingen
    {  568, 2338,   20973,  6, }, // Europe/Dublin
    {  568, 8788,  177694,  8, }, // Europe/Guernsey
    {  568, 7392,  177702,  9, }, // Pacific/Wake
    {  569, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Peru
    {  569, 7886,  177643, 13, }, // America/Fort_Nelson
    {  569, 5481,  177656,  5, }, // America/Nassau
    {  569, 2201,   64147,  8, }, // America/Santiago
    {  569,  855,  177661, 11, }, // America/St_Thomas
    {  569, 8661,  177672,  8, }, // Asia/Khandyga
    {  569, 6628,  177680, 14, }, // Atlantic/Canary
    {  569, 8772,   26905,  8, }, // Europe/Busingen
    {  569, 2338,   20973,  6, }, // Europe/Dublin
    {  569, 8788,  177694,  8, }, // Europe/Guernsey
    {  569, 7392,  177702,  9, }, // Pacific/Wake
    {  571, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Puerto Rico
    {  571, 7886,  177643, 13, }, // America/Fort_Nelson
    {  571, 5481,  177656,  5, }, // America/Nassau
    {  571, 2201,   64147,  8, }, // America/Santiago
    {  571,  855,  177661, 11, }, // America/St_Thomas
    {  571, 8661,  177672,  8, }, // Asia/Khandyga
    {  571, 6628,  177680, 14, }, // Atlantic/Canary
    {  571, 8772,   26905,  8, }, // Europe/Busingen
    {  571, 2338,   20973,  6, }, // Europe/Dublin
    {  571, 8788,  177694,  8, }, // Europe/Guernsey
    {  571, 7392,  177702,  9, }, // Pacific/Wake
    {  572, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/United States
    {  572, 5043,  177742,  6, }, // Africa/Djibouti
    {  572, 7886,  125270, 11, }, // America/Fort_Nelson
    {  572, 5481,  177748,  6, }, // America/Nassau
    {  572, 2201,   64147,  8, }, // America/Santiago
    {  572, 8661,  177672,  8, }, // Asia/Khandyga
    {  572, 4214,  161875,  8, }, // Asia/Pyongyang
    {  572, 6628,  177680, 14, }, // Atlantic/Canary
    {  572, 3620,  177754,  9, }, // Europe/Astrakhan
    {  572, 8772,   26905,  8, }, // Europe/Busingen
    {  572, 2338,   20973,  6, }, // Europe/Dublin
    {  572, 8788,  177694,  8, }, // Europe/Guernsey
    {  572, 8929,  124066,  5, }, // Europe/Kirov
    {  572, 8882,  177763,  9, }, // Europe/Ulyanovsk
    {  572, 2723,    4493,  8, }, // Pacific/Honolulu
    {  572, 7392,   92264,  4, }, // Pacific/Wake
    {  573, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Uruguay
    {  573, 7886,  177643, 13, }, // America/Fort_Nelson
    {  573, 5481,  177656,  5, }, // America/Nassau
    {  573, 2201,   64147,  8, }, // America/Santiago
    {  573,  855,  177661, 11, }, // America/St_Thomas
    {  573, 8661,  177672,  8, }, // Asia/Khandyga
    {  573, 6628,  177680, 14, }, // Atlantic/Canary
    {  573, 8772,   26905,  8, }, // Europe/Busingen
    {  573, 2338,   20973,  6, }, // Europe/Dublin
    {  573, 8788,  177694,  8, }, // Europe/Guernsey
    {  573, 7392,  177702,  9, }, // Pacific/Wake
    {  574, 6773,  177636,  7, }, // Africa/Conakry Spanish/Latin/Venezuela
    {  574, 7886,  177643, 13, }, // America/Fort_Nelson
    {  574, 5481,  177656,  5, }, // America/Nassau
    {  574, 2201,   64147,  8, }, // America/Santiago
    {  574,  855,  177661, 11, }, // America/St_Thomas
    {  574, 8661,  177672,  8, }, // Asia/Khandyga
    {  574, 6628,  177680, 14, }, // Atlantic/Canary
    {  574, 8772,   26905,  8, }, // Europe/Busingen
    {  574, 2338,   20973,  6, }, // Europe/Dublin
    {  574, 8788,  177694,  8, }, // Europe/Guernsey
    {  574, 7392,  177702,  9, }, // Pacific/Wake
    {  575,   14,       0,  6, }, // Africa/Asmara Standard Moroccan Tamazight/Tifinagh/Morocco
    {  575, 4435,       6,  8, }, // Africa/Sao_Tome
    {  575, 4252,      14,  8, }, // America/Asuncion
    {  575,  237,      22,  8, }, // America/Atikokan
    {  575, 7623,      30, 17, }, // America/Bahia_Banderas
    {  575, 3868,      47,  6, }, // America/Cancun
    {  575, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  575, 5723,      66,  7, }, // America/Curacao
    {  575,  481,      73, 13, }, // America/Indiana/Knox
    {  575, 7961,      86, 16, }, // America/Indiana/Marengo
    {  575, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  575, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  575, 8038,     139, 14, }, // America/Indiana/Vevay
    {  575, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  575, 8086,     171, 16, }, // America/Indiana/Winamac
    {  575, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  575, 5932,     207, 22, }, // America/Lower_Princes
    {  575, 8220,     229,  6, }, // America/Merida
    {  575, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  575, 8329,     255, 20, }, // America/North_Dakota/Center
    {  575, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  575,  393,     298,  4, }, // America/Nuuk
    {  575, 6899,     302, 16, }, // America/Scoresbysund
    {  575, 8508,     318, 14, }, // America/St_Barthelemy
    {  575, 2061,     332, 10, }, // America/St_Johns
    {  575, 5863,     342,  9, }, // America/St_Kitts
    {  575, 5880,     351,  9, }, // America/St_Lucia
    {  575,  855,     360, 10, }, // America/St_Thomas
    {  575, 5913,     370, 11, }, // America/St_Vincent
    {  575, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  575, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  575, 1185,     413,  9, }, // Asia/Kathmandu
    {  575,  992,     422,  7, }, // Asia/Kolkata
    {  575, 1211,     429,  5, }, // Asia/Macau
    {  575, 1235,     442,  6, }, // Asia/Yangon
    {  575, 1395,     448,  5, }, // Atlantic/Faroe
    {  575, 6821,     453, 10, }, // Atlantic/St_Helena
    {  575, 2527,     463,  4, }, // Europe/Kyiv
    {  575, 7211,     467,  7, }, // Indian/Reunion
    {  575, 3157,     474,  5, }, // Pacific/Chuuk
    {  575, 3096,     479,  7, }, // Pacific/Pohnpei
    {  576,   14,       0,  6, }, // Africa/Asmara Sundanese/Latin/Indonesia
    {  576, 4435,       6,  8, }, // Africa/Sao_Tome
    {  576, 4252,      14,  8, }, // America/Asuncion
    {  576,  237,      22,  8, }, // America/Atikokan
    {  576, 7623,      30, 17, }, // America/Bahia_Banderas
    {  576, 3868,      47,  6, }, // America/Cancun
    {  576, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  576, 5723,      66,  7, }, // America/Curacao
    {  576,  481,      73, 13, }, // America/Indiana/Knox
    {  576, 7961,      86, 16, }, // America/Indiana/Marengo
    {  576, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  576, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  576, 8038,     139, 14, }, // America/Indiana/Vevay
    {  576, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  576, 8086,     171, 16, }, // America/Indiana/Winamac
    {  576, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  576, 5932,     207, 22, }, // America/Lower_Princes
    {  576, 8220,     229,  6, }, // America/Merida
    {  576, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  576, 8329,     255, 20, }, // America/North_Dakota/Center
    {  576, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  576,  393,     298,  4, }, // America/Nuuk
    {  576, 6899,     302, 16, }, // America/Scoresbysund
    {  576, 8508,     318, 14, }, // America/St_Barthelemy
    {  576, 2061,     332, 10, }, // America/St_Johns
    {  576, 5863,     342,  9, }, // America/St_Kitts
    {  576, 5880,     351,  9, }, // America/St_Lucia
    {  576,  855,     360, 10, }, // America/St_Thomas
    {  576, 5913,     370, 11, }, // America/St_Vincent
    {  576, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  576, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  576, 1185,     413,  9, }, // Asia/Kathmandu
    {  576,  992,     422,  7, }, // Asia/Kolkata
    {  576, 1211,   33405,  5, }, // Asia/Macau
    {  576, 1235,     442,  6, }, // Asia/Yangon
    {  576, 1395,     448,  5, }, // Atlantic/Faroe
    {  576, 6821,     453, 10, }, // Atlantic/St_Helena
    {  576, 2527,     463,  4, }, // Europe/Kyiv
    {  576, 7211,     467,  7, }, // Indian/Reunion
    {  576, 3157,     474,  5, }, // Pacific/Chuuk
    {  576, 3096,     479,  7, }, // Pacific/Pohnpei
    {  577,   14,       0,  6, }, // Africa/Asmara Swahili/Latin/Tanzania
    {  577, 4435,       6,  8, }, // Africa/Sao_Tome
    {  577, 4252,      14,  8, }, // America/Asuncion
    {  577,  237,      22,  8, }, // America/Atikokan
    {  577, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  577, 3868,   33182,  6, }, // America/Cancun
    {  577, 7774,   91149, 13, }, // America/Ciudad_Juarez
    {  577, 5723,      66,  7, }, // America/Curacao
    {  577,  481,      73, 13, }, // America/Indiana/Knox
    {  577, 7961,      86, 16, }, // America/Indiana/Marengo
    {  577, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  577, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  577, 8038,     139, 14, }, // America/Indiana/Vevay
    {  577, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  577, 8086,     171, 16, }, // America/Indiana/Winamac
    {  577, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  577, 5932,     207, 22, }, // America/Lower_Princes
    {  577, 8220,    7008,  6, }, // America/Merida
    {  577, 2949,  177772, 14, }, // America/Mexico_City
    {  577, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  577, 8329,     255, 20, }, // America/North_Dakota/Center
    {  577, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  577,  393,     298,  4, }, // America/Nuuk
    {  577, 6899,     302, 16, }, // America/Scoresbysund
    {  577, 8508,     318, 14, }, // America/St_Barthelemy
    {  577, 2061,     332, 10, }, // America/St_Johns
    {  577, 5863,     342,  9, }, // America/St_Kitts
    {  577, 5880,     351,  9, }, // America/St_Lucia
    {  577,  855,     360, 10, }, // America/St_Thomas
    {  577, 5913,     370, 11, }, // America/St_Vincent
    {  577, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  577, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  577, 1185,     413,  9, }, // Asia/Kathmandu
    {  577,  992,     422,  7, }, // Asia/Kolkata
    {  577, 1211,    4203,  5, }, // Asia/Macau
    {  577, 8708,     434,  8, }, // Asia/Qostanay
    {  577, 1235,   58486,  7, }, // Asia/Yangon
    {  577, 1395,     448,  5, }, // Atlantic/Faroe
    {  577, 7272,  177786, 14, }, // Atlantic/South_Georgia
    {  577, 6821,     453, 10, }, // Atlantic/St_Helena
    {  577, 2527,    8030,  4, }, // Europe/Kyiv
    {  577, 7211,     467,  7, }, // Indian/Reunion
    {  577, 3157,     474,  5, }, // Pacific/Chuuk
    {  577, 2723,    4493,  8, }, // Pacific/Honolulu
    {  577, 3049,    4501,  9, }, // Pacific/Kanton
    {  577, 3096,     479,  7, }, // Pacific/Pohnpei
    {  579, 5654,  177800,  7, }, // America/Barbados Swahili/Latin/Kenya
    {  579, 5422,   33188,  9, }, // America/Costa_Rica
    {  579, 5723,  177807,  7, }, // America/Curacao
    {  579, 5739,   21548,  8, }, // America/Dominica
    {  579, 5441,  177814,  9, }, // America/El_Salvador
    {  579, 5786,   20408,  9, }, // America/Guadeloupe
    {  579, 2799,    3877,  7, }, // America/Jamaica
    {  579, 8301,  177823, 24, }, // America/North_Dakota/Beulah
    {  579, 8329,  177847, 24, }, // America/North_Dakota/Center
    {  579, 8357,  177871, 27, }, // America/North_Dakota/New_Salem
    {  579, 5954,  177898, 20, }, // America/Port_of_Spain
    {  579, 5843,  177918,  9, }, // America/Puerto_Rico
    {  579, 4511,    5519,  7, }, // Asia/Colombo
    {  579, 1259,  177927, 19, }, // Asia/Ho_Chi_Minh
    {  579, 1211,    7613,  5, }, // Asia/Macau
    {  579, 3300,  177946,  8, }, // Asia/Singapore
    {  579, 1235,     442,  6, }, // Asia/Yangon
    {  579, 6628,    4280,  6, }, // Atlantic/Canary
    {  579, 6046,  177954,  8, }, // Indian/Christmas
    {  579, 7050,   33687,  7, }, // Indian/Maldives
    {  581,   14,       0,  6, }, // Africa/Asmara Swati/Latin/South Africa
    {  581, 4435,       6,  8, }, // Africa/Sao_Tome
    {  581, 4252,      14,  8, }, // America/Asuncion
    {  581,  237,      22,  8, }, // America/Atikokan
    {  581, 7623,      30, 17, }, // America/Bahia_Banderas
    {  581, 3868,      47,  6, }, // America/Cancun
    {  581, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  581, 5723,      66,  7, }, // America/Curacao
    {  581,  481,      73, 13, }, // America/Indiana/Knox
    {  581, 7961,      86, 16, }, // America/Indiana/Marengo
    {  581, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  581, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  581, 8038,     139, 14, }, // America/Indiana/Vevay
    {  581, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  581, 8086,     171, 16, }, // America/Indiana/Winamac
    {  581, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  581, 5932,     207, 22, }, // America/Lower_Princes
    {  581, 8220,     229,  6, }, // America/Merida
    {  581, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  581, 8329,     255, 20, }, // America/North_Dakota/Center
    {  581, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  581,  393,     298,  4, }, // America/Nuuk
    {  581, 6899,     302, 16, }, // America/Scoresbysund
    {  581, 8508,     318, 14, }, // America/St_Barthelemy
    {  581, 2061,     332, 10, }, // America/St_Johns
    {  581, 5863,     342,  9, }, // America/St_Kitts
    {  581, 5880,     351,  9, }, // America/St_Lucia
    {  581,  855,     360, 10, }, // America/St_Thomas
    {  581, 5913,     370, 11, }, // America/St_Vincent
    {  581, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  581, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  581, 1185,     413,  9, }, // Asia/Kathmandu
    {  581,  992,     422,  7, }, // Asia/Kolkata
    {  581, 1211,     429,  5, }, // Asia/Macau
    {  581, 1235,     442,  6, }, // Asia/Yangon
    {  581, 1395,     448,  5, }, // Atlantic/Faroe
    {  581, 6821,     453, 10, }, // Atlantic/St_Helena
    {  581, 2527,     463,  4, }, // Europe/Kyiv
    {  581, 7211,     467,  7, }, // Indian/Reunion
    {  581, 3157,     474,  5, }, // Pacific/Chuuk
    {  581, 3096,     479,  7, }, // Pacific/Pohnpei
    {  583, 5059,    3784, 11, }, // Africa/Addis_Ababa Swedish/Latin/Sweden
    {  583, 7420,   43956,  5, }, // Africa/Algiers
    {  583,   14,       0,  6, }, // Africa/Asmara
    {  583, 2320,    4572,  5, }, // Africa/Cairo
    {  583, 5130,   91261, 13, }, // Africa/Dar_es_Salaam
    {  583, 5166,  173826,  8, }, // Africa/El_Aaiun
    {  583, 6869,   20300,  4, }, // Africa/Lome
    {  583, 5271,   43996,  9, }, // Africa/Ndjamena
    {  583, 4435,       6,  8, }, // Africa/Sao_Tome
    {  583, 4554,   26427,  9, }, // America/Araguaina
    {  583,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  583,  441,  162227, 21, }, // America/Argentina/Jujuy
    {  583, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {  583, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  583, 4252,      14,  8, }, // America/Asuncion
    {  583,  237,      22,  8, }, // America/Atikokan
    {  583, 7623,      30, 17, }, // America/Bahia_Banderas
    {  583, 7646,   35764,  5, }, // America/Belem
    {  583, 4361,   20389,  6, }, // America/Bogota
    {  583, 3868,      47,  6, }, // America/Cancun
    {  583, 5496,  177962, 11, }, // America/Cayman
    {  583, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  583, 3791,   26442,  6, }, // America/Cuiaba
    {  583, 5723,      66,  7, }, // America/Curacao
    {  583, 7869,   26448,  8, }, // America/Eirunepe
    {  583, 2281,   63392,  7, }, // America/Havana
    {  583,  481,      73, 13, }, // America/Indiana/Knox
    {  583, 7961,      86, 16, }, // America/Indiana/Marengo
    {  583, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  583, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  583, 8038,     139, 14, }, // America/Indiana/Vevay
    {  583, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  583, 8086,     171, 16, }, // America/Indiana/Winamac
    {  583, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  583, 5932,     207, 22, }, // America/Lower_Princes
    {  583, 8153,   26456,  6, }, // America/Maceio
    {  583, 2917,   26471,  8, }, // America/Mazatlan
    {  583, 8220,     229,  6, }, // America/Merida
    {  583, 1850,   61820, 19, }, // America/Noronha
    {  583, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  583, 8329,     255, 20, }, // America/North_Dakota/Center
    {  583, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  583,  393,     298,  4, }, // America/Nuuk
    {  583, 8477,   26572,  8, }, // America/Santarem
    {  583, 1878,   26580,  9, }, // America/Sao_Paulo
    {  583, 6899,     302, 16, }, // America/Scoresbysund
    {  583, 8508,   26589, 16, }, // America/St_Barthelemy
    {  583, 2061,   26605, 12, }, // America/St_Johns
    {  583, 5863,   26617, 11, }, // America/St_Kitts
    {  583, 5880,   35832, 11, }, // America/St_Lucia
    {  583,  855,   44161, 12, }, // America/St_Thomas
    {  583, 5913,   26651, 13, }, // America/St_Vincent
    {  583, 5756,   26664,  7, }, // America/Thule
    {  583, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  583, 5523,    5491,  5, }, // Asia/Aqtau
    {  583, 5368,   57073,  6, }, // Asia/Aqtobe
    {  583,  964,    4035,  9, }, // Asia/Ashgabat
    {  583, 3607,    4044,  6, }, // Asia/Baghdad
    {  583, 3778,    4069,  7, }, // Asia/Bishkek
    {  583, 4601,  177973,  5, }, // Asia/Chita
    {  583, 4540,    4083,  8, }, // Asia/Damascus
    {  583, 6159,   58449,  8, }, // Asia/Dushanbe
    {  583, 1259,  177978, 18, }, // Asia/Ho_Chi_Minh
    {  583, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  583, 4771,  174021,  5, }, // Asia/Hovd
    {  583, 4316,   58457,  9, }, // Asia/Kamchatka
    {  583, 1185,    5593,  8, }, // Asia/Kathmandu
    {  583, 8661,   57888,  8, }, // Asia/Khandyga
    {  583,  992,     422,  7, }, // Asia/Kolkata
    {  583, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  583, 1211,     429,  5, }, // Asia/Macau
    {  583, 6920,    4208,  6, }, // Asia/Muscat
    {  583, 8708,    7652,  8, }, // Asia/Qostanay
    {  583, 4408,   57952,  8, }, // Asia/Sakhalin
    {  583, 4781,   58478,  8, }, // Asia/Tashkent
    {  583, 2772,    7724,  7, }, // Asia/Tehran
    {  583, 1159,   26801,  6, }, // Asia/Urumqi
    {  583, 4828,    7755,  7, }, // Asia/Yakutsk
    {  583, 1235,     442,  6, }, // Asia/Yangon
    {  583, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  583, 3747,    7780,  7, }, // Asia/Yerevan
    {  583, 3663,  177996,  8, }, // Atlantic/Azores
    {  583, 6628,  178004, 12, }, // Atlantic/Canary
    {  583, 3727,   57248,  9, }, // Atlantic/Cape_Verde
    {  583, 1395,  178016,  8, }, // Atlantic/Faroe
    {  583, 7272,  178024, 11, }, // Atlantic/South_Georgia
    {  583, 6821,  178035, 12, }, // Atlantic/St_Helena
    {  583, 2300,    5793,  4, }, // Europe/Athens
    {  583, 6336,   26883,  7, }, // Europe/Belgrade
    {  583, 2167,   64287,  7, }, // Europe/Brussels
    {  583, 3928,   26897,  8, }, // Europe/Bucharest
    {  583, 8772,   62795, 21, }, // Europe/Busingen
    {  583, 2583,   63672,  8, }, // Europe/Chisinau
    {  583, 6289,  178047,  9, }, // Europe/Copenhagen
    {  583, 6612,  147239, 11, }, // Europe/Helsinki
    {  583, 2527,    8030,  4, }, // Europe/Kyiv
    {  583, 3213,    4381,  8, }, // Europe/Lisbon
    {  583, 6365,    4395,  9, }, // Europe/Luxembourg
    {  583, 3537,   22748,  6, }, // Europe/Moscow
    {  583, 6275,    8083,  4, }, // Europe/Prague
    {  583, 6324,    5946,  3, }, // Europe/Rome
    {  583, 6202,   21095,  6, }, // Europe/Tirane
    {  583, 8882,    8143,  9, }, // Europe/Ulyanovsk
    {  583, 6567,  178056,  9, }, // Europe/Vatican
    {  583, 6231,   57383,  4, }, // Europe/Vienna
    {  583, 3190,   58543,  8, }, // Europe/Warsaw
    {  583, 6540,    4459,  6, }, // Europe/Zurich
    {  583, 6046,  178065,  5, }, // Indian/Christmas
    {  583, 6063,  178070, 10, }, // Indian/Cocos
    {  583, 5029,  178080,  9, }, // Indian/Comoro
    {  583, 6661,  178089, 14, }, // Indian/Kerguelen
    {  583, 7260,   26983,  4, }, // Indian/Mahe
    {  583, 7050,  178103, 10, }, // Indian/Maldives
    {  583, 7211,     467,  7, }, // Indian/Reunion
    {  583, 3157,     474,  5, }, // Pacific/Chuuk
    {  583, 2237,  178113,  6, }, // Pacific/Easter
    {  583, 6678,   21159,  9, }, // Pacific/Galapagos
    {  583, 6696,  178119, 12, }, // Pacific/Gambier
    {  583, 2723,    4493,  8, }, // Pacific/Honolulu
    {  583, 3049,    4501,  9, }, // Pacific/Kanton
    {  583, 4051,  178131, 14, }, // Pacific/Marquesas
    {  583, 8914,  178145, 11, }, // Pacific/Midway
    {  583, 7098,    4510,  6, }, // Pacific/Noumea
    {  583, 7194,  178156, 13, }, // Pacific/Pitcairn
    {  583, 3096,     479,  7, }, // Pacific/Pohnpei
    {  583, 7405,  178169,  8, }, // Pacific/Wallis
    {  586, 6760,   21174,  5, }, // Africa/Accra Swiss German/Latin/Switzerland
    {  586, 5059,    3784, 11, }, // Africa/Addis_Ababa
    {  586, 7420,   58423,  6, }, // Africa/Algiers
    {  586,   14,       0,  6, }, // Africa/Asmara
    {  586, 2320,    4572,  5, }, // Africa/Cairo
    {  586, 5130,   77951, 10, }, // Africa/Dar_es_Salaam
    {  586, 5043,   77961,  9, }, // Africa/Djibouti
    {  586, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  586, 4524,   26372,  7, }, // Africa/Khartoum
    {  586, 5113,   77970, 10, }, // Africa/Mogadishu
    {  586, 6727,    4640,  8, }, // Africa/Ouagadougou
    {  586, 4435,       6,  8, }, // Africa/Sao_Tome
    {  586, 4252,      14,  8, }, // America/Asuncion
    {  586,  237,      22,  8, }, // America/Atikokan
    {  586, 7623,      30, 17, }, // America/Bahia_Banderas
    {  586, 3868,      47,  6, }, // America/Cancun
    {  586, 5496,  178177, 11, }, // America/Cayman
    {  586, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  586, 5723,      66,  7, }, // America/Curacao
    {  586, 5441,    6688,  8, }, // America/El_Salvador
    {  586, 2281,   63392,  7, }, // America/Havana
    {  586,  481,      73,  4, }, // America/Indiana/Knox
    {  586, 7961,      86, 16, }, // America/Indiana/Marengo
    {  586, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  586, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  586, 8038,     139, 14, }, // America/Indiana/Vevay
    {  586, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  586, 8086,     171, 16, }, // America/Indiana/Winamac
    {  586, 2799,    3877,  7, }, // America/Jamaica
    {  586, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  586, 5932,     207, 22, }, // America/Lower_Princes
    {  586, 8220,     229,  6, }, // America/Merida
    {  586, 2949,  178188, 14, }, // America/Mexico_City
    {  586, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  586, 8329,     255, 20, }, // America/North_Dakota/Center
    {  586, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  586,  393,     298,  4, }, // America/Nuuk
    {  586, 5954,   26559, 13, }, // America/Port_of_Spain
    {  586, 6899,     302, 16, }, // America/Scoresbysund
    {  586, 8508,     318, 14, }, // America/St_Barthelemy
    {  586, 2061,     332, 10, }, // America/St_Johns
    {  586, 5863,     342,  9, }, // America/St_Kitts
    {  586, 5880,     351,  9, }, // America/St_Lucia
    {  586,  855,     360, 10, }, // America/St_Thomas
    {  586, 5913,     370, 11, }, // America/St_Vincent
    {  586, 6133,  127763, 16, }, // Antarctica/DumontDUrville
    {  586, 7374,  178202,  8, }, // Antarctica/Vostok
    {  586, 5368,    7488,  6, }, // Asia/Aqtobe
    {  586, 3607,    4044,  6, }, // Asia/Baghdad
    {  586, 3778,   57088,  8, }, // Asia/Bishkek
    {  586, 4540,    4083,  8, }, // Asia/Damascus
    {  586, 6159,   57096,  9, }, // Asia/Dushanbe
    {  586, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  586, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  586, 4316,   78050, 11, }, // Asia/Kamchatka
    {  586, 1185,     413,  9, }, // Asia/Kathmandu
    {  586,  992,     422,  7, }, // Asia/Kolkata
    {  586, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  586, 1211,     429,  5, }, // Asia/Macau
    {  586, 6920,  178210,  8, }, // Asia/Muscat
    {  586, 2554,    5620,  7, }, // Asia/Nicosia
    {  586, 4135,   78082, 11, }, // Asia/Novosibirsk
    {  586, 4214,   78093,  9, }, // Asia/Pyongyang
    {  586, 5559,    4214,  5, }, // Asia/Qatar
    {  586, 3584,    7669,  4, }, // Asia/Riyadh
    {  586, 4408,   57952,  8, }, // Asia/Sakhalin
    {  586, 3300,   20782,  8, }, // Asia/Singapore
    {  586, 3263,   78102,  6, }, // Asia/Taipei
    {  586, 4781,   57196,  9, }, // Asia/Tashkent
    {  586, 3915,   78108,  6, }, // Asia/Tbilisi
    {  586, 2772,    7724,  7, }, // Asia/Tehran
    {  586, 2821,    4238,  5, }, // Asia/Tokyo
    {  586, 1021,  178218, 11, }, // Asia/Ulaanbaatar
    {  586, 4710,    4243, 11, }, // Asia/Vladivostok
    {  586, 4828,    7755,  7, }, // Asia/Yakutsk
    {  586, 1235,     442,  6, }, // Asia/Yangon
    {  586, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  586, 3747,  127970,  6, }, // Asia/Yerevan
    {  586, 3663,    7787,  5, }, // Atlantic/Azores
    {  586, 5671,   74354,  8, }, // Atlantic/Bermuda
    {  586, 6628,   78114,  6, }, // Atlantic/Canary
    {  586, 3727,   57248,  9, }, // Atlantic/Cape_Verde
    {  586, 1395,   78121,  6, }, // Atlantic/Faroe
    {  586, 7272,  178229, 12, }, // Atlantic/South_Georgia
    {  586, 6821,     453, 10, }, // Atlantic/St_Helena
    {  586, 2300,    4320,  5, }, // Europe/Athens
    {  586, 2167,   22634,  7, }, // Europe/Brussels
    {  586, 3928,  178241, 10, }, // Europe/Bucharest
    {  586, 2583,  128001,  9, }, // Europe/Chisinau
    {  586, 6289,  178251,  8, }, // Europe/Copenhagen
    {  586, 2527,   78163,  4, }, // Europe/Kyiv
    {  586, 3213,    4381,  8, }, // Europe/Lisbon
    {  586, 6365,    4395,  9, }, // Europe/Luxembourg
    {  586, 3537,   57353,  6, }, // Europe/Moscow
    {  586, 6324,    5946,  3, }, // Europe/Rome
    {  586, 6202,   21095,  6, }, // Europe/Tirane
    {  586, 6231,   57383,  4, }, // Europe/Vienna
    {  586, 8899,  127875,  5, }, // Europe/Vilnius
    {  586, 3190,   57396,  8, }, // Europe/Warsaw
    {  586, 6540,    4459,  4, }, // Europe/Zurich
    {  586, 6046,  178259, 15, }, // Indian/Christmas
    {  586, 5029,    8187,  6, }, // Indian/Comoro
    {  586, 7050,    4476,  8, }, // Indian/Maldives
    {  586, 7211,     467,  7, }, // Indian/Reunion
    {  586, 3157,     474,  5, }, // Pacific/Chuuk
    {  586, 2237,  178274, 12, }, // Pacific/Easter
    {  586, 3902,   78224,  7, }, // Pacific/Fiji
    {  586, 3096,     479,  7, }, // Pacific/Pohnpei
    {  589, 6788,  178286,  5, }, // Africa/Abidjan Syriac/Syriac/Iraq
    {  589, 6760,  178291,  4, }, // Africa/Accra
    {  589, 5059,  178295,  9, }, // Africa/Addis_Ababa
    {  589, 7420,  178304,  5, }, // Africa/Algiers
    {  589,   14,  178309,  6, }, // Africa/Asmara
    {  589,   44,  178315,  6, }, // Africa/Bamako
    {  589, 5257,  178321,  7, }, // Africa/Bangui
    {  589, 6746,  178328,  6, }, // Africa/Banjul
    {  589, 7435,  178334,  5, }, // Africa/Bissau
    {  589, 4957,  178339,  6, }, // Africa/Blantyre
    {  589, 5287,  178345,  8, }, // Africa/Brazzaville
    {  589, 4922,  178353,  9, }, // Africa/Bujumbura
    {  589, 2320,  178362,  6, }, // Africa/Cairo
    {  589, 4117,  178368,  9, }, // Africa/Casablanca
    {  589, 7449,  178377,  4, }, // Africa/Ceuta
    {  589, 6773,  178381,  7, }, // Africa/Conakry
    {  589, 6840,  178388,  5, }, // Africa/Dakar
    {  589, 5130,  178393,  8, }, // Africa/Dar_es_Salaam
    {  589, 5043,  178401,  6, }, // Africa/Djibouti
    {  589, 5243,  178407,  5, }, // Africa/Douala
    {  589, 5166,  178412,  7, }, // Africa/El_Aaiun
    {  589, 6853,  178419,  7, }, // Africa/Freetown
    {  589, 4906,  178426,  6, }, // Africa/Gaborone
    {  589, 5015,  178432,  5, }, // Africa/Harare
    {  589, 4479,  178437, 10, }, // Africa/Johannesburg
    {  589, 4499,  178447,  4, }, // Africa/Juba
    {  589, 5151,  178451,  7, }, // Africa/Kampala
    {  589, 4524,  178458,  5, }, // Africa/Khartoum
    {  589, 4973,  178463,  6, }, // Africa/Kigali
    {  589, 5306,  178469,  7, }, // Africa/Kinshasa
    {  589, 4744,  178476,  5, }, // Africa/Lagos
    {  589, 5336,  178481,  8, }, // Africa/Libreville
    {  589, 6869,  178489,  4, }, // Africa/Lome
    {  589, 5211,  178493,  6, }, // Africa/Luanda
    {  589, 4939,  178499,  8, }, // Africa/Lubumbashi
    {  589, 5001,  178507,  6, }, // Africa/Lusaka
    {  589, 5322,  178513,  6, }, // Africa/Malabo
    {  589, 4987,  178519,  6, }, // Africa/Maputo
    {  589, 5197,  178525,  6, }, // Africa/Maseru
    {  589, 5182,  178531,  7, }, // Africa/Mbabane
    {  589, 5113,  178538,  7, }, // Africa/Mogadishu
    {  589, 7034,  178545,  8, }, // Africa/Monrovia
    {  589, 3853,  178553,  6, }, // Africa/Nairobi
    {  589, 5271,  178559,  6, }, // Africa/Ndjamena
    {  589, 5354,  178565,  5, }, // Africa/Niamey
    {  589, 6803,  178570,  7, }, // Africa/Nouakchott
    {  589, 6727,  178577, 11, }, // Africa/Ouagadougou
    {  589, 5225,  178588, 10, }, // Africa/Porto-Novo
    {  589, 4435,  178598,  8, }, // Africa/Sao_Tome
    {  589, 2866,  178606,  8, }, // Africa/Tripoli
    {  589, 6554,  178614,  4, }, // Africa/Tunis
    {  589, 4152,  178618,  7, }, // Africa/Windhoek
    {  589,  132,  178625,  4, }, // America/Adak
    {  589, 3346,  178629,  6, }, // America/Anchorage
    {  589, 5607,  178635,  6, }, // America/Anguilla
    {  589, 5624,  178641,  7, }, // America/Antigua
    {  589, 4554,  178648,  9, }, // America/Araguaina
    {  589,  166,  178657, 12, }, // America/Argentina/Buenos_Aires
    {  589,   91,  178669,  8, }, // America/Argentina/Catamarca
    {  589,  270,  178677,  7, }, // America/Argentina/Cordoba
    {  589,  441,  178684,  5, }, // America/Argentina/Jujuy
    {  589, 7462,  178689,  8, }, // America/Argentina/La_Rioja
    {  589,  565,  178697,  6, }, // America/Argentina/Mendoza
    {  589, 7489,  178703,  9, }, // America/Argentina/Rio_Gallegos
    {  589, 7520,  178712,  4, }, // America/Argentina/Salta
    {  589, 7544,  178716,  7, }, // America/Argentina/San_Juan
    {  589, 5580,  178723,  8, }, // America/Argentina/San_Luis
    {  589, 7571,  178731,  7, }, // America/Argentina/Tucuman
    {  589, 7597,  178738,  7, }, // America/Argentina/Ushuaia
    {  589, 5640,  178745,  5, }, // America/Aruba
    {  589, 4252,  178750,  9, }, // America/Asuncion
    {  589,  237,  178759,  8, }, // America/Atikokan
    {  589, 3679,  178767,  5, }, // America/Bahia
    {  589, 7623,  178772, 12, }, // America/Bahia_Banderas
    {  589, 5654,  178784,  7, }, // America/Barbados
    {  589, 7646,  178791,  5, }, // America/Belem
    {  589, 5407,  178796,  5, }, // America/Belize
    {  589, 7660,  178801, 12, }, // America/Blanc-Sablon
    {  589, 7681,  178813,  9, }, // America/Boa_Vista
    {  589, 4361,  178822,  6, }, // America/Bogota
    {  589, 7699,  178828,  5, }, // America/Boise
    {  589, 7713,  178833,  9, }, // America/Cambridge_Bay
    {  589, 7735,  178842, 11, }, // America/Campo_Grande
    {  589, 3868,  178853,  6, }, // America/Cancun
    {  589, 4694,  178859,  6, }, // America/Caracas
    {  589, 4345,  178865,  5, }, // America/Cayenne
    {  589, 5496,  178870,  6, }, // America/Cayman
    {  589, 2260,  178876,  4, }, // America/Chicago
    {  589, 7756,  178880,  6, }, // America/Chihuahua
    {  589, 7774,  178886, 12, }, // America/Ciudad_Juarez
    {  589, 5422,  178898, 10, }, // America/Costa_Rica
    {  589, 7796,  178908,  7, }, // America/Creston
    {  589, 3791,  178915,  6, }, // America/Cuiaba
    {  589, 5723,  178921,  7, }, // America/Curacao
    {  589, 7812,  178928, 10, }, // America/Danmarkshavn
    {  589, 7833,  178938,  5, }, // America/Dawson
    {  589, 7848,  178943, 10, }, // America/Dawson_Creek
    {  589,  805,  178953,  4, }, // America/Denver
    {  589, 3465,  178957,  7, }, // America/Detroit
    {  589, 5739,  178964,  8, }, // America/Dominica
    {  589,  893,  178972,  8, }, // America/Edmonton
    {  589, 7869,  178980,  8, }, // America/Eirunepe
    {  589, 5441,  178988, 10, }, // America/El_Salvador
    {  589, 7886,  178998, 11, }, // America/Fort_Nelson
    {  589, 7906,  179009,  9, }, // America/Fortaleza
    {  589, 7924,  179018,  8, }, // America/Glace_Bay
    {  589, 6881,  179026,  7, }, // America/Goose_Bay
    {  589, 4612,  179033, 10, }, // America/Grand_Turk
    {  589, 5770,  179043,  6, }, // America/Grenada
    {  589, 5786,  179049,  9, }, // America/Guadeloupe
    {  589, 3760,  179058,  8, }, // America/Guatemala
    {  589, 4841,  179066,  9, }, // America/Guayaquil
    {  589, 6932,  179075,  6, }, // America/Guyana
    {  589, 1939,  179081,  7, }, // America/Halifax
    {  589, 2281,  179088,  6, }, // America/Havana
    {  589, 7942,  179094,  8, }, // America/Hermosillo
    {  589,  348,  179102, 12, }, // America/Indiana/Indianapolis
    {  589,  481,  179114, 13, }, // America/Indiana/Knox
    {  589, 7961,  179127, 15, }, // America/Indiana/Marengo
    {  589, 7985,  179142, 19, }, // America/Indiana/Petersburg
    {  589, 8012,  179161, 20, }, // America/Indiana/Tell_City
    {  589, 8038,  179181, 14, }, // America/Indiana/Vevay
    {  589, 8060,  179195, 17, }, // America/Indiana/Vincennes
    {  589, 8086,  179212, 16, }, // America/Indiana/Winamac
    {  589, 8110,  179228,  6, }, // America/Inuvik
    {  589,  660,  179234,  7, }, // America/Iqaluit
    {  589, 2799,  179241,  6, }, // America/Jamaica
    {  589, 5380,  179247,  4, }, // America/Juneau
    {  589,  521,  179251,  7, }, // America/Kentucky/Louisville
    {  589, 8125,  179258, 18, }, // America/Kentucky/Monticello
    {  589, 5704,  179276,  7, }, // America/Kralendijk
    {  589, 4376,  179283,  6, }, // America/La_Paz
    {  589, 7169,  179289,  4, }, // America/Lima
    {  589, 3239,  179293, 10, }, // America/Los_Angeles
    {  589, 5932,  179303, 19, }, // America/Lower_Princes
    {  589, 8153,  179322,  6, }, // America/Maceio
    {  589, 8168,  179328,  7, }, // America/Managua
    {  589, 1908,  179335,  6, }, // America/Manaus
    {  589, 5897,  179341,  6, }, // America/Marigot
    {  589, 5805,  179347,  8, }, // America/Martinique
    {  589, 8184,  179355,  9, }, // America/Matamoros
    {  589, 2917,  179364,  6, }, // America/Mazatlan
    {  589, 8202,  179370,  7, }, // America/Menominee
    {  589, 8220,  179377,  5, }, // America/Merida
    {  589, 8235,  179382,  8, }, // America/Metlakatla
    {  589, 2949,  179390, 14, }, // America/Mexico_City
    {  589, 4391,  179404,  6, }, // America/Miquelon
    {  589, 8254,  179410,  7, }, // America/Moncton
    {  589, 8270,  179417,  7, }, // America/Monterrey
    {  589, 4098,  179424, 10, }, // America/Montevideo
    {  589, 5824,  179434,  9, }, // America/Montserrat
    {  589, 5481,  179443,  5, }, // America/Nassau
    {  589, 2379,  179448,  8, }, // America/New_York
    {  589, 8288,  179371,  3, }, // America/Nome
    {  589, 1850,  179456,  7, }, // America/Noronha
    {  589, 8301,  179463, 21, }, // America/North_Dakota/Beulah
    {  589, 8329,  179484, 21, }, // America/North_Dakota/Center
    {  589, 8357,  179505, 26, }, // America/North_Dakota/New_Salem
    {  589,  393,  179114,  3, }, // America/Nuuk
    {  589, 8388,  179531,  7, }, // America/Ojinaga
    {  589, 2356,  179538,  4, }, // America/Panama
    {  589, 6173,  179542, 10, }, // America/Paramaribo
    {  589, 2973,  179552,  6, }, // America/Phoenix
    {  589, 3945,  179558, 13, }, // America/Port-au-Prince
    {  589, 5954,  179571, 12, }, // America/Port_of_Spain
    {  589, 8404,  179583, 11, }, // America/Porto_Velho
    {  589, 5843,  179594, 10, }, // America/Puerto_Rico
    {  589, 4030,  179604, 11, }, // America/Punta_Arenas
    {  589, 8424,  179615, 11, }, // America/Rankin_Inlet
    {  589, 8445,  179626,  6, }, // America/Recife
    {  589, 1995,  179632,  6, }, // America/Regina
    {  589, 8460,  179638,  7, }, // America/Resolute
    {  589,  695,  179645,  9, }, // America/Rio_Branco
    {  589, 8477,  179654,  7, }, // America/Santarem
    {  589, 2201,  179661,  8, }, // America/Santiago
    {  589, 6111,  179669, 12, }, // America/Santo_Domingo
    {  589, 1878,  179681,  9, }, // America/Sao_Paulo
    {  589, 6899,  179690, 10, }, // America/Scoresbysund
    {  589, 8494,  179700,  5, }, // America/Sitka
    {  589, 8508,  179705, 11, }, // America/St_Barthelemy
    {  589, 2061,  179716,  8, }, // America/St_Johns
    {  589, 5863,  179724,  8, }, // America/St_Kitts
    {  589, 5880,  179732,  9, }, // America/St_Lucia
    {  589,  855,  179741,  8, }, // America/St_Thomas
    {  589, 5913,  179749,  8, }, // America/St_Vincent
    {  589, 8530,  179757,  6, }, // America/Swift_Current
    {  589, 5461,  179763, 11, }, // America/Tegucigalpa
    {  589, 5756,  179711,  3, }, // America/Thule
    {  589,  313,  179774,  7, }, // America/Tijuana
    {  589,  608,  179781,  7, }, // America/Toronto
    {  589, 5688,  179788,  7, }, // America/Tortola
    {  589, 2093,  179795,  7, }, // America/Vancouver
    {  589, 2144,  179802, 11, }, // America/Whitehorse
    {  589,  734,  179813,  6, }, // America/Winnipeg
    {  589, 8552,  179819,  6, }, // America/Yakutat
    {  589, 6001,  179825,  5, }, // Antarctica/Casey
    {  589, 6094,  179830,  4, }, // Antarctica/Davis
    {  589, 6133,  179834, 14, }, // Antarctica/DumontDUrville
    {  589, 8568,  179848,  7, }, // Antarctica/Macquarie
    {  589, 7066,  179855,  6, }, // Antarctica/Mawson
    {  589, 7113,  179861,  7, }, // Antarctica/McMurdo
    {  589, 8589,  179868,  6, }, // Antarctica/Palmer
    {  589, 7226,  179874,  6, }, // Antarctica/Rothera
    {  589, 7295,  179880,  4, }, // Antarctica/Syowa
    {  589, 8607,  179884,  4, }, // Antarctica/Troll
    {  589, 7374,  179888,  6, }, // Antarctica/Vostok
    {  589, 1429,  179894, 11, }, // Arctic/Longyearbyen
    {  589, 5570,  179905,  3, }, // Asia/Aden
    {  589, 5395,  179908,  6, }, // Asia/Almaty
    {  589, 3968,  179914,  4, }, // Asia/Amman
    {  589, 5511,  179918,  6, }, // Asia/Anadyr
    {  589, 5523,  179924,  5, }, // Asia/Aqtau
    {  589, 5368,  179929,  5, }, // Asia/Aqtobe
    {  589,  964,  179934,  7, }, // Asia/Ashgabat
    {  589, 8624,  179941,  5, }, // Asia/Atyrau
    {  589, 3607,  179946,  4, }, // Asia/Baghdad
    {  589, 5534,  179950,  5, }, // Asia/Bahrain
    {  589, 3653,  178919,  4, }, // Asia/Baku
    {  589, 4466,  179955,  6, }, // Asia/Bangkok
    {  589, 3571,  179961,  7, }, // Asia/Barnaul
    {  589, 4086,  179968,  5, }, // Asia/Beirut
    {  589, 3778,  179973,  6, }, // Asia/Bishkek
    {  589, 5989,  179979,  6, }, // Asia/Brunei
    {  589, 4601,  179985,  5, }, // Asia/Chita
    {  589, 4511,  179990,  7, }, // Asia/Colombo
    {  589, 4540,  179997,  6, }, // Asia/Damascus
    {  589, 1093,  180003,  3, }, // Asia/Dhaka
    {  589, 6192,  180006,  4, }, // Asia/Dili
    {  589, 3596,  180010,  5, }, // Asia/Dubai
    {  589, 6159,  180015,  7, }, // Asia/Dushanbe
    {  589, 8636,  180022,  9, }, // Asia/Famagusta
    {  589, 8651,  180031,  3, }, // Asia/Gaza
    {  589, 4795,  180034,  5, }, // Asia/Hebron
    {  589, 1259,  180039, 17, }, // Asia/Ho_Chi_Minh
    {  589, 2704,  180056,  9, }, // Asia/Hong_Kong
    {  589, 4771,  180065,  4, }, // Asia/Hovd
    {  589, 4184,  180069,  8, }, // Asia/Irkutsk
    {  589, 7006,  180077,  7, }, // Asia/Jakarta
    {  589, 6992,  180084,  7, }, // Asia/Jayapura
    {  589, 1290,  180091,  6, }, // Asia/Jerusalem
    {  589, 3560,  180097,  5, }, // Asia/Kabul
    {  589, 4316,  180102,  9, }, // Asia/Kamchatka
    {  589, 4239,  180111,  5, }, // Asia/Karachi
    {  589, 1185,  180116,  8, }, // Asia/Kathmandu
    {  589, 8661,  180124,  7, }, // Asia/Khandyga
    {  589,  992,  180131,  6, }, // Asia/Kolkata
    {  589, 4197,  180137, 10, }, // Asia/Krasnoyarsk
    {  589, 4859,  180147, 12, }, // Asia/Kuala_Lumpur
    {  589, 5976,  180159,  6, }, // Asia/Kuching
    {  589, 5547,  180165,  4, }, // Asia/Kuwait
    {  589, 1211,  179848,  5, }, // Asia/Macau
    {  589, 4017,  180169,  7, }, // Asia/Magadan
    {  589, 1349,  180176,  7, }, // Asia/Makassar
    {  589, 7182,  180183,  6, }, // Asia/Manila
    {  589, 6920,  180189,  4, }, // Asia/Muscat
    {  589, 2554,  180193,  7, }, // Asia/Nicosia
    {  589, 8675,  180200, 12, }, // Asia/Novokuznetsk
    {  589, 4135,  180212, 10, }, // Asia/Novosibirsk
    {  589, 4229,  180222,  5, }, // Asia/Omsk
    {  589, 7145,  180227,  5, }, // Asia/Oral
    {  589, 6961,  180232,  7, }, // Asia/Phnom_Penh
    {  589, 8693,  180239,  9, }, // Asia/Pontianak
    {  589, 4214,  180248,  8, }, // Asia/Pyongyang
    {  589, 5559,  180256,  3, }, // Asia/Qatar
    {  589, 8708,  180259,  8, }, // Asia/Qostanay
    {  589, 4269,  180267,  9, }, // Asia/Qyzylorda
    {  589, 3584,  180276,  4, }, // Asia/Riyadh
    {  589, 4408,  180280,  7, }, // Asia/Sakhalin
    {  589, 7245,  180287,  6, }, // Asia/Samarkand
    {  589, 3279,  180293,  4, }, // Asia/Seoul
    {  589, 1053,  180297,  7, }, // Asia/Shanghai
    {  589, 3300,  180304,  8, }, // Asia/Singapore
    {  589, 4297,  180312, 13, }, // Asia/Srednekolymsk
    {  589, 3263,  180325,  6, }, // Asia/Taipei
    {  589, 4781,  180331,  6, }, // Asia/Tashkent
    {  589, 3915,  180337,  5, }, // Asia/Tbilisi
    {  589, 2772,  180342,  4, }, // Asia/Tehran
    {  589, 1317,  180346,  5, }, // Asia/Thimphu
    {  589, 2821,  180351,  5, }, // Asia/Tokyo
    {  589, 4572,  180356,  5, }, // Asia/Tomsk
    {  589, 1021,  180361, 10, }, // Asia/Ulaanbaatar
    {  589, 1159,  180371,  7, }, // Asia/Urumqi
    {  589, 8722,  180378,  9, }, // Asia/Ust-Nera
    {  589, 6977,  180387,  8, }, // Asia/Vientiane
    {  589, 4710,  180395, 11, }, // Asia/Vladivostok
    {  589, 4828,  180406,  7, }, // Asia/Yakutsk
    {  589, 1235,  180413,  5, }, // Asia/Yangon
    {  589, 3883,  180418, 11, }, // Asia/Yekaterinburg
    {  589, 3747,  180429,  5, }, // Asia/Yerevan
    {  589, 3663,  180434, 11, }, // Atlantic/Azores
    {  589, 5671,  180445,  8, }, // Atlantic/Bermuda
    {  589, 6628,  180453,  5, }, // Atlantic/Canary
    {  589, 3727,  180458,  9, }, // Atlantic/Cape_Verde
    {  589, 1395,  180467,  4, }, // Atlantic/Faroe
    {  589, 8736,  180471,  6, }, // Atlantic/Madeira
    {  589, 2748,  180477,  7, }, // Atlantic/Reykjavik
    {  589, 7272,  180484, 11, }, // Atlantic/South_Georgia
    {  589, 6821,  180495, 11, }, // Atlantic/St_Helena
    {  589, 6644,  180506,  6, }, // Atlantic/Stanley
    {  589, 1670,  180005,  6, }, // Australia/Adelaide
    {  589, 1635,  180512,  7, }, // Australia/Brisbane
    {  589, 1799,  180519, 10, }, // Australia/Broken_Hill
    {  589, 1583,  180529,  5, }, // Australia/Darwin
    {  589, 3637,  180534,  5, }, // Australia/Eucla
    {  589, 1516,  180539,  5, }, // Australia/Hobart
    {  589, 8753,  180544,  7, }, // Australia/Lindeman
    {  589, 1547,  180551,  8, }, // Australia/Lord_Howe
    {  589, 1727,  180559,  7, }, // Australia/Melbourne
    {  589, 1762,  180566,  4, }, // Australia/Perth
    {  589, 1463,  180570,  5, }, // Australia/Sydney
    {  589, 6441,  180575,  7, }, // Europe/Amsterdam
    {  589, 6216,  180582,  6, }, // Europe/Andorra
    {  589, 3620,  180588,  7, }, // Europe/Astrakhan
    {  589, 2300,  180595,  5, }, // Europe/Athens
    {  589, 6336,  180600,  5, }, // Europe/Belgrade
    {  589, 4757,  180605,  5, }, // Europe/Berlin
    {  589, 6488,  180610,  9, }, // Europe/Bratislava
    {  589, 2167,  180619,  6, }, // Europe/Brussels
    {  589, 3928,  180625,  7, }, // Europe/Bucharest
    {  589, 3806,  180632,  6, }, // Europe/Budapest
    {  589, 8772,  180638,  7, }, // Europe/Busingen
    {  589, 2583,  180645,  7, }, // Europe/Chisinau
    {  589, 6289,  180652,  8, }, // Europe/Copenhagen
    {  589, 2338,  180660,  4, }, // Europe/Dublin
    {  589, 6307,  180664,  6, }, // Europe/Gibraltar
    {  589, 8788,  180670,  6, }, // Europe/Guernsey
    {  589, 6612,  180676,  6, }, // Europe/Helsinki
    {  589, 8804,  180682, 10, }, // Europe/Isle_of_Man
    {  589, 1130,  180692,  7, }, // Europe/Istanbul
    {  589, 8823,  180699,  5, }, // Europe/Jersey
    {  589, 3979,  180704, 11, }, // Europe/Kaliningrad
    {  589, 8929,  180715,  5, }, // Europe/Kirov
    {  589, 2527,  180720,  4, }, // Europe/Kyiv
    {  589, 3213,  180724,  6, }, // Europe/Lisbon
    {  589, 6506,  180730,  9, }, // Europe/Ljubljana
    {  589, 2501,  180739,  6, }, // Europe/London
    {  589, 6365,  180745,  9, }, // Europe/Luxembourg
    {  589, 4892,  180754,  5, }, // Europe/Madrid
    {  589, 6397,  180759,  5, }, // Europe/Malta
    {  589, 6582,  180764,  7, }, // Europe/Mariehamn
    {  589, 3693,  180771,  5, }, // Europe/Minsk
    {  589, 6410,  180776,  6, }, // Europe/Monaco
    {  589, 3537,  180782,  5, }, // Europe/Moscow
    {  589, 6458,  180787,  5, }, // Europe/Oslo
    {  589, 4284,  180792,  5, }, // Europe/Paris
    {  589, 6424,  180797,  9, }, // Europe/Podgorica
    {  589, 6275,  180806,  4, }, // Europe/Prague
    {  589, 8837,  180810,  4, }, // Europe/Riga
    {  589, 6324,  180814,  4, }, // Europe/Rome
    {  589, 4331,  178310,  5, }, // Europe/Samara
    {  589, 6470,  180818,  8, }, // Europe/San_Marino
    {  589, 6245,  180826,  7, }, // Europe/Sarajevo
    {  589, 4451,  180833,  6, }, // Europe/Saratov
    {  589, 8849,  180839,  7, }, // Europe/Simferopol
    {  589, 6383,  180846,  6, }, // Europe/Skopje
    {  589, 6599,  180852,  5, }, // Europe/Sofia
    {  589, 6523,  180857,  8, }, // Europe/Stockholm
    {  589, 8867,  180865,  5, }, // Europe/Tallinn
    {  589, 6202,  180870,  5, }, // Europe/Tirane
    {  589, 8882,  180875,  9, }, // Europe/Ulyanovsk
    {  589, 6352,  180884,  4, }, // Europe/Vaduz
    {  589, 6567,  180888,  7, }, // Europe/Vatican
    {  589, 6231,  180895,  5, }, // Europe/Vienna
    {  589, 8899,  180900,  5, }, // Europe/Vilnius
    {  589, 4727,  180905,  9, }, // Europe/Volgograd
    {  589, 3190,  180914,  5, }, // Europe/Warsaw
    {  589, 6261,  180919,  5, }, // Europe/Zagreb
    {  589, 6540,  180924,  5, }, // Europe/Zurich
    {  589, 5078,  180929, 11, }, // Indian/Antananarivo
    {  589, 6947,  180940,  5, }, // Indian/Chagos
    {  589, 6046,  180945,  7, }, // Indian/Christmas
    {  589, 6063,  180952,  5, }, // Indian/Cocos
    {  589, 5029,  180957,  6, }, // Indian/Comoro
    {  589, 6661,  180963,  8, }, // Indian/Kerguelen
    {  589, 7260,  180971,  4, }, // Indian/Mahe
    {  589, 7050,  180975, 15, }, // Indian/Maldives
    {  589, 4069,  180990,  7, }, // Indian/Mauritius
    {  589, 5098,  180997,  5, }, // Indian/Mayotte
    {  589, 7211,  181002,  7, }, // Indian/Reunion
    {  589, 4422,  181009,  4, }, // Pacific/Apia
    {  589,  932,  181013,  6, }, // Pacific/Auckland
    {  589, 3706,  181019,  9, }, // Pacific/Bougainville
    {  589, 3015,  181028,  5, }, // Pacific/Chatham
    {  589, 3157,  181033,  4, }, // Pacific/Chuuk
    {  589, 2237,  181037,  4, }, // Pacific/Easter
    {  589, 7360,  181041,  4, }, // Pacific/Efate
    {  589, 7327,  181045,  7, }, // Pacific/Fakaofo
    {  589, 3902,  181052,  4, }, // Pacific/Fiji
    {  589, 7343,  181056,  8, }, // Pacific/Funafuti
    {  589, 6678,  181064,  8, }, // Pacific/Galapagos
    {  589, 6696,  181072,  5, }, // Pacific/Gambier
    {  589, 3822,  181077, 11, }, // Pacific/Guadalcanal
    {  589, 6018,  179332,  4, }, // Pacific/Guam
    {  589, 3049,  181088,  6, }, // Pacific/Kanton
    {  589, 3998,  181094,  8, }, // Pacific/Kiritimati
    {  589, 7019,  181102,  6, }, // Pacific/Kosrae
    {  589, 2842,  181108,  8, }, // Pacific/Kwajalein
    {  589, 4877,  181116,  6, }, // Pacific/Majuro
    {  589, 4051,  181122,  9, }, // Pacific/Marquesas
    {  589, 8914,  181131,  6, }, // Pacific/Midway
    {  589, 7084,  181137,  5, }, // Pacific/Nauru
    {  589, 7132,  181142,  4, }, // Pacific/Niue
    {  589, 4168,  181146,  6, }, // Pacific/Norfolk
    {  589, 7098,  181152,  5, }, // Pacific/Noumea
    {  589, 3126,  181157,  9, }, // Pacific/Pago_Pago
    {  589, 7155,  181166,  4, }, // Pacific/Palau
    {  589, 7194,  181170,  8, }, // Pacific/Pitcairn
    {  589, 3096,  181178,  7, }, // Pacific/Pohnpei
    {  589, 4807,  181185, 13, }, // Pacific/Port_Moresby
    {  589, 6076,  181198,  9, }, // Pacific/Rarotonga
    {  589, 6031,  181207,  6, }, // Pacific/Saipan
    {  589, 7312,  181213,  6, }, // Pacific/Tahiti
    {  589, 6712,  181219,  6, }, // Pacific/Tarawa
    {  589, 4583,  181225,  9, }, // Pacific/Tongatapu
    {  589, 7392,  181234,  4, }, // Pacific/Wake
    {  589, 7405,  181238,  5, }, // Pacific/Wallis
    {  591,   14,       0,  6, }, // Africa/Asmara Tachelhit/Tifinagh/Morocco
    {  591, 4435,       6,  8, }, // Africa/Sao_Tome
    {  591, 4252,      14,  8, }, // America/Asuncion
    {  591,  237,      22,  8, }, // America/Atikokan
    {  591, 7623,      30, 17, }, // America/Bahia_Banderas
    {  591, 3868,      47,  6, }, // America/Cancun
    {  591, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  591, 5723,      66,  7, }, // America/Curacao
    {  591,  481,      73, 13, }, // America/Indiana/Knox
    {  591, 7961,      86, 16, }, // America/Indiana/Marengo
    {  591, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  591, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  591, 8038,     139, 14, }, // America/Indiana/Vevay
    {  591, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  591, 8086,     171, 16, }, // America/Indiana/Winamac
    {  591, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  591, 5932,     207, 22, }, // America/Lower_Princes
    {  591, 8220,     229,  6, }, // America/Merida
    {  591, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  591, 8329,     255, 20, }, // America/North_Dakota/Center
    {  591, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  591,  393,     298,  4, }, // America/Nuuk
    {  591, 6899,     302, 16, }, // America/Scoresbysund
    {  591, 8508,     318, 14, }, // America/St_Barthelemy
    {  591, 2061,     332, 10, }, // America/St_Johns
    {  591, 5863,     342,  9, }, // America/St_Kitts
    {  591, 5880,     351,  9, }, // America/St_Lucia
    {  591,  855,     360, 10, }, // America/St_Thomas
    {  591, 5913,     370, 11, }, // America/St_Vincent
    {  591, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  591, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  591, 1185,     413,  9, }, // Asia/Kathmandu
    {  591,  992,     422,  7, }, // Asia/Kolkata
    {  591, 1211,     429,  5, }, // Asia/Macau
    {  591, 1235,     442,  6, }, // Asia/Yangon
    {  591, 1395,     448,  5, }, // Atlantic/Faroe
    {  591, 6821,     453, 10, }, // Atlantic/St_Helena
    {  591, 2527,     463,  4, }, // Europe/Kyiv
    {  591, 7211,     467,  7, }, // Indian/Reunion
    {  591, 3157,     474,  5, }, // Pacific/Chuuk
    {  591, 3096,     479,  7, }, // Pacific/Pohnpei
    {  593,   14,       0,  6, }, // Africa/Asmara Tai Dam/Tai Viet/Vietnam
    {  593, 4435,       6,  8, }, // Africa/Sao_Tome
    {  593, 4252,      14,  8, }, // America/Asuncion
    {  593,  237,      22,  8, }, // America/Atikokan
    {  593, 7623,      30, 17, }, // America/Bahia_Banderas
    {  593, 3868,      47,  6, }, // America/Cancun
    {  593, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  593, 5723,      66,  7, }, // America/Curacao
    {  593,  481,      73, 13, }, // America/Indiana/Knox
    {  593, 7961,      86, 16, }, // America/Indiana/Marengo
    {  593, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  593, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  593, 8038,     139, 14, }, // America/Indiana/Vevay
    {  593, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  593, 8086,     171, 16, }, // America/Indiana/Winamac
    {  593, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  593, 5932,     207, 22, }, // America/Lower_Princes
    {  593, 8220,     229,  6, }, // America/Merida
    {  593, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  593, 8329,     255, 20, }, // America/North_Dakota/Center
    {  593, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  593,  393,     298,  4, }, // America/Nuuk
    {  593, 6899,     302, 16, }, // America/Scoresbysund
    {  593, 8508,     318, 14, }, // America/St_Barthelemy
    {  593, 2061,     332, 10, }, // America/St_Johns
    {  593, 5863,     342,  9, }, // America/St_Kitts
    {  593, 5880,     351,  9, }, // America/St_Lucia
    {  593,  855,     360, 10, }, // America/St_Thomas
    {  593, 5913,     370, 11, }, // America/St_Vincent
    {  593, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  593, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  593, 1185,     413,  9, }, // Asia/Kathmandu
    {  593,  992,     422,  7, }, // Asia/Kolkata
    {  593, 1211,     429,  5, }, // Asia/Macau
    {  593, 1235,     442,  6, }, // Asia/Yangon
    {  593, 1395,     448,  5, }, // Atlantic/Faroe
    {  593, 6821,     453, 10, }, // Atlantic/St_Helena
    {  593, 2527,     463,  4, }, // Europe/Kyiv
    {  593, 7211,     467,  7, }, // Indian/Reunion
    {  593, 3157,     474,  5, }, // Pacific/Chuuk
    {  593, 3096,     479,  7, }, // Pacific/Pohnpei
    {  594,   14,       0,  6, }, // Africa/Asmara Taita/Latin/Kenya
    {  594, 4435,       6,  8, }, // Africa/Sao_Tome
    {  594, 4252,      14,  8, }, // America/Asuncion
    {  594,  237,      22,  8, }, // America/Atikokan
    {  594, 7623,      30, 17, }, // America/Bahia_Banderas
    {  594, 3868,      47,  6, }, // America/Cancun
    {  594, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  594, 5723,      66,  7, }, // America/Curacao
    {  594,  481,      73, 13, }, // America/Indiana/Knox
    {  594, 7961,      86, 16, }, // America/Indiana/Marengo
    {  594, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  594, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  594, 8038,     139, 14, }, // America/Indiana/Vevay
    {  594, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  594, 8086,     171, 16, }, // America/Indiana/Winamac
    {  594, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  594, 5932,     207, 22, }, // America/Lower_Princes
    {  594, 8220,     229,  6, }, // America/Merida
    {  594, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  594, 8329,     255, 20, }, // America/North_Dakota/Center
    {  594, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  594,  393,     298,  4, }, // America/Nuuk
    {  594, 6899,     302, 16, }, // America/Scoresbysund
    {  594, 8508,     318, 14, }, // America/St_Barthelemy
    {  594, 2061,     332, 10, }, // America/St_Johns
    {  594, 5863,     342,  9, }, // America/St_Kitts
    {  594, 5880,     351,  9, }, // America/St_Lucia
    {  594,  855,     360, 10, }, // America/St_Thomas
    {  594, 5913,     370, 11, }, // America/St_Vincent
    {  594, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  594, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  594, 1185,     413,  9, }, // Asia/Kathmandu
    {  594,  992,     422,  7, }, // Asia/Kolkata
    {  594, 1211,     429,  5, }, // Asia/Macau
    {  594, 1235,     442,  6, }, // Asia/Yangon
    {  594, 1395,     448,  5, }, // Atlantic/Faroe
    {  594, 6821,     453, 10, }, // Atlantic/St_Helena
    {  594, 2527,     463,  4, }, // Europe/Kyiv
    {  594, 7211,     467,  7, }, // Indian/Reunion
    {  594, 3157,     474,  5, }, // Pacific/Chuuk
    {  594, 3096,     479,  7, }, // Pacific/Pohnpei
    {  595, 6788,  181243,  7, }, // Africa/Abidjan Tajik/Cyrillic/Tajikistan
    {  595, 6760,     492,  5, }, // Africa/Accra
    {  595, 5059,     497, 11, }, // Africa/Addis_Ababa
    {  595, 7420,  181250,  8, }, // Africa/Algiers
    {  595,   14,   27065,  6, }, // Africa/Asmara
    {  595,   44,     519,  6, }, // Africa/Bamako
    {  595, 5257,   33724,  6, }, // Africa/Bangui
    {  595, 6746,  181258,  6, }, // Africa/Banjul
    {  595, 7435,     536,  5, }, // Africa/Bissau
    {  595, 4957,   27088,  8, }, // Africa/Blantyre
    {  595, 5287,     549,  9, }, // Africa/Brazzaville
    {  595, 4922,  181264,  9, }, // Africa/Bujumbura
    {  595, 2320,  181273,  6, }, // Africa/Cairo
    {  595, 4117,     571, 10, }, // Africa/Casablanca
    {  595, 7449,     581,  5, }, // Africa/Ceuta
    {  595, 6773,     586,  7, }, // Africa/Conakry
    {  595, 6840,     593,  5, }, // Africa/Dakar
    {  595, 5130,  181279, 10, }, // Africa/Dar_es_Salaam
    {  595, 5043,  181289,  6, }, // Africa/Djibouti
    {  595, 5243,     616,  5, }, // Africa/Douala
    {  595, 5166,  181295,  7, }, // Africa/El_Aaiun
    {  595, 6853,     628,  7, }, // Africa/Freetown
    {  595, 4906,     635,  8, }, // Africa/Gaborone
    {  595, 5015,   33781,  6, }, // Africa/Harare
    {  595, 4479,   51440, 12, }, // Africa/Johannesburg
    {  595, 4499,  181302,  4, }, // Africa/Juba
    {  595, 5151,     665,  7, }, // Africa/Kampala
    {  595, 4524,     672,  6, }, // Africa/Khartoum
    {  595, 4973,     678,  6, }, // Africa/Kigali
    {  595, 5306,     684,  7, }, // Africa/Kinshasa
    {  595, 4744,     691,  5, }, // Africa/Lagos
    {  595, 5336,   51452,  9, }, // Africa/Libreville
    {  595, 6869,     704,  4, }, // Africa/Lome
    {  595, 5211,     708,  6, }, // Africa/Luanda
    {  595, 4939,     714,  9, }, // Africa/Lubumbashi
    {  595, 5001,     723,  6, }, // Africa/Lusaka
    {  595, 5322,     729,  6, }, // Africa/Malabo
    {  595, 4987,     735,  6, }, // Africa/Maputo
    {  595, 5197,     741,  6, }, // Africa/Maseru
    {  595, 5182,     747,  7, }, // Africa/Mbabane
    {  595, 5113,     754,  8, }, // Africa/Mogadishu
    {  595, 7034,   36287,  8, }, // Africa/Monrovia
    {  595, 3853,   36295,  7, }, // Africa/Nairobi
    {  595, 5271,  181306,  7, }, // Africa/Ndjamena
    {  595, 5354,   36302,  6, }, // Africa/Niamey
    {  595, 6803,     790,  7, }, // Africa/Nouakchott
    {  595, 6727,     797,  8, }, // Africa/Ouagadougou
    {  595, 5225,     805, 10, }, // Africa/Porto-Novo
    {  595, 4435,     815,  8, }, // Africa/Sao_Tome
    {  595, 2866,     823,  7, }, // Africa/Tripoli
    {  595, 6554,     830,  5, }, // Africa/Tunis
    {  595, 4152,     835,  7, }, // Africa/Windhoek
    {  595,  132,     842,  4, }, // America/Adak
    {  595, 3346,  181313,  7, }, // America/Anchorage
    {  595, 5607,  181320,  6, }, // America/Anguilla
    {  595, 5624,     860,  7, }, // America/Antigua
    {  595, 4554,   36323,  9, }, // America/Araguaina
    {  595,  166,   51476, 12, }, // America/Argentina/Buenos_Aires
    {  595,   91,     888,  9, }, // America/Argentina/Catamarca
    {  595,  270,   33903,  7, }, // America/Argentina/Cordoba
    {  595,  441,  181326,  5, }, // America/Argentina/Jujuy
    {  595, 7462,   33915,  8, }, // America/Argentina/La_Rioja
    {  595,  565,   33923,  7, }, // America/Argentina/Mendoza
    {  595, 7489,  118049, 12, }, // America/Argentina/Rio_Gallegos
    {  595, 7520,     935,  5, }, // America/Argentina/Salta
    {  595, 7544,     940,  8, }, // America/Argentina/San_Juan
    {  595, 5580,   36360,  8, }, // America/Argentina/San_Luis
    {  595, 7571,     956,  7, }, // America/Argentina/Tucuman
    {  595, 7597,  106131,  6, }, // America/Argentina/Ushuaia
    {  595, 5640,     969,  5, }, // America/Aruba
    {  595, 4252,   33963,  8, }, // America/Asuncion
    {  595,  237,   33971,  8, }, // America/Atikokan
    {  595, 3679,  181331,  5, }, // America/Bahia
    {  595, 7623,  181336, 17, }, // America/Bahia_Banderas
    {  595, 5654,    1013,  8, }, // America/Barbados
    {  595, 7646,   33998,  5, }, // America/Belem
    {  595, 5407,    1026,  5, }, // America/Belize
    {  595, 7660,   36388, 11, }, // America/Blanc-Sablon
    {  595, 7681,   34022,  9, }, // America/Boa_Vista
    {  595, 4361,    1052,  6, }, // America/Bogota
    {  595, 7699,  181353,  4, }, // America/Boise
    {  595, 7713,  181357, 11, }, // America/Cambridge_Bay
    {  595, 7735,   34047, 12, }, // America/Campo_Grande
    {  595, 3868,    1087,  6, }, // America/Cancun
    {  595, 4694,    1093,  7, }, // America/Caracas
    {  595, 4345,   36417,  5, }, // America/Cayenne
    {  595, 5496,   27497,  6, }, // America/Cayman
    {  595, 2260,    1129,  6, }, // America/Chicago
    {  595, 7756,  118097,  8, }, // America/Chihuahua
    {  595, 7774,  181368, 12, }, // America/Ciudad_Juarez
    {  595, 5422,   36452, 10, }, // America/Costa_Rica
    {  595, 7796,    1151,  7, }, // America/Creston
    {  595, 3791,   27556,  5, }, // America/Cuiaba
    {  595, 5723,   36474,  7, }, // America/Curacao
    {  595, 7812,  137204, 11, }, // America/Danmarkshavn
    {  595, 7833,    1184,  6, }, // America/Dawson
    {  595, 7848,  137215, 11, }, // America/Dawson_Creek
    {  595,  805,    1201,  6, }, // America/Denver
    {  595, 3465,   36517,  7, }, // America/Detroit
    {  595, 5739,    1214,  8, }, // America/Dominica
    {  595,  893,   51549,  8, }, // America/Edmonton
    {  595, 7869,   51557,  8, }, // America/Eirunepe
    {  595, 5441,   27633,  9, }, // America/El_Salvador
    {  595, 7886,    1246, 11, }, // America/Fort_Nelson
    {  595, 7906,    1257,  9, }, // America/Fortaleza
    {  595, 7924,  181380,  9, }, // America/Glace_Bay
    {  595, 6881,  181389,  7, }, // America/Goose_Bay
    {  595, 4612,  181396, 10, }, // America/Grand_Turk
    {  595, 5770,    1292,  7, }, // America/Grenada
    {  595, 5786,    1299,  9, }, // America/Guadeloupe
    {  595, 3760,    1308,  9, }, // America/Guatemala
    {  595, 4841,   36577,  7, }, // America/Guayaquil
    {  595, 6932,   51601,  6, }, // America/Guyana
    {  595, 1939,   51607,  8, }, // America/Halifax
    {  595, 2281,  181406,  6, }, // America/Havana
    {  595, 7942,  118166,  9, }, // America/Hermosillo
    {  595,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  595,  481,    1365, 13, }, // America/Indiana/Knox
    {  595, 7961,    1378, 16, }, // America/Indiana/Marengo
    {  595, 7985,  181412, 18, }, // America/Indiana/Petersburg
    {  595, 8012,  137282, 17, }, // America/Indiana/Tell_City
    {  595, 8038,  181430, 14, }, // America/Indiana/Vevay
    {  595, 8060,  181444, 16, }, // America/Indiana/Vincennes
    {  595, 8086,  137316, 16, }, // America/Indiana/Winamac
    {  595, 8110,    1452,  6, }, // America/Inuvik
    {  595,  660,    1458,  7, }, // America/Iqaluit
    {  595, 2799,   27879,  6, }, // America/Jamaica
    {  595, 5380,  181460,  4, }, // America/Juneau
    {  595,  521,    1476,  8, }, // America/Kentucky/Louisville
    {  595, 8125,  118235, 20, }, // America/Kentucky/Monticello
    {  595, 5704,  181464,  9, }, // America/Kralendijk
    {  595, 4376,    1514,  6, }, // America/La_Paz
    {  595, 7169,    1520,  4, }, // America/Lima
    {  595, 3239,  181473, 11, }, // America/Los_Angeles
    {  595, 5932,  181484, 20, }, // America/Lower_Princes
    {  595, 8153,   36667,  6, }, // America/Maceio
    {  595, 8168,    1559,  7, }, // America/Managua
    {  595, 1908,    1566,  6, }, // America/Manaus
    {  595, 5897,    1572,  6, }, // America/Marigot
    {  595, 5805,    1578,  9, }, // America/Martinique
    {  595, 8184,    1587,  9, }, // America/Matamoros
    {  595, 2917,   34417,  8, }, // America/Mazatlan
    {  595, 8202,    1604,  7, }, // America/Menominee
    {  595, 8220,    1612,  6, }, // America/Merida
    {  595, 8235,    1618, 10, }, // America/Metlakatla
    {  595, 2949,    1628,  6, }, // America/Mexico_City
    {  595, 4391,    1634,  7, }, // America/Miquelon
    {  595, 8254,    1641,  7, }, // America/Moncton
    {  595, 8270,   51662,  9, }, // America/Monterrey
    {  595, 4098,    1657, 10, }, // America/Montevideo
    {  595, 5824,   51671,  9, }, // America/Montserrat
    {  595, 5481,    1677,  6, }, // America/Nassau
    {  595, 2379,  181504,  7, }, // America/New_York
    {  595, 8288,    1691,  3, }, // America/Nome
    {  595, 1850,   51688,  7, }, // America/Noronha
    {  595, 8301,  181511, 21, }, // America/North_Dakota/Beulah
    {  595, 8329,  181532, 21, }, // America/North_Dakota/Center
    {  595, 8357,  181553, 24, }, // America/North_Dakota/New_Salem
    {  595,  393,    1772,  4, }, // America/Nuuk
    {  595, 8388,  137453,  7, }, // America/Ojinaga
    {  595, 2356,    1783,  6, }, // America/Panama
    {  595, 6173,    1789, 10, }, // America/Paramaribo
    {  595, 2973,    1799,  6, }, // America/Phoenix
    {  595, 3945,    1805, 12, }, // America/Port-au-Prince
    {  595, 5954,  181577, 13, }, // America/Port_of_Spain
    {  595, 8404,  181590, 11, }, // America/Porto_Velho
    {  595, 5843,   51756, 11, }, // America/Puerto_Rico
    {  595, 4030,   34579, 12, }, // America/Punta_Arenas
    {  595, 8424,   34591, 12, }, // America/Rankin_Inlet
    {  595, 8445,    1876,  6, }, // America/Recife
    {  595, 1995,   34609,  6, }, // America/Regina
    {  595, 8460,   36760,  7, }, // America/Resolute
    {  595,  695,   34622, 10, }, // America/Rio_Branco
    {  595, 8477,   34632,  8, }, // America/Santarem
    {  595, 2201,   51775,  8, }, // America/Santiago
    {  595, 6111,   34649, 13, }, // America/Santo_Domingo
    {  595, 1878,    1936,  9, }, // America/Sao_Paulo
    {  595, 6899,  118416, 16, }, // America/Scoresbysund
    {  595, 8494,    1956,  5, }, // America/Sitka
    {  595, 8508,  181601, 14, }, // America/St_Barthelemy
    {  595, 2061,  181615,  9, }, // America/St_Johns
    {  595, 5863,   34704,  9, }, // America/St_Kitts
    {  595, 5880,   56356, 10, }, // America/St_Lucia
    {  595,  855,  128473, 10, }, // America/St_Thomas
    {  595, 5913,   34731, 12, }, // America/St_Vincent
    {  595, 8530,  181624, 13, }, // America/Swift_Current
    {  595, 5461,    2038, 11, }, // America/Tegucigalpa
    {  595, 5756,    2049,  4, }, // America/Thule
    {  595,  313,    2053,  7, }, // America/Tijuana
    {  595,  608,    2060,  7, }, // America/Toronto
    {  595, 5688,    2067,  7, }, // America/Tortola
    {  595, 2093,    2074,  8, }, // America/Vancouver
    {  595, 2144,   28435,  8, }, // America/Whitehorse
    {  595,  734,    2090,  8, }, // America/Winnipeg
    {  595, 8552,   28450,  6, }, // America/Yakutat
    {  595, 6001,   36889,  5, }, // Antarctica/Casey
    {  595, 6094,  118487,  5, }, // Antarctica/Davis
    {  595, 6133,  181637, 14, }, // Antarctica/DumontDUrville
    {  595, 8568,  181651,  8, }, // Antarctica/Macquarie
    {  595, 7066,  181659,  6, }, // Antarctica/Mawson
    {  595, 7113,  181665,  8, }, // Antarctica/McMurdo
    {  595, 8589,    2154,  6, }, // Antarctica/Palmer
    {  595, 7226,    2160,  6, }, // Antarctica/Rothera
    {  595, 7295,   28510,  4, }, // Antarctica/Syowa
    {  595, 8607,    2171,  5, }, // Antarctica/Troll
    {  595, 7374,   34825,  6, }, // Antarctica/Vostok
    {  595, 1429,  181673, 10, }, // Arctic/Longyearbyen
    {  595, 5570,  181683,  4, }, // Asia/Aden
    {  595, 5395,  181687,  7, }, // Asia/Almaty
    {  595, 3968,  181694,  5, }, // Asia/Amman
    {  595, 5511,   34848,  6, }, // Asia/Anadyr
    {  595, 5523,    2213,  5, }, // Asia/Aqtau
    {  595, 5368,    2218,  6, }, // Asia/Aqtobe
    {  595,  964,  181699,  7, }, // Asia/Ashgabat
    {  595, 8624,   51877,  6, }, // Asia/Atyrau
    {  595, 3607,  181706,  6, }, // Asia/Baghdad
    {  595, 5534,  181712,  7, }, // Asia/Bahrain
    {  595, 3653,  181719,  4, }, // Asia/Baku
    {  595, 4466,    2254,  7, }, // Asia/Bangkok
    {  595, 3571,    2261,  7, }, // Asia/Barnaul
    {  595, 4086,   28563,  6, }, // Asia/Beirut
    {  595, 3778,    2274,  6, }, // Asia/Bishkek
    {  595, 5989,   28575,  6, }, // Asia/Brunei
    {  595, 4601,    2286,  4, }, // Asia/Chita
    {  595, 4511,    2290,  7, }, // Asia/Colombo
    {  595, 4540,  181723,  6, }, // Asia/Damascus
    {  595, 1093,    2303,  5, }, // Asia/Dhaka
    {  595, 6192,    2308,  4, }, // Asia/Dili
    {  595, 3596,   28596,  5, }, // Asia/Dubai
    {  595, 6159,    2317,  7, }, // Asia/Dushanbe
    {  595, 8636,    2324,  9, }, // Asia/Famagusta
    {  595, 8651,  181729,  5, }, // Asia/Gaza
    {  595, 4795,   34886,  6, }, // Asia/Hebron
    {  595, 1259,   34892,  9, }, // Asia/Ho_Chi_Minh
    {  595, 2704,    2350,  7, }, // Asia/Hong_Kong
    {  595, 4771,    2357,  4, }, // Asia/Hovd
    {  595, 4184,    2361,  7, }, // Asia/Irkutsk
    {  595, 7006,  181734,  7, }, // Asia/Jakarta
    {  595, 6992,  181741,  7, }, // Asia/Jayapura
    {  595, 1290,   36962,  9, }, // Asia/Jerusalem
    {  595, 3560,  181748,  5, }, // Asia/Kabul
    {  595, 4316,    2411,  8, }, // Asia/Kamchatka
    {  595, 4239,    2422,  6, }, // Asia/Karachi
    {  595, 1185,    2428,  8, }, // Asia/Kathmandu
    {  595, 8661,   34933,  7, }, // Asia/Khandyga
    {  595,  992,   36971,  7, }, // Asia/Kolkata
    {  595, 4197,   36978, 10, }, // Asia/Krasnoyarsk
    {  595, 4859,   34958, 12, }, // Asia/Kuala_Lumpur
    {  595, 5976,    2474,  6, }, // Asia/Kuching
    {  595, 5547,  181753,  6, }, // Asia/Kuwait
    {  595, 1211,    2486,  5, }, // Asia/Macau
    {  595, 4017,    2491,  7, }, // Asia/Magadan
    {  595, 1349,    2498,  7, }, // Asia/Makassar
    {  595, 7182,    2505,  6, }, // Asia/Manila
    {  595, 6920,    2511,  6, }, // Asia/Muscat
    {  595, 2554,  106335,  7, }, // Asia/Nicosia
    {  595, 8675,  181759, 12, }, // Asia/Novokuznetsk
    {  595, 4135,    2535, 11, }, // Asia/Novosibirsk
    {  595, 4229,    2546,  4, }, // Asia/Omsk
    {  595, 7145,   35006,  4, }, // Asia/Oral
    {  595, 6961,    2556,  7, }, // Asia/Phnom_Penh
    {  595, 8693,    2563,  9, }, // Asia/Pontianak
    {  595, 4214,   36999,  6, }, // Asia/Pyongyang
    {  595, 5559,  181771,  5, }, // Asia/Qatar
    {  595, 8708,   28770,  8, }, // Asia/Qostanay
    {  595, 4269,  181776,  9, }, // Asia/Qyzylorda
    {  595, 3584,  181785,  4, }, // Asia/Riyadh
    {  595, 4408,    2614,  7, }, // Asia/Sakhalin
    {  595, 7245,  106372,  9, }, // Asia/Samarkand
    {  595, 3279,    2630,  4, }, // Asia/Seoul
    {  595, 1053,   28801,  6, }, // Asia/Shanghai
    {  595, 3300,    2640,  8, }, // Asia/Singapore
    {  595, 4297,   37026, 13, }, // Asia/Srednekolymsk
    {  595, 3263,   28829,  6, }, // Asia/Taipei
    {  595, 4781,  181789,  7, }, // Asia/Tashkent
    {  595, 3915,   35074,  7, }, // Asia/Tbilisi
    {  595, 2772,  181796,  6, }, // Asia/Tehran
    {  595, 1317,  181802,  5, }, // Asia/Thimphu
    {  595, 2821,    2692,  5, }, // Asia/Tokyo
    {  595, 4572,    2697,  5, }, // Asia/Tomsk
    {  595, 1021,    2702, 10, }, // Asia/Ulaanbaatar
    {  595, 1159,    2712,  6, }, // Asia/Urumqi
    {  595, 8722,    2718,  8, }, // Asia/Ust-Nera
    {  595, 6977,  181807,  7, }, // Asia/Vientiane
    {  595, 4710,    2733, 11, }, // Asia/Vladivostok
    {  595, 4828,   37051,  6, }, // Asia/Yakutsk
    {  595, 1235,   56409,  5, }, // Asia/Yangon
    {  595, 3883,    2757, 12, }, // Asia/Yekaterinburg
    {  595, 3747,    2769,  6, }, // Asia/Yerevan
    {  595, 3663,  181814, 14, }, // Atlantic/Azores
    {  595, 5671,   35152,  7, }, // Atlantic/Bermuda
    {  595, 6628,  181828,  7, }, // Atlantic/Canary
    {  595, 3727,    2826, 10, }, // Atlantic/Cape_Verde
    {  595, 1395,    2836,  5, }, // Atlantic/Faroe
    {  595, 8736,   37132,  7, }, // Atlantic/Madeira
    {  595, 2748,   37139,  8, }, // Atlantic/Reykjavik
    {  595, 7272,  181835, 14, }, // Atlantic/South_Georgia
    {  595, 6821,  118611, 10, }, // Atlantic/St_Helena
    {  595, 6644,   52124,  6, }, // Atlantic/Stanley
    {  595, 1670,    2917,  8, }, // Australia/Adelaide
    {  595, 1635,    2925,  7, }, // Australia/Brisbane
    {  595, 1799,    2932, 11, }, // Australia/Broken_Hill
    {  595, 1583,    2943,  6, }, // Australia/Darwin
    {  595, 3637,  181849,  5, }, // Australia/Eucla
    {  595, 1516,    2954,  6, }, // Australia/Hobart
    {  595, 8753,    2960,  8, }, // Australia/Lindeman
    {  595, 1547,  137700,  8, }, // Australia/Lord_Howe
    {  595, 1727,    2976,  7, }, // Australia/Melbourne
    {  595, 1762,    2983,  4, }, // Australia/Perth
    {  595, 1463,   52144,  6, }, // Australia/Sydney
    {  595, 6441,    2993,  9, }, // Europe/Amsterdam
    {  595, 6216,    3002,  7, }, // Europe/Andorra
    {  595, 3620,   29149,  9, }, // Europe/Astrakhan
    {  595, 2300,   52159,  5, }, // Europe/Athens
    {  595, 6336,    3021,  7, }, // Europe/Belgrade
    {  595, 4757,    3028,  6, }, // Europe/Berlin
    {  595, 6488,    3034, 10, }, // Europe/Bratislava
    {  595, 2167,   52164,  7, }, // Europe/Brussels
    {  595, 3928,    3052,  8, }, // Europe/Bucharest
    {  595, 3806,    3060,  8, }, // Europe/Budapest
    {  595, 8772,   35305,  8, }, // Europe/Busingen
    {  595, 2583,  137712,  7, }, // Europe/Chisinau
    {  595, 6289,   52191, 10, }, // Europe/Copenhagen
    {  595, 2338,    3108,  6, }, // Europe/Dublin
    {  595, 6307,    3114,  9, }, // Europe/Gibraltar
    {  595, 8788,    3123,  6, }, // Europe/Guernsey
    {  595, 6612,    3129,  8, }, // Europe/Helsinki
    {  595, 8804,  181854, 11, }, // Europe/Isle_of_Man
    {  595, 1130,   35352,  8, }, // Europe/Istanbul
    {  595, 8823,  181865,  5, }, // Europe/Jersey
    {  595, 3979,    3158, 11, }, // Europe/Kaliningrad
    {  595, 8929,    3169,  5, }, // Europe/Kirov
    {  595, 2527,    3174,  4, }, // Europe/Kyiv
    {  595, 3213,    3178,  8, }, // Europe/Lisbon
    {  595, 6506,   29300,  7, }, // Europe/Ljubljana
    {  595, 2501,    3195,  6, }, // Europe/London
    {  595, 6365,   29313, 10, }, // Europe/Luxembourg
    {  595, 4892,    3212,  6, }, // Europe/Madrid
    {  595, 6397,    3218,  5, }, // Europe/Malta
    {  595, 6582,   35400,  8, }, // Europe/Mariehamn
    {  595, 3693,    3232,  5, }, // Europe/Minsk
    {  595, 6410,    3237,  6, }, // Europe/Monaco
    {  595, 3537,    3243,  6, }, // Europe/Moscow
    {  595, 6458,    3249,  4, }, // Europe/Oslo
    {  595, 4284,    3253,  5, }, // Europe/Paris
    {  595, 6424,  181870, 10, }, // Europe/Podgorica
    {  595, 6275,    3267,  5, }, // Europe/Prague
    {  595, 8837,    3272,  4, }, // Europe/Riga
    {  595, 6324,    3276,  3, }, // Europe/Rome
    {  595, 4331,    3279,  6, }, // Europe/Samara
    {  595, 6470,    3285, 10, }, // Europe/San_Marino
    {  595, 6245,    3295,  7, }, // Europe/Sarajevo
    {  595, 4451,    3302,  7, }, // Europe/Saratov
    {  595, 8849,    3309, 10, }, // Europe/Simferopol
    {  595, 6383,  118645,  6, }, // Europe/Skopje
    {  595, 6599,   37286,  5, }, // Europe/Sofia
    {  595, 6523,  181880,  8, }, // Europe/Stockholm
    {  595, 8867,    3337,  6, }, // Europe/Tallinn
    {  595, 6202,    3343,  5, }, // Europe/Tirane
    {  595, 8882,   37291,  8, }, // Europe/Ulyanovsk
    {  595, 6352,   35474,  5, }, // Europe/Vaduz
    {  595, 6567,    3363,  7, }, // Europe/Vatican
    {  595, 6231,    3370,  4, }, // Europe/Vienna
    {  595, 8899,   37304,  6, }, // Europe/Vilnius
    {  595, 4727,    3381,  9, }, // Europe/Volgograd
    {  595, 3190,    3390,  7, }, // Europe/Warsaw
    {  595, 6261,    3397,  6, }, // Europe/Zagreb
    {  595, 6540,  181888,  5, }, // Europe/Zurich
    {  595, 5078,    3409, 12, }, // Indian/Antananarivo
    {  595, 6947,    3421,  5, }, // Indian/Chagos
    {  595, 6046,  181893,  8, }, // Indian/Christmas
    {  595, 6063,    3438,  5, }, // Indian/Cocos
    {  595, 5029,   35510,  6, }, // Indian/Comoro
    {  595, 6661,  181901,  8, }, // Indian/Kerguelen
    {  595, 7260,   35516,  4, }, // Indian/Mahe
    {  595, 7050,    3471,  6, }, // Indian/Maldives
    {  595, 4069,   56576,  8, }, // Indian/Mauritius
    {  595, 5098,   56584,  7, }, // Indian/Mayotte
    {  595, 7211,   37369,  7, }, // Indian/Reunion
    {  595, 4422,    3498,  4, }, // Pacific/Apia
    {  595,  932,    3502,  6, }, // Pacific/Auckland
    {  595, 3706,  181909,  9, }, // Pacific/Bougainville
    {  595, 3015,   37380,  5, }, // Pacific/Chatham
    {  595, 3157,  128740,  3, }, // Pacific/Chuuk
    {  595, 2237,  118673,  5, }, // Pacific/Easter
    {  595, 7360,   52409,  5, }, // Pacific/Efate
    {  595, 7327,    3541,  7, }, // Pacific/Fakaofo
    {  595, 3902,  181918,  4, }, // Pacific/Fiji
    {  595, 7343,    3552,  8, }, // Pacific/Funafuti
    {  595, 6678,    3560,  9, }, // Pacific/Galapagos
    {  595, 6696,  181922,  6, }, // Pacific/Gambier
    {  595, 3822,   35587, 11, }, // Pacific/Guadalcanal
    {  595, 6018,    3606,  4, }, // Pacific/Guam
    {  595, 3049,    3610,  6, }, // Pacific/Kanton
    {  595, 3998,    3616, 10, }, // Pacific/Kiritimati
    {  595, 7019,    3626,  6, }, // Pacific/Kosrae
    {  595, 2842,  137764,  9, }, // Pacific/Kwajalein
    {  595, 4877,  118697,  6, }, // Pacific/Majuro
    {  595, 4051,  128774,  6, }, // Pacific/Marquesas
    {  595, 8914,  181928,  6, }, // Pacific/Midway
    {  595, 7084,    3680,  5, }, // Pacific/Nauru
    {  595, 7132,   52504,  4, }, // Pacific/Niue
    {  595, 4168,    3689,  7, }, // Pacific/Norfolk
    {  595, 7098,    3696,  5, }, // Pacific/Noumea
    {  595, 3126,   35642,  9, }, // Pacific/Pago_Pago
    {  595, 7155,    3710,  5, }, // Pacific/Palau
    {  595, 7194,   56657,  7, }, // Pacific/Pitcairn
    {  595, 3096,   37471,  6, }, // Pacific/Pohnpei
    {  595, 4807,   37477, 11, }, // Pacific/Port_Moresby
    {  595, 6076,    3739,  9, }, // Pacific/Rarotonga
    {  595, 6031,   29803,  6, }, // Pacific/Saipan
    {  595, 7312,    3754,  5, }, // Pacific/Tahiti
    {  595, 6712,    3759,  6, }, // Pacific/Tarawa
    {  595, 4583,    3765,  9, }, // Pacific/Tongatapu
    {  595, 7392,  181934,  4, }, // Pacific/Wake
    {  595, 7405,    3778,  6, }, // Pacific/Wallis
    {  596, 6788,  181938,  9, }, // Africa/Abidjan Tamil/Tamil/India
    {  596, 6760,  181947,  5, }, // Africa/Accra
    {  596, 5059,  181952, 11, }, // Africa/Addis_Ababa
    {  596, 7420,  181963, 10, }, // Africa/Algiers
    {  596,   14,  181973,  7, }, // Africa/Asmara
    {  596,   44,  181980,  5, }, // Africa/Bamako
    {  596, 5257,  181985,  8, }, // Africa/Bangui
    {  596, 6746,  181993,  7, }, // Africa/Banjul
    {  596, 7435,  182000,  6, }, // Africa/Bissau
    {  596, 4957,  182006, 11, }, // Africa/Blantyre
    {  596, 5287,  182017, 12, }, // Africa/Brazzaville
    {  596, 4922,  182029, 10, }, // Africa/Bujumbura
    {  596, 2320,  182039,  6, }, // Africa/Cairo
    {  596, 4117,  182045, 12, }, // Africa/Casablanca
    {  596, 7449,  182057,  8, }, // Africa/Ceuta
    {  596, 6773,  182065,  7, }, // Africa/Conakry
    {  596, 6840,  182072,  5, }, // Africa/Dakar
    {  596, 5130,  182077, 14, }, // Africa/Dar_es_Salaam
    {  596, 5043,  182091,  8, }, // Africa/Djibouti
    {  596, 5243,  182099,  5, }, // Africa/Douala
    {  596, 5166,  182104,  9, }, // Africa/El_Aaiun
    {  596, 6853,  182113, 10, }, // Africa/Freetown
    {  596, 4906,  182123,  7, }, // Africa/Gaborone
    {  596, 5015,  182130,  5, }, // Africa/Harare
    {  596, 4479,  182135, 14, }, // Africa/Johannesburg
    {  596, 4499,  182149,  4, }, // Africa/Juba
    {  596, 5151,  182153,  7, }, // Africa/Kampala
    {  596, 4524,  182160,  8, }, // Africa/Khartoum
    {  596, 4973,  182168,  5, }, // Africa/Kigali
    {  596, 5306,  182173,  7, }, // Africa/Kinshasa
    {  596, 4744,  182180,  6, }, // Africa/Lagos
    {  596, 5336,  182186, 12, }, // Africa/Libreville
    {  596, 6869,  182198,  4, }, // Africa/Lome
    {  596, 5211,  182202,  8, }, // Africa/Luanda
    {  596, 4939,  182210, 10, }, // Africa/Lubumbashi
    {  596, 5001,  182220,  6, }, // Africa/Lusaka
    {  596, 5322,  182226,  5, }, // Africa/Malabo
    {  596, 4987,  182231,  5, }, // Africa/Maputo
    {  596, 5197,  182236,  5, }, // Africa/Maseru
    {  596, 5182,  182241,  5, }, // Africa/Mbabane
    {  596, 5113,  182246,  8, }, // Africa/Mogadishu
    {  596, 7034,  182254, 10, }, // Africa/Monrovia
    {  596, 3853,  182264,  6, }, // Africa/Nairobi
    {  596, 5271,  182270,  5, }, // Africa/Ndjamena
    {  596, 5354,  182275,  6, }, // Africa/Niamey
    {  596, 6803,  182281, 10, }, // Africa/Nouakchott
    {  596, 6727,  182291,  9, }, // Africa/Ouagadougou
    {  596, 5225,  182300, 13, }, // Africa/Porto-Novo
    {  596, 4435,  182313,  9, }, // Africa/Sao_Tome
    {  596, 2866,  182322,  8, }, // Africa/Tripoli
    {  596, 6554,  182330,  6, }, // Africa/Tunis
    {  596, 4152,  182336, 10, }, // Africa/Windhoek
    {  596,  132,  182346,  4, }, // America/Adak
    {  596, 3346,  182350,  9, }, // America/Anchorage
    {  596, 5607,  182359,  9, }, // America/Anguilla
    {  596, 5624,  182368,  9, }, // America/Antigua
    {  596, 4554,  182377, 10, }, // America/Araguaina
    {  596,  166,  182387, 14, }, // America/Argentina/Buenos_Aires
    {  596,   91,  182401,  8, }, // America/Argentina/Catamarca
    {  596,  270,  182409,  8, }, // America/Argentina/Cordoba
    {  596,  441,  182417,  6, }, // America/Argentina/Jujuy
    {  596, 7462,  182423,  9, }, // America/Argentina/La_Rioja
    {  596,  565,  182432,  8, }, // America/Argentina/Mendoza
    {  596, 7489,  182440, 13, }, // America/Argentina/Rio_Gallegos
    {  596, 7520,  182453,  6, }, // America/Argentina/Salta
    {  596, 7544,  182459, 11, }, // America/Argentina/San_Juan
    {  596, 5580,  182470, 11, }, // America/Argentina/San_Luis
    {  596, 7571,  182481,  7, }, // America/Argentina/Tucuman
    {  596, 7597,  182488,  7, }, // America/Argentina/Ushuaia
    {  596, 5640,  182495,  5, }, // America/Aruba
    {  596, 4252,  182500,  9, }, // America/Asuncion
    {  596,  237,  182509,  8, }, // America/Atikokan
    {  596, 3679,  182517,  6, }, // America/Bahia
    {  596, 7623,  182523, 15, }, // America/Bahia_Banderas
    {  596, 5654,  182538,  9, }, // America/Barbados
    {  596, 7646,  182547,  6, }, // America/Belem
    {  596, 5407,  182553,  6, }, // America/Belize
    {  596, 7660,  182559, 16, }, // America/Blanc-Sablon
    {  596, 7681,  182575, 11, }, // America/Boa_Vista
    {  596, 4361,  182586,  6, }, // America/Bogota
    {  596, 7699,  182592,  6, }, // America/Boise
    {  596, 7713,  182598, 20, }, // America/Cambridge_Bay
    {  596, 7735,  182618, 15, }, // America/Campo_Grande
    {  596, 3868,  182633,  7, }, // America/Cancun
    {  596, 4694,  182640,  6, }, // America/Caracas
    {  596, 4345,  182646,  6, }, // America/Cayenne
    {  596, 5496,  182652,  5, }, // America/Cayman
    {  596, 2260,  182657,  6, }, // America/Chicago
    {  596, 7756,  182663,  6, }, // America/Chihuahua
    {  596, 7774,  182669, 14, }, // America/Ciudad_Juarez
    {  596, 5422,  182683, 11, }, // America/Costa_Rica
    {  596, 7796,  182694, 10, }, // America/Creston
    {  596, 3791,  182704,  6, }, // America/Cuiaba
    {  596, 5723,  182710,  8, }, // America/Curacao
    {  596, 7812,  182718, 16, }, // America/Danmarkshavn
    {  596, 7833,  182734,  7, }, // America/Dawson
    {  596, 7848,  182741, 14, }, // America/Dawson_Creek
    {  596,  805,  182755,  7, }, // America/Denver
    {  596, 3465,  182762, 10, }, // America/Detroit
    {  596, 5739,  182772,  8, }, // America/Dominica
    {  596,  893,  182780, 11, }, // America/Edmonton
    {  596, 7869,  182791,  7, }, // America/Eirunepe
    {  596, 5441,  182798, 13, }, // America/El_Salvador
    {  596, 7886,  182811, 15, }, // America/Fort_Nelson
    {  596, 7906,  182826, 11, }, // America/Fortaleza
    {  596, 7924,  182837, 14, }, // America/Glace_Bay
    {  596, 6881,  182851,  7, }, // America/Goose_Bay
    {  596, 4612,  182858, 14, }, // America/Grand_Turk
    {  596, 5770,  182872,  6, }, // America/Grenada
    {  596, 5786,  182878,  9, }, // America/Guadeloupe
    {  596, 3760,  182887,  8, }, // America/Guatemala
    {  596, 4841,  182895,  9, }, // America/Guayaquil
    {  596, 6932,  182904,  5, }, // America/Guyana
    {  596, 1939,  182909, 10, }, // America/Halifax
    {  596, 2281,  182919,  5, }, // America/Havana
    {  596, 7942,  182924, 10, }, // America/Hermosillo
    {  596,  348,  182934, 15, }, // America/Indiana/Indianapolis
    {  596,  481,  182949, 17, }, // America/Indiana/Knox
    {  596, 7961,  182966, 18, }, // America/Indiana/Marengo
    {  596, 7985,  182984, 26, }, // America/Indiana/Petersburg
    {  596, 8012,  183010, 22, }, // America/Indiana/Tell_City
    {  596, 8038,  183032, 15, }, // America/Indiana/Vevay
    {  596, 8060,  183047, 21, }, // America/Indiana/Vincennes
    {  596, 8086,  183068, 19, }, // America/Indiana/Winamac
    {  596, 8110,  183087,  7, }, // America/Inuvik
    {  596,  660,  183094,  9, }, // America/Iqaluit
    {  596, 2799,  183103,  7, }, // America/Jamaica
    {  596, 5380,  183110,  6, }, // America/Juneau
    {  596,  521,  183116, 12, }, // America/Kentucky/Louisville
    {  596, 8125,  183128, 21, }, // America/Kentucky/Monticello
    {  596, 5704,  183149, 11, }, // America/Kralendijk
    {  596, 4376,  183160,  7, }, // America/La_Paz
    {  596, 7169,  183167,  4, }, // America/Lima
    {  596, 3239,  183171, 13, }, // America/Los_Angeles
    {  596, 5932,  183184, 27, }, // America/Lower_Princes
    {  596, 8153,  183211,  6, }, // America/Maceio
    {  596, 8168,  183217,  8, }, // America/Managua
    {  596, 1908,  183225,  5, }, // America/Manaus
    {  596, 5897,  183230,  8, }, // America/Marigot
    {  596, 5805,  183238, 12, }, // America/Martinique
    {  596, 8184,  183250,  8, }, // America/Matamoros
    {  596, 2917,  183258,  7, }, // America/Mazatlan
    {  596, 8202,  183265,  8, }, // America/Menominee
    {  596, 8220,  183273,  6, }, // America/Merida
    {  596, 8235,  183279, 10, }, // America/Metlakatla
    {  596, 2949,  183289, 14, }, // America/Mexico_City
    {  596, 4391,  183303,  9, }, // America/Miquelon
    {  596, 8254,  183312,  8, }, // America/Moncton
    {  596, 8270,  183320, 10, }, // America/Monterrey
    {  596, 4098,  183330, 12, }, // America/Montevideo
    {  596, 5824,  183342, 10, }, // America/Montserrat
    {  596, 5481,  183352,  4, }, // America/Nassau
    {  596, 2379,  183356, 10, }, // America/New_York
    {  596, 8288,  183366,  4, }, // America/Nome
    {  596, 1850,  183370,  7, }, // America/Noronha
    {  596, 8301,  183377, 22, }, // America/North_Dakota/Beulah
    {  596, 8329,  183399, 21, }, // America/North_Dakota/Center
    {  596, 8357,  183420, 24, }, // America/North_Dakota/New_Salem
    {  596,  393,  183444,  4, }, // America/Nuuk
    {  596, 8388,  183448,  6, }, // America/Ojinaga
    {  596, 2356,  183454,  5, }, // America/Panama
    {  596, 6173,  183459,  7, }, // America/Paramaribo
    {  596, 2973,  183466,  9, }, // America/Phoenix
    {  596, 3945,  183475, 19, }, // America/Port-au-Prince
    {  596, 5954,  183494, 20, }, // America/Port_of_Spain
    {  596, 8404,  183514, 15, }, // America/Porto_Velho
    {  596, 5843,  183529, 13, }, // America/Puerto_Rico
    {  596, 4030,  183542, 14, }, // America/Punta_Arenas
    {  596, 8424,  183556, 16, }, // America/Rankin_Inlet
    {  596, 8445,  183572,  7, }, // America/Recife
    {  596, 1995,  183579,  6, }, // America/Regina
    {  596, 8460,  183585,  7, }, // America/Resolute
    {  596,  695,  183592, 13, }, // America/Rio_Branco
    {  596, 8477,  183605,  9, }, // America/Santarem
    {  596, 2201,  183614, 10, }, // America/Santiago
    {  596, 6111,  183624, 15, }, // America/Santo_Domingo
    {  596, 1878,  183639, 10, }, // America/Sao_Paulo
    {  596, 6899,  183649, 15, }, // America/Scoresbysund
    {  596, 8494,  183664,  6, }, // America/Sitka
    {  596, 8508,  183670, 19, }, // America/St_Barthelemy
    {  596, 2061,  183689, 15, }, // America/St_Johns
    {  596, 5863,  183704, 15, }, // America/St_Kitts
    {  596, 5880,  183719, 15, }, // America/St_Lucia
    {  596,  855,  183734, 14, }, // America/St_Thomas
    {  596, 5913,  183748, 19, }, // America/St_Vincent
    {  596, 8530,  183767, 16, }, // America/Swift_Current
    {  596, 5461,  183783, 11, }, // America/Tegucigalpa
    {  596, 5756,  183794,  4, }, // America/Thule
    {  596,  313,  183798,  8, }, // America/Tijuana
    {  596,  608,  183806,  8, }, // America/Toronto
    {  596, 5688,  183814,  8, }, // America/Tortola
    {  596, 2093,  183822,  9, }, // America/Vancouver
    {  596, 2144,  183831, 12, }, // America/Whitehorse
    {  596,  734,  183843, 10, }, // America/Winnipeg
    {  596, 8552,  183853,  6, }, // America/Yakutat
    {  596, 6001,  183859,  4, }, // Antarctica/Casey
    {  596, 6094,  183863,  6, }, // Antarctica/Davis
    {  596, 6133,  183869, 21, }, // Antarctica/DumontDUrville
    {  596, 8568,  183890, 10, }, // Antarctica/Macquarie
    {  596, 7066,  183900,  5, }, // Antarctica/Mawson
    {  596, 7113,  183905,  9, }, // Antarctica/McMurdo
    {  596, 8589,  183914,  7, }, // Antarctica/Palmer
    {  596, 7226,  183921,  6, }, // Antarctica/Rothera
    {  596, 7295,  183927,  6, }, // Antarctica/Syowa
    {  596, 8607,  183933,  6, }, // Antarctica/Troll
    {  596, 7374,  183939,  8, }, // Antarctica/Vostok
    {  596, 1429,  183947, 13, }, // Arctic/Longyearbyen
    {  596, 5570,  183960,  4, }, // Asia/Aden
    {  596, 5395,  183964,  7, }, // Asia/Almaty
    {  596, 3968,  183971,  7, }, // Asia/Amman
    {  596, 5511,  183978,  6, }, // Asia/Anadyr
    {  596, 5523,  183984,  6, }, // Asia/Aqtau
    {  596, 5368,  183990,  7, }, // Asia/Aqtobe
    {  596,  964,  183997,  9, }, // Asia/Ashgabat
    {  596, 8624,  184006,  5, }, // Asia/Atyrau
    {  596, 3607,  184011,  8, }, // Asia/Baghdad
    {  596, 5534,  184019,  7, }, // Asia/Bahrain
    {  596, 3653,  184026,  6, }, // Asia/Baku
    {  596, 4466,  184032,  8, }, // Asia/Bangkok
    {  596, 3571,  184040,  8, }, // Asia/Barnaul
    {  596, 4086,  184048,  8, }, // Asia/Beirut
    {  596, 3778,  184056,  8, }, // Asia/Bishkek
    {  596, 5989,  184064,  6, }, // Asia/Brunei
    {  596, 4601,  184070,  4, }, // Asia/Chita
    {  596, 4511,  184074,  8, }, // Asia/Colombo
    {  596, 4540,  184082,  8, }, // Asia/Damascus
    {  596, 1093,  184090,  6, }, // Asia/Dhaka
    {  596, 6192,  184096,  4, }, // Asia/Dili
    {  596, 3596,  184100,  6, }, // Asia/Dubai
    {  596, 6159,  184106,  7, }, // Asia/Dushanbe
    {  596, 8636,  184113, 10, }, // Asia/Famagusta
    {  596, 8651,  182045,  4, }, // Asia/Gaza
    {  596, 4795,  184123,  8, }, // Asia/Hebron
    {  596, 1259,  184131, 17, }, // Asia/Ho_Chi_Minh
    {  596, 2704,  184148,  8, }, // Asia/Hong_Kong
    {  596, 4771,  184156,  6, }, // Asia/Hovd
    {  596, 4184,  184162, 11, }, // Asia/Irkutsk
    {  596, 7006,  184173,  9, }, // Asia/Jakarta
    {  596, 6992,  184182,  7, }, // Asia/Jayapura
    {  596, 1290,  184189,  9, }, // Asia/Jerusalem
    {  596, 3560,  184198,  6, }, // Asia/Kabul
    {  596, 4316,  184204,  9, }, // Asia/Kamchatka
    {  596, 4239,  184213,  7, }, // Asia/Karachi
    {  596, 1185,  184220, 10, }, // Asia/Kathmandu
    {  596, 8661,  184230,  8, }, // Asia/Khandyga
    {  596,  992,  184238,  9, }, // Asia/Kolkata
    {  596, 4197,  184247, 16, }, // Asia/Krasnoyarsk
    {  596, 4859,  184263, 11, }, // Asia/Kuala_Lumpur
    {  596, 5976,  184274,  6, }, // Asia/Kuching
    {  596, 5547,  184280,  6, }, // Asia/Kuwait
    {  596, 1211,  184286,  5, }, // Asia/Macau
    {  596, 4017,  184291,  5, }, // Asia/Magadan
    {  596, 1349,  184296,  7, }, // Asia/Makassar
    {  596, 7182,  184303,  5, }, // Asia/Manila
    {  596, 6920,  184308,  6, }, // Asia/Muscat
    {  596, 2554,  184314,  8, }, // Asia/Nicosia
    {  596, 8675,  184322, 16, }, // Asia/Novokuznetsk
    {  596, 4135,  184338, 14, }, // Asia/Novosibirsk
    {  596, 4229,  184352,  7, }, // Asia/Omsk
    {  596, 7145,  184359,  4, }, // Asia/Oral
    {  596, 6961,  184363, 12, }, // Asia/Phnom_Penh
    {  596, 8693,  184375, 12, }, // Asia/Pontianak
    {  596, 4214,  184387, 10, }, // Asia/Pyongyang
    {  596, 5559,  184397,  7, }, // Asia/Qatar
    {  596, 8708,  184404,  8, }, // Asia/Qostanay
    {  596, 4269,  184412, 10, }, // Asia/Qyzylorda
    {  596, 3584,  184422,  6, }, // Asia/Riyadh
    {  596, 4408,  184428,  6, }, // Asia/Sakhalin
    {  596, 7245,  184434, 10, }, // Asia/Samarkand
    {  596, 3279,  184444,  6, }, // Asia/Seoul
    {  596, 1053,  184450,  8, }, // Asia/Shanghai
    {  596, 3300,  184458, 11, }, // Asia/Singapore
    {  596, 4297,  184469, 18, }, // Asia/Srednekolymsk
    {  596, 3263,  184487,  6, }, // Asia/Taipei
    {  596, 4781,  184493,  9, }, // Asia/Tashkent
    {  596, 3915,  184502,  8, }, // Asia/Tbilisi
    {  596, 2772,  184510,  8, }, // Asia/Tehran
    {  596, 1317,  184518,  6, }, // Asia/Thimphu
    {  596, 2821,  184524,  8, }, // Asia/Tokyo
    {  596, 4572,  184532,  8, }, // Asia/Tomsk
    {  596, 1021,  184540, 12, }, // Asia/Ulaanbaatar
    {  596, 1159,  184552,  7, }, // Asia/Urumqi
    {  596, 8722,  184559, 10, }, // Asia/Ust-Nera
    {  596, 6977,  184569, 10, }, // Asia/Vientiane
    {  596, 4710,  184579, 14, }, // Asia/Vladivostok
    {  596, 4828,  184593,  9, }, // Asia/Yakutsk
    {  596, 1235,  184602,  7, }, // Asia/Yangon
    {  596, 3883,  184609, 15, }, // Asia/Yekaterinburg
    {  596, 3747,  184624,  6, }, // Asia/Yerevan
    {  596, 3663,  184630,  6, }, // Atlantic/Azores
    {  596, 5671,  184636,  8, }, // Atlantic/Bermuda
    {  596, 6628,  184644,  5, }, // Atlantic/Canary
    {  596, 3727,  184649, 11, }, // Atlantic/Cape_Verde
    {  596, 1395,  184660,  4, }, // Atlantic/Faroe
    {  596, 8736,  184664,  5, }, // Atlantic/Madeira
    {  596, 2748,  184669, 10, }, // Atlantic/Reykjavik
    {  596, 7272,  184679, 15, }, // Atlantic/South_Georgia
    {  596, 6821,  184694, 15, }, // Atlantic/St_Helena
    {  596, 6644,  184709,  8, }, // Atlantic/Stanley
    {  596, 1670,  184717,  9, }, // Australia/Adelaide
    {  596, 1635,  184726, 10, }, // Australia/Brisbane
    {  596, 1799,  184736, 14, }, // Australia/Broken_Hill
    {  596, 1583,  184750,  8, }, // Australia/Darwin
    {  596, 3637,  184758,  6, }, // Australia/Eucla
    {  596, 1516,  184764,  7, }, // Australia/Hobart
    {  596, 8753,  184771,  9, }, // Australia/Lindeman
    {  596, 1547,  184780, 11, }, // Australia/Lord_Howe
    {  596, 1727,  184791, 10, }, // Australia/Melbourne
    {  596, 1762,  184801,  6, }, // Australia/Perth
    {  596, 1463,  184807,  6, }, // Australia/Sydney
    {  596, 6441,  184813, 12, }, // Europe/Amsterdam
    {  596, 6216,  184825,  7, }, // Europe/Andorra
    {  596, 3620,  184832, 11, }, // Europe/Astrakhan
    {  596, 2300,  184843,  6, }, // Europe/Athens
    {  596, 6336,  184849, 10, }, // Europe/Belgrade
    {  596, 4757,  184859,  8, }, // Europe/Berlin
    {  596, 6488,  184867, 11, }, // Europe/Bratislava
    {  596, 2167,  184878,  9, }, // Europe/Brussels
    {  596, 3928,  184887, 10, }, // Europe/Bucharest
    {  596, 3806,  184897, 10, }, // Europe/Budapest
    {  596, 8772,  184907,  9, }, // Europe/Busingen
    {  596, 2583,  184916,  7, }, // Europe/Chisinau
    {  596, 6289,  184923, 10, }, // Europe/Copenhagen
    {  596, 2338,  184933,  7, }, // Europe/Dublin
    {  596, 6307,  184940, 11, }, // Europe/Gibraltar
    {  596, 8788,  184951,  6, }, // Europe/Guernsey
    {  596, 6612,  184957, 10, }, // Europe/Helsinki
    {  596, 8804,  184967, 13, }, // Europe/Isle_of_Man
    {  596, 1130,  184980, 11, }, // Europe/Istanbul
    {  596, 8823,  184991,  6, }, // Europe/Jersey
    {  596, 3979,  184997, 13, }, // Europe/Kaliningrad
    {  596, 8929,  185010,  6, }, // Europe/Kirov
    {  596, 2527,  185016,  4, }, // Europe/Kyiv
    {  596, 3213,  185020,  7, }, // Europe/Lisbon
    {  596, 6506,  185027,  8, }, // Europe/Ljubljana
    {  596, 2501,  185035,  6, }, // Europe/London
    {  596, 6365,  185041, 11, }, // Europe/Luxembourg
    {  596, 4892,  185052,  8, }, // Europe/Madrid
    {  596, 6397,  185060,  6, }, // Europe/Malta
    {  596, 6582,  185066,  6, }, // Europe/Mariehamn
    {  596, 3693,  185072,  8, }, // Europe/Minsk
    {  596, 6410,  185080,  8, }, // Europe/Monaco
    {  596, 3537,  185088,  6, }, // Europe/Moscow
    {  596, 6458,  185094,  5, }, // Europe/Oslo
    {  596, 4284,  185099,  6, }, // Europe/Paris
    {  596, 6424,  185105, 10, }, // Europe/Podgorica
    {  596, 6275,  185115,  6, }, // Europe/Prague
    {  596, 8837,  182690,  4, }, // Europe/Riga
    {  596, 6324,  185121,  4, }, // Europe/Rome
    {  596, 4331,  185125,  5, }, // Europe/Samara
    {  596, 6470,  185130, 10, }, // Europe/San_Marino
    {  596, 6245,  185140,  6, }, // Europe/Sarajevo
    {  596, 4451,  185146,  6, }, // Europe/Saratov
    {  596, 8849,  185152, 13, }, // Europe/Simferopol
    {  596, 6383,  185165,  8, }, // Europe/Skopje
    {  596, 6599,  185173,  7, }, // Europe/Sofia
    {  596, 6523,  185180, 10, }, // Europe/Stockholm
    {  596, 8867,  185190,  5, }, // Europe/Tallinn
    {  596, 6202,  185195,  6, }, // Europe/Tirane
    {  596, 8882,  185201, 11, }, // Europe/Ulyanovsk
    {  596, 6352,  185212,  5, }, // Europe/Vaduz
    {  596, 6567,  185217,  7, }, // Europe/Vatican
    {  596, 6231,  185224,  7, }, // Europe/Vienna
    {  596, 8899,  185231,  9, }, // Europe/Vilnius
    {  596, 4727,  185240, 12, }, // Europe/Volgograd
    {  596, 3190,  185252,  6, }, // Europe/Warsaw
    {  596, 6261,  185258,  7, }, // Europe/Zagreb
    {  596, 6540,  185265,  6, }, // Europe/Zurich
    {  596, 5078,  185271, 10, }, // Indian/Antananarivo
    {  596, 6947,  185281,  6, }, // Indian/Chagos
    {  596, 6046,  185287, 11, }, // Indian/Christmas
    {  596, 6063,  185298,  6, }, // Indian/Cocos
    {  596, 5029,  185304,  5, }, // Indian/Comoro
    {  596, 6661,  185309, 10, }, // Indian/Kerguelen
    {  596, 7260,  185319,  4, }, // Indian/Mahe
    {  596, 7050,  185323, 12, }, // Indian/Maldives
    {  596, 4069,  185335,  9, }, // Indian/Mauritius
    {  596, 5098,  185344,  7, }, // Indian/Mayotte
    {  596, 7211,  185351,  9, }, // Indian/Reunion
    {  596, 4422,  185360,  5, }, // Pacific/Apia
    {  596,  932,  185365,  9, }, // Pacific/Auckland
    {  596, 3706,  185374, 14, }, // Pacific/Bougainville
    {  596, 3015,  185388,  7, }, // Pacific/Chatham
    {  596, 3157,  185395,  4, }, // Pacific/Chuuk
    {  596, 2237,  185399,  6, }, // Pacific/Easter
    {  596, 7360,  185405,  6, }, // Pacific/Efate
    {  596, 7327,  185411,  7, }, // Pacific/Fakaofo
    {  596, 3902,  185418,  5, }, // Pacific/Fiji
    {  596, 7343,  185423, 10, }, // Pacific/Funafuti
    {  596, 6678,  185433,  7, }, // Pacific/Galapagos
    {  596, 6696,  185440,  9, }, // Pacific/Gambier
    {  596, 3822,  185449, 13, }, // Pacific/Guadalcanal
    {  596, 6018,  185462,  6, }, // Pacific/Guam
    {  596, 2723,  185468,  8, }, // Pacific/Honolulu
    {  596, 3049,  185476,  9, }, // Pacific/Kanton
    {  596, 3998,  185485, 12, }, // Pacific/Kiritimati
    {  596, 7019,  185497,  6, }, // Pacific/Kosrae
    {  596, 2842,  185503, 11, }, // Pacific/Kwajalein
    {  596, 4877,  185514,  5, }, // Pacific/Majuro
    {  596, 4051,  185519, 12, }, // Pacific/Marquesas
    {  596, 8914,  185531,  6, }, // Pacific/Midway
    {  596, 7084,  185537,  5, }, // Pacific/Nauru
    {  596, 7132,  185542,  4, }, // Pacific/Niue
    {  596, 4168,  185546,  9, }, // Pacific/Norfolk
    {  596, 7098,  185555,  8, }, // Pacific/Noumea
    {  596, 3126,  185563,  9, }, // Pacific/Pago_Pago
    {  596, 7155,  185572,  5, }, // Pacific/Palau
    {  596, 7194,  185577, 12, }, // Pacific/Pitcairn
    {  596, 3096,  185589,  9, }, // Pacific/Pohnpei
    {  596, 4807,  185598, 15, }, // Pacific/Port_Moresby
    {  596, 6076,  185613,  9, }, // Pacific/Rarotonga
    {  596, 6031,  185622,  5, }, // Pacific/Saipan
    {  596, 7312,  185627,  5, }, // Pacific/Tahiti
    {  596, 6712,  185632,  5, }, // Pacific/Tarawa
    {  596, 4583,  185637, 10, }, // Pacific/Tongatapu
    {  596, 7392,  185647,  4, }, // Pacific/Wake
    {  596, 7405,  185651,  6, }, // Pacific/Wallis
    {  600,   14,       0,  6, }, // Africa/Asmara Taroko/Latin/Taiwan
    {  600, 4435,       6,  8, }, // Africa/Sao_Tome
    {  600, 3346,  185657, 19, }, // America/Anchorage
    {  600, 4252,      14,  8, }, // America/Asuncion
    {  600,  237,      22,  8, }, // America/Atikokan
    {  600, 7623,      30, 17, }, // America/Bahia_Banderas
    {  600, 7735,  185676, 16, }, // America/Campo_Grande
    {  600, 3868,      47,  6, }, // America/Cancun
    {  600, 2260,  185692, 18, }, // America/Chicago
    {  600, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  600, 5723,      66,  7, }, // America/Curacao
    {  600,  805,  185710, 17, }, // America/Denver
    {  600,  348,  185727, 25, }, // America/Indiana/Indianapolis
    {  600,  481,      73, 13, }, // America/Indiana/Knox
    {  600, 7961,      86, 16, }, // America/Indiana/Marengo
    {  600, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  600, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  600, 8038,     139, 14, }, // America/Indiana/Vevay
    {  600, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  600, 8086,     171, 16, }, // America/Indiana/Winamac
    {  600, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  600, 3239,  185752, 19, }, // America/Los_Angeles
    {  600, 5932,     207, 22, }, // America/Lower_Princes
    {  600, 8220,     229,  6, }, // America/Merida
    {  600, 2379,  185771, 18, }, // America/New_York
    {  600, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  600, 8329,     255, 20, }, // America/North_Dakota/Center
    {  600, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  600,  393,     298,  4, }, // America/Nuuk
    {  600, 2973,  185789, 19, }, // America/Phoenix
    {  600,  695,  185808, 18, }, // America/Rio_Branco
    {  600, 1878,  185826, 20, }, // America/Sao_Paulo
    {  600, 6899,     302, 16, }, // America/Scoresbysund
    {  600, 8508,     318, 14, }, // America/St_Barthelemy
    {  600, 2061,     332, 10, }, // America/St_Johns
    {  600, 5863,     342,  9, }, // America/St_Kitts
    {  600, 5880,     351,  9, }, // America/St_Lucia
    {  600,  855,     360, 10, }, // America/St_Thomas
    {  600, 5913,     370, 11, }, // America/St_Vincent
    {  600, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  600, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  600, 1185,     413,  9, }, // Asia/Kathmandu
    {  600,  992,     422,  7, }, // Asia/Kolkata
    {  600, 1211,     429,  5, }, // Asia/Macau
    {  600, 1235,     442,  6, }, // Asia/Yangon
    {  600, 1395,     448,  5, }, // Atlantic/Faroe
    {  600, 6821,     453, 10, }, // Atlantic/St_Helena
    {  600, 2527,     463,  4, }, // Europe/Kyiv
    {  600, 7211,     467,  7, }, // Indian/Reunion
    {  600, 3157,     474,  5, }, // Pacific/Chuuk
    {  600, 2723,  185846, 20, }, // Pacific/Honolulu
    {  600, 3096,     479,  7, }, // Pacific/Pohnpei
    {  601,   14,       0,  6, }, // Africa/Asmara Tasawaq/Latin/Niger
    {  601, 4435,       6,  8, }, // Africa/Sao_Tome
    {  601, 4252,      14,  8, }, // America/Asuncion
    {  601,  237,      22,  8, }, // America/Atikokan
    {  601, 7623,      30, 17, }, // America/Bahia_Banderas
    {  601, 3868,      47,  6, }, // America/Cancun
    {  601, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  601, 5723,      66,  7, }, // America/Curacao
    {  601,  481,      73, 13, }, // America/Indiana/Knox
    {  601, 7961,      86, 16, }, // America/Indiana/Marengo
    {  601, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  601, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  601, 8038,     139, 14, }, // America/Indiana/Vevay
    {  601, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  601, 8086,     171, 16, }, // America/Indiana/Winamac
    {  601, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  601, 5932,     207, 22, }, // America/Lower_Princes
    {  601, 8220,     229,  6, }, // America/Merida
    {  601, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  601, 8329,     255, 20, }, // America/North_Dakota/Center
    {  601, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  601,  393,     298,  4, }, // America/Nuuk
    {  601, 6899,     302, 16, }, // America/Scoresbysund
    {  601, 8508,     318, 14, }, // America/St_Barthelemy
    {  601, 2061,     332, 10, }, // America/St_Johns
    {  601, 5863,     342,  9, }, // America/St_Kitts
    {  601, 5880,     351,  9, }, // America/St_Lucia
    {  601,  855,     360, 10, }, // America/St_Thomas
    {  601, 5913,     370, 11, }, // America/St_Vincent
    {  601, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  601, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  601, 1185,     413,  9, }, // Asia/Kathmandu
    {  601,  992,     422,  7, }, // Asia/Kolkata
    {  601, 1211,     429,  5, }, // Asia/Macau
    {  601, 1235,     442,  6, }, // Asia/Yangon
    {  601, 1395,     448,  5, }, // Atlantic/Faroe
    {  601, 6821,     453, 10, }, // Atlantic/St_Helena
    {  601, 2527,     463,  4, }, // Europe/Kyiv
    {  601, 7211,     467,  7, }, // Indian/Reunion
    {  601, 3157,     474,  5, }, // Pacific/Chuuk
    {  601, 3096,     479,  7, }, // Pacific/Pohnpei
    {  602, 6788,  185866,  6, }, // Africa/Abidjan Tatar/Cyrillic/Russia
    {  602, 6760,     492,  5, }, // Africa/Accra
    {  602, 5059,     497, 11, }, // Africa/Addis_Ababa
    {  602, 7420,     508,  5, }, // Africa/Algiers
    {  602,   14,   56104,  6, }, // Africa/Asmara
    {  602,   44,     519,  6, }, // Africa/Bamako
    {  602, 5257,     525,  5, }, // Africa/Bangui
    {  602, 6746,  185872,  6, }, // Africa/Banjul
    {  602, 7435,  137123,  6, }, // Africa/Bissau
    {  602, 4957,   27088,  8, }, // Africa/Blantyre
    {  602, 5287,   51422, 10, }, // Africa/Brazzaville
    {  602, 4922,  185878,  9, }, // Africa/Bujumbura
    {  602, 2320,  185887,  6, }, // Africa/Cairo
    {  602, 4117,     571, 10, }, // Africa/Casablanca
    {  602, 7449,     581,  5, }, // Africa/Ceuta
    {  602, 6773,     586,  7, }, // Africa/Conakry
    {  602, 6840,     593,  5, }, // Africa/Dakar
    {  602, 5130,   27116, 12, }, // Africa/Dar_es_Salaam
    {  602, 5043,   36247,  7, }, // Africa/Djibouti
    {  602, 5243,     616,  5, }, // Africa/Douala
    {  602, 5166,   27135,  7, }, // Africa/El_Aaiun
    {  602, 6853,     628,  7, }, // Africa/Freetown
    {  602, 4906,     635,  8, }, // Africa/Gaborone
    {  602, 5015,   33781,  6, }, // Africa/Harare
    {  602, 4479,   51440, 12, }, // Africa/Johannesburg
    {  602, 4499,  185893,  4, }, // Africa/Juba
    {  602, 5151,     665,  7, }, // Africa/Kampala
    {  602, 4524,  185897,  6, }, // Africa/Khartoum
    {  602, 4973,     678,  6, }, // Africa/Kigali
    {  602, 5306,     684,  7, }, // Africa/Kinshasa
    {  602, 4744,     691,  5, }, // Africa/Lagos
    {  602, 5336,   51452,  9, }, // Africa/Libreville
    {  602, 6869,     704,  4, }, // Africa/Lome
    {  602, 5211,     708,  6, }, // Africa/Luanda
    {  602, 4939,     714,  9, }, // Africa/Lubumbashi
    {  602, 5001,     723,  6, }, // Africa/Lusaka
    {  602, 5322,     729,  6, }, // Africa/Malabo
    {  602, 4987,   33813,  6, }, // Africa/Maputo
    {  602, 5197,     741,  6, }, // Africa/Maseru
    {  602, 5182,     747,  7, }, // Africa/Mbabane
    {  602, 5113,     754,  8, }, // Africa/Mogadishu
    {  602, 7034,   36287,  8, }, // Africa/Monrovia
    {  602, 3853,   36295,  7, }, // Africa/Nairobi
    {  602, 5271,  185903,  6, }, // Africa/Ndjamena
    {  602, 5354,   36302,  6, }, // Africa/Niamey
    {  602, 6803,     790,  7, }, // Africa/Nouakchott
    {  602, 6727,     797,  8, }, // Africa/Ouagadougou
    {  602, 5225,     805, 10, }, // Africa/Porto-Novo
    {  602, 4435,     815,  8, }, // Africa/Sao_Tome
    {  602, 2866,     823,  7, }, // Africa/Tripoli
    {  602, 6554,     830,  5, }, // Africa/Tunis
    {  602, 4152,     835,  7, }, // Africa/Windhoek
    {  602,  132,     842,  4, }, // America/Adak
    {  602, 3346,   51461,  8, }, // America/Anchorage
    {  602, 5607,   51469,  7, }, // America/Anguilla
    {  602, 5624,     860,  7, }, // America/Antigua
    {  602, 4554,   36323,  9, }, // America/Araguaina
    {  602,  166,   51476, 12, }, // America/Argentina/Buenos_Aires
    {  602,   91,     888,  9, }, // America/Argentina/Catamarca
    {  602,  270,   33903,  7, }, // America/Argentina/Cordoba
    {  602,  441,   27355,  5, }, // America/Argentina/Jujuy
    {  602, 7462,   33915,  8, }, // America/Argentina/La_Rioja
    {  602,  565,     917,  7, }, // America/Argentina/Mendoza
    {  602, 7489,   51488, 12, }, // America/Argentina/Rio_Gallegos
    {  602, 7520,   27380,  6, }, // America/Argentina/Salta
    {  602, 7544,     940,  8, }, // America/Argentina/San_Juan
    {  602, 5580,     948,  8, }, // America/Argentina/San_Luis
    {  602, 7571,     956,  7, }, // America/Argentina/Tucuman
    {  602, 7597,  106131,  6, }, // America/Argentina/Ushuaia
    {  602, 5640,     969,  5, }, // America/Aruba
    {  602, 4252,   51500,  8, }, // America/Asuncion
    {  602,  237,   33971,  8, }, // America/Atikokan
    {  602, 3679,   36368,  4, }, // America/Bahia
    {  602, 7623,   51508, 16, }, // America/Bahia_Banderas
    {  602, 5654,    1013,  8, }, // America/Barbados
    {  602, 7646,   33998,  5, }, // America/Belem
    {  602, 5407,    1026,  5, }, // America/Belize
    {  602, 7660,   36388, 11, }, // America/Blanc-Sablon
    {  602, 7681,    1043,  9, }, // America/Boa_Vista
    {  602, 4361,    1052,  6, }, // America/Bogota
    {  602, 7699,   51524,  5, }, // America/Boise
    {  602, 7713,  185909, 12, }, // America/Cambridge_Bay
    {  602, 7735,  137179, 12, }, // America/Campo_Grande
    {  602, 3868,    1087,  6, }, // America/Cancun
    {  602, 4694,    1093,  7, }, // America/Caracas
    {  602, 4345,   51542,  7, }, // America/Cayenne
    {  602, 5496,  185921, 16, }, // America/Cayman
    {  602, 2260,    1129,  6, }, // America/Chicago
    {  602, 7756,  118097,  8, }, // America/Chihuahua
    {  602, 7774,   56160, 13, }, // America/Ciudad_Juarez
    {  602, 5422,    1141, 10, }, // America/Costa_Rica
    {  602, 7796,    1151,  7, }, // America/Creston
    {  602, 3791,   27556,  5, }, // America/Cuiaba
    {  602, 5723,   36474,  7, }, // America/Curacao
    {  602, 7812,   36481, 12, }, // America/Danmarkshavn
    {  602, 7833,    1184,  6, }, // America/Dawson
    {  602, 7848,    1190, 11, }, // America/Dawson_Creek
    {  602,  805,    1201,  6, }, // America/Denver
    {  602, 3465,   36517,  7, }, // America/Detroit
    {  602, 5739,    1214,  8, }, // America/Dominica
    {  602,  893,   51549,  8, }, // America/Edmonton
    {  602, 7869,   51557,  8, }, // America/Eirunepe
    {  602, 5441,   27633,  9, }, // America/El_Salvador
    {  602, 7886,   51565, 12, }, // America/Fort_Nelson
    {  602, 7906,    1257,  9, }, // America/Fortaleza
    {  602, 7924,  185937,  9, }, // America/Glace_Bay
    {  602, 6881,  185946, 11, }, // America/Goose_Bay
    {  602, 4612,  181396, 10, }, // America/Grand_Turk
    {  602, 5770,    1292,  7, }, // America/Grenada
    {  602, 5786,    1299,  9, }, // America/Guadeloupe
    {  602, 3760,    1308,  9, }, // America/Guatemala
    {  602, 4841,   51593,  8, }, // America/Guayaquil
    {  602, 6932,   51601,  6, }, // America/Guyana
    {  602, 1939,   51607,  8, }, // America/Halifax
    {  602, 2281,    1339,  6, }, // America/Havana
    {  602, 7942,   51615,  9, }, // America/Hermosillo
    {  602,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  602,  481,    1365, 13, }, // America/Indiana/Knox
    {  602, 7961,    1378, 16, }, // America/Indiana/Marengo
    {  602, 7985,  181412, 18, }, // America/Indiana/Petersburg
    {  602, 8012,  185957, 19, }, // America/Indiana/Tell_City
    {  602, 8038,   56191, 14, }, // America/Indiana/Vevay
    {  602, 8060,   56205, 18, }, // America/Indiana/Vincennes
    {  602, 8086,   56223, 16, }, // America/Indiana/Winamac
    {  602, 8110,    1452,  6, }, // America/Inuvik
    {  602,  660,    1458,  7, }, // America/Iqaluit
    {  602, 2799,   27879,  6, }, // America/Jamaica
    {  602, 5380,   36614,  5, }, // America/Juneau
    {  602,  521,  185976,  8, }, // America/Kentucky/Louisville
    {  602, 8125,  118235, 20, }, // America/Kentucky/Monticello
    {  602, 5704,   36619, 10, }, // America/Kralendijk
    {  602, 4376,    1514,  6, }, // America/La_Paz
    {  602, 7169,    1520,  4, }, // America/Lima
    {  602, 3239,   51629, 12, }, // America/Los_Angeles
    {  602, 5932,  185984, 22, }, // America/Lower_Princes
    {  602, 8153,   36667,  6, }, // America/Maceio
    {  602, 8168,    1559,  7, }, // America/Managua
    {  602, 1908,    1566,  6, }, // America/Manaus
    {  602, 5897,    1572,  6, }, // America/Marigot
    {  602, 5805,    1578,  9, }, // America/Martinique
    {  602, 8184,    1587,  9, }, // America/Matamoros
    {  602, 2917,    1596,  8, }, // America/Mazatlan
    {  602, 8202,    1604,  8, }, // America/Menominee
    {  602, 8220,    1612,  6, }, // America/Merida
    {  602, 8235,    1618, 10, }, // America/Metlakatla
    {  602, 2949,    1628,  6, }, // America/Mexico_City
    {  602, 4391,    1634,  7, }, // America/Miquelon
    {  602, 8254,    1641,  7, }, // America/Moncton
    {  602, 8270,   51662,  9, }, // America/Monterrey
    {  602, 4098,    1657, 10, }, // America/Montevideo
    {  602, 5824,   51671,  9, }, // America/Montserrat
    {  602, 5481,    1677,  6, }, // America/Nassau
    {  602, 2379,   51680,  8, }, // America/New_York
    {  602, 8288,  186006,  4, }, // America/Nome
    {  602, 1850,   56257,  7, }, // America/Noronha
    {  602, 8301,  186010, 20, }, // America/North_Dakota/Beulah
    {  602, 8329,  186030, 19, }, // America/North_Dakota/Center
    {  602, 8357,  186049, 24, }, // America/North_Dakota/New_Salem
    {  602,  393,    1772,  4, }, // America/Nuuk
    {  602, 8388,    1776,  7, }, // America/Ojinaga
    {  602, 2356,    1783,  6, }, // America/Panama
    {  602, 6173,    1789, 10, }, // America/Paramaribo
    {  602, 2973,   34541,  6, }, // America/Phoenix
    {  602, 3945,    1805, 12, }, // America/Port-au-Prince
    {  602, 5954,   28189, 13, }, // America/Port_of_Spain
    {  602, 8404,   28202, 11, }, // America/Porto_Velho
    {  602, 5843,   51756, 11, }, // America/Puerto_Rico
    {  602, 4030,    1852, 12, }, // America/Punta_Arenas
    {  602, 8424,    1864, 12, }, // America/Rankin_Inlet
    {  602, 8445,    1876,  6, }, // America/Recife
    {  602, 1995,   34609,  6, }, // America/Regina
    {  602, 8460,   36760,  7, }, // America/Resolute
    {  602,  695,  137496, 10, }, // America/Rio_Branco
    {  602, 8477,   34632,  8, }, // America/Santarem
    {  602, 2201,   51775,  8, }, // America/Santiago
    {  602, 6111,    1923, 13, }, // America/Santo_Domingo
    {  602, 1878,    1936,  9, }, // America/Sao_Paulo
    {  602, 6899,  118416, 16, }, // America/Scoresbysund
    {  602, 8494,    1956,  5, }, // America/Sitka
    {  602, 8508,  186073, 15, }, // America/St_Barthelemy
    {  602, 2061,   28349, 10, }, // America/St_Johns
    {  602, 5863,    1983,  9, }, // America/St_Kitts
    {  602, 5880,  186088, 10, }, // America/St_Lucia
    {  602,  855,    2003, 10, }, // America/St_Thomas
    {  602, 5913,    2013, 12, }, // America/St_Vincent
    {  602, 5461,   51817, 12, }, // America/Tegucigalpa
    {  602, 5756,    2049,  4, }, // America/Thule
    {  602,  313,    2053,  7, }, // America/Tijuana
    {  602,  608,    2060,  7, }, // America/Toronto
    {  602, 5688,    2067,  7, }, // America/Tortola
    {  602, 2093,    2074,  8, }, // America/Vancouver
    {  602, 2144,   28435,  8, }, // America/Whitehorse
    {  602,  734,    2090,  8, }, // America/Winnipeg
    {  602, 8552,   28450,  6, }, // America/Yakutat
    {  602, 6001,   36889,  5, }, // Antarctica/Casey
    {  602, 6094,  118487,  5, }, // Antarctica/Davis
    {  602, 6133,  137562, 14, }, // Antarctica/DumontDUrville
    {  602, 8568,    2131,  8, }, // Antarctica/Macquarie
    {  602, 7066,    2139,  6, }, // Antarctica/Mawson
    {  602, 7113,  118505,  8, }, // Antarctica/McMurdo
    {  602, 8589,    2154,  6, }, // Antarctica/Palmer
    {  602, 7226,    2160,  6, }, // Antarctica/Rothera
    {  602, 7295,   28510,  4, }, // Antarctica/Syowa
    {  602, 8607,   51843,  6, }, // Antarctica/Troll
    {  602, 7374,   34825,  6, }, // Antarctica/Vostok
    {  602, 1429,  186098, 11, }, // Arctic/Longyearbyen
    {  602, 5570,    2192,  4, }, // Asia/Aden
    {  602, 5395,   51856,  8, }, // Asia/Almaty
    {  602, 3968,  186109,  5, }, // Asia/Amman
    {  602, 5511,   51870,  7, }, // Asia/Anadyr
    {  602, 5523,    2213,  5, }, // Asia/Aqtau
    {  602, 5368,    2218,  6, }, // Asia/Aqtobe
    {  602,  964,    2224,  7, }, // Asia/Ashgabat
    {  602, 8624,    2231,  6, }, // Asia/Atyrau
    {  602, 3607,    2237,  6, }, // Asia/Baghdad
    {  602, 5534,   36948,  7, }, // Asia/Bahrain
    {  602, 3653,    2250,  4, }, // Asia/Baku
    {  602, 4466,    2254,  7, }, // Asia/Bangkok
    {  602, 3571,    2261,  7, }, // Asia/Barnaul
    {  602, 4086,  186114,  6, }, // Asia/Beirut
    {  602, 3778,    2274,  6, }, // Asia/Bishkek
    {  602, 5989,   28575,  6, }, // Asia/Brunei
    {  602, 4601,    2286,  4, }, // Asia/Chita
    {  602, 4511,    2290,  7, }, // Asia/Colombo
    {  602, 4540,  186120,  7, }, // Asia/Damascus
    {  602, 1093,    2303,  5, }, // Asia/Dhaka
    {  602, 6192,    2308,  4, }, // Asia/Dili
    {  602, 3596,   28596,  5, }, // Asia/Dubai
    {  602, 6159,    2317,  7, }, // Asia/Dushanbe
    {  602, 8636,    2324,  9, }, // Asia/Famagusta
    {  602, 8651,    2333,  4, }, // Asia/Gaza
    {  602, 4795,  186127,  6, }, // Asia/Hebron
    {  602, 1259,   34892,  9, }, // Asia/Ho_Chi_Minh
    {  602, 2704,   51902,  7, }, // Asia/Hong_Kong
    {  602, 4771,    2357,  4, }, // Asia/Hovd
    {  602, 4184,    2361,  7, }, // Asia/Irkutsk
    {  602, 7006,  186133,  7, }, // Asia/Jakarta
    {  602, 6992,  186140,  7, }, // Asia/Jayapura
    {  602, 1290,    2383,  9, }, // Asia/Jerusalem
    {  602, 3560,    2392,  5, }, // Asia/Kabul
    {  602, 4316,    2411,  8, }, // Asia/Kamchatka
    {  602, 4239,    2422,  6, }, // Asia/Karachi
    {  602, 1185,    2428,  8, }, // Asia/Kathmandu
    {  602, 8661,    2436,  7, }, // Asia/Khandyga
    {  602,  992,   51950,  9, }, // Asia/Kolkata
    {  602, 4197,   36978, 10, }, // Asia/Krasnoyarsk
    {  602, 4859,    2462, 12, }, // Asia/Kuala_Lumpur
    {  602, 5976,    2474,  6, }, // Asia/Kuching
    {  602, 5547,   28693,  6, }, // Asia/Kuwait
    {  602, 1211,    2486,  5, }, // Asia/Macau
    {  602, 4017,    2491,  7, }, // Asia/Magadan
    {  602, 1349,  118555,  8, }, // Asia/Makassar
    {  602, 7182,    2505,  6, }, // Asia/Manila
    {  602, 6920,    2511,  6, }, // Asia/Muscat
    {  602, 2554,  106335,  7, }, // Asia/Nicosia
    {  602, 8675,    2524, 11, }, // Asia/Novokuznetsk
    {  602, 4135,    2535, 11, }, // Asia/Novosibirsk
    {  602, 4229,    2546,  4, }, // Asia/Omsk
    {  602, 7145,   35006,  4, }, // Asia/Oral
    {  602, 6961,   51959,  8, }, // Asia/Phnom_Penh
    {  602, 8693,    2563,  9, }, // Asia/Pontianak
    {  602, 4214,   28763,  7, }, // Asia/Pyongyang
    {  602, 5559,    2579,  5, }, // Asia/Qatar
    {  602, 8708,   37005,  8, }, // Asia/Qostanay
    {  602, 4269,    2592,  9, }, // Asia/Qyzylorda
    {  602, 3584,   51967,  7, }, // Asia/Riyadh
    {  602, 4408,    2614,  7, }, // Asia/Sakhalin
    {  602, 7245,  186147,  9, }, // Asia/Samarkand
    {  602, 3279,    2630,  4, }, // Asia/Seoul
    {  602, 1053,   28801,  6, }, // Asia/Shanghai
    {  602, 3300,    2640,  8, }, // Asia/Singapore
    {  602, 4297,    2648, 13, }, // Asia/Srednekolymsk
    {  602, 3263,   60896,  6, }, // Asia/Taipei
    {  602, 4781,    2667,  7, }, // Asia/Tashkent
    {  602, 3915,   35074,  7, }, // Asia/Tbilisi
    {  602, 2772,  186156,  6, }, // Asia/Tehran
    {  602, 1317,    2685,  7, }, // Asia/Thimphu
    {  602, 2821,    2692,  5, }, // Asia/Tokyo
    {  602, 4572,    2697,  5, }, // Asia/Tomsk
    {  602, 1021,    2702, 10, }, // Asia/Ulaanbaatar
    {  602, 1159,    2712,  6, }, // Asia/Urumqi
    {  602, 8722,   51993,  9, }, // Asia/Ust-Nera
    {  602, 6977,   52002,  8, }, // Asia/Vientiane
    {  602, 4710,    2733, 11, }, // Asia/Vladivostok
    {  602, 4828,   37051,  6, }, // Asia/Yakutsk
    {  602, 1235,   56409,  5, }, // Asia/Yangon
    {  602, 3883,    2757, 12, }, // Asia/Yekaterinburg
    {  602, 3747,    2769,  6, }, // Asia/Yerevan
    {  602, 3663,  186162, 14, }, // Atlantic/Azores
    {  602, 5671,  186176, 16, }, // Atlantic/Bermuda
    {  602, 6628,  186192, 15, }, // Atlantic/Canary
    {  602, 3727,    2826, 10, }, // Atlantic/Cape_Verde
    {  602, 1395,  186207, 15, }, // Atlantic/Faroe
    {  602, 8736,   37132,  7, }, // Atlantic/Madeira
    {  602, 2748,   52082,  9, }, // Atlantic/Reykjavik
    {  602, 7272,  186222, 15, }, // Atlantic/South_Georgia
    {  602, 6821,  186237, 10, }, // Atlantic/St_Helena
    {  602, 6644,   52124,  6, }, // Atlantic/Stanley
    {  602, 1670,    2917,  8, }, // Australia/Adelaide
    {  602, 1635,    2925,  7, }, // Australia/Brisbane
    {  602, 1799,    2932, 11, }, // Australia/Broken_Hill
    {  602, 1583,    2943,  6, }, // Australia/Darwin
    {  602, 3637,   29108,  4, }, // Australia/Eucla
    {  602, 1516,    2954,  6, }, // Australia/Hobart
    {  602, 8753,    2960,  8, }, // Australia/Lindeman
    {  602, 1547,   37190,  8, }, // Australia/Lord_Howe
    {  602, 1727,   29120,  8, }, // Australia/Melbourne
    {  602, 1762,    2983,  4, }, // Australia/Perth
    {  602, 1463,   52144,  6, }, // Australia/Sydney
    {  602, 6441,    2993,  9, }, // Europe/Amsterdam
    {  602, 6216,    3002,  7, }, // Europe/Andorra
    {  602, 3620,  186247,  8, }, // Europe/Astrakhan
    {  602, 2300,    3017,  4, }, // Europe/Athens
    {  602, 6336,    3021,  7, }, // Europe/Belgrade
    {  602, 4757,    3028,  6, }, // Europe/Berlin
    {  602, 6488,    3034, 10, }, // Europe/Bratislava
    {  602, 2167,   52164,  8, }, // Europe/Brussels
    {  602, 3928,    3052,  8, }, // Europe/Bucharest
    {  602, 3806,    3060,  8, }, // Europe/Budapest
    {  602, 8772,   37236,  8, }, // Europe/Busingen
    {  602, 2583,    3091,  7, }, // Europe/Chisinau
    {  602, 6289,   52191, 10, }, // Europe/Copenhagen
    {  602, 2338,    3108,  6, }, // Europe/Dublin
    {  602, 6307,    3114,  9, }, // Europe/Gibraltar
    {  602, 8788,    3123,  6, }, // Europe/Guernsey
    {  602, 6612,   52201,  9, }, // Europe/Helsinki
    {  602, 8804,  186255, 10, }, // Europe/Isle_of_Man
    {  602, 1130,   35352,  8, }, // Europe/Istanbul
    {  602, 8823,   52219,  6, }, // Europe/Jersey
    {  602, 3979,    3158, 11, }, // Europe/Kaliningrad
    {  602, 8929,    3169,  5, }, // Europe/Kirov
    {  602, 2527,    3174,  4, }, // Europe/Kyiv
    {  602, 3213,    3178,  8, }, // Europe/Lisbon
    {  602, 6506,   29300,  7, }, // Europe/Ljubljana
    {  602, 2501,    3195,  6, }, // Europe/London
    {  602, 6365,   29313, 10, }, // Europe/Luxembourg
    {  602, 4892,    3212,  6, }, // Europe/Madrid
    {  602, 6397,   29329,  6, }, // Europe/Malta
    {  602, 6582,    3223,  9, }, // Europe/Mariehamn
    {  602, 3693,    3232,  5, }, // Europe/Minsk
    {  602, 6410,    3237,  6, }, // Europe/Monaco
    {  602, 3537,  186265,  6, }, // Europe/Moscow
    {  602, 6458,    3249,  4, }, // Europe/Oslo
    {  602, 4284,    3253,  5, }, // Europe/Paris
    {  602, 6424,    3258,  9, }, // Europe/Podgorica
    {  602, 6275,    3267,  5, }, // Europe/Prague
    {  602, 8837,    3272,  4, }, // Europe/Riga
    {  602, 6324,    3276,  3, }, // Europe/Rome
    {  602, 4331,    3279,  6, }, // Europe/Samara
    {  602, 6470,    3285, 10, }, // Europe/San_Marino
    {  602, 6245,    3295,  7, }, // Europe/Sarajevo
    {  602, 4451,    3302,  7, }, // Europe/Saratov
    {  602, 8849,   52225, 11, }, // Europe/Simferopol
    {  602, 6383,   52236,  6, }, // Europe/Skopje
    {  602, 6599,   37286,  5, }, // Europe/Sofia
    {  602, 6523,   52242,  9, }, // Europe/Stockholm
    {  602, 8867,    3337,  6, }, // Europe/Tallinn
    {  602, 6202,    3343,  5, }, // Europe/Tirane
    {  602, 8882,   52251,  9, }, // Europe/Ulyanovsk
    {  602, 6352,    3358,  5, }, // Europe/Vaduz
    {  602, 6567,    3363,  7, }, // Europe/Vatican
    {  602, 6231,    3370,  4, }, // Europe/Vienna
    {  602, 8899,   52260,  7, }, // Europe/Vilnius
    {  602, 4727,    3381,  9, }, // Europe/Volgograd
    {  602, 3190,    3390,  7, }, // Europe/Warsaw
    {  602, 6261,    3397,  6, }, // Europe/Zagreb
    {  602, 6540,   37310,  5, }, // Europe/Zurich
    {  602, 5078,    3409, 12, }, // Indian/Antananarivo
    {  602, 6947,    3421,  5, }, // Indian/Chagos
    {  602, 6046,  186271,  6, }, // Indian/Christmas
    {  602, 6063,    3438,  5, }, // Indian/Cocos
    {  602, 5029,   35510,  6, }, // Indian/Comoro
    {  602, 6661,    3460,  8, }, // Indian/Kerguelen
    {  602, 7260,   29562,  3, }, // Indian/Mahe
    {  602, 7050,  186277, 17, }, // Indian/Maldives
    {  602, 4069,   56576,  8, }, // Indian/Mauritius
    {  602, 5098,   56584,  7, }, // Indian/Mayotte
    {  602, 7211,   52348,  7, }, // Indian/Reunion
    {  602, 4422,    3498,  4, }, // Pacific/Apia
    {  602,  932,    3502,  6, }, // Pacific/Auckland
    {  602, 3706,   52355,  9, }, // Pacific/Bougainville
    {  602, 3015,    3516,  5, }, // Pacific/Chatham
    {  602, 3157,  118669,  4, }, // Pacific/Chuuk
    {  602, 2237,  186294, 12, }, // Pacific/Easter
    {  602, 7360,   52409,  5, }, // Pacific/Efate
    {  602, 7327,    3541,  7, }, // Pacific/Fakaofo
    {  602, 3902,   37407,  5, }, // Pacific/Fiji
    {  602, 7343,    3552,  8, }, // Pacific/Funafuti
    {  602, 6678,  186306, 19, }, // Pacific/Galapagos
    {  602, 6696,   52435,  6, }, // Pacific/Gambier
    {  602, 3822,    3595, 11, }, // Pacific/Guadalcanal
    {  602, 6018,    3606,  4, }, // Pacific/Guam
    {  602, 3049,    3610,  6, }, // Pacific/Kanton
    {  602, 3998,    3616, 10, }, // Pacific/Kiritimati
    {  602, 7019,  186325,  6, }, // Pacific/Kosrae
    {  602, 2842,   29709, 10, }, // Pacific/Kwajalein
    {  602, 4877,   37442,  7, }, // Pacific/Majuro
    {  602, 4051,  186331, 16, }, // Pacific/Marquesas
    {  602, 8914,   52490,  6, }, // Pacific/Midway
    {  602, 7084,    3680,  5, }, // Pacific/Nauru
    {  602, 7132,   52504,  4, }, // Pacific/Niue
    {  602, 4168,    3689,  7, }, // Pacific/Norfolk
    {  602, 7098,    3696,  5, }, // Pacific/Noumea
    {  602, 3126,   35642,  9, }, // Pacific/Pago_Pago
    {  602, 7155,    3710,  5, }, // Pacific/Palau
    {  602, 7194,   56657,  7, }, // Pacific/Pitcairn
    {  602, 3096,    3722,  6, }, // Pacific/Pohnpei
    {  602, 4807,    3728, 11, }, // Pacific/Port_Moresby
    {  602, 6076,    3739,  9, }, // Pacific/Rarotonga
    {  602, 6031,   29803,  6, }, // Pacific/Saipan
    {  602, 7312,    3754,  5, }, // Pacific/Tahiti
    {  602, 6712,    3759,  6, }, // Pacific/Tarawa
    {  602, 4583,    3765,  9, }, // Pacific/Tongatapu
    {  602, 7392,  181934,  4, }, // Pacific/Wake
    {  602, 7405,    3778,  6, }, // Pacific/Wallis
    {  603, 6788,  186347,  9, }, // Africa/Abidjan Telugu/Telugu/India
    {  603, 6760,  186356,  5, }, // Africa/Accra
    {  603, 5059,  186361, 12, }, // Africa/Addis_Ababa
    {  603, 7420,  186373, 10, }, // Africa/Algiers
    {  603,   14,  186383,  7, }, // Africa/Asmara
    {  603,   44,  186390,  6, }, // Africa/Bamako
    {  603, 5257,  186396,  7, }, // Africa/Bangui
    {  603, 6746,  186403,  6, }, // Africa/Banjul
    {  603, 7435,  186409,  8, }, // Africa/Bissau
    {  603, 4957,  186417, 10, }, // Africa/Blantyre
    {  603, 5287,  186427, 12, }, // Africa/Brazzaville
    {  603, 4922,  186439,  9, }, // Africa/Bujumbura
    {  603, 2320,  186448,  4, }, // Africa/Cairo
    {  603, 4117,  186452, 11, }, // Africa/Casablanca
    {  603, 7449,  186463,  6, }, // Africa/Ceuta
    {  603, 6773,  186469,  8, }, // Africa/Conakry
    {  603, 6840,  186477,  5, }, // Africa/Dakar
    {  603, 5130,  186482, 14, }, // Africa/Dar_es_Salaam
    {  603, 5043,  186496,  8, }, // Africa/Djibouti
    {  603, 5243,  186504,  4, }, // Africa/Douala
    {  603, 5166,  186508,  9, }, // Africa/El_Aaiun
    {  603, 6853,  186517,  8, }, // Africa/Freetown
    {  603, 4906,  186525,  7, }, // Africa/Gaborone
    {  603, 5015,  186532,  5, }, // Africa/Harare
    {  603, 4479,  186537, 14, }, // Africa/Johannesburg
    {  603, 4499,  186551,  4, }, // Africa/Juba
    {  603, 5151,  186555,  6, }, // Africa/Kampala
    {  603, 4524,  186561,  8, }, // Africa/Khartoum
    {  603, 4973,  186569,  5, }, // Africa/Kigali
    {  603, 5306,  186574,  8, }, // Africa/Kinshasa
    {  603, 4744,  186582,  6, }, // Africa/Lagos
    {  603, 5336,  186588, 11, }, // Africa/Libreville
    {  603, 6869,  186599,  4, }, // Africa/Lome
    {  603, 5211,  186603,  7, }, // Africa/Luanda
    {  603, 4939,  186610,  8, }, // Africa/Lubumbashi
    {  603, 5001,  186618,  6, }, // Africa/Lusaka
    {  603, 5322,  186624,  5, }, // Africa/Malabo
    {  603, 4987,  186629,  6, }, // Africa/Maputo
    {  603, 5197,  186635,  5, }, // Africa/Maseru
    {  603, 5182,  186640,  6, }, // Africa/Mbabane
    {  603, 5113,  186646,  8, }, // Africa/Mogadishu
    {  603, 7034,  186654, 10, }, // Africa/Monrovia
    {  603, 3853,  186664,  6, }, // Africa/Nairobi
    {  603, 5271,  186670,  8, }, // Africa/Ndjamena
    {  603, 5354,  186678,  6, }, // Africa/Niamey
    {  603, 6803,  186684, 10, }, // Africa/Nouakchott
    {  603, 6727,  186694,  9, }, // Africa/Ouagadougou
    {  603, 5225,  186703, 11, }, // Africa/Porto-Novo
    {  603, 4435,  186714,  9, }, // Africa/Sao_Tome
    {  603, 2866,  186723,  8, }, // Africa/Tripoli
    {  603, 6554,  186731,  8, }, // Africa/Tunis
    {  603, 4152,  186739,  9, }, // Africa/Windhoek
    {  603,  132,  186748,  5, }, // America/Adak
    {  603, 3346,  186753,  8, }, // America/Anchorage
    {  603, 5607,  186761, 10, }, // America/Anguilla
    {  603, 5624,  186771,  8, }, // America/Antigua
    {  603, 4554,  186779, 11, }, // America/Araguaina
    {  603,  166,  186790, 16, }, // America/Argentina/Buenos_Aires
    {  603,   91,  186806,  8, }, // America/Argentina/Catamarca
    {  603,  270,  186814,  8, }, // America/Argentina/Cordoba
    {  603,  441,  186822,  6, }, // America/Argentina/Jujuy
    {  603, 7462,  186828,  8, }, // America/Argentina/La_Rioja
    {  603,  565,  186836,  7, }, // America/Argentina/Mendoza
    {  603, 7489,  186843, 14, }, // America/Argentina/Rio_Gallegos
    {  603, 7520,  186857,  6, }, // America/Argentina/Salta
    {  603, 7544,  186863, 11, }, // America/Argentina/San_Juan
    {  603, 5580,  186874, 11, }, // America/Argentina/San_Luis
    {  603, 7571,  186885,  7, }, // America/Argentina/Tucuman
    {  603, 7597,  186892,  6, }, // America/Argentina/Ushuaia
    {  603, 5640,  186898,  5, }, // America/Aruba
    {  603, 4252,  186903, 11, }, // America/Asuncion
    {  603,  237,  186914,  8, }, // America/Atikokan
    {  603, 3679,  186922,  4, }, // America/Bahia
    {  603, 7623,  186926, 13, }, // America/Bahia_Banderas
    {  603, 5654,  186939,  9, }, // America/Barbados
    {  603, 7646,  186948,  6, }, // America/Belem
    {  603, 5407,  186954,  6, }, // America/Belize
    {  603, 7660,  186960, 15, }, // America/Blanc-Sablon
    {  603, 7681,  186975, 11, }, // America/Boa_Vista
    {  603, 4361,  186986,  5, }, // America/Bogota
    {  603, 7699,  186991,  6, }, // America/Boise
    {  603, 7713,  186997, 14, }, // America/Cambridge_Bay
    {  603, 7735,  187011, 13, }, // America/Campo_Grande
    {  603, 3868,  187024,  8, }, // America/Cancun
    {  603, 4694,  187032,  7, }, // America/Caracas
    {  603, 4345,  187039,  5, }, // America/Cayenne
    {  603, 5496,  187044,  6, }, // America/Cayman
    {  603, 2260,  187050,  6, }, // America/Chicago
    {  603, 7756,  187056,  6, }, // America/Chihuahua
    {  603, 7774,  187062, 17, }, // America/Ciudad_Juarez
    {  603, 5422,  187079, 11, }, // America/Costa_Rica
    {  603, 7796,  187090,  9, }, // America/Creston
    {  603, 3791,  187099,  5, }, // America/Cuiaba
    {  603, 5723,  187104,  7, }, // America/Curacao
    {  603, 7812,  187111, 15, }, // America/Danmarkshavn
    {  603, 7833,  187126,  5, }, // America/Dawson
    {  603, 7848,  187131, 12, }, // America/Dawson_Creek
    {  603,  805,  187143,  8, }, // America/Denver
    {  603, 3465,  187151, 10, }, // America/Detroit
    {  603, 5739,  187161,  8, }, // America/Dominica
    {  603,  893,  187169,  9, }, // America/Edmonton
    {  603, 7869,  187178,  7, }, // America/Eirunepe
    {  603, 5441,  187185, 13, }, // America/El_Salvador
    {  603, 7886,  187198, 14, }, // America/Fort_Nelson
    {  603, 7906,  187212,  9, }, // America/Fortaleza
    {  603, 7924,  187221,  9, }, // America/Glace_Bay
    {  603, 6881,  187230,  7, }, // America/Goose_Bay
    {  603, 4612,  187237, 13, }, // America/Grand_Turk
    {  603, 5770,  187250,  7, }, // America/Grenada
    {  603, 5786,  187257, 10, }, // America/Guadeloupe
    {  603, 3760,  187267, 10, }, // America/Guatemala
    {  603, 4841,  187277,  9, }, // America/Guayaquil
    {  603, 6932,  187286,  5, }, // America/Guyana
    {  603, 1939,  187291, 10, }, // America/Halifax
    {  603, 2281,  187301,  5, }, // America/Havana
    {  603, 7942,  187306, 12, }, // America/Hermosillo
    {  603,  348,  187318, 13, }, // America/Indiana/Indianapolis
    {  603,  481,  187331, 15, }, // America/Indiana/Knox
    {  603, 7961,  187346, 15, }, // America/Indiana/Marengo
    {  603, 7985,  187361, 22, }, // America/Indiana/Petersburg
    {  603, 8012,  187383, 18, }, // America/Indiana/Tell_City
    {  603, 8038,  187401, 14, }, // America/Indiana/Vevay
    {  603, 8060,  187415, 22, }, // America/Indiana/Vincennes
    {  603, 8086,  187437, 17, }, // America/Indiana/Winamac
    {  603, 8110,  187454,  7, }, // America/Inuvik
    {  603,  660,  187461,  9, }, // America/Iqaluit
    {  603, 2799,  187470,  5, }, // America/Jamaica
    {  603, 5380,  187475,  4, }, // America/Juneau
    {  603,  521,  187479,  8, }, // America/Kentucky/Louisville
    {  603, 8125,  187487, 24, }, // America/Kentucky/Monticello
    {  603, 5704,  187511, 12, }, // America/Kralendijk
    {  603, 4376,  187523,  7, }, // America/La_Paz
    {  603, 7169,  187530,  4, }, // America/Lima
    {  603, 3239,  187534, 12, }, // America/Los_Angeles
    {  603, 5932,  187546, 24, }, // America/Lower_Princes
    {  603, 8153,  187570,  6, }, // America/Maceio
    {  603, 8168,  187576,  7, }, // America/Managua
    {  603, 1908,  187583,  5, }, // America/Manaus
    {  603, 5897,  187588,  8, }, // America/Marigot
    {  603, 5805,  187596, 10, }, // America/Martinique
    {  603, 8184,  187606,  9, }, // America/Matamoros
    {  603, 2917,  187615, 10, }, // America/Mazatlan
    {  603, 8202,  187625,  8, }, // America/Menominee
    {  603, 8220,  187633,  6, }, // America/Merida
    {  603, 8235,  187639,  9, }, // America/Metlakatla
    {  603, 2949,  187648, 13, }, // America/Mexico_City
    {  603, 4391,  187661,  8, }, // America/Miquelon
    {  603, 8254,  187669, 11, }, // America/Moncton
    {  603, 8270,  187680,  9, }, // America/Monterrey
    {  603, 4098,  187689, 11, }, // America/Montevideo
    {  603, 5824,  187700, 11, }, // America/Montserrat
    {  603, 5481,  187711,  8, }, // America/Nassau
    {  603, 2379,  187719, 10, }, // America/New_York
    {  603, 8288,  187729,  4, }, // America/Nome
    {  603, 1850,  187733,  7, }, // America/Noronha
    {  603, 8301,  187740, 18, }, // America/North_Dakota/Beulah
    {  603, 8329,  187758, 19, }, // America/North_Dakota/Center
    {  603, 8357,  187777, 22, }, // America/North_Dakota/New_Salem
    {  603,  393,  187799,  4, }, // America/Nuuk
    {  603, 8388,  187803,  5, }, // America/Ojinaga
    {  603, 2356,  187808,  5, }, // America/Panama
    {  603, 6173,  187813,  9, }, // America/Paramaribo
    {  603, 2973,  187822,  8, }, // America/Phoenix
    {  603, 3945,  187830, 19, }, // America/Port-au-Prince
    {  603, 5954,  187849, 19, }, // America/Port_of_Spain
    {  603, 8404,  187868, 13, }, // America/Porto_Velho
    {  603, 5843,  187881, 13, }, // America/Puerto_Rico
    {  603, 4030,  187894, 12, }, // America/Punta_Arenas
    {  603, 8424,  187906, 17, }, // America/Rankin_Inlet
    {  603, 8445,  187923,  6, }, // America/Recife
    {  603, 1995,  187929,  6, }, // America/Regina
    {  603, 8460,  187935,  9, }, // America/Resolute
    {  603,  695,  187944, 12, }, // America/Rio_Branco
    {  603, 8477,  187956,  8, }, // America/Santarem
    {  603, 2201,  187964,  9, }, // America/Santiago
    {  603, 6111,  187973, 13, }, // America/Santo_Domingo
    {  603, 1878,  187986,  9, }, // America/Sao_Paulo
    {  603, 6899,  187995, 17, }, // America/Scoresbysund
    {  603, 8494,  188012,  6, }, // America/Sitka
    {  603, 8508,  188018, 17, }, // America/St_Barthelemy
    {  603, 2061,  188035, 14, }, // America/St_Johns
    {  603, 5863,  188049, 14, }, // America/St_Kitts
    {  603, 5880,  188063, 14, }, // America/St_Lucia
    {  603,  855,  188077, 13, }, // America/St_Thomas
    {  603, 5913,  188090, 17, }, // America/St_Vincent
    {  603, 8530,  188107, 15, }, // America/Swift_Current
    {  603, 5461,  188122, 11, }, // America/Tegucigalpa
    {  603, 5756,  188133,  4, }, // America/Thule
    {  603,  313,  188137,  8, }, // America/Tijuana
    {  603,  608,  188145,  6, }, // America/Toronto
    {  603, 5688,  188151,  8, }, // America/Tortola
    {  603, 2093,  188159,  9, }, // America/Vancouver
    {  603, 2144,  188168, 11, }, // America/Whitehorse
    {  603,  734,  188179, 10, }, // America/Winnipeg
    {  603, 8552,  188189,  7, }, // America/Yakutat
    {  603, 6001,  188196,  4, }, // Antarctica/Casey
    {  603, 6094,  188200,  6, }, // Antarctica/Davis
    {  603, 6133,  188206, 22, }, // Antarctica/DumontDUrville
    {  603, 8568,  188228,  5, }, // Antarctica/Macquarie
    {  603, 7066,  188233,  5, }, // Antarctica/Mawson
    {  603, 7113,  188238, 11, }, // Antarctica/McMurdo
    {  603, 8589,  188249,  7, }, // Antarctica/Palmer
    {  603, 7226,  188256,  6, }, // Antarctica/Rothera
    {  603, 7295,  188262,  6, }, // Antarctica/Syowa
    {  603, 8607,  188268,  6, }, // Antarctica/Troll
    {  603, 7374,  188274,  8, }, // Antarctica/Vostok
    {  603, 1429,  188282, 17, }, // Arctic/Longyearbyen
    {  603, 5570,  188299,  5, }, // Asia/Aden
    {  603, 5395,  188304,  7, }, // Asia/Almaty
    {  603, 3968,  188311,  6, }, // Asia/Amman
    {  603, 5511,  188317,  6, }, // Asia/Anadyr
    {  603, 5523,  188323,  7, }, // Asia/Aqtau
    {  603, 5368,  188330,  7, }, // Asia/Aqtobe
    {  603,  964,  188337, 10, }, // Asia/Ashgabat
    {  603, 8624,  188347,  5, }, // Asia/Atyrau
    {  603, 3607,  188352,  8, }, // Asia/Baghdad
    {  603, 5534,  188360,  9, }, // Asia/Bahrain
    {  603, 3653,  187102,  4, }, // Asia/Baku
    {  603, 4466,  188369,  9, }, // Asia/Bangkok
    {  603, 3571,  188378,  8, }, // Asia/Barnaul
    {  603, 4086,  188386,  5, }, // Asia/Beirut
    {  603, 3778,  188391,  8, }, // Asia/Bishkek
    {  603, 5989,  188399,  6, }, // Asia/Brunei
    {  603, 4601,  188405,  4, }, // Asia/Chita
    {  603, 4511,  188409,  6, }, // Asia/Colombo
    {  603, 4540,  188415,  8, }, // Asia/Damascus
    {  603, 1093,  188423,  4, }, // Asia/Dhaka
    {  603, 6192,  188427,  4, }, // Asia/Dili
    {  603, 3596,  188431,  6, }, // Asia/Dubai
    {  603, 6159,  188437,  7, }, // Asia/Dushanbe
    {  603, 8636,  188444,  9, }, // Asia/Famagusta
    {  603, 8651,  188453,  4, }, // Asia/Gaza
    {  603, 4795,  188457,  8, }, // Asia/Hebron
    {  603, 1259,  188465, 17, }, // Asia/Ho_Chi_Minh
    {  603, 2704,  188482,  8, }, // Asia/Hong_Kong
    {  603, 4771,  188490,  6, }, // Asia/Hovd
    {  603, 4184,  188496, 11, }, // Asia/Irkutsk
    {  603, 7006,  188507,  7, }, // Asia/Jakarta
    {  603, 6992,  188514,  5, }, // Asia/Jayapura
    {  603, 1290,  188519,  7, }, // Asia/Jerusalem
    {  603, 3560,  188526,  6, }, // Asia/Kabul
    {  603, 4316,  188532,  9, }, // Asia/Kamchatka
    {  603, 4239,  188541,  5, }, // Asia/Karachi
    {  603, 1185,  188546,  8, }, // Asia/Kathmandu
    {  603, 8661,  188554,  7, }, // Asia/Khandyga
    {  603,  992,  188561,  8, }, // Asia/Kolkata
    {  603, 4197,  188569, 14, }, // Asia/Krasnoyarsk
    {  603, 4859,  188583, 10, }, // Asia/Kuala_Lumpur
    {  603, 5976,  188593,  7, }, // Asia/Kuching
    {  603, 5547,  188600,  6, }, // Asia/Kuwait
    {  603, 1211,  188606,  5, }, // Asia/Macau
    {  603, 4017,  188611,  6, }, // Asia/Magadan
    {  603, 1349,  188617,  8, }, // Asia/Makassar
    {  603, 7182,  188625,  5, }, // Asia/Manila
    {  603, 6920,  188630,  6, }, // Asia/Muscat
    {  603, 2554,  188636,  8, }, // Asia/Nicosia
    {  603, 8675,  188644, 18, }, // Asia/Novokuznetsk
    {  603, 4135,  188662, 13, }, // Asia/Novosibirsk
    {  603, 4229,  188675,  7, }, // Asia/Omsk
    {  603, 7145,  188682,  4, }, // Asia/Oral
    {  603, 6961,  188686, 11, }, // Asia/Phnom_Penh
    {  603, 8693,  188697, 11, }, // Asia/Pontianak
    {  603, 4214,  188708, 10, }, // Asia/Pyongyang
    {  603, 5559,  188718,  5, }, // Asia/Qatar
    {  603, 8708,  188723,  8, }, // Asia/Qostanay
    {  603, 4269,  188731, 12, }, // Asia/Qyzylorda
    {  603, 3584,  188743,  6, }, // Asia/Riyadh
    {  603, 4408,  188749,  7, }, // Asia/Sakhalin
    {  603, 7245,  188756,  9, }, // Asia/Samarkand
    {  603, 3279,  188765,  6, }, // Asia/Seoul
    {  603, 1053,  188771,  5, }, // Asia/Shanghai
    {  603, 3300,  188776,  8, }, // Asia/Singapore
    {  603, 4297,  188784, 18, }, // Asia/Srednekolymsk
    {  603, 3263,  188802,  4, }, // Asia/Taipei
    {  603, 4781,  188806,  9, }, // Asia/Tashkent
    {  603, 3915,  188815,  8, }, // Asia/Tbilisi
    {  603, 2772,  188823,  8, }, // Asia/Tehran
    {  603, 1317,  188831,  5, }, // Asia/Thimphu
    {  603, 2821,  188836,  6, }, // Asia/Tokyo
    {  603, 4572,  188842,  8, }, // Asia/Tomsk
    {  603, 1021,  188850, 10, }, // Asia/Ulaanbaatar
    {  603, 1159,  188860,  8, }, // Asia/Urumqi
    {  603, 8722,  188868, 10, }, // Asia/Ust-Nera
    {  603, 6977,  188878,  9, }, // Asia/Vientiane
    {  603, 4710,  188887, 14, }, // Asia/Vladivostok
    {  603, 4828,  188901,  9, }, // Asia/Yakutsk
    {  603, 1235,  188910,  6, }, // Asia/Yangon
    {  603, 3883,  188916, 13, }, // Asia/Yekaterinburg
    {  603, 3747,  188929,  7, }, // Asia/Yerevan
    {  603, 3663,  188936,  7, }, // Atlantic/Azores
    {  603, 5671,  188943,  8, }, // Atlantic/Bermuda
    {  603, 6628,  188951,  5, }, // Atlantic/Canary
    {  603, 3727,  188956, 11, }, // Atlantic/Cape_Verde
    {  603, 1395,  188967,  4, }, // Atlantic/Faroe
    {  603, 8736,  188971,  5, }, // Atlantic/Madeira
    {  603, 2748,  188976, 10, }, // Atlantic/Reykjavik
    {  603, 7272,  188986, 15, }, // Atlantic/South_Georgia
    {  603, 6821,  189001, 14, }, // Atlantic/St_Helena
    {  603, 6644,  189015,  8, }, // Atlantic/Stanley
    {  603, 1670,  189023,  7, }, // Australia/Adelaide
    {  603, 1635,  189030, 13, }, // Australia/Brisbane
    {  603, 1799,  189043, 13, }, // Australia/Broken_Hill
    {  603, 1583,  189056,  8, }, // Australia/Darwin
    {  603, 3637,  189064,  6, }, // Australia/Eucla
    {  603, 1516,  189070,  7, }, // Australia/Hobart
    {  603, 8753,  189077,  9, }, // Australia/Lindeman
    {  603, 1547,  189086,  9, }, // Australia/Lord_Howe
    {  603, 1727,  189095, 10, }, // Australia/Melbourne
    {  603, 1762,  189105,  6, }, // Australia/Perth
    {  603, 1463,  189111,  6, }, // Australia/Sydney
    {  603, 6441,  189117, 14, }, // Europe/Amsterdam
    {  603, 6216,  189131,  6, }, // Europe/Andorra
    {  603, 3620,  189137, 10, }, // Europe/Astrakhan
    {  603, 2300,  189147,  7, }, // Europe/Athens
    {  603, 6336,  189154, 11, }, // Europe/Belgrade
    {  603, 4757,  189165,  8, }, // Europe/Berlin
    {  603, 6488,  189173, 12, }, // Europe/Bratislava
    {  603, 2167,  189185, 11, }, // Europe/Brussels
    {  603, 3928,  189196, 10, }, // Europe/Bucharest
    {  603, 3806,  189206, 10, }, // Europe/Budapest
    {  603, 8772,  189216,  7, }, // Europe/Busingen
    {  603, 2583,  189223,  8, }, // Europe/Chisinau
    {  603, 6289,  189231, 11, }, // Europe/Copenhagen
    {  603, 2338,  189242,  7, }, // Europe/Dublin
    {  603, 6307,  189249, 11, }, // Europe/Gibraltar
    {  603, 8788,  189260, 10, }, // Europe/Guernsey
    {  603, 6612,  189270,  9, }, // Europe/Helsinki
    {  603, 8804,  189279, 12, }, // Europe/Isle_of_Man
    {  603, 1130,  189291, 10, }, // Europe/Istanbul
    {  603, 8823,  189301,  6, }, // Europe/Jersey
    {  603, 3979,  189307, 13, }, // Europe/Kaliningrad
    {  603, 8929,  189320,  6, }, // Europe/Kirov
    {  603, 2527,  189326,  4, }, // Europe/Kyiv
    {  603, 3213,  189330,  7, }, // Europe/Lisbon
    {  603, 6506,  189337, 12, }, // Europe/Ljubljana
    {  603, 2501,  189349,  5, }, // Europe/London
    {  603, 6365,  189354, 11, }, // Europe/Luxembourg
    {  603, 4892,  189365,  8, }, // Europe/Madrid
    {  603, 6397,  189373,  6, }, // Europe/Malta
    {  603, 6582,  189379,  9, }, // Europe/Mariehamn
    {  603, 3693,  189388,  8, }, // Europe/Minsk
    {  603, 6410,  189396,  6, }, // Europe/Monaco
    {  603, 3537,  189402,  6, }, // Europe/Moscow
    {  603, 6458,  189408,  5, }, // Europe/Oslo
    {  603, 4284,  189413,  8, }, // Europe/Paris
    {  603, 6424,  189421, 10, }, // Europe/Podgorica
    {  603, 6275,  189431,  6, }, // Europe/Prague
    {  603, 8837,  189437,  4, }, // Europe/Riga
    {  603, 6324,  189441,  4, }, // Europe/Rome
    {  603, 4331,  189445,  4, }, // Europe/Samara
    {  603, 6470,  189449, 11, }, // Europe/San_Marino
    {  603, 6245,  189460,  7, }, // Europe/Sarajevo
    {  603, 4451,  189467,  6, }, // Europe/Saratov
    {  603, 8849,  189473, 13, }, // Europe/Simferopol
    {  603, 6383,  189486,  9, }, // Europe/Skopje
    {  603, 6599,  189495,  6, }, // Europe/Sofia
    {  603, 6523,  189501, 10, }, // Europe/Stockholm
    {  603, 8867,  189511,  8, }, // Europe/Tallinn
    {  603, 6202,  189519,  6, }, // Europe/Tirane
    {  603, 8882,  189525, 12, }, // Europe/Ulyanovsk
    {  603, 6352,  189537,  6, }, // Europe/Vaduz
    {  603, 6567,  189543,  7, }, // Europe/Vatican
    {  603, 6231,  189550,  7, }, // Europe/Vienna
    {  603, 8899,  189557,  9, }, // Europe/Vilnius
    {  603, 4727,  189566, 12, }, // Europe/Volgograd
    {  603, 3190,  189578,  6, }, // Europe/Warsaw
    {  603, 6261,  189584,  8, }, // Europe/Zagreb
    {  603, 6540,  189592,  8, }, // Europe/Zurich
    {  603, 5078,  189600, 12, }, // Indian/Antananarivo
    {  603, 6947,  189612,  6, }, // Indian/Chagos
    {  603, 6046,  189618,  9, }, // Indian/Christmas
    {  603, 6063,  189627,  6, }, // Indian/Cocos
    {  603, 5029,  189633,  6, }, // Indian/Comoro
    {  603, 6661,  189639, 12, }, // Indian/Kerguelen
    {  603, 7260,  189651,  4, }, // Indian/Mahe
    {  603, 7050,  189655, 10, }, // Indian/Maldives
    {  603, 4069,  189665,  7, }, // Indian/Mauritius
    {  603, 5098,  189672,  5, }, // Indian/Mayotte
    {  603, 7211,  189677,  9, }, // Indian/Reunion
    {  603, 4422,  189686,  5, }, // Pacific/Apia
    {  603,  932,  189691,  8, }, // Pacific/Auckland
    {  603, 3706,  189699, 15, }, // Pacific/Bougainville
    {  603, 3015,  189714,  5, }, // Pacific/Chatham
    {  603, 3157,  189719,  4, }, // Pacific/Chuuk
    {  603, 2237,  189723,  6, }, // Pacific/Easter
    {  603, 7360,  189729,  5, }, // Pacific/Efate
    {  603, 7327,  189734,  6, }, // Pacific/Fakaofo
    {  603, 3902,  189740,  4, }, // Pacific/Fiji
    {  603, 7343,  189744,  8, }, // Pacific/Funafuti
    {  603, 6678,  189752, 10, }, // Pacific/Galapagos
    {  603, 6696,  189762,  9, }, // Pacific/Gambier
    {  603, 3822,  189771, 13, }, // Pacific/Guadalcanal
    {  603, 6018,  189784,  6, }, // Pacific/Guam
    {  603, 2723,  189790,  8, }, // Pacific/Honolulu
    {  603, 3049,  189798,  9, }, // Pacific/Kanton
    {  603, 3998,  189807, 10, }, // Pacific/Kiritimati
    {  603, 7019,  189817,  6, }, // Pacific/Kosrae
    {  603, 2842,  189823, 10, }, // Pacific/Kwajalein
    {  603, 4877,  189833,  5, }, // Pacific/Majuro
    {  603, 4051,  189838, 12, }, // Pacific/Marquesas
    {  603, 8914,  189850,  6, }, // Pacific/Midway
    {  603, 7084,  189856,  4, }, // Pacific/Nauru
    {  603, 7132,  189860,  4, }, // Pacific/Niue
    {  603, 4168,  189864,  8, }, // Pacific/Norfolk
    {  603, 7098,  189872,  6, }, // Pacific/Noumea
    {  603, 3126,  189878,  9, }, // Pacific/Pago_Pago
    {  603, 7155,  189887,  6, }, // Pacific/Palau
    {  603, 7194,  189893, 11, }, // Pacific/Pitcairn
    {  603, 3096,  189904,  7, }, // Pacific/Pohnpei
    {  603, 4807,  189911, 15, }, // Pacific/Port_Moresby
    {  603, 6076,  189926,  8, }, // Pacific/Rarotonga
    {  603, 6031,  189934,  8, }, // Pacific/Saipan
    {  603, 7312,  189942,  5, }, // Pacific/Tahiti
    {  603, 6712,  189947,  5, }, // Pacific/Tarawa
    {  603, 4583,  189952,  9, }, // Pacific/Tongatapu
    {  603, 7392,  189961,  4, }, // Pacific/Wake
    {  603, 7405,  189965,  8, }, // Pacific/Wallis
    {  604,   14,       0,  6, }, // Africa/Asmara Teso/Latin/Uganda
    {  604, 4435,       6,  8, }, // Africa/Sao_Tome
    {  604, 4252,      14,  8, }, // America/Asuncion
    {  604,  237,      22,  8, }, // America/Atikokan
    {  604, 7623,      30, 17, }, // America/Bahia_Banderas
    {  604, 3868,      47,  6, }, // America/Cancun
    {  604, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  604, 5723,      66,  7, }, // America/Curacao
    {  604,  481,      73, 13, }, // America/Indiana/Knox
    {  604, 7961,      86, 16, }, // America/Indiana/Marengo
    {  604, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  604, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  604, 8038,     139, 14, }, // America/Indiana/Vevay
    {  604, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  604, 8086,     171, 16, }, // America/Indiana/Winamac
    {  604, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  604, 5932,     207, 22, }, // America/Lower_Princes
    {  604, 8220,     229,  6, }, // America/Merida
    {  604, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  604, 8329,     255, 20, }, // America/North_Dakota/Center
    {  604, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  604,  393,     298,  4, }, // America/Nuuk
    {  604, 6899,     302, 16, }, // America/Scoresbysund
    {  604, 8508,     318, 14, }, // America/St_Barthelemy
    {  604, 2061,     332, 10, }, // America/St_Johns
    {  604, 5863,     342,  9, }, // America/St_Kitts
    {  604, 5880,     351,  9, }, // America/St_Lucia
    {  604,  855,     360, 10, }, // America/St_Thomas
    {  604, 5913,     370, 11, }, // America/St_Vincent
    {  604, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  604, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  604, 1185,     413,  9, }, // Asia/Kathmandu
    {  604,  992,     422,  7, }, // Asia/Kolkata
    {  604, 1211,     429,  5, }, // Asia/Macau
    {  604, 1235,     442,  6, }, // Asia/Yangon
    {  604, 1395,     448,  5, }, // Atlantic/Faroe
    {  604, 6821,     453, 10, }, // Atlantic/St_Helena
    {  604, 2527,     463,  4, }, // Europe/Kyiv
    {  604, 7211,     467,  7, }, // Indian/Reunion
    {  604, 3157,     474,  5, }, // Pacific/Chuuk
    {  604, 3096,     479,  7, }, // Pacific/Pohnpei
    {  606, 6788,  189973,  7, }, // Africa/Abidjan Thai/Thai/Thailand
    {  606, 6760,  189980,  6, }, // Africa/Accra
    {  606, 5059,  189986, 12, }, // Africa/Addis_Ababa
    {  606, 7420,  189998,  9, }, // Africa/Algiers
    {  606,   14,  190007,  7, }, // Africa/Asmara
    {  606,   44,  190014,  6, }, // Africa/Bamako
    {  606, 5257,  190020,  5, }, // Africa/Bangui
    {  606, 6746,  190025,  6, }, // Africa/Banjul
    {  606, 7435,  190031,  6, }, // Africa/Bissau
    {  606, 4957,  190037,  8, }, // Africa/Blantyre
    {  606, 5287,  190045,  9, }, // Africa/Brazzaville
    {  606, 4922,  190054,  9, }, // Africa/Bujumbura
    {  606, 2320,  190063,  4, }, // Africa/Cairo
    {  606, 4117,  190067, 12, }, // Africa/Casablanca
    {  606, 7449,  190079,  5, }, // Africa/Ceuta
    {  606, 6773,  190084,  7, }, // Africa/Conakry
    {  606, 6840,  190091,  6, }, // Africa/Dakar
    {  606, 5130,  190097, 12, }, // Africa/Dar_es_Salaam
    {  606, 5043,  190109,  6, }, // Africa/Djibouti
    {  606, 5243,  190115,  6, }, // Africa/Douala
    {  606, 5166,  190121, 10, }, // Africa/El_Aaiun
    {  606, 6853,  190131,  8, }, // Africa/Freetown
    {  606, 4906,  190139,  8, }, // Africa/Gaborone
    {  606, 5015,  190147,  6, }, // Africa/Harare
    {  606, 4479,  190153, 14, }, // Africa/Johannesburg
    {  606, 4499,  190167,  4, }, // Africa/Juba
    {  606, 5151,  190171,  7, }, // Africa/Kampala
    {  606, 4524,  190178,  7, }, // Africa/Khartoum
    {  606, 4973,  190185,  6, }, // Africa/Kigali
    {  606, 5306,  190191,  7, }, // Africa/Kinshasa
    {  606, 4744,  190198,  5, }, // Africa/Lagos
    {  606, 5336,  190203,  9, }, // Africa/Libreville
    {  606, 6869,  190212,  4, }, // Africa/Lome
    {  606, 5211,  190216,  7, }, // Africa/Luanda
    {  606, 4939,  190223,  9, }, // Africa/Lubumbashi
    {  606, 5001,  190232,  6, }, // Africa/Lusaka
    {  606, 5322,  190238,  6, }, // Africa/Malabo
    {  606, 4987,  190244,  6, }, // Africa/Maputo
    {  606, 5197,  190250,  6, }, // Africa/Maseru
    {  606, 5182,  190256,  9, }, // Africa/Mbabane
    {  606, 5113,  190265,  8, }, // Africa/Mogadishu
    {  606, 7034,  190273,  9, }, // Africa/Monrovia
    {  606, 3853,  190282,  8, }, // Africa/Nairobi
    {  606, 5271,  190290, 10, }, // Africa/Ndjamena
    {  606, 5354,  190300,  8, }, // Africa/Niamey
    {  606, 6803,  190308,  8, }, // Africa/Nouakchott
    {  606, 6727,  190316,  8, }, // Africa/Ouagadougou
    {  606, 5225,  190324, 11, }, // Africa/Porto-Novo
    {  606, 4435,  190335,  7, }, // Africa/Sao_Tome
    {  606, 2866,  190342,  7, }, // Africa/Tripoli
    {  606, 6554,  190349,  5, }, // Africa/Tunis
    {  606, 4152,  190354,  8, }, // Africa/Windhoek
    {  606,  132,  190362,  5, }, // America/Adak
    {  606, 3346,  190367,  9, }, // America/Anchorage
    {  606, 5607,  190376,  8, }, // America/Anguilla
    {  606, 5624,  190384,  7, }, // America/Antigua
    {  606, 4554,  190391,  9, }, // America/Araguaina
    {  606,  166,  190400, 11, }, // America/Argentina/Buenos_Aires
    {  606,   91,  190411, 10, }, // America/Argentina/Catamarca
    {  606,  270,  190421,  8, }, // America/Argentina/Cordoba
    {  606,  441,  190429,  5, }, // America/Argentina/Jujuy
    {  606, 7462,  190434,  8, }, // America/Argentina/La_Rioja
    {  606,  565,  190442,  7, }, // America/Argentina/Mendoza
    {  606, 7489,  190449, 12, }, // America/Argentina/Rio_Gallegos
    {  606, 7520,  190461,  5, }, // America/Argentina/Salta
    {  606, 7544,  190466,  6, }, // America/Argentina/San_Juan
    {  606, 5580,  190472,  8, }, // America/Argentina/San_Luis
    {  606, 7571,  190480,  7, }, // America/Argentina/Tucuman
    {  606, 7597,  190487,  8, }, // America/Argentina/Ushuaia
    {  606, 5640,  190495,  6, }, // America/Aruba
    {  606, 4252,  190501, 10, }, // America/Asuncion
    {  606,  237,  190511, 14, }, // America/Atikokan
    {  606, 3679,  190525,  6, }, // America/Bahia
    {  606, 7623,  190531, 14, }, // America/Bahia_Banderas
    {  606, 5654,  190545,  9, }, // America/Barbados
    {  606, 7646,  190554,  5, }, // America/Belem
    {  606, 5407,  190559,  5, }, // America/Belize
    {  606, 7660,  190564, 13, }, // America/Blanc-Sablon
    {  606, 7681,  190577,  8, }, // America/Boa_Vista
    {  606, 4361,  190585,  6, }, // America/Bogota
    {  606, 7699,  190591,  5, }, // America/Boise
    {  606, 7713,  190596, 13, }, // America/Cambridge_Bay
    {  606, 7735,  190609, 11, }, // America/Campo_Grande
    {  606, 3868,  190620,  6, }, // America/Cancun
    {  606, 4694,  190626,  7, }, // America/Caracas
    {  606, 4345,  190633,  5, }, // America/Cayenne
    {  606, 5496,  190638,  7, }, // America/Cayman
    {  606, 2260,  190645,  6, }, // America/Chicago
    {  606, 7756,  190651,  6, }, // America/Chihuahua
    {  606, 7774,  190657, 12, }, // America/Ciudad_Juarez
    {  606, 5422,  190669,  9, }, // America/Costa_Rica
    {  606, 7796,  190678,  7, }, // America/Creston
    {  606, 3791,  190685,  6, }, // America/Cuiaba
    {  606, 5723,  190691,  7, }, // America/Curacao
    {  606, 7812,  190698, 14, }, // America/Danmarkshavn
    {  606, 7833,  190712,  7, }, // America/Dawson
    {  606, 7848,  190719, 12, }, // America/Dawson_Creek
    {  606,  805,  190731,  8, }, // America/Denver
    {  606, 3465,  190739,  8, }, // America/Detroit
    {  606, 5739,  190747,  8, }, // America/Dominica
    {  606,  893,  190755,  9, }, // America/Edmonton
    {  606, 7869,  190764,  8, }, // America/Eirunepe
    {  606, 5441,  190772, 12, }, // America/El_Salvador
    {  606, 7886,  190784, 11, }, // America/Fort_Nelson
    {  606, 7906,  190795, 10, }, // America/Fortaleza
    {  606, 7924,  190805,  8, }, // America/Glace_Bay
    {  606, 6881,  190813,  7, }, // America/Goose_Bay
    {  606, 4612,  190820, 12, }, // America/Grand_Turk
    {  606, 5770,  190832,  7, }, // America/Grenada
    {  606, 5786,  190839,  9, }, // America/Guadeloupe
    {  606, 3760,  190848,  9, }, // America/Guatemala
    {  606, 4841,  190857,  8, }, // America/Guayaquil
    {  606, 6932,  190865,  7, }, // America/Guyana
    {  606, 1939,  190872,  9, }, // America/Halifax
    {  606, 2281,  190881,  6, }, // America/Havana
    {  606, 7942,  190887, 10, }, // America/Hermosillo
    {  606,  348,  190897, 14, }, // America/Indiana/Indianapolis
    {  606,  481,  190911, 16, }, // America/Indiana/Knox
    {  606, 7961,  190927, 18, }, // America/Indiana/Marengo
    {  606, 7985,  190945, 25, }, // America/Indiana/Petersburg
    {  606, 8012,  190970, 20, }, // America/Indiana/Tell_City
    {  606, 8038,  190990, 17, }, // America/Indiana/Vevay
    {  606, 8060,  191007, 20, }, // America/Indiana/Vincennes
    {  606, 8086,  191027, 18, }, // America/Indiana/Winamac
    {  606, 8110,  191045,  7, }, // America/Inuvik
    {  606,  660,  191052,  8, }, // America/Iqaluit
    {  606, 2799,  191060,  6, }, // America/Jamaica
    {  606, 5380,  191066,  4, }, // America/Juneau
    {  606,  521,  191070,  9, }, // America/Kentucky/Louisville
    {  606, 8125,  191079, 20, }, // America/Kentucky/Monticello
    {  606, 5704,  191099, 11, }, // America/Kralendijk
    {  606, 4376,  191110,  5, }, // America/La_Paz
    {  606, 7169,  191115,  4, }, // America/Lima
    {  606, 3239,  191119, 11, }, // America/Los_Angeles
    {  606, 5932,  191130, 22, }, // America/Lower_Princes
    {  606, 8153,  191152,  6, }, // America/Maceio
    {  606, 8168,  191158,  7, }, // America/Managua
    {  606, 1908,  191165,  7, }, // America/Manaus
    {  606, 5897,  191172,  8, }, // America/Marigot
    {  606, 5805,  191180,  9, }, // America/Martinique
    {  606, 8184,  191189,  9, }, // America/Matamoros
    {  606, 2917,  191198,  8, }, // America/Mazatlan
    {  606, 8202,  191206,  8, }, // America/Menominee
    {  606, 8220,  191214,  6, }, // America/Merida
    {  606, 8235,  191220, 10, }, // America/Metlakatla
    {  606, 2949,  191230, 12, }, // America/Mexico_City
    {  606, 4391,  191242,  7, }, // America/Miquelon
    {  606, 8254,  191249,  6, }, // America/Moncton
    {  606, 8270,  191255, 11, }, // America/Monterrey
    {  606, 4098,  191266, 11, }, // America/Montevideo
    {  606, 5824,  191277, 11, }, // America/Montserrat
    {  606, 5481,  191288,  5, }, // America/Nassau
    {  606, 2379,  191293,  8, }, // America/New_York
    {  606, 8288,  191301,  3, }, // America/Nome
    {  606, 1850,  191304,  7, }, // America/Noronha
    {  606, 8301,  191311, 19, }, // America/North_Dakota/Beulah
    {  606, 8329,  191330, 21, }, // America/North_Dakota/Center
    {  606, 8357,  191351, 21, }, // America/North_Dakota/New_Salem
    {  606,  393,  191372,  6, }, // America/Nuuk
    {  606, 8388,  191378,  8, }, // America/Ojinaga
    {  606, 2356,  191386,  6, }, // America/Panama
    {  606, 6173,  191392, 10, }, // America/Paramaribo
    {  606, 2973,  191402,  7, }, // America/Phoenix
    {  606, 3945,  191409, 12, }, // America/Port-au-Prince
    {  606, 5954,  191421, 12, }, // America/Port_of_Spain
    {  606, 8404,  191433, 11, }, // America/Porto_Velho
    {  606, 5843,  191444,  9, }, // America/Puerto_Rico
    {  606, 4030,  191453, 12, }, // America/Punta_Arenas
    {  606, 8424,  191465, 13, }, // America/Rankin_Inlet
    {  606, 8445,  191478,  6, }, // America/Recife
    {  606, 1995,  191484,  6, }, // America/Regina
    {  606, 8460,  191490,  7, }, // America/Resolute
    {  606,  695,  191497, 10, }, // America/Rio_Branco
    {  606, 8477,  191507,  8, }, // America/Santarem
    {  606, 2201,  191515,  9, }, // America/Santiago
    {  606, 6111,  191524, 13, }, // America/Santo_Domingo
    {  606, 1878,  191537,  8, }, // America/Sao_Paulo
    {  606, 6899,  191545, 13, }, // America/Scoresbysund
    {  606, 8494,  191558,  5, }, // America/Sitka
    {  606, 8508,  191563, 15, }, // America/St_Barthelemy
    {  606, 2061,  191578, 10, }, // America/St_Johns
    {  606, 5863,  191588, 10, }, // America/St_Kitts
    {  606, 5880,  191598, 11, }, // America/St_Lucia
    {  606,  855,  191609, 10, }, // America/St_Thomas
    {  606, 5913,  191619, 13, }, // America/St_Vincent
    {  606, 8530,  191632, 16, }, // America/Swift_Current
    {  606, 5461,  191648, 11, }, // America/Tegucigalpa
    {  606, 5756,  191659,  4, }, // America/Thule
    {  606,  313,  191663,  7, }, // America/Tijuana
    {  606,  608,  191670,  7, }, // America/Toronto
    {  606, 5688,  191677,  8, }, // America/Tortola
    {  606, 2093,  191685, 10, }, // America/Vancouver
    {  606, 2144,  191695,  9, }, // America/Whitehorse
    {  606,  734,  191704,  8, }, // America/Winnipeg
    {  606, 8552,  191712,  7, }, // America/Yakutat
    {  606, 6001,  191719,  6, }, // Antarctica/Casey
    {  606, 6094,  191725,  5, }, // Antarctica/Davis
    {  606, 6133,  191730, 16, }, // Antarctica/DumontDUrville
    {  606, 8568,  191746,  8, }, // Antarctica/Macquarie
    {  606, 7066,  191754,  7, }, // Antarctica/Mawson
    {  606, 7113,  191761,  8, }, // Antarctica/McMurdo
    {  606, 8589,  191769,  9, }, // Antarctica/Palmer
    {  606, 7226,  191778,  6, }, // Antarctica/Rothera
    {  606, 7295,  191784,  6, }, // Antarctica/Syowa
    {  606, 8607,  191790,  4, }, // Antarctica/Troll
    {  606, 7374,  191794,  6, }, // Antarctica/Vostok
    {  606, 1429,  191800, 14, }, // Arctic/Longyearbyen
    {  606, 5570,  191814,  5, }, // Asia/Aden
    {  606, 5395,  191819,  7, }, // Asia/Almaty
    {  606, 3968,  191826,  6, }, // Asia/Amman
    {  606, 5511,  191832,  8, }, // Asia/Anadyr
    {  606, 5523,  191840,  7, }, // Asia/Aqtau
    {  606, 5368,  191847,  7, }, // Asia/Aqtobe
    {  606,  964,  191854,  8, }, // Asia/Ashgabat
    {  606, 8624,  191862,  6, }, // Asia/Atyrau
    {  606, 3607,  191868,  6, }, // Asia/Baghdad
    {  606, 5534,  191874,  7, }, // Asia/Bahrain
    {  606, 3653,  191881,  4, }, // Asia/Baku
    {  606, 4466,  191885,  7, }, // Asia/Bangkok
    {  606, 3571,  191892,  9, }, // Asia/Barnaul
    {  606, 4086,  191901,  5, }, // Asia/Beirut
    {  606, 3778,  191906,  6, }, // Asia/Bishkek
    {  606, 5989,  191912,  5, }, // Asia/Brunei
    {  606, 4601,  191917,  4, }, // Asia/Chita
    {  606, 4511,  191921,  7, }, // Asia/Colombo
    {  606, 4540,  191928,  8, }, // Asia/Damascus
    {  606, 1093,  190091,  4, }, // Asia/Dhaka
    {  606, 6192,  191936,  4, }, // Asia/Dili
    {  606, 3596,  191940,  4, }, // Asia/Dubai
    {  606, 6159,  191944,  7, }, // Asia/Dushanbe
    {  606, 8636,  191951,  9, }, // Asia/Famagusta
    {  606, 8651,  191960,  4, }, // Asia/Gaza
    {  606, 4795,  191964,  6, }, // Asia/Hebron
    {  606, 1259,  191970, 12, }, // Asia/Ho_Chi_Minh
    {  606, 2704,  191982,  6, }, // Asia/Hong_Kong
    {  606, 4771,  191988,  5, }, // Asia/Hovd
    {  606, 4184,  191993, 10, }, // Asia/Irkutsk
    {  606, 7006,  192003,  8, }, // Asia/Jakarta
    {  606, 6992,  192011,  8, }, // Asia/Jayapura
    {  606, 1290,  192019,  9, }, // Asia/Jerusalem
    {  606, 3560,  192028,  5, }, // Asia/Kabul
    {  606, 4316,  192033,  8, }, // Asia/Kamchatka
    {  606, 4239,  192041,  6, }, // Asia/Karachi
    {  606, 1185,  192047,  8, }, // Asia/Kathmandu
    {  606, 8661,  192055,  8, }, // Asia/Khandyga
    {  606,  992,  192063,  7, }, // Asia/Kolkata
    {  606, 4197,  192070, 13, }, // Asia/Krasnoyarsk
    {  606, 4859,  192083, 13, }, // Asia/Kuala_Lumpur
    {  606, 5976,  192096,  5, }, // Asia/Kuching
    {  606, 5547,  192101,  5, }, // Asia/Kuwait
    {  606, 1211,  192106,  6, }, // Asia/Macau
    {  606, 4017,  192112,  7, }, // Asia/Magadan
    {  606, 1349,  192119,  9, }, // Asia/Makassar
    {  606, 7182,  192128,  6, }, // Asia/Manila
    {  606, 6920,  192134,  6, }, // Asia/Muscat
    {  606, 2554,  192140,  8, }, // Asia/Nicosia
    {  606, 8675,  192148, 13, }, // Asia/Novokuznetsk
    {  606, 4135,  192161, 13, }, // Asia/Novosibirsk
    {  606, 4229,  192174,  6, }, // Asia/Omsk
    {  606, 7145,  192180,  5, }, // Asia/Oral
    {  606, 6961,  192185,  6, }, // Asia/Phnom_Penh
    {  606, 8693,  192191, 10, }, // Asia/Pontianak
    {  606, 4214,  192201,  8, }, // Asia/Pyongyang
    {  606, 5559,  192209,  6, }, // Asia/Qatar
    {  606, 8708,  192215,  9, }, // Asia/Qostanay
    {  606, 4269,  192224, 10, }, // Asia/Qyzylorda
    {  606, 3584,  192234,  7, }, // Asia/Riyadh
    {  606, 4408,  192241,  7, }, // Asia/Sakhalin
    {  606, 7245,  192248, 11, }, // Asia/Samarkand
    {  606, 3279,  191492,  3, }, // Asia/Seoul
    {  606, 1053,  192259,  9, }, // Asia/Shanghai
    {  606, 3300,  192268,  8, }, // Asia/Singapore
    {  606, 4297,  192276, 15, }, // Asia/Srednekolymsk
    {  606, 3263,  192291,  4, }, // Asia/Taipei
    {  606, 4781,  192295,  8, }, // Asia/Tashkent
    {  606, 3915,  192303,  7, }, // Asia/Tbilisi
    {  606, 2772,  192310,  7, }, // Asia/Tehran
    {  606, 1317,  192317,  5, }, // Asia/Thimphu
    {  606, 2821,  192322,  7, }, // Asia/Tokyo
    {  606, 4572,  192329,  6, }, // Asia/Tomsk
    {  606, 1021,  192335, 11, }, // Asia/Ulaanbaatar
    {  606, 1159,  192346,  7, }, // Asia/Urumqi
    {  606, 8722,  192353,  9, }, // Asia/Ust-Nera
    {  606, 6977,  192362, 11, }, // Asia/Vientiane
    {  606, 4710,  192373, 12, }, // Asia/Vladivostok
    {  606, 4828,  192385,  8, }, // Asia/Yakutsk
    {  606, 1235,  192393,  8, }, // Asia/Yangon
    {  606, 3883,  192401, 16, }, // Asia/Yekaterinburg
    {  606, 3747,  192417,  7, }, // Asia/Yerevan
    {  606, 3663,  192424,  7, }, // Atlantic/Azores
    {  606, 5671,  192431, 10, }, // Atlantic/Bermuda
    {  606, 6628,  192441,  6, }, // Atlantic/Canary
    {  606, 3727,  192447,  9, }, // Atlantic/Cape_Verde
    {  606, 1395,  192456,  4, }, // Atlantic/Faroe
    {  606, 8736,  192460,  6, }, // Atlantic/Madeira
    {  606, 2748,  192466,  8, }, // Atlantic/Reykjavik
    {  606, 7272,  192474, 14, }, // Atlantic/South_Georgia
    {  606, 6821,  192488, 11, }, // Atlantic/St_Helena
    {  606, 6644,  192499,  8, }, // Atlantic/Stanley
    {  606, 1670,  192507,  7, }, // Australia/Adelaide
    {  606, 1635,  192514,  7, }, // Australia/Brisbane
    {  606, 1799,  192521, 12, }, // Australia/Broken_Hill
    {  606, 1583,  192533,  7, }, // Australia/Darwin
    {  606, 3637,  192540,  5, }, // Australia/Eucla
    {  606, 1516,  192545,  7, }, // Australia/Hobart
    {  606, 8753,  192552,  8, }, // Australia/Lindeman
    {  606, 1547,  192560,  9, }, // Australia/Lord_Howe
    {  606, 1727,  192569,  9, }, // Australia/Melbourne
    {  606, 1762,  192578,  6, }, // Australia/Perth
    {  606, 1463,  192584,  7, }, // Australia/Sydney
    {  606, 6441,  192591, 10, }, // Europe/Amsterdam
    {  606, 6216,  192601,  9, }, // Europe/Andorra
    {  606, 3620,  192610,  9, }, // Europe/Astrakhan
    {  606, 2300,  192619,  7, }, // Europe/Athens
    {  606, 6336,  192626,  7, }, // Europe/Belgrade
    {  606, 4757,  192633,  8, }, // Europe/Berlin
    {  606, 6488,  192641, 10, }, // Europe/Bratislava
    {  606, 2167,  192651,  9, }, // Europe/Brussels
    {  606, 3928,  192660,  7, }, // Europe/Bucharest
    {  606, 3806,  192667,  7, }, // Europe/Budapest
    {  606, 8772,  192674, 10, }, // Europe/Busingen
    {  606, 2583,  192684,  7, }, // Europe/Chisinau
    {  606, 6289,  192691, 10, }, // Europe/Copenhagen
    {  606, 2338,  192701,  7, }, // Europe/Dublin
    {  606, 6307,  192708, 10, }, // Europe/Gibraltar
    {  606, 8788,  192718, 10, }, // Europe/Guernsey
    {  606, 6612,  192728,  8, }, // Europe/Helsinki
    {  606, 8804,  192736,  7, }, // Europe/Isle_of_Man
    {  606, 1130,  192743,  9, }, // Europe/Istanbul
    {  606, 8823,  192752,  9, }, // Europe/Jersey
    {  606, 3979,  192761, 11, }, // Europe/Kaliningrad
    {  606, 8929,  192772,  5, }, // Europe/Kirov
    {  606, 2527,  192777,  5, }, // Europe/Kyiv
    {  606, 3213,  192782,  6, }, // Europe/Lisbon
    {  606, 6506,  192788,  9, }, // Europe/Ljubljana
    {  606, 2501,  192797,  6, }, // Europe/London
    {  606, 6365,  192803, 12, }, // Europe/Luxembourg
    {  606, 4892,  192815,  6, }, // Europe/Madrid
    {  606, 6397,  192821,  5, }, // Europe/Malta
    {  606, 6582,  192826,  9, }, // Europe/Mariehamn
    {  606, 3693,  192835,  6, }, // Europe/Minsk
    {  606, 6410,  192841,  6, }, // Europe/Monaco
    {  606, 3537,  192847,  5, }, // Europe/Moscow
    {  606, 6458,  192852,  5, }, // Europe/Oslo
    {  606, 4284,  192857,  5, }, // Europe/Paris
    {  606, 6424,  192862, 10, }, // Europe/Podgorica
    {  606, 6275,  192872,  4, }, // Europe/Prague
    {  606, 8837,  190674,  4, }, // Europe/Riga
    {  606, 6324,  192458,  3, }, // Europe/Rome
    {  606, 4331,  192876,  6, }, // Europe/Samara
    {  606, 6470,  192882,  9, }, // Europe/San_Marino
    {  606, 6245,  192891,  8, }, // Europe/Sarajevo
    {  606, 4451,  192899,  7, }, // Europe/Saratov
    {  606, 8849,  192906, 11, }, // Europe/Simferopol
    {  606, 6383,  192917,  7, }, // Europe/Skopje
    {  606, 6599,  192924,  6, }, // Europe/Sofia
    {  606, 6523,  192930,  9, }, // Europe/Stockholm
    {  606, 8867,  192939,  8, }, // Europe/Tallinn
    {  606, 6202,  192947,  6, }, // Europe/Tirane
    {  606, 8882,  192953,  9, }, // Europe/Ulyanovsk
    {  606, 6352,  192962,  5, }, // Europe/Vaduz
    {  606, 6567,  192967,  7, }, // Europe/Vatican
    {  606, 6231,  192974,  7, }, // Europe/Vienna
    {  606, 8899,  192981,  8, }, // Europe/Vilnius
    {  606, 4727,  192989,  9, }, // Europe/Volgograd
    {  606, 3190,  192998,  6, }, // Europe/Warsaw
    {  606, 6261,  193004,  7, }, // Europe/Zagreb
    {  606, 6540,  193011,  5, }, // Europe/Zurich
    {  606, 5078,  193016, 13, }, // Indian/Antananarivo
    {  606, 6947,  193029,  5, }, // Indian/Chagos
    {  606, 6046,  193034,  9, }, // Indian/Christmas
    {  606, 6063,  193043,  5, }, // Indian/Cocos
    {  606, 5029,  193048,  6, }, // Indian/Comoro
    {  606, 6661,  193054, 10, }, // Indian/Kerguelen
    {  606, 7260,  193064,  4, }, // Indian/Mahe
    {  606, 7050,  193068,  8, }, // Indian/Maldives
    {  606, 4069,  193076,  9, }, // Indian/Mauritius
    {  606, 5098,  193085,  6, }, // Indian/Mayotte
    {  606, 7211,  193091,  8, }, // Indian/Reunion
    {  606, 4422,  193099,  6, }, // Pacific/Apia
    {  606,  932,  193105,  8, }, // Pacific/Auckland
    {  606, 3706,  193113, 10, }, // Pacific/Bougainville
    {  606, 3015,  193123,  5, }, // Pacific/Chatham
    {  606, 3157,  193128,  4, }, // Pacific/Chuuk
    {  606, 2237,  193132,  8, }, // Pacific/Easter
    {  606, 7360,  193140,  6, }, // Pacific/Efate
    {  606, 7327,  193146,  7, }, // Pacific/Fakaofo
    {  606, 3902,  193153,  4, }, // Pacific/Fiji
    {  606, 7343,  193157,  8, }, // Pacific/Funafuti
    {  606, 6678,  193165,  9, }, // Pacific/Galapagos
    {  606, 6696,  193174,  9, }, // Pacific/Gambier
    {  606, 3822,  193183, 11, }, // Pacific/Guadalcanal
    {  606, 6018,  193194,  3, }, // Pacific/Guam
    {  606, 2723,  193197,  8, }, // Pacific/Honolulu
    {  606, 3049,  193205, 13, }, // Pacific/Kanton
    {  606, 3998,  193218, 10, }, // Pacific/Kiritimati
    {  606, 7019,  193228,  5, }, // Pacific/Kosrae
    {  606, 2842,  193233,  8, }, // Pacific/Kwajalein
    {  606, 4877,  193241,  6, }, // Pacific/Majuro
    {  606, 4051,  193247,  9, }, // Pacific/Marquesas
    {  606, 8914,  193256,  7, }, // Pacific/Midway
    {  606, 7084,  193263,  6, }, // Pacific/Nauru
    {  606, 7132,  193269,  6, }, // Pacific/Niue
    {  606, 4168,  193275,  9, }, // Pacific/Norfolk
    {  606, 7098,  193284,  6, }, // Pacific/Noumea
    {  606, 3126,  193290,  9, }, // Pacific/Pago_Pago
    {  606, 7155,  193299,  5, }, // Pacific/Palau
    {  606, 7194,  193304,  8, }, // Pacific/Pitcairn
    {  606, 3096,  193312,  6, }, // Pacific/Pohnpei
    {  606, 4807,  193318, 12, }, // Pacific/Port_Moresby
    {  606, 6076,  193330,  9, }, // Pacific/Rarotonga
    {  606, 6031,  193339,  5, }, // Pacific/Saipan
    {  606, 7312,  193344,  6, }, // Pacific/Tahiti
    {  606, 6712,  193350,  6, }, // Pacific/Tarawa
    {  606, 4583,  193356,  9, }, // Pacific/Tongatapu
    {  606, 7392,  193365,  3, }, // Pacific/Wake
    {  606, 7405,  193368,  6, }, // Pacific/Wallis
    {  607,   14,       0,  6, }, // Africa/Asmara Tibetan/Tibetan/China
    {  607, 4435,       6,  8, }, // Africa/Sao_Tome
    {  607, 4252,      14,  8, }, // America/Asuncion
    {  607,  237,      22,  8, }, // America/Atikokan
    {  607, 7623,      30, 17, }, // America/Bahia_Banderas
    {  607, 3868,      47,  6, }, // America/Cancun
    {  607, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  607, 5723,      66,  7, }, // America/Curacao
    {  607,  481,      73, 13, }, // America/Indiana/Knox
    {  607, 7961,      86, 16, }, // America/Indiana/Marengo
    {  607, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  607, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  607, 8038,     139, 14, }, // America/Indiana/Vevay
    {  607, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  607, 8086,     171, 16, }, // America/Indiana/Winamac
    {  607, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  607, 5932,     207, 22, }, // America/Lower_Princes
    {  607, 8220,     229,  6, }, // America/Merida
    {  607, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  607, 8329,     255, 20, }, // America/North_Dakota/Center
    {  607, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  607,  393,     298,  4, }, // America/Nuuk
    {  607, 6899,     302, 16, }, // America/Scoresbysund
    {  607, 8508,     318, 14, }, // America/St_Barthelemy
    {  607, 2061,     332, 10, }, // America/St_Johns
    {  607, 5863,     342,  9, }, // America/St_Kitts
    {  607, 5880,     351,  9, }, // America/St_Lucia
    {  607,  855,     360, 10, }, // America/St_Thomas
    {  607, 5913,     370, 11, }, // America/St_Vincent
    {  607, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  607, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  607, 1185,     413,  9, }, // Asia/Kathmandu
    {  607,  992,     422,  7, }, // Asia/Kolkata
    {  607, 1211,     429,  5, }, // Asia/Macau
    {  607, 1235,     442,  6, }, // Asia/Yangon
    {  607, 1395,     448,  5, }, // Atlantic/Faroe
    {  607, 6821,     453, 10, }, // Atlantic/St_Helena
    {  607, 2527,     463,  4, }, // Europe/Kyiv
    {  607, 7211,     467,  7, }, // Indian/Reunion
    {  607, 3157,     474,  5, }, // Pacific/Chuuk
    {  607, 3096,     479,  7, }, // Pacific/Pohnpei
    {  609,   14,       0,  6, }, // Africa/Asmara Tigre/Ethiopic/Eritrea
    {  609, 4435,       6,  8, }, // Africa/Sao_Tome
    {  609, 4252,      14,  8, }, // America/Asuncion
    {  609,  237,      22,  8, }, // America/Atikokan
    {  609, 7623,      30, 17, }, // America/Bahia_Banderas
    {  609, 3868,      47,  6, }, // America/Cancun
    {  609, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  609, 5723,      66,  7, }, // America/Curacao
    {  609,  481,      73, 13, }, // America/Indiana/Knox
    {  609, 7961,      86, 16, }, // America/Indiana/Marengo
    {  609, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  609, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  609, 8038,     139, 14, }, // America/Indiana/Vevay
    {  609, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  609, 8086,     171, 16, }, // America/Indiana/Winamac
    {  609, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  609, 5932,     207, 22, }, // America/Lower_Princes
    {  609, 8220,     229,  6, }, // America/Merida
    {  609, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  609, 8329,     255, 20, }, // America/North_Dakota/Center
    {  609, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  609,  393,     298,  4, }, // America/Nuuk
    {  609, 6899,     302, 16, }, // America/Scoresbysund
    {  609, 8508,     318, 14, }, // America/St_Barthelemy
    {  609, 2061,     332, 10, }, // America/St_Johns
    {  609, 5863,     342,  9, }, // America/St_Kitts
    {  609, 5880,     351,  9, }, // America/St_Lucia
    {  609,  855,     360, 10, }, // America/St_Thomas
    {  609, 5913,     370, 11, }, // America/St_Vincent
    {  609, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  609, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  609, 1185,     413,  9, }, // Asia/Kathmandu
    {  609,  992,     422,  7, }, // Asia/Kolkata
    {  609, 1211,     429,  5, }, // Asia/Macau
    {  609, 1235,     442,  6, }, // Asia/Yangon
    {  609, 1395,     448,  5, }, // Atlantic/Faroe
    {  609, 6821,     453, 10, }, // Atlantic/St_Helena
    {  609, 2527,     463,  4, }, // Europe/Kyiv
    {  609, 7211,     467,  7, }, // Indian/Reunion
    {  609, 3157,     474,  5, }, // Pacific/Chuuk
    {  609, 3096,     479,  7, }, // Pacific/Pohnpei
    {  610, 6788,  193374,  4, }, // Africa/Abidjan Tigrinya/Ethiopic/Ethiopia
    {  610, 6760,  193378,  3, }, // Africa/Accra
    {  610, 5059,  193381,  7, }, // Africa/Addis_Ababa
    {  610, 7420,  193388,  5, }, // Africa/Algiers
    {  610,   14,  193393,  4, }, // Africa/Asmara
    {  610,   44,    8378,  3, }, // Africa/Bamako
    {  610, 5257,  193397,  3, }, // Africa/Bangui
    {  610, 6746,    8385,  4, }, // Africa/Banjul
    {  610, 7435,  193400,  3, }, // Africa/Bissau
    {  610, 4957,    8392,  6, }, // Africa/Blantyre
    {  610, 5287,    8398,  5, }, // Africa/Brazzaville
    {  610, 4922,  193403,  5, }, // Africa/Bujumbura
    {  610, 2320,    8408,  3, }, // Africa/Cairo
    {  610, 4117,    8411,  6, }, // Africa/Casablanca
    {  610, 7449,  193408,  3, }, // Africa/Ceuta
    {  610, 6773,  193411,  4, }, // Africa/Conakry
    {  610, 6840,    8424,  3, }, // Africa/Dakar
    {  610, 5130,  193415,  9, }, // Africa/Dar_es_Salaam
    {  610, 5043,    8433,  3, }, // Africa/Djibouti
    {  610, 5243,  193424,  3, }, // Africa/Douala
    {  610, 5166,  193427,  6, }, // Africa/El_Aaiun
    {  610, 6853,    8446,  5, }, // Africa/Freetown
    {  610, 4906,    8451,  4, }, // Africa/Gaborone
    {  610, 5015,  193433,  3, }, // Africa/Harare
    {  610, 4479,    8458,  7, }, // Africa/Johannesburg
    {  610, 4499,    8465,  2, }, // Africa/Juba
    {  610, 5151,    8467,  4, }, // Africa/Kampala
    {  610, 4524,    8471,  4, }, // Africa/Khartoum
    {  610, 4973,    8475,  3, }, // Africa/Kigali
    {  610, 5306,    8478,  4, }, // Africa/Kinshasa
    {  610, 4744,    8482,  3, }, // Africa/Lagos
    {  610, 5336,  193436,  5, }, // Africa/Libreville
    {  610, 6869,  193441,  2, }, // Africa/Lome
    {  610, 5211,  193443,  4, }, // Africa/Luanda
    {  610, 4939,    8496,  5, }, // Africa/Lubumbashi
    {  610, 5001,    8501,  3, }, // Africa/Lusaka
    {  610, 5322,  193447,  3, }, // Africa/Malabo
    {  610, 4987,  193450,  3, }, // Africa/Maputo
    {  610, 5197,  193453,  3, }, // Africa/Maseru
    {  610, 5182,  193456,  4, }, // Africa/Mbabane
    {  610, 5113,  193460,  4, }, // Africa/Mogadishu
    {  610, 7034,  193464,  5, }, // Africa/Monrovia
    {  610, 3853,    8526,  4, }, // Africa/Nairobi
    {  610, 5271,  193469,  4, }, // Africa/Ndjamena
    {  610, 5354,  193473,  4, }, // Africa/Niamey
    {  610, 6803,  193477,  5, }, // Africa/Nouakchott
    {  610, 6727,  193482,  4, }, // Africa/Ouagadougou
    {  610, 5225,  193486,  6, }, // Africa/Porto-Novo
    {  610, 4435,  193492,  5, }, // Africa/Sao_Tome
    {  610, 2866,    8560,  4, }, // Africa/Tripoli
    {  610, 6554,  193497,  3, }, // Africa/Tunis
    {  610, 4152,    8567,  5, }, // Africa/Windhoek
    {  610,  132,  193500,  3, }, // America/Adak
    {  610, 3346,  193503,  5, }, // America/Anchorage
    {  610, 5607,  193508,  4, }, // America/Anguilla
    {  610, 5624,  193512,  4, }, // America/Antigua
    {  610, 4554,  193516,  5, }, // America/Araguaina
    {  610,  166,  193521,  9, }, // America/Argentina/Buenos_Aires
    {  610,   91,    8602,  5, }, // America/Argentina/Catamarca
    {  610,  270,    8607,  4, }, // America/Argentina/Cordoba
    {  610,  441,  193530,  3, }, // America/Argentina/Jujuy
    {  610, 7462,  193533,  5, }, // America/Argentina/La_Rioja
    {  610,  565,  193538,  4, }, // America/Argentina/Mendoza
    {  610, 7489,  193542,  7, }, // America/Argentina/Rio_Gallegos
    {  610, 7520,    8630,  3, }, // America/Argentina/Salta
    {  610, 7544,  193549,  6, }, // America/Argentina/San_Juan
    {  610, 5580,  193555,  6, }, // America/Argentina/San_Luis
    {  610, 7571,    8645,  4, }, // America/Argentina/Tucuman
    {  610, 7597,  193561,  4, }, // America/Argentina/Ushuaia
    {  610, 5640,  193565,  3, }, // America/Aruba
    {  610, 4252,  193568,  6, }, // America/Asuncion
    {  610,  237,  193574,  5, }, // America/Atikokan
    {  610, 3679,  193579,  3, }, // America/Bahia
    {  610, 7623,  193582, 11, }, // America/Bahia_Banderas
    {  610, 5654,  193593,  5, }, // America/Barbados
    {  610, 7646,  193598,  3, }, // America/Belem
    {  610, 5407,  193601,  3, }, // America/Belize
    {  610, 7660,  193604,  9, }, // America/Blanc-Sablon
    {  610, 7681,  193613,  6, }, // America/Boa_Vista
    {  610, 4361,    8706,  3, }, // America/Bogota
    {  610, 7699,  193619,  3, }, // America/Boise
    {  610, 7713,  193622,  8, }, // America/Cambridge_Bay
    {  610, 7735,  193630,  8, }, // America/Campo_Grande
    {  610, 3868,    8728,  4, }, // America/Cancun
    {  610, 4694,    8732,  4, }, // America/Caracas
    {  610, 4345,  193638,  3, }, // America/Cayenne
    {  610, 5496,    8740,  4, }, // America/Cayman
    {  610, 2260,    8744,  3, }, // America/Chicago
    {  610, 7756,  193641,  3, }, // America/Chihuahua
    {  610, 7774,  193644,  9, }, // America/Ciudad_Juarez
    {  610, 5422,  193653,  6, }, // America/Costa_Rica
    {  610, 7796,    8766,  5, }, // America/Creston
    {  610, 3791,  193659,  3, }, // America/Cuiaba
    {  610, 5723,  193662,  4, }, // America/Curacao
    {  610, 7812,    8779,  8, }, // America/Danmarkshavn
    {  610, 7833,    8787,  4, }, // America/Dawson
    {  610, 7848,  193666,  8, }, // America/Dawson_Creek
    {  610,  805,  193674,  4, }, // America/Denver
    {  610, 3465,    8803,  5, }, // America/Detroit
    {  610, 5739,  193678,  4, }, // America/Dominica
    {  610,  893,    8812,  6, }, // America/Edmonton
    {  610, 7869,  193682,  5, }, // America/Eirunepe
    {  610, 5441,  193687,  8, }, // America/El_Salvador
    {  610, 7886,  193695,  8, }, // America/Fort_Nelson
    {  610, 7906,  193703,  5, }, // America/Fortaleza
    {  610, 7924,  193708,  6, }, // America/Glace_Bay
    {  610, 6881,  193714,  5, }, // America/Goose_Bay
    {  610, 4612,  193719,  8, }, // America/Grand_Turk
    {  610, 5770,  193727,  4, }, // America/Grenada
    {  610, 5786,  193731,  4, }, // America/Guadeloupe
    {  610, 3760,    8869,  4, }, // America/Guatemala
    {  610, 4841,  193735,  4, }, // America/Guayaquil
    {  610, 6932,    8878,  3, }, // America/Guyana
    {  610, 1939,    8881,  5, }, // America/Halifax
    {  610, 2281,    8886,  3, }, // America/Havana
    {  610, 7942,  193739,  5, }, // America/Hermosillo
    {  610,  348,  193744,  8, }, // America/Indiana/Indianapolis
    {  610,  481,  193752, 10, }, // America/Indiana/Knox
    {  610, 7961,  193762, 11, }, // America/Indiana/Marengo
    {  610, 7985,  193773, 14, }, // America/Indiana/Petersburg
    {  610, 8012,  193787, 12, }, // America/Indiana/Tell_City
    {  610, 8038,  193799,  9, }, // America/Indiana/Vevay
    {  610, 8060,  193808, 12, }, // America/Indiana/Vincennes
    {  610, 8086,  193820, 11, }, // America/Indiana/Winamac
    {  610, 8110,    8983,  4, }, // America/Inuvik
    {  610,  660,  193831,  5, }, // America/Iqaluit
    {  610, 2799,    8992,  4, }, // America/Jamaica
    {  610, 5380,  193836,  3, }, // America/Juneau
    {  610,  521,  193839,  4, }, // America/Kentucky/Louisville
    {  610, 8125,  193843, 11, }, // America/Kentucky/Monticello
    {  610, 5704,  193854,  7, }, // America/Kralendijk
    {  610, 4376,    9022,  4, }, // America/La_Paz
    {  610, 7169,    9026,  2, }, // America/Lima
    {  610, 3239,  193861,  8, }, // America/Los_Angeles
    {  610, 5932,  193869, 14, }, // America/Lower_Princes
    {  610, 8153,  193883,  3, }, // America/Maceio
    {  610, 8168,  193886,  3, }, // America/Managua
    {  610, 1908,  193889,  4, }, // America/Manaus
    {  610, 5897,  193893,  4, }, // America/Marigot
    {  610, 5805,    9063,  5, }, // America/Martinique
    {  610, 8184,    9068,  5, }, // America/Matamoros
    {  610, 2917,    9073,  5, }, // America/Mazatlan
    {  610, 8202,    9078,  4, }, // America/Menominee
    {  610, 8220,  193897,  3, }, // America/Merida
    {  610, 8235,    9085,  6, }, // America/Metlakatla
    {  610, 2949,  193900,  8, }, // America/Mexico_City
    {  610, 4391,  193908,  4, }, // America/Miquelon
    {  610, 8254,  193912,  5, }, // America/Moncton
    {  610, 8270,  193917,  5, }, // America/Monterrey
    {  610, 4098,  193922,  6, }, // America/Montevideo
    {  610, 5824,  193928,  6, }, // America/Montserrat
    {  610, 5481,    9127,  3, }, // America/Nassau
    {  610, 2379,  193934,  6, }, // America/New_York
    {  610, 8288,  193940,  3, }, // America/Nome
    {  610, 1850,  193943,  4, }, // America/Noronha
    {  610, 8301,  193947, 12, }, // America/North_Dakota/Beulah
    {  610, 8329,  193959, 13, }, // America/North_Dakota/Center
    {  610, 8357,  193972, 15, }, // America/North_Dakota/New_Salem
    {  610,  393,  193987,  2, }, // America/Nuuk
    {  610, 8388,  193989,  4, }, // America/Ojinaga
    {  610, 2356,    9191,  3, }, // America/Panama
    {  610, 6173,    9194,  5, }, // America/Paramaribo
    {  610, 2973,    9199,  4, }, // America/Phoenix
    {  610, 3945,  193993, 10, }, // America/Port-au-Prince
    {  610, 5954,  194003, 10, }, // America/Port_of_Spain
    {  610, 8404,  194013,  7, }, // America/Porto_Velho
    {  610, 5843,  194020,  6, }, // America/Puerto_Rico
    {  610, 4030,  194026,  8, }, // America/Punta_Arenas
    {  610, 8424,  194034,  9, }, // America/Rankin_Inlet
    {  610, 8445,  194043,  3, }, // America/Recife
    {  610, 1995,    9254,  3, }, // America/Regina
    {  610, 8460,  194046,  4, }, // America/Resolute
    {  610,  695,  194050,  7, }, // America/Rio_Branco
    {  610, 8477,  194057,  5, }, // America/Santarem
    {  610, 2201,  194062,  5, }, // America/Santiago
    {  610, 6111,    9279,  8, }, // America/Santo_Domingo
    {  610, 1878,  194067,  6, }, // America/Sao_Paulo
    {  610, 6899,  194073,  7, }, // America/Scoresbysund
    {  610, 8494,    9301,  3, }, // America/Sitka
    {  610, 8508,  194080,  9, }, // America/St_Barthelemy
    {  610, 2061,  194089,  8, }, // America/St_Johns
    {  610, 5863,    9322,  7, }, // America/St_Kitts
    {  610, 5880,  194097,  8, }, // America/St_Lucia
    {  610,  855,    9337,  7, }, // America/St_Thomas
    {  610, 5913,    9344,  9, }, // America/St_Vincent
    {  610, 8530,  194105,  9, }, // America/Swift_Current
    {  610, 5461,  194114,  6, }, // America/Tegucigalpa
    {  610, 5756,  194120,  2, }, // America/Thule
    {  610,  313,  194122,  4, }, // America/Tijuana
    {  610,  608,    9374,  4, }, // America/Toronto
    {  610, 5688,    9378,  4, }, // America/Tortola
    {  610, 2093,  194126,  5, }, // America/Vancouver
    {  610, 2144,  194131,  5, }, // America/Whitehorse
    {  610,  734,  194136,  4, }, // America/Winnipeg
    {  610, 8552,    9397,  4, }, // America/Yakutat
    {  610, 6001,  194140,  3, }, // Antarctica/Casey
    {  610, 6094,  194143,  3, }, // Antarctica/Davis
    {  610, 6133,  194146, 10, }, // Antarctica/DumontDUrville
    {  610, 8568,  194156,  3, }, // Antarctica/Macquarie
    {  610, 7066,  194159,  4, }, // Antarctica/Mawson
    {  610, 7113,  194163,  5, }, // Antarctica/McMurdo
    {  610, 8589,    9429,  4, }, // Antarctica/Palmer
    {  610, 7226,  194168,  3, }, // Antarctica/Rothera
    {  610, 7295,    9436,  3, }, // Antarctica/Syowa
    {  610, 8607,    9439,  3, }, // Antarctica/Troll
    {  610, 7374,    9800,  4, }, // Antarctica/Vostok
    {  610, 1429,  194171,  8, }, // Arctic/Longyearbyen
    {  610, 5570,  194179,  3, }, // Asia/Aden
    {  610, 5395,  194182,  4, }, // Asia/Almaty
    {  610, 3968,  194186,  3, }, // Asia/Amman
    {  610, 5511,  194189,  4, }, // Asia/Anadyr
    {  610, 5523,  194193,  4, }, // Asia/Aqtau
    {  610, 5368,  194197,  4, }, // Asia/Aqtobe
    {  610,  964,  194201,  5, }, // Asia/Ashgabat
    {  610, 8624,  194206,  4, }, // Asia/Atyrau
    {  610, 3607,  194210,  4, }, // Asia/Baghdad
    {  610, 5534,  194214,  4, }, // Asia/Bahrain
    {  610, 3653,    8774,  2, }, // Asia/Baku
    {  610, 4466,  194218,  5, }, // Asia/Bangkok
    {  610, 3571,  194223,  5, }, // Asia/Barnaul
    {  610, 4086,  194228,  4, }, // Asia/Beirut
    {  610, 3778,    9508,  4, }, // Asia/Bishkek
    {  610, 5989,  194232,  4, }, // Asia/Brunei
    {  610, 4601,    9516,  2, }, // Asia/Chita
    {  610, 4511,    9518,  4, }, // Asia/Colombo
    {  610, 4540,    9522,  4, }, // Asia/Damascus
    {  610, 1093,    8424,  2, }, // Asia/Dhaka
    {  610, 6192,    9526,  2, }, // Asia/Dili
    {  610, 3596,    9528,  3, }, // Asia/Dubai
    {  610, 6159,  194236,  4, }, // Asia/Dushanbe
    {  610, 8636,    9535,  5, }, // Asia/Famagusta
    {  610, 8651,  194240,  2, }, // Asia/Gaza
    {  610, 4795,    9542,  4, }, // Asia/Hebron
    {  610, 1259,  194242, 10, }, // Asia/Ho_Chi_Minh
    {  610, 2704,    9557,  7, }, // Asia/Hong_Kong
    {  610, 4771,    9564,  3, }, // Asia/Hovd
    {  610, 4184,    9567,  6, }, // Asia/Irkutsk
    {  610, 7006,    9573,  4, }, // Asia/Jakarta
    {  610, 6992,    9577,  4, }, // Asia/Jayapura
    {  610, 1290,    9582,  5, }, // Asia/Jerusalem
    {  610, 3560,    9587,  3, }, // Asia/Kabul
    {  610, 4316,    9590,  5, }, // Asia/Kamchatka
    {  610, 4239,    9595,  3, }, // Asia/Karachi
    {  610, 1185,    9598,  5, }, // Asia/Kathmandu
    {  610, 8661,    9603,  4, }, // Asia/Khandyga
    {  610,  992,    9607,  4, }, // Asia/Kolkata
    {  610, 4197,    9611,  8, }, // Asia/Krasnoyarsk
    {  610, 4859,  194252,  7, }, // Asia/Kuala_Lumpur
    {  610, 5976,    9627,  4, }, // Asia/Kuching
    {  610, 5547,    9631,  3, }, // Asia/Kuwait
    {  610, 1211,  194259,  3, }, // Asia/Macau
    {  610, 4017,    9637,  4, }, // Asia/Magadan
    {  610, 1349,    9641,  4, }, // Asia/Makassar
    {  610, 7182,    9645,  3, }, // Asia/Manila
    {  610, 6920,    9648,  4, }, // Asia/Muscat
    {  610, 2554,  194262,  4, }, // Asia/Nicosia
    {  610, 8675,  194266,  8, }, // Asia/Novokuznetsk
    {  610, 4135,  194274,  7, }, // Asia/Novosibirsk
    {  610, 4229,    9670,  4, }, // Asia/Omsk
    {  610, 7145,    9674,  3, }, // Asia/Oral
    {  610, 6961,  194281,  6, }, // Asia/Phnom_Penh
    {  610, 8693,  194287,  6, }, // Asia/Pontianak
    {  610, 4214,    9690,  7, }, // Asia/Pyongyang
    {  610, 5559,  194293,  3, }, // Asia/Qatar
    {  610, 8708,    9700,  5, }, // Asia/Qostanay
    {  610, 4269,  194296,  5, }, // Asia/Qyzylorda
    {  610, 3584,  194301,  3, }, // Asia/Riyadh
    {  610, 4408,  194304,  4, }, // Asia/Sakhalin
    {  610, 7245,    9719,  6, }, // Asia/Samarkand
    {  610, 3279,  194308,  3, }, // Asia/Seoul
    {  610, 1053,  194311,  5, }, // Asia/Shanghai
    {  610, 3300,    9732,  5, }, // Asia/Singapore
    {  610, 4297,  194316,  9, }, // Asia/Srednekolymsk
    {  610, 3263,  194325,  4, }, // Asia/Taipei
    {  610, 4781,  194329,  5, }, // Asia/Tashkent
    {  610, 3915,  194334,  4, }, // Asia/Tbilisi
    {  610, 2772,  194338,  4, }, // Asia/Tehran
    {  610, 1317,    9764,  3, }, // Asia/Thimphu
    {  610, 2821,  194342,  3, }, // Asia/Tokyo
    {  610, 4572,    9770,  4, }, // Asia/Tomsk
    {  610, 1021,  194345,  7, }, // Asia/Ulaanbaatar
    {  610, 1159,    9782,  4, }, // Asia/Urumqi
    {  610, 8722,    9786,  6, }, // Asia/Ust-Nera
    {  610, 6977,  194352,  6, }, // Asia/Vientiane
    {  610, 4710,    9797,  7, }, // Asia/Vladivostok
    {  610, 4828,    9804,  5, }, // Asia/Yakutsk
    {  610, 1235,    9809,  4, }, // Asia/Yangon
    {  610, 3883,    9813,  8, }, // Asia/Yekaterinburg
    {  610, 3747,  194358,  4, }, // Asia/Yerevan
    {  610, 3663,  194362,  4, }, // Atlantic/Azores
    {  610, 5671,  194366,  4, }, // Atlantic/Bermuda
    {  610, 6628,    9833,  3, }, // Atlantic/Canary
    {  610, 3727,  194370,  6, }, // Atlantic/Cape_Verde
    {  610, 1395,  194376,  7, }, // Atlantic/Faroe
    {  610, 8736,  194383,  4, }, // Atlantic/Madeira
    {  610, 2748,  194387,  6, }, // Atlantic/Reykjavik
    {  610, 7272,  194393,  8, }, // Atlantic/South_Georgia
    {  610, 6821,  194401,  8, }, // Atlantic/St_Helena
    {  610, 6644,  194409,  4, }, // Atlantic/Stanley
    {  610, 1670,  194413,  5, }, // Australia/Adelaide
    {  610, 1635,  194418,  5, }, // Australia/Brisbane
    {  610, 1799,    9885,  7, }, // Australia/Broken_Hill
    {  610, 1583,    9892,  4, }, // Australia/Darwin
    {  610, 3637,  194423,  3, }, // Australia/Eucla
    {  610, 1516,    9899,  4, }, // Australia/Hobart
    {  610, 8753,    9903,  5, }, // Australia/Lindeman
    {  610, 1547,  194426,  6, }, // Australia/Lord_Howe
    {  610, 1727,  194432,  5, }, // Australia/Melbourne
    {  610, 1762,    9919,  3, }, // Australia/Perth
    {  610, 1463,    9922,  3, }, // Australia/Sydney
    {  610, 6441,  194437,  7, }, // Europe/Amsterdam
    {  610, 6216,  194444,  4, }, // Europe/Andorra
    {  610, 3620,  194448,  6, }, // Europe/Astrakhan
    {  610, 2300,  194454,  4, }, // Europe/Athens
    {  610, 6336,  194458,  5, }, // Europe/Belgrade
    {  610, 4757,    9952,  4, }, // Europe/Berlin
    {  610, 6488,    9956,  6, }, // Europe/Bratislava
    {  610, 2167,  194463,  5, }, // Europe/Brussels
    {  610, 3928,  194468,  5, }, // Europe/Bucharest
    {  610, 3806,  194473,  5, }, // Europe/Budapest
    {  610, 8772,    9977,  5, }, // Europe/Busingen
    {  610, 2583,  194478,  4, }, // Europe/Chisinau
    {  610, 6289,    9986,  6, }, // Europe/Copenhagen
    {  610, 2338,    9992,  4, }, // Europe/Dublin
    {  610, 6307,    9996,  6, }, // Europe/Gibraltar
    {  610, 8788,  194482,  4, }, // Europe/Guernsey
    {  610, 6612,   10006,  5, }, // Europe/Helsinki
    {  610, 8804,  194486,  9, }, // Europe/Isle_of_Man
    {  610, 1130,   10021,  6, }, // Europe/Istanbul
    {  610, 8823,  194495,  3, }, // Europe/Jersey
    {  610, 3979,   10030,  7, }, // Europe/Kaliningrad
    {  610, 8929,   10037,  3, }, // Europe/Kirov
    {  610, 2527,  194498,  3, }, // Europe/Kyiv
    {  610, 3213,   10043,  4, }, // Europe/Lisbon
    {  610, 6506,  194501,  6, }, // Europe/Ljubljana
    {  610, 2501,  194507,  4, }, // Europe/London
    {  610, 6365,   10057,  7, }, // Europe/Luxembourg
    {  610, 4892,   10064,  4, }, // Europe/Madrid
    {  610, 6397,   10068,  3, }, // Europe/Malta
    {  610, 6582,  194511,  5, }, // Europe/Mariehamn
    {  610, 3693,   10076,  4, }, // Europe/Minsk
    {  610, 6410,   10080,  3, }, // Europe/Monaco
    {  610, 3537,   10083,  3, }, // Europe/Moscow
    {  610, 6458,   10086,  3, }, // Europe/Oslo
    {  610, 4284,   10089,  3, }, // Europe/Paris
    {  610, 6424,  194516,  5, }, // Europe/Podgorica
    {  610, 6275,   10097,  3, }, // Europe/Prague
    {  610, 8837,   10100,  2, }, // Europe/Riga
    {  610, 6324,  194382,  2, }, // Europe/Rome
    {  610, 4331,   10104,  3, }, // Europe/Samara
    {  610, 6470,   10107,  6, }, // Europe/San_Marino
    {  610, 6245,  194521,  4, }, // Europe/Sarajevo
    {  610, 4451,   10117,  4, }, // Europe/Saratov
    {  610, 8849,   10121,  6, }, // Europe/Simferopol
    {  610, 6383,   10127,  4, }, // Europe/Skopje
    {  610, 6599,  194525,  3, }, // Europe/Sofia
    {  610, 6523,   10134,  6, }, // Europe/Stockholm
    {  610, 8867,   10140,  3, }, // Europe/Tallinn
    {  610, 6202,  194528,  3, }, // Europe/Tirane
    {  610, 8882,   10146,  7, }, // Europe/Ulyanovsk
    {  610, 6352,   10153,  3, }, // Europe/Vaduz
    {  610, 6567,   10156,  4, }, // Europe/Vatican
    {  610, 6231,  194531,  3, }, // Europe/Vienna
    {  610, 8899,  194534,  5, }, // Europe/Vilnius
    {  610, 4727,  194539,  6, }, // Europe/Volgograd
    {  610, 3190,  194545,  4, }, // Europe/Warsaw
    {  610, 6261,  194549,  4, }, // Europe/Zagreb
    {  610, 6540,   10180,  3, }, // Europe/Zurich
    {  610, 5078,  194553,  7, }, // Indian/Antananarivo
    {  610, 6947,   10190,  3, }, // Indian/Chagos
    {  610, 6046,  194560,  5, }, // Indian/Christmas
    {  610, 6063,   10195,  3, }, // Indian/Cocos
    {  610, 5029,   10198,  3, }, // Indian/Comoro
    {  610, 6661,  194565,  5, }, // Indian/Kerguelen
    {  610, 7260,   10206,  2, }, // Indian/Mahe
    {  610, 7050,  194570,  5, }, // Indian/Maldives
    {  610, 4069,  194575,  5, }, // Indian/Mauritius
    {  610, 5098,  194580,  3, }, // Indian/Mayotte
    {  610, 7211,  194583,  5, }, // Indian/Reunion
    {  610, 4422,  194588,  3, }, // Pacific/Apia
    {  610,  932,   10228,  5, }, // Pacific/Auckland
    {  610, 3706,  194591,  5, }, // Pacific/Bougainville
    {  610, 3015,   10239,  3, }, // Pacific/Chatham
    {  610, 3157,   10242,  2, }, // Pacific/Chuuk
    {  610, 2237,  194596,  7, }, // Pacific/Easter
    {  610, 7360,  194603,  3, }, // Pacific/Efate
    {  610, 7327,   10250,  4, }, // Pacific/Fakaofo
    {  610, 3902,   10254,  2, }, // Pacific/Fiji
    {  610, 7343,  194606,  4, }, // Pacific/Funafuti
    {  610, 6678,   10260,  5, }, // Pacific/Galapagos
    {  610, 6696,  194610,  5, }, // Pacific/Gambier
    {  610, 3822,  194615,  6, }, // Pacific/Guadalcanal
    {  610, 6018,  194621,  2, }, // Pacific/Guam
    {  610, 2723,   10280,  4, }, // Pacific/Honolulu
    {  610, 3049,  194623,  6, }, // Pacific/Kanton
    {  610, 3998,   10290,  5, }, // Pacific/Kiritimati
    {  610, 7019,   10295,  3, }, // Pacific/Kosrae
    {  610, 2842,   10298,  5, }, // Pacific/Kwajalein
    {  610, 4877,  194629,  3, }, // Pacific/Majuro
    {  610, 4051,  194632,  5, }, // Pacific/Marquesas
    {  610, 8914,   10312,  4, }, // Pacific/Midway
    {  610, 7084,   10316,  3, }, // Pacific/Nauru
    {  610, 7132,    8997,  2, }, // Pacific/Niue
    {  610, 4168,   10321,  5, }, // Pacific/Norfolk
    {  610, 7098,  194637,  3, }, // Pacific/Noumea
    {  610, 3126,   10330,  5, }, // Pacific/Pago_Pago
    {  610, 7155,   10335,  3, }, // Pacific/Palau
    {  610, 7194,  194640,  5, }, // Pacific/Pitcairn
    {  610, 3096,  194645,  4, }, // Pacific/Pohnpei
    {  610, 4807,  194649,  8, }, // Pacific/Port_Moresby
    {  610, 6076,   10357,  5, }, // Pacific/Rarotonga
    {  610, 6031,  194657,  4, }, // Pacific/Saipan
    {  610, 7312,   10366,  3, }, // Pacific/Tahiti
    {  610, 6712,   10369,  3, }, // Pacific/Tarawa
    {  610, 4583,   10372,  5, }, // Pacific/Tongatapu
    {  610, 7392,  194661,  2, }, // Pacific/Wake
    {  610, 7405,   10379,  3, }, // Pacific/Wallis
    {  612,   14,       0,  6, }, // Africa/Asmara Tok Pisin/Latin/Papua New Guinea
    {  612, 4435,       6,  8, }, // Africa/Sao_Tome
    {  612, 4252,      14,  8, }, // America/Asuncion
    {  612,  237,      22,  8, }, // America/Atikokan
    {  612, 7623,      30, 17, }, // America/Bahia_Banderas
    {  612, 3868,      47,  6, }, // America/Cancun
    {  612, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  612, 5723,      66,  7, }, // America/Curacao
    {  612,  481,      73, 13, }, // America/Indiana/Knox
    {  612, 7961,      86, 16, }, // America/Indiana/Marengo
    {  612, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  612, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  612, 8038,     139, 14, }, // America/Indiana/Vevay
    {  612, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  612, 8086,     171, 16, }, // America/Indiana/Winamac
    {  612, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  612, 5932,     207, 22, }, // America/Lower_Princes
    {  612, 8220,     229,  6, }, // America/Merida
    {  612, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  612, 8329,     255, 20, }, // America/North_Dakota/Center
    {  612, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  612,  393,     298,  4, }, // America/Nuuk
    {  612, 6899,     302, 16, }, // America/Scoresbysund
    {  612, 8508,     318, 14, }, // America/St_Barthelemy
    {  612, 2061,     332, 10, }, // America/St_Johns
    {  612, 5863,     342,  9, }, // America/St_Kitts
    {  612, 5880,     351,  9, }, // America/St_Lucia
    {  612,  855,     360, 10, }, // America/St_Thomas
    {  612, 5913,     370, 11, }, // America/St_Vincent
    {  612, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  612, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  612, 1185,     413,  9, }, // Asia/Kathmandu
    {  612,  992,     422,  7, }, // Asia/Kolkata
    {  612, 1211,     429,  5, }, // Asia/Macau
    {  612, 1235,     442,  6, }, // Asia/Yangon
    {  612, 1395,     448,  5, }, // Atlantic/Faroe
    {  612, 6821,     453, 10, }, // Atlantic/St_Helena
    {  612, 2527,     463,  4, }, // Europe/Kyiv
    {  612, 7211,     467,  7, }, // Indian/Reunion
    {  612, 3157,     474,  5, }, // Pacific/Chuuk
    {  612, 3096,     479,  7, }, // Pacific/Pohnpei
    {  613,   14,       0,  6, }, // Africa/Asmara Tongan/Latin/Tonga
    {  613, 4435,   58598,  8, }, // Africa/Sao_Tome
    {  613, 4252,      14,  8, }, // America/Asuncion
    {  613,  237,      22,  8, }, // America/Atikokan
    {  613, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  613, 3868,   33182,  6, }, // America/Cancun
    {  613, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  613, 5723,      66,  7, }, // America/Curacao
    {  613,  481,      73, 13, }, // America/Indiana/Knox
    {  613, 7961,      86, 16, }, // America/Indiana/Marengo
    {  613, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  613, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  613, 8038,     139, 14, }, // America/Indiana/Vevay
    {  613, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  613, 8086,     171, 16, }, // America/Indiana/Winamac
    {  613, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  613, 5932,     207, 22, }, // America/Lower_Princes
    {  613, 8220,    7008,  6, }, // America/Merida
    {  613, 2379,  194663,  9, }, // America/New_York
    {  613, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  613, 8329,     255, 20, }, // America/North_Dakota/Center
    {  613, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  613,  393,     298,  4, }, // America/Nuuk
    {  613, 6899,     302, 16, }, // America/Scoresbysund
    {  613, 8508,     318, 14, }, // America/St_Barthelemy
    {  613, 2061,     332, 10, }, // America/St_Johns
    {  613, 5863,     342,  9, }, // America/St_Kitts
    {  613, 5880,     351,  9, }, // America/St_Lucia
    {  613,  855,     360, 10, }, // America/St_Thomas
    {  613, 5913,     370, 11, }, // America/St_Vincent
    {  613, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  613, 4795,  194672,  8, }, // Asia/Hebron
    {  613, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  613, 1290,  194680, 10, }, // Asia/Jerusalem
    {  613, 1185,     413,  9, }, // Asia/Kathmandu
    {  613,  992,     422,  7, }, // Asia/Kolkata
    {  613, 1211,    4203,  5, }, // Asia/Macau
    {  613, 1235,   58486,  7, }, // Asia/Yangon
    {  613, 1395,     448,  5, }, // Atlantic/Faroe
    {  613, 6821,     453, 10, }, // Atlantic/St_Helena
    {  613, 1670,  194690,  8, }, // Australia/Adelaide
    {  613, 1635,  194698, 10, }, // Australia/Brisbane
    {  613, 1727,  194708,  9, }, // Australia/Melbourne
    {  613, 1463,  194717,  4, }, // Australia/Sydney
    {  613, 2300,  194721,  8, }, // Europe/Athens
    {  613, 2527,    8030,  4, }, // Europe/Kyiv
    {  613, 2501,  194729,  8, }, // Europe/London
    {  613, 3537,  194737,  7, }, // Europe/Moscow
    {  613, 4284,  194744,  6, }, // Europe/Paris
    {  613, 6324,  194750,  4, }, // Europe/Rome
    {  613, 6567,   63941,  8, }, // Europe/Vatican
    {  613, 7211,     467,  7, }, // Indian/Reunion
    {  613,  932,  194754,  9, }, // Pacific/Auckland
    {  613, 3706,  194763, 10, }, // Pacific/Bougainville
    {  613, 3157,  194773,  4, }, // Pacific/Chuuk
    {  613, 2237,  194777,  7, }, // Pacific/Easter
    {  613, 3902,  194784,  4, }, // Pacific/Fiji
    {  613, 6696,  194788,  7, }, // Pacific/Gambier
    {  613, 3822,  194795, 11, }, // Pacific/Guadalcanal
    {  613, 6018,  194806,  5, }, // Pacific/Guam
    {  613, 2723,    4493,  8, }, // Pacific/Honolulu
    {  613, 3049,  194811, 10, }, // Pacific/Kanton
    {  613, 3998,  194821, 10, }, // Pacific/Kiritimati
    {  613, 7019,  194831,  7, }, // Pacific/Kosrae
    {  613, 2842,  194838,  9, }, // Pacific/Kwajalein
    {  613, 4877,  194847,  6, }, // Pacific/Majuro
    {  613, 4051,  194853,  9, }, // Pacific/Marquesas
    {  613, 8914,  194862,  7, }, // Pacific/Midway
    {  613, 7084,  194869,  5, }, // Pacific/Nauru
    {  613, 7132,  194874,  4, }, // Pacific/Niue
    {  613, 4168,  194878,  7, }, // Pacific/Norfolk
    {  613, 7098,  124531,  5, }, // Pacific/Noumea
    {  613, 3126,  194885, 10, }, // Pacific/Pago_Pago
    {  613, 7194,  194895,  8, }, // Pacific/Pitcairn
    {  613, 3096,  194903,  6, }, // Pacific/Pohnpei
    {  613, 4807,  194909, 17, }, // Pacific/Port_Moresby
    {  613, 6076,  194926,  9, }, // Pacific/Rarotonga
    {  613, 6031,  194935,  8, }, // Pacific/Saipan
    {  613, 7312,  194943,  6, }, // Pacific/Tahiti
    {  613, 6712,  194949,  6, }, // Pacific/Tarawa
    {  613, 7392,  194955,  4, }, // Pacific/Wake
    {  613, 7405,  194959,  5, }, // Pacific/Wallis
    {  614,   14,       0,  6, }, // Africa/Asmara Tsonga/Latin/South Africa
    {  614, 4435,       6,  8, }, // Africa/Sao_Tome
    {  614, 4252,      14,  8, }, // America/Asuncion
    {  614,  237,      22,  8, }, // America/Atikokan
    {  614, 7623,      30, 17, }, // America/Bahia_Banderas
    {  614, 3868,      47,  6, }, // America/Cancun
    {  614, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  614, 5723,      66,  7, }, // America/Curacao
    {  614,  481,      73, 13, }, // America/Indiana/Knox
    {  614, 7961,      86, 16, }, // America/Indiana/Marengo
    {  614, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  614, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  614, 8038,     139, 14, }, // America/Indiana/Vevay
    {  614, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  614, 8086,     171, 16, }, // America/Indiana/Winamac
    {  614, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  614, 5932,     207, 22, }, // America/Lower_Princes
    {  614, 8220,     229,  6, }, // America/Merida
    {  614, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  614, 8329,     255, 20, }, // America/North_Dakota/Center
    {  614, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  614,  393,     298,  4, }, // America/Nuuk
    {  614, 6899,     302, 16, }, // America/Scoresbysund
    {  614, 8508,     318, 14, }, // America/St_Barthelemy
    {  614, 2061,     332, 10, }, // America/St_Johns
    {  614, 5863,     342,  9, }, // America/St_Kitts
    {  614, 5880,     351,  9, }, // America/St_Lucia
    {  614,  855,     360, 10, }, // America/St_Thomas
    {  614, 5913,     370, 11, }, // America/St_Vincent
    {  614, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  614, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  614, 1185,     413,  9, }, // Asia/Kathmandu
    {  614,  992,     422,  7, }, // Asia/Kolkata
    {  614, 1211,     429,  5, }, // Asia/Macau
    {  614, 1235,     442,  6, }, // Asia/Yangon
    {  614, 1395,     448,  5, }, // Atlantic/Faroe
    {  614, 6821,     453, 10, }, // Atlantic/St_Helena
    {  614, 2527,     463,  4, }, // Europe/Kyiv
    {  614, 7211,     467,  7, }, // Indian/Reunion
    {  614, 3157,     474,  5, }, // Pacific/Chuuk
    {  614, 3096,     479,  7, }, // Pacific/Pohnpei
    {  615,   14,       0,  6, }, // Africa/Asmara Tswana/Latin/South Africa
    {  615, 4435,       6,  8, }, // Africa/Sao_Tome
    {  615, 4252,      14,  8, }, // America/Asuncion
    {  615,  237,      22,  8, }, // America/Atikokan
    {  615, 7623,      30, 17, }, // America/Bahia_Banderas
    {  615, 3868,      47,  6, }, // America/Cancun
    {  615, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  615, 5723,      66,  7, }, // America/Curacao
    {  615,  481,      73, 13, }, // America/Indiana/Knox
    {  615, 7961,      86, 16, }, // America/Indiana/Marengo
    {  615, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  615, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  615, 8038,     139, 14, }, // America/Indiana/Vevay
    {  615, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  615, 8086,     171, 16, }, // America/Indiana/Winamac
    {  615, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  615, 5932,     207, 22, }, // America/Lower_Princes
    {  615, 8220,     229,  6, }, // America/Merida
    {  615, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  615, 8329,     255, 20, }, // America/North_Dakota/Center
    {  615, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  615,  393,     298,  4, }, // America/Nuuk
    {  615, 6899,     302, 16, }, // America/Scoresbysund
    {  615, 8508,     318, 14, }, // America/St_Barthelemy
    {  615, 2061,     332, 10, }, // America/St_Johns
    {  615, 5863,     342,  9, }, // America/St_Kitts
    {  615, 5880,     351,  9, }, // America/St_Lucia
    {  615,  855,     360, 10, }, // America/St_Thomas
    {  615, 5913,     370, 11, }, // America/St_Vincent
    {  615, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  615, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  615, 1185,     413,  9, }, // Asia/Kathmandu
    {  615,  992,     422,  7, }, // Asia/Kolkata
    {  615, 1211,     429,  5, }, // Asia/Macau
    {  615, 1235,     442,  6, }, // Asia/Yangon
    {  615, 1395,     448,  5, }, // Atlantic/Faroe
    {  615, 6821,     453, 10, }, // Atlantic/St_Helena
    {  615, 2527,     463,  4, }, // Europe/Kyiv
    {  615, 7211,     467,  7, }, // Indian/Reunion
    {  615, 3157,     474,  5, }, // Pacific/Chuuk
    {  615, 3096,     479,  7, }, // Pacific/Pohnpei
    {  617, 6760,    4531,  4, }, // Africa/Accra Turkish/Latin/Turkey
    {  617, 7420,  194964,  7, }, // Africa/Algiers
    {  617,   14,       0,  6, }, // Africa/Asmara
    {  617, 5287,   21216,  9, }, // Africa/Brazzaville
    {  617, 2320,  194971,  6, }, // Africa/Cairo
    {  617, 4117,   33144, 10, }, // Africa/Casablanca
    {  617, 7449,  117138,  5, }, // Africa/Ceuta
    {  617, 6773,    6249,  7, }, // Africa/Conakry
    {  617, 5130,  194977, 10, }, // Africa/Dar_es_Salaam
    {  617, 5043,   21248,  6, }, // Africa/Djibouti
    {  617, 5166,  194987,  5, }, // Africa/El_Aaiun
    {  617, 4499,  194992,  4, }, // Africa/Juba
    {  617, 4524,   63386,  6, }, // Africa/Khartoum
    {  617, 5306,   21296,  7, }, // Africa/Kinshasa
    {  617, 5336,    6311,  8, }, // Africa/Libreville
    {  617, 5113,  194996,  8, }, // Africa/Mogadishu
    {  617, 4435,       6,  8, }, // Africa/Sao_Tome
    {  617, 2866,   98371,  7, }, // Africa/Tripoli
    {  617, 6554,  195004,  5, }, // Africa/Tunis
    {  617, 4252,      14,  8, }, // America/Asuncion
    {  617,  237,      22,  8, }, // America/Atikokan
    {  617, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  617, 3868,   33182,  6, }, // America/Cancun
    {  617, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  617, 5422,   21503, 10, }, // America/Costa_Rica
    {  617, 5723,      66,  7, }, // America/Curacao
    {  617, 5739,   21548,  8, }, // America/Dominica
    {  617,  481,      73, 13, }, // America/Indiana/Knox
    {  617, 7961,      86, 16, }, // America/Indiana/Marengo
    {  617, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  617, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  617, 8038,     139, 14, }, // America/Indiana/Vevay
    {  617, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  617, 8086,     171, 16, }, // America/Indiana/Winamac
    {  617, 2799,    3877,  7, }, // America/Jamaica
    {  617, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  617, 5932,     207, 22, }, // America/Lower_Princes
    {  617, 8220,    7008,  6, }, // America/Merida
    {  617, 8301,  195009, 20, }, // America/North_Dakota/Beulah
    {  617, 8329,  195029, 20, }, // America/North_Dakota/Center
    {  617, 8357,  195049, 23, }, // America/North_Dakota/New_Salem
    {  617,  393,     298,  4, }, // America/Nuuk
    {  617, 5843,  166548, 10, }, // America/Puerto_Rico
    {  617, 6899,     302, 16, }, // America/Scoresbysund
    {  617, 8508,  117379, 16, }, // America/St_Barthelemy
    {  617, 2061,     332, 10, }, // America/St_Johns
    {  617, 5863,     342,  9, }, // America/St_Kitts
    {  617, 5880,     351,  9, }, // America/St_Lucia
    {  617,  855,     360, 10, }, // America/St_Thomas
    {  617, 5913,     370, 11, }, // America/St_Vincent
    {  617, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  617, 7295,   62088,  5, }, // Antarctica/Syowa
    {  617, 5395,   22180,  6, }, // Asia/Almaty
    {  617, 5511,   22186,  6, }, // Asia/Anadyr
    {  617, 5523,  195072,  5, }, // Asia/Aqtau
    {  617, 5368,   63474,  6, }, // Asia/Aqtobe
    {  617,  964,  195077,  7, }, // Asia/Ashgabat
    {  617, 8624,  195084,  6, }, // Asia/Atyrau
    {  617, 3607,  195090,  6, }, // Asia/Baghdad
    {  617, 5534,  195096,  7, }, // Asia/Bahrain
    {  617, 3653,  195103,  4, }, // Asia/Baku
    {  617, 4086,   22229,  6, }, // Asia/Beirut
    {  617, 3778,   22235,  6, }, // Asia/Bishkek
    {  617, 4601,    7513,  4, }, // Asia/Chita
    {  617, 4511,    5519,  7, }, // Asia/Colombo
    {  617, 4540,  117461,  3, }, // Asia/Damascus
    {  617, 1093,   91428,  5, }, // Asia/Dhaka
    {  617, 6159,  195107,  7, }, // Asia/Dushanbe
    {  617, 8636,  195114, 10, }, // Asia/Famagusta
    {  617, 8651,  195124,  5, }, // Asia/Gaza
    {  617, 4795,  195129,  8, }, // Asia/Hebron
    {  617, 1259,  195137, 17, }, // Asia/Ho_Chi_Minh
    {  617, 4184,   22301,  7, }, // Asia/Irkutsk
    {  617, 7006,   22308,  7, }, // Asia/Jakarta
    {  617, 1290,  195154,  5, }, // Asia/Jerusalem
    {  617, 3560,   22332,  5, }, // Asia/Kabul
    {  617, 4316,   22337,  8, }, // Asia/Kamchatka
    {  617, 4239,    7569,  6, }, // Asia/Karachi
    {  617, 1185,    5593,  8, }, // Asia/Kathmandu
    {  617, 8661,   33392,  7, }, // Asia/Khandyga
    {  617,  992,  195159,  7, }, // Asia/Kolkata
    {  617, 5976,    7601,  6, }, // Asia/Kuching
    {  617, 5547,  195166,  6, }, // Asia/Kuwait
    {  617, 1211,    7613,  5, }, // Asia/Macau
    {  617, 6920,   26750,  6, }, // Asia/Muscat
    {  617, 2554,  195172,  7, }, // Asia/Nicosia
    {  617, 5559,    4214,  5, }, // Asia/Qatar
    {  617, 8708,     434,  8, }, // Asia/Qostanay
    {  617, 4269,  195179,  9, }, // Asia/Qyzylorda
    {  617, 3584,   20756,  5, }, // Asia/Riyadh
    {  617, 4408,   33444,  7, }, // Asia/Sakhalin
    {  617, 7245,  117599,  9, }, // Asia/Samarkand
    {  617, 3279,    7680,  4, }, // Asia/Seoul
    {  617, 1053,  117612,  7, }, // Asia/Shanghai
    {  617, 3300,   20782,  8, }, // Asia/Singapore
    {  617, 4781,  117633,  7, }, // Asia/Tashkent
    {  617, 3915,   78108,  6, }, // Asia/Tbilisi
    {  617, 2772,  195188,  6, }, // Asia/Tehran
    {  617, 1021,  195194, 10, }, // Asia/Ulaanbaatar
    {  617, 1159,   22475,  6, }, // Asia/Urumqi
    {  617, 1235,     442,  6, }, // Asia/Yangon
    {  617, 3747,  127970,  6, }, // Asia/Yerevan
    {  617, 3663,  195204, 12, }, // Atlantic/Azores
    {  617, 6628,  195216, 15, }, // Atlantic/Canary
    {  617, 1395,     448,  5, }, // Atlantic/Faroe
    {  617, 8736,  195231, 15, }, // Atlantic/Madeira
    {  617, 7272,  195246, 13, }, // Atlantic/South_Georgia
    {  617, 6821,     453, 10, }, // Atlantic/St_Helena
    {  617, 1463,   22607,  6, }, // Australia/Sydney
    {  617, 3620,   33512,  8, }, // Europe/Astrakhan
    {  617, 2300,   33520,  5, }, // Europe/Athens
    {  617, 6336,   26883,  7, }, // Europe/Belgrade
    {  617, 2167,  195259,  7, }, // Europe/Brussels
    {  617, 3928,  195266,  6, }, // Europe/Bucharest
    {  617, 3806,  195272,  9, }, // Europe/Budapest
    {  617, 8772,   26905,  8, }, // Europe/Busingen
    {  617, 2583,  195281,  7, }, // Europe/Chisinau
    {  617, 6289,    4357,  8, }, // Europe/Copenhagen
    {  617, 6307,  195288, 11, }, // Europe/Gibraltar
    {  617, 8804,  195299,  9, }, // Europe/Isle_of_Man
    {  617, 1130,   22693,  8, }, // Europe/Istanbul
    {  617, 2527,    8030,  4, }, // Europe/Kyiv
    {  617, 3213,  157592,  6, }, // Europe/Lisbon
    {  617, 2501,   95241,  6, }, // Europe/London
    {  617, 6365,  195308, 10, }, // Europe/Luxembourg
    {  617, 6410,    5926,  6, }, // Europe/Monaco
    {  617, 3537,   64333,  7, }, // Europe/Moscow
    {  617, 6275,    8083,  4, }, // Europe/Prague
    {  617, 6324,    5946,  4, }, // Europe/Rome
    {  617, 6245,  117898, 10, }, // Europe/Sarajevo
    {  617, 6383,  195318,  5, }, // Europe/Skopje
    {  617, 6599,   99201,  5, }, // Europe/Sofia
    {  617, 6523,    8124,  8, }, // Europe/Stockholm
    {  617, 6202,    8137,  5, }, // Europe/Tirane
    {  617, 6567,    5978,  7, }, // Europe/Vatican
    {  617, 6231,  117930,  6, }, // Europe/Vienna
    {  617, 3190,  195323,  7, }, // Europe/Warsaw
    {  617, 6540,  195330,  5, }, // Europe/Zurich
    {  617, 5029,    8187,  5, }, // Indian/Comoro
    {  617, 7050,  195335,  9, }, // Indian/Maldives
    {  617, 7211,     467,  7, }, // Indian/Reunion
    {  617, 3157,     474,  5, }, // Pacific/Chuuk
    {  617, 2237,  195344, 14, }, // Pacific/Easter
    {  617, 2723,    4493,  8, }, // Pacific/Honolulu
    {  617, 3049,    4501,  9, }, // Pacific/Kanton
    {  617, 4051,  195358, 14, }, // Pacific/Marquesas
    {  617, 3096,     479,  7, }, // Pacific/Pohnpei
    {  619, 6788,  195372,  6, }, // Africa/Abidjan Turkmen/Latin/Turkmenistan
    {  619, 6760,   21174,  5, }, // Africa/Accra
    {  619, 5059,   64509, 11, }, // Africa/Addis_Ababa
    {  619, 7420,   57447,  5, }, // Africa/Algiers
    {  619,   14,       0,  6, }, // Africa/Asmara
    {  619, 5257,   26348,  5, }, // Africa/Bangui
    {  619, 7435,    6195,  5, }, // Africa/Bissau
    {  619, 4957,  195378,  8, }, // Africa/Blantyre
    {  619, 5287,  195386,  9, }, // Africa/Brazzaville
    {  619, 2320,    4572,  4, }, // Africa/Cairo
    {  619, 4117,    4577, 10, }, // Africa/Casablanca
    {  619, 7449,   21231,  5, }, // Africa/Ceuta
    {  619, 6773,    6249,  7, }, // Africa/Conakry
    {  619, 5130,    6256, 12, }, // Africa/Dar_es_Salaam
    {  619, 5043,  158372,  6, }, // Africa/Djibouti
    {  619, 5243,   21254,  5, }, // Africa/Douala
    {  619, 5166,  195395,  7, }, // Africa/El_Aaiun
    {  619, 6853,    6282,  7, }, // Africa/Freetown
    {  619, 4479,  195402, 12, }, // Africa/Johannesburg
    {  619, 4524,   63386,  6, }, // Africa/Khartoum
    {  619, 5306,   21296,  7, }, // Africa/Kinshasa
    {  619, 5336,  195414,  8, }, // Africa/Libreville
    {  619, 4939,   21308,  9, }, // Africa/Lubumbashi
    {  619, 5113,  195422,  8, }, // Africa/Mogadishu
    {  619, 7034,  195430,  9, }, // Africa/Monrovia
    {  619, 3853,  195439,  7, }, // Africa/Nairobi
    {  619, 5271,  195446,  6, }, // Africa/Ndjamena
    {  619, 5354,  195452,  6, }, // Africa/Niamey
    {  619, 6803,  195458,  7, }, // Africa/Nouakchott
    {  619, 6727,   26402,  8, }, // Africa/Ouagadougou
    {  619, 5225,  195465, 10, }, // Africa/Porto-Novo
    {  619, 4435,  195475,  8, }, // Africa/Sao_Tome
    {  619, 4152,   56676,  7, }, // Africa/Windhoek
    {  619,  132,  195483, 10, }, // America/Adak
    {  619, 3346,  124894,  8, }, // America/Anchorage
    {  619, 5607,  195493,  7, }, // America/Anguilla
    {  619, 4554,  195500,  9, }, // America/Araguaina
    {  619,  166,  195509, 12, }, // America/Argentina/Buenos_Aires
    {  619,   91,    6400,  9, }, // America/Argentina/Catamarca
    {  619,  270,  195521,  7, }, // America/Argentina/Cordoba
    {  619,  441,  195528,  5, }, // America/Argentina/Jujuy
    {  619, 7462,  195533,  8, }, // America/Argentina/La_Rioja
    {  619,  565,   97505,  7, }, // America/Argentina/Mendoza
    {  619, 7489,    6429, 11, }, // America/Argentina/Rio_Gallegos
    {  619, 7544,    6445,  8, }, // America/Argentina/San_Juan
    {  619, 5580,  195541,  8, }, // America/Argentina/San_Luis
    {  619, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {  619, 7597,  195549,  6, }, // America/Argentina/Ushuaia
    {  619, 4252,  195555,  8, }, // America/Asuncion
    {  619,  237,      22,  8, }, // America/Atikokan
    {  619, 3679,  195563,  5, }, // America/Bahia
    {  619, 7623,  195568, 17, }, // America/Bahia_Banderas
    {  619, 7646,  122267,  5, }, // America/Belem
    {  619, 5407,    6496,  5, }, // America/Belize
    {  619, 7660,    6502, 12, }, // America/Blanc-Sablon
    {  619, 7681,  195585,  9, }, // America/Boa_Vista
    {  619, 7699,  195594,  5, }, // America/Boise
    {  619, 7713,  195599, 11, }, // America/Cambridge_Bay
    {  619, 7735,  195610, 12, }, // America/Campo_Grande
    {  619, 3868,    6564,  6, }, // America/Cancun
    {  619, 4694,    4792,  7, }, // America/Caracas
    {  619, 4345,  195622,  7, }, // America/Cayenne
    {  619, 5496,  195629, 14, }, // America/Cayman
    {  619, 2260,    6582,  6, }, // America/Chicago
    {  619, 7756,   21495,  8, }, // America/Chihuahua
    {  619, 7774,  195643, 13, }, // America/Ciudad_Juarez
    {  619, 5422,  195656, 10, }, // America/Costa_Rica
    {  619, 7796,    6618,  7, }, // America/Creston
    {  619, 3791,  195666,  6, }, // America/Cuiaba
    {  619, 5723,  195672,  8, }, // America/Curacao
    {  619, 7812,  195680, 12, }, // America/Danmarkshavn
    {  619, 7833,    6651,  6, }, // America/Dawson
    {  619, 7848,  195692, 11, }, // America/Dawson_Creek
    {  619,  805,  195703,  6, }, // America/Denver
    {  619, 3465,  195709,  7, }, // America/Detroit
    {  619, 5739,   21548,  8, }, // America/Dominica
    {  619, 7869,  195716,  8, }, // America/Eirunepe
    {  619, 5441,  157022,  8, }, // America/El_Salvador
    {  619, 7924,  195724,  9, }, // America/Glace_Bay
    {  619, 6881,  195733,  7, }, // America/Goose_Bay
    {  619, 4612,  195740, 10, }, // America/Grand_Turk
    {  619, 5786,  157030,  9, }, // America/Guadeloupe
    {  619, 3760,  134343,  9, }, // America/Guatemala
    {  619, 4841,  195750,  7, }, // America/Guayaquil
    {  619, 6932,  195757,  6, }, // America/Guyana
    {  619, 1939,  195763,  8, }, // America/Halifax
    {  619, 2281,  195771,  6, }, // America/Havana
    {  619, 7942,  195777,  8, }, // America/Hermosillo
    {  619,  481,  195785, 13, }, // America/Indiana/Knox
    {  619, 7961,      86, 16, }, // America/Indiana/Marengo
    {  619, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  619, 8012,    6843, 18, }, // America/Indiana/Tell_City
    {  619, 8038,  195798, 14, }, // America/Indiana/Vevay
    {  619, 8060,  195812, 16, }, // America/Indiana/Vincennes
    {  619, 8086,    5043, 16, }, // America/Indiana/Winamac
    {  619, 8110,  195828,  6, }, // America/Inuvik
    {  619,  660,    5059,  7, }, // America/Iqaluit
    {  619, 2799,  195834,  7, }, // America/Jamaica
    {  619, 5380,  122696,  5, }, // America/Juneau
    {  619,  521,  195841,  8, }, // America/Kentucky/Louisville
    {  619, 8125,  195849, 19, }, // America/Kentucky/Monticello
    {  619, 5704,  195868, 10, }, // America/Kralendijk
    {  619, 4376,  195878,  6, }, // America/La_Paz
    {  619, 3239,  195884, 11, }, // America/Los_Angeles
    {  619, 5932,  195895, 21, }, // America/Lower_Princes
    {  619, 8153,  195916,  6, }, // America/Maceio
    {  619, 5897,   61739,  6, }, // America/Marigot
    {  619, 5805,   26462,  9, }, // America/Martinique
    {  619, 8202,    7000,  8, }, // America/Menominee
    {  619, 8220,    7008,  6, }, // America/Merida
    {  619, 2949,  122829,  6, }, // America/Mexico_City
    {  619, 4391,    7031,  7, }, // America/Miquelon
    {  619, 8254,    7038,  7, }, // America/Moncton
    {  619, 8270,  195922,  9, }, // America/Monterrey
    {  619, 4098,  195931, 10, }, // America/Montevideo
    {  619, 5824,  158435,  9, }, // America/Montserrat
    {  619, 2379,  195941,  8, }, // America/New_York
    {  619, 8288,   21850,  3, }, // America/Nome
    {  619, 8301,  195949, 24, }, // America/North_Dakota/Beulah
    {  619, 8329,  195973, 24, }, // America/North_Dakota/Center
    {  619, 8357,  195997, 28, }, // America/North_Dakota/New_Salem
    {  619,  393,     298,  4, }, // America/Nuuk
    {  619, 8388,    7158,  7, }, // America/Ojinaga
    {  619, 2973,    7165,  6, }, // America/Phoenix
    {  619, 3945,  165882, 12, }, // America/Port-au-Prince
    {  619, 5954,  196025, 13, }, // America/Port_of_Spain
    {  619, 8404,  196038, 11, }, // America/Porto_Velho
    {  619, 5843,  196049, 11, }, // America/Puerto_Rico
    {  619, 4030,    7204, 12, }, // America/Punta_Arenas
    {  619, 8424,   93659, 12, }, // America/Rankin_Inlet
    {  619, 8445,  123023,  6, }, // America/Recife
    {  619, 1995,  196060,  7, }, // America/Regina
    {  619, 8460,  196067,  8, }, // America/Resolute
    {  619,  695,  196075, 10, }, // America/Rio_Branco
    {  619, 2201,  196085,  8, }, // America/Santiago
    {  619, 6111,    7244, 13, }, // America/Santo_Domingo
    {  619, 1878,  196093,  9, }, // America/Sao_Paulo
    {  619, 6899,  196102, 16, }, // America/Scoresbysund
    {  619, 8508,    7278, 13, }, // America/St_Barthelemy
    {  619, 2061,  196118,  9, }, // America/St_Johns
    {  619, 5863,  196127,  9, }, // America/St_Kitts
    {  619, 5880,  196136, 12, }, // America/St_Lucia
    {  619,  855,  196148, 10, }, // America/St_Thomas
    {  619, 5913,  196158, 12, }, // America/St_Vincent
    {  619, 8530,  196170, 12, }, // America/Swift_Current
    {  619, 5461,    5410, 11, }, // America/Tegucigalpa
    {  619, 5756,  123177,  4, }, // America/Thule
    {  619,  313,    7361,  7, }, // America/Tijuana
    {  619, 2093,  196182,  8, }, // America/Vancouver
    {  619, 2144,  196190,  8, }, // America/Whitehorse
    {  619, 8552,  196198,  7, }, // America/Yakutat
    {  619, 6001,  196205,  5, }, // Antarctica/Casey
    {  619, 6094,  196210,  6, }, // Antarctica/Davis
    {  619, 6133,  196216, 15, }, // Antarctica/DumontDUrville
    {  619, 8568,  196231,  8, }, // Antarctica/Macquarie
    {  619, 7066,   22153,  6, }, // Antarctica/Mawson
    {  619, 7113,  196239,  9, }, // Antarctica/McMurdo
    {  619, 7226,    5465,  6, }, // Antarctica/Rothera
    {  619, 7295,  196248,  5, }, // Antarctica/Syowa
    {  619, 8607,  123271,  4, }, // Antarctica/Troll
    {  619, 7374,    4029,  6, }, // Antarctica/Vostok
    {  619, 1429,  196253,  6, }, // Arctic/Longyearbyen
    {  619, 5523,    5491,  5, }, // Asia/Aqtau
    {  619, 5368,    7488,  6, }, // Asia/Aqtobe
    {  619,  964,   44190,  7, }, // Asia/Ashgabat
    {  619, 3607,  196259,  6, }, // Asia/Baghdad
    {  619, 5534,  196265,  7, }, // Asia/Bahrain
    {  619, 4086,  196272,  6, }, // Asia/Beirut
    {  619, 3778,   22235,  6, }, // Asia/Bishkek
    {  619, 5989,  196278,  6, }, // Asia/Brunei
    {  619, 4601,    7513,  4, }, // Asia/Chita
    {  619, 4511,    5519,  7, }, // Asia/Colombo
    {  619, 4540,    4083,  6, }, // Asia/Damascus
    {  619, 1093,   91428,  5, }, // Asia/Dhaka
    {  619, 3596,  196284,  5, }, // Asia/Dubai
    {  619, 6159,  195107,  7, }, // Asia/Dushanbe
    {  619, 4795,  196289,  6, }, // Asia/Hebron
    {  619, 1259,  196295,  7, }, // Asia/Ho_Chi_Minh
    {  619, 2704,  196302,  7, }, // Asia/Hong_Kong
    {  619, 4771,  196309,  4, }, // Asia/Hovd
    {  619, 6992,  196313,  8, }, // Asia/Jayapura
    {  619, 1290,  196321, 10, }, // Asia/Jerusalem
    {  619, 4316,   22337,  8, }, // Asia/Kamchatka
    {  619, 4239,    7569,  6, }, // Asia/Karachi
    {  619, 1185,    5593,  8, }, // Asia/Kathmandu
    {  619, 8661,   64214,  7, }, // Asia/Khandyga
    {  619,  992,   64221,  8, }, // Asia/Kolkata
    {  619, 4197,  196331, 11, }, // Asia/Krasnoyarsk
    {  619, 4859,    7589, 12, }, // Asia/Kuala_Lumpur
    {  619, 5976,    7601,  6, }, // Asia/Kuching
    {  619, 5547,  196342,  6, }, // Asia/Kuwait
    {  619, 1211,    7613,  5, }, // Asia/Macau
    {  619, 1349,    7618,  7, }, // Asia/Makassar
    {  619, 6920,   26750,  6, }, // Asia/Muscat
    {  619, 2554,  196348,  8, }, // Asia/Nicosia
    {  619, 8675,  196356, 11, }, // Asia/Novokuznetsk
    {  619, 4135,   78082, 11, }, // Asia/Novosibirsk
    {  619, 6961,   62386,  7, }, // Asia/Phnom_Penh
    {  619, 4214,  196367,  7, }, // Asia/Pyongyang
    {  619, 5559,    4214,  5, }, // Asia/Qatar
    {  619, 8708,  196374,  8, }, // Asia/Qostanay
    {  619, 4269,  196382,  9, }, // Asia/Qyzylorda
    {  619, 3584,  196391,  8, }, // Asia/Riyadh
    {  619, 4408,   33444,  7, }, // Asia/Sakhalin
    {  619, 3279,    7680,  4, }, // Asia/Seoul
    {  619, 1053,  196399,  6, }, // Asia/Shanghai
    {  619, 3300,   20782,  8, }, // Asia/Singapore
    {  619, 3263,  196405,  6, }, // Asia/Taipei
    {  619, 4781,  117633,  7, }, // Asia/Tashkent
    {  619, 2772,  196411,  6, }, // Asia/Tehran
    {  619, 1317,   91641,  5, }, // Asia/Thimphu
    {  619, 2821,    4238,  5, }, // Asia/Tokyo
    {  619, 1021,  147043, 10, }, // Asia/Ulaanbaatar
    {  619, 1159,   22475,  6, }, // Asia/Urumqi
    {  619, 6977,  196417,  8, }, // Asia/Vientiane
    {  619, 4710,    4243, 11, }, // Asia/Vladivostok
    {  619, 4828,  196425,  7, }, // Asia/Yakutsk
    {  619, 1235,  196432,  6, }, // Asia/Yangon
    {  619, 3883,  196438, 13, }, // Asia/Yekaterinburg
    {  619, 3747,  196451,  7, }, // Asia/Yerevan
    {  619, 3663,  196458, 12, }, // Atlantic/Azores
    {  619, 6628,  196470, 13, }, // Atlantic/Canary
    {  619, 3727,  196483, 10, }, // Atlantic/Cape_Verde
    {  619, 1395,  196493, 13, }, // Atlantic/Faroe
    {  619, 8736,  196506, 15, }, // Atlantic/Madeira
    {  619, 2748,  196521,  9, }, // Atlantic/Reykjavik
    {  619, 7272,  196530, 16, }, // Atlantic/South_Georgia
    {  619, 6821,  196546, 22, }, // Atlantic/St_Helena
    {  619, 6644,    7847,  6, }, // Atlantic/Stanley
    {  619, 1670,   20912,  8, }, // Australia/Adelaide
    {  619, 1635,  123836,  7, }, // Australia/Brisbane
    {  619, 1799,   93819, 10, }, // Australia/Broken_Hill
    {  619, 3637,  196568,  5, }, // Australia/Eucla
    {  619, 1547,  196573,  8, }, // Australia/Lord_Howe
    {  619, 1727,    7903,  7, }, // Australia/Melbourne
    {  619, 1762,   22603,  4, }, // Australia/Perth
    {  619, 1463,  196581,  6, }, // Australia/Sydney
    {  619, 3620,   33512,  8, }, // Europe/Astrakhan
    {  619, 2300,  196587,  5, }, // Europe/Athens
    {  619, 6336,   26883,  7, }, // Europe/Belgrade
    {  619, 6488,  196592, 10, }, // Europe/Bratislava
    {  619, 2167,  196602,  8, }, // Europe/Brussels
    {  619, 3928,  196610,  8, }, // Europe/Bucharest
    {  619, 3806,   22649,  8, }, // Europe/Budapest
    {  619, 8772,  196618,  9, }, // Europe/Busingen
    {  619, 2583,  196627,  8, }, // Europe/Chisinau
    {  619, 6289,  196635, 10, }, // Europe/Copenhagen
    {  619, 8788,  127105,  6, }, // Europe/Guernsey
    {  619, 8804,  196645,  9, }, // Europe/Isle_of_Man
    {  619, 1130,  124038,  7, }, // Europe/Istanbul
    {  619, 8823,  196654,  5, }, // Europe/Jersey
    {  619, 8929,   78158,  5, }, // Europe/Kirov
    {  619, 2527,  196659,  5, }, // Europe/Kyiv
    {  619, 3213,    4381,  8, }, // Europe/Lisbon
    {  619, 6506,  196664,  9, }, // Europe/Ljubljana
    {  619, 6365,  196673, 11, }, // Europe/Luxembourg
    {  619, 6582,  196684, 10, }, // Europe/Mariehamn
    {  619, 6410,    5926,  6, }, // Europe/Monaco
    {  619, 3537,   93104,  6, }, // Europe/Moscow
    {  619, 4284,  196694,  5, }, // Europe/Paris
    {  619, 6424,  196699,  9, }, // Europe/Podgorica
    {  619, 6275,   21067,  5, }, // Europe/Prague
    {  619, 6324,   33608,  3, }, // Europe/Rome
    {  619, 6470,    8096, 10, }, // Europe/San_Marino
    {  619, 6245,  196708,  8, }, // Europe/Sarajevo
    {  619, 4451,   78167,  7, }, // Europe/Saratov
    {  619, 6383,  196716,  6, }, // Europe/Skopje
    {  619, 6599,  196722,  6, }, // Europe/Sofia
    {  619, 6523,  196728,  8, }, // Europe/Stockholm
    {  619, 8867,   22793,  6, }, // Europe/Tallinn
    {  619, 6202,   21095,  6, }, // Europe/Tirane
    {  619, 8882,  196736,  9, }, // Europe/Ulyanovsk
    {  619, 6352,  196745,  5, }, // Europe/Vaduz
    {  619, 6567,  196750,  7, }, // Europe/Vatican
    {  619, 6231,  196757,  4, }, // Europe/Vienna
    {  619, 8899,  196761,  7, }, // Europe/Vilnius
    {  619, 4727,    4443,  9, }, // Europe/Volgograd
    {  619, 3190,  196768,  7, }, // Europe/Warsaw
    {  619, 6540,  196775,  6, }, // Europe/Zurich
    {  619, 5078,  196781, 12, }, // Indian/Antananarivo
    {  619, 6947,    8170,  5, }, // Indian/Chagos
    {  619, 6046,  196793,  9, }, // Indian/Christmas
    {  619, 6063,    4465,  5, }, // Indian/Cocos
    {  619, 5029,  196802, 13, }, // Indian/Comoro
    {  619, 6661,   22859,  8, }, // Indian/Kerguelen
    {  619, 7260,  196815,  4, }, // Indian/Mahe
    {  619, 7050,  196819,  9, }, // Indian/Maldives
    {  619, 4069,  196828,  8, }, // Indian/Mauritius
    {  619, 5098,  196836,  7, }, // Indian/Mayotte
    {  619, 7211,  196843,  8, }, // Indian/Reunion
    {  619,  932,  124369,  6, }, // Pacific/Auckland
    {  619, 3706,  196851,  8, }, // Pacific/Bougainville
    {  619, 3015,  196859,  5, }, // Pacific/Chatham
    {  619, 3157,   22893,  4, }, // Pacific/Chuuk
    {  619, 2237,  196864, 11, }, // Pacific/Easter
    {  619, 6678,  196875, 17, }, // Pacific/Galapagos
    {  619, 6696,  196892,  6, }, // Pacific/Gambier
    {  619, 3822,  196898, 11, }, // Pacific/Guadalcanal
    {  619, 2723,  196909,  8, }, // Pacific/Honolulu
    {  619, 3049,  124457,  9, }, // Pacific/Kanton
    {  619, 7019,  196917,  7, }, // Pacific/Kosrae
    {  619, 2842,  196924,  9, }, // Pacific/Kwajalein
    {  619, 4051,  196933, 14, }, // Pacific/Marquesas
    {  619, 8914,  196947,  6, }, // Pacific/Midway
    {  619, 7098,  124531,  5, }, // Pacific/Noumea
    {  619, 3126,    8295,  9, }, // Pacific/Pago_Pago
    {  619, 7194,    8304,  7, }, // Pacific/Pitcairn
    {  619, 3096,    8311,  6, }, // Pacific/Pohnpei
    {  619, 4807,  196953, 11, }, // Pacific/Port_Moresby
    {  619, 6031,  196964,  6, }, // Pacific/Saipan
    {  619, 7312,  124568,  5, }, // Pacific/Tahiti
    {  619, 7392,  196970,  4, }, // Pacific/Wake
    {  619, 7405,    8349,  6, }, // Pacific/Wallis
    {  620,   14,       0,  6, }, // Africa/Asmara Tyap/Latin/Nigeria
    {  620, 4435,       6,  8, }, // Africa/Sao_Tome
    {  620, 4252,      14,  8, }, // America/Asuncion
    {  620,  237,      22,  8, }, // America/Atikokan
    {  620, 7623,      30, 17, }, // America/Bahia_Banderas
    {  620, 3868,      47,  6, }, // America/Cancun
    {  620, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  620, 5723,      66,  7, }, // America/Curacao
    {  620,  481,      73, 13, }, // America/Indiana/Knox
    {  620, 7961,      86, 16, }, // America/Indiana/Marengo
    {  620, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  620, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  620, 8038,     139, 14, }, // America/Indiana/Vevay
    {  620, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  620, 8086,     171, 16, }, // America/Indiana/Winamac
    {  620, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  620, 5932,     207, 22, }, // America/Lower_Princes
    {  620, 8220,     229,  6, }, // America/Merida
    {  620, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  620, 8329,     255, 20, }, // America/North_Dakota/Center
    {  620, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  620,  393,     298,  4, }, // America/Nuuk
    {  620, 6899,     302, 16, }, // America/Scoresbysund
    {  620, 8508,     318, 14, }, // America/St_Barthelemy
    {  620, 2061,     332, 10, }, // America/St_Johns
    {  620, 5863,     342,  9, }, // America/St_Kitts
    {  620, 5880,     351,  9, }, // America/St_Lucia
    {  620,  855,     360, 10, }, // America/St_Thomas
    {  620, 5913,     370, 11, }, // America/St_Vincent
    {  620, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  620, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  620, 1185,     413,  9, }, // Asia/Kathmandu
    {  620,  992,     422,  7, }, // Asia/Kolkata
    {  620, 1211,     429,  5, }, // Asia/Macau
    {  620, 1235,     442,  6, }, // Asia/Yangon
    {  620, 1395,     448,  5, }, // Atlantic/Faroe
    {  620, 6821,     453, 10, }, // Atlantic/St_Helena
    {  620, 2527,     463,  4, }, // Europe/Kyiv
    {  620, 7211,     467,  7, }, // Indian/Reunion
    {  620, 3157,     474,  5, }, // Pacific/Chuuk
    {  620, 3096,     479,  7, }, // Pacific/Pohnpei
    {  621, 6788,   27039,  7, }, // Africa/Abidjan Ukrainian/Cyrillic/Ukraine
    {  621, 6760,     492,  5, }, // Africa/Accra
    {  621, 5059,     497, 11, }, // Africa/Addis_Ababa
    {  621, 7420,     508,  5, }, // Africa/Algiers
    {  621,   14,     513,  6, }, // Africa/Asmara
    {  621,   44,     519,  6, }, // Africa/Bamako
    {  621, 5257,  196974,  5, }, // Africa/Bangui
    {  621, 6746,     530,  6, }, // Africa/Banjul
    {  621, 7435,   27083,  5, }, // Africa/Bissau
    {  621, 4957,   27088,  8, }, // Africa/Blantyre
    {  621, 5287,  196979, 10, }, // Africa/Brazzaville
    {  621, 4922,     558,  9, }, // Africa/Bujumbura
    {  621, 2320,  196989,  4, }, // Africa/Cairo
    {  621, 4117,     571, 10, }, // Africa/Casablanca
    {  621, 7449,     581,  5, }, // Africa/Ceuta
    {  621, 6773,  196993,  7, }, // Africa/Conakry
    {  621, 6840,     593,  5, }, // Africa/Dakar
    {  621, 5130,     598, 12, }, // Africa/Dar_es_Salaam
    {  621, 5043,  197000,  7, }, // Africa/Djibouti
    {  621, 5243,     616,  5, }, // Africa/Douala
    {  621, 5166,  197007,  7, }, // Africa/El_Aaiun
    {  621, 6853,  197014,  7, }, // Africa/Freetown
    {  621, 4906,  197021,  8, }, // Africa/Gaborone
    {  621, 5015,   33781,  6, }, // Africa/Harare
    {  621, 4479,  197029, 12, }, // Africa/Johannesburg
    {  621, 4499,   27173,  5, }, // Africa/Juba
    {  621, 5151,     665,  7, }, // Africa/Kampala
    {  621, 4524,     672,  6, }, // Africa/Khartoum
    {  621, 4973,   27178,  6, }, // Africa/Kigali
    {  621, 5306,   27184,  7, }, // Africa/Kinshasa
    {  621, 4744,  197041,  5, }, // Africa/Lagos
    {  621, 5336,  197046,  9, }, // Africa/Libreville
    {  621, 6869,     704,  4, }, // Africa/Lome
    {  621, 5211,     708,  6, }, // Africa/Luanda
    {  621, 4939,  197055,  9, }, // Africa/Lubumbashi
    {  621, 5001,     723,  6, }, // Africa/Lusaka
    {  621, 5322,     729,  6, }, // Africa/Malabo
    {  621, 4987,     735,  6, }, // Africa/Maputo
    {  621, 5197,     741,  6, }, // Africa/Maseru
    {  621, 5182,     747,  7, }, // Africa/Mbabane
    {  621, 5113,  197064,  8, }, // Africa/Mogadishu
    {  621, 7034,  197072,  8, }, // Africa/Monrovia
    {  621, 3853,   27247,  7, }, // Africa/Nairobi
    {  621, 5271,   27254,  8, }, // Africa/Ndjamena
    {  621, 5354,  197080,  6, }, // Africa/Niamey
    {  621, 6803,  197086,  8, }, // Africa/Nouakchott
    {  621, 6727,  197094,  8, }, // Africa/Ouagadougou
    {  621, 5225,     805, 10, }, // Africa/Porto-Novo
    {  621, 4435,     815,  8, }, // Africa/Sao_Tome
    {  621, 2866,  197102,  7, }, // Africa/Tripoli
    {  621, 6554,   27293,  5, }, // Africa/Tunis
    {  621, 4152,  197109,  7, }, // Africa/Windhoek
    {  621,  132,     842,  4, }, // America/Adak
    {  621, 3346,   51461,  8, }, // America/Anchorage
    {  621, 5607,  197116,  7, }, // America/Anguilla
    {  621, 5624,  197123,  7, }, // America/Antigua
    {  621, 4554,  197130,  9, }, // America/Araguaina
    {  621,  166,  197139, 12, }, // America/Argentina/Buenos_Aires
    {  621,   91,     888,  9, }, // America/Argentina/Catamarca
    {  621,  270,     897,  7, }, // America/Argentina/Cordoba
    {  621,  441,   27355,  5, }, // America/Argentina/Jujuy
    {  621, 7462,  197151,  8, }, // America/Argentina/La_Rioja
    {  621,  565,     917,  7, }, // America/Argentina/Mendoza
    {  621, 7489,  197159, 12, }, // America/Argentina/Rio_Gallegos
    {  621, 7520,   27380,  6, }, // America/Argentina/Salta
    {  621, 7544,     940,  8, }, // America/Argentina/San_Juan
    {  621, 5580,  197171,  8, }, // America/Argentina/San_Luis
    {  621, 7571,     956,  7, }, // America/Argentina/Tucuman
    {  621, 7597,   27394,  5, }, // America/Argentina/Ushuaia
    {  621, 5640,     969,  5, }, // America/Aruba
    {  621, 4252,  197179,  9, }, // America/Asuncion
    {  621,  237,  197188,  8, }, // America/Atikokan
    {  621, 3679,  137145,  4, }, // America/Bahia
    {  621, 7623,  197196, 13, }, // America/Bahia_Banderas
    {  621, 5654,    1013,  8, }, // America/Barbados
    {  621, 7646,    1021,  5, }, // America/Belem
    {  621, 5407,   27443,  5, }, // America/Belize
    {  621, 7660,   36388, 11, }, // America/Blanc-Sablon
    {  621, 7681,   27448,  9, }, // America/Boa_Vista
    {  621, 4361,  197209,  6, }, // America/Bogota
    {  621, 7699,   51524,  5, }, // America/Boise
    {  621, 7713,  197215, 13, }, // America/Cambridge_Bay
    {  621, 7735,  197228, 12, }, // America/Campo_Grande
    {  621, 3868,    1087,  6, }, // America/Cancun
    {  621, 4694,    1093,  7, }, // America/Caracas
    {  621, 4345,  197240,  6, }, // America/Cayenne
    {  621, 5496,  197246, 17, }, // America/Cayman
    {  621, 2260,  197263,  6, }, // America/Chicago
    {  621, 7756,  197269,  6, }, // America/Chihuahua
    {  621, 7774,  197275, 13, }, // America/Ciudad_Juarez
    {  621, 5422,  197288, 10, }, // America/Costa_Rica
    {  621, 7796,    1151,  7, }, // America/Creston
    {  621, 3791,   27556,  5, }, // America/Cuiaba
    {  621, 5723,   36474,  7, }, // America/Curacao
    {  621, 7812,    1172, 12, }, // America/Danmarkshavn
    {  621, 7833,    1184,  6, }, // America/Dawson
    {  621, 7848,  197298, 11, }, // America/Dawson_Creek
    {  621,  805,    1201,  6, }, // America/Denver
    {  621, 3465,   36517,  7, }, // America/Detroit
    {  621, 5739,  197309,  8, }, // America/Dominica
    {  621,  893,    1222,  8, }, // America/Edmonton
    {  621, 7869,   36532,  8, }, // America/Eirunepe
    {  621, 5441,   27633,  9, }, // America/El_Salvador
    {  621, 7886,   51565, 12, }, // America/Fort_Nelson
    {  621, 7906,    1257,  9, }, // America/Fortaleza
    {  621, 7924,  197317,  9, }, // America/Glace_Bay
    {  621, 6881,  197326,  7, }, // America/Goose_Bay
    {  621, 4612,  197333, 10, }, // America/Grand_Turk
    {  621, 5770,  197343,  7, }, // America/Grenada
    {  621, 5786,  197350,  9, }, // America/Guadeloupe
    {  621, 3760,  197359,  9, }, // America/Guatemala
    {  621, 4841,  197368,  8, }, // America/Guayaquil
    {  621, 6932,  197376,  6, }, // America/Guyana
    {  621, 1939,   27727,  8, }, // America/Halifax
    {  621, 2281,    1339,  6, }, // America/Havana
    {  621, 7942,  197382, 10, }, // America/Hermosillo
    {  621,  348,  197392, 12, }, // America/Indiana/Indianapolis
    {  621,  481,  197404, 13, }, // America/Indiana/Knox
    {  621, 7961,  197417, 16, }, // America/Indiana/Marengo
    {  621, 7985,  197433, 19, }, // America/Indiana/Petersburg
    {  621, 8012,  197452, 18, }, // America/Indiana/Tell_City
    {  621, 8038,  197470, 14, }, // America/Indiana/Vevay
    {  621, 8060,  197484, 16, }, // America/Indiana/Vincennes
    {  621, 8086,  197500, 16, }, // America/Indiana/Winamac
    {  621, 8110,   27866,  6, }, // America/Inuvik
    {  621,  660,  197516,  7, }, // America/Iqaluit
    {  621, 2799,   27879,  6, }, // America/Jamaica
    {  621, 5380,   36614,  5, }, // America/Juneau
    {  621,  521,  197523,  8, }, // America/Kentucky/Louisville
    {  621, 8125,  197531, 20, }, // America/Kentucky/Monticello
    {  621, 5704,   36619, 10, }, // America/Kralendijk
    {  621, 4376,    1514,  6, }, // America/La_Paz
    {  621, 7169,   27925,  4, }, // America/Lima
    {  621, 3239,   51629, 12, }, // America/Los_Angeles
    {  621, 5932,  197551, 20, }, // America/Lower_Princes
    {  621, 8153,   36667,  6, }, // America/Maceio
    {  621, 8168,  197571,  7, }, // America/Managua
    {  621, 1908,    1566,  6, }, // America/Manaus
    {  621, 5897,  197578,  6, }, // America/Marigot
    {  621, 5805,  197584,  9, }, // America/Martinique
    {  621, 8184,    1587,  9, }, // America/Matamoros
    {  621, 2917,    1596,  8, }, // America/Mazatlan
    {  621, 8202,   27996,  8, }, // America/Menominee
    {  621, 8220,  197593,  6, }, // America/Merida
    {  621, 8235,    1618, 10, }, // America/Metlakatla
    {  621, 2949,  197599,  6, }, // America/Mexico_City
    {  621, 4391,   28016,  7, }, // America/Miquelon
    {  621, 8254,    1641,  7, }, // America/Moncton
    {  621, 8270,   36673,  8, }, // America/Monterrey
    {  621, 4098,  197605, 10, }, // America/Montevideo
    {  621, 5824,    1667, 10, }, // America/Montserrat
    {  621, 5481,   34454,  5, }, // America/Nassau
    {  621, 2379,   51680,  8, }, // America/New_York
    {  621, 8288,    1691,  3, }, // America/Nome
    {  621, 1850,   56257,  7, }, // America/Noronha
    {  621, 8301,  197615, 22, }, // America/North_Dakota/Beulah
    {  621, 8329,  197637, 22, }, // America/North_Dakota/Center
    {  621, 8357,  197659, 26, }, // America/North_Dakota/New_Salem
    {  621,  393,    1772,  4, }, // America/Nuuk
    {  621, 8388,  197685,  7, }, // America/Ojinaga
    {  621, 2356,    1783,  6, }, // America/Panama
    {  621, 6173,    1789, 10, }, // America/Paramaribo
    {  621, 2973,   28171,  6, }, // America/Phoenix
    {  621, 3945,    1805, 12, }, // America/Port-au-Prince
    {  621, 5954,   28189, 13, }, // America/Port_of_Spain
    {  621, 8404,   28202, 11, }, // America/Porto_Velho
    {  621, 5843,  197692, 11, }, // America/Puerto_Rico
    {  621, 4030,    1852, 12, }, // America/Punta_Arenas
    {  621, 8424,  197703, 12, }, // America/Rankin_Inlet
    {  621, 8445,  197715,  6, }, // America/Recife
    {  621, 1995,   51767,  8, }, // America/Regina
    {  621, 8460,   56329,  8, }, // America/Resolute
    {  621,  695,  197721, 10, }, // America/Rio_Branco
    {  621, 8477,    1907,  8, }, // America/Santarem
    {  621, 2201,  197731,  8, }, // America/Santiago
    {  621, 6111,  197739, 13, }, // America/Santo_Domingo
    {  621, 1878,    1936,  9, }, // America/Sao_Paulo
    {  621, 6899,  197752, 16, }, // America/Scoresbysund
    {  621, 8494,   28331,  5, }, // America/Sitka
    {  621, 8508,  197768, 13, }, // America/St_Barthelemy
    {  621, 2061,   28349, 10, }, // America/St_Johns
    {  621, 5863,  197781, 10, }, // America/St_Kitts
    {  621, 5880,   28368, 10, }, // America/St_Lucia
    {  621,  855,    2003, 10, }, // America/St_Thomas
    {  621, 5913,   28378, 12, }, // America/St_Vincent
    {  621, 8530,  197791, 13, }, // America/Swift_Current
    {  621, 5461,  197804, 12, }, // America/Tegucigalpa
    {  621, 5756,    2049,  4, }, // America/Thule
    {  621,  313,  197816,  7, }, // America/Tijuana
    {  621,  608,    2060,  7, }, // America/Toronto
    {  621, 5688,    2067,  7, }, // America/Tortola
    {  621, 2093,    2074,  8, }, // America/Vancouver
    {  621, 2144,  197823,  8, }, // America/Whitehorse
    {  621,  734,  197831,  8, }, // America/Winnipeg
    {  621, 8552,   28450,  6, }, // America/Yakutat
    {  621, 6001,  197839,  5, }, // Antarctica/Casey
    {  621, 6094,  197844,  5, }, // Antarctica/Davis
    {  621, 6133,  197849, 14, }, // Antarctica/DumontDUrville
    {  621, 8568,  197863,  8, }, // Antarctica/Macquarie
    {  621, 7066,    2139,  6, }, // Antarctica/Mawson
    {  621, 7113,    2145,  9, }, // Antarctica/McMurdo
    {  621, 8589,    2154,  6, }, // Antarctica/Palmer
    {  621, 7226,    2160,  6, }, // Antarctica/Rothera
    {  621, 7295,  197871,  5, }, // Antarctica/Syowa
    {  621, 8607,    2171,  5, }, // Antarctica/Troll
    {  621, 7374,   34825,  6, }, // Antarctica/Vostok
    {  621, 1429,  197876,  7, }, // Arctic/Longyearbyen
    {  621, 5570,    2192,  4, }, // Asia/Aden
    {  621, 5395,   34842,  6, }, // Asia/Almaty
    {  621, 3968,    2202,  5, }, // Asia/Amman
    {  621, 5511,   34848,  6, }, // Asia/Anadyr
    {  621, 5523,    2213,  5, }, // Asia/Aqtau
    {  621, 5368,    2218,  6, }, // Asia/Aqtobe
    {  621,  964,   28549,  7, }, // Asia/Ashgabat
    {  621, 8624,   51877,  6, }, // Asia/Atyrau
    {  621, 3607,    2237,  6, }, // Asia/Baghdad
    {  621, 5534,   36948,  7, }, // Asia/Bahrain
    {  621, 3653,    2250,  4, }, // Asia/Baku
    {  621, 4466,    2254,  7, }, // Asia/Bangkok
    {  621, 3571,    2261,  7, }, // Asia/Barnaul
    {  621, 4086,   28563,  6, }, // Asia/Beirut
    {  621, 3778,   28569,  6, }, // Asia/Bishkek
    {  621, 5989,   28575,  6, }, // Asia/Brunei
    {  621, 4601,    2286,  4, }, // Asia/Chita
    {  621, 4511,    2290,  7, }, // Asia/Colombo
    {  621, 4540,    2297,  6, }, // Asia/Damascus
    {  621, 1093,    2303,  5, }, // Asia/Dhaka
    {  621, 6192,  197883,  4, }, // Asia/Dili
    {  621, 3596,   28596,  5, }, // Asia/Dubai
    {  621, 6159,    2317,  7, }, // Asia/Dushanbe
    {  621, 8636,    2324,  9, }, // Asia/Famagusta
    {  621, 8651,    2333,  4, }, // Asia/Gaza
    {  621, 4795,    2337,  6, }, // Asia/Hebron
    {  621, 1259,  197887,  7, }, // Asia/Ho_Chi_Minh
    {  621, 2704,   51902,  7, }, // Asia/Hong_Kong
    {  621, 4771,    2357,  4, }, // Asia/Hovd
    {  621, 4184,  197894,  8, }, // Asia/Irkutsk
    {  621, 7006,   28638,  8, }, // Asia/Jakarta
    {  621, 6992,   51909,  7, }, // Asia/Jayapura
    {  621, 1290,  197902,  8, }, // Asia/Jerusalem
    {  621, 3560,    2392,  5, }, // Asia/Kabul
    {  621, 4316,    2411,  8, }, // Asia/Kamchatka
    {  621, 4239,  197910,  6, }, // Asia/Karachi
    {  621, 1185,    2428,  8, }, // Asia/Kathmandu
    {  621, 8661,   34933,  7, }, // Asia/Khandyga
    {  621,  992,   36971,  7, }, // Asia/Kolkata
    {  621, 4197,  197916, 11, }, // Asia/Krasnoyarsk
    {  621, 4859,    2462, 12, }, // Asia/Kuala_Lumpur
    {  621, 5976,    2474,  6, }, // Asia/Kuching
    {  621, 5547,   28693,  6, }, // Asia/Kuwait
    {  621, 1211,    2486,  5, }, // Asia/Macau
    {  621, 4017,    2491,  7, }, // Asia/Magadan
    {  621, 1349,  118555,  8, }, // Asia/Makassar
    {  621, 7182,   28704,  6, }, // Asia/Manila
    {  621, 6920,    2511,  6, }, // Asia/Muscat
    {  621, 2554,  197927,  7, }, // Asia/Nicosia
    {  621, 8675,  197934, 12, }, // Asia/Novokuznetsk
    {  621, 4135,  197946, 12, }, // Asia/Novosibirsk
    {  621, 4229,  197958,  5, }, // Asia/Omsk
    {  621, 7145,   35006,  4, }, // Asia/Oral
    {  621, 6961,   51959,  8, }, // Asia/Phnom_Penh
    {  621, 8693,  197963,  9, }, // Asia/Pontianak
    {  621, 4214,   28763,  7, }, // Asia/Pyongyang
    {  621, 5559,    2579,  5, }, // Asia/Qatar
    {  621, 8708,   37005,  8, }, // Asia/Qostanay
    {  621, 4269,   35035,  9, }, // Asia/Qyzylorda
    {  621, 3584,  197972,  7, }, // Asia/Riyadh
    {  621, 4408,   28794,  7, }, // Asia/Sakhalin
    {  621, 7245,    2621,  9, }, // Asia/Samarkand
    {  621, 3279,    2630,  4, }, // Asia/Seoul
    {  621, 1053,   28801,  6, }, // Asia/Shanghai
    {  621, 3300,   28807,  8, }, // Asia/Singapore
    {  621, 4297,  197979, 16, }, // Asia/Srednekolymsk
    {  621, 3263,  197995,  6, }, // Asia/Taipei
    {  621, 4781,    2667,  7, }, // Asia/Tashkent
    {  621, 3915,   28835,  7, }, // Asia/Tbilisi
    {  621, 2772,    2678,  7, }, // Asia/Tehran
    {  621, 1317,   28849,  7, }, // Asia/Thimphu
    {  621, 2821,  198001,  5, }, // Asia/Tokyo
    {  621, 4572,  198006,  6, }, // Asia/Tomsk
    {  621, 1021,    2702, 10, }, // Asia/Ulaanbaatar
    {  621, 1159,  198012,  6, }, // Asia/Urumqi
    {  621, 8722,   51993,  9, }, // Asia/Ust-Nera
    {  621, 6977,  198018,  8, }, // Asia/Vientiane
    {  621, 4710,    2733, 11, }, // Asia/Vladivostok
    {  621, 4828,  198026,  7, }, // Asia/Yakutsk
    {  621, 1235,   56409,  5, }, // Asia/Yangon
    {  621, 3883,  198033, 12, }, // Asia/Yekaterinburg
    {  621, 3747,  198045,  6, }, // Asia/Yerevan
    {  621, 3663,  198051, 16, }, // Atlantic/Azores
    {  621, 5671,  128581,  7, }, // Atlantic/Bermuda
    {  621, 6628,  198067, 17, }, // Atlantic/Canary
    {  621, 3727,    2826, 10, }, // Atlantic/Cape_Verde
    {  621, 1395,  198084, 17, }, // Atlantic/Faroe
    {  621, 8736,   37132,  7, }, // Atlantic/Madeira
    {  621, 2748,  198101,  9, }, // Atlantic/Reykjavik
    {  621, 7272,  198110, 17, }, // Atlantic/South_Georgia
    {  621, 6821,  198127, 19, }, // Atlantic/St_Helena
    {  621, 6644,  198146,  6, }, // Atlantic/Stanley
    {  621, 1670,  198152,  8, }, // Australia/Adelaide
    {  621, 1635,  198160,  7, }, // Australia/Brisbane
    {  621, 1799,  198167, 11, }, // Australia/Broken_Hill
    {  621, 1583,   29102,  6, }, // Australia/Darwin
    {  621, 3637,  198178,  5, }, // Australia/Eucla
    {  621, 1516,  198183,  6, }, // Australia/Hobart
    {  621, 8753,  198189,  8, }, // Australia/Lindeman
    {  621, 1547,    2968,  8, }, // Australia/Lord_Howe
    {  621, 1727,   29120,  8, }, // Australia/Melbourne
    {  621, 1762,    2983,  4, }, // Australia/Perth
    {  621, 1463,   29128,  6, }, // Australia/Sydney
    {  621, 6441,    2993,  9, }, // Europe/Amsterdam
    {  621, 6216,    3002,  7, }, // Europe/Andorra
    {  621, 3620,   29149,  9, }, // Europe/Astrakhan
    {  621, 2300,  198197,  5, }, // Europe/Athens
    {  621, 6336,    3021,  7, }, // Europe/Belgrade
    {  621, 4757,   29170,  6, }, // Europe/Berlin
    {  621, 6488,    3034, 10, }, // Europe/Bratislava
    {  621, 2167,   52164,  8, }, // Europe/Brussels
    {  621, 3928,    3052,  8, }, // Europe/Bucharest
    {  621, 3806,    3060,  8, }, // Europe/Budapest
    {  621, 8772,   29201,  8, }, // Europe/Busingen
    {  621, 2583,  198202,  7, }, // Europe/Chisinau
    {  621, 6289,   52191, 10, }, // Europe/Copenhagen
    {  621, 2338,   29226,  6, }, // Europe/Dublin
    {  621, 6307,   29232,  9, }, // Europe/Gibraltar
    {  621, 8788,   29241,  6, }, // Europe/Guernsey
    {  621, 6612,  198209,  9, }, // Europe/Helsinki
    {  621, 8804,  198218, 10, }, // Europe/Isle_of_Man
    {  621, 1130,    3146,  7, }, // Europe/Istanbul
    {  621, 8823,  198228,  6, }, // Europe/Jersey
    {  621, 3979,   29273, 11, }, // Europe/Kaliningrad
    {  621, 8929,  198234,  5, }, // Europe/Kirov
    {  621, 2527,  198239,  4, }, // Europe/Kyiv
    {  621, 3213,   29293,  7, }, // Europe/Lisbon
    {  621, 6506,   29300,  7, }, // Europe/Ljubljana
    {  621, 2501,    3195,  6, }, // Europe/London
    {  621, 6365,   29313, 10, }, // Europe/Luxembourg
    {  621, 4892,    3212,  6, }, // Europe/Madrid
    {  621, 6397,   29329,  6, }, // Europe/Malta
    {  621, 6582,  198243,  9, }, // Europe/Mariehamn
    {  621, 3693,  198252,  6, }, // Europe/Minsk
    {  621, 6410,    3237,  6, }, // Europe/Monaco
    {  621, 3537,    3243,  6, }, // Europe/Moscow
    {  621, 6458,    3249,  4, }, // Europe/Oslo
    {  621, 4284,    3253,  5, }, // Europe/Paris
    {  621, 6424,  198258,  9, }, // Europe/Podgorica
    {  621, 6275,    3267,  5, }, // Europe/Prague
    {  621, 8837,    3272,  4, }, // Europe/Riga
    {  621, 6324,    3276,  3, }, // Europe/Rome
    {  621, 4331,    3279,  6, }, // Europe/Samara
    {  621, 6470,    3285, 10, }, // Europe/San_Marino
    {  621, 6245,  198267,  7, }, // Europe/Sarajevo
    {  621, 4451,    3302,  7, }, // Europe/Saratov
    {  621, 8849,  198274, 11, }, // Europe/Simferopol
    {  621, 6383,  198285,  6, }, // Europe/Skopje
    {  621, 6599,  198291,  5, }, // Europe/Sofia
    {  621, 6523,   52242,  9, }, // Europe/Stockholm
    {  621, 8867,  198296,  7, }, // Europe/Tallinn
    {  621, 6202,    3343,  6, }, // Europe/Tirane
    {  621, 8882,  198303, 10, }, // Europe/Ulyanovsk
    {  621, 6352,    3358,  5, }, // Europe/Vaduz
    {  621, 6567,    3363,  7, }, // Europe/Vatican
    {  621, 6231,  198313,  6, }, // Europe/Vienna
    {  621, 8899,   29471,  7, }, // Europe/Vilnius
    {  621, 4727,    3381,  9, }, // Europe/Volgograd
    {  621, 3190,    3390,  7, }, // Europe/Warsaw
    {  621, 6261,    3397,  6, }, // Europe/Zagreb
    {  621, 6540,  198319,  5, }, // Europe/Zurich
    {  621, 5078,  198324, 12, }, // Indian/Antananarivo
    {  621, 6947,  198336,  5, }, // Indian/Chagos
    {  621, 6046,  198341, 13, }, // Indian/Christmas
    {  621, 6063,  198354, 16, }, // Indian/Cocos
    {  621, 5029,   52314,  6, }, // Indian/Comoro
    {  621, 6661,  198370,  8, }, // Indian/Kerguelen
    {  621, 7260,   35516,  4, }, // Indian/Mahe
    {  621, 7050,  198378,  8, }, // Indian/Maldives
    {  621, 4069,  198386,  8, }, // Indian/Mauritius
    {  621, 5098,   56584,  7, }, // Indian/Mayotte
    {  621, 7211,  198394,  8, }, // Indian/Reunion
    {  621, 4422,  198402,  4, }, // Pacific/Apia
    {  621,  932,    3502,  6, }, // Pacific/Auckland
    {  621, 3706,  198406,  9, }, // Pacific/Bougainville
    {  621, 3015,    3516,  5, }, // Pacific/Chatham
    {  621, 3157,  118669,  4, }, // Pacific/Chuuk
    {  621, 2237,  198415, 12, }, // Pacific/Easter
    {  621, 7360,    3536,  5, }, // Pacific/Efate
    {  621, 7327,    3541,  7, }, // Pacific/Fakaofo
    {  621, 3902,  198427,  5, }, // Pacific/Fiji
    {  621, 7343,  198432,  8, }, // Pacific/Funafuti
    {  621, 6678,  198440,  9, }, // Pacific/Galapagos
    {  621, 6696,  198449,  6, }, // Pacific/Gambier
    {  621, 3822,    3595, 11, }, // Pacific/Guadalcanal
    {  621, 6018,    3606,  4, }, // Pacific/Guam
    {  621, 2723,   52450,  8, }, // Pacific/Honolulu
    {  621, 3049,  198455,  9, }, // Pacific/Kanton
    {  621, 3998,  198464, 10, }, // Pacific/Kiritimati
    {  621, 7019,    3626,  6, }, // Pacific/Kosrae
    {  621, 2842,   29709, 10, }, // Pacific/Kwajalein
    {  621, 4877,   37442,  7, }, // Pacific/Majuro
    {  621, 4051,  198474, 17, }, // Pacific/Marquesas
    {  621, 8914,  198491,  6, }, // Pacific/Midway
    {  621, 7084,    3680,  5, }, // Pacific/Nauru
    {  621, 7132,  198497,  4, }, // Pacific/Niue
    {  621, 4168,    3689,  7, }, // Pacific/Norfolk
    {  621, 7098,    3696,  5, }, // Pacific/Noumea
    {  621, 3126,    3701,  9, }, // Pacific/Pago_Pago
    {  621, 7155,    3710,  5, }, // Pacific/Palau
    {  621, 7194,  198501,  7, }, // Pacific/Pitcairn
    {  621, 3096,   35651,  6, }, // Pacific/Pohnpei
    {  621, 4807,   29783, 11, }, // Pacific/Port_Moresby
    {  621, 6076,    3739,  9, }, // Pacific/Rarotonga
    {  621, 6031,   29803,  6, }, // Pacific/Saipan
    {  621, 7312,  198508,  5, }, // Pacific/Tahiti
    {  621, 6712,    3759,  6, }, // Pacific/Tarawa
    {  621, 4583,    3765,  9, }, // Pacific/Tongatapu
    {  621, 7392,  181934,  4, }, // Pacific/Wake
    {  621, 7405,  198513,  6, }, // Pacific/Wallis
    {  622, 6760,   21174,  5, }, // Africa/Accra Upper Sorbian/Latin/Germany
    {  622, 5059,    3784, 11, }, // Africa/Addis_Ababa
    {  622, 7420,   58423,  6, }, // Africa/Algiers
    {  622,   14,       0,  6, }, // Africa/Asmara
    {  622, 2320,    4572,  5, }, // Africa/Cairo
    {  622, 5130,   77951, 10, }, // Africa/Dar_es_Salaam
    {  622, 5043,  198519,  7, }, // Africa/Djibouti
    {  622, 5166,   63378,  8, }, // Africa/El_Aaiun
    {  622, 4524,   26372,  7, }, // Africa/Khartoum
    {  622, 6869,   20300,  4, }, // Africa/Lome
    {  622, 5113,   57589,  8, }, // Africa/Mogadishu
    {  622, 6727,    4640,  8, }, // Africa/Ouagadougou
    {  622, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  622, 4435,       6,  8, }, // Africa/Sao_Tome
    {  622, 2866,   57624,  8, }, // Africa/Tripoli
    {  622, 4252,      14,  8, }, // America/Asuncion
    {  622,  237,      22,  8, }, // America/Atikokan
    {  622, 7623,      30, 17, }, // America/Bahia_Banderas
    {  622, 4361,   20389,  6, }, // America/Bogota
    {  622, 3868,      47,  6, }, // America/Cancun
    {  622, 5496,  198526, 14, }, // America/Cayman
    {  622, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  622, 5723,      66,  7, }, // America/Curacao
    {  622, 2281,   63392,  7, }, // America/Havana
    {  622,  481,      73, 13, }, // America/Indiana/Knox
    {  622, 7961,      86, 16, }, // America/Indiana/Marengo
    {  622, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  622, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  622, 8038,     139, 14, }, // America/Indiana/Vevay
    {  622, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  622, 8086,     171, 16, }, // America/Indiana/Winamac
    {  622, 2799,    3877,  7, }, // America/Jamaica
    {  622, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  622, 5932,     207, 22, }, // America/Lower_Princes
    {  622, 8220,     229,  6, }, // America/Merida
    {  622, 2949,  198540, 12, }, // America/Mexico_City
    {  622, 8301,  198552, 23, }, // America/North_Dakota/Beulah
    {  622, 8329,  198575, 23, }, // America/North_Dakota/Center
    {  622, 8357,  198598, 26, }, // America/North_Dakota/New_Salem
    {  622,  393,     298,  4, }, // America/Nuuk
    {  622, 5954,   26559, 13, }, // America/Port_of_Spain
    {  622, 1878,   26580,  9, }, // America/Sao_Paulo
    {  622, 6899,     302, 16, }, // America/Scoresbysund
    {  622, 8508,  127750, 13, }, // America/St_Barthelemy
    {  622, 2061,     332, 10, }, // America/St_Johns
    {  622, 5863,     342,  9, }, // America/St_Kitts
    {  622, 5880,     351,  9, }, // America/St_Lucia
    {  622,  855,     360, 10, }, // America/St_Thomas
    {  622, 5913,     370, 11, }, // America/St_Vincent
    {  622, 6133,  127763, 16, }, // Antarctica/DumontDUrville
    {  622, 7374,  198624, 17, }, // Antarctica/Vostok
    {  622, 5368,    7488,  6, }, // Asia/Aqtobe
    {  622, 3607,    4044,  6, }, // Asia/Baghdad
    {  622, 3778,   33323,  6, }, // Asia/Bishkek
    {  622, 4540,    4083,  8, }, // Asia/Damascus
    {  622, 6159,   33336,  7, }, // Asia/Dushanbe
    {  622, 1259,  198641, 20, }, // Asia/Ho_Chi_Minh
    {  622, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  622, 6992,  127796, 11, }, // Asia/Jayapura
    {  622, 4316,   33378,  8, }, // Asia/Kamchatka
    {  622, 4239,   33386,  6, }, // Asia/Karachi
    {  622, 1185,     413,  9, }, // Asia/Kathmandu
    {  622,  992,   64221,  8, }, // Asia/Kolkata
    {  622, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  622, 1211,     429,  5, }, // Asia/Macau
    {  622, 6920,    4208,  6, }, // Asia/Muscat
    {  622, 2554,    5620,  7, }, // Asia/Nicosia
    {  622, 8675,  198661, 12, }, // Asia/Novokuznetsk
    {  622, 4135,   78082, 11, }, // Asia/Novosibirsk
    {  622, 4214,   78093,  9, }, // Asia/Pyongyang
    {  622, 5559,    4214,  5, }, // Asia/Qatar
    {  622, 3584,    7669,  4, }, // Asia/Riyadh
    {  622, 4408,   57952,  8, }, // Asia/Sakhalin
    {  622, 3300,   20782,  8, }, // Asia/Singapore
    {  622, 3263,   78102,  6, }, // Asia/Taipei
    {  622, 4781,   33457,  7, }, // Asia/Tashkent
    {  622, 2772,    7724,  7, }, // Asia/Tehran
    {  622, 2821,    4238,  5, }, // Asia/Tokyo
    {  622, 4710,    4243, 11, }, // Asia/Vladivostok
    {  622, 4828,    7755,  7, }, // Asia/Yakutsk
    {  622, 1235,    7762,  6, }, // Asia/Yangon
    {  622, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  622, 3747,  198673,  8, }, // Asia/Yerevan
    {  622, 3663,  127827,  5, }, // Atlantic/Azores
    {  622, 5671,   58043,  7, }, // Atlantic/Bermuda
    {  622, 6628,  198681, 14, }, // Atlantic/Canary
    {  622, 3727,   57248,  9, }, // Atlantic/Cape_Verde
    {  622, 1395,  198695, 12, }, // Atlantic/Faroe
    {  622, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  622, 7272,  198707, 15, }, // Atlantic/South_Georgia
    {  622, 6821,     453, 10, }, // Atlantic/St_Helena
    {  622, 2300,    4320,  5, }, // Europe/Athens
    {  622, 6336,  198722,  8, }, // Europe/Belgrade
    {  622, 2167,   22634,  7, }, // Europe/Brussels
    {  622, 3928,   26897,  8, }, // Europe/Bucharest
    {  622, 8772,   26905,  8, }, // Europe/Busingen
    {  622, 2583,  198730,  8, }, // Europe/Chisinau
    {  622, 6289,    4357, 10, }, // Europe/Copenhagen
    {  622, 2527,   78163,  4, }, // Europe/Kyiv
    {  622, 3213,    4381,  8, }, // Europe/Lisbon
    {  622, 6365,    4395,  9, }, // Europe/Luxembourg
    {  622, 3537,   93104,  6, }, // Europe/Moscow
    {  622, 6275,   36153,  5, }, // Europe/Prague
    {  622, 6324,    5946,  3, }, // Europe/Rome
    {  622, 6202,   21095,  6, }, // Europe/Tirane
    {  622, 6567,    5978,  7, }, // Europe/Vatican
    {  622, 6231,   57383,  4, }, // Europe/Vienna
    {  622, 4727,    4443,  9, }, // Europe/Volgograd
    {  622, 3190,  127880,  7, }, // Europe/Warsaw
    {  622, 6540,    4459,  6, }, // Europe/Zurich
    {  622, 6046,  198738, 12, }, // Indian/Christmas
    {  622, 5029,   58276,  6, }, // Indian/Comoro
    {  622, 7050,  127899,  8, }, // Indian/Maldives
    {  622, 7211,     467,  7, }, // Indian/Reunion
    {  622, 3157,     474,  5, }, // Pacific/Chuuk
    {  622, 2237,  198750, 13, }, // Pacific/Easter
    {  622, 3902,  198763,  5, }, // Pacific/Fiji
    {  622, 3049,    4501,  9, }, // Pacific/Kanton
    {  622, 3096,     479,  7, }, // Pacific/Pohnpei
    {  623, 6788,  198768,  7, }, // Africa/Abidjan Urdu/Arabic/Pakistan
    {  623, 6760,  198775,  5, }, // Africa/Accra
    {  623, 5059,  102986, 10, }, // Africa/Addis_Ababa
    {  623, 7420,  198780,  7, }, // Africa/Algiers
    {  623,   14,  106626,  6, }, // Africa/Asmara
    {  623,   44,  103013,  5, }, // Africa/Bamako
    {  623, 5257,  198787,  6, }, // Africa/Bangui
    {  623, 6746,  198793,  5, }, // Africa/Banjul
    {  623, 7435,  103033,  5, }, // Africa/Bissau
    {  623, 4957,  198798,  8, }, // Africa/Blantyre
    {  623, 5287,  198806,  9, }, // Africa/Brazzaville
    {  623, 4922,  198815,  6, }, // Africa/Bujumbura
    {  623, 2320,  198821,  5, }, // Africa/Cairo
    {  623, 4117,  198826, 11, }, // Africa/Casablanca
    {  623, 7449,  198837,  5, }, // Africa/Ceuta
    {  623, 6773,  198842,  6, }, // Africa/Conakry
    {  623, 6840,  198848,  4, }, // Africa/Dakar
    {  623, 5130,   10499, 10, }, // Africa/Dar_es_Salaam
    {  623, 5043,  198852,  5, }, // Africa/Djibouti
    {  623, 5243,  198857,  5, }, // Africa/Douala
    {  623, 5166,  154508,  6, }, // Africa/El_Aaiun
    {  623, 6853,  198862,  8, }, // Africa/Freetown
    {  623, 4906,  198870,  5, }, // Africa/Gaborone
    {  623, 5015,  198875,  5, }, // Africa/Harare
    {  623, 4479,  103149,  9, }, // Africa/Johannesburg
    {  623, 4499,   10555,  4, }, // Africa/Juba
    {  623, 5151,  198880,  7, }, // Africa/Kampala
    {  623, 4524,   10568,  5, }, // Africa/Khartoum
    {  623, 4973,  198887,  5, }, // Africa/Kigali
    {  623, 5306,  198892,  6, }, // Africa/Kinshasa
    {  623, 4744,  106771,  5, }, // Africa/Lagos
    {  623, 5336,  198898,  9, }, // Africa/Libreville
    {  623, 6869,   10598,  3, }, // Africa/Lome
    {  623, 5211,  198907,  6, }, // Africa/Luanda
    {  623, 4939,  167857,  8, }, // Africa/Lubumbashi
    {  623, 5001,  198913,  7, }, // Africa/Lusaka
    {  623, 5322,  167871,  5, }, // Africa/Malabo
    {  623, 4987,  198920,  5, }, // Africa/Maputo
    {  623, 5197,  198925,  5, }, // Africa/Maseru
    {  623, 5182,  198930,  6, }, // Africa/Mbabane
    {  623, 5113,  154607,  8, }, // Africa/Mogadishu
    {  623, 7034,  154615,  8, }, // Africa/Monrovia
    {  623, 3853,  103257,  6, }, // Africa/Nairobi
    {  623, 5271,  198936,  9, }, // Africa/Ndjamena
    {  623, 5354,  154637,  5, }, // Africa/Niamey
    {  623, 6803,  198945,  7, }, // Africa/Nouakchott
    {  623, 6727,  198952,  9, }, // Africa/Ouagadougou
    {  623, 5225,  103293, 10, }, // Africa/Porto-Novo
    {  623, 4435,  198961,  7, }, // Africa/Sao_Tome
    {  623, 2866,  198968,  7, }, // Africa/Tripoli
    {  623, 6554,  198975,  5, }, // Africa/Tunis
    {  623, 4152,  198980,  6, }, // Africa/Windhoek
    {  623,  132,  152047,  4, }, // America/Adak
    {  623, 3346,  152051,  7, }, // America/Anchorage
    {  623, 5607,  198986,  7, }, // America/Anguilla
    {  623, 5624,  198993,  7, }, // America/Antigua
    {  623, 4554,  167968,  9, }, // America/Araguaina
    {  623,  166,  199000, 10, }, // America/Argentina/Buenos_Aires
    {  623,   91,  103380,  9, }, // America/Argentina/Catamarca
    {  623,  270,  199010,  7, }, // America/Argentina/Cordoba
    {  623,  441,  199017,  6, }, // America/Argentina/Jujuy
    {  623, 7462,  168012,  9, }, // America/Argentina/La_Rioja
    {  623,  565,  199023,  7, }, // America/Argentina/Mendoza
    {  623, 7489,  168028, 11, }, // America/Argentina/Rio_Gallegos
    {  623, 7520,  103430,  5, }, // America/Argentina/Salta
    {  623, 7544,  168044,  8, }, // America/Argentina/San_Juan
    {  623, 5580,  199030,  8, }, // America/Argentina/San_Luis
    {  623, 7571,  199038,  7, }, // America/Argentina/Tucuman
    {  623, 7597,  168068,  8, }, // America/Argentina/Ushuaia
    {  623, 5640,  154793,  5, }, // America/Aruba
    {  623, 4252,  199045,  7, }, // America/Asuncion
    {  623,  237,  199052,  7, }, // America/Atikokan
    {  623, 3679,  103504,  5, }, // America/Bahia
    {  623, 7623,  199059, 11, }, // America/Bahia_Banderas
    {  623, 5654,  103516,  8, }, // America/Barbados
    {  623, 7646,  199070,  5, }, // America/Belem
    {  623, 5407,  199075,  6, }, // America/Belize
    {  623, 7660,  199081, 11, }, // America/Blanc-Sablon
    {  623, 7681,  199092,  8, }, // America/Boa_Vista
    {  623, 4361,  199100,  5, }, // America/Bogota
    {  623, 7699,  199105,  5, }, // America/Boise
    {  623, 7713,  199110, 15, }, // America/Cambridge_Bay
    {  623, 7735,  199125, 11, }, // America/Campo_Grande
    {  623, 3868,  199136,  6, }, // America/Cancun
    {  623, 4694,  199142,  6, }, // America/Caracas
    {  623, 4345,  199148,  5, }, // America/Cayenne
    {  623, 5496,  199153,  5, }, // America/Cayman
    {  623, 2260,  199158,  5, }, // America/Chicago
    {  623, 7756,  199163,  8, }, // America/Chihuahua
    {  623, 7774,  152286, 13, }, // America/Ciudad_Juarez
    {  623, 5422,  199171, 10, }, // America/Costa_Rica
    {  623, 7796,  199181,  7, }, // America/Creston
    {  623, 3791,  199188,  7, }, // America/Cuiaba
    {  623, 5723,  199195,  8, }, // America/Curacao
    {  623, 7812,  103665, 11, }, // America/Danmarkshavn
    {  623, 7833,  199203,  5, }, // America/Dawson
    {  623, 7848,  199208, 10, }, // America/Dawson_Creek
    {  623,  805,  199218,  5, }, // America/Denver
    {  623, 3465,  199223,  7, }, // America/Detroit
    {  623, 5739,  199230,  7, }, // America/Dominica
    {  623,  893,  199237,  8, }, // America/Edmonton
    {  623, 7869,  199245,  7, }, // America/Eirunepe
    {  623, 5441,  199252, 10, }, // America/El_Salvador
    {  623, 7886,  103745, 10, }, // America/Fort_Nelson
    {  623, 7906,  199262,  9, }, // America/Fortaleza
    {  623, 7924,  199271, 13, }, // America/Glace_Bay
    {  623, 6881,  199284, 12, }, // America/Goose_Bay
    {  623, 4612,  199296,  8, }, // America/Grand_Turk
    {  623, 5770,  199304,  6, }, // America/Grenada
    {  623, 5786,  199310,  8, }, // America/Guadeloupe
    {  623, 3760,  199318, 10, }, // America/Guatemala
    {  623, 4841,  199328,  9, }, // America/Guayaquil
    {  623, 6932,  199337,  5, }, // America/Guyana
    {  623, 1939,  199342,  8, }, // America/Halifax
    {  623, 2281,  199350,  5, }, // America/Havana
    {  623, 7942,  199355,  8, }, // America/Hermosillo
    {  623,  348,  199363, 11, }, // America/Indiana/Indianapolis
    {  623,  481,  199374, 14, }, // America/Indiana/Knox
    {  623, 7961,  199388, 15, }, // America/Indiana/Marengo
    {  623, 7985,  199403, 17, }, // America/Indiana/Petersburg
    {  623, 8012,  199420, 16, }, // America/Indiana/Tell_City
    {  623, 8038,  199436, 13, }, // America/Indiana/Vevay
    {  623, 8060,  199449, 16, }, // America/Indiana/Vincennes
    {  623, 8086,  199465, 16, }, // America/Indiana/Winamac
    {  623, 8110,  199481,  6, }, // America/Inuvik
    {  623,  660,  199487,  8, }, // America/Iqaluit
    {  623, 2799,  199495,  7, }, // America/Jamaica
    {  623, 5380,  199502,  6, }, // America/Juneau
    {  623,  521,  199508,  9, }, // America/Kentucky/Louisville
    {  623, 8125,  199517, 18, }, // America/Kentucky/Monticello
    {  623, 5704,  199535, 10, }, // America/Kralendijk
    {  623, 4376,  104035,  6, }, // America/La_Paz
    {  623, 7169,  152645,  4, }, // America/Lima
    {  623, 3239,  152649, 10, }, // America/Los_Angeles
    {  623, 5932,  199545, 17, }, // America/Lower_Princes
    {  623, 8153,  168623,  6, }, // America/Maceio
    {  623, 8168,  199562,  6, }, // America/Managua
    {  623, 1908,  199568,  5, }, // America/Manaus
    {  623, 5897,  104091,  7, }, // America/Marigot
    {  623, 5805,  199573,  7, }, // America/Martinique
    {  623, 8184,  104106,  9, }, // America/Matamoros
    {  623, 2917,  199580,  7, }, // America/Mazatlan
    {  623, 8202,  199587,  8, }, // America/Menominee
    {  623, 8220,  199595,  6, }, // America/Merida
    {  623, 8235,  104137, 11, }, // America/Metlakatla
    {  623, 2949,  199601, 11, }, // America/Mexico_City
    {  623, 4391,  199612,  8, }, // America/Miquelon
    {  623, 8254,  199620,  6, }, // America/Moncton
    {  623, 8270,  199626,  7, }, // America/Monterrey
    {  623, 4098,  199633, 11, }, // America/Montevideo
    {  623, 5824,  199644,  9, }, // America/Montserrat
    {  623, 5481,  104199,  4, }, // America/Nassau
    {  623, 2379,  199653,  8, }, // America/New_York
    {  623, 8288,   11422,  3, }, // America/Nome
    {  623, 1850,  104212,  7, }, // America/Noronha
    {  623, 8301,  199661, 19, }, // America/North_Dakota/Beulah
    {  623, 8329,  199680, 16, }, // America/North_Dakota/Center
    {  623, 8357,  199696, 21, }, // America/North_Dakota/New_Salem
    {  623,  393,  103866,  3, }, // America/Nuuk
    {  623, 8388,  155480,  8, }, // America/Ojinaga
    {  623, 2356,  199717,  5, }, // America/Panama
    {  623, 6173,  199722,  9, }, // America/Paramaribo
    {  623, 2973,  152890,  5, }, // America/Phoenix
    {  623, 3945,  199731, 12, }, // America/Port-au-Prince
    {  623, 5954,  199743, 13, }, // America/Port_of_Spain
    {  623, 8404,  199756, 11, }, // America/Porto_Velho
    {  623, 5843,  199767, 11, }, // America/Puerto_Rico
    {  623, 4030,  199778, 11, }, // America/Punta_Arenas
    {  623, 8424,  199789, 11, }, // America/Rankin_Inlet
    {  623, 8445,  199800,  6, }, // America/Recife
    {  623, 1995,  168914,  6, }, // America/Regina
    {  623, 8460,  199806,  8, }, // America/Resolute
    {  623,  695,  199814, 11, }, // America/Rio_Branco
    {  623, 8477,  199825,  7, }, // America/Santarem
    {  623, 2201,  199832,  7, }, // America/Santiago
    {  623, 6111,  199839, 12, }, // America/Santo_Domingo
    {  623, 1878,  104449,  8, }, // America/Sao_Paulo
    {  623, 6899,  199851, 15, }, // America/Scoresbysund
    {  623, 8494,  199866,  5, }, // America/Sitka
    {  623, 8508,  199871, 13, }, // America/St_Barthelemy
    {  623, 2061,  199884,  9, }, // America/St_Johns
    {  623, 5863,  199893,  8, }, // America/St_Kitts
    {  623, 5880,  199901, 10, }, // America/St_Lucia
    {  623,  855,  199911, 10, }, // America/St_Thomas
    {  623, 5913,  199921, 10, }, // America/St_Vincent
    {  623, 8530,  199931, 10, }, // America/Swift_Current
    {  623, 5461,  199941, 11, }, // America/Tegucigalpa
    {  623, 5756,  199952,  5, }, // America/Thule
    {  623,  313,  199957,  7, }, // America/Tijuana
    {  623,  608,  199964,  6, }, // America/Toronto
    {  623, 5688,  199970,  7, }, // America/Tortola
    {  623, 2093,  153143,  7, }, // America/Vancouver
    {  623, 2144,  199977, 10, }, // America/Whitehorse
    {  623,  734,  199987,  7, }, // America/Winnipeg
    {  623, 8552,  199994,  6, }, // America/Yakutat
    {  623, 6001,  104629,  4, }, // Antarctica/Casey
    {  623, 6094,  200000,  4, }, // Antarctica/Davis
    {  623, 6133,  200004, 16, }, // Antarctica/DumontDUrville
    {  623, 8568,  200020,  7, }, // Antarctica/Macquarie
    {  623, 7066,  200027,  5, }, // Antarctica/Mawson
    {  623, 7113,  200032,  8, }, // Antarctica/McMurdo
    {  623, 8589,  200040,  5, }, // Antarctica/Palmer
    {  623, 7226,  104671,  7, }, // Antarctica/Rothera
    {  623, 7295,  104678,  5, }, // Antarctica/Syowa
    {  623, 8607,  200045,  4, }, // Antarctica/Troll
    {  623, 7374,  104683,  6, }, // Antarctica/Vostok
    {  623, 1429,  200049, 13, }, // Arctic/Longyearbyen
    {  623, 5570,   11948,  3, }, // Asia/Aden
    {  623, 5395,  200062,  6, }, // Asia/Almaty
    {  623, 3968,  200068,  5, }, // Asia/Amman
    {  623, 5511,  200073,  5, }, // Asia/Anadyr
    {  623, 5523,  200078,  5, }, // Asia/Aqtau
    {  623, 5368,  200083,  5, }, // Asia/Aqtobe
    {  623,  964,  200088,  5, }, // Asia/Ashgabat
    {  623, 8624,  200093,  6, }, // Asia/Atyrau
    {  623, 3607,   11991,  5, }, // Asia/Baghdad
    {  623, 5534,  153282,  5, }, // Asia/Bahrain
    {  623, 3653,  152318,  4, }, // Asia/Baku
    {  623, 4466,  153287,  5, }, // Asia/Bangkok
    {  623, 3571,  200099,  5, }, // Asia/Barnaul
    {  623, 4086,  153292,  5, }, // Asia/Beirut
    {  623, 3778,  153297,  5, }, // Asia/Bishkek
    {  623, 5989,  155867,  6, }, // Asia/Brunei
    {  623, 4601,  155873,  4, }, // Asia/Chita
    {  623, 4511,  153308,  6, }, // Asia/Colombo
    {  623, 4540,   12044,  4, }, // Asia/Damascus
    {  623, 1093,  200104,  5, }, // Asia/Dhaka
    {  623, 6192,  200109,  3, }, // Asia/Dili
    {  623, 3596,  200112,  4, }, // Asia/Dubai
    {  623, 6159,  200116,  7, }, // Asia/Dushanbe
    {  623, 8636,  200123,  9, }, // Asia/Famagusta
    {  623, 8651,  104843,  3, }, // Asia/Gaza
    {  623, 4795,  104846,  6, }, // Asia/Hebron
    {  623, 1259,  200132, 13, }, // Asia/Ho_Chi_Minh
    {  623, 2704,  104857,  9, }, // Asia/Hong_Kong
    {  623, 4771,  200145,  6, }, // Asia/Hovd
    {  623, 4184,  200151,  6, }, // Asia/Irkutsk
    {  623, 7006,  200157,  6, }, // Asia/Jakarta
    {  623, 6992,  200163,  7, }, // Asia/Jayapura
    {  623, 1290,  104892,  6, }, // Asia/Jerusalem
    {  623, 3560,  153394,  4, }, // Asia/Kabul
    {  623, 4316,  200170,  7, }, // Asia/Kamchatka
    {  623, 4239,  155964,  5, }, // Asia/Karachi
    {  623, 1185,  200177,  8, }, // Asia/Kathmandu
    {  623, 8661,  200185,  6, }, // Asia/Khandyga
    {  623,  992,  200191,  7, }, // Asia/Kolkata
    {  623, 4197,  200198, 11, }, // Asia/Krasnoyarsk
    {  623, 4859,  200209, 11, }, // Asia/Kuala_Lumpur
    {  623, 5976,  200220,  6, }, // Asia/Kuching
    {  623, 5547,  156017,  4, }, // Asia/Kuwait
    {  623, 1211,  200226,  4, }, // Asia/Macau
    {  623, 4017,  200230,  6, }, // Asia/Magadan
    {  623, 1349,  200236,  5, }, // Asia/Makassar
    {  623, 7182,  153473,  5, }, // Asia/Manila
    {  623, 6920,   12250,  4, }, // Asia/Muscat
    {  623, 2554,  200241,  6, }, // Asia/Nicosia
    {  623, 8675,  200247, 12, }, // Asia/Novokuznetsk
    {  623, 4135,  105020, 11, }, // Asia/Novosibirsk
    {  623, 4229,  105031,  5, }, // Asia/Omsk
    {  623, 7145,  156068,  5, }, // Asia/Oral
    {  623, 6961,  153508,  7, }, // Asia/Phnom_Penh
    {  623, 8693,  200259,  8, }, // Asia/Pontianak
    {  623, 4214,  200267, 10, }, // Asia/Pyongyang
    {  623, 5559,   12318,  3, }, // Asia/Qatar
    {  623, 8708,  200277,  7, }, // Asia/Qostanay
    {  623, 4269,  200284,  9, }, // Asia/Qyzylorda
    {  623, 3584,  105083,  4, }, // Asia/Riyadh
    {  623, 4408,  200293,  6, }, // Asia/Sakhalin
    {  623, 7245,   12350,  6, }, // Asia/Samarkand
    {  623, 3279,  200299,  5, }, // Asia/Seoul
    {  623, 1053,  153556,  7, }, // Asia/Shanghai
    {  623, 3300,  156136,  7, }, // Asia/Singapore
    {  623, 4297,  200304, 13, }, // Asia/Srednekolymsk
    {  623, 3263,  200317,  5, }, // Asia/Taipei
    {  623, 4781,  153589,  6, }, // Asia/Tashkent
    {  623, 3915,  200322,  6, }, // Asia/Tbilisi
    {  623, 2772,  200328,  5, }, // Asia/Tehran
    {  623, 1317,  200333,  5, }, // Asia/Thimphu
    {  623, 2821,  105166,  5, }, // Asia/Tokyo
    {  623, 4572,  200338,  5, }, // Asia/Tomsk
    {  623, 1021,  200343, 11, }, // Asia/Ulaanbaatar
    {  623, 1159,  200354,  7, }, // Asia/Urumqi
    {  623, 8722,  200361,  9, }, // Asia/Ust-Nera
    {  623, 6977,  200370,  8, }, // Asia/Vientiane
    {  623, 4710,  200378, 11, }, // Asia/Vladivostok
    {  623, 4828,  200389,  6, }, // Asia/Yakutsk
    {  623, 1235,  169586,  5, }, // Asia/Yangon
    {  623, 3883,  200395, 11, }, // Asia/Yekaterinburg
    {  623, 3747,  200406,  6, }, // Asia/Yerevan
    {  623, 3663,  200412,  6, }, // Atlantic/Azores
    {  623, 5671,   12497,  6, }, // Atlantic/Bermuda
    {  623, 6628,  155556,  5, }, // Atlantic/Canary
    {  623, 3727,  200418,  8, }, // Atlantic/Cape_Verde
    {  623, 1395,   12520,  4, }, // Atlantic/Faroe
    {  623, 8736,  200426,  6, }, // Atlantic/Madeira
    {  623, 2748,  200432,  7, }, // Atlantic/Reykjavik
    {  623, 7272,  105299, 12, }, // Atlantic/South_Georgia
    {  623, 6821,  200439, 11, }, // Atlantic/St_Helena
    {  623, 6644,  200450,  7, }, // Atlantic/Stanley
    {  623, 1670,  200457,  7, }, // Australia/Adelaide
    {  623, 1635,  105335,  6, }, // Australia/Brisbane
    {  623, 1799,  200464,  9, }, // Australia/Broken_Hill
    {  623, 1583,  200473,  5, }, // Australia/Darwin
    {  623, 3637,  200478,  6, }, // Australia/Eucla
    {  623, 1516,  200484,  6, }, // Australia/Hobart
    {  623, 8753,  200490,  7, }, // Australia/Lindeman
    {  623, 1547,  200497,  9, }, // Australia/Lord_Howe
    {  623, 1727,  156361,  6, }, // Australia/Melbourne
    {  623, 1762,  200506,  4, }, // Australia/Perth
    {  623, 1463,  200510,  4, }, // Australia/Sydney
    {  623, 6441,  200514,  8, }, // Europe/Amsterdam
    {  623, 6216,  200522,  6, }, // Europe/Andorra
    {  623, 3620,  200528,  8, }, // Europe/Astrakhan
    {  623, 2300,  200536,  6, }, // Europe/Athens
    {  623, 6336,   12667,  6, }, // Europe/Belgrade
    {  623, 4757,  153847,  4, }, // Europe/Berlin
    {  623, 6488,  200542, 10, }, // Europe/Bratislava
    {  623, 2167,  169744,  5, }, // Europe/Brussels
    {  623, 3928,  200552,  6, }, // Europe/Bucharest
    {  623, 3806,  200558,  6, }, // Europe/Budapest
    {  623, 8772,  200564,  5, }, // Europe/Busingen
    {  623, 2583,  200569,  7, }, // Europe/Chisinau
    {  623, 6289,  105498,  9, }, // Europe/Copenhagen
    {  623, 2338,  200576,  4, }, // Europe/Dublin
    {  623, 6307,  153904, 10, }, // Europe/Gibraltar
    {  623, 8788,  156466,  5, }, // Europe/Guernsey
    {  623, 6612,  200580,  7, }, // Europe/Helsinki
    {  623, 8804,  105537, 10, }, // Europe/Isle_of_Man
    {  623, 1130,  153938,  7, }, // Europe/Istanbul
    {  623, 8823,  105556,  4, }, // Europe/Jersey
    {  623, 3979,  200587, 11, }, // Europe/Kaliningrad
    {  623, 8929,  153955,  5, }, // Europe/Kirov
    {  623, 2527,  103079,  3, }, // Europe/Kyiv
    {  623, 3213,  169843,  4, }, // Europe/Lisbon
    {  623, 6506,  156510,  9, }, // Europe/Ljubljana
    {  623, 2501,   12808,  4, }, // Europe/London
    {  623, 6365,  200598,  7, }, // Europe/Luxembourg
    {  623, 4892,  200605,  5, }, // Europe/Madrid
    {  623, 6397,  105621,  5, }, // Europe/Malta
    {  623, 6582,  200610,  7, }, // Europe/Mariehamn
    {  623, 3693,  105634,  5, }, // Europe/Minsk
    {  623, 6410,  105639,  6, }, // Europe/Monaco
    {  623, 3537,  105645,  5, }, // Europe/Moscow
    {  623, 6458,  105650,  5, }, // Europe/Oslo
    {  623, 4284,  200617,  4, }, // Europe/Paris
    {  623, 6424,  200621,  9, }, // Europe/Podgorica
    {  623, 6275,  105669,  4, }, // Europe/Prague
    {  623, 8837,  155316,  4, }, // Europe/Riga
    {  623, 6324,   10638,  3, }, // Europe/Rome
    {  623, 4331,  106627,  5, }, // Europe/Samara
    {  623, 6470,  200630, 11, }, // Europe/San_Marino
    {  623, 6245,  169912,  7, }, // Europe/Sarajevo
    {  623, 4451,  200641,  6, }, // Europe/Saratov
    {  623, 8849,  169924,  8, }, // Europe/Simferopol
    {  623, 6383,  200647,  7, }, // Europe/Skopje
    {  623, 6599,  200654,  5, }, // Europe/Sofia
    {  623, 6523,  200659,  9, }, // Europe/Stockholm
    {  623, 8867,  200668,  4, }, // Europe/Tallinn
    {  623, 6202,  200672,  6, }, // Europe/Tirane
    {  623, 8882,  200678,  8, }, // Europe/Ulyanovsk
    {  623, 6352,  200686,  5, }, // Europe/Vaduz
    {  623, 6567,  200691,  6, }, // Europe/Vatican
    {  623, 6231,  154095,  5, }, // Europe/Vienna
    {  623, 8899,  200697,  7, }, // Europe/Vilnius
    {  623, 4727,  169980,  9, }, // Europe/Volgograd
    {  623, 3190,  105795,  5, }, // Europe/Warsaw
    {  623, 6261,  200704,  6, }, // Europe/Zagreb
    {  623, 6540,  200710,  5, }, // Europe/Zurich
    {  623, 5078,  200715, 12, }, // Indian/Antananarivo
    {  623, 6947,  105823,  5, }, // Indian/Chagos
    {  623, 6046,  200727,  5, }, // Indian/Christmas
    {  623, 6063,  105835,  5, }, // Indian/Cocos
    {  623, 5029,  105840,  6, }, // Indian/Comoro
    {  623, 6661,  200732,  8, }, // Indian/Kerguelen
    {  623, 7260,  105854,  4, }, // Indian/Mahe
    {  623, 7050,  200740,  6, }, // Indian/Maldives
    {  623, 4069,  200746,  6, }, // Indian/Mauritius
    {  623, 5098,  200752,  5, }, // Indian/Mayotte
    {  623, 7211,  200757,  8, }, // Indian/Reunion
    {  623, 4422,  154184,  4, }, // Pacific/Apia
    {  623,  932,  105892,  6, }, // Pacific/Auckland
    {  623, 3706,  200765,  8, }, // Pacific/Bougainville
    {  623, 3015,  200773,  5, }, // Pacific/Chatham
    {  623, 3157,  200778,  4, }, // Pacific/Chuuk
    {  623, 2237,  105420,  5, }, // Pacific/Easter
    {  623, 7360,  200782,  5, }, // Pacific/Efate
    {  623, 7327,  200787,  6, }, // Pacific/Fakaofo
    {  623, 3902,  200793,  3, }, // Pacific/Fiji
    {  623, 7343,  200796, 10, }, // Pacific/Funafuti
    {  623, 6678,  200806,  9, }, // Pacific/Galapagos
    {  623, 6696,  200815,  7, }, // Pacific/Gambier
    {  623, 3822,  200822, 11, }, // Pacific/Guadalcanal
    {  623, 6018,  199565,  4, }, // Pacific/Guam
    {  623, 2723,  200833,  8, }, // Pacific/Honolulu
    {  623, 3049,  200841,  8, }, // Pacific/Kanton
    {  623, 3998,  200849,  9, }, // Pacific/Kiritimati
    {  623, 7019,  200858,  7, }, // Pacific/Kosrae
    {  623, 2842,  156816,  8, }, // Pacific/Kwajalein
    {  623, 4877,  154287,  5, }, // Pacific/Majuro
    {  623, 4051,  200865,  8, }, // Pacific/Marquesas
    {  623, 8914,  200873,  4, }, // Pacific/Midway
    {  623, 7084,  200877,  5, }, // Pacific/Nauru
    {  623, 7132,  199504,  4, }, // Pacific/Niue
    {  623, 4168,  106037,  6, }, // Pacific/Norfolk
    {  623, 7098,  200882,  5, }, // Pacific/Noumea
    {  623, 3126,  170185,  9, }, // Pacific/Pago_Pago
    {  623, 7155,  200887,  4, }, // Pacific/Palau
    {  623, 7194,  200891,  7, }, // Pacific/Pitcairn
    {  623, 3096,  200898,  7, }, // Pacific/Pohnpei
    {  623, 4807,  200905, 12, }, // Pacific/Port_Moresby
    {  623, 6076,  106086,  9, }, // Pacific/Rarotonga
    {  623, 6031,  200917,  6, }, // Pacific/Saipan
    {  623, 7312,  200923,  6, }, // Pacific/Tahiti
    {  623, 6712,  200929,  5, }, // Pacific/Tarawa
    {  623, 4583,  200934,  9, }, // Pacific/Tongatapu
    {  623, 7392,  106123,  3, }, // Pacific/Wake
    {  623, 7405,  200943,  5, }, // Pacific/Wallis
    {  624, 6760,  151787,  4, }, // Africa/Accra Urdu/Arabic/India
    {  624, 7713,  200948,  9, }, // America/Cambridge_Bay
    {  624, 7924,  200957,  7, }, // America/Glace_Bay
    {  624, 6881,  200964,  6, }, // America/Goose_Bay
    {  624, 4612,  200970,  9, }, // America/Grand_Turk
    {  624, 6899,  200979, 14, }, // America/Scoresbysund
    {  624, 5756,  200993,  5, }, // America/Thule
    {  624, 3806,  200998,  8, }, // Europe/Budapest
    {  625,   14,       0,  6, }, // Africa/Asmara Uyghur/Arabic/China
    {  625, 4435,  201006,  8, }, // Africa/Sao_Tome
    {  625, 4252,  201014, 10, }, // America/Asuncion
    {  625,  237,      22,  8, }, // America/Atikokan
    {  625, 7623,      30, 17, }, // America/Bahia_Banderas
    {  625, 3868,      47,  6, }, // America/Cancun
    {  625, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  625, 5723,  201024,  6, }, // America/Curacao
    {  625,  481,      73, 13, }, // America/Indiana/Knox
    {  625, 7961,      86, 16, }, // America/Indiana/Marengo
    {  625, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  625, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  625, 8038,     139, 14, }, // America/Indiana/Vevay
    {  625, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  625, 8086,     171, 16, }, // America/Indiana/Winamac
    {  625, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  625, 5932,     207, 22, }, // America/Lower_Princes
    {  625, 8220,     229,  6, }, // America/Merida
    {  625, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  625, 8329,     255, 20, }, // America/North_Dakota/Center
    {  625, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  625,  393,     298,  4, }, // America/Nuuk
    {  625, 6899,     302, 16, }, // America/Scoresbysund
    {  625, 8508,  201030, 18, }, // America/St_Barthelemy
    {  625, 2061,     332, 10, }, // America/St_Johns
    {  625, 5863,     342,  9, }, // America/St_Kitts
    {  625, 5880,     351,  9, }, // America/St_Lucia
    {  625,  855,     360, 10, }, // America/St_Thomas
    {  625, 5913,     370, 11, }, // America/St_Vincent
    {  625, 6133,  201048, 15, }, // Antarctica/DumontDUrville
    {  625, 1259,  201063, 14, }, // Asia/Ho_Chi_Minh
    {  625, 1185,     413,  9, }, // Asia/Kathmandu
    {  625,  992,     422,  7, }, // Asia/Kolkata
    {  625, 1211,     429,  5, }, // Asia/Macau
    {  625, 1235,     442,  6, }, // Asia/Yangon
    {  625, 1395,     448,  5, }, // Atlantic/Faroe
    {  625, 6821,     453, 10, }, // Atlantic/St_Helena
    {  625, 2527,     463,  4, }, // Europe/Kyiv
    {  625, 7211,  201077,  9, }, // Indian/Reunion
    {  625, 3157,     474,  5, }, // Pacific/Chuuk
    {  625, 3096,     479,  7, }, // Pacific/Pohnpei
    {  626, 6760,   21174,  5, }, // Africa/Accra Uzbek/Latin/Uzbekistan
    {  626, 5059,   64509, 11, }, // Africa/Addis_Ababa
    {  626, 7420,  201086,  6, }, // Africa/Algiers
    {  626,   14,   91122,  6, }, // Africa/Asmara
    {  626, 5257,   26348,  5, }, // Africa/Bangui
    {  626, 7435,    6195,  5, }, // Africa/Bissau
    {  626, 4957,  201092,  8, }, // Africa/Blantyre
    {  626, 5287,   21216,  9, }, // Africa/Brazzaville
    {  626, 2320,  201100,  6, }, // Africa/Cairo
    {  626, 4117,    4577, 10, }, // Africa/Casablanca
    {  626, 7449,   21231,  5, }, // Africa/Ceuta
    {  626, 6773,    6249,  7, }, // Africa/Conakry
    {  626, 5130,  201106, 12, }, // Africa/Dar_es_Salaam
    {  626, 5043,  158372,  6, }, // Africa/Djibouti
    {  626, 5243,   21254,  5, }, // Africa/Douala
    {  626, 5166,  201118,  7, }, // Africa/El_Aaiun
    {  626, 6853,    6282,  7, }, // Africa/Freetown
    {  626, 5015,  201125,  6, }, // Africa/Harare
    {  626, 4479,  201131, 12, }, // Africa/Johannesburg
    {  626, 4524,   21284,  6, }, // Africa/Khartoum
    {  626, 5336,    6311,  8, }, // Africa/Libreville
    {  626, 4987,  122060,  6, }, // Africa/Maputo
    {  626, 5113,  201143,  9, }, // Africa/Mogadishu
    {  626, 7034,  201152,  9, }, // Africa/Monrovia
    {  626, 3853,  201161,  7, }, // Africa/Nairobi
    {  626, 6727,   26402,  8, }, // Africa/Ouagadougou
    {  626, 4435,  195475,  8, }, // Africa/Sao_Tome
    {  626, 4152,  201168,  7, }, // Africa/Windhoek
    {  626,  132,  201175, 10, }, // America/Adak
    {  626, 3346,  201185,  7, }, // America/Anchorage
    {  626, 5607,   21368,  7, }, // America/Anguilla
    {  626,  166,  201192, 12, }, // America/Argentina/Buenos_Aires
    {  626,   91,    6400,  9, }, // America/Argentina/Catamarca
    {  626,  270,    6409,  7, }, // America/Argentina/Cordoba
    {  626, 7462,  201204,  8, }, // America/Argentina/La_Rioja
    {  626, 7489,  201212, 12, }, // America/Argentina/Rio_Gallegos
    {  626, 7544,  201224,  8, }, // America/Argentina/San_Juan
    {  626, 5580,  195541,  8, }, // America/Argentina/San_Luis
    {  626, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {  626, 7597,  201232,  7, }, // America/Argentina/Ushuaia
    {  626, 4252,  201239,  7, }, // America/Asuncion
    {  626,  237,  201246, 12, }, // America/Atikokan
    {  626, 3679,  201258,  5, }, // America/Bahia
    {  626, 7623,  201263, 15, }, // America/Bahia_Banderas
    {  626, 5407,    6496,  5, }, // America/Belize
    {  626, 7660,    6502, 12, }, // America/Blanc-Sablon
    {  626, 7681,    6514,  9, }, // America/Boa_Vista
    {  626, 7699,   21446,  5, }, // America/Boise
    {  626, 7713,  201278, 11, }, // America/Cambridge_Bay
    {  626, 7735,  195610, 12, }, // America/Campo_Grande
    {  626, 3868,    6564,  6, }, // America/Cancun
    {  626, 4694,    4792,  7, }, // America/Caracas
    {  626, 4345,  201289,  7, }, // America/Cayenne
    {  626, 5496,  201296, 15, }, // America/Cayman
    {  626, 2260,  201311,  7, }, // America/Chicago
    {  626, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  626, 5422,  195656, 10, }, // America/Costa_Rica
    {  626, 7796,    6618,  7, }, // America/Creston
    {  626, 3791,   21513,  6, }, // America/Cuiaba
    {  626, 5723,  201318,  8, }, // America/Curacao
    {  626, 7812,  201326, 12, }, // America/Danmarkshavn
    {  626, 7833,    6651,  6, }, // America/Dawson
    {  626, 7848,  195692, 11, }, // America/Dawson_Creek
    {  626, 3465,   21541,  7, }, // America/Detroit
    {  626, 5739,   21548,  8, }, // America/Dominica
    {  626, 7869,  201338,  8, }, // America/Eirunepe
    {  626, 5441,    6688,  8, }, // America/El_Salvador
    {  626, 7924,  201346,  9, }, // America/Glace_Bay
    {  626, 6881,  201355,  7, }, // America/Goose_Bay
    {  626, 5786,  122509,  9, }, // America/Guadeloupe
    {  626, 3760,   33206,  9, }, // America/Guatemala
    {  626, 4841,  201362,  8, }, // America/Guayaquil
    {  626, 6932,    4980,  6, }, // America/Guyana
    {  626, 1939,  195763,  8, }, // America/Halifax
    {  626, 2281,  201370,  6, }, // America/Havana
    {  626, 7942,  201376,  9, }, // America/Hermosillo
    {  626,  481,  195785, 13, }, // America/Indiana/Knox
    {  626, 7961,      86, 16, }, // America/Indiana/Marengo
    {  626, 7985,  201385, 19, }, // America/Indiana/Petersburg
    {  626, 8012,    6843, 18, }, // America/Indiana/Tell_City
    {  626, 8038,  201404, 14, }, // America/Indiana/Vevay
    {  626, 8060,  201418, 16, }, // America/Indiana/Vincennes
    {  626, 8086,  201434, 16, }, // America/Indiana/Winamac
    {  626,  660,    5059,  7, }, // America/Iqaluit
    {  626, 2799,   21718,  7, }, // America/Jamaica
    {  626, 5380,  201450,  4, }, // America/Juneau
    {  626,  521,  201454,  8, }, // America/Kentucky/Louisville
    {  626, 8125,  201462, 20, }, // America/Kentucky/Monticello
    {  626, 5704,  201482, 10, }, // America/Kralendijk
    {  626, 4376,  195878,  6, }, // America/La_Paz
    {  626, 3239,  195884, 11, }, // America/Los_Angeles
    {  626, 5932,  201492, 21, }, // America/Lower_Princes
    {  626, 8153,   21792,  6, }, // America/Maceio
    {  626, 5897,   61739,  6, }, // America/Marigot
    {  626, 5805,   26462,  9, }, // America/Martinique
    {  626, 2917,    5161,  8, }, // America/Mazatlan
    {  626, 8202,    7000,  8, }, // America/Menominee
    {  626, 8220,    7008,  6, }, // America/Merida
    {  626, 2949,   21820,  6, }, // America/Mexico_City
    {  626, 4391,    7031,  7, }, // America/Miquelon
    {  626, 8254,    7038,  7, }, // America/Moncton
    {  626, 2379,  201513,  8, }, // America/New_York
    {  626, 8288,   21850,  3, }, // America/Nome
    {  626, 1850,   21853,  7, }, // America/Noronha
    {  626, 8301,  201521, 22, }, // America/North_Dakota/Beulah
    {  626, 8329,  201543, 23, }, // America/North_Dakota/Center
    {  626, 8357,  201566, 26, }, // America/North_Dakota/New_Salem
    {  626,  393,  201592,  6, }, // America/Nuuk
    {  626, 8388,  201598,  7, }, // America/Ojinaga
    {  626, 2973,    7165,  6, }, // America/Phoenix
    {  626, 3945,  165882, 12, }, // America/Port-au-Prince
    {  626, 5954,  201605, 13, }, // America/Port_of_Spain
    {  626, 8404,  201618, 11, }, // America/Porto_Velho
    {  626, 5843,  196049, 11, }, // America/Puerto_Rico
    {  626, 4030,    7204, 12, }, // America/Punta_Arenas
    {  626, 8424,   93659, 12, }, // America/Rankin_Inlet
    {  626, 8445,  123023,  6, }, // America/Recife
    {  626, 1995,  201629,  7, }, // America/Regina
    {  626, 8460,   21975,  8, }, // America/Resolute
    {  626,  695,  196075, 10, }, // America/Rio_Branco
    {  626, 2201,  201636,  8, }, // America/Santiago
    {  626, 6111,    7244, 13, }, // America/Santo_Domingo
    {  626, 1878,  196093,  9, }, // America/Sao_Paulo
    {  626, 6899,  201644, 16, }, // America/Scoresbysund
    {  626, 8508,    7278, 13, }, // America/St_Barthelemy
    {  626, 2061,  196118,  9, }, // America/St_Johns
    {  626, 5863,  196127,  9, }, // America/St_Kitts
    {  626, 5880,  201660, 12, }, // America/St_Lucia
    {  626,  855,  196148, 10, }, // America/St_Thomas
    {  626, 5913,  201672, 12, }, // America/St_Vincent
    {  626, 8530,  201684, 13, }, // America/Swift_Current
    {  626, 5461,    5410, 11, }, // America/Tegucigalpa
    {  626, 5756,  123177,  4, }, // America/Thule
    {  626,  313,   22112,  7, }, // America/Tijuana
    {  626, 2093,    7375,  8, }, // America/Vancouver
    {  626, 2144,  201697,  8, }, // America/Whitehorse
    {  626,  734,  201705,  8, }, // America/Winnipeg
    {  626, 6001,   22135,  5, }, // Antarctica/Casey
    {  626, 6094,   22140,  6, }, // Antarctica/Davis
    {  626, 6133,  201713, 15, }, // Antarctica/DumontDUrville
    {  626, 8568,  196231,  8, }, // Antarctica/Macquarie
    {  626, 7066,   22153,  6, }, // Antarctica/Mawson
    {  626, 7113,  196239,  9, }, // Antarctica/McMurdo
    {  626, 7226,    5465,  6, }, // Antarctica/Rothera
    {  626, 7295,   22168,  5, }, // Antarctica/Syowa
    {  626, 1429,  201728,  7, }, // Arctic/Longyearbyen
    {  626, 5570,  201735,  4, }, // Asia/Aden
    {  626, 5395,    7476,  6, }, // Asia/Almaty
    {  626, 3968,  201739,  5, }, // Asia/Amman
    {  626, 5511,    7482,  6, }, // Asia/Anadyr
    {  626, 5523,  201744,  5, }, // Asia/Aqtau
    {  626, 5368,  201749,  7, }, // Asia/Aqtobe
    {  626,  964,  201756,  8, }, // Asia/Ashgabat
    {  626, 8624,    7494,  6, }, // Asia/Atyrau
    {  626, 3607,  201764,  7, }, // Asia/Baghdad
    {  626, 5534,  201771,  7, }, // Asia/Bahrain
    {  626, 3653,  201778,  4, }, // Asia/Baku
    {  626, 4086,   35871,  6, }, // Asia/Beirut
    {  626, 5989,   22241,  6, }, // Asia/Brunei
    {  626, 4511,    5519,  7, }, // Asia/Colombo
    {  626, 4540,  201782,  7, }, // Asia/Damascus
    {  626, 1093,   91428,  5, }, // Asia/Dhaka
    {  626, 3596,   22258,  5, }, // Asia/Dubai
    {  626, 8651,  201789,  5, }, // Asia/Gaza
    {  626, 4795,  201794,  6, }, // Asia/Hebron
    {  626, 1259,  201800,  8, }, // Asia/Ho_Chi_Minh
    {  626, 2704,  196302,  7, }, // Asia/Hong_Kong
    {  626, 4771,  201808,  4, }, // Asia/Hovd
    {  626, 6992,  201812,  6, }, // Asia/Jayapura
    {  626, 1290,  201818,  6, }, // Asia/Jerusalem
    {  626, 3560,  201824,  5, }, // Asia/Kabul
    {  626, 1185,    5593,  8, }, // Asia/Kathmandu
    {  626, 8661,  201829,  7, }, // Asia/Khandyga
    {  626,  992,   64221,  8, }, // Asia/Kolkata
    {  626, 4859,    7589, 12, }, // Asia/Kuala_Lumpur
    {  626, 5547,  201836,  6, }, // Asia/Kuwait
    {  626, 1211,    7613,  5, }, // Asia/Macau
    {  626, 1349,    7618,  7, }, // Asia/Makassar
    {  626, 6920,   26750,  6, }, // Asia/Muscat
    {  626, 2554,   22378,  8, }, // Asia/Nicosia
    {  626, 7145,   57921,  6, }, // Asia/Oral
    {  626, 6961,   62386,  7, }, // Asia/Phnom_Penh
    {  626, 4214,   22394,  7, }, // Asia/Pyongyang
    {  626, 8708,  201842,  9, }, // Asia/Qostanay
    {  626, 4269,  201851, 10, }, // Asia/Qyzylorda
    {  626, 3584,  201861,  8, }, // Asia/Riyadh
    {  626, 4408,   22410,  7, }, // Asia/Sakhalin
    {  626, 7245,  201869,  9, }, // Asia/Samarkand
    {  626, 3279,    7680,  4, }, // Asia/Seoul
    {  626, 1053,  201878,  7, }, // Asia/Shanghai
    {  626, 3300,   20782,  8, }, // Asia/Singapore
    {  626, 4297,    7699, 13, }, // Asia/Srednekolymsk
    {  626, 3263,  201885,  6, }, // Asia/Taipei
    {  626, 4781,   35978,  8, }, // Asia/Tashkent
    {  626, 2772,  201891,  6, }, // Asia/Tehran
    {  626, 2821,    4238,  5, }, // Asia/Tokyo
    {  626, 1021,  147043, 10, }, // Asia/Ulaanbaatar
    {  626, 1159,  201897,  7, }, // Asia/Urumqi
    {  626, 6977,   22481,  8, }, // Asia/Vientiane
    {  626, 1235,    7762,  6, }, // Asia/Yangon
    {  626, 3663,  201904, 13, }, // Atlantic/Azores
    {  626, 5671,  201917, 16, }, // Atlantic/Bermuda
    {  626, 6628,  201933, 14, }, // Atlantic/Canary
    {  626, 3727,  201947, 10, }, // Atlantic/Cape_Verde
    {  626, 1395,  201957, 14, }, // Atlantic/Faroe
    {  626, 8736,  201971, 13, }, // Atlantic/Madeira
    {  626, 2748,   22544,  9, }, // Atlantic/Reykjavik
    {  626, 7272,  201984, 16, }, // Atlantic/South_Georgia
    {  626, 6821,  202000, 21, }, // Atlantic/St_Helena
    {  626, 6644,    7847,  6, }, // Atlantic/Stanley
    {  626, 1670,   20912,  8, }, // Australia/Adelaide
    {  626, 1635,  123836,  7, }, // Australia/Brisbane
    {  626, 1799,  202021, 11, }, // Australia/Broken_Hill
    {  626, 1583,    7880,  6, }, // Australia/Darwin
    {  626, 3637,  202032,  5, }, // Australia/Eucla
    {  626, 1516,  202037,  6, }, // Australia/Hobart
    {  626, 1547,  202043, 14, }, // Australia/Lord_Howe
    {  626, 1727,    7903,  7, }, // Australia/Melbourne
    {  626, 1762,   22603,  4, }, // Australia/Perth
    {  626, 1463,   22607,  6, }, // Australia/Sydney
    {  626, 3620,  117732,  8, }, // Europe/Astrakhan
    {  626, 2300,   22622,  5, }, // Europe/Athens
    {  626, 6336,   26883,  7, }, // Europe/Belgrade
    {  626, 2167,  202057,  8, }, // Europe/Brussels
    {  626, 3928,   22641,  8, }, // Europe/Bucharest
    {  626, 3806,  202065,  9, }, // Europe/Budapest
    {  626, 8772,  202074,  9, }, // Europe/Busingen
    {  626, 2583,  202083,  9, }, // Europe/Chisinau
    {  626, 6289,  196635, 10, }, // Europe/Copenhagen
    {  626, 8788,  127105,  6, }, // Europe/Guernsey
    {  626, 6612,  202092,  8, }, // Europe/Helsinki
    {  626, 8804,  202100,  9, }, // Europe/Isle_of_Man
    {  626, 8823,  196654,  5, }, // Europe/Jersey
    {  626, 2527,   22717,  5, }, // Europe/Kyiv
    {  626, 3213,    4381,  8, }, // Europe/Lisbon
    {  626, 6506,   22722,  9, }, // Europe/Ljubljana
    {  626, 6365,  202109, 11, }, // Europe/Luxembourg
    {  626, 6582,  202120, 10, }, // Europe/Mariehamn
    {  626, 6410,    5926,  6, }, // Europe/Monaco
    {  626, 3537,   22748,  6, }, // Europe/Moscow
    {  626, 4284,   58798,  5, }, // Europe/Paris
    {  626, 6424,  202130, 10, }, // Europe/Podgorica
    {  626, 6275,   21067,  5, }, // Europe/Prague
    {  626, 6324,   33608,  3, }, // Europe/Rome
    {  626, 6470,    8096, 10, }, // Europe/San_Marino
    {  626, 6245,   22773,  8, }, // Europe/Sarajevo
    {  626, 6383,   22781,  6, }, // Europe/Skopje
    {  626, 6599,   22787,  6, }, // Europe/Sofia
    {  626, 6523,  196728,  8, }, // Europe/Stockholm
    {  626, 8867,   22793,  6, }, // Europe/Tallinn
    {  626, 6202,   21095,  6, }, // Europe/Tirane
    {  626, 6352,   22799,  6, }, // Europe/Vaduz
    {  626, 6567,    5978,  7, }, // Europe/Vatican
    {  626, 6231,  202140,  4, }, // Europe/Vienna
    {  626, 8899,   99220,  7, }, // Europe/Vilnius
    {  626, 3190,  202144,  8, }, // Europe/Warsaw
    {  626, 6540,  202152,  6, }, // Europe/Zurich
    {  626, 5078,   99232, 12, }, // Indian/Antananarivo
    {  626, 6046,  202158, 15, }, // Indian/Christmas
    {  626, 6063,  202173, 14, }, // Indian/Cocos
    {  626, 5029,  202187, 14, }, // Indian/Comoro
    {  626, 6661,   22859,  8, }, // Indian/Kerguelen
    {  626, 7260,  124329,  3, }, // Indian/Mahe
    {  626, 7050,  202201, 15, }, // Indian/Maldives
    {  626, 4069,  202216,  8, }, // Indian/Mauritius
    {  626, 5098,  202224,  7, }, // Indian/Mayotte
    {  626, 7211,  202231,  8, }, // Indian/Reunion
    {  626,  932,  124369,  6, }, // Pacific/Auckland
    {  626, 3706,  124376,  8, }, // Pacific/Bougainville
    {  626, 3015,  202239, 12, }, // Pacific/Chatham
    {  626, 3157,  202251, 13, }, // Pacific/Chuuk
    {  626, 2237,  202264, 11, }, // Pacific/Easter
    {  626, 6696,  202275, 12, }, // Pacific/Gambier
    {  626, 3822,  124440, 11, }, // Pacific/Guadalcanal
    {  626, 2723,  196909,  8, }, // Pacific/Honolulu
    {  626, 3049,  202287, 15, }, // Pacific/Kanton
    {  626, 2842,   22962,  9, }, // Pacific/Kwajalein
    {  626, 4051,  202302, 15, }, // Pacific/Marquesas
    {  626, 8914,  202317, 15, }, // Pacific/Midway
    {  626, 7098,  124531,  5, }, // Pacific/Noumea
    {  626, 3126,    8295,  9, }, // Pacific/Pago_Pago
    {  626, 7194,    8304,  7, }, // Pacific/Pitcairn
    {  626, 3096,  202332, 12, }, // Pacific/Pohnpei
    {  626, 4807,  196953, 11, }, // Pacific/Port_Moresby
    {  626, 6031,   23021,  6, }, // Pacific/Saipan
    {  626, 7312,  202344, 11, }, // Pacific/Tahiti
    {  626, 6712,   23027,  6, }, // Pacific/Tarawa
    {  626, 7392,  202355, 10, }, // Pacific/Wake
    {  626, 7405,    8349,  6, }, // Pacific/Wallis
    {  627,   14,       0,  6, }, // Africa/Asmara Uzbek/Arabic/Afghanistan
    {  627, 4435,       6,  8, }, // Africa/Sao_Tome
    {  627, 4252,      14,  8, }, // America/Asuncion
    {  627,  237,      22,  8, }, // America/Atikokan
    {  627, 7623,      30, 17, }, // America/Bahia_Banderas
    {  627, 3868,      47,  6, }, // America/Cancun
    {  627, 5723,      66,  7, }, // America/Curacao
    {  627,  481,      73, 13, }, // America/Indiana/Knox
    {  627, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  627, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  627, 8038,     139, 14, }, // America/Indiana/Vevay
    {  627, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  627, 8086,     171, 16, }, // America/Indiana/Winamac
    {  627, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  627, 5932,     207, 22, }, // America/Lower_Princes
    {  627, 8220,     229,  6, }, // America/Merida
    {  627, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  627, 8329,     255, 20, }, // America/North_Dakota/Center
    {  627, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  627,  393,     298,  4, }, // America/Nuuk
    {  627, 6899,     302, 16, }, // America/Scoresbysund
    {  627, 8508,     318, 14, }, // America/St_Barthelemy
    {  627, 2061,     332, 10, }, // America/St_Johns
    {  627, 5863,     342,  9, }, // America/St_Kitts
    {  627, 5880,     351,  9, }, // America/St_Lucia
    {  627,  855,     360, 10, }, // America/St_Thomas
    {  627, 5913,     370, 11, }, // America/St_Vincent
    {  627, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  627, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  627, 3560,  153394,  4, }, // Asia/Kabul
    {  627, 1185,     413,  9, }, // Asia/Kathmandu
    {  627,  992,     422,  7, }, // Asia/Kolkata
    {  627, 1211,     429,  5, }, // Asia/Macau
    {  627, 1235,     442,  6, }, // Asia/Yangon
    {  627, 1395,     448,  5, }, // Atlantic/Faroe
    {  627, 6821,     453, 10, }, // Atlantic/St_Helena
    {  627, 2527,     463,  4, }, // Europe/Kyiv
    {  627, 7211,     467,  7, }, // Indian/Reunion
    {  627, 3157,     474,  5, }, // Pacific/Chuuk
    {  627, 3096,     479,  7, }, // Pacific/Pohnpei
    {  628,   14,       0,  6, }, // Africa/Asmara Uzbek/Cyrillic/Uzbekistan
    {  628, 4435,       6,  8, }, // Africa/Sao_Tome
    {  628, 4252,      14,  8, }, // America/Asuncion
    {  628,  237,      22,  8, }, // America/Atikokan
    {  628, 7623,      30, 17, }, // America/Bahia_Banderas
    {  628, 3868,      47,  6, }, // America/Cancun
    {  628, 5723,      66,  7, }, // America/Curacao
    {  628,  481,      73, 13, }, // America/Indiana/Knox
    {  628, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  628, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  628, 8038,     139, 14, }, // America/Indiana/Vevay
    {  628, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  628, 8086,     171, 16, }, // America/Indiana/Winamac
    {  628, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  628, 5932,  202365, 22, }, // America/Lower_Princes
    {  628, 8220,     229,  6, }, // America/Merida
    {  628, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  628, 8329,     255, 20, }, // America/North_Dakota/Center
    {  628, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  628,  393,     298,  4, }, // America/Nuuk
    {  628, 6899,     302, 16, }, // America/Scoresbysund
    {  628, 8508,     318, 14, }, // America/St_Barthelemy
    {  628, 2061,     332, 10, }, // America/St_Johns
    {  628, 5863,     342,  9, }, // America/St_Kitts
    {  628, 5880,     351,  9, }, // America/St_Lucia
    {  628,  855,     360, 10, }, // America/St_Thomas
    {  628, 5913,     370, 11, }, // America/St_Vincent
    {  628, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  628, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  628, 1185,     413,  9, }, // Asia/Kathmandu
    {  628,  992,     422,  7, }, // Asia/Kolkata
    {  628, 1211,     429,  5, }, // Asia/Macau
    {  628, 1235,     442,  6, }, // Asia/Yangon
    {  628, 1395,     448,  5, }, // Atlantic/Faroe
    {  628, 6821,     453, 10, }, // Atlantic/St_Helena
    {  628, 2527,     463,  4, }, // Europe/Kyiv
    {  628, 7211,     467,  7, }, // Indian/Reunion
    {  628, 3157,     474,  5, }, // Pacific/Chuuk
    {  628, 3096,     479,  7, }, // Pacific/Pohnpei
    {  629,   14,       0,  6, }, // Africa/Asmara Vai/Vai/Liberia
    {  629, 4435,       6,  8, }, // Africa/Sao_Tome
    {  629, 4252,      14,  8, }, // America/Asuncion
    {  629,  237,      22,  8, }, // America/Atikokan
    {  629, 7623,      30, 17, }, // America/Bahia_Banderas
    {  629, 3868,      47,  6, }, // America/Cancun
    {  629, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  629, 5723,      66,  7, }, // America/Curacao
    {  629,  481,      73, 13, }, // America/Indiana/Knox
    {  629, 7961,      86, 16, }, // America/Indiana/Marengo
    {  629, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  629, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  629, 8038,     139, 14, }, // America/Indiana/Vevay
    {  629, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  629, 8086,     171, 16, }, // America/Indiana/Winamac
    {  629, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  629, 5932,     207, 22, }, // America/Lower_Princes
    {  629, 8220,     229,  6, }, // America/Merida
    {  629, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  629, 8329,     255, 20, }, // America/North_Dakota/Center
    {  629, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  629,  393,     298,  4, }, // America/Nuuk
    {  629, 6899,     302, 16, }, // America/Scoresbysund
    {  629, 8508,     318, 14, }, // America/St_Barthelemy
    {  629, 2061,     332, 10, }, // America/St_Johns
    {  629, 5863,     342,  9, }, // America/St_Kitts
    {  629, 5880,     351,  9, }, // America/St_Lucia
    {  629,  855,     360, 10, }, // America/St_Thomas
    {  629, 5913,     370, 11, }, // America/St_Vincent
    {  629, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  629, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  629, 1185,     413,  9, }, // Asia/Kathmandu
    {  629,  992,     422,  7, }, // Asia/Kolkata
    {  629, 1211,     429,  5, }, // Asia/Macau
    {  629, 1235,     442,  6, }, // Asia/Yangon
    {  629, 1395,     448,  5, }, // Atlantic/Faroe
    {  629, 6821,     453, 10, }, // Atlantic/St_Helena
    {  629, 2527,     463,  4, }, // Europe/Kyiv
    {  629, 7211,     467,  7, }, // Indian/Reunion
    {  629, 3157,     474,  5, }, // Pacific/Chuuk
    {  629, 3096,     479,  7, }, // Pacific/Pohnpei
    {  631,   14,       0,  6, }, // Africa/Asmara Venda/Latin/South Africa
    {  631, 4435,       6,  8, }, // Africa/Sao_Tome
    {  631, 4252,      14,  8, }, // America/Asuncion
    {  631,  237,      22,  8, }, // America/Atikokan
    {  631, 7623,      30, 17, }, // America/Bahia_Banderas
    {  631, 3868,      47,  6, }, // America/Cancun
    {  631, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  631, 5723,      66,  7, }, // America/Curacao
    {  631,  481,      73, 13, }, // America/Indiana/Knox
    {  631, 7961,      86, 16, }, // America/Indiana/Marengo
    {  631, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  631, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  631, 8038,     139, 14, }, // America/Indiana/Vevay
    {  631, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  631, 8086,     171, 16, }, // America/Indiana/Winamac
    {  631, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  631, 5932,     207, 22, }, // America/Lower_Princes
    {  631, 8220,     229,  6, }, // America/Merida
    {  631, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  631, 8329,     255, 20, }, // America/North_Dakota/Center
    {  631, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  631,  393,     298,  4, }, // America/Nuuk
    {  631, 6899,     302, 16, }, // America/Scoresbysund
    {  631, 8508,     318, 14, }, // America/St_Barthelemy
    {  631, 2061,     332, 10, }, // America/St_Johns
    {  631, 5863,     342,  9, }, // America/St_Kitts
    {  631, 5880,     351,  9, }, // America/St_Lucia
    {  631,  855,     360, 10, }, // America/St_Thomas
    {  631, 5913,     370, 11, }, // America/St_Vincent
    {  631, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  631, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  631, 1185,     413,  9, }, // Asia/Kathmandu
    {  631,  992,     422,  7, }, // Asia/Kolkata
    {  631, 1211,     429,  5, }, // Asia/Macau
    {  631, 1235,     442,  6, }, // Asia/Yangon
    {  631, 1395,     448,  5, }, // Atlantic/Faroe
    {  631, 6821,     453, 10, }, // Atlantic/St_Helena
    {  631, 2527,     463,  4, }, // Europe/Kyiv
    {  631, 7211,     467,  7, }, // Indian/Reunion
    {  631, 3157,     474,  5, }, // Pacific/Chuuk
    {  631, 3096,     479,  7, }, // Pacific/Pohnpei
    {  632,   14,       0,  6, }, // Africa/Asmara Vietnamese/Latin/Vietnam
    {  632, 4435,       6,  8, }, // Africa/Sao_Tome
    {  632, 4252,      14,  8, }, // America/Asuncion
    {  632,  237,      22,  8, }, // America/Atikokan
    {  632, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  632, 3868,   33182,  6, }, // America/Cancun
    {  632, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  632, 5723,      66,  7, }, // America/Curacao
    {  632,  481,      73, 13, }, // America/Indiana/Knox
    {  632, 7961,      86, 16, }, // America/Indiana/Marengo
    {  632, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  632, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  632, 8038,     139, 14, }, // America/Indiana/Vevay
    {  632, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  632, 8086,     171, 16, }, // America/Indiana/Winamac
    {  632, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  632, 5932,     207, 22, }, // America/Lower_Princes
    {  632, 8220,    7008,  6, }, // America/Merida
    {  632, 8301,  202387, 18, }, // America/North_Dakota/Beulah
    {  632, 8329,  202405, 18, }, // America/North_Dakota/Center
    {  632, 8357,  202423, 21, }, // America/North_Dakota/New_Salem
    {  632,  393,     298,  4, }, // America/Nuuk
    {  632, 6899,     302, 16, }, // America/Scoresbysund
    {  632, 8508,     318, 14, }, // America/St_Barthelemy
    {  632, 2061,     332, 10, }, // America/St_Johns
    {  632, 5863,     342,  9, }, // America/St_Kitts
    {  632, 5880,     351,  9, }, // America/St_Lucia
    {  632,  855,     360, 10, }, // America/St_Thomas
    {  632, 5913,     370, 11, }, // America/St_Vincent
    {  632, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  632, 1259,  202444, 14, }, // Asia/Ho_Chi_Minh
    {  632, 2704,  202458,  9, }, // Asia/Hong_Kong
    {  632, 1185,     413,  9, }, // Asia/Kathmandu
    {  632,  992,     422,  7, }, // Asia/Kolkata
    {  632, 1211,  202467,  6, }, // Asia/Macau
    {  632, 4214,  202473, 11, }, // Asia/Pyongyang
    {  632, 8708,     434,  8, }, // Asia/Qostanay
    {  632, 1053,  202484, 10, }, // Asia/Shanghai
    {  632, 3263,  202494,  7, }, // Asia/Taipei
    {  632, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  632, 6977,  202501, 10, }, // Asia/Vientiane
    {  632, 1235,   58486,  7, }, // Asia/Yangon
    {  632, 1395,     448,  5, }, // Atlantic/Faroe
    {  632, 7272,  202511, 11, }, // Atlantic/South_Georgia
    {  632, 6821,     453, 10, }, // Atlantic/St_Helena
    {  632, 8804,  202522,  7, }, // Europe/Isle_of_Man
    {  632, 2527,    8030,  4, }, // Europe/Kyiv
    {  632, 3537,  202529, 10, }, // Europe/Moscow
    {  632, 6275,   36153,  5, }, // Europe/Prague
    {  632, 7211,     467,  7, }, // Indian/Reunion
    {  632, 3157,     474,  5, }, // Pacific/Chuuk
    {  632, 2723,    4493,  8, }, // Pacific/Honolulu
    {  632, 3049,    4501,  9, }, // Pacific/Kanton
    {  632, 3096,     479,  7, }, // Pacific/Pohnpei
    {  633,   14,       0,  6, }, // Africa/Asmara Volapuk/Latin/world
    {  633, 4435,       6,  8, }, // Africa/Sao_Tome
    {  633, 4252,      14,  8, }, // America/Asuncion
    {  633,  237,      22,  8, }, // America/Atikokan
    {  633, 7623,      30, 17, }, // America/Bahia_Banderas
    {  633, 3868,      47,  6, }, // America/Cancun
    {  633, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  633, 5723,      66,  7, }, // America/Curacao
    {  633,  481,      73, 13, }, // America/Indiana/Knox
    {  633, 7961,      86, 16, }, // America/Indiana/Marengo
    {  633, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  633, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  633, 8038,     139, 14, }, // America/Indiana/Vevay
    {  633, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  633, 8086,     171, 16, }, // America/Indiana/Winamac
    {  633, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  633, 5932,     207, 22, }, // America/Lower_Princes
    {  633, 8220,     229,  6, }, // America/Merida
    {  633, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  633, 8329,     255, 20, }, // America/North_Dakota/Center
    {  633, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  633,  393,     298,  4, }, // America/Nuuk
    {  633, 6899,     302, 16, }, // America/Scoresbysund
    {  633, 8508,     318, 14, }, // America/St_Barthelemy
    {  633, 2061,     332, 10, }, // America/St_Johns
    {  633, 5863,     342,  9, }, // America/St_Kitts
    {  633, 5880,     351,  9, }, // America/St_Lucia
    {  633,  855,     360, 10, }, // America/St_Thomas
    {  633, 5913,     370, 11, }, // America/St_Vincent
    {  633, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  633, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  633, 1185,     413,  9, }, // Asia/Kathmandu
    {  633,  992,     422,  7, }, // Asia/Kolkata
    {  633, 1211,     429,  5, }, // Asia/Macau
    {  633, 1235,     442,  6, }, // Asia/Yangon
    {  633, 1395,     448,  5, }, // Atlantic/Faroe
    {  633, 6821,     453, 10, }, // Atlantic/St_Helena
    {  633, 2527,     463,  4, }, // Europe/Kyiv
    {  633, 7211,     467,  7, }, // Indian/Reunion
    {  633, 3157,     474,  5, }, // Pacific/Chuuk
    {  633, 3096,     479,  7, }, // Pacific/Pohnpei
    {  634,   14,       0,  6, }, // Africa/Asmara Vunjo/Latin/Tanzania
    {  634, 4435,       6,  8, }, // Africa/Sao_Tome
    {  634, 4252,      14,  8, }, // America/Asuncion
    {  634,  237,      22,  8, }, // America/Atikokan
    {  634, 7623,      30, 17, }, // America/Bahia_Banderas
    {  634, 3868,      47,  6, }, // America/Cancun
    {  634, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  634, 5723,      66,  7, }, // America/Curacao
    {  634,  481,      73, 13, }, // America/Indiana/Knox
    {  634, 7961,      86, 16, }, // America/Indiana/Marengo
    {  634, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  634, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  634, 8038,     139, 14, }, // America/Indiana/Vevay
    {  634, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  634, 8086,     171, 16, }, // America/Indiana/Winamac
    {  634, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  634, 5932,     207, 22, }, // America/Lower_Princes
    {  634, 8220,     229,  6, }, // America/Merida
    {  634, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  634, 8329,     255, 20, }, // America/North_Dakota/Center
    {  634, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  634,  393,     298,  4, }, // America/Nuuk
    {  634, 6899,     302, 16, }, // America/Scoresbysund
    {  634, 8508,     318, 14, }, // America/St_Barthelemy
    {  634, 2061,     332, 10, }, // America/St_Johns
    {  634, 5863,     342,  9, }, // America/St_Kitts
    {  634, 5880,     351,  9, }, // America/St_Lucia
    {  634,  855,     360, 10, }, // America/St_Thomas
    {  634, 5913,     370, 11, }, // America/St_Vincent
    {  634, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  634, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  634, 1185,     413,  9, }, // Asia/Kathmandu
    {  634,  992,     422,  7, }, // Asia/Kolkata
    {  634, 1211,     429,  5, }, // Asia/Macau
    {  634, 1235,     442,  6, }, // Asia/Yangon
    {  634, 1395,     448,  5, }, // Atlantic/Faroe
    {  634, 6821,     453, 10, }, // Atlantic/St_Helena
    {  634, 2527,     463,  4, }, // Europe/Kyiv
    {  634, 7211,     467,  7, }, // Indian/Reunion
    {  634, 3157,     474,  5, }, // Pacific/Chuuk
    {  634, 3096,     479,  7, }, // Pacific/Pohnpei
    {  635,   14,       0,  6, }, // Africa/Asmara Walloon/Latin/Belgium
    {  635, 4435,       6,  8, }, // Africa/Sao_Tome
    {  635, 4252,      14,  8, }, // America/Asuncion
    {  635,  237,      22,  8, }, // America/Atikokan
    {  635, 7623,      30, 17, }, // America/Bahia_Banderas
    {  635, 3868,      47,  6, }, // America/Cancun
    {  635, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  635, 5723,      66,  7, }, // America/Curacao
    {  635,  481,      73, 13, }, // America/Indiana/Knox
    {  635, 7961,      86, 16, }, // America/Indiana/Marengo
    {  635, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  635, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  635, 8038,     139, 14, }, // America/Indiana/Vevay
    {  635, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  635, 8086,     171, 16, }, // America/Indiana/Winamac
    {  635, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  635, 5932,     207, 22, }, // America/Lower_Princes
    {  635, 8220,     229,  6, }, // America/Merida
    {  635, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  635, 8329,     255, 20, }, // America/North_Dakota/Center
    {  635, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  635,  393,     298,  4, }, // America/Nuuk
    {  635, 6899,     302, 16, }, // America/Scoresbysund
    {  635, 8508,     318, 14, }, // America/St_Barthelemy
    {  635, 2061,     332, 10, }, // America/St_Johns
    {  635, 5863,     342,  9, }, // America/St_Kitts
    {  635, 5880,     351,  9, }, // America/St_Lucia
    {  635,  855,     360, 10, }, // America/St_Thomas
    {  635, 5913,     370, 11, }, // America/St_Vincent
    {  635, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  635, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  635, 1185,     413,  9, }, // Asia/Kathmandu
    {  635,  992,     422,  7, }, // Asia/Kolkata
    {  635, 1211,     429,  5, }, // Asia/Macau
    {  635, 1235,     442,  6, }, // Asia/Yangon
    {  635, 1395,     448,  5, }, // Atlantic/Faroe
    {  635, 6821,     453, 10, }, // Atlantic/St_Helena
    {  635, 2527,     463,  4, }, // Europe/Kyiv
    {  635, 7211,     467,  7, }, // Indian/Reunion
    {  635, 3157,     474,  5, }, // Pacific/Chuuk
    {  635, 3096,     479,  7, }, // Pacific/Pohnpei
    {  636, 6760,    4531,  4, }, // Africa/Accra Walser/Latin/Switzerland
    {  636, 7420,   58423,  6, }, // Africa/Algiers
    {  636,   14,       0,  6, }, // Africa/Asmara
    {  636, 2320,    4572,  5, }, // Africa/Cairo
    {  636, 5043,  202539,  7, }, // Africa/Djibouti
    {  636, 4524,    6305,  6, }, // Africa/Khartoum
    {  636, 5113,   57589,  8, }, // Africa/Mogadishu
    {  636, 6727,    4640,  8, }, // Africa/Ouagadougou
    {  636, 4435,       6,  8, }, // Africa/Sao_Tome
    {  636, 2866,   57624,  8, }, // Africa/Tripoli
    {  636,  270,    6409,  7, }, // America/Argentina/Cordoba
    {  636, 4252,      14,  8, }, // America/Asuncion
    {  636,  237,      22,  8, }, // America/Atikokan
    {  636, 7623,      30, 17, }, // America/Bahia_Banderas
    {  636, 3868,      47,  6, }, // America/Cancun
    {  636, 5496,  202546, 11, }, // America/Cayman
    {  636, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  636, 5723,      66,  7, }, // America/Curacao
    {  636, 2281,  202557,  7, }, // America/Havana
    {  636,  481,      73, 13, }, // America/Indiana/Knox
    {  636, 7961,      86, 16, }, // America/Indiana/Marengo
    {  636, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  636, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  636, 8038,     139, 14, }, // America/Indiana/Vevay
    {  636, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  636, 8086,     171, 16, }, // America/Indiana/Winamac
    {  636, 2799,    3877,  7, }, // America/Jamaica
    {  636, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  636, 5932,     207, 22, }, // America/Lower_Princes
    {  636, 8220,     229,  6, }, // America/Merida
    {  636, 5824,   21834,  8, }, // America/Montserrat
    {  636, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  636, 8329,     255, 20, }, // America/North_Dakota/Center
    {  636, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  636,  393,     298,  4, }, // America/Nuuk
    {  636, 6899,     302, 16, }, // America/Scoresbysund
    {  636, 8508,     318, 14, }, // America/St_Barthelemy
    {  636, 2061,     332, 10, }, // America/St_Johns
    {  636, 5863,     342,  9, }, // America/St_Kitts
    {  636, 5880,     351,  9, }, // America/St_Lucia
    {  636,  855,     360, 10, }, // America/St_Thomas
    {  636, 5913,     370, 11, }, // America/St_Vincent
    {  636, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  636, 3607,    4044,  6, }, // Asia/Baghdad
    {  636, 4540,    4083,  8, }, // Asia/Damascus
    {  636, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  636, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  636, 1185,     413,  9, }, // Asia/Kathmandu
    {  636,  992,     422,  7, }, // Asia/Kolkata
    {  636, 1211,     429,  5, }, // Asia/Macau
    {  636, 3300,   20782,  8, }, // Asia/Singapore
    {  636, 2821,    4238,  5, }, // Asia/Tokyo
    {  636, 1235,     442,  6, }, // Asia/Yangon
    {  636, 1395,     448,  5, }, // Atlantic/Faroe
    {  636, 2748,  202564,  8, }, // Atlantic/Reykjavik
    {  636, 6821,     453, 10, }, // Atlantic/St_Helena
    {  636, 1463,  202572,  6, }, // Australia/Sydney
    {  636, 6441,  202578,  9, }, // Europe/Amsterdam
    {  636, 6336,   26883,  7, }, // Europe/Belgrade
    {  636, 2167,   22634,  7, }, // Europe/Brussels
    {  636, 3928,  127036,  8, }, // Europe/Bucharest
    {  636, 3806,  123966,  8, }, // Europe/Budapest
    {  636, 6289,  178251,  8, }, // Europe/Copenhagen
    {  636, 1130,  202587, 14, }, // Europe/Istanbul
    {  636, 3979,  202601, 10, }, // Europe/Kaliningrad
    {  636, 2527,     463,  4, }, // Europe/Kyiv
    {  636, 6506,  202611,  6, }, // Europe/Ljubljana
    {  636, 6275,    8083,  4, }, // Europe/Prague
    {  636, 6324,    5946,  3, }, // Europe/Rome
    {  636, 8867,  202617,  5, }, // Europe/Tallinn
    {  636, 6202,    8137,  5, }, // Europe/Tirane
    {  636, 6567,    5978,  7, }, // Europe/Vatican
    {  636, 6231,   57383,  4, }, // Europe/Vienna
    {  636, 8899,  127875,  5, }, // Europe/Vilnius
    {  636, 4727,    4443,  9, }, // Europe/Volgograd
    {  636, 3190,  202622,  6, }, // Europe/Warsaw
    {  636, 6540,  202628,  5, }, // Europe/Zurich
    {  636, 7211,     467,  7, }, // Indian/Reunion
    {  636, 3157,     474,  5, }, // Pacific/Chuuk
    {  636, 3096,     479,  7, }, // Pacific/Pohnpei
    {  637,   14,       0,  6, }, // Africa/Asmara Warlpiri/Latin/Australia
    {  637, 4435,       6,  8, }, // Africa/Sao_Tome
    {  637, 4252,      14,  8, }, // America/Asuncion
    {  637,  237,      22,  8, }, // America/Atikokan
    {  637, 7623,      30, 17, }, // America/Bahia_Banderas
    {  637, 3868,      47,  6, }, // America/Cancun
    {  637, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  637, 5723,      66,  7, }, // America/Curacao
    {  637,  481,      73, 13, }, // America/Indiana/Knox
    {  637, 7961,      86, 16, }, // America/Indiana/Marengo
    {  637, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  637, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  637, 8038,     139, 14, }, // America/Indiana/Vevay
    {  637, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  637, 8086,     171, 16, }, // America/Indiana/Winamac
    {  637, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  637, 5932,     207, 22, }, // America/Lower_Princes
    {  637, 8220,     229,  6, }, // America/Merida
    {  637, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  637, 8329,     255, 20, }, // America/North_Dakota/Center
    {  637, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  637,  393,     298,  4, }, // America/Nuuk
    {  637, 6899,     302, 16, }, // America/Scoresbysund
    {  637, 8508,     318, 14, }, // America/St_Barthelemy
    {  637, 2061,     332, 10, }, // America/St_Johns
    {  637, 5863,     342,  9, }, // America/St_Kitts
    {  637, 5880,     351,  9, }, // America/St_Lucia
    {  637,  855,     360, 10, }, // America/St_Thomas
    {  637, 5913,     370, 11, }, // America/St_Vincent
    {  637, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  637, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  637, 1185,     413,  9, }, // Asia/Kathmandu
    {  637,  992,     422,  7, }, // Asia/Kolkata
    {  637, 1211,     429,  5, }, // Asia/Macau
    {  637, 1235,     442,  6, }, // Asia/Yangon
    {  637, 1395,     448,  5, }, // Atlantic/Faroe
    {  637, 6821,     453, 10, }, // Atlantic/St_Helena
    {  637, 2527,     463,  4, }, // Europe/Kyiv
    {  637, 7211,     467,  7, }, // Indian/Reunion
    {  637, 3157,     474,  5, }, // Pacific/Chuuk
    {  637, 3096,     479,  7, }, // Pacific/Pohnpei
    {  638, 7420,   43956,  5, }, // Africa/Algiers Welsh/Latin/United Kingdom
    {  638,   14,       0,  6, }, // Africa/Asmara
    {  638, 4435,       6,  8, }, // Africa/Sao_Tome
    {  638, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  638, 4252,      14,  8, }, // America/Asuncion
    {  638,  237,      22,  8, }, // America/Atikokan
    {  638, 7623,  202633, 12, }, // America/Bahia_Banderas
    {  638, 7646,   35764,  5, }, // America/Belem
    {  638, 4361,   20389,  6, }, // America/Bogota
    {  638, 7713,  202645, 13, }, // America/Cambridge_Bay
    {  638, 3868,      47,  6, }, // America/Cancun
    {  638, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  638, 3791,   26442,  6, }, // America/Cuiaba
    {  638, 5723,      66,  7, }, // America/Curacao
    {  638, 7869,   26448,  8, }, // America/Eirunepe
    {  638,  481,      73, 13, }, // America/Indiana/Knox
    {  638, 7961,      86, 16, }, // America/Indiana/Marengo
    {  638, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  638, 8012,  202658, 19, }, // America/Indiana/Tell_City
    {  638, 8038,     139, 14, }, // America/Indiana/Vevay
    {  638, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  638, 8086,     171, 16, }, // America/Indiana/Winamac
    {  638, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  638, 5932,     207, 22, }, // America/Lower_Princes
    {  638, 8220,    7008,  6, }, // America/Merida
    {  638, 2949,  202677, 13, }, // America/Mexico_City
    {  638, 2379,  202690, 12, }, // America/New_York
    {  638, 8301,  202702, 22, }, // America/North_Dakota/Beulah
    {  638, 8329,  202724, 22, }, // America/North_Dakota/Center
    {  638, 8357,  202746, 25, }, // America/North_Dakota/New_Salem
    {  638,  393,     298,  4, }, // America/Nuuk
    {  638, 6899,     302, 16, }, // America/Scoresbysund
    {  638, 8508,     318, 14, }, // America/St_Barthelemy
    {  638, 2061,     332, 10, }, // America/St_Johns
    {  638, 5863,     342,  9, }, // America/St_Kitts
    {  638, 5880,     351,  9, }, // America/St_Lucia
    {  638,  855,     360, 10, }, // America/St_Thomas
    {  638, 5913,     370, 11, }, // America/St_Vincent
    {  638, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  638, 8651,   63848,  4, }, // Asia/Gaza
    {  638, 1259,  202771, 17, }, // Asia/Ho_Chi_Minh
    {  638, 1290,  202788,  9, }, // Asia/Jerusalem
    {  638, 1185,     413,  9, }, // Asia/Kathmandu
    {  638,  992,     422,  7, }, // Asia/Kolkata
    {  638, 1211,    4203,  5, }, // Asia/Macau
    {  638, 8708,     434,  8, }, // Asia/Qostanay
    {  638, 3915,   78108,  6, }, // Asia/Tbilisi
    {  638, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  638, 1235,     442,  6, }, // Asia/Yangon
    {  638, 6628,  202797, 19, }, // Atlantic/Canary
    {  638, 1395,  202816,  5, }, // Atlantic/Faroe
    {  638, 2748,   36044,  9, }, // Atlantic/Reykjavik
    {  638, 7272,  202821, 10, }, // Atlantic/South_Georgia
    {  638, 6821,     453, 10, }, // Atlantic/St_Helena
    {  638, 2167,  202831,  6, }, // Europe/Brussels
    {  638, 3928,  202837,  8, }, // Europe/Bucharest
    {  638, 2338,  202845,  5, }, // Europe/Dublin
    {  638, 8788,  202850, 11, }, // Europe/Guernsey
    {  638, 8804,  202861, 10, }, // Europe/Isle_of_Man
    {  638, 1130,  202871, 13, }, // Europe/Istanbul
    {  638, 2527,    8030,  4, }, // Europe/Kyiv
    {  638, 2501,  202884,  8, }, // Europe/London
    {  638, 6365,  202892, 10, }, // Europe/Luxembourg
    {  638, 6275,    8083,  4, }, // Europe/Prague
    {  638, 6324,  202902,  7, }, // Europe/Rome
    {  638, 6567,  202909,  9, }, // Europe/Vatican
    {  638, 6231,  202918,  6, }, // Europe/Vienna
    {  638, 6046,  202924, 14, }, // Indian/Christmas
    {  638, 7211,     467,  7, }, // Indian/Reunion
    {  638, 3157,     474,  5, }, // Pacific/Chuuk
    {  638, 2237,  202938, 11, }, // Pacific/Easter
    {  638, 2723,    4493,  8, }, // Pacific/Honolulu
    {  638, 3049,    4501,  9, }, // Pacific/Kanton
    {  638, 3096,     479,  7, }, // Pacific/Pohnpei
    {  639,   14,       0,  6, }, // Africa/Asmara Western Balochi/Arabic/Pakistan
    {  639, 4435,       6,  8, }, // Africa/Sao_Tome
    {  639, 4252,      14,  8, }, // America/Asuncion
    {  639,  237,      22,  8, }, // America/Atikokan
    {  639, 7623,      30, 17, }, // America/Bahia_Banderas
    {  639, 3868,      47,  6, }, // America/Cancun
    {  639, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  639, 5723,      66,  7, }, // America/Curacao
    {  639,  481,      73, 13, }, // America/Indiana/Knox
    {  639, 7961,      86, 16, }, // America/Indiana/Marengo
    {  639, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  639, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  639, 8038,     139, 14, }, // America/Indiana/Vevay
    {  639, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  639, 8086,     171, 16, }, // America/Indiana/Winamac
    {  639, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  639, 5932,     207, 22, }, // America/Lower_Princes
    {  639, 8220,     229,  6, }, // America/Merida
    {  639, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  639, 8329,     255, 20, }, // America/North_Dakota/Center
    {  639, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  639,  393,     298,  4, }, // America/Nuuk
    {  639, 6899,     302, 16, }, // America/Scoresbysund
    {  639, 8508,     318, 14, }, // America/St_Barthelemy
    {  639, 2061,     332, 10, }, // America/St_Johns
    {  639, 5863,     342,  9, }, // America/St_Kitts
    {  639, 5880,     351,  9, }, // America/St_Lucia
    {  639,  855,     360, 10, }, // America/St_Thomas
    {  639, 5913,     370, 11, }, // America/St_Vincent
    {  639, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  639, 5570,   11948,  3, }, // Asia/Aden
    {  639, 5395,  202949,  6, }, // Asia/Almaty
    {  639, 3968,   11379,  4, }, // Asia/Amman
    {  639, 5523,  202955,  5, }, // Asia/Aqtau
    {  639, 5368,  202960,  6, }, // Asia/Aqtobe
    {  639,  964,  202966,  8, }, // Asia/Ashgabat
    {  639, 3607,   11991,  5, }, // Asia/Baghdad
    {  639, 5534,  153282,  5, }, // Asia/Bahrain
    {  639, 3653,  152318,  4, }, // Asia/Baku
    {  639, 3778,  202974,  6, }, // Asia/Bishkek
    {  639, 4511,  202980,  7, }, // Asia/Colombo
    {  639, 4540,   12044,  4, }, // Asia/Damascus
    {  639, 1093,  202987,  4, }, // Asia/Dhaka
    {  639, 3596,  153322,  4, }, // Asia/Dubai
    {  639, 6159,  155889,  6, }, // Asia/Dushanbe
    {  639, 8651,  153340,  3, }, // Asia/Gaza
    {  639, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  639, 1290,  202991,  7, }, // Asia/Jerusalem
    {  639, 3560,  153394,  4, }, // Asia/Kabul
    {  639, 4239,  155964,  5, }, // Asia/Karachi
    {  639, 1185,  202998,  6, }, // Asia/Kathmandu
    {  639,  992,  155984,  5, }, // Asia/Kolkata
    {  639, 5547,  203004,  4, }, // Asia/Kuwait
    {  639, 1211,     429,  5, }, // Asia/Macau
    {  639, 6920,   12250,  4, }, // Asia/Muscat
    {  639, 7145,  156068,  5, }, // Asia/Oral
    {  639, 5559,   12318,  3, }, // Asia/Qatar
    {  639, 4269,  203008,  9, }, // Asia/Qyzylorda
    {  639, 3584,  105083,  4, }, // Asia/Riyadh
    {  639, 7245,   12350,  6, }, // Asia/Samarkand
    {  639, 3279,  105104,  4, }, // Asia/Seoul
    {  639, 4781,  203017,  7, }, // Asia/Tashkent
    {  639, 3915,  203024,  7, }, // Asia/Tbilisi
    {  639, 2772,  153601,  5, }, // Asia/Tehran
    {  639, 1317,  203031,  5, }, // Asia/Thimphu
    {  639, 2821,  156176,  5, }, // Asia/Tokyo
    {  639, 4572,  156181,  5, }, // Asia/Tomsk
    {  639, 1021,  156186, 11, }, // Asia/Ulaanbaatar
    {  639, 1159,  203036,  6, }, // Asia/Urumqi
    {  639, 1235,     442,  6, }, // Asia/Yangon
    {  639, 1395,     448,  5, }, // Atlantic/Faroe
    {  639, 6821,     453, 10, }, // Atlantic/St_Helena
    {  639, 1130,  156487,  8, }, // Europe/Istanbul
    {  639, 8929,  153955,  5, }, // Europe/Kirov
    {  639, 2527,     463,  4, }, // Europe/Kyiv
    {  639, 4331,  109179,  6, }, // Europe/Samara
    {  639, 7050,  203042,  6, }, // Indian/Maldives
    {  639, 7211,     467,  7, }, // Indian/Reunion
    {  639, 3157,     474,  5, }, // Pacific/Chuuk
    {  639, 3096,     479,  7, }, // Pacific/Pohnpei
    {  644, 5059,    3784, 11, }, // Africa/Addis_Ababa Western Frisian/Latin/Netherlands
    {  644,   14,       0,  6, }, // Africa/Asmara
    {  644, 2320,   58585,  5, }, // Africa/Cairo
    {  644, 4524,   58590,  8, }, // Africa/Khartoum
    {  644, 6869,   20300,  4, }, // Africa/Lome
    {  644, 4435,   58598,  8, }, // Africa/Sao_Tome
    {  644,  270,   20361,  7, }, // America/Argentina/Cordoba
    {  644, 7489,   44005, 12, }, // America/Argentina/Rio_Gallegos
    {  644, 7571,   20368,  7, }, // America/Argentina/Tucuman
    {  644, 4252,      14,  8, }, // America/Asuncion
    {  644,  237,      22,  8, }, // America/Atikokan
    {  644, 7623,      30, 17, }, // America/Bahia_Banderas
    {  644, 7646,   35764,  5, }, // America/Belem
    {  644, 4361,   20389,  6, }, // America/Bogota
    {  644, 3868,   33182,  6, }, // America/Cancun
    {  644, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  644, 3791,   26442,  6, }, // America/Cuiaba
    {  644, 5723,      66,  7, }, // America/Curacao
    {  644, 5441,    6688,  8, }, // America/El_Salvador
    {  644,  481,      73, 13, }, // America/Indiana/Knox
    {  644, 7961,      86, 16, }, // America/Indiana/Marengo
    {  644, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  644, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  644, 8038,     139, 14, }, // America/Indiana/Vevay
    {  644, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  644, 8086,     171, 16, }, // America/Indiana/Winamac
    {  644, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  644, 5932,   58606, 24, }, // America/Lower_Princes
    {  644, 8153,   26456,  6, }, // America/Maceio
    {  644, 2917,   26471,  8, }, // America/Mazatlan
    {  644, 8220,     229,  6, }, // America/Merida
    {  644, 2949,  203048, 11, }, // America/Mexico_City
    {  644, 8301,  203059, 20, }, // America/North_Dakota/Beulah
    {  644, 8329,  203079, 20, }, // America/North_Dakota/Center
    {  644, 8357,  203099, 23, }, // America/North_Dakota/New_Salem
    {  644,  393,     298,  4, }, // America/Nuuk
    {  644, 8404,  203122, 11, }, // America/Porto_Velho
    {  644, 1878,   26580,  9, }, // America/Sao_Paulo
    {  644, 6899,     302, 16, }, // America/Scoresbysund
    {  644, 8508,   26589, 16, }, // America/St_Barthelemy
    {  644, 2061,     332, 10, }, // America/St_Johns
    {  644, 5863,     342,  9, }, // America/St_Kitts
    {  644, 5880,     351,  9, }, // America/St_Lucia
    {  644,  855,     360, 10, }, // America/St_Thomas
    {  644, 5913,     370, 11, }, // America/St_Vincent
    {  644, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  644, 5395,   58641,  8, }, // Asia/Almaty
    {  644, 5368,   57073,  6, }, // Asia/Aqtobe
    {  644,  964,    4035,  9, }, // Asia/Ashgabat
    {  644, 3607,    4044,  6, }, // Asia/Baghdad
    {  644, 5534,    4050,  7, }, // Asia/Bahrain
    {  644, 3653,    4057,  5, }, // Asia/Baku
    {  644, 4086,    4062,  7, }, // Asia/Beirut
    {  644, 3778,    4069,  7, }, // Asia/Bishkek
    {  644, 6159,   58449,  8, }, // Asia/Dushanbe
    {  644, 1259,   58664, 15, }, // Asia/Ho_Chi_Minh
    {  644, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  644, 4184,    4121,  8, }, // Asia/Irkutsk
    {  644, 1290,   33369,  9, }, // Asia/Jerusalem
    {  644, 4316,    4143, 10, }, // Asia/Kamchatka
    {  644, 1185,     413,  9, }, // Asia/Kathmandu
    {  644,  992,   35921,  8, }, // Asia/Kolkata
    {  644, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  644, 5547,    4196,  7, }, // Asia/Kuwait
    {  644, 1211,     429,  5, }, // Asia/Macau
    {  644, 7182,   58679,  7, }, // Asia/Manila
    {  644, 6961,   93778, 10, }, // Asia/Phnom_Penh
    {  644, 3584,   20756,  5, }, // Asia/Riyadh
    {  644, 4408,   57952,  8, }, // Asia/Sakhalin
    {  644, 1053,   58686,  8, }, // Asia/Shanghai
    {  644, 4781,   58478,  8, }, // Asia/Tashkent
    {  644, 2772,    7724,  7, }, // Asia/Tehran
    {  644, 2821,    4238,  5, }, // Asia/Tokyo
    {  644, 4828,    4254,  8, }, // Asia/Yakutsk
    {  644, 1235,     442,  6, }, // Asia/Yangon
    {  644, 3883,   58694, 15, }, // Asia/Yekaterinburg
    {  644, 3747,    7780,  7, }, // Asia/Yerevan
    {  644, 3663,   58709,  6, }, // Atlantic/Azores
    {  644, 6628,  203133, 18, }, // Atlantic/Canary
    {  644, 3727,   58734, 10, }, // Atlantic/Cape_Verde
    {  644, 1395,   58744,  7, }, // Atlantic/Faroe
    {  644, 7272,  203151, 11, }, // Atlantic/South_Georgia
    {  644, 6821,   58763, 11, }, // Atlantic/St_Helena
    {  644, 2300,    4320,  6, }, // Europe/Athens
    {  644, 6336,   58774,  8, }, // Europe/Belgrade
    {  644, 4757,    4326,  6, }, // Europe/Berlin
    {  644, 2167,    4332,  7, }, // Europe/Brussels
    {  644, 3928,    4339,  9, }, // Europe/Bucharest
    {  644, 3806,    4348,  9, }, // Europe/Budapest
    {  644, 6289,    4357, 10, }, // Europe/Copenhagen
    {  644, 6612,   58165,  8, }, // Europe/Helsinki
    {  644, 1130,   58789,  9, }, // Europe/Istanbul
    {  644, 2527,     463,  4, }, // Europe/Kyiv
    {  644, 3213,    4381,  8, }, // Europe/Lisbon
    {  644, 2501,    4389,  6, }, // Europe/London
    {  644, 6365,    4395,  9, }, // Europe/Luxembourg
    {  644, 3537,    4404,  6, }, // Europe/Moscow
    {  644, 4284,    4410,  5, }, // Europe/Paris
    {  644, 6275,  203162,  6, }, // Europe/Prague
    {  644, 6202,   21095,  6, }, // Europe/Tirane
    {  644, 6567,  203168, 12, }, // Europe/Vatican
    {  644, 6231,   58816,  5, }, // Europe/Vienna
    {  644, 4727,    4443,  9, }, // Europe/Volgograd
    {  644, 3190,   57396,  8, }, // Europe/Warsaw
    {  644, 6540,    4459,  6, }, // Europe/Zurich
    {  644, 6947,   58821, 14, }, // Indian/Chagos
    {  644, 6046,  203180, 10, }, // Indian/Christmas
    {  644, 6063,  203190, 13, }, // Indian/Cocos
    {  644, 7260,   26983,  4, }, // Indian/Mahe
    {  644, 7050,   58863,  8, }, // Indian/Maldives
    {  644, 7211,     467,  7, }, // Indian/Reunion
    {  644, 3157,     474,  5, }, // Pacific/Chuuk
    {  644, 2237,  203203, 11, }, // Pacific/Easter
    {  644, 6678,   21159,  9, }, // Pacific/Galapagos
    {  644, 6696,  203214, 15, }, // Pacific/Gambier
    {  644, 3049,  203229, 15, }, // Pacific/Kanton
    {  644, 4051,  203244, 17, }, // Pacific/Marquesas
    {  644, 7098,    4510,  6, }, // Pacific/Noumea
    {  644, 3096,     479,  7, }, // Pacific/Pohnpei
    {  645,   14,       0,  6, }, // Africa/Asmara Wolaytta/Ethiopic/Ethiopia
    {  645, 4435,       6,  8, }, // Africa/Sao_Tome
    {  645, 4252,      14,  8, }, // America/Asuncion
    {  645,  237,      22,  8, }, // America/Atikokan
    {  645, 7623,      30, 17, }, // America/Bahia_Banderas
    {  645, 3868,      47,  6, }, // America/Cancun
    {  645, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  645, 5723,      66,  7, }, // America/Curacao
    {  645,  481,      73, 13, }, // America/Indiana/Knox
    {  645, 7961,      86, 16, }, // America/Indiana/Marengo
    {  645, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  645, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  645, 8038,     139, 14, }, // America/Indiana/Vevay
    {  645, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  645, 8086,     171, 16, }, // America/Indiana/Winamac
    {  645, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  645, 5932,     207, 22, }, // America/Lower_Princes
    {  645, 8220,     229,  6, }, // America/Merida
    {  645, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  645, 8329,     255, 20, }, // America/North_Dakota/Center
    {  645, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  645,  393,     298,  4, }, // America/Nuuk
    {  645, 6899,     302, 16, }, // America/Scoresbysund
    {  645, 8508,     318, 14, }, // America/St_Barthelemy
    {  645, 2061,     332, 10, }, // America/St_Johns
    {  645, 5863,     342,  9, }, // America/St_Kitts
    {  645, 5880,     351,  9, }, // America/St_Lucia
    {  645,  855,     360, 10, }, // America/St_Thomas
    {  645, 5913,     370, 11, }, // America/St_Vincent
    {  645, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  645, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  645, 1185,     413,  9, }, // Asia/Kathmandu
    {  645,  992,     422,  7, }, // Asia/Kolkata
    {  645, 1211,     429,  5, }, // Asia/Macau
    {  645, 1235,     442,  6, }, // Asia/Yangon
    {  645, 1395,     448,  5, }, // Atlantic/Faroe
    {  645, 6821,     453, 10, }, // Atlantic/St_Helena
    {  645, 2527,     463,  4, }, // Europe/Kyiv
    {  645, 7211,     467,  7, }, // Indian/Reunion
    {  645, 3157,     474,  5, }, // Pacific/Chuuk
    {  645, 3096,     479,  7, }, // Pacific/Pohnpei
    {  646,   14,       0,  6, }, // Africa/Asmara Wolof/Latin/Senegal
    {  646, 4435,       6,  8, }, // Africa/Sao_Tome
    {  646, 4252,      14,  8, }, // America/Asuncion
    {  646,  237,      22,  8, }, // America/Atikokan
    {  646, 7623,      30, 17, }, // America/Bahia_Banderas
    {  646, 3868,      47,  6, }, // America/Cancun
    {  646, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  646, 5723,      66,  7, }, // America/Curacao
    {  646,  481,      73, 13, }, // America/Indiana/Knox
    {  646, 7961,      86, 16, }, // America/Indiana/Marengo
    {  646, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  646, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  646, 8038,     139, 14, }, // America/Indiana/Vevay
    {  646, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  646, 8086,     171, 16, }, // America/Indiana/Winamac
    {  646, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  646, 5932,     207, 22, }, // America/Lower_Princes
    {  646, 8220,     229,  6, }, // America/Merida
    {  646, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  646, 8329,     255, 20, }, // America/North_Dakota/Center
    {  646, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  646,  393,     298,  4, }, // America/Nuuk
    {  646, 6899,     302, 16, }, // America/Scoresbysund
    {  646, 8508,     318, 14, }, // America/St_Barthelemy
    {  646, 2061,     332, 10, }, // America/St_Johns
    {  646, 5863,     342,  9, }, // America/St_Kitts
    {  646, 5880,     351,  9, }, // America/St_Lucia
    {  646,  855,     360, 10, }, // America/St_Thomas
    {  646, 5913,     370, 11, }, // America/St_Vincent
    {  646, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  646, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  646, 1185,     413,  9, }, // Asia/Kathmandu
    {  646,  992,     422,  7, }, // Asia/Kolkata
    {  646, 1211,     429,  5, }, // Asia/Macau
    {  646, 1235,     442,  6, }, // Asia/Yangon
    {  646, 1395,     448,  5, }, // Atlantic/Faroe
    {  646, 6821,     453, 10, }, // Atlantic/St_Helena
    {  646, 2527,     463,  4, }, // Europe/Kyiv
    {  646, 7211,     467,  7, }, // Indian/Reunion
    {  646, 3157,     474,  5, }, // Pacific/Chuuk
    {  646, 3096,     479,  7, }, // Pacific/Pohnpei
    {  647,   14,       0,  6, }, // Africa/Asmara Xhosa/Latin/South Africa
    {  647, 4435,       6,  8, }, // Africa/Sao_Tome
    {  647, 4252,      14,  8, }, // America/Asuncion
    {  647,  237,      22,  8, }, // America/Atikokan
    {  647, 7623,    3825, 14, }, // America/Bahia_Banderas
    {  647, 3868,   33182,  6, }, // America/Cancun
    {  647, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  647, 5723,      66,  7, }, // America/Curacao
    {  647,  481,      73, 13, }, // America/Indiana/Knox
    {  647, 7961,      86, 16, }, // America/Indiana/Marengo
    {  647, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  647, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  647, 8038,     139, 14, }, // America/Indiana/Vevay
    {  647, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  647, 8086,     171, 16, }, // America/Indiana/Winamac
    {  647, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  647, 5932,     207, 22, }, // America/Lower_Princes
    {  647, 8220,    7008,  6, }, // America/Merida
    {  647, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  647, 8329,     255, 20, }, // America/North_Dakota/Center
    {  647, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  647,  393,     298,  4, }, // America/Nuuk
    {  647, 6899,     302, 16, }, // America/Scoresbysund
    {  647, 8508,     318, 14, }, // America/St_Barthelemy
    {  647, 2061,     332, 10, }, // America/St_Johns
    {  647, 5863,     342,  9, }, // America/St_Kitts
    {  647, 5880,     351,  9, }, // America/St_Lucia
    {  647,  855,     360, 10, }, // America/St_Thomas
    {  647, 5913,     370, 11, }, // America/St_Vincent
    {  647, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  647, 1259,     397, 16, }, // Asia/Ho_Chi_Minh
    {  647, 1185,     413,  9, }, // Asia/Kathmandu
    {  647,  992,     422,  7, }, // Asia/Kolkata
    {  647, 1211,     429,  5, }, // Asia/Macau
    {  647, 8708,     434,  8, }, // Asia/Qostanay
    {  647, 1235,     442,  6, }, // Asia/Yangon
    {  647, 1395,     448,  5, }, // Atlantic/Faroe
    {  647, 6821,     453, 10, }, // Atlantic/St_Helena
    {  647, 2527,     463,  4, }, // Europe/Kyiv
    {  647, 7211,     467,  7, }, // Indian/Reunion
    {  647, 3157,     474,  5, }, // Pacific/Chuuk
    {  647, 3096,     479,  7, }, // Pacific/Pohnpei
    {  648,   14,       0,  6, }, // Africa/Asmara Yangben/Latin/Cameroon
    {  648, 4435,       6,  8, }, // Africa/Sao_Tome
    {  648, 4252,      14,  8, }, // America/Asuncion
    {  648,  237,      22,  8, }, // America/Atikokan
    {  648, 7623,      30, 17, }, // America/Bahia_Banderas
    {  648, 3868,      47,  6, }, // America/Cancun
    {  648, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  648, 5723,      66,  7, }, // America/Curacao
    {  648,  481,      73, 13, }, // America/Indiana/Knox
    {  648, 7961,      86, 16, }, // America/Indiana/Marengo
    {  648, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  648, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  648, 8038,     139, 14, }, // America/Indiana/Vevay
    {  648, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  648, 8086,     171, 16, }, // America/Indiana/Winamac
    {  648, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  648, 5932,     207, 22, }, // America/Lower_Princes
    {  648, 8220,     229,  6, }, // America/Merida
    {  648, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  648, 8329,     255, 20, }, // America/North_Dakota/Center
    {  648, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  648,  393,     298,  4, }, // America/Nuuk
    {  648, 6899,     302, 16, }, // America/Scoresbysund
    {  648, 8508,     318, 14, }, // America/St_Barthelemy
    {  648, 2061,     332, 10, }, // America/St_Johns
    {  648, 5863,     342,  9, }, // America/St_Kitts
    {  648, 5880,     351,  9, }, // America/St_Lucia
    {  648,  855,     360, 10, }, // America/St_Thomas
    {  648, 5913,     370, 11, }, // America/St_Vincent
    {  648, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  648, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  648, 1185,     413,  9, }, // Asia/Kathmandu
    {  648,  992,     422,  7, }, // Asia/Kolkata
    {  648, 1211,     429,  5, }, // Asia/Macau
    {  648, 1235,     442,  6, }, // Asia/Yangon
    {  648, 1395,     448,  5, }, // Atlantic/Faroe
    {  648, 6821,     453, 10, }, // Atlantic/St_Helena
    {  648, 2527,     463,  4, }, // Europe/Kyiv
    {  648, 7211,     467,  7, }, // Indian/Reunion
    {  648, 3157,     474,  5, }, // Pacific/Chuuk
    {  648, 3096,     479,  7, }, // Pacific/Pohnpei
    {  649, 6788,  203261, 10, }, // Africa/Abidjan Yiddish/Hebrew/Ukraine
    {  649,   14,  203271,  9, }, // Africa/Asmara
    {  649, 4435,       6,  8, }, // Africa/Sao_Tome
    {  649, 6554,  203280,  5, }, // Africa/Tunis
    {  649, 4152,  203285,  8, }, // Africa/Windhoek
    {  649, 4252,      14,  8, }, // America/Asuncion
    {  649,  237,      22,  8, }, // America/Atikokan
    {  649, 7623,      30, 17, }, // America/Bahia_Banderas
    {  649, 3868,      47,  6, }, // America/Cancun
    {  649, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  649, 5723,      66,  7, }, // America/Curacao
    {  649,  481,      73, 13, }, // America/Indiana/Knox
    {  649, 7961,      86, 16, }, // America/Indiana/Marengo
    {  649, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  649, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  649, 8038,     139, 14, }, // America/Indiana/Vevay
    {  649, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  649, 8086,     171, 16, }, // America/Indiana/Winamac
    {  649, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  649, 5932,     207, 22, }, // America/Lower_Princes
    {  649, 8220,     229,  6, }, // America/Merida
    {  649, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  649, 8329,     255, 20, }, // America/North_Dakota/Center
    {  649, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  649,  393,     298,  4, }, // America/Nuuk
    {  649, 6899,     302, 16, }, // America/Scoresbysund
    {  649, 8508,     318, 14, }, // America/St_Barthelemy
    {  649, 2061,     332, 10, }, // America/St_Johns
    {  649, 5863,     342,  9, }, // America/St_Kitts
    {  649, 5880,     351,  9, }, // America/St_Lucia
    {  649,  855,     360, 10, }, // America/St_Thomas
    {  649, 5913,     370, 11, }, // America/St_Vincent
    {  649, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  649, 1259,  203293, 16, }, // Asia/Ho_Chi_Minh
    {  649, 1185,     413,  9, }, // Asia/Kathmandu
    {  649,  992,     422,  7, }, // Asia/Kolkata
    {  649, 1211,     429,  5, }, // Asia/Macau
    {  649, 1235,     442,  6, }, // Asia/Yangon
    {  649, 1395,     448,  5, }, // Atlantic/Faroe
    {  649, 6821,     453, 10, }, // Atlantic/St_Helena
    {  649, 2527,     463,  4, }, // Europe/Kyiv
    {  649, 7211,     467,  7, }, // Indian/Reunion
    {  649, 3157,     474,  5, }, // Pacific/Chuuk
    {  649, 3096,     479,  7, }, // Pacific/Pohnpei
    {  650,   14,       0,  6, }, // Africa/Asmara Yoruba/Latin/Nigeria
    {  650, 4435,       6,  8, }, // Africa/Sao_Tome
    {  650,  132,  203309,  9, }, // America/Adak
    {  650, 3346,  203318, 14, }, // America/Anchorage
    {  650, 5607,  203332, 11, }, // America/Anguilla
    {  650, 5624,  203343, 11, }, // America/Antigua
    {  650, 5640,  203354,  9, }, // America/Aruba
    {  650, 4252,      14,  8, }, // America/Asuncion
    {  650,  237,  203363, 12, }, // America/Atikokan
    {  650, 7623,  203375, 19, }, // America/Bahia_Banderas
    {  650, 5654,  203394, 15, }, // America/Barbados
    {  650, 5407,  203409, 12, }, // America/Belize
    {  650, 7660,  203421, 21, }, // America/Blanc-Sablon
    {  650, 7699,  203442, 10, }, // America/Boise
    {  650, 7713,  203452, 19, }, // America/Cambridge_Bay
    {  650, 3868,  203471, 10, }, // America/Cancun
    {  650, 5496,  203481, 10, }, // America/Cayman
    {  650, 2260,  203491, 11, }, // America/Chicago
    {  650, 7756,  203502, 12, }, // America/Chihuahua
    {  650, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  650, 5422,  203514, 14, }, // America/Costa_Rica
    {  650, 7796,  203528, 16, }, // America/Creston
    {  650, 5723,  203544, 11, }, // America/Curacao
    {  650, 7812,  203555, 15, }, // America/Danmarkshavn
    {  650, 7833,  203570, 10, }, // America/Dawson
    {  650, 7848,  203580, 16, }, // America/Dawson_Creek
    {  650,  805,  203596, 10, }, // America/Denver
    {  650, 3465,  203606, 11, }, // America/Detroit
    {  650, 5739,  203617, 12, }, // America/Dominica
    {  650,  893,  203629, 12, }, // America/Edmonton
    {  650, 5441,  203641, 14, }, // America/El_Salvador
    {  650, 7886,  203655, 15, }, // America/Fort_Nelson
    {  650, 7924,  203670, 13, }, // America/Glace_Bay
    {  650, 6881,  203683, 13, }, // America/Goose_Bay
    {  650, 4612,  203696, 14, }, // America/Grand_Turk
    {  650, 5770,  203710, 11, }, // America/Grenada
    {  650, 5786,  203721, 14, }, // America/Guadeloupe
    {  650, 3760,  203735, 13, }, // America/Guatemala
    {  650, 1939,  203748, 12, }, // America/Halifax
    {  650, 2281,  203760, 10, }, // America/Havana
    {  650, 7942,  203770, 13, }, // America/Hermosillo
    {  650,  348,  203783, 16, }, // America/Indiana/Indianapolis
    {  650,  481,  203799,  9, }, // America/Indiana/Knox
    {  650, 7961,  203808, 12, }, // America/Indiana/Marengo
    {  650, 7985,  203820, 14, }, // America/Indiana/Petersburg
    {  650, 8012,  203834, 13, }, // America/Indiana/Tell_City
    {  650, 8038,  203847,  9, }, // America/Indiana/Vevay
    {  650, 8060,  203856, 24, }, // America/Indiana/Vincennes
    {  650, 8086,  203880, 22, }, // America/Indiana/Winamac
    {  650, 8110,  203902, 10, }, // America/Inuvik
    {  650,  660,  203912, 11, }, // America/Iqaluit
    {  650, 2799,  203923, 11, }, // America/Jamaica
    {  650, 5380,  203934,  8, }, // America/Juneau
    {  650,  521,  203942, 12, }, // America/Kentucky/Louisville
    {  650, 8125,  203954, 14, }, // America/Kentucky/Monticello
    {  650, 5704,  203968, 17, }, // America/Kralendijk
    {  650, 3239,  203985, 15, }, // America/Los_Angeles
    {  650, 5932,  204000, 26, }, // America/Lower_Princes
    {  650, 8168,  204026, 11, }, // America/Managua
    {  650, 5897,  204037, 15, }, // America/Marigot
    {  650, 5805,  204052, 12, }, // America/Martinique
    {  650, 8184,  204064, 14, }, // America/Matamoros
    {  650, 2917,  204078, 13, }, // America/Mazatlan
    {  650, 8202,  204091, 12, }, // America/Menominee
    {  650, 8220,  204103, 10, }, // America/Merida
    {  650, 8235,  204113, 16, }, // America/Metlakatla
    {  650, 2949,  204129, 11, }, // America/Mexico_City
    {  650, 4391,  204140, 13, }, // America/Miquelon
    {  650, 8254,  204153, 11, }, // America/Moncton
    {  650, 8270,  204164, 11, }, // America/Monterrey
    {  650, 5824,  204175, 13, }, // America/Montserrat
    {  650, 5481,  204188,  9, }, // America/Nassau
    {  650, 2379,  204197, 12, }, // America/New_York
    {  650, 8288,  204209,  8, }, // America/Nome
    {  650, 8301,  204217, 25, }, // America/North_Dakota/Beulah
    {  650, 8329,  204242, 22, }, // America/North_Dakota/Center
    {  650, 8357,  204264, 29, }, // America/North_Dakota/New_Salem
    {  650,  393,  204293,  8, }, // America/Nuuk
    {  650, 8388,  204301, 11, }, // America/Ojinaga
    {  650, 2356,  204312, 10, }, // America/Panama
    {  650, 2973,  204322, 10, }, // America/Phoenix
    {  650, 3945,  204332, 18, }, // America/Port-au-Prince
    {  650, 5954,  204350, 19, }, // America/Port_of_Spain
    {  650, 5843,  204369, 15, }, // America/Puerto_Rico
    {  650, 8424,  204384, 16, }, // America/Rankin_Inlet
    {  650, 1995,  204400, 10, }, // America/Regina
    {  650, 8460,  204410, 12, }, // America/Resolute
    {  650, 6111,  204422, 16, }, // America/Santo_Domingo
    {  650, 6899,  204438, 15, }, // America/Scoresbysund
    {  650, 8494,  204453,  8, }, // America/Sitka
    {  650, 8508,  204461, 12, }, // America/St_Barthelemy
    {  650, 2061,  204473, 14, }, // America/St_Johns
    {  650, 5863,  204487, 13, }, // America/St_Kitts
    {  650, 5880,  204500, 12, }, // America/St_Lucia
    {  650,  855,  204512, 14, }, // America/St_Thomas
    {  650, 5913,  204526, 12, }, // America/St_Vincent
    {  650, 8530,  204538, 19, }, // America/Swift_Current
    {  650, 5461,  204557, 14, }, // America/Tegucigalpa
    {  650, 5756,  204571,  8, }, // America/Thule
    {  650,  313,  204579, 11, }, // America/Tijuana
    {  650,  608,  204590, 11, }, // America/Toronto
    {  650, 5688,  204601, 10, }, // America/Tortola
    {  650, 2093,  204611, 11, }, // America/Vancouver
    {  650, 2144,  204622, 13, }, // America/Whitehorse
    {  650,  734,  204635, 12, }, // America/Winnipeg
    {  650, 8552,  204647, 10, }, // America/Yakutat
    {  650, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  650, 1259,  204657, 15, }, // Asia/Ho_Chi_Minh
    {  650, 1185,     413,  9, }, // Asia/Kathmandu
    {  650,  992,     422,  7, }, // Asia/Kolkata
    {  650, 1211,     429,  5, }, // Asia/Macau
    {  650, 1235,     442,  6, }, // Asia/Yangon
    {  650, 5671,  204672, 11, }, // Atlantic/Bermuda
    {  650, 1395,     448,  5, }, // Atlantic/Faroe
    {  650, 6821,     453, 10, }, // Atlantic/St_Helena
    {  650, 2527,     463,  4, }, // Europe/Kyiv
    {  650, 7211,     467,  7, }, // Indian/Reunion
    {  650, 3157,     474,  5, }, // Pacific/Chuuk
    {  650, 3096,     479,  7, }, // Pacific/Pohnpei
    {  651, 3346,  204683, 14, }, // America/Anchorage Yoruba/Latin/Benin
    {  651,  237,  204697, 12, }, // America/Atikokan
    {  651, 7623,  204709, 19, }, // America/Bahia_Banderas
    {  651, 5654,  204728, 15, }, // America/Barbados
    {  651, 5407,  204743, 12, }, // America/Belize
    {  651, 7660,  204755, 21, }, // America/Blanc-Sablon
    {  651, 7699,  204776, 10, }, // America/Boise
    {  651, 7756,  204786, 13, }, // America/Chihuahua
    {  651, 7796,  204799, 16, }, // America/Creston
    {  651, 5723,  204815, 12, }, // America/Curacao
    {  651,  481,  204827,  9, }, // America/Indiana/Knox
    {  651, 7961,  204836, 12, }, // America/Indiana/Marengo
    {  651, 8125,  204848, 14, }, // America/Kentucky/Monticello
    {  651, 5704,  204862, 17, }, // America/Kralendijk
    {  651, 5897,  204879, 15, }, // America/Marigot
    {  651, 2949,  204894, 11, }, // America/Mexico_City
    {  651, 4391,  204905, 13, }, // America/Miquelon
    {  651, 2061,  204918, 14, }, // America/St_Johns
    {  651,  855,  204932, 14, }, // America/St_Thomas
    {  651, 8530,  204946, 19, }, // America/Swift_Current
    {  651, 5671,  204965, 11, }, // Atlantic/Bermuda
    {  652,   14,       0,  6, }, // Africa/Asmara Zarma/Latin/Niger
    {  652, 4435,       6,  8, }, // Africa/Sao_Tome
    {  652, 4252,      14,  8, }, // America/Asuncion
    {  652,  237,      22,  8, }, // America/Atikokan
    {  652, 7623,      30, 17, }, // America/Bahia_Banderas
    {  652, 3868,      47,  6, }, // America/Cancun
    {  652, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  652, 5723,      66,  7, }, // America/Curacao
    {  652,  481,      73, 13, }, // America/Indiana/Knox
    {  652, 7961,      86, 16, }, // America/Indiana/Marengo
    {  652, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  652, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  652, 8038,     139, 14, }, // America/Indiana/Vevay
    {  652, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  652, 8086,     171, 16, }, // America/Indiana/Winamac
    {  652, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  652, 5932,     207, 22, }, // America/Lower_Princes
    {  652, 8220,     229,  6, }, // America/Merida
    {  652, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  652, 8329,     255, 20, }, // America/North_Dakota/Center
    {  652, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  652,  393,     298,  4, }, // America/Nuuk
    {  652, 6899,     302, 16, }, // America/Scoresbysund
    {  652, 8508,     318, 14, }, // America/St_Barthelemy
    {  652, 2061,     332, 10, }, // America/St_Johns
    {  652, 5863,     342,  9, }, // America/St_Kitts
    {  652, 5880,     351,  9, }, // America/St_Lucia
    {  652,  855,     360, 10, }, // America/St_Thomas
    {  652, 5913,     370, 11, }, // America/St_Vincent
    {  652, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  652, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  652, 1185,     413,  9, }, // Asia/Kathmandu
    {  652,  992,     422,  7, }, // Asia/Kolkata
    {  652, 1211,     429,  5, }, // Asia/Macau
    {  652, 1235,     442,  6, }, // Asia/Yangon
    {  652, 1395,     448,  5, }, // Atlantic/Faroe
    {  652, 6821,     453, 10, }, // Atlantic/St_Helena
    {  652, 2527,     463,  4, }, // Europe/Kyiv
    {  652, 7211,     467,  7, }, // Indian/Reunion
    {  652, 3157,     474,  5, }, // Pacific/Chuuk
    {  652, 3096,     479,  7, }, // Pacific/Pohnpei
    {  653,   14,       0,  6, }, // Africa/Asmara Zhuang/Latin/China
    {  653, 4435,       6,  8, }, // Africa/Sao_Tome
    {  653, 4252,      14,  8, }, // America/Asuncion
    {  653,  237,      22,  8, }, // America/Atikokan
    {  653, 7623,      30, 17, }, // America/Bahia_Banderas
    {  653, 3868,      47,  6, }, // America/Cancun
    {  653, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  653, 5723,      66,  7, }, // America/Curacao
    {  653,  481,      73, 13, }, // America/Indiana/Knox
    {  653, 7961,      86, 16, }, // America/Indiana/Marengo
    {  653, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  653, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  653, 8038,     139, 14, }, // America/Indiana/Vevay
    {  653, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  653, 8086,     171, 16, }, // America/Indiana/Winamac
    {  653, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  653, 5932,     207, 22, }, // America/Lower_Princes
    {  653, 8220,     229,  6, }, // America/Merida
    {  653, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  653, 8329,     255, 20, }, // America/North_Dakota/Center
    {  653, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  653,  393,     298,  4, }, // America/Nuuk
    {  653, 6899,     302, 16, }, // America/Scoresbysund
    {  653, 8508,     318, 14, }, // America/St_Barthelemy
    {  653, 2061,     332, 10, }, // America/St_Johns
    {  653, 5863,     342,  9, }, // America/St_Kitts
    {  653, 5880,     351,  9, }, // America/St_Lucia
    {  653,  855,     360, 10, }, // America/St_Thomas
    {  653, 5913,     370, 11, }, // America/St_Vincent
    {  653, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  653, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  653, 1185,     413,  9, }, // Asia/Kathmandu
    {  653,  992,     422,  7, }, // Asia/Kolkata
    {  653, 1211,     429,  5, }, // Asia/Macau
    {  653, 1235,     442,  6, }, // Asia/Yangon
    {  653, 1395,     448,  5, }, // Atlantic/Faroe
    {  653, 6821,     453, 10, }, // Atlantic/St_Helena
    {  653, 2527,     463,  4, }, // Europe/Kyiv
    {  653, 7211,     467,  7, }, // Indian/Reunion
    {  653, 3157,     474,  5, }, // Pacific/Chuuk
    {  653, 3096,     479,  7, }, // Pacific/Pohnpei
    {  654, 6788,  204976,  9, }, // Africa/Abidjan Zulu/Latin/South Africa
    {  654, 6760,  204985,  7, }, // Africa/Accra
    {  654, 5059,  204992, 13, }, // Africa/Addis_Ababa
    {  654, 7420,  205005,  9, }, // Africa/Algiers
    {  654,   14,  205014,  8, }, // Africa/Asmara
    {  654,   44,  205022,  8, }, // Africa/Bamako
    {  654, 5257,  205030,  8, }, // Africa/Bangui
    {  654, 6746,  205038,  8, }, // Africa/Banjul
    {  654, 7435,  205046,  8, }, // Africa/Bissau
    {  654, 4957,  205054, 10, }, // Africa/Blantyre
    {  654, 5287,  205064, 13, }, // Africa/Brazzaville
    {  654, 4922,  205077, 11, }, // Africa/Bujumbura
    {  654, 2320,  205088,  7, }, // Africa/Cairo
    {  654, 4117,  205095, 12, }, // Africa/Casablanca
    {  654, 7449,  205107,  7, }, // Africa/Ceuta
    {  654, 6773,  205114,  9, }, // Africa/Conakry
    {  654, 6840,  205123,  7, }, // Africa/Dakar
    {  654, 5130,  205130, 15, }, // Africa/Dar_es_Salaam
    {  654, 5043,  205145, 10, }, // Africa/Djibouti
    {  654, 5243,  205155,  8, }, // Africa/Douala
    {  654, 5166,  205163, 10, }, // Africa/El_Aaiun
    {  654, 6853,  205173, 10, }, // Africa/Freetown
    {  654, 4906,  205183, 10, }, // Africa/Gaborone
    {  654, 5015,  205193,  8, }, // Africa/Harare
    {  654, 4479,  205201, 14, }, // Africa/Johannesburg
    {  654, 4499,  205215,  5, }, // Africa/Juba
    {  654, 5151,  205220,  9, }, // Africa/Kampala
    {  654, 4524,  205229, 10, }, // Africa/Khartoum
    {  654, 4973,  205239,  8, }, // Africa/Kigali
    {  654, 5306,  205247, 10, }, // Africa/Kinshasa
    {  654, 4744,  205257,  7, }, // Africa/Lagos
    {  654, 5336,  205264, 12, }, // Africa/Libreville
    {  654, 6869,  205276,  6, }, // Africa/Lome
    {  654, 5211,  205282,  8, }, // Africa/Luanda
    {  654, 4939,  205290, 12, }, // Africa/Lubumbashi
    {  654, 5001,  205302,  7, }, // Africa/Lusaka
    {  654, 5322,  205309,  7, }, // Africa/Malabo
    {  654, 4987,  205316,  7, }, // Africa/Maputo
    {  654, 5197,  205323,  7, }, // Africa/Maseru
    {  654, 5182,  205330,  8, }, // Africa/Mbabane
    {  654, 5113,  205338, 11, }, // Africa/Mogadishu
    {  654, 7034,  205349, 10, }, // Africa/Monrovia
    {  654, 3853,  205359,  9, }, // Africa/Nairobi
    {  654, 5271,  205368, 10, }, // Africa/Ndjamena
    {  654, 5354,  205378,  8, }, // Africa/Niamey
    {  654, 6803,  205386, 12, }, // Africa/Nouakchott
    {  654, 6727,  205398, 13, }, // Africa/Ouagadougou
    {  654, 5225,  205411, 12, }, // Africa/Porto-Novo
    {  654, 4435,  205423, 10, }, // Africa/Sao_Tome
    {  654, 2866,  205433,  9, }, // Africa/Tripoli
    {  654, 6554,  205442,  7, }, // Africa/Tunis
    {  654, 4152,  205449, 10, }, // Africa/Windhoek
    {  654,  132,  205459,  6, }, // America/Adak
    {  654, 3346,  205465, 11, }, // America/Anchorage
    {  654, 5607,  205476, 10, }, // America/Anguilla
    {  654, 5624,  205486,  9, }, // America/Antigua
    {  654, 4554,  205495, 11, }, // America/Araguaina
    {  654,  166,  205506, 14, }, // America/Argentina/Buenos_Aires
    {  654,   91,  205520, 11, }, // America/Argentina/Catamarca
    {  654,  270,  205531,  9, }, // America/Argentina/Cordoba
    {  654,  441,  205540,  7, }, // America/Argentina/Jujuy
    {  654, 7462,  205547, 10, }, // America/Argentina/La_Rioja
    {  654,  565,  205557,  9, }, // America/Argentina/Mendoza
    {  654, 7489,  205566, 14, }, // America/Argentina/Rio_Gallegos
    {  654, 7520,  205580,  7, }, // America/Argentina/Salta
    {  654, 7544,  205587, 10, }, // America/Argentina/San_Juan
    {  654, 5580,  205597, 10, }, // America/Argentina/San_Luis
    {  654, 7571,  205607,  9, }, // America/Argentina/Tucuman
    {  654, 7597,  205616,  9, }, // America/Argentina/Ushuaia
    {  654, 5640,  205625,  7, }, // America/Aruba
    {  654, 4252,  205632, 10, }, // America/Asuncion
    {  654,  237,  205642, 10, }, // America/Atikokan
    {  654, 3679,  205652,  7, }, // America/Bahia
    {  654, 7623,  205659, 16, }, // America/Bahia_Banderas
    {  654, 5654,  205675, 10, }, // America/Barbados
    {  654, 7646,  205685,  7, }, // America/Belem
    {  654, 5407,  205692,  8, }, // America/Belize
    {  654, 7660,  205700, 14, }, // America/Blanc-Sablon
    {  654, 7681,  205714, 11, }, // America/Boa_Vista
    {  654, 4361,  205725,  8, }, // America/Bogota
    {  654, 7699,  205733,  7, }, // America/Boise
    {  654, 7713,  205740, 15, }, // America/Cambridge_Bay
    {  654, 7735,  205755, 14, }, // America/Campo_Grande
    {  654, 3868,  205769,  8, }, // America/Cancun
    {  654, 4694,  205777,  9, }, // America/Caracas
    {  654, 4345,  205786,  9, }, // America/Cayenne
    {  654, 5496,  205795,  8, }, // America/Cayman
    {  654, 2260,  205803,  9, }, // America/Chicago
    {  654, 7756,  205812, 11, }, // America/Chihuahua
    {  654, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  654, 5422,  205823, 12, }, // America/Costa_Rica
    {  654, 7796,  205835,  9, }, // America/Creston
    {  654, 3791,  205844,  8, }, // America/Cuiaba
    {  654, 5723,  205852,  9, }, // America/Curacao
    {  654, 7812,  205861, 14, }, // America/Danmarkshavn
    {  654, 7833,  205875,  8, }, // America/Dawson
    {  654, 7848,  205883, 14, }, // America/Dawson_Creek
    {  654,  805,  205897,  8, }, // America/Denver
    {  654, 3465,  205905,  9, }, // America/Detroit
    {  654, 5739,  205914, 10, }, // America/Dominica
    {  654,  893,  205924, 10, }, // America/Edmonton
    {  654, 7869,  205934, 10, }, // America/Eirunepe
    {  654, 5441,  205944, 13, }, // America/El_Salvador
    {  654, 7886,  205957, 13, }, // America/Fort_Nelson
    {  654, 7906,  205970, 11, }, // America/Fortaleza
    {  654, 7924,  205981, 11, }, // America/Glace_Bay
    {  654, 6881,  205992, 11, }, // America/Goose_Bay
    {  654, 4612,  206003, 12, }, // America/Grand_Turk
    {  654, 5770,  206015,  9, }, // America/Grenada
    {  654, 5786,  206024, 12, }, // America/Guadeloupe
    {  654, 3760,  206036, 11, }, // America/Guatemala
    {  654, 4841,  206047, 11, }, // America/Guayaquil
    {  654, 6932,  206058,  8, }, // America/Guyana
    {  654, 1939,  206066,  9, }, // America/Halifax
    {  654, 2281,  206075,  8, }, // America/Havana
    {  654, 7942,  206083, 12, }, // America/Hermosillo
    {  654,  348,  206095, 14, }, // America/Indiana/Indianapolis
    {  654,  481,  206109, 15, }, // America/Indiana/Knox
    {  654, 7961,  206124, 18, }, // America/Indiana/Marengo
    {  654, 7985,  206142, 21, }, // America/Indiana/Petersburg
    {  654, 8012,  206163, 20, }, // America/Indiana/Tell_City
    {  654, 8038,  206183, 16, }, // America/Indiana/Vevay
    {  654, 8060,  206199, 20, }, // America/Indiana/Vincennes
    {  654, 8086,  206219, 18, }, // America/Indiana/Winamac
    {  654, 8110,  206237,  8, }, // America/Inuvik
    {  654,  660,  206245,  9, }, // America/Iqaluit
    {  654, 2799,  206254,  9, }, // America/Jamaica
    {  654, 5380,  206263,  8, }, // America/Juneau
    {  654,  521,  206271, 12, }, // America/Kentucky/Louisville
    {  654, 8125,  206283, 22, }, // America/Kentucky/Monticello
    {  654, 5704,  206305, 12, }, // America/Kralendijk
    {  654, 4376,  206317,  8, }, // America/La_Paz
    {  654, 7169,  206325,  6, }, // America/Lima
    {  654, 3239,  206331, 13, }, // America/Los_Angeles
    {  654, 5932,  206344, 24, }, // America/Lower_Princes
    {  654, 8153,  206368,  8, }, // America/Maceio
    {  654, 8168,  206376,  9, }, // America/Managua
    {  654, 1908,  206385,  8, }, // America/Manaus
    {  654, 5897,  206393,  9, }, // America/Marigot
    {  654, 5805,  206402, 12, }, // America/Martinique
    {  654, 8184,  206414, 11, }, // America/Matamoros
    {  654, 2917,  206425, 10, }, // America/Mazatlan
    {  654, 8202,  206435, 11, }, // America/Menominee
    {  654, 8220,  206446,  8, }, // America/Merida
    {  654, 8235,  206454, 12, }, // America/Metlakatla
    {  654, 2949,  206466, 13, }, // America/Mexico_City
    {  654, 4391,  206479, 10, }, // America/Miquelon
    {  654, 8254,  206489,  9, }, // America/Moncton
    {  654, 8270,  206498, 11, }, // America/Monterrey
    {  654, 4098,  206509, 12, }, // America/Montevideo
    {  654, 5824,  206521, 12, }, // America/Montserrat
    {  654, 5481,  206533,  8, }, // America/Nassau
    {  654, 2379,  206541, 10, }, // America/New_York
    {  654, 8288,  206551,  6, }, // America/Nome
    {  654, 1850,  206557,  9, }, // America/Noronha
    {  654, 8301,  206566, 22, }, // America/North_Dakota/Beulah
    {  654, 8329,  206588, 22, }, // America/North_Dakota/Center
    {  654, 8357,  206610, 25, }, // America/North_Dakota/New_Salem
    {  654,  393,  206635,  6, }, // America/Nuuk
    {  654, 8388,  206641,  9, }, // America/Ojinaga
    {  654, 2356,  206650,  8, }, // America/Panama
    {  654, 6173,  206658, 12, }, // America/Paramaribo
    {  654, 2973,  206670,  9, }, // America/Phoenix
    {  654, 3945,  206679, 16, }, // America/Port-au-Prince
    {  654, 5954,  206695, 15, }, // America/Port_of_Spain
    {  654, 8404,  206710, 13, }, // America/Porto_Velho
    {  654, 5843,  206723, 13, }, // America/Puerto_Rico
    {  654, 4030,  206736, 14, }, // America/Punta_Arenas
    {  654, 8424,  206750, 14, }, // America/Rankin_Inlet
    {  654, 8445,  206764,  8, }, // America/Recife
    {  654, 1995,  206772,  8, }, // America/Regina
    {  654, 8460,  206780, 10, }, // America/Resolute
    {  654,  695,  206790, 12, }, // America/Rio_Branco
    {  654, 8477,  206802, 10, }, // America/Santarem
    {  654, 2201,  206812, 10, }, // America/Santiago
    {  654, 6111,  206822, 15, }, // America/Santo_Domingo
    {  654, 1878,  206837, 11, }, // America/Sao_Paulo
    {  654, 6899,  206848, 18, }, // America/Scoresbysund
    {  654, 8494,  206866,  7, }, // America/Sitka
    {  654, 8508,  206873, 16, }, // America/St_Barthelemy
    {  654, 2061,  206889, 12, }, // America/St_Johns
    {  654, 5863,  206901, 11, }, // America/St_Kitts
    {  654, 5880,  206912, 11, }, // America/St_Lucia
    {  654,  855,  206923, 12, }, // America/St_Thomas
    {  654, 5913,  206935, 13, }, // America/St_Vincent
    {  654, 8530,  206948, 15, }, // America/Swift_Current
    {  654, 5461,  206963, 13, }, // America/Tegucigalpa
    {  654, 5756,  206976,  7, }, // America/Thule
    {  654,  313,  206983,  9, }, // America/Tijuana
    {  654,  608,  206992,  9, }, // America/Toronto
    {  654, 5688,  207001,  9, }, // America/Tortola
    {  654, 2093,  207010, 11, }, // America/Vancouver
    {  654, 2144,  207021, 12, }, // America/Whitehorse
    {  654,  734,  207033, 10, }, // America/Winnipeg
    {  654, 8552,  207043,  9, }, // America/Yakutat
    {  654, 6001,  207052,  7, }, // Antarctica/Casey
    {  654, 6094,  207059,  7, }, // Antarctica/Davis
    {  654, 6133,  207066, 18, }, // Antarctica/DumontDUrville
    {  654, 8568,  207084, 11, }, // Antarctica/Macquarie
    {  654, 7066,  207095,  8, }, // Antarctica/Mawson
    {  654, 7113,  207103,  9, }, // Antarctica/McMurdo
    {  654, 8589,  207112,  8, }, // Antarctica/Palmer
    {  654, 7226,  207120,  9, }, // Antarctica/Rothera
    {  654, 7295,  207129,  7, }, // Antarctica/Syowa
    {  654, 8607,  207136,  7, }, // Antarctica/Troll
    {  654, 7374,  207143,  8, }, // Antarctica/Vostok
    {  654, 1429,  207151, 14, }, // Arctic/Longyearbyen
    {  654, 5570,  207165,  6, }, // Asia/Aden
    {  654, 5395,  207171,  8, }, // Asia/Almaty
    {  654, 3968,  207179,  7, }, // Asia/Amman
    {  654, 5511,  207186,  8, }, // Asia/Anadyr
    {  654, 5523,  207194,  7, }, // Asia/Aqtau
    {  654, 5368,  207201,  8, }, // Asia/Aqtobe
    {  654,  964,  207209, 10, }, // Asia/Ashgabat
    {  654, 3607,  207219,  9, }, // Asia/Baghdad
    {  654, 5534,  207228,  9, }, // Asia/Bahrain
    {  654, 3653,  207237,  6, }, // Asia/Baku
    {  654, 4466,  207243,  9, }, // Asia/Bangkok
    {  654, 3571,  207252,  9, }, // Asia/Barnaul
    {  654, 4086,  207261,  8, }, // Asia/Beirut
    {  654, 3778,  207269,  9, }, // Asia/Bishkek
    {  654, 5989,  207278,  8, }, // Asia/Brunei
    {  654, 4601,  207286,  7, }, // Asia/Chita
    {  654, 4511,  207293,  9, }, // Asia/Colombo
    {  654, 4540,  207302, 10, }, // Asia/Damascus
    {  654, 1093,  207312,  7, }, // Asia/Dhaka
    {  654, 6192,  207319,  6, }, // Asia/Dili
    {  654, 3596,  207325,  7, }, // Asia/Dubai
    {  654, 6159,  207332, 10, }, // Asia/Dushanbe
    {  654, 8651,  207342,  5, }, // Asia/Gaza
    {  654, 4795,  207347,  8, }, // Asia/Hebron
    {  654, 1259,  207355, 18, }, // Asia/Ho_Chi_Minh
    {  654, 2704,  207373, 11, }, // Asia/Hong_Kong
    {  654, 4771,  207384,  6, }, // Asia/Hovd
    {  654, 4184,  207390,  9, }, // Asia/Irkutsk
    {  654, 7006,  207399,  9, }, // Asia/Jakarta
    {  654, 6992,  207408, 10, }, // Asia/Jayapura
    {  654, 1290,  207418, 11, }, // Asia/Jerusalem
    {  654, 3560,  207429,  7, }, // Asia/Kabul
    {  654, 4316,  207436, 11, }, // Asia/Kamchatka
    {  654, 4239,  207447,  9, }, // Asia/Karachi
    {  654, 1185,  207456, 11, }, // Asia/Kathmandu
    {  654, 8661,  207467, 10, }, // Asia/Khandyga
    {  654,  992,  207477,  9, }, // Asia/Kolkata
    {  654, 4197,  207486, 13, }, // Asia/Krasnoyarsk
    {  654, 4859,  207499, 14, }, // Asia/Kuala_Lumpur
    {  654, 5976,  207513,  9, }, // Asia/Kuching
    {  654, 5547,  207522,  8, }, // Asia/Kuwait
    {  654, 1211,  207530,  7, }, // Asia/Macau
    {  654, 4017,  207537,  9, }, // Asia/Magadan
    {  654, 1349,  207546, 10, }, // Asia/Makassar
    {  654, 7182,  207556,  8, }, // Asia/Manila
    {  654, 6920,  207564,  8, }, // Asia/Muscat
    {  654, 2554,  207572,  9, }, // Asia/Nicosia
    {  654, 8675,  207581, 14, }, // Asia/Novokuznetsk
    {  654, 4135,  207595, 13, }, // Asia/Novosibirsk
    {  654, 4229,  207608,  6, }, // Asia/Omsk
    {  654, 7145,  207614,  6, }, // Asia/Oral
    {  654, 6961,  207620, 12, }, // Asia/Phnom_Penh
    {  654, 8693,  207632, 11, }, // Asia/Pontianak
    {  654, 4214,  207643, 11, }, // Asia/Pyongyang
    {  654, 5559,  207654,  7, }, // Asia/Qatar
    {  654, 8708,  207661, 10, }, // Asia/Qostanay
    {  654, 4269,  207671, 11, }, // Asia/Qyzylorda
    {  654, 3584,  207682,  8, }, // Asia/Riyadh
    {  654, 4408,  207690, 10, }, // Asia/Sakhalin
    {  654, 7245,  207700, 11, }, // Asia/Samarkand
    {  654, 3279,  207711,  7, }, // Asia/Seoul
    {  654, 1053,  207718, 10, }, // Asia/Shanghai
    {  654, 3300,  207728, 11, }, // Asia/Singapore
    {  654, 4297,  207739, 15, }, // Asia/Srednekolymsk
    {  654, 3263,  207754,  8, }, // Asia/Taipei
    {  654, 4781,  207762, 10, }, // Asia/Tashkent
    {  654, 3915,  207772,  9, }, // Asia/Tbilisi
    {  654, 2772,  207781,  8, }, // Asia/Tehran
    {  654, 1317,  207789,  9, }, // Asia/Thimphu
    {  654, 2821,  207798,  7, }, // Asia/Tokyo
    {  654, 4572,  207805,  7, }, // Asia/Tomsk
    {  654, 1021,  207812, 13, }, // Asia/Ulaanbaatar
    {  654, 1159,  207825,  8, }, // Asia/Urumqi
    {  654, 8722,  207833, 10, }, // Asia/Ust-Nera
    {  654, 6977,  207843, 11, }, // Asia/Vientiane
    {  654, 4710,  207854, 13, }, // Asia/Vladivostok
    {  654, 4828,  207867,  9, }, // Asia/Yakutsk
    {  654, 1235,  207876,  9, }, // Asia/Yangon
    {  654, 3883,  207885, 15, }, // Asia/Yekaterinburg
    {  654, 3747,  207900,  9, }, // Asia/Yerevan
    {  654, 3663,  207909,  8, }, // Atlantic/Azores
    {  654, 5671,  207917,  9, }, // Atlantic/Bermuda
    {  654, 6628,  207926,  8, }, // Atlantic/Canary
    {  654, 3727,  207934, 12, }, // Atlantic/Cape_Verde
    {  654, 1395,  207946,  7, }, // Atlantic/Faroe
    {  654, 8736,  207953,  9, }, // Atlantic/Madeira
    {  654, 2748,  207962, 11, }, // Atlantic/Reykjavik
    {  654, 7272,  207973, 15, }, // Atlantic/South_Georgia
    {  654, 6821,  207988, 12, }, // Atlantic/St_Helena
    {  654, 6644,  208000,  9, }, // Atlantic/Stanley
    {  654, 1670,  208009, 10, }, // Australia/Adelaide
    {  654, 1635,  208019, 10, }, // Australia/Brisbane
    {  654, 1799,  208029, 13, }, // Australia/Broken_Hill
    {  654, 1583,  208042,  8, }, // Australia/Darwin
    {  654, 3637,  208050,  7, }, // Australia/Eucla
    {  654, 1516,  208057,  8, }, // Australia/Hobart
    {  654, 8753,  208065, 10, }, // Australia/Lindeman
    {  654, 1547,  208075, 11, }, // Australia/Lord_Howe
    {  654, 1727,  208086, 11, }, // Australia/Melbourne
    {  654, 1762,  208097,  7, }, // Australia/Perth
    {  654, 1463,  208104,  8, }, // Australia/Sydney
    {  654, 6441,  208112, 11, }, // Europe/Amsterdam
    {  654, 6216,  208123,  9, }, // Europe/Andorra
    {  654, 3620,  208132, 11, }, // Europe/Astrakhan
    {  654, 2300,  208143,  8, }, // Europe/Athens
    {  654, 6336,  208151, 10, }, // Europe/Belgrade
    {  654, 4757,  208161,  8, }, // Europe/Berlin
    {  654, 6488,  208169, 12, }, // Europe/Bratislava
    {  654, 2167,  208181, 10, }, // Europe/Brussels
    {  654, 3928,  208191, 11, }, // Europe/Bucharest
    {  654, 3806,  208202, 10, }, // Europe/Budapest
    {  654, 8772,  208212, 10, }, // Europe/Busingen
    {  654, 2583,  208222, 10, }, // Europe/Chisinau
    {  654, 6289,  208232, 12, }, // Europe/Copenhagen
    {  654, 2338,  208244,  8, }, // Europe/Dublin
    {  654, 6307,  208252, 11, }, // Europe/Gibraltar
    {  654, 8788,  208263, 10, }, // Europe/Guernsey
    {  654, 6612,  208273, 10, }, // Europe/Helsinki
    {  654, 8804,  208283, 13, }, // Europe/Isle_of_Man
    {  654, 1130,  208296, 10, }, // Europe/Istanbul
    {  654, 8823,  208306,  8, }, // Europe/Jersey
    {  654, 3979,  208314, 13, }, // Europe/Kaliningrad
    {  654, 8929,  208327,  7, }, // Europe/Kirov
    {  654, 2527,  208334,  6, }, // Europe/Kyiv
    {  654, 3213,  208340,  8, }, // Europe/Lisbon
    {  654, 6506,  208348, 11, }, // Europe/Ljubljana
    {  654, 2501,  208359,  8, }, // Europe/London
    {  654, 6365,  208367, 12, }, // Europe/Luxembourg
    {  654, 4892,  208379,  8, }, // Europe/Madrid
    {  654, 6397,  208387,  7, }, // Europe/Malta
    {  654, 6582,  208394, 11, }, // Europe/Mariehamn
    {  654, 3693,  208405,  7, }, // Europe/Minsk
    {  654, 6410,  208412,  8, }, // Europe/Monaco
    {  654, 3537,  208420,  8, }, // Europe/Moscow
    {  654, 6458,  208428,  6, }, // Europe/Oslo
    {  654, 4284,  208434,  7, }, // Europe/Paris
    {  654, 6424,  208441, 11, }, // Europe/Podgorica
    {  654, 6275,  208452,  8, }, // Europe/Prague
    {  654, 8837,  208460,  6, }, // Europe/Riga
    {  654, 6324,  208466,  6, }, // Europe/Rome
    {  654, 4331,  208472,  8, }, // Europe/Samara
    {  654, 6470,  208480, 12, }, // Europe/San_Marino
    {  654, 6245,  208492, 10, }, // Europe/Sarajevo
    {  654, 4451,  208502,  9, }, // Europe/Saratov
    {  654, 8849,  208511, 12, }, // Europe/Simferopol
    {  654, 6383,  208523,  8, }, // Europe/Skopje
    {  654, 6599,  208531,  7, }, // Europe/Sofia
    {  654, 6523,  208538, 11, }, // Europe/Stockholm
    {  654, 8867,  208549,  9, }, // Europe/Tallinn
    {  654, 6202,  208558,  8, }, // Europe/Tirane
    {  654, 8882,  208566, 11, }, // Europe/Ulyanovsk
    {  654, 6352,  208577,  7, }, // Europe/Vaduz
    {  654, 6567,  208584,  9, }, // Europe/Vatican
    {  654, 6231,  208593,  8, }, // Europe/Vienna
    {  654, 8899,  208601,  9, }, // Europe/Vilnius
    {  654, 4727,  208610, 11, }, // Europe/Volgograd
    {  654, 3190,  208621,  8, }, // Europe/Warsaw
    {  654, 6261,  208629,  8, }, // Europe/Zagreb
    {  654, 6540,  208637,  8, }, // Europe/Zurich
    {  654, 5078,  208645, 14, }, // Indian/Antananarivo
    {  654, 6947,  208659,  8, }, // Indian/Chagos
    {  654, 6046,  208667, 10, }, // Indian/Christmas
    {  654, 6063,  208677,  7, }, // Indian/Cocos
    {  654, 5029,  208684,  8, }, // Indian/Comoro
    {  654, 6661,  208692, 11, }, // Indian/Kerguelen
    {  654, 7260,  208703,  5, }, // Indian/Mahe
    {  654, 7050,  208708, 10, }, // Indian/Maldives
    {  654, 4069,  208718, 11, }, // Indian/Mauritius
    {  654, 5098,  208729,  9, }, // Indian/Mayotte
    {  654, 7211,  208738,  9, }, // Indian/Reunion
    {  654, 4422,  208747,  6, }, // Pacific/Apia
    {  654,  932,  208753, 10, }, // Pacific/Auckland
    {  654, 3706,  208763, 14, }, // Pacific/Bougainville
    {  654, 3015,  208777,  9, }, // Pacific/Chatham
    {  654, 3157,  208786,  7, }, // Pacific/Chuuk
    {  654, 2237,  208793,  8, }, // Pacific/Easter
    {  654, 7360,  208801,  7, }, // Pacific/Efate
    {  654, 7327,  208808,  9, }, // Pacific/Fakaofo
    {  654, 3902,  208817,  6, }, // Pacific/Fiji
    {  654, 7343,  208823, 10, }, // Pacific/Funafuti
    {  654, 6678,  208833, 11, }, // Pacific/Galapagos
    {  654, 6696,  208844,  9, }, // Pacific/Gambier
    {  654, 3822,  208853, 13, }, // Pacific/Guadalcanal
    {  654, 6018,  208866,  6, }, // Pacific/Guam
    {  654, 2723,  208872, 10, }, // Pacific/Honolulu
    {  654, 3049,  208882, 11, }, // Pacific/Kanton
    {  654, 3998,  208893, 12, }, // Pacific/Kiritimati
    {  654, 7019,  208905,  8, }, // Pacific/Kosrae
    {  654, 2842,  208913, 11, }, // Pacific/Kwajalein
    {  654, 4877,  208924,  8, }, // Pacific/Majuro
    {  654, 4051,  208932, 11, }, // Pacific/Marquesas
    {  654, 8914,  208943,  8, }, // Pacific/Midway
    {  654, 7084,  208951,  7, }, // Pacific/Nauru
    {  654, 7132,  208958,  6, }, // Pacific/Niue
    {  654, 4168,  208964,  9, }, // Pacific/Norfolk
    {  654, 7098,  208973,  8, }, // Pacific/Noumea
    {  654, 3126,  208981, 11, }, // Pacific/Pago_Pago
    {  654, 7155,  208992,  7, }, // Pacific/Palau
    {  654, 7194,  208999, 10, }, // Pacific/Pitcairn
    {  654, 3096,  209009,  9, }, // Pacific/Pohnpei
    {  654, 4807,  209018, 14, }, // Pacific/Port_Moresby
    {  654, 6076,  209032, 11, }, // Pacific/Rarotonga
    {  654, 6031,  209043,  8, }, // Pacific/Saipan
    {  654, 7312,  209051,  8, }, // Pacific/Tahiti
    {  654, 6712,  209059,  8, }, // Pacific/Tarawa
    {  654, 4583,  209067, 11, }, // Pacific/Tongatapu
    {  654, 7392,  209078,  6, }, // Pacific/Wake
    {  654, 7405,  209084,  8, }, // Pacific/Wallis
    {  655, 6788,  209092,  7, }, // Africa/Abidjan Kaingang/Latin/Brazil
    {  655, 6760,    4531,  4, }, // Africa/Accra
    {  655, 5059,  209099, 11, }, // Africa/Addis_Ababa
    {  655, 7420,  209110,  5, }, // Africa/Algiers
    {  655,   14,  209115,  7, }, // Africa/Asmara
    {  655,   44,  209122,  6, }, // Africa/Bamako
    {  655, 5257,  209128,  4, }, // Africa/Bangui
    {  655, 6746,  209132,  6, }, // Africa/Banjul
    {  655, 7435,  209138,  5, }, // Africa/Bissau
    {  655, 4957,  209143,  8, }, // Africa/Blantyre
    {  655, 5287,  209151,  9, }, // Africa/Brazzaville
    {  655, 4922,  209160,  8, }, // Africa/Bujumbura
    {  655, 2320,    6228,  5, }, // Africa/Cairo
    {  655, 4117,  209168,  7, }, // Africa/Casablanca
    {  655, 7449,  209175,  6, }, // Africa/Ceuta
    {  655, 6773,  209181,  7, }, // Africa/Conakry
    {  655, 6840,  209188,  5, }, // Africa/Dakar
    {  655, 5130,  209193, 13, }, // Africa/Dar_es_Salaam
    {  655, 5043,  209206,  7, }, // Africa/Djibouti
    {  655, 5243,  209213,  6, }, // Africa/Douala
    {  655, 5166,  209219,  9, }, // Africa/El_Aaiun
    {  655, 6853,  209228,  7, }, // Africa/Freetown
    {  655, 4906,  209235,  8, }, // Africa/Gaborone
    {  655, 5015,   98224,  6, }, // Africa/Harare
    {  655, 4479,  209243, 13, }, // Africa/Johannesburg
    {  655, 4499,  209256,  4, }, // Africa/Juba
    {  655, 5151,  209260,  7, }, // Africa/Kampala
    {  655, 4524,  209267,  6, }, // Africa/Khartoum
    {  655, 4973,  209273,  6, }, // Africa/Kigali
    {  655, 5306,  209279,  6, }, // Africa/Kinshasa
    {  655, 4744,  209285,  4, }, // Africa/Lagos
    {  655, 5336,  209289,  9, }, // Africa/Libreville
    {  655, 6869,  209298,  4, }, // Africa/Lome
    {  655, 5211,  209302,  6, }, // Africa/Luanda
    {  655, 4939,  209308,  8, }, // Africa/Lubumbashi
    {  655, 5001,  209316,  6, }, // Africa/Lusaka
    {  655, 5322,  209322,  6, }, // Africa/Malabo
    {  655, 4987,  209328,  6, }, // Africa/Maputo
    {  655, 5197,  209334,  6, }, // Africa/Maseru
    {  655, 5182,  209340,  8, }, // Africa/Mbabane
    {  655, 5113,  209348, 10, }, // Africa/Mogadishu
    {  655, 7034,  209358,  9, }, // Africa/Monrovia
    {  655, 3853,  209367,  7, }, // Africa/Nairobi
    {  655, 5271,  209374, 11, }, // Africa/Ndjamena
    {  655, 5354,  209385,  6, }, // Africa/Niamey
    {  655, 6803,  209391,  9, }, // Africa/Nouakchott
    {  655, 6727,  209400, 10, }, // Africa/Ouagadougou
    {  655, 5225,  209410,  9, }, // Africa/Porto-Novo
    {  655, 4435,  209419,  7, }, // Africa/Sao_Tome
    {  655, 2866,  209426,  7, }, // Africa/Tripoli
    {  655, 6554,  209433,  6, }, // Africa/Tunis
    {  655, 4152,  209439,  8, }, // Africa/Windhoek
    {  655,  132,  209447,  5, }, // America/Adak
    {  655, 3346,  209452,  7, }, // America/Anchorage
    {  655, 5607,  209459,  5, }, // America/Anguilla
    {  655, 5624,  209464,  7, }, // America/Antigua
    {  655, 4554,  209471,  9, }, // America/Araguaina
    {  655,  166,  209480, 10, }, // America/Argentina/Buenos_Aires
    {  655,   91,  209490,  9, }, // America/Argentina/Catamarca
    {  655,  270,  209499,  7, }, // America/Argentina/Cordoba
    {  655,  441,  209506,  5, }, // America/Argentina/Jujuy
    {  655, 7462,  209511,  8, }, // America/Argentina/La_Rioja
    {  655,  565,  209519,  6, }, // America/Argentina/Mendoza
    {  655, 7489,  209525, 10, }, // America/Argentina/Rio_Gallegos
    {  655, 7520,  209535,  5, }, // America/Argentina/Salta
    {  655, 7544,  209540,  7, }, // America/Argentina/San_Juan
    {  655, 5580,  209547,  6, }, // America/Argentina/San_Luis
    {  655, 7571,  209553,  6, }, // America/Argentina/Tucuman
    {  655, 7597,  209559,  6, }, // America/Argentina/Ushuaia
    {  655, 5640,  209565,  5, }, // America/Aruba
    {  655, 4252,  209570,  6, }, // America/Asuncion
    {  655,  237,  209576,  8, }, // America/Atikokan
    {  655, 3679,  209584,  5, }, // America/Bahia
    {  655, 7623,  209589, 16, }, // America/Bahia_Banderas
    {  655, 5654,  209605,  7, }, // America/Barbados
    {  655, 7646,  209612,  5, }, // America/Belem
    {  655, 5407,  209617,  6, }, // America/Belize
    {  655, 7660,  209623, 12, }, // America/Blanc-Sablon
    {  655, 7681,  209635,  7, }, // America/Boa_Vista
    {  655, 4361,  209642,  6, }, // America/Bogota
    {  655, 7699,  209648,  5, }, // America/Boise
    {  655, 7713,  209653, 12, }, // America/Cambridge_Bay
    {  655, 7735,  209665,  6, }, // America/Campo_Grande
    {  655, 3868,  209671,  5, }, // America/Cancun
    {  655, 4694,  209676,  8, }, // America/Caracas
    {  655, 4345,  209684,  6, }, // America/Cayenne
    {  655, 5496,  209690,  6, }, // America/Cayman
    {  655, 2260,  209696,  6, }, // America/Chicago
    {  655, 7756,  209702, 10, }, // America/Chihuahua
    {  655, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  655, 5422,  209712, 11, }, // America/Costa_Rica
    {  655, 7796,  209723,  8, }, // America/Creston
    {  655, 3791,  209731,  6, }, // America/Cuiaba
    {  655, 5723,  209737,  8, }, // America/Curacao
    {  655, 7812,  209745, 13, }, // America/Danmarkshavn
    {  655, 7833,  209758,  6, }, // America/Dawson
    {  655, 7848,  209764, 12, }, // America/Dawson_Creek
    {  655,  805,  209776,  6, }, // America/Denver
    {  655, 3465,  209782,  9, }, // America/Detroit
    {  655, 5739,  209791,  8, }, // America/Dominica
    {  655,  893,  209799,  8, }, // America/Edmonton
    {  655, 7869,  209807,  8, }, // America/Eirunepe
    {  655, 5441,  209815, 11, }, // America/El_Salvador
    {  655, 7886,  209826, 11, }, // America/Fort_Nelson
    {  655, 7906,  209837,  9, }, // America/Fortaleza
    {  655, 7924,  209846,  9, }, // America/Glace_Bay
    {  655, 6881,  209855,  8, }, // America/Goose_Bay
    {  655, 4612,  209863,  9, }, // America/Grand_Turk
    {  655, 5770,  209872,  7, }, // America/Grenada
    {  655, 5786,  209879, 10, }, // America/Guadeloupe
    {  655, 3760,  209889, 10, }, // America/Guatemala
    {  655, 4841,  209899,  9, }, // America/Guayaquil
    {  655, 6932,  209908,  6, }, // America/Guyana
    {  655, 1939,  209914,  7, }, // America/Halifax
    {  655, 2281,  209921,  6, }, // America/Havana
    {  655, 7942,  209927,  9, }, // America/Hermosillo
    {  655,  348,  209936, 13, }, // America/Indiana/Indianapolis
    {  655,  481,  209949, 14, }, // America/Indiana/Knox
    {  655, 7961,  209963, 16, }, // America/Indiana/Marengo
    {  655, 7985,  209979, 19, }, // America/Indiana/Petersburg
    {  655, 8012,  209998, 17, }, // America/Indiana/Tell_City
    {  655, 8038,  210015, 14, }, // America/Indiana/Vevay
    {  655, 8060,  210029, 17, }, // America/Indiana/Vincennes
    {  655, 8086,  210046, 17, }, // America/Indiana/Winamac
    {  655, 8110,  210063,  7, }, // America/Inuvik
    {  655,  660,  210070,  8, }, // America/Iqaluit
    {  655, 2799,  210078,  7, }, // America/Jamaica
    {  655, 5380,  210085,  6, }, // America/Juneau
    {  655,  521,  210091,  9, }, // America/Kentucky/Louisville
    {  655, 8125,  210100, 17, }, // America/Kentucky/Monticello
    {  655, 5704,  210117,  9, }, // America/Kralendijk
    {  655, 4376,  210126,  7, }, // America/La_Paz
    {  655, 7169,  210133,  4, }, // America/Lima
    {  655, 3239,  210137, 11, }, // America/Los_Angeles
    {  655, 5932,  210148, 21, }, // America/Lower_Princes
    {  655, 8153,  210169,  6, }, // America/Maceio
    {  655, 8168,  210175,  8, }, // America/Managua
    {  655, 1908,  210183,  6, }, // America/Manaus
    {  655, 5897,  210189,  8, }, // America/Marigot
    {  655, 5805,  210197,  9, }, // America/Martinique
    {  655, 8184,  210206, 12, }, // America/Matamoros
    {  655, 2917,  210218,  8, }, // America/Mazatlan
    {  655, 8202,  210226,  8, }, // America/Menominee
    {  655, 8220,  210234,  6, }, // America/Merida
    {  655, 8235,  210240, 12, }, // America/Metlakatla
    {  655, 2949,  210252, 13, }, // America/Mexico_City
    {  655, 4391,  210265,  7, }, // America/Miquelon
    {  655, 8254,  210272,  6, }, // America/Moncton
    {  655, 8270,  210278,  7, }, // America/Monterrey
    {  655, 4098,  210285,  9, }, // America/Montevideo
    {  655, 5824,  210294,  9, }, // America/Montserrat
    {  655, 5481,  210303,  5, }, // America/Nassau
    {  655, 2379,  210308, 10, }, // America/New_York
    {  655, 8288,  210318,  4, }, // America/Nome
    {  655, 1850,  210322, 17, }, // America/Noronha
    {  655, 8301,  210339, 19, }, // America/North_Dakota/Beulah
    {  655, 8329,  210358, 20, }, // America/North_Dakota/Center
    {  655, 8357,  210378, 23, }, // America/North_Dakota/New_Salem
    {  655,  393,  210401,  3, }, // America/Nuuk
    {  655, 8388,  210404,  7, }, // America/Ojinaga
    {  655, 2356,  210411,  6, }, // America/Panama
    {  655, 6173,  210417, 10, }, // America/Paramaribo
    {  655, 2973,  210427,  7, }, // America/Phoenix
    {  655, 3945,  210434, 13, }, // America/Port-au-Prince
    {  655, 5954,  210447, 16, }, // America/Port_of_Spain
    {  655, 8404,  210463, 10, }, // America/Porto_Velho
    {  655, 5843,  166548, 10, }, // America/Puerto_Rico
    {  655, 4030,  210473, 10, }, // America/Punta_Arenas
    {  655, 8424,  210483, 11, }, // America/Rankin_Inlet
    {  655, 8445,  210494,  6, }, // America/Recife
    {  655, 1995,  210500,  6, }, // America/Regina
    {  655, 8460,  210506,  8, }, // America/Resolute
    {  655,  695,  210514,  9, }, // America/Rio_Branco
    {  655, 8477,  210523,  7, }, // America/Santarem
    {  655, 2201,  210530,  8, }, // America/Santiago
    {  655, 6111,  210538, 12, }, // America/Santo_Domingo
    {  655, 1878,  210550,  9, }, // America/Sao_Paulo
    {  655, 6899,  210559, 15, }, // America/Scoresbysund
    {  655, 8494,  210574,  5, }, // America/Sitka
    {  655, 8508,  210579, 12, }, // America/St_Barthelemy
    {  655, 2061,  210591,  8, }, // America/St_Johns
    {  655, 5863,  210599, 10, }, // America/St_Kitts
    {  655, 5880,  210609,  9, }, // America/St_Lucia
    {  655,  855,  210618,  8, }, // America/St_Thomas
    {  655, 5913,  210626,  9, }, // America/St_Vincent
    {  655, 8530,  210635, 15, }, // America/Swift_Current
    {  655, 5461,  210650, 11, }, // America/Tegucigalpa
    {  655, 5756,  210661,  4, }, // America/Thule
    {  655,  313,  210665,  6, }, // America/Tijuana
    {  655,  608,  210671,  6, }, // America/Toronto
    {  655, 5688,  210677,  7, }, // America/Tortola
    {  655, 2093,  210684,  8, }, // America/Vancouver
    {  655, 2144,  210692, 12, }, // America/Whitehorse
    {  655,  734,  210704,  7, }, // America/Winnipeg
    {  655, 8552,  210711,  9, }, // America/Yakutat
    {  655, 6001,  210720,  5, }, // Antarctica/Casey
    {  655, 6094,  210725,  6, }, // Antarctica/Davis
    {  655, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  655, 8568,  210731,  9, }, // Antarctica/Macquarie
    {  655, 7066,  210740,  6, }, // Antarctica/Mawson
    {  655, 7113,  210746,  9, }, // Antarctica/McMurdo
    {  655, 8589,  210755,  6, }, // Antarctica/Palmer
    {  655, 7226,    5465,  6, }, // Antarctica/Rothera
    {  655, 7295,  126190,  5, }, // Antarctica/Syowa
    {  655, 8607,  210761,  5, }, // Antarctica/Troll
    {  655, 7374,  210766,  8, }, // Antarctica/Vostok
    {  655, 1429,  210774, 14, }, // Arctic/Longyearbyen
    {  655, 5570,  210788,  4, }, // Asia/Aden
    {  655, 5395,  210792,  6, }, // Asia/Almaty
    {  655, 3968,  210798,  3, }, // Asia/Amman
    {  655, 5511,  210801,  7, }, // Asia/Anadyr
    {  655, 5523,  210808,  6, }, // Asia/Aqtau
    {  655, 5368,  210814,  7, }, // Asia/Aqtobe
    {  655,  964,  210821,  9, }, // Asia/Ashgabat
    {  655, 8624,  210830,  6, }, // Asia/Atyrau
    {  655, 3607,  210836,  6, }, // Asia/Baghdad
    {  655, 5534,  210842,  5, }, // Asia/Bahrain
    {  655, 3653,   22146,  4, }, // Asia/Baku
    {  655, 4466,  210847,  7, }, // Asia/Bangkok
    {  655, 3571,  210854,  7, }, // Asia/Barnaul
    {  655, 4086,  210861,  7, }, // Asia/Beirut
    {  655, 3778,  210868,  8, }, // Asia/Bishkek
    {  655, 5989,  210876,  6, }, // Asia/Brunei
    {  655, 4601,  210882,  4, }, // Asia/Chita
    {  655, 4511,  210886,  7, }, // Asia/Colombo
    {  655, 4540,  210893,  8, }, // Asia/Damascus
    {  655, 1093,  209188,  4, }, // Asia/Dhaka
    {  655, 6192,  210901,  4, }, // Asia/Dili
    {  655, 3596,  210905,  5, }, // Asia/Dubai
    {  655, 6159,  210910,  7, }, // Asia/Dushanbe
    {  655, 8636,  210917,  9, }, // Asia/Famagusta
    {  655, 8651,  210926,  4, }, // Asia/Gaza
    {  655, 4795,  210930,  6, }, // Asia/Hebron
    {  655, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  655, 2704,  210936,  7, }, // Asia/Hong_Kong
    {  655, 4771,  210943,  5, }, // Asia/Hovd
    {  655, 4184,  210948, 10, }, // Asia/Irkutsk
    {  655, 6992,  210958,  8, }, // Asia/Jayapura
    {  655, 1290,  210966,  9, }, // Asia/Jerusalem
    {  655, 3560,  210975,  5, }, // Asia/Kabul
    {  655, 4316,  210980,  8, }, // Asia/Kamchatka
    {  655, 4239,  210988,  6, }, // Asia/Karachi
    {  655, 1185,  210994,  9, }, // Asia/Kathmandu
    {  655, 8661,  211003,  6, }, // Asia/Khandyga
    {  655,  992,  211009,  7, }, // Asia/Kolkata
    {  655, 4197,  211016, 12, }, // Asia/Krasnoyarsk
    {  655, 4859,  211028, 12, }, // Asia/Kuala_Lumpur
    {  655, 5976,  211040,  5, }, // Asia/Kuching
    {  655, 5547,  211045,  7, }, // Asia/Kuwait
    {  655, 1211,  211052,  5, }, // Asia/Macau
    {  655, 4017,  211057,  7, }, // Asia/Magadan
    {  655, 7182,  211064,  6, }, // Asia/Manila
    {  655, 2554,  211070,  7, }, // Asia/Nicosia
    {  655, 8675,  211077, 16, }, // Asia/Novokuznetsk
    {  655, 4135,  211093, 13, }, // Asia/Novosibirsk
    {  655, 4229,  211106,  7, }, // Asia/Omsk
    {  655, 7145,  211113,  4, }, // Asia/Oral
    {  655, 6961,  211117,  9, }, // Asia/Phnom_Penh
    {  655, 8693,  211126, 10, }, // Asia/Pontianak
    {  655, 4214,  211136,  8, }, // Asia/Pyongyang
    {  655, 5559,    4214,  5, }, // Asia/Qatar
    {  655, 8708,  211144,  8, }, // Asia/Qostanay
    {  655, 4269,  211152,  9, }, // Asia/Qyzylorda
    {  655, 3584,  211161,  6, }, // Asia/Riyadh
    {  655, 4408,  211167,  8, }, // Asia/Sakhalin
    {  655, 7245,  211175, 10, }, // Asia/Samarkand
    {  655, 3279,  211185,  5, }, // Asia/Seoul
    {  655, 1053,  211190,  6, }, // Asia/Shanghai
    {  655, 3300,  211196,  8, }, // Asia/Singapore
    {  655, 4297,  211204, 15, }, // Asia/Srednekolymsk
    {  655, 3263,    7712,  6, }, // Asia/Taipei
    {  655, 4781,  211219,  9, }, // Asia/Tashkent
    {  655, 3915,  211228,  8, }, // Asia/Tbilisi
    {  655, 2772,  211236,  6, }, // Asia/Tehran
    {  655, 1317,  211242,  5, }, // Asia/Thimphu
    {  655, 2821,  211247,  6, }, // Asia/Tokyo
    {  655, 4572,  211253,  8, }, // Asia/Tomsk
    {  655, 1021,  211261, 10, }, // Asia/Ulaanbaatar
    {  655, 1159,  211271,  6, }, // Asia/Urumqi
    {  655, 8722,  211277, 10, }, // Asia/Ust-Nera
    {  655, 6977,  211287, 10, }, // Asia/Vientiane
    {  655, 4710,  211297, 13, }, // Asia/Vladivostok
    {  655, 4828,  211310, 11, }, // Asia/Yakutsk
    {  655, 1235,  211321,  6, }, // Asia/Yangon
    {  655, 3883,  211327, 13, }, // Asia/Yekaterinburg
    {  655, 3747,  211340,  6, }, // Asia/Yerevan
    {  655, 3663,  211346,  7, }, // Atlantic/Azores
    {  655, 5671,  211353,  7, }, // Atlantic/Bermuda
    {  655, 6628,  211360, 11, }, // Atlantic/Canary
    {  655, 3727,  211371,  7, }, // Atlantic/Cape_Verde
    {  655, 1395,     448,  5, }, // Atlantic/Faroe
    {  655, 8736,     413,  2, }, // Atlantic/Madeira
    {  655, 2748,  211378, 11, }, // Atlantic/Reykjavik
    {  655, 7272,  211389, 13, }, // Atlantic/South_Georgia
    {  655, 6821,  211402, 10, }, // Atlantic/St_Helena
    {  655, 6644,  211412,  7, }, // Atlantic/Stanley
    {  655, 1670,  211419,  8, }, // Australia/Adelaide
    {  655, 1635,  211427,  9, }, // Australia/Brisbane
    {  655, 1799,  211436, 10, }, // Australia/Broken_Hill
    {  655, 1583,  211446,  6, }, // Australia/Darwin
    {  655, 3637,  211452,  5, }, // Australia/Eucla
    {  655, 1516,  211457,  7, }, // Australia/Hobart
    {  655, 8753,  211464,  8, }, // Australia/Lindeman
    {  655, 1547,  211472, 10, }, // Australia/Lord_Howe
    {  655, 1727,  211482,  8, }, // Australia/Melbourne
    {  655, 1762,  211490,  5, }, // Australia/Perth
    {  655, 1463,  211495,  7, }, // Australia/Sydney
    {  655, 6441,  211502, 10, }, // Europe/Amsterdam
    {  655, 6216,  211512,  5, }, // Europe/Andorra
    {  655, 3620,  211517,  7, }, // Europe/Astrakhan
    {  655, 2300,  211524,  5, }, // Europe/Athens
    {  655, 6336,  211529,  8, }, // Europe/Belgrade
    {  655, 4757,  211537,  7, }, // Europe/Berlin
    {  655, 6488,  211544, 11, }, // Europe/Bratislava
    {  655, 2167,  211555,  7, }, // Europe/Brussels
    {  655, 3928,  211562, 10, }, // Europe/Bucharest
    {  655, 3806,  211572, 10, }, // Europe/Budapest
    {  655, 8772,  211582,  8, }, // Europe/Busingen
    {  655, 2583,  211590,  8, }, // Europe/Chisinau
    {  655, 6289,  211598,  9, }, // Europe/Copenhagen
    {  655, 2338,  211607,  6, }, // Europe/Dublin
    {  655, 6307,  211613,  8, }, // Europe/Gibraltar
    {  655, 8788,  211621,  8, }, // Europe/Guernsey
    {  655, 6612,  211629,  8, }, // Europe/Helsinki
    {  655, 8804,  211637, 15, }, // Europe/Isle_of_Man
    {  655, 1130,  211652,  8, }, // Europe/Istanbul
    {  655, 8823,  211660,  6, }, // Europe/Jersey
    {  655, 3979,  211666, 12, }, // Europe/Kaliningrad
    {  655, 8929,  211678,  6, }, // Europe/Kirov
    {  655, 2527,  211684,  6, }, // Europe/Kyiv
    {  655, 3213,  211690,  8, }, // Europe/Lisbon
    {  655, 6506,  211698,  8, }, // Europe/Ljubljana
    {  655, 2501,  211706,  6, }, // Europe/London
    {  655, 6365,  211712, 10, }, // Europe/Luxembourg
    {  655, 4892,  211722,  6, }, // Europe/Madrid
    {  655, 6397,  211728,  5, }, // Europe/Malta
    {  655, 6582,  211733,  8, }, // Europe/Mariehamn
    {  655, 3693,  211741,  7, }, // Europe/Minsk
    {  655, 6410,  211748,  6, }, // Europe/Monaco
    {  655, 3537,  211754,  7, }, // Europe/Moscow
    {  655, 6458,  211761,  5, }, // Europe/Oslo
    {  655, 4284,  211766,  6, }, // Europe/Paris
    {  655, 6424,  211772,  9, }, // Europe/Podgorica
    {  655, 6275,   21067,  5, }, // Europe/Prague
    {  655, 6324,  211781,  4, }, // Europe/Rome
    {  655, 4331,  211785,  6, }, // Europe/Samara
    {  655, 6470,  211791,  9, }, // Europe/San_Marino
    {  655, 6245,  211800, 17, }, // Europe/Sarajevo
    {  655, 4451,  211817,  8, }, // Europe/Saratov
    {  655, 8849,  211825, 10, }, // Europe/Simferopol
    {  655, 6383,  211835,  8, }, // Europe/Skopje
    {  655, 6599,  211843,  6, }, // Europe/Sofia
    {  655, 6523,  211849, 10, }, // Europe/Stockholm
    {  655, 8867,  211859,  5, }, // Europe/Tallinn
    {  655, 6202,  211864,  6, }, // Europe/Tirane
    {  655, 8882,  211870, 13, }, // Europe/Ulyanovsk
    {  655, 6352,  211883,  5, }, // Europe/Vaduz
    {  655, 6567,  211888,  8, }, // Europe/Vatican
    {  655, 6231,  211896,  5, }, // Europe/Vienna
    {  655, 8899,  211901,  9, }, // Europe/Vilnius
    {  655, 4727,  211910, 10, }, // Europe/Volgograd
    {  655, 3190,  211920,  9, }, // Europe/Warsaw
    {  655, 6261,  211929,  7, }, // Europe/Zagreb
    {  655, 6540,  211936,  6, }, // Europe/Zurich
    {  655, 5078,  211942, 11, }, // Indian/Antananarivo
    {  655, 6947,  211953,  4, }, // Indian/Chagos
    {  655, 6046,  211957,  9, }, // Indian/Christmas
    {  655, 6063,  211966,  7, }, // Indian/Cocos
    {  655, 5029,  211973,  6, }, // Indian/Comoro
    {  655, 6661,  211979,  8, }, // Indian/Kerguelen
    {  655, 7260,  211987,  4, }, // Indian/Mahe
    {  655, 7050,  211991,  7, }, // Indian/Maldives
    {  655, 4069,  211998,  7, }, // Indian/Mauritius
    {  655, 5098,  212005,  6, }, // Indian/Mayotte
    {  655, 7211,  212011,  6, }, // Indian/Reunion
    {  655, 4422,  124364,  5, }, // Pacific/Apia
    {  655,  932,  212017,  6, }, // Pacific/Auckland
    {  655, 3706,  212023,  9, }, // Pacific/Bougainville
    {  655, 3015,  212032,  7, }, // Pacific/Chatham
    {  655, 3157,     474,  5, }, // Pacific/Chuuk
    {  655, 2237,  212039, 17, }, // Pacific/Easter
    {  655, 7360,   27016,  5, }, // Pacific/Efate
    {  655, 7327,  212056,  7, }, // Pacific/Fakaofo
    {  655, 7343,  212063,  8, }, // Pacific/Funafuti
    {  655, 6678,  212071,  9, }, // Pacific/Galapagos
    {  655, 6696,  212080,  7, }, // Pacific/Gambier
    {  655, 3822,  212087, 12, }, // Pacific/Guadalcanal
    {  655, 6018,  212099,  5, }, // Pacific/Guam
    {  655, 2723,  212104,  8, }, // Pacific/Honolulu
    {  655, 3049,  212112,  9, }, // Pacific/Kanton
    {  655, 3998,  212121, 10, }, // Pacific/Kiritimati
    {  655, 7019,  212131,  8, }, // Pacific/Kosrae
    {  655, 2842,  212139,  9, }, // Pacific/Kwajalein
    {  655, 4877,  212148,  6, }, // Pacific/Majuro
    {  655, 4051,  212154, 11, }, // Pacific/Marquesas
    {  655, 8914,  212165,  8, }, // Pacific/Midway
    {  655, 7084,  212173,  6, }, // Pacific/Nauru
    {  655, 7132,  212179,  4, }, // Pacific/Niue
    {  655, 4168,  212183,  7, }, // Pacific/Norfolk
    {  655, 7098,  212190,  6, }, // Pacific/Noumea
    {  655, 7155,  212196,  5, }, // Pacific/Palau
    {  655, 7194,  212201,  8, }, // Pacific/Pitcairn
    {  655, 3096,     479,  7, }, // Pacific/Pohnpei
    {  655, 4807,  212209, 12, }, // Pacific/Port_Moresby
    {  655, 6076,  212221,  8, }, // Pacific/Rarotonga
    {  655, 6031,  212229,  5, }, // Pacific/Saipan
    {  655, 7312,  212234,  5, }, // Pacific/Tahiti
    {  655, 6712,  212239,  7, }, // Pacific/Tarawa
    {  655, 4583,  212246,  9, }, // Pacific/Tongatapu
    {  655, 7392,  212255,  4, }, // Pacific/Wake
    {  655, 7405,  212259,  6, }, // Pacific/Wallis
    {  656, 6788,  212265,  7, }, // Africa/Abidjan Nheengatu/Latin/Brazil
    {  656, 6760,  212272,  5, }, // Africa/Accra
    {  656, 5059,  212277, 11, }, // Africa/Addis_Ababa
    {  656, 7420,  212288,  4, }, // Africa/Algiers
    {  656,   14,  212292,  7, }, // Africa/Asmara
    {  656,   44,   98141,  6, }, // Africa/Bamako
    {  656, 5257,  212299,  4, }, // Africa/Bangui
    {  656, 6746,  212303,  4, }, // Africa/Banjul
    {  656, 7435,    6195,  5, }, // Africa/Bissau
    {  656, 4957,  212307,  8, }, // Africa/Blantyre
    {  656, 5287,  212315, 10, }, // Africa/Brazzaville
    {  656, 4922,  212325,  8, }, // Africa/Bujumbura
    {  656, 2320,  212333,  5, }, // Africa/Cairo
    {  656, 4117,  212338, 12, }, // Africa/Casablanca
    {  656, 7449,   21231,  5, }, // Africa/Ceuta
    {  656, 6773,  212350,  8, }, // Africa/Conakry
    {  656, 5130,  212358, 12, }, // Africa/Dar_es_Salaam
    {  656, 5043,  212370,  8, }, // Africa/Djibouti
    {  656, 5243,  212378,  6, }, // Africa/Douala
    {  656, 5166,  212384,  6, }, // Africa/El_Aaiun
    {  656, 6853,  212390,  6, }, // Africa/Freetown
    {  656, 4906,  212396,  8, }, // Africa/Gaborone
    {  656, 4479,  212404, 11, }, // Africa/Johannesburg
    {  656, 4499,  212415,  4, }, // Africa/Juba
    {  656, 5151,  212419,  6, }, // Africa/Kampala
    {  656, 4524,  212425,  4, }, // Africa/Khartoum
    {  656, 4973,  209273,  6, }, // Africa/Kigali
    {  656, 5306,  212429,  6, }, // Africa/Kinshasa
    {  656, 4744,  212435,  8, }, // Africa/Lagos
    {  656, 5336,  212443, 10, }, // Africa/Libreville
    {  656, 6869,  212453,  4, }, // Africa/Lome
    {  656, 5211,  212457,  5, }, // Africa/Luanda
    {  656, 4939,  212462,  8, }, // Africa/Lubumbashi
    {  656, 5322,   98292,  6, }, // Africa/Malabo
    {  656, 4987,  122060,  6, }, // Africa/Maputo
    {  656, 5182,  212470,  7, }, // Africa/Mbabane
    {  656, 5113,  212477, 10, }, // Africa/Mogadishu
    {  656, 7034,  212487,  8, }, // Africa/Monrovia
    {  656, 3853,  212495,  7, }, // Africa/Nairobi
    {  656, 5271,  212502, 11, }, // Africa/Ndjamena
    {  656, 5354,   26396,  6, }, // Africa/Niamey
    {  656, 6803,  212513, 10, }, // Africa/Nouakchott
    {  656, 6727,  212523,  9, }, // Africa/Ouagadougou
    {  656, 5225,  212532, 16, }, // Africa/Porto-Novo
    {  656, 4435,  212548,  7, }, // Africa/Sao_Tome
    {  656, 2866,  212555,  8, }, // Africa/Tripoli
    {  656, 6554,  212563,  8, }, // Africa/Tunis
    {  656, 4152,  212571,  7, }, // Africa/Windhoek
    {  656,  132,   97814,  5, }, // America/Adak
    {  656, 3346,  212578,  7, }, // America/Anchorage
    {  656, 5607,  212585,  5, }, // America/Anguilla
    {  656, 5624,  212590,  7, }, // America/Antigua
    {  656, 4554,  212597, 10, }, // America/Araguaina
    {  656,  166,  212607, 11, }, // America/Argentina/Buenos_Aires
    {  656,   91,  212618,  8, }, // America/Argentina/Catamarca
    {  656,  270,  212626,  6, }, // America/Argentina/Cordoba
    {  656,  441,  212632,  5, }, // America/Argentina/Jujuy
    {  656, 7462,  212637,  8, }, // America/Argentina/La_Rioja
    {  656,  565,  212645,  6, }, // America/Argentina/Mendoza
    {  656, 7489,  212651, 17, }, // America/Argentina/Rio_Gallegos
    {  656, 7520,  212668,  5, }, // America/Argentina/Salta
    {  656, 7544,  212673,  7, }, // America/Argentina/San_Juan
    {  656, 5580,  212680,  6, }, // America/Argentina/San_Luis
    {  656, 7571,  212686,  6, }, // America/Argentina/Tucuman
    {  656, 7597,  212692,  7, }, // America/Argentina/Ushuaia
    {  656, 4252,  212699,  7, }, // America/Asuncion
    {  656,  237,  212706,  7, }, // America/Atikokan
    {  656, 3679,  212713,  5, }, // America/Bahia
    {  656, 7623,  212718, 16, }, // America/Bahia_Banderas
    {  656, 5654,  212734,  8, }, // America/Barbados
    {  656, 7646,  212742,  4, }, // America/Belem
    {  656, 5407,  212746,  6, }, // America/Belize
    {  656, 7660,  212752, 15, }, // America/Blanc-Sablon
    {  656, 7681,  212767, 10, }, // America/Boa_Vista
    {  656, 4361,  212777,  6, }, // America/Bogota
    {  656, 7699,  212783,  6, }, // America/Boise
    {  656, 7713,  212789, 14, }, // America/Cambridge_Bay
    {  656, 7735,  212803,  9, }, // America/Campo_Grande
    {  656, 3868,  212812,  4, }, // America/Cancun
    {  656, 4694,    4792,  6, }, // America/Caracas
    {  656, 4345,  212816,  6, }, // America/Cayenne
    {  656, 5496,  212822,  5, }, // America/Cayman
    {  656, 2260,   97863,  6, }, // America/Chicago
    {  656, 7756,  212827,  6, }, // America/Chihuahua
    {  656, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  656, 5422,  212833,  9, }, // America/Costa_Rica
    {  656, 7796,  212842,  7, }, // America/Creston
    {  656, 3791,   21513,  6, }, // America/Cuiaba
    {  656, 5723,   97882,  7, }, // America/Curacao
    {  656, 7812,  212849, 13, }, // America/Danmarkshavn
    {  656, 7833,  212862,  5, }, // America/Dawson
    {  656, 7848,  212867, 12, }, // America/Dawson_Creek
    {  656,  805,  212879,  5, }, // America/Denver
    {  656, 5739,  212884,  8, }, // America/Dominica
    {  656,  893,  212892,  7, }, // America/Edmonton
    {  656, 7869,   26448,  8, }, // America/Eirunepe
    {  656, 5441,  212899,  9, }, // America/El_Salvador
    {  656, 7886,  212908, 10, }, // America/Fort_Nelson
    {  656, 7906,  212918,  8, }, // America/Fortaleza
    {  656, 7924,  212926, 19, }, // America/Glace_Bay
    {  656, 6881,  212945, 15, }, // America/Goose_Bay
    {  656, 4612,  212960, 11, }, // America/Grand_Turk
    {  656, 5770,  212971,  8, }, // America/Grenada
    {  656, 5786,  212979,  9, }, // America/Guadeloupe
    {  656, 3760,  212988,  9, }, // America/Guatemala
    {  656, 4841,  212997,  8, }, // America/Guayaquil
    {  656, 6932,  213005,  6, }, // America/Guyana
    {  656, 2281,  157045,  6, }, // America/Havana
    {  656, 7942,  213011,  8, }, // America/Hermosillo
    {  656,  348,  213019, 10, }, // America/Indiana/Indianapolis
    {  656,  481,  213029, 12, }, // America/Indiana/Knox
    {  656, 7961,  213041, 14, }, // America/Indiana/Marengo
    {  656, 7985,  213055, 20, }, // America/Indiana/Petersburg
    {  656, 8012,  213075, 22, }, // America/Indiana/Tell_City
    {  656, 8038,  213097, 13, }, // America/Indiana/Vevay
    {  656, 8060,  213110, 18, }, // America/Indiana/Vincennes
    {  656, 8086,  213128, 16, }, // America/Indiana/Winamac
    {  656, 8110,  213144,  7, }, // America/Inuvik
    {  656,  660,  213151,  8, }, // America/Iqaluit
    {  656, 2799,  213159,  7, }, // America/Jamaica
    {  656, 5380,  213166,  6, }, // America/Juneau
    {  656,  521,  213172, 10, }, // America/Kentucky/Louisville
    {  656, 8125,  213182, 16, }, // America/Kentucky/Monticello
    {  656, 5704,  213198, 10, }, // America/Kralendijk
    {  656, 7169,  213208,  4, }, // America/Lima
    {  656, 5932,  213212, 26, }, // America/Lower_Princes
    {  656, 8153,  213238,  6, }, // America/Maceio
    {  656, 8168,  213244,  8, }, // America/Managua
    {  656, 1908,  122787,  5, }, // America/Manaus
    {  656, 5897,  213252,  8, }, // America/Marigot
    {  656, 5805,  213260,  8, }, // America/Martinique
    {  656, 8184,  213268, 11, }, // America/Matamoros
    {  656, 2917,  213279,  8, }, // America/Mazatlan
    {  656, 8202,    7000,  8, }, // America/Menominee
    {  656, 8220,     229,  6, }, // America/Merida
    {  656, 8235,  213287, 12, }, // America/Metlakatla
    {  656, 2949,  213299, 13, }, // America/Mexico_City
    {  656, 4391,  213312,  6, }, // America/Miquelon
    {  656, 8254,  213318,  6, }, // America/Moncton
    {  656, 8270,  213324,  7, }, // America/Monterrey
    {  656, 4098,  213331, 10, }, // America/Montevideo
    {  656, 5824,  213341, 10, }, // America/Montserrat
    {  656, 5481,  125683,  5, }, // America/Nassau
    {  656, 2379,  213351, 12, }, // America/New_York
    {  656, 8288,  213363,  4, }, // America/Nome
    {  656, 1850,  213367, 13, }, // America/Noronha
    {  656, 8301,  213380, 21, }, // America/North_Dakota/Beulah
    {  656, 8329,  213401, 21, }, // America/North_Dakota/Center
    {  656, 8357,  213422, 28, }, // America/North_Dakota/New_Salem
    {  656,  393,  213450,  4, }, // America/Nuuk
    {  656, 8388,  213454,  7, }, // America/Ojinaga
    {  656, 2356,  213461,  6, }, // America/Panama
    {  656, 6173,   98701, 10, }, // America/Paramaribo
    {  656, 2973,  213467, 10, }, // America/Phoenix
    {  656, 3945,  213477, 18, }, // America/Port-au-Prince
    {  656, 5954,  213495, 17, }, // America/Port_of_Spain
    {  656, 8404,  213512, 20, }, // America/Porto_Velho
    {  656, 5843,  213532, 14, }, // America/Puerto_Rico
    {  656, 4030,  213546, 14, }, // America/Punta_Arenas
    {  656, 8424,  213560, 13, }, // America/Rankin_Inlet
    {  656, 8445,  123023,  6, }, // America/Recife
    {  656, 1995,  213573,  6, }, // America/Regina
    {  656, 8460,  213579,  8, }, // America/Resolute
    {  656,  695,  213587, 15, }, // America/Rio_Branco
    {  656, 8477,  213602,  6, }, // America/Santarem
    {  656, 2201,  213608,  8, }, // America/Santiago
    {  656, 6111,  213616, 11, }, // America/Santo_Domingo
    {  656, 1878,  213627,  8, }, // America/Sao_Paulo
    {  656, 6899,  213635, 11, }, // America/Scoresbysund
    {  656, 8494,  213646,  6, }, // America/Sitka
    {  656, 8508,  213652, 12, }, // America/St_Barthelemy
    {  656, 2061,  213664,  9, }, // America/St_Johns
    {  656, 5863,  213673, 12, }, // America/St_Kitts
    {  656, 5880,  213685, 11, }, // America/St_Lucia
    {  656,  855,  213696,  9, }, // America/St_Thomas
    {  656, 5913,  213705,  9, }, // America/St_Vincent
    {  656, 8530,  213714, 13, }, // America/Swift_Current
    {  656, 5461,  213727, 11, }, // America/Tegucigalpa
    {  656, 5756,  213738,  6, }, // America/Thule
    {  656,  313,  213744,  8, }, // America/Tijuana
    {  656,  608,  213752,  6, }, // America/Toronto
    {  656, 5688,  213758,  6, }, // America/Tortola
    {  656, 2093,  213764,  8, }, // America/Vancouver
    {  656, 2144,  213772, 15, }, // America/Whitehorse
    {  656,  734,  213787,  8, }, // America/Winnipeg
    {  656, 8552,  207045,  8, }, // America/Yakutat
    {  656, 6001,  213795,  5, }, // Antarctica/Casey
    {  656, 6094,  213800,  4, }, // Antarctica/Davis
    {  656, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  656, 8568,  213804,  9, }, // Antarctica/Macquarie
    {  656, 7066,  213813,  5, }, // Antarctica/Mawson
    {  656, 7113,  213818,  8, }, // Antarctica/McMurdo
    {  656, 8589,  213826,  7, }, // Antarctica/Palmer
    {  656, 7226,    5465,  6, }, // Antarctica/Rothera
    {  656, 7295,  176194,  6, }, // Antarctica/Syowa
    {  656, 7374,  213833,  8, }, // Antarctica/Vostok
    {  656, 1429,  213841, 18, }, // Arctic/Longyearbyen
    {  656, 5570,  213859,  3, }, // Asia/Aden
    {  656, 5395,  213862,  6, }, // Asia/Almaty
    {  656, 3968,  158071,  3, }, // Asia/Amman
    {  656, 5511,    7482,  5, }, // Asia/Anadyr
    {  656, 5523,  210808,  6, }, // Asia/Aqtau
    {  656, 5368,  213868,  7, }, // Asia/Aqtobe
    {  656,  964,  213875,  9, }, // Asia/Ashgabat
    {  656, 8624,    7494,  6, }, // Asia/Atyrau
    {  656, 3607,  213884,  5, }, // Asia/Baghdad
    {  656, 5534,  213889,  5, }, // Asia/Bahrain
    {  656, 4466,  213894,  8, }, // Asia/Bangkok
    {  656, 3571,  123351,  6, }, // Asia/Barnaul
    {  656, 4086,  158082,  7, }, // Asia/Beirut
    {  656, 3778,  213902,  8, }, // Asia/Bishkek
    {  656, 5989,  213910,  7, }, // Asia/Brunei
    {  656, 4601,  213917,  4, }, // Asia/Chita
    {  656, 4511,  213921,  6, }, // Asia/Colombo
    {  656, 4540,  213927,  8, }, // Asia/Damascus
    {  656, 1093,    4599,  4, }, // Asia/Dhaka
    {  656, 6192,  213935,  4, }, // Asia/Dili
    {  656, 6159,  213939,  6, }, // Asia/Dushanbe
    {  656, 8636,  213945,  9, }, // Asia/Famagusta
    {  656, 8651,   63848,  4, }, // Asia/Gaza
    {  656, 4795,  213954,  6, }, // Asia/Hebron
    {  656, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  656, 2704,  213960,  7, }, // Asia/Hong_Kong
    {  656, 4771,  213967,  6, }, // Asia/Hovd
    {  656, 4184,  213973, 10, }, // Asia/Irkutsk
    {  656, 7006,  213983,  6, }, // Asia/Jakarta
    {  656, 6992,  213989,  8, }, // Asia/Jayapura
    {  656, 1290,  213997,  9, }, // Asia/Jerusalem
    {  656, 3560,   92579,  4, }, // Asia/Kabul
    {  656, 4316,  214006,  8, }, // Asia/Kamchatka
    {  656, 4239,  214014,  6, }, // Asia/Karachi
    {  656, 1185,  214020,  8, }, // Asia/Kathmandu
    {  656, 8661,  214028,  6, }, // Asia/Khandyga
    {  656,  992,  214034,  7, }, // Asia/Kolkata
    {  656, 4197,  214041, 16, }, // Asia/Krasnoyarsk
    {  656, 4859,  214057, 12, }, // Asia/Kuala_Lumpur
    {  656, 5976,  214069,  6, }, // Asia/Kuching
    {  656, 5547,  207524,  7, }, // Asia/Kuwait
    {  656, 1211,   33405,  5, }, // Asia/Macau
    {  656, 4017,  214075,  6, }, // Asia/Magadan
    {  656, 7182,  134894,  6, }, // Asia/Manila
    {  656, 6920,  214081,  8, }, // Asia/Muscat
    {  656, 2554,   22378,  8, }, // Asia/Nicosia
    {  656, 8675,  214089, 19, }, // Asia/Novokuznetsk
    {  656, 4135,  214108, 17, }, // Asia/Novosibirsk
    {  656, 4229,  214125,  7, }, // Asia/Omsk
    {  656, 7145,  214132,  5, }, // Asia/Oral
    {  656, 6961,  214137,  8, }, // Asia/Phnom_Penh
    {  656, 8693,  214145,  9, }, // Asia/Pontianak
    {  656, 4214,  214154,  8, }, // Asia/Pyongyang
    {  656, 5559,  214162,  6, }, // Asia/Qatar
    {  656, 8708,  214168,  9, }, // Asia/Qostanay
    {  656, 4269,  214177,  8, }, // Asia/Qyzylorda
    {  656, 3584,  158148,  5, }, // Asia/Riyadh
    {  656, 4408,  214185,  8, }, // Asia/Sakhalin
    {  656, 7245,  214193,  8, }, // Asia/Samarkand
    {  656, 3279,   20778,  3, }, // Asia/Seoul
    {  656, 1053,  214201,  5, }, // Asia/Shanghai
    {  656, 3300,  211196,  8, }, // Asia/Singapore
    {  656, 4297,  214206, 18, }, // Asia/Srednekolymsk
    {  656, 4781,  214224,  8, }, // Asia/Tashkent
    {  656, 3915,  176610,  8, }, // Asia/Tbilisi
    {  656, 2772,  214232,  6, }, // Asia/Tehran
    {  656, 1317,  214238,  4, }, // Asia/Thimphu
    {  656, 2821,  214242,  6, }, // Asia/Tokyo
    {  656, 4572,  214248,  8, }, // Asia/Tomsk
    {  656, 1021,  214256, 10, }, // Asia/Ulaanbaatar
    {  656, 1159,  214266,  5, }, // Asia/Urumqi
    {  656, 8722,  214271, 10, }, // Asia/Ust-Nera
    {  656, 6977,  214281,  8, }, // Asia/Vientiane
    {  656, 4710,  214289, 14, }, // Asia/Vladivostok
    {  656, 4828,  214303, 10, }, // Asia/Yakutsk
    {  656, 1235,  214313,  4, }, // Asia/Yangon
    {  656, 3883,  214317, 11, }, // Asia/Yekaterinburg
    {  656, 3747,  214328,  6, }, // Asia/Yerevan
    {  656, 3663,  214334,  9, }, // Atlantic/Azores
    {  656, 5671,  214343,  6, }, // Atlantic/Bermuda
    {  656, 6628,  214349, 12, }, // Atlantic/Canary
    {  656, 3727,  214361, 12, }, // Atlantic/Cape_Verde
    {  656, 1395,  214373, 18, }, // Atlantic/Faroe
    {  656, 2748,  214391, 11, }, // Atlantic/Reykjavik
    {  656, 7272,  214402, 14, }, // Atlantic/South_Georgia
    {  656, 6821,  214416, 10, }, // Atlantic/St_Helena
    {  656, 6644,  214426,  8, }, // Atlantic/Stanley
    {  656, 1670,  214434,  8, }, // Australia/Adelaide
    {  656, 1635,  214442,  8, }, // Australia/Brisbane
    {  656, 1799,  214450, 17, }, // Australia/Broken_Hill
    {  656, 1583,  214467,  6, }, // Australia/Darwin
    {  656, 3637,  214473,  6, }, // Australia/Eucla
    {  656, 1516,  214479,  6, }, // Australia/Hobart
    {  656, 8753,  214485,  6, }, // Australia/Lindeman
    {  656, 1547,  214491, 11, }, // Australia/Lord_Howe
    {  656, 1727,  214502,  7, }, // Australia/Melbourne
    {  656, 1762,  214509,  6, }, // Australia/Perth
    {  656, 1463,  214515,  7, }, // Australia/Sydney
    {  656, 6441,  214522,  9, }, // Europe/Amsterdam
    {  656, 6216,  214531,  5, }, // Europe/Andorra
    {  656, 3620,  214536,  9, }, // Europe/Astrakhan
    {  656, 2300,  214545,  9, }, // Europe/Athens
    {  656, 6336,  214554,  9, }, // Europe/Belgrade
    {  656, 4757,  158245,  6, }, // Europe/Berlin
    {  656, 6488,  214563, 12, }, // Europe/Bratislava
    {  656, 2167,  214575,  8, }, // Europe/Brussels
    {  656, 3928,  214583,  8, }, // Europe/Bucharest
    {  656, 3806,  214591,  8, }, // Europe/Budapest
    {  656, 8772,  214599,  8, }, // Europe/Busingen
    {  656, 2583,  214607,  7, }, // Europe/Chisinau
    {  656, 6289,  214614,  8, }, // Europe/Copenhagen
    {  656, 2338,  214622,  6, }, // Europe/Dublin
    {  656, 6307,  214628,  9, }, // Europe/Gibraltar
    {  656, 8788,  214637,  9, }, // Europe/Guernsey
    {  656, 6612,  214646,  7, }, // Europe/Helsinki
    {  656, 8804,  214653, 10, }, // Europe/Isle_of_Man
    {  656, 1130,   57315,  8, }, // Europe/Istanbul
    {  656, 8823,  214663,  6, }, // Europe/Jersey
    {  656, 3979,  214669, 12, }, // Europe/Kaliningrad
    {  656, 8929,  214681,  6, }, // Europe/Kirov
    {  656, 2527,  214687,  6, }, // Europe/Kyiv
    {  656, 3213,   21019,  6, }, // Europe/Lisbon
    {  656, 6506,  214693, 10, }, // Europe/Ljubljana
    {  656, 2501,   21034,  7, }, // Europe/London
    {  656, 6365,  214703,  8, }, // Europe/Luxembourg
    {  656, 4892,   26943,  5, }, // Europe/Madrid
    {  656, 6397,  167143,  5, }, // Europe/Malta
    {  656, 6582,  214711,  7, }, // Europe/Mariehamn
    {  656, 3693,  214718,  6, }, // Europe/Minsk
    {  656, 6410,   99165,  6, }, // Europe/Monaco
    {  656, 3537,    4404,  6, }, // Europe/Moscow
    {  656, 6458,  214724,  5, }, // Europe/Oslo
    {  656, 6424,  214729, 10, }, // Europe/Podgorica
    {  656, 6275,   21067,  5, }, // Europe/Prague
    {  656, 6324,    5946,  4, }, // Europe/Rome
    {  656, 6470,  214739,  9, }, // Europe/San_Marino
    {  656, 6245,  214748,  8, }, // Europe/Sarajevo
    {  656, 4451,  214756,  8, }, // Europe/Saratov
    {  656, 8849,  214764,  8, }, // Europe/Simferopol
    {  656, 6383,  214772,  9, }, // Europe/Skopje
    {  656, 6599,  214781,  6, }, // Europe/Sofia
    {  656, 6523,   74486,  9, }, // Europe/Stockholm
    {  656, 8867,  211859,  4, }, // Europe/Tallinn
    {  656, 6202,   21095,  6, }, // Europe/Tirane
    {  656, 8882,  214787, 10, }, // Europe/Ulyanovsk
    {  656, 6352,  214797,  6, }, // Europe/Vaduz
    {  656, 6567,  214803,  8, }, // Europe/Vatican
    {  656, 6231,  214811,  6, }, // Europe/Vienna
    {  656, 8899,  214817,  9, }, // Europe/Vilnius
    {  656, 4727,  214826, 10, }, // Europe/Volgograd
    {  656, 3190,  214836, 10, }, // Europe/Warsaw
    {  656, 6261,  214846,  8, }, // Europe/Zagreb
    {  656, 6540,  214854,  6, }, // Europe/Zurich
    {  656, 5078,  214860, 10, }, // Indian/Antananarivo
    {  656, 6947,  214870,  8, }, // Indian/Chagos
    {  656, 6046,  214878, 12, }, // Indian/Christmas
    {  656, 6063,  214890,  8, }, // Indian/Cocos
    {  656, 5029,  214898, 10, }, // Indian/Comoro
    {  656, 6661,  214908,  7, }, // Indian/Kerguelen
    {  656, 7260,  214915,  3, }, // Indian/Mahe
    {  656, 7050,  214918, 11, }, // Indian/Maldives
    {  656, 4069,  214929,  9, }, // Indian/Mauritius
    {  656, 5098,  214938,  6, }, // Indian/Mayotte
    {  656, 7211,  214944, 13, }, // Indian/Reunion
    {  656, 4422,  214957,  5, }, // Pacific/Apia
    {  656,  932,  214962,  7, }, // Pacific/Auckland
    {  656, 3706,  214969,  9, }, // Pacific/Bougainville
    {  656, 3015,  214978,  6, }, // Pacific/Chatham
    {  656, 3157,     474,  5, }, // Pacific/Chuuk
    {  656, 2237,  214984, 16, }, // Pacific/Easter
    {  656, 7360,  215000,  5, }, // Pacific/Efate
    {  656, 7327,  215005,  7, }, // Pacific/Fakaofo
    {  656, 3902,  148101,  4, }, // Pacific/Fiji
    {  656, 6678,  215012, 12, }, // Pacific/Galapagos
    {  656, 6696,  215024,  7, }, // Pacific/Gambier
    {  656, 3822,  215031, 11, }, // Pacific/Guadalcanal
    {  656, 6018,  215042,  4, }, // Pacific/Guam
    {  656, 2723,  215046,  8, }, // Pacific/Honolulu
    {  656, 3049,  215054,  7, }, // Pacific/Kanton
    {  656, 7019,  215061,  7, }, // Pacific/Kosrae
    {  656, 2842,  215068,  8, }, // Pacific/Kwajalein
    {  656, 4877,  215076,  6, }, // Pacific/Majuro
    {  656, 4051,  215082, 10, }, // Pacific/Marquesas
    {  656, 8914,  215092,  7, }, // Pacific/Midway
    {  656, 7132,  215099,  4, }, // Pacific/Niue
    {  656, 4168,  215103,  8, }, // Pacific/Norfolk
    {  656, 7098,  215111,  6, }, // Pacific/Noumea
    {  656, 3126,  215117,  9, }, // Pacific/Pago_Pago
    {  656, 7155,  215126,  5, }, // Pacific/Palau
    {  656, 3096,     479,  7, }, // Pacific/Pohnpei
    {  656, 4807,  215131, 17, }, // Pacific/Port_Moresby
    {  656, 6076,  215148,  8, }, // Pacific/Rarotonga
    {  656, 6031,  215156,  5, }, // Pacific/Saipan
    {  656, 7312,  124568,  5, }, // Pacific/Tahiti
    {  656, 4583,  215161,  8, }, // Pacific/Tongatapu
    {  656, 7405,  215169,  8, }, // Pacific/Wallis
    {  657, 5863,  215177, 12, }, // America/St_Kitts Nheengatu/Latin/Colombia
    {  658, 5863,  215177, 12, }, // America/St_Kitts Nheengatu/Latin/Venezuela
    {  659,   14,       0,  6, }, // Africa/Asmara Haryanvi/Devanagari/India
    {  659, 4435,       6,  8, }, // Africa/Sao_Tome
    {  659, 4252,      14,  8, }, // America/Asuncion
    {  659,  237,      22,  8, }, // America/Atikokan
    {  659, 7623,      30, 17, }, // America/Bahia_Banderas
    {  659, 3868,      47,  6, }, // America/Cancun
    {  659, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  659, 5723,      66,  7, }, // America/Curacao
    {  659,  481,      73, 13, }, // America/Indiana/Knox
    {  659, 7961,      86, 16, }, // America/Indiana/Marengo
    {  659, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  659, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  659, 8038,     139, 14, }, // America/Indiana/Vevay
    {  659, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  659, 8086,     171, 16, }, // America/Indiana/Winamac
    {  659, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  659, 5932,     207, 22, }, // America/Lower_Princes
    {  659, 8220,     229,  6, }, // America/Merida
    {  659, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  659, 8329,     255, 20, }, // America/North_Dakota/Center
    {  659, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  659,  393,     298,  4, }, // America/Nuuk
    {  659, 6899,     302, 16, }, // America/Scoresbysund
    {  659, 8508,     318, 14, }, // America/St_Barthelemy
    {  659, 2061,     332, 10, }, // America/St_Johns
    {  659, 5863,     342,  9, }, // America/St_Kitts
    {  659, 5880,     351,  9, }, // America/St_Lucia
    {  659,  855,     360, 10, }, // America/St_Thomas
    {  659, 5913,     370, 11, }, // America/St_Vincent
    {  659, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  659, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  659, 1185,     413,  9, }, // Asia/Kathmandu
    {  659,  992,     422,  7, }, // Asia/Kolkata
    {  659, 1211,     429,  5, }, // Asia/Macau
    {  659, 1235,     442,  6, }, // Asia/Yangon
    {  659, 1395,     448,  5, }, // Atlantic/Faroe
    {  659, 6821,     453, 10, }, // Atlantic/St_Helena
    {  659, 2527,     463,  4, }, // Europe/Kyiv
    {  659, 7211,     467,  7, }, // Indian/Reunion
    {  659, 3157,     474,  5, }, // Pacific/Chuuk
    {  659, 3096,     479,  7, }, // Pacific/Pohnpei
    {  660, 7420,   58423,  6, }, // Africa/Algiers Northern Frisian/Latin/Germany
    {  660,   14,       0,  6, }, // Africa/Asmara
    {  660, 2320,    4572,  5, }, // Africa/Cairo
    {  660, 4435,       6,  8, }, // Africa/Sao_Tome
    {  660, 2866,   57624,  8, }, // Africa/Tripoli
    {  660, 5624,  215189,  8, }, // America/Antigua
    {  660, 5640,  215197,  6, }, // America/Aruba
    {  660, 4252,      14,  8, }, // America/Asuncion
    {  660,  237,      22,  8, }, // America/Atikokan
    {  660, 7623,   64558, 17, }, // America/Bahia_Banderas
    {  660, 5654,  215203,  9, }, // America/Barbados
    {  660, 3868,      47,  6, }, // America/Cancun
    {  660, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  660, 5723,      66,  7, }, // America/Curacao
    {  660, 5739,  215212,  9, }, // America/Dominica
    {  660, 5770,  215221,  8, }, // America/Grenada
    {  660, 3760,  215229, 10, }, // America/Guatemala
    {  660, 6932,  175301,  7, }, // America/Guyana
    {  660,  481,      73, 13, }, // America/Indiana/Knox
    {  660, 7961,      86, 16, }, // America/Indiana/Marengo
    {  660, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  660, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  660, 8038,     139, 14, }, // America/Indiana/Vevay
    {  660, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  660, 8086,     171, 16, }, // America/Indiana/Winamac
    {  660, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  660, 5932,     207, 22, }, // America/Lower_Princes
    {  660, 8220,     229,  6, }, // America/Merida
    {  660, 2949,  215239, 13, }, // America/Mexico_City
    {  660, 8301,  215252, 20, }, // America/North_Dakota/Beulah
    {  660, 8329,  215272, 20, }, // America/North_Dakota/Center
    {  660, 8357,  215292, 23, }, // America/North_Dakota/New_Salem
    {  660,  393,     298,  4, }, // America/Nuuk
    {  660, 5843,   21940, 11, }, // America/Puerto_Rico
    {  660, 6899,     302, 16, }, // America/Scoresbysund
    {  660, 8508,     318, 14, }, // America/St_Barthelemy
    {  660, 2061,     332, 10, }, // America/St_Johns
    {  660, 5863,     342,  9, }, // America/St_Kitts
    {  660, 5880,     351,  9, }, // America/St_Lucia
    {  660,  855,     360, 10, }, // America/St_Thomas
    {  660, 5913,     370, 11, }, // America/St_Vincent
    {  660, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  660, 7374,    4029,  6, }, // Antarctica/Vostok
    {  660,  964,  215315,  9, }, // Asia/Ashgabat
    {  660, 3607,    4044,  6, }, // Asia/Baghdad
    {  660, 5534,   72942,  8, }, // Asia/Bahrain
    {  660, 3778,   57088,  8, }, // Asia/Bishkek
    {  660, 4540,    4083,  8, }, // Asia/Damascus
    {  660, 6159,   57096,  9, }, // Asia/Dushanbe
    {  660, 1259,  215324, 17, }, // Asia/Ho_Chi_Minh
    {  660, 2704,    4113,  8, }, // Asia/Hong_Kong
    {  660, 4771,   78045,  5, }, // Asia/Hovd
    {  660, 4316,   78050, 11, }, // Asia/Kamchatka
    {  660, 4239,   78061,  9, }, // Asia/Karachi
    {  660, 1185,     413,  9, }, // Asia/Kathmandu
    {  660,  992,   26732,  7, }, // Asia/Kolkata
    {  660, 4197,    4170, 11, }, // Asia/Krasnoyarsk
    {  660, 5976,  215341,  9, }, // Asia/Kuching
    {  660, 1211,     429,  5, }, // Asia/Macau
    {  660, 6920,   26750,  6, }, // Asia/Muscat
    {  660, 2554,    5620,  7, }, // Asia/Nicosia
    {  660, 8675,  215350, 12, }, // Asia/Novokuznetsk
    {  660, 4135,   78082, 11, }, // Asia/Novosibirsk
    {  660, 4214,   78093,  9, }, // Asia/Pyongyang
    {  660, 5559,  215362,  6, }, // Asia/Qatar
    {  660, 3584,    7669,  4, }, // Asia/Riyadh
    {  660, 4408,   57952,  8, }, // Asia/Sakhalin
    {  660, 1053,   57177,  9, }, // Asia/Shanghai
    {  660, 3300,  215368,  9, }, // Asia/Singapore
    {  660, 3263,  215377,  6, }, // Asia/Taipei
    {  660, 4781,   57196,  9, }, // Asia/Tashkent
    {  660, 3915,   78108,  6, }, // Asia/Tbilisi
    {  660, 2772,    7724,  7, }, // Asia/Tehran
    {  660, 2821,    4238,  5, }, // Asia/Tokyo
    {  660, 1159,  215383,  9, }, // Asia/Urumqi
    {  660, 6977,  215392,  9, }, // Asia/Vientiane
    {  660, 4710,    4243, 11, }, // Asia/Vladivostok
    {  660, 4828,    7755,  7, }, // Asia/Yakutsk
    {  660, 1235,  215401,  7, }, // Asia/Yangon
    {  660, 3883,    4262, 13, }, // Asia/Yekaterinburg
    {  660, 3747,  127820,  7, }, // Asia/Yerevan
    {  660, 3663,  215408,  8, }, // Atlantic/Azores
    {  660, 5671,  215416,  8, }, // Atlantic/Bermuda
    {  660, 6628,  215424,  8, }, // Atlantic/Canary
    {  660, 3727,  215432,  9, }, // Atlantic/Cape_Verde
    {  660, 1395,  215441,  7, }, // Atlantic/Faroe
    {  660, 7272,  215448, 12, }, // Atlantic/South_Georgia
    {  660, 6821,     453, 10, }, // Atlantic/St_Helena
    {  660, 6216,   33506,  6, }, // Europe/Andorra
    {  660, 3620,   78149,  9, }, // Europe/Astrakhan
    {  660, 2300,  215460,  6, }, // Europe/Athens
    {  660, 6336,   26883,  7, }, // Europe/Belgrade
    {  660, 2167,   22634,  7, }, // Europe/Brussels
    {  660, 3928,   26897,  8, }, // Europe/Bucharest
    {  660, 6289,  215466, 11, }, // Europe/Copenhagen
    {  660, 6307,  215477, 10, }, // Europe/Gibraltar
    {  660, 8804,  215487,  9, }, // Europe/Isle_of_Man
    {  660, 2527,   78163,  4, }, // Europe/Kyiv
    {  660, 3213,   33596,  7, }, // Europe/Lisbon
    {  660, 6365,  215496, 11, }, // Europe/Luxembourg
    {  660, 6410,    5926,  6, }, // Europe/Monaco
    {  660, 3537,   57353,  6, }, // Europe/Moscow
    {  660, 4284,   63726,  6, }, // Europe/Paris
    {  660, 6275,    4415,  5, }, // Europe/Prague
    {  660, 8837,  215507,  5, }, // Europe/Riga
    {  660, 6324,   63736,  4, }, // Europe/Rome
    {  660, 4451,   78167,  7, }, // Europe/Saratov
    {  660, 6202,   21095,  6, }, // Europe/Tirane
    {  660, 8882,   78174,  9, }, // Europe/Ulyanovsk
    {  660, 6567,  215512, 13, }, // Europe/Vatican
    {  660, 6231,  215525,  4, }, // Europe/Vienna
    {  660, 4727,    4443,  9, }, // Europe/Volgograd
    {  660, 3190,   57396,  8, }, // Europe/Warsaw
    {  660, 6540,    4459,  6, }, // Europe/Zurich
    {  660, 6046,  215529,  9, }, // Indian/Christmas
    {  660, 6063,  215538, 13, }, // Indian/Cocos
    {  660, 7050,  215551, 10, }, // Indian/Maldives
    {  660, 7211,     467,  7, }, // Indian/Reunion
    {  660, 3157,     474,  5, }, // Pacific/Chuuk
    {  660, 2237,  215561, 13, }, // Pacific/Easter
    {  660, 3902,   78224,  7, }, // Pacific/Fiji
    {  660, 4168,  215574, 13, }, // Pacific/Norfolk
    {  660, 3096,     479,  7, }, // Pacific/Pohnpei
    {  661,   14,       0,  6, }, // Africa/Asmara Rajasthani/Devanagari/India
    {  661, 4435,       6,  8, }, // Africa/Sao_Tome
    {  661, 4252,      14,  8, }, // America/Asuncion
    {  661,  237,      22,  8, }, // America/Atikokan
    {  661, 7623,      30, 17, }, // America/Bahia_Banderas
    {  661, 3868,      47,  6, }, // America/Cancun
    {  661, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  661, 5723,      66,  7, }, // America/Curacao
    {  661,  481,      73, 13, }, // America/Indiana/Knox
    {  661, 7961,      86, 16, }, // America/Indiana/Marengo
    {  661, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  661, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  661, 8038,     139, 14, }, // America/Indiana/Vevay
    {  661, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  661, 8086,     171, 16, }, // America/Indiana/Winamac
    {  661, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  661, 5932,     207, 22, }, // America/Lower_Princes
    {  661, 8220,     229,  6, }, // America/Merida
    {  661, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  661, 8329,     255, 20, }, // America/North_Dakota/Center
    {  661, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  661,  393,     298,  4, }, // America/Nuuk
    {  661, 6899,     302, 16, }, // America/Scoresbysund
    {  661, 8508,     318, 14, }, // America/St_Barthelemy
    {  661, 2061,     332, 10, }, // America/St_Johns
    {  661, 5863,     342,  9, }, // America/St_Kitts
    {  661, 5880,     351,  9, }, // America/St_Lucia
    {  661,  855,     360, 10, }, // America/St_Thomas
    {  661, 5913,     370, 11, }, // America/St_Vincent
    {  661, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  661, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  661, 1185,     413,  9, }, // Asia/Kathmandu
    {  661,  992,     422,  7, }, // Asia/Kolkata
    {  661, 1211,     429,  5, }, // Asia/Macau
    {  661, 1235,     442,  6, }, // Asia/Yangon
    {  661, 1395,     448,  5, }, // Atlantic/Faroe
    {  661, 6821,     453, 10, }, // Atlantic/St_Helena
    {  661, 2527,     463,  4, }, // Europe/Kyiv
    {  661, 7211,     467,  7, }, // Indian/Reunion
    {  661, 3157,     474,  5, }, // Pacific/Chuuk
    {  661, 3096,     479,  7, }, // Pacific/Pohnpei
    {  662,   14,       0,  6, }, // Africa/Asmara Moksha/Cyrillic/Russia
    {  662, 4435,       6,  8, }, // Africa/Sao_Tome
    {  662, 4252,      14,  8, }, // America/Asuncion
    {  662,  237,      22,  8, }, // America/Atikokan
    {  662, 7623,      30, 17, }, // America/Bahia_Banderas
    {  662, 3868,      47,  6, }, // America/Cancun
    {  662, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  662, 5723,      66,  7, }, // America/Curacao
    {  662,  481,      73, 13, }, // America/Indiana/Knox
    {  662, 7961,      86, 16, }, // America/Indiana/Marengo
    {  662, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  662, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  662, 8038,     139, 14, }, // America/Indiana/Vevay
    {  662, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  662, 8086,     171, 16, }, // America/Indiana/Winamac
    {  662, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  662, 5932,     207, 22, }, // America/Lower_Princes
    {  662, 8220,     229,  6, }, // America/Merida
    {  662, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  662, 8329,     255, 20, }, // America/North_Dakota/Center
    {  662, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  662,  393,     298,  4, }, // America/Nuuk
    {  662, 6899,     302, 16, }, // America/Scoresbysund
    {  662, 8508,     318, 14, }, // America/St_Barthelemy
    {  662, 2061,     332, 10, }, // America/St_Johns
    {  662, 5863,     342,  9, }, // America/St_Kitts
    {  662, 5880,     351,  9, }, // America/St_Lucia
    {  662,  855,     360, 10, }, // America/St_Thomas
    {  662, 5913,     370, 11, }, // America/St_Vincent
    {  662, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  662, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  662, 1185,     413,  9, }, // Asia/Kathmandu
    {  662,  992,     422,  7, }, // Asia/Kolkata
    {  662, 1211,     429,  5, }, // Asia/Macau
    {  662, 1235,     442,  6, }, // Asia/Yangon
    {  662, 1395,     448,  5, }, // Atlantic/Faroe
    {  662, 6821,     453, 10, }, // Atlantic/St_Helena
    {  662, 2527,     463,  4, }, // Europe/Kyiv
    {  662, 7211,     467,  7, }, // Indian/Reunion
    {  662, 3157,     474,  5, }, // Pacific/Chuuk
    {  662, 3096,     479,  7, }, // Pacific/Pohnpei
    {  663,   14,       0,  6, }, // Africa/Asmara Toki Pona/Latin/world
    {  663, 4435,       6,  8, }, // Africa/Sao_Tome
    {  663, 4252,      14,  8, }, // America/Asuncion
    {  663,  237,      22,  8, }, // America/Atikokan
    {  663, 7623,      30, 17, }, // America/Bahia_Banderas
    {  663, 3868,      47,  6, }, // America/Cancun
    {  663, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  663, 5723,      66,  7, }, // America/Curacao
    {  663,  481,      73, 13, }, // America/Indiana/Knox
    {  663, 7961,      86, 16, }, // America/Indiana/Marengo
    {  663, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  663, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  663, 8038,     139, 14, }, // America/Indiana/Vevay
    {  663, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  663, 8086,     171, 16, }, // America/Indiana/Winamac
    {  663, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  663, 5932,     207, 22, }, // America/Lower_Princes
    {  663, 8220,     229,  6, }, // America/Merida
    {  663, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  663, 8329,     255, 20, }, // America/North_Dakota/Center
    {  663, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  663,  393,     298,  4, }, // America/Nuuk
    {  663, 6899,     302, 16, }, // America/Scoresbysund
    {  663, 8508,     318, 14, }, // America/St_Barthelemy
    {  663, 2061,     332, 10, }, // America/St_Johns
    {  663, 5863,     342,  9, }, // America/St_Kitts
    {  663, 5880,     351,  9, }, // America/St_Lucia
    {  663,  855,     360, 10, }, // America/St_Thomas
    {  663, 5913,     370, 11, }, // America/St_Vincent
    {  663, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  663, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  663, 1185,     413,  9, }, // Asia/Kathmandu
    {  663,  992,     422,  7, }, // Asia/Kolkata
    {  663, 1211,     429,  5, }, // Asia/Macau
    {  663, 1235,     442,  6, }, // Asia/Yangon
    {  663, 1395,     448,  5, }, // Atlantic/Faroe
    {  663, 6821,     453, 10, }, // Atlantic/St_Helena
    {  663, 2527,     463,  4, }, // Europe/Kyiv
    {  663, 7211,     467,  7, }, // Indian/Reunion
    {  663, 3157,     474,  5, }, // Pacific/Chuuk
    {  663, 3096,     479,  7, }, // Pacific/Pohnpei
    {  664,   14,       0,  6, }, // Africa/Asmara Pijin/Latin/Solomon Islands
    {  664, 4435,       6,  8, }, // Africa/Sao_Tome
    {  664, 4252,      14,  8, }, // America/Asuncion
    {  664,  237,      22,  8, }, // America/Atikokan
    {  664, 7623,      30, 17, }, // America/Bahia_Banderas
    {  664, 3868,      47,  6, }, // America/Cancun
    {  664, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  664, 5723,      66,  7, }, // America/Curacao
    {  664,  481,      73, 13, }, // America/Indiana/Knox
    {  664, 7961,      86, 16, }, // America/Indiana/Marengo
    {  664, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  664, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  664, 8038,     139, 14, }, // America/Indiana/Vevay
    {  664, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  664, 8086,     171, 16, }, // America/Indiana/Winamac
    {  664, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  664, 5932,     207, 22, }, // America/Lower_Princes
    {  664, 8220,     229,  6, }, // America/Merida
    {  664, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  664, 8329,     255, 20, }, // America/North_Dakota/Center
    {  664, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  664,  393,     298,  4, }, // America/Nuuk
    {  664, 6899,     302, 16, }, // America/Scoresbysund
    {  664, 8508,     318, 14, }, // America/St_Barthelemy
    {  664, 2061,     332, 10, }, // America/St_Johns
    {  664, 5863,     342,  9, }, // America/St_Kitts
    {  664, 5880,     351,  9, }, // America/St_Lucia
    {  664,  855,     360, 10, }, // America/St_Thomas
    {  664, 5913,     370, 11, }, // America/St_Vincent
    {  664, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  664, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  664, 1185,     413,  9, }, // Asia/Kathmandu
    {  664,  992,     422,  7, }, // Asia/Kolkata
    {  664, 1211,     429,  5, }, // Asia/Macau
    {  664, 1235,     442,  6, }, // Asia/Yangon
    {  664, 1395,     448,  5, }, // Atlantic/Faroe
    {  664, 6821,     453, 10, }, // Atlantic/St_Helena
    {  664, 2527,     463,  4, }, // Europe/Kyiv
    {  664, 7211,     467,  7, }, // Indian/Reunion
    {  664, 3157,     474,  5, }, // Pacific/Chuuk
    {  664, 3096,     479,  7, }, // Pacific/Pohnpei
    {  665,   14,       0,  6, }, // Africa/Asmara Obolo/Latin/Nigeria
    {  665, 4435,       6,  8, }, // Africa/Sao_Tome
    {  665, 4252,      14,  8, }, // America/Asuncion
    {  665,  237,      22,  8, }, // America/Atikokan
    {  665, 7623,      30, 17, }, // America/Bahia_Banderas
    {  665, 3868,      47,  6, }, // America/Cancun
    {  665, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  665, 5723,      66,  7, }, // America/Curacao
    {  665,  481,      73, 13, }, // America/Indiana/Knox
    {  665, 7961,      86, 16, }, // America/Indiana/Marengo
    {  665, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  665, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  665, 8038,     139, 14, }, // America/Indiana/Vevay
    {  665, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  665, 8086,     171, 16, }, // America/Indiana/Winamac
    {  665, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  665, 5932,     207, 22, }, // America/Lower_Princes
    {  665, 8220,     229,  6, }, // America/Merida
    {  665, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  665, 8329,     255, 20, }, // America/North_Dakota/Center
    {  665, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  665,  393,     298,  4, }, // America/Nuuk
    {  665, 6899,     302, 16, }, // America/Scoresbysund
    {  665, 8508,     318, 14, }, // America/St_Barthelemy
    {  665, 2061,     332, 10, }, // America/St_Johns
    {  665, 5863,     342,  9, }, // America/St_Kitts
    {  665, 5880,     351,  9, }, // America/St_Lucia
    {  665,  855,     360, 10, }, // America/St_Thomas
    {  665, 5913,     370, 11, }, // America/St_Vincent
    {  665, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  665, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  665, 1185,     413,  9, }, // Asia/Kathmandu
    {  665,  992,     422,  7, }, // Asia/Kolkata
    {  665, 1211,     429,  5, }, // Asia/Macau
    {  665, 1235,     442,  6, }, // Asia/Yangon
    {  665, 1395,     448,  5, }, // Atlantic/Faroe
    {  665, 6821,     453, 10, }, // Atlantic/St_Helena
    {  665, 2527,     463,  4, }, // Europe/Kyiv
    {  665, 7211,     467,  7, }, // Indian/Reunion
    {  665, 3157,     474,  5, }, // Pacific/Chuuk
    {  665, 3096,     479,  7, }, // Pacific/Pohnpei
    {  666,   14,       0,  6, }, // Africa/Asmara Baluchi/Arabic/Pakistan
    {  666, 4435,       6,  8, }, // Africa/Sao_Tome
    {  666, 4252,      14,  8, }, // America/Asuncion
    {  666,  237,      22,  8, }, // America/Atikokan
    {  666, 7623,      30, 17, }, // America/Bahia_Banderas
    {  666, 3868,      47,  6, }, // America/Cancun
    {  666, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  666, 5723,      66,  7, }, // America/Curacao
    {  666,  481,      73, 13, }, // America/Indiana/Knox
    {  666, 7961,      86, 16, }, // America/Indiana/Marengo
    {  666, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  666, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  666, 8038,     139, 14, }, // America/Indiana/Vevay
    {  666, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  666, 8086,     171, 16, }, // America/Indiana/Winamac
    {  666, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  666, 5932,     207, 22, }, // America/Lower_Princes
    {  666, 8220,     229,  6, }, // America/Merida
    {  666, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  666, 8329,     255, 20, }, // America/North_Dakota/Center
    {  666, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  666,  393,     298,  4, }, // America/Nuuk
    {  666, 6899,     302, 16, }, // America/Scoresbysund
    {  666, 8508,     318, 14, }, // America/St_Barthelemy
    {  666, 2061,     332, 10, }, // America/St_Johns
    {  666, 5863,     342,  9, }, // America/St_Kitts
    {  666, 5880,     351,  9, }, // America/St_Lucia
    {  666,  855,     360, 10, }, // America/St_Thomas
    {  666, 5913,     370, 11, }, // America/St_Vincent
    {  666, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  666, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  666, 1185,     413,  9, }, // Asia/Kathmandu
    {  666,  992,     422,  7, }, // Asia/Kolkata
    {  666, 1211,     429,  5, }, // Asia/Macau
    {  666, 1235,     442,  6, }, // Asia/Yangon
    {  666, 1395,     448,  5, }, // Atlantic/Faroe
    {  666, 6821,     453, 10, }, // Atlantic/St_Helena
    {  666, 2527,     463,  4, }, // Europe/Kyiv
    {  666, 7211,     467,  7, }, // Indian/Reunion
    {  666, 3157,     474,  5, }, // Pacific/Chuuk
    {  666, 3096,     479,  7, }, // Pacific/Pohnpei
    {  667, 6788,  215587,  7, }, // Africa/Abidjan Baluchi/Latin/Pakistan
    {  667, 6760,  215594,  5, }, // Africa/Accra
    {  667, 5059,  215599, 11, }, // Africa/Addis_Ababa
    {  667,   14,  215610,  6, }, // Africa/Asmara
    {  667,   44,  215616,  6, }, // Africa/Bamako
    {  667, 4957,  215622,  9, }, // Africa/Blantyre
    {  667, 5287,  215631, 11, }, // Africa/Brazzaville
    {  667, 4922,  215642,  9, }, // Africa/Bujumbura
    {  667, 2320,  215651,  5, }, // Africa/Cairo
    {  667, 4117,  215656, 10, }, // Africa/Casablanca
    {  667, 7449,  215666,  5, }, // Africa/Ceuta
    {  667, 6773,    6249,  7, }, // Africa/Conakry
    {  667, 5130,  215671, 12, }, // Africa/Dar_es_Salaam
    {  667, 5043,  215683,  7, }, // Africa/Djibouti
    {  667, 5243,  215690,  6, }, // Africa/Douala
    {  667, 5166,  215696,  7, }, // Africa/El_Aaiun
    {  667, 6853,  215703,  9, }, // Africa/Freetown
    {  667, 4906,  215712,  8, }, // Africa/Gaborone
    {  667, 5015,  215720,  6, }, // Africa/Harare
    {  667, 4479,  215726, 12, }, // Africa/Johannesburg
    {  667, 4499,  215738,  4, }, // Africa/Juba
    {  667, 5151,  140433,  7, }, // Africa/Kampala
    {  667, 4524,   26372,  7, }, // Africa/Khartoum
    {  667, 5306,  215742,  8, }, // Africa/Kinshasa
    {  667, 4744,  215750,  5, }, // Africa/Lagos
    {  667, 5336,  215755, 10, }, // Africa/Libreville
    {  667, 6869,  215765,  4, }, // Africa/Lome
    {  667, 5211,  215769,  6, }, // Africa/Luanda
    {  667, 4939,  140478, 10, }, // Africa/Lubumbashi
    {  667, 5001,  215775,  6, }, // Africa/Lusaka
    {  667, 5322,  215781,  6, }, // Africa/Malabo
    {  667, 4987,  215787,  6, }, // Africa/Maputo
    {  667, 5197,  215793,  6, }, // Africa/Maseru
    {  667, 5182,  215799,  7, }, // Africa/Mbabane
    {  667, 5113,  215806,  9, }, // Africa/Mogadishu
    {  667, 7034,  215815,  8, }, // Africa/Monrovia
    {  667, 3853,  215823,  6, }, // Africa/Nairobi
    {  667, 5271,  215829,  8, }, // Africa/Ndjamena
    {  667, 5354,  215837,  6, }, // Africa/Niamey
    {  667, 6803,   73450, 10, }, // Africa/Nouakchott
    {  667, 5225,  215843, 10, }, // Africa/Porto-Novo
    {  667, 4435,  215853,  8, }, // Africa/Sao_Tome
    {  667, 2866,  215861,  7, }, // Africa/Tripoli
    {  667, 6554,   74069,  5, }, // Africa/Tunis
    {  667, 4152,  215868,  7, }, // Africa/Windhoek
    {  667, 3346,  215875,  9, }, // America/Anchorage
    {  667, 5607,  215884,  8, }, // America/Anguilla
    {  667, 5624,  215892,  7, }, // America/Antigua
    {  667,   91,  215899, 10, }, // America/Argentina/Catamarca
    {  667,  270,  215909,  7, }, // America/Argentina/Cordoba
    {  667,  441,  215916,  5, }, // America/Argentina/Jujuy
    {  667,  565,  215921,  7, }, // America/Argentina/Mendoza
    {  667, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {  667, 7597,  215928,  7, }, // America/Argentina/Ushuaia
    {  667, 5640,  215935,  5, }, // America/Aruba
    {  667, 4252,  215940,  9, }, // America/Asuncion
    {  667,  237,  215949,  8, }, // America/Atikokan
    {  667, 7623,  215957, 17, }, // America/Bahia_Banderas
    {  667, 7660,  215974, 13, }, // America/Blanc-Sablon
    {  667, 7681,  215987,  9, }, // America/Boa_Vista
    {  667, 7713,  215996, 11, }, // America/Cambridge_Bay
    {  667, 7735,  216007, 13, }, // America/Campo_Grande
    {  667, 3868,  216020,  8, }, // America/Cancun
    {  667, 4694,  216028,  8, }, // America/Caracas
    {  667, 4345,  216036,  7, }, // America/Cayenne
    {  667, 5496,   21483,  6, }, // America/Cayman
    {  667, 2260,  216043,  7, }, // America/Chicago
    {  667, 7756,  216050,  9, }, // America/Chihuahua
    {  667, 7774,  216059, 14, }, // America/Ciudad_Juarez
    {  667, 5422,  216073, 10, }, // America/Costa_Rica
    {  667, 7796,  216083,  7, }, // America/Creston
    {  667, 3791,  216090,  6, }, // America/Cuiaba
    {  667, 5723,  216096,  9, }, // America/Curacao
    {  667, 7812,  216105, 12, }, // America/Danmarkshavn
    {  667, 7848,  216117, 11, }, // America/Dawson_Creek
    {  667,  805,  216128,  6, }, // America/Denver
    {  667, 5739,  216134,  8, }, // America/Dominica
    {  667, 7869,  216142,  7, }, // America/Eirunepe
    {  667, 5441,  216149, 11, }, // America/El_Salvador
    {  667, 7906,  216160,  9, }, // America/Fortaleza
    {  667, 7924,  216169,  9, }, // America/Glace_Bay
    {  667, 5770,  216178,  7, }, // America/Grenada
    {  667, 5786,  216185,  8, }, // America/Guadeloupe
    {  667, 3760,  216193,  9, }, // America/Guatemala
    {  667, 4841,  216202,  8, }, // America/Guayaquil
    {  667, 6932,  216210,  6, }, // America/Guyana
    {  667, 1939,    6787,  8, }, // America/Halifax
    {  667, 2281,  216216,  6, }, // America/Havana
    {  667,  348,  216222, 12, }, // America/Indiana/Indianapolis
    {  667,  481,  216234, 14, }, // America/Indiana/Knox
    {  667, 7961,  216248, 15, }, // America/Indiana/Marengo
    {  667, 7985,  216263, 19, }, // America/Indiana/Petersburg
    {  667, 8012,  216282, 19, }, // America/Indiana/Tell_City
    {  667, 8038,  216301, 14, }, // America/Indiana/Vevay
    {  667, 8060,  216315, 18, }, // America/Indiana/Vincennes
    {  667, 8086,  216333, 16, }, // America/Indiana/Winamac
    {  667, 8110,  195828,  6, }, // America/Inuvik
    {  667,  660,  216349,  7, }, // America/Iqaluit
    {  667, 2799,  216356,  7, }, // America/Jamaica
    {  667, 5380,  216363,  5, }, // America/Juneau
    {  667,  521,  216368, 10, }, // America/Kentucky/Louisville
    {  667, 8125,  216378, 20, }, // America/Kentucky/Monticello
    {  667, 4376,  216398,  6, }, // America/La_Paz
    {  667, 7169,  216404,  4, }, // America/Lima
    {  667, 3239,  216408, 11, }, // America/Los_Angeles
    {  667, 8153,  216419,  6, }, // America/Maceio
    {  667, 5805,  213260,  7, }, // America/Martinique
    {  667, 8202,    7000,  8, }, // America/Menominee
    {  667, 2949,  216425, 13, }, // America/Mexico_City
    {  667, 4391,  216438,  8, }, // America/Miquelon
    {  667, 8254,    7038,  7, }, // America/Moncton
    {  667, 8270,  216446,  9, }, // America/Monterrey
    {  667, 4098,  216455, 10, }, // America/Montevideo
    {  667, 2379,  216465,  8, }, // America/New_York
    {  667, 1850,  216473,  7, }, // America/Noronha
    {  667, 8301,  216480, 19, }, // America/North_Dakota/Beulah
    {  667, 8329,  216499, 20, }, // America/North_Dakota/Center
    {  667, 8357,  216519, 22, }, // America/North_Dakota/New_Salem
    {  667, 2356,  216541,  6, }, // America/Panama
    {  667, 6173,  216547, 10, }, // America/Paramaribo
    {  667, 2973,  216557,  7, }, // America/Phoenix
    {  667, 3945,  216564, 13, }, // America/Port-au-Prince
    {  667, 5954,  216577, 10, }, // America/Port_of_Spain
    {  667, 8404,  216587, 11, }, // America/Porto_Velho
    {  667, 5843,  216598, 11, }, // America/Puerto_Rico
    {  667, 8445,  216609,  7, }, // America/Recife
    {  667, 8460,  216616,  7, }, // America/Resolute
    {  667,  695,  216623, 10, }, // America/Rio_Branco
    {  667, 2201,  216633,  8, }, // America/Santiago
    {  667, 6111,  216641, 13, }, // America/Santo_Domingo
    {  667, 1878,  216654,  9, }, // America/Sao_Paulo
    {  667, 6899,  216663, 14, }, // America/Scoresbysund
    {  667, 5880,  216677,  9, }, // America/St_Lucia
    {  667,  855,  216686,  9, }, // America/St_Thomas
    {  667, 5913,  216695, 11, }, // America/St_Vincent
    {  667, 8530,  216706, 13, }, // America/Swift_Current
    {  667, 5461,  216719, 12, }, // America/Tegucigalpa
    {  667, 5756,  216731,  4, }, // America/Thule
    {  667,  313,  216735,  7, }, // America/Tijuana
    {  667,  608,  216742,  7, }, // America/Toronto
    {  667, 5688,  216749,  7, }, // America/Tortola
    {  667, 2093,  216756,  8, }, // America/Vancouver
    {  667, 6001,  216764,  4, }, // Antarctica/Casey
    {  667, 6094,  216768,  5, }, // Antarctica/Davis
    {  667, 6133,  216773, 13, }, // Antarctica/DumontDUrville
    {  667, 8568,  216786,  7, }, // Antarctica/Macquarie
    {  667, 7295,  216793,  5, }, // Antarctica/Syowa
    {  667, 7374,    4029,  6, }, // Antarctica/Vostok
    {  667, 5570,  201735,  4, }, // Asia/Aden
    {  667, 5395,    7476,  6, }, // Asia/Almaty
    {  667, 3968,   57792,  5, }, // Asia/Amman
    {  667, 5511,    7482,  6, }, // Asia/Anadyr
    {  667, 5523,    5491,  5, }, // Asia/Aqtau
    {  667, 5368,  216798,  6, }, // Asia/Aqtobe
    {  667, 8624,    7494,  6, }, // Asia/Atyrau
    {  667, 3607,   57805,  6, }, // Asia/Baghdad
    {  667, 5534,  216804,  6, }, // Asia/Bahrain
    {  667, 3653,  141946,  4, }, // Asia/Baku
    {  667, 4466,  216810,  7, }, // Asia/Bangkok
    {  667, 4086,  216817,  5, }, // Asia/Beirut
    {  667, 5989,  216822,  6, }, // Asia/Brunei
    {  667, 4511,  216828,  7, }, // Asia/Colombo
    {  667, 4540,  216835,  8, }, // Asia/Damascus
    {  667, 1093,   57831,  5, }, // Asia/Dhaka
    {  667, 6192,  216843,  5, }, // Asia/Dili
    {  667, 3596,  216848,  5, }, // Asia/Dubai
    {  667, 6159,  216853,  9, }, // Asia/Dushanbe
    {  667, 8636,  216862,  9, }, // Asia/Famagusta
    {  667, 4795,   20653,  6, }, // Asia/Hebron
    {  667, 1259,  216871, 11, }, // Asia/Ho_Chi_Minh
    {  667, 2704,  216882,  9, }, // Asia/Hong_Kong
    {  667, 4771,  216891,  4, }, // Asia/Hovd
    {  667, 7006,  216895,  7, }, // Asia/Jakarta
    {  667, 6992,  201812,  6, }, // Asia/Jayapura
    {  667, 1290,  216902,  8, }, // Asia/Jerusalem
    {  667, 3560,  216910,  5, }, // Asia/Kabul
    {  667, 4239,  142094,  7, }, // Asia/Karachi
    {  667, 8661,   26724,  8, }, // Asia/Khandyga
    {  667,  992,   57896,  7, }, // Asia/Kolkata
    {  667, 4859,  216915, 12, }, // Asia/Kuala_Lumpur
    {  667, 5547,  216927,  5, }, // Asia/Kuwait
    {  667, 1211,  216932,  5, }, // Asia/Macau
    {  667, 6920,   26750,  6, }, // Asia/Muscat
    {  667, 2554,  216937,  8, }, // Asia/Nicosia
    {  667, 8675,   78070, 12, }, // Asia/Novokuznetsk
    {  667, 4135,   78082, 11, }, // Asia/Novosibirsk
    {  667, 7145,  216945,  4, }, // Asia/Oral
    {  667, 4214,  216949,  9, }, // Asia/Pyongyang
    {  667, 5559,  216958,  5, }, // Asia/Qatar
    {  667, 8708,  216963,  8, }, // Asia/Qostanay
    {  667, 4269,  216971,  9, }, // Asia/Qyzylorda
    {  667, 3584,  216980,  4, }, // Asia/Riyadh
    {  667, 3279,  216984,  4, }, // Asia/Seoul
    {  667, 1053,  161891,  7, }, // Asia/Shanghai
    {  667, 3300,  216988,  8, }, // Asia/Singapore
    {  667, 4297,  216996, 13, }, // Asia/Srednekolymsk
    {  667, 3263,  217009,  5, }, // Asia/Taipei
    {  667, 4781,  217014,  8, }, // Asia/Tashkent
    {  667, 3915,  217022,  8, }, // Asia/Tbilisi
    {  667, 2772,  217030,  6, }, // Asia/Tehran
    {  667, 1317,    7731,  6, }, // Asia/Thimphu
    {  667, 2821,  217036,  5, }, // Asia/Tokyo
    {  667, 1021,   58006,  9, }, // Asia/Ulaanbaatar
    {  667, 1159,   73928,  6, }, // Asia/Urumqi
    {  667, 6977,  217041,  9, }, // Asia/Vientiane
    {  667, 4710,  217050, 11, }, // Asia/Vladivostok
    {  667, 3747,  217061,  7, }, // Asia/Yerevan
    {  667, 3663,  151753,  6, }, // Atlantic/Azores
    {  667, 6628,  217068,  7, }, // Atlantic/Canary
    {  667, 3727,  217075,  9, }, // Atlantic/Cape_Verde
    {  667, 2748,  217084,  9, }, // Atlantic/Reykjavik
    {  667, 7272,  217093, 14, }, // Atlantic/South_Georgia
    {  667, 6821,  217107, 10, }, // Atlantic/St_Helena
    {  667, 6644,  217117,  6, }, // Atlantic/Stanley
    {  667, 1670,  217123,  6, }, // Australia/Adelaide
    {  667, 3637,  217129,  5, }, // Australia/Eucla
    {  667, 1727,  217134,  7, }, // Australia/Melbourne
    {  667, 1762,   22603,  4, }, // Australia/Perth
    {  667, 1463,  217141,  6, }, // Australia/Sydney
    {  667, 2300,  217147,  4, }, // Europe/Athens
    {  667, 6336,   91821,  7, }, // Europe/Belgrade
    {  667, 6488,  217151, 10, }, // Europe/Bratislava
    {  667, 3928,  217161,  9, }, // Europe/Bucharest
    {  667, 3806,  217170,  8, }, // Europe/Budapest
    {  667, 6289,  217178, 10, }, // Europe/Copenhagen
    {  667, 8788,  217188,  8, }, // Europe/Guernsey
    {  667, 1130,  217196,  8, }, // Europe/Istanbul
    {  667, 8823,  217204,  6, }, // Europe/Jersey
    {  667, 8929,  217210,  5, }, // Europe/Kirov
    {  667, 6506,  217215,  9, }, // Europe/Ljubljana
    {  667, 2501,  176970,  6, }, // Europe/London
    {  667, 6365,  217224, 10, }, // Europe/Luxembourg
    {  667, 6410,  217234,  6, }, // Europe/Monaco
    {  667, 3537,  217240,  5, }, // Europe/Moscow
    {  667, 6458,  217245,  4, }, // Europe/Oslo
    {  667, 4284,  217249,  6, }, // Europe/Paris
    {  667, 6424,  217255,  9, }, // Europe/Podgorica
    {  667, 6275,   91926,  4, }, // Europe/Prague
    {  667, 6324,   99180,  3, }, // Europe/Rome
    {  667, 6470,  217264, 10, }, // Europe/San_Marino
    {  667, 6245,  217274,  8, }, // Europe/Sarajevo
    {  667, 4451,  157640,  7, }, // Europe/Saratov
    {  667, 8849,  217282, 10, }, // Europe/Simferopol
    {  667, 6383,  217292,  5, }, // Europe/Skopje
    {  667, 6599,  217297,  5, }, // Europe/Sofia
    {  667, 6523,  217302, 10, }, // Europe/Stockholm
    {  667, 6202,  217312,  6, }, // Europe/Tirane
    {  667, 8882,  217318,  9, }, // Europe/Ulyanovsk
    {  667, 6352,  196745,  5, }, // Europe/Vaduz
    {  667, 6567,  196750,  7, }, // Europe/Vatican
    {  667, 6231,  217327,  6, }, // Europe/Vienna
    {  667, 8899,  217333,  7, }, // Europe/Vilnius
    {  667, 4727,    4443,  9, }, // Europe/Volgograd
    {  667, 3190,  217340,  5, }, // Europe/Warsaw
    {  667, 6540,  217345,  7, }, // Europe/Zurich
    {  667, 5078,  217352, 12, }, // Indian/Antananarivo
    {  667, 6947,  217364,  6, }, // Indian/Chagos
    {  667, 6046,  217370,  7, }, // Indian/Christmas
    {  667, 6063,  217377,  5, }, // Indian/Cocos
    {  667, 5029,  217382,  6, }, // Indian/Comoro
    {  667, 6661,  217388,  9, }, // Indian/Kerguelen
    {  667, 7260,   26983,  4, }, // Indian/Mahe
    {  667, 7050,  217397,  6, }, // Indian/Maldives
    {  667, 4069,  217403,  8, }, // Indian/Mauritius
    {  667, 5098,  217411,  7, }, // Indian/Mayotte
    {  667, 7211,  217418,  8, }, // Indian/Reunion
    {  667, 4422,  217426,  4, }, // Pacific/Apia
    {  667,  932,  217430,  7, }, // Pacific/Auckland
    {  667, 3706,  217437, 11, }, // Pacific/Bougainville
    {  667, 2237,  217448,  6, }, // Pacific/Easter
    {  667, 7360,  217454,  5, }, // Pacific/Efate
    {  667, 7327,  217459,  7, }, // Pacific/Fakaofo
    {  667, 3822,    8242, 11, }, // Pacific/Guadalcanal
    {  667, 7019,  217466,  6, }, // Pacific/Kosrae
    {  667, 4051,  217472,  8, }, // Pacific/Marquesas
    {  667, 4168,  217480,  7, }, // Pacific/Norfolk
    {  667, 7194,  217487,  8, }, // Pacific/Pitcairn
    {  668, 5059,    3784, 11, }, // Africa/Addis_Ababa Ligurian/Latin/Italy
    {  668, 7420,  217495,  4, }, // Africa/Algiers
    {  668,   14,       0,  6, }, // Africa/Asmara
    {  668, 7449,  217499,  5, }, // Africa/Ceuta
    {  668, 6869,   20300,  4, }, // Africa/Lome
    {  668, 5113,   43978, 10, }, // Africa/Mogadishu
    {  668, 3853,  217504,  7, }, // Africa/Nairobi
    {  668, 5271,   43996,  9, }, // Africa/Ndjamena
    {  668, 4435,   58598,  8, }, // Africa/Sao_Tome
    {  668, 6554,  217511,  6, }, // Africa/Tunis
    {  668,  270,  217517,  7, }, // America/Argentina/Cordoba
    {  668, 4252,      14,  8, }, // America/Asuncion
    {  668,  237,      22,  8, }, // America/Atikokan
    {  668, 7623,      30, 17, }, // America/Bahia_Banderas
    {  668, 4361,   44029,  6, }, // America/Bogota
    {  668, 3868,      47,  6, }, // America/Cancun
    {  668, 4345,  217524,  6, }, // America/Cayenne
    {  668, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  668, 5422,  147635, 10, }, // America/Costa_Rica
    {  668, 5723,      66,  7, }, // America/Curacao
    {  668, 5770,   20401,  7, }, // America/Grenada
    {  668, 5786,  217530, 10, }, // America/Guadeloupe
    {  668,  481,      73, 13, }, // America/Indiana/Knox
    {  668, 7961,      86, 16, }, // America/Indiana/Marengo
    {  668, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  668, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  668, 8038,     139, 14, }, // America/Indiana/Vevay
    {  668, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  668, 8086,     171, 16, }, // America/Indiana/Winamac
    {  668, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  668, 5932,     207, 22, }, // America/Lower_Princes
    {  668, 5805,   20436,  9, }, // America/Martinique
    {  668, 8220,     229,  6, }, // America/Merida
    {  668, 2949,  217540, 16, }, // America/Mexico_City
    {  668, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  668, 8329,     255, 20, }, // America/North_Dakota/Center
    {  668, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  668,  393,     298,  4, }, // America/Nuuk
    {  668, 1878,  217556,  9, }, // America/Sao_Paulo
    {  668, 6899,     302, 16, }, // America/Scoresbysund
    {  668, 8508,  217565, 11, }, // America/St_Barthelemy
    {  668, 2061,     332, 10, }, // America/St_Johns
    {  668, 5863,  217576, 13, }, // America/St_Kitts
    {  668, 5880,  217589, 11, }, // America/St_Lucia
    {  668,  855,     360, 10, }, // America/St_Thomas
    {  668, 5913,  217600, 11, }, // America/St_Vincent
    {  668, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  668, 5534,    4050,  7, }, // Asia/Bahrain
    {  668, 4601,   93125,  4, }, // Asia/Chita
    {  668, 4540,   74309,  7, }, // Asia/Damascus
    {  668, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  668, 7006,   95138,  8, }, // Asia/Jakarta
    {  668, 1290,   95146, 11, }, // Asia/Jerusalem
    {  668, 1185,     413,  9, }, // Asia/Kathmandu
    {  668,  992,   35921,  8, }, // Asia/Kolkata
    {  668, 1211,     429,  5, }, // Asia/Macau
    {  668, 7245,   20768, 10, }, // Asia/Samarkand
    {  668, 3300,  217611,  8, }, // Asia/Singapore
    {  668, 1021,   26791, 10, }, // Asia/Ulaanbaatar
    {  668, 1235,     442,  6, }, // Asia/Yangon
    {  668, 3883,    7768, 12, }, // Asia/Yekaterinburg
    {  668, 3663,    7787,  5, }, // Atlantic/Azores
    {  668, 6628,  217619,  6, }, // Atlantic/Canary
    {  668, 3727,  217625, 11, }, // Atlantic/Cape_Verde
    {  668, 1395,   95170,  7, }, // Atlantic/Faroe
    {  668, 7272,  217636, 14, }, // Atlantic/South_Georgia
    {  668, 6821,   95192, 10, }, // Atlantic/St_Helena
    {  668, 6216,  147495,  7, }, // Europe/Andorra
    {  668, 2300,    5793,  5, }, // Europe/Athens
    {  668, 6336,  217650,  9, }, // Europe/Belgrade
    {  668, 2167,   57529,  9, }, // Europe/Brussels
    {  668, 2338,  217659,  6, }, // Europe/Dublin
    {  668, 6307,  217665,  7, }, // Europe/Gibraltar
    {  668, 8804,  217672, 11, }, // Europe/Isle_of_Man
    {  668, 2527,     463,  4, }, // Europe/Kyiv
    {  668, 3213,  217683,  7, }, // Europe/Lisbon
    {  668, 6506,  217690,  7, }, // Europe/Ljubljana
    {  668, 2501,   95241,  6, }, // Europe/London
    {  668, 6365,   74445, 10, }, // Europe/Luxembourg
    {  668, 6410,  217697,  6, }, // Europe/Monaco
    {  668, 3537,   95258,  5, }, // Europe/Moscow
    {  668, 6458,  217703,  4, }, // Europe/Oslo
    {  668, 4284,  167148,  7, }, // Europe/Paris
    {  668, 6275,   21067,  5, }, // Europe/Prague
    {  668, 6324,  217707,  5, }, // Europe/Rome
    {  668, 6470,   65739,  9, }, // Europe/San_Marino
    {  668, 6523,   95280,  9, }, // Europe/Stockholm
    {  668, 6202,  217712,  6, }, // Europe/Tirane
    {  668, 3190,   95307,  8, }, // Europe/Warsaw
    {  668, 6261,   95315,  8, }, // Europe/Zagreb
    {  668, 6540,   95323,  6, }, // Europe/Zurich
    {  668, 5029,  217718,  6, }, // Indian/Comoro
    {  668, 7050,    8193,  7, }, // Indian/Maldives
    {  668, 5098,  217724,  7, }, // Indian/Mayotte
    {  668, 7211,     467,  7, }, // Indian/Reunion
    {  668, 3157,     474,  5, }, // Pacific/Chuuk
    {  668, 2237,   44510,  6, }, // Pacific/Easter
    {  668, 3902,   95340,  4, }, // Pacific/Fiji
    {  668, 4051,  217731,  9, }, // Pacific/Marquesas
    {  668, 7098,    4510,  6, }, // Pacific/Noumea
    {  668, 3096,     479,  7, }, // Pacific/Pohnpei
    {  669,   14,       0,  6, }, // Africa/Asmara Rohingya/Hanifi/Myanmar
    {  669, 4435,       6,  8, }, // Africa/Sao_Tome
    {  669, 4252,      14,  8, }, // America/Asuncion
    {  669,  237,      22,  8, }, // America/Atikokan
    {  669, 7623,      30, 17, }, // America/Bahia_Banderas
    {  669, 3868,      47,  6, }, // America/Cancun
    {  669, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  669, 5723,      66,  7, }, // America/Curacao
    {  669,  481,      73, 13, }, // America/Indiana/Knox
    {  669, 7961,      86, 16, }, // America/Indiana/Marengo
    {  669, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  669, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  669, 8038,     139, 14, }, // America/Indiana/Vevay
    {  669, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  669, 8086,     171, 16, }, // America/Indiana/Winamac
    {  669, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  669, 5932,     207, 22, }, // America/Lower_Princes
    {  669, 8220,     229,  6, }, // America/Merida
    {  669, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  669, 8329,     255, 20, }, // America/North_Dakota/Center
    {  669, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  669,  393,     298,  4, }, // America/Nuuk
    {  669, 6899,     302, 16, }, // America/Scoresbysund
    {  669, 8508,     318, 14, }, // America/St_Barthelemy
    {  669, 2061,     332, 10, }, // America/St_Johns
    {  669, 5863,     342,  9, }, // America/St_Kitts
    {  669, 5880,     351,  9, }, // America/St_Lucia
    {  669,  855,     360, 10, }, // America/St_Thomas
    {  669, 5913,     370, 11, }, // America/St_Vincent
    {  669, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  669, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  669, 1185,     413,  9, }, // Asia/Kathmandu
    {  669,  992,     422,  7, }, // Asia/Kolkata
    {  669, 1211,     429,  5, }, // Asia/Macau
    {  669, 1235,     442,  6, }, // Asia/Yangon
    {  669, 1395,     448,  5, }, // Atlantic/Faroe
    {  669, 6821,     453, 10, }, // Atlantic/St_Helena
    {  669, 2527,     463,  4, }, // Europe/Kyiv
    {  669, 7211,     467,  7, }, // Indian/Reunion
    {  669, 3157,     474,  5, }, // Pacific/Chuuk
    {  669, 3096,     479,  7, }, // Pacific/Pohnpei
    {  671, 6788,  198768,  7, }, // Africa/Abidjan Torwali/Arabic/Pakistan
    {  671, 6760,  198775,  5, }, // Africa/Accra
    {  671, 5059,  102986, 10, }, // Africa/Addis_Ababa
    {  671, 7420,  198780,  7, }, // Africa/Algiers
    {  671,   14,  106626,  6, }, // Africa/Asmara
    {  671,   44,  103013,  5, }, // Africa/Bamako
    {  671, 5257,  198787,  6, }, // Africa/Bangui
    {  671, 6746,  198793,  5, }, // Africa/Banjul
    {  671, 7435,  103033,  5, }, // Africa/Bissau
    {  671, 4957,  198798,  8, }, // Africa/Blantyre
    {  671, 5287,  198806,  9, }, // Africa/Brazzaville
    {  671, 4922,  198815,  6, }, // Africa/Bujumbura
    {  671, 2320,  198821,  5, }, // Africa/Cairo
    {  671, 4117,  198826, 11, }, // Africa/Casablanca
    {  671, 7449,  198837,  5, }, // Africa/Ceuta
    {  671, 6773,  198842,  6, }, // Africa/Conakry
    {  671, 6840,  198848,  4, }, // Africa/Dakar
    {  671, 5130,   10499, 10, }, // Africa/Dar_es_Salaam
    {  671, 5043,  198852,  5, }, // Africa/Djibouti
    {  671, 5243,  198857,  5, }, // Africa/Douala
    {  671, 5166,  154508,  6, }, // Africa/El_Aaiun
    {  671, 6853,  198862,  8, }, // Africa/Freetown
    {  671, 4906,  198870,  5, }, // Africa/Gaborone
    {  671, 5015,  198875,  5, }, // Africa/Harare
    {  671, 4479,  103149,  9, }, // Africa/Johannesburg
    {  671, 4499,   10555,  4, }, // Africa/Juba
    {  671, 5151,  198880,  7, }, // Africa/Kampala
    {  671, 4524,   10568,  5, }, // Africa/Khartoum
    {  671, 4973,  198887,  5, }, // Africa/Kigali
    {  671, 5306,  198892,  6, }, // Africa/Kinshasa
    {  671, 4744,  106771,  5, }, // Africa/Lagos
    {  671, 5336,  198898,  9, }, // Africa/Libreville
    {  671, 6869,   10598,  3, }, // Africa/Lome
    {  671, 5211,  198907,  6, }, // Africa/Luanda
    {  671, 4939,  167857,  8, }, // Africa/Lubumbashi
    {  671, 5001,  198913,  7, }, // Africa/Lusaka
    {  671, 5322,  167871,  5, }, // Africa/Malabo
    {  671, 4987,  198920,  5, }, // Africa/Maputo
    {  671, 5197,  198925,  5, }, // Africa/Maseru
    {  671, 5182,  198930,  6, }, // Africa/Mbabane
    {  671, 5113,  154607,  8, }, // Africa/Mogadishu
    {  671, 7034,  154615,  8, }, // Africa/Monrovia
    {  671, 3853,  103257,  6, }, // Africa/Nairobi
    {  671, 5271,  198936,  9, }, // Africa/Ndjamena
    {  671, 5354,  154637,  5, }, // Africa/Niamey
    {  671, 6803,  198945,  7, }, // Africa/Nouakchott
    {  671, 6727,  198952,  9, }, // Africa/Ouagadougou
    {  671, 5225,  103293, 10, }, // Africa/Porto-Novo
    {  671, 4435,  198961,  7, }, // Africa/Sao_Tome
    {  671, 2866,  198968,  7, }, // Africa/Tripoli
    {  671, 6554,  198975,  5, }, // Africa/Tunis
    {  671, 4152,  198980,  6, }, // Africa/Windhoek
    {  671,  132,  152047,  4, }, // America/Adak
    {  671, 3346,  152051,  7, }, // America/Anchorage
    {  671, 5607,  198986,  7, }, // America/Anguilla
    {  671, 5624,  198993,  7, }, // America/Antigua
    {  671, 4554,  167968,  9, }, // America/Araguaina
    {  671,  166,  199000, 10, }, // America/Argentina/Buenos_Aires
    {  671,   91,  217740,  9, }, // America/Argentina/Catamarca
    {  671,  270,  199010,  7, }, // America/Argentina/Cordoba
    {  671,  441,  199017,  6, }, // America/Argentina/Jujuy
    {  671, 7462,  168012,  9, }, // America/Argentina/La_Rioja
    {  671,  565,  199023,  7, }, // America/Argentina/Mendoza
    {  671, 7489,  168028, 11, }, // America/Argentina/Rio_Gallegos
    {  671, 7520,  103430,  5, }, // America/Argentina/Salta
    {  671, 7544,  168044,  8, }, // America/Argentina/San_Juan
    {  671, 5580,  199030,  8, }, // America/Argentina/San_Luis
    {  671, 7571,  199038,  7, }, // America/Argentina/Tucuman
    {  671, 7597,  168068,  8, }, // America/Argentina/Ushuaia
    {  671, 5640,  154793,  5, }, // America/Aruba
    {  671, 4252,  199045,  7, }, // America/Asuncion
    {  671,  237,  199052,  7, }, // America/Atikokan
    {  671, 3679,  103504,  5, }, // America/Bahia
    {  671, 7623,  199059, 11, }, // America/Bahia_Banderas
    {  671, 5654,  103516,  8, }, // America/Barbados
    {  671, 7646,  199070,  5, }, // America/Belem
    {  671, 5407,  199075,  6, }, // America/Belize
    {  671, 7660,  199081, 11, }, // America/Blanc-Sablon
    {  671, 7681,  199092,  8, }, // America/Boa_Vista
    {  671, 4361,  199100,  5, }, // America/Bogota
    {  671, 7699,  199105,  5, }, // America/Boise
    {  671, 7713,  217749, 15, }, // America/Cambridge_Bay
    {  671, 7735,  199125, 11, }, // America/Campo_Grande
    {  671, 3868,  199136,  6, }, // America/Cancun
    {  671, 4694,  217764,  6, }, // America/Caracas
    {  671, 4345,  217770,  5, }, // America/Cayenne
    {  671, 5496,  199153,  5, }, // America/Cayman
    {  671, 2260,  199158,  5, }, // America/Chicago
    {  671, 7756,  199163,  8, }, // America/Chihuahua
    {  671, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  671, 5422,  199171, 10, }, // America/Costa_Rica
    {  671, 7796,  199181,  7, }, // America/Creston
    {  671, 3791,  199188,  7, }, // America/Cuiaba
    {  671, 5723,  199195,  8, }, // America/Curacao
    {  671, 7812,  103665, 11, }, // America/Danmarkshavn
    {  671, 7833,  199203,  5, }, // America/Dawson
    {  671, 7848,  199208, 10, }, // America/Dawson_Creek
    {  671,  805,  199218,  5, }, // America/Denver
    {  671, 3465,  199223,  7, }, // America/Detroit
    {  671, 5739,  199230,  7, }, // America/Dominica
    {  671,  893,  199237,  8, }, // America/Edmonton
    {  671, 7869,  199245,  7, }, // America/Eirunepe
    {  671, 5441,  199252, 10, }, // America/El_Salvador
    {  671, 7886,  103745, 10, }, // America/Fort_Nelson
    {  671, 7906,  199262,  9, }, // America/Fortaleza
    {  671, 7924,  217775, 13, }, // America/Glace_Bay
    {  671, 6881,  217788, 12, }, // America/Goose_Bay
    {  671, 4612,  199296,  8, }, // America/Grand_Turk
    {  671, 5770,  199304,  6, }, // America/Grenada
    {  671, 5786,  199310,  8, }, // America/Guadeloupe
    {  671, 3760,  199318, 10, }, // America/Guatemala
    {  671, 4841,  199328,  9, }, // America/Guayaquil
    {  671, 6932,  199337,  5, }, // America/Guyana
    {  671, 1939,  199342,  8, }, // America/Halifax
    {  671, 2281,  199350,  5, }, // America/Havana
    {  671, 7942,  199355,  8, }, // America/Hermosillo
    {  671,  348,  199363, 11, }, // America/Indiana/Indianapolis
    {  671,  481,  199374, 14, }, // America/Indiana/Knox
    {  671, 7961,  199388, 15, }, // America/Indiana/Marengo
    {  671, 7985,  199403, 17, }, // America/Indiana/Petersburg
    {  671, 8012,  199420, 16, }, // America/Indiana/Tell_City
    {  671, 8038,  199436, 13, }, // America/Indiana/Vevay
    {  671, 8060,  199449, 16, }, // America/Indiana/Vincennes
    {  671, 8086,  199465, 16, }, // America/Indiana/Winamac
    {  671, 8110,  199481,  6, }, // America/Inuvik
    {  671,  660,  199487,  8, }, // America/Iqaluit
    {  671, 2799,  199495,  7, }, // America/Jamaica
    {  671, 5380,  199502,  6, }, // America/Juneau
    {  671,  521,  199508,  9, }, // America/Kentucky/Louisville
    {  671, 8125,  199517, 18, }, // America/Kentucky/Monticello
    {  671, 5704,  199535, 10, }, // America/Kralendijk
    {  671, 4376,  104035,  6, }, // America/La_Paz
    {  671, 7169,  152645,  4, }, // America/Lima
    {  671, 3239,  152649, 10, }, // America/Los_Angeles
    {  671, 5932,  199545, 17, }, // America/Lower_Princes
    {  671, 8153,  168623,  6, }, // America/Maceio
    {  671, 8168,  199562,  6, }, // America/Managua
    {  671, 1908,  199568,  5, }, // America/Manaus
    {  671, 5897,  104091,  7, }, // America/Marigot
    {  671, 5805,  199573,  7, }, // America/Martinique
    {  671, 8184,  104106,  9, }, // America/Matamoros
    {  671, 2917,  199580,  7, }, // America/Mazatlan
    {  671, 8202,  199587,  8, }, // America/Menominee
    {  671, 8220,  199595,  6, }, // America/Merida
    {  671, 8235,  104137, 11, }, // America/Metlakatla
    {  671, 2949,  199601, 11, }, // America/Mexico_City
    {  671, 4391,  199612,  8, }, // America/Miquelon
    {  671, 8254,  199620,  6, }, // America/Moncton
    {  671, 8270,  199626,  7, }, // America/Monterrey
    {  671, 4098,  199633, 11, }, // America/Montevideo
    {  671, 5824,  199644,  9, }, // America/Montserrat
    {  671, 5481,  104199,  4, }, // America/Nassau
    {  671, 2379,  199653,  8, }, // America/New_York
    {  671, 8288,   11422,  3, }, // America/Nome
    {  671, 1850,  104212,  7, }, // America/Noronha
    {  671, 8301,  199661, 19, }, // America/North_Dakota/Beulah
    {  671, 8329,  199680, 16, }, // America/North_Dakota/Center
    {  671, 8357,  199696, 21, }, // America/North_Dakota/New_Salem
    {  671,  393,  103866,  3, }, // America/Nuuk
    {  671, 8388,  155480,  8, }, // America/Ojinaga
    {  671, 2356,  199717,  5, }, // America/Panama
    {  671, 6173,  199722,  9, }, // America/Paramaribo
    {  671, 2973,  152890,  5, }, // America/Phoenix
    {  671, 3945,  199731, 12, }, // America/Port-au-Prince
    {  671, 5954,  199743, 13, }, // America/Port_of_Spain
    {  671, 8404,  199756, 11, }, // America/Porto_Velho
    {  671, 5843,  199767, 11, }, // America/Puerto_Rico
    {  671, 4030,  199778, 11, }, // America/Punta_Arenas
    {  671, 8424,  199789, 11, }, // America/Rankin_Inlet
    {  671, 8445,  199800,  6, }, // America/Recife
    {  671, 1995,  168914,  6, }, // America/Regina
    {  671, 8460,  199806,  8, }, // America/Resolute
    {  671,  695,  199814, 11, }, // America/Rio_Branco
    {  671, 8477,  199825,  7, }, // America/Santarem
    {  671, 2201,  199832,  7, }, // America/Santiago
    {  671, 6111,  199839, 12, }, // America/Santo_Domingo
    {  671, 1878,  104449,  8, }, // America/Sao_Paulo
    {  671, 6899,  199851, 15, }, // America/Scoresbysund
    {  671, 8494,  199866,  5, }, // America/Sitka
    {  671, 8508,  199871, 13, }, // America/St_Barthelemy
    {  671, 2061,  199884,  9, }, // America/St_Johns
    {  671, 5863,  199893,  8, }, // America/St_Kitts
    {  671, 5880,  199901, 10, }, // America/St_Lucia
    {  671,  855,  199911, 10, }, // America/St_Thomas
    {  671, 5913,  199921, 10, }, // America/St_Vincent
    {  671, 8530,  199931, 10, }, // America/Swift_Current
    {  671, 5461,  199941, 11, }, // America/Tegucigalpa
    {  671, 5756,  199952,  5, }, // America/Thule
    {  671,  313,  199957,  7, }, // America/Tijuana
    {  671,  608,  199964,  6, }, // America/Toronto
    {  671, 5688,  199970,  7, }, // America/Tortola
    {  671, 2093,  153143,  7, }, // America/Vancouver
    {  671, 2144,  199977, 10, }, // America/Whitehorse
    {  671,  734,  199987,  7, }, // America/Winnipeg
    {  671, 8552,  199994,  6, }, // America/Yakutat
    {  671, 6001,  104629,  4, }, // Antarctica/Casey
    {  671, 6094,  200000,  4, }, // Antarctica/Davis
    {  671, 6133,  200004, 16, }, // Antarctica/DumontDUrville
    {  671, 8568,  200020,  7, }, // Antarctica/Macquarie
    {  671, 7066,  200027,  5, }, // Antarctica/Mawson
    {  671, 7113,  200032,  8, }, // Antarctica/McMurdo
    {  671, 8589,  200040,  5, }, // Antarctica/Palmer
    {  671, 7226,  104671,  7, }, // Antarctica/Rothera
    {  671, 7295,  104678,  5, }, // Antarctica/Syowa
    {  671, 8607,  200045,  4, }, // Antarctica/Troll
    {  671, 7374,  104683,  6, }, // Antarctica/Vostok
    {  671, 1429,  200049, 13, }, // Arctic/Longyearbyen
    {  671, 5570,   11948,  3, }, // Asia/Aden
    {  671, 5395,  200062,  6, }, // Asia/Almaty
    {  671, 3968,  200068,  5, }, // Asia/Amman
    {  671, 5511,  200073,  5, }, // Asia/Anadyr
    {  671, 5523,  200078,  5, }, // Asia/Aqtau
    {  671, 5368,  200083,  5, }, // Asia/Aqtobe
    {  671,  964,  200088,  5, }, // Asia/Ashgabat
    {  671, 8624,  200093,  6, }, // Asia/Atyrau
    {  671, 3607,   11991,  5, }, // Asia/Baghdad
    {  671, 5534,  153282,  5, }, // Asia/Bahrain
    {  671, 3653,  152318,  4, }, // Asia/Baku
    {  671, 4466,  153287,  5, }, // Asia/Bangkok
    {  671, 3571,  200099,  5, }, // Asia/Barnaul
    {  671, 4086,  153292,  5, }, // Asia/Beirut
    {  671, 3778,  153297,  5, }, // Asia/Bishkek
    {  671, 5989,  155867,  6, }, // Asia/Brunei
    {  671, 4601,  155873,  4, }, // Asia/Chita
    {  671, 4511,  153308,  6, }, // Asia/Colombo
    {  671, 4540,   12044,  4, }, // Asia/Damascus
    {  671, 1093,  200104,  5, }, // Asia/Dhaka
    {  671, 6192,  200109,  3, }, // Asia/Dili
    {  671, 3596,  200112,  4, }, // Asia/Dubai
    {  671, 6159,  200116,  7, }, // Asia/Dushanbe
    {  671, 8636,  200123,  9, }, // Asia/Famagusta
    {  671, 8651,  104843,  3, }, // Asia/Gaza
    {  671, 4795,  104846,  6, }, // Asia/Hebron
    {  671, 1259,  200132, 13, }, // Asia/Ho_Chi_Minh
    {  671, 2704,  217800,  9, }, // Asia/Hong_Kong
    {  671, 4771,  200145,  6, }, // Asia/Hovd
    {  671, 4184,  200151,  6, }, // Asia/Irkutsk
    {  671, 7006,  200157,  6, }, // Asia/Jakarta
    {  671, 6992,  200163,  7, }, // Asia/Jayapura
    {  671, 1290,  104892,  6, }, // Asia/Jerusalem
    {  671, 3560,  217809,  4, }, // Asia/Kabul
    {  671, 4316,  200170,  7, }, // Asia/Kamchatka
    {  671, 4239,  155964,  5, }, // Asia/Karachi
    {  671, 1185,  217813,  8, }, // Asia/Kathmandu
    {  671, 8661,  200185,  6, }, // Asia/Khandyga
    {  671,  992,  217821,  7, }, // Asia/Kolkata
    {  671, 4197,  200198, 11, }, // Asia/Krasnoyarsk
    {  671, 4859,  200209, 11, }, // Asia/Kuala_Lumpur
    {  671, 5976,  200220,  6, }, // Asia/Kuching
    {  671, 5547,  156017,  4, }, // Asia/Kuwait
    {  671, 1211,  217828,  4, }, // Asia/Macau
    {  671, 4017,  200230,  6, }, // Asia/Magadan
    {  671, 1349,  200236,  5, }, // Asia/Makassar
    {  671, 7182,  153473,  5, }, // Asia/Manila
    {  671, 6920,   12250,  4, }, // Asia/Muscat
    {  671, 2554,  200241,  6, }, // Asia/Nicosia
    {  671, 8675,  200247, 12, }, // Asia/Novokuznetsk
    {  671, 4135,  105020, 11, }, // Asia/Novosibirsk
    {  671, 4229,  105031,  5, }, // Asia/Omsk
    {  671, 7145,  156068,  5, }, // Asia/Oral
    {  671, 6961,  153508,  7, }, // Asia/Phnom_Penh
    {  671, 8693,  200259,  8, }, // Asia/Pontianak
    {  671, 4214,  200267, 10, }, // Asia/Pyongyang
    {  671, 5559,   12318,  3, }, // Asia/Qatar
    {  671, 8708,  200277,  7, }, // Asia/Qostanay
    {  671, 4269,  200284,  9, }, // Asia/Qyzylorda
    {  671, 3584,  105083,  4, }, // Asia/Riyadh
    {  671, 4408,  200293,  6, }, // Asia/Sakhalin
    {  671, 7245,   12350,  6, }, // Asia/Samarkand
    {  671, 3279,  200299,  5, }, // Asia/Seoul
    {  671, 1053,  153556,  7, }, // Asia/Shanghai
    {  671, 3300,  156136,  7, }, // Asia/Singapore
    {  671, 4297,  200304, 13, }, // Asia/Srednekolymsk
    {  671, 3263,  200317,  5, }, // Asia/Taipei
    {  671, 4781,  153589,  6, }, // Asia/Tashkent
    {  671, 3915,  200322,  6, }, // Asia/Tbilisi
    {  671, 2772,  200328,  5, }, // Asia/Tehran
    {  671, 1317,  200333,  5, }, // Asia/Thimphu
    {  671, 2821,  105166,  5, }, // Asia/Tokyo
    {  671, 4572,  200338,  5, }, // Asia/Tomsk
    {  671, 1021,  200343, 11, }, // Asia/Ulaanbaatar
    {  671, 1159,  200354,  7, }, // Asia/Urumqi
    {  671, 8722,  200361,  9, }, // Asia/Ust-Nera
    {  671, 6977,  200370,  8, }, // Asia/Vientiane
    {  671, 4710,  200378, 11, }, // Asia/Vladivostok
    {  671, 4828,  200389,  6, }, // Asia/Yakutsk
    {  671, 1235,  169586,  5, }, // Asia/Yangon
    {  671, 3883,  200395, 11, }, // Asia/Yekaterinburg
    {  671, 3747,  200406,  6, }, // Asia/Yerevan
    {  671, 3663,  200412,  6, }, // Atlantic/Azores
    {  671, 5671,   12497,  6, }, // Atlantic/Bermuda
    {  671, 6628,  155556,  5, }, // Atlantic/Canary
    {  671, 3727,  200418,  8, }, // Atlantic/Cape_Verde
    {  671, 1395,   12520,  4, }, // Atlantic/Faroe
    {  671, 8736,  200426,  6, }, // Atlantic/Madeira
    {  671, 2748,  200432,  7, }, // Atlantic/Reykjavik
    {  671, 7272,  105299, 12, }, // Atlantic/South_Georgia
    {  671, 6821,  200439, 11, }, // Atlantic/St_Helena
    {  671, 6644,  200450,  7, }, // Atlantic/Stanley
    {  671, 1670,  217832,  7, }, // Australia/Adelaide
    {  671, 1635,  105335,  6, }, // Australia/Brisbane
    {  671, 1799,  200464,  9, }, // Australia/Broken_Hill
    {  671, 1583,  200473,  5, }, // Australia/Darwin
    {  671, 3637,  200478,  6, }, // Australia/Eucla
    {  671, 1516,  200484,  6, }, // Australia/Hobart
    {  671, 8753,  200490,  7, }, // Australia/Lindeman
    {  671, 1547,  200497,  9, }, // Australia/Lord_Howe
    {  671, 1727,  156361,  6, }, // Australia/Melbourne
    {  671, 1762,  200506,  4, }, // Australia/Perth
    {  671, 1463,  200510,  4, }, // Australia/Sydney
    {  671, 6441,  200514,  8, }, // Europe/Amsterdam
    {  671, 6216,  200522,  6, }, // Europe/Andorra
    {  671, 3620,  200528,  8, }, // Europe/Astrakhan
    {  671, 2300,  200536,  6, }, // Europe/Athens
    {  671, 6336,   12667,  6, }, // Europe/Belgrade
    {  671, 4757,  153847,  4, }, // Europe/Berlin
    {  671, 6488,  200542, 10, }, // Europe/Bratislava
    {  671, 2167,  169744,  5, }, // Europe/Brussels
    {  671, 3928,  200552,  6, }, // Europe/Bucharest
    {  671, 3806,  200558,  6, }, // Europe/Budapest
    {  671, 8772,  200564,  5, }, // Europe/Busingen
    {  671, 2583,  200569,  7, }, // Europe/Chisinau
    {  671, 6289,  105498,  9, }, // Europe/Copenhagen
    {  671, 2338,  200576,  4, }, // Europe/Dublin
    {  671, 6307,  153904, 10, }, // Europe/Gibraltar
    {  671, 8788,  156466,  5, }, // Europe/Guernsey
    {  671, 6612,  200580,  7, }, // Europe/Helsinki
    {  671, 8804,  105537, 10, }, // Europe/Isle_of_Man
    {  671, 1130,  153938,  7, }, // Europe/Istanbul
    {  671, 8823,  105556,  4, }, // Europe/Jersey
    {  671, 3979,  200587, 11, }, // Europe/Kaliningrad
    {  671, 8929,  153955,  5, }, // Europe/Kirov
    {  671, 2527,  103079,  3, }, // Europe/Kyiv
    {  671, 3213,  169843,  4, }, // Europe/Lisbon
    {  671, 6506,  156510,  9, }, // Europe/Ljubljana
    {  671, 2501,   12808,  4, }, // Europe/London
    {  671, 6365,  200598,  7, }, // Europe/Luxembourg
    {  671, 4892,  200605,  5, }, // Europe/Madrid
    {  671, 6397,  105621,  5, }, // Europe/Malta
    {  671, 6582,  200610,  7, }, // Europe/Mariehamn
    {  671, 3693,  105634,  5, }, // Europe/Minsk
    {  671, 6410,  105639,  6, }, // Europe/Monaco
    {  671, 3537,  105645,  5, }, // Europe/Moscow
    {  671, 6458,  105650,  5, }, // Europe/Oslo
    {  671, 4284,  200617,  4, }, // Europe/Paris
    {  671, 6424,  200621,  9, }, // Europe/Podgorica
    {  671, 6275,  105669,  4, }, // Europe/Prague
    {  671, 8837,  155316,  4, }, // Europe/Riga
    {  671, 6324,   10638,  3, }, // Europe/Rome
    {  671, 4331,  106627,  5, }, // Europe/Samara
    {  671, 6470,  200630, 11, }, // Europe/San_Marino
    {  671, 6245,  169912,  7, }, // Europe/Sarajevo
    {  671, 4451,  200641,  6, }, // Europe/Saratov
    {  671, 8849,  169924,  8, }, // Europe/Simferopol
    {  671, 6383,  200647,  7, }, // Europe/Skopje
    {  671, 6599,  200654,  5, }, // Europe/Sofia
    {  671, 6523,  200659,  9, }, // Europe/Stockholm
    {  671, 8867,  200668,  4, }, // Europe/Tallinn
    {  671, 6202,  200672,  6, }, // Europe/Tirane
    {  671, 8882,  200678,  8, }, // Europe/Ulyanovsk
    {  671, 6352,  200686,  5, }, // Europe/Vaduz
    {  671, 6567,  200691,  6, }, // Europe/Vatican
    {  671, 6231,  154095,  5, }, // Europe/Vienna
    {  671, 8899,  200697,  7, }, // Europe/Vilnius
    {  671, 4727,  169980,  9, }, // Europe/Volgograd
    {  671, 3190,  105795,  5, }, // Europe/Warsaw
    {  671, 6261,  200704,  6, }, // Europe/Zagreb
    {  671, 6540,  200710,  5, }, // Europe/Zurich
    {  671, 5078,  200715, 12, }, // Indian/Antananarivo
    {  671, 6947,  105823,  5, }, // Indian/Chagos
    {  671, 6046,  200727,  5, }, // Indian/Christmas
    {  671, 6063,  105835,  5, }, // Indian/Cocos
    {  671, 5029,  105840,  6, }, // Indian/Comoro
    {  671, 6661,  200732,  8, }, // Indian/Kerguelen
    {  671, 7260,  105854,  4, }, // Indian/Mahe
    {  671, 7050,  200740,  6, }, // Indian/Maldives
    {  671, 4069,  200746,  6, }, // Indian/Mauritius
    {  671, 5098,  200752,  5, }, // Indian/Mayotte
    {  671, 7211,  200757,  8, }, // Indian/Reunion
    {  671, 4422,  154184,  4, }, // Pacific/Apia
    {  671,  932,  105892,  6, }, // Pacific/Auckland
    {  671, 3706,  200765,  8, }, // Pacific/Bougainville
    {  671, 3015,  200773,  5, }, // Pacific/Chatham
    {  671, 3157,  200778,  4, }, // Pacific/Chuuk
    {  671, 2237,  105420,  5, }, // Pacific/Easter
    {  671, 7360,  200782,  5, }, // Pacific/Efate
    {  671, 7327,  200787,  6, }, // Pacific/Fakaofo
    {  671, 3902,  200793,  3, }, // Pacific/Fiji
    {  671, 7343,  200796, 10, }, // Pacific/Funafuti
    {  671, 6678,  200806,  9, }, // Pacific/Galapagos
    {  671, 6696,  200815,  7, }, // Pacific/Gambier
    {  671, 3822,  200822, 11, }, // Pacific/Guadalcanal
    {  671, 6018,  199565,  4, }, // Pacific/Guam
    {  671, 3049,  200841,  8, }, // Pacific/Kanton
    {  671, 3998,  200849,  9, }, // Pacific/Kiritimati
    {  671, 7019,  200858,  7, }, // Pacific/Kosrae
    {  671, 2842,  156816,  8, }, // Pacific/Kwajalein
    {  671, 4877,  154287,  5, }, // Pacific/Majuro
    {  671, 4051,  200865,  8, }, // Pacific/Marquesas
    {  671, 8914,  200873,  4, }, // Pacific/Midway
    {  671, 7084,  200877,  5, }, // Pacific/Nauru
    {  671, 7132,  199504,  4, }, // Pacific/Niue
    {  671, 4168,  106037,  6, }, // Pacific/Norfolk
    {  671, 7098,  200882,  5, }, // Pacific/Noumea
    {  671, 3126,  170185,  9, }, // Pacific/Pago_Pago
    {  671, 7155,  200887,  4, }, // Pacific/Palau
    {  671, 7194,  200891,  7, }, // Pacific/Pitcairn
    {  671, 3096,  200898,  7, }, // Pacific/Pohnpei
    {  671, 4807,  200905, 12, }, // Pacific/Port_Moresby
    {  671, 6076,  106086,  9, }, // Pacific/Rarotonga
    {  671, 6031,  200917,  6, }, // Pacific/Saipan
    {  671, 7312,  200923,  6, }, // Pacific/Tahiti
    {  671, 6712,  200929,  5, }, // Pacific/Tarawa
    {  671, 4583,  200934,  9, }, // Pacific/Tongatapu
    {  671, 7392,  106123,  3, }, // Pacific/Wake
    {  671, 7405,  200943,  5, }, // Pacific/Wallis
    {  672, 6788,  217839,  6, }, // Africa/Abidjan Anii/Latin/Benin
    {  672, 6760,    4531,  5, }, // Africa/Accra
    {  672, 5059,  217845, 10, }, // Africa/Addis_Ababa
    {  672, 7420,  217855,  6, }, // Africa/Algiers
    {  672,   14,  217861,  7, }, // Africa/Asmara
    {  672,   44,  217868,  7, }, // Africa/Bamako
    {  672, 5257,  217875,  6, }, // Africa/Bangui
    {  672, 7435,  217881,  5, }, // Africa/Bissau
    {  672, 4957,  217886,  8, }, // Africa/Blantyre
    {  672, 5287,  217894,  8, }, // Africa/Brazzaville
    {  672, 4922,  217902, 10, }, // Africa/Bujumbura
    {  672, 2320,  217912,  6, }, // Africa/Cairo
    {  672, 4117,  217918, 11, }, // Africa/Casablanca
    {  672, 7449,  217929,  6, }, // Africa/Ceuta
    {  672, 6773,  217935,  8, }, // Africa/Conakry
    {  672, 6840,  217943,  6, }, // Africa/Dakar
    {  672, 5130,  217949, 10, }, // Africa/Dar_es_Salaam
    {  672, 5043,  217959,  7, }, // Africa/Djibouti
    {  672, 5243,  217966,  7, }, // Africa/Douala
    {  672, 5166,  217973,  7, }, // Africa/El_Aaiun
    {  672, 6853,  217980,  8, }, // Africa/Freetown
    {  672, 5015,  217988,  7, }, // Africa/Harare
    {  672, 4479,  217995, 11, }, // Africa/Johannesburg
    {  672, 4499,  218006,  5, }, // Africa/Juba
    {  672, 5151,  218011,  8, }, // Africa/Kampala
    {  672, 4524,  218019,  7, }, // Africa/Khartoum
    {  672, 4973,  205241,  7, }, // Africa/Kigali
    {  672, 5306,  218026,  9, }, // Africa/Kinshasa
    {  672, 4744,    4627,  5, }, // Africa/Lagos
    {  672, 5336,  218035,  8, }, // Africa/Libreville
    {  672, 6869,  218043,  4, }, // Africa/Lome
    {  672, 5211,  218047,  8, }, // Africa/Luanda
    {  672, 4939,  205292, 11, }, // Africa/Lubumbashi
    {  672, 5001,  218055,  7, }, // Africa/Lusaka
    {  672, 5322,  218062,  7, }, // Africa/Malabo
    {  672, 4987,  218069,  7, }, // Africa/Maputo
    {  672, 5197,  218076,  7, }, // Africa/Maseru
    {  672, 5182,  218083,  8, }, // Africa/Mbabane
    {  672, 5113,  218091, 10, }, // Africa/Mogadishu
    {  672, 7034,  218101, 10, }, // Africa/Monrovia
    {  672, 3853,  205361,  8, }, // Africa/Nairobi
    {  672, 5271,  218111,  8, }, // Africa/Ndjamena
    {  672, 5354,  218119,  5, }, // Africa/Niamey
    {  672, 6803,  218124,  8, }, // Africa/Nouakchott
    {  672, 6727,  218132,  8, }, // Africa/Ouagadougou
    {  672, 5225,  218140, 11, }, // Africa/Porto-Novo
    {  672, 4435,  218151, 13, }, // Africa/Sao_Tome
    {  672, 2866,  205435,  8, }, // Africa/Tripoli
    {  672, 6554,  218164,  6, }, // Africa/Tunis
    {  672, 4152,  218170,  7, }, // Africa/Windhoek
    {  672,  132,  218177,  4, }, // America/Adak
    {  672, 3346,  218181,  9, }, // America/Anchorage
    {  672, 5607,  218190,  9, }, // America/Anguilla
    {  672, 5624,  218199,  9, }, // America/Antigua
    {  672, 4554,  218208, 10, }, // America/Araguaina
    {  672,  166,  218218, 13, }, // America/Argentina/Buenos_Aires
    {  672,   91,  218231, 10, }, // America/Argentina/Catamarca
    {  672,  270,  218241,  8, }, // America/Argentina/Cordoba
    {  672,  441,  218249,  6, }, // America/Argentina/Jujuy
    {  672, 7462,  218255,  9, }, // America/Argentina/La_Rioja
    {  672,  565,  218264,  8, }, // America/Argentina/Mendoza
    {  672, 7489,  218272, 12, }, // America/Argentina/Rio_Gallegos
    {  672, 7544,  218284,  9, }, // America/Argentina/San_Juan
    {  672, 5580,  174917,  9, }, // America/Argentina/San_Luis
    {  672, 7571,    4724,  7, }, // America/Argentina/Tucuman
    {  672, 7597,  218293,  9, }, // America/Argentina/Ushuaia
    {  672, 5640,  218302,  6, }, // America/Aruba
    {  672, 4252,  218308,  9, }, // America/Asuncion
    {  672,  237,      22,  8, }, // America/Atikokan
    {  672, 3679,  218317,  7, }, // America/Bahia
    {  672, 7623,  218324, 21, }, // America/Bahia_Banderas
    {  672, 5654,  218345, 10, }, // America/Barbados
    {  672, 7646,  218355,  5, }, // America/Belem
    {  672, 5407,  174982,  6, }, // America/Belize
    {  672, 7660,  218360, 11, }, // America/Blanc-Sablon
    {  672, 7681,  218371, 11, }, // America/Boa_Vista
    {  672, 7699,  218382,  7, }, // America/Boise
    {  672, 7713,  218389, 13, }, // America/Cambridge_Bay
    {  672, 7735,  218402, 13, }, // America/Campo_Grande
    {  672, 3868,  218415,  6, }, // America/Cancun
    {  672, 4694,  175047,  8, }, // America/Caracas
    {  672, 4345,  218421,  6, }, // America/Cayenne
    {  672, 5496,  218427,  8, }, // America/Cayman
    {  672, 2260,  218435,  8, }, // America/Chicago
    {  672, 7756,  218443,  8, }, // America/Chihuahua
    {  672, 7774,  218451, 16, }, // America/Ciudad_Juarez
    {  672, 5422,  218467, 11, }, // America/Costa_Rica
    {  672, 7796,  218478,  7, }, // America/Creston
    {  672, 3791,  218485,  7, }, // America/Cuiaba
    {  672, 5723,  218492,  9, }, // America/Curacao
    {  672, 7812,  218501, 13, }, // America/Danmarkshavn
    {  672, 7833,  218514,  6, }, // America/Dawson
    {  672, 7848,  218520, 12, }, // America/Dawson_Creek
    {  672,  805,  218532,  5, }, // America/Denver
    {  672, 3465,  218537,  7, }, // America/Detroit
    {  672, 5739,  218544,  9, }, // America/Dominica
    {  672,  893,  218553,  8, }, // America/Edmonton
    {  672, 7869,  218561,  9, }, // America/Eirunepe
    {  672, 5441,  218570, 12, }, // America/El_Salvador
    {  672, 7886,  218582, 11, }, // America/Fort_Nelson
    {  672, 7906,  218593, 10, }, // America/Fortaleza
    {  672, 7924,  218603,  9, }, // America/Glace_Bay
    {  672, 6881,  218612,  8, }, // America/Goose_Bay
    {  672, 4612,  218620, 10, }, // America/Grand_Turk
    {  672, 5770,  218630,  8, }, // America/Grenada
    {  672, 5786,  218638, 11, }, // America/Guadeloupe
    {  672, 3760,  218649, 11, }, // America/Guatemala
    {  672, 4841,  218660,  9, }, // America/Guayaquil
    {  672, 6932,  218669,  7, }, // America/Guyana
    {  672, 1939,  218676,  8, }, // America/Halifax
    {  672, 2281,  218684,  7, }, // America/Havana
    {  672, 7942,  218691,  9, }, // America/Hermosillo
    {  672,  348,  218700, 13, }, // America/Indiana/Indianapolis
    {  672,  481,  218713, 14, }, // America/Indiana/Knox
    {  672, 7961,  218727, 18, }, // America/Indiana/Marengo
    {  672, 7985,  218745, 21, }, // America/Indiana/Petersburg
    {  672, 8012,  218766, 19, }, // America/Indiana/Tell_City
    {  672, 8038,  218785, 16, }, // America/Indiana/Vevay
    {  672, 8060,  218801, 17, }, // America/Indiana/Vincennes
    {  672, 8086,  218818, 19, }, // America/Indiana/Winamac
    {  672, 8110,  218837,  6, }, // America/Inuvik
    {  672,  660,  218843,  9, }, // America/Iqaluit
    {  672, 2799,  218852,  7, }, // America/Jamaica
    {  672, 5380,  218859,  5, }, // America/Juneau
    {  672,  521,  218864,  8, }, // America/Kentucky/Louisville
    {  672, 8125,  218872, 20, }, // America/Kentucky/Monticello
    {  672, 5704,  218892, 10, }, // America/Kralendijk
    {  672, 4376,   21755,  6, }, // America/La_Paz
    {  672, 7169,  218902,  5, }, // America/Lima
    {  672, 3239,  218907, 12, }, // America/Los_Angeles
    {  672, 5932,  218919, 17, }, // America/Lower_Princes
    {  672, 8153,  218936,  8, }, // America/Maceio
    {  672, 8168,  175593,  9, }, // America/Managua
    {  672, 1908,  218944,  6, }, // America/Manaus
    {  672, 5897,  218950,  7, }, // America/Marigot
    {  672, 5805,  218957, 10, }, // America/Martinique
    {  672, 8184,  218967, 10, }, // America/Matamoros
    {  672, 2917,  218977,  9, }, // America/Mazatlan
    {  672, 8220,  218986,  7, }, // America/Merida
    {  672, 8235,  218993, 13, }, // America/Metlakatla
    {  672, 2949,  219006, 11, }, // America/Mexico_City
    {  672, 4391,  219017,  7, }, // America/Miquelon
    {  672, 8254,  219024,  6, }, // America/Moncton
    {  672, 8270,  219030,  8, }, // America/Monterrey
    {  672, 4098,  219038, 12, }, // America/Montevideo
    {  672, 5824,  219050, 10, }, // America/Montserrat
    {  672, 5481,  125683,  5, }, // America/Nassau
    {  672, 2379,  219060,  8, }, // America/New_York
    {  672, 1850,  219068,  7, }, // America/Noronha
    {  672, 8301,  219075, 19, }, // America/North_Dakota/Beulah
    {  672, 8329,  219094, 19, }, // America/North_Dakota/Center
    {  672, 8357,  219113, 22, }, // America/North_Dakota/New_Salem
    {  672,  393,     298,  4, }, // America/Nuuk
    {  672, 8388,  175823,  8, }, // America/Ojinaga
    {  672, 2356,  219135,  7, }, // America/Panama
    {  672, 6173,  219142, 11, }, // America/Paramaribo
    {  672, 2973,  219153,  6, }, // America/Phoenix
    {  672, 3945,  219159, 12, }, // America/Port-au-Prince
    {  672, 5954,  219171, 13, }, // America/Port_of_Spain
    {  672, 8404,  219184, 11, }, // America/Porto_Velho
    {  672, 5843,  219195, 11, }, // America/Puerto_Rico
    {  672, 8424,  219206, 14, }, // America/Rankin_Inlet
    {  672, 8445,  165918,  6, }, // America/Recife
    {  672, 1995,  219220,  7, }, // America/Regina
    {  672, 8460,  219227,  8, }, // America/Resolute
    {  672,  695,  219235, 12, }, // America/Rio_Branco
    {  672, 8477,  219247,  8, }, // America/Santarem
    {  672, 2201,  219255, 10, }, // America/Santiago
    {  672, 6111,  219265, 13, }, // America/Santo_Domingo
    {  672, 1878,  219278, 10, }, // America/Sao_Paulo
    {  672, 6899,  219288, 14, }, // America/Scoresbysund
    {  672, 8494,  219302,  6, }, // America/Sitka
    {  672, 8508,  219308, 14, }, // America/St_Barthelemy
    {  672, 2061,  219322,  9, }, // America/St_Johns
    {  672, 5863,  219331,  8, }, // America/St_Kitts
    {  672, 5880,  219339, 10, }, // America/St_Lucia
    {  672,  855,  219349, 11, }, // America/St_Thomas
    {  672, 5913,  219360, 10, }, // America/St_Vincent
    {  672, 8530,  219370, 16, }, // America/Swift_Current
    {  672, 5461,    5410, 11, }, // America/Tegucigalpa
    {  672, 5756,  219386,  5, }, // America/Thule
    {  672,  313,  219391,  8, }, // America/Tijuana
    {  672,  608,  219399,  8, }, // America/Toronto
    {  672, 5688,  219407,  8, }, // America/Tortola
    {  672, 2093,  219415,  8, }, // America/Vancouver
    {  672, 2144,  219423,  8, }, // America/Whitehorse
    {  672,  734,  219431,  8, }, // America/Winnipeg
    {  672, 6001,  219439,  5, }, // Antarctica/Casey
    {  672, 6094,  219444,  5, }, // Antarctica/Davis
    {  672, 6133,  219449, 12, }, // Antarctica/DumontDUrville
    {  672, 8568,  219461,  7, }, // Antarctica/Macquarie
    {  672, 7066,  219468,  5, }, // Antarctica/Mawson
    {  672, 7113,  219473,  9, }, // Antarctica/McMurdo
    {  672, 8589,  219482,  7, }, // Antarctica/Palmer
    {  672, 7226,  219489,  7, }, // Antarctica/Rothera
    {  672, 7295,  219496,  7, }, // Antarctica/Syowa
    {  672, 8607,  219503,  4, }, // Antarctica/Troll
    {  672, 7374,  219507,  6, }, // Antarctica/Vostok
    {  672, 1429,  219513, 12, }, // Arctic/Longyearbyen
    {  672, 5570,  219525,  4, }, // Asia/Aden
    {  672, 5395,  219529,  7, }, // Asia/Almaty
    {  672, 3968,  219536,  5, }, // Asia/Amman
    {  672, 5511,  219541,  7, }, // Asia/Anadyr
    {  672, 5523,  219548,  5, }, // Asia/Aqtau
    {  672, 5368,  219553,  7, }, // Asia/Aqtobe
    {  672,  964,  219560,  9, }, // Asia/Ashgabat
    {  672, 8624,  219569,  6, }, // Asia/Atyrau
    {  672, 3607,  219575,  7, }, // Asia/Baghdad
    {  672, 5534,  219582,  6, }, // Asia/Bahrain
    {  672, 3653,   63486,  5, }, // Asia/Baku
    {  672, 4466,  219588,  6, }, // Asia/Bangkok
    {  672, 3571,  219594,  8, }, // Asia/Barnaul
    {  672, 4086,  219602,  7, }, // Asia/Beirut
    {  672, 3778,  219609,  7, }, // Asia/Bishkek
    {  672, 5989,  219616,  6, }, // Asia/Brunei
    {  672, 4601,  219622,  6, }, // Asia/Chita
    {  672, 4511,  219628,  8, }, // Asia/Colombo
    {  672, 4540,  219636, 10, }, // Asia/Damascus
    {  672, 1093,  217943,  5, }, // Asia/Dhaka
    {  672, 6192,  219646,  5, }, // Asia/Dili
    {  672, 3596,  219651,  5, }, // Asia/Dubai
    {  672, 6159,  219656,  9, }, // Asia/Dushanbe
    {  672, 8636,  219665, 10, }, // Asia/Famagusta
    {  672, 8651,  219675,  5, }, // Asia/Gaza
    {  672, 4795,  219680,  6, }, // Asia/Hebron
    {  672, 1259,  219686, 15, }, // Asia/Ho_Chi_Minh
    {  672, 2704,  219701,  7, }, // Asia/Hong_Kong
    {  672, 4771,  219708,  5, }, // Asia/Hovd
    {  672, 4184,   62232,  5, }, // Asia/Irkutsk
    {  672, 7006,  219713,  7, }, // Asia/Jakarta
    {  672, 6992,  219720,  9, }, // Asia/Jayapura
    {  672, 1290,  219729,  9, }, // Asia/Jerusalem
    {  672, 4316,  219738, 10, }, // Asia/Kamchatka
    {  672, 4239,  219748,  6, }, // Asia/Karachi
    {  672, 1185,  219754,  9, }, // Asia/Kathmandu
    {  672, 8661,  219763,  8, }, // Asia/Khandyga
    {  672,  992,  219771,  7, }, // Asia/Kolkata
    {  672, 4197,  219778, 10, }, // Asia/Krasnoyarsk
    {  672, 4859,  219788, 14, }, // Asia/Kuala_Lumpur
    {  672, 5976,  219802,  5, }, // Asia/Kuching
    {  672, 5547,  219807,  7, }, // Asia/Kuwait
    {  672, 1211,  219814,  7, }, // Asia/Macau
    {  672, 4017,  219821,  7, }, // Asia/Magadan
    {  672, 1349,  219828,  8, }, // Asia/Makassar
    {  672, 7182,  219836,  7, }, // Asia/Manila
    {  672, 6920,  219843,  7, }, // Asia/Muscat
    {  672, 2554,   22378,  8, }, // Asia/Nicosia
    {  672, 8675,  219850, 10, }, // Asia/Novokuznetsk
    {  672, 4135,  219860, 10, }, // Asia/Novosibirsk
    {  672, 4229,  219870,  6, }, // Asia/Omsk
    {  672, 7145,  219876,  5, }, // Asia/Oral
    {  672, 6961,  219881,  7, }, // Asia/Phnom_Penh
    {  672, 8693,  219888, 10, }, // Asia/Pontianak
    {  672, 4214,  219898,  8, }, // Asia/Pyongyang
    {  672, 5559,  215362,  5, }, // Asia/Qatar
    {  672, 8708,  219906,  8, }, // Asia/Qostanay
    {  672, 4269,  219914, 10, }, // Asia/Qyzylorda
    {  672, 3584,  219924,  6, }, // Asia/Riyadh
    {  672, 4408,    7673,  7, }, // Asia/Sakhalin
    {  672, 7245,  219930,  9, }, // Asia/Samarkand
    {  672, 3279,  219939,  5, }, // Asia/Seoul
    {  672, 1053,  219944,  7, }, // Asia/Shanghai
    {  672, 3300,  219951,  9, }, // Asia/Singapore
    {  672, 4297,  219960, 11, }, // Asia/Srednekolymsk
    {  672, 3263,  219971,  6, }, // Asia/Taipei
    {  672, 4781,  219977,  8, }, // Asia/Tashkent
    {  672, 3915,  219985,  8, }, // Asia/Tbilisi
    {  672, 2772,  219993,  7, }, // Asia/Tehran
    {  672, 1317,  220000,  5, }, // Asia/Thimphu
    {  672, 2821,  220005,  7, }, // Asia/Tokyo
    {  672, 4572,  220012,  7, }, // Asia/Tomsk
    {  672, 1021,  220019, 10, }, // Asia/Ulaanbaatar
    {  672, 1159,  220029,  7, }, // Asia/Urumqi
    {  672, 8722,  220036,  9, }, // Asia/Ust-Nera
    {  672, 6977,  220045, 10, }, // Asia/Vientiane
    {  672, 4710,  220055, 11, }, // Asia/Vladivostok
    {  672, 4828,  176144,  5, }, // Asia/Yakutsk
    {  672, 1235,  220066,  6, }, // Asia/Yangon
    {  672, 3883,  220072, 12, }, // Asia/Yekaterinburg
    {  672, 3747,  220084,  8, }, // Asia/Yerevan
    {  672, 3663,  220092,  5, }, // Atlantic/Azores
    {  672, 5671,  220097,  8, }, // Atlantic/Bermuda
    {  672, 6628,  220105,  7, }, // Atlantic/Canary
    {  672, 3727,  220112,  7, }, // Atlantic/Cape_Verde
    {  672, 1395,  220119,  5, }, // Atlantic/Faroe
    {  672, 8736,  220124,  8, }, // Atlantic/Madeira
    {  672, 2748,  220132,  9, }, // Atlantic/Reykjavik
    {  672, 7272,  220141, 25, }, // Atlantic/South_Georgia
    {  672, 6821,  220166, 10, }, // Atlantic/St_Helena
    {  672, 6644,  220176,  7, }, // Atlantic/Stanley
    {  672, 1670,  220183,  7, }, // Australia/Adelaide
    {  672, 1635,  220190,  7, }, // Australia/Brisbane
    {  672, 1799,  220197, 10, }, // Australia/Broken_Hill
    {  672, 1583,  220207,  6, }, // Australia/Darwin
    {  672, 3637,  220213,  5, }, // Australia/Eucla
    {  672, 1516,  220218,  5, }, // Australia/Hobart
    {  672, 8753,  220223,  8, }, // Australia/Lindeman
    {  672, 1547,  220231,  8, }, // Australia/Lord_Howe
    {  672, 1727,  220239,  7, }, // Australia/Melbourne
    {  672, 1762,  220246,  4, }, // Australia/Perth
    {  672, 1463,  220250,  6, }, // Australia/Sydney
    {  672, 6441,  220256,  9, }, // Europe/Amsterdam
    {  672, 6216,  220265,  7, }, // Europe/Andorra
    {  672, 3620,  220272,  9, }, // Europe/Astrakhan
    {  672, 2300,  220281,  4, }, // Europe/Athens
    {  672, 6336,  220285,  8, }, // Europe/Belgrade
    {  672, 4757,  220293,  6, }, // Europe/Berlin
    {  672, 6488,  220299, 10, }, // Europe/Bratislava
    {  672, 2167,  220309,  6, }, // Europe/Brussels
    {  672, 3928,  220315,  7, }, // Europe/Bucharest
    {  672, 3806,  220322,  7, }, // Europe/Budapest
    {  672, 8772,  220329,  8, }, // Europe/Busingen
    {  672, 2583,  220337,  8, }, // Europe/Chisinau
    {  672, 6289,  220345,  8, }, // Europe/Copenhagen
    {  672, 2338,  220353,  6, }, // Europe/Dublin
    {  672, 6307,  215477,  9, }, // Europe/Gibraltar
    {  672, 8788,  220359,  8, }, // Europe/Guernsey
    {  672, 6612,  220367,  9, }, // Europe/Helsinki
    {  672, 8804,  220376, 16, }, // Europe/Isle_of_Man
    {  672, 1130,  220392,  9, }, // Europe/Istanbul
    {  672, 8823,  214663,  6, }, // Europe/Jersey
    {  672, 3979,  220401, 12, }, // Europe/Kaliningrad
    {  672, 8929,  220413,  6, }, // Europe/Kirov
    {  672, 2527,  220419,  5, }, // Europe/Kyiv
    {  672, 3213,    5891,  6, }, // Europe/Lisbon
    {  672, 6506,  220424,  9, }, // Europe/Ljubljana
    {  672, 2501,  220433,  6, }, // Europe/London
    {  672, 6365,  220439,  9, }, // Europe/Luxembourg
    {  672, 4892,  220448,  7, }, // Europe/Madrid
    {  672, 6582,  220455,  9, }, // Europe/Mariehamn
    {  672, 3693,  220464,  4, }, // Europe/Minsk
    {  672, 6410,  220468,  7, }, // Europe/Monaco
    {  672, 3537,  220475,  6, }, // Europe/Moscow
    {  672, 6458,  220481,  5, }, // Europe/Oslo
    {  672, 4284,   63726,  5, }, // Europe/Paris
    {  672, 6424,  220486, 10, }, // Europe/Podgorica
    {  672, 6275,    4415,  5, }, // Europe/Prague
    {  672, 8837,  220496,  5, }, // Europe/Riga
    {  672, 6324,   63736,  4, }, // Europe/Rome
    {  672, 4331,  220501,  7, }, // Europe/Samara
    {  672, 6470,  220508, 11, }, // Europe/San_Marino
    {  672, 6245,  220519,  9, }, // Europe/Sarajevo
    {  672, 4451,  220528,  8, }, // Europe/Saratov
    {  672, 6383,  220536,  6, }, // Europe/Skopje
    {  672, 6599,   22787,  6, }, // Europe/Sofia
    {  672, 6523,  220542,  8, }, // Europe/Stockholm
    {  672, 8867,    8132,  5, }, // Europe/Tallinn
    {  672, 6202,  220550,  7, }, // Europe/Tirane
    {  672, 8882,  220557,  8, }, // Europe/Ulyanovsk
    {  672, 6352,  220565,  5, }, // Europe/Vaduz
    {  672, 6567,  220570,  7, }, // Europe/Vatican
    {  672, 6231,  220577,  6, }, // Europe/Vienna
    {  672, 8899,  220583,  8, }, // Europe/Vilnius
    {  672, 4727,  220591, 10, }, // Europe/Volgograd
    {  672, 3190,  220601,  6, }, // Europe/Warsaw
    {  672, 6261,  220607,  6, }, // Europe/Zagreb
    {  672, 6540,  220613,  6, }, // Europe/Zurich
    {  672, 5078,  220619, 13, }, // Indian/Antananarivo
    {  672, 6947,  220632,  6, }, // Indian/Chagos
    {  672, 6046,  220638,  5, }, // Indian/Christmas
    {  672, 6063,  220643,  6, }, // Indian/Cocos
    {  672, 5029,  220649,  7, }, // Indian/Comoro
    {  672, 6661,  220656,  9, }, // Indian/Kerguelen
    {  672, 7260,  220665,  5, }, // Indian/Mahe
    {  672, 7050,  220670,  8, }, // Indian/Maldives
    {  672, 4069,  220678,  6, }, // Indian/Mauritius
    {  672, 5098,  220684,  7, }, // Indian/Mayotte
    {  672, 7211,  220691,  8, }, // Indian/Reunion
    {  672, 4422,  214957,  5, }, // Pacific/Apia
    {  672,  932,  220699,  7, }, // Pacific/Auckland
    {  672, 3706,  220706,  9, }, // Pacific/Bougainville
    {  672, 3015,  220715,  6, }, // Pacific/Chatham
    {  672, 3157,  220721,  4, }, // Pacific/Chuuk
    {  672, 2237,   57315,  4, }, // Pacific/Easter
    {  672, 7360,  220725,  6, }, // Pacific/Efate
    {  672, 7327,  220731,  9, }, // Pacific/Fakaofo
    {  672, 7343,  208825,  9, }, // Pacific/Funafuti
    {  672, 6678,    6080,  9, }, // Pacific/Galapagos
    {  672, 6696,  220740,  8, }, // Pacific/Gambier
    {  672, 3822,  220748, 12, }, // Pacific/Guadalcanal
    {  672, 6018,  177300,  5, }, // Pacific/Guam
    {  672, 3049,  220760,  6, }, // Pacific/Kanton
    {  672, 3998,  220766,  9, }, // Pacific/Kiritimati
    {  672, 7019,  220775,  6, }, // Pacific/Kosrae
    {  672, 2842,  220781,  9, }, // Pacific/Kwajalein
    {  672, 4877,  220790,  7, }, // Pacific/Majuro
    {  672, 4051,   22977,  8, }, // Pacific/Marquesas
    {  672, 8914,  220797,  6, }, // Pacific/Midway
    {  672, 7084,  220803,  6, }, // Pacific/Nauru
    {  672, 7132,  220809,  6, }, // Pacific/Niue
    {  672, 4168,  220815,  7, }, // Pacific/Norfolk
    {  672, 7098,  215111,  6, }, // Pacific/Noumea
    {  672, 3126,  220822, 10, }, // Pacific/Pago_Pago
    {  672, 7155,  220832,  7, }, // Pacific/Palau
    {  672, 7194,  220839,  7, }, // Pacific/Pitcairn
    {  672, 3096,  220846,  7, }, // Pacific/Pohnpei
    {  672, 4807,  220853, 13, }, // Pacific/Port_Moresby
    {  672, 6076,  220866,  9, }, // Pacific/Rarotonga
    {  672, 6031,  220875,  6, }, // Pacific/Saipan
    {  672, 7312,  209053,  7, }, // Pacific/Tahiti
    {  672, 6712,  220881,  7, }, // Pacific/Tarawa
    {  672, 4583,  220888, 10, }, // Pacific/Tongatapu
    {  672, 7392,  220898,  4, }, // Pacific/Wake
    {  672, 7405,  177423,  5, }, // Pacific/Wallis
    {  673, 6788,   29832,  7, }, // Africa/Abidjan Kangri/Devanagari/India
    {  673, 6760,   88123,  5, }, // Africa/Accra
    {  673, 5059,   88128, 10, }, // Africa/Addis_Ababa
    {  673, 7420,   88138,  9, }, // Africa/Algiers
    {  673,   14,   88147,  7, }, // Africa/Asmara
    {  673,   44,  128840,  5, }, // Africa/Bamako
    {  673, 5257,   88160,  6, }, // Africa/Bangui
    {  673, 6746,   88166,  6, }, // Africa/Banjul
    {  673, 7435,   88172,  5, }, // Africa/Bissau
    {  673, 4957,   88177,  9, }, // Africa/Blantyre
    {  673, 5287,   88186, 11, }, // Africa/Brazzaville
    {  673, 4922,   88197,  9, }, // Africa/Bujumbura
    {  673, 2320,   88206,  5, }, // Africa/Cairo
    {  673, 4117,   29933, 11, }, // Africa/Casablanca
    {  673, 7449,   88211,  5, }, // Africa/Ceuta
    {  673, 6773,   88216,  8, }, // Africa/Conakry
    {  673, 6840,   88224,  4, }, // Africa/Dakar
    {  673, 5130,   88228, 11, }, // Africa/Dar_es_Salaam
    {  673, 5043,   88239,  6, }, // Africa/Djibouti
    {  673, 5243,   88245,  5, }, // Africa/Douala
    {  673, 5166,  220902,  9, }, // Africa/El_Aaiun
    {  673, 6853,   88258,  9, }, // Africa/Freetown
    {  673, 4906,   88267,  7, }, // Africa/Gaborone
    {  673, 5015,   88274,  5, }, // Africa/Harare
    {  673, 4479,   88279, 10, }, // Africa/Johannesburg
    {  673, 4499,   30022,  4, }, // Africa/Juba
    {  673, 5151,   88289,  6, }, // Africa/Kampala
    {  673, 4524,   30034,  7, }, // Africa/Khartoum
    {  673, 4973,   88295,  6, }, // Africa/Kigali
    {  673, 5306,   88301,  7, }, // Africa/Kinshasa
    {  673, 4744,   88308,  5, }, // Africa/Lagos
    {  673, 5336,   88313, 10, }, // Africa/Libreville
    {  673, 6869,   88323,  3, }, // Africa/Lome
    {  673, 5211,   88326,  6, }, // Africa/Luanda
    {  673, 4939,   88332,  9, }, // Africa/Lubumbashi
    {  673, 5001,   30089,  6, }, // Africa/Lusaka
    {  673, 5322,   88341,  5, }, // Africa/Malabo
    {  673, 4987,   88346,  6, }, // Africa/Maputo
    {  673, 5197,   30107,  6, }, // Africa/Maseru
    {  673, 5182,   88352,  7, }, // Africa/Mbabane
    {  673, 5113,   88359,  8, }, // Africa/Mogadishu
    {  673, 7034,   88367,  9, }, // Africa/Monrovia
    {  673, 3853,   88376,  6, }, // Africa/Nairobi
    {  673, 5271,   88382, 10, }, // Africa/Ndjamena
    {  673, 5354,   88392,  6, }, // Africa/Niamey
    {  673, 6803,   88398,  7, }, // Africa/Nouakchott
    {  673, 6727,   88405,  7, }, // Africa/Ouagadougou
    {  673, 5225,   88412, 11, }, // Africa/Porto-Novo
    {  673, 4435,   88423,  7, }, // Africa/Sao_Tome
    {  673, 2866,   88430,  8, }, // Africa/Tripoli
    {  673, 6554,   30196,  7, }, // Africa/Tunis
    {  673, 4152,   88438,  7, }, // Africa/Windhoek
    {  673,  132,   88445,  3, }, // America/Adak
    {  673, 3346,   30215,  6, }, // America/Anchorage
    {  673, 5607,   88448,  8, }, // America/Anguilla
    {  673, 5624,   88456,  7, }, // America/Antigua
    {  673, 4554,   88463,  9, }, // America/Araguaina
    {  673,  166,   88472, 11, }, // America/Argentina/Buenos_Aires
    {  673,   91,   30258, 10, }, // America/Argentina/Catamarca
    {  673,  270,   88483,  8, }, // America/Argentina/Cordoba
    {  673,  441,   88491,  5, }, // America/Argentina/Jujuy
    {  673, 7462,   88496,  8, }, // America/Argentina/La_Rioja
    {  673,  565,   88504,  8, }, // America/Argentina/Mendoza
    {  673, 7489,   88512, 12, }, // America/Argentina/Rio_Gallegos
    {  673, 7520,   30305,  6, }, // America/Argentina/Salta
    {  673, 7544,   88524,  9, }, // America/Argentina/San_Juan
    {  673, 5580,   88533,  7, }, // America/Argentina/San_Luis
    {  673, 7571,   88540,  6, }, // America/Argentina/Tucuman
    {  673, 7597,   88546,  7, }, // America/Argentina/Ushuaia
    {  673, 5640,   88553,  5, }, // America/Aruba
    {  673, 4252,   88558,  8, }, // America/Asuncion
    {  673,  237,   88566,  8, }, // America/Atikokan
    {  673, 3679,   88574,  5, }, // America/Bahia
    {  673, 7623,   88579, 15, }, // America/Bahia_Banderas
    {  673, 5654,   88594,  8, }, // America/Barbados
    {  673, 7646,   30394,  5, }, // America/Belem
    {  673, 5407,   88602,  6, }, // America/Belize
    {  673, 7660,   88608, 13, }, // America/Blanc-Sablon
    {  673, 7681,   88621, 10, }, // America/Boa_Vista
    {  673, 4361,   88631,  6, }, // America/Bogota
    {  673, 7699,   88637,  5, }, // America/Boise
    {  673, 7713,   88642, 15, }, // America/Cambridge_Bay
    {  673, 7735,   88657, 13, }, // America/Campo_Grande
    {  673, 3868,   88670,  6, }, // America/Cancun
    {  673, 4694,   88676,  6, }, // America/Caracas
    {  673, 4345,   30477,  5, }, // America/Cayenne
    {  673, 5496,   30482,  5, }, // America/Cayman
    {  673, 2260,   88682,  6, }, // America/Chicago
    {  673, 7756,   88688,  8, }, // America/Chihuahua
    {  673, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  673, 5422,   88712, 11, }, // America/Costa_Rica
    {  673, 7796,   30510,  8, }, // America/Creston
    {  673, 3791,   88723,  7, }, // America/Cuiaba
    {  673, 5723,   88730,  7, }, // America/Curacao
    {  673, 7812,   88737, 11, }, // America/Danmarkshavn
    {  673, 7833,   88748,  4, }, // America/Dawson
    {  673, 7848,   88752, 10, }, // America/Dawson_Creek
    {  673,  805,   88762,  5, }, // America/Denver
    {  673, 3465,   88767,  8, }, // America/Detroit
    {  673, 5739,   88775,  8, }, // America/Dominica
    {  673,  893,   88783,  6, }, // America/Edmonton
    {  673, 7869,   30591,  7, }, // America/Eirunepe
    {  673, 5441,   88789, 11, }, // America/El_Salvador
    {  673, 7886,   88800, 13, }, // America/Fort_Nelson
    {  673, 7906,   88813, 12, }, // America/Fortaleza
    {  673, 7924,   88825, 11, }, // America/Glace_Bay
    {  673, 6881,   88836,  9, }, // America/Goose_Bay
    {  673, 4612,   88845, 11, }, // America/Grand_Turk
    {  673, 5770,   30657,  8, }, // America/Grenada
    {  673, 5786,   88856,  9, }, // America/Guadeloupe
    {  673, 3760,   30674, 10, }, // America/Guatemala
    {  673, 4841,   88865,  9, }, // America/Guayaquil
    {  673, 6932,   30692,  6, }, // America/Guyana
    {  673, 1939,   88879, 10, }, // America/Halifax
    {  673, 2281,   88889,  5, }, // America/Havana
    {  673, 7942,   88894, 11, }, // America/Hermosillo
    {  673,  348,   88905, 13, }, // America/Indiana/Indianapolis
    {  673,  481,   88918, 15, }, // America/Indiana/Knox
    {  673, 7961,   88933, 17, }, // America/Indiana/Marengo
    {  673, 7985,   88950, 20, }, // America/Indiana/Petersburg
    {  673, 8012,   88970, 18, }, // America/Indiana/Tell_City
    {  673, 8038,   88988, 14, }, // America/Indiana/Vevay
    {  673, 8060,   89002, 18, }, // America/Indiana/Vincennes
    {  673, 8086,   89020, 17, }, // America/Indiana/Winamac
    {  673, 8110,   30867,  6, }, // America/Inuvik
    {  673,  660,   89037,  7, }, // America/Iqaluit
    {  673, 2799,   89044,  5, }, // America/Jamaica
    {  673, 5380,   89049,  7, }, // America/Juneau
    {  673,  521,   89056,  8, }, // America/Kentucky/Louisville
    {  673, 8125,   89064, 17, }, // America/Kentucky/Monticello
    {  673, 5704,   89081, 12, }, // America/Kralendijk
    {  673, 4376,   89093,  7, }, // America/La_Paz
    {  673, 7169,   30932,  4, }, // America/Lima
    {  673, 3239,   89100, 11, }, // America/Los_Angeles
    {  673, 5932,   89111, 22, }, // America/Lower_Princes
    {  673, 8153,   89133,  5, }, // America/Maceio
    {  673, 8168,   89138,  7, }, // America/Managua
    {  673, 1908,   89145,  4, }, // America/Manaus
    {  673, 5897,   89149,  7, }, // America/Marigot
    {  673, 5805,   89156,  9, }, // America/Martinique
    {  673, 8184,   89165,  9, }, // America/Matamoros
    {  673, 2917,  220911,  8, }, // America/Mazatlan
    {  673, 8202,   89183,  8, }, // America/Menominee
    {  673, 8220,   31026,  6, }, // America/Merida
    {  673, 8235,   89191, 12, }, // America/Metlakatla
    {  673, 2949,   89203, 13, }, // America/Mexico_City
    {  673, 4391,   89216,  7, }, // America/Miquelon
    {  673, 8254,   89223,  6, }, // America/Moncton
    {  673, 8270,   89229,  9, }, // America/Monterrey
    {  673, 4098,   89238, 11, }, // America/Montevideo
    {  673, 5824,   89249,  9, }, // America/Montserrat
    {  673, 5481,   89258,  5, }, // America/Nassau
    {  673, 2379,   89263,  9, }, // America/New_York
    {  673, 8288,   89185,  3, }, // America/Nome
    {  673, 1850,   89272,  8, }, // America/Noronha
    {  673, 8301,   89280, 20, }, // America/North_Dakota/Beulah
    {  673, 8329,   89300, 19, }, // America/North_Dakota/Center
    {  673, 8357,   89319, 24, }, // America/North_Dakota/New_Salem
    {  673,  393,   89343,  3, }, // America/Nuuk
    {  673, 8388,   89346,  9, }, // America/Ojinaga
    {  673, 2356,   89355,  5, }, // America/Panama
    {  673, 6173,   89360, 10, }, // America/Paramaribo
    {  673, 2973,   89370,  8, }, // America/Phoenix
    {  673, 3945,   89378, 14, }, // America/Port-au-Prince
    {  673, 5954,   89392, 15, }, // America/Port_of_Spain
    {  673, 8404,   89407, 13, }, // America/Porto_Velho
    {  673, 5843,   89420, 11, }, // America/Puerto_Rico
    {  673, 4030,   89431, 12, }, // America/Punta_Arenas
    {  673, 8424,   89443, 12, }, // America/Rankin_Inlet
    {  673, 8445,   89455,  7, }, // America/Recife
    {  673, 1995,   31299,  6, }, // America/Regina
    {  673, 8460,   89462,  9, }, // America/Resolute
    {  673,  695,   89471, 12, }, // America/Rio_Branco
    {  673, 8477,   89483,  8, }, // America/Santarem
    {  673, 2201,   89491,  9, }, // America/Santiago
    {  673, 6111,   89500, 13, }, // America/Santo_Domingo
    {  673, 1878,   89513,  9, }, // America/Sao_Paulo
    {  673, 6899,   89522, 13, }, // America/Scoresbysund
    {  673, 8494,   89535,  6, }, // America/Sitka
    {  673, 8508,   89541, 15, }, // America/St_Barthelemy
    {  673, 2061,   89556,  9, }, // America/St_Johns
    {  673, 5863,   89565, 10, }, // America/St_Kitts
    {  673, 5880,   89575, 11, }, // America/St_Lucia
    {  673,  855,   89586,  9, }, // America/St_Thomas
    {  673, 5913,   89595, 12, }, // America/St_Vincent
    {  673, 8530,   89607, 13, }, // America/Swift_Current
    {  673, 5461,   89620, 11, }, // America/Tegucigalpa
    {  673, 5756,   89631,  6, }, // America/Thule
    {  673,  313,   89637,  7, }, // America/Tijuana
    {  673,  608,   89644,  6, }, // America/Toronto
    {  673, 5688,   89650,  8, }, // America/Tortola
    {  673, 2093,   89658,  7, }, // America/Vancouver
    {  673, 2144,   89665, 11, }, // America/Whitehorse
    {  673,  734,   89676,  7, }, // America/Winnipeg
    {  673, 8552,   89683,  7, }, // America/Yakutat
    {  673, 6001,   31541,  4, }, // Antarctica/Casey
    {  673, 6094,   89690,  5, }, // Antarctica/Davis
    {  673, 6133,   89695, 19, }, // Antarctica/DumontDUrville
    {  673, 8568,   89714,  7, }, // Antarctica/Macquarie
    {  673, 7066,   89721,  4, }, // Antarctica/Mawson
    {  673, 7113,   89725,  9, }, // Antarctica/McMurdo
    {  673, 8589,   89734,  4, }, // Antarctica/Palmer
    {  673, 7226,   89738,  6, }, // Antarctica/Rothera
    {  673, 7295,   89744,  6, }, // Antarctica/Syowa
    {  673, 8607,   89750,  5, }, // Antarctica/Troll
    {  673, 7374,   89755,  7, }, // Antarctica/Vostok
    {  673, 1429,   89762, 13, }, // Arctic/Longyearbyen
    {  673, 5570,  129813,  3, }, // Asia/Aden
    {  673, 5395,   89779,  7, }, // Asia/Almaty
    {  673, 3968,   89786,  6, }, // Asia/Amman
    {  673, 5511,   89792,  6, }, // Asia/Anadyr
    {  673, 5523,   31646,  6, }, // Asia/Aqtau
    {  673, 5368,   89798,  6, }, // Asia/Aqtobe
    {  673,  964,   31659,  8, }, // Asia/Ashgabat
    {  673, 8624,   89804,  5, }, // Asia/Atyrau
    {  673, 3607,   89809,  5, }, // Asia/Baghdad
    {  673, 5534,   89814,  5, }, // Asia/Bahrain
    {  673, 3653,   88728,  4, }, // Asia/Baku
    {  673, 4466,   89819,  6, }, // Asia/Bangkok
    {  673, 3571,   89825,  6, }, // Asia/Barnaul
    {  673, 4086,   89831,  5, }, // Asia/Beirut
    {  673, 3778,   31704,  7, }, // Asia/Bishkek
    {  673, 5989,   89836,  7, }, // Asia/Brunei
    {  673, 4601,   89843,  6, }, // Asia/Chita
    {  673, 4511,   89849,  6, }, // Asia/Colombo
    {  673, 4540,   89855,  7, }, // Asia/Damascus
    {  673, 1093,   89862,  4, }, // Asia/Dhaka
    {  673, 6192,   89866,  4, }, // Asia/Dili
    {  673, 3596,  220919,  3, }, // Asia/Dubai
    {  673, 6159,   89874,  7, }, // Asia/Dushanbe
    {  673, 8636,   89881, 11, }, // Asia/Famagusta
    {  673, 8651,   89892,  5, }, // Asia/Gaza
    {  673, 4795,   89897,  7, }, // Asia/Hebron
    {  673, 1259,  139344, 15, }, // Asia/Ho_Chi_Minh
    {  673, 2704,   89920,  9, }, // Asia/Hong_Kong
    {  673, 4771,   89929,  5, }, // Asia/Hovd
    {  673, 4184,   89934, 10, }, // Asia/Irkutsk
    {  673, 7006,   89944,  7, }, // Asia/Jakarta
    {  673, 6992,   31811,  7, }, // Asia/Jayapura
    {  673, 1290,   89951,  6, }, // Asia/Jerusalem
    {  673, 3560,  220922,  4, }, // Asia/Kabul
    {  673, 4316,   89962,  7, }, // Asia/Kamchatka
    {  673, 4239,   31840,  5, }, // Asia/Karachi
    {  673, 1185,   89969,  8, }, // Asia/Kathmandu
    {  673, 8661,   89977,  7, }, // Asia/Khandyga
    {  673,  992,   31862,  6, }, // Asia/Kolkata
    {  673, 4197,   89991, 15, }, // Asia/Krasnoyarsk
    {  673, 4859,   90006, 10, }, // Asia/Kuala_Lumpur
    {  673, 5976,   31896,  6, }, // Asia/Kuching
    {  673, 5547,   31902,  5, }, // Asia/Kuwait
    {  673, 1211,   90016,  4, }, // Asia/Macau
    {  673, 4017,   31912,  7, }, // Asia/Magadan
    {  673, 1349,   90020,  6, }, // Asia/Makassar
    {  673, 7182,   31928,  5, }, // Asia/Manila
    {  673, 6920,   90026,  5, }, // Asia/Muscat
    {  673, 2554,   90031,  8, }, // Asia/Nicosia
    {  673, 8675,   90039, 16, }, // Asia/Novokuznetsk
    {  673, 4135,   90055, 13, }, // Asia/Novosibirsk
    {  673, 4229,   90068,  6, }, // Asia/Omsk
    {  673, 7145,   90074,  3, }, // Asia/Oral
    {  673, 6961,  114731,  9, }, // Asia/Phnom_Penh
    {  673, 8693,   90086,  9, }, // Asia/Pontianak
    {  673, 4214,   90095, 10, }, // Asia/Pyongyang
    {  673, 5559,   90105,  3, }, // Asia/Qatar
    {  673, 8708,   90108,  8, }, // Asia/Qostanay
    {  673, 4269,   90116, 11, }, // Asia/Qyzylorda
    {  673, 3584,   90127,  5, }, // Asia/Riyadh
    {  673, 4408,   90132,  6, }, // Asia/Sakhalin
    {  673, 7245,   90138,  6, }, // Asia/Samarkand
    {  673, 3279,   90144,  4, }, // Asia/Seoul
    {  673, 1053,   90148,  5, }, // Asia/Shanghai
    {  673, 3300,   32058,  8, }, // Asia/Singapore
    {  673, 4297,   90153, 16, }, // Asia/Srednekolymsk
    {  673, 3263,  130041,  6, }, // Asia/Taipei
    {  673, 4781,   90175,  6, }, // Asia/Tashkent
    {  673, 3915,   90181,  7, }, // Asia/Tbilisi
    {  673, 2772,   32104,  6, }, // Asia/Tehran
    {  673, 1317,   90188,  5, }, // Asia/Thimphu
    {  673, 2821,   90193,  6, }, // Asia/Tokyo
    {  673, 4572,   90199,  7, }, // Asia/Tomsk
    {  673, 1021,   90206,  8, }, // Asia/Ulaanbaatar
    {  673, 1159,   90214,  7, }, // Asia/Urumqi
    {  673, 8722,   90221, 10, }, // Asia/Ust-Nera
    {  673, 6977,   90231,  9, }, // Asia/Vientiane
    {  673, 4710,   90240, 13, }, // Asia/Vladivostok
    {  673, 4828,   32173,  9, }, // Asia/Yakutsk
    {  673, 1235,   32182,  5, }, // Asia/Yangon
    {  673, 3883,   90253, 13, }, // Asia/Yekaterinburg
    {  673, 3747,   90266,  7, }, // Asia/Yerevan
    {  673, 3663,   90273,  7, }, // Atlantic/Azores
    {  673, 5671,   90280,  6, }, // Atlantic/Bermuda
    {  673, 6628,   90286,  6, }, // Atlantic/Canary
    {  673, 3727,   90292,  8, }, // Atlantic/Cape_Verde
    {  673, 1395,   90300,  5, }, // Atlantic/Faroe
    {  673, 8736,   90305,  5, }, // Atlantic/Madeira
    {  673, 2748,   90310,  9, }, // Atlantic/Reykjavik
    {  673, 7272,  220926, 13, }, // Atlantic/South_Georgia
    {  673, 6821,   90334, 11, }, // Atlantic/St_Helena
    {  673, 6644,   90345,  7, }, // Atlantic/Stanley
    {  673, 1670,   90352,  6, }, // Australia/Adelaide
    {  673, 1635,   90358,  8, }, // Australia/Brisbane
    {  673, 1799,   90366, 10, }, // Australia/Broken_Hill
    {  673, 1583,   90376,  7, }, // Australia/Darwin
    {  673, 3637,   32320,  6, }, // Australia/Eucla
    {  673, 1516,   90383,  7, }, // Australia/Hobart
    {  673, 8753,   90390,  8, }, // Australia/Lindeman
    {  673, 1547,   90398, 10, }, // Australia/Lord_Howe
    {  673, 1727,   90408,  8, }, // Australia/Melbourne
    {  673, 1762,   90416,  4, }, // Australia/Perth
    {  673, 1463,   32362,  5, }, // Australia/Sydney
    {  673, 6441,  220939,  9, }, // Europe/Amsterdam
    {  673, 6216,   90430,  6, }, // Europe/Andorra
    {  673, 3620,   90436, 10, }, // Europe/Astrakhan
    {  673, 2300,   90446,  5, }, // Europe/Athens
    {  673, 6336,   32400,  8, }, // Europe/Belgrade
    {  673, 4757,   90451,  6, }, // Europe/Berlin
    {  673, 6488,   90457, 12, }, // Europe/Bratislava
    {  673, 2167,   90469,  9, }, // Europe/Brussels
    {  673, 3928,   90478, 10, }, // Europe/Bucharest
    {  673, 3806,   90488,  9, }, // Europe/Budapest
    {  673, 8772,   90497, 10, }, // Europe/Busingen
    {  673, 2583,   90507,  7, }, // Europe/Chisinau
    {  673, 6289,   90514,  9, }, // Europe/Copenhagen
    {  673, 2338,   90523,  5, }, // Europe/Dublin
    {  673, 6307,   90528, 10, }, // Europe/Gibraltar
    {  673, 8788,   90538,  6, }, // Europe/Guernsey
    {  673, 6612,   90544,  8, }, // Europe/Helsinki
    {  673, 8804,   90552, 11, }, // Europe/Isle_of_Man
    {  673, 1130,   90563,  9, }, // Europe/Istanbul
    {  673, 8823,   90572,  5, }, // Europe/Jersey
    {  673, 3979,   90577, 12, }, // Europe/Kaliningrad
    {  673, 8929,   90589,  5, }, // Europe/Kirov
    {  673, 2527,   90594,  3, }, // Europe/Kyiv
    {  673, 3213,   32558,  6, }, // Europe/Lisbon
    {  673, 6506,   90597, 11, }, // Europe/Ljubljana
    {  673, 2501,   90608,  4, }, // Europe/London
    {  673, 6365,   90612, 10, }, // Europe/Luxembourg
    {  673, 4892,   90622,  7, }, // Europe/Madrid
    {  673, 6397,   32599,  6, }, // Europe/Malta
    {  673, 6582,   90629,  9, }, // Europe/Mariehamn
    {  673, 3693,   90638,  6, }, // Europe/Minsk
    {  673, 6410,   90644,  6, }, // Europe/Monaco
    {  673, 3537,   90650,  6, }, // Europe/Moscow
    {  673, 6458,   90656,  5, }, // Europe/Oslo
    {  673, 4284,  220948,  4, }, // Europe/Paris
    {  673, 6424,   90661, 10, }, // Europe/Podgorica
    {  673, 6275,   90671,  5, }, // Europe/Prague
    {  673, 8837,   90676,  4, }, // Europe/Riga
    {  673, 6324,   90303,  3, }, // Europe/Rome
    {  673, 4331,   90680,  5, }, // Europe/Samara
    {  673, 6470,   90685, 10, }, // Europe/San_Marino
    {  673, 6245,   90695,  8, }, // Europe/Sarajevo
    {  673, 4451,   90703,  7, }, // Europe/Saratov
    {  673, 8849,   90710, 12, }, // Europe/Simferopol
    {  673, 6383,   90722,  8, }, // Europe/Skopje
    {  673, 6599,   90730,  7, }, // Europe/Sofia
    {  673, 6523,   90737,  8, }, // Europe/Stockholm
    {  673, 8867,   90745,  5, }, // Europe/Tallinn
    {  673, 6202,   90750,  6, }, // Europe/Tirane
    {  673, 8882,   90756, 12, }, // Europe/Ulyanovsk
    {  673, 6352,   90768,  6, }, // Europe/Vaduz
    {  673, 6567,   90774,  6, }, // Europe/Vatican
    {  673, 6231,   90780,  5, }, // Europe/Vienna
    {  673, 8899,   90785,  8, }, // Europe/Vilnius
    {  673, 4727,   90793, 11, }, // Europe/Volgograd
    {  673, 3190,   90804,  5, }, // Europe/Warsaw
    {  673, 6261,   90809,  8, }, // Europe/Zagreb
    {  673, 6540,   90817,  9, }, // Europe/Zurich
    {  673, 5078,   90826, 11, }, // Indian/Antananarivo
    {  673, 6947,   90837,  5, }, // Indian/Chagos
    {  673, 6046,  220952,  8, }, // Indian/Christmas
    {  673, 6063,   90849,  5, }, // Indian/Cocos
    {  673, 5029,   90854,  6, }, // Indian/Comoro
    {  673, 6661,   90860,  7, }, // Indian/Kerguelen
    {  673, 7260,   32851,  4, }, // Indian/Mahe
    {  673, 7050,   90867,  6, }, // Indian/Maldives
    {  673, 4069,   90873,  6, }, // Indian/Mauritius
    {  673, 5098,   90879,  8, }, // Indian/Mayotte
    {  673, 7211,   90887,  8, }, // Indian/Reunion
    {  673, 4422,   90895,  5, }, // Pacific/Apia
    {  673,  932,   90900,  6, }, // Pacific/Auckland
    {  673, 3706,   90906,  8, }, // Pacific/Bougainville
    {  673, 3015,   90914,  4, }, // Pacific/Chatham
    {  673, 3157,   90918,  2, }, // Pacific/Chuuk
    {  673, 2237,   90920,  5, }, // Pacific/Easter
    {  673, 7360,   90925,  5, }, // Pacific/Efate
    {  673, 7327,   90930,  9, }, // Pacific/Fakaofo
    {  673, 3902,   90939,  5, }, // Pacific/Fiji
    {  673, 7343,   90944, 12, }, // Pacific/Funafuti
    {  673, 6678,   90956,  9, }, // Pacific/Galapagos
    {  673, 6696,   90965,  7, }, // Pacific/Gambier
    {  673, 3822,   90972, 10, }, // Pacific/Guadalcanal
    {  673, 6018,   30980,  4, }, // Pacific/Guam
    {  673, 3049,  220960,  5, }, // Pacific/Kanton
    {  673, 3998,   33000, 10, }, // Pacific/Kiritimati
    {  673, 7019,   90997,  6, }, // Pacific/Kosrae
    {  673, 2842,   91003, 10, }, // Pacific/Kwajalein
    {  673, 4877,   91013,  6, }, // Pacific/Majuro
    {  673, 4051,   91019,  8, }, // Pacific/Marquesas
    {  673, 8914,   91027,  5, }, // Pacific/Midway
    {  673, 7084,   91032,  4, }, // Pacific/Nauru
    {  673, 7132,   91036,  4, }, // Pacific/Niue
    {  673, 4168,   91040,  7, }, // Pacific/Norfolk
    {  673, 7098,   91047,  6, }, // Pacific/Noumea
    {  673, 3126,  220965,  9, }, // Pacific/Pago_Pago
    {  673, 7155,   91062,  4, }, // Pacific/Palau
    {  673, 7194,   33078,  8, }, // Pacific/Pitcairn
    {  673, 3096,   91066,  6, }, // Pacific/Pohnpei
    {  673, 4807,   91072, 14, }, // Pacific/Port_Moresby
    {  673, 6076,   91086,  9, }, // Pacific/Rarotonga
    {  673, 6031,   91095,  6, }, // Pacific/Saipan
    {  673, 7312,   91101,  6, }, // Pacific/Tahiti
    {  673, 6712,   91107,  6, }, // Pacific/Tarawa
    {  673, 4583,   91113,  9, }, // Pacific/Tongatapu
    {  673, 7392,   33136,  3, }, // Pacific/Wake
    {  673, 7405,   33139,  5, }, // Pacific/Wallis
    {  674, 6788,  195372,  6, }, // Africa/Abidjan Venetian/Latin/Italy
    {  674, 6760,   74000,  4, }, // Africa/Accra
    {  674, 5059,   35689, 10, }, // Africa/Addis_Ababa
    {  674, 7420,  220974,  6, }, // Africa/Algiers
    {  674,   14,  220980,  6, }, // Africa/Asmara
    {  674, 5257,  220986,  6, }, // Africa/Bangui
    {  674, 7435,  220992,  5, }, // Africa/Bissau
    {  674, 4922,  220997,  9, }, // Africa/Bujumbura
    {  674, 2320,  161748,  8, }, // Africa/Cairo
    {  674, 4117,  221006, 10, }, // Africa/Casablanca
    {  674, 7449,  166652,  5, }, // Africa/Ceuta
    {  674, 6773,  157847,  7, }, // Africa/Conakry
    {  674, 5130,   21236, 12, }, // Africa/Dar_es_Salaam
    {  674, 5043,  158372,  6, }, // Africa/Djibouti
    {  674, 5243,   21254,  5, }, // Africa/Douala
    {  674, 5166,    6275,  7, }, // Africa/El_Aaiun
    {  674, 5151,  221016,  7, }, // Africa/Kampala
    {  674, 4524,  157865,  6, }, // Africa/Khartoum
    {  674, 6869,   20300,  4, }, // Africa/Lome
    {  674, 4939,  221023, 10, }, // Africa/Lubumbashi
    {  674, 5197,  221033,  6, }, // Africa/Maseru
    {  674, 5182,  221039,  7, }, // Africa/Mbabane
    {  674, 5113,  221046,  8, }, // Africa/Mogadishu
    {  674, 5271,  195446,  6, }, // Africa/Ndjamena
    {  674, 6803,  147595,  8, }, // Africa/Nouakchott
    {  674, 6727,  221054,  8, }, // Africa/Ouagadougou
    {  674, 5225,   26410, 10, }, // Africa/Porto-Novo
    {  674, 4435,  221062, 15, }, // Africa/Sao_Tome
    {  674, 2866,  164357,  7, }, // Africa/Tripoli
    {  674, 6554,  221077,  6, }, // Africa/Tunis
    {  674, 5607,    4657,  7, }, // America/Anguilla
    {  674,  270,  221083,  7, }, // America/Argentina/Cordoba
    {  674, 7489,  221090, 11, }, // America/Argentina/Rio_Gallegos
    {  674, 4252,    6474,  8, }, // America/Asuncion
    {  674,  237,      22,  8, }, // America/Atikokan
    {  674, 7623,  221101, 16, }, // America/Bahia_Banderas
    {  674, 4361,   44029,  6, }, // America/Bogota
    {  674, 7735,  221117, 12, }, // America/Campo_Grande
    {  674, 3868,   33182,  6, }, // America/Cancun
    {  674, 4345,  147629,  6, }, // America/Cayenne
    {  674, 5496,   44041,  6, }, // America/Cayman
    {  674, 7774,      60,  6, }, // America/Ciudad_Juarez
    {  674, 3791,  221129,  6, }, // America/Cuiaba
    {  674, 5723,      66,  7, }, // America/Curacao
    {  674,  805,  221135,  6, }, // America/Denver
    {  674, 5739,  221141,  8, }, // America/Dominica
    {  674, 7869,   26448,  8, }, // America/Eirunepe
    {  674, 5770,   20401,  7, }, // America/Grenada
    {  674, 5786,   94980,  9, }, // America/Guadeloupe
    {  674, 2281,   94989,  7, }, // America/Havana
    {  674,  348,  221149, 12, }, // America/Indiana/Indianapolis
    {  674,  481,  221161, 14, }, // America/Indiana/Knox
    {  674, 7961,  221175, 17, }, // America/Indiana/Marengo
    {  674, 7985,  221192, 20, }, // America/Indiana/Petersburg
    {  674, 8012,  221212, 19, }, // America/Indiana/Tell_City
    {  674, 8038,  221231, 15, }, // America/Indiana/Vevay
    {  674, 8060,  221246, 19, }, // America/Indiana/Vincennes
    {  674, 8086,  221265, 17, }, // America/Indiana/Winamac
    {  674, 2799,  221282,  7, }, // America/Jamaica
    {  674, 8125,  221289, 21, }, // America/Kentucky/Monticello
    {  674, 3239,  221310, 11, }, // America/Los_Angeles
    {  674, 5932,     207, 22, }, // America/Lower_Princes
    {  674, 8153,   26456,  6, }, // America/Maceio
    {  674, 5805,  221321,  9, }, // America/Martinique
    {  674, 8220,     229,  6, }, // America/Merida
    {  674, 2949,  221330, 16, }, // America/Mexico_City
    {  674, 4391,  221346,  8, }, // America/Miquelon
    {  674, 8270,   21826,  8, }, // America/Monterrey
    {  674, 5824,    7054,  9, }, // America/Montserrat
    {  674, 1850,   61820, 19, }, // America/Noronha
    {  674, 8301,  221354, 20, }, // America/North_Dakota/Beulah
    {  674, 8329,  221374, 20, }, // America/North_Dakota/Center
    {  674, 8357,  221394, 23, }, // America/North_Dakota/New_Salem
    {  674,  393,     298,  4, }, // America/Nuuk
    {  674, 2356,  164484,  6, }, // America/Panama
    {  674, 3945,  221417, 14, }, // America/Port-au-Prince
    {  674, 5954,  221431, 15, }, // America/Port_of_Spain
    {  674, 5843,   95093,  9, }, // America/Puerto_Rico
    {  674, 1878,  221446,  8, }, // America/Sao_Paulo
    {  674, 6899,     302, 16, }, // America/Scoresbysund
    {  674, 8508,  221454, 13, }, // America/St_Barthelemy
    {  674, 2061,  221467,  8, }, // America/St_Johns
    {  674, 5863,  221475, 12, }, // America/St_Kitts
    {  674, 5880,  221487,  8, }, // America/St_Lucia
    {  674,  855,  221068,  9, }, // America/St_Thomas
    {  674, 5913,  221495, 11, }, // America/St_Vincent
    {  674, 5688,  147871,  7, }, // America/Tortola
    {  674, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  674, 8568,  221506, 15, }, // Antarctica/Macquarie
    {  674, 5395,    7476,  6, }, // Asia/Almaty
    {  674, 3968,    5487,  4, }, // Asia/Amman
    {  674, 5511,  221521,  6, }, // Asia/Anadyr
    {  674, 5523,  221527,  5, }, // Asia/Aqtau
    {  674, 5368,    7488,  6, }, // Asia/Aqtobe
    {  674,  964,  221532,  7, }, // Asia/Ashgabat
    {  674, 8624,  221539,  6, }, // Asia/Atyrau
    {  674, 3607,    4044,  6, }, // Asia/Baghdad
    {  674, 5534,   63842,  6, }, // Asia/Bahrain
    {  674, 3778,   91407,  6, }, // Asia/Bishkek
    {  674, 5989,   20633,  6, }, // Asia/Brunei
    {  674, 4601,   93125,  4, }, // Asia/Chita
    {  674, 4511,  221545,  7, }, // Asia/Colombo
    {  674, 4540,   74309,  7, }, // Asia/Damascus
    {  674, 1093,  158367,  4, }, // Asia/Dhaka
    {  674, 3596,  221552,  5, }, // Asia/Dubai
    {  674, 6159,  221557,  7, }, // Asia/Dushanbe
    {  674, 8636,   95129,  9, }, // Asia/Famagusta
    {  674, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  674, 1290,  123430, 10, }, // Asia/Jerusalem
    {  674, 4316,  162051,  9, }, // Asia/Kamchatka
    {  674, 1185,  221564,  8, }, // Asia/Kathmandu
    {  674, 8661,  221572,  7, }, // Asia/Khandyga
    {  674,  992,   20720,  7, }, // Asia/Kolkata
    {  674, 4197,  221579, 11, }, // Asia/Krasnoyarsk
    {  674, 4859,  221590, 12, }, // Asia/Kuala_Lumpur
    {  674, 1211,  221602,  5, }, // Asia/Macau
    {  674, 1349,    7618,  7, }, // Asia/Makassar
    {  674, 6920,   20739,  7, }, // Asia/Muscat
    {  674, 8675,  221607, 13, }, // Asia/Novokuznetsk
    {  674, 7145,   65127,  5, }, // Asia/Oral
    {  674, 6961,   22386,  8, }, // Asia/Phnom_Penh
    {  674, 5559,   73021,  5, }, // Asia/Qatar
    {  674, 8708,  221620,  8, }, // Asia/Qostanay
    {  674, 4269,    7660,  9, }, // Asia/Qyzylorda
    {  674, 3584,    7669,  4, }, // Asia/Riyadh
    {  674, 4408,  158153,  7, }, // Asia/Sakhalin
    {  674, 7245,   20768, 10, }, // Asia/Samarkand
    {  674, 3279,    7680,  4, }, // Asia/Seoul
    {  674, 1053,  221628,  8, }, // Asia/Shanghai
    {  674, 4297,  221636, 13, }, // Asia/Srednekolymsk
    {  674, 3263,  177554,  6, }, // Asia/Taipei
    {  674, 4781,   91626,  7, }, // Asia/Tashkent
    {  674, 3915,  219985,  7, }, // Asia/Tbilisi
    {  674, 2772,    7724,  7, }, // Asia/Tehran
    {  674, 1317,  221649,  5, }, // Asia/Thimphu
    {  674, 1021,  221654, 10, }, // Asia/Ulaanbaatar
    {  674, 1159,  201897,  7, }, // Asia/Urumqi
    {  674, 8722,  221664,  9, }, // Asia/Ust-Nera
    {  674, 6977,   26807,  8, }, // Asia/Vientiane
    {  674, 1235,     442,  6, }, // Asia/Yangon
    {  674, 3883,  221673, 14, }, // Asia/Yekaterinburg
    {  674, 3747,  221687,  7, }, // Asia/Yerevan
    {  674, 3663,  221694, 11, }, // Atlantic/Azores
    {  674, 6628,  221705, 13, }, // Atlantic/Canary
    {  674, 3727,  221718,  9, }, // Atlantic/Cape_Verde
    {  674, 1395,  221727, 11, }, // Atlantic/Faroe
    {  674, 8736,  221738, 13, }, // Atlantic/Madeira
    {  674, 2748,  221751,  8, }, // Atlantic/Reykjavik
    {  674, 7272,  221759, 16, }, // Atlantic/South_Georgia
    {  674, 6821,  221775, 14, }, // Atlantic/St_Helena
    {  674, 1670,  221789,  8, }, // Australia/Adelaide
    {  674, 1635,  221797,  8, }, // Australia/Brisbane
    {  674, 1547,  221805, 15, }, // Australia/Lord_Howe
    {  674, 6441,  221820,  9, }, // Europe/Amsterdam
    {  674, 6216,   33506,  6, }, // Europe/Andorra
    {  674, 3620,   33512,  8, }, // Europe/Astrakhan
    {  674, 2300,    5793,  5, }, // Europe/Athens
    {  674, 6336,   58774,  8, }, // Europe/Belgrade
    {  674, 6488,  221829, 10, }, // Europe/Bratislava
    {  674, 2167,   26890,  6, }, // Europe/Brussels
    {  674, 3928,   20955,  8, }, // Europe/Bucharest
    {  674, 8772,   26905,  8, }, // Europe/Busingen
    {  674, 6289,   95202, 10, }, // Europe/Copenhagen
    {  674, 6307,  221839,  9, }, // Europe/Gibraltar
    {  674, 8804,  221848, 12, }, // Europe/Isle_of_Man
    {  674, 1130,  164764,  8, }, // Europe/Istanbul
    {  674, 3979,   62893, 12, }, // Europe/Kaliningrad
    {  674, 2527,    8030,  4, }, // Europe/Kyiv
    {  674, 3213,  157592,  7, }, // Europe/Lisbon
    {  674, 6506,  221860,  8, }, // Europe/Ljubljana
    {  674, 2501,   95241,  6, }, // Europe/London
    {  674, 6365,  221868, 10, }, // Europe/Luxembourg
    {  674, 6410,   44438,  6, }, // Europe/Monaco
    {  674, 3537,   95258,  5, }, // Europe/Moscow
    {  674, 6458,  221878,  4, }, // Europe/Oslo
    {  674, 4284,  221882,  6, }, // Europe/Paris
    {  674, 6424,  196699,  9, }, // Europe/Podgorica
    {  674, 6275,   21067,  5, }, // Europe/Prague
    {  674, 6324,    5946,  4, }, // Europe/Rome
    {  674, 6470,   65739,  9, }, // Europe/San_Marino
    {  674, 8849,  221888, 11, }, // Europe/Simferopol
    {  674, 6523,  221899,  8, }, // Europe/Stockholm
    {  674, 8867,    8132,  5, }, // Europe/Tallinn
    {  674, 6202,   21095,  6, }, // Europe/Tirane
    {  674, 8882,  221907,  8, }, // Europe/Ulyanovsk
    {  674, 6567,  221915,  7, }, // Europe/Vatican
    {  674, 6231,   21112,  5, }, // Europe/Vienna
    {  674, 4727,   63074, 10, }, // Europe/Volgograd
    {  674, 3190,   95307,  8, }, // Europe/Warsaw
    {  674, 6540,   95323,  6, }, // Europe/Zurich
    {  674, 6947,  221922,  6, }, // Indian/Chagos
    {  674, 6046,  221928, 15, }, // Indian/Christmas
    {  674, 6063,  221943, 11, }, // Indian/Cocos
    {  674, 5029,  221954, 12, }, // Indian/Comoro
    {  674, 7260,   26983,  4, }, // Indian/Mahe
    {  674, 7050,  221966, 13, }, // Indian/Maldives
    {  674, 4069,  221979, 14, }, // Indian/Mauritius
    {  674, 5098,  221993, 12, }, // Indian/Mayotte
    {  674, 7211,  222005, 13, }, // Indian/Reunion
    {  674, 3706,  222018, 18, }, // Pacific/Bougainville
    {  674, 3015,  222036, 12, }, // Pacific/Chatham
    {  674, 3157,  222048, 11, }, // Pacific/Chuuk
    {  674, 2237,  222059, 15, }, // Pacific/Easter
    {  674, 7360,  222074, 11, }, // Pacific/Efate
    {  674, 7327,  222085, 13, }, // Pacific/Fakaofo
    {  674, 7343,  222098, 14, }, // Pacific/Funafuti
    {  674, 6678,  167203,  9, }, // Pacific/Galapagos
    {  674, 6696,  222112, 13, }, // Pacific/Gambier
    {  674, 3822,  222125, 17, }, // Pacific/Guadalcanal
    {  674, 3049,  222142, 12, }, // Pacific/Kanton
    {  674, 3998,  222154, 16, }, // Pacific/Kiritimati
    {  674, 7019,  222170, 12, }, // Pacific/Kosrae
    {  674, 2842,  222182, 15, }, // Pacific/Kwajalein
    {  674, 4051,  222197, 14, }, // Pacific/Marquesas
    {  674, 8914,  222211, 12, }, // Pacific/Midway
    {  674, 7084,  222223, 11, }, // Pacific/Nauru
    {  674, 7132,  222234, 10, }, // Pacific/Niue
    {  674, 4168,  222244, 13, }, // Pacific/Norfolk
    {  674, 7098,    4510,  6, }, // Pacific/Noumea
    {  674, 3126,  135327, 11, }, // Pacific/Pago_Pago
    {  674, 7155,  222257,  5, }, // Pacific/Palau
    {  674, 7194,  222262, 14, }, // Pacific/Pitcairn
    {  674, 3096,  222276, 12, }, // Pacific/Pohnpei
    {  674, 4807,  222288, 13, }, // Pacific/Port_Moresby
    {  674, 6076,  222301, 15, }, // Pacific/Rarotonga
    {  674, 6031,  222316, 12, }, // Pacific/Saipan
    {  674, 7312,  222328, 11, }, // Pacific/Tahiti
    {  674, 6712,  222339, 13, }, // Pacific/Tarawa
    {  674, 4583,  222352, 15, }, // Pacific/Tongatapu
    {  674, 7392,  222367, 10, }, // Pacific/Wake
    {  674, 7405,  222377, 12, }, // Pacific/Wallis
    {  675, 6788,  222389,  7, }, // Africa/Abidjan Kuvi/Latin/India
    {  675, 6760,  222396,  4, }, // Africa/Accra
    {  675, 5059,  222400, 10, }, // Africa/Addis_Ababa
    {  675, 7420,  222410,  7, }, // Africa/Algiers
    {  675,   14,  222417,  6, }, // Africa/Asmara
    {  675,   44,  222423,  6, }, // Africa/Bamako
    {  675, 5257,  222429,  6, }, // Africa/Bangui
    {  675, 6746,  222435,  6, }, // Africa/Banjul
    {  675, 7435,  222441,  6, }, // Africa/Bissau
    {  675, 4957,  222447,  8, }, // Africa/Blantyre
    {  675, 5287,  222455,  9, }, // Africa/Brazzaville
    {  675, 4922,  222464,  9, }, // Africa/Bujumbura
    {  675, 2320,  222473,  5, }, // Africa/Cairo
    {  675, 4117,  222478, 10, }, // Africa/Casablanca
    {  675, 7449,  222488,  5, }, // Africa/Ceuta
    {  675, 6773,  222493,  7, }, // Africa/Conakry
    {  675, 6840,  222500,  4, }, // Africa/Dakar
    {  675, 5130,  222504, 13, }, // Africa/Dar_es_Salaam
    {  675, 5043,   26361,  6, }, // Africa/Djibouti
    {  675, 5243,  222517,  5, }, // Africa/Douala
    {  675, 5166,  222522,  9, }, // Africa/El_Aaiun
    {  675, 6853,  222531,  7, }, // Africa/Freetown
    {  675, 4906,  222538,  7, }, // Africa/Gaborone
    {  675, 5015,  222545,  5, }, // Africa/Harare
    {  675, 4479,  222550, 12, }, // Africa/Johannesburg
    {  675, 4499,  222562,  4, }, // Africa/Juba
    {  675, 5151,  222566,  6, }, // Africa/Kampala
    {  675, 4524,  222572,  7, }, // Africa/Khartoum
    {  675, 4973,  222579,  6, }, // Africa/Kigali
    {  675, 5306,  222585,  7, }, // Africa/Kinshasa
    {  675, 4744,  222592,  5, }, // Africa/Lagos
    {  675, 5336,  222597,  9, }, // Africa/Libreville
    {  675, 6869,    3969,  3, }, // Africa/Lome
    {  675, 5211,  222606,  6, }, // Africa/Luanda
    {  675, 4939,  222612,  9, }, // Africa/Lubumbashi
    {  675, 5001,  222621,  6, }, // Africa/Lusaka
    {  675, 5322,  222627,  6, }, // Africa/Malabo
    {  675, 4987,  222633,  6, }, // Africa/Maputo
    {  675, 5197,  222639,  6, }, // Africa/Maseru
    {  675, 5182,  222645,  7, }, // Africa/Mbabane
    {  675, 5113,  222652,  8, }, // Africa/Mogadishu
    {  675, 7034,  222660,  8, }, // Africa/Monrovia
    {  675, 3853,  222668,  7, }, // Africa/Nairobi
    {  675, 5271,  222675,  7, }, // Africa/Ndjamena
    {  675, 5354,  222682,  6, }, // Africa/Niamey
    {  675, 6803,  222688,  6, }, // Africa/Nouakchott
    {  675, 6727,  222694,  7, }, // Africa/Ouagadougou
    {  675, 5225,  222701, 10, }, // Africa/Porto-Novo
    {  675, 4435,  222711,  7, }, // Africa/Sao_Tome
    {  675, 2866,  222718,  7, }, // Africa/Tripoli
    {  675, 6554,  222725,  5, }, // Africa/Tunis
    {  675, 4152,  222730,  7, }, // Africa/Windhoek
    {  675,  132,  222737,  4, }, // America/Adak
    {  675, 3346,  222741,  7, }, // America/Anchorage
    {  675, 5607,  222748,  8, }, // America/Anguilla
    {  675, 5624,  222756,  7, }, // America/Antigua
    {  675, 4554,  222763,  9, }, // America/Araguaina
    {  675,  166,  222772, 13, }, // America/Argentina/Buenos_Aires
    {  675,   91,  222785,  9, }, // America/Argentina/Catamarca
    {  675,  270,  222794,  7, }, // America/Argentina/Cordoba
    {  675,  441,  222801,  5, }, // America/Argentina/Jujuy
    {  675, 7462,  222806,  8, }, // America/Argentina/La_Rioja
    {  675,  565,  222814,  7, }, // America/Argentina/Mendoza
    {  675, 7489,  222821, 13, }, // America/Argentina/Rio_Gallegos
    {  675, 7520,  222834,  5, }, // America/Argentina/Salta
    {  675, 7544,  222839,  8, }, // America/Argentina/San_Juan
    {  675, 5580,  222847,  8, }, // America/Argentina/San_Luis
    {  675, 7571,  222855,  6, }, // America/Argentina/Tucuman
    {  675, 7597,  222861,  7, }, // America/Argentina/Ushuaia
    {  675, 5640,  222868,  5, }, // America/Aruba
    {  675, 4252,  222873,  8, }, // America/Asuncion
    {  675,  237,  222881,  8, }, // America/Atikokan
    {  675, 3679,  222889,  5, }, // America/Bahia
    {  675, 7623,  222894, 13, }, // America/Bahia_Banderas
    {  675, 5654,  222907,  8, }, // America/Barbados
    {  675, 7646,  222915,  5, }, // America/Belem
    {  675, 5407,  222920,  5, }, // America/Belize
    {  675, 7660,  222925, 12, }, // America/Blanc-Sablon
    {  675, 7681,  222937,  9, }, // America/Boa_Vista
    {  675, 4361,  222946,  6, }, // America/Bogota
    {  675, 7699,  222952,  5, }, // America/Boise
    {  675, 7713,  222957, 10, }, // America/Cambridge_Bay
    {  675, 7735,  222967, 12, }, // America/Campo_Grande
    {  675, 3868,  222979,  6, }, // America/Cancun
    {  675, 4694,  222985,  7, }, // America/Caracas
    {  675, 4345,  222992,  5, }, // America/Cayenne
    {  675, 5496,  222997,  6, }, // America/Cayman
    {  675, 2260,  223003,  6, }, // America/Chicago
    {  675, 7756,  223009,  9, }, // America/Chihuahua
    {  675, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  675, 5422,  223018, 10, }, // America/Costa_Rica
    {  675, 7796,  223028,  7, }, // America/Creston
    {  675, 3791,  223035,  6, }, // America/Cuiaba
    {  675, 5723,  223041,  7, }, // America/Curacao
    {  675, 7812,  223048, 11, }, // America/Danmarkshavn
    {  675, 7833,  223059,  5, }, // America/Dawson
    {  675, 7848,  223064, 11, }, // America/Dawson_Creek
    {  675,  805,  223075,  6, }, // America/Denver
    {  675, 3465,  223081,  7, }, // America/Detroit
    {  675, 5739,  223088,  8, }, // America/Dominica
    {  675,  893,  223096,  8, }, // America/Edmonton
    {  675, 7869,  223104,  8, }, // America/Eirunepe
    {  675, 5441,  223112, 11, }, // America/El_Salvador
    {  675, 7886,  223123, 11, }, // America/Fort_Nelson
    {  675, 7906,  223134,  9, }, // America/Fortaleza
    {  675, 7924,  223143,  8, }, // America/Glace_Bay
    {  675, 6881,  223151,  7, }, // America/Goose_Bay
    {  675, 4612,  223158, 10, }, // America/Grand_Turk
    {  675, 5770,  223168,  7, }, // America/Grenada
    {  675, 5786,  223175, 10, }, // America/Guadeloupe
    {  675, 3760,  223185,  9, }, // America/Guatemala
    {  675, 4841,  223194,  8, }, // America/Guayaquil
    {  675, 6932,  223202,  6, }, // America/Guyana
    {  675, 1939,  223208,  8, }, // America/Halifax
    {  675, 2281,  223216,  6, }, // America/Havana
    {  675, 7942,    6802,  8, }, // America/Hermosillo
    {  675,  348,  223222, 12, }, // America/Indiana/Indianapolis
    {  675,  481,  223234, 14, }, // America/Indiana/Knox
    {  675, 7961,  223248, 17, }, // America/Indiana/Marengo
    {  675, 7985,  223265, 20, }, // America/Indiana/Petersburg
    {  675, 8012,  223285, 19, }, // America/Indiana/Tell_City
    {  675, 8038,  223304, 14, }, // America/Indiana/Vevay
    {  675, 8060,  223318, 17, }, // America/Indiana/Vincennes
    {  675, 8086,  223335, 17, }, // America/Indiana/Winamac
    {  675, 8110,  223352,  6, }, // America/Inuvik
    {  675,  660,  223358,  7, }, // America/Iqaluit
    {  675, 2799,  223365,  7, }, // America/Jamaica
    {  675, 5380,  223372,  5, }, // America/Juneau
    {  675,  521,  223377,  9, }, // America/Kentucky/Louisville
    {  675, 8125,  223386, 19, }, // America/Kentucky/Monticello
    {  675, 5704,  223405, 10, }, // America/Kralendijk
    {  675, 4376,  223415,  6, }, // America/La_Paz
    {  675, 7169,  157244,  4, }, // America/Lima
    {  675, 3239,  223421, 11, }, // America/Los_Angeles
    {  675, 5932,  223432, 21, }, // America/Lower_Princes
    {  675, 8153,  223453,  5, }, // America/Maceio
    {  675, 8168,  223458,  7, }, // America/Managua
    {  675, 1908,  223465,  6, }, // America/Manaus
    {  675, 5897,  223471,  7, }, // America/Marigot
    {  675, 5805,  223478,  8, }, // America/Martinique
    {  675, 8184,  223486,  9, }, // America/Matamoros
    {  675, 2917,  223495,  8, }, // America/Mazatlan
    {  675, 8202,  223503,  8, }, // America/Menominee
    {  675, 8220,  223511,  6, }, // America/Merida
    {  675, 8235,  223517, 10, }, // America/Metlakatla
    {  675, 2949,  223527, 12, }, // America/Mexico_City
    {  675, 4391,  223539,  7, }, // America/Miquelon
    {  675, 8254,  223546,  7, }, // America/Moncton
    {  675, 8270,  223553,  8, }, // America/Monterrey
    {  675, 4098,  223561, 10, }, // America/Montevideo
    {  675, 5824,  223571, 10, }, // America/Montserrat
    {  675, 5481,  223581,  5, }, // America/Nassau
    {  675, 2379,  223586,  9, }, // America/New_York
    {  675, 8288,    7002,  3, }, // America/Nome
    {  675, 1850,  223595,  7, }, // America/Noronha
    {  675, 8301,  223602, 20, }, // America/North_Dakota/Beulah
    {  675, 8329,  223622, 19, }, // America/North_Dakota/Center
    {  675, 8357,  223641, 23, }, // America/North_Dakota/New_Salem
    {  675,  393,  223664,  3, }, // America/Nuuk
    {  675, 8388,  223667,  7, }, // America/Ojinaga
    {  675, 2356,  223674,  6, }, // America/Panama
    {  675, 6173,  223680, 10, }, // America/Paramaribo
    {  675, 2973,  223690,  6, }, // America/Phoenix
    {  675, 3945,  223696, 14, }, // America/Port-au-Prince
    {  675, 5954,  223710, 13, }, // America/Port_of_Spain
    {  675, 8404,  223723, 11, }, // America/Porto_Velho
    {  675, 5843,  223734, 11, }, // America/Puerto_Rico
    {  675, 4030,  223745, 12, }, // America/Punta_Arenas
    {  675, 8424,  223757, 13, }, // America/Rankin_Inlet
    {  675, 8445,  223770,  6, }, // America/Recife
    {  675, 1995,  223776,  6, }, // America/Regina
    {  675, 8460,  223782,  8, }, // America/Resolute
    {  675,  695,  223790, 10, }, // America/Rio_Branco
    {  675, 8477,  223800,  6, }, // America/Santarem
    {  675, 2201,  223806,  8, }, // America/Santiago
    {  675, 6111,  223814, 13, }, // America/Santo_Domingo
    {  675, 1878,  223827,  9, }, // America/Sao_Paulo
    {  675, 6899,  223836, 11, }, // America/Scoresbysund
    {  675, 8494,  223847,  5, }, // America/Sitka
    {  675, 8508,  223852, 14, }, // America/St_Barthelemy
    {  675, 2061,  223866,  9, }, // America/St_Johns
    {  675, 5863,  223875, 10, }, // America/St_Kitts
    {  675, 5880,  223885, 10, }, // America/St_Lucia
    {  675,  855,  223895, 10, }, // America/St_Thomas
    {  675, 5913,  223905, 12, }, // America/St_Vincent
    {  675, 8530,  223917, 13, }, // America/Swift_Current
    {  675, 5461,  223930,  4, }, // America/Tegucigalpa
    {  675, 5756,  223934,  4, }, // America/Thule
    {  675,  313,  223938,  7, }, // America/Tijuana
    {  675,  608,  223945,  7, }, // America/Toronto
    {  675, 5688,  223952,  7, }, // America/Tortola
    {  675, 2093,  223959,  8, }, // America/Vancouver
    {  675, 2144,  223967,  9, }, // America/Whitehorse
    {  675,  734,  223976,  8, }, // America/Winnipeg
    {  675, 8552,  223984,  7, }, // America/Yakutat
    {  675, 6001,  223991,  5, }, // Antarctica/Casey
    {  675, 6094,  223996,  5, }, // Antarctica/Davis
    {  675, 6133,  224001, 17, }, // Antarctica/DumontDUrville
    {  675, 8568,  224018,  7, }, // Antarctica/Macquarie
    {  675, 7066,  224025,  6, }, // Antarctica/Mawson
    {  675, 7113,  224031,  9, }, // Antarctica/McMurdo
    {  675, 8589,  224040,  6, }, // Antarctica/Palmer
    {  675, 7226,  224046,  6, }, // Antarctica/Rothera
    {  675, 7295,  224052,  5, }, // Antarctica/Syowa
    {  675, 8607,  224057,  5, }, // Antarctica/Troll
    {  675, 7374,  224062,  6, }, // Antarctica/Vostok
    {  675, 1429,  224068, 10, }, // Arctic/Longyearbyen
    {  675, 5570,  224078,  4, }, // Asia/Aden
    {  675, 5395,  224082,  6, }, // Asia/Almaty
    {  675, 3968,  224088,  5, }, // Asia/Amman
    {  675, 5511,  224093,  6, }, // Asia/Anadyr
    {  675, 5523,  224099,  5, }, // Asia/Aqtau
    {  675, 5368,  224104,  7, }, // Asia/Aqtobe
    {  675,  964,  224111,  7, }, // Asia/Ashgabat
    {  675, 8624,  224118,  6, }, // Asia/Atyrau
    {  675, 3607,  224124,  6, }, // Asia/Baghdad
    {  675, 5534,  224130,  7, }, // Asia/Bahrain
    {  675, 3653,  224137,  4, }, // Asia/Baku
    {  675, 4466,  224141,  7, }, // Asia/Bangkok
    {  675, 3571,  224148,  7, }, // Asia/Barnaul
    {  675, 4086,  224155,  6, }, // Asia/Beirut
    {  675, 3778,  224161,  6, }, // Asia/Bishkek
    {  675, 5989,  224167,  6, }, // Asia/Brunei
    {  675, 4601,  224173,  4, }, // Asia/Chita
    {  675, 4511,  224177,  7, }, // Asia/Colombo
    {  675, 4540,  224184,  8, }, // Asia/Damascus
    {  675, 1093,  222738,  4, }, // Asia/Dhaka
    {  675, 6192,  224192,  4, }, // Asia/Dili
    {  675, 3596,  224196,  5, }, // Asia/Dubai
    {  675, 6159,  224201,  7, }, // Asia/Dushanbe
    {  675, 8636,  224208,  9, }, // Asia/Famagusta
    {  675, 8651,  224217,  4, }, // Asia/Gaza
    {  675, 4795,  224221,  6, }, // Asia/Hebron
    {  675, 1259,  224227, 15, }, // Asia/Ho_Chi_Minh
    {  675, 2704,  224242,  9, }, // Asia/Hong_Kong
    {  675, 4771,   26704,  4, }, // Asia/Hovd
    {  675, 4184,  224251,  7, }, // Asia/Irkutsk
    {  675, 7006,    4130,  7, }, // Asia/Jakarta
    {  675, 6992,  224258,  8, }, // Asia/Jayapura
    {  675, 1290,  224266,  9, }, // Asia/Jerusalem
    {  675, 3560,  224275,  5, }, // Asia/Kabul
    {  675, 4316,  224280,  8, }, // Asia/Kamchatka
    {  675, 4239,  224288,  6, }, // Asia/Karachi
    {  675, 1185,  224294,  8, }, // Asia/Kathmandu
    {  675, 8661,  224302,  7, }, // Asia/Khandyga
    {  675,  992,  224309,  7, }, // Asia/Kolkata
    {  675, 4197,  224316, 11, }, // Asia/Krasnoyarsk
    {  675, 4859,  224327, 12, }, // Asia/Kuala_Lumpur
    {  675, 5976,  224339,  6, }, // Asia/Kuching
    {  675, 5547,  224345,  6, }, // Asia/Kuwait
    {  675, 1211,  224351,  4, }, // Asia/Macau
    {  675, 4017,  224355,  7, }, // Asia/Magadan
    {  675, 1349,  224362,  7, }, // Asia/Makassar
    {  675, 7182,  224369,  6, }, // Asia/Manila
    {  675, 6920,  224375,  6, }, // Asia/Muscat
    {  675, 2554,  224381,  8, }, // Asia/Nicosia
    {  675, 8675,  224389, 12, }, // Asia/Novokuznetsk
    {  675, 4135,  224401, 11, }, // Asia/Novosibirsk
    {  675, 4229,  123672,  4, }, // Asia/Omsk
    {  675, 7145,   91137,  4, }, // Asia/Oral
    {  675, 6961,  224412,  9, }, // Asia/Phnom_Penh
    {  675, 8693,  224421,  9, }, // Asia/Pontianak
    {  675, 4214,  224430,  9, }, // Asia/Pyongyang
    {  675, 5559,  210246,  5, }, // Asia/Qatar
    {  675, 8708,  224439,  7, }, // Asia/Qostanay
    {  675, 4269,  224446,  8, }, // Asia/Qyzylorda
    {  675, 3584,  224454,  5, }, // Asia/Riyadh
    {  675, 4408,  224459,  7, }, // Asia/Sakhalin
    {  675, 7245,  224466,  9, }, // Asia/Samarkand
    {  675, 3279,  224475,  4, }, // Asia/Seoul
    {  675, 1053,  224479,  6, }, // Asia/Shanghai
    {  675, 3300,  224485,  8, }, // Asia/Singapore
    {  675, 4297,  224493, 13, }, // Asia/Srednekolymsk
    {  675, 3263,  224506,  6, }, // Asia/Taipei
    {  675, 4781,  224512,  7, }, // Asia/Tashkent
    {  675, 3915,  224519,  7, }, // Asia/Tbilisi
    {  675, 2772,  224526,  6, }, // Asia/Tehran
    {  675, 1317,  224532,  6, }, // Asia/Thimphu
    {  675, 2821,  224538,  5, }, // Asia/Tokyo
    {  675, 4572,  224543,  5, }, // Asia/Tomsk
    {  675, 1021,  224548,  9, }, // Asia/Ulaanbaatar
    {  675, 1159,  224557,  6, }, // Asia/Urumqi
    {  675, 8722,  224563,  9, }, // Asia/Ust-Nera
    {  675, 6977,  224572,  9, }, // Asia/Vientiane
    {  675, 4710,  224581, 11, }, // Asia/Vladivostok
    {  675, 4828,  224592,  7, }, // Asia/Yakutsk
    {  675, 1235,  224599,  6, }, // Asia/Yangon
    {  675, 3883,  224605, 13, }, // Asia/Yekaterinburg
    {  675, 3747,  224618,  7, }, // Asia/Yerevan
    {  675, 3663,  224625,  6, }, // Atlantic/Azores
    {  675, 5671,  224631,  7, }, // Atlantic/Bermuda
    {  675, 6628,  224638,  6, }, // Atlantic/Canary
    {  675, 3727,  224644,  9, }, // Atlantic/Cape_Verde
    {  675, 1395,  224653,  4, }, // Atlantic/Faroe
    {  675, 8736,  224657,  7, }, // Atlantic/Madeira
    {  675, 2748,  224664,  9, }, // Atlantic/Reykjavik
    {  675, 7272,  224673, 11, }, // Atlantic/South_Georgia
    {  675, 6821,  224684, 11, }, // Atlantic/St_Helena
    {  675, 6644,  224695,  6, }, // Atlantic/Stanley
    {  675, 1670,  224701,  8, }, // Australia/Adelaide
    {  675, 1635,  224709,  8, }, // Australia/Brisbane
    {  675, 1799,  224717, 11, }, // Australia/Broken_Hill
    {  675, 1583,  224728,  6, }, // Australia/Darwin
    {  675, 3637,  224734,  5, }, // Australia/Eucla
    {  675, 1516,  224739,  6, }, // Australia/Hobart
    {  675, 8753,  224745,  9, }, // Australia/Lindeman
    {  675, 1547,  224754,  9, }, // Australia/Lord_Howe
    {  675, 1727,  224763,  8, }, // Australia/Melbourne
    {  675, 1762,  224771,  4, }, // Australia/Perth
    {  675, 1463,  224775,  5, }, // Australia/Sydney
    {  675, 6441,  224780,  9, }, // Europe/Amsterdam
    {  675, 6216,  224789,  7, }, // Europe/Andorra
    {  675, 3620,  224796,  9, }, // Europe/Astrakhan
    {  675, 2300,  224805,  5, }, // Europe/Athens
    {  675, 6336,  224810,  8, }, // Europe/Belgrade
    {  675, 4757,  224818,  6, }, // Europe/Berlin
    {  675, 6488,  224824, 10, }, // Europe/Bratislava
    {  675, 2167,  224834,  8, }, // Europe/Brussels
    {  675, 3928,  224842,  8, }, // Europe/Bucharest
    {  675, 3806,  224850,  8, }, // Europe/Budapest
    {  675, 8772,  224858,  8, }, // Europe/Busingen
    {  675, 2583,  224866,  7, }, // Europe/Chisinau
    {  675, 6289,  224873, 10, }, // Europe/Copenhagen
    {  675, 2338,  224883,  5, }, // Europe/Dublin
    {  675, 6307,  224888,  9, }, // Europe/Gibraltar
    {  675, 8788,  224897,  6, }, // Europe/Guernsey
    {  675, 6612,  224903,  8, }, // Europe/Helsinki
    {  675, 8804,  224911, 11, }, // Europe/Isle_of_Man
    {  675, 1130,  224922,  8, }, // Europe/Istanbul
    {  675, 8823,  224930,  5, }, // Europe/Jersey
    {  675, 3979,  224935, 11, }, // Europe/Kaliningrad
    {  675, 8929,  224946,  5, }, // Europe/Kirov
    {  675, 2527,  224951,  4, }, // Europe/Kyiv
    {  675, 3213,  224955,  6, }, // Europe/Lisbon
    {  675, 6506,  224961,  9, }, // Europe/Ljubljana
    {  675, 2501,  224970,  4, }, // Europe/London
    {  675, 6365,  224974,  8, }, // Europe/Luxembourg
    {  675, 4892,  224982,  6, }, // Europe/Madrid
    {  675, 6397,  224988,  5, }, // Europe/Malta
    {  675, 6582,  224993,  8, }, // Europe/Mariehamn
    {  675, 3693,  225001,  5, }, // Europe/Minsk
    {  675, 6410,  225006,  6, }, // Europe/Monaco
    {  675, 3537,  225012,  5, }, // Europe/Moscow
    {  675, 6458,  225017,  4, }, // Europe/Oslo
    {  675, 4284,  225021,  5, }, // Europe/Paris
    {  675, 6424,  225026,  9, }, // Europe/Podgorica
    {  675, 6275,  225035,  4, }, // Europe/Prague
    {  675, 8837,  225039,  4, }, // Europe/Riga
    {  675, 6324,   26956,  3, }, // Europe/Rome
    {  675, 4331,  225043,  6, }, // Europe/Samara
    {  675, 6470,  225049, 10, }, // Europe/San_Marino
    {  675, 6245,  225059,  8, }, // Europe/Sarajevo
    {  675, 4451,  225067,  7, }, // Europe/Saratov
    {  675, 8849,  225074, 10, }, // Europe/Simferopol
    {  675, 6383,  225084,  5, }, // Europe/Skopje
    {  675, 6599,  225089,  6, }, // Europe/Sofia
    {  675, 6523,  225095,  7, }, // Europe/Stockholm
    {  675, 8867,  225102,  5, }, // Europe/Tallinn
    {  675, 6202,  225107,  6, }, // Europe/Tirane
    {  675, 8882,  225113,  9, }, // Europe/Ulyanovsk
    {  675, 6352,  225122,  5, }, // Europe/Vaduz
    {  675, 6567,  225127,  7, }, // Europe/Vatican
    {  675, 6231,  225134,  6, }, // Europe/Vienna
    {  675, 8899,  225140,  7, }, // Europe/Vilnius
    {  675, 4727,  225147,  9, }, // Europe/Volgograd
    {  675, 3190,  225156,  5, }, // Europe/Warsaw
    {  675, 6261,  225161, 11, }, // Europe/Zagreb
    {  675, 6540,  225172,  5, }, // Europe/Zurich
    {  675, 5078,  225177,  9, }, // Indian/Antananarivo
    {  675, 6947,  225186,  6, }, // Indian/Chagos
    {  675, 6046,  225192,  8, }, // Indian/Christmas
    {  675, 6063,  225200,  5, }, // Indian/Cocos
    {  675, 5029,  225205,  6, }, // Indian/Comoro
    {  675, 6661,  225211,  9, }, // Indian/Kerguelen
    {  675, 7260,  225220,  4, }, // Indian/Mahe
    {  675, 7050,  225224,  8, }, // Indian/Maldives
    {  675, 4069,  225232,  9, }, // Indian/Mauritius
    {  675, 5098,  225241,  5, }, // Indian/Mayotte
    {  675, 7211,  225246,  7, }, // Indian/Reunion
    {  675, 4422,  225253,  4, }, // Pacific/Apia
    {  675,  932,  225257,  6, }, // Pacific/Auckland
    {  675, 3706,  225263, 10, }, // Pacific/Bougainville
    {  675, 3015,  225273,  5, }, // Pacific/Chatham
    {  675, 3157,  225278,  3, }, // Pacific/Chuuk
    {  675, 2237,  225281,  5, }, // Pacific/Easter
    {  675, 7360,  225286,  5, }, // Pacific/Efate
    {  675, 7327,  225291,  7, }, // Pacific/Fakaofo
    {  675, 3902,  225298,  4, }, // Pacific/Fiji
    {  675, 7343,  225302,  8, }, // Pacific/Funafuti
    {  675, 6678,  225310,  9, }, // Pacific/Galapagos
    {  675, 6696,  225319,  7, }, // Pacific/Gambier
    {  675, 3822,  225326, 11, }, // Pacific/Guadalcanal
    {  675, 6018,  223462,  4, }, // Pacific/Guam
    {  675, 3049,  225337,  6, }, // Pacific/Kanton
    {  675, 3998,  225343, 10, }, // Pacific/Kiritimati
    {  675, 7019,  225353,  6, }, // Pacific/Kosrae
    {  675, 2842,  225359,  9, }, // Pacific/Kwajalein
    {  675, 4877,  225368,  6, }, // Pacific/Majuro
    {  675, 4051,  225374,  8, }, // Pacific/Marquesas
    {  675, 8914,  225382,  6, }, // Pacific/Midway
    {  675, 7084,  225388,  5, }, // Pacific/Nauru
    {  675, 7132,  225393,  4, }, // Pacific/Niue
    {  675, 4168,  225397,  6, }, // Pacific/Norfolk
    {  675, 7098,  225403,  5, }, // Pacific/Noumea
    {  675, 3126,  225408, 11, }, // Pacific/Pago_Pago
    {  675, 7155,  225419,  5, }, // Pacific/Palau
    {  675, 7194,  225424,  7, }, // Pacific/Pitcairn
    {  675, 3096,  225431,  6, }, // Pacific/Pohnpei
    {  675, 4807,  225437, 12, }, // Pacific/Port_Moresby
    {  675, 6076,  225449,  9, }, // Pacific/Rarotonga
    {  675, 6031,  225458,  6, }, // Pacific/Saipan
    {  675, 7312,  225464,  6, }, // Pacific/Tahiti
    {  675, 6712,  225470,  6, }, // Pacific/Tarawa
    {  675, 4583,  225476,  9, }, // Pacific/Tongatapu
    {  675, 7392,  225485,  3, }, // Pacific/Wake
    {  675, 7405,  225488,  6, }, // Pacific/Wallis
    {  676, 6788,   29832,  7, }, // Africa/Abidjan Kuvi/Devanagari/India
    {  676, 6760,   88123,  5, }, // Africa/Accra
    {  676, 5059,   88128, 10, }, // Africa/Addis_Ababa
    {  676, 7420,   88138,  9, }, // Africa/Algiers
    {  676,   14,   88147,  7, }, // Africa/Asmara
    {  676,   44,   88154,  6, }, // Africa/Bamako
    {  676, 5257,   88160,  6, }, // Africa/Bangui
    {  676, 6746,  225494,  9, }, // Africa/Banjul
    {  676, 7435,   88172,  5, }, // Africa/Bissau
    {  676, 4957,   88177,  9, }, // Africa/Blantyre
    {  676, 5287,   88186, 11, }, // Africa/Brazzaville
    {  676, 4922,   88197,  9, }, // Africa/Bujumbura
    {  676, 2320,   88206,  5, }, // Africa/Cairo
    {  676, 4117,   29933, 11, }, // Africa/Casablanca
    {  676, 7449,   88211,  5, }, // Africa/Ceuta
    {  676, 6773,   88216,  8, }, // Africa/Conakry
    {  676, 6840,   88224,  4, }, // Africa/Dakar
    {  676, 5130,   88228, 11, }, // Africa/Dar_es_Salaam
    {  676, 5043,   88239,  6, }, // Africa/Djibouti
    {  676, 5243,   88245,  5, }, // Africa/Douala
    {  676, 5166,   88250,  8, }, // Africa/El_Aaiun
    {  676, 6853,  225503,  9, }, // Africa/Freetown
    {  676, 4906,   88267,  7, }, // Africa/Gaborone
    {  676, 5015,   88274,  5, }, // Africa/Harare
    {  676, 4479,   88279, 10, }, // Africa/Johannesburg
    {  676, 4499,   30022,  4, }, // Africa/Juba
    {  676, 5151,   88289,  6, }, // Africa/Kampala
    {  676, 4524,  225512,  7, }, // Africa/Khartoum
    {  676, 4973,   88295,  6, }, // Africa/Kigali
    {  676, 5306,  225519,  7, }, // Africa/Kinshasa
    {  676, 4744,   88308,  5, }, // Africa/Lagos
    {  676, 5336,   88313, 10, }, // Africa/Libreville
    {  676, 6869,   88323,  3, }, // Africa/Lome
    {  676, 5211,   88326,  6, }, // Africa/Luanda
    {  676, 4939,  225526,  9, }, // Africa/Lubumbashi
    {  676, 5001,   30089,  6, }, // Africa/Lusaka
    {  676, 5322,   88341,  5, }, // Africa/Malabo
    {  676, 4987,   88346,  6, }, // Africa/Maputo
    {  676, 5197,   30107,  6, }, // Africa/Maseru
    {  676, 5182,  225535,  9, }, // Africa/Mbabane
    {  676, 5113,  225544,  8, }, // Africa/Mogadishu
    {  676, 7034,   88367,  9, }, // Africa/Monrovia
    {  676, 3853,  225552,  7, }, // Africa/Nairobi
    {  676, 5271,   88384,  8, }, // Africa/Ndjamena
    {  676, 5354,   88392,  6, }, // Africa/Niamey
    {  676, 6803,  225559,  7, }, // Africa/Nouakchott
    {  676, 6727,   88405,  7, }, // Africa/Ouagadougou
    {  676, 5225,   88412, 11, }, // Africa/Porto-Novo
    {  676, 4435,   88423,  7, }, // Africa/Sao_Tome
    {  676, 2866,   88430,  8, }, // Africa/Tripoli
    {  676, 6554,   30196,  7, }, // Africa/Tunis
    {  676, 4152,   88438,  7, }, // Africa/Windhoek
    {  676,  132,   88445,  3, }, // America/Adak
    {  676, 3346,   30215,  6, }, // America/Anchorage
    {  676, 5607,   88448,  8, }, // America/Anguilla
    {  676, 5624,   88456,  7, }, // America/Antigua
    {  676, 4554,   88463,  9, }, // America/Araguaina
    {  676,  166,   88472, 11, }, // America/Argentina/Buenos_Aires
    {  676,   91,   30258, 10, }, // America/Argentina/Catamarca
    {  676,  270,   88483,  8, }, // America/Argentina/Cordoba
    {  676,  441,   88491,  5, }, // America/Argentina/Jujuy
    {  676, 7462,   88496,  8, }, // America/Argentina/La_Rioja
    {  676,  565,   88504,  8, }, // America/Argentina/Mendoza
    {  676, 7489,   88512, 12, }, // America/Argentina/Rio_Gallegos
    {  676, 7520,   30305,  6, }, // America/Argentina/Salta
    {  676, 7544,  225566, 11, }, // America/Argentina/San_Juan
    {  676, 5580,  225577, 10, }, // America/Argentina/San_Luis
    {  676, 7571,   88540,  6, }, // America/Argentina/Tucuman
    {  676, 7597,  225587,  7, }, // America/Argentina/Ushuaia
    {  676, 5640,   88553,  5, }, // America/Aruba
    {  676, 4252,  225594,  8, }, // America/Asuncion
    {  676,  237,   88566,  8, }, // America/Atikokan
    {  676, 3679,   88574,  5, }, // America/Bahia
    {  676, 7623,   88579, 15, }, // America/Bahia_Banderas
    {  676, 5654,   88594,  8, }, // America/Barbados
    {  676, 7646,   30394,  5, }, // America/Belem
    {  676, 5407,   88602,  6, }, // America/Belize
    {  676, 7660,  225602, 12, }, // America/Blanc-Sablon
    {  676, 7681,   88621, 10, }, // America/Boa_Vista
    {  676, 4361,   88631,  6, }, // America/Bogota
    {  676, 7699,   88637,  5, }, // America/Boise
    {  676, 7713,  225614, 12, }, // America/Cambridge_Bay
    {  676, 7735,  225626, 16, }, // America/Campo_Grande
    {  676, 3868,  225642,  8, }, // America/Cancun
    {  676, 4694,   88676,  6, }, // America/Caracas
    {  676, 4345,   30477,  5, }, // America/Cayenne
    {  676, 5496,  138280,  7, }, // America/Cayman
    {  676, 2260,  225650,  6, }, // America/Chicago
    {  676, 7756,   88688,  8, }, // America/Chihuahua
    {  676, 5422,   88712, 11, }, // America/Costa_Rica
    {  676, 7796,   30510,  8, }, // America/Creston
    {  676, 3791,   88723,  7, }, // America/Cuiaba
    {  676, 5723,   88730,  7, }, // America/Curacao
    {  676, 7812,  225656, 11, }, // America/Danmarkshavn
    {  676, 7833,   88748,  4, }, // America/Dawson
    {  676, 7848,   88752, 10, }, // America/Dawson_Creek
    {  676,  805,   88762,  5, }, // America/Denver
    {  676, 3465,   88767,  8, }, // America/Detroit
    {  676, 5739,   88775,  8, }, // America/Dominica
    {  676,  893,   88783,  6, }, // America/Edmonton
    {  676, 7869,  225667,  8, }, // America/Eirunepe
    {  676, 5441,   88789, 11, }, // America/El_Salvador
    {  676, 7886,  225675, 13, }, // America/Fort_Nelson
    {  676, 7906,  225688, 12, }, // America/Fortaleza
    {  676, 7924,   30628,  8, }, // America/Glace_Bay
    {  676, 6881,  113781,  6, }, // America/Goose_Bay
    {  676, 4612,   88845, 11, }, // America/Grand_Turk
    {  676, 5770,   30657,  8, }, // America/Grenada
    {  676, 5786,   88856,  9, }, // America/Guadeloupe
    {  676, 3760,   30674, 10, }, // America/Guatemala
    {  676, 4841,   88865,  9, }, // America/Guayaquil
    {  676, 6932,   88874,  5, }, // America/Guyana
    {  676, 1939,  225700,  9, }, // America/Halifax
    {  676, 2281,   88889,  5, }, // America/Havana
    {  676, 7942,   88894, 11, }, // America/Hermosillo
    {  676,  348,   88905, 13, }, // America/Indiana/Indianapolis
    {  676,  481,  113846, 15, }, // America/Indiana/Knox
    {  676, 7961,   88933, 17, }, // America/Indiana/Marengo
    {  676, 7985,   88950, 20, }, // America/Indiana/Petersburg
    {  676, 8012,   88970, 18, }, // America/Indiana/Tell_City
    {  676, 8038,   88988, 14, }, // America/Indiana/Vevay
    {  676, 8060,   89002, 18, }, // America/Indiana/Vincennes
    {  676, 8086,   89020, 17, }, // America/Indiana/Winamac
    {  676, 8110,   30867,  6, }, // America/Inuvik
    {  676,  660,  225709,  8, }, // America/Iqaluit
    {  676, 2799,  138605,  6, }, // America/Jamaica
    {  676, 5380,   89049,  7, }, // America/Juneau
    {  676,  521,   89056,  8, }, // America/Kentucky/Louisville
    {  676, 8125,   89064, 17, }, // America/Kentucky/Monticello
    {  676, 5704,  225717, 13, }, // America/Kralendijk
    {  676, 4376,   89093,  7, }, // America/La_Paz
    {  676, 7169,   30932,  4, }, // America/Lima
    {  676, 3239,   89100, 11, }, // America/Los_Angeles
    {  676, 5932,   89111, 22, }, // America/Lower_Princes
    {  676, 8153,   89133,  5, }, // America/Maceio
    {  676, 8168,   89138,  7, }, // America/Managua
    {  676, 1908,  225730,  4, }, // America/Manaus
    {  676, 5897,  225734,  7, }, // America/Marigot
    {  676, 5805,   89156,  9, }, // America/Martinique
    {  676, 8184,   89165,  9, }, // America/Matamoros
    {  676, 2917,   89174,  9, }, // America/Mazatlan
    {  676, 8202,   89183,  8, }, // America/Menominee
    {  676, 8220,   31026,  6, }, // America/Merida
    {  676, 8235,   89191, 12, }, // America/Metlakatla
    {  676, 2949,   89203, 13, }, // America/Mexico_City
    {  676, 4391,   89216,  7, }, // America/Miquelon
    {  676, 8254,   89223,  6, }, // America/Moncton
    {  676, 8270,   89229,  9, }, // America/Monterrey
    {  676, 4098,   89238, 11, }, // America/Montevideo
    {  676, 5824,   89249,  9, }, // America/Montserrat
    {  676, 5481,   89258,  5, }, // America/Nassau
    {  676, 2379,   89263,  9, }, // America/New_York
    {  676, 8288,   89185,  3, }, // America/Nome
    {  676, 1850,   89272,  8, }, // America/Noronha
    {  676, 8301,  225741, 19, }, // America/North_Dakota/Beulah
    {  676, 8329,  225760, 18, }, // America/North_Dakota/Center
    {  676, 8357,  225778, 23, }, // America/North_Dakota/New_Salem
    {  676,  393,   89343,  3, }, // America/Nuuk
    {  676, 8388,  225801,  9, }, // America/Ojinaga
    {  676, 2356,   89355,  5, }, // America/Panama
    {  676, 6173,   89360, 10, }, // America/Paramaribo
    {  676, 2973,  225810,  8, }, // America/Phoenix
    {  676, 3945,   89378, 14, }, // America/Port-au-Prince
    {  676, 5954,  225818, 15, }, // America/Port_of_Spain
    {  676, 8404,   89407, 13, }, // America/Porto_Velho
    {  676, 5843,   89420, 11, }, // America/Puerto_Rico
    {  676, 4030,   89431, 12, }, // America/Punta_Arenas
    {  676, 8424,   89443, 12, }, // America/Rankin_Inlet
    {  676, 8445,  225833,  7, }, // America/Recife
    {  676, 1995,   31299,  6, }, // America/Regina
    {  676, 8460,   89462,  9, }, // America/Resolute
    {  676,  695,   89471, 12, }, // America/Rio_Branco
    {  676, 8477,  225840,  9, }, // America/Santarem
    {  676, 2201,  225849,  8, }, // America/Santiago
    {  676, 6111,   89500, 13, }, // America/Santo_Domingo
    {  676, 1878,   89513,  9, }, // America/Sao_Paulo
    {  676, 6899,   89522, 13, }, // America/Scoresbysund
    {  676, 8494,   89535,  6, }, // America/Sitka
    {  676, 8508,  225857, 15, }, // America/St_Barthelemy
    {  676, 2061,  114367, 10, }, // America/St_Johns
    {  676, 5863,   89565, 10, }, // America/St_Kitts
    {  676, 5880,   89575, 11, }, // America/St_Lucia
    {  676,  855,  225872,  9, }, // America/St_Thomas
    {  676, 5913,   89595, 12, }, // America/St_Vincent
    {  676, 8530,  225881, 13, }, // America/Swift_Current
    {  676, 5461,   89620, 11, }, // America/Tegucigalpa
    {  676, 5756,  129761,  4, }, // America/Thule
    {  676,  313,   89637,  7, }, // America/Tijuana
    {  676,  608,   89644,  6, }, // America/Toronto
    {  676, 5688,   89650,  8, }, // America/Tortola
    {  676, 2093,  225894, 10, }, // America/Vancouver
    {  676, 2144,   89665, 11, }, // America/Whitehorse
    {  676,  734,   89676,  7, }, // America/Winnipeg
    {  676, 8552,   89683,  7, }, // America/Yakutat
    {  676, 6001,   31541,  4, }, // Antarctica/Casey
    {  676, 6094,   89690,  5, }, // Antarctica/Davis
    {  676, 6133,   89695, 19, }, // Antarctica/DumontDUrville
    {  676, 8568,   89714,  7, }, // Antarctica/Macquarie
    {  676, 7066,   89721,  4, }, // Antarctica/Mawson
    {  676, 7113,  225904, 11, }, // Antarctica/McMurdo
    {  676, 8589,   89734,  4, }, // Antarctica/Palmer
    {  676, 7226,   89738,  6, }, // Antarctica/Rothera
    {  676, 7295,   89744,  6, }, // Antarctica/Syowa
    {  676, 8607,   89750,  5, }, // Antarctica/Troll
    {  676, 7374,   89755,  7, }, // Antarctica/Vostok
    {  676, 1429,  225915, 12, }, // Arctic/Longyearbyen
    {  676, 5570,   89775,  4, }, // Asia/Aden
    {  676, 5395,   89779,  7, }, // Asia/Almaty
    {  676, 3968,   89786,  6, }, // Asia/Amman
    {  676, 5511,   89792,  6, }, // Asia/Anadyr
    {  676, 5523,   31646,  6, }, // Asia/Aqtau
    {  676, 5368,  114541,  7, }, // Asia/Aqtobe
    {  676,  964,  225927,  8, }, // Asia/Ashgabat
    {  676, 8624,   89804,  5, }, // Asia/Atyrau
    {  676, 3607,   89809,  5, }, // Asia/Baghdad
    {  676, 5534,   89814,  5, }, // Asia/Bahrain
    {  676, 3653,   88728,  4, }, // Asia/Baku
    {  676, 4466,  225935,  9, }, // Asia/Bangkok
    {  676, 3571,   89825,  6, }, // Asia/Barnaul
    {  676, 4086,   89831,  5, }, // Asia/Beirut
    {  676, 3778,  225944,  7, }, // Asia/Bishkek
    {  676, 5989,  225951,  8, }, // Asia/Brunei
    {  676, 4601,  225959,  6, }, // Asia/Chita
    {  676, 4511,   89849,  6, }, // Asia/Colombo
    {  676, 4540,   89855,  7, }, // Asia/Damascus
    {  676, 1093,   89862,  4, }, // Asia/Dhaka
    {  676, 6192,   89866,  4, }, // Asia/Dili
    {  676, 3596,  225965,  5, }, // Asia/Dubai
    {  676, 6159,  225970,  7, }, // Asia/Dushanbe
    {  676, 8636,  225977, 11, }, // Asia/Famagusta
    {  676, 8651,   89892,  5, }, // Asia/Gaza
    {  676, 4795,   89897,  7, }, // Asia/Hebron
    {  676, 1259,   89904, 16, }, // Asia/Ho_Chi_Minh
    {  676, 2704,   89920,  9, }, // Asia/Hong_Kong
    {  676, 4771,   89929,  5, }, // Asia/Hovd
    {  676, 4184,   89934, 10, }, // Asia/Irkutsk
    {  676, 7006,   89944,  7, }, // Asia/Jakarta
    {  676, 6992,   31811,  7, }, // Asia/Jayapura
    {  676, 1290,  225988,  8, }, // Asia/Jerusalem
    {  676, 3560,   89957,  5, }, // Asia/Kabul
    {  676, 4316,   89962,  7, }, // Asia/Kamchatka
    {  676, 4239,   31840,  5, }, // Asia/Karachi
    {  676, 1185,   89969,  8, }, // Asia/Kathmandu
    {  676, 8661,  225996,  7, }, // Asia/Khandyga
    {  676,  992,   89984,  7, }, // Asia/Kolkata
    {  676, 4197,   89991, 15, }, // Asia/Krasnoyarsk
    {  676, 4859,   90006, 10, }, // Asia/Kuala_Lumpur
    {  676, 5976,   31896,  6, }, // Asia/Kuching
    {  676, 5547,  114687,  5, }, // Asia/Kuwait
    {  676, 1211,   90016,  4, }, // Asia/Macau
    {  676, 4017,   31912,  7, }, // Asia/Magadan
    {  676, 1349,   90020,  6, }, // Asia/Makassar
    {  676, 7182,   31928,  5, }, // Asia/Manila
    {  676, 6920,   90026,  5, }, // Asia/Muscat
    {  676, 2554,   90031,  8, }, // Asia/Nicosia
    {  676, 8675,   90039, 16, }, // Asia/Novokuznetsk
    {  676, 4135,   90055, 13, }, // Asia/Novosibirsk
    {  676, 4229,   90068,  6, }, // Asia/Omsk
    {  676, 7145,   90074,  3, }, // Asia/Oral
    {  676, 6961,  226003, 10, }, // Asia/Phnom_Penh
    {  676, 8693,   90086,  9, }, // Asia/Pontianak
    {  676, 4214,   90095, 10, }, // Asia/Pyongyang
    {  676, 5559,   90105,  3, }, // Asia/Qatar
    {  676, 8708,   90108,  8, }, // Asia/Qostanay
    {  676, 4269,  226013, 10, }, // Asia/Qyzylorda
    {  676, 3584,   90127,  5, }, // Asia/Riyadh
    {  676, 4408,  226023,  7, }, // Asia/Sakhalin
    {  676, 7245,   90138,  6, }, // Asia/Samarkand
    {  676, 3279,   90144,  4, }, // Asia/Seoul
    {  676, 1053,  226030,  6, }, // Asia/Shanghai
    {  676, 3300,   32058,  8, }, // Asia/Singapore
    {  676, 4297,   90153, 16, }, // Asia/Srednekolymsk
    {  676, 3263,  226036,  8, }, // Asia/Taipei
    {  676, 4781,  226044,  6, }, // Asia/Tashkent
    {  676, 3915,  226050,  9, }, // Asia/Tbilisi
    {  676, 2772,   32104,  6, }, // Asia/Tehran
    {  676, 1317,   90188,  5, }, // Asia/Thimphu
    {  676, 2821,   90193,  6, }, // Asia/Tokyo
    {  676, 4572,   90199,  7, }, // Asia/Tomsk
    {  676, 1021,   90206,  8, }, // Asia/Ulaanbaatar
    {  676, 1159,   90214,  7, }, // Asia/Urumqi
    {  676, 8722,   90221, 10, }, // Asia/Ust-Nera
    {  676, 6977,   90231,  9, }, // Asia/Vientiane
    {  676, 4710,   90240, 13, }, // Asia/Vladivostok
    {  676, 4828,   32173,  9, }, // Asia/Yakutsk
    {  676, 1235,  226059,  6, }, // Asia/Yangon
    {  676, 3883,   90253, 13, }, // Asia/Yekaterinburg
    {  676, 3747,   90266,  7, }, // Asia/Yerevan
    {  676, 3663,   90273,  7, }, // Atlantic/Azores
    {  676, 5671,   90280,  6, }, // Atlantic/Bermuda
    {  676, 6628,  226065,  8, }, // Atlantic/Canary
    {  676, 3727,   90292,  8, }, // Atlantic/Cape_Verde
    {  676, 1395,  226073,  6, }, // Atlantic/Faroe
    {  676, 8736,   90305,  5, }, // Atlantic/Madeira
    {  676, 2748,   90310,  9, }, // Atlantic/Reykjavik
    {  676, 7272,  226079, 18, }, // Atlantic/South_Georgia
    {  676, 6821,   90334, 11, }, // Atlantic/St_Helena
    {  676, 6644,  226097,  9, }, // Atlantic/Stanley
    {  676, 1670,   90352,  6, }, // Australia/Adelaide
    {  676, 1635,   90358,  8, }, // Australia/Brisbane
    {  676, 1799,   90366, 10, }, // Australia/Broken_Hill
    {  676, 1583,   90376,  7, }, // Australia/Darwin
    {  676, 3637,   32320,  6, }, // Australia/Eucla
    {  676, 1516,   90383,  7, }, // Australia/Hobart
    {  676, 8753,   90390,  8, }, // Australia/Lindeman
    {  676, 1547,   90398,  9, }, // Australia/Lord_Howe
    {  676, 1727,   90408,  8, }, // Australia/Melbourne
    {  676, 1762,   90416,  4, }, // Australia/Perth
    {  676, 1463,   32362,  5, }, // Australia/Sydney
    {  676, 6441,   90420, 10, }, // Europe/Amsterdam
    {  676, 6216,   90430,  6, }, // Europe/Andorra
    {  676, 3620,  226106, 10, }, // Europe/Astrakhan
    {  676, 2300,  226116,  6, }, // Europe/Athens
    {  676, 6336,   32400,  8, }, // Europe/Belgrade
    {  676, 4757,   90451,  6, }, // Europe/Berlin
    {  676, 6488,   90457, 12, }, // Europe/Bratislava
    {  676, 2167,   90469,  9, }, // Europe/Brussels
    {  676, 3928,   32438,  9, }, // Europe/Bucharest
    {  676, 3806,   90488,  9, }, // Europe/Budapest
    {  676, 8772,   90497, 10, }, // Europe/Busingen
    {  676, 2583,   90507,  7, }, // Europe/Chisinau
    {  676, 6289,   90514,  9, }, // Europe/Copenhagen
    {  676, 2338,   90523,  5, }, // Europe/Dublin
    {  676, 6307,   90528, 10, }, // Europe/Gibraltar
    {  676, 8788,   90538,  6, }, // Europe/Guernsey
    {  676, 6612,   90544,  8, }, // Europe/Helsinki
    {  676, 8804,  226122, 13, }, // Europe/Isle_of_Man
    {  676, 1130,   90563,  9, }, // Europe/Istanbul
    {  676, 8823,   90572,  5, }, // Europe/Jersey
    {  676, 3979,   90577, 12, }, // Europe/Kaliningrad
    {  676, 8929,   90589,  5, }, // Europe/Kirov
    {  676, 2527,   90594,  3, }, // Europe/Kyiv
    {  676, 3213,   32558,  6, }, // Europe/Lisbon
    {  676, 6506,   90597, 11, }, // Europe/Ljubljana
    {  676, 2501,   90608,  4, }, // Europe/London
    {  676, 6365,   90612, 10, }, // Europe/Luxembourg
    {  676, 4892,  139917,  9, }, // Europe/Madrid
    {  676, 6397,   32599,  6, }, // Europe/Malta
    {  676, 6582,   90629,  9, }, // Europe/Mariehamn
    {  676, 3693,   90638,  6, }, // Europe/Minsk
    {  676, 6410,   90644,  6, }, // Europe/Monaco
    {  676, 3537,   90650,  6, }, // Europe/Moscow
    {  676, 6458,   90656,  5, }, // Europe/Oslo
    {  676, 4284,   32634,  5, }, // Europe/Paris
    {  676, 6424,   90661, 10, }, // Europe/Podgorica
    {  676, 6275,   90671,  5, }, // Europe/Prague
    {  676, 8837,   90676,  4, }, // Europe/Riga
    {  676, 6324,   90303,  3, }, // Europe/Rome
    {  676, 4331,   90680,  5, }, // Europe/Samara
    {  676, 6470,   90685, 10, }, // Europe/San_Marino
    {  676, 6245,   90695,  8, }, // Europe/Sarajevo
    {  676, 4451,   90703,  7, }, // Europe/Saratov
    {  676, 8849,  226135, 12, }, // Europe/Simferopol
    {  676, 6383,  226147,  8, }, // Europe/Skopje
    {  676, 6599,  226155,  7, }, // Europe/Sofia
    {  676, 6523,   90737,  8, }, // Europe/Stockholm
    {  676, 8867,   90745,  5, }, // Europe/Tallinn
    {  676, 6202,   90750,  6, }, // Europe/Tirane
    {  676, 8882,   90756, 12, }, // Europe/Ulyanovsk
    {  676, 6352,   90768,  6, }, // Europe/Vaduz
    {  676, 6567,   90774,  6, }, // Europe/Vatican
    {  676, 6231,   90780,  5, }, // Europe/Vienna
    {  676, 8899,   90785,  8, }, // Europe/Vilnius
    {  676, 4727,   90793, 11, }, // Europe/Volgograd
    {  676, 3190,   90804,  5, }, // Europe/Warsaw
    {  676, 6261,   90809,  8, }, // Europe/Zagreb
    {  676, 6540,  226162,  9, }, // Europe/Zurich
    {  676, 5078,   90826, 11, }, // Indian/Antananarivo
    {  676, 6947,  115272,  5, }, // Indian/Chagos
    {  676, 6046,   90842,  7, }, // Indian/Christmas
    {  676, 6063,   90849,  5, }, // Indian/Cocos
    {  676, 5029,   90854,  6, }, // Indian/Comoro
    {  676, 6661,   90860,  7, }, // Indian/Kerguelen
    {  676, 7260,   32851,  4, }, // Indian/Mahe
    {  676, 7050,   90867,  6, }, // Indian/Maldives
    {  676, 4069,  226171,  6, }, // Indian/Mauritius
    {  676, 5098,   90879,  8, }, // Indian/Mayotte
    {  676, 7211,   90887,  8, }, // Indian/Reunion
    {  676, 4422,   90895,  5, }, // Pacific/Apia
    {  676,  932,  226177,  6, }, // Pacific/Auckland
    {  676, 3706,   90906,  8, }, // Pacific/Bougainville
    {  676, 3015,  226183,  6, }, // Pacific/Chatham
    {  676, 3157,   90918,  2, }, // Pacific/Chuuk
    {  676, 2237,  226189,  6, }, // Pacific/Easter
    {  676, 7360,  226195,  5, }, // Pacific/Efate
    {  676, 7327,  226200,  9, }, // Pacific/Fakaofo
    {  676, 3902,  226209,  5, }, // Pacific/Fiji
    {  676, 7343,  226214, 12, }, // Pacific/Funafuti
    {  676, 6678,   90956,  9, }, // Pacific/Galapagos
    {  676, 6696,  226226,  9, }, // Pacific/Gambier
    {  676, 3822,   90972, 10, }, // Pacific/Guadalcanal
    {  676, 6018,   30980,  4, }, // Pacific/Guam
    {  676, 3049,  226235,  5, }, // Pacific/Kanton
    {  676, 3998,   33000, 10, }, // Pacific/Kiritimati
    {  676, 7019,   90997,  6, }, // Pacific/Kosrae
    {  676, 2842,   91003, 10, }, // Pacific/Kwajalein
    {  676, 4877,   91013,  6, }, // Pacific/Majuro
    {  676, 4051,   91019,  8, }, // Pacific/Marquesas
    {  676, 8914,   91027,  5, }, // Pacific/Midway
    {  676, 7084,  226240,  6, }, // Pacific/Nauru
    {  676, 7132,   91036,  4, }, // Pacific/Niue
    {  676, 4168,  226246,  7, }, // Pacific/Norfolk
    {  676, 7098,  226253,  6, }, // Pacific/Noumea
    {  676, 3126,   91053,  9, }, // Pacific/Pago_Pago
    {  676, 7155,   91062,  4, }, // Pacific/Palau
    {  676, 7194,  137073,  8, }, // Pacific/Pitcairn
    {  676, 3096,  226259,  7, }, // Pacific/Pohnpei
    {  676, 4807,   91072, 14, }, // Pacific/Port_Moresby
    {  676, 6076,   91086,  9, }, // Pacific/Rarotonga
    {  676, 6031,   91095,  6, }, // Pacific/Saipan
    {  676, 7312,   91101,  6, }, // Pacific/Tahiti
    {  676, 6712,   91107,  6, }, // Pacific/Tarawa
    {  676, 4583,   91113,  9, }, // Pacific/Tongatapu
    {  676, 7392,   33136,  3, }, // Pacific/Wake
    {  676, 7405,   33139,  5, }, // Pacific/Wallis
    {  677, 6788,  148105,  7, }, // Africa/Abidjan Kuvi/Odia/India
    {  677, 6760,  226266,  5, }, // Africa/Accra
    {  677, 5059,  148117, 12, }, // Africa/Addis_Ababa
    {  677, 7420,  148129,  8, }, // Africa/Algiers
    {  677,   14,  148137,  5, }, // Africa/Asmara
    {  677,   44,  148142,  5, }, // Africa/Bamako
    {  677, 5257,  148148,  7, }, // Africa/Bangui
    {  677, 6746,  148155,  7, }, // Africa/Banjul
    {  677, 7435,  148162,  5, }, // Africa/Bissau
    {  677, 4957,  148167, 13, }, // Africa/Blantyre
    {  677, 5287,  226271, 12, }, // Africa/Brazzaville
    {  677, 4922,  226283, 10, }, // Africa/Bujumbura
    {  677, 2320,  148202,  4, }, // Africa/Cairo
    {  677, 4117,  148207, 12, }, // Africa/Casablanca
    {  677, 7449,  148219,  5, }, // Africa/Ceuta
    {  677, 6773,  226293,  7, }, // Africa/Conakry
    {  677, 6840,  148232,  4, }, // Africa/Dakar
    {  677, 5130,  148236, 15, }, // Africa/Dar_es_Salaam
    {  677, 5043,  226300,  5, }, // Africa/Djibouti
    {  677, 5243,  148257,  4, }, // Africa/Douala
    {  677, 5166,  148261,  9, }, // Africa/El_Aaiun
    {  677, 6853,  226305, 10, }, // Africa/Freetown
    {  677, 4906,  226315,  8, }, // Africa/Gaborone
    {  677, 5015,  148289,  5, }, // Africa/Harare
    {  677, 4479,  226323, 10, }, // Africa/Johannesburg
    {  677, 4499,  148305,  4, }, // Africa/Juba
    {  677, 5151,  148309,  8, }, // Africa/Kampala
    {  677, 4524,  226333,  7, }, // Africa/Khartoum
    {  677, 4973,  148324,  6, }, // Africa/Kigali
    {  677, 5306,  148330,  8, }, // Africa/Kinshasa
    {  677, 4744,  226340,  6, }, // Africa/Lagos
    {  677, 5336,  226346, 12, }, // Africa/Libreville
    {  677, 6869,  226358,  4, }, // Africa/Lome
    {  677, 5211,  148362,  7, }, // Africa/Luanda
    {  677, 4939,  148369, 10, }, // Africa/Lubumbashi
    {  677, 5001,  148379,  6, }, // Africa/Lusaka
    {  677, 5322,  148385,  5, }, // Africa/Malabo
    {  677, 4987,  148391,  6, }, // Africa/Maputo
    {  677, 5197,  148397,  6, }, // Africa/Maseru
    {  677, 5182,  226362,  9, }, // Africa/Mbabane
    {  677, 5113,  226371,  8, }, // Africa/Mogadishu
    {  677, 7034,  226379,  7, }, // Africa/Monrovia
    {  677, 3853,  226386,  6, }, // Africa/Nairobi
    {  677, 5271,  148432,  6, }, // Africa/Ndjamena
    {  677, 5354,  148438,  5, }, // Africa/Niamey
    {  677, 6803,  226392,  6, }, // Africa/Nouakchott
    {  677, 6727,  226398,  6, }, // Africa/Ouagadougou
    {  677, 5225,  226404,  7, }, // Africa/Porto-Novo
    {  677, 4435,  226411,  7, }, // Africa/Sao_Tome
    {  677, 2866,  226418,  7, }, // Africa/Tripoli
    {  677, 6554,  148485,  7, }, // Africa/Tunis
    {  677, 4152,  226425,  8, }, // Africa/Windhoek
    {  677,  132,  148502,  5, }, // America/Adak
    {  677, 3346,  148507,  8, }, // America/Anchorage
    {  677, 5607,  148515,  8, }, // America/Anguilla
    {  677, 5624,  148523,  8, }, // America/Antigua
    {  677, 4554,  148531,  8, }, // America/Araguaina
    {  677,  166,  226433, 13, }, // America/Argentina/Buenos_Aires
    {  677,   91,  226446, 10, }, // America/Argentina/Catamarca
    {  677,  270,  226456,  6, }, // America/Argentina/Cordoba
    {  677,  441,  226462,  5, }, // America/Argentina/Jujuy
    {  677, 7462,  226467,  9, }, // America/Argentina/La_Rioja
    {  677,  565,  226476,  7, }, // America/Argentina/Mendoza
    {  677, 7489,  226483, 12, }, // America/Argentina/Rio_Gallegos
    {  677, 7520,  226495,  6, }, // America/Argentina/Salta
    {  677, 7544,  226501, 11, }, // America/Argentina/San_Juan
    {  677, 5580,  226512,  9, }, // America/Argentina/San_Luis
    {  677, 7571,  226521,  6, }, // America/Argentina/Tucuman
    {  677, 7597,  226527,  7, }, // America/Argentina/Ushuaia
    {  677, 5640,  226534,  5, }, // America/Aruba
    {  677, 4252,  148652,  9, }, // America/Asuncion
    {  677,  237,  148661,  9, }, // America/Atikokan
    {  677, 3679,  148670,  5, }, // America/Bahia
    {  677, 7623,  148675, 18, }, // America/Bahia_Banderas
    {  677, 5654,  226539,  7, }, // America/Barbados
    {  677, 7646,  148703,  5, }, // America/Belem
    {  677, 5407,  148708,  6, }, // America/Belize
    {  677, 7660,  226546, 14, }, // America/Blanc-Sablon
    {  677, 7681,  226560, 10, }, // America/Boa_Vista
    {  677, 4361,  226570,  4, }, // America/Bogota
    {  677, 7699,  226574,  4, }, // America/Boise
    {  677, 7713,  148750, 13, }, // America/Cambridge_Bay
    {  677, 7735,  226578, 14, }, // America/Campo_Grande
    {  677, 3868,  148778,  7, }, // America/Cancun
    {  677, 4694,  148785,  8, }, // America/Caracas
    {  677, 4345,  148793,  8, }, // America/Cayenne
    {  677, 5496,  148801,  9, }, // America/Cayman
    {  677, 2260,  226592,  5, }, // America/Chicago
    {  677, 7756,  148816,  8, }, // America/Chihuahua
    {  677, 5422,  226597, 10, }, // America/Costa_Rica
    {  677, 7796,  226607,  9, }, // America/Creston
    {  677, 3791,  148860,  5, }, // America/Cuiaba
    {  677, 5723,  148865,  5, }, // America/Curacao
    {  677, 7812,  148871, 13, }, // America/Danmarkshavn
    {  677, 7833,  148884,  5, }, // America/Dawson
    {  677, 7848,  148889, 12, }, // America/Dawson_Creek
    {  677,  805,  226616,  7, }, // America/Denver
    {  677, 3465,  226623,  8, }, // America/Detroit
    {  677, 5739,  226631,  7, }, // America/Dominica
    {  677,  893,  226638,  9, }, // America/Edmonton
    {  677, 7869,  148934,  7, }, // America/Eirunepe
    {  677, 5441,  226647, 13, }, // America/El_Salvador
    {  677, 7886,  226660, 11, }, // America/Fort_Nelson
    {  677, 7906,  226671,  8, }, // America/Fortaleza
    {  677, 7924,  148977,  9, }, // America/Glace_Bay
    {  677, 6881,  148986,  7, }, // America/Goose_Bay
    {  677, 4612,  148993, 14, }, // America/Grand_Turk
    {  677, 5770,  149007,  8, }, // America/Grenada
    {  677, 5786,  226679,  9, }, // America/Guadeloupe
    {  677, 3760,  226688,  9, }, // America/Guatemala
    {  677, 4841,  149034,  9, }, // America/Guayaquil
    {  677, 6932,  149043,  5, }, // America/Guyana
    {  677, 1939,  149048, 12, }, // America/Halifax
    {  677, 2281,  226697,  5, }, // America/Havana
    {  677, 7942,  226702,  8, }, // America/Hermosillo
    {  677,  348,  226710, 13, }, // America/Indiana/Indianapolis
    {  677,  481,  149089, 17, }, // America/Indiana/Knox
    {  677, 7961,  226723, 16, }, // America/Indiana/Marengo
    {  677, 7985,  149123, 22, }, // America/Indiana/Petersburg
    {  677, 8012,  149145, 19, }, // America/Indiana/Tell_City
    {  677, 8038,  226739, 15, }, // America/Indiana/Vevay
    {  677, 8060,  149179, 21, }, // America/Indiana/Vincennes
    {  677, 8086,  149200, 18, }, // America/Indiana/Winamac
    {  677, 8110,  226754,  7, }, // America/Inuvik
    {  677,  660,  149225, 10, }, // America/Iqaluit
    {  677, 2799,  149235,  7, }, // America/Jamaica
    {  677, 5380,  149242,  6, }, // America/Juneau
    {  677,  521,  226761, 10, }, // America/Kentucky/Louisville
    {  677, 8125,  226771, 20, }, // America/Kentucky/Monticello
    {  677, 5704,  226791, 13, }, // America/Kralendijk
    {  677, 4376,  149291,  6, }, // America/La_Paz
    {  677, 7169,  149299,  4, }, // America/Lima
    {  677, 3239,  149303, 13, }, // America/Los_Angeles
    {  677, 5932,  149316, 23, }, // America/Lower_Princes
    {  677, 8153,  226804,  5, }, // America/Maceio
    {  677, 8168,  149344,  7, }, // America/Managua
    {  677, 1908,  149351,  8, }, // America/Manaus
    {  677, 5897,  226809,  7, }, // America/Marigot
    {  677, 5805,  149367, 12, }, // America/Martinique
    {  677, 8184,  226816,  8, }, // America/Matamoros
    {  677, 2917,  226824,  9, }, // America/Mazatlan
    {  677, 8202,  149398,  8, }, // America/Menominee
    {  677, 8220,  149406,  6, }, // America/Merida
    {  677, 8235,  149412, 10, }, // America/Metlakatla
    {  677, 2949,  149422, 13, }, // America/Mexico_City
    {  677, 4391,  226833,  9, }, // America/Miquelon
    {  677, 8254,  226842,  7, }, // America/Moncton
    {  677, 8270,  226849,  7, }, // America/Monterrey
    {  677, 4098,  226856, 10, }, // America/Montevideo
    {  677, 5824,  226866, 10, }, // America/Montserrat
    {  677, 5481,  149482,  5, }, // America/Nassau
    {  677, 2379,  226876, 10, }, // America/New_York
    {  677, 8288,  149498,  4, }, // America/Nome
    {  677, 1850,  226886,  5, }, // America/Noronha
    {  677, 8301,  226891, 19, }, // America/North_Dakota/Beulah
    {  677, 8329,  226910, 20, }, // America/North_Dakota/Center
    {  677, 8357,  226930, 24, }, // America/North_Dakota/New_Salem
    {  677,  393,  149576,  5, }, // America/Nuuk
    {  677, 8388,  226954,  7, }, // America/Ojinaga
    {  677, 2356,  149588,  6, }, // America/Panama
    {  677, 6173,  149594,  9, }, // America/Paramaribo
    {  677, 2973,  226961,  7, }, // America/Phoenix
    {  677, 3945,  226968, 17, }, // America/Port-au-Prince
    {  677, 5954,  226985, 16, }, // America/Port_of_Spain
    {  677, 8404,  149647, 11, }, // America/Porto_Velho
    {  677, 5843,  227001, 10, }, // America/Puerto_Rico
    {  677, 4030,  149670, 15, }, // America/Punta_Arenas
    {  677, 8424,  149685, 14, }, // America/Rankin_Inlet
    {  677, 8445,  227011,  6, }, // America/Recife
    {  677, 1995,  149705,  6, }, // America/Regina
    {  677, 8460,  149711,  8, }, // America/Resolute
    {  677,  695,  227017, 11, }, // America/Rio_Branco
    {  677, 8477,  227028,  9, }, // America/Santarem
    {  677, 2201,  149742,  8, }, // America/Santiago
    {  677, 6111,  227037, 12, }, // America/Santo_Domingo
    {  677, 1878,  149766,  9, }, // America/Sao_Paulo
    {  677, 6899,  227049, 14, }, // America/Scoresbysund
    {  677, 8494,  149792,  5, }, // America/Sitka
    {  677, 8508,  227063, 16, }, // America/St_Barthelemy
    {  677, 2061,  149815, 12, }, // America/St_Johns
    {  677, 5863,  227079, 13, }, // America/St_Kitts
    {  677, 5880,  149840, 13, }, // America/St_Lucia
    {  677,  855,  227092, 13, }, // America/St_Thomas
    {  677, 5913,  227105, 17, }, // America/St_Vincent
    {  677, 8530,  149884, 15, }, // America/Swift_Current
    {  677, 5461,  149899, 11, }, // America/Tegucigalpa
    {  677, 5756,  227122,  4, }, // America/Thule
    {  677,  313,  149914,  7, }, // America/Tijuana
    {  677,  608,  227126,  5, }, // America/Toronto
    {  677, 5688,  227131,  5, }, // America/Tortola
    {  677, 2093,  227136,  9, }, // America/Vancouver
    {  677, 2144,  149945, 11, }, // America/Whitehorse
    {  677,  734,  149956,  8, }, // America/Winnipeg
    {  677, 8552,  149964,  8, }, // America/Yakutat
    {  677, 6001,  227145,  4, }, // Antarctica/Casey
    {  677, 6094,  227149,  6, }, // Antarctica/Davis
    {  677, 6133,  227155, 19, }, // Antarctica/DumontDUrville
    {  677, 8568,  227174,  7, }, // Antarctica/Macquarie
    {  677, 7066,  227181,  5, }, // Antarctica/Mawson
    {  677, 7113,  227186, 10, }, // Antarctica/McMurdo
    {  677, 8589,  227196,  5, }, // Antarctica/Palmer
    {  677, 7226,  227201,  6, }, // Antarctica/Rothera
    {  677, 7295,  150038,  4, }, // Antarctica/Syowa
    {  677, 8607,  227207,  4, }, // Antarctica/Troll
    {  677, 7374,  227211,  7, }, // Antarctica/Vostok
    {  677, 1429,  227218, 10, }, // Arctic/Longyearbyen
    {  677, 5570,  150070,  6, }, // Asia/Aden
    {  677, 5395,  150076,  6, }, // Asia/Almaty
    {  677, 3968,  150082,  6, }, // Asia/Amman
    {  677, 5511,  150088,  8, }, // Asia/Anadyr
    {  677, 5523,  150096,  5, }, // Asia/Aqtau
    {  677, 5368,  227228,  6, }, // Asia/Aqtobe
    {  677,  964,  227234,  9, }, // Asia/Ashgabat
    {  677, 8624,  150119,  6, }, // Asia/Atyrau
    {  677, 3607,  150125,  8, }, // Asia/Baghdad
    {  677, 5534,  227243,  6, }, // Asia/Bahrain
    {  677, 3653,  148863,  4, }, // Asia/Baku
    {  677, 4466,  150141, 11, }, // Asia/Bangkok
    {  677, 3571,  150152,  8, }, // Asia/Barnaul
    {  677, 4086,  150160,  7, }, // Asia/Beirut
    {  677, 3778,  227249,  8, }, // Asia/Bishkek
    {  677, 5989,  150175,  7, }, // Asia/Brunei
    {  677, 4601,  150182,  4, }, // Asia/Chita
    {  677, 4511,  150186,  6, }, // Asia/Colombo
    {  677, 4540,  150192,  9, }, // Asia/Damascus
    {  677, 1093,  150720,  4, }, // Asia/Dhaka
    {  677, 6192,  150205,  6, }, // Asia/Dili
    {  677, 3596,  150211,  5, }, // Asia/Dubai
    {  677, 6159,  227257,  7, }, // Asia/Dushanbe
    {  677, 8636,  227264, 10, }, // Asia/Famagusta
    {  677, 8651,  150233,  4, }, // Asia/Gaza
    {  677, 4795,  150237,  8, }, // Asia/Hebron
    {  677, 1259,  227274, 15, }, // Asia/Ho_Chi_Minh
    {  677, 2704,  150261,  5, }, // Asia/Hong_Kong
    {  677, 4771,  227289,  6, }, // Asia/Hovd
    {  677, 4184,  150272,  8, }, // Asia/Irkutsk
    {  677, 7006,  227295,  7, }, // Asia/Jakarta
    {  677, 6992,  150289,  6, }, // Asia/Jayapura
    {  677, 1290,  150295,  8, }, // Asia/Jerusalem
    {  677, 3560,  227302,  5, }, // Asia/Kabul
    {  677, 4316,  150308,  8, }, // Asia/Kamchatka
    {  677, 4239,  150316,  5, }, // Asia/Karachi
    {  677, 1185,  227307,  9, }, // Asia/Kathmandu
    {  677, 8661,  227316,  9, }, // Asia/Khandyga
    {  677,  992,  150339,  7, }, // Asia/Kolkata
    {  677, 4197,  227325, 12, }, // Asia/Krasnoyarsk
    {  677, 4859,  150359, 12, }, // Asia/Kuala_Lumpur
    {  677, 5976,  150371,  5, }, // Asia/Kuching
    {  677, 5547,  150376,  4, }, // Asia/Kuwait
    {  677, 1211,  150380,  5, }, // Asia/Macau
    {  677, 4017,  150385,  8, }, // Asia/Magadan
    {  677, 1349,  150393,  9, }, // Asia/Makassar
    {  677, 7182,  150402,  6, }, // Asia/Manila
    {  677, 6920,  150408,  8, }, // Asia/Muscat
    {  677, 2554,  227337,  6, }, // Asia/Nicosia
    {  677, 8675,  227343, 11, }, // Asia/Novokuznetsk
    {  677, 4135,  227354, 10, }, // Asia/Novosibirsk
    {  677, 4229,  150448,  5, }, // Asia/Omsk
    {  677, 7145,  227364,  6, }, // Asia/Oral
    {  677, 6961,  227370, 10, }, // Asia/Phnom_Penh
    {  677, 8693,  227380, 11, }, // Asia/Pontianak
    {  677, 4214,  150482, 11, }, // Asia/Pyongyang
    {  677, 5559,  150493,  5, }, // Asia/Qatar
    {  677, 8708,  227391,  6, }, // Asia/Qostanay
    {  677, 4269,  227397,  9, }, // Asia/Qyzylorda
    {  677, 3584,  150515,  4, }, // Asia/Riyadh
    {  677, 4408,  227406,  7, }, // Asia/Sakhalin
    {  677, 7245,  150526,  7, }, // Asia/Samarkand
    {  677, 3279,  227413,  4, }, // Asia/Seoul
    {  677, 1053,  227417,  5, }, // Asia/Shanghai
    {  677, 3300,  150543, 11, }, // Asia/Singapore
    {  677, 4297,  227422, 14, }, // Asia/Srednekolymsk
    {  677, 3263,  150569,  6, }, // Asia/Taipei
    {  677, 4781,  227436,  8, }, // Asia/Tashkent
    {  677, 3915,  150583,  8, }, // Asia/Tbilisi
    {  677, 2772,  150591,  8, }, // Asia/Tehran
    {  677, 1317,  227444,  6, }, // Asia/Thimphu
    {  677, 2821,  227450,  4, }, // Asia/Tokyo
    {  677, 4572,  227454,  5, }, // Asia/Tomsk
    {  677, 1021,  150616, 12, }, // Asia/Ulaanbaatar
    {  677, 1159,  150628,  6, }, // Asia/Urumqi
    {  677, 8722,  227459, 11, }, // Asia/Ust-Nera
    {  677, 6977,  227470, 11, }, // Asia/Vientiane
    {  677, 4710,  227481, 13, }, // Asia/Vladivostok
    {  677, 4828,  150669,  8, }, // Asia/Yakutsk
    {  677, 1235,  150677,  9, }, // Asia/Yangon
    {  677, 3883,  150686, 14, }, // Asia/Yekaterinburg
    {  677, 3747,  227494,  8, }, // Asia/Yerevan
    {  677, 3663,  227502,  6, }, // Atlantic/Azores
    {  677, 5671,  227508,  6, }, // Atlantic/Bermuda
    {  677, 6628,  150722,  6, }, // Atlantic/Canary
    {  677, 3727,  227514, 11, }, // Atlantic/Cape_Verde
    {  677, 1395,  227525,  3, }, // Atlantic/Faroe
    {  677, 8736,  150744,  7, }, // Atlantic/Madeira
    {  677, 2748,  227528,  9, }, // Atlantic/Reykjavik
    {  677, 7272,  150761, 13, }, // Atlantic/South_Georgia
    {  677, 6821,  150774, 14, }, // Atlantic/St_Helena
    {  677, 6644,  227537,  8, }, // Atlantic/Stanley
    {  677, 1670,  227545,  7, }, // Australia/Adelaide
    {  677, 1635,  227552,  8, }, // Australia/Brisbane
    {  677, 1799,  227560, 10, }, // Australia/Broken_Hill
    {  677, 1583,  227570,  8, }, // Australia/Darwin
    {  677, 3637,  150836,  5, }, // Australia/Eucla
    {  677, 1516,  150841,  7, }, // Australia/Hobart
    {  677, 8753,  227578,  9, }, // Australia/Lindeman
    {  677, 1547,  227587, 10, }, // Australia/Lord_Howe
    {  677, 1727,  227597,  7, }, // Australia/Melbourne
    {  677, 1762,  227604,  4, }, // Australia/Perth
    {  677, 1463,  150888,  5, }, // Australia/Sydney
    {  677, 6441,  227608, 12, }, // Europe/Amsterdam
    {  677, 6216,  227620,  6, }, // Europe/Andorra
    {  677, 3620,  227626, 10, }, // Europe/Astrakhan
    {  677, 2300,  227636,  6, }, // Europe/Athens
    {  677, 6336,  227642,  9, }, // Europe/Belgrade
    {  677, 4757,  150936,  7, }, // Europe/Berlin
    {  677, 6488,  227651, 11, }, // Europe/Bratislava
    {  677, 2167,  227662, 10, }, // Europe/Brussels
    {  677, 3928,  227672,  9, }, // Europe/Bucharest
    {  677, 3806,  227681,  9, }, // Europe/Budapest
    {  677, 8772,  150982,  9, }, // Europe/Busingen
    {  677, 2583,  227690,  7, }, // Europe/Chisinau
    {  677, 6289,  227697, 10, }, // Europe/Copenhagen
    {  677, 2338,  151008,  6, }, // Europe/Dublin
    {  677, 6307,  151014, 11, }, // Europe/Gibraltar
    {  677, 8788,  151025,  7, }, // Europe/Guernsey
    {  677, 6612,  151032,  8, }, // Europe/Helsinki
    {  677, 8804,  227707, 15, }, // Europe/Isle_of_Man
    {  677, 1130,  151053,  9, }, // Europe/Istanbul
    {  677, 8823,  151062,  5, }, // Europe/Jersey
    {  677, 3979,  151067, 13, }, // Europe/Kaliningrad
    {  677, 8929,  227722,  4, }, // Europe/Kirov
    {  677, 2527,  151085,  5, }, // Europe/Kyiv
    {  677, 3213,  151090,  7, }, // Europe/Lisbon
    {  677, 6506,  151097, 10, }, // Europe/Ljubljana
    {  677, 2501,  151107,  6, }, // Europe/London
    {  677, 6365,  151113,  9, }, // Europe/Luxembourg
    {  677, 4892,  151122,  8, }, // Europe/Madrid
    {  677, 6397,  151130,  5, }, // Europe/Malta
    {  677, 6582,  227726,  9, }, // Europe/Mariehamn
    {  677, 3693,  151145,  6, }, // Europe/Minsk
    {  677, 6410,  227735,  4, }, // Europe/Monaco
    {  677, 3537,  150408,  4, }, // Europe/Moscow
    {  677, 6458,  227739,  4, }, // Europe/Oslo
    {  677, 4284,  151166,  6, }, // Europe/Paris
    {  677, 6424,  227743,  7, }, // Europe/Podgorica
    {  677, 6275,  151181,  6, }, // Europe/Prague
    {  677, 8837,  151187,  4, }, // Europe/Riga
    {  677, 6324,  227750,  3, }, // Europe/Rome
    {  677, 4331,  151195,  6, }, // Europe/Samara
    {  677, 6470,  151201, 11, }, // Europe/San_Marino
    {  677, 6245,  227753,  7, }, // Europe/Sarajevo
    {  677, 4451,  227760,  7, }, // Europe/Saratov
    {  677, 8849,  227767,  9, }, // Europe/Simferopol
    {  677, 6383,  227776,  6, }, // Europe/Skopje
    {  677, 6599,  227782,  5, }, // Europe/Sofia
    {  677, 6523,  227787, 10, }, // Europe/Stockholm
    {  677, 8867,  151260,  7, }, // Europe/Tallinn
    {  677, 6202,  227797,  7, }, // Europe/Tirane
    {  677, 8882,  227804, 10, }, // Europe/Ulyanovsk
    {  677, 6352,  227814,  5, }, // Europe/Vaduz
    {  677, 6567,  227819,  8, }, // Europe/Vatican
    {  677, 6231,  227827,  5, }, // Europe/Vienna
    {  677, 8899,  227832,  8, }, // Europe/Vilnius
    {  677, 4727,  227840,  9, }, // Europe/Volgograd
    {  677, 3190,  151321,  6, }, // Europe/Warsaw
    {  677, 6261,  151327,  8, }, // Europe/Zagreb
    {  677, 6540,  151335,  6, }, // Europe/Zurich
    {  677, 5078,  151341, 12, }, // Indian/Antananarivo
    {  677, 6947,  151354,  7, }, // Indian/Chagos
    {  677, 6046,  227849, 12, }, // Indian/Christmas
    {  677, 6063,  227861,  5, }, // Indian/Cocos
    {  677, 5029,  227866,  3, }, // Indian/Comoro
    {  677, 6661,  227869,  8, }, // Indian/Kerguelen
    {  677, 7260,  151393,  4, }, // Indian/Mahe
    {  677, 7050,  151397,  8, }, // Indian/Maldives
    {  677, 4069,  227877,  6, }, // Indian/Mauritius
    {  677, 5098,  227883,  5, }, // Indian/Mayotte
    {  677, 7211,  151418, 10, }, // Indian/Reunion
    {  677, 4422,  151428,  4, }, // Pacific/Apia
    {  677,  932,  151432,  7, }, // Pacific/Auckland
    {  677, 3706,  227888, 12, }, // Pacific/Bougainville
    {  677, 3015,  227900,  7, }, // Pacific/Chatham
    {  677, 3157,  151459,  5, }, // Pacific/Chuuk
    {  677, 2237,  151464,  7, }, // Pacific/Easter
    {  677, 7360,  151471,  6, }, // Pacific/Efate
    {  677, 7327,  227907,  5, }, // Pacific/Fakaofo
    {  677, 3902,  227912,  4, }, // Pacific/Fiji
    {  677, 7343,  151488,  8, }, // Pacific/Funafuti
    {  677, 6678,  227916,  8, }, // Pacific/Galapagos
    {  677, 6696,  151505, 10, }, // Pacific/Gambier
    {  677, 3822,  151515, 11, }, // Pacific/Guadalcanal
    {  677, 6018,  149348,  4, }, // Pacific/Guam
    {  677, 3049,  227924,  6, }, // Pacific/Kanton
    {  677, 3998,  151544, 10, }, // Pacific/Kiritimati
    {  677, 7019,  151554,  6, }, // Pacific/Kosrae
    {  677, 2842,  151560, 11, }, // Pacific/Kwajalein
    {  677, 4877,  151571,  5, }, // Pacific/Majuro
    {  677, 4051,  151577, 13, }, // Pacific/Marquesas
    {  677, 8914,  151590,  7, }, // Pacific/Midway
    {  677, 7084,  151597,  5, }, // Pacific/Nauru
    {  677, 7132,  151602,  4, }, // Pacific/Niue
    {  677, 4168,  227930,  6, }, // Pacific/Norfolk
    {  677, 7098,  227936,  5, }, // Pacific/Noumea
    {  677, 3126,  151618,  9, }, // Pacific/Pago_Pago
    {  677, 7155,  151627,  5, }, // Pacific/Palau
    {  677, 7194,  151632, 10, }, // Pacific/Pitcairn
    {  677, 3096,  227941,  6, }, // Pacific/Pohnpei
    {  677, 4807,  227947, 13, }, // Pacific/Port_Moresby
    {  677, 6076,  227960,  8, }, // Pacific/Rarotonga
    {  677, 6031,  151674,  7, }, // Pacific/Saipan
    {  677, 7312,  151681,  6, }, // Pacific/Tahiti
    {  677, 6712,  151687,  5, }, // Pacific/Tarawa
    {  677, 4583,  227968,  9, }, // Pacific/Tongatapu
    {  677, 7392,  151702,  4, }, // Pacific/Wake
    {  677, 7405,  151706,  7, }, // Pacific/Wallis
    {  678, 6788,  186347,  9, }, // Africa/Abidjan Kuvi/Telugu/India
    {  678, 6760,  186356,  5, }, // Africa/Accra
    {  678, 5059,  186361, 12, }, // Africa/Addis_Ababa
    {  678, 7420,  186373, 10, }, // Africa/Algiers
    {  678,   14,  186383,  7, }, // Africa/Asmara
    {  678,   44,  186390,  6, }, // Africa/Bamako
    {  678, 5257,  186396,  7, }, // Africa/Bangui
    {  678, 6746,  186403,  6, }, // Africa/Banjul
    {  678, 7435,  186409,  8, }, // Africa/Bissau
    {  678, 4957,  186417, 10, }, // Africa/Blantyre
    {  678, 5287,  186427, 12, }, // Africa/Brazzaville
    {  678, 4922,  186439,  9, }, // Africa/Bujumbura
    {  678, 2320,  186448,  4, }, // Africa/Cairo
    {  678, 4117,  186452, 11, }, // Africa/Casablanca
    {  678, 7449,  186463,  6, }, // Africa/Ceuta
    {  678, 6773,  186469,  8, }, // Africa/Conakry
    {  678, 6840,  186477,  5, }, // Africa/Dakar
    {  678, 5130,  186482, 14, }, // Africa/Dar_es_Salaam
    {  678, 5043,  186496,  8, }, // Africa/Djibouti
    {  678, 5243,  186504,  4, }, // Africa/Douala
    {  678, 5166,  186508,  9, }, // Africa/El_Aaiun
    {  678, 6853,  186517,  8, }, // Africa/Freetown
    {  678, 4906,  186525,  7, }, // Africa/Gaborone
    {  678, 5015,  186532,  5, }, // Africa/Harare
    {  678, 4479,  186537, 14, }, // Africa/Johannesburg
    {  678, 4499,  186551,  4, }, // Africa/Juba
    {  678, 5151,  186555,  6, }, // Africa/Kampala
    {  678, 4524,  186561,  8, }, // Africa/Khartoum
    {  678, 4973,  186569,  5, }, // Africa/Kigali
    {  678, 5306,  186574,  8, }, // Africa/Kinshasa
    {  678, 4744,  186582,  6, }, // Africa/Lagos
    {  678, 5336,  186588, 11, }, // Africa/Libreville
    {  678, 6869,  186599,  4, }, // Africa/Lome
    {  678, 5211,  186603,  7, }, // Africa/Luanda
    {  678, 4939,  186610,  8, }, // Africa/Lubumbashi
    {  678, 5001,  186618,  6, }, // Africa/Lusaka
    {  678, 5322,  186624,  5, }, // Africa/Malabo
    {  678, 4987,  186629,  6, }, // Africa/Maputo
    {  678, 5197,  186635,  5, }, // Africa/Maseru
    {  678, 5182,  186640,  6, }, // Africa/Mbabane
    {  678, 5113,  186646,  8, }, // Africa/Mogadishu
    {  678, 7034,  186654, 10, }, // Africa/Monrovia
    {  678, 3853,  186664,  6, }, // Africa/Nairobi
    {  678, 5271,  186670,  8, }, // Africa/Ndjamena
    {  678, 5354,  186678,  6, }, // Africa/Niamey
    {  678, 6803,  186684, 10, }, // Africa/Nouakchott
    {  678, 6727,  186694,  9, }, // Africa/Ouagadougou
    {  678, 5225,  186703, 11, }, // Africa/Porto-Novo
    {  678, 4435,  186714,  9, }, // Africa/Sao_Tome
    {  678, 2866,  186723,  8, }, // Africa/Tripoli
    {  678, 6554,  186731,  8, }, // Africa/Tunis
    {  678, 4152,  186739,  9, }, // Africa/Windhoek
    {  678,  132,  186748,  5, }, // America/Adak
    {  678, 3346,  186753,  8, }, // America/Anchorage
    {  678, 5607,  227977, 10, }, // America/Anguilla
    {  678, 5624,  186771,  8, }, // America/Antigua
    {  678, 4554,  186779, 11, }, // America/Araguaina
    {  678,  166,  186790, 16, }, // America/Argentina/Buenos_Aires
    {  678,   91,  186806,  8, }, // America/Argentina/Catamarca
    {  678,  270,  186814,  8, }, // America/Argentina/Cordoba
    {  678,  441,  186822,  6, }, // America/Argentina/Jujuy
    {  678, 7462,  186828,  8, }, // America/Argentina/La_Rioja
    {  678,  565,  186836,  7, }, // America/Argentina/Mendoza
    {  678, 7489,  186843, 14, }, // America/Argentina/Rio_Gallegos
    {  678, 7520,  186857,  6, }, // America/Argentina/Salta
    {  678, 7544,  186863, 11, }, // America/Argentina/San_Juan
    {  678, 5580,  186874, 11, }, // America/Argentina/San_Luis
    {  678, 7571,  186885,  7, }, // America/Argentina/Tucuman
    {  678, 7597,  186892,  6, }, // America/Argentina/Ushuaia
    {  678, 5640,  186898,  5, }, // America/Aruba
    {  678, 4252,  186903, 11, }, // America/Asuncion
    {  678,  237,  186914,  8, }, // America/Atikokan
    {  678, 3679,  186922,  4, }, // America/Bahia
    {  678, 7623,  186926, 13, }, // America/Bahia_Banderas
    {  678, 5654,  186939,  9, }, // America/Barbados
    {  678, 7646,  186948,  6, }, // America/Belem
    {  678, 5407,  186954,  6, }, // America/Belize
    {  678, 7660,  186960, 15, }, // America/Blanc-Sablon
    {  678, 7681,  186975, 11, }, // America/Boa_Vista
    {  678, 4361,  186986,  5, }, // America/Bogota
    {  678, 7699,  186991,  6, }, // America/Boise
    {  678, 7713,  186997, 14, }, // America/Cambridge_Bay
    {  678, 7735,  187011, 13, }, // America/Campo_Grande
    {  678, 3868,  187024,  8, }, // America/Cancun
    {  678, 4694,  187032,  7, }, // America/Caracas
    {  678, 4345,  187039,  5, }, // America/Cayenne
    {  678, 5496,  187044,  6, }, // America/Cayman
    {  678, 2260,  187050,  6, }, // America/Chicago
    {  678, 7756,  187056,  6, }, // America/Chihuahua
    {  678, 5422,  187079, 11, }, // America/Costa_Rica
    {  678, 7796,  187090,  9, }, // America/Creston
    {  678, 3791,  187099,  5, }, // America/Cuiaba
    {  678, 5723,  187104,  7, }, // America/Curacao
    {  678, 7812,  187111, 15, }, // America/Danmarkshavn
    {  678, 7833,  187126,  5, }, // America/Dawson
    {  678, 7848,  187131, 12, }, // America/Dawson_Creek
    {  678,  805,  187143,  8, }, // America/Denver
    {  678, 3465,  187151, 10, }, // America/Detroit
    {  678, 5739,  187161,  8, }, // America/Dominica
    {  678,  893,  187169,  9, }, // America/Edmonton
    {  678, 7869,  187178,  7, }, // America/Eirunepe
    {  678, 5441,  187185, 13, }, // America/El_Salvador
    {  678, 7886,  187198, 14, }, // America/Fort_Nelson
    {  678, 7906,  187212,  9, }, // America/Fortaleza
    {  678, 7924,  187221,  9, }, // America/Glace_Bay
    {  678, 6881,  187230,  7, }, // America/Goose_Bay
    {  678, 4612,  187237, 13, }, // America/Grand_Turk
    {  678, 5770,  187250,  7, }, // America/Grenada
    {  678, 5786,  187257, 10, }, // America/Guadeloupe
    {  678, 3760,  187267, 10, }, // America/Guatemala
    {  678, 4841,  187277,  9, }, // America/Guayaquil
    {  678, 6932,  187286,  5, }, // America/Guyana
    {  678, 1939,  187291, 10, }, // America/Halifax
    {  678, 2281,  187301,  5, }, // America/Havana
    {  678, 7942,  187306, 12, }, // America/Hermosillo
    {  678,  348,  187318, 13, }, // America/Indiana/Indianapolis
    {  678,  481,  187331, 15, }, // America/Indiana/Knox
    {  678, 7961,  187346, 15, }, // America/Indiana/Marengo
    {  678, 7985,  187361, 22, }, // America/Indiana/Petersburg
    {  678, 8012,  187383, 18, }, // America/Indiana/Tell_City
    {  678, 8038,  187401, 14, }, // America/Indiana/Vevay
    {  678, 8060,  187415, 22, }, // America/Indiana/Vincennes
    {  678, 8086,  187437, 17, }, // America/Indiana/Winamac
    {  678, 8110,  187454,  7, }, // America/Inuvik
    {  678,  660,  187461,  9, }, // America/Iqaluit
    {  678, 2799,  187470,  5, }, // America/Jamaica
    {  678, 5380,  187475,  4, }, // America/Juneau
    {  678,  521,  187479,  8, }, // America/Kentucky/Louisville
    {  678, 8125,  187487, 24, }, // America/Kentucky/Monticello
    {  678, 5704,  187511, 12, }, // America/Kralendijk
    {  678, 4376,  187523,  7, }, // America/La_Paz
    {  678, 7169,  187530,  4, }, // America/Lima
    {  678, 3239,  187534, 12, }, // America/Los_Angeles
    {  678, 5932,  187546, 24, }, // America/Lower_Princes
    {  678, 8153,  187570,  6, }, // America/Maceio
    {  678, 8168,  187576,  7, }, // America/Managua
    {  678, 1908,  187583,  5, }, // America/Manaus
    {  678, 5897,  187588,  8, }, // America/Marigot
    {  678, 5805,  187596, 10, }, // America/Martinique
    {  678, 8184,  187606,  9, }, // America/Matamoros
    {  678, 2917,  187615, 10, }, // America/Mazatlan
    {  678, 8202,  187625,  8, }, // America/Menominee
    {  678, 8220,  187633,  6, }, // America/Merida
    {  678, 8235,  187639,  9, }, // America/Metlakatla
    {  678, 2949,  187648, 13, }, // America/Mexico_City
    {  678, 4391,  187661,  8, }, // America/Miquelon
    {  678, 8254,  187669, 11, }, // America/Moncton
    {  678, 8270,  187680,  9, }, // America/Monterrey
    {  678, 4098,  187689, 11, }, // America/Montevideo
    {  678, 5824,  187700, 11, }, // America/Montserrat
    {  678, 5481,  187711,  8, }, // America/Nassau
    {  678, 2379,  187719, 10, }, // America/New_York
    {  678, 8288,  187729,  4, }, // America/Nome
    {  678, 1850,  187733,  7, }, // America/Noronha
    {  678, 8301,  187740, 18, }, // America/North_Dakota/Beulah
    {  678, 8329,  187758, 19, }, // America/North_Dakota/Center
    {  678, 8357,  187777, 22, }, // America/North_Dakota/New_Salem
    {  678,  393,  187799,  4, }, // America/Nuuk
    {  678, 8388,  187803,  5, }, // America/Ojinaga
    {  678, 2356,  187808,  5, }, // America/Panama
    {  678, 6173,  187813,  9, }, // America/Paramaribo
    {  678, 2973,  187822,  8, }, // America/Phoenix
    {  678, 3945,  187830, 19, }, // America/Port-au-Prince
    {  678, 5954,  187849, 19, }, // America/Port_of_Spain
    {  678, 8404,  187868, 13, }, // America/Porto_Velho
    {  678, 5843,  187881, 13, }, // America/Puerto_Rico
    {  678, 4030,  187894, 12, }, // America/Punta_Arenas
    {  678, 8424,  187906, 17, }, // America/Rankin_Inlet
    {  678, 8445,  187923,  6, }, // America/Recife
    {  678, 1995,  187929,  6, }, // America/Regina
    {  678, 8460,  187935,  9, }, // America/Resolute
    {  678,  695,  187944, 12, }, // America/Rio_Branco
    {  678, 8477,  187956,  8, }, // America/Santarem
    {  678, 2201,  187964,  9, }, // America/Santiago
    {  678, 6111,  187973, 13, }, // America/Santo_Domingo
    {  678, 1878,  187986,  9, }, // America/Sao_Paulo
    {  678, 6899,  187995, 17, }, // America/Scoresbysund
    {  678, 8494,  188012,  6, }, // America/Sitka
    {  678, 8508,  188018, 17, }, // America/St_Barthelemy
    {  678, 2061,  188035, 14, }, // America/St_Johns
    {  678, 5863,  188049, 14, }, // America/St_Kitts
    {  678, 5880,  188063, 14, }, // America/St_Lucia
    {  678,  855,  188077, 13, }, // America/St_Thomas
    {  678, 5913,  188090, 17, }, // America/St_Vincent
    {  678, 8530,  188107, 15, }, // America/Swift_Current
    {  678, 5461,  188122, 11, }, // America/Tegucigalpa
    {  678, 5756,  188133,  4, }, // America/Thule
    {  678,  313,  188137,  8, }, // America/Tijuana
    {  678,  608,  188145,  6, }, // America/Toronto
    {  678, 5688,  188151,  8, }, // America/Tortola
    {  678, 2093,  188159,  9, }, // America/Vancouver
    {  678, 2144,  188168, 11, }, // America/Whitehorse
    {  678,  734,  188179, 10, }, // America/Winnipeg
    {  678, 8552,  188189,  7, }, // America/Yakutat
    {  678, 6001,  188196,  4, }, // Antarctica/Casey
    {  678, 6094,  188200,  6, }, // Antarctica/Davis
    {  678, 6133,  188206, 22, }, // Antarctica/DumontDUrville
    {  678, 8568,  188228,  5, }, // Antarctica/Macquarie
    {  678, 7066,  188233,  5, }, // Antarctica/Mawson
    {  678, 7113,  188238, 11, }, // Antarctica/McMurdo
    {  678, 8589,  188249,  7, }, // Antarctica/Palmer
    {  678, 7226,  188256,  6, }, // Antarctica/Rothera
    {  678, 7295,  188262,  6, }, // Antarctica/Syowa
    {  678, 8607,  188268,  6, }, // Antarctica/Troll
    {  678, 7374,  188274,  8, }, // Antarctica/Vostok
    {  678, 1429,  188282, 17, }, // Arctic/Longyearbyen
    {  678, 5570,  188299,  5, }, // Asia/Aden
    {  678, 5395,  188304,  7, }, // Asia/Almaty
    {  678, 3968,  188311,  6, }, // Asia/Amman
    {  678, 5511,  188317,  6, }, // Asia/Anadyr
    {  678, 5523,  188323,  7, }, // Asia/Aqtau
    {  678, 5368,  188330,  7, }, // Asia/Aqtobe
    {  678,  964,  188337, 10, }, // Asia/Ashgabat
    {  678, 8624,  188347,  5, }, // Asia/Atyrau
    {  678, 3607,  188352,  8, }, // Asia/Baghdad
    {  678, 5534,  188360,  9, }, // Asia/Bahrain
    {  678, 3653,  187102,  4, }, // Asia/Baku
    {  678, 4466,  188369,  9, }, // Asia/Bangkok
    {  678, 3571,  188378,  8, }, // Asia/Barnaul
    {  678, 4086,  188386,  5, }, // Asia/Beirut
    {  678, 3778,  188391,  8, }, // Asia/Bishkek
    {  678, 5989,  188399,  6, }, // Asia/Brunei
    {  678, 4601,  188405,  4, }, // Asia/Chita
    {  678, 4511,  188409,  6, }, // Asia/Colombo
    {  678, 4540,  188415,  8, }, // Asia/Damascus
    {  678, 1093,  188423,  4, }, // Asia/Dhaka
    {  678, 6192,  188427,  4, }, // Asia/Dili
    {  678, 3596,  188431,  6, }, // Asia/Dubai
    {  678, 6159,  188437,  7, }, // Asia/Dushanbe
    {  678, 8636,  188444,  9, }, // Asia/Famagusta
    {  678, 8651,  188453,  4, }, // Asia/Gaza
    {  678, 4795,  188457,  8, }, // Asia/Hebron
    {  678, 1259,  188465, 17, }, // Asia/Ho_Chi_Minh
    {  678, 2704,  188482,  8, }, // Asia/Hong_Kong
    {  678, 4771,  188490,  6, }, // Asia/Hovd
    {  678, 4184,  188496, 11, }, // Asia/Irkutsk
    {  678, 7006,  188507,  7, }, // Asia/Jakarta
    {  678, 6992,  188514,  5, }, // Asia/Jayapura
    {  678, 1290,  188519,  7, }, // Asia/Jerusalem
    {  678, 3560,  188526,  6, }, // Asia/Kabul
    {  678, 4316,  188532,  9, }, // Asia/Kamchatka
    {  678, 4239,  188541,  5, }, // Asia/Karachi
    {  678, 1185,  188546,  8, }, // Asia/Kathmandu
    {  678, 8661,  188554,  7, }, // Asia/Khandyga
    {  678,  992,  188561,  8, }, // Asia/Kolkata
    {  678, 4197,  188569, 14, }, // Asia/Krasnoyarsk
    {  678, 4859,  188583, 10, }, // Asia/Kuala_Lumpur
    {  678, 5976,  188593,  7, }, // Asia/Kuching
    {  678, 5547,  188600,  6, }, // Asia/Kuwait
    {  678, 1211,  188606,  5, }, // Asia/Macau
    {  678, 4017,  188611,  6, }, // Asia/Magadan
    {  678, 1349,  188617,  8, }, // Asia/Makassar
    {  678, 7182,  188625,  5, }, // Asia/Manila
    {  678, 6920,  188630,  6, }, // Asia/Muscat
    {  678, 2554,  188636,  8, }, // Asia/Nicosia
    {  678, 8675,  188644, 18, }, // Asia/Novokuznetsk
    {  678, 4135,  188662, 13, }, // Asia/Novosibirsk
    {  678, 4229,  188675,  7, }, // Asia/Omsk
    {  678, 7145,  188682,  4, }, // Asia/Oral
    {  678, 6961,  188686, 11, }, // Asia/Phnom_Penh
    {  678, 8693,  188697, 11, }, // Asia/Pontianak
    {  678, 4214,  188708, 10, }, // Asia/Pyongyang
    {  678, 5559,  188718,  5, }, // Asia/Qatar
    {  678, 8708,  188723,  8, }, // Asia/Qostanay
    {  678, 4269,  188731, 12, }, // Asia/Qyzylorda
    {  678, 3584,  188743,  6, }, // Asia/Riyadh
    {  678, 4408,  188749,  7, }, // Asia/Sakhalin
    {  678, 7245,  188756,  9, }, // Asia/Samarkand
    {  678, 3279,  188765,  6, }, // Asia/Seoul
    {  678, 1053,  188771,  5, }, // Asia/Shanghai
    {  678, 3300,  188776,  8, }, // Asia/Singapore
    {  678, 4297,  188784, 18, }, // Asia/Srednekolymsk
    {  678, 3263,  188802,  4, }, // Asia/Taipei
    {  678, 4781,  188806,  9, }, // Asia/Tashkent
    {  678, 3915,  188815,  8, }, // Asia/Tbilisi
    {  678, 2772,  188823,  8, }, // Asia/Tehran
    {  678, 1317,  188831,  5, }, // Asia/Thimphu
    {  678, 2821,  188836,  6, }, // Asia/Tokyo
    {  678, 4572,  188842,  8, }, // Asia/Tomsk
    {  678, 1021,  188850, 10, }, // Asia/Ulaanbaatar
    {  678, 1159,  188860,  8, }, // Asia/Urumqi
    {  678, 8722,  188868, 10, }, // Asia/Ust-Nera
    {  678, 6977,  188878,  9, }, // Asia/Vientiane
    {  678, 4710,  188887, 14, }, // Asia/Vladivostok
    {  678, 4828,  188901,  9, }, // Asia/Yakutsk
    {  678, 1235,  188910,  6, }, // Asia/Yangon
    {  678, 3883,  188916, 13, }, // Asia/Yekaterinburg
    {  678, 3747,  188929,  7, }, // Asia/Yerevan
    {  678, 3663,  188936,  7, }, // Atlantic/Azores
    {  678, 5671,  188943,  8, }, // Atlantic/Bermuda
    {  678, 6628,  188951,  5, }, // Atlantic/Canary
    {  678, 3727,  188956, 11, }, // Atlantic/Cape_Verde
    {  678, 1395,  188967,  4, }, // Atlantic/Faroe
    {  678, 8736,  188971,  5, }, // Atlantic/Madeira
    {  678, 2748,  188976, 10, }, // Atlantic/Reykjavik
    {  678, 7272,  188986, 15, }, // Atlantic/South_Georgia
    {  678, 6821,  189001, 14, }, // Atlantic/St_Helena
    {  678, 6644,  189015,  8, }, // Atlantic/Stanley
    {  678, 1670,  227987, 36, }, // Australia/Adelaide
    {  678, 1635,  189030, 13, }, // Australia/Brisbane
    {  678, 1799,  189043, 13, }, // Australia/Broken_Hill
    {  678, 1583,  189056,  8, }, // Australia/Darwin
    {  678, 3637,  189064,  6, }, // Australia/Eucla
    {  678, 1516,  189070,  7, }, // Australia/Hobart
    {  678, 8753,  189077,  9, }, // Australia/Lindeman
    {  678, 1547,  189086,  9, }, // Australia/Lord_Howe
    {  678, 1727,  189095, 10, }, // Australia/Melbourne
    {  678, 1762,  189105,  6, }, // Australia/Perth
    {  678, 1463,  189111,  6, }, // Australia/Sydney
    {  678, 6441,  189117, 14, }, // Europe/Amsterdam
    {  678, 6216,  189131,  6, }, // Europe/Andorra
    {  678, 3620,  189137, 10, }, // Europe/Astrakhan
    {  678, 2300,  189147,  7, }, // Europe/Athens
    {  678, 6336,  189154, 11, }, // Europe/Belgrade
    {  678, 4757,  189165,  8, }, // Europe/Berlin
    {  678, 6488,  189173, 12, }, // Europe/Bratislava
    {  678, 2167,  189185, 11, }, // Europe/Brussels
    {  678, 3928,  189196, 10, }, // Europe/Bucharest
    {  678, 3806,  189206, 10, }, // Europe/Budapest
    {  678, 8772,  189216,  7, }, // Europe/Busingen
    {  678, 2583,  189223,  8, }, // Europe/Chisinau
    {  678, 6289,  189231, 11, }, // Europe/Copenhagen
    {  678, 2338,  189242,  7, }, // Europe/Dublin
    {  678, 6307,  189249, 11, }, // Europe/Gibraltar
    {  678, 8788,  189260, 10, }, // Europe/Guernsey
    {  678, 6612,  189270,  9, }, // Europe/Helsinki
    {  678, 8804,  189279, 12, }, // Europe/Isle_of_Man
    {  678, 1130,  189291, 10, }, // Europe/Istanbul
    {  678, 8823,  189301,  6, }, // Europe/Jersey
    {  678, 3979,  189307, 13, }, // Europe/Kaliningrad
    {  678, 8929,  189320,  6, }, // Europe/Kirov
    {  678, 2527,  189326,  4, }, // Europe/Kyiv
    {  678, 3213,  189330,  7, }, // Europe/Lisbon
    {  678, 6506,  189337, 12, }, // Europe/Ljubljana
    {  678, 2501,  189349,  5, }, // Europe/London
    {  678, 6365,  189354, 11, }, // Europe/Luxembourg
    {  678, 4892,  189365,  8, }, // Europe/Madrid
    {  678, 6397,  189373,  6, }, // Europe/Malta
    {  678, 6582,  189379,  9, }, // Europe/Mariehamn
    {  678, 3693,  189388,  8, }, // Europe/Minsk
    {  678, 6410,  189396,  6, }, // Europe/Monaco
    {  678, 3537,  189402,  6, }, // Europe/Moscow
    {  678, 6458,  189408,  5, }, // Europe/Oslo
    {  678, 4284,  189413,  8, }, // Europe/Paris
    {  678, 6424,  189421, 10, }, // Europe/Podgorica
    {  678, 6275,  189431,  6, }, // Europe/Prague
    {  678, 8837,  189437,  4, }, // Europe/Riga
    {  678, 6324,  189441,  4, }, // Europe/Rome
    {  678, 4331,  189445,  4, }, // Europe/Samara
    {  678, 6470,  189449, 11, }, // Europe/San_Marino
    {  678, 6245,  189460,  7, }, // Europe/Sarajevo
    {  678, 4451,  189467,  6, }, // Europe/Saratov
    {  678, 8849,  189473, 13, }, // Europe/Simferopol
    {  678, 6383,  189486,  9, }, // Europe/Skopje
    {  678, 6599,  189495,  6, }, // Europe/Sofia
    {  678, 6523,  189501, 10, }, // Europe/Stockholm
    {  678, 8867,  189511,  8, }, // Europe/Tallinn
    {  678, 6202,  189519,  6, }, // Europe/Tirane
    {  678, 8882,  189525, 12, }, // Europe/Ulyanovsk
    {  678, 6352,  189537,  6, }, // Europe/Vaduz
    {  678, 6567,  189543,  7, }, // Europe/Vatican
    {  678, 6231,  189550,  7, }, // Europe/Vienna
    {  678, 8899,  189557,  9, }, // Europe/Vilnius
    {  678, 4727,  189566, 12, }, // Europe/Volgograd
    {  678, 3190,  189578,  6, }, // Europe/Warsaw
    {  678, 6261,  189584,  8, }, // Europe/Zagreb
    {  678, 6540,  189592,  8, }, // Europe/Zurich
    {  678, 5078,  189600, 12, }, // Indian/Antananarivo
    {  678, 6947,  189612,  6, }, // Indian/Chagos
    {  678, 6046,  189618,  9, }, // Indian/Christmas
    {  678, 6063,  189627,  6, }, // Indian/Cocos
    {  678, 5029,  189633,  6, }, // Indian/Comoro
    {  678, 6661,  189639, 12, }, // Indian/Kerguelen
    {  678, 7260,  189651,  4, }, // Indian/Mahe
    {  678, 7050,  189655, 10, }, // Indian/Maldives
    {  678, 4069,  189665,  7, }, // Indian/Mauritius
    {  678, 5098,  189672,  5, }, // Indian/Mayotte
    {  678, 7211,  189677,  9, }, // Indian/Reunion
    {  678, 4422,  189686,  5, }, // Pacific/Apia
    {  678,  932,  189691,  8, }, // Pacific/Auckland
    {  678, 3706,  189699, 15, }, // Pacific/Bougainville
    {  678, 3015,  189714,  5, }, // Pacific/Chatham
    {  678, 3157,  189719,  4, }, // Pacific/Chuuk
    {  678, 2237,  189723,  6, }, // Pacific/Easter
    {  678, 7360,  189729,  5, }, // Pacific/Efate
    {  678, 7327,  189734,  6, }, // Pacific/Fakaofo
    {  678, 3902,  189740,  4, }, // Pacific/Fiji
    {  678, 7343,  189744,  8, }, // Pacific/Funafuti
    {  678, 6678,  189752, 10, }, // Pacific/Galapagos
    {  678, 6696,  189762,  9, }, // Pacific/Gambier
    {  678, 3822,  189771, 13, }, // Pacific/Guadalcanal
    {  678, 6018,  189784,  6, }, // Pacific/Guam
    {  678, 3049,  228023,  6, }, // Pacific/Kanton
    {  678, 3998,  189807, 10, }, // Pacific/Kiritimati
    {  678, 7019,  189817,  6, }, // Pacific/Kosrae
    {  678, 2842,  189823, 10, }, // Pacific/Kwajalein
    {  678, 4877,  189833,  5, }, // Pacific/Majuro
    {  678, 4051,  189838, 12, }, // Pacific/Marquesas
    {  678, 8914,  189850,  6, }, // Pacific/Midway
    {  678, 7084,  189856,  4, }, // Pacific/Nauru
    {  678, 7132,  189860,  4, }, // Pacific/Niue
    {  678, 4168,  189864,  8, }, // Pacific/Norfolk
    {  678, 7098,  189872,  6, }, // Pacific/Noumea
    {  678, 3126,  189878,  9, }, // Pacific/Pago_Pago
    {  678, 7155,  189887,  6, }, // Pacific/Palau
    {  678, 7194,  189893, 11, }, // Pacific/Pitcairn
    {  678, 3096,  189904,  7, }, // Pacific/Pohnpei
    {  678, 4807,  189911, 15, }, // Pacific/Port_Moresby
    {  678, 6076,  189926,  8, }, // Pacific/Rarotonga
    {  678, 6031,  189934,  8, }, // Pacific/Saipan
    {  678, 7312,  189942,  5, }, // Pacific/Tahiti
    {  678, 6712,  189947,  5, }, // Pacific/Tarawa
    {  678, 4583,  189952,  9, }, // Pacific/Tongatapu
    {  678, 7392,  189961,  4, }, // Pacific/Wake
    {  678, 7405,  189965,  8, }, // Pacific/Wallis
    {  679,   14,       0,  6, }, // Africa/Asmara Kara-Kalpak/Cyrillic/Uzbekistan
    {  679, 4435,       6,  8, }, // Africa/Sao_Tome
    {  679, 4252,      14,  8, }, // America/Asuncion
    {  679,  237,      22,  8, }, // America/Atikokan
    {  679, 7623,      30, 17, }, // America/Bahia_Banderas
    {  679, 3868,      47,  6, }, // America/Cancun
    {  679, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  679, 5723,      66,  7, }, // America/Curacao
    {  679,  481,      73, 13, }, // America/Indiana/Knox
    {  679, 7961,      86, 16, }, // America/Indiana/Marengo
    {  679, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  679, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  679, 8038,     139, 14, }, // America/Indiana/Vevay
    {  679, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  679, 8086,     171, 16, }, // America/Indiana/Winamac
    {  679, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  679, 5932,     207, 22, }, // America/Lower_Princes
    {  679, 8220,     229,  6, }, // America/Merida
    {  679, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  679, 8329,     255, 20, }, // America/North_Dakota/Center
    {  679, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  679,  393,     298,  4, }, // America/Nuuk
    {  679, 6899,     302, 16, }, // America/Scoresbysund
    {  679, 8508,     318, 14, }, // America/St_Barthelemy
    {  679, 2061,     332, 10, }, // America/St_Johns
    {  679, 5863,     342,  9, }, // America/St_Kitts
    {  679, 5880,     351,  9, }, // America/St_Lucia
    {  679,  855,     360, 10, }, // America/St_Thomas
    {  679, 5913,     370, 11, }, // America/St_Vincent
    {  679, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  679, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  679, 1185,     413,  9, }, // Asia/Kathmandu
    {  679,  992,     422,  7, }, // Asia/Kolkata
    {  679, 1211,     429,  5, }, // Asia/Macau
    {  679, 1235,     442,  6, }, // Asia/Yangon
    {  679, 1395,     448,  5, }, // Atlantic/Faroe
    {  679, 6821,     453, 10, }, // Atlantic/St_Helena
    {  679, 2527,     463,  4, }, // Europe/Kyiv
    {  679, 7211,     467,  7, }, // Indian/Reunion
    {  679, 3157,     474,  5, }, // Pacific/Chuuk
    {  679, 3096,     479,  7, }, // Pacific/Pohnpei
    {  681,   14,       0,  6, }, // Africa/Asmara Swampy Cree/Canadian Aboriginal/Canada
    {  681, 4435,       6,  8, }, // Africa/Sao_Tome
    {  681, 4252,      14,  8, }, // America/Asuncion
    {  681,  237,  228029,  6, }, // America/Atikokan
    {  681, 7623,      30, 17, }, // America/Bahia_Banderas
    {  681, 3868,      47,  6, }, // America/Cancun
    {  681, 7774,      53, 13, }, // America/Ciudad_Juarez
    {  681, 5723,      66,  7, }, // America/Curacao
    {  681,  893,  228035,  6, }, // America/Edmonton
    {  681,  481,      73, 13, }, // America/Indiana/Knox
    {  681, 7961,      86, 16, }, // America/Indiana/Marengo
    {  681, 7985,     102, 19, }, // America/Indiana/Petersburg
    {  681, 8012,     121, 18, }, // America/Indiana/Tell_City
    {  681, 8038,     139, 14, }, // America/Indiana/Vevay
    {  681, 8060,     153, 18, }, // America/Indiana/Vincennes
    {  681, 8086,     171, 16, }, // America/Indiana/Winamac
    {  681, 8110,  228041,  4, }, // America/Inuvik
    {  681,  660,  228045,  6, }, // America/Iqaluit
    {  681, 8125,     187, 20, }, // America/Kentucky/Monticello
    {  681, 5932,     207, 22, }, // America/Lower_Princes
    {  681, 8220,     229,  6, }, // America/Merida
    {  681, 8301,     235, 20, }, // America/North_Dakota/Beulah
    {  681, 8329,     255, 20, }, // America/North_Dakota/Center
    {  681, 8357,     275, 23, }, // America/North_Dakota/New_Salem
    {  681,  393,     298,  4, }, // America/Nuuk
    {  681, 6899,     302, 16, }, // America/Scoresbysund
    {  681, 8508,     318, 14, }, // America/St_Barthelemy
    {  681, 2061,     332, 10, }, // America/St_Johns
    {  681, 5863,     342,  9, }, // America/St_Kitts
    {  681, 5880,     351,  9, }, // America/St_Lucia
    {  681,  855,     360, 10, }, // America/St_Thomas
    {  681, 5913,     370, 11, }, // America/St_Vincent
    {  681, 8530,  228051,  7, }, // America/Swift_Current
    {  681,  734,  228058,  4, }, // America/Winnipeg
    {  681, 6133,     381, 16, }, // Antarctica/DumontDUrville
    {  681, 1259,     397, 11, }, // Asia/Ho_Chi_Minh
    {  681, 1185,     413,  9, }, // Asia/Kathmandu
    {  681,  992,     422,  7, }, // Asia/Kolkata
    {  681, 1211,     429,  5, }, // Asia/Macau
    {  681, 1235,     442,  6, }, // Asia/Yangon
    {  681, 1395,     448,  5, }, // Atlantic/Faroe
    {  681, 6821,     453, 10, }, // Atlantic/St_Helena
    {  681, 2527,     463,  4, }, // Europe/Kyiv
    {  681, 7211,     467,  7, }, // Indian/Reunion
    {  681, 3157,     474,  5, }, // Pacific/Chuuk
    {  681, 3096,     479,  7, }, // Pacific/Pohnpei
}; // Exemplar city table

// Sorted by locale index, then iana name
static inline constexpr LocaleZoneNames localeZoneNameTable[] = {
    // locInd, ianaInd, (lngGen, lngStd, lngDst, srtGen, srtStd, srtDst){ind, sz}
    {    0, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC C/AnyScript/AnyTerritory
    {    0, 2338,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {    0, 2501,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {    0, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {    1, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Abkhazian/Cyrillic/Georgia
    {    1, 2338,       0,       0,      64,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {    1, 2501,       0,       0,      88,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {    2, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Afar/Latin/Ethiopia
    {    5, 2455,       0,     109,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Afrikaans/Latin/South Africa
    {    5, 2338,       0,       0,     138,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {    5, 2501,       0,       0,     156,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/London
    {    7, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Aghem/Latin/Cameroon
    {    8, 2455,       0,     171,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Akan/Latin/Ghana
    {    8, 2338,       0,       0,     194,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {    8, 2501,       0,       0,     217,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {    9, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Akoose/Latin/Cameroon
    {   10, 2455,       0,     234,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Albanian/Latin/Albania
    {   10, 2338,       0,       0,     261,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {   10, 2501,       0,       0,     286,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {   13, 2455,       0,     306,       0,    0,    0,    0,  0, 16,  0,  0,  3,  0, }, // Etc/UTC Amharic/Ethiopic/Ethiopia
    {   13, 2338,       0,       0,     322,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {   13, 2501,       0,       0,     342,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {   14, 2455,       0,     361,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Arabic/Arabic/Egypt
    {   14, 2338,       0,       0,     383,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {   14, 2501,       0,       0,     403,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {   42, 2455,       0,     424,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Aragonese/Latin/Spain
    {   43, 2455,       0,     451,       0,    0,    0,    0,  0, 34,  0,  0,  3,  0, }, // Etc/UTC Armenian/Armenian/Armenia
    {   43, 2338,       0,       0,     485,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {   43, 2501,       0,       0,     512,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {   44, 2455,       0,     539,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Assamese/Bangla/India
    {   44, 2338,       0,       0,     561,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {   44, 2501,       0,       0,     575,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {   45, 2455,       0,     600,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Asturian/Latin/Spain
    {   45, 2338,       0,       0,     625,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {   45, 2501,       0,       0,     648,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {   45, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {   46, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Asu/Latin/Tanzania
    {   47, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Atsam/Latin/Nigeria
    {   48, 2455,       0,     671,       0,    0,    0,    0,  0, 36,  0,  0,  3,  0, }, // Etc/UTC Azerbaijani/Latin/Azerbaijan
    {   48, 2338,       0,       0,     707,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {   48, 2501,       0,       0,     726,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {   49, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Azerbaijani/Arabic/Iran
    {   52, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Azerbaijani/Cyrillic/Azerbaijan
    {   53, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bafia/Latin/Cameroon
    {   54, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bambara/Latin/Mali
    {   56, 2455,       0,     745,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Bangla/Bangla/Bangladesh
    {   56, 2338,       0,       0,     772,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {   56, 2501,       0,       0,     787,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {   58, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Basaa/Latin/Cameroon
    {   59, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bashkir/Cyrillic/Russia
    {   60, 2455,       0,     812,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Basque/Latin/Spain
    {   60, 2338,       0,       0,     839,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {   60, 2501,       0,       0,     864,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {   60, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {   61, 2455,       0,     885,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Belarusian/Cyrillic/Belarus
    {   61, 2338,       0,       0,     914,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   61, 2501,       0,       0,     938,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {   62, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bemba/Latin/Zambia
    {   63, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bena/Latin/Tanzania
    {   64, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bhojpuri/Devanagari/India
    {   65, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Blin/Ethiopic/Eritrea
    {   66, 2455,       0,     957,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Bodo/Devanagari/India
    {   66, 2338,       0,       0,     979,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {   66, 2501,       0,       0,     995,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {   67, 2455,       0,    1013,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Bosnian/Latin/Bosnia and Herzegovina
    {   67, 2338,       0,       0,    1042,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   67, 2501,       0,       0,    1066,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {   68, 2455,       0,    1090,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Bosnian/Cyrillic/Bosnia and Herzegovina
    {   68, 2338,       0,       0,    1122,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   68, 2501,       0,       0,    1146,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {   69, 2455,       0,    1169,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Breton/Latin/France
    {   69, 2338,       0,       0,    1195,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {   69, 2501,       0,       0,    1215,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {   70, 2455,       0,    1235,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Bulgarian/Cyrillic/Bulgaria
    {   70, 2338,       0,       0,    1265,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {   70, 2501,       0,       0,    1291,    0,    0,    0,  0,  0, 28,  0,  0,  0, }, // Europe/London
    {   71, 2455,       0,    1319,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Burmese/Myanmar/Myanmar
    {   71, 2338,       0,       0,    1348,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   71, 2501,       0,       0,    1372,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {   72, 2455,       0,    1394,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Cantonese/Traditional Han/Hong Kong
    {   72, 2338,       0,       0,    1400,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   72, 2501,       0,       0,    1407,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {   73, 2455,       0,    1413,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Cantonese/Simplified Han/China
    {   73, 2338,       0,       0,    1419,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   73, 2501,       0,       0,    1426,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {   75, 2455,       0,    1432,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Catalan/Latin/Spain
    {   75, 2338,       0,       0,    1457,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   75, 2501,       0,       0,    1481,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {   79, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Cebuano/Latin/Philippines
    {   79, 2338,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {   79, 2501,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {   80, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Central Atlas Tamazight/Latin/Morocco
    {   81, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Central Kurdish/Arabic/Iraq
    {   83, 2455,       0,    1503,       0,    0,    0,    0,  0, 46,  0,  0,  3,  0, }, // Etc/UTC Chakma/Chakma/Bangladesh
    {   83, 2338,       0,       0,    1549,    0,    0,    0,  0,  0, 36,  0,  0,  0, }, // Europe/Dublin
    {   83, 2501,       0,       0,    1585,    0,    0,    0,  0,  0, 42,  0,  0,  0, }, // Europe/London
    {   85, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Chechen/Cyrillic/Russia
    {   85, 2338,       0,       0,    1627,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {   85, 2501,       0,       0,    1648,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {   86, 2455,       0,    1670,       0,    0,    0,    0,  0, 14,  0,  0,  3,  0, }, // Etc/UTC Cherokee/Cherokee/United States
    {   86, 2338,       0,       0,    1684,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {   86, 2501,       0,       0,    1699,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/London
    {   86, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {   87, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Chickasaw/Latin/United States
    {   88, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Chiga/Latin/Uganda
    {   89, 2455,       0,    1413,       0,    0,    0,    0,  0,  5,  0,  0,  3,  0, }, // Etc/UTC Chinese/Simplified Han/China
    {   89, 2338,       0,       0,    1419,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   89, 2501,       0,       0,    1426,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {   98, 2455,       0,    1711,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Chinese/Traditional Han/Taiwan
    {   98, 2338,       0,       0,    1400,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   98, 2501,       0,       0,    1407,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {   99, 2455,       0,    1717,       0,    0,    0,    0,  0, 37,  0,  0,  3,  0, }, // Etc/UTC Church/Cyrillic/Russia
    {  100, 2455,       0,    1754,       0,    0,    9,    0,  0, 35,  0,  0, 10,  0, }, // Etc/UTC Chuvash/Cyrillic/Russia
    {  100, 2338,       0,       0,    1789,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  100, 2501,       0,       0,    1814,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  100, 2723,       0,       0,       0,   19,   19,   42,  0,  0,  0, 23, 23, 20, }, // Pacific/Honolulu
    {  101, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Colognian/Latin/Germany
    {  101, 2338,       0,       0,    1836,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  101, 2501,       0,       0,    1858,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/London
    {  102, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Cornish/Latin/United Kingdom
    {  103, 2455,       0,    1888,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Corsican/Latin/France
    {  104, 2455,       0,    1915,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Croatian/Latin/Croatia
    {  104, 2338,       0,       0,    1944,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  104, 2501,       0,       0,    1968,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  106, 2455,       0,    1992,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Czech/Latin/Czechia
    {  106, 2338,       0,       0,    2016,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  106, 2501,       0,       0,    2031,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  106, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  107, 2455,       0,    2048,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Danish/Latin/Denmark
    {  107, 2338,       0,       0,    2072,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  107, 2501,       0,       0,    2086,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  109, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Divehi/Thaana/Maldives
    {  110, 2455,       0,    2103,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Dogri/Devanagari/India
    {  110, 2338,       0,       0,    2120,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  110, 2501,       0,       0,    2135,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  111, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Duala/Latin/Cameroon
    {  112, 2455,       0,    2159,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Dutch/Latin/Netherlands
    {  112, 2338,       0,       0,    2184,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  112, 2501,       0,       0,    2203,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  112, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  119, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Dzongkha/Tibetan/Bhutan
    {  119, 2338,       0,       0,    2219,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  119, 2501,       0,       0,    2242,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  120, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Embu/Latin/Kenya
    {  121, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC English/Latin/United States
    {  121, 2338,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  121, 2501,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  121, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  122, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC English/Deseret/United States
    {  124, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Anguilla
    {  125, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Antigua and Barbuda
    {  126, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Australia
    {  127, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Austria
    {  128, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Bahamas
    {  129, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Barbados
    {  130, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Belgium
    {  131, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Belize
    {  132, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Bermuda
    {  133, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Botswana
    {  134, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/British Indian Ocean Territory
    {  135, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/British Virgin Islands
    {  137, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cameroon
    {  139, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cayman Islands
    {  140, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Christmas Island
    {  141, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cocos Islands
    {  142, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cook Islands
    {  143, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cyprus
    {  144, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Denmark
    {  145, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Diego Garcia
    {  146, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Dominica
    {  147, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Eritrea
    {  148, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Eswatini
    {  149, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Europe
    {  150, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Falkland Islands
    {  151, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Fiji
    {  152, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Finland
    {  153, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Gambia
    {  154, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Germany
    {  155, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Ghana
    {  156, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Gibraltar
    {  157, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Grenada
    {  159, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Guernsey
    {  160, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Guyana
    {  161, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Hong Kong
    {  162, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/India
    {  163, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Indonesia
    {  164, 2338,       0,       0,      26,    0,    0,   65,  0,  0, 19,  0,  0,  3, }, // Europe/Dublin English/Latin/Ireland
    {  164, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  165, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Isle of Man
    {  166, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Israel
    {  167, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Jamaica
    {  168, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Jersey
    {  169, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Kenya
    {  170, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Kiribati
    {  171, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Lesotho
    {  172, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Liberia
    {  173, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Macao
    {  174, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Madagascar
    {  175, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Malawi
    {  176, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Malaysia
    {  177, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Maldives
    {  178, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Malta
    {  179, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Marshall Islands
    {  180, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Mauritius
    {  181, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Micronesia
    {  182, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Montserrat
    {  183, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Namibia
    {  184, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Nauru
    {  185, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Netherlands
    {  186, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/New Zealand
    {  187, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Nigeria
    {  188, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Niue
    {  189, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Norfolk Island
    {  190, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Northern Mariana Islands
    {  191, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Pakistan
    {  192, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Palau
    {  193, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Papua New Guinea
    {  195, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Pitcairn
    {  197, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Rwanda
    {  198, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Helena
    {  199, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Kitts and Nevis
    {  200, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Lucia
    {  201, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Vincent and Grenadines
    {  202, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Samoa
    {  203, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Seychelles
    {  204, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sierra Leone
    {  205, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Singapore
    {  206, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sint Maarten
    {  207, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Slovenia
    {  208, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Solomon Islands
    {  209, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/South Africa
    {  210, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/South Sudan
    {  211, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sudan
    {  212, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sweden
    {  213, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Switzerland
    {  214, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tanzania
    {  215, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tokelau
    {  216, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tonga
    {  217, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Trinidad and Tobago
    {  218, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Turks and Caicos Islands
    {  219, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tuvalu
    {  220, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Uganda
    {  221, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/United Arab Emirates
    {  222, 2501,       0,       0,      45,    0,    0,   68,  0,  0, 19,  0,  0,  3, }, // Europe/London English/Latin/United Kingdom
    {  222, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  225, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Vanuatu
    {  226, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/world
    {  227, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Zambia
    {  228, 2723,       0,       0,       0,   62,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Zimbabwe
    {  229, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC English/Shavian/United Kingdom
    {  230, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Erzya/Cyrillic/Russia
    {  231, 2455,       0,    2266,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Esperanto/Latin/world
    {  231, 2338,       0,       0,    2294,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  231, 2501,       0,       0,    2313,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {  232, 2455,       0,    2331,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Estonian/Latin/Estonia
    {  232, 2338,       0,       0,    2356,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/Dublin
    {  232, 2501,       0,       0,    2368,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  233, 2455,       0,    2381,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Ewe/Latin/Ghana
    {  233, 2338,       0,       0,    2403,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  233, 2501,       0,       0,    2427,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  235, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ewondo/Latin/Cameroon
    {  236, 2455,       0,    2454,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Faroese/Latin/Faroe Islands
    {  236, 2338,       0,       0,    2472,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  236, 2501,       0,       0,    2487,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  238, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Filipino/Latin/Philippines
    {  238, 2338,       0,       0,    2503,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {  238, 2501,       0,       0,    2530,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  239, 2455,       0,    2557,       0,    0,    0,    0,  0, 13,  0,  0,  3,  0, }, // Etc/UTC Finnish/Latin/Finland
    {  239, 2338,       0,       0,    2570,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  239, 2501,       0,       0,    2587,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  240, 2455,       0,    2606,       0,    0,   71,    0,  0, 25,  0,  0,  2,  0, }, // Etc/UTC French/Latin/France
    {  240, 2338,       0,       0,    2631,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  240, 2501,       0,       0,    2653,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  240, 2723,       0,       0,       0,   73,    3,    6,  0,  0,  0,  2,  3,  3, }, // Pacific/Honolulu
    {  247, 2338,       0,       0,    2676,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin French/Latin/Canada
    {  247, 2501,       0,       0,    2700,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  286, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Friulian/Latin/Italy
    {  287, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Fulah/Latin/Senegal
    {  293, 2455,       0,    2725,       0,    0,   75,    0,  0, 56,  0,  0,  6,  0, }, // Etc/UTC Fulah/Adlam/Guinea
    {  293, 2338,       0,       0,    2781,    0,    0,    0,  0,  0, 60,  0,  0,  0, }, // Europe/Dublin
    {  293, 2501,       0,       0,    2841,    0,    0,    0,  0,  0, 58,  0,  0,  0, }, // Europe/London
    {  293, 2723,       0,       0,       0,   81,   81,   87,  0,  0,  0,  6,  6,  6, }, // Pacific/Honolulu
    {  311, 2455,       0,    2899,       0,    0,    0,    0,  0, 34,  0,  0,  3,  0, }, // Etc/UTC Gaelic/Latin/United Kingdom
    {  311, 2338,       0,       0,    2933,    0,    0,   93,  0,  0, 19,  0,  0, 12, }, // Europe/Dublin
    {  311, 2501,       0,       0,    2952,    0,    0,  105,  0,  0, 25,  0,  0,  3, }, // Europe/London
    {  311, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  312, 2455,       0,    2977,       0,    0,    0,    0,  0, 36,  0,  0,  3,  0, }, // Etc/UTC Ga/Latin/Ghana
    {  312, 2338,       0,       0,    3013,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {  312, 2501,       0,       0,    3040,    0,    0,    0,  0,  0, 42,  0,  0,  0, }, // Europe/London
    {  313, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Galician/Latin/Spain
    {  313, 2338,       0,       0,    3107,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  313, 2501,       0,       0,    3130,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  314, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ganda/Latin/Uganda
    {  315, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Geez/Ethiopic/Ethiopia
    {  317, 2455,       0,    3153,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Georgian/Georgian/Georgia
    {  317, 2338,       0,       0,    3179,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  317, 2501,       0,       0,    3204,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  318, 2455,       0,    3227,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC German/Latin/Germany
    {  318, 2338,       0,       0,    3248,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  318, 2501,       0,       0,    3266,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  325, 2455,       0,    3286,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Greek/Greek/Greece
    {  325, 2338,       0,       0,    3312,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  325, 2501,       0,       0,    3335,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  327, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Guarani/Latin/Paraguay
    {  328, 2455,       0,    3355,       0,    0,    0,    0,  0, 20,  0,  0,  3,  0, }, // Etc/UTC Gujarati/Gujarati/India
    {  328, 2338,       0,       0,    3375,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  328, 2501,       0,       0,    3389,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  329, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Gusii/Latin/Kenya
    {  330, 2455,       0,    3408,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Hausa/Latin/Nigeria
    {  330, 2338,       0,       0,    3431,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  330, 2501,       0,       0,    3454,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  331, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Hausa/Arabic/Nigeria
    {  332, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Hausa/Arabic/Sudan
    {  335, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Hawaiian/Latin/United States
    {  335, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  336, 2455,       0,    3481,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Hebrew/Hebrew/Israel
    {  336, 2338,       0,       0,    3500,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  336, 2501,       0,       0,    3515,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  337, 2455,       0,    3531,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Hindi/Devanagari/India
    {  337, 2338,       0,       0,    3550,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  337, 2501,       0,       0,    3564,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  337, 2723,       0,       0,       0,  108,  108,    3,  0,  0,  0,  6,  6,  3, }, // Pacific/Honolulu
    {  338, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Hindi/Latin/India
    {  338, 2338,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  338, 2501,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  338, 2723,       0,       0,       0,    3,   62,   62,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  339, 2455,       0,    3588,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Hungarian/Latin/Hungary
    {  339, 2338,       0,       0,    3607,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/Dublin
    {  339, 2501,       0,       0,    3619,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/London
    {  340, 2455,       0,    3633,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Icelandic/Latin/Iceland
    {  340, 2338,       0,       0,    3659,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  340, 2501,       0,       0,    3678,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  341, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ido/Latin/world
    {  342, 2455,       0,    3699,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Igbo/Latin/Nigeria
    {  342, 2338,       0,       0,    3718,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  342, 2501,       0,       0,    3738,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  343, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Inari Sami/Latin/Finland
    {  344, 2455,       0,    3758,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Indonesian/Latin/Indonesia
    {  344, 2338,       0,       0,    3787,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  344, 2501,       0,       0,    3809,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  345, 2455,       0,    3834,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Interlingua/Latin/world
    {  345, 2338,       0,       0,    3862,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  345, 2501,       0,       0,    3883,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  346, 2455,       0,    3904,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Interlingue/Latin/Estonia
    {  346, 2338,       0,       0,    3930,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  346, 2501,       0,       0,    3955,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  347, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Inuktitut/Canadian Aboriginal/Canada
    {  349, 2455,       0,    3978,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Irish/Latin/Ireland
    {  349, 2338,       0,       0,    3996,    0,    0,  114,  0,  0, 27,  0,  0,  3, }, // Europe/Dublin
    {  349, 2501,       0,       0,    4023,    0,    0,  117,  0,  0, 25,  0,  0,  3, }, // Europe/London
    {  351, 2455,       0,    4048,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Italian/Latin/Italy
    {  351, 2338,       0,       0,    4075,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  351, 2501,       0,       0,    4098,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  355, 2455,       0,    4124,       0,    0,    0,    0,  0,  5,  0,  0,  3,  0, }, // Etc/UTC Japanese/Japanese/Japan
    {  355, 2338,       0,       0,    4129,    0,    0,    0,  0,  0,  9,  0,  0,  0, }, // Europe/Dublin
    {  355, 2501,       0,       0,    4138,    0,    0,    0,  0,  0,  5,  0,  0,  0, }, // Europe/London
    {  356, 2455,       0,    4143,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Javanese/Latin/Indonesia
    {  356, 2338,       0,       0,    4172,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  356, 2501,       0,       0,    4194,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  357, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Jju/Latin/Nigeria
    {  358, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Jola-Fonyi/Latin/Senegal
    {  359, 2455,       0,    4214,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kabuverdianu/Latin/Cape Verde
    {  359, 2338,       0,       0,    4237,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  359, 2501,       0,       0,    4258,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  360, 2455,       0,    4280,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kabyle/Latin/Algeria
    {  360, 2338,       0,       0,    4303,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  360, 2501,       0,       0,    4324,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  361, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kako/Latin/Cameroon
    {  362, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kalaallisut/Latin/Greenland
    {  363, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kalenjin/Latin/Kenya
    {  364, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kamba/Latin/Kenya
    {  365, 2455,       0,    4348,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Kannada/Kannada/India
    {  365, 2338,       0,       0,    4369,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  365, 2501,       0,       0,    4387,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  366, 2455,       0,    4406,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kashmiri/Arabic/India
    {  366, 2338,       0,       0,    4429,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  366, 2501,       0,       0,    4447,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  367, 2455,       0,    4466,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kashmiri/Devanagari/India
    {  368, 2455,       0,    4489,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Kazakh/Cyrillic/Kazakhstan
    {  368, 2338,       0,       0,    4521,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  368, 2501,       0,       0,    4547,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  369, 2455,       0,    4571,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Kazakh/Arabic/China
    {  369, 2338,       0,       0,    4603,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  369, 2501,       0,       0,    4629,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  370, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kenyang/Latin/Cameroon
    {  371, 2455,       0,    4653,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Khmer/Khmer/Cambodia
    {  371, 2338,       0,       0,    4675,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  371, 2501,       0,       0,    4700,    0,    0,    0,  0,  0, 34,  0,  0,  0, }, // Europe/London
    {  372, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kiche/Latin/Guatemala
    {  373, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kikuyu/Latin/Kenya
    {  374, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kinyarwanda/Latin/Rwanda
    {  375, 2455,       0,    4734,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Konkani/Devanagari/India
    {  375, 2338,       0,       0,    4753,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  375, 2501,       0,       0,    4771,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  376, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Konkani/Latin/India
    {  377, 2455,       0,    4790,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Korean/Korean/South Korea
    {  377, 2338,       0,       0,    4796,    0,    0,    0,  0,  0,  8,  0,  0,  0, }, // Europe/Dublin
    {  377, 2501,       0,       0,    4804,    0,    0,    0,  0,  0,  9,  0,  0,  0, }, // Europe/London
    {  380, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Koyraboro Senni/Latin/Mali
    {  381, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Koyra Chiini/Latin/Mali
    {  382, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kpelle/Latin/Liberia
    {  384, 2455,       0,    4813,       0,    0,  120,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Kurdish/Latin/Turkey
    {  384, 2338,       0,       0,    4837,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  384, 2501,       0,       0,    4863,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  385, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kwasio/Latin/Cameroon
    {  386, 2455,       0,    4889,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Kyrgyz/Cyrillic/Kyrgyzstan
    {  386, 2338,       0,       0,    4914,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  386, 2501,       0,       0,    4936,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  387, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lakota/Latin/United States
    {  388, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Langi/Latin/Tanzania
    {  389, 2455,       0,    4959,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Lao/Lao/Laos
    {  389, 2338,       0,       0,    4977,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  389, 2501,       0,       0,    5001,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  390, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Latin/Latin/Vatican City
    {  391, 2455,       0,    5025,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Latvian/Latin/Latvia
    {  391, 2338,       0,       0,    5056,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  391, 2501,       0,       0,    5075,    0,    0,    0,  0,  0, 28,  0,  0,  0, }, // Europe/London
    {  392, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lingala/Latin/Congo - Kinshasa
    {  396, 2455,       0,    5103,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Lithuanian/Latin/Lithuania
    {  396, 2338,       0,       0,    5131,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  396, 2501,       0,       0,    5153,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  397, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lojban/Latin/world
    {  398, 2455,       0,    5178,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Lower Sorbian/Latin/Germany
    {  398, 2338,       0,       0,    5204,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  398, 2501,       0,       0,    5223,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  399, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Low German/Latin/Germany
    {  401, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luba-Katanga/Latin/Congo - Kinshasa
    {  402, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lule Sami/Latin/Sweden
    {  404, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luo/Latin/Kenya
    {  405, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luxembourgish/Latin/Luxembourg
    {  405, 2338,       0,       0,    5244,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  405, 2501,       0,       0,    5261,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  406, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luyia/Latin/Kenya
    {  407, 2455,       0,    5280,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Macedonian/Cyrillic/Macedonia
    {  407, 2338,       0,       0,    5310,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  407, 2501,       0,       0,    5332,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  408, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Machame/Latin/Tanzania
    {  409, 2455,       0,    3531,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Maithili/Devanagari/India
    {  410, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Makhuwa-Meetto/Latin/Mozambique
    {  411, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Makonde/Latin/Tanzania
    {  412, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Malagasy/Latin/Madagascar
    {  413, 2455,       0,    5353,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Malayalam/Malayalam/India
    {  413, 2338,       0,       0,    5382,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  413, 2501,       0,       0,    5405,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  414, 2455,       0,    5432,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Malay/Latin/Malaysia
    {  414, 2338,       0,       0,    5455,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  414, 2501,       0,       0,    5475,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  415, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Malay/Arabic/Brunei
    {  416, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Malay/Arabic/Malaysia
    {  420, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Maltese/Latin/Malta
    {  421, 2455,       0,    5500,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Manipuri/Bangla/India
    {  422, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Manipuri/Meitei Mayek/India
    {  423, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Manx/Latin/Isle of Man
    {  424, 2455,       0,    5529,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Maori/Latin/New Zealand
    {  424, 2338,       0,       0,    5546,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  424, 2501,       0,       0,    5565,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  425, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mapuche/Latin/Chile
    {  426, 2455,       0,    4734,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Marathi/Devanagari/India
    {  426, 2338,       0,       0,    5584,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  426, 2501,       0,       0,    5600,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  427, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Masai/Latin/Kenya
    {  429, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mazanderani/Arabic/Iran
    {  430, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Meru/Latin/Kenya
    {  431, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Meta/Latin/Cameroon
    {  432, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mohawk/Latin/Canada
    {  433, 2455,       0,    5619,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Mongolian/Cyrillic/Mongolia
    {  433, 2338,       0,       0,    5648,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  433, 2501,       0,       0,    5669,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {  434, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mongolian/Mongolian/China
    {  435, 2455,       0,    5687,       0,    0,    0,    0,  0, 33,  0,  0,  3,  0, }, // Etc/UTC Mongolian/Mongolian/Mongolia
    {  436, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Morisyen/Latin/Mauritius
    {  437, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mundang/Latin/Cameroon
    {  438, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Muscogee/Latin/United States
    {  439, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nama/Latin/Namibia
    {  440, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Navajo/Latin/United States
    {  441, 2455,       0,    5720,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Nepali/Devanagari/Nepal
    {  441, 2338,       0,       0,    3550,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  441, 2501,       0,       0,    5737,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  443, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ngiemboon/Latin/Cameroon
    {  444, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ngomba/Latin/Cameroon
    {  445, 2455,       0,    5761,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Nigerian Pidgin/Latin/Nigeria
    {  445, 2338,       0,       0,    5790,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  445, 2501,       0,       0,    5807,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  446, 2455,       0,    5830,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Nko/Nko/Guinea
    {  446, 2338,       0,       0,    5858,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  446, 2501,       0,       0,    5884,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  447, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Northern Luri/Arabic/Iran
    {  449, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Northern Sami/Latin/Norway
    {  450, 2455,       0,    5905,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Northern Sami/Latin/Finland
    {  450, 2338,       0,       0,    5934,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  450, 2501,       0,       0,    5955,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  452, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Northern Sotho/Latin/South Africa
    {  453, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC North Ndebele/Latin/Zimbabwe
    {  454, 2455,       0,    5975,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Norwegian Bokmal/Latin/Norway
    {  454, 2338,       0,       0,    5998,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  454, 2501,       0,       0,    6012,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  454, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  456, 2455,       0,    5975,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Norwegian Nynorsk/Latin/Norway
    {  456, 2338,       0,       0,    6029,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  456, 2501,       0,       0,    6043,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  456, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  457, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nuer/Latin/South Sudan
    {  458, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nyanja/Latin/Malawi
    {  459, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nyankole/Latin/Uganda
    {  460, 2455,       0,    6060,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Occitan/Latin/France
    {  460, 2338,       0,       0,    6085,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  460, 2501,       0,       0,    6109,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  461, 2455,       0,    6130,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Occitan/Latin/Spain
    {  461, 2338,       0,       0,    6154,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  461, 2501,       0,       0,    6109,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  462, 2455,       0,    6175,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Odia/Odia/India
    {  462, 2338,       0,       0,    6196,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  462, 2501,       0,       0,    6212,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  463, 2455,       0,    6238,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Oromo/Latin/Ethiopia
    {  463, 2338,       0,       0,    6269,    0,    0,    0,  0,  0, 32,  0,  0,  0, }, // Europe/Dublin
    {  463, 2501,       0,       0,    6301,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  465, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Osage/Osage/United States
    {  467, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ossetic/Cyrillic/Russia
    {  468, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Papiamento/Latin/Curacao
    {  470, 2455,       0,    6324,       0,    0,    0,    0,  0, 16,  0,  0,  3,  0, }, // Etc/UTC Pashto/Arabic/Afghanistan
    {  470, 2338,       0,       0,    6340,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  470, 2501,       0,       0,    6355,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  472, 2455,       0,    6374,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Persian/Arabic/Iran
    {  472, 2338,       0,       0,    6391,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  472, 2501,       0,       0,    6406,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  474, 2455,       0,    6427,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Polish/Latin/Poland
    {  474, 2338,       0,       0,    6456,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  474, 2501,       0,       0,    6477,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  475, 2455,       0,    6497,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Brazil
    {  475, 2338,       0,       0,    6525,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  475, 2501,       0,       0,    6548,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  476, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Angola
    {  476, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  476, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  477, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Cape Verde
    {  477, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  477, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  478, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Equatorial Guinea
    {  478, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  478, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  479, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Guinea-Bissau
    {  479, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  479, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  480, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Luxembourg
    {  480, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  480, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  481, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Macao
    {  481, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  481, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  482, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Mozambique
    {  482, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  482, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  483, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Portugal
    {  483, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  483, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  484, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Sao Tome and Principe
    {  484, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  484, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  485, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Switzerland
    {  485, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  485, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  486, 2455,       0,    6574,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Timor-Leste
    {  486, 2338,       0,       0,    6599,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  486, 2501,       0,       0,    6623,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  487, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Prussian/Latin/Poland
    {  488, 2455,       0,    6646,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Punjabi/Gurmukhi/India
    {  488, 2338,       0,       0,    6668,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  488, 2501,       0,       0,    6685,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  489, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Punjabi/Arabic/Pakistan
    {  490, 2455,       0,    6708,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Quechua/Latin/Peru
    {  490, 2338,       0,       0,    6737,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  490, 2501,       0,       0,    6761,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  493, 2455,       0,    6785,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Romanian/Latin/Romania
    {  493, 2338,       0,       0,    6811,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  493, 2501,       0,       0,    6833,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  495, 2455,       0,    6854,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Romansh/Latin/Switzerland
    {  495, 2501,       0,       0,    6877,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  496, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rombo/Latin/Tanzania
    {  497, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rundi/Latin/Burundi
    {  498, 2455,       0,    6899,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Russian/Cyrillic/Russia
    {  498, 2338,       0,       0,    6931,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {  498, 2501,       0,       0,    6958,    0,    0,    0,  0,  0, 28,  0,  0,  0, }, // Europe/London
    {  504, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rwa/Latin/Tanzania
    {  505, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Saho/Latin/Eritrea
    {  506, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sakha/Cyrillic/Russia
    {  507, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Samburu/Latin/Kenya
    {  508, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sango/Latin/Central African Republic
    {  509, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sangu/Latin/Tanzania
    {  510, 2455,       0,    6986,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Sanskrit/Devanagari/India
    {  511, 2455,       0,    7007,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Santali/Ol Chiki/India
    {  512, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Santali/Devanagari/India
    {  513, 2455,       0,    7033,       0,    0,  123,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Sardinian/Latin/Italy
    {  513, 2338,       0,       0,    7061,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  513, 2501,       0,       0,    7084,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  513, 2723,       0,       0,       0,  126,  126,  129,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  514, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sena/Latin/Mozambique
    {  515, 2455,       0,    7106,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Cyrillic/Serbia
    {  515, 2338,       0,       0,    7136,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  515, 2501,       0,       0,    7159,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  516, 2455,       0,    1090,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Serbian/Cyrillic/Bosnia and Herzegovina
    {  516, 2338,       0,       0,    7181,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  516, 2501,       0,       0,    7206,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  519, 2455,       0,    7230,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Bosnia and Herzegovina
    {  519, 2338,       0,       0,    7262,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  519, 2501,       0,       0,    7287,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  520, 2455,       0,    7313,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Kosovo
    {  520, 2338,       0,       0,    7343,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  520, 2501,       0,       0,    7366,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  521, 2455,       0,    7313,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Montenegro
    {  521, 2338,       0,       0,    7343,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  521, 2501,       0,       0,    7366,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  522, 2455,       0,    7313,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Serbia
    {  522, 2338,       0,       0,    7343,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  522, 2501,       0,       0,    7366,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  523, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Shambala/Latin/Tanzania
    {  524, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Shona/Latin/Zimbabwe
    {  525, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sichuan Yi/Yi/China
    {  526, 2455,       0,    7389,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Sicilian/Latin/Italy
    {  526, 2338,       0,       0,    7413,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  526, 2501,       0,       0,    7433,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  527, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sidamo/Latin/Ethiopia
    {  528, 2455,       0,    7454,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Silesian/Latin/Poland
    {  528, 2338,       0,       0,    7483,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  528, 2501,       0,       0,    7506,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  529, 2455,       0,    7527,       0,    0,  132,    0,  0, 15,  0,  0,  5,  0, }, // Etc/UTC Sindhi/Arabic/Pakistan
    {  529, 2338,       0,       0,    7542,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  529, 2501,       0,       0,    7560,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  530, 2455,       0,    7585,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Sindhi/Devanagari/India
    {  531, 2455,       0,    7602,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Sinhala/Sinhala/Sri Lanka
    {  531, 2338,       0,       0,    7620,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  531, 2501,       0,       0,    7640,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  532, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Skolt Sami/Latin/Finland
    {  533, 2455,       0,    7665,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Slovak/Latin/Slovakia
    {  533, 2338,       0,       0,    7689,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  533, 2501,       0,       0,    7709,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  533, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  534, 2455,       0,    7726,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Slovenian/Latin/Slovenia
    {  534, 2338,       0,       0,    7754,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  534, 2501,       0,       0,    7774,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  535, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Soga/Latin/Uganda
    {  536, 2455,       0,    7795,       0,    0,  137,    0,  0, 30,  0,  0, 11,  0, }, // Etc/UTC Somali/Latin/Somalia
    {  536, 2338,       0,       0,    7825,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/Dublin
    {  536, 2501,       0,       0,    7855,    0,    0,    0,  0,  0, 29,  0,  0,  0, }, // Europe/London
    {  540, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Southern Kurdish/Arabic/Iran
    {  542, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Southern Sami/Latin/Sweden
    {  544, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Southern Sotho/Latin/South Africa
    {  546, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC South Ndebele/Latin/South Africa
    {  547, 2455,       0,    7884,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Spain
    {  547, 2338,       0,       0,    7911,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  547, 2501,       0,       0,    7936,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  548, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Argentina
    {  548, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  549, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Belize
    {  549, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  550, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Bolivia
    {  550, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  551, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Brazil
    {  551, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  554, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Chile
    {  554, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  555, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Colombia
    {  555, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  556, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Costa Rica
    {  556, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  557, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Cuba
    {  557, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  558, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Dominican Republic
    {  558, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  559, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Ecuador
    {  559, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  560, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/El Salvador
    {  560, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  562, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Guatemala
    {  562, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  563, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Honduras
    {  563, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  564, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Latin America
    {  564, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  565, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Mexico
    {  565, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  566, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Nicaragua
    {  566, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  567, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Panama
    {  567, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  568, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Paraguay
    {  568, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  569, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Peru
    {  569, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  571, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Puerto Rico
    {  571, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  572, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/United States
    {  572, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  572, 2723,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  573, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Uruguay
    {  573, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  574, 2455,       0,    3082,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Venezuela
    {  574, 2338,       0,       0,    7960,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  575, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Standard Moroccan Tamazight/Tifinagh/Morocco
    {  576, 2455,       0,    3758,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Sundanese/Latin/Indonesia
    {  577, 2455,       0,    7984,       0,    0,    0,    0,  0, 33,  0,  0,  3,  0, }, // Etc/UTC Swahili/Latin/Tanzania
    {  577, 2338,       0,       0,    8017,    0,    0,    0,  0,  0, 33,  0,  0,  0, }, // Europe/Dublin
    {  577, 2501,       0,       0,    8050,    0,    0,    0,  0,  0, 34,  0,  0,  0, }, // Europe/London
    {  579, 2455,       0,    8084,       0,    0,    0,    0,  0, 12,  0,  0,  3,  0, }, // Etc/UTC Swahili/Latin/Kenya
    {  581, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Swati/Latin/South Africa
    {  583, 2455,       0,    8096,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Swedish/Latin/Sweden
    {  583, 2338,       0,       0,    8122,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  583, 2501,       0,       0,    8140,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {  583, 2723,       0,       0,       0,  148,  159,  176,  0,  0,  0, 11, 17, 17, }, // Pacific/Honolulu
    {  586, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Swiss German/Latin/Switzerland
    {  589, 2455,       0,    8158,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Syriac/Syriac/Iraq
    {  589, 2338,       0,       0,    8179,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  589, 2501,       0,       0,    8201,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  591, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tachelhit/Tifinagh/Morocco
    {  593, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tai Dam/Tai Viet/Vietnam
    {  594, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Taita/Latin/Kenya
    {  595, 2455,       0,    8224,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Tajik/Cyrillic/Tajikistan
    {  595, 2338,       0,       0,    8250,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  595, 2501,       0,       0,    8275,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  596, 2455,       0,    8300,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Tamil/Tamil/India
    {  596, 2338,       0,       0,    8326,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  596, 2501,       0,       0,    8345,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  600, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Taroko/Latin/Taiwan
    {  601, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tasawaq/Latin/Niger
    {  602, 2455,       0,    8366,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Tatar/Cyrillic/Russia
    {  602, 2338,       0,       0,    8397,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  602, 2501,       0,       0,    8421,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  603, 2455,       0,    8442,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Telugu/Telugu/India
    {  603, 2338,       0,       0,    8463,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  603, 2501,       0,       0,    8483,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  604, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Teso/Latin/Uganda
    {  606, 2455,       0,    8502,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Thai/Thai/Thailand
    {  606, 2338,       0,       0,    8519,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  606, 2501,       0,       0,    8539,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  607, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tibetan/Tibetan/China
    {  609, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tigre/Ethiopic/Eritrea
    {  610, 2455,       0,    8556,       0,    0,    0,    0,  0, 14,  0,  0,  3,  0, }, // Etc/UTC Tigrinya/Ethiopic/Ethiopia
    {  610, 2338,       0,       0,    8570,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  610, 2501,       0,       0,    8585,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  612, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tok Pisin/Latin/Papua New Guinea
    {  613, 2455,       0,    8598,       0,    0,  193,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Tongan/Latin/Tonga
    {  613, 2338,       0,       0,    8615,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/Dublin
    {  613, 2501,       0,       0,    8645,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/London
    {  613, 2723,       0,       0,       0,  196,  196,  199,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  614, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tsonga/Latin/South Africa
    {  615, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tswana/Latin/South Africa
    {  617, 2455,       0,    8675,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Turkish/Latin/Turkey
    {  617, 2338,       0,       0,    8700,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  617, 2501,       0,       0,    8722,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  619, 2455,       0,    8741,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Turkmen/Latin/Turkmenistan
    {  619, 2338,       0,       0,    8769,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  619, 2501,       0,       0,    8793,    0,    0,    0,  0,  0, 29,  0,  0,  0, }, // Europe/London
    {  620, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tyap/Latin/Nigeria
    {  621, 2455,       0,    8822,       0,    0,    0,    0,  0, 33,  0,  0,  3,  0, }, // Etc/UTC Ukrainian/Cyrillic/Ukraine
    {  621, 2338,       0,       0,    8855,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  621, 2501,       0,       0,    8881,    0,    0,    0,  0,  0, 34,  0,  0,  0, }, // Europe/London
    {  622, 2455,       0,    8915,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Upper Sorbian/Latin/Germany
    {  622, 2338,       0,       0,    8939,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  622, 2501,       0,       0,    8954,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {  623, 2455,       0,    8972,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Urdu/Arabic/Pakistan
    {  623, 2338,       0,       0,    8996,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  623, 2501,       0,       0,    9014,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  625, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Uyghur/Arabic/China
    {  625, 2338,       0,       0,    9027,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  625, 2501,       0,       0,    9048,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  626, 2455,       0,    9069,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Uzbek/Latin/Uzbekistan
    {  626, 2338,       0,       0,    9096,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  626, 2501,       0,       0,    9117,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  627, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Uzbek/Arabic/Afghanistan
    {  628, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Uzbek/Cyrillic/Uzbekistan
    {  628, 2338,       0,       0,    9138,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  628, 2501,       0,       0,    9157,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  629, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Vai/Vai/Liberia
    {  631, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Venda/Latin/South Africa
    {  632, 2455,       0,    9176,       0,    0,    0,    0,  0, 20,  0,  0,  3,  0, }, // Etc/UTC Vietnamese/Latin/Vietnam
    {  632, 2338,       0,       0,    9196,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  632, 2501,       0,       0,    9212,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/London
    {  632, 2723,       0,       0,       0,  202,    3,    6,  0,  0,  0,  7,  3,  3, }, // Pacific/Honolulu
    {  633, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Volapuk/Latin/world
    {  634, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Vunjo/Latin/Tanzania
    {  635, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Walloon/Latin/Belgium
    {  636, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Walser/Latin/Switzerland
    {  637, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Warlpiri/Latin/Australia
    {  638, 2455,       0,    9226,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Welsh/Latin/United Kingdom
    {  638, 2338,       0,       0,    9252,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  638, 2501,       0,       0,    9274,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  639, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Western Balochi/Arabic/Pakistan
    {  644, 2455,       0,    9291,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Western Frisian/Latin/Netherlands
    {  644, 2338,       0,       0,    9313,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  644, 2501,       0,       0,    9329,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  645, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Wolaytta/Ethiopic/Ethiopia
    {  646, 2455,       0,    9346,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Wolof/Latin/Senegal
    {  646, 2338,       0,       0,    9377,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/Dublin
    {  646, 2501,       0,       0,    9390,    0,    0,    0,  0,  0, 28,  0,  0,  0, }, // Europe/London
    {  647, 2455,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Xhosa/Latin/South Africa
    {  647, 2338,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  647, 2501,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  648, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Yangben/Latin/Cameroon
    {  649, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Yiddish/Hebrew/Ukraine
    {  650, 2455,       0,    9418,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Yoruba/Latin/Nigeria
    {  650, 2338,       0,       0,    9437,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  650, 2501,       0,       0,    9458,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  651, 2455,       0,    9474,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Yoruba/Latin/Benin
    {  651, 2338,       0,       0,    9493,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  651, 2501,       0,       0,    9514,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  652, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Zarma/Latin/Niger
    {  653, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Zhuang/Latin/China
    {  654, 2455,       0,    9530,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Zulu/Latin/South Africa
    {  654, 2338,       0,       0,    9561,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/Dublin
    {  654, 2501,       0,       0,    9591,    0,    0,    0,  0,  0, 32,  0,  0,  0, }, // Europe/London
    {  655, 2455,       0,    9623,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kaingang/Latin/Brazil
    {  655, 2338,       0,       0,    9646,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/Dublin
    {  655, 2501,       0,       0,    9658,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  656, 2455,       0,    9677,       0,    0,    0,    0,  0, 34,  0,  0,  3,  0, }, // Etc/UTC Nheengatu/Latin/Brazil
    {  656, 2338,       0,       0,    9711,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  656, 2501,       0,       0,    9733,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/London
    {  659, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Haryanvi/Devanagari/India
    {  660, 2455,       0,    9763,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Northern Frisian/Latin/Germany
    {  660, 2338,       0,       0,    9781,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  660, 2501,       0,       0,    9800,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  661, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rajasthani/Devanagari/India
    {  662, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Moksha/Cyrillic/Russia
    {  663, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Toki Pona/Latin/world
    {  664, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Pijin/Latin/Solomon Islands
    {  665, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Obolo/Latin/Nigeria
    {  666, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Baluchi/Arabic/Pakistan
    {  667, 2455,       0,    9817,       0,    0,    0,    0,  0, 20,  0,  0,  3,  0, }, // Etc/UTC Baluchi/Latin/Pakistan
    {  667, 2338,       0,       0,    9837,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  667, 2501,       0,       0,    9853,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  668, 2455,       0,    9875,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Ligurian/Latin/Italy
    {  668, 2338,       0,       0,    9901,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  668, 2501,       0,       0,    9920,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  669, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rohingya/Hanifi/Myanmar
    {  671, 2455,       0,    8972,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Torwali/Arabic/Pakistan
    {  671, 2338,       0,       0,    8996,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  671, 2501,       0,       0,    9014,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  672, 2455,       0,    9940,       0,    0,  209,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Anii/Latin/Benin
    {  672, 2338,       0,       0,    9966,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  672, 2501,       0,       0,    9984,    0,    0,    0,  0,  0, 33,  0,  0,  0, }, // Europe/London
    {  673, 2455,       0,   10017,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Kangri/Devanagari/India
    {  673, 2338,       0,       0,   10038,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  673, 2501,       0,       0,   10055,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  674, 2455,       0,   10080,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Venetian/Latin/Italy
    {  674, 2338,       0,       0,   10105,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  674, 2501,       0,       0,   10125,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  675, 2455,       0,   10145,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Latin/India
    {  675, 2338,       0,       0,   10166,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  675, 2501,       0,       0,   10184,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  676, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Devanagari/India
    {  677, 2455,       0,   10205,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Odia/India
    {  677, 2338,       0,       0,   10227,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  677, 2501,       0,       0,   10246,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  678, 2455,       0,    8442,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Telugu/India
    {  678, 2338,       0,       0,    8463,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  678, 2501,       0,       0,    8483,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  679, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kara-Kalpak/Cyrillic/Uzbekistan
    {  681, 2455,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Swampy Cree/Canadian Aboriginal/Canada
}; // Zone naming table

// Sorted by locale index, then meta key
static inline constexpr LocaleMetaZoneLongNames localeMetaZoneLongNameTable[] = {
    // locInd, metaKey, (generic, standard, DST){ind, sz}
    {    0,    0,       0,       9,      27,  9, 18, 16, }, // Acre C/AnyScript/AnyTerritory
    {    0,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan
    {    0,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {    0,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {    0,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {    0,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {    0,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {    0,  128,     235,     246,     266, 11, 20, 18, }, // Almaty
    {    0,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {    0,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {    0,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {    0,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {    0,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {    0,  207,     552,     563,     583, 11, 20, 18, }, // Anadyr
    {    0,  214,     601,     610,     628,  9, 18, 18, }, // Apia
    {    0,  219,     646,     656,     675, 10, 19, 17, }, // Aqtau
    {    0,  225,     692,     703,     723, 11, 20, 18, }, // Aqtobe
    {    0,  232,     741,     753,     774, 12, 21, 21, }, // Arabian
    {    0,  240,     795,     809,     832, 14, 23, 21, }, // Argentina
    {    0,  250,     853,     875,     906, 22, 31, 29, }, // Argentina_Western
    {    0,  268,     935,     947,     968, 12, 21, 19, }, // Armenia
    {    0,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {    0,  295,    1044,    1066,    1098, 22, 32, 32, }, // Australia_Central
    {    0,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {    0,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {    0,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {    0,  374,    1413,    1428,    1452, 15, 24, 22, }, // Azerbaijan
    {    0,  385,    1474,    1485,    1505, 11, 20, 18, }, // Azores
    {    0,  397,    1523,    1538,    1562, 15, 24, 22, }, // Bangladesh
    {    0,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {    0,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia
    {    0,  437,    1607,    1620,    1642, 13, 22, 20, }, // Brasilia
    {    0,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {    0,  461,    1684,    1699,    1723, 15, 24, 22, }, // Cape_Verde
    {    0,  472,       0,    1745,       0,  0, 10,  0, }, // Casey
    {    0,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro
    {    0,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {    0,  495,    1831,    1841,    1860, 10, 19, 17, }, // Chile
    {    0,  501,    1877,    1887,    1906, 10, 19, 19, }, // China
    {    0,  507,       0,    1925,       0,  0, 21,  0, }, // Christmas
    {    0,  517,       0,    1946,       0,  0, 18,  0, }, // Cocos
    {    0,  523,    1964,    1977,    1999, 13, 22, 20, }, // Colombia
    {    0,  532,    2019,    2036,    2062, 17, 26, 29, }, // Cook
    {    0,  537,    2091,    2100,    2118,  9, 18, 18, }, // Cuba
    {    0,  548,       0,    2136,       0,  0, 10,  0, }, // Davis
    {    0,  564,       0,    2146,       0,  0, 21,  0, }, // DumontDUrville
    {    0,  601,       0,    2167,       0,  0, 15,  0, }, // East_Timor
    {    0,  612,    2182,    2200,    2227, 18, 27, 25, }, // Easter
    {    0,  619,       0,    2252,       0,  0, 12,  0, }, // Ecuador
    {    0,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {    0,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {    0,  657,       0,    2422,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {    0,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {    0,  695,    2530,    2551,    2581, 21, 30, 28, }, // Falkland
    {    0,  704,    2609,    2618,    2636,  9, 18, 16, }, // Fiji
    {    0,  709,       0,    2652,       0,  0, 18,  0, }, // French_Guiana
    {    0,  723,       0,    2670,       0,  0, 32,  0, }, // French_Southern
    {    0,  746,       0,    2702,       0,  0, 14,  0, }, // Galapagos
    {    0,  756,       0,    2716,       0,  0, 12,  0, }, // Gambier
    {    0,  764,    2728,    2740,    2761, 12, 21, 19, }, // Georgia
    {    0,  772,       0,    2780,       0,  0, 20,  0, }, // Gilbert_Islands
    {    0,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {    0,  802,    2819,    2833,    2856, 14, 23, 21, }, // Greenland
    {    0,  830,    2877,    2896,    2924, 19, 28, 26, }, // Greenland_Eastern
    {    0,  848,    2950,    2969,    2997, 19, 28, 26, }, // Greenland_Western
    {    0,  866,       0,    3023,       0,  0, 18,  0, }, // Guam
    {    0,  871,       0,    3041,       0,  0, 18,  0, }, // Gulf
    {    0,  876,       0,    3059,       0,  0, 11,  0, }, // Guyana
    {    0,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {    0,  899,    3148,    3162,    3185, 14, 23, 21, }, // Hong_Kong
    {    0,  909,    3206,    3215,    3233,  9, 18, 16, }, // Hovd
    {    0,  914,       0,    3249,       0,  0, 19,  0, }, // India
    {    0,  920,       0,    3268,       0,  0, 17,  0, }, // Indian_Ocean
    {    0,  933,       0,    3285,       0,  0, 14,  0, }, // Indochina
    {    0,  943,       0,    3299,       0,  0, 22,  0, }, // Indonesia_Central
    {    0,  961,       0,    3321,       0,  0, 22,  0, }, // Indonesia_Eastern
    {    0,  979,       0,    3343,       0,  0, 22,  0, }, // Indonesia_Western
    {    0,  997,    3365,    3374,    3392,  9, 18, 18, }, // Iran
    {    0, 1008,    3410,    3422,    3443, 12, 21, 19, }, // Irkutsk
    {    0, 1016,    3462,    3473,    3493, 11, 20, 20, }, // Israel
    {    0, 1023,    3513,    3523,    3542, 10, 19, 19, }, // Japan
    {    0, 1029,    3561,    3590,    3628, 29, 38, 36, }, // Kamchatka
    {    0, 1047,       0,    3664,       0,  0, 15,  0, }, // Kazakhstan
    {    0, 1058,       0,    3679,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {    0, 1077,       0,    3699,       0,  0, 20,  0, }, // Kazakhstan_Western
    {    0, 1106,    3719,    3730,    3750, 11, 20, 20, }, // Korea
    {    0, 1112,       0,    3770,       0,  0, 11,  0, }, // Kosrae
    {    0, 1119,    3781,    3797,    3822, 16, 25, 23, }, // Krasnoyarsk
    {    0, 1151,       0,    3845,       0,  0, 15,  0, }, // Kyrgystan
    {    0, 1161,       0,    3860,       0,  0, 10,  0, }, // Lanka
    {    0, 1175,       0,    3870,       0,  0, 17,  0, }, // Line_Islands
    {    0, 1188,    3887,    3901,    3924, 14, 23, 23, }, // Lord_Howe
    {    0, 1198,    3947,    3957,    3976, 10, 19, 17, }, // Macau
    {    0, 1204,    3993,    4005,    4026, 12, 21, 19, }, // Magadan
    {    0, 1219,       0,    4045,       0,  0, 13,  0, }, // Malaysia
    {    0, 1228,       0,    4058,       0,  0, 13,  0, }, // Maldives
    {    0, 1237,       0,    4071,       0,  0, 14,  0, }, // Marquesas
    {    0, 1247,       0,    4085,       0,  0, 21,  0, }, // Marshall_Islands
    {    0, 1264,    4106,    4120,    4143, 14, 23, 21, }, // Mauritius
    {    0, 1274,       0,    4164,       0,  0, 11,  0, }, // Mawson
    {    0, 1281,    4175,    4195,    4224, 20, 29, 29, }, // Mexico_Pacific
    {    0, 1296,    4253,    4269,    4294, 16, 25, 23, }, // Mongolia
    {    0, 1305,    4317,    4328,    4348, 11, 20, 18, }, // Moscow
    {    0, 1312,       0,    4366,       0,  0, 12,  0, }, // Myanmar
    {    0, 1320,       0,    4378,       0,  0, 10,  0, }, // Nauru
    {    0, 1326,       0,    4388,       0,  0, 10,  0, }, // Nepal
    {    0, 1332,    4398,    4416,    4443, 18, 27, 25, }, // New_Caledonia
    {    0, 1346,    4468,    4484,    4509, 16, 25, 25, }, // New_Zealand
    {    0, 1358,    4534,    4551,    4577, 17, 26, 26, }, // Newfoundland
    {    0, 1371,       0,    4603,       0,  0,  9,  0, }, // Niue
    {    0, 1376,    4612,    4631,    4659, 19, 28, 28, }, // Norfolk
    {    0, 1384,    4687,    4711,    4744, 24, 33, 31, }, // Noronha
    {    0, 1392,       0,    4775,       0,  0, 26,  0, }, // North_Mariana
    {    0, 1406,    4801,    4817,    4842, 16, 25, 23, }, // Novosibirsk
    {    0, 1418,    4865,    4874,    4892,  9, 18, 16, }, // Omsk
    {    0, 1428,    4908,    4921,    4943, 13, 22, 20, }, // Pakistan
    {    0, 1437,       0,    4963,       0,  0, 10,  0, }, // Palau
    {    0, 1443,       0,    4973,       0,  0, 21,  0, }, // Papua_New_Guinea
    {    0, 1460,    4994,    5007,    5029, 13, 22, 20, }, // Paraguay
    {    0, 1469,    5049,    5058,    5076,  9, 18, 16, }, // Peru
    {    0, 1474,    5092,    5107,    5131, 15, 24, 22, }, // Philippines
    {    0, 1486,       0,    5153,       0,  0, 20,  0, }, // Phoenix_Islands
    {    0, 1502,    5173,    5199,    5234, 26, 35, 35, }, // Pierre_Miquelon
    {    0, 1518,       0,    5269,       0,  0, 13,  0, }, // Pitcairn
    {    0, 1527,       0,    5282,       0,  0, 11,  0, }, // Ponape
    {    0, 1534,       0,    5293,       0,  0, 14,  0, }, // Pyongyang
    {    0, 1544,    5307,    5321,    5344, 14, 23, 21, }, // Qyzylorda
    {    0, 1554,       0,    5365,       0,  0, 12,  0, }, // Reunion
    {    0, 1562,       0,    5377,       0,  0, 12,  0, }, // Rothera
    {    0, 1570,    5389,    5402,    5424, 13, 22, 20, }, // Sakhalin
    {    0, 1579,    5444,    5455,    5475, 11, 20, 18, }, // Samara
    {    0, 1596,    5493,    5503,    5522, 10, 19, 19, }, // Samoa
    {    0, 1602,       0,    5541,       0,  0, 15,  0, }, // Seychelles
    {    0, 1624,       0,    5556,       0,  0, 23,  0, }, // Singapore
    {    0, 1634,       0,    5579,       0,  0, 20,  0, }, // Solomon
    {    0, 1642,       0,    5599,       0,  0, 18,  0, }, // South_Georgia
    {    0, 1656,       0,    5617,       0,  0, 13,  0, }, // Suriname
    {    0, 1676,       0,    5630,       0,  0, 10,  0, }, // Syowa
    {    0, 1682,       0,    5640,       0,  0, 11,  0, }, // Tahiti
    {    0, 1689,    5651,    5662,    5682, 11, 20, 20, }, // Taipei
    {    0, 1696,       0,    5702,       0,  0, 15,  0, }, // Tajikistan
    {    0, 1724,       0,    5717,       0,  0, 12,  0, }, // Tokelau
    {    0, 1732,    5729,    5739,    5758, 10, 19, 17, }, // Tonga
    {    0, 1738,       0,    5775,       0,  0, 10,  0, }, // Truk
    {    0, 1750,    5785,    5802,    5828, 17, 26, 24, }, // Turkmenistan
    {    0, 1763,       0,    5852,       0,  0, 11,  0, }, // Tuvalu
    {    0, 1777,    5863,    5875,    5896, 12, 21, 19, }, // Uruguay
    {    0, 1792,    5915,    5930,    5954, 15, 24, 22, }, // Uzbekistan
    {    0, 1803,    5976,    5988,    6009, 12, 21, 19, }, // Vanuatu
    {    0, 1811,       0,    6028,       0,  0, 14,  0, }, // Venezuela
    {    0, 1821,    6042,    6058,    6083, 16, 25, 23, }, // Vladivostok
    {    0, 1833,    6106,    6120,    6143, 14, 23, 21, }, // Volgograd
    {    0, 1843,       0,    6164,       0,  0, 11,  0, }, // Vostok
    {    0, 1850,       0,    6175,       0,  0, 16,  0, }, // Wake
    {    0, 1855,       0,    6191,       0,  0, 20,  0, }, // Wallis
    {    0, 1862,    6211,    6223,    6244, 12, 21, 19, }, // Yakutsk
    {    0, 1870,    6263,    6281,    6308, 18, 27, 25, }, // Yekaterinburg
    {    0, 1892,       0,    6333,       0,  0, 10,  0, }, // Yukon
    {    1,    0,    6343,    6353,    6374, 10, 21, 19, }, // Acre Abkhazian/Cyrillic/Georgia
    {    1,    5,       0,    6393,       0,  0, 10,  0, }, // Afghanistan
    {    1,   17,       0,    6403,       0,  0, 16,  0, }, // Africa_Central
    {    1,   32,       0,    6419,       0,  0, 19,  0, }, // Africa_Eastern
    {    1,   65,       0,    6438,       0,  0, 15,  0, }, // Africa_Southern
    {    1,   81,    6453,    6473,    6511, 20, 38, 36, }, // Africa_Western
    {    1,  107,    6547,    6554,    6579,  7, 25, 23, }, // Alaska
    {    1,  128,    6602,    6616,    6641, 14, 25, 23, }, // Almaty
    {    1,  135,    6664,    6672,    6698,  8, 26, 24, }, // Amazon
    {    1,  142,    6722,    6739,    6774, 17, 35, 33, }, // America_Central
    {    1,  158,    6807,    6827,    6865, 20, 38, 36, }, // America_Eastern
    {    1,  174,    6901,    6933,    6976, 32, 43, 41, }, // America_Mountain
    {    1,  191,    7017,    7036,    7066, 19, 30, 28, }, // America_Pacific
    {    1,  207,    7094,    7110,    7133, 16, 23, 21, }, // Anadyr
    {    1,  214,    7154,    7158,    7180,  4, 22, 20, }, // Apia
    {    1,  219,    7200,    7211,    7234, 11, 23, 20, }, // Aqtau
    {    1,  225,    7254,    7266,    7289, 12, 23, 21, }, // Aqtobe
    {    1,  232,    7310,    7326,    7360, 16, 34, 32, }, // Arabian
    {    1,  240,    7392,    7401,    7428,  9, 27, 25, }, // Argentina
    {    1,  250,    7453,    7476,    7517, 23, 41, 39, }, // Argentina_Western
    {    1,  268,    7556,    7566,    7594, 10, 28, 26, }, // Armenia
    {    1,  286,    7620,    7638,    7667, 18, 29, 27, }, // Atlantic
    {    1,  295,    7694,    7713,    7750, 19, 37, 35, }, // Australia_Central
    {    1,  313,    7785,    7825,    7876, 40, 51, 49, }, // Australia_CentralWestern
    {    1,  338,    7925,    7947,    7987, 22, 40, 38, }, // Australia_Eastern
    {    1,  356,    8025,    8048,    8089, 23, 41, 39, }, // Australia_Western
    {    1,  374,    8128,    8139,    8168, 11, 29, 27, }, // Azerbaijan
    {    1,  385,    8195,    8211,    8245, 16, 34, 32, }, // Azores
    {    1,  397,    8277,    8286,    8313,  9, 27, 25, }, // Bangladesh
    {    1,  415,       0,    8338,       0,  0,  5,  0, }, // Bhutan
    {    1,  422,       0,    8343,       0,  0,  7,  0, }, // Bolivia
    {    1,  437,    8350,    8358,    8384,  8, 26, 24, }, // Brasilia
    {    1,  454,       0,    8408,       0,  0, 17,  0, }, // Brunei
    {    1,  461,    8425,    8435,    8463, 10, 28, 26, }, // Cape_Verde
    {    1,  472,       0,    8489,       0,  0,  5,  0, }, // Casey
    {    1,  478,       0,    8494,       0,  0,  7,  0, }, // Chamorro
    {    1,  487,    8501,    8506,    8529,  5, 23, 21, }, // Chatham
    {    1,  495,    8550,    8554,    8576,  4, 22, 20, }, // Chile
    {    1,  501,    8596,    8601,    8624,  5, 23, 21, }, // China
    {    1,  507,       0,    8645,       0,  0, 12,  0, }, // Christmas
    {    1,  517,       0,    8657,       0,  0, 17,  0, }, // Cocos
    {    1,  523,    8674,    8682,    8708,  8, 26, 24, }, // Colombia
    {    1,  532,    8732,    8751,    8788, 19, 37, 39, }, // Cook
    {    1,  537,    8827,    8831,    8853,  4, 22, 20, }, // Cuba
    {    1,  548,       0,    8873,       0,  0,  6,  0, }, // Davis
    {    1,  564,       0,    8879,       0,  0, 15,  0, }, // DumontDUrville
    {    1,  601,       0,    8894,       0,  0, 18,  0, }, // East_Timor
    {    1,  612,    8912,    8923,    8952, 11, 29, 27, }, // Easter
    {    1,  619,       0,    8979,       0,  0,  7,  0, }, // Ecuador
    {    1,  627,    8986,    9002,    9036, 16, 34, 32, }, // Europe_Central
    {    1,  642,    9068,    9087,    9124, 19, 37, 35, }, // Europe_Eastern
    {    1,  657,       0,    9159,       0,  0, 16,  0, }, // Europe_Further_Eastern
    {    1,  680,    9175,    9195,    9233, 20, 38, 36, }, // Europe_Western
    {    1,  695,    9269,    9289,    9327, 20, 38, 36, }, // Falkland
    {    1,  704,    9363,    9367,    9389,  4, 22, 20, }, // Fiji
    {    1,  709,       0,    9409,       0,  0, 17,  0, }, // French_Guiana
    {    1,  723,       0,    9426,       0,  0, 50,  0, }, // French_Southern
    {    1,  746,       0,    9476,       0,  0, 21,  0, }, // Galapagos
    {    1,  756,       0,    9497,       0,  0,  5,  0, }, // Gambier
    {    1,  764,    9502,    9511,    9538,  9, 27, 25, }, // Georgia
    {    1,  772,       0,    9563,       0,  0, 17,  0, }, // Gilbert_Islands
    {    1,  788,       0,    9580,       0,  0, 25,  0, }, // GMT
    {    1,  830,    9605,    9628,    9670, 23, 42, 39, }, // Greenland_Eastern
    {    1,  848,    9709,    9733,    9775, 24, 42, 40, }, // Greenland_Western
    {    1,  866,       0,    9815,       0,  0,  4,  0, }, // Guam
    {    1,  871,       0,    9819,       0,  0, 19,  0, }, // Gulf
    {    1,  876,       0,    9838,       0,  0,  6,  0, }, // Guyana
    {    1,  883,    9844,    9864,    9895, 20, 31, 29, }, // Hawaii_Aleutian
    {    1,  899,    9924,    9931,    9956,  7, 25, 23, }, // Hong_Kong
    {    1,  909,    9979,    9983,   10005,  4, 22, 20, }, // Hovd
    {    1,  914,       0,   10025,       0,  0,  5,  0, }, // India
    {    1,  920,       0,   10030,       0,  0, 15,  0, }, // Indian_Ocean
    {    1,  933,       0,   10045,       0,  0,  9,  0, }, // Indochina
    {    1,  943,       0,   10054,       0,  0, 19,  0, }, // Indonesia_Central
    {    1,  961,       0,   10073,       0,  0, 22,  0, }, // Indonesia_Eastern
    {    1,  979,       0,   10095,       0,  0, 23,  0, }, // Indonesia_Western
    {    1,  997,   10118,   10122,   10144,  4, 22, 20, }, // Iran
    {    1, 1008,   10164,   10171,   10196,  7, 25, 23, }, // Irkutsk
    {    1, 1016,   10219,   10226,   10251,  7, 25, 23, }, // Israel
    {    1, 1023,   10274,   10281,   10306,  7, 25, 23, }, // Japan
    {    1, 1029,   10329,   10354,   10397, 25, 43, 41, }, // Kamchatka
    {    1, 1058,       0,   10438,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {    1, 1077,       0,   10460,       0,  0, 23,  0, }, // Kazakhstan_Western
    {    1, 1106,   10483,   10489,   10513,  6, 24, 22, }, // Korea
    {    1, 1112,       0,   10535,       0,  0,  6,  0, }, // Kosrae
    {    1, 1119,   10541,   10552,   10581, 11, 29, 27, }, // Krasnoyarsk
    {    1, 1151,       0,   10608,       0,  0,  8,  0, }, // Kyrgystan
    {    1, 1161,       0,   10616,       0,  0,  9,  0, }, // Lanka
    {    1, 1175,       0,   10625,       0,  0, 13,  0, }, // Line_Islands
    {    1, 1188,   10638,   10646,   10672,  8, 26, 24, }, // Lord_Howe
    {    1, 1198,   10696,   10701,   10724,  5, 23, 21, }, // Macau
    {    1, 1204,   10745,   10752,   10777,  7, 25, 23, }, // Magadan
    {    1, 1219,       0,   10800,       0,  0,  8,  0, }, // Malaysia
    {    1, 1228,       0,   10808,       0,  0,  7,  0, }, // Maldives
    {    1, 1237,       0,   10815,       0,  0, 18,  0, }, // Marquesas
    {    1, 1247,       0,   10833,       0,  0, 26,  0, }, // Marshall_Islands
    {    1, 1264,   10859,   10866,   10891,  7, 25, 23, }, // Mauritius
    {    1, 1274,       0,   10914,       0,  0,  6,  0, }, // Mawson
    {    1, 1281,   10920,   10950,   10991, 30, 41, 39, }, // Mexico_Pacific
    {    1, 1296,   11030,   11040,   11068, 10, 28, 26, }, // Mongolia
    {    1, 1305,   11094,   11100,   11124,  6, 24, 22, }, // Moscow
    {    1, 1312,       0,   11146,       0,  0,  6,  0, }, // Myanmar
    {    1, 1320,       0,   11152,       0,  0,  5,  0, }, // Nauru
    {    1, 1326,       0,   11157,       0,  0,  5,  0, }, // Nepal
    {    1, 1332,   11162,   11175,   11206, 13, 31, 29, }, // New_Caledonia
    {    1, 1346,   11235,   11247,   11277, 12, 30, 28, }, // New_Zealand
    {    1, 1358,   11305,   11317,   11347, 12, 30, 28, }, // Newfoundland
    {    1, 1371,       0,   11375,       0,  0,  4,  0, }, // Niue
    {    1, 1376,   11379,   11386,   11411,  7, 25, 23, }, // Norfolk
    {    1, 1384,   11434,   11453,   11490, 19, 37, 35, }, // Noronha
    {    1, 1392,       0,   11525,       0,  0, 27,  0, }, // North_Mariana
    {    1, 1406,   11552,   11563,   11592, 11, 29, 27, }, // Novosibirsk
    {    1, 1418,   11619,   11623,   11645,  4, 22, 20, }, // Omsk
    {    1, 1428,   11665,   11673,   11699,  8, 26, 24, }, // Pakistan
    {    1, 1437,       0,   11723,       0,  0,  5,  0, }, // Palau
    {    1, 1443,       0,   11728,       0,  0, 19,  0, }, // Papua_New_Guinea
    {    1, 1460,   11747,   11755,   11781,  8, 26, 24, }, // Paraguay
    {    1, 1469,   11805,   11809,   11831,  4, 22, 20, }, // Peru
    {    1, 1474,   11851,   11859,   11885,  8, 26, 24, }, // Philippines
    {    1, 1486,       0,   11909,       0,  0, 15,  0, }, // Phoenix_Islands
    {    1, 1502,   11924,   11942,   11978, 18, 36, 34, }, // Pierre_Miquelon
    {    1, 1518,       0,   12012,       0,  0,  7,  0, }, // Pitcairn
    {    1, 1527,       0,   12019,       0,  0,  6,  0, }, // Ponape
    {    1, 1534,       0,   12025,       0,  0,  7,  0, }, // Pyongyang
    {    1, 1544,   12032,   12042,   12070, 10, 28, 26, }, // Qyzylorda
    {    1, 1554,       0,   12096,       0,  0,  7,  0, }, // Reunion
    {    1, 1562,       0,   12103,       0,  0,  6,  0, }, // Rothera
    {    1, 1570,   12109,   12116,   12141,  7, 25, 23, }, // Sakhalin
    {    1, 1579,   12164,   12178,   12203, 14, 25, 23, }, // Samara
    {    1, 1596,   12226,   12231,   12254,  5, 23, 21, }, // Samoa
    {    1, 1602,       0,   12275,       0,  0, 25,  0, }, // Seychelles
    {    1, 1624,       0,   12300,       0,  0,  8,  0, }, // Singapore
    {    1, 1634,       0,   12308,       0,  0, 26,  0, }, // Solomon
    {    1, 1642,       0,   12334,       0,  0, 16,  0, }, // South_Georgia
    {    1, 1656,       0,   12350,       0,  0,  7,  0, }, // Suriname
    {    1, 1676,       0,   12357,       0,  0,  5,  0, }, // Syowa
    {    1, 1682,       0,   12362,       0,  0,  5,  0, }, // Tahiti
    {    1, 1689,   12367,   12373,   12397,  6, 24, 22, }, // Taipei
    {    1, 1696,       0,   12419,       0,  0, 10,  0, }, // Tajikistan
    {    1, 1724,       0,   12429,       0,  0,  7,  0, }, // Tokelau
    {    1, 1732,   12436,   12441,   12464,  5, 23, 21, }, // Tonga
    {    1, 1738,       0,   12485,       0,  0,  4,  0, }, // Truk
    {    1, 1750,   12489,   12501,   12531, 12, 30, 28, }, // Turkmenistan
    {    1, 1763,       0,   12559,       0,  0,  6,  0, }, // Tuvalu
    {    1, 1777,   12565,   12572,   12597,  7, 25, 23, }, // Uruguay
    {    1, 1792,   12620,   12631,   12660, 11, 29, 27, }, // Uzbekistan
    {    1, 1803,   12687,   12694,   12719,  7, 25, 23, }, // Vanuatu
    {    1, 1811,       0,   12742,       0,  0,  9,  0, }, // Venezuela
    {    1, 1821,   12751,   12762,   12791, 11, 29, 27, }, // Vladivostok
    {    1, 1833,   12818,   12827,   12854,  9, 27, 25, }, // Volgograd
    {    1, 1843,       0,   12879,       0,  0, 10,  0, }, // Vostok
    {    1, 1850,       0,   12889,       0,  0,  4,  0, }, // Wake
    {    1, 1855,       0,   12893,       0,  0, 15,  0, }, // Wallis
    {    1, 1862,   12908,   12915,   12940,  7, 25, 23, }, // Yakutsk
    {    1, 1870,   12963,   12975,   13005, 12, 30, 28, }, // Yekaterinburg
    {    1, 1892,       0,   13033,       0,  0,  5,  0, }, // Yukon
    {    5,    5,       0,   13038,       0,  0, 14,  0, }, // Afghanistan Afrikaans/Latin/South Africa
    {    5,   17,       0,   13052,       0,  0, 19,  0, }, // Africa_Central
    {    5,   32,       0,   13071,       0,  0, 14,  0, }, // Africa_Eastern
    {    5,   65,       0,   13085,       0,  0, 24,  0, }, // Africa_Southern
    {    5,   81,   13109,   13123,   13146, 14, 23, 19, }, // Africa_Western
    {    5,  107,   13165,   13175,   13194, 10, 19, 16, }, // Alaska
    {    5,  135,   13210,   13221,   13241, 11, 20, 16, }, // Amazon
    {    5,  142,   13257,   13287,   13326, 30, 39, 36, }, // America_Central
    {    5,  158,   13362,   13393,   13433, 31, 40, 37, }, // America_Eastern
    {    5,  174,   13470,   13495,   13530, 25, 35, 32, }, // America_Mountain
    {    5,  191,   13562,   13575,   13597, 13, 22, 19, }, // America_Pacific
    {    5,  207,   13616,   13626,   13645, 10, 19, 15, }, // Anadyr
    {    5,  214,   13660,   13668,   13685,  8, 17, 14, }, // Apia
    {    5,  232,   13699,   13711,   13732, 12, 21, 18, }, // Arabian
    {    5,  240,   13750,   13764,   13787, 14, 23, 19, }, // Argentina
    {    5,  250,   13806,   13824,   13851, 18, 27, 23, }, // Argentina_Western
    {    5,  268,   13874,   13885,   13905, 11, 20, 16, }, // Armenia
    {    5,  286,   13921,   13935,   13958, 14, 23, 20, }, // Atlantic
    {    5,  295,   13978,   14002,   14035, 24, 33, 30, }, // Australia_Central
    {    5,  313,   14065,   14099,   14143, 34, 44, 40, }, // Australia_CentralWestern
    {    5,  338,   14183,   14208,   14242, 25, 34, 31, }, // Australia_Eastern
    {    5,  356,   14273,   14298,   14332, 25, 34, 31, }, // Australia_Western
    {    5,  374,   14363,   14378,   14402, 15, 24, 20, }, // Azerbaijan
    {    5,  385,   14422,   14431,   14449,  9, 18, 14, }, // Azores
    {    5,  397,   14463,   14477,   14500, 14, 23, 19, }, // Bangladesh
    {    5,  415,       0,   14519,       0,  0, 11,  0, }, // Bhutan
    {    5,  422,       0,   14530,       0,  0, 11,  0, }, // Bolivia
    {    5,  437,   14541,   14553,   14574, 12, 21, 17, }, // Brasilia
    {    5,  454,       0,   14591,       0,  0, 22,  0, }, // Brunei
    {    5,  461,   14613,   14627,   14650, 14, 23, 19, }, // Cape_Verde
    {    5,  478,       0,   14669,       0,  0, 21,  0, }, // Chamorro
    {    5,  487,   14690,   14701,   14721, 11, 20, 17, }, // Chatham
    {    5,  495,   14738,   14747,   14765,  9, 18, 14, }, // Chile
    {    5,  501,   14779,   14788,   14806,  9, 18, 15, }, // China
    {    5,  507,       0,   14821,       0,  0, 19,  0, }, // Christmas
    {    5,  517,       0,   14840,       0,  0, 16,  0, }, // Cocos
    {    5,  523,   14856,   14868,   14889, 12, 21, 17, }, // Colombia
    {    5,  532,   14906,   14921,   14945, 15, 24, 24, }, // Cook
    {    5,  537,   14969,   14977,   14994,  8, 17, 14, }, // Cuba
    {    5,  548,       0,   15008,       0,  0,  9,  0, }, // Davis
    {    5,  564,       0,   15017,       0,  0, 20,  0, }, // DumontDUrville
    {    5,  601,       0,   15037,       0,  0, 13,  0, }, // East_Timor
    {    5,  612,   15050,   15064,   15087, 14, 23, 19, }, // Easter
    {    5,  619,       0,   15106,       0,  0, 11,  0, }, // Ecuador
    {    5,  627,   15117,   15138,   15168, 21, 30, 26, }, // Europe_Central
    {    5,  642,   15194,   15210,   15235, 16, 25, 21, }, // Europe_Eastern
    {    5,  657,       0,   15256,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {    5,  680,   15279,   15295,   15320, 16, 25, 21, }, // Europe_Western
    {    5,  695,   15341,   15360,   15388, 19, 28, 24, }, // Falkland
    {    5,  704,   15412,   15421,   15439,  9, 18, 14, }, // Fiji
    {    5,  709,       0,   15453,       0,  0, 16,  0, }, // French_Guiana
    {    5,  723,       0,   15469,       0,  0, 33,  0, }, // French_Southern
    {    5,  746,       0,   15502,       0,  0, 13,  0, }, // Galapagos
    {    5,  756,       0,   15515,       0,  0, 11,  0, }, // Gambier
    {    5,  764,   15526,   15537,   15557, 11, 20, 16, }, // Georgia
    {    5,  772,       0,   15573,       0,  0, 18,  0, }, // Gilbert_Islands
    {    5,  788,       0,   15591,       0,  0, 13,  0, }, // GMT
    {    5,  830,   15604,   15621,   15647, 17, 26, 22, }, // Greenland_Eastern
    {    5,  848,   15669,   15686,   15712, 17, 26, 22, }, // Greenland_Western
    {    5,  871,       0,   15734,       0,  0, 26,  0, }, // Gulf
    {    5,  876,       0,   15459,       0,  0, 10,  0, }, // Guyana
    {    5,  883,   15760,   15778,   15805, 18, 27, 24, }, // Hawaii_Aleutian
    {    5,  899,   15829,   15841,   15862, 12, 21, 17, }, // Hong_Kong
    {    5,  909,   15879,   15887,   15904,  8, 17, 13, }, // Hovd
    {    5,  914,       0,   15917,       0,  0, 18,  0, }, // India
    {    5,  920,       0,   15935,       0,  0, 18,  0, }, // Indian_Ocean
    {    5,  933,       0,   15953,       0,  0, 13,  0, }, // Indochina
    {    5,  943,       0,   15966,       0,  0, 24,  0, }, // Indonesia_Central
    {    5,  961,       0,   15990,       0,  0, 17,  0, }, // Indonesia_Eastern
    {    5,  979,       0,   16007,       0,  0, 17,  0, }, // Indonesia_Western
    {    5,  997,   16024,   16032,   16049,  8, 17, 14, }, // Iran
    {    5, 1008,   16063,   16075,   16096, 12, 21, 17, }, // Irkutsk
    {    5, 1016,   16113,   16123,   16142, 10, 19, 16, }, // Israel
    {    5, 1023,   16158,   16167,   16185,  9, 18, 15, }, // Japan
    {    5, 1029,   16200,   16228,   16265, 28, 37, 33, }, // Kamchatka
    {    5, 1047,       0,   16298,       0,  0, 13,  0, }, // Kazakhstan
    {    5, 1058,       0,   16311,       0,  0, 17,  0, }, // Kazakhstan_Eastern
    {    5, 1077,       0,   16328,       0,  0, 17,  0, }, // Kazakhstan_Western
    {    5, 1106,   16345,   16358,   16380, 13, 22, 19, }, // Korea
    {    5, 1112,       0,   16399,       0,  0, 10,  0, }, // Kosrae
    {    5, 1119,   16409,   16424,   16448, 15, 24, 20, }, // Krasnoyarsk
    {    5, 1151,       0,   16468,       0,  0, 13,  0, }, // Kyrgystan
    {    5, 1175,       0,   16481,       0,  0, 16,  0, }, // Line_Islands
    {    5, 1188,   16497,   16510,   16532, 13, 22, 19, }, // Lord_Howe
    {    5, 1204,   16551,   16562,   16582, 11, 20, 16, }, // Magadan
    {    5, 1219,       0,   16598,       0,  0, 12,  0, }, // Malaysia
    {    5, 1228,       0,   16610,       0,  0, 12,  0, }, // Maldives
    {    5, 1237,       0,   16622,       0,  0, 13,  0, }, // Marquesas
    {    5, 1247,       0,   16635,       0,  0, 19,  0, }, // Marshall_Islands
    {    5, 1264,   16654,   16667,   16689, 13, 22, 18, }, // Mauritius
    {    5, 1274,       0,   16707,       0,  0, 10,  0, }, // Mawson
    {    5, 1281,   16717,   16742,   16776, 25, 34, 31, }, // Mexico_Pacific
    {    5, 1296,   16807,   16822,   16846, 15, 24, 20, }, // Mongolia
    {    5, 1305,   16866,   16876,   16895, 10, 19, 15, }, // Moscow
    {    5, 1312,       0,   16910,       0,  0, 11,  0, }, // Myanmar
    {    5, 1320,       0,   16921,       0,  0,  9,  0, }, // Nauru
    {    5, 1326,       0,   16930,       0,  0,  9,  0, }, // Nepal
    {    5, 1332,   16939,   16957,   16984, 18, 27, 23, }, // New_Caledonia
    {    5, 1346,   17007,   17023,   17048, 16, 25, 22, }, // New_Zealand
    {    5, 1358,   17070,   17086,   17111, 16, 25, 22, }, // Newfoundland
    {    5, 1371,       0,   17133,       0,  0,  8,  0, }, // Niue
    {    5, 1376,   17141,   17158,   17184, 17, 26, 23, }, // Norfolk
    {    5, 1384,   17207,   17230,   17262, 23, 32, 28, }, // Noronha
    {    5, 1406,   17290,   17305,   17329, 15, 24, 20, }, // Novosibirsk
    {    5, 1418,   17349,   17357,   17374,  8, 17, 13, }, // Omsk
    {    5, 1428,   17387,   17399,   17420, 12, 21, 17, }, // Pakistan
    {    5, 1437,       0,   17437,       0,  0,  9,  0, }, // Palau
    {    5, 1443,       0,   17446,       0,  0, 22,  0, }, // Papua_New_Guinea
    {    5, 1460,   17468,   17480,   17501, 12, 21, 17, }, // Paraguay
    {    5, 1469,   17518,   17526,   17543,  8, 17, 13, }, // Peru
    {    5, 1474,   17556,   17570,   17593, 14, 23, 19, }, // Philippines
    {    5, 1486,       0,   17612,       0,  0, 17,  0, }, // Phoenix_Islands
    {    5, 1502,   17629,   17656,   17692, 27, 36, 33, }, // Pierre_Miquelon
    {    5, 1518,       0,   17725,       0,  0, 12,  0, }, // Pitcairn
    {    5, 1527,       0,   17737,       0,  0, 10,  0, }, // Ponape
    {    5, 1534,       0,   17747,       0,  0, 13,  0, }, // Pyongyang
    {    5, 1554,       0,   17760,       0,  0, 11,  0, }, // Reunion
    {    5, 1562,       0,   17771,       0,  0, 11,  0, }, // Rothera
    {    5, 1570,   17782,   17794,   17815, 12, 21, 17, }, // Sakhalin
    {    5, 1579,   17832,   17842,   17861, 10, 19, 16, }, // Samara
    {    5, 1596,   17877,   17886,   17904,  9, 18, 15, }, // Samoa
    {    5, 1602,       0,   17919,       0,  0, 13,  0, }, // Seychelles
    {    5, 1624,       0,   17932,       0,  0, 22,  0, }, // Singapore
    {    5, 1634,       0,   17954,       0,  0, 19,  0, }, // Solomon
    {    5, 1642,       0,   17973,       0,  0, 16,  0, }, // South_Georgia
    {    5, 1656,       0,   17989,       0,  0, 12,  0, }, // Suriname
    {    5, 1676,       0,   18001,       0,  0,  9,  0, }, // Syowa
    {    5, 1682,       0,   18010,       0,  0, 10,  0, }, // Tahiti
    {    5, 1689,   18020,   18030,   18049, 10, 19, 16, }, // Taipei
    {    5, 1696,       0,   18065,       0,  0, 15,  0, }, // Tajikistan
    {    5, 1724,       0,   18080,       0,  0, 11,  0, }, // Tokelau
    {    5, 1732,   18091,   18100,   18118,  9, 18, 14, }, // Tonga
    {    5, 1738,       0,   18132,       0,  0,  9,  0, }, // Truk
    {    5, 1750,   18141,   18157,   18182, 16, 25, 21, }, // Turkmenistan
    {    5, 1763,       0,   18203,       0,  0, 10,  0, }, // Tuvalu
    {    5, 1777,   18213,   18224,   18244, 11, 20, 16, }, // Uruguay
    {    5, 1792,   18260,   18275,   18299, 15, 24, 20, }, // Uzbekistan
    {    5, 1803,   18319,   18330,   18350, 11, 20, 16, }, // Vanuatu
    {    5, 1811,       0,   18366,       0,  0, 13,  0, }, // Venezuela
    {    5, 1821,   18379,   18394,   18418, 15, 24, 20, }, // Vladivostok
    {    5, 1833,   18438,   18451,   18473, 13, 22, 18, }, // Volgograd
    {    5, 1843,       0,   18491,       0,  0, 10,  0, }, // Vostok
    {    5, 1850,       0,   18501,       0,  0, 15,  0, }, // Wake
    {    5, 1855,       0,   18516,       0,  0, 20,  0, }, // Wallis
    {    5, 1862,   18536,   18548,   18569, 12, 21, 17, }, // Yakutsk
    {    5, 1870,   18586,   18603,   18629, 17, 26, 22, }, // Yekaterinburg
    {    5, 1892,       0,   18651,       0,  0,  8,  0, }, // Yukon
    {    8,    5,       0,   18659,       0,  0, 15,  0, }, // Afghanistan Akan/Latin/Ghana
    {    8,   17,       0,   18674,       0,  0, 20,  0, }, // Africa_Central
    {    8,   32,       0,   18694,       0,  0, 18,  0, }, // Africa_Eastern
    {    8,   65,       0,   18712,       0,  0, 27,  0, }, // Africa_Southern
    {    8,   81,   18739,   18756,   18781, 17, 25, 22, }, // Africa_Western
    {    8,  107,   18803,   18814,   18833, 11, 19, 16, }, // Alaska
    {    8,  135,   18849,   18860,   18879, 11, 19, 16, }, // Amazon
    {    8,  142,   18895,   18910,   18933, 15, 23, 20, }, // America_Central
    {    8,  158,   18701,   18953,   18972, 11, 19, 16, }, // America_Eastern
    {    8,  174,   18988,   18997,   19014,  9, 17, 14, }, // America_Mountain
    {    8,  191,   19028,   19040,   19060, 12, 20, 17, }, // America_Pacific
    {    8,  214,   19077,   19086,   19103,  9, 17, 14, }, // Apia
    {    8,  232,   19117,   19128,   19147, 11, 19, 16, }, // Arabian
    {    8,  240,   19163,   19177,   19199, 14, 22, 19, }, // Argentina
    {    8,  250,   19218,   19239,   19268, 21, 29, 26, }, // Argentina_Western
    {    8,  268,   19294,   19306,   19326, 12, 20, 17, }, // Armenia
    {    8,  286,   19343,   19356,   19377, 13, 21, 18, }, // Atlantic
    {    8,  295,   19395,   19419,   19451, 24, 32, 29, }, // Australia_Central
    {    8,  313,   19480,   19511,   19550, 31, 39, 36, }, // Australia_CentralWestern
    {    8,  338,   19586,   19606,   19634, 20, 28, 25, }, // Australia_Eastern
    {    8,  356,   19659,   19679,   19707, 20, 28, 25, }, // Australia_Western
    {    8,  374,   19732,   19746,   19768, 14, 22, 19, }, // Azerbaijan
    {    8,  385,   19787,   19798,   19817, 11, 19, 16, }, // Azores
    {    8,  397,   19833,   19849,   19873, 16, 24, 21, }, // Bangladesh
    {    8,  415,       0,   19894,       0,  0, 10,  0, }, // Bhutan
    {    8,  422,       0,   19904,       0,  0, 12,  0, }, // Bolivia
    {    8,  437,   19916,   19929,   19950, 13, 21, 18, }, // Brasilia
    {    8,  454,       0,   19968,       0,  0, 21,  0, }, // Brunei
    {    8,  461,   19989,   20001,   20021, 12, 20, 17, }, // Cape_Verde
    {    8,  478,       0,   20038,       0,  0, 20,  0, }, // Chamorro
    {    8,  487,   20058,   20069,   20088, 11, 19, 16, }, // Chatham
    {    8,  495,   20104,   20114,   20132, 10, 18, 15, }, // Chile
    {    8,  501,   20147,   20158,   20177, 11, 19, 16, }, // China
    {    8,  507,       0,   20193,       0,  0, 19,  0, }, // Christmas
    {    8,  517,       0,   20212,       0,  0, 18,  0, }, // Cocos
    {    8,  523,   20230,   20243,   20264, 13, 21, 18, }, // Colombia
    {    8,  532,   20282,   20297,   20320, 15, 23, 23, }, // Cook
    {    8,  537,   20343,   20352,   20369,  9, 17, 14, }, // Cuba
    {    8,  548,       0,   20383,       0,  0, 10,  0, }, // Davis
    {    8,  564,       0,   20393,       0,  0, 21,  0, }, // DumontDUrville
    {    8,  601,       0,   20414,       0,  0, 17,  0, }, // East_Timor
    {    8,  612,   20431,   20448,   20473, 17, 25, 22, }, // Easter
    {    8,  619,       0,   20495,       0,  0, 13,  0, }, // Ecuador
    {    8,  627,   20508,   20530,   20560, 22, 30, 27, }, // Europe_Central
    {    8,  642,   20587,   20605,   20631, 18, 26, 23, }, // Europe_Eastern
    {    8,  657,       0,   20654,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {    8,  680,   20678,   20696,   20722, 18, 26, 23, }, // Europe_Western
    {    8,  695,   20745,   20763,   20789, 18, 26, 23, }, // Falkland
    {    8,  704,   20812,   20822,   20840, 10, 18, 15, }, // Fiji
    {    8,  709,       0,   20855,       0,  0, 19,  0, }, // French_Guiana
    {    8,  723,       0,   20874,       0,  0, 32,  0, }, // French_Southern
    {    8,  746,       0,   20906,       0,  0, 14,  0, }, // Galapagos
    {    8,  756,       0,   20920,       0,  0, 12,  0, }, // Gambier
    {    8,  764,   20932,   20944,   20964, 12, 20, 17, }, // Georgia
    {    8,  772,       0,   20981,       0,  0, 18,  0, }, // Gilbert_Islands
    {    8,  788,       0,   20999,       0,  0, 19,  0, }, // GMT
    {    8,  830,   21018,   21039,   21068, 21, 29, 26, }, // Greenland_Eastern
    {    8,  848,   21094,   21114,   21142, 20, 28, 25, }, // Greenland_Western
    {    8,  871,       0,   21167,       0,  0, 18,  0, }, // Gulf
    {    8,  876,       0,   20863,       0,  0, 11,  0, }, // Guyana
    {    8,  883,   21185,   21205,   21233, 20, 28, 25, }, // Hawaii_Aleutian
    {    8,  899,   21258,   21269,   21288, 11, 19, 16, }, // Hong_Kong
    {    8,  909,   21304,   21313,   21330,  9, 17, 14, }, // Hovd
    {    8,  914,       0,   21344,       0,  0, 18,  0, }, // India
    {    8,  920,       0,   21362,       0,  0, 13,  0, }, // Indian_Ocean
    {    8,  933,       0,   21375,       0,  0, 14,  0, }, // Indochina
    {    8,  943,       0,   21389,       0,  0, 26,  0, }, // Indonesia_Central
    {    8,  961,       0,   21415,       0,  0, 22,  0, }, // Indonesia_Eastern
    {    8,  979,       0,   21437,       0,  0, 22,  0, }, // Indonesia_Western
    {    8,  997,   21459,   21468,   21485,  9, 17, 14, }, // Iran
    {    8, 1008,   21499,   21511,   21531, 12, 20, 17, }, // Irkutsk
    {    8, 1016,   21548,   21559,   21578, 11, 19, 16, }, // Israel
    {    8, 1023,   21594,   21605,   21624, 11, 19, 16, }, // Japan
    {    8, 1047,       0,   21640,       0,  0, 14,  0, }, // Kazakhstan
    {    8, 1058,       0,   21654,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {    8, 1077,       0,   21675,       0,  0, 20,  0, }, // Kazakhstan_Western
    {    8, 1106,   21695,   21706,   21725, 11, 19, 16, }, // Korea
    {    8, 1112,       0,   21741,       0,  0, 11,  0, }, // Kosrae
    {    8, 1119,   21752,   21768,   21792, 16, 24, 21, }, // Krasnoyarsk
    {    8, 1151,       0,   21813,       0,  0, 14,  0, }, // Kyrgystan
    {    8, 1175,       0,   21827,       0,  0, 15,  0, }, // Line_Islands
    {    8, 1188,   21842,   21855,   21876, 13, 21, 18, }, // Lord_Howe
    {    8, 1204,   21894,   21906,   21926, 12, 20, 17, }, // Magadan
    {    8, 1219,       0,   21943,       0,  0, 13,  0, }, // Malaysia
    {    8, 1228,       0,   21956,       0,  0, 13,  0, }, // Maldives
    {    8, 1237,       0,   21969,       0,  0, 13,  0, }, // Marquesas
    {    8, 1247,       0,   21982,       0,  0, 18,  0, }, // Marshall_Islands
    {    8, 1264,   22000,   22015,   22038, 15, 23, 20, }, // Mauritius
    {    8, 1274,       0,   22058,       0,  0, 11,  0, }, // Mawson
    {    8, 1281,   22069,   22089,   22117, 20, 28, 25, }, // Mexico_Pacific
    {    8, 1296,   22142,   22156,   22178, 14, 22, 19, }, // Mongolia
    {    8, 1305,   22197,   22207,   22225, 10, 18, 15, }, // Moscow
    {    8, 1312,       0,   22240,       0,  0, 12,  0, }, // Myanmar
    {    8, 1320,       0,   22252,       0,  0, 10,  0, }, // Nauru
    {    8, 1326,       0,   22262,       0,  0, 10,  0, }, // Nepal
    {    8, 1332,   22272,   22293,   22322, 21, 29, 26, }, // New_Caledonia
    {    8, 1346,   22348,   22366,   22392, 18, 26, 23, }, // New_Zealand
    {    8, 1358,   22415,   22432,   22457, 17, 25, 22, }, // Newfoundland
    {    8, 1371,       0,   22479,       0,  0,  9,  0, }, // Niue
    {    8, 1376,   22488,   22506,   22532, 18, 26, 23, }, // Norfolk
    {    8, 1384,   22555,   22579,   22611, 24, 32, 29, }, // Noronha
    {    8, 1406,   22640,   22656,   22680, 16, 24, 21, }, // Novosibirsk
    {    8, 1418,   22701,   22710,   22727,  9, 17, 14, }, // Omsk
    {    8, 1428,   22741,   22754,   22775, 13, 21, 18, }, // Pakistan
    {    8, 1437,       0,   22793,       0,  0, 10,  0, }, // Palau
    {    8, 1443,       0,   22803,       0,  0, 22,  0, }, // Papua_New_Guinea
    {    8, 1460,   22825,   22838,   22859, 13, 21, 18, }, // Paraguay
    {    8, 1469,   22877,   22886,   22903,  9, 17, 14, }, // Peru
    {    8, 1474,   22917,   22929,   22949, 12, 20, 17, }, // Philippines
    {    8, 1486,       0,   22966,       0,  0, 18,  0, }, // Phoenix_Islands
    {    8, 1502,   22984,   23010,   23044, 26, 34, 31, }, // Pierre_Miquelon
    {    8, 1518,       0,   23075,       0,  0, 13,  0, }, // Pitcairn
    {    8, 1527,       0,   23088,       0,  0, 11,  0, }, // Ponape
    {    8, 1534,       0,   23099,       0,  0, 14,  0, }, // Pyongyang
    {    8, 1554,       0,   23113,       0,  0, 12,  0, }, // Reunion
    {    8, 1562,       0,   23125,       0,  0, 11,  0, }, // Rothera
    {    8, 1570,   23136,   23149,   23170, 13, 21, 18, }, // Sakhalin
    {    8, 1596,   23188,   23198,   23216, 10, 18, 15, }, // Samoa
    {    8, 1602,       0,   23231,       0,  0, 12,  0, }, // Seychelles
    {    8, 1624,       0,   23243,       0,  0, 20,  0, }, // Singapore
    {    8, 1634,       0,   23263,       0,  0, 19,  0, }, // Solomon
    {    8, 1642,       0,   23282,       0,  0, 21,  0, }, // South_Georgia
    {    8, 1656,       0,   23303,       0,  0, 13,  0, }, // Suriname
    {    8, 1676,       0,   23316,       0,  0, 10,  0, }, // Syowa
    {    8, 1682,       0,   23326,       0,  0, 11,  0, }, // Tahiti
    {    8, 1689,   23337,   23348,   23367, 11, 19, 16, }, // Taipei
    {    8, 1696,       0,   23383,       0,  0, 16,  0, }, // Tajikistan
    {    8, 1724,       0,   23399,       0,  0, 12,  0, }, // Tokelau
    {    8, 1732,   23411,   23421,   23439, 10, 18, 15, }, // Tonga
    {    8, 1738,       0,   23454,       0,  0, 10,  0, }, // Truk
    {    8, 1750,   23464,   23480,   23504, 16, 24, 21, }, // Turkmenistan
    {    8, 1763,       0,   23525,       0,  0, 11,  0, }, // Tuvalu
    {    8, 1777,   23536,   23549,   23570, 13, 21, 18, }, // Uruguay
    {    8, 1792,   23588,   23603,   23626, 15, 23, 20, }, // Uzbekistan
    {    8, 1803,   23646,   23658,   23678, 12, 20, 17, }, // Vanuatu
    {    8, 1811,       0,   23695,       0,  0, 14,  0, }, // Venezuela
    {    8, 1821,   23709,   23725,   23749, 16, 24, 21, }, // Vladivostok
    {    8, 1833,   23770,   23784,   23806, 14, 22, 19, }, // Volgograd
    {    8, 1843,       0,   23825,       0,  0, 11,  0, }, // Vostok
    {    8, 1850,       0,   23836,       0,  0, 16,  0, }, // Wake
    {    8, 1855,       0,   23852,       0,  0, 21,  0, }, // Wallis
    {    8, 1862,   23873,   23885,   23905, 12, 20, 17, }, // Yakutsk
    {    8, 1870,   23922,   23939,   23964, 17, 25, 22, }, // Yekaterinburg
    {    8, 1892,       0,   23986,       0,  0, 10,  0, }, // Yukon
    {   10,    0,   23996,   24014,   24042, 18, 28, 25, }, // Acre Albanian/Latin/Albania
    {   10,    5,       0,   24067,       0,  0, 18,  0, }, // Afghanistan
    {   10,   17,       0,   24085,       0,  0, 22,  0, }, // Africa_Central
    {   10,   32,       0,   24107,       0,  0, 21,  0, }, // Africa_Eastern
    {   10,   65,       0,   24128,       0,  0, 30,  0, }, // Africa_Southern
    {   10,   81,   24158,   24183,   24218, 25, 35, 32, }, // Africa_Western
    {   10,  107,   24250,   24263,   24286, 13, 23, 20, }, // Alaska
    {   10,  128,   24306,   24319,   24342, 13, 23, 20, }, // Almaty
    {   10,  135,   24362,   24376,   24400, 14, 24, 21, }, // Amazon
    {   10,  142,   24421,   24443,   24475, 22, 32, 29, }, // America_Central
    {   10,  158,   24504,   24525,   24556, 21, 31, 28, }, // America_Eastern
    {   10,  174,   24584,   24627,   24680, 43, 53, 50, }, // America_Mountain
    {   10,  191,   24730,   24780,   24840, 50, 60, 57, }, // America_Pacific
    {   10,  207,   24897,   24911,   24935, 14, 24, 21, }, // Anadyr
    {   10,  214,   24956,   24967,   24988, 11, 21, 18, }, // Apia
    {   10,  219,   25006,   25018,   25040, 12, 22, 19, }, // Aqtau
    {   10,  225,   25059,   25072,   25095, 13, 23, 20, }, // Aqtobe
    {   10,  232,   25115,   25124,   25143,  9, 19, 16, }, // Arabian
    {   10,  240,   25159,   25176,   25203, 17, 27, 24, }, // Argentina
    {   10,  250,   25227,   25256,   25295, 29, 39, 36, }, // Argentina_Western
    {   10,  268,   25331,   25345,   25369, 14, 24, 21, }, // Armenia
    {   10,  286,   25390,   25406,   25432, 16, 26, 23, }, // Atlantic
    {   10,  295,   25455,   25480,   25515, 25, 35, 32, }, // Australia_Central
    {   10,  313,   25547,   25584,   25631, 37, 47, 44, }, // Australia_CentralWestern
    {   10,  338,   25675,   25699,   25733, 24, 34, 31, }, // Australia_Eastern
    {   10,  356,   25764,   25792,   25830, 28, 38, 35, }, // Australia_Western
    {   10,  374,   25865,   25884,   25913, 19, 29, 26, }, // Azerbaijan
    {   10,  385,   25939,   25952,   25975, 13, 23, 20, }, // Azores
    {   10,  397,   25995,   26013,   26041, 18, 28, 25, }, // Bangladesh
    {   10,  415,       0,   26066,       0,  0, 13,  0, }, // Bhutan
    {   10,  422,       0,   26079,       0,  0, 14,  0, }, // Bolivia
    {   10,  437,   26093,   26108,   26133, 15, 25, 22, }, // Brasilia
    {   10,  454,       0,   26155,       0,  0, 24,  0, }, // Brunei
    {   10,  461,   26179,   26201,   26233, 22, 32, 29, }, // Cape_Verde
    {   10,  472,       0,   26262,       0,  0, 12,  0, }, // Casey
    {   10,  478,       0,   26274,       0,  0, 14,  0, }, // Chamorro
    {   10,  487,   26288,   26301,   26324, 13, 23, 20, }, // Chatham
    {   10,  495,   26344,   26355,   26376, 11, 21, 18, }, // Chile
    {   10,  501,   26394,   26405,   26426, 11, 21, 18, }, // China
    {   10,  507,       0,   26444,       0,  0, 32,  0, }, // Christmas
    {   10,  517,       0,   26476,       0,  0, 19,  0, }, // Cocos
    {   10,  523,   26495,   26510,   26535, 15, 25, 22, }, // Colombia
    {   10,  532,   26557,   26574,   26601, 17, 27, 24, }, // Cook
    {   10,  537,   26625,   26636,   26657, 11, 21, 18, }, // Cuba
    {   10,  548,       0,   26675,       0,  0, 14,  0, }, // Davis
    {   10,  564,       0,   26689,       0,  0, 22,  0, }, // DumontDUrville
    {   10,  601,       0,   26711,       0,  0, 20,  0, }, // East_Timor
    {   10,  612,   26731,   26756,   26791, 25, 35, 32, }, // Easter
    {   10,  619,       0,   26823,       0,  0, 15,  0, }, // Ecuador
    {   10,  627,   26838,   26860,   26892, 22, 32, 29, }, // Europe_Central
    {   10,  642,   26921,   26942,   26973, 21, 31, 28, }, // Europe_Eastern
    {   10,  657,       0,   27001,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {   10,  680,   27032,   27057,   27092, 25, 35, 32, }, // Europe_Western
    {   10,  695,   27124,   27146,   27178, 22, 32, 29, }, // Falkland
    {   10,  704,   27207,   27219,   27241, 12, 22, 19, }, // Fiji
    {   10,  709,       0,   27260,       0,  0, 23,  0, }, // French_Guiana
    {   10,  723,       0,   27283,       0,  0, 48,  0, }, // French_Southern
    {   10,  746,       0,   27331,       0,  0, 17,  0, }, // Galapagos
    {   10,  756,       0,   27348,       0,  0, 15,  0, }, // Gambier
    {   10,  764,   27363,   27379,   27405, 16, 26, 23, }, // Georgia
    {   10,  772,       0,   27428,       0,  0, 21,  0, }, // Gilbert_Islands
    {   10,  788,       0,   27449,       0,  0, 15,  0, }, // GMT
    {   10,  830,   27464,   27488,   27522, 24, 34, 31, }, // Greenland_Eastern
    {   10,  848,   27553,   27581,   27619, 28, 38, 35, }, // Greenland_Western
    {   10,  866,       0,   27654,       0,  0, 12,  0, }, // Guam
    {   10,  871,       0,   27666,       0,  0, 12,  0, }, // Gulf
    {   10,  876,       0,   27260,       0,  0, 14,  0, }, // Guyana
    {   10,  883,   27678,   27706,   27744, 28, 38, 35, }, // Hawaii_Aleutian
    {   10,  899,   27779,   27796,   27823, 17, 27, 24, }, // Hong_Kong
    {   10,  909,   27847,   27859,   27881, 12, 22, 19, }, // Hovd
    {   10,  914,       0,   27900,       0,  0, 22,  0, }, // India
    {   10,  920,       0,   27922,       0,  0, 20,  0, }, // Indian_Ocean
    {   10,  933,       0,   27942,       0,  0, 15,  0, }, // Indochina
    {   10,  943,       0,   27957,       0,  0, 25,  0, }, // Indonesia_Central
    {   10,  961,       0,   27982,       0,  0, 24,  0, }, // Indonesia_Eastern
    {   10,  979,       0,   28006,       0,  0, 28,  0, }, // Indonesia_Western
    {   10,  997,   28034,   28046,   28068, 12, 22, 19, }, // Iran
    {   10, 1008,   28087,   28102,   28127, 15, 25, 22, }, // Irkutsk
    {   10, 1016,   28149,   28163,   28187, 14, 24, 21, }, // Israel
    {   10, 1023,   28208,   28222,   28246, 14, 24, 21, }, // Japan
    {   10, 1029,   28267,   28297,   28337, 30, 40, 37, }, // Kamchatka
    {   10, 1047,       0,   28374,       0,  0, 18,  0, }, // Kazakhstan
    {   10, 1058,       0,   28392,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {   10, 1077,       0,   28417,       0,  0, 29,  0, }, // Kazakhstan_Western
    {   10, 1106,   28446,   28457,   28478, 11, 21, 18, }, // Korea
    {   10, 1112,       0,   28496,       0,  0, 12,  0, }, // Kosrae
    {   10, 1119,   28508,   28527,   28556, 19, 29, 26, }, // Krasnoyarsk
    {   10, 1151,       0,   28582,       0,  0, 15,  0, }, // Kyrgystan
    {   10, 1161,       0,   28597,       0,  0, 13,  0, }, // Lanka
    {   10, 1175,       0,   28610,       0,  0, 35,  0, }, // Line_Islands
    {   10, 1188,   28645,   28661,   28687, 16, 26, 23, }, // Lord_Howe
    {   10, 1198,   28710,   28722,   28744, 12, 22, 19, }, // Macau
    {   10, 1204,   28763,   28778,   28803, 15, 25, 22, }, // Magadan
    {   10, 1219,       0,   28825,       0,  0, 15,  0, }, // Malaysia
    {   10, 1228,       0,   28840,       0,  0, 15,  0, }, // Maldives
    {   10, 1237,       0,   28855,       0,  0, 21,  0, }, // Marquesas
    {   10, 1247,       0,   28876,       0,  0, 22,  0, }, // Marshall_Islands
    {   10, 1264,   28898,   28915,   28942, 17, 27, 24, }, // Mauritius
    {   10, 1274,       0,   28966,       0,  0, 14,  0, }, // Mawson
    {   10, 1281,   28980,   29030,   29090, 50, 60, 57, }, // Mexico_Pacific
    {   10, 1296,   29147,   29165,   29193, 18, 28, 25, }, // Mongolia
    {   10, 1305,   29218,   29230,   29252, 12, 22, 19, }, // Moscow
    {   10, 1312,       0,   29271,       0,  0, 15,  0, }, // Myanmar
    {   10, 1320,       0,   29286,       0,  0, 13,  0, }, // Nauru
    {   10, 1326,       0,   29299,       0,  0, 13,  0, }, // Nepal
    {   10, 1332,   29312,   29334,   29366, 22, 32, 29, }, // New_Caledonia
    {   10, 1346,   29395,   29415,   29445, 20, 30, 27, }, // New_Zealand
    {   10, 1358,   29472,   29506,   29550, 34, 44, 41, }, // Newfoundland
    {   10, 1371,       0,   29591,       0,  0, 12,  0, }, // Niue
    {   10, 1376,   29603,   29625,   29657, 22, 32, 29, }, // Norfolk
    {   10, 1384,   29686,   29712,   29748, 26, 36, 33, }, // Noronha
    {   10, 1392,       0,   29781,       0,  0, 35,  0, }, // North_Mariana
    {   10, 1406,   29816,   29835,   29864, 19, 29, 26, }, // Novosibirsk
    {   10, 1418,   29890,   29902,   29924, 12, 22, 19, }, // Omsk
    {   10, 1428,   29943,   29959,   29985, 16, 26, 23, }, // Pakistan
    {   10, 1437,       0,   30008,       0,  0, 12,  0, }, // Palau
    {   10, 1443,       0,   30020,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   10, 1460,   30045,   30060,   30085, 15, 25, 22, }, // Paraguay
    {   10, 1469,   30107,   30119,   30141, 12, 22, 19, }, // Peru
    {   10, 1474,   30160,   30176,   30202, 16, 26, 23, }, // Philippines
    {   10, 1486,       0,   30225,       0,  0, 20,  0, }, // Phoenix_Islands
    {   10, 1502,   30245,   30272,   30309, 27, 37, 34, }, // Pierre_Miquelon
    {   10, 1518,       0,   30343,       0,  0, 15,  0, }, // Pitcairn
    {   10, 1527,       0,   30358,       0,  0, 14,  0, }, // Ponape
    {   10, 1534,       0,   30372,       0,  0, 14,  0, }, // Pyongyang
    {   10, 1544,   30386,   30402,   30428, 16, 26, 23, }, // Qyzylorda
    {   10, 1554,       0,   30451,       0,  0, 15,  0, }, // Reunion
    {   10, 1562,       0,   30466,       0,  0, 14,  0, }, // Rothera
    {   10, 1570,   30480,   30495,   30520, 15, 25, 22, }, // Sakhalin
    {   10, 1579,   30542,   30555,   30578, 13, 23, 20, }, // Samara
    {   10, 1596,   30598,   30610,   30632, 12, 22, 19, }, // Samoa
    {   10, 1602,       0,   30651,       0,  0, 17,  0, }, // Seychelles
    {   10, 1624,       0,   30668,       0,  0, 16,  0, }, // Singapore
    {   10, 1634,       0,   30684,       0,  0, 21,  0, }, // Solomon
    {   10, 1642,       0,   30705,       0,  0, 23,  0, }, // South_Georgia
    {   10, 1656,       0,   30728,       0,  0, 15,  0, }, // Suriname
    {   10, 1676,       0,   30743,       0,  0, 12,  0, }, // Syowa
    {   10, 1682,       0,   30755,       0,  0, 13,  0, }, // Tahiti
    {   10, 1689,   30768,   30781,   30804, 13, 23, 20, }, // Taipei
    {   10, 1696,       0,   30824,       0,  0, 19,  0, }, // Tajikistan
    {   10, 1724,       0,   30843,       0,  0, 14,  0, }, // Tokelau
    {   10, 1732,   30857,   30869,   30891, 12, 22, 19, }, // Tonga
    {   10, 1738,       0,   30910,       0,  0, 11,  0, }, // Truk
    {   10, 1750,   30921,   30941,   30971, 20, 30, 27, }, // Turkmenistan
    {   10, 1763,       0,   30998,       0,  0, 14,  0, }, // Tuvalu
    {   10, 1777,   31012,   31026,   31050, 14, 24, 21, }, // Uruguay
    {   10, 1792,   31071,   31089,   31117, 18, 28, 25, }, // Uzbekistan
    {   10, 1803,   31142,   31157,   31182, 15, 25, 22, }, // Vanuatu
    {   10, 1811,       0,   31204,       0,  0, 16,  0, }, // Venezuela
    {   10, 1821,   31220,   31239,   31268, 19, 29, 26, }, // Vladivostok
    {   10, 1833,   31294,   31311,   31338, 17, 27, 24, }, // Volgograd
    {   10, 1843,       0,   31362,       0,  0, 14,  0, }, // Vostok
    {   10, 1850,       0,   31376,       0,  0, 19,  0, }, // Wake
    {   10, 1855,       0,   31395,       0,  0, 26,  0, }, // Wallis
    {   10, 1862,   31421,   31436,   31461, 15, 25, 22, }, // Yakutsk
    {   10, 1870,   31483,   31503,   31533, 20, 30, 27, }, // Yekaterinburg
    {   10, 1892,       0,   31560,       0,  0, 13,  0, }, // Yukon
    {   13,    5,       0,   31573,       0,  0, 12,  0, }, // Afghanistan Amharic/Ethiopic/Ethiopia
    {   13,   17,       0,   31585,       0,  0, 16,  0, }, // Africa_Central
    {   13,   32,       0,   31601,       0,  0, 14,  0, }, // Africa_Eastern
    {   13,   65,       0,   31615,       0,  0, 18,  0, }, // Africa_Southern
    {   13,   81,   31633,   31647,   31666, 14, 19, 19, }, // Africa_Western
    {   13,  107,   31685,   31700,   31721, 15, 21, 19, }, // Alaska
    {   13,  135,   31740,   31755,   31775, 15, 20, 19, }, // Amazon
    {   13,  142,   31794,   31820,   31851, 26, 31, 20, }, // America_Central
    {   13,  158,   31871,   31886,   31906, 15, 20, 19, }, // America_Eastern
    {   13,  174,   31925,   31940,   31960, 15, 20, 17, }, // America_Mountain
    {   13,  191,   31977,   31992,   32012, 15, 20, 19, }, // America_Pacific
    {   13,  207,   32031,   32047,   32063, 16, 16, 21, }, // Anadyr
    {   13,  214,   32084,   32092,   32105,  8, 13, 15, }, // Apia
    {   13,  232,   32120,   32129,   32143,  9, 14, 18, }, // Arabian
    {   13,  240,   32161,   32179,   32201, 18, 22, 21, }, // Argentina
    {   13,  250,   32222,   32245,   32273, 23, 28, 27, }, // Argentina_Western
    {   13,  268,   32300,   32310,   32325, 10, 15, 15, }, // Armenia
    {   13,  286,   32340,   32358,   32381, 18, 23, 21, }, // Atlantic
    {   13,  295,   32402,   32427,   32457, 25, 30, 28, }, // Australia_Central
    {   13,  313,   32485,   32515,   32550, 30, 35, 34, }, // Australia_CentralWestern
    {   13,  338,   32584,   32609,   32639, 25, 30, 28, }, // Australia_Eastern
    {   13,  356,   32667,   32691,   32720, 24, 29, 28, }, // Australia_Western
    {   13,  374,   32748,   32759,   32775, 11, 16, 16, }, // Azerbaijan
    {   13,  385,   32791,   32800,   32814,  9, 14, 14, }, // Azores
    {   13,  397,   32828,   32839,   32855, 11, 16, 16, }, // Bangladesh
    {   13,  415,       0,   32871,       0,  0,  8,  0, }, // Bhutan
    {   13,  422,       0,   32879,       0,  0,  9,  0, }, // Bolivia
    {   13,  437,   32888,   32904,   32925, 16, 21, 19, }, // Brasilia
    {   13,  454,       0,   32944,       0,  0, 15,  0, }, // Brunei
    {   13,  461,   32959,   32970,   32986, 11, 16, 16, }, // Cape_Verde
    {   13,  478,       0,   33002,       0,  0, 13,  0, }, // Chamorro
    {   13,  487,   33015,   33023,   33036,  8, 13, 17, }, // Chatham
    {   13,  495,   33053,   33060,   33072,  7, 12, 12, }, // Chile
    {   13,  501,   33084,   33092,   33105,  8, 13, 17, }, // China
    {   13,  507,       0,   33122,       0,  0, 11,  0, }, // Christmas
    {   13,  517,       0,   33133,       0,  0, 13,  0, }, // Cocos
    {   13,  523,   33146,   33156,   33171, 10, 15, 15, }, // Colombia
    {   13,  532,   33186,   33198,   33215, 12, 17, 21, }, // Cook
    {   13,  537,   33236,   33242,   33254,  6, 12, 16, }, // Cuba
    {   13,  548,       0,   33270,       0,  0,  8,  0, }, // Davis
    {   13,  564,       0,   33278,       0,  0, 14,  0, }, // DumontDUrville
    {   13,  601,       0,   33292,       0,  0, 13,  0, }, // East_Timor
    {   13,  612,   33305,   33318,   33336, 13, 18, 18, }, // Easter
    {   13,  619,       0,   33354,       0,  0,  9,  0, }, // Ecuador
    {   13,  627,   33363,   33379,   33400, 16, 21, 21, }, // Europe_Central
    {   13,  642,   33421,   33436,   33456, 15, 20, 20, }, // Europe_Eastern
    {   13,  657,       0,   33476,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {   13,  680,   33494,   33509,   33529, 15, 20, 20, }, // Europe_Western
    {   13,  695,   33549,   33565,   33586, 16, 21, 21, }, // Falkland
    {   13,  704,   33607,   33614,   33626,  7, 12, 12, }, // Fiji
    {   13,  709,       0,   33638,       0,  0, 14,  0, }, // French_Guiana
    {   13,  723,       0,   33652,       0,  0, 26,  0, }, // French_Southern
    {   13,  746,       0,   33678,       0,  0, 10,  0, }, // Galapagos
    {   13,  756,       0,   33688,       0,  0, 10,  0, }, // Gambier
    {   13,  764,   33698,   33708,   33723, 10, 15, 15, }, // Georgia
    {   13,  772,       0,   33738,       0,  0, 15,  0, }, // Gilbert_Islands
    {   13,  788,       0,   33753,       0,  0, 15,  0, }, // GMT
    {   13,  830,   33768,   33784,   33805, 16, 21, 21, }, // Greenland_Eastern
    {   13,  848,   33826,   33842,   33863, 16, 21, 21, }, // Greenland_Western
    {   13,  871,       0,   33884,       0,  0, 16,  0, }, // Gulf
    {   13,  876,       0,   33900,       0,  0,  8,  0, }, // Guyana
    {   13,  883,   33908,   33927,   33951, 19, 24, 23, }, // Hawaii_Aleutian
    {   13,  899,   33974,   33986,   34003, 12, 17, 17, }, // Hong_Kong
    {   13,  909,   34020,   34034,   34054, 14, 20, 18, }, // Hovd
    {   13,  914,       0,   34072,       0,  0, 13,  0, }, // India
    {   13,  920,       0,   34085,       0,  0, 14,  0, }, // Indian_Ocean
    {   13,  933,       0,   34099,       0,  0, 11,  0, }, // Indochina
    {   13,  943,       0,   34110,       0,  0, 18,  0, }, // Indonesia_Central
    {   13,  961,       0,   34128,       0,  0, 17,  0, }, // Indonesia_Eastern
    {   13,  979,       0,   34145,       0,  0, 17,  0, }, // Indonesia_Western
    {   13,  997,   34162,   34170,   34183,  8, 13, 17, }, // Iran
    {   13, 1008,   34200,   34217,   34240, 17, 23, 21, }, // Irkutsk
    {   13, 1016,   34261,   34271,   34286, 10, 15, 19, }, // Israel
    {   13, 1023,   34305,   34313,   34326,  8, 13, 17, }, // Japan
    {   13, 1029,   34343,   34359,   34387, 16, 28, 32, }, // Kamchatka
    {   13, 1047,       0,   34419,       0,  0, 10,  0, }, // Kazakhstan
    {   13, 1058,       0,   34429,       0,  0, 16,  0, }, // Kazakhstan_Eastern
    {   13, 1077,       0,   34445,       0,  0, 16,  0, }, // Kazakhstan_Western
    {   13, 1106,   34461,   34469,   34482,  8, 13, 17, }, // Korea
    {   13, 1112,       0,   34499,       0,  0,  9,  0, }, // Kosrae
    {   13, 1119,   34508,   34527,   34552, 19, 25, 23, }, // Krasnoyarsk
    {   13, 1151,       0,   34575,       0,  0, 11,  0, }, // Kyrgystan
    {   13, 1175,       0,   34586,       0,  0, 13,  0, }, // Line_Islands
    {   13, 1188,   34599,   34617,   34640, 18, 23, 21, }, // Lord_Howe
    {   13, 1204,   34661,   34677,   34697, 16, 20, 18, }, // Magadan
    {   13, 1219,       0,   34715,       0,  0, 10,  0, }, // Malaysia
    {   13, 1228,       0,   34725,       0,  0, 10,  0, }, // Maldives
    {   13, 1237,       0,   34735,       0,  0, 10,  0, }, // Marquesas
    {   13, 1247,       0,   34745,       0,  0, 14,  0, }, // Marshall_Islands
    {   13, 1264,   34759,   34770,   34786, 11, 16, 16, }, // Mauritius
    {   13, 1274,       0,   34802,       0,  0,  9,  0, }, // Mawson
    {   13, 1281,   34811,   34831,   34856, 20, 25, 24, }, // Mexico_Pacific
    {   13, 1296,   34880,   34891,   34914, 11, 23, 22, }, // Mongolia
    {   13, 1305,   34936,   34950,   34969, 14, 19, 18, }, // Moscow
    {   13, 1312,       0,   34987,       0,  0, 10,  0, }, // Myanmar
    {   13, 1320,       0,   34997,       0,  0,  8,  0, }, // Nauru
    {   13, 1326,       0,   35005,       0,  0,  8,  0, }, // Nepal
    {   13, 1332,   35013,   35026,   35044, 13, 18, 18, }, // New_Caledonia
    {   13, 1346,   35062,   35074,   35091, 12, 17, 21, }, // New_Zealand
    {   13, 1358,   35112,   35133,   35159, 21, 26, 25, }, // Newfoundland
    {   13, 1371,       0,   35184,       0,  0,  8,  0, }, // Niue
    {   13, 1376,   35192,   35207,   35234, 15, 27, 26, }, // Norfolk
    {   13, 1384,   35260,   35275,   35303, 15, 28, 28, }, // Noronha
    {   13, 1406,   35331,   35350,   35374, 19, 24, 22, }, // Novosibirsk
    {   13, 1418,   35396,   35412,   35432, 16, 20, 19, }, // Omsk
    {   13, 1428,   35451,   35461,   35476, 10, 15, 15, }, // Pakistan
    {   13, 1437,       0,   35491,       0,  0,  8,  0, }, // Palau
    {   13, 1443,       0,   35499,       0,  0, 13,  0, }, // Papua_New_Guinea
    {   13, 1460,   35512,   35521,   35535,  9, 14, 14, }, // Paraguay
    {   13, 1469,   35549,   35556,   35568,  7, 12, 12, }, // Peru
    {   13, 1474,   35580,   35589,   35603,  9, 14, 14, }, // Philippines
    {   13, 1486,       0,   35617,       0,  0, 14,  0, }, // Phoenix_Islands
    {   13, 1502,   35631,   35651,   35676, 20, 25, 29, }, // Pierre_Miquelon
    {   13, 1518,       0,   35705,       0,  0, 11,  0, }, // Pitcairn
    {   13, 1527,       0,   35716,       0,  0,  8,  0, }, // Ponape
    {   13, 1534,       0,   35724,       0,  0, 12,  0, }, // Pyongyang
    {   13, 1554,       0,   35736,       0,  0, 10,  0, }, // Reunion
    {   13, 1562,       0,   35746,       0,  0,  8,  0, }, // Rothera
    {   13, 1570,   35754,   35770,   35791, 16, 21, 20, }, // Sakhalin
    {   13, 1579,   35811,   35825,   35844, 14, 19, 18, }, // Samara
    {   13, 1596,   35862,   35870,   35883,  8, 13, 12, }, // Samoa
    {   13, 1602,       0,   35895,       0,  0,  9,  0, }, // Seychelles
    {   13, 1624,       0,   35904,       0,  0, 15,  0, }, // Singapore
    {   13, 1634,       0,   35919,       0,  0, 14,  0, }, // Solomon
    {   13, 1642,       0,   35933,       0,  0, 14,  0, }, // South_Georgia
    {   13, 1656,       0,   35947,       0,  0,  9,  0, }, // Suriname
    {   13, 1676,       0,   35956,       0,  0,  8,  0, }, // Syowa
    {   13, 1682,       0,   35964,       0,  0,  8,  0, }, // Tahiti
    {   13, 1689,   35972,   35981,   35995,  9, 14, 18, }, // Taipei
    {   13, 1696,       0,   36013,       0,  0, 11,  0, }, // Tajikistan
    {   13, 1724,       0,   36024,       0,  0,  9,  0, }, // Tokelau
    {   13, 1732,   36033,   36041,   36054,  8, 13, 13, }, // Tonga
    {   13, 1738,       0,   36067,       0,  0,  7,  0, }, // Truk
    {   13, 1750,   36074,   36087,   36105, 13, 18, 18, }, // Turkmenistan
    {   13, 1763,       0,   36123,       0,  0,  8,  0, }, // Tuvalu
    {   13, 1777,   36131,   36140,   36154,  9, 14, 14, }, // Uruguay
    {   13, 1792,   36168,   36180,   36197, 12, 17, 17, }, // Uzbekistan
    {   13, 1803,   36214,   36222,   36235,  8, 13, 13, }, // Vanuatu
    {   13, 1811,       0,   36248,       0,  0, 10,  0, }, // Venezuela
    {   13, 1821,   36258,   36277,   36301, 19, 24, 23, }, // Vladivostok
    {   13, 1833,   36324,   36341,   36362, 17, 21, 20, }, // Volgograd
    {   13, 1843,       0,   36382,       0,  0,  9,  0, }, // Vostok
    {   13, 1850,       0,   36391,       0,  0, 11,  0, }, // Wake
    {   13, 1855,       0,   36402,       0,  0, 15,  0, }, // Wallis
    {   13, 1862,   36417,   36433,   36453, 16, 20, 20, }, // Yakutsk
    {   13, 1870,   36473,   36492,   36516, 19, 24, 23, }, // Yekaterinburg
    {   13, 1892,       0,   36539,       0,  0,  7,  0, }, // Yukon
    {   14,    5,       0,   36546,       0,  0, 15,  0, }, // Afghanistan Arabic/Arabic/Egypt
    {   14,   17,       0,   36561,       0,  0, 17,  0, }, // Africa_Central
    {   14,   32,       0,   36578,       0,  0, 17,  0, }, // Africa_Eastern
    {   14,   65,       0,   36595,       0,  0, 18,  0, }, // Africa_Southern
    {   14,   81,   36613,   36630,   36654, 17, 24, 24, }, // Africa_Western
    {   14,  107,   36678,   36690,   36712, 12, 22, 19, }, // Alaska
    {   14,  135,   36731,   36745,   36766, 14, 21, 21, }, // Amazon
    {   14,  142,   36787,   36819,   36858, 32, 39, 39, }, // America_Central
    {   14,  158,   36897,   36928,   36966, 31, 38, 38, }, // America_Eastern
    {   14,  174,   37004,   37035,   37073, 31, 38, 38, }, // America_Mountain
    {   14,  191,   37111,   37130,   37156, 19, 26, 26, }, // America_Pacific
    {   14,  207,   37182,   37194,   37213, 12, 19, 22, }, // Anadyr
    {   14,  214,   37235,   37245,   37265, 10, 20, 20, }, // Apia
    {   14,  232,   37285,   37299,   37320, 14, 21, 21, }, // Arabian
    {   14,  240,   37341,   37356,   37378, 15, 22, 22, }, // Argentina
    {   14,  250,   37400,   37419,   37445, 19, 26, 26, }, // Argentina_Western
    {   14,  268,   37471,   37484,   37504, 13, 20, 20, }, // Armenia
    {   14,  286,   37524,   37537,   37559, 13, 22, 22, }, // Atlantic
    {   14,  295,   37581,   37599,   37624, 18, 25, 25, }, // Australia_Central
    {   14,  313,   37649,   37671,   37700, 22, 29, 29, }, // Australia_CentralWestern
    {   14,  338,   37729,   37747,   37772, 18, 25, 25, }, // Australia_Eastern
    {   14,  356,   37797,   37815,   37840, 18, 25, 25, }, // Australia_Western
    {   14,  374,   37865,   37879,   37900, 14, 21, 21, }, // Azerbaijan
    {   14,  385,   37921,   37932,   37950, 11, 18, 18, }, // Azores
    {   14,  397,   37968,   37982,   38003, 14, 21, 21, }, // Bangladesh
    {   14,  415,       0,   38024,       0,  0, 11,  0, }, // Bhutan
    {   14,  422,       0,   38035,       0,  0, 13,  0, }, // Bolivia
    {   14,  437,   38048,   38062,   38083, 14, 21, 21, }, // Brasilia
    {   14,  454,       0,   38104,       0,  0, 12,  0, }, // Brunei
    {   14,  461,   38116,   38134,   38159, 18, 25, 25, }, // Cape_Verde
    {   14,  478,       0,   38184,       0,  0, 13,  0, }, // Chamorro
    {   14,  487,   38197,   38209,   38228, 12, 19, 19, }, // Chatham
    {   14,  495,   38247,   38258,   38276, 11, 18, 18, }, // Chile
    {   14,  501,   38294,   38305,   38323, 11, 18, 18, }, // China
    {   14,  507,       0,   38341,       0,  0, 19,  0, }, // Christmas
    {   14,  517,       0,   38360,       0,  0, 15,  0, }, // Cocos
    {   14,  523,   38375,   38389,   38410, 14, 21, 21, }, // Colombia
    {   14,  532,   38431,   38445,   38465, 14, 20, 20, }, // Cook
    {   14,  537,   38485,   38495,   38512, 10, 17, 17, }, // Cuba
    {   14,  548,       0,   38529,       0,  0, 11,  0, }, // Davis
    {   14,  564,       0,   38540,       0,  0, 22,  0, }, // DumontDUrville
    {   14,  601,       0,   38562,       0,  0, 19,  0, }, // East_Timor
    {   14,  612,   38581,   38597,   38620, 16, 23, 23, }, // Easter
    {   14,  619,       0,   38643,       0,  0, 15,  0, }, // Ecuador
    {   14,  627,   38658,   38674,   38697, 16, 23, 23, }, // Europe_Central
    {   14,  642,   38720,   38736,   38759, 16, 23, 23, }, // Europe_Eastern
    {   14,  657,       0,   38782,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {   14,  680,   38811,   38827,   38850, 16, 23, 23, }, // Europe_Western
    {   14,  695,   38873,   38890,   38914, 17, 24, 24, }, // Falkland
    {   14,  704,   38938,   38948,   38965, 10, 17, 17, }, // Fiji
    {   14,  709,       0,   38982,       0,  0, 21,  0, }, // French_Guiana
    {   14,  723,       0,   39003,       0,  0, 47,  0, }, // French_Southern
    {   14,  746,       0,   39050,       0,  0, 14,  0, }, // Galapagos
    {   14,  756,       0,   39064,       0,  0, 12,  0, }, // Gambier
    {   14,  764,   39076,   39088,   39107, 12, 19, 19, }, // Georgia
    {   14,  772,       0,   39126,       0,  0, 16,  0, }, // Gilbert_Islands
    {   14,  788,       0,   39142,       0,  0, 12,  0, }, // GMT
    {   14,  830,   39154,   39172,   39197, 18, 25, 25, }, // Greenland_Eastern
    {   14,  848,   39222,   39240,   39265, 18, 25, 25, }, // Greenland_Western
    {   14,  866,       0,   39290,       0,  0, 10,  0, }, // Guam
    {   14,  871,       0,   39300,       0,  0, 12,  0, }, // Gulf
    {   14,  876,       0,   39312,       0,  0, 11,  0, }, // Guyana
    {   14,  883,   39323,   39342,   39368, 19, 26, 26, }, // Hawaii_Aleutian
    {   14,  899,   39394,   39409,   39431, 15, 22, 22, }, // Hong_Kong
    {   14,  909,   39453,   39463,   39480, 10, 17, 17, }, // Hovd
    {   14,  914,       0,   39497,       0,  0, 11,  0, }, // India
    {   14,  920,       0,   39508,       0,  0, 19,  0, }, // Indian_Ocean
    {   14,  933,       0,   39527,       0,  0, 19,  0, }, // Indochina
    {   14,  943,       0,   39546,       0,  0, 19,  0, }, // Indonesia_Central
    {   14,  961,       0,   39565,       0,  0, 19,  0, }, // Indonesia_Eastern
    {   14,  979,       0,   39584,       0,  0, 19,  0, }, // Indonesia_Western
    {   14,  997,   39603,   39614,   39632, 11, 18, 18, }, // Iran
    {   14, 1008,   39650,   39663,   39683, 13, 20, 20, }, // Irkutsk
    {   14, 1016,   39703,   39716,   39736, 13, 20, 20, }, // Israel
    {   14, 1023,   39756,   39769,   39789, 13, 20, 20, }, // Japan
    {   14, 1029,   39809,   39823,   39853, 14, 30, 37, }, // Kamchatka
    {   14, 1047,       0,   39890,       0,  0, 15,  0, }, // Kazakhstan
    {   14, 1058,       0,   39905,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {   14, 1077,       0,   39924,       0,  0, 19,  0, }, // Kazakhstan_Western
    {   14, 1106,   39943,   39954,   39972, 11, 18, 18, }, // Korea
    {   14, 1112,       0,   39990,       0,  0, 11,  0, }, // Kosrae
    {   14, 1119,   40001,   40018,   40042, 17, 24, 27, }, // Krasnoyarsk
    {   14, 1151,       0,   40069,       0,  0, 16,  0, }, // Kyrgystan
    {   14, 1175,       0,   40085,       0,  0, 14,  0, }, // Line_Islands
    {   14, 1188,   40099,   40113,   40134, 14, 21, 24, }, // Lord_Howe
    {   14, 1204,   40158,   40171,   40191, 13, 20, 20, }, // Magadan
    {   14, 1219,       0,   40211,       0,  0, 13,  0, }, // Malaysia
    {   14, 1228,       0,   40224,       0,  0, 18,  0, }, // Maldives
    {   14, 1237,       0,   40242,       0,  0, 14,  0, }, // Marquesas
    {   14, 1247,       0,   40256,       0,  0, 16,  0, }, // Marshall_Islands
    {   14, 1264,   40272,   40286,   40307, 14, 21, 21, }, // Mauritius
    {   14, 1274,       0,   40328,       0,  0, 12,  0, }, // Mawson
    {   14, 1281,   40340,   40367,   40401, 27, 34, 34, }, // Mexico_Pacific
    {   14, 1296,   40435,   40452,   40476, 17, 24, 24, }, // Mongolia
    {   14, 1305,   40500,   40511,   40529, 11, 18, 18, }, // Moscow
    {   14, 1312,       0,   40547,       0,  0, 13,  0, }, // Myanmar
    {   14, 1320,       0,   40560,       0,  0, 11,  0, }, // Nauru
    {   14, 1326,       0,   40571,       0,  0, 11,  0, }, // Nepal
    {   14, 1332,   40582,   40605,   40635, 23, 30, 30, }, // New_Caledonia
    {   14, 1346,   40665,   40680,   40702, 15, 22, 22, }, // New_Zealand
    {   14, 1358,   40724,   40742,   40767, 18, 25, 25, }, // Newfoundland
    {   14, 1371,       0,   40792,       0,  0, 10,  0, }, // Niue
    {   14, 1376,   40802,   40821,   40847, 19, 26, 26, }, // Norfolk
    {   14, 1384,   40873,   40898,   40929, 25, 31, 31, }, // Noronha
    {   14, 1392,       0,   40960,       0,  0, 26,  0, }, // North_Mariana
    {   14, 1406,   40986,   41003,   41027, 17, 24, 24, }, // Novosibirsk
    {   14, 1418,   41051,   41062,   41080, 11, 18, 18, }, // Omsk
    {   14, 1428,   41098,   41111,   41131, 13, 20, 20, }, // Pakistan
    {   14, 1437,       0,   41151,       0,  0, 11,  0, }, // Palau
    {   14, 1443,       0,   41162,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   14, 1460,   41187,   41201,   41222, 14, 21, 21, }, // Paraguay
    {   14, 1469,   41243,   41253,   41270, 10, 17, 17, }, // Peru
    {   14, 1474,   41287,   41301,   41322, 14, 21, 21, }, // Philippines
    {   14, 1486,       0,   41343,       0,  0, 15,  0, }, // Phoenix_Islands
    {   14, 1502,   41358,   41382,   41413, 24, 31, 31, }, // Pierre_Miquelon
    {   14, 1518,       0,   41444,       0,  0, 13,  0, }, // Pitcairn
    {   14, 1527,       0,   41457,       0,  0, 12,  0, }, // Ponape
    {   14, 1534,       0,   41469,       0,  0, 16,  0, }, // Pyongyang
    {   14, 1554,       0,   41485,       0,  0, 13,  0, }, // Reunion
    {   14, 1562,       0,   41498,       0,  0, 12,  0, }, // Rothera
    {   14, 1570,   41510,   41523,   41543, 13, 20, 20, }, // Sakhalin
    {   14, 1579,   41563,   41575,   41586, 12, 11, 18, }, // Samara
    {   14, 1596,   41604,   41615,   41633, 11, 18, 18, }, // Samoa
    {   14, 1602,       0,   41651,       0,  0, 10,  0, }, // Seychelles
    {   14, 1624,       0,   41661,       0,  0, 14,  0, }, // Singapore
    {   14, 1634,       0,   41675,       0,  0, 16,  0, }, // Solomon
    {   14, 1642,       0,   41691,       0,  0, 17,  0, }, // South_Georgia
    {   14, 1656,       0,   41708,       0,  0, 13,  0, }, // Suriname
    {   14, 1676,       0,   41721,       0,  0, 12,  0, }, // Syowa
    {   14, 1682,       0,   41733,       0,  0, 12,  0, }, // Tahiti
    {   14, 1689,   41745,   41757,   41776, 12, 19, 19, }, // Taipei
    {   14, 1696,       0,   41795,       0,  0, 14,  0, }, // Tajikistan
    {   14, 1724,       0,   41809,       0,  0, 13,  0, }, // Tokelau
    {   14, 1732,   41822,   41833,   41851, 11, 18, 18, }, // Tonga
    {   14, 1738,       0,   41869,       0,  0,  9,  0, }, // Truk
    {   14, 1750,   41878,   41894,   41917, 16, 23, 23, }, // Turkmenistan
    {   14, 1763,       0,   41940,       0,  0, 12,  0, }, // Tuvalu
    {   14, 1777,   41952,   41966,   41987, 14, 21, 21, }, // Uruguay
    {   14, 1792,   42008,   42023,   42045, 15, 22, 22, }, // Uzbekistan
    {   14, 1803,   42067,   42080,   42100, 13, 20, 20, }, // Vanuatu
    {   14, 1811,       0,   42120,       0,  0, 13,  0, }, // Venezuela
    {   14, 1821,   42133,   42150,   42174, 17, 24, 24, }, // Vladivostok
    {   14, 1833,   42198,   42213,   42235, 15, 22, 22, }, // Volgograd
    {   14, 1843,       0,   42257,       0,  0, 12,  0, }, // Vostok
    {   14, 1850,       0,   42269,       0,  0, 15,  0, }, // Wake
    {   14, 1855,       0,   42284,       0,  0, 20,  0, }, // Wallis
    {   14, 1862,   42304,   42317,   42337, 13, 20, 20, }, // Yakutsk
    {   14, 1870,   42357,   42374,   42398, 17, 24, 24, }, // Yekaterinburg
    {   14, 1892,       0,   42422,       0,  0, 11,  0, }, // Yukon
    {   42,  142,   42433,   42465,   42506, 32, 41, 42, }, // America_Central Aragonese/Latin/Spain
    {   42,  158,   42548,   42581,   42623, 33, 42, 43, }, // America_Eastern
    {   42,  174,   42666,   42702,   42747, 36, 45, 46, }, // America_Mountain
    {   42,  191,   42793,   42830,   42876, 37, 46, 47, }, // America_Pacific
    {   42,  286,   42923,   42942,   42970, 19, 28, 29, }, // Atlantic
    {   42,  627,   42999,   43023,   43056, 24, 33, 34, }, // Europe_Central
    {   42,  642,   43090,   43113,   43145, 23, 32, 33, }, // Europe_Eastern
    {   42,  680,   43178,   43202,   43235, 24, 33, 34, }, // Europe_Western
    {   42,  788,       0,   43269,       0,  0, 31,  0, }, // GMT
    {   43,    5,       0,   43300,       0,  0, 18,  0, }, // Afghanistan Armenian/Armenian/Armenia
    {   43,   17,       0,   43318,       0,  0, 28,  0, }, // Africa_Central
    {   43,   32,       0,   43346,       0,  0, 25,  0, }, // Africa_Eastern
    {   43,   65,       0,   43371,       0,  0, 26,  0, }, // Africa_Southern
    {   43,   81,   43397,   43422,   43456, 25, 34, 34, }, // Africa_Western
    {   43,  107,   43490,   43507,   43533, 17, 26, 26, }, // Alaska
    {   43,  135,   43559,   43576,   43602, 17, 26, 26, }, // Amazon
    {   43,  142,   43628,   43657,   43695, 29, 38, 38, }, // America_Central
    {   43,  158,   43733,   43759,   43794, 26, 35, 35, }, // America_Eastern
    {   43,  174,   43829,   43851,   43882, 22, 31, 31, }, // America_Mountain
    {   43,  191,   43913,   43937,   43970, 24, 33, 33, }, // America_Pacific
    {   43,  214,   44003,   44017,   44040, 14, 23, 23, }, // Apia
    {   43,  232,   44063,   44088,   44122, 25, 34, 34, }, // Arabian
    {   43,  240,   44156,   44175,   44203, 19, 28, 28, }, // Argentina
    {   43,  250,   44231,   44259,   44296, 28, 37, 37, }, // Argentina_Western
    {   43,  268,   44333,   44350,   44376, 17, 26, 26, }, // Armenia
    {   43,  286,   44402,   44419,   44445, 17, 26, 26, }, // Atlantic
    {   43,  295,   44471,   44502,   44542, 31, 40, 40, }, // Australia_Central
    {   43,  313,   44582,   44622,   44671, 40, 49, 49, }, // Australia_CentralWestern
    {   43,  338,   44720,   44748,   44785, 28, 37, 37, }, // Australia_Eastern
    {   43,  356,   44822,   44850,   44887, 28, 37, 37, }, // Australia_Western
    {   43,  374,   44924,   44941,   44967, 17, 26, 26, }, // Azerbaijan
    {   43,  385,   44993,   45017,   45050, 24, 33, 33, }, // Azores
    {   43,  397,   45083,   45101,   45128, 18, 27, 27, }, // Bangladesh
    {   43,  415,       0,   45155,       0,  0, 15,  0, }, // Bhutan
    {   43,  422,       0,   45170,       0,  0, 17,  0, }, // Bolivia
    {   43,  437,   45187,   45205,   45232, 18, 27, 27, }, // Brasilia
    {   43,  454,       0,   45259,       0,  0, 16,  0, }, // Brunei
    {   43,  461,   45275,   45294,   45322, 19, 28, 28, }, // Cape_Verde
    {   43,  478,       0,   45350,       0,  0, 16,  0, }, // Chamorro
    {   43,  487,   45366,   45385,   45413, 19, 28, 28, }, // Chatham
    {   43,  495,   45441,   45454,   45476, 13, 22, 22, }, // Chile
    {   43,  501,   45498,   45515,   45541, 17, 26, 26, }, // China
    {   43,  507,       0,   45567,       0,  0, 27,  0, }, // Christmas
    {   43,  517,       0,   45594,       0,  0, 25,  0, }, // Cocos
    {   43,  523,   45619,   45638,   45666, 19, 28, 28, }, // Colombia
    {   43,  532,   45694,   45716,   45747, 22, 31, 35, }, // Cook
    {   43,  537,   45782,   45797,   45821, 15, 24, 24, }, // Cuba
    {   43,  548,       0,   45845,       0,  0, 15,  0, }, // Davis
    {   43,  564,       0,   45860,       0,  0, 26,  0, }, // DumontDUrville
    {   43,  601,       0,   45886,       0,  0, 23,  0, }, // East_Timor
    {   43,  612,   45909,   45928,   45956, 19, 28, 28, }, // Easter
    {   43,  619,       0,   45984,       0,  0, 16,  0, }, // Ecuador
    {   43,  627,   46000,   46028,   46065, 28, 37, 37, }, // Europe_Central
    {   43,  642,   46102,   46127,   46161, 25, 34, 34, }, // Europe_Eastern
    {   43,  657,       0,   46195,       0,  0, 14,  0, }, // Europe_Further_Eastern
    {   43,  680,   46209,   46234,   46268, 25, 34, 34, }, // Europe_Western
    {   43,  695,   46302,   46330,   46367, 28, 37, 37, }, // Falkland
    {   43,  704,   46404,   46417,   46439, 13, 22, 22, }, // Fiji
    {   43,  709,       0,   46461,       0,  0, 27,  0, }, // French_Guiana
    {   43,  723,       0,   46488,       0,  0, 43,  0, }, // French_Southern
    {   43,  746,       0,   46531,       0,  0, 29,  0, }, // Galapagos
    {   43,  756,       0,   46560,       0,  0, 23,  0, }, // Gambier
    {   43,  764,   46583,   46599,   46624, 16, 25, 25, }, // Georgia
    {   43,  772,       0,   46649,       0,  0, 25,  0, }, // Gilbert_Islands
    {   43,  788,       0,   46674,       0,  0, 16,  0, }, // GMT
    {   43,  830,   46690,   46719,   46757, 29, 38, 38, }, // Greenland_Eastern
    {   43,  848,   46795,   46824,   46862, 29, 38, 38, }, // Greenland_Western
    {   43,  871,       0,   46900,       0,  0, 28,  0, }, // Gulf
    {   43,  876,       0,   46928,       0,  0, 16,  0, }, // Guyana
    {   43,  883,   46944,   46969,   47003, 25, 34, 34, }, // Hawaii_Aleutian
    {   43,  899,   47037,   47053,   47078, 16, 25, 25, }, // Hong_Kong
    {   43,  909,   47103,   47116,   47138, 13, 22, 22, }, // Hovd
    {   43,  914,       0,   47160,       0,  0, 27,  0, }, // India
    {   43,  920,       0,   47187,       0,  0, 26,  0, }, // Indian_Ocean
    {   43,  933,       0,   47213,       0,  0, 20,  0, }, // Indochina
    {   43,  943,       0,   47233,       0,  0, 31,  0, }, // Indonesia_Central
    {   43,  961,       0,   47264,       0,  0, 28,  0, }, // Indonesia_Eastern
    {   43,  979,       0,   47292,       0,  0, 28,  0, }, // Indonesia_Western
    {   43,  997,   47320,   47333,   47355, 13, 22, 22, }, // Iran
    {   43, 1008,   47377,   47394,   47420, 17, 26, 26, }, // Irkutsk
    {   43, 1016,   47446,   47462,   47487, 16, 25, 25, }, // Israel
    {   43, 1023,   47512,   47529,   47555, 17, 26, 26, }, // Japan
    {   43, 1047,       0,   47581,       0,  0, 18,  0, }, // Kazakhstan
    {   43, 1058,       0,   47599,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {   43, 1077,       0,   47626,       0,  0, 27,  0, }, // Kazakhstan_Western
    {   43, 1106,   47653,   47668,   47692, 15, 24, 24, }, // Korea
    {   43, 1112,       0,   47716,       0,  0, 16,  0, }, // Kosrae
    {   43, 1119,   47732,   47752,   47781, 20, 29, 29, }, // Krasnoyarsk
    {   43, 1151,       0,   47810,       0,  0, 17,  0, }, // Kyrgystan
    {   43, 1175,       0,   47827,       0,  0, 21,  0, }, // Line_Islands
    {   43, 1188,   47848,   47866,   47893, 18, 27, 27, }, // Lord_Howe
    {   43, 1204,   47920,   47936,   47961, 16, 25, 25, }, // Magadan
    {   43, 1219,       0,   47986,       0,  0, 18,  0, }, // Malaysia
    {   43, 1228,       0,   48004,       0,  0, 18,  0, }, // Maldives
    {   43, 1237,       0,   48022,       0,  0, 26,  0, }, // Marquesas
    {   43, 1247,       0,   48048,       0,  0, 26,  0, }, // Marshall_Islands
    {   43, 1264,   48074,   48092,   48119, 18, 27, 27, }, // Mauritius
    {   43, 1274,       0,   48146,       0,  0, 16,  0, }, // Mawson
    {   43, 1281,   48162,   48196,   48239, 34, 43, 43, }, // Mexico_Pacific
    {   43, 1296,   48282,   48302,   48331, 20, 29, 29, }, // Mongolia
    {   43, 1305,   48360,   48376,   48401, 16, 25, 25, }, // Moscow
    {   43, 1312,       0,   48426,       0,  0, 16,  0, }, // Myanmar
    {   43, 1320,       0,   48442,       0,  0, 16,  0, }, // Nauru
    {   43, 1326,       0,   48458,       0,  0, 14,  0, }, // Nepal
    {   43, 1332,   48472,   48495,   48527, 23, 32, 32, }, // New_Caledonia
    {   43, 1346,   48559,   48581,   48612, 22, 31, 31, }, // New_Zealand
    {   43, 1358,   48643,   48666,   48698, 23, 32, 32, }, // Newfoundland
    {   43, 1371,       0,   48730,       0,  0, 15,  0, }, // Niue
    {   43, 1376,   48745,   48766,   48796, 21, 30, 30, }, // Norfolk
    {   43, 1384,   48826,   48856,   48895, 30, 39, 39, }, // Noronha
    {   43, 1406,   48934,   48954,   48983, 20, 29, 29, }, // Novosibirsk
    {   43, 1418,   49012,   49025,   49047, 13, 22, 22, }, // Omsk
    {   43, 1428,   49069,   49086,   49112, 17, 26, 26, }, // Pakistan
    {   43, 1437,       0,   49138,       0,  0, 16,  0, }, // Palau
    {   43, 1443,       0,   49154,       0,  0, 27,  0, }, // Papua_New_Guinea
    {   43, 1460,   49181,   49198,   49224, 17, 26, 26, }, // Paraguay
    {   43, 1469,   49250,   49264,   49287, 14, 23, 23, }, // Peru
    {   43, 1474,   49310,   49329,   49357, 19, 28, 28, }, // Philippines
    {   43, 1486,       0,   49385,       0,  0, 23,  0, }, // Phoenix_Islands
    {   43, 1502,   49408,   49435,   49471, 27, 36, 36, }, // Pierre_Miquelon
    {   43, 1518,       0,   49507,       0,  0, 16,  0, }, // Pitcairn
    {   43, 1527,       0,   49523,       0,  0, 20,  0, }, // Ponape
    {   43, 1534,       0,   49543,       0,  0, 16,  0, }, // Pyongyang
    {   43, 1554,       0,   49559,       0,  0, 18,  0, }, // Reunion
    {   43, 1562,       0,   49577,       0,  0, 16,  0, }, // Rothera
    {   43, 1570,   49593,   49609,   49634, 16, 25, 25, }, // Sakhalin
    {   43, 1596,   49659,   49674,   49698, 15, 24, 24, }, // Samoa
    {   43, 1602,       0,   49722,       0,  0, 26,  0, }, // Seychelles
    {   43, 1624,       0,   49748,       0,  0, 18,  0, }, // Singapore
    {   43, 1634,       0,   49766,       0,  0, 25,  0, }, // Solomon
    {   43, 1642,       0,   49791,       0,  0, 26,  0, }, // South_Georgia
    {   43, 1656,       0,   49817,       0,  0, 17,  0, }, // Suriname
    {   43, 1676,       0,   49834,       0,  0, 15,  0, }, // Syowa
    {   43, 1682,       0,   49849,       0,  0, 14,  0, }, // Tahiti
    {   43, 1689,   49863,   49878,   49902, 15, 24, 24, }, // Taipei
    {   43, 1696,       0,   49926,       0,  0, 18,  0, }, // Tajikistan
    {   43, 1724,       0,   49944,       0,  0, 18,  0, }, // Tokelau
    {   43, 1732,   49962,   49977,   50001, 15, 24, 24, }, // Tonga
    {   43, 1738,       0,   50025,       0,  0, 14,  0, }, // Truk
    {   43, 1750,   50039,   50060,   50090, 21, 30, 30, }, // Turkmenistan
    {   43, 1763,       0,   50120,       0,  0, 18,  0, }, // Tuvalu
    {   43, 1777,   50138,   50156,   50183, 18, 27, 27, }, // Uruguay
    {   43, 1792,   50210,   50229,   50257, 19, 28, 28, }, // Uzbekistan
    {   43, 1803,   50285,   50304,   50332, 19, 28, 28, }, // Vanuatu
    {   43, 1811,       0,   50360,       0,  0, 20,  0, }, // Venezuela
    {   43, 1821,   50380,   50400,   50429, 20, 29, 29, }, // Vladivostok
    {   43, 1833,   50458,   50476,   50503, 18, 27, 27, }, // Volgograd
    {   43, 1843,       0,   50530,       0,  0, 15,  0, }, // Vostok
    {   43, 1850,       0,   50545,       0,  0, 19,  0, }, // Wake
    {   43, 1855,       0,   50564,       0,  0, 27,  0, }, // Wallis
    {   43, 1862,   50591,   50608,   50634, 17, 26, 26, }, // Yakutsk
    {   43, 1870,   50660,   50682,   50713, 22, 31, 31, }, // Yekaterinburg
    {   43, 1892,       0,   50744,       0,  0, 15,  0, }, // Yukon
    {   44,    5,       0,   50759,       0,  0, 17,  0, }, // Afghanistan Assamese/Bangla/India
    {   44,   17,       0,   50776,       0,  0, 18,  0, }, // Africa_Central
    {   44,   32,       0,   50794,       0,  0, 17,  0, }, // Africa_Eastern
    {   44,   65,       0,   50811,       0,  0, 24,  0, }, // Africa_Southern
    {   44,   81,   50835,   50855,   50879, 20, 24, 33, }, // Africa_Western
    {   44,  107,   50912,   50925,   50942, 13, 17, 20, }, // Alaska
    {   44,  135,   50962,   50973,   50988, 11, 15, 24, }, // Amazon
    {   44,  142,   51012,   51042,   51076, 30, 34, 37, }, // America_Central
    {   44,  158,   51113,   51140,   51171, 27, 31, 34, }, // America_Eastern
    {   44,  174,   51205,   51235,   51269, 30, 34, 37, }, // America_Mountain
    {   44,  191,   51306,   51334,   51366, 28, 32, 26, }, // America_Pacific
    {   44,  214,   51392,   51404,   51420, 12, 16, 19, }, // Apia
    {   44,  232,   51439,   51450,   51465, 11, 15, 18, }, // Arabian
    {   44,  240,   51483,   51500,   51521, 17, 21, 30, }, // Argentina
    {   44,  250,   51551,   51578,   51609, 27, 31, 40, }, // Argentina_Western
    {   44,  268,   51649,   51665,   51685, 16, 20, 29, }, // Armenia
    {   44,  286,   51714,   51728,   51746, 14, 18, 21, }, // Atlantic
    {   44,  295,   51767,   51790,   51823, 23, 33, 36, }, // Australia_Central
    {   44,  313,   51859,   51898,   51941, 39, 43, 46, }, // Australia_CentralWestern
    {   44,  338,   51987,   52013,   52043, 26, 30, 33, }, // Australia_Eastern
    {   44,  356,   52076,   52104,   52136, 28, 32, 35, }, // Australia_Western
    {   44,  374,   52171,   52187,   52207, 16, 20, 29, }, // Azerbaijan
    {   44,  385,   52236,   52247,   52263, 11, 16, 25, }, // Azores
    {   44,  397,   52288,   52302,   52320, 14, 18, 27, }, // Bangladesh
    {   44,  415,       0,   52347,       0,  0, 11,  0, }, // Bhutan
    {   44,  422,       0,   52358,       0,  0, 14,  0, }, // Bolivia
    {   44,  437,   52372,   52389,   52410, 17, 21, 30, }, // Brasilia
    {   44,  454,       0,   52440,       0,  0, 23,  0, }, // Brunei
    {   44,  461,   52463,   52479,   52499, 16, 20, 29, }, // Cape_Verde
    {   44,  478,       0,   52528,       0,  0, 16,  0, }, // Chamorro
    {   44,  487,   52544,   52555,   52570, 11, 15, 18, }, // Chatham
    {   44,  495,   52588,   52598,   52612, 10, 14, 23, }, // Chile
    {   44,  501,   52635,   52644,   52657,  9, 13, 16, }, // China
    {   44,  507,       0,   52673,       0,  0, 22,  0, }, // Christmas
    {   44,  517,       0,   52695,       0,  0, 22,  0, }, // Cocos
    {   44,  523,   52717,   52732,   52751, 15, 19, 28, }, // Colombia
    {   44,  532,   52779,   52799,   52823, 20, 24, 38, }, // Cook
    {   44,  537,   52861,   52872,   52887, 11, 15, 18, }, // Cuba
    {   44,  548,       0,   52905,       0,  0, 11,  0, }, // Davis
    {   44,  564,       0,   52916,       0,  0, 23,  0, }, // DumontDUrville
    {   44,  601,       0,   52939,       0,  0, 15,  0, }, // East_Timor
    {   44,  612,   52954,   52972,   52994, 18, 22, 31, }, // Easter
    {   44,  619,       0,   53025,       0,  0, 13,  0, }, // Ecuador
    {   44,  627,   53038,   53056,   53078, 18, 22, 31, }, // Europe_Central
    {   44,  642,   53109,   53130,   53155, 21, 25, 34, }, // Europe_Eastern
    {   44,  657,       0,   53189,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {   44,  680,   53215,   53238,   53265, 23, 27, 36, }, // Europe_Western
    {   44,  695,   53301,   53325,   53353, 24, 28, 37, }, // Falkland
    {   44,  704,   53390,   53400,   53414, 10, 14, 23, }, // Fiji
    {   44,  709,       0,   53437,       0,  0, 20,  0, }, // French_Guiana
    {   44,  723,       0,   53457,       0,  0, 35,  0, }, // French_Southern
    {   44,  746,       0,   53492,       0,  0, 15,  0, }, // Galapagos
    {   44,  756,       0,   53507,       0,  0, 15,  0, }, // Gambier
    {   44,  764,   53522,   53536,   53554, 14, 18, 27, }, // Georgia
    {   44,  772,       0,   53581,       0,  0, 25,  0, }, // Gilbert_Islands
    {   44,  788,       0,   53606,       0,  0, 17,  0, }, // GMT
    {   44,  830,   53623,   53643,   53667, 20, 24, 33, }, // Greenland_Eastern
    {   44,  848,   53700,   53723,   53750, 23, 27, 36, }, // Greenland_Western
    {   44,  871,       0,   53786,       0,  0, 18,  0, }, // Gulf
    {   44,  876,       0,   53804,       0,  0, 13,  0, }, // Guyana
    {   44,  883,   53817,   53837,   53861, 20, 24, 27, }, // Hawaii_Aleutian
    {   44,  899,   53888,   53899,   53914, 11, 15, 24, }, // Hong_Kong
    {   44,  909,   53938,   53950,   53966, 12, 16, 25, }, // Hovd
    {   44,  914,       0,   53991,       0,  0, 16,  0, }, // India
    {   44,  920,       0,   54007,       0,  0, 20,  0, }, // Indian_Ocean
    {   44,  933,       0,   54027,       0,  0, 16,  0, }, // Indochina
    {   44,  943,       0,   54043,       0,  0, 23,  0, }, // Indonesia_Central
    {   44,  961,       0,   54066,       0,  0, 26,  0, }, // Indonesia_Eastern
    {   44,  979,       0,   54092,       0,  0, 28,  0, }, // Indonesia_Western
    {   44,  997,   54120,   54130,   54144, 10, 14, 17, }, // Iran
    {   44, 1008,   54161,   54176,   54195, 15, 19, 28, }, // Irkutsk
    {   44, 1016,   54223,   54235,   54251, 12, 16, 19, }, // Israel
    {   44, 1023,   54270,   54281,   54296, 11, 15, 18, }, // Japan
    {   44, 1047,       0,   54314,       0,  0, 16,  0, }, // Kazakhstan
    {   44, 1058,       0,   54330,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {   44, 1077,       0,   54350,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   44, 1106,   54373,   54386,   54403, 13, 17, 20, }, // Korea
    {   44, 1112,       0,   54423,       0,  0, 13,  0, }, // Kosrae
    {   44, 1119,   54436,   54457,   54482, 21, 25, 33, }, // Krasnoyarsk
    {   44, 1151,       0,   54515,       0,  0, 18,  0, }, // Kyrgystan
    {   44, 1175,       0,   54533,       0,  0, 21,  0, }, // Line_Islands
    {   44, 1188,   54554,   54568,   54586, 14, 18, 21, }, // Lord_Howe
    {   44, 1204,   54607,   54620,   54637, 13, 17, 26, }, // Magadan
    {   44, 1219,       0,   54663,       0,  0, 17,  0, }, // Malaysia
    {   44, 1228,       0,   54680,       0,  0, 14,  0, }, // Maldives
    {   44, 1237,       0,   54694,       0,  0, 14,  0, }, // Marquesas
    {   44, 1247,       0,   54708,       0,  0, 26,  0, }, // Marshall_Islands
    {   44, 1264,   54734,   54746,   54762, 12, 16, 25, }, // Mauritius
    {   44, 1274,       0,   54787,       0,  0, 11,  0, }, // Mawson
    {   44, 1281,   54798,   54821,   54848, 23, 27, 30, }, // Mexico_Pacific
    {   44, 1296,   54878,   54893,   54912, 15, 19, 28, }, // Mongolia
    {   44, 1305,   54940,   54951,   54966, 11, 15, 24, }, // Moscow
    {   44, 1312,       0,   54990,       0,  0, 14,  0, }, // Myanmar
    {   44, 1320,       0,   55004,       0,  0, 11,  0, }, // Nauru
    {   44, 1326,       0,   55015,       0,  0, 11,  0, }, // Nepal
    {   44, 1332,   55026,   55047,   55072, 21, 25, 34, }, // New_Caledonia
    {   44, 1346,   55106,   55122,   55142, 16, 20, 23, }, // New_Zealand
    {   44, 1358,   55165,   55185,   55209, 20, 24, 27, }, // Newfoundland
    {   44, 1371,       0,   55236,       0,  0, 11,  0, }, // Niue
    {   44, 1376,   55247,   55265,   55289, 18, 24, 31, }, // Norfolk
    {   44, 1384,   55320,   55349,   55382, 29, 33, 42, }, // Noronha
    {   44, 1406,   55424,   55442,   55464, 18, 22, 31, }, // Novosibirsk
    {   44, 1418,   55495,   55506,   55521, 11, 15, 24, }, // Omsk
    {   44, 1428,   55545,   55560,   55579, 15, 19, 28, }, // Pakistan
    {   44, 1437,       0,   55607,       0,  0, 11,  0, }, // Palau
    {   44, 1443,       0,   55618,       0,  0, 21,  0, }, // Papua_New_Guinea
    {   44, 1460,   55639,   55653,   55671, 14, 18, 27, }, // Paraguay
    {   44, 1469,   55698,   55708,   55722, 10, 14, 23, }, // Peru
    {   44, 1474,   55745,   55759,   55777, 14, 18, 27, }, // Philippines
    {   44, 1486,       0,   55804,       0,  0, 24,  0, }, // Phoenix_Islands
    {   44, 1502,   55828,   55862,   55900, 34, 38, 41, }, // Pierre_Miquelon
    {   44, 1518,       0,   55941,       0,  0, 15,  0, }, // Pitcairn
    {   44, 1527,       0,   55956,       0,  0, 12,  0, }, // Ponape
    {   44, 1534,       0,   55968,       0,  0, 14,  0, }, // Pyongyang
    {   44, 1554,       0,   55982,       0,  0, 15,  0, }, // Reunion
    {   44, 1562,       0,   55997,       0,  0, 12,  0, }, // Rothera
    {   44, 1570,   56009,   56022,   56039, 13, 17, 26, }, // Sakhalin
    {   44, 1596,   56065,   56077,   56093, 12, 16, 19, }, // Samoa
    {   44, 1602,       0,   56112,       0,  0, 13,  0, }, // Seychelles
    {   44, 1624,       0,   56125,       0,  0, 18,  0, }, // Singapore
    {   44, 1634,       0,   56143,       0,  0, 24,  0, }, // Solomon
    {   44, 1642,       0,   56167,       0,  0, 21,  0, }, // South_Georgia
    {   44, 1656,       0,   56188,       0,  0, 13,  0, }, // Suriname
    {   44, 1676,       0,   56201,       0,  0, 11,  0, }, // Syowa
    {   44, 1682,       0,   56212,       0,  0, 12,  0, }, // Tahiti
    {   44, 1689,   56224,   56236,   56252, 12, 16, 19, }, // Taipei
    {   44, 1696,       0,   56271,       0,  0, 17,  0, }, // Tajikistan
    {   44, 1724,       0,   56288,       0,  0, 13,  0, }, // Tokelau
    {   44, 1732,   56301,   56311,   56325, 10, 14, 23, }, // Tonga
    {   44, 1738,       0,   56348,       0,  0, 11,  0, }, // Truk
    {   44, 1750,   56359,   56379,   56403, 20, 24, 33, }, // Turkmenistan
    {   44, 1763,       0,   56436,       0,  0, 13,  0, }, // Tuvalu
    {   44, 1777,   56449,   56462,   56479, 13, 17, 26, }, // Uruguay
    {   44, 1792,   56505,   56522,   56543, 17, 21, 30, }, // Uzbekistan
    {   44, 1803,   56573,   56585,   56601, 12, 16, 25, }, // Vanuatu
    {   44, 1811,       0,   56626,       0,  0, 16,  0, }, // Venezuela
    {   44, 1821,   56642,   56661,   56684, 19, 23, 32, }, // Vladivostok
    {   44, 1833,   56716,   56733,   56754, 17, 21, 30, }, // Volgograd
    {   44, 1843,       0,   56784,       0,  0, 12,  0, }, // Vostok
    {   44, 1850,       0,   56796,       0,  0, 15,  0, }, // Wake
    {   44, 1855,       0,   56811,       0,  0, 22,  0, }, // Wallis
    {   44, 1862,   56833,   56849,   56869, 16, 20, 29, }, // Yakutsk
    {   44, 1870,   56898,   56919,   56944, 21, 25, 34, }, // Yekaterinburg
    {   44, 1892,       0,   56978,       0,  0, 11,  0, }, // Yukon
    {   45,    0,   56989,   57000,   57020, 11, 20, 20, }, // Acre Asturian/Latin/Spain
    {   45,    5,       0,   57040,       0,  0, 17,  0, }, // Afghanistan
    {   45,   17,       0,   57057,       0,  0, 21,  0, }, // Africa_Central
    {   45,   32,       0,   57078,       0,  0, 22,  0, }, // Africa_Eastern
    {   45,   65,       0,   57100,       0,  0, 17,  0, }, // Africa_Southern
    {   45,   81,   57117,   57140,   57172, 23, 32, 32, }, // Africa_Western
    {   45,  107,   57204,   57217,   57239, 13, 22, 22, }, // Alaska
    {   45,  128,   57261,   57274,   57296, 13, 22, 22, }, // Almaty
    {   45,  135,   57318,   57335,   57361, 17, 26, 26, }, // Amazon
    {   45,  142,   57387,   57414,   57450, 27, 36, 36, }, // America_Central
    {   45,  158,   57486,   57514,   57551, 28, 37, 37, }, // America_Eastern
    {   45,  174,   57588,   57624,   57669, 36, 45, 45, }, // America_Mountain
    {   45,  191,   57714,   57746,   57787, 32, 41, 41, }, // America_Pacific
    {   45,  207,   57828,   57841,   57863, 13, 22, 22, }, // Anadyr
    {   45,  214,   57885,   57896,   57916, 11, 20, 20, }, // Apia
    {   45,  219,   57936,   57948,   57969, 12, 21, 21, }, // Aqtau
    {   45,  225,   57990,   58003,   58025, 13, 22, 22, }, // Aqtobe
    {   45,  232,   58047,   58060,   58082, 13, 22, 22, }, // Arabian
    {   45,  240,   58104,   58120,   58145, 16, 25, 25, }, // Argentina
    {   45,  250,   58170,   58197,   58233, 27, 36, 36, }, // Argentina_Western
    {   45,  268,   58269,   58283,   58306, 14, 23, 23, }, // Armenia
    {   45,  286,   58329,   58347,   58374, 18, 27, 27, }, // Atlantic
    {   45,  295,   58401,   58425,   58458, 24, 33, 33, }, // Australia_Central
    {   45,  313,   58491,   58525,   58568, 34, 43, 43, }, // Australia_CentralWestern
    {   45,  338,   58611,   58636,   58670, 25, 34, 34, }, // Australia_Eastern
    {   45,  356,   58704,   58730,   58765, 26, 35, 35, }, // Australia_Western
    {   45,  374,   58800,   58817,   58843, 17, 26, 26, }, // Azerbaijan
    {   45,  385,   58869,   58887,   58914, 18, 27, 27, }, // Azores
    {   45,  397,   58941,   58958,   58984, 17, 26, 26, }, // Bangladesh
    {   45,  415,       0,   59010,       0,  0, 13,  0, }, // Bhutan
    {   45,  422,       0,   59023,       0,  0, 15,  0, }, // Bolivia
    {   45,  437,   59038,   59054,   59079, 16, 25, 25, }, // Brasilia
    {   45,  454,       0,   59104,       0,  0, 25,  0, }, // Brunei
    {   45,  461,   59129,   59147,   59174, 18, 27, 27, }, // Cape_Verde
    {   45,  472,       0,   59201,       0,  0, 13,  0, }, // Casey
    {   45,  478,       0,   59214,       0,  0, 25,  0, }, // Chamorro
    {   45,  487,   59239,   59254,   59278, 15, 24, 24, }, // Chatham
    {   45,  495,   59302,   59315,   59337, 13, 22, 22, }, // Chile
    {   45,  501,   59359,   59372,   59394, 13, 22, 22, }, // China
    {   45,  507,       0,   59416,       0,  0, 35,  0, }, // Christmas
    {   45,  517,       0,   59451,       0,  0, 24,  0, }, // Cocos
    {   45,  523,   59475,   59491,   59516, 16, 25, 25, }, // Colombia
    {   45,  532,   59541,   59564,   59596, 23, 32, 38, }, // Cook
    {   45,  537,   59634,   59646,   59667, 12, 21, 21, }, // Cuba
    {   45,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {   45,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {   45,  601,       0,   59725,       0,  0, 22,  0, }, // East_Timor
    {   45,  612,   59747,   59773,   59808, 26, 35, 35, }, // Easter
    {   45,  619,       0,   59843,       0,  0, 14,  0, }, // Ecuador
    {   45,  627,   59857,   59878,   59908, 21, 30, 30, }, // Europe_Central
    {   45,  642,   59938,   59960,   59991, 22, 31, 31, }, // Europe_Eastern
    {   45,  657,       0,   60022,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {   45,  680,   60052,   60076,   60109, 24, 33, 33, }, // Europe_Western
    {   45,  695,   60142,   60169,   60205, 27, 36, 36, }, // Falkland
    {   45,  704,   60241,   60253,   60274, 12, 21, 21, }, // Fiji
    {   45,  709,       0,   60295,       0,  0, 26,  0, }, // French_Guiana
    {   45,  723,       0,   60321,       0,  0, 34,  0, }, // French_Southern
    {   45,  746,       0,   60355,       0,  0, 17,  0, }, // Galapagos
    {   45,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {   45,  764,   60387,   60401,   60424, 14, 23, 23, }, // Georgia
    {   45,  772,       0,   60447,       0,  0, 26,  0, }, // Gilbert_Islands
    {   45,  788,       0,   60473,       0,  0, 23,  0, }, // GMT
    {   45,  830,   60496,   60524,   60561, 28, 37, 37, }, // Greenland_Eastern
    {   45,  848,   60598,   60628,   60667, 30, 39, 39, }, // Greenland_Western
    {   45,  866,       0,   60706,       0,  0, 21,  0, }, // Guam
    {   45,  871,       0,   60727,       0,  0, 23,  0, }, // Gulf
    {   45,  876,       0,   60295,       0,  0, 17,  0, }, // Guyana
    {   45,  883,   60750,   60775,   60809, 25, 34, 34, }, // Hawaii_Aleutian
    {   45,  899,   60843,   60860,   60886, 17, 26, 26, }, // Hong_Kong
    {   45,  909,   60912,   60924,   60945, 12, 21, 21, }, // Hovd
    {   45,  914,       0,   60966,       0,  0, 25,  0, }, // India
    {   45,  920,       0,   60991,       0,  0, 22,  0, }, // Indian_Ocean
    {   45,  933,       0,   61013,       0,  0, 16,  0, }, // Indochina
    {   45,  943,       0,   61029,       0,  0, 24,  0, }, // Indonesia_Central
    {   45,  961,       0,   61053,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   45,  979,       0,   61078,       0,  0, 26,  0, }, // Indonesia_Western
    {   45,  997,   61104,   61115,   61135, 11, 20, 20, }, // Iran
    {   45, 1008,   61155,   61169,   61192, 14, 23, 23, }, // Irkutsk
    {   45, 1016,   61215,   61228,   61250, 13, 22, 22, }, // Israel
    {   45, 1023,   61272,   61285,   61307, 13, 22, 22, }, // Japan
    {   45, 1029,   61329,   61361,   61402, 32, 41, 41, }, // Kamchatka
    {   45, 1058,       0,   61443,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {   45, 1077,       0,   61470,       0,  0, 29,  0, }, // Kazakhstan_Western
    {   45, 1106,   61499,   61512,   61534, 13, 22, 22, }, // Korea
    {   45, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {   45, 1119,   61570,   61589,   61617, 19, 28, 28, }, // Krasnoyarsk
    {   45, 1151,       0,   61645,       0,  0, 19,  0, }, // Kyrgystan
    {   45, 1161,       0,   61664,       0,  0, 13,  0, }, // Lanka
    {   45, 1175,       0,   61677,       0,  0, 23,  0, }, // Line_Islands
    {   45, 1188,   61700,   61717,   61743, 17, 26, 26, }, // Lord_Howe
    {   45, 1198,   61769,   61782,   61804, 13, 22, 22, }, // Macau
    {   45, 1204,   61826,   61841,   61865, 15, 24, 24, }, // Magadan
    {   45, 1219,       0,   61889,       0,  0, 15,  0, }, // Malaysia
    {   45, 1228,       0,   61904,       0,  0, 20,  0, }, // Maldives
    {   45, 1237,       0,   61924,       0,  0, 21,  0, }, // Marquesas
    {   45, 1247,       0,   61945,       0,  0, 27,  0, }, // Marshall_Islands
    {   45, 1264,   61972,   61988,   62013, 16, 25, 25, }, // Mauritius
    {   45, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {   45, 1281,   62052,   62079,   62115, 27, 36, 36, }, // Mexico_Pacific
    {   45, 1296,   62151,   62168,   62194, 17, 26, 26, }, // Mongolia
    {   45, 1305,   62220,   62233,   62255, 13, 22, 22, }, // Moscow
    {   45, 1312,       0,   62277,       0,  0, 15,  0, }, // Myanmar
    {   45, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {   45, 1326,       0,   62305,       0,  0, 14,  0, }, // Nepal
    {   45, 1332,   62319,   62342,   62374, 23, 32, 32, }, // New_Caledonia
    {   45, 1346,   62406,   62427,   62457, 21, 30, 30, }, // New_Zealand
    {   45, 1358,   62487,   62507,   62536, 20, 29, 29, }, // Newfoundland
    {   45, 1371,       0,   62565,       0,  0, 12,  0, }, // Niue
    {   45, 1376,   62577,   62601,   62634, 24, 33, 33, }, // Norfolk
    {   45, 1384,   62667,   62694,   62730, 27, 36, 36, }, // Noronha
    {   45, 1392,       0,   62766,       0,  0, 37,  0, }, // North_Mariana
    {   45, 1406,   62803,   62822,   62850, 19, 28, 28, }, // Novosibirsk
    {   45, 1418,   62878,   62889,   62909, 11, 20, 20, }, // Omsk
    {   45, 1428,   62929,   62947,   62974, 18, 27, 27, }, // Pakistan
    {   45, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {   45, 1443,       0,   63014,       0,  0, 26,  0, }, // Papua_New_Guinea
    {   45, 1460,   63040,   63057,   63083, 17, 26, 26, }, // Paraguay
    {   45, 1469,   63109,   63122,   63144, 13, 22, 22, }, // Peru
    {   45, 1474,   63166,   63183,   63209, 17, 26, 26, }, // Philippines
    {   45, 1486,       0,   63235,       0,  0, 26,  0, }, // Phoenix_Islands
    {   45, 1502,   63261,   63292,   63332, 31, 40, 40, }, // Pierre_Miquelon
    {   45, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {   45, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {   45, 1534,       0,   63402,       0,  0, 17,  0, }, // Pyongyang
    {   45, 1544,   63419,   63436,   63462, 17, 26, 26, }, // Qyzylorda
    {   45, 1554,       0,   63488,       0,  0, 15,  0, }, // Reunion
    {   45, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {   45, 1570,   63518,   63533,   63557, 15, 24, 24, }, // Sakhalin
    {   45, 1579,   63581,   63595,   63618, 14, 23, 23, }, // Samara
    {   45, 1596,   63641,   63654,   63676, 13, 22, 22, }, // Samoa
    {   45, 1602,       0,   63698,       0,  0, 20,  0, }, // Seychelles
    {   45, 1624,       0,   63718,       0,  0, 25,  0, }, // Singapore
    {   45, 1634,       0,   63743,       0,  0, 26,  0, }, // Solomon
    {   45, 1642,       0,   63769,       0,  0, 22,  0, }, // South_Georgia
    {   45, 1656,       0,   63791,       0,  0, 16,  0, }, // Suriname
    {   45, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {   45, 1682,       0,   63820,       0,  0, 14,  0, }, // Tahiti
    {   45, 1689,   63834,   63848,   63871, 14, 23, 23, }, // Taipei
    {   45, 1696,       0,   63894,       0,  0, 20,  0, }, // Tajikistan
    {   45, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {   45, 1732,   63929,   63942,   63964, 13, 22, 22, }, // Tonga
    {   45, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {   45, 1750,   63999,   64020,   64050, 21, 30, 30, }, // Turkmenistan
    {   45, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {   45, 1777,   64094,   64110,   64135, 16, 25, 25, }, // Uruguay
    {   45, 1792,   64160,   64180,   64209, 20, 29, 29, }, // Uzbekistan
    {   45, 1803,   64238,   64253,   64277, 15, 24, 24, }, // Vanuatu
    {   45, 1811,       0,   64301,       0,  0, 17,  0, }, // Venezuela
    {   45, 1821,   64318,   64337,   64365, 19, 28, 28, }, // Vladivostok
    {   45, 1833,   64393,   64410,   64436, 17, 26, 26, }, // Volgograd
    {   45, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {   45, 1850,       0,   64476,       0,  0, 21,  0, }, // Wake
    {   45, 1855,       0,   64497,       0,  0, 23,  0, }, // Wallis
    {   45, 1862,   64520,   64535,   64559, 15, 24, 24, }, // Yakutsk
    {   45, 1870,   64583,   64605,   64636, 22, 31, 31, }, // Yekaterinburg
    {   48,    5,       0,   64667,       0,  0, 16,  0, }, // Afghanistan Azerbaijani/Latin/Azerbaijan
    {   48,   17,       0,   64683,       0,  0, 20,  0, }, // Africa_Central
    {   48,   32,       0,   64703,       0,  0, 18,  0, }, // Africa_Eastern
    {   48,   65,       0,   64721,       0,  0, 19,  0, }, // Africa_Southern
    {   48,   81,   64740,   64758,   64785, 18, 27, 22, }, // Africa_Western
    {   48,  107,   64807,   64820,   64842, 13, 22, 17, }, // Alaska
    {   48,  135,   64859,   64871,   64892, 12, 21, 16, }, // Amazon
    {   48,  142,   64908,   64936,   64973, 28, 37, 32, }, // America_Central
    {   48,  158,   65005,   65031,   65066, 26, 35, 30, }, // America_Eastern
    {   48,  174,   65096,   65123,   65159, 27, 36, 31, }, // America_Mountain
    {   48,  191,   65190,   65222,   65263, 32, 41, 36, }, // America_Pacific
    {   48,  214,   65299,   65309,   65328, 10, 19, 14, }, // Apia
    {   48,  232,   65342,   65357,   65381, 15, 24, 19, }, // Arabian
    {   48,  240,   65400,   65415,   65439, 15, 24, 19, }, // Argentina
    {   48,  250,   65458,   65479,   65509, 21, 30, 25, }, // Argentina_Western
    {   48,  268,   65534,   65550,   65575, 16, 25, 20, }, // Armenia
    {   48,  286,   65595,   65608,   65630, 13, 22, 18, }, // Atlantic
    {   48,  295,   65648,   65672,   65705, 24, 33, 28, }, // Australia_Central
    {   48,  313,   65733,   65763,   65802, 30, 39, 34, }, // Australia_CentralWestern
    {   48,  338,   65836,   65858,   65889, 22, 31, 26, }, // Australia_Eastern
    {   48,  356,   65741,   65771,   65810, 22, 31, 26, }, // Australia_Western
    {   48,  374,   65915,   65931,   65956, 16, 25, 20, }, // Azerbaijan
    {   48,  385,   65976,   65986,   66005, 10, 19, 14, }, // Azores
    {   48,  397,   66019,   66034,   66058, 15, 24, 19, }, // Bangladesh
    {   48,  415,       0,   66077,       0,  0, 11,  0, }, // Bhutan
    {   48,  422,       0,   66088,       0,  0, 14,  0, }, // Bolivia
    {   48,  437,   66102,   66117,   66141, 15, 24, 19, }, // Brasilia
    {   48,  454,       0,   66160,       0,  0, 23,  0, }, // Brunei
    {   48,  461,   66183,   66199,   66224, 16, 25, 20, }, // Cape_Verde
    {   48,  478,       0,   66244,       0,  0, 13,  0, }, // Chamorro
    {   48,  487,   66257,   66269,   66290, 12, 21, 16, }, // Chatham
    {   48,  495,   66306,   66316,   66335, 10, 19, 14, }, // Chile
    {   48,  501,   66349,   66358,   66376,  9, 18, 13, }, // China
    {   48,  507,       0,   66389,       0,  0, 17,  0, }, // Christmas
    {   48,  517,       0,   66406,       0,  0, 19,  0, }, // Cocos
    {   48,  523,   66425,   66440,   66464, 15, 24, 19, }, // Colombia
    {   48,  532,   66483,   66500,   66526, 17, 26, 27, }, // Cook
    {   48,  537,   66553,   66563,   66582, 10, 19, 14, }, // Cuba
    {   48,  548,       0,   66596,       0,  0, 11,  0, }, // Davis
    {   48,  564,       0,   66607,       0,  0, 19,  0, }, // DumontDUrville
    {   48,  601,       0,   66626,       0,  0, 17,  0, }, // East_Timor
    {   48,  612,   66643,   66660,   66686, 17, 26, 21, }, // Easter
    {   48,  619,       0,   66707,       0,  0, 13,  0, }, // Ecuador
    {   48,  627,   66720,   66740,   66769, 20, 29, 24, }, // Europe_Central
    {   48,  642,   66793,   66811,   66838, 18, 27, 22, }, // Europe_Eastern
    {   48,  657,       0,   66860,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {   48,  680,   66884,   66902,   66929, 18, 27, 22, }, // Europe_Western
    {   48,  695,   66951,   66973,   67004, 22, 31, 26, }, // Falkland
    {   48,  704,   67030,   67040,   67059, 10, 19, 14, }, // Fiji
    {   48,  709,       0,   67073,       0,  0, 22,  0, }, // French_Guiana
    {   48,  723,       0,   67095,       0,  0, 33,  0, }, // French_Southern
    {   48,  746,       0,   67128,       0,  0, 15,  0, }, // Galapagos
    {   48,  756,       0,   67143,       0,  0, 13,  0, }, // Gambier
    {   48,  764,   67156,   67171,   67195, 15, 24, 19, }, // Georgia
    {   48,  772,       0,   67214,       0,  0, 21,  0, }, // Gilbert_Islands
    {   48,  788,       0,   67235,       0,  0, 18,  0, }, // GMT
    {   48,  830,   67253,   67276,   67308, 23, 32, 27, }, // Greenland_Eastern
    {   48,  848,   67335,   67358,   67390, 23, 32, 27, }, // Greenland_Western
    {   48,  871,       0,   67417,       0,  0, 12,  0, }, // Gulf
    {   48,  876,       0,   67429,       0,  0, 12,  0, }, // Guyana
    {   48,  883,   67441,   67458,   67484, 17, 26, 21, }, // Hawaii_Aleutian
    {   48,  899,   67505,   67520,   67544, 15, 24, 19, }, // Hong_Kong
    {   48,  909,   67563,   67573,   67592, 10, 19, 14, }, // Hovd
    {   48,  914,       0,   67606,       0,  0, 15,  0, }, // India
    {   48,  920,       0,   67621,       0,  0, 17,  0, }, // Indian_Ocean
    {   48,  933,       0,   67638,       0,  0, 13,  0, }, // Indochina
    {   48,  943,       0,   67651,       0,  0, 24,  0, }, // Indonesia_Central
    {   48,  961,       0,   67675,       0,  0, 22,  0, }, // Indonesia_Eastern
    {   48,  979,       0,   67697,       0,  0, 22,  0, }, // Indonesia_Western
    {   48,  997,   67719,   67729,   67748, 10, 19, 14, }, // Iran
    {   48, 1008,   67762,   67775,   67797, 13, 22, 17, }, // Irkutsk
    {   48, 1016,   67814,   67826,   67847, 12, 21, 16, }, // Israel
    {   48, 1023,   67863,   67877,   67900, 14, 23, 18, }, // Japan
    {   48, 1047,       0,   67918,       0,  0, 16,  0, }, // Kazakhstan
    {   48, 1058,       0,   67934,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {   48, 1077,       0,   67956,       0,  0, 22,  0, }, // Kazakhstan_Western
    {   48, 1106,   67978,   67990,   68011, 12, 21, 16, }, // Korea
    {   48, 1112,       0,   68027,       0,  0, 11,  0, }, // Kosrae
    {   48, 1119,   68038,   68055,   68081, 17, 26, 21, }, // Krasnoyarsk
    {   48, 1151,       0,   68102,       0,  0, 17,  0, }, // Kyrgystan
    {   48, 1175,       0,   68119,       0,  0, 18,  0, }, // Line_Islands
    {   48, 1188,   68137,   68151,   68174, 14, 23, 18, }, // Lord_Howe
    {   48, 1204,   68192,   68205,   68227, 13, 22, 17, }, // Magadan
    {   48, 1219,       0,   68244,       0,  0, 15,  0, }, // Malaysia
    {   48, 1228,       0,   68259,       0,  0, 12,  0, }, // Maldives
    {   48, 1237,       0,   68271,       0,  0, 14,  0, }, // Marquesas
    {   48, 1247,       0,   68285,       0,  0, 20,  0, }, // Marshall_Islands
    {   48, 1264,   68305,   68318,   68340, 13, 22, 17, }, // Mauritius
    {   48, 1274,       0,   68357,       0,  0, 12,  0, }, // Mawson
    {   48, 1281,   68369,   68394,   68428, 25, 34, 29, }, // Mexico_Pacific
    {   48, 1296,   68457,   68472,   68496, 15, 24, 19, }, // Mongolia
    {   48, 1305,   68515,   68527,   68548, 12, 21, 16, }, // Moscow
    {   48, 1312,       0,   68564,       0,  0, 12,  0, }, // Myanmar
    {   48, 1320,       0,   68576,       0,  0, 11,  0, }, // Nauru
    {   48, 1326,       0,   68587,       0,  0, 11,  0, }, // Nepal
    {   48, 1332,   68598,   68619,   68649, 21, 30, 25, }, // New_Caledonia
    {   48, 1346,   68674,   68694,   68723, 20, 29, 24, }, // New_Zealand
    {   48, 1358,   68747,   68765,   68792, 18, 27, 22, }, // Newfoundland
    {   48, 1371,       0,   68814,       0,  0, 10,  0, }, // Niue
    {   48, 1376,   68824,   68843,   68871, 19, 28, 23, }, // Norfolk
    {   48, 1384,   68894,   68919,   68953, 25, 34, 29, }, // Noronha
    {   48, 1406,   68982,   68999,   69025, 17, 26, 21, }, // Novosibirsk
    {   48, 1418,   69046,   69056,   69075, 10, 19, 14, }, // Omsk
    {   48, 1428,   69089,   69103,   69126, 14, 23, 18, }, // Pakistan
    {   48, 1437,       0,   69144,       0,  0, 11,  0, }, // Palau
    {   48, 1443,       0,   69155,       0,  0, 24,  0, }, // Papua_New_Guinea
    {   48, 1460,   69179,   69193,   69216, 14, 23, 18, }, // Paraguay
    {   48, 1469,   69234,   69244,   69263, 10, 19, 14, }, // Peru
    {   48, 1474,   69277,   69291,   69314, 14, 23, 18, }, // Philippines
    {   48, 1486,       0,   69332,       0,  0, 20,  0, }, // Phoenix_Islands
    {   48, 1502,   69352,   69382,   69421, 30, 39, 34, }, // Pierre_Miquelon
    {   48, 1518,       0,   69455,       0,  0, 13,  0, }, // Pitcairn
    {   48, 1527,       0,   69468,       0,  0, 12,  0, }, // Ponape
    {   48, 1534,       0,   69480,       0,  0, 13,  0, }, // Pyongyang
    {   48, 1554,       0,   69493,       0,  0,  8,  0, }, // Reunion
    {   48, 1562,       0,   69501,       0,  0, 12,  0, }, // Rothera
    {   48, 1570,   69513,   69526,   69548, 13, 22, 17, }, // Sakhalin
    {   48, 1579,   69565,   69577,   69598, 12, 21, 16, }, // Samara
    {   48, 1596,   69614,   69625,   69645, 11, 20, 15, }, // Samoa
    {   48, 1602,       0,   69660,       0,  0, 20,  0, }, // Seychelles
    {   48, 1624,       0,   69680,       0,  0, 14,  0, }, // Singapore
    {   48, 1634,       0,   69694,       0,  0, 21,  0, }, // Solomon
    {   48, 1642,       0,   69715,       0,  0, 20,  0, }, // South_Georgia
    {   48, 1656,       0,   69735,       0,  0, 13,  0, }, // Suriname
    {   48, 1676,       0,   69748,       0,  0, 11,  0, }, // Syowa
    {   48, 1682,       0,   69759,       0,  0, 12,  0, }, // Tahiti
    {   48, 1689,   69771,   69783,   69804, 12, 21, 16, }, // Taipei
    {   48, 1696,       0,   69820,       0,  0, 16,  0, }, // Tajikistan
    {   48, 1724,       0,   69836,       0,  0, 13,  0, }, // Tokelau
    {   48, 1732,   69849,   69860,   69880, 11, 20, 15, }, // Tonga
    {   48, 1738,       0,   69895,       0,  0, 10,  0, }, // Truk
    {   48, 1750,   69905,   69923,   69950, 18, 27, 22, }, // Turkmenistan
    {   48, 1763,       0,   69972,       0,  0, 12,  0, }, // Tuvalu
    {   48, 1777,   69984,   69997,   70019, 13, 22, 17, }, // Uruguay
    {   48, 1792,   70036,   70052,   70077, 16, 25, 20, }, // Uzbekistan
    {   48, 1803,   70097,   70110,   70132, 13, 22, 17, }, // Vanuatu
    {   48, 1811,       0,   70149,       0,  0, 15,  0, }, // Venezuela
    {   48, 1821,   70164,   70181,   70207, 17, 26, 21, }, // Vladivostok
    {   48, 1833,   70228,   70243,   70267, 15, 24, 19, }, // Volgograd
    {   48, 1843,       0,   70286,       0,  0, 12,  0, }, // Vostok
    {   48, 1850,       0,   70298,       0,  0, 10,  0, }, // Wake
    {   48, 1855,       0,   70308,       0,  0, 22,  0, }, // Wallis
    {   48, 1862,   70330,   70343,   70365, 13, 22, 17, }, // Yakutsk
    {   48, 1870,   70382,   70401,   70429, 19, 28, 23, }, // Yekaterinburg
    {   48, 1892,       0,   70452,       0,  0, 11,  0, }, // Yukon
    {   56,    0,   70463,   70471,   70484,  8, 13, 19, }, // Acre Bangla/Bangla/Bangladesh
    {   56,    5,       0,   70503,       0,  0, 16,  0, }, // Afghanistan
    {   56,   17,       0,   70519,       0,  0, 17,  0, }, // Africa_Central
    {   56,   32,       0,   70536,       0,  0, 18,  0, }, // Africa_Eastern
    {   56,   65,       0,   70554,       0,  0, 24,  0, }, // Africa_Southern
    {   56,   81,   70578,   70597,   70621, 19, 24, 32, }, // Africa_Western
    {   56,  107,   70653,   70665,   70682, 12, 17, 20, }, // Alaska
    {   56,  128,   70702,   70714,   70731, 12, 17, 23, }, // Almaty
    {   56,  135,   70754,   70764,   70779, 10, 15, 23, }, // Amazon
    {   56,  142,   70802,   70817,   70837, 15, 20, 23, }, // America_Central
    {   56,  158,   70860,   70878,   70900, 18, 22, 25, }, // America_Eastern
    {   56,  174,   70925,   70946,   70972, 21, 26, 27, }, // America_Mountain
    {   56,  191,   70999,   71031,   71068, 32, 37, 38, }, // America_Pacific
    {   56,  207,   71106,   71119,   71137, 13, 18, 26, }, // Anadyr
    {   56,  214,   71163,   71174,   71190, 11, 16, 17, }, // Apia
    {   56,  219,   71207,   71217,   71232, 10, 15, 21, }, // Aqtau
    {   56,  225,   71253,   71263,   71278, 10, 15, 21, }, // Aqtobe
    {   56,  232,   71299,   71308,   71322,  9, 14, 17, }, // Arabian
    {   56,  240,   71339,   71355,   71376, 16, 21, 29, }, // Argentina
    {   56,  250,   71405,   71429,   71460, 24, 31, 36, }, // Argentina_Western
    {   56,  268,   71496,   71511,   71531, 15, 20, 28, }, // Armenia
    {   56,  286,   71559,   71575,   71594, 16, 19, 22, }, // Atlantic
    {   56,  295,   71616,   71643,   71675, 27, 32, 35, }, // Australia_Central
    {   56,  313,   71710,   71745,   71785, 35, 40, 43, }, // Australia_CentralWestern
    {   56,  338,   71828,   71850,   71877, 22, 27, 30, }, // Australia_Eastern
    {   56,  356,   71907,   71931,   71960, 24, 29, 32, }, // Australia_Western
    {   56,  374,   71992,   72007,   72027, 15, 20, 28, }, // Azerbaijan
    {   56,  385,   72055,   72066,   72082, 11, 16, 24, }, // Azores
    {   56,  397,   72106,   72119,   72137, 13, 18, 26, }, // Bangladesh
    {   56,  415,       0,   72163,       0,  0, 10,  0, }, // Bhutan
    {   56,  422,       0,   72173,       0,  0, 14,  0, }, // Bolivia
    {   56,  437,   72187,   72203,   72224, 16, 21, 29, }, // Brasilia
    {   56,  454,       0,   72253,       0,  0, 23,  0, }, // Brunei
    {   56,  461,   72276,   72290,   72309, 14, 19, 27, }, // Cape_Verde
    {   56,  478,       0,   72336,       0,  0, 16,  0, }, // Chamorro
    {   56,  487,   72352,   72364,   72381, 12, 17, 20, }, // Chatham
    {   56,  495,   72401,   72410,   72424,  9, 14, 22, }, // Chile
    {   56,  501,   72446,   72454,   72467,  8, 13, 16, }, // China
    {   56,  507,       0,   72483,       0,  0, 19,  0, }, // Christmas
    {   56,  517,       0,   72502,       0,  0, 21,  0, }, // Cocos
    {   56,  523,   72523,   72539,   72560, 16, 21, 29, }, // Colombia
    {   56,  532,   72589,   72608,   72632, 19, 24, 39, }, // Cook
    {   56,  537,   72671,   72682,   72697, 11, 15, 18, }, // Cuba
    {   56,  548,       0,   72715,       0,  0, 10,  0, }, // Davis
    {   56,  564,       0,   72725,       0,  0, 22,  0, }, // DumontDUrville
    {   56,  601,       0,   72747,       0,  0, 15,  0, }, // East_Timor
    {   56,  612,   72762,   72779,   72801, 17, 22, 30, }, // Easter
    {   56,  619,       0,   72831,       0,  0, 13,  0, }, // Ecuador
    {   56,  627,   72844,   72862,   72885, 18, 23, 31, }, // Europe_Central
    {   56,  642,   72916,   72935,   72959, 19, 24, 32, }, // Europe_Eastern
    {   56,  657,       0,   72991,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {   56,  680,   73021,   73041,   73066, 20, 25, 33, }, // Europe_Western
    {   56,  695,   73099,   73124,   73154, 25, 30, 38, }, // Falkland
    {   56,  704,   73192,   73201,   73215,  9, 14, 22, }, // Fiji
    {   56,  709,       0,   73237,       0,  0, 18,  0, }, // French_Guiana
    {   56,  723,       0,   73255,       0,  0, 33,  0, }, // French_Southern
    {   56,  746,       0,   73288,       0,  0, 14,  0, }, // Galapagos
    {   56,  756,       0,   73302,       0,  0, 17,  0, }, // Gambier
    {   56,  764,   73319,   73332,   73350, 13, 18, 26, }, // Georgia
    {   56,  772,       0,   73376,       0,  0, 24,  0, }, // Gilbert_Islands
    {   56,  788,       0,   73400,       0,  0, 16,  0, }, // GMT
    {   56,  830,   73416,   73439,   73467, 23, 28, 36, }, // Greenland_Eastern
    {   56,  848,   73503,   73527,   73556, 24, 29, 37, }, // Greenland_Western
    {   56,  866,       0,   73593,       0,  0, 15,  0, }, // Guam
    {   56,  871,       0,   73608,       0,  0, 19,  0, }, // Gulf
    {   56,  876,       0,   73627,       0,  0, 12,  0, }, // Guyana
    {   56,  883,   73639,   73657,   73680, 18, 23, 26, }, // Hawaii_Aleutian
    {   56,  899,   73706,   73716,   73731, 10, 15, 23, }, // Hong_Kong
    {   56,  909,   73754,   73763,   73777,  9, 14, 22, }, // Hovd
    {   56,  914,       0,   73799,       0,  0, 17,  0, }, // India
    {   56,  920,       0,   73816,       0,  0, 20,  0, }, // Indian_Ocean
    {   56,  933,       0,   73836,       0,  0, 13,  0, }, // Indochina
    {   56,  943,       0,   73849,       0,  0, 28,  0, }, // Indonesia_Central
    {   56,  961,       0,   73877,       0,  0, 23,  0, }, // Indonesia_Eastern
    {   56,  979,       0,   73900,       0,  0, 25,  0, }, // Indonesia_Western
    {   56,  997,   73925,   73934,   73948,  9, 14, 17, }, // Iran
    {   56, 1008,   73965,   73978,   73996, 13, 18, 26, }, // Irkutsk
    {   56, 1016,   74022,   74035,   74053, 13, 18, 21, }, // Israel
    {   56, 1023,   74074,   74084,   74099, 10, 15, 18, }, // Japan
    {   56, 1029,   74117,   74153,   74193, 36, 40, 47, }, // Kamchatka
    {   56, 1047,       0,   74240,       0,  0, 16,  0, }, // Kazakhstan
    {   56, 1058,       0,   74256,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {   56, 1077,       0,   74278,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   56, 1106,   74301,   74314,   74332, 13, 18, 21, }, // Korea
    {   56, 1112,       0,   74353,       0,  0, 11,  0, }, // Kosrae
    {   56, 1119,   74364,   74385,   74411, 21, 26, 34, }, // Krasnoyarsk
    {   56, 1151,       0,   74445,       0,  0, 15,  0, }, // Kyrgystan
    {   56, 1161,       0,   74460,       0,  0, 10,  0, }, // Lanka
    {   56, 1175,       0,   74470,       0,  0, 20,  0, }, // Line_Islands
    {   56, 1188,   74490,   74506,   74527, 16, 21, 24, }, // Lord_Howe
    {   56, 1198,   74551,   74561,   74575, 10, 14, 21, }, // Macau
    {   56, 1204,   74596,   74610,   74629, 14, 19, 27, }, // Magadan
    {   56, 1219,       0,   74656,       0,  0, 16,  0, }, // Malaysia
    {   56, 1228,       0,   74672,       0,  0, 13,  0, }, // Maldives
    {   56, 1237,       0,   74685,       0,  0, 14,  0, }, // Marquesas
    {   56, 1247,       0,   74699,       0,  0, 23,  0, }, // Marshall_Islands
    {   56, 1264,   74722,   74733,   74749, 11, 16, 24, }, // Mauritius
    {   56, 1274,       0,   74773,       0,  0,  8,  0, }, // Mawson
    {   56, 1281,   74781,   74815,   74853, 34, 38, 42, }, // Mexico_Pacific
    {   56, 1296,   74895,   74910,   74930, 15, 20, 28, }, // Mongolia
    {   56, 1305,   74958,   74968,   74983, 10, 15, 23, }, // Moscow
    {   56, 1312,       0,   75006,       0,  0, 14,  0, }, // Myanmar
    {   56, 1320,       0,   75020,       0,  0, 10,  0, }, // Nauru
    {   56, 1326,       0,   75030,       0,  0, 10,  0, }, // Nepal
    {   56, 1332,   75040,   75062,   75089, 22, 27, 35, }, // New_Caledonia
    {   56, 1346,   75124,   75141,   75163, 17, 22, 25, }, // New_Zealand
    {   56, 1358,   75188,   75209,   75235, 21, 26, 29, }, // Newfoundland
    {   56, 1371,       0,   75264,       0,  0,  9,  0, }, // Niue
    {   56, 1376,   75273,   75289,   75310, 16, 21, 24, }, // Norfolk
    {   56, 1384,   75334,   75360,   75391, 26, 31, 39, }, // Noronha
    {   56, 1392,       0,   75430,       0,  0, 27,  0, }, // North_Mariana
    {   56, 1406,   75457,   75475,   75498, 18, 23, 31, }, // Novosibirsk
    {   56, 1418,   75529,   75539,   75554, 10, 15, 23, }, // Omsk
    {   56, 1428,   75577,   75591,   75610, 14, 19, 27, }, // Pakistan
    {   56, 1437,       0,   75637,       0,  0, 10,  0, }, // Palau
    {   56, 1443,       0,   75647,       0,  0, 21,  0, }, // Papua_New_Guinea
    {   56, 1460,   75668,   75684,   75705, 16, 21, 29, }, // Paraguay
    {   56, 1469,   75734,   75743,   75757,  9, 14, 22, }, // Peru
    {   56, 1474,   75779,   75792,   75810, 13, 18, 26, }, // Philippines
    {   56, 1486,       0,   75836,       0,  0, 23,  0, }, // Phoenix_Islands
    {   56, 1502,   75859,   75885,   75916, 26, 31, 34, }, // Pierre_Miquelon
    {   56, 1518,       0,   75950,       0,  0, 16,  0, }, // Pitcairn
    {   56, 1527,       0,   75966,       0,  0, 11,  0, }, // Ponape
    {   56, 1534,       0,   75977,       0,  0, 15,  0, }, // Pyongyang
    {   56, 1544,   75992,   76007,   76026, 15, 19, 26, }, // Qyzylorda
    {   56, 1554,       0,   76052,       0,  0, 14,  0, }, // Reunion
    {   56, 1562,       0,   76066,       0,  0, 10,  0, }, // Rothera
    {   56, 1570,   76076,   76088,   76105, 12, 17, 25, }, // Sakhalin
    {   56, 1579,   76130,   76141,   76156, 11, 15, 22, }, // Samara
    {   56, 1596,   76178,   76190,   76207, 12, 17, 20, }, // Samoa
    {   56, 1602,       0,   76227,       0,  0, 11,  0, }, // Seychelles
    {   56, 1624,       0,   76238,       0,  0, 19,  0, }, // Singapore
    {   56, 1634,       0,   76257,       0,  0, 21,  0, }, // Solomon
    {   56, 1642,       0,   76278,       0,  0, 20,  0, }, // South_Georgia
    {   56, 1656,       0,   76298,       0,  0, 12,  0, }, // Suriname
    {   56, 1676,       0,   76310,       0,  0, 14,  0, }, // Syowa
    {   56, 1682,       0,   76324,       0,  0, 11,  0, }, // Tahiti
    {   56, 1689,   76335,   76346,   76362, 11, 16, 19, }, // Taipei
    {   56, 1696,       0,   76381,       0,  0, 16,  0, }, // Tajikistan
    {   56, 1724,       0,   76397,       0,  0, 12,  0, }, // Tokelau
    {   56, 1732,   76409,   76420,   76436, 11, 16, 24, }, // Tonga
    {   56, 1738,       0,   76460,       0,  0,  8,  0, }, // Truk
    {   56, 1750,   76468,   76487,   76511, 19, 24, 32, }, // Turkmenistan
    {   56, 1763,       0,   76543,       0,  0, 11,  0, }, // Tuvalu
    {   56, 1777,   76554,   76567,   76585, 13, 18, 26, }, // Uruguay
    {   56, 1792,   76611,   76627,   76648, 16, 21, 29, }, // Uzbekistan
    {   56, 1803,   76677,   76691,   76710, 14, 19, 27, }, // Vanuatu
    {   56, 1811,       0,   76737,       0,  0, 16,  0, }, // Venezuela
    {   56, 1821,   76753,   76769,   76790, 16, 21, 29, }, // Vladivostok
    {   56, 1833,   76819,   76833,   76852, 14, 19, 27, }, // Volgograd
    {   56, 1843,       0,   76879,       0,  0,  9,  0, }, // Vostok
    {   56, 1850,       0,   76888,       0,  0, 16,  0, }, // Wake
    {   56, 1855,       0,   76904,       0,  0, 23,  0, }, // Wallis
    {   56, 1862,   76927,   76943,   76964, 16, 21, 29, }, // Yakutsk
    {   56, 1870,   76993,   77014,   77040, 21, 26, 34, }, // Yekaterinburg
    {   56, 1892,       0,   77074,       0,  0, 10,  0, }, // Yukon
    {   60,    0,   77084,   77096,   77118, 12, 22, 18, }, // Acre Basque/Latin/Spain
    {   60,    5,       0,   77136,       0,  0, 18,  0, }, // Afghanistan
    {   60,   17,       0,   77154,       0,  0, 25,  0, }, // Africa_Central
    {   60,   32,       0,   77179,       0,  0, 24,  0, }, // Africa_Eastern
    {   60,   65,       0,   77203,       0,  0, 25,  0, }, // Africa_Southern
    {   60,   81,   77228,   77255,   77292, 27, 37, 33, }, // Africa_Western
    {   60,  107,   77325,   77339,   77363, 14, 24, 20, }, // Alaska
    {   60,  128,   77383,   77397,   77421, 14, 24, 20, }, // Almaty
    {   60,  135,   77441,   77457,   77483, 16, 26, 22, }, // Amazon
    {   60,  142,   77505,   77536,   77577, 31, 41, 37, }, // America_Central
    {   60,  158,   77614,   77644,   77684, 30, 40, 36, }, // America_Eastern
    {   60,  174,   77720,   77752,   77794, 32, 42, 38, }, // America_Mountain
    {   60,  191,   77832,   77863,   77904, 31, 41, 37, }, // America_Pacific
    {   60,  207,   77941,   77957,   77983, 16, 26, 22, }, // Anadyr
    {   60,  214,   78005,   78017,   78039, 12, 22, 18, }, // Apia
    {   60,  219,   78057,   78070,   78093, 13, 23, 19, }, // Aqtau
    {   60,  225,   78112,   78126,   78150, 14, 24, 20, }, // Aqtobe
    {   60,  232,   78170,   78184,   78208, 14, 24, 20, }, // Arabian
    {   60,  240,   78228,   78245,   78272, 17, 27, 23, }, // Argentina
    {   60,  250,   78295,   78323,   78361, 28, 38, 34, }, // Argentina_Western
    {   60,  268,   78395,   78410,   78435, 15, 25, 21, }, // Armenia
    {   60,  286,   78456,   78488,   78530, 32, 42, 38, }, // Atlantic
    {   60,  295,   78568,   78596,   78634, 28, 38, 34, }, // Australia_Central
    {   60,  313,   78668,   78703,   78748, 35, 45, 41, }, // Australia_CentralWestern
    {   60,  338,   78789,   78816,   78853, 27, 37, 33, }, // Australia_Eastern
    {   60,  356,   78886,   78916,   78956, 30, 40, 36, }, // Australia_Western
    {   60,  374,   78992,   79010,   79038, 18, 28, 24, }, // Azerbaijan
    {   60,  385,   79062,   79078,   79104, 16, 26, 22, }, // Azores
    {   60,  397,   79126,   79145,   79174, 19, 29, 25, }, // Bangladesh
    {   60,  415,       0,   79199,       0,  0, 14,  0, }, // Bhutan
    {   60,  422,       0,   79213,       0,  0, 15,  0, }, // Bolivia
    {   60,  437,   79228,   79244,   79270, 16, 26, 22, }, // Brasilia
    {   60,  454,       0,   79292,       0,  0, 25,  0, }, // Brunei
    {   60,  461,   79317,   79335,   79363, 18, 28, 24, }, // Cape_Verde
    {   60,  472,       0,   79387,       0,  0, 13,  0, }, // Casey
    {   60,  478,       0,   79400,       0,  0, 26,  0, }, // Chamorro
    {   60,  487,   79426,   79441,   79466, 15, 25, 21, }, // Chatham
    {   60,  495,   79487,   79500,   79523, 13, 23, 19, }, // Chile
    {   60,  501,   79542,   79555,   79578, 13, 23, 19, }, // China
    {   60,  507,       0,   79597,       0,  0, 24,  0, }, // Christmas
    {   60,  517,       0,   79621,       0,  0, 23,  0, }, // Cocos
    {   60,  523,   79644,   79660,   79686, 16, 26, 22, }, // Colombia
    {   60,  532,   79708,   79730,   79762, 22, 32, 37, }, // Cook
    {   60,  537,   79799,   79811,   79833, 12, 22, 18, }, // Cuba
    {   60,  548,       0,   79851,       0,  0, 14,  0, }, // Davis
    {   60,  564,       0,   79865,       0,  0, 24,  0, }, // DumontDUrville
    {   60,  601,       0,   79889,       0,  0, 25,  0, }, // East_Timor
    {   60,  612,   79914,   79934,   79964, 20, 30, 26, }, // Easter
    {   60,  619,       0,   79990,       0,  0, 17,  0, }, // Ecuador
    {   60,  627,   80007,   80032,   80067, 25, 35, 31, }, // Europe_Central
    {   60,  642,   80098,   80122,   80156, 24, 34, 30, }, // Europe_Eastern
    {   60,  657,       0,   80186,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {   60,  680,   80217,   80244,   80281, 27, 37, 33, }, // Europe_Western
    {   60,  695,   80314,   80340,   80376, 26, 36, 32, }, // Falkland
    {   60,  704,   80408,   80420,   80442, 12, 22, 18, }, // Fiji
    {   60,  709,       0,   80460,       0,  0, 24,  0, }, // French_Guiana
    {   60,  723,       0,   80484,       0,  0, 57,  0, }, // French_Southern
    {   60,  746,       0,   80541,       0,  0, 19,  0, }, // Galapagos
    {   60,  756,       0,   80560,       0,  0, 19,  0, }, // Gambier
    {   60,  764,   80579,   80594,   80619, 15, 25, 21, }, // Georgia
    {   60,  772,       0,   80640,       0,  0, 25,  0, }, // Gilbert_Islands
    {   60,  788,       0,   80665,       0,  0, 31,  0, }, // GMT
    {   60,  830,   80696,   80725,   80764, 29, 39, 35, }, // Greenland_Eastern
    {   60,  848,   80799,   80831,   80873, 32, 42, 38, }, // Greenland_Western
    {   60,  866,       0,   80911,       0,  0, 23,  0, }, // Guam
    {   60,  871,       0,   80934,       0,  0, 23,  0, }, // Gulf
    {   60,  876,       0,   80957,       0,  0, 14,  0, }, // Guyana
    {   60,  883,   80971,   81004,   81047, 33, 43, 39, }, // Hawaii_Aleutian
    {   60,  899,   81086,   81102,   81128, 16, 26, 22, }, // Hong_Kong
    {   60,  909,   81150,   81164,   81188, 14, 24, 20, }, // Hovd
    {   60,  914,       0,   81208,       0,  0, 13,  0, }, // India
    {   60,  920,       0,   81221,       0,  0, 22,  0, }, // Indian_Ocean
    {   60,  933,       0,   81243,       0,  0, 17,  0, }, // Indochina
    {   60,  943,       0,   81260,       0,  0, 28,  0, }, // Indonesia_Central
    {   60,  961,       0,   81288,       0,  0, 27,  0, }, // Indonesia_Eastern
    {   60,  979,       0,   81315,       0,  0, 30,  0, }, // Indonesia_Western
    {   60,  997,   81345,   81357,   81379, 12, 22, 18, }, // Iran
    {   60, 1008,   81397,   81413,   81439, 16, 26, 22, }, // Irkutsk
    {   60, 1016,   81461,   81475,   81499, 14, 24, 20, }, // Israel
    {   60, 1023,   81519,   81534,   81559, 15, 25, 21, }, // Japan
    {   60, 1029,   81580,   81612,   81654, 32, 42, 38, }, // Kamchatka
    {   60, 1047,       0,   81692,       0,  0, 18,  0, }, // Kazakhstan
    {   60, 1058,       0,   81710,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {   60, 1077,       0,   81738,       0,  0, 31,  0, }, // Kazakhstan_Western
    {   60, 1106,   81769,   81782,   81805, 13, 23, 19, }, // Korea
    {   60, 1112,       0,   81824,       0,  0, 14,  0, }, // Kosrae
    {   60, 1119,   81838,   81858,   81888, 20, 30, 26, }, // Krasnoyarsk
    {   60, 1151,       0,   81914,       0,  0, 19,  0, }, // Kyrgystan
    {   60, 1161,       0,   81933,       0,  0, 13,  0, }, // Lanka
    {   60, 1175,       0,   81946,       0,  0, 22,  0, }, // Line_Islands
    {   60, 1188,   81968,   81985,   82012, 17, 27, 23, }, // Lord_Howe
    {   60, 1198,   82035,   82048,   82071, 13, 23, 19, }, // Macau
    {   60, 1204,   82090,   82105,   82130, 15, 25, 21, }, // Magadan
    {   60, 1219,       0,   82151,       0,  0, 16,  0, }, // Malaysia
    {   60, 1228,       0,   82167,       0,  0, 17,  0, }, // Maldives
    {   60, 1237,       0,   82184,       0,  0, 17,  0, }, // Marquesas
    {   60, 1247,       0,   82201,       0,  0, 26,  0, }, // Marshall_Islands
    {   60, 1264,   82227,   82243,   82269, 16, 26, 22, }, // Mauritius
    {   60, 1274,       0,   82291,       0,  0, 15,  0, }, // Mawson
    {   60, 1281,   82306,   82331,   82366, 25, 35, 31, }, // Mexico_Pacific
    {   60, 1296,   82397,   82417,   82447, 20, 30, 26, }, // Mongolia
    {   60, 1305,   82473,   82486,   82509, 13, 23, 19, }, // Moscow
    {   60, 1312,       0,   82528,       0,  0, 17,  0, }, // Myanmar
    {   60, 1320,       0,   82545,       0,  0, 13,  0, }, // Nauru
    {   60, 1326,       0,   82558,       0,  0, 13,  0, }, // Nepal
    {   60, 1332,   82571,   82594,   82627, 23, 33, 29, }, // New_Caledonia
    {   60, 1346,   82656,   82678,   82710, 22, 32, 28, }, // New_Zealand
    {   60, 1358,   82738,   82752,   82776, 14, 24, 20, }, // Newfoundland
    {   60, 1371,       0,   82796,       0,  0, 12,  0, }, // Niue
    {   60, 1376,   82808,   82833,   82868, 25, 35, 31, }, // Norfolk
    {   60, 1384,   82899,   82926,   82963, 27, 37, 33, }, // Noronha
    {   60, 1392,       0,   82996,       0,  0, 30,  0, }, // North_Mariana
    {   60, 1406,   83026,   83046,   83076, 20, 30, 26, }, // Novosibirsk
    {   60, 1418,   83102,   83115,   83138, 13, 23, 19, }, // Omsk
    {   60, 1428,   83157,   83173,   83199, 16, 26, 22, }, // Pakistan
    {   60, 1437,       0,   83221,       0,  0, 13,  0, }, // Palau
    {   60, 1443,       0,   83234,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   60, 1460,   83259,   83275,   83301, 16, 26, 22, }, // Paraguay
    {   60, 1469,   83323,   83335,   83357, 12, 22, 18, }, // Peru
    {   60, 1474,   83375,   83393,   83421, 18, 28, 24, }, // Philippines
    {   60, 1486,       0,   83445,       0,  0, 25,  0, }, // Phoenix_Islands
    {   60, 1502,   83470,   83503,   83546, 33, 43, 39, }, // Pierre_Miquelon
    {   60, 1518,       0,   83585,       0,  0, 17,  0, }, // Pitcairn
    {   60, 1527,       0,   83602,       0,  0, 14,  0, }, // Ponape
    {   60, 1534,       0,   83616,       0,  0, 18,  0, }, // Pyongyang
    {   60, 1544,   83634,   83651,   83678, 17, 27, 23, }, // Qyzylorda
    {   60, 1554,       0,   83701,       0,  0, 16,  0, }, // Reunion
    {   60, 1562,       0,   83717,       0,  0, 15,  0, }, // Rothera
    {   60, 1570,   83732,   83748,   83774, 16, 26, 22, }, // Sakhalin
    {   60, 1579,   83796,   83810,   83834, 14, 24, 20, }, // Samara
    {   60, 1596,   83854,   83867,   83890, 13, 23, 19, }, // Samoa
    {   60, 1602,       0,   83909,       0,  0, 27,  0, }, // Seychelles
    {   60, 1624,       0,   83936,       0,  0, 28,  0, }, // Singapore
    {   60, 1634,       0,   83964,       0,  0, 25,  0, }, // Solomon
    {   60, 1642,       0,   83989,       0,  0, 28,  0, }, // South_Georgia
    {   60, 1656,       0,   84017,       0,  0, 15,  0, }, // Suriname
    {   60, 1676,       0,   84032,       0,  0, 13,  0, }, // Syowa
    {   60, 1682,       0,   84045,       0,  0, 14,  0, }, // Tahiti
    {   60, 1689,   84059,   84073,   84097, 14, 24, 20, }, // Taipei
    {   60, 1696,       0,   84117,       0,  0, 19,  0, }, // Tajikistan
    {   60, 1724,       0,   84136,       0,  0, 15,  0, }, // Tokelau
    {   60, 1732,   84151,   84164,   84187, 13, 23, 19, }, // Tonga
    {   60, 1738,       0,   84206,       0,  0, 14,  0, }, // Truk
    {   60, 1750,   84220,   84240,   84270, 20, 30, 26, }, // Turkmenistan
    {   60, 1763,       0,   84296,       0,  0, 14,  0, }, // Tuvalu
    {   60, 1777,   84310,   84325,   84350, 15, 25, 21, }, // Uruguay
    {   60, 1792,   84371,   84389,   84417, 18, 28, 24, }, // Uzbekistan
    {   60, 1803,   84441,   84456,   84481, 15, 25, 21, }, // Vanuatu
    {   60, 1811,       0,   84502,       0,  0, 17,  0, }, // Venezuela
    {   60, 1821,   84519,   84539,   84569, 20, 30, 26, }, // Vladivostok
    {   60, 1833,   84595,   84613,   84641, 18, 28, 24, }, // Volgograd
    {   60, 1843,       0,   84665,       0,  0, 15,  0, }, // Vostok
    {   60, 1850,       0,   84680,       0,  0, 19,  0, }, // Wake
    {   60, 1855,       0,   84699,       0,  0, 29,  0, }, // Wallis
    {   60, 1862,   84728,   84744,   84770, 16, 26, 22, }, // Yakutsk
    {   60, 1870,   84792,   84814,   84846, 22, 32, 28, }, // Yekaterinburg
    {   60, 1892,       0,   84874,       0,  0, 13,  0, }, // Yukon
    {   61,    5,       0,   84887,       0,  0, 17,  0, }, // Afghanistan Belarusian/Cyrillic/Belarus
    {   61,   17,       0,   84904,       0,  0, 24,  0, }, // Africa_Central
    {   61,   32,       0,   84928,       0,  0, 21,  0, }, // Africa_Eastern
    {   61,   65,       0,   84949,       0,  0, 22,  0, }, // Africa_Southern
    {   61,   81,   84971,   84992,   85024, 21, 32, 27, }, // Africa_Western
    {   61,  107,   85051,   85061,   85082, 10, 21, 16, }, // Alaska
    {   61,  135,   85098,   85111,   85135, 13, 24, 19, }, // Amazon
    {   61,  142,   85154,   85188,   85233, 34, 45, 40, }, // America_Central
    {   61,  158,   85273,   85304,   85346, 31, 42, 37, }, // America_Eastern
    {   61,  174,   85383,   85412,   85452, 29, 40, 35, }, // America_Mountain
    {   61,  191,   85487,   85503,   85530, 16, 27, 22, }, // America_Pacific
    {   61,  214,   85552,   85560,   85579,  8, 19, 14, }, // Apia
    {   61,  232,   85593,   85614,   85646, 21, 32, 27, }, // Arabian
    {   61,  240,   85673,   85688,   85714, 15, 26, 21, }, // Argentina
    {   61,  250,   85735,   85757,   85790, 22, 33, 28, }, // Argentina_Western
    {   61,  268,   85818,   85829,   85851, 11, 22, 17, }, // Armenia
    {   61,  286,   85868,   85882,   85907, 14, 25, 20, }, // Atlantic
    {   61,  295,   85927,   85952,   85988, 25, 36, 31, }, // Australia_Central
    {   61,  313,   86019,   86051,   86094, 32, 43, 38, }, // Australia_CentralWestern
    {   61,  338,   86132,   86154,   86187, 22, 33, 28, }, // Australia_Eastern
    {   61,  356,   86215,   86237,   86270, 22, 33, 28, }, // Australia_Western
    {   61,  374,   86298,   86314,   86341, 16, 27, 22, }, // Azerbaijan
    {   61,  385,   86363,   86384,   86416, 21, 32, 27, }, // Azores
    {   61,  397,   86443,   86456,   86480, 13, 24, 19, }, // Bangladesh
    {   61,  415,       0,   86499,       0,  0, 10,  0, }, // Bhutan
    {   61,  422,       0,   86509,       0,  0, 14,  0, }, // Bolivia
    {   61,  437,   86523,   86537,   86562, 14, 25, 20, }, // Brasilia
    {   61,  454,       0,   86582,       0,  0, 10,  0, }, // Brunei
    {   61,  461,   86592,   86606,   86631, 14, 25, 20, }, // Cape_Verde
    {   61,  478,       0,   86651,       0,  0, 10,  0, }, // Chamorro
    {   61,  487,   86661,   86671,   86692, 10, 21, 16, }, // Chatham
    {   61,  495,   86708,   86720,   86743, 12, 23, 18, }, // Chile
    {   61,  501,   86761,   86770,   86790,  9, 20, 15, }, // China
    {   61,  507,       0,   86805,       0,  0, 18,  0, }, // Christmas
    {   61,  517,       0,   86823,       0,  0, 22,  0, }, // Cocos
    {   61,  523,   86845,   86860,   86886, 15, 26, 21, }, // Colombia
    {   61,  532,   86907,   86924,   86952, 17, 28, 26, }, // Cook
    {   61,  537,   86978,   86986,   87005,  8, 19, 14, }, // Cuba
    {   61,  548,       0,   87019,       0,  0, 18,  0, }, // Davis
    {   61,  564,       0,   87037,       0,  0, 27,  0, }, // DumontDUrville
    {   61,  601,       0,   87064,       0,  0, 20,  0, }, // East_Timor
    {   61,  612,   87084,   87106,   87139, 22, 33, 28, }, // Easter
    {   61,  619,       0,   87167,       0,  0, 14,  0, }, // Ecuador
    {   61,  627,   87181,   87205,   87240, 24, 35, 30, }, // Europe_Central
    {   61,  642,   87270,   87291,   87323, 21, 32, 27, }, // Europe_Eastern
    {   61,  657,       0,   87350,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {   61,  680,   87377,   87398,   87430, 21, 32, 27, }, // Europe_Western
    {   61,  695,   87457,   87482,   87518, 25, 36, 31, }, // Falkland
    {   61,  704,   87549,   87558,   87578,  9, 20, 15, }, // Fiji
    {   61,  709,       0,   87593,       0,  0, 22,  0, }, // French_Guiana
    {   61,  723,       0,   87615,       0,  0, 47,  0, }, // French_Southern
    {   61,  746,       0,   87662,       0,  0, 36,  0, }, // Galapagos
    {   61,  756,       0,   87698,       0,  0, 19,  0, }, // Gambier
    {   61,  764,   87717,   87730,   87754, 13, 24, 19, }, // Georgia
    {   61,  772,       0,   87773,       0,  0, 21,  0, }, // Gilbert_Islands
    {   61,  788,       0,   87794,       0,  0, 15,  0, }, // GMT
    {   61,  830,   87809,   87832,   87866, 23, 34, 29, }, // Greenland_Eastern
    {   61,  848,   87895,   87918,   87952, 23, 34, 29, }, // Greenland_Western
    {   61,  871,       0,   87981,       0,  0, 22,  0, }, // Gulf
    {   61,  876,       0,   88003,       0,  0,  9,  0, }, // Guyana
    {   61,  883,   88012,   88032,   88063, 20, 31, 26, }, // Hawaii_Aleutian
    {   61,  899,   88089,   88101,   88124, 12, 23, 18, }, // Hong_Kong
    {   61,  909,   88142,   88151,   88171,  9, 20, 15, }, // Hovd
    {   61,  914,       0,   88186,       0,  0,  9,  0, }, // India
    {   61,  920,       0,   88195,       0,  0, 21,  0, }, // Indian_Ocean
    {   61,  933,       0,   88216,       0,  0, 16,  0, }, // Indochina
    {   61,  943,       0,   88232,       0,  0, 26,  0, }, // Indonesia_Central
    {   61,  961,       0,   88258,       0,  0, 23,  0, }, // Indonesia_Eastern
    {   61,  979,       0,   88281,       0,  0, 23,  0, }, // Indonesia_Western
    {   61,  997,   88304,   88315,   88337, 11, 22, 17, }, // Iran
    {   61, 1008,   88354,   88365,   88387, 11, 22, 17, }, // Irkutsk
    {   61, 1016,   88404,   88418,   88443, 14, 25, 20, }, // Israel
    {   61, 1023,   88463,   88473,   88494, 10, 21, 16, }, // Japan
    {   61, 1047,       0,   88510,       0,  0, 16,  0, }, // Kazakhstan
    {   61, 1058,       0,   88526,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {   61, 1077,       0,   88549,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   61, 1106,   88572,   88581,   88601,  9, 20, 15, }, // Korea
    {   61, 1112,       0,   88616,       0,  0, 19,  0, }, // Kosrae
    {   61, 1119,   88635,   88650,   88676, 15, 26, 21, }, // Krasnoyarsk
    {   61, 1151,       0,   88697,       0,  0, 15,  0, }, // Kyrgystan
    {   61, 1175,       0,   88712,       0,  0, 17,  0, }, // Line_Islands
    {   61, 1188,   88729,   88741,   88764, 12, 23, 18, }, // Lord_Howe
    {   61, 1204,   88782,   88796,   88821, 14, 25, 20, }, // Magadan
    {   61, 1219,       0,   88841,       0,  0, 12,  0, }, // Malaysia
    {   61, 1228,       0,   88853,       0,  0, 13,  0, }, // Maldives
    {   61, 1237,       0,   88866,       0,  0, 23,  0, }, // Marquesas
    {   61, 1247,       0,   88889,       0,  0, 23,  0, }, // Marshall_Islands
    {   61, 1264,   88912,   88924,   88947, 12, 23, 18, }, // Mauritius
    {   61, 1274,       0,   88965,       0,  0, 18,  0, }, // Mawson
    {   61, 1281,   88983,   89011,   89049, 28, 38, 34, }, // Mexico_Pacific
    {   61, 1296,   89083,   89098,   89124, 15, 26, 21, }, // Mongolia
    {   61, 1305,   89145,   89158,   89182, 13, 24, 19, }, // Moscow
    {   61, 1312,       0,   89201,       0,  0, 10,  0, }, // Myanmar
    {   61, 1320,       0,   89211,       0,  0,  9,  0, }, // Nauru
    {   61, 1326,       0,   89220,       0,  0, 13,  0, }, // Nepal
    {   61, 1332,   89233,   89252,   89282, 19, 30, 25, }, // New_Caledonia
    {   61, 1346,   89307,   89325,   89354, 18, 29, 24, }, // New_Zealand
    {   61, 1358,   89378,   89397,   89427, 19, 30, 25, }, // Newfoundland
    {   61, 1371,       0,   89452,       0,  0,  8,  0, }, // Niue
    {   61, 1376,   89460,   89480,   89511, 20, 31, 26, }, // Norfolk
    {   61, 1384,   89537,   89561,   89596, 24, 35, 30, }, // Noronha
    {   61, 1406,   89626,   89642,   89669, 16, 27, 22, }, // Novosibirsk
    {   61, 1418,   89691,   89700,   89720,  9, 20, 15, }, // Omsk
    {   61, 1428,   89735,   89750,   89776, 15, 26, 21, }, // Pakistan
    {   61, 1437,       0,   89797,       0,  0,  9,  0, }, // Palau
    {   61, 1443,       0,   89806,       0,  0, 22,  0, }, // Papua_New_Guinea
    {   61, 1460,   89828,   89840,   89863, 12, 23, 18, }, // Paraguay
    {   61, 1469,   89881,   89894,   89918, 13, 24, 19, }, // Peru
    {   61, 1474,   89937,   89951,   89976, 14, 25, 20, }, // Philippines
    {   61, 1486,       0,   89996,       0,  0, 19,  0, }, // Phoenix_Islands
    {   61, 1502,   90015,   90037,   90070, 22, 33, 39, }, // Pierre_Miquelon
    {   61, 1518,       0,   90109,       0,  0, 20,  0, }, // Pitcairn
    {   61, 1527,       0,   90129,       0,  0, 19,  0, }, // Ponape
    {   61, 1534,       0,   90148,       0,  0, 14,  0, }, // Pyongyang
    {   61, 1554,       0,   90162,       0,  0, 12,  0, }, // Reunion
    {   61, 1562,       0,   90174,       0,  0, 18,  0, }, // Rothera
    {   61, 1570,   90192,   90206,   90231, 14, 25, 20, }, // Sakhalin
    {   61, 1596,   90251,   90260,   90280,  9, 20, 15, }, // Samoa
    {   61, 1602,       0,   90295,       0,  0, 24,  0, }, // Seychelles
    {   61, 1624,       0,   90319,       0,  0, 15,  0, }, // Singapore
    {   61, 1634,       0,   90334,       0,  0, 24,  0, }, // Solomon
    {   61, 1642,       0,   90358,       0,  0, 21,  0, }, // South_Georgia
    {   61, 1656,       0,   90379,       0,  0, 12,  0, }, // Suriname
    {   61, 1676,       0,   90391,       0,  0, 16,  0, }, // Syowa
    {   61, 1682,       0,   90407,       0,  0,  9,  0, }, // Tahiti
    {   61, 1689,   90416,   90426,   90447, 10, 21, 16, }, // Taipei
    {   61, 1696,       0,   90463,       0,  0, 16,  0, }, // Tajikistan
    {   61, 1724,       0,   90479,       0,  0, 11,  0, }, // Tokelau
    {   61, 1732,   90490,   90499,   90519,  9, 20, 15, }, // Tonga
    {   61, 1738,       0,   90534,       0,  0,  9,  0, }, // Truk
    {   61, 1750,   90543,   90560,   90588, 17, 28, 23, }, // Turkmenistan
    {   61, 1763,       0,   90611,       0,  0, 10,  0, }, // Tuvalu
    {   61, 1777,   90621,   90635,   90660, 14, 25, 20, }, // Uruguay
    {   61, 1792,   90680,   90695,   90721, 15, 26, 21, }, // Uzbekistan
    {   61, 1803,   90742,   90753,   90775, 11, 22, 17, }, // Vanuatu
    {   61, 1811,       0,   90792,       0,  0, 16,  0, }, // Venezuela
    {   61, 1821,   90808,   90826,   90855, 18, 29, 24, }, // Vladivostok
    {   61, 1833,   90879,   90895,   90922, 16, 27, 22, }, // Volgograd
    {   61, 1843,       0,   90944,       0,  0, 18,  0, }, // Vostok
    {   61, 1850,       0,   90962,       0,  0, 17,  0, }, // Wake
    {   61, 1855,       0,   90979,       0,  0, 27,  0, }, // Wallis
    {   61, 1862,   91006,   91016,   91037, 10, 21, 16, }, // Yakutsk
    {   61, 1870,   91053,   91072,   91102, 19, 30, 25, }, // Yekaterinburg
    {   61, 1892,       0,   91127,       0,  0,  9,  0, }, // Yukon
    {   64,  788,       0,   91136,       0,  0, 17,  0, }, // GMT Bhojpuri/Devanagari/India
    {   66,    0,   91153,   91160,   91178,  7, 18, 12, }, // Acre Bodo/Devanagari/India
    {   66,    5,       0,   91190,       0,  0, 14,  0, }, // Afghanistan
    {   66,   17,       0,   91204,       0,  0, 15,  0, }, // Africa_Central
    {   66,   32,       0,   91219,       0,  0, 16,  0, }, // Africa_Eastern
    {   66,   65,       0,   91235,       0,  0, 25,  0, }, // Africa_Southern
    {   66,   81,   91260,   91278,   91304, 18, 26, 26, }, // Africa_Western
    {   66,  107,   91330,   91342,   91364, 12, 22, 19, }, // Alaska
    {   66,  128,   91383,   91394,   91415, 11, 21, 15, }, // Almaty
    {   66,  135,   91430,   91438,   91454,  8, 16, 16, }, // Amazon
    {   66,  142,   91470,   91492,   91525, 22, 33, 29, }, // America_Central
    {   66,  158,   91554,   91581,   91616, 27, 35, 34, }, // America_Eastern
    {   66,  174,   91650,   91678,   91714, 28, 36, 35, }, // America_Mountain
    {   66,  191,   91749,   91774,   91807, 25, 33, 32, }, // America_Pacific
    {   66,  207,   91839,   91848,   91869,  9, 21, 15, }, // Anadyr
    {   66,  214,   91884,   91892,   91908,  8, 16, 12, }, // Apia
    {   66,  219,   91920,   91929,   91951,  9, 22, 16, }, // Aqtau
    {   66,  225,   91967,   91976,   91999,  9, 23, 17, }, // Aqtobe
    {   66,  232,   92016,   92027,   92046, 11, 19, 18, }, // Arabian
    {   66,  240,   92064,   92078,   92100, 14, 22, 22, }, // Argentina
    {   66,  250,   92122,   92142,   92170, 20, 28, 28, }, // Argentina_Western
    {   66,  268,   92198,   92210,   92230, 12, 20, 20, }, // Armenia
    {   66,  286,   92250,   92262,   92282, 12, 20, 19, }, // Atlantic
    {   66,  295,   92301,   92321,   92351, 20, 30, 27, }, // Australia_Central
    {   66,  313,   92378,   92408,   92446, 30, 38, 37, }, // Australia_CentralWestern
    {   66,  338,   92483,   92507,   92541, 24, 34, 33, }, // Australia_Eastern
    {   66,  356,   92574,   92597,   92630, 23, 33, 32, }, // Australia_Western
    {   66,  374,   92662,   92675,   92696, 13, 21, 21, }, // Azerbaijan
    {   66,  385,   92717,   92728,   92747, 11, 19, 19, }, // Azores
    {   66,  397,   92766,   92777,   92796, 11, 19, 19, }, // Bangladesh
    {   66,  415,       0,   92815,       0,  0,  8,  0, }, // Bhutan
    {   66,  422,       0,   92823,       0,  0, 10,  0, }, // Bolivia
    {   66,  437,   92833,   92846,   92867, 13, 21, 21, }, // Brasilia
    {   66,  454,       0,   92888,       0,  0, 33,  0, }, // Brunei
    {   66,  461,   92921,   92934,   92955, 13, 21, 21, }, // Cape_Verde
    {   66,  478,       0,   92976,       0,  0, 19,  0, }, // Chamorro
    {   66,  487,   92995,   93003,   93019,  8, 16, 15, }, // Chatham
    {   66,  495,   93034,   93041,   93056,  7, 15, 15, }, // Chile
    {   66,  501,   93071,   93079,   93095,  8, 16, 15, }, // China
    {   66,  507,       0,   93110,       0,  0, 19,  0, }, // Christmas
    {   66,  517,       0,   93129,       0,  0, 18,  0, }, // Cocos
    {   66,  523,   93147,   93158,   93177, 11, 19, 19, }, // Colombia
    {   66,  532,   93196,   93213,   93238, 17, 25, 31, }, // Cook
    {   66,  537,   93269,   93277,   93293,  8, 16, 15, }, // Cuba
    {   66,  548,       0,   93308,       0,  0,  8,  0, }, // Davis
    {   66,  564,       0,   93316,       0,  0, 18,  0, }, // DumontDUrville
    {   66,  601,       0,   93334,       0,  0, 25,  0, }, // East_Timor
    {   66,  612,   93359,   93377,   93403, 18, 26, 26, }, // Easter
    {   66,  619,       0,   93429,       0,  0, 10,  0, }, // Ecuador
    {   66,  627,   93439,   93454,   93477, 15, 23, 23, }, // Europe_Central
    {   66,  642,   93500,   93520,   93548, 20, 28, 28, }, // Europe_Eastern
    {   66,  657,       0,   93576,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {   66,  680,   93605,   93621,   93645, 16, 24, 24, }, // Europe_Western
    {   66,  695,   93669,   93690,   93719, 21, 29, 29, }, // Falkland
    {   66,  704,   93748,   93755,   93770,  7, 15, 15, }, // Fiji
    {   66,  709,       0,   93785,       0,  0, 18,  0, }, // French_Guiana
    {   66,  723,       0,   93803,       0,  0, 35,  0, }, // French_Southern
    {   66,  746,       0,   93838,       0,  0, 12,  0, }, // Galapagos
    {   66,  756,       0,   93850,       0,  0, 12,  0, }, // Gambier
    {   66,  764,   93862,   93872,   93890, 10, 18, 18, }, // Georgia
    {   66,  772,       0,   93908,       0,  0, 22,  0, }, // Gilbert_Islands
    {   66,  788,       0,   93930,       0,  0, 20,  0, }, // GMT
    {   66,  830,   93950,   93969,   93996, 19, 27, 28, }, // Greenland_Eastern
    {   66,  848,   94024,   94043,   94070, 19, 27, 27, }, // Greenland_Western
    {   66,  866,       0,   94097,       0,  0, 19,  0, }, // Guam
    {   66,  871,       0,   94116,       0,  0, 16,  0, }, // Gulf
    {   66,  876,       0,   94132,       0,  0,  9,  0, }, // Guyana
    {   66,  883,   94141,   94160,   94187, 19, 27, 26, }, // Hawaii_Aleutian
    {   66,  899,   94213,   94220,   94235,  7, 15, 15, }, // Hong_Kong
    {   66,  909,   94250,   94258,   94274,  8, 16, 16, }, // Hovd
    {   66,  914,       0,   94290,       0,  0, 18,  0, }, // India
    {   66,  920,       0,   94308,       0,  0, 18,  0, }, // Indian_Ocean
    {   66,  933,       0,   94326,       0,  0, 24,  0, }, // Indochina
    {   66,  943,       0,   94350,       0,  0, 25,  0, }, // Indonesia_Central
    {   66,  961,       0,   94375,       0,  0, 33,  0, }, // Indonesia_Eastern
    {   66,  979,       0,   94408,       0,  0, 34,  0, }, // Indonesia_Western
    {   66,  997,   94442,   94449,   94464,  7, 15, 14, }, // Iran
    {   66, 1008,   94478,   94491,   94512, 13, 21, 22, }, // Irkutsk
    {   66, 1016,   94534,   94544,   94562, 10, 18, 17, }, // Israel
    {   66, 1023,   94579,   94587,   94603,  8, 16, 15, }, // Japan
    {   66, 1029,   94618,   94650,   94687, 32, 37, 31, }, // Kamchatka
    {   66, 1058,       0,   94718,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {   66, 1077,       0,   94737,       0,  0, 19,  0, }, // Kazakhstan_Western
    {   66, 1106,   94756,   94765,   94782,  9, 17, 16, }, // Korea
    {   66, 1112,       0,   94798,       0,  0,  8,  0, }, // Kosrae
    {   66, 1119,   94806,   94824,   94850, 18, 26, 27, }, // Krasnoyarsk
    {   66, 1151,       0,   94877,       0,  0, 14,  0, }, // Kyrgystan
    {   66, 1161,       0,   94891,       0,  0, 19,  0, }, // Lanka
    {   66, 1175,       0,   94910,       0,  0, 18,  0, }, // Line_Islands
    {   66, 1188,   94928,   94940,   94960, 12, 20, 19, }, // Lord_Howe
    {   66, 1198,   94979,   94987,   95007,  8, 20, 14, }, // Macau
    {   66, 1204,   95021,   95031,   95049, 10, 18, 18, }, // Magadan
    {   66, 1219,       0,   95067,       0,  0, 22,  0, }, // Malaysia
    {   66, 1228,       0,   95089,       0,  0, 11,  0, }, // Maldives
    {   66, 1237,       0,   95100,       0,  0, 12,  0, }, // Marquesas
    {   66, 1247,       0,   95112,       0,  0, 21,  0, }, // Marshall_Islands
    {   66, 1264,   95133,   95147,   95169, 14, 22, 22, }, // Mauritius
    {   66, 1274,       0,   95191,       0,  0,  7,  0, }, // Mawson
    {   66, 1281,   95198,   95219,   95248, 21, 29, 28, }, // Mexico_Pacific
    {   66, 1296,   95276,   95288,   95308, 12, 20, 20, }, // Mongolia
    {   66, 1305,   95328,   95336,   95352,  8, 16, 16, }, // Moscow
    {   66, 1312,       0,   95368,       0,  0, 23,  0, }, // Myanmar
    {   66, 1320,       0,   95391,       0,  0,  8,  0, }, // Nauru
    {   66, 1326,       0,   95399,       0,  0,  8,  0, }, // Nepal
    {   66, 1332,   95407,   95425,   95451, 18, 26, 26, }, // New_Caledonia
    {   66, 1346,   95477,   95490,   95511, 13, 21, 20, }, // New_Zealand
    {   66, 1358,   95531,   95548,   95573, 17, 25, 24, }, // Newfoundland
    {   66, 1371,       0,   95597,       0,  0,  7,  0, }, // Niue
    {   66, 1376,   95604,   95618,   95640, 14, 22, 21, }, // Norfolk
    {   66, 1384,   95661,   95684,   95715, 23, 31, 31, }, // Noronha
    {   66, 1392,       0,   95746,       0,  0, 28,  0, }, // North_Mariana
    {   66, 1406,   95774,   95789,   95812, 15, 23, 23, }, // Novosibirsk
    {   66, 1418,   95835,   95844,   95861,  9, 17, 17, }, // Omsk
    {   66, 1428,   95878,   95890,   95910, 12, 20, 20, }, // Pakistan
    {   66, 1437,       0,   95930,       0,  0,  8,  0, }, // Palau
    {   66, 1443,       0,   95938,       0,  0, 17,  0, }, // Papua_New_Guinea
    {   66, 1460,   95955,   95966,   95985, 11, 19, 19, }, // Paraguay
    {   66, 1469,   96004,   96011,   96026,  7, 15, 15, }, // Peru
    {   66, 1474,   96041,   96053,   96077, 12, 24, 18, }, // Philippines
    {   66, 1486,       0,   96095,       0,  0, 20,  0, }, // Phoenix_Islands
    {   66, 1502,   96115,   96142,   96177, 27, 35, 34, }, // Pierre_Miquelon
    {   66, 1518,       0,   96211,       0,  0, 11,  0, }, // Pitcairn
    {   66, 1527,       0,   96222,       0,  0,  8,  0, }, // Ponape
    {   66, 1534,       0,   96230,       0,  0, 10,  0, }, // Pyongyang
    {   66, 1544,   96240,   96257,   96284, 17, 27, 21, }, // Qyzylorda
    {   66, 1554,       0,   96305,       0,  0, 11,  0, }, // Reunion
    {   66, 1562,       0,   96316,       0,  0,  8,  0, }, // Rothera
    {   66, 1570,   96324,   96334,   96352, 10, 18, 18, }, // Sakhalin
    {   66, 1579,   96370,   96380,   96400, 10, 20, 14, }, // Samara
    {   66, 1596,   96414,   96423,   96440,  9, 17, 16, }, // Samoa
    {   66, 1602,       0,   96456,       0,  0, 10,  0, }, // Seychelles
    {   66, 1624,       0,   96466,       0,  0, 23,  0, }, // Singapore
    {   66, 1634,       0,   96489,       0,  0, 19,  0, }, // Solomon
    {   66, 1642,       0,   96508,       0,  0, 15,  0, }, // South_Georgia
    {   66, 1656,       0,   96523,       0,  0, 10,  0, }, // Suriname
    {   66, 1676,       0,   96533,       0,  0,  8,  0, }, // Syowa
    {   66, 1682,       0,   96541,       0,  0,  9,  0, }, // Tahiti
    {   66, 1689,   96550,   96559,   96576,  9, 17, 16, }, // Taipei
    {   66, 1696,       0,   96592,       0,  0, 14,  0, }, // Tajikistan
    {   66, 1724,       0,   96606,       0,  0,  8,  0, }, // Tokelau
    {   66, 1732,   96614,   96621,   96636,  7, 15, 15, }, // Tonga
    {   66, 1738,       0,   96651,       0,  0,  6,  0, }, // Truk
    {   66, 1750,   96657,   96674,   96699, 17, 25, 25, }, // Turkmenistan
    {   66, 1763,       0,   96724,       0,  0,  9,  0, }, // Tuvalu
    {   66, 1777,   96733,   96743,   96761, 10, 18, 18, }, // Uruguay
    {   66, 1792,   96779,   96793,   96815, 14, 22, 22, }, // Uzbekistan
    {   66, 1803,   96837,   96847,   96865, 10, 18, 18, }, // Vanuatu
    {   66, 1811,       0,   96883,       0,  0, 13,  0, }, // Venezuela
    {   66, 1821,   96896,   96911,   96934, 15, 23, 23, }, // Vladivostok
    {   66, 1833,   96957,   96970,   96991, 13, 21, 21, }, // Volgograd
    {   66, 1843,       0,   96902,       0,  0,  9,  0, }, // Vostok
    {   66, 1850,       0,   97012,       0,  0, 12,  0, }, // Wake
    {   66, 1855,       0,   97024,       0,  0, 19,  0, }, // Wallis
    {   66, 1862,   97043,   97055,   97075, 12, 20, 20, }, // Yakutsk
    {   66, 1870,   97095,   97113,   97139, 18, 26, 26, }, // Yekaterinburg
    {   66, 1892,       0,   97165,       0,  0,  7,  0, }, // Yukon
    {   67,    0,   97172,   97182,   97203, 10, 21, 29, }, // Acre Bosnian/Latin/Bosnia and Herzegovina
    {   67,    5,       0,   97232,       0,  0, 21,  0, }, // Afghanistan
    {   67,   17,       0,   97253,       0,  0, 24,  0, }, // Africa_Central
    {   67,   32,       0,   97277,       0,  0, 22,  0, }, // Africa_Eastern
    {   67,   65,       0,   97299,       0,  0, 31,  0, }, // Africa_Southern
    {   67,   81,   97330,   97352,   97385, 22, 33, 29, }, // Africa_Western
    {   67,  107,   97414,   97433,   97463, 19, 30, 26, }, // Alaska
    {   67,  128,   97489,   97501,   97524, 12, 23, 31, }, // Almaty
    {   67,  135,   97555,   97572,   97600, 17, 28, 24, }, // Amazon
    {   67,  142,   97624,   97658,   97703, 34, 45, 41, }, // America_Central
    {   67,  158,   97744,   97776,   97819, 32, 43, 39, }, // America_Eastern
    {   67,  174,   97858,   97892,   97937, 34, 45, 41, }, // America_Mountain
    {   67,  191,   97978,   98012,   98057, 34, 45, 41, }, // America_Pacific
    {   67,  207,   98098,   98110,   98133, 12, 23, 31, }, // Anadyr
    {   67,  214,   98164,   98179,   98205, 15, 26, 22, }, // Apia
    {   67,  219,   98227,   98239,   98262, 12, 23, 31, }, // Aqtau
    {   67,  225,   98293,   98306,   98330, 13, 24, 32, }, // Aqtobe
    {   67,  232,   98362,   98379,   98407, 17, 28, 24, }, // Arabian
    {   67,  240,   98431,   98450,   98480, 19, 30, 26, }, // Argentina
    {   67,  250,   98506,   98532,   98569, 26, 37, 33, }, // Argentina_Western
    {   67,  268,   98602,   98618,   98645, 16, 27, 23, }, // Armenia
    {   67,  286,   98668,   98702,   98747, 34, 45, 41, }, // Atlantic
    {   67,  295,   98788,   98817,   98857, 29, 40, 36, }, // Australia_Central
    {   67,  313,   98893,   98931,   98979, 38, 48, 44, }, // Australia_CentralWestern
    {   67,  338,   99023,   99050,   99088, 27, 38, 34, }, // Australia_Eastern
    {   67,  356,   99122,   99149,   99187, 27, 38, 34, }, // Australia_Western
    {   67,  374,   99221,   99243,   99276, 22, 33, 29, }, // Azerbaijan
    {   67,  385,   99305,   99320,   99346, 15, 26, 22, }, // Azores
    {   67,  397,   99368,   99387,   99417, 19, 30, 26, }, // Bangladesh
    {   67,  415,       0,   99443,       0,  0, 16,  0, }, // Bhutan
    {   67,  422,       0,   99459,       0,  0, 18,  0, }, // Bolivia
    {   67,  437,   99477,   99496,   99526, 19, 30, 26, }, // Brasilia
    {   67,  454,       0,   99552,       0,  0, 17,  0, }, // Brunei
    {   67,  461,   99569,   99588,   99618, 19, 30, 26, }, // Cape_Verde
    {   67,  478,       0,   99644,       0,  0, 27,  0, }, // Chamorro
    {   67,  487,   99671,   99687,   99714, 16, 27, 23, }, // Chatham
    {   67,  495,   99737,   99754,   99782, 17, 28, 24, }, // Chile
    {   67,  501,   99806,   99821,   99847, 15, 26, 22, }, // China
    {   67,  507,       0,   99869,       0,  0, 26,  0, }, // Christmas
    {   67,  517,       0,   99895,       0,  0, 25,  0, }, // Cocos
    {   67,  523,   99920,   99939,   99969, 19, 30, 26, }, // Colombia
    {   67,  532,   99995,  100022,  100060, 27, 38, 38, }, // Cook
    {   67,  537,  100098,  100114,  100141, 16, 27, 23, }, // Cuba
    {   67,  548,       0,  100164,       0,  0, 21,  0, }, // Davis
    {   67,  564,       0,  100185,       0,  0, 32,  0, }, // DumontDUrville
    {   67,  601,       0,  100217,       0,  0, 23,  0, }, // East_Timor
    {   67,  612,  100240,  100264,  100299, 24, 35, 31, }, // Easter
    {   67,  619,       0,  100330,       0,  0, 18,  0, }, // Ecuador
    {   67,  627,  100348,  100373,  100409, 25, 36, 32, }, // Europe_Central
    {   67,  642,  100441,  100464,  100498, 23, 34, 30, }, // Europe_Eastern
    {   67,  657,       0,  100528,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {   67,  680,  100557,  100580,  100614, 23, 34, 30, }, // Europe_Western
    {   67,  695,  100644,  100663,  100693, 19, 30, 26, }, // Falkland
    {   67,  704,  100719,  100737,  100766, 18, 29, 24, }, // Fiji
    {   67,  709,       0,  100790,       0,  0, 26,  0, }, // French_Guiana
    {   67,  723,       0,  100816,       0,  0, 52,  0, }, // French_Southern
    {   67,  746,       0,  100868,       0,  0, 19,  0, }, // Galapagos
    {   67,  756,       0,  100887,       0,  0, 19,  0, }, // Gambier
    {   67,  764,  100906,  100923,  100951, 17, 28, 24, }, // Georgia
    {   67,  772,       0,  100975,       0,  0, 31,  0, }, // Gilbert_Islands
    {   67,  788,       0,  101006,       0,  0, 16,  0, }, // GMT
    {   67,  830,  101022,  101048,  101085, 26, 37, 33, }, // Greenland_Eastern
    {   67,  848,  101118,  101144,  101181, 26, 37, 33, }, // Greenland_Western
    {   67,  866,       0,  101214,       0,  0, 21,  0, }, // Guam
    {   67,  871,       0,  101235,       0,  0, 27,  0, }, // Gulf
    {   67,  876,       0,  101262,       0,  0, 17,  0, }, // Guyana
    {   67,  883,  101279,  101306,  101344, 27, 38, 34, }, // Hawaii_Aleutian
    {   67,  899,  101378,  101396,  101425, 18, 29, 25, }, // Hong_Kong
    {   67,  909,  101450,  101465,  101491, 15, 26, 22, }, // Hovd
    {   67,  914,       0,  101513,       0,  0, 27,  0, }, // India
    {   67,  920,       0,  101540,       0,  0, 27,  0, }, // Indian_Ocean
    {   67,  933,       0,  101567,       0,  0, 19,  0, }, // Indochina
    {   67,  943,       0,  101586,       0,  0, 29,  0, }, // Indonesia_Central
    {   67,  961,       0,  101615,       0,  0, 27,  0, }, // Indonesia_Eastern
    {   67,  979,       0,  101642,       0,  0, 27,  0, }, // Indonesia_Western
    {   67,  997,  101669,  101684,  101710, 15, 26, 22, }, // Iran
    {   67, 1008,  101732,  101748,  101775, 16, 27, 23, }, // Irkutsk
    {   67, 1016,  101798,  101815,  101843, 17, 28, 24, }, // Israel
    {   67, 1023,  101867,  101883,  101910, 16, 27, 23, }, // Japan
    {   67, 1029,  101933,  101962,  102002, 29, 40, 48, }, // Kamchatka
    {   67, 1047,       0,  102050,       0,  0, 20,  0, }, // Kazakhstan
    {   67, 1058,       0,  102070,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {   67, 1077,       0,  102097,       0,  0, 27,  0, }, // Kazakhstan_Western
    {   67, 1106,  102124,  102140,  102167, 16, 27, 23, }, // Korea
    {   67, 1112,       0,  102190,       0,  0, 24,  0, }, // Kosrae
    {   67, 1119,  102214,  102234,  102265, 20, 31, 27, }, // Krasnoyarsk
    {   67, 1151,       0,  102292,       0,  0, 20,  0, }, // Kyrgystan
    {   67, 1161,       0,  102312,       0,  0, 11,  0, }, // Lanka
    {   67, 1175,       0,  102323,       0,  0, 24,  0, }, // Line_Islands
    {   67, 1188,  102347,  102373,  102410, 26, 37, 33, }, // Lord_Howe
    {   67, 1198,  102443,  102454,  102476, 11, 22, 30, }, // Macau
    {   67, 1204,  102506,  102524,  102553, 18, 29, 25, }, // Magadan
    {   67, 1219,       0,  102578,       0,  0, 18,  0, }, // Malaysia
    {   67, 1228,       0,  102596,       0,  0, 17,  0, }, // Maldives
    {   67, 1237,       0,  102613,       0,  0, 26,  0, }, // Marquesas
    {   67, 1247,       0,  102639,       0,  0, 30,  0, }, // Marshall_Islands
    {   67, 1264,  102669,  102688,  102718, 19, 30, 26, }, // Mauritius
    {   67, 1274,       0,  102744,       0,  0, 22,  0, }, // Mawson
    {   67, 1281,  102766,  102792,  102829, 26, 37, 33, }, // Mexico_Pacific
    {   67, 1296,  102862,  102882,  102913, 20, 31, 27, }, // Mongolia
    {   67, 1305,  102940,  102957,  102985, 17, 28, 24, }, // Moscow
    {   67, 1312,       0,  103009,       0,  0, 19,  0, }, // Myanmar
    {   67, 1320,       0,  103028,       0,  0, 23,  0, }, // Nauru
    {   67, 1326,       0,  103051,       0,  0, 16,  0, }, // Nepal
    {   67, 1332,  103067,  103091,  103126, 24, 35, 31, }, // New_Caledonia
    {   67, 1346,  103157,  103178,  103210, 21, 32, 28, }, // New_Zealand
    {   67, 1358,  103238,  103261,  103295, 23, 34, 30, }, // Newfoundland
    {   67, 1371,       0,  103325,       0,  0, 22,  0, }, // Niue
    {   67, 1376,  103347,  103364,  103392, 17, 28, 24, }, // Norfolk
    {   67, 1384,  103416,  103453,  103501, 37, 48, 44, }, // Noronha
    {   67, 1392,       0,  103545,       0,  0, 31,  0, }, // North_Mariana
    {   67, 1406,  103576,  103596,  103627, 20, 31, 27, }, // Novosibirsk
    {   67, 1418,  103654,  103667,  103691, 13, 24, 20, }, // Omsk
    {   67, 1428,  103711,  103730,  103760, 19, 30, 26, }, // Pakistan
    {   67, 1437,       0,  103786,       0,  0, 23,  0, }, // Palau
    {   67, 1443,       0,  103809,       0,  0, 30,  0, }, // Papua_New_Guinea
    {   67, 1460,  103839,  103858,  103888, 19, 30, 26, }, // Paraguay
    {   67, 1469,  103914,  103931,  103959, 17, 28, 24, }, // Peru
    {   67, 1474,  103983,  104001,  104030, 18, 29, 25, }, // Philippines
    {   67, 1486,       0,  104055,       0,  0, 26,  0, }, // Phoenix_Islands
    {   67, 1502,  104081,  104122,  104174, 41, 52, 48, }, // Pierre_Miquelon
    {   67, 1518,       0,  104222,       0,  0, 27,  0, }, // Pitcairn
    {   67, 1527,       0,  104249,       0,  0, 24,  0, }, // Ponape
    {   67, 1534,       0,  104273,       0,  0, 19,  0, }, // Pyongyang
    {   67, 1544,  104292,  104307,  104333, 15, 26, 34, }, // Qyzylorda
    {   67, 1554,       0,  104367,       0,  0, 18,  0, }, // Reunion
    {   67, 1562,       0,  104385,       0,  0, 23,  0, }, // Rothera
    {   67, 1570,  104408,  104426,  104455, 18, 29, 25, }, // Sakhalin
    {   67, 1579,  104480,  104492,  104515, 12, 23, 31, }, // Samara
    {   67, 1596,  104546,  104563,  104591, 17, 28, 24, }, // Samoa
    {   67, 1602,       0,  104615,       0,  0, 17,  0, }, // Seychelles
    {   67, 1624,       0,  104632,       0,  0, 30,  0, }, // Singapore
    {   67, 1634,       0,  104662,       0,  0, 31,  0, }, // Solomon
    {   67, 1642,       0,  104693,       0,  0, 24,  0, }, // South_Georgia
    {   67, 1656,       0,  104717,       0,  0, 18,  0, }, // Suriname
    {   67, 1676,       0,  104735,       0,  0, 21,  0, }, // Syowa
    {   67, 1682,       0,  104756,       0,  0, 18,  0, }, // Tahiti
    {   67, 1689,  104774,  104791,  104819, 17, 28, 24, }, // Taipei
    {   67, 1696,       0,  104843,       0,  0, 22,  0, }, // Tajikistan
    {   67, 1724,       0,  104865,       0,  0, 25,  0, }, // Tokelau
    {   67, 1732,  104890,  104907,  104935, 17, 28, 24, }, // Tonga
    {   67, 1738,       0,  104959,       0,  0, 13,  0, }, // Truk
    {   67, 1750,  104972,  104995,  105029, 23, 34, 30, }, // Turkmenistan
    {   67, 1763,       0,  105059,       0,  0, 19,  0, }, // Tuvalu
    {   67, 1777,  105078,  105096,  105125, 18, 29, 25, }, // Uruguay
    {   67, 1792,  105150,  105171,  105203, 21, 32, 28, }, // Uzbekistan
    {   67, 1803,  105231,  105251,  105282, 20, 31, 27, }, // Vanuatu
    {   67, 1811,       0,  105309,       0,  0, 21,  0, }, // Venezuela
    {   67, 1821,  105330,  105351,  105383, 21, 32, 28, }, // Vladivostok
    {   67, 1833,  105411,  105431,  105462, 20, 31, 27, }, // Volgograd
    {   67, 1843,       0,  105489,       0,  0, 22,  0, }, // Vostok
    {   67, 1850,       0,  105511,       0,  0, 22,  0, }, // Wake
    {   67, 1855,       0,  105533,       0,  0, 34,  0, }, // Wallis
    {   67, 1862,  105567,  105583,  105610, 16, 27, 23, }, // Yakutsk
    {   67, 1870,  105633,  105656,  105690, 23, 34, 30, }, // Yekaterinburg
    {   67, 1892,       0,  105720,       0,  0, 16,  0, }, // Yukon
    {   68,    0,  105736,  105746,  105767, 10, 21, 27, }, // Acre Bosnian/Cyrillic/Bosnia and Herzegovina
    {   68,    5,       0,  105794,       0,  0, 18,  0, }, // Afghanistan
    {   68,   17,       0,  105812,       0,  0, 25,  0, }, // Africa_Central
    {   68,   32,       0,  105837,       0,  0, 23,  0, }, // Africa_Eastern
    {   68,   65,       0,  105860,       0,  0, 21,  0, }, // Africa_Southern
    {   68,   81,  105881,  105904,  105938, 23, 34, 38, }, // Africa_Western
    {   68,  107,  105976,  105990,  106015, 14, 25, 20, }, // Alaska
    {   68,  128,  106035,  106047,  106070, 12, 23, 29, }, // Almaty
    {   68,  135,  106099,  106113,  106138, 14, 25, 29, }, // Amazon
    {   68,  142,  106167,  106184,  106212, 17, 28, 32, }, // America_Central
    {   68,  158,  106244,  106259,  106285, 15, 26, 30, }, // America_Eastern
    {   68,  174,  106315,  106332,  106360, 17, 28, 32, }, // America_Mountain
    {   68,  191,  106392,  106409,  106437, 17, 28, 32, }, // America_Pacific
    {   68,  207,  106469,  106481,  106504, 12, 23, 29, }, // Anadyr
    {   68,  214,  106533,  106546,  106570, 13, 24, 28, }, // Apia
    {   68,  219,  106598,  106611,  106635, 13, 24, 30, }, // Aqtau
    {   68,  225,  106665,  106678,  106702, 13, 24, 30, }, // Aqtobe
    {   68,  232,  106732,  106749,  106777, 17, 28, 32, }, // Arabian
    {   68,  240,  106809,  106826,  106854, 17, 28, 32, }, // Argentina
    {   68,  250,  106886,  106911,  106947, 25, 36, 40, }, // Argentina_Western
    {   68,  268,  106987,  107003,  107030, 16, 27, 31, }, // Armenia
    {   68,  286,  107061,  107077,  107104, 16, 27, 31, }, // Atlantic
    {   68,  295,  107135,  107165,  107206, 30, 41, 45, }, // Australia_Central
    {   68,  313,  107251,  107289,  107338, 38, 49, 53, }, // Australia_CentralWestern
    {   68,  338,  107391,  107419,  107458, 28, 39, 43, }, // Australia_Eastern
    {   68,  356,  107501,  107529,  107568, 28, 39, 43, }, // Australia_Western
    {   68,  374,  107611,  107629,  107658, 18, 29, 33, }, // Azerbaijan
    {   68,  385,  107691,  107704,  107728, 13, 24, 28, }, // Azores
    {   68,  397,  107756,  107773,  107801, 17, 28, 32, }, // Bangladesh
    {   68,  415,       0,  107833,       0,  0, 13,  0, }, // Bhutan
    {   68,  422,       0,  107846,       0,  0, 16,  0, }, // Bolivia
    {   68,  437,  107862,  107879,  107907, 17, 28, 32, }, // Brasilia
    {   68,  454,       0,  107939,       0,  0, 24,  0, }, // Brunei
    {   68,  461,  107963,  107982,  108012, 19, 30, 34, }, // Cape_Verde
    {   68,  478,       0,  108046,       0,  0, 14,  0, }, // Chamorro
    {   68,  487,  108060,  108073,  108097, 13, 24, 28, }, // Chatham
    {   68,  495,  108125,  108137,  108160, 12, 23, 27, }, // Chile
    {   68,  501,  108187,  108199,  108222, 12, 23, 27, }, // China
    {   68,  507,       0,  108249,       0,  0, 22,  0, }, // Christmas
    {   68,  517,       0,  108271,       0,  0, 29,  0, }, // Cocos
    {   68,  523,  108300,  108317,  108345, 17, 28, 32, }, // Colombia
    {   68,  532,  108377,  108398,  108430, 21, 32, 41, }, // Cook
    {   68,  537,  108471,  108483,  108506, 12, 23, 27, }, // Cuba
    {   68,  548,       0,  108533,       0,  0, 14,  0, }, // Davis
    {   68,  564,       0,  108547,       0,  0, 21,  0, }, // DumontDUrville
    {   68,  601,       0,  108568,       0,  0, 19,  0, }, // East_Timor
    {   68,  612,  108587,  108609,  108642, 22, 33, 37, }, // Easter
    {   68,  619,       0,  108679,       0,  0, 15,  0, }, // Ecuador
    {   68,  627,  108694,  108716,  108749, 22, 33, 37, }, // Europe_Central
    {   68,  642,  108786,  108809,  108843, 23, 34, 38, }, // Europe_Eastern
    {   68,  657,       0,  108881,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {   68,  680,  108909,  108932,  108966, 23, 34, 38, }, // Europe_Western
    {   68,  695,  109004,  109030,  109067, 26, 37, 41, }, // Falkland
    {   68,  704,  109108,  109120,  109143, 12, 23, 27, }, // Fiji
    {   68,  709,       0,  109170,       0,  0, 25,  0, }, // French_Guiana
    {   68,  723,       0,  109195,       0,  0, 37,  0, }, // French_Southern
    {   68,  746,       0,  109232,       0,  0, 17,  0, }, // Galapagos
    {   68,  756,       0,  109249,       0,  0, 16,  0, }, // Gambier
    {   68,  764,  109265,  109280,  109306, 15, 26, 30, }, // Georgia
    {   68,  772,       0,  109336,       0,  0, 22,  0, }, // Gilbert_Islands
    {   68,  788,       0,  109358,       0,  0, 23,  0, }, // GMT
    {   68,  830,  109381,  109405,  109440, 24, 35, 39, }, // Greenland_Eastern
    {   68,  848,  109479,  109503,  109538, 24, 35, 39, }, // Greenland_Western
    {   68,  866,       0,  109577,       0,  0, 21,  0, }, // Guam
    {   68,  871,       0,  109598,       0,  0, 16,  0, }, // Gulf
    {   68,  876,       0,  109180,       0,  0, 15,  0, }, // Guyana
    {   68,  883,  109614,  109639,  109675, 25, 36, 40, }, // Hawaii_Aleutian
    {   68,  899,  109715,  109732,  109760, 17, 28, 33, }, // Hong_Kong
    {   68,  909,  109793,  109805,  109828, 12, 23, 27, }, // Hovd
    {   68,  914,       0,  109855,       0,  0, 27,  0, }, // India
    {   68,  920,       0,  109882,       0,  0, 25,  0, }, // Indian_Ocean
    {   68,  933,       0,  109907,       0,  0, 16,  0, }, // Indochina
    {   68,  943,       0,  109923,       0,  0, 30,  0, }, // Indonesia_Central
    {   68,  961,       0,  109953,       0,  0, 28,  0, }, // Indonesia_Eastern
    {   68,  979,       0,  109981,       0,  0, 28,  0, }, // Indonesia_Western
    {   68,  997,  110009,  110021,  110044, 12, 23, 27, }, // Iran
    {   68, 1008,  110071,  110085,  110110, 14, 25, 29, }, // Irkutsk
    {   68, 1016,  110139,  110156,  110184, 17, 28, 32, }, // Israel
    {   68, 1023,  110216,  110232,  110259, 16, 27, 31, }, // Japan
    {   68, 1029,  110290,  110320,  110361, 30, 41, 46, }, // Kamchatka
    {   68, 1047,       0,    3664,       0,  0, 15,  0, }, // Kazakhstan
    {   68, 1058,       0,  110407,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {   68, 1077,       0,  110435,       0,  0, 28,  0, }, // Kazakhstan_Western
    {   68, 1106,  110463,  110479,  110506, 16, 27, 31, }, // Korea
    {   68, 1112,       0,  110537,       0,  0, 13,  0, }, // Kosrae
    {   68, 1119,  110550,  110570,  110600, 20, 30, 34, }, // Krasnoyarsk
    {   68, 1151,       0,  110634,       0,  0, 17,  0, }, // Kyrgystan
    {   68, 1161,       0,  110651,       0,  0, 15,  0, }, // Lanka
    {   68, 1175,       0,  110666,       0,  0, 19,  0, }, // Line_Islands
    {   68, 1188,  110685,  110701,  110728, 16, 27, 31, }, // Lord_Howe
    {   68, 1198,  110759,  110770,  110792, 11, 22, 27, }, // Macau
    {   68, 1204,  110819,  110834,  110860, 15, 26, 29, }, // Magadan
    {   68, 1219,       0,  110889,       0,  0, 16,  0, }, // Malaysia
    {   68, 1228,       0,  110905,       0,  0, 15,  0, }, // Maldives
    {   68, 1237,       0,  110920,       0,  0, 14,  0, }, // Marquesas
    {   68, 1247,       0,  110934,       0,  0, 24,  0, }, // Marshall_Islands
    {   68, 1264,  110958,  110976,  111005, 18, 29, 33, }, // Mauritius
    {   68, 1274,       0,  111038,       0,  0, 13,  0, }, // Mawson
    {   68, 1281,  111051,  111077,  111114, 26, 37, 41, }, // Mexico_Pacific
    {   68, 1296,  111155,  111173,  111202, 18, 29, 33, }, // Mongolia
    {   68, 1305,  111235,  111249,  111274, 14, 25, 29, }, // Moscow
    {   68, 1312,       0,  111303,       0,  0, 16,  0, }, // Myanmar
    {   68, 1320,       0,  111319,       0,  0, 13,  0, }, // Nauru
    {   68, 1326,       0,  111332,       0,  0, 13,  0, }, // Nepal
    {   68, 1332,  111345,  111368,  111402, 23, 34, 38, }, // New_Caledonia
    {   68, 1346,  111440,  111459,  111489, 19, 30, 34, }, // New_Zealand
    {   68, 1358,  111523,  111542,  111572, 19, 30, 34, }, // Newfoundland
    {   68, 1371,       0,  111606,       0,  0, 12,  0, }, // Niue
    {   68, 1376,  111618,  111640,  111673, 22, 33, 37, }, // Norfolk
    {   68, 1384,  111710,  111736,  111773, 26, 37, 41, }, // Noronha
    {   68, 1392,       0,  111814,       0,  0, 31,  0, }, // North_Mariana
    {   68, 1406,  111845,  111865,  111895, 20, 30, 34, }, // Novosibirsk
    {   68, 1418,  111929,  111941,  111964, 12, 23, 27, }, // Omsk
    {   68, 1428,  111991,  112007,  112034, 16, 27, 31, }, // Pakistan
    {   68, 1437,       0,  112065,       0,  0, 13,  0, }, // Palau
    {   68, 1443,       0,  112078,       0,  0, 26,  0, }, // Papua_New_Guinea
    {   68, 1460,  112104,  112120,  112147, 16, 27, 31, }, // Paraguay
    {   68, 1469,  112178,  112190,  112213, 12, 23, 27, }, // Peru
    {   68, 1474,  112240,  112256,  112283, 16, 27, 31, }, // Philippines
    {   68, 1486,       0,  112314,       0,  0, 21,  0, }, // Phoenix_Islands
    {   68, 1502,  112335,  112361,  112398, 26, 37, 40, }, // Pierre_Miquelon
    {   68, 1518,       0,  112438,       0,  0, 15,  0, }, // Pitcairn
    {   68, 1527,       0,  112453,       0,  0, 14,  0, }, // Ponape
    {   68, 1534,       0,  112467,       0,  0, 17,  0, }, // Pyongyang
    {   68, 1544,  112484,  112499,  112525, 15, 26, 32, }, // Qyzylorda
    {   68, 1554,       0,  112557,       0,  0, 15,  0, }, // Reunion
    {   68, 1562,       0,  112572,       0,  0, 14,  0, }, // Rothera
    {   68, 1570,  112586,  112601,  112627, 15, 26, 30, }, // Sakhalin
    {   68, 1579,  112657,  112669,  112692, 12, 23, 29, }, // Samara
    {   68, 1596,  112721,  112734,  112758, 13, 24, 28, }, // Samoa
    {   68, 1602,       0,  112786,       0,  0, 15,  0, }, // Seychelles
    {   68, 1624,       0,  112801,       0,  0, 27,  0, }, // Singapore
    {   68, 1634,       0,  112828,       0,  0, 25,  0, }, // Solomon
    {   68, 1642,       0,  112853,       0,  0, 21,  0, }, // South_Georgia
    {   68, 1656,       0,  112874,       0,  0, 15,  0, }, // Suriname
    {   68, 1676,       0,  112889,       0,  0, 12,  0, }, // Syowa
    {   68, 1682,       0,  112901,       0,  0, 14,  0, }, // Tahiti
    {   68, 1689,  112915,  112929,  112954, 14, 25, 29, }, // Taipei
    {   68, 1696,       0,  112983,       0,  0, 18,  0, }, // Tajikistan
    {   68, 1724,       0,  113001,       0,  0, 15,  0, }, // Tokelau
    {   68, 1732,  113016,  113029,  113053, 13, 24, 28, }, // Tonga
    {   68, 1738,       0,  113081,       0,  0, 12,  0, }, // Truk
    {   68, 1750,  113093,  113113,  113144, 20, 31, 35, }, // Turkmenistan
    {   68, 1763,       0,  113179,       0,  0, 14,  0, }, // Tuvalu
    {   68, 1777,  113193,  113208,  113234, 15, 26, 30, }, // Uruguay
    {   68, 1792,  113264,  113282,  113311, 18, 29, 33, }, // Uzbekistan
    {   68, 1803,  113344,  113359,  113385, 15, 26, 30, }, // Vanuatu
    {   68, 1811,       0,  113415,       0,  0, 17,  0, }, // Venezuela
    {   68, 1821,  113432,  113451,  113481, 19, 30, 34, }, // Vladivostok
    {   68, 1833,  113515,  113532,  113560, 17, 28, 32, }, // Volgograd
    {   68, 1843,       0,  113592,       0,  0, 14,  0, }, // Vostok
    {   68, 1850,       0,  113606,       0,  0, 19,  0, }, // Wake
    {   68, 1855,       0,  113625,       0,  0, 29,  0, }, // Wallis
    {   68, 1862,  113654,  113669,  113695, 15, 26, 30, }, // Yakutsk
    {   68, 1870,  113725,  113746,  113778, 21, 32, 36, }, // Yekaterinburg
    {   68, 1892,       0,    6333,       0,  0, 10,  0, }, // Yukon
    {   69,    5,       0,  113814,       0,  0, 15,  0, }, // Afghanistan Breton/Latin/France
    {   69,   17,       0,  113829,       0,  0, 15,  0, }, // Africa_Central
    {   69,   32,       0,  113844,       0,  0, 19,  0, }, // Africa_Eastern
    {   69,   65,       0,  113863,       0,  0, 20,  0, }, // Africa_Southern
    {   69,   81,  113883,  113905,  113935, 22, 30, 27, }, // Africa_Western
    {   69,  107,  113962,  113972,  113990, 10, 18, 15, }, // Alaska
    {   69,  128,  114005,  114015,  114033, 10, 18, 15, }, // Almaty
    {   69,  135,  114048,  114061,  114082, 13, 21, 18, }, // Amazon
    {   69,  142,  114100,  114114,  114136, 14, 22, 19, }, // America_Central
    {   69,  158,  114155,  114167,  114187, 12, 20, 17, }, // America_Eastern
    {   69,  174,  114204,  114219,  114242, 15, 23, 20, }, // America_Mountain
    {   69,  191,  114262,  114275,  114296, 13, 21, 18, }, // America_Pacific
    {   69,  207,  114314,  114325,  114344, 11, 19, 16, }, // Anadyr
    {   69,  214,  114360,  114368,  114384,  8, 16, 13, }, // Apia
    {   69,  232,  114397,  114407,  114425, 10, 18, 15, }, // Arabian
    {   69,  240,  114440,  114455,  114478, 15, 23, 20, }, // Argentina
    {   69,  250,  114498,  114525,  114560, 27, 35, 32, }, // Argentina_Western
    {   69,  268,  114592,  114603,  114622, 11, 19, 16, }, // Armenia
    {   69,  286,  114638,  114653,  114676, 15, 23, 20, }, // Atlantic
    {   69,  295,  114696,  114714,  114740, 18, 26, 23, }, // Australia_Central
    {   69,  313,  114763,  114793,  114831, 30, 38, 35, }, // Australia_CentralWestern
    {   69,  338,  114866,  114888,  114918, 22, 30, 27, }, // Australia_Eastern
    {   69,  356,  114945,  114970,  115003, 25, 33, 30, }, // Australia_Western
    {   69,  374,  115033,  115048,  115071, 15, 23, 20, }, // Azerbaijan
    {   69,  385,  115091,  115104,  115125, 13, 21, 18, }, // Azores
    {   69,  397,  115143,  115157,  115179, 14, 22, 19, }, // Bangladesh
    {   69,  415,       0,  115198,       0,  0, 11,  0, }, // Bhutan
    {   69,  422,       0,  115209,       0,  0, 11,  0, }, // Bolivia
    {   69,  437,  115220,  115232,  115252, 12, 20, 17, }, // Brasilia
    {   69,  454,       0,  115269,       0,  0, 21,  0, }, // Brunei
    {   69,  461,  115290,  115307,  115332, 17, 25, 22, }, // Cape_Verde
    {   69,  478,       0,  115354,       0,  0, 12,  0, }, // Chamorro
    {   69,  487,  115366,  115377,  115396, 11, 19, 16, }, // Chatham
    {   69,  495,  115412,  115421,  115438,  9, 17, 14, }, // Chile
    {   69,  501,  115452,  115460,  115476,  8, 16, 13, }, // China
    {   69,  507,       0,  115489,       0,  0, 18,  0, }, // Christmas
    {   69,  517,       0,  115507,       0,  0, 15,  0, }, // Cocos
    {   69,  523,  115522,  115534,  115554, 12, 20, 17, }, // Colombia
    {   69,  532,  115571,  115585,  115607, 14, 22, 19, }, // Cook
    {   69,  537,  115626,  115634,  115650,  8, 16, 13, }, // Cuba
    {   69,  548,       0,  115663,       0,  0,  9,  0, }, // Davis
    {   69,  564,       0,  115672,       0,  0, 20,  0, }, // DumontDUrville
    {   69,  601,       0,  115692,       0,  0, 18,  0, }, // East_Timor
    {   69,  612,  115710,  115723,  115744, 13, 21, 18, }, // Easter
    {   69,  619,       0,  115762,       0,  0, 11,  0, }, // Ecuador
    {   69,  627,  115773,  115788,  115811, 15, 23, 20, }, // Europe_Central
    {   69,  642,  115831,  115850,  115877, 19, 27, 24, }, // Europe_Eastern
    {   69,  657,       0,  115901,       0,  0, 15,  0, }, // Europe_Further_Eastern
    {   69,  680,  115916,  115938,  115968, 22, 30, 27, }, // Europe_Western
    {   69,  695,  115995,  116013,  116039, 18, 26, 23, }, // Falkland
    {   69,  704,  116062,  116071,  116088,  9, 17, 14, }, // Fiji
    {   69,  709,       0,  116102,       0,  0, 17,  0, }, // French_Guiana
    {   69,  723,       0,  116119,       0,  0, 40,  0, }, // French_Southern
    {   69,  746,       0,  116159,       0,  0, 19,  0, }, // Galapagos
    {   69,  756,       0,  116178,       0,  0, 17,  0, }, // Gambier
    {   69,  764,  116195,  116205,  116223, 10, 18, 15, }, // Georgia
    {   69,  772,       0,  116238,       0,  0, 17,  0, }, // Gilbert_Islands
    {   69,  788,       0,  116255,       0,  0, 28,  0, }, // GMT
    {   69,  830,  116283,  116305,  116335, 22, 30, 27, }, // Greenland_Eastern
    {   69,  848,  116362,  116387,  116420, 25, 33, 30, }, // Greenland_Western
    {   69,  866,       0,  116450,       0,  0, 16,  0, }, // Guam
    {   69,  871,       0,  116466,       0,  0, 36,  0, }, // Gulf
    {   69,  876,       0,  116502,       0,  0, 10,  0, }, // Guyana
    {   69,  883,  116512,  116538,  116572, 26, 34, 31, }, // Hawaii_Aleutian
    {   69,  899,  116603,  116616,  116637, 13, 21, 18, }, // Hong_Kong
    {   69,  909,  116655,  116663,  116679,  8, 16, 13, }, // Hovd
    {   69,  914,       0,  116692,       0,  0, 17,  0, }, // India
    {   69,  920,       0,  116709,       0,  0, 17,  0, }, // Indian_Ocean
    {   69,  933,       0,  116726,       0,  0, 14,  0, }, // Indochina
    {   69,  943,       0,  116740,       0,  0, 19,  0, }, // Indonesia_Central
    {   69,  961,       0,  116759,       0,  0, 22,  0, }, // Indonesia_Eastern
    {   69,  979,       0,  116781,       0,  0, 25,  0, }, // Indonesia_Western
    {   69,  997,  116806,  116814,  116830,  8, 16, 13, }, // Iran
    {   69, 1008,  116843,  116854,  116873, 11, 19, 16, }, // Irkutsk
    {   69, 1016,  116889,  116899,  116917, 10, 18, 15, }, // Israel
    {   69, 1023,  116932,  116941,  116958,  9, 17, 14, }, // Japan
    {   69, 1058,       0,  116972,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {   69, 1077,       0,  116994,       0,  0, 25,  0, }, // Kazakhstan_Western
    {   69, 1106,  117019,  117028,  117045,  9, 17, 14, }, // Korea
    {   69, 1112,       0,  117059,       0,  0, 10,  0, }, // Kosrae
    {   69, 1119,  117069,  117084,  117107, 15, 23, 20, }, // Krasnoyarsk
    {   69, 1151,       0,  117127,       0,  0, 14,  0, }, // Kyrgystan
    {   69, 1161,       0,  117141,       0,  0, 13,  0, }, // Lanka
    {   69, 1175,       0,  117154,       0,  0, 16,  0, }, // Line_Islands
    {   69, 1188,  117170,  117183,  117204, 13, 21, 18, }, // Lord_Howe
    {   69, 1198,  117222,  117231,  117248,  9, 17, 14, }, // Macau
    {   69, 1204,  117262,  117273,  117292, 11, 19, 16, }, // Magadan
    {   69, 1219,       0,  117308,       0,  0, 12,  0, }, // Malaysia
    {   69, 1228,       0,  117320,       0,  0, 15,  0, }, // Maldives
    {   69, 1237,       0,  117335,       0,  0, 16,  0, }, // Marquesas
    {   69, 1247,       0,  117351,       0,  0, 18,  0, }, // Marshall_Islands
    {   69, 1264,  117369,  117378,  117395,  9, 17, 14, }, // Mauritius
    {   69, 1274,       0,  117409,       0,  0, 10,  0, }, // Mawson
    {   69, 1281,  117419,  117442,  117473, 23, 31, 28, }, // Mexico_Pacific
    {   69, 1296,  117501,  117516,  117539, 15, 23, 20, }, // Mongolia
    {   69, 1305,  117559,  117569,  117587, 10, 18, 15, }, // Moscow
    {   69, 1312,       0,  117602,       0,  0, 11,  0, }, // Myanmar
    {   69, 1320,       0,  117613,       0,  0,  9,  0, }, // Nauru
    {   69, 1326,       0,  117622,       0,  0,  9,  0, }, // Nepal
    {   69, 1332,  117631,  117650,  117677, 19, 27, 24, }, // New_Caledonia
    {   69, 1346,  117701,  117717,  117741, 16, 24, 21, }, // New_Zealand
    {   69, 1358,  117762,  117778,  117802, 16, 24, 21, }, // Newfoundland
    {   69, 1371,       0,  117823,       0,  0,  8,  0, }, // Niue
    {   69, 1376,  117831,  117847,  117871, 16, 24, 21, }, // Norfolk
    {   69, 1384,  117892,  117915,  117946, 23, 31, 28, }, // Noronha
    {   69, 1392,       0,  117974,       0,  0, 20,  0, }, // North_Mariana
    {   69, 1406,  117994,  118009,  118032, 15, 23, 20, }, // Novosibirsk
    {   69, 1418,  118052,  118060,  118076,  8, 16, 13, }, // Omsk
    {   69, 1428,  118089,  118101,  118121, 12, 20, 17, }, // Pakistan
    {   69, 1437,       0,  118138,       0,  0,  9,  0, }, // Palau
    {   69, 1443,       0,  118147,       0,  0, 22,  0, }, // Papua_New_Guinea
    {   69, 1460,  118169,  118181,  118201, 12, 20, 17, }, // Paraguay
    {   69, 1469,  118218,  118227,  118244,  9, 17, 14, }, // Peru
    {   69, 1474,  118258,  118274,  118298, 16, 24, 21, }, // Philippines
    {   69, 1486,       0,  118319,       0,  0, 17,  0, }, // Phoenix_Islands
    {   69, 1502,  118336,  118359,  118390, 23, 31, 28, }, // Pierre_Miquelon
    {   69, 1518,       0,  118418,       0,  0, 12,  0, }, // Pitcairn
    {   69, 1527,       0,  118430,       0,  0, 11,  0, }, // Ponape
    {   69, 1534,       0,  118441,       0,  0, 14,  0, }, // Pyongyang
    {   69, 1554,       0,  118455,       0,  0, 14,  0, }, // Reunion
    {   69, 1562,       0,  118469,       0,  0, 11,  0, }, // Rothera
    {   69, 1570,  118480,  118492,  118512, 12, 20, 17, }, // Sakhalin
    {   69, 1596,  118529,  118538,  118555,  9, 17, 14, }, // Samoa
    {   69, 1602,       0,  118569,       0,  0, 12,  0, }, // Seychelles
    {   69, 1624,       0,  118581,       0,  0, 21,  0, }, // Singapore
    {   69, 1634,       0,  118602,       0,  0, 17,  0, }, // Solomon
    {   69, 1642,       0,  118619,       0,  0, 17,  0, }, // South_Georgia
    {   69, 1656,       0,  118636,       0,  0, 11,  0, }, // Suriname
    {   69, 1676,       0,  118647,       0,  0,  9,  0, }, // Syowa
    {   69, 1682,       0,  118656,       0,  0, 10,  0, }, // Tahiti
    {   69, 1689,  118666,  118676,  118694, 10, 18, 15, }, // Taipei
    {   69, 1696,       0,  118709,       0,  0, 15,  0, }, // Tajikistan
    {   69, 1724,       0,  118724,       0,  0, 11,  0, }, // Tokelau
    {   69, 1732,  118735,  118744,  118761,  9, 17, 14, }, // Tonga
    {   69, 1738,       0,  118775,       0,  0,  9,  0, }, // Truk
    {   69, 1750,  118784,  118800,  118824, 16, 24, 21, }, // Turkmenistan
    {   69, 1763,       0,  118845,       0,  0, 10,  0, }, // Tuvalu
    {   69, 1777,  118855,  118866,  118885, 11, 19, 16, }, // Uruguay
    {   69, 1792,  118901,  118916,  118939, 15, 23, 20, }, // Uzbekistan
    {   69, 1803,  118959,  118970,  118989, 11, 19, 16, }, // Vanuatu
    {   69, 1811,       0,  119005,       0,  0, 13,  0, }, // Venezuela
    {   69, 1821,  119018,  119033,  119056, 15, 23, 20, }, // Vladivostok
    {   69, 1833,  119076,  119089,  119110, 13, 21, 18, }, // Volgograd
    {   69, 1843,       0,  119128,       0,  0, 10,  0, }, // Vostok
    {   69, 1850,       0,  119138,       0,  0, 15,  0, }, // Wake
    {   69, 1855,       0,  119153,       0,  0, 20,  0, }, // Wallis
    {   69, 1862,  119173,  119184,  119203, 11, 19, 16, }, // Yakutsk
    {   69, 1870,  119219,  119237,  119263, 18, 26, 23, }, // Yekaterinburg
    {   70,    5,       0,  119286,       0,  0, 19,  0, }, // Afghanistan Bulgarian/Cyrillic/Bulgaria
    {   70,   17,       0,  119305,       0,  0, 25,  0, }, // Africa_Central
    {   70,   32,       0,  119330,       0,  0, 23,  0, }, // Africa_Eastern
    {   70,   65,       0,  119353,       0,  0, 20,  0, }, // Africa_Southern
    {   70,   81,  119373,  119396,  119430, 23, 34, 36, }, // Africa_Western
    {   70,  107,  119466,  119472,  119497,  6, 25, 27, }, // Alaska
    {   70,  135,  119524,  119539,  119565, 15, 26, 28, }, // Amazon
    {   70,  142,  119593,  119627,  119672, 34, 45, 47, }, // America_Central
    {   70,  158,  119719,  119751,  119794, 32, 43, 45, }, // America_Eastern
    {   70,  174,  119839,  119873,  119918, 34, 45, 47, }, // America_Mountain
    {   70,  191,  119965,  120002,  120050, 37, 48, 50, }, // America_Pacific
    {   70,  207,  106469,  120100,  120125, 12, 25, 27, }, // Anadyr
    {   70,  214,  120152,  120156,  120179,  4, 23, 25, }, // Apia
    {   70,  232,  120204,  120217,  120241, 13, 24, 26, }, // Arabian
    {   70,  240,  120267,  120284,  120312, 17, 28, 30, }, // Argentina
    {   70,  250,  120342,  120366,  120401, 24, 35, 37, }, // Argentina_Western
    {   70,  268,  120438,  120452,  120477, 14, 25, 27, }, // Armenia
    {   70,  286,  120504,  120541,  120589, 37, 48, 50, }, // Atlantic
    {   70,  295,  120639,  120666,  120704, 27, 38, 40, }, // Australia_Central
    {   70,  313,  120744,  120779,  120825, 35, 46, 48, }, // Australia_CentralWestern
    {   70,  338,  120873,  120898,  120934, 25, 36, 38, }, // Australia_Eastern
    {   70,  356,  120972,  120997,  121033, 25, 36, 38, }, // Australia_Western
    {   70,  374,  121071,  121091,  121122, 20, 31, 33, }, // Azerbaijan
    {   70,  385,  121155,  121170,  121204, 15, 34, 36, }, // Azores
    {   70,  397,  121240,  121257,  121285, 17, 28, 30, }, // Bangladesh
    {   70,  415,       0,  121315,       0,  0, 14,  0, }, // Bhutan
    {   70,  422,       0,  121329,       0,  0, 16,  0, }, // Bolivia
    {   70,  437,  121345,  121360,  121386, 15, 26, 28, }, // Brasilia
    {   70,  454,       0,  121414,       0,  0, 17,  0, }, // Brunei
    {   70,  461,  121431,  121441,  121470, 10, 29, 31, }, // Cape_Verde
    {   70,  478,       0,  121501,       0,  0, 14,  0, }, // Chamorro
    {   70,  487,  121515,  121529,  121554, 14, 25, 27, }, // Chatham
    {   70,  495,  121581,  121595,  121620, 14, 25, 27, }, // Chile
    {   70,  501,  121647,  121661,  121686, 14, 25, 27, }, // China
    {   70,  507,       0,  121713,       0,  0, 16,  0, }, // Christmas
    {   70,  517,       0,  121729,       0,  0, 16,  0, }, // Cocos
    {   70,  523,  121745,  121762,  121790, 17, 28, 30, }, // Colombia
    {   70,  532,  121820,  121831,  121861, 11, 30, 32, }, // Cook
    {   70,  537,  121893,  121907,  121932, 14, 25, 27, }, // Cuba
    {   70,  548,       0,  121959,       0,  0,  6,  0, }, // Davis
    {   70,  564,       0,  121965,       0,  0, 12,  0, }, // DumontDUrville
    {   70,  601,       0,  121977,       0,  0, 21,  0, }, // East_Timor
    {   70,  612,  121998,  122016,  122053, 18, 37, 39, }, // Easter
    {   70,  619,       0,  122092,       0,  0, 16,  0, }, // Ecuador
    {   70,  627,  122108,  122133,  122169, 25, 36, 38, }, // Europe_Central
    {   70,  642,  122207,  122230,  122264, 23, 34, 36, }, // Europe_Eastern
    {   70,  657,       0,  122300,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {   70,  680,  122331,  122354,  122388, 23, 34, 29, }, // Europe_Western
    {   70,  695,  122417,  122436,  122474, 19, 38, 40, }, // Falkland
    {   70,  704,  122514,  122529,  122555, 15, 26, 28, }, // Fiji
    {   70,  709,       0,  122583,       0,  0, 14,  0, }, // French_Guiana
    {   70,  723,       0,  122597,       0,  0, 38,  0, }, // French_Southern
    {   70,  746,       0,  122635,       0,  0, 17,  0, }, // Galapagos
    {   70,  756,       0,  122652,       0,  0,  6,  0, }, // Gambier
    {   70,  764,  122658,  122673,  122699, 15, 26, 28, }, // Georgia
    {   70,  772,       0,  122727,       0,  0, 15,  0, }, // Gilbert_Islands
    {   70,  788,       0,  122742,       0,  0, 22,  0, }, // GMT
    {   70,  830,  122764,  122788,  122823, 24, 35, 37, }, // Greenland_Eastern
    {   70,  848,  122860,  122884,  122919, 24, 35, 37, }, // Greenland_Western
    {   70,  871,       0,  122956,       0,  0, 15,  0, }, // Gulf
    {   70,  876,       0,  122971,       0,  0,  5,  0, }, // Guyana
    {   70,  883,  122976,  122999,  123033, 23, 34, 36, }, // Hawaii_Aleutian
    {   70,  899,  123069,  123085,  123112, 16, 27, 29, }, // Hong_Kong
    {   70,  909,  123141,  123154,  123178, 13, 24, 26, }, // Hovd
    {   70,  914,       0,  123204,       0,  0, 14,  0, }, // India
    {   70,  920,       0,  123218,       0,  0, 14,  0, }, // Indian_Ocean
    {   70,  933,       0,  123232,       0,  0, 18,  0, }, // Indochina
    {   70,  943,       0,  123250,       0,  0, 27,  0, }, // Indonesia_Central
    {   70,  961,       0,  123277,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   70,  979,       0,  123302,       0,  0, 25,  0, }, // Indonesia_Western
    {   70,  997,  123327,  123340,  123364, 13, 24, 26, }, // Iran
    {   70, 1008,  123390,  123404,  123429, 14, 25, 27, }, // Irkutsk
    {   70, 1016,  123456,  123471,  123497, 15, 26, 28, }, // Israel
    {   70, 1023,  123525,  123538,  123562, 13, 24, 26, }, // Japan
    {   70, 1029,  123588,  123617,  123657, 29, 40, 44, }, // Kamchatka
    {   70, 1047,       0,  123701,       0,  0, 18,  0, }, // Kazakhstan
    {   70, 1058,       0,  123719,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {   70, 1077,       0,  123744,       0,  0, 25,  0, }, // Kazakhstan_Western
    {   70, 1106,  123769,  123783,  123808, 14, 25, 27, }, // Korea
    {   70, 1112,       0,  123835,       0,  0,  6,  0, }, // Kosrae
    {   70, 1119,  123841,  123858,  123886, 17, 28, 30, }, // Krasnoyarsk
    {   70, 1151,       0,  123916,       0,  0, 19,  0, }, // Kyrgystan
    {   70, 1175,       0,  123935,       0,  0, 20,  0, }, // Line_Islands
    {   70, 1188,  123955,  123963,  123990,  8, 27, 29, }, // Lord_Howe
    {   70, 1204,  124019,  124035,  124062, 16, 27, 29, }, // Magadan
    {   70, 1219,       0,  124091,       0,  0, 17,  0, }, // Malaysia
    {   70, 1228,       0,  124108,       0,  0, 15,  0, }, // Maldives
    {   70, 1237,       0,  124123,       0,  0, 16,  0, }, // Marquesas
    {   70, 1247,       0,  124139,       0,  0, 17,  0, }, // Marshall_Islands
    {   70, 1264,  124156,  124164,  124191,  8, 27, 29, }, // Mauritius
    {   70, 1274,       0,  124220,       0,  0,  6,  0, }, // Mawson
    {   70, 1281,  124226,  124256,  124297, 30, 41, 43, }, // Mexico_Pacific
    {   70, 1296,  124340,  124358,  124387, 18, 29, 31, }, // Mongolia
    {   70, 1305,  124418,  124433,  124459, 15, 26, 28, }, // Moscow
    {   70, 1312,       0,  124487,       0,  0, 16,  0, }, // Myanmar
    {   70, 1320,       0,   11152,       0,  0,  5,  0, }, // Nauru
    {   70, 1326,       0,  124503,       0,  0, 14,  0, }, // Nepal
    {   70, 1332,  124517,  124537,  124568, 20, 31, 33, }, // New_Caledonia
    {   70, 1346,  124601,  124620,  124650, 19, 30, 32, }, // New_Zealand
    {   70, 1358,  124682,  124702,  124733, 20, 31, 33, }, // Newfoundland
    {   70, 1371,       0,   11375,       0,  0,  4,  0, }, // Niue
    {   70, 1376,  124766,  124782,  124809, 16, 27, 29, }, // Norfolk
    {   70, 1384,  124838,  124856,  124893, 18, 37, 39, }, // Noronha
    {   70, 1406,  124932,  124950,  124979, 18, 29, 31, }, // Novosibirsk
    {   70, 1418,  125010,  125021,  125043, 11, 22, 24, }, // Omsk
    {   70, 1428,  125067,  125084,  125112, 17, 28, 30, }, // Pakistan
    {   70, 1437,       0,   11723,       0,  0,  5,  0, }, // Palau
    {   70, 1443,       0,  125142,       0,  0, 17,  0, }, // Papua_New_Guinea
    {   70, 1460,  125159,  125176,  125204, 17, 28, 30, }, // Paraguay
    {   70, 1469,  125234,  125249,  125275, 15, 26, 28, }, // Peru
    {   70, 1474,  125303,  125319,  125346, 16, 27, 29, }, // Philippines
    {   70, 1486,       0,  125375,       0,  0, 14,  0, }, // Phoenix_Islands
    {   70, 1502,  125389,  125407,  125444, 18, 37, 39, }, // Pierre_Miquelon
    {   70, 1518,       0,   12012,       0,  0,  7,  0, }, // Pitcairn
    {   70, 1527,       0,  112453,       0,  0,  6,  0, }, // Ponape
    {   70, 1534,       0,  125483,       0,  0, 15,  0, }, // Pyongyang
    {   70, 1554,       0,  125498,       0,  0,  7,  0, }, // Reunion
    {   70, 1562,       0,   12103,       0,  0,  6,  0, }, // Rothera
    {   70, 1570,  125505,  125521,  125548, 16, 27, 29, }, // Sakhalin
    {   70, 1579,  112657,  125577,  125602, 12, 25, 27, }, // Samara
    {   70, 1596,  125629,  125644,  125670, 15, 26, 28, }, // Samoa
    {   70, 1602,       0,  125698,       0,  0,  7,  0, }, // Seychelles
    {   70, 1624,       0,  125705,       0,  0, 17,  0, }, // Singapore
    {   70, 1634,       0,  125722,       0,  0, 18,  0, }, // Solomon
    {   70, 1642,       0,  125740,       0,  0, 13,  0, }, // South_Georgia
    {   70, 1656,       0,  125753,       0,  0, 16,  0, }, // Suriname
    {   70, 1676,       0,  112889,       0,  0,  4,  0, }, // Syowa
    {   70, 1682,       0,  125769,       0,  0, 15,  0, }, // Tahiti
    {   70, 1689,  125784,  125789,  125813,  5, 24, 26, }, // Taipei
    {   70, 1696,       0,  125839,       0,  0, 20,  0, }, // Tajikistan
    {   70, 1724,       0,   12429,       0,  0,  7,  0, }, // Tokelau
    {   70, 1732,   12436,  125859,  125883,  5, 24, 26, }, // Tonga
    {   70, 1738,       0,  125909,       0,  0,  4,  0, }, // Truk
    {   70, 1750,  125913,  125934,  125966, 21, 32, 34, }, // Turkmenistan
    {   70, 1763,       0,   12559,       0,  0,  6,  0, }, // Tuvalu
    {   70, 1777,  126000,  126016,  126043, 16, 27, 29, }, // Uruguay
    {   70, 1792,  126072,  126091,  126121, 19, 30, 32, }, // Uzbekistan
    {   70, 1803,   12687,  126153,  126179,  7, 26, 28, }, // Vanuatu
    {   70, 1811,       0,  126207,       0,  0, 17,  0, }, // Venezuela
    {   70, 1821,  126224,  126244,  126275, 20, 31, 33, }, // Vladivostok
    {   70, 1833,  126308,  126326,  126355, 18, 29, 31, }, // Volgograd
    {   70, 1843,       0,  113592,       0,  0,  6,  0, }, // Vostok
    {   70, 1850,       0,  126386,       0,  0, 11,  0, }, // Wake
    {   70, 1855,       0,  126397,       0,  0, 14,  0, }, // Wallis
    {   70, 1862,  126411,  126424,  126450, 13, 26, 28, }, // Yakutsk
    {   70, 1870,  126478,  126499,  126531, 21, 32, 34, }, // Yekaterinburg
    {   70, 1892,       0,  126565,       0,  0,  4,  0, }, // Yukon
    {   71,    5,       0,  126569,       0,  0, 20,  0, }, // Afghanistan Burmese/Myanmar/Myanmar
    {   71,   17,       0,  126589,       0,  0, 17,  0, }, // Africa_Central
    {   71,   32,       0,  126606,       0,  0, 18,  0, }, // Africa_Eastern
    {   71,   65,       0,  126624,       0,  0, 18,  0, }, // Africa_Southern
    {   71,   81,  126642,  126661,  126685, 19, 24, 27, }, // Africa_Western
    {   71,  107,  126712,  126725,  126743, 13, 18, 26, }, // Alaska
    {   71,  135,  126769,  126782,  126800, 13, 18, 20, }, // Amazon
    {   71,  142,  126820,  126836,  126857, 16, 21, 29, }, // America_Central
    {   71,  158,  126886,  126903,  126925, 17, 22, 30, }, // America_Eastern
    {   71,  174,  126955,  126970,  126990, 15, 20, 28, }, // America_Mountain
    {   71,  191,  127018,  127031,  127049, 13, 18, 26, }, // America_Pacific
    {   71,  214,  127075,  127087,  127104, 12, 17, 20, }, // Apia
    {   71,  232,  127124,  127137,  127155, 13, 18, 21, }, // Arabian
    {   71,  240,  127176,  127195,  127219, 19, 24, 26, }, // Argentina
    {   71,  250,  127245,  127271,  127302, 26, 31, 34, }, // Argentina_Western
    {   71,  268,  127336,  127354,  127377, 18, 23, 26, }, // Armenia
    {   71,  286,  127403,  127420,  127442, 17, 22, 30, }, // Atlantic
    {   71,  295,  127472,  127498,  127529, 26, 31, 34, }, // Australia_Central
    {   71,  313,  127563,  127598,  127638, 35, 40, 43, }, // Australia_CentralWestern
    {   71,  338,  127681,  127701,  127726, 20, 25, 28, }, // Australia_Eastern
    {   71,  356,  127754,  127775,  127731, 21, 26, 23, }, // Australia_Western
    {   71,  374,  127801,  127820,  127844, 19, 24, 27, }, // Azerbaijan
    {   71,  385,  127871,  127889,  127912, 18, 23, 26, }, // Azores
    {   71,  397,  127938,  127959,  127985, 21, 26, 29, }, // Bangladesh
    {   71,  415,       0,  128014,       0,  0, 12,  0, }, // Bhutan
    {   71,  422,       0,  128026,       0,  0, 19,  0, }, // Bolivia
    {   71,  437,  128045,  128058,  128076, 13, 18, 21, }, // Brasilia
    {   71,  454,       0,  128097,       0,  0, 21,  0, }, // Brunei
    {   71,  461,  128118,  128133,  128153, 15, 20, 23, }, // Cape_Verde
    {   71,  478,       0,  128176,       0,  0, 16,  0, }, // Chamorro
    {   71,  487,  128192,  128205,  128223, 13, 18, 22, }, // Chatham
    {   71,  495,  128245,  128257,  128274, 12, 17, 20, }, // Chile
    {   71,  501,  128294,  128306,  128323, 12, 17, 20, }, // China
    {   71,  507,       0,  128343,       0,  0, 21,  0, }, // Christmas
    {   71,  517,       0,  128364,       0,  0, 20,  0, }, // Cocos
    {   71,  523,  128384,  128400,  128421, 16, 21, 24, }, // Colombia
    {   71,  532,  128445,  128464,  128488, 19, 24, 27, }, // Cook
    {   71,  537,  128515,  128529,  128548, 14, 19, 27, }, // Cuba
    {   71,  548,       0,  128575,       0,  0, 13,  0, }, // Davis
    {   71,  564,       0,  128588,       0,  0, 26,  0, }, // DumontDUrville
    {   71,  601,       0,  128614,       0,  0, 17,  0, }, // East_Timor
    {   71,  612,  128631,  128649,  128672, 18, 23, 26, }, // Easter
    {   71,  619,       0,  128698,       0,  0, 15,  0, }, // Ecuador
    {   71,  627,  128713,  128735,  128762, 22, 27, 30, }, // Europe_Central
    {   71,  642,  128792,  128809,  128831, 17, 22, 25, }, // Europe_Eastern
    {   71,  657,       0,  128856,       0,  0, 25,  0, }, // Europe_Further_Eastern
    {   71,  680,  128881,  128899,  128922, 18, 23, 26, }, // Europe_Western
    {   71,  695,  128948,  128971,  128999, 23, 28, 31, }, // Falkland
    {   71,  704,  129030,  129042,  129059, 12, 17, 20, }, // Fiji
    {   71,  709,       0,  129079,       0,  0, 23,  0, }, // French_Guiana
    {   71,  723,       0,  129102,       0,  0, 40,  0, }, // French_Southern
    {   71,  746,       0,  129142,       0,  0, 20,  0, }, // Galapagos
    {   71,  756,       0,  129162,       0,  0, 14,  0, }, // Gambier
    {   71,  764,  129176,  129193,  129215, 17, 22, 25, }, // Georgia
    {   71,  772,       0,  129240,       0,  0, 22,  0, }, // Gilbert_Islands
    {   71,  788,       0,  129262,       0,  0, 20,  0, }, // GMT
    {   71,  830,  129282,  129303,  129329, 21, 26, 33, }, // Greenland_Eastern
    {   71,  848,  129362,  129385,  129413, 23, 28, 35, }, // Greenland_Western
    {   71,  871,       0,  129448,       0,  0, 17,  0, }, // Gulf
    {   71,  876,       0,  129465,       0,  0, 16,  0, }, // Guyana
    {   71,  883,  129481,  129509,  129542, 28, 33, 41, }, // Hawaii_Aleutian
    {   71,  899,  129583,  129600,  129622, 17, 22, 25, }, // Hong_Kong
    {   71,  909,  129647,  129660,  129678, 13, 18, 21, }, // Hovd
    {   71,  914,       0,  129699,       0,  0, 19,  0, }, // India
    {   71,  920,       0,  129718,       0,  0, 22,  0, }, // Indian_Ocean
    {   71,  933,       0,  129740,       0,  0, 23,  0, }, // Indochina
    {   71,  943,       0,  129763,       0,  0, 31,  0, }, // Indonesia_Central
    {   71,  961,       0,  129794,       0,  0, 32,  0, }, // Indonesia_Eastern
    {   71,  979,       0,  129826,       0,  0, 33,  0, }, // Indonesia_Western
    {   71,  997,  129859,  129871,  129888, 12, 17, 20, }, // Iran
    {   71, 1008,  129908,  129923,  129943, 15, 20, 23, }, // Irkutsk
    {   71, 1016,  129966,  129980,  129999, 14, 19, 22, }, // Israel
    {   71, 1023,  130021,  130033,  130050, 12, 17, 20, }, // Japan
    {   71, 1047,       0,  130070,       0,  0, 16,  0, }, // Kazakhstan
    {   71, 1058,       0,  130086,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {   71, 1077,       0,  130107,       0,  0, 22,  0, }, // Kazakhstan_Western
    {   71, 1106,  130129,  130145,  130166, 16, 21, 24, }, // Korea
    {   71, 1112,       0,  130190,       0,  0, 17,  0, }, // Kosrae
    {   71, 1119,  130207,  130227,  130252, 20, 25, 28, }, // Krasnoyarsk
    {   71, 1151,       0,  130280,       0,  0, 17,  0, }, // Kyrgystan
    {   71, 1161,       0,  130297,       0,  0, 17,  0, }, // Lanka
    {   71, 1175,       0,  130314,       0,  0, 21,  0, }, // Line_Islands
    {   71, 1188,  130335,  130353,  130376, 18, 23, 26, }, // Lord_Howe
    {   71, 1198,  130402,  130415,  130433, 13, 18, 21, }, // Macau
    {   71, 1204,  130454,  130469,  130489, 15, 20, 23, }, // Magadan
    {   71, 1219,       0,  130512,       0,  0, 15,  0, }, // Malaysia
    {   71, 1228,       0,  130527,       0,  0, 17,  0, }, // Maldives
    {   71, 1237,       0,  130544,       0,  0, 17,  0, }, // Marquesas
    {   71, 1247,       0,  130561,       0,  0, 21,  0, }, // Marshall_Islands
    {   71, 1264,  130582,  130597,  130617, 15, 20, 23, }, // Mauritius
    {   71, 1274,       0,  130640,       0,  0, 15,  0, }, // Mawson
    {   71, 1281,  130655,  130679,  130708, 24, 29, 37, }, // Mexico_Pacific
    {   71, 1296,  130745,  130761,  130782, 16, 21, 24, }, // Mongolia
    {   71, 1305,  130806,  130821,  130841, 15, 20, 23, }, // Moscow
    {   71, 1312,       0,  130864,       0,  0, 13,  0, }, // Myanmar
    {   71, 1320,       0,  130877,       0,  0, 13,  0, }, // Nauru
    {   71, 1326,       0,  130890,       0,  0, 12,  0, }, // Nepal
    {   71, 1332,  130902,  130928,  130958, 26, 30, 33, }, // New_Caledonia
    {   71, 1346,  130991,  131007,  131028, 16, 21, 24, }, // New_Zealand
    {   71, 1358,  131052,  131071,  131095, 19, 24, 32, }, // Newfoundland
    {   71, 1371,       0,  131127,       0,  0, 13,  0, }, // Niue
    {   71, 1376,  131140,  131162,  131189, 22, 27, 35, }, // Norfolk
    {   71, 1384,  131224,  131252,  131285, 28, 33, 36, }, // Noronha
    {   71, 1406,  131321,  131344,  131372, 23, 28, 31, }, // Novosibirsk
    {   71, 1418,  131403,  131419,  131440, 16, 21, 24, }, // Omsk
    {   71, 1428,  131464,  131480,  131501, 16, 21, 24, }, // Pakistan
    {   71, 1437,       0,  131525,       0,  0, 13,  0, }, // Palau
    {   71, 1443,       0,  131538,       0,  0, 21,  0, }, // Papua_New_Guinea
    {   71, 1460,  131559,  131574,  131594, 15, 20, 23, }, // Paraguay
    {   71, 1469,  131617,  131629,  131646, 12, 17, 20, }, // Peru
    {   71, 1474,  131666,  131683,  131705, 17, 22, 25, }, // Philippines
    {   71, 1486,       0,  131730,       0,  0, 21,  0, }, // Phoenix_Islands
    {   71, 1502,  131751,  131780,  131814, 29, 34, 44, }, // Pierre_Miquelon
    {   71, 1518,       0,  131858,       0,  0, 17,  0, }, // Pitcairn
    {   71, 1527,       0,  131875,       0,  0, 16,  0, }, // Ponape
    {   71, 1534,       0,  131891,       0,  0, 15,  0, }, // Pyongyang
    {   71, 1554,       0,  131906,       0,  0, 15,  0, }, // Reunion
    {   71, 1562,       0,  131921,       0,  0, 14,  0, }, // Rothera
    {   71, 1570,  131935,  131949,  131968, 14, 19, 22, }, // Sakhalin
    {   71, 1596,  131990,  132003,  132021, 13, 18, 23, }, // Samoa
    {   71, 1602,       0,  132044,       0,  0, 13,  0, }, // Seychelles
    {   71, 1624,       0,  132057,       0,  0, 15,  0, }, // Singapore
    {   71, 1634,       0,  132072,       0,  0, 24,  0, }, // Solomon
    {   71, 1642,       0,  132096,       0,  0, 22,  0, }, // South_Georgia
    {   71, 1656,       0,  132118,       0,  0, 15,  0, }, // Suriname
    {   71, 1676,       0,  132133,       0,  0, 13,  0, }, // Syowa
    {   71, 1682,       0,  132146,       0,  0, 12,  0, }, // Tahiti
    {   71, 1689,  132158,  132172,  132191, 14, 19, 22, }, // Taipei
    {   71, 1696,       0,  132213,       0,  0, 20,  0, }, // Tajikistan
    {   71, 1724,       0,  132233,       0,  0, 15,  0, }, // Tokelau
    {   71, 1732,  132248,  132261,  132279, 13, 18, 21, }, // Tonga
    {   71, 1738,       0,  132300,       0,  0, 12,  0, }, // Truk
    {   71, 1750,  132312,  132334,  132361, 22, 27, 30, }, // Turkmenistan
    {   71, 1763,       0,  132391,       0,  0, 14,  0, }, // Tuvalu
    {   71, 1777,  132405,  132419,  132438, 14, 19, 22, }, // Uruguay
    {   71, 1792,  132460,  132479,  132503, 19, 24, 27, }, // Uzbekistan
    {   71, 1803,  132530,  132544,  132563, 14, 19, 22, }, // Vanuatu
    {   71, 1811,       0,  132585,       0,  0, 18,  0, }, // Venezuela
    {   71, 1821,  132603,  132626,  132654, 23, 28, 31, }, // Vladivostok
    {   71, 1833,  132685,  132704,  132728, 19, 24, 25, }, // Volgograd
    {   71, 1843,       0,  132753,       0,  0, 17,  0, }, // Vostok
    {   71, 1850,       0,  132770,       0,  0, 19,  0, }, // Wake
    {   71, 1855,       0,  132789,       0,  0, 27,  0, }, // Wallis
    {   71, 1862,  132816,  132832,  132853, 16, 21, 24, }, // Yakutsk
    {   71, 1870,  132877,  132900,  132928, 23, 28, 30, }, // Yekaterinburg
    {   71, 1892,       0,  132958,       0,  0, 14,  0, }, // Yukon
    {   72,    0,  132972,  132976,  132982,  4,  6,  6, }, // Acre Cantonese/Traditional Han/Hong Kong
    {   72,    5,       0,  132988,       0,  0,  5,  0, }, // Afghanistan
    {   72,   17,       0,  132993,       0,  0,  4,  0, }, // Africa_Central
    {   72,   32,       0,  132997,       0,  0,  4,  0, }, // Africa_Eastern
    {   72,   65,       0,  133001,       0,  0,  6,  0, }, // Africa_Southern
    {   72,   81,  133007,  133011,  133017,  4,  6,  6, }, // Africa_Western
    {   72,  107,  133023,  133029,  133037,  6,  8,  8, }, // Alaska
    {   72,  128,  133045,  133051,  133059,  6,  8,  8, }, // Almaty
    {   72,  135,  133067,  133072,  133079,  5,  7,  7, }, // Amazon
    {   72,  142,  133086,  133090,  133096,  4,  6,  6, }, // America_Central
    {   72,  158,  133102,  133106,  133112,  4,  6,  6, }, // America_Eastern
    {   72,  174,  133118,  133122,  133128,  4,  6,  6, }, // America_Mountain
    {   72,  191,  133134,  133139,  133146,  5,  7,  7, }, // America_Pacific
    {   72,  207,  133153,  133159,  133167,  6,  8,  8, }, // Anadyr
    {   72,  214,  133175,  133180,  133187,  5,  7,  7, }, // Apia
    {   72,  219,  133194,  133199,  133206,  5,  7,  7, }, // Aqtau
    {   72,  225,  133213,  133219,  133227,  6,  8,  8, }, // Aqtobe
    {   72,  232,  133235,  133240,  133247,  5,  7,  7, }, // Arabian
    {   72,  240,  133254,  133259,  133266,  5,  7,  7, }, // Argentina
    {   72,  250,  133273,  133280,  133289,  7,  9,  9, }, // Argentina_Western
    {   72,  268,  133298,  133304,  133312,  6,  8,  8, }, // Armenia
    {   72,  286,  133320,  133325,  133332,  5,  7,  7, }, // Atlantic
    {   72,  295,  133339,  133345,  133353,  6,  8,  8, }, // Australia_Central
    {   72,  313,  133361,  133368,  133377,  7,  9,  9, }, // Australia_CentralWestern
    {   72,  338,  133386,  133392,  133400,  6,  8,  8, }, // Australia_Eastern
    {   72,  356,  133408,  133414,  133422,  6,  8,  8, }, // Australia_Western
    {   72,  374,  133430,  133436,  133444,  6,  8,  8, }, // Azerbaijan
    {   72,  385,  133452,  133459,  133468,  7,  9,  9, }, // Azores
    {   72,  397,  133477,  133482,  133489,  5,  7,  7, }, // Bangladesh
    {   72,  415,       0,  133496,       0,  0,  4,  0, }, // Bhutan
    {   72,  422,       0,  133500,       0,  0,  6,  0, }, // Bolivia
    {   72,  437,  133506,  133512,  133520,  6,  8,  8, }, // Brasilia
    {   72,  454,       0,  133528,       0,  0,  4,  0, }, // Brunei
    {   72,  461,  133532,  133537,  133544,  5,  7,  7, }, // Cape_Verde
    {   72,  472,       0,  133551,       0,  0,  5,  0, }, // Casey
    {   72,  478,       0,  133556,       0,  0,  5,  0, }, // Chamorro
    {   72,  487,  133561,  133567,  133575,  6,  8,  8, }, // Chatham
    {   72,  495,  133583,  133587,  133593,  4,  6,  6, }, // Chile
    {   72,  501,  133599,  133603,  133609,  4,  6,  6, }, // China
    {   72,  507,       0,  133615,       0,  0,  5,  0, }, // Christmas
    {   72,  517,       0,  133620,       0,  0,  7,  0, }, // Cocos
    {   72,  523,  133627,  133633,  133641,  6,  8,  8, }, // Colombia
    {   72,  532,  133649,  133655,  133663,  6,  8,  9, }, // Cook
    {   72,  537,  133672,  133676,  133682,  4,  6,  6, }, // Cuba
    {   72,  548,       0,  133688,       0,  0,  5,  0, }, // Davis
    {   72,  564,       0,  133693,       0,  0,  7,  0, }, // DumontDUrville
    {   72,  601,       0,  133700,       0,  0,  5,  0, }, // East_Timor
    {   72,  612,  133705,  133711,  133719,  6,  8,  8, }, // Easter
    {   72,  619,       0,  133727,       0,  0,  5,  0, }, // Ecuador
    {   72,  627,  133732,  133736,  133742,  4,  6,  6, }, // Europe_Central
    {   72,  642,  133748,  133752,  133758,  4,  6,  6, }, // Europe_Eastern
    {   72,  657,       0,  133764,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   72,  680,  133770,  133774,  133780,  4,  6,  6, }, // Europe_Western
    {   72,  695,  133786,  133793,  133802,  7,  9,  9, }, // Falkland
    {   72,  704,  133811,  133815,  133821,  4,  6,  6, }, // Fiji
    {   72,  709,       0,  133827,       0,  0,  7,  0, }, // French_Guiana
    {   72,  723,       0,  133834,       0,  0,  9,  0, }, // French_Southern
    {   72,  746,       0,  133843,       0,  0,  8,  0, }, // Galapagos
    {   72,  756,       0,  133851,       0,  0,  7,  0, }, // Gambier
    {   72,  764,  133858,  133863,  133870,  5,  7,  7, }, // Georgia
    {   72,  772,       0,  133877,       0,  0,  8,  0, }, // Gilbert_Islands
    {   72,  788,       0,  133885,       0,  0,  8,  0, }, // GMT
    {   72,  830,  133893,  133900,  133909,  7,  9,  9, }, // Greenland_Eastern
    {   72,  848,  133918,  133925,  133934,  7,  9,  9, }, // Greenland_Western
    {   72,  866,       0,  133943,       0,  0,  6,  0, }, // Guam
    {   72,  871,       0,  133949,       0,  0,  9,  0, }, // Gulf
    {   72,  876,       0,  133958,       0,  0,  5,  0, }, // Guyana
    {   72,  883,  133963,  133972,  133983,  9, 11, 11, }, // Hawaii_Aleutian
    {   72,  899,  133994,  133998,  134004,  4,  6,  6, }, // Hong_Kong
    {   72,  909,  134010,  134015,  134022,  5,  7,  7, }, // Hovd
    {   72,  914,       0,  134029,       0,  0,  6,  0, }, // India
    {   72,  920,       0,  134035,       0,  0,  5,  0, }, // Indian_Ocean
    {   72,  933,       0,  134040,       0,  0,  6,  0, }, // Indochina
    {   72,  943,       0,  134046,       0,  0,  6,  0, }, // Indonesia_Central
    {   72,  961,       0,  134052,       0,  0,  6,  0, }, // Indonesia_Eastern
    {   72,  979,       0,  134058,       0,  0,  6,  0, }, // Indonesia_Western
    {   72,  997,  134064,  134068,  134074,  4,  6,  6, }, // Iran
    {   72, 1008,  134080,  134087,  134096,  7,  9,  9, }, // Irkutsk
    {   72, 1016,  134105,  134110,  134117,  5,  7,  7, }, // Israel
    {   72, 1023,  134124,  134128,  134134,  4,  6,  6, }, // Japan
    {   72, 1029,  134140,  134151,  134164, 11, 13, 15, }, // Kamchatka
    {   72, 1047,       0,  134179,       0,  0,  5,  0, }, // Kazakhstan
    {   72, 1058,       0,  134184,       0,  0,  6,  0, }, // Kazakhstan_Eastern
    {   72, 1077,       0,  134190,       0,  0,  6,  0, }, // Kazakhstan_Western
    {   72, 1106,  134196,  134200,  134206,  4,  6,  6, }, // Korea
    {   72, 1112,       0,  134212,       0,  0,  5,  0, }, // Kosrae
    {   72, 1119,  134217,  134227,  134239, 10, 12, 12, }, // Krasnoyarsk
    {   72, 1151,       0,  134251,       0,  0,  6,  0, }, // Kyrgystan
    {   72, 1161,       0,  134257,       0,  0,  4,  0, }, // Lanka
    {   72, 1175,       0,  134261,       0,  0,  6,  0, }, // Line_Islands
    {   72, 1188,  134267,  134273,  134281,  6,  8,  8, }, // Lord_Howe
    {   72, 1198,  134289,  134293,  134299,  4,  6,  6, }, // Macau
    {   72, 1204,  134305,  134310,  134317,  5,  7,  7, }, // Magadan
    {   72, 1219,       0,  134324,       0,  0,  6,  0, }, // Malaysia
    {   72, 1228,       0,  134330,       0,  0,  6,  0, }, // Maldives
    {   72, 1237,       0,  134336,       0,  0,  6,  0, }, // Marquesas
    {   72, 1247,       0,  134342,       0,  0,  7,  0, }, // Marshall_Islands
    {   72, 1264,  134349,  134355,  134363,  6,  8,  8, }, // Mauritius
    {   72, 1274,       0,  134371,       0,  0,  4,  0, }, // Mawson
    {   72, 1281,  134375,  134383,  134393,  8, 10, 10, }, // Mexico_Pacific
    {   72, 1296,  134403,  134409,  134417,  6,  8,  8, }, // Mongolia
    {   72, 1305,  134425,  134430,  134437,  5,  7,  7, }, // Moscow
    {   72, 1312,       0,  134444,       0,  0,  4,  0, }, // Myanmar
    {   72, 1320,       0,  134448,       0,  0,  4,  0, }, // Nauru
    {   72, 1326,       0,  134452,       0,  0,  5,  0, }, // Nepal
    {   72, 1332,  134457,  134465,  134475,  8, 10, 12, }, // New_Caledonia
    {   72, 1346,  134487,  134492,  134499,  5,  7,  7, }, // New_Zealand
    {   72, 1358,  134506,  134511,  134518,  5,  7,  7, }, // Newfoundland
    {   72, 1371,       0,  134525,       0,  0,  5,  0, }, // Niue
    {   72, 1376,  134530,  134536,  134544,  6,  8,  8, }, // Norfolk
    {   72, 1384,  134552,  134566,  134582, 14, 16, 16, }, // Noronha
    {   72, 1392,       0,  134598,       0,  0,  9,  0, }, // North_Mariana
    {   72, 1406,  134607,  134614,  134623,  7,  9,  9, }, // Novosibirsk
    {   72, 1418,  134632,  134638,  134646,  6,  8,  8, }, // Omsk
    {   72, 1428,  134654,  134660,  134668,  6,  8,  8, }, // Pakistan
    {   72, 1437,       0,  134676,       0,  0,  4,  0, }, // Palau
    {   72, 1443,       0,  134680,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   72, 1460,  134689,  134694,  134701,  5,  7,  7, }, // Paraguay
    {   72, 1469,  134708,  134712,  134718,  4,  6,  6, }, // Peru
    {   72, 1474,  134724,  134729,  134736,  5,  7,  7, }, // Philippines
    {   72, 1486,       0,  134743,       0,  0,  6,  0, }, // Phoenix_Islands
    {   72, 1502,  134749,  134761,  134775, 12, 14, 14, }, // Pierre_Miquelon
    {   72, 1518,       0,  134789,       0,  0,  5,  0, }, // Pitcairn
    {   72, 1527,       0,  134794,       0,  0,  5,  0, }, // Ponape
    {   72, 1534,       0,  134799,       0,  0,  4,  0, }, // Pyongyang
    {   72, 1544,  134803,  134811,  134821,  8, 10, 10, }, // Qyzylorda
    {   72, 1554,       0,  134831,       0,  0,  5,  0, }, // Reunion
    {   72, 1562,       0,  134836,       0,  0,  5,  0, }, // Rothera
    {   72, 1570,  134841,  134846,  134853,  5,  7,  7, }, // Sakhalin
    {   72, 1579,  134860,  134865,  134872,  5,  7,  7, }, // Samara
    {   72, 1596,  134879,  134884,  134891,  5,  7,  7, }, // Samoa
    {   72, 1602,       0,  134898,       0,  0,  5,  0, }, // Seychelles
    {   72, 1624,       0,  134903,       0,  0,  7,  0, }, // Singapore
    {   72, 1634,       0,  134910,       0,  0,  7,  0, }, // Solomon
    {   72, 1642,       0,  134917,       0,  0,  6,  0, }, // South_Georgia
    {   72, 1656,       0,  134923,       0,  0,  5,  0, }, // Suriname
    {   72, 1676,       0,  134928,       0,  0,  6,  0, }, // Syowa
    {   72, 1682,       0,  134934,       0,  0,  5,  0, }, // Tahiti
    {   72, 1689,  134939,  134943,  134949,  4,  6,  6, }, // Taipei
    {   72, 1696,       0,  134955,       0,  0,  5,  0, }, // Tajikistan
    {   72, 1724,       0,  134960,       0,  0,  7,  0, }, // Tokelau
    {   72, 1732,  134967,  134971,  134977,  4,  6,  6, }, // Tonga
    {   72, 1738,       0,  134983,       0,  0,  5,  0, }, // Truk
    {   72, 1750,  134988,  134993,  135000,  5,  7,  7, }, // Turkmenistan
    {   72, 1763,       0,  135007,       0,  0,  5,  0, }, // Tuvalu
    {   72, 1777,  135012,  135017,  135024,  5,  7,  7, }, // Uruguay
    {   72, 1792,  135031,  135037,  135045,  6,  8,  8, }, // Uzbekistan
    {   72, 1803,  135053,  135058,  135065,  5,  7,  7, }, // Vanuatu
    {   72, 1811,       0,  135072,       0,  0,  6,  0, }, // Venezuela
    {   72, 1821,  135078,  135083,  135090,  5,  7,  7, }, // Vladivostok
    {   72, 1833,  135097,  135104,  135113,  7,  9,  9, }, // Volgograd
    {   72, 1843,       0,  135122,       0,  0,  6,  0, }, // Vostok
    {   72, 1850,       0,  135128,       0,  0,  5,  0, }, // Wake
    {   72, 1855,       0,  135133,       0,  0, 11,  0, }, // Wallis
    {   72, 1862,  135144,  135150,  135158,  6,  8,  8, }, // Yakutsk
    {   72, 1870,  135166,  135173,  135182,  7,  9,  9, }, // Yekaterinburg
    {   72, 1892,       0,  135191,       0,  0,  4,  0, }, // Yukon
    {   73,    0,  135195,  135199,  135205,  4,  6,  6, }, // Acre Cantonese/Simplified Han/China
    {   73,    5,       0,  135211,       0,  0,  5,  0, }, // Afghanistan
    {   73,   17,       0,  135216,       0,  0,  4,  0, }, // Africa_Central
    {   73,   32,       0,  135220,       0,  0,  4,  0, }, // Africa_Eastern
    {   73,   65,       0,  135224,       0,  0,  6,  0, }, // Africa_Southern
    {   73,   81,  135230,  135234,  135240,  4,  6,  6, }, // Africa_Western
    {   73,  107,  135246,  135252,  135260,  6,  8,  8, }, // Alaska
    {   73,  128,  135268,  135274,  135282,  6,  8,  8, }, // Almaty
    {   73,  135,  135290,  135295,  135302,  5,  7,  7, }, // Amazon
    {   73,  142,  135309,  135313,  135319,  4,  6,  6, }, // America_Central
    {   73,  158,  135325,  135329,  135335,  4,  6,  6, }, // America_Eastern
    {   73,  174,  135341,  135345,  135351,  4,  6,  6, }, // America_Mountain
    {   73,  191,  135357,  135362,  135369,  5,  7,  7, }, // America_Pacific
    {   73,  207,  135376,  135382,  135390,  6,  8,  8, }, // Anadyr
    {   73,  214,  135398,  135403,  135410,  5,  7,  7, }, // Apia
    {   73,  219,  135417,  135422,  135429,  5,  7,  7, }, // Aqtau
    {   73,  225,  135436,  135442,  135450,  6,  8,  8, }, // Aqtobe
    {   73,  232,  135458,  135463,  135470,  5,  7,  7, }, // Arabian
    {   73,  240,  135477,  135482,  135489,  5,  7,  7, }, // Argentina
    {   73,  250,  135496,  135503,  135512,  7,  9,  9, }, // Argentina_Western
    {   73,  268,  135521,  135527,  135535,  6,  8,  8, }, // Armenia
    {   73,  286,  135543,  135548,  135555,  5,  7,  7, }, // Atlantic
    {   73,  295,  135562,  135568,  135576,  6,  8,  8, }, // Australia_Central
    {   73,  313,  135584,  135591,  135600,  7,  9,  9, }, // Australia_CentralWestern
    {   73,  338,  135609,  135615,  135623,  6,  8,  8, }, // Australia_Eastern
    {   73,  356,  135631,  135637,  135645,  6,  8,  8, }, // Australia_Western
    {   73,  374,  135653,  135659,  135667,  6,  8,  8, }, // Azerbaijan
    {   73,  385,  135675,  135682,  135691,  7,  9,  9, }, // Azores
    {   73,  397,  135700,  135705,  135712,  5,  7,  7, }, // Bangladesh
    {   73,  415,       0,  135719,       0,  0,  4,  0, }, // Bhutan
    {   73,  422,       0,  135723,       0,  0,  6,  0, }, // Bolivia
    {   73,  437,  135729,  135735,  135743,  6,  8,  8, }, // Brasilia
    {   73,  454,       0,  135751,       0,  0,  4,  0, }, // Brunei
    {   73,  461,  135755,  135760,  135767,  5,  7,  7, }, // Cape_Verde
    {   73,  472,       0,  135774,       0,  0,  5,  0, }, // Casey
    {   73,  478,       0,  135779,       0,  0,  5,  0, }, // Chamorro
    {   73,  487,  135784,  135790,  135798,  6,  8,  8, }, // Chatham
    {   73,  495,  135806,  135810,  135816,  4,  6,  6, }, // Chile
    {   73,  501,  135822,  135826,  135832,  4,  6,  6, }, // China
    {   73,  507,       0,  135838,       0,  0,  5,  0, }, // Christmas
    {   73,  517,       0,  135843,       0,  0,  7,  0, }, // Cocos
    {   73,  523,  135850,  135856,  135864,  6,  8,  8, }, // Colombia
    {   73,  532,  135872,  135878,  135886,  6,  8,  9, }, // Cook
    {   73,  537,  135895,  135899,  135905,  4,  6,  6, }, // Cuba
    {   73,  548,       0,  135911,       0,  0,  5,  0, }, // Davis
    {   73,  564,       0,  135916,       0,  0,  7,  0, }, // DumontDUrville
    {   73,  601,       0,  135923,       0,  0,  5,  0, }, // East_Timor
    {   73,  612,  135928,  135934,  135942,  6,  8,  8, }, // Easter
    {   73,  619,       0,  135950,       0,  0,  5,  0, }, // Ecuador
    {   73,  627,  135955,  135959,  135965,  4,  6,  6, }, // Europe_Central
    {   73,  642,  135971,  135975,  135981,  4,  6,  6, }, // Europe_Eastern
    {   73,  657,       0,  135987,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   73,  680,  135993,  135997,  136003,  4,  6,  6, }, // Europe_Western
    {   73,  695,  136009,  136016,  136025,  7,  9,  9, }, // Falkland
    {   73,  704,  136034,  136038,  136044,  4,  6,  6, }, // Fiji
    {   73,  709,       0,  136050,       0,  0,  7,  0, }, // French_Guiana
    {   73,  723,       0,  136057,       0,  0,  9,  0, }, // French_Southern
    {   73,  746,       0,  136066,       0,  0,  8,  0, }, // Galapagos
    {   73,  756,       0,  136074,       0,  0,  7,  0, }, // Gambier
    {   73,  764,  136081,  136086,  136093,  5,  7,  7, }, // Georgia
    {   73,  772,       0,  136100,       0,  0,  8,  0, }, // Gilbert_Islands
    {   73,  788,       0,  136108,       0,  0,  8,  0, }, // GMT
    {   73,  830,  136116,  136123,  136132,  7,  9,  9, }, // Greenland_Eastern
    {   73,  848,  136141,  136148,  136157,  7,  9,  9, }, // Greenland_Western
    {   73,  866,       0,  136166,       0,  0,  6,  0, }, // Guam
    {   73,  871,       0,  136172,       0,  0,  9,  0, }, // Gulf
    {   73,  876,       0,  136181,       0,  0,  5,  0, }, // Guyana
    {   73,  883,  136186,  136195,  136206,  9, 11, 11, }, // Hawaii_Aleutian
    {   73,  899,  136217,  136221,  136227,  4,  6,  6, }, // Hong_Kong
    {   73,  909,  136233,  136238,  136245,  5,  7,  7, }, // Hovd
    {   73,  914,       0,  136252,       0,  0,  6,  0, }, // India
    {   73,  920,       0,  136258,       0,  0,  5,  0, }, // Indian_Ocean
    {   73,  933,       0,  136263,       0,  0,  6,  0, }, // Indochina
    {   73,  943,       0,  136269,       0,  0,  6,  0, }, // Indonesia_Central
    {   73,  961,       0,  136275,       0,  0,  6,  0, }, // Indonesia_Eastern
    {   73,  979,       0,  136281,       0,  0,  6,  0, }, // Indonesia_Western
    {   73,  997,  136287,  136291,  136297,  4,  6,  6, }, // Iran
    {   73, 1008,  136303,  136310,  136319,  7,  9,  9, }, // Irkutsk
    {   73, 1016,  136328,  136333,  136340,  5,  7,  7, }, // Israel
    {   73, 1023,  136347,  136351,  136357,  4,  6,  6, }, // Japan
    {   73, 1029,  136363,  136374,  136387, 11, 13, 15, }, // Kamchatka
    {   73, 1047,       0,  136402,       0,  0,  5,  0, }, // Kazakhstan
    {   73, 1058,       0,  136407,       0,  0,  6,  0, }, // Kazakhstan_Eastern
    {   73, 1077,       0,  136413,       0,  0,  6,  0, }, // Kazakhstan_Western
    {   73, 1106,  136419,  136423,  136429,  4,  6,  6, }, // Korea
    {   73, 1112,       0,  136435,       0,  0,  5,  0, }, // Kosrae
    {   73, 1119,  136440,  136450,  136462, 10, 12, 12, }, // Krasnoyarsk
    {   73, 1151,       0,  136474,       0,  0,  6,  0, }, // Kyrgystan
    {   73, 1161,       0,  136480,       0,  0,  4,  0, }, // Lanka
    {   73, 1175,       0,  136484,       0,  0,  6,  0, }, // Line_Islands
    {   73, 1188,  136490,  136496,  136504,  6,  8,  8, }, // Lord_Howe
    {   73, 1198,  136512,  136516,  136522,  4,  6,  6, }, // Macau
    {   73, 1204,  136528,  136533,  136540,  5,  7,  7, }, // Magadan
    {   73, 1219,       0,  136547,       0,  0,  6,  0, }, // Malaysia
    {   73, 1228,       0,  136553,       0,  0,  6,  0, }, // Maldives
    {   73, 1237,       0,  136559,       0,  0,  6,  0, }, // Marquesas
    {   73, 1247,       0,  136565,       0,  0,  7,  0, }, // Marshall_Islands
    {   73, 1264,  136572,  136578,  136586,  6,  8,  8, }, // Mauritius
    {   73, 1274,       0,  136594,       0,  0,  4,  0, }, // Mawson
    {   73, 1281,  136598,  136606,  136616,  8, 10, 10, }, // Mexico_Pacific
    {   73, 1296,  136626,  136632,  136640,  6,  8,  8, }, // Mongolia
    {   73, 1305,  136648,  136653,  136660,  5,  7,  7, }, // Moscow
    {   73, 1312,       0,  136667,       0,  0,  4,  0, }, // Myanmar
    {   73, 1320,       0,  136671,       0,  0,  4,  0, }, // Nauru
    {   73, 1326,       0,  136675,       0,  0,  5,  0, }, // Nepal
    {   73, 1332,  136680,  136688,  136698,  8, 10, 12, }, // New_Caledonia
    {   73, 1346,  136710,  136715,  136722,  5,  7,  7, }, // New_Zealand
    {   73, 1358,  136729,  136734,  136741,  5,  7,  7, }, // Newfoundland
    {   73, 1371,       0,  136748,       0,  0,  5,  0, }, // Niue
    {   73, 1376,  136753,  136759,  136767,  6,  8,  8, }, // Norfolk
    {   73, 1384,  136775,  136789,  136805, 14, 16, 16, }, // Noronha
    {   73, 1392,       0,  136821,       0,  0,  9,  0, }, // North_Mariana
    {   73, 1406,  136830,  136837,  136846,  7,  9,  9, }, // Novosibirsk
    {   73, 1418,  136855,  136861,  136869,  6,  8,  8, }, // Omsk
    {   73, 1428,  136877,  136883,  136891,  6,  8,  8, }, // Pakistan
    {   73, 1437,       0,  136899,       0,  0,  4,  0, }, // Palau
    {   73, 1443,       0,  136903,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   73, 1460,  136912,  136917,  136924,  5,  7,  7, }, // Paraguay
    {   73, 1469,  136931,  136935,  136941,  4,  6,  6, }, // Peru
    {   73, 1474,  136947,  136952,  136959,  5,  7,  7, }, // Philippines
    {   73, 1486,       0,  136966,       0,  0,  6,  0, }, // Phoenix_Islands
    {   73, 1502,  136972,  136984,  136998, 12, 14, 14, }, // Pierre_Miquelon
    {   73, 1518,       0,  137012,       0,  0,  5,  0, }, // Pitcairn
    {   73, 1527,       0,  137017,       0,  0,  5,  0, }, // Ponape
    {   73, 1534,       0,  137022,       0,  0,  4,  0, }, // Pyongyang
    {   73, 1544,  137026,  137034,  137044,  8, 10, 10, }, // Qyzylorda
    {   73, 1554,       0,  137054,       0,  0,  5,  0, }, // Reunion
    {   73, 1562,       0,  137059,       0,  0,  5,  0, }, // Rothera
    {   73, 1570,  137064,  137069,  137076,  5,  7,  7, }, // Sakhalin
    {   73, 1579,  137083,  137088,  137095,  5,  7,  7, }, // Samara
    {   73, 1596,  137102,  137107,  137114,  5,  7,  7, }, // Samoa
    {   73, 1602,       0,  137121,       0,  0,  5,  0, }, // Seychelles
    {   73, 1624,       0,  137126,       0,  0,  7,  0, }, // Singapore
    {   73, 1634,       0,  137133,       0,  0,  7,  0, }, // Solomon
    {   73, 1642,       0,  137140,       0,  0,  6,  0, }, // South_Georgia
    {   73, 1656,       0,  137146,       0,  0,  5,  0, }, // Suriname
    {   73, 1676,       0,  137151,       0,  0,  6,  0, }, // Syowa
    {   73, 1682,       0,  137157,       0,  0,  5,  0, }, // Tahiti
    {   73, 1689,  137162,  137166,  137172,  4,  6,  6, }, // Taipei
    {   73, 1696,       0,  137178,       0,  0,  5,  0, }, // Tajikistan
    {   73, 1724,       0,  137183,       0,  0,  7,  0, }, // Tokelau
    {   73, 1732,  137190,  137194,  137200,  4,  6,  6, }, // Tonga
    {   73, 1738,       0,  137206,       0,  0,  5,  0, }, // Truk
    {   73, 1750,  137211,  137216,  137223,  5,  7,  7, }, // Turkmenistan
    {   73, 1763,       0,  137230,       0,  0,  5,  0, }, // Tuvalu
    {   73, 1777,  137235,  137240,  137247,  5,  7,  7, }, // Uruguay
    {   73, 1792,  137254,  137260,  137268,  6,  8,  8, }, // Uzbekistan
    {   73, 1803,  137276,  137281,  137288,  5,  7,  7, }, // Vanuatu
    {   73, 1811,       0,  137295,       0,  0,  6,  0, }, // Venezuela
    {   73, 1821,  137301,  137306,  137313,  5,  7,  7, }, // Vladivostok
    {   73, 1833,  137320,  137327,  137336,  7,  9,  9, }, // Volgograd
    {   73, 1843,       0,  137345,       0,  0,  6,  0, }, // Vostok
    {   73, 1850,       0,  137351,       0,  0,  5,  0, }, // Wake
    {   73, 1855,       0,  137356,       0,  0, 11,  0, }, // Wallis
    {   73, 1862,  137367,  137373,  137381,  6,  8,  8, }, // Yakutsk
    {   73, 1870,  137389,  137396,  137405,  7,  9,  9, }, // Yekaterinburg
    {   73, 1892,       0,  137414,       0,  0,  4,  0, }, // Yukon
    {   75,    5,       0,  137418,       0,  0, 20,  0, }, // Afghanistan Catalan/Latin/Spain
    {   75,   17,       0,  137438,       0,  0, 24,  0, }, // Africa_Central
    {   75,   32,       0,  137462,       0,  0, 25,  0, }, // Africa_Eastern
    {   75,   65,       0,  137487,       0,  0, 37,  0, }, // Africa_Southern
    {   75,   81,  137524,  137551,  137588, 27, 37, 35, }, // Africa_Western
    {   75,  107,   57204,  137623,  137646, 13, 23, 21, }, // Alaska
    {   75,  135,  137667,  137685,  137713, 18, 28, 26, }, // Amazon
    {   75,  142,  137739,  137770,  137811, 31, 41, 39, }, // America_Central
    {   75,  158,  137850,  137882,  137924, 32, 42, 40, }, // America_Eastern
    {   75,  174,  137964,  137999,  138044, 35, 45, 43, }, // America_Mountain
    {   75,  191,  138087,  138122,  138167, 35, 45, 43, }, // America_Pacific
    {   75,  207,  138210,  138223,  138246, 13, 23, 23, }, // Anadyr
    {   75,  214,   57885,  138269,  138290, 11, 21, 19, }, // Apia
    {   75,  232,  138309,  138318,  138337,  9, 19, 17, }, // Arabian
    {   75,  240,  138354,  138373,  138402, 19, 29, 27, }, // Argentina
    {   75,  250,  138429,  138458,  138497, 29, 39, 37, }, // Argentina_Western
    {   75,  268,  138534,  138548,  138572, 14, 24, 22, }, // Armenia
    {   75,  286,  138594,  138612,  138640, 18, 28, 26, }, // Atlantic
    {   75,  295,  138666,  138690,  138724, 24, 34, 32, }, // Australia_Central
    {   75,  313,  138756,  138790,  138834, 34, 44, 42, }, // Australia_CentralWestern
    {   75,  338,  138876,  138901,  138936, 25, 35, 33, }, // Australia_Eastern
    {   75,  356,  138969,  138996,  139033, 27, 37, 35, }, // Australia_Western
    {   75,  374,  139068,  139089,  139120, 21, 31, 29, }, // Azerbaijan
    {   75,  385,  139149,  139167,  139195, 18, 28, 26, }, // Azores
    {   75,  397,  139221,  139239,  139267, 18, 28, 26, }, // Bangladesh
    {   75,  415,       0,  139293,       0,  0, 14,  0, }, // Bhutan
    {   75,  422,       0,  139307,       0,  0, 15,  0, }, // Bolivia
    {   75,  437,  139322,  139338,  139364, 16, 26, 24, }, // Brasilia
    {   75,  454,       0,  139388,       0,  0, 25,  0, }, // Brunei
    {   75,  461,  139413,  139429,  139455, 16, 26, 24, }, // Cape_Verde
    {   75,  478,       0,  139479,       0,  0, 26,  0, }, // Chamorro
    {   75,  487,   59239,  139505,  139530, 15, 25, 23, }, // Chatham
    {   75,  495,  139553,  139565,  139587, 12, 22, 20, }, // Chile
    {   75,  501,  139607,  139622,  139647, 15, 25, 23, }, // China
    {   75,  507,       0,  139670,       0,  0, 18,  0, }, // Christmas
    {   75,  517,       0,  139688,       0,  0, 23,  0, }, // Cocos
    {   75,  523,  139711,  139727,  139753, 16, 26, 24, }, // Colombia
    {   75,  532,  139777,  139799,  139831, 22, 32, 35, }, // Cook
    {   75,  537,   59634,  139866,  139888, 12, 22, 20, }, // Cuba
    {   75,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {   75,  564,       0,  139908,       0,  0, 24,  0, }, // DumontDUrville
    {   75,  601,       0,   59725,       0,  0, 22,  0, }, // East_Timor
    {   75,  612,  139932,  139956,  139990, 24, 34, 32, }, // Easter
    {   75,  619,       0,  140022,       0,  0, 17,  0, }, // Ecuador
    {   75,  627,  140039,  140060,  140091, 21, 31, 29, }, // Europe_Central
    {   75,  642,  140120,  140142,  140174, 22, 32, 30, }, // Europe_Eastern
    {   75,  657,       0,  140204,       0,  0, 34,  0, }, // Europe_Further_Eastern
    {   75,  680,  140238,  140262,  140296, 24, 34, 32, }, // Europe_Western
    {   75,  695,  140328,  140354,  140390, 26, 36, 34, }, // Falkland
    {   75,  704,  140424,  140436,  140458, 12, 22, 20, }, // Fiji
    {   75,  709,       0,  140478,       0,  0, 27,  0, }, // French_Guiana
    {   75,  723,       0,  140505,       0,  0, 63,  0, }, // French_Southern
    {   75,  746,       0,   60355,       0,  0, 17,  0, }, // Galapagos
    {   75,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {   75,  764,  140568,  140583,  140608, 15, 25, 23, }, // Georgia
    {   75,  772,       0,  140631,       0,  0, 25,  0, }, // Gilbert_Islands
    {   75,  788,       0,  140656,       0,  0, 29,  0, }, // GMT
    {   75,  802,  140685,  140704,  140733, 19, 29, 27, }, // Greenland
    {   75,  830,  140760,  140788,  140826, 28, 38, 36, }, // Greenland_Eastern
    {   75,  848,  140862,  140891,  140930, 29, 39, 37, }, // Greenland_Western
    {   75,  871,       0,  140967,       0,  0, 23,  0, }, // Gulf
    {   75,  876,       0,  140990,       0,  0, 14,  0, }, // Guyana
    {   75,  883,   60750,  141004,  141039, 25, 35, 33, }, // Hawaii_Aleutian
    {   75,  899,  141072,  141089,  141116, 17, 27, 25, }, // Hong_Kong
    {   75,  909,  141141,  141154,  141177, 13, 23, 21, }, // Hovd
    {   75,  914,       0,  141198,       0,  0, 15,  0, }, // India
    {   75,  920,       0,  141213,       0,  0, 20,  0, }, // Indian_Ocean
    {   75,  933,       0,  141233,       0,  0, 18,  0, }, // Indochina
    {   75,  943,       0,  141251,       0,  0, 24,  0, }, // Indonesia_Central
    {   75,  961,       0,  141275,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   75,  979,       0,  141300,       0,  0, 26,  0, }, // Indonesia_Western
    {   75,  997,  141326,  141340,  141364, 14, 24, 22, }, // Iran
    {   75, 1008,   61155,  141386,  141410, 14, 24, 22, }, // Irkutsk
    {   75, 1016,   61215,  141432,  141455, 13, 23, 21, }, // Israel
    {   75, 1023,  141476,  141489,  141512, 13, 23, 21, }, // Japan
    {   75, 1029,  141533,  141550,  141594, 17, 44, 44, }, // Kamchatka
    {   75, 1047,       0,  141638,       0,  0, 19,  0, }, // Kazakhstan
    {   75, 1058,       0,  141657,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {   75, 1077,       0,  141685,       0,  0, 29,  0, }, // Kazakhstan_Western
    {   75, 1106,   61499,  141714,  141737, 13, 23, 21, }, // Korea
    {   75, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {   75, 1119,  141758,  141777,  141806, 19, 29, 27, }, // Krasnoyarsk
    {   75, 1151,       0,  141833,       0,  0, 20,  0, }, // Kyrgystan
    {   75, 1175,       0,  141853,       0,  0, 22,  0, }, // Line_Islands
    {   75, 1188,   61700,  141875,  141902, 17, 27, 27, }, // Lord_Howe
    {   75, 1198,  141929,  141942,  141965, 13, 23, 21, }, // Macau
    {   75, 1204,  141986,  142001,  142026, 15, 25, 23, }, // Magadan
    {   75, 1219,       0,  142049,       0,  0, 16,  0, }, // Malaysia
    {   75, 1228,       0,  142065,       0,  0, 20,  0, }, // Maldives
    {   75, 1237,       0,   61924,       0,  0, 21,  0, }, // Marquesas
    {   75, 1247,       0,  142085,       0,  0, 26,  0, }, // Marshall_Islands
    {   75, 1264,   61972,  142111,  142136, 15, 25, 23, }, // Mauritius
    {   75, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {   75, 1281,  142159,  142184,  142219, 25, 35, 33, }, // Mexico_Pacific
    {   75, 1296,  142252,  142270,  142298, 18, 28, 26, }, // Mongolia
    {   75, 1305,  142324,  142338,  142362, 14, 24, 22, }, // Moscow
    {   75, 1312,       0,   62277,       0,  0, 15,  0, }, // Myanmar
    {   75, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {   75, 1326,       0,   62305,       0,  0, 14,  0, }, // Nepal
    {   75, 1332,  142384,  142406,  142438, 22, 32, 30, }, // New_Caledonia
    {   75, 1346,  142468,  142488,  142518, 20, 30, 28, }, // New_Zealand
    {   75, 1358,  142546,  142563,  142590, 17, 27, 25, }, // Newfoundland
    {   75, 1371,       0,   62565,       0,  0, 12,  0, }, // Niue
    {   75, 1376,  142615,  142637,  142669, 22, 32, 30, }, // Norfolk
    {   75, 1384,   62667,  142699,  142736, 27, 37, 35, }, // Noronha
    {   75, 1406,  142771,  142791,  142821, 20, 30, 28, }, // Novosibirsk
    {   75, 1418,   62878,  142849,  142870, 11, 21, 19, }, // Omsk
    {   75, 1428,  142889,  142906,  142933, 17, 27, 25, }, // Pakistan
    {   75, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {   75, 1443,       0,  142958,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   75, 1460,  142983,  143000,  143027, 17, 27, 25, }, // Paraguay
    {   75, 1469,   63109,  143052,  143075, 13, 23, 21, }, // Peru
    {   75, 1474,  143096,  143117,  143148, 21, 31, 29, }, // Philippines
    {   75, 1486,       0,  143177,       0,  0, 25,  0, }, // Phoenix_Islands
    {   75, 1502,  143202,  143234,  143276, 32, 42, 40, }, // Pierre_Miquelon
    {   75, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {   75, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {   75, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {   75, 1554,       0,   63488,       0,  0, 14,  0, }, // Reunion
    {   75, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {   75, 1570,  143333,  143349,  143375, 16, 26, 24, }, // Sakhalin
    {   75, 1579,   63581,  143399,  143423, 14, 24, 22, }, // Samara
    {   75, 1596,   63641,  143445,  143468, 13, 23, 21, }, // Samoa
    {   75, 1602,       0,  143489,       0,  0, 22,  0, }, // Seychelles
    {   75, 1624,       0,  143511,       0,  0, 16,  0, }, // Singapore
    {   75, 1634,       0,  143527,       0,  0, 24,  0, }, // Solomon
    {   75, 1642,       0,  143551,       0,  0, 23,  0, }, // South_Georgia
    {   75, 1656,       0,  143574,       0,  0, 15,  0, }, // Suriname
    {   75, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {   75, 1682,       0,  143589,       0,  0, 14,  0, }, // Tahiti
    {   75, 1689,  143603,  143617,  143641, 14, 24, 22, }, // Taipei
    {   75, 1696,       0,  143663,       0,  0, 20,  0, }, // Tajikistan
    {   75, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {   75, 1732,   63929,  143683,  143706, 13, 23, 21, }, // Tonga
    {   75, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {   75, 1750,  143727,  143748,  143779, 21, 31, 29, }, // Turkmenistan
    {   75, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {   75, 1777,  143808,  143825,  143852, 17, 27, 25, }, // Uruguay
    {   75, 1792,  143877,  143897,  143927, 20, 30, 28, }, // Uzbekistan
    {   75, 1803,  143955,  143969,  143993, 14, 24, 22, }, // Vanuatu
    {   75, 1811,       0,  144015,       0,  0, 17,  0, }, // Venezuela
    {   75, 1821,   64318,  144032,  144061, 19, 29, 27, }, // Vladivostok
    {   75, 1833,  144088,  144105,  144132, 17, 27, 25, }, // Volgograd
    {   75, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {   75, 1850,       0,  144157,       0,  0, 22,  0, }, // Wake
    {   75, 1855,       0,  144179,       0,  0, 23,  0, }, // Wallis
    {   75, 1862,  144202,  144217,  144242, 15, 25, 23, }, // Yakutsk
    {   75, 1870,  144265,  144286,  144317, 21, 31, 29, }, // Yekaterinburg
    {   75, 1892,       0,  144346,       0,  0, 13,  0, }, // Yukon
    {   79,    5,       0,  144359,       0,  0, 19,  0, }, // Afghanistan Cebuano/Latin/Philippines
    {   79,   17,       0,  144378,       0,  0, 22,  0, }, // Africa_Central
    {   79,   32,       0,  144400,       0,  0, 19,  0, }, // Africa_Eastern
    {   79,   65,       0,  144419,       0,  0, 33,  0, }, // Africa_Southern
    {   79,   81,  144452,  144471,  144503, 19, 32, 32, }, // Africa_Western
    {   79,  107,  144535,  144549,  144576, 14, 27, 27, }, // Alaska
    {   79,  135,  144603,  144617,  144644, 14, 27, 27, }, // Amazon
    {   79,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {   79,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {   79,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {   79,  191,  144671,  144687,  144716, 16, 29, 29, }, // America_Pacific
    {   79,  214,  144745,  144757,  144782, 12, 25, 25, }, // Apia
    {   79,  232,  144807,  144821,  144848, 14, 27, 27, }, // Arabian
    {   79,  240,  144875,  144892,  144922, 17, 30, 30, }, // Argentina
    {   79,  250,  144952,  144977,  145015, 25, 38, 38, }, // Argentina_Western
    {   79,  268,  145053,  145068,  145096, 15, 28, 28, }, // Armenia
    {   79,  286,  145124,  145140,  145169, 16, 29, 29, }, // Atlantic
    {   79,  295,  145198,  145223,  145261, 25, 38, 38, }, // Australia_Central
    {   79,  313,  145299,  145332,  145378, 33, 46, 46, }, // Australia_CentralWestern
    {   79,  338,  145424,  145449,  145487, 25, 38, 38, }, // Australia_Eastern
    {   79,  356,  145525,  145550,  145588, 25, 38, 38, }, // Australia_Western
    {   79,  374,  145626,  145644,  145675, 18, 31, 31, }, // Azerbaijan
    {   79,  385,  145706,  145720,  145747, 14, 27, 27, }, // Azores
    {   79,  397,  145774,  145792,  145823, 18, 31, 31, }, // Bangladesh
    {   79,  415,       0,  145854,       0,  0, 14,  0, }, // Bhutan
    {   79,  422,       0,  145868,       0,  0, 15,  0, }, // Bolivia
    {   79,  437,  145883,  145899,  145928, 16, 29, 29, }, // Brasilia
    {   79,  454,       0,  145957,       0,  0, 25,  0, }, // Brunei
    {   79,  461,  145982,  146000,  146031, 18, 31, 31, }, // Cape_Verde
    {   79,  478,       0,  146062,       0,  0, 29,  0, }, // Chamorro
    {   79,  487,  146091,  146106,  146134, 15, 28, 28, }, // Chatham
    {   79,  495,  146162,  146175,  146201, 13, 26, 26, }, // Chile
    {   79,  501,  146227,  146240,  146266, 13, 26, 26, }, // China
    {   79,  507,       0,  146292,       0,  0, 24,  0, }, // Christmas
    {   79,  517,       0,  146316,       0,  0, 21,  0, }, // Cocos
    {   79,  523,  146337,  146353,  146382, 16, 29, 29, }, // Colombia
    {   79,  532,  146411,  146431,  146464, 20, 33, 43, }, // Cook
    {   79,  537,  146507,  146519,  146544, 12, 25, 25, }, // Cuba
    {   79,  548,       0,  146569,       0,  0, 13,  0, }, // Davis
    {   79,  564,       0,  146582,       0,  0, 24,  0, }, // DumontDUrville
    {   79,  601,       0,  146606,       0,  0, 18,  0, }, // East_Timor
    {   79,  612,  146624,  146645,  146679, 21, 34, 34, }, // Easter
    {   79,  619,       0,  146713,       0,  0, 15,  0, }, // Ecuador
    {   79,  627,  146728,  146750,  146785, 22, 35, 35, }, // Europe_Central
    {   79,  642,  146820,  146842,  146877, 22, 35, 35, }, // Europe_Eastern
    {   79,  657,       0,  146912,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {   79,  680,  146942,  146964,  146999, 22, 35, 35, }, // Europe_Western
    {   79,  695,  147034,  147058,  147095, 24, 37, 37, }, // Falkland
    {   79,  704,  147132,  147144,  147169, 12, 25, 25, }, // Fiji
    {   79,  709,       0,  147194,       0,  0, 21,  0, }, // French_Guiana
    {   79,  723,       0,  147215,       0,  0, 36,  0, }, // French_Southern
    {   79,  746,       0,  147251,       0,  0, 17,  0, }, // Galapagos
    {   79,  756,       0,  147268,       0,  0, 15,  0, }, // Gambier
    {   79,  764,  147283,  147298,  147326, 15, 28, 28, }, // Georgia
    {   79,  772,       0,  147354,       0,  0, 23,  0, }, // Gilbert_Islands
    {   79,  788,       0,  147377,       0,  0, 22,  0, }, // GMT
    {   79,  830,  147399,  147421,  147456, 22, 35, 35, }, // Greenland_Eastern
    {   79,  848,  147491,  147513,  147548, 22, 35, 35, }, // Greenland_Western
    {   79,  871,       0,  147583,       0,  0, 25,  0, }, // Gulf
    {   79,  876,       0,  147608,       0,  0, 14,  0, }, // Guyana
    {   79,  883,  147622,  147645,  147681, 23, 36, 36, }, // Hawaii_Aleutian
    {   79,  899,  147717,  147734,  147764, 17, 30, 30, }, // Hong_Kong
    {   79,  909,  147794,  147806,  147831, 12, 25, 25, }, // Hovd
    {   79,  914,       0,  147856,       0,  0, 26,  0, }, // India
    {   79,  920,       0,  147882,       0,  0, 20,  0, }, // Indian_Ocean
    {   79,  933,       0,  147902,       0,  0, 17,  0, }, // Indochina
    {   79,  943,       0,  147919,       0,  0, 25,  0, }, // Indonesia_Central
    {   79,  961,       0,  147944,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   79,  979,       0,  147969,       0,  0, 25,  0, }, // Indonesia_Western
    {   79,  997,  147994,  148006,  148031, 12, 25, 25, }, // Iran
    {   79, 1008,  148056,  148071,  148099, 15, 28, 28, }, // Irkutsk
    {   79, 1016,  148127,  148141,  148168, 14, 27, 27, }, // Israel
    {   79, 1023,  148195,  148208,  148234, 13, 26, 26, }, // Japan
    {   79, 1047,       0,  148260,       0,  0, 18,  0, }, // Kazakhstan
    {   79, 1058,       0,  148278,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {   79, 1077,       0,  148301,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   79, 1106,  148324,  148337,  148363, 13, 26, 26, }, // Korea
    {   79, 1112,       0,  148389,       0,  0, 14,  0, }, // Kosrae
    {   79, 1119,  148403,  148422,  148454, 19, 32, 32, }, // Krasnoyarsk
    {   79, 1151,       0,  148486,       0,  0, 18,  0, }, // Kyrgystan
    {   79, 1175,       0,  148504,       0,  0, 20,  0, }, // Line_Islands
    {   79, 1188,  148524,  148541,  148571, 17, 30, 30, }, // Lord_Howe
    {   79, 1204,  148601,  148616,  148644, 15, 28, 28, }, // Magadan
    {   79, 1219,       0,  148672,       0,  0, 16,  0, }, // Malaysia
    {   79, 1228,       0,  148688,       0,  0, 16,  0, }, // Maldives
    {   79, 1237,       0,  148704,       0,  0, 17,  0, }, // Marquesas
    {   79, 1247,       0,  148721,       0,  0, 24,  0, }, // Marshall_Islands
    {   79, 1264,  148745,  148762,  148792, 17, 30, 30, }, // Mauritius
    {   79, 1274,       0,  148822,       0,  0, 14,  0, }, // Mawson
    {   79, 1281,  148836,  148859,  148895, 23, 36, 36, }, // Mexico_Pacific
    {   79, 1296,  148931,  148950,  148982, 19, 32, 32, }, // Mongolia
    {   79, 1305,  149014,  149028,  149055, 14, 27, 27, }, // Moscow
    {   79, 1312,       0,  149082,       0,  0, 15,  0, }, // Myanmar
    {   79, 1320,       0,  149097,       0,  0, 13,  0, }, // Nauru
    {   79, 1326,       0,  149110,       0,  0, 13,  0, }, // Nepal
    {   79, 1332,  149123,  149144,  149178, 21, 34, 34, }, // New_Caledonia
    {   79, 1346,  149212,  149231,  149263, 19, 32, 32, }, // New_Zealand
    {   79, 1358,  149295,  149315,  149348, 20, 33, 33, }, // Newfoundland
    {   79, 1371,       0,  149381,       0,  0, 12,  0, }, // Niue
    {   79, 1376,  149393,  149415,  149450, 22, 35, 35, }, // Norfolk
    {   79, 1384,  149485,  149512,  149552, 27, 40, 40, }, // Noronha
    {   79, 1406,  149592,  149611,  149643, 19, 32, 32, }, // Novosibirsk
    {   79, 1418,  149675,  149687,  149712, 12, 25, 25, }, // Omsk
    {   79, 1428,  149737,  149753,  149782, 16, 29, 29, }, // Pakistan
    {   79, 1437,       0,  149811,       0,  0, 13,  0, }, // Palau
    {   79, 1443,       0,  149824,       0,  0, 24,  0, }, // Papua_New_Guinea
    {   79, 1460,  149848,  149864,  149893, 16, 29, 29, }, // Paraguay
    {   79, 1469,  149922,  149934,  149959, 12, 25, 25, }, // Peru
    {   79, 1474,  149984,  150001,  150031, 17, 30, 30, }, // Philippines
    {   79, 1486,       0,  150061,       0,  0, 23,  0, }, // Phoenix_Islands
    {   79, 1502,  150084,  150113,  150155, 29, 42, 42, }, // Pierre_Miquelon
    {   79, 1518,       0,  150197,       0,  0, 16,  0, }, // Pitcairn
    {   79, 1527,       0,  150213,       0,  0, 14,  0, }, // Ponape
    {   79, 1534,       0,  150227,       0,  0, 17,  0, }, // Pyongyang
    {   79, 1554,       0,  150244,       0,  0, 15,  0, }, // Reunion
    {   79, 1562,       0,  150259,       0,  0, 15,  0, }, // Rothera
    {   79, 1570,  150274,  150290,  150319, 16, 29, 29, }, // Sakhalin
    {   79, 1596,  150348,  150361,  150387, 13, 26, 26, }, // Samoa
    {   79, 1602,       0,  150413,       0,  0, 18,  0, }, // Seychelles
    {   79, 1624,       0,  150431,       0,  0, 30,  0, }, // Singapore
    {   79, 1634,       0,  150461,       0,  0, 23,  0, }, // Solomon
    {   79, 1642,       0,  150484,       0,  0, 21,  0, }, // South_Georgia
    {   79, 1656,       0,  150505,       0,  0, 16,  0, }, // Suriname
    {   79, 1676,       0,  150521,       0,  0, 13,  0, }, // Syowa
    {   79, 1682,       0,  150534,       0,  0, 14,  0, }, // Tahiti
    {   79, 1689,  150548,  150562,  150589, 14, 27, 27, }, // Taipei
    {   79, 1696,       0,  150616,       0,  0, 18,  0, }, // Tajikistan
    {   79, 1724,       0,  150634,       0,  0, 15,  0, }, // Tokelau
    {   79, 1732,  150649,  150662,  150688, 13, 26, 26, }, // Tonga
    {   79, 1738,       0,  150714,       0,  0, 13,  0, }, // Truk
    {   79, 1750,  150727,  150747,  150780, 20, 33, 33, }, // Turkmenistan
    {   79, 1763,       0,  150813,       0,  0, 14,  0, }, // Tuvalu
    {   79, 1777,  150827,  150842,  150870, 15, 28, 28, }, // Uruguay
    {   79, 1792,  150898,  150916,  150947, 18, 31, 31, }, // Uzbekistan
    {   79, 1803,  150978,  150993,  151021, 15, 28, 28, }, // Vanuatu
    {   79, 1811,       0,  151049,       0,  0, 17,  0, }, // Venezuela
    {   79, 1821,  151066,  151085,  151117, 19, 32, 32, }, // Vladivostok
    {   79, 1833,  151149,  151166,  151196, 17, 30, 30, }, // Volgograd
    {   79, 1843,       0,  151226,       0,  0, 14,  0, }, // Vostok
    {   79, 1850,       0,  151240,       0,  0, 19,  0, }, // Wake
    {   79, 1855,       0,  151259,       0,  0, 23,  0, }, // Wallis
    {   79, 1862,  151282,  151297,  151325, 15, 28, 28, }, // Yakutsk
    {   79, 1870,  151353,  151374,  151408, 21, 34, 34, }, // Yekaterinburg
    {   79, 1892,       0,  151442,       0,  0, 13,  0, }, // Yukon
    {   83,    0,  151455,  151480,  151516, 25, 36, 48, }, // Acre Chakma/Chakma/Bangladesh
    {   83,    5,       0,  151564,       0,  0, 35,  0, }, // Afghanistan
    {   83,   17,       0,  151599,       0,  0, 40,  0, }, // Africa_Central
    {   83,   32,       0,  151639,       0,  0, 38,  0, }, // Africa_Eastern
    {   83,   65,       0,  151677,       0,  0, 49,  0, }, // Africa_Southern
    {   83,   81,  151726,  151768,  151821, 42, 53, 65, }, // Africa_Western
    {   83,  107,  151886,  151909,  151943, 23, 34, 38, }, // Alaska
    {   83,  128,  151981,  152006,  152042, 25, 36, 44, }, // Almaty
    {   83,  135,  152086,  152115,  152155, 29, 40, 48, }, // Amazon
    {   83,  142,  152203,  152235,  152280, 32, 45, 49, }, // America_Central
    {   83,  158,  152329,  152363,  152412, 34, 49, 53, }, // America_Eastern
    {   83,  174,  152465,  152503,  152556, 38, 53, 51, }, // America_Mountain
    {   83,  191,  152607,  152672,  152748, 65, 76, 78, }, // America_Pacific
    {   83,  207,  152826,  152855,  152895, 29, 40, 52, }, // Anadyr
    {   83,  214,  152947,  152970,  153004, 23, 34, 36, }, // Apia
    {   83,  219,  153040,  153065,  153101, 25, 36, 48, }, // Aqtau
    {   83,  225,  153149,  153176,  153214, 27, 38, 50, }, // Aqtobe
    {   83,  232,  153264,  153287,  153321, 23, 34, 42, }, // Arabian
    {   83,  240,  153363,  153396,  153440, 33, 44, 56, }, // Argentina
    {   83,  250,  153496,  153546,  153611, 50, 65, 73, }, // Argentina_Western
    {   83,  268,  153684,  153713,  153753, 29, 40, 52, }, // Armenia
    {   83,  286,  153805,  153842,  153890, 37, 48, 56, }, // Atlantic
    {   83,  295,  153946,  154009,  154083, 63, 74, 82, }, // Australia_Central
    {   83,  338,  154165,  154215,  154276, 50, 61, 69, }, // Australia_Eastern
    {   83,  356,  154345,  154399,  154464, 54, 65, 73, }, // Australia_Western
    {   83,  374,  154537,  154568,  154610, 31, 42, 54, }, // Azerbaijan
    {   83,  385,  154664,  154693,  154733, 29, 40, 52, }, // Azores
    {   83,  397,  154785,  154812,  154850, 27, 38, 50, }, // Bangladesh
    {   83,  415,       0,  154900,       0,  0, 23,  0, }, // Bhutan
    {   83,  422,       0,  154923,       0,  0, 27,  0, }, // Bolivia
    {   83,  437,  154950,  154979,  155019, 29, 40, 52, }, // Brasilia
    {   83,  454,       0,  155071,       0,  0, 46,  0, }, // Brunei
    {   83,  461,  155117,  155149,  155192, 32, 43, 55, }, // Cape_Verde
    {   83,  478,       0,  155247,       0,  0, 34,  0, }, // Chamorro
    {   83,  487,  155281,  155306,  155342, 25, 36, 44, }, // Chatham
    {   83,  495,  155386,  155407,  155439, 21, 32, 44, }, // Chile
    {   83,  501,  155483,  155504,  155536, 21, 32, 40, }, // China
    {   83,  507,       0,  155576,       0,  0, 50,  0, }, // Christmas
    {   83,  517,       0,  155626,       0,  0, 56,  0, }, // Cocos
    {   83,  523,  155682,  155713,  155755, 31, 42, 54, }, // Colombia
    {   83,  532,  155809,  155861,  155924, 52, 63, 92, }, // Cook
    {   83,  537,  156016,  156043,  156077, 27, 34, 42, }, // Cuba
    {   83,  548,       0,  156119,       0,  0, 25,  0, }, // Davis
    {   83,  564,       0,  156144,       0,  0, 55,  0, }, // DumontDUrville
    {   83,  601,       0,  156199,       0,  0, 38,  0, }, // East_Timor
    {   83,  612,  156237,  156283,  156340, 46, 57, 69, }, // Easter
    {   83,  619,       0,  156409,       0,  0, 33,  0, }, // Ecuador
    {   83,  627,  156442,  156490,  156549, 48, 59, 71, }, // Europe_Central
    {   83,  642,  156620,  156666,  156723, 46, 57, 69, }, // Europe_Eastern
    {   83,  657,       0,  156792,       0,  0, 71,  0, }, // Europe_Further_Eastern
    {   83,  680,  156863,  156911,  156972, 48, 61, 73, }, // Europe_Western
    {   83,  695,  157045,  157111,  157188, 66, 77, 89, }, // Falkland
    {   83,  704,  157277,  157298,  157330, 21, 32, 44, }, // Fiji
    {   83,  709,       0,  157374,       0,  0, 30,  0, }, // French_Guiana
    {   83,  723,       0,  157404,       0,  0, 66,  0, }, // French_Southern
    {   83,  746,       0,  157470,       0,  0, 27,  0, }, // Galapagos
    {   83,  756,       0,  157497,       0,  0, 33,  0, }, // Gambier
    {   83,  764,  157530,  157557,  157595, 27, 38, 50, }, // Georgia
    {   83,  772,       0,  157645,       0,  0, 54,  0, }, // Gilbert_Islands
    {   83,  788,       0,  157699,       0,  0, 34,  0, }, // GMT
    {   83,  830,  157733,  157785,  157848, 52, 63, 75, }, // Greenland_Eastern
    {   83,  848,  157923,  157979,  158046, 56, 67, 79, }, // Greenland_Western
    {   83,  866,       0,  158125,       0,  0, 30,  0, }, // Guam
    {   83,  871,       0,  158155,       0,  0, 46,  0, }, // Gulf
    {   83,  876,       0,  158201,       0,  0, 21,  0, }, // Guyana
    {   83,  883,  158222,  158270,  158321, 48, 51, 59, }, // Hawaii_Aleutian
    {   83,  899,  158380,  158406,  158443, 26, 37, 49, }, // Hong_Kong
    {   83,  909,  158492,  158517,  158553, 25, 36, 48, }, // Hovd
    {   83,  914,       0,  158601,       0,  0, 40,  0, }, // India
    {   83,  920,       0,  158641,       0,  0, 50,  0, }, // Indian_Ocean
    {   83,  933,       0,  158691,       0,  0, 33,  0, }, // Indochina
    {   83,  943,       0,  158724,       0,  0, 55,  0, }, // Indonesia_Central
    {   83,  961,       0,  158779,       0,  0, 48,  0, }, // Indonesia_Eastern
    {   83,  979,       0,  158827,       0,  0, 52,  0, }, // Indonesia_Western
    {   83,  997,  158879,  158902,  158936, 23, 34, 42, }, // Iran
    {   83, 1008,  158978,  159015,  159063, 37, 48, 60, }, // Irkutsk
    {   83, 1016,  159123,  159152,  159194, 29, 42, 50, }, // Israel
    {   83, 1023,  159244,  159265,  159297, 21, 32, 40, }, // Japan
    {   83, 1029,  159337,  159417,  159504, 80, 87,103, }, // Kamchatka
    {   83, 1058,       0,  159607,       0,  0, 55,  0, }, // Kazakhstan_Eastern
    {   83, 1077,       0,  159662,       0,  0, 59,  0, }, // Kazakhstan_Western
    {   83, 1106,  159721,  159748,  159786, 27, 38, 46, }, // Korea
    {   83, 1112,       0,  159832,       0,  0, 29,  0, }, // Kosrae
    {   83, 1119,  159861,  159902,  159954, 41, 52, 64, }, // Krasnoyarsk
    {   83, 1151,       0,  160018,       0,  0, 35,  0, }, // Kyrgystan
    {   83, 1161,       0,  160053,       0,  0, 21,  0, }, // Lanka
    {   83, 1175,       0,  160074,       0,  0, 52,  0, }, // Line_Islands
    {   83, 1188,  160126,  160158,  160201, 32, 43, 51, }, // Lord_Howe
    {   83, 1198,  160252,  160273,  160301, 21, 28, 44, }, // Macau
    {   83, 1204,  160345,  160372,  160410, 27, 38, 50, }, // Magadan
    {   83, 1219,       0,  160460,       0,  0, 29,  0, }, // Malaysia
    {   83, 1228,       0,  160489,       0,  0, 27,  0, }, // Maldives
    {   83, 1237,       0,  160516,       0,  0, 29,  0, }, // Marquesas
    {   83, 1247,       0,  160545,       0,  0, 56,  0, }, // Marshall_Islands
    {   83, 1264,  160601,  160628,  160666, 27, 38, 50, }, // Mauritius
    {   83, 1274,       0,  160716,       0,  0, 29,  0, }, // Mawson
    {   83, 1281,  160745,  160816,  160898, 71, 82, 90, }, // Mexico_Pacific
    {   83, 1296,  160988,  161022,  161067, 34, 45, 57, }, // Mongolia
    {   83, 1305,  161124,  161149,  161185, 25, 36, 48, }, // Moscow
    {   83, 1312,       0,  161233,       0,  0, 27,  0, }, // Myanmar
    {   83, 1320,       0,  161260,       0,  0, 23,  0, }, // Nauru
    {   83, 1326,       0,  161283,       0,  0, 23,  0, }, // Nepal
    {   83, 1332,  161306,  161348,  161401, 42, 53, 65, }, // New_Caledonia
    {   83, 1346,  161466,  161505,  161555, 39, 50, 58, }, // New_Zealand
    {   83, 1358,  161613,  161658,  161714, 45, 56, 64, }, // Newfoundland
    {   83, 1371,       0,  161778,       0,  0, 25,  0, }, // Niue
    {   83, 1376,  161803,  161863,  161934, 60, 71, 79, }, // Norfolk
    {   83, 1384,  162013,  162061,  162120, 48, 59, 71, }, // Noronha
    {   83, 1392,       0,  162191,       0,  0, 73,  0, }, // North_Mariana
    {   83, 1406,  162264,  162305,  162357, 41, 52, 64, }, // Novosibirsk
    {   83, 1418,  162421,  162450,  162490, 29, 40, 52, }, // Omsk
    {   83, 1428,  162542,  162571,  162611, 29, 40, 52, }, // Pakistan
    {   83, 1437,       0,  162663,       0,  0, 21,  0, }, // Palau
    {   83, 1443,       0,  162684,       0,  0, 39,  0, }, // Papua_New_Guinea
    {   83, 1460,  162723,  162752,  162792, 29, 40, 52, }, // Paraguay
    {   83, 1469,  162844,  162865,  162897, 21, 32, 44, }, // Peru
    {   83, 1474,  162941,  162970,  163010, 29, 40, 52, }, // Philippines
    {   83, 1486,       0,  163062,       0,  0, 60,  0, }, // Phoenix_Islands
    {   83, 1502,  163122,  163182,  163253, 60, 71, 79, }, // Pierre_Miquelon
    {   83, 1518,       0,  163332,       0,  0, 35,  0, }, // Pitcairn
    {   83, 1527,       0,  163367,       0,  0, 23,  0, }, // Ponape
    {   83, 1534,       0,  163390,       0,  0, 32,  0, }, // Pyongyang
    {   83, 1544,  163422,  163453,  163491, 31, 38, 54, }, // Qyzylorda
    {   83, 1554,       0,  163545,       0,  0, 35,  0, }, // Reunion
    {   83, 1562,       0,  163580,       0,  0, 23,  0, }, // Rothera
    {   83, 1570,  163603,  163628,  163664, 25, 36, 48, }, // Sakhalin
    {   83, 1579,  163712,  163731,  163757, 19, 26, 42, }, // Samara
    {   83, 1596,  163799,  163820,  163852, 21, 32, 44, }, // Samoa
    {   83, 1602,       0,  163896,       0,  0, 29,  0, }, // Seychelles
    {   83, 1624,       0,  163925,       0,  0, 38,  0, }, // Singapore
    {   83, 1634,       0,  163963,       0,  0, 60,  0, }, // Solomon
    {   83, 1642,       0,  164023,       0,  0, 40,  0, }, // South_Georgia
    {   83, 1656,       0,  164063,       0,  0, 27,  0, }, // Suriname
    {   83, 1676,       0,  164090,       0,  0, 21,  0, }, // Syowa
    {   83, 1682,       0,  164111,       0,  0, 23,  0, }, // Tahiti
    {   83, 1689,  164134,  164159,  164195, 25, 36, 44, }, // Taipei
    {   83, 1696,       0,  164239,       0,  0, 29,  0, }, // Tajikistan
    {   83, 1724,       0,  164268,       0,  0, 27,  0, }, // Tokelau
    {   83, 1732,  164295,  164318,  164352, 23, 34, 46, }, // Tonga
    {   83, 1738,       0,  164398,       0,  0, 21,  0, }, // Truk
    {   83, 1750,  164419,  164462,  164516, 43, 54, 66, }, // Turkmenistan
    {   83, 1763,       0,  164582,       0,  0, 23,  0, }, // Tuvalu
    {   83, 1777,  164605,  164634,  164674, 29, 40, 52, }, // Uruguay
    {   83, 1792,  164726,  164761,  164807, 35, 46, 58, }, // Uzbekistan
    {   83, 1803,  164865,  164890,  164926, 25, 36, 48, }, // Vanuatu
    {   83, 1811,       0,  164974,       0,  0, 31,  0, }, // Venezuela
    {   83, 1821,  165005,  165044,  165094, 39, 50, 62, }, // Vladivostok
    {   83, 1833,  165156,  165191,  165239, 35, 48, 58, }, // Volgograd
    {   83, 1843,       0,  165297,       0,  0, 27,  0, }, // Vostok
    {   83, 1850,       0,  165324,       0,  0, 44,  0, }, // Wake
    {   83, 1855,       0,  165368,       0,  0, 41,  0, }, // Wallis
    {   83, 1862,  165409,  165440,  165482, 31, 42, 54, }, // Yakutsk
    {   83, 1870,  165536,  165583,  165641, 47, 58, 70, }, // Yekaterinburg
    {   85,    5,       0,  165711,       0,  0,  6,  0, }, // Afghanistan Chechen/Cyrillic/Russia
    {   85,   17,       0,  165717,       0,  0, 14,  0, }, // Africa_Central
    {   85,   32,       0,  165731,       0,  0, 16,  0, }, // Africa_Eastern
    {   85,   65,       0,  165747,       0,  0, 13,  0, }, // Africa_Southern
    {   85,   81,  165760,  165776,  165808, 16, 32, 30, }, // Africa_Western
    {   85,  107,  119466,  165838,  165860,  6, 22, 20, }, // Alaska
    {   85,  135,    6664,  165880,  165904,  8, 24, 22, }, // Amazon
    {   85,  142,  165926,  165941,  165972, 15, 31, 29, }, // America_Central
    {   85,  158,  166001,  166018,  166051, 17, 33, 31, }, // America_Eastern
    {   85,  174,  166082,  166100,  166129, 18, 29, 27, }, // America_Mountain
    {   85,  191,  166156,  166173,  166201, 17, 28, 26, }, // America_Pacific
    {   85,  214,  166227,  166242,  166268, 15, 26, 24, }, // Apia
    {   85,  232,  166292,  166311,  166348, 19, 37, 36, }, // Arabian
    {   85,  240,    7392,  166384,  166409,  9, 25, 23, }, // Argentina
    {   85,  250,  166432,  166451,  166486, 19, 35, 33, }, // Argentina_Western
    {   85,  268,  166519,  166529,  166555, 10, 26, 24, }, // Armenia
    {   85,  286,  166579,  166593,  166618, 14, 25, 23, }, // Atlantic
    {   85,  295,  166641,  166657,  166689, 16, 32, 30, }, // Australia_Central
    {   85,  313,  166719,  166750,  166792, 31, 42, 40, }, // Australia_CentralWestern
    {   85,  338,  166832,  166850,  166884, 18, 34, 32, }, // Australia_Eastern
    {   85,  356,  166916,  166934,  166968, 18, 34, 32, }, // Australia_Western
    {   85,  374,   86302,  167000,  167027, 11, 27, 25, }, // Azerbaijan
    {   85,  385,  167052,  167068,  167100, 16, 32, 30, }, // Azores
    {   85,  397,    8277,  167130,  167155,  9, 25, 23, }, // Bangladesh
    {   85,  415,       0,    8338,       0,  0,  5,  0, }, // Bhutan
    {   85,  422,       0,    8343,       0,  0,  6,  0, }, // Bolivia
    {   85,  437,    8350,  167178,  167201,  7, 23, 21, }, // Brasilia
    {   85,  454,       0,  167222,       0,  0, 17,  0, }, // Brunei
    {   85,  461,    8425,  167239,  167265, 10, 26, 24, }, // Cape_Verde
    {   85,  478,       0,    8494,       0,  0,  7,  0, }, // Chamorro
    {   85,  487,    8501,  167289,  167310,  5, 21, 19, }, // Chatham
    {   85,  495,    8550,  167329,  167349,  4, 20, 18, }, // Chile
    {   85,  501,  167367,  167373,  167398,  6, 25, 23, }, // China
    {   85,  507,       0,  167421,       0,  0, 31,  0, }, // Christmas
    {   85,  517,       0,  167452,       0,  0, 16,  0, }, // Cocos
    {   85,  523,    8674,  167468,  167491,  7, 23, 21, }, // Colombia
    {   85,  532,  167512,  167525,  167554, 13, 29, 27, }, // Cook
    {   85,  537,    8827,  167581,  167601,  4, 20, 18, }, // Cuba
    {   85,  548,       0,  121959,       0,  0,  6,  0, }, // Davis
    {   85,  564,       0,  167619,       0,  0, 14,  0, }, // DumontDUrville
    {   85,  601,       0,  167633,       0,  0, 15,  0, }, // East_Timor
    {   85,  612,  167648,  167661,  167690, 13, 29, 27, }, // Easter
    {   85,  619,       0,   87167,       0,  0,  7,  0, }, // Ecuador
    {   85,  627,  167717,  167731,  167761, 14, 30, 28, }, // Europe_Central
    {   85,  642,  167789,  167805,  167837, 16, 32, 30, }, // Europe_Eastern
    {   85,  657,       0,  167867,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {   85,  680,  167899,  167915,  167947, 16, 32, 30, }, // Europe_Western
    {   85,  695,  167977,  167997,  168033, 20, 36, 34, }, // Falkland
    {   85,  704,  122514,  168067,  168088,  5, 21, 19, }, // Fiji
    {   85,  709,       0,  168107,       0,  0, 17,  0, }, // French_Guiana
    {   85,  723,       0,  168124,       0,  0, 38,  0, }, // French_Southern
    {   85,  746,       0,  168162,       0,  0, 21,  0, }, // Galapagos
    {   85,  756,       0,  168183,       0,  0,  6,  0, }, // Gambier
    {   85,  764,  168189,  168199,  168225, 10, 26, 24, }, // Georgia
    {   85,  772,       0,  168249,       0,  0, 17,  0, }, // Gilbert_Islands
    {   85,  788,       0,  168266,       0,  0, 21,  0, }, // GMT
    {   85,  830,  168287,  168306,  168341, 19, 35, 33, }, // Greenland_Eastern
    {   85,  848,  168374,  168393,  168428, 19, 35, 33, }, // Greenland_Western
    {   85,  871,       0,  168461,       0,  0, 14,  0, }, // Gulf
    {   85,  876,       0,  168475,       0,  0,  6,  0, }, // Guyana
    {   85,  883,  168481,  168500,  168530, 19, 30, 28, }, // Hawaii_Aleutian
    {   85,  899,  168558,  168565,  168588,  7, 23, 21, }, // Hong_Kong
    {   85,  909,    9979,  168609,  168629,  4, 20, 18, }, // Hovd
    {   85,  914,       0,  168647,       0,  0,  6,  0, }, // India
    {   85,  920,       0,  168653,       0,  0, 11,  0, }, // Indian_Ocean
    {   85,  933,       0,  123232,       0,  0,  9,  0, }, // Indochina
    {   85,  943,       0,  168664,       0,  0, 16,  0, }, // Indonesia_Central
    {   85,  961,       0,  168680,       0,  0, 18,  0, }, // Indonesia_Eastern
    {   85,  979,       0,  168698,       0,  0, 18,  0, }, // Indonesia_Western
    {   85,  997,  168716,  168727,  168754, 11, 27, 25, }, // Iran
    {   85, 1008,   10164,  168779,  168802,  7, 23, 21, }, // Irkutsk
    {   85, 1016,   10219,  168823,  168846,  7, 23, 21, }, // Israel
    {   85, 1023,   88467,  168867,  168887,  4, 20, 18, }, // Japan
    {   85, 1058,       0,  168905,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {   85, 1077,       0,  168924,       0,  0, 19,  0, }, // Kazakhstan_Western
    {   85, 1106,  123769,  168943,  168964,  5, 21, 19, }, // Korea
    {   85, 1112,       0,  168983,       0,  0,  6,  0, }, // Kosrae
    {   85, 1119,  123841,  168989,  169015, 10, 26, 24, }, // Krasnoyarsk
    {   85, 1151,       0,   10608,       0,  0,  7,  0, }, // Kyrgystan
    {   85, 1175,       0,  169039,       0,  0, 12,  0, }, // Line_Islands
    {   85, 1188,   88733,  169051,  169075,  8, 24, 22, }, // Lord_Howe
    {   85, 1204,   10745,  169097,  169120,  7, 23, 21, }, // Magadan
    {   85, 1219,       0,  124091,       0,  0,  7,  0, }, // Malaysia
    {   85, 1228,       0,  169141,       0,  0,  9,  0, }, // Maldives
    {   85, 1237,       0,  169150,       0,  0, 16,  0, }, // Marquesas
    {   85, 1247,       0,  169166,       0,  0, 17,  0, }, // Marshall_Islands
    {   85, 1264,   10859,  169183,  169206,  7, 23, 21, }, // Mauritius
    {   85, 1274,       0,   10914,       0,  0,  6,  0, }, // Mawson
    {   85, 1281,  169227,  169253,  169290, 26, 37, 35, }, // Mexico_Pacific
    {   85, 1296,   11030,  169325,  169351, 10, 26, 24, }, // Mongolia
    {   85, 1305,   11094,  169375,  169397,  6, 22, 20, }, // Moscow
    {   85, 1312,       0,  169417,       0,  0,  6,  0, }, // Myanmar
    {   85, 1320,       0,   11152,       0,  0,  5,  0, }, // Nauru
    {   85, 1326,       0,   11157,       0,  0,  5,  0, }, // Nepal
    {   85, 1332,  169423,  169437,  169467, 14, 30, 28, }, // New_Caledonia
    {   85, 1346,  169495,  169508,  169537, 13, 29, 27, }, // New_Zealand
    {   85, 1358,  169564,  169576,  169604, 12, 28, 26, }, // Newfoundland
    {   85, 1371,       0,  169630,       0,  0,  4,  0, }, // Niue
    {   85, 1376,   11379,  169634,  169657,  7, 23, 21, }, // Norfolk
    {   85, 1384,  169678,  169697,  169732, 19, 35, 33, }, // Noronha
    {   85, 1406,   11552,  169765,  169792, 11, 27, 25, }, // Novosibirsk
    {   85, 1418,   11619,  169817,  169837,  4, 20, 18, }, // Omsk
    {   85, 1428,   11665,  169855,  169879,  8, 24, 22, }, // Pakistan
    {   85, 1437,       0,   11723,       0,  0,  5,  0, }, // Palau
    {   85, 1443,       0,  169901,       0,  0, 20,  0, }, // Papua_New_Guinea
    {   85, 1460,  125159,  169921,  169945,  8, 24, 22, }, // Paraguay
    {   85, 1469,   11805,  169967,  169987,  4, 20, 18, }, // Peru
    {   85, 1474,  170005,  170015,  170041, 10, 26, 24, }, // Philippines
    {   85, 1486,       0,  170065,       0,  0, 14,  0, }, // Phoenix_Islands
    {   85, 1502,  170079,  170100,  170137, 21, 37, 35, }, // Pierre_Miquelon
    {   85, 1518,       0,  170172,       0,  0,  7,  0, }, // Pitcairn
    {   85, 1527,       0,  170179,       0,  0, 14,  0, }, // Ponape
    {   85, 1534,       0,   90148,       0,  0,  7,  0, }, // Pyongyang
    {   85, 1554,       0,  170193,       0,  0,  7,  0, }, // Reunion
    {   85, 1562,       0,   12103,       0,  0,  6,  0, }, // Rothera
    {   85, 1570,   12109,  170200,  170223,  7, 23, 21, }, // Sakhalin
    {   85, 1596,   12226,  170244,  170265,  5, 21, 19, }, // Samoa
    {   85, 1602,       0,  170284,       0,  0, 18,  0, }, // Seychelles
    {   85, 1624,       0,   12300,       0,  0,  8,  0, }, // Singapore
    {   85, 1634,       0,  170302,       0,  0, 17,  0, }, // Solomon
    {   85, 1642,       0,  170319,       0,  0, 13,  0, }, // South_Georgia
    {   85, 1656,       0,   12350,       0,  0,  7,  0, }, // Suriname
    {   85, 1676,       0,   90403,       0,  0,  4,  0, }, // Syowa
    {   85, 1682,       0,  170332,       0,  0, 13,  0, }, // Tahiti
    {   85, 1689,  170345,  170352,  170375,  7, 23, 21, }, // Taipei
    {   85, 1696,       0,  125839,       0,  0, 11,  0, }, // Tajikistan
    {   85, 1724,       0,   12429,       0,  0,  7,  0, }, // Tokelau
    {   85, 1732,   12436,  170396,  170417,  5, 21, 19, }, // Tonga
    {   85, 1738,       0,  170436,       0,  0,  4,  0, }, // Truk
    {   85, 1750,   12489,  170440,  170464,  8, 24, 22, }, // Turkmenistan
    {   85, 1763,       0,   12559,       0,  0,  6,  0, }, // Tuvalu
    {   85, 1777,   90621,  170486,  170509,  7, 23, 21, }, // Uruguay
    {   85, 1792,  113264,  170530,  170557, 10, 27, 25, }, // Uzbekistan
    {   85, 1803,   12687,  170582,  170605,  7, 23, 21, }, // Vanuatu
    {   85, 1811,       0,  170626,       0,  0,  9,  0, }, // Venezuela
    {   85, 1821,   12751,  170635,  170662, 11, 27, 25, }, // Vladivostok
    {   85, 1833,   12818,  170687,  170712,  9, 25, 23, }, // Volgograd
    {   85, 1843,       0,  113592,       0,  0,  6,  0, }, // Vostok
    {   85, 1850,       0,  170735,       0,  0, 10,  0, }, // Wake
    {   85, 1855,       0,  170745,       0,  0, 18,  0, }, // Wallis
    {   85, 1862,  126411,  170763,  170785,  6, 22, 20, }, // Yakutsk
    {   85, 1870,   12963,  170805,  170833, 12, 28, 26, }, // Yekaterinburg
    {   86,    5,       0,  170859,       0,  0, 13,  0, }, // Afghanistan Cherokee/Cherokee/United States
    {   86,   17,       0,  170872,       0,  0, 15,  0, }, // Africa_Central
    {   86,   32,       0,  170887,       0,  0, 16,  0, }, // Africa_Eastern
    {   86,   65,       0,  170903,       0,  0, 22,  0, }, // Africa_Southern
    {   86,   81,  170925,  170941,  170963, 16, 22, 19, }, // Africa_Western
    {   86,  107,  170982,  170992,  171008, 10, 16, 16, }, // Alaska
    {   86,  135,  171024,  171034,  171050, 10, 16, 13, }, // Amazon
    {   86,  142,  171063,  171072,  171087,  9, 15, 15, }, // America_Central
    {   86,  158,  171102,  171115,  171134, 13, 19, 19, }, // America_Eastern
    {   86,  174,  171153,  171162,  171177,  9, 15, 15, }, // America_Mountain
    {   86,  191,  171192,  171202,  171218, 10, 16, 16, }, // America_Pacific
    {   86,  214,  171234,  171243,  171258,  9, 15, 15, }, // Apia
    {   86,  232,  171273,  171283,  171299, 10, 16, 16, }, // Arabian
    {   86,  240,  171315,  171327,  171345, 12, 18, 15, }, // Argentina
    {   86,  250,  171360,  171380,  171406, 20, 26, 29, }, // Argentina_Western
    {   86,  268,  171435,  171446,  171463, 11, 17, 14, }, // Armenia
    {   86,  286,  171477,  171487,  171503, 10, 16, 16, }, // Atlantic
    {   86,  295,  171519,  171533,  171553, 14, 20, 20, }, // Australia_Central
    {   86,  313,  171573,  171595,  171623, 22, 28, 28, }, // Australia_CentralWestern
    {   86,  338,  171651,  171669,  171693, 18, 24, 24, }, // Australia_Eastern
    {   86,  356,  171577,  171599,  171627, 18, 24, 24, }, // Australia_Western
    {   86,  374,  171717,  171729,  171747, 12, 18, 15, }, // Azerbaijan
    {   86,  385,  171762,  171772,  171788, 10, 16, 13, }, // Azores
    {   86,  397,  171801,  171813,  171831, 12, 18, 15, }, // Bangladesh
    {   86,  415,       0,  171846,       0,  0,  9,  0, }, // Bhutan
    {   86,  422,       0,  171855,       0,  0, 10,  0, }, // Bolivia
    {   86,  437,  171865,  171875,  171891, 10, 16, 13, }, // Brasilia
    {   86,  454,       0,  171904,       0,  0, 15,  0, }, // Brunei
    {   86,  461,  171919,  171936,  171959, 17, 23, 20, }, // Cape_Verde
    {   86,  478,       0,  171979,       0,  0, 15,  0, }, // Chamorro
    {   86,  487,  171994,  172003,  172018,  9, 15, 15, }, // Chatham
    {   86,  495,  172033,  172041,  172055,  8, 14, 11, }, // Chile
    {   86,  501,  172066,  172078,  172096, 12, 18, 18, }, // China
    {   86,  507,       0,  172114,       0,  0, 19,  0, }, // Christmas
    {   86,  517,       0,  172133,       0,  0, 15,  0, }, // Cocos
    {   86,  523,  172148,  172160,  172178, 12, 18, 15, }, // Colombia
    {   86,  532,  172193,  172213,  172239, 20, 26, 27, }, // Cook
    {   86,  537,  172266,  172274,  172288,  8, 14, 14, }, // Cuba
    {   86,  548,       0,  172302,       0,  0,  9,  0, }, // Davis
    {   86,  564,       0,  172311,       0,  0, 17,  0, }, // DumontDUrville
    {   86,  601,       0,  172328,       0,  0, 14,  0, }, // East_Timor
    {   86,  612,  172342,  172361,  172386, 19, 25, 22, }, // Easter
    {   86,  619,       0,  172408,       0,  0, 10,  0, }, // Ecuador
    {   86,  627,  172418,  172431,  172450, 13, 19, 16, }, // Europe_Central
    {   86,  642,  172466,  172483,  172506, 17, 23, 20, }, // Europe_Eastern
    {   86,  657,       0,  172526,       0,  0, 14,  0, }, // Europe_Further_Eastern
    {   86,  680,  172540,  172557,  172580, 17, 23, 20, }, // Europe_Western
    {   86,  695,  172600,  172614,  172634, 14, 20, 17, }, // Falkland
    {   86,  704,  172651,  172659,  172673,  8, 14, 11, }, // Fiji
    {   86,  709,       0,  172684,       0,  0, 14,  0, }, // French_Guiana
    {   86,  723,       0,  172698,       0,  0, 24,  0, }, // French_Southern
    {   86,  746,       0,  172722,       0,  0, 18,  0, }, // Galapagos
    {   86,  756,       0,  172740,       0,  0, 10,  0, }, // Gambier
    {   86,  764,  172750,  172760,  172776, 10, 16, 13, }, // Georgia
    {   86,  772,       0,  172789,       0,  0, 16,  0, }, // Gilbert_Islands
    {   86,  788,       0,  172805,       0,  0, 13,  0, }, // GMT
    {   86,  830,  172818,  172834,  172856, 16, 22, 19, }, // Greenland_Eastern
    {   86,  848,  172875,  172891,  172913, 16, 22, 19, }, // Greenland_Western
    {   86,  871,       0,  172932,       0,  0, 17,  0, }, // Gulf
    {   86,  876,       0,  172949,       0,  0,  9,  0, }, // Guyana
    {   86,  883,  172958,  172974,  172996, 16, 22, 22, }, // Hawaii_Aleutian
    {   86,  899,  173018,  173031,  173050, 13, 19, 16, }, // Hong_Kong
    {   86,  909,  173066,  173075,  173090,  9, 15, 12, }, // Hovd
    {   86,  914,       0,  173102,       0,  0, 17,  0, }, // India
    {   86,  920,       0,  173119,       0,  0, 16,  0, }, // Indian_Ocean
    {   86,  933,       0,  173135,       0,  0, 15,  0, }, // Indochina
    {   86,  943,       0,  173150,       0,  0, 16,  0, }, // Indonesia_Central
    {   86,  961,       0,  173166,       0,  0, 20,  0, }, // Indonesia_Eastern
    {   86,  979,       0,  173186,       0,  0, 20,  0, }, // Indonesia_Western
    {   86,  997,  173206,  173215,  173230,  9, 15, 15, }, // Iran
    {   86, 1008,  173245,  173255,  173271, 10, 16, 13, }, // Irkutsk
    {   86, 1016,  173284,  173294,  173310, 10, 16, 17, }, // Israel
    {   86, 1023,  173327,  173337,  173353, 10, 16, 16, }, // Japan
    {   86, 1047,       0,  173369,       0,  0, 12,  0, }, // Kazakhstan
    {   86, 1058,       0,  173381,       0,  0, 17,  0, }, // Kazakhstan_Eastern
    {   86, 1077,       0,  173398,       0,  0, 17,  0, }, // Kazakhstan_Western
    {   86, 1106,  173415,  173425,  173441, 10, 16, 16, }, // Korea
    {   86, 1112,       0,  173457,       0,  0,  9,  0, }, // Kosrae
    {   86, 1119,  173466,  173478,  173496, 12, 18, 15, }, // Krasnoyarsk
    {   86, 1151,       0,  173511,       0,  0, 11,  0, }, // Kyrgystan
    {   86, 1175,       0,  173522,       0,  0, 17,  0, }, // Line_Islands
    {   86, 1188,  173539,  173553,  173573, 14, 20, 20, }, // Lord_Howe
    {   86, 1204,  173593,  173603,  173619, 10, 16, 13, }, // Magadan
    {   86, 1219,       0,  173632,       0,  0, 11,  0, }, // Malaysia
    {   86, 1228,       0,  173643,       0,  0, 11,  0, }, // Maldives
    {   86, 1237,       0,  173654,       0,  0, 11,  0, }, // Marquesas
    {   86, 1247,       0,  173665,       0,  0, 15,  0, }, // Marshall_Islands
    {   86, 1264,  173680,  173691,  173708, 11, 17, 14, }, // Mauritius
    {   86, 1274,       0,  173722,       0,  0,  9,  0, }, // Mawson
    {   86, 1281,  173731,  173747,  173769, 16, 22, 22, }, // Mexico_Pacific
    {   86, 1296,  173791,  173804,  173823, 13, 19, 16, }, // Mongolia
    {   86, 1305,  173839,  173849,  173865, 10, 16, 13, }, // Moscow
    {   86, 1312,       0,  173878,       0,  0, 10,  0, }, // Myanmar
    {   86, 1320,       0,  173888,       0,  0,  8,  0, }, // Nauru
    {   86, 1326,       0,  173896,       0,  0,  9,  0, }, // Nepal
    {   86, 1332,  173905,  173920,  173941, 15, 21, 18, }, // New_Caledonia
    {   86, 1346,  173959,  173973,  173993, 14, 20, 20, }, // New_Zealand
    {   86, 1358,  174013,  174029,  174051, 16, 22, 22, }, // Newfoundland
    {   86, 1371,       0,  174073,       0,  0,  8,  0, }, // Niue
    {   86, 1376,  174081,  174098,  174121, 17, 23, 20, }, // Norfolk
    {   86, 1384,  174141,  174158,  174181, 17, 23, 20, }, // Noronha
    {   86, 1406,  174201,  174213,  174231, 12, 18, 15, }, // Novosibirsk
    {   86, 1418,  174246,  174256,  174272, 10, 16, 13, }, // Omsk
    {   86, 1428,  174285,  174296,  174313, 11, 17, 14, }, // Pakistan
    {   86, 1437,       0,  174327,       0,  0,  8,  0, }, // Palau
    {   86, 1443,       0,  174335,       0,  0, 15,  0, }, // Papua_New_Guinea
    {   86, 1460,  174350,  174359,  174374,  9, 15, 12, }, // Paraguay
    {   86, 1469,  174386,  174394,  174408,  8, 14, 11, }, // Peru
    {   86, 1474,  174419,  174431,  174449, 12, 18, 15, }, // Philippines
    {   86, 1486,       0,  174464,       0,  0, 18,  0, }, // Phoenix_Islands
    {   86, 1502,  174482,  174502,  174528, 20, 26, 26, }, // Pierre_Miquelon
    {   86, 1518,       0,  174554,       0,  0, 10,  0, }, // Pitcairn
    {   86, 1527,       0,  174564,       0,  0,  9,  0, }, // Ponape
    {   86, 1534,       0,  174573,       0,  0, 13,  0, }, // Pyongyang
    {   86, 1554,       0,  174586,       0,  0, 11,  0, }, // Reunion
    {   86, 1562,       0,  174597,       0,  0,  9,  0, }, // Rothera
    {   86, 1570,  174606,  174616,  174632, 10, 16, 13, }, // Sakhalin
    {   86, 1596,  174645,  174654,  174669,  9, 15, 15, }, // Samoa
    {   86, 1602,       0,  174684,       0,  0, 10,  0, }, // Seychelles
    {   86, 1624,       0,  174694,       0,  0, 17,  0, }, // Singapore
    {   86, 1634,       0,  174711,       0,  0, 16,  0, }, // Solomon
    {   86, 1642,       0,  174727,       0,  0, 15,  0, }, // South_Georgia
    {   86, 1656,       0,  174742,       0,  0, 10,  0, }, // Suriname
    {   86, 1676,       0,  174752,       0,  0,  9,  0, }, // Syowa
    {   86, 1682,       0,  174761,       0,  0,  9,  0, }, // Tahiti
    {   86, 1689,  174770,  174779,  174794,  9, 15, 15, }, // Taipei
    {   86, 1696,       0,  174809,       0,  0, 12,  0, }, // Tajikistan
    {   86, 1724,       0,  174821,       0,  0, 10,  0, }, // Tokelau
    {   86, 1732,  174831,  174840,  174855,  9, 15, 12, }, // Tonga
    {   86, 1738,       0,  174867,       0,  0,  8,  0, }, // Truk
    {   86, 1750,  174875,  174889,  174909, 14, 20, 17, }, // Turkmenistan
    {   86, 1763,       0,  174926,       0,  0,  9,  0, }, // Tuvalu
    {   86, 1777,  174935,  174944,  174959,  9, 15, 12, }, // Uruguay
    {   86, 1792,  174971,  174984,  175003, 13, 19, 16, }, // Uzbekistan
    {   86, 1803,  175019,  175029,  175045, 10, 16, 13, }, // Vanuatu
    {   86, 1811,       0,  175058,       0,  0, 11,  0, }, // Venezuela
    {   86, 1821,  175069,  175082,  175101, 13, 19, 16, }, // Vladivostok
    {   86, 1833,  175117,  175128,  175145, 11, 17, 14, }, // Volgograd
    {   86, 1843,       0,  175159,       0,  0, 10,  0, }, // Vostok
    {   86, 1850,       0,  175169,       0,  0, 15,  0, }, // Wake
    {   86, 1855,       0,  175184,       0,  0, 16,  0, }, // Wallis
    {   86, 1862,  175200,  175210,  175226, 10, 16, 13, }, // Yakutsk
    {   86, 1870,  175239,  175252,  175271, 13, 19, 16, }, // Yekaterinburg
    {   86, 1892,       0,  175287,       0,  0,  9,  0, }, // Yukon
    {   89,    0,  175296,  175301,  175308,  5,  7,  7, }, // Acre Chinese/Simplified Han/China
    {   89,    5,       0,  135211,       0,  0,  5,  0, }, // Afghanistan
    {   89,   17,       0,  175315,       0,  0,  6,  0, }, // Africa_Central
    {   89,   32,       0,  175321,       0,  0,  6,  0, }, // Africa_Eastern
    {   89,   65,       0,  135224,       0,  0,  6,  0, }, // Africa_Southern
    {   89,   81,  175327,  175333,  175341,  6,  8,  8, }, // Africa_Western
    {   89,  107,  135246,  135252,  135260,  6,  8,  8, }, // Alaska
    {   89,  128,  135268,  135274,  135282,  6,  8,  8, }, // Almaty
    {   89,  135,  135290,  135295,  135302,  5,  7,  7, }, // Amazon
    {   89,  142,  175349,  175355,  175363,  6,  8,  8, }, // America_Central
    {   89,  158,  175371,  175377,  175385,  6,  8,  8, }, // America_Eastern
    {   89,  174,  175393,  175399,  175407,  6,  8,  8, }, // America_Mountain
    {   89,  191,  175415,  175422,  175431,  7,  9,  9, }, // America_Pacific
    {   89,  207,  135376,  175440,  175448,  6,  8,  8, }, // Anadyr
    {   89,  214,  135398,  135403,  135410,  5,  7,  7, }, // Apia
    {   89,  219,  175456,  175461,  175468,  5,  7,  7, }, // Aqtau
    {   89,  225,  175475,  175481,  175489,  6,  8,  8, }, // Aqtobe
    {   89,  232,  135458,  135463,  135470,  5,  7,  7, }, // Arabian
    {   89,  240,  135477,  135482,  135489,  5,  7,  7, }, // Argentina
    {   89,  250,  135496,  135503,  135512,  7,  9,  9, }, // Argentina_Western
    {   89,  268,  135521,  135527,  135535,  6,  8,  8, }, // Armenia
    {   89,  286,  135543,  135548,  135555,  5,  7,  7, }, // Atlantic
    {   89,  295,  175497,  175505,  175515,  8, 10, 10, }, // Australia_Central
    {   89,  313,  175525,  175534,  175545,  9, 11, 11, }, // Australia_CentralWestern
    {   89,  338,  175556,  175564,  175574,  8, 10, 10, }, // Australia_Eastern
    {   89,  356,  175584,  175592,  175602,  8, 10, 10, }, // Australia_Western
    {   89,  374,  175612,  175618,  175626,  6,  8,  8, }, // Azerbaijan
    {   89,  385,  135675,  135682,  135691,  7,  9,  9, }, // Azores
    {   89,  397,  135700,  135705,  135712,  5,  7,  7, }, // Bangladesh
    {   89,  415,       0,  135719,       0,  0,  4,  0, }, // Bhutan
    {   89,  422,       0,  175634,       0,  0,  8,  0, }, // Bolivia
    {   89,  437,  135729,  135735,  135743,  6,  8,  8, }, // Brasilia
    {   89,  454,       0,  175642,       0,  0,  8,  0, }, // Brunei
    {   89,  461,  175650,  175655,  175662,  5,  7,  7, }, // Cape_Verde
    {   89,  472,       0,  175669,       0,  0,  4,  0, }, // Casey
    {   89,  478,       0,  175673,       0,  0,  5,  0, }, // Chamorro
    {   89,  487,  175678,  175683,  175690,  5,  7,  7, }, // Chatham
    {   89,  495,  135806,  135810,  135816,  4,  6,  6, }, // Chile
    {   89,  501,  135822,  135826,  135832,  4,  6,  6, }, // China
    {   89,  507,       0,  135838,       0,  0,  5,  0, }, // Christmas
    {   89,  517,       0,  135843,       0,  0,  7,  0, }, // Cocos
    {   89,  523,  135850,  135856,  135864,  6,  8,  8, }, // Colombia
    {   89,  532,  135872,  135878,  175697,  6,  8,  8, }, // Cook
    {   89,  537,  135895,  135899,  135905,  4,  6,  6, }, // Cuba
    {   89,  548,       0,  135911,       0,  0,  5,  0, }, // Davis
    {   89,  564,       0,  175705,       0,  0,  8,  0, }, // DumontDUrville
    {   89,  601,       0,  135923,       0,  0,  5,  0, }, // East_Timor
    {   89,  612,  135928,  135934,  135942,  6,  8,  8, }, // Easter
    {   89,  619,       0,  175713,       0,  0,  8,  0, }, // Ecuador
    {   89,  627,  135955,  135959,  135965,  4,  6,  6, }, // Europe_Central
    {   89,  642,  135971,  135975,  135981,  4,  6,  6, }, // Europe_Eastern
    {   89,  657,       0,  175721,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   89,  680,  135993,  135997,  136003,  4,  6,  6, }, // Europe_Western
    {   89,  695,  136009,  136016,  136025,  7,  9,  9, }, // Falkland
    {   89,  704,  136034,  136038,  136044,  4,  6,  6, }, // Fiji
    {   89,  709,       0,  175727,       0,  0,  9,  0, }, // French_Guiana
    {   89,  723,       0,  175736,       0,  0, 11,  0, }, // French_Southern
    {   89,  746,       0,  175747,       0,  0,  6,  0, }, // Galapagos
    {   89,  756,       0,  175753,       0,  0,  5,  0, }, // Gambier
    {   89,  764,  175758,  175764,  175772,  6,  8,  8, }, // Georgia
    {   89,  772,       0,  136100,       0,  0,  8,  0, }, // Gilbert_Islands
    {   89,  788,       0,  175780,       0,  0,  8,  0, }, // GMT
    {   89,  830,  175788,  175796,  175806,  8, 10, 10, }, // Greenland_Eastern
    {   89,  848,  175816,  175824,  175834,  8, 10, 10, }, // Greenland_Western
    {   89,  866,       0,  175844,       0,  0,  4,  0, }, // Guam
    {   89,  871,       0,  175848,       0,  0,  6,  0, }, // Gulf
    {   89,  876,       0,  136052,       0,  0,  5,  0, }, // Guyana
    {   89,  883,  136186,  136195,  136206,  9, 11, 11, }, // Hawaii_Aleutian
    {   89,  899,  136217,  136221,  136227,  4,  6,  6, }, // Hong_Kong
    {   89,  909,  136233,  136238,  136245,  5,  7,  7, }, // Hovd
    {   89,  914,       0,  175854,       0,  0,  4,  0, }, // India
    {   89,  920,       0,  136258,       0,  0,  5,  0, }, // Indian_Ocean
    {   89,  933,       0,  175858,       0,  0,  6,  0, }, // Indochina
    {   89,  943,       0,  175864,       0,  0,  9,  0, }, // Indonesia_Central
    {   89,  961,       0,  175873,       0,  0,  9,  0, }, // Indonesia_Eastern
    {   89,  979,       0,  175882,       0,  0,  9,  0, }, // Indonesia_Western
    {   89,  997,  136287,  136291,  136297,  4,  6,  6, }, // Iran
    {   89, 1008,  175891,  175898,  175907,  7,  9,  9, }, // Irkutsk
    {   89, 1016,  136328,  136333,  136340,  5,  7,  7, }, // Israel
    {   89, 1023,  136347,  136351,  136357,  4,  6,  6, }, // Japan
    {   89, 1029,  175916,  175931,  175948, 15, 17, 17, }, // Kamchatka
    {   89, 1047,       0,  175965,       0,  0,  7,  0, }, // Kazakhstan
    {   89, 1058,       0,  175972,       0,  0,  9,  0, }, // Kazakhstan_Eastern
    {   89, 1077,       0,  175981,       0,  0,  9,  0, }, // Kazakhstan_Western
    {   89, 1106,  136419,  136423,  136429,  4,  6,  6, }, // Korea
    {   89, 1112,       0,  175990,       0,  0,  5,  0, }, // Kosrae
    {   89, 1119,  136440,  136450,  136462, 10, 12, 12, }, // Krasnoyarsk
    {   89, 1151,       0,  175995,       0,  0,  8,  0, }, // Kyrgystan
    {   89, 1161,       0,  136480,       0,  0,  4,  0, }, // Lanka
    {   89, 1175,       0,  136484,       0,  0,  6,  0, }, // Line_Islands
    {   89, 1188,  136490,  136496,  136504,  6,  8,  8, }, // Lord_Howe
    {   89, 1198,  136512,  136516,  136522,  4,  6,  6, }, // Macau
    {   89, 1204,  136528,  136533,  136540,  5,  7,  7, }, // Magadan
    {   89, 1219,       0,  136547,       0,  0,  6,  0, }, // Malaysia
    {   89, 1228,       0,  176003,       0,  0,  6,  0, }, // Maldives
    {   89, 1237,       0,  176009,       0,  0,  8,  0, }, // Marquesas
    {   89, 1247,       0,  136565,       0,  0,  7,  0, }, // Marshall_Islands
    {   89, 1264,  176017,  176023,  176031,  6,  8,  8, }, // Mauritius
    {   89, 1274,       0,  136594,       0,  0,  4,  0, }, // Mawson
    {   89, 1281,  136598,  136606,  136616,  8, 10, 10, }, // Mexico_Pacific
    {   89, 1296,  136626,  136632,  136640,  6,  8,  8, }, // Mongolia
    {   89, 1305,  136648,  136653,  136660,  5,  7,  7, }, // Moscow
    {   89, 1312,       0,  136667,       0,  0,  4,  0, }, // Myanmar
    {   89, 1320,       0,  176039,       0,  0,  4,  0, }, // Nauru
    {   89, 1326,       0,  136675,       0,  0,  5,  0, }, // Nepal
    {   89, 1332,  136680,  136688,  176043,  8, 10, 10, }, // New_Caledonia
    {   89, 1346,  176053,  176058,  176065,  5,  7,  7, }, // New_Zealand
    {   89, 1358,  136729,  136734,  136741,  5,  7,  7, }, // Newfoundland
    {   89, 1371,       0,  176072,       0,  0,  4,  0, }, // Niue
    {   89, 1376,  136753,  136759,  136767,  6,  8,  8, }, // Norfolk
    {   89, 1384,  176076,  176089,  176104, 13, 15, 15, }, // Noronha
    {   89, 1392,       0,  136821,       0,  0,  9,  0, }, // North_Mariana
    {   89, 1406,  136830,  136837,  136846,  7,  9,  9, }, // Novosibirsk
    {   89, 1418,  136855,  136861,  136869,  6,  8,  8, }, // Omsk
    {   89, 1428,  136877,  136883,  136891,  6,  8,  8, }, // Pakistan
    {   89, 1437,       0,  176119,       0,  0,  4,  0, }, // Palau
    {   89, 1443,       0,  176123,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   89, 1460,  136912,  136917,  136924,  5,  7,  7, }, // Paraguay
    {   89, 1469,  136931,  136935,  136941,  4,  6,  6, }, // Peru
    {   89, 1474,  136947,  136952,  136959,  5,  7,  7, }, // Philippines
    {   89, 1486,       0,  176132,       0,  0,  8,  0, }, // Phoenix_Islands
    {   89, 1502,  136972,  136984,  136998, 12, 14, 14, }, // Pierre_Miquelon
    {   89, 1518,       0,  176140,       0,  0,  6,  0, }, // Pitcairn
    {   89, 1527,       0,  137017,       0,  0,  5,  0, }, // Ponape
    {   89, 1534,       0,  137022,       0,  0,  4,  0, }, // Pyongyang
    {   89, 1544,  176146,  176153,  176162,  7,  9,  9, }, // Qyzylorda
    {   89, 1554,       0,  176171,       0,  0,  5,  0, }, // Reunion
    {   89, 1562,       0,  137059,       0,  0,  5,  0, }, // Rothera
    {   89, 1570,  137064,  137069,  137076,  5,  7,  7, }, // Sakhalin
    {   89, 1579,  137083,  137088,  137095,  5,  7,  7, }, // Samara
    {   89, 1596,  137102,  137107,  137114,  5,  7,  7, }, // Samoa
    {   89, 1602,       0,  176176,       0,  0,  5,  0, }, // Seychelles
    {   89, 1624,       0,  137126,       0,  0,  7,  0, }, // Singapore
    {   89, 1634,       0,  176181,       0,  0,  7,  0, }, // Solomon
    {   89, 1642,       0,  176188,       0,  0,  7,  0, }, // South_Georgia
    {   89, 1656,       0,  176195,       0,  0,  5,  0, }, // Suriname
    {   89, 1676,       0,  176200,       0,  0,  4,  0, }, // Syowa
    {   89, 1682,       0,  176204,       0,  0,  6,  0, }, // Tahiti
    {   89, 1689,  137162,  137166,  137172,  4,  6,  6, }, // Taipei
    {   89, 1696,       0,  176210,       0,  0,  7,  0, }, // Tajikistan
    {   89, 1724,       0,  176217,       0,  0,  5,  0, }, // Tokelau
    {   89, 1732,  176222,  176226,  176232,  4,  6,  6, }, // Tonga
    {   89, 1738,       0,  176238,       0,  0,  4,  0, }, // Truk
    {   89, 1750,  176242,  176249,  176258,  7,  9,  9, }, // Turkmenistan
    {   89, 1763,       0,  176267,       0,  0,  5,  0, }, // Tuvalu
    {   89, 1777,  137235,  137240,  137247,  5,  7,  7, }, // Uruguay
    {   89, 1792,  176272,  176280,  176290,  8, 10, 10, }, // Uzbekistan
    {   89, 1803,  176300,  176306,  176314,  6,  8,  8, }, // Vanuatu
    {   89, 1811,       0,  137295,       0,  0,  6,  0, }, // Venezuela
    {   89, 1821,  137301,  137306,  137313,  5,  7,  7, }, // Vladivostok
    {   89, 1833,  137320,  137327,  137336,  7,  9,  9, }, // Volgograd
    {   89, 1843,       0,  137345,       0,  0,  6,  0, }, // Vostok
    {   89, 1850,       0,  137351,       0,  0,  5,  0, }, // Wake
    {   89, 1855,       0,  176322,       0,  0,  9,  0, }, // Wallis
    {   89, 1862,  176331,  176337,  176345,  6,  8,  8, }, // Yakutsk
    {   89, 1870,  137389,  137396,  137405,  7,  9,  9, }, // Yekaterinburg
    {   89, 1892,       0,  137414,       0,  0,  4,  0, }, // Yukon
    {   94, 1219,       0,  136547,       0,  0,  6,  0, }, // Malaysia Chinese/Simplified Han/Singapore
    {   94, 1624,       0,  137126,       0,  0,  7,  0, }, // Singapore
    {   95,   65,       0,  176353,       0,  0,  4,  0, }, // Africa_Southern Chinese/Traditional Han/Hong Kong
    {   95,  142,  176357,  176363,  176371,  6,  8,  8, }, // America_Central
    {   95,  158,  176379,  176385,  176393,  6,  8,  8, }, // America_Eastern
    {   95,  174,  176401,  176407,  176415,  6,  8,  8, }, // America_Mountain
    {   95,  191,  176423,  176430,  176439,  7,  9,  9, }, // America_Pacific
    {   95,  374,  176448,  176454,  176462,  6,  8,  8, }, // Azerbaijan
    {   95,  461,  176470,  176475,  176482,  5,  7,  7, }, // Cape_Verde
    {   95,  517,       0,  176489,       0,  0,  7,  0, }, // Cocos
    {   95,  564,       0,  176496,       0,  0,  7,  0, }, // DumontDUrville
    {   95,  619,       0,  176503,       0,  0,  6,  0, }, // Ecuador
    {   95,  746,       0,  176509,       0,  0,  8,  0, }, // Galapagos
    {   95,  764,  176517,  176523,  176531,  6,  8,  8, }, // Georgia
    {   95,  871,       0,  176539,       0,  0,  7,  0, }, // Gulf
    {   95,  876,       0,  133829,       0,  0,  5,  0, }, // Guyana
    {   95,  914,       0,  176546,       0,  0,  4,  0, }, // India
    {   95, 1008,  176550,  176557,  176566,  7,  9,  9, }, // Irkutsk
    {   95, 1112,       0,  176575,       0,  0,  5,  0, }, // Kosrae
    {   95, 1228,       0,  176580,       0,  0,  6,  0, }, // Maldives
    {   95, 1237,       0,  176586,       0,  0,  6,  0, }, // Marquesas
    {   95, 1264,  176592,  176598,  176606,  6,  8,  8, }, // Mauritius
    {   95, 1320,       0,  176614,       0,  0,  4,  0, }, // Nauru
    {   95, 1332,  134457,  134465,  176618,  8, 10, 10, }, // New_Caledonia
    {   95, 1384,  176628,  176639,  176652, 11, 13, 13, }, // Noronha
    {   95, 1443,       0,  176665,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   95, 1518,       0,  176674,       0,  0,  5,  0, }, // Pitcairn
    {   95, 1602,       0,  176679,       0,  0,  5,  0, }, // Seychelles
    {   95, 1624,       0,  176684,       0,  0,  5,  0, }, // Singapore
    {   95, 1634,       0,  176689,       0,  0,  7,  0, }, // Solomon
    {   95, 1656,       0,  176696,       0,  0,  5,  0, }, // Suriname
    {   95, 1732,  176701,  176705,  176711,  4,  6,  6, }, // Tonga
    {   95, 1763,       0,  176717,       0,  0,  5,  0, }, // Tuvalu
    {   95, 1803,  176722,  176728,  176736,  6,  8,  8, }, // Vanuatu
    {   95, 1862,  176744,  176750,  176758,  6,  8,  8, }, // Yakutsk
    {   96,   65,       0,  176353,       0,  0,  4,  0, }, // Africa_Southern Chinese/Traditional Han/Macao
    {   96,  142,  176357,  176363,  176371,  6,  8,  8, }, // America_Central
    {   96,  158,  176379,  176385,  176393,  6,  8,  8, }, // America_Eastern
    {   96,  174,  176401,  176407,  176415,  6,  8,  8, }, // America_Mountain
    {   96,  191,  176423,  176430,  176439,  7,  9,  9, }, // America_Pacific
    {   96,  374,  176448,  176454,  176462,  6,  8,  8, }, // Azerbaijan
    {   96,  461,  176470,  176475,  176482,  5,  7,  7, }, // Cape_Verde
    {   96,  517,       0,  176489,       0,  0,  7,  0, }, // Cocos
    {   96,  564,       0,  176496,       0,  0,  7,  0, }, // DumontDUrville
    {   96,  619,       0,  176503,       0,  0,  6,  0, }, // Ecuador
    {   96,  746,       0,  176509,       0,  0,  8,  0, }, // Galapagos
    {   96,  764,  176517,  176523,  176531,  6,  8,  8, }, // Georgia
    {   96,  871,       0,  176539,       0,  0,  7,  0, }, // Gulf
    {   96,  876,       0,  133829,       0,  0,  5,  0, }, // Guyana
    {   96,  914,       0,  176546,       0,  0,  4,  0, }, // India
    {   96, 1008,  176550,  176557,  176566,  7,  9,  9, }, // Irkutsk
    {   96, 1112,       0,  176575,       0,  0,  5,  0, }, // Kosrae
    {   96, 1228,       0,  176580,       0,  0,  6,  0, }, // Maldives
    {   96, 1237,       0,  176586,       0,  0,  6,  0, }, // Marquesas
    {   96, 1264,  176592,  176598,  176606,  6,  8,  8, }, // Mauritius
    {   96, 1320,       0,  176614,       0,  0,  4,  0, }, // Nauru
    {   96, 1332,  134457,  134465,  176618,  8, 10, 10, }, // New_Caledonia
    {   96, 1384,  176628,  176639,  176652, 11, 13, 13, }, // Noronha
    {   96, 1443,       0,  176665,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   96, 1518,       0,  176674,       0,  0,  5,  0, }, // Pitcairn
    {   96, 1602,       0,  176679,       0,  0,  5,  0, }, // Seychelles
    {   96, 1624,       0,  176684,       0,  0,  5,  0, }, // Singapore
    {   96, 1634,       0,  176689,       0,  0,  7,  0, }, // Solomon
    {   96, 1656,       0,  176696,       0,  0,  5,  0, }, // Suriname
    {   96, 1732,  176701,  176705,  176711,  4,  6,  6, }, // Tonga
    {   96, 1763,       0,  176717,       0,  0,  5,  0, }, // Tuvalu
    {   96, 1803,  176722,  176728,  176736,  6,  8,  8, }, // Vanuatu
    {   96, 1862,  176744,  176750,  176758,  6,  8,  8, }, // Yakutsk
    {   98,    0,  132972,  132976,  132982,  4,  6,  6, }, // Acre Chinese/Traditional Han/Taiwan
    {   98,    5,       0,  132988,       0,  0,  5,  0, }, // Afghanistan
    {   98,   17,       0,  132993,       0,  0,  4,  0, }, // Africa_Central
    {   98,   32,       0,  132997,       0,  0,  4,  0, }, // Africa_Eastern
    {   98,   65,       0,  133001,       0,  0,  6,  0, }, // Africa_Southern
    {   98,   81,  133007,  133011,  133017,  4,  6,  6, }, // Africa_Western
    {   98,  107,  133023,  133029,  133037,  6,  8,  8, }, // Alaska
    {   98,  128,  133045,  133051,  133059,  6,  8,  8, }, // Almaty
    {   98,  135,  133067,  133072,  133079,  5,  7,  7, }, // Amazon
    {   98,  142,  133086,  133090,  133096,  4,  6,  6, }, // America_Central
    {   98,  158,  133102,  133106,  133112,  4,  6,  6, }, // America_Eastern
    {   98,  174,  133118,  133122,  133128,  4,  6,  6, }, // America_Mountain
    {   98,  191,  133134,  133139,  133146,  5,  7,  7, }, // America_Pacific
    {   98,  207,  133153,  133159,  133167,  6,  8,  8, }, // Anadyr
    {   98,  214,  133175,  133180,  133187,  5,  7,  7, }, // Apia
    {   98,  219,  133194,  133199,  133206,  5,  7,  7, }, // Aqtau
    {   98,  225,  133213,  133219,  133227,  6,  8,  8, }, // Aqtobe
    {   98,  232,  133235,  133240,  133247,  5,  7,  7, }, // Arabian
    {   98,  240,  133254,  133259,  133266,  5,  7,  7, }, // Argentina
    {   98,  250,  133273,  133280,  133289,  7,  9,  9, }, // Argentina_Western
    {   98,  268,  133298,  133304,  133312,  6,  8,  8, }, // Armenia
    {   98,  286,  133320,  133325,  133332,  5,  7,  7, }, // Atlantic
    {   98,  295,  133339,  133345,  133353,  6,  8,  8, }, // Australia_Central
    {   98,  313,  133361,  133368,  133377,  7,  9,  9, }, // Australia_CentralWestern
    {   98,  338,  133386,  133392,  133400,  6,  8,  8, }, // Australia_Eastern
    {   98,  356,  133408,  133414,  133422,  6,  8,  8, }, // Australia_Western
    {   98,  374,  133430,  133436,  133444,  6,  8,  8, }, // Azerbaijan
    {   98,  385,  133452,  133459,  133468,  7,  9,  9, }, // Azores
    {   98,  397,  133477,  133482,  133489,  5,  7,  7, }, // Bangladesh
    {   98,  415,       0,  133496,       0,  0,  4,  0, }, // Bhutan
    {   98,  422,       0,  133500,       0,  0,  6,  0, }, // Bolivia
    {   98,  437,  133506,  133512,  133520,  6,  8,  8, }, // Brasilia
    {   98,  454,       0,  133528,       0,  0,  4,  0, }, // Brunei
    {   98,  461,  133532,  133537,  133544,  5,  7,  7, }, // Cape_Verde
    {   98,  472,       0,  133551,       0,  0,  5,  0, }, // Casey
    {   98,  478,       0,  133556,       0,  0,  5,  0, }, // Chamorro
    {   98,  487,  133561,  133567,  133575,  6,  8,  8, }, // Chatham
    {   98,  495,  133583,  133587,  133593,  4,  6,  6, }, // Chile
    {   98,  501,  133599,  133603,  133609,  4,  6,  6, }, // China
    {   98,  507,       0,  133615,       0,  0,  5,  0, }, // Christmas
    {   98,  517,       0,  133620,       0,  0,  7,  0, }, // Cocos
    {   98,  523,  133627,  133633,  133641,  6,  8,  8, }, // Colombia
    {   98,  532,  133649,  133655,  133663,  6,  8,  9, }, // Cook
    {   98,  537,  133672,  133676,  133682,  4,  6,  6, }, // Cuba
    {   98,  548,       0,  133688,       0,  0,  5,  0, }, // Davis
    {   98,  564,       0,  133693,       0,  0,  7,  0, }, // DumontDUrville
    {   98,  601,       0,  133700,       0,  0,  5,  0, }, // East_Timor
    {   98,  612,  133705,  133711,  133719,  6,  8,  8, }, // Easter
    {   98,  619,       0,  133727,       0,  0,  5,  0, }, // Ecuador
    {   98,  627,  133732,  133736,  133742,  4,  6,  6, }, // Europe_Central
    {   98,  642,  133748,  133752,  133758,  4,  6,  6, }, // Europe_Eastern
    {   98,  657,       0,  133764,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   98,  680,  133770,  133774,  133780,  4,  6,  6, }, // Europe_Western
    {   98,  695,  133786,  133793,  133802,  7,  9,  9, }, // Falkland
    {   98,  704,  133811,  133815,  133821,  4,  6,  6, }, // Fiji
    {   98,  709,       0,  133827,       0,  0,  7,  0, }, // French_Guiana
    {   98,  723,       0,  133834,       0,  0,  9,  0, }, // French_Southern
    {   98,  746,       0,  133843,       0,  0,  8,  0, }, // Galapagos
    {   98,  756,       0,  133851,       0,  0,  7,  0, }, // Gambier
    {   98,  764,  133858,  133863,  133870,  5,  7,  7, }, // Georgia
    {   98,  772,       0,  133877,       0,  0,  8,  0, }, // Gilbert_Islands
    {   98,  788,       0,  133885,       0,  0,  8,  0, }, // GMT
    {   98,  830,  133893,  133900,  133909,  7,  9,  9, }, // Greenland_Eastern
    {   98,  848,  133918,  133925,  133934,  7,  9,  9, }, // Greenland_Western
    {   98,  866,       0,  133943,       0,  0,  6,  0, }, // Guam
    {   98,  871,       0,  133949,       0,  0,  9,  0, }, // Gulf
    {   98,  876,       0,  133958,       0,  0,  5,  0, }, // Guyana
    {   98,  883,  133963,  133972,  133983,  9, 11, 11, }, // Hawaii_Aleutian
    {   98,  899,  133994,  133998,  134004,  4,  6,  6, }, // Hong_Kong
    {   98,  909,  134010,  134015,  134022,  5,  7,  7, }, // Hovd
    {   98,  914,       0,  134029,       0,  0,  6,  0, }, // India
    {   98,  920,       0,  134035,       0,  0,  5,  0, }, // Indian_Ocean
    {   98,  933,       0,  176766,       0,  0,  6,  0, }, // Indochina
    {   98,  943,       0,  134046,       0,  0,  6,  0, }, // Indonesia_Central
    {   98,  961,       0,  134052,       0,  0,  6,  0, }, // Indonesia_Eastern
    {   98,  979,       0,  134058,       0,  0,  6,  0, }, // Indonesia_Western
    {   98,  997,  134064,  134068,  134074,  4,  6,  6, }, // Iran
    {   98, 1008,  134080,  134087,  134096,  7,  9,  9, }, // Irkutsk
    {   98, 1016,  134105,  134110,  134117,  5,  7,  7, }, // Israel
    {   98, 1023,  134124,  134128,  134134,  4,  6,  6, }, // Japan
    {   98, 1029,  134140,  134151,  134164, 11, 13, 15, }, // Kamchatka
    {   98, 1047,       0,  134179,       0,  0,  5,  0, }, // Kazakhstan
    {   98, 1058,       0,  134184,       0,  0,  6,  0, }, // Kazakhstan_Eastern
    {   98, 1077,       0,  134190,       0,  0,  6,  0, }, // Kazakhstan_Western
    {   98, 1106,  134196,  134200,  134206,  4,  6,  6, }, // Korea
    {   98, 1112,       0,  134212,       0,  0,  5,  0, }, // Kosrae
    {   98, 1119,  134217,  134227,  134239, 10, 12, 12, }, // Krasnoyarsk
    {   98, 1151,       0,  134251,       0,  0,  6,  0, }, // Kyrgystan
    {   98, 1161,       0,  134257,       0,  0,  4,  0, }, // Lanka
    {   98, 1175,       0,  134261,       0,  0,  6,  0, }, // Line_Islands
    {   98, 1188,  134267,  134273,  134281,  6,  8,  8, }, // Lord_Howe
    {   98, 1198,  134289,  134293,  134299,  4,  6,  6, }, // Macau
    {   98, 1204,  134305,  134310,  134317,  5,  7,  7, }, // Magadan
    {   98, 1219,       0,  134324,       0,  0,  6,  0, }, // Malaysia
    {   98, 1228,       0,  134330,       0,  0,  6,  0, }, // Maldives
    {   98, 1237,       0,  134336,       0,  0,  6,  0, }, // Marquesas
    {   98, 1247,       0,  134342,       0,  0,  7,  0, }, // Marshall_Islands
    {   98, 1264,  134349,  134355,  134363,  6,  8,  8, }, // Mauritius
    {   98, 1274,       0,  134371,       0,  0,  4,  0, }, // Mawson
    {   98, 1281,  134375,  134383,  134393,  8, 10, 10, }, // Mexico_Pacific
    {   98, 1296,  134403,  134409,  134417,  6,  8,  8, }, // Mongolia
    {   98, 1305,  134425,  134430,  134437,  5,  7,  7, }, // Moscow
    {   98, 1312,       0,  134444,       0,  0,  4,  0, }, // Myanmar
    {   98, 1320,       0,  134448,       0,  0,  4,  0, }, // Nauru
    {   98, 1326,       0,  134452,       0,  0,  5,  0, }, // Nepal
    {   98, 1332,  134457,  134465,  134475,  8, 10, 12, }, // New_Caledonia
    {   98, 1346,  134487,  134492,  134499,  5,  7,  7, }, // New_Zealand
    {   98, 1358,  134506,  134511,  134518,  5,  7,  7, }, // Newfoundland
    {   98, 1371,       0,  134525,       0,  0,  5,  0, }, // Niue
    {   98, 1376,  134530,  134536,  134544,  6,  8,  8, }, // Norfolk
    {   98, 1384,  134552,  134566,  134582, 14, 16, 16, }, // Noronha
    {   98, 1392,       0,  134598,       0,  0,  9,  0, }, // North_Mariana
    {   98, 1406,  134607,  134614,  134623,  7,  9,  9, }, // Novosibirsk
    {   98, 1418,  134632,  134638,  134646,  6,  8,  8, }, // Omsk
    {   98, 1428,  134654,  134660,  134668,  6,  8,  8, }, // Pakistan
    {   98, 1437,       0,  134676,       0,  0,  4,  0, }, // Palau
    {   98, 1443,       0,  134680,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   98, 1460,  134689,  134694,  134701,  5,  7,  7, }, // Paraguay
    {   98, 1469,  134708,  134712,  134718,  4,  6,  6, }, // Peru
    {   98, 1474,  134724,  134729,  134736,  5,  7,  7, }, // Philippines
    {   98, 1486,       0,  134743,       0,  0,  6,  0, }, // Phoenix_Islands
    {   98, 1502,  176772,  176783,  176796, 11, 13, 13, }, // Pierre_Miquelon
    {   98, 1518,       0,  134789,       0,  0,  5,  0, }, // Pitcairn
    {   98, 1527,       0,  134794,       0,  0,  5,  0, }, // Ponape
    {   98, 1534,       0,  134799,       0,  0,  4,  0, }, // Pyongyang
    {   98, 1544,  134803,  134811,  134821,  8, 10, 10, }, // Qyzylorda
    {   98, 1554,       0,  134831,       0,  0,  5,  0, }, // Reunion
    {   98, 1562,       0,  134836,       0,  0,  5,  0, }, // Rothera
    {   98, 1570,  134841,  134846,  134853,  5,  7,  7, }, // Sakhalin
    {   98, 1579,  134860,  134865,  134872,  5,  7,  7, }, // Samara
    {   98, 1596,  134879,  134884,  134891,  5,  7,  7, }, // Samoa
    {   98, 1602,       0,  134898,       0,  0,  5,  0, }, // Seychelles
    {   98, 1624,       0,  134903,       0,  0,  7,  0, }, // Singapore
    {   98, 1634,       0,  134910,       0,  0,  7,  0, }, // Solomon
    {   98, 1642,       0,  134917,       0,  0,  6,  0, }, // South_Georgia
    {   98, 1656,       0,  134923,       0,  0,  5,  0, }, // Suriname
    {   98, 1676,       0,  134928,       0,  0,  6,  0, }, // Syowa
    {   98, 1682,       0,  134934,       0,  0,  5,  0, }, // Tahiti
    {   98, 1689,  134939,  134943,  134949,  4,  6,  6, }, // Taipei
    {   98, 1696,       0,  134955,       0,  0,  5,  0, }, // Tajikistan
    {   98, 1724,       0,  134960,       0,  0,  7,  0, }, // Tokelau
    {   98, 1732,  134967,  134971,  134977,  4,  6,  6, }, // Tonga
    {   98, 1738,       0,  134983,       0,  0,  5,  0, }, // Truk
    {   98, 1750,  134988,  134993,  135000,  5,  7,  7, }, // Turkmenistan
    {   98, 1763,       0,  135007,       0,  0,  5,  0, }, // Tuvalu
    {   98, 1777,  135012,  135017,  135024,  5,  7,  7, }, // Uruguay
    {   98, 1792,  135031,  135037,  135045,  6,  8,  8, }, // Uzbekistan
    {   98, 1803,  135053,  135058,  135065,  5,  7,  7, }, // Vanuatu
    {   98, 1811,       0,  135072,       0,  0,  6,  0, }, // Venezuela
    {   98, 1821,  135078,  135083,  135090,  5,  7,  7, }, // Vladivostok
    {   98, 1833,  135097,  135104,  135113,  7,  9,  9, }, // Volgograd
    {   98, 1843,       0,  135122,       0,  0,  6,  0, }, // Vostok
    {   98, 1850,       0,  135128,       0,  0,  5,  0, }, // Wake
    {   98, 1855,       0,  135133,       0,  0, 11,  0, }, // Wallis
    {   98, 1862,  135144,  135150,  135158,  6,  8,  8, }, // Yakutsk
    {   98, 1870,  135166,  135173,  135182,  7,  9,  9, }, // Yekaterinburg
    {   98, 1892,       0,  176809,       0,  0,  6,  0, }, // Yukon
    {   99,  142,  176815,  176841,  176875, 26, 34, 34, }, // America_Central Church/Cyrillic/Russia
    {   99,  158,  176909,  176937,  176973, 28, 36, 36, }, // America_Eastern
    {   99,  174,  177009,  177040,  177079, 31, 39, 39, }, // America_Mountain
    {   99,  191,  177118,  177139,  177168, 21, 29, 29, }, // America_Pacific
    {   99,  286,  177197,  177219,  177249, 22, 30, 30, }, // Atlantic
    {   99,  627,  177279,  177304,  177337, 25, 33, 33, }, // Europe_Central
    {   99,  642,  177370,  177397,  177432, 27, 35, 35, }, // Europe_Eastern
    {   99,  657,       0,  177467,       0,  0, 34,  0, }, // Europe_Further_Eastern
    {   99,  680,  177501,  177527,  177561, 26, 34, 34, }, // Europe_Western
    {   99,  788,       0,  177595,       0,  0, 28,  0, }, // GMT
    {   99, 1008,  177623,  177641,  177667, 18, 26, 26, }, // Irkutsk
    {   99, 1058,       0,  177693,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {   99, 1077,       0,  177715,       0,  0, 21,  0, }, // Kazakhstan_Western
    {   99, 1119,  177736,  177756,  177784, 20, 28, 28, }, // Krasnoyarsk
    {   99, 1151,       0,  177812,       0,  0,  9,  0, }, // Kyrgystan
    {   99, 1204,  177821,  177840,  177867, 19, 27, 27, }, // Magadan
    {   99, 1305,  177894,  177912,  177938, 18, 26, 26, }, // Moscow
    {   99, 1406,  177964,  177985,  178014, 21, 29, 29, }, // Novosibirsk
    {   99, 1418,  178043,  178058,  178081, 15, 23, 23, }, // Omsk
    {   99, 1570,  178104,  178123,  178150, 19, 27, 27, }, // Sakhalin
    {   99, 1821,  178177,  178199,  178229, 22, 30, 30, }, // Vladivostok
    {   99, 1833,  178259,  178280,  178309, 21, 29, 29, }, // Volgograd
    {   99, 1862,  178338,  178355,  178380, 17, 25, 25, }, // Yakutsk
    {   99, 1870,  178405,  178430,  178463, 25, 33, 33, }, // Yekaterinburg
    {  100,    0,  178496,  178507,  178529, 11, 22, 19, }, // Acre Chuvash/Cyrillic/Russia
    {  100,    5,       0,  178548,       0,  0, 17,  0, }, // Afghanistan
    {  100,   17,       0,  178565,       0,  0, 17,  0, }, // Africa_Central
    {  100,   32,       0,  178582,       0,  0, 24,  0, }, // Africa_Eastern
    {  100,   65,       0,  178606,       0,  0, 20,  0, }, // Africa_Southern
    {  100,   81,  178626,  178644,  178673, 18, 29, 26, }, // Africa_Western
    {  100,  107,  178699,  178712,  178736, 13, 24, 21, }, // Alaska
    {  100,  128,  178757,  178770,  178794, 13, 24, 21, }, // Almaty
    {  100,  135,  178815,  178830,  178856, 15, 26, 23, }, // Amazon
    {  100,  142,  178879,  178897,  178926, 18, 29, 26, }, // America_Central
    {  100,  158,  178952,  178977,  179013, 25, 36, 33, }, // America_Eastern
    {  100,  174,  179046,  179072,  179109, 26, 37, 34, }, // America_Mountain
    {  100,  191,  179143,  179161,  179190, 18, 29, 26, }, // America_Pacific
    {  100,  207,  179216,  179230,  179255, 14, 25, 22, }, // Anadyr
    {  100,  214,  179277,  179288,  179310, 11, 22, 19, }, // Apia
    {  100,  219,  179329,  179341,  179364, 12, 23, 20, }, // Aqtau
    {  100,  225,  179384,  179397,  179421, 13, 24, 21, }, // Aqtobe
    {  100,  232,  179442,  179453,  179475, 11, 22, 19, }, // Arabian
    {  100,  240,  179494,  179510,  179537, 16, 27, 24, }, // Argentina
    {  100,  250,  179561,  179582,  179614, 21, 32, 29, }, // Argentina_Western
    {  100,  268,  179643,  179656,  179680, 13, 24, 21, }, // Armenia
    {  100,  286,  179701,  179717,  179744, 16, 27, 24, }, // Atlantic
    {  100,  295,  179768,  179787,  179817, 19, 30, 27, }, // Australia_Central
    {  100,  313,  179844,  179868,  179903, 24, 35, 32, }, // Australia_CentralWestern
    {  100,  338,  179935,  179961,  179998, 26, 37, 34, }, // Australia_Eastern
    {  100,  356,  180032,  180052,  180083, 20, 31, 28, }, // Australia_Western
    {  100,  374,  180111,  180129,  180158, 18, 29, 26, }, // Azerbaijan
    {  100,  385,  180184,  180205,  180237, 21, 32, 29, }, // Azores
    {  100,  397,  180266,  180282,  180309, 16, 27, 24, }, // Bangladesh
    {  100,  415,       0,  180333,       0,  0, 12,  0, }, // Bhutan
    {  100,  422,       0,  180345,       0,  0, 13,  0, }, // Bolivia
    {  100,  437,  180358,  180372,  180397, 14, 25, 22, }, // Brasilia
    {  100,  454,       0,  180419,       0,  0, 24,  0, }, // Brunei
    {  100,  461,  180443,  180460,  180488, 17, 28, 25, }, // Cape_Verde
    {  100,  472,       0,  180513,       0,  0, 12,  0, }, // Casey
    {  100,  478,       0,  180525,       0,  0, 14,  0, }, // Chamorro
    {  100,  487,  180539,  180551,  180574, 12, 23, 20, }, // Chatham
    {  100,  495,  180594,  180605,  180627, 11, 22, 19, }, // Chile
    {  100,  501,  180646,  180658,  180681, 12, 23, 20, }, // China
    {  100,  507,       0,  180701,       0,  0, 21,  0, }, // Christmas
    {  100,  517,       0,  180722,       0,  0, 22,  0, }, // Cocos
    {  100,  523,  180744,  180758,  180783, 14, 25, 22, }, // Colombia
    {  100,  532,  180805,  180827,  180860, 22, 33, 30, }, // Cook
    {  100,  537,  180890,  180901,  180923, 11, 22, 19, }, // Cuba
    {  100,  548,       0,  180942,       0,  0, 13,  0, }, // Davis
    {  100,  564,       0,  180955,       0,  0, 21,  0, }, // DumontDUrville
    {  100,  601,       0,  180976,       0,  0, 23,  0, }, // East_Timor
    {  100,  612,  180999,  181020,  181052, 21, 32, 29, }, // Easter
    {  100,  619,       0,  181081,       0,  0, 14,  0, }, // Ecuador
    {  100,  627,  181095,  181112,  181140, 17, 28, 25, }, // Europe_Central
    {  100,  642,  181165,  181189,  181224, 24, 35, 32, }, // Europe_Eastern
    {  100,  657,       0,  181256,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  100,  680,  181286,  181304,  181333, 18, 29, 26, }, // Europe_Western
    {  100,  695,  181359,  181384,  181420, 25, 36, 33, }, // Falkland
    {  100,  704,  181453,  181465,  181488, 12, 23, 20, }, // Fiji
    {  100,  709,       0,  181508,       0,  0, 20,  0, }, // French_Guiana
    {  100,  723,       0,  181528,       0,  0, 48,  0, }, // French_Southern
    {  100,  746,       0,  181576,       0,  0, 26,  0, }, // Galapagos
    {  100,  756,       0,  181602,       0,  0, 13,  0, }, // Gambier
    {  100,  764,  181615,  181627,  181650, 12, 23, 20, }, // Georgia
    {  100,  772,       0,  181670,       0,  0, 26,  0, }, // Gilbert_Islands
    {  100,  788,       0,  181696,       0,  0, 22,  0, }, // GMT
    {  100,  802,  181718,  181734,  181761, 16, 27, 24, }, // Greenland
    {  100,  830,  181785,  181812,  181850, 27, 38, 35, }, // Greenland_Eastern
    {  100,  848,  181885,  181906,  181938, 21, 32, 19, }, // Greenland_Western
    {  100,  866,       0,  181957,       0,  0, 22,  0, }, // Guam
    {  100,  871,       0,  181979,       0,  0, 18,  0, }, // Gulf
    {  100,  876,       0,  181997,       0,  0, 13,  0, }, // Guyana
    {  100,  883,  182010,  182029,  182059, 19, 30, 27, }, // Hawaii_Aleutian
    {  100,  899,  182086,  182100,  182125, 14, 25, 22, }, // Hong_Kong
    {  100,  909,  182147,  182158,  182180, 11, 22, 19, }, // Hovd
    {  100,  914,       0,  182199,       0,  0, 11,  0, }, // India
    {  100,  920,       0,  182210,       0,  0, 18,  0, }, // Indian_Ocean
    {  100,  933,       0,  182228,       0,  0, 16,  0, }, // Indochina
    {  100,  943,       0,  182244,       0,  0, 19,  0, }, // Indonesia_Central
    {  100,  961,       0,  182263,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  100,  979,       0,  182289,       0,  0, 20,  0, }, // Indonesia_Western
    {  100,  997,  182309,  182320,  182342, 11, 22, 19, }, // Iran
    {  100, 1008,  182361,  182375,  182400, 14, 25, 22, }, // Irkutsk
    {  100, 1016,  182422,  182436,  182461, 14, 25, 22, }, // Israel
    {  100, 1023,  182483,  182495,  182518, 12, 23, 20, }, // Japan
    {  100, 1029,  182538,  182569,  182611, 31, 42, 39, }, // Kamchatka
    {  100, 1047,       0,  182650,       0,  0, 16,  0, }, // Kazakhstan
    {  100, 1058,       0,  182666,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  100, 1077,       0,  182693,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  100, 1106,  182714,  182726,  182749, 12, 23, 20, }, // Korea
    {  100, 1112,       0,  182769,       0,  0, 13,  0, }, // Kosrae
    {  100, 1119,  182782,  182799,  182827, 17, 28, 25, }, // Krasnoyarsk
    {  100, 1151,       0,  182852,       0,  0, 14,  0, }, // Kyrgystan
    {  100, 1161,       0,  182866,       0,  0, 12,  0, }, // Lanka
    {  100, 1175,       0,  182878,       0,  0, 21,  0, }, // Line_Islands
    {  100, 1188,  182899,  182914,  182940, 15, 26, 23, }, // Lord_Howe
    {  100, 1198,  182963,  182975,  182998, 12, 23, 20, }, // Macau
    {  100, 1204,  183018,  183032,  183057, 14, 25, 22, }, // Magadan
    {  100, 1219,       0,  183079,       0,  0, 14,  0, }, // Malaysia
    {  100, 1228,       0,  183093,       0,  0, 17,  0, }, // Maldives
    {  100, 1237,       0,  183110,       0,  0, 25,  0, }, // Marquesas
    {  100, 1247,       0,  183135,       0,  0, 24,  0, }, // Marshall_Islands
    {  100, 1264,  183159,  183174,  183200, 15, 26, 23, }, // Mauritius
    {  100, 1274,       0,  183223,       0,  0, 13,  0, }, // Mawson
    {  100, 1281,  183236,  183262,  183299, 26, 37, 34, }, // Mexico_Pacific
    {  100, 1296,  183333,  183350,  183378, 17, 28, 25, }, // Mongolia
    {  100, 1305,  183403,  183416,  183440, 13, 24, 21, }, // Moscow
    {  100, 1312,       0,  183461,       0,  0, 13,  0, }, // Myanmar
    {  100, 1320,       0,  183474,       0,  0, 12,  0, }, // Nauru
    {  100, 1326,       0,  183486,       0,  0, 12,  0, }, // Nepal
    {  100, 1332,  183498,  183518,  183549, 20, 31, 28, }, // New_Caledonia
    {  100, 1346,  183577,  183596,  183626, 19, 30, 27, }, // New_Zealand
    {  100, 1358,  183653,  183672,  183702, 19, 30, 27, }, // Newfoundland
    {  100, 1371,       0,  183729,       0,  0, 11,  0, }, // Niue
    {  100, 1376,  183740,  183754,  183779, 14, 25, 22, }, // Norfolk
    {  100, 1384,  183801,  183827,  183864, 26, 37, 34, }, // Noronha
    {  100, 1392,       0,  183898,       0,  0, 35,  0, }, // North_Mariana
    {  100, 1406,  183933,  183951,  183980, 18, 29, 26, }, // Novosibirsk
    {  100, 1418,  184006,  184017,  184039, 11, 22, 19, }, // Omsk
    {  100, 1428,  184058,  184073,  184099, 15, 26, 23, }, // Pakistan
    {  100, 1437,       0,  184122,       0,  0, 12,  0, }, // Palau
    {  100, 1443,       0,  184134,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  100, 1460,  184160,  184175,  184201, 15, 26, 23, }, // Paraguay
    {  100, 1469,  184224,  184235,  184257, 11, 22, 19, }, // Peru
    {  100, 1474,  184276,  184294,  184323, 18, 29, 26, }, // Philippines
    {  100, 1486,       0,  184349,       0,  0, 13,  0, }, // Phoenix_Islands
    {  100, 1502,  184362,  184390,  184429, 28, 39, 36, }, // Pierre_Miquelon
    {  100, 1518,       0,  184465,       0,  0, 14,  0, }, // Pitcairn
    {  100, 1527,       0,  184479,       0,  0, 13,  0, }, // Ponape
    {  100, 1534,       0,   90148,       0,  0,  7,  0, }, // Pyongyang
    {  100, 1544,  184492,  184508,  184535, 16, 27, 24, }, // Qyzylorda
    {  100, 1554,       0,  184559,       0,  0, 14,  0, }, // Reunion
    {  100, 1562,       0,  184573,       0,  0, 13,  0, }, // Rothera
    {  100, 1570,  184586,  184600,  184625, 14, 25, 22, }, // Sakhalin
    {  100, 1579,  184647,  184659,  184682, 12, 23, 20, }, // Samara
    {  100, 1596,  184702,  184714,  184737, 12, 23, 20, }, // Samoa
    {  100, 1602,       0,  184757,       0,  0, 23,  0, }, // Seychelles
    {  100, 1624,       0,  184780,       0,  0, 15,  0, }, // Singapore
    {  100, 1634,       0,  184795,       0,  0, 14,  0, }, // Solomon
    {  100, 1642,       0,  184809,       0,  0, 20,  0, }, // South_Georgia
    {  100, 1656,       0,  184829,       0,  0, 14,  0, }, // Suriname
    {  100, 1676,       0,  184843,       0,  0, 11,  0, }, // Syowa
    {  100, 1682,       0,  184854,       0,  0, 12,  0, }, // Tahiti
    {  100, 1689,  184866,  184879,  184903, 13, 24, 21, }, // Taipei
    {  100, 1696,       0,  184924,       0,  0, 18,  0, }, // Tajikistan
    {  100, 1724,       0,  184942,       0,  0, 14,  0, }, // Tokelau
    {  100, 1732,  184956,  184968,  184991, 12, 23, 20, }, // Tonga
    {  100, 1738,       0,  185011,       0,  0, 11,  0, }, // Truk
    {  100, 1750,  185022,  185041,  185071, 19, 30, 27, }, // Turkmenistan
    {  100, 1763,       0,  185098,       0,  0, 13,  0, }, // Tuvalu
    {  100, 1777,  185111,  185125,  185150, 14, 25, 22, }, // Uruguay
    {  100, 1792,  185172,  185189,  185217, 17, 28, 25, }, // Uzbekistan
    {  100, 1803,  185242,  185256,  185281, 14, 25, 22, }, // Vanuatu
    {  100, 1811,       0,  185303,       0,  0, 16,  0, }, // Venezuela
    {  100, 1821,  185319,  185337,  185366, 18, 29, 26, }, // Vladivostok
    {  100, 1833,  185392,  185408,  185435, 16, 27, 24, }, // Volgograd
    {  100, 1843,       0,  185459,       0,  0, 13,  0, }, // Vostok
    {  100, 1850,       0,  185472,       0,  0, 11,  0, }, // Wake
    {  100, 1855,       0,  185483,       0,  0, 25,  0, }, // Wallis
    {  100, 1862,  185508,  185521,  185545, 13, 24, 21, }, // Yakutsk
    {  100, 1870,  185566,  185585,  185615, 19, 30, 27, }, // Yekaterinburg
    {  100, 1892,       0,  185642,       0,  0, 11,  0, }, // Yukon
    {  101,   17,       0,  185653,       0,  0, 28,  0, }, // Africa_Central Colognian/Latin/Germany
    {  101,   32,       0,  185681,       0,  0, 22,  0, }, // Africa_Eastern
    {  101,   65,       0,  185703,       0,  0, 23,  0, }, // Africa_Southern
    {  101,   81,  185726,  185749,  185784, 23, 35, 29, }, // Africa_Western
    {  101,  385,  185813,  185830,  185859, 17, 29, 23, }, // Azores
    {  101,  461,  185882,  185903,  185936, 21, 33, 27, }, // Cape_Verde
    {  101,  627,  185963,  185987,  186023, 24, 36, 30, }, // Europe_Central
    {  101,  642,  186053,  186073,  186105, 20, 32, 26, }, // Europe_Eastern
    {  101,  680,  186131,  186152,  186185, 21, 33, 27, }, // Europe_Western
    {  101,  788,       0,  186212,       0,  0, 27,  0, }, // GMT
    {  101,  920,       0,  186239,       0,  0, 29,  0, }, // Indian_Ocean
    {  101, 1264,  186268,  186286,  186316, 18, 30, 24, }, // Mauritius
    {  101, 1554,       0,  186340,       0,  0, 16,  0, }, // Reunion
    {  101, 1602,       0,  186356,       0,  0, 22,  0, }, // Seychelles
    {  103,  788,       0,  186378,       0,  0, 24,  0, }, // GMT Corsican/Latin/France
    {  104,    0,  186402,  186414,  186437, 12, 23, 19, }, // Acre Croatian/Latin/Croatia
    {  104,    5,       0,  186456,       0,  0, 21,  0, }, // Afghanistan
    {  104,   17,       0,  186477,       0,  0, 22,  0, }, // Africa_Central
    {  104,   32,       0,  186499,       0,  0, 22,  0, }, // Africa_Eastern
    {  104,   65,       0,  186521,       0,  0, 20,  0, }, // Africa_Southern
    {  104,   81,  186541,  186563,  186596, 22, 33, 29, }, // Africa_Western
    {  104,  107,  186625,  186640,  186666, 15, 26, 22, }, // Alaska
    {  104,  128,  186688,  186705,  186733, 17, 28, 24, }, // Almaty
    {  104,  135,  186757,  186774,  186802, 17, 28, 24, }, // Amazon
    {  104,  142,  186826,  186843,  186871, 17, 28, 24, }, // America_Central
    {  104,  158,   97761,   97793,   97836, 15, 26, 22, }, // America_Eastern
    {  104,  174,   97875,   97909,   97954, 17, 28, 24, }, // America_Mountain
    {  104,  191,   97995,   98029,   98074, 17, 28, 24, }, // America_Pacific
    {  104,  207,  186895,  186912,  186940, 17, 28, 24, }, // Anadyr
    {  104,  214,  186964,  186977,  187001, 13, 24, 20, }, // Apia
    {  104,  219,  187021,  187040,  187070, 19, 30, 26, }, // Aqtau
    {  104,  225,  187096,  187116,  187147, 20, 31, 27, }, // Aqtobe
    {  104,  232,  187174,  187189,  187215, 15, 26, 22, }, // Arabian
    {  104,  240,   98513,   98539,   98576, 19, 30, 26, }, // Argentina
    {  104,  250,  187237,  187263,  187300, 26, 37, 33, }, // Argentina_Western
    {  104,  268,  187333,  187349,  187376, 16, 27, 23, }, // Armenia
    {  104,  286,   98685,   98719,   98764, 17, 28, 24, }, // Atlantic
    {  104,  295,  187399,  187424,  187460, 25, 36, 32, }, // Australia_Central
    {  104,  313,  187492,  187528,  187575, 36, 47, 43, }, // Australia_CentralWestern
    {  104,  338,  187618,  187643,  187679, 25, 36, 32, }, // Australia_Eastern
    {  104,  356,  187711,  187736,  187772, 25, 36, 32, }, // Australia_Western
    {  104,  374,  187804,  187826,  187859, 22, 33, 29, }, // Azerbaijan
    {  104,  385,  187888,  187903,  187929, 15, 26, 22, }, // Azores
    {  104,  397,  187951,  187970,  188000, 19, 30, 26, }, // Bangladesh
    {  104,  415,       0,  188026,       0,  0, 16,  0, }, // Bhutan
    {  104,  422,       0,  188042,       0,  0, 18,  0, }, // Bolivia
    {  104,  437,  188060,  188077,  188105, 17, 28, 24, }, // Brasilia
    {  104,  454,       0,  188129,       0,  0, 28,  0, }, // Brunei
    {  104,  461,  188157,  188184,  188222, 27, 38, 34, }, // Cape_Verde
    {  104,  472,       0,  188256,       0,  0, 15,  0, }, // Casey
    {  104,  478,       0,  188271,       0,  0, 27,  0, }, // Chamorro
    {  104,  487,  188298,  188314,  188341, 16, 27, 23, }, // Chatham
    {  104,  495,  188364,  188381,  188409, 17, 28, 24, }, // Chile
    {  104,  501,  101571,  188433,  188459, 15, 26, 22, }, // China
    {  104,  507,       0,  188481,       0,  0, 22,  0, }, // Christmas
    {  104,  517,       0,  188503,       0,  0, 23,  0, }, // Cocos
    {  104,  523,  188526,  188545,  188575, 19, 30, 26, }, // Colombia
    {  104,  532,  188601,  188623,  188656, 22, 33, 46, }, // Cook
    {  104,  537,  188702,  188718,  188745, 16, 27, 23, }, // Cuba
    {  104,  548,       0,  188768,       0,  0, 14,  0, }, // Davis
    {  104,  564,       0,  188782,       0,  0, 25,  0, }, // DumontDUrville
    {  104,  601,       0,  188807,       0,  0, 23,  0, }, // East_Timor
    {  104,  612,  188830,  188853,  188887, 23, 34, 30, }, // Easter
    {  104,  619,       0,  188917,       0,  0, 18,  0, }, // Ecuador
    {  104,  627,  188935,  188958,  188992, 23, 34, 30, }, // Europe_Central
    {  104,  642,  189022,  189045,  189079, 23, 34, 30, }, // Europe_Eastern
    {  104,  657,       0,  189109,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  104,  680,  189139,  189162,  189196, 23, 34, 30, }, // Europe_Western
    {  104,  695,  189226,  189245,  189275, 19, 30, 26, }, // Falkland
    {  104,  704,  189301,  189316,  189342, 15, 26, 22, }, // Fiji
    {  104,  709,       0,  189364,       0,  0, 24,  0, }, // French_Guiana
    {  104,  723,       0,  189388,       0,  0, 36,  0, }, // French_Southern
    {  104,  746,       0,  189424,       0,  0, 18,  0, }, // Galapagos
    {  104,  756,       0,  189442,       0,  0, 16,  0, }, // Gambier
    {  104,  764,  189458,  189475,  189503, 17, 28, 24, }, // Georgia
    {  104,  772,       0,  189527,       0,  0, 25,  0, }, // Gilbert_Islands
    {  104,  788,       0,  189552,       0,  0, 19,  0, }, // GMT
    {  104,  830,  189571,  189597,  189634, 26, 37, 33, }, // Greenland_Eastern
    {  104,  848,  189667,  189693,  189730, 26, 37, 33, }, // Greenland_Western
    {  104,  866,       0,  189763,       0,  0, 26,  0, }, // Guam
    {  104,  871,       0,  189789,       0,  0, 28,  0, }, // Gulf
    {  104,  876,       0,  100799,       0,  0, 17,  0, }, // Guyana
    {  104,  883,  189817,  189842,  189878, 25, 36, 32, }, // Hawaii_Aleutian
    {  104,  899,  189910,  189928,  189957, 18, 29, 25, }, // Hong_Kong
    {  104,  909,  189982,  189997,  190023, 15, 26, 22, }, // Hovd
    {  104,  914,       0,  190045,       0,  0, 16,  0, }, // India
    {  104,  920,       0,  190061,       0,  0, 24,  0, }, // Indian_Ocean
    {  104,  933,       0,  190085,       0,  0, 19,  0, }, // Indochina
    {  104,  943,       0,  190104,       0,  0, 27,  0, }, // Indonesia_Central
    {  104,  961,       0,  190131,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  104,  979,       0,  190158,       0,  0, 27,  0, }, // Indonesia_Western
    {  104,  997,  190185,  190200,  190226, 15, 26, 22, }, // Iran
    {  104, 1008,  190248,  190264,  190291, 16, 27, 23, }, // Irkutsk
    {  104, 1016,  190314,  190331,  190359, 17, 28, 24, }, // Israel
    {  104, 1023,  190383,  190399,  190426, 16, 27, 23, }, // Japan
    {  104, 1029,  190449,  190480,  190522, 31, 42, 38, }, // Kamchatka
    {  104, 1047,       0,  102050,       0,  0, 20,  0, }, // Kazakhstan
    {  104, 1058,       0,  102070,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  104, 1077,       0,  102097,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  104, 1106,  190560,  190576,  190603, 16, 27, 23, }, // Korea
    {  104, 1112,       0,  190626,       0,  0, 14,  0, }, // Kosrae
    {  104, 1119,  190640,  190660,  190691, 20, 31, 27, }, // Krasnoyarsk
    {  104, 1151,       0,  102292,       0,  0, 20,  0, }, // Kyrgystan
    {  104, 1161,       0,  190718,       0,  0, 17,  0, }, // Lanka
    {  104, 1175,       0,  190735,       0,  0, 25,  0, }, // Line_Islands
    {  104, 1188,  190760,  190783,  190817, 23, 34, 30, }, // Lord_Howe
    {  104, 1198,  190847,  190863,  190890, 16, 27, 23, }, // Macau
    {  104, 1204,  190913,  190931,  190960, 18, 29, 25, }, // Magadan
    {  104, 1219,       0,  190985,       0,  0, 18,  0, }, // Malaysia
    {  104, 1228,       0,  191003,       0,  0, 17,  0, }, // Maldives
    {  104, 1237,       0,  191020,       0,  0, 19,  0, }, // Marquesas
    {  104, 1247,       0,  191039,       0,  0, 24,  0, }, // Marshall_Islands
    {  104, 1264,  191063,  191082,  191112, 19, 30, 26, }, // Mauritius
    {  104, 1274,       0,  191138,       0,  0, 17,  0, }, // Mawson
    {  104, 1281,  191155,  191181,  191218, 26, 37, 33, }, // Mexico_Pacific
    {  104, 1296,  191251,  191271,  191302, 20, 31, 27, }, // Mongolia
    {  104, 1305,  191329,  191346,  191374, 17, 28, 24, }, // Moscow
    {  104, 1312,       0,  191398,       0,  0, 18,  0, }, // Myanmar
    {  104, 1320,       0,  191416,       0,  0, 14,  0, }, // Nauru
    {  104, 1326,       0,  191430,       0,  0, 16,  0, }, // Nepal
    {  104, 1332,  191446,  191469,  191503, 23, 34, 30, }, // New_Caledonia
    {  104, 1346,  191533,  191554,  191586, 21, 32, 28, }, // New_Zealand
    {  104, 1358,  191614,  191637,  191671, 23, 34, 30, }, // Newfoundland
    {  104, 1371,       0,  191701,       0,  0, 13,  0, }, // Niue
    {  104, 1376,  191714,  191735,  191767, 21, 32, 28, }, // Norfolk
    {  104, 1384,  191795,  191828,  191872, 33, 44, 40, }, // Noronha
    {  104, 1392,       0,  191912,       0,  0, 33,  0, }, // North_Mariana
    {  104, 1406,  191945,  191965,  191996, 20, 31, 27, }, // Novosibirsk
    {  104, 1418,  192023,  192036,  192060, 13, 24, 20, }, // Omsk
    {  104, 1428,  192080,  192099,  192129, 19, 30, 26, }, // Pakistan
    {  104, 1437,       0,  192155,       0,  0, 14,  0, }, // Palau
    {  104, 1443,       0,  192169,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  104, 1460,  192195,  192214,  192244, 19, 30, 26, }, // Paraguay
    {  104, 1469,  192270,  192287,  192315, 17, 28, 24, }, // Peru
    {  104, 1474,  192339,  192357,  192386, 18, 29, 25, }, // Philippines
    {  104, 1486,       0,  192411,       0,  0, 21,  0, }, // Phoenix_Islands
    {  104, 1502,  192432,  192464,  192507, 32, 43, 39, }, // Pierre_Miquelon
    {  104, 1518,       0,  192546,       0,  0, 19,  0, }, // Pitcairn
    {  104, 1527,       0,  192565,       0,  0, 18,  0, }, // Ponape
    {  104, 1534,       0,  192583,       0,  0, 19,  0, }, // Pyongyang
    {  104, 1544,  192602,  192625,  192659, 23, 34, 30, }, // Qyzylorda
    {  104, 1554,       0,  192689,       0,  0, 16,  0, }, // Reunion
    {  104, 1562,       0,  192705,       0,  0, 15,  0, }, // Rothera
    {  104, 1570,  192720,  192738,  192767, 18, 29, 25, }, // Sakhalin
    {  104, 1579,  192792,  192808,  192835, 16, 27, 23, }, // Samara
    {  104, 1596,  192858,  192875,  192903, 17, 28, 24, }, // Samoa
    {  104, 1602,       0,  192927,       0,  0, 17,  0, }, // Seychelles
    {  104, 1624,       0,  192944,       0,  0, 19,  0, }, // Singapore
    {  104, 1634,       0,  192963,       0,  0, 25,  0, }, // Solomon
    {  104, 1642,       0,  192988,       0,  0, 22,  0, }, // South_Georgia
    {  104, 1656,       0,  193010,       0,  0, 18,  0, }, // Suriname
    {  104, 1676,       0,  193028,       0,  0, 13,  0, }, // Syowa
    {  104, 1682,       0,  193041,       0,  0, 16,  0, }, // Tahiti
    {  104, 1689,  193057,  193073,  193100, 16, 27, 23, }, // Taipei
    {  104, 1696,       0,  104843,       0,  0, 22,  0, }, // Tajikistan
    {  104, 1724,       0,  193123,       0,  0, 16,  0, }, // Tokelau
    {  104, 1732,  193139,  193152,  193176, 13, 24, 20, }, // Tonga
    {  104, 1738,       0,  193196,       0,  0, 14,  0, }, // Truk
    {  104, 1750,  104972,  104995,  105029, 23, 34, 30, }, // Turkmenistan
    {  104, 1763,       0,  193210,       0,  0, 15,  0, }, // Tuvalu
    {  104, 1777,  193225,  193243,  193272, 18, 29, 25, }, // Uruguay
    {  104, 1792,  105150,  105171,  105203, 21, 32, 28, }, // Uzbekistan
    {  104, 1803,  193297,  193313,  193340, 16, 27, 23, }, // Vanuatu
    {  104, 1811,       0,  193363,       0,  0, 19,  0, }, // Venezuela
    {  104, 1821,  193382,  193403,  193435, 21, 32, 28, }, // Vladivostok
    {  104, 1833,  193463,  193483,  193514, 20, 31, 27, }, // Volgograd
    {  104, 1843,       0,  105335,       0,  0, 16,  0, }, // Vostok
    {  104, 1850,       0,  193541,       0,  0, 18,  0, }, // Wake
    {  104, 1855,       0,  193559,       0,  0, 29,  0, }, // Wallis
    {  104, 1862,  193588,  193604,  193631, 16, 27, 23, }, // Yakutsk
    {  104, 1870,  193654,  193677,  193711, 23, 34, 30, }, // Yekaterinburg
    {  104, 1892,       0,  193741,       0,  0, 16,  0, }, // Yukon
    {  106,    0,  193757,  193769,  193792, 12, 23, 18, }, // Acre Czech/Latin/Czechia
    {  106,    5,       0,  193810,       0,  0, 13,  0, }, // Afghanistan
    {  106,   17,       0,  193823,       0,  0, 17,  0, }, // Africa_Central
    {  106,   32,       0,  193840,       0,  0, 18,  0, }, // Africa_Eastern
    {  106,   65,       0,  193858,       0,  0, 15,  0, }, // Africa_Southern
    {  106,   81,  193873,  193890,  193918, 17, 28, 23, }, // Africa_Western
    {  106,  107,  193941,  193953,  193976, 12, 23, 18, }, // Alaska
    {  106,  128,  193994,  194006,  194029, 12, 23, 18, }, // Almaty
    {  106,  135,  194047,  194060,  194084, 13, 24, 19, }, // Amazon
    {  106,  142,  194103,  194131,  194170, 28, 39, 34, }, // America_Central
    {  106,  158,  194204,  194231,  194269, 27, 38, 33, }, // America_Eastern
    {  106,  174,  194302,  194327,  194363, 25, 36, 31, }, // America_Mountain
    {  106,  191,  194394,  194422,  194461, 28, 39, 34, }, // America_Pacific
    {  106,  207,  194495,  194508,  194532, 13, 24, 19, }, // Anadyr
    {  106,  214,  194551,  194562,  194584, 11, 22, 17, }, // Apia
    {  106,  219,  194601,  194613,  194636, 12, 23, 18, }, // Aqtau
    {  106,  225,  194654,  194666,  194689, 12, 23, 18, }, // Aqtobe
    {  106,  232,  194707,  194718,  194740, 11, 22, 17, }, // Arabian
    {  106,  240,  194757,  194772,  194798, 15, 26, 21, }, // Argentina
    {  106,  250,  194819,  194840,  194872, 21, 32, 27, }, // Argentina_Western
    {  106,  268,  194899,  194911,  194934, 12, 23, 18, }, // Armenia
    {  106,  286,  194952,  194966,  194991, 14, 25, 20, }, // Atlantic
    {  106,  295,  195011,  195031,  195062, 20, 31, 26, }, // Australia_Central
    {  106,  313,  195088,  195116,  195155, 28, 39, 34, }, // Australia_CentralWestern
    {  106,  338,  195189,  195210,  195242, 21, 32, 27, }, // Australia_Eastern
    {  106,  356,  195269,  195289,  195320, 20, 31, 26, }, // Australia_Western
    {  106,  374,  195346,  195364,  195393, 18, 29, 24, }, // Azerbaijan
    {  106,  385,  195417,  195428,  195450, 11, 22, 17, }, // Azores
    {  106,  397,  195467,  195483,  195510, 16, 27, 22, }, // Bangladesh
    {  106,  415,       0,  195532,       0,  0, 13,  0, }, // Bhutan
    {  106,  422,       0,  195545,       0,  0, 14,  0, }, // Bolivia
    {  106,  437,  195559,  195574,  195600, 15, 26, 21, }, // Brasilia
    {  106,  454,       0,  195621,       0,  0, 13,  0, }, // Brunei
    {  106,  461,  195634,  195648,  195673, 14, 25, 20, }, // Cape_Verde
    {  106,  472,       0,  195693,       0,  0, 19,  0, }, // Casey
    {  106,  478,       0,  195712,       0,  0, 14,  0, }, // Chamorro
    {  106,  487,  195726,  195740,  195765, 14, 25, 20, }, // Chatham
    {  106,  495,  195785,  195796,  195818, 11, 22, 17, }, // Chile
    {  106,  501,  195835,  195845,  195866, 10, 21, 16, }, // China
    {  106,  507,       0,  195882,       0,  0, 21,  0, }, // Christmas
    {  106,  517,       0,  195903,       0,  0, 22,  0, }, // Cocos
    {  106,  523,  195925,  195940,  195966, 15, 26, 21, }, // Colombia
    {  106,  532,  195987,  196008,  196040, 21, 32, 27, }, // Cook
    {  106,  537,  196067,  196079,  196102, 12, 23, 18, }, // Cuba
    {  106,  548,       0,  196120,       0,  0, 20,  0, }, // Davis
    {  106,  564,       0,  196140,       0,  0, 29,  0, }, // DumontDUrville
    {  106,  601,       0,  196169,       0,  0, 19,  0, }, // East_Timor
    {  106,  612,  196188,  196213,  196249, 25, 36, 31, }, // Easter
    {  106,  619,       0,  196280,       0,  0, 14,  0, }, // Ecuador
    {  106,  627,  196294,  196312,  196341, 18, 29, 24, }, // Europe_Central
    {  106,  642,  196365,  196384,  196414, 19, 30, 25, }, // Europe_Eastern
    {  106,  657,       0,  196439,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  106,  680,  196463,  196481,  196510, 18, 29, 24, }, // Europe_Western
    {  106,  695,  196534,  196549,  196575, 15, 26, 21, }, // Falkland
    {  106,  704,  196596,  196609,  196633, 13, 24, 19, }, // Fiji
    {  106,  709,       0,  196652,       0,  0, 23,  0, }, // French_Guiana
    {  106,  723,       0,  196675,       0,  0, 47,  0, }, // French_Southern
    {  106,  746,       0,  196722,       0,  0, 14,  0, }, // Galapagos
    {  106,  756,       0,  196736,       0,  0, 14,  0, }, // Gambier
    {  106,  764,  196750,  196763,  196787, 13, 24, 19, }, // Georgia
    {  106,  772,       0,  196806,       0,  0, 24,  0, }, // Gilbert_Islands
    {  106,  788,       0,  196830,       0,  0, 24,  0, }, // GMT
    {  106,  830,  196854,  196872,  196901, 18, 29, 24, }, // Greenland_Eastern
    {  106,  848,  196925,  196942,  196970, 17, 28, 23, }, // Greenland_Western
    {  106,  866,       0,  196993,       0,  0, 11,  0, }, // Guam
    {  106,  871,       0,  197004,       0,  0, 30,  0, }, // Gulf
    {  106,  876,       0,  196663,       0,  0, 12,  0, }, // Guyana
    {  106,  883,  197034,  197055,  197087, 21, 32, 27, }, // Hawaii_Aleutian
    {  106,  899,  197114,  197129,  197155, 15, 26, 21, }, // Hong_Kong
    {  106,  909,  197176,  197187,  197209, 11, 22, 17, }, // Hovd
    {  106,  914,       0,  197226,       0,  0, 11,  0, }, // India
    {  106,  920,       0,  197237,       0,  0, 19,  0, }, // Indian_Ocean
    {  106,  933,       0,  197256,       0,  0, 14,  0, }, // Indochina
    {  106,  943,       0,  197270,       0,  0, 19,  0, }, // Indonesia_Central
    {  106,  961,       0,  197289,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  106,  979,       0,  197309,       0,  0, 19,  0, }, // Indonesia_Western
    {  106,  997,  197328,  197339,  197361, 11, 22, 17, }, // Iran
    {  106, 1008,  197378,  197390,  197413, 12, 23, 18, }, // Irkutsk
    {  106, 1016,  197431,  197444,  197468, 13, 24, 19, }, // Israel
    {  106, 1023,  197487,  197499,  197522, 12, 23, 18, }, // Japan
    {  106, 1029,  197540,  197568,  197607, 28, 39, 34, }, // Kamchatka
    {  106, 1047,       0,  197641,       0,  0, 17,  0, }, // Kazakhstan
    {  106, 1058,       0,  197658,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  106, 1077,       0,  197682,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  106, 1106,  197705,  197717,  197740, 12, 23, 18, }, // Korea
    {  106, 1112,       0,  197758,       0,  0, 13,  0, }, // Kosrae
    {  106, 1119,  197771,  197787,  197814, 16, 27, 22, }, // Krasnoyarsk
    {  106, 1151,       0,  197836,       0,  0, 13,  0, }, // Kyrgystan
    {  106, 1161,       0,  197849,       0,  0, 13,  0, }, // Lanka
    {  106, 1175,       0,  197862,       0,  0, 23,  0, }, // Line_Islands
    {  106, 1188,  197885,  197907,  197940, 22, 33, 28, }, // Lord_Howe
    {  106, 1198,  197968,  197980,  198003, 12, 23, 18, }, // Macau
    {  106, 1204,  198021,  198035,  198060, 14, 25, 20, }, // Magadan
    {  106, 1219,       0,  198080,       0,  0, 12,  0, }, // Malaysia
    {  106, 1228,       0,  198092,       0,  0, 14,  0, }, // Maldives
    {  106, 1237,       0,  198106,       0,  0, 13,  0, }, // Marquesas
    {  106, 1247,       0,  198119,       0,  0, 25,  0, }, // Marshall_Islands
    {  106, 1264,  198144,  198159,  198185, 15, 26, 21, }, // Mauritius
    {  106, 1274,       0,  198206,       0,  0, 21,  0, }, // Mawson
    {  106, 1281,  198227,  198248,  198280, 21, 32, 27, }, // Mexico_Pacific
    {  106, 1296,  198307,  198323,  198350, 16, 27, 22, }, // Mongolia
    {  106, 1305,  198372,  198385,  198409, 13, 24, 19, }, // Moscow
    {  106, 1312,       0,  198428,       0,  0, 14,  0, }, // Myanmar
    {  106, 1320,       0,  198442,       0,  0, 11,  0, }, // Nauru
    {  106, 1326,       0,  198453,       0,  0, 12,  0, }, // Nepal
    {  106, 1332,  198465,  198483,  198512, 18, 29, 24, }, // New_Caledonia
    {  106, 1346,  198536,  198553,  198581, 17, 28, 23, }, // New_Zealand
    {  106, 1358,  198604,  198623,  198653, 19, 30, 25, }, // Newfoundland
    {  106, 1371,       0,  198678,       0,  0, 12,  0, }, // Niue
    {  106, 1376,  198690,  198704,  198729, 14, 25, 20, }, // Norfolk
    {  106, 1384,  198749,  198783,  198828, 34, 45, 40, }, // Noronha
    {  106, 1392,       0,  198868,       0,  0, 19,  0, }, // North_Mariana
    {  106, 1406,  198887,  198903,  198930, 16, 27, 22, }, // Novosibirsk
    {  106, 1418,  198952,  198961,  198981,  9, 20, 15, }, // Omsk
    {  106, 1428,  198996,  199011,  199037, 15, 26, 21, }, // Pakistan
    {  106, 1437,       0,  199058,       0,  0, 12,  0, }, // Palau
    {  106, 1443,       0,  199070,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  106, 1460,  199091,  199106,  199132, 15, 26, 21, }, // Paraguay
    {  106, 1469,  199153,  199166,  199190, 13, 24, 19, }, // Peru
    {  106, 1474,  199209,  199223,  199248, 14, 25, 20, }, // Philippines
    {  106, 1486,       0,  199268,       0,  0, 22,  0, }, // Phoenix_Islands
    {  106, 1502,  199290,  199312,  199345, 22, 33, 28, }, // Pierre_Miquelon
    {  106, 1518,       0,  199373,       0,  0, 25,  0, }, // Pitcairn
    {  106, 1527,       0,  199398,       0,  0, 12,  0, }, // Ponape
    {  106, 1534,       0,  199410,       0,  0, 18,  0, }, // Pyongyang
    {  106, 1544,  199428,  199443,  199469, 15, 26, 21, }, // Qyzylorda
    {  106, 1554,       0,  199490,       0,  0, 14,  0, }, // Reunion
    {  106, 1562,       0,  199504,       0,  0, 21,  0, }, // Rothera
    {  106, 1570,  199525,  199540,  199566, 15, 26, 21, }, // Sakhalin
    {  106, 1579,  199587,  199599,  199622, 12, 23, 18, }, // Samara
    {  106, 1596,  199640,  199652,  199675, 12, 23, 18, }, // Samoa
    {  106, 1602,       0,  199693,       0,  0, 14,  0, }, // Seychelles
    {  106, 1624,       0,  199707,       0,  0, 15,  0, }, // Singapore
    {  106, 1634,       0,  199722,       0,  0, 25,  0, }, // Solomon
    {  106, 1642,       0,  199747,       0,  0, 17,  0, }, // South_Georgia
    {  106, 1656,       0,  199764,       0,  0, 14,  0, }, // Suriname
    {  106, 1676,       0,  199778,       0,  0, 16,  0, }, // Syowa
    {  106, 1682,       0,  199794,       0,  0, 12,  0, }, // Tahiti
    {  106, 1689,  199806,  199821,  199847, 15, 26, 21, }, // Taipei
    {  106, 1696,       0,  199868,       0,  0, 12,  0, }, // Tajikistan
    {  106, 1724,       0,  199880,       0,  0, 14,  0, }, // Tokelau
    {  106, 1732,  199894,  199905,  199927, 11, 22, 17, }, // Tonga
    {  106, 1738,       0,  199944,       0,  0, 12,  0, }, // Truk
    {  106, 1750,  199956,  199970,  199995, 14, 25, 20, }, // Turkmenistan
    {  106, 1763,       0,  200015,       0,  0, 12,  0, }, // Tuvalu
    {  106, 1777,  200027,  200041,  200066, 14, 25, 20, }, // Uruguay
    {  106, 1792,  200086,  200097,  200119, 11, 22, 17, }, // Uzbekistan
    {  106, 1803,  200136,  200149,  200173, 13, 24, 19, }, // Vanuatu
    {  106, 1811,       0,  200192,       0,  0, 15,  0, }, // Venezuela
    {  106, 1821,  200207,  200224,  200252, 17, 28, 23, }, // Vladivostok
    {  106, 1833,  200275,  200291,  200318, 16, 27, 22, }, // Volgograd
    {  106, 1843,       0,  200340,       0,  0, 18,  0, }, // Vostok
    {  106, 1850,       0,  200358,       0,  0, 16,  0, }, // Wake
    {  106, 1855,       0,  200374,       0,  0, 27,  0, }, // Wallis
    {  106, 1862,  200401,  200413,  200436, 12, 23, 18, }, // Yakutsk
    {  106, 1870,  200454,  200473,  200503, 19, 30, 25, }, // Yekaterinburg
    {  106, 1892,       0,  200528,       0,  0, 12,  0, }, // Yukon
    {  107,    0,  200540,  200548,  200562,  8, 14, 14, }, // Acre Danish/Latin/Denmark
    {  107,    5,       0,  200576,       0,  0, 12,  0, }, // Afghanistan
    {  107,   17,       0,  200588,       0,  0, 20,  0, }, // Africa_Central
    {  107,   32,       0,  200608,       0,  0, 16,  0, }, // Africa_Eastern
    {  107,   65,       0,  200624,       0,  0, 16,  0, }, // Africa_Southern
    {  107,   81,  200640,  200657,  200680, 17, 23, 23, }, // Africa_Western
    {  107,  107,  200703,  200713,  200729, 10, 16, 16, }, // Alaska
    {  107,  128,  200745,  200755,  200771, 10, 16, 16, }, // Almaty
    {  107,  135,  200787,  200799,  200817, 12, 18, 18, }, // Amazon
    {  107,  142,  200835,  200846,  200863, 11, 17, 17, }, // America_Central
    {  107,  158,  200880,  200891,  200908, 11, 17, 17, }, // America_Eastern
    {  107,  174,  200925,  200937,  200955, 12, 18, 18, }, // America_Mountain
    {  107,  191,  200973,  200984,  201001, 11, 17, 17, }, // America_Pacific
    {  107,  207,  201018,  201028,  201044, 10, 16, 16, }, // Anadyr
    {  107,  214,  201060,  201068,  201082,  8, 14, 14, }, // Apia
    {  107,  219,  201096,  201105,  201120,  9, 15, 15, }, // Aqtau
    {  107,  225,  201135,  201145,  201161, 10, 16, 16, }, // Aqtobe
    {  107,  232,  201177,  201188,  201205, 11, 17, 17, }, // Arabian
    {  107,  240,  201222,  201235,  201255, 13, 20, 20, }, // Argentina
    {  107,  250,  201275,  201293,  201317, 18, 24, 24, }, // Argentina_Western
    {  107,  268,  201341,  201352,  201369, 11, 17, 17, }, // Armenia
    {  107,  286,  201386,  201398,  201416, 12, 18, 18, }, // Atlantic
    {  107,  295,  201434,  201454,  201480, 20, 26, 26, }, // Australia_Central
    {  107,  313,  201506,  201534,  201568, 28, 34, 34, }, // Australia_CentralWestern
    {  107,  338,  201602,  201618,  201640, 16, 22, 22, }, // Australia_Eastern
    {  107,  356,  201662,  201679,  201702, 17, 23, 23, }, // Australia_Western
    {  107,  374,  201725,  201743,  201767, 18, 24, 24, }, // Azerbaijan
    {  107,  385,  201791,  201803,  201821, 12, 18, 18, }, // Azores
    {  107,  397,  201839,  201853,  201873, 14, 20, 20, }, // Bangladesh
    {  107,  415,       0,  201893,       0,  0, 10,  0, }, // Bhutan
    {  107,  422,       0,  201903,       0,  0, 14,  0, }, // Bolivia
    {  107,  437,  201917,  201932,  201953, 15, 21, 21, }, // Brasilia
    {  107,  454,       0,  201974,       0,  0, 21,  0, }, // Brunei
    {  107,  461,  201995,  202008,  202027, 13, 19, 19, }, // Cape_Verde
    {  107,  478,       0,  202046,       0,  0, 12,  0, }, // Chamorro
    {  107,  487,  202058,  202069,  202086, 11, 17, 17, }, // Chatham
    {  107,  495,  202103,  202115,  202133, 12, 18, 18, }, // Chile
    {  107,  501,  202151,  202163,  202181, 12, 18, 18, }, // China
    {  107,  507,       0,  202199,       0,  0, 17,  0, }, // Christmas
    {  107,  517,       0,  202216,       0,  0, 20,  0, }, // Cocos
    {  107,  523,  202236,  202251,  202272, 15, 21, 21, }, // Colombia
    {  107,  532,  202293,  202306,  202325, 13, 19, 19, }, // Cook
    {  107,  537,  202344,  202355,  202372, 11, 17, 17, }, // Cuba
    {  107,  548,       0,  202389,       0,  0,  9,  0, }, // Davis
    {  107,  564,       0,  202398,       0,  0, 20,  0, }, // DumontDUrville
    {  107,  601,       0,  202418,       0,  0, 12,  0, }, // East_Timor
    {  107,  612,  202430,  202442,  202460, 12, 18, 18, }, // Easter
    {  107,  619,       0,  202478,       0,  0, 16,  0, }, // Ecuador
    {  107,  627,  202494,  202514,  202540, 20, 26, 26, }, // Europe_Central
    {  107,  642,  202566,  202582,  202604, 16, 22, 22, }, // Europe_Eastern
    {  107,  657,       0,  202626,       0,  0, 21,  0, }, // Europe_Further_Eastern
    {  107,  680,  202647,  202664,  202687, 17, 23, 23, }, // Europe_Western
    {  107,  695,  202710,  202728,  202752, 18, 24, 24, }, // Falkland
    {  107,  704,  202776,  202788,  202806, 12, 18, 18, }, // Fiji
    {  107,  709,       0,  202824,       0,  0, 17,  0, }, // French_Guiana
    {  107,  723,       0,  202841,       0,  0, 46,  0, }, // French_Southern
    {  107,  746,       0,  202887,       0,  0, 13,  0, }, // Galapagos
    {  107,  756,       0,  202900,       0,  0, 11,  0, }, // Gambier
    {  107,  764,  202911,  202923,  202941, 12, 18, 18, }, // Georgia
    {  107,  772,       0,  202959,       0,  0, 16,  0, }, // Gilbert_Islands
    {  107,  788,       0,  202975,       0,  0,  3,  0, }, // GMT
    {  107,  830,  202978,  202995,  203018, 17, 23, 23, }, // Greenland_Eastern
    {  107,  848,  203041,  203059,  203083, 18, 24, 24, }, // Greenland_Western
    {  107,  866,       0,  203107,       0,  0, 14,  0, }, // Guam
    {  107,  871,       0,  203121,       0,  0, 21,  0, }, // Gulf
    {  107,  876,       0,  202831,       0,  0, 10,  0, }, // Guyana
    {  107,  883,  203142,  203161,  203186, 19, 25, 25, }, // Hawaii_Aleutian
    {  107,  899,  203211,  203223,  203241, 12, 18, 18, }, // Hong_Kong
    {  107,  909,  203259,  203267,  203281,  8, 14, 14, }, // Hovd
    {  107,  914,       0,  203295,       0,  0, 16,  0, }, // India
    {  107,  920,       0,  203311,       0,  0, 23,  0, }, // Indian_Ocean
    {  107,  933,       0,  203334,       0,  0, 12,  0, }, // Indochina
    {  107,  943,       0,  203346,       0,  0, 21,  0, }, // Indonesia_Central
    {  107,  961,       0,  203367,       0,  0, 17,  0, }, // Indonesia_Eastern
    {  107,  979,       0,  203384,       0,  0, 18,  0, }, // Indonesia_Western
    {  107,  997,  203402,  203412,  203428, 10, 16, 16, }, // Iran
    {  107, 1008,  203444,  203455,  203472, 11, 17, 17, }, // Irkutsk
    {  107, 1016,  203489,  203501,  203519, 12, 18, 18, }, // Israel
    {  107, 1023,  203537,  203548,  203565, 11, 17, 17, }, // Japan
    {  107, 1029,  203582,  203610,  203644, 28, 34, 34, }, // Kamchatka
    {  107, 1047,       0,  203678,       0,  0, 16,  0, }, // Kazakhstan
    {  107, 1058,       0,  203694,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  107, 1077,       0,  203713,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  107, 1106,  203733,  203745,  203763, 12, 18, 18, }, // Korea
    {  107, 1112,       0,  203781,       0,  0, 10,  0, }, // Kosrae
    {  107, 1119,  203791,  203806,  203827, 15, 21, 21, }, // Krasnoyarsk
    {  107, 1151,       0,  203848,       0,  0, 13,  0, }, // Kyrgystan
    {  107, 1161,       0,  203861,       0,  0,  9,  0, }, // Lanka
    {  107, 1175,       0,  203870,       0,  0, 14,  0, }, // Line_Islands
    {  107, 1188,  203884,  203897,  203916, 13, 19, 19, }, // Lord_Howe
    {  107, 1198,  203935,  203944,  203959,  9, 15, 15, }, // Macau
    {  107, 1204,  203974,  203985,  204002, 11, 17, 17, }, // Magadan
    {  107, 1219,       0,  204019,       0,  0, 12,  0, }, // Malaysia
    {  107, 1228,       0,  204031,       0,  0, 14,  0, }, // Maldives
    {  107, 1237,       0,  204045,       0,  0, 13,  0, }, // Marquesas
    {  107, 1247,       0,  204058,       0,  0, 17,  0, }, // Marshall_Islands
    {  107, 1264,  204075,  204088,  204107, 13, 19, 19, }, // Mauritius
    {  107, 1274,       0,  204126,       0,  0, 10,  0, }, // Mawson
    {  107, 1281,  204136,  204157,  204184, 21, 27, 27, }, // Mexico_Pacific
    {  107, 1296,  204211,  204225,  204245, 14, 20, 20, }, // Mongolia
    {  107, 1305,  204265,  204275,  204291, 10, 16, 16, }, // Moscow
    {  107, 1312,       0,  204307,       0,  0, 11,  0, }, // Myanmar
    {  107, 1320,       0,  204318,       0,  0,  9,  0, }, // Nauru
    {  107, 1326,       0,  204327,       0,  0, 14,  0, }, // Nepal
    {  107, 1332,  204341,  204358,  204381, 17, 23, 23, }, // New_Caledonia
    {  107, 1346,  204404,  204420,  204442, 16, 22, 22, }, // New_Zealand
    {  107, 1358,  204464,  204482,  204506, 18, 24, 24, }, // Newfoundland
    {  107, 1371,       0,  204530,       0,  0,  8,  0, }, // Niue
    {  107, 1376,  204538,  204556,  204580, 18, 24, 24, }, // Norfolk
    {  107, 1384,  204604,  204627,  204656, 23, 29, 29, }, // Noronha
    {  107, 1392,       0,  204685,       0,  0, 18,  0, }, // North_Mariana
    {  107, 1406,  204703,  204718,  204739, 15, 21, 21, }, // Novosibirsk
    {  107, 1418,  204760,  204768,  204782,  8, 14, 14, }, // Omsk
    {  107, 1428,  204796,  204810,  204830, 14, 20, 20, }, // Pakistan
    {  107, 1437,       0,  204850,       0,  0,  9,  0, }, // Palau
    {  107, 1443,       0,  204859,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  107, 1460,  204878,  204894,  204916, 16, 22, 22, }, // Paraguay
    {  107, 1469,  204938,  204952,  204972, 14, 20, 20, }, // Peru
    {  107, 1474,  204992,  205006,  205026, 14, 20, 20, }, // Philippines
    {  107, 1486,       0,  205046,       0,  0, 14,  0, }, // Phoenix_Islands
    {  107, 1502,  205060,  205089,  205124, 29, 35, 35, }, // Pierre_Miquelon
    {  107, 1518,       0,  205159,       0,  0, 12,  0, }, // Pitcairn
    {  107, 1527,       0,  205171,       0,  0, 10,  0, }, // Ponape
    {  107, 1534,       0,  205181,       0,  0, 13,  0, }, // Pyongyang
    {  107, 1544,  205194,  205207,  205226, 13, 19, 19, }, // Qyzylorda
    {  107, 1554,       0,  205245,       0,  0, 11,  0, }, // Reunion
    {  107, 1562,       0,  205256,       0,  0, 11,  0, }, // Rothera
    {  107, 1570,  205267,  205279,  205297, 12, 18, 18, }, // Sakhalin
    {  107, 1579,  205315,  205325,  205341, 10, 16, 16, }, // Samara
    {  107, 1596,  205357,  205369,  205387, 12, 18, 18, }, // Samoa
    {  107, 1602,       0,  205405,       0,  0, 15,  0, }, // Seychelles
    {  107, 1624,       0,  205420,       0,  0, 13,  0, }, // Singapore
    {  107, 1634,       0,  205433,       0,  0, 16,  0, }, // Solomon
    {  107, 1642,       0,  205449,       0,  0, 17,  0, }, // South_Georgia
    {  107, 1656,       0,  205466,       0,  0, 11,  0, }, // Suriname
    {  107, 1676,       0,  205477,       0,  0,  9,  0, }, // Syowa
    {  107, 1682,       0,  205486,       0,  0, 10,  0, }, // Tahiti
    {  107, 1689,  205496,  205506,  205522, 10, 16, 16, }, // Taipei
    {  107, 1696,       0,  205538,       0,  0, 14,  0, }, // Tajikistan
    {  107, 1724,       0,  205552,       0,  0, 11,  0, }, // Tokelau
    {  107, 1732,  205563,  205575,  205593, 12, 18, 18, }, // Tonga
    {  107, 1738,       0,  205611,       0,  0,  9,  0, }, // Truk
    {  107, 1750,  205620,  205633,  205652, 13, 19, 19, }, // Turkmenistan
    {  107, 1763,       0,  205671,       0,  0, 10,  0, }, // Tuvalu
    {  107, 1777,  205681,  205696,  205717, 15, 21, 21, }, // Uruguay
    {  107, 1792,  205738,  205750,  205768, 12, 18, 18, }, // Uzbekistan
    {  107, 1803,  205786,  205797,  205814, 11, 17, 17, }, // Vanuatu
    {  107, 1811,       0,  205831,       0,  0, 16,  0, }, // Venezuela
    {  107, 1821,  205847,  205862,  205883, 15, 21, 21, }, // Vladivostok
    {  107, 1833,  205904,  205917,  205936, 13, 19, 19, }, // Volgograd
    {  107, 1843,       0,  205955,       0,  0, 10,  0, }, // Vostok
    {  107, 1850,       0,  205965,       0,  0, 15,  0, }, // Wake
    {  107, 1855,       0,  205980,       0,  0, 20,  0, }, // Wallis
    {  107, 1862,  206000,  206011,  206028, 11, 17, 17, }, // Yakutsk
    {  107, 1870,  206045,  206062,  206085, 17, 23, 23, }, // Yekaterinburg
    {  107, 1892,       0,  206108,       0,  0,  9,  0, }, // Yukon
    {  110,  142,  206117,  206142,  206172, 25, 30, 32, }, // America_Central Dogri/Devanagari/India
    {  110,  158,  206204,  206229,  206259, 25, 30, 32, }, // America_Eastern
    {  110,  174,  206291,  206317,  206348, 26, 31, 33, }, // America_Mountain
    {  110,  191,  206381,  206407,  206438, 26, 31, 33, }, // America_Pacific
    {  110,  286,  206471,  206484,  206502, 13, 18, 20, }, // Atlantic
    {  110,  627,  206522,  206540,  206563, 18, 23, 30, }, // Europe_Central
    {  110,  642,  206593,  206611,  206634, 18, 23, 30, }, // Europe_Eastern
    {  110,  680,  206664,  206682,  206705, 18, 23, 30, }, // Europe_Western
    {  110,  788,       0,   91136,       0,  0, 17,  0, }, // GMT
    {  112,    0,  206735,  206744,  206762,  9, 18, 14, }, // Acre Dutch/Latin/Netherlands
    {  112,    5,       0,  206776,       0,  0, 14,  0, }, // Afghanistan
    {  112,   17,       0,  206790,       0,  0, 24,  0, }, // Africa_Central
    {  112,   32,       0,  206814,       0,  0, 20,  0, }, // Africa_Eastern
    {  112,   65,       0,  206834,       0,  0, 20,  0, }, // Africa_Southern
    {  112,   81,  206854,  206874,  206903, 20, 29, 25, }, // Africa_Western
    {  112,  107,  206928,  206939,  206959, 11, 20, 16, }, // Alaska
    {  112,  128,  206975,  206988,  207010, 13, 22, 18, }, // Almaty
    {  112,  135,  207028,  207040,  207061, 12, 21, 17, }, // Amazon
    {  112,  142,  207078,  207090,  207111, 12, 21, 17, }, // America_Central
    {  112,  158,  207128,  207140,  207161, 12, 21, 17, }, // America_Eastern
    {  112,  174,  207178,  207191,  207213, 13, 22, 18, }, // America_Mountain
    {  112,  191,  207231,  207243,  207264, 12, 21, 17, }, // America_Pacific
    {  112,  207,  207281,  207292,  207312, 11, 20, 16, }, // Anadyr
    {  112,  214,  207328,  207337,  207355,  9, 18, 14, }, // Apia
    {  112,  219,  207369,  207379,  207398, 10, 19, 15, }, // Aqtau
    {  112,  225,  207413,  207424,  207444, 11, 20, 16, }, // Aqtobe
    {  112,  232,  207460,  207474,  207497, 14, 23, 19, }, // Arabian
    {  112,  240,  207516,  207532,  207557, 16, 25, 21, }, // Argentina
    {  112,  250,  207578,  207599,  207629, 21, 30, 26, }, // Argentina_Western
    {  112,  268,  207655,  207668,  207690, 13, 22, 18, }, // Armenia
    {  112,  286,  207708,  207721,  207743, 13, 22, 18, }, // Atlantic
    {  112,  295,  207761,  207785,  207818, 24, 33, 29, }, // Australia_Central
    {  112,  313,  207847,  207882,  207926, 35, 44, 40, }, // Australia_CentralWestern
    {  112,  338,  207966,  207988,  208019, 22, 31, 27, }, // Australia_Eastern
    {  112,  356,  208046,  208068,  208099, 22, 31, 27, }, // Australia_Western
    {  112,  374,  208126,  208146,  208175, 20, 29, 25, }, // Azerbaijan
    {  112,  385,  208200,  208211,  208231, 11, 20, 16, }, // Azores
    {  112,  397,  208247,  208261,  208284, 14, 23, 19, }, // Bangladesh
    {  112,  415,       0,  208303,       0,  0, 14,  0, }, // Bhutan
    {  112,  422,       0,  208317,       0,  0, 16,  0, }, // Bolivia
    {  112,  437,  208333,  208350,  208376, 17, 26, 22, }, // Brasilia
    {  112,  454,       0,  208398,       0,  0, 13,  0, }, // Brunei
    {  112,  461,  208411,  208429,  208456, 18, 27, 23, }, // Cape_Verde
    {  112,  472,       0,  208479,       0,  0, 10,  0, }, // Casey
    {  112,  478,       0,  208489,       0,  0, 13,  0, }, // Chamorro
    {  112,  487,  208502,  208514,  208535, 12, 21, 17, }, // Chatham
    {  112,  495,  208552,  208566,  208589, 14, 23, 19, }, // Chile
    {  112,  501,  208608,  208620,  208641, 12, 21, 17, }, // China
    {  112,  507,       0,  208658,       0,  0, 22,  0, }, // Christmas
    {  112,  517,       0,  208680,       0,  0, 18,  0, }, // Cocos
    {  112,  523,  208698,  208715,  208741, 17, 26, 22, }, // Colombia
    {  112,  532,  208763,  208780,  208806, 17, 26, 28, }, // Cook
    {  112,  537,  208834,  208847,  208869, 13, 22, 18, }, // Cuba
    {  112,  548,       0,  208887,       0,  0, 10,  0, }, // Davis
    {  112,  564,       0,  208897,       0,  0, 21,  0, }, // DumontDUrville
    {  112,  601,       0,  208918,       0,  0, 18,  0, }, // East_Timor
    {  112,  612,  208936,  208953,  208979, 17, 26, 22, }, // Easter
    {  112,  619,       0,  209001,       0,  0, 17,  0, }, // Ecuador
    {  112,  627,  209018,  209038,  209067, 20, 29, 25, }, // Europe_Central
    {  112,  642,  209092,  209110,  209137, 18, 27, 23, }, // Europe_Eastern
    {  112,  657,       0,  209160,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  112,  680,  209190,  209208,  209235, 18, 27, 23, }, // Europe_Western
    {  112,  695,  209258,  209279,  209309, 21, 30, 26, }, // Falkland
    {  112,  704,  209335,  209348,  209370, 13, 22, 18, }, // Fiji
    {  112,  709,       0,  209388,       0,  0, 19,  0, }, // French_Guiana
    {  112,  723,       0,  209407,       0,  0, 38,  0, }, // French_Southern
    {  112,  746,       0,  209445,       0,  0, 22,  0, }, // Galapagos
    {  112,  756,       0,  209467,       0,  0, 20,  0, }, // Gambier
    {  112,  764,  209487,  209502,  209526, 15, 24, 20, }, // Georgia
    {  112,  772,       0,  209546,       0,  0, 20,  0, }, // Gilbert_Islands
    {  112,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {  112,  830,  209566,  209587,  209617, 21, 30, 26, }, // Greenland_Eastern
    {  112,  848,  209643,  209664,  209694, 21, 30, 26, }, // Greenland_Western
    {  112,  866,       0,  209720,       0,  0, 21,  0, }, // Guam
    {  112,  871,       0,  209741,       0,  0, 18,  0, }, // Gulf
    {  112,  876,       0,  209394,       0,  0, 13,  0, }, // Guyana
    {  112,  883,  209759,  209782,  209814, 23, 32, 28, }, // Hawaii_Aleutian
    {  112,  899,  209842,  209857,  209881, 15, 24, 20, }, // Hong_Kong
    {  112,  909,  209901,  209910,  209928,  9, 18, 14, }, // Hovd
    {  112,  914,       0,  209942,       0,  0, 12,  0, }, // India
    {  112,  920,       0,  209954,       0,  0, 20,  0, }, // Indian_Ocean
    {  112,  933,       0,  209974,       0,  0, 16,  0, }, // Indochina
    {  112,  943,       0,  209990,       0,  0, 26,  0, }, // Indonesia_Central
    {  112,  961,       0,  210016,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  112,  979,       0,  210038,       0,  0, 22,  0, }, // Indonesia_Western
    {  112,  997,  210060,  210072,  210093, 12, 21, 17, }, // Iran
    {  112, 1008,  210110,  210123,  210145, 13, 22, 18, }, // Irkutsk
    {  112, 1016,  210163,  210179,  210204, 16, 25, 21, }, // Israel
    {  112, 1023,  210225,  210237,  210258, 12, 21, 17, }, // Japan
    {  112, 1029,  210275,  210305,  210344, 30, 39, 35, }, // Kamchatka
    {  112, 1047,       0,  210379,       0,  0, 13,  0, }, // Kazakhstan
    {  112, 1058,       0,  210392,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  112, 1077,       0,  210410,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  112, 1106,  210428,  210442,  210465, 14, 23, 19, }, // Korea
    {  112, 1112,       0,  210484,       0,  0, 13,  0, }, // Kosrae
    {  112, 1119,  210497,  210513,  210538, 16, 25, 21, }, // Krasnoyarsk
    {  112, 1151,       0,  210559,       0,  0, 16,  0, }, // Kyrgystan
    {  112, 1161,       0,  210575,       0,  0, 10,  0, }, // Lanka
    {  112, 1175,       0,  210585,       0,  0, 18,  0, }, // Line_Islands
    {  112, 1188,  210603,  210626,  210658, 23, 32, 28, }, // Lord_Howe
    {  112, 1198,  210686,  210698,  210719, 12, 21, 17, }, // Macau
    {  112, 1204,  210736,  210748,  210769, 12, 21, 17, }, // Magadan
    {  112, 1219,       0,  210786,       0,  0, 16,  0, }, // Malaysia
    {  112, 1228,       0,  210802,       0,  0, 16,  0, }, // Maldives
    {  112, 1237,       0,  210818,       0,  0, 22,  0, }, // Marquesas
    {  112, 1247,       0,  210840,       0,  0, 21,  0, }, // Marshall_Islands
    {  112, 1264,  210861,  210878,  210904, 17, 26, 22, }, // Mauritius
    {  112, 1274,       0,  210926,       0,  0, 11,  0, }, // Mawson
    {  112, 1281,  210937,  210960,  210992, 23, 32, 28, }, // Mexico_Pacific
    {  112, 1296,  211020,  211036,  211061, 16, 25, 21, }, // Mongolia
    {  112, 1305,  211082,  211093,  211113, 11, 20, 16, }, // Moscow
    {  112, 1312,       0,  211129,       0,  0, 15,  0, }, // Myanmar
    {  112, 1320,       0,  211144,       0,  0, 15,  0, }, // Nauru
    {  112, 1326,       0,  211159,       0,  0, 13,  0, }, // Nepal
    {  112, 1332,  211172,  211195,  211227, 23, 32, 28, }, // New_Caledonia
    {  112, 1346,  211255,  211275,  211304, 20, 29, 25, }, // New_Zealand
    {  112, 1358,  211329,  211346,  211372, 17, 26, 22, }, // Newfoundland
    {  112, 1371,       0,  211394,       0,  0, 11,  0, }, // Niue
    {  112, 1376,  211405,  211425,  211454, 20, 29, 25, }, // Norfolk
    {  112, 1384,  211479,  211503,  211536, 24, 33, 29, }, // Noronha
    {  112, 1392,       0,  211565,       0,  0, 25,  0, }, // North_Mariana
    {  112, 1406,  211590,  211606,  211631, 16, 25, 21, }, // Novosibirsk
    {  112, 1418,  211652,  211661,  211679,  9, 18, 14, }, // Omsk
    {  112, 1428,  211693,  211709,  211734, 16, 25, 21, }, // Pakistan
    {  112, 1437,       0,  211755,       0,  0, 12,  0, }, // Palau
    {  112, 1443,       0,  211767,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  112, 1460,  211793,  211811,  211838, 18, 27, 23, }, // Paraguay
    {  112, 1469,  211861,  211875,  211898, 14, 23, 19, }, // Peru
    {  112, 1474,  211917,  211932,  211956, 15, 24, 20, }, // Philippines
    {  112, 1486,       0,  211976,       0,  0, 20,  0, }, // Phoenix_Islands
    {  112, 1502,  211996,  212025,  212063, 29, 38, 34, }, // Pierre_Miquelon
    {  112, 1518,       0,  212097,       0,  0, 21,  0, }, // Pitcairn
    {  112, 1527,       0,  212118,       0,  0, 12,  0, }, // Ponape
    {  112, 1534,       0,  212130,       0,  0, 14,  0, }, // Pyongyang
    {  112, 1544,  212144,  212158,  212181, 14, 23, 19, }, // Qyzylorda
    {  112, 1554,       0,  212200,       0,  0, 14,  0, }, // Reunion
    {  112, 1562,       0,  212214,       0,  0, 12,  0, }, // Rothera
    {  112, 1570,  212226,  212239,  212261, 13, 22, 18, }, // Sakhalin
    {  112, 1579,  212279,  212290,  212310, 11, 20, 16, }, // Samara
    {  112, 1596,  212326,  212340,  212363, 14, 23, 19, }, // Samoa
    {  112, 1602,       0,  212382,       0,  0, 14,  0, }, // Seychelles
    {  112, 1624,       0,  212396,       0,  0, 25,  0, }, // Singapore
    {  112, 1634,       0,  212421,       0,  0, 21,  0, }, // Solomon
    {  112, 1642,       0,  212442,       0,  0, 20,  0, }, // South_Georgia
    {  112, 1656,       0,  212462,       0,  0, 15,  0, }, // Suriname
    {  112, 1676,       0,  212477,       0,  0, 10,  0, }, // Syowa
    {  112, 1682,       0,  212487,       0,  0, 16,  0, }, // Tahiti
    {  112, 1689,  212503,  212514,  212534, 11, 20, 16, }, // Taipei
    {  112, 1696,       0,  212550,       0,  0, 15,  0, }, // Tajikistan
    {  112, 1724,       0,  212565,       0,  0, 21,  0, }, // Tokelau
    {  112, 1732,  212586,  212600,  212623, 14, 23, 19, }, // Tonga
    {  112, 1738,       0,  212642,       0,  0, 12,  0, }, // Truk
    {  112, 1750,  212654,  212669,  212693, 15, 24, 20, }, // Turkmenistan
    {  112, 1763,       0,  212713,       0,  0, 16,  0, }, // Tuvalu
    {  112, 1777,  212729,  212746,  212772, 17, 26, 22, }, // Uruguay
    {  112, 1792,  212794,  212808,  212831, 14, 23, 19, }, // Uzbekistan
    {  112, 1803,  212850,  212867,  212893, 17, 26, 22, }, // Vanuatu
    {  112, 1811,       0,  212915,       0,  0, 17,  0, }, // Venezuela
    {  112, 1821,  212932,  212948,  212973, 16, 25, 21, }, // Vladivostok
    {  112, 1833,  212994,  213008,  213031, 14, 23, 19, }, // Volgograd
    {  112, 1843,       0,  213050,       0,  0, 11,  0, }, // Vostok
    {  112, 1850,       0,  213061,       0,  0, 18,  0, }, // Wake
    {  112, 1855,       0,  213079,       0,  0, 23,  0, }, // Wallis
    {  112, 1862,  213102,  213115,  213137, 13, 22, 18, }, // Yakutsk
    {  112, 1870,  213155,  213175,  213204, 20, 29, 25, }, // Yekaterinburg
    {  112, 1892,       0,  213229,       0,  0, 10,  0, }, // Yukon
    {  118, 1656,       0,  212462,       0,  0, 15,  0, }, // Suriname Dutch/Latin/Suriname
    {  119,    5,       0,  213239,       0,  0, 19,  0, }, // Afghanistan Dzongkha/Tibetan/Bhutan
    {  119,   17,       0,  213258,       0,  0, 26,  0, }, // Africa_Central
    {  119,   32,       0,  213284,       0,  0, 24,  0, }, // Africa_Eastern
    {  119,   65,       0,  213308,       0,  0, 25,  0, }, // Africa_Southern
    {  119,   81,  213333,  213358,  213390, 25, 32, 34, }, // Africa_Western
    {  119,  107,  213424,  213438,  213459, 14, 21, 23, }, // Alaska
    {  119,  135,  213482,  213498,  213521, 16, 23, 25, }, // Amazon
    {  119,  142,  213546,  213577,  213615, 31, 38, 40, }, // America_Central
    {  119,  158,  213655,  213684,  213720, 29, 36, 38, }, // America_Eastern
    {  119,  174,  213758,  213786,  213821, 28, 35, 37, }, // America_Mountain
    {  119,  191,  213858,  213888,  213925, 30, 37, 39, }, // America_Pacific
    {  119,  232,  213964,  213982,  214007, 18, 25, 23, }, // Arabian
    {  119,  240,  214030,  214048,  214073, 18, 25, 27, }, // Argentina
    {  119,  250,  214100,  214128,  214163, 28, 35, 37, }, // Argentina_Western
    {  119,  268,  214200,  214217,  214241, 17, 24, 26, }, // Armenia
    {  119,  286,  214267,  214285,  214310, 18, 25, 27, }, // Atlantic
    {  119,  295,  214337,  214369,  214408, 32, 39, 41, }, // Australia_Central
    {  119,  313,  214449,  214484,  214527, 35, 43, 45, }, // Australia_CentralWestern
    {  119,  338,  214572,  214608,  214651, 36, 43, 45, }, // Australia_Eastern
    {  119,  356,  214453,  214489,  214532, 31, 38, 40, }, // Australia_Western
    {  119,  374,  214696,  214716,  214743, 20, 27, 29, }, // Azerbaijan
    {  119,  385,  214772,  214786,  214807, 14, 21, 23, }, // Azores
    {  119,  397,  214830,  214846,  214869, 16, 23, 25, }, // Bangladesh
    {  119,  415,       0,  214894,       0,  0, 16,  0, }, // Bhutan
    {  119,  422,       0,  214910,       0,  0, 18,  0, }, // Bolivia
    {  119,  437,  214928,  214945,  214969, 17, 24, 26, }, // Brasilia
    {  119,  461,  214995,  215009,  215030, 14, 21, 23, }, // Cape_Verde
    {  119,  495,  215053,  215065,  215084, 12, 19, 21, }, // Chile
    {  119,  501,  215105,  215118,  215138, 13, 20, 22, }, // China
    {  119,  507,       0,  215160,       0,  0, 27,  0, }, // Christmas
    {  119,  523,  215187,  215206,  215232, 19, 26, 28, }, // Colombia
    {  119,  537,  215260,  215274,  215295, 14, 21, 23, }, // Cuba
    {  119,  612,  215318,  215342,  215373, 24, 31, 33, }, // Easter
    {  119,  619,       0,  215406,       0,  0, 15,  0, }, // Ecuador
    {  119,  627,  215421,  215448,  215482, 27, 34, 36, }, // Europe_Central
    {  119,  642,  215518,  215543,  215575, 25, 32, 34, }, // Europe_Eastern
    {  119,  680,  215609,  215635,  215668, 26, 33, 35, }, // Europe_Western
    {  119,  695,  215703,  215729,  215762, 26, 33, 35, }, // Falkland
    {  119,  709,       0,  215797,       0,  0, 20,  0, }, // French_Guiana
    {  119,  746,       0,  215817,       0,  0, 17,  0, }, // Galapagos
    {  119,  764,  215834,  215846,  215865, 12, 19, 21, }, // Georgia
    {  119,  788,       0,  215886,       0,  0, 27,  0, }, // GMT
    {  119,  830,  215913,  215939,  215972, 26, 33, 35, }, // Greenland_Eastern
    {  119,  848,  216007,  216034,  216068, 27, 34, 36, }, // Greenland_Western
    {  119,  871,       0,  216104,       0,  0, 12,  0, }, // Gulf
    {  119,  876,       0,  216116,       0,  0, 13,  0, }, // Guyana
    {  119,  883,  216129,  216154,  216186, 25, 32, 34, }, // Hawaii_Aleutian
    {  119,  914,       0,  216220,       0,  0, 13,  0, }, // India
    {  119,  920,       0,  216233,       0,  0, 25,  0, }, // Indian_Ocean
    {  119,  933,       0,  216258,       0,  0, 20,  0, }, // Indochina
    {  119,  943,       0,  216278,       0,  0, 32,  0, }, // Indonesia_Central
    {  119,  961,       0,  216310,       0,  0, 30,  0, }, // Indonesia_Eastern
    {  119,  979,       0,  216340,       0,  0, 31,  0, }, // Indonesia_Western
    {  119,  997,  216371,  216384,  216404, 13, 20, 22, }, // Iran
    {  119, 1008,  216426,  216440,  216461, 14, 21, 23, }, // Irkutsk
    {  119, 1016,  216484,  216498,  216519, 14, 21, 23, }, // Israel
    {  119, 1023,  216542,  216554,  216573, 12, 19, 21, }, // Japan
    {  119, 1106,  216594,  216608,  216629, 14, 21, 23, }, // Korea
    {  119, 1119,  216652,  216672,  216699, 20, 27, 29, }, // Krasnoyarsk
    {  119, 1204,  216728,  216743,  216765, 15, 22, 24, }, // Magadan
    {  119, 1228,       0,  216789,       0,  0, 15,  0, }, // Maldives
    {  119, 1264,  216804,  216820,  216843, 16, 23, 25, }, // Mauritius
    {  119, 1305,  216868,  216881,  216901, 13, 20, 22, }, // Moscow
    {  119, 1326,       0,  216923,       0,  0, 13,  0, }, // Nepal
    {  119, 1346,  216936,  216955,  216981, 19, 26, 28, }, // New_Zealand
    {  119, 1358,  217009,  217031,  217060, 22, 29, 31, }, // Newfoundland
    {  119, 1384,  217091,  217122,  217160, 31, 38, 40, }, // Noronha
    {  119, 1406,  217200,  217222,  217251, 22, 29, 31, }, // Novosibirsk
    {  119, 1418,  217282,  217295,  217315, 13, 20, 22, }, // Omsk
    {  119, 1428,  217337,  217354,  217378, 17, 24, 26, }, // Pakistan
    {  119, 1460,  217404,  217421,  217445, 17, 24, 26, }, // Paraguay
    {  119, 1469,  217471,  217482,  217500, 11, 18, 20, }, // Peru
    {  119, 1502,  217520,  217547,  217581, 27, 34, 36, }, // Pierre_Miquelon
    {  119, 1554,       0,  217617,       0,  0, 19,  0, }, // Reunion
    {  119, 1570,  217636,  217650,  217671, 14, 21, 23, }, // Sakhalin
    {  119, 1602,       0,  217694,       0,  0, 14,  0, }, // Seychelles
    {  119, 1656,       0,  217708,       0,  0, 16,  0, }, // Suriname
    {  119, 1777,  217724,  217744,  217771, 20, 27, 29, }, // Uruguay
    {  119, 1811,       0,  217800,       0,  0, 20,  0, }, // Venezuela
    {  119, 1821,  217820,  217842,  217871, 22, 29, 31, }, // Vladivostok
    {  119, 1833,  217902,  217922,  217949, 20, 27, 29, }, // Volgograd
    {  119, 1862,  217978,  217993,  218015, 15, 22, 24, }, // Yakutsk
    {  119, 1870,  218039,  218061,  218090, 22, 29, 31, }, // Yekaterinburg
    {  121,    0,       0,       9,      27,  9, 18, 16, }, // Acre English/Latin/United States
    {  121,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan
    {  121,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {  121,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  121,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  121,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  121,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  121,  128,     235,     246,     266, 11, 20, 18, }, // Almaty
    {  121,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {  121,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  121,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  121,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  121,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  121,  207,     552,     563,     583, 11, 20, 18, }, // Anadyr
    {  121,  214,     601,     610,     628,  9, 18, 18, }, // Apia
    {  121,  219,     646,     656,     675, 10, 19, 17, }, // Aqtau
    {  121,  225,     692,     703,     723, 11, 20, 18, }, // Aqtobe
    {  121,  232,     741,     753,     774, 12, 21, 21, }, // Arabian
    {  121,  240,     795,     809,     832, 14, 23, 21, }, // Argentina
    {  121,  250,     853,     875,     906, 22, 31, 29, }, // Argentina_Western
    {  121,  268,     935,     947,     968, 12, 21, 19, }, // Armenia
    {  121,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  121,  295,    1044,    1066,    1098, 22, 32, 32, }, // Australia_Central
    {  121,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  121,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {  121,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {  121,  374,    1413,    1428,    1452, 15, 24, 22, }, // Azerbaijan
    {  121,  385,    1474,    1485,    1505, 11, 20, 18, }, // Azores
    {  121,  397,    1523,    1538,    1562, 15, 24, 22, }, // Bangladesh
    {  121,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {  121,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia
    {  121,  437,    1607,    1620,    1642, 13, 22, 20, }, // Brasilia
    {  121,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {  121,  461,    1684,    1699,    1723, 15, 24, 22, }, // Cape_Verde
    {  121,  472,       0,    1745,       0,  0, 10,  0, }, // Casey
    {  121,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro
    {  121,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {  121,  495,    1831,    1841,    1860, 10, 19, 17, }, // Chile
    {  121,  501,    1877,    1887,    1906, 10, 19, 19, }, // China
    {  121,  507,       0,    1925,       0,  0, 21,  0, }, // Christmas
    {  121,  517,       0,    1946,       0,  0, 18,  0, }, // Cocos
    {  121,  523,    1964,    1977,    1999, 13, 22, 20, }, // Colombia
    {  121,  532,    2019,    2036,    2062, 17, 26, 29, }, // Cook
    {  121,  537,    2091,    2100,    2118,  9, 18, 18, }, // Cuba
    {  121,  548,       0,    2136,       0,  0, 10,  0, }, // Davis
    {  121,  564,       0,    2146,       0,  0, 21,  0, }, // DumontDUrville
    {  121,  601,       0,    2167,       0,  0, 15,  0, }, // East_Timor
    {  121,  612,    2182,    2200,    2227, 18, 27, 25, }, // Easter
    {  121,  619,       0,    2252,       0,  0, 12,  0, }, // Ecuador
    {  121,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  121,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  121,  657,       0,    2422,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  121,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  121,  695,    2530,    2551,    2581, 21, 30, 28, }, // Falkland
    {  121,  704,    2609,    2618,    2636,  9, 18, 16, }, // Fiji
    {  121,  709,       0,    2652,       0,  0, 18,  0, }, // French_Guiana
    {  121,  723,       0,    2670,       0,  0, 32,  0, }, // French_Southern
    {  121,  746,       0,    2702,       0,  0, 14,  0, }, // Galapagos
    {  121,  756,       0,    2716,       0,  0, 12,  0, }, // Gambier
    {  121,  764,    2728,    2740,    2761, 12, 21, 19, }, // Georgia
    {  121,  772,       0,    2780,       0,  0, 20,  0, }, // Gilbert_Islands
    {  121,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {  121,  802,    2819,    2833,    2856, 14, 23, 21, }, // Greenland
    {  121,  830,    2877,    2896,    2924, 19, 28, 26, }, // Greenland_Eastern
    {  121,  848,    2950,    2969,    2997, 19, 28, 26, }, // Greenland_Western
    {  121,  866,       0,    3023,       0,  0, 18,  0, }, // Guam
    {  121,  871,       0,    3041,       0,  0, 18,  0, }, // Gulf
    {  121,  876,       0,    3059,       0,  0, 11,  0, }, // Guyana
    {  121,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  121,  899,    3148,    3162,    3185, 14, 23, 21, }, // Hong_Kong
    {  121,  909,    3206,    3215,    3233,  9, 18, 16, }, // Hovd
    {  121,  914,       0,    3249,       0,  0, 19,  0, }, // India
    {  121,  920,       0,    3268,       0,  0, 17,  0, }, // Indian_Ocean
    {  121,  933,       0,    3285,       0,  0, 14,  0, }, // Indochina
    {  121,  943,       0,    3299,       0,  0, 22,  0, }, // Indonesia_Central
    {  121,  961,       0,    3321,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  121,  979,       0,    3343,       0,  0, 22,  0, }, // Indonesia_Western
    {  121,  997,    3365,    3374,    3392,  9, 18, 18, }, // Iran
    {  121, 1008,    3410,    3422,    3443, 12, 21, 19, }, // Irkutsk
    {  121, 1016,    3462,    3473,    3493, 11, 20, 20, }, // Israel
    {  121, 1023,    3513,    3523,    3542, 10, 19, 19, }, // Japan
    {  121, 1029,    3561,    3590,    3628, 29, 38, 36, }, // Kamchatka
    {  121, 1047,       0,    3664,       0,  0, 15,  0, }, // Kazakhstan
    {  121, 1058,       0,    3679,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  121, 1077,       0,    3699,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  121, 1106,    3719,    3730,    3750, 11, 20, 20, }, // Korea
    {  121, 1112,       0,    3770,       0,  0, 11,  0, }, // Kosrae
    {  121, 1119,    3781,    3797,    3822, 16, 25, 23, }, // Krasnoyarsk
    {  121, 1151,       0,    3845,       0,  0, 15,  0, }, // Kyrgystan
    {  121, 1161,       0,    3860,       0,  0, 10,  0, }, // Lanka
    {  121, 1175,       0,    3870,       0,  0, 17,  0, }, // Line_Islands
    {  121, 1188,    3887,    3901,    3924, 14, 23, 23, }, // Lord_Howe
    {  121, 1198,    3947,    3957,    3976, 10, 19, 17, }, // Macau
    {  121, 1204,    3993,    4005,    4026, 12, 21, 19, }, // Magadan
    {  121, 1219,       0,    4045,       0,  0, 13,  0, }, // Malaysia
    {  121, 1228,       0,    4058,       0,  0, 13,  0, }, // Maldives
    {  121, 1237,       0,    4071,       0,  0, 14,  0, }, // Marquesas
    {  121, 1247,       0,    4085,       0,  0, 21,  0, }, // Marshall_Islands
    {  121, 1264,    4106,    4120,    4143, 14, 23, 21, }, // Mauritius
    {  121, 1274,       0,    4164,       0,  0, 11,  0, }, // Mawson
    {  121, 1281,    4175,    4195,    4224, 20, 29, 29, }, // Mexico_Pacific
    {  121, 1296,    4253,    4269,    4294, 16, 25, 23, }, // Mongolia
    {  121, 1305,    4317,    4328,    4348, 11, 20, 18, }, // Moscow
    {  121, 1312,       0,    4366,       0,  0, 12,  0, }, // Myanmar
    {  121, 1320,       0,    4378,       0,  0, 10,  0, }, // Nauru
    {  121, 1326,       0,    4388,       0,  0, 10,  0, }, // Nepal
    {  121, 1332,    4398,    4416,    4443, 18, 27, 25, }, // New_Caledonia
    {  121, 1346,    4468,    4484,    4509, 16, 25, 25, }, // New_Zealand
    {  121, 1358,    4534,    4551,    4577, 17, 26, 26, }, // Newfoundland
    {  121, 1371,       0,    4603,       0,  0,  9,  0, }, // Niue
    {  121, 1376,    4612,    4631,    4659, 19, 28, 28, }, // Norfolk
    {  121, 1384,    4687,    4711,    4744, 24, 33, 31, }, // Noronha
    {  121, 1392,       0,    4775,       0,  0, 26,  0, }, // North_Mariana
    {  121, 1406,    4801,    4817,    4842, 16, 25, 23, }, // Novosibirsk
    {  121, 1418,    4865,    4874,    4892,  9, 18, 16, }, // Omsk
    {  121, 1428,    4908,    4921,    4943, 13, 22, 20, }, // Pakistan
    {  121, 1437,       0,    4963,       0,  0, 10,  0, }, // Palau
    {  121, 1443,       0,    4973,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  121, 1460,    4994,    5007,    5029, 13, 22, 20, }, // Paraguay
    {  121, 1469,    5049,    5058,    5076,  9, 18, 16, }, // Peru
    {  121, 1474,    5092,    5107,    5131, 15, 24, 22, }, // Philippines
    {  121, 1486,       0,    5153,       0,  0, 20,  0, }, // Phoenix_Islands
    {  121, 1502,    5173,    5199,    5234, 26, 35, 35, }, // Pierre_Miquelon
    {  121, 1518,       0,    5269,       0,  0, 13,  0, }, // Pitcairn
    {  121, 1527,       0,    5282,       0,  0, 11,  0, }, // Ponape
    {  121, 1534,       0,    5293,       0,  0, 14,  0, }, // Pyongyang
    {  121, 1544,    5307,    5321,    5344, 14, 23, 21, }, // Qyzylorda
    {  121, 1554,       0,    5365,       0,  0, 12,  0, }, // Reunion
    {  121, 1562,       0,    5377,       0,  0, 12,  0, }, // Rothera
    {  121, 1570,    5389,    5402,    5424, 13, 22, 20, }, // Sakhalin
    {  121, 1579,    5444,    5455,    5475, 11, 20, 18, }, // Samara
    {  121, 1596,    5493,    5503,    5522, 10, 19, 19, }, // Samoa
    {  121, 1602,       0,    5541,       0,  0, 15,  0, }, // Seychelles
    {  121, 1624,       0,    5556,       0,  0, 23,  0, }, // Singapore
    {  121, 1634,       0,    5579,       0,  0, 20,  0, }, // Solomon
    {  121, 1642,       0,    5599,       0,  0, 18,  0, }, // South_Georgia
    {  121, 1656,       0,    5617,       0,  0, 13,  0, }, // Suriname
    {  121, 1676,       0,    5630,       0,  0, 10,  0, }, // Syowa
    {  121, 1682,       0,    5640,       0,  0, 11,  0, }, // Tahiti
    {  121, 1689,    5651,    5662,    5682, 11, 20, 20, }, // Taipei
    {  121, 1696,       0,    5702,       0,  0, 15,  0, }, // Tajikistan
    {  121, 1724,       0,    5717,       0,  0, 12,  0, }, // Tokelau
    {  121, 1732,    5729,    5739,    5758, 10, 19, 17, }, // Tonga
    {  121, 1738,       0,    5775,       0,  0, 10,  0, }, // Truk
    {  121, 1750,    5785,    5802,    5828, 17, 26, 24, }, // Turkmenistan
    {  121, 1763,       0,    5852,       0,  0, 11,  0, }, // Tuvalu
    {  121, 1777,    5863,    5875,    5896, 12, 21, 19, }, // Uruguay
    {  121, 1792,    5915,    5930,    5954, 15, 24, 22, }, // Uzbekistan
    {  121, 1803,    5976,    5988,    6009, 12, 21, 19, }, // Vanuatu
    {  121, 1811,       0,    6028,       0,  0, 14,  0, }, // Venezuela
    {  121, 1821,    6042,    6058,    6083, 16, 25, 23, }, // Vladivostok
    {  121, 1833,    6106,    6120,    6143, 14, 23, 21, }, // Volgograd
    {  121, 1843,       0,    6164,       0,  0, 11,  0, }, // Vostok
    {  121, 1850,       0,    6175,       0,  0, 16,  0, }, // Wake
    {  121, 1855,       0,    6191,       0,  0, 20,  0, }, // Wallis
    {  121, 1862,    6211,    6223,    6244, 12, 21, 19, }, // Yakutsk
    {  121, 1870,    6263,    6281,    6308, 18, 27, 25, }, // Yekaterinburg
    {  121, 1892,       0,    6333,       0,  0, 10,  0, }, // Yukon
    {  122,  107,  218121,  218140,  218176, 19, 36, 30, }, // Alaska English/Deseret/United States
    {  122,  142,  218206,  218227,  218265, 21, 38, 32, }, // America_Central
    {  122,  158,  218297,  218316,  218352, 19, 36, 30, }, // America_Eastern
    {  122,  174,  218382,  218401,  218437, 19, 36, 30, }, // America_Mountain
    {  122,  191,  218467,  218488,  218526, 21, 38, 32, }, // America_Pacific
    {  122,  286,  218558,  218581,  218621, 23, 40, 34, }, // Atlantic
    {  122,  899,  218655,  218675,  218712, 20, 37, 31, }, // Hong_Kong
    {  122, 1358,  218743,  218770,  218814, 27, 44, 38, }, // Newfoundland
    {  124,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Anguilla
    {  124,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  124,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  124,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  124,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  124,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  124,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  124, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  125,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Antigua and Barbuda
    {  125,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  125,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  125,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  125,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  125,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  125,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  125, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  126,   32,       0,  218945,       0,  0, 19,  0, }, // Africa_Eastern English/Latin/Australia
    {  126,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  126,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  126,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  126,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  126,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  126,  232,  218964,  218975,  218995, 11, 20, 20, }, // Arabian
    {  126,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  126,  295,  219015,    1066,    1098, 23, 32, 32, }, // Australia_Central
    {  126,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  126,  338,  219038,    1263,    1295, 23, 32, 32, }, // Australia_Eastern
    {  126,  356,  219061,    1349,    1381, 23, 32, 32, }, // Australia_Western
    {  126,  501,    1877,    1887,  219084, 10, 19, 17, }, // China
    {  126,  532,  219101,  219117,  219142, 16, 25, 23, }, // Cook
    {  126,  871,       0,    3041,       0,  0, 18,  0, }, // Gulf
    {  126,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  126, 1023,    3513,    3523,  219165, 10, 19, 17, }, // Japan
    {  126, 1106,  219182,    3730,  219192, 10, 20, 18, }, // Korea
    {  126, 1188,    3887,    3901,    3924, 14, 23, 23, }, // Lord_Howe
    {  126, 1305,    4317,    4328,  219210, 11, 20, 20, }, // Moscow
    {  126, 1346,    4468,    4484,    4509, 16, 25, 25, }, // New_Zealand
    {  126, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  126, 1596,    5493,    5503,  219230, 10, 19, 17, }, // Samoa
    {  126, 1689,    5651,    5662,  219247, 11, 20, 18, }, // Taipei
    {  127,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Austria
    {  127,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  127,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  127,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  127,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  127,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  127,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  127,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  127,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  127,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  127, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  128,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Bahamas
    {  128,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  128,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  128,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  128,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  128,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  128,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  128, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  129,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Barbados
    {  129,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  129,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  129,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  129,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  129,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  129,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  129, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  130,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Belgium
    {  130,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  130,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  130,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  130,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  130,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  130,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  130,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  130,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  130,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  130, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  131,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Belize
    {  131,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  131,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  131,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  131,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  131,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  131,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  131, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  132,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Bermuda
    {  132,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  132,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  132,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  132,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  132,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  132,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  132, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  133,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Botswana
    {  133,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  133,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  133,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  133,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  133,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  133,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  133,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  133,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  133,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  133,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  133, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  134,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/British Indian Ocean Territory
    {  134,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  134,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  134,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  134,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  134,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  134,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  134, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  135,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/British Virgin Islands
    {  135,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  135,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  135,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  135,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  135,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  135,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  135, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  137,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Cameroon
    {  137,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  137,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  137,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  137,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  137,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  137,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  137,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  137,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  137,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  137,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  137, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  138,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan English/Latin/Canada
    {  138,  107,     184,     195,  219265, 11, 20, 27, }, // Alaska
    {  138,  142,     333,     345,  219292, 12, 21, 28, }, // America_Central
    {  138,  158,     387,     399,  219320, 12, 21, 28, }, // America_Eastern
    {  138,  240,     795,     809,     832, 14, 23, 21, }, // Argentina
    {  138,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  138,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {  138,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {  138,  397,    1523,    1538,    1562, 15, 24, 22, }, // Bangladesh
    {  138,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {  138,  437,    1607,    1620,    1642, 13, 22, 20, }, // Brasilia
    {  138,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {  138,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {  138,  507,       0,    1925,       0,  0, 21,  0, }, // Christmas
    {  138,  517,       0,    1946,       0,  0, 18,  0, }, // Cocos
    {  138,  523,    1964,    1977,    1999, 13, 22, 20, }, // Colombia
    {  138,  601,       0,    2167,       0,  0, 15,  0, }, // East_Timor
    {  138,  612,    2182,    2200,    2227, 18, 27, 25, }, // Easter
    {  138,  619,       0,    2252,       0,  0, 12,  0, }, // Ecuador
    {  138,  695,    2530,    2551,    2581, 21, 30, 28, }, // Falkland
    {  138,  723,       0,  219348,       0,  0, 34,  0, }, // French_Southern
    {  138,  746,       0,    2702,       0,  0, 14,  0, }, // Galapagos
    {  138,  830,    2877,    2896,    2924, 19, 28, 26, }, // Greenland_Eastern
    {  138,  871,       0,    3041,       0,  0, 18,  0, }, // Gulf
    {  138,  876,       0,    3059,       0,  0, 11,  0, }, // Guyana
    {  138,  914,       0,    3249,       0,  0, 19,  0, }, // India
    {  138,  933,       0,    3285,       0,  0, 14,  0, }, // Indochina
    {  138,  943,       0,    3299,       0,  0, 22,  0, }, // Indonesia_Central
    {  138,  961,       0,    3321,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  138,  979,       0,    3343,       0,  0, 22,  0, }, // Indonesia_Western
    {  138,  997,    3365,    3374,  219382,  9, 18, 25, }, // Iran
    {  138, 1219,       0,    4045,       0,  0, 13,  0, }, // Malaysia
    {  138, 1228,       0,    4058,       0,  0, 13,  0, }, // Maldives
    {  138, 1326,       0,    4388,       0,  0, 10,  0, }, // Nepal
    {  138, 1358,    4534,    4551,    4577, 17, 26, 26, }, // Newfoundland
    {  138, 1384,    4687,    4711,    4744, 24, 33, 31, }, // Noronha
    {  138, 1428,    4908,    4921,    4943, 13, 22, 20, }, // Pakistan
    {  138, 1460,    4994,    5007,    5029, 13, 22, 20, }, // Paraguay
    {  138, 1469,    5049,    5058,    5076,  9, 18, 16, }, // Peru
    {  138, 1502,  219407,  219436,  219474, 29, 38, 45, }, // Pierre_Miquelon
    {  138, 1596,    5493,    5503,  219519, 10, 19, 11, }, // Samoa
    {  138, 1777,    5863,    5875,    5896, 12, 21, 19, }, // Uruguay
    {  138, 1811,       0,    6028,       0,  0, 14,  0, }, // Venezuela
    {  138, 1855,       0,  219530,       0,  0, 22,  0, }, // Wallis
    {  139,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cayman Islands
    {  139,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  139,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  139,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  139,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  139,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  139,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  139, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  140,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Christmas Island
    {  140,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  140,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  140,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  140,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  140,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  140,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  140, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  141,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cocos Islands
    {  141,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  141,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  141,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  141,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  141,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  141,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  141, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  142,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cook Islands
    {  142,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  142,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  142,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  142,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  142,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  142,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  142, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  143,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cyprus
    {  143,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  143,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  143,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  143,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  143,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  143,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  143, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  144,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Denmark
    {  144,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  144,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  144,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  144,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  144,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  144,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  144,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  144,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  144,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  144, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  145,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Diego Garcia
    {  145,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  145,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  145,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  145,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  145,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  145,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  145, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  146,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Dominica
    {  146,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  146,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  146,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  146,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  146,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  146,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  146, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  147,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Eritrea
    {  147,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  147,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  147,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  147,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  147,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  147,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  147,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  147,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  147,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  147,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  147, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  148,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Eswatini
    {  148,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  148,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  148,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  148,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  148,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  148,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  148,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  148,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  148,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  148,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  148, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  149,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Europe
    {  149,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  149,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  149,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  149,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  149,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  149,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  149,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  149,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  149,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  149, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  150,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Falkland Islands
    {  150,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  150,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  150,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  150,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  150,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  150,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  150, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  151,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Fiji
    {  151,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  151,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  151,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  151,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  151,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  151,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  151, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  152,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Finland
    {  152,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  152,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  152,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  152,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  152,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  152,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  152,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  152,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  152,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  152, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  153,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Gambia
    {  153,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  153,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  153,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  153,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  153,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  153,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  153,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  153,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  153,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  153,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  153, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  154,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Germany
    {  154,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  154,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  154,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  154,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  154,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  154,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  154,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  154,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  154,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  154, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  155,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Ghana
    {  155,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  155,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  155,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  155,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  155,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  155,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  155,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  155,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  155,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  155,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  155, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  156,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Gibraltar
    {  156,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  156,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  156,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  156,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  156,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  156,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  156, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  157,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Grenada
    {  157,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  157,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  157,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  157,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  157,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  157,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  157, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  158,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro English/Latin/Guam
    {  159,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Guernsey
    {  159,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  159,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  159,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  159,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  159,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  159,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  159, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  160,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Guyana
    {  160,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  160,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  160,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  160,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  160,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  160,  876,       0,    3059,       0,  0, 11,  0, }, // Guyana
    {  160,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  160, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  161,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Hong Kong
    {  161,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  161,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  161,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  161,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  161,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  161,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  161,  899,    3148,    3162,    3185, 14, 23, 21, }, // Hong_Kong
    {  161, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  162,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/India
    {  162,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  162,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  162,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  162,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  162,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  162,  871,       0,    3041,       0,  0, 18,  0, }, // Gulf
    {  162,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  162,  909,  219552,  219562,  219581, 10, 19, 17, }, // Hovd
    {  162,  914,       0,    3249,       0,  0, 19,  0, }, // India
    {  162, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  163,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Indonesia
    {  163,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  163,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  163,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  163,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  163,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  163,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  163,  943,       0,    3299,       0,  0, 22,  0, }, // Indonesia_Central
    {  163,  961,       0,    3321,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  163,  979,       0,    3343,       0,  0, 22,  0, }, // Indonesia_Western
    {  163, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  164,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Ireland
    {  164,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  164,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  164,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  164,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  164,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  164,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  164, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  165,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Isle of Man
    {  165,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  165,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  165,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  165,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  165,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  165,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  165, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  166,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Israel
    {  166,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  166,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  166,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  166,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  166,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  166,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  166, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  167,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Jamaica
    {  167,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  167,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  167,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  167,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  167,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  167,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  167, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  168,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Jersey
    {  168,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  168,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  168,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  168,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  168,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  168,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  168, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  169,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Kenya
    {  169,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  169,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  169,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  169,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  169,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  169,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  169,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  169,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  169,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  169,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  169, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  170,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Kiribati
    {  170,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  170,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  170,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  170,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  170,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  170,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  170, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  171,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Lesotho
    {  171,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  171,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  171,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  171,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  171,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  171,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  171,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  171,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  171,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  171,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  171, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  172,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Liberia
    {  172,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  172,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  172,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  172,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  172,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  172,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  172,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  172,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  172,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  172,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  172, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  173,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Macao
    {  173,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  173,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  173,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  173,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  173,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  173,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  173,  899,    3148,    3162,    3185, 14, 23, 21, }, // Hong_Kong
    {  173, 1198,    3947,    3957,    3976, 10, 19, 17, }, // Macau
    {  173, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  174,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Madagascar
    {  174,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  174,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  174,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  174,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  174,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  174,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  174,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  174,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  174,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  174,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  174, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  175,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Malawi
    {  175,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  175,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  175,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  175,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  175,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  175,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  175,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  175,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  175,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  175,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  175, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  176,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Malaysia
    {  176,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  176,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  176,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  176,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  176,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  176,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  176, 1219,       0,    4045,       0,  0, 13,  0, }, // Malaysia
    {  176, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  177,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Maldives
    {  177,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  177,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  177,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  177,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  177,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  177,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  177, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  178,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Malta
    {  178,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  178,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  178,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  178,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  178,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  178,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  178, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  179,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Marshall Islands
    {  179,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  179,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  179,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  179,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  179,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  179,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  180,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Mauritius
    {  180,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  180,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  180,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  180,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  180,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  180,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  180,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  180,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  180,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  180,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  180, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  181,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Micronesia
    {  181,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  181,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  181,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  181,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  181,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  181,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  181, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  182,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Montserrat
    {  182,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  182,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  182,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  182,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  182,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  182,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  182, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  183,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Namibia
    {  183,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  183,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  183,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  183,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  183,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  183,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  183,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  183,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  183,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  183,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  183, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  184,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Nauru
    {  184,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  184,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  184,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  184,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  184,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  184,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  184, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  185,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Netherlands
    {  185,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  185,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  185,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  185,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  185,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  185,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  185,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  185,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  185,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  185, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  186,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/New Zealand
    {  186,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  186,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  186,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  186,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  186,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  186,  295,    1044,    1066,    1098, 22, 32, 32, }, // Australia_Central
    {  186,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  186,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {  186,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {  186,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {  186,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  186, 1188,    3887,    3901,    3924, 14, 23, 23, }, // Lord_Howe
    {  186, 1346,    4468,    4484,    4509, 16, 25, 25, }, // New_Zealand
    {  186, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  187,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Nigeria
    {  187,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  187,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  187,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  187,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  187,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  187,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  187,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  187,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  187,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  187,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  187, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  188,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Niue
    {  188,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  188,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  188,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  188,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  188,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  188,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  188, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  189,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Norfolk Island
    {  189,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  189,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  189,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  189,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  189,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  189,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  189, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  190,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Northern Mariana Islands
    {  190,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  190,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  190,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  190,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  190,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  190,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  191,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Pakistan
    {  191,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  191,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  191,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  191,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  191,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  191,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  191, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  192,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Palau
    {  192,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  192,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  192,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  192,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  192,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  192,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  192, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  193,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Papua New Guinea
    {  193,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  193,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  193,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  193,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  193,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  193,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  193, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  195,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Pitcairn
    {  195,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  195,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  195,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  195,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  195,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  195,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  195, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  197,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Rwanda
    {  197,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  197,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  197,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  197,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  197,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  197,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  197,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  197,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  197,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  197,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  197, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  198,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Helena
    {  198,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  198,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  198,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  198,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  198,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  198,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  198, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  199,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Kitts and Nevis
    {  199,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  199,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  199,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  199,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  199,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  199,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  199, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  200,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Lucia
    {  200,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  200,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  200,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  200,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  200,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  200,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  200, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  201,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Vincent and Grenadines
    {  201,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  201,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  201,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  201,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  201,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  201,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  201, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  202,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Samoa
    {  202,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  202,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  202,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  202,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  202,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  202,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  202, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  203,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Seychelles
    {  203,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  203,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  203,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  203,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  203,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  203,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  203, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  204,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Sierra Leone
    {  204,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  204,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  204,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  204,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  204,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  204,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  204,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  204,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  204,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  204,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  204, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  205,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Singapore
    {  205,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  205,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  205,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  205,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  205,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  205,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  205, 1219,       0,    4045,       0,  0, 13,  0, }, // Malaysia
    {  205, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  205, 1624,       0,    5556,       0,  0, 23,  0, }, // Singapore
    {  206,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Sint Maarten
    {  206,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  206,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  206,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  206,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  206,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  206,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  206, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  207,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Slovenia
    {  207,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  207,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  207,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  207,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  207,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  207,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  207,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  207,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  207,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  207, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  208,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Solomon Islands
    {  208,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  208,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  208,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  208,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  208,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  208,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  208, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  209,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/South Africa
    {  209,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  209,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  209,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  209,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  209,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  209,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  209,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  209,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  209,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  209,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  209, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  210,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/South Sudan
    {  210,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  210,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  210,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  210,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  210,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  210,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  210,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  210,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  210,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  210,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  210, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  211,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Sudan
    {  211,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  211,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  211,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  211,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  211,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  211,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  211,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  211,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  211,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  211,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  211, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  212,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Sweden
    {  212,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  212,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  212,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  212,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  212,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  212,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  212,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  212,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  212,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  212, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  213,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Switzerland
    {  213,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  213,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  213,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  213,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  213,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  213,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  213,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  213,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  213,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  213, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  214,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Tanzania
    {  214,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  214,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  214,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  214,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  214,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  214,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  214,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  214,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  214,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  214,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  214, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  215,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Tokelau
    {  215,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  215,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  215,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  215,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  215,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  215,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  215, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  216,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Tonga
    {  216,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  216,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  216,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  216,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  216,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  216,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  216, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  217,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Trinidad and Tobago
    {  217,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  217,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  217,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  217,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  217,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  217,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  217, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  218,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Turks and Caicos Islands
    {  218,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  218,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  218,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  218,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  218,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  218,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  218, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  219,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Tuvalu
    {  219,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  219,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  219,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  219,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  219,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  219,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  219, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  220,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Uganda
    {  220,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  220,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  220,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  220,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  220,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  220,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  220,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  220,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  220,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  220,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  220, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  221,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/United Arab Emirates
    {  221,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  221,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  221,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  221,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  221,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  221,  871,       0,    3041,       0,  0, 18,  0, }, // Gulf
    {  221,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  222,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/United Kingdom
    {  222,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  222,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  222,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  222,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  222,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  222,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  222,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  222,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  222,  871,       0,    3041,       0,  0, 18,  0, }, // Gulf
    {  222,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  222, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  225,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Vanuatu
    {  225,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  225,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  225,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  225,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  225,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  225,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  225, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  226,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/world
    {  226,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  226,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  226,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  226,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  226,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  226,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  226, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  227,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Zambia
    {  227,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  227,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  227,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  227,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  227,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  227,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  227,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  227,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  227,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  227,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  227, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  228,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Zimbabwe
    {  228,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  228,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  228,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  228,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  228,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  228,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  228,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  228,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  228,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  228,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  228, 1502,  218852,  218877,  218911, 25, 34, 34, }, // Pierre_Miquelon
    {  230,    5,       0,  219598,       0,  0, 17,  0, }, // Afghanistan Erzya/Cyrillic/Russia
    {  230,  107,  219615,  219627,  219647, 12, 20, 19, }, // Alaska
    {  230,  135,  219666,  219679,  219700, 13, 21, 20, }, // Amazon
    {  230,  397,  219720,  219736,  219760, 16, 24, 23, }, // Bangladesh
    {  230,  437,  219783,  219796,  219817, 13, 21, 20, }, // Brasilia
    {  230,  495,  219837,  219847,  219865, 10, 18, 17, }, // Chile
    {  230,  501,  219882,  219893,  219912, 11, 19, 18, }, // China
    {  230,  537,  219930,  219940,  219958, 10, 18, 17, }, // Cuba
    {  230,  914,       0,  219975,       0,  0, 19,  0, }, // India
    {  230,  997,  219994,  220005,  220024, 11, 19, 18, }, // Iran
    {  230, 1023,  220042,  220054,  220074, 12, 20, 19, }, // Japan
    {  230, 1058,       0,  220093,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  230, 1077,       0,  220120,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  230, 1106,  220148,  220159,  220178, 11, 19, 18, }, // Korea
    {  230, 1305,  220196,  220209,  220230, 13, 21, 20, }, // Moscow
    {  230, 1346,  220250,  220267,  220292, 17, 25, 24, }, // New_Zealand
    {  230, 1418,  220316,  220328,  220348, 12, 20, 19, }, // Omsk
    {  230, 1460,  220367,  220381,  220403, 14, 22, 21, }, // Paraguay
    {  230, 1570,  220424,  220438,  220460, 14, 22, 21, }, // Sakhalin
    {  230, 1777,  220481,  220494,  220515, 13, 21, 20, }, // Uruguay
    {  230, 1792,  220535,  220552,  220577, 17, 25, 24, }, // Uzbekistan
    {  231,    5,       0,  220601,       0,  0, 12,  0, }, // Afghanistan Esperanto/Latin/world
    {  231,   17,       0,  220613,       0,  0, 17,  0, }, // Africa_Central
    {  231,   32,       0,  220630,       0,  0, 18,  0, }, // Africa_Eastern
    {  231,   65,       0,  220648,       0,  0, 15,  0, }, // Africa_Southern
    {  231,   81,  220663,  220683,  220709, 20, 26, 27, }, // Africa_Western
    {  231,  107,  220736,  220748,  220766, 12, 18, 19, }, // Alaska
    {  231,  135,  220785,  220799,  220819, 14, 20, 21, }, // Amazon
    {  231,  142,  220840,  220864,  220894, 24, 30, 31, }, // America_Central
    {  231,  158,  220925,  220950,  220981, 25, 31, 32, }, // America_Eastern
    {  231,  174,  221013,  221038,  221069, 25, 31, 32, }, // America_Mountain
    {  231,  191,  221101,  221127,  221159, 26, 32, 33, }, // America_Pacific
    {  231,  214,  221192,  221203,  221221, 11, 18, 19, }, // Apia
    {  231,  232,  221240,  221251,  221268, 11, 17, 18, }, // Arabian
    {  231,  240,  221286,  221301,  221322, 15, 21, 22, }, // Argentina
    {  231,  250,  221344,  221368,  221398, 24, 30, 31, }, // Argentina_Western
    {  231,  268,  221429,  221441,  221459, 12, 18, 19, }, // Armenia
    {  231,  286,  221478,  221505,  221538, 27, 33, 34, }, // Atlantic
    {  231,  295,  221572,  221594,  221622, 22, 28, 29, }, // Australia_Central
    {  231,  313,  221651,  221681,  221717, 30, 36, 37, }, // Australia_CentralWestern
    {  231,  338,  221754,  221777,  221806, 23, 29, 30, }, // Australia_Eastern
    {  231,  356,  221656,  221686,  221722, 25, 31, 32, }, // Australia_Western
    {  231,  374,  221836,  221853,  221876, 17, 23, 24, }, // Azerbaijan
    {  231,  385,  221900,  221913,  221933, 13, 20, 21, }, // Azores
    {  231,  397,  221954,  221970,  221992, 16, 22, 23, }, // Bangladesh
    {  231,  415,       0,  222015,       0,  0, 12,  0, }, // Bhutan
    {  231,  422,       0,  222027,       0,  0, 13,  0, }, // Bolivia
    {  231,  437,  222040,  222054,  222074, 14, 20, 21, }, // Brasilia
    {  231,  454,       0,  222095,       0,  0, 13,  0, }, // Brunei
    {  231,  461,  222108,  222123,  222144, 15, 21, 22, }, // Cape_Verde
    {  231,  478,       0,  222166,       0,  0, 12,  0, }, // Chamorro
    {  231,  487,  222178,  222191,  222210, 13, 19, 20, }, // Chatham
    {  231,  495,  222230,  222241,  222258, 11, 17, 18, }, // Chile
    {  231,  501,  222276,  222286,  222302, 10, 16, 17, }, // China
    {  231,  507,       0,  222319,       0,  0, 21,  0, }, // Christmas
    {  231,  517,       0,  222340,       0,  0, 17,  0, }, // Cocos
    {  231,  523,  222357,  222371,  222391, 14, 20, 21, }, // Colombia
    {  231,  532,  222412,  222427,  222448, 15, 21, 22, }, // Cook
    {  231,  537,  222470,  222481,  222499, 11, 18, 19, }, // Cuba
    {  231,  548,       0,  222518,       0,  0, 12,  0, }, // Davis
    {  231,  564,       0,  222530,       0,  0, 23,  0, }, // DumontDUrville
    {  231,  601,       0,  222553,       0,  0, 19,  0, }, // East_Timor
    {  231,  612,  222572,  222588,  222610, 16, 22, 23, }, // Easter
    {  231,  619,       0,  222633,       0,  0, 14,  0, }, // Ecuador
    {  231,  627,  222647,  222664,  222687, 17, 23, 24, }, // Europe_Central
    {  231,  642,  222711,  222729,  222753, 18, 24, 25, }, // Europe_Eastern
    {  231,  657,       0,  222778,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  231,  680,  222804,  222824,  222850, 20, 26, 27, }, // Europe_Western
    {  231,  695,  222877,  222892,  222913, 15, 21, 22, }, // Falkland
    {  231,  704,  222935,  222946,  222963, 11, 17, 18, }, // Fiji
    {  231,  709,       0,  222981,       0,  0, 20,  0, }, // French_Guiana
    {  231,  723,       0,  223001,       0,  0, 31,  0, }, // French_Southern
    {  231,  746,       0,  223032,       0,  0, 14,  0, }, // Galapagos
    {  231,  756,       0,  223046,       0,  0, 14,  0, }, // Gambier
    {  231,  764,  223060,  223074,  223094, 14, 20, 21, }, // Georgia
    {  231,  772,       0,  223115,       0,  0, 19,  0, }, // Gilbert_Islands
    {  231,  788,       0,  223134,       0,  0, 28,  0, }, // GMT
    {  231,  830,  223162,  223185,  223214, 23, 29, 30, }, // Greenland_Eastern
    {  231,  848,  223244,  223269,  223300, 25, 31, 32, }, // Greenland_Western
    {  231,  871,       0,  223332,       0,  0, 21,  0, }, // Gulf
    {  231,  876,       0,  223353,       0,  0, 12,  0, }, // Guyana
    {  231,  883,  223365,  223386,  223414, 21, 28, 29, }, // Hawaii_Aleutian
    {  231,  899,  223443,  223457,  223477, 14, 20, 21, }, // Hong_Kong
    {  231,  909,  223498,  223509,  223526, 11, 17, 18, }, // Hovd
    {  231,  914,       0,  223544,       0,  0, 17,  0, }, // India
    {  231,  920,       0,  223561,       0,  0, 16,  0, }, // Indian_Ocean
    {  231,  933,       0,  223577,       0,  0, 15,  0, }, // Indochina
    {  231,  943,       0,  223592,       0,  0, 21,  0, }, // Indonesia_Central
    {  231,  961,       0,  223613,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  231,  979,       0,  223635,       0,  0, 24,  0, }, // Indonesia_Western
    {  231,  997,  223659,  223670,  223687, 11, 17, 18, }, // Iran
    {  231, 1008,  223705,  223719,  223739, 14, 20, 21, }, // Irkutsk
    {  231, 1016,  223760,  223773,  223792, 13, 19, 20, }, // Israel
    {  231, 1023,  223812,  223824,  223842, 12, 18, 19, }, // Japan
    {  231, 1047,       0,  223861,       0,  0, 12,  0, }, // Kazakhstan
    {  231, 1058,       0,  223873,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  231, 1077,       0,  223892,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  231, 1106,  223913,  223924,  223941, 11, 17, 18, }, // Korea
    {  231, 1112,       0,  223959,       0,  0, 13,  0, }, // Kosrae
    {  231, 1119,  223972,  223990,  224014, 18, 24, 25, }, // Krasnoyarsk
    {  231, 1151,       0,  224039,       0,  0, 13,  0, }, // Kyrgystan
    {  231, 1175,       0,  224052,       0,  0, 21,  0, }, // Line_Islands
    {  231, 1188,  224073,  224089,  224112, 16, 23, 24, }, // Lord_Howe
    {  231, 1204,  224136,  224150,  224170, 14, 20, 21, }, // Magadan
    {  231, 1219,       0,  224191,       0,  0, 14,  0, }, // Malaysia
    {  231, 1228,       0,  224205,       0,  0, 13,  0, }, // Maldives
    {  231, 1237,       0,  224218,       0,  0, 18,  0, }, // Marquesas
    {  231, 1247,       0,  224236,       0,  0, 18,  0, }, // Marshall_Islands
    {  231, 1264,  224254,  224268,  224288, 14, 20, 21, }, // Mauritius
    {  231, 1274,       0,  224309,       0,  0, 13,  0, }, // Mawson
    {  231, 1281,  224322,  224344,  224372, 22, 28, 29, }, // Mexico_Pacific
    {  231, 1296,  224401,  224417,  224439, 16, 22, 23, }, // Mongolia
    {  231, 1305,  224462,  224474,  224492, 12, 18, 19, }, // Moscow
    {  231, 1312,       0,  224511,       0,  0, 11,  0, }, // Myanmar
    {  231, 1320,       0,  224522,       0,  0, 11,  0, }, // Nauru
    {  231, 1326,       0,  224533,       0,  0, 12,  0, }, // Nepal
    {  231, 1332,  224545,  224563,  224587, 18, 24, 25, }, // New_Caledonia
    {  231, 1346,  224612,  224628,  224650, 16, 22, 23, }, // New_Zealand
    {  231, 1358,  224673,  224688,  224710, 15, 22, 23, }, // Newfoundland
    {  231, 1371,       0,  224733,       0,  0, 10,  0, }, // Niue
    {  231, 1376,  224743,  224762,  224787, 19, 25, 26, }, // Norfolk
    {  231, 1384,  224813,  224839,  224872, 26, 33, 34, }, // Noronha
    {  231, 1406,  224906,  224924,  224948, 18, 24, 25, }, // Novosibirsk
    {  231, 1418,  224973,  224984,  225001, 11, 17, 18, }, // Omsk
    {  231, 1428,  225019,  225034,  225055, 15, 21, 22, }, // Pakistan
    {  231, 1437,       0,  225077,       0,  0, 12,  0, }, // Palau
    {  231, 1443,       0,  225089,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  231, 1460,  225112,  225127,  225148, 15, 21, 22, }, // Paraguay
    {  231, 1469,  225170,  225181,  225198, 11, 17, 18, }, // Peru
    {  231, 1474,  225216,  225230,  225250, 14, 20, 21, }, // Philippines
    {  231, 1486,       0,  225271,       0,  0, 18,  0, }, // Phoenix_Islands
    {  231, 1502,  225289,  225321,  225360, 32, 39, 40, }, // Pierre_Miquelon
    {  231, 1518,       0,  225400,       0,  0, 19,  0, }, // Pitcairn
    {  231, 1527,       0,  225419,       0,  0, 13,  0, }, // Ponape
    {  231, 1534,       0,  225432,       0,  0, 16,  0, }, // Pyongyang
    {  231, 1554,       0,  225448,       0,  0, 13,  0, }, // Reunion
    {  231, 1562,       0,  225461,       0,  0, 14,  0, }, // Rothera
    {  231, 1570,  225475,  225489,  225509, 14, 20, 21, }, // Sakhalin
    {  231, 1596,  225530,  225541,  225558, 11, 17, 18, }, // Samoa
    {  231, 1602,       0,  225576,       0,  0, 13,  0, }, // Seychelles
    {  231, 1624,       0,  225589,       0,  0, 21,  0, }, // Singapore
    {  231, 1634,       0,  225610,       0,  0, 16,  0, }, // Solomon
    {  231, 1642,       0,  225626,       0,  0, 18,  0, }, // South_Georgia
    {  231, 1656,       0,  225644,       0,  0, 14,  0, }, // Suriname
    {  231, 1676,       0,  225658,       0,  0, 12,  0, }, // Syowa
    {  231, 1682,       0,  225670,       0,  0, 13,  0, }, // Tahiti
    {  231, 1689,  225683,  225695,  225713, 12, 18, 19, }, // Taipei
    {  231, 1696,       0,  225732,       0,  0, 12,  0, }, // Tajikistan
    {  231, 1724,       0,  225744,       0,  0, 13,  0, }, // Tokelau
    {  231, 1732,  225757,  225768,  225785, 11, 17, 18, }, // Tonga
    {  231, 1738,       0,  225803,       0,  0, 11,  0, }, // Truk
    {  231, 1750,  225814,  225828,  225848, 14, 20, 21, }, // Turkmenistan
    {  231, 1763,       0,  225869,       0,  0, 12,  0, }, // Tuvalu
    {  231, 1777,  225881,  225895,  225915, 14, 20, 21, }, // Uruguay
    {  231, 1792,  225936,  225948,  225966, 12, 18, 19, }, // Uzbekistan
    {  231, 1803,  225985,  225999,  226019, 14, 20, 21, }, // Vanuatu
    {  231, 1811,       0,  226040,       0,  0, 15,  0, }, // Venezuela
    {  231, 1821,  226055,  226073,  226097, 18, 24, 25, }, // Vladivostok
    {  231, 1833,  226122,  226138,  226160, 16, 22, 23, }, // Volgograd
    {  231, 1843,       0,  226183,       0,  0, 13,  0, }, // Vostok
    {  231, 1850,       0,  226196,       0,  0, 16,  0, }, // Wake
    {  231, 1855,       0,  226212,       0,  0, 24,  0, }, // Wallis
    {  231, 1862,  226236,  226250,  226270, 14, 20, 21, }, // Yakutsk
    {  231, 1870,  226291,  226311,  226337, 20, 26, 27, }, // Yekaterinburg
    {  231, 1892,       0,  226364,       0,  0, 13,  0, }, // Yukon
    {  232,    0,  226377,  226385,  226401,  8, 16, 12, }, // Acre Estonian/Latin/Estonia
    {  232,    5,       0,  226413,       0,  0, 15,  0, }, // Afghanistan
    {  232,   17,       0,  226428,       0,  0, 16,  0, }, // Africa_Central
    {  232,   32,       0,  226444,       0,  0, 15,  0, }, // Africa_Eastern
    {  232,   65,       0,  226459,       0,  0, 25,  0, }, // Africa_Southern
    {  232,   81,  226484,  226501,  226526, 17, 25, 21, }, // Africa_Western
    {  232,  107,  226547,  226557,  226575, 10, 18, 14, }, // Alaska
    {  232,  128,  226589,  226599,  226617, 10, 18, 14, }, // Almaty
    {  232,  135,  226631,  226644,  226665, 13, 21, 17, }, // Amazon
    {  232,  142,  226682,  226699,  226724, 17, 25, 21, }, // America_Central
    {  232,  158,  226745,  226759,  226781, 14, 22, 18, }, // America_Eastern
    {  232,  174,  226799,  226817,  226843, 18, 26, 22, }, // America_Mountain
    {  232,  191,  226865,  226883,  226909, 18, 26, 22, }, // America_Pacific
    {  232,  207,  226931,  226942,  226961, 11, 19, 15, }, // Anadyr
    {  232,  214,  226976,  226984,  227000,  8, 16, 12, }, // Apia
    {  232,  219,  227012,  227021,  227038,  9, 17, 13, }, // Aqtau
    {  232,  225,  227051,  227061,  227079, 10, 18, 14, }, // Aqtobe
    {  232,  232,  227093,  227104,  227123, 11, 19, 15, }, // Arabian
    {  232,  240,  227138,  227151,  227172, 13, 21, 17, }, // Argentina
    {  232,  250,  227189,  227208,  227235, 19, 27, 23, }, // Argentina_Western
    {  232,  268,  227258,  227270,  227290, 12, 20, 16, }, // Armenia
    {  232,  286,  227306,  227317,  227336, 11, 19, 15, }, // Atlantic
    {  232,  295,  227351,  227370,  227397, 19, 27, 23, }, // Australia_Central
    {  232,  313,  227420,  227445,  227478, 25, 33, 29, }, // Australia_CentralWestern
    {  232,  338,  227507,  227525,  227551, 18, 26, 22, }, // Australia_Eastern
    {  232,  356,  227573,  227593,  227621, 20, 28, 24, }, // Australia_Western
    {  232,  374,  227645,  227662,  227687, 17, 25, 21, }, // Azerbaijan
    {  232,  385,  227708,  227721,  227742, 13, 21, 17, }, // Azores
    {  232,  397,  227759,  227774,  227797, 15, 23, 19, }, // Bangladesh
    {  232,  415,       0,  227816,       0,  0, 11,  0, }, // Bhutan
    {  232,  422,       0,  227827,       0,  0, 12,  0, }, // Bolivia
    {  232,  437,  227839,  227852,  227873, 13, 21, 17, }, // Brasilia
    {  232,  454,       0,  227890,       0,  0, 10,  0, }, // Brunei
    {  232,  461,  227900,  227919,  227946, 19, 27, 23, }, // Cape_Verde
    {  232,  472,       0,  227969,       0,  0,  9,  0, }, // Casey
    {  232,  478,       0,  227978,       0,  0, 20,  0, }, // Chamorro
    {  232,  487,  227998,  228010,  228030, 12, 20, 16, }, // Chatham
    {  232,  495,  228046,  228056,  228074, 10, 18, 14, }, // Chile
    {  232,  501,  228088,  228097,  228114,  9, 17, 13, }, // China
    {  232,  507,       0,  228127,       0,  0, 14,  0, }, // Christmas
    {  232,  517,       0,  228141,       0,  0, 16,  0, }, // Cocos
    {  232,  523,  228157,  228169,  228189, 12, 20, 16, }, // Colombia
    {  232,  532,  228205,  228221,  228245, 16, 24, 28, }, // Cook
    {  232,  537,  228273,  228282,  228299,  9, 17, 13, }, // Cuba
    {  232,  548,       0,  228312,       0,  0, 10,  0, }, // Davis
    {  232,  564,       0,  228322,       0,  0, 22,  0, }, // DumontDUrville
    {  232,  601,       0,  228344,       0,  0, 14,  0, }, // East_Timor
    {  232,  612,  228358,  228376,  228402, 18, 26, 22, }, // Easter
    {  232,  619,       0,  228424,       0,  0, 12,  0, }, // Ecuador
    {  232,  627,  228436,  228452,  228476, 16, 24, 20, }, // Europe_Central
    {  232,  642,  228496,  228511,  228534, 15, 23, 19, }, // Europe_Eastern
    {  232,  657,       0,  228553,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  232,  680,  228582,  228599,  228624, 17, 25, 21, }, // Europe_Western
    {  232,  695,  228645,  228665,  228693, 20, 28, 24, }, // Falkland
    {  232,  704,  228717,  228726,  228743,  9, 17, 13, }, // Fiji
    {  232,  709,       0,  228756,       0,  0, 22,  0, }, // French_Guiana
    {  232,  723,       0,  228778,       0,  0, 41,  0, }, // French_Southern
    {  232,  746,       0,  228819,       0,  0, 14,  0, }, // Galapagos
    {  232,  756,       0,  228833,       0,  0, 12,  0, }, // Gambier
    {  232,  764,  228845,  228856,  228875, 11, 19, 15, }, // Georgia
    {  232,  772,       0,  228890,       0,  0, 19,  0, }, // Gilbert_Islands
    {  232,  788,       0,  228909,       0,  0, 14,  0, }, // GMT
    {  232,  802,  228923,  228936,  228957, 13, 21, 17, }, // Greenland
    {  232,  830,  228974,  228991,  229016, 17, 25, 21, }, // Greenland_Eastern
    {  232,  848,  229037,  229056,  229083, 19, 27, 23, }, // Greenland_Western
    {  232,  866,       0,  229106,       0,  0, 17,  0, }, // Guam
    {  232,  871,       0,  229123,       0,  0, 23,  0, }, // Gulf
    {  232,  876,       0,  229146,       0,  0, 10,  0, }, // Guyana
    {  232,  883,  229156,  229174,  229200, 18, 26, 22, }, // Hawaii_Aleutian
    {  232,  899,  229222,  229235,  229256, 13, 21, 17, }, // Hong_Kong
    {  232,  909,  229273,  229282,  229299,  9, 17, 13, }, // Hovd
    {  232,  914,       0,  229312,       0,  0,  9,  0, }, // India
    {  232,  920,       0,  229321,       0,  0, 17,  0, }, // Indian_Ocean
    {  232,  933,       0,  229338,       0,  0, 13,  0, }, // Indochina
    {  232,  943,       0,  229351,       0,  0, 19,  0, }, // Indonesia_Central
    {  232,  961,       0,  229370,       0,  0, 18,  0, }, // Indonesia_Eastern
    {  232,  979,       0,  229388,       0,  0, 20,  0, }, // Indonesia_Western
    {  232,  997,  229408,  229418,  229436, 10, 18, 14, }, // Iran
    {  232, 1008,  229450,  229462,  229482, 12, 20, 16, }, // Irkutsk
    {  232, 1016,  229498,  229510,  229530, 12, 20, 16, }, // Israel
    {  232, 1023,  229546,  229557,  229576, 11, 19, 15, }, // Japan
    {  232, 1029,  229591,  229619,  229640, 28, 21, 17, }, // Kamchatka
    {  232, 1047,       0,  229657,       0,  0, 14,  0, }, // Kazakhstan
    {  232, 1058,       0,  229671,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  232, 1077,       0,  229689,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  232, 1106,  229709,  229718,  229735,  9, 17, 13, }, // Korea
    {  232, 1112,       0,  229748,       0,  0, 10,  0, }, // Kosrae
    {  232, 1119,  229758,  229774,  229798, 16, 24, 20, }, // Krasnoyarsk
    {  232, 1151,       0,  229818,       0,  0, 15,  0, }, // Kyrgystan
    {  232, 1161,       0,  229833,       0,  0, 13,  0, }, // Lanka
    {  232, 1175,       0,  229846,       0,  0, 17,  0, }, // Line_Islands
    {  232, 1188,  229863,  229878,  229901, 15, 23, 19, }, // Lord_Howe
    {  232, 1198,  229920,  229929,  229946,  9, 17, 13, }, // Macau
    {  232, 1204,  229959,  229971,  229991, 12, 20, 16, }, // Magadan
    {  232, 1219,       0,  230007,       0,  0, 12,  0, }, // Malaysia
    {  232, 1228,       0,  230019,       0,  0, 12,  0, }, // Maldives
    {  232, 1237,       0,  230031,       0,  0, 16,  0, }, // Marquesas
    {  232, 1247,       0,  230047,       0,  0, 20,  0, }, // Marshall_Islands
    {  232, 1264,  230067,  230081,  230103, 14, 22, 18, }, // Mauritius
    {  232, 1274,       0,  230121,       0,  0, 11,  0, }, // Mawson
    {  232, 1281,  230132,  230158,  230192, 26, 34, 30, }, // Mexico_Pacific
    {  232, 1296,  230222,  230238,  230262, 16, 24, 20, }, // Mongolia
    {  232, 1305,  230282,  230292,  230310, 10, 18, 14, }, // Moscow
    {  232, 1312,       0,  230324,       0,  0,  9,  0, }, // Myanmar
    {  232, 1320,       0,  230333,       0,  0,  9,  0, }, // Nauru
    {  232, 1326,       0,  230342,       0,  0, 10,  0, }, // Nepal
    {  232, 1332,  230352,  230370,  230396, 18, 26, 22, }, // New_Caledonia
    {  232, 1346,  230418,  230433,  230456, 15, 23, 19, }, // New_Zealand
    {  232, 1358,  230475,  230492,  230517, 17, 25, 21, }, // Newfoundland
    {  232, 1371,       0,  230538,       0,  0,  8,  0, }, // Niue
    {  232, 1376,  230546,  230564,  230590, 18, 26, 22, }, // Norfolk
    {  232, 1384,  230612,  230635,  230666, 23, 31, 27, }, // Noronha
    {  232, 1392,       0,  230693,       0,  0, 18,  0, }, // North_Mariana
    {  232, 1406,  230711,  230727,  230751, 16, 24, 20, }, // Novosibirsk
    {  232, 1418,  230771,  230780,  230797,  9, 17, 13, }, // Omsk
    {  232, 1428,  230810,  230823,  230844, 13, 21, 17, }, // Pakistan
    {  232, 1437,       0,  230861,       0,  0,  9,  0, }, // Palau
    {  232, 1443,       0,  230870,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  232, 1460,  230891,  230903,  230923, 12, 20, 16, }, // Paraguay
    {  232, 1469,  230939,  230948,  230965,  9, 17, 13, }, // Peru
    {  232, 1474,  230978,  230991,  231012, 13, 21, 17, }, // Philippines
    {  232, 1486,       0,  231029,       0,  0, 18,  0, }, // Phoenix_Islands
    {  232, 1502,  231047,  231078,  231117, 31, 39, 35, }, // Pierre_Miquelon
    {  232, 1518,       0,  231152,       0,  0, 13,  0, }, // Pitcairn
    {  232, 1527,       0,  231165,       0,  0, 11,  0, }, // Ponape
    {  232, 1534,       0,  231176,       0,  0, 14,  0, }, // Pyongyang
    {  232, 1544,  231190,  231203,  231224, 13, 21, 17, }, // Qyzylorda
    {  232, 1554,       0,  231241,       0,  0, 12,  0, }, // Reunion
    {  232, 1562,       0,  231253,       0,  0, 11,  0, }, // Rothera
    {  232, 1570,  231264,  231277,  231298, 13, 21, 17, }, // Sakhalin
    {  232, 1579,  231315,  231325,  231343, 10, 18, 14, }, // Samara
    {  232, 1596,  231357,  231366,  231383,  9, 17, 13, }, // Samoa
    {  232, 1602,       0,  231396,       0,  0, 12,  0, }, // Seychelles
    {  232, 1624,       0,  231408,       0,  0, 21,  0, }, // Singapore
    {  232, 1634,       0,  231429,       0,  0, 20,  0, }, // Solomon
    {  232, 1642,       0,  231449,       0,  0, 17,  0, }, // South_Georgia
    {  232, 1656,       0,  231466,       0,  0, 12,  0, }, // Suriname
    {  232, 1676,       0,  231478,       0,  0,  9,  0, }, // Syowa
    {  232, 1682,       0,  231487,       0,  0, 10,  0, }, // Tahiti
    {  232, 1689,  231497,  231507,  231525, 10, 18, 14, }, // Taipei
    {  232, 1696,       0,  231539,       0,  0, 16,  0, }, // Tajikistan
    {  232, 1724,       0,  231555,       0,  0, 11,  0, }, // Tokelau
    {  232, 1732,  231566,  231575,  231592,  9, 17, 13, }, // Tonga
    {  232, 1738,       0,  231605,       0,  0, 10,  0, }, // Truk
    {  232, 1750,  231615,  231632,  231657, 17, 25, 21, }, // Turkmenistan
    {  232, 1763,       0,  231678,       0,  0, 10,  0, }, // Tuvalu
    {  232, 1777,  231688,  231699,  231718, 11, 19, 15, }, // Uruguay
    {  232, 1792,  231733,  231748,  231771, 15, 23, 19, }, // Uzbekistan
    {  232, 1803,  231790,  231801,  231820, 11, 19, 15, }, // Vanuatu
    {  232, 1811,       0,  231835,       0,  0, 13,  0, }, // Venezuela
    {  232, 1821,  231848,  231864,  231888, 16, 24, 20, }, // Vladivostok
    {  232, 1833,  231908,  231922,  231944, 14, 22, 18, }, // Volgograd
    {  232, 1843,       0,  231962,       0,  0, 11,  0, }, // Vostok
    {  232, 1850,       0,  231973,       0,  0, 10,  0, }, // Wake
    {  232, 1855,       0,  231983,       0,  0, 21,  0, }, // Wallis
    {  232, 1862,  232004,  232016,  232036, 12, 20, 16, }, // Yakutsk
    {  232, 1870,  232052,  232070,  232096, 18, 26, 22, }, // Yekaterinburg
    {  232, 1892,       0,  232118,       0,  0, 10,  0, }, // Yukon
    {  233,    0,  232128,  232141,  232160, 13, 19, 23, }, // Acre Ewe/Latin/Ghana
    {  233,    5,       0,  232183,       0,  0, 21,  0, }, // Afghanistan
    {  233,   17,       0,  232204,       0,  0, 24,  0, }, // Africa_Central
    {  233,   32,       0,  232228,       0,  0, 21,  0, }, // Africa_Eastern
    {  233,   65,       0,  232249,       0,  0, 29,  0, }, // Africa_Southern
    {  233,   81,  232278,  232294,  232322, 16, 28, 31, }, // Africa_Western
    {  233,  107,  232353,  232368,  232391, 15, 23, 21, }, // Alaska
    {  233,  128,  232412,  232427,  232448, 15, 21, 25, }, // Almaty
    {  233,  135,  232473,  232488,  232511, 15, 23, 26, }, // Amazon
    {  233,  142,  232537,  232560,  232591, 23, 31, 29, }, // America_Central
    {  233,  158,  232620,  232645,  232677, 25, 32, 30, }, // America_Eastern
    {  233,  174,  232707,  232725,  232750, 18, 25, 23, }, // America_Mountain
    {  233,  191,  232773,  232789,  232813, 16, 24, 22, }, // America_Pacific
    {  233,  207,  232835,  232850,  232871, 15, 21, 23, }, // Anadyr
    {  233,  214,  232894,  232908,  232929, 14, 21, 19, }, // Apia
    {  233,  219,  232948,  232962,  232982, 14, 20, 24, }, // Aqtau
    {  233,  225,  233006,  233021,  233042, 15, 21, 16, }, // Aqtobe
    {  233,  232,  233058,  233074,  233097, 16, 23, 26, }, // Arabian
    {  233,  240,  233123,  233142,  233168, 19, 26, 29, }, // Argentina
    {  233,  250,  233197,  233225,  233260, 28, 35, 38, }, // Argentina_Western
    {  233,  268,  233298,  233315,  233339, 17, 24, 27, }, // Armenia
    {  233,  286,  233366,  233383,  233407, 17, 24, 22, }, // Atlantic
    {  233,  295,  233429,  233456,  233491, 27, 35, 36, }, // Australia_Central
    {  233,  313,  233527,  233574,  233617, 47, 43, 41, }, // Australia_CentralWestern
    {  233,  338,  233658,  233685,  233720, 27, 35, 33, }, // Australia_Eastern
    {  233,  356,  233753,  233780,  233815, 27, 35, 33, }, // Australia_Western
    {  233,  374,  233848,  233868,  233895, 20, 27, 30, }, // Azerbaijan
    {  233,  385,  233925,  233941,  233964, 16, 23, 26, }, // Azores
    {  233,  397,  233990,  234010,  234037, 20, 27, 30, }, // Bangladesh
    {  233,  415,       0,  234067,       0,  0, 16,  0, }, // Bhutan
    {  233,  422,       0,  234083,       0,  0, 17,  0, }, // Bolivia
    {  233,  437,  234100,  234118,  234143, 18, 25, 28, }, // Brasilia
    {  233,  454,       0,  234171,       0,  0, 27,  0, }, // Brunei
    {  233,  461,  234198,  234218,  234245, 20, 27, 30, }, // Cape_Verde
    {  233,  478,       0,  234275,       0,  0, 18,  0, }, // Chamorro
    {  233,  487,  234293,  234310,  234334, 17, 24, 22, }, // Chatham
    {  233,  495,  234356,  234371,  234393, 15, 22, 25, }, // Chile
    {  233,  501,  234418,  234433,  234455, 15, 22, 20, }, // China
    {  233,  507,       0,  234475,       0,  0, 26,  0, }, // Christmas
    {  233,  517,       0,  234501,       0,  0, 23,  0, }, // Cocos
    {  233,  523,  234524,  234542,  234567, 18, 25, 28, }, // Colombia
    {  233,  532,  234595,  234617,  234646, 22, 29, 32, }, // Cook
    {  233,  537,  234678,  234691,  234711, 13, 20, 18, }, // Cuba
    {  233,  548,       0,  234729,       0,  0, 15,  0, }, // Davis
    {  233,  564,       0,  234744,       0,  0, 26,  0, }, // DumontDUrville
    {  233,  601,       0,  234770,       0,  0, 20,  0, }, // East_Timor
    {  233,  612,  234790,  234813,  234843, 23, 30, 33, }, // Easter
    {  233,  619,       0,  234876,       0,  0, 17,  0, }, // Ecuador
    {  233,  627,  234893,  234917,  234948, 24, 31, 34, }, // Europe_Central
    {  233,  642,  234982,  235005,  235034, 23, 29, 31, }, // Europe_Eastern
    {  233,  680,  235065,  235089,  235120, 24, 31, 34, }, // Europe_Western
    {  233,  695,  235154,  235180,  235213, 26, 33, 36, }, // Falkland
    {  233,  704,  235249,  235263,  235284, 14, 21, 24, }, // Fiji
    {  233,  709,       0,  235308,       0,  0, 23,  0, }, // French_Guiana
    {  233,  723,       0,  235331,       0,  0, 37,  0, }, // French_Southern
    {  233,  746,       0,  235368,       0,  0, 19,  0, }, // Galapagos
    {  233,  756,       0,  235387,       0,  0, 17,  0, }, // Gambier
    {  233,  764,  235404,  235421,  235445, 17, 24, 27, }, // Georgia
    {  233,  772,       0,  235472,       0,  0, 25,  0, }, // Gilbert_Islands
    {  233,  788,       0,  235497,       0,  0, 19,  0, }, // GMT
    {  233,  830,  235516,  235539,  235570, 23, 31, 34, }, // Greenland_Eastern
    {  233,  848,  235604,  235628,  235659, 24, 31, 29, }, // Greenland_Western
    {  233,  871,       0,  235688,       0,  0, 21,  0, }, // Gulf
    {  233,  876,       0,  235709,       0,  0, 16,  0, }, // Guyana
    {  233,  883,  235725,  235749,  235780, 24, 31, 29, }, // Hawaii_Aleutian
    {  233,  899,  235809,  235828,  235854, 19, 26, 29, }, // Hong_Kong
    {  233,  909,  235883,  235897,  235918, 14, 21, 24, }, // Hovd
    {  233,  914,       0,  235942,       0,  0, 15,  0, }, // India
    {  233,  920,       0,  235957,       0,  0, 22,  0, }, // Indian_Ocean
    {  233,  933,       0,  235979,       0,  0, 19,  0, }, // Indochina
    {  233,  943,       0,  235998,       0,  0, 27,  0, }, // Indonesia_Central
    {  233,  961,       0,  236025,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  233,  979,       0,  236052,       0,  0, 27,  0, }, // Indonesia_Western
    {  233,  997,  236079,  236093,  236114, 14, 21, 19, }, // Iran
    {  233, 1008,  236133,  236150,  236174, 17, 24, 27, }, // Irkutsk
    {  233, 1016,  236201,  236217,  236240, 16, 23, 21, }, // Israel
    {  233, 1023,  236261,  236276,  236298, 15, 22, 25, }, // Japan
    {  233, 1029,  236323,  236356,  236395, 33, 39, 41, }, // Kamchatka
    {  233, 1058,       0,  236436,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {  233, 1077,       0,  236461,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  233, 1106,  236486,  236501,  236523, 15, 22, 25, }, // Korea
    {  233, 1112,       0,  236548,       0,  0, 16,  0, }, // Kosrae
    {  233, 1119,  236564,  236585,  236613, 21, 28, 31, }, // Krasnoyarsk
    {  233, 1151,       0,  236644,       0,  0, 19,  0, }, // Kyrgystan
    {  233, 1175,       0,  236663,       0,  0, 22,  0, }, // Line_Islands
    {  233, 1188,  236685,  236704,  236730, 19, 26, 24, }, // Lord_Howe
    {  233, 1198,  236754,  236768,  236788, 14, 20, 22, }, // Macau
    {  233, 1204,  236810,  236827,  236851, 17, 24, 27, }, // Magadan
    {  233, 1219,       0,  236878,       0,  0, 18,  0, }, // Malaysia
    {  233, 1228,       0,  236896,       0,  0, 18,  0, }, // Maldives
    {  233, 1237,       0,  236914,       0,  0, 19,  0, }, // Marquesas
    {  233, 1247,       0,  236933,       0,  0, 26,  0, }, // Marshall_Islands
    {  233, 1264,  236959,  236978,  237004, 19, 26, 29, }, // Mauritius
    {  233, 1274,       0,  237033,       0,  0, 16,  0, }, // Mawson
    {  233, 1281,  237049,  237074,  237106, 25, 32, 29, }, // Mexico_Pacific
    {  233, 1296,  237135,  237155,  237182, 20, 27, 30, }, // Mongolia
    {  233, 1305,  237212,  237228,  237251, 16, 23, 26, }, // Moscow
    {  233, 1312,       0,  237277,       0,  0, 17,  0, }, // Myanmar
    {  233, 1320,       0,  237294,       0,  0, 15,  0, }, // Nauru
    {  233, 1326,       0,  237309,       0,  0, 15,  0, }, // Nepal
    {  233, 1332,  237324,  237347,  237377, 23, 30, 33, }, // New_Caledonia
    {  233, 1346,  237410,  237431,  237459, 21, 28, 26, }, // New_Zealand
    {  233, 1358,  237485,  237506,  237534, 21, 28, 26, }, // Newfoundland
    {  233, 1371,       0,  237560,       0,  0, 14,  0, }, // Niue
    {  233, 1376,  237574,  237598,  237629, 24, 31, 34, }, // Norfolk
    {  233, 1384,  237663,  237692,  237728, 29, 36, 39, }, // Noronha
    {  233, 1406,  237767,  237788,  237816, 21, 28, 31, }, // Novosibirsk
    {  233, 1418,  237847,  237861,  237882, 14, 21, 24, }, // Omsk
    {  233, 1428,  237906,  237924,  237949, 18, 25, 28, }, // Pakistan
    {  233, 1437,       0,  237977,       0,  0, 15,  0, }, // Palau
    {  233, 1443,       0,  237992,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  233, 1460,  238018,  238036,  238061, 18, 25, 28, }, // Paraguay
    {  233, 1469,  238089,  238103,  238124, 14, 21, 23, }, // Peru
    {  233, 1474,  238147,  238167,  238194, 20, 27, 30, }, // Philippines
    {  233, 1486,       0,  238224,       0,  0, 25,  0, }, // Phoenix_Islands
    {  233, 1502,  238249,  238279,  238316, 30, 37, 35, }, // Pierre_Miquelon
    {  233, 1518,       0,  238351,       0,  0, 18,  0, }, // Pitcairn
    {  233, 1527,       0,  238369,       0,  0, 16,  0, }, // Ponape
    {  233, 1534,       0,  238385,       0,  0, 19,  0, }, // Pyongyang
    {  233, 1544,  238404,  238422,  238446, 18, 24, 28, }, // Qyzylorda
    {  233, 1554,       0,  238474,       0,  0, 17,  0, }, // Reunion
    {  233, 1562,       0,  238491,       0,  0, 17,  0, }, // Rothera
    {  233, 1570,  238508,  238526,  238551, 18, 25, 28, }, // Sakhalin
    {  233, 1579,  238579,  238594,  238615, 15, 21, 23, }, // Samara
    {  233, 1596,  238638,  238653,  238675, 15, 22, 20, }, // Samoa
    {  233, 1602,       0,  238695,       0,  0, 20,  0, }, // Seychelles
    {  233, 1624,       0,  238715,       0,  0, 26,  0, }, // Singapore
    {  233, 1634,       0,  238741,       0,  0, 25,  0, }, // Solomon
    {  233, 1642,       0,  238766,       0,  0, 23,  0, }, // South_Georgia
    {  233, 1656,       0,  238789,       0,  0, 17,  0, }, // Suriname
    {  233, 1676,       0,  238806,       0,  0, 15,  0, }, // Syowa
    {  233, 1682,       0,  238821,       0,  0, 16,  0, }, // Tahiti
    {  233, 1689,  238837,  238853,  238876, 16, 23, 21, }, // Taipei
    {  233, 1696,       0,  238897,       0,  0, 20,  0, }, // Tajikistan
    {  233, 1724,       0,  238917,       0,  0, 17,  0, }, // Tokelau
    {  233, 1732,  238934,  238949,  238971, 15, 22, 25, }, // Tonga
    {  233, 1738,       0,  238996,       0,  0, 15,  0, }, // Truk
    {  233, 1750,  239011,  239033,  239062, 22, 29, 32, }, // Turkmenistan
    {  233, 1763,       0,  239094,       0,  0, 16,  0, }, // Tuvalu
    {  233, 1777,  239110,  239127,  239151, 17, 24, 27, }, // Uruguay
    {  233, 1792,  239178,  239198,  239225, 20, 27, 30, }, // Uzbekistan
    {  233, 1803,  239255,  239272,  239296, 17, 24, 27, }, // Vanuatu
    {  233, 1811,       0,  239323,       0,  0, 19,  0, }, // Venezuela
    {  233, 1821,  239342,  239363,  239391, 21, 28, 31, }, // Vladivostok
    {  233, 1833,  239422,  239441,  239467, 19, 26, 28, }, // Volgograd
    {  233, 1843,       0,  239495,       0,  0, 16,  0, }, // Vostok
    {  233, 1850,       0,  239511,       0,  0, 21,  0, }, // Wake
    {  233, 1855,       0,  239532,       0,  0, 25,  0, }, // Wallis
    {  233, 1862,  239557,  239574,  239598, 17, 24, 27, }, // Yakutsk
    {  233, 1870,  239625,  239648,  239678, 23, 30, 33, }, // Yekaterinburg
    {  236,    5,       0,  239711,       0,  0, 14,  0, }, // Afghanistan Faroese/Latin/Faroe Islands
    {  236,   17,       0,  239725,       0,  0, 13,  0, }, // Africa_Central
    {  236,   32,       0,  239738,       0,  0, 16,  0, }, // Africa_Eastern
    {  236,   65,       0,  239754,       0,  0, 22,  0, }, // Africa_Southern
    {  236,   81,  239776,  239792,  239815, 16, 23, 22, }, // Africa_Western
    {  236,  107,  239837,  239847,  239864, 10, 17, 16, }, // Alaska
    {  236,  135,  239880,  239891,  239909, 11, 18, 17, }, // Amazon
    {  236,  142,  239926,  239937,  239955, 11, 18, 17, }, // America_Central
    {  236,  158,  239972,  239983,  240001, 11, 18, 17, }, // America_Eastern
    {  236,  174,  240018,  240030,  240049, 12, 19, 18, }, // America_Mountain
    {  236,  191,  240067,  240078,  240096, 11, 18, 17, }, // America_Pacific
    {  236,  214,  240113,  240121,  240136,  8, 15, 14, }, // Apia
    {  236,  232,  240150,  240161,  240179, 11, 18, 17, }, // Arabian
    {  236,  240,  240196,  240209,  240229, 13, 20, 19, }, // Argentina
    {  236,  250,  240248,  240268,  240295, 20, 27, 26, }, // Argentina_Western
    {  236,  268,  240321,  240332,  240350, 11, 18, 17, }, // Armenia
    {  236,  286,  240367,  240379,  240398, 12, 19, 18, }, // Atlantic
    {  236,  295,  240416,  240433,  240457, 17, 24, 23, }, // Australia_Central
    {  236,  313,  240480,  240503,  240533, 23, 30, 29, }, // Australia_CentralWestern
    {  236,  338,  240562,  240582,  240609, 20, 27, 26, }, // Australia_Eastern
    {  236,  356,  240483,  240506,  240536, 20, 27, 26, }, // Australia_Western
    {  236,  374,  240635,  240649,  240670, 14, 21, 20, }, // Azerbaijan
    {  236,  385,  240690,  240703,  240723, 13, 20, 19, }, // Azores
    {  236,  397,  240742,  240756,  240777, 14, 21, 20, }, // Bangladesh
    {  236,  415,       0,  240797,       0,  0,  9,  0, }, // Bhutan
    {  236,  422,       0,  240806,       0,  0, 11,  0, }, // Bolivia
    {  236,  437,  240817,  240829,  240848, 12, 19, 18, }, // Brasilia
    {  236,  454,       0,  240866,       0,  0, 21,  0, }, // Brunei
    {  236,  461,  240887,  240907,  240934, 20, 27, 26, }, // Cape_Verde
    {  236,  478,       0,  240960,       0,  0, 19,  0, }, // Chamorro
    {  236,  487,  240979,  240990,  241008, 11, 18, 17, }, // Chatham
    {  236,  495,  241025,  241033,  241048,  8, 15, 14, }, // Chile
    {  236,  501,  241062,  241070,  241085,  8, 15, 14, }, // China
    {  236,  507,       0,  241099,       0,  0, 13,  0, }, // Christmas
    {  236,  517,       0,  241112,       0,  0, 16,  0, }, // Cocos
    {  236,  523,  241128,  241140,  241159, 12, 19, 18, }, // Colombia
    {  236,  532,  241177,  241193,  241216, 16, 23, 22, }, // Cook
    {  236,  537,  241238,  241246,  241261,  8, 15, 14, }, // Cuba
    {  236,  548,       0,  241275,       0,  0,  9,  0, }, // Davis
    {  236,  564,       0,  241284,       0,  0, 20,  0, }, // DumontDUrville
    {  236,  601,       0,  241304,       0,  0, 15,  0, }, // East_Timor
    {  236,  612,  241319,  241334,  241356, 15, 22, 21, }, // Easter
    {  236,  619,       0,  241377,       0,  0, 11,  0, }, // Ecuador
    {  236,  627,  241388,  241401,  241421, 13, 20, 19, }, // Europe_Central
    {  236,  642,  241440,  241456,  241479, 16, 23, 22, }, // Europe_Eastern
    {  236,  657,       0,  241501,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  236,  680,  241524,  241540,  241563, 16, 23, 22, }, // Europe_Western
    {  236,  695,  241585,  241605,  241632, 20, 27, 26, }, // Falkland
    {  236,  704,  241658,  241666,  241681,  8, 15, 14, }, // Fiji
    {  236,  709,       0,  241695,       0,  0, 18,  0, }, // French_Guiana
    {  236,  723,       0,  241713,       0,  0, 41,  0, }, // French_Southern
    {  236,  746,       0,  241754,       0,  0, 13,  0, }, // Galapagos
    {  236,  756,       0,  241767,       0,  0, 11,  0, }, // Gambier
    {  236,  764,  241778,  241789,  241807, 11, 18, 17, }, // Georgia
    {  236,  772,       0,  241824,       0,  0, 18,  0, }, // Gilbert_Islands
    {  236,  788,       0,  241842,       0,  0, 18,  0, }, // GMT
    {  236,  830,  241860,  241881,  241909, 21, 28, 27, }, // Greenland_Eastern
    {  236,  848,  241936,  241957,  241985, 21, 28, 27, }, // Greenland_Western
    {  236,  871,       0,  242012,       0,  0, 15,  0, }, // Gulf
    {  236,  876,       0,  241703,       0,  0, 10,  0, }, // Guyana
    {  236,  883,  242027,  242046,  242072, 19, 26, 25, }, // Hawaii_Aleutian
    {  236,  899,  242097,  242110,  242130, 13, 20, 19, }, // Hong_Kong
    {  236,  909,  242149,  242157,  242172,  8, 15, 14, }, // Hovd
    {  236,  914,       0,  242186,       0,  0,  9,  0, }, // India
    {  236,  920,       0,  242195,       0,  0, 12,  0, }, // Indian_Ocean
    {  236,  933,       0,  242207,       0,  0, 12,  0, }, // Indochina
    {  236,  943,       0,  242219,       0,  0, 17,  0, }, // Indonesia_Central
    {  236,  961,       0,  242236,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  236,  979,       0,  242256,       0,  0, 20,  0, }, // Indonesia_Western
    {  236,  997,  242276,  242284,  242299,  8, 15, 14, }, // Iran
    {  236, 1008,  242313,  242324,  242342, 11, 18, 17, }, // Irkutsk
    {  236, 1016,  242359,  242369,  242386, 10, 17, 16, }, // Israel
    {  236, 1023,  242402,  242411,  242427,  9, 16, 15, }, // Japan
    {  236, 1047,       0,  242442,       0,  0, 13,  0, }, // Kazakhstan
    {  236, 1058,       0,  242455,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  236, 1077,       0,  242475,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  236, 1106,  242495,  242504,  242520,  9, 16, 15, }, // Korea
    {  236, 1112,       0,  242535,       0,  0, 10,  0, }, // Kosrae
    {  236, 1119,  242545,  242560,  242582, 15, 22, 21, }, // Krasnoyarsk
    {  236, 1151,       0,  242603,       0,  0, 12,  0, }, // Kyrgystan
    {  236, 1175,       0,  242615,       0,  0, 15,  0, }, // Line_Islands
    {  236, 1188,  242630,  242643,  242663, 13, 20, 19, }, // Lord_Howe
    {  236, 1204,  242682,  242693,  242711, 11, 18, 17, }, // Magadan
    {  236, 1219,       0,  242728,       0,  0, 12,  0, }, // Malaysia
    {  236, 1228,       0,  242740,       0,  0, 17,  0, }, // Maldives
    {  236, 1237,       0,  242757,       0,  0, 13,  0, }, // Marquesas
    {  236, 1247,       0,  242770,       0,  0, 19,  0, }, // Marshall_Islands
    {  236, 1264,  242789,  242801,  242820, 12, 19, 18, }, // Mauritius
    {  236, 1274,       0,  242838,       0,  0, 10,  0, }, // Mawson
    {  236, 1281,  242848,  242867,  242893, 19, 26, 25, }, // Mexico_Pacific
    {  236, 1296,  242918,  242932,  242953, 14, 21, 20, }, // Mongolia
    {  236, 1305,  242973,  242983,  243000, 10, 17, 16, }, // Moscow
    {  236, 1312,       0,  243016,       0,  0, 19,  0, }, // Myanmar
    {  236, 1320,       0,  243035,       0,  0,  9,  0, }, // Nauru
    {  236, 1326,       0,  243044,       0,  0,  9,  0, }, // Nepal
    {  236, 1332,  243053,  243068,  243090, 15, 22, 21, }, // New_Caledonia
    {  236, 1346,  243111,  243123,  243142, 12, 19, 18, }, // New_Zealand
    {  236, 1358,  243160,  243176,  243199, 16, 23, 22, }, // Newfoundland
    {  236, 1371,       0,  243221,       0,  0,  8,  0, }, // Niue
    {  236, 1376,  243229,  243245,  243268, 16, 23, 22, }, // Norfolk
    {  236, 1384,  243290,  243313,  243343, 23, 30, 29, }, // Noronha
    {  236, 1406,  243372,  243387,  243409, 15, 22, 21, }, // Novosibirsk
    {  236, 1418,  243430,  243438,  243453,  8, 15, 14, }, // Omsk
    {  236, 1428,  243467,  243479,  243498, 12, 19, 18, }, // Pakistan
    {  236, 1437,       0,  243516,       0,  0,  9,  0, }, // Palau
    {  236, 1443,       0,  243525,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  236, 1460,  243543,  243555,  243574, 12, 19, 18, }, // Paraguay
    {  236, 1469,  243592,  243600,  243615,  8, 15, 14, }, // Peru
    {  236, 1474,  243629,  243646,  243670, 17, 24, 23, }, // Philippines
    {  236, 1486,       0,  243693,       0,  0, 18,  0, }, // Phoenix_Islands
    {  236, 1502,  243711,  243736,  243768, 25, 32, 31, }, // Pierre_Miquelon
    {  236, 1518,       0,  243799,       0,  0, 19,  0, }, // Pitcairn
    {  236, 1527,       0,  243818,       0,  0, 10,  0, }, // Ponape
    {  236, 1534,       0,  243828,       0,  0, 13,  0, }, // Pyongyang
    {  236, 1554,       0,  243841,       0,  0, 11,  0, }, // Reunion
    {  236, 1562,       0,  243852,       0,  0, 11,  0, }, // Rothera
    {  236, 1570,  243863,  243875,  243894, 12, 19, 18, }, // Sakhalin
    {  236, 1596,  243912,  243921,  243937,  9, 16, 15, }, // Samoa
    {  236, 1602,       0,  243952,       0,  0, 19,  0, }, // Seychelles
    {  236, 1624,       0,  243971,       0,  0, 12,  0, }, // Singapore
    {  236, 1634,       0,  243983,       0,  0, 18,  0, }, // Solomon
    {  236, 1642,       0,  244001,       0,  0, 23,  0, }, // South_Georgia
    {  236, 1656,       0,  244024,       0,  0, 11,  0, }, // Suriname
    {  236, 1676,       0,  244035,       0,  0,  9,  0, }, // Syowa
    {  236, 1682,       0,  244044,       0,  0, 10,  0, }, // Tahiti
    {  236, 1689,  244054,  244064,  244081, 10, 17, 16, }, // Taipei
    {  236, 1696,       0,  244097,       0,  0, 16,  0, }, // Tajikistan
    {  236, 1724,       0,  244113,       0,  0, 11,  0, }, // Tokelau
    {  236, 1732,  244124,  244133,  244149,  9, 16, 15, }, // Tonga
    {  236, 1738,       0,  244164,       0,  0,  9,  0, }, // Truk
    {  236, 1750,  244173,  244189,  244212, 16, 23, 22, }, // Turkmenistan
    {  236, 1763,       0,  244234,       0,  0, 10,  0, }, // Tuvalu
    {  236, 1777,  244244,  244255,  244273, 11, 18, 17, }, // Uruguay
    {  236, 1792,  244290,  244304,  244325, 14, 21, 20, }, // Uzbekistan
    {  236, 1803,  244345,  244356,  244374, 11, 18, 17, }, // Vanuatu
    {  236, 1811,       0,  244391,       0,  0, 13,  0, }, // Venezuela
    {  236, 1821,  244404,  244419,  244441, 15, 22, 21, }, // Vladivostok
    {  236, 1833,  244462,  244475,  244495, 13, 20, 19, }, // Volgograd
    {  236, 1843,       0,  244514,       0,  0, 10,  0, }, // Vostok
    {  236, 1850,       0,  244524,       0,  0, 13,  0, }, // Wake
    {  236, 1855,       0,  244537,       0,  0, 28,  0, }, // Wallis
    {  236, 1862,  244565,  244576,  244594, 11, 18, 17, }, // Yakutsk
    {  236, 1870,  244611,  244628,  244652, 17, 24, 23, }, // Yekaterinburg
    {  236, 1892,       0,  244675,       0,  0,  9,  0, }, // Yukon
    {  238,    5,       0,  144359,       0,  0, 19,  0, }, // Afghanistan Filipino/Latin/Philippines
    {  238,   17,       0,  244684,       0,  0, 22,  0, }, // Africa_Central
    {  238,   32,       0,  244706,       0,  0, 24,  0, }, // Africa_Eastern
    {  238,   65,       0,  244730,       0,  0, 20,  0, }, // Africa_Southern
    {  238,   81,  244750,  244774,  244810, 24, 36, 36, }, // Africa_Western
    {  238,  107,  144535,  244846,  244872, 14, 26, 23, }, // Alaska
    {  238,  135,  144603,  244895,  244921, 14, 26, 26, }, // Amazon
    {  238,  142,  244947,  244979,  245023, 32, 44, 44, }, // America_Central
    {  238,  158,  245067,  245103,  245152, 36, 49, 48, }, // America_Eastern
    {  238,  174,  245200,  245234,  245280, 34, 46, 46, }, // America_Mountain
    {  238,  191,  245326,  245362,  245410, 36, 48, 48, }, // America_Pacific
    {  238,  207,  245458,  245472,  245495, 14, 23, 21, }, // Anadyr
    {  238,  214,  144745,  245516,  245540, 12, 24, 21, }, // Apia
    {  238,  232,  144807,  245561,  245587, 14, 26, 23, }, // Arabian
    {  238,  240,  144875,  245610,  245639, 17, 29, 29, }, // Argentina
    {  238,  250,  245668,  245695,  245734, 27, 39, 39, }, // Argentina_Western
    {  238,  268,  145053,  245773,  245800, 15, 27, 27, }, // Armenia
    {  238,  286,  245827,  245844,  245873, 17, 29, 29, }, // Atlantic
    {  238,  295,  245902,  245927,  245964, 25, 37, 34, }, // Australia_Central
    {  238,  313,  245998,  246035,  246081, 37, 46, 44, }, // Australia_CentralWestern
    {  238,  338,  246125,  246152,  246191, 27, 39, 36, }, // Australia_Eastern
    {  238,  356,  246227,  246254,  246293, 27, 39, 36, }, // Australia_Western
    {  238,  374,  145626,  246329,  246359, 18, 30, 30, }, // Azerbaijan
    {  238,  385,  145706,  246389,  246415, 14, 26, 26, }, // Azores
    {  238,  397,  145774,  246441,  246471, 18, 30, 30, }, // Bangladesh
    {  238,  415,       0,  145854,       0,  0, 14,  0, }, // Bhutan
    {  238,  422,       0,  145868,       0,  0, 15,  0, }, // Bolivia
    {  238,  437,  145883,  246501,  246529, 16, 28, 28, }, // Brasilia
    {  238,  454,       0,  145957,       0,  0, 25,  0, }, // Brunei
    {  238,  461,  145982,  246557,  246587, 18, 30, 30, }, // Cape_Verde
    {  238,  478,       0,  246617,       0,  0, 28,  0, }, // Chamorro
    {  238,  487,  146091,  246645,  246672, 15, 27, 24, }, // Chatham
    {  238,  495,  146162,  246696,  246721, 13, 25, 25, }, // Chile
    {  238,  501,  146227,  246746,  246771, 13, 25, 22, }, // China
    {  238,  507,       0,  146292,       0,  0, 24,  0, }, // Christmas
    {  238,  517,       0,  146316,       0,  0, 21,  0, }, // Cocos
    {  238,  523,  146337,  246793,  246821, 16, 28, 28, }, // Colombia
    {  238,  532,  146411,  246849,  246881, 20, 32, 44, }, // Cook
    {  238,  537,  146507,  246925,  246949, 12, 24, 24, }, // Cuba
    {  238,  548,       0,  146569,       0,  0, 13,  0, }, // Davis
    {  238,  564,       0,  146582,       0,  0, 24,  0, }, // DumontDUrville
    {  238,  601,       0,  146606,       0,  0, 18,  0, }, // East_Timor
    {  238,  612,  146624,  246973,  247006, 21, 33, 33, }, // Easter
    {  238,  619,       0,  146713,       0,  0, 15,  0, }, // Ecuador
    {  238,  627,  247039,  247061,  247095, 22, 34, 34, }, // Europe_Central
    {  238,  642,  247129,  247153,  247189, 24, 36, 36, }, // Europe_Eastern
    {  238,  657,       0,  247225,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  238,  680,  247256,  247280,  247316, 24, 36, 36, }, // Europe_Western
    {  238,  695,  147034,  247352,  247388, 24, 36, 36, }, // Falkland
    {  238,  704,  147132,  247424,  247448, 12, 24, 24, }, // Fiji
    {  238,  709,       0,  147194,       0,  0, 21,  0, }, // French_Guiana
    {  238,  723,       0,  247472,       0,  0, 38,  0, }, // French_Southern
    {  238,  746,       0,  147251,       0,  0, 17,  0, }, // Galapagos
    {  238,  756,       0,  147268,       0,  0, 15,  0, }, // Gambier
    {  238,  764,  147283,  247510,  247537, 15, 27, 27, }, // Georgia
    {  238,  772,       0,  147354,       0,  0, 23,  0, }, // Gilbert_Islands
    {  238,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {  238,  830,  247564,  247591,  247630, 27, 39, 39, }, // Greenland_Eastern
    {  238,  848,  247669,  247696,  247735, 27, 39, 39, }, // Greenland_Western
    {  238,  871,       0,  147596,       0,  0, 12,  0, }, // Gulf
    {  238,  876,       0,  147608,       0,  0, 14,  0, }, // Guyana
    {  238,  883,  147622,  247774,  247809, 23, 35, 35, }, // Hawaii_Aleutian
    {  238,  899,  147717,  247844,  247873, 17, 29, 29, }, // Hong_Kong
    {  238,  909,  147794,  247902,  247926, 12, 24, 24, }, // Hovd
    {  238,  914,       0,  247950,       0,  0, 25,  0, }, // India
    {  238,  920,       0,  147882,       0,  0, 20,  0, }, // Indian_Ocean
    {  238,  933,       0,  147902,       0,  0, 17,  0, }, // Indochina
    {  238,  943,       0,  247975,       0,  0, 25,  0, }, // Indonesia_Central
    {  238,  961,       0,  248000,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  238,  979,       0,  248027,       0,  0, 27,  0, }, // Indonesia_Western
    {  238,  997,  147994,  248054,  248078, 12, 24, 21, }, // Iran
    {  238, 1008,  148056,  248099,  248126, 15, 27, 27, }, // Irkutsk
    {  238, 1016,  148127,  248153,  248179, 14, 26, 23, }, // Israel
    {  238, 1023,  148195,  248202,  248227, 13, 25, 22, }, // Japan
    {  238, 1029,  248249,  248281,  248322, 32, 41, 39, }, // Kamchatka
    {  238, 1047,       0,  248361,       0,  0, 18,  0, }, // Kazakhstan
    {  238, 1058,       0,  248379,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  238, 1077,       0,  248407,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  238, 1106,  148324,  248435,  248460, 13, 25, 22, }, // Korea
    {  238, 1112,       0,  148389,       0,  0, 14,  0, }, // Kosrae
    {  238, 1119,  148403,  248482,  248513, 19, 31, 31, }, // Krasnoyarsk
    {  238, 1151,       0,  248544,       0,  0, 17,  0, }, // Kyrgystan
    {  238, 1175,       0,  148504,       0,  0, 20,  0, }, // Line_Islands
    {  238, 1188,  148524,  248561,  248590, 17, 29, 27, }, // Lord_Howe
    {  238, 1204,  148601,  248617,  248644, 15, 27, 27, }, // Magadan
    {  238, 1219,       0,  148672,       0,  0, 16,  0, }, // Malaysia
    {  238, 1228,       0,  148688,       0,  0, 16,  0, }, // Maldives
    {  238, 1237,       0,  148704,       0,  0, 17,  0, }, // Marquesas
    {  238, 1247,       0,  148721,       0,  0, 24,  0, }, // Marshall_Islands
    {  238, 1264,  148745,  248671,  248700, 17, 29, 29, }, // Mauritius
    {  238, 1274,       0,  148822,       0,  0, 14,  0, }, // Mawson
    {  238, 1281,  248729,  248755,  248793, 26, 38, 38, }, // Mexico_Pacific
    {  238, 1296,  248831,  248849,  248879, 18, 30, 30, }, // Mongolia
    {  238, 1305,  149014,  248909,  248935, 14, 26, 26, }, // Moscow
    {  238, 1312,       0,  149082,       0,  0, 15,  0, }, // Myanmar
    {  238, 1320,       0,  149097,       0,  0, 13,  0, }, // Nauru
    {  238, 1326,       0,  149110,       0,  0, 13,  0, }, // Nepal
    {  238, 1332,  149123,  248961,  248994, 21, 33, 33, }, // New_Caledonia
    {  238, 1346,  149212,  249027,  249058, 19, 31, 28, }, // New_Zealand
    {  238, 1358,  149295,  249086,  249118, 20, 32, 32, }, // Newfoundland
    {  238, 1371,       0,  149381,       0,  0, 12,  0, }, // Niue
    {  238, 1376,  149393,  249150,  249184, 22, 34, 31, }, // Norfolk
    {  238, 1384,  149485,  249215,  249254, 27, 39, 39, }, // Noronha
    {  238, 1406,  149592,  249293,  249324, 19, 31, 31, }, // Novosibirsk
    {  238, 1418,  149675,  249355,  249379, 12, 24, 24, }, // Omsk
    {  238, 1428,  149737,  249403,  249431, 16, 28, 28, }, // Pakistan
    {  238, 1437,       0,  149811,       0,  0, 13,  0, }, // Palau
    {  238, 1443,       0,  149824,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  238, 1460,  149848,  249459,  249487, 16, 28, 28, }, // Paraguay
    {  238, 1469,  149922,  249515,  249539, 12, 24, 24, }, // Peru
    {  238, 1474,  149984,  249563,  249592, 17, 29, 29, }, // Philippines
    {  238, 1486,       0,  150061,       0,  0, 23,  0, }, // Phoenix_Islands
    {  238, 1502,  249621,  249652,  249695, 31, 43, 43, }, // Pierre_Miquelon
    {  238, 1518,       0,  150197,       0,  0, 16,  0, }, // Pitcairn
    {  238, 1527,       0,  150213,       0,  0, 14,  0, }, // Ponape
    {  238, 1534,       0,  150227,       0,  0, 17,  0, }, // Pyongyang
    {  238, 1554,       0,  150244,       0,  0, 15,  0, }, // Reunion
    {  238, 1562,       0,  150259,       0,  0, 15,  0, }, // Rothera
    {  238, 1570,  150274,  249738,  249766, 16, 28, 28, }, // Sakhalin
    {  238, 1579,  249794,  249808,  249831, 14, 23, 15, }, // Samara
    {  238, 1596,  150348,  249846,  249871, 13, 25, 22, }, // Samoa
    {  238, 1602,       0,  150413,       0,  0, 18,  0, }, // Seychelles
    {  238, 1624,       0,  249893,       0,  0, 29,  0, }, // Singapore
    {  238, 1634,       0,  150461,       0,  0, 23,  0, }, // Solomon
    {  238, 1642,       0,  249922,       0,  0, 21,  0, }, // South_Georgia
    {  238, 1656,       0,  150505,       0,  0, 16,  0, }, // Suriname
    {  238, 1676,       0,  150521,       0,  0, 13,  0, }, // Syowa
    {  238, 1682,       0,  150534,       0,  0, 14,  0, }, // Tahiti
    {  238, 1689,  150548,  249943,  249969, 14, 26, 23, }, // Taipei
    {  238, 1696,       0,  150616,       0,  0, 18,  0, }, // Tajikistan
    {  238, 1724,       0,  150634,       0,  0, 15,  0, }, // Tokelau
    {  238, 1732,  150649,  249992,  250017, 13, 25, 25, }, // Tonga
    {  238, 1738,       0,  150714,       0,  0, 13,  0, }, // Truk
    {  238, 1750,  150727,  250042,  250074, 20, 32, 32, }, // Turkmenistan
    {  238, 1763,       0,  150813,       0,  0, 14,  0, }, // Tuvalu
    {  238, 1777,  150827,  250106,  250133, 15, 27, 27, }, // Uruguay
    {  238, 1792,  150898,  250160,  250190, 18, 30, 30, }, // Uzbekistan
    {  238, 1803,  150978,  250220,  250247, 15, 27, 27, }, // Vanuatu
    {  238, 1811,       0,  151049,       0,  0, 17,  0, }, // Venezuela
    {  238, 1821,  151066,  250274,  250305, 19, 31, 31, }, // Vladivostok
    {  238, 1833,  151149,  250336,  250365, 17, 29, 29, }, // Volgograd
    {  238, 1843,       0,  151226,       0,  0, 14,  0, }, // Vostok
    {  238, 1850,       0,  151240,       0,  0, 19,  0, }, // Wake
    {  238, 1855,       0,  151259,       0,  0, 23,  0, }, // Wallis
    {  238, 1862,  151282,  250394,  250421, 15, 27, 27, }, // Yakutsk
    {  238, 1870,  151353,  250448,  250481, 21, 33, 33, }, // Yekaterinburg
    {  238, 1892,       0,  151442,       0,  0, 13,  0, }, // Yukon
    {  239,    0,  250514,  250524,  250542, 10, 18, 14, }, // Acre Finnish/Latin/Finland
    {  239,    5,       0,  250556,       0,  0, 17,  0, }, // Afghanistan
    {  239,   17,       0,  250573,       0,  0, 18,  0, }, // Africa_Central
    {  239,   32,       0,  250591,       0,  0, 16,  0, }, // Africa_Eastern
    {  239,   65,       0,  250607,       0,  0, 18,  0, }, // Africa_Southern
    {  239,   81,  250625,  250643,  250669, 18, 26, 22, }, // Africa_Western
    {  239,  107,  250691,  250703,  250723, 12, 20, 16, }, // Alaska
    {  239,  128,  250739,  250751,  250771, 12, 20, 16, }, // Almaty
    {  239,  135,  250787,  250800,  250821, 13, 21, 17, }, // Amazon
    {  239,  142,  250838,  250864,  250898, 26, 34, 30, }, // America_Central
    {  239,  158,  250928,  250953,  250986, 25, 33, 29, }, // America_Eastern
    {  239,  174,  251015,  251033,  251059, 18, 26, 22, }, // America_Mountain
    {  239,  191,  251081,  251110,  251147, 29, 37, 33, }, // America_Pacific
    {  239,  207,  251180,  251193,  251214, 13, 21, 17, }, // Anadyr
    {  239,  214,  251231,  251241,  251259, 10, 18, 14, }, // Apia
    {  239,  219,  251273,  251285,  251305, 12, 20, 16, }, // Aqtau
    {  239,  225,  251321,  251333,  251353, 12, 20, 16, }, // Aqtobe
    {  239,  232,  251369,  251387,  251413, 18, 26, 22, }, // Arabian
    {  239,  240,  251435,  251451,  251475, 16, 24, 20, }, // Argentina
    {  239,  250,  251495,  251517,  251547, 22, 30, 26, }, // Argentina_Western
    {  239,  268,  251573,  251586,  251607, 13, 21, 17, }, // Armenia
    {  239,  286,  251624,  251645,  251674, 21, 29, 25, }, // Atlantic
    {  239,  295,  251699,  251720,  251749, 21, 29, 25, }, // Australia_Central
    {  239,  313,  251774,  251804,  251842, 30, 38, 34, }, // Australia_CentralWestern
    {  239,  338,  251876,  251895,  251922, 19, 27, 23, }, // Australia_Eastern
    {  239,  356,  251945,  251966,  251995, 21, 29, 25, }, // Australia_Western
    {  239,  374,  252020,  252038,  252064, 18, 26, 22, }, // Azerbaijan
    {  239,  385,  252086,  252098,  252118, 12, 20, 16, }, // Azores
    {  239,  397,  252134,  252151,  252176, 17, 25, 21, }, // Bangladesh
    {  239,  415,       0,  252197,       0,  0, 13,  0, }, // Bhutan
    {  239,  422,       0,  252210,       0,  0, 13,  0, }, // Bolivia
    {  239,  437,  252223,  252237,  252259, 14, 22, 18, }, // Brasilia
    {  239,  454,       0,  252277,       0,  0, 12,  0, }, // Brunei
    {  239,  461,  252289,  252304,  252327, 15, 23, 19, }, // Cape_Verde
    {  239,  472,       0,  252346,       0,  0, 11,  0, }, // Casey
    {  239,  478,       0,  252357,       0,  0, 14,  0, }, // Chamorro
    {  239,  487,  252371,  252385,  252407, 14, 22, 18, }, // Chatham
    {  239,  495,  252425,  252436,  252455, 11, 19, 15, }, // Chile
    {  239,  501,  252470,  252481,  252500, 11, 19, 15, }, // China
    {  239,  507,       0,  252515,       0,  0, 16,  0, }, // Christmas
    {  239,  517,       0,  252531,       0,  0, 18,  0, }, // Cocos
    {  239,  523,  252549,  252563,  252585, 14, 22, 18, }, // Colombia
    {  239,  532,  252603,  252621,  252647, 18, 26, 22, }, // Cook
    {  239,  537,  252669,  252680,  252699, 11, 19, 15, }, // Cuba
    {  239,  548,       0,  252714,       0,  0, 12,  0, }, // Davis
    {  239,  564,       0,  252726,       0,  0, 22,  0, }, // DumontDUrville
    {  239,  601,       0,  252748,       0,  0, 16,  0, }, // East_Timor
    {  239,  612,  252764,  252783,  252810, 19, 27, 23, }, // Easter
    {  239,  619,       0,  252833,       0,  0, 14,  0, }, // Ecuador
    {  239,  627,  252847,  252866,  252893, 19, 27, 23, }, // Europe_Central
    {  239,  642,  252916,  252933,  252958, 17, 25, 21, }, // Europe_Eastern
    {  239,  657,       0,  252979,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  239,  680,  253003,  253022,  253049, 19, 27, 23, }, // Europe_Western
    {  239,  695,  253072,  253094,  253124, 22, 30, 26, }, // Falkland
    {  239,  704,  253150,  253161,  253180, 11, 19, 15, }, // Fiji
    {  239,  709,       0,  253195,       0,  0, 21,  0, }, // French_Guiana
    {  239,  723,       0,  253216,       0,  0, 48,  0, }, // French_Southern
    {  239,  746,       0,  253264,       0,  0, 21,  0, }, // Galapagos
    {  239,  756,       0,  253285,       0,  0, 19,  0, }, // Gambier
    {  239,  764,  253304,  253317,  253338, 13, 21, 17, }, // Georgia
    {  239,  772,       0,  253355,       0,  0, 19,  0, }, // Gilbert_Islands
    {  239,  788,       0,  253374,       0,  0, 24,  0, }, // GMT
    {  239,  830,  253398,  253417,  253444, 19, 27, 23, }, // Greenland_Eastern
    {  239,  848,  253467,  253488,  253517, 21, 29, 25, }, // Greenland_Western
    {  239,  866,       0,  253542,       0,  0, 11,  0, }, // Guam
    {  239,  871,       0,  253553,       0,  0, 31,  0, }, // Gulf
    {  239,  876,       0,  253584,       0,  0, 12,  0, }, // Guyana
    {  239,  883,  253596,  253619,  253650, 23, 31, 27, }, // Hawaii_Aleutian
    {  239,  899,  253677,  253692,  253715, 15, 23, 19, }, // Hong_Kong
    {  239,  909,  253734,  253745,  253764, 11, 19, 15, }, // Hovd
    {  239,  914,       0,  253779,       0,  0, 11,  0, }, // India
    {  239,  920,       0,  253790,       0,  0, 22,  0, }, // Indian_Ocean
    {  239,  933,       0,  253812,       0,  0, 15,  0, }, // Indochina
    {  239,  943,       0,  253827,       0,  0, 21,  0, }, // Indonesia_Central
    {  239,  961,       0,  253848,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  239,  979,       0,  253867,       0,  0, 21,  0, }, // Indonesia_Western
    {  239,  997,  253888,  253899,  253918, 11, 19, 15, }, // Iran
    {  239, 1008,  253933,  253947,  253969, 14, 22, 18, }, // Irkutsk
    {  239, 1016,  253987,  254000,  254021, 13, 21, 17, }, // Israel
    {  239, 1023,  254038,  254050,  254070, 12, 20, 16, }, // Japan
    {  239, 1029,  254086,  254101,  254124, 15, 23, 19, }, // Kamchatka
    {  239, 1047,       0,  254143,       0,  0, 16,  0, }, // Kazakhstan
    {  239, 1058,       0,  254159,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  239, 1077,       0,  254179,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  239, 1106,  254201,  254212,  254231, 11, 19, 15, }, // Korea
    {  239, 1112,       0,  254246,       0,  0, 12,  0, }, // Kosrae
    {  239, 1119,  254258,  254276,  254302, 18, 26, 22, }, // Krasnoyarsk
    {  239, 1151,       0,  254324,       0,  0, 14,  0, }, // Kyrgystan
    {  239, 1161,       0,  254338,       0,  0, 15,  0, }, // Lanka
    {  239, 1175,       0,  254353,       0,  0, 16,  0, }, // Line_Islands
    {  239, 1188,  254369,  254384,  254407, 15, 23, 19, }, // Lord_Howe
    {  239, 1198,  254426,  254437,  254456, 11, 19, 15, }, // Macau
    {  239, 1204,  254471,  254485,  254507, 14, 22, 18, }, // Magadan
    {  239, 1219,       0,  254525,       0,  0, 13,  0, }, // Malaysia
    {  239, 1228,       0,  254538,       0,  0, 16,  0, }, // Maldives
    {  239, 1237,       0,  254554,       0,  0, 21,  0, }, // Marquesas
    {  239, 1247,       0,  254575,       0,  0, 22,  0, }, // Marshall_Islands
    {  239, 1264,  254597,  254614,  254639, 17, 25, 21, }, // Mauritius
    {  239, 1274,       0,  254660,       0,  0, 13,  0, }, // Mawson
    {  239, 1281,  254673,  254698,  254731, 25, 33, 29, }, // Mexico_Pacific
    {  239, 1296,  254760,  254777,  254802, 17, 25, 21, }, // Mongolia
    {  239, 1305,  254823,  254836,  254857, 13, 21, 17, }, // Moscow
    {  239, 1312,       0,  254874,       0,  0, 14,  0, }, // Myanmar
    {  239, 1320,       0,  254888,       0,  0, 11,  0, }, // Nauru
    {  239, 1326,       0,  254899,       0,  0, 12,  0, }, // Nepal
    {  239, 1332,  254911,  254932,  254961, 21, 29, 25, }, // New_Caledonia
    {  239, 1346,  254986,  255006,  255034, 20, 28, 24, }, // New_Zealand
    {  239, 1358,  255058,  255077,  255104, 19, 27, 23, }, // Newfoundland
    {  239, 1371,       0,  255127,       0,  0, 10,  0, }, // Niue
    {  239, 1376,  255137,  255157,  255185, 20, 28, 24, }, // Norfolk
    {  239, 1384,  255209,  255234,  255267, 25, 33, 29, }, // Noronha
    {  239, 1392,       0,  255296,       0,  0, 23,  0, }, // North_Mariana
    {  239, 1406,  255319,  255337,  255363, 18, 26, 22, }, // Novosibirsk
    {  239, 1418,  255385,  255396,  255415, 11, 19, 15, }, // Omsk
    {  239, 1428,  255430,  255445,  255468, 15, 23, 19, }, // Pakistan
    {  239, 1437,       0,  255487,       0,  0, 11,  0, }, // Palau
    {  239, 1443,       0,  255498,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  239, 1460,  255522,  255536,  255558, 14, 22, 18, }, // Paraguay
    {  239, 1469,  255576,  255586,  255604, 10, 18, 14, }, // Peru
    {  239, 1474,  255618,  255635,  255660, 17, 25, 21, }, // Philippines
    {  239, 1486,       0,  255681,       0,  0, 19,  0, }, // Phoenix_Islands
    {  239, 1502,  255700,  255732,  255772, 32, 40, 36, }, // Pierre_Miquelon
    {  239, 1518,       0,  255808,       0,  0, 15,  0, }, // Pitcairn
    {  239, 1527,       0,  255823,       0,  0, 13,  0, }, // Ponape
    {  239, 1534,       0,  255836,       0,  0, 16,  0, }, // Pyongyang
    {  239, 1544,  255852,  255867,  255890, 15, 23, 19, }, // Qyzylorda
    {  239, 1554,       0,  255909,       0,  0, 14,  0, }, // Reunion
    {  239, 1562,       0,  255923,       0,  0, 13,  0, }, // Rothera
    {  239, 1570,  255936,  255950,  255972, 14, 22, 18, }, // Sakhalin
    {  239, 1579,  255990,  256002,  256022, 12, 20, 16, }, // Samara
    {  239, 1596,  256038,  256049,  256068, 11, 19, 15, }, // Samoa
    {  239, 1602,       0,  256083,       0,  0, 16,  0, }, // Seychelles
    {  239, 1624,       0,  256099,       0,  0, 15,  0, }, // Singapore
    {  239, 1634,       0,  256114,       0,  0, 19,  0, }, // Solomon
    {  239, 1642,       0,  256133,       0,  0, 19,  0, }, // South_Georgia
    {  239, 1656,       0,  256152,       0,  0, 14,  0, }, // Suriname
    {  239, 1676,       0,  256166,       0,  0, 11,  0, }, // Syowa
    {  239, 1682,       0,  256177,       0,  0, 12,  0, }, // Tahiti
    {  239, 1689,  256189,  256201,  256221, 12, 20, 16, }, // Taipei
    {  239, 1696,       0,  256237,       0,  0, 18,  0, }, // Tajikistan
    {  239, 1724,       0,  256255,       0,  0, 13,  0, }, // Tokelau
    {  239, 1732,  256268,  256279,  256298, 11, 19, 15, }, // Tonga
    {  239, 1738,       0,  256313,       0,  0, 12,  0, }, // Truk
    {  239, 1750,  256325,  256344,  256371, 19, 27, 23, }, // Turkmenistan
    {  239, 1763,       0,  256394,       0,  0, 12,  0, }, // Tuvalu
    {  239, 1777,  256406,  256419,  256440, 13, 21, 17, }, // Uruguay
    {  239, 1792,  256457,  256474,  256499, 17, 25, 21, }, // Uzbekistan
    {  239, 1803,  256520,  256533,  256554, 13, 21, 17, }, // Vanuatu
    {  239, 1811,       0,  256571,       0,  0, 15,  0, }, // Venezuela
    {  239, 1821,  256586,  256604,  256630, 18, 26, 22, }, // Vladivostok
    {  239, 1833,  256652,  256668,  256692, 16, 24, 20, }, // Volgograd
    {  239, 1843,       0,  256712,       0,  0, 13,  0, }, // Vostok
    {  239, 1850,       0,  256725,       0,  0, 10,  0, }, // Wake
    {  239, 1855,       0,  256735,       0,  0, 24,  0, }, // Wallis
    {  239, 1862,  256759,  256773,  256795, 14, 22, 18, }, // Yakutsk
    {  239, 1870,  256813,  256833,  256861, 20, 28, 24, }, // Yekaterinburg
    {  239, 1892,       0,  256885,       0,  0, 12,  0, }, // Yukon
    {  240,    0,  256897,  256912,  256935, 15, 23, 21, }, // Acre French/Latin/France
    {  240,    5,       0,  256956,       0,  0, 22,  0, }, // Afghanistan
    {  240,   17,       0,  256978,       0,  0, 32,  0, }, // Africa_Central
    {  240,   32,       0,  257010,       0,  0, 32,  0, }, // Africa_Eastern
    {  240,   65,       0,  257042,       0,  0, 35,  0, }, // Africa_Southern
    {  240,   81,  257077,  257103,  257137, 26, 34, 32, }, // Africa_Western
    {  240,  107,  257169,  257186,  257211, 17, 25, 23, }, // Alaska
    {  240,  128,  257234,  257250,  257274, 16, 24, 22, }, // Almaty
    {  240,  135,  257296,  257315,  257342, 19, 27, 25, }, // Amazon
    {  240,  142,  257367,  257397,  257435, 30, 38, 36, }, // America_Central
    {  240,  158,  257471,  257500,  257537, 29, 37, 35, }, // America_Eastern
    {  240,  174,  257572,  257591,  257618, 19, 27, 25, }, // America_Mountain
    {  240,  191,  257643,  257676,  257717, 33, 41, 39, }, // America_Pacific
    {  240,  207,  257756,  257770,  257792, 14, 22, 20, }, // Anadyr
    {  240,  214,  257812,  257824,  257844, 12, 20, 18, }, // Apia
    {  240,  219,  257862,  257876,  257898, 14, 22, 20, }, // Aqtau
    {  240,  225,  257918,  257932,  257954, 14, 22, 20, }, // Aqtobe
    {  240,  232,  257974,  257991,  258016, 17, 25, 23, }, // Arabian
    {  240,  240,  258039,  258059,  258084, 20, 25, 26, }, // Argentina
    {  240,  250,  258110,  258135,  258168, 25, 33, 31, }, // Argentina_Western
    {  240,  268,  258199,  258217,  258243, 18, 26, 21, }, // Armenia
    {  240,  286,  258264,  258285,  258314, 21, 29, 27, }, // Atlantic
    {  240,  295,  258341,  258371,  258409, 30, 38, 36, }, // Australia_Central
    {  240,  313,  258445,  258481,  258525, 36, 44, 42, }, // Australia_CentralWestern
    {  240,  338,  258567,  258596,  258633, 29, 37, 35, }, // Australia_Eastern
    {  240,  356,  258668,  258699,  258738, 31, 39, 37, }, // Australia_Western
    {  240,  374,  258775,  258797,  258827, 22, 30, 25, }, // Azerbaijan
    {  240,  385,  258852,  258868,  258892, 16, 24, 22, }, // Azores
    {  240,  397,  258914,  258933,  258960, 19, 27, 25, }, // Bangladesh
    {  240,  415,       0,  258985,       0,  0, 16,  0, }, // Bhutan
    {  240,  422,       0,  259001,       0,  0, 16,  0, }, // Bolivia
    {  240,  437,  259017,  259034,  259059, 17, 25, 23, }, // Brasilia
    {  240,  454,       0,  259082,       0,  0, 15,  0, }, // Brunei
    {  240,  461,  259097,  259114,  259139, 17, 25, 23, }, // Cape_Verde
    {  240,  472,       0,  259162,       0,  0, 14,  0, }, // Casey
    {  240,  478,       0,  259176,       0,  0, 18,  0, }, // Chamorro
    {  240,  487,  259194,  259216,  259246, 22, 30, 28, }, // Chatham
    {  240,  495,  259274,  259288,  259310, 14, 22, 20, }, // Chile
    {  240,  501,  259330,  259347,  259372, 17, 25, 20, }, // China
    {  240,  507,       0,  259392,       0,  0, 24,  0, }, // Christmas
    {  240,  517,       0,  259416,       0,  0, 20,  0, }, // Cocos
    {  240,  523,  259436,  259453,  259478, 17, 25, 23, }, // Colombia
    {  240,  532,  259501,  259520,  259547, 19, 27, 25, }, // Cook
    {  240,  537,  259572,  259585,  259606, 13, 21, 19, }, // Cuba
    {  240,  548,       0,  259625,       0,  0, 14,  0, }, // Davis
    {  240,  564,       0,  259639,       0,  0, 25,  0, }, // DumontDUrville
    {  240,  601,       0,  259664,       0,  0, 23,  0, }, // East_Timor
    {  240,  612,  259687,  259711,  259743, 24, 32, 30, }, // Easter
    {  240,  619,       0,  259773,       0,  0, 19,  0, }, // Ecuador
    {  240,  627,  259792,  259815,  259846, 23, 31, 29, }, // Europe_Central
    {  240,  642,  259875,  259898,  259929, 23, 31, 29, }, // Europe_Eastern
    {  240,  657,       0,  259958,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  240,  680,  259978,  260003,  260036, 25, 33, 31, }, // Europe_Western
    {  240,  695,  260067,  260091,  260123, 24, 32, 30, }, // Falkland
    {  240,  704,  260153,  260173,  260201, 20, 28, 26, }, // Fiji
    {  240,  709,       0,  260227,       0,  0, 28,  0, }, // French_Guiana
    {  240,  723,       0,  260255,       0,  0, 53,  0, }, // French_Southern
    {  240,  746,       0,  260308,       0,  0, 24,  0, }, // Galapagos
    {  240,  756,       0,  260332,       0,  0, 22,  0, }, // Gambier
    {  240,  764,  260354,  260373,  260400, 19, 27, 22, }, // Georgia
    {  240,  772,       0,  260422,       0,  0, 22,  0, }, // Gilbert_Islands
    {  240,  788,       0,  260444,       0,  0, 26,  0, }, // GMT
    {  240,  830,  260470,  260497,  260532, 27, 35, 33, }, // Greenland_Eastern
    {  240,  848,  260565,  260594,  260631, 29, 37, 35, }, // Greenland_Western
    {  240,  866,       0,  260666,       0,  0, 13,  0, }, // Guam
    {  240,  871,       0,  260679,       0,  0, 14,  0, }, // Gulf
    {  240,  876,       0,  260693,       0,  0, 15,  0, }, // Guyana
    {  240,  883,  260708,  260736,  260772, 28, 36, 34, }, // Hawaii_Aleutian
    {  240,  899,  260806,  260824,  260850, 18, 26, 24, }, // Hong_Kong
    {  240,  909,  260874,  260887,  260908, 13, 21, 19, }, // Hovd
    {  240,  914,       0,  260927,       0,  0, 15,  0, }, // India
    {  240,  920,       0,  260942,       0,  0, 23,  0, }, // Indian_Ocean
    {  240,  933,       0,  260965,       0,  0, 17,  0, }, // Indochina
    {  240,  943,       0,  260982,       0,  0, 26,  0, }, // Indonesia_Central
    {  240,  961,       0,  261008,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  240,  979,       0,  261033,       0,  0, 27,  0, }, // Indonesia_Western
    {  240,  997,  261060,  261075,  261095, 15, 20, 18, }, // Iran
    {  240, 1008,  261113,  261129,  261153, 16, 24, 22, }, // Irkutsk
    {  240, 1016,  261175,  261189,  261211, 14, 22, 20, }, // Israel
    {  240, 1023,  261231,  261245,  261267, 14, 22, 20, }, // Japan
    {  240, 1029,  261287,  261320,  261361, 33, 41, 39, }, // Kamchatka
    {  240, 1047,       0,  261400,       0,  0, 19,  0, }, // Kazakhstan
    {  240, 1058,       0,  261419,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  240, 1077,       0,  261447,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  240, 1106,  261477,  261494,  261519, 17, 25, 20, }, // Korea
    {  240, 1112,       0,  261539,       0,  0, 15,  0, }, // Kosrae
    {  240, 1119,  261554,  261574,  261602, 20, 28, 26, }, // Krasnoyarsk
    {  240, 1151,       0,  261628,       0,  0, 21,  0, }, // Kyrgystan
    {  240, 1161,       0,  261649,       0,  0, 14,  0, }, // Lanka
    {  240, 1175,       0,  261663,       0,  0, 26,  0, }, // Line_Islands
    {  240, 1188,  261689,  261707,  261733, 18, 26, 24, }, // Lord_Howe
    {  240, 1198,  261757,  261771,  261793, 14, 22, 20, }, // Macau
    {  240, 1204,  261813,  261829,  261853, 16, 24, 22, }, // Magadan
    {  240, 1219,       0,  261875,       0,  0, 20,  0, }, // Malaysia
    {  240, 1228,       0,  261895,       0,  0, 18,  0, }, // Maldives
    {  240, 1237,       0,  261913,       0,  0, 24,  0, }, // Marquesas
    {  240, 1247,       0,  261937,       0,  0, 23,  0, }, // Marshall_Islands
    {  240, 1264,  261960,  261976,  262000, 16, 24, 22, }, // Mauritius
    {  240, 1274,       0,  262022,       0,  0, 15,  0, }, // Mawson
    {  240, 1281,  262037,  262064,  262099, 27, 35, 33, }, // Mexico_Pacific
    {  240, 1296,  262132,  262151,  262178, 19, 27, 25, }, // Mongolia
    {  240, 1305,  262203,  262218,  262241, 15, 23, 21, }, // Moscow
    {  240, 1312,       0,  262262,       0,  0, 16,  0, }, // Myanmar
    {  240, 1320,       0,  262278,       0,  0, 14,  0, }, // Nauru
    {  240, 1326,       0,  262292,       0,  0, 14,  0, }, // Nepal
    {  240, 1332,  262306,  262336,  262374, 30, 38, 33, }, // New_Caledonia
    {  240, 1346,  262407,  262435,  262471, 28, 36, 34, }, // New_Zealand
    {  240, 1358,  262505,  262525,  262553, 20, 28, 26, }, // Newfoundland
    {  240, 1371,       0,  262579,       0,  0, 13,  0, }, // Niue
    {  240, 1376,  262592,  262614,  262644, 22, 30, 28, }, // Norfolk
    {  240, 1384,  262672,  262700,  262736, 28, 36, 34, }, // Noronha
    {  240, 1392,       0,  262770,       0,  0, 32,  0, }, // North_Mariana
    {  240, 1406,  262802,  262823,  262852, 21, 29, 27, }, // Novosibirsk
    {  240, 1418,  262879,  262892,  262913, 13, 21, 19, }, // Omsk
    {  240, 1428,  262932,  262949,  262974, 17, 25, 23, }, // Pakistan
    {  240, 1437,       0,  262997,       0,  0, 16,  0, }, // Palau
    {  240, 1443,       0,  263013,       0,  0, 37,  0, }, // Papua_New_Guinea
    {  240, 1460,  263050,  263067,  263092, 17, 25, 23, }, // Paraguay
    {  240, 1469,  263115,  263129,  263151, 14, 22, 20, }, // Peru
    {  240, 1474,  263171,  263192,  263221, 21, 29, 27, }, // Philippines
    {  240, 1486,       0,  263248,       0,  0, 22,  0, }, // Phoenix_Islands
    {  240, 1502,  263270,  263303,  263344, 33, 41, 39, }, // Pierre_Miquelon
    {  240, 1518,       0,  263383,       0,  0, 23,  0, }, // Pitcairn
    {  240, 1527,       0,  263406,       0,  0, 25,  0, }, // Ponape
    {  240, 1534,       0,  263431,       0,  0, 18,  0, }, // Pyongyang
    {  240, 1544,  263449,  263467,  263493, 18, 26, 24, }, // Qyzylorda
    {  240, 1554,       0,  263517,       0,  0, 19,  0, }, // Reunion
    {  240, 1562,       0,  263536,       0,  0, 16,  0, }, // Rothera
    {  240, 1570,  263552,  263570,  263596, 18, 26, 24, }, // Sakhalin
    {  240, 1579,  263620,  263635,  263658, 15, 23, 21, }, // Samara
    {  240, 1596,  263679,  263694,  263717, 15, 23, 21, }, // Samoa
    {  240, 1602,       0,  263738,       0,  0, 20,  0, }, // Seychelles
    {  240, 1624,       0,  263758,       0,  0, 18,  0, }, // Singapore
    {  240, 1634,       0,  263776,       0,  0, 22,  0, }, // Solomon
    {  240, 1642,       0,  263798,       0,  0, 23,  0, }, // South_Georgia
    {  240, 1656,       0,  263821,       0,  0, 17,  0, }, // Suriname
    {  240, 1676,       0,  263838,       0,  0, 14,  0, }, // Syowa
    {  240, 1682,       0,  263852,       0,  0, 15,  0, }, // Tahiti
    {  240, 1689,  263867,  263882,  263905, 15, 23, 21, }, // Taipei
    {  240, 1696,       0,  263926,       0,  0, 20,  0, }, // Tajikistan
    {  240, 1724,       0,  263946,       0,  0, 16,  0, }, // Tokelau
    {  240, 1732,  263962,  263977,  264000, 15, 23, 20, }, // Tonga
    {  240, 1738,       0,  264020,       0,  0, 14,  0, }, // Truk
    {  240, 1750,  264034,  264055,  264084, 21, 29, 27, }, // Turkmenistan
    {  240, 1763,       0,  264111,       0,  0, 16,  0, }, // Tuvalu
    {  240, 1777,  264127,  264145,  264171, 18, 26, 24, }, // Uruguay
    {  240, 1792,  264195,  264217,  264247, 22, 30, 28, }, // Uzbekistan
    {  240, 1803,  264275,  264291,  264315, 16, 24, 22, }, // Vanuatu
    {  240, 1811,       0,  264337,       0,  0, 18,  0, }, // Venezuela
    {  240, 1821,  264355,  264375,  264403, 20, 28, 26, }, // Vladivostok
    {  240, 1833,  264429,  264447,  264473, 18, 26, 24, }, // Volgograd
    {  240, 1843,       0,  264497,       0,  0, 15,  0, }, // Vostok
    {  240, 1850,       0,  264512,       0,  0, 19,  0, }, // Wake
    {  240, 1855,       0,  264531,       0,  0, 25,  0, }, // Wallis
    {  240, 1862,  264556,  264573,  264598, 17, 25, 23, }, // Yakutsk
    {  240, 1870,  264621,  264642,  264671, 21, 29, 27, }, // Yekaterinburg
    {  240, 1892,       0,  264698,       0,  0, 22,  0, }, // Yukon
    {  247,    0,  256897,  256912,  264720, 15, 23, 23, }, // Acre French/Latin/Canada
    {  247,   17,       0,  264743,       0,  0, 24,  0, }, // Africa_Central
    {  247,   32,       0,  264767,       0,  0, 25,  0, }, // Africa_Eastern
    {  247,   65,       0,  264792,       0,  0, 30,  0, }, // Africa_Southern
    {  247,   81,  257077,  257103,  264822, 26, 34, 34, }, // Africa_Western
    {  247,  107,  257169,  257186,  264856, 17, 25, 25, }, // Alaska
    {  247,  128,  257234,  257250,  264881, 16, 24, 24, }, // Almaty
    {  247,  135,  257296,  257315,  264905, 19, 27, 27, }, // Amazon
    {  247,  142,  260982,  264932,  264955, 15, 23, 23, }, // America_Central
    {  247,  158,  257471,  257500,  264978, 14, 22, 22, }, // America_Eastern
    {  247,  174,  257572,  257591,  265000, 19, 27, 27, }, // America_Mountain
    {  247,  191,  257643,  257676,  265027, 18, 26, 26, }, // America_Pacific
    {  247,  207,  257756,  257770,  265053, 14, 22, 22, }, // Anadyr
    {  247,  214,  257812,  257824,  265075, 12, 20, 20, }, // Apia
    {  247,  219,  257862,  257876,  265095, 14, 22, 22, }, // Aqtau
    {  247,  225,  257918,  257932,  265117, 14, 22, 22, }, // Aqtobe
    {  247,  232,  257974,  257991,  265139, 17, 25, 25, }, // Arabian
    {  247,  240,  258039,  258059,  265164, 20, 25, 28, }, // Argentina
    {  247,  250,  258110,  258135,  265192, 25, 33, 33, }, // Argentina_Western
    {  247,  268,  258199,  258217,  265225, 18, 26, 23, }, // Armenia
    {  247,  286,  258264,  258285,  265248, 21, 29, 29, }, // Atlantic
    {  247,  295,  258341,  258371,  265277, 30, 38, 38, }, // Australia_Central
    {  247,  313,  258445,  258481,  265315, 36, 44, 44, }, // Australia_CentralWestern
    {  247,  338,  258567,  258596,  265359, 29, 37, 37, }, // Australia_Eastern
    {  247,  356,  258668,  258699,  265396, 31, 39, 39, }, // Australia_Western
    {  247,  374,  258775,  258797,  265435, 22, 30, 27, }, // Azerbaijan
    {  247,  385,  258852,  258868,  265462, 16, 24, 24, }, // Azores
    {  247,  397,  258914,  258933,  265486, 19, 27, 27, }, // Bangladesh
    {  247,  437,  259017,  259034,  265513, 17, 25, 25, }, // Brasilia
    {  247,  454,       0,  265538,       0,  0, 15,  0, }, // Brunei
    {  247,  461,  259097,  259114,  265553, 17, 25, 25, }, // Cape_Verde
    {  247,  495,  259274,  259288,  265578, 14, 22, 22, }, // Chile
    {  247,  501,  265600,  265614,  265636, 14, 22, 22, }, // China
    {  247,  523,  259436,  259453,  265658, 17, 25, 25, }, // Colombia
    {  247,  532,  259501,  259520,  265683, 19, 27, 27, }, // Cook
    {  247,  537,  259572,  259585,  265710, 13, 21, 21, }, // Cuba
    {  247,  612,  259687,  259711,  265731, 24, 32, 32, }, // Easter
    {  247,  627,  265763,  265789,  265823, 26, 34, 34, }, // Europe_Central
    {  247,  642,  265857,  265883,  265917, 26, 34, 34, }, // Europe_Eastern
    {  247,  680,  265951,  265979,  266015, 28, 36, 36, }, // Europe_Western
    {  247,  695,  260067,  260091,  266051, 24, 32, 32, }, // Falkland
    {  247,  704,  260153,  260173,  266083, 20, 28, 28, }, // Fiji
    {  247,  709,       0,  266111,       0,  0, 25,  0, }, // French_Guiana
    {  247,  764,  260354,  260373,  266136, 19, 27, 24, }, // Georgia
    {  247,  830,  260470,  260497,  266160, 27, 35, 35, }, // Greenland_Eastern
    {  247,  848,  260565,  260594,  266195, 29, 37, 37, }, // Greenland_Western
    {  247,  883,  266232,  266258,  266292, 26, 34, 34, }, // Hawaii_Aleutian
    {  247,  899,  260806,  260824,  266326, 18, 26, 26, }, // Hong_Kong
    {  247,  909,  260874,  260887,  266352, 13, 21, 21, }, // Hovd
    {  247,  920,       0,  266373,       0,  0, 23,  0, }, // Indian_Ocean
    {  247,  997,  261060,  261075,  266396, 15, 20, 20, }, // Iran
    {  247, 1008,  261113,  261129,  266416, 16, 24, 24, }, // Irkutsk
    {  247, 1016,  261175,  261189,  266440, 14, 22, 22, }, // Israel
    {  247, 1023,  261231,  261245,  266462, 14, 22, 22, }, // Japan
    {  247, 1029,  261287,  261320,  266484, 33, 41, 41, }, // Kamchatka
    {  247, 1106,  261477,  261494,  266525, 17, 25, 22, }, // Korea
    {  247, 1119,  261554,  261574,  266547, 20, 28, 28, }, // Krasnoyarsk
    {  247, 1188,  261689,  261707,  266575, 18, 26, 26, }, // Lord_Howe
    {  247, 1198,  261757,  261771,  266601, 14, 22, 22, }, // Macau
    {  247, 1204,  261813,  261829,  266623, 16, 24, 24, }, // Magadan
    {  247, 1264,  261960,  261976,  266647, 16, 24, 24, }, // Mauritius
    {  247, 1281,  262037,  262064,  266671, 27, 35, 35, }, // Mexico_Pacific
    {  247, 1296,  262132,  262151,  266706, 19, 27, 27, }, // Mongolia
    {  247, 1305,  262203,  262218,  266733, 15, 23, 23, }, // Moscow
    {  247, 1332,  262306,  262336,  266756, 30, 38, 35, }, // New_Caledonia
    {  247, 1346,  262407,  262435,  266791, 28, 36, 36, }, // New_Zealand
    {  247, 1358,  262505,  262525,  266827, 20, 28, 28, }, // Newfoundland
    {  247, 1371,       0,  266855,       0,  0, 14,  0, }, // Niue
    {  247, 1376,  262592,  262614,  266869, 22, 30, 30, }, // Norfolk
    {  247, 1384,  262672,  262700,  266899, 28, 36, 36, }, // Noronha
    {  247, 1406,  262802,  262823,  266935, 21, 29, 29, }, // Novosibirsk
    {  247, 1418,  266964,  266976,  266996, 12, 20, 20, }, // Omsk
    {  247, 1428,  262932,  262949,  267016, 17, 25, 25, }, // Pakistan
    {  247, 1460,  263050,  263067,  267041, 17, 25, 25, }, // Paraguay
    {  247, 1469,  263115,  263129,  267066, 14, 22, 22, }, // Peru
    {  247, 1474,  263171,  263192,  267088, 21, 29, 29, }, // Philippines
    {  247, 1502,  263270,  263303,  267117, 33, 41, 41, }, // Pierre_Miquelon
    {  247, 1554,       0,  267158,       0,  0, 19,  0, }, // Reunion
    {  247, 1570,  263552,  263570,  267177, 18, 26, 26, }, // Sakhalin
    {  247, 1596,  263679,  263694,  267203, 15, 23, 23, }, // Samoa
    {  247, 1689,  263867,  263882,  267226, 15, 23, 23, }, // Taipei
    {  247, 1732,  263962,  263977,  267249, 15, 23, 22, }, // Tonga
    {  247, 1750,  264034,  264055,  267271, 21, 29, 29, }, // Turkmenistan
    {  247, 1777,  264127,  264145,  267300, 18, 26, 26, }, // Uruguay
    {  247, 1792,  264195,  264217,  267326, 22, 30, 30, }, // Uzbekistan
    {  247, 1803,  264275,  264291,  267356, 16, 24, 24, }, // Vanuatu
    {  247, 1821,  264355,  264375,  267380, 20, 28, 28, }, // Vladivostok
    {  247, 1833,  264429,  264447,  267408, 18, 26, 26, }, // Volgograd
    {  247, 1862,  264556,  264573,  267434, 17, 25, 25, }, // Yakutsk
    {  247, 1870,  264621,  264642,  267459, 21, 29, 29, }, // Yekaterinburg
    {  255,  709,       0,  260227,       0,  0, 28,  0, }, // French_Guiana French/Latin/French Guiana
    {  286,  627,  267488,  267509,  267539, 21, 30, 28, }, // Europe_Central Friulian/Latin/Italy
    {  286,  642,  267567,  267589,  267620, 22, 31, 29, }, // Europe_Eastern
    {  286,  680,  267649,  267672,  267704, 23, 32, 30, }, // Europe_Western
    {  286, 1305,  267734,  267747,  267769, 13, 22, 20, }, // Moscow
    {  293,    0,  267789,  267814,  267858, 25, 44, 36, }, // Acre Fulah/Adlam/Guinea
    {  293,    5,       0,  267894,       0,  0, 37,  0, }, // Afghanistan
    {  293,   17,       0,  267931,       0,  0, 46,  0, }, // Africa_Central
    {  293,   32,       0,  267977,       0,  0, 48,  0, }, // Africa_Eastern
    {  293,   65,       0,  268025,       0,  0, 67,  0, }, // Africa_Southern
    {  293,   81,  268092,  268142,  268211, 50, 69, 61, }, // Africa_Western
    {  293,  107,  268272,  268301,  268358, 29, 57, 49, }, // Alaska
    {  293,  128,  268407,  268434,  268480, 27, 46, 38, }, // Almaty
    {  293,  135,  268518,  268547,  268595, 29, 48, 40, }, // Amazon
    {  293,  142,  268635,  268696,  268776, 61, 80, 72, }, // America_Central
    {  293,  158,  268848,  268923,  269017, 75, 94, 86, }, // America_Eastern
    {  293,  174,  269103,  269176,  269268, 73, 92, 61, }, // America_Mountain
    {  293,  191,  269329,  269390,  269470, 61, 80, 72, }, // America_Pacific
    {  293,  207,  269542,  269571,  269619, 29, 48, 40, }, // Anadyr
    {  293,  214,  269659,  269686,  269732, 27, 46, 38, }, // Apia
    {  293,  219,  269770,  269795,  269839, 25, 44, 36, }, // Aqtau
    {  293,  225,  269875,  269904,  269952, 29, 48, 40, }, // Aqtobe
    {  293,  232,  269992,  270023,  270073, 31, 50, 42, }, // Arabian
    {  293,  240,  270115,  270150,  270204, 35, 54, 46, }, // Argentina
    {  293,  250,  270250,  270308,  270385, 58, 77, 69, }, // Argentina_Western
    {  293,  268,  270454,  270487,  270539, 33, 52, 44, }, // Armenia
    {  293,  286,  270583,  270606,  270648, 23, 42, 34, }, // Atlantic
    {  293,  295,  270682,  270734,  270805, 52, 71, 63, }, // Australia_Central
    {  293,  313,  270868,  270941,  271033, 73, 92, 84, }, // Australia_CentralWestern
    {  293,  338,  271117,  271177,  271256, 60, 79, 71, }, // Australia_Eastern
    {  293,  356,  271327,  271389,  271470, 62, 81, 73, }, // Australia_Western
    {  293,  374,  271543,  271578,  271632, 35, 54, 46, }, // Azerbaijan
    {  293,  385,  271678,  271705,  271751, 27, 46, 38, }, // Azores
    {  293,  397,  271789,  271826,  271882, 37, 56, 48, }, // Bangladesh
    {  293,  415,       0,  271930,       0,  0, 27,  0, }, // Bhutan
    {  293,  422,       0,  271957,       0,  0, 33,  0, }, // Bolivia
    {  293,  437,  271990,  272025,  272079, 35, 54, 46, }, // Brasilia
    {  293,  454,       0,  272125,       0,  0, 27,  0, }, // Brunei
    {  293,  461,  272152,  272188,  272243, 36, 55, 47, }, // Cape_Verde
    {  293,  472,       0,  272290,       0,  0, 27,  0, }, // Casey
    {  293,  478,       0,  272317,       0,  0, 48,  0, }, // Chamorro
    {  293,  487,  272365,  272392,  272438, 27, 46, 38, }, // Chatham
    {  293,  495,  272476,  272501,  272545, 25, 44, 36, }, // Chile
    {  293,  501,  272581,  272606,  272650, 25, 44, 36, }, // China
    {  293,  507,       0,  272686,       0,  0, 48,  0, }, // Christmas
    {  293,  517,       0,  272734,       0,  0, 27,  0, }, // Cocos
    {  293,  523,  272761,  272796,  272850, 35, 54, 46, }, // Colombia
    {  293,  532,  272896,  272934,  272991, 38, 57, 60, }, // Cook
    {  293,  537,  273051,  273078,  273124, 27, 46, 38, }, // Cuba
    {  293,  548,       0,  273162,       0,  0, 27,  0, }, // Davis
    {  293,  564,       0,  273189,       0,  0, 42,  0, }, // DumontDUrville
    {  293,  601,       0,  273231,       0,  0, 48,  0, }, // East_Timor
    {  293,  612,  273279,  273317,  273374, 38, 57, 49, }, // Easter
    {  293,  619,       0,  273423,       0,  0, 33,  0, }, // Ecuador
    {  293,  627,  273456,  273500,  273563, 44, 63, 55, }, // Europe_Central
    {  293,  642,  273618,  273666,  273733, 48, 67, 59, }, // Europe_Eastern
    {  293,  657,       0,  273792,       0,  0, 54,  0, }, // Europe_Further_Eastern
    {  293,  680,  273846,  273896,  273965, 50, 69, 61, }, // Europe_Western
    {  293,  695,  274026,  274070,  274133, 44, 63, 55, }, // Falkland
    {  293,  704,  274188,  274213,  274257, 25, 44, 36, }, // Fiji
    {  293,  709,       0,  274293,       0,  0, 42,  0, }, // French_Guiana
    {  293,  723,       0,  274335,       0,  0, 59,  0, }, // French_Southern
    {  293,  746,       0,  274394,       0,  0, 35,  0, }, // Galapagos
    {  293,  756,       0,  274429,       0,  0, 31,  0, }, // Gambier
    {  293,  764,  274460,  274489,  274537, 29, 48, 40, }, // Georgia
    {  293,  772,       0,  274577,       0,  0, 46,  0, }, // Gilbert_Islands
    {  293,  788,       0,  274623,       0,  0, 54,  0, }, // GMT
    {  293,  830,  274677,  274725,  274792, 48, 67, 59, }, // Greenland_Eastern
    {  293,  848,  274851,  274901,  274970, 50, 69, 61, }, // Greenland_Western
    {  293,  866,       0,  275031,       0,  0, 46,  0, }, // Guam
    {  293,  871,       0,  275077,       0,  0, 25,  0, }, // Gulf
    {  293,  876,       0,  274293,       0,  0, 29,  0, }, // Guyana
    {  293,  883,  275102,  275150,  275217, 48, 67, 59, }, // Hawaii_Aleutian
    {  293,  899,  275276,  275308,  275359, 32, 51, 43, }, // Hong_Kong
    {  293,  909,  275402,  275427,  275471, 25, 44, 36, }, // Hovd
    {  293,  914,       0,  275507,       0,  0, 46,  0, }, // India
    {  293,  920,       0,  275553,       0,  0, 42,  0, }, // Indian_Ocean
    {  293,  933,       0,  275595,       0,  0, 31,  0, }, // Indochina
    {  293,  943,       0,  275626,       0,  0, 54,  0, }, // Indonesia_Central
    {  293,  961,       0,  275680,       0,  0, 58,  0, }, // Indonesia_Eastern
    {  293,  979,       0,  275738,       0,  0, 60,  0, }, // Indonesia_Western
    {  293,  997,  275798,  275823,  275867, 25, 44, 36, }, // Iran
    {  293, 1008,  275903,  275936,  275988, 33, 52, 44, }, // Irkutsk
    {  293, 1016,  276032,  276065,  276117, 33, 52, 44, }, // Israel
    {  293, 1023,  276161,  276188,  276234, 27, 46, 38, }, // Japan
    {  293, 1058,       0,  276272,       0,  0, 37,  0, }, // Kazakhstan_Eastern
    {  293, 1077,       0,  276309,       0,  0, 58,  0, }, // Kazakhstan_Western
    {  293, 1106,  276367,  276398,  276448, 31, 50, 42, }, // Korea
    {  293, 1112,       0,  276490,       0,  0, 27,  0, }, // Kosrae
    {  293, 1119,  276517,  276558,  276616, 41, 58, 52, }, // Krasnoyarsk
    {  293, 1151,       0,  276668,       0,  0, 35,  0, }, // Kyrgystan
    {  293, 1175,       0,  276703,       0,  0, 40,  0, }, // Line_Islands
    {  293, 1188,  276743,  276777,  276830, 34, 53, 45, }, // Lord_Howe
    {  293, 1198,  276875,  276904,  276952, 29, 48, 40, }, // Macau
    {  293, 1204,  276992,  277021,  277069, 29, 48, 40, }, // Magadan
    {  293, 1219,       0,  277109,       0,  0, 33,  0, }, // Malaysia
    {  293, 1228,       0,  277142,       0,  0, 33,  0, }, // Maldives
    {  293, 1237,       0,  277175,       0,  0, 31,  0, }, // Marquesas
    {  293, 1247,       0,  277206,       0,  0, 42,  0, }, // Marshall_Islands
    {  293, 1264,  277248,  277277,  277325, 29, 48, 40, }, // Mauritius
    {  293, 1274,       0,  277365,       0,  0, 27,  0, }, // Mawson
    {  293, 1281,  277392,  277434,  277495, 42, 61, 53, }, // Mexico_Pacific
    {  293, 1296,  277548,  277581,  277633, 33, 52, 44, }, // Mongolia
    {  293, 1305,  277677,  277702,  277746, 25, 44, 36, }, // Moscow
    {  293, 1312,       0,  277782,       0,  0, 31,  0, }, // Myanmar
    {  293, 1320,       0,  277813,       0,  0, 25,  0, }, // Nauru
    {  293, 1326,       0,  277838,       0,  0, 25,  0, }, // Nepal
    {  293, 1332,  277863,  277909,  277974, 46, 65, 57, }, // New_Caledonia
    {  293, 1346,  278031,  278067,  278122, 36, 55, 47, }, // New_Zealand
    {  293, 1358,  278169,  278209,  278268, 40, 59, 51, }, // Newfoundland
    {  293, 1371,       0,  278319,       0,  0, 25,  0, }, // Niue
    {  293, 1376,  278344,  278390,  278455, 46, 65, 57, }, // Norfolk
    {  293, 1384,  278512,  278567,  278641, 55, 74, 66, }, // Noronha
    {  293, 1406,  278707,  278744,  278800, 37, 56, 48, }, // Novosibirsk
    {  293, 1418,  278848,  278875,  278921, 27, 46, 38, }, // Omsk
    {  293, 1428,  278959,  278992,  279044, 33, 52, 44, }, // Pakistan
    {  293, 1437,       0,  279088,       0,  0, 29,  0, }, // Palau
    {  293, 1443,       0,  279117,       0,  0, 45,  0, }, // Papua_New_Guinea
    {  293, 1460,  279162,  279193,  279243, 31, 50, 42, }, // Paraguay
    {  293, 1469,  279285,  279310,  279354, 25, 44, 59, }, // Peru
    {  293, 1474,  279413,  279444,  279494, 31, 50, 42, }, // Philippines
    {  293, 1486,       0,  279536,       0,  0, 44,  0, }, // Phoenix_Islands
    {  293, 1502,  279580,  279630,  279699, 50, 69, 61, }, // Pierre_Miquelon
    {  293, 1518,       0,  279760,       0,  0, 33,  0, }, // Pitcairn
    {  293, 1527,       0,  279793,       0,  0, 29,  0, }, // Ponape
    {  293, 1534,       0,  279822,       0,  0, 31,  0, }, // Pyongyang
    {  293, 1544,  279853,  279886,  279938, 33, 52, 44, }, // Qyzylorda
    {  293, 1554,       0,  279982,       0,  0, 29,  0, }, // Reunion
    {  293, 1562,       0,  280011,       0,  0, 27,  0, }, // Rothera
    {  293, 1570,  280038,  280069,  280119, 31, 50, 42, }, // Sakhalin
    {  293, 1596,  280161,  280188,  280234, 27, 46, 38, }, // Samoa
    {  293, 1602,       0,  280272,       0,  0, 29,  0, }, // Seychelles
    {  293, 1624,       0,  280301,       0,  0, 52,  0, }, // Singapore
    {  293, 1634,       0,  280353,       0,  0, 48,  0, }, // Solomon
    {  293, 1642,       0,  280401,       0,  0, 40,  0, }, // South_Georgia
    {  293, 1656,       0,  280441,       0,  0, 29,  0, }, // Suriname
    {  293, 1676,       0,  280470,       0,  0, 29,  0, }, // Syowa
    {  293, 1682,       0,  280499,       0,  0, 29,  0, }, // Tahiti
    {  293, 1689,  280528,  280555,  280601, 27, 46, 38, }, // Taipei
    {  293, 1696,       0,  280639,       0,  0, 37,  0, }, // Tajikistan
    {  293, 1724,       0,  280676,       0,  0, 35,  0, }, // Tokelau
    {  293, 1732,  280711,  280736,  280780, 25, 44, 36, }, // Tonga
    {  293, 1738,       0,  280816,       0,  0, 25,  0, }, // Truk
    {  293, 1750,  280841,  280884,  280946, 43, 62, 54, }, // Turkmenistan
    {  293, 1763,       0,  281000,       0,  0, 29,  0, }, // Tuvalu
    {  293, 1777,  281029,  281058,  281106, 29, 48, 40, }, // Uruguay
    {  293, 1792,  281146,  281183,  281239, 37, 56, 48, }, // Uzbekistan
    {  293, 1803,  281287,  281320,  281372, 33, 52, 44, }, // Vanuatu
    {  293, 1811,       0,  281416,       0,  0, 35,  0, }, // Venezuela
    {  293, 1821,  281451,  281486,  281540, 35, 54, 46, }, // Vladivostok
    {  293, 1833,  281586,  281623,  281679, 37, 56, 48, }, // Volgograd
    {  293, 1843,       0,  281727,       0,  0, 27,  0, }, // Vostok
    {  293, 1850,       0,  281754,       0,  0, 40,  0, }, // Wake
    {  293, 1855,       0,  281794,       0,  0, 44,  0, }, // Wallis
    {  293, 1862,  281838,  281871,  281923, 33, 52, 44, }, // Yakutsk
    {  293, 1870,  281967,  282010,  282072, 43, 62, 54, }, // Yekaterinburg
    {  311,    0,  282126,  282133,  282144,  7, 11, 19, }, // Acre Gaelic/Latin/United Kingdom
    {  311,    5,       0,  282163,       0,  0, 15,  0, }, // Afghanistan
    {  311,   17,       0,  282178,       0,  0, 17,  0, }, // Africa_Central
    {  311,   32,       0,  282195,       0,  0, 16,  0, }, // Africa_Eastern
    {  311,   65,       0,  282211,       0,  0, 16,  0, }, // Africa_Southern
    {  311,   81,  282227,  282243,  282263, 16, 20, 28, }, // Africa_Western
    {  311,  107,  282291,  282300,  282313,  9, 13, 21, }, // Alaska
    {  311,  128,  282334,  282343,  282356,  9, 13, 21, }, // Almaty
    {  311,  135,  282377,  282387,  282401, 10, 14, 22, }, // Amazon
    {  311,  142,  282423,  282444,  282469, 21, 25, 33, }, // America_Central
    {  311,  158,  282502,  282522,  282546, 20, 24, 32, }, // America_Eastern
    {  311,  174,  282578,  282598,  282622, 20, 24, 32, }, // America_Mountain
    {  311,  191,  282654,  282673,  282696, 19, 23, 31, }, // America_Pacific
    {  311,  207,  282727,  282736,  282749,  9, 13, 21, }, // Anadyr
    {  311,  214,  282770,  282777,  282788,  7, 11, 19, }, // Apia
    {  311,  219,  282807,  282815,  282827,  8, 12, 20, }, // Aqtau
    {  311,  225,  282847,  282856,  282869,  9, 13, 21, }, // Aqtobe
    {  311,  232,  282890,  282900,  282914, 10, 14, 22, }, // Arabian
    {  311,  240,  282936,  282954,  282976, 18, 22, 30, }, // Argentina
    {  311,  250,  283006,  283033,  283064, 27, 31, 39, }, // Argentina_Western
    {  311,  268,  283103,  283115,  283131, 12, 16, 24, }, // Armenia
    {  311,  286,  283155,  283173,  283195, 18, 22, 30, }, // Atlantic
    {  311,  295,  283225,  283245,  283269, 20, 24, 32, }, // Australia_Central
    {  311,  313,  283301,  283328,  283359, 27, 31, 39, }, // Australia_CentralWestern
    {  311,  338,  283398,  283417,  283440, 19, 23, 31, }, // Australia_Eastern
    {  311,  356,  283471,  283490,  283513, 19, 23, 31, }, // Australia_Western
    {  311,  374,  283544,  283559,  283578, 15, 19, 27, }, // Azerbaijan
    {  311,  385,  283605,  283628,  283655, 23, 27, 35, }, // Azores
    {  311,  397,  283690,  283703,  283720, 13, 17, 25, }, // Bangladesh
    {  311,  415,       0,  283745,       0,  0,  9,  0, }, // Bhutan
    {  311,  422,       0,  283754,       0,  0, 12,  0, }, // Bolivia
    {  311,  437,  283766,  283778,  283794, 12, 16, 24, }, // Brasilia
    {  311,  454,       0,  283818,       0,  0, 25,  0, }, // Brunei
    {  311,  461,  283843,  283859,  283879, 16, 20, 28, }, // Cape_Verde
    {  311,  472,       0,  283907,       0,  0,  9,  0, }, // Casey
    {  311,  478,       0,  283916,       0,  0, 11,  0, }, // Chamorro
    {  311,  487,  283927,  283937,  283951, 10, 14, 22, }, // Chatham
    {  311,  495,  283973,  283983,  283997, 10, 14, 22, }, // Chile
    {  311,  501,  284019,  284029,  284043, 10, 14, 22, }, // China
    {  311,  507,       0,  284065,       0,  0, 20,  0, }, // Christmas
    {  311,  517,       0,  284085,       0,  0, 18,  0, }, // Cocos
    {  311,  523,  284103,  284115,  284131, 12, 16, 24, }, // Colombia
    {  311,  532,  284155,  284171,  284191, 16, 20, 34, }, // Cook
    {  311,  537,  284225,  284232,  284243,  7, 11, 19, }, // Cuba
    {  311,  548,       0,  284262,       0,  0,  9,  0, }, // Davis
    {  311,  564,       0,  284271,       0,  0, 19,  0, }, // DumontDUrville
    {  311,  601,       0,  284290,       0,  0, 17,  0, }, // East_Timor
    {  311,  612,  284307,  284326,  284349, 19, 23, 31, }, // Easter
    {  311,  619,       0,  284380,       0,  0, 12,  0, }, // Ecuador
    {  311,  627,  284392,  284417,  284446, 25, 29, 37, }, // Europe_Central
    {  311,  642,  284483,  284507,  284535, 24, 28, 36, }, // Europe_Eastern
    {  311,  657,       0,  284571,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  311,  680,  284603,  284627,  284655, 24, 28, 36, }, // Europe_Western
    {  311,  695,  284691,  284717,  284747, 26, 30, 38, }, // Falkland
    {  311,  704,  284785,  284792,  284803,  7, 11, 19, }, // Fiji
    {  311,  709,       0,  284822,       0,  0, 23,  0, }, // French_Guiana
    {  311,  723,       0,  284845,       0,  0, 40,  0, }, // French_Southern
    {  311,  746,       0,  284885,       0,  0, 13,  0, }, // Galapagos
    {  311,  756,       0,  284898,       0,  0, 11,  0, }, // Gambier
    {  311,  764,  284909,  284926,  284947, 17, 21, 29, }, // Georgia
    {  311,  772,       0,  284976,       0,  0, 23,  0, }, // Gilbert_Islands
    {  311,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {  311,  802,  284999,  285016,  285037, 17, 21, 29, }, // Greenland
    {  311,  830,  285066,  285089,  285116, 23, 27, 35, }, // Greenland_Eastern
    {  311,  848,  285151,  285174,  285201, 23, 27, 35, }, // Greenland_Western
    {  311,  866,       0,  285236,       0,  0,  7,  0, }, // Guam
    {  311,  871,       0,  285243,       0,  0, 13,  0, }, // Gulf
    {  311,  876,       0,  284822,       0,  0, 12,  0, }, // Guyana
    {  311,  883,  285256,  285291,  285330, 35, 39, 47, }, // Hawaii_Aleutian
    {  311,  899,  285377,  285389,  285405, 12, 16, 24, }, // Hong_Kong
    {  311,  909,  285429,  285436,  285447,  7, 11, 19, }, // Hovd
    {  311,  914,       0,  285466,       0,  0, 17,  0, }, // India
    {  311,  920,       0,  285483,       0,  0, 22,  0, }, // Indian_Ocean
    {  311,  933,       0,  285505,       0,  0, 17,  0, }, // Indochina
    {  311,  943,       0,  285522,       0,  0, 25,  0, }, // Indonesia_Central
    {  311,  961,       0,  285547,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  311,  979,       0,  285571,       0,  0, 24,  0, }, // Indonesia_Western
    {  311,  997,  285595,  285604,  285617,  9, 13, 21, }, // Iran
    {  311, 1008,  285638,  285648,  285662, 10, 14, 22, }, // Irkutsk
    {  311, 1016,  285684,  285694,  285708, 10, 14, 22, }, // Israel
    {  311, 1023,  285730,  285744,  285762, 14, 18, 26, }, // Japan
    {  311, 1029,  285788,  285815,  285846, 27, 31, 39, }, // Kamchatka
    {  311, 1047,       0,  285885,       0,  0, 14,  0, }, // Kazakhstan
    {  311, 1058,       0,  285899,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  311, 1077,       0,  285920,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  311, 1106,  285941,  285951,  285965, 10, 14, 22, }, // Korea
    {  311, 1112,       0,  285987,       0,  0,  9,  0, }, // Kosrae
    {  311, 1119,  285996,  286010,  286028, 14, 18, 26, }, // Krasnoyarsk
    {  311, 1151,       0,  286054,       0,  0, 14,  0, }, // Kyrgystan
    {  311, 1161,       0,  286068,       0,  0,  8,  0, }, // Lanka
    {  311, 1175,       0,  286076,       0,  0, 19,  0, }, // Line_Islands
    {  311, 1188,  286095,  286107,  286123, 12, 16, 24, }, // Lord_Howe
    {  311, 1198,  286147,  286157,  286171, 10, 14, 22, }, // Macau
    {  311, 1204,  286193,  286203,  286217, 10, 14, 22, }, // Magadan
    {  311, 1219,       0,  286239,       0,  0, 14,  0, }, // Malaysia
    {  311, 1228,       0,  286253,       0,  0, 26,  0, }, // Maldives
    {  311, 1237,       0,  286279,       0,  0, 23,  0, }, // Marquesas
    {  311, 1247,       0,  286302,       0,  0, 21,  0, }, // Marshall_Islands
    {  311, 1264,  286323,  286349,  286379, 26, 30, 38, }, // Mauritius
    {  311, 1274,       0,  286417,       0,  0, 10,  0, }, // Mawson
    {  311, 1281,  286427,  286456,  286489, 29, 33, 41, }, // Mexico_Pacific
    {  311, 1296,  286530,  286543,  286560, 13, 17, 25, }, // Mongolia
    {  311, 1305,  286585,  286594,  286607,  9, 13, 21, }, // Moscow
    {  311, 1312,       0,  286628,       0,  0, 10,  0, }, // Myanmar
    {  311, 1320,       0,  286638,       0,  0,  9,  0, }, // Nauru
    {  311, 1326,       0,  286647,       0,  0,  9,  0, }, // Nepal
    {  311, 1332,  286656,  286676,  286700, 20, 24, 32, }, // New_Caledonia
    {  311, 1346,  286732,  286751,  286774, 19, 23, 31, }, // New_Zealand
    {  311, 1358,  286805,  286822,  286843, 17, 21, 29, }, // Newfoundland
    {  311, 1371,       0,  286872,       0,  0,  7,  0, }, // Niue
    {  311, 1376,  286879,  286896,  286917, 17, 21, 29, }, // Norfolk
    {  311, 1384,  286946,  286968,  286994, 22, 26, 34, }, // Noronha
    {  311, 1392,       0,  287028,       0,  0, 34,  0, }, // North_Mariana
    {  311, 1406,  287062,  287076,  287094, 14, 18, 26, }, // Novosibirsk
    {  311, 1418,  287120,  287127,  287138,  7, 11, 19, }, // Omsk
    {  311, 1428,  287157,  287169,  287185, 12, 16, 24, }, // Pakistan
    {  311, 1437,       0,  287209,       0,  0,  9,  0, }, // Palau
    {  311, 1443,       0,  287218,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  311, 1460,  287244,  287257,  287274, 13, 17, 25, }, // Paraguay
    {  311, 1469,  287299,  287307,  287319,  8, 12, 20, }, // Peru
    {  311, 1474,  287339,  287364,  287393, 25, 29, 37, }, // Philippines
    {  311, 1486,       0,  287430,       0,  0, 19,  0, }, // Phoenix_Islands
    {  311, 1502,  287449,  287478,  287511, 29, 33, 41, }, // Pierre_Miquelon
    {  311, 1518,       0,  287552,       0,  0, 17,  0, }, // Pitcairn
    {  311, 1527,       0,  287569,       0,  0, 10,  0, }, // Ponape
    {  311, 1534,       0,  287579,       0,  0, 13,  0, }, // Pyongyang
    {  311, 1544,  287592,  287604,  287620, 12, 16, 24, }, // Qyzylorda
    {  311, 1554,       0,  287644,       0,  0, 10,  0, }, // Reunion
    {  311, 1562,       0,  287654,       0,  0, 10,  0, }, // Rothera
    {  311, 1570,  287664,  287675,  287690, 11, 15, 23, }, // Sakhalin
    {  311, 1579,  287713,  287722,  287735,  9, 13, 21, }, // Samara
    {  311, 1596,  287756,  287766,  287780, 10, 14, 22, }, // Samoa
    {  311, 1602,       0,  287802,       0,  0, 25,  0, }, // Seychelles
    {  311, 1624,       0,  287827,       0,  0, 12,  0, }, // Singapore
    {  311, 1634,       0,  287839,       0,  0, 20,  0, }, // Solomon
    {  311, 1642,       0,  287859,       0,  0, 17,  0, }, // South_Georgia
    {  311, 1656,       0,  287876,       0,  0, 11,  0, }, // Suriname
    {  311, 1676,       0,  287887,       0,  0,  9,  0, }, // Syowa
    {  311, 1682,       0,  287896,       0,  0,  9,  0, }, // Tahiti
    {  311, 1689,  287905,  287914,  287927,  9, 13, 21, }, // Taipei
    {  311, 1696,       0,  287948,       0,  0, 16,  0, }, // Tajikistan
    {  311, 1724,       0,  287964,       0,  0, 10,  0, }, // Tokelau
    {  311, 1732,  287974,  287982,  287994,  8, 12, 20, }, // Tonga
    {  311, 1738,       0,  288014,       0,  0,  8,  0, }, // Truk
    {  311, 1750,  288022,  288038,  288058, 16, 20, 28, }, // Turkmenistan
    {  311, 1763,       0,  288086,       0,  0, 10,  0, }, // Tuvalu
    {  311, 1777,  288096,  288108,  288124, 12, 16, 24, }, // Uruguay
    {  311, 1792,  288148,  288161,  288178, 13, 17, 25, }, // Uzbekistan
    {  311, 1803,  288203,  288213,  288227, 10, 14, 22, }, // Vanuatu
    {  311, 1811,       0,  288249,       0,  0, 17,  0, }, // Venezuela
    {  311, 1821,  288266,  288280,  288298, 14, 18, 26, }, // Vladivostok
    {  311, 1833,  288324,  288336,  288352, 12, 16, 24, }, // Volgograd
    {  311, 1843,       0,  288376,       0,  0,  9,  0, }, // Vostok
    {  311, 1850,       0,  288385,       0,  0, 14,  0, }, // Wake
    {  311, 1855,       0,  288399,       0,  0, 21,  0, }, // Wallis
    {  311, 1862,  288420,  288430,  288444, 10, 14, 22, }, // Yakutsk
    {  311, 1870,  288466,  288482,  288502, 16, 20, 28, }, // Yekaterinburg
    {  311, 1892,       0,  288530,       0,  0,  8,  0, }, // Yukon
    {  312,   17,       0,  288538,       0,  0, 16,  0, }, // Africa_Central Ga/Latin/Ghana
    {  312,   32,       0,  288554,       0,  0, 17,  0, }, // Africa_Eastern
    {  312,   65,       0,  288571,       0,  0, 15,  0, }, // Africa_Southern
    {  312,   81,  288586,  288603,  288633, 17, 30, 32, }, // Africa_Western
    {  312,  107,   18803,  288665,  288687,  9, 22, 24, }, // Alaska
    {  312,  142,  288711,  288732,  288766, 21, 34, 36, }, // America_Central
    {  312,  158,  288802,  288824,  288859, 22, 35, 37, }, // America_Eastern
    {  312,  174,  288896,  288918,  288953, 22, 35, 37, }, // America_Mountain
    {  312,  191,   19028,  288990,  289013, 10, 23, 25, }, // America_Pacific
    {  312,  286,   19343,  289038,  289062, 11, 24, 26, }, // Atlantic
    {  312,  385,   19787,  289088,  289110,  9, 22, 24, }, // Azores
    {  312,  461,  289134,  289147,  289173, 13, 26, 28, }, // Cape_Verde
    {  312,  537,   20343,  289201,  289221,  7, 20, 22, }, // Cuba
    {  312,  723,       0,  289243,       0,  0, 43,  0, }, // French_Southern
    {  312,  788,       0,  289286,       0,  0, 27,  0, }, // GMT
    {  312,  830,  289313,  289333,  289366, 20, 33, 35, }, // Greenland_Eastern
    {  312,  848,  289401,  289421,  289454, 20, 33, 35, }, // Greenland_Western
    {  312,  883,  289489,  289506,  289536, 17, 30, 32, }, // Hawaii_Aleutian
    {  312,  920,       0,  289568,       0,  0, 17,  0, }, // Indian_Ocean
    {  312, 1264,  289585,  289597,  289622, 12, 25, 27, }, // Mauritius
    {  312, 1281,  289649,  289666,  289696, 17, 30, 32, }, // Mexico_Pacific
    {  312, 1358,   22415,  289728,  289756, 15, 28, 30, }, // Newfoundland
    {  312, 1502,  289786,  289810,  289847, 24, 37, 39, }, // Pierre_Miquelon
    {  312, 1554,       0,   23113,       0,  0, 10,  0, }, // Reunion
    {  312, 1602,       0,  289886,       0,  0, 13,  0, }, // Seychelles
    {  313,    0,  289899,       0,       0, 12,  0,  0, }, // Acre Galician/Latin/Spain
    {  313,    5,       0,  289911,       0,  0, 18,  0, }, // Afghanistan
    {  313,   17,       0,  289929,       0,  0, 22,  0, }, // Africa_Central
    {  313,   32,       0,  289951,       0,  0, 23,  0, }, // Africa_Eastern
    {  313,   65,       0,  289974,       0,  0, 25,  0, }, // Africa_Southern
    {  313,   81,  289999,  290024,  290058, 25, 34, 34, }, // Africa_Western
    {  313,  107,  290092,  290106,  290129, 14, 23, 23, }, // Alaska
    {  313,  135,  290152,  290168,  290193, 16, 25, 25, }, // Amazon
    {  313,  142,  290218,  290244,  290279, 26, 35, 35, }, // America_Central
    {  313,  158,  290314,  290345,  290385, 31, 40, 40, }, // America_Eastern
    {  313,  174,  290425,  290458,  290500, 33, 42, 42, }, // America_Mountain
    {  313,  191,  290542,  290576,  290619, 34, 43, 43, }, // America_Pacific
    {  313,  207,  290662,  290676,  290702, 14, 26, 26, }, // Anadyr
    {  313,  214,  290728,  290740,  290761, 12, 21, 21, }, // Apia
    {  313,  232,  290782,  290792,  290811, 10, 19, 19, }, // Arabian
    {  313,  240,  290830,  290847,  290873, 17, 26, 26, }, // Argentina
    {  313,  250,  290899,  290927,  290964, 28, 37, 37, }, // Argentina_Western
    {  313,  268,  291001,  291016,  291040, 15, 24, 24, }, // Armenia
    {  313,  286,  291064,  291081,  291107, 17, 26, 26, }, // Atlantic
    {  313,  295,  291133,  291158,  291192, 25, 34, 34, }, // Australia_Central
    {  313,  313,  291226,  291262,  291307, 36, 45, 45, }, // Australia_CentralWestern
    {  313,  338,  291352,  291378,  291413, 26, 35, 35, }, // Australia_Eastern
    {  313,  356,  291226,  291262,  291307, 28, 37, 37, }, // Australia_Western
    {  313,  374,  291448,  291466,  291493, 18, 27, 27, }, // Azerbaijan
    {  313,  385,  291520,  291535,  291559, 15, 24, 24, }, // Azores
    {  313,  397,  291583,  291601,  291628, 18, 27, 27, }, // Bangladesh
    {  313,  415,       0,  291655,       0,  0, 13,  0, }, // Bhutan
    {  313,  422,       0,  291668,       0,  0, 15,  0, }, // Bolivia
    {  313,  437,  291683,  291699,  291724, 16, 25, 25, }, // Brasilia
    {  313,  454,       0,  291749,       0,  0, 25,  0, }, // Brunei
    {  313,  461,  291774,  291792,  291819, 18, 27, 27, }, // Cape_Verde
    {  313,  478,       0,  291846,       0,  0, 22,  0, }, // Chamorro
    {  313,  487,  291868,  291883,  291907, 15, 24, 24, }, // Chatham
    {  313,  495,  291931,  291944,  291966, 13, 22, 22, }, // Chile
    {  313,  501,  291988,  292001,  292023, 13, 22, 22, }, // China
    {  313,  507,       0,  292045,       0,  0, 22,  0, }, // Christmas
    {  313,  517,       0,  292067,       0,  0, 20,  0, }, // Cocos
    {  313,  523,  292087,  292103,  292128, 16, 25, 25, }, // Colombia
    {  313,  532,  292153,  292172,  292200, 19, 28, 34, }, // Cook
    {  313,  537,  292234,  292246,  292267, 12, 21, 21, }, // Cuba
    {  313,  548,       0,  292288,       0,  0, 13,  0, }, // Davis
    {  313,  564,       0,  292301,       0,  0, 24,  0, }, // DumontDUrville
    {  313,  601,       0,  292325,       0,  0, 19,  0, }, // East_Timor
    {  313,  612,  292344,  292366,  292397, 22, 31, 31, }, // Easter
    {  313,  619,       0,  292428,       0,  0, 15,  0, }, // Ecuador
    {  313,  627,  292443,  292465,  292496, 22, 31, 31, }, // Europe_Central
    {  313,  642,  292527,  292550,  292582, 23, 32, 32, }, // Europe_Eastern
    {  313,  657,       0,  292614,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  313,  680,  292643,  292668,  292702, 25, 34, 34, }, // Europe_Western
    {  313,  695,  292736,  292759,  292791, 23, 32, 32, }, // Falkland
    {  313,  704,  292823,  292835,  292856, 12, 21, 21, }, // Fiji
    {  313,  709,       0,  292877,       0,  0, 23,  0, }, // French_Guiana
    {  313,  723,       0,  292900,       0,  0, 47,  0, }, // French_Southern
    {  313,  746,       0,  292947,       0,  0, 18,  0, }, // Galapagos
    {  313,  756,       0,  292965,       0,  0, 15,  0, }, // Gambier
    {  313,  764,  292980,  292995,  293019, 15, 24, 24, }, // Georgia
    {  313,  772,       0,  293043,       0,  0, 22,  0, }, // Gilbert_Islands
    {  313,  788,       0,  293065,       0,  0, 30,  0, }, // GMT
    {  313,  830,  293095,  293123,  293160, 28, 37, 37, }, // Greenland_Eastern
    {  313,  848,  293197,  293227,  293266, 30, 39, 39, }, // Greenland_Western
    {  313,  871,       0,  293305,       0,  0, 13,  0, }, // Gulf
    {  313,  876,       0,  292877,       0,  0, 14,  0, }, // Guyana
    {  313,  883,  293318,  293348,  293387, 30, 39, 39, }, // Hawaii_Aleutian
    {  313,  899,  293426,  293443,  293469, 17, 26, 26, }, // Hong_Kong
    {  313,  909,  293495,  293507,  293528, 12, 21, 21, }, // Hovd
    {  313,  914,       0,  293549,       0,  0, 13,  0, }, // India
    {  313,  920,       0,  293562,       0,  0, 21,  0, }, // Indian_Ocean
    {  313,  933,       0,  293583,       0,  0, 17,  0, }, // Indochina
    {  313,  943,       0,  293600,       0,  0, 25,  0, }, // Indonesia_Central
    {  313,  961,       0,  293625,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  313,  979,       0,  293651,       0,  0, 28,  0, }, // Indonesia_Western
    {  313,  997,  293679,  293691,  293712, 12, 21, 21, }, // Iran
    {  313, 1008,  293733,  293748,  293772, 15, 24, 24, }, // Irkutsk
    {  313, 1016,  293796,  293810,  293833, 14, 23, 23, }, // Israel
    {  313, 1023,  293856,  293869,  293891, 13, 22, 22, }, // Japan
    {  313, 1029,  293913,  293954,  293998, 41, 44, 44, }, // Kamchatka
    {  313, 1047,       0,  294042,       0,  0, 18,  0, }, // Kazakhstan
    {  313, 1058,       0,  294060,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  313, 1077,       0,  294087,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  313, 1106,  294116,  294129,  294151, 13, 22, 22, }, // Korea
    {  313, 1112,       0,  294173,       0,  0, 14,  0, }, // Kosrae
    {  313, 1119,  294187,  294206,  294234, 19, 28, 28, }, // Krasnoyarsk
    {  313, 1151,       0,  294262,       0,  0, 20,  0, }, // Kyrgystan
    {  313, 1175,       0,  294282,       0,  0, 22,  0, }, // Line_Islands
    {  313, 1188,  294304,  294321,  294347, 17, 26, 26, }, // Lord_Howe
    {  313, 1204,  294373,  294388,  294412, 15, 24, 24, }, // Magadan
    {  313, 1219,       0,  294436,       0,  0, 16,  0, }, // Malaysia
    {  313, 1228,       0,  294452,       0,  0, 17,  0, }, // Maldives
    {  313, 1237,       0,  294469,       0,  0, 18,  0, }, // Marquesas
    {  313, 1247,       0,  294487,       0,  0, 23,  0, }, // Marshall_Islands
    {  313, 1264,  294510,  294526,  294551, 16, 25, 25, }, // Mauritius
    {  313, 1274,       0,  294576,       0,  0, 14,  0, }, // Mawson
    {  313, 1281,  294590,  294615,  294649, 25, 34, 34, }, // Mexico_Pacific
    {  313, 1296,  294683,  294702,  294730, 19, 28, 28, }, // Mongolia
    {  313, 1305,  294758,  294773,  294797, 15, 24, 24, }, // Moscow
    {  313, 1312,       0,  294821,       0,  0, 15,  0, }, // Myanmar
    {  313, 1320,       0,  294836,       0,  0, 13,  0, }, // Nauru
    {  313, 1326,       0,  294849,       0,  0, 13,  0, }, // Nepal
    {  313, 1332,  294862,  294884,  294915, 22, 31, 31, }, // New_Caledonia
    {  313, 1346,  294946,  294967,  294997, 21, 30, 30, }, // New_Zealand
    {  313, 1358,  295027,  295045,  295072, 18, 27, 27, }, // Newfoundland
    {  313, 1371,       0,  295099,       0,  0, 12,  0, }, // Niue
    {  313, 1376,  295111,  295131,  295160, 20, 29, 29, }, // Norfolk
    {  313, 1384,  295189,  295216,  295252, 27, 36, 36, }, // Noronha
    {  313, 1406,  295288,  295307,  295335, 19, 28, 28, }, // Novosibirsk
    {  313, 1418,  295363,  295375,  295396, 12, 21, 21, }, // Omsk
    {  313, 1428,  295417,  295434,  295460, 17, 26, 26, }, // Pakistan
    {  313, 1437,       0,  295486,       0,  0, 13,  0, }, // Palau
    {  313, 1443,       0,  295499,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  313, 1460,  295524,  295540,  295565, 16, 25, 25, }, // Paraguay
    {  313, 1469,  295590,  295602,  295623, 12, 21, 21, }, // Peru
    {  313, 1474,  295644,  295661,  295687, 17, 26, 26, }, // Philippines
    {  313, 1486,       0,  295713,       0,  0, 20,  0, }, // Phoenix_Islands
    {  313, 1502,  295733,  295765,  295806, 32, 41, 41, }, // Pierre_Miquelon
    {  313, 1518,       0,  295847,       0,  0, 16,  0, }, // Pitcairn
    {  313, 1527,       0,  295863,       0,  0, 15,  0, }, // Ponape
    {  313, 1534,       0,   63402,       0,  0, 17,  0, }, // Pyongyang
    {  313, 1554,       0,  295878,       0,  0, 15,  0, }, // Reunion
    {  313, 1562,       0,  295893,       0,  0, 15,  0, }, // Rothera
    {  313, 1570,  295908,  295924,  295949, 16, 25, 25, }, // Sakhalin
    {  313, 1579,  295974,  295988,  296014, 14, 26, 26, }, // Samara
    {  313, 1596,  296040,  296053,  296075, 13, 22, 22, }, // Samoa
    {  313, 1602,       0,  296097,       0,  0, 19,  0, }, // Seychelles
    {  313, 1624,       0,  296116,       0,  0, 16,  0, }, // Singapore
    {  313, 1634,       0,  296132,       0,  0, 22,  0, }, // Solomon
    {  313, 1642,       0,  296154,       0,  0, 22,  0, }, // South_Georgia
    {  313, 1656,       0,  296176,       0,  0, 16,  0, }, // Suriname
    {  313, 1676,       0,  296192,       0,  0, 13,  0, }, // Syowa
    {  313, 1682,       0,  296205,       0,  0, 14,  0, }, // Tahiti
    {  313, 1689,  296219,  296233,  296256, 14, 23, 23, }, // Taipei
    {  313, 1696,       0,  296279,       0,  0, 19,  0, }, // Tajikistan
    {  313, 1724,       0,  296298,       0,  0, 15,  0, }, // Tokelau
    {  313, 1732,  296313,  296326,  296348, 13, 22, 22, }, // Tonga
    {  313, 1738,       0,  296370,       0,  0, 13,  0, }, // Truk
    {  313, 1750,  296383,  296403,  296432, 20, 29, 29, }, // Turkmenistan
    {  313, 1763,       0,  296461,       0,  0, 14,  0, }, // Tuvalu
    {  313, 1777,  296475,  296490,  296514, 15, 24, 24, }, // Uruguay
    {  313, 1792,  296538,  296556,  296583, 18, 27, 27, }, // Uzbekistan
    {  313, 1803,  296610,  296625,  296649, 15, 24, 24, }, // Vanuatu
    {  313, 1811,       0,  296673,       0,  0, 17,  0, }, // Venezuela
    {  313, 1821,  296690,  296709,  296737, 19, 28, 28, }, // Vladivostok
    {  313, 1833,  296765,  296783,  296810, 18, 27, 27, }, // Volgograd
    {  313, 1843,       0,  296837,       0,  0, 14,  0, }, // Vostok
    {  313, 1850,       0,  296851,       0,  0, 17,  0, }, // Wake
    {  313, 1855,       0,  296868,       0,  0, 23,  0, }, // Wallis
    {  313, 1862,  296891,  296906,  296930, 15, 24, 24, }, // Yakutsk
    {  313, 1870,  296954,  296975,  297005, 21, 30, 30, }, // Yekaterinburg
    {  313, 1892,       0,  297035,       0,  0, 13,  0, }, // Yukon
    {  317,    5,       0,  297048,       0,  0, 13,  0, }, // Afghanistan Georgian/Georgian/Georgia
    {  317,   17,       0,  297061,       0,  0, 22,  0, }, // Africa_Central
    {  317,   32,       0,  297083,       0,  0, 22,  0, }, // Africa_Eastern
    {  317,   65,       0,  297105,       0,  0, 19,  0, }, // Africa_Southern
    {  317,   81,  297124,  297144,  297176, 20, 32, 29, }, // Africa_Western
    {  317,  107,  297205,  297216,  297239, 11, 23, 20, }, // Alaska
    {  317,  135,  297259,  297272,  297297, 13, 25, 22, }, // Amazon
    {  317,  142,  297319,  297351,  297395, 32, 44, 41, }, // America_Central
    {  317,  158,  297436,  297470,  297516, 34, 46, 43, }, // America_Eastern
    {  317,  174,  297559,  297590,  297633, 31, 43, 40, }, // America_Mountain
    {  317,  191,  297673,  297709,  297757, 36, 48, 45, }, // America_Pacific
    {  317,  214,  297802,  297811,  297832,  9, 21, 18, }, // Apia
    {  317,  232,  297850,  297862,  297886, 12, 24, 21, }, // Arabian
    {  317,  240,  297907,  297921,  297947, 14, 26, 23, }, // Argentina
    {  317,  250,  297970,  297993,  298028, 23, 35, 32, }, // Argentina_Western
    {  317,  268,  298060,  298072,  298096, 12, 24, 21, }, // Armenia
    {  317,  286,  298117,  298139,  298173, 22, 34, 31, }, // Atlantic
    {  317,  295,  298204,  298229,  298266, 25, 37, 34, }, // Australia_Central
    {  317,  313,  298300,  298337,  298386, 37, 49, 46, }, // Australia_CentralWestern
    {  317,  338,  298432,  298457,  298494, 25, 37, 34, }, // Australia_Eastern
    {  317,  356,  298314,  298351,  298400, 23, 35, 32, }, // Australia_Western
    {  317,  374,  298528,  298544,  298572, 16, 28, 25, }, // Azerbaijan
    {  317,  385,  298597,  298618,  298651, 21, 33, 30, }, // Azores
    {  317,  397,  298681,  298696,  298723, 15, 27, 24, }, // Bangladesh
    {  317,  415,       0,  298747,       0,  0, 11,  0, }, // Bhutan
    {  317,  422,       0,  298758,       0,  0, 12,  0, }, // Bolivia
    {  317,  437,  298770,  298783,  298808, 13, 25, 22, }, // Brasilia
    {  317,  454,       0,  298830,       0,  0, 22,  0, }, // Brunei
    {  317,  461,  298852,  298867,  298894, 15, 27, 24, }, // Cape_Verde
    {  317,  478,       0,  298918,       0,  0, 11,  0, }, // Chamorro
    {  317,  487,  298929,  298940,  298963, 11, 23, 20, }, // Chatham
    {  317,  495,  298983,  298992,  299013,  9, 21, 18, }, // Chile
    {  317,  501,  299031,  299042,  299065, 11, 23, 22, }, // China
    {  317,  507,       0,  299087,       0,  0, 18,  0, }, // Christmas
    {  317,  517,       0,  299105,       0,  0, 22,  0, }, // Cocos
    {  317,  523,  299127,  299140,  299165, 13, 25, 22, }, // Colombia
    {  317,  532,  299187,  299207,  299239, 20, 32, 38, }, // Cook
    {  317,  537,  299277,  299286,  299307,  9, 21, 18, }, // Cuba
    {  317,  548,       0,  299325,       0,  0, 11,  0, }, // Davis
    {  317,  564,       0,  299336,       0,  0, 18,  0, }, // DumontDUrville
    {  317,  601,       0,  299354,       0,  0, 22,  0, }, // East_Timor
    {  317,  612,  299376,  299397,  299430, 21, 33, 30, }, // Easter
    {  317,  619,       0,  299460,       0,  0, 13,  0, }, // Ecuador
    {  317,  627,  299473,  299495,  299529, 22, 34, 31, }, // Europe_Central
    {  317,  642,  299560,  299582,  299616, 22, 34, 31, }, // Europe_Eastern
    {  317,  657,       0,  299647,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  317,  680,  299677,  299697,  299729, 20, 32, 29, }, // Europe_Western
    {  317,  695,  299758,  299783,  299820, 25, 37, 34, }, // Falkland
    {  317,  704,  299854,  299863,  299884,  9, 21, 18, }, // Fiji
    {  317,  709,       0,  299902,       0,  0, 23,  0, }, // French_Guiana
    {  317,  723,       0,  299925,       0,  0, 37,  0, }, // French_Southern
    {  317,  746,       0,  299962,       0,  0, 15,  0, }, // Galapagos
    {  317,  756,       0,  299977,       0,  0, 13,  0, }, // Gambier
    {  317,  764,  299990,  300005,  300032, 15, 27, 24, }, // Georgia
    {  317,  772,       0,  300056,       0,  0, 24,  0, }, // Gilbert_Islands
    {  317,  788,       0,  300080,       0,  0, 21,  0, }, // GMT
    {  317,  830,  300101,  300127,  300165, 26, 38, 35, }, // Greenland_Eastern
    {  317,  848,  300200,  300224,  300260, 24, 36, 33, }, // Greenland_Western
    {  317,  871,       0,  300293,       0,  0, 31,  0, }, // Gulf
    {  317,  876,       0,  300324,       0,  0, 11,  0, }, // Guyana
    {  317,  883,  300335,  300357,  300391, 22, 34, 31, }, // Hawaii_Aleutian
    {  317,  899,  300422,  300435,  300460, 13, 25, 22, }, // Hong_Kong
    {  317,  909,  300482,  300492,  300514, 10, 22, 19, }, // Hovd
    {  317,  914,       0,  300533,       0,  0, 12,  0, }, // India
    {  317,  920,       0,  300545,       0,  0, 31,  0, }, // Indian_Ocean
    {  317,  933,       0,  300576,       0,  0, 15,  0, }, // Indochina
    {  317,  943,       0,  300591,       0,  0, 25,  0, }, // Indonesia_Central
    {  317,  961,       0,  300616,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  317,  979,       0,  300641,       0,  0, 23,  0, }, // Indonesia_Western
    {  317,  997,  300664,  300674,  300696, 10, 22, 21, }, // Iran
    {  317, 1008,  300717,  300730,  300755, 13, 25, 22, }, // Irkutsk
    {  317, 1016,  300777,  300789,  300813, 12, 24, 21, }, // Israel
    {  317, 1023,  300834,  300846,  300870, 12, 24, 21, }, // Japan
    {  317, 1047,       0,  300891,       0,  0, 13,  0, }, // Kazakhstan
    {  317, 1058,       0,  300904,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  317, 1077,       0,  300928,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  317, 1106,  300950,  300960,  300982, 10, 22, 19, }, // Korea
    {  317, 1112,       0,  301001,       0,  0, 10,  0, }, // Kosrae
    {  317, 1119,  301011,  301028,  301057, 17, 29, 26, }, // Krasnoyarsk
    {  317, 1151,       0,  301083,       0,  0, 14,  0, }, // Kyrgystan
    {  317, 1175,       0,  301097,       0,  0, 21,  0, }, // Line_Islands
    {  317, 1188,  301118,  301131,  301156, 13, 25, 22, }, // Lord_Howe
    {  317, 1204,  301178,  301191,  301216, 13, 25, 22, }, // Magadan
    {  317, 1219,       0,  301238,       0,  0, 13,  0, }, // Malaysia
    {  317, 1228,       0,  301251,       0,  0, 14,  0, }, // Maldives
    {  317, 1237,       0,  301265,       0,  0, 23,  0, }, // Marquesas
    {  317, 1247,       0,  301288,       0,  0, 23,  0, }, // Marshall_Islands
    {  317, 1264,  301311,  301323,  301347, 12, 24, 21, }, // Mauritius
    {  317, 1274,       0,  301368,       0,  0, 12,  0, }, // Mawson
    {  317, 1281,  301380,  301407,  301446, 27, 39, 36, }, // Mexico_Pacific
    {  317, 1296,  301482,  301498,  301526, 16, 28, 25, }, // Mongolia
    {  317, 1305,  301551,  301563,  301587, 12, 24, 21, }, // Moscow
    {  317, 1312,       0,  301608,       0,  0, 13,  0, }, // Myanmar
    {  317, 1320,       0,  301621,       0,  0, 10,  0, }, // Nauru
    {  317, 1326,       0,  301631,       0,  0, 11,  0, }, // Nepal
    {  317, 1332,  301642,  301662,  301694, 20, 32, 29, }, // New_Caledonia
    {  317, 1346,  301723,  301742,  301773, 19, 31, 28, }, // New_Zealand
    {  317, 1358,  301801,  301819,  301849, 18, 30, 27, }, // Newfoundland
    {  317, 1371,       0,  301876,       0,  0,  9,  0, }, // Niue
    {  317, 1376,  301885,  301907,  301941, 22, 34, 31, }, // Norfolk
    {  317, 1384,  301972,  301996,  302032, 24, 36, 33, }, // Noronha
    {  317, 1406,  302065,  302082,  302111, 17, 29, 26, }, // Novosibirsk
    {  317, 1418,  302137,  302147,  302169, 10, 22, 19, }, // Omsk
    {  317, 1428,  302188,  302202,  302228, 14, 26, 23, }, // Pakistan
    {  317, 1437,       0,  302251,       0,  0, 10,  0, }, // Palau
    {  317, 1443,       0,  302261,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  317, 1460,  302284,  302297,  302322, 13, 25, 22, }, // Paraguay
    {  317, 1469,  302344,  302353,  302374,  9, 21, 18, }, // Peru
    {  317, 1474,  302392,  302407,  302434, 15, 27, 24, }, // Philippines
    {  317, 1486,       0,  302458,       0,  0, 23,  0, }, // Phoenix_Islands
    {  317, 1502,  302481,  302508,  302547, 27, 39, 36, }, // Pierre_Miquelon
    {  317, 1518,       0,  302583,       0,  0, 13,  0, }, // Pitcairn
    {  317, 1527,       0,  302596,       0,  0, 11,  0, }, // Ponape
    {  317, 1534,       0,  302607,       0,  0, 13,  0, }, // Pyongyang
    {  317, 1554,       0,  302620,       0,  0, 14,  0, }, // Reunion
    {  317, 1562,       0,  302634,       0,  0, 11,  0, }, // Rothera
    {  317, 1570,  302645,  302658,  302683, 13, 25, 22, }, // Sakhalin
    {  317, 1596,  302705,  302715,  302737, 10, 22, 19, }, // Samoa
    {  317, 1602,       0,  302756,       0,  0, 23,  0, }, // Seychelles
    {  317, 1624,       0,  302779,       0,  0, 14,  0, }, // Singapore
    {  317, 1634,       0,  302793,       0,  0, 24,  0, }, // Solomon
    {  317, 1642,       0,  302817,       0,  0, 20,  0, }, // South_Georgia
    {  317, 1656,       0,  302837,       0,  0, 13,  0, }, // Suriname
    {  317, 1676,       0,  302850,       0,  0, 10,  0, }, // Syowa
    {  317, 1682,       0,  302860,       0,  0, 10,  0, }, // Tahiti
    {  317, 1689,  302870,  302881,  302904, 11, 23, 20, }, // Taipei
    {  317, 1696,       0,  302924,       0,  0, 13,  0, }, // Tajikistan
    {  317, 1724,       0,  302937,       0,  0, 12,  0, }, // Tokelau
    {  317, 1732,  302949,  302959,  302981, 10, 22, 19, }, // Tonga
    {  317, 1738,       0,  303000,       0,  0, 10,  0, }, // Truk
    {  317, 1750,  303010,  303025,  303052, 15, 27, 24, }, // Turkmenistan
    {  317, 1763,       0,  303076,       0,  0, 11,  0, }, // Tuvalu
    {  317, 1777,  303087,  303099,  303123, 12, 24, 21, }, // Uruguay
    {  317, 1792,  303144,  303157,  303182, 13, 25, 22, }, // Uzbekistan
    {  317, 1803,  303204,  303216,  303240, 12, 24, 21, }, // Vanuatu
    {  317, 1811,       0,  303261,       0,  0, 14,  0, }, // Venezuela
    {  317, 1821,  303275,  303292,  303321, 17, 29, 26, }, // Vladivostok
    {  317, 1833,  303347,  303362,  303389, 15, 27, 24, }, // Volgograd
    {  317, 1843,       0,  303280,       0,  0, 12,  0, }, // Vostok
    {  317, 1850,       0,  303413,       0,  0, 19,  0, }, // Wake
    {  317, 1855,       0,  303432,       0,  0, 21,  0, }, // Wallis
    {  317, 1862,  303453,  303466,  303491, 13, 25, 22, }, // Yakutsk
    {  317, 1870,  303513,  303531,  303561, 18, 30, 27, }, // Yekaterinburg
    {  317, 1892,       0,  303588,       0,  0, 11,  0, }, // Yukon
    {  318,    0,  303599,  303608,  303623,  9, 15, 15, }, // Acre German/Latin/Germany
    {  318,    5,       0,  303638,       0,  0, 16,  0, }, // Afghanistan
    {  318,   17,       0,  303654,       0,  0, 24,  0, }, // Africa_Central
    {  318,   32,       0,  303678,       0,  0, 20,  0, }, // Africa_Eastern
    {  318,   65,       0,  303698,       0,  0, 20,  0, }, // Africa_Southern
    {  318,   81,  303718,  303739,  303766, 21, 27, 27, }, // Africa_Western
    {  318,  107,  303793,  303804,  303821, 11, 17, 17, }, // Alaska
    {  318,  128,  303838,  303849,  303866, 11, 17, 17, }, // Almaty
    {  318,  135,  303883,  303896,  303915, 13, 19, 19, }, // Amazon
    {  318,  142,  303934,  303963,  303999, 29, 36, 36, }, // America_Central
    {  318,  158,  304035,  304066,  304104, 31, 38, 38, }, // America_Eastern
    {  318,  174,  304142,  304162,  304188, 20, 26, 26, }, // America_Mountain
    {  318,  191,  304214,  304246,  304285, 32, 39, 39, }, // America_Pacific
    {  318,  207,  304324,  304335,  304352, 11, 17, 17, }, // Anadyr
    {  318,  214,  304369,  304378,  304393,  9, 15, 15, }, // Apia
    {  318,  219,  304408,  304418,  304434, 10, 16, 16, }, // Aqtau
    {  318,  225,  304450,  304461,  304478, 11, 17, 17, }, // Aqtobe
    {  318,  232,  304495,  304509,  304529, 14, 20, 20, }, // Arabian
    {  318,  240,  304549,  304567,  304591, 18, 24, 24, }, // Argentina
    {  318,  250,  304615,  304637,  304665, 22, 28, 28, }, // Argentina_Western
    {  318,  268,  304693,  304708,  304729, 15, 21, 21, }, // Armenia
    {  318,  286,  304750,  304763,  304782, 13, 19, 19, }, // Atlantic
    {  318,  295,  304801,  304825,  304855, 24, 30, 30, }, // Australia_Central
    {  318,  313,  304885,  304915,  304951, 30, 36, 36, }, // Australia_CentralWestern
    {  318,  338,  304987,  305007,  305033, 20, 26, 26, }, // Australia_Eastern
    {  318,  356,  304894,  304924,  304960, 21, 27, 27, }, // Australia_Western
    {  318,  374,  305059,  305082,  305111, 23, 29, 29, }, // Azerbaijan
    {  318,  385,  305140,  305151,  305168, 11, 17, 17, }, // Azores
    {  318,  397,  305185,  305201,  305223, 16, 22, 22, }, // Bangladesh
    {  318,  415,       0,  305245,       0,  0, 11,  0, }, // Bhutan
    {  318,  422,       0,  305256,       0,  0, 18,  0, }, // Bolivia
    {  318,  437,  305274,  305287,  305306, 13, 19, 19, }, // Brasilia
    {  318,  454,       0,  305325,       0,  0, 22,  0, }, // Brunei
    {  318,  461,  305347,  305362,  305383, 15, 21, 21, }, // Cape_Verde
    {  318,  472,       0,  305404,       0,  0, 10,  0, }, // Casey
    {  318,  478,       0,  305414,       0,  0, 13,  0, }, // Chamorro
    {  318,  487,  305427,  305439,  305457, 12, 18, 18, }, // Chatham
    {  318,  495,  305475,  305491,  305513, 16, 22, 22, }, // Chile
    {  318,  501,  305535,  305551,  305573, 16, 22, 22, }, // China
    {  318,  507,       0,  305595,       0,  0, 20,  0, }, // Christmas
    {  318,  517,       0,  305615,       0,  0, 16,  0, }, // Cocos
    {  318,  523,  305631,  305650,  305675, 19, 25, 25, }, // Colombia
    {  318,  532,  305700,  305715,  305736, 15, 21, 21, }, // Cook
    {  318,  537,  305757,  305772,  305793, 15, 21, 21, }, // Cuba
    {  318,  548,       0,  305814,       0,  0, 10,  0, }, // Davis
    {  318,  564,       0,  305824,       0,  0, 21,  0, }, // DumontDUrville
    {  318,  601,       0,  305845,       0,  0, 13,  0, }, // East_Timor
    {  318,  612,  305858,  305873,  305894, 15, 21, 21, }, // Easter
    {  318,  619,       0,  305915,       0,  0, 20,  0, }, // Ecuador
    {  318,  627,  305935,  305957,  305985, 22, 28, 28, }, // Europe_Central
    {  318,  642,  306013,  306032,  306057, 19, 25, 25, }, // Europe_Eastern
    {  318,  657,       0,  306082,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {  318,  680,  306100,  306120,  306146, 20, 26, 26, }, // Europe_Western
    {  318,  695,  306172,  306191,  306216, 19, 25, 25, }, // Falkland
    {  318,  704,  306241,  306253,  306271, 12, 18, 18, }, // Fiji
    {  318,  709,       0,  306289,       0,  0, 24,  0, }, // French_Guiana
    {  318,  723,       0,  306313,       0,  0, 42,  0, }, // French_Southern
    {  318,  746,       0,  306355,       0,  0, 14,  0, }, // Galapagos
    {  318,  756,       0,  306369,       0,  0, 12,  0, }, // Gambier
    {  318,  764,  306381,  306396,  306417, 15, 21, 21, }, // Georgia
    {  318,  772,       0,  306438,       0,  0, 19,  0, }, // Gilbert_Islands
    {  318,  788,       0,  306457,       0,  0, 23,  0, }, // GMT
    {  318,  830,  306480,  306496,  306518, 16, 22, 22, }, // Greenland_Eastern
    {  318,  848,  306540,  306557,  306580, 17, 23, 23, }, // Greenland_Western
    {  318,  866,       0,  306603,       0,  0,  9,  0, }, // Guam
    {  318,  871,       0,  306612,       0,  0,  9,  0, }, // Gulf
    {  318,  876,       0,  306621,       0,  0, 11,  0, }, // Guyana
    {  318,  883,  306632,  306651,  306676, 19, 25, 25, }, // Hawaii_Aleutian
    {  318,  899,  306701,  306714,  306733, 13, 19, 19, }, // Hong_Kong
    {  318,  909,  306752,  306762,  306778, 10, 16, 16, }, // Hovd
    {  318,  914,       0,  306794,       0,  0, 19,  0, }, // India
    {  318,  920,       0,  306813,       0,  0, 20,  0, }, // Indian_Ocean
    {  318,  933,       0,  306833,       0,  0, 14,  0, }, // Indochina
    {  318,  943,       0,  306847,       0,  0, 24,  0, }, // Indonesia_Central
    {  318,  961,       0,  306871,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  318,  979,       0,  306891,       0,  0, 21,  0, }, // Indonesia_Western
    {  318,  997,  306912,  306926,  306946, 14, 20, 20, }, // Iran
    {  318, 1008,  306966,  306980,  307000, 14, 20, 20, }, // Irkutsk
    {  318, 1016,  307020,  307036,  307058, 16, 22, 22, }, // Israel
    {  318, 1023,  307080,  307095,  307116, 15, 21, 21, }, // Japan
    {  318, 1029,  307137,  307153,  307175, 16, 22, 22, }, // Kamchatka
    {  318, 1047,       0,  307197,       0,  0, 16,  0, }, // Kazakhstan
    {  318, 1058,       0,  307213,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  318, 1077,       0,  307232,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  318, 1106,  307252,  307268,  307290, 16, 22, 22, }, // Korea
    {  318, 1112,       0,  307312,       0,  0, 11,  0, }, // Kosrae
    {  318, 1119,  307323,  307341,  307365, 18, 24, 24, }, // Krasnoyarsk
    {  318, 1151,       0,  307389,       0,  0, 16,  0, }, // Kyrgystan
    {  318, 1161,       0,  307405,       0,  0, 14,  0, }, // Lanka
    {  318, 1175,       0,  307419,       0,  0, 17,  0, }, // Line_Islands
    {  318, 1188,  307436,  307450,  307470, 14, 20, 20, }, // Lord_Howe
    {  318, 1198,  307490,  307500,  307516, 10, 16, 16, }, // Macau
    {  318, 1204,  307532,  307544,  307562, 12, 18, 18, }, // Magadan
    {  318, 1219,       0,  307580,       0,  0, 16,  0, }, // Malaysia
    {  318, 1228,       0,  307596,       0,  0, 14,  0, }, // Maldives
    {  318, 1237,       0,  307610,       0,  0, 14,  0, }, // Marquesas
    {  318, 1247,       0,  307624,       0,  0, 19,  0, }, // Marshall_Islands
    {  318, 1264,  307643,  307657,  307677, 14, 20, 20, }, // Mauritius
    {  318, 1274,       0,  307697,       0,  0, 11,  0, }, // Mawson
    {  318, 1281,  307708,  307732,  307763, 24, 31, 31, }, // Mexico_Pacific
    {  318, 1296,  307794,  307810,  307832, 16, 22, 22, }, // Mongolia
    {  318, 1305,  307854,  307867,  307886, 13, 19, 19, }, // Moscow
    {  318, 1312,       0,  307905,       0,  0, 12,  0, }, // Myanmar
    {  318, 1320,       0,  307917,       0,  0, 10,  0, }, // Nauru
    {  318, 1326,       0,  307927,       0,  0, 17,  0, }, // Nepal
    {  318, 1332,  307944,  307964,  307990, 20, 26, 26, }, // New_Caledonia
    {  318, 1346,  308016,  308031,  308052, 15, 21, 21, }, // New_Zealand
    {  318, 1358,  308073,  308089,  308111, 16, 22, 22, }, // Newfoundland
    {  318, 1371,       0,  308133,       0,  0,  9,  0, }, // Niue
    {  318, 1376,  308142,  308159,  308182, 17, 23, 23, }, // Norfolk
    {  318, 1384,  308205,  308229,  308259, 24, 30, 30, }, // Noronha
    {  318, 1392,       0,  308289,       0,  0, 23,  0, }, // North_Mariana
    {  318, 1406,  308312,  308330,  308354, 18, 24, 24, }, // Novosibirsk
    {  318, 1418,  308378,  308389,  308406, 11, 17, 17, }, // Omsk
    {  318, 1428,  308423,  308441,  308465, 18, 24, 24, }, // Pakistan
    {  318, 1437,       0,  308489,       0,  0, 10,  0, }, // Palau
    {  318, 1443,       0,  308499,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  318, 1460,  308519,  308537,  308561, 18, 24, 24, }, // Paraguay
    {  318, 1469,  308585,  308601,  308623, 16, 22, 22, }, // Peru
    {  318, 1474,  308645,  308664,  308689, 19, 25, 25, }, // Philippines
    {  318, 1486,       0,  308714,       0,  0, 18,  0, }, // Phoenix_Islands
    {  318, 1502,  308732,  308760,  308794, 28, 34, 34, }, // Pierre_Miquelon
    {  318, 1518,       0,  308828,       0,  0, 19,  0, }, // Pitcairn
    {  318, 1527,       0,  308847,       0,  0, 11,  0, }, // Ponape
    {  318, 1534,       0,  308858,       0,  0, 14,  0, }, // Pyongyang
    {  318, 1544,  308872,  308887,  308908, 15, 21, 20, }, // Qyzylorda
    {  318, 1554,       0,  308928,       0,  0, 12,  0, }, // Reunion
    {  318, 1562,       0,  308940,       0,  0, 12,  0, }, // Rothera
    {  318, 1570,  308952,  308965,  308984, 13, 19, 19, }, // Sakhalin
    {  318, 1579,  309003,  309014,  309031, 11, 17, 17, }, // Samara
    {  318, 1596,  309048,  309058,  309074, 10, 16, 16, }, // Samoa
    {  318, 1602,       0,  309090,       0,  0, 15,  0, }, // Seychelles
    {  318, 1624,       0,  309105,       0,  0, 24,  0, }, // Singapore
    {  318, 1634,       0,  309129,       0,  0, 14,  0, }, // Solomon
    {  318, 1642,       0,  309143,       0,  0, 18,  0, }, // South_Georgia
    {  318, 1656,       0,  309161,       0,  0, 13,  0, }, // Suriname
    {  318, 1676,       0,  309174,       0,  0, 10,  0, }, // Syowa
    {  318, 1682,       0,  309184,       0,  0, 11,  0, }, // Tahiti
    {  318, 1689,  309195,  309206,  309223, 11, 17, 17, }, // Taipei
    {  318, 1696,       0,  309240,       0,  0, 18,  0, }, // Tajikistan
    {  318, 1724,       0,  309258,       0,  0, 12,  0, }, // Tokelau
    {  318, 1732,  309270,  309285,  309306, 15, 21, 21, }, // Tonga
    {  318, 1738,       0,  309327,       0,  0, 10,  0, }, // Truk
    {  318, 1750,  309337,  309354,  309377, 17, 23, 23, }, // Turkmenistan
    {  318, 1763,       0,  309400,       0,  0, 11,  0, }, // Tuvalu
    {  318, 1777,  309411,  309428,  309451, 17, 23, 23, }, // Uruguay
    {  318, 1792,  309474,  309489,  309510, 15, 21, 21, }, // Uzbekistan
    {  318, 1803,  309531,  309543,  309561, 12, 18, 18, }, // Vanuatu
    {  318, 1811,       0,  309579,       0,  0, 14,  0, }, // Venezuela
    {  318, 1821,  309593,  309611,  309635, 18, 24, 24, }, // Vladivostok
    {  318, 1833,  309659,  309675,  309697, 16, 22, 22, }, // Volgograd
    {  318, 1843,       0,  309719,       0,  0, 11,  0, }, // Vostok
    {  318, 1850,       0,  309730,       0,  0, 15,  0, }, // Wake
    {  318, 1855,       0,  309745,       0,  0, 22,  0, }, // Wallis
    {  318, 1862,  309767,  309781,  309801, 14, 20, 20, }, // Yakutsk
    {  318, 1870,  309821,  309841,  309867, 20, 26, 26, }, // Yekaterinburg
    {  318, 1892,       0,  309893,       0,  0, 10,  0, }, // Yukon
    {  324,  454,       0,  309903,       0,  0, 11,  0, }, // Brunei German/Latin/Switzerland
    {  324, 1634,       0,  309914,       0,  0, 18,  0, }, // Solomon
    {  325,    5,       0,  309932,       0,  0, 14,  0, }, // Afghanistan Greek/Greek/Greece
    {  325,   17,       0,  309946,       0,  0, 21,  0, }, // Africa_Central
    {  325,   32,       0,  309967,       0,  0, 22,  0, }, // Africa_Eastern
    {  325,   65,       0,  309989,       0,  0, 28,  0, }, // Africa_Southern
    {  325,   81,  310017,  310036,  310065, 19, 29, 26, }, // Africa_Western
    {  325,  107,  310091,  310102,  310123, 11, 21, 18, }, // Alaska
    {  325,  135,  310141,  310154,  310177, 13, 23, 20, }, // Amazon
    {  325,  142,  310197,  310226,  310265, 29, 39, 36, }, // America_Central
    {  325,  158,  310301,  310331,  310371, 30, 40, 37, }, // America_Eastern
    {  325,  174,  310408,  310435,  310472, 27, 37, 34, }, // America_Mountain
    {  325,  191,  310506,  310519,  310542, 13, 23, 20, }, // America_Pacific
    {  325,  207,  310562,  310573,  310594, 11, 21, 18, }, // Anadyr
    {  325,  214,  310612,  310620,  310638,  8, 18, 15, }, // Apia
    {  325,  232,  310653,  310664,  310685, 11, 21, 18, }, // Arabian
    {  325,  240,  310703,  310717,  310741, 14, 24, 21, }, // Argentina
    {  325,  250,  310762,  310784,  310816, 22, 32, 29, }, // Argentina_Western
    {  325,  268,  310845,  310857,  310879, 12, 22, 19, }, // Armenia
    {  325,  286,  310898,  310912,  310936, 14, 24, 21, }, // Atlantic
    {  325,  295,  310957,  310981,  311015, 24, 34, 31, }, // Australia_Central
    {  325,  313,  311046,  311074,  311112, 28, 38, 35, }, // Australia_CentralWestern
    {  325,  338,  311147,  311172,  311207, 25, 35, 32, }, // Australia_Eastern
    {  325,  356,  311239,  311261,  311293, 22, 32, 29, }, // Australia_Western
    {  325,  374,  311322,  311338,  311364, 16, 26, 23, }, // Azerbaijan
    {  325,  385,  311387,  311397,  311417, 10, 20, 17, }, // Azores
    {  325,  397,  311434,  311450,  311476, 16, 26, 23, }, // Bangladesh
    {  325,  415,       0,  311499,       0,  0, 11,  0, }, // Bhutan
    {  325,  422,       0,  311510,       0,  0, 12,  0, }, // Bolivia
    {  325,  437,  311522,  311536,  311560, 14, 24, 21, }, // Brasilia
    {  325,  454,       0,  311581,       0,  0, 24,  0, }, // Brunei
    {  325,  461,  311605,  311628,  311661, 23, 33, 30, }, // Cape_Verde
    {  325,  478,       0,  311691,       0,  0, 11,  0, }, // Chamorro
    {  325,  487,  311702,  311712,  311732, 10, 20, 17, }, // Chatham
    {  325,  495,  311749,  311758,  311777,  9, 19, 16, }, // Chile
    {  325,  501,  311793,  311802,  311821,  9, 19, 16, }, // China
    {  325,  507,       0,  311837,       0,  0, 23,  0, }, // Christmas
    {  325,  517,       0,  311860,       0,  0, 15,  0, }, // Cocos
    {  325,  523,  311875,  311888,  311911, 13, 23, 20, }, // Colombia
    {  325,  532,  311931,  311945,  311969, 14, 24, 21, }, // Cook
    {  325,  537,  311990,  312000,  312020, 10, 20, 17, }, // Cuba
    {  325,  548,       0,  312037,       0,  0, 11,  0, }, // Davis
    {  325,  564,       0,  312048,       0,  0, 21,  0, }, // DumontDUrville
    {  325,  601,       0,  312069,       0,  0, 20,  0, }, // East_Timor
    {  325,  612,  312089,  312104,  312129, 15, 25, 22, }, // Easter
    {  325,  619,       0,  312151,       0,  0, 14,  0, }, // Ecuador
    {  325,  627,  312165,  312186,  312217, 21, 31, 28, }, // Europe_Central
    {  325,  642,  312245,  312267,  312299, 22, 32, 29, }, // Europe_Eastern
    {  325,  657,       0,  312328,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  325,  680,  312360,  312379,  312408, 19, 29, 26, }, // Europe_Western
    {  325,  695,  312434,  312451,  312478, 17, 27, 24, }, // Falkland
    {  325,  704,  312502,  312511,  312530,  9, 19, 16, }, // Fiji
    {  325,  709,       0,  312546,       0,  0, 21,  0, }, // French_Guiana
    {  325,  723,       0,  312567,       0,  0, 34,  0, }, // French_Southern
    {  325,  746,       0,  312601,       0,  0, 15,  0, }, // Galapagos
    {  325,  756,       0,  312616,       0,  0, 12,  0, }, // Gambier
    {  325,  764,  312628,  312640,  312662, 12, 22, 19, }, // Georgia
    {  325,  772,       0,  312681,       0,  0, 19,  0, }, // Gilbert_Islands
    {  325,  788,       0,  312700,       0,  0, 19,  0, }, // GMT
    {  325,  830,  312719,  312745,  312781, 26, 36, 33, }, // Greenland_Eastern
    {  325,  848,  312814,  312837,  312870, 23, 33, 30, }, // Greenland_Western
    {  325,  866,       0,  312900,       0,  0, 10,  0, }, // Guam
    {  325,  871,       0,  312910,       0,  0, 10,  0, }, // Gulf
    {  325,  876,       0,  312920,       0,  0, 12,  0, }, // Guyana
    {  325,  883,  312932,  312958,  312994, 26, 36, 33, }, // Hawaii_Aleutian
    {  325,  899,  313027,  313042,  313067, 15, 25, 22, }, // Hong_Kong
    {  325,  909,  313089,  313098,  313117,  9, 19, 16, }, // Hovd
    {  325,  914,       0,  313133,       0,  0, 10,  0, }, // India
    {  325,  920,       0,  313143,       0,  0, 19,  0, }, // Indian_Ocean
    {  325,  933,       0,  313162,       0,  0, 13,  0, }, // Indochina
    {  325,  943,       0,  313175,       0,  0, 24,  0, }, // Indonesia_Central
    {  325,  961,       0,  313199,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  325,  979,       0,  313224,       0,  0, 22,  0, }, // Indonesia_Western
    {  325,  997,  313246,  313254,  313272,  8, 18, 15, }, // Iran
    {  325, 1008,  313287,  313299,  313321, 12, 22, 19, }, // Irkutsk
    {  325, 1016,  313340,  313350,  313370, 10, 20, 17, }, // Israel
    {  325, 1023,  313387,  313399,  313421, 12, 22, 19, }, // Japan
    {  325, 1029,  313440,  313453,  313491, 13, 38, 35, }, // Kamchatka
    {  325, 1047,       0,  313526,       0,  0, 13,  0, }, // Kazakhstan
    {  325, 1058,       0,  313539,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  325, 1077,       0,  313563,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  325, 1106,  313584,  313594,  313614, 10, 20, 17, }, // Korea
    {  325, 1112,       0,  313631,       0,  0, 10,  0, }, // Kosrae
    {  325, 1119,  313641,  313657,  313683, 16, 26, 23, }, // Krasnoyarsk
    {  325, 1151,       0,  313706,       0,  0, 13,  0, }, // Kyrgystan
    {  325, 1175,       0,  313719,       0,  0, 14,  0, }, // Line_Islands
    {  325, 1188,  313733,  313747,  313771, 14, 24, 21, }, // Lord_Howe
    {  325, 1198,  313792,  313801,  313820,  9, 19, 16, }, // Macau
    {  325, 1204,  313836,  313849,  313872, 13, 23, 20, }, // Magadan
    {  325, 1219,       0,  313892,       0,  0, 13,  0, }, // Malaysia
    {  325, 1228,       0,  313905,       0,  0, 12,  0, }, // Maldives
    {  325, 1237,       0,  313917,       0,  0, 12,  0, }, // Marquesas
    {  325, 1247,       0,  313929,       0,  0, 16,  0, }, // Marshall_Islands
    {  325, 1264,  313945,  313958,  313981, 13, 23, 20, }, // Mauritius
    {  325, 1274,       0,  314001,       0,  0,  9,  0, }, // Mawson
    {  325, 1281,  314010,  314031,  314062, 21, 31, 28, }, // Mexico_Pacific
    {  325, 1296,  314090,  314106,  314132, 16, 26, 23, }, // Mongolia
    {  325, 1305,  314155,  314165,  314185, 10, 20, 17, }, // Moscow
    {  325, 1312,       0,  314202,       0,  0, 11,  0, }, // Myanmar
    {  325, 1320,       0,  314213,       0,  0, 11,  0, }, // Nauru
    {  325, 1326,       0,  314224,       0,  0,  9,  0, }, // Nepal
    {  325, 1332,  314233,  314252,  314281, 19, 29, 26, }, // New_Caledonia
    {  325, 1346,  314307,  314325,  314353, 18, 28, 25, }, // New_Zealand
    {  325, 1358,  314378,  314390,  314412, 12, 22, 19, }, // Newfoundland
    {  325, 1371,       0,  314431,       0,  0,  9,  0, }, // Niue
    {  325, 1376,  314440,  314457,  314484, 17, 27, 24, }, // Norfolk
    {  325, 1384,  314508,  314533,  314568, 25, 35, 32, }, // Noronha
    {  325, 1392,       0,  314600,       0,  0, 27,  0, }, // North_Mariana
    {  325, 1406,  314627,  314643,  314669, 16, 26, 23, }, // Novosibirsk
    {  325, 1418,  314692,  314700,  314718,  8, 18, 15, }, // Omsk
    {  325, 1428,  314733,  314745,  314767, 12, 22, 19, }, // Pakistan
    {  325, 1437,       0,  314786,       0,  0, 10,  0, }, // Palau
    {  325, 1443,       0,  314796,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  325, 1460,  314821,  314835,  314859, 14, 24, 21, }, // Paraguay
    {  325, 1469,  314880,  314889,  314908,  9, 19, 16, }, // Peru
    {  325, 1474,  314924,  314938,  314962, 14, 24, 21, }, // Philippines
    {  325, 1486,       0,  314983,       0,  0, 16,  0, }, // Phoenix_Islands
    {  325, 1502,  314999,  315023,  315057, 24, 34, 31, }, // Pierre_Miquelon
    {  325, 1518,       0,  315088,       0,  0, 11,  0, }, // Pitcairn
    {  325, 1527,       0,  315099,       0,  0, 10,  0, }, // Ponape
    {  325, 1534,       0,  315109,       0,  0, 14,  0, }, // Pyongyang
    {  325, 1554,       0,  315123,       0,  0, 11,  0, }, // Reunion
    {  325, 1562,       0,  315134,       0,  0, 10,  0, }, // Rothera
    {  325, 1570,  315144,  315157,  315180, 13, 23, 20, }, // Sakhalin
    {  325, 1579,  315200,  315211,  315232, 11, 21, 18, }, // Samara
    {  325, 1596,  315250,  315259,  315278,  9, 19, 16, }, // Samoa
    {  325, 1602,       0,  315294,       0,  0, 13,  0, }, // Seychelles
    {  325, 1624,       0,  315307,       0,  0, 15,  0, }, // Singapore
    {  325, 1634,       0,  315322,       0,  0, 20,  0, }, // Solomon
    {  325, 1642,       0,  315342,       0,  0, 19,  0, }, // South_Georgia
    {  325, 1656,       0,  315361,       0,  0, 12,  0, }, // Suriname
    {  325, 1676,       0,  315373,       0,  0,  9,  0, }, // Syowa
    {  325, 1682,       0,  315382,       0,  0, 10,  0, }, // Tahiti
    {  325, 1689,  315392,  315402,  315422, 10, 20, 17, }, // Taipei
    {  325, 1696,       0,  315439,       0,  0, 15,  0, }, // Tajikistan
    {  325, 1724,       0,  315454,       0,  0, 12,  0, }, // Tokelau
    {  325, 1732,  315466,  315476,  315496, 10, 20, 17, }, // Tonga
    {  325, 1738,       0,  315513,       0,  0,  9,  0, }, // Truk
    {  325, 1750,  315522,  315539,  315566, 17, 27, 24, }, // Turkmenistan
    {  325, 1763,       0,  315590,       0,  0, 12,  0, }, // Tuvalu
    {  325, 1777,  315602,  315617,  315642, 15, 25, 22, }, // Uruguay
    {  325, 1792,  315664,  315680,  315706, 16, 26, 23, }, // Uzbekistan
    {  325, 1803,  315729,  315742,  315765, 13, 23, 20, }, // Vanuatu
    {  325, 1811,       0,  315785,       0,  0, 15,  0, }, // Venezuela
    {  325, 1821,  315800,  315815,  315840, 15, 25, 22, }, // Vladivostok
    {  325, 1833,  315862,  315878,  315904, 16, 26, 23, }, // Volgograd
    {  325, 1843,       0,  315927,       0,  0, 10,  0, }, // Vostok
    {  325, 1850,       0,  315937,       0,  0, 16,  0, }, // Wake
    {  325, 1855,       0,  315953,       0,  0, 23,  0, }, // Wallis
    {  325, 1862,  315976,  315989,  316012, 13, 23, 20, }, // Yakutsk
    {  325, 1870,  316032,  316052,  316082, 20, 30, 27, }, // Yekaterinburg
    {  325, 1892,       0,  316109,       0,  0, 11,  0, }, // Yukon
    {  327,  422,       0,  316120,       0,  0, 11,  0, }, // Bolivia Guarani/Latin/Paraguay
    {  327,  619,       0,  316131,       0,  0, 11,  0, }, // Ecuador
    {  327,  746,       0,  316142,       0,  0, 13,  0, }, // Galapagos
    {  327, 1811,       0,  316155,       0,  0, 13,  0, }, // Venezuela
    {  328,    0,  316168,  316175,  316192,  7, 17, 15, }, // Acre Gujarati/Gujarati/India
    {  328,    5,       0,  316207,       0,  0, 15,  0, }, // Afghanistan
    {  328,   17,       0,  316222,       0,  0, 16,  0, }, // Africa_Central
    {  328,   32,       0,  316238,       0,  0, 17,  0, }, // Africa_Eastern
    {  328,   65,       0,  316255,       0,  0, 23,  0, }, // Africa_Southern
    {  328,   81,  316278,  316296,  316319, 18, 23, 26, }, // Africa_Western
    {  328,  107,  316345,  316356,  316372, 11, 16, 16, }, // Alaska
    {  328,  128,  316388,  316399,  316420, 11, 21, 19, }, // Almaty
    {  328,  135,  316439,  316449,  316464, 10, 15, 18, }, // Amazon
    {  328,  142,  316482,  316509,  316541, 27, 32, 32, }, // America_Central
    {  328,  158,  316573,  316597,  316626, 24, 29, 29, }, // America_Eastern
    {  328,  174,  316655,  316680,  316710, 25, 30, 30, }, // America_Mountain
    {  328,  191,  316740,  316765,  316795, 25, 30, 30, }, // America_Pacific
    {  328,  207,  316825,  316835,  316855, 10, 20, 18, }, // Anadyr
    {  328,  214,  316873,  316882,  316896,  9, 14, 14, }, // Apia
    {  328,  219,  316910,  316920,  316940, 10, 20, 18, }, // Aqtau
    {  328,  225,  316958,  316968,  316988, 10, 20, 18, }, // Aqtobe
    {  328,  232,  317006,  317017,  317033, 11, 16, 16, }, // Arabian
    {  328,  240,  317049,  317064,  317084, 15, 20, 23, }, // Argentina
    {  328,  250,  317107,  317130,  317158, 23, 28, 31, }, // Argentina_Western
    {  328,  268,  317189,  317202,  317220, 13, 18, 21, }, // Armenia
    {  328,  286,  317241,  317254,  317272, 13, 18, 18, }, // Atlantic
    {  328,  295,  317290,  317315,  317345, 25, 30, 30, }, // Australia_Central
    {  328,  313,  317375,  317408,  317446, 33, 38, 38, }, // Australia_CentralWestern
    {  328,  338,  317484,  317507,  317535, 23, 28, 28, }, // Australia_Eastern
    {  328,  356,  317563,  317586,  317614, 23, 28, 28, }, // Australia_Western
    {  328,  374,  317642,  317654,  317671, 12, 17, 20, }, // Azerbaijan
    {  328,  385,  317691,  317701,  317716, 10, 15, 18, }, // Azores
    {  328,  397,  317734,  317748,  317767, 14, 19, 22, }, // Bangladesh
    {  328,  415,       0,  317789,       0,  0,  9,  0, }, // Bhutan
    {  328,  422,       0,  317798,       0,  0, 12,  0, }, // Bolivia
    {  328,  437,  317810,  317824,  317843, 14, 19, 22, }, // Brasilia
    {  328,  454,       0,  317865,       0,  0, 19,  0, }, // Brunei
    {  328,  461,  317884,  317897,  317915, 13, 18, 21, }, // Cape_Verde
    {  328,  478,       0,  317936,       0,  0, 15,  0, }, // Chamorro
    {  328,  487,  317951,  317961,  317976, 10, 15, 15, }, // Chatham
    {  328,  495,  317991,  317999,  318012,  8, 13, 16, }, // Chile
    {  328,  501,  318028,  318035,  318047,  7, 12, 12, }, // China
    {  328,  507,       0,  318059,       0,  0, 19,  0, }, // Christmas
    {  328,  517,       0,  318078,       0,  0, 19,  0, }, // Cocos
    {  328,  523,  318097,  318110,  318128, 13, 18, 21, }, // Colombia
    {  328,  532,  318149,  318166,  318188, 17, 22, 30, }, // Cook
    {  328,  537,  318218,  318228,  318243, 10, 15, 15, }, // Cuba
    {  328,  548,       0,  318258,       0,  0,  9,  0, }, // Davis
    {  328,  564,       0,  318267,       0,  0, 23,  0, }, // DumontDUrville
    {  328,  601,       0,  318290,       0,  0, 15,  0, }, // East_Timor
    {  328,  612,  318305,  318322,  318344, 17, 22, 25, }, // Easter
    {  328,  619,       0,  318369,       0,  0, 12,  0, }, // Ecuador
    {  328,  627,  318381,  318398,  318420, 17, 22, 25, }, // Europe_Central
    {  328,  642,  318445,  318464,  318488, 19, 24, 27, }, // Europe_Eastern
    {  328,  657,       0,  318515,       0,  0, 25,  0, }, // Europe_Further_Eastern
    {  328,  680,  318540,  318560,  318585, 20, 25, 28, }, // Europe_Western
    {  328,  695,  318613,  318635,  318662, 22, 27, 30, }, // Falkland
    {  328,  704,  318692,  318700,  318713,  8, 13, 16, }, // Fiji
    {  328,  709,       0,  318729,       0,  0, 16,  0, }, // French_Guiana
    {  328,  723,       0,  318745,       0,  0, 32,  0, }, // French_Southern
    {  328,  746,       0,  318777,       0,  0, 13,  0, }, // Galapagos
    {  328,  756,       0,  318790,       0,  0, 12,  0, }, // Gambier
    {  328,  764,  318802,  318816,  318835, 14, 19, 22, }, // Georgia
    {  328,  772,       0,  318857,       0,  0, 21,  0, }, // Gilbert_Islands
    {  328,  788,       0,  318878,       0,  0, 18,  0, }, // GMT
    {  328,  830,  318896,  318916,  318941, 20, 25, 28, }, // Greenland_Eastern
    {  328,  848,  318969,  318990,  319016, 21, 26, 29, }, // Greenland_Western
    {  328,  866,       0,  319045,       0,  0, 19,  0, }, // Guam
    {  328,  871,       0,  319064,       0,  0, 13,  0, }, // Gulf
    {  328,  876,       0,  318736,       0,  0,  9,  0, }, // Guyana
    {  328,  883,  319077,  319095,  319118, 18, 23, 23, }, // Hawaii_Aleutian
    {  328,  899,  319141,  319154,  319172, 13, 18, 21, }, // Hong_Kong
    {  328,  909,  319193,  319202,  319216,  9, 14, 17, }, // Hovd
    {  328,  914,       0,  319233,       0,  0, 15,  0, }, // India
    {  328,  920,       0,  319248,       0,  0, 18,  0, }, // Indian_Ocean
    {  328,  933,       0,  319266,       0,  0, 14,  0, }, // Indochina
    {  328,  943,       0,  319280,       0,  0, 20,  0, }, // Indonesia_Central
    {  328,  961,       0,  319300,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  328,  979,       0,  319323,       0,  0, 23,  0, }, // Indonesia_Western
    {  328,  997,  319346,  319354,  319367,  8, 13, 13, }, // Iran
    {  328, 1008,  319380,  319394,  319413, 14, 19, 22, }, // Irkutsk
    {  328, 1016,  319435,  319445,  319460, 10, 15, 15, }, // Israel
    {  328, 1023,  319475,  319484,  319498,  9, 14, 14, }, // Japan
    {  328, 1029,  319512,  319542,  319582, 30, 40, 38, }, // Kamchatka
    {  328, 1047,       0,  319620,       0,  0, 14,  0, }, // Kazakhstan
    {  328, 1058,       0,  319634,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  328, 1077,       0,  319654,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  328, 1106,  319675,  319685,  319700, 10, 15, 15, }, // Korea
    {  328, 1112,       0,  319715,       0,  0,  9,  0, }, // Kosrae
    {  328, 1119,  319724,  319743,  319767, 19, 24, 27, }, // Krasnoyarsk
    {  328, 1151,       0,  319794,       0,  0, 17,  0, }, // Kyrgystan
    {  328, 1161,       0,  319811,       0,  0,  8,  0, }, // Lanka
    {  328, 1175,       0,  319819,       0,  0, 18,  0, }, // Line_Islands
    {  328, 1188,  319837,  319850,  319868, 13, 18, 18, }, // Lord_Howe
    {  328, 1198,  319886,  319894,  319912,  8, 18, 16, }, // Macau
    {  328, 1204,  319928,  319937,  319951,  9, 14, 17, }, // Magadan
    {  328, 1219,       0,  319968,       0,  0, 11,  0, }, // Malaysia
    {  328, 1228,       0,  319979,       0,  0, 10,  0, }, // Maldives
    {  328, 1237,       0,  319989,       0,  0, 14,  0, }, // Marquesas
    {  328, 1247,       0,  320003,       0,  0, 20,  0, }, // Marshall_Islands
    {  328, 1264,  320023,  320035,  320052, 12, 17, 20, }, // Mauritius
    {  328, 1274,       0,  320072,       0,  0,  8,  0, }, // Mawson
    {  328, 1281,  320080,  320100,  320125, 20, 25, 25, }, // Mexico_Pacific
    {  328, 1296,  320150,  320164,  320183, 14, 19, 21, }, // Mongolia
    {  328, 1305,  320204,  320214,  320229, 10, 15, 18, }, // Moscow
    {  328, 1312,       0,  320247,       0,  0, 12,  0, }, // Myanmar
    {  328, 1320,       0,  320259,       0,  0,  8,  0, }, // Nauru
    {  328, 1326,       0,  320267,       0,  0,  9,  0, }, // Nepal
    {  328, 1332,  320276,  320295,  320319, 19, 24, 27, }, // New_Caledonia
    {  328, 1346,  320346,  320361,  320381, 15, 20, 20, }, // New_Zealand
    {  328, 1358,  320401,  320420,  320444, 19, 24, 24, }, // Newfoundland
    {  328, 1371,       0,  320468,       0,  0,  8,  0, }, // Niue
    {  328, 1376,  320476,  320494,  320517, 18, 23, 23, }, // Norfolk
    {  328, 1384,  320540,  320565,  320595, 25, 30, 31, }, // Noronha
    {  328, 1392,       0,  320626,       0,  0, 24,  0, }, // North_Mariana
    {  328, 1406,  320650,  320666,  320687, 16, 21, 24, }, // Novosibirsk
    {  328, 1418,  320711,  320721,  320736, 10, 15, 18, }, // Omsk
    {  328, 1428,  320754,  320767,  320785, 13, 18, 21, }, // Pakistan
    {  328, 1437,       0,  320806,       0,  0,  8,  0, }, // Palau
    {  328, 1443,       0,  320814,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  328, 1460,  320833,  320845,  320862, 12, 17, 20, }, // Paraguay
    {  328, 1469,  320882,  320890,  320903,  8, 13, 16, }, // Peru
    {  328, 1474,  320919,  320931,  320948, 12, 17, 20, }, // Philippines
    {  328, 1486,       0,  320968,       0,  0, 21,  0, }, // Phoenix_Islands
    {  328, 1502,  320989,  321017,  321050, 28, 33, 33, }, // Pierre_Miquelon
    {  328, 1518,       0,  321083,       0,  0, 13,  0, }, // Pitcairn
    {  328, 1527,       0,  321096,       0,  0,  9,  0, }, // Ponape
    {  328, 1534,       0,  321105,       0,  0, 14,  0, }, // Pyongyang
    {  328, 1544,  321119,  321133,  321157, 14, 24, 22, }, // Qyzylorda
    {  328, 1554,       0,  321179,       0,  0, 12,  0, }, // Reunion
    {  328, 1562,       0,  321191,       0,  0, 10,  0, }, // Rothera
    {  328, 1570,  321201,  321211,  321226, 10, 15, 18, }, // Sakhalin
    {  328, 1579,  321244,  321253,  321272,  9, 19, 17, }, // Samara
    {  328, 1596,  321289,  321297,  321310,  8, 13, 16, }, // Samoa
    {  328, 1602,       0,  321326,       0,  0, 10,  0, }, // Seychelles
    {  328, 1624,       0,  321336,       0,  0, 17,  0, }, // Singapore
    {  328, 1634,       0,  321353,       0,  0, 20,  0, }, // Solomon
    {  328, 1642,       0,  321373,       0,  0, 21,  0, }, // South_Georgia
    {  328, 1656,       0,  321394,       0,  0, 11,  0, }, // Suriname
    {  328, 1676,       0,  321405,       0,  0, 10,  0, }, // Syowa
    {  328, 1682,       0,  321415,       0,  0, 10,  0, }, // Tahiti
    {  328, 1689,  321425,  321435,  321450, 10, 15, 15, }, // Taipei
    {  328, 1696,       0,  321465,       0,  0, 15,  0, }, // Tajikistan
    {  328, 1724,       0,  321480,       0,  0, 10,  0, }, // Tokelau
    {  328, 1732,  321490,  321499,  321513,  9, 14, 17, }, // Tonga
    {  328, 1738,       0,  321530,       0,  0,  8,  0, }, // Truk
    {  328, 1750,  321538,  321556,  321579, 18, 23, 26, }, // Turkmenistan
    {  328, 1763,       0,  321605,       0,  0,  9,  0, }, // Tuvalu
    {  328, 1777,  321614,  321625,  321641, 11, 16, 19, }, // Uruguay
    {  328, 1792,  321660,  321676,  321697, 16, 21, 24, }, // Uzbekistan
    {  328, 1803,  321721,  321730,  321744,  9, 14, 17, }, // Vanuatu
    {  328, 1811,       0,  321761,       0,  0, 13,  0, }, // Venezuela
    {  328, 1821,  321774,  321791,  321813, 17, 22, 25, }, // Vladivostok
    {  328, 1833,  321838,  321853,  321873, 15, 20, 23, }, // Volgograd
    {  328, 1843,       0,  321780,       0,  0, 11,  0, }, // Vostok
    {  328, 1850,       0,  321896,       0,  0, 15,  0, }, // Wake
    {  328, 1855,       0,  321911,       0,  0, 22,  0, }, // Wallis
    {  328, 1862,  321933,  321946,  321964, 13, 18, 21, }, // Yakutsk
    {  328, 1870,  321985,  322002,  322024, 17, 22, 25, }, // Yekaterinburg
    {  328, 1892,       0,  322049,       0,  0,  9,  0, }, // Yukon
    {  330,    5,       0,  322058,       0,  0, 19,  0, }, // Afghanistan Hausa/Latin/Nigeria
    {  330,   17,       0,  322077,       0,  0, 25,  0, }, // Africa_Central
    {  330,   32,       0,  322102,       0,  0, 23,  0, }, // Africa_Eastern
    {  330,   65,       0,  322125,       0,  0, 32,  0, }, // Africa_Southern
    {  330,   81,  322157,  322180,  322213, 23, 33, 33, }, // Africa_Western
    {  330,  107,  322246,  322260,  322284, 14, 24, 22, }, // Alaska
    {  330,  135,  322306,  322320,  322344, 14, 24, 24, }, // Amazon
    {  330,  142,  322368,  322405,  322454, 37, 49, 45, }, // America_Central
    {  330,  158,  322499,  322535,  322580, 36, 45, 43, }, // America_Eastern
    {  330,  174,  322623,  322656,  322700, 33, 44, 39, }, // America_Mountain
    {  330,  191,  322739,  322763,  322798, 24, 35, 31, }, // America_Pacific
    {  330,  214,  322829,  322841,  322863, 12, 22, 20, }, // Apia
    {  330,  232,  322883,  322898,  322923, 15, 25, 23, }, // Arabian
    {  330,  240,  322946,  322963,  322990, 17, 27, 27, }, // Argentina
    {  330,  250,  323017,  323043,  323079, 26, 36, 36, }, // Argentina_Western
    {  330,  268,  323115,  323130,  323155, 15, 25, 25, }, // Armenia
    {  330,  286,  323180,  323225,  323280, 45, 55, 53, }, // Atlantic
    {  330,  295,  323333,  323360,  323398, 27, 38, 36, }, // Australia_Central
    {  330,  313,  323434,  323471,  323518, 37, 47, 45, }, // Australia_CentralWestern
    {  330,  338,  323563,  323591,  323627, 28, 36, 36, }, // Australia_Eastern
    {  330,  356,  323663,  323690,  323727, 27, 37, 35, }, // Australia_Western
    {  330,  374,  323762,  323780,  323808, 18, 28, 28, }, // Azerbaijan
    {  330,  385,  323836,  323850,  323876, 14, 26, 23, }, // Azores
    {  330,  397,  323899,  323917,  323945, 18, 28, 28, }, // Bangladesh
    {  330,  415,       0,  323973,       0,  0, 14,  0, }, // Bhutan
    {  330,  422,       0,  323987,       0,  0, 15,  0, }, // Bolivia
    {  330,  437,  324002,  324019,  324046, 17, 27, 27, }, // Brasilia
    {  330,  454,       0,  324073,       0,  0, 25,  0, }, // Brunei
    {  330,  461,  324098,  324116,  324144, 18, 28, 28, }, // Cape_Verde
    {  330,  478,       0,  324172,       0,  0, 26,  0, }, // Chamorro
    {  330,  487,  324198,  324213,  324238, 15, 25, 23, }, // Chatham
    {  330,  495,  324261,  324274,  324297, 13, 23, 23, }, // Chile
    {  330,  501,  324320,  324331,  324352, 11, 21, 19, }, // China
    {  330,  507,       0,  324371,       0,  0, 24,  0, }, // Christmas
    {  330,  517,       0,  324395,       0,  0, 21,  0, }, // Cocos
    {  330,  523,  324416,  324432,  324458, 16, 26, 26, }, // Colombia
    {  330,  532,  324484,  324504,  324534, 20, 30, 36, }, // Cook
    {  330,  537,  324570,  324584,  324606, 14, 22, 20, }, // Cuba
    {  330,  548,       0,  324626,       0,  0, 13,  0, }, // Davis
    {  330,  564,       0,  324639,       0,  0, 24,  0, }, // DumontDUrville
    {  330,  601,       0,  324663,       0,  0, 18,  0, }, // East_Timor
    {  330,  612,  324681,  324702,  324733, 21, 31, 31, }, // Easter
    {  330,  619,       0,  324764,       0,  0, 15,  0, }, // Ecuador
    {  330,  627,  324779,  324802,  324829, 23, 27, 30, }, // Europe_Central
    {  330,  642,  324859,  324879,  324903, 20, 24, 30, }, // Europe_Eastern
    {  330,  657,       0,  324933,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  330,  680,  324964,  324988,  325016, 24, 28, 38, }, // Europe_Western
    {  330,  695,  325054,  325078,  325112, 24, 34, 34, }, // Falkland
    {  330,  704,  325146,  325158,  325180, 12, 22, 22, }, // Fiji
    {  330,  709,       0,  325202,       0,  0, 21,  0, }, // French_Guiana
    {  330,  723,       0,  325223,       0,  0, 37,  0, }, // French_Southern
    {  330,  746,       0,  325260,       0,  0, 17,  0, }, // Galapagos
    {  330,  756,       0,  325277,       0,  0, 15,  0, }, // Gambier
    {  330,  764,  325292,  325307,  325332, 15, 25, 25, }, // Georgia
    {  330,  772,       0,  325357,       0,  0, 23,  0, }, // Gilbert_Islands
    {  330,  788,       0,  325380,       0,  0, 26,  0, }, // GMT
    {  330,  830,  325406,  325432,  325468, 26, 36, 34, }, // Greenland_Eastern
    {  330,  848,  325502,  325528,  325564, 26, 36, 34, }, // Greenland_Western
    {  330,  871,       0,  325598,       0,  0, 22,  0, }, // Gulf
    {  330,  876,       0,  325620,       0,  0, 14,  0, }, // Guyana
    {  330,  883,  325634,  325659,  325692, 25, 33, 31, }, // Hawaii_Aleutian
    {  330,  899,  325723,  325740,  325767, 17, 27, 27, }, // Hong_Kong
    {  330,  909,  325794,  325806,  325828, 12, 22, 22, }, // Hovd
    {  330,  914,       0,  325850,       0,  0, 24,  0, }, // India
    {  330,  920,       0,  325874,       0,  0, 20,  0, }, // Indian_Ocean
    {  330,  933,       0,  325894,       0,  0, 17,  0, }, // Indochina
    {  330,  943,       0,  325911,       0,  0, 28,  0, }, // Indonesia_Central
    {  330,  961,       0,  325939,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  330,  979,       0,  325965,       0,  0, 26,  0, }, // Indonesia_Western
    {  330,  997,  325991,  326003,  326025, 12, 22, 20, }, // Iran
    {  330, 1008,  326045,  326060,  326085, 15, 25, 25, }, // Irkutsk
    {  330, 1016,  326110,  326126,  326152, 16, 26, 31, }, // Israel
    {  330, 1023,  326183,  326196,  326219, 13, 23, 28, }, // Japan
    {  330, 1047,       0,  326247,       0,  0, 18,  0, }, // Kazakhstan
    {  330, 1058,       0,  326265,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  330, 1077,       0,  326292,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  330, 1106,  326319,  326333,  326357, 14, 24, 22, }, // Korea
    {  330, 1112,       0,  326379,       0,  0, 14,  0, }, // Kosrae
    {  330, 1119,  326393,  326412,  326441, 19, 29, 29, }, // Krasnoyarsk
    {  330, 1151,       0,  326470,       0,  0, 18,  0, }, // Kyrgystan
    {  330, 1175,       0,  326488,       0,  0, 20,  0, }, // Line_Islands
    {  330, 1188,  326508,  326525,  326552, 17, 27, 30, }, // Lord_Howe
    {  330, 1204,  326582,  326597,  326622, 15, 25, 25, }, // Magadan
    {  330, 1219,       0,  326647,       0,  0, 16,  0, }, // Malaysia
    {  330, 1228,       0,  326663,       0,  0, 16,  0, }, // Maldives
    {  330, 1237,       0,  326679,       0,  0, 17,  0, }, // Marquesas
    {  330, 1247,       0,  326696,       0,  0, 24,  0, }, // Marshall_Islands
    {  330, 1264,  326720,  326737,  326764, 17, 27, 27, }, // Mauritius
    {  330, 1274,       0,  326791,       0,  0, 14,  0, }, // Mawson
    {  330, 1281,  326805,  326828,  326861, 23, 33, 31, }, // Mexico_Pacific
    {  330, 1296,  326892,  326911,  326940, 19, 29, 29, }, // Mongolia
    {  330, 1305,  326969,  326983,  327007, 14, 24, 24, }, // Moscow
    {  330, 1312,       0,  327031,       0,  0, 15,  0, }, // Myanmar
    {  330, 1320,       0,  327046,       0,  0, 13,  0, }, // Nauru
    {  330, 1326,       0,  327059,       0,  0, 13,  0, }, // Nepal
    {  330, 1332,  327072,  327093,  327124, 21, 31, 31, }, // New_Caledonia
    {  330, 1346,  327155,  327174,  327203, 19, 29, 27, }, // New_Zealand
    {  330, 1358,  327230,  327250,  327281, 20, 31, 27, }, // Newfoundland
    {  330, 1371,       0,  327308,       0,  0, 12,  0, }, // Niue
    {  330, 1376,  327320,  327342,  327374, 22, 32, 30, }, // Norfolk
    {  330, 1384,  327404,  327431,  327468, 27, 37, 37, }, // Noronha
    {  330, 1406,  327505,    4817,  327524, 19, 25, 29, }, // Novosibirsk
    {  330, 1418,  327553,  327565,  327587, 12, 22, 22, }, // Omsk
    {  330, 1428,  327609,  327625,  327651, 16, 26, 26, }, // Pakistan
    {  330, 1437,       0,  327677,       0,  0, 13,  0, }, // Palau
    {  330, 1443,       0,  327690,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  330, 1460,  327714,  327730,  327756, 16, 26, 26, }, // Paraguay
    {  330, 1469,  327782,  327794,  327816, 12, 22, 22, }, // Peru
    {  330, 1474,  327838,  327856,  327884, 18, 28, 28, }, // Philippines
    {  330, 1486,       0,  327912,       0,  0, 23,  0, }, // Phoenix_Islands
    {  330, 1502,  327935,  327965,  328005, 30, 40, 38, }, // Pierre_Miquelon
    {  330, 1518,       0,  328043,       0,  0, 16,  0, }, // Pitcairn
    {  330, 1527,       0,  328059,       0,  0, 14,  0, }, // Ponape
    {  330, 1534,       0,  328073,       0,  0, 17,  0, }, // Pyongyang
    {  330, 1554,       0,  328090,       0,  0, 15,  0, }, // Reunion
    {  330, 1562,       0,  328105,       0,  0, 15,  0, }, // Rothera
    {  330, 1570,  328120,  328136,  328162, 16, 26, 26, }, // Sakhalin
    {  330, 1596,  328188,  328201,  328224, 13, 23, 26, }, // Samoa
    {  330, 1602,       0,  328250,       0,  0, 18,  0, }, // Seychelles
    {  330, 1624,       0,  328268,       0,  0, 27,  0, }, // Singapore
    {  330, 1634,       0,  328295,       0,  0, 23,  0, }, // Solomon
    {  330, 1642,       0,  328318,       0,  0, 24,  0, }, // South_Georgia
    {  330, 1656,       0,  328342,       0,  0, 16,  0, }, // Suriname
    {  330, 1676,       0,  328358,       0,  0, 13,  0, }, // Syowa
    {  330, 1682,       0,  328371,       0,  0, 14,  0, }, // Tahiti
    {  330, 1689,  328385,  328399,  328423, 14, 24, 22, }, // Taipei
    {  330, 1696,       0,  328445,       0,  0, 18,  0, }, // Tajikistan
    {  330, 1724,       0,  328463,       0,  0, 15,  0, }, // Tokelau
    {  330, 1732,  328478,  328491,  328514, 13, 23, 23, }, // Tonga
    {  330, 1738,       0,  328537,       0,  0, 13,  0, }, // Truk
    {  330, 1750,  328550,  328570,  328600, 20, 30, 30, }, // Turkmenistan
    {  330, 1763,       0,  328630,       0,  0, 14,  0, }, // Tuvalu
    {  330, 1777,  328644,  328659,  328684, 15, 25, 25, }, // Uruguay
    {  330, 1792,  328709,  328727,  328755, 18, 28, 28, }, // Uzbekistan
    {  330, 1803,  328783,  328798,  328823, 15, 25, 25, }, // Vanuatu
    {  330, 1811,       0,  328848,       0,  0, 17,  0, }, // Venezuela
    {  330, 1821,  328865,  328884,  328913, 19, 29, 29, }, // Vladivostok
    {  330, 1833,  328942,  328959,  328986, 17, 27, 27, }, // Volgograd
    {  330, 1843,       0,  329013,       0,  0, 14,  0, }, // Vostok
    {  330, 1850,       0,  329027,       0,  0, 19,  0, }, // Wake
    {  330, 1855,       0,  329046,       0,  0, 24,  0, }, // Wallis
    {  330, 1862,  329070,  329085,  329110, 15, 25, 25, }, // Yakutsk
    {  330, 1870,  329135,  329156,  329187, 21, 31, 31, }, // Yekaterinburg
    {  330, 1892,       0,  329218,       0,  0, 13,  0, }, // Yukon
    {  336,    5,       0,  329231,       0,  0, 13,  0, }, // Afghanistan Hebrew/Hebrew/Israel
    {  336,   17,       0,  329244,       0,  0, 16,  0, }, // Africa_Central
    {  336,   32,       0,  329260,       0,  0, 16,  0, }, // Africa_Eastern
    {  336,   65,       0,  329276,       0,  0, 16,  0, }, // Africa_Southern
    {  336,   81,  329292,  329308,  329331, 16, 23, 22, }, // Africa_Western
    {  336,  107,  329353,  329363,  329380, 10, 17, 16, }, // Alaska
    {  336,  128,  329396,  329406,  329423, 10, 17, 16, }, // Almaty
    {  336,  135,  329439,  329450,  329468, 11, 18, 17, }, // Amazon
    {  336,  142,  329485,  329500,  329522, 15, 22, 21, }, // America_Central
    {  336,  158,  329543,  329559,  329582, 16, 23, 22, }, // America_Eastern
    {  336,  174,  329604,  329626,  329655, 22, 29, 28, }, // America_Mountain
    {  336,  191,  329683,  329698,  329720, 15, 22, 21, }, // America_Pacific
    {  336,  207,  329741,  329751,  329766, 10, 15, 14, }, // Anadyr
    {  336,  214,  329780,  329789,  329805,  9, 16, 15, }, // Apia
    {  336,  219,  329820,  329830,  329847, 10, 17, 16, }, // Aqtau
    {  336,  225,  329863,  329874,  329893, 11, 19, 18, }, // Aqtobe
    {  336,  232,  329911,  329927,  329950, 16, 23, 22, }, // Arabian
    {  336,  240,  329972,  329985,  330005, 13, 20, 19, }, // Argentina
    {  336,  250,  330024,  330042,  330067, 18, 25, 24, }, // Argentina_Western
    {  336,  268,  330091,  330102,  330120, 11, 18, 17, }, // Armenia
    {  336,  286,  330137,  330159,  330188, 22, 29, 28, }, // Atlantic
    {  336,  295,  330216,  330234,  330259, 18, 25, 24, }, // Australia_Central
    {  336,  313,  330283,  330306,  330336, 23, 30, 29, }, // Australia_CentralWestern
    {  336,  338,  330365,  330383,  330408, 18, 25, 24, }, // Australia_Eastern
    {  336,  356,  330432,  330450,  330475, 18, 25, 24, }, // Australia_Western
    {  336,  374,  330499,  330513,  330534, 14, 21, 20, }, // Azerbaijan
    {  336,  385,  330554,  330573,  330599, 19, 26, 25, }, // Azores
    {  336,  397,  330624,  330635,  330653, 11, 18, 17, }, // Bangladesh
    {  336,  415,       0,  330670,       0,  0, 10,  0, }, // Bhutan
    {  336,  422,       0,  330680,       0,  0, 12,  0, }, // Bolivia
    {  336,  437,  330692,  330704,  330723, 12, 19, 18, }, // Brasilia
    {  336,  454,       0,  330741,       0,  0, 20,  0, }, // Brunei
    {  336,  461,  330761,  330773,  330792, 12, 19, 18, }, // Cape_Verde
    {  336,  478,       0,  330810,       0,  0, 12,  0, }, // Chamorro
    {  336,  487,  330822,  330833,  330851, 11, 18, 17, }, // Chatham
    {  336,  495,  330868,  330878,  330895, 10, 17, 16, }, // Chile
    {  336,  501,  330911,  330919,  330934,  8, 15, 14, }, // China
    {  336,  507,       0,  330948,       0,  0, 16,  0, }, // Christmas
    {  336,  517,       0,  330964,       0,  0, 14,  0, }, // Cocos
    {  336,  523,  330978,  330991,  331011, 13, 20, 19, }, // Colombia
    {  336,  532,  330964,  331030,  331049, 12, 19, 25, }, // Cook
    {  336,  537,  331074,  331083,  331099,  9, 16, 15, }, // Cuba
    {  336,  548,       0,  331114,       0,  0, 11,  0, }, // Davis
    {  336,  564,       0,  331125,       0,  0, 20,  0, }, // DumontDUrville
    {  336,  601,       0,  331145,       0,  0, 15,  0, }, // East_Timor
    {  336,  612,  331160,  331173,  331193, 13, 20, 19, }, // Easter
    {  336,  619,       0,  331212,       0,  0, 12,  0, }, // Ecuador
    {  336,  627,  331224,  331240,  331263, 16, 23, 22, }, // Europe_Central
    {  336,  642,  331285,  331301,  331324, 16, 23, 22, }, // Europe_Eastern
    {  336,  657,       0,  331346,       0,  0, 10,  0, }, // Europe_Further_Eastern
    {  336,  680,  331356,  331372,  331395, 16, 23, 22, }, // Europe_Western
    {  336,  695,  331417,  331432,  331454, 15, 22, 21, }, // Falkland
    {  336,  704,  331475,  331485,  331502, 10, 17, 16, }, // Fiji
    {  336,  709,       0,  331518,       0,  0, 18,  0, }, // French_Guiana
    {  336,  723,       0,  331536,       0,  0, 41,  0, }, // French_Southern
    {  336,  746,       0,  331577,       0,  0, 16,  0, }, // Galapagos
    {  336,  756,       0,  331593,       0,  0, 15,  0, }, // Gambier
    {  336,  764,  331608,  331620,  331639, 12, 19, 18, }, // Georgia
    {  336,  772,       0,  331657,       0,  0, 15,  0, }, // Gilbert_Islands
    {  336,  788,       0,  331672,       0,  0, 13,  0, }, // GMT
    {  336,  830,  331685,  331702,  331726, 17, 24, 23, }, // Greenland_Eastern
    {  336,  848,  331749,  331766,  331790, 17, 24, 23, }, // Greenland_Western
    {  336,  866,       0,  331813,       0,  0,  9,  0, }, // Guam
    {  336,  871,       0,  331822,       0,  0, 17,  0, }, // Gulf
    {  336,  876,       0,  331518,       0,  0, 10,  0, }, // Guyana
    {  336,  883,  331839,  331865,  331898, 26, 33, 32, }, // Hawaii_Aleutian
    {  336,  899,  331930,  331944,  331965, 14, 21, 20, }, // Hong_Kong
    {  336,  909,  331985,  331994,  332010,  9, 16, 15, }, // Hovd
    {  336,  914,       0,  332025,       0,  0,  9,  0, }, // India
    {  336,  920,       0,  332034,       0,  0, 20,  0, }, // Indian_Ocean
    {  336,  933,       0,  332054,       0,  0, 13,  0, }, // Indochina
    {  336,  943,       0,  332067,       0,  0, 19,  0, }, // Indonesia_Central
    {  336,  961,       0,  332086,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  336,  979,       0,  332105,       0,  0, 19,  0, }, // Indonesia_Western
    {  336,  997,  332124,  332134,  332151, 10, 17, 16, }, // Iran
    {  336, 1008,  332167,  332180,  332200, 13, 20, 19, }, // Irkutsk
    {  336, 1016,  332219,  332229,  332246, 10, 17, 16, }, // Israel
    {  336, 1023,  332262,  332270,  332285,  8, 15, 14, }, // Japan
    {  336, 1029,  332299,  332324,  332354, 25, 30, 29, }, // Kamchatka
    {  336, 1047,       0,  332383,       0,  0, 11,  0, }, // Kazakhstan
    {  336, 1058,       0,  332394,       0,  0, 16,  0, }, // Kazakhstan_Eastern
    {  336, 1077,       0,  332410,       0,  0, 16,  0, }, // Kazakhstan_Western
    {  336, 1106,  332426,  332437,  332455, 11, 18, 17, }, // Korea
    {  336, 1112,       0,  332472,       0,  0, 11,  0, }, // Kosrae
    {  336, 1119,  332483,  332498,  332520, 15, 22, 21, }, // Krasnoyarsk
    {  336, 1151,       0,  332541,       0,  0, 14,  0, }, // Kyrgystan
    {  336, 1175,       0,  332555,       0,  0, 13,  0, }, // Line_Islands
    {  336, 1188,  332568,  332585,  332609, 17, 24, 23, }, // Lord_Howe
    {  336, 1198,  332632,  332641,  332655,  9, 14, 13, }, // Macau
    {  336, 1204,  332668,  332677,  332693,  9, 16, 15, }, // Magadan
    {  336, 1219,       0,  332708,       0,  0, 10,  0, }, // Malaysia
    {  336, 1228,       0,  332718,       0,  0, 20,  0, }, // Maldives
    {  336, 1237,       0,  332738,       0,  0, 14,  0, }, // Marquesas
    {  336, 1247,       0,  332752,       0,  0, 13,  0, }, // Marshall_Islands
    {  336, 1264,  332765,  332779,  332800, 14, 21, 20, }, // Mauritius
    {  336, 1274,       0,  332820,       0,  0, 11,  0, }, // Mawson
    {  336, 1281,  332831,  332847,  332870, 16, 23, 22, }, // Mexico_Pacific
    {  336, 1296,  332892,  332908,  332931, 16, 23, 22, }, // Mongolia
    {  336, 1305,  332953,  332964,  332982, 11, 18, 17, }, // Moscow
    {  336, 1312,       0,  332999,       0,  0, 11,  0, }, // Myanmar
    {  336, 1320,       0,  333010,       0,  0, 10,  0, }, // Nauru
    {  336, 1326,       0,  333020,       0,  0,  9,  0, }, // Nepal
    {  336, 1332,  333029,  333047,  333072, 18, 25, 24, }, // New_Caledonia
    {  336, 1346,  333096,  333110,  333131, 14, 21, 20, }, // New_Zealand
    {  336, 1358,  333151,  333167,  333190, 16, 23, 22, }, // Newfoundland
    {  336, 1371,       0,  333212,       0,  0, 10,  0, }, // Niue
    {  336, 1376,  333222,  333237,  333259, 15, 22, 21, }, // Norfolk
    {  336, 1384,  333280,  333302,  333331, 22, 29, 28, }, // Noronha
    {  336, 1392,       0,  333359,       0,  0, 24,  0, }, // North_Mariana
    {  336, 1406,  333383,  333399,  333422, 16, 23, 22, }, // Novosibirsk
    {  336, 1418,  333444,  333454,  333471, 10, 17, 16, }, // Omsk
    {  336, 1428,  333487,  333498,  333516, 11, 18, 17, }, // Pakistan
    {  336, 1437,       0,  333533,       0,  0,  9,  0, }, // Palau
    {  336, 1443,       0,  333542,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  336, 1460,  333565,  333577,  333596, 12, 19, 18, }, // Paraguay
    {  336, 1469,  333614,  333622,  333637,  8, 15, 14, }, // Peru
    {  336, 1474,  333651,  333666,  333688, 15, 22, 21, }, // Philippines
    {  336, 1486,       0,  333709,       0,  0, 15,  0, }, // Phoenix_Islands
    {  336, 1502,  333724,  333745,  333773, 21, 28, 27, }, // Pierre_Miquelon
    {  336, 1518,       0,  333800,       0,  0, 11,  0, }, // Pitcairn
    {  336, 1527,       0,  333811,       0,  0, 11,  0, }, // Ponape
    {  336, 1534,       0,  333822,       0,  0, 14,  0, }, // Pyongyang
    {  336, 1544,  333836,  333850,  333871, 14, 21, 20, }, // Qyzylorda
    {  336, 1554,       0,  333891,       0,  0, 12,  0, }, // Reunion
    {  336, 1562,       0,  333903,       0,  0, 11,  0, }, // Rothera
    {  336, 1570,  333914,  333924,  333941, 10, 17, 16, }, // Sakhalin
    {  336, 1579,  333957,  333966,  333980,  9, 14, 13, }, // Samara
    {  336, 1596,  333993,  334003,  334020, 10, 17, 16, }, // Samoa
    {  336, 1602,       0,  334036,       0,  0, 14,  0, }, // Seychelles
    {  336, 1624,       0,  334050,       0,  0, 12,  0, }, // Singapore
    {  336, 1634,       0,  334062,       0,  0, 13,  0, }, // Solomon
    {  336, 1642,       0,  334075,       0,  0, 18,  0, }, // South_Georgia
    {  336, 1656,       0,  334093,       0,  0, 12,  0, }, // Suriname
    {  336, 1676,       0,  334105,       0,  0, 11,  0, }, // Syowa
    {  336, 1682,       0,  334116,       0,  0, 10,  0, }, // Tahiti
    {  336, 1689,  334126,  334137,  334155, 11, 18, 17, }, // Taipei
    {  336, 1696,       0,  334172,       0,  0, 14,  0, }, // Tajikistan
    {  336, 1724,       0,  334186,       0,  0, 11,  0, }, // Tokelau
    {  336, 1732,  334197,  334207,  334224, 10, 17, 16, }, // Tonga
    {  336, 1738,       0,  334240,       0,  0,  9,  0, }, // Truk
    {  336, 1750,  334249,  334264,  334286, 15, 22, 21, }, // Turkmenistan
    {  336, 1763,       0,  334307,       0,  0, 11,  0, }, // Tuvalu
    {  336, 1777,  334318,  334332,  334353, 14, 21, 20, }, // Uruguay
    {  336, 1792,  334373,  334387,  334408, 14, 21, 20, }, // Uzbekistan
    {  336, 1803,  334428,  334439,  334457, 11, 18, 17, }, // Vanuatu
    {  336, 1811,       0,  334474,       0,  0, 12,  0, }, // Venezuela
    {  336, 1821,  334486,  334501,  334523, 15, 22, 21, }, // Vladivostok
    {  336, 1833,  334544,  334557,  334577, 13, 20, 19, }, // Volgograd
    {  336, 1843,       0,  334596,       0,  0, 11,  0, }, // Vostok
    {  336, 1850,       0,  334607,       0,  0, 13,  0, }, // Wake
    {  336, 1855,       0,  334620,       0,  0, 19,  0, }, // Wallis
    {  336, 1862,  334639,  334650,  334668, 11, 18, 17, }, // Yakutsk
    {  336, 1870,  334685,  334700,  334722, 15, 22, 21, }, // Yekaterinburg
    {  336, 1892,       0,  334743,       0,  0, 10,  0, }, // Yukon
    {  337,    5,       0,  334753,       0,  0, 16,  0, }, // Afghanistan Hindi/Devanagari/India
    {  337,   17,       0,  334769,       0,  0, 17,  0, }, // Africa_Central
    {  337,   32,       0,  334786,       0,  0, 19,  0, }, // Africa_Eastern
    {  337,   65,       0,  334805,       0,  0, 24,  0, }, // Africa_Southern
    {  337,   81,  334829,  334848,  334872, 19, 24, 32, }, // Africa_Western
    {  337,  107,  334904,  334915,  334932, 11, 17, 19, }, // Alaska
    {  337,  135,  334951,  334961,  334976, 10, 15, 23, }, // Amazon
    {  337,  142,  334999,  335026,  335058, 27, 32, 34, }, // America_Central
    {  337,  158,  335092,  335117,  335147, 25, 30, 32, }, // America_Eastern
    {  337,  174,  335179,  335205,  335236, 26, 31, 33, }, // America_Mountain
    {  337,  191,  335269,  335295,  335326, 26, 31, 33, }, // America_Pacific
    {  337,  207,  335359,  335370,  335386, 11, 16, 24, }, // Anadyr
    {  337,  214,  335410,  335418,  335431,  8, 13, 15, }, // Apia
    {  337,  232,  335446,  335453,  335465,  7, 12, 14, }, // Arabian
    {  337,  240,  335479,  335493,  335512, 14, 19, 27, }, // Argentina
    {  337,  250,  335539,  335561,  335588, 22, 27, 35, }, // Argentina_Western
    {  337,  268,  335623,  335636,  335654, 13, 18, 26, }, // Armenia
    {  337,  286,  335680,  335692,  335709, 12, 17, 19, }, // Atlantic
    {  337,  295,  335728,  335749,  335780, 21, 31, 33, }, // Australia_Central
    {  337,  313,  335813,  335847,  335886, 34, 39, 41, }, // Australia_CentralWestern
    {  337,  338,  335927,  335949,  335978, 22, 29, 31, }, // Australia_Eastern
    {  337,  356,  336009,  336032,  336061, 23, 29, 31, }, // Australia_Western
    {  337,  374,  336092,  336105,  336123, 13, 18, 26, }, // Azerbaijan
    {  337,  385,  336149,  336160,  336176, 11, 16, 24, }, // Azores
    {  337,  397,  336200,  336214,  336233, 14, 19, 27, }, // Bangladesh
    {  337,  415,       0,  336260,       0,  0,  9,  0, }, // Bhutan
    {  337,  422,       0,  336269,       0,  0, 12,  0, }, // Bolivia
    {  337,  437,  336281,  336296,  336316, 15, 20, 28, }, // Brasilia
    {  337,  454,       0,  336344,       0,  0, 21,  0, }, // Brunei
    {  337,  461,  336365,  336377,  336394, 12, 17, 25, }, // Cape_Verde
    {  337,  478,       0,  336419,       0,  0, 15,  0, }, // Chamorro
    {  337,  487,  336434,  336442,  336455,  8, 13, 15, }, // Chatham
    {  337,  495,  336470,  336478,  336491,  8, 13, 21, }, // Chile
    {  337,  501,  336512,  336519,  336531,  7, 12, 14, }, // China
    {  337,  507,       0,  336545,       0,  0, 17,  0, }, // Christmas
    {  337,  517,       0,  336562,       0,  0, 19,  0, }, // Cocos
    {  337,  523,  336581,  336593,  336610, 12, 17, 25, }, // Colombia
    {  337,  532,  336635,  336652,  336674, 17, 22, 37, }, // Cook
    {  337,  537,  336711,  336721,  336736, 10, 15, 17, }, // Cuba
    {  337,  548,       0,  336753,       0,  0,  9,  0, }, // Davis
    {  337,  564,       0,  336762,       0,  0, 23,  0, }, // DumontDUrville
    {  337,  601,       0,  336785,       0,  0, 16,  0, }, // East_Timor
    {  337,  612,  336801,  336816,  336836, 15, 20, 28, }, // Easter
    {  337,  619,       0,  336864,       0,  0, 12,  0, }, // Ecuador
    {  337,  627,  336876,  336892,  336913, 16, 21, 31, }, // Europe_Central
    {  337,  642,  336944,  336962,  336985, 18, 23, 31, }, // Europe_Eastern
    {  337,  657,       0,  337016,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  337,  680,  337039,  337058,  337082, 19, 24, 33, }, // Europe_Western
    {  337,  695,  337115,  337137,  337164, 22, 27, 35, }, // Falkland
    {  337,  704,  337199,  337208,  337222,  9, 14, 22, }, // Fiji
    {  337,  709,       0,  337244,       0,  0, 18,  0, }, // French_Guiana
    {  337,  723,       0,  337262,       0,  0, 33,  0, }, // French_Southern
    {  337,  746,       0,  337295,       0,  0, 16,  0, }, // Galapagos
    {  337,  756,       0,  337311,       0,  0, 11,  0, }, // Gambier
    {  337,  764,  337322,  337334,  337351, 12, 17, 25, }, // Georgia
    {  337,  772,       0,  337376,       0,  0, 22,  0, }, // Gilbert_Islands
    {  337,  788,       0,   91136,       0,  0, 17,  0, }, // GMT
    {  337,  830,  337398,  337418,  337443, 20, 25, 33, }, // Greenland_Eastern
    {  337,  848,  337476,  337497,  337523, 21, 26, 34, }, // Greenland_Western
    {  337,  871,       0,  337557,       0,  0, 14,  0, }, // Gulf
    {  337,  876,       0,  337252,       0,  0, 10,  0, }, // Guyana
    {  337,  883,  337571,  337587,  337608, 16, 21, 23, }, // Hawaii_Aleutian
    {  337,  899,  337631,  337644,  337662, 13, 18, 26, }, // Hong_Kong
    {  337,  909,  337688,  337697,  337711,  9, 14, 22, }, // Hovd
    {  337,  914,       0,  337733,       0,  0, 15,  0, }, // India
    {  337,  920,       0,  337748,       0,  0, 16,  0, }, // Indian_Ocean
    {  337,  933,       0,  337764,       0,  0, 13,  0, }, // Indochina
    {  337,  943,       0,  337777,       0,  0, 19,  0, }, // Indonesia_Central
    {  337,  961,       0,  337796,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  337,  979,       0,  337817,       0,  0, 22,  0, }, // Indonesia_Western
    {  337,  997,  337839,  337847,  337860,  8, 13, 15, }, // Iran
    {  337, 1008,  337875,  337889,  337908, 14, 19, 27, }, // Irkutsk
    {  337, 1016,  337935,  337946,  337962, 11, 16, 18, }, // Israel
    {  337, 1023,  337980,  337989,  338003,  9, 14, 16, }, // Japan
    {  337, 1029,  338019,  338048,  338082, 29, 34, 42, }, // Kamchatka
    {  337, 1047,       0,  338124,       0,  0, 14,  0, }, // Kazakhstan
    {  337, 1058,       0,  338138,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  337, 1077,       0,  338158,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  337, 1106,  338179,  338190,  338206, 11, 16, 18, }, // Korea
    {  337, 1112,       0,  338224,       0,  0, 10,  0, }, // Kosrae
    {  337, 1119,  338234,  338253,  338277, 19, 24, 32, }, // Krasnoyarsk
    {  337, 1151,       0,  338309,       0,  0, 16,  0, }, // Kyrgystan
    {  337, 1175,       0,  338325,       0,  0, 18,  0, }, // Line_Islands
    {  337, 1188,  338343,  338357,  338376, 14, 19, 21, }, // Lord_Howe
    {  337, 1204,  338397,  338408,  338424, 11, 16, 24, }, // Magadan
    {  337, 1219,       0,  338448,       0,  0, 11,  0, }, // Malaysia
    {  337, 1228,       0,  338459,       0,  0, 10,  0, }, // Maldives
    {  337, 1237,       0,  338469,       0,  0, 12,  0, }, // Marquesas
    {  337, 1247,       0,  338481,       0,  0, 20,  0, }, // Marshall_Islands
    {  337, 1264,  338501,  338511,  338526, 10, 15, 23, }, // Mauritius
    {  337, 1274,       0,  338549,       0,  0, 10,  0, }, // Mawson
    {  337, 1281,  338559,  338579,  338604, 20, 25, 27, }, // Mexico_Pacific
    {  337, 1296,  338631,  338644,  338662, 13, 18, 26, }, // Mongolia
    {  337, 1305,  338688,  338698,  338713, 10, 15, 23, }, // Moscow
    {  337, 1312,       0,  338736,       0,  0, 12,  0, }, // Myanmar
    {  337, 1320,       0,  338748,       0,  0,  8,  0, }, // Nauru
    {  337, 1326,       0,  338756,       0,  0,  9,  0, }, // Nepal
    {  337, 1332,  338765,  338784,  338808, 19, 24, 32, }, // New_Caledonia
    {  337, 1346,  338840,  338855,  338875, 15, 20, 22, }, // New_Zealand
    {  337, 1358,  338897,  338915,  338938, 18, 23, 25, }, // Newfoundland
    {  337, 1371,       0,  338963,       0,  0,  8,  0, }, // Niue
    {  337, 1376,  338971,  338988,  339010, 17, 22, 24, }, // Norfolk
    {  337, 1384,  339034,  339061,  339093, 27, 32, 40, }, // Noronha
    {  337, 1406,  339133,  339150,  339172, 17, 22, 30, }, // Novosibirsk
    {  337, 1418,  339202,  339212,  339227, 10, 15, 23, }, // Omsk
    {  337, 1428,  339250,  339263,  339281, 13, 18, 26, }, // Pakistan
    {  337, 1437,       0,  339307,       0,  0,  8,  0, }, // Palau
    {  337, 1443,       0,  339315,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  337, 1460,  339334,  339346,  339363, 12, 17, 25, }, // Paraguay
    {  337, 1469,  339388,  339396,  339409,  8, 13, 21, }, // Peru
    {  337, 1474,  339430,  339442,  339459, 12, 17, 25, }, // Philippines
    {  337, 1486,       0,  339484,       0,  0, 22,  0, }, // Phoenix_Islands
    {  337, 1502,  339506,  339533,  339565, 27, 32, 34, }, // Pierre_Miquelon
    {  337, 1518,       0,  339599,       0,  0, 12,  0, }, // Pitcairn
    {  337, 1527,       0,  339611,       0,  0, 10,  0, }, // Ponape
    {  337, 1534,       0,  339621,       0,  0, 14,  0, }, // Pyongyang
    {  337, 1554,       0,  339635,       0,  0, 12,  0, }, // Reunion
    {  337, 1562,       0,  339647,       0,  0, 10,  0, }, // Rothera
    {  337, 1570,  339657,  339667,  339682, 10, 15, 23, }, // Sakhalin
    {  337, 1579,  339705,  339714,  339728,  9, 14, 22, }, // Samara
    {  337, 1596,  339750,  339758,  339771,  8, 13, 15, }, // Samoa
    {  337, 1602,       0,  339786,       0,  0, 11,  0, }, // Seychelles
    {  337, 1624,       0,  339797,       0,  0, 12,  0, }, // Singapore
    {  337, 1634,       0,  339809,       0,  0, 20,  0, }, // Solomon
    {  337, 1642,       0,  339829,       0,  0, 20,  0, }, // South_Georgia
    {  337, 1656,       0,  339849,       0,  0, 11,  0, }, // Suriname
    {  337, 1676,       0,  339860,       0,  0, 10,  0, }, // Syowa
    {  337, 1682,       0,  339870,       0,  0, 10,  0, }, // Tahiti
    {  337, 1689,  339880,  339889,  339903,  9, 14, 16, }, // Taipei
    {  337, 1696,       0,  339919,       0,  0, 15,  0, }, // Tajikistan
    {  337, 1724,       0,  339934,       0,  0, 11,  0, }, // Tokelau
    {  337, 1732,  339945,  339954,  339968,  9, 14, 22, }, // Tonga
    {  337, 1738,       0,  339990,       0,  0,  7,  0, }, // Truk
    {  337, 1750,  339997,  340015,  340038, 18, 23, 31, }, // Turkmenistan
    {  337, 1763,       0,  340069,       0,  0, 10,  0, }, // Tuvalu
    {  337, 1777,  340079,  340090,  340106, 11, 16, 24, }, // Uruguay
    {  337, 1792,  340130,  340147,  340169, 17, 22, 30, }, // Uzbekistan
    {  337, 1803,  340199,  340209,  340224, 10, 15, 23, }, // Vanuatu
    {  337, 1811,       0,  340247,       0,  0, 14,  0, }, // Venezuela
    {  337, 1821,  340261,  340278,  340300, 17, 22, 30, }, // Vladivostok
    {  337, 1833,  340330,  340345,  340365, 15, 20, 28, }, // Volgograd
    {  337, 1843,       0,  340267,       0,  0, 11,  0, }, // Vostok
    {  337, 1850,       0,  340393,       0,  0, 13,  0, }, // Wake
    {  337, 1855,       0,  340406,       0,  0, 22,  0, }, // Wallis
    {  337, 1862,   97043,  340428,  340446, 13, 18, 26, }, // Yakutsk
    {  337, 1870,  340472,  340489,  340511, 17, 22, 30, }, // Yekaterinburg
    {  337, 1892,       0,  340541,       0,  0,  9,  0, }, // Yukon
    {  338,    0,       0,       9,      27,  9, 18, 16, }, // Acre Hindi/Latin/India
    {  338,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan
    {  338,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {  338,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  338,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  338,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  338,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  338,  128,     235,     246,     266, 11, 20, 18, }, // Almaty
    {  338,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {  338,  142,  340550,  340576,  340611, 26, 35, 35, }, // America_Central
    {  338,  158,  340646,  340672,  340707, 26, 35, 35, }, // America_Eastern
    {  338,  174,  340742,  340769,  340805, 27, 36, 36, }, // America_Mountain
    {  338,  191,  340841,  340867,  340902, 26, 35, 35, }, // America_Pacific
    {  338,  207,     552,     563,     583, 11, 20, 18, }, // Anadyr
    {  338,  214,     601,     610,     628,  9, 18, 18, }, // Apia
    {  338,  219,     646,     656,     675, 10, 19, 17, }, // Aqtau
    {  338,  225,     692,     703,     723, 11, 20, 18, }, // Aqtobe
    {  338,  232,     741,     753,     774, 12, 21, 21, }, // Arabian
    {  338,  240,     795,     809,     832, 14, 23, 21, }, // Argentina
    {  338,  250,     853,     875,     906, 22, 31, 29, }, // Argentina_Western
    {  338,  268,     935,     947,     968, 12, 21, 19, }, // Armenia
    {  338,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  338,  295,    1044,    1066,    1098, 22, 32, 32, }, // Australia_Central
    {  338,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  338,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {  338,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {  338,  374,    1413,    1428,    1452, 15, 24, 22, }, // Azerbaijan
    {  338,  385,    1474,    1485,    1505, 11, 20, 18, }, // Azores
    {  338,  397,    1523,    1538,    1562, 15, 24, 22, }, // Bangladesh
    {  338,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {  338,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia
    {  338,  437,    1607,    1620,    1642, 13, 22, 20, }, // Brasilia
    {  338,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {  338,  461,    1684,    1699,    1723, 15, 24, 22, }, // Cape_Verde
    {  338,  472,       0,    1745,       0,  0, 10,  0, }, // Casey
    {  338,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro
    {  338,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {  338,  495,    1831,    1841,    1860, 10, 19, 17, }, // Chile
    {  338,  501,    1877,    1887,    1906, 10, 19, 19, }, // China
    {  338,  507,       0,    1925,       0,  0, 21,  0, }, // Christmas
    {  338,  517,       0,    1946,       0,  0, 18,  0, }, // Cocos
    {  338,  523,    1964,    1977,    1999, 13, 22, 20, }, // Colombia
    {  338,  532,    2019,    2036,    2062, 17, 26, 29, }, // Cook
    {  338,  537,    2091,    2100,    2118,  9, 18, 18, }, // Cuba
    {  338,  548,       0,    2136,       0,  0, 10,  0, }, // Davis
    {  338,  564,       0,    2146,       0,  0, 21,  0, }, // DumontDUrville
    {  338,  601,       0,    2167,       0,  0, 15,  0, }, // East_Timor
    {  338,  612,    2182,    2200,    2227, 18, 27, 25, }, // Easter
    {  338,  619,       0,    2252,       0,  0, 12,  0, }, // Ecuador
    {  338,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  338,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  338,  657,       0,    2422,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  338,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  338,  695,    2530,    2551,    2581, 21, 30, 28, }, // Falkland
    {  338,  704,    2609,    2618,    2636,  9, 18, 16, }, // Fiji
    {  338,  709,       0,    2652,       0,  0, 18,  0, }, // French_Guiana
    {  338,  723,       0,    2670,       0,  0, 32,  0, }, // French_Southern
    {  338,  746,       0,    2702,       0,  0, 14,  0, }, // Galapagos
    {  338,  756,       0,    2716,       0,  0, 12,  0, }, // Gambier
    {  338,  764,    2728,    2740,    2761, 12, 21, 19, }, // Georgia
    {  338,  772,       0,    2780,       0,  0, 20,  0, }, // Gilbert_Islands
    {  338,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {  338,  802,    2819,    2833,    2856, 14, 23, 21, }, // Greenland
    {  338,  830,    2877,    2896,    2924, 19, 28, 26, }, // Greenland_Eastern
    {  338,  848,    2950,    2969,    2997, 19, 28, 26, }, // Greenland_Western
    {  338,  866,       0,    3023,       0,  0, 18,  0, }, // Guam
    {  338,  871,       0,    3041,       0,  0, 18,  0, }, // Gulf
    {  338,  876,       0,    3059,       0,  0, 11,  0, }, // Guyana
    {  338,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  338,  899,    3148,    3162,    3185, 14, 23, 21, }, // Hong_Kong
    {  338,  909,  219552,  219562,  219581, 10, 19, 17, }, // Hovd
    {  338,  914,       0,    3249,       0,  0, 19,  0, }, // India
    {  338,  920,       0,    3268,       0,  0, 17,  0, }, // Indian_Ocean
    {  338,  933,       0,    3285,       0,  0, 14,  0, }, // Indochina
    {  338,  943,       0,    3299,       0,  0, 22,  0, }, // Indonesia_Central
    {  338,  961,       0,    3321,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  338,  979,       0,    3343,       0,  0, 22,  0, }, // Indonesia_Western
    {  338,  997,    3365,    3374,    3392,  9, 18, 18, }, // Iran
    {  338, 1008,    3410,    3422,    3443, 12, 21, 19, }, // Irkutsk
    {  338, 1016,    3462,    3473,    3493, 11, 20, 20, }, // Israel
    {  338, 1023,    3513,    3523,    3542, 10, 19, 19, }, // Japan
    {  338, 1029,    3561,    3590,    3628, 29, 38, 36, }, // Kamchatka
    {  338, 1047,       0,    3664,       0,  0, 15,  0, }, // Kazakhstan
    {  338, 1058,       0,    3679,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  338, 1077,       0,    3699,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  338, 1106,    3719,    3730,    3750, 11, 20, 20, }, // Korea
    {  338, 1112,       0,    3770,       0,  0, 11,  0, }, // Kosrae
    {  338, 1119,    3781,    3797,    3822, 16, 25, 23, }, // Krasnoyarsk
    {  338, 1151,       0,    3845,       0,  0, 15,  0, }, // Kyrgystan
    {  338, 1161,       0,    3860,       0,  0, 10,  0, }, // Lanka
    {  338, 1175,       0,    3870,       0,  0, 17,  0, }, // Line_Islands
    {  338, 1188,    3887,    3901,    3924, 14, 23, 23, }, // Lord_Howe
    {  338, 1198,    3947,    3957,    3976, 10, 19, 17, }, // Macau
    {  338, 1204,    3993,    4005,    4026, 12, 21, 19, }, // Magadan
    {  338, 1219,       0,    4045,       0,  0, 13,  0, }, // Malaysia
    {  338, 1228,       0,    4058,       0,  0, 13,  0, }, // Maldives
    {  338, 1237,       0,    4071,       0,  0, 14,  0, }, // Marquesas
    {  338, 1247,       0,    4085,       0,  0, 21,  0, }, // Marshall_Islands
    {  338, 1264,    4106,    4120,    4143, 14, 23, 21, }, // Mauritius
    {  338, 1274,       0,    4164,       0,  0, 11,  0, }, // Mawson
    {  338, 1281,    4175,    4195,    4224, 20, 29, 29, }, // Mexico_Pacific
    {  338, 1296,    4253,    4269,    4294, 16, 25, 23, }, // Mongolia
    {  338, 1305,    4317,    4328,    4348, 11, 20, 18, }, // Moscow
    {  338, 1312,       0,    4366,       0,  0, 12,  0, }, // Myanmar
    {  338, 1320,       0,    4378,       0,  0, 10,  0, }, // Nauru
    {  338, 1326,       0,    4388,       0,  0, 10,  0, }, // Nepal
    {  338, 1332,    4398,    4416,    4443, 18, 27, 25, }, // New_Caledonia
    {  338, 1346,    4468,    4484,    4509, 16, 25, 25, }, // New_Zealand
    {  338, 1358,    4534,    4551,    4577, 17, 26, 26, }, // Newfoundland
    {  338, 1371,       0,    4603,       0,  0,  9,  0, }, // Niue
    {  338, 1376,    4612,    4631,    4659, 19, 28, 28, }, // Norfolk
    {  338, 1384,    4687,    4711,    4744, 24, 33, 31, }, // Noronha
    {  338, 1392,       0,    4775,       0,  0, 26,  0, }, // North_Mariana
    {  338, 1406,    4801,    4817,    4842, 16, 25, 23, }, // Novosibirsk
    {  338, 1418,    4865,    4874,    4892,  9, 18, 16, }, // Omsk
    {  338, 1428,    4908,    4921,    4943, 13, 22, 20, }, // Pakistan
    {  338, 1437,       0,    4963,       0,  0, 10,  0, }, // Palau
    {  338, 1443,       0,    4973,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  338, 1460,    4994,    5007,    5029, 13, 22, 20, }, // Paraguay
    {  338, 1469,    5049,    5058,    5076,  9, 18, 16, }, // Peru
    {  338, 1474,    5092,    5107,    5131, 15, 24, 22, }, // Philippines
    {  338, 1486,       0,    5153,       0,  0, 20,  0, }, // Phoenix_Islands
    {  338, 1502,    5173,    5199,    5234, 26, 35, 35, }, // Pierre_Miquelon
    {  338, 1518,       0,    5269,       0,  0, 13,  0, }, // Pitcairn
    {  338, 1527,       0,    5282,       0,  0, 11,  0, }, // Ponape
    {  338, 1534,       0,    5293,       0,  0, 14,  0, }, // Pyongyang
    {  338, 1544,    5307,    5321,    5344, 14, 23, 21, }, // Qyzylorda
    {  338, 1554,       0,  340937,       0,  0, 12,  0, }, // Reunion
    {  338, 1562,       0,    5377,       0,  0, 12,  0, }, // Rothera
    {  338, 1570,    5389,    5402,    5424, 13, 22, 20, }, // Sakhalin
    {  338, 1579,    5444,    5455,    5475, 11, 20, 18, }, // Samara
    {  338, 1596,    5493,    5503,    5522, 10, 19, 19, }, // Samoa
    {  338, 1602,       0,    5541,       0,  0, 15,  0, }, // Seychelles
    {  338, 1624,       0,    5556,       0,  0, 23,  0, }, // Singapore
    {  338, 1634,       0,    5579,       0,  0, 20,  0, }, // Solomon
    {  338, 1642,       0,    5599,       0,  0, 18,  0, }, // South_Georgia
    {  338, 1656,       0,    5617,       0,  0, 13,  0, }, // Suriname
    {  338, 1676,       0,    5630,       0,  0, 10,  0, }, // Syowa
    {  338, 1682,       0,    5640,       0,  0, 11,  0, }, // Tahiti
    {  338, 1689,    5651,    5662,    5682, 11, 20, 20, }, // Taipei
    {  338, 1696,       0,    5702,       0,  0, 15,  0, }, // Tajikistan
    {  338, 1724,       0,    5717,       0,  0, 12,  0, }, // Tokelau
    {  338, 1732,    5729,    5739,    5758, 10, 19, 17, }, // Tonga
    {  338, 1738,       0,    5775,       0,  0, 10,  0, }, // Truk
    {  338, 1750,    5785,    5802,    5828, 17, 26, 24, }, // Turkmenistan
    {  338, 1763,       0,    5852,       0,  0, 11,  0, }, // Tuvalu
    {  338, 1777,    5863,    5875,    5896, 12, 21, 19, }, // Uruguay
    {  338, 1792,    5915,    5930,    5954, 15, 24, 22, }, // Uzbekistan
    {  338, 1803,    5976,    5988,    6009, 12, 21, 19, }, // Vanuatu
    {  338, 1811,       0,    6028,       0,  0, 14,  0, }, // Venezuela
    {  338, 1821,    6042,    6058,    6083, 16, 25, 23, }, // Vladivostok
    {  338, 1833,    6106,    6120,    6143, 14, 23, 21, }, // Volgograd
    {  338, 1843,       0,    6164,       0,  0, 11,  0, }, // Vostok
    {  338, 1850,       0,    6175,       0,  0, 16,  0, }, // Wake
    {  338, 1855,       0,    6191,       0,  0, 20,  0, }, // Wallis
    {  338, 1862,    6211,    6223,    6244, 12, 21, 19, }, // Yakutsk
    {  338, 1870,    6263,    6281,    6308, 18, 27, 25, }, // Yekaterinburg
    {  338, 1892,       0,    6333,       0,  0, 10,  0, }, // Yukon
    {  339,    0,  340949,  340957,  340969,  8, 12, 14, }, // Acre Hungarian/Latin/Hungary
    {  339,    5,       0,  340983,       0,  0, 16,  0, }, // Afghanistan
    {  339,   17,       0,  340999,       0,  0, 22,  0, }, // Africa_Central
    {  339,   32,       0,  341021,       0,  0, 22,  0, }, // Africa_Eastern
    {  339,   65,       0,  341043,       0,  0, 20,  0, }, // Africa_Southern
    {  339,   81,  341063,  341085,  341108, 22, 23, 24, }, // Africa_Western
    {  339,  107,  341132,  341144,  341160, 12, 16, 18, }, // Alaska
    {  339,  128,  341178,  341188,  341202, 10, 14, 16, }, // Almaty
    {  339,  135,  341218,  341231,  341249, 13, 18, 19, }, // Amazon
    {  339,  142,  341268,  341291,  341318, 23, 27, 29, }, // America_Central
    {  339,  158,  341347,  341369,  341395, 22, 26, 28, }, // America_Eastern
    {  339,  174,  341423,  341437,  341455, 14, 18, 20, }, // America_Mountain
    {  339,  191,  341475,  341493,  341515, 18, 22, 24, }, // America_Pacific
    {  339,  207,  341539,  341550,  341565, 11, 15, 17, }, // Anadyr
    {  339,  214,  341582,  341591,  341605,  9, 14, 15, }, // Apia
    {  339,  219,  341620,  341630,  341644, 10, 14, 16, }, // Aqtau
    {  339,  225,  341660,  341671,  341686, 11, 15, 17, }, // Aqtobe
    {  339,  232,  341703,  341711,  341724,  8, 13, 14, }, // Arabian
    {  339,  240,  341738,  341752,  341771, 14, 19, 20, }, // Argentina
    {  339,  250,  341791,  341816,  341842, 25, 26, 27, }, // Argentina_Western
    {  339,  268,  341869,  341886,  341908, 17, 22, 23, }, // Armenia
    {  339,  286,  341931,  341949,  341971, 18, 22, 24, }, // Atlantic
    {  339,  295,  341995,  342016,  342042, 21, 26, 27, }, // Australia_Central
    {  339,  313,  342069,  342097,  342130, 28, 33, 34, }, // Australia_CentralWestern
    {  339,  338,  342164,  342185,  342211, 21, 26, 27, }, // Australia_Eastern
    {  339,  356,  342075,  342103,  342136, 22, 27, 28, }, // Australia_Western
    {  339,  374,  342238,  342255,  342277, 17, 22, 23, }, // Azerbaijan
    {  339,  385,  342300,  342313,  342327, 13, 14, 15, }, // Azores
    {  339,  397,  342342,  342356,  342375, 14, 19, 20, }, // Bangladesh
    {  339,  415,       0,  342395,       0,  0, 10,  0, }, // Bhutan
    {  339,  422,       0,  342405,       0,  0, 17,  0, }, // Bolivia
    {  339,  437,  342422,  342435,  342453, 13, 18, 19, }, // Brasilia
    {  339,  454,       0,  342472,       0,  0, 23,  0, }, // Brunei
    {  339,  461,  342495,  342522,  342550, 27, 28, 29, }, // Cape_Verde
    {  339,  472,       0,  342579,       0,  0, 11,  0, }, // Casey
    {  339,  478,       0,  342590,       0,  0, 18,  0, }, // Chamorro
    {  339,  487,  342608,  342620,  342637, 12, 17, 18, }, // Chatham
    {  339,  495,  342655,  342669,  342684, 14, 15, 16, }, // Chile
    {  339,  501,  342700,  342709,  342723,  9, 14, 15, }, // China
    {  339,  507,       0,  342738,       0,  0, 21,  0, }, // Christmas
    {  339,  517,       0,  342759,       0,  0, 25,  0, }, // Cocos
    {  339,  523,  342784,  342797,  342815, 13, 18, 19, }, // Colombia
    {  339,  532,  342834,  342852,  342875, 18, 23, 28, }, // Cook
    {  339,  537,  342903,  342916,  342930, 13, 14, 15, }, // Cuba
    {  339,  548,       0,  342945,       0,  0, 10,  0, }, // Davis
    {  339,  564,       0,  342955,       0,  0, 22,  0, }, // DumontDUrville
    {  339,  601,       0,  342977,       0,  0, 21,  0, }, // East_Timor
    {  339,  612,  342998,  343020,  343043, 22, 23, 24, }, // Easter
    {  339,  619,       0,  343067,       0,  0, 17,  0, }, // Ecuador
    {  339,  627,  343084,  343105,  343127, 21, 22, 23, }, // Europe_Central
    {  339,  642,  343150,  343171,  343193, 21, 22, 23, }, // Europe_Eastern
    {  339,  657,       0,  343216,       0,  0, 11,  0, }, // Europe_Further_Eastern
    {  339,  680,  343227,  343249,  343272, 22, 23, 24, }, // Europe_Western
    {  339,  695,  343296,  343318,  343345, 22, 27, 28, }, // Falkland
    {  339,  704,  343373,  343383,  343398, 10, 15, 16, }, // Fiji
    {  339,  709,       0,  343414,       0,  0, 19,  0, }, // French_Guiana
    {  339,  723,       0,  343433,       0,  0, 31,  0, }, // French_Southern
    {  339,  746,       0,  343464,       0,  0, 19,  0, }, // Galapagos
    {  339,  756,       0,  343483,       0,  0, 12,  0, }, // Gambier
    {  339,  764,  343495,  343506,  343522, 11, 16, 17, }, // Georgia
    {  339,  772,       0,  343539,       0,  0, 21,  0, }, // Gilbert_Islands
    {  339,  788,       0,  343560,       0,  0, 29,  0, }, // GMT
    {  339,  830,  343589,  343612,  343636, 23, 24, 25, }, // Greenland_Eastern
    {  339,  848,  343661,  343685,  343710, 24, 25, 26, }, // Greenland_Western
    {  339,  866,       0,  343736,       0,  0, 13,  0, }, // Guam
    {  339,  871,       0,  343749,       0,  0, 17,  0, }, // Gulf
    {  339,  876,       0,  343766,       0,  0, 16,  0, }, // Guyana
    {  339,  883,  343782,  343803,  343825, 21, 22, 23, }, // Hawaii_Aleutian
    {  339,  899,  343848,  343865,  343883, 17, 18, 19, }, // Hong_Kong
    {  339,  909,  343902,  343911,  343925,  9, 14, 15, }, // Hovd
    {  339,  914,       0,  343940,       0,  0, 15,  0, }, // India
    {  339,  920,       0,  343955,       0,  0, 17,  0, }, // Indian_Ocean
    {  339,  933,       0,  343972,       0,  0, 13,  0, }, // Indochina
    {  339,  943,       0,  343985,       0,  0, 20,  0, }, // Indonesia_Central
    {  339,  961,       0,  344005,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  339,  979,       0,  344025,       0,  0, 26,  0, }, // Indonesia_Western
    {  339,  997,  344051,  344060,  344074,  9, 14, 15, }, // Iran
    {  339, 1008,  344089,  344102,  344120, 13, 18, 19, }, // Irkutsk
    {  339, 1016,  344139,  344150,  344166, 11, 16, 17, }, // Israel
    {  339, 1023,  344183,  344192,  344206,  9, 14, 15, }, // Japan
    {  339, 1029,  344221,  344250,  344283, 29, 33, 35, }, // Kamchatka
    {  339, 1047,       0,  344318,       0,  0, 15,  0, }, // Kazakhstan
    {  339, 1058,       0,  344333,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  339, 1077,       0,  344354,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  339, 1106,  344376,  344386,  344401, 10, 15, 16, }, // Korea
    {  339, 1112,       0,  344417,       0,  0, 11,  0, }, // Kosrae
    {  339, 1119,  344428,  344446,  344469, 18, 23, 24, }, // Krasnoyarsk
    {  339, 1151,       0,  344493,       0,  0, 17,  0, }, // Kyrgystan
    {  339, 1161,       0,  344510,       0,  0, 10,  0, }, // Lanka
    {  339, 1175,       0,  344520,       0,  0, 17,  0, }, // Line_Islands
    {  339, 1188,  344537,  344558,  344584, 21, 26, 27, }, // Lord_Howe
    {  339, 1198,  344611,  344621,  344635, 10, 14, 16, }, // Macau
    {  339, 1204,  344651,  344663,  344680, 12, 17, 18, }, // Magadan
    {  339, 1219,       0,  344698,       0,  0, 13,  0, }, // Malaysia
    {  339, 1228,       0,  344711,       0,  0, 20,  0, }, // Maldives
    {  339, 1237,       0,  344731,       0,  0, 23,  0, }, // Marquesas
    {  339, 1247,       0,  344754,       0,  0, 22,  0, }, // Marshall_Islands
    {  339, 1264,  344776,  344794,  344813, 18, 19, 20, }, // Mauritius
    {  339, 1274,       0,  344833,       0,  0, 11,  0, }, // Mawson
    {  339, 1281,  344844,  344870,  344900, 26, 30, 32, }, // Mexico_Pacific
    {  339, 1296,  344932,  344946,  344965, 14, 19, 20, }, // Mongolia
    {  339, 1305,  344985,  344997,  345014, 12, 17, 18, }, // Moscow
    {  339, 1312,       0,  345032,       0,  0, 12,  0, }, // Myanmar
    {  339, 1320,       0,  345044,       0,  0, 10,  0, }, // Nauru
    {  339, 1326,       0,  345054,       0,  0, 10,  0, }, // Nepal
    {  339, 1332,  345064,  345081,  345103, 17, 22, 23, }, // New_Caledonia
    {  339, 1346,  345126,  345140,  345159, 14, 19, 20, }, // New_Zealand
    {  339, 1358,  345179,  345195,  345215, 16, 20, 22, }, // Newfoundland
    {  339, 1371,       0,  345237,       0,  0,  9,  0, }, // Niue
    {  339, 1376,  345246,  345267,  345293, 21, 26, 27, }, // Norfolk
    {  339, 1384,  345320,  345345,  345375, 25, 30, 31, }, // Noronha
    {  339, 1392,       0,  345406,       0,  0, 27,  0, }, // North_Mariana
    {  339, 1406,  345433,  345451,  345474, 18, 23, 24, }, // Novosibirsk
    {  339, 1418,  345498,  345508,  345523, 10, 15, 16, }, // Omsk
    {  339, 1428,  345539,  345553,  345572, 14, 19, 20, }, // Pakistan
    {  339, 1437,       0,  345592,       0,  0, 10,  0, }, // Palau
    {  339, 1443,       0,  345602,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  339, 1460,  345622,  345635,  345653, 13, 18, 19, }, // Paraguay
    {  339, 1469,  345672,  345681,  345695,  9, 14, 15, }, // Peru
    {  339, 1474,  345710,  345729,  345753, 19, 24, 25, }, // Philippines
    {  339, 1486,       0,  345778,       0,  0, 26,  0, }, // Phoenix_Islands
    {  339, 1502,  345804,  345834,  345868, 30, 34, 36, }, // Pierre_Miquelon
    {  339, 1518,       0,  345904,       0,  0, 22,  0, }, // Pitcairn
    {  339, 1527,       0,  345926,       0,  0, 18,  0, }, // Ponape
    {  339, 1534,       0,  345944,       0,  0, 12,  0, }, // Pyongyang
    {  339, 1544,  345956,  345970,  345988, 14, 18, 20, }, // Qyzylorda
    {  339, 1554,       0,  346008,       0,  0, 12,  0, }, // Reunion
    {  339, 1562,       0,  346020,       0,  0, 12,  0, }, // Rothera
    {  339, 1570,  346032,  346045,  346063, 13, 18, 19, }, // Sakhalin
    {  339, 1579,  346082,  346094,  346110, 12, 16, 18, }, // Samara
    {  339, 1596,  346128,  346139,  346155, 11, 16, 17, }, // Samoa
    {  339, 1602,       0,  346172,       0,  0, 23,  0, }, // Seychelles
    {  339, 1624,       0,  346195,       0,  0, 19,  0, }, // Singapore
    {  339, 1634,       0,  346214,       0,  0, 21,  0, }, // Solomon
    {  339, 1642,       0,  346235,       0,  0, 17,  0, }, // South_Georgia
    {  339, 1656,       0,  346252,       0,  0, 13,  0, }, // Suriname
    {  339, 1676,       0,  346265,       0,  0, 10,  0, }, // Syowa
    {  339, 1682,       0,  346275,       0,  0, 10,  0, }, // Tahiti
    {  339, 1689,  346285,  346295,  346310, 10, 15, 16, }, // Taipei
    {  339, 1696,       0,  346326,       0,  0, 18,  0, }, // Tajikistan
    {  339, 1724,       0,  346344,       0,  0, 12,  0, }, // Tokelau
    {  339, 1732,  346356,  346366,  346381, 10, 15, 16, }, // Tonga
    {  339, 1738,       0,  346397,       0,  0,  9,  0, }, // Truk
    {  339, 1750,  346406,  346424,  346447, 18, 23, 24, }, // Turkmenistan
    {  339, 1763,       0,  346471,       0,  0, 11,  0, }, // Tuvalu
    {  339, 1777,  346482,  346494,  346511, 12, 17, 18, }, // Uruguay
    {  339, 1792,  346529,  346545,  346566, 16, 21, 22, }, // Uzbekistan
    {  339, 1803,  346588,  346600,  346617, 12, 17, 18, }, // Vanuatu
    {  339, 1811,       0,  346635,       0,  0, 14,  0, }, // Venezuela
    {  339, 1821,  346649,  346667,  346690, 18, 23, 24, }, // Vladivostok
    {  339, 1833,  346714,  346728,  346747, 14, 19, 20, }, // Volgograd
    {  339, 1843,       0,  346655,       0,  0, 12,  0, }, // Vostok
    {  339, 1850,       0,  346767,       0,  0, 16,  0, }, // Wake
    {  339, 1855,       0,  346783,       0,  0, 22,  0, }, // Wallis
    {  339, 1862,  346805,  346818,  346836, 13, 18, 19, }, // Yakutsk
    {  339, 1870,  346855,  346874,  346898, 19, 24, 25, }, // Yekaterinburg
    {  339, 1892,       0,  346923,       0,  0, 10,  0, }, // Yukon
    {  340,    5,       0,  346933,       0,  0, 14,  0, }, // Afghanistan Icelandic/Latin/Iceland
    {  340,   17,       0,  346947,       0,  0, 14,  0, }, // Africa_Central
    {  340,   32,       0,  346961,       0,  0, 17,  0, }, // Africa_Eastern
    {  340,   65,       0,  346978,       0,  0, 16,  0, }, // Africa_Southern
    {  340,   81,  346994,  347011,  347037, 17, 26, 25, }, // Africa_Western
    {  340,  107,  347062,  347075,  347094, 13, 19, 18, }, // Alaska
    {  340,  135,  347112,  347122,  347148, 10, 26, 25, }, // Amazon
    {  340,  142,  347173,  347212,  347257, 39, 45, 44, }, // America_Central
    {  340,  158,  347301,  347343,  347391, 42, 48, 47, }, // America_Eastern
    {  340,  174,  347438,  347458,  347484, 20, 26, 25, }, // America_Mountain
    {  340,  191,  347509,  347532,  347561, 23, 29, 28, }, // America_Pacific
    {  340,  207,  347589,  347602,  347621, 13, 19, 18, }, // Anadyr
    {  340,  214,  347639,  347650,  347667, 11, 17, 16, }, // Apia
    {  340,  232,  347683,  347693,  347712, 10, 19, 18, }, // Arabian
    {  340,  240,  347730,  347743,  347765, 13, 22, 21, }, // Argentina
    {  340,  250,  347786,  347806,  347835, 20, 29, 28, }, // Argentina_Western
    {  340,  268,  347863,  347874,  347894, 11, 20, 19, }, // Armenia
    {  340,  286,  347913,  347938,  347969, 25, 31, 30, }, // Atlantic
    {  340,  295,  347999,  348018,  348043, 19, 25, 24, }, // Australia_Central
    {  340,  313,  348067,  348097,  348133, 30, 36, 35, }, // Australia_CentralWestern
    {  340,  338,  348168,  348190,  348218, 22, 28, 27, }, // Australia_Eastern
    {  340,  356,  348245,  348267,  348295, 22, 28, 27, }, // Australia_Western
    {  340,  374,  348322,  348338,  348363, 16, 25, 24, }, // Azerbaijan
    {  340,  385,  348387,  348399,  348421, 12, 22, 21, }, // Azores
    {  340,  397,  348442,  348457,  348480, 15, 23, 22, }, // Bangladesh
    {  340,  415,       0,  348502,       0,  0,  9,  0, }, // Bhutan
    {  340,  422,       0,  348511,       0,  0, 11,  0, }, // Bolivia
    {  340,  437,  348522,  348534,  348555, 12, 21, 20, }, // Brasilia
    {  340,  454,       0,  348575,       0,  0, 10,  0, }, // Brunei
    {  340,  461,  348585,  348602,  348629, 17, 27, 26, }, // Cape_Verde
    {  340,  478,       0,  348655,       0,  0, 19,  0, }, // Chamorro
    {  340,  487,  348674,  348686,  348706, 12, 20, 19, }, // Chatham
    {  340,  495,  348725,  348733,  348750,  8, 17, 16, }, // Chile
    {  340,  501,  348766,  348774,  348791,  8, 17, 16, }, // China
    {  340,  507,       0,  348807,       0,  0, 13,  0, }, // Christmas
    {  340,  517,       0,  348820,       0,  0, 13,  0, }, // Cocos
    {  340,  523,  348833,  348845,  348866, 12, 21, 20, }, // Colombia
    {  340,  532,  348886,  348900,  348924, 14, 24, 27, }, // Cook
    {  340,  537,  348951,  348959,  348976,  8, 17, 16, }, // Cuba
    {  340,  548,       0,  348992,       0,  0, 10,  0, }, // Davis
    {  340,  564,       0,  349002,       0,  0, 23,  0, }, // DumontDUrville
    {  340,  601,       0,  349025,       0,  0, 18,  0, }, // East_Timor
    {  340,  612,  349043,  349056,  349078, 13, 22, 21, }, // Easter
    {  340,  619,       0,  349099,       0,  0, 11,  0, }, // Ecuador
    {  340,  627,  349110,  349124,  349147, 14, 23, 22, }, // Europe_Central
    {  340,  642,  349169,  349186,  349212, 17, 26, 25, }, // Europe_Eastern
    {  340,  657,       0,  349237,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  340,  680,  349259,  349276,  349302, 17, 26, 25, }, // Europe_Western
    {  340,  695,  349327,  349344,  349371, 17, 27, 26, }, // Falkland
    {  340,  704,  349397,  349410,  349433, 13, 23, 22, }, // Fiji
    {  340,  709,       0,  349455,       0,  0, 22,  0, }, // French_Guiana
    {  340,  723,       0,  349477,       0,  0, 55,  0, }, // French_Southern
    {  340,  746,       0,  349532,       0,  0, 14,  0, }, // Galapagos
    {  340,  756,       0,  349546,       0,  0, 12,  0, }, // Gambier
    {  340,  764,  349558,  349569,  349589, 11, 20, 19, }, // Georgia
    {  340,  772,       0,  349608,       0,  0, 20,  0, }, // Gilbert_Islands
    {  340,  788,       0,  349628,       0,  0, 20,  0, }, // GMT
    {  340,  830,  349648,  349668,  349697, 20, 29, 28, }, // Greenland_Eastern
    {  340,  848,  349725,  349745,  349774, 20, 29, 28, }, // Greenland_Western
    {  340,  871,       0,  349802,       0,  0, 24,  0, }, // Gulf
    {  340,  876,       0,  349826,       0,  0, 11,  0, }, // Guyana
    {  340,  883,  349837,  349859,  349887, 22, 28, 27, }, // Hawaii_Aleutian
    {  340,  899,  349914,  349928,  349950, 14, 22, 21, }, // Hong_Kong
    {  340,  909,  349971,  349980,  349997,  9, 17, 16, }, // Hovd
    {  340,  914,       0,  350013,       0,  0, 12,  0, }, // India
    {  340,  920,       0,  350025,       0,  0, 16,  0, }, // Indian_Ocean
    {  340,  933,       0,  350041,       0,  0, 12,  0, }, // Indochina
    {  340,  943,       0,  350053,       0,  0, 17,  0, }, // Indonesia_Central
    {  340,  961,       0,  350070,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  340,  979,       0,  350090,       0,  0, 20,  0, }, // Indonesia_Western
    {  340,  997,  350110,  350119,  350136,  9, 17, 16, }, // Iran
    {  340, 1008,  350152,  350166,  350186, 14, 20, 19, }, // Irkutsk
    {  340, 1016,  350205,  350216,  350235, 11, 19, 18, }, // Israel
    {  340, 1023,  350253,  350263,  350281, 10, 18, 17, }, // Japan
    {  340, 1029,  350298,  350329,  350366, 31, 37, 36, }, // Kamchatka
    {  340, 1047,       0,  350402,       0,  0, 16,  0, }, // Kazakhstan
    {  340, 1058,       0,  350418,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  340, 1077,       0,  350441,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  340, 1106,  350464,  350473,  350491,  9, 18, 17, }, // Korea
    {  340, 1112,       0,  350508,       0,  0, 11,  0, }, // Kosrae
    {  340, 1119,  350519,  350537,  350561, 18, 24, 23, }, // Krasnoyarsk
    {  340, 1151,       0,  350584,       0,  0, 14,  0, }, // Kyrgystan
    {  340, 1175,       0,  350598,       0,  0, 13,  0, }, // Line_Islands
    {  340, 1188,  350611,  350632,  350659, 21, 27, 26, }, // Lord_Howe
    {  340, 1204,  350685,  350699,  350719, 14, 20, 19, }, // Magadan
    {  340, 1219,       0,  350738,       0,  0, 11,  0, }, // Malaysia
    {  340, 1228,       0,  350749,       0,  0, 15,  0, }, // Maldives
    {  340, 1237,       0,  350764,       0,  0, 26,  0, }, // Marquesas
    {  340, 1247,       0,  350790,       0,  0, 21,  0, }, // Marshall_Islands
    {  340, 1264,  350811,  350823,  350844, 12, 21, 20, }, // Mauritius
    {  340, 1274,       0,  350864,       0,  0, 11,  0, }, // Mawson
    {  340, 1281,  350875,  350897,  350935, 22, 38, 37, }, // Mexico_Pacific
    {  340, 1296,  350972,  350989,  351012, 17, 23, 22, }, // Mongolia
    {  340, 1305,  351034,  351044,  351063, 10, 19, 18, }, // Moscow
    {  340, 1312,       0,  351081,       0,  0, 12,  0, }, // Myanmar
    {  340, 1320,       0,  351093,       0,  0,  9,  0, }, // Nauru
    {  340, 1326,       0,  351102,       0,  0,  9,  0, }, // Nepal
    {  340, 1332,  351111,  351132,  351159, 21, 27, 26, }, // New_Caledonia
    {  340, 1346,  351185,  351205,  351231, 20, 26, 25, }, // New_Zealand
    {  340, 1358,  351256,  351276,  351302, 20, 26, 25, }, // Newfoundland
    {  340, 1371,       0,  351327,       0,  0,  9,  0, }, // Niue
    {  340, 1376,  351336,  351355,  351380, 19, 25, 24, }, // Norfolk
    {  340, 1384,  351404,  351430,  351462, 26, 32, 31, }, // Noronha
    {  340, 1406,  351493,  351511,  351535, 18, 24, 23, }, // Novosibirsk
    {  340, 1418,  351558,  351569,  351586, 11, 17, 16, }, // Omsk
    {  340, 1428,  351602,  351614,  351635, 12, 21, 20, }, // Pakistan
    {  340, 1437,       0,  351655,       0,  0,  8,  0, }, // Palau
    {  340, 1443,       0,  351663,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  340, 1460,  351686,  351697,  351717, 11, 20, 19, }, // Paraguay
    {  340, 1469,  351736,  351744,  351761,  8, 17, 16, }, // Peru
    {  340, 1474,  351777,  351792,  351817, 15, 25, 24, }, // Philippines
    {  340, 1486,       0,  351841,       0,  0, 13,  0, }, // Phoenix_Islands
    {  340, 1502,  351854,  351886,  351924, 32, 38, 37, }, // Pierre_Miquelon
    {  340, 1518,       0,  351961,       0,  0, 13,  0, }, // Pitcairn
    {  340, 1527,       0,  351974,       0,  0, 11,  0, }, // Ponape
    {  340, 1534,       0,  351985,       0,  0, 16,  0, }, // Pyongyang
    {  340, 1554,       0,  352001,       0,  0, 12,  0, }, // Reunion
    {  340, 1562,       0,  352013,       0,  0, 12,  0, }, // Rothera
    {  340, 1570,  352025,  352040,  352061, 15, 21, 20, }, // Sakhalin
    {  340, 1579,  352081,  352094,  352113, 13, 19, 18, }, // Samara
    {  340, 1596,  352131,  352141,  352159, 10, 18, 17, }, // Samoa
    {  340, 1602,       0,  352176,       0,  0, 19,  0, }, // Seychelles
    {  340, 1624,       0,  352195,       0,  0, 12,  0, }, // Singapore
    {  340, 1634,       0,  352207,       0,  0, 16,  0, }, // Solomon
    {  340, 1642,       0,  352223,       0,  0, 17,  0, }, // South_Georgia
    {  340, 1656,       0,  352240,       0,  0, 11,  0, }, // Suriname
    {  340, 1676,       0,  352251,       0,  0, 10,  0, }, // Syowa
    {  340, 1682,       0,  352261,       0,  0, 11,  0, }, // Tahiti
    {  340, 1689,  352272,  352283,  352302, 11, 19, 18, }, // Taipei
    {  340, 1696,       0,  352320,       0,  0, 17,  0, }, // Tajikistan
    {  340, 1724,       0,  352337,       0,  0, 11,  0, }, // Tokelau
    {  340, 1732,  352348,  352357,  352375,  9, 18, 17, }, // Tonga
    {  340, 1738,       0,  352392,       0,  0, 10,  0, }, // Truk
    {  340, 1750,  352402,  352419,  352444, 17, 25, 24, }, // Turkmenistan
    {  340, 1763,       0,  352468,       0,  0, 10,  0, }, // Tuvalu
    {  340, 1777,  352478,  352488,  352507, 10, 19, 18, }, // Uruguay
    {  340, 1792,  352525,  352540,  352563, 15, 23, 22, }, // Uzbekistan
    {  340, 1803,  352585,  352597,  352617, 12, 20, 19, }, // Vanuatu
    {  340, 1811,       0,  352636,       0,  0, 13,  0, }, // Venezuela
    {  340, 1821,  352649,  352667,  352691, 18, 24, 23, }, // Vladivostok
    {  340, 1833,  352714,  352730,  352752, 16, 22, 21, }, // Volgograd
    {  340, 1843,       0,  352773,       0,  0, 11,  0, }, // Vostok
    {  340, 1850,       0,  352784,       0,  0, 16,  0, }, // Wake
    {  340, 1855,       0,  352800,       0,  0, 29,  0, }, // Wallis
    {  340, 1862,  352829,  352843,  352863, 14, 20, 19, }, // Yakutsk
    {  340, 1870,  352882,  352902,  352928, 20, 26, 25, }, // Yekaterinburg
    {  340, 1892,       0,  352953,       0,  0, 12,  0, }, // Yukon
    {  342,    5,       0,  352965,       0,  0, 15,  0, }, // Afghanistan Igbo/Latin/Nigeria
    {  342,   17,       0,  352980,       0,  0, 16,  0, }, // Africa_Central
    {  342,   32,       0,  352996,       0,  0, 33,  0, }, // Africa_Eastern
    {  342,   65,       0,  353029,       0,  0, 38,  0, }, // Africa_Southern
    {  342,   81,  353067,  353100,  353140, 33, 40, 42, }, // Africa_Western
    {  342,  107,  353182,  353192,  353209, 10, 17, 14, }, // Alaska
    {  342,  135,  353223,  353233,  353250, 10, 17, 19, }, // Amazon
    {  342,  142,  353269,  353287,  353312, 18, 25, 22, }, // America_Central
    {  342,  158,  352996,  353334,  353367, 26, 33, 30, }, // America_Eastern
    {  342,  174,  353397,  353414,  353438, 17, 24, 21, }, // America_Mountain
    {  342,  191,  353459,  353479,  353506, 20, 27, 24, }, // America_Pacific
    {  342,  214,  353530,  353538,  353553,  8, 15, 12, }, // Apia
    {  342,  232,  353565,  353573,  353588,  8, 15, 12, }, // Arabian
    {  342,  240,  353600,  353613,  353633, 13, 20, 22, }, // Argentina
    {  342,  250,  353655,  353691,  353734, 36, 43, 45, }, // Argentina_Western
    {  342,  268,  353779,  353790,  353808, 11, 18, 20, }, // Armenia
    {  342,  286,  353828,  353849,  353877, 21, 28, 25, }, // Atlantic
    {  342,  295,  353902,  353921,  353947, 19, 26, 23, }, // Australia_Central
    {  342,  313,  353970,  354012,  354061, 42, 49, 46, }, // Australia_CentralWestern
    {  342,  338,  354107,  354143,  354186, 36, 43, 40, }, // Australia_Eastern
    {  342,  356,  354226,  354262,  354305, 36, 43, 40, }, // Australia_Western
    {  342,  374,  354345,  354359,  354380, 14, 21, 23, }, // Azerbaijan
    {  342,  385,  354403,  354413,  354430, 10, 17, 19, }, // Azores
    {  342,  397,  354449,  354463,  354484, 14, 21, 23, }, // Bangladesh
    {  342,  415,       0,  354507,       0,  0, 10,  0, }, // Bhutan
    {  342,  422,       0,  354517,       0,  0, 11,  0, }, // Bolivia
    {  342,  437,  354528,  354540,  354559, 12, 19, 21, }, // Brasilia
    {  342,  454,       0,  354580,       0,  0, 21,  0, }, // Brunei
    {  342,  461,  354601,  354615,  354636, 14, 21, 23, }, // Cape_Verde
    {  342,  478,       0,  354659,       0,  0, 19,  0, }, // Chamorro
    {  342,  487,  354678,  354689,  354707, 11, 18, 15, }, // Chatham
    {  342,  495,  354722,  354731,  354747,  9, 16, 18, }, // Chile
    {  342,  501,  354765,  354774,  354790,  9, 16, 13, }, // China
    {  342,  507,       0,  354803,       0,  0, 22,  0, }, // Christmas
    {  342,  517,       0,  354825,       0,  0, 17,  0, }, // Cocos
    {  342,  523,  354842,  354854,  354873, 12, 19, 21, }, // Colombia
    {  342,  532,  354894,  354910,  354933, 16, 23, 31, }, // Cook
    {  342,  537,  354964,  354972,  354987,  8, 15, 21, }, // Cuba
    {  342,  548,       0,  355008,       0,  0,  9,  0, }, // Davis
    {  342,  564,       0,  355017,       0,  0, 20,  0, }, // DumontDUrville
    {  342,  601,       0,  355037,       0,  0, 32,  0, }, // East_Timor
    {  342,  612,  355069,  355102,  355142, 33, 40, 42, }, // Easter
    {  342,  619,       0,  355184,       0,  0, 11,  0, }, // Ecuador
    {  342,  627,  355195,  355220,  355252, 25, 32, 34, }, // Europe_Central
    {  342,  642,  355286,  355319,  355359, 33, 40, 42, }, // Europe_Eastern
    {  342,  657,       0,    2422,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  342,  680,  355401,  355434,  355474, 33, 40, 42, }, // Europe_Western
    {  342,  695,  355516,  355536,  355563, 20, 27, 29, }, // Falkland
    {  342,  704,  355592,  355600,  355615,  8, 15, 17, }, // Fiji
    {  342,  709,       0,  355632,       0,  0, 17,  0, }, // French_Guiana
    {  342,  723,       0,  355649,       0,  0, 31,  0, }, // French_Southern
    {  342,  746,       0,  355680,       0,  0, 13,  0, }, // Galapagos
    {  342,  756,       0,  355693,       0,  0, 11,  0, }, // Gambier
    {  342,  764,  355704,  355715,  355733, 11, 18, 20, }, // Georgia
    {  342,  772,       0,  355753,       0,  0, 19,  0, }, // Gilbert_Islands
    {  342,  788,       0,  355772,       0,  0, 27,  0, }, // GMT
    {  342,  830,  355799,  355835,  355878, 36, 43, 45, }, // Greenland_Eastern
    {  342,  848,  355923,  355959,  356002, 36, 43, 45, }, // Greenland_Western
    {  342,  871,       0,  356047,       0,  0, 15,  0, }, // Gulf
    {  342,  876,       0,  356062,       0,  0, 10,  0, }, // Guyana
    {  342,  883,  356072,  356091,  356117, 19, 26, 23, }, // Hawaii_Aleutian
    {  342,  899,  356140,  356153,  356173, 13, 20, 22, }, // Hong_Kong
    {  342,  909,  356195,  356203,  356218,  8, 15, 17, }, // Hovd
    {  342,  914,       0,  356235,       0,  0, 16,  0, }, // India
    {  342,  920,       0,  356251,       0,  0, 17,  0, }, // Indian_Ocean
    {  342,  933,       0,  356268,       0,  0, 13,  0, }, // Indochina
    {  342,  943,       0,  356281,       0,  0, 19,  0, }, // Indonesia_Central
    {  342,  961,       0,  356300,       0,  0, 36,  0, }, // Indonesia_Eastern
    {  342,  979,       0,  356336,       0,  0, 36,  0, }, // Indonesia_Western
    {  342,  997,  356372,  356380,  356395,  8, 15, 12, }, // Iran
    {  342, 1008,  356407,  356418,  356436, 11, 18, 20, }, // Irkutsk
    {  342, 1016,  356456,  356466,  356483, 10, 17, 14, }, // Israel
    {  342, 1023,  356497,  356506,  356522,  9, 16, 13, }, // Japan
    {  342, 1047,       0,  356535,       0,  0, 14,  0, }, // Kazakhstan
    {  342, 1058,       0,  356549,       0,  0, 37,  0, }, // Kazakhstan_Eastern
    {  342, 1077,       0,  356586,       0,  0, 37,  0, }, // Kazakhstan_Western
    {  342, 1106,  356623,  356632,  356648,  9, 16, 13, }, // Korea
    {  342, 1112,       0,  356661,       0,  0, 10,  0, }, // Kosrae
    {  342, 1119,  356671,  356686,  356708, 15, 22, 24, }, // Krasnoyarsk
    {  342, 1151,       0,  356732,       0,  0, 14,  0, }, // Kyrgystan
    {  342, 1175,       0,  356746,       0,  0, 16,  0, }, // Line_Islands
    {  342, 1188,  356762,  356775,  356795, 13, 20, 17, }, // Lord_Howe
    {  342, 1204,  356812,  356823,  356841, 11, 18, 20, }, // Magadan
    {  342, 1219,       0,  356861,       0,  0, 12,  0, }, // Malaysia
    {  342, 1228,       0,  356873,       0,  0, 12,  0, }, // Maldives
    {  342, 1237,       0,  356885,       0,  0, 13,  0, }, // Marquesas
    {  342, 1247,       0,  356898,       0,  0, 20,  0, }, // Marshall_Islands
    {  342, 1264,  356918,  356931,  356951, 13, 20, 22, }, // Mauritius
    {  342, 1274,       0,  356973,       0,  0, 10,  0, }, // Mawson
    {  342, 1281,  356983,  357002,  357028, 19, 26, 23, }, // Mexico_Pacific
    {  342, 1296,  357051,  357066,  357088, 15, 22, 24, }, // Mongolia
    {  342, 1305,  357112,  357122,  357139, 10, 17, 19, }, // Moscow
    {  342, 1312,       0,  357158,       0,  0, 11,  0, }, // Myanmar
    {  342, 1320,       0,  357169,       0,  0,  9,  0, }, // Nauru
    {  342, 1326,       0,  357178,       0,  0,  9,  0, }, // Nepal
    {  342, 1332,  357187,  357204,  357228, 17, 24, 26, }, // New_Caledonia
    {  342, 1346,  357254,  357269,  357291, 15, 22, 19, }, // New_Zealand
    {  342, 1358,  357310,  357326,  357349, 16, 23, 20, }, // Newfoundland
    {  342, 1371,       0,  357369,       0,  0,  8,  0, }, // Niue
    {  342, 1376,  357377,  357395,  357420, 18, 25, 27, }, // Norfolk
    {  342, 1384,  357447,  357470,  357500, 23, 30, 32, }, // Noronha
    {  342, 1406,  357532,  357547,  357569, 15, 22, 24, }, // Novosibirsk
    {  342, 1418,  357593,  357601,  357616,  8, 15, 17, }, // Omsk
    {  342, 1428,  357633,  357645,  357664, 12, 19, 21, }, // Pakistan
    {  342, 1437,       0,  357685,       0,  0,  9,  0, }, // Palau
    {  342, 1443,       0,  357694,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  342, 1460,  357714,  357726,  357745, 12, 19, 21, }, // Paraguay
    {  342, 1469,  357766,  357774,  357789,  8, 15, 17, }, // Peru
    {  342, 1474,  357806,  357820,  357841, 14, 21, 23, }, // Philippines
    {  342, 1486,       0,  357864,       0,  0, 19,  0, }, // Phoenix_Islands
    {  342, 1502,  357883,  357908,  357940, 25, 32, 29, }, // Pierre_Miquelon
    {  342, 1518,       0,  357969,       0,  0, 12,  0, }, // Pitcairn
    {  342, 1527,       0,  357981,       0,  0, 10,  0, }, // Ponape
    {  342, 1534,       0,  357991,       0,  0, 13,  0, }, // Pyongyang
    {  342, 1554,       0,  358004,       0,  0, 11,  0, }, // Reunion
    {  342, 1562,       0,  358015,       0,  0, 11,  0, }, // Rothera
    {  342, 1570,  358026,  358038,  358057, 12, 19, 21, }, // Sakhalin
    {  342, 1596,  358078,  358087,  358103,  9, 16, 13, }, // Samoa
    {  342, 1602,       0,  358116,       0,  0, 14,  0, }, // Seychelles
    {  342, 1624,       0,  358130,       0,  0, 20,  0, }, // Singapore
    {  342, 1634,       0,  358150,       0,  0, 19,  0, }, // Solomon
    {  342, 1642,       0,  358169,       0,  0, 17,  0, }, // South_Georgia
    {  342, 1656,       0,  358186,       0,  0, 12,  0, }, // Suriname
    {  342, 1676,       0,  358198,       0,  0,  9,  0, }, // Syowa
    {  342, 1682,       0,  358207,       0,  0, 10,  0, }, // Tahiti
    {  342, 1689,  358217,  358227,  358244, 10, 17, 14, }, // Taipei
    {  342, 1696,       0,  358258,       0,  0, 14,  0, }, // Tajikistan
    {  342, 1724,       0,  358272,       0,  0, 11,  0, }, // Tokelau
    {  342, 1732,  358283,  358292,  358308,  9, 16, 18, }, // Tonga
    {  342, 1738,       0,  358326,       0,  0,  9,  0, }, // Truk
    {  342, 1750,  358335,  358349,  358370, 14, 21, 23, }, // Turkmenistan
    {  342, 1763,       0,  358393,       0,  0, 10,  0, }, // Tuvalu
    {  342, 1777,  358403,  358414,  358432, 11, 18, 20, }, // Uruguay
    {  342, 1792,  358452,  358464,  358483, 12, 19, 21, }, // Uzbekistan
    {  342, 1803,  358504,  358515,  358533, 11, 18, 20, }, // Vanuatu
    {  342, 1811,       0,  358553,       0,  0, 13,  0, }, // Venezuela
    {  342, 1821,  358566,  358581,  358603, 15, 22, 24, }, // Vladivostok
    {  342, 1833,  358627,  358640,  358660, 13, 20, 22, }, // Volgograd
    {  342, 1843,       0,  358682,       0,  0, 10,  0, }, // Vostok
    {  342, 1850,       0,  358692,       0,  0, 15,  0, }, // Wake
    {  342, 1855,       0,  358707,       0,  0, 19,  0, }, // Wallis
    {  342, 1862,  358726,  358737,  358755, 11, 18, 20, }, // Yakutsk
    {  342, 1870,  358775,  358792,  358816, 17, 24, 26, }, // Yekaterinburg
    {  342, 1892,       0,  358842,       0,  0,  9,  0, }, // Yukon
    {  344,    0,  358851,  358861,  358879, 10, 18, 22, }, // Acre Indonesian/Latin/Indonesia
    {  344,    5,       0,  358901,       0,  0, 16,  0, }, // Afghanistan
    {  344,   17,       0,  358917,       0,  0, 19,  0, }, // Africa_Central
    {  344,   32,       0,  358936,       0,  0, 18,  0, }, // Africa_Eastern
    {  344,   65,       0,  358954,       0,  0, 28,  0, }, // Africa_Southern
    {  344,   81,  358982,  359000,  359026, 18, 26, 30, }, // Africa_Western
    {  344,  107,  359056,  359068,  359088, 12, 20, 24, }, // Alaska
    {  344,  128,  359112,  359124,  359144, 12, 20, 24, }, // Almaty
    {  344,  135,  359168,  359180,  359200, 12, 20, 24, }, // Amazon
    {  344,  142,  359224,  359236,  359256, 12, 20, 24, }, // America_Central
    {  344,  158,  359280,  359291,  359310, 11, 19, 23, }, // America_Eastern
    {  344,  174,  359333,  359349,  359373, 16, 24, 28, }, // America_Mountain
    {  344,  191,  359401,  359414,  359435, 13, 21, 25, }, // America_Pacific
    {  344,  207,  359460,  359472,  359492, 12, 20, 24, }, // Anadyr
    {  344,  214,  359516,  359526,  359544, 10, 18, 22, }, // Apia
    {  344,  219,  359566,  359577,  359596, 11, 19, 23, }, // Aqtau
    {  344,  225,  359619,  359631,  359651, 12, 20, 24, }, // Aqtobe
    {  344,  232,  359675,  359685,  359703, 10, 18, 22, }, // Arabian
    {  344,  240,  359725,  359740,  359763, 15, 23, 27, }, // Argentina
    {  344,  250,  359790,  359818,  359854, 28, 36, 40, }, // Argentina_Western
    {  344,  268,  359894,  359907,  359928, 13, 21, 25, }, // Armenia
    {  344,  286,  359953,  359967,  359989, 14, 22, 26, }, // Atlantic
    {  344,  295,  360015,  360037,  360067, 22, 30, 34, }, // Australia_Central
    {  344,  313,  360101,  360129,  360165, 28, 36, 40, }, // Australia_CentralWestern
    {  344,  338,  360205,  360226,  360255, 21, 29, 33, }, // Australia_Eastern
    {  344,  356,  360288,  360309,  360338, 21, 29, 33, }, // Australia_Western
    {  344,  374,  360371,  360387,  360411, 16, 24, 28, }, // Azerbaijan
    {  344,  385,  360439,  360451,  360471, 12, 20, 24, }, // Azores
    {  344,  397,  360495,  360511,  360535, 16, 24, 28, }, // Bangladesh
    {  344,  415,       0,  360563,       0,  0, 12,  0, }, // Bhutan
    {  344,  422,       0,  360575,       0,  0, 13,  0, }, // Bolivia
    {  344,  437,  360588,  360600,  360620, 12, 20, 24, }, // Brasilia
    {  344,  454,       0,  360644,       0,  0, 23,  0, }, // Brunei
    {  344,  461,  360667,  360686,  360713, 19, 27, 31, }, // Cape_Verde
    {  344,  472,       0,  360744,       0,  0, 11,  0, }, // Casey
    {  344,  478,       0,  360755,       0,  0, 22,  0, }, // Chamorro
    {  344,  487,  360777,  360790,  360811, 13, 21, 25, }, // Chatham
    {  344,  495,  360836,  360846,  360864, 10, 18, 22, }, // Chile
    {  344,  501,  360886,  360900,  360922, 14, 22, 26, }, // China
    {  344,  507,       0,  360948,       0,  0, 17,  0, }, // Christmas
    {  344,  517,       0,  360965,       0,  0, 21,  0, }, // Cocos
    {  344,  523,  360986,  361000,  361022, 14, 22, 26, }, // Colombia
    {  344,  532,  361048,  361063,  361086, 15, 23, 34, }, // Cook
    {  344,  537,  361120,  361130,  361148, 10, 18, 22, }, // Cuba
    {  344,  548,       0,  361170,       0,  0, 11,  0, }, // Davis
    {  344,  564,       0,  361181,       0,  0, 22,  0, }, // DumontDUrville
    {  344,  601,       0,  361203,       0,  0, 17,  0, }, // East_Timor
    {  344,  612,  361220,  361238,  361264, 18, 26, 30, }, // Easter
    {  344,  619,       0,  361294,       0,  0, 13,  0, }, // Ecuador
    {  344,  627,  361307,  361325,  361351, 18, 26, 30, }, // Europe_Central
    {  344,  642,  361381,  361398,  361423, 17, 25, 29, }, // Europe_Eastern
    {  344,  657,       0,  361452,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  344,  680,  361474,  361491,  361516, 17, 25, 29, }, // Europe_Western
    {  344,  695,  361545,  361569,  361601, 24, 32, 36, }, // Falkland
    {  344,  704,  361637,  361647,  361665, 10, 18, 22, }, // Fiji
    {  344,  709,       0,  361687,       0,  0, 20,  0, }, // French_Guiana
    {  344,  723,       0,  361707,       0,  0, 44,  0, }, // French_Southern
    {  344,  746,       0,  361751,       0,  0, 15,  0, }, // Galapagos
    {  344,  756,       0,  361766,       0,  0, 13,  0, }, // Gambier
    {  344,  764,  361779,  361792,  361813, 13, 21, 25, }, // Georgia
    {  344,  772,       0,  361838,       0,  0, 18,  0, }, // Gilbert_Islands
    {  344,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {  344,  830,  361856,  361877,  361906, 21, 29, 33, }, // Greenland_Eastern
    {  344,  848,  361939,  361960,  361989, 21, 29, 33, }, // Greenland_Western
    {  344,  866,       0,  362022,       0,  0, 10,  0, }, // Guam
    {  344,  871,       0,  362032,       0,  0, 19,  0, }, // Gulf
    {  344,  876,       0,  361687,       0,  0, 12,  0, }, // Guyana
    {  344,  883,  362051,  362072,  362101, 21, 29, 33, }, // Hawaii_Aleutian
    {  344,  899,  362134,  362149,  362172, 15, 23, 27, }, // Hong_Kong
    {  344,  909,  362199,  362209,  362227, 10, 18, 22, }, // Hovd
    {  344,  914,       0,  362249,       0,  0, 11,  0, }, // India
    {  344,  920,       0,  362260,       0,  0, 21,  0, }, // Indian_Ocean
    {  344,  933,       0,  362281,       0,  0, 15,  0, }, // Indochina
    {  344,  943,       0,  362296,       0,  0, 22,  0, }, // Indonesia_Central
    {  344,  961,       0,  362318,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  344,  979,       0,  362339,       0,  0, 21,  0, }, // Indonesia_Western
    {  344,  997,  362360,  362370,  362388, 10, 18, 22, }, // Iran
    {  344, 1008,  362410,  362423,  362444, 13, 21, 25, }, // Irkutsk
    {  344, 1016,  362469,  362481,  362501, 12, 20, 24, }, // Israel
    {  344, 1023,  362525,  362537,  362557, 12, 20, 24, }, // Japan
    {  344, 1029,  362581,  362611,  362649, 30, 38, 42, }, // Kamchatka
    {  344, 1047,       0,  362691,       0,  0, 16,  0, }, // Kazakhstan
    {  344, 1058,       0,  362707,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  344, 1077,       0,  362729,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  344, 1106,  362751,  362762,  362781, 11, 19, 23, }, // Korea
    {  344, 1112,       0,  362804,       0,  0, 12,  0, }, // Kosrae
    {  344, 1119,  362816,  362833,  362858, 17, 25, 29, }, // Krasnoyarsk
    {  344, 1151,       0,  362887,       0,  0, 16,  0, }, // Kyrgystan
    {  344, 1161,       0,  362903,       0,  0, 11,  0, }, // Lanka
    {  344, 1175,       0,  362914,       0,  0, 15,  0, }, // Line_Islands
    {  344, 1188,  362929,  362944,  362967, 15, 23, 27, }, // Lord_Howe
    {  344, 1198,  362994,  363005,  363024, 11, 19, 23, }, // Macau
    {  344, 1204,  363047,  363060,  363081, 13, 21, 25, }, // Magadan
    {  344, 1219,       0,  363106,       0,  0, 14,  0, }, // Malaysia
    {  344, 1228,       0,  363120,       0,  0, 14,  0, }, // Maldives
    {  344, 1237,       0,  363134,       0,  0, 15,  0, }, // Marquesas
    {  344, 1247,       0,  363149,       0,  0, 19,  0, }, // Marshall_Islands
    {  344, 1264,  363168,  363183,  363206, 15, 23, 27, }, // Mauritius
    {  344, 1274,       0,  363233,       0,  0, 12,  0, }, // Mawson
    {  344, 1281,  363245,  363266,  363295, 21, 29, 33, }, // Mexico_Pacific
    {  344, 1296,  363328,  363344,  363368, 16, 24, 28, }, // Mongolia
    {  344, 1305,  363396,  363408,  363428, 12, 20, 24, }, // Moscow
    {  344, 1312,       0,  363452,       0,  0, 13,  0, }, // Myanmar
    {  344, 1320,       0,  363465,       0,  0, 11,  0, }, // Nauru
    {  344, 1326,       0,  363476,       0,  0, 11,  0, }, // Nepal
    {  344, 1332,  363487,  363507,  363535, 20, 28, 32, }, // New_Caledonia
    {  344, 1346,  363567,  363586,  363613, 19, 27, 31, }, // New_Zealand
    {  344, 1358,  363644,  363662,  363688, 18, 26, 30, }, // Newfoundland
    {  344, 1371,       0,  363718,       0,  0, 10,  0, }, // Niue
    {  344, 1376,  363728,  363747,  363774, 19, 27, 31, }, // Norfolk
    {  344, 1384,  363805,  363830,  363863, 25, 33, 37, }, // Noronha
    {  344, 1392,       0,  363900,       0,  0, 24,  0, }, // North_Mariana
    {  344, 1406,  363924,  363941,  363966, 17, 25, 29, }, // Novosibirsk
    {  344, 1418,  363995,  364005,  364023, 10, 18, 22, }, // Omsk
    {  344, 1428,  364045,  364059,  364081, 14, 22, 26, }, // Pakistan
    {  344, 1437,       0,  364107,       0,  0, 11,  0, }, // Palau
    {  344, 1443,       0,  364118,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  344, 1460,  364136,  364150,  364172, 14, 22, 26, }, // Paraguay
    {  344, 1469,  364198,  364208,  364226, 10, 18, 22, }, // Peru
    {  344, 1474,  364248,  364262,  364284, 14, 22, 26, }, // Philippines
    {  344, 1486,       0,  364310,       0,  0, 23,  0, }, // Phoenix_Islands
    {  344, 1502,  364333,  364364,  364403, 31, 39, 43, }, // Pierre_Miquelon
    {  344, 1518,       0,  364446,       0,  0, 14,  0, }, // Pitcairn
    {  344, 1527,       0,  364460,       0,  0, 12,  0, }, // Ponape
    {  344, 1534,       0,  364472,       0,  0, 15,  0, }, // Pyongyang
    {  344, 1544,  364487,  364502,  364525, 15, 23, 27, }, // Qyzylorda
    {  344, 1554,       0,  364552,       0,  0, 13,  0, }, // Reunion
    {  344, 1562,       0,  364565,       0,  0, 13,  0, }, // Rothera
    {  344, 1570,  364578,  364592,  364614, 14, 22, 26, }, // Sakhalin
    {  344, 1579,  364640,  364652,  364672, 12, 20, 24, }, // Samara
    {  344, 1596,  364696,  364707,  364726, 11, 19, 23, }, // Samoa
    {  344, 1602,       0,  364749,       0,  0, 16,  0, }, // Seychelles
    {  344, 1624,       0,  364765,       0,  0, 23,  0, }, // Singapore
    {  344, 1634,       0,  364788,       0,  0, 23,  0, }, // Solomon
    {  344, 1642,       0,  364811,       0,  0, 21,  0, }, // South_Georgia
    {  344, 1656,       0,  364832,       0,  0, 14,  0, }, // Suriname
    {  344, 1676,       0,  364846,       0,  0, 11,  0, }, // Syowa
    {  344, 1682,       0,  364857,       0,  0, 12,  0, }, // Tahiti
    {  344, 1689,  364869,  364881,  364901, 12, 20, 24, }, // Taipei
    {  344, 1696,       0,  364925,       0,  0, 16,  0, }, // Tajikistan
    {  344, 1724,       0,  364941,       0,  0, 13,  0, }, // Tokelau
    {  344, 1732,  364954,  364965,  364984, 11, 19, 23, }, // Tonga
    {  344, 1738,       0,  365007,       0,  0, 11,  0, }, // Truk
    {  344, 1750,  365018,  365036,  365062, 18, 26, 30, }, // Turkmenistan
    {  344, 1763,       0,  365092,       0,  0, 12,  0, }, // Tuvalu
    {  344, 1777,  365104,  365117,  365138, 13, 21, 25, }, // Uruguay
    {  344, 1792,  365163,  365179,  365203, 16, 24, 28, }, // Uzbekistan
    {  344, 1803,  365231,  365244,  365265, 13, 21, 25, }, // Vanuatu
    {  344, 1811,       0,  365290,       0,  0, 15,  0, }, // Venezuela
    {  344, 1821,  365305,  365322,  365347, 17, 25, 29, }, // Vladivostok
    {  344, 1833,  365376,  365391,  365414, 15, 23, 27, }, // Volgograd
    {  344, 1843,       0,  365441,       0,  0, 12,  0, }, // Vostok
    {  344, 1850,       0,  365453,       0,  0, 20,  0, }, // Wake
    {  344, 1855,       0,  365473,       0,  0, 23,  0, }, // Wallis
    {  344, 1862,  365496,  365509,  365530, 13, 21, 25, }, // Yakutsk
    {  344, 1870,  365555,  365574,  365601, 19, 27, 31, }, // Yekaterinburg
    {  344, 1892,       0,  365632,       0,  0, 11,  0, }, // Yukon
    {  345,    5,       0,  365643,       0,  0, 19,  0, }, // Afghanistan Interlingua/Latin/world
    {  345,   17,       0,  365662,       0,  0, 22,  0, }, // Africa_Central
    {  345,   32,       0,  365684,       0,  0, 22,  0, }, // Africa_Eastern
    {  345,   65,       0,  365706,       0,  0, 31,  0, }, // Africa_Southern
    {  345,   81,  365737,  365760,  365792, 23, 32, 30, }, // Africa_Western
    {  345,  107,  365822,  365836,  365857, 14, 21, 21, }, // Alaska
    {  345,  135,  365878,  365894,  365919, 16, 25, 23, }, // Amazon
    {  345,  142,   42433,  365942,  365961, 12, 19, 19, }, // America_Central
    {  345,  158,  365980,  365992,  366011, 12, 19, 19, }, // America_Eastern
    {  345,  174,  366030,  366048,  366073, 18, 25, 25, }, // America_Mountain
    {  345,  191,  366098,  366110,  366129, 12, 19, 19, }, // America_Pacific
    {  345,  214,  290728,  366148,  366169, 12, 21, 19, }, // Apia
    {  345,  232,  366188,  366198,  366217, 10, 19, 17, }, // Arabian
    {  345,  240,  366234,  366251,  366277, 17, 26, 24, }, // Argentina
    {  345,  250,  366301,  366329,  366366, 28, 37, 35, }, // Argentina_Western
    {  345,  268,  291001,  366401,  366425, 15, 24, 22, }, // Armenia
    {  345,  286,  366447,  366460,  366480, 13, 20, 20, }, // Atlantic
    {  345,  295,  366500,  366525,  366559, 25, 34, 32, }, // Australia_Central
    {  345,  313,  366591,  366626,  366670, 35, 44, 42, }, // Australia_CentralWestern
    {  345,  338,  366712,  366738,  366773, 26, 35, 33, }, // Australia_Eastern
    {  345,  356,  366806,  366834,  366871, 28, 37, 35, }, // Australia_Western
    {  345,  374,  366906,  366926,  366955, 20, 29, 27, }, // Azerbaijan
    {  345,  385,  366982,  366997,  367019, 15, 22, 22, }, // Azores
    {  345,  397,  291583,  367041,  367068, 18, 27, 25, }, // Bangladesh
    {  345,  415,       0,  367093,       0,  0, 14,  0, }, // Bhutan
    {  345,  422,       0,  291668,       0,  0, 15,  0, }, // Bolivia
    {  345,  437,  291683,  367107,  367132, 16, 25, 23, }, // Brasilia
    {  345,  454,       0,  291749,       0,  0, 25,  0, }, // Brunei
    {  345,  461,  367155,  367173,  367200, 18, 27, 25, }, // Cape_Verde
    {  345,  478,       0,  367225,       0,  0, 25,  0, }, // Chamorro
    {  345,  487,  291868,  367250,  367274, 15, 24, 22, }, // Chatham
    {  345,  495,  291931,  367296,  367318, 13, 22, 20, }, // Chile
    {  345,  501,  367338,  367351,  367373, 13, 22, 20, }, // China
    {  345,  507,       0,  367393,       0,  0, 24,  0, }, // Christmas
    {  345,  517,       0,  367417,       0,  0, 22,  0, }, // Cocos
    {  345,  523,  292087,  367439,  367464, 16, 25, 23, }, // Colombia
    {  345,  532,  367487,  367508,  367538, 21, 30, 28, }, // Cook
    {  345,  537,  292234,  367566,  367585, 12, 19, 19, }, // Cuba
    {  345,  548,       0,  292288,       0,  0, 13,  0, }, // Davis
    {  345,  564,       0,  292301,       0,  0, 24,  0, }, // DumontDUrville
    {  345,  601,       0,  367604,       0,  0, 21,  0, }, // East_Timor
    {  345,  612,  367625,  367650,  367684, 25, 34, 32, }, // Easter
    {  345,  619,       0,  292428,       0,  0, 15,  0, }, // Ecuador
    {  345,  627,  367716,  367738,  367767, 22, 29, 29, }, // Europe_Central
    {  345,  642,  367796,  367819,  367849, 23, 30, 30, }, // Europe_Eastern
    {  345,  657,       0,  367879,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  345,  680,  367908,  367933,  367965, 25, 32, 32, }, // Europe_Western
    {  345,  695,  367997,  368022,  368056, 25, 34, 32, }, // Falkland
    {  345,  704,  368088,  368100,  368121, 12, 21, 19, }, // Fiji
    {  345,  709,       0,  368140,       0,  0, 23,  0, }, // French_Guiana
    {  345,  723,       0,  368163,       0,  0, 36,  0, }, // French_Southern
    {  345,  746,       0,  368199,       0,  0, 18,  0, }, // Galapagos
    {  345,  756,       0,  292965,       0,  0, 15,  0, }, // Gambier
    {  345,  764,  368217,  368232,  368256, 15, 24, 22, }, // Georgia
    {  345,  772,       0,  368278,       0,  0, 24,  0, }, // Gilbert_Islands
    {  345,  788,       0,  368302,       0,  0, 23,  0, }, // GMT
    {  345,  830,  368325,  368353,  368388, 28, 35, 35, }, // Greenland_Eastern
    {  345,  848,  368423,  368453,  368490, 30, 37, 37, }, // Greenland_Western
    {  345,  871,       0,  368527,       0,  0, 23,  0, }, // Gulf
    {  345,  876,       0,  368550,       0,  0, 14,  0, }, // Guyana
    {  345,  883,  368564,  368589,  368621, 25, 32, 32, }, // Hawaii_Aleutian
    {  345,  899,  368653,  368669,  368694, 16, 25, 23, }, // Hong_Kong
    {  345,  909,  293495,  368717,  368738, 12, 21, 19, }, // Hovd
    {  345,  914,       0,  368757,       0,  0, 22,  0, }, // India
    {  345,  920,       0,  368779,       0,  0, 22,  0, }, // Indian_Ocean
    {  345,  933,       0,  293583,       0,  0, 17,  0, }, // Indochina
    {  345,  943,       0,  293600,       0,  0, 25,  0, }, // Indonesia_Central
    {  345,  961,       0,  368801,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  345,  979,       0,  368826,       0,  0, 26,  0, }, // Indonesia_Western
    {  345,  997,  368852,  368864,  368885, 12, 21, 19, }, // Iran
    {  345, 1008,  293733,  368904,  368926, 15, 22, 22, }, // Irkutsk
    {  345, 1016,  293796,  368948,  368971, 14, 23, 21, }, // Israel
    {  345, 1023,  368992,  369005,  369027, 13, 22, 20, }, // Japan
    {  345, 1047,       0,  369047,       0,  0, 18,  0, }, // Kazakhstan
    {  345, 1058,       0,  369065,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  345, 1077,       0,  369091,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  345, 1106,  294116,  369118,  369140, 13, 22, 20, }, // Korea
    {  345, 1112,       0,  294173,       0,  0, 14,  0, }, // Kosrae
    {  345, 1119,  369160,  369179,  369205, 19, 26, 26, }, // Krasnoyarsk
    {  345, 1151,       0,  369231,       0,  0, 20,  0, }, // Kyrgystan
    {  345, 1175,       0,  369251,       0,  0, 26,  0, }, // Line_Islands
    {  345, 1188,  294304,  369277,  369303, 17, 26, 24, }, // Lord_Howe
    {  345, 1204,  294373,  369327,  369349, 15, 22, 22, }, // Magadan
    {  345, 1219,       0,  369371,       0,  0, 16,  0, }, // Malaysia
    {  345, 1228,       0,  369387,       0,  0, 17,  0, }, // Maldives
    {  345, 1237,       0,  369404,       0,  0, 17,  0, }, // Marquesas
    {  345, 1247,       0,  369421,       0,  0, 25,  0, }, // Marshall_Islands
    {  345, 1264,  369446,  369462,  369487, 16, 25, 23, }, // Mauritius
    {  345, 1274,       0,  294576,       0,  0, 14,  0, }, // Mawson
    {  345, 1281,  369510,  369535,  369567, 25, 32, 32, }, // Mexico_Pacific
    {  345, 1296,  369599,  369617,  369644, 18, 27, 25, }, // Mongolia
    {  345, 1305,  294758,  369669,  369691, 15, 22, 22, }, // Moscow
    {  345, 1312,       0,  294821,       0,  0, 15,  0, }, // Myanmar
    {  345, 1320,       0,  294836,       0,  0, 13,  0, }, // Nauru
    {  345, 1326,       0,  294849,       0,  0, 13,  0, }, // Nepal
    {  345, 1332,  369713,  369735,  369766, 22, 31, 29, }, // New_Caledonia
    {  345, 1346,  369795,  369815,  369844, 20, 29, 27, }, // New_Zealand
    {  345, 1358,  369871,  369888,  369912, 17, 24, 24, }, // Newfoundland
    {  345, 1371,       0,  295099,       0,  0, 12,  0, }, // Niue
    {  345, 1376,  369936,  369959,  369991, 23, 32, 30, }, // Norfolk
    {  345, 1384,  295189,  370021,  370057, 27, 36, 34, }, // Noronha
    {  345, 1406,  295288,  370091,  370117, 19, 26, 26, }, // Novosibirsk
    {  345, 1418,  295363,  370143,  370162, 12, 19, 19, }, // Omsk
    {  345, 1428,  370181,  370197,  370222, 16, 25, 23, }, // Pakistan
    {  345, 1437,       0,  295486,       0,  0, 13,  0, }, // Palau
    {  345, 1443,       0,  370245,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  345, 1460,  370270,  370286,  370311, 16, 25, 23, }, // Paraguay
    {  345, 1469,  370334,  370346,  370367, 12, 21, 19, }, // Peru
    {  345, 1474,  370386,  370406,  370435, 20, 29, 27, }, // Philippines
    {  345, 1486,       0,  370462,       0,  0, 24,  0, }, // Phoenix_Islands
    {  345, 1502,  370486,  370517,  370555, 31, 38, 38, }, // Pierre_Miquelon
    {  345, 1518,       0,  295847,       0,  0, 16,  0, }, // Pitcairn
    {  345, 1527,       0,  370593,       0,  0, 14,  0, }, // Ponape
    {  345, 1534,       0,   63402,       0,  0, 17,  0, }, // Pyongyang
    {  345, 1554,       0,  370607,       0,  0, 15,  0, }, // Reunion
    {  345, 1562,       0,  295893,       0,  0, 15,  0, }, // Rothera
    {  345, 1570,  370622,  370638,  370661, 16, 23, 23, }, // Sakhalin
    {  345, 1596,  296040,  370684,  370706, 13, 22, 20, }, // Samoa
    {  345, 1602,       0,  370726,       0,  0, 19,  0, }, // Seychelles
    {  345, 1624,       0,  370745,       0,  0, 26,  0, }, // Singapore
    {  345, 1634,       0,  370771,       0,  0, 24,  0, }, // Solomon
    {  345, 1642,       0,  370795,       0,  0, 23,  0, }, // South_Georgia
    {  345, 1656,       0,  296176,       0,  0, 16,  0, }, // Suriname
    {  345, 1676,       0,  296192,       0,  0, 13,  0, }, // Syowa
    {  345, 1682,       0,  370818,       0,  0, 14,  0, }, // Tahiti
    {  345, 1689,  296219,  370832,  370855, 14, 23, 21, }, // Taipei
    {  345, 1696,       0,  370876,       0,  0, 18,  0, }, // Tajikistan
    {  345, 1724,       0,  296298,       0,  0, 15,  0, }, // Tokelau
    {  345, 1732,  296313,  370894,  370916, 13, 22, 20, }, // Tonga
    {  345, 1738,       0,  296370,       0,  0, 13,  0, }, // Truk
    {  345, 1750,  370936,  370956,  370985, 20, 29, 27, }, // Turkmenistan
    {  345, 1763,       0,  296461,       0,  0, 14,  0, }, // Tuvalu
    {  345, 1777,  371012,  371027,  371051, 15, 24, 22, }, // Uruguay
    {  345, 1792,  371073,  371091,  371118, 18, 27, 25, }, // Uzbekistan
    {  345, 1803,  296610,  371143,  371167, 15, 24, 22, }, // Vanuatu
    {  345, 1811,       0,  296673,       0,  0, 17,  0, }, // Venezuela
    {  345, 1821,  296690,  371189,  371215, 19, 26, 26, }, // Vladivostok
    {  345, 1833,  296765,  371241,  371265, 17, 24, 24, }, // Volgograd
    {  345, 1843,       0,  296837,       0,  0, 14,  0, }, // Vostok
    {  345, 1850,       0,  371289,       0,  0, 20,  0, }, // Wake
    {  345, 1855,       0,  296868,       0,  0, 23,  0, }, // Wallis
    {  345, 1862,  371309,  371324,  371346, 15, 22, 22, }, // Yakutsk
    {  345, 1870,  371368,  371388,  371415, 20, 27, 27, }, // Yekaterinburg
    {  345, 1892,       0,  297035,       0,  0, 13,  0, }, // Yukon
    {  346,  107,  371442,  371456,  371479, 14, 23, 22, }, // Alaska Interlingue/Latin/Estonia
    {  346,  142,  371501,  371524,  371556, 23, 32, 31, }, // America_Central
    {  346,  158,  371587,  371611,  371644, 24, 33, 32, }, // America_Eastern
    {  346,  174,  371676,  371698,  371729, 22, 31, 30, }, // America_Mountain
    {  346,  191,  371759,  371782,  371814, 23, 32, 31, }, // America_Pacific
    {  346,  286,  371845,  371869,  371902, 24, 33, 32, }, // Atlantic
    {  346,  385,  371934,  371951,  371977, 17, 26, 25, }, // Azores
    {  346,  507,       0,  372002,       0,  0, 26,  0, }, // Christmas
    {  346,  517,       0,  372028,       0,  0, 24,  0, }, // Cocos
    {  346,  532,  372052,  372075,  372107, 23, 32, 31, }, // Cook
    {  346,  537,  372138,  372150,  372171, 12, 21, 20, }, // Cuba
    {  346,  627,  372191,  372213,  372244, 22, 31, 30, }, // Europe_Central
    {  346,  642,  372274,  372292,  372319, 18, 27, 26, }, // Europe_Eastern
    {  346,  680,  372345,  372364,  372392, 19, 28, 27, }, // Europe_Western
    {  346,  695,  372419,  372446,  372482, 27, 36, 35, }, // Falkland
    {  346,  772,       0,  372517,       0,  0, 26,  0, }, // Gilbert_Islands
    {  346,  788,       0,  372543,       0,  0, 27,  0, }, // GMT
    {  346,  830,  372570,  372599,  372637, 29, 38, 37, }, // Greenland_Eastern
    {  346,  848,  372674,  372705,  372745, 31, 40, 39, }, // Greenland_Western
    {  346,  883,  372784,  372809,  372843, 25, 34, 33, }, // Hawaii_Aleutian
    {  346, 1175,       0,  372876,       0,  0, 23,  0, }, // Line_Islands
    {  346, 1188,  372899,  372925,  372960, 26, 35, 34, }, // Lord_Howe
    {  346, 1237,       0,  372994,       0,  0, 20,  0, }, // Marquesas
    {  346, 1247,       0,  373014,       0,  0, 27,  0, }, // Marshall_Islands
    {  346, 1281,  373041,  373063,  373094, 22, 31, 30, }, // Mexico_Pacific
    {  346, 1358,  373124,  373146,  373177, 22, 31, 30, }, // Newfoundland
    {  346, 1376,  373207,  373231,  373264, 24, 33, 32, }, // Norfolk
    {  346, 1486,       0,  373296,       0,  0, 24,  0, }, // Phoenix_Islands
    {  346, 1634,       0,  373320,       0,  0, 26,  0, }, // Solomon
    {  346, 1850,       0,  373346,       0,  0, 21,  0, }, // Wake
    {  346, 1892,       0,  373367,       0,  0, 15,  0, }, // Yukon
    {  349,    0,  373382,  373389,  373409,  7, 20, 17, }, // Acre Irish/Latin/Ireland
    {  349,    5,       0,  373426,       0,  0, 19,  0, }, // Afghanistan
    {  349,   17,       0,  373445,       0,  0, 19,  0, }, // Africa_Central
    {  349,   32,       0,  373464,       0,  0, 23,  0, }, // Africa_Eastern
    {  349,   65,       0,  373487,       0,  0, 20,  0, }, // Africa_Southern
    {  349,   81,  373507,  373529,  373564, 22, 35, 32, }, // Africa_Western
    {  349,  107,  373596,  373605,  373627,  9, 22, 19, }, // Alaska
    {  349,  128,  373646,  373655,  373677,  9, 22, 19, }, // Almaty
    {  349,  135,  373696,  373711,  373739, 15, 28, 25, }, // Amazon
    {  349,  142,  373764,  373792,  373833, 28, 41, 38, }, // America_Central
    {  349,  158,  373871,  373903,  373948, 32, 45, 42, }, // America_Eastern
    {  349,  174,  373990,  374019,  374061, 29, 42, 39, }, // America_Mountain
    {  349,  191,  374100,  374119,  374151, 19, 32, 29, }, // America_Pacific
    {  349,  207,  374180,  374189,  374211,  9, 22, 19, }, // Anadyr
    {  349,  214,  374230,  374237,  374257,  7, 20, 17, }, // Apia
    {  349,  219,  374274,  374282,  374303,  8, 21, 18, }, // Aqtau
    {  349,  225,  374321,  374330,  374352,  9, 22, 19, }, // Aqtobe
    {  349,  232,  374371,  374384,  374410, 13, 26, 23, }, // Arabian
    {  349,  240,  374433,  374450,  374480, 17, 30, 27, }, // Argentina
    {  349,  250,  374507,  374535,  374576, 28, 41, 38, }, // Argentina_Western
    {  349,  268,  374614,  374629,  374657, 15, 28, 25, }, // Armenia
    {  349,  286,  374682,  374698,  374727, 16, 29, 26, }, // Atlantic
    {  349,  295,  374753,  374772,  374804, 19, 32, 29, }, // Australia_Central
    {  349,  313,  374833,  374861,  374902, 28, 41, 38, }, // Australia_CentralWestern
    {  349,  338,  374940,  374964,  375001, 24, 37, 34, }, // Australia_Eastern
    {  349,  356,  375035,  375058,  375094, 23, 36, 33, }, // Australia_Western
    {  349,  374,  375127,  375147,  375180, 20, 33, 30, }, // Azerbaijan
    {  349,  385,  375210,  375221,  375245, 11, 24, 21, }, // Azores
    {  349,  397,  375266,  375284,  375315, 18, 31, 28, }, // Bangladesh
    {  349,  415,       0,  375343,       0,  0, 13,  0, }, // Bhutan
    {  349,  422,       0,  375356,       0,  0, 13,  0, }, // Bolivia
    {  349,  437,  375369,  375381,  375406, 12, 25, 22, }, // Brasilia
    {  349,  454,       0,  375428,       0,  0, 20,  0, }, // Brunei
    {  349,  461,  375448,  375461,  375487, 13, 26, 23, }, // Cape_Verde
    {  349,  472,       0,  375510,       0,  0, 17,  0, }, // Casey
    {  349,  478,       0,  375527,       0,  0, 25,  0, }, // Chamorro
    {  349,  487,  375552,  375562,  375585, 10, 23, 20, }, // Chatham
    {  349,  495,  375605,  375615,  375638, 10, 23, 20, }, // Chile
    {  349,  501,  375658,  375668,  375691, 10, 23, 20, }, // China
    {  349,  507,       0,  375711,       0,  0, 19,  0, }, // Christmas
    {  349,  517,       0,  375730,       0,  0, 16,  0, }, // Cocos
    {  349,  523,  375746,  375759,  375785, 13, 26, 23, }, // Colombia
    {  349,  532,  375808,  375823,  375851, 15, 28, 31, }, // Cook
    {  349,  537,  375882,  375890,  375911,  8, 21, 18, }, // Cuba
    {  349,  548,       0,  375929,       0,  0,  8,  0, }, // Davis
    {  349,  564,       0,  375937,       0,  0, 19,  0, }, // DumontDUrville
    {  349,  601,       0,  375956,       0,  0, 16,  0, }, // East_Timor
    {  349,  612,  375972,  375990,  376021, 18, 31, 28, }, // Easter
    {  349,  619,       0,  376049,       0,  0, 11,  0, }, // Ecuador
    {  349,  627,  376060,  376076,  376105, 16, 29, 26, }, // Europe_Central
    {  349,  642,  376131,  376152,  376186, 21, 34, 31, }, // Europe_Eastern
    {  349,  657,       0,  376217,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  349,  680,  376243,  376263,  376296, 20, 33, 30, }, // Europe_Western
    {  349,  695,  376326,  376347,  376381, 21, 34, 31, }, // Falkland
    {  349,  704,  376412,  376421,  376443,  9, 22, 19, }, // Fiji
    {  349,  709,       0,  376462,       0,  0, 20,  0, }, // French_Guiana
    {  349,  723,       0,  376482,       0,  0, 52,  0, }, // French_Southern
    {  349,  746,       0,  376534,       0,  0, 20,  0, }, // Galapagos
    {  349,  756,       0,  376554,       0,  0, 10,  0, }, // Gambier
    {  349,  764,  376564,  376578,  376605, 14, 27, 24, }, // Georgia
    {  349,  772,       0,  376629,       0,  0, 14,  0, }, // Gilbert_Islands
    {  349,  788,       0,  376643,       0,  0, 17,  0, }, // GMT
    {  349,  830,  376660,  376686,  376725, 26, 39, 36, }, // Greenland_Eastern
    {  349,  848,  376761,  376786,  376824, 25, 38, 35, }, // Greenland_Western
    {  349,  866,       0,  376859,       0,  0, 21,  0, }, // Guam
    {  349,  871,       0,  376880,       0,  0, 30,  0, }, // Gulf
    {  349,  876,       0,  376910,       0,  0, 12,  0, }, // Guyana
    {  349,  883,  376922,  376938,  376967, 16, 29, 26, }, // Hawaii_Aleutian
    {  349,  899,  376993,  377005,  377030, 12, 25, 22, }, // Hong_Kong
    {  349,  909,  377052,  377059,  377079,  7, 20, 17, }, // Hovd
    {  349,  914,       0,  377096,       0,  0, 25,  0, }, // India
    {  349,  920,       0,  377121,       0,  0, 21,  0, }, // Indian_Ocean
    {  349,  933,       0,  377142,       0,  0, 15,  0, }, // Indochina
    {  349,  943,       0,  377157,       0,  0, 20,  0, }, // Indonesia_Central
    {  349,  961,       0,  377177,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  349,  979,       0,  377202,       0,  0, 24,  0, }, // Indonesia_Western
    {  349,  997,  377226,  377240,  377267, 14, 27, 24, }, // Iran
    {  349, 1008,  377291,  377301,  377324, 10, 23, 20, }, // Irkutsk
    {  349, 1016,  377344,  377354,  377377, 10, 23, 20, }, // Israel
    {  349, 1023,  377397,  377411,  377438, 14, 27, 24, }, // Japan
    {  349, 1029,  377462,  377490,  377531, 28, 41, 38, }, // Kamchatka
    {  349, 1047,       0,  377569,       0,  0, 17,  0, }, // Kazakhstan
    {  349, 1058,       0,  377586,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  349, 1077,       0,  377612,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  349, 1106,  377637,  377648,  377672, 11, 24, 21, }, // Korea
    {  349, 1112,       0,  377693,       0,  0,  9,  0, }, // Kosrae
    {  349, 1119,  377702,  377716,  377743, 14, 27, 24, }, // Krasnoyarsk
    {  349, 1151,       0,  377767,       0,  0, 18,  0, }, // Kyrgystan
    {  349, 1161,       0,  377785,       0,  0, 13,  0, }, // Lanka
    {  349, 1175,       0,  377798,       0,  0, 18,  0, }, // Line_Islands
    {  349, 1188,  377816,  377828,  377853, 12, 25, 22, }, // Lord_Howe
    {  349, 1198,  377875,  377884,  377906,  9, 22, 19, }, // Macau
    {  349, 1204,  377925,  377936,  377960, 11, 24, 21, }, // Magadan
    {  349, 1219,       0,  377981,       0,  0, 15,  0, }, // Malaysia
    {  349, 1228,       0,  377996,       0,  0, 20,  0, }, // Maldives
    {  349, 1237,       0,  378016,       0,  0, 23,  0, }, // Marquesas
    {  349, 1247,       0,  378039,       0,  0, 19,  0, }, // Marshall_Islands
    {  349, 1264,  378058,  378075,  378105, 17, 30, 27, }, // Mauritius
    {  349, 1274,       0,  378132,       0,  0,  9,  0, }, // Mawson
    {  349, 1281,  378141,  378172,  378216, 31, 44, 41, }, // Mexico_Pacific
    {  349, 1296,  378257,  378269,  378294, 12, 25, 22, }, // Mongolia
    {  349, 1305,  378316,  378325,  378347,  9, 22, 19, }, // Moscow
    {  349, 1312,       0,  378366,       0,  0, 11,  0, }, // Myanmar
    {  349, 1320,       0,  378377,       0,  0,  7,  0, }, // Nauru
    {  349, 1326,       0,  378384,       0,  0, 10,  0, }, // Nepal
    {  349, 1332,  378394,  378414,  378447, 20, 33, 30, }, // New_Caledonia
    {  349, 1346,  378477,  378497,  378530, 20, 33, 30, }, // New_Zealand
    {  349, 1358,  378560,  378578,  378609, 18, 31, 28, }, // Newfoundland
    {  349, 1371,       0,  378637,       0,  0,  7,  0, }, // Niue
    {  349, 1376,  378644,  378661,  378691, 17, 30, 27, }, // Norfolk
    {  349, 1384,  378718,  378740,  378775, 22, 35, 32, }, // Noronha
    {  349, 1392,       0,  378807,       0,  0, 32,  0, }, // North_Mariana
    {  349, 1406,  378839,  378853,  378880, 14, 27, 24, }, // Novosibirsk
    {  349, 1418,  378904,  378911,  378931,  7, 20, 17, }, // Omsk
    {  349, 1428,  378948,  378964,  378993, 16, 29, 26, }, // Pakistan
    {  349, 1437,       0,  379019,       0,  0, 16,  0, }, // Palau
    {  349, 1443,       0,  379035,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  349, 1460,  379055,  379066,  379090, 11, 24, 21, }, // Paraguay
    {  349, 1469,  379111,  379121,  379144, 10, 23, 20, }, // Peru
    {  349, 1474,  379164,  379189,  379227, 25, 38, 35, }, // Philippines
    {  349, 1486,       0,  379262,       0,  0, 22,  0, }, // Phoenix_Islands
    {  349, 1502,  379284,  379311,  379351, 27, 40, 37, }, // Pierre_Miquelon
    {  349, 1518,       0,  379388,       0,  0, 11,  0, }, // Pitcairn
    {  349, 1527,       0,  379399,       0,  0, 11,  0, }, // Ponape
    {  349, 1534,       0,  379410,       0,  0, 12,  0, }, // Pyongyang
    {  349, 1544,  379422,  379434,  379459, 12, 25, 22, }, // Qyzylorda
    {  349, 1554,       0,  379481,       0,  0, 10,  0, }, // Reunion
    {  349, 1562,       0,  379491,       0,  0, 10,  0, }, // Rothera
    {  349, 1570,  379501,  379513,  379538, 12, 25, 22, }, // Sakhalin
    {  349, 1579,  379560,  379570,  379593, 10, 23, 20, }, // Samara
    {  349, 1596,  379613,  379621,  379642,  8, 21, 18, }, // Samoa
    {  349, 1602,       0,  379660,       0,  0, 13,  0, }, // Seychelles
    {  349, 1624,       0,  379673,       0,  0, 26,  0, }, // Singapore
    {  349, 1634,       0,  379699,       0,  0, 19,  0, }, // Solomon
    {  349, 1642,       0,  379718,       0,  0, 20,  0, }, // South_Georgia
    {  349, 1656,       0,  379738,       0,  0, 11,  0, }, // Suriname
    {  349, 1676,       0,  379749,       0,  0,  8,  0, }, // Syowa
    {  349, 1682,       0,  379757,       0,  0, 11,  0, }, // Tahiti
    {  349, 1689,  379768,  379777,  379799,  9, 22, 19, }, // Taipei
    {  349, 1696,       0,  379818,       0,  0, 21,  0, }, // Tajikistan
    {  349, 1724,       0,  379839,       0,  0, 17,  0, }, // Tokelau
    {  349, 1732,  379856,  379865,  379887,  9, 22, 19, }, // Tonga
    {  349, 1738,       0,  379906,       0,  0,  8,  0, }, // Truk
    {  349, 1750,  379914,  379936,  379971, 22, 35, 32, }, // Turkmenistan
    {  349, 1763,       0,  380003,       0,  0, 10,  0, }, // Tuvalu
    {  349, 1777,  380013,  380022,  380044,  9, 22, 19, }, // Uruguay
    {  349, 1792,  380063,  380085,  380120, 22, 35, 32, }, // Uzbekistan
    {  349, 1803,  380152,  380162,  380185, 10, 23, 20, }, // Vanuatu
    {  349, 1811,       0,  380205,       0,  0, 13,  0, }, // Venezuela
    {  349, 1821,  380218,  380232,  380259, 14, 27, 24, }, // Vladivostok
    {  349, 1833,  380283,  380295,  380320, 12, 25, 22, }, // Volgograd
    {  349, 1843,       0,  380342,       0,  0,  9,  0, }, // Vostok
    {  349, 1850,       0,  380351,       0,  0, 14,  0, }, // Wake
    {  349, 1855,       0,  380365,       0,  0, 21,  0, }, // Wallis
    {  349, 1862,  380386,  380396,  380419, 10, 23, 20, }, // Yakutsk
    {  349, 1870,  380439,  380455,  380484, 16, 29, 26, }, // Yekaterinburg
    {  349, 1892,       0,  380510,       0,  0,  8,  0, }, // Yukon
    {  351,    5,       0,  380518,       0,  0, 20,  0, }, // Afghanistan Italian/Latin/Italy
    {  351,   17,       0,  380538,       0,  0, 24,  0, }, // Africa_Central
    {  351,   32,       0,  380562,       0,  0, 25,  0, }, // Africa_Eastern
    {  351,   65,       0,  380587,       0,  0, 27,  0, }, // Africa_Southern
    {  351,   81,  380614,  380641,  380677, 27, 36, 34, }, // Africa_Western
    {  351,  107,  380711,  380726,  380750, 15, 24, 22, }, // Alaska
    {  351,  135,  380772,  380790,  380817, 18, 27, 25, }, // Amazon
    {  351,  142,  380842,  380858,  380883, 16, 25, 23, }, // America_Central
    {  351,  158,  380906,  380923,  380949, 17, 26, 24, }, // America_Eastern
    {  351,  174,  380973,  380998,  381032, 25, 34, 32, }, // America_Mountain
    {  351,  191,  381064,  381084,  381113, 20, 29, 27, }, // America_Pacific
    {  351,  207,  381140,  381153,  381175, 13, 22, 20, }, // Anadyr
    {  351,  214,  381195,  381206,  381226, 11, 20, 18, }, // Apia
    {  351,  232,  381244,  381253,  381271,  9, 18, 16, }, // Arabian
    {  351,  240,  381287,  381305,  381332, 18, 27, 25, }, // Argentina
    {  351,  250,  381357,  381387,  381426, 30, 39, 37, }, // Argentina_Western
    {  351,  268,  381463,  381479,  381504, 16, 25, 23, }, // Armenia
    {  351,  286,  381527,  381545,  381572, 18, 27, 25, }, // Atlantic
    {  351,  295,  381597,  381624,  381660, 27, 36, 34, }, // Australia_Central
    {  351,  313,  381694,  381729,  381773, 35, 44, 42, }, // Australia_CentralWestern
    {  351,  338,  381815,  381843,  381880, 28, 37, 35, }, // Australia_Eastern
    {  351,  356,  381915,  381945,  381984, 30, 39, 37, }, // Australia_Western
    {  351,  374,  382021,  382041,  382070, 20, 29, 27, }, // Azerbaijan
    {  351,  385,  382097,  382114,  382140, 17, 26, 24, }, // Azores
    {  351,  397,  382164,  382182,  382209, 18, 27, 25, }, // Bangladesh
    {  351,  415,       0,  382234,       0,  0, 14,  0, }, // Bhutan
    {  351,  422,       0,  382248,       0,  0, 17,  0, }, // Bolivia
    {  351,  437,  382265,  382280,  382304, 15, 24, 22, }, // Brasilia
    {  351,  454,       0,  382326,       0,  0, 25,  0, }, // Brunei
    {  351,  461,  382351,  382368,  382394, 17, 26, 24, }, // Cape_Verde
    {  351,  478,       0,  382418,       0,  0, 15,  0, }, // Chamorro
    {  351,  487,  382433,  382450,  382476, 17, 26, 24, }, // Chatham
    {  351,  495,  382500,  382512,  382533, 12, 21, 19, }, // Chile
    {  351,  501,  382552,  382566,  382589, 14, 23, 21, }, // China
    {  351,  507,       0,  382610,       0,  0, 24,  0, }, // Christmas
    {  351,  517,       0,  382634,       0,  0, 21,  0, }, // Cocos
    {  351,  523,  382655,  382673,  382700, 18, 27, 25, }, // Colombia
    {  351,  532,  382725,  382745,  382774, 20, 29, 33, }, // Cook
    {  351,  537,  382807,  382818,  382838, 11, 20, 18, }, // Cuba
    {  351,  548,       0,  382856,       0,  0, 12,  0, }, // Davis
    {  351,  564,       0,  382868,       0,  0, 23,  0, }, // DumontDUrville
    {  351,  601,       0,  382891,       0,  0, 16,  0, }, // East_Timor
    {  351,  612,  382907,  382931,  382964, 24, 33, 31, }, // Easter
    {  351,  619,       0,  382995,       0,  0, 16,  0, }, // Ecuador
    {  351,  627,  383011,  383035,  383068, 24, 33, 31, }, // Europe_Central
    {  351,  642,  383099,  383124,  383158, 25, 34, 32, }, // Europe_Eastern
    {  351,  657,       0,  383190,       0,  0, 39,  0, }, // Europe_Further_Eastern
    {  351,  680,  383229,  383256,  383292, 27, 36, 34, }, // Europe_Western
    {  351,  695,  383326,  383350,  383383, 24, 33, 31, }, // Falkland
    {  351,  704,  383414,  383428,  383451, 14, 23, 21, }, // Fiji
    {  351,  709,       0,  383472,       0,  0, 25,  0, }, // French_Guiana
    {  351,  723,       0,  383497,       0,  0, 46,  0, }, // French_Southern
    {  351,  746,       0,  383543,       0,  0, 19,  0, }, // Galapagos
    {  351,  756,       0,  383562,       0,  0, 14,  0, }, // Gambier
    {  351,  764,  383576,  383593,  383619, 17, 26, 24, }, // Georgia
    {  351,  772,       0,  383643,       0,  0, 23,  0, }, // Gilbert_Islands
    {  351,  788,       0,  383666,       0,  0, 30,  0, }, // GMT
    {  351,  830,  383696,  383727,  383767, 31, 40, 38, }, // Greenland_Eastern
    {  351,  848,  383805,  383838,  383880, 33, 42, 40, }, // Greenland_Western
    {  351,  871,       0,  383920,       0,  0, 13,  0, }, // Gulf
    {  351,  876,       0,  383933,       0,  0, 16,  0, }, // Guyana
    {  351,  883,  383949,  383980,  384020, 31, 40, 38, }, // Hawaii_Aleutian
    {  351,  899,  384058,  384074,  384099, 16, 25, 23, }, // Hong_Kong
    {  351,  909,  384122,  384133,  384153, 11, 20, 18, }, // Hovd
    {  351,  914,       0,  384171,       0,  0, 23,  0, }, // India
    {  351,  920,       0,  384194,       0,  0, 23,  0, }, // Indian_Ocean
    {  351,  933,       0,  384217,       0,  0, 17,  0, }, // Indochina
    {  351,  943,       0,  384234,       0,  0, 27,  0, }, // Indonesia_Central
    {  351,  961,       0,  384261,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  351,  979,       0,  384289,       0,  0, 30,  0, }, // Indonesia_Western
    {  351,  997,  384319,  384332,  384354, 13, 22, 20, }, // Iran
    {  351, 1008,  384374,  384388,  384411, 14, 23, 21, }, // Irkutsk
    {  351, 1016,  384432,  384446,  384469, 14, 23, 21, }, // Israel
    {  351, 1023,  384490,  384506,  384531, 16, 25, 23, }, // Japan
    {  351, 1029,  384554,  384585,  384625, 31, 40, 38, }, // Kamchatka
    {  351, 1047,       0,  384663,       0,  0, 18,  0, }, // Kazakhstan
    {  351, 1058,       0,  384681,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  351, 1077,       0,  384709,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  351, 1106,  384739,  384750,  384770, 11, 20, 18, }, // Korea
    {  351, 1112,       0,  384788,       0,  0, 14,  0, }, // Kosrae
    {  351, 1119,  384802,  384820,  384847, 18, 27, 25, }, // Krasnoyarsk
    {  351, 1151,       0,  384872,       0,  0, 20,  0, }, // Kyrgystan
    {  351, 1175,       0,  384892,       0,  0, 29,  0, }, // Line_Islands
    {  351, 1188,  384921,  384937,  384962, 16, 25, 23, }, // Lord_Howe
    {  351, 1198,  384985,  384997,  385018, 12, 21, 19, }, // Macau
    {  351, 1204,  385037,  385051,  385074, 14, 23, 21, }, // Magadan
    {  351, 1219,       0,  385095,       0,  0, 17,  0, }, // Malaysia
    {  351, 1228,       0,  385112,       0,  0, 17,  0, }, // Maldives
    {  351, 1237,       0,  385129,       0,  0, 18,  0, }, // Marquesas
    {  351, 1247,       0,  385147,       0,  0, 24,  0, }, // Marshall_Islands
    {  351, 1264,  385171,  385190,  385218, 19, 28, 26, }, // Mauritius
    {  351, 1274,       0,  385244,       0,  0, 13,  0, }, // Mawson
    {  351, 1281,  385257,  385283,  385318, 26, 35, 33, }, // Mexico_Pacific
    {  351, 1296,  385351,  385368,  385394, 17, 26, 24, }, // Mongolia
    {  351, 1305,  385418,  385430,  385451, 12, 21, 19, }, // Moscow
    {  351, 1312,       0,  385470,       0,  0, 18,  0, }, // Myanmar
    {  351, 1320,       0,  385488,       0,  0, 12,  0, }, // Nauru
    {  351, 1326,       0,  385500,       0,  0, 13,  0, }, // Nepal
    {  351, 1332,  385513,  385538,  385572, 25, 34, 32, }, // New_Caledonia
    {  351, 1346,  385604,  385627,  385659, 23, 32, 30, }, // New_Zealand
    {  351, 1358,  385689,  385705,  385730, 16, 25, 23, }, // Newfoundland
    {  351, 1371,       0,  385753,       0,  0, 11,  0, }, // Niue
    {  351, 1376,  385764,  385787,  385819, 23, 32, 30, }, // Norfolk
    {  351, 1384,  385849,  385875,  385910, 26, 35, 33, }, // Noronha
    {  351, 1406,  385943,  385961,  385988, 18, 27, 25, }, // Novosibirsk
    {  351, 1418,  386013,  386024,  386044, 11, 20, 18, }, // Omsk
    {  351, 1428,  386062,  386078,  386103, 16, 25, 23, }, // Pakistan
    {  351, 1437,       0,  386126,       0,  0, 12,  0, }, // Palau
    {  351, 1443,       0,  386138,       0,  0, 28,  0, }, // Papua_New_Guinea
    {  351, 1460,  386166,  386182,  386207, 16, 25, 23, }, // Paraguay
    {  351, 1469,  386230,  386242,  386263, 12, 21, 19, }, // Peru
    {  351, 1474,  386282,  386301,  386329, 19, 28, 26, }, // Philippines
    {  351, 1486,       0,  386355,       0,  0, 28,  0, }, // Phoenix_Islands
    {  351, 1502,  386383,  386413,  386452, 30, 39, 37, }, // Pierre_Miquelon
    {  351, 1518,       0,  386489,       0,  0, 18,  0, }, // Pitcairn
    {  351, 1527,       0,  386507,       0,  0, 14,  0, }, // Ponape
    {  351, 1534,       0,  386521,       0,  0, 16,  0, }, // Pyongyang
    {  351, 1554,       0,  386537,       0,  0, 15,  0, }, // Reunion
    {  351, 1562,       0,  386552,       0,  0, 14,  0, }, // Rothera
    {  351, 1570,  386566,  386581,  386605, 15, 24, 22, }, // Sakhalin
    {  351, 1579,  386627,  386640,  386662, 13, 22, 20, }, // Samara
    {  351, 1596,  386682,  386694,  386715, 12, 21, 19, }, // Samoa
    {  351, 1602,       0,  386734,       0,  0, 20,  0, }, // Seychelles
    {  351, 1624,       0,  386754,       0,  0, 16,  0, }, // Singapore
    {  351, 1634,       0,  386770,       0,  0, 24,  0, }, // Solomon
    {  351, 1642,       0,  386794,       0,  0, 25,  0, }, // South_Georgia
    {  351, 1656,       0,  386819,       0,  0, 16,  0, }, // Suriname
    {  351, 1676,       0,  386835,       0,  0, 12,  0, }, // Syowa
    {  351, 1682,       0,  386847,       0,  0, 13,  0, }, // Tahiti
    {  351, 1689,  386860,  386873,  386895, 13, 22, 20, }, // Taipei
    {  351, 1696,       0,  386915,       0,  0, 18,  0, }, // Tajikistan
    {  351, 1724,       0,  386933,       0,  0, 14,  0, }, // Tokelau
    {  351, 1732,  386947,  386959,  386980, 12, 21, 19, }, // Tonga
    {  351, 1738,       0,  386999,       0,  0, 13,  0, }, // Truk
    {  351, 1750,  387012,  387032,  387061, 20, 29, 27, }, // Turkmenistan
    {  351, 1763,       0,  387088,       0,  0, 13,  0, }, // Tuvalu
    {  351, 1777,  387101,  387117,  387142, 16, 25, 23, }, // Uruguay
    {  351, 1792,  387165,  387184,  387212, 19, 28, 26, }, // Uzbekistan
    {  351, 1803,  387238,  387253,  387277, 15, 24, 22, }, // Vanuatu
    {  351, 1811,       0,  387299,       0,  0, 17,  0, }, // Venezuela
    {  351, 1821,  387316,  387334,  387361, 18, 27, 25, }, // Vladivostok
    {  351, 1833,  387386,  387402,  387427, 16, 25, 23, }, // Volgograd
    {  351, 1843,       0,  387450,       0,  0, 13,  0, }, // Vostok
    {  351, 1850,       0,  387463,       0,  0, 22,  0, }, // Wake
    {  351, 1855,       0,  387485,       0,  0, 22,  0, }, // Wallis
    {  351, 1862,  387507,  387521,  387544, 14, 23, 21, }, // Yakutsk
    {  351, 1870,  387565,  387584,  387612, 19, 28, 26, }, // Yekaterinburg
    {  351, 1892,       0,  387638,       0,  0, 15,  0, }, // Yukon
    {  355,    0,  387653,  387658,  387664,  5,  6,  6, }, // Acre Japanese/Japanese/Japan
    {  355,    5,       0,  387670,       0,  0,  9,  0, }, // Afghanistan
    {  355,   17,       0,  387679,       0,  0,  8,  0, }, // Africa_Central
    {  355,   32,       0,  387687,       0,  0,  7,  0, }, // Africa_Eastern
    {  355,   65,       0,  387694,       0,  0,  8,  0, }, // Africa_Southern
    {  355,   81,  387702,  387709,  387717,  7,  8,  8, }, // Africa_Western
    {  355,  107,  387725,  387731,  387738,  6,  7,  7, }, // Alaska
    {  355,  128,  387745,  387752,  387760,  7,  8,  8, }, // Almaty
    {  355,  135,  387768,  387774,  387781,  6,  7,  7, }, // Amazon
    {  355,  142,  387788,  387796,  387805,  8,  9,  9, }, // America_Central
    {  355,  158,  387814,  387822,  387831,  8,  9,  9, }, // America_Eastern
    {  355,  174,  387840,  387848,  387857,  8,  9,  9, }, // America_Mountain
    {  355,  191,  387866,  387875,  387885,  9, 10, 10, }, // America_Pacific
    {  355,  207,  387895,  387902,  387910,  7,  8,  8, }, // Anadyr
    {  355,  214,  387918,  387923,  387929,  5,  6,  6, }, // Apia
    {  355,  219,  387935,  387941,  387948,  6,  7,  7, }, // Aqtau
    {  355,  225,  387955,  387961,  387968,  6,  7,  7, }, // Aqtobe
    {  355,  232,  387975,  387981,  387988,  6,  7,  7, }, // Arabian
    {  355,  240,  387995,  388003,  388012,  8,  9,  9, }, // Argentina
    {  355,  250,  388021,  388031,  388042, 10, 11, 11, }, // Argentina_Western
    {  355,  268,  388053,  388060,  388068,  7,  8,  8, }, // Armenia
    {  355,  286,  133320,  133325,  388076,  5,  6,  6, }, // Atlantic
    {  355,  295,  388082,  388093,  388105, 11, 12, 12, }, // Australia_Central
    {  355,  313,  388117,  388129,  388142, 12, 13, 13, }, // Australia_CentralWestern
    {  355,  338,  388155,  388166,  388178, 11, 12, 12, }, // Australia_Eastern
    {  355,  356,  388190,  388201,  388213, 11, 12, 12, }, // Australia_Western
    {  355,  374,  388225,  388235,  388246, 10, 11, 11, }, // Azerbaijan
    {  355,  385,  388257,  388263,  388270,  6,  7,  7, }, // Azores
    {  355,  397,  388277,  388286,  388296,  9, 10, 10, }, // Bangladesh
    {  355,  415,       0,  388306,       0,  0,  6,  0, }, // Bhutan
    {  355,  422,       0,  388312,       0,  0,  6,  0, }, // Bolivia
    {  355,  437,  388318,  388325,  388333,  7,  8,  8, }, // Brasilia
    {  355,  454,       0,  388341,       0,  0, 13,  0, }, // Brunei
    {  355,  461,  388354,  388362,  388371,  8,  9,  9, }, // Cape_Verde
    {  355,  472,       0,  388380,       0,  0,  8,  0, }, // Casey
    {  355,  478,       0,  388388,       0,  0,  6,  0, }, // Chamorro
    {  355,  487,  388394,  388400,  388407,  6,  7,  7, }, // Chatham
    {  355,  495,  388414,  388418,  388423,  4,  5,  5, }, // Chile
    {  355,  501,  388428,  388432,  388437,  4,  5,  5, }, // China
    {  355,  507,       0,  388442,       0,  0,  8,  0, }, // Christmas
    {  355,  517,       0,  388450,       0,  0,  7,  0, }, // Cocos
    {  355,  523,  388457,  388464,  388472,  7,  8,  8, }, // Colombia
    {  355,  532,  388480,  388487,  388495,  7,  8,  8, }, // Cook
    {  355,  537,  388503,  388509,  388516,  6,  7,  7, }, // Cuba
    {  355,  548,       0,  388523,       0,  0,  8,  0, }, // Davis
    {  355,  564,       0,  388531,       0,  0, 15,  0, }, // DumontDUrville
    {  355,  601,       0,  388546,       0,  0,  8,  0, }, // East_Timor
    {  355,  612,  388554,  388562,  388571,  8,  9,  9, }, // Easter
    {  355,  619,       0,  388580,       0,  0,  7,  0, }, // Ecuador
    {  355,  627,  388587,  388596,  388606,  9, 10, 10, }, // Europe_Central
    {  355,  642,  388616,  388624,  388633,  8,  9,  9, }, // Europe_Eastern
    {  355,  657,       0,  388642,       0,  0,  9,  0, }, // Europe_Further_Eastern
    {  355,  680,  388651,  388659,  388668,  8,  9,  9, }, // Europe_Western
    {  355,  695,  388677,  388688,  388700, 11, 12, 12, }, // Falkland
    {  355,  704,  388712,  388718,  388725,  6,  7,  7, }, // Fiji
    {  355,  709,       0,  388732,       0,  0,  7,  0, }, // French_Guiana
    {  355,  723,       0,  388739,       0,  0,  8,  0, }, // French_Southern
    {  355,  746,       0,  388747,       0,  0,  7,  0, }, // Galapagos
    {  355,  756,       0,  388754,       0,  0,  8,  0, }, // Gambier
    {  355,  764,  388762,  388769,  388777,  7,  8,  8, }, // Georgia
    {  355,  772,       0,  388785,       0,  0,  9,  0, }, // Gilbert_Islands
    {  355,  788,       0,  388794,       0,  0,  8,  0, }, // GMT
    {  355,  830,  388802,  388813,  388825, 11, 12, 12, }, // Greenland_Eastern
    {  355,  848,  388837,  388848,  388860, 11, 12, 12, }, // Greenland_Western
    {  355,  866,       0,  388872,       0,  0,  5,  0, }, // Guam
    {  355,  871,       0,  388877,       0,  0,  5,  0, }, // Gulf
    {  355,  876,       0,  388882,       0,  0,  6,  0, }, // Guyana
    {  355,  883,  388888,  388901,  388915, 13, 14, 14, }, // Hawaii_Aleutian
    {  355,  899,  133994,  133998,  388929,  4,  5,  5, }, // Hong_Kong
    {  355,  909,  388934,  388939,  388945,  5,  6,  6, }, // Hovd
    {  355,  914,       0,  388951,       0,  0,  6,  0, }, // India
    {  355,  920,       0,  388957,       0,  0,  6,  0, }, // Indian_Ocean
    {  355,  933,       0,  388963,       0,  0,  7,  0, }, // Indochina
    {  355,  943,       0,  388970,       0,  0, 10,  0, }, // Indonesia_Central
    {  355,  961,       0,  388980,       0,  0, 10,  0, }, // Indonesia_Eastern
    {  355,  979,       0,  388990,       0,  0, 10,  0, }, // Indonesia_Western
    {  355,  997,  389000,  389005,  389011,  5,  6,  6, }, // Iran
    {  355, 1008,  389017,  389025,  389034,  8,  9,  9, }, // Irkutsk
    {  355, 1016,  389043,  389050,  389058,  7,  8,  8, }, // Israel
    {  355, 1023,  134124,  134128,  389066,  4,  5,  5, }, // Japan
    {  355, 1029,  389071,  389090,  389110, 19, 20, 20, }, // Kamchatka
    {  355, 1047,       0,  389130,       0,  0,  8,  0, }, // Kazakhstan
    {  355, 1058,       0,  389138,       0,  0,  9,  0, }, // Kazakhstan_Eastern
    {  355, 1077,       0,  389147,       0,  0,  9,  0, }, // Kazakhstan_Western
    {  355, 1106,  389156,  389160,  389165,  4,  5,  5, }, // Korea
    {  355, 1112,       0,  389170,       0,  0,  6,  0, }, // Kosrae
    {  355, 1119,  389176,  389186,  389197, 10, 11, 11, }, // Krasnoyarsk
    {  355, 1151,       0,  389208,       0,  0,  6,  0, }, // Kyrgystan
    {  355, 1161,       0,  389214,       0,  0,  5,  0, }, // Lanka
    {  355, 1175,       0,  389219,       0,  0,  7,  0, }, // Line_Islands
    {  355, 1188,  389226,  389233,  389241,  7,  8,  8, }, // Lord_Howe
    {  355, 1198,  389249,  389254,  389260,  5,  6,  6, }, // Macau
    {  355, 1204,  389266,  389272,  389279,  6,  7,  7, }, // Magadan
    {  355, 1219,       0,  389286,       0,  0,  7,  0, }, // Malaysia
    {  355, 1228,       0,  389293,       0,  0,  7,  0, }, // Maldives
    {  355, 1237,       0,  389300,       0,  0,  7,  0, }, // Marquesas
    {  355, 1247,       0,  389307,       0,  0,  9,  0, }, // Marshall_Islands
    {  355, 1264,  389316,  389324,  389333,  8,  9,  9, }, // Mauritius
    {  355, 1274,       0,  389342,       0,  0,  8,  0, }, // Mawson
    {  355, 1281,  389350,  389359,  389369,  9, 10, 10, }, // Mexico_Pacific
    {  355, 1296,  389379,  389388,  389398,  9, 10, 10, }, // Mongolia
    {  355, 1305,  389408,  389414,  389421,  6,  7,  7, }, // Moscow
    {  355, 1312,       0,  389428,       0,  0,  7,  0, }, // Myanmar
    {  355, 1320,       0,  389435,       0,  0,  5,  0, }, // Nauru
    {  355, 1326,       0,  389440,       0,  0,  6,  0, }, // Nepal
    {  355, 1332,  389446,  389456,  389467, 10, 11, 11, }, // New_Caledonia
    {  355, 1346,  389478,  389488,  389499, 10, 11, 11, }, // New_Zealand
    {  355, 1358,  389510,  389522,  389535, 12, 13, 13, }, // Newfoundland
    {  355, 1371,       0,  389548,       0,  0,  5,  0, }, // Niue
    {  355, 1376,  389553,  389562,  389572,  9, 10, 10, }, // Norfolk
    {  355, 1384,  389582,  389598,  389615, 16, 17, 17, }, // Noronha
    {  355, 1392,       0,  389632,       0,  0,  9,  0, }, // North_Mariana
    {  355, 1406,  389641,  389651,  389662, 10, 11, 11, }, // Novosibirsk
    {  355, 1418,  389673,  389679,  389686,  6,  7,  7, }, // Omsk
    {  355, 1428,  389693,  389700,  389708,  7,  8,  8, }, // Pakistan
    {  355, 1437,       0,  389716,       0,  0,  5,  0, }, // Palau
    {  355, 1443,       0,  389721,       0,  0, 11,  0, }, // Papua_New_Guinea
    {  355, 1460,  389732,  389739,  389747,  7,  8,  8, }, // Paraguay
    {  355, 1469,  389755,  389760,  389766,  5,  6,  6, }, // Peru
    {  355, 1474,  389772,  389779,  389787,  7,  8,  8, }, // Philippines
    {  355, 1486,       0,  389795,       0,  0, 10,  0, }, // Phoenix_Islands
    {  355, 1502,  389805,  389820,  389836, 15, 16, 16, }, // Pierre_Miquelon
    {  355, 1518,       0,  389852,       0,  0,  7,  0, }, // Pitcairn
    {  355, 1527,       0,  389859,       0,  0,  5,  0, }, // Ponape
    {  355, 1534,       0,  389864,       0,  0,  4,  0, }, // Pyongyang
    {  355, 1544,  389868,  389875,  389883,  7,  8,  8, }, // Qyzylorda
    {  355, 1554,       0,  389891,       0,  0,  7,  0, }, // Reunion
    {  355, 1562,       0,  389898,       0,  0,  7,  0, }, // Rothera
    {  355, 1570,  389905,  389911,  389918,  6,  7,  7, }, // Sakhalin
    {  355, 1579,  389925,  389930,  389936,  5,  6,  6, }, // Samara
    {  355, 1596,  389942,  389947,  389953,  5,  6,  6, }, // Samoa
    {  355, 1602,       0,  389959,       0,  0,  7,  0, }, // Seychelles
    {  355, 1624,       0,  389966,       0,  0,  9,  0, }, // Singapore
    {  355, 1634,       0,  389975,       0,  0,  8,  0, }, // Solomon
    {  355, 1642,       0,  389983,       0,  0, 10,  0, }, // South_Georgia
    {  355, 1656,       0,  389993,       0,  0,  6,  0, }, // Suriname
    {  355, 1676,       0,  134928,       0,  0,  6,  0, }, // Syowa
    {  355, 1682,       0,  389999,       0,  0,  5,  0, }, // Tahiti
    {  355, 1689,  134939,  134943,  390004,  4,  5,  5, }, // Taipei
    {  355, 1696,       0,  390009,       0,  0,  8,  0, }, // Tajikistan
    {  355, 1724,       0,  390017,       0,  0,  6,  0, }, // Tokelau
    {  355, 1732,  390023,  390028,  390034,  5,  6,  6, }, // Tonga
    {  355, 1738,       0,  390040,       0,  0,  6,  0, }, // Truk
    {  355, 1750,  390046,  390056,  390067, 10, 11, 11, }, // Turkmenistan
    {  355, 1763,       0,  390078,       0,  0,  5,  0, }, // Tuvalu
    {  355, 1777,  390083,  390090,  390098,  7,  8,  8, }, // Uruguay
    {  355, 1792,  390106,  390115,  390125,  9, 10, 10, }, // Uzbekistan
    {  355, 1803,  390135,  390141,  390148,  6,  7,  7, }, // Vanuatu
    {  355, 1811,       0,  390155,       0,  0,  7,  0, }, // Venezuela
    {  355, 1821,  390162,  390171,  390181,  9, 10, 10, }, // Vladivostok
    {  355, 1833,  390191,  390200,  390210,  9, 10, 10, }, // Volgograd
    {  355, 1843,       0,  390220,       0,  0,  9,  0, }, // Vostok
    {  355, 1850,       0,  390229,       0,  0,  7,  0, }, // Wake
    {  355, 1855,       0,  390236,       0,  0, 10,  0, }, // Wallis
    {  355, 1862,  390246,  390253,  390261,  7,  8,  8, }, // Yakutsk
    {  355, 1870,  390269,  390279,  390290, 10, 11, 11, }, // Yekaterinburg
    {  355, 1892,       0,  390301,       0,  0,  6,  0, }, // Yukon
    {  356,    5,       0,  390307,       0,  0, 17,  0, }, // Afghanistan Javanese/Latin/Indonesia
    {  356,   17,       0,  390324,       0,  0, 19,  0, }, // Africa_Central
    {  356,   32,       0,  390343,       0,  0, 18,  0, }, // Africa_Eastern
    {  356,   65,       0,  390361,       0,  0, 26,  0, }, // Africa_Southern
    {  356,   81,  390387,  390405,  390431, 18, 26, 25, }, // Africa_Western
    {  356,  107,  390456,  390468,  390488, 12, 20, 19, }, // Alaska
    {  356,  135,  390507,  390519,  390539, 12, 20, 19, }, // Amazon
    {  356,  142,  390558,  390570,  390590, 12, 20, 19, }, // America_Central
    {  356,  158,  390609,  390626,  390651, 17, 25, 24, }, // America_Eastern
    {  356,  174,  390675,  390685,  390703, 10, 18, 17, }, // America_Mountain
    {  356,  191,  390720,  390733,  390754, 13, 21, 20, }, // America_Pacific
    {  356,  214,  390774,  390784,  390802, 10, 18, 17, }, // Apia
    {  356,  232,  390819,  390829,  390847, 10, 18, 17, }, // Arabian
    {  356,  240,  390864,  390879,  390902, 15, 23, 22, }, // Argentina
    {  356,  250,  390924,  390951,  390986, 27, 35, 34, }, // Argentina_Western
    {  356,  268,  391020,  391033,  391054, 13, 21, 20, }, // Armenia
    {  356,  286,  391074,  391088,  391110, 14, 22, 21, }, // Atlantic
    {  356,  295,  391131,  391153,  391183, 22, 30, 29, }, // Australia_Central
    {  356,  313,  391212,  391246,  391288, 34, 42, 41, }, // Australia_CentralWestern
    {  356,  338,  391329,  391356,  391391, 27, 35, 34, }, // Australia_Eastern
    {  356,  356,  391425,  391452,  391487, 27, 35, 34, }, // Australia_Western
    {  356,  374,  391521,  391537,  391561, 16, 24, 23, }, // Azerbaijan
    {  356,  385,  391584,  391596,  391616, 12, 20, 19, }, // Azores
    {  356,  397,  391635,  391651,  391675, 16, 24, 23, }, // Bangladesh
    {  356,  415,       0,  391698,       0,  0, 12,  0, }, // Bhutan
    {  356,  422,       0,  391710,       0,  0, 13,  0, }, // Bolivia
    {  356,  437,  391723,  391737,  391759, 14, 22, 21, }, // Brasilia
    {  356,  454,       0,  391780,       0,  0, 23,  0, }, // Brunei
    {  356,  461,  391803,  391822,  391849, 19, 27, 26, }, // Cape_Verde
    {  356,  478,       0,  391875,       0,  0, 14,  0, }, // Chamorro
    {  356,  487,  391889,  391902,  391923, 13, 21, 20, }, // Chatham
    {  356,  495,  391943,  391954,  391973, 11, 19, 18, }, // Chile
    {  356,  501,  391991,  392001,  392019, 10, 18, 17, }, // China
    {  356,  507,       0,  392036,       0,  0, 16,  0, }, // Christmas
    {  356,  517,       0,  392052,       0,  0, 20,  0, }, // Cocos
    {  356,  523,  392072,  392086,  392108, 14, 22, 21, }, // Colombia
    {  356,  532,  392129,  392148,  392175, 19, 27, 26, }, // Cook
    {  356,  537,  392201,  392211,  392229, 10, 18, 17, }, // Cuba
    {  356,  548,       0,  392246,       0,  0, 11,  0, }, // Davis
    {  356,  564,       0,  392257,       0,  0, 22,  0, }, // DumontDUrville
    {  356,  601,       0,  392279,       0,  0, 17,  0, }, // East_Timor
    {  356,  612,  392296,  392313,  392338, 17, 25, 24, }, // Easter
    {  356,  619,       0,  392362,       0,  0, 13,  0, }, // Ecuador
    {  356,  627,  392375,  392393,  392419, 18, 26, 25, }, // Europe_Central
    {  356,  642,  392444,  392467,  392498, 23, 31, 30, }, // Europe_Eastern
    {  356,  657,       0,  392528,       0,  0, 39,  0, }, // Europe_Further_Eastern
    {  356,  680,  392567,  392590,  392621, 23, 31, 30, }, // Europe_Western
    {  356,  695,  392651,  392674,  392705, 23, 31, 30, }, // Falkland
    {  356,  704,  392735,  392745,  392763, 10, 18, 17, }, // Fiji
    {  356,  709,       0,  392780,       0,  0, 20,  0, }, // French_Guiana
    {  356,  723,       0,  392800,       0,  0, 39,  0, }, // French_Southern
    {  356,  746,       0,  392839,       0,  0, 15,  0, }, // Galapagos
    {  356,  756,       0,  392854,       0,  0, 13,  0, }, // Gambier
    {  356,  764,  392867,  392880,  392901, 13, 21, 20, }, // Georgia
    {  356,  772,       0,  392921,       0,  0, 22,  0, }, // Gilbert_Islands
    {  356,  788,       0,  392943,       0,  0, 22,  0, }, // GMT
    {  356,  830,  392965,  392985,  393013, 20, 28, 27, }, // Greenland_Eastern
    {  356,  848,  393040,  393060,  393088, 20, 28, 27, }, // Greenland_Western
    {  356,  871,       0,  393115,       0,  0, 19,  0, }, // Gulf
    {  356,  876,       0,  393134,       0,  0, 12,  0, }, // Guyana
    {  356,  883,  393146,  393167,  393196, 21, 29, 28, }, // Hawaii_Aleutian
    {  356,  899,  393224,  393239,  393262, 15, 23, 22, }, // Hong_Kong
    {  356,  909,  393284,  393294,  393312, 10, 18, 17, }, // Hovd
    {  356,  914,       0,  393329,       0,  0, 19,  0, }, // India
    {  356,  920,       0,  393348,       0,  0, 19,  0, }, // Indian_Ocean
    {  356,  933,       0,  393367,       0,  0, 14,  0, }, // Indochina
    {  356,  943,       0,  393381,       0,  0, 22,  0, }, // Indonesia_Central
    {  356,  961,       0,  393403,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  356,  979,       0,  393430,       0,  0, 27,  0, }, // Indonesia_Western
    {  356,  997,  393457,  393467,  393485, 10, 18, 17, }, // Iran
    {  356, 1008,  393502,  393515,  393536, 13, 21, 20, }, // Irkutsk
    {  356, 1016,  393556,  393568,  393588, 12, 20, 19, }, // Israel
    {  356, 1023,  393607,  393619,  393639, 12, 20, 19, }, // Japan
    {  356, 1047,       0,  393658,       0,  0, 16,  0, }, // Kazakhstan
    {  356, 1058,       0,  393674,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  356, 1077,       0,  393696,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  356, 1106,  393718,  393729,  393748, 11, 19, 18, }, // Korea
    {  356, 1112,       0,  393766,       0,  0, 12,  0, }, // Kosrae
    {  356, 1119,  393778,  393795,  393820, 17, 25, 24, }, // Krasnoyarsk
    {  356, 1151,       0,  393844,       0,  0, 16,  0, }, // Kyrgystan
    {  356, 1175,       0,  393860,       0,  0, 19,  0, }, // Line_Islands
    {  356, 1188,  393879,  393894,  393917, 15, 23, 22, }, // Lord_Howe
    {  356, 1204,  393939,  393952,  393973, 13, 21, 20, }, // Magadan
    {  356, 1219,       0,  393993,       0,  0, 14,  0, }, // Malaysia
    {  356, 1228,       0,  394007,       0,  0, 14,  0, }, // Maldives
    {  356, 1237,       0,  394021,       0,  0, 15,  0, }, // Marquesas
    {  356, 1247,       0,  394036,       0,  0, 23,  0, }, // Marshall_Islands
    {  356, 1264,  394059,  394074,  394097, 15, 23, 22, }, // Mauritius
    {  356, 1274,       0,  394119,       0,  0, 12,  0, }, // Mawson
    {  356, 1281,  394131,  394152,  394181, 21, 29, 28, }, // Mexico_Pacific
    {  356, 1296,  394209,  394226,  394251, 17, 25, 24, }, // Mongolia
    {  356, 1305,  394275,  394287,  394307, 12, 20, 19, }, // Moscow
    {  356, 1312,       0,  394326,       0,  0, 13,  0, }, // Myanmar
    {  356, 1320,       0,  394339,       0,  0, 11,  0, }, // Nauru
    {  356, 1326,       0,  394350,       0,  0, 11,  0, }, // Nepal
    {  356, 1332,  394361,  394382,  394411, 21, 29, 28, }, // New_Caledonia
    {  356, 1346,  394439,  394459,  394487, 20, 28, 27, }, // New_Zealand
    {  356, 1358,  394514,  394532,  394558, 18, 26, 25, }, // Newfoundland
    {  356, 1371,       0,  394583,       0,  0, 10,  0, }, // Niue
    {  356, 1376,  394593,  394611,  394637, 18, 26, 25, }, // Norfolk
    {  356, 1384,  394662,  394687,  394720, 25, 33, 32, }, // Noronha
    {  356, 1406,  394752,  394769,  394794, 17, 25, 24, }, // Novosibirsk
    {  356, 1418,  394818,  394828,  394846, 10, 18, 17, }, // Omsk
    {  356, 1428,  394863,  394877,  394899, 14, 22, 21, }, // Pakistan
    {  356, 1437,       0,  394920,       0,  0, 11,  0, }, // Palau
    {  356, 1443,       0,  394931,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  356, 1460,  394949,  394963,  394985, 14, 22, 21, }, // Paraguay
    {  356, 1469,  395006,  395016,  395034, 10, 18, 17, }, // Peru
    {  356, 1474,  395051,  395065,  395087, 14, 22, 21, }, // Philippines
    {  356, 1486,       0,  395108,       0,  0, 22,  0, }, // Phoenix_Islands
    {  356, 1502,  395130,  395161,  395200, 31, 39, 38, }, // Pierre_Miquelon
    {  356, 1518,       0,  395238,       0,  0, 14,  0, }, // Pitcairn
    {  356, 1527,       0,  395252,       0,  0, 12,  0, }, // Ponape
    {  356, 1534,       0,  395264,       0,  0, 15,  0, }, // Pyongyang
    {  356, 1554,       0,  395279,       0,  0, 13,  0, }, // Reunion
    {  356, 1562,       0,  395292,       0,  0, 13,  0, }, // Rothera
    {  356, 1570,  395305,  395319,  395341, 14, 22, 21, }, // Sakhalin
    {  356, 1596,  395362,  395373,  395392, 11, 19, 18, }, // Samoa
    {  356, 1602,       0,  395410,       0,  0, 16,  0, }, // Seychelles
    {  356, 1624,       0,  395426,       0,  0, 15,  0, }, // Singapore
    {  356, 1634,       0,  395441,       0,  0, 22,  0, }, // Solomon
    {  356, 1642,       0,  395463,       0,  0, 19,  0, }, // South_Georgia
    {  356, 1656,       0,  395482,       0,  0, 14,  0, }, // Suriname
    {  356, 1676,       0,  395496,       0,  0, 11,  0, }, // Syowa
    {  356, 1682,       0,  395507,       0,  0, 12,  0, }, // Tahiti
    {  356, 1689,  395519,  395531,  395551, 12, 20, 19, }, // Taipei
    {  356, 1696,       0,  395570,       0,  0, 16,  0, }, // Tajikistan
    {  356, 1724,       0,  395586,       0,  0, 13,  0, }, // Tokelau
    {  356, 1732,  395599,  395610,  395629, 11, 19, 18, }, // Tonga
    {  356, 1738,       0,  395647,       0,  0, 11,  0, }, // Truk
    {  356, 1750,  395658,  395676,  395702, 18, 26, 25, }, // Turkmenistan
    {  356, 1763,       0,  395727,       0,  0, 12,  0, }, // Tuvalu
    {  356, 1777,  395739,  395752,  395773, 13, 21, 20, }, // Uruguay
    {  356, 1792,  395793,  395809,  395833, 16, 24, 23, }, // Uzbekistan
    {  356, 1803,  395856,  395869,  395890, 13, 21, 20, }, // Vanuatu
    {  356, 1811,       0,  395910,       0,  0, 15,  0, }, // Venezuela
    {  356, 1821,  395925,  395942,  395967, 17, 25, 24, }, // Vladivostok
    {  356, 1833,  395991,  396006,  396029, 15, 23, 22, }, // Volgograd
    {  356, 1843,       0,  396051,       0,  0, 12,  0, }, // Vostok
    {  356, 1850,       0,  396063,       0,  0, 15,  0, }, // Wake
    {  356, 1855,       0,  396078,       0,  0, 23,  0, }, // Wallis
    {  356, 1862,  396101,  396114,  396135, 13, 21, 20, }, // Yakutsk
    {  356, 1870,  396155,  396174,  396201, 19, 27, 26, }, // Yekaterinburg
    {  356, 1892,       0,  396227,       0,  0, 11,  0, }, // Yukon
    {  359,   17,       0,  396238,       0,  0, 21,  0, }, // Africa_Central Kabuverdianu/Latin/Cape Verde
    {  359,   32,       0,  396259,       0,  0, 22,  0, }, // Africa_Eastern
    {  359,   65,       0,  396281,       0,  0, 20,  0, }, // Africa_Southern
    {  359,   81,  396301,  396324,  396354, 23, 30, 32, }, // Africa_Western
    {  359,  107,  396386,  396399,  396419, 13, 20, 22, }, // Alaska
    {  359,  142,  396441,  396452,  396470, 11, 18, 20, }, // America_Central
    {  359,  158,  396490,  396502,  396521, 12, 19, 21, }, // America_Eastern
    {  359,  174,  396542,  396557,  396579, 15, 22, 24, }, // America_Mountain
    {  359,  191,  396603,  396618,  396640, 15, 22, 24, }, // America_Pacific
    {  359,  286,  396664,  396680,  396703, 16, 23, 25, }, // Atlantic
    {  359,  295,  396728,  396752,  396783, 24, 31, 33, }, // Australia_Central
    {  359,  313,  396816,  396849,  396889, 33, 40, 42, }, // Australia_CentralWestern
    {  359,  338,  396931,  396956,  396988, 25, 32, 34, }, // Australia_Eastern
    {  359,  356,  397022,  397048,  397081, 26, 33, 35, }, // Australia_Western
    {  359,  461,  397116,  397133,  397157, 17, 24, 26, }, // Cape_Verde
    {  359,  537,  397183,  397194,  397212, 11, 18, 20, }, // Cuba
    {  359,  627,  397232,  397253,  397281, 21, 28, 30, }, // Europe_Central
    {  359,  642,  397311,  397333,  397362, 22, 29, 31, }, // Europe_Eastern
    {  359,  680,  397393,  397416,  397446, 23, 30, 32, }, // Europe_Western
    {  359,  788,       0,  397478,       0,  0, 22,  0, }, // GMT
    {  359,  830,  397500,  397527,  397561, 27, 34, 36, }, // Greenland_Eastern
    {  359,  848,  397597,  397625,  397660, 28, 35, 37, }, // Greenland_Western
    {  359,  883,  397697,  397718,  397746, 21, 28, 30, }, // Hawaii_Aleutian
    {  359, 1281,  397776,  397800,  397831, 24, 31, 33, }, // Mexico_Pacific
    {  359, 1358,  397864,  397880,  397903, 16, 23, 25, }, // Newfoundland
    {  359, 1502,  397928,  397954,  397987, 26, 33, 35, }, // Pierre_Miquelon
    {  359, 1892,       0,  398022,       0,  0, 12,  0, }, // Yukon
    {  360,    5,       0,  398034,       0,  0, 17,  0, }, // Afghanistan Kabyle/Latin/Algeria
    {  360,   17,       0,  398051,       0,  0, 24,  0, }, // Africa_Central
    {  360,   32,       0,  398075,       0,  0, 24,  0, }, // Africa_Eastern
    {  360,   65,       0,  398099,       0,  0, 30,  0, }, // Africa_Southern
    {  360,   81,  398129,  398151,  398181, 22, 30, 31, }, // Africa_Western
    {  360,  107,  398212,  398225,  398246, 13, 21, 22, }, // Alaska
    {  360,  135,  398268,  398281,  398302, 13, 21, 22, }, // Amazon
    {  360,  142,  398324,  398349,  398382, 25, 33, 34, }, // America_Central
    {  360,  158,  398416,  398440,  398472, 24, 32, 33, }, // America_Eastern
    {  360,  174,  398505,  398529,  398561, 24, 32, 33, }, // America_Mountain
    {  360,  191,  398594,  398605,  398624, 11, 19, 20, }, // America_Pacific
    {  360,  214,  398644,  398655,  398674, 11, 19, 20, }, // Apia
    {  360,  232,  398694,  398704,  398722, 10, 18, 19, }, // Arabian
    {  360,  240,  398741,  398756,  398779, 15, 23, 24, }, // Argentina
    {  360,  250,  398803,  398828,  398861, 25, 33, 34, }, // Argentina_Western
    {  360,  268,  398895,  398909,  398931, 14, 22, 23, }, // Armenia
    {  360,  286,  398954,  398966,  398986, 12, 20, 21, }, // Atlantic
    {  360,  295,  399007,  399032,  399065, 25, 33, 34, }, // Australia_Central
    {  360,  313,  399099,  399133,  399175, 34, 42, 43, }, // Australia_CentralWestern
    {  360,  338,  399218,  399243,  399276, 25, 33, 34, }, // Australia_Eastern
    {  360,  356,  399310,  399333,  399364, 23, 31, 30, }, // Australia_Western
    {  360,  374,  399394,  399410,  399434, 16, 24, 25, }, // Azerbaijan
    {  360,  385,  399459,  399472,  399493, 13, 21, 22, }, // Azores
    {  360,  397,  399515,  399531,  399555, 16, 24, 25, }, // Bangladesh
    {  360,  415,       0,  399580,       0,  0, 12,  0, }, // Bhutan
    {  360,  422,       0,  399592,       0,  0, 13,  0, }, // Bolivia
    {  360,  437,  399605,  399620,  399643, 15, 23, 24, }, // Brasilia
    {  360,  454,       0,  399667,       0,  0, 23,  0, }, // Brunei
    {  360,  461,  399690,  399704,  399726, 14, 22, 23, }, // Cape_Verde
    {  360,  478,       0,  399749,       0,  0, 22,  0, }, // Chamorro
    {  360,  487,  399771,  399784,  399805, 13, 21, 20, }, // Chatham
    {  360,  495,  399825,  399836,  399855, 11, 19, 20, }, // Chile
    {  360,  501,  399875,  399885,  399903, 10, 18, 19, }, // China
    {  360,  507,       0,  399922,       0,  0, 25,  0, }, // Christmas
    {  360,  517,       0,  399947,       0,  0, 22,  0, }, // Cocos
    {  360,  523,  399969,  399984,  400007, 15, 23, 24, }, // Colombia
    {  360,  532,  399947,  400031,  400060, 21, 29, 30, }, // Cook
    {  360,  537,  400090,  400101,  400120, 11, 19, 20, }, // Cuba
    {  360,  548,       0,  400140,       0,  0, 12,  0, }, // Davis
    {  360,  564,       0,  400152,       0,  0, 23,  0, }, // DumontDUrville
    {  360,  601,       0,  400175,       0,  0, 22,  0, }, // East_Timor
    {  360,  612,  400197,  400220,  400251, 23, 31, 32, }, // Easter
    {  360,  619,       0,  400283,       0,  0, 14,  0, }, // Ecuador
    {  360,  627,  400297,  400320,  400351, 23, 31, 32, }, // Europe_Central
    {  360,  642,  400383,  400406,  400437, 23, 31, 32, }, // Europe_Eastern
    {  360,  657,       0,  400469,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  360,  680,  400499,  400520,  400549, 21, 29, 28, }, // Europe_Western
    {  360,  695,  400577,  400603,  400637, 26, 34, 33, }, // Falkland
    {  360,  704,  400670,  400681,  400700, 11, 19, 20, }, // Fiji
    {  360,  709,       0,  400720,       0,  0, 24,  0, }, // French_Guiana
    {  360,  723,       0,  400744,       0,  0, 41,  0, }, // French_Southern
    {  360,  746,       0,  400785,       0,  0, 16,  0, }, // Galapagos
    {  360,  756,       0,  400801,       0,  0, 23,  0, }, // Gambier
    {  360,  764,  400824,  400838,  400860, 14, 22, 23, }, // Georgia
    {  360,  772,       0,  400883,       0,  0, 22,  0, }, // Gilbert_Islands
    {  360,  788,       0,  400905,       0,  0, 24,  0, }, // GMT
    {  360,  830,  400929,  400954,  400987, 25, 33, 34, }, // Greenland_Eastern
    {  360,  848,  401021,  401044,  401075, 23, 31, 32, }, // Greenland_Western
    {  360,  871,       0,  401107,       0,  0, 19,  0, }, // Gulf
    {  360,  876,       0,  400720,       0,  0, 13,  0, }, // Guyana
    {  360,  883,  401126,  401147,  401176, 21, 29, 30, }, // Hawaii_Aleutian
    {  360,  899,  401206,  401222,  401246, 16, 24, 25, }, // Hong_Kong
    {  360,  909,  401271,  401282,  401301, 11, 19, 20, }, // Hovd
    {  360,  914,       0,  401321,       0,  0, 20,  0, }, // India
    {  360,  920,       0,  401341,       0,  0, 20,  0, }, // Indian_Ocean
    {  360,  933,       0,  401361,       0,  0, 14,  0, }, // Indochina
    {  360,  943,       0,  401375,       0,  0, 27,  0, }, // Indonesia_Central
    {  360,  961,       0,  401402,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  360,  979,       0,  401428,       0,  0, 24,  0, }, // Indonesia_Western
    {  360,  997,  401452,  401463,  401482, 11, 19, 18, }, // Iran
    {  360, 1008,  401500,  401514,  401536, 14, 22, 23, }, // Irkutsk
    {  360, 1016,  401559,  401573,  401595, 14, 22, 23, }, // Israel
    {  360, 1023,  401618,  401630,  401650, 12, 20, 21, }, // Japan
    {  360, 1058,       0,  401671,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  360, 1077,       0,  401698,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  360, 1106,  401725,  401737,  401757, 12, 20, 21, }, // Korea
    {  360, 1112,       0,  401778,       0,  0, 13,  0, }, // Kosrae
    {  360, 1119,  401791,  401809,  401834, 18, 25, 27, }, // Krasnoyarsk
    {  360, 1151,       0,  401861,       0,  0, 17,  0, }, // Kyrgystan
    {  360, 1175,       0,  401878,       0,  0, 19,  0, }, // Line_Islands
    {  360, 1188,  401897,  401913,  401936, 16, 23, 25, }, // Lord_Howe
    {  360, 1204,  401961,  401975,  401997, 14, 22, 23, }, // Magadan
    {  360, 1219,       0,  402020,       0,  0, 14,  0, }, // Malaysia
    {  360, 1228,       0,  402034,       0,  0, 13,  0, }, // Maldives
    {  360, 1237,       0,  402047,       0,  0, 22,  0, }, // Marquesas
    {  360, 1247,       0,  402069,       0,  0, 22,  0, }, // Marshall_Islands
    {  360, 1264,  402091,  402103,  402123, 12, 20, 21, }, // Mauritius
    {  360, 1274,       0,  402144,       0,  0, 13,  0, }, // Mawson
    {  360, 1281,  402157,  402177,  402205, 20, 28, 29, }, // Mexico_Pacific
    {  360, 1296,  402234,  402250,  402274, 16, 24, 25, }, // Mongolia
    {  360, 1305,  402299,  402311,  402331, 12, 20, 21, }, // Moscow
    {  360, 1312,       0,  402352,       0,  0, 14,  0, }, // Myanmar
    {  360, 1320,       0,  402366,       0,  0, 11,  0, }, // Nauru
    {  360, 1326,       0,  402377,       0,  0, 12,  0, }, // Nepal
    {  360, 1332,  402389,  402414,  402447, 25, 33, 34, }, // New_Caledonia
    {  360, 1346,  402481,  402503,  402533, 22, 30, 29, }, // New_Zealand
    {  360, 1358,  402562,  402582,  402610, 20, 28, 29, }, // Newfoundland
    {  360, 1371,       0,  402639,       0,  0, 11,  0, }, // Niue
    {  360, 1376,  402650,  402674,  402706, 24, 32, 33, }, // Norfolk
    {  360, 1384,  402739,  402764,  402797, 25, 33, 34, }, // Noronha
    {  360, 1406,  402831,  402849,  402874, 18, 25, 27, }, // Novosibirsk
    {  360, 1418,  402901,  402912,  402930, 11, 18, 20, }, // Omsk
    {  360, 1428,  402950,  402965,  402988, 15, 23, 24, }, // Pakistan
    {  360, 1437,       0,  403012,       0,  0, 12,  0, }, // Palau
    {  360, 1443,       0,  403024,       0,  0, 29,  0, }, // Papua_New_Guinea
    {  360, 1460,  403053,  403068,  403091, 15, 23, 24, }, // Paraguay
    {  360, 1469,  403115,  403126,  403145, 11, 19, 20, }, // Peru
    {  360, 1474,  403165,  403179,  403201, 14, 22, 23, }, // Philippines
    {  360, 1486,       0,  403224,       0,  0, 24,  0, }, // Phoenix_Islands
    {  360, 1502,  403248,  403272,  403304, 24, 32, 33, }, // Pierre_Miquelon
    {  360, 1518,       0,  403337,       0,  0, 14,  0, }, // Pitcairn
    {  360, 1527,       0,  403351,       0,  0, 13,  0, }, // Ponape
    {  360, 1534,       0,  403364,       0,  0, 16,  0, }, // Pyongyang
    {  360, 1554,       0,  403380,       0,  0, 15,  0, }, // Reunion
    {  360, 1562,       0,  403395,       0,  0, 14,  0, }, // Rothera
    {  360, 1570,  403409,  403424,  403446, 15, 22, 24, }, // Sakhalin
    {  360, 1596,  403470,  403482,  403502, 12, 20, 21, }, // Samoa
    {  360, 1602,       0,  403523,       0,  0, 13,  0, }, // Seychelles
    {  360, 1624,       0,  403536,       0,  0, 23,  0, }, // Singapore
    {  360, 1634,       0,  403559,       0,  0, 23,  0, }, // Solomon
    {  360, 1642,       0,  403582,       0,  0, 22,  0, }, // South_Georgia
    {  360, 1656,       0,  403604,       0,  0, 14,  0, }, // Suriname
    {  360, 1676,       0,  403618,       0,  0, 12,  0, }, // Syowa
    {  360, 1682,       0,  403630,       0,  0, 13,  0, }, // Tahiti
    {  360, 1689,  403643,  403656,  403677, 13, 21, 22, }, // Taipei
    {  360, 1696,       0,  403699,       0,  0, 17,  0, }, // Tajikistan
    {  360, 1724,       0,  403716,       0,  0, 14,  0, }, // Tokelau
    {  360, 1732,  403730,  403742,  403762, 12, 20, 21, }, // Tonga
    {  360, 1738,       0,  403783,       0,  0, 12,  0, }, // Truk
    {  360, 1750,  403795,  403814,  403841, 19, 27, 28, }, // Turkmenistan
    {  360, 1763,       0,  403869,       0,  0, 13,  0, }, // Tuvalu
    {  360, 1777,  403882,  403896,  403918, 14, 22, 23, }, // Uruguay
    {  360, 1792,  403941,  403958,  403983, 17, 25, 26, }, // Uzbekistan
    {  360, 1803,  404009,  404024,  404046, 15, 22, 24, }, // Vanuatu
    {  360, 1811,       0,  404070,       0,  0, 16,  0, }, // Venezuela
    {  360, 1821,  404086,  404104,  404129, 18, 25, 27, }, // Vladivostok
    {  360, 1833,  404156,  404172,  404195, 16, 23, 25, }, // Volgograd
    {  360, 1843,       0,  404220,       0,  0, 13,  0, }, // Vostok
    {  360, 1850,       0,  404233,       0,  0, 22,  0, }, // Wake
    {  360, 1855,       0,  404255,       0,  0, 26,  0, }, // Wallis
    {  360, 1862,  404281,  404295,  404317, 14, 22, 23, }, // Yakutsk
    {  360, 1870,  404340,  404360,  404387, 20, 27, 29, }, // Yekaterinburg
    {  365,    0,  404416,  404424,  404441,  8, 17, 15, }, // Acre Kannada/Kannada/India
    {  365,    5,       0,  404456,       0,  0, 15,  0, }, // Afghanistan
    {  365,   17,       0,  404471,       0,  0, 16,  0, }, // Africa_Central
    {  365,   32,       0,  404487,       0,  0, 17,  0, }, // Africa_Eastern
    {  365,   65,       0,  404504,       0,  0, 27,  0, }, // Africa_Southern
    {  365,   81,  404531,  404549,  404576, 18, 27, 25, }, // Africa_Western
    {  365,  107,  404601,  404612,  404632, 11, 20, 17, }, // Alaska
    {  365,  128,  404649,  404660,  404680, 11, 20, 18, }, // Almaty
    {  365,  135,  404698,  404709,  404729, 11, 20, 18, }, // Amazon
    {  365,  142,  404747,  404771,  404804, 24, 33, 31, }, // America_Central
    {  365,  158,  404835,  404859,  404892, 24, 33, 29, }, // America_Eastern
    {  365,  174,  404921,  404944,  404976, 23, 32, 28, }, // America_Mountain
    {  365,  191,  405004,  405030,  405065, 26, 35, 31, }, // America_Pacific
    {  365,  207,  405096,  405107,  405127, 11, 20, 16, }, // Anadyr
    {  365,  214,  405143,  405152,  405170,  9, 18, 14, }, // Apia
    {  365,  219,  405184,  405193,  405211,  9, 18, 16, }, // Aqtau
    {  365,  225,  405227,  405238,  405258, 11, 20, 18, }, // Aqtobe
    {  365,  232,  405276,  405288,  405309, 12, 21, 17, }, // Arabian
    {  365,  240,  405326,  405340,  405363, 14, 23, 21, }, // Argentina
    {  365,  250,  405384,  405405,  405435, 21, 30, 28, }, // Argentina_Western
    {  365,  268,  405463,  405475,  405496, 12, 21, 19, }, // Armenia
    {  365,  286,  405515,  405529,  405552, 14, 23, 19, }, // Atlantic
    {  365,  295,  405571,  405593,  405625, 22, 32, 28, }, // Australia_Central
    {  365,  313,  405653,  405683,  405722, 30, 39, 35, }, // Australia_CentralWestern
    {  365,  338,  405757,  405778,  405809, 21, 31, 27, }, // Australia_Eastern
    {  365,  356,  405836,  405858,  405890, 22, 32, 28, }, // Australia_Western
    {  365,  374,  405918,  405932,  405955, 14, 23, 21, }, // Azerbaijan
    {  365,  385,  405976,  405986,  406005, 10, 19, 17, }, // Azores
    {  365,  397,  406022,  406036,  406059, 14, 23, 21, }, // Bangladesh
    {  365,  415,       0,  406080,       0,  0, 10,  0, }, // Bhutan
    {  365,  422,       0,  406090,       0,  0, 12,  0, }, // Bolivia
    {  365,  437,  406102,  406117,  406140, 15, 23, 21, }, // Brasilia
    {  365,  454,       0,  406161,       0,  0, 20,  0, }, // Brunei
    {  365,  461,  406181,  406195,  406218, 14, 23, 21, }, // Cape_Verde
    {  365,  478,       0,  406239,       0,  0, 18,  0, }, // Chamorro
    {  365,  487,  406257,  406266,  406284,  9, 18, 14, }, // Chatham
    {  365,  495,  406298,  406306,  406323,  8, 17, 15, }, // Chile
    {  365,  501,  406338,  406346,  406363,  8, 17, 13, }, // China
    {  365,  507,       0,  406376,       0,  0, 19,  0, }, // Christmas
    {  365,  517,       0,  406395,       0,  0, 18,  0, }, // Cocos
    {  365,  523,  406413,  406425,  406446, 12, 21, 19, }, // Colombia
    {  365,  532,  406465,  406481,  406506, 16, 25, 33, }, // Cook
    {  365,  537,  406539,  406549,  406568, 10, 19, 15, }, // Cuba
    {  365,  548,       0,  406583,       0,  0, 10,  0, }, // Davis
    {  365,  564,       0,  406593,       0,  0, 21,  0, }, // DumontDUrville
    {  365,  601,       0,  406614,       0,  0, 16,  0, }, // East_Timor
    {  365,  612,  406630,  406646,  406671, 16, 25, 23, }, // Easter
    {  365,  619,       0,  406694,       0,  0, 13,  0, }, // Ecuador
    {  365,  627,  406707,  406725,  406752, 18, 27, 25, }, // Europe_Central
    {  365,  642,  406777,  406796,  406824, 19, 28, 26, }, // Europe_Eastern
    {  365,  657,       0,  406850,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {  365,  680,  406878,  406898,  406927, 20, 29, 27, }, // Europe_Western
    {  365,  695,  406954,  406978,  407011, 24, 33, 31, }, // Falkland
    {  365,  704,  407042,  407050,  407067,  8, 17, 15, }, // Fiji
    {  365,  709,       0,  407082,       0,  0, 17,  0, }, // French_Guiana
    {  365,  723,       0,  407099,       0,  0, 35,  0, }, // French_Southern
    {  365,  746,       0,  407134,       0,  0, 14,  0, }, // Galapagos
    {  365,  756,       0,  407148,       0,  0, 14,  0, }, // Gambier
    {  365,  764,  407162,  407174,  407195, 12, 21, 19, }, // Georgia
    {  365,  772,       0,  407214,       0,  0, 21,  0, }, // Gilbert_Islands
    {  365,  788,       0,  407235,       0,  0, 25,  0, }, // GMT
    {  365,  830,  407260,  407284,  407317, 24, 33, 31, }, // Greenland_Eastern
    {  365,  848,  407348,  407373,  407407, 25, 34, 32, }, // Greenland_Western
    {  365,  866,       0,  407439,       0,  0, 19,  0, }, // Guam
    {  365,  871,       0,  407458,       0,  0, 18,  0, }, // Gulf
    {  365,  876,       0,  407090,       0,  0,  9,  0, }, // Guyana
    {  365,  883,  407476,  407496,  407525, 20, 29, 25, }, // Hawaii_Aleutian
    {  365,  899,  407550,  407565,  407589, 15, 24, 22, }, // Hong_Kong
    {  365,  909,  407611,  407620,  407638,  9, 18, 16, }, // Hovd
    {  365,  914,       0,  407654,       0,  0, 19,  0, }, // India
    {  365,  920,       0,  407673,       0,  0, 17,  0, }, // Indian_Ocean
    {  365,  933,       0,  407690,       0,  0, 12,  0, }, // Indochina
    {  365,  943,       0,  407702,       0,  0, 19,  0, }, // Indonesia_Central
    {  365,  961,       0,  407721,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  365,  979,       0,  407741,       0,  0, 20,  0, }, // Indonesia_Western
    {  365,  997,  407761,  407770,  407788,  9, 18, 14, }, // Iran
    {  365, 1008,  407802,  407817,  407841, 15, 24, 22, }, // Irkutsk
    {  365, 1016,  407863,  407874,  407894, 11, 20, 16, }, // Israel
    {  365, 1023,  407910,  407919,  407937,  9, 18, 14, }, // Japan
    {  365, 1029,  407951,  407985,  408028, 34, 43, 41, }, // Kamchatka
    {  365, 1047,       0,  408069,       0,  0, 16,  0, }, // Kazakhstan
    {  365, 1058,       0,  408085,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  365, 1077,       0,  408107,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  365, 1106,  408130,  408141,  408161, 11, 20, 16, }, // Korea
    {  365, 1112,       0,  408177,       0,  0,  9,  0, }, // Kosrae
    {  365, 1119,  408186,  408207,  408237, 21, 30, 28, }, // Krasnoyarsk
    {  365, 1151,       0,  408265,       0,  0, 16,  0, }, // Kyrgystan
    {  365, 1161,       0,  408281,       0,  0,  8,  0, }, // Lanka
    {  365, 1175,       0,  408289,       0,  0, 16,  0, }, // Line_Islands
    {  365, 1188,  408305,  408320,  408344, 15, 24, 22, }, // Lord_Howe
    {  365, 1198,  408366,  408375,  408393,  9, 18, 16, }, // Macau
    {  365, 1204,  408409,  408419,  408438, 10, 19, 17, }, // Magadan
    {  365, 1219,       0,  408455,       0,  0, 11,  0, }, // Malaysia
    {  365, 1228,       0,  408466,       0,  0, 14,  0, }, // Maldives
    {  365, 1237,       0,  408480,       0,  0, 15,  0, }, // Marquesas
    {  365, 1247,       0,  408495,       0,  0, 19,  0, }, // Marshall_Islands
    {  365, 1264,  408514,  408525,  408545, 11, 20, 18, }, // Mauritius
    {  365, 1274,       0,  408563,       0,  0, 12,  0, }, // Mawson
    {  365, 1281,  408575,  408597,  408628, 22, 31, 27, }, // Mexico_Pacific
    {  365, 1296,  408655,  408670,  408694, 15, 24, 22, }, // Mongolia
    {  365, 1305,  408716,  408726,  408745, 10, 19, 17, }, // Moscow
    {  365, 1312,       0,  408762,       0,  0, 14,  0, }, // Myanmar
    {  365, 1320,       0,  408776,       0,  0,  8,  0, }, // Nauru
    {  365, 1326,       0,  408784,       0,  0,  9,  0, }, // Nepal
    {  365, 1332,  408793,  408813,  408842, 20, 29, 28, }, // New_Caledonia
    {  365, 1346,  408870,  408887,  408913, 17, 26, 22, }, // New_Zealand
    {  365, 1358,  408935,  408956,  408986, 21, 30, 26, }, // Newfoundland
    {  365, 1371,       0,  409012,       0,  0,  8,  0, }, // Niue
    {  365, 1376,  409020,  409038,  409065, 18, 27, 25, }, // Norfolk
    {  365, 1384,  409090,  409115,  409149, 25, 34, 31, }, // Noronha
    {  365, 1392,       0,  409180,       0,  0, 25,  0, }, // North_Mariana
    {  365, 1406,  409205,  409224,  409252, 19, 28, 26, }, // Novosibirsk
    {  365, 1418,  409278,  409289,  409309, 11, 20, 18, }, // Omsk
    {  365, 1428,  409327,  409340,  409362, 13, 22, 20, }, // Pakistan
    {  365, 1437,       0,  409382,       0,  0, 10,  0, }, // Palau
    {  365, 1443,       0,  409392,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  365, 1460,  409413,  409424,  409444, 11, 20, 18, }, // Paraguay
    {  365, 1469,  409462,  409470,  409487,  8, 17, 15, }, // Peru
    {  365, 1474,  409502,  409514,  409535, 12, 21, 19, }, // Philippines
    {  365, 1486,       0,  409554,       0,  0, 20,  0, }, // Phoenix_Islands
    {  365, 1502,  409574,  409605,  409645, 31, 40, 36, }, // Pierre_Miquelon
    {  365, 1518,       0,  409681,       0,  0, 15,  0, }, // Pitcairn
    {  365, 1527,       0,  409696,       0,  0, 10,  0, }, // Ponape
    {  365, 1534,       0,  409706,       0,  0, 16,  0, }, // Pyongyang
    {  365, 1544,  409722,  409736,  409759, 14, 23, 21, }, // Qyzylorda
    {  365, 1554,       0,  409780,       0,  0, 13,  0, }, // Reunion
    {  365, 1562,       0,  409793,       0,  0, 10,  0, }, // Rothera
    {  365, 1570,  409803,  409819,  409844, 16, 25, 23, }, // Sakhalin
    {  365, 1579,  409867,  409874,  409890,  7, 16, 14, }, // Samara
    {  365, 1596,  409904,  409913,  409931,  9, 18, 16, }, // Samoa
    {  365, 1602,       0,  409947,       0,  0, 12,  0, }, // Seychelles
    {  365, 1624,       0,  409959,       0,  0, 21,  0, }, // Singapore
    {  365, 1634,       0,  409980,       0,  0, 18,  0, }, // Solomon
    {  365, 1642,       0,  409998,       0,  0, 19,  0, }, // South_Georgia
    {  365, 1656,       0,  410017,       0,  0, 12,  0, }, // Suriname
    {  365, 1676,       0,  410029,       0,  0, 10,  0, }, // Syowa
    {  365, 1682,       0,  410039,       0,  0, 10,  0, }, // Tahiti
    {  365, 1689,  410049,  410057,  410074,  8, 17, 13, }, // Taipei
    {  365, 1696,       0,  410087,       0,  0, 14,  0, }, // Tajikistan
    {  365, 1724,       0,  410101,       0,  0, 12,  0, }, // Tokelau
    {  365, 1732,  410113,  410122,  410140,  9, 18, 16, }, // Tonga
    {  365, 1738,       0,  410156,       0,  0,  8,  0, }, // Truk
    {  365, 1750,  410164,  410185,  410215, 21, 30, 28, }, // Turkmenistan
    {  365, 1763,       0,  410243,       0,  0, 10,  0, }, // Tuvalu
    {  365, 1777,  410253,  410264,  410284, 11, 20, 18, }, // Uruguay
    {  365, 1792,  410302,  410319,  410345, 17, 26, 24, }, // Uzbekistan
    {  365, 1803,  410369,  410378,  410396,  9, 18, 16, }, // Vanuatu
    {  365, 1811,       0,  410412,       0,  0, 14,  0, }, // Venezuela
    {  365, 1821,  410426,  410444,  410471, 18, 27, 25, }, // Vladivostok
    {  365, 1833,  410496,  410512,  410537, 16, 25, 23, }, // Volgograd
    {  365, 1843,       0,  410560,       0,  0, 12,  0, }, // Vostok
    {  365, 1850,       0,  410572,       0,  0, 14,  0, }, // Wake
    {  365, 1855,       0,  410586,       0,  0, 27,  0, }, // Wallis
    {  365, 1862,  410613,  410626,  410648, 13, 22, 20, }, // Yakutsk
    {  365, 1870,  410668,  410686,  410713, 18, 27, 26, }, // Yekaterinburg
    {  365, 1892,       0,  410739,       0,  0, 10,  0, }, // Yukon
    {  366,    0,  410749,  410760,  410779, 11, 19, 17, }, // Acre Kashmiri/Arabic/India
    {  366,    5,       0,  410796,       0,  0, 16,  0, }, // Afghanistan
    {  366,   17,       0,  410812,       0,  0, 19,  0, }, // Africa_Central
    {  366,   32,       0,  410831,       0,  0, 19,  0, }, // Africa_Eastern
    {  366,   65,       0,  410850,       0,  0, 19,  0, }, // Africa_Southern
    {  366,   81,  410869,  410888,  410915, 19, 27, 25, }, // Africa_Western
    {  366,  107,  410940,  410953,  410974, 13, 21, 22, }, // Alaska
    {  366,  128,  410996,  411009,  411030, 13, 21, 19, }, // Almaty
    {  366,  135,  411049,  411062,  411083, 13, 21, 19, }, // Amazon
    {  366,  142,  411102,  411113,  411132, 11, 19, 20, }, // America_Central
    {  366,  158,  411152,  411163,  411182, 11, 19, 20, }, // America_Eastern
    {  366,  174,  411202,  411215,  411236, 13, 21, 22, }, // America_Mountain
    {  366,  191,  411258,  411271,  411292, 13, 21, 22, }, // America_Pacific
    {  366,  207,  411314,  411328,  411350, 14, 22, 19, }, // Anadyr
    {  366,  219,  411369,  411381,  411401, 12, 20, 18, }, // Aqtau
    {  366,  225,  411419,  411431,  411451, 12, 20, 18, }, // Aqtobe
    {  366,  232,  411469,  411483,  411505, 14, 22, 23, }, // Arabian
    {  366,  240,  411528,  411544,  411568, 16, 24, 22, }, // Argentina
    {  366,  250,  411590,  411612,  411642, 22, 30, 28, }, // Argentina_Western
    {  366,  268,  411670,  411685,  411708, 15, 23, 21, }, // Armenia
    {  366,  286,  411729,  411744,  411767, 15, 23, 24, }, // Atlantic
    {  366,  295,  411791,  411813,  411843, 22, 30, 31, }, // Australia_Central
    {  366,  313,  411874,  411902,  411938, 28, 36, 37, }, // Australia_CentralWestern
    {  366,  338,  411975,  411997,  412027, 22, 30, 31, }, // Australia_Eastern
    {  366,  356,  412058,  412080,  412111, 22, 31, 32, }, // Australia_Western
    {  366,  374,  412143,  412158,  412181, 15, 23, 21, }, // Azerbaijan
    {  366,  385,  412202,  412215,  412236, 13, 21, 18, }, // Azores
    {  366,  397,  412254,  412269,  412292, 15, 23, 21, }, // Bangladesh
    {  366,  415,       0,  412313,       0,  0, 12,  0, }, // Bhutan
    {  366,  422,       0,  412325,       0,  0, 14,  0, }, // Bolivia
    {  366,  437,  412339,  412354,  412377, 15, 23, 21, }, // Brasilia
    {  366,  454,       0,  412398,       0,  0, 24,  0, }, // Brunei
    {  366,  461,  412422,  412436,  412458, 14, 22, 15, }, // Cape_Verde
    {  366,  478,       0,  412473,       0,  0, 20,  0, }, // Chamorro
    {  366,  487,  412493,  412505,  412525, 12, 20, 18, }, // Chatham
    {  366,  495,  412543,  412553,  412571, 10, 18, 16, }, // Chile
    {  366,  501,  412587,  412598,  412617, 11, 19, 20, }, // China
    {  366,  507,       0,  412637,       0,  0, 12,  0, }, // Christmas
    {  366,  517,       0,  412649,       0,  0, 20,  0, }, // Cocos
    {  366,  523,  412669,  412684,  412707, 15, 23, 21, }, // Colombia
    {  366,  532,  412728,  412746,  412772, 18, 26, 28, }, // Cook
    {  366,  537,  412800,  412812,  412832, 12, 20, 21, }, // Cuba
    {  366,  548,       0,  412853,       0,  0, 11,  0, }, // Davis
    {  366,  564,       0,  412864,       0,  0, 22,  0, }, // DumontDUrville
    {  366,  601,       0,  412886,       0,  0, 18,  0, }, // East_Timor
    {  366,  612,  412904,  412923,  412950, 19, 27, 25, }, // Easter
    {  366,  619,       0,  412975,       0,  0, 14,  0, }, // Ecuador
    {  366,  627,  412989,  413007,  413033, 18, 26, 24, }, // Europe_Central
    {  366,  642,  413057,  413075,  413101, 18, 26, 24, }, // Europe_Eastern
    {  366,  657,       0,  413125,       0,  0, 21,  0, }, // Europe_Further_Eastern
    {  366,  680,  413146,  413165,  413192, 19, 27, 26, }, // Europe_Western
    {  366,  695,  413218,  413238,  413266, 20, 28, 26, }, // Falkland
    {  366,  704,  413292,  413303,  413322, 11, 19, 17, }, // Fiji
    {  366,  709,       0,  413339,       0,  0, 19,  0, }, // French_Guiana
    {  366,  723,       0,  413358,       0,  0, 30,  0, }, // French_Southern
    {  366,  746,       0,  413388,       0,  0, 15,  0, }, // Galapagos
    {  366,  756,       0,  413403,       0,  0, 14,  0, }, // Gambier
    {  366,  764,  413417,  413433,  413457, 16, 24, 22, }, // Georgia
    {  366,  772,       0,  413479,       0,  0, 20,  0, }, // Gilbert_Islands
    {  366,  788,       0,  413499,       0,  0, 20,  0, }, // GMT
    {  366,  830,  413519,  413544,  413577, 25, 33, 31, }, // Greenland_Eastern
    {  366,  848,  413608,  413633,  413666, 25, 33, 31, }, // Greenland_Western
    {  366,  866,       0,  413697,       0,  0, 11,  0, }, // Guam
    {  366,  871,       0,  413708,       0,  0, 18,  0, }, // Gulf
    {  366,  876,       0,  413726,       0,  0, 12,  0, }, // Guyana
    {  366,  883,  413738,  413761,  413792, 23, 31, 29, }, // Hawaii_Aleutian
    {  366,  899,  413821,  413835,  413857, 14, 22, 19, }, // Hong_Kong
    {  366,  909,  413876,  413886,  413904, 10, 18, 16, }, // Hovd
    {  366,  914,       0,  413920,       0,  0,  9,  0, }, // India
    {  366,  920,       0,  413929,       0,  0, 21,  0, }, // Indian_Ocean
    {  366,  933,       0,  413950,       0,  0, 16,  0, }, // Indochina
    {  366,  943,       0,  413966,       0,  0, 23,  0, }, // Indonesia_Central
    {  366,  961,       0,  413989,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  366,  979,       0,  414013,       0,  0, 24,  0, }, // Indonesia_Western
    {  366,  997,  414037,  414051,  414073, 14, 22, 20, }, // Iran
    {  366, 1008,  414093,  414107,  414129, 14, 22, 20, }, // Irkutsk
    {  366, 1016,  414149,  414164,  414187, 15, 23, 24, }, // Israel
    {  366, 1023,  414211,  414223,  414243, 12, 20, 21, }, // Japan
    {  366, 1029,  414264,  414278,  414300, 14, 22, 20, }, // Kamchatka
    {  366, 1058,       0,  414320,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  366, 1077,       0,  414339,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  366, 1106,  414358,  414370,  414390, 12, 20, 21, }, // Korea
    {  366, 1112,       0,  414411,       0,  0, 12,  0, }, // Kosrae
    {  366, 1119,  414423,  414441,  414467, 18, 26, 24, }, // Krasnoyarsk
    {  366, 1151,       0,  414491,       0,  0, 13,  0, }, // Kyrgystan
    {  366, 1161,       0,  414504,       0,  0, 11,  0, }, // Lanka
    {  366, 1175,       0,  414515,       0,  0, 20,  0, }, // Line_Islands
    {  366, 1188,  414535,  414550,  414573, 15, 23, 18, }, // Lord_Howe
    {  366, 1198,  414591,  414604,  414625, 13, 21, 19, }, // Macau
    {  366, 1204,  414644,  414657,  414678, 13, 21, 19, }, // Magadan
    {  366, 1219,       0,  414697,       0,  0, 14,  0, }, // Malaysia
    {  366, 1228,       0,  414711,       0,  0, 15,  0, }, // Maldives
    {  366, 1237,       0,  414726,       0,  0, 16,  0, }, // Marquesas
    {  366, 1247,       0,  414742,       0,  0, 20,  0, }, // Marshall_Islands
    {  366, 1264,  414762,  414775,  414796, 13, 21, 19, }, // Mauritius
    {  366, 1274,       0,  414815,       0,  0, 11,  0, }, // Mawson
    {  366, 1281,  414826,  414845,  414872, 19, 27, 27, }, // Mexico_Pacific
    {  366, 1296,  414899,  414913,  414935, 14, 22, 21, }, // Mongolia
    {  366, 1305,  414956,  414968,  414987, 12, 19, 17, }, // Moscow
    {  366, 1312,       0,  415004,       0,  0, 14,  0, }, // Myanmar
    {  366, 1320,       0,  415018,       0,  0, 14,  0, }, // Nauru
    {  366, 1326,       0,  415032,       0,  0, 12,  0, }, // Nepal
    {  366, 1332,  415044,  415064,  415092, 20, 28, 26, }, // New_Caledonia
    {  366, 1346,  415118,  415133,  415156, 15, 23, 23, }, // New_Zealand
    {  366, 1358,  415179,  415198,  415225, 19, 27, 27, }, // Newfoundland
    {  366, 1371,       0,  415252,       0,  0, 11,  0, }, // Niue
    {  366, 1376,  415263,  415276,  415297, 13, 21, 19, }, // Norfolk
    {  366, 1384,  415316,  415339,  415370, 23, 31, 29, }, // Noronha
    {  366, 1392,       0,  415399,       0,  0, 21,  0, }, // North_Mariana
    {  366, 1406,  415420,  415437,  415462, 17, 25, 23, }, // Novosibirsk
    {  366, 1418,  415485,  415496,  415515, 11, 19, 17, }, // Omsk
    {  366, 1428,  415532,  415546,  415568, 14, 22, 20, }, // Pakistan
    {  366, 1437,       0,  415588,       0,  0, 11,  0, }, // Palau
    {  366, 1443,       0,  415599,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  366, 1460,  415621,  415634,  415655, 13, 21, 19, }, // Paraguay
    {  366, 1469,  415674,  415685,  415704, 11, 19, 17, }, // Peru
    {  366, 1474,  415721,  415737,  415761, 16, 24, 22, }, // Philippines
    {  366, 1486,       0,  415783,       0,  0, 21,  0, }, // Phoenix_Islands
    {  366, 1502,  415804,  415830,  415864, 26, 34, 35, }, // Pierre_Miquelon
    {  366, 1518,       0,  415899,       0,  0, 14,  0, }, // Pitcairn
    {  366, 1527,       0,  415913,       0,  0, 11,  0, }, // Ponape
    {  366, 1534,       0,  415924,       0,  0, 14,  0, }, // Pyongyang
    {  366, 1544,  415938,  415951,  415972, 13, 21, 19, }, // Qyzylorda
    {  366, 1554,       0,  415991,       0,  0, 16,  0, }, // Reunion
    {  366, 1562,       0,  416007,       0,  0, 13,  0, }, // Rothera
    {  366, 1570,  416020,  416034,  416056, 14, 22, 20, }, // Sakhalin
    {  366, 1579,  416076,  416087,  416106, 11, 19, 17, }, // Samara
    {  366, 1596,  416123,  416134,  416153, 11, 19, 17, }, // Samoa
    {  366, 1602,       0,  416170,       0,  0, 13,  0, }, // Seychelles
    {  366, 1624,       0,  416183,       0,  0, 15,  0, }, // Singapore
    {  366, 1634,       0,  416198,       0,  0, 26,  0, }, // Solomon
    {  366, 1642,       0,  416224,       0,  0, 20,  0, }, // South_Georgia
    {  366, 1656,       0,  416244,       0,  0, 13,  0, }, // Suriname
    {  366, 1676,       0,  416257,       0,  0, 11,  0, }, // Syowa
    {  366, 1682,       0,  416268,       0,  0, 12,  0, }, // Tahiti
    {  366, 1689,  416280,  416290,  416308, 10, 18, 18, }, // Taipei
    {  366, 1696,       0,  416326,       0,  0, 13,  0, }, // Tajikistan
    {  366, 1724,       0,  416339,       0,  0, 12,  0, }, // Tokelau
    {  366, 1732,  416351,  416363,  416383, 12, 20, 18, }, // Tonga
    {  366, 1738,       0,  416401,       0,  0, 10,  0, }, // Truk
    {  366, 1750,  416411,  416426,  416449, 15, 23, 24, }, // Turkmenistan
    {  366, 1763,       0,  416473,       0,  0, 14,  0, }, // Tuvalu
    {  366, 1777,  416487,  416502,  416525, 15, 23, 21, }, // Uruguay
    {  366, 1792,  416546,  416563,  416588, 17, 25, 25, }, // Uzbekistan
    {  366, 1803,  416613,  416629,  416653, 16, 24, 22, }, // Vanuatu
    {  366, 1811,       0,  416675,       0,  0, 16,  0, }, // Venezuela
    {  366, 1821,  416691,  416708,  416733, 17, 25, 23, }, // Vladivostok
    {  366, 1833,  416756,  416771,  416794, 15, 23, 21, }, // Volgograd
    {  366, 1843,       0,  416696,       0,  0, 12,  0, }, // Vostok
    {  366, 1850,       0,  416815,       0,  0, 17,  0, }, // Wake
    {  366, 1855,       0,  416832,       0,  0, 26,  0, }, // Wallis
    {  366, 1862,  416858,  416871,  416892, 13, 21, 20, }, // Yakutsk
    {  366, 1870,  416912,  416931,  416958, 19, 27, 24, }, // Yekaterinburg
    {  366, 1892,       0,  416982,       0,  0, 10,  0, }, // Yukon
    {  367,  142,  416992,  417002,  417023, 10, 21, 18, }, // America_Central Kashmiri/Devanagari/India
    {  367,  158,  417041,  417050,  417070,  9, 20, 17, }, // America_Eastern
    {  367,  174,  417087,  417097,  417118, 10, 21, 18, }, // America_Mountain
    {  367,  191,  417136,  417146,  417167, 10, 21, 18, }, // America_Pacific
    {  367,  286,  417185,  417196,  417218, 11, 22, 19, }, // Atlantic
    {  367,  627,  417237,  417252,  417278, 15, 26, 19, }, // Europe_Central
    {  367,  642,  417297,  417312,  417338, 15, 26, 19, }, // Europe_Eastern
    {  367,  680,  417357,  417372,  417398, 15, 26, 19, }, // Europe_Western
    {  367,  788,       0,  417417,       0,  0, 15,  0, }, // GMT
    {  368,    5,       0,  417432,       0,  0, 16,  0, }, // Afghanistan Kazakh/Cyrillic/Kazakhstan
    {  368,   17,       0,  417448,       0,  0, 21,  0, }, // Africa_Central
    {  368,   32,       0,  417469,       0,  0, 19,  0, }, // Africa_Eastern
    {  368,   65,       0,  417488,       0,  0, 33,  0, }, // Africa_Southern
    {  368,   81,  417521,  417540,  417570, 19, 30, 25, }, // Africa_Western
    {  368,  107,  417595,  417608,  417632, 13, 24, 19, }, // Alaska
    {  368,  135,  417651,  417666,  417692, 15, 26, 21, }, // Amazon
    {  368,  142,  417713,  417745,  417788, 32, 43, 38, }, // America_Central
    {  368,  158,  417826,  417856,  417897, 30, 41, 36, }, // America_Eastern
    {  368,  174,  417933,  417961,  418000, 28, 39, 34, }, // America_Mountain
    {  368,  191,  418034,  418071,  418119, 37, 48, 43, }, // America_Pacific
    {  368,  214,  418162,  418173,  418195, 11, 22, 17, }, // Apia
    {  368,  232,  418212,  418232,  418263, 20, 31, 26, }, // Arabian
    {  368,  240,  418289,  418305,  418332, 16, 27, 22, }, // Argentina
    {  368,  250,  418354,  418376,  418409, 22, 33, 28, }, // Argentina_Western
    {  368,  268,  418437,  418451,  418476, 14, 25, 20, }, // Armenia
    {  368,  286,  418496,  418512,  418539, 16, 27, 22, }, // Atlantic
    {  368,  295,  418561,  418585,  418620, 24, 35, 30, }, // Australia_Central
    {  368,  313,  418650,  418680,  418721, 30, 41, 36, }, // Australia_CentralWestern
    {  368,  338,  418757,  418779,  418812, 22, 33, 28, }, // Australia_Eastern
    {  368,  356,  418840,  418862,  418895, 22, 33, 28, }, // Australia_Western
    {  368,  374,  418923,  418940,  418968, 17, 28, 23, }, // Azerbaijan
    {  368,  385,  418991,  419011,  419042, 20, 31, 26, }, // Azores
    {  368,  397,  419068,  419084,  419111, 16, 27, 22, }, // Bangladesh
    {  368,  415,       0,  419133,       0,  0, 12,  0, }, // Bhutan
    {  368,  422,       0,  419145,       0,  0, 14,  0, }, // Bolivia
    {  368,  437,  419159,  419174,  419200, 15, 26, 21, }, // Brasilia
    {  368,  454,       0,  419221,       0,  0, 24,  0, }, // Brunei
    {  368,  461,  419245,  419262,  419290, 17, 28, 23, }, // Cape_Verde
    {  368,  478,       0,  419313,       0,  0, 25,  0, }, // Chamorro
    {  368,  487,  419338,  419350,  419373, 12, 23, 18, }, // Chatham
    {  368,  495,  419391,  419402,  419424, 11, 22, 17, }, // Chile
    {  368,  501,  419441,  419453,  419476, 12, 23, 18, }, // China
    {  368,  507,       0,  419494,       0,  0, 25,  0, }, // Christmas
    {  368,  517,       0,  419519,       0,  0, 24,  0, }, // Cocos
    {  368,  523,  419543,  419558,  419584, 15, 26, 21, }, // Colombia
    {  368,  532,  419605,  419627,  419660, 22, 33, 37, }, // Cook
    {  368,  537,  419697,  419708,  419730, 11, 22, 17, }, // Cuba
    {  368,  548,       0,  419747,       0,  0, 13,  0, }, // Davis
    {  368,  564,       0,  419760,       0,  0, 21,  0, }, // DumontDUrville
    {  368,  601,       0,  419781,       0,  0, 18,  0, }, // East_Timor
    {  368,  612,  419799,  419817,  419846, 18, 29, 24, }, // Easter
    {  368,  619,       0,  419870,       0,  0, 14,  0, }, // Ecuador
    {  368,  627,  419884,  419905,  419937, 21, 32, 27, }, // Europe_Central
    {  368,  642,  419964,  419983,  420013, 19, 30, 25, }, // Europe_Eastern
    {  368,  657,       0,  420038,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  368,  680,  420062,  420081,  420111, 19, 30, 25, }, // Europe_Western
    {  368,  695,  420136,  420160,  420195, 24, 35, 30, }, // Falkland
    {  368,  704,  420225,  420237,  420260, 12, 23, 18, }, // Fiji
    {  368,  709,       0,  420278,       0,  0, 23,  0, }, // French_Guiana
    {  368,  723,       0,  420301,       0,  0, 49,  0, }, // French_Southern
    {  368,  746,       0,  420350,       0,  0, 16,  0, }, // Galapagos
    {  368,  756,       0,  420366,       0,  0, 13,  0, }, // Gambier
    {  368,  764,  420379,  420392,  420416, 13, 24, 19, }, // Georgia
    {  368,  772,       0,  420435,       0,  0, 26,  0, }, // Gilbert_Islands
    {  368,  788,       0,  420461,       0,  0, 14,  0, }, // GMT
    {  368,  830,  420475,  420498,  420532, 23, 34, 29, }, // Greenland_Eastern
    {  368,  848,  420561,  420584,  420618, 23, 34, 29, }, // Greenland_Western
    {  368,  871,       0,  420647,       0,  0, 32,  0, }, // Gulf
    {  368,  876,       0,  420679,       0,  0, 13,  0, }, // Guyana
    {  368,  883,  420692,  420724,  420767, 32, 43, 38, }, // Hawaii_Aleutian
    {  368,  899,  420805,  420819,  420844, 14, 25, 20, }, // Hong_Kong
    {  368,  909,  420864,  420875,  420897, 11, 22, 17, }, // Hovd
    {  368,  914,       0,  420914,       0,  0, 26,  0, }, // India
    {  368,  920,       0,  420940,       0,  0, 18,  0, }, // Indian_Ocean
    {  368,  933,       0,  420958,       0,  0, 16,  0, }, // Indochina
    {  368,  943,       0,  420974,       0,  0, 24,  0, }, // Indonesia_Central
    {  368,  961,       0,  420998,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  368,  979,       0,  421020,       0,  0, 22,  0, }, // Indonesia_Western
    {  368,  997,  421042,  421053,  421075, 11, 22, 17, }, // Iran
    {  368, 1008,  421092,  421106,  421131, 14, 25, 20, }, // Irkutsk
    {  368, 1016,  421151,  421165,  421190, 14, 25, 20, }, // Israel
    {  368, 1023,  421210,  421224,  421249, 14, 25, 20, }, // Japan
    {  368, 1047,       0,  421269,       0,  0, 16,  0, }, // Kazakhstan
    {  368, 1058,       0,  421285,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  368, 1077,       0,  421307,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  368, 1106,  421329,  421341,  421364, 12, 23, 18, }, // Korea
    {  368, 1112,       0,  421382,       0,  0, 13,  0, }, // Kosrae
    {  368, 1119,  421395,  421412,  421440, 17, 28, 23, }, // Krasnoyarsk
    {  368, 1151,       0,  421463,       0,  0, 17,  0, }, // Kyrgystan
    {  368, 1175,       0,  421480,       0,  0, 20,  0, }, // Line_Islands
    {  368, 1188,  421500,  421515,  421541, 15, 26, 21, }, // Lord_Howe
    {  368, 1204,  421562,  421576,  421601, 14, 25, 20, }, // Magadan
    {  368, 1219,       0,  421621,       0,  0, 15,  0, }, // Malaysia
    {  368, 1228,       0,  421636,       0,  0, 23,  0, }, // Maldives
    {  368, 1237,       0,  421659,       0,  0, 22,  0, }, // Marquesas
    {  368, 1247,       0,  421681,       0,  0, 23,  0, }, // Marshall_Islands
    {  368, 1264,  421704,  421719,  421745, 15, 26, 21, }, // Mauritius
    {  368, 1274,       0,  421766,       0,  0, 13,  0, }, // Mawson
    {  368, 1281,  421779,  421805,  421842, 26, 37, 32, }, // Mexico_Pacific
    {  368, 1296,  421874,  421890,  421917, 16, 27, 22, }, // Mongolia
    {  368, 1305,  421939,  421952,  421976, 13, 24, 19, }, // Moscow
    {  368, 1312,       0,  421995,       0,  0, 13,  0, }, // Myanmar
    {  368, 1320,       0,  422008,       0,  0, 12,  0, }, // Nauru
    {  368, 1326,       0,  422020,       0,  0, 12,  0, }, // Nepal
    {  368, 1332,  422032,  422053,  422085, 21, 32, 27, }, // New_Caledonia
    {  368, 1346,  422112,  422132,  422163, 20, 31, 26, }, // New_Zealand
    {  368, 1358,  422189,  422208,  422238, 19, 30, 25, }, // Newfoundland
    {  368, 1371,       0,  422263,       0,  0, 11,  0, }, // Niue
    {  368, 1376,  422274,  422294,  422325, 20, 31, 26, }, // Norfolk
    {  368, 1384,  422351,  422377,  422414, 26, 37, 32, }, // Noronha
    {  368, 1406,  422446,  422462,  422489, 16, 27, 22, }, // Novosibirsk
    {  368, 1418,  422511,  422522,  422544, 11, 22, 17, }, // Omsk
    {  368, 1428,  422561,  422576,  422602, 15, 26, 21, }, // Pakistan
    {  368, 1437,       0,  422623,       0,  0, 12,  0, }, // Palau
    {  368, 1443,       0,  422635,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  368, 1460,  422661,  422676,  422702, 15, 26, 21, }, // Paraguay
    {  368, 1469,  422723,  422734,  422756, 11, 22, 17, }, // Peru
    {  368, 1474,  422773,  422797,  422832, 24, 35, 30, }, // Philippines
    {  368, 1486,       0,  422862,       0,  0, 22,  0, }, // Phoenix_Islands
    {  368, 1502,  422884,  422912,  422951, 28, 39, 34, }, // Pierre_Miquelon
    {  368, 1518,       0,  422985,       0,  0, 14,  0, }, // Pitcairn
    {  368, 1527,       0,  422999,       0,  0, 13,  0, }, // Ponape
    {  368, 1534,       0,  423012,       0,  0, 14,  0, }, // Pyongyang
    {  368, 1554,       0,  423026,       0,  0, 14,  0, }, // Reunion
    {  368, 1562,       0,  423040,       0,  0, 13,  0, }, // Rothera
    {  368, 1570,  423053,  423067,  423092, 14, 25, 20, }, // Sakhalin
    {  368, 1596,  423112,  423124,  423147, 12, 23, 18, }, // Samoa
    {  368, 1602,       0,  423165,       0,  0, 23,  0, }, // Seychelles
    {  368, 1624,       0,  423188,       0,  0, 26,  0, }, // Singapore
    {  368, 1634,       0,  423214,       0,  0, 23,  0, }, // Solomon
    {  368, 1642,       0,  423237,       0,  0, 23,  0, }, // South_Georgia
    {  368, 1656,       0,  423260,       0,  0, 14,  0, }, // Suriname
    {  368, 1676,       0,  423274,       0,  0, 11,  0, }, // Syowa
    {  368, 1682,       0,  423285,       0,  0, 12,  0, }, // Tahiti
    {  368, 1689,  423297,  423310,  423334, 13, 24, 19, }, // Taipei
    {  368, 1696,       0,  423353,       0,  0, 16,  0, }, // Tajikistan
    {  368, 1724,       0,  423369,       0,  0, 14,  0, }, // Tokelau
    {  368, 1732,  423383,  423395,  423418, 12, 23, 18, }, // Tonga
    {  368, 1738,       0,  423436,       0,  0, 11,  0, }, // Truk
    {  368, 1750,  423447,  423466,  423496, 19, 30, 25, }, // Turkmenistan
    {  368, 1763,       0,  423521,       0,  0, 13,  0, }, // Tuvalu
    {  368, 1777,  423534,  423548,  423573, 14, 25, 20, }, // Uruguay
    {  368, 1792,  423593,  423609,  423636, 16, 27, 22, }, // Uzbekistan
    {  368, 1803,  423658,  423672,  423697, 14, 25, 20, }, // Vanuatu
    {  368, 1811,       0,  423717,       0,  0, 16,  0, }, // Venezuela
    {  368, 1821,  423733,  423751,  423780, 18, 29, 24, }, // Vladivostok
    {  368, 1833,  423804,  423820,  423847, 16, 27, 22, }, // Volgograd
    {  368, 1843,       0,  423869,       0,  0, 13,  0, }, // Vostok
    {  368, 1850,       0,  423882,       0,  0, 17,  0, }, // Wake
    {  368, 1855,       0,  423899,       0,  0, 25,  0, }, // Wallis
    {  368, 1862,  423924,  423937,  423961, 13, 24, 19, }, // Yakutsk
    {  368, 1870,  423980,  423999,  424029, 19, 30, 25, }, // Yekaterinburg
    {  368, 1892,       0,  424054,       0,  0, 11,  0, }, // Yukon
    {  369,    5,       0,  424065,       0,  0, 16,  0, }, // Afghanistan Kazakh/Arabic/China
    {  369,   17,       0,  424081,       0,  0, 21,  0, }, // Africa_Central
    {  369,   32,       0,  424102,       0,  0, 19,  0, }, // Africa_Eastern
    {  369,   65,       0,  424121,       0,  0, 33,  0, }, // Africa_Southern
    {  369,   81,  424154,  424173,  424203, 19, 30, 25, }, // Africa_Western
    {  369,  107,  424228,  424241,  424265, 13, 24, 19, }, // Alaska
    {  369,  128,  424284,  424297,  424321, 13, 24, 19, }, // Almaty
    {  369,  135,  424340,  424353,  424377, 13, 24, 19, }, // Amazon
    {  369,  142,  424396,  424428,  424471, 32, 43, 37, }, // America_Central
    {  369,  158,  424508,  424538,  424579, 30, 41, 36, }, // America_Eastern
    {  369,  174,  424615,  424643,  424682, 28, 39, 34, }, // America_Mountain
    {  369,  191,  424716,  424753,  424801, 37, 48, 43, }, // America_Pacific
    {  369,  214,  424844,  424855,  424877, 11, 22, 17, }, // Apia
    {  369,  219,  424894,  424906,  424929, 12, 23, 18, }, // Aqtau
    {  369,  225,  424947,  424960,  424984, 13, 24, 19, }, // Aqtobe
    {  369,  232,  425003,  425023,  425054, 20, 31, 26, }, // Arabian
    {  369,  240,  425080,  425096,  425123, 16, 27, 22, }, // Argentina
    {  369,  250,  425145,  425167,  425200, 22, 33, 28, }, // Argentina_Western
    {  369,  268,  425228,  425242,  425267, 14, 25, 20, }, // Armenia
    {  369,  286,  425287,  425303,  425330, 16, 27, 22, }, // Atlantic
    {  369,  295,  425352,  425376,  425411, 24, 35, 30, }, // Australia_Central
    {  369,  313,  425441,  425471,  425512, 30, 41, 36, }, // Australia_CentralWestern
    {  369,  338,  425548,  425570,  425603, 22, 33, 28, }, // Australia_Eastern
    {  369,  356,  425631,  425653,  425686, 22, 33, 28, }, // Australia_Western
    {  369,  374,  425714,  425732,  425761, 18, 29, 24, }, // Azerbaijan
    {  369,  385,  425785,  425805,  425836, 20, 31, 26, }, // Azores
    {  369,  397,  425862,  425878,  425905, 16, 27, 22, }, // Bangladesh
    {  369,  415,       0,  425927,       0,  0, 12,  0, }, // Bhutan
    {  369,  422,       0,  425939,       0,  0, 14,  0, }, // Bolivia
    {  369,  437,  425953,  425968,  425994, 15, 26, 21, }, // Brasilia
    {  369,  454,       0,  426015,       0,  0, 24,  0, }, // Brunei
    {  369,  461,  426039,  426056,  426084, 17, 28, 23, }, // Cape_Verde
    {  369,  478,       0,  426107,       0,  0, 25,  0, }, // Chamorro
    {  369,  487,  426132,  426144,  426167, 12, 23, 18, }, // Chatham
    {  369,  495,  426185,  426196,  426218, 11, 22, 17, }, // Chile
    {  369,  501,  426235,  426247,  426270, 12, 23, 18, }, // China
    {  369,  507,       0,  426288,       0,  0, 24,  0, }, // Christmas
    {  369,  517,       0,  426312,       0,  0, 24,  0, }, // Cocos
    {  369,  523,  426336,  426351,  426377, 15, 26, 21, }, // Colombia
    {  369,  532,  426398,  426420,  426453, 22, 33, 28, }, // Cook
    {  369,  537,  426481,  426492,  426514, 11, 22, 17, }, // Cuba
    {  369,  548,       0,  426531,       0,  0, 13,  0, }, // Davis
    {  369,  564,       0,  426544,       0,  0, 21,  0, }, // DumontDUrville
    {  369,  601,       0,  426565,       0,  0, 18,  0, }, // East_Timor
    {  369,  612,  426583,  426601,  426630, 18, 29, 24, }, // Easter
    {  369,  619,       0,  426654,       0,  0, 14,  0, }, // Ecuador
    {  369,  627,  426668,  426689,  426721, 21, 32, 27, }, // Europe_Central
    {  369,  642,  426748,  426767,  426797, 19, 30, 25, }, // Europe_Eastern
    {  369,  657,       0,  426822,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  369,  680,  426846,  426865,  426895, 19, 30, 25, }, // Europe_Western
    {  369,  695,  426920,  426944,  426979, 24, 35, 30, }, // Falkland
    {  369,  704,  427009,  427020,  427042, 11, 22, 17, }, // Fiji
    {  369,  709,       0,  427059,       0,  0, 24,  0, }, // French_Guiana
    {  369,  723,       0,  427083,       0,  0, 50,  0, }, // French_Southern
    {  369,  746,       0,  427133,       0,  0, 16,  0, }, // Galapagos
    {  369,  756,       0,  427149,       0,  0, 12,  0, }, // Gambier
    {  369,  764,  427161,  427174,  427198, 13, 24, 19, }, // Georgia
    {  369,  772,       0,  427217,       0,  0, 26,  0, }, // Gilbert_Islands
    {  369,  788,       0,  427243,       0,  0, 14,  0, }, // GMT
    {  369,  830,  427257,  427280,  427314, 23, 34, 29, }, // Greenland_Eastern
    {  369,  848,  427343,  427366,  427400, 23, 34, 29, }, // Greenland_Western
    {  369,  871,       0,  427429,       0,  0, 32,  0, }, // Gulf
    {  369,  876,       0,  427461,       0,  0, 13,  0, }, // Guyana
    {  369,  883,  427474,  427506,  427549, 32, 43, 38, }, // Hawaii_Aleutian
    {  369,  899,  427587,  427600,  427624, 13, 24, 19, }, // Hong_Kong
    {  369,  909,  427643,  427654,  427676, 11, 22, 17, }, // Hovd
    {  369,  914,       0,  427693,       0,  0, 27,  0, }, // India
    {  369,  920,       0,  427720,       0,  0, 19,  0, }, // Indian_Ocean
    {  369,  933,       0,  427739,       0,  0, 18,  0, }, // Indochina
    {  369,  943,       0,  427757,       0,  0, 24,  0, }, // Indonesia_Central
    {  369,  961,       0,  427781,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  369,  979,       0,  427803,       0,  0, 22,  0, }, // Indonesia_Western
    {  369,  997,  427825,  427836,  427858, 11, 22, 17, }, // Iran
    {  369, 1008,  427875,  427889,  427914, 14, 25, 20, }, // Irkutsk
    {  369, 1016,  427934,  427947,  427971, 13, 24, 19, }, // Israel
    {  369, 1023,  427990,  428004,  428029, 14, 25, 20, }, // Japan
    {  369, 1047,       0,  428049,       0,  0, 16,  0, }, // Kazakhstan
    {  369, 1058,       0,  428065,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  369, 1077,       0,  428087,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  369, 1106,  428109,  428122,  428146, 13, 24, 19, }, // Korea
    {  369, 1112,       0,  428165,       0,  0, 13,  0, }, // Kosrae
    {  369, 1119,  428178,  428196,  428225, 18, 29, 24, }, // Krasnoyarsk
    {  369, 1151,       0,  428249,       0,  0, 17,  0, }, // Kyrgystan
    {  369, 1175,       0,  428266,       0,  0, 20,  0, }, // Line_Islands
    {  369, 1188,  428286,  428301,  428327, 15, 26, 21, }, // Lord_Howe
    {  369, 1204,  428348,  428362,  428387, 14, 25, 20, }, // Magadan
    {  369, 1219,       0,  428407,       0,  0, 15,  0, }, // Malaysia
    {  369, 1228,       0,  428422,       0,  0, 22,  0, }, // Maldives
    {  369, 1237,       0,  428444,       0,  0, 22,  0, }, // Marquesas
    {  369, 1247,       0,  428466,       0,  0, 22,  0, }, // Marshall_Islands
    {  369, 1264,  428488,  428503,  428529, 15, 26, 21, }, // Mauritius
    {  369, 1274,       0,  428550,       0,  0, 13,  0, }, // Mawson
    {  369, 1281,  428563,  428589,  428626, 26, 37, 32, }, // Mexico_Pacific
    {  369, 1296,  428658,  428674,  428701, 16, 27, 22, }, // Mongolia
    {  369, 1305,  428723,  428736,  428760, 13, 24, 19, }, // Moscow
    {  369, 1312,       0,  428779,       0,  0, 13,  0, }, // Myanmar
    {  369, 1320,       0,  428792,       0,  0, 12,  0, }, // Nauru
    {  369, 1326,       0,  428804,       0,  0, 12,  0, }, // Nepal
    {  369, 1332,  428816,  428837,  428869, 21, 32, 27, }, // New_Caledonia
    {  369, 1346,  428896,  428916,  428947, 20, 31, 26, }, // New_Zealand
    {  369, 1358,  428973,  428992,  429022, 19, 30, 25, }, // Newfoundland
    {  369, 1371,       0,  429047,       0,  0, 11,  0, }, // Niue
    {  369, 1376,  429058,  429078,  429109, 20, 31, 26, }, // Norfolk
    {  369, 1384,  429135,  429161,  429198, 26, 37, 32, }, // Noronha
    {  369, 1406,  429230,  429246,  429273, 16, 27, 22, }, // Novosibirsk
    {  369, 1418,  429295,  429306,  429328, 11, 22, 17, }, // Omsk
    {  369, 1428,  429345,  429360,  429386, 15, 26, 21, }, // Pakistan
    {  369, 1437,       0,  429407,       0,  0, 12,  0, }, // Palau
    {  369, 1443,       0,  429419,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  369, 1460,  429446,  429461,  429487, 15, 26, 21, }, // Paraguay
    {  369, 1469,  429508,  429519,  429541, 11, 22, 17, }, // Peru
    {  369, 1474,  429558,  429581,  429615, 23, 34, 29, }, // Philippines
    {  369, 1486,       0,  429644,       0,  0, 22,  0, }, // Phoenix_Islands
    {  369, 1502,  429666,  429693,  429731, 27, 38, 33, }, // Pierre_Miquelon
    {  369, 1518,       0,  429764,       0,  0, 14,  0, }, // Pitcairn
    {  369, 1527,       0,  429778,       0,  0, 13,  0, }, // Ponape
    {  369, 1534,       0,  429791,       0,  0, 14,  0, }, // Pyongyang
    {  369, 1544,  429805,  429821,  429848, 16, 27, 22, }, // Qyzylorda
    {  369, 1554,       0,  429870,       0,  0, 14,  0, }, // Reunion
    {  369, 1562,       0,  429884,       0,  0, 12,  0, }, // Rothera
    {  369, 1570,  429896,  429910,  429935, 14, 25, 20, }, // Sakhalin
    {  369, 1596,  429955,  429967,  429990, 12, 23, 18, }, // Samoa
    {  369, 1602,       0,  430008,       0,  0, 22,  0, }, // Seychelles
    {  369, 1624,       0,  430030,       0,  0, 26,  0, }, // Singapore
    {  369, 1634,       0,  430056,       0,  0, 23,  0, }, // Solomon
    {  369, 1642,       0,  430079,       0,  0, 23,  0, }, // South_Georgia
    {  369, 1656,       0,  430102,       0,  0, 14,  0, }, // Suriname
    {  369, 1676,       0,  430116,       0,  0, 11,  0, }, // Syowa
    {  369, 1682,       0,  430127,       0,  0, 12,  0, }, // Tahiti
    {  369, 1689,  430139,  430152,  430176, 13, 24, 19, }, // Taipei
    {  369, 1696,       0,  430195,       0,  0, 16,  0, }, // Tajikistan
    {  369, 1724,       0,  430211,       0,  0, 14,  0, }, // Tokelau
    {  369, 1732,  430225,  430237,  430260, 12, 23, 18, }, // Tonga
    {  369, 1738,       0,  430278,       0,  0, 11,  0, }, // Truk
    {  369, 1750,  430289,  430308,  430338, 19, 30, 25, }, // Turkmenistan
    {  369, 1763,       0,  430363,       0,  0, 13,  0, }, // Tuvalu
    {  369, 1777,  430376,  430390,  430415, 14, 25, 20, }, // Uruguay
    {  369, 1792,  430435,  430451,  430478, 16, 27, 22, }, // Uzbekistan
    {  369, 1803,  430500,  430514,  430539, 14, 25, 20, }, // Vanuatu
    {  369, 1811,       0,  430559,       0,  0, 16,  0, }, // Venezuela
    {  369, 1821,  430575,  430593,  430622, 18, 29, 24, }, // Vladivostok
    {  369, 1833,  430646,  430662,  430689, 16, 27, 22, }, // Volgograd
    {  369, 1843,       0,  430580,       0,  0, 13,  0, }, // Vostok
    {  369, 1850,       0,  430711,       0,  0, 17,  0, }, // Wake
    {  369, 1855,       0,  430728,       0,  0, 25,  0, }, // Wallis
    {  369, 1862,  430753,  430767,  430792, 14, 25, 20, }, // Yakutsk
    {  369, 1870,  430812,  430831,  430861, 19, 30, 25, }, // Yekaterinburg
    {  369, 1892,       0,  430886,       0,  0, 12,  0, }, // Yukon
    {  371,    5,       0,  430898,       0,  0, 24,  0, }, // Afghanistan Khmer/Khmer/Cambodia
    {  371,   17,       0,  430922,       0,  0, 24,  0, }, // Africa_Central
    {  371,   32,       0,  430946,       0,  0, 25,  0, }, // Africa_Eastern
    {  371,   65,       0,  430971,       0,  0, 27,  0, }, // Africa_Southern
    {  371,   81,  430998,  431023,  431057, 25, 34, 39, }, // Africa_Western
    {  371,  107,  431096,  431112,  431137, 16, 25, 26, }, // Alaska
    {  371,  135,  431163,  431181,  431207, 18, 26, 28, }, // Amazon
    {  371,  142,  431235,  431273,  431319, 38, 46, 45, }, // America_Central
    {  371,  158,  431364,  431396,  431437, 32, 41, 40, }, // America_Eastern
    {  371,  174,  431477,  431518,  431567, 41, 49, 48, }, // America_Mountain
    {  371,  191,  431615,  431639,  431671, 24, 32, 31, }, // America_Pacific
    {  371,  214,  431702,  431716,  431738, 14, 22, 23, }, // Apia
    {  371,  232,  431761,  431776,  431800, 15, 24, 24, }, // Arabian
    {  371,  240,  431824,  431842,  431868, 18, 26, 28, }, // Argentina
    {  371,  250,  431896,  431923,  431958, 27, 35, 37, }, // Argentina_Western
    {  371,  268,  431995,  432009,  432032, 14, 23, 26, }, // Armenia
    {  371,  286,  432058,  432076,  432103, 18, 27, 27, }, // Atlantic
    {  371,  295,  432130,  432154,  432188, 24, 34, 36, }, // Australia_Central
    {  371,  313,  432224,  432266,  432315, 42, 49, 50, }, // Australia_CentralWestern
    {  371,  338,  432365,  432391,  432426, 26, 35, 35, }, // Australia_Eastern
    {  371,  356,  432461,  432489,  432525, 28, 36, 35, }, // Australia_Western
    {  371,  374,  432560,  432580,  432609, 20, 29, 32, }, // Azerbaijan
    {  371,  385,  432641,  432655,  432678, 14, 23, 26, }, // Azores
    {  371,  397,  432704,  432722,  432749, 18, 27, 30, }, // Bangladesh
    {  371,  415,       0,  432779,       0,  0, 12,  0, }, // Bhutan
    {  371,  422,       0,  432791,       0,  0, 14,  0, }, // Bolivia
    {  371,  437,  432805,  432825,  432854, 20, 29, 33, }, // Brasilia
    {  371,  454,       0,  432887,       0,  0, 22,  0, }, // Brunei
    {  371,  461,  432909,  432924,  432948, 15, 24, 27, }, // Cape_Verde
    {  371,  478,       0,  432975,       0,  0, 25,  0, }, // Chamorro
    {  371,  487,  433000,  433013,  433035, 13, 22, 22, }, // Chatham
    {  371,  495,  433057,  433068,  433087, 11, 19, 21, }, // Chile
    {  371,  501,  433108,  433119,  433139, 11, 20, 20, }, // China
    {  371,  507,       0,  433159,       0,  0, 21,  0, }, // Christmas
    {  371,  517,       0,  433180,       0,  0, 23,  0, }, // Cocos
    {  371,  523,  433203,  433219,  433244, 16, 25, 28, }, // Colombia
    {  371,  532,  433272,  433293,  433323, 21, 30, 45, }, // Cook
    {  371,  537,  433368,  433381,  433403, 13, 22, 22, }, // Cuba
    {  371,  548,       0,  433425,       0,  0, 13,  0, }, // Davis
    {  371,  564,       0,  433438,       0,  0, 24,  0, }, // DumontDUrville
    {  371,  601,       0,  433462,       0,  0, 22,  0, }, // East_Timor
    {  371,  612,  433484,  433501,  433526, 17, 25, 27, }, // Easter
    {  371,  619,       0,  433553,       0,  0, 17,  0, }, // Ecuador
    {  371,  627,  433570,  433591,  433621, 21, 30, 33, }, // Europe_Central
    {  371,  642,  433654,  433678,  433712, 24, 34, 36, }, // Europe_Eastern
    {  371,  657,       0,  433748,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  371,  680,  433779,  433801,  433832, 22, 31, 34, }, // Europe_Western
    {  371,  695,  433866,  433892,  433927, 26, 35, 38, }, // Falkland
    {  371,  704,  433965,  433979,  434002, 14, 23, 28, }, // Fiji
    {  371,  709,       0,  434030,       0,  0, 23,  0, }, // French_Guiana
    {  371,  723,       0,  434053,       0,  0, 36,  0, }, // French_Southern
    {  371,  746,       0,  434089,       0,  0, 18,  0, }, // Galapagos
    {  371,  756,       0,  434107,       0,  0, 13,  0, }, // Gambier
    {  371,  764,  434120,  434137,  434163, 17, 26, 30, }, // Georgia
    {  371,  772,       0,  434193,       0,  0, 18,  0, }, // Gilbert_Islands
    {  371,  788,       0,  434211,       0,  0, 14,  0, }, // GMT
    {  371,  830,  434225,  434252,  434290, 27, 38, 37, }, // Greenland_Eastern
    {  371,  848,  434327,  434349,  434379, 22, 30, 32, }, // Greenland_Western
    {  371,  871,       0,  434411,       0,  0, 20,  0, }, // Gulf
    {  371,  876,       0,  434030,       0,  0, 16,  0, }, // Guyana
    {  371,  883,  434431,  434456,  434490, 25, 34, 33, }, // Hawaii_Aleutian
    {  371,  899,  434523,  434537,  434560, 14, 23, 26, }, // Hong_Kong
    {  371,  909,  434586,  434597,  434616, 11, 19, 23, }, // Hovd
    {  371,  914,       0,  434639,       0,  0, 21,  0, }, // India
    {  371,  920,       0,  434660,       0,  0, 23,  0, }, // Indian_Ocean
    {  371,  933,       0,  434683,       0,  0, 16,  0, }, // Indochina
    {  371,  943,       0,  434699,       0,  0, 27,  0, }, // Indonesia_Central
    {  371,  961,       0,  434726,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  371,  979,       0,  434753,       0,  0, 27,  0, }, // Indonesia_Western
    {  371,  997,  434780,  434795,  434819, 15, 24, 24, }, // Iran
    {  371, 1008,  434843,  434860,  434885, 17, 25, 27, }, // Irkutsk
    {  371, 1016,  434912,  434930,  434957, 18, 27, 27, }, // Israel
    {  371, 1023,  434984,  434997,  435019, 13, 22, 21, }, // Japan
    {  371, 1047,       0,  435040,       0,  0, 22,  0, }, // Kazakhstan
    {  371, 1058,       0,  435062,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  371, 1077,       0,  435089,       0,  0, 31,  0, }, // Kazakhstan_Western
    {  371, 1106,  435120,  435133,  435155, 13, 22, 22, }, // Korea
    {  371, 1112,       0,  435177,       0,  0, 14,  0, }, // Kosrae
    {  371, 1119,  435191,  435209,  435236, 18, 27, 30, }, // Krasnoyarsk
    {  371, 1151,       0,  435266,       0,  0, 21,  0, }, // Kyrgystan
    {  371, 1175,       0,  435287,       0,  0, 15,  0, }, // Line_Islands
    {  371, 1188,  435302,  435314,  435336, 12, 22, 21, }, // Lord_Howe
    {  371, 1204,  435357,  435375,  435402, 18, 27, 30, }, // Magadan
    {  371, 1219,       0,  435432,       0,  0, 16,  0, }, // Malaysia
    {  371, 1228,       0,  435448,       0,  0, 16,  0, }, // Maldives
    {  371, 1237,       0,  435464,       0,  0, 21,  0, }, // Marquesas
    {  371, 1247,       0,  435485,       0,  0, 14,  0, }, // Marshall_Islands
    {  371, 1264,  435499,  435513,  435536, 14, 23, 24, }, // Mauritius
    {  371, 1274,       0,  435560,       0,  0, 16,  0, }, // Mawson
    {  371, 1281,  435576,  435604,  435640, 28, 36, 37, }, // Mexico_Pacific
    {  371, 1296,  435677,  435696,  435724, 19, 28, 31, }, // Mongolia
    {  371, 1305,  435755,  435769,  435792, 14, 23, 26, }, // Moscow
    {  371, 1312,       0,  435818,       0,  0, 18,  0, }, // Myanmar
    {  371, 1320,       0,  435836,       0,  0, 12,  0, }, // Nauru
    {  371, 1326,       0,  435848,       0,  0, 13,  0, }, // Nepal
    {  371, 1332,  435861,  435882,  435911, 21, 29, 31, }, // New_Caledonia
    {  371, 1346,  435942,  435960,  435987, 18, 27, 27, }, // New_Zealand
    {  371, 1358,  436014,  436033,  436063, 19, 30, 27, }, // Newfoundland
    {  371, 1371,       0,  436090,       0,  0, 12,  0, }, // Niue
    {  371, 1376,  436102,  436122,  436147, 20, 25, 27, }, // Norfolk
    {  371, 1384,  436174,  436203,  436240, 29, 37, 39, }, // Noronha
    {  371, 1406,  436279,  436299,  436328, 20, 29, 32, }, // Novosibirsk
    {  371, 1418,  436360,  436371,  436391, 11, 20, 23, }, // Omsk
    {  371, 1428,  436414,  436432,  436459, 18, 27, 30, }, // Pakistan
    {  371, 1437,       0,  436489,       0,  0, 12,  0, }, // Palau
    {  371, 1443,       0,  436501,       0,  0, 29,  0, }, // Papua_New_Guinea
    {  371, 1460,  436530,  436549,  436577, 19, 28, 30, }, // Paraguay
    {  371, 1469,  436607,  436620,  436642, 13, 22, 25, }, // Peru
    {  371, 1474,  436667,  436684,  436710, 17, 26, 30, }, // Philippines
    {  371, 1486,       0,  436740,       0,  0, 17,  0, }, // Phoenix_Islands
    {  371, 1502,  436757,  436788,  436826, 31, 38, 38, }, // Pierre_Miquelon
    {  371, 1518,       0,  436864,       0,  0, 13,  0, }, // Pitcairn
    {  371, 1527,       0,  436877,       0,  0, 15,  0, }, // Ponape
    {  371, 1534,       0,  436892,       0,  0, 15,  0, }, // Pyongyang
    {  371, 1554,       0,  436907,       0,  0, 15,  0, }, // Reunion
    {  371, 1562,       0,  436922,       0,  0, 16,  0, }, // Rothera
    {  371, 1570,  436938,  436955,  436981, 17, 26, 29, }, // Sakhalin
    {  371, 1596,  437010,  437023,  437044, 13, 21, 23, }, // Samoa
    {  371, 1602,       0,  437067,       0,  0, 15,  0, }, // Seychelles
    {  371, 1624,       0,  437082,       0,  0, 17,  0, }, // Singapore
    {  371, 1634,       0,  437099,       0,  0, 20,  0, }, // Solomon
    {  371, 1642,       0,  437119,       0,  0, 21,  0, }, // South_Georgia
    {  371, 1656,       0,  437140,       0,  0, 15,  0, }, // Suriname
    {  371, 1676,       0,  437155,       0,  0, 16,  0, }, // Syowa
    {  371, 1682,       0,  437171,       0,  0, 14,  0, }, // Tahiti
    {  371, 1689,  437185,  437198,  437220, 13, 22, 22, }, // Taipei
    {  371, 1696,       0,  437242,       0,  0, 17,  0, }, // Tajikistan
    {  371, 1724,       0,  437259,       0,  0, 14,  0, }, // Tokelau
    {  371, 1732,  437273,  437288,  437313, 15, 25, 27, }, // Tonga
    {  371, 1738,       0,  437340,       0,  0, 13,  0, }, // Truk
    {  371, 1750,  437353,  437374,  437403, 21, 29, 32, }, // Turkmenistan
    {  371, 1763,       0,  437435,       0,  0, 16,  0, }, // Tuvalu
    {  371, 1777,  437451,  437470,  437498, 19, 28, 32, }, // Uruguay
    {  371, 1792,  437530,  437551,  437581, 21, 30, 33, }, // Uzbekistan
    {  371, 1803,  437614,  437629,  437654, 15, 25, 27, }, // Vanuatu
    {  371, 1811,       0,  437681,       0,  0, 21,  0, }, // Venezuela
    {  371, 1821,  437702,  437725,  437757, 23, 32, 35, }, // Vladivostok
    {  371, 1833,  437792,  437812,  437841, 20, 29, 32, }, // Volgograd
    {  371, 1843,       0,  437873,       0,  0, 16,  0, }, // Vostok
    {  371, 1850,       0,  437889,       0,  0, 14,  0, }, // Wake
    {  371, 1855,       0,  437903,       0,  0, 27,  0, }, // Wallis
    {  371, 1862,  437930,  437949,  437977, 19, 28, 31, }, // Yakutsk
    {  371, 1870,  438008,  438030,  438061, 22, 31, 35, }, // Yekaterinburg
    {  371, 1892,       0,  438096,       0,  0, 11,  0, }, // Yukon
    {  374,  788,       0,    2800,       0,  0, 19,  0, }, // GMT Kinyarwanda/Latin/Rwanda
    {  375,    5,       0,  438107,       0,  0, 15,  0, }, // Afghanistan Konkani/Devanagari/India
    {  375,   17,       0,  438122,       0,  0, 16,  0, }, // Africa_Central
    {  375,   32,       0,  438138,       0,  0, 17,  0, }, // Africa_Eastern
    {  375,   65,       0,  438155,       0,  0, 27,  0, }, // Africa_Southern
    {  375,   81,  438182,  438200,  438227, 18, 27, 26, }, // Africa_Western
    {  375,  107,  438253,  438264,  438284, 11, 20, 18, }, // Alaska
    {  375,  135,  438302,  438312,  438331, 10, 19, 18, }, // Amazon
    {  375,  142,  438349,  438357,  438374,  8, 17, 15, }, // America_Central
    {  375,  158,  438389,  438398,  438416,  9, 18, 16, }, // America_Eastern
    {  375,  174,  438432,  438443,  438463, 11, 20, 18, }, // America_Mountain
    {  375,  191,  438481,  438492,  438512, 11, 20, 18, }, // America_Pacific
    {  375,  214,  438530,  438539,  438557,  9, 18, 16, }, // Apia
    {  375,  232,  438573,  438581,  438598,  8, 17, 15, }, // Arabian
    {  375,  240,  438613,  438627,  438650, 14, 23, 22, }, // Argentina
    {  375,  250,  438672,  438693,  438723, 21, 30, 29, }, // Argentina_Western
    {  375,  268,  438752,  438765,  438787, 13, 22, 21, }, // Armenia
    {  375,  286,  438808,  438820,  438841, 12, 21, 19, }, // Atlantic
    {  375,  295,  438860,  438880,  438909, 20, 29, 27, }, // Australia_Central
    {  375,  313,  438936,  438963,  438999, 27, 36, 34, }, // Australia_CentralWestern
    {  375,  338,  439033,  439054,  439084, 21, 30, 28, }, // Australia_Eastern
    {  375,  356,  439112,  439134,  439165, 22, 31, 29, }, // Australia_Western
    {  375,  374,  439194,  439206,  439227, 12, 21, 20, }, // Azerbaijan
    {  375,  385,  439247,  439253,  439272,  6, 19, 18, }, // Azores
    {  375,  397,  439290,  439303,  439325, 13, 22, 21, }, // Bangladesh
    {  375,  415,       0,  439346,       0,  0,  5,  0, }, // Bhutan
    {  375,  422,       0,  439351,       0,  0, 14,  0, }, // Bolivia
    {  375,  437,  439365,  439379,  439402, 14, 23, 22, }, // Brasilia
    {  375,  454,       0,  439424,       0,  0, 22,  0, }, // Brunei
    {  375,  461,  439446,  439459,  439481, 13, 22, 21, }, // Cape_Verde
    {  375,  478,       0,  439502,       0,  0, 19,  0, }, // Chamorro
    {  375,  487,  439521,  439529,  439546,  8, 17, 15, }, // Chatham
    {  375,  495,  439561,  439569,  439586,  8, 17, 16, }, // Chile
    {  375,  501,  439602,  439609,  439625,  7, 16, 14, }, // China
    {  375,  507,       0,  439639,       0,  0, 17,  0, }, // Christmas
    {  375,  517,       0,  439656,       0,  0, 15,  0, }, // Cocos
    {  375,  523,  439671,  439683,  439704, 12, 21, 20, }, // Colombia
    {  375,  532,  439724,  439737,  439759, 13, 22, 26, }, // Cook
    {  375,  537,  439785,  439795,  439814, 10, 19, 17, }, // Cuba
    {  375,  548,       0,  439831,       0,  0, 11,  0, }, // Davis
    {  375,  564,       0,  439842,       0,  0, 18,  0, }, // DumontDUrville
    {  375,  601,       0,  439860,       0,  0, 15,  0, }, // East_Timor
    {  375,  612,  439875,  439890,  439914, 15, 24, 23, }, // Easter
    {  375,  619,       0,  439937,       0,  0, 12,  0, }, // Ecuador
    {  375,  627,  439949,  439966,  439992, 17, 26, 25, }, // Europe_Central
    {  375,  642,  440017,  440035,  440062, 18, 27, 26, }, // Europe_Eastern
    {  375,  657,       0,  440088,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  375,  680,  440111,  440130,  440158, 19, 28, 27, }, // Europe_Western
    {  375,  695,  440185,  440203,  440230, 18, 27, 26, }, // Falkland
    {  375,  704,  440256,  440264,  440281,  8, 17, 16, }, // Fiji
    {  375,  709,       0,  440297,       0,  0, 16,  0, }, // French_Guiana
    {  375,  723,       0,  440313,       0,  0, 34,  0, }, // French_Southern
    {  375,  746,       0,  440347,       0,  0, 13,  0, }, // Galapagos
    {  375,  756,       0,  440360,       0,  0, 12,  0, }, // Gambier
    {  375,  764,  440372,  440384,  440405, 12, 21, 20, }, // Georgia
    {  375,  772,       0,  440425,       0,  0, 18,  0, }, // Gilbert_Islands
    {  375,  788,       0,  440443,       0,  0, 17,  0, }, // GMT
    {  375,  830,  440460,  440478,  440505, 18, 27, 26, }, // Greenland_Eastern
    {  375,  848,  440531,  440550,  440578, 19, 28, 27, }, // Greenland_Western
    {  375,  871,       0,  440605,       0,  0, 17,  0, }, // Gulf
    {  375,  876,       0,  440622,       0,  0, 10,  0, }, // Guyana
    {  375,  883,  440632,  440649,  440675, 17, 26, 24, }, // Hawaii_Aleutian
    {  375,  899,  440699,  440712,  440734, 13, 22, 21, }, // Hong_Kong
    {  375,  909,  440755,  440761,  440780,  6, 19, 18, }, // Hovd
    {  375,  914,       0,  440798,       0,  0, 19,  0, }, // India
    {  375,  920,       0,  440817,       0,  0, 17,  0, }, // Indian_Ocean
    {  375,  933,       0,  440834,       0,  0, 11,  0, }, // Indochina
    {  375,  943,       0,  440845,       0,  0, 19,  0, }, // Indonesia_Central
    {  375,  961,       0,  440864,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  375,  979,       0,  440884,       0,  0, 21,  0, }, // Indonesia_Western
    {  375,  997,   94442,  440905,  440922,  4, 17, 15, }, // Iran
    {  375, 1008,  440937,  440950,  440972, 13, 22, 21, }, // Irkutsk
    {  375, 1016,  440993,  441004,  441024, 11, 20, 18, }, // Israel
    {  375, 1023,  441042,  441050,  441067,  8, 17, 15, }, // Japan
    {  375, 1047,       0,  441082,       0,  0, 13,  0, }, // Kazakhstan
    {  375, 1058,       0,  441095,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  375, 1077,       0,  441114,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  375, 1106,  441134,  441144,  441163, 10, 19, 17, }, // Korea
    {  375, 1112,       0,  441180,       0,  0,  9,  0, }, // Kosrae
    {  375, 1119,  441189,  441208,  441236, 19, 28, 27, }, // Krasnoyarsk
    {  375, 1151,       0,  441263,       0,  0, 17,  0, }, // Kyrgystan
    {  375, 1175,       0,  441280,       0,  0, 14,  0, }, // Line_Islands
    {  375, 1188,  441294,  441308,  441331, 14, 23, 21, }, // Lord_Howe
    {  375, 1204,  441352,  441362,  441381, 10, 19, 18, }, // Magadan
    {  375, 1219,       0,  441399,       0,  0, 11,  0, }, // Malaysia
    {  375, 1228,       0,  441410,       0,  0, 10,  0, }, // Maldives
    {  375, 1237,       0,  441420,       0,  0, 12,  0, }, // Marquesas
    {  375, 1247,       0,  441432,       0,  0, 16,  0, }, // Marshall_Islands
    {  375, 1264,  441448,  441458,  441477, 10, 19, 18, }, // Mauritius
    {  375, 1274,       0,  441495,       0,  0,  8,  0, }, // Mawson
    {  375, 1281,  441503,  441523,  441552, 20, 29, 27, }, // Mexico_Pacific
    {  375, 1296,  441579,  441591,  441612, 12, 21, 20, }, // Mongolia
    {  375, 1305,  441632,  441642,  441661, 10, 19, 18, }, // Moscow
    {  375, 1312,       0,  441679,       0,  0, 12,  0, }, // Myanmar
    {  375, 1320,       0,  441691,       0,  0,  7,  0, }, // Nauru
    {  375, 1326,       0,  441698,       0,  0,  9,  0, }, // Nepal
    {  375, 1332,  441707,  441726,  441754, 19, 28, 27, }, // New_Caledonia
    {  375, 1346,  441781,  441796,  441820, 15, 24, 22, }, // New_Zealand
    {  375, 1358,  441842,  441858,  441883, 16, 25, 23, }, // Newfoundland
    {  375, 1371,       0,  441906,       0,  0,  9,  0, }, // Niue
    {  375, 1376,  441915,  441931,  441956, 16, 25, 24, }, // Norfolk
    {  375, 1384,  441980,  442004,  442037, 24, 33, 32, }, // Noronha
    {  375, 1406,  339133,  442069,  442095, 13, 26, 25, }, // Novosibirsk
    {  375, 1418,  442120,  442130,  442149, 10, 19, 18, }, // Omsk
    {  375, 1428,   95878,  442167,  442189,  9, 22, 21, }, // Pakistan
    {  375, 1437,       0,  442210,       0,  0,  8,  0, }, // Palau
    {  375, 1443,       0,  442218,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  375, 1460,  442237,  442247,  442266, 10, 19, 18, }, // Paraguay
    {  375, 1469,  442284,  442292,  442309,  8, 17, 16, }, // Peru
    {  375, 1474,  442325,  442337,  442358, 12, 21, 20, }, // Philippines
    {  375, 1486,       0,  442378,       0,  0, 17,  0, }, // Phoenix_Islands
    {  375, 1502,  442395,  442419,  442452, 24, 33, 31, }, // Pierre_Miquelon
    {  375, 1518,       0,  442483,       0,  0, 11,  0, }, // Pitcairn
    {  375, 1527,       0,  442494,       0,  0,  9,  0, }, // Ponape
    {  375, 1534,       0,  442503,       0,  0, 14,  0, }, // Pyongyang
    {  375, 1554,       0,  442517,       0,  0, 12,  0, }, // Reunion
    {  375, 1562,       0,  442529,       0,  0, 10,  0, }, // Rothera
    {  375, 1570,  442539,  442548,  442566,  9, 18, 17, }, // Sakhalin
    {  375, 1596,  442583,  442592,  442610,  9, 18, 16, }, // Samoa
    {  375, 1602,       0,  442626,       0,  0, 11,  0, }, // Seychelles
    {  375, 1624,       0,  442637,       0,  0, 21,  0, }, // Singapore
    {  375, 1634,       0,  442658,       0,  0, 18,  0, }, // Solomon
    {  375, 1642,       0,  442676,       0,  0, 19,  0, }, // South_Georgia
    {  375, 1656,       0,  442695,       0,  0, 11,  0, }, // Suriname
    {  375, 1676,       0,  442706,       0,  0, 10,  0, }, // Syowa
    {  375, 1682,       0,  442716,       0,  0, 10,  0, }, // Tahiti
    {  375, 1689,  442726,  442735,  442753,  9, 18, 16, }, // Taipei
    {  375, 1696,       0,  442769,       0,  0, 14,  0, }, // Tajikistan
    {  375, 1724,       0,  442783,       0,  0, 10,  0, }, // Tokelau
    {  375, 1732,  442793,  442802,  442820,  9, 18, 17, }, // Tonga
    {  375, 1738,       0,  442837,       0,  0,  7,  0, }, // Truk
    {  375, 1750,  339997,  442844,  442871, 14, 27, 26, }, // Turkmenistan
    {  375, 1763,       0,  442897,       0,  0, 10,  0, }, // Tuvalu
    {  375, 1777,  442907,  442918,  442938, 11, 20, 19, }, // Uruguay
    {  375, 1792,  442957,  442974,  443000, 17, 26, 25, }, // Uzbekistan
    {  375, 1803,  443025,  443034,  443052,  9, 18, 17, }, // Vanuatu
    {  375, 1811,       0,  443069,       0,  0, 13,  0, }, // Venezuela
    {  375, 1821,  443082,  443099,  443125, 17, 26, 25, }, // Vladivostok
    {  375, 1833,  443150,  443167,  443193, 17, 26, 25, }, // Volgograd
    {  375, 1843,       0,  443088,       0,  0, 11,  0, }, // Vostok
    {  375, 1850,       0,  443218,       0,  0, 13,  0, }, // Wake
    {  375, 1855,       0,  443231,       0,  0, 22,  0, }, // Wallis
    {  375, 1862,  443253,  443265,  443286, 12, 21, 20, }, // Yakutsk
    {  375, 1870,  443306,  443323,  443349, 17, 26, 25, }, // Yekaterinburg
    {  375, 1892,       0,  443374,       0,  0,  9,  0, }, // Yukon
    {  376,  788,       0,    2800,       0,  0, 19,  0, }, // GMT Konkani/Latin/India
    {  377,    0,  443383,  443389,  443396,  6,  7, 10, }, // Acre Korean/Korean/South Korea
    {  377,    5,       0,  443406,       0,  0,  9,  0, }, // Afghanistan
    {  377,   17,       0,  443415,       0,  0,  9,  0, }, // Africa_Central
    {  377,   32,       0,  443424,       0,  0,  8,  0, }, // Africa_Eastern
    {  377,   65,       0,  443432,       0,  0,  8,  0, }, // Africa_Southern
    {  377,   81,  443440,  443448,  443457,  8,  9, 12, }, // Africa_Western
    {  377,  107,  443469,  443476,  443484,  7,  8, 11, }, // Alaska
    {  377,  128,  443495,  443504,  443514,  9, 10, 10, }, // Almaty
    {  377,  135,  443524,  443530,  443537,  6,  7, 10, }, // Amazon
    {  377,  142,  443547,  443554,  443562,  7,  8, 11, }, // America_Central
    {  377,  158,  443573,  443580,  443588,  7,  8, 11, }, // America_Eastern
    {  377,  174,  443599,  443606,  443614,  7,  8, 11, }, // America_Mountain
    {  377,  191,  443625,  443633,  443642,  8,  9, 12, }, // America_Pacific
    {  377,  207,  443654,  443661,  443669,  7,  8, 11, }, // Anadyr
    {  377,  214,  443680,  443686,  443693,  6,  7, 10, }, // Apia
    {  377,  219,  443703,  443712,  443722,  9, 10, 10, }, // Aqtau
    {  377,  225,  443732,  443741,  443751,  9, 10, 10, }, // Aqtobe
    {  377,  232,  443761,  443768,  443776,  7,  8, 11, }, // Arabian
    {  377,  240,  443787,  443795,  443804,  8,  9, 12, }, // Argentina
    {  377,  250,  443816,  443827,  443839, 11, 12, 15, }, // Argentina_Western
    {  377,  268,  443854,  443862,  443871,  8,  9, 12, }, // Armenia
    {  377,  286,  443883,  443889,  443896,  6,  7, 10, }, // Atlantic
    {  377,  295,  443906,  443919,  443933, 13, 14, 17, }, // Australia_Central
    {  377,  313,  443950,  443964,  443979, 14, 15, 18, }, // Australia_CentralWestern
    {  377,  338,  443997,  444010,  444024, 13, 14, 17, }, // Australia_Eastern
    {  377,  356,  444041,  444054,  444068, 13, 14, 17, }, // Australia_Western
    {  377,  374,  444085,  444094,  444104,  9, 10, 13, }, // Azerbaijan
    {  377,  385,  444117,  444124,  444132,  7,  8, 11, }, // Azores
    {  377,  397,  444143,  444151,  444160,  8,  9, 12, }, // Bangladesh
    {  377,  415,       0,  444172,       0,  0,  5,  0, }, // Bhutan
    {  377,  422,       0,  444177,       0,  0,  7,  0, }, // Bolivia
    {  377,  437,  444184,  444192,  444201,  8,  9, 12, }, // Brasilia
    {  377,  454,       0,  444213,       0,  0,  7,  0, }, // Brunei
    {  377,  461,  444220,  444229,  444239,  9, 10, 13, }, // Cape_Verde
    {  377,  472,       0,  444252,       0,  0,  6,  0, }, // Casey
    {  377,  478,       0,  444258,       0,  0,  6,  0, }, // Chamorro
    {  377,  487,  444264,  444269,  444275,  5,  6,  9, }, // Chatham
    {  377,  495,  444284,  444289,  444295,  5,  6,  9, }, // Chile
    {  377,  501,  444304,  444309,  444315,  5,  6,  9, }, // China
    {  377,  507,       0,  444324,       0,  0,  9,  0, }, // Christmas
    {  377,  517,       0,  444333,       0,  0,  9,  0, }, // Cocos
    {  377,  523,  444342,  444349,  444357,  7,  8, 11, }, // Colombia
    {  377,  532,  444368,  444375,  444383,  7,  8, 14, }, // Cook
    {  377,  537,  444397,  444402,  444408,  5,  6,  9, }, // Cuba
    {  377,  548,       0,  444417,       0,  0,  7,  0, }, // Davis
    {  377,  564,       0,  444424,       0,  0,  8,  0, }, // DumontDUrville
    {  377,  601,       0,  444432,       0,  0,  7,  0, }, // East_Timor
    {  377,  612,  444439,  444446,  444454,  7,  8, 11, }, // Easter
    {  377,  619,       0,  444465,       0,  0,  7,  0, }, // Ecuador
    {  377,  627,  444472,  444479,  444487,  7,  8, 11, }, // Europe_Central
    {  377,  642,  444498,  444504,  444511,  6,  7, 10, }, // Europe_Eastern
    {  377,  657,       0,  444521,       0,  0,  8,  0, }, // Europe_Further_Eastern
    {  377,  680,  444529,  444535,  444542,  6,  7, 10, }, // Europe_Western
    {  377,  695,  444552,  444562,  444573, 10, 11, 14, }, // Falkland
    {  377,  704,  444587,  444592,  444598,  5,  6,  9, }, // Fiji
    {  377,  709,       0,  444607,       0,  0, 12,  0, }, // French_Guiana
    {  377,  723,       0,  444619,       0,  0, 19,  0, }, // French_Southern
    {  377,  746,       0,  444638,       0,  0,  8,  0, }, // Galapagos
    {  377,  756,       0,  444646,       0,  0,  6,  0, }, // Gambier
    {  377,  764,  444652,  444658,  444665,  6,  7, 10, }, // Georgia
    {  377,  772,       0,  444675,       0,  0,  9,  0, }, // Gilbert_Islands
    {  377,  788,       0,  444684,       0,  0,  8,  0, }, // GMT
    {  377,  830,  444692,  444702,  444713, 10, 11, 14, }, // Greenland_Eastern
    {  377,  848,  444727,  444737,  444748, 10, 11, 14, }, // Greenland_Western
    {  377,  866,       0,  444762,       0,  0,  7,  0, }, // Guam
    {  377,  871,       0,  444769,       0,  0,  7,  0, }, // Gulf
    {  377,  876,       0,  444612,       0,  0,  7,  0, }, // Guyana
    {  377,  883,  444776,  444786,  444797, 10, 11, 14, }, // Hawaii_Aleutian
    {  377,  899,  444811,  444816,  444822,  5,  6,  9, }, // Hong_Kong
    {  377,  909,  444831,  444837,  444844,  6,  7, 10, }, // Hovd
    {  377,  914,       0,  444854,       0,  0,  6,  0, }, // India
    {  377,  920,       0,  444860,       0,  0,  6,  0, }, // Indian_Ocean
    {  377,  933,       0,  444866,       0,  0,  8,  0, }, // Indochina
    {  377,  943,       0,  444874,       0,  0, 11,  0, }, // Indonesia_Central
    {  377,  961,       0,  444885,       0,  0, 11,  0, }, // Indonesia_Eastern
    {  377,  979,       0,  444896,       0,  0, 11,  0, }, // Indonesia_Western
    {  377,  997,  444907,  444912,  444918,  5,  6,  9, }, // Iran
    {  377, 1008,  444927,  444935,  444944,  8,  9, 12, }, // Irkutsk
    {  377, 1016,  444956,  444963,  444971,  7,  8, 11, }, // Israel
    {  377, 1023,  444982,  444987,  444993,  5,  6,  9, }, // Japan
    {  377, 1029,  445002,  445018,  445035, 16, 17, 20, }, // Kamchatka
    {  377, 1047,       0,  445055,       0,  0,  8,  0, }, // Kazakhstan
    {  377, 1058,       0,  445063,       0,  0, 11,  0, }, // Kazakhstan_Eastern
    {  377, 1077,       0,  445074,       0,  0, 11,  0, }, // Kazakhstan_Western
    {  377, 1106,  445085,  445092,  445100,  7,  8, 11, }, // Korea
    {  377, 1112,       0,  445111,       0,  0,  8,  0, }, // Kosrae
    {  377, 1119,  445119,  445130,  445142, 11, 12, 15, }, // Krasnoyarsk
    {  377, 1151,       0,  445157,       0,  0,  9,  0, }, // Kyrgystan
    {  377, 1161,       0,  445166,       0,  0,  8,  0, }, // Lanka
    {  377, 1175,       0,  445174,       0,  0,  8,  0, }, // Line_Islands
    {  377, 1188,  445182,  445190,  445199,  8,  9, 12, }, // Lord_Howe
    {  377, 1198,  445211,  445217,  445226,  6,  9, 10, }, // Macau
    {  377, 1204,  445236,  445242,  445249,  6,  7, 10, }, // Magadan
    {  377, 1219,       0,  445259,       0,  0,  8,  0, }, // Malaysia
    {  377, 1228,       0,  445267,       0,  0,  6,  0, }, // Maldives
    {  377, 1237,       0,  445273,       0,  0, 10,  0, }, // Marquesas
    {  377, 1247,       0,  445283,       0,  0,  8,  0, }, // Marshall_Islands
    {  377, 1264,  445291,  445298,  445306,  7,  8, 11, }, // Mauritius
    {  377, 1274,       0,  445317,       0,  0,  5,  0, }, // Mawson
    {  377, 1281,  445322,  445332,  445343, 10, 11, 14, }, // Mexico_Pacific
    {  377, 1296,  445357,  445365,  445374,  8,  9, 12, }, // Mongolia
    {  377, 1305,  445386,  445393,  445401,  7,  8, 11, }, // Moscow
    {  377, 1312,       0,  445412,       0,  0,  6,  0, }, // Myanmar
    {  377, 1320,       0,  445418,       0,  0,  6,  0, }, // Nauru
    {  377, 1326,       0,  445424,       0,  0,  5,  0, }, // Nepal
    {  377, 1332,  445429,  445438,  445448,  9, 10, 13, }, // New_Caledonia
    {  377, 1346,  445461,  445468,  445476,  7,  8, 11, }, // New_Zealand
    {  377, 1358,  445487,  445495,  445504,  8,  9, 12, }, // Newfoundland
    {  377, 1371,       0,  445516,       0,  0,  6,  0, }, // Niue
    {  377, 1376,  445522,  445528,  445535,  6,  7, 10, }, // Norfolk
    {  377, 1384,  445545,  445558,  445572, 13, 14, 17, }, // Noronha
    {  377, 1392,       0,  445589,       0,  0, 14,  0, }, // North_Mariana
    {  377, 1406,  445603,  445613,  445624, 10, 11, 14, }, // Novosibirsk
    {  377, 1418,  445638,  445644,  445651,  6,  7, 10, }, // Omsk
    {  377, 1428,  445661,  445668,  445676,  7,  8, 11, }, // Pakistan
    {  377, 1437,       0,  445687,       0,  0,  6,  0, }, // Palau
    {  377, 1443,       0,  445693,       0,  0,  9,  0, }, // Papua_New_Guinea
    {  377, 1460,  445702,  445709,  445717,  7,  8, 11, }, // Paraguay
    {  377, 1469,  445728,  445733,  445739,  5,  6,  9, }, // Peru
    {  377, 1474,  445748,  445754,  445761,  6,  7, 10, }, // Philippines
    {  377, 1486,       0,  445771,       0,  0,  9,  0, }, // Phoenix_Islands
    {  377, 1502,  445780,  445793,  445807, 13, 14, 17, }, // Pierre_Miquelon
    {  377, 1518,       0,  445824,       0,  0,  6,  0, }, // Pitcairn
    {  377, 1527,       0,  445830,       0,  0,  6,  0, }, // Ponape
    {  377, 1534,       0,  443628,       0,  0,  5,  0, }, // Pyongyang
    {  377, 1544,  445836,  445844,  445855,  8, 11, 12, }, // Qyzylorda
    {  377, 1554,       0,  445867,       0,  0,  7,  0, }, // Reunion
    {  377, 1562,       0,  445874,       0,  0,  6,  0, }, // Rothera
    {  377, 1570,  445880,  445886,  445893,  6,  7, 10, }, // Sakhalin
    {  377, 1579,  445903,  445909,  445916,  6,  7, 10, }, // Samara
    {  377, 1596,  445926,  445932,  445939,  6,  7, 10, }, // Samoa
    {  377, 1602,       0,  445949,       0,  0,  6,  0, }, // Seychelles
    {  377, 1624,       0,  445955,       0,  0,  8,  0, }, // Singapore
    {  377, 1634,       0,  445963,       0,  0,  9,  0, }, // Solomon
    {  377, 1642,       0,  445972,       0,  0, 10,  0, }, // South_Georgia
    {  377, 1656,       0,  445982,       0,  0,  6,  0, }, // Suriname
    {  377, 1676,       0,  445988,       0,  0,  5,  0, }, // Syowa
    {  377, 1682,       0,  445993,       0,  0,  6,  0, }, // Tahiti
    {  377, 1689,  445999,  446004,  446010,  5,  6,  9, }, // Taipei
    {  377, 1696,       0,  446019,       0,  0,  8,  0, }, // Tajikistan
    {  377, 1724,       0,  446027,       0,  0,  7,  0, }, // Tokelau
    {  377, 1732,  446034,  446039,  446045,  5,  6,  9, }, // Tonga
    {  377, 1738,       0,  446054,       0,  0,  5,  0, }, // Truk
    {  377, 1750,  446059,  446069,  446080, 10, 11, 14, }, // Turkmenistan
    {  377, 1763,       0,  446094,       0,  0,  6,  0, }, // Tuvalu
    {  377, 1777,  446100,  446107,  446115,  7,  8, 11, }, // Uruguay
    {  377, 1792,  446126,  446135,  446145,  9, 10, 13, }, // Uzbekistan
    {  377, 1803,  446158,  446165,  446173,  7,  8, 11, }, // Vanuatu
    {  377, 1811,       0,  446184,       0,  0,  8,  0, }, // Venezuela
    {  377, 1821,  446192,  446202,  446213, 10, 11, 14, }, // Vladivostok
    {  377, 1833,  446227,  446235,  446244,  8,  9, 12, }, // Volgograd
    {  377, 1843,       0,  446256,       0,  0,  6,  0, }, // Vostok
    {  377, 1850,       0,  446262,       0,  0,  7,  0, }, // Wake
    {  377, 1855,       0,  446269,       0,  0, 12,  0, }, // Wallis
    {  377, 1862,  446281,  446288,  446296,  7,  8, 11, }, // Yakutsk
    {  377, 1870,  446307,  446317,  446328, 10, 11, 14, }, // Yekaterinburg
    {  377, 1892,       0,  446342,       0,  0,  5,  0, }, // Yukon
    {  379, 1106,  446347,  446352,  446358,  5,  6,  9, }, // Korea Korean/Korean/North Korea
    {  384,    5,       0,  446367,       0,  0, 17,  0, }, // Afghanistan Kurdish/Latin/Turkey
    {  384,   17,       0,  446384,       0,  0, 20,  0, }, // Africa_Central
    {  384,   32,       0,  446404,       0,  0, 23,  0, }, // Africa_Eastern
    {  384,   65,       0,  446427,       0,  0, 31,  0, }, // Africa_Southern
    {  384,   81,  446458,  446479,  446511, 21, 32, 31, }, // Africa_Western
    {  384,  107,  446542,  446556,  446581, 14, 25, 24, }, // Alaska
    {  384,  135,  446605,  446618,  446642, 13, 24, 23, }, // Amazon
    {  384,  142,  446665,  446697,  446740, 32, 43, 42, }, // America_Central
    {  384,  158,  446782,  446814,  446858, 32, 44, 43, }, // America_Eastern
    {  384,  174,  446901,  446932,  446974, 31, 42, 41, }, // America_Mountain
    {  384,  191,  447015,  447048,  447092, 33, 44, 43, }, // America_Pacific
    {  384,  214,  447135,  447147,  447170, 12, 23, 22, }, // Apia
    {  384,  232,  447192,  447208,  447235, 16, 27, 26, }, // Arabian
    {  384,  240,  447261,  447276,  447302, 15, 26, 25, }, // Argentina
    {  384,  250,  447327,  447349,  447382, 22, 33, 32, }, // Argentina_Western
    {  384,  268,  447414,  447431,  447459, 17, 28, 27, }, // Armenia
    {  384,  286,  447486,  447501,  447527, 15, 26, 25, }, // Atlantic
    {  384,  295,  447552,  447576,  447611, 24, 35, 34, }, // Australia_Central
    {  384,  313,  447645,  447680,  447726, 35, 46, 45, }, // Australia_CentralWestern
    {  384,  338,  447771,  447798,  447836, 27, 38, 37, }, // Australia_Eastern
    {  384,  356,  447873,  447898,  447934, 25, 36, 35, }, // Australia_Western
    {  384,  374,  447969,  447986,  448014, 17, 28, 27, }, // Azerbaijan
    {  384,  385,  448041,  448053,  448076, 12, 23, 22, }, // Azores
    {  384,  397,  448098,  448114,  448141, 16, 27, 26, }, // Bangladesh
    {  384,  415,       0,  448167,       0,  0, 12,  0, }, // Bhutan
    {  384,  422,       0,  448179,       0,  0, 15,  0, }, // Bolivia
    {  384,  437,  448194,  448210,  448237, 16, 27, 26, }, // Brasilia
    {  384,  454,       0,  448263,       0,  0, 24,  0, }, // Brunei
    {  384,  461,  448287,  448305,  448334, 18, 29, 28, }, // Cape_Verde
    {  384,  478,       0,  448362,       0,  0, 27,  0, }, // Chamorro
    {  384,  487,  448389,  448403,  448428, 14, 25, 24, }, // Chatham
    {  384,  495,  448452,  448464,  448487, 12, 23, 22, }, // Chile
    {  384,  501,  448509,  448519,  448540, 10, 21, 20, }, // China
    {  384,  507,       0,  448560,       0,  0, 23,  0, }, // Christmas
    {  384,  517,       0,  448583,       0,  0, 20,  0, }, // Cocos
    {  384,  523,  448603,  448620,  448648, 17, 28, 27, }, // Colombia
    {  384,  532,  448675,  448694,  448724, 19, 30, 32, }, // Cook
    {  384,  537,  448756,  448768,  448791, 12, 23, 22, }, // Cuba
    {  384,  548,       0,  448813,       0,  0, 12,  0, }, // Davis
    {  384,  564,       0,  448825,       0,  0, 24,  0, }, // DumontDUrville
    {  384,  601,       0,  448849,       0,  0, 21,  0, }, // East_Timor
    {  384,  612,  448870,  448893,  448927, 23, 34, 33, }, // Easter
    {  384,  619,       0,  448960,       0,  0, 14,  0, }, // Ecuador
    {  384,  627,  448974,  448994,  449025, 20, 31, 30, }, // Europe_Central
    {  384,  642,  449055,  449078,  449112, 23, 34, 33, }, // Europe_Eastern
    {  384,  657,       0,  449145,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  384,  680,  449177,  449198,  449230, 21, 32, 31, }, // Europe_Western
    {  384,  695,  449261,  449284,  449318, 23, 34, 33, }, // Falkland
    {  384,  704,  449351,  449363,  449386, 12, 23, 22, }, // Fiji
    {  384,  709,       0,  449408,       0,  0, 22,  0, }, // French_Guiana
    {  384,  723,       0,  449430,       0,  0, 34,  0, }, // French_Southern
    {  384,  746,       0,  449464,       0,  0, 16,  0, }, // Galapagos
    {  384,  756,       0,  449480,       0,  0, 14,  0, }, // Gambier
    {  384,  764,  449494,  449510,  449537, 16, 27, 26, }, // Georgia
    {  384,  772,       0,  449563,       0,  0, 22,  0, }, // Gilbert_Islands
    {  384,  788,       0,  449585,       0,  0, 26,  0, }, // GMT
    {  384,  830,  449611,  449635,  449670, 24, 35, 34, }, // Greenland_Eastern
    {  384,  848,  449704,  449726,  449759, 22, 33, 32, }, // Greenland_Western
    {  384,  871,       0,  449791,       0,  0, 24,  0, }, // Gulf
    {  384,  876,       0,  449815,       0,  0, 14,  0, }, // Guyana
    {  384,  883,  449829,  449851,  449884, 22, 33, 32, }, // Hawaii_Aleutian
    {  384,  899,  449916,  449932,  449959, 16, 27, 26, }, // Hong_Kong
    {  384,  909,  449985,  449996,  450018, 11, 22, 21, }, // Hovd
    {  384,  914,       0,  450039,       0,  0, 27,  0, }, // India
    {  384,  920,       0,  450066,       0,  0, 20,  0, }, // Indian_Ocean
    {  384,  933,       0,  450086,       0,  0, 15,  0, }, // Indochina
    {  384,  943,       0,  450101,       0,  0, 23,  0, }, // Indonesia_Central
    {  384,  961,       0,  450124,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  384,  979,       0,  450150,       0,  0, 24,  0, }, // Indonesia_Western
    {  384,  997,  450174,  450185,  450207, 11, 22, 21, }, // Iran
    {  384, 1008,  450228,  450242,  450267, 14, 25, 24, }, // Irkutsk
    {  384, 1016,  450291,  450304,  450328, 13, 24, 23, }, // Israel
    {  384, 1023,  450351,  450366,  450392, 15, 26, 25, }, // Japan
    {  384, 1047,       0,  450417,       0,  0, 17,  0, }, // Kazakhstan
    {  384, 1058,       0,  450434,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  384, 1077,       0,  450460,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  384, 1106,  450484,  450496,  450519, 12, 23, 22, }, // Korea
    {  384, 1112,       0,  450541,       0,  0, 14,  0, }, // Kosrae
    {  384, 1119,  450555,  450573,  450602, 18, 29, 28, }, // Krasnoyarsk
    {  384, 1151,       0,  450630,       0,  0, 18,  0, }, // Kyrgystan
    {  384, 1175,       0,  450648,       0,  0, 20,  0, }, // Line_Islands
    {  384, 1188,  450668,  450685,  450713, 17, 28, 27, }, // Lord_Howe
    {  384, 1204,  450740,  450754,  450779, 14, 25, 24, }, // Magadan
    {  384, 1219,       0,  450803,       0,  0, 15,  0, }, // Malaysia
    {  384, 1228,       0,  450818,       0,  0, 14,  0, }, // Maldives
    {  384, 1237,       0,  450832,       0,  0, 16,  0, }, // Marquesas
    {  384, 1247,       0,  450848,       0,  0, 21,  0, }, // Marshall_Islands
    {  384, 1264,  450869,  450885,  450912, 16, 27, 26, }, // Mauritius
    {  384, 1274,       0,  450938,       0,  0, 13,  0, }, // Mawson
    {  384, 1281,  450951,  450975,  451010, 24, 35, 34, }, // Mexico_Pacific
    {  384, 1296,  451044,  451060,  451087, 16, 27, 26, }, // Mongolia
    {  384, 1305,  451113,  451128,  451154, 15, 26, 25, }, // Moscow
    {  384, 1312,       0,  451179,       0,  0, 14,  0, }, // Myanmar
    {  384, 1320,       0,  451193,       0,  0, 13,  0, }, // Nauru
    {  384, 1326,       0,  451206,       0,  0, 12,  0, }, // Nepal
    {  384, 1332,  451218,  451238,  451269, 20, 31, 30, }, // New_Caledonia
    {  384, 1346,  451299,  451317,  451346, 18, 29, 28, }, // New_Zealand
    {  384, 1358,  451374,  451393,  451423, 19, 30, 29, }, // Newfoundland
    {  384, 1371,       0,  451452,       0,  0, 12,  0, }, // Niue
    {  384, 1376,  451464,  451485,  451517, 21, 32, 31, }, // Norfolk
    {  384, 1384,  451548,  451575,  451613, 27, 38, 37, }, // Noronha
    {  384, 1406,  451650,  451668,  451697, 18, 29, 28, }, // Novosibirsk
    {  384, 1418,  451725,  451736,  451758, 11, 22, 21, }, // Omsk
    {  384, 1428,  451779,  451794,  451820, 15, 26, 25, }, // Pakistan
    {  384, 1437,       0,  451845,       0,  0, 13,  0, }, // Palau
    {  384, 1443,       0,  451858,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  384, 1460,  451884,  451899,  451925, 15, 26, 25, }, // Paraguay
    {  384, 1469,  451950,  451962,  451985, 12, 23, 22, }, // Peru
    {  384, 1474,  452007,  452021,  452046, 14, 25, 24, }, // Philippines
    {  384, 1486,       0,  452070,       0,  0, 22,  0, }, // Phoenix_Islands
    {  384, 1502,  452092,  452122,  452163, 30, 41, 40, }, // Pierre_Miquelon
    {  384, 1518,       0,  452203,       0,  0, 15,  0, }, // Pitcairn
    {  384, 1527,       0,  452218,       0,  0, 14,  0, }, // Ponape
    {  384, 1534,       0,  452232,       0,  0, 16,  0, }, // Pyongyang
    {  384, 1554,       0,  452248,       0,  0, 14,  0, }, // Reunion
    {  384, 1562,       0,  452262,       0,  0, 15,  0, }, // Rothera
    {  384, 1570,  452277,  452291,  452316, 14, 25, 24, }, // Sakhalin
    {  384, 1596,  452340,  452353,  452377, 13, 24, 23, }, // Samoa
    {  384, 1602,       0,  452400,       0,  0, 15,  0, }, // Seychelles
    {  384, 1624,       0,  452415,       0,  0, 26,  0, }, // Singapore
    {  384, 1634,       0,  452441,       0,  0, 22,  0, }, // Solomon
    {  384, 1642,       0,  452463,       0,  0, 21,  0, }, // South_Georgia
    {  384, 1656,       0,  452484,       0,  0, 14,  0, }, // Suriname
    {  384, 1676,       0,  452498,       0,  0, 13,  0, }, // Syowa
    {  384, 1682,       0,  452511,       0,  0, 14,  0, }, // Tahiti
    {  384, 1689,  452525,  452539,  452564, 14, 25, 24, }, // Taipei
    {  384, 1696,       0,  452588,       0,  0, 17,  0, }, // Tajikistan
    {  384, 1724,       0,  452605,       0,  0, 15,  0, }, // Tokelau
    {  384, 1732,  452620,  452633,  452657, 13, 24, 23, }, // Tonga
    {  384, 1738,       0,  452680,       0,  0, 12,  0, }, // Truk
    {  384, 1750,  452692,  452711,  452741, 19, 30, 29, }, // Turkmenistan
    {  384, 1763,       0,  452770,       0,  0, 14,  0, }, // Tuvalu
    {  384, 1777,  452784,  452798,  452823, 14, 25, 24, }, // Uruguay
    {  384, 1792,  452847,  452864,  452892, 17, 28, 27, }, // Uzbekistan
    {  384, 1803,  452919,  452934,  452960, 15, 26, 25, }, // Vanuatu
    {  384, 1811,       0,  452985,       0,  0, 17,  0, }, // Venezuela
    {  384, 1821,  453002,  453020,  453049, 18, 29, 28, }, // Vladivostok
    {  384, 1833,  453077,  453093,  453120, 16, 27, 26, }, // Volgograd
    {  384, 1843,       0,  453146,       0,  0, 13,  0, }, // Vostok
    {  384, 1850,       0,  453159,       0,  0, 19,  0, }, // Wake
    {  384, 1855,       0,  453178,       0,  0, 23,  0, }, // Wallis
    {  384, 1862,  453201,  453215,  453240, 14, 25, 24, }, // Yakutsk
    {  384, 1870,  453264,  453284,  453315, 20, 31, 30, }, // Yekaterinburg
    {  384, 1892,       0,  453345,       0,  0, 12,  0, }, // Yukon
    {  386,    5,       0,  453357,       0,  0, 19,  0, }, // Afghanistan Kyrgyz/Cyrillic/Kyrgyzstan
    {  386,   17,       0,  453376,       0,  0, 25,  0, }, // Africa_Central
    {  386,   32,       0,  453401,       0,  0, 21,  0, }, // Africa_Eastern
    {  386,   65,       0,  453422,       0,  0, 22,  0, }, // Africa_Southern
    {  386,   81,  453444,  453465,  453491, 21, 26, 27, }, // Africa_Western
    {  386,  107,  453518,  453533,  453554, 15, 21, 21, }, // Alaska
    {  386,  135,  453575,  453590,  453611, 15, 21, 21, }, // Amazon
    {  386,  142,  453632,  453664,  453704, 32, 40, 40, }, // America_Central
    {  386,  158,  453744,  453774,  453810, 30, 36, 36, }, // America_Eastern
    {  386,  174,  453846,  453874,  453908, 28, 34, 34, }, // America_Mountain
    {  386,  191,  453942,  453977,  454018, 35, 41, 41, }, // America_Pacific
    {  386,  214,  454059,  454072,  454091, 13, 19, 19, }, // Apia
    {  386,  232,  454110,  454125,  454145, 15, 20, 20, }, // Arabian
    {  386,  240,  454165,  454183,  454207, 18, 24, 24, }, // Argentina
    {  386,  250,  454231,  454255,  454285, 24, 30, 30, }, // Argentina_Western
    {  386,  268,  454315,  454331,  454352, 16, 21, 22, }, // Armenia
    {  386,  286,  454374,  454392,  454416, 18, 24, 24, }, // Atlantic
    {  386,  295,  454440,  454468,  454501, 28, 33, 34, }, // Australia_Central
    {  386,  313,  454535,  454569,  454608, 34, 39, 40, }, // Australia_CentralWestern
    {  386,  338,  454648,  454672,  454701, 24, 29, 30, }, // Australia_Eastern
    {  386,  356,  454731,  454755,  454784, 24, 29, 30, }, // Australia_Western
    {  386,  374,  454814,  454833,  454857, 19, 24, 25, }, // Azerbaijan
    {  386,  385,  454882,  454895,  454913, 13, 18, 20, }, // Azores
    {  386,  397,  454933,  454951,  454974, 18, 23, 24, }, // Bangladesh
    {  386,  415,       0,  454998,       0,  0, 14,  0, }, // Bhutan
    {  386,  422,       0,  455012,       0,  0, 16,  0, }, // Bolivia
    {  386,  437,  455028,  455045,  455068, 17, 23, 23, }, // Brasilia
    {  386,  454,       0,  455091,       0,  0, 26,  0, }, // Brunei
    {  386,  461,  455117,  455136,  455160, 19, 24, 25, }, // Cape_Verde
    {  386,  478,       0,  455185,       0,  0, 16,  0, }, // Chamorro
    {  386,  487,  455201,  455215,  455233, 14, 18, 20, }, // Chatham
    {  386,  495,  455253,  455266,  455285, 13, 19, 19, }, // Chile
    {  386,  501,  455304,  455318,  455337, 14, 19, 19, }, // China
    {  386,  507,       0,  455356,       0,  0, 25,  0, }, // Christmas
    {  386,  517,       0,  455381,       0,  0, 26,  0, }, // Cocos
    {  386,  523,  455407,  455424,  455447, 17, 23, 23, }, // Colombia
    {  386,  532,  455470,  455494,  455524, 24, 30, 36, }, // Cook
    {  386,  537,  455560,  455573,  455592, 13, 19, 19, }, // Cuba
    {  386,  548,       0,  455611,       0,  0, 14,  0, }, // Davis
    {  386,  564,       0,  455625,       0,  0, 22,  0, }, // DumontDUrville
    {  386,  601,       0,  455647,       0,  0, 20,  0, }, // East_Timor
    {  386,  612,  455667,  455690,  455718, 23, 28, 28, }, // Easter
    {  386,  619,       0,  455746,       0,  0, 16,  0, }, // Ecuador
    {  386,  627,  455762,  455787,  455817, 25, 30, 31, }, // Europe_Central
    {  386,  642,  455848,  455869,  455895, 21, 26, 27, }, // Europe_Eastern
    {  386,  657,       0,  455922,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  386,  680,  455953,  455974,  456000, 21, 26, 27, }, // Europe_Western
    {  386,  695,  456027,  456056,  456090, 29, 34, 35, }, // Falkland
    {  386,  704,  456125,  456138,  456156, 13, 18, 19, }, // Fiji
    {  386,  709,       0,  456175,       0,  0, 23,  0, }, // French_Guiana
    {  386,  723,       0,  456198,       0,  0, 40,  0, }, // French_Southern
    {  386,  746,       0,  456238,       0,  0, 18,  0, }, // Galapagos
    {  386,  756,       0,  456256,       0,  0, 15,  0, }, // Gambier
    {  386,  764,  456271,  456286,  456306, 15, 20, 21, }, // Georgia
    {  386,  772,       0,  456327,       0,  0, 16,  0, }, // Gilbert_Islands
    {  386,  788,       0,  456343,       0,  0, 28,  0, }, // GMT
    {  386,  830,  456371,  456396,  456427, 25, 31, 31, }, // Greenland_Eastern
    {  386,  848,  456458,  456483,  456514, 25, 31, 31, }, // Greenland_Western
    {  386,  871,       0,  456545,       0,  0, 28,  0, }, // Gulf
    {  386,  876,       0,  456183,       0,  0, 15,  0, }, // Guyana
    {  386,  883,  456573,  456594,  456621, 21, 27, 27, }, // Hawaii_Aleutian
    {  386,  899,  456648,  456664,  456685, 16, 21, 22, }, // Hong_Kong
    {  386,  909,  456707,  456720,  456738, 13, 18, 19, }, // Hovd
    {  386,  914,       0,  456757,       0,  0, 14,  0, }, // India
    {  386,  920,       0,  456771,       0,  0, 19,  0, }, // Indian_Ocean
    {  386,  933,       0,  456790,       0,  0, 18,  0, }, // Indochina
    {  386,  943,       0,  456808,       0,  0, 28,  0, }, // Indonesia_Central
    {  386,  961,       0,  456836,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  386,  979,       0,  456860,       0,  0, 24,  0, }, // Indonesia_Western
    {  386,  997,  456884,  456897,  456915, 13, 18, 22, }, // Iran
    {  386, 1008,  456937,  456953,  456974, 16, 21, 21, }, // Irkutsk
    {  386, 1016,  456995,  457010,  457031, 15, 21, 21, }, // Israel
    {  386, 1023,  457052,  457066,  457085, 14, 19, 20, }, // Japan
    {  386, 1047,       0,  457105,       0,  0, 18,  0, }, // Kazakhstan
    {  386, 1058,       0,  457123,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  386, 1077,       0,  457147,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  386, 1106,  457171,  457185,  457204, 14, 19, 20, }, // Korea
    {  386, 1112,       0,  457224,       0,  0, 15,  0, }, // Kosrae
    {  386, 1119,  457239,  457258,  457282, 19, 24, 25, }, // Krasnoyarsk
    {  386, 1151,       0,  457307,       0,  0, 19,  0, }, // Kyrgystan
    {  386, 1175,       0,  457326,       0,  0, 25,  0, }, // Line_Islands
    {  386, 1188,  457351,  457368,  457390, 17, 22, 23, }, // Lord_Howe
    {  386, 1204,  457413,  457429,  457450, 16, 21, 22, }, // Magadan
    {  386, 1219,       0,  457472,       0,  0, 17,  0, }, // Malaysia
    {  386, 1228,       0,  457489,       0,  0, 16,  0, }, // Maldives
    {  386, 1237,       0,  457505,       0,  0, 17,  0, }, // Marquesas
    {  386, 1247,       0,  457522,       0,  0, 28,  0, }, // Marshall_Islands
    {  386, 1264,  457550,  457567,  457589, 17, 22, 23, }, // Mauritius
    {  386, 1274,       0,  457612,       0,  0, 15,  0, }, // Mawson
    {  386, 1281,  457627,  457655,  457689, 28, 34, 34, }, // Mexico_Pacific
    {  386, 1296,  457723,  457742,  457766, 19, 24, 25, }, // Mongolia
    {  386, 1305,  457791,  457806,  457826, 15, 20, 21, }, // Moscow
    {  386, 1312,       0,  457847,       0,  0, 16,  0, }, // Myanmar
    {  386, 1320,       0,  457863,       0,  0, 14,  0, }, // Nauru
    {  386, 1326,       0,  457877,       0,  0, 14,  0, }, // Nepal
    {  386, 1332,  457891,  457914,  457943, 23, 29, 29, }, // New_Caledonia
    {  386, 1346,  457972,  457994,  458021, 22, 27, 27, }, // New_Zealand
    {  386, 1358,  458048,  458068,  458094, 20, 26, 26, }, // Newfoundland
    {  386, 1371,       0,  458120,       0,  0, 13,  0, }, // Niue
    {  386, 1376,  458133,  458149,  458171, 16, 22, 22, }, // Norfolk
    {  386, 1384,  458193,  458221,  458255, 28, 34, 34, }, // Noronha
    {  386, 1406,  458289,  458309,  458334, 20, 25, 26, }, // Novosibirsk
    {  386, 1418,  458360,  458373,  458391, 13, 18, 19, }, // Omsk
    {  386, 1428,  458410,  458427,  458449, 17, 22, 23, }, // Pakistan
    {  386, 1437,       0,  458472,       0,  0, 14,  0, }, // Palau
    {  386, 1443,       0,  458486,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  386, 1460,  458511,  458528,  458551, 17, 23, 23, }, // Paraguay
    {  386, 1469,  458574,  458587,  458606, 13, 19, 19, }, // Peru
    {  386, 1474,  458625,  458654,  458689, 29, 35, 35, }, // Philippines
    {  386, 1486,       0,  458724,       0,  0, 27,  0, }, // Phoenix_Islands
    {  386, 1502,  458751,  458781,  458817, 30, 36, 36, }, // Pierre_Miquelon
    {  386, 1518,       0,  458853,       0,  0, 17,  0, }, // Pitcairn
    {  386, 1527,       0,  458870,       0,  0, 15,  0, }, // Ponape
    {  386, 1534,       0,  458885,       0,  0, 15,  0, }, // Pyongyang
    {  386, 1554,       0,  458900,       0,  0, 16,  0, }, // Reunion
    {  386, 1562,       0,  458916,       0,  0, 15,  0, }, // Rothera
    {  386, 1570,  458931,  458947,  458968, 16, 21, 22, }, // Sakhalin
    {  386, 1596,  458990,  459004,  459024, 14, 20, 20, }, // Samoa
    {  386, 1602,       0,  459044,       0,  0, 15,  0, }, // Seychelles
    {  386, 1624,       0,  459059,       0,  0, 17,  0, }, // Singapore
    {  386, 1634,       0,  459076,       0,  0, 28,  0, }, // Solomon
    {  386, 1642,       0,  459104,       0,  0, 22,  0, }, // South_Georgia
    {  386, 1656,       0,  459126,       0,  0, 17,  0, }, // Suriname
    {  386, 1676,       0,  459143,       0,  0, 13,  0, }, // Syowa
    {  386, 1682,       0,  459156,       0,  0, 14,  0, }, // Tahiti
    {  386, 1689,  459170,  459185,  459205, 15, 20, 20, }, // Taipei
    {  386, 1696,       0,  459225,       0,  0, 18,  0, }, // Tajikistan
    {  386, 1724,       0,  459243,       0,  0, 16,  0, }, // Tokelau
    {  386, 1732,  459259,  459273,  459293, 14, 20, 20, }, // Tonga
    {  386, 1738,       0,  459313,       0,  0, 13,  0, }, // Truk
    {  386, 1750,  459326,  459346,  459371, 20, 25, 26, }, // Turkmenistan
    {  386, 1763,       0,  459397,       0,  0, 15,  0, }, // Tuvalu
    {  386, 1777,  459412,  459428,  459450, 16, 22, 22, }, // Uruguay
    {  386, 1792,  459472,  459490,  459513, 18, 23, 24, }, // Uzbekistan
    {  386, 1803,  459537,  459553,  459574, 16, 21, 22, }, // Vanuatu
    {  386, 1811,       0,  459596,       0,  0, 18,  0, }, // Venezuela
    {  386, 1821,  459614,  459634,  459659, 20, 25, 26, }, // Vladivostok
    {  386, 1833,  459685,  459703,  459726, 18, 23, 24, }, // Volgograd
    {  386, 1843,       0,  459750,       0,  0, 15,  0, }, // Vostok
    {  386, 1850,       0,  459765,       0,  0, 25,  0, }, // Wake
    {  386, 1855,       0,  459790,       0,  0, 26,  0, }, // Wallis
    {  386, 1862,  459816,  459831,  459851, 15, 20, 21, }, // Yakutsk
    {  386, 1870,  459872,  459893,  459919, 21, 26, 27, }, // Yekaterinburg
    {  386, 1892,       0,  459946,       0,  0, 13,  0, }, // Yukon
    {  389,    0,  459959,  459971,  459991, 12, 20, 20, }, // Acre Lao/Lao/Laos
    {  389,    5,       0,  460011,       0,  0, 16,  0, }, // Afghanistan
    {  389,   17,       0,  460027,       0,  0, 19,  0, }, // Africa_Central
    {  389,   32,       0,  460046,       0,  0, 27,  0, }, // Africa_Eastern
    {  389,   65,       0,  460073,       0,  0, 19,  0, }, // Africa_Southern
    {  389,   81,  460092,  460119,  460157, 27, 38, 38, }, // Africa_Western
    {  389,  107,  460195,  460206,  460225, 11, 19, 18, }, // Alaska
    {  389,  128,  460243,  460253,  460271, 10, 18, 18, }, // Almaty
    {  389,  135,  460289,  460306,  460331, 17, 25, 25, }, // Amazon
    {  389,  142,  460225,  460356,  460371,  7, 15, 14, }, // America_Central
    {  389,  158,  460385,  460398,  460419, 13, 21, 20, }, // America_Eastern
    {  389,  174,  460439,  460452,  460473, 13, 21, 20, }, // America_Mountain
    {  389,  191,  460493,  460504,  460523, 11, 19, 18, }, // America_Pacific
    {  389,  214,  460541,  460550,  460567,  9, 17, 16, }, // Apia
    {  389,  219,  460583,  460594,  460613, 11, 19, 19, }, // Aqtau
    {  389,  225,  460632,  460643,  460662, 11, 19, 19, }, // Aqtobe
    {  389,  232,  460681,  460696,  460721, 15, 25, 18, }, // Arabian
    {  389,  240,  460739,  460756,  460785, 17, 29, 30, }, // Argentina
    {  389,  250,  460815,  460842,  460879, 27, 37, 37, }, // Argentina_Western
    {  389,  268,  460916,  460927,  460946, 11, 19, 19, }, // Armenia
    {  389,  286,  460965,  460980,  461003, 15, 23, 22, }, // Atlantic
    {  389,  295,  461025,  461046,  461078, 21, 32, 30, }, // Australia_Central
    {  389,  313,  461108,  461140,  461183, 32, 43, 42, }, // Australia_CentralWestern
    {  389,  338,  461225,  461253,  461293, 28, 40, 38, }, // Australia_Eastern
    {  389,  356,  461331,  461360,  461400, 29, 40, 38, }, // Australia_Western
    {  389,  374,  461438,  461453,  461476, 15, 23, 23, }, // Azerbaijan
    {  389,  385,  461499,  461515,  461541, 16, 26, 26, }, // Azores
    {  389,  397,  461567,  461582,  461605, 15, 23, 24, }, // Bangladesh
    {  389,  415,       0,  461629,       0,  0, 12,  0, }, // Bhutan
    {  389,  422,       0,  461641,       0,  0, 15,  0, }, // Bolivia
    {  389,  437,  461656,  461674,  461694, 18, 20, 32, }, // Brasilia
    {  389,  454,       0,  461726,       0,  0, 25,  0, }, // Brunei
    {  389,  461,  461751,  461765,  461791, 14, 26, 25, }, // Cape_Verde
    {  389,  472,       0,  461816,       0,  0,  8,  0, }, // Casey
    {  389,  478,       0,  461824,       0,  0, 14,  0, }, // Chamorro
    {  389,  487,  461838,  461850,  461873, 12, 23, 21, }, // Chatham
    {  389,  495,  461894,  461905,  461926, 11, 21, 21, }, // Chile
    {  389,  501,  461947,  461956,  461971,  9, 15, 19, }, // China
    {  389,  507,       0,  461990,       0,  0, 20,  0, }, // Christmas
    {  389,  517,       0,  462010,       0,  0, 17,  0, }, // Cocos
    {  389,  523,  462027,  462039,  462058, 12, 19, 20, }, // Colombia
    {  389,  532,  462078,  462093,  462116, 15, 23, 35, }, // Cook
    {  389,  537,  462151,  462160,  462180,  9, 20, 16, }, // Cuba
    {  389,  548,       0,  462196,       0,  0,  9,  0, }, // Davis
    {  389,  564,       0,  462205,       0,  0, 14,  0, }, // DumontDUrville
    {  389,  601,       0,  462219,       0,  0, 17,  0, }, // East_Timor
    {  389,  612,  462236,  462255,  462284, 19, 29, 29, }, // Easter
    {  389,  619,       0,  462313,       0,  0, 15,  0, }, // Ecuador
    {  389,  627,  462328,  462344,  462370, 16, 26, 28, }, // Europe_Central
    {  389,  642,  462398,  462422,  462458, 24, 36, 34, }, // Europe_Eastern
    {  389,  657,       0,  462492,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  389,  680,  462523,  462547,  462581, 24, 34, 34, }, // Europe_Western
    {  389,  695,  462615,  462641,  462677, 26, 36, 36, }, // Falkland
    {  389,  704,  462713,  462721,  462737,  8, 16, 16, }, // Fiji
    {  389,  709,       0,  462753,       0,  0, 21,  0, }, // French_Guiana
    {  389,  723,       0,  462774,       0,  0, 29,  0, }, // French_Southern
    {  389,  746,       0,  462803,       0,  0, 19,  0, }, // Galapagos
    {  389,  756,       0,  462822,       0,  0, 10,  0, }, // Gambier
    {  389,  764,  462832,  462841,  462858,  9, 17, 17, }, // Georgia
    {  389,  772,       0,  462875,       0,  0, 19,  0, }, // Gilbert_Islands
    {  389,  788,       0,  462894,       0,  0, 14,  0, }, // GMT
    {  389,  830,  462908,  462931,  462959, 23, 28, 28, }, // Greenland_Eastern
    {  389,  848,  462987,  463007,  463035, 20, 28, 27, }, // Greenland_Western
    {  389,  866,       0,  463062,       0,  0,  7,  0, }, // Guam
    {  389,  871,       0,  463069,       0,  0, 13,  0, }, // Gulf
    {  389,  876,       0,  463082,       0,  0, 11,  0, }, // Guyana
    {  389,  883,  463093,  463110,  463135, 17, 25, 24, }, // Hawaii_Aleutian
    {  389,  899,  463159,  463172,  463196, 13, 24, 25, }, // Hong_Kong
    {  389,  909,  463221,  463233,  463257, 12, 24, 24, }, // Hovd
    {  389,  914,       0,  463281,       0,  0, 11,  0, }, // India
    {  389,  920,       0,  463292,       0,  0, 20,  0, }, // Indian_Ocean
    {  389,  933,       0,  463312,       0,  0, 12,  0, }, // Indochina
    {  389,  943,       0,  463324,       0,  0, 23,  0, }, // Indonesia_Central
    {  389,  961,       0,  463347,       0,  0, 31,  0, }, // Indonesia_Eastern
    {  389,  979,       0,  463378,       0,  0, 31,  0, }, // Indonesia_Western
    {  389,  997,  463409,  463421,  463443, 12, 22, 22, }, // Iran
    {  389, 1008,  463465,  463482,  463508, 17, 26, 26, }, // Irkutsk
    {  389, 1016,  463534,  463551,  463578, 17, 27, 19, }, // Israel
    {  389, 1023,  463597,  463611,  463636, 14, 25, 23, }, // Japan
    {  389, 1047,       0,  463659,       0,  0, 13,  0, }, // Kazakhstan
    {  389, 1058,       0,  463672,       0,  0, 30,  0, }, // Kazakhstan_Eastern
    {  389, 1077,       0,  463702,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  389, 1106,  463732,  463743,  463768, 11, 25, 23, }, // Korea
    {  389, 1112,       0,  463791,       0,  0,  9,  0, }, // Kosrae
    {  389, 1119,  463800,  463821,  463852, 21, 31, 31, }, // Krasnoyarsk
    {  389, 1151,       0,  463883,       0,  0, 13,  0, }, // Kyrgystan
    {  389, 1161,       0,  463896,       0,  0,  9,  0, }, // Lanka
    {  389, 1175,       0,  463905,       0,  0, 18,  0, }, // Line_Islands
    {  389, 1188,  463923,  463937,  463962, 14, 25, 28, }, // Lord_Howe
    {  389, 1198,  463990,  464000,  464018, 10, 18, 18, }, // Macau
    {  389, 1204,  464036,  464049,  464070, 13, 21, 21, }, // Magadan
    {  389, 1219,       0,  464091,       0,  0, 15,  0, }, // Malaysia
    {  389, 1228,       0,  464106,       0,  0, 10,  0, }, // Maldives
    {  389, 1237,       0,  464116,       0,  0, 12,  0, }, // Marquesas
    {  389, 1247,       0,  464128,       0,  0, 21,  0, }, // Marshall_Islands
    {  389, 1264,  464149,  464167,  464195, 18, 28, 30, }, // Mauritius
    {  389, 1274,       0,  464225,       0,  0,  9,  0, }, // Mawson
    {  389, 1281,  464234,  464254,  464282, 20, 28, 27, }, // Mexico_Pacific
    {  389, 1296,  464309,  464324,  464347, 15, 23, 22, }, // Mongolia
    {  389, 1305,  464369,  464382,  464405, 13, 23, 23, }, // Moscow
    {  389, 1312,       0,  464428,       0,  0,  9,  0, }, // Myanmar
    {  389, 1320,       0,  464437,       0,  0, 14,  0, }, // Nauru
    {  389, 1326,       0,  464451,       0,  0, 13,  0, }, // Nepal
    {  389, 1332,  464464,  464480,  464504, 16, 24, 24, }, // New_Caledonia
    {  389, 1346,  464528,  464544,  464570, 16, 26, 25, }, // New_Zealand
    {  389, 1358,  464595,  464612,  464640, 17, 28, 20, }, // Newfoundland
    {  389, 1371,       0,  464660,       0,  0, 10,  0, }, // Niue
    {  389, 1376,  464670,  464690,  464720, 20, 30, 30, }, // Norfolk
    {  389, 1384,  464750,  464770,  464798, 20, 28, 28, }, // Noronha
    {  389, 1392,       0,  464826,       0,  0, 24,  0, }, // North_Mariana
    {  389, 1406,  464850,  464872,  464904, 22, 32, 32, }, // Novosibirsk
    {  389, 1418,  464936,  464950,  464974, 14, 24, 24, }, // Omsk
    {  389, 1428,  464998,  465013,  465039, 15, 26, 28, }, // Pakistan
    {  389, 1437,       0,  465067,       0,  0, 10,  0, }, // Palau
    {  389, 1443,       0,  465077,       0,  0, 13,  0, }, // Papua_New_Guinea
    {  389, 1460,  465090,  465105,  465131, 15, 26, 26, }, // Paraguay
    {  389, 1469,  465157,  465168,  465191, 11, 23, 22, }, // Peru
    {  389, 1474,  465213,  465230,  465257, 17, 27, 27, }, // Philippines
    {  389, 1486,       0,  465284,       0,  0, 17,  0, }, // Phoenix_Islands
    {  389, 1502,  465301,  465331,  465371, 30, 40, 38, }, // Pierre_Miquelon
    {  389, 1518,       0,  465409,       0,  0, 12,  0, }, // Pitcairn
    {  389, 1527,       0,  465421,       0,  0,  9,  0, }, // Ponape
    {  389, 1534,       0,  465430,       0,  0, 10,  0, }, // Pyongyang
    {  389, 1544,  465440,  465451,  465470, 11, 19, 19, }, // Qyzylorda
    {  389, 1554,       0,  465489,       0,  0, 18,  0, }, // Reunion
    {  389, 1562,       0,  465507,       0,  0, 11,  0, }, // Rothera
    {  389, 1570,  465518,  465533,  465558, 15, 25, 25, }, // Sakhalin
    {  389, 1596,  465583,  465592,  465609,  9, 17, 17, }, // Samoa
    {  389, 1602,       0,  465626,       0,  0, 16,  0, }, // Seychelles
    {  389, 1624,       0,  465642,       0,  0, 15,  0, }, // Singapore
    {  389, 1634,       0,  465657,       0,  0, 19,  0, }, // Solomon
    {  389, 1642,       0,  465676,       0,  0, 12,  0, }, // South_Georgia
    {  389, 1656,       0,  465688,       0,  0, 15,  0, }, // Suriname
    {  389, 1676,       0,  465703,       0,  0,  9,  0, }, // Syowa
    {  389, 1682,       0,  465712,       0,  0, 10,  0, }, // Tahiti
    {  389, 1689,  465722,  465733,  465755, 11, 22, 21, }, // Taipei
    {  389, 1696,       0,  465776,       0,  0, 14,  0, }, // Tajikistan
    {  389, 1724,       0,  465790,       0,  0, 12,  0, }, // Tokelau
    {  389, 1732,  465802,  465811,  465828,  9, 17, 17, }, // Tonga
    {  389, 1738,       0,  465845,       0,  0,  7,  0, }, // Truk
    {  389, 1750,  465852,  465867,  465890, 15, 23, 23, }, // Turkmenistan
    {  389, 1763,       0,  465913,       0,  0, 10,  0, }, // Tuvalu
    {  389, 1777,  465923,  465939,  465965, 16, 26, 26, }, // Uruguay
    {  389, 1792,  465991,  466008,  466033, 17, 25, 25, }, // Uzbekistan
    {  389, 1803,  466058,  466070,  466090, 12, 20, 20, }, // Vanuatu
    {  389, 1811,       0,  466110,       0,  0, 20,  0, }, // Venezuela
    {  389, 1821,  466130,  466144,  466166, 14, 22, 22, }, // Vladivostok
    {  389, 1833,  466188,  466200,  466220, 12, 20, 20, }, // Volgograd
    {  389, 1843,       0,  466240,       0,  0, 11,  0, }, // Vostok
    {  389, 1850,       0,  466251,       0,  0, 11,  0, }, // Wake
    {  389, 1855,       0,  466262,       0,  0, 21,  0, }, // Wallis
    {  389, 1862,  466283,  466293,  466311, 10, 18, 18, }, // Yakutsk
    {  389, 1870,  466329,  466346,  466371, 17, 25, 25, }, // Yekaterinburg
    {  389, 1892,       0,  466396,       0,  0,  9,  0, }, // Yukon
    {  391,    5,       0,  466405,       0,  0, 18,  0, }, // Afghanistan Latvian/Latin/Latvia
    {  391,   17,       0,  466423,       0,  0, 20,  0, }, // Africa_Central
    {  391,   32,       0,  466443,       0,  0, 20,  0, }, // Africa_Eastern
    {  391,   65,       0,  466463,       0,  0, 27,  0, }, // Africa_Southern
    {  391,   81,  466490,  466509,  466535, 19, 26, 27, }, // Africa_Western
    {  391,  107,  466562,  466575,  466595, 13, 20, 21, }, // Alaska
    {  391,  135,  466616,  466630,  466651, 14, 21, 22, }, // Amazon
    {  391,  142,  466673,  466689,  466712, 16, 23, 24, }, // America_Central
    {  391,  158,  466736,  466750,  466771, 14, 21, 22, }, // America_Eastern
    {  391,  174,  466793,  466804,  466822, 11, 18, 19, }, // America_Mountain
    {  391,  191,  466841,  466859,  466884, 18, 25, 26, }, // America_Pacific
    {  391,  207,  466910,  466924,  466945, 14, 21, 22, }, // Anadyr
    {  391,  214,  466967,  466979,  466998, 12, 19, 20, }, // Apia
    {  391,  232,  467018,  467041,  467071, 23, 30, 31, }, // Arabian
    {  391,  240,  467102,  467118,  467141, 16, 23, 24, }, // Argentina
    {  391,  250,  467165,  467187,  467216, 22, 29, 30, }, // Argentina_Western
    {  391,  268,  467246,  467261,  467283, 15, 22, 23, }, // Armenia
    {  391,  286,  467306,  467322,  467345, 16, 23, 24, }, // Atlantic
    {  391,  295,  467369,  467397,  467432, 28, 35, 36, }, // Australia_Central
    {  391,  313,  467468,  467504,  467547, 36, 43, 44, }, // Australia_CentralWestern
    {  391,  338,  467591,  467617,  467650, 26, 33, 34, }, // Australia_Eastern
    {  391,  356,  467684,  467709,  467741, 25, 32, 33, }, // Australia_Western
    {  391,  374,  467774,  467793,  467819, 19, 26, 27, }, // Azerbaijan
    {  391,  385,  467846,  467862,  467885, 16, 23, 24, }, // Azores
    {  391,  397,  467909,  467926,  467950, 17, 24, 25, }, // Bangladesh
    {  391,  415,       0,  467975,       0,  0, 13,  0, }, // Bhutan
    {  391,  422,       0,  467988,       0,  0, 15,  0, }, // Bolivia
    {  391,  437,  468003,  468019,  468042, 16, 23, 24, }, // Brasilia
    {  391,  454,       0,  468066,       0,  0, 26,  0, }, // Brunei
    {  391,  461,  468092,  468108,  468131, 16, 23, 24, }, // Cape_Verde
    {  391,  478,       0,  468155,       0,  0, 20,  0, }, // Chamorro
    {  391,  487,  468175,  468188,  468208, 13, 20, 21, }, // Chatham
    {  391,  495,  468229,  468240,  468258, 11, 18, 19, }, // Chile
    {  391,  501,  468277,  468288,  468306, 11, 18, 19, }, // China
    {  391,  507,       0,  468325,       0,  0, 22,  0, }, // Christmas
    {  391,  517,       0,  468347,       0,  0, 27,  0, }, // Cocos
    {  391,  523,  468374,  468390,  468413, 16, 23, 24, }, // Colombia
    {  391,  532,  468437,  468452,  468474, 15, 22, 23, }, // Cook
    {  391,  537,  468497,  468508,  468526, 11, 18, 19, }, // Cuba
    {  391,  548,       0,  468545,       0,  0, 14,  0, }, // Davis
    {  391,  564,       0,  468559,       0,  0, 19,  0, }, // DumontDUrville
    {  391,  601,       0,  468578,       0,  0, 20,  0, }, // East_Timor
    {  391,  612,  468598,  468619,  468647, 21, 28, 29, }, // Easter
    {  391,  619,       0,  468676,       0,  0, 15,  0, }, // Ecuador
    {  391,  627,  468691,  468711,  468738, 20, 27, 28, }, // Europe_Central
    {  391,  642,  468766,  468786,  468813, 20, 27, 28, }, // Europe_Eastern
    {  391,  657,       0,  468841,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  391,  680,  468873,  468892,  468918, 19, 26, 27, }, // Europe_Western
    {  391,  695,  468945,  468975,  469012, 30, 37, 38, }, // Falkland
    {  391,  704,  469050,  469061,  469079, 11, 18, 19, }, // Fiji
    {  391,  709,       0,  469098,       0,  0, 23,  0, }, // French_Guiana
    {  391,  723,       0,  469121,       0,  0, 54,  0, }, // French_Southern
    {  391,  746,       0,  469175,       0,  0, 14,  0, }, // Galapagos
    {  391,  756,       0,  469189,       0,  0, 17,  0, }, // Gambier
    {  391,  764,  469206,  469220,  469241, 14, 21, 22, }, // Georgia
    {  391,  772,       0,  469263,       0,  0, 19,  0, }, // Gilbert_Islands
    {  391,  788,       0,  469282,       0,  0, 14,  0, }, // GMT
    {  391,  830,  469296,  469319,  469349, 23, 30, 31, }, // Greenland_Eastern
    {  391,  848,  469380,  469402,  469431, 22, 29, 30, }, // Greenland_Western
    {  391,  871,       0,  469461,       0,  0, 19,  0, }, // Gulf
    {  391,  876,       0,  469480,       0,  0, 13,  0, }, // Guyana
    {  391,  883,  469493,  469512,  469538, 19, 26, 27, }, // Hawaii_Aleutian
    {  391,  899,  469565,  469580,  469602, 15, 22, 23, }, // Hong_Kong
    {  391,  909,  469625,  469637,  469656, 12, 19, 20, }, // Hovd
    {  391,  914,       0,  469676,       0,  0, 20,  0, }, // India
    {  391,  920,       0,  469696,       0,  0, 20,  0, }, // Indian_Ocean
    {  391,  933,       0,  469716,       0,  0, 15,  0, }, // Indochina
    {  391,  943,       0,  469731,       0,  0, 24,  0, }, // Indonesia_Central
    {  391,  961,       0,  469755,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  391,  979,       0,  469779,       0,  0, 23,  0, }, // Indonesia_Western
    {  391,  997,  469802,  469814,  469833, 12, 19, 20, }, // Iran
    {  391, 1008,  469853,  469868,  469890, 15, 22, 23, }, // Irkutsk
    {  391, 1016,  469913,  469927,  469948, 14, 21, 22, }, // Israel
    {  391, 1023,  469970,  469983,  470003, 13, 20, 21, }, // Japan
    {  391, 1029,  470024,  470056,  470095, 32, 39, 40, }, // Kamchatka
    {  391, 1047,       0,  470135,       0,  0, 17,  0, }, // Kazakhstan
    {  391, 1058,       0,  470152,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  391, 1077,       0,  470176,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  391, 1106,  470199,  470212,  470232, 13, 20, 21, }, // Korea
    {  391, 1112,       0,  470253,       0,  0, 12,  0, }, // Kosrae
    {  391, 1119,  470265,  470284,  470310, 19, 26, 27, }, // Krasnoyarsk
    {  391, 1151,       0,  470337,       0,  0, 18,  0, }, // Kyrgystan
    {  391, 1175,       0,  470355,       0,  0, 17,  0, }, // Line_Islands
    {  391, 1188,  470372,  470394,  470423, 22, 29, 30, }, // Lord_Howe
    {  391, 1204,  470453,  470468,  470490, 15, 22, 23, }, // Magadan
    {  391, 1219,       0,  470513,       0,  0, 16,  0, }, // Malaysia
    {  391, 1228,       0,  470529,       0,  0, 16,  0, }, // Maldives
    {  391, 1237,       0,  470545,       0,  0, 18,  0, }, // Marquesas
    {  391, 1247,       0,  470563,       0,  0, 18,  0, }, // Marshall_Islands
    {  391, 1264,  470581,  470597,  470620, 16, 23, 24, }, // Mauritius
    {  391, 1274,       0,  470644,       0,  0, 13,  0, }, // Mawson
    {  391, 1281,  470657,  470695,  470740, 38, 45, 46, }, // Mexico_Pacific
    {  391, 1296,  470786,  470803,  470827, 17, 24, 25, }, // Mongolia
    {  391, 1305,  470852,  470866,  470887, 14, 21, 22, }, // Moscow
    {  391, 1312,       0,  470909,       0,  0, 13,  0, }, // Myanmar
    {  391, 1320,       0,  470922,       0,  0, 11,  0, }, // Nauru
    {  391, 1326,       0,  470933,       0,  0, 13,  0, }, // Nepal
    {  391, 1332,  470946,  470967,  470995, 21, 28, 29, }, // New_Caledonia
    {  391, 1346,  471024,  471042,  471067, 18, 25, 26, }, // New_Zealand
    {  391, 1358,  471093,  471112,  471138, 19, 26, 27, }, // Newfoundland
    {  391, 1371,       0,  471165,       0,  0, 11,  0, }, // Niue
    {  391, 1376,  471176,  471197,  471225, 21, 28, 29, }, // Norfolk
    {  391, 1384,  471254,  471279,  471311, 25, 32, 33, }, // Noronha
    {  391, 1406,  471344,  471363,  471389, 19, 26, 27, }, // Novosibirsk
    {  391, 1418,  471416,  471428,  471447, 12, 19, 20, }, // Omsk
    {  391, 1428,  471467,  471483,  471506, 16, 23, 24, }, // Pakistan
    {  391, 1437,       0,  471530,       0,  0, 11,  0, }, // Palau
    {  391, 1443,       0,  471541,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  391, 1460,  471565,  471581,  471604, 16, 23, 24, }, // Paraguay
    {  391, 1469,  471628,  471638,  471655, 10, 17, 18, }, // Peru
    {  391, 1474,  471673,  471687,  471708, 14, 21, 22, }, // Philippines
    {  391, 1486,       0,  471730,       0,  0, 18,  0, }, // Phoenix_Islands
    {  391, 1502,  471748,  471776,  471811, 28, 35, 36, }, // Pierre_Miquelon
    {  391, 1518,       0,  471847,       0,  0, 15,  0, }, // Pitcairn
    {  391, 1527,       0,  471862,       0,  0, 12,  0, }, // Ponape
    {  391, 1534,       0,  471874,       0,  0, 15,  0, }, // Pyongyang
    {  391, 1554,       0,  471889,       0,  0, 15,  0, }, // Reunion
    {  391, 1562,       0,  471904,       0,  0, 13,  0, }, // Rothera
    {  391, 1570,  471917,  471932,  471954, 15, 22, 23, }, // Sakhalin
    {  391, 1579,  471977,  471990,  472010, 13, 20, 21, }, // Samara
    {  391, 1596,  472031,  472042,  472060, 11, 18, 19, }, // Samoa
    {  391, 1602,       0,  472079,       0,  0, 18,  0, }, // Seychelles
    {  391, 1624,       0,  472097,       0,  0, 16,  0, }, // Singapore
    {  391, 1634,       0,  472113,       0,  0, 19,  0, }, // Solomon
    {  391, 1642,       0,  472132,       0,  0, 23,  0, }, // South_Georgia
    {  391, 1656,       0,  472155,       0,  0, 15,  0, }, // Suriname
    {  391, 1676,       0,  472170,       0,  0, 11,  0, }, // Syowa
    {  391, 1682,       0,  472181,       0,  0, 11,  0, }, // Tahiti
    {  391, 1689,  472192,  472204,  472223, 12, 19, 20, }, // Taipei
    {  391, 1696,       0,  472243,       0,  0, 19,  0, }, // Tajikistan
    {  391, 1724,       0,  472262,       0,  0, 13,  0, }, // Tokelau
    {  391, 1732,  472275,  472287,  472306, 12, 19, 20, }, // Tonga
    {  391, 1738,       0,  472326,       0,  0, 11,  0, }, // Truk
    {  391, 1750,  472337,  472357,  472384, 20, 27, 28, }, // Turkmenistan
    {  391, 1763,       0,  472412,       0,  0, 12,  0, }, // Tuvalu
    {  391, 1777,  472424,  472439,  472461, 15, 22, 23, }, // Uruguay
    {  391, 1792,  472484,  472502,  472527, 18, 25, 26, }, // Uzbekistan
    {  391, 1803,  472553,  472566,  472586, 13, 20, 21, }, // Vanuatu
    {  391, 1811,       0,  472607,       0,  0, 16,  0, }, // Venezuela
    {  391, 1821,  472623,  472642,  472668, 19, 26, 27, }, // Vladivostok
    {  391, 1833,  472695,  472712,  472736, 17, 24, 25, }, // Volgograd
    {  391, 1843,       0,  472761,       0,  0, 14,  0, }, // Vostok
    {  391, 1850,       0,  472775,       0,  0, 17,  0, }, // Wake
    {  391, 1855,       0,  472792,       0,  0, 24,  0, }, // Wallis
    {  391, 1862,  472816,  472831,  472853, 15, 22, 23, }, // Yakutsk
    {  391, 1870,  472876,  472897,  472925, 21, 28, 29, }, // Yekaterinburg
    {  391, 1892,       0,  472954,       0,  0, 13,  0, }, // Yukon
    {  392,   17,       0,  472967,       0,  0, 20,  0, }, // Africa_Central Lingala/Latin/Congo - Kinshasa
    {  392,   32,       0,  472987,       0,  0, 26,  0, }, // Africa_Eastern
    {  392,   65,       0,  473013,       0,  0, 24,  0, }, // Africa_Southern
    {  392,  788,       0,  473037,       0,  0, 17,  0, }, // GMT
    {  392, 1602,       0,  473054,       0,  0, 17,  0, }, // Seychelles
    {  396,    0,  473071,  473081,  473104, 10, 23, 18, }, // Acre Lithuanian/Latin/Lithuania
    {  396,    5,       0,  473122,       0,  0, 18,  0, }, // Afghanistan
    {  396,   17,       0,  473140,       0,  0, 24,  0, }, // Africa_Central
    {  396,   32,       0,  473164,       0,  0, 19,  0, }, // Africa_Eastern
    {  396,   65,       0,  473183,       0,  0, 20,  0, }, // Africa_Southern
    {  396,   81,  473203,  473224,  473252, 21, 28, 29, }, // Africa_Western
    {  396,  107,  473281,  473296,  473318, 15, 22, 23, }, // Alaska
    {  396,  128,  473341,  473355,  473376, 14, 21, 22, }, // Almaty
    {  396,  135,  473398,  473413,  473435, 15, 22, 23, }, // Amazon
    {  396,  142,  473458,  473488,  473525, 30, 37, 38, }, // America_Central
    {  396,  158,  473563,  473591,  473626, 28, 35, 36, }, // America_Eastern
    {  396,  174,  473662,  473691,  473727, 29, 36, 37, }, // America_Mountain
    {  396,  191,  473764,  473805,  473853, 41, 48, 49, }, // America_Pacific
    {  396,  207,  473902,  473917,  473939, 15, 22, 23, }, // Anadyr
    {  396,  214,  473962,  473975,  473995, 13, 20, 21, }, // Apia
    {  396,  219,  474016,  474028,  474047, 12, 19, 20, }, // Aqtau
    {  396,  225,  474067,  474081,  474102, 14, 21, 22, }, // Aqtobe
    {  396,  232,  474124,  474139,  474161, 15, 22, 23, }, // Arabian
    {  396,  240,  474184,  474201,  474225, 17, 24, 25, }, // Argentina
    {  396,  250,  474250,  474274,  474305, 24, 31, 32, }, // Argentina_Western
    {  396,  268,  474337,  474353,  474376, 16, 23, 24, }, // Armenia
    {  396,  286,  474400,  474414,  474435, 14, 21, 22, }, // Atlantic
    {  396,  295,  474457,  474485,  474520, 28, 35, 36, }, // Australia_Central
    {  396,  313,  474556,  474591,  474633, 35, 42, 43, }, // Australia_CentralWestern
    {  396,  338,  474676,  474699,  474729, 23, 30, 31, }, // Australia_Eastern
    {  396,  356,  474760,  474785,  474817, 25, 32, 33, }, // Australia_Western
    {  396,  374,  474850,  474869,  474895, 19, 26, 27, }, // Azerbaijan
    {  396,  385,  474922,  474939,  474963, 17, 24, 25, }, // Azores
    {  396,  397,  474988,  475005,  475029, 17, 24, 25, }, // Bangladesh
    {  396,  415,       0,  475054,       0,  0, 13,  0, }, // Bhutan
    {  396,  422,       0,  475067,       0,  0, 16,  0, }, // Bolivia
    {  396,  437,  475083,  475100,  475124, 17, 24, 25, }, // Brasilia
    {  396,  454,       0,  475149,       0,  0, 27,  0, }, // Brunei
    {  396,  461,  475176,  475198,  475227, 22, 29, 30, }, // Cape_Verde
    {  396,  472,       0,  475257,       0,  0, 13,  0, }, // Casey
    {  396,  478,       0,  475270,       0,  0, 13,  0, }, // Chamorro
    {  396,  487,  475283,  475296,  475316, 13, 20, 21, }, // Chatham
    {  396,  495,  475337,  475349,  475368, 12, 19, 20, }, // Chile
    {  396,  501,  475388,  475402,  475423, 14, 21, 22, }, // China
    {  396,  507,       0,  475445,       0,  0, 19,  0, }, // Christmas
    {  396,  517,       0,  475464,       0,  0, 18,  0, }, // Cocos
    {  396,  523,  475482,  475499,  475523, 17, 24, 25, }, // Colombia
    {  396,  532,  475548,  475564,  475587, 16, 23, 30, }, // Cook
    {  396,  537,  475617,  475629,  475648, 12, 19, 20, }, // Cuba
    {  396,  548,       0,  475668,       0,  0, 14,  0, }, // Davis
    {  396,  564,       0,  475682,       0,  0, 24,  0, }, // DumontDUrville
    {  396,  601,       0,  475706,       0,  0, 18,  0, }, // East_Timor
    {  396,  612,  475724,  475743,  475769, 19, 26, 27, }, // Easter
    {  396,  619,       0,  475796,       0,  0, 15,  0, }, // Ecuador
    {  396,  627,  475811,  475833,  475862, 22, 29, 30, }, // Europe_Central
    {  396,  642,  475892,  475911,  475937, 19, 26, 27, }, // Europe_Eastern
    {  396,  657,       0,  475964,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {  396,  680,  475992,  476013,  476041, 21, 28, 29, }, // Europe_Western
    {  396,  695,  476070,  476091,  476119, 21, 28, 29, }, // Falkland
    {  396,  704,  476148,  476161,  476181, 13, 20, 21, }, // Fiji
    {  396,  709,       0,  476202,       0,  0, 26,  0, }, // French_Guiana
    {  396,  723,       0,  476228,       0,  0, 39,  0, }, // French_Southern
    {  396,  746,       0,  476267,       0,  0, 15,  0, }, // Galapagos
    {  396,  756,       0,  476282,       0,  0, 14,  0, }, // Gambier
    {  396,  764,  476296,  476311,  476333, 15, 22, 23, }, // Georgia
    {  396,  772,       0,  476356,       0,  0, 20,  0, }, // Gilbert_Islands
    {  396,  788,       0,  476376,       0,  0, 15,  0, }, // GMT
    {  396,  830,  476391,  476415,  476446, 24, 31, 32, }, // Greenland_Eastern
    {  396,  848,  476478,  476504,  476537, 26, 33, 34, }, // Greenland_Western
    {  396,  866,       0,  476571,       0,  0, 12,  0, }, // Guam
    {  396,  871,       0,  476583,       0,  0, 23,  0, }, // Gulf
    {  396,  876,       0,  476606,       0,  0, 14,  0, }, // Guyana
    {  396,  883,  476620,  476640,  476667, 20, 27, 28, }, // Hawaii_Aleutian
    {  396,  899,  476695,  476710,  476732, 15, 22, 23, }, // Hong_Kong
    {  396,  909,  476755,  476767,  476786, 12, 19, 20, }, // Hovd
    {  396,  914,       0,  476806,       0,  0, 14,  0, }, // India
    {  396,  920,       0,  476820,       0,  0, 24,  0, }, // Indian_Ocean
    {  396,  933,       0,  476844,       0,  0, 18,  0, }, // Indochina
    {  396,  943,       0,  476862,       0,  0, 28,  0, }, // Indonesia_Central
    {  396,  961,       0,  476890,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  396,  979,       0,  476913,       0,  0, 25,  0, }, // Indonesia_Western
    {  396,  997,  476938,  476950,  476969, 12, 19, 20, }, // Iran
    {  396, 1008,  476989,  477004,  477026, 15, 22, 23, }, // Irkutsk
    {  396, 1016,  477049,  477064,  477086, 15, 22, 23, }, // Israel
    {  396, 1023,  477109,  477125,  477148, 16, 23, 24, }, // Japan
    {  396, 1029,  477172,  477204,  477243, 32, 39, 40, }, // Kamchatka
    {  396, 1047,       0,  477283,       0,  0, 18,  0, }, // Kazakhstan
    {  396, 1058,       0,  477301,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  396, 1077,       0,  477324,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  396, 1106,  477349,  477363,  477384, 14, 21, 22, }, // Korea
    {  396, 1112,       0,  477406,       0,  0, 14,  0, }, // Kosrae
    {  396, 1119,  477420,  477439,  477465, 19, 26, 27, }, // Krasnoyarsk
    {  396, 1151,       0,  477492,       0,  0, 17,  0, }, // Kyrgystan
    {  396, 1161,       0,  477509,       0,  0, 13,  0, }, // Lanka
    {  396, 1175,       0,  477522,       0,  0, 17,  0, }, // Line_Islands
    {  396, 1188,  477539,  477555,  477578, 16, 23, 24, }, // Lord_Howe
    {  396, 1198,  477602,  477614,  477633, 12, 19, 20, }, // Macau
    {  396, 1204,  477653,  477668,  477690, 15, 22, 23, }, // Magadan
    {  396, 1219,       0,  477713,       0,  0, 17,  0, }, // Malaysia
    {  396, 1228,       0,  477730,       0,  0, 14,  0, }, // Maldives
    {  396, 1237,       0,  477744,       0,  0, 19,  0, }, // Marquesas
    {  396, 1247,       0,  477763,       0,  0, 19,  0, }, // Marshall_Islands
    {  396, 1264,  477782,  477800,  477825, 18, 25, 26, }, // Mauritius
    {  396, 1274,       0,  477851,       0,  0, 13,  0, }, // Mawson
    {  396, 1281,  477864,  477897,  477937, 33, 40, 41, }, // Mexico_Pacific
    {  396, 1296,  477978,  477996,  478021, 18, 25, 26, }, // Mongolia
    {  396, 1305,  478047,  478061,  478082, 14, 21, 22, }, // Moscow
    {  396, 1312,       0,  478104,       0,  0, 15,  0, }, // Myanmar
    {  396, 1320,       0,  478119,       0,  0, 12,  0, }, // Nauru
    {  396, 1326,       0,  478131,       0,  0, 13,  0, }, // Nepal
    {  396, 1332,  478144,  478172,  478207, 28, 35, 36, }, // New_Caledonia
    {  396, 1346,  478243,  478270,  478304, 27, 34, 35, }, // New_Zealand
    {  396, 1358,  478339,  478359,  478386, 20, 27, 28, }, // Newfoundland
    {  396, 1371,       0,  478414,       0,  0, 12,  0, }, // Niue
    {  396, 1376,  478426,  478446,  478473, 20, 27, 28, }, // Norfolk
    {  396, 1384,  478501,  478528,  478562, 27, 34, 35, }, // Noronha
    {  396, 1392,       0,  478597,       0,  0, 28,  0, }, // North_Mariana
    {  396, 1406,  478625,  478644,  478670, 19, 26, 27, }, // Novosibirsk
    {  396, 1418,  478697,  478709,  478728, 12, 19, 20, }, // Omsk
    {  396, 1428,  478748,  478764,  478787, 16, 23, 24, }, // Pakistan
    {  396, 1437,       0,  478811,       0,  0, 12,  0, }, // Palau
    {  396, 1443,       0,  478823,       0,  0, 31,  0, }, // Papua_New_Guinea
    {  396, 1460,  478854,  478872,  478897, 18, 25, 26, }, // Paraguay
    {  396, 1469,  478923,  478934,  478952, 11, 18, 19, }, // Peru
    {  396, 1474,  478971,  478986,  479008, 15, 22, 23, }, // Philippines
    {  396, 1486,       0,  479031,       0,  0, 19,  0, }, // Phoenix_Islands
    {  396, 1502,  479050,  479078,  479113, 28, 35, 36, }, // Pierre_Miquelon
    {  396, 1518,       0,  479149,       0,  0, 15,  0, }, // Pitcairn
    {  396, 1527,       0,  479164,       0,  0, 14,  0, }, // Ponape
    {  396, 1534,       0,  479178,       0,  0, 16,  0, }, // Pyongyang
    {  396, 1544,  479194,  479211,  479235, 17, 24, 25, }, // Qyzylorda
    {  396, 1554,       0,  479260,       0,  0, 15,  0, }, // Reunion
    {  396, 1562,       0,  479275,       0,  0, 14,  0, }, // Rothera
    {  396, 1570,  479289,  479305,  479328, 16, 23, 24, }, // Sakhalin
    {  396, 1579,  479352,  479366,  479387, 14, 21, 22, }, // Samara
    {  396, 1596,  479409,  479421,  479440, 12, 19, 20, }, // Samoa
    {  396, 1602,       0,  479460,       0,  0, 15,  0, }, // Seychelles
    {  396, 1624,       0,  479475,       0,  0, 16,  0, }, // Singapore
    {  396, 1634,       0,  479491,       0,  0, 21,  0, }, // Solomon
    {  396, 1642,       0,  479512,       0,  0, 23,  0, }, // South_Georgia
    {  396, 1656,       0,  479535,       0,  0, 15,  0, }, // Suriname
    {  396, 1676,       0,  479550,       0,  0, 13,  0, }, // Syowa
    {  396, 1682,       0,  479563,       0,  0, 14,  0, }, // Tahiti
    {  396, 1689,  479577,  479593,  479616, 16, 23, 24, }, // Taipei
    {  396, 1696,       0,  479640,       0,  0, 19,  0, }, // Tajikistan
    {  396, 1724,       0,  479659,       0,  0, 14,  0, }, // Tokelau
    {  396, 1732,  479673,  479686,  479706, 13, 20, 21, }, // Tonga
    {  396, 1738,       0,  479727,       0,  0, 11,  0, }, // Truk
    {  396, 1750,  479738,  479758,  479785, 20, 27, 28, }, // Turkmenistan
    {  396, 1763,       0,  479813,       0,  0, 13,  0, }, // Tuvalu
    {  396, 1777,  479826,  479843,  479867, 17, 24, 25, }, // Uruguay
    {  396, 1792,  479892,  479910,  479935, 18, 25, 26, }, // Uzbekistan
    {  396, 1803,  479961,  479975,  479996, 14, 21, 22, }, // Vanuatu
    {  396, 1811,       0,  480018,       0,  0, 17,  0, }, // Venezuela
    {  396, 1821,  480035,  480054,  480080, 19, 26, 27, }, // Vladivostok
    {  396, 1833,  480107,  480124,  480148, 17, 24, 25, }, // Volgograd
    {  396, 1843,       0,  480173,       0,  0, 14,  0, }, // Vostok
    {  396, 1850,       0,  480187,       0,  0, 18,  0, }, // Wake
    {  396, 1855,       0,  480205,       0,  0, 24,  0, }, // Wallis
    {  396, 1862,  480229,  480244,  480266, 15, 22, 23, }, // Yakutsk
    {  396, 1870,  480289,  480310,  480338, 21, 28, 29, }, // Yekaterinburg
    {  396, 1892,       0,  480367,       0,  0, 13,  0, }, // Yukon
    {  397,  523,  480380,  480402,  480430, 22, 28, 25, }, // Colombia Lojban/Latin/world
    {  397,  746,       0,  480455,       0,  0, 21,  0, }, // Galapagos
    {  398,    5,       0,  480476,       0,  0, 14,  0, }, // Afghanistan Lower Sorbian/Latin/Germany
    {  398,   17,       0,  480490,       0,  0, 18,  0, }, // Africa_Central
    {  398,   32,       0,  480508,       0,  0, 21,  0, }, // Africa_Eastern
    {  398,   65,       0,  480529,       0,  0, 23,  0, }, // Africa_Southern
    {  398,   81,  480552,  480574,  480607, 22, 33, 31, }, // Africa_Western
    {  398,  107,  480638,  480652,  480677, 14, 25, 23, }, // Alaska
    {  398,  135,  480700,  480714,  480739, 14, 25, 23, }, // Amazon
    {  398,  142,  480762,  480795,  480839, 33, 44, 42, }, // America_Central
    {  398,  158,  480881,  480915,  480960, 34, 45, 43, }, // America_Eastern
    {  398,  174,  481003,  481033,  481074, 30, 41, 39, }, // America_Mountain
    {  398,  191,  481113,  481146,  481190, 33, 44, 42, }, // America_Pacific
    {  398,  214,  481232,  481243,  481265, 11, 22, 20, }, // Apia
    {  398,  232,  481285,  481297,  481320, 12, 23, 21, }, // Arabian
    {  398,  240,  481341,  481356,  481382, 15, 26, 24, }, // Argentina
    {  398,  250,  481406,  481432,  481469, 26, 37, 35, }, // Argentina_Western
    {  398,  268,  481504,  481516,  481539, 12, 23, 21, }, // Armenia
    {  398,  286,  481560,  481574,  481599, 14, 25, 23, }, // Atlantic
    {  398,  295,  481622,  481643,  481675, 21, 32, 30, }, // Australia_Central
    {  398,  313,  481705,  481738,  481782, 33, 44, 42, }, // Australia_CentralWestern
    {  398,  338,  481824,  481848,  481883, 24, 35, 33, }, // Australia_Eastern
    {  398,  356,  481916,  481941,  481977, 25, 36, 34, }, // Australia_Western
    {  398,  374,  482011,  482030,  482060, 19, 30, 28, }, // Azerbaijan
    {  398,  385,  482088,  482099,  482121, 11, 22, 20, }, // Azores
    {  398,  397,  482141,  482157,  482184, 16, 27, 25, }, // Bangladesh
    {  398,  415,       0,  482209,       0,  0, 13,  0, }, // Bhutan
    {  398,  422,       0,  482222,       0,  0, 13,  0, }, // Bolivia
    {  398,  437,  482235,  482247,  482270, 12, 23, 21, }, // Brasilia
    {  398,  454,       0,  482291,       0,  0, 13,  0, }, // Brunei
    {  398,  461,  482304,  482318,  482343, 14, 25, 23, }, // Cape_Verde
    {  398,  478,       0,  482366,       0,  0, 14,  0, }, // Chamorro
    {  398,  487,  482380,  482394,  482419, 14, 25, 23, }, // Chatham
    {  398,  495,  482442,  482453,  482475, 11, 22, 20, }, // Chile
    {  398,  501,  482495,  482506,  482528, 11, 22, 20, }, // China
    {  398,  507,       0,  482548,       0,  0, 19,  0, }, // Christmas
    {  398,  517,       0,  482567,       0,  0, 20,  0, }, // Cocos
    {  398,  523,  482587,  482601,  482626, 14, 25, 23, }, // Colombia
    {  398,  532,  482649,  482668,  482698, 19, 30, 28, }, // Cook
    {  398,  537,  482726,  482738,  482761, 12, 23, 21, }, // Cuba
    {  398,  548,       0,  482782,       0,  0,  9,  0, }, // Davis
    {  398,  564,       0,  482791,       0,  0, 18,  0, }, // DumontDUrville
    {  398,  601,       0,  482809,       0,  0, 22,  0, }, // East_Timor
    {  398,  612,  482831,  482850,  482880, 19, 30, 28, }, // Easter
    {  398,  619,       0,  482908,       0,  0, 14,  0, }, // Ecuador
    {  398,  627,  482922,  482944,  482977, 22, 33, 31, }, // Europe_Central
    {  398,  642,  483008,  483032,  483067, 24, 35, 33, }, // Europe_Eastern
    {  398,  657,       0,  483100,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {  398,  680,  483118,  483143,  483179, 25, 36, 34, }, // Europe_Western
    {  398,  695,  483213,  483228,  483254, 15, 26, 24, }, // Falkland
    {  398,  704,  483278,  483290,  483313, 12, 23, 21, }, // Fiji
    {  398,  709,       0,  483334,       0,  0, 22,  0, }, // French_Guiana
    {  398,  723,       0,  483356,       0,  0, 59,  0, }, // French_Southern
    {  398,  746,       0,  483415,       0,  0, 15,  0, }, // Galapagos
    {  398,  756,       0,  483430,       0,  0, 14,  0, }, // Gambier
    {  398,  764,  483444,  483457,  483481, 13, 24, 22, }, // Georgia
    {  398,  772,       0,  483503,       0,  0, 22,  0, }, // Gilbert_Islands
    {  398,  788,       0,  483525,       0,  0, 16,  0, }, // GMT
    {  398,  830,  483541,  483566,  483602, 25, 36, 34, }, // Greenland_Eastern
    {  398,  848,  483636,  483662,  483699, 26, 37, 35, }, // Greenland_Western
    {  398,  871,       0,  483734,       0,  0, 20,  0, }, // Gulf
    {  398,  876,       0,  483754,       0,  0, 12,  0, }, // Guyana
    {  398,  883,  483766,  483788,  483821, 22, 33, 31, }, // Hawaii_Aleutian
    {  398,  899,  483852,  483867,  483893, 15, 26, 24, }, // Hong_Kong
    {  398,  909,  483917,  483929,  483952, 12, 23, 21, }, // Hovd
    {  398,  914,       0,  483973,       0,  0, 11,  0, }, // India
    {  398,  920,       0,  483984,       0,  0, 20,  0, }, // Indian_Ocean
    {  398,  933,       0,  484004,       0,  0, 15,  0, }, // Indochina
    {  398,  943,       0,  484019,       0,  0, 20,  0, }, // Indonesia_Central
    {  398,  961,       0,  484039,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  398,  979,       0,  484058,       0,  0, 24,  0, }, // Indonesia_Western
    {  398,  997,  484082,  484093,  484115, 11, 22, 20, }, // Iran
    {  398, 1008,  484135,  484147,  484170, 12, 23, 21, }, // Irkutsk
    {  398, 1016,  484191,  484204,  484228, 13, 24, 22, }, // Israel
    {  398, 1023,  484250,  484262,  484285, 12, 23, 21, }, // Japan
    {  398, 1047,       0,  484306,       0,  0, 14,  0, }, // Kazakhstan
    {  398, 1058,       0,  484320,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  398, 1077,       0,  484343,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  398, 1106,  484367,  484379,  484402, 12, 23, 21, }, // Korea
    {  398, 1112,       0,  484423,       0,  0, 13,  0, }, // Kosrae
    {  398, 1119,  484436,  484452,  484479, 16, 27, 25, }, // Krasnoyarsk
    {  398, 1151,       0,  484504,       0,  0, 12,  0, }, // Kyrgystan
    {  398, 1175,       0,  484516,       0,  0, 20,  0, }, // Line_Islands
    {  398, 1188,  484536,  484554,  484583, 18, 29, 27, }, // Lord_Howe
    {  398, 1204,  484610,  484624,  484649, 14, 25, 23, }, // Magadan
    {  398, 1219,       0,  484672,       0,  0, 14,  0, }, // Malaysia
    {  398, 1228,       0,  484686,       0,  0, 14,  0, }, // Maldives
    {  398, 1237,       0,  484700,       0,  0, 13,  0, }, // Marquesas
    {  398, 1247,       0,  484713,       0,  0, 23,  0, }, // Marshall_Islands
    {  398, 1264,  484736,  484750,  484775, 14, 25, 23, }, // Mauritius
    {  398, 1274,       0,  484798,       0,  0, 10,  0, }, // Mawson
    {  398, 1281,  484808,  484829,  484861, 21, 32, 30, }, // Mexico_Pacific
    {  398, 1296,  484891,  484908,  484936, 17, 28, 26, }, // Mongolia
    {  398, 1305,  484962,  484975,  484999, 13, 24, 22, }, // Moscow
    {  398, 1312,       0,  485021,       0,  0, 14,  0, }, // Myanmar
    {  398, 1320,       0,  485035,       0,  0, 12,  0, }, // Nauru
    {  398, 1326,       0,  485047,       0,  0, 12,  0, }, // Nepal
    {  398, 1332,  485059,  485078,  485108, 19, 30, 28, }, // New_Caledonia
    {  398, 1346,  485136,  485154,  485183, 18, 29, 27, }, // New_Zealand
    {  398, 1358,  485210,  485229,  485259, 19, 30, 28, }, // Newfoundland
    {  398, 1371,       0,  485287,       0,  0, 11,  0, }, // Niue
    {  398, 1376,  485298,  485314,  485341, 16, 27, 25, }, // Norfolk
    {  398, 1384,  485366,  485389,  485423, 23, 34, 32, }, // Noronha
    {  398, 1406,  485455,  485471,  485498, 16, 27, 25, }, // Novosibirsk
    {  398, 1418,  485523,  485532,  485552,  9, 20, 18, }, // Omsk
    {  398, 1428,  485570,  485585,  485611, 15, 26, 24, }, // Pakistan
    {  398, 1437,       0,  485635,       0,  0, 12,  0, }, // Palau
    {  398, 1443,       0,  485647,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  398, 1460,  485669,  485684,  485710, 15, 26, 24, }, // Paraguay
    {  398, 1469,  485734,  485745,  485767, 11, 22, 20, }, // Peru
    {  398, 1474,  485787,  485801,  485826, 14, 25, 23, }, // Philippines
    {  398, 1486,       0,  485849,       0,  0, 22,  0, }, // Phoenix_Islands
    {  398, 1502,  485871,  485899,  485938, 28, 39, 37, }, // Pierre_Miquelon
    {  398, 1518,       0,  485975,       0,  0, 23,  0, }, // Pitcairn
    {  398, 1527,       0,  485998,       0,  0, 12,  0, }, // Ponape
    {  398, 1534,       0,  486010,       0,  0, 16,  0, }, // Pyongyang
    {  398, 1554,       0,  486026,       0,  0, 14,  0, }, // Reunion
    {  398, 1562,       0,  486040,       0,  0, 11,  0, }, // Rothera
    {  398, 1570,  486051,  486066,  486092, 15, 26, 24, }, // Sakhalin
    {  398, 1596,  486116,  486128,  486151, 12, 23, 21, }, // Samoa
    {  398, 1602,       0,  486172,       0,  0, 14,  0, }, // Seychelles
    {  398, 1624,       0,  486186,       0,  0, 15,  0, }, // Singapore
    {  398, 1634,       0,  486201,       0,  0, 14,  0, }, // Solomon
    {  398, 1642,       0,  486215,       0,  0, 25,  0, }, // South_Georgia
    {  398, 1656,       0,  486240,       0,  0, 14,  0, }, // Suriname
    {  398, 1676,       0,  486254,       0,  0,  9,  0, }, // Syowa
    {  398, 1682,       0,  486263,       0,  0, 13,  0, }, // Tahiti
    {  398, 1689,  486276,  486291,  486317, 15, 26, 24, }, // Taipei
    {  398, 1696,       0,  486341,       0,  0, 14,  0, }, // Tajikistan
    {  398, 1724,       0,  486355,       0,  0, 14,  0, }, // Tokelau
    {  398, 1732,  486369,  486381,  486404, 12, 23, 21, }, // Tonga
    {  398, 1738,       0,  486425,       0,  0, 12,  0, }, // Truk
    {  398, 1750,  486437,  486452,  486478, 15, 26, 24, }, // Turkmenistan
    {  398, 1763,       0,  486502,       0,  0, 12,  0, }, // Tuvalu
    {  398, 1777,  486514,  486528,  486553, 14, 25, 23, }, // Uruguay
    {  398, 1792,  486576,  486589,  486613, 13, 24, 22, }, // Uzbekistan
    {  398, 1803,  486635,  486648,  486672, 13, 24, 22, }, // Vanuatu
    {  398, 1811,       0,  486694,       0,  0, 15,  0, }, // Venezuela
    {  398, 1821,  486709,  486727,  486756, 18, 29, 27, }, // Vladivostok
    {  398, 1833,  486783,  486799,  486826, 16, 27, 25, }, // Volgograd
    {  398, 1843,       0,  486851,       0,  0, 10,  0, }, // Vostok
    {  398, 1850,       0,  486861,       0,  0, 13,  0, }, // Wake
    {  398, 1855,       0,  486874,       0,  0, 25,  0, }, // Wallis
    {  398, 1862,  486899,  486911,  486934, 12, 23, 21, }, // Yakutsk
    {  398, 1870,  486955,  486975,  487006, 20, 31, 29, }, // Yekaterinburg
    {  398, 1892,       0,  487035,       0,  0, 12,  0, }, // Yukon
    {  399,   17,       0,  487047,       0,  0, 24,  0, }, // Africa_Central Low German/Latin/Germany
    {  399,   32,       0,  487071,       0,  0, 20,  0, }, // Africa_Eastern
    {  399,   65,       0,  487091,       0,  0, 20,  0, }, // Africa_Southern
    {  399,   81,  487111,  487131,  487159, 20, 28, 26, }, // Africa_Western
    {  399,  142,  487185,  487215,  487254, 30, 39, 37, }, // America_Central
    {  399,  158,  487291,  487320,  487357, 29, 37, 35, }, // America_Eastern
    {  399,  174,  487392,  487418,  487453, 26, 35, 33, }, // America_Mountain
    {  399,  191,  487486,  487515,  487553, 29, 38, 36, }, // America_Pacific
    {  399,  232,  487589,  487602,  487623, 13, 21, 19, }, // Arabian
    {  399,  286,  487642,  487672,  487711, 30, 39, 37, }, // Atlantic
    {  399,  295,  487748,  487772,  487804, 24, 32, 30, }, // Australia_Central
    {  399,  313,  487834,  487862,  487898, 28, 36, 34, }, // Australia_CentralWestern
    {  399,  338,  487932,  487952,  487980, 20, 28, 26, }, // Australia_Eastern
    {  399,  356,  488006,  488026,  488054, 20, 28, 26, }, // Australia_Western
    {  399,  501,  488080,  488090,  488108, 10, 18, 16, }, // China
    {  399,  627,  488124,  488147,  488178, 23, 31, 29, }, // Europe_Central
    {  399,  642,  488207,  488226,  488253, 19, 27, 25, }, // Europe_Eastern
    {  399,  680,  488278,  488297,  488324, 19, 27, 25, }, // Europe_Western
    {  399,  788,       0,  488349,       0,  0, 19,  0, }, // GMT
    {  399,  914,       0,  488368,       0,  0, 11,  0, }, // India
    {  399,  933,       0,  488379,       0,  0, 22,  0, }, // Indochina
    {  399,  943,       0,  488401,       0,  0, 23,  0, }, // Indonesia_Central
    {  399,  961,       0,  488424,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  399,  979,       0,  488443,       0,  0, 19,  0, }, // Indonesia_Western
    {  399, 1016,  488462,  488473,  488492, 11, 19, 17, }, // Israel
    {  399, 1023,  488509,  488523,  488545, 14, 22, 20, }, // Japan
    {  399, 1106,  488565,  488580,  488603, 15, 23, 21, }, // Korea
    {  399, 1305,  488624,  488635,  488654, 11, 19, 17, }, // Moscow
    {  405,    0,  488671,  488680,  488695,  9, 15, 15, }, // Acre Luxembourgish/Latin/Luxembourg
    {  405,    5,       0,  488710,       0,  0, 16,  0, }, // Afghanistan
    {  405,   17,       0,  488726,       0,  0, 23,  0, }, // Africa_Central
    {  405,   32,       0,  488749,       0,  0, 19,  0, }, // Africa_Eastern
    {  405,   65,       0,  488768,       0,  0, 19,  0, }, // Africa_Southern
    {  405,   81,  488787,  488807,  488833, 20, 26, 26, }, // Africa_Western
    {  405,  107,  488859,  488870,  488887, 11, 17, 17, }, // Alaska
    {  405,  128,  488904,  488915,  488932, 11, 17, 17, }, // Almaty
    {  405,  135,  488949,  488962,  488981, 13, 19, 19, }, // Amazon
    {  405,  142,  489000,  489027,  489061, 27, 34, 34, }, // America_Central
    {  405,  158,  489095,  489125,  489162, 30, 37, 37, }, // America_Eastern
    {  405,  174,  489199,  489218,  489243, 19, 25, 25, }, // America_Mountain
    {  405,  191,  489268,  489299,  489337, 31, 38, 38, }, // America_Pacific
    {  405,  207,  489375,  489386,  489403, 11, 17, 17, }, // Anadyr
    {  405,  232,  489420,  489433,  489452, 13, 19, 19, }, // Arabian
    {  405,  240,  489471,  489488,  489511, 17, 23, 23, }, // Argentina
    {  405,  250,  489534,  489555,  489582, 21, 27, 27, }, // Argentina_Western
    {  405,  268,  489609,  489623,  489643, 14, 20, 20, }, // Armenia
    {  405,  286,  489663,  489676,  489695, 13, 19, 19, }, // Atlantic
    {  405,  295,  489714,  489737,  489766, 23, 29, 29, }, // Australia_Central
    {  405,  313,  489795,  489824,  489859, 29, 35, 35, }, // Australia_CentralWestern
    {  405,  338,  489894,  489913,  489938, 19, 25, 25, }, // Australia_Eastern
    {  405,  356,  489804,  489833,  489868, 20, 26, 26, }, // Australia_Western
    {  405,  374,  489963,  489985,  490013, 22, 28, 28, }, // Azerbaijan
    {  405,  385,  490041,  490052,  490069, 11, 17, 17, }, // Azores
    {  405,  397,  490086,  490102,  490124, 16, 22, 22, }, // Bangladesh
    {  405,  415,       0,  490146,       0,  0, 11,  0, }, // Bhutan
    {  405,  422,       0,  490157,       0,  0, 17,  0, }, // Bolivia
    {  405,  437,  490174,  490187,  490206, 13, 19, 19, }, // Brasilia
    {  405,  454,       0,  490225,       0,  0, 11,  0, }, // Brunei
    {  405,  461,  490236,  490250,  490270, 14, 20, 20, }, // Cape_Verde
    {  405,  478,       0,  490290,       0,  0, 13,  0, }, // Chamorro
    {  405,  487,  490303,  490315,  490333, 12, 18, 18, }, // Chatham
    {  405,  495,  490351,  490366,  490387, 15, 21, 21, }, // Chile
    {  405,  501,  490408,  490423,  490444, 15, 21, 21, }, // China
    {  405,  507,       0,  490465,       0,  0, 22,  0, }, // Christmas
    {  405,  517,       0,  490487,       0,  0, 17,  0, }, // Cocos
    {  405,  523,  490504,  490522,  490546, 18, 24, 24, }, // Colombia
    {  405,  532,  490570,  490586,  490608, 16, 22, 22, }, // Cook
    {  405,  537,  490630,  490644,  490664, 14, 20, 20, }, // Cuba
    {  405,  548,       0,  490684,       0,  0, 10,  0, }, // Davis
    {  405,  564,       0,  490694,       0,  0, 21,  0, }, // DumontDUrville
    {  405,  601,       0,  490715,       0,  0, 13,  0, }, // East_Timor
    {  405,  612,  490728,  490746,  490770, 18, 24, 24, }, // Easter
    {  405,  619,       0,  490794,       0,  0, 19,  0, }, // Ecuador
    {  405,  627,  490813,  490834,  490861, 21, 27, 27, }, // Europe_Central
    {  405,  642,  490888,  490906,  490930, 18, 24, 24, }, // Europe_Eastern
    {  405,  680,  490954,  490973,  490998, 19, 25, 25, }, // Europe_Western
    {  405,  695,  491023,  491043,  491069, 20, 26, 26, }, // Falkland
    {  405,  704,  491095,  491107,  491125, 12, 18, 18, }, // Fiji
    {  405,  709,       0,  491143,       0,  0, 23,  0, }, // French_Guiana
    {  405,  723,       0,  491166,       0,  0, 41,  0, }, // French_Southern
    {  405,  746,       0,  491207,       0,  0, 14,  0, }, // Galapagos
    {  405,  756,       0,  491221,       0,  0, 12,  0, }, // Gambier
    {  405,  764,  491233,  491247,  491267, 14, 20, 20, }, // Georgia
    {  405,  772,       0,  491287,       0,  0, 20,  0, }, // Gilbert_Islands
    {  405,  788,       0,  491307,       0,  0, 22,  0, }, // GMT
    {  405,  830,  491329,  491345,  491367, 16, 22, 22, }, // Greenland_Eastern
    {  405,  848,  491389,  491406,  491429, 17, 23, 23, }, // Greenland_Western
    {  405,  866,       0,  491452,       0,  0,  9,  0, }, // Guam
    {  405,  871,       0,  491461,       0,  0,  9,  0, }, // Gulf
    {  405,  876,       0,  491470,       0,  0, 11,  0, }, // Guyana
    {  405,  883,  491481,  491500,  491525, 19, 25, 25, }, // Hawaii_Aleutian
    {  405,  899,  491550,  491564,  491584, 14, 20, 20, }, // Hong_Kong
    {  405,  909,  491604,  491613,  491628,  9, 15, 15, }, // Hovd
    {  405,  914,       0,  491643,       0,  0, 12,  0, }, // India
    {  405,  920,       0,  491655,       0,  0, 20,  0, }, // Indian_Ocean
    {  405,  933,       0,  491675,       0,  0, 14,  0, }, // Indochina
    {  405,  943,       0,  491689,       0,  0, 23,  0, }, // Indonesia_Central
    {  405,  961,       0,  491712,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  405,  979,       0,  491731,       0,  0, 20,  0, }, // Indonesia_Western
    {  405,  997,  491751,  491764,  491783, 13, 19, 19, }, // Iran
    {  405, 1008,  491802,  491814,  491832, 12, 18, 18, }, // Irkutsk
    {  405, 1016,  491850,  491865,  491886, 15, 21, 21, }, // Israel
    {  405, 1023,  491907,  491921,  491941, 14, 20, 20, }, // Japan
    {  405, 1029,  491961,  491977,  491999, 16, 22, 22, }, // Kamchatka
    {  405, 1058,       0,  492021,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  405, 1077,       0,  492039,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  405, 1106,  492058,  492073,  492094, 15, 21, 21, }, // Korea
    {  405, 1112,       0,  492115,       0,  0, 11,  0, }, // Kosrae
    {  405, 1119,  492126,  492142,  492164, 16, 22, 22, }, // Krasnoyarsk
    {  405, 1151,       0,  492186,       0,  0, 16,  0, }, // Kyrgystan
    {  405, 1175,       0,  492202,       0,  0, 18,  0, }, // Line_Islands
    {  405, 1188,  492220,  492234,  492254, 14, 20, 20, }, // Lord_Howe
    {  405, 1204,  492274,  492286,  492304, 12, 18, 18, }, // Magadan
    {  405, 1219,       0,  492322,       0,  0, 15,  0, }, // Malaysia
    {  405, 1228,       0,  492337,       0,  0, 13,  0, }, // Maldives
    {  405, 1237,       0,  492350,       0,  0, 14,  0, }, // Marquesas
    {  405, 1247,       0,  492364,       0,  0, 20,  0, }, // Marshall_Islands
    {  405, 1264,  492384,  492398,  492418, 14, 20, 20, }, // Mauritius
    {  405, 1274,       0,  492438,       0,  0, 11,  0, }, // Mawson
    {  405, 1281,  492449,  492472,  492502, 23, 30, 30, }, // Mexico_Pacific
    {  405, 1296,  492532,  492548,  492570, 16, 22, 22, }, // Mongolia
    {  405, 1305,  492592,  492605,  492624, 13, 19, 19, }, // Moscow
    {  405, 1312,       0,  492643,       0,  0, 12,  0, }, // Myanmar
    {  405, 1320,       0,  492655,       0,  0, 10,  0, }, // Nauru
    {  405, 1326,       0,  492665,       0,  0, 16,  0, }, // Nepal
    {  405, 1332,  492681,  492700,  492725, 19, 25, 25, }, // New_Caledonia
    {  405, 1346,  492750,  492765,  492786, 15, 21, 21, }, // New_Zealand
    {  405, 1358,  492807,  492823,  492845, 16, 22, 22, }, // Newfoundland
    {  405, 1371,       0,  492867,       0,  0,  9,  0, }, // Niue
    {  405, 1376,  492876,  492895,  492920, 19, 25, 25, }, // Norfolk
    {  405, 1384,  492945,  492969,  492999, 24, 30, 30, }, // Noronha
    {  405, 1406,  493029,  493045,  493067, 16, 22, 22, }, // Novosibirsk
    {  405, 1418,  493089,  493098,  493113,  9, 15, 15, }, // Omsk
    {  405, 1428,  493128,  493145,  493168, 17, 23, 23, }, // Pakistan
    {  405, 1437,       0,  493191,       0,  0, 10,  0, }, // Palau
    {  405, 1443,       0,  493201,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  405, 1460,  493221,  493240,  493265, 19, 25, 25, }, // Paraguay
    {  405, 1469,  493290,  493305,  493326, 15, 21, 21, }, // Peru
    {  405, 1474,  493347,  493366,  493391, 19, 25, 25, }, // Philippines
    {  405, 1486,       0,  493416,       0,  0, 19,  0, }, // Phoenix_Islands
    {  405, 1502,  493435,  493463,  493497, 28, 34, 34, }, // Pierre_Miquelon
    {  405, 1518,       0,  493531,       0,  0, 20,  0, }, // Pitcairn
    {  405, 1527,       0,  493551,       0,  0, 11,  0, }, // Ponape
    {  405, 1554,       0,  493562,       0,  0, 12,  0, }, // Reunion
    {  405, 1562,       0,  493574,       0,  0, 12,  0, }, // Rothera
    {  405, 1570,  493586,  493599,  493618, 13, 19, 19, }, // Sakhalin
    {  405, 1579,  493637,  493648,  493665, 11, 17, 17, }, // Samara
    {  405, 1596,  493682,  493692,  493708, 10, 16, 16, }, // Samoa
    {  405, 1602,       0,  493724,       0,  0, 15,  0, }, // Seychelles
    {  405, 1624,       0,  493739,       0,  0, 21,  0, }, // Singapore
    {  405, 1634,       0,  493760,       0,  0, 19,  0, }, // Solomon
    {  405, 1642,       0,  493779,       0,  0, 17,  0, }, // South_Georgia
    {  405, 1656,       0,  493796,       0,  0, 13,  0, }, // Suriname
    {  405, 1676,       0,  493809,       0,  0, 10,  0, }, // Syowa
    {  405, 1682,       0,  493819,       0,  0, 11,  0, }, // Tahiti
    {  405, 1689,  493830,  493841,  493858, 11, 17, 17, }, // Taipei
    {  405, 1696,       0,  493875,       0,  0, 18,  0, }, // Tajikistan
    {  405, 1724,       0,  493893,       0,  0, 12,  0, }, // Tokelau
    {  405, 1732,  493905,  493920,  493941, 15, 21, 21, }, // Tonga
    {  405, 1738,       0,  493962,       0,  0, 10,  0, }, // Truk
    {  405, 1750,  493972,  493989,  494012, 17, 23, 23, }, // Turkmenistan
    {  405, 1763,       0,  494035,       0,  0, 11,  0, }, // Tuvalu
    {  405, 1777,  494046,  494064,  494087, 18, 23, 24, }, // Uruguay
    {  405, 1792,  494111,  494126,  494147, 15, 21, 21, }, // Uzbekistan
    {  405, 1803,  494168,  494180,  494198, 12, 18, 18, }, // Vanuatu
    {  405, 1811,       0,  494216,       0,  0, 14,  0, }, // Venezuela
    {  405, 1821,  494230,  494246,  494268, 16, 22, 22, }, // Vladivostok
    {  405, 1833,  494290,  494304,  494324, 14, 20, 20, }, // Volgograd
    {  405, 1843,       0,  494344,       0,  0, 11,  0, }, // Vostok
    {  405, 1850,       0,  494355,       0,  0, 15,  0, }, // Wake
    {  405, 1855,       0,  494370,       0,  0, 20,  0, }, // Wallis
    {  405, 1862,  494390,  494402,  494420, 12, 18, 18, }, // Yakutsk
    {  405, 1870,  494438,  494457,  494482, 19, 25, 25, }, // Yekaterinburg
    {  407,    0,  105736,  105746,  494507, 10, 21, 29, }, // Acre Macedonian/Cyrillic/Macedonia
    {  407,    5,       0,  494536,       0,  0, 19,  0, }, // Afghanistan
    {  407,   17,       0,  494555,       0,  0, 22,  0, }, // Africa_Central
    {  407,   32,       0,  494577,       0,  0, 23,  0, }, // Africa_Eastern
    {  407,   65,       0,  494600,       0,  0, 34,  0, }, // Africa_Southern
    {  407,   81,  119373,  494634,  494668, 23, 34, 42, }, // Africa_Western
    {  407,  107,  494710,  494726,  494753, 16, 27, 35, }, // Alaska
    {  407,  135,  494788,  494803,  494829, 15, 26, 34, }, // Amazon
    {  407,  142,  494863,  494897,  494942, 34, 45, 53, }, // America_Central
    {  407,  158,  494995,  495027,  495070, 32, 43, 51, }, // America_Eastern
    {  407,  174,  495121,  495155,  495200, 34, 45, 53, }, // America_Mountain
    {  407,  191,  495253,  495287,  495332, 34, 45, 53, }, // America_Pacific
    {  407,  207,  495385,  495400,  495426, 15, 26, 21, }, // Anadyr
    {  407,  214,  495447,  495461,  495486, 14, 25, 20, }, // Apia
    {  407,  232,  495506,  495519,  495543, 13, 24, 32, }, // Arabian
    {  407,  240,  495575,  495593,  495622, 18, 29, 37, }, // Argentina
    {  407,  250,  495659,  495685,  495722, 26, 37, 45, }, // Argentina_Western
    {  407,  268,  495767,  495784,  495812, 17, 28, 23, }, // Armenia
    {  407,  286,  495835,  495850,  495876, 15, 26, 34, }, // Atlantic
    {  407,  295,  495910,  495939,  495979, 29, 40, 48, }, // Australia_Central
    {  407,  313,  496027,  496066,  496116, 39, 50, 58, }, // Australia_CentralWestern
    {  407,  338,  496174,  496201,  496239, 27, 38, 46, }, // Australia_Eastern
    {  407,  356,  496285,  496312,  496350, 27, 38, 46, }, // Australia_Western
    {  407,  374,  496396,  496415,  496445, 19, 30, 25, }, // Azerbaijan
    {  407,  385,  496470,  496494,  496529, 24, 35, 30, }, // Azores
    {  407,  397,  496559,  496577,  496606, 18, 29, 24, }, // Bangladesh
    {  407,  415,       0,  496630,       0,  0, 14,  0, }, // Bhutan
    {  407,  422,       0,  496644,       0,  0, 17,  0, }, // Bolivia
    {  407,  437,  496661,  496679,  496708, 18, 29, 37, }, // Brasilia
    {  407,  454,       0,  496745,       0,  0, 25,  0, }, // Brunei
    {  407,  461,  496770,  496789,  496819, 19, 30, 38, }, // Cape_Verde
    {  407,  478,       0,  496857,       0,  0, 15,  0, }, // Chamorro
    {  407,  487,  496872,  496886,  496911, 14, 25, 33, }, // Chatham
    {  407,  495,  496944,  496957,  496981, 13, 24, 32, }, // Chile
    {  407,  501,  497013,  497026,  497050, 13, 24, 32, }, // China
    {  407,  507,       0,  497082,       0,  0, 23,  0, }, // Christmas
    {  407,  517,       0,  497105,       0,  0, 25,  0, }, // Cocos
    {  407,  523,  497130,  497148,  497177, 18, 29, 37, }, // Colombia
    {  407,  532,  497214,  497237,  497271, 23, 34, 29, }, // Cook
    {  407,  537,  497300,  497313,  497337, 13, 24, 32, }, // Cuba
    {  407,  548,       0,  497369,       0,  0, 15,  0, }, // Davis
    {  407,  564,       0,  497384,       0,  0, 21,  0, }, // DumontDUrville
    {  407,  601,       0,  497405,       0,  0, 22,  0, }, // East_Timor
    {  407,  612,  497427,  497454,  497492, 27, 38, 33, }, // Easter
    {  407,  619,       0,  497525,       0,  0, 16,  0, }, // Ecuador
    {  407,  627,  497541,  497561,  497592, 20, 31, 26, }, // Europe_Central
    {  407,  642,  497618,  497639,  497671, 21, 32, 27, }, // Europe_Eastern
    {  407,  657,       0,  497698,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  407,  680,  497718,  497739,  497771, 21, 32, 27, }, // Europe_Western
    {  407,  695,  497798,  497826,  497865, 28, 39, 47, }, // Falkland
    {  407,  704,  497912,  497925,  497949, 13, 24, 19, }, // Fiji
    {  407,  709,       0,  497968,       0,  0, 26,  0, }, // French_Guiana
    {  407,  723,       0,  497994,       0,  0, 35,  0, }, // French_Southern
    {  407,  746,       0,  498029,       0,  0, 18,  0, }, // Galapagos
    {  407,  756,       0,  498047,       0,  0, 14,  0, }, // Gambier
    {  407,  764,  498061,  498077,  498104, 16, 27, 22, }, // Georgia
    {  407,  772,       0,  498126,       0,  0, 27,  0, }, // Gilbert_Islands
    {  407,  788,       0,  498153,       0,  0, 22,  0, }, // GMT
    {  407,  830,  498175,  498200,  498236, 25, 36, 44, }, // Greenland_Eastern
    {  407,  848,  498280,  498305,  498341, 25, 36, 44, }, // Greenland_Western
    {  407,  871,       0,  498385,       0,  0, 23,  0, }, // Gulf
    {  407,  876,       0,  498408,       0,  0, 16,  0, }, // Guyana
    {  407,  883,  498424,  498457,  498501, 33, 44, 52, }, // Hawaii_Aleutian
    {  407,  899,  498553,  498571,  498600, 18, 29, 24, }, // Hong_Kong
    {  407,  909,  498624,  498637,  498661, 13, 24, 19, }, // Hovd
    {  407,  914,       0,  498680,       0,  0, 15,  0, }, // India
    {  407,  920,       0,  498695,       0,  0, 22,  0, }, // Indian_Ocean
    {  407,  933,       0,  498717,       0,  0, 17,  0, }, // Indochina
    {  407,  943,       0,  498734,       0,  0, 29,  0, }, // Indonesia_Central
    {  407,  961,       0,  498763,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  407,  979,       0,  498790,       0,  0, 27,  0, }, // Indonesia_Western
    {  407,  997,  498817,  498830,  498854, 13, 24, 32, }, // Iran
    {  407, 1008,  498886,  498902,  498929, 16, 27, 22, }, // Irkutsk
    {  407, 1016,  498951,  498966,  498992, 15, 26, 34, }, // Israel
    {  407, 1023,  499026,  499043,  499071, 17, 28, 36, }, // Japan
    {  407, 1047,       0,  499107,       0,  0, 18,  0, }, // Kazakhstan
    {  407, 1058,       0,  499125,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  407, 1077,       0,  499151,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  407, 1106,  499177,  499192,  499218, 15, 26, 34, }, // Korea
    {  407, 1112,       0,  499252,       0,  0, 14,  0, }, // Kosrae
    {  407, 1119,  499266,  499286,  499317, 20, 31, 26, }, // Krasnoyarsk
    {  407, 1151,       0,  499343,       0,  0, 18,  0, }, // Kyrgystan
    {  407, 1175,       0,  499361,       0,  0, 24,  0, }, // Line_Islands
    {  407, 1188,  499385,  499402,  499430, 17, 28, 36, }, // Lord_Howe
    {  407, 1204,  499466,  499482,  499509, 16, 27, 22, }, // Magadan
    {  407, 1219,       0,  499531,       0,  0, 17,  0, }, // Malaysia
    {  407, 1228,       0,  499548,       0,  0, 16,  0, }, // Maldives
    {  407, 1237,       0,  499564,       0,  0, 17,  0, }, // Marquesas
    {  407, 1247,       0,  499581,       0,  0, 26,  0, }, // Marshall_Islands
    {  407, 1264,  499607,  499625,  499654, 18, 29, 37, }, // Mauritius
    {  407, 1274,       0,  499691,       0,  0, 14,  0, }, // Mawson
    {  407, 1281,  499705,  499731,  499768, 26, 37, 32, }, // Mexico_Pacific
    {  407, 1296,  499800,  499819,  499849, 19, 30, 25, }, // Mongolia
    {  407, 1305,  499874,  499889,  499915, 15, 26, 34, }, // Moscow
    {  407, 1312,       0,  499949,       0,  0, 16,  0, }, // Myanmar
    {  407, 1320,       0,  499965,       0,  0, 14,  0, }, // Nauru
    {  407, 1326,       0,  499979,       0,  0, 14,  0, }, // Nepal
    {  407, 1332,  499993,  500017,  500052, 24, 35, 30, }, // New_Caledonia
    {  407, 1346,  500082,  500101,  500131, 19, 30, 38, }, // New_Zealand
    {  407, 1358,  500169,  500189,  500220, 20, 31, 39, }, // Newfoundland
    {  407, 1371,       0,  500259,       0,  0, 13,  0, }, // Niue
    {  407, 1376,  500272,  500297,  500333, 25, 36, 44, }, // Norfolk
    {  407, 1384,  500377,  500404,  500442, 27, 38, 46, }, // Noronha
    {  407, 1406,  500488,  500508,  500539, 20, 31, 26, }, // Novosibirsk
    {  407, 1418,  500565,  500578,  500602, 13, 24, 19, }, // Omsk
    {  407, 1428,  500621,  500638,  500666, 17, 28, 23, }, // Pakistan
    {  407, 1437,       0,  500689,       0,  0, 14,  0, }, // Palau
    {  407, 1443,       0,  500703,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  407, 1460,  500730,  500747,  500775, 17, 28, 36, }, // Paraguay
    {  407, 1469,  500811,  500824,  500848, 13, 24, 32, }, // Peru
    {  407, 1474,  500880,  500897,  500925, 17, 28, 23, }, // Philippines
    {  407, 1486,       0,  500948,       0,  0, 25,  0, }, // Phoenix_Islands
    {  407, 1502,  500973,  501001,  501040, 28, 39, 47, }, // Pierre_Miquelon
    {  407, 1518,       0,  501087,       0,  0, 16,  0, }, // Pitcairn
    {  407, 1527,       0,  501103,       0,  0, 15,  0, }, // Ponape
    {  407, 1534,       0,  501118,       0,  0, 18,  0, }, // Pyongyang
    {  407, 1554,       0,  501136,       0,  0, 17,  0, }, // Reunion
    {  407, 1562,       0,  501153,       0,  0, 15,  0, }, // Rothera
    {  407, 1570,  501168,  501184,  501211, 16, 27, 22, }, // Sakhalin
    {  407, 1579,  112657,  112669,  501233, 12, 23, 31, }, // Samara
    {  407, 1596,  501264,  501278,  501303, 14, 25, 20, }, // Samoa
    {  407, 1602,       0,  501323,       0,  0, 16,  0, }, // Seychelles
    {  407, 1624,       0,  501339,       0,  0, 17,  0, }, // Singapore
    {  407, 1634,       0,  501356,       0,  0, 27,  0, }, // Solomon
    {  407, 1642,       0,  501383,       0,  0, 22,  0, }, // South_Georgia
    {  407, 1656,       0,  501405,       0,  0, 16,  0, }, // Suriname
    {  407, 1676,       0,  501421,       0,  0, 15,  0, }, // Syowa
    {  407, 1682,       0,  501436,       0,  0, 15,  0, }, // Tahiti
    {  407, 1689,  501451,  501466,  501492, 15, 26, 34, }, // Taipei
    {  407, 1696,       0,  501526,       0,  0, 19,  0, }, // Tajikistan
    {  407, 1724,       0,  501545,       0,  0, 16,  0, }, // Tokelau
    {  407, 1732,  501561,  501575,  501600, 14, 25, 20, }, // Tonga
    {  407, 1738,       0,  501620,       0,  0, 13,  0, }, // Truk
    {  407, 1750,  501633,  501654,  501686, 21, 32, 27, }, // Turkmenistan
    {  407, 1763,       0,  501713,       0,  0, 15,  0, }, // Tuvalu
    {  407, 1777,  501728,  501744,  501771, 16, 27, 35, }, // Uruguay
    {  407, 1792,  501806,  501825,  501855, 19, 30, 25, }, // Uzbekistan
    {  407, 1803,  501880,  501896,  501923, 16, 27, 22, }, // Vanuatu
    {  407, 1811,       0,  501945,       0,  0, 18,  0, }, // Venezuela
    {  407, 1821,  501963,  501983,  502014, 20, 31, 26, }, // Vladivostok
    {  407, 1833,  502040,  502058,  502087, 18, 29, 37, }, // Volgograd
    {  407, 1843,       0,  502124,       0,  0, 15,  0, }, // Vostok
    {  407, 1850,       0,  502139,       0,  0, 22,  0, }, // Wake
    {  407, 1855,       0,  502161,       0,  0, 23,  0, }, // Wallis
    {  407, 1862,  502184,  502200,  502227, 16, 27, 22, }, // Yakutsk
    {  407, 1870,  502249,  502270,  502302, 21, 32, 27, }, // Yekaterinburg
    {  407, 1892,       0,  502329,       0,  0, 14,  0, }, // Yukon
    {  409,    5,       0,  502343,       0,  0, 16,  0, }, // Afghanistan Maithili/Devanagari/India
    {  409,   17,       0,  502359,       0,  0, 17,  0, }, // Africa_Central
    {  409,   32,       0,  502376,       0,  0, 17,  0, }, // Africa_Eastern
    {  409,   65,       0,  502393,       0,  0, 24,  0, }, // Africa_Southern
    {  409,   81,  502417,  502436,  502460, 19, 24, 21, }, // Africa_Western
    {  409,  107,  334904,  502481,  502497, 11, 16, 18, }, // Alaska
    {  409,  135,  502515,  502525,  502540, 10, 15, 14, }, // Amazon
    {  409,  142,  334999,  335026,  335058, 27, 32, 34, }, // America_Central
    {  409,  158,  335092,  335117,  335147, 25, 30, 32, }, // America_Eastern
    {  409,  174,  335179,  335205,  335236, 26, 31, 33, }, // America_Mountain
    {  409,  191,  335269,  335295,  335326, 26, 31, 33, }, // America_Pacific
    {  409,  232,  502554,  502566,  502583, 12, 17, 19, }, // Arabian
    {  409,  240,  502602,  502617,  502637, 15, 20, 19, }, // Argentina
    {  409,  250,  502656,  502679,  502707, 23, 28, 27, }, // Argentina_Western
    {  409,  268,  502734,  502748,  502767, 14, 19, 18, }, // Armenia
    {  409,  286,  335680,  335692,  335709, 12, 17, 19, }, // Atlantic
    {  409,  295,  502785,  502806,  502833, 21, 27, 29, }, // Australia_Central
    {  409,  313,  502862,  502892,  502927, 30, 35, 37, }, // Australia_CentralWestern
    {  409,  338,  502964,  502986,  503014, 22, 28, 30, }, // Australia_Eastern
    {  409,  356,  503044,  503068,  503098, 24, 30, 32, }, // Australia_Western
    {  409,  374,  503130,  503143,  503161, 13, 18, 17, }, // Azerbaijan
    {  409,  385,  503178,  503189,  503205, 11, 16, 15, }, // Azores
    {  409,  397,  503220,  503233,  503251, 13, 18, 17, }, // Bangladesh
    {  409,  415,       0,  503268,       0,  0, 10,  0, }, // Bhutan
    {  409,  422,       0,  503278,       0,  0, 13,  0, }, // Bolivia
    {  409,  437,  503291,  503306,  503326, 15, 20, 19, }, // Brasilia
    {  409,  454,       0,  503345,       0,  0, 21,  0, }, // Brunei
    {  409,  461,  503366,  503380,  503398, 14, 18, 17, }, // Cape_Verde
    {  409,  487,  503415,  503424,  503438,  9, 14, 16, }, // Chatham
    {  409,  495,  503454,  503463,  503477,  9, 14, 13, }, // Chile
    {  409,  501,  503490,  503499,  503513,  9, 14, 16, }, // China
    {  409,  507,       0,  503529,       0,  0, 18,  0, }, // Christmas
    {  409,  517,       0,  503547,       0,  0, 16,  0, }, // Cocos
    {  409,  523,  503563,  503576,  503594, 13, 18, 17, }, // Colombia
    {  409,  537,  503611,  503622,  503638, 11, 16, 18, }, // Cuba
    {  409,  601,       0,  503656,       0,  0, 16,  0, }, // East_Timor
    {  409,  612,  503672,  503689,  503711, 17, 22, 21, }, // Easter
    {  409,  619,       0,  503732,       0,  0, 13,  0, }, // Ecuador
    {  409,  627,  336876,  336892,  336913, 16, 21, 31, }, // Europe_Central
    {  409,  642,  336944,  336962,  336985, 18, 23, 31, }, // Europe_Eastern
    {  409,  657,       0,  503745,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  409,  680,  337039,  337058,  337082, 19, 24, 33, }, // Europe_Western
    {  409,  695,  503771,  503790,  503814, 19, 24, 23, }, // Falkland
    {  409,  709,       0,  503837,       0,  0, 18,  0, }, // French_Guiana
    {  409,  723,       0,  503855,       0,  0, 44,  0, }, // French_Southern
    {  409,  746,       0,  503899,       0,  0, 14,  0, }, // Galapagos
    {  409,  764,  503913,  503926,  503944, 13, 18, 17, }, // Georgia
    {  409,  788,       0,   91136,       0,  0, 17,  0, }, // GMT
    {  409,  830,  503961,  503981,  504006, 20, 25, 24, }, // Greenland_Eastern
    {  409,  848,  504030,  504052,  504079, 22, 27, 26, }, // Greenland_Western
    {  409,  871,       0,  504105,       0,  0, 13,  0, }, // Gulf
    {  409,  876,       0,  503844,       0,  0, 11,  0, }, // Guyana
    {  409,  883,  504118,  504137,  504161, 19, 24, 26, }, // Hawaii_Aleutian
    {  409,  899,  504187,  504201,  504220, 14, 19, 18, }, // Hong_Kong
    {  409,  909,  504238,  504248,  504263, 10, 15, 14, }, // Hovd
    {  409,  914,       0,  337733,       0,  0, 15,  0, }, // India
    {  409,  920,       0,  504277,       0,  0, 17,  0, }, // Indian_Ocean
    {  409,  933,       0,  504294,       0,  0, 14,  0, }, // Indochina
    {  409,  943,       0,  504308,       0,  0, 20,  0, }, // Indonesia_Central
    {  409,  961,       0,  504328,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  409,  979,       0,  504349,       0,  0, 23,  0, }, // Indonesia_Western
    {  409,  997,  504372,  504381,  504394,  9, 13, 16, }, // Iran
    {  409, 1008,  504410,  504424,  504443, 14, 19, 18, }, // Irkutsk
    {  409, 1016,  504461,  504472,  504488, 11, 16, 18, }, // Israel
    {  409, 1023,  504506,  504516,  504531, 10, 15, 17, }, // Japan
    {  409, 1058,       0,  504548,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  409, 1077,       0,  504567,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  409, 1106,  504588,  504599,  504615, 11, 16, 18, }, // Korea
    {  409, 1119,  504633,  504648,  504673, 15, 25, 24, }, // Krasnoyarsk
    {  409, 1151,       0,  504697,       0,  0, 11,  0, }, // Kyrgystan
    {  409, 1188,  504708,  504723,  504743, 15, 20, 22, }, // Lord_Howe
    {  409, 1204,  504765,  504776,  504792, 11, 16, 15, }, // Magadan
    {  409, 1219,       0,  504807,       0,  0, 12,  0, }, // Malaysia
    {  409, 1228,       0,  504819,       0,  0, 11,  0, }, // Maldives
    {  409, 1264,  504830,  504841,  504857, 11, 16, 15, }, // Mauritius
    {  409, 1281,  504872,  504893,  504919, 21, 26, 28, }, // Mexico_Pacific
    {  409, 1296,  504947,  504959,  504976, 12, 17, 16, }, // Mongolia
    {  409, 1305,  504992,  505003,  505019, 11, 16, 15, }, // Moscow
    {  409, 1312,       0,  505034,       0,  0, 11,  0, }, // Myanmar
    {  409, 1326,       0,  505045,       0,  0, 10,  0, }, // Nepal
    {  409, 1332,  505055,  505074,  505098, 19, 24, 26, }, // New_Caledonia
    {  409, 1346,  505124,  505139,  505159, 15, 20, 22, }, // New_Zealand
    {  409, 1358,  505181,  505199,  505222, 18, 23, 25, }, // Newfoundland
    {  409, 1376,  505247,  505265,  505288, 18, 23, 25, }, // Norfolk
    {  409, 1384,  505313,  505338,  505368, 25, 30, 29, }, // Noronha
    {  409, 1406,  505397,  505415,  505438, 18, 23, 22, }, // Novosibirsk
    {  409, 1418,  505460,  505471,  505487, 11, 16, 15, }, // Omsk
    {  409, 1428,  505502,  505516,  505535, 14, 19, 18, }, // Pakistan
    {  409, 1460,  505553,  505565,  505582, 12, 17, 16, }, // Paraguay
    {  409, 1469,  505598,  505607,  505621,  9, 14, 13, }, // Peru
    {  409, 1474,  505634,  505646,  505663, 12, 17, 16, }, // Philippines
    {  409, 1502,  505679,  505706,  505738, 27, 32, 34, }, // Pierre_Miquelon
    {  409, 1534,       0,  505772,       0,  0, 15,  0, }, // Pyongyang
    {  409, 1554,       0,  505787,       0,  0, 13,  0, }, // Reunion
    {  409, 1570,  505800,  505811,  505827, 11, 16, 15, }, // Sakhalin
    {  409, 1596,  505842,  505851,  505865,  9, 14, 13, }, // Samoa
    {  409, 1602,       0,  505878,       0,  0, 12,  0, }, // Seychelles
    {  409, 1624,       0,  505890,       0,  0, 18,  0, }, // Singapore
    {  409, 1642,       0,  505908,       0,  0, 20,  0, }, // South_Georgia
    {  409, 1656,       0,  505928,       0,  0, 12,  0, }, // Suriname
    {  409, 1689,  505940,  505951,  505967, 11, 16, 18, }, // Taipei
    {  409, 1696,       0,   96592,       0,  0, 11,  0, }, // Tajikistan
    {  409, 1750,  505985,  506004,  506028, 19, 24, 23, }, // Turkmenistan
    {  409, 1777,  506051,  506063,  506080, 12, 17, 16, }, // Uruguay
    {  409, 1792,  506096,  506112,  506133, 16, 21, 20, }, // Uzbekistan
    {  409, 1811,       0,  506153,       0,  0, 14,  0, }, // Venezuela
    {  409, 1821,  506167,  506185,  506208, 18, 23, 22, }, // Vladivostok
    {  409, 1833,  506230,  506246,  506267, 16, 21, 20, }, // Volgograd
    {  409, 1862,  506287,  506300,  506318, 13, 18, 17, }, // Yakutsk
    {  409, 1870,  506335,  506353,  506376, 18, 23, 22, }, // Yekaterinburg
    {  409, 1892,       0,  506398,       0,  0, 10,  0, }, // Yukon
    {  413,    0,  506408,  506420,  506445, 12, 25, 22, }, // Acre Malayalam/Malayalam/India
    {  413,    5,       0,  506467,       0,  0, 18,  0, }, // Afghanistan
    {  413,   17,       0,  506485,       0,  0, 18,  0, }, // Africa_Central
    {  413,   32,       0,  506503,       0,  0, 21,  0, }, // Africa_Eastern
    {  413,   65,       0,  506524,       0,  0, 32,  0, }, // Africa_Southern
    {  413,   81,  506556,  506579,  506615, 23, 36, 35, }, // Africa_Western
    {  413,  107,  506650,  506662,  506686, 12, 24, 21, }, // Alaska
    {  413,  128,  506707,  506717,  506740, 10, 23, 20, }, // Almaty
    {  413,  135,  506760,  506770,  506793, 10, 23, 22, }, // Amazon
    {  413,  142,  506815,  506844,  506886, 29, 42, 38, }, // America_Central
    {  413,  158,  506924,  506953,  506995, 29, 42, 38, }, // America_Eastern
    {  413,  174,  507033,  507061,  507102, 28, 41, 37, }, // America_Mountain
    {  413,  191,  507139,  507167,  507208, 28, 41, 37, }, // America_Pacific
    {  413,  207,  507245,  507256,  507280, 11, 24, 21, }, // Anadyr
    {  413,  214,  507301,  507310,  507332,  9, 22, 18, }, // Apia
    {  413,  219,  507350,  507360,  507383, 10, 23, 20, }, // Aqtau
    {  413,  225,  507403,  507415,  507440, 12, 25, 22, }, // Aqtobe
    {  413,  232,  507462,  507474,  507499, 12, 25, 21, }, // Arabian
    {  413,  240,  507520,  507533,  507559, 13, 26, 25, }, // Argentina
    {  413,  250,  507584,  507607,  507643, 23, 36, 35, }, // Argentina_Western
    {  413,  268,  507678,  507690,  507715, 12, 25, 24, }, // Armenia
    {  413,  286,  507739,  507758,  507790, 19, 32, 28, }, // Atlantic
    {  413,  295,  507818,  507841,  507878, 23, 37, 33, }, // Australia_Central
    {  413,  313,  507911,  507945,  507992, 34, 47, 43, }, // Australia_CentralWestern
    {  413,  338,  508035,  508059,  508097, 24, 38, 34, }, // Australia_Eastern
    {  413,  356,  508131,  508157,  508197, 26, 40, 36, }, // Australia_Western
    {  413,  374,  508233,  508246,  508272, 13, 26, 25, }, // Azerbaijan
    {  413,  385,  508297,  508308,  508332, 11, 24, 23, }, // Azores
    {  413,  397,  508355,  508370,  508398, 15, 28, 27, }, // Bangladesh
    {  413,  415,       0,  508425,       0,  0, 12,  0, }, // Bhutan
    {  413,  422,       0,  508437,       0,  0, 12,  0, }, // Bolivia
    {  413,  437,  508449,  508462,  508488, 13, 26, 25, }, // Brasilia
    {  413,  454,       0,  508513,       0,  0, 22,  0, }, // Brunei
    {  413,  461,  508535,  508550,  508578, 15, 28, 27, }, // Cape_Verde
    {  413,  478,       0,  508605,       0,  0, 23,  0, }, // Chamorro
    {  413,  487,  508628,  508639,  508663, 11, 24, 23, }, // Chatham
    {  413,  495,  508686,  508695,  508717,  9, 22, 21, }, // Chile
    {  413,  501,  508738,  508746,  508767,  8, 21, 17, }, // China
    {  413,  507,       0,  508784,       0,  0, 22,  0, }, // Christmas
    {  413,  517,       0,  508806,       0,  0, 22,  0, }, // Cocos
    {  413,  523,  508828,  508840,  508865, 12, 25, 24, }, // Colombia
    {  413,  532,  508889,  508909,  508942, 20, 33, 38, }, // Cook
    {  413,  537,  508980,  508990,  509013, 10, 23, 19, }, // Cuba
    {  413,  548,       0,  509032,       0,  0, 11,  0, }, // Davis
    {  413,  564,       0,  509043,       0,  0, 25,  0, }, // DumontDUrville
    {  413,  601,       0,  509068,       0,  0, 18,  0, }, // East_Timor
    {  413,  612,  509086,  509105,  509137, 19, 32, 31, }, // Easter
    {  413,  619,       0,  509168,       0,  0, 12,  0, }, // Ecuador
    {  413,  627,  509180,  509201,  509235, 21, 34, 32, }, // Europe_Central
    {  413,  642,  509267,  509288,  509322, 21, 34, 32, }, // Europe_Eastern
    {  413,  657,       0,  509354,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  413,  680,  509383,  509406,  509442, 23, 36, 35, }, // Europe_Western
    {  413,  695,  509477,  509503,  509542, 26, 39, 38, }, // Falkland
    {  413,  704,  509580,  509589,  509611,  9, 22, 21, }, // Fiji
    {  413,  709,       0,  509632,       0,  0, 17,  0, }, // French_Guiana
    {  413,  723,       0,  509649,       0,  0, 31,  0, }, // French_Southern
    {  413,  746,       0,  509680,       0,  0, 15,  0, }, // Galapagos
    {  413,  756,       0,  509695,       0,  0, 13,  0, }, // Gambier
    {  413,  764,  509708,  509721,  509747, 13, 26, 25, }, // Georgia
    {  413,  772,       0,  509772,       0,  0, 24,  0, }, // Gilbert_Islands
    {  413,  788,       0,  509796,       0,  0, 20,  0, }, // GMT
    {  413,  830,  509816,  509839,  509875, 23, 36, 35, }, // Greenland_Eastern
    {  413,  848,  509910,  509935,  509973, 25, 38, 37, }, // Greenland_Western
    {  413,  866,       0,  510010,       0,  0, 23,  0, }, // Guam
    {  413,  871,       0,  510033,       0,  0, 22,  0, }, // Gulf
    {  413,  876,       0,  509640,       0,  0,  9,  0, }, // Guyana
    {  413,  883,  510055,  510073,  510104, 18, 31, 27, }, // Hawaii_Aleutian
    {  413,  899,  510131,  510146,  510174, 15, 28, 27, }, // Hong_Kong
    {  413,  909,  510201,  510210,  510232,  9, 22, 21, }, // Hovd
    {  413,  914,       0,  510253,       0,  0, 25,  0, }, // India
    {  413,  920,       0,  510278,       0,  0, 22,  0, }, // Indian_Ocean
    {  413,  933,       0,  510300,       0,  0, 12,  0, }, // Indochina
    {  413,  943,       0,  510312,       0,  0, 20,  0, }, // Indonesia_Central
    {  413,  961,       0,  510332,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  413,  979,       0,  510355,       0,  0, 25,  0, }, // Indonesia_Western
    {  413,  997,  510380,  510389,  510411,  9, 22, 18, }, // Iran
    {  413, 1008,  510429,  510441,  510466, 12, 25, 25, }, // Irkutsk
    {  413, 1016,  510491,  510504,  510530, 13, 26, 22, }, // Israel
    {  413, 1023,  510552,  510563,  510587, 11, 24, 20, }, // Japan
    {  413, 1029,  510607,  510638,  510682, 31, 44, 41, }, // Kamchatka
    {  413, 1047,       0,  510723,       0,  0, 15,  0, }, // Kazakhstan
    {  413, 1058,       0,  510738,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  413, 1077,       0,  510761,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  413, 1106,  510786,  510797,  510821, 11, 24, 20, }, // Korea
    {  413, 1112,       0,  510841,       0,  0, 10,  0, }, // Kosrae
    {  413, 1119,  510851,  510873,  510908, 22, 35, 34, }, // Krasnoyarsk
    {  413, 1151,       0,  510942,       0,  0, 15,  0, }, // Kyrgystan
    {  413, 1161,       0,  510957,       0,  0,  9,  0, }, // Lanka
    {  413, 1175,       0,  510966,       0,  0, 17,  0, }, // Line_Islands
    {  413, 1188,  510983,  510998,  511026, 15, 28, 24, }, // Lord_Howe
    {  413, 1198,  511050,  511058,  511079,  8, 21, 18, }, // Macau
    {  413, 1204,  511097,  511107,  511130, 10, 23, 22, }, // Magadan
    {  413, 1219,       0,  511152,       0,  0, 11,  0, }, // Malaysia
    {  413, 1228,       0,  511163,       0,  0, 15,  0, }, // Maldives
    {  413, 1237,       0,  511178,       0,  0, 13,  0, }, // Marquesas
    {  413, 1247,       0,  511191,       0,  0, 19,  0, }, // Marshall_Islands
    {  413, 1264,  511210,  511224,  511251, 14, 27, 26, }, // Mauritius
    {  413, 1274,       0,  511277,       0,  0,  9,  0, }, // Mawson
    {  413, 1281,  511286,  511308,  511344, 22, 36, 31, }, // Mexico_Pacific
    {  413, 1296,  511375,  511390,  511418, 15, 28, 27, }, // Mongolia
    {  413, 1305,  511445,  511456,  511480, 11, 24, 24, }, // Moscow
    {  413, 1312,       0,  511504,       0,  0, 13,  0, }, // Myanmar
    {  413, 1320,       0,  511517,       0,  0,  9,  0, }, // Nauru
    {  413, 1326,       0,  511526,       0,  0, 12,  0, }, // Nepal
    {  413, 1332,  511538,  511557,  511589, 19, 32, 31, }, // New_Caledonia
    {  413, 1346,  511620,  511636,  511665, 16, 29, 25, }, // New_Zealand
    {  413, 1358,  511690,  511712,  511747, 22, 35, 31, }, // Newfoundland
    {  413, 1371,       0,  511778,       0,  0, 11,  0, }, // Niue
    {  413, 1376,  511789,  511811,  511846, 22, 35, 31, }, // Norfolk
    {  413, 1384,  511877,  511899,  511934, 22, 35, 34, }, // Noronha
    {  413, 1392,       0,  511968,       0,  0, 29,  0, }, // North_Mariana
    {  413, 1406,  511997,  512016,  512047, 19, 31, 31, }, // Novosibirsk
    {  413, 1418,  512078,  512092,  512119, 14, 27, 26, }, // Omsk
    {  413, 1428,  512145,  512161,  512190, 16, 29, 28, }, // Pakistan
    {  413, 1437,       0,  512218,       0,  0, 10,  0, }, // Palau
    {  413, 1443,       0,  512228,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  413, 1460,  512251,  512263,  512288, 12, 25, 24, }, // Paraguay
    {  413, 1469,  512312,  512321,  512343,  9, 22, 21, }, // Peru
    {  413, 1474,  512364,  512378,  512405, 14, 27, 26, }, // Philippines
    {  413, 1486,       0,  512431,       0,  0, 21,  0, }, // Phoenix_Islands
    {  413, 1502,  512452,  512482,  512525, 30, 43, 39, }, // Pierre_Miquelon
    {  413, 1518,       0,  512564,       0,  0, 14,  0, }, // Pitcairn
    {  413, 1527,       0,  512578,       0,  0, 13,  0, }, // Ponape
    {  413, 1534,       0,  512591,       0,  0, 18,  0, }, // Pyongyang
    {  413, 1544,  512609,  512622,  512648, 13, 26, 23, }, // Qyzylorda
    {  413, 1554,       0,  512671,       0,  0, 13,  0, }, // Reunion
    {  413, 1562,       0,  512684,       0,  0, 10,  0, }, // Rothera
    {  413, 1570,  512694,  512705,  512729, 11, 24, 23, }, // Sakhalin
    {  413, 1579,  512752,  512761,  512783,  9, 22, 19, }, // Samara
    {  413, 1596,  512802,  512811,  512833,  9, 22, 22, }, // Samoa
    {  413, 1602,       0,  512855,       0,  0, 12,  0, }, // Seychelles
    {  413, 1624,       0,  512867,       0,  0, 27,  0, }, // Singapore
    {  413, 1634,       0,  512894,       0,  0, 19,  0, }, // Solomon
    {  413, 1642,       0,  512913,       0,  0, 21,  0, }, // South_Georgia
    {  413, 1656,       0,  512934,       0,  0, 14,  0, }, // Suriname
    {  413, 1676,       0,  512948,       0,  0,  9,  0, }, // Syowa
    {  413, 1682,       0,  512957,       0,  0, 11,  0, }, // Tahiti
    {  413, 1689,  512968,  512982,  513009, 14, 27, 23, }, // Taipei
    {  413, 1696,       0,  513032,       0,  0, 18,  0, }, // Tajikistan
    {  413, 1724,       0,  513050,       0,  0, 13,  0, }, // Tokelau
    {  413, 1732,  513063,  513072,  513094,  9, 22, 21, }, // Tonga
    {  413, 1738,       0,  513115,       0,  0, 11,  0, }, // Truk
    {  413, 1750,  513126,  513148,  513183, 22, 35, 34, }, // Turkmenistan
    {  413, 1763,       0,  513217,       0,  0, 11,  0, }, // Tuvalu
    {  413, 1777,  513228,  513240,  513265, 12, 25, 24, }, // Uruguay
    {  413, 1792,  513289,  513309,  513342, 20, 33, 32, }, // Uzbekistan
    {  413, 1803,  513374,  513386,  513411, 12, 25, 24, }, // Vanuatu
    {  413, 1811,       0,  513435,       0,  0, 14,  0, }, // Venezuela
    {  413, 1821,  513449,  513470,  513504, 21, 34, 33, }, // Vladivostok
    {  413, 1833,  513537,  513553,  513582, 16, 29, 28, }, // Volgograd
    {  413, 1843,       0,  513455,       0,  0, 15,  0, }, // Vostok
    {  413, 1850,       0,  513610,       0,  0, 18,  0, }, // Wake
    {  413, 1855,       0,  513628,       0,  0, 27,  0, }, // Wallis
    {  413, 1862,  513655,  513670,  513698, 15, 28, 27, }, // Yakutsk
    {  413, 1870,  513725,  513746,  513780, 21, 34, 33, }, // Yekaterinburg
    {  413, 1892,       0,  513813,       0,  0, 12,  0, }, // Yukon
    {  414,    0,  358851,  513825,  358879, 10, 17, 22, }, // Acre Malay/Latin/Malaysia
    {  414,    5,       0,  513842,       0,  0, 17,  0, }, // Afghanistan
    {  414,   17,       0,  358917,       0,  0, 19,  0, }, // Africa_Central
    {  414,   32,       0,  358936,       0,  0, 18,  0, }, // Africa_Eastern
    {  414,   65,       0,  513859,       0,  0, 27,  0, }, // Africa_Southern
    {  414,   81,  358982,  513886,  359026, 18, 25, 30, }, // Africa_Western
    {  414,  107,  359056,  513911,  513930, 12, 19, 18, }, // Alaska
    {  414,  128,  359112,  513948,  359144, 12, 19, 24, }, // Almaty
    {  414,  135,  359168,  513967,  359200, 12, 19, 24, }, // Amazon
    {  414,  142,  513986,  513997,  514015, 11, 18, 18, }, // America_Central
    {  414,  158,  359280,  514033,  514051, 11, 18, 17, }, // America_Eastern
    {  414,  174,  514068,  514085,  514109, 17, 24, 23, }, // America_Mountain
    {  414,  191,  359401,  514132,  514152, 13, 20, 19, }, // America_Pacific
    {  414,  207,  359460,  514171,  359492, 12, 19, 24, }, // Anadyr
    {  414,  214,  359516,  514190,  514207, 10, 17, 16, }, // Apia
    {  414,  219,  359566,  514223,  359596, 11, 20, 23, }, // Aqtau
    {  414,  225,  359619,  514243,  359651, 12, 19, 24, }, // Aqtobe
    {  414,  232,  359675,  514262,  514279, 10, 17, 16, }, // Arabian
    {  414,  240,  359725,  514295,  359763, 15, 22, 27, }, // Argentina
    {  414,  250,  514317,  514338,  514366, 21, 28, 33, }, // Argentina_Western
    {  414,  268,  359894,  514399,  359928, 13, 20, 25, }, // Armenia
    {  414,  286,  359953,  514419,  514440, 14, 21, 20, }, // Atlantic
    {  414,  295,  514460,  514482,  514511, 22, 29, 28, }, // Australia_Central
    {  414,  313,  360101,  514539,  514574, 28, 35, 34, }, // Australia_CentralWestern
    {  414,  338,  514608,  514629,  514657, 21, 28, 27, }, // Australia_Eastern
    {  414,  356,  514684,  514705,  514733, 21, 28, 27, }, // Australia_Western
    {  414,  374,  360371,  514760,  360411, 16, 23, 28, }, // Azerbaijan
    {  414,  385,  360439,  514783,  360471, 12, 19, 24, }, // Azores
    {  414,  397,  360495,  514802,  360535, 16, 23, 28, }, // Bangladesh
    {  414,  415,       0,  360563,       0,  0, 12,  0, }, // Bhutan
    {  414,  422,       0,  360575,       0,  0, 13,  0, }, // Bolivia
    {  414,  437,  514825,  514839,  514860, 14, 21, 26, }, // Brasilia
    {  414,  454,       0,  360644,       0,  0, 23,  0, }, // Brunei
    {  414,  461,  360667,  514886,  360713, 19, 26, 31, }, // Cape_Verde
    {  414,  478,       0,  514912,       0,  0, 21,  0, }, // Chamorro
    {  414,  487,  360777,  514933,  514953, 13, 20, 19, }, // Chatham
    {  414,  495,  514972,  514983,  515001, 11, 18, 23, }, // Chile
    {  414,  501,  515024,  515035,  515053, 11, 18, 17, }, // China
    {  414,  507,       0,  515070,       0,  0, 21,  0, }, // Christmas
    {  414,  517,       0,  360965,       0,  0, 21,  0, }, // Cocos
    {  414,  523,  515091,  515105,  515126, 14, 21, 26, }, // Colombia
    {  414,  532,  515152,  515172,  515199, 20, 27, 40, }, // Cook
    {  414,  537,  515239,  515249,  515266, 10, 17, 16, }, // Cuba
    {  414,  548,       0,  361170,       0,  0, 11,  0, }, // Davis
    {  414,  564,       0,  361181,       0,  0, 22,  0, }, // DumontDUrville
    {  414,  601,       0,  515282,       0,  0, 17,  0, }, // East_Timor
    {  414,  612,  515299,  515317,  515342, 18, 25, 30, }, // Easter
    {  414,  619,       0,  515372,       0,  0, 13,  0, }, // Ecuador
    {  414,  627,  515385,  515404,  515430, 19, 26, 31, }, // Europe_Central
    {  414,  642,  515461,  515479,  515504, 18, 25, 30, }, // Europe_Eastern
    {  414,  657,       0,  515534,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  414,  680,  515558,  515576,  515601, 18, 25, 30, }, // Europe_Western
    {  414,  695,  361545,  515631,  361601, 24, 31, 36, }, // Falkland
    {  414,  704,  361637,  515662,  361665, 10, 17, 22, }, // Fiji
    {  414,  709,       0,  515679,       0,  0, 21,  0, }, // French_Guiana
    {  414,  723,       0,  515700,       0,  0, 36,  0, }, // French_Southern
    {  414,  746,       0,  361751,       0,  0, 15,  0, }, // Galapagos
    {  414,  756,       0,  361766,       0,  0, 13,  0, }, // Gambier
    {  414,  764,  361779,  515736,  361813, 13, 20, 25, }, // Georgia
    {  414,  772,       0,  515756,       0,  0, 23,  0, }, // Gilbert_Islands
    {  414,  788,       0,  515779,       0,  0, 19,  0, }, // GMT
    {  414,  830,  361856,  515798,  361906, 21, 28, 33, }, // Greenland_Eastern
    {  414,  848,  361939,  515826,  361989, 21, 28, 33, }, // Greenland_Western
    {  414,  871,       0,  515854,       0,  0, 18,  0, }, // Gulf
    {  414,  876,       0,  361687,       0,  0, 12,  0, }, // Guyana
    {  414,  883,  362051,  515872,  515900, 21, 28, 27, }, // Hawaii_Aleutian
    {  414,  899,  362134,  515927,  362172, 15, 22, 27, }, // Hong_Kong
    {  414,  909,  362199,  515949,  362227, 10, 17, 22, }, // Hovd
    {  414,  914,       0,  515966,       0,  0, 18,  0, }, // India
    {  414,  920,       0,  515984,       0,  0, 18,  0, }, // Indian_Ocean
    {  414,  933,       0,  362281,       0,  0, 15,  0, }, // Indochina
    {  414,  943,       0,  362296,       0,  0, 22,  0, }, // Indonesia_Central
    {  414,  961,       0,  362318,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  414,  979,       0,  362339,       0,  0, 21,  0, }, // Indonesia_Western
    {  414,  997,  362360,  516002,  516019, 10, 17, 16, }, // Iran
    {  414, 1008,  362410,  516035,  362444, 13, 20, 25, }, // Irkutsk
    {  414, 1016,  362469,  516055,  516074, 12, 19, 18, }, // Israel
    {  414, 1023,  516092,  516103,  516121, 11, 18, 17, }, // Japan
    {  414, 1029,  516138,  516168,  362649, 30, 37, 42, }, // Kamchatka
    {  414, 1047,       0,  362691,       0,  0, 16,  0, }, // Kazakhstan
    {  414, 1058,       0,  362707,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  414, 1077,       0,  362729,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  414, 1106,  362751,  516205,  516223, 11, 18, 17, }, // Korea
    {  414, 1112,       0,  362804,       0,  0, 12,  0, }, // Kosrae
    {  414, 1119,  362816,  516240,  362858, 17, 24, 29, }, // Krasnoyarsk
    {  414, 1151,       0,  516264,       0,  0, 15,  0, }, // Kyrgystan
    {  414, 1175,       0,  516279,       0,  0, 20,  0, }, // Line_Islands
    {  414, 1188,  362929,  516299,  516321, 15, 22, 21, }, // Lord_Howe
    {  414, 1198,  516342,  516353,  516371, 11, 18, 23, }, // Macau
    {  414, 1204,  363047,  516394,  363081, 13, 20, 25, }, // Magadan
    {  414, 1219,       0,  363106,       0,  0, 14,  0, }, // Malaysia
    {  414, 1228,       0,  516414,       0,  0, 14,  0, }, // Maldives
    {  414, 1237,       0,  363134,       0,  0, 15,  0, }, // Marquesas
    {  414, 1247,       0,  516428,       0,  0, 24,  0, }, // Marshall_Islands
    {  414, 1264,  363168,  516452,  363206, 15, 22, 27, }, // Mauritius
    {  414, 1274,       0,  363233,       0,  0, 12,  0, }, // Mawson
    {  414, 1281,  516474,  516494,  516521, 20, 27, 26, }, // Mexico_Pacific
    {  414, 1296,  363328,  516547,  363368, 16, 23, 28, }, // Mongolia
    {  414, 1305,  516570,  516582,  516601, 12, 19, 24, }, // Moscow
    {  414, 1312,       0,  363452,       0,  0, 13,  0, }, // Myanmar
    {  414, 1320,       0,  363465,       0,  0, 11,  0, }, // Nauru
    {  414, 1326,       0,  363476,       0,  0, 11,  0, }, // Nepal
    {  414, 1332,  516625,  516644,  516670, 19, 26, 31, }, // New_Caledonia
    {  414, 1346,  516701,  516718,  516742, 17, 24, 23, }, // New_Zealand
    {  414, 1358,  363644,  516765,  516790, 18, 25, 24, }, // Newfoundland
    {  414, 1371,       0,  363718,       0,  0, 10,  0, }, // Niue
    {  414, 1376,  516814,  516837,  516867, 23, 30, 29, }, // Norfolk
    {  414, 1384,  363805,  516896,  363863, 25, 32, 37, }, // Noronha
    {  414, 1392,       0,  516928,       0,  0, 29,  0, }, // North_Mariana
    {  414, 1406,  363924,  516957,  363966, 17, 24, 29, }, // Novosibirsk
    {  414, 1418,  363995,  516981,  364023, 10, 17, 22, }, // Omsk
    {  414, 1428,  364045,  516998,  364081, 14, 21, 26, }, // Pakistan
    {  414, 1437,       0,  364107,       0,  0, 11,  0, }, // Palau
    {  414, 1443,       0,  517019,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  414, 1460,  364136,  517041,  364172, 14, 21, 26, }, // Paraguay
    {  414, 1469,  364198,  517062,  364226, 10, 17, 22, }, // Peru
    {  414, 1474,  364248,  517079,  364284, 14, 21, 26, }, // Philippines
    {  414, 1486,       0,  364310,       0,  0, 23,  0, }, // Phoenix_Islands
    {  414, 1502,  364333,  517100,  517138, 31, 38, 37, }, // Pierre_Miquelon
    {  414, 1518,       0,  364446,       0,  0, 14,  0, }, // Pitcairn
    {  414, 1527,       0,  364460,       0,  0, 12,  0, }, // Ponape
    {  414, 1534,       0,  364472,       0,  0, 15,  0, }, // Pyongyang
    {  414, 1544,  364487,  517175,  364525, 15, 22, 27, }, // Qyzylorda
    {  414, 1554,       0,  364552,       0,  0, 13,  0, }, // Reunion
    {  414, 1562,       0,  364565,       0,  0, 13,  0, }, // Rothera
    {  414, 1570,  364578,  517197,  364614, 14, 21, 26, }, // Sakhalin
    {  414, 1579,  364640,  517218,  364672, 12, 19, 24, }, // Samara
    {  414, 1596,  364696,  517237,  364726, 11, 18, 23, }, // Samoa
    {  414, 1602,       0,  364749,       0,  0, 16,  0, }, // Seychelles
    {  414, 1624,       0,  517255,       0,  0, 22,  0, }, // Singapore
    {  414, 1634,       0,  364788,       0,  0, 23,  0, }, // Solomon
    {  414, 1642,       0,  364811,       0,  0, 21,  0, }, // South_Georgia
    {  414, 1656,       0,  364832,       0,  0, 14,  0, }, // Suriname
    {  414, 1676,       0,  364846,       0,  0, 11,  0, }, // Syowa
    {  414, 1682,       0,  364857,       0,  0, 12,  0, }, // Tahiti
    {  414, 1689,  364869,  517277,  517296, 12, 19, 18, }, // Taipei
    {  414, 1696,       0,  364925,       0,  0, 16,  0, }, // Tajikistan
    {  414, 1724,       0,  364941,       0,  0, 13,  0, }, // Tokelau
    {  414, 1732,  364954,  517314,  364984, 11, 18, 23, }, // Tonga
    {  414, 1738,       0,  365007,       0,  0, 11,  0, }, // Truk
    {  414, 1750,  365018,  517332,  365062, 18, 25, 30, }, // Turkmenistan
    {  414, 1763,       0,  365092,       0,  0, 12,  0, }, // Tuvalu
    {  414, 1777,  365104,  517357,  365138, 13, 20, 25, }, // Uruguay
    {  414, 1792,  365163,  517377,  365203, 16, 23, 28, }, // Uzbekistan
    {  414, 1803,  365231,  517400,  365265, 13, 20, 25, }, // Vanuatu
    {  414, 1811,       0,  365290,       0,  0, 15,  0, }, // Venezuela
    {  414, 1821,  365305,  517420,  365347, 17, 24, 29, }, // Vladivostok
    {  414, 1833,  365376,  517444,  365414, 15, 22, 27, }, // Volgograd
    {  414, 1843,       0,  365441,       0,  0, 12,  0, }, // Vostok
    {  414, 1850,       0,  517466,       0,  0, 16,  0, }, // Wake
    {  414, 1855,       0,  365473,       0,  0, 23,  0, }, // Wallis
    {  414, 1862,  365496,  517482,  365530, 13, 20, 25, }, // Yakutsk
    {  414, 1870,  365555,  517502,  365601, 19, 26, 31, }, // Yekaterinburg
    {  414, 1892,       0,  517528,       0,  0, 10,  0, }, // Yukon
    {  415,  454,       0,  517538,       0,  0, 20,  0, }, // Brunei Malay/Arabic/Brunei
    {  415,  914,       0,  517558,       0,  0, 18,  0, }, // India
    {  415,  920,       0,  517576,       0,  0, 17,  0, }, // Indian_Ocean
    {  415,  943,       0,  517593,       0,  0, 18,  0, }, // Indonesia_Central
    {  415,  961,       0,  517611,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  415,  979,       0,  517631,       0,  0, 20,  0, }, // Indonesia_Western
    {  415, 1219,       0,  517651,       0,  0, 11,  0, }, // Malaysia
    {  416,  454,       0,  517538,       0,  0, 20,  0, }, // Brunei Malay/Arabic/Malaysia
    {  416,  914,       0,  517558,       0,  0, 18,  0, }, // India
    {  416,  920,       0,  517576,       0,  0, 17,  0, }, // Indian_Ocean
    {  416,  943,       0,  517593,       0,  0, 18,  0, }, // Indonesia_Central
    {  416,  961,       0,  517611,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  416,  979,       0,  517631,       0,  0, 20,  0, }, // Indonesia_Western
    {  416, 1219,       0,  517651,       0,  0, 11,  0, }, // Malaysia
    {  420,  627,  517662,  517682,  517711, 20, 29, 29, }, // Europe_Central Maltese/Latin/Malta
    {  420,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {  421,  142,  517740,  517769,  517809, 29, 40, 36, }, // America_Central Manipuri/Bangla/India
    {  421,  158,  517845,  517872,  517910, 27, 38, 34, }, // America_Eastern
    {  421,  174,  517944,  517972,  518011, 28, 39, 35, }, // America_Mountain
    {  421,  191,  518046,  518073,  518111, 27, 38, 34, }, // America_Pacific
    {  421,  286,  518145,  518159,  518184, 14, 25, 21, }, // Atlantic
    {  421,  627,  518205,  518231,  518268, 26, 37, 30, }, // Europe_Central
    {  421,  642,  518298,  518322,  518357, 24, 35, 28, }, // Europe_Eastern
    {  421,  680,  518385,  518410,  518446, 25, 36, 29, }, // Europe_Western
    {  421,  788,       0,  518475,       0,  0, 17,  0, }, // GMT
    {  424,    5,       0,  518492,       0,  0, 15,  0, }, // Afghanistan Maori/Latin/New Zealand
    {  424,   17,       0,  518507,       0,  0, 23,  0, }, // Africa_Central
    {  424,   32,       0,  518530,       0,  0, 27,  0, }, // Africa_Eastern
    {  424,   65,       0,  518557,       0,  0, 35,  0, }, // Africa_Southern
    {  424,   81,  518592,  518615,  518647, 23, 32, 31, }, // Africa_Western
    {  424,  107,  518678,  518687,  518706,  9, 19, 16, }, // Alaska
    {  424,  135,  518722,  518732,  518752, 10, 20, 18, }, // Amazon
    {  424,  142,  518770,  518779,  518798,  9, 19, 16, }, // America_Central
    {  424,  158,  518814,  518824,  518844, 10, 20, 17, }, // America_Eastern
    {  424,  174,  518861,  518870,  518889,  9, 19, 16, }, // America_Mountain
    {  424,  191,  518905,  518912,  518929,  7, 17, 14, }, // America_Pacific
    {  424,  214,  518943,  518950,  518967,  7, 17, 14, }, // Apia
    {  424,  232,  518981,  518990,  519009,  9, 19, 16, }, // Arabian
    {  424,  240,  519025,  519035,  519055, 10, 20, 18, }, // Argentina
    {  424,  250,  519073,  519093,  519123, 20, 30, 28, }, // Argentina_Western
    {  424,  268,  519151,  519160,  519179,  9, 19, 17, }, // Armenia
    {  424,  286,  519196,  519207,  519228, 11, 21, 18, }, // Atlantic
    {  424,  295,  519246,  519267,  519298, 21, 31, 28, }, // Australia_Central
    {  424,  313,  519326,  519351,  519386, 25, 35, 32, }, // Australia_CentralWestern
    {  424,  338,  519418,  519446,  519484, 28, 38, 35, }, // Australia_Eastern
    {  424,  356,  519519,  519543,  519577, 24, 34, 31, }, // Australia_Western
    {  424,  374,  519608,  519622,  519646, 14, 24, 22, }, // Azerbaijan
    {  424,  385,  519668,  519677,  519696,  9, 19, 17, }, // Azores
    {  424,  397,  519713,  519726,  519749, 13, 23, 21, }, // Bangladesh
    {  424,  415,       0,  519770,       0,  0,  9,  0, }, // Bhutan
    {  424,  422,       0,  519779,       0,  0, 10,  0, }, // Bolivia
    {  424,  437,  519789,  519799,  519819, 10, 20, 18, }, // Brasilia
    {  424,  454,       0,  519837,       0,  0, 21,  0, }, // Brunei
    {  424,  461,  519858,  519880,  519912, 22, 32, 30, }, // Cape_Verde
    {  424,  478,       0,  519942,       0,  0, 21,  0, }, // Chamorro
    {  424,  487,  519963,  519972,  519991,  9, 19, 16, }, // Chatham
    {  424,  495,  520007,  520014,  520031,  7, 17, 15, }, // Chile
    {  424,  501,  520046,  520054,  520072,  8, 18, 15, }, // China
    {  424,  507,       0,  520087,       0,  0, 26,  0, }, // Christmas
    {  424,  517,       0,  520113,       0,  0, 23,  0, }, // Cocos
    {  424,  523,  520136,  520148,  520170, 12, 22, 20, }, // Colombia
    {  424,  532,  520190,  520204,  520228, 14, 24, 29, }, // Cook
    {  424,  537,  520257,  520265,  520283,  8, 18, 15, }, // Cuba
    {  424,  548,       0,  520298,       0,  0,  9,  0, }, // Davis
    {  424,  564,       0,  520307,       0,  0, 19,  0, }, // DumontDUrville
    {  424,  601,       0,  520326,       0,  0, 24,  0, }, // East_Timor
    {  424,  612,  520350,  520375,  520408, 25, 33, 31, }, // Easter
    {  424,  619,       0,  520439,       0,  0, 10,  0, }, // Ecuador
    {  424,  627,  520449,  520464,  520489, 15, 25, 23, }, // Europe_Central
    {  424,  642,  520512,  520528,  520554, 16, 26, 24, }, // Europe_Eastern
    {  424,  657,       0,  520578,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  424,  680,  520605,  520617,  520639, 12, 22, 20, }, // Europe_Western
    {  424,  695,  520659,  520681,  520713, 22, 32, 30, }, // Falkland
    {  424,  704,  520743,  520751,  520769,  8, 18, 16, }, // Fiji
    {  424,  709,       0,  520785,       0,  0, 13,  0, }, // French_Guiana
    {  424,  723,       0,  520798,       0,  0, 48,  0, }, // French_Southern
    {  424,  746,       0,  520846,       0,  0, 12,  0, }, // Galapagos
    {  424,  756,       0,  520858,       0,  0, 10,  0, }, // Gambier
    {  424,  764,  520868,  520876,  520894,  8, 18, 16, }, // Georgia
    {  424,  772,       0,  520910,       0,  0, 11,  0, }, // Gilbert_Islands
    {  424,  788,       0,  520921,       0,  0, 20,  0, }, // GMT
    {  424,  830,  520941,  520970,  521011, 29, 41, 39, }, // Greenland_Eastern
    {  424,  848,  521050,  521075,  521112, 25, 37, 35, }, // Greenland_Western
    {  424,  871,       0,  521147,       0,  0, 19,  0, }, // Gulf
    {  424,  876,       0,  521166,       0,  0,  9,  0, }, // Guyana
    {  424,  883,  521175,  521193,  521221, 18, 28, 25, }, // Hawaii_Aleutian
    {  424,  899,  521246,  521257,  521278, 11, 21, 19, }, // Hong_Kong
    {  424,  909,  521297,  521304,  521321,  7, 17, 15, }, // Hovd
    {  424,  914,       0,  521336,       0,  0,  7,  0, }, // India
    {  424,  920,       0,  521343,       0,  0, 18,  0, }, // Indian_Ocean
    {  424,  933,       0,  521361,       0,  0, 12,  0, }, // Indochina
    {  424,  943,       0,  521373,       0,  0, 20,  0, }, // Indonesia_Central
    {  424,  961,       0,  521393,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  424,  979,       0,  521420,       0,  0, 23,  0, }, // Indonesia_Western
    {  424,  997,  521443,  521451,  521469,  8, 18, 15, }, // Iran
    {  424, 1008,  521484,  521494,  521514, 10, 20, 18, }, // Irkutsk
    {  424, 1016,  521532,  521543,  521564, 11, 21, 18, }, // Israel
    {  424, 1023,  521582,  521591,  521610,  9, 19, 16, }, // Japan
    {  424, 1047,       0,  521626,       0,  0, 12,  0, }, // Kazakhstan
    {  424, 1058,       0,  521638,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  424, 1077,       0,  521664,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  424, 1106,  521686,  521694,  521712,  8, 18, 15, }, // Korea
    {  424, 1112,       0,  521727,       0,  0,  9,  0, }, // Kosrae
    {  424, 1119,  521736,  521750,  521774, 14, 24, 22, }, // Krasnoyarsk
    {  424, 1151,       0,  521796,       0,  0, 12,  0, }, // Kyrgystan
    {  424, 1175,       0,  521808,       0,  0, 26,  0, }, // Line_Islands
    {  424, 1188,  521834,  521846,  521868, 12, 22, 19, }, // Lord_Howe
    {  424, 1204,  521887,  521897,  521917, 10, 20, 18, }, // Magadan
    {  424, 1219,       0,  521935,       0,  0,  9,  0, }, // Malaysia
    {  424, 1228,       0,  521944,       0,  0, 11,  0, }, // Maldives
    {  424, 1237,       0,  521955,       0,  0, 12,  0, }, // Marquesas
    {  424, 1247,       0,  521967,       0,  0, 20,  0, }, // Marshall_Islands
    {  424, 1264,  521987,  521996,  522015,  9, 19, 17, }, // Mauritius
    {  424, 1274,       0,  522032,       0,  0,  9,  0, }, // Mawson
    {  424, 1281,  522041,  522055,  522079, 14, 24, 21, }, // Mexico_Pacific
    {  424, 1296,  522100,  522114,  522138, 14, 24, 22, }, // Mongolia
    {  424, 1305,  522160,  522170,  522190, 10, 20, 18, }, // Moscow
    {  424, 1312,       0,  522208,       0,  0,  7,  0, }, // Myanmar
    {  424, 1320,       0,  522215,       0,  0,  8,  0, }, // Nauru
    {  424, 1326,       0,  522223,       0,  0,  9,  0, }, // Nepal
    {  424, 1332,  522232,  522248,  522274, 16, 26, 24, }, // New_Caledonia
    {  424, 1346,  522298,  522309,  522330, 11, 21, 18, }, // New_Zealand
    {  424, 1358,  522348,  522363,  522388, 15, 25, 22, }, // Newfoundland
    {  424, 1371,       0,  522410,       0,  0,  7,  0, }, // Niue
    {  424, 1376,  522417,  522439,  522471, 22, 32, 29, }, // Norfolk
    {  424, 1384,  522500,  522522,  522554, 22, 32, 30, }, // Noronha
    {  424, 1406,  522584,  522598,  522622, 14, 24, 22, }, // Novosibirsk
    {  424, 1418,  522644,  522651,  522668,  7, 17, 15, }, // Omsk
    {  424, 1428,  522683,  522694,  522715, 11, 21, 19, }, // Pakistan
    {  424, 1437,       0,  522734,       0,  0,  8,  0, }, // Palau
    {  424, 1443,       0,  522742,       0,  0, 15,  0, }, // Papua_New_Guinea
    {  424, 1460,  522757,  522767,  522787, 10, 20, 18, }, // Paraguay
    {  424, 1469,  522805,  522812,  522829,  7, 17, 15, }, // Peru
    {  424, 1474,  522844,  522855,  522876, 11, 21, 19, }, // Philippines
    {  424, 1486,       0,  522895,       0,  0, 24,  0, }, // Phoenix_Islands
    {  424, 1502,  522919,  522944,  522981, 25, 37, 34, }, // Pierre_Miquelon
    {  424, 1518,       0,  523015,       0,  0, 11,  0, }, // Pitcairn
    {  424, 1527,       0,  523026,       0,  0,  9,  0, }, // Ponape
    {  424, 1534,       0,  523035,       0,  0, 12,  0, }, // Pyongyang
    {  424, 1554,       0,  523047,       0,  0, 10,  0, }, // Reunion
    {  424, 1562,       0,  523057,       0,  0, 10,  0, }, // Rothera
    {  424, 1570,  523067,  523078,  523099, 11, 21, 19, }, // Sakhalin
    {  424, 1596,  523118,  523126,  523144,  8, 18, 15, }, // Samoa
    {  424, 1602,       0,  523159,       0,  0, 10,  0, }, // Seychelles
    {  424, 1624,       0,  523169,       0,  0, 21,  0, }, // Singapore
    {  424, 1634,       0,  523190,       0,  0, 22,  0, }, // Solomon
    {  424, 1642,       0,  523212,       0,  0, 20,  0, }, // South_Georgia
    {  424, 1656,       0,  523232,       0,  0, 11,  0, }, // Suriname
    {  424, 1676,       0,  523243,       0,  0,  8,  0, }, // Syowa
    {  424, 1682,       0,  523251,       0,  0,  9,  0, }, // Tahiti
    {  424, 1689,  523260,  523269,  523288,  9, 19, 16, }, // Taipei
    {  424, 1696,       0,  523304,       0,  0, 14,  0, }, // Tajikistan
    {  424, 1724,       0,  523318,       0,  0, 10,  0, }, // Tokelau
    {  424, 1732,  523328,  523336,  523354,  8, 18, 16, }, // Tonga
    {  424, 1738,       0,  523370,       0,  0,  8,  0, }, // Truk
    {  424, 1750,  523378,  523394,  523420, 16, 26, 24, }, // Turkmenistan
    {  424, 1763,       0,  523444,       0,  0,  9,  0, }, // Tuvalu
    {  424, 1777,  523453,  523462,  523481,  9, 19, 17, }, // Uruguay
    {  424, 1792,  523498,  523512,  523536, 14, 24, 22, }, // Uzbekistan
    {  424, 1803,  523558,  523569,  523590, 11, 21, 19, }, // Vanuatu
    {  424, 1811,       0,  523609,       0,  0, 12,  0, }, // Venezuela
    {  424, 1821,  523621,  523635,  523659, 14, 24, 22, }, // Vladivostok
    {  424, 1833,  523681,  523693,  523715, 12, 22, 20, }, // Volgograd
    {  424, 1843,       0,  523735,       0,  0,  9,  0, }, // Vostok
    {  424, 1850,       0,  523744,       0,  0, 17,  0, }, // Wake
    {  424, 1855,       0,  523761,       0,  0, 20,  0, }, // Wallis
    {  424, 1862,  523781,  523791,  523811, 10, 20, 18, }, // Yakutsk
    {  424, 1870,  523829,  523845,  523871, 16, 26, 24, }, // Yekaterinburg
    {  424, 1892,       0,  523895,       0,  0,  8,  0, }, // Yukon
    {  426,    0,  523903,  523910,  523923,  7, 13, 15, }, // Acre Marathi/Devanagari/India
    {  426,    5,       0,  523938,       0,  0, 15,  0, }, // Afghanistan
    {  426,   17,       0,  523953,       0,  0, 17,  0, }, // Africa_Central
    {  426,   32,       0,  523970,       0,  0, 17,  0, }, // Africa_Eastern
    {  426,   65,       0,  523987,       0,  0, 25,  0, }, // Africa_Southern
    {  426,   81,  524012,  524030,  524055, 18, 25, 26, }, // Africa_Western
    {  426,  107,  438253,  524081,  524099, 11, 18, 23, }, // Alaska
    {  426,  128,  524122,  524133,  524150, 11, 17, 24, }, // Almaty
    {  426,  135,  524174,  524186,  524205, 12, 19, 20, }, // Amazon
    {  426,  142,  524225,  524237,  524256, 12, 19, 24, }, // America_Central
    {  426,  158,  524280,  524293,  524313, 13, 20, 25, }, // America_Eastern
    {  426,  174,  438432,  524338,  524356, 11, 18, 23, }, // America_Mountain
    {  426,  191,  524379,  524390,  524408, 11, 18, 23, }, // America_Pacific
    {  426,  207,  524431,  524442,  524458, 11, 16, 23, }, // Anadyr
    {  426,  214,  524481,  524490,  524506,  9, 16, 21, }, // Apia
    {  426,  219,  524527,  524539,  524557, 12, 18, 25, }, // Aqtau
    {  426,  225,  524582,  524595,  524614, 13, 19, 26, }, // Aqtobe
    {  426,  232,  524640,  524651,  524669, 11, 18, 23, }, // Arabian
    {  426,  240,  438613,  524692,  524713, 14, 21, 22, }, // Argentina
    {  426,  250,  524735,  524757,  524786, 22, 29, 30, }, // Argentina_Western
    {  426,  268,  438752,  524816,  524836, 13, 20, 21, }, // Armenia
    {  426,  286,  524857,  524869,  524888, 12, 19, 24, }, // Atlantic
    {  426,  295,  524912,  524932,  524959, 20, 27, 32, }, // Australia_Central
    {  426,  313,  524991,  525018,  525052, 27, 34, 39, }, // Australia_CentralWestern
    {  426,  338,  525091,  525112,  525140, 21, 28, 33, }, // Australia_Eastern
    {  426,  356,  525173,  525195,  525224, 22, 29, 34, }, // Australia_Western
    {  426,  374,  525258,  525270,  525289, 12, 19, 20, }, // Azerbaijan
    {  426,  385,  525309,  525321,  525340, 12, 19, 20, }, // Azores
    {  426,  397,  439290,  525360,  525380, 13, 20, 21, }, // Bangladesh
    {  426,  415,       0,  525401,       0,  0,  9,  0, }, // Bhutan
    {  426,  422,       0,  439351,       0,  0, 14,  0, }, // Bolivia
    {  426,  437,  439365,  525410,  525431, 14, 21, 22, }, // Brasilia
    {  426,  454,       0,  525453,       0,  0, 20,  0, }, // Brunei
    {  426,  461,  525473,  525488,  525510, 15, 22, 23, }, // Cape_Verde
    {  426,  478,       0,  525533,       0,  0, 17,  0, }, // Chamorro
    {  426,  487,  439521,  525550,  525565,  8, 15, 20, }, // Chatham
    {  426,  495,  439561,  525585,  525600,  8, 15, 16, }, // Chile
    {  426,  501,  525616,  525624,  525639,  8, 15, 20, }, // China
    {  426,  507,       0,  525659,       0,  0, 15,  0, }, // Christmas
    {  426,  517,       0,  525674,       0,  0, 14,  0, }, // Cocos
    {  426,  523,  439671,  525688,  525707, 12, 19, 20, }, // Colombia
    {  426,  532,  525727,  525739,  525758, 12, 19, 25, }, // Cook
    {  426,  537,  525783,  525793,  525810, 10, 17, 22, }, // Cuba
    {  426,  548,       0,  525832,       0,  0, 11,  0, }, // Davis
    {  426,  564,       0,  525843,       0,  0, 23,  0, }, // DumontDUrville
    {  426,  601,       0,  525866,       0,  0, 15,  0, }, // East_Timor
    {  426,  612,  525881,  525894,  525914, 13, 20, 21, }, // Easter
    {  426,  619,       0,  439937,       0,  0, 12,  0, }, // Ecuador
    {  426,  627,  525935,  525953,  525978, 18, 25, 26, }, // Europe_Central
    {  426,  642,  526004,  526022,  526047, 18, 25, 26, }, // Europe_Eastern
    {  426,  657,       0,  526073,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  426,  680,  526100,  526119,  526145, 19, 26, 27, }, // Europe_Western
    {  426,  695,  526172,  526187,  526209, 15, 22, 23, }, // Falkland
    {  426,  704,  440256,  526232,  526247,  8, 15, 16, }, // Fiji
    {  426,  709,       0,  440297,       0,  0, 16,  0, }, // French_Guiana
    {  426,  723,       0,  526263,       0,  0, 33,  0, }, // French_Southern
    {  426,  746,       0,  526296,       0,  0, 13,  0, }, // Galapagos
    {  426,  756,       0,  526309,       0,  0, 12,  0, }, // Gambier
    {  426,  764,  440372,  526321,  526340, 12, 19, 20, }, // Georgia
    {  426,  772,       0,  526360,       0,  0, 17,  0, }, // Gilbert_Islands
    {  426,  788,       0,  526377,       0,  0, 18,  0, }, // GMT
    {  426,  830,  526395,  526413,  526438, 18, 25, 26, }, // Greenland_Eastern
    {  426,  848,  526464,  526483,  526509, 19, 26, 27, }, // Greenland_Western
    {  426,  866,       0,  526536,       0,  0, 14,  0, }, // Guam
    {  426,  871,       0,  526550,       0,  0, 15,  0, }, // Gulf
    {  426,  876,       0,  440304,       0,  0,  9,  0, }, // Guyana
    {  426,  883,  526565,  526579,  526600, 14, 21, 26, }, // Hawaii_Aleutian
    {  426,  899,  440699,  526626,  526646, 13, 20, 21, }, // Hong_Kong
    {  426,  909,  526667,  526678,  526696, 11, 18, 19, }, // Hovd
    {  426,  914,       0,  526715,       0,  0, 17,  0, }, // India
    {  426,  920,       0,  526732,       0,  0, 16,  0, }, // Indian_Ocean
    {  426,  933,       0,  526748,       0,  0, 13,  0, }, // Indochina
    {  426,  943,       0,  526761,       0,  0, 20,  0, }, // Indonesia_Central
    {  426,  961,       0,  526781,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  426,  979,       0,  526805,       0,  0, 22,  0, }, // Indonesia_Western
    {  426,  997,  526827,  526835,  526850,  8, 15, 20, }, // Iran
    {  426, 1008,  526870,  526883,  526903, 13, 20, 21, }, // Irkutsk
    {  426, 1016,  526924,  526931,  526949,  7, 18, 23, }, // Israel
    {  426, 1023,  441042,  526972,  526987,  8, 15, 20, }, // Japan
    {  426, 1029,  527007,  527040,  527079, 33, 39, 46, }, // Kamchatka
    {  426, 1047,       0,  441101,       0,  0, 13,  0, }, // Kazakhstan
    {  426, 1058,       0,  527125,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  426, 1077,       0,  527144,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  426, 1106,  441134,  527164,  527181, 10, 17, 22, }, // Korea
    {  426, 1112,       0,  527203,       0,  0, 11,  0, }, // Kosrae
    {  426, 1119,  441189,  527214,  527240, 19, 26, 27, }, // Krasnoyarsk
    {  426, 1151,       0,  527267,       0,  0, 14,  0, }, // Kyrgystan
    {  426, 1161,       0,  527281,       0,  0,  8,  0, }, // Lanka
    {  426, 1175,       0,  527289,       0,  0, 13,  0, }, // Line_Islands
    {  426, 1188,  441294,  527302,  527323, 14, 21, 26, }, // Lord_Howe
    {  426, 1198,  527349,  527357,  527371,  8, 14, 21, }, // Macau
    {  426, 1204,  527392,  527402,  527419, 10, 17, 18, }, // Magadan
    {  426, 1219,       0,  441399,       0,  0, 11,  0, }, // Malaysia
    {  426, 1228,       0,  527437,       0,  0, 10,  0, }, // Maldives
    {  426, 1237,       0,  527447,       0,  0, 15,  0, }, // Marquesas
    {  426, 1247,       0,  527462,       0,  0, 15,  0, }, // Marshall_Islands
    {  426, 1264,  527477,  527487,  527504, 10, 17, 18, }, // Mauritius
    {  426, 1274,       0,  441495,       0,  0,  8,  0, }, // Mawson
    {  426, 1281,  527522,  527542,  527569, 20, 27, 32, }, // Mexico_Pacific
    {  426, 1296,  527601,  527615,  527636, 14, 21, 22, }, // Mongolia
    {  426, 1305,  441632,  527658,  527675, 10, 17, 18, }, // Moscow
    {  426, 1312,       0,  441679,       0,  0, 12,  0, }, // Myanmar
    {  426, 1320,       0,  527693,       0,  0,  8,  0, }, // Nauru
    {  426, 1326,       0,  441698,       0,  0,  9,  0, }, // Nepal
    {  426, 1332,  527701,  527720,  527746, 19, 26, 27, }, // New_Caledonia
    {  426, 1346,  527773,  527786,  527806, 13, 20, 25, }, // New_Zealand
    {  426, 1358,  527831,  527848,  527872, 17, 24, 29, }, // Newfoundland
    {  426, 1371,       0,  441906,       0,  0,  9,  0, }, // Niue
    {  426, 1376,  527901,  527915,  527936, 14, 21, 26, }, // Norfolk
    {  426, 1384,  441980,  527962,  527993, 24, 31, 32, }, // Noronha
    {  426, 1392,       0,  528025,       0,  0, 20,  0, }, // North_Mariana
    {  426, 1406,  528045,  528062,  528086, 17, 24, 25, }, // Novosibirsk
    {  426, 1418,  442120,  528111,  528128, 10, 17, 18, }, // Omsk
    {  426, 1428,  528146,  528159,  528179, 13, 20, 21, }, // Pakistan
    {  426, 1437,       0,  442210,       0,  0,  8,  0, }, // Palau
    {  426, 1443,       0,  528200,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  426, 1460,  528219,  528231,  528250, 12, 19, 20, }, // Paraguay
    {  426, 1469,  528270,  528278,  528293,  8, 15, 16, }, // Peru
    {  426, 1474,  528309,  528321,  528340, 12, 19, 20, }, // Philippines
    {  426, 1486,       0,  528360,       0,  0, 17,  0, }, // Phoenix_Islands
    {  426, 1502,  528377,  528404,  528439, 27, 35, 40, }, // Pierre_Miquelon
    {  426, 1518,       0,  528479,       0,  0, 12,  0, }, // Pitcairn
    {  426, 1527,       0,  528491,       0,  0, 10,  0, }, // Ponape
    {  426, 1534,       0,  442503,       0,  0, 14,  0, }, // Pyongyang
    {  426, 1544,  528501,  528516,  528537, 15, 21, 28, }, // Qyzylorda
    {  426, 1554,       0,  442517,       0,  0, 12,  0, }, // Reunion
    {  426, 1562,       0,  442529,       0,  0, 10,  0, }, // Rothera
    {  426, 1570,  442539,  528565,  528581,  9, 16, 17, }, // Sakhalin
    {  426, 1579,  528598,  528607,  528622,  9, 15, 22, }, // Samara
    {  426, 1596,  442583,  528644,  528660,  9, 16, 21, }, // Samoa
    {  426, 1602,       0,  442626,       0,  0, 11,  0, }, // Seychelles
    {  426, 1624,       0,  528681,       0,  0, 19,  0, }, // Singapore
    {  426, 1634,       0,  528700,       0,  0, 16,  0, }, // Solomon
    {  426, 1642,       0,  442676,       0,  0, 19,  0, }, // South_Georgia
    {  426, 1656,       0,  442695,       0,  0, 11,  0, }, // Suriname
    {  426, 1676,       0,  442706,       0,  0, 10,  0, }, // Syowa
    {  426, 1682,       0,  442716,       0,  0, 10,  0, }, // Tahiti
    {  426, 1689,  442726,  528716,  528732,  9, 16, 21, }, // Taipei
    {  426, 1696,       0,  528753,       0,  0, 15,  0, }, // Tajikistan
    {  426, 1724,       0,  528768,       0,  0, 11,  0, }, // Tokelau
    {  426, 1732,  442793,  528779,  528795,  9, 16, 17, }, // Tonga
    {  426, 1738,       0,  528812,       0,  0,  7,  0, }, // Truk
    {  426, 1750,  528819,  528837,  528862, 18, 25, 26, }, // Turkmenistan
    {  426, 1763,       0,  528888,       0,  0, 10,  0, }, // Tuvalu
    {  426, 1777,  442907,  528898,  528916, 11, 18, 19, }, // Uruguay
    {  426, 1792,  528935,  528950,  528972, 15, 22, 23, }, // Uzbekistan
    {  426, 1803,  528995,  529006,  529024, 11, 18, 19, }, // Vanuatu
    {  426, 1811,       0,  529043,       0,  0, 15,  0, }, // Venezuela
    {  426, 1821,  443082,  529058,  529082, 17, 24, 25, }, // Vladivostok
    {  426, 1833,  443150,  529107,  529131, 17, 24, 25, }, // Volgograd
    {  426, 1843,       0,  529156,       0,  0, 13,  0, }, // Vostok
    {  426, 1850,       0,  529169,       0,  0, 11,  0, }, // Wake
    {  426, 1855,       0,  529180,       0,  0, 20,  0, }, // Wallis
    {  426, 1862,  529200,  529212,  529231, 12, 19, 20, }, // Yakutsk
    {  426, 1870,  529251,  529266,  529288, 15, 22, 23, }, // Yekaterinburg
    {  426, 1892,       0,  529311,       0,  0,  9,  0, }, // Yukon
    {  433,    5,       0,  529320,       0,  0, 15,  0, }, // Afghanistan Mongolian/Cyrillic/Mongolia
    {  433,   17,       0,  529335,       0,  0, 16,  0, }, // Africa_Central
    {  433,   32,       0,  529351,       0,  0, 17,  0, }, // Africa_Eastern
    {  433,   65,       0,  529368,       0,  0, 27,  0, }, // Africa_Southern
    {  433,   81,  529395,  529414,  529442, 19, 28, 24, }, // Africa_Western
    {  433,  107,  529466,  529478,  529499, 12, 21, 17, }, // Alaska
    {  433,  135,  529516,  529527,  529547, 11, 20, 16, }, // Amazon
    {  433,  142,  529563,  529570,  529586,  7, 16, 12, }, // America_Central
    {  433,  158,  529598,  529613,  529637, 15, 24, 20, }, // America_Eastern
    {  433,  174,  529657,  529666,  529684,  9, 18, 14, }, // America_Mountain
    {  433,  191,  529698,  529715,  529741, 17, 26, 22, }, // America_Pacific
    {  433,  214,  529763,  529775,  529796, 12, 21, 17, }, // Apia
    {  433,  232,  529813,  529823,  529842, 10, 19, 15, }, // Arabian
    {  433,  240,  529857,  529870,  529892, 13, 22, 18, }, // Argentina
    {  433,  250,  529910,  529930,  529959, 20, 29, 25, }, // Argentina_Western
    {  433,  268,  529984,  529996,  530017, 12, 21, 17, }, // Armenia
    {  433,  286,  530034,  530046,  530067, 12, 21, 17, }, // Atlantic
    {  433,  295,  530084,  530102,  530129, 18, 27, 23, }, // Australia_Central
    {  433,  313,  530152,  530184,  530225, 32, 41, 37, }, // Australia_CentralWestern
    {  433,  338,  530262,  530281,  530316, 19, 35, 31, }, // Australia_Eastern
    {  433,  356,  530347,  530368,  530405, 21, 37, 33, }, // Australia_Western
    {  433,  374,  530438,  530453,  530477, 15, 24, 20, }, // Azerbaijan
    {  433,  385,  530497,  530507,  530526, 10, 19, 15, }, // Azores
    {  433,  397,  530541,  530557,  530582, 16, 25, 21, }, // Bangladesh
    {  433,  415,       0,  530603,       0,  0, 10,  0, }, // Bhutan
    {  433,  422,       0,  530613,       0,  0, 12,  0, }, // Bolivia
    {  433,  437,  530625,  530638,  530660, 13, 22, 18, }, // Brasilia
    {  433,  454,       0,  530678,       0,  0, 23,  0, }, // Brunei
    {  433,  461,  530701,  530717,  530742, 16, 25, 21, }, // Cape_Verde
    {  433,  478,       0,  530763,       0,  0, 15,  0, }, // Chamorro
    {  433,  487,  530778,  530789,  530809, 11, 20, 16, }, // Chatham
    {  433,  495,  530825,  530835,  530854, 10, 19, 15, }, // Chile
    {  433,  501,  530869,  530880,  530900, 11, 20, 16, }, // China
    {  433,  507,       0,  530916,       0,  0, 17,  0, }, // Christmas
    {  433,  517,       0,  530933,       0,  0, 15,  0, }, // Cocos
    {  433,  523,  530948,  530960,  530981, 12, 21, 17, }, // Colombia
    {  433,  532,  530998,  531018,  531047, 20, 29, 31, }, // Cook
    {  433,  537,  531078,  531087,  531105,  9, 18, 14, }, // Cuba
    {  433,  548,       0,  531119,       0,  0, 12,  0, }, // Davis
    {  433,  564,       0,  531131,       0,  0, 20,  0, }, // DumontDUrville
    {  433,  601,       0,  531151,       0,  0, 16,  0, }, // East_Timor
    {  433,  612,  531167,  531184,  531210, 17, 26, 22, }, // Easter
    {  433,  619,       0,  531232,       0,  0, 13,  0, }, // Ecuador
    {  433,  627,  531245,  531260,  531284, 15, 24, 20, }, // Europe_Central
    {  433,  642,  531304,  531320,  531345, 16, 25, 21, }, // Europe_Eastern
    {  433,  657,       0,  531366,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  433,  680,  531388,  531406,  531433, 18, 27, 23, }, // Europe_Western
    {  433,  695,  531456,  531480,  531513, 24, 33, 29, }, // Falkland
    {  433,  704,  531542,  531554,  531575, 12, 21, 17, }, // Fiji
    {  433,  709,       0,  531592,       0,  0, 22,  0, }, // French_Guiana
    {  433,  723,       0,  531614,       0,  0, 33,  0, }, // French_Southern
    {  433,  746,       0,  531647,       0,  0, 15,  0, }, // Galapagos
    {  433,  756,       0,  531662,       0,  0, 14,  0, }, // Gambier
    {  433,  764,  531676,  531687,  531707, 11, 20, 16, }, // Georgia
    {  433,  772,       0,  531723,       0,  0, 24,  0, }, // Gilbert_Islands
    {  433,  788,       0,  531747,       0,  0, 14,  0, }, // GMT
    {  433,  830,  531761,  531780,  531808, 19, 28, 24, }, // Greenland_Eastern
    {  433,  848,  531832,  531853,  531883, 21, 30, 26, }, // Greenland_Western
    {  433,  871,       0,  531909,       0,  0, 21,  0, }, // Gulf
    {  433,  876,       0,  531930,       0,  0, 14,  0, }, // Guyana
    {  433,  883,  531944,  531961,  531987, 17, 26, 22, }, // Hawaii_Aleutian
    {  433,  899,  532009,  532025,  532050, 16, 25, 21, }, // Hong_Kong
    {  433,  909,  532071,  532081,  532100, 10, 19, 15, }, // Hovd
    {  433,  914,       0,  532115,       0,  0, 14,  0, }, // India
    {  433,  920,       0,  532129,       0,  0, 21,  0, }, // Indian_Ocean
    {  433,  933,       0,  532150,       0,  0, 27,  0, }, // Indochina
    {  433,  943,       0,  532177,       0,  0, 18,  0, }, // Indonesia_Central
    {  433,  961,       0,  532195,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  433,  979,       0,  532214,       0,  0, 21,  0, }, // Indonesia_Western
    {  433,  997,  532235,  532244,  532262,  9, 18, 14, }, // Iran
    {  433, 1008,  532276,  532289,  532311, 13, 22, 18, }, // Irkutsk
    {  433, 1016,  532329,  532342,  532364, 13, 22, 18, }, // Israel
    {  433, 1023,  532382,  532391,  532409,  9, 18, 14, }, // Japan
    {  433, 1047,       0,  532423,       0,  0, 14,  0, }, // Kazakhstan
    {  433, 1058,       0,  532437,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  433, 1077,       0,  532456,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  433, 1106,  532477,  532491,  532514, 14, 23, 19, }, // Korea
    {  433, 1112,       0,  532533,       0,  0, 13,  0, }, // Kosrae
    {  433, 1119,  532546,  532563,  532589, 17, 26, 22, }, // Krasnoyarsk
    {  433, 1151,       0,  532611,       0,  0, 13,  0, }, // Kyrgystan
    {  433, 1175,       0,  532624,       0,  0, 17,  0, }, // Line_Islands
    {  433, 1188,  532641,  532657,  532682, 16, 25, 21, }, // Lord_Howe
    {  433, 1204,  532703,  532715,  532736, 12, 21, 17, }, // Magadan
    {  433, 1219,       0,  532753,       0,  0, 12,  0, }, // Malaysia
    {  433, 1228,       0,  532765,       0,  0, 14,  0, }, // Maldives
    {  433, 1237,       0,  532779,       0,  0, 21,  0, }, // Marquesas
    {  433, 1247,       0,  532800,       0,  0, 22,  0, }, // Marshall_Islands
    {  433, 1264,  532822,  532835,  532857, 13, 22, 18, }, // Mauritius
    {  433, 1274,       0,  532875,       0,  0, 11,  0, }, // Mawson
    {  433, 1281,  532886,  532910,  532943, 24, 33, 29, }, // Mexico_Pacific
    {  433, 1296,  532972,  532989,  533015, 17, 26, 22, }, // Mongolia
    {  433, 1305,  533037,  533051,  533074, 14, 23, 19, }, // Moscow
    {  433, 1312,       0,  533093,       0,  0, 13,  0, }, // Myanmar
    {  433, 1320,       0,  533106,       0,  0, 13,  0, }, // Nauru
    {  433, 1326,       0,  533119,       0,  0, 10,  0, }, // Nepal
    {  433, 1332,  533129,  533148,  533176, 19, 28, 24, }, // New_Caledonia
    {  433, 1346,  533200,  533217,  533243, 17, 26, 22, }, // New_Zealand
    {  433, 1358,  533265,  533284,  533312, 19, 28, 24, }, // Newfoundland
    {  433, 1371,       0,  533336,       0,  0, 12,  0, }, // Niue
    {  433, 1376,  533348,  533365,  533391, 17, 26, 22, }, // Norfolk
    {  433, 1384,  533413,  533439,  533474, 26, 35, 31, }, // Noronha
    {  433, 1406,  533505,  533523,  533550, 18, 27, 23, }, // Novosibirsk
    {  433, 1418,  533573,  533584,  533604, 11, 20, 16, }, // Omsk
    {  433, 1428,  533620,  533633,  533655, 13, 22, 18, }, // Pakistan
    {  433, 1437,       0,  533673,       0,  0, 13,  0, }, // Palau
    {  433, 1443,       0,  533686,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  433, 1460,  533708,  533721,  533743, 13, 22, 18, }, // Paraguay
    {  433, 1469,  533761,  533773,  533794, 12, 21, 17, }, // Peru
    {  433, 1474,  533811,  533825,  533848, 14, 23, 19, }, // Philippines
    {  433, 1486,       0,  533867,       0,  0, 16,  0, }, // Phoenix_Islands
    {  433, 1502,  533883,  533908,  533942, 25, 34, 30, }, // Pierre_Miquelon
    {  433, 1518,       0,  533972,       0,  0, 14,  0, }, // Pitcairn
    {  433, 1527,       0,  533986,       0,  0, 14,  0, }, // Ponape
    {  433, 1534,       0,  534000,       0,  0, 11,  0, }, // Pyongyang
    {  433, 1554,       0,  534011,       0,  0, 12,  0, }, // Reunion
    {  433, 1562,       0,  534023,       0,  0, 14,  0, }, // Rothera
    {  433, 1570,  534037,  534049,  534070, 12, 21, 17, }, // Sakhalin
    {  433, 1596,  534087,  534100,  534122, 13, 22, 18, }, // Samoa
    {  433, 1602,       0,  534140,       0,  0, 22,  0, }, // Seychelles
    {  433, 1624,       0,  534162,       0,  0, 14,  0, }, // Singapore
    {  433, 1634,       0,  534176,       0,  0, 21,  0, }, // Solomon
    {  433, 1642,       0,  534197,       0,  0, 20,  0, }, // South_Georgia
    {  433, 1656,       0,  534217,       0,  0, 13,  0, }, // Suriname
    {  433, 1676,       0,  534230,       0,  0, 12,  0, }, // Syowa
    {  433, 1682,       0,  534242,       0,  0, 13,  0, }, // Tahiti
    {  433, 1689,  534255,  534266,  534286, 11, 20, 16, }, // Taipei
    {  433, 1696,       0,  534302,       0,  0, 15,  0, }, // Tajikistan
    {  433, 1724,       0,  534317,       0,  0, 15,  0, }, // Tokelau
    {  433, 1732,  534332,  534345,  534367, 13, 22, 18, }, // Tonga
    {  433, 1738,       0,  534385,       0,  0, 11,  0, }, // Truk
    {  433, 1750,  534396,  534413,  534439, 17, 26, 22, }, // Turkmenistan
    {  433, 1763,       0,  534461,       0,  0, 14,  0, }, // Tuvalu
    {  433, 1777,  534475,  534487,  534508, 12, 21, 17, }, // Uruguay
    {  433, 1792,  534525,  534540,  534564, 15, 24, 20, }, // Uzbekistan
    {  433, 1803,  534584,  534599,  534623, 15, 24, 20, }, // Vanuatu
    {  433, 1811,       0,  534643,       0,  0, 15,  0, }, // Venezuela
    {  433, 1821,  534658,  534676,  534703, 18, 27, 23, }, // Vladivostok
    {  433, 1833,  534726,  534741,  534765, 15, 24, 20, }, // Volgograd
    {  433, 1843,       0,  534785,       0,  0, 13,  0, }, // Vostok
    {  433, 1850,       0,  534798,       0,  0, 19,  0, }, // Wake
    {  433, 1855,       0,  534817,       0,  0, 24,  0, }, // Wallis
    {  433, 1862,  534841,  534851,  534870, 10, 19, 15, }, // Yakutsk
    {  433, 1870,  534885,  534904,  534932, 19, 28, 24, }, // Yekaterinburg
    {  433, 1892,       0,  534956,       0,  0, 15,  0, }, // Yukon
    {  435,  142,  534971,  534978,  534997,  7, 19, 16, }, // America_Central Mongolian/Mongolian/Mongolia
    {  435,  158,  535013,  535032,  535063, 19, 31, 28, }, // America_Eastern
    {  435,  174,  535091,  535108,  535137, 17, 29, 22, }, // America_Mountain
    {  435,  191,  535159,  535181,  535216, 22, 35, 31, }, // America_Pacific
    {  435,  286,  535247,  535264,  535292, 17, 28, 23, }, // Atlantic
    {  435,  627,  535315,  535335,  535367, 20, 32, 28, }, // Europe_Central
    {  435,  642,  535395,  535417,  535451, 22, 34, 30, }, // Europe_Eastern
    {  435,  680,  535481,  535505,  535541, 24, 36, 32, }, // Europe_Western
    {  435,  788,       0,  535573,       0,  0, 17,  0, }, // GMT
    {  441,    5,       0,  535590,       0,  0, 15,  0, }, // Afghanistan Nepali/Devanagari/Nepal
    {  441,   17,       0,  535605,       0,  0, 21,  0, }, // Africa_Central
    {  441,   32,       0,  535626,       0,  0, 18,  0, }, // Africa_Eastern
    {  441,   65,       0,  535644,       0,  0, 18,  0, }, // Africa_Southern
    {  441,   81,  535662,  535680,  535703, 18, 23, 31, }, // Africa_Western
    {  441,  107,  535734,  535746,  535763, 12, 17, 17, }, // Alaska
    {  441,  135,  535780,  535790,  535805, 10, 15, 23, }, // Amazon
    {  441,  142,  535828,  535841,  535859, 13, 18, 18, }, // America_Central
    {  441,  158,  335107,  335132,  535877, 10, 15, 15, }, // America_Eastern
    {  441,  174,  535892,  535902,  535917, 10, 15, 15, }, // America_Mountain
    {  441,  191,  535932,  535945,  535963, 13, 18, 18, }, // America_Pacific
    {  441,  214,  535981,  535990,  536004,  9, 14, 14, }, // Apia
    {  441,  232,  536018,  536026,  536039,  8, 13, 13, }, // Arabian
    {  441,  240,  536052,  536066,  536085, 14, 19, 27, }, // Argentina
    {  441,  250,  536112,  536134,  536161, 22, 27, 35, }, // Argentina_Western
    {  441,  268,  536196,  536209,  536227, 13, 18, 26, }, // Armenia
    {  441,  286,  536253,  536267,  536286, 14, 19, 19, }, // Atlantic
    {  441,  295,  536305,  536330,  536360, 25, 30, 30, }, // Australia_Central
    {  441,  313,  536390,  536423,  536461, 33, 38, 38, }, // Australia_CentralWestern
    {  441,  338,  536499,  536521,  536548, 22, 27, 27, }, // Australia_Eastern
    {  441,  356,  536400,  536433,  536471, 23, 28, 28, }, // Australia_Western
    {  441,  374,  536575,  536587,  536604, 12, 17, 25, }, // Azerbaijan
    {  441,  385,  536629,  536640,  536656, 11, 16, 24, }, // Azores
    {  441,  397,  536680,  536693,  536711, 13, 18, 26, }, // Bangladesh
    {  441,  415,       0,  536737,       0,  0, 10,  0, }, // Bhutan
    {  441,  422,       0,  536747,       0,  0, 12,  0, }, // Bolivia
    {  441,  437,  536759,  536773,  536792, 14, 19, 27, }, // Brasilia
    {  441,  454,       0,  536819,       0,  0, 21,  0, }, // Brunei
    {  441,  461,  536840,  536853,  536871, 13, 18, 26, }, // Cape_Verde
    {  441,  478,       0,  536897,       0,  0, 17,  0, }, // Chamorro
    {  441,  487,  536914,  536923,  536937,  9, 14, 14, }, // Chatham
    {  441,  495,  336470,  336478,  336491,  8, 13, 21, }, // Chile
    {  441,  501,  336512,  336519,  536951,  7, 12, 12, }, // China
    {  441,  507,       0,  536963,       0,  0, 17,  0, }, // Christmas
    {  441,  517,       0,  536980,       0,  0, 14,  0, }, // Cocos
    {  441,  523,  536994,  537009,  537029, 15, 20, 28, }, // Colombia
    {  441,  532,  537057,  537069,  537086, 12, 17, 29, }, // Cook
    {  441,  537,  537115,  537127,  537144, 12, 17, 17, }, // Cuba
    {  441,  548,       0,  537161,       0,  0,  9,  0, }, // Davis
    {  441,  564,       0,  537170,       0,  0, 23,  0, }, // DumontDUrville
    {  441,  601,       0,  537193,       0,  0, 16,  0, }, // East_Timor
    {  441,  612,  537209,  537223,  537242, 14, 19, 22, }, // Easter
    {  441,  619,       0,  537264,       0,  0, 12,  0, }, // Ecuador
    {  441,  627,  537276,  537298,  537325, 22, 27, 35, }, // Europe_Central
    {  441,  642,  537360,  537379,  537403, 19, 24, 32, }, // Europe_Eastern
    {  441,  657,       0,  537435,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  441,  680,  537457,  537477,  537335, 20, 25, 25, }, // Europe_Western
    {  441,  695,  537502,  537522,  537547, 20, 25, 33, }, // Falkland
    {  441,  704,  537580,  537588,  537601,  8, 13, 21, }, // Fiji
    {  441,  709,       0,  537622,       0,  0, 18,  0, }, // French_Guiana
    {  441,  723,       0,  537640,       0,  0, 32,  0, }, // French_Southern
    {  441,  746,       0,  537672,       0,  0, 14,  0, }, // Galapagos
    {  441,  756,       0,  537686,       0,  0, 14,  0, }, // Gambier
    {  441,  764,  537700,  537711,  537727, 11, 16, 24, }, // Georgia
    {  441,  772,       0,  537751,       0,  0, 17,  0, }, // Gilbert_Islands
    {  441,  788,       0,  537768,       0,  0, 16,  0, }, // GMT
    {  441,  830,  537784,  537809,  537839, 25, 30, 38, }, // Greenland_Eastern
    {  441,  848,  537877,  537903,  537934, 26, 31, 39, }, // Greenland_Western
    {  441,  871,       0,  537973,       0,  0, 13,  0, }, // Gulf
    {  441,  876,       0,  337252,       0,  0, 10,  0, }, // Guyana
    {  441,  883,  537986,  538002,  538023, 16, 21, 21, }, // Hawaii_Aleutian
    {  441,  899,  538044,  538052,  538065,  8, 13, 21, }, // Hong_Kong
    {  441,  909,  538086,  538095,  538109,  9, 14, 22, }, // Hovd
    {  441,  914,       0,  337733,       0,  0, 15,  0, }, // India
    {  441,  920,       0,  538131,       0,  0, 17,  0, }, // Indian_Ocean
    {  441,  933,       0,  538148,       0,  0, 14,  0, }, // Indochina
    {  441,  943,       0,  538162,       0,  0, 25,  0, }, // Indonesia_Central
    {  441,  961,       0,  538187,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  441,  979,       0,  538209,       0,  0, 23,  0, }, // Indonesia_Western
    {  441,  997,  538232,  538241,  538255,  9, 14, 14, }, // Iran
    {  441, 1008,  538269,  538283,  538302, 14, 19, 27, }, // Irkutsk
    {  441, 1016,  538329,  538339,  538354, 10, 15, 15, }, // Israel
    {  441, 1023,  337980,  337989,  538369,  9, 14, 14, }, // Japan
    {  441, 1047,       0,  538383,       0,  0, 15,  0, }, // Kazakhstan
    {  441, 1058,       0,  538398,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  441, 1077,       0,  538418,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  441, 1106,  538438,  538450,  538467, 12, 17, 17, }, // Korea
    {  441, 1112,       0,  338224,       0,  0, 10,  0, }, // Kosrae
    {  441, 1119,  538484,  538500,  538521, 16, 21, 29, }, // Krasnoyarsk
    {  441, 1151,       0,  538550,       0,  0, 15,  0, }, // Kyrgystan
    {  441, 1175,       0,  538565,       0,  0, 13,  0, }, // Line_Islands
    {  441, 1188,  538578,  538591,  538609, 13, 18, 18, }, // Lord_Howe
    {  441, 1204,  338397,  338408,  338424, 11, 16, 24, }, // Magadan
    {  441, 1219,       0,  538627,       0,  0, 11,  0, }, // Malaysia
    {  441, 1228,       0,  538638,       0,  0, 13,  0, }, // Maldives
    {  441, 1237,       0,  538651,       0,  0, 12,  0, }, // Marquesas
    {  441, 1247,       0,  538663,       0,  0, 15,  0, }, // Marshall_Islands
    {  441, 1264,  538678,  538688,  538703, 10, 15, 23, }, // Mauritius
    {  441, 1274,       0,  538726,       0,  0,  9,  0, }, // Mawson
    {  441, 1281,  538735,  538757,  538784, 22, 27, 27, }, // Mexico_Pacific
    {  441, 1296,  538811,  538825,  538844, 14, 19, 27, }, // Mongolia
    {  441, 1305,  538871,  538880,  538894,  9, 14, 22, }, // Moscow
    {  441, 1312,       0,  538916,       0,  0, 12,  0, }, // Myanmar
    {  441, 1320,       0,  538928,       0,  0,  9,  0, }, // Nauru
    {  441, 1326,       0,  538937,       0,  0, 10,  0, }, // Nepal
    {  441, 1332,  538947,  538966,  538990, 19, 24, 32, }, // New_Caledonia
    {  441, 1346,  539022,  539039,  539061, 17, 22, 22, }, // New_Zealand
    {  441, 1358,  539083,  539106,  539133, 23, 27, 26, }, // Newfoundland
    {  441, 1371,       0,  539159,       0,  0,  8,  0, }, // Niue
    {  441, 1376,  539167,  539185,  539208, 18, 23, 31, }, // Norfolk
    {  441, 1384,  539239,  539264,  539294, 25, 30, 38, }, // Noronha
    {  441, 1406,  539332,  539349,  539371, 17, 22, 30, }, // Novosibirsk
    {  441, 1418,  339202,  339212,  339227, 10, 15, 23, }, // Omsk
    {  441, 1428,  539401,  539415,  539434, 14, 19, 27, }, // Pakistan
    {  441, 1437,       0,  539461,       0,  0,  9,  0, }, // Palau
    {  441, 1443,       0,  539470,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  441, 1460,  539489,  539501,  539518, 12, 17, 25, }, // Paraguay
    {  441, 1469,  539543,  339396,  539551,  8, 13, 21, }, // Peru
    {  441, 1474,  539572,  539584,  539601, 12, 17, 25, }, // Philippines
    {  441, 1486,       0,  539626,       0,  0, 16,  0, }, // Phoenix_Islands
    {  441, 1502,  539642,  539673,  539709, 31, 36, 36, }, // Pierre_Miquelon
    {  441, 1518,       0,  539745,       0,  0, 11,  0, }, // Pitcairn
    {  441, 1527,       0,  339611,       0,  0, 10,  0, }, // Ponape
    {  441, 1534,       0,  539756,       0,  0, 12,  0, }, // Pyongyang
    {  441, 1554,       0,  539768,       0,  0, 12,  0, }, // Reunion
    {  441, 1562,       0,  339647,       0,  0, 10,  0, }, // Rothera
    {  441, 1570,  539780,  539791,  539807, 11, 16, 24, }, // Sakhalin
    {  441, 1596,  539831,  539840,  539854,  9, 14, 14, }, // Samoa
    {  441, 1602,       0,  539868,       0,  0, 14,  0, }, // Seychelles
    {  441, 1624,       0,  539882,       0,  0, 17,  0, }, // Singapore
    {  441, 1634,       0,  539899,       0,  0, 16,  0, }, // Solomon
    {  441, 1642,       0,  539915,       0,  0, 18,  0, }, // South_Georgia
    {  441, 1656,       0,  539933,       0,  0, 12,  0, }, // Suriname
    {  441, 1676,       0,  339860,       0,  0, 10,  0, }, // Syowa
    {  441, 1682,       0,  339870,       0,  0, 10,  0, }, // Tahiti
    {  441, 1689,  539945,  539955,  539970, 10, 15, 15, }, // Taipei
    {  441, 1696,       0,  539985,       0,  0, 14,  0, }, // Tajikistan
    {  441, 1724,       0,  539999,       0,  0, 11,  0, }, // Tokelau
    {  441, 1732,  339945,  339954,  339968,  9, 14, 22, }, // Tonga
    {  441, 1738,       0,  540010,       0,  0,  8,  0, }, // Truk
    {  441, 1750,  339997,  340015,  540018, 18, 23, 36, }, // Turkmenistan
    {  441, 1763,       0,  540054,       0,  0, 10,  0, }, // Tuvalu
    {  441, 1777,  340079,  540064,  340106, 11, 16, 24, }, // Uruguay
    {  441, 1792,  540080,  540096,  540117, 16, 21, 29, }, // Uzbekistan
    {  441, 1803,  540146,  540157,  540173, 11, 16, 24, }, // Vanuatu
    {  441, 1811,       0,  540197,       0,  0, 15,  0, }, // Venezuela
    {  441, 1821,  540212,  540229,  540251, 17, 22, 30, }, // Vladivostok
    {  441, 1833,  540281,  540296,  540316, 15, 20, 28, }, // Volgograd
    {  441, 1843,       0,  540218,       0,  0, 11,  0, }, // Vostok
    {  441, 1850,       0,  540344,       0,  0, 12,  0, }, // Wake
    {  441, 1855,       0,  540356,       0,  0, 19,  0, }, // Wallis
    {  441, 1862,  540375,  540386,  540402, 11, 16, 24, }, // Yakutsk
    {  441, 1870,  540426,  540443,  540465, 17, 22, 30, }, // Yekaterinburg
    {  441, 1892,       0,  540495,       0,  0, 11,  0, }, // Yukon
    {  442,  914,       0,  337733,       0,  0, 15,  0, }, // India Nepali/Devanagari/India
    {  445,    5,       0,  540506,       0,  0, 15,  0, }, // Afghanistan Nigerian Pidgin/Latin/Nigeria
    {  445,   17,       0,  540521,       0,  0, 17,  0, }, // Africa_Central
    {  445,   32,       0,  540538,       0,  0, 15,  0, }, // Africa_Eastern
    {  445,   65,       0,  540553,       0,  0, 22,  0, }, // Africa_Southern
    {  445,   81,  540575,  540592,  540615, 17, 23, 28, }, // Africa_Western
    {  445,  107,  540643,  540654,  540671, 11, 17, 18, }, // Alaska
    {  445,  135,  540689,  540700,  540717, 11, 17, 22, }, // Amazon
    {  445,  142,  540739,  540768,  540803, 29, 35, 36, }, // America_Central
    {  445,  158,  540839,  540868,  540903, 29, 35, 36, }, // America_Eastern
    {  445,  174,  540939,  540970,  541007, 31, 37, 38, }, // America_Mountain
    {  445,  191,  541045,  541076,  541113, 31, 37, 38, }, // America_Pacific
    {  445,  214,  541151,  541160,  541175,  9, 15, 16, }, // Apia
    {  445,  232,  541191,  541202,  541219, 11, 17, 18, }, // Arabian
    {  445,  240,  541237,  541250,  541269, 13, 19, 24, }, // Argentina
    {  445,  250,  541293,  541314,  541341, 21, 27, 32, }, // Argentina_Western
    {  445,  268,  541373,  541386,  541405, 13, 19, 24, }, // Armenia
    {  445,  286,  541429,  541442,  541461, 13, 19, 20, }, // Atlantic
    {  445,  295,  541481,  541500,  541525, 19, 25, 26, }, // Australia_Central
    {  445,  313,  541551,  541578,  541611, 27, 33, 34, }, // Australia_CentralWestern
    {  445,  338,  541645,  541664,  541689, 19, 25, 26, }, // Australia_Eastern
    {  445,  356,  541715,  541736,  541763, 21, 27, 28, }, // Australia_Western
    {  445,  374,  541791,  541806,  541827, 15, 21, 26, }, // Azerbaijan
    {  445,  385,  541853,  541862,  541877,  9, 15, 20, }, // Azores
    {  445,  397,  541897,  541912,  541933, 15, 21, 22, }, // Bangladesh
    {  445,  415,       0,  541955,       0,  0, 10,  0, }, // Bhutan
    {  445,  422,       0,  541965,       0,  0, 18,  0, }, // Bolivia
    {  445,  437,  541983,  541996,  542015, 13, 19, 24, }, // Brasilia
    {  445,  454,       0,  542039,       0,  0, 21,  0, }, // Brunei
    {  445,  461,  542060,  542073,  542092, 13, 19, 24, }, // Cape_Verde
    {  445,  478,       0,  542116,       0,  0, 18,  0, }, // Chamorro
    {  445,  487,  542134,  542145,  542162, 11, 17, 18, }, // Chatham
    {  445,  495,  542180,  542190,  542206, 10, 16, 21, }, // Chile
    {  445,  501,  542227,  542238,  542255, 11, 17, 18, }, // China
    {  445,  507,       0,  542273,       0,  0, 19,  0, }, // Christmas
    {  445,  517,       0,  542292,       0,  0, 18,  0, }, // Cocos
    {  445,  523,  542310,  542323,  542342, 13, 19, 24, }, // Colombia
    {  445,  532,  542366,  542382,  542404, 16, 22, 31, }, // Cook
    {  445,  537,  542435,  542444,  542459,  9, 15, 16, }, // Cuba
    {  445,  548,       0,  542475,       0,  0, 10,  0, }, // Davis
    {  445,  564,       0,  542485,       0,  0, 19,  0, }, // DumontDUrville
    {  445,  601,       0,  542504,       0,  0, 13,  0, }, // East_Timor
    {  445,  612,  542517,  542526,  542541,  9, 15, 20, }, // Easter
    {  445,  619,       0,  542561,       0,  0, 13,  0, }, // Ecuador
    {  445,  627,  542574,  542590,  542612, 16, 22, 27, }, // Europe_Central
    {  445,  642,  542639,  542655,  542677, 16, 22, 27, }, // Europe_Eastern
    {  445,  657,       0,  542704,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  445,  680,  542724,  542742,  542766, 18, 24, 29, }, // Europe_Western
    {  445,  695,  542795,  542809,  542829, 14, 20, 25, }, // Falkland
    {  445,  704,  542854,  542863,  542878,  9, 15, 20, }, // Fiji
    {  445,  709,       0,  542898,       0,  0, 18,  0, }, // French_Guiana
    {  445,  723,       0,  542916,       0,  0, 30,  0, }, // French_Southern
    {  445,  746,       0,  542946,       0,  0, 14,  0, }, // Galapagos
    {  445,  756,       0,  542960,       0,  0, 12,  0, }, // Gambier
    {  445,  764,  542972,  542983,  543000, 11, 17, 22, }, // Georgia
    {  445,  772,       0,  543022,       0,  0, 19,  0, }, // Gilbert_Islands
    {  445,  788,       0,  543041,       0,  0, 17,  0, }, // GMT
    {  445,  830,  543058,  543076,  543100, 18, 24, 29, }, // Greenland_Eastern
    {  445,  848,  543129,  543148,  543173, 19, 25, 30, }, // Greenland_Western
    {  445,  871,       0,  543203,       0,  0, 16,  0, }, // Gulf
    {  445,  876,       0,  543219,       0,  0, 11,  0, }, // Guyana
    {  445,  883,  543230,  543250,  543276, 20, 26, 27, }, // Hawaii_Aleutian
    {  445,  899,  543303,  543317,  543337, 14, 20, 25, }, // Hong_Kong
    {  445,  909,  543362,  543371,  543386,  9, 15, 20, }, // Hovd
    {  445,  914,       0,  543406,       0,  0, 16,  0, }, // India
    {  445,  920,       0,  543422,       0,  0, 18,  0, }, // Indian_Ocean
    {  445,  933,       0,  543440,       0,  0, 15,  0, }, // Indochina
    {  445,  943,       0,  543455,       0,  0, 22,  0, }, // Indonesia_Central
    {  445,  961,       0,  543477,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  445,  979,       0,  543499,       0,  0, 24,  0, }, // Indonesia_Western
    {  445,  997,  543523,  543532,  543547,  9, 15, 16, }, // Iran
    {  445, 1008,  543563,  543575,  543593, 12, 18, 23, }, // Irkutsk
    {  445, 1016,  543616,  543626,  543642, 10, 16, 17, }, // Israel
    {  445, 1023,  543659,  543669,  543685, 10, 16, 17, }, // Japan
    {  445, 1047,       0,  543702,       0,  0, 14,  0, }, // Kazakhstan
    {  445, 1058,       0,  543716,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  445, 1077,       0,  543734,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  445, 1106,  543754,  543764,  543780, 10, 16, 17, }, // Korea
    {  445, 1112,       0,  543797,       0,  0, 12,  0, }, // Kosrae
    {  445, 1119,  543809,  543824,  543845, 15, 21, 26, }, // Krasnoyarsk
    {  445, 1151,       0,  543871,       0,  0, 13,  0, }, // Kyrgystan
    {  445, 1175,       0,  543884,       0,  0, 17,  0, }, // Line_Islands
    {  445, 1188,  543901,  543913,  543931, 12, 18, 19, }, // Lord_Howe
    {  445, 1204,  543950,  543962,  543980, 12, 18, 23, }, // Magadan
    {  445, 1219,       0,  544003,       0,  0, 13,  0, }, // Malaysia
    {  445, 1228,       0,  544016,       0,  0, 12,  0, }, // Maldives
    {  445, 1237,       0,  544028,       0,  0, 14,  0, }, // Marquesas
    {  445, 1247,       0,  544042,       0,  0, 19,  0, }, // Marshall_Islands
    {  445, 1264,  544061,  544074,  544093, 13, 19, 24, }, // Mauritius
    {  445, 1274,       0,  544117,       0,  0, 11,  0, }, // Mawson
    {  445, 1281,  544128,  544149,  544176, 21, 27, 28, }, // Mexico_Pacific
    {  445, 1296,  544204,  544217,  544236, 13, 19, 24, }, // Mongolia
    {  445, 1305,  544260,  544271,  544288, 11, 17, 22, }, // Moscow
    {  445, 1312,       0,  544310,       0,  0, 11,  0, }, // Myanmar
    {  445, 1320,       0,  544321,       0,  0, 10,  0, }, // Nauru
    {  445, 1326,       0,  544331,       0,  0, 10,  0, }, // Nepal
    {  445, 1332,  544341,  544359,  544383, 18, 24, 29, }, // New_Caledonia
    {  445, 1346,  544412,  544427,  544448, 15, 21, 22, }, // New_Zealand
    {  445, 1358,  544470,  544487,  544510, 17, 23, 24, }, // Newfoundland
    {  445, 1371,       0,  544534,       0,  0,  9,  0, }, // Niue
    {  445, 1376,  544543,  544564,  544591, 21, 27, 28, }, // Norfolk
    {  445, 1384,  544619,  544643,  544673, 24, 30, 35, }, // Noronha
    {  445, 1406,  544708,  544723,  544744, 15, 21, 26, }, // Novosibirsk
    {  445, 1418,  544770,  544779,  544794,  9, 15, 20, }, // Omsk
    {  445, 1428,  544814,  544827,  544846, 13, 19, 24, }, // Pakistan
    {  445, 1437,       0,  544870,       0,  0, 10,  0, }, // Palau
    {  445, 1443,       0,  544880,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  445, 1460,  544899,  544912,  544931, 13, 19, 24, }, // Paraguay
    {  445, 1469,  544955,  544964,  544979,  9, 15, 20, }, // Peru
    {  445, 1474,  544999,  545011,  545029, 12, 18, 22, }, // Philippines
    {  445, 1486,       0,  545051,       0,  0, 19,  0, }, // Phoenix_Islands
    {  445, 1502,  545070,  545094,  545125, 24, 31, 32, }, // Pierre_Miquelon
    {  445, 1518,       0,  545157,       0,  0, 11,  0, }, // Pitcairn
    {  445, 1527,       0,  545168,       0,  0, 11,  0, }, // Ponape
    {  445, 1534,       0,  545179,       0,  0, 14,  0, }, // Pyongyang
    {  445, 1554,       0,  545193,       0,  0, 13,  0, }, // Reunion
    {  445, 1562,       0,  545206,       0,  0, 12,  0, }, // Rothera
    {  445, 1570,  545218,  545231,  545250, 13, 19, 24, }, // Sakhalin
    {  445, 1596,  545274,  545284,  545300, 10, 16, 17, }, // Samoa
    {  445, 1602,       0,  545317,       0,  0, 13,  0, }, // Seychelles
    {  445, 1624,       0,  545330,       0,  0, 12,  0, }, // Singapore
    {  445, 1634,       0,  545342,       0,  0, 21,  0, }, // Solomon
    {  445, 1642,       0,  545363,       0,  0, 16,  0, }, // South_Georgia
    {  445, 1656,       0,  545379,       0,  0, 12,  0, }, // Suriname
    {  445, 1676,       0,  545391,       0,  0, 10,  0, }, // Syowa
    {  445, 1682,       0,  545401,       0,  0, 11,  0, }, // Tahiti
    {  445, 1689,  545412,  545423,  545440, 11, 17, 18, }, // Taipei
    {  445, 1696,       0,  545458,       0,  0, 15,  0, }, // Tajikistan
    {  445, 1724,       0,  545473,       0,  0, 12,  0, }, // Tokelau
    {  445, 1732,  545485,  545496,  545513, 11, 17, 22, }, // Tonga
    {  445, 1738,       0,  545535,       0,  0,  9,  0, }, // Truk
    {  445, 1750,  545544,  545560,  545582, 16, 22, 27, }, // Turkmenistan
    {  445, 1763,       0,  545609,       0,  0, 11,  0, }, // Tuvalu
    {  445, 1777,  545620,  545633,  545652, 13, 19, 24, }, // Uruguay
    {  445, 1792,  545676,  545691,  545712, 15, 21, 26, }, // Uzbekistan
    {  445, 1803,  545738,  545750,  545768, 12, 18, 18, }, // Vanuatu
    {  445, 1811,       0,  545786,       0,  0, 15,  0, }, // Venezuela
    {  445, 1821,  545801,  545817,  545839, 16, 22, 27, }, // Vladivostok
    {  445, 1833,  545866,  545880,  545900, 14, 20, 25, }, // Volgograd
    {  445, 1843,       0,  545925,       0,  0, 12,  0, }, // Vostok
    {  445, 1850,       0,  545937,       0,  0, 15,  0, }, // Wake
    {  445, 1855,       0,  545952,       0,  0, 20,  0, }, // Wallis
    {  445, 1862,  545972,  545984,  546002, 12, 18, 23, }, // Yakutsk
    {  445, 1870,  546025,  546042,  546065, 17, 23, 28, }, // Yekaterinburg
    {  445, 1892,       0,  546093,       0,  0, 10,  0, }, // Yukon
    {  446,    5,       0,  546103,       0,  0, 23,  0, }, // Afghanistan Nko/Nko/Guinea
    {  446,   17,       0,  546126,       0,  0, 34,  0, }, // Africa_Central
    {  446,   32,       0,  546160,       0,  0, 33,  0, }, // Africa_Eastern
    {  446,   65,       0,  546193,       0,  0, 35,  0, }, // Africa_Southern
    {  446,   81,  546228,  546256,  546290, 28, 34, 34, }, // Africa_Western
    {  446,  107,  546324,  546341,  546368, 17, 27, 22, }, // Alaska
    {  446,  135,  546390,  546407,  546433, 17, 26, 22, }, // Amazon
    {  446,  142,  546455,  546479,  546521, 24, 42, 29, }, // America_Central
    {  446,  158,  546550,  546582,  546623, 32, 41, 29, }, // America_Eastern
    {  446,  174,  546652,  546671,  546700, 19, 29, 24, }, // America_Mountain
    {  446,  191,  546724,  546763,  546812, 39, 49, 44, }, // America_Pacific
    {  446,  214,  546856,  546871,  546896, 15, 25, 33, }, // Apia
    {  446,  232,  546929,  546946,  546973, 17, 27, 33, }, // Arabian
    {  446,  240,  547006,  547026,  547056, 20, 30, 25, }, // Argentina
    {  446,  250,  547081,  547108,  547133, 27, 25, 32, }, // Argentina_Western
    {  446,  268,  547165,  547182,  547209, 17, 27, 33, }, // Armenia
    {  446,  286,  547242,  547264,  547296, 22, 32, 27, }, // Atlantic
    {  446,  295,  547323,  547347,  547381, 24, 34, 29, }, // Australia_Central
    {  446,  313,  547410,  547442,  547484, 32, 42, 37, }, // Australia_CentralWestern
    {  446,  338,  547521,  547546,  547581, 25, 35, 30, }, // Australia_Eastern
    {  446,  356,  547611,  547636,  547671, 25, 35, 30, }, // Australia_Western
    {  446,  374,  547701,  547724,  547757, 23, 33, 39, }, // Azerbaijan
    {  446,  385,  547796,  547811,  547836, 15, 25, 20, }, // Azores
    {  446,  397,  547856,  547876,  547906, 20, 30, 25, }, // Bangladesh
    {  446,  415,       0,  547931,       0,  0, 15,  0, }, // Bhutan
    {  446,  422,       0,  547946,       0,  0, 16,  0, }, // Bolivia
    {  446,  437,  547962,  547981,  548010, 19, 29, 24, }, // Brasilia
    {  446,  454,       0,  548034,       0,  0, 28,  0, }, // Brunei
    {  446,  461,  548062,  548086,  548116, 24, 30, 30, }, // Cape_Verde
    {  446,  478,       0,  548146,       0,  0, 27,  0, }, // Chamorro
    {  446,  487,  548173,  548186,  548209, 13, 23, 18, }, // Chatham
    {  446,  495,  548227,  548240,  548263, 13, 23, 18, }, // Chile
    {  446,  501,  548281,  548293,  548315, 12, 22, 20, }, // China
    {  446,  507,       0,  548335,       0,  0, 22,  0, }, // Christmas
    {  446,  517,       0,  548357,       0,  0, 18,  0, }, // Cocos
    {  446,  523,  548375,  548392,  548419, 17, 27, 22, }, // Colombia
    {  446,  532,  548441,  548453,  548475, 12, 22, 30, }, // Cook
    {  446,  537,  548505,  548520,  548545, 15, 25, 20, }, // Cuba
    {  446,  548,       0,  548565,       0,  0, 15,  0, }, // Davis
    {  446,  564,       0,  548580,       0,  0, 23,  0, }, // DumontDUrville
    {  446,  601,       0,  548603,       0,  0, 21,  0, }, // East_Timor
    {  446,  612,  548624,  548644,  548674, 20, 30, 25, }, // Easter
    {  446,  619,       0,  548699,       0,  0, 17,  0, }, // Ecuador
    {  446,  627,  548716,  548737,  548768, 21, 31, 26, }, // Europe_Central
    {  446,  642,  548794,  548816,  548848, 22, 32, 27, }, // Europe_Eastern
    {  446,  657,       0,  548875,       0,  0, 21,  0, }, // Europe_Further_Eastern
    {  446,  680,  548896,  548918,  548950, 22, 32, 27, }, // Europe_Western
    {  446,  695,  548977,  548999,  549031, 22, 32, 27, }, // Falkland
    {  446,  704,  549058,  549071,  549094, 13, 23, 31, }, // Fiji
    {  446,  709,       0,  549125,       0,  0, 27,  0, }, // French_Guiana
    {  446,  723,       0,  549152,       0,  0, 50,  0, }, // French_Southern
    {  446,  746,       0,  549202,       0,  0, 20,  0, }, // Galapagos
    {  446,  756,       0,  549222,       0,  0, 18,  0, }, // Gambier
    {  446,  764,  549240,  549258,  549286, 18, 28, 34, }, // Georgia
    {  446,  772,       0,  549320,       0,  0, 22,  0, }, // Gilbert_Islands
    {  446,  788,       0,  549342,       0,  0, 16,  0, }, // GMT
    {  446,  830,  549358,  549384,  549420, 26, 36, 31, }, // Greenland_Eastern
    {  446,  848,  549451,  549477,  549513, 26, 36, 31, }, // Greenland_Western
    {  446,  871,       0,  549544,       0,  0, 14,  0, }, // Gulf
    {  446,  876,       0,  549136,       0,  0, 16,  0, }, // Guyana
    {  446,  883,  549558,  549585,  549622, 27, 37, 32, }, // Hawaii_Aleutian
    {  446,  899,  549654,  549674,  549704, 20, 30, 28, }, // Hong_Kong
    {  446,  909,  549732,  549746,  549770, 14, 24, 22, }, // Hovd
    {  446,  914,       0,  549792,       0,  0, 25,  0, }, // India
    {  446,  920,       0,  549817,       0,  0, 25,  0, }, // Indian_Ocean
    {  446,  933,       0,  549842,       0,  0, 19,  0, }, // Indochina
    {  446,  943,       0,  549861,       0,  0, 24,  0, }, // Indonesia_Central
    {  446,  961,       0,  549885,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  446,  979,       0,  549910,       0,  0, 25,  0, }, // Indonesia_Western
    {  446,  997,  549935,  549949,  549973, 14, 24, 19, }, // Iran
    {  446, 1008,  549992,  550010,  550038, 18, 28, 23, }, // Irkutsk
    {  446, 1016,  550061,  550082,  550113, 21, 31, 37, }, // Israel
    {  446, 1023,  550150,  550166,  550192, 16, 26, 24, }, // Japan
    {  446, 1047,       0,  550216,       0,  0, 21,  0, }, // Kazakhstan
    {  446, 1058,       0,  550237,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  446, 1077,       0,  550265,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  446, 1106,  550293,  550307,  550331, 14, 24, 22, }, // Korea
    {  446, 1112,       0,  550353,       0,  0, 16,  0, }, // Kosrae
    {  446, 1119,  550369,  550393,  550427, 24, 34, 29, }, // Krasnoyarsk
    {  446, 1151,       0,  550456,       0,  0, 23,  0, }, // Kyrgystan
    {  446, 1175,       0,  550479,       0,  0, 26,  0, }, // Line_Islands
    {  446, 1188,  550505,  550518,  550541, 13, 23, 18, }, // Lord_Howe
    {  446, 1204,  550559,  550576,  550603, 17, 27, 22, }, // Magadan
    {  446, 1219,       0,  550625,       0,  0, 16,  0, }, // Malaysia
    {  446, 1228,       0,  550641,       0,  0, 21,  0, }, // Maldives
    {  446, 1237,       0,  550662,       0,  0, 17,  0, }, // Marquesas
    {  446, 1247,       0,  550679,       0,  0, 20,  0, }, // Marshall_Islands
    {  446, 1264,  550699,  550714,  550735, 15, 21, 21, }, // Mauritius
    {  446, 1274,       0,  550756,       0,  0, 17,  0, }, // Mawson
    {  446, 1281,  550773,  550802,  550841, 29, 39, 34, }, // Mexico_Pacific
    {  446, 1296,  550875,  550892,  550919, 17, 27, 25, }, // Mongolia
    {  446, 1305,  550944,  550960,  550986, 16, 26, 21, }, // Moscow
    {  446, 1312,       0,  551007,       0,  0, 18,  0, }, // Myanmar
    {  446, 1320,       0,  551025,       0,  0, 15,  0, }, // Nauru
    {  446, 1326,       0,  551040,       0,  0, 14,  0, }, // Nepal
    {  446, 1332,  551054,  551078,  551112, 24, 34, 29, }, // New_Caledonia
    {  446, 1346,  551141,  551162,  551193, 21, 31, 26, }, // New_Zealand
    {  446, 1358,  551219,  551243,  551277, 24, 34, 29, }, // Newfoundland
    {  446, 1371,       0,  551306,       0,  0, 14,  0, }, // Niue
    {  446, 1376,  551320,  551341,  551372, 21, 31, 26, }, // Norfolk
    {  446, 1384,  551398,  551430,  551472, 32, 42, 37, }, // Noronha
    {  446, 1406,  551509,  551529,  551559, 20, 30, 25, }, // Novosibirsk
    {  446, 1418,  551584,  551598,  551622, 14, 24, 19, }, // Omsk
    {  446, 1428,  551641,  551660,  551689, 19, 29, 24, }, // Pakistan
    {  446, 1437,       0,  551713,       0,  0, 14,  0, }, // Palau
    {  446, 1443,       0,  551727,       0,  0, 29,  0, }, // Papua_New_Guinea
    {  446, 1460,  551756,  551774,  551802, 18, 28, 23, }, // Paraguay
    {  446, 1469,  551825,  551839,  551863, 14, 24, 19, }, // Peru
    {  446, 1474,  551882,  551898,  551924, 16, 26, 21, }, // Philippines
    {  446, 1486,       0,  551945,       0,  0, 23,  0, }, // Phoenix_Islands
    {  446, 1502,  551968,  551997,  552036, 29, 39, 34, }, // Pierre_Miquelon
    {  446, 1518,       0,  552070,       0,  0, 17,  0, }, // Pitcairn
    {  446, 1527,       0,  552087,       0,  0, 15,  0, }, // Ponape
    {  446, 1534,       0,  552102,       0,  0, 21,  0, }, // Pyongyang
    {  446, 1554,       0,  552123,       0,  0, 18,  0, }, // Reunion
    {  446, 1562,       0,  552141,       0,  0, 16,  0, }, // Rothera
    {  446, 1570,  552157,  552175,  552203, 18, 28, 23, }, // Sakhalin
    {  446, 1596,  552226,  552242,  552268, 16, 26, 21, }, // Samoa
    {  446, 1602,       0,  552289,       0,  0, 17,  0, }, // Seychelles
    {  446, 1624,       0,  552306,       0,  0, 28,  0, }, // Singapore
    {  446, 1634,       0,  552334,       0,  0, 21,  0, }, // Solomon
    {  446, 1642,       0,  552355,       0,  0, 28,  0, }, // South_Georgia
    {  446, 1656,       0,  552383,       0,  0, 16,  0, }, // Suriname
    {  446, 1676,       0,  552399,       0,  0, 15,  0, }, // Syowa
    {  446, 1682,       0,  552414,       0,  0, 15,  0, }, // Tahiti
    {  446, 1689,  552429,  552445,  552471, 16, 26, 24, }, // Taipei
    {  446, 1696,       0,  552495,       0,  0, 20,  0, }, // Tajikistan
    {  446, 1724,       0,  552515,       0,  0, 16,  0, }, // Tokelau
    {  446, 1732,  552531,  552547,  552573, 16, 26, 21, }, // Tonga
    {  446, 1738,       0,  552594,       0,  0, 12,  0, }, // Truk
    {  446, 1750,  552606,  552630,  552664, 24, 34, 29, }, // Turkmenistan
    {  446, 1763,       0,  552693,       0,  0, 17,  0, }, // Tuvalu
    {  446, 1777,  552710,  552729,  552758, 19, 29, 24, }, // Uruguay
    {  446, 1792,  552782,  552805,  552838, 23, 33, 28, }, // Uzbekistan
    {  446, 1803,  552866,  552885,  552914, 19, 29, 24, }, // Vanuatu
    {  446, 1811,       0,  552938,       0,  0, 21,  0, }, // Venezuela
    {  446, 1821,  552959,  552982,  553015, 23, 33, 28, }, // Vladivostok
    {  446, 1833,  553043,  553065,  553097, 22, 32, 27, }, // Volgograd
    {  446, 1843,       0,  553124,       0,  0, 17,  0, }, // Vostok
    {  446, 1850,       0,  553141,       0,  0, 16,  0, }, // Wake
    {  446, 1855,       0,  553157,       0,  0, 25,  0, }, // Wallis
    {  446, 1862,  553182,  553201,  553230, 19, 29, 24, }, // Yakutsk
    {  446, 1870,  553254,  553277,  553310, 23, 33, 28, }, // Yekaterinburg
    {  446, 1892,       0,  553338,       0,  0, 25,  0, }, // Yukon
    {  447,  142,  553363,  553376,  553401, 13, 25, 21, }, // America_Central Northern Luri/Arabic/Iran
    {  449,  627,  553422,  553440,  553465, 18, 25, 24, }, // Europe_Central Northern Sami/Latin/Norway
    {  449,  642,  553489,  553508,  553534, 19, 26, 25, }, // Europe_Eastern
    {  449,  680,  553559,  553577,  553602, 18, 25, 24, }, // Europe_Western
    {  449,  788,       0,  553626,       0,  0, 21,  0, }, // GMT
    {  449, 1305,  553647,  553658,  553676, 11, 18, 17, }, // Moscow
    {  450,    5,       0,  553693,       0,  0, 17,  0, }, // Afghanistan Northern Sami/Latin/Finland
    {  450,   17,       0,  553710,       0,  0, 18,  0, }, // Africa_Central
    {  450,   32,       0,  553728,       0,  0, 19,  0, }, // Africa_Eastern
    {  450,   65,       0,  553747,       0,  0, 23,  0, }, // Africa_Southern
    {  450,   81,  553770,  553788,  553811, 18, 23, 24, }, // Africa_Western
    {  450,  107,  553835,  553846,  553862, 11, 16, 17, }, // Alaska
    {  450,  135,  553879,  553891,  553908, 12, 17, 18, }, // Amazon
    {  450,  142,  553454,  553926,  553943, 11, 17, 18, }, // America_Central
    {  450,  158,  553961,  553973,  553990, 12, 17, 18, }, // America_Eastern
    {  450,  174,  554008,  554019,  554035, 11, 16, 17, }, // America_Mountain
    {  450,  191,  554052,  554066,  554085, 14, 19, 20, }, // America_Pacific
    {  450,  214,  554105,  554114,  554128,  9, 14, 15, }, // Apia
    {  450,  232,  554143,  554154,  554170, 11, 16, 17, }, // Arabian
    {  450,  240,  554187,  554201,  554220, 14, 19, 20, }, // Argentina
    {  450,  250,  554240,  554260,  554285, 20, 25, 26, }, // Argentina_Western
    {  450,  268,  554311,  554323,  554340, 12, 17, 18, }, // Armenia
    {  450,  286,  554358,  554373,  554393, 15, 20, 21, }, // Atlantic
    {  450,  295,  554414,  554434,  554459, 20, 25, 26, }, // Australia_Central
    {  450,  313,  554485,  554517,  554554, 32, 37, 38, }, // Australia_CentralWestern
    {  450,  338,  554592,  554613,  554639, 21, 26, 27, }, // Australia_Eastern
    {  450,  356,  554666,  554686,  554711, 20, 25, 26, }, // Australia_Western
    {  450,  374,  554737,  554753,  554774, 16, 21, 22, }, // Azerbaijan
    {  450,  385,  554796,  554808,  554825, 12, 17, 18, }, // Azores
    {  450,  397,  554843,  554859,  554880, 16, 21, 22, }, // Bangladesh
    {  450,  415,       0,  554902,       0,  0, 12,  0, }, // Bhutan
    {  450,  422,       0,  554914,       0,  0, 12,  0, }, // Bolivia
    {  450,  437,  554926,  554939,  554957, 13, 18, 19, }, // Brasilia
    {  450,  454,       0,  554976,       0,  0, 23,  0, }, // Brunei
    {  450,  461,  554999,  555013,  555032, 14, 19, 20, }, // Cape_Verde
    {  450,  478,       0,  555052,       0,  0, 17,  0, }, // Chamorro
    {  450,  487,  555069,  555082,  555100, 13, 18, 19, }, // Chatham
    {  450,  495,  555119,  555129,  555144, 10, 15, 16, }, // Chile
    {  450,  501,  555160,  555171,  555187, 11, 16, 17, }, // China
    {  450,  507,       0,  555204,       0,  0, 16,  0, }, // Christmas
    {  450,  517,       0,  555220,       0,  0, 17,  0, }, // Cocos
    {  450,  523,  555237,  555250,  555268, 13, 18, 19, }, // Colombia
    {  450,  532,  555287,  555303,  555324, 16, 21, 34, }, // Cook
    {  450,  537,  555358,  555367,  555381,  9, 14, 15, }, // Cuba
    {  450,  548,       0,  555396,       0,  0, 11,  0, }, // Davis
    {  450,  564,       0,  555407,       0,  0, 21,  0, }, // DumontDUrville
    {  450,  601,       0,  555428,       0,  0, 18,  0, }, // East_Timor
    {  450,  612,  555446,  555463,  555485, 17, 22, 23, }, // Easter
    {  450,  619,       0,  555508,       0,  0, 13,  0, }, // Ecuador
    {  450,  627,  555521,  555539,  555562, 18, 23, 24, }, // Europe_Central
    {  450,  642,  555586,  555605,  555629, 19, 24, 25, }, // Europe_Eastern
    {  450,  657,       0,  555654,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  450,  680,  555685,  555703,  555726, 18, 23, 24, }, // Europe_Western
    {  450,  695,  555750,  555770,  555795, 20, 25, 26, }, // Falkland
    {  450,  704,  555821,  555830,  555844,  9, 14, 15, }, // Fiji
    {  450,  709,       0,  555859,       0,  0, 23,  0, }, // French_Guiana
    {  450,  723,       0,  555882,       0,  0, 35,  0, }, // French_Southern
    {  450,  746,       0,  555917,       0,  0, 15,  0, }, // Galapagos
    {  450,  756,       0,  555932,       0,  0, 13,  0, }, // Gambier
    {  450,  764,  555945,  555957,  555974, 12, 17, 18, }, // Georgia
    {  450,  772,       0,  555992,       0,  0, 19,  0, }, // Gilbert_Islands
    {  450,  788,       0,  556011,       0,  0, 15,  0, }, // GMT
    {  450,  830,  556026,  556050,  556079, 24, 29, 30, }, // Greenland_Eastern
    {  450,  848,  556109,  556132,  556160, 23, 28, 29, }, // Greenland_Western
    {  450,  871,       0,  556189,       0,  0, 15,  0, }, // Gulf
    {  450,  876,       0,  555871,       0,  0, 11,  0, }, // Guyana
    {  450,  883,  556204,  556226,  556253, 22, 27, 28, }, // Hawaii_Aleutian
    {  450,  899,  556281,  556296,  556316, 15, 20, 21, }, // Hong_Kong
    {  450,  909,  556337,  556347,  556362, 10, 15, 16, }, // Hovd
    {  450,  914,       0,  556378,       0,  0, 15,  0, }, // India
    {  450,  920,       0,  556393,       0,  0, 13,  0, }, // Indian_Ocean
    {  450,  933,       0,  556406,       0,  0, 15,  0, }, // Indochina
    {  450,  943,       0,  556421,       0,  0, 20,  0, }, // Indonesia_Central
    {  450,  961,       0,  556441,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  450,  979,       0,  556462,       0,  0, 20,  0, }, // Indonesia_Western
    {  450,  997,  556482,  556492,  556507, 10, 15, 16, }, // Iran
    {  450, 1008,  556523,  556535,  556552, 12, 17, 18, }, // Irkutsk
    {  450, 1016,  556570,  556582,  556599, 12, 17, 18, }, // Israel
    {  450, 1023,  556617,  556628,  556644, 11, 16, 17, }, // Japan
    {  450, 1058,       0,  556661,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  450, 1077,       0,  556683,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  450, 1106,  556704,  556714,  556729, 10, 15, 16, }, // Korea
    {  450, 1112,       0,  556745,       0,  0, 12,  0, }, // Kosrae
    {  450, 1119,  556757,  556774,  556796, 17, 22, 23, }, // Krasnoyarsk
    {  450, 1151,       0,  556819,       0,  0, 13,  0, }, // Kyrgystan
    {  450, 1175,       0,  556832,       0,  0, 16,  0, }, // Line_Islands
    {  450, 1188,  556848,  556862,  556881, 14, 19, 20, }, // Lord_Howe
    {  450, 1204,  556901,  556914,  556932, 13, 18, 19, }, // Magadan
    {  450, 1219,       0,  556951,       0,  0, 12,  0, }, // Malaysia
    {  450, 1228,       0,  556963,       0,  0, 17,  0, }, // Maldives
    {  450, 1237,       0,  556980,       0,  0, 17,  0, }, // Marquesas
    {  450, 1247,       0,  556997,       0,  0, 20,  0, }, // Marshall_Islands
    {  450, 1264,  557017,  557032,  557052, 15, 20, 21, }, // Mauritius
    {  450, 1274,       0,  557073,       0,  0, 12,  0, }, // Mawson
    {  450, 1281,  557085,  557107,  557134, 22, 27, 28, }, // Mexico_Pacific
    {  450, 1296,  557162,  557178,  557199, 16, 21, 22, }, // Mongolia
    {  450, 1305,  557221,  557232,  557248, 11, 16, 17, }, // Moscow
    {  450, 1312,       0,  557265,       0,  0, 13,  0, }, // Myanmar
    {  450, 1320,       0,  557278,       0,  0, 10,  0, }, // Nauru
    {  450, 1326,       0,  557288,       0,  0, 11,  0, }, // Nepal
    {  450, 1332,  557299,  557318,  557342, 19, 24, 25, }, // New_Caledonia
    {  450, 1346,  557367,  557384,  557406, 17, 22, 23, }, // New_Zealand
    {  450, 1358,  557429,  557447,  557470, 18, 23, 24, }, // Newfoundland
    {  450, 1371,       0,  557494,       0,  0, 10,  0, }, // Niue
    {  450, 1376,       0,  557504,       0,  0, 17,  0, }, // Norfolk
    {  450, 1384,  557521,  557545,  557574, 24, 29, 30, }, // Noronha
    {  450, 1406,  557604,  557621,  557643, 17, 22, 23, }, // Novosibirsk
    {  450, 1418,  557666,  557676,  557691, 10, 15, 16, }, // Omsk
    {  450, 1428,  557707,  557721,  557740, 14, 19, 20, }, // Pakistan
    {  450, 1437,       0,  557760,       0,  0, 11,  0, }, // Palau
    {  450, 1443,       0,  557771,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  450, 1460,  557793,  557807,  557826, 14, 19, 20, }, // Paraguay
    {  450, 1469,  557846,  557855,  557869,  9, 14, 15, }, // Peru
    {  450, 1474,  557884,  557902,  557925, 18, 23, 24, }, // Philippines
    {  450, 1486,       0,  557949,       0,  0, 19,  0, }, // Phoenix_Islands
    {  450, 1502,  557968,  557993,  558023, 25, 30, 31, }, // Pierre_Miquelon
    {  450, 1518,       0,  558054,       0,  0, 20,  0, }, // Pitcairn
    {  450, 1527,       0,  558074,       0,  0, 11,  0, }, // Ponape
    {  450, 1534,       0,  558085,       0,  0, 15,  0, }, // Pyongyang
    {  450, 1554,       0,  558100,       0,  0, 13,  0, }, // Reunion
    {  450, 1562,       0,  558113,       0,  0, 12,  0, }, // Rothera
    {  450, 1570,  558125,  558138,  558156, 13, 18, 19, }, // Sakhalin
    {  450, 1596,  558175,  558185,  558200, 10, 15, 16, }, // Samoa
    {  450, 1602,       0,  558216,       0,  0, 16,  0, }, // Seychelles
    {  450, 1624,       0,  558232,       0,  0, 19,  0, }, // Singapore
    {  450, 1634,       0,  558251,       0,  0, 19,  0, }, // Solomon
    {  450, 1642,       0,  558270,       0,  0, 18,  0, }, // South_Georgia
    {  450, 1656,       0,  558288,       0,  0, 13,  0, }, // Suriname
    {  450, 1676,       0,  558301,       0,  0, 10,  0, }, // Syowa
    {  450, 1682,       0,  558311,       0,  0, 11,  0, }, // Tahiti
    {  450, 1689,  558322,  558334,  558351, 12, 17, 18, }, // Taipei
    {  450, 1696,       0,  558369,       0,  0, 16,  0, }, // Tajikistan
    {  450, 1724,       0,  558385,       0,  0, 13,  0, }, // Tokelau
    {  450, 1732,  558398,  558408,  558423, 10, 15, 16, }, // Tonga
    {  450, 1738,       0,  558439,       0,  0, 11,  0, }, // Truk
    {  450, 1750,  558450,  558468,  558491, 18, 23, 24, }, // Turkmenistan
    {  450, 1763,       0,  558515,       0,  0, 11,  0, }, // Tuvalu
    {  450, 1777,  558526,  558539,  558557, 13, 18, 19, }, // Uruguay
    {  450, 1792,  558576,  558592,  558613, 16, 21, 22, }, // Uzbekistan
    {  450, 1803,  558635,  558647,  558664, 12, 17, 18, }, // Vanuatu
    {  450, 1811,       0,  558682,       0,  0, 14,  0, }, // Venezuela
    {  450, 1821,  558696,  558713,  558735, 17, 22, 23, }, // Vladivostok
    {  450, 1833,  558758,  558773,  558793, 15, 20, 21, }, // Volgograd
    {  450, 1843,       0,  558814,       0,  0, 12,  0, }, // Vostok
    {  450, 1850,       0,  558826,       0,  0, 14,  0, }, // Wake
    {  450, 1855,       0,  558840,       0,  0, 22,  0, }, // Wallis
    {  450, 1862,  558862,  558874,  558891, 12, 17, 18, }, // Yakutsk
    {  450, 1870,  558909,  558928,  558952, 19, 24, 25, }, // Yekaterinburg
    {  452,  788,       0,  558977,       0,  0, 26,  0, }, // GMT Northern Sotho/Latin/South Africa
    {  454,    0,  200540,  559003,  559017,  8, 14, 14, }, // Acre Norwegian Bokmal/Latin/Norway
    {  454,    5,       0,  559031,       0,  0, 12,  0, }, // Afghanistan
    {  454,   17,       0,  559043,       0,  0, 20,  0, }, // Africa_Central
    {  454,   32,       0,  559063,       0,  0, 16,  0, }, // Africa_Eastern
    {  454,   65,       0,  559079,       0,  0, 16,  0, }, // Africa_Southern
    {  454,   81,  559095,  559112,  559135, 17, 23, 23, }, // Africa_Western
    {  454,  107,  559158,  559170,  559188, 12, 18, 18, }, // Alaska
    {  454,  128,  200745,  559206,  559225, 10, 19, 17, }, // Almaty
    {  454,  135,  559242,  559263,  559285, 21, 22, 22, }, // Amazon
    {  454,  142,  559307,  559345,  559384, 38, 39, 39, }, // America_Central
    {  454,  158,  559423,  559465,  559508, 42, 43, 43, }, // America_Eastern
    {  454,  174,  559551,  559585,  559620, 34, 35, 35, }, // America_Mountain
    {  454,  191,  559655,  559704,  559754, 49, 50, 50, }, // America_Pacific
    {  454,  207,  559804,  559824,  559850, 20, 26, 26, }, // Anadyr
    {  454,  214,  559876,  559893,  559911, 17, 18, 18, }, // Apia
    {  454,  219,  201096,  559929,  559947,  9, 18, 16, }, // Aqtau
    {  454,  225,  201135,  559963,  559982, 10, 19, 17, }, // Aqtobe
    {  454,  232,  559999,  560010,  560029, 11, 19, 17, }, // Arabian
    {  454,  240,  201279,  201297,  201321, 14, 20, 20, }, // Argentina
    {  454,  250,  560046,  560064,  560088, 18, 24, 24, }, // Argentina_Western
    {  454,  268,  560112,  560123,  560140, 11, 17, 17, }, // Armenia
    {  454,  286,  560157,  560208,  560260, 51, 52, 52, }, // Atlantic
    {  454,  295,  560312,  560332,  560358, 20, 26, 26, }, // Australia_Central
    {  454,  313,  560384,  560409,  560440, 25, 31, 31, }, // Australia_CentralWestern
    {  454,  338,  560471,  560487,  560509, 16, 22, 22, }, // Australia_Eastern
    {  454,  356,  560531,  560548,  560571, 17, 23, 23, }, // Australia_Western
    {  454,  374,  560594,  560612,  560636, 18, 24, 24, }, // Azerbaijan
    {  454,  385,  560660,  560671,  560688, 11, 17, 17, }, // Azores
    {  454,  397,  560705,  560722,  560745, 17, 23, 23, }, // Bangladesh
    {  454,  415,       0,  560768,       0,  0, 12,  0, }, // Bhutan
    {  454,  422,       0,  560780,       0,  0, 14,  0, }, // Bolivia
    {  454,  437,  560794,  560815,  560837, 21, 22, 22, }, // Brasilia
    {  454,  454,       0,  560859,       0,  0, 30,  0, }, // Brunei
    {  454,  461,  560889,  560904,  560925, 15, 21, 21, }, // Cape_Verde
    {  454,  472,       0,  560946,       0,  0,  9,  0, }, // Casey
    {  454,  478,       0,  560955,       0,  0, 21,  0, }, // Chamorro
    {  454,  487,  560976,  560996,  561017, 20, 21, 21, }, // Chatham
    {  454,  495,  561038,  561050,  561068, 12, 18, 18, }, // Chile
    {  454,  501,  561086,  561098,  561116, 12, 18, 18, }, // China
    {  454,  507,       0,  561134,       0,  0, 25,  0, }, // Christmas
    {  454,  517,       0,  561159,       0,  0, 23,  0, }, // Cocos
    {  454,  523,  561182,  561197,  561218, 15, 21, 21, }, // Colombia
    {  454,  532,  561239,  561261,  561284, 22, 23, 28, }, // Cook
    {  454,  537,  561312,  561323,  561340, 11, 17, 17, }, // Cuba
    {  454,  548,       0,  561357,       0,  0, 18,  0, }, // Davis
    {  454,  564,       0,  561375,       0,  0, 29,  0, }, // DumontDUrville
    {  454,  601,       0,  561404,       0,  0, 17,  0, }, // East_Timor
    {  454,  612,  561421,  561442,  561464, 21, 22, 22, }, // Easter
    {  454,  619,       0,  561486,       0,  0, 16,  0, }, // Ecuador
    {  454,  627,  561502,  561522,  561548, 20, 26, 26, }, // Europe_Central
    {  454,  642,  561574,  561590,  561612, 16, 22, 22, }, // Europe_Eastern
    {  454,  657,       0,  561634,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  454,  680,  561656,  561673,  561696, 17, 23, 23, }, // Europe_Western
    {  454,  695,  561719,  561746,  561774, 27, 28, 28, }, // Falkland
    {  454,  704,  561802,  561814,  561832, 12, 18, 18, }, // Fiji
    {  454,  709,       0,  561850,       0,  0, 26,  0, }, // French_Guiana
    {  454,  723,       0,  561876,       0,  0, 38,  0, }, // French_Southern
    {  454,  746,       0,  561914,       0,  0, 27,  0, }, // Galapagos
    {  454,  756,       0,  561941,       0,  0, 20,  0, }, // Gambier
    {  454,  764,  561961,  561973,  561991, 12, 18, 18, }, // Georgia
    {  454,  772,       0,  562009,       0,  0, 25,  0, }, // Gilbert_Islands
    {  454,  788,       0,  562034,       0,  0, 19,  0, }, // GMT
    {  454,  830,  562053,  562070,  562093, 17, 23, 23, }, // Greenland_Eastern
    {  454,  848,  562116,  562134,  562158, 18, 24, 24, }, // Greenland_Western
    {  454,  866,       0,  562182,       0,  0,  8,  0, }, // Guam
    {  454,  871,       0,  562190,       0,  0, 24,  0, }, // Gulf
    {  454,  876,       0,  562214,       0,  0, 11,  0, }, // Guyana
    {  454,  883,  562225,  562256,  562288, 31, 32, 32, }, // Hawaii_Aleutian
    {  454,  899,  562320,  562341,  562363, 21, 22, 22, }, // Hong_Kong
    {  454,  909,  562385,  562403,  562422, 18, 19, 19, }, // Hovd
    {  454,  914,       0,  562441,       0,  0, 10,  0, }, // India
    {  454,  920,       0,  562451,       0,  0, 23,  0, }, // Indian_Ocean
    {  454,  933,       0,  562474,       0,  0, 16,  0, }, // Indochina
    {  454,  943,       0,  562490,       0,  0, 21,  0, }, // Indonesia_Central
    {  454,  961,       0,  562511,       0,  0, 17,  0, }, // Indonesia_Eastern
    {  454,  979,       0,  562528,       0,  0, 18,  0, }, // Indonesia_Western
    {  454,  997,  562546,  562556,  562572, 10, 16, 16, }, // Iran
    {  454, 1008,  562588,  562608,  562629, 20, 21, 21, }, // Irkutsk
    {  454, 1016,  562650,  562662,  562680, 12, 18, 18, }, // Israel
    {  454, 1023,  562698,  562709,  562726, 11, 17, 17, }, // Japan
    {  454, 1029,  562743,  562783,  562829, 40, 46, 46, }, // Kamchatka
    {  454, 1047,       0,  203697,       0,  0, 16,  0, }, // Kazakhstan
    {  454, 1058,       0,  562875,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  454, 1077,       0,  562894,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  454, 1106,  562914,  562926,  562944, 12, 18, 18, }, // Korea
    {  454, 1112,       0,  562962,       0,  0, 19,  0, }, // Kosrae
    {  454, 1119,  562981,  563005,  563030, 24, 25, 25, }, // Krasnoyarsk
    {  454, 1151,       0,  563055,       0,  0, 13,  0, }, // Kyrgystan
    {  454, 1161,       0,  563068,       0,  0,  9,  0, }, // Lanka
    {  454, 1175,       0,  563077,       0,  0, 23,  0, }, // Line_Islands
    {  454, 1188,  563100,  563126,  563153, 26, 27, 27, }, // Lord_Howe
    {  454, 1198,  563180,  563189,  563207,  9, 18, 16, }, // Macau
    {  454, 1204,  563223,  563243,  563264, 20, 21, 21, }, // Magadan
    {  454, 1219,       0,  563285,       0,  0, 13,  0, }, // Malaysia
    {  454, 1228,       0,  563298,       0,  0, 13,  0, }, // Maldives
    {  454, 1237,       0,  563311,       0,  0, 27,  0, }, // Marquesas
    {  454, 1247,       0,  563338,       0,  0, 17,  0, }, // Marshall_Islands
    {  454, 1264,  563355,  563368,  563387, 13, 19, 19, }, // Mauritius
    {  454, 1274,       0,  563406,       0,  0, 19,  0, }, // Mawson
    {  454, 1281,  563425,  563470,  563516, 45, 46, 46, }, // Mexico_Pacific
    {  454, 1296,  563562,  563585,  563609, 23, 24, 24, }, // Mongolia
    {  454, 1305,  563633,  563652,  563672, 19, 20, 20, }, // Moscow
    {  454, 1312,       0,  563692,       0,  0, 13,  0, }, // Myanmar
    {  454, 1320,       0,  563705,       0,  0, 11,  0, }, // Nauru
    {  454, 1326,       0,  563716,       0,  0, 11,  0, }, // Nepal
    {  454, 1332,  563727,  563740,  563759, 13, 19, 19, }, // New_Caledonia
    {  454, 1346,  563778,  563794,  563816, 16, 22, 22, }, // New_Zealand
    {  454, 1358,  563838,  563863,  563889, 25, 26, 26, }, // Newfoundland
    {  454, 1371,       0,  563915,       0,  0, 17,  0, }, // Niue
    {  454, 1376,  563932,  563955,  563979, 23, 24, 24, }, // Norfolk
    {  454, 1384,  564003,  564035,  564068, 32, 33, 33, }, // Noronha
    {  454, 1392,       0,  564101,       0,  0, 18,  0, }, // North_Mariana
    {  454, 1406,  564119,  564143,  564168, 24, 25, 25, }, // Novosibirsk
    {  454, 1418,  564193,  564210,  564228, 17, 18, 18, }, // Omsk
    {  454, 1428,  564246,  564260,  564280, 14, 20, 20, }, // Pakistan
    {  454, 1437,       0,  564300,       0,  0, 12,  0, }, // Palau
    {  454, 1443,       0,  564312,       0,  0, 12,  0, }, // Papua_New_Guinea
    {  454, 1460,  564324,  564340,  564362, 16, 22, 22, }, // Paraguay
    {  454, 1469,  564384,  564396,  564414, 12, 18, 18, }, // Peru
    {  454, 1474,  564432,  564446,  564466, 14, 20, 20, }, // Philippines
    {  454, 1486,       0,  564486,       0,  0, 25,  0, }, // Phoenix_Islands
    {  454, 1502,  564511,  564548,  564586, 37, 38, 38, }, // Pierre_Miquelon
    {  454, 1518,       0,  564624,       0,  0, 21,  0, }, // Pitcairn
    {  454, 1527,       0,  564645,       0,  0, 20,  0, }, // Ponape
    {  454, 1534,       0,  564665,       0,  0, 22,  0, }, // Pyongyang
    {  454, 1544,  205194,  564687,  564709, 13, 22, 20, }, // Qyzylorda
    {  454, 1554,       0,  564729,       0,  0, 20,  0, }, // Reunion
    {  454, 1562,       0,  564749,       0,  0, 20,  0, }, // Rothera
    {  454, 1570,  564769,  564790,  564812, 21, 22, 22, }, // Sakhalin
    {  454, 1579,  564834,  564854,  564880, 20, 26, 26, }, // Samara
    {  454, 1596,  564906,  564918,  564936, 12, 18, 18, }, // Samoa
    {  454, 1602,       0,  564954,       0,  0, 15,  0, }, // Seychelles
    {  454, 1624,       0,  564969,       0,  0, 14,  0, }, // Singapore
    {  454, 1634,       0,  564983,       0,  0, 13,  0, }, // Solomon
    {  454, 1642,       0,  564996,       0,  0, 24,  0, }, // South_Georgia
    {  454, 1656,       0,  565020,       0,  0, 13,  0, }, // Suriname
    {  454, 1676,       0,  565033,       0,  0, 18,  0, }, // Syowa
    {  454, 1682,       0,  565051,       0,  0, 12,  0, }, // Tahiti
    {  454, 1689,  565063,  565082,  565102, 19, 20, 20, }, // Taipei
    {  454, 1696,       0,  565122,       0,  0, 14,  0, }, // Tajikistan
    {  454, 1724,       0,  565136,       0,  0, 20,  0, }, // Tokelau
    {  454, 1732,  565156,  565168,  565186, 12, 18, 18, }, // Tonga
    {  454, 1738,       0,  565204,       0,  0, 23,  0, }, // Truk
    {  454, 1750,  565227,  565240,  565259, 13, 19, 19, }, // Turkmenistan
    {  454, 1763,       0,  565278,       0,  0, 11,  0, }, // Tuvalu
    {  454, 1777,  565289,  565304,  565325, 15, 21, 21, }, // Uruguay
    {  454, 1792,  565346,  565358,  565376, 12, 18, 18, }, // Uzbekistan
    {  454, 1803,  565394,  565407,  565426, 13, 19, 19, }, // Vanuatu
    {  454, 1811,       0,  565445,       0,  0, 16,  0, }, // Venezuela
    {  454, 1821,  565461,  565485,  565510, 24, 25, 25, }, // Vladivostok
    {  454, 1833,  565535,  565557,  565580, 22, 23, 23, }, // Volgograd
    {  454, 1843,       0,  565603,       0,  0, 19,  0, }, // Vostok
    {  454, 1850,       0,  565622,       0,  0, 24,  0, }, // Wake
    {  454, 1855,       0,  565646,       0,  0, 35,  0, }, // Wallis
    {  454, 1862,  565681,  565701,  565722, 20, 21, 21, }, // Yakutsk
    {  454, 1870,  565743,  565769,  565796, 26, 27, 27, }, // Yekaterinburg
    {  454, 1892,       0,  565823,       0,  0, 18,  0, }, // Yukon
    {  456,    0,  200540,  559003,  559017,  8, 14, 14, }, // Acre Norwegian Nynorsk/Latin/Norway
    {  456,    5,       0,  559031,       0,  0, 12,  0, }, // Afghanistan
    {  456,   17,       0,  559043,       0,  0, 20,  0, }, // Africa_Central
    {  456,   32,       0,  565841,       0,  0, 17,  0, }, // Africa_Eastern
    {  456,   65,       0,  559079,       0,  0, 16,  0, }, // Africa_Southern
    {  456,   81,  559095,  565858,  565883, 17, 25, 23, }, // Africa_Western
    {  456,  107,  559158,  559170,  565906, 12, 18, 18, }, // Alaska
    {  456,  128,  200745,  559206,  559225, 10, 19, 17, }, // Almaty
    {  456,  135,  559242,  559263,  565924, 21, 22, 22, }, // Amazon
    {  456,  142,  565946,  565980,  566015, 34, 35, 35, }, // America_Central
    {  456,  158,  566050,  566093,  566137, 43, 44, 44, }, // America_Eastern
    {  456,  174,  559551,  559585,  566181, 34, 35, 35, }, // America_Mountain
    {  456,  191,  566216,  566265,  566315, 49, 50, 50, }, // America_Pacific
    {  456,  207,  559804,  559824,  559850, 20, 26, 26, }, // Anadyr
    {  456,  214,  559876,  559893,  566365, 17, 18, 18, }, // Apia
    {  456,  219,  201096,  559929,  559947,  9, 18, 16, }, // Aqtau
    {  456,  225,  201135,  559963,  559982, 10, 19, 17, }, // Aqtobe
    {  456,  232,  559999,  566383,  566400, 11, 17, 17, }, // Arabian
    {  456,  240,  201279,  201297,  566417, 14, 20, 20, }, // Argentina
    {  456,  250,  560046,  560064,  566437, 18, 24, 24, }, // Argentina_Western
    {  456,  268,  560112,  560123,  566461, 11, 17, 17, }, // Armenia
    {  456,  286,  560157,  560208,  566478, 51, 52, 52, }, // Atlantic
    {  456,  295,  560312,  566530,  566558, 20, 28, 26, }, // Australia_Central
    {  456,  313,  560384,  566584,  566617, 25, 33, 31, }, // Australia_CentralWestern
    {  456,  338,  566648,  566665,  566690, 17, 25, 23, }, // Australia_Eastern
    {  456,  356,  560531,  566713,  566738, 17, 25, 23, }, // Australia_Western
    {  456,  374,  560594,  560612,  566761, 18, 24, 24, }, // Azerbaijan
    {  456,  385,  560660,  560671,  566785, 11, 17, 17, }, // Azores
    {  456,  397,  560705,  560722,  566802, 17, 23, 23, }, // Bangladesh
    {  456,  415,       0,  560768,       0,  0, 12,  0, }, // Bhutan
    {  456,  422,       0,  560780,       0,  0, 14,  0, }, // Bolivia
    {  456,  437,  560794,  560815,  566825, 21, 22, 22, }, // Brasilia
    {  456,  454,       0,  560859,       0,  0, 30,  0, }, // Brunei
    {  456,  461,  560889,  560904,  566847, 15, 21, 21, }, // Cape_Verde
    {  456,  472,       0,  560946,       0,  0,  9,  0, }, // Casey
    {  456,  478,       0,  560955,       0,  0, 21,  0, }, // Chamorro
    {  456,  487,  560976,  560996,  566868, 20, 21, 21, }, // Chatham
    {  456,  495,  561038,  561050,  566889, 12, 18, 18, }, // Chile
    {  456,  501,  561086,  561098,  566907, 12, 18, 18, }, // China
    {  456,  507,       0,  561134,       0,  0, 25,  0, }, // Christmas
    {  456,  517,       0,  566925,       0,  0, 23,  0, }, // Cocos
    {  456,  523,  566948,  566963,  566984, 15, 21, 21, }, // Colombia
    {  456,  532,  567005,  567027,  567050, 22, 23, 23, }, // Cook
    {  456,  537,  567073,  567084,  567101, 11, 17, 17, }, // Cuba
    {  456,  548,       0,  561357,       0,  0, 18,  0, }, // Davis
    {  456,  564,       0,  567118,       0,  0, 29,  0, }, // DumontDUrville
    {  456,  601,       0,  567147,       0,  0, 18,  0, }, // East_Timor
    {  456,  612,  561421,  561442,  567165, 21, 22, 22, }, // Easter
    {  456,  619,       0,  561486,       0,  0, 16,  0, }, // Ecuador
    {  456,  627,  561502,  567187,  567215, 20, 28, 26, }, // Europe_Central
    {  456,  642,  567241,  567258,  567283, 17, 25, 23, }, // Europe_Eastern
    {  456,  657,       0,  567306,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  456,  680,  561656,  567329,  567354, 17, 25, 23, }, // Europe_Western
    {  456,  695,  567377,  567404,  567432, 27, 28, 28, }, // Falkland
    {  456,  704,  561802,  561814,  567460, 12, 18, 18, }, // Fiji
    {  456,  709,       0,  561850,       0,  0, 26,  0, }, // French_Guiana
    {  456,  723,       0,  567478,       0,  0, 38,  0, }, // French_Southern
    {  456,  746,       0,  567516,       0,  0, 27,  0, }, // Galapagos
    {  456,  756,       0,  561941,       0,  0, 20,  0, }, // Gambier
    {  456,  764,  561961,  561973,  567543, 12, 18, 18, }, // Georgia
    {  456,  772,       0,  567561,       0,  0, 25,  0, }, // Gilbert_Islands
    {  456,  788,       0,  562034,       0,  0, 19,  0, }, // GMT
    {  456,  830,  567586,  567604,  567628, 18, 24, 24, }, // Greenland_Eastern
    {  456,  848,  562116,  562134,  567652, 18, 24, 24, }, // Greenland_Western
    {  456,  866,       0,  562182,       0,  0,  8,  0, }, // Guam
    {  456,  871,       0,  562190,       0,  0, 24,  0, }, // Gulf
    {  456,  876,       0,  562214,       0,  0, 11,  0, }, // Guyana
    {  456,  883,  562225,  562256,  567676, 31, 32, 32, }, // Hawaii_Aleutian
    {  456,  899,  567708,  567728,  567754, 20, 26, 26, }, // Hong_Kong
    {  456,  909,  562385,  562403,  567780, 18, 19, 19, }, // Hovd
    {  456,  914,       0,  562441,       0,  0, 10,  0, }, // India
    {  456,  920,       0,  562451,       0,  0, 23,  0, }, // Indian_Ocean
    {  456,  933,       0,  562474,       0,  0, 16,  0, }, // Indochina
    {  456,  943,       0,  562490,       0,  0, 21,  0, }, // Indonesia_Central
    {  456,  961,       0,  567799,       0,  0, 18,  0, }, // Indonesia_Eastern
    {  456,  979,       0,  562528,       0,  0, 18,  0, }, // Indonesia_Western
    {  456,  997,  562546,  562556,  567817, 10, 16, 16, }, // Iran
    {  456, 1008,  562588,  562608,  567833, 20, 21, 21, }, // Irkutsk
    {  456, 1016,  562650,  562662,  567854, 12, 18, 18, }, // Israel
    {  456, 1023,  562698,  562709,  567872, 11, 17, 17, }, // Japan
    {  456, 1029,  562743,  562783,  562829, 40, 46, 46, }, // Kamchatka
    {  456, 1047,       0,  203697,       0,  0, 16,  0, }, // Kazakhstan
    {  456, 1058,       0,  567889,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  456, 1077,       0,  562894,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  456, 1106,  562914,  562926,  567909, 12, 18, 18, }, // Korea
    {  456, 1112,       0,  562962,       0,  0, 19,  0, }, // Kosrae
    {  456, 1119,  562981,  563005,  567927, 24, 25, 25, }, // Krasnoyarsk
    {  456, 1151,       0,  563055,       0,  0, 13,  0, }, // Kyrgystan
    {  456, 1161,       0,  563068,       0,  0,  9,  0, }, // Lanka
    {  456, 1175,       0,  567952,       0,  0, 22,  0, }, // Line_Islands
    {  456, 1188,  563100,  563126,  567974, 26, 27, 27, }, // Lord_Howe
    {  456, 1198,  563180,  563189,  563207,  9, 18, 16, }, // Macau
    {  456, 1204,  563223,  563243,  568001, 20, 21, 21, }, // Magadan
    {  456, 1219,       0,  563285,       0,  0, 13,  0, }, // Malaysia
    {  456, 1228,       0,  563298,       0,  0, 13,  0, }, // Maldives
    {  456, 1237,       0,  568022,       0,  0, 27,  0, }, // Marquesas
    {  456, 1247,       0,  563338,       0,  0, 17,  0, }, // Marshall_Islands
    {  456, 1264,  563355,  563368,  568049, 13, 19, 19, }, // Mauritius
    {  456, 1274,       0,  563406,       0,  0, 19,  0, }, // Mawson
    {  456, 1281,  568068,  568113,  568159, 45, 46, 46, }, // Mexico_Pacific
    {  456, 1296,  563562,  563585,  568205, 23, 24, 24, }, // Mongolia
    {  456, 1305,  563633,  563652,  568229, 19, 20, 20, }, // Moscow
    {  456, 1312,       0,  563692,       0,  0, 13,  0, }, // Myanmar
    {  456, 1320,       0,  563705,       0,  0, 11,  0, }, // Nauru
    {  456, 1326,       0,  563716,       0,  0, 11,  0, }, // Nepal
    {  456, 1332,  563727,  563740,  568249, 13, 19, 19, }, // New_Caledonia
    {  456, 1346,  568268,  568283,  568304, 15, 21, 21, }, // New_Zealand
    {  456, 1358,  563838,  563863,  568325, 25, 26, 26, }, // Newfoundland
    {  456, 1371,       0,  563915,       0,  0, 17,  0, }, // Niue
    {  456, 1376,  563932,  563955,  568351, 23, 24, 24, }, // Norfolk
    {  456, 1384,  564003,  564035,  568375, 32, 33, 33, }, // Noronha
    {  456, 1392,       0,  564101,       0,  0, 18,  0, }, // North_Mariana
    {  456, 1406,  564119,  564143,  568408, 24, 25, 25, }, // Novosibirsk
    {  456, 1418,  564193,  564210,  568433, 17, 18, 18, }, // Omsk
    {  456, 1428,  564246,  564260,  568451, 14, 20, 20, }, // Pakistan
    {  456, 1437,       0,  564300,       0,  0, 12,  0, }, // Palau
    {  456, 1443,       0,  564312,       0,  0, 12,  0, }, // Papua_New_Guinea
    {  456, 1460,  564324,  564340,  568471, 16, 22, 22, }, // Paraguay
    {  456, 1469,  564384,  564396,  568493, 12, 18, 18, }, // Peru
    {  456, 1474,  564432,  564446,  568511, 14, 20, 20, }, // Philippines
    {  456, 1486,       0,  568531,       0,  0, 25,  0, }, // Phoenix_Islands
    {  456, 1502,  564511,  564548,  568556, 37, 38, 38, }, // Pierre_Miquelon
    {  456, 1518,       0,  564624,       0,  0, 21,  0, }, // Pitcairn
    {  456, 1527,       0,  564645,       0,  0, 20,  0, }, // Ponape
    {  456, 1534,       0,  564665,       0,  0, 22,  0, }, // Pyongyang
    {  456, 1544,  205194,  564687,  564709, 13, 22, 20, }, // Qyzylorda
    {  456, 1554,       0,  564729,       0,  0, 20,  0, }, // Reunion
    {  456, 1562,       0,  564749,       0,  0, 20,  0, }, // Rothera
    {  456, 1570,  564769,  564790,  568594, 21, 22, 22, }, // Sakhalin
    {  456, 1579,  564834,  564854,  564880, 20, 26, 26, }, // Samara
    {  456, 1596,  564906,  564918,  568616, 12, 18, 18, }, // Samoa
    {  456, 1602,       0,  564954,       0,  0, 15,  0, }, // Seychelles
    {  456, 1624,       0,  564969,       0,  0, 14,  0, }, // Singapore
    {  456, 1634,       0,  564983,       0,  0, 13,  0, }, // Solomon
    {  456, 1642,       0,  564996,       0,  0, 24,  0, }, // South_Georgia
    {  456, 1656,       0,  565020,       0,  0, 13,  0, }, // Suriname
    {  456, 1676,       0,  565033,       0,  0, 18,  0, }, // Syowa
    {  456, 1682,       0,  565051,       0,  0, 12,  0, }, // Tahiti
    {  456, 1689,  565063,  565082,  568634, 19, 20, 20, }, // Taipei
    {  456, 1696,       0,  565122,       0,  0, 14,  0, }, // Tajikistan
    {  456, 1724,       0,  565136,       0,  0, 20,  0, }, // Tokelau
    {  456, 1732,  565156,  565168,  568654, 12, 18, 18, }, // Tonga
    {  456, 1738,       0,  568672,       0,  0, 23,  0, }, // Truk
    {  456, 1750,  565227,  565240,  568695, 13, 19, 19, }, // Turkmenistan
    {  456, 1763,       0,  565278,       0,  0, 11,  0, }, // Tuvalu
    {  456, 1777,  565289,  565304,  568714, 15, 21, 21, }, // Uruguay
    {  456, 1792,  565346,  565358,  568735, 12, 18, 18, }, // Uzbekistan
    {  456, 1803,  565394,  565407,  568753, 13, 19, 19, }, // Vanuatu
    {  456, 1811,       0,  565445,       0,  0, 16,  0, }, // Venezuela
    {  456, 1821,  565461,  565485,  568772, 24, 25, 25, }, // Vladivostok
    {  456, 1833,  565535,  565557,  568797, 22, 23, 23, }, // Volgograd
    {  456, 1843,       0,  565603,       0,  0, 19,  0, }, // Vostok
    {  456, 1850,       0,  565622,       0,  0, 24,  0, }, // Wake
    {  456, 1855,       0,  568820,       0,  0, 35,  0, }, // Wallis
    {  456, 1862,  565681,  565701,  568855, 20, 21, 21, }, // Yakutsk
    {  456, 1870,  565743,  565769,  568876, 26, 27, 27, }, // Yekaterinburg
    {  456, 1892,       0,  565823,       0,  0, 18,  0, }, // Yukon
    {  460,    5,       0,  568903,       0,  0, 16,  0, }, // Afghanistan Occitan/Latin/France
    {  460,   17,       0,  568919,       0,  0, 21,  0, }, // Africa_Central
    {  460,   32,       0,  568940,       0,  0, 21,  0, }, // Africa_Eastern
    {  460,   65,       0,  568961,       0,  0, 16,  0, }, // Africa_Southern
    {  460,   81,  568977,  569001,  569036, 24, 35, 32, }, // Africa_Western
    {  460,  107,   57205,  569068,  137647, 12, 23, 20, }, // Alaska
    {  460,  135,  569091,  569108,  569136, 17, 28, 22, }, // Amazon
    {  460,  142,  569158,  569170,  569193, 12, 23, 20, }, // America_Central
    {  460,  158,  569213,  569225,  569248, 12, 23, 20, }, // America_Eastern
    {  460,  174,  569268,  569286,  569315, 18, 29, 26, }, // America_Mountain
    {  460,  191,  369511,  569341,  569367, 15, 26, 23, }, // America_Pacific
    {  460,  214,   57886,  569390,  569410, 10, 20, 17, }, // Apia
    {  460,  232,   58048,  569427,  569450, 12, 23, 20, }, // Arabian
    {  460,  240,  569470,  569485,  569511, 15, 26, 23, }, // Argentina
    {  460,  250,  569534,  569559,  569595, 25, 36, 33, }, // Argentina_Western
    {  460,  268,   58270,  569628,  569652, 13, 24, 21, }, // Armenia
    {  460,  286,   42924,  569673,  569701, 17, 28, 25, }, // Atlantic
    {  460,  295,  569726,  569750,  569784, 24, 34, 32, }, // Australia_Central
    {  460,  313,  569816,  569849,  569892, 33, 43, 52, }, // Australia_CentralWestern
    {  460,  338,  138877,  569944,  138937, 24, 34, 32, }, // Australia_Eastern
    {  460,  356,  138970,  569978,  139034, 26, 36, 34, }, // Australia_Western
    {  460,  374,  570014,  570030,  570057, 16, 27, 24, }, // Azerbaijan
    {  460,  385,  570081,  570098,  570126, 17, 28, 25, }, // Azores
    {  460,  397,  570151,  570168,  570196, 17, 28, 25, }, // Bangladesh
    {  460,  415,       0,  570221,       0,  0, 12,  0, }, // Bhutan
    {  460,  422,       0,   59024,       0,  0, 14,  0, }, // Bolivia
    {  460,  437,   59039,  570233,  570259, 15, 26, 23, }, // Brasilia
    {  460,  454,       0,  139389,       0,  0, 13,  0, }, // Brunei
    {  460,  461,  570282,  570297,  570323, 15, 26, 23, }, // Cape_Verde
    {  460,  478,       0,  570346,       0,  0, 25,  0, }, // Chamorro
    {  460,  487,   59240,  570371,  139531, 14, 24, 22, }, // Chatham
    {  460,  495,   59303,  570395,  570418, 12, 23, 20, }, // Chile
    {  460,  501,   59360,  570438,  570461, 12, 23, 20, }, // China
    {  460,  507,       0,  570481,       0,  0, 22,  0, }, // Christmas
    {  460,  517,       0,  570503,       0,  0, 22,  0, }, // Cocos
    {  460,  523,  139712,  570525,  139754, 15, 26, 23, }, // Colombia
    {  460,  532,  570551,  570572,  570603, 21, 31, 35, }, // Cook
    {  460,  537,   59635,  570638,  139889, 11, 22, 19, }, // Cuba
    {  460,  548,       0,   59689,       0,  0, 12,  0, }, // Davis
    {  460,  564,       0,   59702,       0,  0, 23,  0, }, // DumontDUrville
    {  460,  601,       0,  570660,       0,  0, 21,  0, }, // East_Timor
    {  460,  612,  570681,  570705,  570740, 24, 35, 32, }, // Easter
    {  460,  619,       0,  570772,       0,  0, 13,  0, }, // Ecuador
    {  460,  627,  570785,  570806,  570838, 21, 32, 29, }, // Europe_Central
    {  460,  642,  570867,  570888,  570920, 21, 32, 29, }, // Europe_Eastern
    {  460,  657,       0,  570949,       0,  0, 34,  0, }, // Europe_Further_Eastern
    {  460,  680,  570983,  571005,  571038, 22, 33, 30, }, // Europe_Western
    {  460,  695,  571068,  571093,  571129, 25, 36, 33, }, // Falkland
    {  460,  704,  140425,  571162,  140459, 11, 21, 19, }, // Fiji
    {  460,  709,       0,  571183,       0,  0, 26,  0, }, // French_Guiana
    {  460,  723,       0,  571209,       0,  0, 51,  0, }, // French_Southern
    {  460,  746,       0,  571260,       0,  0, 16,  0, }, // Galapagos
    {  460,  756,       0,   60373,       0,  0, 14,  0, }, // Gambier
    {  460,  764,  140569,  571276,  140609, 14, 25, 22, }, // Georgia
    {  460,  772,       0,  140632,       0,  0, 24,  0, }, // Gilbert_Islands
    {  460,  788,       0,  571301,       0,  0, 28,  0, }, // GMT
    {  460,  830,  571329,  571356,  571394, 27, 38, 35, }, // Greenland_Eastern
    {  460,  848,  571429,  571457,  571496, 28, 39, 36, }, // Greenland_Western
    {  460,  871,       0,  571532,       0,  0, 23,  0, }, // Gulf
    {  460,  876,       0,  571183,       0,  0, 17,  0, }, // Guyana
    {  460,  883,  571555,  571578,  571612, 23, 34, 31, }, // Hawaii_Aleutian
    {  460,  899,  141073,  571643,  141117, 16, 27, 24, }, // Hong_Kong
    {  460,  909,   60913,  571670,  571692, 11, 22, 19, }, // Hovd
    {  460,  914,       0,  571711,       0,  0, 25,  0, }, // India
    {  460,  920,       0,  571736,       0,  0, 20,  0, }, // Indian_Ocean
    {  460,  933,       0,   61014,       0,  0, 15,  0, }, // Indochina
    {  460,  943,       0,  571756,       0,  0, 25,  0, }, // Indonesia_Central
    {  460,  961,       0,  571781,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  460,  979,       0,  571807,       0,  0, 28,  0, }, // Indonesia_Western
    {  460,  997,  571835,  571845,  571866, 10, 21, 18, }, // Iran
    {  460, 1008,   61156,  571884,  141411, 13, 24, 21, }, // Irkutsk
    {  460, 1016,  571908,  571920,  571943, 12, 23, 20, }, // Israel
    {  460, 1023,  368993,  571963,  571986, 12, 23, 20, }, // Japan
    {  460, 1058,       0,  572006,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  460, 1077,       0,  572033,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  460, 1106,  572062,  572074,  572097, 12, 23, 20, }, // Korea
    {  460, 1112,       0,   61557,       0,  0, 13,  0, }, // Kosrae
    {  460, 1119,   61571,  572117,  572146, 18, 29, 26, }, // Krasnoyarsk
    {  460, 1151,       0,  572172,       0,  0, 17,  0, }, // Kyrgystan
    {  460, 1175,       0,  572189,       0,  0, 32,  0, }, // Line_Islands
    {  460, 1188,   61701,  572221,  572247, 16, 26, 24, }, // Lord_Howe
    {  460, 1204,  141987,  572271,  142027, 14, 25, 22, }, // Magadan
    {  460, 1219,       0,  142050,       0,  0, 15,  0, }, // Malaysia
    {  460, 1228,       0,  572296,       0,  0, 16,  0, }, // Maldives
    {  460, 1237,       0,  572312,       0,  0, 26,  0, }, // Marquesas
    {  460, 1247,       0,  572338,       0,  0, 25,  0, }, // Marshall_Islands
    {  460, 1264,   61973,  572363,  142137, 14, 25, 22, }, // Mauritius
    {  460, 1274,       0,   62039,       0,  0, 13,  0, }, // Mawson
    {  460, 1281,  572388,  572411,  572445, 23, 34, 31, }, // Mexico_Pacific
    {  460, 1296,  369600,  572476,  572504, 17, 28, 25, }, // Mongolia
    {  460, 1305,  572529,  572542,  572566, 13, 24, 21, }, // Moscow
    {  460, 1312,       0,   62278,       0,  0, 14,  0, }, // Myanmar
    {  460, 1320,       0,   62293,       0,  0, 12,  0, }, // Nauru
    {  460, 1326,       0,  294850,       0,  0, 12,  0, }, // Nepal
    {  460, 1332,  572587,  572608,  572639, 21, 31, 29, }, // New_Caledonia
    {  460, 1346,  572668,  572687,  572716, 19, 29, 27, }, // New_Zealand
    {  460, 1358,  572743,  572760,  572788, 17, 28, 25, }, // Newfoundland
    {  460, 1371,       0,   62566,       0,  0, 11,  0, }, // Niue
    {  460, 1376,  142616,  572813,  142670, 21, 31, 29, }, // Norfolk
    {  460, 1384,  572844,  572867,  572901, 23, 34, 31, }, // Noronha
    {  460, 1406,   62804,  572932,  572961, 18, 29, 26, }, // Novosibirsk
    {  460, 1418,   62879,  572987,  142871, 10, 21, 18, }, // Omsk
    {  460, 1428,  370182,  573008,  573034, 15, 26, 23, }, // Pakistan
    {  460, 1437,       0,  573057,       0,  0, 13,  0, }, // Palau
    {  460, 1443,       0,  573070,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  460, 1460,  370271,  573094,  573120, 15, 26, 23, }, // Paraguay
    {  460, 1469,  573143,  573154,  573176, 11, 22, 19, }, // Peru
    {  460, 1474,   63167,  573195,  573222, 16, 27, 24, }, // Philippines
    {  460, 1486,       0,  573246,       0,  0, 22,  0, }, // Phoenix_Islands
    {  460, 1502,  573268,  573296,  573335, 28, 39, 36, }, // Pierre_Miquelon
    {  460, 1518,       0,   63373,       0,  0, 15,  0, }, // Pitcairn
    {  460, 1527,       0,  295864,       0,  0, 14,  0, }, // Ponape
    {  460, 1534,       0,   63403,       0,  0, 16,  0, }, // Pyongyang
    {  460, 1554,       0,  573371,       0,  0, 17,  0, }, // Reunion
    {  460, 1562,       0,   63504,       0,  0, 14,  0, }, // Rothera
    {  460, 1570,  573388,  573402,  573427, 14, 25, 22, }, // Sakhalin
    {  460, 1596,   63642,  573449,  143469, 12, 22, 20, }, // Samoa
    {  460, 1602,       0,  573471,       0,  0, 17,  0, }, // Seychelles
    {  460, 1624,       0,  573488,       0,  0, 15,  0, }, // Singapore
    {  460, 1634,       0,  573503,       0,  0, 24,  0, }, // Solomon
    {  460, 1642,       0,  143552,       0,  0, 22,  0, }, // South_Georgia
    {  460, 1656,       0,  143575,       0,  0, 14,  0, }, // Suriname
    {  460, 1676,       0,   63808,       0,  0, 12,  0, }, // Syowa
    {  460, 1682,       0,   63821,       0,  0, 13,  0, }, // Tahiti
    {  460, 1689,  143604,  573527,  143642, 13, 24, 21, }, // Taipei
    {  460, 1696,       0,  370877,       0,  0, 17,  0, }, // Tajikistan
    {  460, 1724,       0,   63915,       0,  0, 14,  0, }, // Tokelau
    {  460, 1732,   63930,  573551,  143707, 12, 22, 20, }, // Tonga
    {  460, 1738,       0,   63987,       0,  0, 12,  0, }, // Truk
    {  460, 1750,  370937,  573573,  573603, 19, 30, 27, }, // Turkmenistan
    {  460, 1763,       0,   64081,       0,  0, 13,  0, }, // Tuvalu
    {  460, 1777,  573630,  573646,  573673, 16, 27, 24, }, // Uruguay
    {  460, 1792,  371074,  573697,  573725, 17, 28, 25, }, // Uzbekistan
    {  460, 1803,   64239,  573750,  573774, 14, 24, 22, }, // Vanuatu
    {  460, 1811,       0,  144016,       0,  0, 16,  0, }, // Venezuela
    {  460, 1821,   64319,  573796,  144062, 18, 29, 26, }, // Vladivostok
    {  460, 1833,  144089,  573825,  144133, 16, 27, 24, }, // Volgograd
    {  460, 1843,       0,   64463,       0,  0, 13,  0, }, // Vostok
    {  460, 1850,       0,  573852,       0,  0, 18,  0, }, // Wake
    {  460, 1855,       0,  296869,       0,  0, 22,  0, }, // Wallis
    {  460, 1862,   64521,  573870,  573895, 14, 25, 22, }, // Yakutsk
    {  460, 1870,  573917,  573935,  573964, 18, 29, 26, }, // Yekaterinburg
    {  460, 1892,       0,  144347,       0,  0, 12,  0, }, // Yukon
    {  461,   17,       0,  568919,       0,  0, 20,  0, }, // Africa_Central Occitan/Latin/Spain
    {  461,   32,       0,  573990,       0,  0, 21,  0, }, // Africa_Eastern
    {  461,   81,  574011,  574034,  574067, 23, 33, 31, }, // Africa_Western
    {  461,  107,   57205,  574098,  137647, 12, 22, 20, }, // Alaska
    {  461,  135,  574120,  574136,  574162, 16, 26, 24, }, // Amazon
    {  461,  142,  574186,  574197,  574218, 11, 21, 19, }, // America_Central
    {  461,  158,  574237,  574249,  574271, 12, 22, 20, }, // America_Eastern
    {  461,  174,  574291,  574316,  574351, 25, 35, 33, }, // America_Mountain
    {  461,  191,  574384,  574400,  574426, 16, 26, 24, }, // America_Pacific
    {  461,  232,   58048,  574450,  574472, 12, 22, 20, }, // Arabian
    {  461,  240,  569470,  574492,  569511, 15, 25, 23, }, // Argentina
    {  461,  250,  574517,  574542,  574578, 25, 36, 34, }, // Argentina_Western
    {  461,  268,   58270,  574612,  569652, 13, 23, 21, }, // Armenia
    {  461,  286,  574635,  574651,  574677, 16, 26, 24, }, // Atlantic
    {  461,  295,  574701,  574724,  574757, 23, 33, 31, }, // Australia_Central
    {  461,  313,  574788,  574820,  574863, 32, 43, 40, }, // Australia_CentralWestern
    {  461,  338,  574903,  574927,  574961, 24, 34, 32, }, // Australia_Eastern
    {  461,  356,  574993,  575019,  575055, 26, 36, 44, }, // Australia_Western
    {  461,  374,  570014,  575099,  570057, 16, 26, 24, }, // Azerbaijan
    {  461,  385,  575125,  575139,  575163, 14, 24, 22, }, // Azores
    {  461,  397,  570151,  575185,  139268, 17, 27, 25, }, // Bangladesh
    {  461,  437,   59039,  575212,  570259, 15, 25, 23, }, // Brasilia
    {  461,  461,  570282,  575237,  570323, 15, 25, 23, }, // Cape_Verde
    {  461,  495,   59303,  575262,  570418, 12, 22, 20, }, // Chile
    {  461,  501,   59360,  575284,  570461, 12, 22, 20, }, // China
    {  461,  507,       0,  575306,       0,  0, 22,  0, }, // Christmas
    {  461,  517,       0,  575328,       0,  0, 22,  0, }, // Cocos
    {  461,  523,  139712,  575350,  139754, 15, 25, 23, }, // Colombia
    {  461,  532,  575375,  575393,  575421, 18, 28, 32, }, // Cook
    {  461,  537,   59635,  575453,  139889, 11, 21, 19, }, // Cuba
    {  461,  601,       0,  575474,       0,  0, 21,  0, }, // East_Timor
    {  461,  612,  575495,  575518,  575551, 23, 33, 31, }, // Easter
    {  461,  627,  575582,  575602,  575632, 20, 30, 28, }, // Europe_Central
    {  461,  642,  570867,  575660,  570920, 21, 31, 29, }, // Europe_Eastern
    {  461,  657,       0,  575691,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  461,  680,  575723,  575746,  575779, 23, 33, 31, }, // Europe_Western
    {  461,  695,  575810,  575832,  575864, 22, 32, 30, }, // Falkland
    {  461,  709,       0,  575894,       0,  0, 25,  0, }, // French_Guiana
    {  461,  723,       0,  575919,       0,  0, 47,  0, }, // French_Southern
    {  461,  746,       0,   60356,       0,  0, 16,  0, }, // Galapagos
    {  461,  764,  140569,  575966,  140609, 14, 24, 22, }, // Georgia
    {  461,  772,       0,  575990,       0,  0, 21,  0, }, // Gilbert_Islands
    {  461,  788,       0,  576011,       0,  0, 30,  0, }, // GMT
    {  461,  830,  576041,  576068,  576105, 27, 37, 35, }, // Greenland_Eastern
    {  461,  848,  576140,  576169,  576208, 29, 39, 37, }, // Greenland_Western
    {  461,  876,       0,  140991,       0,  0, 13,  0, }, // Guyana
    {  461,  883,  571555,  576245,  571612, 23, 33, 31, }, // Hawaii_Aleutian
    {  461,  899,  141073,  576278,  141117, 16, 26, 24, }, // Hong_Kong
    {  461,  909,   60913,  576304,  571692, 11, 21, 19, }, // Hovd
    {  461,  914,       0,  576325,       0,  0, 24,  0, }, // India
    {  461,  920,       0,  576349,       0,  0, 19,  0, }, // Indian_Ocean
    {  461,  943,       0,  576368,       0,  0, 24,  0, }, // Indonesia_Central
    {  461,  961,       0,  576392,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  461,  979,       0,  576417,       0,  0, 27,  0, }, // Indonesia_Western
    {  461,  997,  571835,  576444,  571866, 10, 20, 18, }, // Iran
    {  461, 1008,   61156,  576464,  141411, 13, 23, 21, }, // Irkutsk
    {  461, 1016,  571908,  576487,  571943, 12, 22, 20, }, // Israel
    {  461, 1023,  368993,  576509,  571986, 12, 22, 20, }, // Japan
    {  461, 1058,       0,  576531,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  461, 1077,       0,  576557,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  461, 1106,  572062,  576585,  572097, 12, 22, 20, }, // Korea
    {  461, 1119,   61571,  576607,  572146, 18, 28, 26, }, // Krasnoyarsk
    {  461, 1175,       0,  576635,       0,  0, 29,  0, }, // Line_Islands
    {  461, 1204,  141987,  576664,  142027, 14, 24, 22, }, // Magadan
    {  461, 1237,       0,  576688,       0,  0, 23,  0, }, // Marquesas
    {  461, 1247,       0,  576711,       0,  0, 22,  0, }, // Marshall_Islands
    {  461, 1264,   61973,  576733,  142137, 14, 24, 22, }, // Mauritius
    {  461, 1281,  576757,  576782,  576817, 25, 35, 33, }, // Mexico_Pacific
    {  461, 1296,  369600,  576850,  572504, 17, 27, 25, }, // Mongolia
    {  461, 1305,  572529,  576877,  572566, 13, 23, 21, }, // Moscow
    {  461, 1326,       0,  576900,       0,  0, 14,  0, }, // Nepal
    {  461, 1358,  572743,  576914,  572788, 17, 27, 25, }, // Newfoundland
    {  461, 1376,  576941,  576962,  576993, 21, 31, 29, }, // Norfolk
    {  461, 1384,   62668,  577022,  142737, 26, 36, 34, }, // Noronha
    {  461, 1406,   62804,  577058,  572961, 18, 28, 26, }, // Novosibirsk
    {  461, 1418,   62879,  577086,  142871, 10, 20, 18, }, // Omsk
    {  461, 1428,  577106,  577123,  577150, 17, 27, 25, }, // Pakistan
    {  461, 1460,  370271,  577175,  573120, 15, 25, 23, }, // Paraguay
    {  461, 1469,  573143,  577200,  577221, 11, 21, 18, }, // Peru
    {  461, 1474,   63167,  577239,  573222, 16, 26, 24, }, // Philippines
    {  461, 1486,       0,  577265,       0,  0, 19,  0, }, // Phoenix_Islands
    {  461, 1502,  573268,  577284,  573335, 28, 38, 36, }, // Pierre_Miquelon
    {  461, 1554,       0,  577322,       0,  0, 14,  0, }, // Reunion
    {  461, 1570,  573388,  577336,  573427, 14, 24, 22, }, // Sakhalin
    {  461, 1634,       0,  577360,       0,  0, 21,  0, }, // Solomon
    {  461, 1642,       0,  577381,       0,  0, 23,  0, }, // South_Georgia
    {  461, 1689,  143604,  577404,  143642, 13, 23, 21, }, // Taipei
    {  461, 1750,  370937,  577427,  573603, 19, 29, 27, }, // Turkmenistan
    {  461, 1777,  577456,  577469,  577492, 13, 23, 21, }, // Uruguay
    {  461, 1792,  371074,  577513,  573725, 17, 27, 25, }, // Uzbekistan
    {  461, 1821,   64319,  577540,  144062, 18, 28, 26, }, // Vladivostok
    {  461, 1833,  144089,  577568,  144133, 16, 26, 24, }, // Volgograd
    {  461, 1850,       0,  577594,       0,  0, 18,  0, }, // Wake
    {  461, 1862,   64521,  577612,  573895, 14, 24, 22, }, // Yakutsk
    {  461, 1870,  573917,  577636,  573964, 18, 28, 26, }, // Yekaterinburg
    {  462,    0,  577664,  577671,  577683,  7, 12, 15, }, // Acre Odia/Odia/India
    {  462,    5,       0,  577698,       0,  0, 15,  0, }, // Afghanistan
    {  462,   17,       0,  577713,       0,  0, 16,  0, }, // Africa_Central
    {  462,   32,       0,  577729,       0,  0, 17,  0, }, // Africa_Eastern
    {  462,   65,       0,  577746,       0,  0, 26,  0, }, // Africa_Southern
    {  462,   81,  577772,  577790,  577816, 18, 26, 25, }, // Africa_Western
    {  462,  107,  577841,  577852,  577871, 11, 19, 19, }, // Alaska
    {  462,  128,  577890,  577900,  577915, 10, 15, 18, }, // Almaty
    {  462,  135,  577933,  577943,  577961, 10, 18, 23, }, // Amazon
    {  462,  142,  577984,  577997,  578018, 13, 21, 21, }, // America_Central
    {  462,  158,  578039,  578053,  578075, 14, 22, 22, }, // America_Eastern
    {  462,  174,  578097,  578109,  578129, 12, 20, 20, }, // America_Mountain
    {  462,  191,  578149,  578161,  578181, 12, 20, 20, }, // America_Pacific
    {  462,  207,  578201,  578212,  578228, 11, 16, 19, }, // Anadyr
    {  462,  214,  578247,  578255,  578271,  8, 16, 16, }, // Apia
    {  462,  219,  578287,  578295,  578308,  8, 13, 16, }, // Aqtau
    {  462,  225,  578324,  578334,  578349, 10, 15, 18, }, // Aqtobe
    {  462,  232,  578367,  578376,  578393,  9, 17, 17, }, // Arabian
    {  462,  240,  578410,  578425,  578448, 15, 23, 26, }, // Argentina
    {  462,  250,  578474,  578496,  578526, 22, 30, 33, }, // Argentina_Western
    {  462,  268,  578559,  578571,  578591, 12, 20, 23, }, // Armenia
    {  462,  286,  578614,  578628,  578650, 14, 22, 22, }, // Atlantic
    {  462,  295,  578672,  578691,  578718, 19, 27, 27, }, // Australia_Central
    {  462,  313,  578745,  578771,  578805, 26, 34, 34, }, // Australia_CentralWestern
    {  462,  338,  578839,  578859,  578887, 20, 28, 28, }, // Australia_Eastern
    {  462,  356,  578915,  578936,  578965, 21, 29, 29, }, // Australia_Western
    {  462,  374,  578994,  579008,  579029, 14, 21, 21, }, // Azerbaijan
    {  462,  385,  579050,  579061,  579077, 11, 16, 22, }, // Azores
    {  462,  397,  579099,  579111,  579131, 12, 20, 25, }, // Bangladesh
    {  462,  415,       0,  579156,       0,  0,  9,  0, }, // Bhutan
    {  462,  422,       0,  579165,       0,  0, 10,  0, }, // Bolivia
    {  462,  437,  579175,  579188,  579209, 13, 21, 24, }, // Brasilia
    {  462,  454,       0,  579233,       0,  0, 21,  0, }, // Brunei
    {  462,  461,  579254,  579269,  579292, 15, 23, 28, }, // Cape_Verde
    {  462,  478,       0,  579320,       0,  0, 15,  0, }, // Chamorro
    {  462,  487,  579335,  579346,  579365, 11, 19, 19, }, // Chatham
    {  462,  495,  579384,  579392,  579408,  8, 16, 19, }, // Chile
    {  462,  501,  579427,  579434,  579446,  7, 12, 15, }, // China
    {  462,  507,       0,  579461,       0,  0, 20,  0, }, // Christmas
    {  462,  517,       0,  579481,       0,  0, 22,  0, }, // Cocos
    {  462,  523,  579503,  579514,  579530, 11, 16, 22, }, // Colombia
    {  462,  532,  579552,  579572,  579597, 20, 25, 38, }, // Cook
    {  462,  537,  579635,  579645,  579663, 10, 18, 18, }, // Cuba
    {  462,  548,       0,  579681,       0,  0, 11,  0, }, // Davis
    {  462,  564,       0,  579692,       0,  0, 23,  0, }, // DumontDUrville
    {  462,  601,       0,  579715,       0,  0, 17,  0, }, // East_Timor
    {  462,  612,  579732,  579753,  579782, 21, 29, 32, }, // Easter
    {  462,  619,       0,  579814,       0,  0, 11,  0, }, // Ecuador
    {  462,  627,  579825,  579846,  579870, 21, 24, 32, }, // Europe_Central
    {  462,  642,  579902,  579924,  579954, 22, 30, 33, }, // Europe_Eastern
    {  462,  657,       0,  579987,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  462,  680,  580019,  580042,  580073, 23, 31, 34, }, // Europe_Western
    {  462,  695,  580107,  580131,  580160, 24, 29, 35, }, // Falkland
    {  462,  704,  580195,  580203,  580216,  8, 13, 21, }, // Fiji
    {  462,  709,       0,  580237,       0,  0, 17,  0, }, // French_Guiana
    {  462,  723,       0,  580254,       0,  0, 37,  0, }, // French_Southern
    {  462,  746,       0,  580291,       0,  0, 14,  0, }, // Galapagos
    {  462,  756,       0,  580305,       0,  0, 13,  0, }, // Gambier
    {  462,  764,  580318,  580328,  580346, 10, 18, 17, }, // Georgia
    {  462,  772,       0,  580363,       0,  0, 24,  0, }, // Gilbert_Islands
    {  462,  788,       0,  580387,       0,  0, 18,  0, }, // GMT
    {  462,  830,  580405,  580428,  580459, 23, 31, 34, }, // Greenland_Eastern
    {  462,  848,  580493,  580517,  580549, 24, 32, 32, }, // Greenland_Western
    {  462,  871,       0,  580581,       0,  0, 16,  0, }, // Gulf
    {  462,  876,       0,  580597,       0,  0,  9,  0, }, // Guyana
    {  462,  883,  580606,  580622,  580646, 16, 24, 24, }, // Hawaii_Aleutian
    {  462,  899,  580670,  580679,  580693,  9, 14, 20, }, // Hong_Kong
    {  462,  909,  580713,  580723,  580738, 10, 15, 21, }, // Hovd
    {  462,  914,       0,  580759,       0,  0, 15,  0, }, // India
    {  462,  920,       0,  580774,       0,  0, 17,  0, }, // Indian_Ocean
    {  462,  933,       0,  580791,       0,  0, 14,  0, }, // Indochina
    {  462,  943,       0,  580805,       0,  0, 19,  0, }, // Indonesia_Central
    {  462,  961,       0,  580824,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  462,  979,       0,  580844,       0,  0, 21,  0, }, // Indonesia_Western
    {  462,  997,  580865,  580873,  580886,  8, 13, 16, }, // Iran
    {  462, 1008,  580902,  580915,  580934, 13, 19, 27, }, // Irkutsk
    {  462, 1016,  580961,  580972,  580991, 11, 19, 19, }, // Israel
    {  462, 1023,  581010,  581019,  581033,  9, 14, 17, }, // Japan
    {  462, 1029,  581050,  581078,  581111, 28, 33, 36, }, // Kamchatka
    {  462, 1047,       0,  581147,       0,  0, 15,  0, }, // Kazakhstan
    {  462, 1058,       0,  581162,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  462, 1077,       0,  581183,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  462, 1106,  581204,  581213,  581227,  9, 14, 17, }, // Korea
    {  462, 1112,       0,  581244,       0,  0, 10,  0, }, // Kosrae
    {  462, 1119,  581254,  581271,  581296, 17, 25, 28, }, // Krasnoyarsk
    {  462, 1151,       0,  581324,       0,  0, 15,  0, }, // Kyrgystan
    {  462, 1161,       0,  581339,       0,  0,  9,  0, }, // Lanka
    {  462, 1175,       0,  581348,       0,  0, 21,  0, }, // Line_Islands
    {  462, 1188,  581369,  581382,  581403, 13, 21, 21, }, // Lord_Howe
    {  462, 1198,  581424,  581433,  581446,  9, 13, 16, }, // Macau
    {  462, 1204,  581462,  581474,  581494, 12, 20, 23, }, // Magadan
    {  462, 1219,       0,  581517,       0,  0, 11,  0, }, // Malaysia
    {  462, 1228,       0,  581528,       0,  0, 12,  0, }, // Maldives
    {  462, 1237,       0,  581540,       0,  0, 17,  0, }, // Marquesas
    {  462, 1247,       0,  581557,       0,  0, 24,  0, }, // Marshall_Islands
    {  462, 1264,  581581,  581593,  581613, 12, 20, 19, }, // Mauritius
    {  462, 1274,       0,  581632,       0,  0, 11,  0, }, // Mawson
    {  462, 1281,  581643,  581666,  581697, 23, 31, 31, }, // Mexico_Pacific
    {  462, 1296,  581728,  581744,  581765, 16, 21, 29, }, // Mongolia
    {  462, 1305,  581794,  581803,  581820,  9, 17, 20, }, // Moscow
    {  462, 1312,       0,  581840,       0,  0, 13,  0, }, // Myanmar
    {  462, 1320,       0,  581853,       0,  0,  9,  0, }, // Nauru
    {  462, 1326,       0,  581862,       0,  0,  9,  0, }, // Nepal
    {  462, 1332,  581871,  581889,  581915, 18, 26, 29, }, // New_Caledonia
    {  462, 1346,  581944,  581959,  581982, 15, 23, 23, }, // New_Zealand
    {  462, 1358,  582005,  582027,  582057, 22, 30, 30, }, // Newfoundland
    {  462, 1371,       0,  582087,       0,  0,  8,  0, }, // Niue
    {  462, 1376,  582095,  582112,  582137, 17, 25, 28, }, // Norfolk
    {  462, 1384,  582165,  582192,  582227, 27, 35, 38, }, // Noronha
    {  462, 1392,       0,  582265,       0,  0, 27,  0, }, // North_Mariana
    {  462, 1406,  582292,  582308,  582332, 16, 24, 27, }, // Novosibirsk
    {  462, 1418,  582359,  582368,  582385,  9, 17, 20, }, // Omsk
    {  462, 1428,  582405,  582418,  582436, 13, 18, 24, }, // Pakistan
    {  462, 1437,       0,  582460,       0,  0,  9,  0, }, // Palau
    {  462, 1443,       0,  582469,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  462, 1460,  582487,  582498,  582517, 11, 19, 22, }, // Paraguay
    {  462, 1469,  582539,  582547,  582563,  8, 16, 19, }, // Peru
    {  462, 1474,  582582,  582596,  582618, 14, 22, 25, }, // Philippines
    {  462, 1486,       0,  582643,       0,  0, 22,  0, }, // Phoenix_Islands
    {  462, 1502,  582665,  582696,  582735, 31, 39, 39, }, // Pierre_Miquelon
    {  462, 1518,       0,  582774,       0,  0, 14,  0, }, // Pitcairn
    {  462, 1527,       0,  582788,       0,  0, 10,  0, }, // Ponape
    {  462, 1534,       0,  582798,       0,  0, 15,  0, }, // Pyongyang
    {  462, 1544,  582813,  582827,  582846, 14, 19, 22, }, // Qyzylorda
    {  462, 1554,       0,  582868,       0,  0, 14,  0, }, // Reunion
    {  462, 1562,       0,  582882,       0,  0, 10,  0, }, // Rothera
    {  462, 1570,  582892,  582903,  582922, 11, 19, 22, }, // Sakhalin
    {  462, 1579,  582944,  582953,  582967,  9, 14, 17, }, // Samara
    {  462, 1596,  582984,  582993,  583007,  9, 14, 17, }, // Samoa
    {  462, 1602,       0,  583024,       0,  0, 15,  0, }, // Seychelles
    {  462, 1624,       0,  583039,       0,  0, 20,  0, }, // Singapore
    {  462, 1634,       0,  583059,       0,  0, 21,  0, }, // Solomon
    {  462, 1642,       0,  583080,       0,  0, 17,  0, }, // South_Georgia
    {  462, 1656,       0,  583097,       0,  0, 13,  0, }, // Suriname
    {  462, 1676,       0,  583110,       0,  0,  8,  0, }, // Syowa
    {  462, 1682,       0,  583118,       0,  0, 10,  0, }, // Tahiti
    {  462, 1689,  583128,  583138,  583153, 10, 15, 18, }, // Taipei
    {  462, 1696,       0,  583171,       0,  0, 15,  0, }, // Tajikistan
    {  462, 1724,       0,  583186,       0,  0, 11,  0, }, // Tokelau
    {  462, 1732,  583197,  583207,  583222, 10, 15, 23, }, // Tonga
    {  462, 1738,       0,  583245,       0,  0,  9,  0, }, // Truk
    {  462, 1750,  583254,  583272,  583298, 18, 26, 25, }, // Turkmenistan
    {  462, 1763,       0,  583323,       0,  0, 10,  0, }, // Tuvalu
    {  462, 1777,  583333,  583343,  583361, 10, 18, 21, }, // Uruguay
    {  462, 1792,  583382,  583397,  583417, 15, 20, 28, }, // Uzbekistan
    {  462, 1803,  583445,  583456,  583472, 11, 16, 24, }, // Vanuatu
    {  462, 1811,       0,  583496,       0,  0, 13,  0, }, // Venezuela
    {  462, 1821,  583509,  583527,  583553, 18, 26, 29, }, // Vladivostok
    {  462, 1833,  583582,  583597,  583620, 15, 23, 26, }, // Volgograd
    {  462, 1843,       0,  583646,       0,  0, 13,  0, }, // Vostok
    {  462, 1850,       0,  583659,       0,  0, 15,  0, }, // Wake
    {  462, 1855,       0,  583674,       0,  0, 22,  0, }, // Wallis
    {  462, 1862,  583696,  583708,  583728, 12, 20, 23, }, // Yakutsk
    {  462, 1870,  583751,  583769,  583795, 18, 26, 29, }, // Yekaterinburg
    {  462, 1892,       0,  583824,       0,  0, 10,  0, }, // Yukon
    {  463,    5,       0,  583834,       0,  0, 21,  0, }, // Afghanistan Oromo/Latin/Ethiopia
    {  463,   17,       0,  583855,       0,  0, 32,  0, }, // Africa_Central
    {  463,   32,       0,  583887,       0,  0, 21,  0, }, // Africa_Eastern
    {  463,   65,       0,  583908,       0,  0, 37,  0, }, // Africa_Southern
    {  463,   81,  583945,  583968,  584005, 23, 37, 29, }, // Africa_Western
    {  463,  107,  584034,  584051,  584082, 17, 31, 24, }, // Alaska
    {  463,  135,  584106,  584122,  584152, 16, 30, 22, }, // Amazon
    {  463,  142,  584174,  584198,  584236, 24, 38, 31, }, // America_Central
    {  463,  158,  584267,  584281,  584309, 14, 28, 21, }, // America_Eastern
    {  463,  174,  584330,  584349,  584382, 19, 33, 26, }, // America_Mountain
    {  463,  191,  584408,  584425,  584456, 17, 31, 24, }, // America_Pacific
    {  463,  214,  584480,  584493,  584520, 13, 27, 20, }, // Apia
    {  463,  232,  584540,  584555,  584584, 15, 29, 22, }, // Arabian
    {  463,  240,  584606,  584626,  584660, 20, 34, 26, }, // Argentina
    {  463,  250,  584686,  584713,  584754, 27, 41, 33, }, // Argentina_Western
    {  463,  268,  584787,  584806,  584839, 19, 33, 25, }, // Armenia
    {  463,  286,  584864,  584884,  584918, 20, 34, 27, }, // Atlantic
    {  463,  295,  584945,  584984,  585037, 39, 53, 46, }, // Australia_Central
    {  463,  313,  585083,  585128,  585187, 45, 59, 52, }, // Australia_CentralWestern
    {  463,  338,  585239,  585268,  585311, 29, 43, 36, }, // Australia_Eastern
    {  463,  356,  585347,  585377,  585450, 30, 73, 38, }, // Australia_Western
    {  463,  374,  585488,  585510,  585546, 22, 36, 28, }, // Azerbaijan
    {  463,  385,  585574,  585591,  585622, 17, 31, 23, }, // Azores
    {  463,  397,  585645,  585667,  585703, 22, 36, 28, }, // Bangladesh
    {  463,  415,       0,  585731,       0,  0, 17,  0, }, // Bhutan
    {  463,  422,       0,  585748,       0,  0, 19,  0, }, // Bolivia
    {  463,  437,  585767,  585788,  585823, 21, 35, 27, }, // Brasilia
    {  463,  454,       0,  585850,       0,  0, 27,  0, }, // Brunei
    {  463,  461,  585877,  585898,  585933, 21, 35, 27, }, // Cape_Verde
    {  463,  478,       0,  585960,       0,  0, 30,  0, }, // Chamorro
    {  463,  487,  585990,  586006,  586036, 16, 30, 23, }, // Chatham
    {  463,  495,  586059,  586075,  586105, 16, 30, 22, }, // Chile
    {  463,  501,  586127,  586145,  586177, 18, 32, 25, }, // China
    {  463,  507,       0,  586202,       0,  0, 24,  0, }, // Christmas
    {  463,  517,       0,  586226,       0,  0, 23,  0, }, // Cocos
    {  463,  523,  586249,  586268,  586301, 19, 33, 25, }, // Colombia
    {  463,  532,  586326,  586348,  586384, 22, 36, 36, }, // Cook
    {  463,  537,  586420,  586435,  586464, 15, 29, 22, }, // Cuba
    {  463,  548,       0,  586486,       0,  0, 16,  0, }, // Davis
    {  463,  564,       0,  586502,       0,  0, 25,  0, }, // DumontDUrville
    {  463,  601,       0,  586527,       0,  0, 22,  0, }, // East_Timor
    {  463,  612,  586549,  586569,  586603, 20, 34, 26, }, // Easter
    {  463,  619,       0,  586629,       0,  0, 20,  0, }, // Ecuador
    {  463,  627,  586649,  586684,  586733, 35, 49, 41, }, // Europe_Central
    {  463,  642,  586774,  586798,  586837, 24, 39, 31, }, // Europe_Eastern
    {  463,  657,       0,  586868,       0,  0, 35,  0, }, // Europe_Further_Eastern
    {  463,  680,  586903,  586929,  586969, 26, 40, 32, }, // Europe_Western
    {  463,  695,  587001,  587029,  587071, 28, 42, 34, }, // Falkland
    {  463,  704,  587105,  587120,  587149, 15, 29, 21, }, // Fiji
    {  463,  709,       0,  587170,       0,  0, 25,  0, }, // French_Guiana
    {  463,  723,       0,  587195,       0,  0, 39,  0, }, // French_Southern
    {  463,  746,       0,  587234,       0,  0, 20,  0, }, // Galapagos
    {  463,  756,       0,  587254,       0,  0, 19,  0, }, // Gambier
    {  463,  764,  587273,  587291,  587323, 18, 32, 24, }, // Georgia
    {  463,  772,       0,  587347,       0,  0, 26,  0, }, // Gilbert_Islands
    {  463,  788,       0,  587373,       0,  0, 36,  0, }, // GMT
    {  463,  830,  587409,  587435,  587475, 26, 40, 32, }, // Greenland_Eastern
    {  463,  848,  587507,  587534,  587575, 27, 41, 33, }, // Greenland_Western
    {  463,  866,       0,  587608,       0,  0, 29,  0, }, // Guam
    {  463,  871,       0,  587637,       0,  0, 30,  0, }, // Gulf
    {  463,  876,       0,  587667,       0,  0, 17,  0, }, // Guyana
    {  463,  883,  587684,  587711,  587752, 27, 41, 34, }, // Hawaii_Aleutian
    {  463,  899,  587786,  587806,  587840, 20, 34, 26, }, // Hong_Kong
    {  463,  909,  587866,  587880,  587908, 14, 28, 20, }, // Hovd
    {  463,  914,       0,  587928,       0,  0, 29,  0, }, // India
    {  463,  920,       0,  587957,       0,  0, 23,  0, }, // Indian_Ocean
    {  463,  933,       0,  587980,       0,  0, 23,  0, }, // Indochina
    {  463,  943,       0,  588003,       0,  0, 39,  0, }, // Indonesia_Central
    {  463,  961,       0,  588042,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  463,  979,       0,  588070,       0,  0, 30,  0, }, // Indonesia_Western
    {  463,  997,  588100,  588114,  588142, 14, 28, 21, }, // Iran
    {  463, 1008,  588163,  588179,  588209, 16, 30, 22, }, // Irkutsk
    {  463, 1016,  588231,  588249,  588281, 18, 32, 25, }, // Israel
    {  463, 1023,  588306,  588323,  588354, 17, 31, 24, }, // Japan
    {  463, 1047,       0,  588378,       0,  0, 22,  0, }, // Kazakhstan
    {  463, 1058,       0,  588400,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  463, 1077,       0,  588428,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  463, 1106,  588457,  588474,  588505, 17, 31, 24, }, // Korea
    {  463, 1112,       0,  588529,       0,  0, 18,  0, }, // Kosrae
    {  463, 1119,  588547,  588567,  588601, 20, 34, 26, }, // Krasnoyarsk
    {  463, 1151,       0,  588627,       0,  0, 23,  0, }, // Kyrgystan
    {  463, 1161,       0,  588650,       0,  0, 16,  0, }, // Lanka
    {  463, 1175,       0,  588666,       0,  0, 22,  0, }, // Line_Islands
    {  463, 1188,  588688,  588706,  588738, 18, 32, 25, }, // Lord_Howe
    {  463, 1204,  588763,  588779,  588809, 16, 30, 22, }, // Magadan
    {  463, 1219,       0,  588831,       0,  0, 21,  0, }, // Malaysia
    {  463, 1228,       0,  588852,       0,  0, 18,  0, }, // Maldives
    {  463, 1237,       0,  588870,       0,  0, 18,  0, }, // Marquesas
    {  463, 1247,       0,  588888,       0,  0, 27,  0, }, // Marshall_Islands
    {  463, 1264,  588915,  588937,  588972, 22, 35, 27, }, // Mauritius
    {  463, 1274,       0,  588999,       0,  0, 15,  0, }, // Mawson
    {  463, 1281,  589014,  589042,  589084, 28, 42, 35, }, // Mexico_Pacific
    {  463, 1296,  589119,  589139,  589173, 20, 34, 26, }, // Mongolia
    {  463, 1305,  589199,  589215,  589245, 16, 30, 22, }, // Moscow
    {  463, 1312,       0,  589267,       0,  0, 21,  0, }, // Myanmar
    {  463, 1320,       0,  589288,       0,  0, 17,  0, }, // Nauru
    {  463, 1326,       0,  589305,       0,  0, 17,  0, }, // Nepal
    {  463, 1332,  589322,  589351,  589394, 29, 43, 35, }, // New_Caledonia
    {  463, 1346,  589429,  589449,  589483, 20, 34, 27, }, // New_Zealand
    {  463, 1358,  589510,  589535,  589574, 25, 39, 32, }, // Newfoundland
    {  463, 1371,       0,  589606,       0,  0, 13,  0, }, // Niue
    {  463, 1376,  589619,  589642,  589679, 23, 37, 30, }, // Norfolk
    {  463, 1384,  589709,  589737,  589779, 28, 42, 34, }, // Noronha
    {  463, 1392,       0,  589813,       0,  0, 36,  0, }, // North_Mariana
    {  463, 1406,  589849,  589870,  589905, 21, 35, 27, }, // Novosibirsk
    {  463, 1418,  589932,  589945,  589972, 13, 27, 19, }, // Omsk
    {  463, 1428,  589991,  590010,  590043, 19, 33, 25, }, // Pakistan
    {  463, 1437,       0,  590068,       0,  0, 15,  0, }, // Palau
    {  463, 1443,       0,  590083,       0,  0, 31,  0, }, // Papua_New_Guinea
    {  463, 1460,  590114,  590136,  590172, 22, 36, 28, }, // Paraguay
    {  463, 1469,  590200,  590215,  590244, 15, 29, 21, }, // Peru
    {  463, 1474,  590265,  590284,  590317, 19, 33, 25, }, // Philippines
    {  463, 1486,       0,  590342,       0,  0, 26,  0, }, // Phoenix_Islands
    {  463, 1502,  590368,  590401,  590448, 33, 47, 40, }, // Pierre_Miquelon
    {  463, 1518,       0,  590488,       0,  0, 17,  0, }, // Pitcairn
    {  463, 1527,       0,  590505,       0,  0, 15,  0, }, // Ponape
    {  463, 1534,       0,  590520,       0,  0, 21,  0, }, // Pyongyang
    {  463, 1554,       0,  590541,       0,  0, 21,  0, }, // Reunion
    {  463, 1562,       0,  590562,       0,  0, 16,  0, }, // Rothera
    {  463, 1570,  590578,  590595,  590626, 17, 31, 23, }, // Sakhalin
    {  463, 1596,  590649,  590666,  590697, 17, 31, 24, }, // Samoa
    {  463, 1602,       0,  590721,       0,  0, 18,  0, }, // Seychelles
    {  463, 1624,       0,  590739,       0,  0, 33,  0, }, // Singapore
    {  463, 1634,       0,  590772,       0,  0, 26,  0, }, // Solomon
    {  463, 1642,       0,  590798,       0,  0, 25,  0, }, // South_Georgia
    {  463, 1656,       0,  590823,       0,  0, 18,  0, }, // Suriname
    {  463, 1676,       0,  590841,       0,  0, 14,  0, }, // Syowa
    {  463, 1682,       0,  590855,       0,  0, 15,  0, }, // Tahiti
    {  463, 1689,  590870,  590887,  590918, 17, 31, 24, }, // Taipei
    {  463, 1696,       0,  590942,       0,  0, 20,  0, }, // Tajikistan
    {  463, 1724,       0,  590962,       0,  0, 17,  0, }, // Tokelau
    {  463, 1732,  590979,  590993,  591021, 14, 28, 20, }, // Tonga
    {  463, 1738,       0,  591041,       0,  0, 14,  0, }, // Truk
    {  463, 1750,  591055,  591078,  591115, 23, 37, 29, }, // Turkmenistan
    {  463, 1763,       0,  591144,       0,  0, 15,  0, }, // Tuvalu
    {  463, 1777,  591159,  591178,  591211, 19, 33, 25, }, // Uruguay
    {  463, 1792,  591236,  591257,  591292, 21, 35, 27, }, // Uzbekistan
    {  463, 1803,  591319,  591336,  591367, 17, 31, 23, }, // Vanuatu
    {  463, 1811,       0,  591390,       0,  0, 21,  0, }, // Venezuela
    {  463, 1821,  591411,  591431,  591465, 20, 34, 26, }, // Vladivostok
    {  463, 1833,  591491,  591509,  591541, 18, 32, 24, }, // Volgograd
    {  463, 1843,       0,  591565,       0,  0, 15,  0, }, // Vostok
    {  463, 1850,       0,  591580,       0,  0, 19,  0, }, // Wake
    {  463, 1855,       0,  591599,       0,  0, 25,  0, }, // Wallis
    {  463, 1862,  591624,  591640,  591669, 16, 29, 22, }, // Yakutsk
    {  463, 1870,  591691,  591713,  591749, 22, 36, 28, }, // Yekaterinburg
    {  463, 1892,       0,  591777,       0,  0, 16,  0, }, // Yukon
    {  467,  627,  591793,  591819,  591856, 26, 37, 35, }, // Europe_Central Ossetic/Cyrillic/Russia
    {  467,  642,  591891,  591914,  591948, 23, 34, 32, }, // Europe_Eastern
    {  467,  680,  591980,  592006,  592043, 26, 37, 35, }, // Europe_Western
    {  467,  764,  592078,  592097,  592127, 19, 30, 28, }, // Georgia
    {  467,  788,       0,  592155,       0,  0, 25,  0, }, // GMT
    {  467, 1305,  592180,  592195,  592221, 15, 26, 24, }, // Moscow
    {  468,   17,       0,  396238,       0,  0, 21,  0, }, // Africa_Central Papiamento/Latin/Curacao
    {  468,   32,       0,  592245,       0,  0, 17,  0, }, // Africa_Eastern
    {  468,  385,  592262,  592275,  592295, 13, 20, 32, }, // Azores
    {  468,  627,  397232,  592327,  592355, 21, 28, 40, }, // Europe_Central
    {  468,  642,  592395,  592412,  592436, 17, 24, 36, }, // Europe_Eastern
    {  468,  680,  592472,  592490,  592515, 18, 25, 37, }, // Europe_Western
    {  468,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {  470,    5,       0,  592552,       0,  0, 13,  0, }, // Afghanistan Pashto/Arabic/Afghanistan
    {  470,   17,       0,  592565,       0,  0, 16,  0, }, // Africa_Central
    {  470,   32,       0,  592581,       0,  0, 15,  0, }, // Africa_Eastern
    {  470,   65,       0,  592596,       0,  0, 23,  0, }, // Africa_Southern
    {  470,   81,  592619,  592636,  592662, 17, 26, 26, }, // Africa_Western
    {  470,  107,  592688,  592698,  592715, 10, 17, 24, }, // Alaska
    {  470,  135,  592739,  592750,  592768, 11, 18, 16, }, // Amazon
    {  470,  142,  592784,  592793,  592809,  9, 16, 23, }, // America_Central
    {  470,  158,  592832,  592840,  592855,  8, 15, 19, }, // America_Eastern
    {  470,  174,  592874,  592885,  592901, 11, 16, 20, }, // America_Mountain
    {  470,  191,  592921,  592930,  592948,  9, 18, 20, }, // America_Pacific
    {  470,  214,  592968,  592976,  592991,  8, 15, 22, }, // Apia
    {  470,  232,  593013,  593021,  593036,  8, 15, 22, }, // Arabian
    {  470,  240,  593058,  593070,  593089, 12, 19, 17, }, // Argentina
    {  470,  250,  593106,  593125,  593151, 19, 26, 24, }, // Argentina_Western
    {  470,  268,  593175,  593187,  593206, 12, 19, 17, }, // Armenia
    {  470,  286,  593223,  593235,  593254, 12, 19, 23, }, // Atlantic
    {  470,  295,  593277,  593295,  593320, 18, 25, 32, }, // Australia_Central
    {  470,  313,  593352,  593377,  593409, 25, 32, 39, }, // Australia_CentralWestern
    {  470,  338,  593448,  593465,  593489, 17, 24, 31, }, // Australia_Eastern
    {  470,  356,  593520,  593539,  593565, 19, 26, 33, }, // Australia_Western
    {  470,  374,  593598,  593613,  593633, 15, 20, 19, }, // Azerbaijan
    {  470,  385,  593652,  593662,  593679, 10, 17, 15, }, // Azores
    {  470,  397,  593694,  593707,  593726, 13, 19, 18, }, // Bangladesh
    {  470,  415,       0,  593744,       0,  0, 10,  0, }, // Bhutan
    {  470,  422,       0,  593754,       0,  0, 11,  0, }, // Bolivia
    {  470,  437,  593765,  593775,  593792, 10, 17, 15, }, // Brasilia
    {  470,  454,       0,  593807,       0,  0, 21,  0, }, // Brunei
    {  470,  461,  593828,  593840,  593859, 12, 19, 16, }, // Cape_Verde
    {  470,  478,       0,  593875,       0,  0, 16,  0, }, // Chamorro
    {  470,  487,  593891,  593900,  593916,  9, 16, 23, }, // Chatham
    {  470,  495,  593939,  593946,  593960,  7, 14, 12, }, // Chile
    {  470,  501,  593972,  593979,  593993,  7, 14, 20, }, // China
    {  470,  507,       0,  594013,       0,  0, 15,  0, }, // Christmas
    {  470,  517,       0,  594028,       0,  0, 18,  0, }, // Cocos
    {  470,  523,  594046,  594057,  594075, 11, 18, 16, }, // Colombia
    {  470,  532,  594091,  594107,  594130, 16, 23, 25, }, // Cook
    {  470,  537,  594155,  594164,  594180,  9, 16, 20, }, // Cuba
    {  470,  548,       0,  594200,       0,  0,  8,  0, }, // Davis
    {  470,  564,       0,  594208,       0,  0, 14,  0, }, // DumontDUrville
    {  470,  601,       0,  594222,       0,  0, 14,  0, }, // East_Timor
    {  470,  612,  594236,  594250,  594271, 14, 21, 19, }, // Easter
    {  470,  619,       0,  594290,       0,  0, 13,  0, }, // Ecuador
    {  470,  627,  594303,  594320,  594344, 17, 24, 21, }, // Europe_Central
    {  470,  642,  594365,  594381,  594404, 16, 23, 21, }, // Europe_Eastern
    {  470,  657,       0,  594425,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  470,  680,  594445,  594463,  594488, 18, 25, 23, }, // Europe_Western
    {  470,  695,  594511,  594526,  594550, 15, 24, 22, }, // Falkland
    {  470,  704,  594572,  594579,  594593,  7, 14, 14, }, // Fiji
    {  470,  709,       0,  594607,       0,  0, 18,  0, }, // French_Guiana
    {  470,  723,       0,  594625,       0,  0, 29,  0, }, // French_Southern
    {  470,  746,       0,  594654,       0,  0, 11,  0, }, // Galapagos
    {  470,  756,       0,  594665,       0,  0, 10,  0, }, // Gambier
    {  470,  764,  594675,  594685,  594702, 10, 17, 19, }, // Georgia
    {  470,  772,       0,  594721,       0,  0, 18,  0, }, // Gilbert_Islands
    {  470,  788,       0,  594739,       0,  0, 18,  0, }, // GMT
    {  470,  830,  594757,  594776,  594802, 19, 26, 24, }, // Greenland_Eastern
    {  470,  848,  594826,  594845,  594871, 19, 26, 24, }, // Greenland_Western
    {  470,  871,       0,  594895,       0,  0, 15,  0, }, // Gulf
    {  470,  876,       0,  594910,       0,  0, 13,  0, }, // Guyana
    {  470,  883,  594923,  594940,  594964, 17, 24, 26, }, // Hawaii_Aleutian
    {  470,  899,  594990,  595003,  595023, 13, 20, 18, }, // Hong_Kong
    {  470,  909,  595041,  595049,  595064,  8, 15, 13, }, // Hovd
    {  470,  914,       0,  595077,       0,  0, 14,  0, }, // India
    {  470,  920,       0,  595091,       0,  0, 15,  0, }, // Indian_Ocean
    {  470,  933,       0,  595106,       0,  0, 13,  0, }, // Indochina
    {  470,  943,       0,  595119,       0,  0, 18,  0, }, // Indonesia_Central
    {  470,  961,       0,  595137,       0,  0, 13,  0, }, // Indonesia_Eastern
    {  470,  979,       0,  595150,       0,  0, 20,  0, }, // Indonesia_Western
    {  470,  997,  595170,  595179,  595195,  9, 16, 23, }, // Iran
    {  470, 1008,  595218,  595229,  595247, 11, 18, 16, }, // Irkutsk
    {  470, 1016,  595263,  595274,  595292, 11, 18, 25, }, // Israel
    {  470, 1023,  595317,  595326,  595342,  9, 16, 23, }, // Japan
    {  470, 1047,       0,  595365,       0,  0, 12,  0, }, // Kazakhstan
    {  470, 1058,       0,  595377,       0,  0, 17,  0, }, // Kazakhstan_Eastern
    {  470, 1077,       0,  595394,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  470, 1106,  595413,  595424,  595442, 11, 18, 25, }, // Korea
    {  470, 1112,       0,  595467,       0,  0, 10,  0, }, // Kosrae
    {  470, 1119,  595477,  595492,  595514, 15, 22, 20, }, // Krasnoyarsk
    {  470, 1151,       0,  595534,       0,  0, 13,  0, }, // Kyrgystan
    {  470, 1175,       0,  595547,       0,  0, 17,  0, }, // Line_Islands
    {  470, 1188,  595564,  595576,  595595, 12, 19, 26, }, // Lord_Howe
    {  470, 1204,  595621,  595631,  595648, 10, 17, 15, }, // Magadan
    {  470, 1219,       0,  595663,       0,  0, 12,  0, }, // Malaysia
    {  470, 1228,       0,  595675,       0,  0, 10,  0, }, // Maldives
    {  470, 1237,       0,  595685,       0,  0, 10,  0, }, // Marquesas
    {  470, 1247,       0,  595695,       0,  0, 18,  0, }, // Marshall_Islands
    {  470, 1264,  595713,  595724,  595742, 11, 18, 18, }, // Mauritius
    {  470, 1274,       0,  595760,       0,  0,  9,  0, }, // Mawson
    {  470, 1281,  595769,  595785,  595808, 16, 23, 25, }, // Mexico_Pacific
    {  470, 1296,  595833,  595847,  595868, 14, 21, 19, }, // Mongolia
    {  470, 1305,  595887,  595896,  595912,  9, 16, 13, }, // Moscow
    {  470, 1312,       0,  595925,       0,  0, 11,  0, }, // Myanmar
    {  470, 1320,       0,  595936,       0,  0,  9,  0, }, // Nauru
    {  470, 1326,       0,  595945,       0,  0,  9,  0, }, // Nepal
    {  470, 1332,  595954,  595971,  595995, 17, 24, 21, }, // New_Caledonia
    {  470, 1346,  596016,  596030,  596051, 14, 21, 28, }, // New_Zealand
    {  470, 1358,  596079,  596096,  596121, 17, 25, 31, }, // Newfoundland
    {  470, 1371,       0,  596152,       0,  0,  8,  0, }, // Niue
    {  470, 1376,  596160,  596175,  596200, 15, 25, 23, }, // Norfolk
    {  470, 1384,  596223,  596245,  596274, 22, 29, 27, }, // Noronha
    {  470, 1406,  596301,  596314,  596334, 13, 20, 18, }, // Novosibirsk
    {  470, 1418,  596352,  596361,  596377,  9, 16, 14, }, // Omsk
    {  470, 1428,  596391,  596402,  596420, 11, 18, 16, }, // Pakistan
    {  470, 1437,       0,  596436,       0,  0,  9,  0, }, // Palau
    {  470, 1443,       0,  596445,       0,  0, 17,  0, }, // Papua_New_Guinea
    {  470, 1460,  596462,  596474,  596493, 12, 19, 17, }, // Paraguay
    {  470, 1469,  596510,  596518,  596533,  8, 15, 13, }, // Peru
    {  470, 1474,  596546,  596556,  596573, 10, 17, 15, }, // Philippines
    {  470, 1486,       0,  596588,       0,  0, 16,  0, }, // Phoenix_Islands
    {  470, 1502,  596604,  596627,  596657, 23, 30, 32, }, // Pierre_Miquelon
    {  470, 1518,       0,  596689,       0,  0, 13,  0, }, // Pitcairn
    {  470, 1527,       0,  596702,       0,  0,  9,  0, }, // Ponape
    {  470, 1534,       0,  596711,       0,  0, 14,  0, }, // Pyongyang
    {  470, 1554,       0,  596725,       0,  0, 12,  0, }, // Reunion
    {  470, 1562,       0,  596737,       0,  0,  9,  0, }, // Rothera
    {  470, 1570,  596746,  596755,  596771,  9, 16, 14, }, // Sakhalin
    {  470, 1596,  596785,  596793,  596808,  8, 15, 22, }, // Samoa
    {  470, 1602,       0,  596830,       0,  0, 10,  0, }, // Seychelles
    {  470, 1624,       0,  596840,       0,  0, 18,  0, }, // Singapore
    {  470, 1634,       0,  596858,       0,  0, 19,  0, }, // Solomon
    {  470, 1642,       0,  596877,       0,  0, 17,  0, }, // South_Georgia
    {  470, 1656,       0,  596894,       0,  0, 11,  0, }, // Suriname
    {  470, 1676,       0,  596905,       0,  0,  8,  0, }, // Syowa
    {  470, 1682,       0,  596913,       0,  0,  9,  0, }, // Tahiti
    {  470, 1689,  596922,  596931,  596947,  9, 16, 23, }, // Taipei
    {  470, 1696,       0,  596970,       0,  0, 12,  0, }, // Tajikistan
    {  470, 1724,       0,  596982,       0,  0, 11,  0, }, // Tokelau
    {  470, 1732,  596993,  597002,  597019,  9, 17, 14, }, // Tonga
    {  470, 1738,       0,  597033,       0,  0,  7,  0, }, // Truk
    {  470, 1750,  597040,  597054,  597074, 14, 20, 18, }, // Turkmenistan
    {  470, 1763,       0,  597092,       0,  0, 10,  0, }, // Tuvalu
    {  470, 1777,  597102,  597114,  597133, 12, 19, 17, }, // Uruguay
    {  470, 1792,  597150,  597162,  597181, 12, 19, 17, }, // Uzbekistan
    {  470, 1803,  597198,  597209,  597226, 11, 17, 15, }, // Vanuatu
    {  470, 1811,       0,  597241,       0,  0, 12,  0, }, // Venezuela
    {  470, 1821,  597253,  597267,  597288, 14, 21, 19, }, // Vladivostok
    {  470, 1833,  597307,  597320,  597340, 13, 20, 18, }, // Volgograd
    {  470, 1843,       0,  597358,       0,  0, 10,  0, }, // Vostok
    {  470, 1850,       0,  597368,       0,  0, 12,  0, }, // Wake
    {  470, 1855,       0,  597380,       0,  0, 18,  0, }, // Wallis
    {  470, 1862,  597398,  597409,  597427, 11, 18, 18, }, // Yakutsk
    {  470, 1870,  597445,  597460,  597483, 15, 23, 22, }, // Yekaterinburg
    {  470, 1892,       0,  597505,       0,  0, 11,  0, }, // Yukon
    {  471,  107,  592688,  592698,  597516, 10, 17, 26, }, // Alaska Pashto/Arabic/Pakistan
    {  471,  142,  592784,  592793,  597542,  9, 16, 18, }, // America_Central
    {  471,  158,  592832,  592840,  597560,  8, 15, 19, }, // America_Eastern
    {  471,  174,  592874,  592885,  597579, 11, 16, 20, }, // America_Mountain
    {  471,  191,  592921,  592930,  597599,  9, 18, 20, }, // America_Pacific
    {  471,  214,  592968,  592976,  597619,  8, 15, 17, }, // Apia
    {  471,  232,  593013,  593021,  597636,  8, 15, 18, }, // Arabian
    {  471,  286,  593223,  593235,  597654, 12, 19, 23, }, // Atlantic
    {  471,  356,  593520,  593539,  597677, 19, 26, 28, }, // Australia_Western
    {  471,  501,  593972,  593979,  597705,  7, 14, 20, }, // China
    {  471,  537,  594155,  594164,  597725,  9, 16, 20, }, // Cuba
    {  471,  680,  597745,  597762,  597787, 17, 25, 25, }, // Europe_Western
    {  471,  723,       0,  597812,       0,  0, 29,  0, }, // French_Southern
    {  471,  883,  594923,  594940,  597841, 17, 24, 26, }, // Hawaii_Aleutian
    {  471,  997,  595170,  595179,  597867,  9, 16, 18, }, // Iran
    {  471, 1016,  595263,  595274,  597885, 11, 18, 21, }, // Israel
    {  471, 1023,  595317,  595326,  597906,  9, 16, 20, }, // Japan
    {  471, 1106,  595413,  595424,  597926, 11, 18, 25, }, // Korea
    {  471, 1175,       0,  597951,       0,  0, 17,  0, }, // Line_Islands
    {  471, 1188,  595564,  595576,  597968, 12, 19, 23, }, // Lord_Howe
    {  471, 1281,  595769,  595785,  597991, 16, 23, 25, }, // Mexico_Pacific
    {  471, 1346,  596016,  596030,  598016, 14, 21, 29, }, // New_Zealand
    {  471, 1358,  596079,  596096,  598045, 17, 25, 27, }, // Newfoundland
    {  471, 1502,  596604,  596627,  598072, 23, 30, 32, }, // Pierre_Miquelon
    {  471, 1596,  596785,  596793,  598104,  8, 15, 24, }, // Samoa
    {  472,    5,       0,  598128,       0,  0, 13,  0, }, // Afghanistan Persian/Arabic/Iran
    {  472,   17,       0,  598141,       0,  0, 15,  0, }, // Africa_Central
    {  472,   32,       0,  598156,       0,  0, 14,  0, }, // Africa_Eastern
    {  472,   65,       0,  598170,       0,  0, 20,  0, }, // Africa_Southern
    {  472,   81,  598190,  598204,  598223, 14, 19, 23, }, // Africa_Western
    {  472,  107,  598246,  598256,  598271, 10, 15, 19, }, // Alaska
    {  472,  128,  598290,  598301,  598317, 11, 16, 20, }, // Almaty
    {  472,  135,  598337,  598347,  598362, 10, 15, 19, }, // Amazon
    {  472,  142,  598381,  598396,  598416, 15, 20, 24, }, // America_Central
    {  472,  158,  598440,  598454,  598473, 14, 19, 23, }, // America_Eastern
    {  472,  174,  598496,  598515,  598539, 19, 24, 28, }, // America_Mountain
    {  472,  191,  598567,  598581,  598600, 14, 19, 23, }, // America_Pacific
    {  472,  207,  598623,  598633,  598648, 10, 15, 19, }, // Anadyr
    {  472,  214,  598667,  598675,  598688,  8, 13, 17, }, // Apia
    {  472,  232,  598705,  598716,  598732, 11, 16, 20, }, // Arabian
    {  472,  240,  598752,  598764,  598781, 12, 17, 21, }, // Argentina
    {  472,  250,  598802,  598818,  598839, 16, 21, 25, }, // Argentina_Western
    {  472,  268,  598864,  598876,  598893, 12, 17, 21, }, // Armenia
    {  472,  286,  598914,  598926,  598943, 12, 17, 21, }, // Atlantic
    {  472,  295,  598964,  598981,  599003, 17, 22, 26, }, // Australia_Central
    {  472,  313,  599029,  599052,  599080, 23, 28, 32, }, // Australia_CentralWestern
    {  472,  338,  599112,  599130,  599153, 18, 23, 27, }, // Australia_Eastern
    {  472,  356,  599180,  599198,  599221, 18, 23, 27, }, // Australia_Western
    {  472,  374,  599248,  599268,  599293, 20, 25, 29, }, // Azerbaijan
    {  472,  385,  599322,  599330,  599343,  8, 13, 17, }, // Azores
    {  472,  397,  599360,  599371,  599387, 11, 16, 20, }, // Bangladesh
    {  472,  415,       0,  599407,       0,  0,  9,  0, }, // Bhutan
    {  472,  422,       0,  599416,       0,  0, 10,  0, }, // Bolivia
    {  472,  437,  599426,  599438,  599455, 12, 17, 21, }, // Brasilia
    {  472,  454,       0,  599476,       0,  0, 20,  0, }, // Brunei
    {  472,  461,  599496,  599507,  599523, 11, 16, 20, }, // Cape_Verde
    {  472,  478,       0,  599543,       0,  0, 15,  0, }, // Chamorro
    {  472,  487,  599558,  599568,  599583, 10, 15, 19, }, // Chatham
    {  472,  495,  599602,  599610,  599623,  8, 13, 17, }, // Chile
    {  472,  501,  599640,  599647,  599659,  7, 12, 16, }, // China
    {  472,  507,       0,  599675,       0,  0, 17,  0, }, // Christmas
    {  472,  517,       0,  599692,       0,  0, 15,  0, }, // Cocos
    {  472,  523,  599707,  599717,  599732, 10, 15, 19, }, // Colombia
    {  472,  532,  599692,  599751,  599769, 13, 18, 22, }, // Cook
    {  472,  537,  599791,  599799,  599812,  8, 13, 17, }, // Cuba
    {  472,  548,       0,  599829,       0,  0,  9,  0, }, // Davis
    {  472,  564,       0,  599838,       0,  0, 16,  0, }, // DumontDUrville
    {  472,  601,       0,  599854,       0,  0, 14,  0, }, // East_Timor
    {  472,  612,  599868,  599884,  599905, 16, 21, 25, }, // Easter
    {  472,  619,       0,  599930,       0,  0, 11,  0, }, // Ecuador
    {  472,  627,  599941,  599955,  599974, 14, 19, 23, }, // Europe_Central
    {  472,  642,  599997,  600010,  600028, 13, 18, 22, }, // Europe_Eastern
    {  472,  657,       0,  600050,       0,  0, 36,  0, }, // Europe_Further_Eastern
    {  472,  680,  600086,  600099,  600117, 13, 18, 22, }, // Europe_Western
    {  472,  695,  600139,  600156,  600178, 17, 22, 26, }, // Falkland
    {  472,  704,  600204,  600212,  600225,  8, 13, 17, }, // Fiji
    {  472,  709,       0,  600242,       0,  0, 16,  0, }, // French_Guiana
    {  472,  723,       0,  600258,       0,  0, 37,  0, }, // French_Southern
    {  472,  746,       0,  600295,       0,  0, 13,  0, }, // Galapagos
    {  472,  756,       0,  600308,       0,  0, 10,  0, }, // Gambier
    {  472,  764,  600318,  600329,  600345, 11, 16, 20, }, // Georgia
    {  472,  772,       0,  600365,       0,  0, 16,  0, }, // Gilbert_Islands
    {  472,  788,       0,  600381,       0,  0, 11,  0, }, // GMT
    {  472,  830,  600392,  600407,  600427, 15, 20, 24, }, // Greenland_Eastern
    {  472,  848,  600451,  600466,  600486, 15, 20, 24, }, // Greenland_Western
    {  472,  866,       0,  600510,       0,  0, 13,  0, }, // Guam
    {  472,  871,       0,  600523,       0,  0, 18,  0, }, // Gulf
    {  472,  876,       0,  600242,       0,  0,  9,  0, }, // Guyana
    {  472,  883,  600541,  600557,  600578, 16, 21, 25, }, // Hawaii_Aleutian
    {  472,  899,  600603,  600614,  600630, 11, 16, 20, }, // Hong_Kong
    {  472,  909,  600650,  600658,  600671,  8, 13, 17, }, // Hovd
    {  472,  914,       0,  600688,       0,  0,  7,  0, }, // India
    {  472,  920,       0,  600695,       0,  0, 15,  0, }, // Indian_Ocean
    {  472,  933,       0,  600710,       0,  0, 11,  0, }, // Indochina
    {  472,  943,       0,  600721,       0,  0, 16,  0, }, // Indonesia_Central
    {  472,  961,       0,  600737,       0,  0, 15,  0, }, // Indonesia_Eastern
    {  472,  979,       0,  600752,       0,  0, 15,  0, }, // Indonesia_Western
    {  472,  997,  600767,  600776,  600790,  9, 14, 18, }, // Iran
    {  472, 1008,  600808,  600820,  600837, 12, 17, 21, }, // Irkutsk
    {  472, 1016,  600858,  600869,  600885, 11, 16, 20, }, // Israel
    {  472, 1023,  600905,  600913,  600926,  8, 13, 17, }, // Japan
    {  472, 1029,  600943,  600968,  600998, 25, 30, 34, }, // Kamchatka
    {  472, 1047,       0,  601032,       0,  0, 12,  0, }, // Kazakhstan
    {  472, 1058,       0,  601044,       0,  0, 16,  0, }, // Kazakhstan_Eastern
    {  472, 1077,       0,  601060,       0,  0, 16,  0, }, // Kazakhstan_Western
    {  472, 1106,  601076,  601083,  601095,  7, 12, 16, }, // Korea
    {  472, 1112,       0,  601111,       0,  0, 10,  0, }, // Kosrae
    {  472, 1119,  601121,  601136,  601156, 15, 20, 24, }, // Krasnoyarsk
    {  472, 1151,       0,  601180,       0,  0, 13,  0, }, // Kyrgystan
    {  472, 1161,       0,  601193,       0,  0,  9,  0, }, // Lanka
    {  472, 1175,       0,  601202,       0,  0, 14,  0, }, // Line_Islands
    {  472, 1188,  601216,  601225,  601239,  9, 14, 18, }, // Lord_Howe
    {  472, 1198,  601257,  601267,  601282, 10, 15, 19, }, // Macau
    {  472, 1204,  601301,  601312,  601328, 11, 16, 20, }, // Magadan
    {  472, 1219,       0,  601348,       0,  0,  9,  0, }, // Malaysia
    {  472, 1228,       0,  601357,       0,  0, 10,  0, }, // Maldives
    {  472, 1237,       0,  601367,       0,  0, 11,  0, }, // Marquesas
    {  472, 1247,       0,  601378,       0,  0, 16,  0, }, // Marshall_Islands
    {  472, 1264,  601394,  601403,  601417,  9, 14, 18, }, // Mauritius
    {  472, 1274,       0,  601435,       0,  0, 10,  0, }, // Mawson
    {  472, 1281,  601445,  601458,  601476, 13, 18, 22, }, // Mexico_Pacific
    {  472, 1296,  601498,  601513,  601533, 15, 20, 24, }, // Mongolia
    {  472, 1305,  601557,  601565,  601578,  8, 13, 17, }, // Moscow
    {  472, 1312,       0,  601595,       0,  0, 11,  0, }, // Myanmar
    {  472, 1320,       0,  601606,       0,  0, 10,  0, }, // Nauru
    {  472, 1326,       0,  601616,       0,  0,  8,  0, }, // Nepal
    {  472, 1332,  601624,  601642,  601665, 18, 23, 27, }, // New_Caledonia
    {  472, 1346,  601692,  601704,  601721, 12, 17, 21, }, // New_Zealand
    {  472, 1358,  601742,  601756,  601775, 14, 19, 23, }, // Newfoundland
    {  472, 1371,       0,  601798,       0,  0,  9,  0, }, // Niue
    {  472, 1376,  601807,  601825,  601848, 18, 23, 27, }, // Norfolk
    {  472, 1384,  601875,  601897,  601924, 22, 27, 31, }, // Noronha
    {  472, 1392,       0,  601955,       0,  0, 24,  0, }, // North_Mariana
    {  472, 1406,  601979,  601993,  602012, 14, 19, 23, }, // Novosibirsk
    {  472, 1418,  602035,  602044,  602058,  9, 14, 18, }, // Omsk
    {  472, 1428,  602076,  602087,  602103, 11, 16, 20, }, // Pakistan
    {  472, 1437,       0,  602123,       0,  0, 10,  0, }, // Palau
    {  472, 1443,       0,  602133,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  472, 1460,  602151,  602163,  602180, 12, 17, 21, }, // Paraguay
    {  472, 1469,  602201,  602208,  602220,  7, 12, 16, }, // Peru
    {  472, 1474,  602236,  602247,  602263, 11, 16, 20, }, // Philippines
    {  472, 1486,       0,  602283,       0,  0, 16,  0, }, // Phoenix_Islands
    {  472, 1502,  602299,  602319,  602344, 20, 25, 29, }, // Pierre_Miquelon
    {  472, 1518,       0,  602373,       0,  0, 12,  0, }, // Pitcairn
    {  472, 1527,       0,  602385,       0,  0, 10,  0, }, // Ponape
    {  472, 1534,       0,  602395,       0,  0, 14,  0, }, // Pyongyang
    {  472, 1544,  602409,  602424,  602444, 15, 20, 24, }, // Qyzylorda
    {  472, 1554,       0,  602468,       0,  0, 11,  0, }, // Reunion
    {  472, 1562,       0,  602479,       0,  0,  9,  0, }, // Rothera
    {  472, 1570,  602488,  602499,  602515, 11, 16, 20, }, // Sakhalin
    {  472, 1579,  602535,  602545,  602560, 10, 15, 19, }, // Samara
    {  472, 1596,  602579,  602588,  602602,  9, 14, 18, }, // Samoa
    {  472, 1602,       0,  602620,       0,  0,  8,  0, }, // Seychelles
    {  472, 1624,       0,  602628,       0,  0, 11,  0, }, // Singapore
    {  472, 1634,       0,  602639,       0,  0, 16,  0, }, // Solomon
    {  472, 1642,       0,  602655,       0,  0, 17,  0, }, // South_Georgia
    {  472, 1656,       0,  602672,       0,  0, 11,  0, }, // Suriname
    {  472, 1676,       0,  602683,       0,  0,  8,  0, }, // Syowa
    {  472, 1682,       0,  602691,       0,  0, 10,  0, }, // Tahiti
    {  472, 1689,  602701,  602710,  602724,  9, 14, 18, }, // Taipei
    {  472, 1696,       0,  602742,       0,  0, 13,  0, }, // Tajikistan
    {  472, 1724,       0,  602755,       0,  0, 11,  0, }, // Tokelau
    {  472, 1732,  602766,  602775,  602789,  9, 14, 18, }, // Tonga
    {  472, 1738,       0,  602807,       0,  0,  9,  0, }, // Truk
    {  472, 1750,  602816,  602829,  602847, 13, 18, 22, }, // Turkmenistan
    {  472, 1763,       0,  602869,       0,  0, 10,  0, }, // Tuvalu
    {  472, 1777,  602879,  602890,  602906, 11, 16, 20, }, // Uruguay
    {  472, 1792,  602926,  602938,  602955, 12, 17, 21, }, // Uzbekistan
    {  472, 1803,  602976,  602986,  603001, 10, 15, 19, }, // Vanuatu
    {  472, 1811,       0,  603020,       0,  0, 11,  0, }, // Venezuela
    {  472, 1821,  603031,  603046,  603066, 15, 20, 24, }, // Vladivostok
    {  472, 1833,  603090,  603102,  603119, 12, 17, 21, }, // Volgograd
    {  472, 1843,       0,  603140,       0,  0,  9,  0, }, // Vostok
    {  472, 1850,       0,  603149,       0,  0, 14,  0, }, // Wake
    {  472, 1855,       0,  603163,       0,  0, 18,  0, }, // Wallis
    {  472, 1862,  603181,  603192,  603208, 11, 16, 20, }, // Yakutsk
    {  472, 1870,  603228,  603243,  603263, 15, 20, 24, }, // Yekaterinburg
    {  472, 1892,       0,  603287,       0,  0,  9,  0, }, // Yukon
    {  474,    5,       0,  603296,       0,  0, 15,  0, }, // Afghanistan Polish/Latin/Poland
    {  474,   17,       0,  603311,       0,  0, 23,  0, }, // Africa_Central
    {  474,   32,       0,  603334,       0,  0, 24,  0, }, // Africa_Eastern
    {  474,   65,       0,  603358,       0,  0, 25,  0, }, // Africa_Southern
    {  474,   81,  603383,  603407,  603443, 24, 36, 30, }, // Africa_Western
    {  474,  107,  603473,  603484,  603509, 11, 25, 19, }, // Alaska
    {  474,  128,  603528,  603540,  603564, 12, 24, 18, }, // Almaty
    {  474,  135,  603582,  603596,  603622, 14, 26, 20, }, // Amazon
    {  474,  142,  603642,  603666,  603702, 24, 36, 30, }, // America_Central
    {  474,  158,  603732,  603757,  603794, 25, 37, 31, }, // America_Eastern
    {  474,  174,  603825,  603836,  603859, 11, 23, 17, }, // America_Mountain
    {  474,  191,  603876,  603891,  603918, 15, 27, 21, }, // America_Pacific
    {  474,  207,  603939,  603950,  603973, 11, 23, 17, }, // Anadyr
    {  474,  214,  603990,  603999,  604022,  9, 23, 17, }, // Apia
    {  474,  219,  604039,  604053,  604079, 14, 26, 20, }, // Aqtau
    {  474,  225,  604099,  604115,  604143, 16, 28, 22, }, // Aqtobe
    {  474,  232,  604165,  604186,  604221, 21, 35, 29, }, // Arabian
    {  474,  240,  604250,  604264,  604292, 14, 28, 22, }, // Argentina
    {  474,  250,  604314,  604338,  604376, 24, 38, 32, }, // Argentina_Western
    {  474,  268,  604408,  604420,  604446, 12, 26, 20, }, // Armenia
    {  474,  286,  604466,  604481,  604508, 15, 27, 21, }, // Atlantic
    {  474,  295,  604529,  604554,  604591, 25, 37, 31, }, // Australia_Central
    {  474,  313,  604622,  604657,  604704, 35, 47, 41, }, // Australia_CentralWestern
    {  474,  338,  604745,  604771,  604809, 26, 38, 32, }, // Australia_Eastern
    {  474,  356,  604841,  604867,  604905, 26, 38, 32, }, // Australia_Western
    {  474,  374,  604937,  604953,  604983, 16, 30, 24, }, // Azerbaijan
    {  474,  385,  605007,  605017,  605041, 10, 24, 18, }, // Azores
    {  474,  397,  605059,  605074,  605103, 15, 29, 23, }, // Bangladesh
    {  474,  415,       0,  605126,       0,  0, 11,  0, }, // Bhutan
    {  474,  422,       0,  605137,       0,  0, 12,  0, }, // Bolivia
    {  474,  437,  605149,  605162,  605189, 13, 27, 21, }, // Brasilia
    {  474,  454,       0,  605210,       0,  0, 11,  0, }, // Brunei
    {  474,  461,  605221,  605251,  605295, 30, 44, 38, }, // Cape_Verde
    {  474,  478,       0,  605333,       0,  0, 13,  0, }, // Chamorro
    {  474,  487,  605346,  605358,  605384, 12, 26, 20, }, // Chatham
    {  474,  495,  605404,  605414,  605438, 10, 24, 18, }, // Chile
    {  474,  501,  605456,  605466,  605490, 10, 24, 18, }, // China
    {  474,  507,       0,  605508,       0,  0, 28,  0, }, // Christmas
    {  474,  517,       0,  605536,       0,  0, 19,  0, }, // Cocos
    {  474,  523,  605555,  605568,  605595, 13, 27, 21, }, // Colombia
    {  474,  532,  605616,  605632,  605662, 16, 30, 24, }, // Cook
    {  474,  537,  605686,  605695,  605718,  9, 23, 17, }, // Cuba
    {  474,  548,       0,  605735,       0,  0, 10,  0, }, // Davis
    {  474,  564,       0,  605745,       0,  0, 21,  0, }, // DumontDUrville
    {  474,  601,       0,  605766,       0,  0, 19,  0, }, // East_Timor
    {  474,  612,  605785,  605807,  605843, 22, 36, 30, }, // Easter
    {  474,  619,       0,  605873,       0,  0, 12,  0, }, // Ecuador
    {  474,  627,  605885,  605908,  605943, 23, 35, 29, }, // Europe_Central
    {  474,  642,  605972,  605996,  606032, 24, 36, 30, }, // Europe_Eastern
    {  474,  657,       0,  606062,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  474,  680,  606093,  606117,  606153, 24, 36, 30, }, // Europe_Western
    {  474,  695,  606183,  606197,  606225, 14, 28, 22, }, // Falkland
    {  474,  704,  606247,  606257,  606281, 10, 24, 18, }, // Fiji
    {  474,  709,       0,  606299,       0,  0, 21,  0, }, // French_Guiana
    {  474,  723,       0,  606320,       0,  0, 51,  0, }, // French_Southern
    {  474,  746,       0,  606371,       0,  0, 14,  0, }, // Galapagos
    {  474,  756,       0,  606385,       0,  0, 19,  0, }, // Gambier
    {  474,  764,  606404,  606415,  606440, 11, 25, 19, }, // Georgia
    {  474,  772,       0,  606459,       0,  0, 19,  0, }, // Gilbert_Islands
    {  474,  788,       0,  606478,       0,  0, 16,  0, }, // GMT
    {  474,  830,  606494,  606519,  606558, 25, 39, 33, }, // Greenland_Eastern
    {  474,  848,  606591,  606616,  606655, 25, 39, 33, }, // Greenland_Western
    {  474,  871,       0,  606688,       0,  0, 18,  0, }, // Gulf
    {  474,  876,       0,  606299,       0,  0, 11,  0, }, // Guyana
    {  474,  883,  606706,  606724,  606756, 18, 32, 26, }, // Hawaii_Aleutian
    {  474,  899,  606782,  606795,  606822, 13, 27, 21, }, // Hong_Kong
    {  474,  909,  606843,  606853,  606877, 10, 24, 18, }, // Hovd
    {  474,  914,       0,  606895,       0,  0, 25,  0, }, // India
    {  474,  920,       0,  606920,       0,  0, 19,  0, }, // Indian_Ocean
    {  474,  933,       0,  606939,       0,  0, 16,  0, }, // Indochina
    {  474,  943,       0,  606955,       0,  0, 23,  0, }, // Indonesia_Central
    {  474,  961,       0,  606978,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  474,  979,       0,  607002,       0,  0, 24,  0, }, // Indonesia_Western
    {  474,  997,  607026,  607035,  607058,  9, 23, 17, }, // Iran
    {  474, 1008,  607075,  607086,  607111, 11, 25, 19, }, // Irkutsk
    {  474, 1016,  607130,  607141,  607166, 11, 25, 19, }, // Israel
    {  474, 1023,  607185,  607197,  607223, 12, 26, 20, }, // Japan
    {  474, 1029,  607243,  607272,  607313, 29, 41, 35, }, // Kamchatka
    {  474, 1047,       0,  607348,       0,  0, 15,  0, }, // Kazakhstan
    {  474, 1058,       0,  607363,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  474, 1077,       0,  607387,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  474, 1106,  607411,  607421,  607445, 10, 24, 18, }, // Korea
    {  474, 1112,       0,  607463,       0,  0, 11,  0, }, // Kosrae
    {  474, 1119,  607474,  607490,  607520, 16, 30, 24, }, // Krasnoyarsk
    {  474, 1151,       0,  607544,       0,  0, 14,  0, }, // Kyrgystan
    {  474, 1175,       0,  607558,       0,  0, 17,  0, }, // Line_Islands
    {  474, 1188,  607575,  607589,  607617, 14, 28, 22, }, // Lord_Howe
    {  474, 1204,  607639,  607651,  607677, 12, 26, 20, }, // Magadan
    {  474, 1219,       0,  607697,       0,  0, 12,  0, }, // Malaysia
    {  474, 1228,       0,  607709,       0,  0, 13,  0, }, // Maldives
    {  474, 1237,       0,  607722,       0,  0, 12,  0, }, // Marquesas
    {  474, 1247,       0,  607734,       0,  0, 20,  0, }, // Marshall_Islands
    {  474, 1264,  607754,  607768,  607796, 14, 28, 22, }, // Mauritius
    {  474, 1274,       0,  607818,       0,  0, 11,  0, }, // Mawson
    {  474, 1281,  607829,  607853,  607889, 24, 36, 30, }, // Mexico_Pacific
    {  474, 1296,  607919,  607934,  607963, 15, 29, 23, }, // Mongolia
    {  474, 1305,  607986,  607997,  608022, 11, 25, 19, }, // Moscow
    {  474, 1312,       0,  608041,       0,  0, 11,  0, }, // Myanmar
    {  474, 1320,       0,  608052,       0,  0, 10,  0, }, // Nauru
    {  474, 1326,       0,  608062,       0,  0, 10,  0, }, // Nepal
    {  474, 1332,  608072,  608091,  608124, 19, 33, 27, }, // New_Caledonia
    {  474, 1346,  608151,  608169,  608201, 18, 32, 26, }, // New_Zealand
    {  474, 1358,  608227,  608247,  608281, 20, 34, 28, }, // Newfoundland
    {  474, 1371,       0,  608309,       0,  0,  9,  0, }, // Niue
    {  474, 1376,  608318,  608330,  608356, 12, 26, 20, }, // Norfolk
    {  474, 1384,  608376,  608400,  608438, 24, 38, 32, }, // Noronha
    {  474, 1406,  608470,  608486,  608516, 16, 30, 24, }, // Novosibirsk
    {  474, 1418,  608540,  608549,  608572,  9, 23, 17, }, // Omsk
    {  474, 1428,  608589,  608602,  608629, 13, 27, 21, }, // Pakistan
    {  474, 1437,       0,  608650,       0,  0, 10,  0, }, // Palau
    {  474, 1443,       0,  608660,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  474, 1460,  608682,  608695,  608722, 13, 27, 21, }, // Paraguay
    {  474, 1469,  608743,  608752,  608775,  9, 23, 17, }, // Peru
    {  474, 1474,  608792,  608805,  608832, 13, 27, 21, }, // Philippines
    {  474, 1486,       0,  608853,       0,  0, 11,  0, }, // Phoenix_Islands
    {  474, 1502,  608864,  608892,  608934, 28, 42, 36, }, // Pierre_Miquelon
    {  474, 1518,       0,  608970,       0,  0, 13,  0, }, // Pitcairn
    {  474, 1527,       0,  608983,       0,  0, 12,  0, }, // Ponape
    {  474, 1534,       0,  608995,       0,  0, 14,  0, }, // Pyongyang
    {  474, 1544,  609009,  609025,  609053, 16, 28, 22, }, // Qyzylorda
    {  474, 1554,       0,  609075,       0,  0, 12,  0, }, // Reunion
    {  474, 1562,       0,  609087,       0,  0, 12,  0, }, // Rothera
    {  474, 1570,  609099,  609112,  609139, 13, 27, 21, }, // Sakhalin
    {  474, 1579,  609160,  609171,  609194, 11, 23, 17, }, // Samara
    {  474, 1596,  609211,  609221,  609245, 10, 24, 18, }, // Samoa
    {  474, 1602,       0,  609263,       0,  0, 12,  0, }, // Seychelles
    {  474, 1624,       0,  609275,       0,  0, 13,  0, }, // Singapore
    {  474, 1634,       0,  609288,       0,  0, 19,  0, }, // Solomon
    {  474, 1642,       0,  609307,       0,  0, 23,  0, }, // South_Georgia
    {  474, 1656,       0,  609330,       0,  0, 12,  0, }, // Suriname
    {  474, 1676,       0,  609342,       0,  0, 10,  0, }, // Syowa
    {  474, 1682,       0,  609352,       0,  0, 11,  0, }, // Tahiti
    {  474, 1689,  609363,  609374,  609399, 11, 25, 19, }, // Taipei
    {  474, 1696,       0,  609418,       0,  0, 16,  0, }, // Tajikistan
    {  474, 1724,       0,  609434,       0,  0, 12,  0, }, // Tokelau
    {  474, 1732,  609446,  609456,  609480, 10, 24, 18, }, // Tonga
    {  474, 1738,       0,  609498,       0,  0, 10,  0, }, // Truk
    {  474, 1750,  609508,  609525,  609556, 17, 31, 25, }, // Turkmenistan
    {  474, 1763,       0,  609581,       0,  0, 11,  0, }, // Tuvalu
    {  474, 1777,  609592,  609604,  609630, 12, 26, 20, }, // Uruguay
    {  474, 1792,  609650,  609665,  609694, 15, 29, 23, }, // Uzbekistan
    {  474, 1803,  609717,  609729,  609755, 12, 26, 20, }, // Vanuatu
    {  474, 1811,       0,  609775,       0,  0, 14,  0, }, // Venezuela
    {  474, 1821,  609789,  609805,  609835, 16, 30, 24, }, // Vladivostok
    {  474, 1833,  609859,  609873,  609901, 14, 28, 22, }, // Volgograd
    {  474, 1843,       0,  609923,       0,  0, 11,  0, }, // Vostok
    {  474, 1850,       0,  609934,       0,  0,  9,  0, }, // Wake
    {  474, 1855,       0,  609943,       0,  0, 20,  0, }, // Wallis
    {  474, 1862,  609963,  609974,  609999, 11, 25, 19, }, // Yakutsk
    {  474, 1870,  610018,  610036,  610068, 18, 32, 26, }, // Yekaterinburg
    {  474, 1892,       0,  610094,       0,  0, 10,  0, }, // Yukon
    {  475,    0,  610104,  610119,  610141, 15, 22, 24, }, // Acre Portuguese/Latin/Brazil
    {  475,    5,       0,  610165,       0,  0, 22,  0, }, // Afghanistan
    {  475,   17,       0,  610187,       0,  0, 25,  0, }, // Africa_Central
    {  475,   32,       0,  610212,       0,  0, 26,  0, }, // Africa_Eastern
    {  475,   65,       0,  610238,       0,  0, 24,  0, }, // Africa_Southern
    {  475,   81,  610262,  610289,  610323, 27, 34, 36, }, // Africa_Western
    {  475,  107,  610359,  610376,  610400, 17, 24, 26, }, // Alaska
    {  475,  128,  610426,  610443,  610467, 17, 24, 26, }, // Almaty
    {  475,  135,  610493,  610512,  610538, 19, 26, 28, }, // Amazon
    {  475,  142,  610566,  610581,  610603, 15, 22, 24, }, // America_Central
    {  475,  158,  610627,  610643,  610666, 16, 23, 25, }, // America_Eastern
    {  475,  174,  610691,  610712,  610740, 21, 28, 30, }, // America_Mountain
    {  475,  191,  610770,  610789,  610815, 19, 26, 28, }, // America_Pacific
    {  475,  207,  610843,  610860,  610884, 17, 24, 26, }, // Anadyr
    {  475,  214,  610910,  610925,  610947, 15, 22, 24, }, // Apia
    {  475,  219,  610971,  610987,  611010, 16, 23, 25, }, // Aqtau
    {  475,  225,  611035,  611052,  611076, 17, 24, 26, }, // Aqtobe
    {  475,  232,  611102,  611119,  611143, 17, 24, 26, }, // Arabian
    {  475,  240,  611169,  611189,  611216, 20, 27, 29, }, // Argentina
    {  475,  250,  611245,  611275,  611312, 30, 37, 39, }, // Argentina_Western
    {  475,  268,  611351,  611369,  611394, 18, 25, 27, }, // Armenia
    {  475,  286,  611421,  611441,  611468, 20, 27, 29, }, // Atlantic
    {  475,  295,  611497,  611525,  611560, 28, 35, 37, }, // Australia_Central
    {  475,  313,  611597,  611634,  611678, 37, 44, 46, }, // Australia_CentralWestern
    {  475,  338,  611724,  611753,  611789, 29, 36, 38, }, // Australia_Eastern
    {  475,  356,  611827,  611857,  611894, 30, 37, 39, }, // Australia_Western
    {  475,  374,  611933,  611955,  611984, 22, 29, 31, }, // Azerbaijan
    {  475,  385,  612015,  612033,  612058, 18, 25, 27, }, // Azores
    {  475,  397,  612085,  612106,  612134, 21, 28, 30, }, // Bangladesh
    {  475,  415,       0,  612164,       0,  0, 16,  0, }, // Bhutan
    {  475,  422,       0,  612180,       0,  0, 18,  0, }, // Bolivia
    {  475,  437,  612198,  612217,  612243, 19, 26, 28, }, // Brasilia
    {  475,  454,       0,  612271,       0,  0, 28,  0, }, // Brunei
    {  475,  461,  612299,  612320,  612348, 21, 28, 30, }, // Cape_Verde
    {  475,  478,       0,  612378,       0,  0, 19,  0, }, // Chamorro
    {  475,  487,  612397,  612415,  612440, 18, 25, 27, }, // Chatham
    {  475,  495,  612467,  612483,  612506, 16, 23, 25, }, // Chile
    {  475,  501,  612531,  612547,  612570, 16, 23, 25, }, // China
    {  475,  507,       0,  612595,       0,  0, 25,  0, }, // Christmas
    {  475,  517,       0,  612620,       0,  0, 22,  0, }, // Cocos
    {  475,  523,  612642,  612661,  612687, 19, 26, 28, }, // Colombia
    {  475,  532,  612715,  612737,  612766, 22, 29, 36, }, // Cook
    {  475,  537,  612802,  612817,  612839, 15, 22, 24, }, // Cuba
    {  475,  548,       0,  612863,       0,  0, 16,  0, }, // Davis
    {  475,  564,       0,  612879,       0,  0, 27,  0, }, // DumontDUrville
    {  475,  601,       0,  612906,       0,  0, 22,  0, }, // East_Timor
    {  475,  612,  612928,  612953,  612985, 25, 32, 34, }, // Easter
    {  475,  619,       0,  613019,       0,  0, 18,  0, }, // Ecuador
    {  475,  627,  613037,  613062,  613094, 25, 32, 34, }, // Europe_Central
    {  475,  642,  613128,  613154,  613187, 26, 33, 35, }, // Europe_Eastern
    {  475,  657,       0,  613222,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  475,  680,  613254,  613281,  613315, 27, 34, 36, }, // Europe_Western
    {  475,  695,  613351,  613377,  613410, 26, 33, 35, }, // Falkland
    {  475,  704,  613445,  613460,  613482, 15, 22, 24, }, // Fiji
    {  475,  709,       0,  613506,       0,  0, 26,  0, }, // French_Guiana
    {  475,  723,       0,  613532,       0,  0, 52,  0, }, // French_Southern
    {  475,  746,       0,  613584,       0,  0, 20,  0, }, // Galapagos
    {  475,  756,       0,  613604,       0,  0, 18,  0, }, // Gambier
    {  475,  764,  613622,  613640,  613665, 18, 25, 27, }, // Georgia
    {  475,  772,       0,  613692,       0,  0, 26,  0, }, // Gilbert_Islands
    {  475,  788,       0,  613718,       0,  0, 33,  0, }, // GMT
    {  475,  830,  613751,  613781,  613818, 30, 37, 39, }, // Greenland_Eastern
    {  475,  848,  613857,  613889,  613928, 32, 39, 41, }, // Greenland_Western
    {  475,  866,       0,  613969,       0,  0, 22,  0, }, // Guam
    {  475,  871,       0,  613991,       0,  0, 16,  0, }, // Gulf
    {  475,  876,       0,  613506,       0,  0, 17,  0, }, // Guyana
    {  475,  883,  614007,  614039,  614078, 32, 39, 41, }, // Hawaii_Aleutian
    {  475,  899,  614119,  614139,  614166, 20, 27, 29, }, // Hong_Kong
    {  475,  909,  614195,  614210,  614232, 15, 22, 24, }, // Hovd
    {  475,  914,       0,  614256,       0,  0, 23,  0, }, // India
    {  475,  920,       0,  614279,       0,  0, 24,  0, }, // Indian_Ocean
    {  475,  933,       0,  614303,       0,  0, 20,  0, }, // Indochina
    {  475,  943,       0,  614323,       0,  0, 28,  0, }, // Indonesia_Central
    {  475,  961,       0,  614351,       0,  0, 29,  0, }, // Indonesia_Eastern
    {  475,  979,       0,  614380,       0,  0, 30,  0, }, // Indonesia_Western
    {  475,  997,  614410,  614424,  614445, 14, 21, 23, }, // Iran
    {  475, 1008,  614468,  614486,  614511, 18, 25, 27, }, // Irkutsk
    {  475, 1016,  614538,  614555,  614579, 17, 24, 26, }, // Israel
    {  475, 1023,  614605,  614621,  614644, 16, 23, 25, }, // Japan
    {  475, 1029,  614669,  614704,  614746, 35, 42, 44, }, // Kamchatka
    {  475, 1047,       0,  614790,       0,  0, 22,  0, }, // Kazakhstan
    {  475, 1058,       0,  614812,       0,  0, 31,  0, }, // Kazakhstan_Eastern
    {  475, 1077,       0,  614843,       0,  0, 32,  0, }, // Kazakhstan_Western
    {  475, 1106,  614875,  614892,  614916, 17, 24, 26, }, // Korea
    {  475, 1112,       0,  614942,       0,  0, 17,  0, }, // Kosrae
    {  475, 1119,  614959,  614981,  615010, 22, 29, 31, }, // Krasnoyarsk
    {  475, 1151,       0,  615041,       0,  0, 22,  0, }, // Kyrgystan
    {  475, 1161,       0,  615063,       0,  0, 16,  0, }, // Lanka
    {  475, 1175,       0,  615079,       0,  0, 26,  0, }, // Line_Islands
    {  475, 1188,  615105,  615125,  615152, 20, 27, 29, }, // Lord_Howe
    {  475, 1198,  615181,  615197,  615220, 16, 23, 25, }, // Macau
    {  475, 1204,  615245,  615263,  615288, 18, 25, 27, }, // Magadan
    {  475, 1219,       0,  615315,       0,  0, 18,  0, }, // Malaysia
    {  475, 1228,       0,  615333,       0,  0, 26,  0, }, // Maldives
    {  475, 1237,       0,  615359,       0,  0, 21,  0, }, // Marquesas
    {  475, 1247,       0,  615380,       0,  0, 26,  0, }, // Marshall_Islands
    {  475, 1264,  615406,  615425,  615451, 19, 26, 28, }, // Mauritius
    {  475, 1274,       0,  615479,       0,  0, 17,  0, }, // Mawson
    {  475, 1281,  615496,  615524,  615559, 28, 35, 37, }, // Mexico_Pacific
    {  475, 1296,  615596,  615617,  615645, 21, 28, 30, }, // Mongolia
    {  475, 1305,  615675,  615692,  615716, 17, 24, 26, }, // Moscow
    {  475, 1312,       0,  615742,       0,  0, 18,  0, }, // Myanmar
    {  475, 1320,       0,  615760,       0,  0, 16,  0, }, // Nauru
    {  475, 1326,       0,  615776,       0,  0, 16,  0, }, // Nepal
    {  475, 1332,  615792,  615817,  615849, 25, 32, 34, }, // New_Caledonia
    {  475, 1346,  615883,  615907,  615938, 24, 31, 33, }, // New_Zealand
    {  475, 1358,  615971,  615992,  616020, 21, 28, 30, }, // Newfoundland
    {  475, 1371,       0,  616050,       0,  0, 15,  0, }, // Niue
    {  475, 1376,  616065,  616088,  616118, 23, 30, 32, }, // Norfolk
    {  475, 1384,  616150,  616180,  616217, 30, 37, 39, }, // Noronha
    {  475, 1392,       0,  616256,       0,  0, 34,  0, }, // North_Mariana
    {  475, 1406,  616290,  616312,  616341, 22, 29, 31, }, // Novosibirsk
    {  475, 1418,  616372,  616387,  616409, 15, 22, 24, }, // Omsk
    {  475, 1428,  616433,  616453,  616480, 20, 27, 29, }, // Pakistan
    {  475, 1437,       0,  616509,       0,  0, 16,  0, }, // Palau
    {  475, 1443,       0,  616525,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  475, 1460,  616552,  616571,  616597, 19, 26, 28, }, // Paraguay
    {  475, 1469,  616625,  616640,  616662, 15, 22, 24, }, // Peru
    {  475, 1474,  616686,  616707,  616735, 21, 28, 30, }, // Philippines
    {  475, 1486,       0,  616765,       0,  0, 23,  0, }, // Phoenix_Islands
    {  475, 1502,  616788,  616819,  616857, 31, 38, 37, }, // Pierre_Miquelon
    {  475, 1518,       0,  616894,       0,  0, 19,  0, }, // Pitcairn
    {  475, 1527,       0,  616913,       0,  0, 17,  0, }, // Ponape
    {  475, 1534,       0,  616930,       0,  0, 20,  0, }, // Pyongyang
    {  475, 1544,  616950,  616970,  616997, 20, 27, 29, }, // Qyzylorda
    {  475, 1554,       0,  617026,       0,  0, 18,  0, }, // Reunion
    {  475, 1562,       0,  617044,       0,  0, 18,  0, }, // Rothera
    {  475, 1570,  617062,  617081,  617107, 19, 26, 28, }, // Sakhalin
    {  475, 1579,  617135,  617152,  617176, 17, 24, 26, }, // Samara
    {  475, 1596,  617202,  617218,  617241, 16, 23, 25, }, // Samoa
    {  475, 1602,       0,  617266,       0,  0, 20,  0, }, // Seychelles
    {  475, 1624,       0,  617286,       0,  0, 27,  0, }, // Singapore
    {  475, 1634,       0,  617313,       0,  0, 25,  0, }, // Solomon
    {  475, 1642,       0,  617338,       0,  0, 25,  0, }, // South_Georgia
    {  475, 1656,       0,  617363,       0,  0, 19,  0, }, // Suriname
    {  475, 1676,       0,  617382,       0,  0, 16,  0, }, // Syowa
    {  475, 1682,       0,  617398,       0,  0, 16,  0, }, // Tahiti
    {  475, 1689,  617414,  617431,  617455, 17, 24, 26, }, // Taipei
    {  475, 1696,       0,  617481,       0,  0, 22,  0, }, // Tajikistan
    {  475, 1724,       0,  617503,       0,  0, 18,  0, }, // Tokelau
    {  475, 1732,  617521,  617537,  617560, 16, 23, 25, }, // Tonga
    {  475, 1738,       0,  617585,       0,  0, 16,  0, }, // Truk
    {  475, 1750,  617601,  617625,  617656, 24, 31, 33, }, // Turkmenistan
    {  475, 1763,       0,  617689,       0,  0, 17,  0, }, // Tuvalu
    {  475, 1777,  617706,  617724,  617749, 18, 25, 27, }, // Uruguay
    {  475, 1792,  617776,  617798,  617827, 22, 29, 31, }, // Uzbekistan
    {  475, 1803,  617858,  617876,  617901, 18, 25, 27, }, // Vanuatu
    {  475, 1811,       0,  617928,       0,  0, 20,  0, }, // Venezuela
    {  475, 1821,  617948,  617970,  617999, 22, 29, 31, }, // Vladivostok
    {  475, 1833,  618030,  618051,  618079, 21, 28, 30, }, // Volgograd
    {  475, 1843,       0,  618109,       0,  0, 17,  0, }, // Vostok
    {  475, 1850,       0,  618126,       0,  0, 22,  0, }, // Wake
    {  475, 1855,       0,  618148,       0,  0, 26,  0, }, // Wallis
    {  475, 1862,  618174,  618192,  618217, 18, 25, 27, }, // Yakutsk
    {  475, 1870,  618244,  618268,  618299, 24, 31, 33, }, // Yekaterinburg
    {  475, 1892,       0,  618332,       0,  0, 16,  0, }, // Yukon
    {  476,    0,  618348,  618360,  618379, 12, 19, 21, }, // Acre Portuguese/Latin/Angola
    {  476,    5,       0,  618400,       0,  0, 19,  0, }, // Afghanistan
    {  476,   17,       0,  618419,       0,  0, 22,  0, }, // Africa_Central
    {  476,   32,       0,  618441,       0,  0, 23,  0, }, // Africa_Eastern
    {  476,   65,       0,  618464,       0,  0, 21,  0, }, // Africa_Southern
    {  476,   81,  618485,  618509,  618540, 24, 31, 33, }, // Africa_Western
    {  476,  107,  618573,  618587,  618608, 14, 21, 23, }, // Alaska
    {  476,  128,  618631,  618645,  618666, 14, 21, 23, }, // Almaty
    {  476,  135,  618689,  618705,  618728, 16, 23, 25, }, // Amazon
    {  476,  142,  618753,  618781,  618816, 28, 35, 37, }, // America_Central
    {  476,  158,  618853,  618882,  618918, 29, 36, 38, }, // America_Eastern
    {  476,  174,  618956,  618988,  619027, 32, 39, 41, }, // America_Mountain
    {  476,  191,  619068,  619100,  619139, 32, 39, 41, }, // America_Pacific
    {  476,  207,  619180,  619194,  619215, 14, 21, 23, }, // Anadyr
    {  476,  214,  619238,  619250,  619269, 12, 19, 21, }, // Apia
    {  476,  219,  619290,  619303,  619323, 13, 20, 22, }, // Aqtau
    {  476,  225,  619345,  619359,  619380, 14, 21, 23, }, // Aqtobe
    {  476,  232,  619403,  619417,  619438, 14, 21, 23, }, // Arabian
    {  476,  240,  619461,  619478,  619502, 17, 24, 26, }, // Argentina
    {  476,  250,  619528,  619555,  619589, 27, 34, 36, }, // Argentina_Western
    {  476,  268,  619625,  619640,  619662, 15, 22, 24, }, // Armenia
    {  476,  286,  619686,  619703,  619727, 17, 24, 26, }, // Atlantic
    {  476,  295,  619753,  619778,  619810, 25, 32, 34, }, // Australia_Central
    {  476,  313,  619844,  619879,  619921, 35, 42, 44, }, // Australia_CentralWestern
    {  476,  338,  619965,  619991,  620024, 26, 33, 35, }, // Australia_Eastern
    {  476,  356,  620059,  620086,  620120, 27, 34, 36, }, // Australia_Western
    {  476,  374,  620156,  620174,  620199, 18, 25, 27, }, // Azerbaijan
    {  476,  385,  620226,  620241,  620263, 15, 22, 24, }, // Azores
    {  476,  397,  620287,  620306,  620332, 19, 26, 28, }, // Bangladesh
    {  476,  415,       0,  620360,       0,  0, 13,  0, }, // Bhutan
    {  476,  422,       0,  620373,       0,  0, 15,  0, }, // Bolivia
    {  476,  437,  139322,  620388,  620411, 16, 23, 25, }, // Brasilia
    {  476,  454,       0,  620436,       0,  0, 25,  0, }, // Brunei
    {  476,  461,  620461,  620479,  620504, 18, 25, 27, }, // Cape_Verde
    {  476,  478,       0,  620531,       0,  0, 23,  0, }, // Chamorro
    {  476,  487,   59239,  620554,  620576, 15, 22, 24, }, // Chatham
    {  476,  495,  620600,  620613,  620633, 13, 20, 22, }, // Chile
    {  476,  501,  620655,  620668,  620688, 13, 20, 22, }, // China
    {  476,  507,       0,  620710,       0,  0, 21,  0, }, // Christmas
    {  476,  517,       0,  620731,       0,  0, 20,  0, }, // Cocos
    {  476,  523,  620751,  620767,  620790, 16, 23, 25, }, // Colombia
    {  476,  532,  620815,  620834,  620860, 19, 26, 28, }, // Cook
    {  476,  537,   59634,  620888,  620907, 12, 19, 21, }, // Cuba
    {  476,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  476,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  476,  601,       0,  620928,       0,  0, 19,  0, }, // East_Timor
    {  476,  612,  620947,  620969,  620998, 22, 29, 31, }, // Easter
    {  476,  619,       0,  621029,       0,  0, 15,  0, }, // Ecuador
    {  476,  627,  621044,  621066,  621095, 22, 29, 31, }, // Europe_Central
    {  476,  642,  621126,  621149,  621179, 23, 30, 32, }, // Europe_Eastern
    {  476,  657,       0,  621211,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  476,  680,  621242,  621266,  621297, 24, 31, 33, }, // Europe_Western
    {  476,  695,  621330,  621353,  621383, 23, 30, 32, }, // Falkland
    {  476,  704,  140424,  621415,  621434, 12, 19, 21, }, // Fiji
    {  476,  709,       0,  621455,       0,  0, 23,  0, }, // French_Guiana
    {  476,  723,       0,  621478,       0,  0, 48,  0, }, // French_Southern
    {  476,  746,       0,  621526,       0,  0, 18,  0, }, // Galapagos
    {  476,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  476,  764,  621544,  621559,  621581, 15, 22, 24, }, // Georgia
    {  476,  772,       0,  621605,       0,  0, 22,  0, }, // Gilbert_Islands
    {  476,  788,       0,  621627,       0,  0, 17,  0, }, // GMT
    {  476,  830,  621644,  621672,  621707, 28, 35, 37, }, // Greenland_Eastern
    {  476,  848,  621744,  621773,  621809, 29, 36, 38, }, // Greenland_Western
    {  476,  866,       0,  621847,       0,  0, 19,  0, }, // Guam
    {  476,  871,       0,  621866,       0,  0, 20,  0, }, // Gulf
    {  476,  876,       0,  621455,       0,  0, 14,  0, }, // Guyana
    {  476,  883,  621886,  621909,  621939, 23, 30, 32, }, // Hawaii_Aleutian
    {  476,  899,  141072,  621971,  621995, 17, 24, 26, }, // Hong_Kong
    {  476,  909,   60912,  622021,  622040, 12, 19, 21, }, // Hovd
    {  476,  914,       0,  622061,       0,  0, 20,  0, }, // India
    {  476,  920,       0,  622081,       0,  0, 21,  0, }, // Indian_Ocean
    {  476,  933,       0,  622102,       0,  0, 17,  0, }, // Indochina
    {  476,  943,       0,  622119,       0,  0, 25,  0, }, // Indonesia_Central
    {  476,  961,       0,  622144,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  476,  979,       0,  622170,       0,  0, 27,  0, }, // Indonesia_Western
    {  476,  997,  622197,  622209,  622228, 12, 19, 21, }, // Iran
    {  476, 1008,  622249,  622264,  622286, 15, 22, 24, }, // Irkutsk
    {  476, 1016,  622310,  622324,  622345, 14, 21, 23, }, // Israel
    {  476, 1023,  622368,  622381,  622401, 13, 20, 22, }, // Japan
    {  476, 1029,  622423,  622455,  622494, 32, 39, 41, }, // Kamchatka
    {  476, 1047,       0,  622535,       0,  0, 19,  0, }, // Kazakhstan
    {  476, 1058,       0,  622554,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  476, 1077,       0,  622582,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  476, 1106,  622611,  622625,  622646, 14, 21, 23, }, // Korea
    {  476, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  476, 1119,   61570,  622669,  622695, 19, 26, 28, }, // Krasnoyarsk
    {  476, 1151,       0,  622723,       0,  0, 19,  0, }, // Kyrgystan
    {  476, 1161,       0,  622742,       0,  0, 17,  0, }, // Lanka
    {  476, 1175,       0,  622759,       0,  0, 19,  0, }, // Line_Islands
    {  476, 1188,   61700,  622778,  622802, 17, 24, 26, }, // Lord_Howe
    {  476, 1198,  141929,  622828,  622848, 13, 20, 22, }, // Macau
    {  476, 1204,  141986,  622870,  622892, 15, 22, 24, }, // Magadan
    {  476, 1219,       0,  622916,       0,  0, 15,  0, }, // Malaysia
    {  476, 1228,       0,  622931,       0,  0, 17,  0, }, // Maldives
    {  476, 1237,       0,  622948,       0,  0, 24,  0, }, // Marquesas
    {  476, 1247,       0,  622972,       0,  0, 23,  0, }, // Marshall_Islands
    {  476, 1264,  622995,  623011,  623034, 16, 23, 25, }, // Mauritius
    {  476, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  476, 1281,  623059,  623084,  623116, 25, 32, 34, }, // Mexico_Pacific
    {  476, 1296,  623150,  623168,  623193, 18, 25, 27, }, // Mongolia
    {  476, 1305,  623220,  623235,  623257, 15, 22, 24, }, // Moscow
    {  476, 1312,       0,  623281,       0,  0, 15,  0, }, // Myanmar
    {  476, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  476, 1326,       0,  623296,       0,  0, 13,  0, }, // Nepal
    {  476, 1332,  623309,  623331,  623360, 22, 29, 31, }, // New_Caledonia
    {  476, 1346,  623391,  623412,  623440, 21, 28, 30, }, // New_Zealand
    {  476, 1358,  623470,  623488,  623513, 18, 25, 27, }, // Newfoundland
    {  476, 1371,       0,  623540,       0,  0, 12,  0, }, // Niue
    {  476, 1376,  623552,  623572,  623599, 20, 27, 29, }, // Norfolk
    {  476, 1384,   62667,  623628,  623662, 27, 34, 36, }, // Noronha
    {  476, 1392,       0,  623698,       0,  0, 31,  0, }, // North_Mariana
    {  476, 1406,   62803,  623729,  623755, 19, 26, 28, }, // Novosibirsk
    {  476, 1418,  623783,  623795,  623814, 12, 19, 21, }, // Omsk
    {  476, 1428,  623835,  623852,  623876, 17, 24, 26, }, // Pakistan
    {  476, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  476, 1443,       0,  623902,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  476, 1460,  623926,  623942,  623965, 16, 23, 25, }, // Paraguay
    {  476, 1469,  623990,  624002,  624021, 12, 19, 21, }, // Peru
    {  476, 1474,  624042,  624060,  624085, 18, 25, 27, }, // Philippines
    {  476, 1486,       0,  624112,       0,  0, 20,  0, }, // Phoenix_Islands
    {  476, 1502,  624132,  624160,  624195, 28, 35, 37, }, // Pierre_Miquelon
    {  476, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  476, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {  476, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {  476, 1544,   63419,  624232,  624256, 17, 24, 26, }, // Qyzylorda
    {  476, 1554,       0,  624282,       0,  0, 15,  0, }, // Reunion
    {  476, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  476, 1570,  624297,  624313,  624336, 16, 23, 25, }, // Sakhalin
    {  476, 1579,   63581,  624361,  624382, 14, 21, 23, }, // Samara
    {  476, 1596,   63641,  624405,  624425, 13, 20, 22, }, // Samoa
    {  476, 1602,       0,  624447,       0,  0, 18,  0, }, // Seychelles
    {  476, 1624,       0,  624465,       0,  0, 24,  0, }, // Singapore
    {  476, 1634,       0,  624489,       0,  0, 22,  0, }, // Solomon
    {  476, 1642,       0,  624511,       0,  0, 22,  0, }, // South_Georgia
    {  476, 1656,       0,  624533,       0,  0, 16,  0, }, // Suriname
    {  476, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  476, 1682,       0,  624549,       0,  0, 13,  0, }, // Tahiti
    {  476, 1689,   63834,  624562,  624582, 13, 20, 22, }, // Taipei
    {  476, 1696,       0,  624604,       0,  0, 19,  0, }, // Tajikistan
    {  476, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  476, 1732,   63929,  624623,  624643, 13, 20, 22, }, // Tonga
    {  476, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  476, 1750,  624665,  624687,  624716, 22, 29, 31, }, // Turkmenistan
    {  476, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  476, 1777,  624747,  624762,  624784, 15, 22, 24, }, // Uruguay
    {  476, 1792,  624808,  624827,  624853, 19, 26, 28, }, // Uzbekistan
    {  476, 1803,  624881,  624896,  624918, 15, 22, 24, }, // Vanuatu
    {  476, 1811,       0,  624942,       0,  0, 17,  0, }, // Venezuela
    {  476, 1821,   64318,  624959,  624985, 19, 26, 28, }, // Vladivostok
    {  476, 1833,  625013,  625031,  625056, 18, 25, 27, }, // Volgograd
    {  476, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  476, 1850,       0,  625083,       0,  0, 17,  0, }, // Wake
    {  476, 1855,       0,  625100,       0,  0, 23,  0, }, // Wallis
    {  476, 1862,   64520,  625123,  625145, 15, 22, 24, }, // Yakutsk
    {  476, 1870,  625169,  625190,  625218, 21, 28, 30, }, // Yekaterinburg
    {  476, 1892,       0,  625248,       0,  0, 13,  0, }, // Yukon
    {  477,    0,  618348,  618360,  618379, 12, 19, 21, }, // Acre Portuguese/Latin/Cape Verde
    {  477,    5,       0,  618400,       0,  0, 19,  0, }, // Afghanistan
    {  477,   17,       0,  618419,       0,  0, 22,  0, }, // Africa_Central
    {  477,   32,       0,  618441,       0,  0, 23,  0, }, // Africa_Eastern
    {  477,   65,       0,  618464,       0,  0, 21,  0, }, // Africa_Southern
    {  477,   81,  618485,  618509,  618540, 24, 31, 33, }, // Africa_Western
    {  477,  107,  618573,  618587,  618608, 14, 21, 23, }, // Alaska
    {  477,  128,  618631,  618645,  618666, 14, 21, 23, }, // Almaty
    {  477,  135,  618689,  618705,  618728, 16, 23, 25, }, // Amazon
    {  477,  142,  618753,  618781,  618816, 28, 35, 37, }, // America_Central
    {  477,  158,  618853,  618882,  618918, 29, 36, 38, }, // America_Eastern
    {  477,  174,  618956,  618988,  619027, 32, 39, 41, }, // America_Mountain
    {  477,  191,  619068,  619100,  619139, 32, 39, 41, }, // America_Pacific
    {  477,  207,  619180,  619194,  619215, 14, 21, 23, }, // Anadyr
    {  477,  214,  619238,  619250,  619269, 12, 19, 21, }, // Apia
    {  477,  219,  619290,  619303,  619323, 13, 20, 22, }, // Aqtau
    {  477,  225,  619345,  619359,  619380, 14, 21, 23, }, // Aqtobe
    {  477,  232,  619403,  619417,  619438, 14, 21, 23, }, // Arabian
    {  477,  240,  619461,  619478,  619502, 17, 24, 26, }, // Argentina
    {  477,  250,  619528,  619555,  619589, 27, 34, 36, }, // Argentina_Western
    {  477,  268,  619625,  619640,  619662, 15, 22, 24, }, // Armenia
    {  477,  286,  619686,  619703,  619727, 17, 24, 26, }, // Atlantic
    {  477,  295,  619753,  619778,  619810, 25, 32, 34, }, // Australia_Central
    {  477,  313,  619844,  619879,  619921, 35, 42, 44, }, // Australia_CentralWestern
    {  477,  338,  619965,  619991,  620024, 26, 33, 35, }, // Australia_Eastern
    {  477,  356,  620059,  620086,  620120, 27, 34, 36, }, // Australia_Western
    {  477,  374,  620156,  620174,  620199, 18, 25, 27, }, // Azerbaijan
    {  477,  385,  620226,  620241,  620263, 15, 22, 24, }, // Azores
    {  477,  397,  620287,  620306,  620332, 19, 26, 28, }, // Bangladesh
    {  477,  415,       0,  620360,       0,  0, 13,  0, }, // Bhutan
    {  477,  422,       0,  620373,       0,  0, 15,  0, }, // Bolivia
    {  477,  437,  139322,  620388,  620411, 16, 23, 25, }, // Brasilia
    {  477,  454,       0,  620436,       0,  0, 25,  0, }, // Brunei
    {  477,  461,  620461,  620479,  620504, 18, 25, 27, }, // Cape_Verde
    {  477,  478,       0,  620531,       0,  0, 23,  0, }, // Chamorro
    {  477,  487,   59239,  620554,  620576, 15, 22, 24, }, // Chatham
    {  477,  495,  620600,  620613,  620633, 13, 20, 22, }, // Chile
    {  477,  501,  620655,  620668,  620688, 13, 20, 22, }, // China
    {  477,  507,       0,  620710,       0,  0, 21,  0, }, // Christmas
    {  477,  517,       0,  620731,       0,  0, 20,  0, }, // Cocos
    {  477,  523,  620751,  620767,  620790, 16, 23, 25, }, // Colombia
    {  477,  532,  620815,  620834,  620860, 19, 26, 28, }, // Cook
    {  477,  537,   59634,  620888,  620907, 12, 19, 21, }, // Cuba
    {  477,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  477,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  477,  601,       0,  620928,       0,  0, 19,  0, }, // East_Timor
    {  477,  612,  620947,  620969,  620998, 22, 29, 31, }, // Easter
    {  477,  619,       0,  621029,       0,  0, 15,  0, }, // Ecuador
    {  477,  627,  621044,  621066,  621095, 22, 29, 31, }, // Europe_Central
    {  477,  642,  621126,  621149,  621179, 23, 30, 32, }, // Europe_Eastern
    {  477,  657,       0,  621211,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  477,  680,  621242,  621266,  621297, 24, 31, 33, }, // Europe_Western
    {  477,  695,  621330,  621353,  621383, 23, 30, 32, }, // Falkland
    {  477,  704,  140424,  621415,  621434, 12, 19, 21, }, // Fiji
    {  477,  709,       0,  621455,       0,  0, 23,  0, }, // French_Guiana
    {  477,  723,       0,  621478,       0,  0, 48,  0, }, // French_Southern
    {  477,  746,       0,  621526,       0,  0, 18,  0, }, // Galapagos
    {  477,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  477,  764,  621544,  621559,  621581, 15, 22, 24, }, // Georgia
    {  477,  772,       0,  621605,       0,  0, 22,  0, }, // Gilbert_Islands
    {  477,  788,       0,  621627,       0,  0, 17,  0, }, // GMT
    {  477,  830,  621644,  621672,  621707, 28, 35, 37, }, // Greenland_Eastern
    {  477,  848,  621744,  621773,  621809, 29, 36, 38, }, // Greenland_Western
    {  477,  866,       0,  621847,       0,  0, 19,  0, }, // Guam
    {  477,  871,       0,  621866,       0,  0, 20,  0, }, // Gulf
    {  477,  876,       0,  621455,       0,  0, 14,  0, }, // Guyana
    {  477,  883,  621886,  621909,  621939, 23, 30, 32, }, // Hawaii_Aleutian
    {  477,  899,  141072,  621971,  621995, 17, 24, 26, }, // Hong_Kong
    {  477,  909,   60912,  622021,  622040, 12, 19, 21, }, // Hovd
    {  477,  914,       0,  622061,       0,  0, 20,  0, }, // India
    {  477,  920,       0,  622081,       0,  0, 21,  0, }, // Indian_Ocean
    {  477,  933,       0,  622102,       0,  0, 17,  0, }, // Indochina
    {  477,  943,       0,  622119,       0,  0, 25,  0, }, // Indonesia_Central
    {  477,  961,       0,  622144,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  477,  979,       0,  622170,       0,  0, 27,  0, }, // Indonesia_Western
    {  477,  997,  622197,  622209,  622228, 12, 19, 21, }, // Iran
    {  477, 1008,  622249,  622264,  622286, 15, 22, 24, }, // Irkutsk
    {  477, 1016,  622310,  622324,  622345, 14, 21, 23, }, // Israel
    {  477, 1023,  622368,  622381,  622401, 13, 20, 22, }, // Japan
    {  477, 1029,  622423,  622455,  622494, 32, 39, 41, }, // Kamchatka
    {  477, 1047,       0,  622535,       0,  0, 19,  0, }, // Kazakhstan
    {  477, 1058,       0,  622554,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  477, 1077,       0,  622582,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  477, 1106,  622611,  622625,  622646, 14, 21, 23, }, // Korea
    {  477, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  477, 1119,   61570,  622669,  622695, 19, 26, 28, }, // Krasnoyarsk
    {  477, 1151,       0,  622723,       0,  0, 19,  0, }, // Kyrgystan
    {  477, 1161,       0,  622742,       0,  0, 17,  0, }, // Lanka
    {  477, 1175,       0,  622759,       0,  0, 19,  0, }, // Line_Islands
    {  477, 1188,   61700,  622778,  622802, 17, 24, 26, }, // Lord_Howe
    {  477, 1198,  141929,  622828,  622848, 13, 20, 22, }, // Macau
    {  477, 1204,  141986,  622870,  622892, 15, 22, 24, }, // Magadan
    {  477, 1219,       0,  622916,       0,  0, 15,  0, }, // Malaysia
    {  477, 1228,       0,  622931,       0,  0, 17,  0, }, // Maldives
    {  477, 1237,       0,  622948,       0,  0, 24,  0, }, // Marquesas
    {  477, 1247,       0,  622972,       0,  0, 23,  0, }, // Marshall_Islands
    {  477, 1264,  622995,  623011,  623034, 16, 23, 25, }, // Mauritius
    {  477, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  477, 1281,  623059,  623084,  623116, 25, 32, 34, }, // Mexico_Pacific
    {  477, 1296,  623150,  623168,  623193, 18, 25, 27, }, // Mongolia
    {  477, 1305,  623220,  623235,  623257, 15, 22, 24, }, // Moscow
    {  477, 1312,       0,  623281,       0,  0, 15,  0, }, // Myanmar
    {  477, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  477, 1326,       0,  623296,       0,  0, 13,  0, }, // Nepal
    {  477, 1332,  623309,  623331,  623360, 22, 29, 31, }, // New_Caledonia
    {  477, 1346,  623391,  623412,  623440, 21, 28, 30, }, // New_Zealand
    {  477, 1358,  623470,  623488,  623513, 18, 25, 27, }, // Newfoundland
    {  477, 1371,       0,  623540,       0,  0, 12,  0, }, // Niue
    {  477, 1376,  623552,  623572,  623599, 20, 27, 29, }, // Norfolk
    {  477, 1384,   62667,  623628,  623662, 27, 34, 36, }, // Noronha
    {  477, 1392,       0,  623698,       0,  0, 31,  0, }, // North_Mariana
    {  477, 1406,   62803,  623729,  623755, 19, 26, 28, }, // Novosibirsk
    {  477, 1418,  623783,  623795,  623814, 12, 19, 21, }, // Omsk
    {  477, 1428,  623835,  623852,  623876, 17, 24, 26, }, // Pakistan
    {  477, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  477, 1443,       0,  623902,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  477, 1460,  623926,  623942,  623965, 16, 23, 25, }, // Paraguay
    {  477, 1469,  623990,  624002,  624021, 12, 19, 21, }, // Peru
    {  477, 1474,  624042,  624060,  624085, 18, 25, 27, }, // Philippines
    {  477, 1486,       0,  624112,       0,  0, 20,  0, }, // Phoenix_Islands
    {  477, 1502,  624132,  624160,  624195, 28, 35, 37, }, // Pierre_Miquelon
    {  477, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  477, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {  477, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {  477, 1544,   63419,  624232,  624256, 17, 24, 26, }, // Qyzylorda
    {  477, 1554,       0,  624282,       0,  0, 15,  0, }, // Reunion
    {  477, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  477, 1570,  624297,  624313,  624336, 16, 23, 25, }, // Sakhalin
    {  477, 1579,   63581,  624361,  624382, 14, 21, 23, }, // Samara
    {  477, 1596,   63641,  624405,  624425, 13, 20, 22, }, // Samoa
    {  477, 1602,       0,  624447,       0,  0, 18,  0, }, // Seychelles
    {  477, 1624,       0,  624465,       0,  0, 24,  0, }, // Singapore
    {  477, 1634,       0,  624489,       0,  0, 22,  0, }, // Solomon
    {  477, 1642,       0,  624511,       0,  0, 22,  0, }, // South_Georgia
    {  477, 1656,       0,  624533,       0,  0, 16,  0, }, // Suriname
    {  477, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  477, 1682,       0,  624549,       0,  0, 13,  0, }, // Tahiti
    {  477, 1689,   63834,  624562,  624582, 13, 20, 22, }, // Taipei
    {  477, 1696,       0,  624604,       0,  0, 19,  0, }, // Tajikistan
    {  477, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  477, 1732,   63929,  624623,  624643, 13, 20, 22, }, // Tonga
    {  477, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  477, 1750,  624665,  624687,  624716, 22, 29, 31, }, // Turkmenistan
    {  477, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  477, 1777,  624747,  624762,  624784, 15, 22, 24, }, // Uruguay
    {  477, 1792,  624808,  624827,  624853, 19, 26, 28, }, // Uzbekistan
    {  477, 1803,  624881,  624896,  624918, 15, 22, 24, }, // Vanuatu
    {  477, 1811,       0,  624942,       0,  0, 17,  0, }, // Venezuela
    {  477, 1821,   64318,  624959,  624985, 19, 26, 28, }, // Vladivostok
    {  477, 1833,  625013,  625031,  625056, 18, 25, 27, }, // Volgograd
    {  477, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  477, 1850,       0,  625083,       0,  0, 17,  0, }, // Wake
    {  477, 1855,       0,  625100,       0,  0, 23,  0, }, // Wallis
    {  477, 1862,   64520,  625123,  625145, 15, 22, 24, }, // Yakutsk
    {  477, 1870,  625169,  625190,  625218, 21, 28, 30, }, // Yekaterinburg
    {  477, 1892,       0,  625248,       0,  0, 13,  0, }, // Yukon
    {  478,    0,  618348,  618360,  618379, 12, 19, 21, }, // Acre Portuguese/Latin/Equatorial Guinea
    {  478,    5,       0,  618400,       0,  0, 19,  0, }, // Afghanistan
    {  478,   17,       0,  618419,       0,  0, 22,  0, }, // Africa_Central
    {  478,   32,       0,  618441,       0,  0, 23,  0, }, // Africa_Eastern
    {  478,   65,       0,  618464,       0,  0, 21,  0, }, // Africa_Southern
    {  478,   81,  618485,  618509,  618540, 24, 31, 33, }, // Africa_Western
    {  478,  107,  618573,  618587,  618608, 14, 21, 23, }, // Alaska
    {  478,  128,  618631,  618645,  618666, 14, 21, 23, }, // Almaty
    {  478,  135,  618689,  618705,  618728, 16, 23, 25, }, // Amazon
    {  478,  142,  618753,  618781,  618816, 28, 35, 37, }, // America_Central
    {  478,  158,  618853,  618882,  618918, 29, 36, 38, }, // America_Eastern
    {  478,  174,  618956,  618988,  619027, 32, 39, 41, }, // America_Mountain
    {  478,  191,  619068,  619100,  619139, 32, 39, 41, }, // America_Pacific
    {  478,  207,  619180,  619194,  619215, 14, 21, 23, }, // Anadyr
    {  478,  214,  619238,  619250,  619269, 12, 19, 21, }, // Apia
    {  478,  219,  619290,  619303,  619323, 13, 20, 22, }, // Aqtau
    {  478,  225,  619345,  619359,  619380, 14, 21, 23, }, // Aqtobe
    {  478,  232,  619403,  619417,  619438, 14, 21, 23, }, // Arabian
    {  478,  240,  619461,  619478,  619502, 17, 24, 26, }, // Argentina
    {  478,  250,  619528,  619555,  619589, 27, 34, 36, }, // Argentina_Western
    {  478,  268,  619625,  619640,  619662, 15, 22, 24, }, // Armenia
    {  478,  286,  619686,  619703,  619727, 17, 24, 26, }, // Atlantic
    {  478,  295,  619753,  619778,  619810, 25, 32, 34, }, // Australia_Central
    {  478,  313,  619844,  619879,  619921, 35, 42, 44, }, // Australia_CentralWestern
    {  478,  338,  619965,  619991,  620024, 26, 33, 35, }, // Australia_Eastern
    {  478,  356,  620059,  620086,  620120, 27, 34, 36, }, // Australia_Western
    {  478,  374,  620156,  620174,  620199, 18, 25, 27, }, // Azerbaijan
    {  478,  385,  620226,  620241,  620263, 15, 22, 24, }, // Azores
    {  478,  397,  620287,  620306,  620332, 19, 26, 28, }, // Bangladesh
    {  478,  415,       0,  620360,       0,  0, 13,  0, }, // Bhutan
    {  478,  422,       0,  620373,       0,  0, 15,  0, }, // Bolivia
    {  478,  437,  139322,  620388,  620411, 16, 23, 25, }, // Brasilia
    {  478,  454,       0,  620436,       0,  0, 25,  0, }, // Brunei
    {  478,  461,  620461,  620479,  620504, 18, 25, 27, }, // Cape_Verde
    {  478,  478,       0,  620531,       0,  0, 23,  0, }, // Chamorro
    {  478,  487,   59239,  620554,  620576, 15, 22, 24, }, // Chatham
    {  478,  495,  620600,  620613,  620633, 13, 20, 22, }, // Chile
    {  478,  501,  620655,  620668,  620688, 13, 20, 22, }, // China
    {  478,  507,       0,  620710,       0,  0, 21,  0, }, // Christmas
    {  478,  517,       0,  620731,       0,  0, 20,  0, }, // Cocos
    {  478,  523,  620751,  620767,  620790, 16, 23, 25, }, // Colombia
    {  478,  532,  620815,  620834,  620860, 19, 26, 28, }, // Cook
    {  478,  537,   59634,  620888,  620907, 12, 19, 21, }, // Cuba
    {  478,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  478,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  478,  601,       0,  620928,       0,  0, 19,  0, }, // East_Timor
    {  478,  612,  620947,  620969,  620998, 22, 29, 31, }, // Easter
    {  478,  619,       0,  621029,       0,  0, 15,  0, }, // Ecuador
    {  478,  627,  621044,  621066,  621095, 22, 29, 31, }, // Europe_Central
    {  478,  642,  621126,  621149,  621179, 23, 30, 32, }, // Europe_Eastern
    {  478,  657,       0,  621211,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  478,  680,  621242,  621266,  621297, 24, 31, 33, }, // Europe_Western
    {  478,  695,  621330,  621353,  621383, 23, 30, 32, }, // Falkland
    {  478,  704,  140424,  621415,  621434, 12, 19, 21, }, // Fiji
    {  478,  709,       0,  621455,       0,  0, 23,  0, }, // French_Guiana
    {  478,  723,       0,  621478,       0,  0, 48,  0, }, // French_Southern
    {  478,  746,       0,  621526,       0,  0, 18,  0, }, // Galapagos
    {  478,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  478,  764,  621544,  621559,  621581, 15, 22, 24, }, // Georgia
    {  478,  772,       0,  621605,       0,  0, 22,  0, }, // Gilbert_Islands
    {  478,  788,       0,  621627,       0,  0, 17,  0, }, // GMT
    {  478,  830,  621644,  621672,  621707, 28, 35, 37, }, // Greenland_Eastern
    {  478,  848,  621744,  621773,  621809, 29, 36, 38, }, // Greenland_Western
    {  478,  866,       0,  621847,       0,  0, 19,  0, }, // Guam
    {  478,  871,       0,  621866,       0,  0, 20,  0, }, // Gulf
    {  478,  876,       0,  621455,       0,  0, 14,  0, }, // Guyana
    {  478,  883,  621886,  621909,  621939, 23, 30, 32, }, // Hawaii_Aleutian
    {  478,  899,  141072,  621971,  621995, 17, 24, 26, }, // Hong_Kong
    {  478,  909,   60912,  622021,  622040, 12, 19, 21, }, // Hovd
    {  478,  914,       0,  622061,       0,  0, 20,  0, }, // India
    {  478,  920,       0,  622081,       0,  0, 21,  0, }, // Indian_Ocean
    {  478,  933,       0,  622102,       0,  0, 17,  0, }, // Indochina
    {  478,  943,       0,  622119,       0,  0, 25,  0, }, // Indonesia_Central
    {  478,  961,       0,  622144,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  478,  979,       0,  622170,       0,  0, 27,  0, }, // Indonesia_Western
    {  478,  997,  622197,  622209,  622228, 12, 19, 21, }, // Iran
    {  478, 1008,  622249,  622264,  622286, 15, 22, 24, }, // Irkutsk
    {  478, 1016,  622310,  622324,  622345, 14, 21, 23, }, // Israel
    {  478, 1023,  622368,  622381,  622401, 13, 20, 22, }, // Japan
    {  478, 1029,  622423,  622455,  622494, 32, 39, 41, }, // Kamchatka
    {  478, 1047,       0,  622535,       0,  0, 19,  0, }, // Kazakhstan
    {  478, 1058,       0,  622554,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  478, 1077,       0,  622582,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  478, 1106,  622611,  622625,  622646, 14, 21, 23, }, // Korea
    {  478, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  478, 1119,   61570,  622669,  622695, 19, 26, 28, }, // Krasnoyarsk
    {  478, 1151,       0,  622723,       0,  0, 19,  0, }, // Kyrgystan
    {  478, 1161,       0,  622742,       0,  0, 17,  0, }, // Lanka
    {  478, 1175,       0,  622759,       0,  0, 19,  0, }, // Line_Islands
    {  478, 1188,   61700,  622778,  622802, 17, 24, 26, }, // Lord_Howe
    {  478, 1198,  141929,  622828,  622848, 13, 20, 22, }, // Macau
    {  478, 1204,  141986,  622870,  622892, 15, 22, 24, }, // Magadan
    {  478, 1219,       0,  622916,       0,  0, 15,  0, }, // Malaysia
    {  478, 1228,       0,  622931,       0,  0, 17,  0, }, // Maldives
    {  478, 1237,       0,  622948,       0,  0, 24,  0, }, // Marquesas
    {  478, 1247,       0,  622972,       0,  0, 23,  0, }, // Marshall_Islands
    {  478, 1264,  622995,  623011,  623034, 16, 23, 25, }, // Mauritius
    {  478, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  478, 1281,  623059,  623084,  623116, 25, 32, 34, }, // Mexico_Pacific
    {  478, 1296,  623150,  623168,  623193, 18, 25, 27, }, // Mongolia
    {  478, 1305,  623220,  623235,  623257, 15, 22, 24, }, // Moscow
    {  478, 1312,       0,  623281,       0,  0, 15,  0, }, // Myanmar
    {  478, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  478, 1326,       0,  623296,       0,  0, 13,  0, }, // Nepal
    {  478, 1332,  623309,  623331,  623360, 22, 29, 31, }, // New_Caledonia
    {  478, 1346,  623391,  623412,  623440, 21, 28, 30, }, // New_Zealand
    {  478, 1358,  623470,  623488,  623513, 18, 25, 27, }, // Newfoundland
    {  478, 1371,       0,  623540,       0,  0, 12,  0, }, // Niue
    {  478, 1376,  623552,  623572,  623599, 20, 27, 29, }, // Norfolk
    {  478, 1384,   62667,  623628,  623662, 27, 34, 36, }, // Noronha
    {  478, 1392,       0,  623698,       0,  0, 31,  0, }, // North_Mariana
    {  478, 1406,   62803,  623729,  623755, 19, 26, 28, }, // Novosibirsk
    {  478, 1418,  623783,  623795,  623814, 12, 19, 21, }, // Omsk
    {  478, 1428,  623835,  623852,  623876, 17, 24, 26, }, // Pakistan
    {  478, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  478, 1443,       0,  623902,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  478, 1460,  623926,  623942,  623965, 16, 23, 25, }, // Paraguay
    {  478, 1469,  623990,  624002,  624021, 12, 19, 21, }, // Peru
    {  478, 1474,  624042,  624060,  624085, 18, 25, 27, }, // Philippines
    {  478, 1486,       0,  624112,       0,  0, 20,  0, }, // Phoenix_Islands
    {  478, 1502,  624132,  624160,  624195, 28, 35, 37, }, // Pierre_Miquelon
    {  478, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  478, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {  478, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {  478, 1544,   63419,  624232,  624256, 17, 24, 26, }, // Qyzylorda
    {  478, 1554,       0,  624282,       0,  0, 15,  0, }, // Reunion
    {  478, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  478, 1570,  624297,  624313,  624336, 16, 23, 25, }, // Sakhalin
    {  478, 1579,   63581,  624361,  624382, 14, 21, 23, }, // Samara
    {  478, 1596,   63641,  624405,  624425, 13, 20, 22, }, // Samoa
    {  478, 1602,       0,  624447,       0,  0, 18,  0, }, // Seychelles
    {  478, 1624,       0,  624465,       0,  0, 24,  0, }, // Singapore
    {  478, 1634,       0,  624489,       0,  0, 22,  0, }, // Solomon
    {  478, 1642,       0,  624511,       0,  0, 22,  0, }, // South_Georgia
    {  478, 1656,       0,  624533,       0,  0, 16,  0, }, // Suriname
    {  478, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  478, 1682,       0,  624549,       0,  0, 13,  0, }, // Tahiti
    {  478, 1689,   63834,  624562,  624582, 13, 20, 22, }, // Taipei
    {  478, 1696,       0,  624604,       0,  0, 19,  0, }, // Tajikistan
    {  478, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  478, 1732,   63929,  624623,  624643, 13, 20, 22, }, // Tonga
    {  478, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  478, 1750,  624665,  624687,  624716, 22, 29, 31, }, // Turkmenistan
    {  478, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  478, 1777,  624747,  624762,  624784, 15, 22, 24, }, // Uruguay
    {  478, 1792,  624808,  624827,  624853, 19, 26, 28, }, // Uzbekistan
    {  478, 1803,  624881,  624896,  624918, 15, 22, 24, }, // Vanuatu
    {  478, 1811,       0,  624942,       0,  0, 17,  0, }, // Venezuela
    {  478, 1821,   64318,  624959,  624985, 19, 26, 28, }, // Vladivostok
    {  478, 1833,  625013,  625031,  625056, 18, 25, 27, }, // Volgograd
    {  478, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  478, 1850,       0,  625083,       0,  0, 17,  0, }, // Wake
    {  478, 1855,       0,  625100,       0,  0, 23,  0, }, // Wallis
    {  478, 1862,   64520,  625123,  625145, 15, 22, 24, }, // Yakutsk
    {  478, 1870,  625169,  625190,  625218, 21, 28, 30, }, // Yekaterinburg
    {  478, 1892,       0,  625248,       0,  0, 13,  0, }, // Yukon
    {  479,    0,  618348,  618360,  618379, 12, 19, 21, }, // Acre Portuguese/Latin/Guinea-Bissau
    {  479,    5,       0,  618400,       0,  0, 19,  0, }, // Afghanistan
    {  479,   17,       0,  618419,       0,  0, 22,  0, }, // Africa_Central
    {  479,   32,       0,  618441,       0,  0, 23,  0, }, // Africa_Eastern
    {  479,   65,       0,  618464,       0,  0, 21,  0, }, // Africa_Southern
    {  479,   81,  618485,  618509,  618540, 24, 31, 33, }, // Africa_Western
    {  479,  107,  618573,  618587,  618608, 14, 21, 23, }, // Alaska
    {  479,  128,  618631,  618645,  618666, 14, 21, 23, }, // Almaty
    {  479,  135,  618689,  618705,  618728, 16, 23, 25, }, // Amazon
    {  479,  142,  618753,  618781,  618816, 28, 35, 37, }, // America_Central
    {  479,  158,  618853,  618882,  618918, 29, 36, 38, }, // America_Eastern
    {  479,  174,  618956,  618988,  619027, 32, 39, 41, }, // America_Mountain
    {  479,  191,  619068,  619100,  619139, 32, 39, 41, }, // America_Pacific
    {  479,  207,  619180,  619194,  619215, 14, 21, 23, }, // Anadyr
    {  479,  214,  619238,  619250,  619269, 12, 19, 21, }, // Apia
    {  479,  219,  619290,  619303,  619323, 13, 20, 22, }, // Aqtau
    {  479,  225,  619345,  619359,  619380, 14, 21, 23, }, // Aqtobe
    {  479,  232,  619403,  619417,  619438, 14, 21, 23, }, // Arabian
    {  479,  240,  619461,  619478,  619502, 17, 24, 26, }, // Argentina
    {  479,  250,  619528,  619555,  619589, 27, 34, 36, }, // Argentina_Western
    {  479,  268,  619625,  619640,  619662, 15, 22, 24, }, // Armenia
    {  479,  286,  619686,  619703,  619727, 17, 24, 26, }, // Atlantic
    {  479,  295,  619753,  619778,  619810, 25, 32, 34, }, // Australia_Central
    {  479,  313,  619844,  619879,  619921, 35, 42, 44, }, // Australia_CentralWestern
    {  479,  338,  619965,  619991,  620024, 26, 33, 35, }, // Australia_Eastern
    {  479,  356,  620059,  620086,  620120, 27, 34, 36, }, // Australia_Western
    {  479,  374,  620156,  620174,  620199, 18, 25, 27, }, // Azerbaijan
    {  479,  385,  620226,  620241,  620263, 15, 22, 24, }, // Azores
    {  479,  397,  620287,  620306,  620332, 19, 26, 28, }, // Bangladesh
    {  479,  415,       0,  620360,       0,  0, 13,  0, }, // Bhutan
    {  479,  422,       0,  620373,       0,  0, 15,  0, }, // Bolivia
    {  479,  437,  139322,  620388,  620411, 16, 23, 25, }, // Brasilia
    {  479,  454,       0,  620436,       0,  0, 25,  0, }, // Brunei
    {  479,  461,  620461,  620479,  620504, 18, 25, 27, }, // Cape_Verde
    {  479,  478,       0,  620531,       0,  0, 23,  0, }, // Chamorro
    {  479,  487,   59239,  620554,  620576, 15, 22, 24, }, // Chatham
    {  479,  495,  620600,  620613,  620633, 13, 20, 22, }, // Chile
    {  479,  501,  620655,  620668,  620688, 13, 20, 22, }, // China
    {  479,  507,       0,  620710,       0,  0, 21,  0, }, // Christmas
    {  479,  517,       0,  620731,       0,  0, 20,  0, }, // Cocos
    {  479,  523,  620751,  620767,  620790, 16, 23, 25, }, // Colombia
    {  479,  532,  620815,  620834,  620860, 19, 26, 28, }, // Cook
    {  479,  537,   59634,  620888,  620907, 12, 19, 21, }, // Cuba
    {  479,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  479,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  479,  601,       0,  620928,       0,  0, 19,  0, }, // East_Timor
    {  479,  612,  620947,  620969,  620998, 22, 29, 31, }, // Easter
    {  479,  619,       0,  621029,       0,  0, 15,  0, }, // Ecuador
    {  479,  627,  621044,  621066,  621095, 22, 29, 31, }, // Europe_Central
    {  479,  642,  621126,  621149,  621179, 23, 30, 32, }, // Europe_Eastern
    {  479,  657,       0,  621211,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  479,  680,  621242,  621266,  621297, 24, 31, 33, }, // Europe_Western
    {  479,  695,  621330,  621353,  621383, 23, 30, 32, }, // Falkland
    {  479,  704,  140424,  621415,  621434, 12, 19, 21, }, // Fiji
    {  479,  709,       0,  621455,       0,  0, 23,  0, }, // French_Guiana
    {  479,  723,       0,  621478,       0,  0, 48,  0, }, // French_Southern
    {  479,  746,       0,  621526,       0,  0, 18,  0, }, // Galapagos
    {  479,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  479,  764,  621544,  621559,  621581, 15, 22, 24, }, // Georgia
    {  479,  772,       0,  621605,       0,  0, 22,  0, }, // Gilbert_Islands
    {  479,  788,       0,  621627,       0,  0, 17,  0, }, // GMT
    {  479,  830,  621644,  621672,  621707, 28, 35, 37, }, // Greenland_Eastern
    {  479,  848,  621744,  621773,  621809, 29, 36, 38, }, // Greenland_Western
    {  479,  866,       0,  621847,       0,  0, 19,  0, }, // Guam
    {  479,  871,       0,  621866,       0,  0, 20,  0, }, // Gulf
    {  479,  876,       0,  621455,       0,  0, 14,  0, }, // Guyana
    {  479,  883,  621886,  621909,  621939, 23, 30, 32, }, // Hawaii_Aleutian
    {  479,  899,  141072,  621971,  621995, 17, 24, 26, }, // Hong_Kong
    {  479,  909,   60912,  622021,  622040, 12, 19, 21, }, // Hovd
    {  479,  914,       0,  622061,       0,  0, 20,  0, }, // India
    {  479,  920,       0,  622081,       0,  0, 21,  0, }, // Indian_Ocean
    {  479,  933,       0,  622102,       0,  0, 17,  0, }, // Indochina
    {  479,  943,       0,  622119,       0,  0, 25,  0, }, // Indonesia_Central
    {  479,  961,       0,  622144,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  479,  979,       0,  622170,       0,  0, 27,  0, }, // Indonesia_Western
    {  479,  997,  622197,  622209,  622228, 12, 19, 21, }, // Iran
    {  479, 1008,  622249,  622264,  622286, 15, 22, 24, }, // Irkutsk
    {  479, 1016,  622310,  622324,  622345, 14, 21, 23, }, // Israel
    {  479, 1023,  622368,  622381,  622401, 13, 20, 22, }, // Japan
    {  479, 1029,  622423,  622455,  622494, 32, 39, 41, }, // Kamchatka
    {  479, 1047,       0,  622535,       0,  0, 19,  0, }, // Kazakhstan
    {  479, 1058,       0,  622554,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  479, 1077,       0,  622582,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  479, 1106,  622611,  622625,  622646, 14, 21, 23, }, // Korea
    {  479, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  479, 1119,   61570,  622669,  622695, 19, 26, 28, }, // Krasnoyarsk
    {  479, 1151,       0,  622723,       0,  0, 19,  0, }, // Kyrgystan
    {  479, 1161,       0,  622742,       0,  0, 17,  0, }, // Lanka
    {  479, 1175,       0,  622759,       0,  0, 19,  0, }, // Line_Islands
    {  479, 1188,   61700,  622778,  622802, 17, 24, 26, }, // Lord_Howe
    {  479, 1198,  141929,  622828,  622848, 13, 20, 22, }, // Macau
    {  479, 1204,  141986,  622870,  622892, 15, 22, 24, }, // Magadan
    {  479, 1219,       0,  622916,       0,  0, 15,  0, }, // Malaysia
    {  479, 1228,       0,  622931,       0,  0, 17,  0, }, // Maldives
    {  479, 1237,       0,  622948,       0,  0, 24,  0, }, // Marquesas
    {  479, 1247,       0,  622972,       0,  0, 23,  0, }, // Marshall_Islands
    {  479, 1264,  622995,  623011,  623034, 16, 23, 25, }, // Mauritius
    {  479, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  479, 1281,  623059,  623084,  623116, 25, 32, 34, }, // Mexico_Pacific
    {  479, 1296,  623150,  623168,  623193, 18, 25, 27, }, // Mongolia
    {  479, 1305,  623220,  623235,  623257, 15, 22, 24, }, // Moscow
    {  479, 1312,       0,  623281,       0,  0, 15,  0, }, // Myanmar
    {  479, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  479, 1326,       0,  623296,       0,  0, 13,  0, }, // Nepal
    {  479, 1332,  623309,  623331,  623360, 22, 29, 31, }, // New_Caledonia
    {  479, 1346,  623391,  623412,  623440, 21, 28, 30, }, // New_Zealand
    {  479, 1358,  623470,  623488,  623513, 18, 25, 27, }, // Newfoundland
    {  479, 1371,       0,  623540,       0,  0, 12,  0, }, // Niue
    {  479, 1376,  623552,  623572,  623599, 20, 27, 29, }, // Norfolk
    {  479, 1384,   62667,  623628,  623662, 27, 34, 36, }, // Noronha
    {  479, 1392,       0,  623698,       0,  0, 31,  0, }, // North_Mariana
    {  479, 1406,   62803,  623729,  623755, 19, 26, 28, }, // Novosibirsk
    {  479, 1418,  623783,  623795,  623814, 12, 19, 21, }, // Omsk
    {  479, 1428,  623835,  623852,  623876, 17, 24, 26, }, // Pakistan
    {  479, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  479, 1443,       0,  623902,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  479, 1460,  623926,  623942,  623965, 16, 23, 25, }, // Paraguay
    {  479, 1469,  623990,  624002,  624021, 12, 19, 21, }, // Peru
    {  479, 1474,  624042,  624060,  624085, 18, 25, 27, }, // Philippines
    {  479, 1486,       0,  624112,       0,  0, 20,  0, }, // Phoenix_Islands
    {  479, 1502,  624132,  624160,  624195, 28, 35, 37, }, // Pierre_Miquelon
    {  479, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  479, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {  479, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {  479, 1544,   63419,  624232,  624256, 17, 24, 26, }, // Qyzylorda
    {  479, 1554,       0,  624282,       0,  0, 15,  0, }, // Reunion
    {  479, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  479, 1570,  624297,  624313,  624336, 16, 23, 25, }, // Sakhalin
    {  479, 1579,   63581,  624361,  624382, 14, 21, 23, }, // Samara
    {  479, 1596,   63641,  624405,  624425, 13, 20, 22, }, // Samoa
    {  479, 1602,       0,  624447,       0,  0, 18,  0, }, // Seychelles
    {  479, 1624,       0,  624465,       0,  0, 24,  0, }, // Singapore
    {  479, 1634,       0,  624489,       0,  0, 22,  0, }, // Solomon
    {  479, 1642,       0,  624511,       0,  0, 22,  0, }, // South_Georgia
    {  479, 1656,       0,  624533,       0,  0, 16,  0, }, // Suriname
    {  479, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  479, 1682,       0,  624549,       0,  0, 13,  0, }, // Tahiti
    {  479, 1689,   63834,  624562,  624582, 13, 20, 22, }, // Taipei
    {  479, 1696,       0,  624604,       0,  0, 19,  0, }, // Tajikistan
    {  479, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  479, 1732,   63929,  624623,  624643, 13, 20, 22, }, // Tonga
    {  479, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  479, 1750,  624665,  624687,  624716, 22, 29, 31, }, // Turkmenistan
    {  479, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  479, 1777,  624747,  624762,  624784, 15, 22, 24, }, // Uruguay
    {  479, 1792,  624808,  624827,  624853, 19, 26, 28, }, // Uzbekistan
    {  479, 1803,  624881,  624896,  624918, 15, 22, 24, }, // Vanuatu
    {  479, 1811,       0,  624942,       0,  0, 17,  0, }, // Venezuela
    {  479, 1821,   64318,  624959,  624985, 19, 26, 28, }, // Vladivostok
    {  479, 1833,  625013,  625031,  625056, 18, 25, 27, }, // Volgograd
    {  479, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  479, 1850,       0,  625083,       0,  0, 17,  0, }, // Wake
    {  479, 1855,       0,  625100,       0,  0, 23,  0, }, // Wallis
    {  479, 1862,   64520,  625123,  625145, 15, 22, 24, }, // Yakutsk
    {  479, 1870,  625169,  625190,  625218, 21, 28, 30, }, // Yekaterinburg
    {  479, 1892,       0,  625248,       0,  0, 13,  0, }, // Yukon
    {  480,    0,  618348,  618360,  618379, 12, 19, 21, }, // Acre Portuguese/Latin/Luxembourg
    {  480,    5,       0,  618400,       0,  0, 19,  0, }, // Afghanistan
    {  480,   17,       0,  618419,       0,  0, 22,  0, }, // Africa_Central
    {  480,   32,       0,  618441,       0,  0, 23,  0, }, // Africa_Eastern
    {  480,   65,       0,  618464,       0,  0, 21,  0, }, // Africa_Southern
    {  480,   81,  618485,  618509,  618540, 24, 31, 33, }, // Africa_Western
    {  480,  107,  618573,  618587,  618608, 14, 21, 23, }, // Alaska
    {  480,  128,  618631,  618645,  618666, 14, 21, 23, }, // Almaty
    {  480,  135,  618689,  618705,  618728, 16, 23, 25, }, // Amazon
    {  480,  142,  618753,  618781,  618816, 28, 35, 37, }, // America_Central
    {  480,  158,  618853,  618882,  618918, 29, 36, 38, }, // America_Eastern
    {  480,  174,  618956,  618988,  619027, 32, 39, 41, }, // America_Mountain
    {  480,  191,  619068,  619100,  619139, 32, 39, 41, }, // America_Pacific
    {  480,  207,  619180,  619194,  619215, 14, 21, 23, }, // Anadyr
    {  480,  214,  619238,  619250,  619269, 12, 19, 21, }, // Apia
    {  480,  219,  619290,  619303,  619323, 13, 20, 22, }, // Aqtau
    {  480,  225,  619345,  619359,  619380, 14, 21, 23, }, // Aqtobe
    {  480,  232,  619403,  619417,  619438, 14, 21, 23, }, // Arabian
    {  480,  240,  619461,  619478,  619502, 17, 24, 26, }, // Argentina
    {  480,  250,  619528,  619555,  619589, 27, 34, 36, }, // Argentina_Western
    {  480,  268,  619625,  619640,  619662, 15, 22, 24, }, // Armenia
    {  480,  286,  619686,  619703,  619727, 17, 24, 26, }, // Atlantic
    {  480,  295,  619753,  619778,  619810, 25, 32, 34, }, // Australia_Central
    {  480,  313,  619844,  619879,  619921, 35, 42, 44, }, // Australia_CentralWestern
    {  480,  338,  619965,  619991,  620024, 26, 33, 35, }, // Australia_Eastern
    {  480,  356,  620059,  620086,  620120, 27, 34, 36, }, // Australia_Western
    {  480,  374,  620156,  620174,  620199, 18, 25, 27, }, // Azerbaijan
    {  480,  385,  620226,  620241,  620263, 15, 22, 24, }, // Azores
    {  480,  397,  620287,  620306,  620332, 19, 26, 28, }, // Bangladesh
    {  480,  415,       0,  620360,       0,  0, 13,  0, }, // Bhutan
    {  480,  422,       0,  620373,       0,  0, 15,  0, }, // Bolivia
    {  480,  437,  139322,  620388,  620411, 16, 23, 25, }, // Brasilia
    {  480,  454,       0,  620436,       0,  0, 25,  0, }, // Brunei
    {  480,  461,  620461,  620479,  620504, 18, 25, 27, }, // Cape_Verde
    {  480,  478,       0,  620531,       0,  0, 23,  0, }, // Chamorro
    {  480,  487,   59239,  620554,  620576, 15, 22, 24, }, // Chatham
    {  480,  495,  620600,  620613,  620633, 13, 20, 22, }, // Chile
    {  480,  501,  620655,  620668,  620688, 13, 20, 22, }, // China
    {  480,  507,       0,  620710,       0,  0, 21,  0, }, // Christmas
    {  480,  517,       0,  620731,       0,  0, 20,  0, }, // Cocos
    {  480,  523,  620751,  620767,  620790, 16, 23, 25, }, // Colombia
    {  480,  532,  620815,  620834,  620860, 19, 26, 28, }, // Cook
    {  480,  537,   59634,  620888,  620907, 12, 19, 21, }, // Cuba
    {  480,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  480,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  480,  601,       0,  620928,       0,  0, 19,  0, }, // East_Timor
    {  480,  612,  620947,  620969,  620998, 22, 29, 31, }, // Easter
    {  480,  619,       0,  621029,       0,  0, 15,  0, }, // Ecuador
    {  480,  627,  621044,  621066,  621095, 22, 29, 31, }, // Europe_Central
    {  480,  642,  621126,  621149,  621179, 23, 30, 32, }, // Europe_Eastern
    {  480,  657,       0,  621211,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  480,  680,  621242,  621266,  621297, 24, 31, 33, }, // Europe_Western
    {  480,  695,  621330,  621353,  621383, 23, 30, 32, }, // Falkland
    {  480,  704,  140424,  621415,  621434, 12, 19, 21, }, // Fiji
    {  480,  709,       0,  621455,       0,  0, 23,  0, }, // French_Guiana
    {  480,  723,       0,  621478,       0,  0, 48,  0, }, // French_Southern
    {  480,  746,       0,  621526,       0,  0, 18,  0, }, // Galapagos
    {  480,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  480,  764,  621544,  621559,  621581, 15, 22, 24, }, // Georgia
    {  480,  772,       0,  621605,       0,  0, 22,  0, }, // Gilbert_Islands
    {  480,  788,       0,  621627,       0,  0, 17,  0, }, // GMT
    {  480,  830,  621644,  621672,  621707, 28, 35, 37, }, // Greenland_Eastern
    {  480,  848,  621744,  621773,  621809, 29, 36, 38, }, // Greenland_Western
    {  480,  866,       0,  621847,       0,  0, 19,  0, }, // Guam
    {  480,  871,       0,  621866,       0,  0, 20,  0, }, // Gulf
    {  480,  876,       0,  621455,       0,  0, 14,  0, }, // Guyana
    {  480,  883,  621886,  621909,  621939, 23, 30, 32, }, // Hawaii_Aleutian
    {  480,  899,  141072,  621971,  621995, 17, 24, 26, }, // Hong_Kong
    {  480,  909,   60912,  622021,  622040, 12, 19, 21, }, // Hovd
    {  480,  914,       0,  622061,       0,  0, 20,  0, }, // India
    {  480,  920,       0,  622081,       0,  0, 21,  0, }, // Indian_Ocean
    {  480,  933,       0,  622102,       0,  0, 17,  0, }, // Indochina
    {  480,  943,       0,  622119,       0,  0, 25,  0, }, // Indonesia_Central
    {  480,  961,       0,  622144,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  480,  979,       0,  622170,       0,  0, 27,  0, }, // Indonesia_Western
    {  480,  997,  622197,  622209,  622228, 12, 19, 21, }, // Iran
    {  480, 1008,  622249,  622264,  622286, 15, 22, 24, }, // Irkutsk
    {  480, 1016,  622310,  622324,  622345, 14, 21, 23, }, // Israel
    {  480, 1023,  622368,  622381,  622401, 13, 20, 22, }, // Japan
    {  480, 1029,  622423,  622455,  622494, 32, 39, 41, }, // Kamchatka
    {  480, 1047,       0,  622535,       0,  0, 19,  0, }, // Kazakhstan
    {  480, 1058,       0,  622554,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  480, 1077,       0,  622582,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  480, 1106,  622611,  622625,  622646, 14, 21, 23, }, // Korea
    {  480, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  480, 1119,   61570,  622669,  622695, 19, 26, 28, }, // Krasnoyarsk
    {  480, 1151,       0,  622723,       0,  0, 19,  0, }, // Kyrgystan
    {  480, 1161,       0,  622742,       0,  0, 17,  0, }, // Lanka
    {  480, 1175,       0,  622759,       0,  0, 19,  0, }, // Line_Islands
    {  480, 1188,   61700,  622778,  622802, 17, 24, 26, }, // Lord_Howe
    {  480, 1198,  141929,  622828,  622848, 13, 20, 22, }, // Macau
    {  480, 1204,  141986,  622870,  622892, 15, 22, 24, }, // Magadan
    {  480, 1219,       0,  622916,       0,  0, 15,  0, }, // Malaysia
    {  480, 1228,       0,  622931,       0,  0, 17,  0, }, // Maldives
    {  480, 1237,       0,  622948,       0,  0, 24,  0, }, // Marquesas
    {  480, 1247,       0,  622972,       0,  0, 23,  0, }, // Marshall_Islands
    {  480, 1264,  622995,  623011,  623034, 16, 23, 25, }, // Mauritius
    {  480, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  480, 1281,  623059,  623084,  623116, 25, 32, 34, }, // Mexico_Pacific
    {  480, 1296,  623150,  623168,  623193, 18, 25, 27, }, // Mongolia
    {  480, 1305,  623220,  623235,  623257, 15, 22, 24, }, // Moscow
    {  480, 1312,       0,  623281,       0,  0, 15,  0, }, // Myanmar
    {  480, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  480, 1326,       0,  623296,       0,  0, 13,  0, }, // Nepal
    {  480, 1332,  623309,  623331,  623360, 22, 29, 31, }, // New_Caledonia
    {  480, 1346,  623391,  623412,  623440, 21, 28, 30, }, // New_Zealand
    {  480, 1358,  623470,  623488,  623513, 18, 25, 27, }, // Newfoundland
    {  480, 1371,       0,  623540,       0,  0, 12,  0, }, // Niue
    {  480, 1376,  623552,  623572,  623599, 20, 27, 29, }, // Norfolk
    {  480, 1384,   62667,  623628,  623662, 27, 34, 36, }, // Noronha
    {  480, 1392,       0,  623698,       0,  0, 31,  0, }, // North_Mariana
    {  480, 1406,   62803,  623729,  623755, 19, 26, 28, }, // Novosibirsk
    {  480, 1418,  623783,  623795,  623814, 12, 19, 21, }, // Omsk
    {  480, 1428,  623835,  623852,  623876, 17, 24, 26, }, // Pakistan
    {  480, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  480, 1443,       0,  623902,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  480, 1460,  623926,  623942,  623965, 16, 23, 25, }, // Paraguay
    {  480, 1469,  623990,  624002,  624021, 12, 19, 21, }, // Peru
    {  480, 1474,  624042,  624060,  624085, 18, 25, 27, }, // Philippines
    {  480, 1486,       0,  624112,       0,  0, 20,  0, }, // Phoenix_Islands
    {  480, 1502,  624132,  624160,  624195, 28, 35, 37, }, // Pierre_Miquelon
    {  480, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  480, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {  480, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {  480, 1544,   63419,  624232,  624256, 17, 24, 26, }, // Qyzylorda
    {  480, 1554,       0,  624282,       0,  0, 15,  0, }, // Reunion
    {  480, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  480, 1570,  624297,  624313,  624336, 16, 23, 25, }, // Sakhalin
    {  480, 1579,   63581,  624361,  624382, 14, 21, 23, }, // Samara
    {  480, 1596,   63641,  624405,  624425, 13, 20, 22, }, // Samoa
    {  480, 1602,       0,  624447,       0,  0, 18,  0, }, // Seychelles
    {  480, 1624,       0,  624465,       0,  0, 24,  0, }, // Singapore
    {  480, 1634,       0,  624489,       0,  0, 22,  0, }, // Solomon
    {  480, 1642,       0,  624511,       0,  0, 22,  0, }, // South_Georgia
    {  480, 1656,       0,  624533,       0,  0, 16,  0, }, // Suriname
    {  480, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  480, 1682,       0,  624549,       0,  0, 13,  0, }, // Tahiti
    {  480, 1689,   63834,  624562,  624582, 13, 20, 22, }, // Taipei
    {  480, 1696,       0,  624604,       0,  0, 19,  0, }, // Tajikistan
    {  480, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  480, 1732,   63929,  624623,  624643, 13, 20, 22, }, // Tonga
    {  480, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  480, 1750,  624665,  624687,  624716, 22, 29, 31, }, // Turkmenistan
    {  480, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  480, 1777,  624747,  624762,  624784, 15, 22, 24, }, // Uruguay
    {  480, 1792,  624808,  624827,  624853, 19, 26, 28, }, // Uzbekistan
    {  480, 1803,  624881,  624896,  624918, 15, 22, 24, }, // Vanuatu
    {  480, 1811,       0,  624942,       0,  0, 17,  0, }, // Venezuela
    {  480, 1821,   64318,  624959,  624985, 19, 26, 28, }, // Vladivostok
    {  480, 1833,  625013,  625031,  625056, 18, 25, 27, }, // Volgograd
    {  480, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  480, 1850,       0,  625083,       0,  0, 17,  0, }, // Wake
    {  480, 1855,       0,  625100,       0,  0, 23,  0, }, // Wallis
    {  480, 1862,   64520,  625123,  625145, 15, 22, 24, }, // Yakutsk
    {  480, 1870,  625169,  625190,  625218, 21, 28, 30, }, // Yekaterinburg
    {  480, 1892,       0,  625248,       0,  0, 13,  0, }, // Yukon
    {  481,    0,  618348,  618360,  618379, 12, 19, 21, }, // Acre Portuguese/Latin/Macao
    {  481,    5,       0,  618400,       0,  0, 19,  0, }, // Afghanistan
    {  481,   17,       0,  618419,       0,  0, 22,  0, }, // Africa_Central
    {  481,   32,       0,  618441,       0,  0, 23,  0, }, // Africa_Eastern
    {  481,   65,       0,  618464,       0,  0, 21,  0, }, // Africa_Southern
    {  481,   81,  618485,  618509,  618540, 24, 31, 33, }, // Africa_Western
    {  481,  107,  618573,  618587,  618608, 14, 21, 23, }, // Alaska
    {  481,  128,  618631,  618645,  618666, 14, 21, 23, }, // Almaty
    {  481,  135,  618689,  618705,  618728, 16, 23, 25, }, // Amazon
    {  481,  142,  618753,  618781,  618816, 28, 35, 37, }, // America_Central
    {  481,  158,  618853,  618882,  618918, 29, 36, 38, }, // America_Eastern
    {  481,  174,  618956,  618988,  619027, 32, 39, 41, }, // America_Mountain
    {  481,  191,  619068,  619100,  619139, 32, 39, 41, }, // America_Pacific
    {  481,  207,  619180,  619194,  619215, 14, 21, 23, }, // Anadyr
    {  481,  214,  619238,  619250,  619269, 12, 19, 21, }, // Apia
    {  481,  219,  619290,  619303,  619323, 13, 20, 22, }, // Aqtau
    {  481,  225,  619345,  619359,  619380, 14, 21, 23, }, // Aqtobe
    {  481,  232,  619403,  619417,  619438, 14, 21, 23, }, // Arabian
    {  481,  240,  619461,  619478,  619502, 17, 24, 26, }, // Argentina
    {  481,  250,  619528,  619555,  619589, 27, 34, 36, }, // Argentina_Western
    {  481,  268,  619625,  619640,  619662, 15, 22, 24, }, // Armenia
    {  481,  286,  619686,  619703,  619727, 17, 24, 26, }, // Atlantic
    {  481,  295,  619753,  619778,  619810, 25, 32, 34, }, // Australia_Central
    {  481,  313,  619844,  619879,  619921, 35, 42, 44, }, // Australia_CentralWestern
    {  481,  338,  619965,  619991,  620024, 26, 33, 35, }, // Australia_Eastern
    {  481,  356,  620059,  620086,  620120, 27, 34, 36, }, // Australia_Western
    {  481,  374,  620156,  620174,  620199, 18, 25, 27, }, // Azerbaijan
    {  481,  385,  620226,  620241,  620263, 15, 22, 24, }, // Azores
    {  481,  397,  620287,  620306,  620332, 19, 26, 28, }, // Bangladesh
    {  481,  415,       0,  620360,       0,  0, 13,  0, }, // Bhutan
    {  481,  422,       0,  620373,       0,  0, 15,  0, }, // Bolivia
    {  481,  437,  139322,  620388,  620411, 16, 23, 25, }, // Brasilia
    {  481,  454,       0,  620436,       0,  0, 25,  0, }, // Brunei
    {  481,  461,  620461,  620479,  620504, 18, 25, 27, }, // Cape_Verde
    {  481,  478,       0,  620531,       0,  0, 23,  0, }, // Chamorro
    {  481,  487,   59239,  620554,  620576, 15, 22, 24, }, // Chatham
    {  481,  495,  620600,  620613,  620633, 13, 20, 22, }, // Chile
    {  481,  501,  620655,  620668,  620688, 13, 20, 22, }, // China
    {  481,  507,       0,  620710,       0,  0, 21,  0, }, // Christmas
    {  481,  517,       0,  620731,       0,  0, 20,  0, }, // Cocos
    {  481,  523,  620751,  620767,  620790, 16, 23, 25, }, // Colombia
    {  481,  532,  620815,  620834,  620860, 19, 26, 28, }, // Cook
    {  481,  537,   59634,  620888,  620907, 12, 19, 21, }, // Cuba
    {  481,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  481,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  481,  601,       0,  620928,       0,  0, 19,  0, }, // East_Timor
    {  481,  612,  620947,  620969,  620998, 22, 29, 31, }, // Easter
    {  481,  619,       0,  621029,       0,  0, 15,  0, }, // Ecuador
    {  481,  627,  621044,  621066,  621095, 22, 29, 31, }, // Europe_Central
    {  481,  642,  621126,  621149,  621179, 23, 30, 32, }, // Europe_Eastern
    {  481,  657,       0,  621211,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  481,  680,  621242,  621266,  621297, 24, 31, 33, }, // Europe_Western
    {  481,  695,  621330,  621353,  621383, 23, 30, 32, }, // Falkland
    {  481,  704,  140424,  621415,  621434, 12, 19, 21, }, // Fiji
    {  481,  709,       0,  621455,       0,  0, 23,  0, }, // French_Guiana
    {  481,  723,       0,  621478,       0,  0, 48,  0, }, // French_Southern
    {  481,  746,       0,  621526,       0,  0, 18,  0, }, // Galapagos
    {  481,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  481,  764,  621544,  621559,  621581, 15, 22, 24, }, // Georgia
    {  481,  772,       0,  621605,       0,  0, 22,  0, }, // Gilbert_Islands
    {  481,  788,       0,  621627,       0,  0, 17,  0, }, // GMT
    {  481,  830,  621644,  621672,  621707, 28, 35, 37, }, // Greenland_Eastern
    {  481,  848,  621744,  621773,  621809, 29, 36, 38, }, // Greenland_Western
    {  481,  866,       0,  621847,       0,  0, 19,  0, }, // Guam
    {  481,  871,       0,  621866,       0,  0, 20,  0, }, // Gulf
    {  481,  876,       0,  621455,       0,  0, 14,  0, }, // Guyana
    {  481,  883,  621886,  621909,  621939, 23, 30, 32, }, // Hawaii_Aleutian
    {  481,  899,  141072,  621971,  621995, 17, 24, 26, }, // Hong_Kong
    {  481,  909,   60912,  622021,  622040, 12, 19, 21, }, // Hovd
    {  481,  914,       0,  622061,       0,  0, 20,  0, }, // India
    {  481,  920,       0,  622081,       0,  0, 21,  0, }, // Indian_Ocean
    {  481,  933,       0,  622102,       0,  0, 17,  0, }, // Indochina
    {  481,  943,       0,  622119,       0,  0, 25,  0, }, // Indonesia_Central
    {  481,  961,       0,  622144,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  481,  979,       0,  622170,       0,  0, 27,  0, }, // Indonesia_Western
    {  481,  997,  622197,  622209,  622228, 12, 19, 21, }, // Iran
    {  481, 1008,  622249,  622264,  622286, 15, 22, 24, }, // Irkutsk
    {  481, 1016,  622310,  622324,  622345, 14, 21, 23, }, // Israel
    {  481, 1023,  622368,  622381,  622401, 13, 20, 22, }, // Japan
    {  481, 1029,  622423,  622455,  622494, 32, 39, 41, }, // Kamchatka
    {  481, 1047,       0,  622535,       0,  0, 19,  0, }, // Kazakhstan
    {  481, 1058,       0,  622554,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  481, 1077,       0,  622582,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  481, 1106,  622611,  622625,  622646, 14, 21, 23, }, // Korea
    {  481, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  481, 1119,   61570,  622669,  622695, 19, 26, 28, }, // Krasnoyarsk
    {  481, 1151,       0,  622723,       0,  0, 19,  0, }, // Kyrgystan
    {  481, 1161,       0,  622742,       0,  0, 17,  0, }, // Lanka
    {  481, 1175,       0,  622759,       0,  0, 19,  0, }, // Line_Islands
    {  481, 1188,   61700,  622778,  622802, 17, 24, 26, }, // Lord_Howe
    {  481, 1198,  141929,  622828,  622848, 13, 20, 22, }, // Macau
    {  481, 1204,  141986,  622870,  622892, 15, 22, 24, }, // Magadan
    {  481, 1219,       0,  622916,       0,  0, 15,  0, }, // Malaysia
    {  481, 1228,       0,  622931,       0,  0, 17,  0, }, // Maldives
    {  481, 1237,       0,  622948,       0,  0, 24,  0, }, // Marquesas
    {  481, 1247,       0,  622972,       0,  0, 23,  0, }, // Marshall_Islands
    {  481, 1264,  622995,  623011,  623034, 16, 23, 25, }, // Mauritius
    {  481, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  481, 1281,  623059,  623084,  623116, 25, 32, 34, }, // Mexico_Pacific
    {  481, 1296,  623150,  623168,  623193, 18, 25, 27, }, // Mongolia
    {  481, 1305,  623220,  623235,  623257, 15, 22, 24, }, // Moscow
    {  481, 1312,       0,  623281,       0,  0, 15,  0, }, // Myanmar
    {  481, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  481, 1326,       0,  623296,       0,  0, 13,  0, }, // Nepal
    {  481, 1332,  623309,  623331,  623360, 22, 29, 31, }, // New_Caledonia
    {  481, 1346,  623391,  623412,  623440, 21, 28, 30, }, // New_Zealand
    {  481, 1358,  623470,  623488,  623513, 18, 25, 27, }, // Newfoundland
    {  481, 1371,       0,  623540,       0,  0, 12,  0, }, // Niue
    {  481, 1376,  623552,  623572,  623599, 20, 27, 29, }, // Norfolk
    {  481, 1384,   62667,  623628,  623662, 27, 34, 36, }, // Noronha
    {  481, 1392,       0,  623698,       0,  0, 31,  0, }, // North_Mariana
    {  481, 1406,   62803,  623729,  623755, 19, 26, 28, }, // Novosibirsk
    {  481, 1418,  623783,  623795,  623814, 12, 19, 21, }, // Omsk
    {  481, 1428,  623835,  623852,  623876, 17, 24, 26, }, // Pakistan
    {  481, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  481, 1443,       0,  623902,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  481, 1460,  623926,  623942,  623965, 16, 23, 25, }, // Paraguay
    {  481, 1469,  623990,  624002,  624021, 12, 19, 21, }, // Peru
    {  481, 1474,  624042,  624060,  624085, 18, 25, 27, }, // Philippines
    {  481, 1486,       0,  624112,       0,  0, 20,  0, }, // Phoenix_Islands
    {  481, 1502,  624132,  624160,  624195, 28, 35, 37, }, // Pierre_Miquelon
    {  481, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  481, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {  481, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {  481, 1544,   63419,  624232,  624256, 17, 24, 26, }, // Qyzylorda
    {  481, 1554,       0,  624282,       0,  0, 15,  0, }, // Reunion
    {  481, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  481, 1570,  624297,  624313,  624336, 16, 23, 25, }, // Sakhalin
    {  481, 1579,   63581,  624361,  624382, 14, 21, 23, }, // Samara
    {  481, 1596,   63641,  624405,  624425, 13, 20, 22, }, // Samoa
    {  481, 1602,       0,  624447,       0,  0, 18,  0, }, // Seychelles
    {  481, 1624,       0,  624465,       0,  0, 24,  0, }, // Singapore
    {  481, 1634,       0,  624489,       0,  0, 22,  0, }, // Solomon
    {  481, 1642,       0,  624511,       0,  0, 22,  0, }, // South_Georgia
    {  481, 1656,       0,  624533,       0,  0, 16,  0, }, // Suriname
    {  481, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  481, 1682,       0,  624549,       0,  0, 13,  0, }, // Tahiti
    {  481, 1689,   63834,  624562,  624582, 13, 20, 22, }, // Taipei
    {  481, 1696,       0,  624604,       0,  0, 19,  0, }, // Tajikistan
    {  481, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  481, 1732,   63929,  624623,  624643, 13, 20, 22, }, // Tonga
    {  481, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  481, 1750,  624665,  624687,  624716, 22, 29, 31, }, // Turkmenistan
    {  481, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  481, 1777,  624747,  624762,  624784, 15, 22, 24, }, // Uruguay
    {  481, 1792,  624808,  624827,  624853, 19, 26, 28, }, // Uzbekistan
    {  481, 1803,  624881,  624896,  624918, 15, 22, 24, }, // Vanuatu
    {  481, 1811,       0,  624942,       0,  0, 17,  0, }, // Venezuela
    {  481, 1821,   64318,  624959,  624985, 19, 26, 28, }, // Vladivostok
    {  481, 1833,  625013,  625031,  625056, 18, 25, 27, }, // Volgograd
    {  481, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  481, 1850,       0,  625083,       0,  0, 17,  0, }, // Wake
    {  481, 1855,       0,  625100,       0,  0, 23,  0, }, // Wallis
    {  481, 1862,   64520,  625123,  625145, 15, 22, 24, }, // Yakutsk
    {  481, 1870,  625169,  625190,  625218, 21, 28, 30, }, // Yekaterinburg
    {  481, 1892,       0,  625248,       0,  0, 13,  0, }, // Yukon
    {  482,    0,  618348,  618360,  618379, 12, 19, 21, }, // Acre Portuguese/Latin/Mozambique
    {  482,    5,       0,  618400,       0,  0, 19,  0, }, // Afghanistan
    {  482,   17,       0,  618419,       0,  0, 22,  0, }, // Africa_Central
    {  482,   32,       0,  618441,       0,  0, 23,  0, }, // Africa_Eastern
    {  482,   65,       0,  618464,       0,  0, 21,  0, }, // Africa_Southern
    {  482,   81,  618485,  618509,  618540, 24, 31, 33, }, // Africa_Western
    {  482,  107,  618573,  618587,  618608, 14, 21, 23, }, // Alaska
    {  482,  128,  618631,  618645,  618666, 14, 21, 23, }, // Almaty
    {  482,  135,  618689,  618705,  618728, 16, 23, 25, }, // Amazon
    {  482,  142,  618753,  618781,  618816, 28, 35, 37, }, // America_Central
    {  482,  158,  618853,  618882,  618918, 29, 36, 38, }, // America_Eastern
    {  482,  174,  618956,  618988,  619027, 32, 39, 41, }, // America_Mountain
    {  482,  191,  619068,  619100,  619139, 32, 39, 41, }, // America_Pacific
    {  482,  207,  619180,  619194,  619215, 14, 21, 23, }, // Anadyr
    {  482,  214,  619238,  619250,  619269, 12, 19, 21, }, // Apia
    {  482,  219,  619290,  619303,  619323, 13, 20, 22, }, // Aqtau
    {  482,  225,  619345,  619359,  619380, 14, 21, 23, }, // Aqtobe
    {  482,  232,  619403,  619417,  619438, 14, 21, 23, }, // Arabian
    {  482,  240,  619461,  619478,  619502, 17, 24, 26, }, // Argentina
    {  482,  250,  619528,  619555,  619589, 27, 34, 36, }, // Argentina_Western
    {  482,  268,  619625,  619640,  619662, 15, 22, 24, }, // Armenia
    {  482,  286,  619686,  619703,  619727, 17, 24, 26, }, // Atlantic
    {  482,  295,  619753,  619778,  619810, 25, 32, 34, }, // Australia_Central
    {  482,  313,  619844,  619879,  619921, 35, 42, 44, }, // Australia_CentralWestern
    {  482,  338,  619965,  619991,  620024, 26, 33, 35, }, // Australia_Eastern
    {  482,  356,  620059,  620086,  620120, 27, 34, 36, }, // Australia_Western
    {  482,  374,  620156,  620174,  620199, 18, 25, 27, }, // Azerbaijan
    {  482,  385,  620226,  620241,  620263, 15, 22, 24, }, // Azores
    {  482,  397,  620287,  620306,  620332, 19, 26, 28, }, // Bangladesh
    {  482,  415,       0,  620360,       0,  0, 13,  0, }, // Bhutan
    {  482,  422,       0,  620373,       0,  0, 15,  0, }, // Bolivia
    {  482,  437,  139322,  620388,  620411, 16, 23, 25, }, // Brasilia
    {  482,  454,       0,  620436,       0,  0, 25,  0, }, // Brunei
    {  482,  461,  620461,  620479,  620504, 18, 25, 27, }, // Cape_Verde
    {  482,  478,       0,  620531,       0,  0, 23,  0, }, // Chamorro
    {  482,  487,   59239,  620554,  620576, 15, 22, 24, }, // Chatham
    {  482,  495,  620600,  620613,  620633, 13, 20, 22, }, // Chile
    {  482,  501,  620655,  620668,  620688, 13, 20, 22, }, // China
    {  482,  507,       0,  620710,       0,  0, 21,  0, }, // Christmas
    {  482,  517,       0,  620731,       0,  0, 20,  0, }, // Cocos
    {  482,  523,  620751,  620767,  620790, 16, 23, 25, }, // Colombia
    {  482,  532,  620815,  620834,  620860, 19, 26, 28, }, // Cook
    {  482,  537,   59634,  620888,  620907, 12, 19, 21, }, // Cuba
    {  482,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  482,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  482,  601,       0,  620928,       0,  0, 19,  0, }, // East_Timor
    {  482,  612,  620947,  620969,  620998, 22, 29, 31, }, // Easter
    {  482,  619,       0,  621029,       0,  0, 15,  0, }, // Ecuador
    {  482,  627,  621044,  621066,  621095, 22, 29, 31, }, // Europe_Central
    {  482,  642,  621126,  621149,  621179, 23, 30, 32, }, // Europe_Eastern
    {  482,  657,       0,  621211,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  482,  680,  621242,  621266,  621297, 24, 31, 33, }, // Europe_Western
    {  482,  695,  621330,  621353,  621383, 23, 30, 32, }, // Falkland
    {  482,  704,  140424,  621415,  621434, 12, 19, 21, }, // Fiji
    {  482,  709,       0,  621455,       0,  0, 23,  0, }, // French_Guiana
    {  482,  723,       0,  621478,       0,  0, 48,  0, }, // French_Southern
    {  482,  746,       0,  621526,       0,  0, 18,  0, }, // Galapagos
    {  482,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  482,  764,  621544,  621559,  621581, 15, 22, 24, }, // Georgia
    {  482,  772,       0,  621605,       0,  0, 22,  0, }, // Gilbert_Islands
    {  482,  788,       0,  621627,       0,  0, 17,  0, }, // GMT
    {  482,  830,  621644,  621672,  621707, 28, 35, 37, }, // Greenland_Eastern
    {  482,  848,  621744,  621773,  621809, 29, 36, 38, }, // Greenland_Western
    {  482,  866,       0,  621847,       0,  0, 19,  0, }, // Guam
    {  482,  871,       0,  621866,       0,  0, 20,  0, }, // Gulf
    {  482,  876,       0,  621455,       0,  0, 14,  0, }, // Guyana
    {  482,  883,  621886,  621909,  621939, 23, 30, 32, }, // Hawaii_Aleutian
    {  482,  899,  141072,  621971,  621995, 17, 24, 26, }, // Hong_Kong
    {  482,  909,   60912,  622021,  622040, 12, 19, 21, }, // Hovd
    {  482,  914,       0,  622061,       0,  0, 20,  0, }, // India
    {  482,  920,       0,  622081,       0,  0, 21,  0, }, // Indian_Ocean
    {  482,  933,       0,  622102,       0,  0, 17,  0, }, // Indochina
    {  482,  943,       0,  622119,       0,  0, 25,  0, }, // Indonesia_Central
    {  482,  961,       0,  622144,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  482,  979,       0,  622170,       0,  0, 27,  0, }, // Indonesia_Western
    {  482,  997,  622197,  622209,  622228, 12, 19, 21, }, // Iran
    {  482, 1008,  622249,  622264,  622286, 15, 22, 24, }, // Irkutsk
    {  482, 1016,  622310,  622324,  622345, 14, 21, 23, }, // Israel
    {  482, 1023,  622368,  622381,  622401, 13, 20, 22, }, // Japan
    {  482, 1029,  622423,  622455,  622494, 32, 39, 41, }, // Kamchatka
    {  482, 1047,       0,  622535,       0,  0, 19,  0, }, // Kazakhstan
    {  482, 1058,       0,  622554,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  482, 1077,       0,  622582,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  482, 1106,  622611,  622625,  622646, 14, 21, 23, }, // Korea
    {  482, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  482, 1119,   61570,  622669,  622695, 19, 26, 28, }, // Krasnoyarsk
    {  482, 1151,       0,  622723,       0,  0, 19,  0, }, // Kyrgystan
    {  482, 1161,       0,  622742,       0,  0, 17,  0, }, // Lanka
    {  482, 1175,       0,  622759,       0,  0, 19,  0, }, // Line_Islands
    {  482, 1188,   61700,  622778,  622802, 17, 24, 26, }, // Lord_Howe
    {  482, 1198,  141929,  622828,  622848, 13, 20, 22, }, // Macau
    {  482, 1204,  141986,  622870,  622892, 15, 22, 24, }, // Magadan
    {  482, 1219,       0,  622916,       0,  0, 15,  0, }, // Malaysia
    {  482, 1228,       0,  622931,       0,  0, 17,  0, }, // Maldives
    {  482, 1237,       0,  622948,       0,  0, 24,  0, }, // Marquesas
    {  482, 1247,       0,  622972,       0,  0, 23,  0, }, // Marshall_Islands
    {  482, 1264,  622995,  623011,  623034, 16, 23, 25, }, // Mauritius
    {  482, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  482, 1281,  623059,  623084,  623116, 25, 32, 34, }, // Mexico_Pacific
    {  482, 1296,  623150,  623168,  623193, 18, 25, 27, }, // Mongolia
    {  482, 1305,  623220,  623235,  623257, 15, 22, 24, }, // Moscow
    {  482, 1312,       0,  623281,       0,  0, 15,  0, }, // Myanmar
    {  482, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  482, 1326,       0,  623296,       0,  0, 13,  0, }, // Nepal
    {  482, 1332,  623309,  623331,  623360, 22, 29, 31, }, // New_Caledonia
    {  482, 1346,  623391,  623412,  623440, 21, 28, 30, }, // New_Zealand
    {  482, 1358,  623470,  623488,  623513, 18, 25, 27, }, // Newfoundland
    {  482, 1371,       0,  623540,       0,  0, 12,  0, }, // Niue
    {  482, 1376,  623552,  623572,  623599, 20, 27, 29, }, // Norfolk
    {  482, 1384,   62667,  623628,  623662, 27, 34, 36, }, // Noronha
    {  482, 1392,       0,  623698,       0,  0, 31,  0, }, // North_Mariana
    {  482, 1406,   62803,  623729,  623755, 19, 26, 28, }, // Novosibirsk
    {  482, 1418,  623783,  623795,  623814, 12, 19, 21, }, // Omsk
    {  482, 1428,  623835,  623852,  623876, 17, 24, 26, }, // Pakistan
    {  482, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  482, 1443,       0,  623902,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  482, 1460,  623926,  623942,  623965, 16, 23, 25, }, // Paraguay
    {  482, 1469,  623990,  624002,  624021, 12, 19, 21, }, // Peru
    {  482, 1474,  624042,  624060,  624085, 18, 25, 27, }, // Philippines
    {  482, 1486,       0,  624112,       0,  0, 20,  0, }, // Phoenix_Islands
    {  482, 1502,  624132,  624160,  624195, 28, 35, 37, }, // Pierre_Miquelon
    {  482, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  482, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {  482, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {  482, 1544,   63419,  624232,  624256, 17, 24, 26, }, // Qyzylorda
    {  482, 1554,       0,  624282,       0,  0, 15,  0, }, // Reunion
    {  482, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  482, 1570,  624297,  624313,  624336, 16, 23, 25, }, // Sakhalin
    {  482, 1579,   63581,  624361,  624382, 14, 21, 23, }, // Samara
    {  482, 1596,   63641,  624405,  624425, 13, 20, 22, }, // Samoa
    {  482, 1602,       0,  624447,       0,  0, 18,  0, }, // Seychelles
    {  482, 1624,       0,  624465,       0,  0, 24,  0, }, // Singapore
    {  482, 1634,       0,  624489,       0,  0, 22,  0, }, // Solomon
    {  482, 1642,       0,  624511,       0,  0, 22,  0, }, // South_Georgia
    {  482, 1656,       0,  624533,       0,  0, 16,  0, }, // Suriname
    {  482, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  482, 1682,       0,  624549,       0,  0, 13,  0, }, // Tahiti
    {  482, 1689,   63834,  624562,  624582, 13, 20, 22, }, // Taipei
    {  482, 1696,       0,  624604,       0,  0, 19,  0, }, // Tajikistan
    {  482, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  482, 1732,   63929,  624623,  624643, 13, 20, 22, }, // Tonga
    {  482, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  482, 1750,  624665,  624687,  624716, 22, 29, 31, }, // Turkmenistan
    {  482, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  482, 1777,  624747,  624762,  624784, 15, 22, 24, }, // Uruguay
    {  482, 1792,  624808,  624827,  624853, 19, 26, 28, }, // Uzbekistan
    {  482, 1803,  624881,  624896,  624918, 15, 22, 24, }, // Vanuatu
    {  482, 1811,       0,  624942,       0,  0, 17,  0, }, // Venezuela
    {  482, 1821,   64318,  624959,  624985, 19, 26, 28, }, // Vladivostok
    {  482, 1833,  625013,  625031,  625056, 18, 25, 27, }, // Volgograd
    {  482, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  482, 1850,       0,  625083,       0,  0, 17,  0, }, // Wake
    {  482, 1855,       0,  625100,       0,  0, 23,  0, }, // Wallis
    {  482, 1862,   64520,  625123,  625145, 15, 22, 24, }, // Yakutsk
    {  482, 1870,  625169,  625190,  625218, 21, 28, 30, }, // Yekaterinburg
    {  482, 1892,       0,  625248,       0,  0, 13,  0, }, // Yukon
    {  483,    0,  618348,  618360,  618379, 12, 19, 21, }, // Acre Portuguese/Latin/Portugal
    {  483,    5,       0,  618400,       0,  0, 19,  0, }, // Afghanistan
    {  483,   17,       0,  618419,       0,  0, 22,  0, }, // Africa_Central
    {  483,   32,       0,  618441,       0,  0, 23,  0, }, // Africa_Eastern
    {  483,   65,       0,  618464,       0,  0, 21,  0, }, // Africa_Southern
    {  483,   81,  618485,  618509,  618540, 24, 31, 33, }, // Africa_Western
    {  483,  107,  618573,  618587,  618608, 14, 21, 23, }, // Alaska
    {  483,  128,  618631,  618645,  618666, 14, 21, 23, }, // Almaty
    {  483,  135,  618689,  618705,  618728, 16, 23, 25, }, // Amazon
    {  483,  142,  618753,  618781,  618816, 28, 35, 37, }, // America_Central
    {  483,  158,  618853,  618882,  618918, 29, 36, 38, }, // America_Eastern
    {  483,  174,  618956,  618988,  619027, 32, 39, 41, }, // America_Mountain
    {  483,  191,  619068,  619100,  619139, 32, 39, 41, }, // America_Pacific
    {  483,  207,  619180,  619194,  619215, 14, 21, 23, }, // Anadyr
    {  483,  214,  619238,  619250,  619269, 12, 19, 21, }, // Apia
    {  483,  219,  619290,  619303,  619323, 13, 20, 22, }, // Aqtau
    {  483,  225,  619345,  619359,  619380, 14, 21, 23, }, // Aqtobe
    {  483,  232,  619403,  619417,  619438, 14, 21, 23, }, // Arabian
    {  483,  240,  619461,  619478,  619502, 17, 24, 26, }, // Argentina
    {  483,  250,  619528,  619555,  619589, 27, 34, 36, }, // Argentina_Western
    {  483,  268,  619625,  619640,  619662, 15, 22, 24, }, // Armenia
    {  483,  286,  619686,  619703,  619727, 17, 24, 26, }, // Atlantic
    {  483,  295,  619753,  619778,  619810, 25, 32, 34, }, // Australia_Central
    {  483,  313,  619844,  619879,  619921, 35, 42, 44, }, // Australia_CentralWestern
    {  483,  338,  619965,  619991,  620024, 26, 33, 35, }, // Australia_Eastern
    {  483,  356,  620059,  620086,  620120, 27, 34, 36, }, // Australia_Western
    {  483,  374,  620156,  620174,  620199, 18, 25, 27, }, // Azerbaijan
    {  483,  385,  620226,  620241,  620263, 15, 22, 24, }, // Azores
    {  483,  397,  620287,  620306,  620332, 19, 26, 28, }, // Bangladesh
    {  483,  415,       0,  620360,       0,  0, 13,  0, }, // Bhutan
    {  483,  422,       0,  620373,       0,  0, 15,  0, }, // Bolivia
    {  483,  437,  139322,  620388,  620411, 16, 23, 25, }, // Brasilia
    {  483,  454,       0,  620436,       0,  0, 25,  0, }, // Brunei
    {  483,  461,  620461,  620479,  620504, 18, 25, 27, }, // Cape_Verde
    {  483,  478,       0,  620531,       0,  0, 23,  0, }, // Chamorro
    {  483,  487,   59239,  620554,  620576, 15, 22, 24, }, // Chatham
    {  483,  495,  620600,  620613,  620633, 13, 20, 22, }, // Chile
    {  483,  501,  620655,  620668,  620688, 13, 20, 22, }, // China
    {  483,  507,       0,  620710,       0,  0, 21,  0, }, // Christmas
    {  483,  517,       0,  620731,       0,  0, 20,  0, }, // Cocos
    {  483,  523,  620751,  620767,  620790, 16, 23, 25, }, // Colombia
    {  483,  532,  620815,  620834,  620860, 19, 26, 28, }, // Cook
    {  483,  537,   59634,  620888,  620907, 12, 19, 21, }, // Cuba
    {  483,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  483,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  483,  601,       0,  620928,       0,  0, 19,  0, }, // East_Timor
    {  483,  612,  620947,  620969,  620998, 22, 29, 31, }, // Easter
    {  483,  619,       0,  621029,       0,  0, 15,  0, }, // Ecuador
    {  483,  627,  621044,  621066,  621095, 22, 29, 31, }, // Europe_Central
    {  483,  642,  621126,  621149,  621179, 23, 30, 32, }, // Europe_Eastern
    {  483,  657,       0,  621211,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  483,  680,  621242,  621266,  621297, 24, 31, 33, }, // Europe_Western
    {  483,  695,  621330,  621353,  621383, 23, 30, 32, }, // Falkland
    {  483,  704,  140424,  621415,  621434, 12, 19, 21, }, // Fiji
    {  483,  709,       0,  621455,       0,  0, 23,  0, }, // French_Guiana
    {  483,  723,       0,  621478,       0,  0, 48,  0, }, // French_Southern
    {  483,  746,       0,  621526,       0,  0, 18,  0, }, // Galapagos
    {  483,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  483,  764,  621544,  621559,  621581, 15, 22, 24, }, // Georgia
    {  483,  772,       0,  621605,       0,  0, 22,  0, }, // Gilbert_Islands
    {  483,  788,       0,  621627,       0,  0, 17,  0, }, // GMT
    {  483,  830,  621644,  621672,  621707, 28, 35, 37, }, // Greenland_Eastern
    {  483,  848,  621744,  621773,  621809, 29, 36, 38, }, // Greenland_Western
    {  483,  866,       0,  621847,       0,  0, 19,  0, }, // Guam
    {  483,  871,       0,  621866,       0,  0, 20,  0, }, // Gulf
    {  483,  876,       0,  621455,       0,  0, 14,  0, }, // Guyana
    {  483,  883,  621886,  621909,  621939, 23, 30, 32, }, // Hawaii_Aleutian
    {  483,  899,  141072,  621971,  621995, 17, 24, 26, }, // Hong_Kong
    {  483,  909,   60912,  622021,  622040, 12, 19, 21, }, // Hovd
    {  483,  914,       0,  622061,       0,  0, 20,  0, }, // India
    {  483,  920,       0,  622081,       0,  0, 21,  0, }, // Indian_Ocean
    {  483,  933,       0,  622102,       0,  0, 17,  0, }, // Indochina
    {  483,  943,       0,  622119,       0,  0, 25,  0, }, // Indonesia_Central
    {  483,  961,       0,  622144,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  483,  979,       0,  622170,       0,  0, 27,  0, }, // Indonesia_Western
    {  483,  997,  622197,  622209,  622228, 12, 19, 21, }, // Iran
    {  483, 1008,  622249,  622264,  622286, 15, 22, 24, }, // Irkutsk
    {  483, 1016,  622310,  622324,  622345, 14, 21, 23, }, // Israel
    {  483, 1023,  622368,  622381,  622401, 13, 20, 22, }, // Japan
    {  483, 1029,  622423,  622455,  622494, 32, 39, 41, }, // Kamchatka
    {  483, 1047,       0,  622535,       0,  0, 19,  0, }, // Kazakhstan
    {  483, 1058,       0,  622554,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  483, 1077,       0,  622582,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  483, 1106,  622611,  622625,  622646, 14, 21, 23, }, // Korea
    {  483, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  483, 1119,   61570,  622669,  622695, 19, 26, 28, }, // Krasnoyarsk
    {  483, 1151,       0,  622723,       0,  0, 19,  0, }, // Kyrgystan
    {  483, 1161,       0,  622742,       0,  0, 17,  0, }, // Lanka
    {  483, 1175,       0,  622759,       0,  0, 19,  0, }, // Line_Islands
    {  483, 1188,   61700,  622778,  622802, 17, 24, 26, }, // Lord_Howe
    {  483, 1198,  141929,  622828,  622848, 13, 20, 22, }, // Macau
    {  483, 1204,  141986,  622870,  622892, 15, 22, 24, }, // Magadan
    {  483, 1219,       0,  622916,       0,  0, 15,  0, }, // Malaysia
    {  483, 1228,       0,  622931,       0,  0, 17,  0, }, // Maldives
    {  483, 1237,       0,  622948,       0,  0, 24,  0, }, // Marquesas
    {  483, 1247,       0,  622972,       0,  0, 23,  0, }, // Marshall_Islands
    {  483, 1264,  622995,  623011,  623034, 16, 23, 25, }, // Mauritius
    {  483, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  483, 1281,  623059,  623084,  623116, 25, 32, 34, }, // Mexico_Pacific
    {  483, 1296,  623150,  623168,  623193, 18, 25, 27, }, // Mongolia
    {  483, 1305,  623220,  623235,  623257, 15, 22, 24, }, // Moscow
    {  483, 1312,       0,  623281,       0,  0, 15,  0, }, // Myanmar
    {  483, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  483, 1326,       0,  623296,       0,  0, 13,  0, }, // Nepal
    {  483, 1332,  623309,  623331,  623360, 22, 29, 31, }, // New_Caledonia
    {  483, 1346,  623391,  623412,  623440, 21, 28, 30, }, // New_Zealand
    {  483, 1358,  623470,  623488,  623513, 18, 25, 27, }, // Newfoundland
    {  483, 1371,       0,  623540,       0,  0, 12,  0, }, // Niue
    {  483, 1376,  623552,  623572,  623599, 20, 27, 29, }, // Norfolk
    {  483, 1384,   62667,  623628,  623662, 27, 34, 36, }, // Noronha
    {  483, 1392,       0,  623698,       0,  0, 31,  0, }, // North_Mariana
    {  483, 1406,   62803,  623729,  623755, 19, 26, 28, }, // Novosibirsk
    {  483, 1418,  623783,  623795,  623814, 12, 19, 21, }, // Omsk
    {  483, 1428,  623835,  623852,  623876, 17, 24, 26, }, // Pakistan
    {  483, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  483, 1443,       0,  623902,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  483, 1460,  623926,  623942,  623965, 16, 23, 25, }, // Paraguay
    {  483, 1469,  623990,  624002,  624021, 12, 19, 21, }, // Peru
    {  483, 1474,  624042,  624060,  624085, 18, 25, 27, }, // Philippines
    {  483, 1486,       0,  624112,       0,  0, 20,  0, }, // Phoenix_Islands
    {  483, 1502,  624132,  624160,  624195, 28, 35, 37, }, // Pierre_Miquelon
    {  483, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  483, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {  483, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {  483, 1544,   63419,  624232,  624256, 17, 24, 26, }, // Qyzylorda
    {  483, 1554,       0,  624282,       0,  0, 15,  0, }, // Reunion
    {  483, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  483, 1570,  624297,  624313,  624336, 16, 23, 25, }, // Sakhalin
    {  483, 1579,   63581,  624361,  624382, 14, 21, 23, }, // Samara
    {  483, 1596,   63641,  624405,  624425, 13, 20, 22, }, // Samoa
    {  483, 1602,       0,  624447,       0,  0, 18,  0, }, // Seychelles
    {  483, 1624,       0,  624465,       0,  0, 24,  0, }, // Singapore
    {  483, 1634,       0,  624489,       0,  0, 22,  0, }, // Solomon
    {  483, 1642,       0,  624511,       0,  0, 22,  0, }, // South_Georgia
    {  483, 1656,       0,  624533,       0,  0, 16,  0, }, // Suriname
    {  483, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  483, 1682,       0,  624549,       0,  0, 13,  0, }, // Tahiti
    {  483, 1689,   63834,  624562,  624582, 13, 20, 22, }, // Taipei
    {  483, 1696,       0,  624604,       0,  0, 19,  0, }, // Tajikistan
    {  483, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  483, 1732,   63929,  624623,  624643, 13, 20, 22, }, // Tonga
    {  483, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  483, 1750,  624665,  624687,  624716, 22, 29, 31, }, // Turkmenistan
    {  483, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  483, 1777,  624747,  624762,  624784, 15, 22, 24, }, // Uruguay
    {  483, 1792,  624808,  624827,  624853, 19, 26, 28, }, // Uzbekistan
    {  483, 1803,  624881,  624896,  624918, 15, 22, 24, }, // Vanuatu
    {  483, 1811,       0,  624942,       0,  0, 17,  0, }, // Venezuela
    {  483, 1821,   64318,  624959,  624985, 19, 26, 28, }, // Vladivostok
    {  483, 1833,  625013,  625031,  625056, 18, 25, 27, }, // Volgograd
    {  483, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  483, 1850,       0,  625083,       0,  0, 17,  0, }, // Wake
    {  483, 1855,       0,  625100,       0,  0, 23,  0, }, // Wallis
    {  483, 1862,   64520,  625123,  625145, 15, 22, 24, }, // Yakutsk
    {  483, 1870,  625169,  625190,  625218, 21, 28, 30, }, // Yekaterinburg
    {  483, 1892,       0,  625248,       0,  0, 13,  0, }, // Yukon
    {  484,    0,  618348,  618360,  618379, 12, 19, 21, }, // Acre Portuguese/Latin/Sao Tome and Principe
    {  484,    5,       0,  618400,       0,  0, 19,  0, }, // Afghanistan
    {  484,   17,       0,  618419,       0,  0, 22,  0, }, // Africa_Central
    {  484,   32,       0,  618441,       0,  0, 23,  0, }, // Africa_Eastern
    {  484,   65,       0,  618464,       0,  0, 21,  0, }, // Africa_Southern
    {  484,   81,  618485,  618509,  618540, 24, 31, 33, }, // Africa_Western
    {  484,  107,  618573,  618587,  618608, 14, 21, 23, }, // Alaska
    {  484,  128,  618631,  618645,  618666, 14, 21, 23, }, // Almaty
    {  484,  135,  618689,  618705,  618728, 16, 23, 25, }, // Amazon
    {  484,  142,  618753,  618781,  618816, 28, 35, 37, }, // America_Central
    {  484,  158,  618853,  618882,  618918, 29, 36, 38, }, // America_Eastern
    {  484,  174,  618956,  618988,  619027, 32, 39, 41, }, // America_Mountain
    {  484,  191,  619068,  619100,  619139, 32, 39, 41, }, // America_Pacific
    {  484,  207,  619180,  619194,  619215, 14, 21, 23, }, // Anadyr
    {  484,  214,  619238,  619250,  619269, 12, 19, 21, }, // Apia
    {  484,  219,  619290,  619303,  619323, 13, 20, 22, }, // Aqtau
    {  484,  225,  619345,  619359,  619380, 14, 21, 23, }, // Aqtobe
    {  484,  232,  619403,  619417,  619438, 14, 21, 23, }, // Arabian
    {  484,  240,  619461,  619478,  619502, 17, 24, 26, }, // Argentina
    {  484,  250,  619528,  619555,  619589, 27, 34, 36, }, // Argentina_Western
    {  484,  268,  619625,  619640,  619662, 15, 22, 24, }, // Armenia
    {  484,  286,  619686,  619703,  619727, 17, 24, 26, }, // Atlantic
    {  484,  295,  619753,  619778,  619810, 25, 32, 34, }, // Australia_Central
    {  484,  313,  619844,  619879,  619921, 35, 42, 44, }, // Australia_CentralWestern
    {  484,  338,  619965,  619991,  620024, 26, 33, 35, }, // Australia_Eastern
    {  484,  356,  620059,  620086,  620120, 27, 34, 36, }, // Australia_Western
    {  484,  374,  620156,  620174,  620199, 18, 25, 27, }, // Azerbaijan
    {  484,  385,  620226,  620241,  620263, 15, 22, 24, }, // Azores
    {  484,  397,  620287,  620306,  620332, 19, 26, 28, }, // Bangladesh
    {  484,  415,       0,  620360,       0,  0, 13,  0, }, // Bhutan
    {  484,  422,       0,  620373,       0,  0, 15,  0, }, // Bolivia
    {  484,  437,  139322,  620388,  620411, 16, 23, 25, }, // Brasilia
    {  484,  454,       0,  620436,       0,  0, 25,  0, }, // Brunei
    {  484,  461,  620461,  620479,  620504, 18, 25, 27, }, // Cape_Verde
    {  484,  478,       0,  620531,       0,  0, 23,  0, }, // Chamorro
    {  484,  487,   59239,  620554,  620576, 15, 22, 24, }, // Chatham
    {  484,  495,  620600,  620613,  620633, 13, 20, 22, }, // Chile
    {  484,  501,  620655,  620668,  620688, 13, 20, 22, }, // China
    {  484,  507,       0,  620710,       0,  0, 21,  0, }, // Christmas
    {  484,  517,       0,  620731,       0,  0, 20,  0, }, // Cocos
    {  484,  523,  620751,  620767,  620790, 16, 23, 25, }, // Colombia
    {  484,  532,  620815,  620834,  620860, 19, 26, 28, }, // Cook
    {  484,  537,   59634,  620888,  620907, 12, 19, 21, }, // Cuba
    {  484,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  484,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  484,  601,       0,  620928,       0,  0, 19,  0, }, // East_Timor
    {  484,  612,  620947,  620969,  620998, 22, 29, 31, }, // Easter
    {  484,  619,       0,  621029,       0,  0, 15,  0, }, // Ecuador
    {  484,  627,  621044,  621066,  621095, 22, 29, 31, }, // Europe_Central
    {  484,  642,  621126,  621149,  621179, 23, 30, 32, }, // Europe_Eastern
    {  484,  657,       0,  621211,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  484,  680,  621242,  621266,  621297, 24, 31, 33, }, // Europe_Western
    {  484,  695,  621330,  621353,  621383, 23, 30, 32, }, // Falkland
    {  484,  704,  140424,  621415,  621434, 12, 19, 21, }, // Fiji
    {  484,  709,       0,  621455,       0,  0, 23,  0, }, // French_Guiana
    {  484,  723,       0,  621478,       0,  0, 48,  0, }, // French_Southern
    {  484,  746,       0,  621526,       0,  0, 18,  0, }, // Galapagos
    {  484,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  484,  764,  621544,  621559,  621581, 15, 22, 24, }, // Georgia
    {  484,  772,       0,  621605,       0,  0, 22,  0, }, // Gilbert_Islands
    {  484,  788,       0,  621627,       0,  0, 17,  0, }, // GMT
    {  484,  830,  621644,  621672,  621707, 28, 35, 37, }, // Greenland_Eastern
    {  484,  848,  621744,  621773,  621809, 29, 36, 38, }, // Greenland_Western
    {  484,  866,       0,  621847,       0,  0, 19,  0, }, // Guam
    {  484,  871,       0,  621866,       0,  0, 20,  0, }, // Gulf
    {  484,  876,       0,  621455,       0,  0, 14,  0, }, // Guyana
    {  484,  883,  621886,  621909,  621939, 23, 30, 32, }, // Hawaii_Aleutian
    {  484,  899,  141072,  621971,  621995, 17, 24, 26, }, // Hong_Kong
    {  484,  909,   60912,  622021,  622040, 12, 19, 21, }, // Hovd
    {  484,  914,       0,  622061,       0,  0, 20,  0, }, // India
    {  484,  920,       0,  622081,       0,  0, 21,  0, }, // Indian_Ocean
    {  484,  933,       0,  622102,       0,  0, 17,  0, }, // Indochina
    {  484,  943,       0,  622119,       0,  0, 25,  0, }, // Indonesia_Central
    {  484,  961,       0,  622144,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  484,  979,       0,  622170,       0,  0, 27,  0, }, // Indonesia_Western
    {  484,  997,  622197,  622209,  622228, 12, 19, 21, }, // Iran
    {  484, 1008,  622249,  622264,  622286, 15, 22, 24, }, // Irkutsk
    {  484, 1016,  622310,  622324,  622345, 14, 21, 23, }, // Israel
    {  484, 1023,  622368,  622381,  622401, 13, 20, 22, }, // Japan
    {  484, 1029,  622423,  622455,  622494, 32, 39, 41, }, // Kamchatka
    {  484, 1047,       0,  622535,       0,  0, 19,  0, }, // Kazakhstan
    {  484, 1058,       0,  622554,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  484, 1077,       0,  622582,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  484, 1106,  622611,  622625,  622646, 14, 21, 23, }, // Korea
    {  484, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  484, 1119,   61570,  622669,  622695, 19, 26, 28, }, // Krasnoyarsk
    {  484, 1151,       0,  622723,       0,  0, 19,  0, }, // Kyrgystan
    {  484, 1161,       0,  622742,       0,  0, 17,  0, }, // Lanka
    {  484, 1175,       0,  622759,       0,  0, 19,  0, }, // Line_Islands
    {  484, 1188,   61700,  622778,  622802, 17, 24, 26, }, // Lord_Howe
    {  484, 1198,  141929,  622828,  622848, 13, 20, 22, }, // Macau
    {  484, 1204,  141986,  622870,  622892, 15, 22, 24, }, // Magadan
    {  484, 1219,       0,  622916,       0,  0, 15,  0, }, // Malaysia
    {  484, 1228,       0,  622931,       0,  0, 17,  0, }, // Maldives
    {  484, 1237,       0,  622948,       0,  0, 24,  0, }, // Marquesas
    {  484, 1247,       0,  622972,       0,  0, 23,  0, }, // Marshall_Islands
    {  484, 1264,  622995,  623011,  623034, 16, 23, 25, }, // Mauritius
    {  484, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  484, 1281,  623059,  623084,  623116, 25, 32, 34, }, // Mexico_Pacific
    {  484, 1296,  623150,  623168,  623193, 18, 25, 27, }, // Mongolia
    {  484, 1305,  623220,  623235,  623257, 15, 22, 24, }, // Moscow
    {  484, 1312,       0,  623281,       0,  0, 15,  0, }, // Myanmar
    {  484, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  484, 1326,       0,  623296,       0,  0, 13,  0, }, // Nepal
    {  484, 1332,  623309,  623331,  623360, 22, 29, 31, }, // New_Caledonia
    {  484, 1346,  623391,  623412,  623440, 21, 28, 30, }, // New_Zealand
    {  484, 1358,  623470,  623488,  623513, 18, 25, 27, }, // Newfoundland
    {  484, 1371,       0,  623540,       0,  0, 12,  0, }, // Niue
    {  484, 1376,  623552,  623572,  623599, 20, 27, 29, }, // Norfolk
    {  484, 1384,   62667,  623628,  623662, 27, 34, 36, }, // Noronha
    {  484, 1392,       0,  623698,       0,  0, 31,  0, }, // North_Mariana
    {  484, 1406,   62803,  623729,  623755, 19, 26, 28, }, // Novosibirsk
    {  484, 1418,  623783,  623795,  623814, 12, 19, 21, }, // Omsk
    {  484, 1428,  623835,  623852,  623876, 17, 24, 26, }, // Pakistan
    {  484, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  484, 1443,       0,  623902,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  484, 1460,  623926,  623942,  623965, 16, 23, 25, }, // Paraguay
    {  484, 1469,  623990,  624002,  624021, 12, 19, 21, }, // Peru
    {  484, 1474,  624042,  624060,  624085, 18, 25, 27, }, // Philippines
    {  484, 1486,       0,  624112,       0,  0, 20,  0, }, // Phoenix_Islands
    {  484, 1502,  624132,  624160,  624195, 28, 35, 37, }, // Pierre_Miquelon
    {  484, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  484, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {  484, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {  484, 1544,   63419,  624232,  624256, 17, 24, 26, }, // Qyzylorda
    {  484, 1554,       0,  624282,       0,  0, 15,  0, }, // Reunion
    {  484, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  484, 1570,  624297,  624313,  624336, 16, 23, 25, }, // Sakhalin
    {  484, 1579,   63581,  624361,  624382, 14, 21, 23, }, // Samara
    {  484, 1596,   63641,  624405,  624425, 13, 20, 22, }, // Samoa
    {  484, 1602,       0,  624447,       0,  0, 18,  0, }, // Seychelles
    {  484, 1624,       0,  624465,       0,  0, 24,  0, }, // Singapore
    {  484, 1634,       0,  624489,       0,  0, 22,  0, }, // Solomon
    {  484, 1642,       0,  624511,       0,  0, 22,  0, }, // South_Georgia
    {  484, 1656,       0,  624533,       0,  0, 16,  0, }, // Suriname
    {  484, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  484, 1682,       0,  624549,       0,  0, 13,  0, }, // Tahiti
    {  484, 1689,   63834,  624562,  624582, 13, 20, 22, }, // Taipei
    {  484, 1696,       0,  624604,       0,  0, 19,  0, }, // Tajikistan
    {  484, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  484, 1732,   63929,  624623,  624643, 13, 20, 22, }, // Tonga
    {  484, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  484, 1750,  624665,  624687,  624716, 22, 29, 31, }, // Turkmenistan
    {  484, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  484, 1777,  624747,  624762,  624784, 15, 22, 24, }, // Uruguay
    {  484, 1792,  624808,  624827,  624853, 19, 26, 28, }, // Uzbekistan
    {  484, 1803,  624881,  624896,  624918, 15, 22, 24, }, // Vanuatu
    {  484, 1811,       0,  624942,       0,  0, 17,  0, }, // Venezuela
    {  484, 1821,   64318,  624959,  624985, 19, 26, 28, }, // Vladivostok
    {  484, 1833,  625013,  625031,  625056, 18, 25, 27, }, // Volgograd
    {  484, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  484, 1850,       0,  625083,       0,  0, 17,  0, }, // Wake
    {  484, 1855,       0,  625100,       0,  0, 23,  0, }, // Wallis
    {  484, 1862,   64520,  625123,  625145, 15, 22, 24, }, // Yakutsk
    {  484, 1870,  625169,  625190,  625218, 21, 28, 30, }, // Yekaterinburg
    {  484, 1892,       0,  625248,       0,  0, 13,  0, }, // Yukon
    {  485,    0,  618348,  618360,  618379, 12, 19, 21, }, // Acre Portuguese/Latin/Switzerland
    {  485,    5,       0,  618400,       0,  0, 19,  0, }, // Afghanistan
    {  485,   17,       0,  618419,       0,  0, 22,  0, }, // Africa_Central
    {  485,   32,       0,  618441,       0,  0, 23,  0, }, // Africa_Eastern
    {  485,   65,       0,  618464,       0,  0, 21,  0, }, // Africa_Southern
    {  485,   81,  618485,  618509,  618540, 24, 31, 33, }, // Africa_Western
    {  485,  107,  618573,  618587,  618608, 14, 21, 23, }, // Alaska
    {  485,  128,  618631,  618645,  618666, 14, 21, 23, }, // Almaty
    {  485,  135,  618689,  618705,  618728, 16, 23, 25, }, // Amazon
    {  485,  142,  618753,  618781,  618816, 28, 35, 37, }, // America_Central
    {  485,  158,  618853,  618882,  618918, 29, 36, 38, }, // America_Eastern
    {  485,  174,  618956,  618988,  619027, 32, 39, 41, }, // America_Mountain
    {  485,  191,  619068,  619100,  619139, 32, 39, 41, }, // America_Pacific
    {  485,  207,  619180,  619194,  619215, 14, 21, 23, }, // Anadyr
    {  485,  214,  619238,  619250,  619269, 12, 19, 21, }, // Apia
    {  485,  219,  619290,  619303,  619323, 13, 20, 22, }, // Aqtau
    {  485,  225,  619345,  619359,  619380, 14, 21, 23, }, // Aqtobe
    {  485,  232,  619403,  619417,  619438, 14, 21, 23, }, // Arabian
    {  485,  240,  619461,  619478,  619502, 17, 24, 26, }, // Argentina
    {  485,  250,  619528,  619555,  619589, 27, 34, 36, }, // Argentina_Western
    {  485,  268,  619625,  619640,  619662, 15, 22, 24, }, // Armenia
    {  485,  286,  619686,  619703,  619727, 17, 24, 26, }, // Atlantic
    {  485,  295,  619753,  619778,  619810, 25, 32, 34, }, // Australia_Central
    {  485,  313,  619844,  619879,  619921, 35, 42, 44, }, // Australia_CentralWestern
    {  485,  338,  619965,  619991,  620024, 26, 33, 35, }, // Australia_Eastern
    {  485,  356,  620059,  620086,  620120, 27, 34, 36, }, // Australia_Western
    {  485,  374,  620156,  620174,  620199, 18, 25, 27, }, // Azerbaijan
    {  485,  385,  620226,  620241,  620263, 15, 22, 24, }, // Azores
    {  485,  397,  620287,  620306,  620332, 19, 26, 28, }, // Bangladesh
    {  485,  415,       0,  620360,       0,  0, 13,  0, }, // Bhutan
    {  485,  422,       0,  620373,       0,  0, 15,  0, }, // Bolivia
    {  485,  437,  139322,  620388,  620411, 16, 23, 25, }, // Brasilia
    {  485,  454,       0,  620436,       0,  0, 25,  0, }, // Brunei
    {  485,  461,  620461,  620479,  620504, 18, 25, 27, }, // Cape_Verde
    {  485,  478,       0,  620531,       0,  0, 23,  0, }, // Chamorro
    {  485,  487,   59239,  620554,  620576, 15, 22, 24, }, // Chatham
    {  485,  495,  620600,  620613,  620633, 13, 20, 22, }, // Chile
    {  485,  501,  620655,  620668,  620688, 13, 20, 22, }, // China
    {  485,  507,       0,  620710,       0,  0, 21,  0, }, // Christmas
    {  485,  517,       0,  620731,       0,  0, 20,  0, }, // Cocos
    {  485,  523,  620751,  620767,  620790, 16, 23, 25, }, // Colombia
    {  485,  532,  620815,  620834,  620860, 19, 26, 28, }, // Cook
    {  485,  537,   59634,  620888,  620907, 12, 19, 21, }, // Cuba
    {  485,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  485,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  485,  601,       0,  620928,       0,  0, 19,  0, }, // East_Timor
    {  485,  612,  620947,  620969,  620998, 22, 29, 31, }, // Easter
    {  485,  619,       0,  621029,       0,  0, 15,  0, }, // Ecuador
    {  485,  627,  621044,  621066,  621095, 22, 29, 31, }, // Europe_Central
    {  485,  642,  621126,  621149,  621179, 23, 30, 32, }, // Europe_Eastern
    {  485,  657,       0,  621211,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  485,  680,  621242,  621266,  621297, 24, 31, 33, }, // Europe_Western
    {  485,  695,  621330,  621353,  621383, 23, 30, 32, }, // Falkland
    {  485,  704,  140424,  621415,  621434, 12, 19, 21, }, // Fiji
    {  485,  709,       0,  621455,       0,  0, 23,  0, }, // French_Guiana
    {  485,  723,       0,  621478,       0,  0, 48,  0, }, // French_Southern
    {  485,  746,       0,  621526,       0,  0, 18,  0, }, // Galapagos
    {  485,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  485,  764,  621544,  621559,  621581, 15, 22, 24, }, // Georgia
    {  485,  772,       0,  621605,       0,  0, 22,  0, }, // Gilbert_Islands
    {  485,  788,       0,  621627,       0,  0, 17,  0, }, // GMT
    {  485,  830,  621644,  621672,  621707, 28, 35, 37, }, // Greenland_Eastern
    {  485,  848,  621744,  621773,  621809, 29, 36, 38, }, // Greenland_Western
    {  485,  866,       0,  621847,       0,  0, 19,  0, }, // Guam
    {  485,  871,       0,  621866,       0,  0, 20,  0, }, // Gulf
    {  485,  876,       0,  621455,       0,  0, 14,  0, }, // Guyana
    {  485,  883,  621886,  621909,  621939, 23, 30, 32, }, // Hawaii_Aleutian
    {  485,  899,  141072,  621971,  621995, 17, 24, 26, }, // Hong_Kong
    {  485,  909,   60912,  622021,  622040, 12, 19, 21, }, // Hovd
    {  485,  914,       0,  622061,       0,  0, 20,  0, }, // India
    {  485,  920,       0,  622081,       0,  0, 21,  0, }, // Indian_Ocean
    {  485,  933,       0,  622102,       0,  0, 17,  0, }, // Indochina
    {  485,  943,       0,  622119,       0,  0, 25,  0, }, // Indonesia_Central
    {  485,  961,       0,  622144,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  485,  979,       0,  622170,       0,  0, 27,  0, }, // Indonesia_Western
    {  485,  997,  622197,  622209,  622228, 12, 19, 21, }, // Iran
    {  485, 1008,  622249,  622264,  622286, 15, 22, 24, }, // Irkutsk
    {  485, 1016,  622310,  622324,  622345, 14, 21, 23, }, // Israel
    {  485, 1023,  622368,  622381,  622401, 13, 20, 22, }, // Japan
    {  485, 1029,  622423,  622455,  622494, 32, 39, 41, }, // Kamchatka
    {  485, 1047,       0,  622535,       0,  0, 19,  0, }, // Kazakhstan
    {  485, 1058,       0,  622554,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  485, 1077,       0,  622582,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  485, 1106,  622611,  622625,  622646, 14, 21, 23, }, // Korea
    {  485, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  485, 1119,   61570,  622669,  622695, 19, 26, 28, }, // Krasnoyarsk
    {  485, 1151,       0,  622723,       0,  0, 19,  0, }, // Kyrgystan
    {  485, 1161,       0,  622742,       0,  0, 17,  0, }, // Lanka
    {  485, 1175,       0,  622759,       0,  0, 19,  0, }, // Line_Islands
    {  485, 1188,   61700,  622778,  622802, 17, 24, 26, }, // Lord_Howe
    {  485, 1198,  141929,  622828,  622848, 13, 20, 22, }, // Macau
    {  485, 1204,  141986,  622870,  622892, 15, 22, 24, }, // Magadan
    {  485, 1219,       0,  622916,       0,  0, 15,  0, }, // Malaysia
    {  485, 1228,       0,  622931,       0,  0, 17,  0, }, // Maldives
    {  485, 1237,       0,  622948,       0,  0, 24,  0, }, // Marquesas
    {  485, 1247,       0,  622972,       0,  0, 23,  0, }, // Marshall_Islands
    {  485, 1264,  622995,  623011,  623034, 16, 23, 25, }, // Mauritius
    {  485, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  485, 1281,  623059,  623084,  623116, 25, 32, 34, }, // Mexico_Pacific
    {  485, 1296,  623150,  623168,  623193, 18, 25, 27, }, // Mongolia
    {  485, 1305,  623220,  623235,  623257, 15, 22, 24, }, // Moscow
    {  485, 1312,       0,  623281,       0,  0, 15,  0, }, // Myanmar
    {  485, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  485, 1326,       0,  623296,       0,  0, 13,  0, }, // Nepal
    {  485, 1332,  623309,  623331,  623360, 22, 29, 31, }, // New_Caledonia
    {  485, 1346,  623391,  623412,  623440, 21, 28, 30, }, // New_Zealand
    {  485, 1358,  623470,  623488,  623513, 18, 25, 27, }, // Newfoundland
    {  485, 1371,       0,  623540,       0,  0, 12,  0, }, // Niue
    {  485, 1376,  623552,  623572,  623599, 20, 27, 29, }, // Norfolk
    {  485, 1384,   62667,  623628,  623662, 27, 34, 36, }, // Noronha
    {  485, 1392,       0,  623698,       0,  0, 31,  0, }, // North_Mariana
    {  485, 1406,   62803,  623729,  623755, 19, 26, 28, }, // Novosibirsk
    {  485, 1418,  623783,  623795,  623814, 12, 19, 21, }, // Omsk
    {  485, 1428,  623835,  623852,  623876, 17, 24, 26, }, // Pakistan
    {  485, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  485, 1443,       0,  623902,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  485, 1460,  623926,  623942,  623965, 16, 23, 25, }, // Paraguay
    {  485, 1469,  623990,  624002,  624021, 12, 19, 21, }, // Peru
    {  485, 1474,  624042,  624060,  624085, 18, 25, 27, }, // Philippines
    {  485, 1486,       0,  624112,       0,  0, 20,  0, }, // Phoenix_Islands
    {  485, 1502,  624132,  624160,  624195, 28, 35, 37, }, // Pierre_Miquelon
    {  485, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  485, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {  485, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {  485, 1544,   63419,  624232,  624256, 17, 24, 26, }, // Qyzylorda
    {  485, 1554,       0,  624282,       0,  0, 15,  0, }, // Reunion
    {  485, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  485, 1570,  624297,  624313,  624336, 16, 23, 25, }, // Sakhalin
    {  485, 1579,   63581,  624361,  624382, 14, 21, 23, }, // Samara
    {  485, 1596,   63641,  624405,  624425, 13, 20, 22, }, // Samoa
    {  485, 1602,       0,  624447,       0,  0, 18,  0, }, // Seychelles
    {  485, 1624,       0,  624465,       0,  0, 24,  0, }, // Singapore
    {  485, 1634,       0,  624489,       0,  0, 22,  0, }, // Solomon
    {  485, 1642,       0,  624511,       0,  0, 22,  0, }, // South_Georgia
    {  485, 1656,       0,  624533,       0,  0, 16,  0, }, // Suriname
    {  485, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  485, 1682,       0,  624549,       0,  0, 13,  0, }, // Tahiti
    {  485, 1689,   63834,  624562,  624582, 13, 20, 22, }, // Taipei
    {  485, 1696,       0,  624604,       0,  0, 19,  0, }, // Tajikistan
    {  485, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  485, 1732,   63929,  624623,  624643, 13, 20, 22, }, // Tonga
    {  485, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  485, 1750,  624665,  624687,  624716, 22, 29, 31, }, // Turkmenistan
    {  485, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  485, 1777,  624747,  624762,  624784, 15, 22, 24, }, // Uruguay
    {  485, 1792,  624808,  624827,  624853, 19, 26, 28, }, // Uzbekistan
    {  485, 1803,  624881,  624896,  624918, 15, 22, 24, }, // Vanuatu
    {  485, 1811,       0,  624942,       0,  0, 17,  0, }, // Venezuela
    {  485, 1821,   64318,  624959,  624985, 19, 26, 28, }, // Vladivostok
    {  485, 1833,  625013,  625031,  625056, 18, 25, 27, }, // Volgograd
    {  485, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  485, 1850,       0,  625083,       0,  0, 17,  0, }, // Wake
    {  485, 1855,       0,  625100,       0,  0, 23,  0, }, // Wallis
    {  485, 1862,   64520,  625123,  625145, 15, 22, 24, }, // Yakutsk
    {  485, 1870,  625169,  625190,  625218, 21, 28, 30, }, // Yekaterinburg
    {  485, 1892,       0,  625248,       0,  0, 13,  0, }, // Yukon
    {  486,    0,  618348,  618360,  618379, 12, 19, 21, }, // Acre Portuguese/Latin/Timor-Leste
    {  486,    5,       0,  618400,       0,  0, 19,  0, }, // Afghanistan
    {  486,   17,       0,  618419,       0,  0, 22,  0, }, // Africa_Central
    {  486,   32,       0,  618441,       0,  0, 23,  0, }, // Africa_Eastern
    {  486,   65,       0,  618464,       0,  0, 21,  0, }, // Africa_Southern
    {  486,   81,  618485,  618509,  618540, 24, 31, 33, }, // Africa_Western
    {  486,  107,  618573,  618587,  618608, 14, 21, 23, }, // Alaska
    {  486,  128,  618631,  618645,  618666, 14, 21, 23, }, // Almaty
    {  486,  135,  618689,  618705,  618728, 16, 23, 25, }, // Amazon
    {  486,  142,  618753,  618781,  618816, 28, 35, 37, }, // America_Central
    {  486,  158,  618853,  618882,  618918, 29, 36, 38, }, // America_Eastern
    {  486,  174,  618956,  618988,  619027, 32, 39, 41, }, // America_Mountain
    {  486,  191,  619068,  619100,  619139, 32, 39, 41, }, // America_Pacific
    {  486,  207,  619180,  619194,  619215, 14, 21, 23, }, // Anadyr
    {  486,  214,  619238,  619250,  619269, 12, 19, 21, }, // Apia
    {  486,  219,  619290,  619303,  619323, 13, 20, 22, }, // Aqtau
    {  486,  225,  619345,  619359,  619380, 14, 21, 23, }, // Aqtobe
    {  486,  232,  619403,  619417,  619438, 14, 21, 23, }, // Arabian
    {  486,  240,  619461,  619478,  619502, 17, 24, 26, }, // Argentina
    {  486,  250,  619528,  619555,  619589, 27, 34, 36, }, // Argentina_Western
    {  486,  268,  619625,  619640,  619662, 15, 22, 24, }, // Armenia
    {  486,  286,  619686,  619703,  619727, 17, 24, 26, }, // Atlantic
    {  486,  295,  619753,  619778,  619810, 25, 32, 34, }, // Australia_Central
    {  486,  313,  619844,  619879,  619921, 35, 42, 44, }, // Australia_CentralWestern
    {  486,  338,  619965,  619991,  620024, 26, 33, 35, }, // Australia_Eastern
    {  486,  356,  620059,  620086,  620120, 27, 34, 36, }, // Australia_Western
    {  486,  374,  620156,  620174,  620199, 18, 25, 27, }, // Azerbaijan
    {  486,  385,  620226,  620241,  620263, 15, 22, 24, }, // Azores
    {  486,  397,  620287,  620306,  620332, 19, 26, 28, }, // Bangladesh
    {  486,  415,       0,  620360,       0,  0, 13,  0, }, // Bhutan
    {  486,  422,       0,  620373,       0,  0, 15,  0, }, // Bolivia
    {  486,  437,  139322,  620388,  620411, 16, 23, 25, }, // Brasilia
    {  486,  454,       0,  620436,       0,  0, 25,  0, }, // Brunei
    {  486,  461,  620461,  620479,  620504, 18, 25, 27, }, // Cape_Verde
    {  486,  478,       0,  620531,       0,  0, 23,  0, }, // Chamorro
    {  486,  487,   59239,  620554,  620576, 15, 22, 24, }, // Chatham
    {  486,  495,  620600,  620613,  620633, 13, 20, 22, }, // Chile
    {  486,  501,  620655,  620668,  620688, 13, 20, 22, }, // China
    {  486,  507,       0,  620710,       0,  0, 21,  0, }, // Christmas
    {  486,  517,       0,  620731,       0,  0, 20,  0, }, // Cocos
    {  486,  523,  620751,  620767,  620790, 16, 23, 25, }, // Colombia
    {  486,  532,  620815,  620834,  620860, 19, 26, 28, }, // Cook
    {  486,  537,   59634,  620888,  620907, 12, 19, 21, }, // Cuba
    {  486,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  486,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  486,  601,       0,  620928,       0,  0, 19,  0, }, // East_Timor
    {  486,  612,  620947,  620969,  620998, 22, 29, 31, }, // Easter
    {  486,  619,       0,  621029,       0,  0, 15,  0, }, // Ecuador
    {  486,  627,  621044,  621066,  621095, 22, 29, 31, }, // Europe_Central
    {  486,  642,  621126,  621149,  621179, 23, 30, 32, }, // Europe_Eastern
    {  486,  657,       0,  621211,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  486,  680,  621242,  621266,  621297, 24, 31, 33, }, // Europe_Western
    {  486,  695,  621330,  621353,  621383, 23, 30, 32, }, // Falkland
    {  486,  704,  140424,  621415,  621434, 12, 19, 21, }, // Fiji
    {  486,  709,       0,  621455,       0,  0, 23,  0, }, // French_Guiana
    {  486,  723,       0,  621478,       0,  0, 48,  0, }, // French_Southern
    {  486,  746,       0,  621526,       0,  0, 18,  0, }, // Galapagos
    {  486,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  486,  764,  621544,  621559,  621581, 15, 22, 24, }, // Georgia
    {  486,  772,       0,  621605,       0,  0, 22,  0, }, // Gilbert_Islands
    {  486,  788,       0,  621627,       0,  0, 17,  0, }, // GMT
    {  486,  830,  621644,  621672,  621707, 28, 35, 37, }, // Greenland_Eastern
    {  486,  848,  621744,  621773,  621809, 29, 36, 38, }, // Greenland_Western
    {  486,  866,       0,  621847,       0,  0, 19,  0, }, // Guam
    {  486,  871,       0,  621866,       0,  0, 20,  0, }, // Gulf
    {  486,  876,       0,  621455,       0,  0, 14,  0, }, // Guyana
    {  486,  883,  621886,  621909,  621939, 23, 30, 32, }, // Hawaii_Aleutian
    {  486,  899,  141072,  621971,  621995, 17, 24, 26, }, // Hong_Kong
    {  486,  909,   60912,  622021,  622040, 12, 19, 21, }, // Hovd
    {  486,  914,       0,  622061,       0,  0, 20,  0, }, // India
    {  486,  920,       0,  622081,       0,  0, 21,  0, }, // Indian_Ocean
    {  486,  933,       0,  622102,       0,  0, 17,  0, }, // Indochina
    {  486,  943,       0,  622119,       0,  0, 25,  0, }, // Indonesia_Central
    {  486,  961,       0,  622144,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  486,  979,       0,  622170,       0,  0, 27,  0, }, // Indonesia_Western
    {  486,  997,  622197,  622209,  622228, 12, 19, 21, }, // Iran
    {  486, 1008,  622249,  622264,  622286, 15, 22, 24, }, // Irkutsk
    {  486, 1016,  622310,  622324,  622345, 14, 21, 23, }, // Israel
    {  486, 1023,  622368,  622381,  622401, 13, 20, 22, }, // Japan
    {  486, 1029,  622423,  622455,  622494, 32, 39, 41, }, // Kamchatka
    {  486, 1047,       0,  622535,       0,  0, 19,  0, }, // Kazakhstan
    {  486, 1058,       0,  622554,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  486, 1077,       0,  622582,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  486, 1106,  622611,  622625,  622646, 14, 21, 23, }, // Korea
    {  486, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  486, 1119,   61570,  622669,  622695, 19, 26, 28, }, // Krasnoyarsk
    {  486, 1151,       0,  622723,       0,  0, 19,  0, }, // Kyrgystan
    {  486, 1161,       0,  622742,       0,  0, 17,  0, }, // Lanka
    {  486, 1175,       0,  622759,       0,  0, 19,  0, }, // Line_Islands
    {  486, 1188,   61700,  622778,  622802, 17, 24, 26, }, // Lord_Howe
    {  486, 1198,  141929,  622828,  622848, 13, 20, 22, }, // Macau
    {  486, 1204,  141986,  622870,  622892, 15, 22, 24, }, // Magadan
    {  486, 1219,       0,  622916,       0,  0, 15,  0, }, // Malaysia
    {  486, 1228,       0,  622931,       0,  0, 17,  0, }, // Maldives
    {  486, 1237,       0,  622948,       0,  0, 24,  0, }, // Marquesas
    {  486, 1247,       0,  622972,       0,  0, 23,  0, }, // Marshall_Islands
    {  486, 1264,  622995,  623011,  623034, 16, 23, 25, }, // Mauritius
    {  486, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  486, 1281,  623059,  623084,  623116, 25, 32, 34, }, // Mexico_Pacific
    {  486, 1296,  623150,  623168,  623193, 18, 25, 27, }, // Mongolia
    {  486, 1305,  623220,  623235,  623257, 15, 22, 24, }, // Moscow
    {  486, 1312,       0,  623281,       0,  0, 15,  0, }, // Myanmar
    {  486, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  486, 1326,       0,  623296,       0,  0, 13,  0, }, // Nepal
    {  486, 1332,  623309,  623331,  623360, 22, 29, 31, }, // New_Caledonia
    {  486, 1346,  623391,  623412,  623440, 21, 28, 30, }, // New_Zealand
    {  486, 1358,  623470,  623488,  623513, 18, 25, 27, }, // Newfoundland
    {  486, 1371,       0,  623540,       0,  0, 12,  0, }, // Niue
    {  486, 1376,  623552,  623572,  623599, 20, 27, 29, }, // Norfolk
    {  486, 1384,   62667,  623628,  623662, 27, 34, 36, }, // Noronha
    {  486, 1392,       0,  623698,       0,  0, 31,  0, }, // North_Mariana
    {  486, 1406,   62803,  623729,  623755, 19, 26, 28, }, // Novosibirsk
    {  486, 1418,  623783,  623795,  623814, 12, 19, 21, }, // Omsk
    {  486, 1428,  623835,  623852,  623876, 17, 24, 26, }, // Pakistan
    {  486, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  486, 1443,       0,  623902,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  486, 1460,  623926,  623942,  623965, 16, 23, 25, }, // Paraguay
    {  486, 1469,  623990,  624002,  624021, 12, 19, 21, }, // Peru
    {  486, 1474,  624042,  624060,  624085, 18, 25, 27, }, // Philippines
    {  486, 1486,       0,  624112,       0,  0, 20,  0, }, // Phoenix_Islands
    {  486, 1502,  624132,  624160,  624195, 28, 35, 37, }, // Pierre_Miquelon
    {  486, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  486, 1527,       0,   63388,       0,  0, 14,  0, }, // Ponape
    {  486, 1534,       0,  143316,       0,  0, 17,  0, }, // Pyongyang
    {  486, 1544,   63419,  624232,  624256, 17, 24, 26, }, // Qyzylorda
    {  486, 1554,       0,  624282,       0,  0, 15,  0, }, // Reunion
    {  486, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  486, 1570,  624297,  624313,  624336, 16, 23, 25, }, // Sakhalin
    {  486, 1579,   63581,  624361,  624382, 14, 21, 23, }, // Samara
    {  486, 1596,   63641,  624405,  624425, 13, 20, 22, }, // Samoa
    {  486, 1602,       0,  624447,       0,  0, 18,  0, }, // Seychelles
    {  486, 1624,       0,  624465,       0,  0, 24,  0, }, // Singapore
    {  486, 1634,       0,  624489,       0,  0, 22,  0, }, // Solomon
    {  486, 1642,       0,  624511,       0,  0, 22,  0, }, // South_Georgia
    {  486, 1656,       0,  624533,       0,  0, 16,  0, }, // Suriname
    {  486, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  486, 1682,       0,  624549,       0,  0, 13,  0, }, // Tahiti
    {  486, 1689,   63834,  624562,  624582, 13, 20, 22, }, // Taipei
    {  486, 1696,       0,  624604,       0,  0, 19,  0, }, // Tajikistan
    {  486, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  486, 1732,   63929,  624623,  624643, 13, 20, 22, }, // Tonga
    {  486, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  486, 1750,  624665,  624687,  624716, 22, 29, 31, }, // Turkmenistan
    {  486, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  486, 1777,  624747,  624762,  624784, 15, 22, 24, }, // Uruguay
    {  486, 1792,  624808,  624827,  624853, 19, 26, 28, }, // Uzbekistan
    {  486, 1803,  624881,  624896,  624918, 15, 22, 24, }, // Vanuatu
    {  486, 1811,       0,  624942,       0,  0, 17,  0, }, // Venezuela
    {  486, 1821,   64318,  624959,  624985, 19, 26, 28, }, // Vladivostok
    {  486, 1833,  625013,  625031,  625056, 18, 25, 27, }, // Volgograd
    {  486, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  486, 1850,       0,  625083,       0,  0, 17,  0, }, // Wake
    {  486, 1855,       0,  625100,       0,  0, 23,  0, }, // Wallis
    {  486, 1862,   64520,  625123,  625145, 15, 22, 24, }, // Yakutsk
    {  486, 1870,  625169,  625190,  625218, 21, 28, 30, }, // Yekaterinburg
    {  486, 1892,       0,  625248,       0,  0, 13,  0, }, // Yukon
    {  487,  142,  625261,  625285,  625316, 24, 31, 31, }, // America_Central Prussian/Latin/Poland
    {  487,  158,  625347,  625371,  625402, 24, 31, 31, }, // America_Eastern
    {  487,  174,  625433,  625454,  625482, 21, 28, 28, }, // America_Mountain
    {  487,  191,  625510,  625535,  625567, 25, 32, 32, }, // America_Pacific
    {  487,  286,  625599,  625615,  625638, 16, 23, 23, }, // Atlantic
    {  487,  627,  625661,  625684,  625714, 23, 30, 30, }, // Europe_Central
    {  487,  642,  625744,  625767,  625797, 23, 30, 30, }, // Europe_Eastern
    {  487,  680,  625827,  625852,  625884, 25, 32, 32, }, // Europe_Western
    {  487,  788,       0,  625916,       0,  0, 15,  0, }, // GMT
    {  488,    5,       0,  625931,       0,  0, 16,  0, }, // Afghanistan Punjabi/Gurmukhi/India
    {  488,   17,       0,  625947,       0,  0, 18,  0, }, // Africa_Central
    {  488,   32,       0,  625965,       0,  0, 17,  0, }, // Africa_Eastern
    {  488,   65,       0,  625982,       0,  0, 24,  0, }, // Africa_Southern
    {  488,   81,  626006,  626023,  626046, 17, 23, 27, }, // Africa_Western
    {  488,  107,  626073,  626084,  626101, 11, 17, 19, }, // Alaska
    {  488,  128,  626120,  626131,  626148, 11, 17, 21, }, // Almaty
    {  488,  135,  626169,  626180,  626197, 11, 17, 21, }, // Amazon
    {  488,  142,  626218,  626242,  626272, 24, 30, 32, }, // America_Central
    {  488,  158,  626304,  626327,  626356, 23, 29, 31, }, // America_Eastern
    {  488,  174,  626387,  626412,  626443, 25, 31, 33, }, // America_Mountain
    {  488,  191,  626476,  626501,  626532, 25, 31, 33, }, // America_Pacific
    {  488,  214,  626565,  626574,  626589,  9, 15, 17, }, // Apia
    {  488,  219,  626606,  626616,  626632, 10, 16, 20, }, // Aqtau
    {  488,  225,  626652,  626662,  626678, 10, 16, 20, }, // Aqtobe
    {  488,  232,  626698,  626707,  626722,  9, 15, 17, }, // Arabian
    {  488,  240,  626739,  626752,  626771, 13, 19, 23, }, // Argentina
    {  488,  250,  626794,  626813,  626838, 19, 25, 29, }, // Argentina_Western
    {  488,  268,  626867,  626879,  626897, 12, 18, 22, }, // Armenia
    {  488,  286,  626919,  626932,  626951, 13, 19, 21, }, // Atlantic
    {  488,  295,  626972,  626994,  627022, 22, 28, 30, }, // Australia_Central
    {  488,  313,  627052,  627080,  627114, 28, 34, 36, }, // Australia_CentralWestern
    {  488,  338,  627150,  627171,  627198, 21, 27, 29, }, // Australia_Eastern
    {  488,  356,  627227,  627248,  627275, 21, 27, 29, }, // Australia_Western
    {  488,  374,  627304,  627319,  627340, 15, 21, 25, }, // Azerbaijan
    {  488,  385,  627365,  627376,  627393, 11, 17, 21, }, // Azores
    {  488,  397,  627414,  627428,  627448, 14, 20, 24, }, // Bangladesh
    {  488,  415,       0,  627472,       0,  0, 10,  0, }, // Bhutan
    {  488,  422,       0,  627482,       0,  0, 12,  0, }, // Bolivia
    {  488,  437,  627494,  627509,  627530, 15, 21, 25, }, // Brasilia
    {  488,  454,       0,  627555,       0,  0, 20,  0, }, // Brunei
    {  488,  461,  627575,  627587,  627605, 12, 18, 22, }, // Cape_Verde
    {  488,  472,       0,  627627,       0,  0,  9,  0, }, // Casey
    {  488,  478,       0,  627636,       0,  0, 17,  0, }, // Chamorro
    {  488,  487,  627653,  627662,  627677,  9, 15, 17, }, // Chatham
    {  488,  495,  627694,  627703,  627718,  9, 15, 19, }, // Chile
    {  488,  501,  627737,  627745,  627759,  8, 14, 16, }, // China
    {  488,  507,       0,  627775,       0,  0, 19,  0, }, // Christmas
    {  488,  517,       0,  627794,       0,  0, 16,  0, }, // Cocos
    {  488,  523,  627810,  627822,  627840, 12, 18, 22, }, // Colombia
    {  488,  532,  627862,  627878,  627900, 16, 22, 30, }, // Cook
    {  488,  537,  627930,  627940,  627956, 10, 16, 18, }, // Cuba
    {  488,  548,       0,  627974,       0,  0, 10,  0, }, // Davis
    {  488,  564,       0,  627984,       0,  0, 21,  0, }, // DumontDUrville
    {  488,  601,       0,  628005,       0,  0, 16,  0, }, // East_Timor
    {  488,  612,  628021,  628037,  628059, 16, 22, 26, }, // Easter
    {  488,  619,       0,  628085,       0,  0, 12,  0, }, // Ecuador
    {  488,  627,  628097,  628111,  628131, 14, 20, 24, }, // Europe_Central
    {  488,  642,  628155,  628171,  628193, 16, 22, 26, }, // Europe_Eastern
    {  488,  657,       0,  628219,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  488,  680,  628239,  628255,  628277, 16, 22, 26, }, // Europe_Western
    {  488,  695,  628303,  628324,  628351, 21, 27, 31, }, // Falkland
    {  488,  704,  628382,  628393,  628410, 11, 17, 21, }, // Fiji
    {  488,  709,       0,  628431,       0,  0, 17,  0, }, // French_Guiana
    {  488,  723,       0,  628448,       0,  0, 30,  0, }, // French_Southern
    {  488,  746,       0,  628478,       0,  0, 13,  0, }, // Galapagos
    {  488,  756,       0,  628491,       0,  0, 12,  0, }, // Gambier
    {  488,  764,  628503,  628514,  628531, 11, 17, 21, }, // Georgia
    {  488,  772,       0,  628552,       0,  0, 18,  0, }, // Gilbert_Islands
    {  488,  788,       0,  628570,       0,  0, 17,  0, }, // GMT
    {  488,  830,  628587,  628607,  628633, 20, 26, 30, }, // Greenland_Eastern
    {  488,  848,  628663,  628683,  628709, 20, 26, 30, }, // Greenland_Western
    {  488,  866,       0,  628739,       0,  0,  9,  0, }, // Guam
    {  488,  871,       0,  628748,       0,  0, 15,  0, }, // Gulf
    {  488,  876,       0,  628763,       0,  0, 11,  0, }, // Guyana
    {  488,  883,  628774,  628794,  628820, 20, 26, 28, }, // Hawaii_Aleutian
    {  488,  899,  628848,  628862,  628882, 14, 20, 24, }, // Hong_Kong
    {  488,  909,  628906,  628915,  628930,  9, 15, 19, }, // Hovd
    {  488,  914,       0,  628949,       0,  0, 16,  0, }, // India
    {  488,  920,       0,  628965,       0,  0, 18,  0, }, // Indian_Ocean
    {  488,  933,       0,  628983,       0,  0, 14,  0, }, // Indochina
    {  488,  943,       0,  628997,       0,  0, 20,  0, }, // Indonesia_Central
    {  488,  961,       0,  629017,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  488,  979,       0,  629038,       0,  0, 21,  0, }, // Indonesia_Western
    {  488,  997,  629059,  629068,  629083,  9, 15, 17, }, // Iran
    {  488, 1008,  629100,  629112,  629130, 12, 18, 22, }, // Irkutsk
    {  488, 1016,  629152,  629164,  629182, 12, 18, 20, }, // Israel
    {  488, 1023,  629202,  629211,  629226,  9, 15, 17, }, // Japan
    {  488, 1047,       0,  629243,       0,  0, 15,  0, }, // Kazakhstan
    {  488, 1058,       0,  629258,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  488, 1077,       0,  629279,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  488, 1106,  629300,  629311,  629328, 11, 17, 19, }, // Korea
    {  488, 1112,       0,  629347,       0,  0, 10,  0, }, // Kosrae
    {  488, 1119,  629357,  629374,  629397, 17, 23, 27, }, // Krasnoyarsk
    {  488, 1151,       0,  629424,       0,  0, 14,  0, }, // Kyrgystan
    {  488, 1161,       0,  629438,       0,  0,  9,  0, }, // Lanka
    {  488, 1175,       0,  629447,       0,  0, 16,  0, }, // Line_Islands
    {  488, 1188,  629463,  629477,  629497, 14, 20, 22, }, // Lord_Howe
    {  488, 1198,  629519,  629528,  629543,  9, 15, 19, }, // Macau
    {  488, 1204,  629562,  629573,  629590, 11, 17, 21, }, // Magadan
    {  488, 1219,       0,  629611,       0,  0, 12,  0, }, // Malaysia
    {  488, 1228,       0,  629623,       0,  0, 11,  0, }, // Maldives
    {  488, 1237,       0,  629634,       0,  0, 13,  0, }, // Marquesas
    {  488, 1247,       0,  629647,       0,  0, 18,  0, }, // Marshall_Islands
    {  488, 1264,  629665,  629677,  629695, 12, 18, 22, }, // Mauritius
    {  488, 1274,       0,  629717,       0,  0,  9,  0, }, // Mawson
    {  488, 1281,  629726,  629746,  629772, 20, 26, 28, }, // Mexico_Pacific
    {  488, 1296,  629800,  629813,  629832, 13, 19, 23, }, // Mongolia
    {  488, 1305,  629855,  629865,  629881, 10, 16, 20, }, // Moscow
    {  488, 1312,       0,  629901,       0,  0, 12,  0, }, // Myanmar
    {  488, 1320,       0,  629913,       0,  0, 10,  0, }, // Nauru
    {  488, 1326,       0,  629923,       0,  0, 10,  0, }, // Nepal
    {  488, 1332,  629933,  629951,  629975, 18, 24, 28, }, // New_Caledonia
    {  488, 1346,  630003,  630018,  630039, 15, 21, 23, }, // New_Zealand
    {  488, 1358,  630062,  630079,  630102, 17, 23, 25, }, // Newfoundland
    {  488, 1371,       0,  630127,       0,  0,  8,  0, }, // Niue
    {  488, 1376,  630135,  630153,  630177, 18, 24, 26, }, // Norfolk
    {  488, 1384,  630203,  630226,  630255, 23, 29, 33, }, // Noronha
    {  488, 1392,       0,  630288,       0,  0, 24,  0, }, // North_Mariana
    {  488, 1406,  630312,  630328,  630350, 16, 22, 26, }, // Novosibirsk
    {  488, 1418,  630376,  630385,  630400,  9, 15, 19, }, // Omsk
    {  488, 1428,  630419,  630432,  630451, 13, 19, 23, }, // Pakistan
    {  488, 1437,       0,  630474,       0,  0,  9,  0, }, // Palau
    {  488, 1443,       0,  630483,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  488, 1460,  630502,  630514,  630532, 12, 18, 22, }, // Paraguay
    {  488, 1469,  630554,  630563,  630578,  9, 15, 19, }, // Peru
    {  488, 1474,  630597,  630610,  630629, 13, 19, 23, }, // Philippines
    {  488, 1486,       0,  630652,       0,  0, 18,  0, }, // Phoenix_Islands
    {  488, 1502,  630670,  630696,  630728, 26, 32, 34, }, // Pierre_Miquelon
    {  488, 1518,       0,  630762,       0,  0, 12,  0, }, // Pitcairn
    {  488, 1527,       0,  630774,       0,  0, 11,  0, }, // Ponape
    {  488, 1534,       0,  630785,       0,  0, 14,  0, }, // Pyongyang
    {  488, 1544,  630799,  630813,  630833, 14, 20, 24, }, // Qyzylorda
    {  488, 1554,       0,  630857,       0,  0, 13,  0, }, // Reunion
    {  488, 1562,       0,  630870,       0,  0, 11,  0, }, // Rothera
    {  488, 1570,  630881,  630891,  630907, 10, 16, 20, }, // Sakhalin
    {  488, 1596,  630927,  630937,  630953, 10, 16, 18, }, // Samoa
    {  488, 1602,       0,  630971,       0,  0, 11,  0, }, // Seychelles
    {  488, 1624,       0,  630982,       0,  0, 19,  0, }, // Singapore
    {  488, 1634,       0,  631001,       0,  0, 19,  0, }, // Solomon
    {  488, 1642,       0,  631020,       0,  0, 17,  0, }, // South_Georgia
    {  488, 1656,       0,  631037,       0,  0, 12,  0, }, // Suriname
    {  488, 1676,       0,  631049,       0,  0, 10,  0, }, // Syowa
    {  488, 1682,       0,  631059,       0,  0, 11,  0, }, // Tahiti
    {  488, 1689,  631070,  631079,  631094,  9, 15, 17, }, // Taipei
    {  488, 1696,       0,  631111,       0,  0, 15,  0, }, // Tajikistan
    {  488, 1724,       0,  631126,       0,  0, 12,  0, }, // Tokelau
    {  488, 1732,  631138,  631148,  631164, 10, 16, 20, }, // Tonga
    {  488, 1738,       0,  631184,       0,  0,  8,  0, }, // Truk
    {  488, 1750,  631192,  631209,  631232, 17, 23, 27, }, // Turkmenistan
    {  488, 1763,       0,  631259,       0,  0, 11,  0, }, // Tuvalu
    {  488, 1777,  631270,  631281,  631298, 11, 17, 21, }, // Uruguay
    {  488, 1792,  631319,  631335,  631357, 16, 22, 26, }, // Uzbekistan
    {  488, 1803,  631383,  631395,  631413, 12, 18, 22, }, // Vanuatu
    {  488, 1811,       0,  631435,       0,  0, 15,  0, }, // Venezuela
    {  488, 1821,  631450,  631465,  631486, 15, 21, 25, }, // Vladivostok
    {  488, 1833,  631511,  631526,  631547, 15, 21, 25, }, // Volgograd
    {  488, 1843,       0,  631572,       0,  0, 11,  0, }, // Vostok
    {  488, 1850,       0,  631583,       0,  0, 15,  0, }, // Wake
    {  488, 1855,       0,  631598,       0,  0, 21,  0, }, // Wallis
    {  488, 1862,  631619,  631630,  631647, 11, 17, 21, }, // Yakutsk
    {  488, 1870,  631668,  631683,  631704, 15, 21, 25, }, // Yekaterinburg
    {  488, 1892,       0,  631729,       0,  0, 10,  0, }, // Yukon
    {  490,    5,       0,  631739,       0,  0, 18,  0, }, // Afghanistan Quechua/Latin/Peru
    {  490,   17,       0,  631757,       0,  0, 22,  0, }, // Africa_Central
    {  490,   32,       0,  631779,       0,  0, 23,  0, }, // Africa_Eastern
    {  490,   65,       0,  631802,       0,  0, 17,  0, }, // Africa_Southern
    {  490,   81,  631819,  631844,  631878, 25, 34, 44, }, // Africa_Western
    {  490,  107,  631922,  631936,  631959, 14, 23, 24, }, // Alaska
    {  490,  135,  631983,  631999,  632024, 16, 25, 26, }, // Amazon
    {  490,  142,  632050,  632062,  632084, 12, 22, 22, }, // America_Central
    {  490,  158,  632106,  632119,  632141, 13, 22, 23, }, // America_Eastern
    {  490,  174,  632164,  632182,  632209, 18, 27, 28, }, // America_Mountain
    {  490,  191,  632237,  632254,  632280, 17, 26, 27, }, // America_Pacific
    {  490,  214,  619238,  632307,  632328, 12, 21, 22, }, // Apia
    {  490,  232,  632350,  632364,  632387, 14, 23, 24, }, // Arabian
    {  490,  240,  632411,  632428,  632454, 17, 26, 27, }, // Argentina
    {  490,  250,  632481,  632508,  632544, 27, 36, 37, }, // Argentina_Western
    {  490,  268,  632581,  632596,  632620, 15, 24, 25, }, // Armenia
    {  490,  286,  632645,  632663,  632690, 18, 27, 28, }, // Atlantic
    {  490,  295,  632718,  632743,  632777, 25, 34, 35, }, // Australia_Central
    {  490,  313,  632812,  632848,  632893, 36, 45, 46, }, // Australia_CentralWestern
    {  490,  338,  632939,  632965,  633000, 26, 35, 36, }, // Australia_Eastern
    {  490,  356,  633036,  633064,  633101, 28, 37, 38, }, // Australia_Western
    {  490,  374,  633139,  633157,  633184, 18, 27, 28, }, // Azerbaijan
    {  490,  385,  633212,  633230,  633257, 18, 27, 28, }, // Azores
    {  490,  397,  139221,  633285,  633312, 18, 27, 28, }, // Bangladesh
    {  490,  415,       0,   59010,       0,  0, 13,  0, }, // Bhutan
    {  490,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia
    {  490,  437,   59038,  633340,  633365, 16, 25, 26, }, // Brasilia
    {  490,  454,       0,  139388,       0,  0, 25,  0, }, // Brunei
    {  490,  461,  620461,  633391,  633418, 18, 27, 28, }, // Cape_Verde
    {  490,  478,       0,  633446,       0,  0, 25,  0, }, // Chamorro
    {  490,  487,   59239,  633471,  633495, 15, 24, 25, }, // Chatham
    {  490,  495,   59302,  633520,  633542, 13, 22, 23, }, // Chile
    {  490,  501,   59359,  633565,  633587, 13, 22, 23, }, // China
    {  490,  507,       0,  633610,       0,  0, 22,  0, }, // Christmas
    {  490,  517,       0,  633632,       0,  0, 19,  0, }, // Cocos
    {  490,  523,   59475,  633651,  633676, 16, 25, 26, }, // Colombia
    {  490,  532,  633702,  633720,  633747, 18, 27, 28, }, // Cook
    {  490,  537,   59634,  633775,  633796, 12, 21, 22, }, // Cuba
    {  490,  548,       0,   59688,       0,  0, 13,  0, }, // Davis
    {  490,  564,       0,   59701,       0,  0, 24,  0, }, // DumontDUrville
    {  490,  601,       0,   59725,       0,  0, 22,  0, }, // East_Timor
    {  490,  612,  633818,  633840,  633871, 22, 31, 32, }, // Easter
    {  490,  619,       0,  633903,       0,  0, 15,  0, }, // Ecuador
    {  490,  627,  633918,  633940,  633971, 22, 31, 32, }, // Europe_Central
    {  490,  642,  634003,  634026,  634058, 23, 32, 33, }, // Europe_Eastern
    {  490,  657,       0,  634091,       0,  0, 37,  0, }, // Europe_Further_Eastern
    {  490,  680,  634128,  634153,  634187, 25, 34, 35, }, // Europe_Western
    {  490,  695,  634222,  634244,  634275, 22, 31, 32, }, // Falkland
    {  490,  704,  140424,  634307,  634328, 12, 21, 22, }, // Fiji
    {  490,  709,       0,  634350,       0,  0, 24,  0, }, // French_Guiana
    {  490,  723,       0,  634374,       0,  0, 35,  0, }, // French_Southern
    {  490,  746,       0,  634409,       0,  0, 23,  0, }, // Galapagos
    {  490,  756,       0,   60372,       0,  0, 15,  0, }, // Gambier
    {  490,  764,  634432,  634447,  634471, 15, 24, 25, }, // Georgia
    {  490,  772,       0,  634496,       0,  0, 21,  0, }, // Gilbert_Islands
    {  490,  788,       0,  634517,       0,  0, 31,  0, }, // GMT
    {  490,  830,   60496,  634548,  634576, 19, 28, 29, }, // Greenland_Eastern
    {  490,  848,  634605,  634635,  634674, 30, 39, 40, }, // Greenland_Western
    {  490,  871,       0,  634714,       0,  0, 23,  0, }, // Gulf
    {  490,  876,       0,  140990,       0,  0, 14,  0, }, // Guyana
    {  490,  883,  634737,  634760,  634792, 23, 32, 33, }, // Hawaii_Aleutian
    {  490,  899,  141072,  634825,  634851, 17, 26, 27, }, // Hong_Kong
    {  490,  909,   60912,  634878,  634899, 12, 21, 22, }, // Hovd
    {  490,  914,       0,  634921,       0,  0, 22,  0, }, // India
    {  490,  920,       0,  634943,       0,  0, 22,  0, }, // Indian_Ocean
    {  490,  933,       0,  634965,       0,  0, 17,  0, }, // Indochina
    {  490,  943,       0,  634982,       0,  0, 25,  0, }, // Indonesia_Central
    {  490,  961,       0,  635007,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  490,  979,       0,  635033,       0,  0, 28,  0, }, // Indonesia_Western
    {  490,  997,  635061,  635073,  635094, 12, 21, 22, }, // Iran
    {  490, 1008,  622249,  635116,  635140, 15, 24, 25, }, // Irkutsk
    {  490, 1016,  622310,  635165,  635188, 14, 23, 24, }, // Israel
    {  490, 1023,  635212,  635225,  635247, 13, 22, 23, }, // Japan
    {  490, 1047,       0,  635270,       0,  0, 18,  0, }, // Kazakhstan
    {  490, 1058,       0,  635288,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  490, 1077,       0,  635315,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  490, 1106,   61499,  635343,  635365, 13, 22, 23, }, // Korea
    {  490, 1112,       0,   61556,       0,  0, 14,  0, }, // Kosrae
    {  490, 1119,   61570,  635388,  635416, 19, 28, 29, }, // Krasnoyarsk
    {  490, 1151,       0,  635445,       0,  0, 18,  0, }, // Kyrgystan
    {  490, 1175,       0,  635463,       0,  0, 30,  0, }, // Line_Islands
    {  490, 1188,   61700,  635493,  635519, 17, 26, 27, }, // Lord_Howe
    {  490, 1204,  141986,  635546,  635570, 15, 24, 25, }, // Magadan
    {  490, 1219,       0,   61889,       0,  0, 15,  0, }, // Malaysia
    {  490, 1228,       0,  635595,       0,  0, 16,  0, }, // Maldives
    {  490, 1237,       0,  635611,       0,  0, 17,  0, }, // Marquesas
    {  490, 1247,       0,  635628,       0,  0, 22,  0, }, // Marshall_Islands
    {  490, 1264,  635650,  635666,  635691, 16, 25, 26, }, // Mauritius
    {  490, 1274,       0,   62038,       0,  0, 14,  0, }, // Mawson
    {  490, 1281,  635717,  635743,  635778, 26, 35, 36, }, // Mexico_Pacific
    {  490, 1296,  635814,  635832,  635859, 18, 27, 28, }, // Mongolia
    {  490, 1305,   62220,  635887,  635909, 13, 22, 23, }, // Moscow
    {  490, 1312,       0,   62277,       0,  0, 15,  0, }, // Myanmar
    {  490, 1320,       0,   62292,       0,  0, 13,  0, }, // Nauru
    {  490, 1326,       0,  635932,       0,  0, 13,  0, }, // Nepal
    {  490, 1332,   62319,  635945,  635977, 23, 32, 33, }, // New_Caledonia
    {  490, 1346,   62406,  636010,  636040, 21, 30, 31, }, // New_Zealand
    {  490, 1358,  142546,  636071,  636097, 17, 26, 27, }, // Newfoundland
    {  490, 1371,       0,   62565,       0,  0, 12,  0, }, // Niue
    {  490, 1376,  636124,  636147,  636179, 23, 32, 33, }, // Norfolk
    {  490, 1384,   62667,  636212,  636248, 27, 36, 37, }, // Noronha
    {  490, 1406,   62803,  636285,  636313, 19, 28, 29, }, // Novosibirsk
    {  490, 1418,  623783,  636342,  636363, 12, 21, 22, }, // Omsk
    {  490, 1428,  636385,  636401,  636426, 16, 25, 26, }, // Pakistan
    {  490, 1437,       0,   63001,       0,  0, 13,  0, }, // Palau
    {  490, 1443,       0,  636452,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  490, 1460,  636478,  636494,  636519, 16, 25, 26, }, // Paraguay
    {  490, 1469,  636545,  636557,  636578, 12, 21, 22, }, // Peru
    {  490, 1474,  636600,  636617,  636643, 17, 26, 27, }, // Philippines
    {  490, 1486,       0,  636670,       0,  0, 21,  0, }, // Phoenix_Islands
    {  490, 1502,  636691,  636719,  636756, 28, 37, 38, }, // Pierre_Miquelon
    {  490, 1518,       0,   63372,       0,  0, 16,  0, }, // Pitcairn
    {  490, 1527,       0,  636794,       0,  0, 15,  0, }, // Ponape
    {  490, 1534,       0,  636809,       0,  0, 16,  0, }, // Pyongyang
    {  490, 1554,       0,  636825,       0,  0, 15,  0, }, // Reunion
    {  490, 1562,       0,   63503,       0,  0, 15,  0, }, // Rothera
    {  490, 1570,  636840,  636856,  636881, 16, 25, 26, }, // Sakhalin
    {  490, 1596,   63641,  636907,  636929, 13, 22, 23, }, // Samoa
    {  490, 1602,       0,  636952,       0,  0, 18,  0, }, // Seychelles
    {  490, 1624,       0,  636970,       0,  0, 25,  0, }, // Singapore
    {  490, 1634,       0,  636995,       0,  0, 21,  0, }, // Solomon
    {  490, 1642,       0,  637016,       0,  0, 23,  0, }, // South_Georgia
    {  490, 1656,       0,  143574,       0,  0, 15,  0, }, // Suriname
    {  490, 1676,       0,   63807,       0,  0, 13,  0, }, // Syowa
    {  490, 1682,       0,   63820,       0,  0, 14,  0, }, // Tahiti
    {  490, 1689,   63834,  637039,  637062, 14, 23, 24, }, // Taipei
    {  490, 1696,       0,  637086,       0,  0, 18,  0, }, // Tajikistan
    {  490, 1724,       0,   63914,       0,  0, 15,  0, }, // Tokelau
    {  490, 1732,   63929,  637104,  637126, 13, 22, 23, }, // Tonga
    {  490, 1738,       0,   63986,       0,  0, 13,  0, }, // Truk
    {  490, 1750,  637149,  637169,  637198, 20, 29, 30, }, // Turkmenistan
    {  490, 1763,       0,   64080,       0,  0, 14,  0, }, // Tuvalu
    {  490, 1777,  637228,  637243,  637267, 15, 24, 25, }, // Uruguay
    {  490, 1792,  637292,  637310,  637337, 18, 27, 28, }, // Uzbekistan
    {  490, 1803,   64238,  637365,  637389, 15, 24, 25, }, // Vanuatu
    {  490, 1811,       0,   64301,       0,  0, 17,  0, }, // Venezuela
    {  490, 1821,   64318,  637414,  637442, 19, 28, 29, }, // Vladivostok
    {  490, 1833,  625013,  637471,  637498, 18, 27, 28, }, // Volgograd
    {  490, 1843,       0,   64462,       0,  0, 14,  0, }, // Vostok
    {  490, 1850,       0,  637526,       0,  0, 17,  0, }, // Wake
    {  490, 1855,       0,   64497,       0,  0, 23,  0, }, // Wallis
    {  490, 1862,   64520,  637543,  637567, 15, 24, 25, }, // Yakutsk
    {  490, 1870,  637592,  637613,  637643, 21, 30, 31, }, // Yekaterinburg
    {  490, 1892,       0,  637674,       0,  0,  9,  0, }, // Yukon
    {  491,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia Quechua/Latin/Bolivia
    {  491, 1469,  636545,  636557,  636578, 12, 21, 22, }, // Peru
    {  492,  619,       0,  633903,       0,  0, 15,  0, }, // Ecuador Quechua/Latin/Ecuador
    {  492, 1469,  636545,  636557,  636578, 12, 21, 22, }, // Peru
    {  493,    0,  637683,  637691,  637708,  8, 17, 16, }, // Acre Romanian/Latin/Romania
    {  493,    5,       0,  637724,       0,  0, 18,  0, }, // Afghanistan
    {  493,   17,       0,  637742,       0,  0, 20,  0, }, // Africa_Central
    {  493,   32,       0,  637762,       0,  0, 21,  0, }, // Africa_Eastern
    {  493,   65,       0,  637783,       0,  0, 23,  0, }, // Africa_Southern
    {  493,   81,  637806,  637829,  637863, 23, 34, 33, }, // Africa_Western
    {  493,  107,  637896,  637910,  637933, 14, 23, 22, }, // Alaska
    {  493,  128,  637955,  637965,  637984, 10, 19, 18, }, // Almaty
    {  493,  135,  638002,  638016,  638041, 14, 25, 24, }, // Amazon
    {  493,  142,  638065,  638092,  638128, 27, 36, 35, }, // America_Central
    {  493,  158,  638163,  638191,  638228, 28, 37, 36, }, // America_Eastern
    {  493,  174,  638264,  638296,  638339, 32, 43, 42, }, // America_Mountain
    {  493,  191,  638381,  638413,  638456, 32, 43, 42, }, // America_Pacific
    {  493,  207,  638498,  638512,  638535, 14, 23, 22, }, // Anadyr
    {  493,  214,  638557,  638569,  638590, 12, 21, 20, }, // Apia
    {  493,  219,  638610,  638619,  638637,  9, 18, 25, }, // Aqtau
    {  493,  225,  638662,  638672,  638691, 10, 19, 26, }, // Aqtobe
    {  493,  232,  638717,  638726,  638744,  9, 18, 17, }, // Arabian
    {  493,  240,  638761,  638775,  638800, 14, 25, 24, }, // Argentina
    {  493,  250,  638824,  638850,  638887, 26, 37, 36, }, // Argentina_Western
    {  493,  268,  638923,  638935,  638958, 12, 23, 22, }, // Armenia
    {  493,  286,  638980,  639013,  639057, 33, 44, 43, }, // Atlantic
    {  493,  295,  639100,  639123,  639157, 23, 34, 33, }, // Australia_Central
    {  493,  313,  639190,  639224,  639269, 34, 45, 44, }, // Australia_CentralWestern
    {  493,  338,  639313,  639337,  639372, 24, 35, 34, }, // Australia_Eastern
    {  493,  356,  639406,  639432,  639469, 26, 37, 36, }, // Australia_Western
    {  493,  374,  639505,  639524,  639554, 19, 30, 29, }, // Azerbaijan
    {  493,  385,  639583,  639596,  639618, 13, 22, 21, }, // Azores
    {  493,  397,  639639,  639657,  639684, 18, 27, 26, }, // Bangladesh
    {  493,  415,       0,  639710,       0,  0, 14,  0, }, // Bhutan
    {  493,  422,       0,  639724,       0,  0, 12,  0, }, // Bolivia
    {  493,  437,  639736,  639749,  639773, 13, 24, 23, }, // Brasilia
    {  493,  454,       0,  639796,       0,  0, 25,  0, }, // Brunei
    {  493,  461,  639821,  639840,  639868, 19, 28, 27, }, // Cape_Verde
    {  493,  478,       0,  639895,       0,  0, 16,  0, }, // Chamorro
    {  493,  487,  639911,  639926,  639950, 15, 24, 23, }, // Chatham
    {  493,  495,  639973,  639986,  640008, 13, 22, 21, }, // Chile
    {  493,  501,  640029,  640039,  640060, 10, 21, 20, }, // China
    {  493,  507,       0,  640080,       0,  0, 24,  0, }, // Christmas
    {  493,  517,       0,  640104,       0,  0, 19,  0, }, // Cocos
    {  493,  523,  640123,  640136,  640160, 13, 24, 23, }, // Colombia
    {  493,  532,  640183,  640201,  640230, 18, 29, 28, }, // Cook
    {  493,  537,  640258,  640267,  640287,  9, 20, 19, }, // Cuba
    {  493,  548,       0,  640306,       0,  0, 13,  0, }, // Davis
    {  493,  564,       0,  640319,       0,  0, 24,  0, }, // DumontDUrville
    {  493,  601,       0,  640343,       0,  0, 20,  0, }, // East_Timor
    {  493,  612,  640363,  640386,  640418, 23, 32, 31, }, // Easter
    {  493,  619,       0,  640449,       0,  0, 15,  0, }, // Ecuador
    {  493,  627,  640464,  640484,  640515, 20, 31, 30, }, // Europe_Central
    {  493,  642,  640545,  640563,  640592, 18, 29, 28, }, // Europe_Eastern
    {  493,  657,       0,  640620,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  493,  680,  640650,  640669,  640699, 19, 30, 29, }, // Europe_Western
    {  493,  695,  640728,  640753,  640787, 25, 34, 33, }, // Falkland
    {  493,  704,  640820,  640832,  640853, 12, 21, 20, }, // Fiji
    {  493,  709,       0,  640873,       0,  0, 23,  0, }, // French_Guiana
    {  493,  723,       0,  640896,       0,  0, 51,  0, }, // French_Southern
    {  493,  746,       0,  640947,       0,  0, 17,  0, }, // Galapagos
    {  493,  756,       0,  640964,       0,  0, 15,  0, }, // Gambier
    {  493,  764,  640979,  640991,  641014, 12, 23, 22, }, // Georgia
    {  493,  772,       0,  641036,       0,  0, 21,  0, }, // Gilbert_Islands
    {  493,  788,       0,  641057,       0,  0, 17,  0, }, // GMT
    {  493,  830,  641074,  641099,  641135, 25, 36, 35, }, // Greenland_Eastern
    {  493,  848,  641170,  641197,  641235, 27, 38, 37, }, // Greenland_Western
    {  493,  871,       0,  641272,       0,  0, 23,  0, }, // Gulf
    {  493,  876,       0,  640873,       0,  0, 14,  0, }, // Guyana
    {  493,  883,  641295,  641318,  641350, 23, 32, 31, }, // Hawaii_Aleutian
    {  493,  899,  641381,  641398,  641424, 17, 26, 25, }, // Hong_Kong
    {  493,  909,  641449,  641461,  641482, 12, 21, 20, }, // Hovd
    {  493,  914,       0,  641502,       0,  0, 10,  0, }, // India
    {  493,  920,       0,  641512,       0,  0, 20,  0, }, // Indian_Ocean
    {  493,  933,       0,  641532,       0,  0, 14,  0, }, // Indochina
    {  493,  943,       0,  641546,       0,  0, 23,  0, }, // Indonesia_Central
    {  493,  961,       0,  641569,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  493,  979,       0,  641590,       0,  0, 22,  0, }, // Indonesia_Western
    {  493,  997,  641612,  641624,  641647, 12, 23, 22, }, // Iran
    {  493, 1008,  641669,  641683,  641706, 14, 23, 22, }, // Irkutsk
    {  493, 1016,  641728,  641742,  641767, 14, 25, 24, }, // Israel
    {  493, 1023,  641791,  641803,  641826, 12, 23, 22, }, // Japan
    {  493, 1029,  641848,  641879,  641919, 31, 40, 39, }, // Kamchatka
    {  493, 1047,       0,  641958,       0,  0, 17,  0, }, // Kazakhstan
    {  493, 1058,       0,  641975,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  493, 1077,       0,  642001,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  493, 1106,  642028,  642038,  642059, 10, 21, 20, }, // Korea
    {  493, 1112,       0,  642079,       0,  0, 14,  0, }, // Kosrae
    {  493, 1119,  642093,  642112,  642140, 19, 28, 27, }, // Krasnoyarsk
    {  493, 1151,       0,  642167,       0,  0, 18,  0, }, // Kyrgystan
    {  493, 1175,       0,  642185,       0,  0, 21,  0, }, // Line_Islands
    {  493, 1188,  642206,  642223,  642249, 17, 26, 25, }, // Lord_Howe
    {  493, 1204,  642274,  642289,  642313, 15, 24, 23, }, // Magadan
    {  493, 1219,       0,  642336,       0,  0, 16,  0, }, // Malaysia
    {  493, 1228,       0,  642352,       0,  0, 15,  0, }, // Maldives
    {  493, 1237,       0,  642367,       0,  0, 22,  0, }, // Marquesas
    {  493, 1247,       0,  642389,       0,  0, 22,  0, }, // Marshall_Islands
    {  493, 1264,  642411,  642428,  642454, 17, 26, 25, }, // Mauritius
    {  493, 1274,       0,  642479,       0,  0, 14,  0, }, // Mawson
    {  493, 1281,  642493,  642519,  642556, 26, 37, 36, }, // Mexico_Pacific
    {  493, 1296,  642592,  642610,  642637, 18, 27, 26, }, // Mongolia
    {  493, 1305,  642663,  642675,  642698, 12, 23, 22, }, // Moscow
    {  493, 1312,       0,  642720,       0,  0, 15,  0, }, // Myanmar
    {  493, 1320,       0,  642735,       0,  0, 13,  0, }, // Nauru
    {  493, 1326,       0,  642748,       0,  0, 13,  0, }, // Nepal
    {  493, 1332,  642761,  642779,  642808, 18, 29, 28, }, // New_Caledonia
    {  493, 1346,  642836,  642853,  642881, 17, 28, 27, }, // New_Zealand
    {  493, 1358,  642908,  642928,  642957, 20, 29, 28, }, // Newfoundland
    {  493, 1371,       0,  642985,       0,  0, 12,  0, }, // Niue
    {  493, 1376,  642997,  643016,  643046, 19, 30, 29, }, // Norfolk
    {  493, 1384,  643075,  643102,  643138, 27, 36, 35, }, // Noronha
    {  493, 1406,  643173,  643192,  643220, 19, 28, 27, }, // Novosibirsk
    {  493, 1418,  643247,  643259,  643280, 12, 21, 20, }, // Omsk
    {  493, 1428,  643300,  643316,  643343, 16, 27, 26, }, // Pakistan
    {  493, 1437,       0,  643369,       0,  0, 13,  0, }, // Palau
    {  493, 1443,       0,  643382,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  493, 1460,  643407,  643423,  643448, 16, 25, 24, }, // Paraguay
    {  493, 1469,  643472,  643484,  643505, 12, 21, 20, }, // Peru
    {  493, 1474,  643525,  643541,  643566, 16, 25, 24, }, // Philippines
    {  493, 1486,       0,  643590,       0,  0, 21,  0, }, // Phoenix_Islands
    {  493, 1502,  643611,  643643,  643684, 32, 41, 40, }, // Pierre_Miquelon
    {  493, 1518,       0,  643724,       0,  0, 16,  0, }, // Pitcairn
    {  493, 1527,       0,  643740,       0,  0, 14,  0, }, // Ponape
    {  493, 1534,       0,  643754,       0,  0, 17,  0, }, // Pyongyang
    {  493, 1554,       0,  643771,       0,  0, 15,  0, }, // Reunion
    {  493, 1562,       0,  643786,       0,  0, 15,  0, }, // Rothera
    {  493, 1570,  643801,  643816,  643840, 15, 24, 23, }, // Sakhalin
    {  493, 1579,  643863,  643877,  643900, 14, 23, 22, }, // Samara
    {  493, 1596,  643922,  643935,  643957, 13, 22, 21, }, // Samoa
    {  493, 1602,       0,  643978,       0,  0, 18,  0, }, // Seychelles
    {  493, 1624,       0,  643996,       0,  0, 17,  0, }, // Singapore
    {  493, 1634,       0,  644013,       0,  0, 21,  0, }, // Solomon
    {  493, 1642,       0,  644034,       0,  0, 19,  0, }, // South_Georgia
    {  493, 1656,       0,  644053,       0,  0, 15,  0, }, // Suriname
    {  493, 1676,       0,  644068,       0,  0, 13,  0, }, // Syowa
    {  493, 1682,       0,  644081,       0,  0, 14,  0, }, // Tahiti
    {  493, 1689,  644095,  644109,  644132, 14, 23, 22, }, // Taipei
    {  493, 1696,       0,  644154,       0,  0, 19,  0, }, // Tajikistan
    {  493, 1724,       0,  644173,       0,  0, 15,  0, }, // Tokelau
    {  493, 1732,  644188,  644201,  644223, 13, 22, 21, }, // Tonga
    {  493, 1738,       0,  644244,       0,  0, 13,  0, }, // Truk
    {  493, 1750,  644257,  644277,  644306, 20, 29, 28, }, // Turkmenistan
    {  493, 1763,       0,  644334,       0,  0, 14,  0, }, // Tuvalu
    {  493, 1777,  644348,  644363,  644389, 15, 26, 25, }, // Uruguay
    {  493, 1792,  644414,  644432,  644459, 18, 27, 26, }, // Uzbekistan
    {  493, 1803,  644485,  644500,  644524, 15, 24, 23, }, // Vanuatu
    {  493, 1811,       0,  644547,       0,  0, 14,  0, }, // Venezuela
    {  493, 1821,  644561,  644580,  644608, 19, 28, 27, }, // Vladivostok
    {  493, 1833,  644635,  644652,  644678, 17, 26, 25, }, // Volgograd
    {  493, 1843,       0,  644703,       0,  0, 14,  0, }, // Vostok
    {  493, 1850,       0,  644717,       0,  0, 16,  0, }, // Wake
    {  493, 1855,       0,  644733,       0,  0, 24,  0, }, // Wallis
    {  493, 1862,  644757,  644771,  644794, 14, 23, 22, }, // Yakutsk
    {  493, 1870,  644816,  644836,  644865, 20, 29, 28, }, // Yekaterinburg
    {  493, 1892,       0,  644893,       0,  0, 13,  0, }, // Yukon
    {  495,  142,  644906,  644918,  644942, 12, 24, 20, }, // America_Central Romansh/Latin/Switzerland
    {  495,  158,  644962,  644975,  645000, 13, 25, 21, }, // America_Eastern
    {  495,  174,  645021,  645037,  645065, 16, 28, 24, }, // America_Mountain
    {  495,  191,  645089,  645101,  645125, 12, 24, 20, }, // America_Pacific
    {  495,  286,  645145,  645158,  645183, 13, 25, 21, }, // Atlantic
    {  495,  627,  645204,  645229,  645266, 25, 37, 33, }, // Europe_Central
    {  495,  642,  645299,  645325,  645363, 26, 38, 34, }, // Europe_Eastern
    {  495,  680,  645397,  645422,  645459, 25, 37, 33, }, // Europe_Western
    {  495,  788,       0,  645492,       0,  0, 14,  0, }, // GMT
    {  498,    0,  645506,  645516,  645538, 10, 22, 17, }, // Acre Russian/Cyrillic/Russia
    {  498,    5,       0,    6393,       0,  0, 10,  0, }, // Afghanistan
    {  498,   17,       0,  645555,       0,  0, 18,  0, }, // Africa_Central
    {  498,   32,       0,  645573,       0,  0, 16,  0, }, // Africa_Eastern
    {  498,   65,       0,  645589,       0,  0, 12,  0, }, // Africa_Southern
    {  498,   81,  645601,  645616,  645650, 15, 34, 29, }, // Africa_Western
    {  498,  107,  119466,  645679,  645704,  6, 25, 20, }, // Alaska
    {  498,  128,  645724,  645738,  645764, 14, 26, 21, }, // Almaty
    {  498,  135,    6664,  645785,  645812,  8, 27, 22, }, // Amazon
    {  498,  142,  645834,  645853,  645891, 19, 38, 33, }, // America_Central
    {  498,  158,  645924,  645941,  645977, 17, 36, 31, }, // America_Eastern
    {  498,  174,  646008,  646039,  646082, 31, 43, 38, }, // America_Mountain
    {  498,  191,  646120,  646139,  646170, 19, 31, 26, }, // America_Pacific
    {  498,  207,  646196,  646212,  646237, 16, 25, 20, }, // Anadyr
    {  498,  214,    7154,  646257,  646280,  4, 23, 18, }, // Apia
    {  498,  219,  646298,  646309,  646333, 11, 24, 18, }, // Aqtau
    {  498,  225,  646351,  646363,  646387, 12, 24, 19, }, // Aqtobe
    {  498,  232,  646406,  646423,  646459, 17, 36, 31, }, // Arabian
    {  498,  240,    7392,  646490,  646518,  9, 28, 23, }, // Argentina
    {  498,  250,  646541,  646559,  646596, 18, 37, 32, }, // Argentina_Western
    {  498,  268,  418437,  646628,  646654,  7, 26, 21, }, // Armenia
    {  498,  286,  646675,  646694,  646725, 19, 31, 26, }, // Atlantic
    {  498,  295,  646751,  646772,  646812, 21, 40, 35, }, // Australia_Central
    {  498,  313,  646847,  646884,  646933, 37, 49, 44, }, // Australia_CentralWestern
    {  498,  338,  646977,  646996,  647034, 19, 38, 33, }, // Australia_Eastern
    {  498,  356,  647067,  647085,  647122, 18, 37, 32, }, // Australia_Western
    {  498,  374,   86302,  647154,  647184, 11, 30, 25, }, // Azerbaijan
    {  498,  385,  647209,  647222,  647254, 13, 32, 27, }, // Azores
    {  498,  397,    8277,  647281,  647309,  9, 28, 23, }, // Bangladesh
    {  498,  415,       0,    8338,       0,  0,  5,  0, }, // Bhutan
    {  498,  422,       0,  419145,       0,  0,  7,  0, }, // Bolivia
    {  498,  437,  419159,  647332,  647359,  8, 27, 22, }, // Brasilia
    {  498,  454,       0,  167222,       0,  0, 17,  0, }, // Brunei
    {  498,  461,    8425,  647381,  647410, 10, 29, 24, }, // Cape_Verde
    {  498,  472,       0,  180513,       0,  0,  5,  0, }, // Casey
    {  498,  478,       0,    8494,       0,  0,  7,  0, }, // Chamorro
    {  498,  487,    8501,  647434,  647458,  5, 24, 19, }, // Chatham
    {  498,  495,    8550,  647477,  647500,  4, 23, 18, }, // Chile
    {  498,  501,  121647,  647518,  647542,  5, 24, 19, }, // China
    {  498,  507,       0,  647561,       0,  0, 13,  0, }, // Christmas
    {  498,  517,       0,  647574,       0,  0, 14,  0, }, // Cocos
    {  498,  523,  419543,  647588,  647615,  8, 27, 22, }, // Colombia
    {  498,  532,  647637,  647649,  647680, 12, 31, 30, }, // Cook
    {  498,  537,    8827,  647710,  647733,  4, 23, 18, }, // Cuba
    {  498,  548,       0,  121959,       0,  0,  6,  0, }, // Davis
    {  498,  564,       0,  167619,       0,  0, 14,  0, }, // DumontDUrville
    {  498,  601,       0,  647751,       0,  0, 15,  0, }, // East_Timor
    {  498,  612,  647766,  647775,  647803,  9, 28, 23, }, // Easter
    {  498,  619,       0,   87167,       0,  0,  7,  0, }, // Ecuador
    {  498,  627,  647826,  647844,  647881, 18, 37, 32, }, // Europe_Central
    {  498,  642,  647913,  647929,  647964, 16, 35, 30, }, // Europe_Eastern
    {  498,  657,       0,  647994,       0,  0, 16,  0, }, // Europe_Further_Eastern
    {  498,  680,  648010,  648025,  648059, 15, 34, 29, }, // Europe_Western
    {  498,  695,  648088,  648105,  648141, 17, 36, 31, }, // Falkland
    {  498,  704,  122514,  648172,  648196,  5, 24, 19, }, // Fiji
    {  498,  709,       0,  648215,       0,  0, 18,  0, }, // French_Guiana
    {  498,  723,       0,  648233,       0,  0, 45,  0, }, // French_Southern
    {  498,  746,       0,  648278,       0,  0, 18,  0, }, // Galapagos
    {  498,  756,       0,  168183,       0,  0,  6,  0, }, // Gambier
    {  498,  764,  420379,  648296,  648321,  6, 25, 20, }, // Georgia
    {  498,  772,       0,  648341,       0,  0, 13,  0, }, // Gilbert_Islands
    {  498,  788,       0,  648354,       0,  0, 25,  0, }, // GMT
    {  498,  830,  648379,  648399,  648437, 20, 38, 34, }, // Greenland_Eastern
    {  498,  848,  648471,  648490,  648528, 19, 38, 33, }, // Greenland_Western
    {  498,  866,       0,    9815,       0,  0,  4,  0, }, // Guam
    {  498,  871,       0,  648561,       0,  0, 16,  0, }, // Gulf
    {  498,  876,       0,  168475,       0,  0,  6,  0, }, // Guyana
    {  498,  883,  648577,  648601,  648637, 24, 36, 31, }, // Hawaii_Aleutian
    {  498,  899,  168558,  648668,  648694,  7, 26, 21, }, // Hong_Kong
    {  498,  909,    9979,  648715,  648738,  4, 23, 18, }, // Hovd
    {  498,  914,       0,  219975,       0,  0,  5,  0, }, // India
    {  498,  920,       0,  648756,       0,  0, 15,  0, }, // Indian_Ocean
    {  498,  933,       0,  123232,       0,  0,  9,  0, }, // Indochina
    {  498,  943,       0,  648771,       0,  0, 21,  0, }, // Indonesia_Central
    {  498,  961,       0,  648792,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  498,  979,       0,  648811,       0,  0, 18,  0, }, // Indonesia_Western
    {  498,  997,   10118,  648829,  648852,  4, 23, 18, }, // Iran
    {  498, 1008,   10164,  648870,  648896,  7, 26, 21, }, // Irkutsk
    {  498, 1016,   10219,  648917,  648943,  7, 26, 21, }, // Israel
    {  498, 1023,  220042,  648964,  648989,  6, 25, 20, }, // Japan
    {  498, 1029,  649009,  649033,  649076, 24, 43, 38, }, // Kamchatka
    {  498, 1047,       0,   88510,       0,  0,  9,  0, }, // Kazakhstan
    {  498, 1058,       0,  649114,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  498, 1077,       0,  649133,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  498, 1106,  421329,  649151,  649175,  5, 24, 19, }, // Korea
    {  498, 1112,       0,   10535,       0,  0,  6,  0, }, // Kosrae
    {  498, 1119,  123841,  649194,  649223, 10, 29, 24, }, // Krasnoyarsk
    {  498, 1151,       0,  649247,       0,  0,  8,  0, }, // Kyrgystan
    {  498, 1161,       0,   10616,       0,  0,  9,  0, }, // Lanka
    {  498, 1175,       0,  649255,       0,  0,  9,  0, }, // Line_Islands
    {  498, 1188,   88733,  649264,  649291,  8, 27, 22, }, // Lord_Howe
    {  498, 1198,   10696,  649313,  649337,  5, 24, 19, }, // Macau
    {  498, 1204,   10745,  649356,  649382,  7, 26, 21, }, // Magadan
    {  498, 1219,       0,  421621,       0,  0,  8,  0, }, // Malaysia
    {  498, 1228,       0,  649403,       0,  0,  8,  0, }, // Maldives
    {  498, 1237,       0,  649411,       0,  0, 15,  0, }, // Marquesas
    {  498, 1247,       0,  649426,       0,  0, 18,  0, }, // Marshall_Islands
    {  498, 1264,  183159,  649444,  649471,  8, 27, 22, }, // Mauritius
    {  498, 1274,       0,   10914,       0,  0,  6,  0, }, // Mawson
    {  498, 1281,  649493,  649525,  649569, 32, 44, 39, }, // Mexico_Pacific
    {  498, 1296,   11030,  649608,  649637, 10, 29, 24, }, // Mongolia
    {  498, 1305,   11094,  649661,  649686,  6, 25, 20, }, // Moscow
    {  498, 1312,       0,  169417,       0,  0,  6,  0, }, // Myanmar
    {  498, 1320,       0,   11152,       0,  0,  5,  0, }, // Nauru
    {  498, 1326,       0,   11157,       0,  0,  5,  0, }, // Nepal
    {  498, 1332,  649706,  649721,  649755, 15, 34, 29, }, // New_Caledonia
    {  498, 1346,  649784,  649798,  649831, 14, 33, 28, }, // New_Zealand
    {  498, 1358,  169564,  649859,  649890, 12, 31, 26, }, // Newfoundland
    {  498, 1371,       0,  169630,       0,  0,  4,  0, }, // Niue
    {  498, 1376,   11379,  649916,  649942,  7, 26, 21, }, // Norfolk
    {  498, 1384,  169678,  649963,  650001, 19, 38, 33, }, // Noronha
    {  498, 1392,       0,  650034,       0,  0, 24,  0, }, // North_Mariana
    {  498, 1406,   11552,  650058,  650088, 11, 30, 25, }, // Novosibirsk
    {  498, 1418,   11619,  650113,  650136,  4, 23, 18, }, // Omsk
    {  498, 1428,   11665,  650154,  650181,  8, 27, 22, }, // Pakistan
    {  498, 1437,       0,   11723,       0,  0,  5,  0, }, // Palau
    {  498, 1443,       0,  650203,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  498, 1460,  125159,  650223,  650250,  8, 27, 22, }, // Paraguay
    {  498, 1469,   11805,  650272,  650295,  4, 23, 18, }, // Peru
    {  498, 1474,  650313,  650322,  650350,  9, 28, 23, }, // Philippines
    {  498, 1486,       0,  650373,       0,  0, 11,  0, }, // Phoenix_Islands
    {  498, 1502,  650384,  650402,  650439, 18, 37, 32, }, // Pierre_Miquelon
    {  498, 1518,       0,  170172,       0,  0,  7,  0, }, // Pitcairn
    {  498, 1527,       0,   12019,       0,  0,  6,  0, }, // Ponape
    {  498, 1534,       0,   90148,       0,  0,  7,  0, }, // Pyongyang
    {  498, 1544,   12032,  650471,  650500, 10, 29, 24, }, // Qyzylorda
    {  498, 1554,       0,  170193,       0,  0,  7,  0, }, // Reunion
    {  498, 1562,       0,   12103,       0,  0,  6,  0, }, // Rothera
    {  498, 1570,   12109,  650524,  650550,  7, 26, 21, }, // Sakhalin
    {  498, 1579,  650571,  650585,  650612, 14, 27, 22, }, // Samara
    {  498, 1596,   12226,  650634,  650658,  5, 24, 19, }, // Samoa
    {  498, 1602,       0,  650677,       0,  0, 19,  0, }, // Seychelles
    {  498, 1624,       0,   12300,       0,  0,  8,  0, }, // Singapore
    {  498, 1634,       0,  650696,       0,  0, 18,  0, }, // Solomon
    {  498, 1642,       0,  650714,       0,  0, 13,  0, }, // South_Georgia
    {  498, 1656,       0,   12350,       0,  0,  7,  0, }, // Suriname
    {  498, 1676,       0,   90403,       0,  0,  4,  0, }, // Syowa
    {  498, 1682,       0,   12362,       0,  0,  5,  0, }, // Tahiti
    {  498, 1689,  170345,  650727,  650753,  7, 26, 21, }, // Taipei
    {  498, 1696,       0,  125839,       0,  0, 11,  0, }, // Tajikistan
    {  498, 1724,       0,   12429,       0,  0,  7,  0, }, // Tokelau
    {  498, 1732,   12436,  650774,  650798,  5, 24, 19, }, // Tonga
    {  498, 1738,       0,   12485,       0,  0,  4,  0, }, // Truk
    {  498, 1750,   12489,  650817,  650848, 12, 31, 26, }, // Turkmenistan
    {  498, 1763,       0,   12559,       0,  0,  6,  0, }, // Tuvalu
    {  498, 1777,   90621,  650874,  650900,  7, 26, 21, }, // Uruguay
    {  498, 1792,  113264,  650921,  650950, 10, 29, 24, }, // Uzbekistan
    {  498, 1803,   12687,  650974,  651000,  7, 26, 21, }, // Vanuatu
    {  498, 1811,       0,  170626,       0,  0,  9,  0, }, // Venezuela
    {  498, 1821,   12751,  651021,  651051, 11, 30, 25, }, // Vladivostok
    {  498, 1833,   12818,  651076,  651104,  9, 28, 23, }, // Volgograd
    {  498, 1843,       0,  113592,       0,  0,  6,  0, }, // Vostok
    {  498, 1850,       0,   90975,       0,  0,  4,  0, }, // Wake
    {  498, 1855,       0,  651127,       0,  0, 15,  0, }, // Wallis
    {  498, 1862,  126411,  651142,  651167,  6, 25, 20, }, // Yakutsk
    {  498, 1870,   12963,  651187,  651218, 12, 31, 26, }, // Yekaterinburg
    {  498, 1892,       0,  126565,       0,  0,  4,  0, }, // Yukon
    {  506,  232,  651244,  651254,  651270, 10, 16, 18, }, // Arabian Sakha/Cyrillic/Russia
    {  506,  268,  651288,  651299,  651316, 11, 17, 19, }, // Armenia
    {  506,  295,  651335,  651354,  651379, 19, 25, 27, }, // Australia_Central
    {  506,  338,  651406,  651425,  651450, 19, 25, 27, }, // Australia_Eastern
    {  506,  356,  651477,  651497,  651523, 20, 26, 28, }, // Australia_Western
    {  506,  501,  651551,  651561,  651577, 10, 16, 18, }, // China
    {  506,  764,  651595,  651608,  651627, 13, 19, 21, }, // Georgia
    {  506,  788,       0,  202975,       0,  0,  3,  0, }, // GMT
    {  506,  914,       0,  651648,       0,  0, 18,  0, }, // India
    {  506,  997,  651666,  651676,  651692, 10, 16, 18, }, // Iran
    {  506, 1023,  651710,  651723,  651742, 13, 19, 21, }, // Japan
    {  506, 1058,       0,  651763,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  506, 1077,       0,  651783,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  506, 1106,  651804,  651815,  651832, 11, 17, 19, }, // Korea
    {  506, 1119,  651851,  651868,  651891, 17, 23, 25, }, // Krasnoyarsk
    {  506, 1151,       0,  651916,       0,  0, 15,  0, }, // Kyrgystan
    {  506, 1204,  651931,  651944,  651963, 13, 19, 21, }, // Magadan
    {  506, 1296,  651984,  652000,  652022, 16, 22, 24, }, // Mongolia
    {  506, 1305,  652046,  652058,  652076, 12, 18, 20, }, // Moscow
    {  506, 1346,  652096,  652114,  652137, 18, 23, 25, }, // New_Zealand
    {  506, 1406,  652162,  652180,  652204, 18, 24, 26, }, // Novosibirsk
    {  506, 1418,  652230,  652241,  652258, 11, 17, 19, }, // Omsk
    {  506, 1428,  652277,  652291,  652311, 14, 20, 22, }, // Pakistan
    {  506, 1570,  652333,  652346,  652365, 13, 19, 21, }, // Sakhalin
    {  506, 1821,  652386,  652402,  652427, 16, 25, 27, }, // Vladivostok
    {  506, 1833,  652454,  652468,  652488, 14, 20, 22, }, // Volgograd
    {  506, 1862,  652510,  652525,  652546, 15, 21, 23, }, // Yakutsk
    {  506, 1870,  652569,  652586,  652609, 17, 23, 25, }, // Yekaterinburg
    {  510,  142,  652634,  652658,  652688, 24, 30, 29, }, // America_Central Sanskrit/Devanagari/India
    {  510,  158,  652717,  652742,  652773, 25, 31, 30, }, // America_Eastern
    {  510,  174,  652803,  652826,  652855, 23, 29, 28, }, // America_Mountain
    {  510,  191,  652883,  652913,  652949, 30, 36, 35, }, // America_Pacific
    {  510,  286,  652984,  652998,  653018, 14, 20, 19, }, // Atlantic
    {  510,  627,  653037,  653054,  653077, 17, 23, 25, }, // Europe_Central
    {  510,  642,  653102,  653120,  653144, 18, 24, 26, }, // Europe_Eastern
    {  510,  680,  653170,  653192,  653220, 22, 28, 30, }, // Europe_Western
    {  510,  788,       0,  653250,       0,  0, 17,  0, }, // GMT
    {  511,  107,  653267,  653279,  653301, 12, 22, 18, }, // Alaska Santali/Ol Chiki/India
    {  511,  135,  653319,  653330,  653350, 11, 20, 17, }, // Amazon
    {  511,  142,  653367,  653378,  653399, 11, 21, 17, }, // America_Central
    {  511,  158,  653416,  653427,  653448, 11, 21, 18, }, // America_Eastern
    {  511,  174,  653466,  653478,  653500, 12, 22, 18, }, // America_Mountain
    {  511,  191,  653518,  653531,  653554, 13, 23, 19, }, // America_Pacific
    {  511,  240,  653573,  653588,  653612, 15, 24, 21, }, // Argentina
    {  511,  250,  653633,  653653,  653683, 20, 30, 26, }, // Argentina_Western
    {  511,  286,  653709,  653722,  653745, 13, 23, 19, }, // Atlantic
    {  511,  537,  653764,  653774,  653790, 10, 16, 16, }, // Cuba
    {  511,  627,  653806,  653827,  653858, 21, 31, 27, }, // Europe_Central
    {  511,  642,  653885,  653905,  653935, 20, 30, 26, }, // Europe_Eastern
    {  511,  680,  653961,  653982,  654013, 21, 31, 27, }, // Europe_Western
    {  511,  788,       0,  654040,       0,  0, 16,  0, }, // GMT
    {  511,  830,  654056,  654076,  654102, 20, 26, 26, }, // Greenland_Eastern
    {  511,  848,  654128,  654147,  654172, 19, 25, 25, }, // Greenland_Western
    {  511,  883,  654197,  654218,  654245, 21, 27, 27, }, // Hawaii_Aleutian
    {  511, 1281,  654272,  654293,  654320, 21, 27, 27, }, // Mexico_Pacific
    {  511, 1358,  654347,  654366,  654391, 19, 25, 25, }, // Newfoundland
    {  511, 1502,  654416,  654444,  654478, 28, 34, 34, }, // Pierre_Miquelon
    {  511, 1892,       0,  654512,       0,  0, 10,  0, }, // Yukon
    {  513,    0,  654522,  654533,  654554, 11, 21, 18, }, // Acre Sardinian/Latin/Italy
    {  513,    5,       0,  654572,       0,  0, 20,  0, }, // Afghanistan
    {  513,   17,       0,  654592,       0,  0, 25,  0, }, // Africa_Central
    {  513,   32,       0,  654617,       0,  0, 25,  0, }, // Africa_Eastern
    {  513,   65,       0,  654642,       0,  0, 37,  0, }, // Africa_Southern
    {  513,   81,  654679,  654706,  654743, 27, 37, 34, }, // Africa_Western
    {  513,  107,  654777,  654792,  654817, 15, 25, 22, }, // Alaska
    {  513,  128,  654839,  654852,  654875, 13, 23, 20, }, // Almaty
    {  513,  135,  654895,  654913,  654941, 18, 28, 25, }, // Amazon
    {  513,  142,  654966,  654983,  655010, 17, 27, 24, }, // America_Central
    {  513,  158,  380906,  655034,  380949, 17, 27, 24, }, // America_Eastern
    {  513,  174,  655061,  655084,  655117, 23, 33, 30, }, // America_Mountain
    {  513,  191,  655147,  655170,  655203, 23, 33, 30, }, // America_Pacific
    {  513,  207,  655233,  655246,  655269, 13, 23, 20, }, // Anadyr
    {  513,  214,  655289,  655300,  655321, 11, 21, 18, }, // Apia
    {  513,  219,  655339,  655351,  655373, 12, 22, 19, }, // Aqtau
    {  513,  225,  655392,  655405,  655428, 13, 23, 20, }, // Aqtobe
    {  513,  232,  655448,  655457,  655476,  9, 19, 16, }, // Arabian
    {  513,  240,  655492,  655510,  655538, 18, 28, 25, }, // Argentina
    {  513,  250,  655563,  655593,  655633, 30, 40, 37, }, // Argentina_Western
    {  513,  268,  655670,  655686,  655712, 16, 26, 23, }, // Armenia
    {  513,  286,  655735,  655753,  655781, 18, 28, 25, }, // Atlantic
    {  513,  295,  655806,  655834,  655872, 28, 38, 35, }, // Australia_Central
    {  513,  313,  655907,  655945,  655993, 38, 48, 45, }, // Australia_CentralWestern
    {  513,  338,  656038,  656066,  656104, 28, 38, 35, }, // Australia_Eastern
    {  513,  356,  656139,  656169,  656209, 30, 40, 37, }, // Australia_Western
    {  513,  374,  656246,  656266,  656296, 20, 30, 27, }, // Azerbaijan
    {  513,  385,  656323,  656341,  656369, 18, 28, 25, }, // Azores
    {  513,  397,  656394,  656414,  656444, 20, 30, 27, }, // Bangladesh
    {  513,  415,       0,  656471,       0,  0, 16,  0, }, // Bhutan
    {  513,  422,       0,  656487,       0,  0, 17,  0, }, // Bolivia
    {  513,  437,  656504,  656519,  656544, 15, 25, 22, }, // Brasilia
    {  513,  454,       0,  656566,       0,  0, 16,  0, }, // Brunei
    {  513,  461,  656582,  656602,  656632, 20, 30, 27, }, // Cape_Verde
    {  513,  472,       0,  656659,       0,  0, 12,  0, }, // Casey
    {  513,  478,       0,  656671,       0,  0, 25,  0, }, // Chamorro
    {  513,  487,  656696,  656714,  656742, 18, 28, 25, }, // Chatham
    {  513,  495,  656767,  656782,  656807, 15, 25, 22, }, // Chile
    {  513,  501,  656829,  656844,  656869, 15, 25, 22, }, // China
    {  513,  507,       0,  656891,       0,  0, 31,  0, }, // Christmas
    {  513,  517,       0,  656922,       0,  0, 23,  0, }, // Cocos
    {  513,  523,  656945,  656963,  656991, 18, 28, 25, }, // Colombia
    {  513,  532,  657016,  657038,  657070, 22, 32, 29, }, // Cook
    {  513,  537,  657099,  657110,  657131, 11, 21, 18, }, // Cuba
    {  513,  548,       0,  657149,       0,  0, 12,  0, }, // Davis
    {  513,  564,       0,  657161,       0,  0, 23,  0, }, // DumontDUrville
    {  513,  601,       0,  657184,       0,  0, 19,  0, }, // East_Timor
    {  513,  612,  657203,  657226,  657259, 23, 33, 30, }, // Easter
    {  513,  619,       0,  657289,       0,  0, 16,  0, }, // Ecuador
    {  513,  627,  657305,  657330,  657365, 25, 35, 32, }, // Europe_Central
    {  513,  642,  657397,  657422,  657457, 25, 35, 32, }, // Europe_Eastern
    {  513,  657,       0,  657489,       0,  0, 46,  0, }, // Europe_Further_Eastern
    {  513,  680,  657535,  657562,  657599, 27, 37, 34, }, // Europe_Western
    {  513,  695,  657633,  657659,  657695, 26, 36, 33, }, // Falkland
    {  513,  704,  657728,  657743,  657768, 15, 25, 22, }, // Fiji
    {  513,  709,       0,  657790,       0,  0, 26,  0, }, // French_Guiana
    {  513,  723,       0,  657816,       0,  0, 50,  0, }, // French_Southern
    {  513,  746,       0,  657866,       0,  0, 20,  0, }, // Galapagos
    {  513,  756,       0,  657886,       0,  0, 14,  0, }, // Gambier
    {  513,  764,  657900,  657917,  657944, 17, 27, 24, }, // Georgia
    {  513,  772,       0,  657968,       0,  0, 25,  0, }, // Gilbert_Islands
    {  513,  788,       0,  657993,       0,  0, 32,  0, }, // GMT
    {  513,  830,  658025,  658056,  658097, 31, 41, 38, }, // Greenland_Eastern
    {  513,  848,  658135,  658168,  658211, 33, 43, 40, }, // Greenland_Western
    {  513,  866,       0,  658251,       0,  0, 21,  0, }, // Guam
    {  513,  871,       0,  658272,       0,  0, 25,  0, }, // Gulf
    {  513,  876,       0,  658297,       0,  0, 16,  0, }, // Guyana
    {  513,  883,  658313,  658347,  658391, 34, 44, 41, }, // Hawaii_Aleutian
    {  513,  899,  658432,  658448,  658474, 16, 26, 23, }, // Hong_Kong
    {  513,  909,  658497,  658508,  658529, 11, 21, 18, }, // Hovd
    {  513,  914,       0,  658547,       0,  0, 24,  0, }, // India
    {  513,  920,       0,  658571,       0,  0, 24,  0, }, // Indian_Ocean
    {  513,  933,       0,  658595,       0,  0, 18,  0, }, // Indochina
    {  513,  943,       0,  658613,       0,  0, 28,  0, }, // Indonesia_Central
    {  513,  961,       0,  658641,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  513,  979,       0,  658669,       0,  0, 30,  0, }, // Indonesia_Western
    {  513,  997,  658699,  658712,  658735, 13, 23, 20, }, // Iran
    {  513, 1008,  658755,  658769,  658793, 14, 24, 21, }, // Irkutsk
    {  513, 1016,  658814,  658828,  658852, 14, 24, 21, }, // Israel
    {  513, 1023,  658873,  658890,  658917, 17, 27, 24, }, // Japan
    {  513, 1029,  658941,  658972,  659013, 31, 41, 38, }, // Kamchatka
    {  513, 1047,       0,  659051,       0,  0, 20,  0, }, // Kazakhstan
    {  513, 1058,       0,  659071,       0,  0, 30,  0, }, // Kazakhstan_Eastern
    {  513, 1077,       0,  659101,       0,  0, 32,  0, }, // Kazakhstan_Western
    {  513, 1106,  384739,  659133,  384770, 11, 21, 18, }, // Korea
    {  513, 1112,       0,  659154,       0,  0, 13,  0, }, // Kosrae
    {  513, 1119,  659167,  659185,  659213, 18, 28, 25, }, // Krasnoyarsk
    {  513, 1151,       0,  659238,       0,  0, 22,  0, }, // Kyrgystan
    {  513, 1161,       0,  659260,       0,  0, 12,  0, }, // Lanka
    {  513, 1175,       0,  659272,       0,  0, 29,  0, }, // Line_Islands
    {  513, 1188,  659301,  659317,  659343, 16, 26, 23, }, // Lord_Howe
    {  513, 1198,  659366,  659378,  659400, 12, 22, 19, }, // Macau
    {  513, 1204,  659419,  659433,  659457, 14, 24, 21, }, // Magadan
    {  513, 1219,       0,  659478,       0,  0, 17,  0, }, // Malaysia
    {  513, 1228,       0,  659495,       0,  0, 19,  0, }, // Maldives
    {  513, 1237,       0,  659514,       0,  0, 20,  0, }, // Marquesas
    {  513, 1247,       0,  659534,       0,  0, 26,  0, }, // Marshall_Islands
    {  513, 1264,  659560,  659581,  659612, 21, 31, 28, }, // Mauritius
    {  513, 1274,       0,  659640,       0,  0, 13,  0, }, // Mawson
    {  513, 1281,  659653,  659682,  659721, 29, 39, 36, }, // Mexico_Pacific
    {  513, 1296,  659757,  659774,  659801, 17, 27, 24, }, // Mongolia
    {  513, 1305,  659825,  659837,  659859, 12, 22, 19, }, // Moscow
    {  513, 1312,       0,  659878,       0,  0, 17,  0, }, // Myanmar
    {  513, 1320,       0,  659895,       0,  0, 12,  0, }, // Nauru
    {  513, 1326,       0,  659907,       0,  0, 15,  0, }, // Nepal
    {  513, 1332,  659922,  659945,  659978, 23, 33, 30, }, // New_Caledonia
    {  513, 1346,  660008,  660029,  660060, 21, 31, 28, }, // New_Zealand
    {  513, 1358,  660088,  660104,  660130, 16, 26, 23, }, // Newfoundland
    {  513, 1371,       0,  660153,       0,  0, 11,  0, }, // Niue
    {  513, 1376,  660164,  660186,  660218, 22, 32, 29, }, // Norfolk
    {  513, 1384,  660247,  660276,  660315, 29, 39, 36, }, // Noronha
    {  513, 1392,       0,  660351,       0,  0, 42,  0, }, // North_Mariana
    {  513, 1406,  660393,  660411,  660439, 18, 28, 25, }, // Novosibirsk
    {  513, 1418,  660464,  660475,  660496, 11, 21, 18, }, // Omsk
    {  513, 1428,  660514,  660532,  660560, 18, 28, 25, }, // Pakistan
    {  513, 1437,       0,  660585,       0,  0, 12,  0, }, // Palau
    {  513, 1443,       0,  660597,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  513, 1460,  660623,  660641,  660669, 18, 28, 25, }, // Paraguay
    {  513, 1469,  660694,  660708,  660732, 14, 24, 21, }, // Peru
    {  513, 1474,  660753,  660773,  660803, 20, 30, 27, }, // Philippines
    {  513, 1486,       0,  660830,       0,  0, 32,  0, }, // Phoenix_Islands
    {  513, 1502,  660862,  660892,  660932, 30, 40, 37, }, // Pierre_Miquelon
    {  513, 1518,       0,  660969,       0,  0, 19,  0, }, // Pitcairn
    {  513, 1527,       0,  660988,       0,  0, 14,  0, }, // Ponape
    {  513, 1534,       0,  661002,       0,  0, 16,  0, }, // Pyongyang
    {  513, 1544,  661018,  661034,  661060, 16, 26, 23, }, // Qyzylorda
    {  513, 1554,       0,  661083,       0,  0, 18,  0, }, // Reunion
    {  513, 1562,       0,  661101,       0,  0, 14,  0, }, // Rothera
    {  513, 1570,  661115,  661130,  661155, 15, 25, 22, }, // Sakhalin
    {  513, 1579,  661177,  661190,  661213, 13, 23, 20, }, // Samara
    {  513, 1596,  661233,  661249,  661275, 16, 26, 23, }, // Samoa
    {  513, 1602,       0,  661298,       0,  0, 21,  0, }, // Seychelles
    {  513, 1624,       0,  661319,       0,  0, 16,  0, }, // Singapore
    {  513, 1634,       0,  661335,       0,  0, 26,  0, }, // Solomon
    {  513, 1642,       0,  661361,       0,  0, 27,  0, }, // South_Georgia
    {  513, 1656,       0,  661388,       0,  0, 18,  0, }, // Suriname
    {  513, 1676,       0,  661406,       0,  0, 12,  0, }, // Syowa
    {  513, 1682,       0,  661418,       0,  0, 13,  0, }, // Tahiti
    {  513, 1689,  661431,  661444,  661467, 13, 23, 20, }, // Taipei
    {  513, 1696,       0,  661487,       0,  0, 20,  0, }, // Tajikistan
    {  513, 1724,       0,  661507,       0,  0, 17,  0, }, // Tokelau
    {  513, 1732,  661524,  661539,  661564, 15, 25, 22, }, // Tonga
    {  513, 1738,       0,  661586,       0,  0, 15,  0, }, // Truk
    {  513, 1750,  661601,  661623,  661655, 22, 32, 29, }, // Turkmenistan
    {  513, 1763,       0,  661684,       0,  0, 16,  0, }, // Tuvalu
    {  513, 1777,  661700,  661716,  661742, 16, 26, 23, }, // Uruguay
    {  513, 1792,  661765,  661784,  661813, 19, 29, 26, }, // Uzbekistan
    {  513, 1803,  661839,  661856,  661883, 17, 27, 24, }, // Vanuatu
    {  513, 1811,       0,  661907,       0,  0, 20,  0, }, // Venezuela
    {  513, 1821,  661927,  661945,  661973, 18, 28, 25, }, // Vladivostok
    {  513, 1833,  661998,  662014,  662040, 16, 26, 23, }, // Volgograd
    {  513, 1843,       0,  662063,       0,  0, 13,  0, }, // Vostok
    {  513, 1850,       0,  662076,       0,  0, 22,  0, }, // Wake
    {  513, 1855,       0,  662098,       0,  0, 22,  0, }, // Wallis
    {  513, 1862,  662120,  662134,  662158, 14, 24, 21, }, // Yakutsk
    {  513, 1870,  662179,  662199,  662229, 20, 30, 27, }, // Yekaterinburg
    {  513, 1892,       0,  662256,       0,  0, 15,  0, }, // Yukon
    {  515,    0,  105736,  105746,  105767, 10, 21, 27, }, // Acre Serbian/Cyrillic/Serbia
    {  515,    5,       0,  662271,       0,  0, 16,  0, }, // Afghanistan
    {  515,   17,       0,  662287,       0,  0, 23,  0, }, // Africa_Central
    {  515,   32,       0,  662310,       0,  0, 21,  0, }, // Africa_Eastern
    {  515,   65,       0,  662331,       0,  0, 19,  0, }, // Africa_Southern
    {  515,   81,  662350,  662371,  662403, 21, 32, 27, }, // Africa_Western
    {  515,  107,  105976,  662430,  662454,  6, 24, 19, }, // Alaska
    {  515,  128,  106035,  106047,  106070, 12, 23, 29, }, // Almaty
    {  515,  135,  662473,  662485,  662509, 12, 24, 19, }, // Amazon
    {  515,  142,  662528,  662559,  662601, 31, 42, 37, }, // America_Central
    {  515,  158,  662638,  662667,  662707, 29, 40, 35, }, // America_Eastern
    {  515,  174,  662742,  662773,  662815, 31, 42, 37, }, // America_Mountain
    {  515,  191,  662852,  662883,  662925, 31, 42, 37, }, // America_Pacific
    {  515,  207,  106469,  106481,  106504, 12, 23, 29, }, // Anadyr
    {  515,  214,  662962,  662973,  662996, 11, 23, 18, }, // Apia
    {  515,  219,  106598,  106611,  106635, 13, 24, 30, }, // Aqtau
    {  515,  225,  106665,  106678,  106702, 13, 24, 30, }, // Aqtobe
    {  515,  232,  663014,  663029,  663055, 15, 26, 21, }, // Arabian
    {  515,  240,  663076,  663091,  663118, 15, 27, 22, }, // Argentina
    {  515,  250,  663140,  663163,  663198, 23, 35, 30, }, // Argentina_Western
    {  515,  268,  663228,  663243,  663270, 15, 27, 22, }, // Armenia
    {  515,  286,  495835,  495850,  663292, 15, 26, 21, }, // Atlantic
    {  515,  295,  663313,  663341,  663380, 28, 39, 34, }, // Australia_Central
    {  515,  313,  663414,  663450,  663497, 36, 47, 42, }, // Australia_CentralWestern
    {  515,  338,  663539,  663565,  663602, 26, 37, 32, }, // Australia_Eastern
    {  515,  356,  663634,  663660,  663697, 26, 37, 32, }, // Australia_Western
    {  515,  374,  663729,  663745,  663773, 16, 28, 23, }, // Azerbaijan
    {  515,  385,  663796,  663807,  663830, 11, 23, 18, }, // Azores
    {  515,  397,  663848,  663863,  663890, 15, 27, 22, }, // Bangladesh
    {  515,  415,       0,  663912,       0,  0, 11,  0, }, // Bhutan
    {  515,  422,       0,  663923,       0,  0, 14,  0, }, // Bolivia
    {  515,  437,  663937,  663952,  663979, 15, 27, 22, }, // Brasilia
    {  515,  454,       0,  664001,       0,  0, 22,  0, }, // Brunei
    {  515,  461,  664023,  664047,  664083, 24, 36, 31, }, // Cape_Verde
    {  515,  478,       0,  664114,       0,  0, 12,  0, }, // Chamorro
    {  515,  487,  664126,  664137,  664160, 11, 23, 18, }, // Chatham
    {  515,  495,  664178,  664188,  664210, 10, 22, 17, }, // Chile
    {  515,  501,  664227,  664237,  664261, 10, 24, 17, }, // China
    {  515,  507,       0,  664278,       0,  0, 20,  0, }, // Christmas
    {  515,  517,       0,  664298,       0,  0, 27,  0, }, // Cocos
    {  515,  523,  664325,  664340,  664367, 15, 27, 22, }, // Colombia
    {  515,  532,  664389,  664408,  664439, 19, 31, 31, }, // Cook
    {  515,  537,    8827,  664470,  664492,  4, 22, 17, }, // Cuba
    {  515,  548,       0,  664509,       0,  0, 12,  0, }, // Davis
    {  515,  564,       0,  664521,       0,  0, 19,  0, }, // DumontDUrville
    {  515,  601,       0,  664540,       0,  0, 19,  0, }, // East_Timor
    {  515,  612,  664559,  664579,  664611, 20, 32, 27, }, // Easter
    {  515,  619,       0,  664638,       0,  0, 13,  0, }, // Ecuador
    {  515,  627,  664651,  664671,  664702, 20, 31, 26, }, // Europe_Central
    {  515,  642,  497618,  497639,  664728, 21, 32, 27, }, // Europe_Eastern
    {  515,  657,       0,  664755,       0,  0, 25,  0, }, // Europe_Further_Eastern
    {  515,  680,  497718,  497739,  664780, 21, 32, 27, }, // Europe_Western
    {  515,  695,  664807,  664831,  664867, 24, 36, 31, }, // Falkland
    {  515,  704,  664898,  664908,  664930, 10, 22, 17, }, // Fiji
    {  515,  709,       0,  664947,       0,  0, 23,  0, }, // French_Guiana
    {  515,  723,       0,  497994,       0,  0, 35,  0, }, // French_Southern
    {  515,  746,       0,  664970,       0,  0, 15,  0, }, // Galapagos
    {  515,  756,       0,  664985,       0,  0, 13,  0, }, // Gambier
    {  515,  764,  664998,  665011,  665036, 13, 25, 20, }, // Georgia
    {  515,  772,       0,  665056,       0,  0, 20,  0, }, // Gilbert_Islands
    {  515,  788,       0,  665076,       0,  0, 23,  0, }, // GMT
    {  515,  830,  109381,  665099,  665133, 16, 34, 29, }, // Greenland_Eastern
    {  515,  848,  109479,  665162,  665196, 16, 34, 29, }, // Greenland_Western
    {  515,  866,       0,  109577,       0,  0, 21,  0, }, // Guam
    {  515,  871,       0,  665225,       0,  0, 14,  0, }, // Gulf
    {  515,  876,       0,  664957,       0,  0, 13,  0, }, // Guyana
    {  515,  883,  665239,  665262,  665296, 23, 34, 29, }, // Hawaii_Aleutian
    {  515,  899,  665325,  665340,  665367, 15, 27, 22, }, // Hong_Kong
    {  515,  909,  665389,  665399,  665421, 10, 22, 17, }, // Hovd
    {  515,  914,       0,  665438,       0,  0, 25,  0, }, // India
    {  515,  920,       0,  665463,       0,  0, 23,  0, }, // Indian_Ocean
    {  515,  933,       0,  665486,       0,  0, 14,  0, }, // Indochina
    {  515,  943,       0,  665500,       0,  0, 28,  0, }, // Indonesia_Central
    {  515,  961,       0,  665528,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  515,  979,       0,  665554,       0,  0, 26,  0, }, // Indonesia_Western
    {  515,  997,  665580,  665590,  665612, 10, 22, 17, }, // Iran
    {  515, 1008,  665629,  665641,  665665, 12, 24, 19, }, // Irkutsk
    {  515, 1016,  123456,  665684,  665710, 15, 26, 21, }, // Israel
    {  515, 1023,  665731,  665745,  665770, 14, 25, 20, }, // Japan
    {  515, 1029,  110290,  110320,  665790, 30, 41, 47, }, // Kamchatka
    {  515, 1047,       0,  123701,       0,  0, 18,  0, }, // Kazakhstan
    {  515, 1058,       0,  665837,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  515, 1077,       0,  665863,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  515, 1106,  665889,  665903,  665928, 14, 25, 20, }, // Korea
    {  515, 1112,       0,  665948,       0,  0, 11,  0, }, // Kosrae
    {  515, 1119,  665959,  665976,  666005, 17, 29, 24, }, // Krasnoyarsk
    {  515, 1151,       0,  666029,       0,  0, 15,  0, }, // Kyrgystan
    {  515, 1161,       0,  110651,       0,  0, 15,  0, }, // Lanka
    {  515, 1175,       0,  666044,       0,  0, 17,  0, }, // Line_Islands
    {  515, 1188,  666061,  666075,  666101, 14, 26, 21, }, // Lord_Howe
    {  515, 1198,  110759,  110770,  666122, 11, 22, 28, }, // Macau
    {  515, 1204,  666150,  666163,  666188, 13, 25, 20, }, // Magadan
    {  515, 1219,       0,  666208,       0,  0, 14,  0, }, // Malaysia
    {  515, 1228,       0,  666222,       0,  0, 13,  0, }, // Maldives
    {  515, 1237,       0,  666235,       0,  0, 12,  0, }, // Marquesas
    {  515, 1247,       0,  666247,       0,  0, 22,  0, }, // Marshall_Islands
    {  515, 1264,  666269,  666285,  666313, 16, 28, 23, }, // Mauritius
    {  515, 1274,       0,  666336,       0,  0, 11,  0, }, // Mawson
    {  515, 1281,  666347,  666363,  666397, 16, 34, 29, }, // Mexico_Pacific
    {  515, 1296,  666426,  666442,  666470, 16, 28, 23, }, // Mongolia
    {  515, 1305,  666493,  666505,  666529, 12, 24, 19, }, // Moscow
    {  515, 1312,       0,  666548,       0,  0, 14,  0, }, // Myanmar
    {  515, 1320,       0,  666562,       0,  0, 11,  0, }, // Nauru
    {  515, 1326,       0,  666573,       0,  0, 11,  0, }, // Nepal
    {  515, 1332,  666584,  666605,  666638, 21, 33, 28, }, // New_Caledonia
    {  515, 1346,  666666,  666683,  666712, 17, 29, 24, }, // New_Zealand
    {  515, 1358,  111523,  666736,  666765, 11, 29, 24, }, // Newfoundland
    {  515, 1371,       0,  666789,       0,  0, 10,  0, }, // Niue
    {  515, 1376,  666799,  666819,  666851, 20, 32, 27, }, // Norfolk
    {  515, 1384,  666878,  666902,  666938, 24, 36, 31, }, // Noronha
    {  515, 1392,       0,  111814,       0,  0, 31,  0, }, // North_Mariana
    {  515, 1406,  666969,  666986,  667015, 17, 29, 24, }, // Novosibirsk
    {  515, 1418,  667039,  667049,  667071, 10, 22, 17, }, // Omsk
    {  515, 1428,  667088,  667102,  667128, 14, 26, 21, }, // Pakistan
    {  515, 1437,       0,  667149,       0,  0, 11,  0, }, // Palau
    {  515, 1443,       0,  667160,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  515, 1460,  667184,  667198,  667224, 14, 26, 21, }, // Paraguay
    {  515, 1469,  667245,  667255,  667277, 10, 22, 17, }, // Peru
    {  515, 1474,  667294,  667308,  667334, 14, 26, 21, }, // Philippines
    {  515, 1486,       0,  667355,       0,  0, 19,  0, }, // Phoenix_Islands
    {  515, 1502,  112335,  667374,  667410, 18, 36, 31, }, // Pierre_Miquelon
    {  515, 1518,       0,  667441,       0,  0, 13,  0, }, // Pitcairn
    {  515, 1527,       0,  667454,       0,  0, 12,  0, }, // Ponape
    {  515, 1534,       0,  667466,       0,  0, 17,  0, }, // Pyongyang
    {  515, 1544,  112484,  112499,  112525, 15, 26, 32, }, // Qyzylorda
    {  515, 1554,       0,  667483,       0,  0, 13,  0, }, // Reunion
    {  515, 1562,       0,  667496,       0,  0, 12,  0, }, // Rothera
    {  515, 1570,  667508,  667521,  667546, 13, 25, 20, }, // Sakhalin
    {  515, 1579,  112657,  112669,  112692, 12, 23, 29, }, // Samara
    {  515, 1596,  667566,  667577,  667600, 11, 23, 18, }, // Samoa
    {  515, 1602,       0,  667618,       0,  0, 13,  0, }, // Seychelles
    {  515, 1624,       0,  667631,       0,  0, 26,  0, }, // Singapore
    {  515, 1634,       0,  667657,       0,  0, 23,  0, }, // Solomon
    {  515, 1642,       0,  667680,       0,  0, 19,  0, }, // South_Georgia
    {  515, 1656,       0,  667699,       0,  0, 13,  0, }, // Suriname
    {  515, 1676,       0,  667712,       0,  0, 10,  0, }, // Syowa
    {  515, 1682,       0,  667722,       0,  0, 12,  0, }, // Tahiti
    {  515, 1689,  667734,  667746,  667770, 12, 24, 19, }, // Taipei
    {  515, 1696,       0,  667789,       0,  0, 16,  0, }, // Tajikistan
    {  515, 1724,       0,  667805,       0,  0, 13,  0, }, // Tokelau
    {  515, 1732,  667818,  667829,  667852, 11, 23, 18, }, // Tonga
    {  515, 1738,       0,  667870,       0,  0, 10,  0, }, // Truk
    {  515, 1750,  667880,  667898,  667928, 18, 30, 25, }, // Turkmenistan
    {  515, 1763,       0,  667953,       0,  0, 12,  0, }, // Tuvalu
    {  515, 1777,  667965,  667978,  668003, 13, 25, 20, }, // Uruguay
    {  515, 1792,  668023,  668039,  668067, 16, 28, 23, }, // Uzbekistan
    {  515, 1803,  668090,  668103,  668128, 13, 25, 20, }, // Vanuatu
    {  515, 1811,       0,  668148,       0,  0, 15,  0, }, // Venezuela
    {  515, 1821,  668163,  668180,  668209, 17, 29, 24, }, // Vladivostok
    {  515, 1833,  668233,  668248,  668275, 15, 27, 22, }, // Volgograd
    {  515, 1843,       0,  668297,       0,  0, 12,  0, }, // Vostok
    {  515, 1850,       0,  668309,       0,  0, 17,  0, }, // Wake
    {  515, 1855,       0,  668326,       0,  0, 27,  0, }, // Wallis
    {  515, 1862,  668353,  668366,  668391, 13, 25, 20, }, // Yakutsk
    {  515, 1870,  668411,  668430,  668461, 19, 31, 26, }, // Yekaterinburg
    {  515, 1892,       0,  502338,       0,  0,  5,  0, }, // Yukon
    {  516,    5,       0,  105794,       0,  0, 18,  0, }, // Afghanistan Serbian/Cyrillic/Bosnia and Herzegovina
    {  516,   17,       0,  105812,       0,  0, 25,  0, }, // Africa_Central
    {  516,   32,       0,  105837,       0,  0, 23,  0, }, // Africa_Eastern
    {  516,   65,       0,  105860,       0,  0, 21,  0, }, // Africa_Southern
    {  516,   81,  105881,  105904,  668487, 23, 34, 29, }, // Africa_Western
    {  516,  107,  105976,  668516,  668542,  6, 26, 21, }, // Alaska
    {  516,  135,  106099,  668563,  668589, 14, 26, 21, }, // Amazon
    {  516,  142,  668610,  668644,  668689, 34, 45, 40, }, // America_Central
    {  516,  158,  668729,  668761,  668804, 32, 43, 38, }, // America_Eastern
    {  516,  174,  668842,  668876,  668921, 34, 45, 40, }, // America_Mountain
    {  516,  191,  668961,  668995,  669040, 34, 45, 40, }, // America_Pacific
    {  516,  214,  106533,  669080,  669105, 13, 25, 20, }, // Apia
    {  516,  232,  106732,  106749,  669125, 17, 28, 23, }, // Arabian
    {  516,  240,  106809,  669148,  669177, 17, 29, 24, }, // Argentina
    {  516,  250,  106886,  669201,  669238, 25, 37, 32, }, // Argentina_Western
    {  516,  268,  669270,  669287,  669316, 17, 29, 24, }, // Armenia
    {  516,  286,  669340,  669357,  669385, 17, 28, 23, }, // Atlantic
    {  516,  295,  107135,  107165,  669408, 30, 41, 36, }, // Australia_Central
    {  516,  313,  107251,  107289,  669444, 38, 49, 44, }, // Australia_CentralWestern
    {  516,  338,  107391,  107419,  669488, 28, 39, 34, }, // Australia_Eastern
    {  516,  356,  107501,  107529,  669522, 28, 39, 34, }, // Australia_Western
    {  516,  374,  107611,  669556,  669586, 18, 30, 25, }, // Azerbaijan
    {  516,  385,  107691,  669611,  669636, 13, 25, 20, }, // Azores
    {  516,  397,  107756,  669656,  669685, 17, 29, 24, }, // Bangladesh
    {  516,  415,       0,  107833,       0,  0, 13,  0, }, // Bhutan
    {  516,  422,       0,  107846,       0,  0, 16,  0, }, // Bolivia
    {  516,  437,  107862,  669709,  669738, 17, 29, 24, }, // Brasilia
    {  516,  454,       0,  669762,       0,  0, 24,  0, }, // Brunei
    {  516,  461,  669786,  669812,  669850, 26, 38, 33, }, // Cape_Verde
    {  516,  478,       0,  108046,       0,  0, 14,  0, }, // Chamorro
    {  516,  487,  108060,  669883,  669908, 13, 25, 20, }, // Chatham
    {  516,  495,  108125,  669928,  669952, 12, 24, 19, }, // Chile
    {  516,  501,  108187,  669971,  669997, 12, 26, 19, }, // China
    {  516,  507,       0,  670016,       0,  0, 22,  0, }, // Christmas
    {  516,  517,       0,  670038,       0,  0, 32,  0, }, // Cocos
    {  516,  523,  108300,  670070,  670099, 17, 29, 24, }, // Colombia
    {  516,  532,  670123,  670144,  670177, 21, 33, 32, }, // Cook
    {  516,  537,    8827,  670209,  670233,  4, 24, 19, }, // Cuba
    {  516,  548,       0,  108533,       0,  0, 14,  0, }, // Davis
    {  516,  564,       0,  670252,       0,  0, 21,  0, }, // DumontDUrville
    {  516,  601,       0,  670273,       0,  0, 21,  0, }, // East_Timor
    {  516,  612,  108587,  670294,  670328, 22, 34, 29, }, // Easter
    {  516,  619,       0,  108679,       0,  0, 15,  0, }, // Ecuador
    {  516,  627,  670357,  670379,  670412, 22, 33, 28, }, // Europe_Central
    {  516,  642,  108786,  108809,  670440, 23, 34, 29, }, // Europe_Eastern
    {  516,  680,  108909,  108932,  670469, 23, 34, 29, }, // Europe_Western
    {  516,  695,  109004,  670498,  670536, 26, 38, 33, }, // Falkland
    {  516,  704,  109108,  670569,  670593, 12, 24, 19, }, // Fiji
    {  516,  709,       0,  109170,       0,  0, 25,  0, }, // French_Guiana
    {  516,  723,       0,  109195,       0,  0, 37,  0, }, // French_Southern
    {  516,  746,       0,  109232,       0,  0, 17,  0, }, // Galapagos
    {  516,  756,       0,  670612,       0,  0, 15,  0, }, // Gambier
    {  516,  764,  109265,  670627,  670654, 15, 27, 22, }, // Georgia
    {  516,  772,       0,  670676,       0,  0, 25,  0, }, // Gilbert_Islands
    {  516,  788,       0,  670701,       0,  0, 25,  0, }, // GMT
    {  516,  830,  109381,  670726,  670762, 16, 36, 31, }, // Greenland_Eastern
    {  516,  848,  109479,  670793,  670829, 16, 36, 31, }, // Greenland_Western
    {  516,  871,       0,  109598,       0,  0, 16,  0, }, // Gulf
    {  516,  876,       0,  109180,       0,  0, 15,  0, }, // Guyana
    {  516,  883,  109614,  109639,  670860, 25, 36, 31, }, // Hawaii_Aleutian
    {  516,  899,  109715,  670891,  670920, 17, 29, 24, }, // Hong_Kong
    {  516,  909,  109793,  670944,  670968, 12, 24, 19, }, // Hovd
    {  516,  914,       0,  109855,       0,  0, 27,  0, }, // India
    {  516,  920,       0,  109882,       0,  0, 25,  0, }, // Indian_Ocean
    {  516,  933,       0,  109907,       0,  0, 16,  0, }, // Indochina
    {  516,  943,       0,  109923,       0,  0, 30,  0, }, // Indonesia_Central
    {  516,  961,       0,  109953,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  516,  979,       0,  109981,       0,  0, 28,  0, }, // Indonesia_Western
    {  516,  997,  110009,  670987,  671011, 12, 24, 19, }, // Iran
    {  516, 1008,  110071,  671030,  671056, 14, 26, 21, }, // Irkutsk
    {  516, 1016,  110139,  110156,  671077, 17, 28, 23, }, // Israel
    {  516, 1023,  110216,  110232,  671100, 16, 27, 22, }, // Japan
    {  516, 1047,       0,  671122,       0,  0, 20,  0, }, // Kazakhstan
    {  516, 1058,       0,  110407,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  516, 1077,       0,  110435,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  516, 1106,  110463,  110479,  671142, 16, 27, 22, }, // Korea
    {  516, 1112,       0,  110537,       0,  0, 13,  0, }, // Kosrae
    {  516, 1119,  671164,  671183,  671214, 19, 31, 26, }, // Krasnoyarsk
    {  516, 1151,       0,  110634,       0,  0, 17,  0, }, // Kyrgystan
    {  516, 1175,       0,  671240,       0,  0, 23,  0, }, // Line_Islands
    {  516, 1188,  110685,  671263,  671291, 16, 28, 23, }, // Lord_Howe
    {  516, 1204,  110819,  671314,  671341, 15, 27, 22, }, // Magadan
    {  516, 1219,       0,  110889,       0,  0, 16,  0, }, // Malaysia
    {  516, 1228,       0,  110905,       0,  0, 15,  0, }, // Maldives
    {  516, 1237,       0,  110920,       0,  0, 14,  0, }, // Marquesas
    {  516, 1247,       0,  110934,       0,  0, 24,  0, }, // Marshall_Islands
    {  516, 1264,  110958,  671363,  671393, 18, 30, 25, }, // Mauritius
    {  516, 1274,       0,  111038,       0,  0, 13,  0, }, // Mawson
    {  516, 1281,  666347,  671418,  671454, 16, 36, 31, }, // Mexico_Pacific
    {  516, 1296,  111155,  671485,  671515, 18, 30, 26, }, // Mongolia
    {  516, 1305,  111235,  671541,  671567, 14, 26, 21, }, // Moscow
    {  516, 1312,       0,  671588,       0,  0, 15,  0, }, // Myanmar
    {  516, 1320,       0,  111319,       0,  0, 13,  0, }, // Nauru
    {  516, 1326,       0,  111332,       0,  0, 13,  0, }, // Nepal
    {  516, 1332,  111345,  671603,  671638, 23, 35, 30, }, // New_Caledonia
    {  516, 1346,  111440,  671668,  671699, 19, 31, 26, }, // New_Zealand
    {  516, 1358,  111523,  671725,  671756, 11, 31, 26, }, // Newfoundland
    {  516, 1371,       0,  671782,       0,  0, 13,  0, }, // Niue
    {  516, 1376,  671795,  671817,  671851, 22, 34, 29, }, // Norfolk
    {  516, 1384,  111710,  671880,  671918, 26, 38, 33, }, // Noronha
    {  516, 1406,  671951,  671970,  672001, 19, 31, 26, }, // Novosibirsk
    {  516, 1418,  111929,  672027,  672051, 12, 24, 19, }, // Omsk
    {  516, 1428,  111991,  672070,  672098, 16, 28, 23, }, // Pakistan
    {  516, 1437,       0,  112065,       0,  0, 13,  0, }, // Palau
    {  516, 1443,       0,  112078,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  516, 1460,  112104,  672121,  672149, 16, 28, 23, }, // Paraguay
    {  516, 1469,  112178,  672172,  672196, 12, 24, 19, }, // Peru
    {  516, 1474,  112240,  672215,  672243, 16, 28, 23, }, // Philippines
    {  516, 1486,       0,  112314,       0,  0, 21,  0, }, // Phoenix_Islands
    {  516, 1502,  112335,  672266,  672304, 18, 38, 33, }, // Pierre_Miquelon
    {  516, 1518,       0,  112438,       0,  0, 15,  0, }, // Pitcairn
    {  516, 1527,       0,  672337,       0,  0, 14,  0, }, // Ponape
    {  516, 1534,       0,  672351,       0,  0, 19,  0, }, // Pyongyang
    {  516, 1554,       0,  672370,       0,  0, 15,  0, }, // Reunion
    {  516, 1562,       0,  112572,       0,  0, 14,  0, }, // Rothera
    {  516, 1570,  112586,  672385,  672412, 15, 27, 22, }, // Sakhalin
    {  516, 1596,  112721,  672434,  672459, 13, 25, 20, }, // Samoa
    {  516, 1602,       0,  112786,       0,  0, 15,  0, }, // Seychelles
    {  516, 1624,       0,  672479,       0,  0, 28,  0, }, // Singapore
    {  516, 1634,       0,  112828,       0,  0, 25,  0, }, // Solomon
    {  516, 1642,       0,  112853,       0,  0, 21,  0, }, // South_Georgia
    {  516, 1656,       0,  112874,       0,  0, 15,  0, }, // Suriname
    {  516, 1676,       0,  112889,       0,  0, 12,  0, }, // Syowa
    {  516, 1682,       0,  112901,       0,  0, 14,  0, }, // Tahiti
    {  516, 1689,  112915,  672507,  672533, 14, 26, 21, }, // Taipei
    {  516, 1696,       0,  112983,       0,  0, 18,  0, }, // Tajikistan
    {  516, 1724,       0,  113001,       0,  0, 15,  0, }, // Tokelau
    {  516, 1732,  113016,  672554,  672579, 13, 25, 20, }, // Tonga
    {  516, 1738,       0,  672599,       0,  0, 11,  0, }, // Truk
    {  516, 1750,  113093,  672610,  672642, 20, 32, 27, }, // Turkmenistan
    {  516, 1763,       0,  113179,       0,  0, 14,  0, }, // Tuvalu
    {  516, 1777,  113193,  672669,  672696, 15, 27, 22, }, // Uruguay
    {  516, 1792,  113264,  672718,  672748, 18, 30, 25, }, // Uzbekistan
    {  516, 1803,  113344,  672773,  672800, 15, 27, 22, }, // Vanuatu
    {  516, 1811,       0,  113415,       0,  0, 17,  0, }, // Venezuela
    {  516, 1821,  113432,  672822,  672853, 19, 31, 26, }, // Vladivostok
    {  516, 1833,  113515,  672879,  672908, 17, 29, 24, }, // Volgograd
    {  516, 1843,       0,  113592,       0,  0, 14,  0, }, // Vostok
    {  516, 1850,       0,  672932,       0,  0, 19,  0, }, // Wake
    {  516, 1855,       0,  672951,       0,  0, 29,  0, }, // Wallis
    {  516, 1862,  113654,  672980,  673007, 15, 27, 22, }, // Yakutsk
    {  516, 1870,  113725,  673029,  673062, 21, 33, 28, }, // Yekaterinburg
    {  519,    0,  673090,  673100,  673121, 10, 21, 29, }, // Acre Serbian/Latin/Bosnia and Herzegovina
    {  519,    5,       0,  673150,       0,  0, 18,  0, }, // Afghanistan
    {  519,   17,       0,  673168,       0,  0, 25,  0, }, // Africa_Central
    {  519,   32,       0,  673193,       0,  0, 23,  0, }, // Africa_Eastern
    {  519,   65,       0,  673216,       0,  0, 21,  0, }, // Africa_Southern
    {  519,   81,  673237,  673260,  673294, 23, 34, 31, }, // Africa_Western
    {  519,  107,   97414,  673325,  673352,  7, 27, 24, }, // Alaska
    {  519,  128,  673376,  673388,  673411, 12, 23, 31, }, // Almaty
    {  519,  135,  673442,  673456,  673482, 14, 26, 23, }, // Amazon
    {  519,  142,   97624,   97658,  673505, 34, 45, 42, }, // America_Central
    {  519,  158,   97744,   97776,  673547, 32, 43, 40, }, // America_Eastern
    {  519,  174,   97858,   97892,  673587, 34, 45, 42, }, // America_Mountain
    {  519,  191,   97978,   98012,  673629, 34, 45, 41, }, // America_Pacific
    {  519,  207,   98098,   98110,   98133, 12, 23, 31, }, // Anadyr
    {  519,  214,  673670,  673683,  673708, 13, 25, 22, }, // Apia
    {  519,  219,  673730,  673743,  673767, 13, 24, 32, }, // Aqtau
    {  519,  225,  673799,  673812,  673836, 13, 24, 32, }, // Aqtobe
    {  519,  232,   98362,   98379,  673868, 17, 28, 25, }, // Arabian
    {  519,  240,  673893,  673910,  673939, 17, 29, 26, }, // Argentina
    {  519,  250,  673965,  673990,  674027, 25, 37, 34, }, // Argentina_Western
    {  519,  268,  674061,  674078,  674107, 17, 29, 26, }, // Armenia
    {  519,  286,  674133,  674150,  674178, 17, 28, 25, }, // Atlantic
    {  519,  295,  674203,  674233,  674274, 30, 41, 38, }, // Australia_Central
    {  519,  313,   98893,  674312,  674361, 38, 49, 46, }, // Australia_CentralWestern
    {  519,  338,  674407,  674435,  674474, 28, 39, 36, }, // Australia_Eastern
    {  519,  356,  674510,  674538,  674577, 28, 39, 36, }, // Australia_Western
    {  519,  374,  674613,  674632,  674663, 19, 31, 28, }, // Azerbaijan
    {  519,  385,  674691,  674704,  674729, 13, 25, 22, }, // Azores
    {  519,  397,  674751,  674768,  674797, 17, 29, 26, }, // Bangladesh
    {  519,  415,       0,  674823,       0,  0, 13,  0, }, // Bhutan
    {  519,  422,       0,  674836,       0,  0, 16,  0, }, // Bolivia
    {  519,  437,  674852,  674869,  674898, 17, 29, 26, }, // Brasilia
    {  519,  454,       0,  674924,       0,  0, 24,  0, }, // Brunei
    {  519,  461,  674948,  674974,  675012, 26, 38, 35, }, // Cape_Verde
    {  519,  478,       0,  675047,       0,  0, 14,  0, }, // Chamorro
    {  519,  487,  675061,  675074,  675099, 13, 25, 22, }, // Chatham
    {  519,  495,  675121,  675133,  675157, 12, 24, 21, }, // Chile
    {  519,  501,  675178,   99821,  675190, 12, 26, 21, }, // China
    {  519,  507,       0,  675211,       0,  0, 22,  0, }, // Christmas
    {  519,  517,       0,  675233,       0,  0, 32,  0, }, // Cocos
    {  519,  523,  675265,  675282,  675311, 17, 29, 26, }, // Colombia
    {  519,  532,  675337,  675358,  675391, 21, 33, 34, }, // Cook
    {  519,  537,   14969,  675425,  675449,  4, 24, 21, }, // Cuba
    {  519,  548,       0,  675470,       0,  0, 14,  0, }, // Davis
    {  519,  564,       0,  675484,       0,  0, 21,  0, }, // DumontDUrville
    {  519,  601,       0,  675505,       0,  0, 21,  0, }, // East_Timor
    {  519,  612,  675526,  675549,  675584, 23, 35, 32, }, // Easter
    {  519,  619,       0,  675616,       0,  0, 15,  0, }, // Ecuador
    {  519,  627,  675631,  675654,  675688, 23, 34, 31, }, // Europe_Central
    {  519,  642,  100441,  100464,  675719, 23, 34, 31, }, // Europe_Eastern
    {  519,  657,       0,  675750,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  519,  680,  100557,  100580,  675776, 23, 34, 31, }, // Europe_Western
    {  519,  695,  675807,  675833,  675871, 26, 38, 35, }, // Falkland
    {  519,  704,  675906,  675919,  675944, 13, 25, 22, }, // Fiji
    {  519,  709,       0,  675966,       0,  0, 25,  0, }, // French_Guiana
    {  519,  723,       0,  675991,       0,  0, 37,  0, }, // French_Southern
    {  519,  746,       0,  676028,       0,  0, 17,  0, }, // Galapagos
    {  519,  756,       0,  676045,       0,  0, 15,  0, }, // Gambier
    {  519,  764,  676060,  676075,  676102, 15, 27, 24, }, // Georgia
    {  519,  772,       0,  676126,       0,  0, 25,  0, }, // Gilbert_Islands
    {  519,  788,       0,  676151,       0,  0, 26,  0, }, // GMT
    {  519,  830,  676177,  676193,  676229, 16, 36, 33, }, // Greenland_Eastern
    {  519,  848,  676262,  676278,  676314, 16, 36, 33, }, // Greenland_Western
    {  519,  866,       0,  101214,       0,  0, 21,  0, }, // Guam
    {  519,  871,       0,  676347,       0,  0, 16,  0, }, // Gulf
    {  519,  876,       0,  675976,       0,  0, 15,  0, }, // Guyana
    {  519,  883,  676363,  676388,  676424, 25, 36, 33, }, // Hawaii_Aleutian
    {  519,  899,  676457,  676474,  676503, 17, 29, 26, }, // Hong_Kong
    {  519,  909,  676529,  676541,  676565, 12, 24, 21, }, // Hovd
    {  519,  914,       0,  101513,       0,  0, 27,  0, }, // India
    {  519,  920,       0,  676586,       0,  0, 25,  0, }, // Indian_Ocean
    {  519,  933,       0,  676611,       0,  0, 16,  0, }, // Indochina
    {  519,  943,       0,  676627,       0,  0, 30,  0, }, // Indonesia_Central
    {  519,  961,       0,  676657,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  519,  979,       0,  676685,       0,  0, 28,  0, }, // Indonesia_Western
    {  519,  997,  676713,  676725,  676749, 12, 24, 21, }, // Iran
    {  519, 1008,  676770,  676784,  676810, 14, 26, 23, }, // Irkutsk
    {  519, 1016,  101798,  101815,  676833, 17, 28, 25, }, // Israel
    {  519, 1023,  101867,  101883,  676858, 16, 27, 24, }, // Japan
    {  519, 1029,  676882,  676912,  676953, 30, 41, 49, }, // Kamchatka
    {  519, 1047,       0,  677002,       0,  0, 20,  0, }, // Kazakhstan
    {  519, 1058,       0,  677022,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  519, 1077,       0,  677050,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  519, 1106,  102124,  102140,  677078, 16, 27, 24, }, // Korea
    {  519, 1112,       0,  677102,       0,  0, 13,  0, }, // Kosrae
    {  519, 1119,  677115,  677134,  677165, 19, 31, 28, }, // Krasnoyarsk
    {  519, 1151,       0,  677193,       0,  0, 17,  0, }, // Kyrgystan
    {  519, 1161,       0,  677210,       0,  0, 15,  0, }, // Lanka
    {  519, 1175,       0,  677225,       0,  0, 23,  0, }, // Line_Islands
    {  519, 1188,  677248,  677264,  677292, 16, 28, 25, }, // Lord_Howe
    {  519, 1198,  102443,  102454,  102476, 11, 22, 30, }, // Macau
    {  519, 1204,  677317,  677332,  677359, 15, 27, 24, }, // Magadan
    {  519, 1219,       0,  677383,       0,  0, 16,  0, }, // Malaysia
    {  519, 1228,       0,  677399,       0,  0, 15,  0, }, // Maldives
    {  519, 1237,       0,  677414,       0,  0, 14,  0, }, // Marquesas
    {  519, 1247,       0,  677428,       0,  0, 24,  0, }, // Marshall_Islands
    {  519, 1264,  677452,  677470,  677500, 18, 30, 27, }, // Mauritius
    {  519, 1274,       0,  677527,       0,  0, 13,  0, }, // Mawson
    {  519, 1281,  677540,  677556,  677592, 16, 36, 33, }, // Mexico_Pacific
    {  519, 1296,  677625,  677643,  677673, 18, 30, 28, }, // Mongolia
    {  519, 1305,  677701,  677715,  677741, 14, 26, 23, }, // Moscow
    {  519, 1312,       0,  677764,       0,  0, 15,  0, }, // Myanmar
    {  519, 1320,       0,  677779,       0,  0, 13,  0, }, // Nauru
    {  519, 1326,       0,  677792,       0,  0, 13,  0, }, // Nepal
    {  519, 1332,  677805,  677828,  677863, 23, 35, 32, }, // New_Caledonia
    {  519, 1346,  677895,  677914,  677945, 19, 31, 28, }, // New_Zealand
    {  519, 1358,   29478,  677973,  678005, 12, 32, 29, }, // Newfoundland
    {  519, 1371,       0,  678034,       0,  0, 13,  0, }, // Niue
    {  519, 1376,  678047,  678069,  678103, 22, 34, 31, }, // Norfolk
    {  519, 1384,  678134,  678161,  678200, 27, 39, 36, }, // Noronha
    {  519, 1392,       0,  103545,       0,  0, 31,  0, }, // North_Mariana
    {  519, 1406,  678236,  678255,  678286, 19, 31, 28, }, // Novosibirsk
    {  519, 1418,  678314,  678326,  678350, 12, 24, 21, }, // Omsk
    {  519, 1428,  678371,  678387,  678415, 16, 28, 25, }, // Pakistan
    {  519, 1437,       0,  678440,       0,  0, 13,  0, }, // Palau
    {  519, 1443,       0,  678453,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  519, 1460,  678479,  678495,  678523, 16, 28, 25, }, // Paraguay
    {  519, 1469,  678548,  678560,  678584, 12, 24, 21, }, // Peru
    {  519, 1474,  678605,  678621,  678649, 16, 28, 25, }, // Philippines
    {  519, 1486,       0,  678674,       0,  0, 21,  0, }, // Phoenix_Islands
    {  519, 1502,  678695,  678713,  678751, 18, 38, 35, }, // Pierre_Miquelon
    {  519, 1518,       0,  678786,       0,  0, 15,  0, }, // Pitcairn
    {  519, 1527,       0,  678801,       0,  0, 14,  0, }, // Ponape
    {  519, 1534,       0,  104273,       0,  0, 19,  0, }, // Pyongyang
    {  519, 1544,  104292,  104307,  104333, 15, 26, 34, }, // Qyzylorda
    {  519, 1554,       0,  678815,       0,  0, 15,  0, }, // Reunion
    {  519, 1562,       0,  678830,       0,  0, 14,  0, }, // Rothera
    {  519, 1570,  678844,  678859,  678886, 15, 27, 24, }, // Sakhalin
    {  519, 1579,  104480,  104492,  104515, 12, 23, 31, }, // Samara
    {  519, 1596,  678910,  678923,  678948, 13, 25, 22, }, // Samoa
    {  519, 1602,       0,  678970,       0,  0, 15,  0, }, // Seychelles
    {  519, 1624,       0,  678985,       0,  0, 28,  0, }, // Singapore
    {  519, 1634,       0,  679013,       0,  0, 25,  0, }, // Solomon
    {  519, 1642,       0,  679038,       0,  0, 23,  0, }, // South_Georgia
    {  519, 1656,       0,  679061,       0,  0, 15,  0, }, // Suriname
    {  519, 1676,       0,  679076,       0,  0, 12,  0, }, // Syowa
    {  519, 1682,       0,  679088,       0,  0, 14,  0, }, // Tahiti
    {  519, 1689,  679102,  679116,  679142, 14, 26, 23, }, // Taipei
    {  519, 1696,       0,  679165,       0,  0, 19,  0, }, // Tajikistan
    {  519, 1724,       0,  679184,       0,  0, 15,  0, }, // Tokelau
    {  519, 1732,  679199,  679212,  679237, 13, 25, 22, }, // Tonga
    {  519, 1738,       0,  679259,       0,  0, 11,  0, }, // Truk
    {  519, 1750,  679270,  679290,  679322, 20, 32, 29, }, // Turkmenistan
    {  519, 1763,       0,  679351,       0,  0, 14,  0, }, // Tuvalu
    {  519, 1777,  679365,  679380,  679407, 15, 27, 24, }, // Uruguay
    {  519, 1792,  679431,  679449,  679479, 18, 30, 27, }, // Uzbekistan
    {  519, 1803,  679506,  679521,  679548, 15, 27, 24, }, // Vanuatu
    {  519, 1811,       0,  679572,       0,  0, 17,  0, }, // Venezuela
    {  519, 1821,  679589,  679608,  679639, 19, 31, 28, }, // Vladivostok
    {  519, 1833,  679667,  679684,  679713, 17, 29, 26, }, // Volgograd
    {  519, 1843,       0,  679739,       0,  0, 14,  0, }, // Vostok
    {  519, 1850,       0,  679753,       0,  0, 19,  0, }, // Wake
    {  519, 1855,       0,  679772,       0,  0, 29,  0, }, // Wallis
    {  519, 1862,  679801,  679816,  679843, 15, 27, 24, }, // Yakutsk
    {  519, 1870,  679867,  679888,  679921, 21, 33, 30, }, // Yekaterinburg
    {  519, 1892,       0,   31566,       0,  0,  5,  0, }, // Yukon
    {  520,    0,  673090,  673100,  673121, 10, 21, 29, }, // Acre Serbian/Latin/Kosovo
    {  520,    5,       0,  679951,       0,  0, 16,  0, }, // Afghanistan
    {  520,   17,       0,  679967,       0,  0, 23,  0, }, // Africa_Central
    {  520,   32,       0,  679990,       0,  0, 21,  0, }, // Africa_Eastern
    {  520,   65,       0,  680011,       0,  0, 19,  0, }, // Africa_Southern
    {  520,   81,  680030,  680051,  680083, 21, 32, 28, }, // Africa_Western
    {  520,  107,   97414,  680111,  680136,  7, 25, 21, }, // Alaska
    {  520,  128,  673376,  673388,  673411, 12, 23, 31, }, // Almaty
    {  520,  135,  680157,  680169,  680193, 12, 24, 20, }, // Amazon
    {  520,  142,  680213,  680244,  680286, 31, 42, 38, }, // America_Central
    {  520,  158,  680324,  680353,  680393, 29, 40, 36, }, // America_Eastern
    {  520,  174,  680429,  680460,  680502, 31, 42, 38, }, // America_Mountain
    {  520,  191,  680540,  680571,  680613, 31, 42, 38, }, // America_Pacific
    {  520,  207,   98098,   98110,   98133, 12, 23, 31, }, // Anadyr
    {  520,  214,  680651,  680662,  680685, 11, 23, 19, }, // Apia
    {  520,  219,  673730,  673743,  673767, 13, 24, 32, }, // Aqtau
    {  520,  225,  673799,  673812,  673836, 13, 24, 32, }, // Aqtobe
    {  520,  232,  680704,  680719,  680745, 15, 26, 22, }, // Arabian
    {  520,  240,  680767,  680782,  680809, 15, 27, 23, }, // Argentina
    {  520,  250,  680832,  680855,  680890, 23, 35, 31, }, // Argentina_Western
    {  520,  268,  680921,  680936,  680963, 15, 27, 23, }, // Armenia
    {  520,  286,  680986,  681001,  681027, 15, 26, 22, }, // Atlantic
    {  520,  295,  681049,  681077,  681116, 28, 39, 35, }, // Australia_Central
    {  520,  313,  681151,  681187,  681234, 36, 47, 43, }, // Australia_CentralWestern
    {  520,  338,  681277,  681303,  681340, 26, 37, 33, }, // Australia_Eastern
    {  520,  356,  681373,  681399,  681436, 26, 37, 33, }, // Australia_Western
    {  520,  374,  681469,  681486,  681515, 17, 29, 25, }, // Azerbaijan
    {  520,  385,  681540,  681551,  681574, 11, 23, 19, }, // Azores
    {  520,  397,  681593,  681608,  681635, 15, 27, 23, }, // Bangladesh
    {  520,  415,       0,  681658,       0,  0, 11,  0, }, // Bhutan
    {  520,  422,       0,  681669,       0,  0, 14,  0, }, // Bolivia
    {  520,  437,  681683,  681698,  681725, 15, 27, 23, }, // Brasilia
    {  520,  454,       0,  681748,       0,  0, 22,  0, }, // Brunei
    {  520,  461,  681770,  681794,  681830, 24, 36, 32, }, // Cape_Verde
    {  520,  478,       0,  681862,       0,  0, 12,  0, }, // Chamorro
    {  520,  487,  681874,  681885,  681908, 11, 23, 19, }, // Chatham
    {  520,  495,  681927,  681937,  681959, 10, 22, 18, }, // Chile
    {  520,  501,  681977,  681987,  682011, 10, 24, 18, }, // China
    {  520,  507,       0,  682029,       0,  0, 20,  0, }, // Christmas
    {  520,  517,       0,  682049,       0,  0, 27,  0, }, // Cocos
    {  520,  523,  682076,  682091,  682118, 15, 27, 23, }, // Colombia
    {  520,  532,  682141,  682160,  682191, 19, 31, 32, }, // Cook
    {  520,  537,   14969,  682223,  682245,  4, 22, 18, }, // Cuba
    {  520,  548,       0,  682263,       0,  0, 12,  0, }, // Davis
    {  520,  564,       0,  682275,       0,  0, 19,  0, }, // DumontDUrville
    {  520,  601,       0,  682294,       0,  0, 19,  0, }, // East_Timor
    {  520,  612,  682313,  682334,  682367, 21, 33, 29, }, // Easter
    {  520,  619,       0,  682396,       0,  0, 13,  0, }, // Ecuador
    {  520,  627,  682409,  682430,  682462, 21, 32, 28, }, // Europe_Central
    {  520,  642,  682490,  682511,  682543, 21, 32, 28, }, // Europe_Eastern
    {  520,  657,       0,  675750,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  520,  680,  682571,  682592,  682624, 21, 32, 28, }, // Europe_Western
    {  520,  695,  682652,  682676,  682712, 24, 36, 32, }, // Falkland
    {  520,  704,  682744,  682755,  682778, 11, 23, 19, }, // Fiji
    {  520,  709,       0,  682797,       0,  0, 23,  0, }, // French_Guiana
    {  520,  723,       0,  682820,       0,  0, 35,  0, }, // French_Southern
    {  520,  746,       0,  682855,       0,  0, 15,  0, }, // Galapagos
    {  520,  756,       0,  682870,       0,  0, 13,  0, }, // Gambier
    {  520,  764,  682883,  682896,  682921, 13, 25, 21, }, // Georgia
    {  520,  772,       0,  682942,       0,  0, 20,  0, }, // Gilbert_Islands
    {  520,  788,       0,  682962,       0,  0, 24,  0, }, // GMT
    {  520,  830,  676177,  682986,  683020, 16, 34, 30, }, // Greenland_Eastern
    {  520,  848,  676262,  683050,  683084, 16, 34, 30, }, // Greenland_Western
    {  520,  866,       0,  101214,       0,  0, 21,  0, }, // Guam
    {  520,  871,       0,  683114,       0,  0, 14,  0, }, // Gulf
    {  520,  876,       0,  682807,       0,  0, 13,  0, }, // Guyana
    {  520,  883,  683128,  683151,  683185, 23, 34, 30, }, // Hawaii_Aleutian
    {  520,  899,  683215,  683230,  683257, 15, 27, 23, }, // Hong_Kong
    {  520,  909,  683280,  683290,  683312, 10, 22, 18, }, // Hovd
    {  520,  914,       0,  683330,       0,  0, 25,  0, }, // India
    {  520,  920,       0,  683355,       0,  0, 23,  0, }, // Indian_Ocean
    {  520,  933,       0,  683378,       0,  0, 14,  0, }, // Indochina
    {  520,  943,       0,  683392,       0,  0, 28,  0, }, // Indonesia_Central
    {  520,  961,       0,  683420,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  520,  979,       0,  683446,       0,  0, 26,  0, }, // Indonesia_Western
    {  520,  997,  683472,  683482,  683504, 10, 22, 18, }, // Iran
    {  520, 1008,  683522,  683534,  683558, 12, 24, 20, }, // Irkutsk
    {  520, 1016,  683578,  683593,  683619, 15, 26, 22, }, // Israel
    {  520, 1023,  683641,  683655,  683680, 14, 25, 21, }, // Japan
    {  520, 1029,  676882,  676912,  676953, 30, 41, 49, }, // Kamchatka
    {  520, 1047,       0,  683701,       0,  0, 18,  0, }, // Kazakhstan
    {  520, 1058,       0,  683719,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  520, 1077,       0,  683745,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  520, 1106,  683771,  683785,  683810, 14, 25, 21, }, // Korea
    {  520, 1112,       0,  683831,       0,  0, 11,  0, }, // Kosrae
    {  520, 1119,  683842,  683859,  683888, 17, 29, 25, }, // Krasnoyarsk
    {  520, 1151,       0,  683913,       0,  0, 15,  0, }, // Kyrgystan
    {  520, 1161,       0,  677210,       0,  0, 15,  0, }, // Lanka
    {  520, 1175,       0,  683928,       0,  0, 17,  0, }, // Line_Islands
    {  520, 1188,  683945,  683959,  683985, 14, 26, 22, }, // Lord_Howe
    {  520, 1198,  102443,  102454,  102476, 11, 22, 30, }, // Macau
    {  520, 1204,  684007,  684020,  684045, 13, 25, 21, }, // Magadan
    {  520, 1219,       0,  684066,       0,  0, 14,  0, }, // Malaysia
    {  520, 1228,       0,  684080,       0,  0, 13,  0, }, // Maldives
    {  520, 1237,       0,  684093,       0,  0, 12,  0, }, // Marquesas
    {  520, 1247,       0,  684105,       0,  0, 22,  0, }, // Marshall_Islands
    {  520, 1264,  684127,  684143,  684171, 16, 28, 24, }, // Mauritius
    {  520, 1274,       0,  684195,       0,  0, 11,  0, }, // Mawson
    {  520, 1281,  677540,  684206,  684240, 16, 34, 30, }, // Mexico_Pacific
    {  520, 1296,  684270,  684286,  684314, 16, 28, 24, }, // Mongolia
    {  520, 1305,  684338,  684350,  684374, 12, 24, 20, }, // Moscow
    {  520, 1312,       0,  684394,       0,  0, 14,  0, }, // Myanmar
    {  520, 1320,       0,  684408,       0,  0, 11,  0, }, // Nauru
    {  520, 1326,       0,  684419,       0,  0, 11,  0, }, // Nepal
    {  520, 1332,  684430,  684451,  684484, 21, 33, 29, }, // New_Caledonia
    {  520, 1346,  684513,  684530,  684559, 17, 29, 25, }, // New_Zealand
    {  520, 1358,   29478,  684584,  684614, 12, 30, 26, }, // Newfoundland
    {  520, 1371,       0,  684640,       0,  0, 10,  0, }, // Niue
    {  520, 1376,  684650,  684670,  684702, 20, 32, 28, }, // Norfolk
    {  520, 1384,  684730,  684755,  684792, 25, 37, 33, }, // Noronha
    {  520, 1392,       0,  103545,       0,  0, 31,  0, }, // North_Mariana
    {  520, 1406,  684825,  684842,  684871, 17, 29, 25, }, // Novosibirsk
    {  520, 1418,  684896,  684906,  684928, 10, 22, 18, }, // Omsk
    {  520, 1428,  684946,  684960,  684986, 14, 26, 22, }, // Pakistan
    {  520, 1437,       0,  685008,       0,  0, 11,  0, }, // Palau
    {  520, 1443,       0,  685019,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  520, 1460,  685043,  685057,  685083, 14, 26, 22, }, // Paraguay
    {  520, 1469,  685105,  685115,  685137, 10, 22, 18, }, // Peru
    {  520, 1474,  685155,  685169,  685195, 14, 26, 22, }, // Philippines
    {  520, 1486,       0,  685217,       0,  0, 19,  0, }, // Phoenix_Islands
    {  520, 1502,  678695,  685236,  685272, 18, 36, 32, }, // Pierre_Miquelon
    {  520, 1518,       0,  685304,       0,  0, 13,  0, }, // Pitcairn
    {  520, 1527,       0,  685317,       0,  0, 12,  0, }, // Ponape
    {  520, 1534,       0,  685329,       0,  0, 17,  0, }, // Pyongyang
    {  520, 1544,  104292,  104307,  104333, 15, 26, 34, }, // Qyzylorda
    {  520, 1554,       0,  685346,       0,  0, 13,  0, }, // Reunion
    {  520, 1562,       0,  685359,       0,  0, 12,  0, }, // Rothera
    {  520, 1570,  685371,  685384,  685409, 13, 25, 21, }, // Sakhalin
    {  520, 1579,  104480,  104492,  104515, 12, 23, 31, }, // Samara
    {  520, 1596,  685430,  685441,  685464, 11, 23, 19, }, // Samoa
    {  520, 1602,       0,  685483,       0,  0, 13,  0, }, // Seychelles
    {  520, 1624,       0,  685496,       0,  0, 26,  0, }, // Singapore
    {  520, 1634,       0,  685522,       0,  0, 23,  0, }, // Solomon
    {  520, 1642,       0,  685545,       0,  0, 21,  0, }, // South_Georgia
    {  520, 1656,       0,  685566,       0,  0, 13,  0, }, // Suriname
    {  520, 1676,       0,  685579,       0,  0, 10,  0, }, // Syowa
    {  520, 1682,       0,  685589,       0,  0, 12,  0, }, // Tahiti
    {  520, 1689,  685601,  685613,  685637, 12, 24, 20, }, // Taipei
    {  520, 1696,       0,  685657,       0,  0, 17,  0, }, // Tajikistan
    {  520, 1724,       0,  685674,       0,  0, 13,  0, }, // Tokelau
    {  520, 1732,  685687,  685698,  685721, 11, 23, 19, }, // Tonga
    {  520, 1738,       0,  685740,       0,  0, 10,  0, }, // Truk
    {  520, 1750,  685750,  685768,  685798, 18, 30, 26, }, // Turkmenistan
    {  520, 1763,       0,  685824,       0,  0, 12,  0, }, // Tuvalu
    {  520, 1777,  685836,  685849,  685874, 13, 25, 21, }, // Uruguay
    {  520, 1792,  685895,  685911,  685939, 16, 28, 24, }, // Uzbekistan
    {  520, 1803,  685963,  685976,  686001, 13, 25, 21, }, // Vanuatu
    {  520, 1811,       0,  686022,       0,  0, 15,  0, }, // Venezuela
    {  520, 1821,  686037,  686054,  686083, 17, 29, 25, }, // Vladivostok
    {  520, 1833,  686108,  686123,  686150, 15, 27, 23, }, // Volgograd
    {  520, 1843,       0,  686173,       0,  0, 12,  0, }, // Vostok
    {  520, 1850,       0,  686185,       0,  0, 17,  0, }, // Wake
    {  520, 1855,       0,  686202,       0,  0, 27,  0, }, // Wallis
    {  520, 1862,  686229,  686242,  686267, 13, 25, 21, }, // Yakutsk
    {  520, 1870,  686288,  686307,  686338, 19, 31, 27, }, // Yekaterinburg
    {  520, 1892,       0,   31566,       0,  0,  5,  0, }, // Yukon
    {  521,    0,  673090,  673100,  673121, 10, 21, 29, }, // Acre Serbian/Latin/Montenegro
    {  521,    5,       0,  679951,       0,  0, 16,  0, }, // Afghanistan
    {  521,   17,       0,  679967,       0,  0, 23,  0, }, // Africa_Central
    {  521,   32,       0,  679990,       0,  0, 21,  0, }, // Africa_Eastern
    {  521,   65,       0,  680011,       0,  0, 19,  0, }, // Africa_Southern
    {  521,   81,  680030,  680051,  680083, 21, 32, 28, }, // Africa_Western
    {  521,  107,   97414,  680111,  680136,  7, 25, 21, }, // Alaska
    {  521,  128,  673376,  673388,  673411, 12, 23, 31, }, // Almaty
    {  521,  135,  680157,  680169,  680193, 12, 24, 20, }, // Amazon
    {  521,  142,  680213,  680244,  680286, 31, 42, 38, }, // America_Central
    {  521,  158,  680324,  680353,  680393, 29, 40, 36, }, // America_Eastern
    {  521,  174,  680429,  680460,  680502, 31, 42, 38, }, // America_Mountain
    {  521,  191,  680540,  680571,  680613, 31, 42, 38, }, // America_Pacific
    {  521,  207,   98098,   98110,   98133, 12, 23, 31, }, // Anadyr
    {  521,  214,  680651,  680662,  680685, 11, 23, 19, }, // Apia
    {  521,  219,  673730,  673743,  673767, 13, 24, 32, }, // Aqtau
    {  521,  225,  673799,  673812,  673836, 13, 24, 32, }, // Aqtobe
    {  521,  232,  680704,  680719,  680745, 15, 26, 22, }, // Arabian
    {  521,  240,  680767,  680782,  680809, 15, 27, 23, }, // Argentina
    {  521,  250,  680832,  680855,  680890, 23, 35, 31, }, // Argentina_Western
    {  521,  268,  680921,  680936,  680963, 15, 27, 23, }, // Armenia
    {  521,  286,  680986,  681001,  681027, 15, 26, 22, }, // Atlantic
    {  521,  295,  681049,  681077,  681116, 28, 39, 35, }, // Australia_Central
    {  521,  313,  681151,  681187,  681234, 36, 47, 43, }, // Australia_CentralWestern
    {  521,  338,  681277,  681303,  681340, 26, 37, 33, }, // Australia_Eastern
    {  521,  356,  681373,  681399,  681436, 26, 37, 33, }, // Australia_Western
    {  521,  374,  681469,  681486,  681515, 17, 29, 25, }, // Azerbaijan
    {  521,  385,  681540,  681551,  681574, 11, 23, 19, }, // Azores
    {  521,  397,  681593,  681608,  681635, 15, 27, 23, }, // Bangladesh
    {  521,  415,       0,  681658,       0,  0, 11,  0, }, // Bhutan
    {  521,  422,       0,  681669,       0,  0, 14,  0, }, // Bolivia
    {  521,  437,  681683,  681698,  681725, 15, 27, 23, }, // Brasilia
    {  521,  454,       0,  681748,       0,  0, 22,  0, }, // Brunei
    {  521,  461,  681770,  681794,  681830, 24, 36, 32, }, // Cape_Verde
    {  521,  478,       0,  681862,       0,  0, 12,  0, }, // Chamorro
    {  521,  487,  681874,  681885,  681908, 11, 23, 19, }, // Chatham
    {  521,  495,  681927,  681937,  681959, 10, 22, 18, }, // Chile
    {  521,  501,  681977,  681987,  682011, 10, 24, 18, }, // China
    {  521,  507,       0,  682029,       0,  0, 20,  0, }, // Christmas
    {  521,  517,       0,  682049,       0,  0, 27,  0, }, // Cocos
    {  521,  523,  682076,  682091,  682118, 15, 27, 23, }, // Colombia
    {  521,  532,  682141,  682160,  682191, 19, 31, 32, }, // Cook
    {  521,  537,   14969,  682223,  682245,  4, 22, 18, }, // Cuba
    {  521,  548,       0,  682263,       0,  0, 12,  0, }, // Davis
    {  521,  564,       0,  682275,       0,  0, 19,  0, }, // DumontDUrville
    {  521,  601,       0,  682294,       0,  0, 19,  0, }, // East_Timor
    {  521,  612,  682313,  682334,  682367, 21, 33, 29, }, // Easter
    {  521,  619,       0,  682396,       0,  0, 13,  0, }, // Ecuador
    {  521,  627,  682409,  682430,  682462, 21, 32, 28, }, // Europe_Central
    {  521,  642,  682490,  682511,  682543, 21, 32, 28, }, // Europe_Eastern
    {  521,  657,       0,  675750,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  521,  680,  682571,  682592,  682624, 21, 32, 28, }, // Europe_Western
    {  521,  695,  682652,  682676,  682712, 24, 36, 32, }, // Falkland
    {  521,  704,  682744,  682755,  682778, 11, 23, 19, }, // Fiji
    {  521,  709,       0,  682797,       0,  0, 23,  0, }, // French_Guiana
    {  521,  723,       0,  682820,       0,  0, 35,  0, }, // French_Southern
    {  521,  746,       0,  682855,       0,  0, 15,  0, }, // Galapagos
    {  521,  756,       0,  682870,       0,  0, 13,  0, }, // Gambier
    {  521,  764,  682883,  682896,  682921, 13, 25, 21, }, // Georgia
    {  521,  772,       0,  682942,       0,  0, 20,  0, }, // Gilbert_Islands
    {  521,  788,       0,  682962,       0,  0, 24,  0, }, // GMT
    {  521,  830,  676177,  682986,  683020, 16, 34, 30, }, // Greenland_Eastern
    {  521,  848,  676262,  683050,  683084, 16, 34, 30, }, // Greenland_Western
    {  521,  866,       0,  101214,       0,  0, 21,  0, }, // Guam
    {  521,  871,       0,  683114,       0,  0, 14,  0, }, // Gulf
    {  521,  876,       0,  682807,       0,  0, 13,  0, }, // Guyana
    {  521,  883,  683128,  683151,  683185, 23, 34, 30, }, // Hawaii_Aleutian
    {  521,  899,  683215,  683230,  683257, 15, 27, 23, }, // Hong_Kong
    {  521,  909,  683280,  683290,  683312, 10, 22, 18, }, // Hovd
    {  521,  914,       0,  683330,       0,  0, 25,  0, }, // India
    {  521,  920,       0,  683355,       0,  0, 23,  0, }, // Indian_Ocean
    {  521,  933,       0,  683378,       0,  0, 14,  0, }, // Indochina
    {  521,  943,       0,  683392,       0,  0, 28,  0, }, // Indonesia_Central
    {  521,  961,       0,  683420,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  521,  979,       0,  683446,       0,  0, 26,  0, }, // Indonesia_Western
    {  521,  997,  683472,  683482,  683504, 10, 22, 18, }, // Iran
    {  521, 1008,  683522,  683534,  683558, 12, 24, 20, }, // Irkutsk
    {  521, 1016,  683578,  683593,  683619, 15, 26, 22, }, // Israel
    {  521, 1023,  683641,  683655,  683680, 14, 25, 21, }, // Japan
    {  521, 1029,  676882,  676912,  676953, 30, 41, 49, }, // Kamchatka
    {  521, 1047,       0,  683701,       0,  0, 18,  0, }, // Kazakhstan
    {  521, 1058,       0,  683719,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  521, 1077,       0,  683745,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  521, 1106,  683771,  683785,  683810, 14, 25, 21, }, // Korea
    {  521, 1112,       0,  683831,       0,  0, 11,  0, }, // Kosrae
    {  521, 1119,  683842,  683859,  683888, 17, 29, 25, }, // Krasnoyarsk
    {  521, 1151,       0,  683913,       0,  0, 15,  0, }, // Kyrgystan
    {  521, 1161,       0,  677210,       0,  0, 15,  0, }, // Lanka
    {  521, 1175,       0,  683928,       0,  0, 17,  0, }, // Line_Islands
    {  521, 1188,  683945,  683959,  683985, 14, 26, 22, }, // Lord_Howe
    {  521, 1198,  102443,  102454,  102476, 11, 22, 30, }, // Macau
    {  521, 1204,  684007,  684020,  684045, 13, 25, 21, }, // Magadan
    {  521, 1219,       0,  684066,       0,  0, 14,  0, }, // Malaysia
    {  521, 1228,       0,  684080,       0,  0, 13,  0, }, // Maldives
    {  521, 1237,       0,  684093,       0,  0, 12,  0, }, // Marquesas
    {  521, 1247,       0,  684105,       0,  0, 22,  0, }, // Marshall_Islands
    {  521, 1264,  684127,  684143,  684171, 16, 28, 24, }, // Mauritius
    {  521, 1274,       0,  684195,       0,  0, 11,  0, }, // Mawson
    {  521, 1281,  677540,  684206,  684240, 16, 34, 30, }, // Mexico_Pacific
    {  521, 1296,  684270,  684286,  684314, 16, 28, 24, }, // Mongolia
    {  521, 1305,  684338,  684350,  684374, 12, 24, 20, }, // Moscow
    {  521, 1312,       0,  684394,       0,  0, 14,  0, }, // Myanmar
    {  521, 1320,       0,  684408,       0,  0, 11,  0, }, // Nauru
    {  521, 1326,       0,  684419,       0,  0, 11,  0, }, // Nepal
    {  521, 1332,  684430,  684451,  684484, 21, 33, 29, }, // New_Caledonia
    {  521, 1346,  684513,  684530,  684559, 17, 29, 25, }, // New_Zealand
    {  521, 1358,   29478,  684584,  684614, 12, 30, 26, }, // Newfoundland
    {  521, 1371,       0,  684640,       0,  0, 10,  0, }, // Niue
    {  521, 1376,  684650,  684670,  684702, 20, 32, 28, }, // Norfolk
    {  521, 1384,  684730,  684755,  684792, 25, 37, 33, }, // Noronha
    {  521, 1392,       0,  103545,       0,  0, 31,  0, }, // North_Mariana
    {  521, 1406,  684825,  684842,  684871, 17, 29, 25, }, // Novosibirsk
    {  521, 1418,  684896,  684906,  684928, 10, 22, 18, }, // Omsk
    {  521, 1428,  684946,  684960,  684986, 14, 26, 22, }, // Pakistan
    {  521, 1437,       0,  685008,       0,  0, 11,  0, }, // Palau
    {  521, 1443,       0,  685019,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  521, 1460,  685043,  685057,  685083, 14, 26, 22, }, // Paraguay
    {  521, 1469,  685105,  685115,  685137, 10, 22, 18, }, // Peru
    {  521, 1474,  685155,  685169,  685195, 14, 26, 22, }, // Philippines
    {  521, 1486,       0,  685217,       0,  0, 19,  0, }, // Phoenix_Islands
    {  521, 1502,  678695,  685236,  685272, 18, 36, 32, }, // Pierre_Miquelon
    {  521, 1518,       0,  685304,       0,  0, 13,  0, }, // Pitcairn
    {  521, 1527,       0,  685317,       0,  0, 12,  0, }, // Ponape
    {  521, 1534,       0,  685329,       0,  0, 17,  0, }, // Pyongyang
    {  521, 1544,  104292,  104307,  104333, 15, 26, 34, }, // Qyzylorda
    {  521, 1554,       0,  685346,       0,  0, 13,  0, }, // Reunion
    {  521, 1562,       0,  685359,       0,  0, 12,  0, }, // Rothera
    {  521, 1570,  685371,  685384,  685409, 13, 25, 21, }, // Sakhalin
    {  521, 1579,  104480,  104492,  104515, 12, 23, 31, }, // Samara
    {  521, 1596,  685430,  685441,  685464, 11, 23, 19, }, // Samoa
    {  521, 1602,       0,  685483,       0,  0, 13,  0, }, // Seychelles
    {  521, 1624,       0,  685496,       0,  0, 26,  0, }, // Singapore
    {  521, 1634,       0,  685522,       0,  0, 23,  0, }, // Solomon
    {  521, 1642,       0,  685545,       0,  0, 21,  0, }, // South_Georgia
    {  521, 1656,       0,  685566,       0,  0, 13,  0, }, // Suriname
    {  521, 1676,       0,  685579,       0,  0, 10,  0, }, // Syowa
    {  521, 1682,       0,  685589,       0,  0, 12,  0, }, // Tahiti
    {  521, 1689,  685601,  685613,  685637, 12, 24, 20, }, // Taipei
    {  521, 1696,       0,  685657,       0,  0, 17,  0, }, // Tajikistan
    {  521, 1724,       0,  685674,       0,  0, 13,  0, }, // Tokelau
    {  521, 1732,  685687,  685698,  685721, 11, 23, 19, }, // Tonga
    {  521, 1738,       0,  685740,       0,  0, 10,  0, }, // Truk
    {  521, 1750,  685750,  685768,  685798, 18, 30, 26, }, // Turkmenistan
    {  521, 1763,       0,  685824,       0,  0, 12,  0, }, // Tuvalu
    {  521, 1777,  685836,  685849,  685874, 13, 25, 21, }, // Uruguay
    {  521, 1792,  685895,  685911,  685939, 16, 28, 24, }, // Uzbekistan
    {  521, 1803,  685963,  685976,  686001, 13, 25, 21, }, // Vanuatu
    {  521, 1811,       0,  686022,       0,  0, 15,  0, }, // Venezuela
    {  521, 1821,  686037,  686054,  686083, 17, 29, 25, }, // Vladivostok
    {  521, 1833,  686108,  686123,  686150, 15, 27, 23, }, // Volgograd
    {  521, 1843,       0,  686173,       0,  0, 12,  0, }, // Vostok
    {  521, 1850,       0,  686185,       0,  0, 17,  0, }, // Wake
    {  521, 1855,       0,  686202,       0,  0, 27,  0, }, // Wallis
    {  521, 1862,  686229,  686242,  686267, 13, 25, 21, }, // Yakutsk
    {  521, 1870,  686288,  686307,  686338, 19, 31, 27, }, // Yekaterinburg
    {  521, 1892,       0,   31566,       0,  0,  5,  0, }, // Yukon
    {  522,    0,  673090,  673100,  673121, 10, 21, 29, }, // Acre Serbian/Latin/Serbia
    {  522,    5,       0,  679951,       0,  0, 16,  0, }, // Afghanistan
    {  522,   17,       0,  679967,       0,  0, 23,  0, }, // Africa_Central
    {  522,   32,       0,  679990,       0,  0, 21,  0, }, // Africa_Eastern
    {  522,   65,       0,  680011,       0,  0, 19,  0, }, // Africa_Southern
    {  522,   81,  680030,  680051,  680083, 21, 32, 28, }, // Africa_Western
    {  522,  107,   97414,  680111,  680136,  7, 25, 21, }, // Alaska
    {  522,  128,  673376,  673388,  673411, 12, 23, 31, }, // Almaty
    {  522,  135,  680157,  680169,  680193, 12, 24, 20, }, // Amazon
    {  522,  142,  680213,  680244,  680286, 31, 42, 38, }, // America_Central
    {  522,  158,  680324,  680353,  680393, 29, 40, 36, }, // America_Eastern
    {  522,  174,  680429,  680460,  680502, 31, 42, 38, }, // America_Mountain
    {  522,  191,  680540,  680571,  680613, 31, 42, 38, }, // America_Pacific
    {  522,  207,   98098,   98110,   98133, 12, 23, 31, }, // Anadyr
    {  522,  214,  680651,  680662,  680685, 11, 23, 19, }, // Apia
    {  522,  219,  673730,  673743,  673767, 13, 24, 32, }, // Aqtau
    {  522,  225,  673799,  673812,  673836, 13, 24, 32, }, // Aqtobe
    {  522,  232,  680704,  680719,  680745, 15, 26, 22, }, // Arabian
    {  522,  240,  680767,  680782,  680809, 15, 27, 23, }, // Argentina
    {  522,  250,  680832,  680855,  680890, 23, 35, 31, }, // Argentina_Western
    {  522,  268,  680921,  680936,  680963, 15, 27, 23, }, // Armenia
    {  522,  286,  680986,  681001,  681027, 15, 26, 22, }, // Atlantic
    {  522,  295,  681049,  681077,  681116, 28, 39, 35, }, // Australia_Central
    {  522,  313,  681151,  681187,  681234, 36, 47, 43, }, // Australia_CentralWestern
    {  522,  338,  681277,  681303,  681340, 26, 37, 33, }, // Australia_Eastern
    {  522,  356,  681373,  681399,  681436, 26, 37, 33, }, // Australia_Western
    {  522,  374,  681469,  681486,  681515, 17, 29, 25, }, // Azerbaijan
    {  522,  385,  681540,  681551,  681574, 11, 23, 19, }, // Azores
    {  522,  397,  681593,  681608,  681635, 15, 27, 23, }, // Bangladesh
    {  522,  415,       0,  681658,       0,  0, 11,  0, }, // Bhutan
    {  522,  422,       0,  681669,       0,  0, 14,  0, }, // Bolivia
    {  522,  437,  681683,  681698,  681725, 15, 27, 23, }, // Brasilia
    {  522,  454,       0,  681748,       0,  0, 22,  0, }, // Brunei
    {  522,  461,  681770,  681794,  681830, 24, 36, 32, }, // Cape_Verde
    {  522,  478,       0,  681862,       0,  0, 12,  0, }, // Chamorro
    {  522,  487,  681874,  681885,  681908, 11, 23, 19, }, // Chatham
    {  522,  495,  681927,  681937,  681959, 10, 22, 18, }, // Chile
    {  522,  501,  681977,  681987,  682011, 10, 24, 18, }, // China
    {  522,  507,       0,  682029,       0,  0, 20,  0, }, // Christmas
    {  522,  517,       0,  682049,       0,  0, 27,  0, }, // Cocos
    {  522,  523,  682076,  682091,  682118, 15, 27, 23, }, // Colombia
    {  522,  532,  682141,  682160,  682191, 19, 31, 32, }, // Cook
    {  522,  537,   14969,  682223,  682245,  4, 22, 18, }, // Cuba
    {  522,  548,       0,  682263,       0,  0, 12,  0, }, // Davis
    {  522,  564,       0,  682275,       0,  0, 19,  0, }, // DumontDUrville
    {  522,  601,       0,  682294,       0,  0, 19,  0, }, // East_Timor
    {  522,  612,  682313,  682334,  682367, 21, 33, 29, }, // Easter
    {  522,  619,       0,  682396,       0,  0, 13,  0, }, // Ecuador
    {  522,  627,  682409,  682430,  682462, 21, 32, 28, }, // Europe_Central
    {  522,  642,  682490,  682511,  682543, 21, 32, 28, }, // Europe_Eastern
    {  522,  657,       0,  675750,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  522,  680,  682571,  682592,  682624, 21, 32, 28, }, // Europe_Western
    {  522,  695,  682652,  682676,  682712, 24, 36, 32, }, // Falkland
    {  522,  704,  682744,  682755,  682778, 11, 23, 19, }, // Fiji
    {  522,  709,       0,  682797,       0,  0, 23,  0, }, // French_Guiana
    {  522,  723,       0,  682820,       0,  0, 35,  0, }, // French_Southern
    {  522,  746,       0,  682855,       0,  0, 15,  0, }, // Galapagos
    {  522,  756,       0,  682870,       0,  0, 13,  0, }, // Gambier
    {  522,  764,  682883,  682896,  682921, 13, 25, 21, }, // Georgia
    {  522,  772,       0,  682942,       0,  0, 20,  0, }, // Gilbert_Islands
    {  522,  788,       0,  682962,       0,  0, 24,  0, }, // GMT
    {  522,  830,  676177,  682986,  683020, 16, 34, 30, }, // Greenland_Eastern
    {  522,  848,  676262,  683050,  683084, 16, 34, 30, }, // Greenland_Western
    {  522,  866,       0,  101214,       0,  0, 21,  0, }, // Guam
    {  522,  871,       0,  683114,       0,  0, 14,  0, }, // Gulf
    {  522,  876,       0,  682807,       0,  0, 13,  0, }, // Guyana
    {  522,  883,  683128,  683151,  683185, 23, 34, 30, }, // Hawaii_Aleutian
    {  522,  899,  683215,  683230,  683257, 15, 27, 23, }, // Hong_Kong
    {  522,  909,  683280,  683290,  683312, 10, 22, 18, }, // Hovd
    {  522,  914,       0,  683330,       0,  0, 25,  0, }, // India
    {  522,  920,       0,  683355,       0,  0, 23,  0, }, // Indian_Ocean
    {  522,  933,       0,  683378,       0,  0, 14,  0, }, // Indochina
    {  522,  943,       0,  683392,       0,  0, 28,  0, }, // Indonesia_Central
    {  522,  961,       0,  683420,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  522,  979,       0,  683446,       0,  0, 26,  0, }, // Indonesia_Western
    {  522,  997,  683472,  683482,  683504, 10, 22, 18, }, // Iran
    {  522, 1008,  683522,  683534,  683558, 12, 24, 20, }, // Irkutsk
    {  522, 1016,  683578,  683593,  683619, 15, 26, 22, }, // Israel
    {  522, 1023,  683641,  683655,  683680, 14, 25, 21, }, // Japan
    {  522, 1029,  676882,  676912,  676953, 30, 41, 49, }, // Kamchatka
    {  522, 1047,       0,  683701,       0,  0, 18,  0, }, // Kazakhstan
    {  522, 1058,       0,  683719,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  522, 1077,       0,  683745,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  522, 1106,  683771,  683785,  683810, 14, 25, 21, }, // Korea
    {  522, 1112,       0,  683831,       0,  0, 11,  0, }, // Kosrae
    {  522, 1119,  683842,  683859,  683888, 17, 29, 25, }, // Krasnoyarsk
    {  522, 1151,       0,  683913,       0,  0, 15,  0, }, // Kyrgystan
    {  522, 1161,       0,  677210,       0,  0, 15,  0, }, // Lanka
    {  522, 1175,       0,  683928,       0,  0, 17,  0, }, // Line_Islands
    {  522, 1188,  683945,  683959,  683985, 14, 26, 22, }, // Lord_Howe
    {  522, 1198,  102443,  102454,  102476, 11, 22, 30, }, // Macau
    {  522, 1204,  684007,  684020,  684045, 13, 25, 21, }, // Magadan
    {  522, 1219,       0,  684066,       0,  0, 14,  0, }, // Malaysia
    {  522, 1228,       0,  684080,       0,  0, 13,  0, }, // Maldives
    {  522, 1237,       0,  684093,       0,  0, 12,  0, }, // Marquesas
    {  522, 1247,       0,  684105,       0,  0, 22,  0, }, // Marshall_Islands
    {  522, 1264,  684127,  684143,  684171, 16, 28, 24, }, // Mauritius
    {  522, 1274,       0,  684195,       0,  0, 11,  0, }, // Mawson
    {  522, 1281,  677540,  684206,  684240, 16, 34, 30, }, // Mexico_Pacific
    {  522, 1296,  684270,  684286,  684314, 16, 28, 24, }, // Mongolia
    {  522, 1305,  684338,  684350,  684374, 12, 24, 20, }, // Moscow
    {  522, 1312,       0,  684394,       0,  0, 14,  0, }, // Myanmar
    {  522, 1320,       0,  684408,       0,  0, 11,  0, }, // Nauru
    {  522, 1326,       0,  684419,       0,  0, 11,  0, }, // Nepal
    {  522, 1332,  684430,  684451,  684484, 21, 33, 29, }, // New_Caledonia
    {  522, 1346,  684513,  684530,  684559, 17, 29, 25, }, // New_Zealand
    {  522, 1358,   29478,  684584,  684614, 12, 30, 26, }, // Newfoundland
    {  522, 1371,       0,  684640,       0,  0, 10,  0, }, // Niue
    {  522, 1376,  684650,  684670,  684702, 20, 32, 28, }, // Norfolk
    {  522, 1384,  684730,  684755,  684792, 25, 37, 33, }, // Noronha
    {  522, 1392,       0,  103545,       0,  0, 31,  0, }, // North_Mariana
    {  522, 1406,  684825,  684842,  684871, 17, 29, 25, }, // Novosibirsk
    {  522, 1418,  684896,  684906,  684928, 10, 22, 18, }, // Omsk
    {  522, 1428,  684946,  684960,  684986, 14, 26, 22, }, // Pakistan
    {  522, 1437,       0,  685008,       0,  0, 11,  0, }, // Palau
    {  522, 1443,       0,  685019,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  522, 1460,  685043,  685057,  685083, 14, 26, 22, }, // Paraguay
    {  522, 1469,  685105,  685115,  685137, 10, 22, 18, }, // Peru
    {  522, 1474,  685155,  685169,  685195, 14, 26, 22, }, // Philippines
    {  522, 1486,       0,  685217,       0,  0, 19,  0, }, // Phoenix_Islands
    {  522, 1502,  678695,  685236,  685272, 18, 36, 32, }, // Pierre_Miquelon
    {  522, 1518,       0,  685304,       0,  0, 13,  0, }, // Pitcairn
    {  522, 1527,       0,  685317,       0,  0, 12,  0, }, // Ponape
    {  522, 1534,       0,  685329,       0,  0, 17,  0, }, // Pyongyang
    {  522, 1544,  104292,  104307,  104333, 15, 26, 34, }, // Qyzylorda
    {  522, 1554,       0,  685346,       0,  0, 13,  0, }, // Reunion
    {  522, 1562,       0,  685359,       0,  0, 12,  0, }, // Rothera
    {  522, 1570,  685371,  685384,  685409, 13, 25, 21, }, // Sakhalin
    {  522, 1579,  104480,  104492,  104515, 12, 23, 31, }, // Samara
    {  522, 1596,  685430,  685441,  685464, 11, 23, 19, }, // Samoa
    {  522, 1602,       0,  685483,       0,  0, 13,  0, }, // Seychelles
    {  522, 1624,       0,  685496,       0,  0, 26,  0, }, // Singapore
    {  522, 1634,       0,  685522,       0,  0, 23,  0, }, // Solomon
    {  522, 1642,       0,  685545,       0,  0, 21,  0, }, // South_Georgia
    {  522, 1656,       0,  685566,       0,  0, 13,  0, }, // Suriname
    {  522, 1676,       0,  685579,       0,  0, 10,  0, }, // Syowa
    {  522, 1682,       0,  685589,       0,  0, 12,  0, }, // Tahiti
    {  522, 1689,  685601,  685613,  685637, 12, 24, 20, }, // Taipei
    {  522, 1696,       0,  685657,       0,  0, 17,  0, }, // Tajikistan
    {  522, 1724,       0,  685674,       0,  0, 13,  0, }, // Tokelau
    {  522, 1732,  685687,  685698,  685721, 11, 23, 19, }, // Tonga
    {  522, 1738,       0,  685740,       0,  0, 10,  0, }, // Truk
    {  522, 1750,  685750,  685768,  685798, 18, 30, 26, }, // Turkmenistan
    {  522, 1763,       0,  685824,       0,  0, 12,  0, }, // Tuvalu
    {  522, 1777,  685836,  685849,  685874, 13, 25, 21, }, // Uruguay
    {  522, 1792,  685895,  685911,  685939, 16, 28, 24, }, // Uzbekistan
    {  522, 1803,  685963,  685976,  686001, 13, 25, 21, }, // Vanuatu
    {  522, 1811,       0,  686022,       0,  0, 15,  0, }, // Venezuela
    {  522, 1821,  686037,  686054,  686083, 17, 29, 25, }, // Vladivostok
    {  522, 1833,  686108,  686123,  686150, 15, 27, 23, }, // Volgograd
    {  522, 1843,       0,  686173,       0,  0, 12,  0, }, // Vostok
    {  522, 1850,       0,  686185,       0,  0, 17,  0, }, // Wake
    {  522, 1855,       0,  686202,       0,  0, 27,  0, }, // Wallis
    {  522, 1862,  686229,  686242,  686267, 13, 25, 21, }, // Yakutsk
    {  522, 1870,  686288,  686307,  686338, 19, 31, 27, }, // Yekaterinburg
    {  522, 1892,       0,   31566,       0,  0,  5,  0, }, // Yukon
    {  525,  788,       0,  686365,       0,  0,  6,  0, }, // GMT Sichuan Yi/Yi/China
    {  526,    5,       0,  686371,       0,  0, 20,  0, }, // Afghanistan Sicilian/Latin/Italy
    {  526,   17,       0,  686391,       0,  0, 24,  0, }, // Africa_Central
    {  526,   32,       0,  686415,       0,  0, 26,  0, }, // Africa_Eastern
    {  526,   65,       0,  686441,       0,  0, 35,  0, }, // Africa_Southern
    {  526,   81,  686476,  686502,  686535, 26, 33, 33, }, // Africa_Western
    {  526,  107,  686568,  686583,  686605, 15, 22, 22, }, // Alaska
    {  526,  135,  686627,  686645,  686670, 18, 25, 25, }, // Amazon
    {  526,  142,  686695,  686707,  686726, 12, 19, 19, }, // America_Central
    {  526,  158,  686745,  686758,  686778, 13, 20, 20, }, // America_Eastern
    {  526,  174,  686798,  686813,  686835, 15, 22, 22, }, // America_Mountain
    {  526,  191,  686857,  686872,  686894, 15, 22, 22, }, // America_Pacific
    {  526,  214,  686916,  686927,  686945, 11, 18, 18, }, // Apia
    {  526,  232,  686963,  686973,  686990, 10, 17, 17, }, // Arabian
    {  526,  240,  687007,  687025,  687050, 18, 25, 25, }, // Argentina
    {  526,  250,  687075,  687104,  687140, 29, 36, 36, }, // Argentina_Western
    {  526,  268,  687176,  687192,  687215, 16, 23, 23, }, // Armenia
    {  526,  286,  687238,  687256,  687281, 18, 25, 25, }, // Atlantic
    {  526,  295,  687306,  687333,  687367, 27, 34, 34, }, // Australia_Central
    {  526,  313,  687401,  687439,  687484, 38, 45, 45, }, // Australia_CentralWestern
    {  526,  338,  687529,  687558,  687594, 29, 36, 36, }, // Australia_Eastern
    {  526,  356,  687630,  687659,  687695, 29, 36, 36, }, // Australia_Western
    {  526,  374,  687731,  687750,  687776, 19, 26, 26, }, // Azerbaijan
    {  526,  385,  687802,  687818,  687841, 16, 23, 23, }, // Azores
    {  526,  397,  687864,  687881,  687905, 17, 24, 24, }, // Bangladesh
    {  526,  415,       0,  687929,       0,  0, 13,  0, }, // Bhutan
    {  526,  422,       0,  687942,       0,  0, 14,  0, }, // Bolivia
    {  526,  437,  687956,  687971,  687993, 15, 22, 22, }, // Brasilia
    {  526,  454,       0,  688015,       0,  0, 13,  0, }, // Brunei
    {  526,  461,  688028,  688045,  688069, 17, 24, 24, }, // Cape_Verde
    {  526,  478,       0,  688093,       0,  0, 15,  0, }, // Chamorro
    {  526,  487,  688108,  688122,  688143, 14, 21, 21, }, // Chatham
    {  526,  495,  688164,  688175,  688193, 11, 18, 18, }, // Chile
    {  526,  501,  688211,  688222,  688240, 11, 18, 18, }, // China
    {  526,  507,       0,  688258,       0,  0, 24,  0, }, // Christmas
    {  526,  517,       0,  688282,       0,  0, 20,  0, }, // Cocos
    {  526,  523,  688302,  688317,  688339, 15, 22, 22, }, // Colombia
    {  526,  532,  688361,  688380,  688406, 19, 26, 26, }, // Cook
    {  526,  537,  688432,  688444,  688463, 12, 19, 19, }, // Cuba
    {  526,  548,       0,  688482,       0,  0, 12,  0, }, // Davis
    {  526,  564,       0,  688494,       0,  0, 23,  0, }, // DumontDUrville
    {  526,  601,       0,  688517,       0,  0, 23,  0, }, // East_Timor
    {  526,  612,  688540,  688563,  688593, 23, 30, 30, }, // Easter
    {  526,  619,       0,  688623,       0,  0, 16,  0, }, // Ecuador
    {  526,  627,  688639,  688659,  688686, 20, 27, 27, }, // Europe_Central
    {  526,  642,  688713,  688739,  688772, 26, 33, 33, }, // Europe_Eastern
    {  526,  657,       0,  688805,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  526,  680,  688836,  688862,  688895, 26, 33, 33, }, // Europe_Western
    {  526,  695,  688928,  688951,  688981, 23, 30, 30, }, // Falkland
    {  526,  704,  689011,  689022,  689040, 11, 18, 18, }, // Fiji
    {  526,  709,       0,  689058,       0,  0, 22,  0, }, // French_Guiana
    {  526,  723,       0,  689080,       0,  0, 41,  0, }, // French_Southern
    {  526,  746,       0,  689121,       0,  0, 16,  0, }, // Galapagos
    {  526,  756,       0,  689137,       0,  0, 14,  0, }, // Gambier
    {  526,  764,  689151,  689165,  689186, 14, 21, 21, }, // Georgia
    {  526,  772,       0,  689207,       0,  0, 22,  0, }, // Gilbert_Islands
    {  526,  788,       0,  689229,       0,  0, 24,  0, }, // GMT
    {  526,  802,  689253,  689271,  689296, 18, 25, 25, }, // Greenland
    {  526,  830,  689321,  689349,  689384, 28, 35, 38, }, // Greenland_Eastern
    {  526,  848,  689422,  689450,  689485, 28, 35, 35, }, // Greenland_Western
    {  526,  871,       0,  689520,       0,  0, 20,  0, }, // Gulf
    {  526,  876,       0,  689058,       0,  0, 13,  0, }, // Guyana
    {  526,  883,  689540,  689555,  689577, 15, 22, 22, }, // Hawaii_Aleutian
    {  526,  899,  689599,  689615,  689638, 16, 23, 23, }, // Hong_Kong
    {  526,  909,  689661,  689672,  689690, 11, 18, 18, }, // Hovd
    {  526,  914,       0,  689708,       0,  0, 21,  0, }, // India
    {  526,  920,       0,  689729,       0,  0, 23,  0, }, // Indian_Ocean
    {  526,  933,       0,  689752,       0,  0, 17,  0, }, // Indochina
    {  526,  943,       0,  689769,       0,  0, 27,  0, }, // Indonesia_Central
    {  526,  961,       0,  689796,       0,  0, 29,  0, }, // Indonesia_Eastern
    {  526,  979,       0,  689825,       0,  0, 29,  0, }, // Indonesia_Western
    {  526,  997,  689854,  689867,  689887, 13, 20, 20, }, // Iran
    {  526, 1008,  689907,  689922,  689944, 15, 22, 22, }, // Irkutsk
    {  526, 1016,  689966,  689981,  690003, 15, 22, 22, }, // Israel
    {  526, 1023,  690025,  690040,  690062, 15, 22, 22, }, // Japan
    {  526, 1047,       0,  690084,       0,  0, 18,  0, }, // Kazakhstan
    {  526, 1058,       0,  690102,       0,  0, 29,  0, }, // Kazakhstan_Eastern
    {  526, 1077,       0,  690131,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  526, 1106,  690160,  690172,  690191, 12, 19, 19, }, // Korea
    {  526, 1112,       0,  690210,       0,  0, 13,  0, }, // Kosrae
    {  526, 1119,  690223,  690241,  690266, 18, 25, 25, }, // Krasnoyarsk
    {  526, 1151,       0,  690291,       0,  0, 20,  0, }, // Kyrgystan
    {  526, 1175,       0,  690311,       0,  0, 19,  0, }, // Line_Islands
    {  526, 1188,  690330,  690346,  690369, 16, 23, 23, }, // Lord_Howe
    {  526, 1204,  690392,  690405,  690425, 13, 20, 20, }, // Magadan
    {  526, 1219,       0,  690445,       0,  0, 14,  0, }, // Malaysia
    {  526, 1228,       0,  690459,       0,  0, 14,  0, }, // Maldives
    {  526, 1237,       0,  690473,       0,  0, 15,  0, }, // Marquesas
    {  526, 1247,       0,  690488,       0,  0, 23,  0, }, // Marshall_Islands
    {  526, 1264,  690511,  690527,  690550, 16, 23, 23, }, // Mauritius
    {  526, 1274,       0,  690573,       0,  0, 13,  0, }, // Mawson
    {  526, 1281,  690586,  690609,  690639, 23, 30, 30, }, // Mexico_Pacific
    {  526, 1296,  690669,  690687,  690712, 18, 25, 25, }, // Mongolia
    {  526, 1305,  690737,  690749,  690768, 12, 19, 19, }, // Moscow
    {  526, 1312,       0,  690787,       0,  0, 14,  0, }, // Myanmar
    {  526, 1320,       0,  690801,       0,  0, 12,  0, }, // Nauru
    {  526, 1326,       0,  690813,       0,  0, 12,  0, }, // Nepal
    {  526, 1332,  690825,  690846,  690874, 21, 28, 28, }, // New_Caledonia
    {  526, 1346,  690902,  690922,  690949, 20, 27, 27, }, // New_Zealand
    {  526, 1358,  690976,  690992,  691015, 16, 23, 23, }, // Newfoundland
    {  526, 1371,       0,  691038,       0,  0, 11,  0, }, // Niue
    {  526, 1376,  691049,  691071,  691100, 22, 29, 29, }, // Norfolk
    {  526, 1384,  691129,  691155,  691188, 26, 33, 33, }, // Noronha
    {  526, 1406,  691221,  691239,  691264, 18, 25, 25, }, // Novosibirsk
    {  526, 1418,  691289,  691300,  691318, 11, 18, 18, }, // Omsk
    {  526, 1428,  691336,  691351,  691373, 15, 22, 22, }, // Pakistan
    {  526, 1437,       0,  691395,       0,  0, 12,  0, }, // Palau
    {  526, 1443,       0,  691407,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  526, 1460,  691431,  691446,  691468, 15, 22, 22, }, // Paraguay
    {  526, 1469,  691490,  691501,  691519, 11, 18, 18, }, // Peru
    {  526, 1474,  691537,  691553,  691576, 16, 23, 23, }, // Philippines
    {  526, 1486,       0,  691599,       0,  0, 22,  0, }, // Phoenix_Islands
    {  526, 1502,  691621,  691648,  691682, 27, 34, 34, }, // Pierre_Miquelon
    {  526, 1518,       0,  691716,       0,  0, 15,  0, }, // Pitcairn
    {  526, 1527,       0,  691731,       0,  0, 13,  0, }, // Ponape
    {  526, 1534,       0,  691744,       0,  0, 16,  0, }, // Pyongyang
    {  526, 1554,       0,  691760,       0,  0, 14,  0, }, // Reunion
    {  526, 1562,       0,  691774,       0,  0, 14,  0, }, // Rothera
    {  526, 1570,  691788,  691803,  691825, 15, 22, 22, }, // Sakhalin
    {  526, 1596,  691847,  691859,  691878, 12, 19, 19, }, // Samoa
    {  526, 1602,       0,  691897,       0,  0, 17,  0, }, // Seychelles
    {  526, 1624,       0,  691914,       0,  0, 16,  0, }, // Singapore
    {  526, 1634,       0,  691930,       0,  0, 23,  0, }, // Solomon
    {  526, 1642,       0,  691953,       0,  0, 26,  0, }, // South_Georgia
    {  526, 1656,       0,  691979,       0,  0, 15,  0, }, // Suriname
    {  526, 1676,       0,  691994,       0,  0, 12,  0, }, // Syowa
    {  526, 1682,       0,  692006,       0,  0, 13,  0, }, // Tahiti
    {  526, 1689,  692019,  692032,  692052, 13, 20, 20, }, // Taipei
    {  526, 1696,       0,  692072,       0,  0, 18,  0, }, // Tajikistan
    {  526, 1724,       0,  692090,       0,  0, 14,  0, }, // Tokelau
    {  526, 1732,  692104,  692116,  692135, 12, 19, 19, }, // Tonga
    {  526, 1738,       0,  692154,       0,  0, 12,  0, }, // Truk
    {  526, 1750,  692166,  692185,  692211, 19, 26, 26, }, // Turkmenistan
    {  526, 1763,       0,  692237,       0,  0, 13,  0, }, // Tuvalu
    {  526, 1777,  692250,  692266,  692289, 16, 23, 23, }, // Uruguay
    {  526, 1792,  692312,  692331,  692357, 19, 26, 26, }, // Uzbekistan
    {  526, 1803,  692383,  692397,  692418, 14, 21, 21, }, // Vanuatu
    {  526, 1811,       0,  692439,       0,  0, 17,  0, }, // Venezuela
    {  526, 1821,  692456,  692474,  692499, 18, 25, 25, }, // Vladivostok
    {  526, 1833,  692524,  692540,  692563, 16, 23, 23, }, // Volgograd
    {  526, 1843,       0,  692586,       0,  0, 13,  0, }, // Vostok
    {  526, 1850,       0,  692599,       0,  0, 19,  0, }, // Wake
    {  526, 1855,       0,  692618,       0,  0, 22,  0, }, // Wallis
    {  526, 1862,  692640,  692654,  692675, 14, 21, 21, }, // Yakutsk
    {  526, 1870,  692696,  692717,  692745, 21, 28, 28, }, // Yekaterinburg
    {  526, 1892,       0,  692773,       0,  0, 12,  0, }, // Yukon
    {  528,    5,       0,   13038,       0,  0, 10,  0, }, // Afghanistan Silesian/Latin/Poland
    {  528,   17,       0,  692785,       0,  0, 27,  0, }, // Africa_Central
    {  528,   32,       0,  692812,       0,  0, 24,  0, }, // Africa_Eastern
    {  528,   65,       0,  692836,       0,  0, 25,  0, }, // Africa_Southern
    {  528,   81,  692861,  692885,  692922, 24, 37, 31, }, // Africa_Western
    {  528,  107,  692953,  692968,  692996, 15, 28, 22, }, // Alaska
    {  528,  128,  603528,  693018,  693043, 12, 25, 19, }, // Almaty
    {  528,  135,  693062,  693076,  693103, 14, 27, 21, }, // Amazon
    {  528,  142,  693124,  693152,  693193, 28, 41, 35, }, // America_Central
    {  528,  158,  603732,  693228,  693266, 25, 38, 32, }, // America_Eastern
    {  528,  174,  693298,  693309,  693333, 11, 24, 18, }, // America_Mountain
    {  528,  191,  603876,  693351,  693379, 15, 28, 22, }, // America_Pacific
    {  528,  207,  603939,  693401,  693425, 11, 24, 18, }, // Anadyr
    {  528,  219,  604039,  693443,  693470, 14, 27, 21, }, // Aqtau
    {  528,  225,  604099,  693491,  693520, 16, 29, 23, }, // Aqtobe
    {  528,  232,  693543,  693559,  693595, 16, 36, 30, }, // Arabian
    {  528,  240,  693625,  693634,  693663,  9, 29, 23, }, // Argentina
    {  528,  250,  693686,  693705,  693744, 19, 39, 33, }, // Argentina_Western
    {  528,  268,  693777,  693785,  693813,  8, 28, 22, }, // Armenia
    {  528,  286,  604466,  693835,  693863, 15, 28, 22, }, // Atlantic
    {  528,  295,  693885,  693914,  693956, 29, 42, 36, }, // Australia_Central
    {  528,  313,  693992,  694031,  694083, 39, 52, 46, }, // Australia_CentralWestern
    {  528,  338,  694129,  694155,  694194, 26, 39, 33, }, // Australia_Eastern
    {  528,  356,  694005,  694044,  694096, 26, 39, 33, }, // Australia_Western
    {  528,  374,  604942,  694227,  694258, 11, 31, 25, }, // Azerbaijan
    {  528,  385,  605012,  694283,  694308,  5, 25, 19, }, // Azores
    {  528,  397,  605064,  694327,  694357, 10, 30, 24, }, // Bangladesh
    {  528,  422,       0,  694381,       0,  0,  8,  0, }, // Bolivia
    {  528,  437,  115224,  694389,  694417,  8, 28, 22, }, // Brasilia
    {  528,  461,  694439,  694465,  694511, 26, 46, 40, }, // Cape_Verde
    {  528,  478,       0,  605338,       0,  0,  8,  0, }, // Chamorro
    {  528,  495,  605404,  694551,  694576, 10, 25, 19, }, // Chile
    {  528,  501,  605461,  694595,  694620,  5, 25, 19, }, // China
    {  528,  507,       0,  694639,       0,  0, 12,  0, }, // Christmas
    {  528,  517,       0,  605541,       0,  0, 14,  0, }, // Cocos
    {  528,  523,  475482,  694651,  694680,  9, 29, 23, }, // Colombia
    {  528,  532,  605621,  694703,  694734, 11, 31, 25, }, // Cook
    {  528,  537,   14969,  694759,  694783,  4, 24, 18, }, // Cuba
    {  528,  564,       0,    2146,       0,  0, 16,  0, }, // DumontDUrville
    {  528,  601,       0,  605771,       0,  0, 14,  0, }, // East_Timor
    {  528,  612,  694801,  694818,  694855, 17, 37, 31, }, // Easter
    {  528,  619,       0,  448966,       0,  0,  7,  0, }, // Ecuador
    {  528,  627,  694886,  694913,  694953, 27, 40, 34, }, // Europe_Central
    {  528,  642,  694987,  695011,  695048, 24, 37, 31, }, // Europe_Eastern
    {  528,  657,       0,  695079,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  528,  680,  695110,  695134,  695171, 24, 37, 31, }, // Europe_Western
    {  528,  695,  606188,  695202,  695231,  9, 29, 23, }, // Falkland
    {  528,  704,  606252,  695254,  695279,  5, 25, 19, }, // Fiji
    {  528,  709,       0,  695298,       0,  0, 16,  0, }, // French_Guiana
    {  528,  723,       0,  695314,       0,  0, 45,  0, }, // French_Southern
    {  528,  746,       0,  606371,       0,  0, 14,  0, }, // Galapagos
    {  528,  756,       0,  606390,       0,  0, 14,  0, }, // Gambier
    {  528,  764,  695359,  695366,  695393,  7, 27, 21, }, // Georgia
    {  528,  772,       0,  695414,       0,  0, 16,  0, }, // Gilbert_Islands
    {  528,  788,       0,  695430,       0,  0, 19,  0, }, // GMT
    {  528,  830,  695449,  695470,  695511, 21, 41, 35, }, // Greenland_Eastern
    {  528,  848,  695546,  695567,  695608, 21, 41, 35, }, // Greenland_Western
    {  528,  871,       0,  606693,       0,  0, 13,  0, }, // Gulf
    {  528,  876,       0,  241703,       0,  0,  6,  0, }, // Guyana
    {  528,  883,  606711,  695643,  695676, 13, 33, 27, }, // Hawaii_Aleutian
    {  528,  899,  695703,  695711,  695739,  8, 28, 22, }, // Hong_Kong
    {  528,  909,  606848,  695761,  695786,  5, 25, 19, }, // Hovd
    {  528,  914,       0,  695805,       0,  0, 26,  0, }, // India
    {  528,  920,       0,  695831,       0,  0, 14,  0, }, // Indian_Ocean
    {  528,  933,       0,  695845,       0,  0, 16,  0, }, // Indochina
    {  528,  943,       0,  695861,       0,  0, 23,  0, }, // Indonesia_Central
    {  528,  961,       0,  695884,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  528,  979,       0,  695904,       0,  0, 20,  0, }, // Indonesia_Western
    {  528, 1008,  556523,  695924,  695950,  6, 26, 20, }, // Irkutsk
    {  528, 1016,   28155,  695970,  695996,  6, 26, 20, }, // Israel
    {  528, 1023,  696016,  696024,  696052,  8, 28, 22, }, // Japan
    {  528, 1029,  607243,  696074,  696116, 29, 42, 36, }, // Kamchatka
    {  528, 1058,       0,  607368,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  528, 1077,       0,  607392,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  528, 1119,   16409,  696152,  696183, 11, 31, 25, }, // Krasnoyarsk
    {  528, 1151,       0,   16468,       0,  0,  9,  0, }, // Kyrgystan
    {  528, 1175,       0,    3870,       0,  0, 12,  0, }, // Line_Islands
    {  528, 1188,    3887,  696208,  696237,  9, 29, 23, }, // Lord_Howe
    {  528, 1219,       0,  696260,       0,  0,  8,  0, }, // Malaysia
    {  528, 1228,       0,  607714,       0,  0,  8,  0, }, // Maldives
    {  528, 1237,       0,  607727,       0,  0,  7,  0, }, // Marquesas
    {  528, 1247,       0,  607739,       0,  0, 15,  0, }, // Marshall_Islands
    {  528, 1281,  607829,  696268,  696305, 24, 37, 31, }, // Mexico_Pacific
    {  528, 1296,  607924,  696336,  696366, 10, 30, 24, }, // Mongolia
    {  528, 1305,  607991,  696390,  696411,  6, 21, 15, }, // Moscow
    {  528, 1312,       0,  351081,       0,  0,  6,  0, }, // Myanmar
    {  528, 1332,  696426,  696441,  696476, 15, 35, 29, }, // New_Caledonia
    {  528, 1346,  696505,  696519,  696553, 14, 34, 28, }, // New_Zealand
    {  528, 1358,  696581,  696597,  696633, 16, 36, 30, }, // Newfoundland
    {  528, 1384,    4687,  696663,  696702, 19, 39, 33, }, // Noronha
    {  528, 1406,  608475,  696735,  696766, 11, 31, 25, }, // Novosibirsk
    {  528, 1418,  696791,  696795,  696819,  4, 24, 18, }, // Omsk
    {  528, 1443,       0,  696837,       0,  0, 17,  0, }, // Papua_New_Guinea
    {  528, 1460,  608687,  696854,  696882,  8, 28, 22, }, // Paraguay
    {  528, 1469,  608743,  696904,  696928,  9, 24, 18, }, // Peru
    {  528, 1474,  608797,  696946,  696974,  8, 28, 22, }, // Philippines
    {  528, 1486,       0,  696996,       0,  0,  6,  0, }, // Phoenix_Islands
    {  528, 1502,  608869,  697002,  697045, 23, 43, 37, }, // Pierre_Miquelon
    {  528, 1527,       0,  118434,       0,  0,  7,  0, }, // Ponape
    {  528, 1534,       0,  697082,       0,  0,  9,  0, }, // Pyongyang
    {  528, 1544,  609009,  697091,  697120, 16, 29, 23, }, // Qyzylorda
    {  528, 1570,  212226,  697143,  697171,  8, 28, 22, }, // Sakhalin
    {  528, 1579,  609160,  697193,  697217, 11, 24, 18, }, // Samara
    {  528, 1602,       0,  609268,       0,  0,  7,  0, }, // Seychelles
    {  528, 1624,       0,   63735,       0,  0,  8,  0, }, // Singapore
    {  528, 1634,       0,  697235,       0,  0, 14,  0, }, // Solomon
    {  528, 1642,       0,  697249,       0,  0, 18,  0, }, // South_Georgia
    {  528, 1656,       0,    5617,       0,  0,  7,  0, }, // Suriname
    {  528, 1689,  104774,  697267,  697293,  6, 26, 20, }, // Taipei
    {  528, 1696,       0,  609423,       0,  0, 11,  0, }, // Tajikistan
    {  528, 1732,  697313,  697318,  697343,  5, 25, 19, }, // Tonga
    {  528, 1738,       0,    5775,       0,  0,  5,  0, }, // Truk
    {  528, 1750,  697362,  697374,  697406, 12, 32, 26, }, // Turkmenistan
    {  528, 1777,  609597,  697432,  697459,  7, 27, 21, }, // Uruguay
    {  528, 1811,       0,  697480,       0,  0,  9,  0, }, // Venezuela
    {  528, 1821,  609794,  697489,  697520, 11, 31, 25, }, // Vladivostok
    {  528, 1833,  609864,  697545,  697574,  9, 29, 23, }, // Volgograd
    {  528, 1843,       0,   18491,       0,  0,  6,  0, }, // Vostok
    {  528, 1855,       0,  144187,       0,  0, 15,  0, }, // Wallis
    {  528, 1862,  558862,  697597,  697623,  6, 26, 20, }, // Yakutsk
    {  528, 1870,  610023,  697643,  697676, 13, 33, 27, }, // Yekaterinburg
    {  528, 1892,       0,  697703,       0,  0, 13,  0, }, // Yukon
    {  529,    5,       0,  697716,       0,  0, 16,  0, }, // Afghanistan Sindhi/Arabic/Pakistan
    {  529,   17,       0,  697732,       0,  0, 19,  0, }, // Africa_Central
    {  529,   32,       0,  697751,       0,  0, 18,  0, }, // Africa_Eastern
    {  529,   65,       0,  697769,       0,  0, 24,  0, }, // Africa_Southern
    {  529,   81,  697793,  697812,  697838, 19, 26, 30, }, // Africa_Western
    {  529,  107,  697868,  697881,  697901, 13, 20, 22, }, // Alaska
    {  529,  135,  697923,  697937,  697958, 14, 21, 25, }, // Amazon
    {  529,  142,  697983,  697992,  698008,  9, 16, 18, }, // America_Central
    {  529,  158,  698026,  698035,  698051,  9, 16, 18, }, // America_Eastern
    {  529,  174,  698069,  698078,  698094,  9, 16, 18, }, // America_Mountain
    {  529,  191,  698112,  698122,  698139, 10, 17, 19, }, // America_Pacific
    {  529,  214,  698158,  698169,  698187, 11, 18, 20, }, // Apia
    {  529,  232,  698207,  698219,  698238, 12, 19, 21, }, // Arabian
    {  529,  240,  698259,  698274,  698293, 15, 19, 26, }, // Argentina
    {  529,  250,  698319,  698340,  698368, 21, 28, 32, }, // Argentina_Western
    {  529,  268,  698400,  698414,  698435, 14, 21, 25, }, // Armenia
    {  529,  286,  698460,  698475,  698497, 15, 22, 24, }, // Atlantic
    {  529,  295,  698521,  698542,  698570, 21, 28, 30, }, // Australia_Central
    {  529,  313,  698600,  698627,  698661, 27, 34, 36, }, // Australia_CentralWestern
    {  529,  338,  698697,  698717,  698745, 20, 28, 30, }, // Australia_Eastern
    {  529,  356,  698775,  698796,  698824, 21, 28, 30, }, // Australia_Western
    {  529,  374,  698854,  698871,  698895, 17, 24, 28, }, // Azerbaijan
    {  529,  385,  698923,  698935,  698954, 12, 19, 23, }, // Azores
    {  529,  397,  698977,  698992,  699014, 15, 22, 26, }, // Bangladesh
    {  529,  415,       0,  699040,       0,  0, 12,  0, }, // Bhutan
    {  529,  422,       0,  699052,       0,  0, 12,  0, }, // Bolivia
    {  529,  437,  699064,  699078,  699100, 14, 22, 26, }, // Brasilia
    {  529,  454,       0,  699126,       0,  0, 24,  0, }, // Brunei
    {  529,  461,  699150,  699164,  699185, 14, 21, 25, }, // Cape_Verde
    {  529,  478,       0,  699210,       0,  0, 19,  0, }, // Chamorro
    {  529,  487,  699229,  699240,  699258, 11, 18, 20, }, // Chatham
    {  529,  495,  699278,  699288,  699305, 10, 17, 21, }, // Chile
    {  529,  501,  699326,  699336,  699355, 10, 19, 21, }, // China
    {  529,  507,       0,  699376,       0,  0, 21,  0, }, // Christmas
    {  529,  517,       0,  699397,       0,  0, 21,  0, }, // Cocos
    {  529,  523,  699418,  699432,  699453, 14, 21, 25, }, // Colombia
    {  529,  532,  699478,  699497,  699523, 19, 26, 33, }, // Cook
    {  529,  537,  699556,  699568,  699587, 12, 19, 21, }, // Cuba
    {  529,  548,       0,  699608,       0,  0, 11,  0, }, // Davis
    {  529,  564,       0,  699619,       0,  0, 20,  0, }, // DumontDUrville
    {  529,  601,       0,  699639,       0,  0, 17,  0, }, // East_Timor
    {  529,  612,  699656,  699677,  699705, 21, 28, 32, }, // Easter
    {  529,  619,       0,  699737,       0,  0, 15,  0, }, // Ecuador
    {  529,  627,  699752,  699767,  699789, 15, 22, 26, }, // Europe_Central
    {  529,  642,  699815,  699830,  699852, 15, 22, 26, }, // Europe_Eastern
    {  529,  657,       0,  699878,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  529,  680,  699898,  699913,  699935, 15, 22, 24, }, // Europe_Western
    {  529,  695,  699959,  699983,  700014, 24, 31, 35, }, // Falkland
    {  529,  704,  700049,  700059,  700076, 10, 17, 21, }, // Fiji
    {  529,  709,       0,  700097,       0,  0, 21,  0, }, // French_Guiana
    {  529,  723,       0,  700118,       0,  0, 30,  0, }, // French_Southern
    {  529,  746,       0,  700148,       0,  0, 16,  0, }, // Galapagos
    {  529,  756,       0,  700164,       0,  0, 14,  0, }, // Gambier
    {  529,  764,  700178,  700191,  700211, 13, 20, 24, }, // Georgia
    {  529,  772,       0,  700235,       0,  0, 21,  0, }, // Gilbert_Islands
    {  529,  788,       0,  700256,       0,  0, 17,  0, }, // GMT
    {  529,  830,  700273,  700295,  700324, 22, 29, 33, }, // Greenland_Eastern
    {  529,  848,  700357,  700379,  700408, 22, 29, 33, }, // Greenland_Western
    {  529,  871,       0,  700441,       0,  0, 16,  0, }, // Gulf
    {  529,  876,       0,  700457,       0,  0, 11,  0, }, // Guyana
    {  529,  883,  700468,  700488,  700515, 20, 27, 29, }, // Hawaii_Aleutian
    {  529,  899,  700544,  700560,  700583, 16, 23, 27, }, // Hong_Kong
    {  529,  909,  700610,  700621,  700639, 11, 18, 22, }, // Hovd
    {  529,  914,       0,  700661,       0,  0, 18,  0, }, // India
    {  529,  920,       0,  700679,       0,  0, 15,  0, }, // Indian_Ocean
    {  529,  933,       0,  700694,       0,  0, 17,  0, }, // Indochina
    {  529,  943,       0,  700711,       0,  0, 22,  0, }, // Indonesia_Central
    {  529,  961,       0,  700733,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  529,  979,       0,  700754,       0,  0, 22,  0, }, // Indonesia_Western
    {  529,  997,  700776,  700788,  700807, 12, 19, 21, }, // Iran
    {  529, 1008,  700828,  700841,  700861, 13, 20, 22, }, // Irkutsk
    {  529, 1016,  700883,  700897,  700918, 14, 21, 23, }, // Israel
    {  529, 1023,  700941,  700953,  700972, 12, 19, 21, }, // Japan
    {  529, 1047,       0,  700993,       0,  0, 12,  0, }, // Kazakhstan
    {  529, 1058,       0,  701005,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  529, 1077,       0,  701025,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  529, 1106,  701046,  701058,  701077, 12, 19, 21, }, // Korea
    {  529, 1112,       0,  701098,       0,  0, 14,  0, }, // Kosrae
    {  529, 1119,  701112,  701130,  701155, 18, 25, 27, }, // Krasnoyarsk
    {  529, 1151,       0,  701182,       0,  0, 15,  0, }, // Kyrgystan
    {  529, 1175,       0,  701197,       0,  0, 20,  0, }, // Line_Islands
    {  529, 1188,  701217,  701233,  701256, 16, 23, 25, }, // Lord_Howe
    {  529, 1204,  701281,  701294,  701314, 13, 20, 22, }, // Magadan
    {  529, 1219,       0,  701336,       0,  0, 15,  0, }, // Malaysia
    {  529, 1228,       0,  701351,       0,  0, 13,  0, }, // Maldives
    {  529, 1237,       0,  701364,       0,  0, 14,  0, }, // Marquesas
    {  529, 1247,       0,  701378,       0,  0, 21,  0, }, // Marshall_Islands
    {  529, 1264,  701399,  701413,  701434, 14, 21, 25, }, // Mauritius
    {  529, 1274,       0,  701459,       0,  0, 13,  0, }, // Mawson
    {  529, 1281,  701472,  701490,  701518, 18, 28, 30, }, // Mexico_Pacific
    {  529, 1296,  701548,  701565,  701589, 17, 24, 28, }, // Mongolia
    {  529, 1305,  701617,  701629,  701648, 12, 19, 21, }, // Moscow
    {  529, 1312,       0,  701669,       0,  0, 14,  0, }, // Myanmar
    {  529, 1320,       0,  701683,       0,  0, 13,  0, }, // Nauru
    {  529, 1326,       0,  701696,       0,  0, 12,  0, }, // Nepal
    {  529, 1332,  701708,  701728,  701755, 20, 27, 31, }, // New_Caledonia
    {  529, 1346,  701786,  701802,  701825, 16, 23, 25, }, // New_Zealand
    {  529, 1358,  701850,  701872,  701901, 22, 29, 31, }, // Newfoundland
    {  529, 1371,       0,  701932,       0,  0, 12,  0, }, // Niue
    {  529, 1376,  701944,  701967,  701997, 23, 30, 32, }, // Norfolk
    {  529, 1384,  702029,  702054,  702086, 25, 32, 33, }, // Noronha
    {  529, 1406,  702119,  702137,  702162, 18, 25, 27, }, // Novosibirsk
    {  529, 1418,  702189,  702201,  702220, 12, 19, 21, }, // Omsk
    {  529, 1428,  702241,  702255,  702276, 14, 21, 25, }, // Pakistan
    {  529, 1437,       0,  702301,       0,  0, 12,  0, }, // Palau
    {  529, 1443,       0,  702313,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  529, 1460,  702333,  702348,  702370, 15, 22, 26, }, // Paraguay
    {  529, 1469,  702396,  702407,  702425, 11, 18, 22, }, // Peru
    {  529, 1474,  702447,  702460,  702480, 13, 20, 24, }, // Philippines
    {  529, 1486,       0,  702504,       0,  0, 22,  0, }, // Phoenix_Islands
    {  529, 1502,  702526,  702551,  702583, 25, 32, 34, }, // Pierre_Miquelon
    {  529, 1518,       0,  702617,       0,  0, 13,  0, }, // Pitcairn
    {  529, 1527,       0,  702630,       0,  0, 13,  0, }, // Ponape
    {  529, 1534,       0,  702643,       0,  0, 17,  0, }, // Pyongyang
    {  529, 1554,       0,  702660,       0,  0, 15,  0, }, // Reunion
    {  529, 1562,       0,  702675,       0,  0, 13,  0, }, // Rothera
    {  529, 1570,  702688,  702701,  702721, 13, 20, 22, }, // Sakhalin
    {  529, 1596,  702743,  702755,  702774, 12, 19, 21, }, // Samoa
    {  529, 1602,       0,  702795,       0,  0, 13,  0, }, // Seychelles
    {  529, 1624,       0,  702808,       0,  0, 21,  0, }, // Singapore
    {  529, 1634,       0,  702829,       0,  0, 22,  0, }, // Solomon
    {  529, 1642,       0,  702851,       0,  0, 17,  0, }, // South_Georgia
    {  529, 1656,       0,  702868,       0,  0, 15,  0, }, // Suriname
    {  529, 1676,       0,  702883,       0,  0, 13,  0, }, // Syowa
    {  529, 1682,       0,  702896,       0,  0, 13,  0, }, // Tahiti
    {  529, 1689,  702909,  702921,  702940, 12, 19, 21, }, // Taipei
    {  529, 1696,       0,  702961,       0,  0, 15,  0, }, // Tajikistan
    {  529, 1724,       0,  702976,       0,  0, 15,  0, }, // Tokelau
    {  529, 1732,  702991,  703003,  703022, 12, 19, 23, }, // Tonga
    {  529, 1738,       0,  703045,       0,  0, 11,  0, }, // Truk
    {  529, 1750,  703056,  703073,  703097, 17, 24, 28, }, // Turkmenistan
    {  529, 1763,       0,  703125,       0,  0, 13,  0, }, // Tuvalu
    {  529, 1777,  703138,  703153,  703175, 15, 22, 26, }, // Uruguay
    {  529, 1792,  703201,  703216,  703238, 15, 22, 26, }, // Uzbekistan
    {  529, 1803,  703264,  703278,  703299, 14, 21, 23, }, // Vanuatu
    {  529, 1811,       0,  703322,       0,  0, 15,  0, }, // Venezuela
    {  529, 1821,  703337,  703354,  703378, 17, 24, 27, }, // Vladivostok
    {  529, 1833,  703405,  703421,  703444, 16, 23, 25, }, // Volgograd
    {  529, 1843,       0,  703469,       0,  0, 13,  0, }, // Vostok
    {  529, 1850,       0,  703482,       0,  0, 19,  0, }, // Wake
    {  529, 1855,       0,  703501,       0,  0, 19,  0, }, // Wallis
    {  529, 1862,  703520,  703533,  703553, 13, 20, 22, }, // Yakutsk
    {  529, 1870,  703575,  703592,  703616, 17, 24, 26, }, // Yekaterinburg
    {  529, 1892,       0,  703642,       0,  0, 11,  0, }, // Yukon
    {  530,  142,  703653,  703664,  703682, 11, 18, 20, }, // America_Central Sindhi/Devanagari/India
    {  530,  158,  703702,  703711,  703727,  9, 16, 18, }, // America_Eastern
    {  530,  174,  703745,  703756,  703774, 11, 18, 20, }, // America_Mountain
    {  530,  191,  703794,  703806,  703825, 12, 19, 21, }, // America_Pacific
    {  530,  286,  703846,  703859,  703879, 13, 20, 22, }, // Atlantic
    {  530,  627,  703901,  703919,  703945, 18, 26, 29, }, // Europe_Central
    {  530,  642,  703974,  703991,  704015, 17, 24, 27, }, // Europe_Eastern
    {  530,  680,  704042,  704061,  704087, 19, 26, 29, }, // Europe_Western
    {  530,  788,       0,  704116,       0,  0, 19,  0, }, // GMT
    {  531,    5,       0,  704135,       0,  0, 17,  0, }, // Afghanistan Sinhala/Sinhala/Sri Lanka
    {  531,   17,       0,  704152,       0,  0, 23,  0, }, // Africa_Central
    {  531,   32,       0,  704175,       0,  0, 25,  0, }, // Africa_Eastern
    {  531,   65,       0,  704200,       0,  0, 22,  0, }, // Africa_Southern
    {  531,   81,  704222,  704244,  704272, 22, 28, 30, }, // Africa_Western
    {  531,  107,  704302,  704314,  704332, 12, 18, 21, }, // Alaska
    {  531,  135,  704353,  704366,  704385, 13, 19, 21, }, // Amazon
    {  531,  142,  704406,  704433,  704466, 27, 33, 36, }, // America_Central
    {  531,  158,  704502,  704531,  704566, 29, 35, 38, }, // America_Eastern
    {  531,  174,  704604,  704630,  704662, 26, 32, 35, }, // America_Mountain
    {  531,  191,  704697,  704726,  704761, 29, 35, 38, }, // America_Pacific
    {  531,  214,  704799,  704810,  704827, 11, 17, 16, }, // Apia
    {  531,  232,  704843,  704854,  704871, 11, 17, 17, }, // Arabian
    {  531,  240,  704888,  704904,  704926, 16, 22, 24, }, // Argentina
    {  531,  250,  704950,  704972,  705000, 22, 28, 30, }, // Argentina_Western
    {  531,  268,  705030,  705045,  705066, 15, 21, 24, }, // Armenia
    {  531,  286,  705090,  705107,  705130, 17, 23, 26, }, // Atlantic
    {  531,  295,  705156,  705183,  705209, 27, 26, 33, }, // Australia_Central
    {  531,  313,  705242,  705275,  705314, 33, 39, 39, }, // Australia_CentralWestern
    {  531,  338,  705353,  705382,  705417, 29, 35, 35, }, // Australia_Eastern
    {  531,  356,  705452,  705478,  705321, 26, 32, 32, }, // Australia_Western
    {  531,  374,  705510,  705527,  705550, 17, 23, 26, }, // Azerbaijan
    {  531,  385,  705576,  705589,  705608, 13, 19, 22, }, // Azores
    {  531,  397,  705630,  705644,  705664, 14, 20, 22, }, // Bangladesh
    {  531,  415,       0,  705686,       0,  0, 11,  0, }, // Bhutan
    {  531,  422,       0,  705697,       0,  0, 14,  0, }, // Bolivia
    {  531,  437,  705711,  705724,  705743, 13, 19, 21, }, // Brasilia
    {  531,  454,       0,  705764,       0,  0, 23,  0, }, // Brunei
    {  531,  461,  705787,  705801,  705821, 14, 20, 22, }, // Cape_Verde
    {  531,  478,       0,  705843,       0,  0, 11,  0, }, // Chamorro
    {  531,  487,  705854,  705865,  705882, 11, 17, 16, }, // Chatham
    {  531,  495,  705898,  705908,  705924, 10, 16, 18, }, // Chile
    {  531,  501,  705942,  705951,  705966,  9, 15, 15, }, // China
    {  531,  507,       0,  705981,       0,  0, 22,  0, }, // Christmas
    {  531,  517,       0,  706003,       0,  0, 18,  0, }, // Cocos
    {  531,  523,  706021,  706037,  706059, 16, 22, 24, }, // Colombia
    {  531,  532,  706083,  706099,  706121, 16, 22, 29, }, // Cook
    {  531,  537,  706150,  706164,  706184, 14, 20, 23, }, // Cuba
    {  531,  548,       0,  706207,       0,  0, 12,  0, }, // Davis
    {  531,  564,       0,  706219,       0,  0, 24,  0, }, // DumontDUrville
    {  531,  601,       0,  706243,       0,  0, 21,  0, }, // East_Timor
    {  531,  612,  706264,  706282,  706306, 18, 24, 26, }, // Easter
    {  531,  619,       0,  706332,       0,  0, 14,  0, }, // Ecuador
    {  531,  627,  706346,  706366,  706392, 20, 26, 29, }, // Europe_Central
    {  531,  642,  706421,  706443,  706471, 22, 28, 31, }, // Europe_Eastern
    {  531,  657,       0,  706502,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  531,  680,  706529,  706548,  706573, 19, 25, 28, }, // Europe_Western
    {  531,  695,  706601,  706622,  706649, 21, 27, 29, }, // Falkland
    {  531,  704,  706678,  706688,  706704, 10, 16, 19, }, // Fiji
    {  531,  709,       0,  706723,       0,  0, 17,  0, }, // French_Guiana
    {  531,  723,       0,  706740,       0,  0, 38,  0, }, // French_Southern
    {  531,  746,       0,  706778,       0,  0, 13,  0, }, // Galapagos
    {  531,  756,       0,  706791,       0,  0, 15,  0, }, // Gambier
    {  531,  764,  706806,  706822,  706844, 16, 22, 25, }, // Georgia
    {  531,  772,       0,  706869,       0,  0, 21,  0, }, // Gilbert_Islands
    {  531,  788,       0,  706890,       0,  0, 22,  0, }, // GMT
    {  531,  830,  706912,  706938,  706970, 26, 32, 34, }, // Greenland_Eastern
    {  531,  848,  707004,  707027,  707056, 23, 29, 31, }, // Greenland_Western
    {  531,  871,       0,  707087,       0,  0, 11,  0, }, // Gulf
    {  531,  876,       0,  706730,       0,  0, 10,  0, }, // Guyana
    {  531,  883,  707098,  707121,  707150, 23, 29, 32, }, // Hawaii_Aleutian
    {  531,  899,  707182,  707194,  707212, 12, 18, 21, }, // Hong_Kong
    {  531,  909,  707233,  707245,  707263, 12, 18, 21, }, // Hovd
    {  531,  914,       0,  707284,       0,  0, 15,  0, }, // India
    {  531,  920,       0,  707299,       0,  0, 19,  0, }, // Indian_Ocean
    {  531,  933,       0,  707318,       0,  0, 14,  0, }, // Indochina
    {  531,  943,       0,  707332,       0,  0, 26,  0, }, // Indonesia_Central
    {  531,  961,       0,  707358,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  531,  979,       0,  707386,       0,  0, 25,  0, }, // Indonesia_Western
    {  531,  997,  707411,  707421,  707437, 10, 16, 14, }, // Iran
    {  531, 1008,  707451,  707468,  707491, 17, 23, 26, }, // Irkutsk
    {  531, 1016,  707517,  707531,  707551, 14, 20, 20, }, // Israel
    {  531, 1023,  707571,  707581,  707597, 10, 16, 16, }, // Japan
    {  531, 1047,       0,  707613,       0,  0, 14,  0, }, // Kazakhstan
    {  531, 1058,       0,  707627,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  531, 1077,       0,  707650,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  531, 1106,  707670,  707684,  707704, 14, 20, 20, }, // Korea
    {  531, 1112,       0,  707724,       0,  0, 12,  0, }, // Kosrae
    {  531, 1119,  707736,  707758,  707786, 22, 28, 31, }, // Krasnoyarsk
    {  531, 1151,       0,  707817,       0,  0, 17,  0, }, // Kyrgystan
    {  531, 1161,       0,  707834,       0,  0, 16,  0, }, // Lanka
    {  531, 1175,       0,  707850,       0,  0, 17,  0, }, // Line_Islands
    {  531, 1188,  707867,  707884,  707907, 17, 23, 22, }, // Lord_Howe
    {  531, 1204,  707929,  707941,  707959, 12, 18, 21, }, // Magadan
    {  531, 1219,       0,  707980,       0,  0, 16,  0, }, // Malaysia
    {  531, 1228,       0,  707996,       0,  0, 16,  0, }, // Maldives
    {  531, 1237,       0,  708012,       0,  0, 17,  0, }, // Marquesas
    {  531, 1247,       0,  708029,       0,  0, 19,  0, }, // Marshall_Islands
    {  531, 1264,  708048,  708060,  708078, 12, 18, 20, }, // Mauritius
    {  531, 1274,       0,  708098,       0,  0, 13,  0, }, // Mawson
    {  531, 1281,  708111,  708134,  708163, 23, 29, 32, }, // Mexico_Pacific
    {  531, 1296,  708195,  708212,  708235, 17, 23, 26, }, // Mongolia
    {  531, 1305,  708261,  708274,  708293, 13, 19, 22, }, // Moscow
    {  531, 1312,       0,  708315,       0,  0, 15,  0, }, // Myanmar
    {  531, 1320,       0,  708330,       0,  0, 12,  0, }, // Nauru
    {  531, 1326,       0,  708342,       0,  0, 11,  0, }, // Nepal
    {  531, 1332,  708353,  708373,  708399, 20, 26, 29, }, // New_Caledonia
    {  531, 1346,  708428,  708442,  708462, 14, 20, 19, }, // New_Zealand
    {  531, 1358,  708481,  708500,  708525, 19, 25, 28, }, // Newfoundland
    {  531, 1371,       0,  708553,       0,  0, 10,  0, }, // Niue
    {  531, 1376,  708563,  708583,  708609, 20, 26, 29, }, // Norfolk
    {  531, 1384,  708638,  708665,  708698, 27, 33, 35, }, // Noronha
    {  531, 1406,  708733,  708752,  708777, 19, 25, 28, }, // Novosibirsk
    {  531, 1418,  708805,  708818,  708837, 13, 19, 22, }, // Omsk
    {  531, 1428,  708859,  708874,  708895, 15, 21, 23, }, // Pakistan
    {  531, 1437,       0,  708918,       0,  0, 11,  0, }, // Palau
    {  531, 1443,       0,  708929,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  531, 1460,  708953,  708966,  708985, 13, 19, 21, }, // Paraguay
    {  531, 1469,  709006,  709016,  709032, 10, 16, 18, }, // Peru
    {  531, 1474,  709050,  709063,  709082, 13, 19, 22, }, // Philippines
    {  531, 1486,       0,  709104,       0,  0, 20,  0, }, // Phoenix_Islands
    {  531, 1502,  709124,  709152,  709186, 28, 34, 37, }, // Pierre_Miquelon
    {  531, 1518,       0,  709223,       0,  0, 16,  0, }, // Pitcairn
    {  531, 1527,       0,  709239,       0,  0, 11,  0, }, // Ponape
    {  531, 1534,       0,  709250,       0,  0, 17,  0, }, // Pyongyang
    {  531, 1554,       0,  709267,       0,  0, 15,  0, }, // Reunion
    {  531, 1562,       0,  709282,       0,  0, 12,  0, }, // Rothera
    {  531, 1570,  709294,  709306,  709324, 12, 18, 21, }, // Sakhalin
    {  531, 1596,  709345,  709357,  709375, 12, 18, 21, }, // Samoa
    {  531, 1602,       0,  709396,       0,  0, 14,  0, }, // Seychelles
    {  531, 1624,       0,  709410,       0,  0, 16,  0, }, // Singapore
    {  531, 1634,       0,  709426,       0,  0, 19,  0, }, // Solomon
    {  531, 1642,       0,  709445,       0,  0, 18,  0, }, // South_Georgia
    {  531, 1656,       0,  709463,       0,  0, 13,  0, }, // Suriname
    {  531, 1676,       0,  709476,       0,  0, 12,  0, }, // Syowa
    {  531, 1682,       0,  709488,       0,  0, 12,  0, }, // Tahiti
    {  531, 1689,  709500,  709512,  709530, 12, 18, 18, }, // Taipei
    {  531, 1696,       0,  709548,       0,  0, 16,  0, }, // Tajikistan
    {  531, 1724,       0,  709564,       0,  0, 14,  0, }, // Tokelau
    {  531, 1732,  709578,  709589,  709606, 11, 17, 20, }, // Tonga
    {  531, 1738,       0,  709626,       0,  0, 10,  0, }, // Truk
    {  531, 1750,  709636,  709656,  709682, 20, 26, 29, }, // Turkmenistan
    {  531, 1763,       0,  709711,       0,  0, 12,  0, }, // Tuvalu
    {  531, 1777,  709723,  709736,  709755, 13, 19, 21, }, // Uruguay
    {  531, 1792,  709776,  709794,  709818, 18, 24, 27, }, // Uzbekistan
    {  531, 1803,  709845,  709857,  709875, 12, 18, 20, }, // Vanuatu
    {  531, 1811,       0,  709895,       0,  0, 16,  0, }, // Venezuela
    {  531, 1821,  709911,  709930,  709955, 19, 25, 28, }, // Vladivostok
    {  531, 1833,  709983,  710002,  710027, 19, 25, 28, }, // Volgograd
    {  531, 1843,       0,  710055,       0,  0, 14,  0, }, // Vostok
    {  531, 1850,       0,  710069,       0,  0, 16,  0, }, // Wake
    {  531, 1855,       0,  710085,       0,  0, 22,  0, }, // Wallis
    {  531, 1862,  710107,  710122,  710143, 15, 21, 24, }, // Yakutsk
    {  531, 1870,  710167,  710187,  710213, 20, 26, 29, }, // Yekaterinburg
    {  531, 1892,       0,  710242,       0,  0, 12,  0, }, // Yukon
    {  533,    0,  193757,  710254,  710277, 12, 23, 18, }, // Acre Slovak/Latin/Slovakia
    {  533,    5,       0,  710295,       0,  0, 12,  0, }, // Afghanistan
    {  533,   17,       0,  710307,       0,  0, 17,  0, }, // Africa_Central
    {  533,   32,       0,  193840,       0,  0, 18,  0, }, // Africa_Eastern
    {  533,   65,       0,  710324,       0,  0, 15,  0, }, // Africa_Southern
    {  533,   81,  193873,  710339,  710367, 17, 28, 23, }, // Africa_Western
    {  533,  107,  193941,  710390,  710413, 12, 23, 18, }, // Alaska
    {  533,  128,  710431,  710444,  710468, 13, 24, 19, }, // Almaty
    {  533,  135,  194047,  710487,  710511, 13, 24, 19, }, // Amazon
    {  533,  142,  710530,  710558,  710597, 28, 39, 34, }, // America_Central
    {  533,  158,  710631,  710658,  710696, 27, 38, 33, }, // America_Eastern
    {  533,  174,  194302,  710729,  710765, 25, 36, 31, }, // America_Mountain
    {  533,  191,  710796,  710826,  710867, 30, 41, 36, }, // America_Pacific
    {  533,  207,  710903,  710916,  710940, 13, 24, 19, }, // Anadyr
    {  533,  214,  194551,  710959,  710981, 11, 22, 17, }, // Apia
    {  533,  219,  710998,  711010,  711033, 12, 23, 18, }, // Aqtau
    {  533,  225,  711051,  711063,  711086, 12, 23, 18, }, // Aqtobe
    {  533,  232,  194707,  711104,  711126, 11, 22, 17, }, // Arabian
    {  533,  240,  711143,  711158,  711184, 15, 26, 21, }, // Argentina
    {  533,  250,  711205,  711226,  711258, 21, 32, 27, }, // Argentina_Western
    {  533,  268,  711285,  711297,  711320, 12, 23, 18, }, // Armenia
    {  533,  286,  194952,  711338,  711363, 14, 25, 20, }, // Atlantic
    {  533,  295,  711383,  711403,  711434, 20, 31, 26, }, // Australia_Central
    {  533,  313,  711460,  711488,  711527, 28, 39, 34, }, // Australia_CentralWestern
    {  533,  338,  711561,  711582,  711614, 21, 32, 27, }, // Australia_Eastern
    {  533,  356,  711641,  711661,  711692, 20, 31, 26, }, // Australia_Western
    {  533,  374,  711718,  711736,  711765, 18, 29, 24, }, // Azerbaijan
    {  533,  385,  195417,  711789,  711811, 11, 22, 17, }, // Azores
    {  533,  397,  711828,  711844,  711871, 16, 27, 22, }, // Bangladesh
    {  533,  415,       0,  711893,       0,  0, 13,  0, }, // Bhutan
    {  533,  422,       0,  711906,       0,  0, 14,  0, }, // Bolivia
    {  533,  437,  711920,  711933,  711957, 13, 24, 19, }, // Brasilia
    {  533,  454,       0,  195621,       0,  0, 13,  0, }, // Brunei
    {  533,  461,  195634,  711976,  712001, 14, 25, 20, }, // Cape_Verde
    {  533,  472,       0,  195693,       0,  0, 19,  0, }, // Casey
    {  533,  478,       0,  195712,       0,  0, 14,  0, }, // Chamorro
    {  533,  487,  195726,  712021,  712046, 14, 25, 20, }, // Chatham
    {  533,  495,  712066,  712076,  712097, 10, 21, 16, }, // Chile
    {  533,  501,  712113,  712123,  712144, 10, 21, 16, }, // China
    {  533,  507,       0,  712160,       0,  0, 22,  0, }, // Christmas
    {  533,  517,       0,  712182,       0,  0, 23,  0, }, // Cocos
    {  533,  523,  195925,  712205,  712231, 15, 26, 21, }, // Colombia
    {  533,  532,  712252,  712274,  712307, 22, 33, 28, }, // Cook
    {  533,  537,  712335,  712347,  712370, 12, 23, 18, }, // Cuba
    {  533,  548,       0,  712388,       0,  0, 21,  0, }, // Davis
    {  533,  564,       0,  712409,       0,  0, 30,  0, }, // DumontDUrville
    {  533,  601,       0,  196169,       0,  0, 19,  0, }, // East_Timor
    {  533,  612,  712439,  712463,  712498, 24, 35, 30, }, // Easter
    {  533,  619,       0,  196280,       0,  0, 14,  0, }, // Ecuador
    {  533,  627,  712528,  712546,  712575, 18, 29, 24, }, // Europe_Central
    {  533,  642,  712599,  712618,  712648, 19, 30, 25, }, // Europe_Eastern
    {  533,  657,       0,  712673,       0,  0, 10,  0, }, // Europe_Further_Eastern
    {  533,  680,  712683,  712701,  712730, 18, 29, 24, }, // Europe_Western
    {  533,  695,  196534,  712754,  712780, 15, 26, 21, }, // Falkland
    {  533,  704,  196596,  712801,  712825, 13, 24, 19, }, // Fiji
    {  533,  709,       0,  712844,       0,  0, 22,  0, }, // French_Guiana
    {  533,  723,       0,  712866,       0,  0, 46,  0, }, // French_Southern
    {  533,  746,       0,  712912,       0,  0, 14,  0, }, // Galapagos
    {  533,  756,       0,  196736,       0,  0, 14,  0, }, // Gambier
    {  533,  764,  712926,  712939,  712963, 13, 24, 19, }, // Georgia
    {  533,  772,       0,  712982,       0,  0, 25,  0, }, // Gilbert_Islands
    {  533,  788,       0,  713007,       0,  0, 16,  0, }, // GMT
    {  533,  830,  713023,  713041,  713070, 18, 29, 24, }, // Greenland_Eastern
    {  533,  848,  713094,  713111,  713139, 17, 28, 23, }, // Greenland_Western
    {  533,  866,       0,  713162,       0,  0, 11,  0, }, // Guam
    {  533,  871,       0,  713173,       0,  0, 31,  0, }, // Gulf
    {  533,  876,       0,  196663,       0,  0, 12,  0, }, // Guyana
    {  533,  883,  197034,  713204,  713236, 21, 32, 27, }, // Hawaii_Aleutian
    {  533,  899,  713263,  713277,  713302, 14, 25, 20, }, // Hong_Kong
    {  533,  909,  713322,  713334,  713357, 12, 23, 18, }, // Hovd
    {  533,  914,       0,  197226,       0,  0, 11,  0, }, // India
    {  533,  920,       0,  713375,       0,  0, 19,  0, }, // Indian_Ocean
    {  533,  933,       0,  713394,       0,  0, 14,  0, }, // Indochina
    {  533,  943,       0,  713408,       0,  0, 20,  0, }, // Indonesia_Central
    {  533,  961,       0,  713428,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  533,  979,       0,  713449,       0,  0, 20,  0, }, // Indonesia_Western
    {  533,  997,  713469,  713480,  713502, 11, 22, 17, }, // Iran
    {  533, 1008,  197378,  713519,  713542, 12, 23, 18, }, // Irkutsk
    {  533, 1016,  197431,  713560,  713584, 13, 24, 19, }, // Israel
    {  533, 1023,  197487,  713603,  713626, 12, 23, 18, }, // Japan
    {  533, 1029,  713644,  713671,  713709, 27, 38, 37, }, // Kamchatka
    {  533, 1047,       0,  713746,       0,  0, 17,  0, }, // Kazakhstan
    {  533, 1058,       0,  713763,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  533, 1077,       0,  713787,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  533, 1106,  713810,  713822,  713845, 12, 23, 18, }, // Korea
    {  533, 1112,       0,  197758,       0,  0, 13,  0, }, // Kosrae
    {  533, 1119,  197771,  713863,  713890, 16, 27, 22, }, // Krasnoyarsk
    {  533, 1151,       0,  713912,       0,  0, 13,  0, }, // Kyrgystan
    {  533, 1161,       0,  713925,       0,  0, 13,  0, }, // Lanka
    {  533, 1175,       0,  713938,       0,  0, 24,  0, }, // Line_Islands
    {  533, 1188,  197885,  713962,  713995, 22, 33, 28, }, // Lord_Howe
    {  533, 1198,  714023,  714035,  714058, 12, 23, 18, }, // Macau
    {  533, 1204,  198021,  714076,  714101, 14, 25, 20, }, // Magadan
    {  533, 1219,       0,  714121,       0,  0, 15,  0, }, // Malaysia
    {  533, 1228,       0,  714136,       0,  0, 13,  0, }, // Maldives
    {  533, 1237,       0,  714149,       0,  0, 13,  0, }, // Marquesas
    {  533, 1247,       0,  714162,       0,  0, 26,  0, }, // Marshall_Islands
    {  533, 1264,  714188,  714203,  714229, 15, 26, 21, }, // Mauritius
    {  533, 1274,       0,  714250,       0,  0, 22,  0, }, // Mawson
    {  533, 1281,  714272,  714295,  714329, 23, 34, 29, }, // Mexico_Pacific
    {  533, 1296,  714358,  714374,  714401, 16, 27, 22, }, // Mongolia
    {  533, 1305,  714423,  714436,  714460, 13, 24, 19, }, // Moscow
    {  533, 1312,       0,  714479,       0,  0, 14,  0, }, // Myanmar
    {  533, 1320,       0,  714493,       0,  0, 12,  0, }, // Nauru
    {  533, 1326,       0,  714505,       0,  0, 12,  0, }, // Nepal
    {  533, 1332,  714517,  714535,  714564, 18, 29, 24, }, // New_Caledonia
    {  533, 1346,  198536,  714588,  714616, 17, 28, 23, }, // New_Zealand
    {  533, 1358,  198604,  714639,  714669, 19, 30, 25, }, // Newfoundland
    {  533, 1371,       0,  198678,       0,  0, 12,  0, }, // Niue
    {  533, 1376,  714694,  714707,  714731, 13, 24, 19, }, // Norfolk
    {  533, 1384,  714750,  714784,  714829, 34, 45, 40, }, // Noronha
    {  533, 1392,       0,  714869,       0,  0, 19,  0, }, // North_Mariana
    {  533, 1406,  198887,  714888,  714915, 16, 27, 22, }, // Novosibirsk
    {  533, 1418,  198952,  714937,  714957,  9, 20, 15, }, // Omsk
    {  533, 1428,  714972,  714987,  715013, 15, 26, 21, }, // Pakistan
    {  533, 1437,       0,  199058,       0,  0, 12,  0, }, // Palau
    {  533, 1443,       0,  715034,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  533, 1460,  715056,  715071,  715097, 15, 26, 21, }, // Paraguay
    {  533, 1469,  715118,  715131,  715155, 13, 24, 19, }, // Peru
    {  533, 1474,  715174,  715188,  715213, 14, 25, 20, }, // Philippines
    {  533, 1486,       0,  715233,       0,  0, 23,  0, }, // Phoenix_Islands
    {  533, 1502,  199290,  715256,  715289, 22, 33, 28, }, // Pierre_Miquelon
    {  533, 1518,       0,  715317,       0,  0, 26,  0, }, // Pitcairn
    {  533, 1527,       0,  199398,       0,  0, 12,  0, }, // Ponape
    {  533, 1534,       0,  715343,       0,  0, 17,  0, }, // Pyongyang
    {  533, 1544,  715360,  715375,  715401, 15, 26, 21, }, // Qyzylorda
    {  533, 1554,       0,  199490,       0,  0, 14,  0, }, // Reunion
    {  533, 1562,       0,  715422,       0,  0, 22,  0, }, // Rothera
    {  533, 1570,  199525,  715444,  715470, 15, 26, 21, }, // Sakhalin
    {  533, 1579,  715491,  715503,  715526, 12, 23, 18, }, // Samara
    {  533, 1596,  199640,  715544,  715567, 12, 23, 18, }, // Samoa
    {  533, 1602,       0,  199693,       0,  0, 14,  0, }, // Seychelles
    {  533, 1624,       0,  715585,       0,  0, 26,  0, }, // Singapore
    {  533, 1634,       0,  715611,       0,  0, 25,  0, }, // Solomon
    {  533, 1642,       0,  715636,       0,  0, 18,  0, }, // South_Georgia
    {  533, 1656,       0,  199764,       0,  0, 14,  0, }, // Suriname
    {  533, 1676,       0,  199778,       0,  0, 16,  0, }, // Syowa
    {  533, 1682,       0,  199794,       0,  0, 12,  0, }, // Tahiti
    {  533, 1689,  199806,  715654,  715680, 15, 26, 21, }, // Taipei
    {  533, 1696,       0,  715701,       0,  0, 12,  0, }, // Tajikistan
    {  533, 1724,       0,  199880,       0,  0, 14,  0, }, // Tokelau
    {  533, 1732,  199894,  715713,  715735, 11, 22, 17, }, // Tonga
    {  533, 1738,       0,  199944,       0,  0, 12,  0, }, // Truk
    {  533, 1750,  715752,  715766,  715791, 14, 25, 20, }, // Turkmenistan
    {  533, 1763,       0,  200015,       0,  0, 12,  0, }, // Tuvalu
    {  533, 1777,  715811,  715825,  715850, 14, 25, 20, }, // Uruguay
    {  533, 1792,  200086,  715870,  715892, 11, 22, 17, }, // Uzbekistan
    {  533, 1803,  200136,  715909,  715933, 13, 24, 19, }, // Vanuatu
    {  533, 1811,       0,  200192,       0,  0, 15,  0, }, // Venezuela
    {  533, 1821,  200207,  715952,  715980, 17, 28, 23, }, // Vladivostok
    {  533, 1833,  200275,  716003,  716030, 16, 27, 22, }, // Volgograd
    {  533, 1843,       0,  200340,       0,  0, 18,  0, }, // Vostok
    {  533, 1850,       0,  200358,       0,  0, 16,  0, }, // Wake
    {  533, 1855,       0,  716052,       0,  0, 28,  0, }, // Wallis
    {  533, 1862,  200401,  716080,  716103, 12, 23, 18, }, // Yakutsk
    {  533, 1870,  716121,  716140,  716170, 19, 30, 25, }, // Yekaterinburg
    {  533, 1892,       0,  200528,       0,  0, 12,  0, }, // Yukon
    {  534,    5,       0,  716195,       0,  0, 17,  0, }, // Afghanistan Slovenian/Latin/Slovenia
    {  534,   17,       0,  716212,       0,  0, 20,  0, }, // Africa_Central
    {  534,   32,       0,  716232,       0,  0, 18,  0, }, // Africa_Eastern
    {  534,   65,       0,  716250,       0,  0, 16,  0, }, // Africa_Southern
    {  534,   81,  716266,  716284,  716313, 18, 29, 26, }, // Africa_Western
    {  534,  107,  716339,  716350,  716372, 11, 22, 19, }, // Alaska
    {  534,  135,  716391,  716404,  716428, 13, 24, 21, }, // Amazon
    {  534,  142,  716449,  716462,  716486, 13, 24, 21, }, // America_Central
    {  534,  158,  716507,  716518,  716540, 11, 22, 19, }, // America_Eastern
    {  534,  174,  716559,  716569,  716590, 10, 21, 18, }, // America_Mountain
    {  534,  191,  716608,  716621,  716645, 13, 24, 21, }, // America_Pacific
    {  534,  207,  716666,  716679,  716703, 13, 24, 21, }, // Anadyr
    {  534,  214,  716724,  716733,  716753,  9, 20, 17, }, // Apia
    {  534,  232,  716770,  716781,  716803, 11, 22, 19, }, // Arabian
    {  534,  240,  716822,  716837,  716863, 15, 26, 23, }, // Argentina
    {  534,  250,  716886,  716909,  716943, 23, 34, 31, }, // Argentina_Western
    {  534,  268,  716974,  716986,  717009, 12, 23, 20, }, // Armenia
    {  534,  286,  717029,  717042,  717066, 13, 24, 21, }, // Atlantic
    {  534,  295,  717087,  717111,  717146, 24, 35, 32, }, // Australia_Central
    {  534,  313,  717178,  717210,  717253, 32, 43, 40, }, // Australia_CentralWestern
    {  534,  338,  717293,  717315,  717348, 22, 33, 30, }, // Australia_Eastern
    {  534,  356,  717378,  717400,  717433, 22, 33, 30, }, // Australia_Western
    {  534,  374,  717463,  717481,  717510, 18, 29, 26, }, // Azerbaijan
    {  534,  385,  717536,  717547,  717569, 11, 22, 19, }, // Azores
    {  534,  397,  717588,  717603,  717629, 15, 26, 23, }, // Bangladesh
    {  534,  415,       0,  717652,       0,  0, 12,  0, }, // Bhutan
    {  534,  422,       0,  717664,       0,  0, 14,  0, }, // Bolivia
    {  534,  437,  717678,  717691,  717715, 13, 24, 21, }, // Brasilia
    {  534,  454,       0,  717736,       0,  0, 13,  0, }, // Brunei
    {  534,  461,  717749,  717763,  717788, 14, 25, 22, }, // Cape_Verde
    {  534,  478,       0,  717810,       0,  0, 23,  0, }, // Chamorro
    {  534,  487,  717833,  717845,  717868, 12, 23, 20, }, // Chatham
    {  534,  495,  717888,  717898,  717919, 10, 21, 18, }, // Chile
    {  534,  501,  717937,  717949,  717972, 12, 23, 20, }, // China
    {  534,  507,       0,  717992,       0,  0, 17,  0, }, // Christmas
    {  534,  517,       0,  718009,       0,  0, 19,  0, }, // Cocos
    {  534,  523,  718028,  718043,  718069, 15, 26, 23, }, // Colombia
    {  534,  532,  718092,  718109,  718137, 17, 28, 32, }, // Cook
    {  534,  537,  718169,  718181,  718204, 12, 23, 20, }, // Cuba
    {  534,  548,       0,  718224,       0,  0, 10,  0, }, // Davis
    {  534,  564,       0,  718234,       0,  0, 21,  0, }, // DumontDUrville
    {  534,  601,       0,  718255,       0,  0, 19,  0, }, // East_Timor
    {  534,  612,  718274,  718295,  718327, 21, 32, 29, }, // Easter
    {  534,  619,       0,  718356,       0,  0, 14,  0, }, // Ecuador
    {  534,  627,  718370,  718389,  718419, 19, 30, 27, }, // Europe_Central
    {  534,  642,  718446,  718465,  718495, 19, 30, 27, }, // Europe_Eastern
    {  534,  657,       0,  718522,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  534,  680,  718549,  718568,  718598, 19, 30, 27, }, // Europe_Western
    {  534,  695,  718625,  718648,  718682, 23, 34, 31, }, // Falkland
    {  534,  704,  718713,  718726,  718750, 13, 24, 21, }, // Fiji
    {  534,  709,       0,  718771,       0,  0, 22,  0, }, // French_Guiana
    {  534,  723,       0,  718793,       0,  0, 34,  0, }, // French_Southern
    {  534,  746,       0,  718827,       0,  0, 13,  0, }, // Galapagos
    {  534,  756,       0,  718840,       0,  0, 14,  0, }, // Gambier
    {  534,  764,  718854,  718867,  718891, 13, 24, 21, }, // Georgia
    {  534,  772,       0,  718912,       0,  0, 21,  0, }, // Gilbert_Islands
    {  534,  788,       0,  718933,       0,  0, 22,  0, }, // GMT
    {  534,  830,  718955,  718977,  719010, 22, 33, 30, }, // Greenland_Eastern
    {  534,  848,  719040,  719062,  719095, 22, 33, 30, }, // Greenland_Western
    {  534,  871,       0,  719125,       0,  0, 23,  0, }, // Gulf
    {  534,  876,       0,  719148,       0,  0, 13,  0, }, // Guyana
    {  534,  883,  719161,  719182,  719214, 21, 32, 29, }, // Hawaii_Aleutian
    {  534,  899,  719243,  719257,  719282, 14, 25, 22, }, // Hong_Kong
    {  534,  909,  719304,  719315,  719337, 11, 22, 19, }, // Hovd
    {  534,  914,       0,  719356,       0,  0, 23,  0, }, // India
    {  534,  920,       0,  719379,       0,  0, 20,  0, }, // Indian_Ocean
    {  534,  933,       0,  719399,       0,  0, 16,  0, }, // Indochina
    {  534,  943,       0,  719415,       0,  0, 25,  0, }, // Indonesia_Central
    {  534,  961,       0,  719440,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  534,  979,       0,  719464,       0,  0, 24,  0, }, // Indonesia_Western
    {  534,  997,  719488,  719499,  719521, 11, 22, 19, }, // Iran
    {  534, 1008,  719540,  719552,  719575, 12, 23, 20, }, // Irkutsk
    {  534, 1016,  719595,  719608,  719632, 13, 24, 21, }, // Israel
    {  534, 1023,  719653,  719665,  719688, 12, 23, 20, }, // Japan
    {  534, 1029,  719708,  719735,  719773, 27, 38, 35, }, // Kamchatka
    {  534, 1047,       0,  719808,       0,  0, 16,  0, }, // Kazakhstan
    {  534, 1058,       0,  719824,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  534, 1077,       0,  719848,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  534, 1106,  719872,  719884,  719907, 12, 23, 20, }, // Korea
    {  534, 1112,       0,  719927,       0,  0, 13,  0, }, // Kosrae
    {  534, 1119,  719940,  719956,  719983, 16, 27, 24, }, // Krasnoyarsk
    {  534, 1151,       0,  720007,       0,  0, 18,  0, }, // Kyrgystan
    {  534, 1175,       0,  720025,       0,  0, 21,  0, }, // Line_Islands
    {  534, 1188,  720046,  720065,  720095, 19, 30, 27, }, // Lord_Howe
    {  534, 1204,  720122,  720136,  720161, 14, 25, 22, }, // Magadan
    {  534, 1219,       0,  720183,       0,  0, 14,  0, }, // Malaysia
    {  534, 1228,       0,  720197,       0,  0, 13,  0, }, // Maldives
    {  534, 1237,       0,  720210,       0,  0, 19,  0, }, // Marquesas
    {  534, 1247,       0,  720229,       0,  0, 22,  0, }, // Marshall_Islands
    {  534, 1264,  720251,  720266,  720292, 15, 26, 23, }, // Mauritius
    {  534, 1274,       0,  720315,       0,  0, 13,  0, }, // Mawson
    {  534, 1281,  720328,  720349,  720381, 21, 32, 29, }, // Mexico_Pacific
    {  534, 1296,  720410,  720426,  720453, 16, 27, 24, }, // Mongolia
    {  534, 1305,  720477,  720490,  720514, 13, 24, 21, }, // Moscow
    {  534, 1312,       0,  720535,       0,  0, 14,  0, }, // Myanmar
    {  534, 1320,       0,  720549,       0,  0, 13,  0, }, // Nauru
    {  534, 1326,       0,  720562,       0,  0, 12,  0, }, // Nepal
    {  534, 1332,  720574,  720594,  720625, 20, 31, 28, }, // New_Caledonia
    {  534, 1346,  720653,  720670,  720698, 17, 28, 25, }, // New_Zealand
    {  534, 1358,  720723,  720742,  720772, 19, 30, 27, }, // Newfoundland
    {  534, 1371,       0,  720799,       0,  0, 12,  0, }, // Niue
    {  534, 1376,  720811,  720831,  720862, 20, 31, 28, }, // Norfolk
    {  534, 1384,  720890,  720914,  720949, 24, 35, 32, }, // Noronha
    {  534, 1406,  720981,  720997,  721024, 16, 27, 24, }, // Novosibirsk
    {  534, 1418,  721048,  721057,  721077,  9, 20, 17, }, // Omsk
    {  534, 1428,  721094,  721109,  721135, 15, 26, 23, }, // Pakistan
    {  534, 1437,       0,  721158,       0,  0, 12,  0, }, // Palau
    {  534, 1443,       0,  721170,       0,  0, 13,  0, }, // Papua_New_Guinea
    {  534, 1460,  721183,  721198,  721224, 15, 26, 23, }, // Paraguay
    {  534, 1469,  721247,  721259,  721282, 12, 23, 20, }, // Peru
    {  534, 1474,  721302,  721316,  721341, 14, 25, 22, }, // Philippines
    {  534, 1486,       0,  721363,       0,  0, 18,  0, }, // Phoenix_Islands
    {  534, 1502,  721381,  721410,  721450, 29, 40, 37, }, // Pierre_Miquelon
    {  534, 1518,       0,  721487,       0,  0, 15,  0, }, // Pitcairn
    {  534, 1527,       0,  721502,       0,  0, 12,  0, }, // Ponape
    {  534, 1534,       0,  721514,       0,  0, 15,  0, }, // Pyongyang
    {  534, 1554,       0,  721529,       0,  0, 14,  0, }, // Reunion
    {  534, 1562,       0,  721543,       0,  0, 13,  0, }, // Rothera
    {  534, 1570,  721556,  721570,  721595, 14, 25, 22, }, // Sakhalin
    {  534, 1579,  721617,  721629,  721652, 12, 23, 20, }, // Samara
    {  534, 1596,  721672,  721685,  721709, 13, 24, 21, }, // Samoa
    {  534, 1602,       0,  721730,       0,  0, 13,  0, }, // Seychelles
    {  534, 1624,       0,  721743,       0,  0, 26,  0, }, // Singapore
    {  534, 1634,       0,  721769,       0,  0, 20,  0, }, // Solomon
    {  534, 1642,       0,  721789,       0,  0, 19,  0, }, // South_Georgia
    {  534, 1656,       0,  721808,       0,  0, 14,  0, }, // Suriname
    {  534, 1676,       0,  721822,       0,  0, 10,  0, }, // Syowa
    {  534, 1682,       0,  721832,       0,  0, 14,  0, }, // Tahiti
    {  534, 1689,  721846,  721859,  721883, 13, 24, 21, }, // Taipei
    {  534, 1696,       0,  721904,       0,  0, 18,  0, }, // Tajikistan
    {  534, 1724,       0,  721922,       0,  0, 14,  0, }, // Tokelau
    {  534, 1732,  721936,  721949,  721973, 13, 24, 21, }, // Tonga
    {  534, 1738,       0,  721994,       0,  0, 15,  0, }, // Truk
    {  534, 1750,  722009,  722028,  722058, 19, 30, 27, }, // Turkmenistan
    {  534, 1763,       0,  722085,       0,  0, 14,  0, }, // Tuvalu
    {  534, 1777,  722099,  722113,  722138, 14, 25, 22, }, // Uruguay
    {  534, 1792,  722160,  722177,  722205, 17, 28, 25, }, // Uzbekistan
    {  534, 1803,  722230,  722245,  722271, 15, 26, 23, }, // Vanuatu
    {  534, 1811,       0,  722294,       0,  0, 15,  0, }, // Venezuela
    {  534, 1821,  722309,  722326,  722354, 17, 28, 25, }, // Vladivostok
    {  534, 1833,  722379,  722395,  722422, 16, 27, 24, }, // Volgograd
    {  534, 1843,       0,  722446,       0,  0, 12,  0, }, // Vostok
    {  534, 1850,       0,  722458,       0,  0, 14,  0, }, // Wake
    {  534, 1855,       0,  722472,       0,  0, 21,  0, }, // Wallis
    {  534, 1862,  722493,  722505,  722528, 12, 23, 20, }, // Yakutsk
    {  534, 1870,  722548,  722567,  722597, 19, 30, 27, }, // Yekaterinburg
    {  534, 1892,       0,  722624,       0,  0, 12,  0, }, // Yukon
    {  536,    0,  722636,  722649,  722676, 13, 27, 22, }, // Acre Somali/Latin/Somalia
    {  536,    5,       0,  722698,       0,  0, 21,  0, }, // Afghanistan
    {  536,   17,       0,  722719,       0,  0, 24,  0, }, // Africa_Central
    {  536,   32,       0,  722743,       0,  0, 21,  0, }, // Africa_Eastern
    {  536,   65,       0,  722764,       0,  0, 38,  0, }, // Africa_Southern
    {  536,   81,  722802,  722826,  722864, 24, 38, 35, }, // Africa_Western
    {  536,  107,  722899,  722913,  722941, 14, 28, 27, }, // Alaska
    {  536,  128,  722968,  722982,  723010, 14, 28, 35, }, // Almaty
    {  536,  135,  723045,  723059,  723087, 14, 28, 25, }, // Amazon
    {  536,  142,  723112,  723148,  723198, 36, 50, 49, }, // America_Central
    {  536,  158,  723247,  723283,  723330, 36, 47, 46, }, // America_Eastern
    {  536,  174,  723376,  723416,  723467, 40, 51, 49, }, // America_Mountain
    {  536,  191,  723516,  723554,  723603, 38, 49, 48, }, // America_Pacific
    {  536,  207,  723651,  723666,  723695, 15, 29, 24, }, // Anadyr
    {  536,  214,  723719,  723732,  723759, 13, 27, 26, }, // Apia
    {  536,  219,  723785,  723798,  723825, 13, 27, 31, }, // Aqtau
    {  536,  225,  723856,  723870,  723898, 14, 28, 32, }, // Aqtobe
    {  536,  232,  723930,  723945,  723974, 15, 29, 28, }, // Arabian
    {  536,  240,  724002,  724019,  724051, 17, 32, 29, }, // Argentina
    {  536,  250,  724080,  724108,  724150, 28, 42, 39, }, // Argentina_Western
    {  536,  268,  724189,  724206,  724237, 17, 31, 28, }, // Armenia
    {  536,  286,  724265,  724304,  724354, 39, 50, 49, }, // Atlantic
    {  536,  295,  724403,  724432,  724475, 29, 43, 42, }, // Australia_Central
    {  536,  313,  724517,  724556,  724609, 39, 53, 48, }, // Australia_CentralWestern
    {  536,  338,  724657,  724683,  724725, 26, 42, 39, }, // Australia_Eastern
    {  536,  356,  724764,  724793,  724836, 29, 43, 42, }, // Australia_Western
    {  536,  374,  724878,  724896,  724928, 18, 32, 29, }, // Azerbaijan
    {  536,  385,  724957,  724971,  724999, 14, 28, 25, }, // Azores
    {  536,  397,  725024,  725043,  725076, 19, 33, 30, }, // Bangladesh
    {  536,  415,       0,  725106,       0,  0, 14,  0, }, // Bhutan
    {  536,  422,       0,  725120,       0,  0, 17,  0, }, // Bolivia
    {  536,  437,  725137,  725156,  725189, 19, 33, 30, }, // Brasilia
    {  536,  454,       0,  725219,       0,  0, 27,  0, }, // Brunei
    {  536,  461,  725246,  725265,  725298, 19, 33, 30, }, // Cape_Verde
    {  536,  478,       0,  725328,       0,  0, 14,  0, }, // Chamorro
    {  536,  487,  725342,  725356,  725384, 14, 28, 27, }, // Chatham
    {  536,  495,  725411,  725423,  725449, 12, 26, 23, }, // Chile
    {  536,  501,  725472,  725488,  725518, 16, 30, 29, }, // China
    {  536,  507,       0,  725547,       0,  0, 23,  0, }, // Christmas
    {  536,  517,       0,  725570,       0,  0, 20,  0, }, // Cocos
    {  536,  523,  725590,  725607,  725638, 17, 31, 28, }, // Colombia
    {  536,  532,  725666,  725687,  725722, 21, 35, 39, }, // Cook
    {  536,  537,  725761,  725774,  725801, 13, 27, 26, }, // Cuba
    {  536,  548,       0,  725827,       0,  0, 13,  0, }, // Davis
    {  536,  564,       0,  725840,       0,  0, 26,  0, }, // DumontDUrville
    {  536,  601,       0,  725866,       0,  0, 19,  0, }, // East_Timor
    {  536,  612,  725885,  725906,  725941, 21, 35, 32, }, // Easter
    {  536,  619,       0,  725973,       0,  0, 17,  0, }, // Ecuador
    {  536,  627,  725990,  726013,  726050, 23, 37, 34, }, // Europe_Central
    {  536,  642,  726084,  726104,  726138, 20, 34, 31, }, // Europe_Eastern
    {  536,  657,       0,  726169,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  536,  680,  726196,  726219,  726256, 23, 37, 34, }, // Europe_Western
    {  536,  695,  726290,  726316,  726356, 26, 40, 37, }, // Falkland
    {  536,  704,  726393,  726405,  726431, 12, 26, 23, }, // Fiji
    {  536,  709,       0,  726454,       0,  0, 21,  0, }, // French_Guiana
    {  536,  723,       0,  726475,       0,  0, 42,  0, }, // French_Southern
    {  536,  746,       0,  726517,       0,  0, 17,  0, }, // Galapagos
    {  536,  756,       0,  726534,       0,  0, 16,  0, }, // Gambier
    {  536,  764,  726550,  726566,  726596, 16, 30, 27, }, // Georgia
    {  536,  772,       0,  726623,       0,  0, 23,  0, }, // Gilbert_Islands
    {  536,  788,       0,  726646,       0,  0, 18,  0, }, // GMT
    {  536,  830,  726664,  726692,  726731, 28, 39, 36, }, // Greenland_Eastern
    {  536,  848,  726767,  726795,  726837, 28, 42, 39, }, // Greenland_Western
    {  536,  871,       0,  726876,       0,  0, 15,  0, }, // Gulf
    {  536,  876,       0,  726891,       0,  0, 15,  0, }, // Guyana
    {  536,  883,  726906,  726930,  726968, 24, 38, 37, }, // Hawaii_Aleutian
    {  536,  899,  727005,  727024,  727057, 19, 33, 30, }, // Hong_Kong
    {  536,  909,  727087,  727100,  727127, 13, 27, 24, }, // Hovd
    {  536,  914,       0,  727151,       0,  0, 29,  0, }, // India
    {  536,  920,       0,  727180,       0,  0, 25,  0, }, // Indian_Ocean
    {  536,  933,       0,  727205,       0,  0, 18,  0, }, // Indochina
    {  536,  943,       0,  727223,       0,  0, 30,  0, }, // Indonesia_Central
    {  536,  961,       0,  727253,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  536,  979,       0,  727273,       0,  0, 30,  0, }, // Indonesia_Western
    {  536,  997,  727303,  727317,  727345, 14, 28, 27, }, // Iran
    {  536, 1008,  727372,  727388,  727418, 16, 30, 27, }, // Irkutsk
    {  536, 1016,  727445,  727461,  727491, 16, 30, 29, }, // Israel
    {  536, 1023,  727520,  727534,  727562, 14, 28, 27, }, // Japan
    {  536, 1029,  727589,  727622,  727669, 33, 47, 42, }, // Kamchatka
    {  536, 1047,       0,  727711,       0,  0, 20,  0, }, // Kazakhstan
    {  536, 1058,       0,  727731,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  536, 1077,       0,  727758,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  536, 1106,  727787,  727802,  727831, 15, 29, 28, }, // Korea
    {  536, 1112,       0,  727859,       0,  0, 14,  0, }, // Kosrae
    {  536, 1119,  727873,  727894,  727929, 21, 35, 32, }, // Krasnoyarsk
    {  536, 1151,       0,  727961,       0,  0, 20,  0, }, // Kyrgystan
    {  536, 1175,       0,  727981,       0,  0, 19,  0, }, // Line_Islands
    {  536, 1188,  728000,  728015,  728044, 15, 29, 28, }, // Lord_Howe
    {  536, 1204,  728072,  728086,  728115, 14, 29, 26, }, // Magadan
    {  536, 1219,       0,  728141,       0,  0, 18,  0, }, // Malaysia
    {  536, 1228,       0,  728159,       0,  0, 16,  0, }, // Maldives
    {  536, 1237,       0,  728175,       0,  0, 18,  0, }, // Marquesas
    {  536, 1247,       0,  728193,       0,  0, 26,  0, }, // Marshall_Islands
    {  536, 1264,  728219,  728238,  728271, 19, 33, 30, }, // Mauritius
    {  536, 1274,       0,  728301,       0,  0, 14,  0, }, // Mawson
    {  536, 1281,  728315,  728342,  728383, 27, 41, 40, }, // Mexico_Pacific
    {  536, 1296,  728423,  728443,  728477, 20, 34, 31, }, // Mongolia
    {  536, 1305,  728508,  728522,  728550, 14, 28, 25, }, // Moscow
    {  536, 1312,       0,  728575,       0,  0, 17,  0, }, // Myanmar
    {  536, 1320,       0,  728592,       0,  0, 14,  0, }, // Nauru
    {  536, 1326,       0,  728606,       0,  0, 15,  0, }, // Nepal
    {  536, 1332,  728621,  728644,  728682, 23, 38, 35, }, // New_Caledonia
    {  536, 1346,  728717,  728738,  728773, 21, 35, 34, }, // New_Zealand
    {  536, 1358,  728807,  728828,  728863, 21, 35, 34, }, // Newfoundland
    {  536, 1371,       0,  728621,       0,  0, 13,  0, }, // Niue
    {  536, 1376,  728897,  728919,  728955, 22, 36, 34, }, // Norfolk
    {  536, 1384,  728989,  729017,  729060, 28, 43, 40, }, // Noronha
    {  536, 1406,  729100,  729120,  729154, 20, 34, 31, }, // Novosibirsk
    {  536, 1418,  729185,  729197,  729223, 12, 26, 23, }, // Omsk
    {  536, 1428,  729246,  729263,  729294, 17, 31, 28, }, // Pakistan
    {  536, 1437,       0,  729322,       0,  0, 13,  0, }, // Palau
    {  536, 1443,       0,  729335,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  536, 1460,  729361,  729378,  729409, 17, 31, 28, }, // Paraguay
    {  536, 1469,  729437,  729450,  729477, 13, 27, 24, }, // Peru
    {  536, 1474,  729501,  729517,  729547, 16, 30, 27, }, // Philippines
    {  536, 1486,       0,  729574,       0,  0, 25,  0, }, // Phoenix_Islands
    {  536, 1502,  729599,  729628,  729668, 29, 40, 42, }, // Pierre_Miquelon
    {  536, 1518,       0,  729710,       0,  0, 15,  0, }, // Pitcairn
    {  536, 1527,       0,  729725,       0,  0, 14,  0, }, // Ponape
    {  536, 1534,       0,  729739,       0,  0, 18,  0, }, // Pyongyang
    {  536, 1544,  729757,  729774,  729805, 17, 31, 35, }, // Qyzylorda
    {  536, 1554,       0,  729840,       0,  0, 18,  0, }, // Reunion
    {  536, 1562,       0,  729858,       0,  0, 14,  0, }, // Rothera
    {  536, 1570,  729872,  729888,  729918, 16, 30, 27, }, // Sakhalin
    {  536, 1579,  729945,  729960,  729989, 15, 29, 24, }, // Samara
    {  536, 1596,  730013,  730026,  730053, 13, 27, 26, }, // Samoa
    {  536, 1602,       0,  730079,       0,  0, 17,  0, }, // Seychelles
    {  536, 1624,       0,  730096,       0,  0, 17,  0, }, // Singapore
    {  536, 1634,       0,  730113,       0,  0, 24,  0, }, // Solomon
    {  536, 1642,       0,  730137,       0,  0, 21,  0, }, // South_Georgia
    {  536, 1656,       0,  730158,       0,  0, 16,  0, }, // Suriname
    {  536, 1676,       0,  730174,       0,  0, 14,  0, }, // Syowa
    {  536, 1682,       0,  730188,       0,  0, 14,  0, }, // Tahiti
    {  536, 1689,  730202,  730216,  730244, 14, 28, 27, }, // Taipei
    {  536, 1696,       0,  730271,       0,  0, 18,  0, }, // Tajikistan
    {  536, 1724,       0,  730289,       0,  0, 15,  0, }, // Tokelau
    {  536, 1732,  730304,  730317,  730344, 13, 27, 24, }, // Tonga
    {  536, 1738,       0,  730368,       0,  0, 12,  0, }, // Truk
    {  536, 1750,  730380,  730401,  730435, 21, 34, 31, }, // Turkmenistan
    {  536, 1763,       0,  730466,       0,  0, 14,  0, }, // Tuvalu
    {  536, 1777,  730480,  730496,  730526, 16, 30, 27, }, // Uruguay
    {  536, 1792,  730553,  730571,  730603, 18, 32, 29, }, // Uzbekistan
    {  536, 1803,  730632,  730647,  730676, 15, 29, 26, }, // Vanuatu
    {  536, 1811,       0,  730702,       0,  0, 19,  0, }, // Venezuela
    {  536, 1821,  730721,  730741,  730775, 20, 34, 31, }, // Vladivostok
    {  536, 1833,  730806,  730823,  730854, 17, 31, 28, }, // Volgograd
    {  536, 1843,       0,  730882,       0,  0, 14,  0, }, // Vostok
    {  536, 1850,       0,  730896,       0,  0, 21,  0, }, // Wake
    {  536, 1855,       0,  730917,       0,  0, 22,  0, }, // Wallis
    {  536, 1862,  730939,  730954,  730981, 15, 27, 24, }, // Yakutsk
    {  536, 1870,  731005,  731026,  731061, 21, 35, 32, }, // Yekaterinburg
    {  536, 1892,       0,  731093,       0,  0, 13,  0, }, // Yukon
    {  544,  788,       0,    2800,       0,  0, 19,  0, }, // GMT Southern Sotho/Latin/South Africa
    {  547,    0,  731106,  731118,  731139, 12, 21, 22, }, // Acre Spanish/Latin/Spain
    {  547,    5,       0,  289911,       0,  0, 18,  0, }, // Afghanistan
    {  547,   17,       0,  731161,       0,  0, 22,  0, }, // Africa_Central
    {  547,   32,       0,  731183,       0,  0, 23,  0, }, // Africa_Eastern
    {  547,   65,       0,  731206,       0,  0, 17,  0, }, // Africa_Southern
    {  547,   81,  731223,  731248,  731282, 25, 34, 35, }, // Africa_Western
    {  547,  107,  365822,  731317,  731340, 14, 23, 24, }, // Alaska
    {  547,  135,  731364,  731381,  731407, 17, 26, 27, }, // Amazon
    {  547,  142,   42433,  290244,   42506, 12, 21, 22, }, // America_Central
    {  547,  158,   42548,  731434,   42623, 13, 22, 23, }, // America_Eastern
    {  547,  174,  731456,  731484,  731521, 28, 37, 38, }, // America_Mountain
    {  547,  191,  731559,  731576,  731602, 17, 26, 27, }, // America_Pacific
    {  547,  207,  290662,  731629,  731652, 14, 23, 24, }, // Anadyr
    {  547,  214,  290728,  290740,  731676, 12, 21, 25, }, // Apia
    {  547,  219,  731701,  731714,  731736, 13, 22, 23, }, // Aqtau
    {  547,  225,  731759,  731773,  731796, 14, 23, 24, }, // Aqtobe
    {  547,  232,  731820,  731834,  731857, 14, 23, 24, }, // Arabian
    {  547,  240,  366234,  731881,  731907, 17, 26, 27, }, // Argentina
    {  547,  250,  366301,  731934,  731971, 28, 37, 38, }, // Argentina_Western
    {  547,  268,  291001,  291016,  732009, 15, 24, 25, }, // Armenia
    {  547,  286,  732034,  732052,  732079, 18, 27, 28, }, // Atlantic
    {  547,  295,  366500,  732107,  732141, 25, 34, 35, }, // Australia_Central
    {  547,  313,  732176,  732209,  732251, 33, 42, 43, }, // Australia_CentralWestern
    {  547,  338,  366712,  732294,  732329, 26, 35, 36, }, // Australia_Eastern
    {  547,  356,  366806,  732365,  732402, 28, 37, 38, }, // Australia_Western
    {  547,  374,  732440,  732458,  732485, 18, 27, 28, }, // Azerbaijan
    {  547,  385,  732513,  732531,  732558, 18, 27, 28, }, // Azores
    {  547,  397,  732586,  732603,  732629, 17, 26, 27, }, // Bangladesh
    {  547,  415,       0,  291655,       0,  0, 13,  0, }, // Bhutan
    {  547,  422,       0,  291668,       0,  0, 15,  0, }, // Bolivia
    {  547,  437,  291683,  291699,  732656, 16, 25, 26, }, // Brasilia
    {  547,  454,       0,  732682,       0,  0, 14,  0, }, // Brunei
    {  547,  461,  291774,  291792,  732696, 18, 27, 28, }, // Cape_Verde
    {  547,  478,       0,  732724,       0,  0, 25,  0, }, // Chamorro
    {  547,  487,  291868,  291883,  732749, 15, 24, 25, }, // Chatham
    {  547,  495,  291931,  291944,  732774, 13, 22, 23, }, // Chile
    {  547,  501,  367338,  732797,  732819, 13, 22, 23, }, // China
    {  547,  507,       0,  732842,       0,  0, 26,  0, }, // Christmas
    {  547,  517,       0,  732868,       0,  0, 23,  0, }, // Cocos
    {  547,  523,  292087,  292103,  732891, 16, 25, 26, }, // Colombia
    {  547,  532,  732917,  732939,  732970, 22, 31, 38, }, // Cook
    {  547,  537,  292234,  292246,  733008, 12, 21, 22, }, // Cuba
    {  547,  548,       0,  292288,       0,  0, 13,  0, }, // Davis
    {  547,  564,       0,  292301,       0,  0, 24,  0, }, // DumontDUrville
    {  547,  601,       0,  733030,       0,  0, 22,  0, }, // East_Timor
    {  547,  612,  733052,  733077,  733111, 25, 34, 35, }, // Easter
    {  547,  619,       0,  292428,       0,  0, 15,  0, }, // Ecuador
    {  547,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  547,  642,  367796,  733209,  733241, 23, 32, 33, }, // Europe_Eastern
    {  547,  657,       0,  733274,       0,  0, 35,  0, }, // Europe_Further_Eastern
    {  547,  680,  367908,  733309,  733343, 25, 34, 35, }, // Europe_Western
    {  547,  695,  733378,  733404,  733439, 26, 35, 36, }, // Falkland
    {  547,  704,  733475,  733487,  733508, 12, 21, 22, }, // Fiji
    {  547,  709,       0,  733530,       0,  0, 27,  0, }, // French_Guiana
    {  547,  723,       0,  733557,       0,  0, 51,  0, }, // French_Southern
    {  547,  746,       0,  733608,       0,  0, 17,  0, }, // Galapagos
    {  547,  756,       0,  292965,       0,  0, 15,  0, }, // Gambier
    {  547,  764,  368217,  733625,  733649, 15, 24, 25, }, // Georgia
    {  547,  772,       0,  733674,       0,  0, 25,  0, }, // Gilbert_Islands
    {  547,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  547,  830,  368325,  733730,  733767, 28, 37, 38, }, // Greenland_Eastern
    {  547,  848,  368423,  733805,  733844, 30, 39, 40, }, // Greenland_Western
    {  547,  866,       0,   60706,       0,  0, 21,  0, }, // Guam
    {  547,  871,       0,  733884,       0,  0, 23,  0, }, // Gulf
    {  547,  876,       0,  368550,       0,  0, 14,  0, }, // Guyana
    {  547,  883,  733907,  733931,  733964, 24, 33, 34, }, // Hawaii_Aleutian
    {  547,  899,  293426,  293443,  733998, 17, 26, 27, }, // Hong_Kong
    {  547,  909,  293495,  293507,  734025, 12, 21, 22, }, // Hovd
    {  547,  914,       0,  734047,       0,  0, 25,  0, }, // India
    {  547,  920,       0,  734072,       0,  0, 22,  0, }, // Indian_Ocean
    {  547,  933,       0,  293583,       0,  0, 17,  0, }, // Indochina
    {  547,  943,       0,  734094,       0,  0, 25,  0, }, // Indonesia_Central
    {  547,  961,       0,  734119,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  547,  979,       0,  734145,       0,  0, 28,  0, }, // Indonesia_Western
    {  547,  997,  293679,  293691,  734173, 12, 21, 22, }, // Iran
    {  547, 1008,  293733,  293748,  734195, 15, 24, 25, }, // Irkutsk
    {  547, 1016,  293796,  293810,  734220, 14, 23, 24, }, // Israel
    {  547, 1023,  734244,  734257,  734279, 13, 22, 23, }, // Japan
    {  547, 1029,  734302,  734319,  734345, 17, 26, 27, }, // Kamchatka
    {  547, 1047,       0,  734372,       0,  0, 18,  0, }, // Kazakhstan
    {  547, 1058,       0,  734390,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  547, 1077,       0,  734417,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  547, 1106,  294116,  294129,  734446, 13, 22, 23, }, // Korea
    {  547, 1112,       0,  294173,       0,  0, 14,  0, }, // Kosrae
    {  547, 1119,  369160,  734469,  734497, 19, 28, 29, }, // Krasnoyarsk
    {  547, 1151,       0,  734526,       0,  0, 18,  0, }, // Kyrgystan
    {  547, 1161,       0,  734544,       0,  0, 17,  0, }, // Lanka
    {  547, 1175,       0,  734561,       0,  0, 34,  0, }, // Line_Islands
    {  547, 1188,  294304,  294321,  734595, 17, 26, 27, }, // Lord_Howe
    {  547, 1198,  734622,  734635,  734657, 13, 22, 23, }, // Macau
    {  547, 1204,  734680,  734695,  734719, 15, 24, 25, }, // Magadan
    {  547, 1219,       0,  734744,       0,  0, 15,  0, }, // Malaysia
    {  547, 1228,       0,  734759,       0,  0, 16,  0, }, // Maldives
    {  547, 1237,       0,  369404,       0,  0, 17,  0, }, // Marquesas
    {  547, 1247,       0,  734775,       0,  0, 26,  0, }, // Marshall_Islands
    {  547, 1264,  294510,  294526,  734801, 16, 25, 26, }, // Mauritius
    {  547, 1274,       0,  294576,       0,  0, 14,  0, }, // Mawson
    {  547, 1281,  734827,  734854,  734890, 27, 36, 37, }, // Mexico_Pacific
    {  547, 1296,  734927,  734945,  734972, 18, 27, 28, }, // Mongolia
    {  547, 1305,  735000,  735013,  735035, 13, 22, 23, }, // Moscow
    {  547, 1312,       0,  294821,       0,  0, 15,  0, }, // Myanmar
    {  547, 1320,       0,  294836,       0,  0, 13,  0, }, // Nauru
    {  547, 1326,       0,  294849,       0,  0, 13,  0, }, // Nepal
    {  547, 1332,  735058,  735081,  735113, 23, 32, 33, }, // New_Caledonia
    {  547, 1346,  735146,  735167,  735197, 21, 30, 31, }, // New_Zealand
    {  547, 1358,  369871,  735228,  735254, 17, 26, 27, }, // Newfoundland
    {  547, 1371,       0,  295099,       0,  0, 12,  0, }, // Niue
    {  547, 1376,  735281,  735304,  735336, 23, 32, 33, }, // Norfolk
    {  547, 1384,  295189,  295216,  735369, 27, 36, 37, }, // Noronha
    {  547, 1392,       0,  735406,       0,  0, 36,  0, }, // North_Mariana
    {  547, 1406,  295288,  295307,  735442, 19, 28, 29, }, // Novosibirsk
    {  547, 1418,  295363,  295375,  735471, 12, 21, 22, }, // Omsk
    {  547, 1428,  735493,  735509,  735534, 16, 25, 26, }, // Pakistan
    {  547, 1437,       0,  735560,       0,  0, 14,  0, }, // Palau
    {  547, 1443,       0,  735574,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  547, 1460,  370270,  735600,  735625, 16, 25, 26, }, // Paraguay
    {  547, 1469,  735651,  735663,  735684, 12, 21, 22, }, // Peru
    {  547, 1474,  295644,  295661,  735706, 17, 26, 27, }, // Philippines
    {  547, 1486,       0,  735733,       0,  0, 23,  0, }, // Phoenix_Islands
    {  547, 1502,  735756,  735784,  735821, 28, 37, 38, }, // Pierre_Miquelon
    {  547, 1518,       0,  295847,       0,  0, 16,  0, }, // Pitcairn
    {  547, 1527,       0,  295863,       0,  0, 15,  0, }, // Ponape
    {  547, 1534,       0,   63402,       0,  0, 17,  0, }, // Pyongyang
    {  547, 1544,   63419,   63436,  735859, 17, 26, 27, }, // Qyzylorda
    {  547, 1554,       0,  295878,       0,  0, 15,  0, }, // Reunion
    {  547, 1562,       0,  295893,       0,  0, 15,  0, }, // Rothera
    {  547, 1570,  735886,  735901,  735925, 15, 24, 25, }, // Sakhalin
    {  547, 1579,  295974,  735950,  735973, 14, 23, 24, }, // Samara
    {  547, 1596,  296040,  296053,  735997, 13, 22, 23, }, // Samoa
    {  547, 1602,       0,  736020,       0,  0, 18,  0, }, // Seychelles
    {  547, 1624,       0,  296116,       0,  0, 16,  0, }, // Singapore
    {  547, 1634,       0,  736038,       0,  0, 25,  0, }, // Solomon
    {  547, 1642,       0,  736063,       0,  0, 23,  0, }, // South_Georgia
    {  547, 1656,       0,  296176,       0,  0, 15,  0, }, // Suriname
    {  547, 1676,       0,  296192,       0,  0, 13,  0, }, // Syowa
    {  547, 1682,       0,  296205,       0,  0, 14,  0, }, // Tahiti
    {  547, 1689,  736086,  736100,  736123, 14, 23, 24, }, // Taipei
    {  547, 1696,       0,  736147,       0,  0, 18,  0, }, // Tajikistan
    {  547, 1724,       0,  296298,       0,  0, 15,  0, }, // Tokelau
    {  547, 1732,  296313,  296326,  736165, 13, 22, 23, }, // Tonga
    {  547, 1738,       0,  296370,       0,  0, 13,  0, }, // Truk
    {  547, 1750,  296383,  296403,  736188, 20, 29, 30, }, // Turkmenistan
    {  547, 1763,       0,  296461,       0,  0, 14,  0, }, // Tuvalu
    {  547, 1777,  371012,  736218,  736242, 15, 24, 25, }, // Uruguay
    {  547, 1792,  296538,  296556,  736267, 18, 27, 28, }, // Uzbekistan
    {  547, 1803,  296610,  296625,  736295, 15, 24, 25, }, // Vanuatu
    {  547, 1811,       0,  296673,       0,  0, 17,  0, }, // Venezuela
    {  547, 1821,  296690,  296709,  736320, 19, 28, 29, }, // Vladivostok
    {  547, 1833,  296765,  296783,  736349, 18, 27, 28, }, // Volgograd
    {  547, 1843,       0,  296837,       0,  0, 14,  0, }, // Vostok
    {  547, 1850,       0,  736377,       0,  0, 20,  0, }, // Wake
    {  547, 1855,       0,  736397,       0,  0, 23,  0, }, // Wallis
    {  547, 1862,  371309,  736420,  736444, 15, 24, 25, }, // Yakutsk
    {  547, 1870,  296954,  296975,  736469, 21, 30, 31, }, // Yekaterinburg
    {  547, 1892,       0,  736500,       0,  0, 13,  0, }, // Yukon
    {  548,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Argentina
    {  548,  240,  366234,  731881,  731907, 17, 26, 27, }, // Argentina
    {  548,  250,  366301,  731934,  731971, 28, 37, 38, }, // Argentina_Western
    {  548,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  548,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  548,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  548,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  548,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  548,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  548,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  548,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  548,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  548,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  548,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  548,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  548, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  548, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  548, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  548, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  548, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  548, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  549,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Belize
    {  549,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  549,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  549,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  549,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  549,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  549,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  549,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  549,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  549,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  549,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  549,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  549,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  549, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  549, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  549, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  549, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  549, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  549, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  550,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Bolivia
    {  550,  422,       0,  291668,       0,  0, 15,  0, }, // Bolivia
    {  550,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  550,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  550,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  550,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  550,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  550,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  550,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  550,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  550,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  550,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  550,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  550,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  550, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  550, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  550, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  550, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  550, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  550, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  551,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Brazil
    {  551,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  551,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  551,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  551,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  551,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  551,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  551,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  551,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  551,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  551,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  551,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  551,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  551, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  551, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  551, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  551, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  551, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  551, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  554,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Chile
    {  554,  495,  291931,  291944,  732774, 13, 22, 23, }, // Chile
    {  554,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  554,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  554,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  554,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  554,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  554,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  554,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  554,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  554,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  554,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  554,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  554,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  554, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  554, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  554, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  554, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  554, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  554, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  555,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Colombia
    {  555,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  555,  523,  292087,  292103,  732891, 16, 25, 26, }, // Colombia
    {  555,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  555,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  555,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  555,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  555,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  555,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  555,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  555,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  555,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  555,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  555,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  555, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  555, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  555, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  555, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  555, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  555, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  556,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Costa Rica
    {  556,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  556,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  556,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  556,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  556,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  556,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  556,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  556,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  556,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  556,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  556,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  556,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  556, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  556, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  556, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  556, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  556, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  556, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  557,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Cuba
    {  557,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  557,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  557,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  557,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  557,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  557,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  557,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  557,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  557,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  557,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  557,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  557,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  557, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  557, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  557, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  557, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  557, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  557, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  558,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Dominican Republic
    {  558,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  558,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  558,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  558,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  558,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  558,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  558,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  558,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  558,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  558,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  558,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  558,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  558, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  558, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  558, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  558, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  558, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  558, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  559,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Ecuador
    {  559,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  559,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  559,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  559,  619,       0,  292428,       0,  0, 15,  0, }, // Ecuador
    {  559,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  559,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  559,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  559,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  559,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  559,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  559,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  559,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  559,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  559, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  559, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  559, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  559, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  559, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  559, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  560,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/El Salvador
    {  560,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  560,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  560,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  560,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  560,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  560,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  560,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  560,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  560,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  560,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  560,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  560,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  560, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  560, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  560, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  560, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  560, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  560, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  562,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Guatemala
    {  562,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  562,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  562,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  562,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  562,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  562,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  562,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  562,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  562,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  562,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  562,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  562,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  562, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  562, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  562, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  562, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  562, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  562, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  563,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Honduras
    {  563,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  563,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  563,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  563,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  563,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  563,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  563,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  563,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  563,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  563,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  563,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  563,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  563, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  563, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  563, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  563, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  563, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  563, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  564,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Latin America
    {  564,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  564,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  564,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  564,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  564,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  564,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  564,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  564,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  564,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  564,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  564,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  564,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  564, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  564, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  564, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  564, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  564, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  564, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  565,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Mexico
    {  565,  214,  290728,  290740,  737372, 12, 21, 22, }, // Apia
    {  565,  507,       0,  737394,       0,  0, 26,  0, }, // Christmas
    {  565,  612,  737420,  733077,  733111, 22, 34, 35, }, // Easter
    {  565,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  565,  642,  367796,  733209,  733241, 23, 32, 33, }, // Europe_Eastern
    {  565,  680,  367908,  733309,  733343, 25, 34, 35, }, // Europe_Western
    {  565,  695,  737442,  737464,  737495, 22, 31, 32, }, // Falkland
    {  565,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  565,  772,       0,  737527,       0,  0, 25,  0, }, // Gilbert_Islands
    {  565,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  565,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  565, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  565, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  565, 1850,       0,  737552,       0,  0, 20,  0, }, // Wake
    {  566,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Nicaragua
    {  566,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  566,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  566,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  566,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  566,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  566,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  566,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  566,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  566,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  566,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  566,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  566,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  566, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  566, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  566, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  566, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  566, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  566, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  567,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Panama
    {  567,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  567,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  567,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  567,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  567,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  567,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  567,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  567,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  567,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  567,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  567,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  567,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  567, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  567, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  567, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  567, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  567, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  567, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  568,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Paraguay
    {  568,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  568,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  568,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  568,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  568,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  568,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  568,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  568,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  568,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  568,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  568,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  568,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  568, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  568, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  568, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  568, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  568, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  568, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  569,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Peru
    {  569,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  569,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  569,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  569,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  569,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  569,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  569,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  569,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  569,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  569,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  569,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  569,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  569, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  569, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  569, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  569, 1469,  735651,  735663,  735684, 12, 21, 22, }, // Peru
    {  569, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  569, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  569, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  571,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Puerto Rico
    {  571,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  571,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  571,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  571,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  571,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  571,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  571,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  571,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  571,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  571,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  571,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  571,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  571, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  571, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  571, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  571, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  571, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  571, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  572,  107,  365822,  731317,  731340, 14, 23, 24, }, // Alaska Spanish/Latin/United States
    {  572,  142,   42433,  290244,   42506, 12, 21, 22, }, // America_Central
    {  572,  158,   42548,  731434,   42623, 13, 22, 23, }, // America_Eastern
    {  572,  174,  731456,  731484,  731521, 28, 37, 38, }, // America_Mountain
    {  572,  191,  731559,  731576,  731602, 17, 26, 27, }, // America_Pacific
    {  572,  214,  290728,  290740,  737372, 12, 21, 22, }, // Apia
    {  572,  286,  732034,  732052,  732079, 18, 27, 28, }, // Atlantic
    {  572,  478,       0,  737572,       0,  0, 16,  0, }, // Chamorro
    {  572,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  572,  642,  367796,  733209,  733241, 23, 32, 33, }, // Europe_Eastern
    {  572,  680,  367908,  733309,  733343, 25, 34, 35, }, // Europe_Western
    {  572,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  572,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  572,  883,  733907,  733931,  733964, 24, 33, 34, }, // Hawaii_Aleutian
    {  572,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  572,  920,       0,  737588,       0,  0, 22,  0, }, // Indian_Ocean
    {  572, 1237,       0,  737610,       0,  0, 27,  0, }, // Marquesas
    {  572, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  572, 1486,       0,  737637,       0,  0, 23,  0, }, // Phoenix_Islands
    {  573,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Uruguay
    {  573,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  573,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  573,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  573,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  573,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  573,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  573,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  573,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  573,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  573,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  573,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  573,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  573, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  573, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  573, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  573, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  573, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  573, 1777,  371012,  736218,  736242, 15, 24, 25, }, // Uruguay
    {  573, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  574,  174,  736513,  736531,  736558, 18, 27, 28, }, // America_Mountain Spanish/Latin/Venezuela
    {  574,  517,       0,  736586,       0,  0, 19,  0, }, // Cocos
    {  574,  532,  736605,  736627,  736658, 22, 31, 38, }, // Cook
    {  574,  612,  736696,  736721,  736752, 25, 31, 35, }, // Easter
    {  574,  627,  367716,  733146,  733177, 22, 31, 32, }, // Europe_Central
    {  574,  642,  736787,  736810,  736842, 23, 32, 33, }, // Europe_Eastern
    {  574,  657,       0,  736875,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  574,  680,  736908,  736932,  736965, 24, 33, 34, }, // Europe_Western
    {  574,  695,  736999,  737025,  737060, 26, 35, 36, }, // Falkland
    {  574,  723,       0,  737096,       0,  0, 52,  0, }, // French_Southern
    {  574,  772,       0,  737148,       0,  0, 21,  0, }, // Gilbert_Islands
    {  574,  788,       0,  733699,       0,  0, 31,  0, }, // GMT
    {  574,  914,       0,  737169,       0,  0, 13,  0, }, // India
    {  574, 1247,       0,  737182,       0,  0, 22,  0, }, // Marshall_Islands
    {  574, 1312,       0,  737204,       0,  0, 26,  0, }, // Myanmar
    {  574, 1376,  737230,  737253,  737285, 23, 32, 33, }, // Norfolk
    {  574, 1534,       0,  737318,       0,  0, 16,  0, }, // Pyongyang
    {  574, 1634,       0,  737334,       0,  0, 21,  0, }, // Solomon
    {  574, 1811,       0,  296673,       0,  0, 17,  0, }, // Venezuela
    {  574, 1850,       0,  737355,       0,  0, 17,  0, }, // Wake
    {  576,  142,  359224,  359236,  737660, 12, 20, 23, }, // America_Central Sundanese/Latin/Indonesia
    {  576,  158,  737683,  737694,  737713, 11, 19, 22, }, // America_Eastern
    {  576,  174,  737735,  737751,  737775, 16, 24, 27, }, // America_Mountain
    {  576,  191,  359401,  359414,  737802, 13, 21, 24, }, // America_Pacific
    {  576,  286,  359953,  359967,  737826, 14, 22, 25, }, // Atlantic
    {  576,  523,  360986,  361000,  737851, 14, 22, 25, }, // Colombia
    {  576,  627,  737876,  737894,  737920, 18, 26, 29, }, // Europe_Central
    {  576,  642,  737949,  737966,  737991, 17, 25, 28, }, // Europe_Eastern
    {  576,  680,  738019,  738036,  738061, 17, 25, 28, }, // Europe_Western
    {  576,  746,       0,  361751,       0,  0, 15,  0, }, // Galapagos
    {  576,  788,       0,  738089,       0,  0, 15,  0, }, // GMT
    {  577,    5,       0,  738104,       0,  0, 18,  0, }, // Afghanistan Swahili/Latin/Tanzania
    {  577,   17,       0,  738122,       0,  0, 21,  0, }, // Africa_Central
    {  577,   32,       0,  738143,       0,  0, 23,  0, }, // Africa_Eastern
    {  577,   65,       0,  738166,       0,  0, 31,  0, }, // Africa_Southern
    {  577,   81,  738197,  738220,  738254, 23, 34, 41, }, // Africa_Western
    {  577,  107,  738295,  738308,  738332, 13, 24, 23, }, // Alaska
    {  577,  135,  738355,  738368,  738392, 13, 24, 31, }, // Amazon
    {  577,  142,  738423,  738434,  738456, 11, 22, 21, }, // America_Central
    {  577,  158,  738477,  738493,  738520, 16, 27, 26, }, // America_Eastern
    {  577,  174,  738546,  738561,  738587, 15, 26, 25, }, // America_Mountain
    {  577,  191,  738612,  738627,  738653, 15, 26, 25, }, // America_Pacific
    {  577,  207,  738678,  738691,  738715, 13, 24, 25, }, // Anadyr
    {  577,  214,  738740,  738751,  738773, 11, 22, 21, }, // Apia
    {  577,  232,  738794,  738809,  738835, 15, 26, 25, }, // Arabian
    {  577,  240,  738860,  738876,  738903, 16, 27, 34, }, // Argentina
    {  577,  250,  738937,  738967,  739008, 30, 41, 48, }, // Argentina_Western
    {  577,  268,  739056,  739070,  739095, 14, 25, 32, }, // Armenia
    {  577,  286,  739127,  739143,  739170, 16, 27, 26, }, // Atlantic
    {  577,  295,  739196,  739220,  739255, 24, 35, 34, }, // Australia_Central
    {  577,  313,  739289,  739326,  739374, 37, 48, 47, }, // Australia_CentralWestern
    {  577,  338,  739421,  739447,  739488, 26, 41, 40, }, // Australia_Eastern
    {  577,  356,  739528,  739554,  739591, 26, 37, 36, }, // Australia_Western
    {  577,  374,  739627,  739644,  739672, 17, 28, 35, }, // Azerbaijan
    {  577,  385,  739707,  739720,  739744, 13, 24, 31, }, // Azores
    {  577,  397,  739775,  739792,  739820, 17, 28, 35, }, // Bangladesh
    {  577,  415,       0,  739855,       0,  0, 13,  0, }, // Bhutan
    {  577,  422,       0,  739868,       0,  0, 14,  0, }, // Bolivia
    {  577,  437,  739882,  739897,  739923, 15, 26, 33, }, // Brasilia
    {  577,  454,       0,  739956,       0,  0, 24,  0, }, // Brunei
    {  577,  461,  739980,  739997,  740025, 17, 28, 35, }, // Cape_Verde
    {  577,  478,       0,  740060,       0,  0, 26,  0, }, // Chamorro
    {  577,  487,  740086,  740100,  740125, 14, 25, 24, }, // Chatham
    {  577,  495,  740149,  740161,  740184, 12, 23, 30, }, // Chile
    {  577,  501,  740214,  740227,  740251, 13, 24, 23, }, // China
    {  577,  507,       0,  740274,       0,  0, 26,  0, }, // Christmas
    {  577,  517,       0,  740300,       0,  0, 23,  0, }, // Cocos
    {  577,  523,  740323,  740338,  740364, 15, 26, 33, }, // Colombia
    {  577,  532,  740397,  740419,  740452, 22, 33, 41, }, // Cook
    {  577,  537,  740493,  740504,  740526, 11, 22, 21, }, // Cuba
    {  577,  548,       0,  740547,       0,  0, 12,  0, }, // Davis
    {  577,  564,       0,  740559,       0,  0, 23,  0, }, // DumontDUrville
    {  577,  601,       0,  740582,       0,  0, 22,  0, }, // East_Timor
    {  577,  612,  740604,  740628,  740663, 24, 35, 42, }, // Easter
    {  577,  619,       0,  740705,       0,  0, 13,  0, }, // Ecuador
    {  577,  627,  740718,  740738,  740769, 20, 31, 38, }, // Europe_Central
    {  577,  642,  740807,  740833,  740870, 26, 37, 44, }, // Europe_Eastern
    {  577,  657,       0,  740914,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  577,  680,  740946,  740972,  741009, 26, 37, 44, }, // Europe_Western
    {  577,  695,  741053,  741079,  741116, 26, 37, 44, }, // Falkland
    {  577,  704,  741160,  741171,  741193, 11, 22, 29, }, // Fiji
    {  577,  709,       0,  741222,       0,  0, 25,  0, }, // French_Guiana
    {  577,  723,       0,  741247,       0,  0, 39,  0, }, // French_Southern
    {  577,  746,       0,  741286,       0,  0, 16,  0, }, // Galapagos
    {  577,  756,       0,  741302,       0,  0, 14,  0, }, // Gambier
    {  577,  764,  741316,  741328,  741351, 12, 23, 30, }, // Georgia
    {  577,  772,       0,  741381,       0,  0, 25,  0, }, // Gilbert_Islands
    {  577,  788,       0,  741406,       0,  0, 16,  0, }, // GMT
    {  577,  830,  741422,  741448,  741485, 26, 37, 44, }, // Greenland_Eastern
    {  577,  848,  741529,  741555,  741592, 26, 37, 44, }, // Greenland_Western
    {  577,  871,       0,  741636,       0,  0, 22,  0, }, // Gulf
    {  577,  876,       0,  741658,       0,  0, 13,  0, }, // Guyana
    {  577,  883,  741671,  741693,  741726, 22, 33, 32, }, // Hawaii_Aleutian
    {  577,  899,  741758,  741774,  741801, 16, 27, 34, }, // Hong_Kong
    {  577,  909,  741835,  741846,  741868, 11, 22, 29, }, // Hovd
    {  577,  914,       0,  741897,       0,  0, 23,  0, }, // India
    {  577,  920,       0,  741920,       0,  0, 19,  0, }, // Indian_Ocean
    {  577,  933,       0,  741939,       0,  0, 16,  0, }, // Indochina
    {  577,  943,       0,  741955,       0,  0, 24,  0, }, // Indonesia_Central
    {  577,  961,       0,  741979,       0,  0, 30,  0, }, // Indonesia_Eastern
    {  577,  979,       0,  742009,       0,  0, 30,  0, }, // Indonesia_Western
    {  577,  997,  742039,  742050,  742072, 11, 22, 21, }, // Iran
    {  577, 1008,  742093,  742107,  742132, 14, 25, 32, }, // Irkutsk
    {  577, 1016,  742164,  742178,  742203, 14, 25, 24, }, // Israel
    {  577, 1023,  742227,  742239,  742263, 12, 24, 22, }, // Japan
    {  577, 1029,  742285,  742316,  742358, 31, 42, 43, }, // Kamchatka
    {  577, 1047,       0,  742401,       0,  0, 17,  0, }, // Kazakhstan
    {  577, 1058,       0,  742418,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  577, 1077,       0,  742445,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  577, 1106,  742472,  742484,  742507, 12, 23, 22, }, // Korea
    {  577, 1112,       0,  742529,       0,  0, 13,  0, }, // Kosrae
    {  577, 1119,  742542,  742560,  742588, 18, 28, 36, }, // Krasnoyarsk
    {  577, 1151,       0,  742624,       0,  0, 16,  0, }, // Kyrgystan
    {  577, 1175,       0,  742640,       0,  0, 22,  0, }, // Line_Islands
    {  577, 1188,  742662,  742678,  742705, 16, 27, 26, }, // Lord_Howe
    {  577, 1204,  742731,  742745,  742770, 14, 25, 32, }, // Magadan
    {  577, 1219,       0,  742802,       0,  0, 15,  0, }, // Malaysia
    {  577, 1228,       0,  742817,       0,  0, 15,  0, }, // Maldives
    {  577, 1237,       0,  742832,       0,  0, 16,  0, }, // Marquesas
    {  577, 1247,       0,  742848,       0,  0, 26,  0, }, // Marshall_Islands
    {  577, 1264,  742874,  742887,  742911, 13, 24, 31, }, // Mauritius
    {  577, 1274,       0,  742942,       0,  0, 13,  0, }, // Mawson
    {  577, 1281,  742955,  742981,  743018, 26, 37, 36, }, // Mexico_Pacific
    {  577, 1296,  743054,  743071,  743099, 17, 28, 35, }, // Mongolia
    {  577, 1305,  743134,  743147,  743171, 13, 24, 31, }, // Moscow
    {  577, 1312,       0,  743202,       0,  0, 14,  0, }, // Myanmar
    {  577, 1320,       0,  743216,       0,  0, 12,  0, }, // Nauru
    {  577, 1326,       0,  743228,       0,  0, 12,  0, }, // Nepal
    {  577, 1332,  743240,  743260,  743291, 20, 31, 38, }, // New_Caledonia
    {  577, 1346,  743329,  743347,  743376, 18, 29, 28, }, // New_Zealand
    {  577, 1358,  743404,  743423,  743453, 19, 30, 29, }, // Newfoundland
    {  577, 1371,       0,  743482,       0,  0, 11,  0, }, // Niue
    {  577, 1376,  743493,  743518,  743554, 25, 36, 43, }, // Norfolk
    {  577, 1384,  743597,  743623,  743660, 26, 37, 44, }, // Noronha
    {  577, 1406,  743704,  743722,  743751, 18, 29, 36, }, // Novosibirsk
    {  577, 1418,  743787,  743798,  743820, 11, 22, 29, }, // Omsk
    {  577, 1428,  743849,  743864,  743890, 15, 26, 33, }, // Pakistan
    {  577, 1437,       0,  743923,       0,  0, 12,  0, }, // Palau
    {  577, 1443,       0,  743935,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  577, 1460,  743958,  743973,  743999, 15, 26, 33, }, // Paraguay
    {  577, 1469,  744032,  744043,  744065, 11, 22, 29, }, // Peru
    {  577, 1474,  744094,  744110,  744137, 16, 27, 34, }, // Philippines
    {  577, 1486,       0,  744171,       0,  0, 25,  0, }, // Phoenix_Islands
    {  577, 1502,  744196,  744227,  744269, 31, 42, 41, }, // Pierre_Miquelon
    {  577, 1518,       0,  744310,       0,  0, 15,  0, }, // Pitcairn
    {  577, 1527,       0,  744325,       0,  0, 13,  0, }, // Ponape
    {  577, 1534,       0,  744338,       0,  0, 16,  0, }, // Pyongyang
    {  577, 1554,       0,  744354,       0,  0, 14,  0, }, // Reunion
    {  577, 1562,       0,  744368,       0,  0, 14,  0, }, // Rothera
    {  577, 1570,  744382,  744397,  744423, 15, 26, 33, }, // Sakhalin
    {  577, 1579,  744456,  744469,  744493, 13, 24, 25, }, // Samara
    {  577, 1596,  744518,  744530,  744553, 12, 23, 30, }, // Samoa
    {  577, 1602,       0,  744583,       0,  0, 18,  0, }, // Seychelles
    {  577, 1624,       0,  744601,       0,  0, 27,  0, }, // Singapore
    {  577, 1634,       0,  744628,       0,  0, 25,  0, }, // Solomon
    {  577, 1642,       0,  744653,       0,  0, 21,  0, }, // South_Georgia
    {  577, 1656,       0,  744674,       0,  0, 15,  0, }, // Suriname
    {  577, 1676,       0,  744689,       0,  0, 12,  0, }, // Syowa
    {  577, 1682,       0,  744701,       0,  0, 13,  0, }, // Tahiti
    {  577, 1689,  744714,  744727,  744751, 13, 24, 23, }, // Taipei
    {  577, 1696,       0,  744774,       0,  0, 17,  0, }, // Tajikistan
    {  577, 1724,       0,  744791,       0,  0, 14,  0, }, // Tokelau
    {  577, 1732,  744805,  744817,  744840, 12, 23, 30, }, // Tonga
    {  577, 1738,       0,  744870,       0,  0, 12,  0, }, // Truk
    {  577, 1750,  744882,  744901,  744931, 19, 30, 37, }, // Turkmenistan
    {  577, 1763,       0,  744968,       0,  0, 13,  0, }, // Tuvalu
    {  577, 1777,  744981,  744995,  745020, 14, 25, 32, }, // Uruguay
    {  577, 1792,  745052,  745069,  745097, 17, 28, 35, }, // Uzbekistan
    {  577, 1803,  745132,  745146,  745171, 14, 25, 32, }, // Vanuatu
    {  577, 1811,       0,  745203,       0,  0, 16,  0, }, // Venezuela
    {  577, 1821,  745219,  745237,  745266, 18, 29, 36, }, // Vladivostok
    {  577, 1833,  745302,  745318,  745345, 16, 27, 34, }, // Volgograd
    {  577, 1843,       0,  745379,       0,  0, 13,  0, }, // Vostok
    {  577, 1850,       0,  745392,       0,  0, 22,  0, }, // Wake
    {  577, 1855,       0,  745414,       0,  0, 23,  0, }, // Wallis
    {  577, 1862,  745437,  745451,  745476, 14, 25, 32, }, // Yakutsk
    {  577, 1870,  745508,  745528,  745559, 20, 31, 38, }, // Yekaterinburg
    {  577, 1892,       0,  745597,       0,  0, 12,  0, }, // Yukon
    {  579,    5,       0,  745609,       0,  0, 19,  0, }, // Afghanistan Swahili/Latin/Kenya
    {  579,   81,  738197,  738220,  745628, 23, 34, 41, }, // Africa_Western
    {  579,  135,  738355,  738368,  745669, 13, 24, 31, }, // Amazon
    {  579,  240,  745700,  745715,  745741, 15, 26, 30, }, // Argentina
    {  579,  250,  745771,  745800,  745840, 29, 40, 47, }, // Argentina_Western
    {  579,  268,  739056,  739070,  745887, 14, 25, 32, }, // Armenia
    {  579,  313,  745919,  745958,  746007, 39, 49, 48, }, // Australia_CentralWestern
    {  579,  374,  746055,  746071,  746098, 16, 27, 34, }, // Azerbaijan
    {  579,  385,  739707,  739720,  746132, 13, 24, 31, }, // Azores
    {  579,  397,  746163,  746181,  746210, 18, 29, 36, }, // Bangladesh
    {  579,  415,       0,  746246,       0,  0, 13,  0, }, // Bhutan
    {  579,  437,  746259,  746273,  746298, 14, 25, 32, }, // Brasilia
    {  579,  461,  746330,  746345,  746371, 15, 26, 33, }, // Cape_Verde
    {  579,  523,  740323,  740338,  746404, 15, 26, 33, }, // Colombia
    {  579,  532,  740397,  740419,  746437, 22, 33, 45, }, // Cook
    {  579,  537,  740493,  746482,  740526, 11, 22, 21, }, // Cuba
    {  579,  612,  740604,  740628,  746504, 24, 35, 42, }, // Easter
    {  579,  627,  740718,  740738,  746546, 20, 31, 38, }, // Europe_Central
    {  579,  642,  740807,  740833,  746584, 26, 37, 44, }, // Europe_Eastern
    {  579,  680,  740946,  740972,  746628, 26, 37, 44, }, // Europe_Western
    {  579,  709,       0,  741222,       0,  0, 13,  0, }, // French_Guiana
    {  579,  764,  746672,  746685,  746709, 13, 24, 31, }, // Georgia
    {  579,  830,  741422,  741448,  746740, 26, 37, 44, }, // Greenland_Eastern
    {  579,  871,       0,  746784,       0,  0, 23,  0, }, // Gulf
    {  579,  899,  741758,  741774,  746807, 16, 27, 34, }, // Hong_Kong
    {  579,  909,  741835,  741846,  746841, 11, 22, 29, }, // Hovd
    {  579,  997,  746870,  746882,  746905, 12, 23, 22, }, // Iran
    {  579, 1008,  742093,  742107,  746927, 14, 25, 32, }, // Irkutsk
    {  579, 1023,  746959,  742239,  746972, 13, 24, 23, }, // Japan
    {  579, 1058,       0,  746995,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  579, 1077,       0,  747023,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  579, 1119,  742542,  742560,  747051, 18, 28, 36, }, // Krasnoyarsk
    {  579, 1204,  742731,  742745,  747087, 14, 25, 32, }, // Magadan
    {  579, 1219,       0,  747119,       0,  0, 14,  0, }, // Malaysia
    {  579, 1228,       0,  747133,       0,  0, 14,  0, }, // Maldives
    {  579, 1264,  742874,  742887,  747147, 13, 24, 31, }, // Mauritius
    {  579, 1296,  747178,  747196,  747225, 18, 29, 36, }, // Mongolia
    {  579, 1305,  743134,  743147,  747261, 13, 24, 31, }, // Moscow
    {  579, 1312,       0,  743202,       0,  0, 13,  0, }, // Myanmar
    {  579, 1326,       0,  747292,       0,  0, 13,  0, }, // Nepal
    {  579, 1332,  747305,  747326,  747358, 21, 32, 39, }, // New_Caledonia
    {  579, 1346,  747397,  747414,  747442, 17, 28, 27, }, // New_Zealand
    {  579, 1376,  743493,  743518,  747469, 25, 36, 43, }, // Norfolk
    {  579, 1406,  743704,  743722,  747512, 18, 29, 36, }, // Novosibirsk
    {  579, 1418,  743787,  743798,  747548, 11, 22, 29, }, // Omsk
    {  579, 1428,  747577,  747593,  747620, 16, 27, 34, }, // Pakistan
    {  579, 1443,       0,  743935,       0,  0, 12,  0, }, // Papua_New_Guinea
    {  579, 1460,  743958,  743973,  747654, 15, 26, 33, }, // Paraguay
    {  579, 1469,  744032,  744043,  747687, 11, 22, 29, }, // Peru
    {  579, 1474,  744094,  744110,  747716, 16, 27, 34, }, // Philippines
    {  579, 1486,       0,  747750,       0,  0, 25,  0, }, // Phoenix_Islands
    {  579, 1570,  744382,  744397,  747775, 15, 26, 33, }, // Sakhalin
    {  579, 1596,  744518,  744530,  747808, 12, 23, 22, }, // Samoa
    {  579, 1624,       0,  747830,       0,  0, 26,  0, }, // Singapore
    {  579, 1642,       0,  747856,       0,  0, 19,  0, }, // South_Georgia
    {  579, 1696,       0,  747875,       0,  0, 17,  0, }, // Tajikistan
    {  579, 1732,  744805,  744817,  747892, 12, 23, 30, }, // Tonga
    {  579, 1750,  747922,  747942,  747973, 20, 31, 38, }, // Turkmenistan
    {  579, 1777,  744981,  744995,  748011, 14, 25, 32, }, // Uruguay
    {  579, 1792,  748043,  748061,  748090, 18, 29, 36, }, // Uzbekistan
    {  579, 1803,  745132,  745146,  748126, 14, 25, 32, }, // Vanuatu
    {  579, 1821,  745219,  745237,  748158, 18, 29, 36, }, // Vladivostok
    {  579, 1833,  745302,  745318,  748194, 16, 27, 34, }, // Volgograd
    {  579, 1862,  745437,  745451,  748228, 14, 25, 32, }, // Yakutsk
    {  579, 1870,  745508,  745528,  748260, 20, 31, 38, }, // Yekaterinburg
    {  583,    0,  748298,  748317,  748342, 19, 25, 25, }, // Acre Swedish/Latin/Sweden
    {  583,    5,       0,  559031,       0,  0, 12,  0, }, // Afghanistan
    {  583,   17,       0,  748367,       0,  0, 20,  0, }, // Africa_Central
    {  583,   32,       0,  748387,       0,  0, 16,  0, }, // Africa_Eastern
    {  583,   65,       0,  748403,       0,  0, 16,  0, }, // Africa_Southern
    {  583,   81,  748419,  748436,  748459, 17, 23, 23, }, // Africa_Western
    {  583,  107,  748482,  748491,  748508,  9, 17, 17, }, // Alaska
    {  583,  128,  748525,  748534,  748549,  9, 15, 15, }, // Almaty
    {  583,  135,  559299,  748564,  748583, 11, 19, 19, }, // Amazon
    {  583,  142,  748602,  748627,  748658, 25, 31, 31, }, // America_Central
    {  583,  158,  748689,  748710,  748737, 21, 27, 27, }, // America_Eastern
    {  583,  174,  748764,  748782,  748808, 18, 26, 26, }, // America_Mountain
    {  583,  191,  748834,  748856,  748884, 22, 28, 28, }, // America_Pacific
    {  583,  207,  748912,  748921,  748936,  9, 15, 15, }, // Anadyr
    {  583,  214,  748951,  748958,  748973,  7, 15, 15, }, // Apia
    {  583,  219,  748988,  748996,  749010,  8, 14, 14, }, // Aqtau
    {  583,  225,  749024,  749033,  749048,  9, 15, 15, }, // Aqtobe
    {  583,  232,  749063,  749079,  749101, 16, 22, 22, }, // Arabian
    {  583,  240,  749123,  749140,  749163, 17, 23, 23, }, // Argentina
    {  583,  250,  749186,  749204,  749228, 18, 24, 24, }, // Argentina_Western
    {  583,  268,  749252,  749264,  749282, 12, 18, 18, }, // Armenia
    {  583,  286,  749300,  749324,  749354, 24, 30, 30, }, // Atlantic
    {  583,  295,  749384,  749405,  749432, 21, 27, 27, }, // Australia_Central
    {  583,  313,  749459,  749484,  749515, 25, 31, 31, }, // Australia_CentralWestern
    {  583,  338,  749546,  749563,  749586, 17, 23, 23, }, // Australia_Eastern
    {  583,  356,  749609,  749627,  749651, 18, 24, 24, }, // Australia_Western
    {  583,  374,  749675,  749693,  749717, 18, 24, 24, }, // Azerbaijan
    {  583,  385,  749741,  749752,  749769, 11, 17, 17, }, // Azores
    {  583,  397,  560705,  560722,  566802, 17, 23, 23, }, // Bangladesh
    {  583,  415,       0,  560768,       0,  0, 12,  0, }, // Bhutan
    {  583,  422,       0,  560780,       0,  0, 14,  0, }, // Bolivia
    {  583,  437,  560851,  749786,  749805, 11, 19, 19, }, // Brasilia
    {  583,  454,       0,  749824,       0,  0,  9,  0, }, // Brunei
    {  583,  461,  749833,  749845,  749865, 12, 20, 20, }, // Cape_Verde
    {  583,  472,       0,  749885,       0,  0,  8,  0, }, // Casey
    {  583,  478,       0,  560968,       0,  0, 11,  0, }, // Chamorro
    {  583,  487,  749893,  749903,  749921, 10, 18, 18, }, // Chatham
    {  583,  495,  561038,  561050,  566889, 12, 18, 18, }, // Chile
    {  583,  501,  561086,  561098,  566907, 12, 18, 18, }, // China
    {  583,  507,       0,  749939,       0,  0, 10,  0, }, // Christmas
    {  583,  517,       0,  749949,       0,  0, 17,  0, }, // Cocos
    {  583,  523,  561182,  561197,  749966, 15, 21, 21, }, // Colombia
    {  583,  532,  749987,  750001,  750021, 14, 20, 20, }, // Cook
    {  583,  537,  567073,  567084,  567101, 11, 17, 17, }, // Cuba
    {  583,  548,       0,  561370,       0,  0,  8,  0, }, // Davis
    {  583,  564,       0,  750041,       0,  0, 20,  0, }, // DumontDUrville
    {  583,  601,       0,  750061,       0,  0, 15,  0, }, // East_Timor
    {  583,  612,  750076,  750084,  750101,  8, 17, 17, }, // Easter
    {  583,  619,       0,  561486,       0,  0, 16,  0, }, // Ecuador
    {  583,  627,  750118,  750138,  750164, 20, 26, 26, }, // Europe_Central
    {  583,  642,  750190,  750206,  750228, 16, 22, 22, }, // Europe_Eastern
    {  583,  657,       0,  750250,       0,  0, 14,  0, }, // Europe_Further_Eastern
    {  583,  680,  750264,  750281,  750304, 17, 23, 23, }, // Europe_Western
    {  583,  695,  750327,  750346,  750371, 19, 25, 25, }, // Falkland
    {  583,  704,  750396,  750403,  750418,  7, 15, 15, }, // Fiji
    {  583,  709,       0,  750433,       0,  0, 17,  0, }, // French_Guiana
    {  583,  723,       0,  750450,       0,  0, 29,  0, }, // French_Southern
    {  583,  746,       0,  750479,       0,  0, 12,  0, }, // Galapagos
    {  583,  756,       0,  750491,       0,  0, 10,  0, }, // Gambier
    {  583,  764,  561961,  561973,  567543, 12, 18, 18, }, // Georgia
    {  583,  772,       0,  750501,       0,  0, 11,  0, }, // Gilbert_Islands
    {  583,  788,       0,  750512,       0,  0, 12,  0, }, // GMT
    {  583,  830,  750524,  750541,  750564, 17, 23, 23, }, // Greenland_Eastern
    {  583,  848,  750587,  750605,  750629, 18, 24, 24, }, // Greenland_Western
    {  583,  866,       0,  750653,       0,  0,  7,  0, }, // Guam
    {  583,  871,       0,  750660,       0,  0, 17,  0, }, // Gulf
    {  583,  876,       0,  561870,       0,  0,  9,  0, }, // Guyana
    {  583,  883,  750677,  750688,  750707, 11, 19, 19, }, // Hawaii_Aleutian
    {  583,  899,  562377,  750726,  750745, 11, 19, 19, }, // Hong_Kong
    {  583,  909,  750764,  750772,  750788,  8, 16, 16, }, // Hovd
    {  583,  914,       0,  562441,       0,  0, 10,  0, }, // India
    {  583,  920,       0,  750804,       0,  0, 33,  0, }, // Indian_Ocean
    {  583,  933,       0,  562474,       0,  0, 16,  0, }, // Indochina
    {  583,  943,       0,  750837,       0,  0, 21,  0, }, // Indonesia_Central
    {  583,  961,       0,  750858,       0,  0, 17,  0, }, // Indonesia_Eastern
    {  583,  979,       0,  750875,       0,  0, 18,  0, }, // Indonesia_Western
    {  583,  997,  562546,  562556,  567817, 10, 16, 16, }, // Iran
    {  583, 1008,  750893,  750903,  750921, 10, 18, 18, }, // Irkutsk
    {  583, 1016,  750939,  750952,  750971, 13, 19, 19, }, // Israel
    {  583, 1023,  562698,  562709,  567872, 11, 17, 17, }, // Japan
    {  583, 1029,  750990,  751002,  751020, 12, 18, 18, }, // Kamchatka
    {  583, 1047,       0,  751038,       0,  0, 15,  0, }, // Kazakhstan
    {  583, 1058,       0,  751053,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  583, 1077,       0,  751071,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  583, 1106,  562914,  562926,  567909, 12, 18, 18, }, // Korea
    {  583, 1112,       0,  562975,       0,  0,  9,  0, }, // Kosrae
    {  583, 1119,  567941,  751090,  751112, 14, 22, 22, }, // Krasnoyarsk
    {  583, 1151,       0,  751134,       0,  0, 13,  0, }, // Kyrgystan
    {  583, 1161,       0,  751147,       0,  0, 12,  0, }, // Lanka
    {  583, 1175,       0,  751159,       0,  0, 14,  0, }, // Line_Islands
    {  583, 1188,  751173,  751185,  751205, 12, 20, 20, }, // Lord_Howe
    {  583, 1198,  751225,  751233,  751247,  8, 14, 14, }, // Macau
    {  583, 1204,  568015,  751261,  751279, 10, 18, 18, }, // Magadan
    {  583, 1219,       0,  563285,       0,  0, 13,  0, }, // Malaysia
    {  583, 1228,       0,  751297,       0,  0, 13,  0, }, // Maldives
    {  583, 1237,       0,  751310,       0,  0, 12,  0, }, // Marquesas
    {  583, 1247,       0,  751322,       0,  0, 18,  0, }, // Marshall_Islands
    {  583, 1264,  751340,  751352,  751372, 12, 20, 20, }, // Mauritius
    {  583, 1274,       0,  563419,       0,  0,  9,  0, }, // Mawson
    {  583, 1281,  751392,  751415,  751449, 23, 34, 34, }, // Mexico_Pacific
    {  583, 1296,  751483,  751497,  751519, 14, 22, 22, }, // Mongolia
    {  583, 1305,  751541,  751550,  751567,  9, 17, 17, }, // Moscow
    {  583, 1312,       0,  751584,       0,  0, 13,  0, }, // Myanmar
    {  583, 1320,       0,  751597,       0,  0,  8,  0, }, // Nauru
    {  583, 1326,       0,  751605,       0,  0, 14,  0, }, // Nepal
    {  583, 1332,  751619,  751636,  751661, 17, 25, 25, }, // New_Caledonia
    {  583, 1346,  751686,  751701,  751722, 15, 21, 21, }, // New_Zealand
    {  583, 1358,  563903,  751743,  751766, 15, 23, 23, }, // Newfoundland
    {  583, 1371,       0,  563928,       0,  0,  7,  0, }, // Niue
    {  583, 1376,  751789,  751803,  751823, 14, 20, 20, }, // Norfolk
    {  583, 1384,  751843,  751865,  751895, 22, 30, 30, }, // Noronha
    {  583, 1392,       0,  751925,       0,  0, 19,  0, }, // North_Mariana
    {  583, 1406,  564182,  751944,  751966, 14, 22, 22, }, // Novosibirsk
    {  583, 1418,  751988,  751995,  752010,  7, 15, 15, }, // Omsk
    {  583, 1428,  564246,  564260,  568451, 14, 20, 20, }, // Pakistan
    {  583, 1437,       0,  752025,       0,  0,  8,  0, }, // Palau
    {  583, 1443,       0,  752033,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  583, 1460,  564324,  564340,  568471, 16, 22, 22, }, // Paraguay
    {  583, 1469,  564384,  564396,  568493, 12, 18, 18, }, // Peru
    {  583, 1474,  564432,  564446,  568511, 14, 20, 20, }, // Philippines
    {  583, 1486,       0,  752054,       0,  0, 12,  0, }, // Phoenix_Islands
    {  583, 1502,  752066,  752094,  752129, 28, 35, 35, }, // Pierre_Miquelon
    {  583, 1518,       0,  564637,       0,  0, 11,  0, }, // Pitcairn
    {  583, 1527,       0,  752164,       0,  0,  9,  0, }, // Ponape
    {  583, 1534,       0,  752173,       0,  0, 12,  0, }, // Pyongyang
    {  583, 1544,  752185,  752197,  752215, 12, 18, 18, }, // Qyzylorda
    {  583, 1554,       0,  564742,       0,  0, 10,  0, }, // Reunion
    {  583, 1562,       0,  564762,       0,  0, 10,  0, }, // Rothera
    {  583, 1570,  752233,  752244,  752263, 11, 19, 19, }, // Sakhalin
    {  583, 1579,  752282,  752291,  752306,  9, 15, 15, }, // Samara
    {  583, 1596,  564906,  564918,  568616, 12, 18, 18, }, // Samoa
    {  583, 1602,       0,  752321,       0,  0, 15,  0, }, // Seychelles
    {  583, 1624,       0,  752336,       0,  0, 12,  0, }, // Singapore
    {  583, 1634,       0,  752348,       0,  0, 17,  0, }, // Solomon
    {  583, 1642,       0,  752365,       0,  0, 15,  0, }, // South_Georgia
    {  583, 1656,       0,  752380,       0,  0, 10,  0, }, // Suriname
    {  583, 1676,       0,  752390,       0,  0,  8,  0, }, // Syowa
    {  583, 1682,       0,  752398,       0,  0,  9,  0, }, // Tahiti
    {  583, 1689,  752407,  752416,  752433,  9, 17, 17, }, // Taipei
    {  583, 1696,       0,  752450,       0,  0, 15,  0, }, // Tajikistan
    {  583, 1724,       0,  752465,       0,  0, 10,  0, }, // Tokelau
    {  583, 1732,  752475,  752483,  752499,  8, 16, 16, }, // Tonga
    {  583, 1738,       0,  752515,       0,  0,  8,  0, }, // Truk
    {  583, 1750,  565227,  565240,  568695, 13, 19, 19, }, // Turkmenistan
    {  583, 1763,       0,  752523,       0,  0,  9,  0, }, // Tuvalu
    {  583, 1777,  565289,  565304,  568714, 15, 21, 21, }, // Uruguay
    {  583, 1792,  752532,  752544,  752562, 12, 18, 18, }, // Uzbekistan
    {  583, 1803,  752580,  752590,  752608, 10, 18, 18, }, // Vanuatu
    {  583, 1811,       0,  565445,       0,  0, 16,  0, }, // Venezuela
    {  583, 1821,  565524,  752626,  752648, 14, 22, 22, }, // Vladivostok
    {  583, 1833,  565594,  752670,  752690, 12, 20, 20, }, // Volgograd
    {  583, 1843,       0,  565616,       0,  0,  9,  0, }, // Vostok
    {  583, 1850,       0,  752710,       0,  0, 14,  0, }, // Wake
    {  583, 1855,       0,  752724,       0,  0, 28,  0, }, // Wallis
    {  583, 1862,  565736,  752752,  752770, 10, 18, 18, }, // Yakutsk
    {  583, 1870,  565810,  752788,  752812, 16, 24, 24, }, // Yekaterinburg
    {  583, 1892,       0,  752836,       0,  0,  8,  0, }, // Yukon
    {  586,    0,  752844,  752853,  752872,  9, 19, 15, }, // Acre Swiss German/Latin/Switzerland
    {  586,    5,       0,  752887,       0,  0, 18,  0, }, // Afghanistan
    {  586,   17,       0,  752905,       0,  0, 24,  0, }, // Africa_Central
    {  586,   32,       0,  752929,       0,  0, 22,  0, }, // Africa_Eastern
    {  586,   65,       0,  752951,       0,  0, 21,  0, }, // Africa_Southern
    {  586,   81,  752972,  752995,  753028, 23, 33, 29, }, // Africa_Western
    {  586,  107,  753057,  753068,  753089, 11, 21, 17, }, // Alaska
    {  586,  128,  753106,  753117,  753138, 11, 21, 17, }, // Almaty
    {  586,  135,  753155,  753168,  753191, 13, 23, 19, }, // Amazon
    {  586,  142,  753210,  753231,  753262, 21, 31, 27, }, // America_Central
    {  586,  627,  753289,  753311,  753343, 22, 32, 28, }, // Europe_Central
    {  586,  642,  753371,  753392,  753423, 21, 31, 27, }, // Europe_Eastern
    {  586,  680,  753450,  753472,  753504, 22, 32, 28, }, // Europe_Western
    {  586, 1305,  753532,  753545,  753568, 13, 23, 19, }, // Moscow
    {  589,    0,  753587,  753600,  753634, 13, 34, 20, }, // Acre Syriac/Syriac/Iraq
    {  589,    5,       0,  753654,       0,  0, 15,  0, }, // Afghanistan
    {  589,   17,       0,  753669,       0,  0, 19,  0, }, // Africa_Central
    {  589,   32,       0,  753688,       0,  0, 17,  0, }, // Africa_Eastern
    {  589,   65,       0,  753705,       0,  0, 27,  0, }, // Africa_Southern
    {  589,   81,  753732,  753749,  753776, 17, 27, 24, }, // Africa_Western
    {  589,  107,  753800,  753812,  753834, 12, 22, 22, }, // Alaska
    {  589,  128,  753856,  753868,  753890, 12, 22, 22, }, // Almaty
    {  589,  135,  753912,  753922,  753942, 10, 20, 20, }, // Amazon
    {  589,  142,  753962,  753989,  754026, 27, 37, 37, }, // America_Central
    {  589,  158,  754063,  754091,  754129, 28, 38, 38, }, // America_Eastern
    {  589,  174,  754167,  754194,  754231, 27, 37, 37, }, // America_Mountain
    {  589,  191,  754268,  754295,  754332, 27, 37, 37, }, // America_Pacific
    {  589,  207,  754369,  754381,  754403, 12, 22, 22, }, // Anadyr
    {  589,  214,  754425,  754435,  754455, 10, 20, 20, }, // Apia
    {  589,  219,  754475,  754485,  754506, 10, 21, 21, }, // Aqtau
    {  589,  225,  754527,  754538,  754559, 11, 21, 21, }, // Aqtobe
    {  589,  232,  754580,  754591,  754612, 11, 21, 21, }, // Arabian
    {  589,  240,  754633,  754647,  754671, 14, 24, 24, }, // Argentina
    {  589,  250,  754695,  754717,  754749, 22, 32, 32, }, // Argentina_Western
    {  589,  268,  754781,  754793,  754815, 12, 22, 22, }, // Armenia
    {  589,  286,  754837,  754866,  754905, 29, 39, 39, }, // Atlantic
    {  589,  295,  754944,  754965,  754996, 21, 31, 31, }, // Australia_Central
    {  589,  313,  755027,  755056,  755095, 29, 39, 39, }, // Australia_CentralWestern
    {  589,  338,  755134,  755156,  755188, 22, 32, 32, }, // Australia_Eastern
    {  589,  356,  755220,  755242,  755274, 22, 32, 32, }, // Australia_Western
    {  589,  374,  755306,  755320,  755344, 14, 24, 24, }, // Azerbaijan
    {  589,  385,  755368,  755378,  755398, 10, 20, 20, }, // Azores
    {  589,  397,  755418,  755432,  755456, 14, 24, 24, }, // Bangladesh
    {  589,  415,       0,  755480,       0,  0, 11,  0, }, // Bhutan
    {  589,  422,       0,  755491,       0,  0, 13,  0, }, // Bolivia
    {  589,  437,  755504,  755518,  755542, 14, 24, 24, }, // Brasilia
    {  589,  454,       0,  755566,       0,  0, 21,  0, }, // Brunei
    {  589,  461,  755587,  755602,  755627, 15, 25, 25, }, // Cape_Verde
    {  589,  472,       0,  755652,       0,  0, 11,  0, }, // Casey
    {  589,  478,       0,  755663,       0,  0, 23,  0, }, // Chamorro
    {  589,  487,  755686,  755697,  755718, 11, 21, 21, }, // Chatham
    {  589,  495,  755739,  755750,  755771, 11, 21, 21, }, // Chile
    {  589,  501,  755792,  755801,  755820,  9, 19, 19, }, // China
    {  589,  507,       0,  755839,       0,  0, 20,  0, }, // Christmas
    {  589,  517,       0,  755859,       0,  0, 19,  0, }, // Cocos
    {  589,  523,  755878,  755892,  755916, 14, 24, 24, }, // Colombia
    {  589,  532,  755940,  755956,  755982, 16, 26, 26, }, // Cook
    {  589,  537,  756008,  756018,  756038, 10, 20, 20, }, // Cuba
    {  589,  548,       0,  756058,       0,  0, 10,  0, }, // Davis
    {  589,  564,       0,  756068,       0,  0, 20,  0, }, // DumontDUrville
    {  589,  601,       0,  756088,       0,  0, 16,  0, }, // East_Timor
    {  589,  612,  756104,  756120,  756146, 16, 26, 26, }, // Easter
    {  589,  619,       0,  756172,       0,  0, 13,  0, }, // Ecuador
    {  589,  627,  756185,  756204,  756233, 19, 29, 29, }, // Europe_Central
    {  589,  642,  756262,  756282,  756312, 20, 30, 30, }, // Europe_Eastern
    {  589,  657,       0,  756342,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  589,  680,  756365,  756385,  756415, 20, 30, 30, }, // Europe_Western
    {  589,  695,  756445,  756465,  756495, 20, 30, 30, }, // Falkland
    {  589,  704,  756525,  756535,  756555, 10, 20, 20, }, // Fiji
    {  589,  709,       0,  756575,       0,  0, 20,  0, }, // French_Guiana
    {  589,  723,       0,  756595,       0,  0, 29,  0, }, // French_Southern
    {  589,  746,       0,  756624,       0,  0, 14,  0, }, // Galapagos
    {  589,  756,       0,  756638,       0,  0, 11,  0, }, // Gambier
    {  589,  764,  756649,  756661,  756683, 12, 22, 22, }, // Georgia
    {  589,  772,       0,  756705,       0,  0, 20,  0, }, // Gilbert_Islands
    {  589,  788,       0,  756725,       0,  0, 11,  0, }, // GMT
    {  589,  830,  756736,  756756,  756786, 20, 30, 30, }, // Greenland_Eastern
    {  589,  848,  756816,  756838,  756870, 22, 32, 32, }, // Greenland_Western
    {  589,  866,       0,  756902,       0,  0, 20,  0, }, // Guam
    {  589,  871,       0,  756922,       0,  0, 21,  0, }, // Gulf
    {  589,  876,       0,  756575,       0,  0, 12,  0, }, // Guyana
    {  589,  883,  756943,  756962,  756991, 19, 29, 29, }, // Hawaii_Aleutian
    {  589,  899,  757020,  757035,  757060, 15, 25, 25, }, // Hong_Kong
    {  589,  909,  757085,  757090,  757110,  5, 20, 20, }, // Hovd
    {  589,  914,       0,  757130,       0,  0, 20,  0, }, // India
    {  589,  920,       0,  757150,       0,  0, 20,  0, }, // Indian_Ocean
    {  589,  933,       0,  757170,       0,  0, 13,  0, }, // Indochina
    {  589,  943,       0,  757183,       0,  0, 20,  0, }, // Indonesia_Central
    {  589,  961,       0,  757203,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  589,  979,       0,  757224,       0,  0, 21,  0, }, // Indonesia_Western
    {  589,  997,  757245,  757256,  757277, 11, 21, 21, }, // Iran
    {  589, 1008,  757298,  757312,  757336, 14, 24, 24, }, // Irkutsk
    {  589, 1016,  757360,  757373,  757396, 13, 23, 23, }, // Israel
    {  589, 1023,  757419,  757428,  757447,  9, 19, 19, }, // Japan
    {  589, 1029,  757466,  757493,  757530, 27, 37, 37, }, // Kamchatka
    {  589, 1058,       0,  757567,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  589, 1077,       0,  757585,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  589, 1106,  757603,  757615,  757637, 12, 22, 22, }, // Korea
    {  589, 1112,       0,  757659,       0,  0, 11,  0, }, // Kosrae
    {  589, 1119,  757670,  757686,  757712, 16, 26, 26, }, // Krasnoyarsk
    {  589, 1151,       0,  757738,       0,  0, 16,  0, }, // Kyrgystan
    {  589, 1161,       0,  757754,       0,  0, 11,  0, }, // Lanka
    {  589, 1175,       0,  757765,       0,  0, 18,  0, }, // Line_Islands
    {  589, 1188,  757783,  757797,  757821, 14, 24, 24, }, // Lord_Howe
    {  589, 1198,  757845,  757856,  757877, 11, 21, 21, }, // Macau
    {  589, 1204,  757898,  757911,  757933, 13, 22, 22, }, // Magadan
    {  589, 1219,       0,  757955,       0,  0, 12,  0, }, // Malaysia
    {  589, 1228,       0,  757967,       0,  0, 21,  0, }, // Maldives
    {  589, 1237,       0,  757988,       0,  0, 15,  0, }, // Marquesas
    {  589, 1247,       0,  758003,       0,  0, 20,  0, }, // Marshall_Islands
    {  589, 1264,  758023,  758037,  758061, 14, 24, 24, }, // Mauritius
    {  589, 1274,       0,  758085,       0,  0, 12,  0, }, // Mawson
    {  589, 1281,  758097,  758116,  758145, 19, 29, 29, }, // Mexico_Pacific
    {  589, 1296,  758174,  758190,  758216, 16, 26, 26, }, // Mongolia
    {  589, 1305,  758242,  758253,  758274, 11, 21, 21, }, // Moscow
    {  589, 1312,       0,  758295,       0,  0, 13,  0, }, // Myanmar
    {  589, 1320,       0,  758308,       0,  0, 11,  0, }, // Nauru
    {  589, 1326,       0,  758319,       0,  0, 11,  0, }, // Nepal
    {  589, 1332,  758330,  758348,  758376, 18, 28, 28, }, // New_Caledonia
    {  589, 1346,  758404,  758419,  758444, 15, 25, 25, }, // New_Zealand
    {  589, 1358,  758469,  758485,  758511, 16, 26, 26, }, // Newfoundland
    {  589, 1371,       0,  758537,       0,  0, 10,  0, }, // Niue
    {  589, 1376,  758547,  758566,  758595, 19, 29, 29, }, // Norfolk
    {  589, 1384,  758624,  758645,  758676, 21, 31, 31, }, // Noronha
    {  589, 1392,       0,  758707,       0,  0, 28,  0, }, // North_Mariana
    {  589, 1406,  758735,  758751,  758777, 16, 26, 26, }, // Novosibirsk
    {  589, 1418,  758803,  758813,  758833, 10, 20, 20, }, // Omsk
    {  589, 1428,  758853,  758866,  758889, 13, 23, 23, }, // Pakistan
    {  589, 1437,       0,  758912,       0,  0, 10,  0, }, // Palau
    {  589, 1443,       0,  758922,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  589, 1460,  758944,  758956,  758978, 12, 22, 22, }, // Paraguay
    {  589, 1469,  759000,  759010,  759030, 10, 20, 20, }, // Peru
    {  589, 1474,  759050,  759065,  759090, 15, 25, 25, }, // Philippines
    {  589, 1486,       0,  759115,       0,  0, 20,  0, }, // Phoenix_Islands
    {  589, 1502,  759135,  759158,  759191, 23, 33, 33, }, // Pierre_Miquelon
    {  589, 1518,       0,  759224,       0,  0, 14,  0, }, // Pitcairn
    {  589, 1527,       0,  759238,       0,  0, 12,  0, }, // Ponape
    {  589, 1534,       0,  759250,       0,  0, 14,  0, }, // Pyongyang
    {  589, 1544,  759264,  759279,  759304, 15, 25, 25, }, // Qyzylorda
    {  589, 1554,       0,  759329,       0,  0, 13,  0, }, // Reunion
    {  589, 1562,       0,  759342,       0,  0, 12,  0, }, // Rothera
    {  589, 1570,  759354,  759367,  759390, 13, 23, 23, }, // Sakhalin
    {  589, 1579,  759413,  759424,  759445, 11, 21, 21, }, // Samara
    {  589, 1596,  759466,  759476,  759496, 10, 20, 20, }, // Samoa
    {  589, 1602,       0,  759516,       0,  0, 12,  0, }, // Seychelles
    {  589, 1624,       0,  759528,       0,  0, 24,  0, }, // Singapore
    {  589, 1634,       0,  759552,       0,  0, 20,  0, }, // Solomon
    {  589, 1642,       0,  759572,       0,  0, 21,  0, }, // South_Georgia
    {  589, 1656,       0,  759593,       0,  0, 13,  0, }, // Suriname
    {  589, 1676,       0,  759606,       0,  0, 10,  0, }, // Syowa
    {  589, 1682,       0,  759616,       0,  0, 11,  0, }, // Tahiti
    {  589, 1689,  759627,  759639,  759661, 12, 22, 22, }, // Taipei
    {  589, 1696,       0,  759683,       0,  0, 15,  0, }, // Tajikistan
    {  589, 1724,       0,  759698,       0,  0, 13,  0, }, // Tokelau
    {  589, 1732,  759711,  759721,  759741, 10, 20, 20, }, // Tonga
    {  589, 1738,       0,  759761,       0,  0, 10,  0, }, // Truk
    {  589, 1750,  759771,  759787,  759813, 16, 26, 26, }, // Turkmenistan
    {  589, 1763,       0,  759839,       0,  0, 12,  0, }, // Tuvalu
    {  589, 1777,  759851,  759865,  759889, 14, 24, 24, }, // Uruguay
    {  589, 1792,  759913,  759928,  759953, 15, 25, 25, }, // Uzbekistan
    {  589, 1803,  759978,  759991,  760014, 13, 23, 23, }, // Vanuatu
    {  589, 1811,       0,  760037,       0,  0, 13,  0, }, // Venezuela
    {  589, 1821,  760050,  760067,  760094, 17, 27, 27, }, // Vladivostok
    {  589, 1833,  760121,  760136,  760161, 15, 25, 25, }, // Volgograd
    {  589, 1843,       0,  760186,       0,  0, 12,  0, }, // Vostok
    {  589, 1850,       0,  760198,       0,  0, 18,  0, }, // Wake
    {  589, 1855,       0,  760216,       0,  0, 19,  0, }, // Wallis
    {  589, 1862,  760235,  760248,  760271, 13, 23, 23, }, // Yakutsk
    {  589, 1870,  760294,  760311,  760338, 17, 27, 27, }, // Yekaterinburg
    {  589, 1892,       0,  760365,       0,  0, 11,  0, }, // Yukon
    {  595,    5,       0,  760376,       0,  0, 16,  0, }, // Afghanistan Tajik/Cyrillic/Tajikistan
    {  595,   17,       0,  760392,       0,  0, 21,  0, }, // Africa_Central
    {  595,   32,       0,  760413,       0,  0, 19,  0, }, // Africa_Eastern
    {  595,   65,       0,  760432,       0,  0, 31,  0, }, // Africa_Southern
    {  595,   81,  760463,  760482,  760512, 19, 30, 30, }, // Africa_Western
    {  595,  107,  760542,  760554,  760577, 12, 23, 20, }, // Alaska
    {  595,  135,  760597,  760609,  760632, 12, 23, 23, }, // Amazon
    {  595,  142,  760655,  760668,  760692, 13, 24, 21, }, // America_Central
    {  595,  158,  760713,  760724,  760746, 11, 22, 19, }, // America_Eastern
    {  595,  174,  760765,  760775,  760796, 10, 21, 18, }, // America_Mountain
    {  595,  191,  760814,  760832,  760861, 18, 29, 26, }, // America_Pacific
    {  595,  214,  760887,  760897,  760918, 10, 21, 18, }, // Apia
    {  595,  232,  760936,  760951,  760977, 15, 26, 23, }, // Arabian
    {  595,  240,  761000,  761015,  761041, 15, 26, 26, }, // Argentina
    {  595,  250,  761067,  761089,  761122, 22, 33, 33, }, // Argentina_Western
    {  595,  268,  761155,  761171,  761198, 16, 27, 24, }, // Armenia
    {  595,  286,  761222,  761237,  761263, 15, 26, 23, }, // Atlantic
    {  595,  295,  761286,  761310,  761345, 24, 35, 31, }, // Australia_Central
    {  595,  313,  761376,  761407,  761448, 31, 41, 38, }, // Australia_CentralWestern
    {  595,  338,  761486,  761508,  761541, 22, 33, 30, }, // Australia_Eastern
    {  595,  356,  761571,  761593,  761626, 22, 33, 30, }, // Australia_Western
    {  595,  374,  761656,  761672,  761699, 16, 27, 27, }, // Azerbaijan
    {  595,  385,  761726,  761736,  761757, 10, 21, 21, }, // Azores
    {  595,  397,  761778,  761793,  761819, 15, 26, 26, }, // Bangladesh
    {  595,  415,       0,  761845,       0,  0, 11,  0, }, // Bhutan
    {  595,  422,       0,  761856,       0,  0, 13,  0, }, // Bolivia
    {  595,  437,  761869,  761883,  761908, 14, 25, 25, }, // Brasilia
    {  595,  454,       0,  761933,       0,  0, 23,  0, }, // Brunei
    {  595,  461,  761956,  761972,  761999, 16, 27, 27, }, // Cape_Verde
    {  595,  478,       0,  762026,       0,  0, 24,  0, }, // Chamorro
    {  595,  487,  762050,  762061,  762083, 11, 22, 19, }, // Chatham
    {  595,  495,  762102,  762112,  762133, 10, 21, 21, }, // Chile
    {  595,  501,  762154,  762163,  762183,  9, 20, 17, }, // China
    {  595,  507,       0,  762200,       0,  0, 25,  0, }, // Christmas
    {  595,  517,       0,  762225,       0,  0, 21,  0, }, // Cocos
    {  595,  523,  762246,  762260,  762285, 14, 25, 25, }, // Colombia
    {  595,  532,  762310,  762329,  762359, 19, 30, 36, }, // Cook
    {  595,  537,  762395,  762405,  762426, 10, 21, 18, }, // Cuba
    {  595,  548,       0,  762444,       0,  0, 11,  0, }, // Davis
    {  595,  564,       0,  762455,       0,  0, 19,  0, }, // DumontDUrville
    {  595,  601,       0,  762474,       0,  0, 18,  0, }, // East_Timor
    {  595,  612,  762492,  762511,  762541, 19, 30, 30, }, // Easter
    {  595,  619,       0,  762571,       0,  0, 13,  0, }, // Ecuador
    {  595,  627,  762584,  762606,  762639, 22, 33, 33, }, // Europe_Central
    {  595,  642,  762672,  762692,  762723, 20, 31, 31, }, // Europe_Eastern
    {  595,  657,       0,  762754,       0,  0, 19,  0, }, // Europe_Further_Eastern
    {  595,  680,  762773,  762793,  762824, 20, 31, 31, }, // Europe_Western
    {  595,  695,  762855,  762879,  762914, 24, 35, 35, }, // Falkland
    {  595,  704,  762949,  762959,  762980, 10, 21, 21, }, // Fiji
    {  595,  709,       0,  763001,       0,  0, 21,  0, }, // French_Guiana
    {  595,  723,       0,  763022,       0,  0, 35,  0, }, // French_Southern
    {  595,  746,       0,  763057,       0,  0, 15,  0, }, // Galapagos
    {  595,  756,       0,  763072,       0,  0, 12,  0, }, // Gambier
    {  595,  764,  763084,  763099,  763125, 15, 26, 26, }, // Georgia
    {  595,  772,       0,  763151,       0,  0, 23,  0, }, // Gilbert_Islands
    {  595,  788,       0,  763174,       0,  0, 20,  0, }, // GMT
    {  595,  830,  763194,  763217,  763251, 23, 34, 34, }, // Greenland_Eastern
    {  595,  848,  763285,  763308,  763342, 23, 34, 34, }, // Greenland_Western
    {  595,  871,       0,  763376,       0,  0, 28,  0, }, // Gulf
    {  595,  876,       0,  763404,       0,  0, 12,  0, }, // Guyana
    {  595,  883,  763416,  763435,  763465, 19, 30, 27, }, // Hawaii_Aleutian
    {  595,  899,  763492,  763505,  763529, 13, 24, 24, }, // Hong_Kong
    {  595,  909,  763553,  763563,  763584, 10, 21, 21, }, // Hovd
    {  595,  914,       0,  763605,       0,  0, 26,  0, }, // India
    {  595,  920,       0,  763631,       0,  0, 18,  0, }, // Indian_Ocean
    {  595,  933,       0,  763649,       0,  0, 14,  0, }, // Indochina
    {  595,  943,       0,  763663,       0,  0, 24,  0, }, // Indonesia_Central
    {  595,  961,       0,  763687,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  595,  979,       0,  763709,       0,  0, 22,  0, }, // Indonesia_Western
    {  595,  997,  763731,  763741,  763762, 10, 21, 18, }, // Iran
    {  595, 1008,  763780,  763793,  763817, 13, 24, 24, }, // Irkutsk
    {  595, 1016,  763841,  763853,  763876, 12, 23, 20, }, // Israel
    {  595, 1023,  763896,  763907,  763929, 11, 22, 20, }, // Japan
    {  595, 1047,       0,  763949,       0,  0, 16,  0, }, // Kazakhstan
    {  595, 1058,       0,  763965,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  595, 1077,       0,  763988,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  595, 1106,  764011,  764022,  764044, 11, 22, 19, }, // Korea
    {  595, 1112,       0,  764063,       0,  0, 12,  0, }, // Kosrae
    {  595, 1119,  764075,  764091,  764118, 16, 27, 27, }, // Krasnoyarsk
    {  595, 1151,       0,  764145,       0,  0, 17,  0, }, // Kyrgystan
    {  595, 1175,       0,  764162,       0,  0, 19,  0, }, // Line_Islands
    {  595, 1188,  764181,  764194,  764219, 13, 25, 22, }, // Lord_Howe
    {  595, 1204,  764241,  764254,  764278, 13, 24, 24, }, // Magadan
    {  595, 1219,       0,  764302,       0,  0, 14,  0, }, // Malaysia
    {  595, 1228,       0,  764316,       0,  0, 12,  0, }, // Maldives
    {  595, 1237,       0,  764328,       0,  0, 12,  0, }, // Marquesas
    {  595, 1247,       0,  764340,       0,  0, 23,  0, }, // Marshall_Islands
    {  595, 1264,  764363,  764377,  764402, 14, 25, 25, }, // Mauritius
    {  595, 1274,       0,  764427,       0,  0, 12,  0, }, // Mawson
    {  595, 1281,  764439,  764465,  764502, 26, 37, 34, }, // Mexico_Pacific
    {  595, 1296,  764536,  764552,  764579, 16, 27, 27, }, // Mongolia
    {  595, 1305,  764606,  764618,  764641, 12, 23, 23, }, // Moscow
    {  595, 1312,       0,  764664,       0,  0, 11,  0, }, // Myanmar
    {  595, 1320,       0,  764675,       0,  0, 11,  0, }, // Nauru
    {  595, 1326,       0,  764686,       0,  0, 11,  0, }, // Nepal
    {  595, 1332,  764697,  764717,  764748, 20, 31, 31, }, // New_Caledonia
    {  595, 1346,  764779,  764798,  764828, 19, 30, 27, }, // New_Zealand
    {  595, 1358,  764855,  764872,  764900, 17, 28, 25, }, // Newfoundland
    {  595, 1371,       0,  764925,       0,  0, 10,  0, }, // Niue
    {  595, 1376,  764935,  764956,  764988, 21, 32, 29, }, // Norfolk
    {  595, 1384,  765017,  765042,  765078, 25, 36, 36, }, // Noronha
    {  595, 1406,  765114,  765131,  765159, 17, 28, 28, }, // Novosibirsk
    {  595, 1418,  765187,  765197,  765217, 10, 20, 21, }, // Omsk
    {  595, 1428,  765238,  765252,  765277, 14, 25, 25, }, // Pakistan
    {  595, 1437,       0,  765302,       0,  0, 11,  0, }, // Palau
    {  595, 1443,       0,  765313,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  595, 1460,  765336,  765350,  765375, 14, 25, 25, }, // Paraguay
    {  595, 1469,  765400,  765410,  765431, 10, 21, 21, }, // Peru
    {  595, 1474,  765452,  765466,  765491, 14, 25, 25, }, // Philippines
    {  595, 1486,       0,  765516,       0,  0, 22,  0, }, // Phoenix_Islands
    {  595, 1502,  765538,  765564,  765601, 26, 37, 34, }, // Pierre_Miquelon
    {  595, 1518,       0,  765635,       0,  0, 13,  0, }, // Pitcairn
    {  595, 1527,       0,    5282,       0,  0, 11,  0, }, // Ponape
    {  595, 1534,       0,  765648,       0,  0, 12,  0, }, // Pyongyang
    {  595, 1554,       0,  765660,       0,  0, 13,  0, }, // Reunion
    {  595, 1562,       0,  765673,       0,  0, 12,  0, }, // Rothera
    {  595, 1570,  765685,  765698,  765722, 13, 24, 24, }, // Sakhalin
    {  595, 1596,  765746,  765757,  765779, 11, 22, 19, }, // Samoa
    {  595, 1602,       0,  765798,       0,  0, 12,  0, }, // Seychelles
    {  595, 1624,       0,  765810,       0,  0, 25,  0, }, // Singapore
    {  595, 1634,       0,  765835,       0,  0, 23,  0, }, // Solomon
    {  595, 1642,       0,  765858,       0,  0, 20,  0, }, // South_Georgia
    {  595, 1656,       0,  765878,       0,  0, 13,  0, }, // Suriname
    {  595, 1676,       0,  765891,       0,  0, 10,  0, }, // Syowa
    {  595, 1682,       0,  765901,       0,  0, 11,  0, }, // Tahiti
    {  595, 1689,  765912,  765924,  765947, 12, 23, 20, }, // Taipei
    {  595, 1696,       0,  765967,       0,  0, 16,  0, }, // Tajikistan
    {  595, 1724,       0,  765983,       0,  0, 13,  0, }, // Tokelau
    {  595, 1732,  765996,  766007,  766029, 11, 22, 22, }, // Tonga
    {  595, 1738,       0,  766051,       0,  0,  9,  0, }, // Truk
    {  595, 1750,  766060,  766078,  766107, 18, 29, 29, }, // Turkmenistan
    {  595, 1763,       0,  766136,       0,  0, 12,  0, }, // Tuvalu
    {  595, 1777,  766148,  766161,  766185, 13, 24, 24, }, // Uruguay
    {  595, 1792,  766209,  766225,  766252, 16, 27, 27, }, // Uzbekistan
    {  595, 1803,  766279,  766292,  766316, 13, 24, 24, }, // Vanuatu
    {  595, 1811,       0,  766340,       0,  0, 15,  0, }, // Venezuela
    {  595, 1821,  766355,  766372,  766400, 17, 28, 28, }, // Vladivostok
    {  595, 1833,  766428,  766443,  766469, 15, 26, 26, }, // Volgograd
    {  595, 1843,       0,  766495,       0,  0, 12,  0, }, // Vostok
    {  595, 1850,       0,  766507,       0,  0, 20,  0, }, // Wake
    {  595, 1855,       0,  766527,       0,  0, 22,  0, }, // Wallis
    {  595, 1862,  766549,  766561,  766583, 12, 22, 22, }, // Yakutsk
    {  595, 1870,  766605,  766623,  766652, 18, 29, 29, }, // Yekaterinburg
    {  595, 1892,       0,  766681,       0,  0, 10,  0, }, // Yukon
    {  596,    0,  766691,  766702,  766716, 11, 14, 16, }, // Acre Tamil/Tamil/India
    {  596,    5,       0,  766732,       0,  0, 20,  0, }, // Afghanistan
    {  596,   17,       0,  766752,       0,  0, 23,  0, }, // Africa_Central
    {  596,   32,       0,  766775,       0,  0, 24,  0, }, // Africa_Eastern
    {  596,   65,       0,  766799,       0,  0, 29,  0, }, // Africa_Southern
    {  596,   81,  766828,  766851,  766882, 23, 31, 28, }, // Africa_Western
    {  596,  107,  766910,  766923,  766944, 13, 21, 20, }, // Alaska
    {  596,  128,  766964,  766977,  766993, 13, 16, 18, }, // Almaty
    {  596,  135,  767011,  767024,  767045, 13, 21, 18, }, // Amazon
    {  596,  142,  767063,  767075,  767095, 12, 20, 19, }, // America_Central
    {  596,  158,  767114,  767131,  767156, 17, 25, 24, }, // America_Eastern
    {  596,  174,  767180,  767195,  767218, 15, 23, 22, }, // America_Mountain
    {  596,  191,  767240,  767253,  767274, 13, 21, 20, }, // America_Pacific
    {  596,  207,  767294,  767306,  767323, 12, 17, 19, }, // Anadyr
    {  596,  214,  767342,  767353,  767372, 11, 19, 18, }, // Apia
    {  596,  219,  767390,  767401,  767415, 11, 14, 16, }, // Aqtau
    {  596,  225,  767431,  767444,  767460, 13, 16, 18, }, // Aqtobe
    {  596,  232,  767478,  767490,  767510, 12, 20, 19, }, // Arabian
    {  596,  240,  767529,  767546,  767571, 17, 25, 22, }, // Argentina
    {  596,  250,  767593,  767621,  767657, 28, 36, 33, }, // Argentina_Western
    {  596,  268,  767690,  767704,  767726, 14, 22, 19, }, // Armenia
    {  596,  286,  767745,  767762,  767787, 17, 25, 24, }, // Atlantic
    {  596,  295,  767811,  767834,  767867, 23, 33, 32, }, // Australia_Central
    {  596,  313,  767899,  767935,  767979, 36, 44, 43, }, // Australia_CentralWestern
    {  596,  338,  768022,  768050,  768088, 28, 38, 37, }, // Australia_Eastern
    {  596,  356,  768125,  768152,  768189, 27, 37, 36, }, // Australia_Western
    {  596,  374,  768225,  768241,  768265, 16, 24, 21, }, // Azerbaijan
    {  596,  385,  768286,  768298,  768318, 12, 20, 18, }, // Azores
    {  596,  397,  768336,  768349,  768370, 13, 21, 18, }, // Bangladesh
    {  596,  415,       0,  768388,       0,  0, 12,  0, }, // Bhutan
    {  596,  422,       0,  768400,       0,  0, 14,  0, }, // Bolivia
    {  596,  437,  768414,  768430,  768454, 16, 24, 21, }, // Brasilia
    {  596,  454,       0,  768475,       0,  0, 23,  0, }, // Brunei
    {  596,  461,  768498,  768515,  768540, 17, 25, 22, }, // Cape_Verde
    {  596,  478,       0,  768562,       0,  0, 20,  0, }, // Chamorro
    {  596,  487,  768582,  768595,  768616, 13, 21, 20, }, // Chatham
    {  596,  495,  768636,  768646,  768664, 10, 18, 15, }, // Chile
    {  596,  501,  768679,  768688,  768705,  9, 17, 16, }, // China
    {  596,  507,       0,  768721,       0,  0, 22,  0, }, // Christmas
    {  596,  517,       0,  768743,       0,  0, 20,  0, }, // Cocos
    {  596,  523,  768763,  768778,  768801, 15, 23, 20, }, // Colombia
    {  596,  532,  768821,  768839,  768865, 18, 26, 27, }, // Cook
    {  596,  537,  768892,  768904,  768924, 12, 20, 19, }, // Cuba
    {  596,  548,       0,  768943,       0,  0, 12,  0, }, // Davis
    {  596,  564,       0,  768955,       0,  0, 27,  0, }, // DumontDUrville
    {  596,  601,       0,  768982,       0,  0, 20,  0, }, // East_Timor
    {  596,  612,  769002,  769019,  769044, 17, 25, 22, }, // Easter
    {  596,  619,       0,  769066,       0,  0, 14,  0, }, // Ecuador
    {  596,  627,  769080,  769101,  769130, 21, 29, 26, }, // Europe_Central
    {  596,  642,  769156,  769182,  769216, 26, 34, 31, }, // Europe_Eastern
    {  596,  657,       0,  769247,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  596,  680,  769273,  769298,  769331, 25, 33, 30, }, // Europe_Western
    {  596,  695,  769361,  769386,  769419, 25, 33, 30, }, // Falkland
    {  596,  704,  769449,  769460,  769479, 11, 19, 16, }, // Fiji
    {  596,  709,       0,  769495,       0,  0, 20,  0, }, // French_Guiana
    {  596,  723,       0,  769515,       0,  0, 39,  0, }, // French_Southern
    {  596,  746,       0,  769554,       0,  0, 13,  0, }, // Galapagos
    {  596,  756,       0,  769567,       0,  0, 15,  0, }, // Gambier
    {  596,  764,  769582,  769596,  769618, 14, 22, 19, }, // Georgia
    {  596,  772,       0,  769637,       0,  0, 23,  0, }, // Gilbert_Islands
    {  596,  788,       0,  769660,       0,  0, 23,  0, }, // GMT
    {  596,  830,  769683,  769709,  769743, 26, 34, 31, }, // Greenland_Eastern
    {  596,  848,  769774,  769799,  769832, 25, 33, 30, }, // Greenland_Western
    {  596,  866,       0,  769862,       0,  0, 12,  0, }, // Guam
    {  596,  871,       0,  769874,       0,  0, 21,  0, }, // Gulf
    {  596,  876,       0,  769504,       0,  0, 11,  0, }, // Guyana
    {  596,  883,  769895,  769915,  769943, 20, 28, 27, }, // Hawaii_Aleutian
    {  596,  899,  769970,  769984,  770006, 14, 22, 19, }, // Hong_Kong
    {  596,  909,  770025,  770037,  770057, 12, 20, 17, }, // Hovd
    {  596,  914,       0,  770074,       0,  0, 20,  0, }, // India
    {  596,  920,       0,  770094,       0,  0, 25,  0, }, // Indian_Ocean
    {  596,  933,       0,  770119,       0,  0, 14,  0, }, // Indochina
    {  596,  943,       0,  770133,       0,  0, 23,  0, }, // Indonesia_Central
    {  596,  961,       0,  770156,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  596,  979,       0,  770184,       0,  0, 27,  0, }, // Indonesia_Western
    {  596,  997,  770211,  770222,  770241, 11, 19, 18, }, // Iran
    {  596, 1008,  770259,  770276,  770301, 17, 25, 22, }, // Irkutsk
    {  596, 1016,  770323,  770336,  770357, 13, 21, 20, }, // Israel
    {  596, 1023,  770377,  770390,  770411, 13, 21, 20, }, // Japan
    {  596, 1029,  770431,  770465,  770502, 34, 37, 39, }, // Kamchatka
    {  596, 1047,       0,  770541,       0,  0, 15,  0, }, // Kazakhstan
    {  596, 1058,       0,  770556,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  596, 1077,       0,  770579,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  596, 1106,  770601,  770612,  770631, 11, 19, 18, }, // Korea
    {  596, 1112,       0,  770649,       0,  0, 12,  0, }, // Kosrae
    {  596, 1119,  770661,  770682,  770711, 21, 29, 26, }, // Krasnoyarsk
    {  596, 1151,       0,  770737,       0,  0, 18,  0, }, // Kyrgystan
    {  596, 1161,       0,  770755,       0,  0, 11,  0, }, // Lanka
    {  596, 1175,       0,  770766,       0,  0, 18,  0, }, // Line_Islands
    {  596, 1188,  770784,  770801,  770826, 17, 25, 24, }, // Lord_Howe
    {  596, 1198,  770850,  770863,  770879, 13, 16, 18, }, // Macau
    {  596, 1204,  770897,  770908,  770927, 11, 19, 16, }, // Magadan
    {  596, 1219,       0,  770943,       0,  0, 12,  0, }, // Malaysia
    {  596, 1228,       0,  770955,       0,  0, 18,  0, }, // Maldives
    {  596, 1237,       0,  770973,       0,  0, 18,  0, }, // Marquesas
    {  596, 1247,       0,  770991,       0,  0, 21,  0, }, // Marshall_Islands
    {  596, 1264,  771012,  771027,  771050, 15, 23, 20, }, // Mauritius
    {  596, 1274,       0,  771070,       0,  0, 11,  0, }, // Mawson
    {  596, 1281,  771081,  771104,  771135, 23, 31, 30, }, // Mexico_Pacific
    {  596, 1296,  771165,  771181,  771205, 16, 24, 21, }, // Mongolia
    {  596, 1305,  771226,  771238,  771258, 12, 20, 17, }, // Moscow
    {  596, 1312,       0,  771275,       0,  0, 15,  0, }, // Myanmar
    {  596, 1320,       0,  771290,       0,  0, 11,  0, }, // Nauru
    {  596, 1326,       0,  771301,       0,  0, 11,  0, }, // Nepal
    {  596, 1332,  771312,  771332,  771360, 20, 28, 25, }, // New_Caledonia
    {  596, 1346,  771385,  771403,  771429, 18, 26, 25, }, // New_Zealand
    {  596, 1358,  771454,  771478,  771510, 24, 32, 31, }, // Newfoundland
    {  596, 1371,       0,  771541,       0,  0, 10,  0, }, // Niue
    {  596, 1376,  771551,  771571,  771599, 20, 28, 27, }, // Norfolk
    {  596, 1384,  771626,  771653,  771689, 27, 36, 33, }, // Noronha
    {  596, 1392,       0,  771722,       0,  0, 26,  0, }, // North_Mariana
    {  596, 1406,  771748,  771768,  771796, 20, 28, 25, }, // Novosibirsk
    {  596, 1418,  771821,  771834,  771855, 13, 21, 18, }, // Omsk
    {  596, 1428,  771873,  771889,  771913, 16, 24, 21, }, // Pakistan
    {  596, 1437,       0,  771934,       0,  0, 11,  0, }, // Palau
    {  596, 1443,       0,  771945,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  596, 1460,  771968,  771981,  772002, 13, 21, 18, }, // Paraguay
    {  596, 1469,  772020,  772030,  772048, 10, 18, 15, }, // Peru
    {  596, 1474,  772063,  772079,  772103, 16, 24, 21, }, // Philippines
    {  596, 1486,       0,  772124,       0,  0, 23,  0, }, // Phoenix_Islands
    {  596, 1502,  772147,  772182,  772225, 35, 43, 42, }, // Pierre_Miquelon
    {  596, 1518,       0,  772267,       0,  0, 18,  0, }, // Pitcairn
    {  596, 1527,       0,  772285,       0,  0, 12,  0, }, // Ponape
    {  596, 1534,       0,  772297,       0,  0, 16,  0, }, // Pyongyang
    {  596, 1544,  772313,  772329,  772348, 16, 19, 21, }, // Qyzylorda
    {  596, 1554,       0,  772369,       0,  0, 15,  0, }, // Reunion
    {  596, 1562,       0,  772384,       0,  0, 12,  0, }, // Rothera
    {  596, 1570,  772396,  772408,  772428, 12, 20, 17, }, // Sakhalin
    {  596, 1579,  772445,  772455,  772468, 10, 13, 15, }, // Samara
    {  596, 1596,  772483,  772494,  772513, 11, 19, 18, }, // Samoa
    {  596, 1602,       0,  772531,       0,  0, 14,  0, }, // Seychelles
    {  596, 1624,       0,  772545,       0,  0, 25,  0, }, // Singapore
    {  596, 1634,       0,  772570,       0,  0, 20,  0, }, // Solomon
    {  596, 1642,       0,  772590,       0,  0, 21,  0, }, // South_Georgia
    {  596, 1656,       0,  772611,       0,  0, 14,  0, }, // Suriname
    {  596, 1676,       0,  772625,       0,  0, 12,  0, }, // Syowa
    {  596, 1682,       0,  772637,       0,  0, 11,  0, }, // Tahiti
    {  596, 1689,  772648,  772660,  772680, 12, 20, 19, }, // Taipei
    {  596, 1696,       0,  772699,       0,  0, 17,  0, }, // Tajikistan
    {  596, 1724,       0,  772716,       0,  0, 16,  0, }, // Tokelau
    {  596, 1732,  772732,  772744,  772764, 12, 20, 17, }, // Tonga
    {  596, 1738,       0,  772781,       0,  0, 10,  0, }, // Truk
    {  596, 1750,  772791,  772813,  772843, 22, 30, 27, }, // Turkmenistan
    {  596, 1763,       0,  772870,       0,  0, 12,  0, }, // Tuvalu
    {  596, 1777,  772882,  772895,  772916, 13, 21, 18, }, // Uruguay
    {  596, 1792,  772934,  772953,  772980, 19, 27, 24, }, // Uzbekistan
    {  596, 1803,  773004,  773019,  773042, 15, 23, 20, }, // Vanuatu
    {  596, 1811,       0,  773062,       0,  0, 14,  0, }, // Venezuela
    {  596, 1821,  773076,  773096,  773124, 20, 28, 25, }, // Vladivostok
    {  596, 1833,  773149,  773167,  773193, 18, 26, 23, }, // Volgograd
    {  596, 1843,       0,  773082,       0,  0, 14,  0, }, // Vostok
    {  596, 1850,       0,  773216,       0,  0, 15,  0, }, // Wake
    {  596, 1855,       0,  773231,       0,  0, 30,  0, }, // Wallis
    {  596, 1862,  773261,  773276,  773299, 15, 23, 20, }, // Yakutsk
    {  596, 1870,  773319,  773340,  773369, 21, 29, 26, }, // Yekaterinburg
    {  596, 1892,       0,  773395,       0,  0, 12,  0, }, // Yukon
    {  597,  914,       0,  770074,       0,  0, 20,  0, }, // India Tamil/Tamil/Malaysia
    {  597, 1219,       0,  770943,       0,  0, 12,  0, }, // Malaysia
    {  597, 1624,       0,  772545,       0,  0, 25,  0, }, // Singapore
    {  598,  914,       0,  770074,       0,  0, 20,  0, }, // India Tamil/Tamil/Singapore
    {  598, 1219,       0,  770943,       0,  0, 12,  0, }, // Malaysia
    {  598, 1624,       0,  772545,       0,  0, 25,  0, }, // Singapore
    {  600,  142,  773407,  773424,  773448, 17, 24, 32, }, // America_Central Taroko/Latin/Taiwan
    {  600,  158,  773480,  773497,  773521, 17, 24, 32, }, // America_Eastern
    {  600,  174,  773553,  773571,  773596, 18, 25, 28, }, // America_Mountain
    {  600,  191,  773624,  773639,  773669, 15, 30, 38, }, // America_Pacific
    {  600,  286,  773707,  773726,  773752, 19, 26, 34, }, // Atlantic
    {  600,  627,  773786,  773797,  773815, 11, 18, 26, }, // Europe_Central
    {  600,  642,  773841,  773852,  773870, 11, 18, 26, }, // Europe_Eastern
    {  600,  680,  773896,  773906,  773923, 10, 17, 25, }, // Europe_Western
    {  600,  788,       0,  773948,       0,  0, 20,  0, }, // GMT
    {  602,    0,  773968,  773978,  773997, 10, 19, 16, }, // Acre Tatar/Cyrillic/Russia
    {  602,    5,       0,  774013,       0,  0, 16,  0, }, // Afghanistan
    {  602,   17,       0,  774029,       0,  0, 18,  0, }, // Africa_Central
    {  602,   32,       0,  774047,       0,  0, 22,  0, }, // Africa_Eastern
    {  602,   65,       0,  774069,       0,  0, 20,  0, }, // Africa_Southern
    {  602,   81,  774089,  774111,  774142, 22, 31, 28, }, // Africa_Western
    {  602,  107,  774170,  774183,  774205, 13, 22, 19, }, // Alaska
    {  602,  135,  774224,  774239,  774263, 15, 24, 21, }, // Amazon
    {  602,  142,  774284,  774310,  774343, 26, 33, 32, }, // America_Central
    {  602,  158,  774375,  774405,  774442, 30, 37, 36, }, // America_Eastern
    {  602,  174,  774478,  774503,  774535, 25, 32, 31, }, // America_Mountain
    {  602,  191,  774566,  774597,  774635, 31, 38, 37, }, // America_Pacific
    {  602,  207,  774672,  774686,  774709, 14, 23, 20, }, // Anadyr
    {  602,  214,  774729,  774740,  774760, 11, 20, 17, }, // Apia
    {  602,  232,  774777,  774789,  774810, 12, 21, 18, }, // Arabian
    {  602,  240,  774828,  774844,  774869, 16, 25, 22, }, // Argentina
    {  602,  250,  774891,  774916,  774950, 25, 34, 31, }, // Argentina_Western
    {  602,  268,  774981,  774995,  775018, 14, 23, 20, }, // Armenia
    {  602,  286,  775038,  775068,  775105, 30, 37, 36, }, // Atlantic
    {  602,  295,  775141,  775162,  775192, 21, 30, 27, }, // Australia_Central
    {  602,  313,  775219,  775249,  775288, 30, 39, 36, }, // Australia_CentralWestern
    {  602,  338,  775324,  775349,  775383, 25, 34, 31, }, // Australia_Eastern
    {  602,  356,  775414,  775439,  775473, 25, 34, 31, }, // Australia_Western
    {  602,  374,  775504,  775521,  775547, 17, 26, 23, }, // Azerbaijan
    {  602,  385,  775570,  775591,  775621, 21, 30, 27, }, // Azores
    {  602,  397,  775648,  775664,  775689, 16, 25, 22, }, // Bangladesh
    {  602,  415,       0,  775711,       0,  0, 12,  0, }, // Bhutan
    {  602,  422,       0,  775723,       0,  0, 14,  0, }, // Bolivia
    {  602,  437,  775737,  775752,  775776, 15, 24, 21, }, // Brasilia
    {  602,  454,       0,  775797,       0,  0, 24,  0, }, // Brunei
    {  602,  461,  775821,  775838,  775864, 17, 26, 23, }, // Cape_Verde
    {  602,  478,       0,  775887,       0,  0, 23,  0, }, // Chamorro
    {  602,  487,  775910,  775922,  775943, 12, 21, 18, }, // Chatham
    {  602,  495,  775961,  775972,  775992, 11, 20, 17, }, // Chile
    {  602,  501,  776009,  776021,  776042, 12, 21, 18, }, // China
    {  602,  507,       0,  776060,       0,  0, 20,  0, }, // Christmas
    {  602,  517,       0,  776080,       0,  0, 22,  0, }, // Cocos
    {  602,  523,  776102,  776117,  776141, 15, 24, 21, }, // Colombia
    {  602,  532,  776162,  776182,  776211, 20, 29, 33, }, // Cook
    {  602,  537,  776244,  776255,  776275, 11, 20, 17, }, // Cuba
    {  602,  548,       0,  776292,       0,  0, 12,  0, }, // Davis
    {  602,  564,       0,  776304,       0,  0, 21,  0, }, // DumontDUrville
    {  602,  601,       0,  776325,       0,  0, 21,  0, }, // East_Timor
    {  602,  612,  776346,  776365,  776393, 19, 28, 25, }, // Easter
    {  602,  619,       0,  776418,       0,  0, 14,  0, }, // Ecuador
    {  602,  627,  776432,  776450,  776475, 18, 25, 24, }, // Europe_Central
    {  602,  642,  776499,  776521,  776550, 22, 29, 28, }, // Europe_Eastern
    {  602,  657,       0,  776578,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  602,  680,  776605,  776627,  776656, 22, 29, 28, }, // Europe_Western
    {  602,  695,  776684,  776709,  776743, 25, 34, 31, }, // Falkland
    {  602,  704,  776774,  776786,  776807, 12, 21, 18, }, // Fiji
    {  602,  709,       0,  776825,       0,  0, 21,  0, }, // French_Guiana
    {  602,  723,       0,  776846,       0,  0, 36,  0, }, // French_Southern
    {  602,  746,       0,  776882,       0,  0, 26,  0, }, // Galapagos
    {  602,  756,       0,  776908,       0,  0, 13,  0, }, // Gambier
    {  602,  764,  776921,  776934,  776956, 13, 22, 19, }, // Georgia
    {  602,  772,       0,  776975,       0,  0, 24,  0, }, // Gilbert_Islands
    {  602,  788,       0,  776999,       0,  0, 21,  0, }, // GMT
    {  602,  830,  777020,  777046,  777081, 26, 35, 32, }, // Greenland_Eastern
    {  602,  848,  777113,  777139,  777174, 26, 35, 32, }, // Greenland_Western
    {  602,  871,       0,  777206,       0,  0, 32,  0, }, // Gulf
    {  602,  876,       0,  777238,       0,  0, 13,  0, }, // Guyana
    {  602,  883,  777251,  777269,  777296, 18, 27, 24, }, // Hawaii_Aleutian
    {  602,  899,  777320,  777334,  777357, 14, 23, 20, }, // Hong_Kong
    {  602,  909,  777377,  777388,  777408, 11, 20, 17, }, // Hovd
    {  602,  914,       0,  777425,       0,  0, 20,  0, }, // India
    {  602,  920,       0,  777445,       0,  0, 18,  0, }, // Indian_Ocean
    {  602,  933,       0,  777463,       0,  0, 17,  0, }, // Indochina
    {  602,  943,       0,  777480,       0,  0, 21,  0, }, // Indonesia_Central
    {  602,  961,       0,  777501,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  602,  979,       0,  777526,       0,  0, 25,  0, }, // Indonesia_Western
    {  602,  997,  777551,  777562,  777582, 11, 20, 17, }, // Iran
    {  602, 1008,  777599,  777613,  777636, 14, 23, 20, }, // Irkutsk
    {  602, 1016,  777656,  777669,  777691, 13, 22, 19, }, // Israel
    {  602, 1023,  777710,  777721,  777741, 11, 20, 17, }, // Japan
    {  602, 1029,  777758,  777789,  777829, 31, 40, 37, }, // Kamchatka
    {  602, 1047,       0,  777866,       0,  0, 16,  0, }, // Kazakhstan
    {  602, 1058,       0,  777882,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {  602, 1077,       0,  777907,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  602, 1106,  777932,  777944,  777965, 12, 21, 18, }, // Korea
    {  602, 1112,       0,  777983,       0,  0, 13,  0, }, // Kosrae
    {  602, 1119,  777996,  778013,  778039, 17, 26, 23, }, // Krasnoyarsk
    {  602, 1151,       0,  778062,       0,  0, 17,  0, }, // Kyrgystan
    {  602, 1175,       0,  778079,       0,  0, 21,  0, }, // Line_Islands
    {  602, 1188,  778100,  778115,  778139, 15, 24, 21, }, // Lord_Howe
    {  602, 1204,  778160,  778174,  778197, 14, 23, 20, }, // Magadan
    {  602, 1219,       0,  778217,       0,  0, 15,  0, }, // Malaysia
    {  602, 1228,       0,  778232,       0,  0, 24,  0, }, // Maldives
    {  602, 1237,       0,  778256,       0,  0, 23,  0, }, // Marquesas
    {  602, 1247,       0,  778279,       0,  0, 24,  0, }, // Marshall_Islands
    {  602, 1264,  778303,  778318,  778342, 15, 24, 21, }, // Mauritius
    {  602, 1274,       0,  778363,       0,  0, 13,  0, }, // Mawson
    {  602, 1281,  778376,  778402,  778436, 26, 34, 32, }, // Mexico_Pacific
    {  602, 1296,  778468,  778485,  778511, 17, 26, 23, }, // Mongolia
    {  602, 1305,  778534,  778547,  778569, 13, 22, 19, }, // Moscow
    {  602, 1312,       0,  778588,       0,  0, 13,  0, }, // Myanmar
    {  602, 1320,       0,  778601,       0,  0, 12,  0, }, // Nauru
    {  602, 1326,       0,  778613,       0,  0, 12,  0, }, // Nepal
    {  602, 1332,  778625,  778645,  778674, 20, 29, 26, }, // New_Caledonia
    {  602, 1346,  778700,  778719,  778747, 19, 28, 25, }, // New_Zealand
    {  602, 1358,  778772,  778791,  778819, 19, 28, 25, }, // Newfoundland
    {  602, 1371,       0,  778844,       0,  0, 11,  0, }, // Niue
    {  602, 1376,  778855,  778876,  778906, 21, 30, 27, }, // Norfolk
    {  602, 1384,  778933,  778959,  778994, 26, 35, 32, }, // Noronha
    {  602, 1406,  779026,  779044,  779071, 18, 27, 24, }, // Novosibirsk
    {  602, 1418,  779095,  779106,  779126, 11, 20, 17, }, // Omsk
    {  602, 1428,  779143,  779158,  779182, 15, 24, 21, }, // Pakistan
    {  602, 1437,       0,  779203,       0,  0, 12,  0, }, // Palau
    {  602, 1443,       0,  779215,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  602, 1460,  779238,  779253,  779277, 15, 24, 21, }, // Paraguay
    {  602, 1469,  779298,  779309,  779329, 11, 20, 17, }, // Peru
    {  602, 1474,  779346,  779361,  779385, 15, 24, 21, }, // Philippines
    {  602, 1486,       0,  779406,       0,  0, 23,  0, }, // Phoenix_Islands
    {  602, 1502,  779429,  779456,  779492, 27, 36, 33, }, // Pierre_Miquelon
    {  602, 1518,       0,  779525,       0,  0, 14,  0, }, // Pitcairn
    {  602, 1527,       0,  779539,       0,  0, 13,  0, }, // Ponape
    {  602, 1534,       0,  779552,       0,  0, 14,  0, }, // Pyongyang
    {  602, 1554,       0,  779566,       0,  0, 14,  0, }, // Reunion
    {  602, 1562,       0,  779580,       0,  0, 13,  0, }, // Rothera
    {  602, 1570,  779593,  779607,  779630, 14, 23, 20, }, // Sakhalin
    {  602, 1579,  779650,  779663,  779685, 13, 22, 19, }, // Samara
    {  602, 1596,  779704,  779716,  779737, 12, 21, 18, }, // Samoa
    {  602, 1602,       0,  779755,       0,  0, 23,  0, }, // Seychelles
    {  602, 1624,       0,  779778,       0,  0, 24,  0, }, // Singapore
    {  602, 1634,       0,  779802,       0,  0, 24,  0, }, // Solomon
    {  602, 1642,       0,  779826,       0,  0, 22,  0, }, // South_Georgia
    {  602, 1656,       0,  779848,       0,  0, 14,  0, }, // Suriname
    {  602, 1676,       0,  779862,       0,  0, 11,  0, }, // Syowa
    {  602, 1682,       0,  779873,       0,  0, 12,  0, }, // Tahiti
    {  602, 1689,  779885,  779898,  779920, 13, 22, 19, }, // Taipei
    {  602, 1696,       0,  779939,       0,  0, 16,  0, }, // Tajikistan
    {  602, 1724,       0,  779955,       0,  0, 14,  0, }, // Tokelau
    {  602, 1732,  779969,  779981,  780002, 12, 21, 18, }, // Tonga
    {  602, 1738,       0,  780020,       0,  0, 11,  0, }, // Truk
    {  602, 1750,  780031,  780049,  780076, 18, 27, 24, }, // Turkmenistan
    {  602, 1763,       0,  780100,       0,  0, 13,  0, }, // Tuvalu
    {  602, 1777,  780113,  780127,  780150, 14, 23, 20, }, // Uruguay
    {  602, 1792,  780170,  780186,  780211, 16, 25, 22, }, // Uzbekistan
    {  602, 1803,  780233,  780247,  780270, 14, 23, 20, }, // Vanuatu
    {  602, 1811,       0,  780290,       0,  0, 16,  0, }, // Venezuela
    {  602, 1821,  780306,  780324,  780351, 18, 27, 24, }, // Vladivostok
    {  602, 1833,  780375,  780391,  780416, 16, 25, 22, }, // Volgograd
    {  602, 1843,       0,  780438,       0,  0, 13,  0, }, // Vostok
    {  602, 1850,       0,  780451,       0,  0, 18,  0, }, // Wake
    {  602, 1855,       0,  780469,       0,  0, 24,  0, }, // Wallis
    {  602, 1862,  780493,  780506,  780528, 13, 22, 19, }, // Yakutsk
    {  602, 1870,  780547,  780566,  780594, 19, 28, 25, }, // Yekaterinburg
    {  602, 1892,       0,  780619,       0,  0, 11,  0, }, // Yukon
    {  603,    0,  780630,  780639,  780658,  9, 19, 15, }, // Acre Telugu/Telugu/India
    {  603,    5,       0,  780673,       0,  0, 17,  0, }, // Afghanistan
    {  603,   17,       0,  780690,       0,  0, 21,  0, }, // Africa_Central
    {  603,   32,       0,  780711,       0,  0, 19,  0, }, // Africa_Eastern
    {  603,   65,       0,  780730,       0,  0, 29,  0, }, // Africa_Southern
    {  603,   81,  780759,  780778,  780807, 19, 29, 25, }, // Africa_Western
    {  603,  107,  780832,  780844,  780866, 12, 22, 26, }, // Alaska
    {  603,  128,  780892,  780904,  780926, 12, 22, 18, }, // Almaty
    {  603,  135,  780944,  780956,  780978, 12, 22, 18, }, // Amazon
    {  603,  142,  780996,  781006,  781026, 10, 20, 24, }, // America_Central
    {  603,  158,  781050,  781061,  781082, 11, 21, 25, }, // America_Eastern
    {  603,  174,  781107,  781121,  781145, 14, 24, 28, }, // America_Mountain
    {  603,  191,  781173,  781185,  781207, 12, 22, 26, }, // America_Pacific
    {  603,  207,  781233,  781244,  781268, 11, 24, 20, }, // Anadyr
    {  603,  214,  781288,  781298,  781318, 10, 20, 15, }, // Apia
    {  603,  219,  781333,  781345,  781367, 12, 22, 18, }, // Aqtau
    {  603,  225,  781385,  781397,  781419, 12, 22, 18, }, // Aqtobe
    {  603,  232,  781437,  781450,  781473, 13, 23, 27, }, // Arabian
    {  603,  240,  781500,  781515,  781540, 15, 25, 21, }, // Argentina
    {  603,  250,  781561,  781583,  781615, 22, 32, 28, }, // Argentina_Western
    {  603,  268,  781643,  781657,  781681, 14, 24, 20, }, // Armenia
    {  603,  286,  781701,  781716,  781741, 15, 25, 29, }, // Atlantic
    {  603,  295,  781770,  781792,  781824, 22, 32, 36, }, // Australia_Central
    {  603,  313,  781860,  781889,  781929, 29, 40, 43, }, // Australia_CentralWestern
    {  603,  338,  781972,  781995,  782029, 23, 34, 38, }, // Australia_Eastern
    {  603,  356,  782067,  781895,  782090, 23, 34, 38, }, // Australia_Western
    {  603,  374,  782128,  782143,  782168, 15, 25, 21, }, // Azerbaijan
    {  603,  385,  782189,  782201,  782223, 12, 22, 18, }, // Azores
    {  603,  397,  782241,  782256,  782281, 15, 25, 21, }, // Bangladesh
    {  603,  415,       0,  782302,       0,  0, 11,  0, }, // Bhutan
    {  603,  422,       0,  782313,       0,  0, 13,  0, }, // Bolivia
    {  603,  437,  782326,  782341,  782366, 15, 25, 21, }, // Brasilia
    {  603,  454,       0,  782387,       0,  0, 20,  0, }, // Brunei
    {  603,  461,  782407,  782423,  782449, 16, 26, 22, }, // Cape_Verde
    {  603,  478,       0,  782471,       0,  0, 22,  0, }, // Chamorro
    {  603,  487,  782493,  782503,  782523, 10, 20, 24, }, // Chatham
    {  603,  495,  782547,  782556,  782575,  9, 19, 15, }, // Chile
    {  603,  501,  782590,  782599,  782618,  9, 19, 23, }, // China
    {  603,  507,       0,  782641,       0,  0, 19,  0, }, // Christmas
    {  603,  517,       0,  782660,       0,  0, 17,  0, }, // Cocos
    {  603,  523,  782677,  782690,  782713, 13, 23, 19, }, // Colombia
    {  603,  532,  782732,  782747,  782772, 15, 25, 26, }, // Cook
    {  603,  537,  782798,  782809,  782830, 11, 21, 25, }, // Cuba
    {  603,  548,       0,  782855,       0,  0, 11,  0, }, // Davis
    {  603,  564,       0,  782866,       0,  0, 27,  0, }, // DumontDUrville
    {  603,  601,       0,  782893,       0,  0, 18,  0, }, // East_Timor
    {  603,  612,  782911,  782927,  782953, 16, 26, 22, }, // Easter
    {  603,  619,       0,  782975,       0,  0, 13,  0, }, // Ecuador
    {  603,  627,  782988,  783011,  783044, 23, 33, 29, }, // Europe_Central
    {  603,  642,  783073,  783094,  783125, 21, 31, 27, }, // Europe_Eastern
    {  603,  657,       0,  783152,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  603,  680,  783179,  783200,  783231, 21, 31, 27, }, // Europe_Western
    {  603,  695,  783258,  783281,  783314, 23, 33, 29, }, // Falkland
    {  603,  704,  783343,  783352,  783371,  9, 19, 15, }, // Fiji
    {  603,  709,       0,  783386,       0,  0, 18,  0, }, // French_Guiana
    {  603,  723,       0,  783404,       0,  0, 38,  0, }, // French_Southern
    {  603,  746,       0,  783442,       0,  0, 15,  0, }, // Galapagos
    {  603,  756,       0,  783457,       0,  0, 13,  0, }, // Gambier
    {  603,  764,  783470,  783483,  783506, 13, 23, 19, }, // Georgia
    {  603,  772,       0,  783525,       0,  0, 20,  0, }, // Gilbert_Islands
    {  603,  788,       0,  783545,       0,  0, 21,  0, }, // GMT
    {  603,  830,  783566,  783592,  783628, 26, 36, 32, }, // Greenland_Eastern
    {  603,  848,  783660,  783686,  783722, 26, 36, 32, }, // Greenland_Western
    {  603,  866,       0,  783754,       0,  0, 21,  0, }, // Guam
    {  603,  871,       0,  783775,       0,  0, 20,  0, }, // Gulf
    {  603,  876,       0,  783394,       0,  0, 10,  0, }, // Guyana
    {  603,  883,  783795,  783816,  783847, 21, 31, 35, }, // Hawaii_Aleutian
    {  603,  899,  783882,  783895,  783918, 13, 23, 19, }, // Hong_Kong
    {  603,  909,  783937,  783948,  783969, 11, 21, 17, }, // Hovd
    {  603,  914,       0,  783986,       0,  0, 22,  0, }, // India
    {  603,  920,       0,  784008,       0,  0, 21,  0, }, // Indian_Ocean
    {  603,  933,       0,  784029,       0,  0, 13,  0, }, // Indochina
    {  603,  943,       0,  784042,       0,  0, 24,  0, }, // Indonesia_Central
    {  603,  961,       0,  784066,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  603,  979,       0,  784088,       0,  0, 22,  0, }, // Indonesia_Western
    {  603,  997,  784110,  784120,  784140, 10, 20, 24, }, // Iran
    {  603, 1008,  784164,  784180,  784206, 16, 26, 22, }, // Irkutsk
    {  603, 1016,  784228,  784242,  784266, 14, 24, 28, }, // Israel
    {  603, 1023,  784294,  784304,  784324, 10, 20, 24, }, // Japan
    {  603, 1029,  784348,  784384,  784430, 36, 46, 42, }, // Kamchatka
    {  603, 1047,       0,  784472,       0,  0, 15,  0, }, // Kazakhstan
    {  603, 1058,       0,  784487,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  603, 1077,       0,  784510,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  603, 1106,  784532,  784544,  784566, 12, 22, 26, }, // Korea
    {  603, 1112,       0,  784592,       0,  0, 13,  0, }, // Kosrae
    {  603, 1119,  784605,  784626,  784657, 21, 31, 27, }, // Krasnoyarsk
    {  603, 1151,       0,  784684,       0,  0, 17,  0, }, // Kyrgystan
    {  603, 1161,       0,  784701,       0,  0,  9,  0, }, // Lanka
    {  603, 1175,       0,  784710,       0,  0, 15,  0, }, // Line_Islands
    {  603, 1188,  784725,  784741,  784767, 16, 26, 21, }, // Lord_Howe
    {  603, 1198,  784788,  784798,  784818, 10, 20, 16, }, // Macau
    {  603, 1204,  784834,  784845,  784866, 11, 21, 17, }, // Magadan
    {  603, 1219,       0,  784883,       0,  0, 12,  0, }, // Malaysia
    {  603, 1228,       0,  784895,       0,  0, 14,  0, }, // Maldives
    {  603, 1237,       0,  784909,       0,  0, 17,  0, }, // Marquesas
    {  603, 1247,       0,  784926,       0,  0, 18,  0, }, // Marshall_Islands
    {  603, 1264,  784944,  784956,  784978, 12, 22, 18, }, // Mauritius
    {  603, 1274,       0,  784996,       0,  0, 10,  0, }, // Mawson
    {  603, 1281,  785006,  785028,  785060, 22, 32, 36, }, // Mexico_Pacific
    {  603, 1296,  785096,  785111,  785136, 15, 25, 21, }, // Mongolia
    {  603, 1305,  785157,  785168,  785189, 11, 21, 17, }, // Moscow
    {  603, 1312,       0,  785206,       0,  0, 13,  0, }, // Myanmar
    {  603, 1320,       0,  785219,       0,  0,  9,  0, }, // Nauru
    {  603, 1326,       0,  785228,       0,  0, 11,  0, }, // Nepal
    {  603, 1332,  785239,  785259,  785289, 20, 30, 26, }, // New_Caledonia
    {  603, 1346,  785315,  785333,  785361, 18, 28, 32, }, // New_Zealand
    {  603, 1358,  785393,  785415,  785447, 22, 32, 36, }, // Newfoundland
    {  603, 1371,       0,  785483,       0,  0,  9,  0, }, // Niue
    {  603, 1376,  785492,  785510,  785538, 18, 28, 23, }, // Norfolk
    {  603, 1384,  785561,  785587,  785623, 26, 36, 32, }, // Noronha
    {  603, 1392,       0,  785655,       0,  0, 25,  0, }, // North_Mariana
    {  603, 1406,  785680,  785699,  785728, 19, 29, 25, }, // Novosibirsk
    {  603, 1418,  785753,  785765,  785787, 12, 22, 18, }, // Omsk
    {  603, 1428,  785805,  785820,  785845, 15, 25, 21, }, // Pakistan
    {  603, 1437,       0,  785866,       0,  0, 11,  0, }, // Palau
    {  603, 1443,       0,  785877,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  603, 1460,  785900,  785912,  785934, 12, 22, 18, }, // Paraguay
    {  603, 1469,  785952,  785961,  785980,  9, 19, 15, }, // Peru
    {  603, 1474,  785995,  786010,  786035, 15, 25, 21, }, // Philippines
    {  603, 1486,       0,  786056,       0,  0, 19,  0, }, // Phoenix_Islands
    {  603, 1502,  786075,  786111,  786157, 36, 46, 49, }, // Pierre_Miquelon
    {  603, 1518,       0,  786206,       0,  0, 16,  0, }, // Pitcairn
    {  603, 1527,       0,  786222,       0,  0, 11,  0, }, // Ponape
    {  603, 1534,       0,  786233,       0,  0, 15,  0, }, // Pyongyang
    {  603, 1544,  786248,  786263,  786288, 15, 25, 21, }, // Qyzylorda
    {  603, 1554,       0,  786309,       0,  0, 14,  0, }, // Reunion
    {  603, 1562,       0,  786323,       0,  0, 11,  0, }, // Rothera
    {  603, 1570,  786334,  786346,  786368, 12, 22, 18, }, // Sakhalin
    {  603, 1579,  786386,  786396,  786416, 10, 20, 16, }, // Samara
    {  603, 1596,  786432,  786442,  786462, 10, 20, 24, }, // Samoa
    {  603, 1602,       0,  786486,       0,  0, 13,  0, }, // Seychelles
    {  603, 1624,       0,  786499,       0,  0, 23,  0, }, // Singapore
    {  603, 1634,       0,  786522,       0,  0, 17,  0, }, // Solomon
    {  603, 1642,       0,  786539,       0,  0, 20,  0, }, // South_Georgia
    {  603, 1656,       0,  786559,       0,  0, 13,  0, }, // Suriname
    {  603, 1676,       0,  786572,       0,  0, 11,  0, }, // Syowa
    {  603, 1682,       0,  786583,       0,  0, 10,  0, }, // Tahiti
    {  603, 1689,  786593,  786602,  786621,  9, 19, 23, }, // Taipei
    {  603, 1696,       0,  786644,       0,  0, 16,  0, }, // Tajikistan
    {  603, 1724,       0,  786660,       0,  0, 13,  0, }, // Tokelau
    {  603, 1732,  786673,  786683,  786703, 10, 20, 16, }, // Tonga
    {  603, 1738,       0,  786719,       0,  0,  8,  0, }, // Truk
    {  603, 1750,  786727,  786749,  786781, 22, 32, 28, }, // Turkmenistan
    {  603, 1763,       0,  786809,       0,  0, 11,  0, }, // Tuvalu
    {  603, 1777,  786820,  786832,  786854, 12, 22, 18, }, // Uruguay
    {  603, 1792,  786872,  786890,  786918, 18, 28, 24, }, // Uzbekistan
    {  603, 1803,  786942,  786952,  786972, 10, 20, 16, }, // Vanuatu
    {  603, 1811,       0,  786988,       0,  0, 13,  0, }, // Venezuela
    {  603, 1821,  787001,  787020,  787049, 19, 29, 25, }, // Vladivostok
    {  603, 1833,  787074,  787091,  787118, 17, 27, 23, }, // Volgograd
    {  603, 1843,       0,  787007,       0,  0, 13,  0, }, // Vostok
    {  603, 1850,       0,  787141,       0,  0, 14,  0, }, // Wake
    {  603, 1855,       0,  787155,       0,  0, 24,  0, }, // Wallis
    {  603, 1862,  787179,  787194,  787219, 15, 25, 21, }, // Yakutsk
    {  603, 1870,  787240,  787260,  787290, 20, 30, 26, }, // Yekaterinburg
    {  603, 1892,       0,  787316,       0,  0, 11,  0, }, // Yukon
    {  606,    0,  787327,  787336,  787352,  9, 16, 16, }, // Acre Thai/Thai/Thailand
    {  606,    5,       0,  787368,       0,  0, 15,  0, }, // Afghanistan
    {  606,   17,       0,  787383,       0,  0, 15,  0, }, // Africa_Central
    {  606,   32,       0,  787398,       0,  0, 19,  0, }, // Africa_Eastern
    {  606,   65,       0,  787417,       0,  0, 14,  0, }, // Africa_Southern
    {  606,   81,  787431,  787449,  787474, 18, 25, 25, }, // Africa_Western
    {  606,  107,  787499,  787510,  787528, 11, 18, 20, }, // Alaska
    {  606,  128,  787548,  787559,  787577, 11, 18, 18, }, // Almaty
    {  606,  135,  787595,  787606,  787624, 11, 18, 18, }, // Amazon
    {  606,  142,  787642,  787667,  787699, 25, 32, 31, }, // America_Central
    {  606,  158,  787730,  787759,  787795, 29, 36, 35, }, // America_Eastern
    {  606,  174,  787830,  787856,  787889, 26, 33, 32, }, // America_Mountain
    {  606,  191,  787921,  787946,  787978, 25, 32, 31, }, // America_Pacific
    {  606,  207,  788009,  788021,  788040, 12, 19, 19, }, // Anadyr
    {  606,  214,  788059,  788069,  788086, 10, 17, 16, }, // Apia
    {  606,  219,  788102,  788113,  788131, 11, 18, 18, }, // Aqtau
    {  606,  225,  788149,  788160,  788178, 11, 18, 18, }, // Aqtobe
    {  606,  232,  788196,  788206,  788223, 10, 17, 16, }, // Arabian
    {  606,  240,  788239,  788254,  788276, 15, 22, 22, }, // Argentina
    {  606,  250,  788298,  788323,  788358, 25, 35, 35, }, // Argentina_Western
    {  606,  268,  788393,  788407,  788428, 14, 21, 21, }, // Armenia
    {  606,  286,  788449,  788462,  788482, 13, 20, 22, }, // Atlantic
    {  606,  295,  788504,  788522,  788556, 18, 34, 33, }, // Australia_Central
    {  606,  313,  788589,  788623,  788664, 34, 41, 40, }, // Australia_CentralWestern
    {  606,  338,  788704,  788726,  788761, 22, 35, 34, }, // Australia_Eastern
    {  606,  356,  788795,  788816,  788850, 21, 34, 33, }, // Australia_Western
    {  606,  374,  788883,  788899,  788922, 16, 23, 23, }, // Azerbaijan
    {  606,  385,  788945,  788956,  788974, 11, 18, 18, }, // Azores
    {  606,  397,  788992,  789005,  789025, 13, 20, 20, }, // Bangladesh
    {  606,  415,       0,  789045,       0,  0,  9,  0, }, // Bhutan
    {  606,  422,       0,  789054,       0,  0, 12,  0, }, // Bolivia
    {  606,  437,  789066,  789079,  789099, 13, 20, 20, }, // Brasilia
    {  606,  454,       0,  789119,       0,  0, 19,  0, }, // Brunei
    {  606,  461,  789138,  789151,  789171, 13, 20, 20, }, // Cape_Verde
    {  606,  472,       0,  789191,       0,  0, 10,  0, }, // Casey
    {  606,  478,       0,  789201,       0,  0, 12,  0, }, // Chamorro
    {  606,  487,  789213,  789222,  789238,  9, 16, 15, }, // Chatham
    {  606,  495,  789253,  789261,  789276,  8, 15, 15, }, // Chile
    {  606,  501,  789291,  789298,  789312,  7, 14, 13, }, // China
    {  606,  507,       0,  789325,       0,  0, 17,  0, }, // Christmas
    {  606,  517,       0,  789342,       0,  0, 17,  0, }, // Cocos
    {  606,  523,  789359,  789372,  789392, 13, 20, 20, }, // Colombia
    {  606,  532,  789412,  789427,  789449, 15, 22, 27, }, // Cook
    {  606,  537,  789476,  789485,  789501,  9, 16, 18, }, // Cuba
    {  606,  548,       0,  789519,       0,  0,  9,  0, }, // Davis
    {  606,  564,       0,  789528,       0,  0, 20,  0, }, // DumontDUrville
    {  606,  601,       0,  789548,       0,  0, 18,  0, }, // East_Timor
    {  606,  612,  789566,  789582,  789605, 16, 23, 23, }, // Easter
    {  606,  619,       0,  789628,       0,  0, 13,  0, }, // Ecuador
    {  606,  627,  789641,  789654,  789674, 13, 20, 20, }, // Europe_Central
    {  606,  642,  789694,  789711,  789735, 17, 24, 24, }, // Europe_Eastern
    {  606,  657,       0,  789759,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  606,  680,  789779,  789795,  789818, 16, 23, 23, }, // Europe_Western
    {  606,  695,  789841,  789863,  789892, 22, 29, 29, }, // Falkland
    {  606,  704,  789921,  789929,  789944,  8, 15, 15, }, // Fiji
    {  606,  709,       0,  789959,       0,  0, 16,  0, }, // French_Guiana
    {  606,  723,       0,  789975,       0,  0, 33,  0, }, // French_Southern
    {  606,  746,       0,  790008,       0,  0, 13,  0, }, // Galapagos
    {  606,  756,       0,  790021,       0,  0, 13,  0, }, // Gambier
    {  606,  764,  790034,  790046,  790065, 12, 19, 19, }, // Georgia
    {  606,  772,       0,  790084,       0,  0, 21,  0, }, // Gilbert_Islands
    {  606,  788,       0,  790105,       0,  0, 17,  0, }, // GMT
    {  606,  830,  790122,  790143,  790171, 21, 28, 28, }, // Greenland_Eastern
    {  606,  848,  790199,  790219,  790246, 20, 27, 27, }, // Greenland_Western
    {  606,  866,       0,  790273,       0,  0,  7,  0, }, // Guam
    {  606,  871,       0,  790280,       0,  0,  9,  0, }, // Gulf
    {  606,  876,       0,  790289,       0,  0, 11,  0, }, // Guyana
    {  606,  883,  790300,  790319,  790345, 19, 26, 25, }, // Hawaii_Aleutian
    {  606,  899,  790370,  790380,  790397, 10, 17, 17, }, // Hong_Kong
    {  606,  909,  790414,  790423,  790439,  9, 16, 16, }, // Hovd
    {  606,  914,       0,  790455,       0,  0, 11,  0, }, // India
    {  606,  920,       0,  790466,       0,  0, 19,  0, }, // Indian_Ocean
    {  606,  933,       0,  790485,       0,  0, 12,  0, }, // Indochina
    {  606,  943,       0,  790497,       0,  0, 22,  0, }, // Indonesia_Central
    {  606,  961,       0,  790519,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  606,  979,       0,  790546,       0,  0, 26,  0, }, // Indonesia_Western
    {  606,  997,  790572,  790583,  790601, 11, 18, 17, }, // Iran
    {  606, 1008,  790618,  790632,  790653, 14, 21, 21, }, // Irkutsk
    {  606, 1016,  790674,  790686,  790705, 12, 19, 18, }, // Israel
    {  606, 1023,  790723,  790734,  790752, 11, 18, 17, }, // Japan
    {  606, 1029,  790769,  790781,  790809, 12, 28, 35, }, // Kamchatka
    {  606, 1047,       0,  790844,       0,  0, 13,  0, }, // Kazakhstan
    {  606, 1058,       0,  790857,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  606, 1077,       0,  790878,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  606, 1106,  790898,  790908,  790925, 10, 17, 16, }, // Korea
    {  606, 1112,       0,  790941,       0,  0,  9,  0, }, // Kosrae
    {  606, 1119,  790950,  790965,  790987, 15, 22, 22, }, // Krasnoyarsk
    {  606, 1151,       0,  791009,       0,  0, 15,  0, }, // Kyrgystan
    {  606, 1161,       0,  791024,       0,  0,  9,  0, }, // Lanka
    {  606, 1175,       0,  791033,       0,  0, 16,  0, }, // Line_Islands
    {  606, 1188,  791049,  791062,  791082, 13, 20, 19, }, // Lord_Howe
    {  606, 1198,  791101,  791111,  791128, 10, 17, 17, }, // Macau
    {  606, 1204,  791145,  791156,  791174, 11, 18, 18, }, // Magadan
    {  606, 1219,       0,  791192,       0,  0, 12,  0, }, // Malaysia
    {  606, 1228,       0,  791204,       0,  0, 12,  0, }, // Maldives
    {  606, 1237,       0,  791216,       0,  0, 13,  0, }, // Marquesas
    {  606, 1247,       0,  791229,       0,  0, 21,  0, }, // Marshall_Islands
    {  606, 1264,  791250,  791263,  791283, 13, 20, 23, }, // Mauritius
    {  606, 1274,       0,  791306,       0,  0, 11,  0, }, // Mawson
    {  606, 1281,  791317,  791336,  791362, 19, 26, 25, }, // Mexico_Pacific
    {  606, 1296,  791387,  791402,  791424, 15, 22, 22, }, // Mongolia
    {  606, 1305,  791446,  791455,  791471,  9, 16, 16, }, // Moscow
    {  606, 1312,       0,  791487,       0,  0,  8,  0, }, // Myanmar
    {  606, 1320,       0,  791495,       0,  0, 10,  0, }, // Nauru
    {  606, 1326,       0,  791505,       0,  0,  9,  0, }, // Nepal
    {  606, 1332,  791514,  791531,  791555, 17, 24, 24, }, // New_Caledonia
    {  606, 1346,  791579,  791593,  791614, 14, 21, 20, }, // New_Zealand
    {  606, 1358,  791634,  791651,  791675, 17, 24, 23, }, // Newfoundland
    {  606, 1371,       0,  791698,       0,  0, 10,  0, }, // Niue
    {  606, 1376,  791708,  791725,  791749, 17, 24, 23, }, // Norfolk
    {  606, 1384,  791772,  791794,  791823, 22, 29, 32, }, // Noronha
    {  606, 1392,       0,  791855,       0,  0, 25,  0, }, // North_Mariana
    {  606, 1406,  791880,  791895,  791917, 15, 22, 22, }, // Novosibirsk
    {  606, 1418,  791939,  791949,  791966, 10, 17, 17, }, // Omsk
    {  606, 1428,  791983,  791995,  792014, 12, 19, 19, }, // Pakistan
    {  606, 1437,       0,  792033,       0,  0,  9,  0, }, // Palau
    {  606, 1443,       0,  792042,       0,  0, 16,  0, }, // Papua_New_Guinea
    {  606, 1460,  792058,  792070,  792089, 12, 19, 19, }, // Paraguay
    {  606, 1469,  792108,  792116,  792131,  8, 15, 15, }, // Peru
    {  606, 1474,  792146,  792160,  792181, 14, 21, 21, }, // Philippines
    {  606, 1486,       0,  792202,       0,  0, 19,  0, }, // Phoenix_Islands
    {  606, 1502,  792221,  792244,  792274, 23, 30, 32, }, // Pierre_Miquelon
    {  606, 1518,       0,  792306,       0,  0, 12,  0, }, // Pitcairn
    {  606, 1527,       0,  792318,       0,  0, 10,  0, }, // Ponape
    {  606, 1534,       0,  792328,       0,  0, 12,  0, }, // Pyongyang
    {  606, 1544,  792340,  792356,  792379, 16, 23, 23, }, // Qyzylorda
    {  606, 1554,       0,  792402,       0,  0, 12,  0, }, // Reunion
    {  606, 1562,       0,  792414,       0,  0, 10,  0, }, // Rothera
    {  606, 1570,  792424,  792435,  792453, 11, 18, 18, }, // Sakhalin
    {  606, 1579,  792471,  792481,  792498, 10, 17, 17, }, // Samara
    {  606, 1596,  792515,  792524,  792540,  9, 16, 16, }, // Samoa
    {  606, 1602,       0,  792556,       0,  0, 11,  0, }, // Seychelles
    {  606, 1624,       0,  792567,       0,  0, 12,  0, }, // Singapore
    {  606, 1634,       0,  792579,       0,  0, 19,  0, }, // Solomon
    {  606, 1642,       0,  792598,       0,  0, 17,  0, }, // South_Georgia
    {  606, 1656,       0,  792615,       0,  0, 12,  0, }, // Suriname
    {  606, 1676,       0,  792627,       0,  0, 10,  0, }, // Syowa
    {  606, 1682,       0,  792637,       0,  0, 10,  0, }, // Tahiti
    {  606, 1689,  792647,  792655,  792670,  8, 15, 14, }, // Taipei
    {  606, 1696,       0,  792684,       0,  0, 14,  0, }, // Tajikistan
    {  606, 1724,       0,  792698,       0,  0, 11,  0, }, // Tokelau
    {  606, 1732,  792709,  792718,  792734,  9, 16, 16, }, // Tonga
    {  606, 1738,       0,  792750,       0,  0,  7,  0, }, // Truk
    {  606, 1750,  792757,  792775,  792800, 18, 25, 25, }, // Turkmenistan
    {  606, 1763,       0,  792825,       0,  0, 10,  0, }, // Tuvalu
    {  606, 1777,  792835,  792847,  792866, 12, 19, 19, }, // Uruguay
    {  606, 1792,  792885,  792900,  792922, 15, 22, 22, }, // Uzbekistan
    {  606, 1803,  792944,  792956,  792975, 12, 19, 19, }, // Vanuatu
    {  606, 1811,       0,  792994,       0,  0, 14,  0, }, // Venezuela
    {  606, 1821,  793008,  793023,  793045, 15, 22, 22, }, // Vladivostok
    {  606, 1833,  793067,  793080,  793100, 13, 20, 20, }, // Volgograd
    {  606, 1843,       0,  793120,       0,  0, 10,  0, }, // Vostok
    {  606, 1850,       0,  793130,       0,  0, 11,  0, }, // Wake
    {  606, 1855,       0,  793141,       0,  0, 19,  0, }, // Wallis
    {  606, 1862,  793160,  793172,  793191, 12, 19, 19, }, // Yakutsk
    {  606, 1870,  793210,  793228,  793253, 18, 25, 25, }, // Yekaterinburg
    {  606, 1892,       0,  793278,       0,  0,  9,  0, }, // Yukon
    {  610,    0,  793287,  793293,  793307,  6, 14, 11, }, // Acre Tigrinya/Ethiopic/Ethiopia
    {  610,    5,       0,  793318,       0,  0, 13,  0, }, // Afghanistan
    {  610,   17,       0,  793331,       0,  0, 13,  0, }, // Africa_Central
    {  610,   32,       0,  793344,       0,  0, 12,  0, }, // Africa_Eastern
    {  610,   65,       0,  793356,       0,  0, 11,  0, }, // Africa_Southern
    {  610,   81,  793367,  793379,  793399, 12, 20, 17, }, // Africa_Western
    {  610,  107,  793416,  793423,  793438,  7, 15, 15, }, // Alaska
    {  610,  135,  793453,  793460,  793475,  7, 15, 12, }, // Amazon
    {  610,  142,  793487,  793500,  793521, 13, 21, 21, }, // America_Central
    {  610,  158,  793542,  793553,  793574, 11, 21, 21, }, // America_Eastern
    {  610,  174,  793595,  793603,  793621,  8, 18, 18, }, // America_Mountain
    {  610,  191,  793639,  793649,  793664, 10, 15, 15, }, // America_Pacific
    {  610,  214,  793679,  793688,  793702,  9, 14, 14, }, // Apia
    {  610,  232,  793716,  793725,  793739,  9, 14, 14, }, // Arabian
    {  610,  240,  793753,  793762,  793779,  9, 17, 14, }, // Argentina
    {  610,  250,  793793,  793808,  793831, 15, 23, 20, }, // Argentina_Western
    {  610,  268,  793851,  793862,  793878, 11, 16, 16, }, // Armenia
    {  610,  286,  793894,  793906,  793923, 12, 17, 17, }, // Atlantic
    {  610,  295,  793940,  793952,  793975, 12, 23, 23, }, // Australia_Central
    {  610,  313,  793998,  794016,  794039, 18, 23, 23, }, // Australia_CentralWestern
    {  610,  338,  794062,  794081,  794105, 19, 24, 24, }, // Australia_Eastern
    {  610,  356,  794129,  794148,  794172, 19, 24, 24, }, // Australia_Western
    {  610,  374,  794196,  794208,  794225, 12, 17, 17, }, // Azerbaijan
    {  610,  385,  794242,  794252,  794267, 10, 15, 15, }, // Azores
    {  610,  397,  794282,  794294,  794311, 12, 17, 17, }, // Bangladesh
    {  610,  415,       0,  794328,       0,  0, 10,  0, }, // Bhutan
    {  610,  422,       0,  794338,       0,  0,  7,  0, }, // Bolivia
    {  610,  437,  794345,  794353,  794369,  8, 16, 13, }, // Brasilia
    {  610,  454,       0,  794382,       0,  0, 15,  0, }, // Brunei
    {  610,  461,  794397,  794406,  794423,  9, 17, 14, }, // Cape_Verde
    {  610,  478,       0,  794437,       0,  0, 14,  0, }, // Chamorro
    {  610,  487,  794451,  794461,  794476, 10, 15, 15, }, // Chatham
    {  610,  495,  794491,  794496,  794509,  5, 13, 10, }, // Chile
    {  610,  501,  794519,  794528,  794542,  9, 14, 14, }, // China
    {  610,  507,       0,  794556,       0,  0, 13,  0, }, // Christmas
    {  610,  517,       0,  794569,       0,  0, 13,  0, }, // Cocos
    {  610,  523,  794582,  794590,  794606,  8, 16, 13, }, // Colombia
    {  610,  532,  794619,  794631,  794648, 12, 17, 21, }, // Cook
    {  610,  537,  794669,  794677,  794690,  8, 13, 13, }, // Cuba
    {  610,  548,       0,  794703,       0,  0,  9,  0, }, // Davis
    {  610,  564,       0,  794712,       0,  0, 13,  0, }, // DumontDUrville
    {  610,  601,       0,  794725,       0,  0, 14,  0, }, // East_Timor
    {  610,  612,  794739,  794749,  794767, 10, 18, 15, }, // Easter
    {  610,  619,       0,  794782,       0,  0,  7,  0, }, // Ecuador
    {  610,  627,  794789,  794805,  794826, 16, 21, 12, }, // Europe_Central
    {  610,  642,  794838,  794853,  794873, 15, 20, 17, }, // Europe_Eastern
    {  610,  657,       0,  794890,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  610,  680,  794910,  794927,  794949, 17, 22, 22, }, // Europe_Western
    {  610,  695,  794971,  794984,  795005, 13, 21, 18, }, // Falkland
    {  610,  704,  795023,  795031,  795044,  8, 13, 13, }, // Fiji
    {  610,  709,       0,  795057,       0,  0, 13,  0, }, // French_Guiana
    {  610,  723,       0,  795070,       0,  0, 18,  0, }, // French_Southern
    {  610,  746,       0,  795088,       0,  0,  8,  0, }, // Galapagos
    {  610,  756,       0,  795096,       0,  0, 11,  0, }, // Gambier
    {  610,  764,  795107,  795117,  795132, 10, 15, 15, }, // Georgia
    {  610,  772,       0,  795147,       0,  0, 16,  0, }, // Gilbert_Islands
    {  610,  788,       0,  202975,       0,  0,  3,  0, }, // GMT
    {  610,  830,  795163,  795181,  795204, 18, 23, 23, }, // Greenland_Eastern
    {  610,  848,  795227,  795244,  795266, 17, 22, 22, }, // Greenland_Western
    {  610,  871,       0,  795288,       0,  0, 14,  0, }, // Gulf
    {  610,  876,       0,  795302,       0,  0,  6,  0, }, // Guyana
    {  610,  883,  795308,  795324,  795345, 16, 21, 21, }, // Hawaii_Aleutian
    {  610,  899,  795366,  795379,  795397, 13, 18, 18, }, // Hong_Kong
    {  610,  909,  795415,  795424,  795438,  9, 14, 14, }, // Hovd
    {  610,  914,       0,  795452,       0,  0, 14,  0, }, // India
    {  610,  920,       0,  795466,       0,  0, 13,  0, }, // Indian_Ocean
    {  610,  933,       0,  795479,       0,  0, 12,  0, }, // Indochina
    {  610,  943,       0,  795491,       0,  0, 18,  0, }, // Indonesia_Central
    {  610,  961,       0,  795509,       0,  0, 18,  0, }, // Indonesia_Eastern
    {  610,  979,       0,  795527,       0,  0, 18,  0, }, // Indonesia_Western
    {  610,  997,  795545,  795554,  795568,  9, 14, 14, }, // Iran
    {  610, 1008,  795582,  795594,  795611, 12, 17, 17, }, // Irkutsk
    {  610, 1016,  795628,  795639,  795655, 11, 16, 16, }, // Israel
    {  610, 1023,  795671,  795680,  795694,  9, 14, 14, }, // Japan
    {  610, 1047,       0,  795708,       0,  0, 12,  0, }, // Kazakhstan
    {  610, 1058,       0,  795720,       0,  0, 17,  0, }, // Kazakhstan_Eastern
    {  610, 1077,       0,  795737,       0,  0, 17,  0, }, // Kazakhstan_Western
    {  610, 1106,  795754,  795764,  795779, 10, 15, 15, }, // Korea
    {  610, 1112,       0,  795794,       0,  0, 10,  0, }, // Kosrae
    {  610, 1119,  795804,  795818,  795837, 14, 19, 19, }, // Krasnoyarsk
    {  610, 1151,       0,  795856,       0,  0, 12,  0, }, // Kyrgystan
    {  610, 1175,       0,  795868,       0,  0, 14,  0, }, // Line_Islands
    {  610, 1188,  795882,  795894,  795911, 12, 17, 17, }, // Lord_Howe
    {  610, 1204,  795928,  795938,  795953, 10, 15, 15, }, // Magadan
    {  610, 1219,       0,  795968,       0,  0, 10,  0, }, // Malaysia
    {  610, 1228,       0,  795978,       0,  0, 11,  0, }, // Maldives
    {  610, 1237,       0,  795989,       0,  0, 12,  0, }, // Marquesas
    {  610, 1247,       0,  796001,       0,  0, 15,  0, }, // Marshall_Islands
    {  610, 1264,  796016,  796024,  796040,  8, 16, 13, }, // Mauritius
    {  610, 1274,       0,  796053,       0,  0, 10,  0, }, // Mawson
    {  610, 1281,  796063,  796078,  796098, 15, 20, 20, }, // Mexico_Pacific
    {  610, 1296,  796118,  796130,  796147, 12, 17, 17, }, // Mongolia
    {  610, 1305,  796164,  796174,  796189, 10, 15, 15, }, // Moscow
    {  610, 1312,       0,  796204,       0,  0, 11,  0, }, // Myanmar
    {  610, 1320,       0,  796215,       0,  0,  9,  0, }, // Nauru
    {  610, 1326,       0,  796224,       0,  0,  9,  0, }, // Nepal
    {  610, 1332,  796233,  796247,  796266, 14, 19, 19, }, // New_Caledonia
    {  610, 1346,  796285,  796298,  796316, 13, 18, 18, }, // New_Zealand
    {  610, 1358,  796334,  796349,  796369, 15, 20, 20, }, // Newfoundland
    {  610, 1371,       0,  796389,       0,  0,  8,  0, }, // Niue
    {  610, 1376,  796397,  796412,  796432, 15, 20, 20, }, // Norfolk
    {  610, 1384,  796452,  796467,  796490, 15, 23, 20, }, // Noronha
    {  610, 1406,  796510,  796522,  796539, 12, 17, 17, }, // Novosibirsk
    {  610, 1418,  796556,  796566,  796581, 10, 15, 15, }, // Omsk
    {  610, 1428,  796596,  796607,  796623, 11, 16, 16, }, // Pakistan
    {  610, 1437,       0,  796639,       0,  0,  9,  0, }, // Palau
    {  610, 1443,       0,  796648,       0,  0, 14,  0, }, // Papua_New_Guinea
    {  610, 1460,  796662,  796669,  796684,  7, 15, 12, }, // Paraguay
    {  610, 1469,  796696,  796701,  796714,  5, 13, 10, }, // Peru
    {  610, 1474,  796724,  796734,  796749, 10, 15, 15, }, // Philippines
    {  610, 1486,       0,  796764,       0,  0, 15,  0, }, // Phoenix_Islands
    {  610, 1502,  796779,  796798,  796822, 19, 24, 24, }, // Pierre_Miquelon
    {  610, 1518,       0,  796846,       0,  0, 12,  0, }, // Pitcairn
    {  610, 1527,       0,  796858,       0,  0,  9,  0, }, // Ponape
    {  610, 1534,       0,  796867,       0,  0, 13,  0, }, // Pyongyang
    {  610, 1554,       0,  796880,       0,  0,  8,  0, }, // Reunion
    {  610, 1562,       0,  796888,       0,  0,  9,  0, }, // Rothera
    {  610, 1570,  796897,  796908,  796924, 11, 16, 16, }, // Sakhalin
    {  610, 1596,  796940,  796949,  796963,  9, 14, 14, }, // Samoa
    {  610, 1602,       0,  796977,       0,  0,  7,  0, }, // Seychelles
    {  610, 1624,       0,  796984,       0,  0, 15,  0, }, // Singapore
    {  610, 1634,       0,  796999,       0,  0, 15,  0, }, // Solomon
    {  610, 1642,       0,  797014,       0,  0, 11,  0, }, // South_Georgia
    {  610, 1656,       0,  797025,       0,  0,  7,  0, }, // Suriname
    {  610, 1676,       0,  797032,       0,  0,  9,  0, }, // Syowa
    {  610, 1682,       0,  797041,       0,  0,  9,  0, }, // Tahiti
    {  610, 1689,  797050,  797058,  797071,  8, 13, 13, }, // Taipei
    {  610, 1696,       0,  797084,       0,  0, 12,  0, }, // Tajikistan
    {  610, 1724,       0,  797096,       0,  0, 10,  0, }, // Tokelau
    {  610, 1732,  797106,  797115,  797129,  9, 14, 14, }, // Tonga
    {  610, 1738,       0,  797143,       0,  0,  8,  0, }, // Truk
    {  610, 1750,  797151,  797165,  797184, 14, 19, 19, }, // Turkmenistan
    {  610, 1763,       0,  797203,       0,  0,  9,  0, }, // Tuvalu
    {  610, 1777,  797212,  797219,  797234,  7, 15, 12, }, // Uruguay
    {  610, 1792,  797246,  797259,  797277, 13, 18, 18, }, // Uzbekistan
    {  610, 1803,  797295,  797305,  797320, 10, 15, 15, }, // Vanuatu
    {  610, 1811,       0,  797335,       0,  0,  8,  0, }, // Venezuela
    {  610, 1821,  797343,  797356,  797374, 13, 18, 18, }, // Vladivostok
    {  610, 1833,  797392,  797404,  797421, 12, 17, 17, }, // Volgograd
    {  610, 1843,       0,  797438,       0,  0, 10,  0, }, // Vostok
    {  610, 1850,       0,  797448,       0,  0, 12,  0, }, // Wake
    {  610, 1855,       0,  797460,       0,  0, 13,  0, }, // Wallis
    {  610, 1862,  797473,  797484,  797500, 11, 16, 16, }, // Yakutsk
    {  610, 1870,  797516,  797530,  797549, 14, 19, 19, }, // Yekaterinburg
    {  610, 1892,       0,  797568,       0,  0,  9,  0, }, // Yukon
    {  613,    0,  797577,  797592,  797620, 15, 28, 27, }, // Acre Tongan/Latin/Tonga
    {  613,    5,       0,  797647,       0,  0, 23,  0, }, // Afghanistan
    {  613,   17,       0,  797670,       0,  0, 22,  0, }, // Africa_Central
    {  613,   32,       0,  797692,       0,  0, 24,  0, }, // Africa_Eastern
    {  613,   65,       0,  797716,       0,  0, 23,  0, }, // Africa_Southern
    {  613,   81,  797739,  797763,  797800, 24, 37, 36, }, // Africa_Western
    {  613,  107,  797836,  797853,  797883, 17, 30, 29, }, // Alaska
    {  613,  128,  797912,  797929,  797959, 17, 30, 29, }, // Almaty
    {  613,  135,  797988,  798005,  798035, 17, 30, 29, }, // Amazon
    {  613,  142,  798064,  798094,  798137, 30, 43, 42, }, // America_Central
    {  613,  158,  798179,  798211,  798256, 32, 45, 44, }, // America_Eastern
    {  613,  174,  798300,  798333,  798379, 33, 46, 45, }, // America_Mountain
    {  613,  191,  798424,  798458,  798505, 34, 47, 46, }, // America_Pacific
    {  613,  207,  798551,  798574,  798610, 23, 36, 35, }, // Anadyr
    {  613,  214,  798645,  798658,  798684, 13, 26, 25, }, // Apia
    {  613,  219,  798709,  798725,  798754, 16, 29, 28, }, // Aqtau
    {  613,  225,  798782,  798799,  798829, 17, 30, 29, }, // Aqtobe
    {  613,  232,  798858,  798874,  798903, 16, 29, 28, }, // Arabian
    {  613,  240,  798931,  798950,  798982, 19, 32, 31, }, // Argentina
    {  613,  250,  799013,  799039,  799078, 26, 39, 38, }, // Argentina_Western
    {  613,  268,  799116,  799132,  799161, 16, 29, 28, }, // Armenia
    {  613,  286,  799189,  799227,  799278, 38, 51, 50, }, // Atlantic
    {  613,  295,  799328,  799354,  799393, 26, 39, 38, }, // Australia_Central
    {  613,  313,  799431,  799464,  799510, 33, 46, 45, }, // Australia_CentralWestern
    {  613,  338,  799555,  799583,  799624, 28, 41, 40, }, // Australia_Eastern
    {  613,  356,  799664,  799692,  799733, 28, 41, 40, }, // Australia_Western
    {  613,  374,  799773,  799793,  799826, 20, 33, 32, }, // Azerbaijan
    {  613,  385,  799858,  799875,  799905, 17, 30, 29, }, // Azores
    {  613,  397,  799934,  799954,  799987, 20, 33, 32, }, // Bangladesh
    {  613,  415,       0,  800019,       0,  0, 15,  0, }, // Bhutan
    {  613,  422,       0,  800034,       0,  0, 16,  0, }, // Bolivia
    {  613,  437,  800050,  800067,  800097, 17, 30, 29, }, // Brasilia
    {  613,  454,       0,  800126,       0,  0, 16,  0, }, // Brunei
    {  613,  461,  800142,  800161,  800193, 19, 32, 31, }, // Cape_Verde
    {  613,  472,       0,  800224,       0,  0, 14,  0, }, // Casey
    {  613,  478,       0,  800238,       0,  0, 15,  0, }, // Chamorro
    {  613,  487,  800253,  800270,  800300, 17, 30, 29, }, // Chatham
    {  613,  495,  800329,  800342,  800368, 13, 26, 25, }, // Chile
    {  613,  501,  800393,  800408,  800436, 15, 28, 27, }, // China
    {  613,  507,       0,  800463,       0,  0, 23,  0, }, // Christmas
    {  613,  517,       0,  800486,       0,  0, 19,  0, }, // Cocos
    {  613,  523,  800505,  800523,  800554, 18, 31, 30, }, // Colombia
    {  613,  532,  800584,  800605,  800639, 21, 34, 33, }, // Cook
    {  613,  537,  800672,  800686,  800713, 14, 27, 26, }, // Cuba
    {  613,  548,       0,  800739,       0,  0, 15,  0, }, // Davis
    {  613,  564,       0,  800754,       0,  0, 22,  0, }, // DumontDUrville
    {  613,  601,       0,  800776,       0,  0, 21,  0, }, // East_Timor
    {  613,  612,  800797,  800813,  800842, 16, 29, 28, }, // Easter
    {  613,  619,       0,  800870,       0,  0, 17,  0, }, // Ecuador
    {  613,  627,  800887,  800908,  800942, 21, 34, 33, }, // Europe_Central
    {  613,  642,  800975,  800998,  801034, 23, 36, 35, }, // Europe_Eastern
    {  613,  657,       0,  801069,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {  613,  680,  801097,  801120,  801156, 23, 36, 35, }, // Europe_Western
    {  613,  695,  801191,  801217,  801256, 26, 39, 38, }, // Falkland
    {  613,  704,  801294,  801307,  801333, 13, 26, 25, }, // Fiji
    {  613,  709,       0,  801358,       0,  0, 28,  0, }, // French_Guiana
    {  613,  723,       0,  801386,       0,  0, 27,  0, }, // French_Southern
    {  613,  746,       0,  801413,       0,  0, 19,  0, }, // Galapagos
    {  613,  756,       0,  801432,       0,  0, 16,  0, }, // Gambier
    {  613,  764,  801448,  801463,  801491, 15, 28, 27, }, // Georgia
    {  613,  772,       0,  801518,       0,  0, 17,  0, }, // Gilbert_Islands
    {  613,  788,       0,  801535,       0,  0, 25,  0, }, // GMT
    {  613,  830,  801560,  801585,  801623, 25, 38, 37, }, // Greenland_Eastern
    {  613,  848,  801660,  801685,  801723, 25, 38, 37, }, // Greenland_Western
    {  613,  866,       0,  801760,       0,  0, 14,  0, }, // Guam
    {  613,  871,       0,  801774,       0,  0, 15,  0, }, // Gulf
    {  613,  876,       0,  801358,       0,  0, 15,  0, }, // Guyana
    {  613,  883,  801789,  801812,  801848, 23, 36, 35, }, // Hawaii_Aleutian
    {  613,  899,  801883,  801903,  801936, 20, 33, 32, }, // Hong_Kong
    {  613,  909,  801968,  801983,  802011, 15, 28, 27, }, // Hovd
    {  613,  914,       0,  802038,       0,  0, 16,  0, }, // India
    {  613,  920,       0,  802054,       0,  0, 21,  0, }, // Indian_Ocean
    {  613,  933,       0,  802075,       0,  0, 21,  0, }, // Indochina
    {  613,  943,       0,  802096,       0,  0, 25,  0, }, // Indonesia_Central
    {  613,  961,       0,  802121,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  613,  979,       0,  802148,       0,  0, 27,  0, }, // Indonesia_Western
    {  613,  997,  802175,  802191,  802220, 16, 29, 28, }, // Iran
    {  613, 1008,  802248,  802273,  802311, 25, 38, 37, }, // Irkutsk
    {  613, 1016,  802348,  802365,  802395, 17, 30, 29, }, // Israel
    {  613, 1023,  802424,  802440,  802469, 16, 29, 28, }, // Japan
    {  613, 1029,  802497,  802530,  802576, 33, 46, 45, }, // Kamchatka
    {  613, 1047,       0,  802621,       0,  0, 19,  0, }, // Kazakhstan
    {  613, 1058,       0,  802640,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  613, 1077,       0,  802666,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  613, 1106,  802692,  802706,  802733, 14, 27, 26, }, // Korea
    {  613, 1112,       0,  802759,       0,  0, 16,  0, }, // Kosrae
    {  613, 1119,  802775,  802804,  802846, 29, 42, 41, }, // Krasnoyarsk
    {  613, 1151,       0,  802887,       0,  0, 19,  0, }, // Kyrgystan
    {  613, 1161,       0,  802906,       0,  0, 16,  0, }, // Lanka
    {  613, 1175,       0,  802922,       0,  0, 22,  0, }, // Line_Islands
    {  613, 1188,  802944,  802966,  803001, 22, 35, 34, }, // Lord_Howe
    {  613, 1198,  803035,  803049,  803076, 14, 27, 26, }, // Macau
    {  613, 1204,  803102,  803125,  803161, 23, 36, 35, }, // Magadan
    {  613, 1219,       0,  803196,       0,  0, 17,  0, }, // Malaysia
    {  613, 1228,       0,  803213,       0,  0, 19,  0, }, // Maldives
    {  613, 1237,       0,  803232,       0,  0, 17,  0, }, // Marquesas
    {  613, 1247,       0,  803249,       0,  0, 23,  0, }, // Marshall_Islands
    {  613, 1264,  803272,  803291,  803323, 19, 32, 31, }, // Mauritius
    {  613, 1274,       0,  803354,       0,  0, 16,  0, }, // Mawson
    {  613, 1281,  803370,  803397,  803437, 27, 40, 39, }, // Mexico_Pacific
    {  613, 1296,  803476,  803495,  803527, 19, 32, 31, }, // Mongolia
    {  613, 1305,  803558,  803580,  803615, 22, 35, 34, }, // Moscow
    {  613, 1312,       0,  803649,       0,  0, 13,  0, }, // Myanmar
    {  613, 1320,       0,  803662,       0,  0, 14,  0, }, // Nauru
    {  613, 1326,       0,  803676,       0,  0, 15,  0, }, // Nepal
    {  613, 1332,  803691,  803714,  803750, 23, 36, 35, }, // New_Caledonia
    {  613, 1346,  803785,  803802,  803832, 17, 30, 29, }, // New_Zealand
    {  613, 1358,  803861,  803884,  803920, 23, 36, 35, }, // Newfoundland
    {  613, 1371,       0,  803955,       0,  0, 13,  0, }, // Niue
    {  613, 1376,  803968,  803984,  804013, 16, 29, 28, }, // Norfolk
    {  613, 1384,  804041,  804069,  804110, 28, 41, 40, }, // Noronha
    {  613, 1392,       0,  804150,       0,  0, 23,  0, }, // North_Mariana
    {  613, 1406,  804173,  804200,  804240, 27, 40, 39, }, // Novosibirsk
    {  613, 1418,  804279,  804302,  804338, 23, 36, 35, }, // Omsk
    {  613, 1428,  804373,  804392,  804424, 19, 32, 31, }, // Pakistan
    {  613, 1437,       0,  804455,       0,  0, 14,  0, }, // Palau
    {  613, 1443,       0,  804469,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  613, 1460,  804490,  804507,  804537, 17, 30, 29, }, // Paraguay
    {  613, 1469,  804566,  804579,  804605, 13, 26, 25, }, // Peru
    {  613, 1474,  804630,  804648,  804679, 18, 31, 30, }, // Philippines
    {  613, 1486,       0,  804709,       0,  0, 26,  0, }, // Phoenix_Islands
    {  613, 1502,  804735,  804763,  804804, 28, 41, 40, }, // Pierre_Miquelon
    {  613, 1518,       0,  804844,       0,  0, 17,  0, }, // Pitcairn
    {  613, 1527,       0,  804861,       0,  0, 15,  0, }, // Ponape
    {  613, 1534,       0,  804876,       0,  0, 19,  0, }, // Pyongyang
    {  613, 1544,  804895,  804912,  804942, 17, 30, 29, }, // Qyzylorda
    {  613, 1554,       0,  804971,       0,  0, 17,  0, }, // Reunion
    {  613, 1562,       0,  804988,       0,  0, 15,  0, }, // Rothera
    {  613, 1570,  805003,  805026,  805062, 23, 36, 35, }, // Sakhalin
    {  613, 1579,  805097,  805118,  805152, 21, 34, 33, }, // Samara
    {  613, 1596,  805185,  805201,  805230, 16, 29, 28, }, // Samoa
    {  613, 1602,       0,  805258,       0,  0, 25,  0, }, // Seychelles
    {  613, 1624,       0,  805283,       0,  0, 17,  0, }, // Singapore
    {  613, 1634,       0,  805300,       0,  0, 25,  0, }, // Solomon
    {  613, 1642,       0,  805325,       0,  0, 20,  0, }, // South_Georgia
    {  613, 1656,       0,  805345,       0,  0, 17,  0, }, // Suriname
    {  613, 1676,       0,  805362,       0,  0, 14,  0, }, // Syowa
    {  613, 1682,       0,  805376,       0,  0, 15,  0, }, // Tahiti
    {  613, 1689,  805391,  805406,  805434, 15, 28, 27, }, // Taipei
    {  613, 1696,       0,  805461,       0,  0, 19,  0, }, // Tajikistan
    {  613, 1724,       0,  805480,       0,  0, 16,  0, }, // Tokelau
    {  613, 1732,  805496,  805510,  805537, 14, 27, 26, }, // Tonga
    {  613, 1738,       0,  805563,       0,  0, 13,  0, }, // Truk
    {  613, 1750,  805576,  805599,  805635, 23, 36, 35, }, // Turkmenistan
    {  613, 1763,       0,  805670,       0,  0, 15,  0, }, // Tuvalu
    {  613, 1777,  805685,  805702,  805732, 17, 30, 29, }, // Uruguay
    {  613, 1792,  805761,  805782,  805816, 21, 34, 33, }, // Uzbekistan
    {  613, 1803,  805849,  805865,  805894, 16, 29, 28, }, // Vanuatu
    {  613, 1811,       0,  805922,       0,  0, 18,  0, }, // Venezuela
    {  613, 1821,  805940,  805969,  806011, 29, 42, 41, }, // Vladivostok
    {  613, 1833,  806052,  806079,  806119, 27, 40, 39, }, // Volgograd
    {  613, 1843,       0,  806158,       0,  0, 17,  0, }, // Vostok
    {  613, 1850,       0,  806175,       0,  0, 17,  0, }, // Wake
    {  613, 1855,       0,  806192,       0,  0, 24,  0, }, // Wallis
    {  613, 1862,  806216,  806242,  806281, 26, 39, 38, }, // Yakutsk
    {  613, 1870,  806319,  806349,  806392, 30, 43, 42, }, // Yekaterinburg
    {  613, 1892,       0,  806434,       0,  0, 15,  0, }, // Yukon
    {  615,  788,       0,  806449,       0,  0, 49,  0, }, // GMT Tswana/Latin/South Africa
    {  617,    0,  806498,  806508,  806527, 10, 19, 14, }, // Acre Turkish/Latin/Turkey
    {  617,    5,       0,  806541,       0,  0, 16,  0, }, // Afghanistan
    {  617,   17,       0,  806557,       0,  0, 17,  0, }, // Africa_Central
    {  617,   32,       0,  806574,       0,  0, 17,  0, }, // Africa_Eastern
    {  617,   65,       0,  806591,       0,  0, 27,  0, }, // Africa_Southern
    {  617,   81,  806618,  806635,  806661, 17, 26, 21, }, // Africa_Western
    {  617,  107,  806682,  806694,  806715, 12, 21, 16, }, // Alaska
    {  617,  128,  806731,  806743,  806764, 12, 21, 16, }, // Almaty
    {  617,  135,  806780,  806792,  806813, 12, 21, 16, }, // Amazon
    {  617,  142,  806829,  806856,  806892, 27, 36, 31, }, // America_Central
    {  617,  158,  806923,  806947,  806980, 24, 33, 28, }, // America_Eastern
    {  617,  174,  807008,  807031,  807063, 23, 32, 27, }, // America_Mountain
    {  617,  191,  807090,  807117,  807153, 27, 36, 31, }, // America_Pacific
    {  617,  207,  807184,  807196,  807217, 12, 21, 16, }, // Anadyr
    {  617,  214,  807233,  807243,  807262, 10, 19, 14, }, // Apia
    {  617,  219,  807276,  807287,  807307, 11, 20, 15, }, // Aqtau
    {  617,  225,  807322,  807334,  807355, 12, 21, 16, }, // Aqtobe
    {  617,  232,  807371,  807386,  807410, 15, 24, 19, }, // Arabian
    {  617,  240,  807429,  807443,  807466, 14, 23, 18, }, // Argentina
    {  617,  250,  807484,  807503,  807531, 19, 28, 23, }, // Argentina_Western
    {  617,  268,  807554,  807570,  807595, 16, 25, 20, }, // Armenia
    {  617,  286,  807615,  807629,  807652, 14, 23, 18, }, // Atlantic
    {  617,  295,  807670,  807691,  807721, 21, 30, 25, }, // Australia_Central
    {  617,  313,  807746,  807770,  807803, 24, 33, 28, }, // Australia_CentralWestern
    {  617,  338,  807831,  807852,  807882, 21, 30, 25, }, // Australia_Eastern
    {  617,  356,  807749,  807773,  807806, 21, 30, 25, }, // Australia_Western
    {  617,  374,  807907,  807923,  807948, 16, 25, 20, }, // Azerbaijan
    {  617,  385,  807968,  807981,  808003, 13, 22, 17, }, // Azores
    {  617,  397,  808020,  808035,  808059, 15, 24, 19, }, // Bangladesh
    {  617,  415,       0,  808078,       0,  0, 11,  0, }, // Bhutan
    {  617,  422,       0,  808089,       0,  0, 13,  0, }, // Bolivia
    {  617,  437,  808102,  808116,  808139, 14, 23, 18, }, // Brasilia
    {  617,  454,       0,  808157,       0,  0, 25,  0, }, // Brunei
    {  617,  461,  808182,  808198,  808223, 16, 25, 20, }, // Cape_Verde
    {  617,  472,       0,  808243,       0,  0, 11,  0, }, // Casey
    {  617,  478,       0,  808254,       0,  0, 14,  0, }, // Chamorro
    {  617,  487,  808268,  808281,  808303, 13, 22, 17, }, // Chatham
    {  617,  495,  808320,  808330,  808349, 10, 19, 14, }, // Chile
    {  617,  501,  808363,  808372,  808390,  9, 18, 13, }, // China
    {  617,  507,       0,  808403,       0,  0, 21,  0, }, // Christmas
    {  617,  517,       0,  808424,       0,  0, 19,  0, }, // Cocos
    {  617,  523,  808443,  808458,  808482, 15, 24, 19, }, // Colombia
    {  617,  532,  808501,  808519,  808546, 18, 27, 27, }, // Cook
    {  617,  537,  808573,  808583,  808602, 10, 19, 14, }, // Cuba
    {  617,  548,       0,  808616,       0,  0, 11,  0, }, // Davis
    {  617,  564,       0,  808627,       0,  0, 22,  0, }, // DumontDUrville
    {  617,  601,       0,  808649,       0,  0, 16,  0, }, // East_Timor
    {  617,  612,  808665,  808685,  808714, 20, 29, 24, }, // Easter
    {  617,  619,       0,  808738,       0,  0, 13,  0, }, // Ecuador
    {  617,  627,  808751,  808768,  808794, 17, 26, 21, }, // Europe_Central
    {  617,  642,  808815,  808832,  808858, 17, 26, 21, }, // Europe_Eastern
    {  617,  657,       0,  808879,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  617,  680,  808902,  808919,  808945, 17, 26, 21, }, // Europe_Western
    {  617,  695,  808966,  808988,  809019, 22, 31, 26, }, // Falkland
    {  617,  704,  809045,  809055,  809074, 10, 19, 14, }, // Fiji
    {  617,  709,       0,  809088,       0,  0, 22,  0, }, // French_Guiana
    {  617,  723,       0,  809110,       0,  0, 33,  0, }, // French_Southern
    {  617,  746,       0,  809143,       0,  0, 15,  0, }, // Galapagos
    {  617,  756,       0,  809158,       0,  0, 13,  0, }, // Gambier
    {  617,  764,  809171,  809186,  809210, 15, 24, 19, }, // Georgia
    {  617,  772,       0,  809229,       0,  0, 21,  0, }, // Gilbert_Islands
    {  617,  788,       0,  809250,       0,  0, 24,  0, }, // GMT
    {  617,  830,  809274,  809293,  809321, 19, 28, 23, }, // Greenland_Eastern
    {  617,  848,  809344,  809363,  809391, 19, 28, 23, }, // Greenland_Western
    {  617,  866,       0,  809414,       0,  0, 19,  0, }, // Guam
    {  617,  871,       0,  809433,       0,  0, 12,  0, }, // Gulf
    {  617,  876,       0,  809445,       0,  0, 12,  0, }, // Guyana
    {  617,  883,  809457,  809475,  809502, 18, 27, 22, }, // Hawaii_Aleutian
    {  617,  899,  809524,  809539,  809563, 15, 24, 19, }, // Hong_Kong
    {  617,  909,  809582,  809592,  809611, 10, 19, 14, }, // Hovd
    {  617,  914,       0,  809625,       0,  0, 24,  0, }, // India
    {  617,  920,       0,  809649,       0,  0, 19,  0, }, // Indian_Ocean
    {  617,  933,       0,  809668,       0,  0, 14,  0, }, // Indochina
    {  617,  943,       0,  809682,       0,  0, 20,  0, }, // Indonesia_Central
    {  617,  961,       0,  809702,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  617,  979,       0,  809722,       0,  0, 20,  0, }, // Indonesia_Western
    {  617,  997,  809742,  809752,  809771, 10, 19, 14, }, // Iran
    {  617, 1008,  809785,  809798,  809820, 13, 22, 17, }, // Irkutsk
    {  617, 1016,  809837,  809849,  809870, 12, 21, 16, }, // Israel
    {  617, 1023,  809886,  809899,  809921, 13, 22, 17, }, // Japan
    {  617, 1029,  809938,  809967,  810005, 29, 38, 33, }, // Kamchatka
    {  617, 1047,       0,  810038,       0,  0, 16,  0, }, // Kazakhstan
    {  617, 1058,       0,  810054,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  617, 1077,       0,  810075,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  617, 1106,  810096,  810106,  810125, 10, 19, 14, }, // Korea
    {  617, 1112,       0,  810139,       0,  0, 12,  0, }, // Kosrae
    {  617, 1119,  810151,  810168,  810194, 17, 26, 21, }, // Krasnoyarsk
    {  617, 1151,       0,  810215,       0,  0, 17,  0, }, // Kyrgystan
    {  617, 1161,       0,  810232,       0,  0, 11,  0, }, // Lanka
    {  617, 1175,       0,  810243,       0,  0, 18,  0, }, // Line_Islands
    {  617, 1188,  810261,  810276,  810300, 15, 24, 19, }, // Lord_Howe
    {  617, 1198,  810319,  810330,  810350, 11, 20, 15, }, // Macau
    {  617, 1204,  810365,  810378,  810400, 13, 22, 17, }, // Magadan
    {  617, 1219,       0,  810417,       0,  0, 13,  0, }, // Malaysia
    {  617, 1228,       0,  810430,       0,  0, 15,  0, }, // Maldives
    {  617, 1237,       0,  810445,       0,  0, 20,  0, }, // Marquesas
    {  617, 1247,       0,  810465,       0,  0, 22,  0, }, // Marshall_Islands
    {  617, 1264,  810487,  810502,  810526, 15, 24, 19, }, // Mauritius
    {  617, 1274,       0,  810545,       0,  0, 12,  0, }, // Mawson
    {  617, 1281,  810557,  810585,  810622, 28, 37, 32, }, // Mexico_Pacific
    {  617, 1296,  810654,  810670,  810695, 16, 25, 20, }, // Mongolia
    {  617, 1305,  810715,  810728,  810750, 13, 22, 17, }, // Moscow
    {  617, 1312,       0,  810767,       0,  0, 13,  0, }, // Myanmar
    {  617, 1320,       0,  810780,       0,  0, 11,  0, }, // Nauru
    {  617, 1326,       0,  810791,       0,  0, 11,  0, }, // Nepal
    {  617, 1332,  810802,  810822,  810851, 20, 29, 24, }, // New_Caledonia
    {  617, 1346,  810875,  810893,  810920, 18, 27, 22, }, // New_Zealand
    {  617, 1358,  810942,  810960,  810987, 18, 27, 22, }, // Newfoundland
    {  617, 1371,       0,  811009,       0,  0, 10,  0, }, // Niue
    {  617, 1376,  811019,  811038,  811066, 19, 28, 23, }, // Norfolk
    {  617, 1384,  811089,  811114,  811148, 25, 34, 29, }, // Noronha
    {  617, 1392,       0,  811177,       0,  0, 27,  0, }, // North_Mariana
    {  617, 1406,  811204,  811221,  811247, 17, 26, 21, }, // Novosibirsk
    {  617, 1418,  811268,  811278,  811297, 10, 19, 14, }, // Omsk
    {  617, 1428,  811311,  811325,  811348, 14, 23, 18, }, // Pakistan
    {  617, 1437,       0,  811366,       0,  0, 11,  0, }, // Palau
    {  617, 1443,       0,  811377,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  617, 1460,  811398,  811412,  811435, 14, 23, 18, }, // Paraguay
    {  617, 1469,  811453,  811463,  811482, 10, 19, 14, }, // Peru
    {  617, 1474,  811496,  811512,  811537, 16, 25, 20, }, // Philippines
    {  617, 1486,       0,  811557,       0,  0, 21,  0, }, // Phoenix_Islands
    {  617, 1502,  811578,  811608,  811647, 30, 39, 34, }, // Pierre_Miquelon
    {  617, 1518,       0,  811681,       0,  0, 14,  0, }, // Pitcairn
    {  617, 1527,       0,  811695,       0,  0, 12,  0, }, // Ponape
    {  617, 1534,       0,  811707,       0,  0, 15,  0, }, // Pyongyang
    {  617, 1544,  811722,  811737,  811761, 15, 24, 19, }, // Qyzylorda
    {  617, 1554,       0,  811780,       0,  0, 13,  0, }, // Reunion
    {  617, 1562,       0,  811793,       0,  0, 13,  0, }, // Rothera
    {  617, 1570,  811806,  811819,  811841, 13, 22, 17, }, // Sakhalin
    {  617, 1579,  811858,  811870,  811891, 12, 21, 16, }, // Samara
    {  617, 1596,  811907,  811918,  811938, 11, 20, 15, }, // Samoa
    {  617, 1602,       0,  811953,       0,  0, 15,  0, }, // Seychelles
    {  617, 1624,       0,  811968,       0,  0, 23,  0, }, // Singapore
    {  617, 1634,       0,  811991,       0,  0, 21,  0, }, // Solomon
    {  617, 1642,       0,  812012,       0,  0, 19,  0, }, // South_Georgia
    {  617, 1656,       0,  812031,       0,  0, 13,  0, }, // Suriname
    {  617, 1676,       0,  812044,       0,  0, 11,  0, }, // Syowa
    {  617, 1682,       0,  812055,       0,  0, 12,  0, }, // Tahiti
    {  617, 1689,  812067,  812079,  812100, 12, 21, 16, }, // Taipei
    {  617, 1696,       0,  812116,       0,  0, 16,  0, }, // Tajikistan
    {  617, 1724,       0,  812132,       0,  0, 13,  0, }, // Tokelau
    {  617, 1732,  812145,  812156,  812176, 11, 20, 15, }, // Tonga
    {  617, 1738,       0,  812191,       0,  0, 11,  0, }, // Truk
    {  617, 1750,  812202,  812220,  812247, 18, 27, 22, }, // Turkmenistan
    {  617, 1763,       0,  812269,       0,  0, 12,  0, }, // Tuvalu
    {  617, 1777,  812281,  812294,  812316, 13, 22, 17, }, // Uruguay
    {  617, 1792,  812333,  812349,  812374, 16, 25, 20, }, // Uzbekistan
    {  617, 1803,  812394,  812407,  812429, 13, 22, 17, }, // Vanuatu
    {  617, 1811,       0,  812446,       0,  0, 15,  0, }, // Venezuela
    {  617, 1821,  812461,  812478,  812504, 17, 26, 21, }, // Vladivostok
    {  617, 1833,  812525,  812540,  812564, 15, 24, 19, }, // Volgograd
    {  617, 1843,       0,  812583,       0,  0, 12,  0, }, // Vostok
    {  617, 1850,       0,  812595,       0,  0, 16,  0, }, // Wake
    {  617, 1855,       0,  812611,       0,  0, 22,  0, }, // Wallis
    {  617, 1862,  812633,  812646,  812668, 13, 22, 17, }, // Yakutsk
    {  617, 1870,  812685,  812704,  812732, 19, 28, 23, }, // Yekaterinburg
    {  617, 1892,       0,  812755,       0,  0, 11,  0, }, // Yukon
    {  619,    5,       0,  812766,       0,  0, 16,  0, }, // Afghanistan Turkmen/Latin/Turkmenistan
    {  619,   17,       0,  812782,       0,  0, 20,  0, }, // Africa_Central
    {  619,   32,       0,  812802,       0,  0, 21,  0, }, // Africa_Eastern
    {  619,   65,       0,  812823,       0,  0, 29,  0, }, // Africa_Southern
    {  619,   81,  812852,  812873,  812903, 21, 30, 29, }, // Africa_Western
    {  619,  107,  812932,  812945,  812967, 13, 22, 21, }, // Alaska
    {  619,  135,  812988,  813000,  813021, 12, 21, 20, }, // Amazon
    {  619,  142,  813041,  813056,  813086, 15, 30, 29, }, // America_Central
    {  619,  158,  813115,  813148,  813190, 33, 42, 41, }, // America_Eastern
    {  619,  174,  813231,  813259,  813296, 28, 37, 36, }, // America_Mountain
    {  619,  191,  813332,  813368,  813413, 36, 45, 44, }, // America_Pacific
    {  619,  207,  813457,       0,       0, 12,  0,  0, }, // Anadyr
    {  619,  214,  813469,  813479,  813498, 10, 19, 18, }, // Apia
    {  619,  232,  813516,  813535,  813563, 19, 28, 27, }, // Arabian
    {  619,  240,  813590,  813605,  813629, 15, 24, 23, }, // Argentina
    {  619,  250,  813652,  813676,  813709, 24, 33, 32, }, // Argentina_Western
    {  619,  268,  813741,  813757,  813782, 16, 25, 24, }, // Armenia
    {  619,  286,  813806,  813820,  813843, 14, 23, 22, }, // Atlantic
    {  619,  295,  813865,  813889,  813922, 24, 33, 32, }, // Australia_Central
    {  619,  313,  813954,  813987,  814029, 33, 42, 41, }, // Australia_CentralWestern
    {  619,  338,  814070,  814095,  814129, 25, 34, 33, }, // Australia_Eastern
    {  619,  356,  814162,  814187,  814221, 25, 34, 33, }, // Australia_Western
    {  619,  374,  814254,  814270,  814295, 16, 25, 24, }, // Azerbaijan
    {  619,  385,  814319,  814337,  814364, 18, 27, 26, }, // Azores
    {  619,  397,  814390,  814405,  814429, 15, 24, 23, }, // Bangladesh
    {  619,  415,       0,  814452,       0,  0, 11,  0, }, // Bhutan
    {  619,  422,       0,  814463,       0,  0, 14,  0, }, // Bolivia
    {  619,  437,  814477,  814492,  814516, 15, 24, 23, }, // Brasilia
    {  619,  454,       0,  814539,       0,  0, 23,  0, }, // Brunei
    {  619,  461,  814562,  814578,  814603, 16, 25, 24, }, // Cape_Verde
    {  619,  478,       0,  814627,       0,  0, 13,  0, }, // Chamorro
    {  619,  487,  814640,  814651,  814671, 11, 20, 19, }, // Chatham
    {  619,  495,  814690,  814700,  814719, 10, 19, 18, }, // Chile
    {  619,  501,  814737,  814748,  814768, 11, 20, 19, }, // China
    {  619,  507,       0,  814787,       0,  0, 21,  0, }, // Christmas
    {  619,  517,       0,  814808,       0,  0, 19,  0, }, // Cocos
    {  619,  523,  814827,  814842,  814866, 15, 24, 23, }, // Colombia
    {  619,  532,  814889,  814906,  814932, 17, 26, 25, }, // Cook
    {  619,  537,  814957,  814967,  814986, 10, 19, 18, }, // Cuba
    {  619,  548,       0,  815004,       0,  0, 12,  0, }, // Davis
    {  619,  564,       0,  815016,       0,  0, 21,  0, }, // DumontDUrville
    {  619,  601,       0,  815037,       0,  0, 20,  0, }, // East_Timor
    {  619,  612,  815057,  815074,  815100, 17, 26, 25, }, // Easter
    {  619,  619,       0,  815125,       0,  0, 13,  0, }, // Ecuador
    {  619,  627,  815138,  815159,  815189, 21, 30, 29, }, // Europe_Central
    {  619,  642,  815218,  815240,  815271, 22, 31, 30, }, // Europe_Eastern
    {  619,  657,       0,  815301,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  619,  680,  815328,  815350,  815381, 22, 31, 30, }, // Europe_Western
    {  619,  695,  815411,  815433,  815464, 22, 31, 30, }, // Falkland
    {  619,  704,  815494,  815504,  815523, 10, 19, 18, }, // Fiji
    {  619,  709,       0,  815541,       0,  0, 22,  0, }, // French_Guiana
    {  619,  723,       0,  815563,       0,  0, 43,  0, }, // French_Southern
    {  619,  746,       0,  815606,       0,  0, 23,  0, }, // Galapagos
    {  619,  756,       0,  815629,       0,  0, 12,  0, }, // Gambier
    {  619,  764,  815641,  815654,  815676, 13, 22, 21, }, // Georgia
    {  619,  772,       0,  815697,       0,  0, 21,  0, }, // Gilbert_Islands
    {  619,  788,       0,  815718,       0,  0, 20,  0, }, // GMT
    {  619,  830,  815738,  815764,  815799, 26, 35, 34, }, // Greenland_Eastern
    {  619,  848,  815833,  815859,  815894, 26, 35, 34, }, // Greenland_Western
    {  619,  871,       0,  815928,       0,  0, 26,  0, }, // Gulf
    {  619,  876,       0,  815954,       0,  0, 12,  0, }, // Guyana
    {  619,  883,  815966,  815983,  816009, 17, 26, 25, }, // Hawaii_Aleutian
    {  619,  899,  816034,  816047,  816069, 13, 22, 21, }, // Hong_Kong
    {  619,  909,  816090,  816100,  816119, 10, 19, 18, }, // Hovd
    {  619,  914,       0,  816137,       0,  0, 24,  0, }, // India
    {  619,  920,       0,  816161,       0,  0, 18,  0, }, // Indian_Ocean
    {  619,  933,       0,  816179,       0,  0, 16,  0, }, // Indochina
    {  619,  943,       0,  816195,       0,  0, 24,  0, }, // Indonesia_Central
    {  619,  961,       0,  816219,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  619,  979,       0,  816244,       0,  0, 25,  0, }, // Indonesia_Western
    {  619,  997,  816269,  816280,  816300, 11, 20, 19, }, // Iran
    {  619, 1008,  816319,  816332,  816354, 13, 22, 21, }, // Irkutsk
    {  619, 1016,  816375,  816388,  816410, 13, 22, 21, }, // Israel
    {  619, 1023,  816431,  816445,  816468, 14, 23, 22, }, // Japan
    {  619, 1029,  816490,       0,       0, 30,  0,  0, }, // Kamchatka
    {  619, 1047,       0,  816520,       0,  0, 16,  0, }, // Kazakhstan
    {  619, 1058,       0,  816536,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {  619, 1077,       0,  816561,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  619, 1106,  816586,  816598,  816619, 12, 21, 20, }, // Korea
    {  619, 1112,       0,  816639,       0,  0, 13,  0, }, // Kosrae
    {  619, 1119,  816652,  816669,  816695, 17, 26, 25, }, // Krasnoyarsk
    {  619, 1151,       0,  816720,       0,  0, 17,  0, }, // Kyrgystan
    {  619, 1175,       0,  816737,       0,  0, 18,  0, }, // Line_Islands
    {  619, 1188,  816755,  816769,  816792, 14, 23, 22, }, // Lord_Howe
    {  619, 1204,  816814,  816827,  816849, 13, 22, 21, }, // Magadan
    {  619, 1219,       0,  816870,       0,  0, 15,  0, }, // Malaysia
    {  619, 1228,       0,  816885,       0,  0, 15,  0, }, // Maldives
    {  619, 1237,       0,  816900,       0,  0, 20,  0, }, // Marquesas
    {  619, 1247,       0,  816920,       0,  0, 21,  0, }, // Marshall_Islands
    {  619, 1264,  816941,  816955,  816978, 14, 23, 22, }, // Mauritius
    {  619, 1274,       0,  817000,       0,  0, 12,  0, }, // Mawson
    {  619, 1281,  817012,  817038,  817073, 26, 35, 34, }, // Mexico_Pacific
    {  619, 1296,  817107,  817123,  817148, 16, 25, 24, }, // Mongolia
    {  619, 1305,  817172,  817184,  817205, 12, 21, 20, }, // Moscow
    {  619, 1312,       0,  817225,       0,  0, 12,  0, }, // Myanmar
    {  619, 1320,       0,  817237,       0,  0, 11,  0, }, // Nauru
    {  619, 1326,       0,  817248,       0,  0, 11,  0, }, // Nepal
    {  619, 1332,  817259,  817280,  817310, 21, 30, 29, }, // New_Caledonia
    {  619, 1346,  817339,  817359,  817388, 20, 29, 28, }, // New_Zealand
    {  619, 1358,  817416,  817434,  817461, 18, 27, 26, }, // Newfoundland
    {  619, 1371,       0,  817487,       0,  0, 10,  0, }, // Niue
    {  619, 1376,  817497,  817516,  817544, 19, 28, 27, }, // Norfolk
    {  619, 1384,  817571,  817596,  817630, 25, 34, 33, }, // Noronha
    {  619, 1406,  817663,  817680,  817706, 17, 26, 24, }, // Novosibirsk
    {  619, 1418,  817730,  817740,  817759, 10, 19, 18, }, // Omsk
    {  619, 1428,  817777,  817791,  817814, 14, 23, 22, }, // Pakistan
    {  619, 1437,       0,  817836,       0,  0, 11,  0, }, // Palau
    {  619, 1443,       0,  817847,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  619, 1460,  817873,  817887,  817910, 14, 23, 22, }, // Paraguay
    {  619, 1469,  817932,  817942,  817961, 10, 19, 18, }, // Peru
    {  619, 1474,  817979,  817996,  818022, 17, 26, 25, }, // Philippines
    {  619, 1486,       0,  818047,       0,  0, 20,  0, }, // Phoenix_Islands
    {  619, 1502,  818067,  818086,  818120, 19, 34, 33, }, // Pierre_Miquelon
    {  619, 1518,       0,  818153,       0,  0, 13,  0, }, // Pitcairn
    {  619, 1527,       0,  818166,       0,  0, 12,  0, }, // Ponape
    {  619, 1534,       0,  818178,       0,  0, 13,  0, }, // Pyongyang
    {  619, 1554,       0,  818191,       0,  0, 14,  0, }, // Reunion
    {  619, 1562,       0,  818205,       0,  0, 12,  0, }, // Rothera
    {  619, 1570,  818217,  818230,  818252, 13, 22, 21, }, // Sakhalin
    {  619, 1579,  818273,       0,       0, 12,  0,  0, }, // Samara
    {  619, 1596,  818285,  818296,  818316, 11, 20, 19, }, // Samoa
    {  619, 1602,       0,  818335,       0,  0, 20,  0, }, // Seychelles
    {  619, 1624,       0,  818355,       0,  0, 14,  0, }, // Singapore
    {  619, 1634,       0,  818369,       0,  0, 21,  0, }, // Solomon
    {  619, 1642,       0,  818390,       0,  0, 22,  0, }, // South_Georgia
    {  619, 1656,       0,  818412,       0,  0, 13,  0, }, // Suriname
    {  619, 1676,       0,  818425,       0,  0, 11,  0, }, // Syowa
    {  619, 1682,       0,  818436,       0,  0, 11,  0, }, // Tahiti
    {  619, 1689,  818447,  818459,  818480, 12, 21, 20, }, // Taipei
    {  619, 1696,       0,  818500,       0,  0, 16,  0, }, // Tajikistan
    {  619, 1724,       0,  818516,       0,  0, 13,  0, }, // Tokelau
    {  619, 1732,  818529,  818540,  818560, 11, 20, 19, }, // Tonga
    {  619, 1738,       0,  818579,       0,  0, 10,  0, }, // Truk
    {  619, 1750,  818589,  818607,  818634, 18, 27, 26, }, // Turkmenistan
    {  619, 1763,       0,  818660,       0,  0, 12,  0, }, // Tuvalu
    {  619, 1777,  818672,  818685,  818707, 13, 22, 21, }, // Uruguay
    {  619, 1792,  818728,  818744,  818769, 16, 25, 24, }, // Uzbekistan
    {  619, 1803,  818793,  818806,  818828, 13, 22, 21, }, // Vanuatu
    {  619, 1811,       0,  818849,       0,  0, 15,  0, }, // Venezuela
    {  619, 1821,  818864,  818881,  818907, 17, 26, 25, }, // Vladivostok
    {  619, 1833,  818932,  818947,  818971, 15, 24, 23, }, // Volgograd
    {  619, 1843,       0,  818994,       0,  0, 12,  0, }, // Vostok
    {  619, 1850,       0,  819006,       0,  0, 16,  0, }, // Wake
    {  619, 1855,       0,  819022,       0,  0, 22,  0, }, // Wallis
    {  619, 1862,  819044,  819057,  819079, 13, 22, 21, }, // Yakutsk
    {  619, 1870,  819100,  819119,  819147, 19, 28, 27, }, // Yekaterinburg
    {  619, 1892,       0,  819174,       0,  0, 11,  0, }, // Yukon
    {  621,    0,  819185,  819194,  819216,  9, 22, 17, }, // Acre Ukrainian/Cyrillic/Ukraine
    {  621,    5,       0,  819233,       0,  0, 22,  0, }, // Afghanistan
    {  621,   17,       0,  819255,       0,  0, 31,  0, }, // Africa_Central
    {  621,   32,       0,  819286,       0,  0, 27,  0, }, // Africa_Eastern
    {  621,   65,       0,  819313,       0,  0, 29,  0, }, // Africa_Southern
    {  621,   81,  819342,  819370,  819410, 28, 40, 35, }, // Africa_Western
    {  621,  107,  819445,  819463,  819493, 18, 30, 25, }, // Alaska
    {  621,  135,  819518,  819538,  819570, 20, 32, 27, }, // Amazon
    {  621,  142,  819597,  819639,  819693, 42, 54, 49, }, // America_Central
    {  621,  158,  819742,  819780,  819830, 38, 50, 45, }, // America_Eastern
    {  621,  174,  819875,  819914,  819965, 39, 51, 46, }, // America_Mountain
    {  621,  191,  820011,  820056,  820113, 45, 57, 52, }, // America_Pacific
    {  621,  207,  820165,  820176,  820200, 11, 24, 19, }, // Anadyr
    {  621,  214,  820219,  820234,  820261, 15, 27, 22, }, // Apia
    {  621,  232,  820283,  820301,  820331, 18, 30, 25, }, // Arabian
    {  621,  240,  820356,  820378,  820412, 22, 34, 29, }, // Argentina
    {  621,  250,  820441,  820470,  820511, 29, 41, 39, }, // Argentina_Western
    {  621,  268,  820550,  820570,  820602, 20, 32, 27, }, // Armenia
    {  621,  286,  820629,  820649,  820681, 20, 32, 27, }, // Atlantic
    {  621,  295,  820708,  820741,  820786, 33, 45, 40, }, // Australia_Central
    {  621,  313,  820826,  820868,  820922, 42, 54, 49, }, // Australia_CentralWestern
    {  621,  338,  820971,  821000,  821041, 29, 41, 36, }, // Australia_Eastern
    {  621,  356,  821077,  821107,  821149, 30, 42, 37, }, // Australia_Western
    {  621,  374,  821186,  821211,  821248, 25, 37, 32, }, // Azerbaijan
    {  621,  385,  821280,  821310,  821352, 30, 42, 37, }, // Azores
    {  621,  397,  821389,  821409,  821441, 20, 32, 27, }, // Bangladesh
    {  621,  415,       0,  821468,       0,  0, 17,  0, }, // Bhutan
    {  621,  422,       0,  821485,       0,  0, 21,  0, }, // Bolivia
    {  621,  437,  821506,  821527,  821560, 21, 33, 28, }, // Brasilia
    {  621,  454,       0,  821588,       0,  0, 17,  0, }, // Brunei
    {  621,  461,  821605,  821636,  821679, 31, 43, 38, }, // Cape_Verde
    {  621,  478,       0,  821717,       0,  0, 42,  0, }, // Chamorro
    {  621,  487,  821759,  821787,  821827, 28, 40, 35, }, // Chatham
    {  621,  495,  821862,  821881,  821912, 19, 31, 26, }, // Chile
    {  621,  501,  821938,  821957,  821988, 19, 31, 26, }, // China
    {  621,  507,       0,  822014,       0,  0, 26,  0, }, // Christmas
    {  621,  517,       0,  822040,       0,  0, 30,  0, }, // Cocos
    {  621,  523,  822070,  822092,  822126, 22, 34, 29, }, // Colombia
    {  621,  532,  822155,  822180,  822217, 25, 37, 32, }, // Cook
    {  621,  537,  822249,  822265,  822293, 16, 28, 23, }, // Cuba
    {  621,  548,       0,  822316,       0,  0, 25,  0, }, // Davis
    {  621,  564,       0,  822341,       0,  0, 34,  0, }, // DumontDUrville
    {  621,  601,       0,  822375,       0,  0, 26,  0, }, // East_Timor
    {  621,  612,  822401,  822426,  822463, 25, 37, 32, }, // Easter
    {  621,  619,       0,  822495,       0,  0, 19,  0, }, // Ecuador
    {  621,  627,  822514,  822545,  822588, 31, 43, 38, }, // Europe_Central
    {  621,  642,  822626,  822653,  822692, 27, 39, 34, }, // Europe_Eastern
    {  621,  657,       0,  822726,       0,  0, 35,  0, }, // Europe_Further_Eastern
    {  621,  680,  822761,  822789,  822829, 28, 40, 35, }, // Europe_Western
    {  621,  695,  822864,  822898,  822944, 34, 46, 41, }, // Falkland
    {  621,  704,  822985,  823001,  823029, 16, 28, 23, }, // Fiji
    {  621,  709,       0,  823052,       0,  0, 27,  0, }, // French_Guiana
    {  621,  723,       0,  823079,       0,  0, 59,  0, }, // French_Southern
    {  621,  746,       0,  823138,       0,  0, 19,  0, }, // Galapagos
    {  621,  756,       0,  823157,       0,  0, 26,  0, }, // Gambier
    {  621,  764,  823183,  823203,  823235, 20, 32, 27, }, // Georgia
    {  621,  772,       0,  823262,       0,  0, 29,  0, }, // Gilbert_Islands
    {  621,  788,       0,  823291,       0,  0, 12,  0, }, // GMT
    {  621,  830,  823303,  823332,  823373, 29, 41, 36, }, // Greenland_Eastern
    {  621,  848,  823409,  823439,  823481, 30, 42, 37, }, // Greenland_Western
    {  621,  866,       0,  823518,       0,  0, 24,  0, }, // Guam
    {  621,  871,       0,  823542,       0,  0, 24,  0, }, // Gulf
    {  621,  876,       0,  823566,       0,  0, 16,  0, }, // Guyana
    {  621,  883,  823582,  823611,  823652, 29, 41, 36, }, // Hawaii_Aleutian
    {  621,  899,  823688,  823707,  823738, 19, 31, 26, }, // Hong_Kong
    {  621,  909,  823764,  823780,  823808, 16, 28, 23, }, // Hovd
    {  621,  914,       0,  823831,       0,  0, 31,  0, }, // India
    {  621,  920,       0,  823862,       0,  0, 29,  0, }, // Indian_Ocean
    {  621,  933,       0,  823891,       0,  0, 20,  0, }, // Indochina
    {  621,  943,       0,  823911,       0,  0, 33,  0, }, // Indonesia_Central
    {  621,  961,       0,  823944,       0,  0, 29,  0, }, // Indonesia_Eastern
    {  621,  979,       0,  823973,       0,  0, 30,  0, }, // Indonesia_Western
    {  621,  997,  824003,  824021,  824051, 18, 30, 25, }, // Iran
    {  621, 1008,  824076,  824095,  824126, 19, 31, 26, }, // Irkutsk
    {  621, 1016,  824152,  824173,  824206, 21, 33, 28, }, // Israel
    {  621, 1023,  824234,  824252,  824282, 18, 30, 25, }, // Japan
    {  621, 1029,  824307,  824327,  824359, 20, 32, 27, }, // Kamchatka
    {  621, 1047,       0,  824386,       0,  0, 21,  0, }, // Kazakhstan
    {  621, 1058,       0,  824407,       0,  0, 29,  0, }, // Kazakhstan_Eastern
    {  621, 1077,       0,  824436,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  621, 1106,  824466,  824485,  824516, 19, 31, 26, }, // Korea
    {  621, 1112,       0,  824542,       0,  0, 26,  0, }, // Kosrae
    {  621, 1119,  824568,  824590,  824624, 22, 34, 29, }, // Krasnoyarsk
    {  621, 1151,       0,  824653,       0,  0, 22,  0, }, // Kyrgystan
    {  621, 1161,       0,  824675,       0,  0, 10,  0, }, // Lanka
    {  621, 1175,       0,  824685,       0,  0, 24,  0, }, // Line_Islands
    {  621, 1188,  824709,  824737,  824777, 28, 40, 35, }, // Lord_Howe
    {  621, 1204,  824812,  824833,  824866, 21, 33, 28, }, // Magadan
    {  621, 1219,       0,  824894,       0,  0, 19,  0, }, // Malaysia
    {  621, 1228,       0,  824913,       0,  0, 21,  0, }, // Maldives
    {  621, 1237,       0,  824934,       0,  0, 31,  0, }, // Marquesas
    {  621, 1247,       0,  824965,       0,  0, 32,  0, }, // Marshall_Islands
    {  621, 1264,  824997,  825025,  825065, 28, 40, 35, }, // Mauritius
    {  621, 1274,       0,  825100,       0,  0, 26,  0, }, // Mawson
    {  621, 1281,  825126,  825159,  825204, 33, 45, 40, }, // Mexico_Pacific
    {  621, 1296,  825244,  825266,  825300, 22, 34, 29, }, // Mongolia
    {  621, 1305,  825329,  825349,  825381, 20, 32, 27, }, // Moscow
    {  621, 1312,       0,  825408,       0,  0, 17,  0, }, // Myanmar
    {  621, 1320,       0,  825425,       0,  0, 25,  0, }, // Nauru
    {  621, 1326,       0,  825450,       0,  0, 17,  0, }, // Nepal
    {  621, 1332,  825467,  825503,  825551, 36, 48, 43, }, // New_Caledonia
    {  621, 1346,  825594,  825619,  825656, 25, 37, 32, }, // New_Zealand
    {  621, 1358,  825688,  825720,  825764, 32, 44, 30, }, // Newfoundland
    {  621, 1371,       0,  825794,       0,  0, 24,  0, }, // Niue
    {  621, 1376,  825818,  825845,  825884, 27, 39, 34, }, // Norfolk
    {  621, 1384,  825918,  825960,  826014, 42, 54, 49, }, // Noronha
    {  621, 1406,  826063,  826086,  826121, 23, 35, 30, }, // Novosibirsk
    {  621, 1418,  826151,  826167,  826195, 16, 28, 23, }, // Omsk
    {  621, 1428,  826218,  826238,  826270, 20, 32, 27, }, // Pakistan
    {  621, 1437,       0,  826297,       0,  0, 25,  0, }, // Palau
    {  621, 1443,       0,  826322,       0,  0, 38,  0, }, // Papua_New_Guinea
    {  621, 1460,  826360,  826382,  826416, 22, 34, 29, }, // Paraguay
    {  621, 1469,  826445,  826460,  826487, 15, 27, 22, }, // Peru
    {  621, 1474,  826509,  826531,  826565, 22, 34, 29, }, // Philippines
    {  621, 1486,       0,  826594,       0,  0, 27,  0, }, // Phoenix_Islands
    {  621, 1502,  826621,  826660,  826711, 39, 51, 46, }, // Pierre_Miquelon
    {  621, 1518,       0,  826757,       0,  0, 28,  0, }, // Pitcairn
    {  621, 1527,       0,  826785,       0,  0, 26,  0, }, // Ponape
    {  621, 1534,       0,  826811,       0,  0, 19,  0, }, // Pyongyang
    {  621, 1544,  826830,  826844,  826871, 14, 27, 22, }, // Qyzylorda
    {  621, 1554,       0,  826893,       0,  0, 28,  0, }, // Reunion
    {  621, 1562,       0,  826921,       0,  0, 26,  0, }, // Rothera
    {  621, 1570,  826947,  826968,  827001, 21, 33, 28, }, // Sakhalin
    {  621, 1579,  827029,  827048,  827079, 19, 31, 26, }, // Samara
    {  621, 1596,  827105,  827130,  827167, 25, 37, 32, }, // Samoa
    {  621, 1602,       0,  827199,       0,  0, 33,  0, }, // Seychelles
    {  621, 1624,       0,  827232,       0,  0, 20,  0, }, // Singapore
    {  621, 1634,       0,  827252,       0,  0, 32,  0, }, // Solomon
    {  621, 1642,       0,  827284,       0,  0, 37,  0, }, // South_Georgia
    {  621, 1656,       0,  827321,       0,  0, 19,  0, }, // Suriname
    {  621, 1676,       0,  827340,       0,  0, 24,  0, }, // Syowa
    {  621, 1682,       0,  827364,       0,  0, 25,  0, }, // Tahiti
    {  621, 1689,  827389,  827406,  827435, 17, 29, 24, }, // Taipei
    {  621, 1696,       0,  827459,       0,  0, 23,  0, }, // Tajikistan
    {  621, 1724,       0,  827482,       0,  0, 28,  0, }, // Tokelau
    {  621, 1732,  827510,  827536,  827574, 26, 38, 33, }, // Tonga
    {  621, 1738,       0,  827607,       0,  0, 25,  0, }, // Truk
    {  621, 1750,  827632,  827656,  827692, 24, 36, 31, }, // Turkmenistan
    {  621, 1763,       0,  827723,       0,  0, 27,  0, }, // Tuvalu
    {  621, 1777,  827750,  827768,  827798, 18, 30, 25, }, // Uruguay
    {  621, 1792,  827823,  827845,  827879, 22, 34, 29, }, // Uzbekistan
    {  621, 1803,  827908,  827936,  827976, 28, 40, 35, }, // Vanuatu
    {  621, 1811,       0,  828011,       0,  0, 20,  0, }, // Venezuela
    {  621, 1821,  828031,  828055,  828091, 24, 36, 31, }, // Vladivostok
    {  621, 1833,  828122,  828145,  828180, 23, 35, 30, }, // Volgograd
    {  621, 1843,       0,  828210,       0,  0, 26,  0, }, // Vostok
    {  621, 1850,       0,  828236,       0,  0, 24,  0, }, // Wake
    {  621, 1855,       0,  828260,       0,  0, 36,  0, }, // Wallis
    {  621, 1862,  828296,  828314,  828344, 18, 30, 25, }, // Yakutsk
    {  621, 1870,  828369,  828394,  828431, 25, 37, 32, }, // Yekaterinburg
    {  621, 1892,       0,  828463,       0,  0, 29,  0, }, // Yukon
    {  622,    5,       0,  828492,       0,  0, 13,  0, }, // Afghanistan Upper Sorbian/Latin/Germany
    {  622,   17,       0,  828505,       0,  0, 20,  0, }, // Africa_Central
    {  622,   32,       0,  828525,       0,  0, 18,  0, }, // Africa_Eastern
    {  622,   65,       0,  828543,       0,  0, 16,  0, }, // Africa_Southern
    {  622,   81,  828559,  828576,  828604, 17, 28, 23, }, // Africa_Western
    {  622,  107,  828627,  828640,  828664, 13, 24, 19, }, // Alaska
    {  622,  135,  828683,  828697,  828722, 14, 25, 20, }, // Amazon
    {  622,  142,  828742,  828771,  828811, 29, 40, 35, }, // America_Central
    {  622,  158,  828846,  828874,  828913, 28, 39, 34, }, // America_Eastern
    {  622,  174,  828947,  828973,  829010, 26, 37, 32, }, // America_Mountain
    {  622,  191,  829042,  829071,  829111, 29, 40, 35, }, // America_Pacific
    {  622,  214,  829146,  829157,  829179, 11, 22, 17, }, // Apia
    {  622,  232,  829196,  829207,  829229, 11, 22, 17, }, // Arabian
    {  622,  240,  829246,  829261,  829287, 15, 26, 21, }, // Argentina
    {  622,  250,  829308,  829329,  829361, 21, 32, 27, }, // Argentina_Western
    {  622,  268,  829388,  829400,  829423, 12, 23, 18, }, // Armenia
    {  622,  286,  829441,  829455,  829480, 14, 25, 20, }, // Atlantic
    {  622,  295,  829500,  829521,  829553, 21, 32, 27, }, // Australia_Central
    {  622,  313,  829580,  829609,  829649, 29, 40, 34, }, // Australia_CentralWestern
    {  622,  338,  829683,  829704,  829736, 21, 32, 27, }, // Australia_Eastern
    {  622,  356,  829763,  829783,  829814, 20, 31, 26, }, // Australia_Western
    {  622,  374,  829840,  829858,  829887, 18, 29, 24, }, // Azerbaijan
    {  622,  385,  829911,  829922,  829944, 11, 22, 17, }, // Azores
    {  622,  397,  829961,  829977,  830004, 16, 27, 22, }, // Bangladesh
    {  622,  415,       0,  830026,       0,  0, 13,  0, }, // Bhutan
    {  622,  422,       0,  830039,       0,  0, 13,  0, }, // Bolivia
    {  622,  437,  830052,  830066,  830091, 14, 25, 20, }, // Brasilia
    {  622,  454,       0,  830111,       0,  0, 13,  0, }, // Brunei
    {  622,  461,  830124,  830138,  830163, 14, 25, 20, }, // Cape_Verde
    {  622,  478,       0,  830183,       0,  0, 15,  0, }, // Chamorro
    {  622,  487,  830198,  830212,  830237, 14, 25, 20, }, // Chatham
    {  622,  495,  830257,  830268,  830290, 11, 22, 17, }, // Chile
    {  622,  501,  830307,  830318,  830340, 11, 22, 17, }, // China
    {  622,  507,       0,  830357,       0,  0, 18,  0, }, // Christmas
    {  622,  517,       0,  830375,       0,  0, 20,  0, }, // Cocos
    {  622,  523,  830395,  830409,  830434, 14, 25, 20, }, // Colombia
    {  622,  532,  830454,  830473,  830503, 19, 30, 25, }, // Cook
    {  622,  537,  830528,  830539,  830561, 11, 22, 17, }, // Cuba
    {  622,  548,       0,  830578,       0,  0, 11,  0, }, // Davis
    {  622,  564,       0,  830589,       0,  0, 24,  0, }, // DumontDUrville
    {  622,  601,       0,  830613,       0,  0, 20,  0, }, // East_Timor
    {  622,  612,  830633,  830652,  830682, 19, 30, 25, }, // Easter
    {  622,  619,       0,  830707,       0,  0, 14,  0, }, // Ecuador
    {  622,  627,  830721,  830740,  830770, 19, 30, 25, }, // Europe_Central
    {  622,  642,  830795,  830814,  830844, 19, 30, 25, }, // Europe_Eastern
    {  622,  657,       0,  830869,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {  622,  680,  830887,  830905,  830934, 18, 29, 24, }, // Europe_Western
    {  622,  695,  830958,  830973,  830999, 15, 26, 21, }, // Falkland
    {  622,  704,  831020,  831032,  831055, 12, 23, 18, }, // Fiji
    {  622,  709,       0,  831073,       0,  0, 21,  0, }, // French_Guiana
    {  622,  723,       0,  831094,       0,  0, 50,  0, }, // French_Southern
    {  622,  746,       0,  831144,       0,  0, 15,  0, }, // Galapagos
    {  622,  756,       0,  831159,       0,  0, 14,  0, }, // Gambier
    {  622,  764,  831173,  831186,  831210, 13, 24, 19, }, // Georgia
    {  622,  772,       0,  831229,       0,  0, 22,  0, }, // Gilbert_Islands
    {  622,  788,       0,  831251,       0,  0, 16,  0, }, // GMT
    {  622,  830,  831267,  831289,  831322, 22, 33, 28, }, // Greenland_Eastern
    {  622,  848,  831350,  831371,  831403, 21, 32, 27, }, // Greenland_Western
    {  622,  871,       0,  831430,       0,  0, 20,  0, }, // Gulf
    {  622,  876,       0,  831082,       0,  0, 12,  0, }, // Guyana
    {  622,  883,  831450,  831472,  831505, 22, 33, 28, }, // Hawaii_Aleutian
    {  622,  899,  831533,  831548,  831574, 15, 26, 21, }, // Hong_Kong
    {  622,  909,  831595,  831607,  831630, 12, 23, 18, }, // Hovd
    {  622,  914,       0,  831648,       0,  0, 11,  0, }, // India
    {  622,  920,       0,  831659,       0,  0, 19,  0, }, // Indian_Ocean
    {  622,  933,       0,  831678,       0,  0, 15,  0, }, // Indochina
    {  622,  943,       0,  831693,       0,  0, 20,  0, }, // Indonesia_Central
    {  622,  961,       0,  831713,       0,  0, 16,  0, }, // Indonesia_Eastern
    {  622,  979,       0,  831729,       0,  0, 19,  0, }, // Indonesia_Western
    {  622,  997,  831748,  831759,  831781, 11, 22, 17, }, // Iran
    {  622, 1008,  719540,  831798,  831821, 12, 23, 18, }, // Irkutsk
    {  622, 1016,  831839,  831852,  831876, 13, 24, 19, }, // Israel
    {  622, 1023,  831895,  831907,  831930, 12, 23, 18, }, // Japan
    {  622, 1047,       0,  831948,       0,  0, 13,  0, }, // Kazakhstan
    {  622, 1058,       0,  831961,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  622, 1077,       0,  831982,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  622, 1106,  832002,  832014,  832037, 12, 23, 18, }, // Korea
    {  622, 1112,       0,  832055,       0,  0, 13,  0, }, // Kosrae
    {  622, 1119,  719940,  832068,  832095, 16, 27, 22, }, // Krasnoyarsk
    {  622, 1151,       0,  832117,       0,  0, 12,  0, }, // Kyrgystan
    {  622, 1175,       0,  832129,       0,  0, 20,  0, }, // Line_Islands
    {  622, 1188,  832149,  832167,  832196, 18, 29, 24, }, // Lord_Howe
    {  622, 1204,  720122,  832220,  832245, 14, 25, 20, }, // Magadan
    {  622, 1219,       0,  832265,       0,  0, 14,  0, }, // Malaysia
    {  622, 1228,       0,  832279,       0,  0, 14,  0, }, // Maldives
    {  622, 1237,       0,  832293,       0,  0, 15,  0, }, // Marquesas
    {  622, 1247,       0,  832308,       0,  0, 23,  0, }, // Marshall_Islands
    {  622, 1264,  832331,  832346,  832372, 15, 26, 21, }, // Mauritius
    {  622, 1274,       0,  720315,       0,  0, 13,  0, }, // Mawson
    {  622, 1281,  832393,  832414,  832446, 21, 32, 27, }, // Mexico_Pacific
    {  622, 1296,  832473,  832490,  832518, 17, 28, 23, }, // Mongolia
    {  622, 1305,  832541,  832554,  832578, 13, 24, 19, }, // Moscow
    {  622, 1312,       0,  832597,       0,  0, 14,  0, }, // Myanmar
    {  622, 1320,       0,  832611,       0,  0, 12,  0, }, // Nauru
    {  622, 1326,       0,  832623,       0,  0, 12,  0, }, // Nepal
    {  622, 1332,  832635,  832653,  832682, 18, 29, 24, }, // New_Caledonia
    {  622, 1346,  832706,  832724,  832753, 18, 29, 24, }, // New_Zealand
    {  622, 1358,  832777,  832796,  832826, 19, 30, 25, }, // Newfoundland
    {  622, 1371,       0,  832851,       0,  0, 11,  0, }, // Niue
    {  622, 1376,  832862,  832878,  832905, 16, 27, 22, }, // Norfolk
    {  622, 1384,  832927,  832956,  832996, 29, 40, 35, }, // Noronha
    {  622, 1406,  833031,  833047,  833074, 16, 27, 22, }, // Novosibirsk
    {  622, 1418,  721048,  833096,  833116,  9, 20, 15, }, // Omsk
    {  622, 1428,  833131,  833146,  833172, 15, 26, 21, }, // Pakistan
    {  622, 1437,       0,  833193,       0,  0, 12,  0, }, // Palau
    {  622, 1443,       0,  833205,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  622, 1460,  833227,  833242,  833268, 15, 26, 21, }, // Paraguay
    {  622, 1469,  833289,  833300,  833322, 11, 22, 17, }, // Peru
    {  622, 1474,  833339,  833353,  833378, 14, 25, 20, }, // Philippines
    {  622, 1486,       0,  833398,       0,  0, 22,  0, }, // Phoenix_Islands
    {  622, 1502,  833420,  833451,  833493, 31, 42, 37, }, // Pierre_Miquelon
    {  622, 1518,       0,  833530,       0,  0, 23,  0, }, // Pitcairn
    {  622, 1527,       0,  833553,       0,  0, 13,  0, }, // Ponape
    {  622, 1534,       0,  833566,       0,  0, 16,  0, }, // Pyongyang
    {  622, 1554,       0,  833582,       0,  0, 14,  0, }, // Reunion
    {  622, 1562,       0,  833596,       0,  0, 14,  0, }, // Rothera
    {  622, 1570,  833610,  833625,  833651, 15, 26, 21, }, // Sakhalin
    {  622, 1596,  833672,  833684,  833707, 12, 23, 18, }, // Samoa
    {  622, 1602,       0,  833725,       0,  0, 15,  0, }, // Seychelles
    {  622, 1624,       0,  833740,       0,  0, 15,  0, }, // Singapore
    {  622, 1634,       0,  833755,       0,  0, 22,  0, }, // Solomon
    {  622, 1642,       0,  833777,       0,  0, 18,  0, }, // South_Georgia
    {  622, 1656,       0,  833795,       0,  0, 14,  0, }, // Suriname
    {  622, 1676,       0,  833809,       0,  0, 12,  0, }, // Syowa
    {  622, 1682,       0,  833821,       0,  0, 13,  0, }, // Tahiti
    {  622, 1689,  833834,  833847,  833871, 13, 24, 19, }, // Taipei
    {  622, 1696,       0,  833890,       0,  0, 13,  0, }, // Tajikistan
    {  622, 1724,       0,  833903,       0,  0, 14,  0, }, // Tokelau
    {  622, 1732,  833917,  833929,  833952, 12, 23, 18, }, // Tonga
    {  622, 1738,       0,  833970,       0,  0, 12,  0, }, // Truk
    {  622, 1750,  833982,  833996,  834021, 14, 25, 20, }, // Turkmenistan
    {  622, 1763,       0,  834041,       0,  0, 13,  0, }, // Tuvalu
    {  622, 1777,  834054,  834068,  834093, 14, 25, 20, }, // Uruguay
    {  622, 1792,  834113,  834125,  834148, 12, 23, 18, }, // Uzbekistan
    {  622, 1803,  834166,  834180,  834205, 14, 25, 20, }, // Vanuatu
    {  622, 1811,       0,  834225,       0,  0, 15,  0, }, // Venezuela
    {  622, 1821,  834240,  834258,  834287, 18, 29, 24, }, // Vladivostok
    {  622, 1833,  834311,  834327,  834354, 16, 27, 22, }, // Volgograd
    {  622, 1843,       0,  834376,       0,  0, 13,  0, }, // Vostok
    {  622, 1850,       0,  834389,       0,  0, 13,  0, }, // Wake
    {  622, 1855,       0,  834402,       0,  0, 25,  0, }, // Wallis
    {  622, 1862,  722493,  834427,  834450, 12, 23, 18, }, // Yakutsk
    {  622, 1870,  834468,  834488,  834519, 20, 31, 26, }, // Yekaterinburg
    {  622, 1892,       0,  834545,       0,  0, 12,  0, }, // Yukon
    {  623,    5,       0,  834557,       0,  0, 16,  0, }, // Afghanistan Urdu/Arabic/Pakistan
    {  623,   17,       0,  834573,       0,  0, 16,  0, }, // Africa_Central
    {  623,   32,       0,  834589,       0,  0, 17,  0, }, // Africa_Eastern
    {  623,   65,       0,  834606,       0,  0, 25,  0, }, // Africa_Southern
    {  623,   81,  834631,  834648,  834673, 17, 25, 21, }, // Africa_Western
    {  623,  107,  834694,  834705,  834725, 11, 20, 19, }, // Alaska
    {  623,  135,  834744,  834755,  834776, 11, 21, 26, }, // Amazon
    {  623,  142,  834802,  834812,  834831, 10, 19, 18, }, // America_Central
    {  623,  158,  834849,  834860,  834880, 11, 20, 19, }, // America_Eastern
    {  623,  174,  834899,  834911,  834932, 12, 21, 20, }, // America_Mountain
    {  623,  191,  834952,  834962,  834981, 10, 19, 18, }, // America_Pacific
    {  623,  207,  834999,  835009,  835028, 10, 19, 14, }, // Anadyr
    {  623,  214,  835042,  835052,  835070, 10, 18, 18, }, // Apia
    {  623,  232,  835088,  835098,  835115, 10, 17, 16, }, // Arabian
    {  623,  240,  835131,  835146,  835168, 15, 22, 28, }, // Argentina
    {  623,  250,  835196,  835217,  835245, 21, 28, 34, }, // Argentina_Western
    {  623,  268,  835279,  835293,  835314, 14, 21, 27, }, // Armenia
    {  623,  286,  835341,  835353,  835374, 12, 21, 20, }, // Atlantic
    {  623,  295,  835394,  835413,  835441, 19, 28, 27, }, // Australia_Central
    {  623,  313,  835468,  835494,  835529, 26, 35, 34, }, // Australia_CentralWestern
    {  623,  338,  835563,  835583,  835612, 20, 29, 28, }, // Australia_Eastern
    {  623,  356,  835640,  835660,  835689, 20, 29, 28, }, // Australia_Western
    {  623,  374,  835717,  835734,  835758, 17, 24, 30, }, // Azerbaijan
    {  623,  385,  835788,  835801,  835821, 13, 20, 26, }, // Azores
    {  623,  397,  835847,  835863,  835886, 16, 23, 29, }, // Bangladesh
    {  623,  415,       0,  835915,       0,  0, 13,  0, }, // Bhutan
    {  623,  422,       0,  835928,       0,  0, 14,  0, }, // Bolivia
    {  623,  437,  835942,  835955,  835977, 13, 22, 17, }, // Brasilia
    {  623,  454,       0,  835994,       0,  0, 21,  0, }, // Brunei
    {  623,  461,  836015,  836028,  836049, 13, 21, 17, }, // Cape_Verde
    {  623,  478,       0,  836066,       0,  0, 19,  0, }, // Chamorro
    {  623,  487,  836085,  836095,  836114, 10, 19, 18, }, // Chatham
    {  623,  495,  836132,  836142,  836159, 10, 17, 23, }, // Chile
    {  623,  501,  836182,  836192,  836208, 10, 16, 17, }, // China
    {  623,  507,       0,  836225,       0,  0, 17,  0, }, // Christmas
    {  623,  517,       0,  836242,       0,  0, 18,  0, }, // Cocos
    {  623,  523,  836260,  836272,  836293, 12, 21, 27, }, // Colombia
    {  623,  532,  836320,  836335,  836358, 15, 23, 23, }, // Cook
    {  623,  537,  836381,  836391,  836410, 10, 19, 18, }, // Cuba
    {  623,  548,       0,  836428,       0,  0,  9,  0, }, // Davis
    {  623,  564,       0,  836437,       0,  0, 21,  0, }, // DumontDUrville
    {  623,  601,       0,  836458,       0,  0, 16,  0, }, // East_Timor
    {  623,  612,  836474,  836493,  836519, 19, 26, 32, }, // Easter
    {  623,  619,       0,  836551,       0,  0, 15,  0, }, // Ecuador
    {  623,  627,  836566,  836581,  836604, 15, 23, 29, }, // Europe_Central
    {  623,  642,  836633,  836650,  836674, 17, 24, 30, }, // Europe_Eastern
    {  623,  657,       0,  836704,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  623,  680,  836724,  836741,  836765, 17, 24, 30, }, // Europe_Western
    {  623,  695,  836795,  836818,  836848, 23, 30, 36, }, // Falkland
    {  623,  704,  836884,  836892,  836908,  8, 16, 12, }, // Fiji
    {  623,  709,       0,  836920,       0,  0, 18,  0, }, // French_Guiana
    {  623,  723,       0,  836938,       0,  0, 29,  0, }, // French_Southern
    {  623,  746,       0,  836967,       0,  0, 16,  0, }, // Galapagos
    {  623,  756,       0,  836983,       0,  0, 12,  0, }, // Gambier
    {  623,  764,  836995,  837008,  837028, 13, 20, 26, }, // Georgia
    {  623,  772,       0,  837054,       0,  0, 18,  0, }, // Gilbert_Islands
    {  623,  788,       0,  837072,       0,  0, 18,  0, }, // GMT
    {  623,  830,  837090,  837110,  837139, 20, 29, 35, }, // Greenland_Eastern
    {  623,  848,  837174,  837194,  837223, 20, 29, 35, }, // Greenland_Western
    {  623,  871,       0,  837258,       0,  0, 18,  0, }, // Gulf
    {  623,  876,       0,  836926,       0,  0, 12,  0, }, // Guyana
    {  623,  883,  837276,  837295,  837323, 19, 28, 27, }, // Hawaii_Aleutian
    {  623,  899,  413821,  413835,  837350, 14, 22, 18, }, // Hong_Kong
    {  623,  909,  837368,  837377,  837394,  9, 17, 13, }, // Hovd
    {  623,  914,       0,  837407,       0,  0, 22,  0, }, // India
    {  623,  920,       0,  837429,       0,  0, 12,  0, }, // Indian_Ocean
    {  623,  933,       0,  837441,       0,  0, 12,  0, }, // Indochina
    {  623,  943,       0,  837453,       0,  0, 19,  0, }, // Indonesia_Central
    {  623,  961,       0,  837472,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  623,  979,       0,  837492,       0,  0, 20,  0, }, // Indonesia_Western
    {  623,  997,  837512,  837524,  837543, 12, 19, 18, }, // Iran
    {  623, 1008,  837561,  837572,  837591, 11, 19, 15, }, // Irkutsk
    {  623, 1016,  837606,  837620,  837641, 14, 21, 20, }, // Israel
    {  623, 1023,  837661,  837671,  837689, 10, 18, 18, }, // Japan
    {  623, 1029,  837707,  837733,  837768, 26, 35, 30, }, // Kamchatka
    {  623, 1047,       0,  837798,       0,  0, 15,  0, }, // Kazakhstan
    {  623, 1058,       0,  837813,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  623, 1077,       0,  837834,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  623, 1106,  837855,  837865,  837883, 10, 18, 18, }, // Korea
    {  623, 1112,       0,  837901,       0,  0, 10,  0, }, // Kosrae
    {  623, 1119,  837911,  837927,  837950, 16, 23, 20, }, // Krasnoyarsk
    {  623, 1151,       0,  837970,       0,  0, 14,  0, }, // Kyrgystan
    {  623, 1175,       0,  837984,       0,  0, 17,  0, }, // Line_Islands
    {  623, 1188,  838001,  838015,  838038, 14, 23, 22, }, // Lord_Howe
    {  623, 1204,  838060,  838071,  838091, 11, 20, 15, }, // Magadan
    {  623, 1219,       0,  838106,       0,  0, 11,  0, }, // Malaysia
    {  623, 1228,       0,  838117,       0,  0, 13,  0, }, // Maldives
    {  623, 1237,       0,  838130,       0,  0, 12,  0, }, // Marquesas
    {  623, 1247,       0,  838142,       0,  0, 18,  0, }, // Marshall_Islands
    {  623, 1264,  838160,  838171,  838190, 11, 19, 15, }, // Mauritius
    {  623, 1274,       0,  838205,       0,  0, 10,  0, }, // Mawson
    {  623, 1281,  838215,  838233,  838260, 18, 27, 26, }, // Mexico_Pacific
    {  623, 1296,  838286,  838302,  838326, 16, 24, 20, }, // Mongolia
    {  623, 1305,  838346,  838356,  838375, 10, 19, 14, }, // Moscow
    {  623, 1312,       0,  838389,       0,  0, 12,  0, }, // Myanmar
    {  623, 1320,       0,  838401,       0,  0, 10,  0, }, // Nauru
    {  623, 1326,       0,  838411,       0,  0, 12,  0, }, // Nepal
    {  623, 1332,  838423,  838441,  838467, 18, 26, 22, }, // New_Caledonia
    {  623, 1346,  838489,  838506,  838530, 17, 24, 23, }, // New_Zealand
    {  623, 1358,  415179,  838553,  415225, 19, 28, 27, }, // Newfoundland
    {  623, 1371,       0,  838581,       0,  0,  9,  0, }, // Niue
    {  623, 1376,  838590,  838610,  838637, 20, 27, 25, }, // Norfolk
    {  623, 1384,  838662,  838686,  838717, 24, 31, 27, }, // Noronha
    {  623, 1406,  838744,  838759,  838782, 15, 23, 19, }, // Novosibirsk
    {  623, 1418,  838801,  838811,  838829, 10, 18, 14, }, // Omsk
    {  623, 1428,  838843,  838857,  838878, 14, 21, 27, }, // Pakistan
    {  623, 1437,       0,  838905,       0,  0,  9,  0, }, // Palau
    {  623, 1443,       0,  838914,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  623, 1460,  838932,  838947,  838969, 15, 22, 28, }, // Paraguay
    {  623, 1469,  838997,  839008,  839026, 11, 18, 24, }, // Peru
    {  623, 1474,  839050,  839061,  839080, 11, 19, 15, }, // Philippines
    {  623, 1486,       0,  839095,       0,  0, 18,  0, }, // Phoenix_Islands
    {  623, 1502,  839113,  839139,  839174, 26, 35, 34, }, // Pierre_Miquelon
    {  623, 1518,       0,  839208,       0,  0, 12,  0, }, // Pitcairn
    {  623, 1527,       0,  839220,       0,  0, 11,  0, }, // Ponape
    {  623, 1534,       0,  839231,       0,  0, 14,  0, }, // Pyongyang
    {  623, 1554,       0,  839245,       0,  0, 13,  0, }, // Reunion
    {  623, 1562,       0,  839258,       0,  0, 14,  0, }, // Rothera
    {  623, 1570,  839272,  839283,  839302, 11, 19, 15, }, // Sakhalin
    {  623, 1579,  839317,  839327,  839346, 10, 19, 14, }, // Samara
    {  623, 1596,  839360,  839370,  839388, 10, 18, 18, }, // Samoa
    {  623, 1602,       0,  839406,       0,  0, 11,  0, }, // Seychelles
    {  623, 1624,       0,  839417,       0,  0, 20,  0, }, // Singapore
    {  623, 1634,       0,  839437,       0,  0, 18,  0, }, // Solomon
    {  623, 1642,       0,  839455,       0,  0, 17,  0, }, // South_Georgia
    {  623, 1656,       0,  839472,       0,  0, 14,  0, }, // Suriname
    {  623, 1676,       0,  839486,       0,  0, 10,  0, }, // Syowa
    {  623, 1682,       0,  839496,       0,  0, 11,  0, }, // Tahiti
    {  623, 1689,  839507,  839521,  839544, 14, 23, 21, }, // Taipei
    {  623, 1696,       0,  839565,       0,  0, 15,  0, }, // Tajikistan
    {  623, 1724,       0,  839580,       0,  0, 12,  0, }, // Tokelau
    {  623, 1732,  839592,  839602,  839620, 10, 18, 14, }, // Tonga
    {  623, 1738,       0,  839634,       0,  0,  8,  0, }, // Truk
    {  623, 1750,  839642,  839659,  839683, 17, 24, 30, }, // Turkmenistan
    {  623, 1763,       0,  839713,       0,  0, 10,  0, }, // Tuvalu
    {  623, 1777,  839723,  839738,  839760, 15, 22, 28, }, // Uruguay
    {  623, 1792,  839788,  839803,  839825, 15, 22, 28, }, // Uzbekistan
    {  623, 1803,  839853,  839865,  839885, 12, 20, 16, }, // Vanuatu
    {  623, 1811,       0,  839901,       0,  0, 16,  0, }, // Venezuela
    {  623, 1821,  839917,  839933,  839957, 16, 24, 20, }, // Vladivostok
    {  623, 1833,  839977,  839991,  840014, 14, 23, 18, }, // Volgograd
    {  623, 1843,       0,  840032,       0,  0, 13,  0, }, // Vostok
    {  623, 1850,       0,  840045,       0,  0, 15,  0, }, // Wake
    {  623, 1855,       0,  840060,       0,  0, 20,  0, }, // Wallis
    {  623, 1862,  840080,  840091,  840111, 11, 20, 15, }, // Yakutsk
    {  623, 1870,  840126,  840142,  840167, 16, 25, 20, }, // Yekaterinburg
    {  623, 1892,       0,  416982,       0,  0, 10,  0, }, // Yukon
    {  624,    5,       0,  840187,       0,  0, 14,  0, }, // Afghanistan Urdu/Arabic/India
    {  624,  135,  840201,  840213,  840233, 12, 20, 16, }, // Amazon
    {  624,  232,  840249,  840257,  835115,  8, 16, 16, }, // Arabian
    {  624,  250,  840273,  840292,  840319, 19, 27, 23, }, // Argentina_Western
    {  624,  268,  840342,  840354,  840374, 12, 20, 16, }, // Armenia
    {  624,  374,  840390,  840405,  840428, 15, 23, 19, }, // Azerbaijan
    {  624,  397,  840447,  840461,  840483, 14, 22, 18, }, // Bangladesh
    {  624,  415,       0,  840501,       0,  0, 11,  0, }, // Bhutan
    {  624,  422,       0,  840512,       0,  0, 12,  0, }, // Bolivia
    {  624,  437,  835942,  840524,  835977, 13, 21, 17, }, // Brasilia
    {  624,  495,  840545,  840553,  840569,  8, 16, 12, }, // Chile
    {  624,  523,  836260,  840581,  840601, 12, 20, 16, }, // Colombia
    {  624,  612,  840617,  840634,  840659, 17, 25, 21, }, // Easter
    {  624,  619,       0,  840680,       0,  0, 13,  0, }, // Ecuador
    {  624,  627,  840693,  836581,  836604, 16, 23, 29, }, // Europe_Central
    {  624,  695,  840709,  840730,  840759, 21, 29, 25, }, // Falkland
    {  624,  709,       0,  840784,       0,  0, 16,  0, }, // French_Guiana
    {  624,  746,       0,  840800,       0,  0, 14,  0, }, // Galapagos
    {  624,  764,  839461,  840814,  840833, 11, 19, 15, }, // Georgia
    {  624,  788,       0,  840848,       0,  0, 16,  0, }, // GMT
    {  624,  871,       0,  840864,       0,  0, 17,  0, }, // Gulf
    {  624,  876,       0,  840790,       0,  0, 10,  0, }, // Guyana
    {  624,  914,       0,  840881,       0,  0, 18,  0, }, // India
    {  624,  997,  840899,  840909,  837543, 10, 18, 18, }, // Iran
    {  624, 1016,  840927,  840939,  837641, 12, 20, 20, }, // Israel
    {  624, 1058,       0,  840959,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  624, 1077,       0,  840978,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  624, 1151,       0,  840997,       0,  0, 12,  0, }, // Kyrgystan
    {  624, 1228,       0,  841009,       0,  0, 11,  0, }, // Maldives
    {  624, 1326,       0,  841020,       0,  0, 10,  0, }, // Nepal
    {  624, 1346,  841030,  841045,  838530, 15, 23, 23, }, // New_Zealand
    {  624, 1384,  841068,  841090,  838717, 22, 30, 27, }, // Noronha
    {  624, 1428,  841120,  841132,  841152, 12, 20, 16, }, // Pakistan
    {  624, 1460,  841168,  841181,  841202, 13, 21, 17, }, // Paraguay
    {  624, 1469,  841219,  841228,  841245,  9, 17, 13, }, // Peru
    {  624, 1562,       0,  841258,       0,  0, 12,  0, }, // Rothera
    {  624, 1656,       0,  841270,       0,  0, 12,  0, }, // Suriname
    {  624, 1696,       0,  416326,       0,  0, 13,  0, }, // Tajikistan
    {  624, 1750,  416411,  416426,  841282, 15, 23, 19, }, // Turkmenistan
    {  624, 1777,  841301,  841314,  841335, 13, 21, 17, }, // Uruguay
    {  624, 1792,  841352,  841365,  841386, 13, 21, 17, }, // Uzbekistan
    {  624, 1811,       0,  841403,       0,  0, 14,  0, }, // Venezuela
    {  624, 1843,       0,  841417,       0,  0, 11,  0, }, // Vostok
    {  625,    0,  841428,  841439,  841460, 11, 21, 18, }, // Acre Uyghur/Arabic/China
    {  625,    5,       0,  841478,       0,  0, 17,  0, }, // Afghanistan
    {  625,   17,       0,  841495,       0,  0, 21,  0, }, // Africa_Central
    {  625,   32,       0,  841516,       0,  0, 20,  0, }, // Africa_Eastern
    {  625,   65,       0,  841536,       0,  0, 31,  0, }, // Africa_Southern
    {  625,   81,  841567,  841587,  841617, 20, 30, 27, }, // Africa_Western
    {  625,  107,  841644,  841658,  841682, 14, 24, 21, }, // Alaska
    {  625,  128,  841703,  841716,  841739, 13, 23, 20, }, // Almaty
    {  625,  135,  841759,  841772,  841795, 13, 23, 20, }, // Amazon
    {  625,  142,  841815,  841834,  841863, 19, 29, 26, }, // America_Central
    {  625,  158,  841889,  841907,  841935, 18, 28, 25, }, // America_Eastern
    {  625,  174,  841960,  841969,  841988,  9, 19, 16, }, // America_Mountain
    {  625,  191,  842004,  842021,  842048, 17, 27, 24, }, // America_Pacific
    {  625,  207,  842072,  842085,  842108, 13, 23, 20, }, // Anadyr
    {  625,  219,  842128,  842140,  842162, 12, 22, 19, }, // Aqtau
    {  625,  225,  842181,  842194,  842217, 13, 23, 20, }, // Aqtobe
    {  625,  232,  842237,  842248,  842269, 11, 21, 18, }, // Arabian
    {  625,  240,  842287,  842303,  842329, 16, 26, 23, }, // Argentina
    {  625,  250,  842352,  842375,  842408, 23, 33, 30, }, // Argentina_Western
    {  625,  268,  842438,  842453,  842478, 15, 25, 22, }, // Armenia
    {  625,  286,  842500,  842522,  842554, 22, 32, 29, }, // Atlantic
    {  625,  295,  842583,  842614,  842655, 31, 41, 38, }, // Australia_Central
    {  625,  313,  842693,  842731,  842780, 38, 49, 45, }, // Australia_CentralWestern
    {  625,  338,  842825,  842855,  842895, 30, 40, 37, }, // Australia_Eastern
    {  625,  356,  842932,  842962,  843002, 30, 40, 37, }, // Australia_Western
    {  625,  374,  843039,  843056,  843083, 17, 27, 24, }, // Azerbaijan
    {  625,  385,  843107,  843118,  843139, 11, 21, 18, }, // Azores
    {  625,  397,  843157,  843171,  843195, 14, 24, 21, }, // Bangladesh
    {  625,  415,       0,  843216,       0,  0, 11,  0, }, // Bhutan
    {  625,  422,       0,  843227,       0,  0, 14,  0, }, // Bolivia
    {  625,  437,  843241,  843257,  843283, 16, 26, 23, }, // Brasilia
    {  625,  454,       0,  843306,       0,  0, 24,  0, }, // Brunei
    {  625,  461,  843330,  843348,  843376, 18, 28, 25, }, // Cape_Verde
    {  625,  472,       0,  843401,       0,  0, 11,  0, }, // Casey
    {  625,  478,       0,  843412,       0,  0, 23,  0, }, // Chamorro
    {  625,  487,  843435,  843446,  843467, 11, 21, 18, }, // Chatham
    {  625,  495,  843485,  843495,  843515, 10, 20, 17, }, // Chile
    {  625,  501,  843532,  843543,  843564, 11, 21, 18, }, // China
    {  625,  507,       0,  843582,       0,  0, 22,  0, }, // Christmas
    {  625,  517,       0,  843604,       0,  0, 18,  0, }, // Cocos
    {  625,  523,  843622,  843637,  843662, 15, 25, 22, }, // Colombia
    {  625,  532,  843684,  843703,  843732, 19, 29, 32, }, // Cook
    {  625,  537,  843764,  843774,  843794, 10, 20, 17, }, // Cuba
    {  625,  548,       0,  843811,       0,  0, 11,  0, }, // Davis
    {  625,  564,       0,  843822,       0,  0, 19,  0, }, // DumontDUrville
    {  625,  601,       0,  843841,       0,  0, 18,  0, }, // East_Timor
    {  625,  612,  843859,  843878,  843909, 19, 31, 26, }, // Easter
    {  625,  619,       0,  843935,       0,  0, 14,  0, }, // Ecuador
    {  625,  627,  843949,  843970,  844001, 21, 31, 28, }, // Europe_Central
    {  625,  642,  844029,  844049,  844079, 20, 30, 27, }, // Europe_Eastern
    {  625,  680,  844106,  844126,  844156, 20, 30, 27, }, // Europe_Western
    {  625,  695,  844183,  844207,  844241, 24, 34, 31, }, // Falkland
    {  625,  704,  844272,  844282,  844302, 10, 20, 17, }, // Fiji
    {  625,  709,       0,  844319,       0,  0, 35,  0, }, // French_Guiana
    {  625,  723,       0,  844354,       0,  0, 49,  0, }, // French_Southern
    {  625,  746,       0,  844403,       0,  0, 15,  0, }, // Galapagos
    {  625,  756,       0,  844418,       0,  0, 14,  0, }, // Gambier
    {  625,  764,  844432,  844446,  844470, 14, 24, 21, }, // Georgia
    {  625,  772,       0,  844491,       0,  0, 23,  0, }, // Gilbert_Islands
    {  625,  788,       0,  844514,       0,  0, 14,  0, }, // GMT
    {  625,  830,  844528,  844550,  844582, 22, 32, 29, }, // Greenland_Eastern
    {  625,  848,  844611,  844633,  844665, 22, 32, 29, }, // Greenland_Western
    {  625,  866,       0,  844694,       0,  0, 21,  0, }, // Guam
    {  625,  871,       0,  844715,       0,  0, 20,  0, }, // Gulf
    {  625,  876,       0,  844340,       0,  0, 14,  0, }, // Guyana
    {  625,  883,  844735,  844754,  844783, 19, 29, 26, }, // Hawaii_Aleutian
    {  625,  899,  844809,  844822,  844845, 13, 23, 20, }, // Hong_Kong
    {  625,  909,  844865,  844875,  844895, 10, 20, 17, }, // Hovd
    {  625,  914,       0,  844912,       0,  0, 25,  0, }, // India
    {  625,  920,       0,  844937,       0,  0, 18,  0, }, // Indian_Ocean
    {  625,  933,       0,  844955,       0,  0, 16,  0, }, // Indochina
    {  625,  943,       0,  844971,       0,  0, 25,  0, }, // Indonesia_Central
    {  625,  961,       0,  844996,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  625,  979,       0,  845020,       0,  0, 24,  0, }, // Indonesia_Western
    {  625,  997,  845044,  845055,  845076, 11, 21, 18, }, // Iran
    {  625, 1008,  845094,  845108,  845132, 14, 24, 21, }, // Irkutsk
    {  625, 1016,  845153,  845170,  845197, 17, 27, 24, }, // Israel
    {  625, 1023,  845221,  845235,  845259, 14, 24, 21, }, // Japan
    {  625, 1029,  845280,  845310,  845350, 30, 40, 37, }, // Kamchatka
    {  625, 1058,       0,  845387,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  625, 1077,       0,  845410,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  625, 1106,  845433,  845445,  845467, 12, 22, 19, }, // Korea
    {  625, 1112,       0,  845486,       0,  0, 13,  0, }, // Kosrae
    {  625, 1119,  845499,  845517,  845545, 18, 28, 25, }, // Krasnoyarsk
    {  625, 1151,       0,  845570,       0,  0, 17,  0, }, // Kyrgystan
    {  625, 1161,       0,  845587,       0,  0, 16,  0, }, // Lanka
    {  625, 1175,       0,  845603,       0,  0, 20,  0, }, // Line_Islands
    {  625, 1188,  845623,  845637,  845661, 14, 24, 21, }, // Lord_Howe
    {  625, 1198,  845682,  845694,  845716, 12, 22, 19, }, // Macau
    {  625, 1204,  845735,  845748,  845771, 13, 23, 20, }, // Magadan
    {  625, 1219,       0,  845791,       0,  0, 15,  0, }, // Malaysia
    {  625, 1228,       0,  845806,       0,  0, 13,  0, }, // Maldives
    {  625, 1237,       0,  845819,       0,  0, 12,  0, }, // Marquesas
    {  625, 1247,       0,  845831,       0,  0, 22,  0, }, // Marshall_Islands
    {  625, 1264,  845853,  845869,  845895, 16, 26, 23, }, // Mauritius
    {  625, 1274,       0,  845918,       0,  0, 12,  0, }, // Mawson
    {  625, 1281,  845930,  845955,  845990, 25, 35, 32, }, // Mexico_Pacific
    {  625, 1296,  846022,  846038,  846064, 16, 26, 23, }, // Mongolia
    {  625, 1305,  846087,  846099,  846121, 12, 22, 19, }, // Moscow
    {  625, 1312,       0,  846140,       0,  0, 11,  0, }, // Myanmar
    {  625, 1320,       0,  846151,       0,  0, 11,  0, }, // Nauru
    {  625, 1326,       0,  846162,       0,  0, 11,  0, }, // Nepal
    {  625, 1332,  846173,  846194,  846225, 21, 31, 28, }, // New_Caledonia
    {  625, 1346,  846253,  846273,  846303, 20, 30, 27, }, // New_Zealand
    {  625, 1358,  846330,  846348,  846376, 18, 28, 25, }, // Newfoundland
    {  625, 1371,       0,  846401,       0,  0, 12,  0, }, // Niue
    {  625, 1376,  846413,  846436,  846469, 23, 33, 30, }, // Norfolk
    {  625, 1384,  846499,  846521,  846553, 22, 32, 29, }, // Noronha
    {  625, 1392,       0,  846582,       0,  0, 32,  0, }, // North_Mariana
    {  625, 1406,  846614,  846631,  846658, 17, 27, 24, }, // Novosibirsk
    {  625, 1418,  846682,  846693,  846714, 11, 21, 18, }, // Omsk
    {  625, 1428,  846732,  846746,  846770, 14, 24, 21, }, // Pakistan
    {  625, 1437,       0,  846791,       0,  0, 11,  0, }, // Palau
    {  625, 1443,       0,  846802,       0,  0, 28,  0, }, // Papua_New_Guinea
    {  625, 1460,  846830,  846844,  846868, 14, 24, 21, }, // Paraguay
    {  625, 1469,  846889,  846899,  846919, 10, 20, 17, }, // Peru
    {  625, 1474,  846936,  846950,  846974, 14, 24, 21, }, // Philippines
    {  625, 1486,       0,  846995,       0,  0, 22,  0, }, // Phoenix_Islands
    {  625, 1502,  847017,  847046,  847085, 29, 39, 36, }, // Pierre_Miquelon
    {  625, 1518,       0,  847121,       0,  0, 13,  0, }, // Pitcairn
    {  625, 1527,       0,  847134,       0,  0, 12,  0, }, // Ponape
    {  625, 1544,  847146,  847162,  847188, 16, 26, 23, }, // Qyzylorda
    {  625, 1554,       0,  847211,       0,  0, 15,  0, }, // Reunion
    {  625, 1562,       0,  847226,       0,  0, 12,  0, }, // Rothera
    {  625, 1570,  847238,  847251,  847274, 13, 23, 20, }, // Sakhalin
    {  625, 1579,  847294,  847306,  847328, 12, 22, 19, }, // Samara
    {  625, 1596,  847347,  847359,  847381, 12, 22, 22, }, // Samoa
    {  625, 1602,       0,  847403,       0,  0, 12,  0, }, // Seychelles
    {  625, 1624,       0,  847415,       0,  0, 14,  0, }, // Singapore
    {  625, 1634,       0,  847429,       0,  0, 23,  0, }, // Solomon
    {  625, 1642,       0,  847452,       0,  0, 21,  0, }, // South_Georgia
    {  625, 1656,       0,  847473,       0,  0, 13,  0, }, // Suriname
    {  625, 1676,       0,  847486,       0,  0, 10,  0, }, // Syowa
    {  625, 1682,       0,  847496,       0,  0, 11,  0, }, // Tahiti
    {  625, 1689,  847507,  847519,  847541, 12, 22, 19, }, // Taipei
    {  625, 1696,       0,  847560,       0,  0, 16,  0, }, // Tajikistan
    {  625, 1724,       0,  847576,       0,  0, 13,  0, }, // Tokelau
    {  625, 1732,  847589,  847600,  847621, 11, 21, 18, }, // Tonga
    {  625, 1738,       0,  847639,       0,  0,  9,  0, }, // Truk
    {  625, 1750,  847648,  847666,  847694, 18, 28, 25, }, // Turkmenistan
    {  625, 1763,       0,  847719,       0,  0, 12,  0, }, // Tuvalu
    {  625, 1777,  847731,  847745,  847769, 14, 24, 21, }, // Uruguay
    {  625, 1792,  847790,  847807,  847834, 17, 27, 24, }, // Uzbekistan
    {  625, 1803,  847858,  847872,  847896, 14, 24, 21, }, // Vanuatu
    {  625, 1811,       0,  847917,       0,  0, 16,  0, }, // Venezuela
    {  625, 1821,  847933,  847951,  847979, 18, 28, 25, }, // Vladivostok
    {  625, 1833,  848004,  848019,  848044, 15, 25, 22, }, // Volgograd
    {  625, 1843,       0,  847939,       0,  0, 12,  0, }, // Vostok
    {  625, 1850,       0,  848066,       0,  0, 17,  0, }, // Wake
    {  625, 1855,       0,  848083,       0,  0, 22,  0, }, // Wallis
    {  625, 1862,  848105,  848118,  848141, 13, 23, 20, }, // Yakutsk
    {  625, 1870,  848161,  848180,  848209, 19, 29, 26, }, // Yekaterinburg
    {  626,    5,       0,  848235,       0,  0, 17,  0, }, // Afghanistan Uzbek/Latin/Uzbekistan
    {  626,   17,       0,  848252,       0,  0, 21,  0, }, // Africa_Central
    {  626,   32,       0,  848273,       0,  0, 20,  0, }, // Africa_Eastern
    {  626,   65,       0,  848293,       0,  0, 29,  0, }, // Africa_Southern
    {  626,   81,  848322,  848342,  848371, 20, 29, 26, }, // Africa_Western
    {  626,  107,  848397,  848410,  848432, 13, 22, 19, }, // Alaska
    {  626,  135,  848451,  848465,  848488, 14, 23, 20, }, // Amazon
    {  626,  142,  848508,  848530,  848561, 22, 31, 28, }, // America_Central
    {  626,  158,  848589,  848610,  848640, 21, 30, 27, }, // America_Eastern
    {  626,  174,  848667,  848684,  848710, 17, 26, 23, }, // America_Mountain
    {  626,  191,  848733,  848751,  848778, 18, 27, 24, }, // America_Pacific
    {  626,  214,  848802,  848812,  848831, 10, 19, 16, }, // Apia
    {  626,  232,  848847,  848871,  848904, 24, 33, 30, }, // Arabian
    {  626,  240,  848934,  848949,  848973, 15, 24, 21, }, // Argentina
    {  626,  250,  848994,  849017,  849049, 23, 32, 29, }, // Argentina_Western
    {  626,  268,  849078,  849094,  849119, 16, 25, 22, }, // Armenia
    {  626,  286,  849141,  849156,  849180, 15, 24, 21, }, // Atlantic
    {  626,  295,  849201,  849226,  849260, 25, 34, 31, }, // Australia_Central
    {  626,  313,  849291,  849324,  849366, 33, 42, 39, }, // Australia_CentralWestern
    {  626,  338,  849405,  849429,  849462, 24, 33, 30, }, // Australia_Eastern
    {  626,  356,  849492,  849516,  849549, 24, 33, 30, }, // Australia_Western
    {  626,  374,  849579,  849595,  849620, 16, 25, 22, }, // Azerbaijan
    {  626,  385,  849642,  849661,  849689, 19, 28, 25, }, // Azores
    {  626,  397,  849714,  849730,  849755, 16, 25, 22, }, // Bangladesh
    {  626,  415,       0,  849777,       0,  0, 11,  0, }, // Bhutan
    {  626,  422,       0,  849788,       0,  0, 14,  0, }, // Bolivia
    {  626,  437,  849802,  849817,  849841, 15, 24, 21, }, // Brasilia
    {  626,  454,       0,  849862,       0,  0, 23,  0, }, // Brunei
    {  626,  461,  849885,  849901,  849926, 16, 25, 22, }, // Cape_Verde
    {  626,  478,       0,  849948,       0,  0, 23,  0, }, // Chamorro
    {  626,  487,  849971,  849983,  850004, 12, 21, 18, }, // Chatham
    {  626,  495,  850022,  850033,  850053, 11, 20, 17, }, // Chile
    {  626,  501,  850070,  850081,  850101, 11, 20, 17, }, // China
    {  626,  507,       0,  850118,       0,  0, 21,  0, }, // Christmas
    {  626,  517,       0,  850139,       0,  0, 20,  0, }, // Cocos
    {  626,  523,  850159,  850174,  850198, 15, 24, 21, }, // Colombia
    {  626,  532,  850219,  850237,  850264, 18, 27, 30, }, // Cook
    {  626,  537,  850294,  850304,  850323, 10, 19, 16, }, // Cuba
    {  626,  548,       0,  850339,       0,  0, 12,  0, }, // Davis
    {  626,  564,       0,  850351,       0,  0, 21,  0, }, // DumontDUrville
    {  626,  601,       0,  850372,       0,  0, 19,  0, }, // East_Timor
    {  626,  612,  850391,  850408,  850434, 17, 26, 23, }, // Easter
    {  626,  619,       0,  850457,       0,  0, 13,  0, }, // Ecuador
    {  626,  627,  850470,  850492,  850523, 22, 31, 28, }, // Europe_Central
    {  626,  642,  850551,  850572,  850602, 21, 30, 27, }, // Europe_Eastern
    {  626,  657,       0,  850629,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  626,  680,  850655,  850676,  850706, 21, 30, 27, }, // Europe_Western
    {  626,  695,  850733,  850756,  850788, 23, 32, 29, }, // Falkland
    {  626,  704,  850817,  850827,  850846, 10, 19, 16, }, // Fiji
    {  626,  709,       0,  850862,       0,  0, 22,  0, }, // French_Guiana
    {  626,  723,       0,  850884,       0,  0, 45,  0, }, // French_Southern
    {  626,  746,       0,  850929,       0,  0, 15,  0, }, // Galapagos
    {  626,  756,       0,  850944,       0,  0, 12,  0, }, // Gambier
    {  626,  764,  850956,  850969,  850991, 13, 22, 19, }, // Georgia
    {  626,  772,       0,  851010,       0,  0, 22,  0, }, // Gilbert_Islands
    {  626,  788,       0,  851032,       0,  0, 23,  0, }, // GMT
    {  626,  830,  851055,  851080,  851114, 25, 34, 31, }, // Greenland_Eastern
    {  626,  848,  851145,  851170,  851204, 25, 34, 31, }, // Greenland_Western
    {  626,  871,       0,  851235,       0,  0, 28,  0, }, // Gulf
    {  626,  876,       0,  851263,       0,  0, 12,  0, }, // Guyana
    {  626,  883,  851275,  851293,  851320, 18, 27, 24, }, // Hawaii_Aleutian
    {  626,  899,  851344,  851357,  851379, 13, 22, 19, }, // Hong_Kong
    {  626,  909,  851398,  851408,  851427, 10, 19, 16, }, // Hovd
    {  626,  914,       0,  851443,       0,  0, 24,  0, }, // India
    {  626,  920,       0,  851467,       0,  0, 17,  0, }, // Indian_Ocean
    {  626,  933,       0,  851484,       0,  0, 16,  0, }, // Indochina
    {  626,  943,       0,  851500,       0,  0, 25,  0, }, // Indonesia_Central
    {  626,  961,       0,  851525,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  626,  979,       0,  851549,       0,  0, 24,  0, }, // Indonesia_Western
    {  626,  997,  851573,  851583,  851602, 10, 19, 16, }, // Iran
    {  626, 1008,  851618,  851631,  851653, 13, 22, 19, }, // Irkutsk
    {  626, 1016,  851672,  851684,  851705, 12, 21, 18, }, // Israel
    {  626, 1023,  851723,  851737,  851760, 14, 23, 20, }, // Japan
    {  626, 1047,       0,  851780,       0,  0, 17,  0, }, // Kazakhstan
    {  626, 1058,       0,  851797,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {  626, 1077,       0,  851822,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  626, 1106,  851847,  851859,  851880, 12, 21, 18, }, // Korea
    {  626, 1112,       0,  851898,       0,  0, 12,  0, }, // Kosrae
    {  626, 1119,  851910,  851927,  851953, 17, 26, 23, }, // Krasnoyarsk
    {  626, 1151,       0,  851976,       0,  0, 18,  0, }, // Kyrgystan
    {  626, 1175,       0,  851994,       0,  0, 19,  0, }, // Line_Islands
    {  626, 1188,  852013,  852027,  852050, 14, 23, 20, }, // Lord_Howe
    {  626, 1204,  852070,  852083,  852105, 13, 22, 19, }, // Magadan
    {  626, 1219,       0,  852124,       0,  0, 15,  0, }, // Malaysia
    {  626, 1228,       0,  852139,       0,  0, 21,  0, }, // Maldives
    {  626, 1237,       0,  852160,       0,  0, 21,  0, }, // Marquesas
    {  626, 1247,       0,  852181,       0,  0, 23,  0, }, // Marshall_Islands
    {  626, 1264,  852204,  852218,  852241, 14, 23, 20, }, // Mauritius
    {  626, 1274,       0,  852261,       0,  0, 12,  0, }, // Mawson
    {  626, 1281,  852273,  852299,  852334, 26, 35, 32, }, // Mexico_Pacific
    {  626, 1296,  852366,  852382,  852407, 16, 25, 22, }, // Mongolia
    {  626, 1305,  852429,  852441,  852462, 12, 21, 18, }, // Moscow
    {  626, 1312,       0,  852480,       0,  0, 12,  0, }, // Myanmar
    {  626, 1320,       0,  852492,       0,  0, 11,  0, }, // Nauru
    {  626, 1326,       0,  852503,       0,  0, 11,  0, }, // Nepal
    {  626, 1332,  852514,  852536,  852567, 22, 31, 28, }, // New_Caledonia
    {  626, 1346,  852595,  852616,  852646, 21, 30, 27, }, // New_Zealand
    {  626, 1358,  852673,  852691,  852718, 18, 27, 24, }, // Newfoundland
    {  626, 1371,       0,  852742,       0,  0, 11,  0, }, // Niue
    {  626, 1376,  852753,  852772,  852800, 19, 28, 25, }, // Norfolk
    {  626, 1384,  852825,  852850,  852884, 25, 34, 31, }, // Noronha
    {  626, 1406,  852915,  852932,  852958, 17, 26, 23, }, // Novosibirsk
    {  626, 1418,  852981,  852991,  853010, 10, 19, 16, }, // Omsk
    {  626, 1428,  853026,  853040,  853063, 14, 23, 20, }, // Pakistan
    {  626, 1437,       0,  853083,       0,  0, 11,  0, }, // Palau
    {  626, 1443,       0,  853094,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  626, 1460,  853119,  853133,  853156, 14, 23, 20, }, // Paraguay
    {  626, 1469,  853176,  853186,  853205, 10, 19, 16, }, // Peru
    {  626, 1474,  853221,  853235,  853258, 14, 23, 20, }, // Philippines
    {  626, 1486,       0,  853278,       0,  0, 21,  0, }, // Phoenix_Islands
    {  626, 1502,  853299,  853324,  853358, 25, 34, 31, }, // Pierre_Miquelon
    {  626, 1518,       0,  853389,       0,  0, 13,  0, }, // Pitcairn
    {  626, 1527,       0,  853402,       0,  0, 12,  0, }, // Ponape
    {  626, 1534,       0,  853414,       0,  0, 13,  0, }, // Pyongyang
    {  626, 1554,       0,  853427,       0,  0, 14,  0, }, // Reunion
    {  626, 1562,       0,  853441,       0,  0, 12,  0, }, // Rothera
    {  626, 1570,  853453,  853466,  853488, 13, 22, 19, }, // Sakhalin
    {  626, 1596,  853507,  853518,  853538, 11, 20, 17, }, // Samoa
    {  626, 1602,       0,  853555,       0,  0, 22,  0, }, // Seychelles
    {  626, 1624,       0,  853577,       0,  0, 14,  0, }, // Singapore
    {  626, 1634,       0,  853591,       0,  0, 22,  0, }, // Solomon
    {  626, 1642,       0,  853613,       0,  0, 22,  0, }, // South_Georgia
    {  626, 1656,       0,  853635,       0,  0, 13,  0, }, // Suriname
    {  626, 1676,       0,  853648,       0,  0, 11,  0, }, // Syowa
    {  626, 1682,       0,  853659,       0,  0, 11,  0, }, // Tahiti
    {  626, 1689,  853670,  853682,  853703, 12, 21, 18, }, // Taipei
    {  626, 1696,       0,  853721,       0,  0, 16,  0, }, // Tajikistan
    {  626, 1724,       0,  853737,       0,  0, 13,  0, }, // Tokelau
    {  626, 1732,  853750,  853761,  853781, 11, 20, 17, }, // Tonga
    {  626, 1738,       0,  853798,       0,  0, 11,  0, }, // Truk
    {  626, 1750,  853809,  853827,  853854, 18, 27, 24, }, // Turkmenistan
    {  626, 1763,       0,  853878,       0,  0, 12,  0, }, // Tuvalu
    {  626, 1777,  853890,  853903,  853925, 13, 22, 19, }, // Uruguay
    {  626, 1792,  853944,  853961,  853987, 17, 26, 23, }, // Uzbekistan
    {  626, 1803,  854010,  854023,  854045, 13, 22, 19, }, // Vanuatu
    {  626, 1811,       0,  854064,       0,  0, 15,  0, }, // Venezuela
    {  626, 1821,  854079,  854096,  854122, 17, 26, 23, }, // Vladivostok
    {  626, 1833,  854145,  854160,  854184, 15, 24, 21, }, // Volgograd
    {  626, 1843,       0,  854205,       0,  0, 12,  0, }, // Vostok
    {  626, 1850,       0,  854217,       0,  0, 16,  0, }, // Wake
    {  626, 1855,       0,  854233,       0,  0, 22,  0, }, // Wallis
    {  626, 1862,  854255,  854268,  854290, 13, 22, 19, }, // Yakutsk
    {  626, 1870,  854309,  854328,  854356, 19, 28, 25, }, // Yekaterinburg
    {  626, 1892,       0,  854381,       0,  0, 11,  0, }, // Yukon
    {  627,    5,       0,  854392,       0,  0, 14,  0, }, // Afghanistan Uzbek/Arabic/Afghanistan
    {  628,    5,       0,  854406,       0,  0, 16,  0, }, // Afghanistan Uzbek/Cyrillic/Uzbekistan
    {  628,   17,       0,  854422,       0,  0, 21,  0, }, // Africa_Central
    {  628,   32,       0,  854443,       0,  0, 19,  0, }, // Africa_Eastern
    {  628,   65,       0,  854462,       0,  0, 20,  0, }, // Africa_Southern
    {  628,   81,  854482,  854501,  854529, 19, 28, 24, }, // Africa_Western
    {  628,  107,  854553,  854565,  854586, 12, 21, 21, }, // Alaska
    {  628,  135,  854607,  854621,  854644, 14, 23, 19, }, // Amazon
    {  628,  142,  854663,  854678,  854717, 15, 39, 39, }, // America_Central
    {  628,  158,  854756,  854784,  854821, 28, 37, 37, }, // America_Eastern
    {  628,  174,  854858,  854883,  854917, 25, 34, 34, }, // America_Mountain
    {  628,  191,  854951,  854984,  855026, 33, 42, 42, }, // America_Pacific
    {  628,  232,  855068,  855083,  855107, 15, 24, 24, }, // Arabian
    {  628,  240,  855131,  855146,  855170, 15, 24, 20, }, // Argentina
    {  628,  250,  855190,  855212,  855243, 22, 31, 27, }, // Argentina_Western
    {  628,  268,  855270,  855286,  855311, 16, 25, 21, }, // Armenia
    {  628,  286,  855332,  855347,  855371, 15, 24, 24, }, // Atlantic
    {  628,  295,  855395,  855419,  855452, 24, 33, 33, }, // Australia_Central
    {  628,  313,  855485,  855516,  855556, 31, 40, 40, }, // Australia_CentralWestern
    {  628,  338,  855596,  855618,  855649, 22, 31, 31, }, // Australia_Eastern
    {  628,  356,  855680,  855702,  855733, 22, 31, 31, }, // Australia_Western
    {  628,  374,  855764,  855780,  855805, 16, 25, 21, }, // Azerbaijan
    {  628,  385,  855826,  855836,  855855, 10, 19, 15, }, // Azores
    {  628,  397,  855870,  855885,  855909, 15, 24, 20, }, // Bangladesh
    {  628,  415,       0,  855929,       0,  0, 11,  0, }, // Bhutan
    {  628,  422,       0,  855940,       0,  0, 13,  0, }, // Bolivia
    {  628,  437,  855953,  855967,  855990, 14, 23, 19, }, // Brasilia
    {  628,  454,       0,  856009,       0,  0, 23,  0, }, // Brunei
    {  628,  461,  856032,  856048,  856073, 16, 25, 21, }, // Cape_Verde
    {  628,  478,       0,  856094,       0,  0, 13,  0, }, // Chamorro
    {  628,  487,  856107,  856119,  856140, 12, 21, 21, }, // Chatham
    {  628,  495,  856161,  856171,  856190, 10, 19, 15, }, // Chile
    {  628,  501,  856205,  856216,  856236, 11, 20, 20, }, // China
    {  628,  507,       0,  856256,       0,  0, 21,  0, }, // Christmas
    {  628,  517,       0,  856277,       0,  0, 20,  0, }, // Cocos
    {  628,  523,  856297,  856311,  856334, 14, 23, 19, }, // Colombia
    {  628,  532,  856353,  856371,  856398, 18, 27, 28, }, // Cook
    {  628,  537,  856426,  856436,  856455, 10, 19, 19, }, // Cuba
    {  628,  548,       0,  856474,       0,  0, 11,  0, }, // Davis
    {  628,  564,       0,  856485,       0,  0, 20,  0, }, // DumontDUrville
    {  628,  601,       0,  856505,       0,  0, 18,  0, }, // East_Timor
    {  628,  612,  856523,  856540,  856566, 17, 26, 22, }, // Easter
    {  628,  619,       0,  856588,       0,  0, 13,  0, }, // Ecuador
    {  628,  627,  856601,  856622,  856652, 21, 30, 26, }, // Europe_Central
    {  628,  642,  856678,  856697,  856725, 19, 28, 24, }, // Europe_Eastern
    {  628,  680,  856749,  856768,  856796, 19, 28, 24, }, // Europe_Western
    {  628,  695,  856820,  856844,  856877, 24, 33, 29, }, // Falkland
    {  628,  704,  856906,  856916,  856935, 10, 19, 15, }, // Fiji
    {  628,  709,       0,  856950,       0,  0, 22,  0, }, // French_Guiana
    {  628,  723,       0,  856972,       0,  0, 35,  0, }, // French_Southern
    {  628,  746,       0,  857007,       0,  0, 15,  0, }, // Galapagos
    {  628,  756,       0,  857022,       0,  0, 13,  0, }, // Gambier
    {  628,  764,  857035,  857047,  857068, 12, 21, 17, }, // Georgia
    {  628,  772,       0,  857085,       0,  0, 22,  0, }, // Gilbert_Islands
    {  628,  788,       0,  857107,       0,  0, 13,  0, }, // GMT
    {  628,  830,  857120,  857143,  857175, 23, 32, 28, }, // Greenland_Eastern
    {  628,  848,  857203,  857226,  857258, 23, 32, 28, }, // Greenland_Western
    {  628,  871,       0,  857286,       0,  0, 12,  0, }, // Gulf
    {  628,  876,       0,  857298,       0,  0, 12,  0, }, // Guyana
    {  628,  883,  857310,  857328,  857355, 18, 27, 27, }, // Hawaii_Aleutian
    {  628,  899,  857382,  857395,  857417, 13, 22, 18, }, // Hong_Kong
    {  628,  909,  857435,  857445,  857464, 10, 19, 15, }, // Hovd
    {  628,  914,       0,  857479,       0,  0, 15,  0, }, // India
    {  628,  920,       0,  857494,       0,  0, 17,  0, }, // Indian_Ocean
    {  628,  933,       0,  857511,       0,  0, 16,  0, }, // Indochina
    {  628,  943,       0,  857527,       0,  0, 24,  0, }, // Indonesia_Central
    {  628,  961,       0,  857551,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  628,  979,       0,  857573,       0,  0, 22,  0, }, // Indonesia_Western
    {  628,  997,  857595,  857605,  857624, 10, 19, 19, }, // Iran
    {  628, 1008,  857643,  857656,  857678, 13, 22, 18, }, // Irkutsk
    {  628, 1016,  857696,  857708,  857729, 12, 21, 21, }, // Israel
    {  628, 1023,  857750,  857762,  857783, 12, 21, 21, }, // Japan
    {  628, 1058,       0,  857804,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  628, 1077,       0,  857827,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  628, 1106,  857850,  857861,  857881, 11, 20, 20, }, // Korea
    {  628, 1112,       0,  857901,       0,  0, 12,  0, }, // Kosrae
    {  628, 1119,  857913,  857929,  857954, 16, 25, 21, }, // Krasnoyarsk
    {  628, 1151,       0,  857975,       0,  0, 17,  0, }, // Kyrgystan
    {  628, 1175,       0,  857992,       0,  0, 19,  0, }, // Line_Islands
    {  628, 1188,  858011,  858026,  858050, 15, 24, 24, }, // Lord_Howe
    {  628, 1204,  858074,  858087,  858109, 13, 22, 18, }, // Magadan
    {  628, 1219,       0,  858127,       0,  0, 14,  0, }, // Malaysia
    {  628, 1228,       0,  858141,       0,  0, 15,  0, }, // Maldives
    {  628, 1237,       0,  858156,       0,  0, 14,  0, }, // Marquesas
    {  628, 1247,       0,  858170,       0,  0, 22,  0, }, // Marshall_Islands
    {  628, 1264,  858192,  858206,  858229, 14, 23, 19, }, // Mauritius
    {  628, 1274,       0,  858248,       0,  0, 13,  0, }, // Mawson
    {  628, 1296,  858261,  858277,  858302, 16, 25, 21, }, // Mongolia
    {  628, 1305,  858323,  858335,  858356, 12, 21, 17, }, // Moscow
    {  628, 1312,       0,  858373,       0,  0, 12,  0, }, // Myanmar
    {  628, 1320,       0,  858385,       0,  0, 11,  0, }, // Nauru
    {  628, 1326,       0,  858396,       0,  0, 11,  0, }, // Nepal
    {  628, 1332,  858407,  858427,  858456, 20, 29, 25, }, // New_Caledonia
    {  628, 1346,  858481,  858500,  858528, 19, 28, 28, }, // New_Zealand
    {  628, 1358,  858556,  858574,  858601, 18, 27, 27, }, // Newfoundland
    {  628, 1371,       0,  858628,       0,  0, 10,  0, }, // Niue
    {  628, 1376,  858638,  858657,  858685, 19, 28, 24, }, // Norfolk
    {  628, 1384,  858709,  858734,  858768, 25, 34, 30, }, // Noronha
    {  628, 1406,  858798,  858815,  858841, 17, 26, 22, }, // Novosibirsk
    {  628, 1418,  765197,  858863,  858882, 10, 19, 15, }, // Omsk
    {  628, 1428,  858897,  858911,  858934, 14, 23, 19, }, // Pakistan
    {  628, 1437,       0,  858953,       0,  0, 11,  0, }, // Palau
    {  628, 1443,       0,  858964,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  628, 1460,  858987,  859001,  859024, 14, 23, 19, }, // Paraguay
    {  628, 1469,  859043,  859053,  859072, 10, 19, 15, }, // Peru
    {  628, 1474,  859087,  859101,  859124, 14, 23, 19, }, // Philippines
    {  628, 1486,       0,  859143,       0,  0, 21,  0, }, // Phoenix_Islands
    {  628, 1502,  859164,  859190,  859225, 26, 35, 35, }, // Pierre_Miquelon
    {  628, 1518,       0,  859260,       0,  0, 13,  0, }, // Pitcairn
    {  628, 1527,       0,  859273,       0,  0, 12,  0, }, // Ponape
    {  628, 1554,       0,  859285,       0,  0, 13,  0, }, // Reunion
    {  628, 1562,       0,  859298,       0,  0, 12,  0, }, // Rothera
    {  628, 1570,  859310,  859323,  859345, 13, 22, 18, }, // Sakhalin
    {  628, 1596,  859363,  859374,  859394, 11, 20, 20, }, // Samoa
    {  628, 1602,       0,  859414,       0,  0, 21,  0, }, // Seychelles
    {  628, 1624,       0,  859435,       0,  0, 14,  0, }, // Singapore
    {  628, 1634,       0,  859449,       0,  0, 22,  0, }, // Solomon
    {  628, 1642,       0,  859471,       0,  0, 22,  0, }, // South_Georgia
    {  628, 1656,       0,  859493,       0,  0, 13,  0, }, // Suriname
    {  628, 1676,       0,  859506,       0,  0, 11,  0, }, // Syowa
    {  628, 1682,       0,  859517,       0,  0, 11,  0, }, // Tahiti
    {  628, 1689,  859528,  859540,  859561, 12, 21, 21, }, // Taipei
    {  628, 1696,       0,  859582,       0,  0, 16,  0, }, // Tajikistan
    {  628, 1724,       0,  859598,       0,  0, 13,  0, }, // Tokelau
    {  628, 1732,  859611,  859622,  859642, 11, 20, 16, }, // Tonga
    {  628, 1738,       0,  859658,       0,  0, 10,  0, }, // Truk
    {  628, 1750,  859668,  859686,  859713, 18, 27, 23, }, // Turkmenistan
    {  628, 1763,       0,  859736,       0,  0, 12,  0, }, // Tuvalu
    {  628, 1777,  859748,  859761,  859783, 13, 22, 18, }, // Uruguay
    {  628, 1792,  859801,  859817,  859842, 16, 25, 21, }, // Uzbekistan
    {  628, 1803,  859863,  859876,  859898, 13, 22, 18, }, // Vanuatu
    {  628, 1811,       0,  859916,       0,  0, 15,  0, }, // Venezuela
    {  628, 1821,  859931,  859948,  859974, 17, 26, 22, }, // Vladivostok
    {  628, 1833,  859996,  860011,  860035, 15, 24, 20, }, // Volgograd
    {  628, 1843,       0,  860055,       0,  0, 12,  0, }, // Vostok
    {  628, 1850,       0,  860067,       0,  0, 16,  0, }, // Wake
    {  628, 1855,       0,  860083,       0,  0, 22,  0, }, // Wallis
    {  628, 1862,  766561,  860105,  860126, 12, 21, 17, }, // Yakutsk
    {  628, 1870,  860143,  860161,  860188, 18, 27, 23, }, // Yekaterinburg
    {  632,    0,  860211,  860219,  860233,  8, 14, 15, }, // Acre Vietnamese/Latin/Vietnam
    {  632,    5,       0,  860248,       0,  0, 15,  0, }, // Afghanistan
    {  632,   17,       0,  860263,       0,  0, 13,  0, }, // Africa_Central
    {  632,   32,       0,  860276,       0,  0, 12,  0, }, // Africa_Eastern
    {  632,   65,       0,  860288,       0,  0, 17,  0, }, // Africa_Southern
    {  632,   81,  860305,  860316,  860333, 11, 17, 18, }, // Africa_Western
    {  632,  107,  860351,  860361,  860377, 10, 16, 17, }, // Alaska
    {  632,  128,  860394,  860404,  860420, 10, 16, 17, }, // Almaty
    {  632,  135,  860437,  860447,  860463, 10, 16, 17, }, // Amazon
    {  632,  142,  860480,  860494,  860514, 14, 20, 21, }, // America_Central
    {  632,  158,  860535,  860548,  860567, 13, 19, 20, }, // America_Eastern
    {  632,  174,  860587,  860599,  860617, 12, 18, 19, }, // America_Mountain
    {  632,  191,  860636,  860655,  860680, 19, 25, 26, }, // America_Pacific
    {  632,  207,  860706,  860716,  860732, 10, 16, 17, }, // Anadyr
    {  632,  214,  860749,  860757,  860771,  8, 14, 15, }, // Apia
    {  632,  219,  860786,  860795,  860810,  9, 15, 16, }, // Aqtau
    {  632,  225,  860826,  860836,  860852, 10, 16, 17, }, // Aqtobe
    {  632,  232,  860869,  860878,  860893,  9, 15, 16, }, // Arabian
    {  632,  240,  860909,  860922,  860941, 13, 19, 20, }, // Argentina
    {  632,  250,  860961,  860983,  861011, 22, 28, 29, }, // Argentina_Western
    {  632,  268,  861040,  861051,  861068, 11, 17, 18, }, // Armenia
    {  632,  286,  861086,  861103,  861126, 17, 23, 24, }, // Atlantic
    {  632,  295,  861150,  861174,  861204, 24, 30, 31, }, // Australia_Central
    {  632,  313,  861235,  861263,  861297, 28, 34, 35, }, // Australia_CentralWestern
    {  632,  338,  861332,  861355,  861384, 23, 29, 30, }, // Australia_Eastern
    {  632,  356,  861414,  861436,  861464, 22, 28, 29, }, // Australia_Western
    {  632,  374,  861493,  861507,  861527, 14, 20, 21, }, // Azerbaijan
    {  632,  385,  861548,  861558,  861574, 10, 16, 17, }, // Azores
    {  632,  397,  861591,  861605,  861625, 14, 20, 21, }, // Bangladesh
    {  632,  415,       0,  861646,       0,  0, 10,  0, }, // Bhutan
    {  632,  422,       0,  861656,       0,  0, 11,  0, }, // Bolivia
    {  632,  437,  861667,  861679,  861697, 12, 18, 19, }, // Brasilia
    {  632,  454,       0,  861716,       0,  0, 21,  0, }, // Brunei
    {  632,  461,  861737,  861751,  861771, 14, 20, 21, }, // Cape_Verde
    {  632,  472,       0,  861792,       0,  0,  9,  0, }, // Casey
    {  632,  478,       0,  861801,       0,  0, 12,  0, }, // Chamorro
    {  632,  487,  861813,  861824,  861841, 11, 17, 18, }, // Chatham
    {  632,  495,  861859,  861868,  861883,  9, 15, 16, }, // Chile
    {  632,  501,  861899,  861913,  861933, 14, 20, 21, }, // China
    {  632,  507,       0,  861954,       0,  0, 17,  0, }, // Christmas
    {  632,  517,       0,  861971,       0,  0, 18,  0, }, // Cocos
    {  632,  523,  861989,  862001,  862019, 12, 18, 19, }, // Colombia
    {  632,  532,  862038,  862055,  862078, 17, 23, 28, }, // Cook
    {  632,  537,  862106,  862114,  862128,  8, 14, 15, }, // Cuba
    {  632,  548,       0,  862143,       0,  0,  9,  0, }, // Davis
    {  632,  564,       0,  862152,       0,  0, 20,  0, }, // DumontDUrville
    {  632,  601,       0,  862172,       0,  0, 14,  0, }, // East_Timor
    {  632,  612,  862186,  862203,  862226, 17, 23, 24, }, // Easter
    {  632,  619,       0,  862250,       0,  0, 11,  0, }, // Ecuador
    {  632,  627,  862261,  862273,  862291, 12, 18, 19, }, // Europe_Central
    {  632,  642,  862310,  862321,  862338, 11, 17, 18, }, // Europe_Eastern
    {  632,  657,       0,  862356,       0,  0, 21,  0, }, // Europe_Further_Eastern
    {  632,  680,  862377,  862387,  862403, 10, 16, 17, }, // Europe_Western
    {  632,  695,  862420,  862441,  862468, 21, 27, 28, }, // Falkland
    {  632,  704,  862496,  862504,  862518,  8, 14, 15, }, // Fiji
    {  632,  709,       0,  862533,       0,  0, 21,  0, }, // French_Guiana
    {  632,  723,       0,  862554,       0,  0, 28,  0, }, // French_Southern
    {  632,  746,       0,  862582,       0,  0, 13,  0, }, // Galapagos
    {  632,  756,       0,  862595,       0,  0, 11,  0, }, // Gambier
    {  632,  764,  862606,  862617,  862634, 11, 17, 18, }, // Georgia
    {  632,  772,       0,  862652,       0,  0, 20,  0, }, // Gilbert_Islands
    {  632,  788,       0,  862672,       0,  0, 24,  0, }, // GMT
    {  632,  830,  862696,  862719,  862748, 23, 29, 30, }, // Greenland_Eastern
    {  632,  848,  862778,  862800,  862828, 22, 28, 29, }, // Greenland_Western
    {  632,  866,       0,  862857,       0,  0, 14,  0, }, // Guam
    {  632,  871,       0,  862871,       0,  0, 19,  0, }, // Gulf
    {  632,  876,       0,  862890,       0,  0, 10,  0, }, // Guyana
    {  632,  883,  862900,  862916,  862938, 16, 22, 23, }, // Hawaii_Aleutian
    {  632,  899,  862961,  862974,  862993, 13, 19, 20, }, // Hong_Kong
    {  632,  909,  863013,  863021,  863035,  8, 14, 15, }, // Hovd
    {  632,  914,       0,  863050,       0,  0, 15,  0, }, // India
    {  632,  920,       0,  863065,       0,  0, 15,  0, }, // Indian_Ocean
    {  632,  933,       0,  863080,       0,  0, 14,  0, }, // Indochina
    {  632,  943,       0,  863094,       0,  0, 24,  0, }, // Indonesia_Central
    {  632,  961,       0,  863118,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  632,  979,       0,  863141,       0,  0, 22,  0, }, // Indonesia_Western
    {  632,  997,  863163,  863171,  863185,  8, 14, 15, }, // Iran
    {  632, 1008,  863200,  863211,  863228, 11, 17, 18, }, // Irkutsk
    {  632, 1016,  863246,  863256,  863272, 10, 16, 17, }, // Israel
    {  632, 1023,  863289,  863301,  863319, 12, 18, 19, }, // Japan
    {  632, 1029,  863338,  863366,  863400, 28, 34, 35, }, // Kamchatka
    {  632, 1047,       0,  863435,       0,  0, 14,  0, }, // Kazakhstan
    {  632, 1058,       0,  863449,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  632, 1077,       0,  863473,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  632, 1106,  863496,  863508,  863526, 12, 18, 19, }, // Korea
    {  632, 1112,       0,  863545,       0,  0, 10,  0, }, // Kosrae
    {  632, 1119,  863555,  863570,  863591, 15, 21, 22, }, // Krasnoyarsk
    {  632, 1151,       0,  863613,       0,  0, 13,  0, }, // Kyrgystan
    {  632, 1161,       0,  863626,       0,  0,  9,  0, }, // Lanka
    {  632, 1175,       0,  863635,       0,  0, 17,  0, }, // Line_Islands
    {  632, 1188,  863652,  863665,  863684, 13, 19, 20, }, // Lord_Howe
    {  632, 1198,  863704,  863714,  863730, 10, 16, 17, }, // Macau
    {  632, 1204,  863747,  863758,  863775, 11, 17, 18, }, // Magadan
    {  632, 1219,       0,  863793,       0,  0, 12,  0, }, // Malaysia
    {  632, 1228,       0,  863805,       0,  0, 12,  0, }, // Maldives
    {  632, 1237,       0,  863817,       0,  0, 13,  0, }, // Marquesas
    {  632, 1247,       0,  863830,       0,  0, 21,  0, }, // Marshall_Islands
    {  632, 1264,  863851,  863864,  863883, 13, 19, 20, }, // Mauritius
    {  632, 1274,       0,  863903,       0,  0, 10,  0, }, // Mawson
    {  632, 1281,  863913,  863939,  863971, 26, 32, 33, }, // Mexico_Pacific
    {  632, 1296,  864004,  864018,  864038, 14, 20, 21, }, // Mongolia
    {  632, 1305,  864059,  864071,  864089, 12, 18, 19, }, // Moscow
    {  632, 1312,       0,  864108,       0,  0, 11,  0, }, // Myanmar
    {  632, 1320,       0,  864119,       0,  0,  9,  0, }, // Nauru
    {  632, 1326,       0,  864128,       0,  0,  9,  0, }, // Nepal
    {  632, 1332,  864137,  864154,  864177, 17, 23, 24, }, // New_Caledonia
    {  632, 1346,  864201,  864216,  864237, 15, 21, 22, }, // New_Zealand
    {  632, 1358,  864259,  864275,  864297, 16, 22, 23, }, // Newfoundland
    {  632, 1371,       0,  864320,       0,  0,  8,  0, }, // Niue
    {  632, 1376,  864328,  864343,  864364, 15, 21, 22, }, // Norfolk
    {  632, 1384,  864386,  864409,  864438, 23, 29, 30, }, // Noronha
    {  632, 1392,       0,  864468,       0,  0, 24,  0, }, // North_Mariana
    {  632, 1406,  864492,  864507,  864528, 15, 21, 22, }, // Novosibirsk
    {  632, 1418,  864550,  864558,  864572,  8, 14, 15, }, // Omsk
    {  632, 1428,  864587,  864599,  864617, 12, 18, 19, }, // Pakistan
    {  632, 1437,       0,  864636,       0,  0,  9,  0, }, // Palau
    {  632, 1443,       0,  864645,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  632, 1460,  864665,  864677,  864695, 12, 18, 19, }, // Paraguay
    {  632, 1469,  864714,  864722,  864736,  8, 14, 15, }, // Peru
    {  632, 1474,  864751,  864764,  864783, 13, 19, 20, }, // Philippines
    {  632, 1486,       0,  864803,       0,  0, 20,  0, }, // Phoenix_Islands
    {  632, 1502,  864823,  864849,  864881, 26, 32, 35, }, // Pierre_Miquelon
    {  632, 1518,       0,  864916,       0,  0, 12,  0, }, // Pitcairn
    {  632, 1527,       0,  864928,       0,  0, 10,  0, }, // Ponape
    {  632, 1534,       0,  864938,       0,  0, 15,  0, }, // Pyongyang
    {  632, 1544,  864953,  864966,  864985, 13, 19, 20, }, // Qyzylorda
    {  632, 1554,       0,  865005,       0,  0, 11,  0, }, // Reunion
    {  632, 1562,       0,  865016,       0,  0, 11,  0, }, // Rothera
    {  632, 1570,  865027,  865039,  865057, 12, 18, 19, }, // Sakhalin
    {  632, 1579,  865076,  865086,  865102, 10, 16, 17, }, // Samara
    {  632, 1596,  865119,  865128,  865143,  9, 15, 18, }, // Samoa
    {  632, 1602,       0,  865161,       0,  0, 14,  0, }, // Seychelles
    {  632, 1624,       0,  865175,       0,  0, 13,  0, }, // Singapore
    {  632, 1634,       0,  865188,       0,  0, 20,  0, }, // Solomon
    {  632, 1642,       0,  865208,       0,  0, 15,  0, }, // South_Georgia
    {  632, 1656,       0,  865223,       0,  0, 12,  0, }, // Suriname
    {  632, 1676,       0,  865235,       0,  0,  9,  0, }, // Syowa
    {  632, 1682,       0,  865244,       0,  0, 10,  0, }, // Tahiti
    {  632, 1689,  865254,  865265,  865282, 11, 17, 18, }, // Taipei
    {  632, 1696,       0,  865300,       0,  0, 14,  0, }, // Tajikistan
    {  632, 1724,       0,  865314,       0,  0, 11,  0, }, // Tokelau
    {  632, 1732,  865325,  865334,  865349,  9, 15, 16, }, // Tonga
    {  632, 1738,       0,  865365,       0,  0,  9,  0, }, // Truk
    {  632, 1750,  865374,  865390,  865412, 16, 22, 23, }, // Turkmenistan
    {  632, 1763,       0,  865435,       0,  0, 10,  0, }, // Tuvalu
    {  632, 1777,  865445,  865456,  865473, 11, 17, 18, }, // Uruguay
    {  632, 1792,  865491,  865505,  865525, 14, 20, 21, }, // Uzbekistan
    {  632, 1803,  865546,  865557,  865574, 11, 17, 18, }, // Vanuatu
    {  632, 1811,       0,  865592,       0,  0, 13,  0, }, // Venezuela
    {  632, 1821,  865605,  865620,  865641, 15, 21, 22, }, // Vladivostok
    {  632, 1833,  865663,  865676,  865695, 13, 19, 20, }, // Volgograd
    {  632, 1843,       0,  865715,       0,  0, 10,  0, }, // Vostok
    {  632, 1850,       0,  865725,       0,  0, 12,  0, }, // Wake
    {  632, 1855,       0,  865737,       0,  0, 20,  0, }, // Wallis
    {  632, 1862,  865757,  865768,  865785, 11, 17, 18, }, // Yakutsk
    {  632, 1870,  865803,  865820,  865843, 17, 23, 24, }, // Yekaterinburg
    {  632, 1892,       0,  865867,       0,  0,  9,  0, }, // Yukon
    {  636,  286,  865876,  865889,  865910, 13, 21, 19, }, // Atlantic Walser/Latin/Switzerland
    {  636,  627,  865929,  865948,  865975, 19, 27, 25, }, // Europe_Central
    {  636,  642,  866000,  866016,  866040, 16, 24, 22, }, // Europe_Eastern
    {  636,  680,  866062,  866079,  866104, 17, 25, 23, }, // Europe_Western
    {  638,    5,       0,  866127,       0,  0, 17,  0, }, // Afghanistan Welsh/Latin/United Kingdom
    {  638,   17,       0,  866144,       0,  0, 24,  0, }, // Africa_Central
    {  638,   32,       0,  866168,       0,  0, 21,  0, }, // Africa_Eastern
    {  638,   65,       0,  866189,       0,  0, 24,  0, }, // Africa_Southern
    {  638,   81,  866213,  866236,  866267, 23, 31, 27, }, // Africa_Western
    {  638,  107,  866294,  866306,  866326, 12, 20, 16, }, // Alaska
    {  638,  135,  866342,  866356,  866378, 14, 22, 18, }, // Amazon
    {  638,  142,  866396,  866428,  866468, 32, 40, 36, }, // America_Central
    {  638,  158,  866504,  866533,  866570, 29, 37, 33, }, // America_Eastern
    {  638,  174,  866603,  866635,  866675, 32, 40, 36, }, // America_Mountain
    {  638,  191,  866711,  866746,  866789, 35, 43, 39, }, // America_Pacific
    {  638,  214,  866828,  866838,  866856, 10, 18, 14, }, // Apia
    {  638,  232,  866870,  866884,  866906, 14, 22, 18, }, // Arabian
    {  638,  240,  866924,  866941,  866963, 17, 22, 18, }, // Argentina
    {  638,  250,  866981,  867005,  867037, 24, 32, 28, }, // Argentina_Western
    {  638,  268,  867065,  867078,  867099, 13, 21, 17, }, // Armenia
    {  638,  286,  867116,  867140,  867172, 24, 32, 28, }, // Atlantic
    {  638,  295,  867200,  867226,  867260, 26, 34, 30, }, // Australia_Central
    {  638,  313,  867290,  867326,  867370, 36, 44, 40, }, // Australia_CentralWestern
    {  638,  338,  867410,  867433,  867464, 23, 31, 27, }, // Australia_Eastern
    {  638,  356,  867491,  867516,  867549, 25, 33, 29, }, // Australia_Western
    {  638,  374,  867578,  867594,  867618, 16, 24, 20, }, // Azerbaijan
    {  638,  385,  867638,  867653,  867676, 15, 23, 19, }, // Azores
    {  638,  397,  867695,  867711,  867735, 16, 24, 20, }, // Bangladesh
    {  638,  415,       0,  867755,       0,  0, 12,  0, }, // Bhutan
    {  638,  422,       0,  867767,       0,  0, 13,  0, }, // Bolivia
    {  638,  437,  867780,  867794,  867816, 14, 22, 18, }, // Brasilia
    {  638,  454,       0,  867834,       0,  0, 23,  0, }, // Brunei
    {  638,  461,  867857,  867873,  867897, 16, 24, 20, }, // Cape_Verde
    {  638,  478,       0,  867917,       0,  0, 14,  0, }, // Chamorro
    {  638,  487,  867931,  867944,  867965, 13, 21, 17, }, // Chatham
    {  638,  495,  867982,  867993,  868012, 11, 19, 15, }, // Chile
    {  638,  501,  868027,  868040,  868061, 13, 21, 17, }, // China
    {  638,  507,       0,  868078,       0,  0, 20,  0, }, // Christmas
    {  638,  517,       0,  868098,       0,  0, 20,  0, }, // Cocos
    {  638,  523,  868118,  868132,  868154, 14, 22, 18, }, // Colombia
    {  638,  532,  868172,  868191,  868218, 19, 27, 30, }, // Cook
    {  638,  537,  868248,  868259,  868278, 11, 19, 14, }, // Cuba
    {  638,  548,       0,  868292,       0,  0, 11,  0, }, // Davis
    {  638,  564,       0,  868303,       0,  0, 22,  0, }, // DumontDUrville
    {  638,  601,       0,  868325,       0,  0, 19,  0, }, // East_Timor
    {  638,  612,  868344,  868361,  868386, 17, 25, 21, }, // Easter
    {  638,  619,       0,  868407,       0,  0, 13,  0, }, // Ecuador
    {  638,  627,  868420,  868442,  868472, 22, 30, 26, }, // Europe_Central
    {  638,  642,  868498,  868517,  868544, 19, 27, 23, }, // Europe_Eastern
    {  638,  657,       0,  868567,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  638,  680,  868591,  868612,  868641, 21, 29, 25, }, // Europe_Western
    {  638,  695,  868666,  868699,  868740, 33, 41, 37, }, // Falkland
    {  638,  704,  868777,  868787,  868805, 10, 18, 14, }, // Fiji
    {  638,  709,       0,  868819,       0,  0, 21,  0, }, // French_Guiana
    {  638,  723,       0,  868840,       0,  0, 43,  0, }, // French_Southern
    {  638,  746,       0,  868883,       0,  0, 15,  0, }, // Galapagos
    {  638,  756,       0,  868898,       0,  0, 13,  0, }, // Gambier
    {  638,  764,  868911,  868924,  868945, 13, 21, 17, }, // Georgia
    {  638,  772,       0,  868962,       0,  0, 22,  0, }, // Gilbert_Islands
    {  638,  788,       0,  868984,       0,  0, 23,  0, }, // GMT
    {  638,  830,  869007,  869032,  869065, 25, 33, 29, }, // Greenland_Eastern
    {  638,  848,  869094,  869121,  869156, 27, 35, 31, }, // Greenland_Western
    {  638,  871,       0,  869187,       0,  0, 21,  0, }, // Gulf
    {  638,  876,       0,  869208,       0,  0, 12,  0, }, // Guyana
    {  638,  883,  869220,  869241,  869270, 21, 29, 25, }, // Hawaii_Aleutian
    {  638,  899,  869295,  869310,  869333, 15, 23, 19, }, // Hong_Kong
    {  638,  909,  869352,  869362,  869380, 10, 18, 14, }, // Hovd
    {  638,  914,       0,  869394,       0,  0, 11,  0, }, // India
    {  638,  920,       0,  869405,       0,  0, 19,  0, }, // Indian_Ocean
    {  638,  933,       0,  869424,       0,  0, 18,  0, }, // Indochina
    {  638,  943,       0,  869442,       0,  0, 26,  0, }, // Indonesia_Central
    {  638,  961,       0,  869468,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  638,  979,       0,  869491,       0,  0, 25,  0, }, // Indonesia_Western
    {  638,  997,  869516,  869526,  869544, 10, 18, 14, }, // Iran
    {  638, 1008,  869558,  869571,  869592, 13, 21, 17, }, // Irkutsk
    {  638, 1016,  869609,  869621,  869641, 12, 20, 16, }, // Israel
    {  638, 1023,  869657,  869668,  869687, 11, 19, 15, }, // Japan
    {  638, 1047,       0,  869702,       0,  0, 16,  0, }, // Kazakhstan
    {  638, 1058,       0,  869718,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  638, 1077,       0,  869742,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  638, 1106,  869768,  869779,  869798, 11, 19, 15, }, // Korea
    {  638, 1112,       0,  869813,       0,  0, 12,  0, }, // Kosrae
    {  638, 1119,  869825,  869842,  869867, 17, 25, 21, }, // Krasnoyarsk
    {  638, 1151,       0,  869888,       0,  0, 16,  0, }, // Kyrgystan
    {  638, 1175,       0,  869904,       0,  0, 19,  0, }, // Line_Islands
    {  638, 1188,  869923,  869945,  869975, 22, 30, 26, }, // Lord_Howe
    {  638, 1204,  870001,  870014,  870035, 13, 21, 17, }, // Magadan
    {  638, 1219,       0,  870052,       0,  0, 14,  0, }, // Malaysia
    {  638, 1228,       0,  870066,       0,  0, 16,  0, }, // Maldives
    {  638, 1237,       0,  870082,       0,  0, 15,  0, }, // Marquesas
    {  638, 1247,       0,  870097,       0,  0, 23,  0, }, // Marshall_Islands
    {  638, 1264,  870120,  870135,  870158, 15, 23, 19, }, // Mauritius
    {  638, 1274,       0,  870177,       0,  0, 12,  0, }, // Mawson
    {  638, 1281,  870189,  870211,  870241, 22, 30, 26, }, // Mexico_Pacific
    {  638, 1296,  870267,  870283,  870307, 16, 24, 20, }, // Mongolia
    {  638, 1305,  870327,  870339,  870359, 12, 20, 16, }, // Moscow
    {  638, 1312,       0,  870375,       0,  0, 13,  0, }, // Myanmar
    {  638, 1320,       0,  870388,       0,  0, 11,  0, }, // Nauru
    {  638, 1326,       0,  870399,       0,  0, 11,  0, }, // Nepal
    {  638, 1332,  870410,  870432,  870462, 22, 30, 26, }, // New_Caledonia
    {  638, 1346,  870488,  870507,  870534, 19, 27, 23, }, // New_Zealand
    {  638, 1358,  870557,  870575,  870601, 18, 26, 22, }, // Newfoundland
    {  638, 1371,       0,  870623,       0,  0, 10,  0, }, // Niue
    {  638, 1376,  870633,  870651,  870677, 18, 26, 22, }, // Norfolk
    {  638, 1384,  870699,  870724,  870757, 25, 33, 29, }, // Noronha
    {  638, 1406,  870786,  870803,  870828, 17, 25, 21, }, // Novosibirsk
    {  638, 1418,  870849,  870859,  870877, 10, 18, 14, }, // Omsk
    {  638, 1428,  870891,  870905,  870927, 14, 22, 18, }, // Pakistan
    {  638, 1437,       0,  870945,       0,  0, 11,  0, }, // Palau
    {  638, 1443,       0,  870956,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  638, 1460,  870981,  870995,  871017, 14, 22, 18, }, // Paraguay
    {  638, 1469,  871035,  871046,  871065, 11, 19, 15, }, // Peru
    {  638, 1474,  871080,  871095,  871118, 15, 23, 19, }, // Philippines
    {  638, 1486,       0,  871137,       0,  0, 22,  0, }, // Phoenix_Islands
    {  638, 1502,  871159,  871189,  871227, 30, 38, 34, }, // Pierre_Miquelon
    {  638, 1518,       0,  871261,       0,  0, 14,  0, }, // Pitcairn
    {  638, 1527,       0,  871275,       0,  0, 13,  0, }, // Ponape
    {  638, 1534,       0,  871288,       0,  0, 15,  0, }, // Pyongyang
    {  638, 1554,       0,  871303,       0,  0, 13,  0, }, // Reunion
    {  638, 1562,       0,  871316,       0,  0, 13,  0, }, // Rothera
    {  638, 1570,  871329,  871343,  871365, 14, 22, 18, }, // Sakhalin
    {  638, 1579,  871383,  871395,  871415, 12, 20, 16, }, // Samara
    {  638, 1596,  871431,  871442,  871461, 11, 19, 15, }, // Samoa
    {  638, 1602,       0,  871476,       0,  0, 16,  0, }, // Seychelles
    {  638, 1624,       0,  871492,       0,  0, 15,  0, }, // Singapore
    {  638, 1634,       0,  871507,       0,  0, 22,  0, }, // Solomon
    {  638, 1642,       0,  871529,       0,  0, 16,  0, }, // South_Georgia
    {  638, 1656,       0,  871545,       0,  0, 14,  0, }, // Suriname
    {  638, 1676,       0,  871559,       0,  0, 11,  0, }, // Syowa
    {  638, 1682,       0,  871570,       0,  0, 12,  0, }, // Tahiti
    {  638, 1689,  871582,  871594,  871614, 12, 20, 16, }, // Taipei
    {  638, 1696,       0,  871630,       0,  0, 16,  0, }, // Tajikistan
    {  638, 1724,       0,  871646,       0,  0, 13,  0, }, // Tokelau
    {  638, 1732,  871659,  871670,  871689, 11, 19, 15, }, // Tonga
    {  638, 1738,       0,  871704,       0,  0, 11,  0, }, // Truk
    {  638, 1750,  871715,  871733,  871759, 18, 26, 22, }, // Turkmenistan
    {  638, 1763,       0,  871781,       0,  0, 12,  0, }, // Tuvalu
    {  638, 1777,  871793,  871806,  871827, 13, 21, 17, }, // Uruguay
    {  638, 1792,  871844,  871860,  871884, 16, 24, 20, }, // Uzbekistan
    {  638, 1803,  871904,  871917,  871938, 13, 21, 17, }, // Vanuatu
    {  638, 1811,       0,  871955,       0,  0, 15,  0, }, // Venezuela
    {  638, 1821,  871970,  871987,  872012, 17, 25, 21, }, // Vladivostok
    {  638, 1833,  872033,  872048,  872071, 15, 23, 19, }, // Volgograd
    {  638, 1843,       0,  872090,       0,  0, 12,  0, }, // Vostok
    {  638, 1850,       0,  872102,       0,  0, 15,  0, }, // Wake
    {  638, 1855,       0,  872117,       0,  0, 21,  0, }, // Wallis
    {  638, 1862,  872138,  872151,  872172, 13, 21, 17, }, // Yakutsk
    {  638, 1870,  872189,  872208,  872235, 19, 27, 23, }, // Yekaterinburg
    {  638, 1892,       0,  872258,       0,  0, 11,  0, }, // Yukon
    {  639,    5,       0,  872269,       0,  0, 14,  0, }, // Afghanistan Western Balochi/Arabic/Pakistan
    {  639, 1058,       0,  872283,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  639, 1077,       0,  872307,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  639, 1151,       0,  872330,       0,  0, 15,  0, }, // Kyrgystan
    {  639, 1696,       0,  872345,       0,  0, 14,  0, }, // Tajikistan
    {  644,    0,  872359,  872368,  872385,  9, 17, 15, }, // Acre Western Frisian/Latin/Netherlands
    {  644,    5,       0,  872400,       0,  0, 15,  0, }, // Afghanistan
    {  644,   17,       0,  872415,       0,  0, 25,  0, }, // Africa_Central
    {  644,   32,       0,  872440,       0,  0, 21,  0, }, // Africa_Eastern
    {  644,   65,       0,  872461,       0,  0, 20,  0, }, // Africa_Southern
    {  644,   81,  872481,  872502,  872531, 21, 29, 27, }, // Africa_Western
    {  644,  107,  872558,  872569,  872588, 11, 19, 17, }, // Alaska
    {  644,  128,  872605,  872618,  872639, 13, 21, 19, }, // Almaty
    {  644,  135,  872658,  872670,  872690, 12, 20, 18, }, // Amazon
    {  644,  142,  872708,  872720,  872740, 12, 20, 18, }, // America_Central
    {  644,  158,  872758,  872770,  872790, 12, 20, 18, }, // America_Eastern
    {  644,  174,  872808,  872821,  872842, 13, 21, 19, }, // America_Mountain
    {  644,  191,  872861,  872873,  872893, 12, 20, 18, }, // America_Pacific
    {  644,  207,  872911,  872922,  872941, 11, 19, 17, }, // Anadyr
    {  644,  214,  872958,  872967,  872984,  9, 17, 15, }, // Apia
    {  644,  219,  872999,  873009,  873027, 10, 18, 16, }, // Aqtau
    {  644,  225,  873043,  873054,  873073, 11, 19, 17, }, // Aqtobe
    {  644,  232,  873090,  873103,  873124, 13, 21, 19, }, // Arabian
    {  644,  240,  873143,  873159,  873183, 16, 24, 22, }, // Argentina
    {  644,  250,  873205,  873226,  873255, 21, 29, 27, }, // Argentina_Western
    {  644,  268,  873282,  873295,  873316, 13, 21, 19, }, // Armenia
    {  644,  286,  873335,  873348,  873369, 13, 21, 19, }, // Atlantic
    {  644,  295,  873388,  873411,  873442, 23, 31, 29, }, // Australia_Central
    {  644,  313,  873471,  873505,  873547, 34, 42, 40, }, // Australia_CentralWestern
    {  644,  338,  873587,  873608,  873637, 21, 29, 27, }, // Australia_Eastern
    {  644,  356,  873664,  873685,  873714, 21, 29, 27, }, // Australia_Western
    {  644,  374,  873741,  873762,  873791, 21, 29, 27, }, // Azerbaijan
    {  644,  385,  873818,  873829,  873848, 11, 19, 17, }, // Azores
    {  644,  397,  873865,  873879,  873901, 14, 22, 20, }, // Bangladesh
    {  644,  415,       0,  873921,       0,  0, 15,  0, }, // Bhutan
    {  644,  422,       0,  873936,       0,  0, 17,  0, }, // Bolivia
    {  644,  437,  873953,  873971,  873997, 18, 26, 24, }, // Brasilia
    {  644,  454,       0,  874021,       0,  0, 13,  0, }, // Brunei
    {  644,  461,  874034,  874051,  874076, 17, 25, 23, }, // Cape_Verde
    {  644,  478,       0,  874099,       0,  0, 13,  0, }, // Chamorro
    {  644,  487,  874112,  874124,  874144, 12, 20, 18, }, // Chatham
    {  644,  495,  874162,  874176,  874198, 14, 22, 20, }, // Chile
    {  644,  501,  874218,  874231,  874252, 13, 21, 19, }, // China
    {  644,  507,       0,  874271,       0,  0, 18,  0, }, // Christmas
    {  644,  517,       0,  874289,       0,  0, 18,  0, }, // Cocos
    {  644,  523,  874307,  874325,  874351, 18, 26, 24, }, // Colombia
    {  644,  532,  874375,  874391,  874415, 16, 24, 28, }, // Cook
    {  644,  537,  874443,  874457,  874479, 14, 22, 20, }, // Cuba
    {  644,  548,       0,  874499,       0,  0, 10,  0, }, // Davis
    {  644,  564,       0,  874509,       0,  0, 21,  0, }, // DumontDUrville
    {  644,  601,       0,  874530,       0,  0, 18,  0, }, // East_Timor
    {  644,  612,  874548,  874567,  874594, 19, 27, 25, }, // Easter
    {  644,  619,       0,  874619,       0,  0, 18,  0, }, // Ecuador
    {  644,  627,  874637,  874659,  874689, 22, 30, 28, }, // Europe_Central
    {  644,  642,  874717,  874737,  874765, 20, 28, 26, }, // Europe_Eastern
    {  644,  657,       0,  874791,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  644,  680,  874822,  874842,  874870, 20, 28, 26, }, // Europe_Western
    {  644,  695,  874896,  874916,  874944, 20, 28, 26, }, // Falkland
    {  644,  704,  874970,  874982,  875002, 12, 20, 18, }, // Fiji
    {  644,  709,       0,  875020,       0,  0, 20,  0, }, // French_Guiana
    {  644,  723,       0,  875040,       0,  0, 35,  0, }, // French_Southern
    {  644,  746,       0,  875075,       0,  0, 22,  0, }, // Galapagos
    {  644,  756,       0,  875097,       0,  0, 20,  0, }, // Gambier
    {  644,  764,  875117,  875131,  875153, 14, 22, 20, }, // Georgia
    {  644,  772,       0,  875173,       0,  0, 20,  0, }, // Gilbert_Islands
    {  644,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {  644,  830,  875193,  875214,  875243, 21, 29, 27, }, // Greenland_Eastern
    {  644,  848,  875270,  875291,  875320, 21, 29, 27, }, // Greenland_Western
    {  644,  866,       0,  875347,       0,  0, 20,  0, }, // Guam
    {  644,  871,       0,  875367,       0,  0, 17,  0, }, // Gulf
    {  644,  876,       0,  875026,       0,  0, 14,  0, }, // Guyana
    {  644,  883,  875384,  875406,  875436, 22, 30, 28, }, // Hawaii_Aleutian
    {  644,  899,  875464,  875479,  875502, 15, 23, 21, }, // Hong_Kong
    {  644,  909,  875523,  875532,  875549,  9, 17, 15, }, // Hovd
    {  644,  914,       0,  875564,       0,  0, 14,  0, }, // India
    {  644,  920,       0,  875578,       0,  0, 19,  0, }, // Indian_Ocean
    {  644,  933,       0,  875597,       0,  0, 16,  0, }, // Indochina
    {  644,  943,       0,  875613,       0,  0, 25,  0, }, // Indonesia_Central
    {  644,  961,       0,  875638,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  644,  979,       0,  875659,       0,  0, 21,  0, }, // Indonesia_Western
    {  644,  997,  875680,  875693,  875714, 13, 21, 19, }, // Iran
    {  644, 1008,  875733,  875746,  875767, 13, 21, 19, }, // Irkutsk
    {  644, 1016,  875786,  875801,  875824, 15, 23, 21, }, // Israel
    {  644, 1023,  875845,  875858,  875879, 13, 21, 19, }, // Japan
    {  644, 1029,  875898,  875928,  875966, 30, 38, 36, }, // Kamchatka
    {  644, 1047,       0,  876002,       0,  0, 15,  0, }, // Kazakhstan
    {  644, 1058,       0,  876017,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  644, 1077,       0,  876035,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  644, 1106,  876053,  876068,  876091, 15, 23, 21, }, // Korea
    {  644, 1112,       0,  876112,       0,  0, 13,  0, }, // Kosrae
    {  644, 1119,  876125,  876141,  876165, 16, 24, 22, }, // Krasnoyarsk
    {  644, 1151,       0,  876187,       0,  0, 15,  0, }, // Kyrgystan
    {  644, 1161,       0,  876202,       0,  0, 10,  0, }, // Lanka
    {  644, 1175,       0,  876212,       0,  0, 18,  0, }, // Line_Islands
    {  644, 1188,  876230,  876253,  876284, 23, 31, 29, }, // Lord_Howe
    {  644, 1198,  876313,  876325,  876345, 12, 20, 18, }, // Macau
    {  644, 1204,  876363,  876375,  876395, 12, 20, 18, }, // Magadan
    {  644, 1219,       0,  876413,       0,  0, 15,  0, }, // Malaysia
    {  644, 1228,       0,  876428,       0,  0, 15,  0, }, // Maldives
    {  644, 1237,       0,  876443,       0,  0, 22,  0, }, // Marquesas
    {  644, 1247,       0,  876465,       0,  0, 21,  0, }, // Marshall_Islands
    {  644, 1264,  876486,  876504,  876530, 18, 26, 24, }, // Mauritius
    {  644, 1274,       0,  876554,       0,  0, 11,  0, }, // Mawson
    {  644, 1281,  876565,  876590,  876623, 25, 33, 32, }, // Mexico_Pacific
    {  644, 1296,  876655,  876671,  876695, 16, 24, 22, }, // Mongolia
    {  644, 1305,  876717,  876728,  876747, 11, 19, 17, }, // Moscow
    {  644, 1312,       0,  876764,       0,  0, 15,  0, }, // Myanmar
    {  644, 1320,       0,  876779,       0,  0, 16,  0, }, // Nauru
    {  644, 1326,       0,  876795,       0,  0, 13,  0, }, // Nepal
    {  644, 1332,  876808,  876828,  876856, 20, 28, 26, }, // New_Caledonia
    {  644, 1346,  876882,  876900,  876926, 18, 26, 24, }, // New_Zealand
    {  644, 1358,  876950,  876969,  876996, 19, 27, 25, }, // Newfoundland
    {  644, 1371,       0,  877021,       0,  0, 11,  0, }, // Niue
    {  644, 1376,  877032,  877052,  877080, 20, 28, 26, }, // Norfolk
    {  644, 1384,  877106,  877130,  877162, 24, 32, 30, }, // Noronha
    {  644, 1392,       0,  877192,       0,  0, 25,  0, }, // North_Mariana
    {  644, 1406,  877217,  877233,  877257, 16, 24, 22, }, // Novosibirsk
    {  644, 1418,  877279,  877288,  877305,  9, 17, 15, }, // Omsk
    {  644, 1428,  877320,  877337,  877362, 17, 25, 23, }, // Pakistan
    {  644, 1437,       0,  877385,       0,  0, 12,  0, }, // Palau
    {  644, 1443,       0,  877397,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  644, 1460,  877422,  877441,  877468, 19, 27, 25, }, // Paraguay
    {  644, 1469,  877493,  877508,  877531, 15, 23, 21, }, // Peru
    {  644, 1474,  877552,  877567,  877590, 15, 23, 21, }, // Philippines
    {  644, 1486,       0,  877611,       0,  0, 20,  0, }, // Phoenix_Islands
    {  644, 1502,  877631,  877660,  877697, 29, 37, 35, }, // Pierre_Miquelon
    {  644, 1518,       0,  877732,       0,  0, 22,  0, }, // Pitcairn
    {  644, 1527,       0,  877754,       0,  0, 12,  0, }, // Ponape
    {  644, 1534,       0,  877766,       0,  0, 14,  0, }, // Pyongyang
    {  644, 1544,  877780,  877794,  877816, 14, 22, 20, }, // Qyzylorda
    {  644, 1554,       0,  877836,       0,  0, 14,  0, }, // Reunion
    {  644, 1562,       0,  877850,       0,  0, 12,  0, }, // Rothera
    {  644, 1570,  877862,  877875,  877896, 13, 21, 19, }, // Sakhalin
    {  644, 1579,  877915,  877926,  877945, 11, 19, 17, }, // Samara
    {  644, 1596,  877962,  877977,  878000, 15, 23, 21, }, // Samoa
    {  644, 1602,       0,  878021,       0,  0, 14,  0, }, // Seychelles
    {  644, 1624,       0,  878035,       0,  0, 24,  0, }, // Singapore
    {  644, 1634,       0,  878059,       0,  0, 21,  0, }, // Solomon
    {  644, 1642,       0,  878080,       0,  0, 18,  0, }, // South_Georgia
    {  644, 1656,       0,  878098,       0,  0, 16,  0, }, // Suriname
    {  644, 1676,       0,  878114,       0,  0, 10,  0, }, // Syowa
    {  644, 1682,       0,  878124,       0,  0, 17,  0, }, // Tahiti
    {  644, 1689,  878141,  878152,  878171, 11, 19, 17, }, // Taipei
    {  644, 1696,       0,  878188,       0,  0, 15,  0, }, // Tajikistan
    {  644, 1724,       0,  878203,       0,  0, 21,  0, }, // Tokelau
    {  644, 1732,  878224,  878239,  878262, 15, 23, 21, }, // Tonga
    {  644, 1738,       0,  878283,       0,  0, 12,  0, }, // Truk
    {  644, 1750,  878295,  878310,  878333, 15, 23, 21, }, // Turkmenistan
    {  644, 1763,       0,  878354,       0,  0, 17,  0, }, // Tuvalu
    {  644, 1777,  878371,  878389,  878415, 18, 26, 24, }, // Uruguay
    {  644, 1792,  878439,  878453,  878475, 14, 22, 20, }, // Uzbekistan
    {  644, 1803,  878495,  878513,  878539, 18, 26, 24, }, // Vanuatu
    {  644, 1811,       0,  878563,       0,  0, 18,  0, }, // Venezuela
    {  644, 1821,  878581,  878597,  878621, 16, 24, 22, }, // Vladivostok
    {  644, 1833,  878643,  878657,  878679, 14, 22, 20, }, // Volgograd
    {  644, 1843,       0,  878699,       0,  0, 11,  0, }, // Vostok
    {  644, 1850,       0,  878710,       0,  0, 18,  0, }, // Wake
    {  644, 1855,       0,  878728,       0,  0, 23,  0, }, // Wallis
    {  644, 1862,  878751,  878764,  878785, 13, 21, 19, }, // Yakutsk
    {  644, 1870,  878804,  878824,  878852, 20, 28, 26, }, // Yekaterinburg
    {  644, 1892,       0,  878878,       0,  0, 10,  0, }, // Yukon
    {  646,    5,       0,  878888,       0,  0, 16,  0, }, // Afghanistan Wolof/Latin/Senegal
    {  646,   17,       0,  878904,       0,  0, 22,  0, }, // Africa_Central
    {  646,   32,       0,  878926,       0,  0, 24,  0, }, // Africa_Eastern
    {  646,   65,       0,  264808,       0,  0, 14,  0, }, // Africa_Southern
    {  646,   81,  878950,  878969,  879000, 19, 31, 21, }, // Africa_Western
    {  646,  107,  879021,  879033,  879057, 12, 24, 22, }, // Alaska
    {  646,  135,  879079,  879091,  879115, 12, 24, 19, }, // Amazon
    {  646,  142,  879134,  879153,  879185, 19, 32, 28, }, // America_Central
    {  646,  158,  879213,  879227,  879255, 14, 28, 25, }, // America_Eastern
    {  646,  174,  879280,  879296,  879324, 16, 28, 25, }, // America_Mountain
    {  646,  191,  879349,  879367,  879397, 18, 30, 27, }, // America_Pacific
    {  646,  214,  879424,  879434,  879456, 10, 22, 17, }, // Apia
    {  646,  232,  879473,  879487,  879510, 14, 23, 21, }, // Arabian
    {  646,  240,  879531,  879546,  879573, 15, 27, 22, }, // Argentina
    {  646,  250,  879595,  879616,  879649, 21, 33, 28, }, // Argentina_Western
    {  646,  268,  879677,  879689,  879713, 12, 24, 19, }, // Armenia
    {  646,  286,  879732,  879751,  879779, 19, 28, 28, }, // Atlantic
    {  646,  295,  879807,  879834,  879861, 27, 27, 31, }, // Australia_Central
    {  646,  313,  879892,  879913,  879952, 21, 39, 37, }, // Australia_CentralWestern
    {  646,  338,  879989,  880010,  880034, 21, 24, 33, }, // Australia_Eastern
    {  646,  356,  880067,  880095,  880128, 28, 33, 31, }, // Australia_Western
    {  646,  374,  880159,  880176,  880193, 17, 17, 24, }, // Azerbaijan
    {  646,  385,  880217,  880229,  880253, 12, 24, 16, }, // Azores
    {  646,  397,  880269,  880285,  880313, 16, 28, 23, }, // Bangladesh
    {  646,  415,       0,  880336,       0,  0, 13,  0, }, // Bhutan
    {  646,  422,       0,  880349,       0,  0, 13,  0, }, // Bolivia
    {  646,  437,  880362,  880374,  880387, 12, 13, 20, }, // Brasilia
    {  646,  454,       0,    1662,       0,  0, 17,  0, }, // Brunei
    {  646,  461,    1684,  880407,  880423, 10, 16, 25, }, // Cape_Verde
    {  646,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro
    {  646,  487,  880448,    1789,    1810, 13, 21, 21, }, // Chatham
    {  646,  495,  880461,  880471,  880493, 10, 22, 17, }, // Chile
    {  646,  501,  880510,  880521,  880544, 11, 23, 18, }, // China
    {  646,  507,       0,  880562,       0,  0, 17,  0, }, // Christmas
    {  646,  517,       0,  880579,       0,  0, 15,  0, }, // Cocos
    {  646,  523,  880594,  880607,  880632, 13, 25, 22, }, // Colombia
    {  646,  532,  880654,  880668,  880694, 14, 26, 18, }, // Cook
    {  646,  537,  880712,  880722,  880744, 10, 22, 17, }, // Cuba
    {  646,  548,       0,  880761,       0,  0, 11,  0, }, // Davis
    {  646,  564,       0,    2146,       0,  0, 16,  0, }, // DumontDUrville
    {  646,  601,       0,  259673,       0,  0, 14,  0, }, // East_Timor
    {  646,  612,  880772,  880791,  880822, 19, 31, 28, }, // Easter
    {  646,  619,       0,  880850,       0,  0, 13,  0, }, // Ecuador
    {  646,  627,  880863,  880890,  880929, 27, 39, 35, }, // Europe_Central
    {  646,  642,  880964,  880989,  881026, 25, 37, 33, }, // Europe_Eastern
    {  646,  657,       0,  881059,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {  646,  680,  881077,  881106,  881148, 29, 42, 38, }, // Europe_Western
    {  646,  695,  881186,  881199,  881222, 13, 23, 27, }, // Falkland
    {  646,  704,  881249,   15412,  881260, 11,  5, 19, }, // Fiji
    {  646,  709,       0,  260239,       0,  0, 16,  0, }, // French_Guiana
    {  646,  723,       0,  881279,       0,  0, 36,  0, }, // French_Southern
    {  646,  746,       0,  881315,       0,  0, 15,  0, }, // Galapagos
    {  646,  756,       0,  881330,       0,  0, 13,  0, }, // Gambier
    {  646,  764,  881343,  881356,  881369, 13, 13, 17, }, // Georgia
    {  646,  772,       0,  881386,       0,  0, 17,  0, }, // Gilbert_Islands
    {  646,  788,       0,  881403,       0,  0, 21,  0, }, // GMT
    {  646,  830,  881424,  881445,  881478, 21, 33, 28, }, // Greenland_Eastern
    {  646,  848,  881506,  881527,  881560, 21, 33, 28, }, // Greenland_Western
    {  646,  871,       0,  881588,       0,  0, 10,  0, }, // Gulf
    {  646,  876,       0,  881598,       0,  0, 12,  0, }, // Guyana
    {  646,  883,  881610,  881631,  881664, 21, 33, 31, }, // Hawaii_Aleutian
    {  646,  899,  881695,  881710,  881737, 15, 27, 24, }, // Hong_Kong
    {  646,  909,  881761,  881771,  881790, 10, 19, 14, }, // Hovd
    {  646,  914,       0,  881804,       0,  0, 10,  0, }, // India
    {  646,  920,       0,  881814,       0,  0, 16,  0, }, // Indian_Ocean
    {  646,  933,       0,  881830,       0,  0, 15,  0, }, // Indochina
    {  646,  943,       0,  881845,       0,  0, 27,  0, }, // Indonesia_Central
    {  646,  961,       0,  881872,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  646,  979,       0,  881893,       0,  0, 21,  0, }, // Indonesia_Western
    {  646,  997,  881914,  881924,  881943, 10, 19, 20, }, // Iran
    {  646, 1008,  881963,  881975,  881993, 12, 18, 20, }, // Irkutsk
    {  646, 1016,  882013,  882025,  882049, 12, 24, 19, }, // Israel
    {  646, 1023,   28214,  882068,  882079,  5, 11, 18, }, // Japan
    {  646, 1047,       0,  882097,       0,  0, 17,  0, }, // Kazakhstan
    {  646, 1058,       0,  882114,       0,  0, 16,  0, }, // Kazakhstan_Eastern
    {  646, 1077,       0,  882130,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  646, 1106,  882154,  882164,  882186, 10, 22, 20, }, // Korea
    {  646, 1112,       0,  882206,       0,  0, 12,  0, }, // Kosrae
    {  646, 1119,  882218,  882235,  882252, 17, 17, 26, }, // Krasnoyarsk
    {  646, 1151,       0,  882278,       0,  0, 15,  0, }, // Kyrgystan
    {  646, 1175,       0,  882293,       0,  0, 14,  0, }, // Line_Islands
    {  646, 1188,    3887,  882307,    3925, 14, 24, 22, }, // Lord_Howe
    {  646, 1204,  882331,  882345,  882359, 14, 14, 20, }, // Magadan
    {  646, 1219,       0,  882379,       0,  0,  8,  0, }, // Malaysia
    {  646, 1228,       0,  882387,       0,  0, 14,  0, }, // Maldives
    {  646, 1237,       0,  882401,       0,  0, 15,  0, }, // Marquesas
    {  646, 1247,       0,  882416,       0,  0, 18,  0, }, // Marshall_Islands
    {  646, 1264,  882434,  882447,  882472, 13, 25, 20, }, // Mauritius
    {  646, 1274,       0,  882492,       0,  0, 12,  0, }, // Mawson
    {  646, 1281,  882504,  882526,  882561, 22, 35, 31, }, // Mexico_Pacific
    {  646, 1296,  882592,  882604,  882614, 12, 10, 17, }, // Mongolia
    {  646, 1305,  882631,  882643,  882655, 12, 12, 19, }, // Moscow
    {  646, 1312,       0,  882674,       0,  0, 13,  0, }, // Myanmar
    {  646, 1320,       0,  882687,       0,  0, 11,  0, }, // Nauru
    {  646, 1326,       0,  882698,       0,  0, 11,  0, }, // Nepal
    {  646, 1332,  882709,  882728,  882763, 19, 35, 31, }, // New_Caledonia
    {  646, 1346,  882794,  882816,  262419, 22, 34, 16, }, // New_Zealand
    {  646, 1358,  882850,  882867,  262514, 17, 11, 11, }, // Newfoundland
    {  646, 1371,       0,  882878,       0,  0, 10,  0, }, // Niue
    {  646, 1376,  882888,  882905,  882934, 17, 29, 27, }, // Norfolk
    {  646, 1384,  882961,  882980,  882999, 19, 19, 31, }, // Noronha
    {  646, 1406,  883030,  883047,  883064, 17, 17, 26, }, // Novosibirsk
    {  646, 1418,  883090,  883100,  883122, 10, 22, 14, }, // Omsk
    {  646, 1428,  883136,  883150,  883176, 14, 26, 21, }, // Pakistan
    {  646, 1437,       0,  883197,       0,  0, 11,  0, }, // Palau
    {  646, 1443,       0,  263025,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  646, 1460,  883208,  883222,  883236, 14, 14, 18, }, // Paraguay
    {  646, 1469,  883254,  883264,  883283, 10, 19, 14, }, // Peru
    {  646, 1474,  883297,  883313,  883339, 16, 26, 26, }, // Philippines
    {  646, 1486,       0,  883365,       0,  0, 17,  0, }, // Phoenix_Islands
    {  646, 1502,  883382,  249629,  883406, 24, 23, 25, }, // Pierre_Miquelon
    {  646, 1518,       0,  883431,       0,  0, 14,  0, }, // Pitcairn
    {  646, 1527,       0,  883445,       0,  0, 12,  0, }, // Ponape
    {  646, 1534,       0,  883457,       0,  0, 15,  0, }, // Pyongyang
    {  646, 1554,       0,  883472,       0,  0, 11,  0, }, // Reunion
    {  646, 1562,       0,  883483,       0,  0, 12,  0, }, // Rothera
    {  646, 1570,  883495,   69513,  883508, 13,  7, 17, }, // Sakhalin
    {  646, 1596,  883525,    5503,  883536, 11, 19, 18, }, // Samoa
    {  646, 1602,       0,  883554,       0,  0, 16,  0, }, // Seychelles
    {  646, 1624,       0,  883570,       0,  0, 27,  0, }, // Singapore
    {  646, 1634,       0,  883597,       0,  0, 17,  0, }, // Solomon
    {  646, 1642,       0,  883614,       0,  0, 14,  0, }, // South_Georgia
    {  646, 1656,       0,  883628,       0,  0, 13,  0, }, // Suriname
    {  646, 1676,       0,  883641,       0,  0, 11,  0, }, // Syowa
    {  646, 1682,       0,  883652,       0,  0, 12,  0, }, // Tahiti
    {  646, 1689,  883664,  883676,  883700, 12, 24, 24, }, // Taipei
    {  646, 1696,       0,  883724,       0,  0, 17,  0, }, // Tajikistan
    {  646, 1724,       0,  883741,       0,  0, 12,  0, }, // Tokelau
    {  646, 1732,  883753,  883764,  883784, 11, 20, 20, }, // Tonga
    {  646, 1738,       0,  883804,       0,  0, 11,  0, }, // Truk
    {  646, 1750,  883815,  878969,  883833, 18, 14, 25, }, // Turkmenistan
    {  646, 1763,       0,  883858,       0,  0, 12,  0, }, // Tuvalu
    {  646, 1777,  883870,  883883,  883905, 13, 22, 17, }, // Uruguay
    {  646, 1792,  883922,  883939,  883968, 17, 29, 24, }, // Uzbekistan
    {  646, 1803,  883992,  884005,  884015, 13, 10, 20, }, // Vanuatu
    {  646, 1811,       0,  884035,       0,  0, 15,  0, }, // Venezuela
    {  646, 1821,  884050,  884067,  884087, 17, 20, 26, }, // Vladivostok
    {  646, 1833,  884113,  884128,  884152, 15, 24, 24, }, // Volgograd
    {  646, 1843,       0,  884176,       0,  0, 12,  0, }, // Vostok
    {  646, 1850,       0,  884188,       0,  0, 14,  0, }, // Wake
    {  646, 1855,       0,    6191,       0,  0, 20,  0, }, // Wallis
    {  646, 1862,  884202,  884215,  884228, 13, 13, 20, }, // Yakutsk
    {  646, 1870,  884248,  884267,  880632, 19, 14, 11, }, // Yekaterinburg
    {  646, 1892,       0,  884281,       0,  0, 11,  0, }, // Yukon
    {  647,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan Xhosa/Latin/South Africa
    {  647,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {  647,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  647,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  647,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  647,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  647,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {  647,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  647,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  647,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  647,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  647,  214,     601,     610,     628,  9, 18, 18, }, // Apia
    {  647,  232,     741,     753,     774, 12, 21, 21, }, // Arabian
    {  647,  240,     795,     809,     832, 14, 23, 21, }, // Argentina
    {  647,  250,     853,     875,     906, 22, 31, 29, }, // Argentina_Western
    {  647,  268,     935,     947,     968, 12, 21, 19, }, // Armenia
    {  647,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  647,  295,    1044,    1066,    1098, 22, 32, 32, }, // Australia_Central
    {  647,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  647,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {  647,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {  647,  374,    1413,    1428,    1452, 15, 24, 22, }, // Azerbaijan
    {  647,  385,    1474,    1485,    1505, 11, 20, 18, }, // Azores
    {  647,  397,    1523,    1538,    1562, 15, 24, 22, }, // Bangladesh
    {  647,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {  647,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia
    {  647,  437,    1607,    1620,    1642, 13, 22, 20, }, // Brasilia
    {  647,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {  647,  461,    1684,    1699,    1723, 15, 24, 22, }, // Cape_Verde
    {  647,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro
    {  647,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {  647,  495,    1831,    1841,    1860, 10, 19, 17, }, // Chile
    {  647,  501,    1877,    1887,    1906, 10, 19, 19, }, // China
    {  647,  507,       0,    1925,       0,  0, 21,  0, }, // Christmas
    {  647,  517,       0,    1946,       0,  0, 18,  0, }, // Cocos
    {  647,  523,    1964,    1977,    1999, 13, 22, 20, }, // Colombia
    {  647,  532,    2019,    2036,    2062, 17, 26, 29, }, // Cook
    {  647,  537,    2091,    2100,    2118,  9, 18, 18, }, // Cuba
    {  647,  548,       0,    2136,       0,  0, 10,  0, }, // Davis
    {  647,  564,       0,    2146,       0,  0, 21,  0, }, // DumontDUrville
    {  647,  601,       0,    2167,       0,  0, 15,  0, }, // East_Timor
    {  647,  612,    2182,    2200,    2227, 18, 27, 25, }, // Easter
    {  647,  619,       0,    2252,       0,  0, 12,  0, }, // Ecuador
    {  647,  627,    2264,    2285,    2315, 21, 30, 28, }, // Europe_Central
    {  647,  642,    2343,    2364,    2394, 21, 30, 28, }, // Europe_Eastern
    {  647,  657,       0,    2422,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  647,  680,    2451,    2472,    2502, 21, 30, 28, }, // Europe_Western
    {  647,  695,    2530,    2551,    2581, 21, 30, 28, }, // Falkland
    {  647,  704,    2609,    2618,    2636,  9, 18, 16, }, // Fiji
    {  647,  709,       0,    2652,       0,  0, 18,  0, }, // French_Guiana
    {  647,  723,       0,    2670,       0,  0, 32,  0, }, // French_Southern
    {  647,  746,       0,    2702,       0,  0, 14,  0, }, // Galapagos
    {  647,  756,       0,    2716,       0,  0, 12,  0, }, // Gambier
    {  647,  764,    2728,    2740,    2761, 12, 21, 19, }, // Georgia
    {  647,  772,       0,    2780,       0,  0, 20,  0, }, // Gilbert_Islands
    {  647,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {  647,  830,    2877,    2896,    2924, 19, 28, 26, }, // Greenland_Eastern
    {  647,  848,    2950,    2969,    2997, 19, 28, 26, }, // Greenland_Western
    {  647,  871,       0,    3041,       0,  0, 18,  0, }, // Gulf
    {  647,  876,       0,    3059,       0,  0, 11,  0, }, // Guyana
    {  647,  883,    3070,    3090,    3119, 20, 29, 29, }, // Hawaii_Aleutian
    {  647,  899,    3148,    3162,    3185, 14, 23, 21, }, // Hong_Kong
    {  647,  909,    3206,    3215,    3233,  9, 18, 16, }, // Hovd
    {  647,  914,       0,    3249,       0,  0, 19,  0, }, // India
    {  647,  920,       0,    3268,       0,  0, 17,  0, }, // Indian_Ocean
    {  647,  933,       0,    3285,       0,  0, 14,  0, }, // Indochina
    {  647,  943,       0,    3299,       0,  0, 22,  0, }, // Indonesia_Central
    {  647,  961,       0,    3321,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  647,  979,       0,    3343,       0,  0, 22,  0, }, // Indonesia_Western
    {  647,  997,    3365,    3374,    3392,  9, 18, 18, }, // Iran
    {  647, 1008,    3410,    3422,    3443, 12, 21, 19, }, // Irkutsk
    {  647, 1016,    3462,    3473,    3493, 11, 20, 20, }, // Israel
    {  647, 1023,    3513,    3523,    3542, 10, 19, 19, }, // Japan
    {  647, 1047,       0,    3664,       0,  0, 15,  0, }, // Kazakhstan
    {  647, 1058,       0,    3679,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  647, 1077,       0,    3699,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  647, 1106,    3719,    3730,    3750, 11, 20, 20, }, // Korea
    {  647, 1112,       0,    3770,       0,  0, 11,  0, }, // Kosrae
    {  647, 1119,    3781,    3797,    3822, 16, 25, 23, }, // Krasnoyarsk
    {  647, 1151,       0,    3845,       0,  0, 15,  0, }, // Kyrgystan
    {  647, 1175,       0,    3870,       0,  0, 17,  0, }, // Line_Islands
    {  647, 1188,    3887,    3901,    3924, 14, 23, 23, }, // Lord_Howe
    {  647, 1204,    3993,    4005,    4026, 12, 21, 19, }, // Magadan
    {  647, 1219,       0,    4045,       0,  0, 13,  0, }, // Malaysia
    {  647, 1228,       0,    4058,       0,  0, 13,  0, }, // Maldives
    {  647, 1237,       0,    4071,       0,  0, 14,  0, }, // Marquesas
    {  647, 1247,       0,    4085,       0,  0, 21,  0, }, // Marshall_Islands
    {  647, 1264,    4106,    4120,    4143, 14, 23, 21, }, // Mauritius
    {  647, 1274,       0,    4164,       0,  0, 11,  0, }, // Mawson
    {  647, 1281,    4175,    4195,    4224, 20, 29, 29, }, // Mexico_Pacific
    {  647, 1296,    4253,    4269,    4294, 16, 25, 23, }, // Mongolia
    {  647, 1305,    4317,    4328,    4348, 11, 20, 18, }, // Moscow
    {  647, 1312,       0,    4366,       0,  0, 12,  0, }, // Myanmar
    {  647, 1320,       0,    4378,       0,  0, 10,  0, }, // Nauru
    {  647, 1326,       0,    4388,       0,  0, 10,  0, }, // Nepal
    {  647, 1332,    4398,    4416,    4443, 18, 27, 25, }, // New_Caledonia
    {  647, 1346,    4468,    4484,    4509, 16, 25, 25, }, // New_Zealand
    {  647, 1358,    4534,    4551,    4577, 17, 26, 26, }, // Newfoundland
    {  647, 1371,       0,    4603,       0,  0,  9,  0, }, // Niue
    {  647, 1376,    4612,    4631,    4659, 19, 28, 28, }, // Norfolk
    {  647, 1384,    4687,    4711,    4744, 24, 33, 31, }, // Noronha
    {  647, 1406,    4801,    4817,    4842, 16, 25, 23, }, // Novosibirsk
    {  647, 1418,    4865,    4874,    4892,  9, 18, 16, }, // Omsk
    {  647, 1428,    4908,    4921,    4943, 13, 22, 20, }, // Pakistan
    {  647, 1437,       0,    4963,       0,  0, 10,  0, }, // Palau
    {  647, 1443,       0,    4973,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  647, 1460,    4994,    5007,    5029, 13, 22, 20, }, // Paraguay
    {  647, 1469,    5049,    5058,    5076,  9, 18, 16, }, // Peru
    {  647, 1474,    5092,    5107,    5131, 15, 24, 22, }, // Philippines
    {  647, 1486,       0,    5153,       0,  0, 20,  0, }, // Phoenix_Islands
    {  647, 1502,    5173,    5199,    5234, 26, 35, 35, }, // Pierre_Miquelon
    {  647, 1518,       0,    5269,       0,  0, 13,  0, }, // Pitcairn
    {  647, 1527,       0,    5282,       0,  0, 11,  0, }, // Ponape
    {  647, 1534,       0,    5293,       0,  0, 14,  0, }, // Pyongyang
    {  647, 1554,       0,    5365,       0,  0, 12,  0, }, // Reunion
    {  647, 1562,       0,    5377,       0,  0, 12,  0, }, // Rothera
    {  647, 1570,    5389,    5402,    5424, 13, 22, 20, }, // Sakhalin
    {  647, 1596,    5493,    5503,    5522, 10, 19, 19, }, // Samoa
    {  647, 1602,       0,    5541,       0,  0, 15,  0, }, // Seychelles
    {  647, 1624,       0,    5556,       0,  0, 23,  0, }, // Singapore
    {  647, 1634,       0,    5579,       0,  0, 20,  0, }, // Solomon
    {  647, 1642,       0,    5599,       0,  0, 18,  0, }, // South_Georgia
    {  647, 1656,       0,    5617,       0,  0, 13,  0, }, // Suriname
    {  647, 1676,       0,    5630,       0,  0, 10,  0, }, // Syowa
    {  647, 1682,       0,    5640,       0,  0, 11,  0, }, // Tahiti
    {  647, 1689,    5651,    5662,    5682, 11, 20, 20, }, // Taipei
    {  647, 1696,       0,    5702,       0,  0, 15,  0, }, // Tajikistan
    {  647, 1724,       0,    5717,       0,  0, 12,  0, }, // Tokelau
    {  647, 1732,    5729,    5739,    5758, 10, 19, 17, }, // Tonga
    {  647, 1738,       0,    5775,       0,  0, 10,  0, }, // Truk
    {  647, 1750,    5785,    5802,    5828, 17, 26, 24, }, // Turkmenistan
    {  647, 1763,       0,    5852,       0,  0, 11,  0, }, // Tuvalu
    {  647, 1777,    5863,    5875,    5896, 12, 21, 19, }, // Uruguay
    {  647, 1792,    5915,    5930,    5954, 15, 24, 22, }, // Uzbekistan
    {  647, 1803,    5976,    5988,    6009, 12, 21, 19, }, // Vanuatu
    {  647, 1811,       0,    6028,       0,  0, 14,  0, }, // Venezuela
    {  647, 1821,    6042,    6058,    6083, 16, 25, 23, }, // Vladivostok
    {  647, 1833,    6106,    6120,    6143, 14, 23, 21, }, // Volgograd
    {  647, 1843,       0,    6164,       0,  0, 11,  0, }, // Vostok
    {  647, 1850,       0,    6175,       0,  0, 16,  0, }, // Wake
    {  647, 1855,       0,    6191,       0,  0, 20,  0, }, // Wallis
    {  647, 1862,    6211,    6223,    6244, 12, 21, 19, }, // Yakutsk
    {  647, 1870,    6263,    6281,    6308, 18, 27, 25, }, // Yekaterinburg
    {  647, 1892,       0,    6333,       0,  0, 10,  0, }, // Yukon
    {  650,    5,       0,  884292,       0,  0, 17,  0, }, // Afghanistan Yoruba/Latin/Nigeria
    {  650,   17,       0,  884309,       0,  0, 19,  0, }, // Africa_Central
    {  650,   32,       0,  884328,       0,  0, 23,  0, }, // Africa_Eastern
    {  650,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  650,   81,  884351,  884375,  884407, 24, 32, 29, }, // Africa_Western
    {  650,  107,  884436,  884449,  884470, 13, 21, 20, }, // Alaska
    {  650,  135,  884490,  884504,  884526, 14, 22, 18, }, // Amazon
    {  650,  142,  884544,  884564,  884590, 20, 26, 28, }, // America_Central
    {  650,  158,  884618,  884637,  884662, 19, 25, 27, }, // America_Eastern
    {  650,  174,  884689,  884698,  884713,  9, 15, 21, }, // America_Mountain
    {  650,  191,  884734,  884749,  884770, 15, 21, 23, }, // America_Pacific
    {  650,  214,  884793,  884803,  884821, 10, 18, 18, }, // Apia
    {  650,  232,  884839,  884851,  884871, 12, 20, 20, }, // Arabian
    {  650,  240,  884891,  884904,  884924, 13, 20, 19, }, // Argentina
    {  650,  250,  884943,  884969,  885003, 26, 34, 30, }, // Argentina_Western
    {  650,  268,  885033,  885046,  885067, 13, 21, 17, }, // Armenia
    {  650,  286,  885084,  885101,  885124, 17, 23, 25, }, // Atlantic
    {  650,  295,  885149,  885177,  885213, 28, 36, 36, }, // Australia_Central
    {  650,  313,  885249,  885288,  885335, 39, 47, 47, }, // Australia_CentralWestern
    {  650,  338,  885382,  885407,  885440, 25, 33, 33, }, // Australia_Eastern
    {  650,  356,  885473,  885499,  885533, 26, 34, 34, }, // Australia_Western
    {  650,  374,  885567,  885583,  885607, 16, 24, 21, }, // Azerbaijan
    {  650,  385,  885628,  885640,  885660, 12, 20, 17, }, // Azores
    {  650,  397,  885677,  885693,  885717, 16, 24, 21, }, // Bangladesh
    {  650,  415,       0,  885738,       0,  0, 12,  0, }, // Bhutan
    {  650,  422,       0,  885750,       0,  0, 12,  0, }, // Bolivia
    {  650,  437,  885762,  885776,  885796, 14, 20, 18, }, // Brasilia
    {  650,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {  650,  461,  885814,  885830,  885854, 16, 24, 22, }, // Cape_Verde
    {  650,  478,       0,  885876,       0,  0, 22,  0, }, // Chamorro
    {  650,  487,  885898,  885910,  885930, 12, 20, 20, }, // Chatham
    {  650,  495,  885950,  885960,  885978, 10, 18, 14, }, // Chile
    {  650,  501,  885992,  886003,  886024, 11, 21, 19, }, // China
    {  650,  507,       0,  886043,       0,  0, 23,  0, }, // Christmas
    {  650,  517,       0,  886066,       0,  0, 24,  0, }, // Cocos
    {  650,  523,  886090,  886103,  886122, 13, 19, 18, }, // Colombia
    {  650,  532,  886140,  886164,  886196, 24, 32, 35, }, // Cook
    {  650,  537,  886231,  886241,  886259, 10, 18, 18, }, // Cuba
    {  650,  548,       0,  886277,       0,  0, 11,  0, }, // Davis
    {  650,  564,       0,  886288,       0,  0, 22,  0, }, // DumontDUrville
    {  650,  601,       0,  886310,       0,  0, 21,  0, }, // East_Timor
    {  650,  612,  886331,  886352,  886376, 21, 24, 23, }, // Easter
    {  650,  619,       0,  886399,       0,  0, 12,  0, }, // Ecuador
    {  650,  627,  886411,  886429,  886453, 18, 24, 23, }, // Europe_Central
    {  650,  642,  886476,  886502,  886534, 26, 32, 31, }, // Europe_Eastern
    {  650,  657,       0,  886565,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  650,  680,  886594,  886616,  886644, 22, 28, 31, }, // Europe_Western
    {  650,  695,  886675,  886690,  886720, 15, 30, 27, }, // Falkland
    {  650,  704,  886747,  886757,  886775, 10, 18, 15, }, // Fiji
    {  650,  709,       0,  886790,       0,  0, 20,  0, }, // French_Guiana
    {  650,  723,       0,  886810,       0,  0, 32,  0, }, // French_Southern
    {  650,  746,       0,  886842,       0,  0, 13,  0, }, // Galapagos
    {  650,  756,       0,  886855,       0,  0, 12,  0, }, // Gambier
    {  650,  764,  886867,  886880,  886901, 13, 21, 18, }, // Georgia
    {  650,  772,       0,  886919,       0,  0, 27,  0, }, // Gilbert_Islands
    {  650,  788,       0,    2800,       0,  0, 19,  0, }, // GMT
    {  650,  830,  886946,  886971,  887005, 25, 34, 25, }, // Greenland_Eastern
    {  650,  848,  887030,  887056,  887090, 26, 34, 33, }, // Greenland_Western
    {  650,  871,       0,  887123,       0,  0, 18,  0, }, // Gulf
    {  650,  876,       0,  887141,       0,  0, 12,  0, }, // Guyana
    {  650,  883,  887153,  887174,  887203, 21, 29, 29, }, // Hawaii_Aleutian
    {  650,  899,  887232,  887247,  887272, 15, 25, 20, }, // Hong_Kong
    {  650,  909,  887292,  887304,  887326, 12, 22, 17, }, // Hovd
    {  650,  914,       0,  887343,       0,  0, 19,  0, }, // India
    {  650,  920,       0,  887362,       0,  0, 18,  0, }, // Indian_Ocean
    {  650,  933,       0,  887380,       0,  0, 15,  0, }, // Indochina
    {  650,  943,       0,  887395,       0,  0, 30,  0, }, // Indonesia_Central
    {  650,  961,       0,  887425,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  650,  979,       0,  887450,       0,  0, 26,  0, }, // Indonesia_Western
    {  650,  997,  887476,  887487,  887506, 11, 19, 19, }, // Iran
    {  650, 1008,  887525,  887538,  887559, 13, 21, 18, }, // Irkutsk
    {  650, 1016,  887577,  887589,  887609, 12, 20, 20, }, // Israel
    {  650, 1023,  887629,  887640,  887661, 11, 21, 16, }, // Japan
    {  650, 1047,       0,  887677,       0,  0, 16,  0, }, // Kazakhstan
    {  650, 1058,       0,  887693,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  650, 1077,       0,  887720,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  650, 1106,  887748,  887759,  887780, 11, 21, 19, }, // Korea
    {  650, 1112,       0,  887799,       0,  0, 13,  0, }, // Kosrae
    {  650, 1119,  887812,  887831,  887858, 19, 27, 24, }, // Krasnoyarsk
    {  650, 1151,       0,  887882,       0,  0, 19,  0, }, // Kyrgystan
    {  650, 1175,       0,  887901,       0,  0, 24,  0, }, // Line_Islands
    {  650, 1188,  887925,  887940,  887963, 15, 23, 23, }, // Lord_Howe
    {  650, 1204,  887986,  888000,  888022, 14, 22, 19, }, // Magadan
    {  650, 1219,       0,  888041,       0,  0, 14,  0, }, // Malaysia
    {  650, 1228,       0,  888055,       0,  0, 14,  0, }, // Maldives
    {  650, 1237,       0,  888069,       0,  0, 15,  0, }, // Marquesas
    {  650, 1247,       0,  888084,       0,  0, 26,  0, }, // Marshall_Islands
    {  650, 1264,  888110,  888124,  888146, 14, 22, 19, }, // Mauritius
    {  650, 1274,       0,  888165,       0,  0, 12,  0, }, // Mawson
    {  650, 1281,  888177,  888200,  888231, 23, 31, 31, }, // Mexico_Pacific
    {  650, 1296,  888262,  888279,  888306, 17, 27, 22, }, // Mongolia
    {  650, 1305,  888328,  888340,  888360, 12, 20, 17, }, // Moscow
    {  650, 1312,       0,  888377,       0,  0, 17,  0, }, // Myanmar
    {  650, 1320,       0,  888394,       0,  0, 11,  0, }, // Nauru
    {  650, 1326,       0,  888405,       0,  0, 12,  0, }, // Nepal
    {  650, 1332,  888417,  888439,  888469, 22, 30, 27, }, // New_Caledonia
    {  650, 1346,  888496,  888513,  888538, 17, 25, 25, }, // New_Zealand
    {  650, 1358,  888563,  888581,  888607, 18, 26, 26, }, // Newfoundland
    {  650, 1371,       0,  888633,       0,  0, 10,  0, }, // Niue
    {  650, 1376,  888643,  888664,  888693, 21, 29, 29, }, // Norfolk
    {  650, 1384,  888722,  888744,  888772, 22, 28, 29, }, // Noronha
    {  650, 1406,  888801,  888820,  888846, 19, 26, 23, }, // Novosibirsk
    {  650, 1418,  888869,  888882,  888903, 13, 21, 18, }, // Omsk
    {  650, 1428,  888921,  888936,  888959, 15, 23, 20, }, // Pakistan
    {  650, 1437,       0,  888979,       0,  0, 11,  0, }, // Palau
    {  650, 1443,       0,  888990,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  650, 1460,  889012,  889026,  889048, 14, 22, 19, }, // Paraguay
    {  650, 1469,  889067,  889077,  889095, 10, 18, 15, }, // Peru
    {  650, 1474,  889110,  889127,  889152, 17, 25, 22, }, // Philippines
    {  650, 1486,       0,  889174,       0,  0, 26,  0, }, // Phoenix_Islands
    {  650, 1502,  889200,  889223,  889254, 23, 31, 31, }, // Pierre_Miquelon
    {  650, 1518,       0,  889285,       0,  0, 14,  0, }, // Pitcairn
    {  650, 1527,       0,  889299,       0,  0, 12,  0, }, // Ponape
    {  650, 1534,       0,  889311,       0,  0, 16,  0, }, // Pyongyang
    {  650, 1554,       0,  889327,       0,  0, 17,  0, }, // Reunion
    {  650, 1562,       0,  889344,       0,  0, 13,  0, }, // Rothera
    {  650, 1570,  889357,  889371,  889393, 14, 22, 19, }, // Sakhalin
    {  650, 1596,  889412,  889423,  889442, 11, 19, 19, }, // Samoa
    {  650, 1602,       0,  889461,       0,  0, 15,  0, }, // Seychelles
    {  650, 1624,       0,  889476,       0,  0, 23,  0, }, // Singapore
    {  650, 1634,       0,  889499,       0,  0, 26,  0, }, // Solomon
    {  650, 1642,       0,  889525,       0,  0, 17,  0, }, // South_Georgia
    {  650, 1656,       0,  889542,       0,  0, 14,  0, }, // Suriname
    {  650, 1676,       0,  889556,       0,  0, 11,  0, }, // Syowa
    {  650, 1682,       0,  889567,       0,  0, 12,  0, }, // Tahiti
    {  650, 1689,  889579,  889591,  889613, 12, 22, 20, }, // Taipei
    {  650, 1696,       0,  889633,       0,  0, 19,  0, }, // Tajikistan
    {  650, 1724,       0,  889652,       0,  0, 13,  0, }, // Tokelau
    {  650, 1732,  889665,  889676,  889695, 11, 19, 16, }, // Tonga
    {  650, 1738,       0,  889711,       0,  0, 11,  0, }, // Truk
    {  650, 1750,  889722,  889742,  889770, 20, 28, 24, }, // Turkmenistan
    {  650, 1763,       0,  889794,       0,  0, 12,  0, }, // Tuvalu
    {  650, 1777,  889806,  889818,  889839, 12, 21, 17, }, // Uruguay
    {  650, 1792,  889856,  889874,  889900, 18, 26, 23, }, // Uzbekistan
    {  650, 1803,  889923,  889936,  889957, 13, 21, 18, }, // Vanuatu
    {  650, 1811,       0,  889975,       0,  0, 14,  0, }, // Venezuela
    {  650, 1821,  889989,  890009,  890037, 20, 28, 25, }, // Vladivostok
    {  650, 1833,  890062,  890080,  890106, 18, 26, 23, }, // Volgograd
    {  650, 1843,       0,  890129,       0,  0, 12,  0, }, // Vostok
    {  650, 1850,       0,  890141,       0,  0, 18,  0, }, // Wake
    {  650, 1855,       0,  890159,       0,  0, 22,  0, }, // Wallis
    {  650, 1862,  890181,  890194,  890215, 13, 21, 18, }, // Yakutsk
    {  650, 1870,  890233,  890252,  890279, 19, 27, 24, }, // Yekaterinburg
    {  650, 1892,       0,  890303,       0,  0, 13,  0, }, // Yukon
    {  651,   81,  890316,  890340,  890372, 24, 32, 29, }, // Africa_Western Yoruba/Latin/Benin
    {  651,  107,  884436,  890401,  890422, 13, 21, 20, }, // Alaska
    {  651,  135,  890442,  890456,  890478, 14, 22, 18, }, // Amazon
    {  651,  142,  884544,  884564,  890496, 20, 26, 28, }, // America_Central
    {  651,  158,  884618,  884637,  890524, 19, 25, 27, }, // America_Eastern
    {  651,  174,  884689,  884698,  890551,  9, 15, 21, }, // America_Mountain
    {  651,  191,  884734,  884749,  890572, 15, 21, 23, }, // America_Pacific
    {  651,  214,  884793,  890595,  890613, 10, 18, 18, }, // Apia
    {  651,  232,  884839,  890631,  890651, 12, 20, 20, }, // Arabian
    {  651,  240,  890671,  884904,  884924, 13, 20, 19, }, // Argentina
    {  651,  250,  890684,  890710,  890744, 26, 34, 30, }, // Argentina_Western
    {  651,  268,  885033,  890774,  890795, 13, 21, 17, }, // Armenia
    {  651,  286,  885084,  885101,  890812, 17, 23, 25, }, // Atlantic
    {  651,  295,  885149,  890837,  890873, 28, 36, 36, }, // Australia_Central
    {  651,  313,  890909,  890948,  890995, 39, 47, 47, }, // Australia_CentralWestern
    {  651,  338,  885382,  891042,  891075, 25, 33, 33, }, // Australia_Eastern
    {  651,  356,  891108,  891134,  891168, 26, 34, 34, }, // Australia_Western
    {  651,  374,  885567,  891202,  891226, 16, 24, 21, }, // Azerbaijan
    {  651,  385,  891247,  891259,  891279, 12, 20, 17, }, // Azores
    {  651,  397,  885677,  891296,  891320, 16, 24, 21, }, // Bangladesh
    {  651,  461,  885814,  891341,  891365, 16, 24, 22, }, // Cape_Verde
    {  651,  478,       0,  891387,       0,  0, 22,  0, }, // Chamorro
    {  651,  487,  885898,  891409,  891429, 12, 20, 20, }, // Chatham
    {  651,  495,  891449,  891460,  891479, 11, 19, 15, }, // Chile
    {  651,  501,  891494,  891506,  891528, 12, 22, 20, }, // China
    {  651,  517,       0,  891548,       0,  0, 24,  0, }, // Cocos
    {  651,  532,  891572,  891596,  891628, 24, 32, 35, }, // Cook
    {  651,  537,  886231,  891663,  891681, 10, 18, 18, }, // Cuba
    {  651,  627,  886411,  886429,  891699, 18, 24, 23, }, // Europe_Central
    {  651,  642,  886476,  886502,  891722, 26, 32, 31, }, // Europe_Eastern
    {  651,  680,  891753,  891775,  891803, 22, 28, 31, }, // Europe_Western
    {  651,  695,  886675,  891834,  886720, 15, 30, 27, }, // Falkland
    {  651,  704,  886747,  891864,  891882, 10, 18, 15, }, // Fiji
    {  651,  764,  886867,  891897,  891918, 13, 21, 18, }, // Georgia
    {  651,  772,       0,  891936,       0,  0, 27,  0, }, // Gilbert_Islands
    {  651,  830,  886946,  891963,  887005, 25, 34, 25, }, // Greenland_Eastern
    {  651,  848,  891997,  892023,  892057, 26, 34, 33, }, // Greenland_Western
    {  651,  871,       0,  892090,       0,  0, 18,  0, }, // Gulf
    {  651,  883,  887153,  892108,  892137, 21, 29, 29, }, // Hawaii_Aleutian
    {  651,  899,  887232,  892166,  892191, 15, 25, 20, }, // Hong_Kong
    {  651,  909,  887292,  892211,  892233, 12, 22, 17, }, // Hovd
    {  651,  914,       0,  892250,       0,  0, 19,  0, }, // India
    {  651,  979,       0,  892269,       0,  0, 26,  0, }, // Indonesia_Western
    {  651,  997,  887476,  892295,  892314, 11, 19, 19, }, // Iran
    {  651, 1008,  887525,  892333,  892354, 13, 21, 18, }, // Irkutsk
    {  651, 1016,  887577,  892372,  892392, 12, 20, 20, }, // Israel
    {  651, 1023,  887629,  892412,  892433, 11, 21, 16, }, // Japan
    {  651, 1077,       0,  892449,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  651, 1106,  887748,  892477,  892498, 11, 21, 19, }, // Korea
    {  651, 1119,  887812,  892517,  892544, 19, 27, 24, }, // Krasnoyarsk
    {  651, 1175,       0,  892568,       0,  0, 24,  0, }, // Line_Islands
    {  651, 1188,  887925,  892592,  892615, 15, 23, 23, }, // Lord_Howe
    {  651, 1204,  887986,  892638,  892660, 14, 22, 19, }, // Magadan
    {  651, 1247,       0,  892679,       0,  0, 26,  0, }, // Marshall_Islands
    {  651, 1264,  892705,  892721,  892745, 16, 24, 21, }, // Mauritius
    {  651, 1281,  892766,  892790,  892821, 24, 31, 31, }, // Mexico_Pacific
    {  651, 1296,  888262,  892852,  892879, 17, 27, 22, }, // Mongolia
    {  651, 1305,  888328,  892901,  892921, 12, 20, 17, }, // Moscow
    {  651, 1332,  888417,  892938,  892968, 22, 30, 27, }, // New_Caledonia
    {  651, 1346,  888496,  892995,  893020, 17, 25, 25, }, // New_Zealand
    {  651, 1358,  888563,  893045,  893071, 18, 26, 26, }, // Newfoundland
    {  651, 1376,  888643,  893097,  893126, 21, 29, 29, }, // Norfolk
    {  651, 1406,  888801,  893155,  893181, 19, 26, 23, }, // Novosibirsk
    {  651, 1418,  888869,  893204,  893225, 13, 21, 18, }, // Omsk
    {  651, 1428,  888921,  893243,  893266, 15, 23, 20, }, // Pakistan
    {  651, 1460,  889012,  893286,  889048, 14, 22, 19, }, // Paraguay
    {  651, 1469,  889067,  893308,  889095, 10, 18, 15, }, // Peru
    {  651, 1474,  889110,  893326,  893351, 17, 25, 22, }, // Philippines
    {  651, 1486,       0,  893373,       0,  0, 26,  0, }, // Phoenix_Islands
    {  651, 1502,  889200,  893399,  893430, 23, 31, 31, }, // Pierre_Miquelon
    {  651, 1554,       0,  893461,       0,  0, 17,  0, }, // Reunion
    {  651, 1570,  889357,  893478,  893500, 14, 22, 19, }, // Sakhalin
    {  651, 1596,  889412,  893519,  893538, 11, 19, 19, }, // Samoa
    {  651, 1602,       0,  893557,       0,  0, 16,  0, }, // Seychelles
    {  651, 1624,       0,  893573,       0,  0, 23,  0, }, // Singapore
    {  651, 1634,       0,  893596,       0,  0, 26,  0, }, // Solomon
    {  651, 1642,       0,  893622,       0,  0, 17,  0, }, // South_Georgia
    {  651, 1689,  889579,  893639,  893661, 12, 22, 20, }, // Taipei
    {  651, 1732,  889665,  893681,  893700, 11, 19, 16, }, // Tonga
    {  651, 1750,  889722,  893716,  893744, 20, 28, 24, }, // Turkmenistan
    {  651, 1777,  889806,  893768,  889839, 12, 21, 17, }, // Uruguay
    {  651, 1792,  889856,  893789,  893815, 18, 26, 23, }, // Uzbekistan
    {  651, 1803,  889923,  893838,  893859, 13, 21, 18, }, // Vanuatu
    {  651, 1821,  889989,  893877,  893905, 20, 28, 25, }, // Vladivostok
    {  651, 1833,  890062,  893930,  893956, 18, 26, 23, }, // Volgograd
    {  651, 1862,  890181,  893979,  894000, 13, 21, 18, }, // Yakutsk
    {  651, 1870,  890233,  894018,  894045, 19, 27, 24, }, // Yekaterinburg
    {  651, 1892,       0,  894069,       0,  0, 13,  0, }, // Yukon
    {  653,  788,       0,  894082,       0,  0, 31,  0, }, // GMT Zhuang/Latin/China
    {  654,    5,       0,  894113,       0,  0, 26,  0, }, // Afghanistan Zulu/Latin/South Africa
    {  654,   17,       0,  894139,       0,  0, 29,  0, }, // Africa_Central
    {  654,   32,       0,  894168,       0,  0, 31,  0, }, // Africa_Eastern
    {  654,   65,       0,  894199,       0,  0, 40,  0, }, // Africa_Southern
    {  654,   81,  894239,  894272,  894315, 33, 43, 43, }, // Africa_Western
    {  654,  107,  894358,  894379,  894415, 21, 36, 30, }, // Alaska
    {  654,  135,  894445,  894466,  894502, 21, 36, 31, }, // Amazon
    {  654,  142,  894533,  894570,  894622, 37, 52, 46, }, // America_Central
    {  654,  158,  894668,  894702,  894751, 34, 49, 43, }, // America_Eastern
    {  654,  174,  894794,  894832,  894885, 38, 53, 47, }, // America_Mountain
    {  654,  191,  894932,  894969,  895021, 37, 52, 46, }, // America_Pacific
    {  654,  207,  895067,  895084,  895110, 17, 26, 24, }, // Anadyr
    {  654,  214,  895134,  895153,  895182, 19, 29, 28, }, // Apia
    {  654,  232,  895210,  895232,  895264, 22, 32, 29, }, // Arabian
    {  654,  240,  895293,  895317,  895356, 24, 39, 34, }, // Argentina
    {  654,  250,  895390,  895425,  895475, 35, 50, 45, }, // Argentina_Western
    {  654,  268,  895520,  895541,  895573, 21, 32, 30, }, // Armenia
    {  654,  286,  895603,  895626,  895664, 23, 38, 32, }, // Atlantic
    {  654,  295,  895696,  895728,  895771, 32, 43, 42, }, // Australia_Central
    {  654,  313,  895813,  895851,  895899, 38, 48, 47, }, // Australia_CentralWestern
    {  654,  338,  895946,  895978,  896018, 32, 40, 39, }, // Australia_Eastern
    {  654,  356,  896057,  896089,  896132, 32, 43, 42, }, // Australia_Western
    {  654,  374,  896174,  896199,  896234, 25, 35, 33, }, // Azerbaijan
    {  654,  385,  896267,  896288,  896324, 21, 36, 31, }, // Azores
    {  654,  397,  896355,  896380,  896415, 25, 35, 35, }, // Bangladesh
    {  654,  415,       0,  896450,       0,  0, 21,  0, }, // Bhutan
    {  654,  422,       0,  896471,       0,  0, 22,  0, }, // Bolivia
    {  654,  437,  896493,  896516,  896554, 23, 38, 33, }, // Brasilia
    {  654,  454,       0,  896587,       0,  0, 32,  0, }, // Brunei
    {  654,  461,  896619,  896644,  896680, 25, 36, 33, }, // Cape_Verde
    {  654,  478,       0,  896713,       0,  0, 33,  0, }, // Chamorro
    {  654,  487,  896746,  896768,  896800, 22, 32, 31, }, // Chatham
    {  654,  495,  896831,  896851,  896886, 20, 35, 30, }, // Chile
    {  654,  501,  896916,  896936,  896966, 20, 30, 27, }, // China
    {  654,  507,       0,  896993,       0,  0, 31,  0, }, // Christmas
    {  654,  517,       0,  897024,       0,  0, 28,  0, }, // Cocos
    {  654,  523,  897052,  897075,  897113, 23, 38, 33, }, // Colombia
    {  654,  532,  897146,  897173,  897210, 27, 37, 50, }, // Cook
    {  654,  537,  897260,  897279,  897313, 19, 34, 28, }, // Cuba
    {  654,  548,       0,  897341,       0,  0, 20,  0, }, // Davis
    {  654,  564,       0,  897361,       0,  0, 31,  0, }, // DumontDUrville
    {  654,  601,       0,  897392,       0,  0, 25,  0, }, // East_Timor
    {  654,  612,  897417,  897445,  897488, 28, 43, 38, }, // Easter
    {  654,  619,       0,  897526,       0,  0, 22,  0, }, // Ecuador
    {  654,  627,  897548,  897577,  897621, 29, 44, 39, }, // Europe_Central
    {  654,  642,  897660,  897689,  897733, 29, 44, 39, }, // Europe_Eastern
    {  654,  657,       0,  897772,       0,  0, 37,  0, }, // Europe_Further_Eastern
    {  654,  680,  897809,  897838,  897882, 29, 44, 39, }, // Europe_Western
    {  654,  695,  897921,  897952,  897998, 31, 46, 41, }, // Falkland
    {  654,  704,  898039,  898058,  898087, 19, 29, 27, }, // Fiji
    {  654,  709,       0,  898114,       0,  0, 28,  0, }, // French_Guiana
    {  654,  723,       0,  898142,       0,  0, 45,  0, }, // French_Southern
    {  654,  746,       0,  898187,       0,  0, 24,  0, }, // Galapagos
    {  654,  756,       0,  898211,       0,  0, 22,  0, }, // Gambier
    {  654,  764,  898233,  898255,  898287, 22, 32, 30, }, // Georgia
    {  654,  772,       0,  898317,       0,  0, 30,  0, }, // Gilbert_Islands
    {  654,  788,       0,  898347,       0,  0, 29,  0, }, // GMT
    {  654,  830,  898376,  898405,  898449, 29, 44, 38, }, // Greenland_Eastern
    {  654,  848,  898487,  898516,  898560, 29, 44, 39, }, // Greenland_Western
    {  654,  871,       0,  898599,       0,  0, 29,  0, }, // Gulf
    {  654,  876,       0,  898628,       0,  0, 21,  0, }, // Guyana
    {  654,  883,  898649,  898678,  898722, 29, 44, 38, }, // Hawaii_Aleutian
    {  654,  899,  898760,  898784,  898818, 24, 34, 32, }, // Hong_Kong
    {  654,  909,  898850,  898869,  898898, 19, 29, 24, }, // Hovd
    {  654,  914,       0,  898922,       0,  0, 30,  0, }, // India
    {  654,  920,       0,  898952,       0,  0, 27,  0, }, // Indian_Ocean
    {  654,  933,       0,  898979,       0,  0, 24,  0, }, // Indochina
    {  654,  943,       0,  899003,       0,  0, 32,  0, }, // Indonesia_Central
    {  654,  961,       0,  899035,       0,  0, 32,  0, }, // Indonesia_Eastern
    {  654,  979,       0,  899067,       0,  0, 32,  0, }, // Indonesia_Western
    {  654,  997,  899099,  899118,  899147, 19, 29, 28, }, // Iran
    {  654, 1008,  899175,  899197,  899229, 22, 32, 29, }, // Irkutsk
    {  654, 1016,  899258,  899279,  899310, 21, 31, 31, }, // Israel
    {  654, 1023,  899341,  899361,  899391, 20, 30, 27, }, // Japan
    {  654, 1029,  899418,  899453,  899497, 35, 44, 42, }, // Kamchatka
    {  654, 1047,       0,  899539,       0,  0, 24,  0, }, // Kazakhstan
    {  654, 1058,       0,  899563,       0,  0, 39,  0, }, // Kazakhstan_Eastern
    {  654, 1077,       0,  899602,       0,  0, 40,  0, }, // Kazakhstan_Western
    {  654, 1106,  899642,  899662,  899692, 20, 30, 27, }, // Korea
    {  654, 1112,       0,  899719,       0,  0, 21,  0, }, // Kosrae
    {  654, 1119,  899740,  899766,  899802, 26, 36, 33, }, // Krasnoyarsk
    {  654, 1151,       0,  899835,       0,  0, 24,  0, }, // Kyrgystan
    {  654, 1175,       0,  899859,       0,  0, 27,  0, }, // Line_Islands
    {  654, 1188,  899886,  899910,  899944, 24, 34, 33, }, // Lord_Howe
    {  654, 1204,  899977,  899999,  900031, 22, 32, 29, }, // Magadan
    {  654, 1219,       0,  900060,       0,  0, 23,  0, }, // Malaysia
    {  654, 1228,       0,  900083,       0,  0, 23,  0, }, // Maldives
    {  654, 1237,       0,  900106,       0,  0, 24,  0, }, // Marquesas
    {  654, 1247,       0,  900130,       0,  0, 31,  0, }, // Marshall_Islands
    {  654, 1264,  900161,  900185,  900219, 24, 34, 32, }, // Mauritius
    {  654, 1274,       0,  900251,       0,  0, 21,  0, }, // Mawson
    {  654, 1281,  900272,  900302,  900347, 30, 45, 39, }, // Mexico_Pacific
    {  654, 1296,  900386,  900411,  900446, 25, 35, 30, }, // Mongolia
    {  654, 1305,  900476,  900497,  900533, 21, 36, 28, }, // Moscow
    {  654, 1312,       0,  900561,       0,  0, 22,  0, }, // Myanmar
    {  654, 1320,       0,  900583,       0,  0, 20,  0, }, // Nauru
    {  654, 1326,       0,  900603,       0,  0, 20,  0, }, // Nepal
    {  654, 1332,  900623,  900651,  900689, 28, 38, 38, }, // New_Caledonia
    {  654, 1346,  900727,  900753,  900789, 26, 36, 35, }, // New_Zealand
    {  654, 1358,  900824,  900851,  900893, 27, 42, 36, }, // Newfoundland
    {  654, 1371,       0,  900929,       0,  0, 19,  0, }, // Niue
    {  654, 1376,  900948,  900978,  901018, 30, 40, 40, }, // Norfolk
    {  654, 1384,  901058,  901092,  901136, 34, 44, 44, }, // Noronha
    {  654, 1406,  901180,  901206,  901242, 26, 36, 36, }, // Novosibirsk
    {  654, 1418,  901278,  901297,  901326, 19, 29, 29, }, // Omsk
    {  654, 1428,  901355,  901378,  901411, 23, 33, 33, }, // Pakistan
    {  654, 1437,       0,  901444,       0,  0, 20,  0, }, // Palau
    {  654, 1443,       0,  901464,       0,  0, 31,  0, }, // Papua_New_Guinea
    {  654, 1460,  901495,  901518,  901551, 23, 33, 33, }, // Paraguay
    {  654, 1469,  901584,  901603,  901632, 19, 29, 29, }, // Peru
    {  654, 1474,  901661,  901686,  901721, 25, 35, 33, }, // Philippines
    {  654, 1486,       0,  901754,       0,  0, 30,  0, }, // Phoenix_Islands
    {  654, 1502,  901784,  901825,  901880, 41, 55, 50, }, // Pierre_Miquelon
    {  654, 1518,       0,  901930,       0,  0, 23,  0, }, // Pitcairn
    {  654, 1527,       0,  901953,       0,  0, 21,  0, }, // Ponape
    {  654, 1534,       0,  901974,       0,  0, 24,  0, }, // Pyongyang
    {  654, 1554,       0,  901998,       0,  0, 22,  0, }, // Reunion
    {  654, 1562,       0,  902020,       0,  0, 22,  0, }, // Rothera
    {  654, 1570,  902042,  902065,  902098, 23, 33, 30, }, // Sakhalin
    {  654, 1579,  902128,  902145,  902171, 17, 26, 24, }, // Samara
    {  654, 1596,  902195,  902215,  902245, 20, 30, 29, }, // Samoa
    {  654, 1602,       0,  902274,       0,  0, 25,  0, }, // Seychelles
    {  654, 1624,       0,  902299,       0,  0, 34,  0, }, // Singapore
    {  654, 1634,       0,  902333,       0,  0, 30,  0, }, // Solomon
    {  654, 1642,       0,  902363,       0,  0, 28,  0, }, // South_Georgia
    {  654, 1656,       0,  902391,       0,  0, 23,  0, }, // Suriname
    {  654, 1676,       0,  902414,       0,  0, 20,  0, }, // Syowa
    {  654, 1682,       0,  902434,       0,  0, 21,  0, }, // Tahiti
    {  654, 1689,  902455,  902476,  902507, 21, 31, 28, }, // Taipei
    {  654, 1696,       0,  902535,       0,  0, 25,  0, }, // Tajikistan
    {  654, 1724,       0,  902560,       0,  0, 22,  0, }, // Tokelau
    {  654, 1732,  902582,  902602,  902632, 20, 30, 30, }, // Tonga
    {  654, 1738,       0,  902662,       0,  0, 20,  0, }, // Truk
    {  654, 1750,  902682,  902709,  902746, 27, 37, 35, }, // Turkmenistan
    {  654, 1763,       0,  902781,       0,  0, 21,  0, }, // Tuvalu
    {  654, 1777,  902802,  902824,  902861, 22, 37, 32, }, // Uruguay
    {  654, 1792,  902893,  902918,  902953, 25, 35, 33, }, // Uzbekistan
    {  654, 1803,  902986,  903008,  903045, 22, 37, 32, }, // Vanuatu
    {  654, 1811,       0,  903077,       0,  0, 24,  0, }, // Venezuela
    {  654, 1821,  903101,  903127,  903163, 26, 36, 33, }, // Vladivostok
    {  654, 1833,  903196,  903220,  903254, 24, 34, 34, }, // Volgograd
    {  654, 1843,       0,  903288,       0,  0, 21,  0, }, // Vostok
    {  654, 1850,       0,  903309,       0,  0, 26,  0, }, // Wake
    {  654, 1855,       0,  903335,       0,  0, 33,  0, }, // Wallis
    {  654, 1862,  903368,  903390,  903422, 22, 32, 29, }, // Yakutsk
    {  654, 1870,  903451,  903479,  903517, 28, 38, 35, }, // Yekaterinburg
    {  654, 1892,       0,    6333,       0,  0, 10,  0, }, // Yukon
    {  655,    0,  903552,  903563,  903577, 11, 14, 17, }, // Acre Kaingang/Latin/Brazil
    {  655,    5,       0,  903594,       0,  0, 16,  0, }, // Afghanistan
    {  655,   17,       0,  903610,       0,  0, 18,  0, }, // Africa_Central
    {  655,   32,       0,  903628,       0,  0, 19,  0, }, // Africa_Eastern
    {  655,   65,       0,  903647,       0,  0, 17,  0, }, // Africa_Southern
    {  655,   81,  903664,  903683,  903705, 19, 22, 25, }, // Africa_Western
    {  655,  107,  903730,  903744,  903761, 14, 17, 20, }, // Alaska
    {  655,  128,  903781,  903794,  903810, 13, 16, 19, }, // Almaty
    {  655,  135,  903829,  903843,  903860, 14, 17, 20, }, // Amazon
    {  655,  142,  903617,  903880,  903894, 11, 14, 17, }, // America_Central
    {  655,  158,  903911,  903923,  903938, 12, 15, 18, }, // America_Eastern
    {  655,  174,  903956,  903966,  903979, 10, 13, 16, }, // America_Mountain
    {  655,  191,  903995,  904007,  904022, 12, 15, 18, }, // America_Pacific
    {  655,  207,  904040,  904054,  904071, 14, 17, 20, }, // Anadyr
    {  655,  214,  904091,  904103,  904118, 12, 15, 18, }, // Apia
    {  655,  219,  904136,  904148,  904163, 12, 15, 18, }, // Aqtau
    {  655,  225,  904181,  904194,  904210, 13, 16, 19, }, // Aqtobe
    {  655,  232,  904229,  904243,  904260, 14, 17, 20, }, // Arabian
    {  655,  240,  904280,  904295,  904314, 15, 19, 21, }, // Argentina
    {  655,  250,  904335,  904356,  904380, 21, 24, 26, }, // Argentina_Western
    {  655,  268,  904406,  904421,  904439, 15, 18, 21, }, // Armenia
    {  655,  286,  904460,  904475,  904493, 15, 18, 21, }, // Atlantic
    {  655,  295,  904514,  904536,  904561, 22, 25, 28, }, // Australia_Central
    {  655,  313,  904589,  904616,  904647, 27, 31, 34, }, // Australia_CentralWestern
    {  655,  338,  904681,  904704,  904730, 23, 26, 29, }, // Australia_Eastern
    {  655,  356,  904759,  904782,  904808, 23, 26, 29, }, // Australia_Western
    {  655,  374,  904837,  904854,  904874, 17, 20, 23, }, // Azerbaijan
    {  655,  385,  904897,  904911,  904928, 14, 17, 19, }, // Azores
    {  655,  397,  904947,  904964,  904984, 17, 20, 23, }, // Bangladesh
    {  655,  415,       0,  905007,       0,  0, 12,  0, }, // Bhutan
    {  655,  422,       0,  905019,       0,  0, 14,  0, }, // Bolivia
    {  655,  437,  905033,  905048,  905066, 15, 18, 21, }, // Brasilia
    {  655,  454,       0,  905087,       0,  0, 22,  0, }, // Brunei
    {  655,  461,  905109,  905123,  905140, 14, 17, 20, }, // Cape_Verde
    {  655,  478,       0,  905160,       0,  0, 13,  0, }, // Chamorro
    {  655,  487,  905173,  905187,  905204, 14, 17, 20, }, // Chatham
    {  655,  495,  905224,  905235,  905249, 11, 14, 17, }, // Chile
    {  655,  501,  905266,  905277,  905291, 11, 14, 17, }, // China
    {  655,  507,       0,  905308,       0,  0, 25,  0, }, // Christmas
    {  655,  517,       0,  905333,       0,  0, 22,  0, }, // Cocos
    {  655,  523,  905355,  905370,  905388, 15, 18, 21, }, // Colombia
    {  655,  532,  905409,  905429,  905452, 20, 23, 26, }, // Cook
    {  655,  537,  905478,  905489,  905503, 11, 14, 17, }, // Cuba
    {  655,  548,       0,  905520,       0,  0, 13,  0, }, // Davis
    {  655,  564,       0,  905533,       0,  0, 19,  0, }, // DumontDUrville
    {  655,  601,       0,  905552,       0,  0, 18,  0, }, // East_Timor
    {  655,  612,  905570,  905594,  905621, 24, 27, 30, }, // Easter
    {  655,  619,       0,  905651,       0,  0, 15,  0, }, // Ecuador
    {  655,  627,  905666,  905683,  905703, 17, 20, 23, }, // Europe_Central
    {  655,  642,  905726,  905744,  905765, 18, 21, 24, }, // Europe_Eastern
    {  655,  657,       0,  905789,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  655,  680,  905816,  905834,  905855, 18, 21, 24, }, // Europe_Western
    {  655,  695,  905879,  905902,  905927, 23, 25, 29, }, // Falkland
    {  655,  704,  905956,  905967,  905981, 11, 14, 17, }, // Fiji
    {  655,  709,       0,  905998,       0,  0, 19,  0, }, // French_Guiana
    {  655,  723,       0,  906017,       0,  0, 32,  0, }, // French_Southern
    {  655,  746,       0,  906049,       0,  0, 16,  0, }, // Galapagos
    {  655,  756,       0,  906065,       0,  0, 12,  0, }, // Gambier
    {  655,  764,  906077,  906090,  906106, 13, 16, 19, }, // Georgia
    {  655,  772,       0,  906125,       0,  0, 24,  0, }, // Gilbert_Islands
    {  655,  788,       0,  906149,       0,  0, 26,  0, }, // GMT
    {  655,  830,  906175,  906198,  906224, 23, 26, 29, }, // Greenland_Eastern
    {  655,  848,  906253,  906276,  906302, 23, 26, 29, }, // Greenland_Western
    {  655,  866,       0,  906331,       0,  0, 15,  0, }, // Guam
    {  655,  871,       0,  906346,       0,  0, 12,  0, }, // Gulf
    {  655,  876,       0,  906358,       0,  0, 13,  0, }, // Guyana
    {  655,  883,  906371,  906404,  906440, 33, 36, 39, }, // Hawaii_Aleutian
    {  655,  899,  906479,  906493,  906510, 14, 17, 20, }, // Hong_Kong
    {  655,  909,  906530,  906542,  906557, 12, 15, 18, }, // Hovd
    {  655,  914,       0,  906575,       0,  0, 15,  0, }, // India
    {  655,  920,       0,  906590,       0,  0, 19,  0, }, // Indian_Ocean
    {  655,  933,       0,  906609,       0,  0, 14,  0, }, // Indochina
    {  655,  943,       0,  906623,       0,  0, 19,  0, }, // Indonesia_Central
    {  655,  961,       0,  906642,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  655,  979,       0,  906662,       0,  0, 20,  0, }, // Indonesia_Western
    {  655,  997,  906682,  906692,  906705, 10, 13, 16, }, // Iran
    {  655, 1008,  906721,  906738,  906758, 17, 20, 23, }, // Irkutsk
    {  655, 1016,  906781,  906794,  906810, 13, 16, 19, }, // Israel
    {  655, 1023,  906829,  906840,  906854, 11, 14, 17, }, // Japan
    {  655, 1029,  906871,  906904,  906940, 33, 36, 39, }, // Kamchatka
    {  655, 1058,       0,  906979,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  655, 1077,       0,  907003,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  655, 1106,  907027,  907040,  907056, 13, 16, 19, }, // Korea
    {  655, 1112,       0,  907075,       0,  0, 18,  0, }, // Kosrae
    {  655, 1119,  907093,  907112,  907134, 19, 22, 25, }, // Krasnoyarsk
    {  655, 1151,       0,  907159,       0,  0, 17,  0, }, // Kyrgystan
    {  655, 1161,       0,  907176,       0,  0, 12,  0, }, // Lanka
    {  655, 1175,       0,  907188,       0,  0, 23,  0, }, // Line_Islands
    {  655, 1188,  907211,  907228,  907248, 17, 20, 23, }, // Lord_Howe
    {  655, 1198,  907271,  907283,  907298, 12, 15, 18, }, // Macau
    {  655, 1204,  907316,  907330,  907347, 14, 17, 20, }, // Magadan
    {  655, 1219,       0,  907367,       0,  0, 13,  0, }, // Malaysia
    {  655, 1228,       0,  907380,       0,  0, 23,  0, }, // Maldives
    {  655, 1237,       0,  907403,       0,  0, 18,  0, }, // Marquesas
    {  655, 1247,       0,  907421,       0,  0, 22,  0, }, // Marshall_Islands
    {  655, 1264,  907443,  907456,  907472, 13, 16, 19, }, // Mauritius
    {  655, 1274,       0,  907491,       0,  0, 13,  0, }, // Mawson
    {  655, 1281,  907504,  907526,  907551, 22, 25, 28, }, // Mexico_Pacific
    {  655, 1296,  907579,  907596,  907616, 17, 20, 23, }, // Mongolia
    {  655, 1305,  907639,  907653,  907670, 14, 17, 20, }, // Moscow
    {  655, 1312,       0,  907690,       0,  0, 14,  0, }, // Myanmar
    {  655, 1320,       0,  907704,       0,  0, 13,  0, }, // Nauru
    {  655, 1326,       0,  907717,       0,  0, 12,  0, }, // Nepal
    {  655, 1332,  907729,  907750,  907774, 21, 24, 27, }, // New_Caledonia
    {  655, 1346,  907801,  907820,  907842, 19, 22, 25, }, // New_Zealand
    {  655, 1358,  907867,  907880,  907896, 13, 16, 19, }, // Newfoundland
    {  655, 1371,       0,  907915,       0,  0, 11,  0, }, // Niue
    {  655, 1376,  907926,  907950,  907977, 24, 27, 30, }, // Norfolk
    {  655, 1384,  908007,  908031,  908058, 24, 27, 30, }, // Noronha
    {  655, 1392,       0,  908088,       0,  0, 30,  0, }, // North_Mariana
    {  655, 1406,  908118,  908138,  908161, 20, 23, 29, }, // Novosibirsk
    {  655, 1418,  908190,  908204,  908221, 14, 17, 20, }, // Omsk
    {  655, 1428,  908241,  908257,  908276, 16, 19, 22, }, // Pakistan
    {  655, 1437,       0,  908298,       0,  0, 13,  0, }, // Palau
    {  655, 1443,       0,  908311,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  655, 1460,  908333,  908349,  908368, 16, 19, 22, }, // Paraguay
    {  655, 1469,  908390,  908401,  908415, 11, 14, 17, }, // Peru
    {  655, 1474,  908432,  908447,  908465, 15, 18, 17, }, // Philippines
    {  655, 1486,       0,  908482,       0,  0, 21,  0, }, // Phoenix_Islands
    {  655, 1502,  908503,  908530,  908560, 27, 30, 33, }, // Pierre_Miquelon
    {  655, 1518,       0,  908593,       0,  0, 24,  0, }, // Pitcairn
    {  655, 1527,       0,  908617,       0,  0, 13,  0, }, // Ponape
    {  655, 1534,       0,  908630,       0,  0, 15,  0, }, // Pyongyang
    {  655, 1544,  908645,  908661,  908680, 16, 19, 22, }, // Qyzylorda
    {  655, 1554,       0,  908702,       0,  0, 12,  0, }, // Reunion
    {  655, 1562,       0,  908714,       0,  0, 13,  0, }, // Rothera
    {  655, 1570,  908727,  908742,  908760, 15, 18, 21, }, // Sakhalin
    {  655, 1579,  908781,  908794,  908810, 13, 16, 19, }, // Samara
    {  655, 1596,  908829,  908842,  908858, 13, 16, 19, }, // Samoa
    {  655, 1602,       0,  908877,       0,  0, 16,  0, }, // Seychelles
    {  655, 1624,       0,  908893,       0,  0, 18,  0, }, // Singapore
    {  655, 1634,       0,  908911,       0,  0, 23,  0, }, // Solomon
    {  655, 1642,       0,  908934,       0,  0, 20,  0, }, // South_Georgia
    {  655, 1656,       0,  908954,       0,  0, 15,  0, }, // Suriname
    {  655, 1676,       0,  908969,       0,  0, 12,  0, }, // Syowa
    {  655, 1682,       0,  908981,       0,  0, 12,  0, }, // Tahiti
    {  655, 1689,  908993,  909006,  909022, 13, 16, 19, }, // Taipei
    {  655, 1696,       0,  909041,       0,  0, 18,  0, }, // Tajikistan
    {  655, 1724,       0,  909059,       0,  0, 15,  0, }, // Tokelau
    {  655, 1732,  909074,  909085,  909099, 11, 14, 17, }, // Tonga
    {  655, 1738,       0,  909116,       0,  0, 12,  0, }, // Truk
    {  655, 1750,  909128,  909149,  909173, 21, 24, 27, }, // Turkmenistan
    {  655, 1763,       0,  909200,       0,  0, 13,  0, }, // Tuvalu
    {  655, 1777,  909213,  909228,  909246, 15, 18, 21, }, // Uruguay
    {  655, 1792,  909267,  909286,  909308, 19, 22, 25, }, // Uzbekistan
    {  655, 1803,  909333,  909348,  909366, 15, 18, 21, }, // Vanuatu
    {  655, 1811,       0,  909387,       0,  0, 17,  0, }, // Venezuela
    {  655, 1821,  909404,  909425,  909448, 21, 23, 26, }, // Vladivostok
    {  655, 1833,  909474,  909491,  909511, 17, 20, 23, }, // Volgograd
    {  655, 1843,       0,  909534,       0,  0, 15,  0, }, // Vostok
    {  655, 1850,       0,  909549,       0,  0, 21,  0, }, // Wake
    {  655, 1855,       0,  909570,       0,  0, 24,  0, }, // Wallis
    {  655, 1862,  909594,  909612,  909634, 18, 22, 24, }, // Yakutsk
    {  655, 1870,  909658,  909678,  909701, 20, 23, 26, }, // Yekaterinburg
    {  656,    0,  909727,  909745,  909770, 18, 25, 29, }, // Acre Nheengatu/Latin/Brazil
    {  656,    5,       0,  909799,       0,  0, 22,  0, }, // Afghanistan
    {  656,   17,       0,  909821,       0,  0, 27,  0, }, // Africa_Central
    {  656,   32,       0,  909848,       0,  0, 27,  0, }, // Africa_Eastern
    {  656,   65,       0,  909875,       0,  0, 23,  0, }, // Africa_Southern
    {  656,   81,  909898,  909927,  909962, 29, 35, 39, }, // Africa_Western
    {  656,  107,  910001,  910017,  910037, 16, 20, 27, }, // Alaska
    {  656,  128,  910064,  910079,  910098, 15, 19, 26, }, // Almaty
    {  656,  135,  910124,  910140,  910160, 16, 20, 27, }, // Amazon
    {  656,  142,  910187,  910195,  910221,  8, 26, 30, }, // America_Central
    {  656,  158,  910251,  910266,  910293, 15, 27, 26, }, // America_Eastern
    {  656,  174,  910319,  910332,  910348, 13, 16, 19, }, // America_Mountain
    {  656,  191,  910367,  910382,  910400, 15, 18, 21, }, // America_Pacific
    {  656,  207,  910421,  910435,  910456, 14, 21, 25, }, // Anadyr
    {  656,  214,  910481,  910495,  910516, 14, 21, 25, }, // Apia
    {  656,  219,  910541,  910556,  910578, 15, 22, 26, }, // Aqtau
    {  656,  225,  910604,  910620,  910643, 16, 23, 27, }, // Aqtobe
    {  656,  232,  910670,  910686,  910709, 16, 23, 27, }, // Arabian
    {  656,  240,  910736,  910753,  910777, 17, 24, 28, }, // Argentina
    {  656,  250,  910805,  910834,  910870, 29, 36, 40, }, // Argentina_Western
    {  656,  268,  910910,  910928,  910953, 18, 25, 29, }, // Armenia
    {  656,  286,  910982,  911000,  911025, 18, 25, 28, }, // Atlantic
    {  656,  295,  911053,  911085,  911124, 32, 39, 43, }, // Australia_Central
    {  656,  313,  911167,  911211,  911262, 44, 51, 55, }, // Australia_CentralWestern
    {  656,  338,  911317,  911349,  911388, 32, 39, 43, }, // Australia_Eastern
    {  656,  356,  911431,  911464,  911504, 33, 40, 44, }, // Australia_Western
    {  656,  374,  911548,  911568,  911595, 20, 27, 31, }, // Azerbaijan
    {  656,  385,  911626,  911644,  911669, 18, 25, 29, }, // Azores
    {  656,  397,  911698,  911717,  911743, 19, 26, 30, }, // Bangladesh
    {  656,  415,       0,  911773,       0,  0, 14,  0, }, // Bhutan
    {  656,  422,       0,  911787,       0,  0, 16,  0, }, // Bolivia
    {  656,  437,  911803,  911818,  911836, 15, 18, 21, }, // Brasilia
    {  656,  454,       0,  911857,       0,  0, 26,  0, }, // Brunei
    {  656,  461,  911883,  911904,  911932, 21, 28, 32, }, // Cape_Verde
    {  656,  478,       0,  911964,       0,  0, 15,  0, }, // Chamorro
    {  656,  487,  911979,  911994,  912016, 15, 22, 26, }, // Chatham
    {  656,  495,  912042,  912055,  912075, 13, 20, 24, }, // Chile
    {  656,  501,  912099,  912112,  912132, 13, 20, 24, }, // China
    {  656,  507,       0,  912156,       0,  0, 24,  0, }, // Christmas
    {  656,  517,       0,  912180,       0,  0, 27,  0, }, // Cocos
    {  656,  523,  912207,  912223,  912246, 16, 23, 27, }, // Colombia
    {  656,  532,  912273,  912297,  912328, 24, 31, 42, }, // Cook
    {  656,  537,  912370,  912383,  912403, 13, 20, 24, }, // Cuba
    {  656,  548,       0,  912427,       0,  0, 13,  0, }, // Davis
    {  656,  564,       0,  912440,       0,  0, 25,  0, }, // DumontDUrville
    {  656,  601,       0,  912465,       0,  0, 22,  0, }, // East_Timor
    {  656,  612,  912487,  912512,  912544, 25, 32, 36, }, // Easter
    {  656,  619,       0,  912580,       0,  0, 15,  0, }, // Ecuador
    {  656,  627,  912595,  912621,  912654, 26, 33, 37, }, // Europe_Central
    {  656,  642,  912691,  912717,  912750, 26, 33, 37, }, // Europe_Eastern
    {  656,  657,       0,  912787,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  656,  680,  912813,  912840,  912874, 27, 34, 38, }, // Europe_Western
    {  656,  695,  912912,  912937,  912969, 25, 32, 36, }, // Falkland
    {  656,  704,  913005,  913018,  913038, 13, 20, 24, }, // Fiji
    {  656,  709,       0,  913062,       0,  0, 26,  0, }, // French_Guiana
    {  656,  723,       0,  913088,       0,  0, 47,  0, }, // French_Southern
    {  656,  746,       0,  913135,       0,  0, 21,  0, }, // Galapagos
    {  656,  756,       0,  913156,       0,  0, 16,  0, }, // Gambier
    {  656,  764,  913172,  913188,  913211, 16, 23, 27, }, // Georgia
    {  656,  772,       0,  913238,       0,  0, 25,  0, }, // Gilbert_Islands
    {  656,  788,       0,  913263,       0,  0, 34,  0, }, // GMT
    {  656,  830,  913297,  913328,  913366, 31, 38, 42, }, // Greenland_Eastern
    {  656,  848,  913408,  913440,  913479, 32, 39, 43, }, // Greenland_Western
    {  656,  866,       0,  913522,       0,  0, 20,  0, }, // Guam
    {  656,  871,       0,  913542,       0,  0, 14,  0, }, // Gulf
    {  656,  876,       0,  913556,       0,  0, 15,  0, }, // Guyana
    {  656,  883,  913571,  913611,  913658, 40, 47, 51, }, // Hawaii_Aleutian
    {  656,  899,  913709,  913725,  913748, 16, 23, 27, }, // Hong_Kong
    {  656,  909,  913775,  913790,  913812, 15, 22, 26, }, // Hovd
    {  656,  914,       0,  913838,       0,  0, 20,  0, }, // India
    {  656,  920,       0,  913858,       0,  0, 22,  0, }, // Indian_Ocean
    {  656,  933,       0,  913880,       0,  0, 16,  0, }, // Indochina
    {  656,  943,       0,  913896,       0,  0, 29,  0, }, // Indonesia_Central
    {  656,  961,       0,  913925,       0,  0, 29,  0, }, // Indonesia_Eastern
    {  656,  979,       0,  913954,       0,  0, 30,  0, }, // Indonesia_Western
    {  656,  997,  913984,  913996,  914015, 12, 19, 23, }, // Iran
    {  656, 1008,  914038,  914057,  914083, 19, 26, 30, }, // Irkutsk
    {  656, 1016,  914113,  914130,  914154, 17, 24, 28, }, // Israel
    {  656, 1023,  914182,  914195,  914215, 13, 20, 24, }, // Japan
    {  656, 1029,  914239,  914272,  914312, 33, 40, 44, }, // Kamchatka
    {  656, 1058,       0,  914356,       0,  0, 32,  0, }, // Kazakhstan_Eastern
    {  656, 1077,       0,  914388,       0,  0, 33,  0, }, // Kazakhstan_Western
    {  656, 1106,  914421,  914436,  914458, 15, 22, 26, }, // Korea
    {  656, 1112,       0,  914484,       0,  0, 16,  0, }, // Kosrae
    {  656, 1119,  914500,  914525,  914557, 25, 32, 36, }, // Krasnoyarsk
    {  656, 1151,       0,  914593,       0,  0, 21,  0, }, // Kyrgystan
    {  656, 1161,       0,  914614,       0,  0, 13,  0, }, // Lanka
    {  656, 1175,       0,  914627,       0,  0, 24,  0, }, // Line_Islands
    {  656, 1188,  914651,  914671,  914698, 20, 27, 31, }, // Lord_Howe
    {  656, 1198,  914729,  914743,  914764, 14, 21, 25, }, // Macau
    {  656, 1204,  914789,  914804,  914826, 15, 22, 26, }, // Magadan
    {  656, 1219,       0,  914852,       0,  0, 17,  0, }, // Malaysia
    {  656, 1228,       0,  914869,       0,  0, 30,  0, }, // Maldives
    {  656, 1237,       0,  914899,       0,  0, 19,  0, }, // Marquesas
    {  656, 1247,       0,  914918,       0,  0, 27,  0, }, // Marshall_Islands
    {  656, 1264,  914945,  914963,  914988, 18, 25, 29, }, // Mauritius
    {  656, 1274,       0,  915017,       0,  0, 14,  0, }, // Mawson
    {  656, 1281,  915031,  915057,  915090, 26, 33, 37, }, // Mexico_Pacific
    {  656, 1296,  915127,  915146,  915172, 19, 26, 30, }, // Mongolia
    {  656, 1305,  915202,  915217,  915239, 15, 22, 26, }, // Moscow
    {  656, 1312,       0,  915265,       0,  0, 15,  0, }, // Myanmar
    {  656, 1320,       0,  915280,       0,  0, 14,  0, }, // Nauru
    {  656, 1326,       0,  915294,       0,  0, 14,  0, }, // Nepal
    {  656, 1332,  915308,  915334,  915367, 26, 33, 37, }, // New_Caledonia
    {  656, 1346,  915404,  915428,  915459, 24, 31, 35, }, // New_Zealand
    {  656, 1358,  915494,  915513,  915539, 19, 26, 30, }, // Newfoundland
    {  656, 1371,       0,  915569,       0,  0, 13,  0, }, // Niue
    {  656, 1376,  915582,  915606,  915637, 24, 31, 35, }, // Norfolk
    {  656, 1384,  915672,  915694,  915723, 22, 29, 33, }, // Noronha
    {  656, 1392,       0,  915756,       0,  0, 36,  0, }, // North_Mariana
    {  656, 1406,  915792,  915818,  915851, 26, 33, 37, }, // Novosibirsk
    {  656, 1418,  915888,  915904,  915927, 16, 23, 27, }, // Omsk
    {  656, 1428,  915954,  915973,  915999, 19, 26, 30, }, // Pakistan
    {  656, 1437,       0,  916029,       0,  0, 14,  0, }, // Palau
    {  656, 1443,       0,  916043,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  656, 1460,  916070,  916088,  916113, 18, 25, 28, }, // Paraguay
    {  656, 1469,  916141,  916154,  916174, 13, 20, 24, }, // Peru
    {  656, 1474,  916198,  916215,  916239, 17, 24, 28, }, // Philippines
    {  656, 1486,       0,  916267,       0,  0, 15,  0, }, // Phoenix_Islands
    {  656, 1502,  916282,  916313,  916351, 31, 38, 42, }, // Pierre_Miquelon
    {  656, 1518,       0,  916393,       0,  0, 17,  0, }, // Pitcairn
    {  656, 1527,       0,  916410,       0,  0, 15,  0, }, // Ponape
    {  656, 1534,       0,  916425,       0,  0, 17,  0, }, // Pyongyang
    {  656, 1544,  916442,  916459,  916483, 17, 24, 28, }, // Qyzylorda
    {  656, 1554,       0,  916511,       0,  0, 22,  0, }, // Reunion
    {  656, 1562,       0,  916533,       0,  0, 15,  0, }, // Rothera
    {  656, 1570,  916548,  916565,  916589, 17, 24, 28, }, // Sakhalin
    {  656, 1579,  916617,  916632,  916654, 15, 22, 26, }, // Samara
    {  656, 1596,  916680,  916695,  916717, 15, 22, 26, }, // Samoa
    {  656, 1602,       0,  916743,       0,  0, 16,  0, }, // Seychelles
    {  656, 1624,       0,  916759,       0,  0, 24,  0, }, // Singapore
    {  656, 1634,       0,  916783,       0,  0, 27,  0, }, // Solomon
    {  656, 1642,       0,  916810,       0,  0, 23,  0, }, // South_Georgia
    {  656, 1656,       0,  916833,       0,  0, 17,  0, }, // Suriname
    {  656, 1676,       0,  916850,       0,  0, 15,  0, }, // Syowa
    {  656, 1682,       0,  916865,       0,  0, 14,  0, }, // Tahiti
    {  656, 1689,  916879,  916894,  916916, 15, 22, 26, }, // Taipei
    {  656, 1696,       0,  916942,       0,  0, 21,  0, }, // Tajikistan
    {  656, 1724,       0,  916963,       0,  0, 16,  0, }, // Tokelau
    {  656, 1732,  916979,  916992,  917012, 13, 20, 24, }, // Tonga
    {  656, 1738,       0,  917036,       0,  0, 14,  0, }, // Truk
    {  656, 1750,  917050,  917074,  917105, 24, 31, 35, }, // Turkmenistan
    {  656, 1763,       0,  917140,       0,  0, 15,  0, }, // Tuvalu
    {  656, 1777,  917155,  917172,  917196, 17, 24, 28, }, // Uruguay
    {  656, 1792,  917224,  917249,  917281, 25, 32, 36, }, // Uzbekistan
    {  656, 1803,  917317,  917333,  917356, 16, 23, 27, }, // Vanuatu
    {  656, 1811,       0,  917383,       0,  0, 18,  0, }, // Venezuela
    {  656, 1821,  917401,  917424,  917454, 23, 30, 34, }, // Vladivostok
    {  656, 1833,  917488,  917508,  917535, 20, 27, 31, }, // Volgograd
    {  656, 1843,       0,  917566,       0,  0, 17,  0, }, // Vostok
    {  656, 1850,       0,  917583,       0,  0, 14,  0, }, // Wake
    {  656, 1855,       0,  917597,       0,  0, 25,  0, }, // Wallis
    {  656, 1862,  917622,  917641,  917667, 19, 26, 30, }, // Yakutsk
    {  656, 1870,  917697,  917717,  917744, 20, 27, 31, }, // Yekaterinburg
    {  657,  415,       0,  917775,       0,  0, 14,  0, }, // Bhutan Nheengatu/Latin/Colombia
    {  657, 1502,  917789,  917820,  917858, 31, 38, 42, }, // Pierre_Miquelon
    {  658,  415,       0,  917775,       0,  0, 14,  0, }, // Bhutan Nheengatu/Latin/Venezuela
    {  658, 1502,  917789,  917820,  917858, 31, 38, 42, }, // Pierre_Miquelon
    {  659,  788,       0,   91136,       0,  0, 17,  0, }, // GMT Haryanvi/Devanagari/India
    {  660,    5,       0,  917900,       0,  0, 18,  0, }, // Afghanistan Northern Frisian/Latin/Germany
    {  660,   17,       0,  917918,       0,  0, 21,  0, }, // Africa_Central
    {  660,   32,       0,  917939,       0,  0, 17,  0, }, // Africa_Eastern
    {  660,   65,       0,  917956,       0,  0, 29,  0, }, // Africa_Southern
    {  660,   81,  917985,  918003,  918030, 18, 27, 23, }, // Africa_Western
    {  660,  107,  918053,  918064,  918084, 11, 20, 16, }, // Alaska
    {  660,  135,  918100,  918113,  918135, 13, 22, 18, }, // Amazon
    {  660,  142,  918153,  918176,  918208, 23, 32, 28, }, // America_Central
    {  660,  158,  918236,  918255,  918283, 19, 28, 24, }, // America_Eastern
    {  660,  174,  918307,  918327,  918356, 20, 29, 25, }, // America_Mountain
    {  660,  191,  918381,  918403,  918435, 22, 32, 28, }, // America_Pacific
    {  660,  214,  918463,  918472,  918490,  9, 18, 14, }, // Apia
    {  660,  232,  918504,  918517,  918539, 13, 22, 18, }, // Arabian
    {  660,  240,  918557,  918574,  918600, 17, 26, 22, }, // Argentina
    {  660,  250,  918622,  918645,  918677, 23, 32, 28, }, // Argentina_Western
    {  660,  268,  918705,  918719,  918742, 14, 23, 19, }, // Armenia
    {  660,  286,  918761,  918784,  918816, 23, 32, 28, }, // Atlantic
    {  660,  295,  918844,  918869,  918903, 25, 34, 30, }, // Australia_Central
    {  660,  313,  918933,  918963,  919002, 30, 39, 35, }, // Australia_CentralWestern
    {  660,  338,  919037,  919058,  919088, 21, 30, 26, }, // Australia_Eastern
    {  660,  356,  919114,  919136,  919167, 22, 31, 27, }, // Australia_Western
    {  660,  374,  919194,  919213,  919241, 19, 28, 24, }, // Azerbaijan
    {  660,  385,  919265,  919278,  919300, 13, 22, 18, }, // Azores
    {  660,  397,  919318,  919334,  919359, 16, 25, 21, }, // Bangladesh
    {  660,  415,       0,  919380,       0,  0, 13,  0, }, // Bhutan
    {  660,  422,       0,  919393,       0,  0, 14,  0, }, // Bolivia
    {  660,  437,  919407,  919422,  919446, 15, 24, 20, }, // Brasilia
    {  660,  454,       0,  919466,       0,  0, 22,  0, }, // Brunei
    {  660,  461,  919488,  919502,  919525, 14, 23, 20, }, // Cape_Verde
    {  660,  478,       0,  919545,       0,  0, 13,  0, }, // Chamorro
    {  660,  487,  919558,  919570,  919591, 12, 21, 17, }, // Chatham
    {  660,  495,  919608,  919619,  919639, 11, 20, 16, }, // Chile
    {  660,  501,  919655,  919667,  919688, 12, 21, 17, }, // China
    {  660,  507,       0,  919705,       0,  0, 14,  0, }, // Christmas
    {  660,  517,       0,  919719,       0,  0, 18,  0, }, // Cocos
    {  660,  523,  919737,  919751,  919774, 14, 23, 19, }, // Colombia
    {  660,  532,  919793,  919810,  919836, 17, 26, 29, }, // Cook
    {  660,  537,  919865,  919875,  919894, 10, 19, 15, }, // Cuba
    {  660,  548,       0,  919909,       0,  0, 10,  0, }, // Davis
    {  660,  564,       0,  919919,       0,  0, 21,  0, }, // DumontDUrville
    {  660,  601,       0,  919940,       0,  0, 16,  0, }, // East_Timor
    {  660,  612,  919956,  919972,  919997, 16, 25, 21, }, // Easter
    {  660,  619,       0,  920018,       0,  0, 13,  0, }, // Ecuador
    {  660,  627,  920031,  920050,  920078, 19, 28, 24, }, // Europe_Central
    {  660,  642,  920102,  920120,  920147, 18, 27, 23, }, // Europe_Eastern
    {  660,  657,       0,  920170,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  660,  680,  920196,  920215,  920243, 19, 28, 24, }, // Europe_Western
    {  660,  695,  920267,  920287,  920316, 20, 29, 25, }, // Falkland
    {  660,  704,  920341,  920353,  920374, 12, 21, 17, }, // Fiji
    {  660,  709,       0,  920391,       0,  0, 23,  0, }, // French_Guiana
    {  660,  723,       0,  920414,       0,  0, 32,  0, }, // French_Southern
    {  660,  746,       0,  920446,       0,  0, 14,  0, }, // Galapagos
    {  660,  756,       0,  920460,       0,  0, 12,  0, }, // Gambier
    {  660,  764,  920472,  920485,  920507, 13, 22, 18, }, // Georgia
    {  660,  772,       0,  920525,       0,  0, 20,  0, }, // Gilbert_Islands
    {  660,  788,       0,  920545,       0,  0, 20,  0, }, // GMT
    {  660,  830,  920565,  920583,  920610, 18, 27, 23, }, // Greenland_Eastern
    {  660,  848,  920633,  920652,  920680, 19, 28, 24, }, // Greenland_Western
    {  660,  871,       0,  920704,       0,  0,  9,  0, }, // Gulf
    {  660,  876,       0,  920713,       0,  0, 12,  0, }, // Guyana
    {  660,  883,  920725,  920744,  920772, 19, 28, 24, }, // Hawaii_Aleutian
    {  660,  899,  920796,  920809,  920831, 13, 22, 18, }, // Hong_Kong
    {  660,  909,  920849,  920859,  920878, 10, 19, 15, }, // Hovd
    {  660,  914,       0,  920893,       0,  0, 11,  0, }, // India
    {  660,  920,       0,  920904,       0,  0, 19,  0, }, // Indian_Ocean
    {  660,  933,       0,  920923,       0,  0, 16,  0, }, // Indochina
    {  660,  943,       0,  920939,       0,  0, 25,  0, }, // Indonesia_Central
    {  660,  961,       0,  920964,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  660,  979,       0,  920985,       0,  0, 22,  0, }, // Indonesia_Western
    {  660,  997,  921007,  921017,  921036, 10, 19, 15, }, // Iran
    {  660, 1008,  921051,  921063,  921084, 12, 21, 17, }, // Irkutsk
    {  660, 1016,  921101,  921112,  921132, 11, 20, 16, }, // Israel
    {  660, 1023,  921148,  921159,  921179, 11, 20, 16, }, // Japan
    {  660, 1047,       0,  921195,       0,  0, 16,  0, }, // Kazakhstan
    {  660, 1058,       0,  921211,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  660, 1077,       0,  921232,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  660, 1106,  921254,  921264,  921283, 10, 19, 15, }, // Korea
    {  660, 1112,       0,  921298,       0,  0, 11,  0, }, // Kosrae
    {  660, 1119,  921309,  921325,  921350, 16, 25, 21, }, // Krasnoyarsk
    {  660, 1151,       0,  921371,       0,  0, 15,  0, }, // Kyrgystan
    {  660, 1175,       0,  921386,       0,  0, 17,  0, }, // Line_Islands
    {  660, 1188,  921403,  921417,  921440, 14, 23, 19, }, // Lord_Howe
    {  660, 1204,  921459,  921471,  921492, 12, 21, 17, }, // Magadan
    {  660, 1219,       0,  921509,       0,  0, 13,  0, }, // Malaysia
    {  660, 1228,       0,  921522,       0,  0, 15,  0, }, // Maldives
    {  660, 1237,       0,  921537,       0,  0, 14,  0, }, // Marquesas
    {  660, 1247,       0,  921551,       0,  0, 21,  0, }, // Marshall_Islands
    {  660, 1264,  921572,  921586,  921609, 14, 23, 19, }, // Mauritius
    {  660, 1274,       0,  921628,       0,  0, 11,  0, }, // Mawson
    {  660, 1281,  921639,  921660,  921690, 21, 30, 26, }, // Mexico_Pacific
    {  660, 1296,  921716,  921732,  921757, 16, 25, 21, }, // Mongolia
    {  660, 1305,  921778,  921789,  921809, 11, 20, 16, }, // Moscow
    {  660, 1312,       0,  921825,       0,  0, 13,  0, }, // Myanmar
    {  660, 1320,       0,  921838,       0,  0, 10,  0, }, // Nauru
    {  660, 1326,       0,  921848,       0,  0, 12,  0, }, // Nepal
    {  660, 1332,  921860,  921879,  921907, 19, 28, 24, }, // New_Caledonia
    {  660, 1346,  921931,  921945,  921968, 14, 23, 19, }, // New_Zealand
    {  660, 1358,  921987,  922002,  922026, 15, 24, 20, }, // Newfoundland
    {  660, 1371,       0,  922046,       0,  0,  9,  0, }, // Niue
    {  660, 1376,  922055,  922073,  922100, 18, 27, 23, }, // Norfolk
    {  660, 1384,  922123,  922147,  922180, 24, 33, 29, }, // Noronha
    {  660, 1406,  922209,  922225,  922250, 16, 25, 21, }, // Novosibirsk
    {  660, 1418,  922271,  922280,  922298,  9, 18, 14, }, // Omsk
    {  660, 1428,  922312,  922326,  922349, 14, 23, 19, }, // Pakistan
    {  660, 1437,       0,  922368,       0,  0, 10,  0, }, // Palau
    {  660, 1443,       0,  922378,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  660, 1460,  922398,  922411,  922433, 13, 22, 18, }, // Paraguay
    {  660, 1469,  922451,  922461,  922480, 10, 19, 15, }, // Peru
    {  660, 1474,  922495,  922510,  922534, 15, 24, 20, }, // Philippines
    {  660, 1486,       0,  922554,       0,  0, 20,  0, }, // Phoenix_Islands
    {  660, 1502,  922574,  922601,  922637, 27, 36, 32, }, // Pierre_Miquelon
    {  660, 1518,       0,  922669,       0,  0, 21,  0, }, // Pitcairn
    {  660, 1527,       0,  922690,       0,  0, 11,  0, }, // Ponape
    {  660, 1534,       0,  922701,       0,  0, 14,  0, }, // Pyongyang
    {  660, 1554,       0,  922715,       0,  0, 12,  0, }, // Reunion
    {  660, 1562,       0,  922727,       0,  0, 12,  0, }, // Rothera
    {  660, 1570,  922739,  922752,  922774, 13, 22, 18, }, // Sakhalin
    {  660, 1596,  922792,  922802,  922821, 10, 19, 15, }, // Samoa
    {  660, 1602,       0,  922836,       0,  0, 14,  0, }, // Seychelles
    {  660, 1624,       0,  922850,       0,  0, 14,  0, }, // Singapore
    {  660, 1634,       0,  922864,       0,  0, 15,  0, }, // Solomon
    {  660, 1642,       0,  922879,       0,  0, 17,  0, }, // South_Georgia
    {  660, 1656,       0,  922896,       0,  0, 13,  0, }, // Suriname
    {  660, 1676,       0,  922909,       0,  0, 10,  0, }, // Syowa
    {  660, 1682,       0,  922919,       0,  0, 11,  0, }, // Tahiti
    {  660, 1689,  922930,  922941,  922961, 11, 20, 16, }, // Taipei
    {  660, 1696,       0,  922977,       0,  0, 17,  0, }, // Tajikistan
    {  660, 1724,       0,  922994,       0,  0, 12,  0, }, // Tokelau
    {  660, 1732,  923006,  923016,  923035, 10, 19, 15, }, // Tonga
    {  660, 1738,       0,  923050,       0,  0, 10,  0, }, // Truk
    {  660, 1750,  923060,  923079,  923107, 19, 28, 24, }, // Turkmenistan
    {  660, 1763,       0,  923131,       0,  0, 12,  0, }, // Tuvalu
    {  660, 1777,  923143,  923155,  923176, 12, 21, 17, }, // Uruguay
    {  660, 1792,  923193,  923209,  923234, 16, 25, 21, }, // Uzbekistan
    {  660, 1803,  923255,  923267,  923288, 12, 21, 17, }, // Vanuatu
    {  660, 1811,       0,  923305,       0,  0, 15,  0, }, // Venezuela
    {  660, 1821,  923320,  923336,  923361, 16, 25, 21, }, // Vladivostok
    {  660, 1833,  923382,  923396,  923419, 14, 23, 19, }, // Volgograd
    {  660, 1843,       0,  923438,       0,  0, 11,  0, }, // Vostok
    {  660, 1850,       0,  923449,       0,  0, 15,  0, }, // Wake
    {  660, 1855,       0,  923464,       0,  0, 21,  0, }, // Wallis
    {  660, 1862,  923485,  923497,  923518, 12, 21, 17, }, // Yakutsk
    {  660, 1870,  923535,  923553,  923580, 18, 27, 23, }, // Yekaterinburg
    {  660, 1892,       0,  923603,       0,  0, 10,  0, }, // Yukon
    {  661,  788,       0,  923613,       0,  0, 18,  0, }, // GMT Rajasthani/Devanagari/India
    {  663,  788,       0,  923631,       0,  0, 16,  0, }, // GMT Toki Pona/Latin/world
    {  664,  788,       0,  923647,       0,  0, 15,  0, }, // GMT Pijin/Latin/Solomon Islands
    {  666,  107,  923662,  923676,  923699, 14, 23, 21, }, // Alaska Baluchi/Arabic/Pakistan
    {  666,  135,  923720,  923733,  923755, 13, 22, 20, }, // Amazon
    {  666,  142,  923775,  923794,  923822, 19, 28, 26, }, // America_Central
    {  666,  158,  923848,  923870,  923901, 22, 31, 29, }, // America_Eastern
    {  666,  174,  923930,  923952,  923983, 22, 31, 29, }, // America_Mountain
    {  666,  191,  924012,  924034,  924065, 22, 31, 29, }, // America_Pacific
    {  666,  240,  924094,  924109,  924133, 15, 24, 22, }, // Argentina
    {  666,  250,  924155,  924177,  924208, 22, 31, 29, }, // Argentina_Western
    {  666,  295,  924237,  924259,  924290, 22, 31, 29, }, // Australia_Central
    {  666,  313,  924319,  924347,  924384, 28, 37, 35, }, // Australia_CentralWestern
    {  666,  338,  924419,  924444,  924478, 25, 34, 32, }, // Australia_Eastern
    {  666,  356,  924324,  924352,  924389, 23, 32, 30, }, // Australia_Western
    {  666,  437,  924510,  924523,  924545, 13, 22, 20, }, // Brasilia
    {  666,  627,  924565,  924582,  924608, 17, 26, 24, }, // Europe_Central
    {  666,  642,  924632,  924652,  924681, 20, 29, 27, }, // Europe_Eastern
    {  666,  657,       0,  924708,       0,  0, 36,  0, }, // Europe_Further_Eastern
    {  666,  680,  924744,  924762,  924789, 18, 27, 25, }, // Europe_Western
    {  666,  788,       0,  924814,       0,  0, 17,  0, }, // GMT
    {  666,  883,  924831,  924844,  924866, 13, 22, 20, }, // Hawaii_Aleutian
    {  666,  943,       0,  924886,       0,  0, 31,  0, }, // Indonesia_Central
    {  666,  961,       0,  924917,       0,  0, 34,  0, }, // Indonesia_Eastern
    {  666,  979,       0,  924951,       0,  0, 32,  0, }, // Indonesia_Western
    {  666, 1008,  924983,  924998,  925022, 15, 24, 22, }, // Irkutsk
    {  666, 1058,       0,  925044,       0,  0, 33,  0, }, // Kazakhstan_Eastern
    {  666, 1077,       0,  925077,       0,  0, 31,  0, }, // Kazakhstan_Western
    {  666, 1119,  925108,  925127,  925155, 19, 28, 26, }, // Krasnoyarsk
    {  666, 1188,  925181,  925206,  925240, 25, 34, 32, }, // Lord_Howe
    {  666, 1204,  925272,  925286,  925309, 14, 23, 21, }, // Magadan
    {  666, 1281,  925330,  925353,  925385, 23, 32, 30, }, // Mexico_Pacific
    {  666, 1305,  925415,  925427,  925448, 12, 21, 19, }, // Moscow
    {  666, 1358,  925467,  925488,  925518, 21, 30, 28, }, // Newfoundland
    {  666, 1384,  925546,  925560,  925583, 14, 23, 21, }, // Noronha
    {  666, 1406,  925604,  925622,  925649, 18, 27, 25, }, // Novosibirsk
    {  666, 1418,  925674,  925686,  925707, 12, 21, 19, }, // Omsk
    {  666, 1821,  925726,  925744,  925771, 18, 27, 25, }, // Vladivostok
    {  666, 1862,  925796,  925810,  925833, 14, 23, 21, }, // Yakutsk
    {  666, 1870,  925854,  925873,  925901, 19, 28, 26, }, // Yekaterinburg
    {  667,    5,       0,  925927,       0,  0, 17,  0, }, // Afghanistan Baluchi/Latin/Pakistan
    {  667,   17,       0,  925944,       0,  0, 21,  0, }, // Africa_Central
    {  667,   32,       0,  925965,       0,  0, 23,  0, }, // Africa_Eastern
    {  667,   65,       0,  925988,       0,  0, 30,  0, }, // Africa_Southern
    {  667,   81,  926018,  926039,  926067, 21, 28, 29, }, // Africa_Western
    {  667,  107,  926096,  926109,  926129, 13, 20, 21, }, // Alaska
    {  667,  135,  926150,  926163,  926183, 13, 20, 21, }, // Amazon
    {  667,  142,  926204,  926225,  926253, 21, 28, 29, }, // America_Central
    {  667,  158,  926282,  926305,  926335, 23, 30, 31, }, // America_Eastern
    {  667,  174,  926366,  926389,  926419, 23, 30, 31, }, // America_Mountain
    {  667,  191,  926450,  926472,  926501, 22, 29, 30, }, // America_Pacific
    {  667,  214,  926531,  926542,  926560, 11, 18, 17, }, // Apia
    {  667,  232,  926577,  926587,  926604, 10, 17, 16, }, // Arabian
    {  667,  240,  926620,  926636,  926659, 16, 23, 24, }, // Argentina
    {  667,  250,  926683,  926707,  926738, 24, 31, 32, }, // Argentina_Western
    {  667,  268,  926770,  926784,  926805, 14, 21, 22, }, // Armenia
    {  667,  286,  926827,  926841,  926862, 14, 21, 20, }, // Atlantic
    {  667,  295,  926882,  926905,  926935, 23, 30, 31, }, // Australia_Central
    {  667,  313,  926966,  926996,  927033, 30, 37, 38, }, // Australia_CentralWestern
    {  667,  338,  927071,  927096,  927128, 25, 32, 33, }, // Australia_Eastern
    {  667,  356,  927161,  927184,  927214, 23, 30, 31, }, // Australia_Western
    {  667,  374,  927245,  927262,  927286, 17, 24, 25, }, // Azerbaijan
    {  667,  385,  927311,  927324,  927344, 13, 20, 21, }, // Azores
    {  667,  397,  927365,  927383,  927408, 18, 25, 26, }, // Bangladesh
    {  667,  415,       0,  927434,       0,  0, 13,  0, }, // Bhutan
    {  667,  422,       0,  927447,       0,  0, 14,  0, }, // Bolivia
    {  667,  437,  927461,  927474,  927494, 13, 20, 21, }, // Brasilia
    {  667,  454,       0,  927515,       0,  0, 13,  0, }, // Brunei
    {  667,  461,  927528,  927544,  927567, 16, 23, 24, }, // Cape_Verde
    {  667,  478,       0,  927591,       0,  0, 15,  0, }, // Chamorro
    {  667,  487,  927606,  927617,  927635, 11, 18, 17, }, // Chatham
    {  667,  495,  927652,  927665,  927685, 13, 20, 21, }, // Chile
    {  667,  501,  927706,  927717,  927735, 11, 18, 17, }, // China
    {  667,  507,       0,  927752,       0,  0, 21,  0, }, // Christmas
    {  667,  517,       0,  927773,       0,  0, 19,  0, }, // Cocos
    {  667,  523,  927792,  927807,  927829, 15, 22, 23, }, // Colombia
    {  667,  532,  927852,  927869,  927893, 17, 24, 29, }, // Cook
    {  667,  537,  927922,  927934,  927953, 12, 19, 19, }, // Cuba
    {  667,  548,       0,  927972,       0,  0, 12,  0, }, // Davis
    {  667,  564,       0,  927984,       0,  0, 19,  0, }, // DumontDUrville
    {  667,  601,       0,  928003,       0,  0, 22,  0, }, // East_Timor
    {  667,  612,  928025,  928045,  928072, 20, 27, 28, }, // Easter
    {  667,  619,       0,  928100,       0,  0, 14,  0, }, // Ecuador
    {  667,  627,  928114,  928134,  928161, 20, 27, 28, }, // Europe_Central
    {  667,  642,  928189,  928211,  928240, 22, 29, 30, }, // Europe_Eastern
    {  667,  657,       0,  928270,       0,  0, 38,  0, }, // Europe_Further_Eastern
    {  667,  680,  928308,  928328,  928355, 20, 27, 28, }, // Europe_Western
    {  667,  695,  928383,  928405,  928434, 22, 29, 30, }, // Falkland
    {  667,  704,  928464,  928475,  928493, 11, 18, 19, }, // Fiji
    {  667,  709,       0,  928512,       0,  0, 20,  0, }, // French_Guiana
    {  667,  723,       0,  928532,       0,  0, 35,  0, }, // French_Southern
    {  667,  746,       0,  928567,       0,  0, 16,  0, }, // Galapagos
    {  667,  756,       0,  928583,       0,  0, 13,  0, }, // Gambier
    {  667,  764,  928596,  928609,  928629, 13, 20, 21, }, // Georgia
    {  667,  772,       0,  928650,       0,  0, 21,  0, }, // Gilbert_Islands
    {  667,  788,       0,  928671,       0,  0, 17,  0, }, // GMT
    {  667,  830,  928688,  928713,  928745, 25, 32, 33, }, // Greenland_Eastern
    {  667,  848,  928778,  928801,  928831, 23, 30, 29, }, // Greenland_Western
    {  667,  871,       0,  928860,       0,  0, 20,  0, }, // Gulf
    {  667,  876,       0,  928880,       0,  0, 13,  0, }, // Guyana
    {  667,  883,  928893,  928912,  928938, 19, 26, 27, }, // Hawaii_Aleutian
    {  667,  899,  928965,  928981,  929004, 16, 23, 24, }, // Hong_Kong
    {  667,  909,  929028,  929039,  929057, 11, 18, 19, }, // Hovd
    {  667,  914,       0,  929076,       0,  0, 18,  0, }, // India
    {  667,  920,       0,  929094,       0,  0, 16,  0, }, // Indian_Ocean
    {  667,  933,       0,  929110,       0,  0, 16,  0, }, // Indochina
    {  667,  943,       0,  929126,       0,  0, 33,  0, }, // Indonesia_Central
    {  667,  961,       0,  929159,       0,  0, 35,  0, }, // Indonesia_Eastern
    {  667,  979,       0,  929194,       0,  0, 33,  0, }, // Indonesia_Western
    {  667,  997,  929227,  929238,  929256, 11, 18, 17, }, // Iran
    {  667, 1008,  929273,  929286,  929306, 13, 20, 21, }, // Irkutsk
    {  667, 1016,  929327,  929340,  929360, 13, 20, 19, }, // Israel
    {  667, 1023,  929379,  929391,  929410, 12, 19, 18, }, // Japan
    {  667, 1047,       0,  929428,       0,  0, 17,  0, }, // Kazakhstan
    {  667, 1058,       0,  929445,       0,  0, 34,  0, }, // Kazakhstan_Eastern
    {  667, 1077,       0,  929479,       0,  0, 32,  0, }, // Kazakhstan_Western
    {  667, 1106,  929511,  929523,  929542, 12, 19, 18, }, // Korea
    {  667, 1112,       0,  929560,       0,  0, 13,  0, }, // Kosrae
    {  667, 1119,  929573,  929590,  929614, 17, 24, 25, }, // Krasnoyarsk
    {  667, 1151,       0,  929639,       0,  0, 18,  0, }, // Kyrgystan
    {  667, 1175,       0,  929657,       0,  0, 18,  0, }, // Line_Islands
    {  667, 1188,  929675,  929700,  929732, 25, 32, 33, }, // Lord_Howe
    {  667, 1204,  929765,  929779,  929800, 14, 21, 22, }, // Magadan
    {  667, 1219,       0,  929822,       0,  0, 15,  0, }, // Malaysia
    {  667, 1228,       0,  929837,       0,  0, 13,  0, }, // Maldives
    {  667, 1237,       0,  929850,       0,  0, 15,  0, }, // Marquesas
    {  667, 1247,       0,  929865,       0,  0, 21,  0, }, // Marshall_Islands
    {  667, 1264,  929886,  929902,  929925, 16, 23, 24, }, // Mauritius
    {  667, 1274,       0,  929949,       0,  0, 13,  0, }, // Mawson
    {  667, 1281,  929962,  929985,  930015, 23, 30, 31, }, // Mexico_Pacific
    {  667, 1296,  930046,  930062,  930085, 16, 23, 24, }, // Mongolia
    {  667, 1305,  930109,  930121,  930140, 12, 19, 20, }, // Moscow
    {  667, 1312,       0,  930160,       0,  0, 14,  0, }, // Myanmar
    {  667, 1320,       0,  930174,       0,  0, 12,  0, }, // Nauru
    {  667, 1326,       0,  930186,       0,  0, 12,  0, }, // Nepal
    {  667, 1332,  930198,  930218,  930245, 20, 27, 28, }, // New_Caledonia
    {  667, 1346,  930273,  930290,  930314, 17, 24, 23, }, // New_Zealand
    {  667, 1358,  930337,  930355,  930380, 18, 25, 26, }, // Newfoundland
    {  667, 1371,       0,  930406,       0,  0, 10,  0, }, // Niue
    {  667, 1376,  930416,  930437,  930465, 21, 28, 27, }, // Norfolk
    {  667, 1384,  930492,  930505,  930525, 13, 20, 21, }, // Noronha
    {  667, 1406,  930546,  930563,  930587, 17, 24, 25, }, // Novosibirsk
    {  667, 1418,  930612,  930623,  930641, 11, 18, 19, }, // Omsk
    {  667, 1428,  930660,  930675,  930697, 15, 22, 23, }, // Pakistan
    {  667, 1437,       0,  930720,       0,  0, 12,  0, }, // Palau
    {  667, 1443,       0,  930732,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  667, 1460,  930754,  930769,  930791, 15, 22, 23, }, // Paraguay
    {  667, 1469,  930814,  930825,  930843, 11, 18, 19, }, // Peru
    {  667, 1474,  930862,  930875,  930895, 13, 20, 21, }, // Philippines
    {  667, 1486,       0,  930916,       0,  0, 22,  0, }, // Phoenix_Islands
    {  667, 1502,  930938,  930962,  930992, 24, 30, 30, }, // Pierre_Miquelon
    {  667, 1518,       0,  931022,       0,  0, 15,  0, }, // Pitcairn
    {  667, 1527,       0,  931037,       0,  0, 12,  0, }, // Ponape
    {  667, 1534,       0,  931049,       0,  0, 16,  0, }, // Pyongyang
    {  667, 1554,       0,  931065,       0,  0, 15,  0, }, // Reunion
    {  667, 1562,       0,  931080,       0,  0, 14,  0, }, // Rothera
    {  667, 1570,  931094,  931109,  931131, 15, 22, 23, }, // Sakhalin
    {  667, 1596,  931154,  931167,  931187, 13, 20, 19, }, // Samoa
    {  667, 1602,       0,  931206,       0,  0, 15,  0, }, // Seychelles
    {  667, 1624,       0,  931221,       0,  0, 22,  0, }, // Singapore
    {  667, 1634,       0,  931243,       0,  0, 21,  0, }, // Solomon
    {  667, 1642,       0,  931264,       0,  0, 21,  0, }, // South_Georgia
    {  667, 1656,       0,  931285,       0,  0, 15,  0, }, // Suriname
    {  667, 1676,       0,  931300,       0,  0, 12,  0, }, // Syowa
    {  667, 1682,       0,  931312,       0,  0, 13,  0, }, // Tahiti
    {  667, 1689,  931325,  931337,  931356, 12, 19, 18, }, // Taipei
    {  667, 1696,       0,  931374,       0,  0, 17,  0, }, // Tajikistan
    {  667, 1724,       0,  931391,       0,  0, 14,  0, }, // Tokelau
    {  667, 1732,  931405,  931417,  931436, 12, 19, 20, }, // Tonga
    {  667, 1738,       0,  931456,       0,  0, 11,  0, }, // Truk
    {  667, 1750,  931467,  931486,  931512, 19, 26, 27, }, // Turkmenistan
    {  667, 1763,       0,  931539,       0,  0, 13,  0, }, // Tuvalu
    {  667, 1777,  931552,  931567,  931589, 15, 22, 23, }, // Uruguay
    {  667, 1792,  931612,  931629,  931653, 17, 24, 25, }, // Uzbekistan
    {  667, 1803,  931678,  931692,  931713, 14, 21, 22, }, // Vanuatu
    {  667, 1811,       0,  931735,       0,  0, 16,  0, }, // Venezuela
    {  667, 1821,  931751,  931770,  931796, 19, 26, 27, }, // Vladivostok
    {  667, 1833,  931823,  931839,  931862, 16, 23, 24, }, // Volgograd
    {  667, 1843,       0,  931886,       0,  0, 13,  0, }, // Vostok
    {  667, 1850,       0,  931899,       0,  0, 18,  0, }, // Wake
    {  667, 1855,       0,  931917,       0,  0, 22,  0, }, // Wallis
    {  667, 1862,  931939,  931952,  931972, 13, 20, 21, }, // Yakutsk
    {  667, 1870,  931993,  932012,  932038, 19, 26, 27, }, // Yekaterinburg
    {  667, 1892,       0,  932065,       0,  0, 10,  0, }, // Yukon
    {  668,    5,       0,  932075,       0,  0, 19,  0, }, // Afghanistan Ligurian/Latin/Italy
    {  668,   17,       0,  932094,       0,  0, 22,  0, }, // Africa_Central
    {  668,   32,       0,  932116,       0,  0, 25,  0, }, // Africa_Eastern
    {  668,   65,       0,  932141,       0,  0, 26,  0, }, // Africa_Southern
    {  668,   81,  932167,  932192,  932226, 25, 34, 32, }, // Africa_Western
    {  668,  107,  932258,  932272,  932295, 14, 23, 21, }, // Alaska
    {  668,  135,  932316,  932333,  932359, 17, 26, 24, }, // Amazon
    {  668,  142,  932383,  932407,  932440, 24, 33, 31, }, // America_Central
    {  668,  158,  932471,  932498,  932534, 27, 36, 34, }, // America_Eastern
    {  668,  174,  932568,  932591,  932623, 23, 32, 30, }, // America_Mountain
    {  668,  191,  932653,  932681,  932718, 28, 37, 35, }, // America_Pacific
    {  668,  214,  932753,  932763,  932782, 10, 19, 17, }, // Apia
    {  668,  232,  932799,  932813,  932836, 14, 23, 21, }, // Arabian
    {  668,  240,  932857,  932874,  932900, 17, 26, 24, }, // Argentina
    {  668,  250,  932924,  932952,  932989, 28, 37, 35, }, // Argentina_Western
    {  668,  268,  933024,  933039,  933063, 15, 24, 22, }, // Armenia
    {  668,  286,  933085,  933116,  933156, 31, 40, 38, }, // Atlantic
    {  668,  295,  933194,  933219,  933253, 25, 34, 32, }, // Australia_Central
    {  668,  313,  933285,  933318,  933360, 33, 42, 40, }, // Australia_CentralWestern
    {  668,  338,  933400,  933428,  933465, 28, 37, 35, }, // Australia_Eastern
    {  668,  356,  933500,  933528,  933565, 28, 37, 35, }, // Australia_Western
    {  668,  374,  933600,  933619,  933647, 19, 28, 26, }, // Azerbaijan
    {  668,  385,  933673,  933684,  933704, 11, 20, 18, }, // Azores
    {  668,  397,  933722,  933738,  933763, 16, 25, 23, }, // Bangladesh
    {  668,  415,       0,  933786,       0,  0, 12,  0, }, // Bhutan
    {  668,  422,       0,  933798,       0,  0, 13,  0, }, // Bolivia
    {  668,  437,  933811,  933825,  933848, 14, 23, 21, }, // Brasilia
    {  668,  454,       0,  933869,       0,  0, 23,  0, }, // Brunei
    {  668,  461,  933892,  933909,  933935, 17, 26, 24, }, // Cape_Verde
    {  668,  478,       0,  933959,       0,  0, 14,  0, }, // Chamorro
    {  668,  487,  933973,  933986,  934008, 13, 22, 20, }, // Chatham
    {  668,  495,  934028,  934038,  934057, 10, 19, 17, }, // Chile
    {  668,  501,  934074,  934084,  934103, 10, 19, 17, }, // China
    {  668,  507,       0,  934120,       0,  0, 22,  0, }, // Christmas
    {  668,  517,       0,  934142,       0,  0, 16,  0, }, // Cocos
    {  668,  523,  934158,  934172,  934195, 14, 23, 21, }, // Colombia
    {  668,  532,  934216,  934231,  934255, 15, 24, 22, }, // Cook
    {  668,  537,  934277,  934288,  934308, 11, 20, 18, }, // Cuba
    {  668,  548,       0,  934326,       0,  0, 11,  0, }, // Davis
    {  668,  564,       0,  934337,       0,  0, 22,  0, }, // DumontDUrville
    {  668,  601,       0,  934359,       0,  0, 15,  0, }, // East_Timor
    {  668,  612,  934374,  934396,  934427, 22, 31, 29, }, // Easter
    {  668,  619,       0,  934456,       0,  0, 15,  0, }, // Ecuador
    {  668,  627,  934471,  934493,  934524, 22, 31, 29, }, // Europe_Central
    {  668,  642,  934553,  934578,  934612, 25, 34, 32, }, // Europe_Eastern
    {  668,  657,       0,  934644,       0,  0, 17,  0, }, // Europe_Further_Eastern
    {  668,  680,  934661,  934686,  934720, 25, 34, 32, }, // Europe_Western
    {  668,  695,  934752,  934770,  934797, 18, 27, 25, }, // Falkland
    {  668,  704,  934822,  934832,  934851, 10, 19, 17, }, // Fiji
    {  668,  709,       0,  934868,       0,  0, 22,  0, }, // French_Guiana
    {  668,  723,       0,  934890,       0,  0, 42,  0, }, // French_Southern
    {  668,  746,       0,  934932,       0,  0, 15,  0, }, // Galapagos
    {  668,  756,       0,  934947,       0,  0, 13,  0, }, // Gambier
    {  668,  764,  934960,  934973,  934995, 13, 22, 20, }, // Georgia
    {  668,  772,       0,  935015,       0,  0, 18,  0, }, // Gilbert_Islands
    {  668,  788,       0,  935033,       0,  0, 27,  0, }, // GMT
    {  668,  830,  935060,  935088,  935125, 28, 37, 35, }, // Greenland_Eastern
    {  668,  848,  935160,  935188,  935225, 28, 37, 35, }, // Greenland_Western
    {  668,  871,       0,  935260,       0,  0, 20,  0, }, // Gulf
    {  668,  876,       0,  934868,       0,  0, 12,  0, }, // Guyana
    {  668,  883,  935280,  935301,  935331, 21, 30, 28, }, // Hawaii_Aleutian
    {  668,  899,  935359,  935374,  935398, 15, 24, 22, }, // Hong_Kong
    {  668,  909,  935420,  935430,  935449, 10, 19, 17, }, // Hovd
    {  668,  914,       0,  935466,       0,  0, 13,  0, }, // India
    {  668,  920,       0,  935479,       0,  0, 21,  0, }, // Indian_Ocean
    {  668,  933,       0,  935500,       0,  0, 16,  0, }, // Indochina
    {  668,  943,       0,  935516,       0,  0, 25,  0, }, // Indonesia_Central
    {  668,  961,       0,  935541,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  668,  979,       0,  935569,       0,  0, 28,  0, }, // Indonesia_Western
    {  668,  997,  935597,  935609,  935630, 12, 21, 19, }, // Iran
    {  668, 1008,  935649,  935662,  935684, 13, 22, 20, }, // Irkutsk
    {  668, 1016,  935704,  935716,  935737, 12, 21, 19, }, // Israel
    {  668, 1023,  935756,  935769,  935791, 13, 22, 20, }, // Japan
    {  668, 1058,       0,  935811,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  668, 1077,       0,  935838,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  668, 1106,  935865,  935876,  935896, 11, 20, 18, }, // Korea
    {  668, 1112,       0,  935914,       0,  0, 12,  0, }, // Kosrae
    {  668, 1119,  935926,  935943,  935969, 17, 26, 24, }, // Krasnoyarsk
    {  668, 1151,       0,  935993,       0,  0, 18,  0, }, // Kyrgystan
    {  668, 1175,       0,  936011,       0,  0, 19,  0, }, // Line_Islands
    {  668, 1188,  936030,  936045,  936069, 15, 24, 22, }, // Lord_Howe
    {  668, 1204,  936091,  936104,  936126, 13, 22, 20, }, // Magadan
    {  668, 1219,       0,  936146,       0,  0, 13,  0, }, // Malaysia
    {  668, 1228,       0,  936159,       0,  0, 13,  0, }, // Maldives
    {  668, 1237,       0,  936172,       0,  0, 15,  0, }, // Marquesas
    {  668, 1247,       0,  936187,       0,  0, 19,  0, }, // Marshall_Islands
    {  668, 1264,  936206,  936221,  936245, 15, 24, 22, }, // Mauritius
    {  668, 1274,       0,  936267,       0,  0, 12,  0, }, // Mawson
    {  668, 1281,  936279,  936303,  936336, 24, 33, 31, }, // Mexico_Pacific
    {  668, 1296,  936367,  936382,  936406, 15, 24, 22, }, // Mongolia
    {  668, 1305,  936428,  936439,  936459, 11, 20, 18, }, // Moscow
    {  668, 1312,       0,  936477,       0,  0, 14,  0, }, // Myanmar
    {  668, 1320,       0,  936491,       0,  0, 11,  0, }, // Nauru
    {  668, 1326,       0,  936502,       0,  0, 11,  0, }, // Nepal
    {  668, 1332,  936513,  936534,  936564, 21, 30, 28, }, // New_Caledonia
    {  668, 1346,  936592,  936611,  936639, 19, 28, 26, }, // New_Zealand
    {  668, 1358,  936665,  936680,  936704, 15, 24, 22, }, // Newfoundland
    {  668, 1371,       0,  936726,       0,  0, 10,  0, }, // Niue
    {  668, 1376,  936736,  936756,  936785, 20, 29, 27, }, // Norfolk
    {  668, 1384,  936812,  936837,  936871, 25, 34, 32, }, // Noronha
    {  668, 1406,  936903,  936920,  936946, 17, 26, 24, }, // Novosibirsk
    {  668, 1418,  936970,  936980,  936999, 10, 19, 17, }, // Omsk
    {  668, 1428,  937016,  937030,  937053, 14, 23, 21, }, // Pakistan
    {  668, 1437,       0,  937074,       0,  0, 11,  0, }, // Palau
    {  668, 1443,       0,  937085,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  668, 1460,  937109,  937123,  937146, 14, 23, 21, }, // Paraguay
    {  668, 1469,  937167,  937177,  937196, 10, 19, 17, }, // Peru
    {  668, 1474,  937213,  937227,  937250, 14, 23, 21, }, // Philippines
    {  668, 1486,       0,  937271,       0,  0, 18,  0, }, // Phoenix_Islands
    {  668, 1502,  937289,  937312,  937344, 23, 32, 30, }, // Pierre_Miquelon
    {  668, 1518,       0,  937374,       0,  0, 14,  0, }, // Pitcairn
    {  668, 1527,       0,  937388,       0,  0, 13,  0, }, // Ponape
    {  668, 1534,       0,  937401,       0,  0, 15,  0, }, // Pyongyang
    {  668, 1554,       0,  937416,       0,  0, 13,  0, }, // Reunion
    {  668, 1562,       0,  937429,       0,  0, 13,  0, }, // Rothera
    {  668, 1570,  937442,  937456,  937479, 14, 23, 21, }, // Sakhalin
    {  668, 1596,  937500,  937511,  937531, 11, 20, 18, }, // Samoa
    {  668, 1602,       0,  937549,       0,  0, 16,  0, }, // Seychelles
    {  668, 1624,       0,  937565,       0,  0, 14,  0, }, // Singapore
    {  668, 1634,       0,  937579,       0,  0, 18,  0, }, // Solomon
    {  668, 1642,       0,  937597,       0,  0, 20,  0, }, // South_Georgia
    {  668, 1656,       0,  937617,       0,  0, 14,  0, }, // Suriname
    {  668, 1676,       0,  937631,       0,  0, 11,  0, }, // Syowa
    {  668, 1682,       0,  937642,       0,  0, 12,  0, }, // Tahiti
    {  668, 1689,  937654,  937666,  937687, 12, 21, 19, }, // Taipei
    {  668, 1696,       0,  937706,       0,  0, 16,  0, }, // Tajikistan
    {  668, 1724,       0,  937722,       0,  0, 13,  0, }, // Tokelau
    {  668, 1732,  937735,  937746,  937766, 11, 20, 18, }, // Tonga
    {  668, 1738,       0,  937784,       0,  0, 11,  0, }, // Truk
    {  668, 1750,  937795,  937813,  937840, 18, 27, 25, }, // Turkmenistan
    {  668, 1763,       0,  937865,       0,  0, 12,  0, }, // Tuvalu
    {  668, 1777,  937877,  937892,  937916, 15, 24, 22, }, // Uruguay
    {  668, 1792,  937938,  937956,  937983, 18, 27, 25, }, // Uzbekistan
    {  668, 1803,  938008,  938021,  938043, 13, 22, 20, }, // Vanuatu
    {  668, 1811,       0,  938063,       0,  0, 16,  0, }, // Venezuela
    {  668, 1821,  938079,  938096,  938122, 17, 26, 24, }, // Vladivostok
    {  668, 1833,  938146,  938161,  938185, 15, 24, 22, }, // Volgograd
    {  668, 1843,       0,  938207,       0,  0, 12,  0, }, // Vostok
    {  668, 1850,       0,  938219,       0,  0, 20,  0, }, // Wake
    {  668, 1855,       0,  938239,       0,  0, 21,  0, }, // Wallis
    {  668, 1862,  938260,  938273,  938295, 13, 22, 20, }, // Yakutsk
    {  668, 1870,  938315,  938332,  938358, 17, 26, 24, }, // Yekaterinburg
    {  668, 1892,       0,  938382,       0,  0, 11,  0, }, // Yukon
    {  671,    5,       0,  938393,       0,  0, 16,  0, }, // Afghanistan Torwali/Arabic/Pakistan
    {  671,   17,       0,  834573,       0,  0, 16,  0, }, // Africa_Central
    {  671,   32,       0,  834589,       0,  0, 17,  0, }, // Africa_Eastern
    {  671,   65,       0,  834606,       0,  0, 25,  0, }, // Africa_Southern
    {  671,   81,  834631,  834648,  834673, 17, 25, 21, }, // Africa_Western
    {  671,  107,  834694,  834705,  834725, 11, 20, 19, }, // Alaska
    {  671,  135,  834744,  938409,  938430, 11, 21, 26, }, // Amazon
    {  671,  142,  834802,  834812,  834831, 10, 19, 18, }, // America_Central
    {  671,  158,  834849,  834860,  834880, 11, 20, 19, }, // America_Eastern
    {  671,  174,  834899,  834911,  834932, 12, 21, 20, }, // America_Mountain
    {  671,  191,  834952,  834962,  834981, 10, 19, 18, }, // America_Pacific
    {  671,  214,  835042,  835052,  835070, 10, 18, 18, }, // Apia
    {  671,  232,  938456,  938466,  835115, 10, 17, 16, }, // Arabian
    {  671,  240,  938483,  938498,  938520, 15, 22, 28, }, // Argentina
    {  671,  250,  938548,  938569,  938597, 21, 28, 34, }, // Argentina_Western
    {  671,  268,  938631,  938645,  938666, 14, 21, 27, }, // Armenia
    {  671,  286,  835341,  835353,  835374, 12, 21, 20, }, // Atlantic
    {  671,  295,  835394,  835413,  835441, 19, 28, 27, }, // Australia_Central
    {  671,  313,  835468,  835494,  835529, 26, 35, 34, }, // Australia_CentralWestern
    {  671,  338,  835563,  835583,  835612, 20, 29, 28, }, // Australia_Eastern
    {  671,  356,  835640,  835661,  835689, 20, 28, 28, }, // Australia_Western
    {  671,  374,  938693,  938710,  938734, 17, 24, 30, }, // Azerbaijan
    {  671,  385,  938764,  938777,  938797, 13, 20, 26, }, // Azores
    {  671,  397,  938823,  938839,  938862, 16, 23, 29, }, // Bangladesh
    {  671,  415,       0,  938891,       0,  0, 13,  0, }, // Bhutan
    {  671,  422,       0,  938904,       0,  0, 14,  0, }, // Bolivia
    {  671,  437,  835942,  835955,  835977, 13, 22, 17, }, // Brasilia
    {  671,  454,       0,  835994,       0,  0, 21,  0, }, // Brunei
    {  671,  461,  836015,  836028,  836049, 13, 21, 17, }, // Cape_Verde
    {  671,  478,       0,  836066,       0,  0, 19,  0, }, // Chamorro
    {  671,  487,  836085,  836095,  836114, 10, 19, 18, }, // Chatham
    {  671,  495,  938918,  938928,  938945, 10, 17, 23, }, // Chile
    {  671,  501,  938968,  836192,  836208, 10, 16, 17, }, // China
    {  671,  507,       0,  836225,       0,  0, 17,  0, }, // Christmas
    {  671,  517,       0,  836242,       0,  0, 18,  0, }, // Cocos
    {  671,  523,  836260,  938978,  938999, 12, 21, 27, }, // Colombia
    {  671,  532,  836320,  836335,  836358, 15, 23, 23, }, // Cook
    {  671,  537,  836381,  836391,  836410, 10, 19, 18, }, // Cuba
    {  671,  548,       0,  836428,       0,  0,  9,  0, }, // Davis
    {  671,  564,       0,  836437,       0,  0, 21,  0, }, // DumontDUrville
    {  671,  601,       0,  836458,       0,  0, 16,  0, }, // East_Timor
    {  671,  612,  939026,  939045,  939071, 19, 26, 32, }, // Easter
    {  671,  619,       0,  939103,       0,  0, 15,  0, }, // Ecuador
    {  671,  627,  939118,  939133,  939156, 15, 23, 29, }, // Europe_Central
    {  671,  642,  939185,  939202,  939226, 17, 24, 30, }, // Europe_Eastern
    {  671,  657,       0,  939256,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  671,  680,  939276,  939293,  939317, 17, 24, 30, }, // Europe_Western
    {  671,  695,  939347,  939370,  939400, 23, 30, 36, }, // Falkland
    {  671,  704,  836884,  836892,  836908,  8, 16, 12, }, // Fiji
    {  671,  709,       0,  939436,       0,  0, 18,  0, }, // French_Guiana
    {  671,  723,       0,  939454,       0,  0, 28,  0, }, // French_Southern
    {  671,  746,       0,  939482,       0,  0, 16,  0, }, // Galapagos
    {  671,  756,       0,  836983,       0,  0, 12,  0, }, // Gambier
    {  671,  764,  939498,  939511,  939531, 13, 20, 26, }, // Georgia
    {  671,  772,       0,  837054,       0,  0, 18,  0, }, // Gilbert_Islands
    {  671,  788,       0,  939557,       0,  0, 18,  0, }, // GMT
    {  671,  830,  837090,  837110,  939575, 20, 29, 35, }, // Greenland_Eastern
    {  671,  848,  837174,  837194,  939610, 20, 29, 35, }, // Greenland_Western
    {  671,  871,       0,  939645,       0,  0, 18,  0, }, // Gulf
    {  671,  876,       0,  939442,       0,  0, 12,  0, }, // Guyana
    {  671,  883,  837276,  837295,  837323, 19, 28, 27, }, // Hawaii_Aleutian
    {  671,  899,  939663,  939677,  939699, 14, 22, 18, }, // Hong_Kong
    {  671,  909,  837368,  837377,  837394,  9, 17, 13, }, // Hovd
    {  671,  914,       0,  939717,       0,  0, 22,  0, }, // India
    {  671,  920,       0,  837429,       0,  0, 12,  0, }, // Indian_Ocean
    {  671,  933,       0,  837441,       0,  0, 12,  0, }, // Indochina
    {  671,  943,       0,  837453,       0,  0, 19,  0, }, // Indonesia_Central
    {  671,  961,       0,  837472,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  671,  979,       0,  837492,       0,  0, 20,  0, }, // Indonesia_Western
    {  671,  997,  939739,  939751,  837543, 12, 19, 18, }, // Iran
    {  671, 1008,  837561,  837572,  837591, 11, 19, 15, }, // Irkutsk
    {  671, 1016,  939770,  939784,  837641, 14, 21, 20, }, // Israel
    {  671, 1023,  837661,  837671,  837689, 10, 18, 18, }, // Japan
    {  671, 1058,       0,  939805,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  671, 1077,       0,  939826,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  671, 1106,  837855,  837865,  837883, 10, 18, 18, }, // Korea
    {  671, 1112,       0,  837901,       0,  0, 10,  0, }, // Kosrae
    {  671, 1119,  837911,  837927,  837950, 16, 23, 20, }, // Krasnoyarsk
    {  671, 1151,       0,  939847,       0,  0, 14,  0, }, // Kyrgystan
    {  671, 1175,       0,  837984,       0,  0, 17,  0, }, // Line_Islands
    {  671, 1188,  838001,  838015,  838038, 14, 23, 22, }, // Lord_Howe
    {  671, 1204,  838060,  838071,  838091, 11, 20, 15, }, // Magadan
    {  671, 1219,       0,  838106,       0,  0, 11,  0, }, // Malaysia
    {  671, 1228,       0,  939861,       0,  0, 13,  0, }, // Maldives
    {  671, 1237,       0,  838130,       0,  0, 12,  0, }, // Marquesas
    {  671, 1247,       0,  838142,       0,  0, 18,  0, }, // Marshall_Islands
    {  671, 1264,  838160,  838171,  838190, 11, 19, 15, }, // Mauritius
    {  671, 1274,       0,  838205,       0,  0, 10,  0, }, // Mawson
    {  671, 1281,  838215,  838233,  838260, 18, 27, 26, }, // Mexico_Pacific
    {  671, 1296,  838286,  838302,  838326, 16, 24, 20, }, // Mongolia
    {  671, 1305,  838346,  838356,  838375, 10, 19, 14, }, // Moscow
    {  671, 1312,       0,  838389,       0,  0, 12,  0, }, // Myanmar
    {  671, 1320,       0,  838401,       0,  0, 10,  0, }, // Nauru
    {  671, 1326,       0,  939874,       0,  0, 12,  0, }, // Nepal
    {  671, 1332,  838423,  838441,  838467, 18, 26, 22, }, // New_Caledonia
    {  671, 1346,  939886,  939903,  838530, 17, 24, 23, }, // New_Zealand
    {  671, 1358,  415179,  838553,  415225, 19, 28, 27, }, // Newfoundland
    {  671, 1371,       0,  838581,       0,  0,  9,  0, }, // Niue
    {  671, 1376,  939927,  939947,  939974, 20, 27, 33, }, // Norfolk
    {  671, 1384,  940007,  940031,  838717, 24, 31, 27, }, // Noronha
    {  671, 1406,  838744,  838759,  838782, 15, 23, 19, }, // Novosibirsk
    {  671, 1418,  838801,  838811,  838829, 10, 18, 14, }, // Omsk
    {  671, 1428,  940062,  940076,  940097, 14, 21, 27, }, // Pakistan
    {  671, 1437,       0,  838905,       0,  0,  9,  0, }, // Palau
    {  671, 1443,       0,  838914,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  671, 1460,  940124,  940139,  940161, 15, 22, 28, }, // Paraguay
    {  671, 1469,  940189,  940200,  940218, 11, 18, 24, }, // Peru
    {  671, 1474,  839050,  839061,  839080, 11, 19, 15, }, // Philippines
    {  671, 1486,       0,  839095,       0,  0, 18,  0, }, // Phoenix_Islands
    {  671, 1502,  940242,  940267,  940301, 25, 34, 33, }, // Pierre_Miquelon
    {  671, 1518,       0,  839208,       0,  0, 12,  0, }, // Pitcairn
    {  671, 1527,       0,  839220,       0,  0, 11,  0, }, // Ponape
    {  671, 1534,       0,  940334,       0,  0, 14,  0, }, // Pyongyang
    {  671, 1554,       0,  839245,       0,  0, 13,  0, }, // Reunion
    {  671, 1562,       0,  940348,       0,  0, 14,  0, }, // Rothera
    {  671, 1570,  839272,  839283,  839302, 11, 19, 15, }, // Sakhalin
    {  671, 1596,  839360,  839370,  839388, 10, 18, 18, }, // Samoa
    {  671, 1602,       0,  839406,       0,  0, 11,  0, }, // Seychelles
    {  671, 1624,       0,  839417,       0,  0, 20,  0, }, // Singapore
    {  671, 1634,       0,  839437,       0,  0, 18,  0, }, // Solomon
    {  671, 1642,       0,  839455,       0,  0, 17,  0, }, // South_Georgia
    {  671, 1656,       0,  940362,       0,  0, 14,  0, }, // Suriname
    {  671, 1676,       0,  839486,       0,  0, 10,  0, }, // Syowa
    {  671, 1682,       0,  839496,       0,  0, 11,  0, }, // Tahiti
    {  671, 1689,  839507,  839521,  839544, 14, 23, 21, }, // Taipei
    {  671, 1696,       0,  940376,       0,  0, 15,  0, }, // Tajikistan
    {  671, 1724,       0,  839580,       0,  0, 12,  0, }, // Tokelau
    {  671, 1732,  839592,  839602,  839620, 10, 18, 14, }, // Tonga
    {  671, 1738,       0,  839634,       0,  0,  8,  0, }, // Truk
    {  671, 1750,  940391,  940408,  940432, 17, 24, 30, }, // Turkmenistan
    {  671, 1763,       0,  839713,       0,  0, 10,  0, }, // Tuvalu
    {  671, 1777,  940462,  940477,  940499, 15, 22, 28, }, // Uruguay
    {  671, 1792,  940527,  940542,  940564, 15, 22, 28, }, // Uzbekistan
    {  671, 1803,  839853,  839865,  839885, 12, 20, 16, }, // Vanuatu
    {  671, 1811,       0,  940592,       0,  0, 16,  0, }, // Venezuela
    {  671, 1821,  839917,  839933,  839957, 16, 24, 20, }, // Vladivostok
    {  671, 1833,  839977,  839991,  840014, 14, 23, 18, }, // Volgograd
    {  671, 1843,       0,  940608,       0,  0, 13,  0, }, // Vostok
    {  671, 1850,       0,  840045,       0,  0, 15,  0, }, // Wake
    {  671, 1855,       0,  840060,       0,  0, 20,  0, }, // Wallis
    {  671, 1862,  840080,  840091,  840111, 11, 20, 15, }, // Yakutsk
    {  671, 1870,  840126,  840142,  840167, 16, 25, 20, }, // Yekaterinburg
    {  671, 1892,       0,  416982,       0,  0, 10,  0, }, // Yukon
    {  672,    5,       0,  940621,       0,  0, 22,  0, }, // Afghanistan Anii/Latin/Benin
    {  672,   17,       0,  940643,       0,  0, 31,  0, }, // Africa_Central
    {  672,   32,       0,  940674,       0,  0, 33,  0, }, // Africa_Eastern
    {  672,   65,       0,  940707,       0,  0, 36,  0, }, // Africa_Southern
    {  672,   81,  940743,  940780,  940824, 37, 44, 46, }, // Africa_Western
    {  672,  107,  940870,  940889,  940915, 19, 26, 28, }, // Alaska
    {  672,  135,  940943,  940961,  940986, 18, 25, 27, }, // Amazon
    {  672,  142,  941013,  941055,  941104, 42, 49, 51, }, // America_Central
    {  672,  158,  941155,  941198,  941248, 43, 50, 52, }, // America_Eastern
    {  672,  174,  941300,  941339,  941385, 39, 46, 48, }, // America_Mountain
    {  672,  191,  941433,  941463,  941500, 30, 37, 39, }, // America_Pacific
    {  672,  214,  941539,  941556,  941580, 17, 24, 26, }, // Apia
    {  672,  232,  941606,  941631,  941663, 25, 32, 34, }, // Arabian
    {  672,  240,  941697,  941717,  941744, 20, 27, 29, }, // Argentina
    {  672,  250,  941773,  941807,  941848, 34, 41, 43, }, // Argentina_Western
    {  672,  268,  941891,  941910,  941936, 19, 26, 28, }, // Armenia
    {  672,  286,  941964,  941995,  942033, 31, 38, 40, }, // Atlantic
    {  672,  295,  942073,  942104,  942142, 31, 38, 40, }, // Australia_Central
    {  672,  313,  942182,  942227,  942279, 45, 52, 54, }, // Australia_CentralWestern
    {  672,  338,  942333,  942365,  942404, 32, 39, 41, }, // Australia_Eastern
    {  672,  356,  942445,  942482,  942526, 37, 44, 46, }, // Australia_Western
    {  672,  374,  942572,  942594,  942623, 22, 29, 31, }, // Azerbaijan
    {  672,  385,  942654,  942671,  942695, 17, 24, 26, }, // Azores
    {  672,  397,  942721,  942745,  942776, 24, 31, 33, }, // Bangladesh
    {  672,  415,       0,  942809,       0,  0, 17,  0, }, // Bhutan
    {  672,  422,       0,  942826,       0,  0, 20,  0, }, // Bolivia
    {  672,  437,  942846,  942867,  942895, 21, 28, 30, }, // Brasilia
    {  672,  454,       0,  942925,       0,  0, 29,  0, }, // Brunei
    {  672,  461,  942954,  942973,  942999, 19, 26, 28, }, // Cape_Verde
    {  672,  478,       0,  943027,       0,  0, 20,  0, }, // Chamorro
    {  672,  487,  943047,  943065,  943090, 18, 25, 27, }, // Chatham
    {  672,  495,  943117,  943135,  943160, 18, 25, 27, }, // Chile
    {  672,  501,  943187,  943204,  943228, 17, 24, 26, }, // China
    {  672,  507,       0,  943254,       0,  0, 30,  0, }, // Christmas
    {  672,  517,       0,  943284,       0,  0, 32,  0, }, // Cocos
    {  672,  523,  943316,  943336,  943363, 20, 27, 29, }, // Colombia
    {  672,  532,  943392,  943422,  943459, 30, 37, 39, }, // Cook
    {  672,  537,  943498,  943515,  943539, 17, 24, 26, }, // Cuba
    {  672,  548,       0,  943565,       0,  0, 17,  0, }, // Davis
    {  672,  564,       0,  943582,       0,  0, 24,  0, }, // DumontDUrville
    {  672,  601,       0,  943606,       0,  0, 27,  0, }, // East_Timor
    {  672,  612,  943633,  943664,  943701, 31, 37, 39, }, // Easter
    {  672,  619,       0,  943740,       0,  0, 21,  0, }, // Ecuador
    {  672,  627,  943761,  943792,  943830, 31, 38, 40, }, // Europe_Central
    {  672,  642,  943870,  943903,  943943, 33, 40, 42, }, // Europe_Eastern
    {  672,  657,       0,  943985,       0,  0, 44,  0, }, // Europe_Further_Eastern
    {  672,  680,  944029,  944066,  944110, 37, 44, 46, }, // Europe_Western
    {  672,  695,  944156,  944189,  944229, 33, 40, 42, }, // Falkland
    {  672,  704,  944271,  944287,  944310, 16, 23, 25, }, // Fiji
    {  672,  709,       0,  944335,       0,  0, 34,  0, }, // French_Guiana
    {  672,  723,       0,  944369,       0,  0, 49,  0, }, // French_Southern
    {  672,  746,       0,  944418,       0,  0, 21,  0, }, // Galapagos
    {  672,  756,       0,  944439,       0,  0, 20,  0, }, // Gambier
    {  672,  764,  944459,  944478,  944504, 19, 26, 28, }, // Georgia
    {  672,  772,       0,  944532,       0,  0, 33,  0, }, // Gilbert_Islands
    {  672,  788,       0,  944565,       0,  0, 21,  0, }, // GMT
    {  672,  830,  944586,  944616,  944653, 30, 37, 39, }, // Greenland_Eastern
    {  672,  848,  944692,  944726,  944767, 34, 41, 43, }, // Greenland_Western
    {  672,  866,       0,  944810,       0,  0, 17,  0, }, // Guam
    {  672,  871,       0,  944827,       0,  0, 16,  0, }, // Gulf
    {  672,  876,       0,  944843,       0,  0, 19,  0, }, // Guyana
    {  672,  883,  944862,  944891,  944927, 29, 36, 38, }, // Hawaii_Aleutian
    {  672,  899,  944965,  944984,  945010, 19, 26, 28, }, // Hong_Kong
    {  672,  909,  945038,  945055,  945079, 17, 24, 26, }, // Hovd
    {  672,  914,       0,  945105,       0,  0, 18,  0, }, // India
    {  672,  920,       0,  945123,       0,  0, 26,  0, }, // Indian_Ocean
    {  672,  933,       0,  945149,       0,  0, 21,  0, }, // Indochina
    {  672,  943,       0,  945170,       0,  0, 31,  0, }, // Indonesia_Central
    {  672,  961,       0,  945201,       0,  0, 32,  0, }, // Indonesia_Eastern
    {  672,  979,       0,  945233,       0,  0, 37,  0, }, // Indonesia_Western
    {  672,  997,  945270,  945286,  945309, 16, 23, 25, }, // Iran
    {  672, 1008,  945334,  945351,  945375, 17, 24, 26, }, // Irkutsk
    {  672, 1016,  945401,  945421,  945448, 20, 27, 29, }, // Israel
    {  672, 1023,  945477,  945494,  945518, 17, 24, 26, }, // Japan
    {  672, 1047,       0,  945544,       0,  0, 20,  0, }, // Kazakhstan
    {  672, 1058,       0,  945564,       0,  0, 30,  0, }, // Kazakhstan_Eastern
    {  672, 1077,       0,  945594,       0,  0, 34,  0, }, // Kazakhstan_Western
    {  672, 1106,  945628,  945645,  945669, 17, 24, 26, }, // Korea
    {  672, 1112,       0,  945695,       0,  0, 18,  0, }, // Kosrae
    {  672, 1119,  945713,  945735,  945764, 22, 29, 31, }, // Krasnoyarsk
    {  672, 1151,       0,  945795,       0,  0, 21,  0, }, // Kyrgystan
    {  672, 1161,       0,  945816,       0,  0, 23,  0, }, // Lanka
    {  672, 1175,       0,  945839,       0,  0, 30,  0, }, // Line_Islands
    {  672, 1188,  945869,  945889,  945916, 20, 27, 29, }, // Lord_Howe
    {  672, 1198,  945945,  945964,  945990, 19, 26, 27, }, // Macau
    {  672, 1204,  946017,  946036,  946062, 19, 26, 28, }, // Magadan
    {  672, 1219,       0,  946090,       0,  0, 19,  0, }, // Malaysia
    {  672, 1228,       0,  946109,       0,  0, 20,  0, }, // Maldives
    {  672, 1237,       0,  946129,       0,  0, 20,  0, }, // Marquesas
    {  672, 1247,       0,  946149,       0,  0, 33,  0, }, // Marshall_Islands
    {  672, 1264,  946182,  946200,  946225, 18, 25, 27, }, // Mauritius
    {  672, 1274,       0,  946252,       0,  0, 17,  0, }, // Mawson
    {  672, 1281,  946269,  946297,  946332, 28, 35, 37, }, // Mexico_Pacific
    {  672, 1296,  946369,  946391,  946420, 22, 29, 31, }, // Mongolia
    {  672, 1305,  946451,  946469,  946494, 18, 25, 27, }, // Moscow
    {  672, 1312,       0,  946521,       0,  0, 20,  0, }, // Myanmar
    {  672, 1320,       0,  946541,       0,  0, 18,  0, }, // Nauru
    {  672, 1326,       0,  946559,       0,  0, 18,  0, }, // Nepal
    {  672, 1332,  946577,  946604,  946638, 27, 34, 36, }, // New_Caledonia
    {  672, 1346,  946674,  946698,  946729, 24, 31, 33, }, // New_Zealand
    {  672, 1358,  946762,  946789,  946823, 27, 34, 36, }, // Newfoundland
    {  672, 1371,       0,  946859,       0,  0, 18,  0, }, // Niue
    {  672, 1376,  946877,  946909,  946948, 32, 39, 41, }, // Norfolk
    {  672, 1384,  946989,  947020,  947058, 31, 38, 40, }, // Noronha
    {  672, 1392,       0,  947098,       0,  0, 46,  0, }, // North_Mariana
    {  672, 1406,  947144,  947166,  947195, 22, 29, 31, }, // Novosibirsk
    {  672, 1418,  947226,  947244,  947269, 18, 25, 27, }, // Omsk
    {  672, 1428,  947296,  947316,  947343, 20, 27, 29, }, // Pakistan
    {  672, 1437,       0,  947372,       0,  0, 19,  0, }, // Palau
    {  672, 1443,       0,  947391,       0,  0, 32,  0, }, // Papua_New_Guinea
    {  672, 1460,  947423,  947444,  947472, 21, 28, 30, }, // Paraguay
    {  672, 1469,  947502,  947519,  947543, 17, 24, 26, }, // Peru
    {  672, 1474,  947569,  947589,  947616, 20, 27, 29, }, // Philippines
    {  672, 1486,       0,  947645,       0,  0, 33,  0, }, // Phoenix_Islands
    {  672, 1502,  947678,  947711,  947751, 33, 40, 42, }, // Pierre_Miquelon
    {  672, 1518,       0,  947793,       0,  0, 33,  0, }, // Pitcairn
    {  672, 1527,       0,  947826,       0,  0, 19,  0, }, // Ponape
    {  672, 1534,       0,  947845,       0,  0, 20,  0, }, // Pyongyang
    {  672, 1554,       0,  947865,       0,  0, 20,  0, }, // Reunion
    {  672, 1562,       0,  947885,       0,  0, 19,  0, }, // Rothera
    {  672, 1570,  947904,  947923,  947949, 19, 26, 28, }, // Sakhalin
    {  672, 1596,  947977,  947995,  948020, 18, 25, 27, }, // Samoa
    {  672, 1602,       0,  948047,       0,  0, 18,  0, }, // Seychelles
    {  672, 1624,       0,  948065,       0,  0, 21,  0, }, // Singapore
    {  672, 1634,       0,  948086,       0,  0, 34,  0, }, // Solomon
    {  672, 1642,       0,  948120,       0,  0, 29,  0, }, // South_Georgia
    {  672, 1656,       0,  948149,       0,  0, 19,  0, }, // Suriname
    {  672, 1676,       0,  948168,       0,  0, 19,  0, }, // Syowa
    {  672, 1682,       0,  948187,       0,  0, 19,  0, }, // Tahiti
    {  672, 1689,  948206,  948224,  948249, 18, 25, 27, }, // Taipei
    {  672, 1696,       0,  948276,       0,  0, 22,  0, }, // Tajikistan
    {  672, 1724,       0,  948298,       0,  0, 19,  0, }, // Tokelau
    {  672, 1732,  948317,  948334,  948358, 17, 24, 26, }, // Tonga
    {  672, 1738,       0,  948384,       0,  0, 16,  0, }, // Truk
    {  672, 1750,  948400,  948424,  948455, 24, 31, 33, }, // Turkmenistan
    {  672, 1763,       0,  948488,       0,  0, 18,  0, }, // Tuvalu
    {  672, 1777,  948506,  948526,  948553, 20, 27, 29, }, // Uruguay
    {  672, 1792,  948582,  948605,  948635, 23, 30, 32, }, // Uzbekistan
    {  672, 1803,  948667,  948687,  948714, 20, 27, 29, }, // Vanuatu
    {  672, 1811,       0,  948743,       0,  0, 23,  0, }, // Venezuela
    {  672, 1821,  948766,  948789,  948819, 23, 30, 32, }, // Vladivostok
    {  672, 1833,  948851,  948873,  948902, 22, 29, 31, }, // Volgograd
    {  672, 1843,       0,  948933,       0,  0, 18,  0, }, // Vostok
    {  672, 1850,       0,  948951,       0,  0, 30,  0, }, // Wake
    {  672, 1855,       0,  948981,       0,  0, 27,  0, }, // Wallis
    {  672, 1862,  949008,  949025,  949049, 17, 24, 26, }, // Yakutsk
    {  672, 1870,  949075,  949099,  949130, 24, 31, 33, }, // Yekaterinburg
    {  672, 1892,       0,  949163,       0,  0, 17,  0, }, // Yukon
    {  673,    5,       0,  949180,       0,  0, 20,  0, }, // Afghanistan Kangri/Devanagari/India
    {  673,   17,       0,  949200,       0,  0, 20,  0, }, // Africa_Central
    {  673,   32,       0,  949220,       0,  0, 21,  0, }, // Africa_Eastern
    {  673,   65,       0,  949241,       0,  0, 24,  0, }, // Africa_Southern
    {  673,   81,  949265,  949287,  949314, 22, 27, 33, }, // Africa_Western
    {  673,  107,  949347,  949361,  949381, 14, 20, 35, }, // Alaska
    {  673,  135,  949416,  949430,  949449, 14, 19, 25, }, // Amazon
    {  673,  142,  949474,  949503,  949537, 29, 34, 49, }, // America_Central
    {  673,  158,  949586,  949609,  949640, 23, 31, 46, }, // America_Eastern
    {  673,  174,  949686,  949713,  949746, 27, 33, 48, }, // America_Mountain
    {  673,  191,  949794,  949824,  949859, 30, 35, 50, }, // America_Pacific
    {  673,  214,  949909,  949920,  949936, 11, 16, 31, }, // Apia
    {  673,  232,  949967,  949977,  949992, 10, 15, 30, }, // Arabian
    {  673,  240,  950022,  950039,  950061, 17, 22, 28, }, // Argentina
    {  673,  250,  950089,  950114,  950144, 25, 30, 36, }, // Argentina_Western
    {  673,  268,  950180,  950196,  950217, 16, 21, 27, }, // Armenia
    {  673,  286,  950244,  950259,  950279, 15, 20, 35, }, // Atlantic
    {  673,  295,  950314,  950338,  950374, 24, 36, 51, }, // Australia_Central
    {  673,  313,  950425,  950462,  950503, 37, 41, 54, }, // Australia_CentralWestern
    {  673,  338,  950557,  950581,  950612, 24, 31, 46, }, // Australia_Eastern
    {  673,  356,  950658,  950684,  950715, 26, 31, 46, }, // Australia_Western
    {  673,  374,  950761,  950777,  950798, 16, 21, 27, }, // Azerbaijan
    {  673,  385,  950825,  950839,  950858, 14, 19, 25, }, // Azores
    {  673,  397,  950883,  950901,  950924, 18, 23, 29, }, // Bangladesh
    {  673,  415,       0,  950953,       0,  0, 13,  0, }, // Bhutan
    {  673,  422,       0,  950966,       0,  0, 15,  0, }, // Bolivia
    {  673,  437,  950981,  950999,  951022, 18, 23, 29, }, // Brasilia
    {  673,  454,       0,  951051,       0,  0, 24,  0, }, // Brunei
    {  673,  461,  951075,  951090,  951110, 15, 20, 26, }, // Cape_Verde
    {  673,  478,       0,  951136,       0,  0, 18,  0, }, // Chamorro
    {  673,  487,  951154,  951165,  951181, 11, 16, 31, }, // Chatham
    {  673,  495,  951212,  951223,  951239, 11, 16, 22, }, // Chile
    {  673,  501,  951261,  951271,  951286, 10, 15, 30, }, // China
    {  673,  507,       0,  951316,       0,  0, 19,  0, }, // Christmas
    {  673,  517,       0,  951335,       0,  0, 28,  0, }, // Cocos
    {  673,  523,  951363,  951378,  951398, 15, 20, 26, }, // Colombia
    {  673,  532,  951424,  951450,  951481, 26, 31, 43, }, // Cook
    {  673,  537,  951524,  951537,  951555, 13, 18, 33, }, // Cuba
    {  673,  548,       0,  951588,       0,  0, 12,  0, }, // Davis
    {  673,  564,       0,  951600,       0,  0, 26,  0, }, // DumontDUrville
    {  673,  601,       0,  951626,       0,  0, 19,  0, }, // East_Timor
    {  673,  612,  951645,  951662,  951684, 17, 22, 28, }, // Easter
    {  673,  619,       0,  951712,       0,  0, 15,  0, }, // Ecuador
    {  673,  627,  951727,  951746,  951770, 19, 24, 30, }, // Europe_Central
    {  673,  642,  951800,  951819,  951843, 19, 24, 30, }, // Europe_Eastern
    {  673,  657,       0,  951873,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  673,  680,  951897,  951918,  951944, 21, 26, 32, }, // Europe_Western
    {  673,  695,  951976,  952007,  952043, 31, 36, 42, }, // Falkland
    {  673,  704,  952085,  952097,  952114, 12, 17, 23, }, // Fiji
    {  673,  709,       0,  952137,       0,  0, 21,  0, }, // French_Guiana
    {  673,  723,       0,  952158,       0,  0, 39,  0, }, // French_Southern
    {  673,  746,       0,  952197,       0,  0, 16,  0, }, // Galapagos
    {  673,  756,       0,  952213,       0,  0, 14,  0, }, // Gambier
    {  673,  764,  952227,  952242,  952262, 15, 20, 26, }, // Georgia
    {  673,  772,       0,  952288,       0,  0, 31,  0, }, // Gilbert_Islands
    {  673,  788,       0,  952319,       0,  0, 16,  0, }, // GMT
    {  673,  830,  952335,  952354,  952381, 19, 27, 33, }, // Greenland_Eastern
    {  673,  848,  952414,  952435,  952464, 21, 29, 35, }, // Greenland_Western
    {  673,  871,       0,  952499,       0,  0, 17,  0, }, // Gulf
    {  673,  876,       0,  952145,       0,  0, 13,  0, }, // Guyana
    {  673,  883,  952516,  952535,  952559, 19, 24, 39, }, // Hawaii_Aleutian
    {  673,  899,  952598,  952614,  952635, 16, 21, 27, }, // Hong_Kong
    {  673,  909,  952662,  952674,  952691, 12, 17, 23, }, // Hovd
    {  673,  914,       0,  952714,       0,  0, 17,  0, }, // India
    {  673,  920,       0,  952731,       0,  0, 19,  0, }, // Indian_Ocean
    {  673,  933,       0,  952750,       0,  0, 16,  0, }, // Indochina
    {  673,  943,       0,  952766,       0,  0, 23,  0, }, // Indonesia_Central
    {  673,  961,       0,  952789,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  673,  979,       0,  952812,       0,  0, 25,  0, }, // Indonesia_Western
    {  673,  997,  952837,  952848,  952864, 11, 16, 31, }, // Iran
    {  673, 1008,  952895,  952912,  952934, 17, 22, 28, }, // Irkutsk
    {  673, 1016,  952962,  952976,  952995, 14, 19, 34, }, // Israel
    {  673, 1023,  953029,  953042,  953060, 13, 18, 33, }, // Japan
    {  673, 1058,       0,  953093,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  673, 1077,       0,  953115,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  673, 1106,  953139,  953153,  953172, 14, 19, 34, }, // Korea
    {  673, 1112,       0,  953206,       0,  0, 13,  0, }, // Kosrae
    {  673, 1119,  953219,  953241,  953268, 22, 27, 33, }, // Krasnoyarsk
    {  673, 1151,       0,  953301,       0,  0, 19,  0, }, // Kyrgystan
    {  673, 1175,       0,  953320,       0,  0, 27,  0, }, // Line_Islands
    {  673, 1188,  953347,  953364,  953386, 17, 22, 37, }, // Lord_Howe
    {  673, 1204,  953423,  953437,  953456, 14, 19, 25, }, // Magadan
    {  673, 1219,       0,  953481,       0,  0, 14,  0, }, // Malaysia
    {  673, 1228,       0,  953495,       0,  0, 13,  0, }, // Maldives
    {  673, 1237,       0,  953508,       0,  0, 15,  0, }, // Marquesas
    {  673, 1247,       0,  953523,       0,  0, 29,  0, }, // Marshall_Islands
    {  673, 1264,  953552,  953565,  953583, 13, 18, 24, }, // Mauritius
    {  673, 1274,       0,  953607,       0,  0, 13,  0, }, // Mawson
    {  673, 1281,  953620,  953643,  953671, 23, 28, 43, }, // Mexico_Pacific
    {  673, 1296,  953714,  953730,  953751, 16, 21, 27, }, // Mongolia
    {  673, 1305,  953778,  953791,  953809, 13, 18, 24, }, // Moscow
    {  673, 1312,       0,  953833,       0,  0, 15,  0, }, // Myanmar
    {  673, 1320,       0,  953848,       0,  0, 11,  0, }, // Nauru
    {  673, 1326,       0,  953859,       0,  0, 12,  0, }, // Nepal
    {  673, 1332,  953871,  953893,  953920, 22, 27, 33, }, // New_Caledonia
    {  673, 1346,  953953,  953971,  953994, 18, 23, 38, }, // New_Zealand
    {  673, 1358,  954032,  954053,  954079, 21, 26, 41, }, // Newfoundland
    {  673, 1371,       0,  954120,       0,  0, 11,  0, }, // Niue
    {  673, 1376,  954131,  954150,  954174, 19, 24, 39, }, // Norfolk
    {  673, 1384,  954213,  954243,  954278, 30, 35, 41, }, // Noronha
    {  673, 1406,  954319,  954339,  954364, 20, 25, 31, }, // Novosibirsk
    {  673, 1418,  954395,  954408,  954426, 13, 18, 24, }, // Omsk
    {  673, 1428,  954450,  954467,  954489, 17, 22, 28, }, // Pakistan
    {  673, 1437,       0,  954517,       0,  0, 11,  0, }, // Palau
    {  673, 1443,       0,  954528,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  673, 1460,  954550,  954565,  954585, 15, 20, 26, }, // Paraguay
    {  673, 1469,  954611,  954622,  954638, 11, 16, 22, }, // Peru
    {  673, 1474,  954660,  954675,  954695, 15, 20, 26, }, // Philippines
    {  673, 1486,       0,  954721,       0,  0, 31,  0, }, // Phoenix_Islands
    {  673, 1502,  954752,  954783,  954819, 31, 36, 51, }, // Pierre_Miquelon
    {  673, 1518,       0,  954870,       0,  0, 15,  0, }, // Pitcairn
    {  673, 1527,       0,  954885,       0,  0, 13,  0, }, // Ponape
    {  673, 1534,       0,  954898,       0,  0, 17,  0, }, // Pyongyang
    {  673, 1554,       0,  954915,       0,  0, 15,  0, }, // Reunion
    {  673, 1562,       0,  954930,       0,  0, 13,  0, }, // Rothera
    {  673, 1570,  954943,  954956,  954974, 13, 18, 24, }, // Sakhalin
    {  673, 1596,  954998,  955009,  955025, 11, 16, 31, }, // Samoa
    {  673, 1602,       0,  955056,       0,  0, 14,  0, }, // Seychelles
    {  673, 1624,       0,  955070,       0,  0, 16,  0, }, // Singapore
    {  673, 1634,       0,  955086,       0,  0, 29,  0, }, // Solomon
    {  673, 1642,       0,  955115,       0,  0, 19,  0, }, // South_Georgia
    {  673, 1656,       0,  955134,       0,  0, 15,  0, }, // Suriname
    {  673, 1676,       0,  955149,       0,  0, 13,  0, }, // Syowa
    {  673, 1682,       0,  955162,       0,  0, 13,  0, }, // Tahiti
    {  673, 1689,  955175,  955187,  955204, 12, 17, 32, }, // Taipei
    {  673, 1696,       0,  955236,       0,  0, 19,  0, }, // Tajikistan
    {  673, 1724,       0,  955255,       0,  0, 14,  0, }, // Tokelau
    {  673, 1732,  955269,  955281,  955298, 12, 17, 23, }, // Tonga
    {  673, 1738,       0,  955321,       0,  0, 10,  0, }, // Truk
    {  673, 1750,  955331,  955352,  955379, 21, 27, 33, }, // Turkmenistan
    {  673, 1763,       0,  955412,       0,  0, 13,  0, }, // Tuvalu
    {  673, 1777,  955425,  955439,  955458, 14, 19, 25, }, // Uruguay
    {  673, 1792,  955483,  955503,  955528, 20, 25, 31, }, // Uzbekistan
    {  673, 1803,  955559,  955572,  955590, 13, 18, 24, }, // Vanuatu
    {  673, 1811,       0,  955614,       0,  0, 17,  0, }, // Venezuela
    {  673, 1821,  955631,  955651,  955676, 20, 25, 31, }, // Vladivostok
    {  673, 1833,  955707,  955725,  955748, 18, 23, 29, }, // Volgograd
    {  673, 1843,       0,  955637,       0,  0, 14,  0, }, // Vostok
    {  673, 1850,       0,  955777,       0,  0, 15,  0, }, // Wake
    {  673, 1855,       0,  955792,       0,  0, 26,  0, }, // Wallis
    {  673, 1862,  955818,  955834,  955855, 16, 21, 27, }, // Yakutsk
    {  673, 1870,  955882,  955902,  955927, 20, 25, 31, }, // Yekaterinburg
    {  673, 1892,       0,  955958,       0,  0,  9,  0, }, // Yukon
    {  674,    5,       0,  955967,       0,  0, 20,  0, }, // Afghanistan Venetian/Latin/Italy
    {  674,   17,       0,  955987,       0,  0, 24,  0, }, // Africa_Central
    {  674,   32,       0,  956011,       0,  0, 25,  0, }, // Africa_Eastern
    {  674,   65,       0,  956036,       0,  0, 27,  0, }, // Africa_Southern
    {  674,   81,  956063,  956089,  956123, 26, 34, 33, }, // Africa_Western
    {  674,  107,  956156,  956171,  956194, 15, 23, 22, }, // Alaska
    {  674,  135,  956216,  956233,  956258, 17, 25, 24, }, // Amazon
    {  674,  142,  956282,  956312,  956350, 30, 38, 37, }, // America_Central
    {  674,  158,  956387,  956418,  956457, 31, 39, 38, }, // America_Eastern
    {  674,  174,  956495,  956531,  956575, 36, 44, 43, }, // America_Mountain
    {  674,  191,  956618,  956635,  956660, 17, 25, 24, }, // America_Pacific
    {  674,  214,  655289,  956684,  956703, 11, 19, 18, }, // Apia
    {  674,  232,  956721,  956736,  956759, 15, 23, 22, }, // Arabian
    {  674,  240,  956781,  956799,  956825, 18, 26, 25, }, // Argentina
    {  674,  250,  956850,  956879,  956916, 29, 37, 36, }, // Argentina_Western
    {  674,  268,  956952,  956968,  956992, 16, 24, 23, }, // Armenia
    {  674,  286,  957015,  957033,  957059, 18, 26, 25, }, // Atlantic
    {  674,  295,  957084,  957111,  957146, 27, 35, 34, }, // Australia_Central
    {  674,  313,  957180,  957216,  957260, 36, 44, 43, }, // Australia_CentralWestern
    {  674,  338,  957303,  957331,  957367, 28, 36, 35, }, // Australia_Eastern
    {  674,  356,  957402,  957431,  957468, 29, 37, 36, }, // Australia_Western
    {  674,  374,  957504,  957523,  957550, 19, 27, 26, }, // Azerbaijan
    {  674,  385,  957576,  957591,  957614, 15, 23, 22, }, // Azores
    {  674,  397,  957636,  957655,  957682, 19, 27, 26, }, // Bangladesh
    {  674,  415,       0,  957708,       0,  0, 14,  0, }, // Bhutan
    {  674,  422,       0,  957722,       0,  0, 17,  0, }, // Bolivia
    {  674,  437,  957739,  957754,  957777, 15, 23, 22, }, // Brasilia
    {  674,  454,       0,  957799,       0,  0, 15,  0, }, // Brunei
    {  674,  461,  957814,  957830,  957854, 16, 24, 23, }, // Cape_Verde
    {  674,  478,       0,  957877,       0,  0, 14,  0, }, // Chamorro
    {  674,  487,  957891,  957913,  957943, 22, 30, 29, }, // Chatham
    {  674,  495,  957972,  957985,  958006, 13, 21, 20, }, // Chile
    {  674,  501,  958026,  958040,  958062, 14, 22, 21, }, // China
    {  674,  507,       0,  958083,       0,  0, 24,  0, }, // Christmas
    {  674,  517,       0,  958107,       0,  0, 21,  0, }, // Cocos
    {  674,  523,  958128,  958146,  958172, 18, 26, 25, }, // Colombia
    {  674,  532,  958197,  958217,  958245, 20, 28, 27, }, // Cook
    {  674,  537,  657099,  958272,  958291, 11, 19, 18, }, // Cuba
    {  674,  548,       0,  657149,       0,  0, 12,  0, }, // Davis
    {  674,  564,       0,  958309,       0,  0, 23,  0, }, // DumontDUrville
    {  674,  601,       0,  958332,       0,  0, 18,  0, }, // East_Timor
    {  674,  612,  958350,  958374,  958406, 24, 32, 31, }, // Easter
    {  674,  619,       0,  958437,       0,  0, 16,  0, }, // Ecuador
    {  674,  627,  958453,  958477,  958509, 24, 32, 31, }, // Europe_Central
    {  674,  642,  958540,  958565,  958598, 25, 33, 32, }, // Europe_Eastern
    {  674,  657,       0,  958630,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  674,  680,  958660,  958686,  958720, 26, 34, 33, }, // Europe_Western
    {  674,  695,  958753,  958776,  958807, 23, 31, 30, }, // Falkland
    {  674,  704,  958837,  958857,  958885, 20, 28, 27, }, // Fiji
    {  674,  709,       0,  958912,       0,  0, 25,  0, }, // French_Guiana
    {  674,  723,       0,  958937,       0,  0, 49,  0, }, // French_Southern
    {  674,  746,       0,  958986,       0,  0, 19,  0, }, // Galapagos
    {  674,  756,       0,  959005,       0,  0, 23,  0, }, // Gambier
    {  674,  764,  959028,  959044,  959068, 16, 24, 23, }, // Georgia
    {  674,  772,       0,  959091,       0,  0, 23,  0, }, // Gilbert_Islands
    {  674,  788,       0,  959114,       0,  0, 30,  0, }, // GMT
    {  674,  830,  959144,  959174,  959212, 30, 38, 37, }, // Greenland_Eastern
    {  674,  848,  959249,  959280,  959319, 31, 39, 38, }, // Greenland_Western
    {  674,  871,       0,  959357,       0,  0, 14,  0, }, // Gulf
    {  674,  876,       0,  958912,       0,  0, 16,  0, }, // Guyana
    {  674,  883,  959371,  959394,  959425, 23, 31, 30, }, // Hawaii_Aleutian
    {  674,  899,  658432,  959455,  959479, 16, 24, 23, }, // Hong_Kong
    {  674,  909,  658497,  959502,  959521, 11, 19, 18, }, // Hovd
    {  674,  914,       0,  959539,       0,  0, 14,  0, }, // India
    {  674,  920,       0,  959553,       0,  0, 22,  0, }, // Indian_Ocean
    {  674,  933,       0,  959575,       0,  0, 17,  0, }, // Indochina
    {  674,  943,       0,  959592,       0,  0, 27,  0, }, // Indonesia_Central
    {  674,  961,       0,  959619,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  674,  979,       0,  959647,       0,  0, 29,  0, }, // Indonesia_Western
    {  674,  997,  959676,  959689,  959710, 13, 21, 20, }, // Iran
    {  674, 1008,  658755,  959730,  959752, 14, 22, 21, }, // Irkutsk
    {  674, 1016,  959773,  959787,  959809, 14, 22, 21, }, // Israel
    {  674, 1023,  959830,  959844,  959866, 14, 22, 21, }, // Japan
    {  674, 1047,       0,  959887,       0,  0, 19,  0, }, // Kazakhstan
    {  674, 1058,       0,  959906,       0,  0, 29,  0, }, // Kazakhstan_Eastern
    {  674, 1077,       0,  959935,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  674, 1106,  959965,  959980,  960003, 15, 23, 22, }, // Korea
    {  674, 1112,       0,  960025,       0,  0, 21,  0, }, // Kosrae
    {  674, 1119,  659167,  960046,  960072, 18, 26, 25, }, // Krasnoyarsk
    {  674, 1151,       0,  960097,       0,  0, 21,  0, }, // Kyrgystan
    {  674, 1175,       0,  960118,       0,  0, 27,  0, }, // Line_Islands
    {  674, 1188,  960145,  960169,  960201, 24, 32, 31, }, // Lord_Howe
    {  674, 1204,  659419,  960232,  960254, 14, 22, 21, }, // Magadan
    {  674, 1219,       0,  960275,       0,  0, 17,  0, }, // Malaysia
    {  674, 1228,       0,  960292,       0,  0, 23,  0, }, // Maldives
    {  674, 1237,       0,  960315,       0,  0, 24,  0, }, // Marquesas
    {  674, 1247,       0,  960339,       0,  0, 24,  0, }, // Marshall_Islands
    {  674, 1264,  960363,  960387,  960419, 24, 32, 31, }, // Mauritius
    {  674, 1274,       0,  659640,       0,  0, 13,  0, }, // Mawson
    {  674, 1281,  960450,  960479,  960516, 29, 37, 36, }, // Mexico_Pacific
    {  674, 1296,  960552,  960569,  960594, 17, 25, 24, }, // Mongolia
    {  674, 1305,  659825,  960618,  960638, 12, 20, 19, }, // Moscow
    {  674, 1312,       0,  960657,       0,  0, 25,  0, }, // Myanmar
    {  674, 1320,       0,  960682,       0,  0, 20,  0, }, // Nauru
    {  674, 1326,       0,  960702,       0,  0, 14,  0, }, // Nepal
    {  674, 1332,  960716,  960740,  960772, 24, 32, 31, }, // New_Caledonia
    {  674, 1346,  960803,  960825,  960855, 22, 30, 29, }, // New_Zealand
    {  674, 1358,  960884,  960899,  960922, 15, 23, 22, }, // Newfoundland
    {  674, 1371,       0,  960944,       0,  0, 19,  0, }, // Niue
    {  674, 1376,  960963,  960985,  961015, 22, 30, 29, }, // Norfolk
    {  674, 1384,  961044,  961070,  961104, 26, 34, 33, }, // Noronha
    {  674, 1406,  660393,  961137,  961163, 18, 26, 25, }, // Novosibirsk
    {  674, 1418,  660464,  961188,  961207, 11, 19, 18, }, // Omsk
    {  674, 1428,  961225,  961242,  961267, 17, 25, 24, }, // Pakistan
    {  674, 1437,       0,  961291,       0,  0, 21,  0, }, // Palau
    {  674, 1443,       0,  961312,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  674, 1460,  961339,  961356,  961381, 17, 25, 24, }, // Paraguay
    {  674, 1469,  961405,  961418,  961439, 13, 21, 20, }, // Peru
    {  674, 1474,  961459,  961483,  961515, 24, 32, 31, }, // Philippines
    {  674, 1486,       0,  961546,       0,  0, 22,  0, }, // Phoenix_Islands
    {  674, 1502,  961568,  961594,  961628, 26, 34, 33, }, // Pierre_Miquelon
    {  674, 1518,       0,  961661,       0,  0, 24,  0, }, // Pitcairn
    {  674, 1527,       0,  961685,       0,  0, 21,  0, }, // Ponape
    {  674, 1534,       0,  661002,       0,  0, 16,  0, }, // Pyongyang
    {  674, 1554,       0,  961706,       0,  0, 22,  0, }, // Reunion
    {  674, 1562,       0,  661101,       0,  0, 14,  0, }, // Rothera
    {  674, 1570,  961728,  961742,  961764, 14, 22, 21, }, // Sakhalin
    {  674, 1596,  961785,  961806,  961835, 21, 29, 28, }, // Samoa
    {  674, 1602,       0,  961863,       0,  0, 29,  0, }, // Seychelles
    {  674, 1624,       0,  661319,       0,  0, 16,  0, }, // Singapore
    {  674, 1634,       0,  961892,       0,  0, 23,  0, }, // Solomon
    {  674, 1642,       0,  961915,       0,  0, 26,  0, }, // South_Georgia
    {  674, 1656,       0,  961941,       0,  0, 17,  0, }, // Suriname
    {  674, 1676,       0,  661406,       0,  0, 12,  0, }, // Syowa
    {  674, 1682,       0,  961958,       0,  0, 20,  0, }, // Tahiti
    {  674, 1689,  961978,  961991,  962012, 13, 21, 20, }, // Taipei
    {  674, 1696,       0,  962032,       0,  0, 19,  0, }, // Tajikistan
    {  674, 1724,       0,  962051,       0,  0, 23,  0, }, // Tokelau
    {  674, 1732,  962074,  962095,  962124, 21, 29, 28, }, // Tonga
    {  674, 1738,       0,  962152,       0,  0, 14,  0, }, // Truk
    {  674, 1750,  962166,  962187,  962216, 21, 29, 28, }, // Turkmenistan
    {  674, 1763,       0,  962244,       0,  0, 22,  0, }, // Tuvalu
    {  674, 1777,  962266,  962282,  962306, 16, 24, 23, }, // Uruguay
    {  674, 1792,  962329,  962348,  962375, 19, 27, 26, }, // Uzbekistan
    {  674, 1803,  962401,  962424,  962455, 23, 31, 30, }, // Vanuatu
    {  674, 1811,       0,  962485,       0,  0, 18,  0, }, // Venezuela
    {  674, 1821,  661927,  962503,  962529, 18, 26, 25, }, // Vladivostok
    {  674, 1833,  962554,  962571,  962596, 17, 25, 24, }, // Volgograd
    {  674, 1843,       0,  662063,       0,  0, 13,  0, }, // Vostok
    {  674, 1850,       0,  962620,       0,  0, 19,  0, }, // Wake
    {  674, 1855,       0,  962639,       0,  0, 31,  0, }, // Wallis
    {  674, 1862,  662120,  962670,  962692, 14, 22, 21, }, // Yakutsk
    {  674, 1870,  962713,  962733,  962761, 20, 28, 27, }, // Yekaterinburg
    {  674, 1892,       0,  962788,       0,  0, 14,  0, }, // Yukon
    {  675,    5,       0,  962802,       0,  0, 16,  0, }, // Afghanistan Kuvi/Latin/India
    {  675,   17,       0,  962818,       0,  0, 19,  0, }, // Africa_Central
    {  675,   32,       0,  962837,       0,  0, 21,  0, }, // Africa_Eastern
    {  675,   65,       0,  962858,       0,  0, 17,  0, }, // Africa_Southern
    {  675,   81,  962875,  962897,  962928, 22, 31, 28, }, // Africa_Western
    {  675,  107,  962956,  962967,  962986, 11, 19, 19, }, // Alaska
    {  675,  135,  963005,  963016,  963035, 11, 19, 21, }, // Amazon
    {  675,  142,  963056,  963067,  963086, 11, 19, 19, }, // America_Central
    {  675,  158,  963105,  963118,  963139, 13, 21, 21, }, // America_Eastern
    {  675,  174,  963160,  963170,  963188, 10, 18, 18, }, // America_Mountain
    {  675,  191,  963206,  963218,  963238, 12, 20, 20, }, // America_Pacific
    {  675,  214,  963258,  963267,  963284,  9, 17, 16, }, // Apia
    {  675,  232,  963300,  963312,  963332, 12, 20, 19, }, // Arabian
    {  675,  240,  963351,  963365,  963387, 14, 22, 25, }, // Argentina
    {  675,  250,  963412,  963437,  963470, 25, 33, 31, }, // Argentina_Western
    {  675,  268,  963501,  963513,  963533, 12, 20, 18, }, // Armenia
    {  675,  286,  963551,  963564,  963585, 13, 21, 20, }, // Atlantic
    {  675,  295,  963605,  963625,  963655, 20, 30, 29, }, // Australia_Central
    {  675,  313,  963684,  963717,  963758, 33, 41, 40, }, // Australia_CentralWestern
    {  675,  338,  963798,  963820,  963852, 22, 32, 32, }, // Australia_Eastern
    {  675,  356,  963884,  963908,  963942, 24, 34, 33, }, // Australia_Western
    {  675,  374,  963975,  963990,  964013, 15, 23, 21, }, // Azerbaijan
    {  675,  385,  964034,  964045,  964064, 11, 19, 17, }, // Azores
    {  675,  397,  964081,  964095,  964117, 14, 22, 20, }, // Bangladesh
    {  675,  415,       0,  964137,       0,  0, 10,  0, }, // Bhutan
    {  675,  422,       0,  964147,       0,  0, 11,  0, }, // Bolivia
    {  675,  437,  964158,  964171,  964192, 13, 21, 19, }, // Brasilia
    {  675,  454,       0,  964211,       0,  0, 22,  0, }, // Brunei
    {  675,  461,  964233,  964244,  964263, 11, 19, 17, }, // Cape_Verde
    {  675,  478,       0,  964280,       0,  0, 18,  0, }, // Chamorro
    {  675,  487,  964298,  964309,  964328, 11, 19, 18, }, // Chatham
    {  675,  495,  964346,  964355,  964372,  9, 17, 15, }, // Chile
    {  675,  501,  964387,  964395,  964411,  8, 16, 15, }, // China
    {  675,  507,       0,  964426,       0,  0, 16,  0, }, // Christmas
    {  675,  517,       0,  964442,       0,  0, 14,  0, }, // Cocos
    {  675,  523,  964456,  964469,  964490, 13, 21, 19, }, // Colombia
    {  675,  532,  964509,  964521,  964541, 12, 20, 22, }, // Cook
    {  675,  537,  964563,  964572,  964589,  9, 17, 16, }, // Cuba
    {  675,  548,       0,  964605,       0,  0, 10,  0, }, // Davis
    {  675,  564,       0,  964615,       0,  0, 21,  0, }, // DumontDUrville
    {  675,  601,       0,  964636,       0,  0, 19,  0, }, // East_Timor
    {  675,  612,  964655,  964668,  964689, 13, 21, 19, }, // Easter
    {  675,  619,       0,  964708,       0,  0, 12,  0, }, // Ecuador
    {  675,  627,  964720,  964741,  964770, 21, 29, 27, }, // Europe_Central
    {  675,  642,  964797,  964819,  964849, 22, 30, 28, }, // Europe_Eastern
    {  675,  657,       0,  964877,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  675,  680,  964901,  964926,  964959, 25, 33, 31, }, // Europe_Western
    {  675,  695,  964990,  965010,  965038, 20, 28, 26, }, // Falkland
    {  675,  704,  965064,  965073,  965090,  9, 17, 15, }, // Fiji
    {  675,  709,       0,  965105,       0,  0, 17,  0, }, // French_Guiana
    {  675,  723,       0,  965122,       0,  0, 31,  0, }, // French_Southern
    {  675,  746,       0,  965153,       0,  0, 13,  0, }, // Galapagos
    {  675,  756,       0,  965166,       0,  0, 12,  0, }, // Gambier
    {  675,  764,  965178,  965189,  965208, 11, 19, 17, }, // Georgia
    {  675,  772,       0,  965225,       0,  0, 15,  0, }, // Gilbert_Islands
    {  675,  788,       0,  965240,       0,  0, 16,  0, }, // GMT
    {  675,  830,  965256,  965277,  965306, 21, 29, 27, }, // Greenland_Eastern
    {  675,  848,  965333,  965356,  965387, 23, 31, 29, }, // Greenland_Western
    {  675,  871,       0,  965416,       0,  0, 17,  0, }, // Gulf
    {  675,  876,       0,  965112,       0,  0, 10,  0, }, // Guyana
    {  675,  883,  965433,  965452,  965479, 19, 27, 26, }, // Hawaii_Aleutian
    {  675,  899,  965505,  965519,  965541, 14, 22, 20, }, // Hong_Kong
    {  675,  909,  965561,  965570,  965587,  9, 17, 15, }, // Hovd
    {  675,  914,       0,  965602,       0,  0, 18,  0, }, // India
    {  675,  920,       0,  965620,       0,  0, 15,  0, }, // Indian_Ocean
    {  675,  933,       0,  965635,       0,  0, 13,  0, }, // Indochina
    {  675,  943,       0,  965648,       0,  0, 20,  0, }, // Indonesia_Central
    {  675,  961,       0,  965668,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  675,  979,       0,  965689,       0,  0, 23,  0, }, // Indonesia_Western
    {  675,  997,  965712,  965721,  965738,  9, 17, 16, }, // Iran
    {  675, 1008,  965754,  965767,  965788, 13, 21, 19, }, // Irkutsk
    {  675, 1016,  965807,  965819,  965839, 12, 20, 19, }, // Israel
    {  675, 1023,  965858,  965868,  965886, 10, 18, 17, }, // Japan
    {  675, 1047,       0,  965903,       0,  0, 14,  0, }, // Kazakhstan
    {  675, 1058,       0,  965917,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  675, 1077,       0,  965940,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  675, 1106,  965964,  965975,  965994, 11, 19, 18, }, // Korea
    {  675, 1112,       0,  966012,       0,  0, 11,  0, }, // Kosrae
    {  675, 1119,  966023,  966038,  966061, 15, 23, 21, }, // Krasnoyarsk
    {  675, 1151,       0,  966082,       0,  0, 15,  0, }, // Kyrgystan
    {  675, 1175,       0,  966097,       0,  0, 13,  0, }, // Line_Islands
    {  675, 1188,  966110,  966123,  966144, 13, 21, 20, }, // Lord_Howe
    {  675, 1204,  966164,  966176,  966196, 12, 20, 19, }, // Magadan
    {  675, 1219,       0,  966215,       0,  0, 12,  0, }, // Malaysia
    {  675, 1228,       0,  966227,       0,  0, 11,  0, }, // Maldives
    {  675, 1237,       0,  966238,       0,  0, 12,  0, }, // Marquesas
    {  675, 1247,       0,  966250,       0,  0, 16,  0, }, // Marshall_Islands
    {  675, 1264,  966266,  966278,  966297, 12, 19, 18, }, // Mauritius
    {  675, 1274,       0,  966315,       0,  0, 11,  0, }, // Mawson
    {  675, 1281,  966326,  966347,  966376, 21, 29, 28, }, // Mexico_Pacific
    {  675, 1296,  966404,  966418,  966440, 14, 22, 20, }, // Mongolia
    {  675, 1305,  966460,  966470,  966488, 10, 18, 16, }, // Moscow
    {  675, 1312,       0,  966504,       0,  0, 13,  0, }, // Myanmar
    {  675, 1320,       0,  966517,       0,  0, 10,  0, }, // Nauru
    {  675, 1326,       0,  966527,       0,  0, 10,  0, }, // Nepal
    {  675, 1332,  966537,  966555,  966581, 18, 26, 24, }, // New_Caledonia
    {  675, 1346,  966605,  966619,  966641, 14, 22, 21, }, // New_Zealand
    {  675, 1358,  966662,  966680,  966706, 18, 26, 25, }, // Newfoundland
    {  675, 1371,       0,  966731,       0,  0,  8,  0, }, // Niue
    {  675, 1376,  966739,  966754,  966777, 15, 23, 22, }, // Norfolk
    {  675, 1384,  966799,  966822,  966853, 23, 31, 29, }, // Noronha
    {  675, 1406,  966882,  966898,  966922, 16, 24, 22, }, // Novosibirsk
    {  675, 1418,  966944,  966954,  966972, 10, 18, 16, }, // Omsk
    {  675, 1428,  966988,  967001,  967022, 13, 21, 19, }, // Pakistan
    {  675, 1437,       0,  967041,       0,  0, 10,  0, }, // Palau
    {  675, 1443,       0,  967051,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  675, 1460,  967071,  967084,  967105, 13, 21, 19, }, // Paraguay
    {  675, 1469,  967124,  967133,  967150,  9, 17, 15, }, // Peru
    {  675, 1474,  967165,  967177,  967197, 12, 20, 18, }, // Philippines
    {  675, 1486,       0,  967215,       0,  0, 19,  0, }, // Phoenix_Islands
    {  675, 1502,  967234,  967262,  967298, 28, 36, 35, }, // Pierre_Miquelon
    {  675, 1518,       0,  967333,       0,  0, 13,  0, }, // Pitcairn
    {  675, 1527,       0,  967346,       0,  0, 11,  0, }, // Ponape
    {  675, 1534,       0,  967357,       0,  0, 15,  0, }, // Pyongyang
    {  675, 1554,       0,  967372,       0,  0, 12,  0, }, // Reunion
    {  675, 1562,       0,  967384,       0,  0, 11,  0, }, // Rothera
    {  675, 1570,  967395,  967407,  967427, 12, 20, 18, }, // Sakhalin
    {  675, 1596,  967445,  967456,  967475, 11, 19, 13, }, // Samoa
    {  675, 1602,       0,  967488,       0,  0, 11,  0, }, // Seychelles
    {  675, 1624,       0,  967499,       0,  0, 21,  0, }, // Singapore
    {  675, 1634,       0,  967520,       0,  0, 19,  0, }, // Solomon
    {  675, 1642,       0,  967539,       0,  0, 17,  0, }, // South_Georgia
    {  675, 1656,       0,  967556,       0,  0, 12,  0, }, // Suriname
    {  675, 1676,       0,  967568,       0,  0,  9,  0, }, // Syowa
    {  675, 1682,       0,  967577,       0,  0, 11,  0, }, // Tahiti
    {  675, 1689,  967588,  967598,  967616, 10, 18, 12, }, // Taipei
    {  675, 1696,       0,  967628,       0,  0, 15,  0, }, // Tajikistan
    {  675, 1724,       0,  967643,       0,  0, 12,  0, }, // Tokelau
    {  675, 1732,  967655,  967665,  967683, 10, 18, 16, }, // Tonga
    {  675, 1738,       0,  967699,       0,  0,  8,  0, }, // Truk
    {  675, 1750,  967707,  967724,  967748, 17, 24, 23, }, // Turkmenistan
    {  675, 1763,       0,  967771,       0,  0, 11,  0, }, // Tuvalu
    {  675, 1777,  967782,  967793,  967811, 11, 18, 17, }, // Uruguay
    {  675, 1792,  967828,  967843,  967866, 15, 23, 20, }, // Uzbekistan
    {  675, 1803,  967886,  967898,  967918, 12, 20, 18, }, // Vanuatu
    {  675, 1811,       0,  967936,       0,  0, 14,  0, }, // Venezuela
    {  675, 1821,  967950,  967966,  967990, 16, 24, 22, }, // Vladivostok
    {  675, 1833,  968012,  968026,  968048, 14, 22, 20, }, // Volgograd
    {  675, 1843,       0,  967955,       0,  0, 11,  0, }, // Vostok
    {  675, 1850,       0,  968068,       0,  0, 14,  0, }, // Wake
    {  675, 1855,       0,  968082,       0,  0, 16,  0, }, // Wallis
    {  675, 1862,  968098,  968111,  968132, 13, 21, 19, }, // Yakutsk
    {  675, 1870,  968151,  968169,  968195, 18, 26, 24, }, // Yekaterinburg
    {  676,    5,       0,  968219,       0,  0, 16,  0, }, // Afghanistan Kuvi/Devanagari/India
    {  676,   17,       0,  968235,       0,  0, 19,  0, }, // Africa_Central
    {  676,   32,       0,  968254,       0,  0, 23,  0, }, // Africa_Eastern
    {  676,   65,       0,  968277,       0,  0, 27,  0, }, // Africa_Southern
    {  676,   81,  968304,  968323,  968350, 19, 27, 30, }, // Africa_Western
    {  676,  107,  968380,  968392,  968413, 12, 21, 20, }, // Alaska
    {  676,  135,  968433,  968444,  968463, 11, 19, 22, }, // Amazon
    {  676,  142,  968485,  968496,  968515, 11, 19, 18, }, // America_Central
    {  676,  158,  968533,  968548,  968571, 15, 23, 22, }, // America_Eastern
    {  676,  174,  968593,  968604,  968623, 11, 19, 18, }, // America_Mountain
    {  676,  191,  968641,  968653,  968673, 12, 20, 19, }, // America_Pacific
    {  676,  214,  968692,  968701,  968718,  9, 17, 16, }, // Apia
    {  676,  232,  968734,  968742,  968758,  8, 16, 15, }, // Arabian
    {  676,  240,  968773,  968788,  968811, 15, 23, 26, }, // Argentina
    {  676,  250,  968837,  968865,  968901, 28, 36, 39, }, // Argentina_Western
    {  676,  268,  968940,  968954,  968976, 14, 22, 25, }, // Armenia
    {  676,  286,  969001,  969014,  969035, 13, 21, 20, }, // Atlantic
    {  676,  295,  969055,  969080,  969114, 25, 34, 33, }, // Australia_Central
    {  676,  313,  969147,  969186,  969233, 39, 47, 46, }, // Australia_CentralWestern
    {  676,  338,  969279,  969306,  969344, 27, 38, 37, }, // Australia_Eastern
    {  676,  356,  969381,  969410,  969449, 29, 39, 38, }, // Australia_Western
    {  676,  374,  969487,  969501,  969523, 14, 22, 25, }, // Azerbaijan
    {  676,  385,  969548,  969560,  969580, 12, 20, 23, }, // Azores
    {  676,  397,  969603,  969618,  969641, 15, 23, 26, }, // Bangladesh
    {  676,  415,       0,  969667,       0,  0, 10,  0, }, // Bhutan
    {  676,  422,       0,  969677,       0,  0, 13,  0, }, // Bolivia
    {  676,  437,  969690,  969706,  969730, 16, 24, 27, }, // Brasilia
    {  676,  454,       0,  969757,       0,  0, 23,  0, }, // Brunei
    {  676,  461,  969780,  969793,  969814, 13, 21, 24, }, // Cape_Verde
    {  676,  478,       0,  969838,       0,  0, 19,  0, }, // Chamorro
    {  676,  487,  969857,  969869,  969889, 12, 20, 19, }, // Chatham
    {  676,  495,  969908,  969917,  969934,  9, 17, 20, }, // Chile
    {  676,  501,  969954,  969962,  969978,  8, 16, 15, }, // China
    {  676,  507,       0,  969993,       0,  0, 18,  0, }, // Christmas
    {  676,  517,       0,  970011,       0,  0, 16,  0, }, // Cocos
    {  676,  523,  970027,  970040,  970061, 13, 21, 24, }, // Colombia
    {  676,  532,  970085,  970099,  970121, 14, 22, 29, }, // Cook
    {  676,  537,  970150,  970161,  970180, 11, 19, 18, }, // Cuba
    {  676,  548,       0,  970198,       0,  0, 10,  0, }, // Davis
    {  676,  564,       0,  970208,       0,  0, 24,  0, }, // DumontDUrville
    {  676,  601,       0,  970232,       0,  0, 21,  0, }, // East_Timor
    {  676,  612,  970253,  970269,  970293, 16, 24, 27, }, // Easter
    {  676,  619,       0,  970320,       0,  0, 13,  0, }, // Ecuador
    {  676,  627,  970333,  970353,  970381, 20, 28, 31, }, // Europe_Central
    {  676,  642,  970412,  970436,  970468, 24, 32, 35, }, // Europe_Eastern
    {  676,  657,       0,  970503,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {  676,  680,  970531,  970557,  970591, 26, 34, 37, }, // Europe_Western
    {  676,  695,  970628,  970648,  970676, 20, 28, 31, }, // Falkland
    {  676,  704,  970707,  970717,  970735, 10, 18, 21, }, // Fiji
    {  676,  709,       0,  970756,       0,  0, 19,  0, }, // French_Guiana
    {  676,  723,       0,  970775,       0,  0, 35,  0, }, // French_Southern
    {  676,  746,       0,  970810,       0,  0, 19,  0, }, // Galapagos
    {  676,  756,       0,  970829,       0,  0, 15,  0, }, // Gambier
    {  676,  764,  970844,  970857,  970878, 13, 21, 24, }, // Georgia
    {  676,  772,       0,  970902,       0,  0, 19,  0, }, // Gilbert_Islands
    {  676,  788,       0,  970921,       0,  0, 17,  0, }, // GMT
    {  676,  830,  970938,  970963,  970996, 25, 33, 36, }, // Greenland_Eastern
    {  676,  848,  971032,  971059,  971094, 27, 35, 38, }, // Greenland_Western
    {  676,  871,       0,  971132,       0,  0, 17,  0, }, // Gulf
    {  676,  876,       0,  970764,       0,  0, 11,  0, }, // Guyana
    {  676,  883,  971149,  971167,  971193, 18, 26, 25, }, // Hawaii_Aleutian
    {  676,  899,  971218,  971232,  971254, 14, 22, 25, }, // Hong_Kong
    {  676,  909,  971279,  971289,  971307, 10, 18, 21, }, // Hovd
    {  676,  914,       0,  971328,       0,  0, 19,  0, }, // India
    {  676,  920,       0,  971347,       0,  0, 24,  0, }, // Indian_Ocean
    {  676,  933,       0,  971371,       0,  0, 14,  0, }, // Indochina
    {  676,  943,       0,  971385,       0,  0, 20,  0, }, // Indonesia_Central
    {  676,  961,       0,  971405,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  676,  979,       0,  971431,       0,  0, 28,  0, }, // Indonesia_Western
    {  676,  997,  971459,  971468,  971485,  9, 17, 17, }, // Iran
    {  676, 1008,  971502,  971517,  971540, 15, 23, 26, }, // Irkutsk
    {  676, 1016,  971566,  971578,  971598, 12, 20, 19, }, // Israel
    {  676, 1023,  971617,  971627,  971645, 10, 18, 17, }, // Japan
    {  676, 1047,       0,  971662,       0,  0, 15,  0, }, // Kazakhstan
    {  676, 1058,       0,  971677,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  676, 1077,       0,  971703,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  676, 1106,  971731,  971744,  971765, 13, 21, 20, }, // Korea
    {  676, 1112,       0,  971785,       0,  0, 11,  0, }, // Kosrae
    {  676, 1119,  971796,  971816,  971844, 20, 28, 31, }, // Krasnoyarsk
    {  676, 1151,       0,  971875,       0,  0, 17,  0, }, // Kyrgystan
    {  676, 1175,       0,  971892,       0,  0, 15,  0, }, // Line_Islands
    {  676, 1188,  971907,  971922,  971945, 15, 23, 22, }, // Lord_Howe
    {  676, 1204,  971967,  971979,  971999, 12, 20, 23, }, // Magadan
    {  676, 1219,       0,  972022,       0,  0, 12,  0, }, // Malaysia
    {  676, 1228,       0,  972034,       0,  0, 11,  0, }, // Maldives
    {  676, 1237,       0,  972045,       0,  0, 13,  0, }, // Marquesas
    {  676, 1247,       0,  972058,       0,  0, 17,  0, }, // Marshall_Islands
    {  676, 1264,  972075,  972086,  972105, 11, 19, 22, }, // Mauritius
    {  676, 1274,       0,  972127,       0,  0, 11,  0, }, // Mawson
    {  676, 1281,  972138,  972161,  972192, 23, 31, 30, }, // Mexico_Pacific
    {  676, 1296,  972222,  972236,  972258, 14, 22, 25, }, // Mongolia
    {  676, 1305,  972283,  972294,  972313, 11, 19, 22, }, // Moscow
    {  676, 1312,       0,  972335,       0,  0, 13,  0, }, // Myanmar
    {  676, 1320,       0,  972348,       0,  0, 10,  0, }, // Nauru
    {  676, 1326,       0,  972358,       0,  0, 10,  0, }, // Nepal
    {  676, 1332,  972368,  972388,  972416, 20, 28, 31, }, // New_Caledonia
    {  676, 1346,  972447,  972463,  972487, 16, 24, 23, }, // New_Zealand
    {  676, 1358,  972510,  972529,  972556, 19, 27, 26, }, // Newfoundland
    {  676, 1371,       0,  972582,       0,  0,  9,  0, }, // Niue
    {  676, 1376,  972591,  972609,  972635, 18, 26, 25, }, // Norfolk
    {  676, 1384,  972660,  972688,  972724, 28, 36, 39, }, // Noronha
    {  676, 1406,  972763,  972781,  972807, 18, 26, 29, }, // Novosibirsk
    {  676, 1418,  972836,  972847,  972866, 11, 19, 22, }, // Omsk
    {  676, 1428,  972888,  972902,  972924, 14, 22, 25, }, // Pakistan
    {  676, 1437,       0,  972949,       0,  0,  9,  0, }, // Palau
    {  676, 1443,       0,  972958,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  676, 1460,  972978,  972991,  973012, 13, 21, 24, }, // Paraguay
    {  676, 1469,  973036,  973045,  973062,  9, 17, 20, }, // Peru
    {  676, 1474,  973082,  973095,  973116, 13, 21, 24, }, // Philippines
    {  676, 1486,       0,  973140,       0,  0, 19,  0, }, // Phoenix_Islands
    {  676, 1502,  973159,  973189,  973227, 30, 38, 37, }, // Pierre_Miquelon
    {  676, 1518,       0,  973264,       0,  0, 13,  0, }, // Pitcairn
    {  676, 1527,       0,  973277,       0,  0, 11,  0, }, // Ponape
    {  676, 1534,       0,  973288,       0,  0, 15,  0, }, // Pyongyang
    {  676, 1554,       0,  973303,       0,  0, 13,  0, }, // Reunion
    {  676, 1562,       0,  973316,       0,  0, 11,  0, }, // Rothera
    {  676, 1570,  973327,  973338,  973357, 11, 19, 22, }, // Sakhalin
    {  676, 1596,  973379,  973388,  973405,  9, 17, 16, }, // Samoa
    {  676, 1602,       0,  973421,       0,  0, 12,  0, }, // Seychelles
    {  676, 1624,       0,  973433,       0,  0, 13,  0, }, // Singapore
    {  676, 1634,       0,  973446,       0,  0, 17,  0, }, // Solomon
    {  676, 1642,       0,  973463,       0,  0, 21,  0, }, // South_Georgia
    {  676, 1656,       0,  973484,       0,  0, 12,  0, }, // Suriname
    {  676, 1676,       0,  973496,       0,  0, 11,  0, }, // Syowa
    {  676, 1682,       0,  973507,       0,  0, 11,  0, }, // Tahiti
    {  676, 1689,  973518,  973528,  973546, 10, 18, 17, }, // Taipei
    {  676, 1696,       0,  973563,       0,  0, 16,  0, }, // Tajikistan
    {  676, 1724,       0,  973579,       0,  0, 12,  0, }, // Tokelau
    {  676, 1732,  973591,  973601,  973619, 10, 18, 21, }, // Tonga
    {  676, 1738,       0,  973640,       0,  0,  8,  0, }, // Truk
    {  676, 1750,  973648,  973667,  973694, 19, 27, 30, }, // Turkmenistan
    {  676, 1763,       0,  973724,       0,  0, 11,  0, }, // Tuvalu
    {  676, 1777,  973735,  973747,  973767, 12, 20, 23, }, // Uruguay
    {  676, 1792,  973790,  973808,  973834, 18, 26, 29, }, // Uzbekistan
    {  676, 1803,  973863,  973874,  973893, 11, 19, 22, }, // Vanuatu
    {  676, 1811,       0,  973915,       0,  0, 15,  0, }, // Venezuela
    {  676, 1821,  973930,  973948,  973974, 18, 26, 29, }, // Vladivostok
    {  676, 1833,  974003,  974019,  974043, 16, 24, 27, }, // Volgograd
    {  676, 1843,       0,  973936,       0,  0, 12,  0, }, // Vostok
    {  676, 1850,       0,  974070,       0,  0, 14,  0, }, // Wake
    {  676, 1855,       0,  974084,       0,  0, 25,  0, }, // Wallis
    {  676, 1862,  974109,  974123,  974145, 14, 22, 25, }, // Yakutsk
    {  676, 1870,  974170,  974188,  974214, 18, 26, 29, }, // Yekaterinburg
    {  677,    5,       0,  974243,       0,  0, 16,  0, }, // Afghanistan Kuvi/Odia/India
    {  677,   17,       0,  974259,       0,  0, 19,  0, }, // Africa_Central
    {  677,   32,       0,  974278,       0,  0, 22,  0, }, // Africa_Eastern
    {  677,   65,       0,  974300,       0,  0, 25,  0, }, // Africa_Southern
    {  677,   81,  974325,  974351,  974385, 26, 34, 32, }, // Africa_Western
    {  677,  107,  974417,  974429,  974449, 12, 20, 20, }, // Alaska
    {  677,  135,  974469,  974479,  974497, 10, 18, 16, }, // Amazon
    {  677,  142,  974513,  974524,  974543, 11, 19, 19, }, // America_Central
    {  677,  158,  974562,  974582,  974610, 20, 28, 28, }, // America_Eastern
    {  677,  174,  974638,  974648,  974666, 10, 18, 18, }, // America_Mountain
    {  677,  191,  974684,  974697,  974718, 13, 21, 21, }, // America_Pacific
    {  677,  214,  974739,  974748,  974765,  9, 17, 17, }, // Apia
    {  677,  232,  974782,  974794,  974814, 12, 20, 19, }, // Arabian
    {  677,  240,  974833,  974848,  974871, 15, 23, 21, }, // Argentina
    {  677,  250,  974892,  974922,  974960, 30, 38, 36, }, // Argentina_Western
    {  677,  268,  974996,  975009,  975030, 13, 21, 19, }, // Armenia
    {  677,  286,  975049,  975064,  975087, 15, 23, 22, }, // Atlantic
    {  677,  295,  975109,  975131,  975163, 22, 32, 31, }, // Australia_Central
    {  677,  313,  975194,  975231,  975276, 37, 45, 44, }, // Australia_CentralWestern
    {  677,  338,  975320,  975344,  975378, 24, 34, 33, }, // Australia_Eastern
    {  677,  356,  975411,  975439,  975477, 28, 38, 37, }, // Australia_Western
    {  677,  374,  975514,  975529,  975552, 15, 23, 21, }, // Azerbaijan
    {  677,  385,  975573,  975583,  975601, 10, 18, 16, }, // Azores
    {  677,  397,  975617,  975633,  975657, 16, 24, 22, }, // Bangladesh
    {  677,  415,       0,  975679,       0,  0, 10,  0, }, // Bhutan
    {  677,  422,       0,  975689,       0,  0, 10,  0, }, // Bolivia
    {  677,  437,  975699,  975714,  975734, 15, 20, 21, }, // Brasilia
    {  677,  454,       0,  975755,       0,  0, 22,  0, }, // Brunei
    {  677,  461,  975777,  975790,  975811, 13, 21, 19, }, // Cape_Verde
    {  677,  478,       0,  975830,       0,  0, 16,  0, }, // Chamorro
    {  677,  487,  975846,  975856,  975874, 10, 18, 18, }, // Chatham
    {  677,  495,  975892,  975901,  975918,  9, 17, 15, }, // Chile
    {  677,  501,  975933,  975941,  975957,  8, 16, 15, }, // China
    {  677,  507,       0,  975972,       0,  0, 17,  0, }, // Christmas
    {  677,  517,       0,  975989,       0,  0, 12,  0, }, // Cocos
    {  677,  523,  976001,  976014,  976035, 13, 21, 19, }, // Colombia
    {  677,  532,  976054,  976066,  976086, 12, 20, 24, }, // Cook
    {  677,  537,  976110,  976121,  976140, 11, 19, 19, }, // Cuba
    {  677,  548,       0,  976159,       0,  0, 11,  0, }, // Davis
    {  677,  564,       0,  976170,       0,  0, 24,  0, }, // DumontDUrville
    {  677,  601,       0,  976194,       0,  0, 18,  0, }, // East_Timor
    {  677,  612,  976212,  976226,  976248, 14, 22, 20, }, // Easter
    {  677,  619,       0,  976268,       0,  0, 12,  0, }, // Ecuador
    {  677,  627,  976280,  976301,  976330, 21, 29, 27, }, // Europe_Central
    {  677,  642,  976357,  976378,  976408, 21, 30, 28, }, // Europe_Eastern
    {  677,  657,       0,  976436,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  677,  680,  976459,  976483,  976517, 24, 34, 30, }, // Europe_Western
    {  677,  695,  976547,  976564,  976591, 17, 27, 25, }, // Falkland
    {  677,  704,  976616,  976625,  976642,  9, 17, 15, }, // Fiji
    {  677,  709,       0,  976657,       0,  0, 20,  0, }, // French_Guiana
    {  677,  723,       0,  976677,       0,  0, 32,  0, }, // French_Southern
    {  677,  746,       0,  976709,       0,  0, 14,  0, }, // Galapagos
    {  677,  756,       0,  976723,       0,  0, 12,  0, }, // Gambier
    {  677,  764,  976735,  976747,  976767, 12, 20, 17, }, // Georgia
    {  677,  772,       0,  976784,       0,  0, 21,  0, }, // Gilbert_Islands
    {  677,  788,       0,  976805,       0,  0, 17,  0, }, // GMT
    {  677,  830,  976822,  976847,  976880, 25, 33, 31, }, // Greenland_Eastern
    {  677,  848,  976911,  976940,  976977, 29, 37, 35, }, // Greenland_Western
    {  677,  871,       0,  977012,       0,  0, 17,  0, }, // Gulf
    {  677,  876,       0,  976666,       0,  0, 11,  0, }, // Guyana
    {  677,  883,  977029,  977048,  977075, 19, 27, 27, }, // Hawaii_Aleutian
    {  677,  899,  977102,  977116,  977138, 14, 22, 20, }, // Hong_Kong
    {  677,  909,  977158,  977166,  977182,  8, 16, 14, }, // Hovd
    {  677,  914,       0,  977196,       0,  0, 17,  0, }, // India
    {  677,  920,       0,  977213,       0,  0, 23,  0, }, // Indian_Ocean
    {  677,  933,       0,  977236,       0,  0, 13,  0, }, // Indochina
    {  677,  943,       0,  977249,       0,  0, 21,  0, }, // Indonesia_Central
    {  677,  961,       0,  977270,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  677,  979,       0,  977293,       0,  0, 26,  0, }, // Indonesia_Western
    {  677,  997,  977319,  977328,  977345,  9, 17, 18, }, // Iran
    {  677, 1008,  977363,  977376,  977397, 13, 21, 19, }, // Irkutsk
    {  677, 1016,  977416,  977427,  977446, 11, 19, 18, }, // Israel
    {  677, 1023,  977464,  977474,  977492, 10, 18, 17, }, // Japan
    {  677, 1047,       0,  977509,       0,  0, 15,  0, }, // Kazakhstan
    {  677, 1058,       0,  977524,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  677, 1077,       0,  977548,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  677, 1106,  977575,  977585,  977603, 10, 18, 17, }, // Korea
    {  677, 1112,       0,  977620,       0,  0, 10,  0, }, // Kosrae
    {  677, 1119,  977630,  977649,  977676, 19, 27, 24, }, // Krasnoyarsk
    {  677, 1151,       0,  977700,       0,  0, 16,  0, }, // Kyrgystan
    {  677, 1175,       0,  977716,       0,  0, 15,  0, }, // Line_Islands
    {  677, 1188,  977731,  977744,  977765, 13, 21, 21, }, // Lord_Howe
    {  677, 1204,  977786,  977802,  977821, 16, 19, 18, }, // Magadan
    {  677, 1219,       0,  977839,       0,  0, 13,  0, }, // Malaysia
    {  677, 1228,       0,  977852,       0,  0, 14,  0, }, // Maldives
    {  677, 1237,       0,  977866,       0,  0, 12,  0, }, // Marquesas
    {  677, 1247,       0,  977878,       0,  0, 16,  0, }, // Marshall_Islands
    {  677, 1264,  977894,  977905,  977924, 11, 19, 17, }, // Mauritius
    {  677, 1274,       0,  977941,       0,  0, 10,  0, }, // Mawson
    {  677, 1281,  977951,  977973,  978003, 22, 30, 29, }, // Mexico_Pacific
    {  677, 1296,  978032,  978045,  978066, 13, 21, 19, }, // Mongolia
    {  677, 1305,  978085,  978094,  978111,  9, 17, 15, }, // Moscow
    {  677, 1312,       0,  978126,       0,  0, 13,  0, }, // Myanmar
    {  677, 1320,       0,  978139,       0,  0,  9,  0, }, // Nauru
    {  677, 1326,       0,  978148,       0,  0, 10,  0, }, // Nepal
    {  677, 1332,  978158,  978177,  978204, 19, 27, 25, }, // New_Caledonia
    {  677, 1346,  978229,  978245,  978269, 16, 24, 23, }, // New_Zealand
    {  677, 1358,  978292,  978312,  978340, 20, 28, 27, }, // Newfoundland
    {  677, 1371,       0,  978367,       0,  0,  9,  0, }, // Niue
    {  677, 1376,  978376,  978391,  978414, 15, 23, 22, }, // Norfolk
    {  677, 1384,  978436,  978457,  978486, 21, 29, 26, }, // Noronha
    {  677, 1406,  978512,  978528,  978552, 16, 24, 20, }, // Novosibirsk
    {  677, 1418,  978572,  978582,  978600, 10, 18, 16, }, // Omsk
    {  677, 1428,  978616,  978630,  978652, 14, 22, 20, }, // Pakistan
    {  677, 1437,       0,  978672,       0,  0,  9,  0, }, // Palau
    {  677, 1443,       0,  978681,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  677, 1460,  978702,  978715,  978736, 13, 21, 19, }, // Paraguay
    {  677, 1469,  978755,  978764,  978781,  9, 17, 15, }, // Peru
    {  677, 1474,  978796,  978808,  978828, 12, 20, 18, }, // Philippines
    {  677, 1486,       0,  978846,       0,  0, 18,  0, }, // Phoenix_Islands
    {  677, 1502,  978864,  978895,  978934, 31, 39, 38, }, // Pierre_Miquelon
    {  677, 1518,       0,  978972,       0,  0, 12,  0, }, // Pitcairn
    {  677, 1527,       0,  978984,       0,  0, 10,  0, }, // Ponape
    {  677, 1534,       0,  978994,       0,  0, 16,  0, }, // Pyongyang
    {  677, 1554,       0,  979010,       0,  0, 13,  0, }, // Reunion
    {  677, 1562,       0,  979023,       0,  0, 10,  0, }, // Rothera
    {  677, 1570,  979033,  979044,  979063, 11, 19, 17, }, // Sakhalin
    {  677, 1596,  979080,  979089,  979106,  9, 17, 16, }, // Samoa
    {  677, 1602,       0,  979122,       0,  0, 12,  0, }, // Seychelles
    {  677, 1624,       0,  979134,       0,  0, 22,  0, }, // Singapore
    {  677, 1634,       0,  979156,       0,  0, 15,  0, }, // Solomon
    {  677, 1642,       0,  979171,       0,  0, 17,  0, }, // South_Georgia
    {  677, 1656,       0,  979188,       0,  0, 12,  0, }, // Suriname
    {  677, 1676,       0,  979200,       0,  0,  8,  0, }, // Syowa
    {  677, 1682,       0,  979208,       0,  0, 11,  0, }, // Tahiti
    {  677, 1689,  979219,  979229,  979246, 10, 17, 17, }, // Taipei
    {  677, 1696,       0,  979263,       0,  0, 16,  0, }, // Tajikistan
    {  677, 1724,       0,  979279,       0,  0, 11,  0, }, // Tokelau
    {  677, 1732,  979290,  979300,  979318, 10, 18, 16, }, // Tonga
    {  677, 1738,       0,  979334,       0,  0,  8,  0, }, // Truk
    {  677, 1750,  979342,  979361,  979388, 19, 27, 25, }, // Turkmenistan
    {  677, 1763,       0,  979413,       0,  0, 11,  0, }, // Tuvalu
    {  677, 1777,  979424,  979436,  979456, 12, 20, 18, }, // Uruguay
    {  677, 1792,  979474,  979491,  979516, 17, 25, 23, }, // Uzbekistan
    {  677, 1803,  979539,  979550,  979569, 11, 19, 17, }, // Vanuatu
    {  677, 1811,       0,  979586,       0,  0, 14,  0, }, // Venezuela
    {  677, 1821,  979600,  979616,  979640, 16, 24, 22, }, // Vladivostok
    {  677, 1833,  979662,  979676,  979698, 14, 22, 20, }, // Volgograd
    {  677, 1843,       0,  979606,       0,  0, 10,  0, }, // Vostok
    {  677, 1850,       0,  979718,       0,  0, 12,  0, }, // Wake
    {  677, 1855,       0,  979730,       0,  0, 22,  0, }, // Wallis
    {  677, 1862,  979752,  979765,  979786, 13, 21, 19, }, // Yakutsk
    {  677, 1870,  979805,  979823,  979849, 18, 26, 24, }, // Yekaterinburg
    {  678,    5,       0,  780673,       0,  0, 17,  0, }, // Afghanistan Kuvi/Telugu/India
    {  678,   17,       0,  780690,       0,  0, 21,  0, }, // Africa_Central
    {  678,   32,       0,  780711,       0,  0, 19,  0, }, // Africa_Eastern
    {  678,   65,       0,  780730,       0,  0, 29,  0, }, // Africa_Southern
    {  678,   81,  780759,  780778,  780807, 19, 29, 25, }, // Africa_Western
    {  678,  107,  780832,  780844,  780866, 12, 22, 26, }, // Alaska
    {  678,  135,  780944,  780956,  780978, 12, 22, 18, }, // Amazon
    {  678,  142,  780996,  781006,  781026, 10, 20, 24, }, // America_Central
    {  678,  158,  781050,  781061,  781082, 11, 21, 25, }, // America_Eastern
    {  678,  174,  781107,  781121,  781145, 14, 24, 28, }, // America_Mountain
    {  678,  191,  781173,  781185,  781207, 12, 22, 26, }, // America_Pacific
    {  678,  214,  781288,  781298,  781318, 10, 20, 15, }, // Apia
    {  678,  232,  781437,  781450,  781473, 13, 23, 27, }, // Arabian
    {  678,  240,  781500,  781515,  781540, 15, 25, 21, }, // Argentina
    {  678,  250,  781561,  781583,  781615, 22, 32, 28, }, // Argentina_Western
    {  678,  268,  781643,  781657,  781681, 14, 24, 20, }, // Armenia
    {  678,  286,  781701,  781716,  781741, 15, 25, 29, }, // Atlantic
    {  678,  295,  781770,  781792,  781824, 22, 32, 36, }, // Australia_Central
    {  678,  313,  781860,  781889,  781929, 29, 40, 43, }, // Australia_CentralWestern
    {  678,  338,  781972,  781995,  782029, 23, 34, 38, }, // Australia_Eastern
    {  678,  356,  782067,  781895,  782090, 23, 34, 38, }, // Australia_Western
    {  678,  374,  782128,  782143,  782168, 15, 25, 21, }, // Azerbaijan
    {  678,  385,  782189,  782201,  782223, 12, 22, 18, }, // Azores
    {  678,  397,  782241,  782256,  782281, 15, 25, 21, }, // Bangladesh
    {  678,  415,       0,  782302,       0,  0, 11,  0, }, // Bhutan
    {  678,  422,       0,  782313,       0,  0, 13,  0, }, // Bolivia
    {  678,  437,  782326,  782341,  782366, 15, 25, 21, }, // Brasilia
    {  678,  454,       0,  782387,       0,  0, 20,  0, }, // Brunei
    {  678,  461,  782407,  782423,  782449, 16, 26, 22, }, // Cape_Verde
    {  678,  478,       0,  782471,       0,  0, 22,  0, }, // Chamorro
    {  678,  487,  782493,  782503,  782523, 10, 20, 24, }, // Chatham
    {  678,  495,  782547,  782556,  782575,  9, 19, 15, }, // Chile
    {  678,  501,  782590,  782599,  782618,  9, 19, 23, }, // China
    {  678,  507,       0,  782641,       0,  0, 19,  0, }, // Christmas
    {  678,  517,       0,  782660,       0,  0, 17,  0, }, // Cocos
    {  678,  523,  782677,  782690,  782713, 13, 23, 19, }, // Colombia
    {  678,  532,  782732,  782747,  782772, 15, 25, 26, }, // Cook
    {  678,  537,  782798,  782809,  782830, 11, 21, 25, }, // Cuba
    {  678,  548,       0,  782855,       0,  0, 11,  0, }, // Davis
    {  678,  564,       0,  782866,       0,  0, 27,  0, }, // DumontDUrville
    {  678,  601,       0,  782893,       0,  0, 18,  0, }, // East_Timor
    {  678,  612,  782911,  782927,  782953, 16, 26, 22, }, // Easter
    {  678,  619,       0,  782975,       0,  0, 13,  0, }, // Ecuador
    {  678,  627,  782988,  783011,  783044, 23, 33, 29, }, // Europe_Central
    {  678,  642,  783073,  783094,  783125, 21, 31, 27, }, // Europe_Eastern
    {  678,  657,       0,  783152,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  678,  680,  783179,  783200,  783231, 21, 31, 27, }, // Europe_Western
    {  678,  695,  783258,  783281,  783314, 23, 33, 29, }, // Falkland
    {  678,  704,  783343,  783352,  783371,  9, 19, 15, }, // Fiji
    {  678,  709,       0,  783386,       0,  0, 18,  0, }, // French_Guiana
    {  678,  723,       0,  783404,       0,  0, 38,  0, }, // French_Southern
    {  678,  746,       0,  783442,       0,  0, 15,  0, }, // Galapagos
    {  678,  756,       0,  783457,       0,  0, 13,  0, }, // Gambier
    {  678,  764,  783470,  783483,  783506, 13, 23, 19, }, // Georgia
    {  678,  772,       0,  783525,       0,  0, 20,  0, }, // Gilbert_Islands
    {  678,  788,       0,  783545,       0,  0, 21,  0, }, // GMT
    {  678,  830,  783566,  783592,  783628, 26, 36, 32, }, // Greenland_Eastern
    {  678,  848,  783660,  783686,  783722, 26, 36, 32, }, // Greenland_Western
    {  678,  871,       0,  783775,       0,  0, 20,  0, }, // Gulf
    {  678,  876,       0,  783394,       0,  0, 10,  0, }, // Guyana
    {  678,  883,  783795,  783816,  783847, 21, 31, 35, }, // Hawaii_Aleutian
    {  678,  899,  783882,  783895,  783918, 13, 23, 19, }, // Hong_Kong
    {  678,  909,  783937,  783948,  783969, 11, 21, 17, }, // Hovd
    {  678,  914,       0,  783986,       0,  0, 22,  0, }, // India
    {  678,  920,       0,  784008,       0,  0, 21,  0, }, // Indian_Ocean
    {  678,  933,       0,  784029,       0,  0, 13,  0, }, // Indochina
    {  678,  943,       0,  784042,       0,  0, 24,  0, }, // Indonesia_Central
    {  678,  961,       0,  784066,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  678,  979,       0,  784088,       0,  0, 22,  0, }, // Indonesia_Western
    {  678,  997,  784110,  784120,  784140, 10, 20, 24, }, // Iran
    {  678, 1008,  784164,  784180,  784206, 16, 26, 22, }, // Irkutsk
    {  678, 1016,  784228,  784242,  784266, 14, 24, 28, }, // Israel
    {  678, 1023,  784294,  784304,  784324, 10, 20, 24, }, // Japan
    {  678, 1047,       0,  784494,       0,  0, 16,  0, }, // Kazakhstan
    {  678, 1058,       0,  784487,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  678, 1077,       0,  784510,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  678, 1106,  784532,  784544,  784566, 12, 22, 26, }, // Korea
    {  678, 1112,       0,  784592,       0,  0, 13,  0, }, // Kosrae
    {  678, 1119,  784605,  784626,  784657, 21, 31, 27, }, // Krasnoyarsk
    {  678, 1151,       0,  784684,       0,  0, 17,  0, }, // Kyrgystan
    {  678, 1175,       0,  784710,       0,  0, 15,  0, }, // Line_Islands
    {  678, 1188,  784725,  784741,  784767, 16, 26, 21, }, // Lord_Howe
    {  678, 1204,  784834,  784845,  784866, 11, 21, 17, }, // Magadan
    {  678, 1219,       0,  784883,       0,  0, 12,  0, }, // Malaysia
    {  678, 1228,       0,  784895,       0,  0, 14,  0, }, // Maldives
    {  678, 1237,       0,  784909,       0,  0, 17,  0, }, // Marquesas
    {  678, 1247,       0,  784926,       0,  0, 18,  0, }, // Marshall_Islands
    {  678, 1264,  784944,  784956,  784978, 12, 22, 18, }, // Mauritius
    {  678, 1274,       0,  784996,       0,  0, 10,  0, }, // Mawson
    {  678, 1281,  785006,  785028,  785060, 22, 32, 36, }, // Mexico_Pacific
    {  678, 1296,  785096,  785111,  785136, 15, 25, 21, }, // Mongolia
    {  678, 1305,  785157,  785168,  785189, 11, 21, 17, }, // Moscow
    {  678, 1312,       0,  785206,       0,  0, 13,  0, }, // Myanmar
    {  678, 1320,       0,  785219,       0,  0,  9,  0, }, // Nauru
    {  678, 1326,       0,  785228,       0,  0, 11,  0, }, // Nepal
    {  678, 1332,  785239,  785259,  785289, 20, 30, 26, }, // New_Caledonia
    {  678, 1346,  785315,  785333,  785361, 18, 28, 32, }, // New_Zealand
    {  678, 1358,  785393,  785415,  785447, 22, 32, 36, }, // Newfoundland
    {  678, 1371,       0,  785483,       0,  0,  9,  0, }, // Niue
    {  678, 1376,  785492,  785510,  785538, 18, 28, 23, }, // Norfolk
    {  678, 1384,  785561,  785587,  785623, 26, 36, 32, }, // Noronha
    {  678, 1406,  785680,  785699,  785728, 19, 29, 25, }, // Novosibirsk
    {  678, 1418,  785753,  785765,  785787, 12, 22, 18, }, // Omsk
    {  678, 1428,  785805,  785820,  785845, 15, 25, 21, }, // Pakistan
    {  678, 1437,       0,  785866,       0,  0, 11,  0, }, // Palau
    {  678, 1443,       0,  785877,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  678, 1460,  785900,  785912,  785934, 12, 22, 18, }, // Paraguay
    {  678, 1469,  785952,  785961,  785980,  9, 19, 15, }, // Peru
    {  678, 1474,  785995,  786010,  786035, 15, 25, 21, }, // Philippines
    {  678, 1486,       0,  786056,       0,  0, 19,  0, }, // Phoenix_Islands
    {  678, 1502,  786075,  786111,  786157, 36, 46, 49, }, // Pierre_Miquelon
    {  678, 1518,       0,  786206,       0,  0, 16,  0, }, // Pitcairn
    {  678, 1527,       0,  786222,       0,  0, 11,  0, }, // Ponape
    {  678, 1534,       0,  786233,       0,  0, 15,  0, }, // Pyongyang
    {  678, 1554,       0,  786309,       0,  0, 14,  0, }, // Reunion
    {  678, 1562,       0,  786323,       0,  0, 11,  0, }, // Rothera
    {  678, 1570,  786334,  786346,  786368, 12, 22, 18, }, // Sakhalin
    {  678, 1596,  786432,  786442,  786462, 10, 20, 24, }, // Samoa
    {  678, 1602,       0,  786486,       0,  0, 13,  0, }, // Seychelles
    {  678, 1624,       0,  786499,       0,  0, 23,  0, }, // Singapore
    {  678, 1634,       0,  786522,       0,  0, 17,  0, }, // Solomon
    {  678, 1642,       0,  786539,       0,  0, 20,  0, }, // South_Georgia
    {  678, 1656,       0,  786559,       0,  0, 13,  0, }, // Suriname
    {  678, 1676,       0,  786572,       0,  0, 11,  0, }, // Syowa
    {  678, 1682,       0,  786583,       0,  0, 10,  0, }, // Tahiti
    {  678, 1689,  786593,  786602,  786621,  9, 19, 23, }, // Taipei
    {  678, 1696,       0,  786644,       0,  0, 16,  0, }, // Tajikistan
    {  678, 1724,       0,  786660,       0,  0, 13,  0, }, // Tokelau
    {  678, 1732,  786673,  786683,  786703, 10, 20, 16, }, // Tonga
    {  678, 1738,       0,  786719,       0,  0,  8,  0, }, // Truk
    {  678, 1750,  786727,  786749,  786781, 22, 32, 28, }, // Turkmenistan
    {  678, 1763,       0,  786809,       0,  0, 11,  0, }, // Tuvalu
    {  678, 1777,  786820,  786832,  786854, 12, 22, 18, }, // Uruguay
    {  678, 1792,  786872,  786890,  786918, 18, 28, 24, }, // Uzbekistan
    {  678, 1803,  786942,  786952,  786972, 10, 20, 16, }, // Vanuatu
    {  678, 1811,       0,  786988,       0,  0, 13,  0, }, // Venezuela
    {  678, 1821,  787001,  787020,  787049, 19, 29, 25, }, // Vladivostok
    {  678, 1833,  787074,  787091,  787118, 17, 27, 23, }, // Volgograd
    {  678, 1843,       0,  787007,       0,  0, 13,  0, }, // Vostok
    {  678, 1850,       0,  787141,       0,  0, 14,  0, }, // Wake
    {  678, 1855,       0,  787155,       0,  0, 24,  0, }, // Wallis
    {  678, 1862,  787179,  787194,  787219, 15, 25, 21, }, // Yakutsk
    {  678, 1870,  787240,  787260,  787290, 20, 30, 26, }, // Yekaterinburg
    {  681,  788,       0,  979873,       0,  0, 20,  0, }, // GMT Swampy Cree/Canadian Aboriginal/Canada
}; // Metazone long name table

// Sorted by locale index, then meta key
static inline constexpr LocaleMetaZoneShortNames localeMetaZoneShortNameTable[] = {
    // locInd, metaKey, (generic, standard, DST){ind, sz}
    {    0,  107,    0,    3,    7,  3,  4,  4, }, // Alaska C/AnyScript/AnyTerritory
    {    0,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {    0,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {    0,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {    0,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {    0,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {    0,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {    0,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {    5,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central Afrikaans/Latin/South Africa
    {    5,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {    5,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {    5,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {   14,  871,    0,   82,    0,  0,  3,  0, }, // Gulf Arabic/Arabic/Egypt
    {   44,  914,    0,   85,    0,  0,  6,  0, }, // India Assamese/Bangla/India
    {   45,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Asturian/Latin/Spain
    {   45,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   45,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   45,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   45,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   45,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   45,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {   45,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   45,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   45,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {   45,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {   56,  914,    0,  112,    0,  0,  3,  0, }, // India Bangla/Bangla/Bangladesh
    {   60,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Basque/Latin/Spain
    {   60,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   60,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   60,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   60,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   60,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   60,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {   60,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   60,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   60,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {   60,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {   66,  914,    0,  112,    0,  0,  3,  0, }, // India Bodo/Devanagari/India
    {   67,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Bosnian/Latin/Bosnia and Herzegovina
    {   67,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   67,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   67,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {   75,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Catalan/Latin/Spain
    {   75,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   75,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   75,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {   83,  914,    0,  112,    0,  0,  3,  0, }, // India Chakma/Chakma/Bangladesh
    {   86,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Cherokee/Cherokee/United States
    {   86,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   86,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   86,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   86,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   86,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   86,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {   94, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia Chinese/Simplified Han/Singapore
    {   94, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {   95,  142,   11,   13,   16,  2,  3,  3, }, // America_Central Chinese/Traditional Han/Hong Kong
    {   95,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   95,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   95,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   96,  142,   11,   13,   16,  2,  3,  3, }, // America_Central Chinese/Traditional Han/Macao
    {   96,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   96,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   96,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   98,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Chinese/Traditional Han/Taiwan
    {   98,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   98,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   98,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   98,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   98,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   98,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {   98,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  100,    0,  121,  121,  129,  8,  8,  9, }, // Acre Chuvash/Cyrillic/Russia
    {  100,    5,    0,  138,    0,  0,  3,  0, }, // Afghanistan
    {  100,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central
    {  100,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  100,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  100,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  100,  107,    0,    3,    7,  3,  4,  4, }, // Alaska
    {  100,  128,  141,  141,  145,  4,  4,  8, }, // Almaty
    {  100,  135,  153,  153,  156,  3,  3,  4, }, // Amazon
    {  100,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  100,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  100,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  100,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  100,  207,  160,  160,  164,  4,  4,  5, }, // Anadyr
    {  100,  214,  169,  169,  183, 14, 14, 18, }, // Apia
    {  100,  219,  201,  201,  211, 10, 10, 14, }, // Aqtau
    {  100,  225,  225,  225,  229,  4,  4,  8, }, // Aqtobe
    {  100,  232,  237,  237,  247, 10, 10, 10, }, // Arabian
    {  100,  240,  257,  257,  260,  3,  3,  4, }, // Argentina
    {  100,  250,  264,  264,  268,  4,  4,  5, }, // Argentina_Western
    {  100,  268,  273,  273,  284, 11, 11, 12, }, // Armenia
    {  100,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  100,  295,  129,  129,  296,  4,  4,  4, }, // Australia_Central
    {  100,  313,  300,  300,  305,  5,  5,  5, }, // Australia_CentralWestern
    {  100,  338,  310,  310,  314,  4,  4,  4, }, // Australia_Eastern
    {  100,  356,  318,  318,  322,  4,  4,  4, }, // Australia_Western
    {  100,  374,  326,  326,  329,  3,  3,  4, }, // Azerbaijan
    {  100,  385,  333,  333,  337,  4,  4,  5, }, // Azores
    {  100,  397,  342,  342,  356, 14, 14, 14, }, // Bangladesh
    {  100,  415,    0,  370,    0,  0,  3,  0, }, // Bhutan
    {  100,  422,    0,  373,    0,  0,  3,  0, }, // Bolivia
    {  100,  437,  376,  376,  379,  3,  3,  4, }, // Brasilia
    {  100,  454,    0,  383,    0,  0,  3,  0, }, // Brunei
    {  100,  461,  386,  386,  389,  3,  3,  4, }, // Cape_Verde
    {  100,  472,    0,  393,    0,  0,  4,  0, }, // Casey
    {  100,  478,    0,  397,    0,  0,  4,  0, }, // Chamorro
    {  100,  487,  401,  401,  406,  5,  5,  5, }, // Chatham
    {  100,  495,  411,  411,  414,  3,  3,  4, }, // Chile
    {  100,  501,  418,  418,  427,  9,  9, 13, }, // China
    {  100,  507,    0,  440,    0,  0,  3,  0, }, // Christmas
    {  100,  517,    0,  443,    0,  0,  3,  0, }, // Cocos
    {  100,  523,  446,  446,  449,  3,  3,  4, }, // Colombia
    {  100,  532,  453,  453,  456,  3,  3,  7, }, // Cook
    {  100,  537,  463,  463,  471,  8,  8,  8, }, // Cuba
    {  100,  548,    0,  479,    0,  0,  4,  0, }, // Davis
    {  100,  564,    0,  483,    0,  0,  4,  0, }, // DumontDUrville
    {  100,  601,    0,  487,    0,  0,  3,  0, }, // East_Timor
    {  100,  612,  490,  490,  494,  4,  4,  5, }, // Easter
    {  100,  619,    0,  499,    0,  0, 11,  0, }, // Ecuador
    {  100,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  100,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  100,  657,    0,  510,    0,  0,  4,  0, }, // Europe_Further_Eastern
    {  100,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  100,  695,  514,  514,  517,  3,  3,  4, }, // Falkland
    {  100,  704,  521,  521,  524,  3,  3, 10, }, // Fiji
    {  100,  709,    0,  534,    0,  0,  3,  0, }, // French_Guiana
    {  100,  723,    0,  537,    0,  0,  4,  0, }, // French_Southern
    {  100,  746,    0,  541,    0,  0,  4,  0, }, // Galapagos
    {  100,  756,    0,  545,    0,  0,  4,  0, }, // Gambier
    {  100,  764,  549,  549,  552,  3,  3,  4, }, // Georgia
    {  100,  772,    0,  556,    0,  0,  3,  0, }, // Gilbert_Islands
    {  100,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  100,  802,  559,  559,  563,  4,  4,  5, }, // Greenland
    {  100,  830,  568,  568,  571,  3,  3,  4, }, // Greenland_Eastern
    {  100,  848,  575,  575,  578,  3,  3,  4, }, // Greenland_Western
    {  100,  866,    0,  582,    0,  0,  8,  0, }, // Guam
    {  100,  871,    0,  590,    0,  0,  8,  0, }, // Gulf
    {  100,  876,    0,  598,    0,  0,  3,  0, }, // Guyana
    {  100,  883,  601,  601,  604,  3,  3,  3, }, // Hawaii_Aleutian
    {  100,  899,  607,  607,  610,  3,  3,  7, }, // Hong_Kong
    {  100,  909,  617,  617,  621,  4,  4,  5, }, // Hovd
    {  100,  914,    0,  626,    0,  0, 10,  0, }, // India
    {  100,  920,    0,  636,    0,  0,  3,  0, }, // Indian_Ocean
    {  100,  933,    0,  639,    0,  0,  3,  0, }, // Indochina
    {  100,  943,    0,  642,    0,  0,  3,  0, }, // Indonesia_Central
    {  100,  961,    0,  645,    0,  0,  3,  0, }, // Indonesia_Eastern
    {  100,  979,    0,  648,    0,  0,  3,  0, }, // Indonesia_Western
    {  100,  997,  651,  651,  655,  4,  4,  8, }, // Iran
    {  100, 1008,  663,  663,  667,  4,  4,  5, }, // Irkutsk
    {  100, 1016,  672,  672,  682, 10, 10,  3, }, // Israel
    {  100, 1023,  685,  685,  688,  3,  3,  7, }, // Japan
    {  100, 1029,  695,  699,  704,  4,  5,  5, }, // Kamchatka
    {  100, 1047,    0,  709,    0,  0, 13,  0, }, // Kazakhstan
    {  100, 1058,    0,  722,    0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  100, 1077,    0,  746,    0,  0, 18,  0, }, // Kazakhstan_Western
    {  100, 1106,    4,    4,  764,  3,  3,  7, }, // Korea
    {  100, 1112,    0,  771,    0,  0,  4,  0, }, // Kosrae
    {  100, 1119,  775,  775,  779,  4,  4,  5, }, // Krasnoyarsk
    {  100, 1151,    0,  784,    0,  0,  3,  0, }, // Kyrgystan
    {  100, 1161,    0,  787,    0,  0,  4,  0, }, // Lanka
    {  100, 1175,    0,  791,    0,  0,  4,  0, }, // Line_Islands
    {  100, 1188,  795,  795,  799,  4,  4, 11, }, // Lord_Howe
    {  100, 1198,  810,  810,  819,  9,  9, 13, }, // Macau
    {  100, 1204,  832,  832,  836,  4,  4,  5, }, // Magadan
    {  100, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  100, 1228,    0,  841,    0,  0,  3,  0, }, // Maldives
    {  100, 1237,    0,  844,    0,  0,  4,  0, }, // Marquesas
    {  100, 1247,    0,  848,    0,  0,  3,  0, }, // Marshall_Islands
    {  100, 1264,  851,  851,  854,  3,  3,  4, }, // Mauritius
    {  100, 1274,    0,  858,    0,  0,  4,  0, }, // Mawson
    {  100, 1281,  862,  862,  866,  4,  4,  4, }, // Mexico_Pacific
    {  100, 1296,  870,  870,  874,  4,  4,  5, }, // Mongolia
    {  100, 1305,  879,  882,  885,  3,  3,  3, }, // Moscow
    {  100, 1312,    0,  888,    0,  0,  3,  0, }, // Myanmar
    {  100, 1320,    0,  891,    0,  0,  3,  0, }, // Nauru
    {  100, 1326,    0,  894,    0,  0,  3,  0, }, // Nepal
    {  100, 1332,  897,  897,  900,  3,  3, 17, }, // New_Caledonia
    {  100, 1346,  917,  917,  921,  4,  4,  4, }, // New_Zealand
    {  100, 1358,  560,  560,  900,  3,  3,  3, }, // Newfoundland
    {  100, 1371,    0,  925,    0,  0,  3,  0, }, // Niue
    {  100, 1376,  928,  928,  931,  3,  3,  4, }, // Norfolk
    {  100, 1384,  935,  935,  938,  3,  3,  4, }, // Noronha
    {  100, 1392,    0,  942,    0,  0,  8,  0, }, // North_Mariana
    {  100, 1406,  950,  950,  954,  4,  4,  5, }, // Novosibirsk
    {  100, 1418,  959,  959,  963,  4,  4,  5, }, // Omsk
    {  100, 1428,  968,  968,  971,  3,  3,  7, }, // Pakistan
    {  100, 1437,    0,  978,    0,  0,  3,  0, }, // Palau
    {  100, 1443,    0,  981,    0,  0,  3,  0, }, // Papua_New_Guinea
    {  100, 1460,  984,  984,  987,  3,  3,  4, }, // Paraguay
    {  100, 1469,  695,  695,  991,  3,  3,  8, }, // Peru
    {  100, 1474,  999,  999, 1013, 14, 14, 18, }, // Philippines
    {  100, 1486,    0, 1031,    0,  0,  4,  0, }, // Phoenix_Islands
    {  100, 1502, 1035, 1035, 1039,  4,  4,  4, }, // Pierre_Miquelon
    {  100, 1518,    0, 1043,    0,  0, 12,  0, }, // Pitcairn
    {  100, 1527,    0, 1055,    0,  0,  4,  0, }, // Ponape
    {  100, 1534,    0, 1059,    0,  0,  9,  0, }, // Pyongyang
    {  100, 1544, 1068, 1068, 1072,  4,  4,  8, }, // Qyzylorda
    {  100, 1554,    0, 1080,    0,  0,  3,  0, }, // Reunion
    {  100, 1562,    0, 1083,    0,  0,  4,  0, }, // Rothera
    {  100, 1570, 1087, 1087, 1091,  4,  4,  5, }, // Sakhalin
    {  100, 1579, 1096, 1096, 1100,  4,  4,  5, }, // Samara
    {  100, 1596,  169,  169, 1105,  9,  9, 13, }, // Samoa
    {  100, 1602,    0, 1118,    0,  0,  3,  0, }, // Seychelles
    {  100, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  100, 1634,    0, 1121,    0,  0,  3,  0, }, // Solomon
    {  100, 1642,    0,   82,    0,  0,  3,  0, }, // South_Georgia
    {  100, 1656,    0, 1124,    0,  0,  3,  0, }, // Suriname
    {  100, 1676,    0, 1127,    0,  0,  4,  0, }, // Syowa
    {  100, 1682,    0, 1131,    0,  0,  4,  0, }, // Tahiti
    {  100, 1689, 1135, 1135, 1138,  3,  3,  7, }, // Taipei
    {  100, 1696,    0, 1145,    0,  0,  3,  0, }, // Tajikistan
    {  100, 1724,    0, 1148,    0,  0,  3,  0, }, // Tokelau
    {  100, 1732, 1151, 1151, 1154,  3,  3,  4, }, // Tonga
    {  100, 1738,    0, 1158,    0,  0,  4,  0, }, // Truk
    {  100, 1750,   26,   26, 1162,  3,  3,  7, }, // Turkmenistan
    {  100, 1763,    0, 1169,    0,  0,  3,  0, }, // Tuvalu
    {  100, 1777, 1172, 1172, 1175,  3,  3,  4, }, // Uruguay
    {  100, 1792, 1179, 1179, 1182,  3,  3,  7, }, // Uzbekistan
    {  100, 1803, 1189, 1189, 1192,  3,  3,  7, }, // Vanuatu
    {  100, 1811,    0, 1199,    0,  0,  5,  0, }, // Venezuela
    {  100, 1821, 1204, 1204, 1208,  4,  4,  5, }, // Vladivostok
    {  100, 1833, 1213, 1213, 1217,  4,  4,  5, }, // Volgograd
    {  100, 1843,    0, 1222,    0,  0,  4,  0, }, // Vostok
    {  100, 1850,    0, 1226,    0,  0,  4,  0, }, // Wake
    {  100, 1855,    0, 1230,    0,  0,  3,  0, }, // Wallis
    {  100, 1862, 1233, 1233, 1237,  4,  4,  5, }, // Yakutsk
    {  100, 1870, 1242, 1242, 1246,  4,  4,  5, }, // Yekaterinburg
    {  100, 1892,    0,  988,    0,  0,  3,  0, }, // Yukon
    {  101,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central Colognian/Latin/Germany
    {  101,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  101,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  102,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Cornish/Latin/United Kingdom
    {  102,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  102,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  102,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  104,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Croatian/Latin/Croatia
    {  104,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  104,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  104,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  106,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Czech/Latin/Czechia
    {  106,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  106,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  106,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  106,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  106,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  106,  627, 1272, 1272, 1275,  3,  3,  4, }, // Europe_Central
    {  107,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Danish/Latin/Denmark
    {  107,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  107,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  112,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Dutch/Latin/Netherlands
    {  112,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  112,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  112,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  112,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  112,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  112,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  112,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  112,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  112,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  112,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  118, 1656,    0, 1124,    0,  0,  3,  0, }, // Suriname Dutch/Latin/Suriname
    {  119,  415,    0,  370,    0,  0,  2,  0, }, // Bhutan Dzongkha/Tibetan/Bhutan
    {  121,  107,    0,    3,    7,  3,  4,  4, }, // Alaska English/Latin/United States
    {  121,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  121,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  121,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  121,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  121,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  121,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  121,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  124,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Anguilla
    {  124,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  124,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  124,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  124,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  124,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  124,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  125,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Antigua and Barbuda
    {  125,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  125,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  125,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  125,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  125,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  125,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  126,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Australia
    {  126,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  126,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  126,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  126,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  126,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  126,  295,  121,  129,  296,  3,  4,  4, }, // Australia_Central
    {  126,  313, 1282,  300,  305,  4,  5,  5, }, // Australia_CentralWestern
    {  126,  338, 1286,  310,  314,  3,  4,  4, }, // Australia_Eastern
    {  126,  356,  859,  318,  322,  3,  4,  4, }, // Australia_Western
    {  126,  871,    0, 1289,    0,  0,  7,  0, }, // Gulf
    {  126,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  126, 1188, 1296,  795, 1299,  3,  4,  4, }, // Lord_Howe
    {  126, 1346, 1303,  917,  921,  3,  4,  4, }, // New_Zealand
    {  127,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Austria
    {  127,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  127,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  127,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  127,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  127,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  127,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  127,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  127,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  127,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  128,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Bahamas
    {  128,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  128,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  128,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  128,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  128,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  128,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  129,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Barbados
    {  129,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  129,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  129,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  129,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  129,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  129,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  130,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Belgium
    {  130,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  130,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  130,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  130,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  130,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  130,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  130,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  130,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  130,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  131,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Belize
    {  131,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  131,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  131,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  131,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  131,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  131,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  132,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Bermuda
    {  132,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  132,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  132,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  132,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  132,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  132,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  133,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Botswana
    {  133,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  133,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  133,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  133,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  133,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  133,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  133,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  133,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  133,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  133,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  134,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/British Indian Ocean Territory
    {  134,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  134,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  134,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  134,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  134,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  134,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  135,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/British Virgin Islands
    {  135,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  135,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  135,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  135,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  135,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  135,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  137,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Cameroon
    {  137,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  137,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  137,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  137,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  137,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  137,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  137,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  137,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  137,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  137,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  138,    5,    0,  138,    0,  0,  3,  0, }, // Afghanistan English/Latin/Canada
    {  138,  107,    0,    3,    7,  3,  4,  4, }, // Alaska
    {  138,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  138,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  138,  240,  257,    0,    0,  3,  0,  0, }, // Argentina
    {  138,  313, 1282,  300,  305,  4,  5,  5, }, // Australia_CentralWestern
    {  138,  338, 1286,  310,  314,  3,  4,  4, }, // Australia_Eastern
    {  138,  356,    0,  318,  322,  0,  4,  4, }, // Australia_Western
    {  138,  397,    0,  342,    0,  0,  3,  0, }, // Bangladesh
    {  138,  415,    0,  370,    0,  0,  3,  0, }, // Bhutan
    {  138,  437,  376,    0,  379,  3,  0,  4, }, // Brasilia
    {  138,  454,    0,  383,    0,  0,  3,  0, }, // Brunei
    {  138,  487,    0,  401,  406,  0,  5,  5, }, // Chatham
    {  138,  507,    0,  440,    0,  0,  3,  0, }, // Christmas
    {  138,  517,    0,  443,    0,  0,  3,  0, }, // Cocos
    {  138,  523,    0,    0,  449,  0,  0,  4, }, // Colombia
    {  138,  601,    0,  487,    0,  0,  3,  0, }, // East_Timor
    {  138,  612,    0,  490,  494,  0,  4,  5, }, // Easter
    {  138,  619,    0,  499,    0,  0,  3,  0, }, // Ecuador
    {  138,  695,  514,    0,  517,  3,  0,  4, }, // Falkland
    {  138,  746,    0,  541,    0,  0,  4,  0, }, // Galapagos
    {  138,  830,  568,    0,    0,  3,  0,  0, }, // Greenland_Eastern
    {  138,  871,    0, 1306,    0,  0,  9,  0, }, // Gulf
    {  138,  876,    0,  598,    0,  0,  3,  0, }, // Guyana
    {  138,  914,    0,  112,    0,  0,  3,  0, }, // India
    {  138,  933,    0,  639,    0,  0,  3,  0, }, // Indochina
    {  138,  943,    0, 1315,    0,  0,  4,  0, }, // Indonesia_Central
    {  138,  961,    0,  648,    0,  0,  3,  0, }, // Indonesia_Eastern
    {  138,  979,    0, 1319,    0,  0,  3,  0, }, // Indonesia_Western
    {  138,  997,    0,  651, 1322,  0,  4,  4, }, // Iran
    {  138, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  138, 1228,    0,  841,    0,  0,  3,  0, }, // Maldives
    {  138, 1326,    0,  894,    0,  0,  3,  0, }, // Nepal
    {  138, 1358,  384,  560,  900,  2,  3,  3, }, // Newfoundland
    {  138, 1384,  935,    0,    0,  3,  0,  0, }, // Noronha
    {  138, 1428,    0,  968,    0,  0,  3,  0, }, // Pakistan
    {  138, 1460,  984,    0,  987,  3,  0,  4, }, // Paraguay
    {  138, 1469,  695,    0,    0,  3,  0,  0, }, // Peru
    {  138, 1502, 1326, 1035, 1039,  3,  4,  4, }, // Pierre_Miquelon
    {  138, 1777,    0, 1172, 1175,  0,  3,  4, }, // Uruguay
    {  138, 1811,    0, 1329,    0,  0,  3,  0, }, // Venezuela
    {  139,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Cayman Islands
    {  139,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  139,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  139,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  139,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  139,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  139,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  140,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Christmas Island
    {  140,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  140,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  140,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  140,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  140,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  140,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  141,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Cocos Islands
    {  141,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  141,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  141,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  141,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  141,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  141,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  142,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Cook Islands
    {  142,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  142,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  142,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  142,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  142,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  142,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  143,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Cyprus
    {  143,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  143,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  143,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  143,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  143,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  143,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  144,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Denmark
    {  144,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  144,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  144,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  144,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  144,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  144,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  144,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  144,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  144,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  145,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Diego Garcia
    {  145,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  145,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  145,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  145,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  145,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  145,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  146,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Dominica
    {  146,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  146,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  146,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  146,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  146,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  146,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  147,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Eritrea
    {  147,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  147,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  147,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  147,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  147,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  147,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  147,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  147,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  147,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  147,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  148,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Eswatini
    {  148,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  148,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  148,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  148,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  148,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  148,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  148,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  148,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  148,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  148,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  149,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Europe
    {  149,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  149,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  149,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  149,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  149,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  149,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  149,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  149,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  149,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  150,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Falkland Islands
    {  150,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  150,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  150,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  150,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  150,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  150,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  151,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Fiji
    {  151,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  151,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  151,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  151,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  151,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  151,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  152,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Finland
    {  152,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  152,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  152,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  152,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  152,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  152,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  152,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  152,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  152,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  153,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Gambia
    {  153,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  153,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  153,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  153,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  153,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  153,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  153,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  153,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  153,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  153,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  154,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Germany
    {  154,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  154,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  154,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  154,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  154,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  154,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  154,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  154,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  154,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  155,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Ghana
    {  155,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  155,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  155,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  155,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  155,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  155,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  155,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  155,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  155,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  155,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  156,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Gibraltar
    {  156,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  156,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  156,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  156,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  156,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  156,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  157,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Grenada
    {  157,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  157,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  157,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  157,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  157,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  157,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  158,  478,    0,  397,    0,  0,  4,  0, }, // Chamorro English/Latin/Guam
    {  159,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Guernsey
    {  159,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  159,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  159,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  159,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  159,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  159,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  160,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Guyana
    {  160,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  160,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  160,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  160,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  160,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  160,  876,    0,  598,    0,  0,  3,  0, }, // Guyana
    {  160,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  161,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Hong Kong
    {  161,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  161,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  161,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  161,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  161,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  161,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  161,  899,  607,  607, 1332,  3,  3,  4, }, // Hong_Kong
    {  162,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/India
    {  162,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  162,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  162,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  162,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  162,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  162,  871,    0,   82,    0,  0,  3,  0, }, // Gulf
    {  162,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  162,  914,    0,  112,    0,  0,  3,  0, }, // India
    {  163,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Indonesia
    {  163,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  163,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  163,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  163,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  163,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  163,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  163,  943,    0, 1315,    0,  0,  4,  0, }, // Indonesia_Central
    {  163,  961,    0,  648,    0,  0,  3,  0, }, // Indonesia_Eastern
    {  163,  979,    0, 1319,    0,  0,  3,  0, }, // Indonesia_Western
    {  164,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Ireland
    {  164,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  164,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  164,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  164,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  164,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  164,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  165,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Isle of Man
    {  165,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  165,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  165,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  165,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  165,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  165,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  166,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Israel
    {  166,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  166,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  166,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  166,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  166,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  166,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  167,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Jamaica
    {  167,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  167,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  167,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  167,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  167,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  167,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  168,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Jersey
    {  168,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  168,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  168,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  168,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  168,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  168,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  169,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Kenya
    {  169,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  169,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  169,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  169,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  169,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  169,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  169,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  169,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  169,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  169,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  170,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Kiribati
    {  170,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  170,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  170,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  170,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  170,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  170,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  171,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Lesotho
    {  171,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  171,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  171,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  171,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  171,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  171,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  171,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  171,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  171,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  171,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  172,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Liberia
    {  172,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  172,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  172,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  172,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  172,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  172,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  172,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  172,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  172,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  172,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  173,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Macao
    {  173,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  173,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  173,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  173,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  173,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  173,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  173,  899,  607,  607, 1332,  3,  3,  4, }, // Hong_Kong
    {  173, 1198,   29,   29,   32,  3,  3,  3, }, // Macau
    {  174,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Madagascar
    {  174,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  174,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  174,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  174,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  174,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  174,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  174,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  174,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  174,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  174,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  175,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Malawi
    {  175,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  175,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  175,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  175,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  175,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  175,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  175,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  175,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  175,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  175,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  176,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Malaysia
    {  176,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  176,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  176,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  176,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  176,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  176,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  176, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  177,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Maldives
    {  177,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  177,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  177,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  177,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  177,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  177,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  178,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Malta
    {  178,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  178,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  178,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  178,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  178,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  178,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  179,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Marshall Islands
    {  179,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  179,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  179,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  179,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  179,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  179,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  180,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Mauritius
    {  180,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  180,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  180,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  180,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  180,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  180,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  180,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  180,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  180,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  180,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  181,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Micronesia
    {  181,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  181,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  181,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  181,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  181,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  181,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  182,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Montserrat
    {  182,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  182,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  182,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  182,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  182,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  182,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  183,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Namibia
    {  183,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  183,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  183,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  183,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  183,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  183,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  183,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  183,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  183,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  183,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  184,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Nauru
    {  184,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  184,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  184,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  184,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  184,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  184,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  185,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Netherlands
    {  185,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  185,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  185,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  185,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  185,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  185,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  185,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  185,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  185,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  186,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/New Zealand
    {  186,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  186,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  186,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  186,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  186,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  186,  295,  121,  129,  296,  3,  4,  4, }, // Australia_Central
    {  186,  313, 1282,  300,  305,  4,  5,  5, }, // Australia_CentralWestern
    {  186,  338, 1286,  310,  314,  3,  4,  4, }, // Australia_Eastern
    {  186,  356,  859,  318,  322,  3,  4,  4, }, // Australia_Western
    {  186,  487, 1336,  401,  406,  4,  5,  5, }, // Chatham
    {  186,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  186, 1188, 1296,  795, 1299,  3,  4,  4, }, // Lord_Howe
    {  186, 1346, 1303,  917,  921,  3,  4,  4, }, // New_Zealand
    {  187,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Nigeria
    {  187,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  187,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  187,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  187,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  187,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  187,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  187,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  187,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  187,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  187,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  188,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Niue
    {  188,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  188,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  188,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  188,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  188,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  188,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  189,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Norfolk Island
    {  189,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  189,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  189,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  189,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  189,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  189,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  190,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Northern Mariana Islands
    {  190,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  190,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  190,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  190,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  190,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  190,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  191,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Pakistan
    {  191,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  191,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  191,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  191,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  191,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  191,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  192,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Palau
    {  192,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  192,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  192,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  192,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  192,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  192,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  193,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Papua New Guinea
    {  193,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  193,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  193,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  193,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  193,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  193,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  195,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Pitcairn
    {  195,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  195,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  195,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  195,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  195,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  195,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  197,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Rwanda
    {  197,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  197,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  197,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  197,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  197,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  197,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  197,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  197,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  197,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  197,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  198,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Saint Helena
    {  198,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  198,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  198,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  198,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  198,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  198,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  199,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Saint Kitts and Nevis
    {  199,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  199,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  199,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  199,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  199,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  199,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  200,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Saint Lucia
    {  200,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  200,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  200,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  200,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  200,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  200,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  201,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Saint Vincent and Grenadines
    {  201,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  201,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  201,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  201,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  201,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  201,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  202,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Samoa
    {  202,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  202,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  202,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  202,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  202,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  202,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  203,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Seychelles
    {  203,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  203,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  203,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  203,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  203,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  203,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  204,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Sierra Leone
    {  204,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  204,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  204,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  204,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  204,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  204,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  204,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  204,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  204,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  204,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  205,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Singapore
    {  205,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  205,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  205,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  205,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  205,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  205,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  205, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  205, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  206,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Sint Maarten
    {  206,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  206,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  206,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  206,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  206,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  206,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  207,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Slovenia
    {  207,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  207,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  207,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  207,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  207,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  207,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  207,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  207,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  207,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  208,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Solomon Islands
    {  208,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  208,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  208,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  208,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  208,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  208,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  209,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/South Africa
    {  209,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  209,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  209,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  209,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  209,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  209,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  209,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  209,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  209,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  209,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  210,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/South Sudan
    {  210,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  210,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  210,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  210,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  210,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  210,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  210,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  210,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  210,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  210,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  211,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Sudan
    {  211,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  211,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  211,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  211,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  211,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  211,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  211,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  211,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  211,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  211,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  212,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Sweden
    {  212,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  212,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  212,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  212,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  212,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  212,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  212,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  212,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  212,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  213,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Switzerland
    {  213,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  213,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  213,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  213,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  213,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  213,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  213,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  213,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  213,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  214,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Tanzania
    {  214,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  214,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  214,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  214,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  214,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  214,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  214,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  214,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  214,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  214,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  215,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Tokelau
    {  215,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  215,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  215,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  215,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  215,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  215,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  216,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Tonga
    {  216,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  216,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  216,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  216,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  216,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  216,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  217,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Trinidad and Tobago
    {  217,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  217,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  217,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  217,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  217,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  217,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  218,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Turks and Caicos Islands
    {  218,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  218,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  218,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  218,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  218,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  218,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  219,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Tuvalu
    {  219,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  219,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  219,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  219,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  219,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  219,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  220,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Uganda
    {  220,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  220,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  220,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  220,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  220,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  220,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  220,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  220,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  220,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  220,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  221,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/United Arab Emirates
    {  221,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  221,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  221,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  221,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  221,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  221,  871,    0,   82,    0,  0,  3,  0, }, // Gulf
    {  221,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  222,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/United Kingdom
    {  222,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  222,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  222,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  222,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  222,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  222,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  222,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  222,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  222,  871,    0,   82,    0,  0,  3,  0, }, // Gulf
    {  222,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  225,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/Vanuatu
    {  225,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  225,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  225,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  225,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  225,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  225,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  226,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska English/Latin/world
    {  226,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  226,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  226,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  226,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  226,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  226,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  227,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Zambia
    {  227,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  227,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  227,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  227,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  227,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  227,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  227,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  227,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  227,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  227,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  228,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Zimbabwe
    {  228,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  228,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  228,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  228,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska
    {  228,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  228,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  228,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  228,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  228,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  228,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  233,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central Ewe/Latin/Ghana
    {  233,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  233,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  233,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  240,  107, 1340, 1343, 1347,  3,  4,  4, }, // Alaska French/Latin/France
    {  240,  142, 1351, 1353, 1356,  2,  3,  3, }, // America_Central
    {  240,  158, 1347, 1359, 1362,  2,  3,  3, }, // America_Eastern
    {  240,  174, 1365, 1367, 1370,  2,  3,  3, }, // America_Mountain
    {  240,  191, 1373, 1375, 1378,  2,  3,  3, }, // America_Pacific
    {  240,  286,   54, 1343, 1347,  2,  3,  3, }, // Atlantic
    {  240,  537, 1381, 1384, 1388,  3,  4,  4, }, // Cuba
    {  240,  642,   98,  101, 1392,  3,  4,  4, }, // Europe_Eastern
    {  240,  680,  105,  108, 1396,  3,  4,  4, }, // Europe_Western
    {  240,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  240,  830, 1400, 1403, 1407,  3,  4,  4, }, // Greenland_Eastern
    {  240,  848, 1411, 1414, 1418,  3,  4,  4, }, // Greenland_Western
    {  240,  883, 1422, 1425, 1429,  3,  4,  4, }, // Hawaii_Aleutian
    {  240, 1281, 1433, 1437, 1442,  4,  5,  5, }, // Mexico_Pacific
    {  240, 1358, 1447, 1450, 1454,  3,  4,  4, }, // Newfoundland
    {  240, 1502, 1433, 1437, 1442,  3,  4,  4, }, // Pierre_Miquelon
    {  247,  107, 1340, 1343, 1347,  3,  4,  4, }, // Alaska French/Latin/Canada
    {  247,  142, 1351, 1353, 1458,  2,  3,  3, }, // America_Central
    {  247,  158, 1347, 1359, 1461,  2,  3,  3, }, // America_Eastern
    {  247,  174, 1365, 1367, 1464,  2,  3,  3, }, // America_Mountain
    {  247,  191, 1373, 1375, 1467,  2,  3,  3, }, // America_Pacific
    {  247,  286,   54, 1343, 1347,  2,  3,  3, }, // Atlantic
    {  247,  537, 1381, 1384, 1388,  3,  4,  4, }, // Cuba
    {  247,  642,   98,  101, 1392,  3,  4,  4, }, // Europe_Eastern
    {  247,  680,  105,  108, 1396,  3,  4,  4, }, // Europe_Western
    {  247,  830, 1400, 1403, 1407,  3,  4,  4, }, // Greenland_Eastern
    {  247,  848, 1411, 1414, 1418,  3,  4,  4, }, // Greenland_Western
    {  247,  883, 1422, 1425, 1429,  3,  4,  4, }, // Hawaii_Aleutian
    {  247, 1281, 1433, 1437, 1442,  4,  5,  5, }, // Mexico_Pacific
    {  247, 1358,  849, 1450,   54,  2,  3,  3, }, // Newfoundland
    {  247, 1502, 1433, 1437, 1442,  3,  4,  4, }, // Pierre_Miquelon
    {  255,  709,    0,  534,    0,  0,  3,  0, }, // French_Guiana French/Latin/French Guiana
    {  286,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Friulian/Latin/Italy
    {  286,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  286,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  286,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  311,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Gaelic/Latin/United Kingdom
    {  311,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  311,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  311,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  311,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  311,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  311,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  311,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  311,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  311,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  311,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  313,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Galician/Latin/Spain
    {  313,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  313,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  313,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  318,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central German/Latin/Germany
    {  318,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  318,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  325,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Greek/Greek/Greece
    {  325,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  325,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  328,  914,    0,  112,    0,  0,  3,  0, }, // India Gujarati/Gujarati/India
    {  335,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Hawaiian/Latin/United States
    {  335,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  337,  914,    0,  112,    0,  0,  3,  0, }, // India Hindi/Devanagari/India
    {  338,  107, 1279, 1279, 1279,  3,  3,  3, }, // Alaska Hindi/Latin/India
    {  338,  142, 1279, 1279, 1279,  3,  3,  3, }, // America_Central
    {  338,  158, 1279, 1279, 1279,  3,  3,  3, }, // America_Eastern
    {  338,  174, 1279, 1279, 1279,  3,  3,  3, }, // America_Mountain
    {  338,  191, 1279, 1279, 1279,  3,  3,  3, }, // America_Pacific
    {  338,  286, 1279, 1279, 1279,  3,  3,  3, }, // Atlantic
    {  338,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  338,  871,    0,   82,    0,  0,  3,  0, }, // Gulf
    {  338,  883, 1279, 1279, 1279,  3,  3,  3, }, // Hawaii_Aleutian
    {  338,  914,    0,  112,    0,  0,  3,  0, }, // India
    {  339,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Hungarian/Latin/Hungary
    {  339,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  339,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  339,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  344,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Indonesian/Latin/Indonesia
    {  344,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  344,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  344,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  344,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  344,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  344,  537, 1470, 1479, 1489,  9, 10, 10, }, // Cuba
    {  344,  830,  568,  571, 1499,  3,  4,  4, }, // Greenland_Eastern
    {  344,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  344,  943,    0, 1315,    0,  0,  4,  0, }, // Indonesia_Central
    {  344,  961,    0,  648,    0,  0,  3,  0, }, // Indonesia_Eastern
    {  344,  979,    0, 1319,    0,  0,  3,  0, }, // Indonesia_Western
    {  344, 1358,  384,  560,  900,  2,  3,  3, }, // Newfoundland
    {  344, 1502, 1326, 1035, 1039,  3,  4,  4, }, // Pierre_Miquelon
    {  349,  191, 1503, 1506, 1510,  3,  4,  4, }, // America_Pacific Irish/Latin/Ireland
    {  349,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  349,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  349,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  349,  788,    0,  832,    0,  0,  3,  0, }, // GMT
    {  351,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Italian/Latin/Italy
    {  351,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  351,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  355, 1023, 1279,  685, 1514,  3,  3,  3, }, // Japan Japanese/Japanese/Japan
    {  365,  914,    0,  112,    0,  0,  3,  0, }, // India Kannada/Kannada/India
    {  368,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern Kazakh/Cyrillic/Kazakhstan
    {  369,  128, 1517, 1517, 1523,  6,  6, 13, }, // Almaty Kazakh/Arabic/China
    {  369,  219, 1536, 1536, 1541,  5,  5, 12, }, // Aqtau
    {  369,  225, 1553, 1553, 1559,  6,  6, 13, }, // Aqtobe
    {  369, 1047,    0, 1572,    0,  0,  9,  0, }, // Kazakhstan
    {  369, 1058,    0, 1581,    0,  0, 15,  0, }, // Kazakhstan_Eastern
    {  369, 1077,    0, 1596,    0,  0, 15,  0, }, // Kazakhstan_Western
    {  369, 1151,    0, 1611,    0,  0, 10,  0, }, // Kyrgystan
    {  369, 1544, 1621, 1621, 1630,  9,  9, 16, }, // Qyzylorda
    {  375,  914,    0,  112,    0,  0,  3,  0, }, // India Konkani/Devanagari/India
    {  384,  107, 1087, 1646, 1650,  3,  4,  4, }, // Alaska Kurdish/Latin/Turkey
    {  384,  142, 1654, 1656, 1659,  2,  3,  3, }, // America_Central
    {  384,  158, 1124, 1662, 1665,  2,  3,  3, }, // America_Eastern
    {  384,  174, 1668, 1670, 1673,  2,  3,  3, }, // America_Mountain
    {  384,  191, 1676, 1678, 1681,  2,  3,  3, }, // America_Pacific
    {  384,  286,   71, 1646, 1650,  2,  3,  3, }, // Atlantic
    {  384,  537,  880, 1684, 1687,  2,  3,  3, }, // Cuba
    {  384,  830, 1690, 1693, 1697,  3,  4,  4, }, // Greenland_Eastern
    {  384,  848, 1701, 1705, 1710,  4,  5,  5, }, // Greenland_Western
    {  384,  883, 1715, 1719, 1723,  4,  4,  4, }, // Hawaii_Aleutian
    {  384, 1358, 1727, 1730, 1734,  3,  4,  4, }, // Newfoundland
    {  384, 1892,    0, 1127,    0,  0,  2,  0, }, // Yukon
    {  391,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Latvian/Latin/Latvia
    {  391,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  391,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  398,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central Lower Sorbian/Latin/Germany
    {  398,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  398,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  407,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Macedonian/Cyrillic/Macedonia
    {  407,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  407,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  413,  914,    0,  112,    0,  0,  3,  0, }, // India Malayalam/Malayalam/India
    {  414,  788,    0,   51,    0,  0,  3,  0, }, // GMT Malay/Latin/Malaysia
    {  414, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  414, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  420,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Maltese/Latin/Malta
    {  420,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  420,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  420,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  423,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Manx/Latin/Isle of Man
    {  423,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  423,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  423,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  426,  914,    0,  112,    0,  0,  3,  0, }, // India Marathi/Devanagari/India
    {  442,  914,    0,  112,    0,  0,  3,  0, }, // India Nepali/Devanagari/India
    {  449,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Northern Sami/Latin/Norway
    {  449,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  449,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  449,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  450,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Northern Sami/Latin/Finland
    {  450,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  450,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  450,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  454,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Norwegian Bokmal/Latin/Norway
    {  454,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  454,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  454,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  454,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  454,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  454,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  454,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  454,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  454,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  454,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  456,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Norwegian Nynorsk/Latin/Norway
    {  456,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  456,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  456,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  456,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  456,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  456,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  456,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  456,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  456,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  456,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  462,  788,    0,   51,    0,  0,  3,  0, }, // GMT Odia/Odia/India
    {  462,  914,    0,  112,    0,  0,  3,  0, }, // India
    {  463,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Oromo/Latin/Ethiopia
    {  463,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  463,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  463,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  463,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  463,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  463,  883,   54,   57,   61,  3,  3,  4, }, // Hawaii_Aleutian
    {  474,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Polish/Latin/Poland
    {  474,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  474,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  475,    0,  121,  121,  129,  3,  3,  4, }, // Acre Portuguese/Latin/Brazil
    {  475,  135,  153,  153,  156,  3,  3,  4, }, // Amazon
    {  475,  437,  376,  376,  379,  3,  3,  4, }, // Brasilia
    {  476,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Angola
    {  476,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  476,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  476,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  476,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  476,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  476,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  477,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Cape Verde
    {  477,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  477,  385,  333, 1279, 1279,  4,  3,  3, }, // Azores
    {  477,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  477,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  477,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  477,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  478,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Equatorial Guinea
    {  478,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  478,  385,  333,  333,  337,  4,  4,  5, }, // Azores
    {  478,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  478,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  478,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  478,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  479,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Guinea-Bissau
    {  479,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  479,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  479,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  479,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  479,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  479,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  480,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Luxembourg
    {  480,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  480,  385,  333,  333,  337,  4,  4,  5, }, // Azores
    {  480,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  480,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  480,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  480,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  481,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Macao
    {  481,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  481,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  481,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  481,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  481,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  481,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  482,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Mozambique
    {  482,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  482,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  482,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  482,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  482,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  482,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  483,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Portugal
    {  483,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  483,  385,  333,  333,  337,  4,  4,  5, }, // Azores
    {  483,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  483,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  483,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  483,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  484,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Sao Tome and Principe
    {  484,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  484,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  484,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  484,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  484,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  484,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  485,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Switzerland
    {  485,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  485,  385,  333,  333,  337,  4,  4,  5, }, // Azores
    {  485,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  485,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  485,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  485,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  486,    0, 1279, 1279, 1279,  3,  3,  3, }, // Acre Portuguese/Latin/Timor-Leste
    {  486,  135, 1279, 1279, 1279,  3,  3,  3, }, // Amazon
    {  486,  385, 1279, 1279, 1279,  3,  3,  3, }, // Azores
    {  486,  437, 1279, 1279, 1279,  3,  3,  3, }, // Brasilia
    {  486,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  486,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  486,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  488,  914,    0,  112,    0,  0,  3,  0, }, // India Punjabi/Gurmukhi/India
    {  490, 1469,  695,  695, 1738,  3,  3,  4, }, // Peru Quechua/Latin/Peru
    {  491,  422,    0,  373,    0,  0,  3,  0, }, // Bolivia Quechua/Latin/Bolivia
    {  491, 1469, 1279, 1279, 1279,  3,  3,  3, }, // Peru
    {  492,  619,    0,  499,    0,  0,  3,  0, }, // Ecuador Quechua/Latin/Ecuador
    {  492, 1469, 1279, 1279, 1279,  3,  3,  3, }, // Peru
    {  493,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Romanian/Latin/Romania
    {  493,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  493,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  493,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  513,  107, 1742, 1745, 1749,  3,  4,  4, }, // Alaska Sardinian/Latin/Italy
    {  513,  142,  335, 1753, 1214,  2,  3,  3, }, // America_Central
    {  513,  158, 1756, 1758, 1761,  2,  3,  3, }, // America_Eastern
    {  513,  174, 1764, 1767, 1771,  3,  4,  4, }, // America_Mountain
    {  513,  191, 1775, 1777, 1780,  2,  3,  3, }, // America_Pacific
    {  513,  286, 1742, 1745, 1749,  2,  3,  3, }, // Atlantic
    {  513,  788,    0, 1783,    0,  0,  3,  0, }, // GMT
    {  515,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Cyrillic/Serbia
    {  515,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  515,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  515,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  516,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Cyrillic/Bosnia and Herzegovina
    {  516,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  516,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  516,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  519,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Bosnia and Herzegovina
    {  519,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  519,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  519,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  520,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Kosovo
    {  520,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  520,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  520,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  521,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Montenegro
    {  521,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  521,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  521,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  522,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Serbia
    {  522,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  522,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  522,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  526,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Sicilian/Latin/Italy
    {  526,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  526,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  526,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  526,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  526,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  526,  537, 1786, 1789, 1793,  3,  4,  4, }, // Cuba
    {  526,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  528,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Silesian/Latin/Poland
    {  528,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  528,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  533,  627, 1272, 1272, 1275,  3,  3,  4, }, // Europe_Central Slovak/Latin/Slovakia
    {  536,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian Somali/Latin/Somalia
    {  547,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Spanish/Latin/Spain
    {  547,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  547,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  547,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  548,  240,  257,  257,  260,  3,  3,  4, }, // Argentina Spanish/Latin/Argentina
    {  548,  250,  264,  264,  268,  4,  4,  5, }, // Argentina_Western
    {  548,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  548,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  548,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  548,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  549,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Belize
    {  549,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  549,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  549,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  550,  422,    0,  373,    0,  0,  3,  0, }, // Bolivia Spanish/Latin/Bolivia
    {  550,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  550,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  550,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  550,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  551,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Brazil
    {  551,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  551,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  551,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  554,  495,  411,  411,  414,  3,  3,  4, }, // Chile Spanish/Latin/Chile
    {  554,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  554,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  554,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  554,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  555,  523,  446,  446,  449,  3,  3,  4, }, // Colombia Spanish/Latin/Colombia
    {  555,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  555,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  555,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  555,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  556,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Costa Rica
    {  556,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  556,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  556,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  557,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Cuba
    {  557,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  557,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  557,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  558,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Dominican Republic
    {  558,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  558,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  558,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  559,  619,    0,  499,    0,  0,  3,  0, }, // Ecuador Spanish/Latin/Ecuador
    {  559,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  559,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  559,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  559,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  560,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/El Salvador
    {  560,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  560,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  560,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  562,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Guatemala
    {  562,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  562,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  562,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  563,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Honduras
    {  563,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  563,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  563,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  564,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Latin America
    {  564,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  564,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  564,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  565,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Mexico
    {  565,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  565,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  565,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  566,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Nicaragua
    {  566,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  566,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  566,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  567,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Panama
    {  567,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  567,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  567,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  568,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Paraguay
    {  568,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  568,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  568,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  569,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Peru
    {  569,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  569,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  569,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  569, 1469,  695,  695, 1738,  3,  3,  4, }, // Peru
    {  571,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Puerto Rico
    {  571,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  571,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  571,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  572,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Spanish/Latin/United States
    {  572,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  572,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  572,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  572,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  572,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  572,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central
    {  572,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  572,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  572,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  572,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  573,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Uruguay
    {  573,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  573,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  573,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  573, 1777, 1172, 1172, 1175,  3,  3,  4, }, // Uruguay
    {  574,  627, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Central Spanish/Latin/Venezuela
    {  574,  642, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Eastern
    {  574,  680, 1279, 1279, 1279,  3,  3,  3, }, // Europe_Western
    {  574,  788,    0, 1279,    0,  0,  3,  0, }, // GMT
    {  574, 1811,    0, 1329,    0,  0,  3,  0, }, // Venezuela
    {  583,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Swedish/Latin/Sweden
    {  583,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  583,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  583,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  586,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central Swiss German/Latin/Switzerland
    {  586,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  586,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  596,  914,    0,  112,    0,  0,  3,  0, }, // India Tamil/Tamil/India
    {  597,  914,    0, 1279,    0,  0,  3,  0, }, // India Tamil/Tamil/Malaysia
    {  597, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  597, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  598,  914,    0, 1279,    0,  0,  3,  0, }, // India Tamil/Tamil/Singapore
    {  598, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  598, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  600,  142, 1797, 1800, 1804,  3,  4,  5, }, // America_Central Taroko/Latin/Taiwan
    {  600,  158, 1809, 1812, 1816,  3,  4,  5, }, // America_Eastern
    {  600,  174, 1821, 1824, 1828,  3,  4,  4, }, // America_Mountain
    {  600,  191, 1514, 1832, 1836,  2,  4,  5, }, // America_Pacific
    {  600,  286, 1841, 1844, 1848,  3,  4,  5, }, // Atlantic
    {  600,  627, 1797, 1797, 1853,  2,  2,  4, }, // Europe_Central
    {  600,  642,  522,  522, 1857,  2,  2,  4, }, // Europe_Eastern
    {  600,  680,  685,  685, 1861,  2,  2,  4, }, // Europe_Western
    {  600,  788,    0, 1865,    0,  0,  3,  0, }, // GMT
    {  603,  914,    0,  112,    0,  0,  3,  0, }, // India Telugu/Telugu/India
    {  619, 1750,   26,   26,   28,  3,  3,  4, }, // Turkmenistan Turkmen/Latin/Turkmenistan
    {  622,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central Upper Sorbian/Latin/Germany
    {  622,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  622,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  632,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Vietnamese/Latin/Vietnam
    {  632,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  632,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  632,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  632,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  632,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  632,  883,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  636,  627, 1251, 1251, 1254,  3,  3,  4, }, // Europe_Central Walser/Latin/Switzerland
    {  636,  642, 1258, 1258, 1261,  3,  3,  4, }, // Europe_Eastern
    {  636,  680, 1265, 1265, 1268,  3,  3,  4, }, // Europe_Western
    {  638,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Welsh/Latin/United Kingdom
    {  638,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  638,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  638,  788,    0,   51,    0,  0,  3,  0, }, // GMT
    {  644,  627,   91,   91,   94,  3,  3,  4, }, // Europe_Central Western Frisian/Latin/Netherlands
    {  644,  642,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  644,  680,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  655,    0,  121,  121,  129,  3,  3,  4, }, // Acre Kaingang/Latin/Brazil
    {  655,  135,  153,  153,  156,  3,  3,  4, }, // Amazon
    {  655,  437,  376,  376,  379,  3,  3,  4, }, // Brasilia
    {  656,    0,  121,  121,  129,  3,  3,  4, }, // Acre Nheengatu/Latin/Brazil
    {  656,  135,  153,  153,  156,  3,  3,  4, }, // Amazon
    {  656,  437,  376,  376,  379,  3,  3,  4, }, // Brasilia
    {  672,  788,    0, 1868,    0,  0,  2,  0, }, // GMT Anii/Latin/Benin
}; // Metazone short name table

// Indexing matches that of locale_data in qlocale_data_p.h
static inline constexpr LocaleZoneData localeZoneData[] = {
    // LOCALE_TAGS(lng,scp,ter) xct1st, zn1st, ml1st, ms1st, (+hr, -hr, gmt, flbk, rgen, rstd, rdst){ind,sz}
    { LOCALE_TAGS(  1,  0,  0)     0,     0,     0,     0,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // C/AnyScript/AnyTerritory
    { LOCALE_TAGS(  2, 27, 90)    42,   158,     8,     4,    0,    6,    0,    0,   39,   47,   66,  6,  6,  5,  7,  8, 19, 17, }, // Abkhazian/Cyrillic/Georgia
    { LOCALE_TAGS(  3, 66, 77)   458,   314,     8,     7,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Afar/Latin/Ethiopia
    { LOCALE_TAGS(  3, 66, 67)   499,   314,     8,     8,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Afar/Latin/Djibouti
    { LOCALE_TAGS(  3, 66, 74)   499,   314,     8,     8,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Afar/Latin/Eritrea
    { LOCALE_TAGS(  4, 66,216)   499,   314,     8,     8,    0,    6,    0,    0,   97,  103,  118,  6,  6,  5,  7,  6, 15, 12, }, // Afrikaans/Latin/South Africa
    { LOCALE_TAGS(  4, 66,162)   612,   461,    12,    11,    0,    6,    0,    0,   97,  103,  118,  6,  6,  5,  7,  6, 15, 12, }, // Afrikaans/Latin/Namibia
    { LOCALE_TAGS(  5, 66, 40)   612,   461,    12,    11,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Aghem/Latin/Cameroon
    { LOCALE_TAGS(  6, 66, 92)   653,   461,    12,    12,    0,    6,    0,    0,  130,  137,  152,  6,  6,  5,  7,  7, 15, 11, }, // Akan/Latin/Ghana
    { LOCALE_TAGS(  8, 66, 40)   886,   605,    12,    15,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Akoose/Latin/Cameroon
    { LOCALE_TAGS(  9, 66,  3)   927,   605,    12,    16,    0,    6,    0,    0,  163,  170,  187,  6,  6,  5,  7,  7, 17, 14, }, // Albanian/Latin/Albania
    { LOCALE_TAGS(  9, 66,126)  1217,   762,    12,    19,    0,    6,    0,    0,  163,  170,  187,  6,  6,  5,  7,  7, 17, 14, }, // Albanian/Latin/Kosovo
    { LOCALE_TAGS(  9, 66,140)  1217,   762,    12,    19,    0,    6,    0,    0,  163,  170,  187,  6,  6,  5,  7,  7, 17, 14, }, // Albanian/Latin/Macedonia
    { LOCALE_TAGS( 11, 33, 77)  1217,   762,    12,    19,   12,   17,    5,    0,  201,  207,  218,  5,  5,  8,  7,  6, 11, 15, }, // Amharic/Ethiopic/Ethiopia
    { LOCALE_TAGS( 14,  4, 71)  1634,   909,    12,    22,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Egypt
    { LOCALE_TAGS( 14,  4,  4)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Algeria
    { LOCALE_TAGS( 14,  4, 19)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Bahrain
    { LOCALE_TAGS( 14,  4, 48)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Chad
    { LOCALE_TAGS( 14,  4, 55)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Comoros
    { LOCALE_TAGS( 14,  4, 67)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Djibouti
    { LOCALE_TAGS( 14,  4, 74)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Eritrea
    { LOCALE_TAGS( 14,  4,113)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Iraq
    { LOCALE_TAGS( 14,  4,116)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Israel
    { LOCALE_TAGS( 14,  4,122)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Jordan
    { LOCALE_TAGS( 14,  4,127)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Kuwait
    { LOCALE_TAGS( 14,  4,132)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Lebanon
    { LOCALE_TAGS( 14,  4,135)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Libya
    { LOCALE_TAGS( 14,  4,149)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Mauritania
    { LOCALE_TAGS( 14,  4,159)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Morocco
    { LOCALE_TAGS( 14,  4,176)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Oman
    { LOCALE_TAGS( 14,  4,180)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Palestinian Territories
    { LOCALE_TAGS( 14,  4,190)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Qatar
    { LOCALE_TAGS( 14,  4,205)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Saudi Arabia
    { LOCALE_TAGS( 14,  4,215)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Somalia
    { LOCALE_TAGS( 14,  4,219)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/South Sudan
    { LOCALE_TAGS( 14,  4,222)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Sudan
    { LOCALE_TAGS( 14,  4,227)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Syria
    { LOCALE_TAGS( 14,  4,238)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Tunisia
    { LOCALE_TAGS( 14,  4,245)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/United Arab Emirates
    { LOCALE_TAGS( 14,  4,257)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Western Sahara
    { LOCALE_TAGS( 14,  4,258)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/world
    { LOCALE_TAGS( 14,  4,259)  2051,  1058,    13,    25,    0,    6,   13,    0,  233,  241,  256,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Yemen
    { LOCALE_TAGS( 15, 66,220)  2051,  1058,    13,    25,    0,    6,    0,    0,  271,  281,  300,  6,  6,  5,  7, 10, 19, 20, }, // Aragonese/Latin/Spain
    { LOCALE_TAGS( 17,  5, 12)  2092,  1067,    13,    26,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Armenian/Armenian/Armenia
    { LOCALE_TAGS( 18,  9,110)  2509,  1211,    13,    29,    0,    6,    0,    0,  320,  327,  343,  6,  6,  5,  7,  7, 16, 19, }, // Assamese/Bangla/India
    { LOCALE_TAGS( 19, 66,220)  2925,  1355,    14,    32,    0,    6,    0,    0,  271,  362,  381,  6,  6,  5,  7, 10, 19, 19, }, // Asturian/Latin/Spain
    { LOCALE_TAGS( 20, 66,230)  3065,  1510,    25,    36,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Asu/Latin/Tanzania
    { LOCALE_TAGS( 21, 66,169)  3106,  1510,    25,    37,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Atsam/Latin/Nigeria
    { LOCALE_TAGS( 25, 66, 17)  3147,  1510,    25,    38,    0,    6,    0,    0,  400,  408,  425,  6,  6,  5,  7,  8, 17, 12, }, // Azerbaijani/Latin/Azerbaijan
    { LOCALE_TAGS( 25,  4,112)  3453,  1655,    25,    41,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Arabic/Iran
    { LOCALE_TAGS( 25,  4,113)  3489,  1655,    25,    42,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Arabic/Iraq
    { LOCALE_TAGS( 25,  4,239)  3489,  1655,    25,    42,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Arabic/Turkey
    { LOCALE_TAGS( 25, 27, 17)  3489,  1655,    25,    42,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Cyrillic/Azerbaijan
    { LOCALE_TAGS( 26, 66, 40)  3525,  1655,    25,    43,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bafia/Latin/Cameroon
    { LOCALE_TAGS( 28, 66,145)  3566,  1655,    25,    44,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bambara/Latin/Mali
    { LOCALE_TAGS( 28, 90,145)  3607,  1655,    25,    45,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bambara/Nko/Mali
    { LOCALE_TAGS( 30,  9, 20)  3607,  1655,    25,    45,    0,    6,   21,    0,  320,  437,  449,  6,  6,  6,  7,  7, 12, 15, }, // Bangla/Bangla/Bangladesh
    { LOCALE_TAGS( 30,  9,110)  4024,  1811,    26,    48,    0,    6,   21,    0,  320,  437,  449,  6,  6,  6,  7,  7, 12, 15, }, // Bangla/Bangla/India
    { LOCALE_TAGS( 31, 66, 40)  4024,  1811,    26,    48,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Basaa/Latin/Cameroon
    { LOCALE_TAGS( 32, 27,193)  4065,  1811,    26,    49,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bashkir/Cyrillic/Russia
    { LOCALE_TAGS( 33, 66,220)  4106,  1811,    26,    50,    0,   22,    0,    0,  464,  479,  504,  6,  6,  5,  7, 15, 25, 16, }, // Basque/Latin/Spain
    { LOCALE_TAGS( 35, 27, 22)  4252,  1968,    37,    54,    0,    6,    0,    0,  520,  527,  545,  6,  6,  5,  7,  7, 18, 13, }, // Belarusian/Cyrillic/Belarus
    { LOCALE_TAGS( 36, 66,260)  4669,  2112,    37,    57,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bemba/Latin/Zambia
    { LOCALE_TAGS( 37, 66,230)  4710,  2112,    37,    58,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bena/Latin/Tanzania
    { LOCALE_TAGS( 38, 29,110)  4751,  2112,    37,    59,    0,    6,    0,    0,  558,  565,  576,  6,  6,  5,  7,  7, 11, 13, }, // Bhojpuri/Devanagari/India
    { LOCALE_TAGS( 40, 33, 74)  4792,  2113,    37,    60,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Blin/Ethiopic/Eritrea
    { LOCALE_TAGS( 41, 29,110)  4833,  2113,    37,    61,    0,    6,   27,    0,  589,  594,  607,  6,  6, 11,  7,  5, 13, 17, }, // Bodo/Devanagari/India
    { LOCALE_TAGS( 42, 66, 29)  5250,  2268,    38,    64,    0,   28,   21,    0,    0,  624,  646,  6,  7,  6,  7,  2, 22, 18, }, // Bosnian/Latin/Bosnia and Herzegovina
    { LOCALE_TAGS( 42, 27, 29)  5402,  2424,    42,    67,    0,    6,    0,    0,  664,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Bosnian/Cyrillic/Bosnia and Herzegovina
    { LOCALE_TAGS( 43, 66, 84)  5818,  2580,    42,    70,    0,    6,    0,    0,  673,  679,  693,  6,  6,  5,  7,  6, 14, 11, }, // Breton/Latin/France
    { LOCALE_TAGS( 45, 27, 36)  5939,  2728,    42,    73,    0,    6,   38,    0,    0,  704,  725,  6,  6,  9,  7,  2, 21, 23, }, // Bulgarian/Cyrillic/Bulgaria
    { LOCALE_TAGS( 46, 86,161)  6356,  2875,    42,    76,    0,    6,    0,    0,  748,  757,  771,  6,  6,  5,  7,  9, 14, 22, }, // Burmese/Myanmar/Myanmar
    { LOCALE_TAGS( 47,137,107)  6773,  3021,    42,    79,    0,    6,    0,    0,  793,  797,  803,  6,  6,  5,  7,  4,  6,  6, }, // Cantonese/Traditional Han/Hong Kong
    { LOCALE_TAGS( 47,118, 50)  7190,  3178,    42,    82,    0,    6,    0,    0,  809,   83,   90,  6,  6,  5,  7,  4,  7,  7, }, // Cantonese/Simplified Han/China
    { LOCALE_TAGS( 47,137, 50)  7452,  3335,    42,    85,    0,    6,    0,    0,  793,  797,  803,  6,  6,  5,  7,  4,  6,  6, }, // Cantonese/Traditional Han/China
    { LOCALE_TAGS( 48, 66,220)  7452,  3335,    42,    85,    0,    6,    0,    0,  813,  824,  842,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/Spain
    { LOCALE_TAGS( 48, 66,  6)  7596,  3484,    46,    88,    0,    6,    0,    0,  813,  824,  842,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/Andorra
    { LOCALE_TAGS( 48, 66, 84)  7596,  3484,    46,    88,    0,    6,    0,    0,  813,  824,  842,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/France
    { LOCALE_TAGS( 48, 66,117)  7596,  3484,    46,    88,    0,    6,    0,    0,  813,  824,  842,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/Italy
    { LOCALE_TAGS( 49, 66,185)  7596,  3484,    46,    88,    0,    6,   21,    7,  858,  868,  891,  6,  6,  6,  5, 10, 23, 23, }, // Cebuano/Latin/Philippines
    { LOCALE_TAGS( 50, 66,159)  7638,  3628,    46,    91,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Central Atlas Tamazight/Latin/Morocco
    { LOCALE_TAGS( 51,  4,113)  7679,  3628,    46,    92,    0,    6,   47,    0,  914,  923,  936,  6,  6,  9,  7,  9, 13, 14, }, // Central Kurdish/Arabic/Iraq
    { LOCALE_TAGS( 51,  4,112)  7720,  3628,    46,    93,    0,    6,   47,    0,  914,  923,  936,  6,  6,  9,  7,  9, 13, 14, }, // Central Kurdish/Arabic/Iran
    { LOCALE_TAGS( 52, 21, 20)  7720,  3628,    46,    93,    0,    6,   21,    0,  950,  965,  995,  6,  6,  6,  7, 15, 30, 38, }, // Chakma/Chakma/Bangladesh
    { LOCALE_TAGS( 52, 21,110)  8132,  3781,    47,    96,    0,    6,   21,    0,  950,  965,  995,  6,  6,  6,  7, 15, 30, 38, }, // Chakma/Chakma/India
    { LOCALE_TAGS( 54, 27,193)  8132,  3781,    47,    96,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Chechen/Cyrillic/Russia
    { LOCALE_TAGS( 55, 23,248)  8548,  3923,    47,    99,    0,    6,    0,    0, 1033, 1041, 1055,  6,  6,  5,  7,  8, 14, 14, }, // Cherokee/Cherokee/United States
    { LOCALE_TAGS( 56, 66,248)  8965,  4067,    54,   103,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Chickasaw/Latin/United States
    { LOCALE_TAGS( 57, 66,243)  9006,  4067,    54,   104,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Chiga/Latin/Uganda
    { LOCALE_TAGS( 58,118, 50)  9047,  4067,    54,   105,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/China
    { LOCALE_TAGS( 58, 66, 50)  9464,  4224,    54,   108,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Latin/China
    { LOCALE_TAGS( 58,118,107)  9464,  4224,    54,   108,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/Hong Kong
    { LOCALE_TAGS( 58,118,139)  9464,  4224,    54,   108,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/Macao
    { LOCALE_TAGS( 58,118,143)  9464,  4224,    54,   108,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/Malaysia
    { LOCALE_TAGS( 58,118,210)  9464,  4224,    54,   108,    0,    6,    0,   12,  809, 1069, 1075,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/Singapore
    { LOCALE_TAGS( 58,137,107)  9465,  4226,    56,   108,    0,    6,    0,   12,  793,  797,  803,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Traditional Han/Hong Kong
    { LOCALE_TAGS( 58,137,139)  9595,  4259,    60,   108,    0,    6,    0,   12,  793,  797,  803,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Traditional Han/Macao
    { LOCALE_TAGS( 58,137,143)  9725,  4292,    64,   108,    0,    6,    0,   12,  793,   83,   90,  6,  6,  5,  6,  4,  7,  7, }, // Chinese/Traditional Han/Malaysia
    { LOCALE_TAGS( 58,137,228)  9725,  4292,    64,   108,    0,    6,    0,   12,  793,   83,   90,  6,  6,  5,  6,  4,  7,  7, }, // Chinese/Traditional Han/Taiwan
    { LOCALE_TAGS( 59, 27,193) 10059,  4449,    72,   111,    0,    6,    0,    0, 1081, 1092, 1111,  6,  6,  5,  7, 11, 19, 19, }, // Church/Cyrillic/Russia
    { LOCALE_TAGS( 60, 27,193) 10128,  4473,    72,   112,    0,    6,    0,    0,    0, 1130, 1150,  6,  6,  5,  7,  2, 20, 17, }, // Chuvash/Cyrillic/Russia
    { LOCALE_TAGS( 61, 66, 91) 10544,  4631,   230,   116,    0,    6,    0,    0, 1167, 1178, 1199,  6,  6,  5,  7, 11, 21, 17, }, // Colognian/Latin/Germany
    { LOCALE_TAGS( 63, 66,246) 10650,  4645,   233,   119,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Cornish/Latin/United Kingdom
    { LOCALE_TAGS( 64, 66, 84) 10691,  4645,   237,   120,    0,    6,   56,    0, 1216, 1224, 1239,  6,  6,  5,  7,  8, 15, 17, }, // Corsican/Latin/France
    { LOCALE_TAGS( 66, 66, 60) 10732,  4646,   237,   121,    0,   28,    0,    0,    0,  624,  646,  6,  7,  5,  7,  2, 22, 18, }, // Croatian/Latin/Croatia
    { LOCALE_TAGS( 66, 66, 29) 10864,  4803,   241,   124,    0,   28,    0,    0,    0,  624,  646,  6,  7,  5,  7,  2, 22, 18, }, // Croatian/Latin/Bosnia and Herzegovina
    { LOCALE_TAGS( 67, 66, 64) 10864,  4803,   241,   124,   35,   40,    0,    0, 1256,   83,   90,  5,  5,  5,  7, 15,  7,  7, }, // Czech/Latin/Czechia
    { LOCALE_TAGS( 68, 66, 65) 11032,  4960,   248,   128,   45,   51,    0,    0, 1271, 1277, 1289,  6,  6,  5,  7,  6, 12, 12, }, // Danish/Latin/Denmark
    { LOCALE_TAGS( 68, 66, 95) 11121,  5116,   251,   131,   45,   51,    0,    0, 1271, 1277, 1289,  6,  6,  5,  7,  6, 12, 12, }, // Danish/Latin/Greenland
    { LOCALE_TAGS( 69,132,144) 11121,  5116,   251,   131,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Divehi/Thaana/Maldives
    { LOCALE_TAGS( 70, 29,110) 11162,  5116,   251,   132,    0,    6,    0,    0, 1301, 1308, 1320,  6,  6,  5,  7,  7, 12, 14, }, // Dogri/Devanagari/India
    { LOCALE_TAGS( 71, 66, 40) 11203,  5125,   251,   135,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Duala/Latin/Cameroon
    { LOCALE_TAGS( 72, 66,165) 11244,  5125,   251,   136,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Netherlands
    { LOCALE_TAGS( 72, 66, 13) 11359,  5282,   262,   140,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Aruba
    { LOCALE_TAGS( 72, 66, 23) 11359,  5282,   262,   140,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Belgium
    { LOCALE_TAGS( 72, 66, 44) 11359,  5282,   262,   140,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Caribbean Netherlands
    { LOCALE_TAGS( 72, 66, 62) 11359,  5282,   262,   140,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Curacao
    { LOCALE_TAGS( 72, 66,211) 11359,  5282,   262,   140,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Sint Maarten
    { LOCALE_TAGS( 72, 66,223) 11359,  5282,   262,   140,    0,    6,    0,    0, 1334, 1344, 1363,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Suriname
    { LOCALE_TAGS( 73,134, 27) 11359,  5283,   263,   140,    0,    6,   61,   18, 1378,   83,   90,  6,  6, 12,  9, 10,  7,  7, }, // Dzongkha/Tibetan/Bhutan
    { LOCALE_TAGS( 74, 66,124) 11547,  5366,   264,   143,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Embu/Latin/Kenya
    { LOCALE_TAGS( 75, 66,248) 11588,  5366,   264,   144,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United States
    { LOCALE_TAGS( 75, 28,248) 11630,  5524,   272,   148,    0,    6,   73,    0, 1388,   83,   90,  6,  6,  9,  7,  9,  7,  7, }, // English/Deseret/United States
    { LOCALE_TAGS( 75, 66,  5) 11659,  5532,   272,   149,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/American Samoa
    { LOCALE_TAGS( 75, 66,  8) 11659,  5532,   272,   149,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Anguilla
    { LOCALE_TAGS( 75, 66, 10) 11668,  5540,   279,   150,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Antigua and Barbuda
    { LOCALE_TAGS( 75, 66, 15) 11677,  5548,   286,   151,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Australia
    { LOCALE_TAGS( 75, 66, 16) 11686,  5572,   300,   152,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Austria
    { LOCALE_TAGS( 75, 66, 18) 11695,  5583,   310,   153,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Bahamas
    { LOCALE_TAGS( 75, 66, 21) 11704,  5591,   317,   154,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Barbados
    { LOCALE_TAGS( 75, 66, 23) 11713,  5599,   324,   155,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Belgium
    { LOCALE_TAGS( 75, 66, 24) 11722,  5610,   334,   156,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Belize
    { LOCALE_TAGS( 75, 66, 26) 11731,  5618,   341,   157,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Bermuda
    { LOCALE_TAGS( 75, 66, 30) 11740,  5626,   348,   158,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Botswana
    { LOCALE_TAGS( 75, 66, 33) 11749,  5638,   359,   159,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/British Indian Ocean Territory
    { LOCALE_TAGS( 75, 66, 34) 11758,  5646,   366,   160,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/British Virgin Islands
    { LOCALE_TAGS( 75, 66, 38) 11767,  5654,   373,   161,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Burundi
    { LOCALE_TAGS( 75, 66, 40) 11767,  5654,   373,   161,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cameroon
    { LOCALE_TAGS( 75, 66, 41) 11776,  5666,   384,   162,    0,    6,    0,    0,    0,    7, 1397,  6,  6,  5,  7,  7, 16, 23, }, // English/Latin/Canada
    { LOCALE_TAGS( 75, 66, 45) 11784,  5710,   425,   162,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cayman Islands
    { LOCALE_TAGS( 75, 66, 51) 11793,  5718,   432,   163,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Christmas Island
    { LOCALE_TAGS( 75, 66, 53) 11802,  5726,   439,   164,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cocos Islands
    { LOCALE_TAGS( 75, 66, 58) 11811,  5734,   446,   165,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cook Islands
    { LOCALE_TAGS( 75, 66, 63) 11820,  5742,   453,   166,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cyprus
    { LOCALE_TAGS( 75, 66, 65) 11829,  5750,   460,   167,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Denmark
    { LOCALE_TAGS( 75, 66, 66) 11838,  5761,   470,   168,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Diego Garcia
    { LOCALE_TAGS( 75, 66, 68) 11847,  5769,   477,   169,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Dominica
    { LOCALE_TAGS( 75, 66, 74) 11856,  5777,   484,   170,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Eritrea
    { LOCALE_TAGS( 75, 66, 76) 11865,  5789,   495,   171,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Eswatini
    { LOCALE_TAGS( 75, 66, 78) 11874,  5801,   506,   172,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Europe
    { LOCALE_TAGS( 75, 66, 80) 11883,  5812,   516,   173,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Falkland Islands
    { LOCALE_TAGS( 75, 66, 82) 11892,  5820,   523,   174,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Fiji
    { LOCALE_TAGS( 75, 66, 83) 11901,  5828,   530,   175,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Finland
    { LOCALE_TAGS( 75, 66, 89) 11910,  5839,   540,   176,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Gambia
    { LOCALE_TAGS( 75, 66, 91) 11919,  5851,   551,   177,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Germany
    { LOCALE_TAGS( 75, 66, 92) 11928,  5862,   561,   178,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Ghana
    { LOCALE_TAGS( 75, 66, 93) 11937,  5874,   572,   179,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Gibraltar
    { LOCALE_TAGS( 75, 66, 96) 11946,  5882,   579,   180,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Grenada
    { LOCALE_TAGS( 75, 66, 98) 11955,  5890,   586,   181,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Guam
    { LOCALE_TAGS( 75, 66,100) 11955,  5891,   587,   181,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Guernsey
    { LOCALE_TAGS( 75, 66,103) 11964,  5899,   594,   182,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Guyana
    { LOCALE_TAGS( 75, 66,107) 11973,  5908,   602,   183,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Hong Kong
    { LOCALE_TAGS( 75, 66,110) 11982,  5917,   610,   184,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/India
    { LOCALE_TAGS( 75, 66,111) 11993,  5928,   619,   185,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Indonesia
    { LOCALE_TAGS( 75, 66,114) 12002,  5939,   629,   186,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Ireland
    { LOCALE_TAGS( 75, 66,115) 12011,  5947,   636,   188,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Isle of Man
    { LOCALE_TAGS( 75, 66,116) 12020,  5955,   643,   189,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Israel
    { LOCALE_TAGS( 75, 66,119) 12029,  5963,   650,   190,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Jamaica
    { LOCALE_TAGS( 75, 66,121) 12038,  5971,   657,   191,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Jersey
    { LOCALE_TAGS( 75, 66,124) 12047,  5979,   664,   192,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Kenya
    { LOCALE_TAGS( 75, 66,125) 12056,  5991,   675,   193,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Kiribati
    { LOCALE_TAGS( 75, 66,133) 12065,  5999,   682,   194,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Lesotho
    { LOCALE_TAGS( 75, 66,134) 12074,  6011,   693,   195,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Liberia
    { LOCALE_TAGS( 75, 66,139) 12083,  6023,   704,   196,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Macao
    { LOCALE_TAGS( 75, 66,141) 12092,  6033,   713,   197,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Madagascar
    { LOCALE_TAGS( 75, 66,142) 12101,  6045,   724,   198,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Malawi
    { LOCALE_TAGS( 75, 66,143) 12110,  6057,   735,   199,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Malaysia
    { LOCALE_TAGS( 75, 66,144) 12119,  6066,   743,   200,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Maldives
    { LOCALE_TAGS( 75, 66,146) 12128,  6074,   750,   201,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Malta
    { LOCALE_TAGS( 75, 66,147) 12137,  6082,   757,   202,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Marshall Islands
    { LOCALE_TAGS( 75, 66,150) 12137,  6089,   764,   203,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Mauritius
    { LOCALE_TAGS( 75, 66,153) 12146,  6101,   775,   204,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Micronesia
    { LOCALE_TAGS( 75, 66,158) 12155,  6109,   782,   205,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Montserrat
    { LOCALE_TAGS( 75, 66,162) 12164,  6117,   789,   206,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Namibia
    { LOCALE_TAGS( 75, 66,163) 12173,  6129,   800,   207,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Nauru
    { LOCALE_TAGS( 75, 66,165) 12182,  6137,   807,   208,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Netherlands
    { LOCALE_TAGS( 75, 66,167) 12191,  6148,   817,   209,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/New Zealand
    { LOCALE_TAGS( 75, 66,169) 12200,  6163,   831,   210,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Nigeria
    { LOCALE_TAGS( 75, 66,171) 12209,  6175,   842,   211,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Niue
    { LOCALE_TAGS( 75, 66,172) 12218,  6183,   849,   212,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Norfolk Island
    { LOCALE_TAGS( 75, 66,173) 12227,  6191,   856,   213,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Northern Mariana Islands
    { LOCALE_TAGS( 75, 66,178) 12227,  6198,   863,   214,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Pakistan
    { LOCALE_TAGS( 75, 66,179) 12236,  6206,   870,   215,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Palau
    { LOCALE_TAGS( 75, 66,182) 12245,  6214,   877,   216,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Papua New Guinea
    { LOCALE_TAGS( 75, 66,185) 12254,  6222,   884,   217,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Philippines
    { LOCALE_TAGS( 75, 66,186) 12254,  6222,   884,   217,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Pitcairn
    { LOCALE_TAGS( 75, 66,189) 12263,  6230,   891,   218,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Puerto Rico
    { LOCALE_TAGS( 75, 66,194) 12263,  6230,   891,   218,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Rwanda
    { LOCALE_TAGS( 75, 66,196) 12272,  6242,   902,   219,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Helena
    { LOCALE_TAGS( 75, 66,197) 12281,  6250,   909,   220,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Kitts and Nevis
    { LOCALE_TAGS( 75, 66,198) 12290,  6258,   916,   221,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Lucia
    { LOCALE_TAGS( 75, 66,201) 12299,  6266,   923,   222,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Vincent and Grenadines
    { LOCALE_TAGS( 75, 66,202) 12308,  6274,   930,   223,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Samoa
    { LOCALE_TAGS( 75, 66,208) 12317,  6282,   937,   224,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Seychelles
    { LOCALE_TAGS( 75, 66,209) 12326,  6290,   944,   225,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sierra Leone
    { LOCALE_TAGS( 75, 66,210) 12335,  6302,   955,   226,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Singapore
    { LOCALE_TAGS( 75, 66,211) 12344,  6312,   964,   227,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sint Maarten
    { LOCALE_TAGS( 75, 66,213) 12353,  6320,   971,   228,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Slovenia
    { LOCALE_TAGS( 75, 66,214) 12362,  6331,   981,   229,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Solomon Islands
    { LOCALE_TAGS( 75, 66,216) 12371,  6339,   988,   230,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/South Africa
    { LOCALE_TAGS( 75, 66,219) 12380,  6351,   999,   231,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/South Sudan
    { LOCALE_TAGS( 75, 66,222) 12389,  6363,  1010,   232,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sudan
    { LOCALE_TAGS( 75, 66,225) 12398,  6375,  1021,   233,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sweden
    { LOCALE_TAGS( 75, 66,226) 12407,  6386,  1031,   234,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Switzerland
    { LOCALE_TAGS( 75, 66,230) 12416,  6397,  1041,   235,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tanzania
    { LOCALE_TAGS( 75, 66,234) 12425,  6409,  1052,   236,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tokelau
    { LOCALE_TAGS( 75, 66,235) 12434,  6417,  1059,   237,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tonga
    { LOCALE_TAGS( 75, 66,236) 12443,  6425,  1066,   238,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Trinidad and Tobago
    { LOCALE_TAGS( 75, 66,241) 12452,  6433,  1073,   239,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Turks and Caicos Islands
    { LOCALE_TAGS( 75, 66,242) 12461,  6441,  1080,   240,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tuvalu
    { LOCALE_TAGS( 75, 66,243) 12470,  6449,  1087,   241,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Uganda
    { LOCALE_TAGS( 75, 66,245) 12479,  6461,  1098,   242,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United Arab Emirates
    { LOCALE_TAGS( 75, 66,246) 12479,  6469,  1106,   243,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United Kingdom
    { LOCALE_TAGS( 75, 66,247) 12491,  6481,  1117,   245,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United States Outlying Islands
    { LOCALE_TAGS( 75, 66,249) 12491,  6481,  1117,   245,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United States Virgin Islands
    { LOCALE_TAGS( 75, 66,252) 12491,  6481,  1117,   245,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Vanuatu
    { LOCALE_TAGS( 75, 66,258) 12500,  6489,  1124,   246,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/world
    { LOCALE_TAGS( 75, 66,260) 12509,  6497,  1131,   247,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Zambia
    { LOCALE_TAGS( 75, 66,261) 12518,  6509,  1142,   248,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Zimbabwe
    { LOCALE_TAGS( 75,115,246) 12527,  6521,  1153,   249,    0,    6,   82,    0, 1420,   83,   90,  6,  6,  9,  7,  9,  7,  7, }, // English/Shavian/United Kingdom
    { LOCALE_TAGS( 76, 27,193) 12528,  6521,  1153,   250,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Erzya/Cyrillic/Russia
    { LOCALE_TAGS( 77, 66,258) 12636,  6542,  1153,   251,    0,    6,   56,    0, 1429, 1440, 1457,  6,  6,  5,  7, 11, 17, 18, }, // Esperanto/Latin/world
    { LOCALE_TAGS( 78, 66, 75) 12959,  6686,  1153,   254,    0,   57,   21,    0, 1475,   83,   90,  6,  6,  6,  7,  4,  7,  7, }, // Estonian/Latin/Estonia
    { LOCALE_TAGS( 79, 66, 92) 13101,  6844,  1153,   257,    0,    6,   91,    0, 1479, 1486, 1500,  6,  6,  6,  7,  7, 14, 16, }, // Ewe/Latin/Ghana
    { LOCALE_TAGS( 79, 66,233) 13144,  6994,  1157,   260,    0,    6,   91,    0, 1479, 1486, 1500,  6,  6,  6,  7,  7, 14, 16, }, // Ewe/Latin/Togo
    { LOCALE_TAGS( 80, 66, 40) 13144,  6994,  1157,   260,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ewondo/Latin/Cameroon
    { LOCALE_TAGS( 81, 66, 81) 13185,  6994,  1157,   261,    0,    6,    0,    0, 1516, 1522, 1535,  6,  6,  5,  7,  6, 13, 12, }, // Faroese/Latin/Faroe Islands
    { LOCALE_TAGS( 81, 66, 65) 13261,  7138,  1157,   264,    0,    6,    0,    0, 1516, 1522, 1535,  6,  6,  5,  7,  6, 13, 12, }, // Faroese/Latin/Denmark
    { LOCALE_TAGS( 83, 66,185) 13261,  7138,  1157,   264,    0,    6,    0,    0,  858, 1547, 1569,  6,  6,  5,  7, 10, 22, 19, }, // Filipino/Latin/Philippines
    { LOCALE_TAGS( 84, 66, 83) 13308,  7285,  1157,   267,   63,   68,   56,    0, 1588, 1603, 1620,  5,  5,  5,  7, 15, 17, 13, }, // Finnish/Latin/Finland
    { LOCALE_TAGS( 85, 66, 84) 13450,  7442,  1157,   270,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/France
    { LOCALE_TAGS( 85, 66,  4) 13605,  7599,  1173,   274,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Algeria
    { LOCALE_TAGS( 85, 66, 23) 13605,  7599,  1173,   274,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Belgium
    { LOCALE_TAGS( 85, 66, 25) 13605,  7599,  1173,   274,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Benin
    { LOCALE_TAGS( 85, 66, 37) 13605,  7599,  1173,   274,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Burkina Faso
    { LOCALE_TAGS( 85, 66, 38) 13605,  7599,  1173,   274,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Burundi
    { LOCALE_TAGS( 85, 66, 40) 13605,  7599,  1173,   274,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Cameroon
    { LOCALE_TAGS( 85, 66, 41) 13605,  7599,  1173,   274,    0,   57,   56,    0, 1633, 1678, 1696,  6,  6,  5,  7, 10, 18, 18, }, // French/Latin/Canada
    { LOCALE_TAGS( 85, 66, 46) 13625,  7689,  1188,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Central African Republic
    { LOCALE_TAGS( 85, 66, 48) 13625,  7689,  1188,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Chad
    { LOCALE_TAGS( 85, 66, 55) 13625,  7689,  1188,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Comoros
    { LOCALE_TAGS( 85, 66, 56) 13625,  7689,  1188,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Congo - Brazzaville
    { LOCALE_TAGS( 85, 66, 57) 13625,  7689,  1188,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Congo - Kinshasa
    { LOCALE_TAGS( 85, 66, 67) 13625,  7689,  1188,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Djibouti
    { LOCALE_TAGS( 85, 66, 73) 13625,  7689,  1188,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Equatorial Guinea
    { LOCALE_TAGS( 85, 66, 85) 13625,  7689,  1188,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/French Guiana
    { LOCALE_TAGS( 85, 66, 86) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/French Polynesia
    { LOCALE_TAGS( 85, 66, 88) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Gabon
    { LOCALE_TAGS( 85, 66, 97) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Guadeloupe
    { LOCALE_TAGS( 85, 66,102) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Guinea
    { LOCALE_TAGS( 85, 66,104) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Haiti
    { LOCALE_TAGS( 85, 66,118) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Ivory Coast
    { LOCALE_TAGS( 85, 66,138) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Luxembourg
    { LOCALE_TAGS( 85, 66,141) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Madagascar
    { LOCALE_TAGS( 85, 66,145) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mali
    { LOCALE_TAGS( 85, 66,148) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Martinique
    { LOCALE_TAGS( 85, 66,149) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mauritania
    { LOCALE_TAGS( 85, 66,150) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mauritius
    { LOCALE_TAGS( 85, 66,151) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mayotte
    { LOCALE_TAGS( 85, 66,155) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Monaco
    { LOCALE_TAGS( 85, 66,159) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Morocco
    { LOCALE_TAGS( 85, 66,166) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/New Caledonia
    { LOCALE_TAGS( 85, 66,170) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Niger
    { LOCALE_TAGS( 85, 66,191) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Reunion
    { LOCALE_TAGS( 85, 66,194) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Rwanda
    { LOCALE_TAGS( 85, 66,195) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Saint Barthelemy
    { LOCALE_TAGS( 85, 66,199) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Saint Martin
    { LOCALE_TAGS( 85, 66,200) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Saint Pierre and Miquelon
    { LOCALE_TAGS( 85, 66,206) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Senegal
    { LOCALE_TAGS( 85, 66,208) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Seychelles
    { LOCALE_TAGS( 85, 66,226) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Switzerland
    { LOCALE_TAGS( 85, 66,227) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Syria
    { LOCALE_TAGS( 85, 66,233) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Togo
    { LOCALE_TAGS( 85, 66,238) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Tunisia
    { LOCALE_TAGS( 85, 66,252) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Vanuatu
    { LOCALE_TAGS( 85, 66,256) 13625,  7690,  1189,   276,    0,   57,   56,    0, 1633, 1643, 1662,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Wallis and Futuna
    { LOCALE_TAGS( 86, 66,117) 13625,  7690,  1189,   276,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Friulian/Latin/Italy
    { LOCALE_TAGS( 87, 66,206) 13679,  7694,  1193,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Senegal
    { LOCALE_TAGS( 87,  1, 37) 13720,  7694,  1193,   278,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Burkina Faso
    { LOCALE_TAGS( 87,  1, 40) 13720,  7694,  1193,   278,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Cameroon
    { LOCALE_TAGS( 87,  1, 89) 13720,  7694,  1193,   278,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Gambia
    { LOCALE_TAGS( 87,  1, 92) 13720,  7694,  1193,   278,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Ghana
    { LOCALE_TAGS( 87,  1,101) 13720,  7694,  1193,   278,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Guinea-Bissau
    { LOCALE_TAGS( 87,  1,102) 13720,  7694,  1193,   278,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Guinea
    { LOCALE_TAGS( 87,  1,134) 14135,  7845,  1193,   282,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Liberia
    { LOCALE_TAGS( 87,  1,149) 14135,  7845,  1193,   282,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Mauritania
    { LOCALE_TAGS( 87,  1,169) 14135,  7845,  1193,   282,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Nigeria
    { LOCALE_TAGS( 87,  1,170) 14135,  7845,  1193,   282,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Niger
    { LOCALE_TAGS( 87,  1,206) 14135,  7845,  1193,   282,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Senegal
    { LOCALE_TAGS( 87,  1,209) 14135,  7845,  1193,   282,    0,    6,   97,    0, 1714, 1731, 1767,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Sierra Leone
    { LOCALE_TAGS( 87, 66, 37) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Burkina Faso
    { LOCALE_TAGS( 87, 66, 40) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Cameroon
    { LOCALE_TAGS( 87, 66, 89) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Gambia
    { LOCALE_TAGS( 87, 66, 92) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Ghana
    { LOCALE_TAGS( 87, 66,101) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Guinea-Bissau
    { LOCALE_TAGS( 87, 66,102) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Guinea
    { LOCALE_TAGS( 87, 66,134) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Liberia
    { LOCALE_TAGS( 87, 66,149) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Mauritania
    { LOCALE_TAGS( 87, 66,169) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Nigeria
    { LOCALE_TAGS( 87, 66,170) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Niger
    { LOCALE_TAGS( 87, 66,209) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Sierra Leone
    { LOCALE_TAGS( 88, 66,246) 14135,  7845,  1193,   282,    0,    6,    0,    0,    0, 1795, 1805,  6,  6,  5,  7,  2, 10, 18, }, // Gaelic/Latin/United Kingdom
    { LOCALE_TAGS( 89, 66, 92) 14288,  8003,  1204,   286,    0,    6,    0,    0, 1823, 1828, 1846,  6,  6,  5,  7,  5, 18, 20, }, // Ga/Latin/Ghana
    { LOCALE_TAGS( 90, 66,220) 14392,  8028,  1204,   289,    0,    6,    0,    0, 1866, 1877, 1897,  6,  6,  5,  7, 11, 20, 20, }, // Galician/Latin/Spain
    { LOCALE_TAGS( 91, 66,243) 14564,  8176,  1208,   292,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ganda/Latin/Uganda
    { LOCALE_TAGS( 92, 33, 77) 14605,  8176,  1208,   293,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Geez/Ethiopic/Ethiopia
    { LOCALE_TAGS( 92, 33, 74) 14646,  8176,  1208,   294,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Geez/Ethiopic/Eritrea
    { LOCALE_TAGS( 93, 35, 90) 14646,  8176,  1208,   294,    0,    6,    0,    0, 1917, 1924, 1942,  6,  6,  5,  7,  7, 18, 15, }, // Georgian/Georgian/Georgia
    { LOCALE_TAGS( 94, 66, 91) 15063,  8320,  1208,   297,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Germany
    { LOCALE_TAGS( 94, 66, 16) 15192,  8477,  1211,   300,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Austria
    { LOCALE_TAGS( 94, 66, 23) 15192,  8477,  1211,   300,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Belgium
    { LOCALE_TAGS( 94, 66,117) 15192,  8477,  1211,   300,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Italy
    { LOCALE_TAGS( 94, 66,136) 15192,  8477,  1211,   300,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Liechtenstein
    { LOCALE_TAGS( 94, 66,138) 15192,  8477,  1211,   300,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Luxembourg
    { LOCALE_TAGS( 94, 66,226) 15192,  8477,  1211,   300,    0,    6,    0,    0, 1957, 1970, 1985,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Switzerland
    { LOCALE_TAGS( 96, 39, 94) 15195,  8479,  1211,   300,    0,    6,    0,   27, 2000, 2008, 2026,  6,  6,  5,  9,  8, 18, 15, }, // Greek/Greek/Greece
    { LOCALE_TAGS( 96, 39, 63) 15612,  8629,  1214,   303,    0,    6,    0,   27, 2000, 2008, 2026,  6,  6,  5,  9,  8, 18, 15, }, // Greek/Greek/Cyprus
    { LOCALE_TAGS( 97, 66,183) 15612,  8629,  1214,   303,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Guarani/Latin/Paraguay
    { LOCALE_TAGS( 98, 40,110) 15653,  8633,  1214,   304,    0,    6,    0,    0, 2041, 2047, 2058,  6,  6,  5,  7,  6, 11, 11, }, // Gujarati/Gujarati/India
    { LOCALE_TAGS( 99, 66,124) 16070,  8789,  1215,   307,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Gusii/Latin/Kenya
    { LOCALE_TAGS(101, 66,169) 16111,  8789,  1215,   308,    0,    6,    0,    0, 2069, 2078, 2100,  6,  6,  5,  7,  9, 22, 15, }, // Hausa/Latin/Nigeria
    { LOCALE_TAGS(101,  4,169) 16152,  8933,  1215,   311,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Hausa/Arabic/Nigeria
    { LOCALE_TAGS(101,  4,222) 16155,  8933,  1215,   312,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Hausa/Arabic/Sudan
    { LOCALE_TAGS(101, 66, 92) 16158,  8933,  1215,   313,    0,    6,    0,    0, 2069, 2078, 2100,  6,  6,  5,  7,  9, 22, 15, }, // Hausa/Latin/Ghana
    { LOCALE_TAGS(101, 66,170) 16158,  8933,  1215,   313,    0,    6,    0,    0, 2069, 2078, 2100,  6,  6,  5,  7,  9, 22, 15, }, // Hausa/Latin/Niger
    { LOCALE_TAGS(102, 66,248) 16158,  8933,  1215,   313,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Hawaiian/Latin/United States
    { LOCALE_TAGS(103, 47,116) 16199,  8933,  1217,   315,   73,   80,  105,    0, 2115, 2122, 2136,  7,  7,  6,  7,  7, 14, 13, }, // Hebrew/Hebrew/Israel
    { LOCALE_TAGS(105, 29,110) 16616,  9087,  1217,   318,    0,    6,    0,    0, 2149,  565,  576,  6,  6,  5,  7,  6, 11, 13, }, // Hindi/Devanagari/India
    { LOCALE_TAGS(105, 66,110) 17033,  9234,  1218,   322,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // Hindi/Latin/India
    { LOCALE_TAGS(107, 66,108) 17082,  9392,  1228,   326,    0,    6,    0,    0, 2155, 2161, 2171,  6,  6,  5,  7,  6, 10, 12, }, // Hungarian/Latin/Hungary
    { LOCALE_TAGS(108, 66,109) 17269,  9549,  1232,   329,    0,    6,    0,    0,    0, 2183, 2198,  6,  6,  5,  7,  2, 15, 14, }, // Icelandic/Latin/Iceland
    { LOCALE_TAGS(109, 66,258) 17407,  9696,  1232,   332,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ido/Latin/world
    { LOCALE_TAGS(110, 66,169) 17448,  9696,  1232,   333,    0,    6,    0,    0, 2212, 2218, 2231,  6,  6,  5,  7,  6, 13, 10, }, // Igbo/Latin/Nigeria
    { LOCALE_TAGS(111, 66, 83) 17490,  9840,  1232,   336,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Inari Sami/Latin/Finland
    { LOCALE_TAGS(112, 66,111) 17531,  9840,  1232,   337,   45,   51,    0,    0, 2241, 2249, 2265,  6,  6,  5,  7,  8, 16, 20, }, // Indonesian/Latin/Indonesia
    { LOCALE_TAGS(114, 66,258) 17608,  9997,  1246,   340,    0,    6,    0,    0, 2285, 2295, 2312,  6,  6,  5,  7, 10, 17, 17, }, // Interlingua/Latin/world
    { LOCALE_TAGS(115, 66, 75) 17684, 10141,  1246,   343,    0,    6,  111,    0, 2329, 2341, 2362,  6,  6,  5,  7, 12, 21, 20, }, // Interlingue/Latin/Estonia
    { LOCALE_TAGS(116, 18, 41) 17802, 10172,  1246,   346,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Inuktitut/Canadian Aboriginal/Canada
    { LOCALE_TAGS(116, 66, 41) 17843, 10172,  1246,   347,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Inuktitut/Latin/Canada
    { LOCALE_TAGS(118, 66,114) 17843, 10172,  1246,   347,    0,    6,  116,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Irish/Latin/Ireland
    { LOCALE_TAGS(118, 66,246) 18013, 10329,  1251,   350,    0,    6,  116,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Irish/Latin/United Kingdom
    { LOCALE_TAGS(119, 66,117) 18013, 10329,  1251,   350,    0,    6,    0,    0, 2382, 2388, 2404,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/Italy
    { LOCALE_TAGS(119, 66,203) 18135, 10477,  1254,   353,    0,    6,    0,    0, 2382, 2388, 2404,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/San Marino
    { LOCALE_TAGS(119, 66,226) 18135, 10477,  1254,   353,    0,    6,    0,    0, 2382, 2388, 2404,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/Switzerland
    { LOCALE_TAGS(119, 66,253) 18135, 10477,  1254,   353,    0,    6,    0,    0, 2382, 2388, 2404,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/Vatican City
    { LOCALE_TAGS(120, 53,120) 18135, 10477,  1254,   353,    0,    6,    0,   12,  793,  797, 2418,  6,  6,  5,  6,  4,  5,  5, }, // Japanese/Japanese/Japan
    { LOCALE_TAGS(121, 66,111) 18552, 10634,  1255,   356,    0,    6,    0,    0, 2423, 2431, 2447,  6,  6,  5,  7,  8, 16, 15, }, // Javanese/Latin/Indonesia
    { LOCALE_TAGS(122, 66,169) 18641, 10778,  1255,   359,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Jju/Latin/Nigeria
    { LOCALE_TAGS(123, 66,206) 18682, 10778,  1255,   360,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Jola-Fonyi/Latin/Senegal
    { LOCALE_TAGS(124, 66, 43) 18723, 10778,  1255,   361,    0,    6,    0,    0, 2462, 2471, 2487,  6,  6,  5,  7,  9, 16, 18, }, // Kabuverdianu/Latin/Cape Verde
    { LOCALE_TAGS(125, 66,  4) 18795, 10805,  1255,   364,    0,    6,  121,    0, 2505, 2513, 2528,  6,  6,  6,  7,  8, 15, 16, }, // Kabyle/Latin/Algeria
    { LOCALE_TAGS(126, 66, 40) 19029, 10947,  1255,   367,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kako/Latin/Cameroon
    { LOCALE_TAGS(127, 66, 95) 19070, 10947,  1255,   368,    0,   57,    0,   36,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kalaallisut/Latin/Greenland
    { LOCALE_TAGS(128, 66,124) 19112, 10947,  1255,   369,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kalenjin/Latin/Kenya
    { LOCALE_TAGS(129, 66,124) 19153, 10947,  1255,   370,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kamba/Latin/Kenya
    { LOCALE_TAGS(130, 56,110) 19194, 10947,  1255,   371,    0,    6,    0,    0, 2544, 2550, 2565,  6,  6,  5,  7,  6, 15, 11, }, // Kannada/Kannada/India
    { LOCALE_TAGS(132,  4,110) 19611, 11103,  1256,   374,    0,    6,    0,    0, 2576, 2583, 2597,  6,  6,  5,  7,  7, 14, 15, }, // Kashmiri/Arabic/India
    { LOCALE_TAGS(132, 29,110) 20025, 11257,  1256,   377,    0,    6,  127,    0, 2612, 2617, 2628,  6,  6, 11,  7,  5, 11, 13, }, // Kashmiri/Devanagari/India
    { LOCALE_TAGS(133, 27,123) 20065, 11266,  1256,   378,    0,    6,    0,    0, 2641, 2650, 2670,  6,  6,  5,  7,  9, 20, 15, }, // Kazakh/Cyrillic/Kazakhstan
    { LOCALE_TAGS(133,  4, 50) 20482, 11410,  1257,   381,    0,    6,    0,    0, 2685, 2694, 2714,  6,  6,  5,  7,  9, 20, 15, }, // Kazakh/Arabic/China
    { LOCALE_TAGS(134, 66, 40) 20898, 11558,  1265,   384,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kenyang/Latin/Cameroon
    { LOCALE_TAGS(135, 60, 39) 20939, 11558,  1265,   385,    0,    6,  138,    0, 2729, 2740, 2760,  6,  6, 11,  7, 11, 20, 20, }, // Khmer/Khmer/Cambodia
    { LOCALE_TAGS(136, 66, 99) 21356, 11702,  1265,   388,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kiche/Latin/Guatemala
    { LOCALE_TAGS(137, 66,124) 21397, 11702,  1265,   389,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kikuyu/Latin/Kenya
    { LOCALE_TAGS(138, 66,194) 21438, 11702,  1265,   390,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kinyarwanda/Latin/Rwanda
    { LOCALE_TAGS(141, 29,110) 21479, 11703,  1265,   391,    0,    6,    0,    0, 2780, 2786, 2801,  6,  6,  5,  7,  6, 15, 13, }, // Konkani/Devanagari/India
    { LOCALE_TAGS(141, 66,110) 21895, 11847,  1266,   394,    0,    6,    0,    0, 2814, 2821, 2836,  6,  6,  5,  7,  7, 15, 26, }, // Konkani/Latin/India
    { LOCALE_TAGS(142, 63,218) 21936, 11848,  1266,   395,    0,    6,    0,   43, 2862, 2867, 2873,  6,  6,  5,  6,  5,  6,  9, }, // Korean/Korean/South Korea
    { LOCALE_TAGS(142, 63, 50) 22353, 12005,  1266,   398,    0,    6,    0,   43, 2862, 2867, 2873,  6,  6,  5,  6,  5,  6,  9, }, // Korean/Korean/China
    { LOCALE_TAGS(142, 63,174) 22353, 12005,  1266,   398,    0,    6,    0,   43, 2862, 2867, 2873,  6,  6,  5,  6,  5,  6,  9, }, // Korean/Korean/North Korea
    { LOCALE_TAGS(144, 66,145) 22353, 12006,  1266,   398,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Koyraboro Senni/Latin/Mali
    { LOCALE_TAGS(145, 66,145) 22394, 12006,  1266,   399,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Koyra Chiini/Latin/Mali
    { LOCALE_TAGS(146, 66,134) 22435, 12006,  1266,   400,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kpelle/Latin/Liberia
    { LOCALE_TAGS(146, 66,102) 22476, 12006,  1266,   401,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kpelle/Latin/Guinea
    { LOCALE_TAGS(148, 66,239) 22476, 12006,  1266,   401,    0,    6,    0,    0, 2882, 2894, 2917,  6,  6,  5,  7, 12, 23, 22, }, // Kurdish/Latin/Turkey
    { LOCALE_TAGS(149, 66, 40) 22642, 12150,  1278,   404,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kwasio/Latin/Cameroon
    { LOCALE_TAGS(150, 27,128) 22683, 12150,  1278,   405,    0,    6,    0,    0, 2939,   83,   90,  6,  6,  5,  7, 11,  7,  7, }, // Kyrgyz/Cyrillic/Kyrgyzstan
    { LOCALE_TAGS(151, 66,248) 23100, 12294,  1278,   408,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lakota/Latin/United States
    { LOCALE_TAGS(152, 66,230) 23141, 12294,  1278,   409,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Langi/Latin/Tanzania
    { LOCALE_TAGS(153, 65,129) 23182, 12294,  1278,   410,    0,    6,    0,    0, 2950, 2957, 2972,  6,  6,  5,  7,  7, 15, 14, }, // Lao/Lao/Laos
    { LOCALE_TAGS(154, 66,253) 23599, 12448,  1278,   413,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Latin/Latin/Vatican City
    { LOCALE_TAGS(155, 66,131) 23640, 12448,  1278,   414,    0,    6,    0,   36, 2986, 3001, 3020,  6,  6,  5,  7, 15, 19, 17, }, // Latvian/Latin/Latvia
    { LOCALE_TAGS(158, 66, 57) 24007, 12595,  1281,   417,    0,    6,    0,    0, 3037,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Congo - Kinshasa
    { LOCALE_TAGS(158, 66,  7) 24048, 12600,  1281,   418,    0,    6,    0,    0, 3037,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Angola
    { LOCALE_TAGS(158, 66, 46) 24048, 12600,  1281,   418,    0,    6,    0,    0, 3037,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Central African Republic
    { LOCALE_TAGS(158, 66, 56) 24048, 12600,  1281,   418,    0,    6,    0,    0, 3037,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Congo - Brazzaville
    { LOCALE_TAGS(160, 66,137) 24048, 12600,  1281,   418,    0,   57,    0,    0, 3049, 3059, 3076,  6,  6,  5,  7, 10, 17, 18, }, // Lithuanian/Latin/Lithuania
    { LOCALE_TAGS(161, 66,258) 24435, 12757,  1281,   421,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lojban/Latin/world
    { LOCALE_TAGS(162, 66, 91) 24476, 12759,  1281,   422,    0,    6,    0,    0, 3094, 3109, 3122,  6,  6,  5,  7, 15, 13, 15, }, // Lower Sorbian/Latin/Germany
    { LOCALE_TAGS(163, 66, 91) 24596, 12903,  1284,   425,   45,   51,   56,    0, 3137, 3144, 3159,  6,  6,  5,  7,  7, 15, 13, }, // Low German/Latin/Germany
    { LOCALE_TAGS(163, 66,165) 24651, 12931,  1284,   426,   45,   51,   56,    0, 3137, 3144, 3159,  6,  6,  5,  7,  7, 15, 13, }, // Low German/Latin/Netherlands
    { LOCALE_TAGS(164, 66, 57) 24651, 12931,  1284,   426,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Luba-Katanga/Latin/Congo - Kinshasa
    { LOCALE_TAGS(165, 66,225) 24692, 12931,  1284,   427,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lule Sami/Latin/Sweden
    { LOCALE_TAGS(165, 66,175) 24733, 12931,  1284,   428,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lule Sami/Latin/Norway
    { LOCALE_TAGS(166, 66,124) 24733, 12931,  1284,   428,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Luo/Latin/Kenya
    { LOCALE_TAGS(167, 66,138) 24774, 12931,  1284,   429,    0,    6,    0,    0, 3172, 3179, 3192,  6,  6,  5,  7,  7, 13, 13, }, // Luxembourgish/Latin/Luxembourg
    { LOCALE_TAGS(168, 66,124) 24883, 13076,  1284,   432,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Luyia/Latin/Kenya
    { LOCALE_TAGS(169, 27,140) 24924, 13076,  1284,   433,    0,    6,    0,    0, 3205,   83,   90,  6,  6,  5,  7, 11,  7,  7, }, // Macedonian/Cyrillic/Macedonia
    { LOCALE_TAGS(170, 66,230) 25341, 13223,  1287,   436,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Machame/Latin/Tanzania
    { LOCALE_TAGS(171, 29,110) 25382, 13223,  1287,   437,    0,    6,  149,    0, 2149,  565,  576,  6,  6,  8,  7,  6, 11, 13, }, // Maithili/Devanagari/India
    { LOCALE_TAGS(172, 66,160) 25760, 13334,  1287,   438,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Makhuwa-Meetto/Latin/Mozambique
    { LOCALE_TAGS(173, 66,230) 25801, 13334,  1287,   439,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Makonde/Latin/Tanzania
    { LOCALE_TAGS(174, 66,141) 25842, 13334,  1287,   440,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Malagasy/Latin/Madagascar
    { LOCALE_TAGS(175, 74,110) 25883, 13334,  1287,   441,    0,    6,  157,    0, 3216, 3223, 3243,  6,  6,  9,  7,  7, 20, 16, }, // Malayalam/Malayalam/India
    { LOCALE_TAGS(176, 66,143) 26300, 13490,  1288,   444,    0,    6,    0,    0, 2241, 3259, 3274,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Malaysia
    { LOCALE_TAGS(176,  4, 35) 26351, 13644,  1291,   447,    0,    6,    0,    0, 3288,   83,   90,  6,  6,  5,  7,  7,  7,  7, }, // Malay/Arabic/Brunei
    { LOCALE_TAGS(176,  4,143) 26358, 13651,  1291,   448,    0,    6,    0,    0, 3288,   83,   90,  6,  6,  5,  7,  7,  7,  7, }, // Malay/Arabic/Malaysia
    { LOCALE_TAGS(176, 66, 35) 26365, 13658,  1291,   449,    0,    6,    0,    0, 2241, 3259, 3274,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Brunei
    { LOCALE_TAGS(176, 66,111) 26365, 13658,  1291,   449,   45,   51,    0,    0, 2241, 3259, 3274,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Indonesia
    { LOCALE_TAGS(176, 66,210) 26365, 13658,  1291,   449,    0,    6,    0,    0, 2241, 3259, 3274,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Singapore
    { LOCALE_TAGS(177, 66,146) 26365, 13658,  1291,   449,    0,    6,    0,    0, 3295, 3305,   90,  6,  6,  5,  7, 10, 15,  7, }, // Maltese/Latin/Malta
    { LOCALE_TAGS(179,  9,110) 26436, 13660,  1295,   450,    0,    6,  166,    0, 3320, 3327, 3350,  6,  6, 11,  7,  7, 23, 19, }, // Manipuri/Bangla/India
    { LOCALE_TAGS(179, 78,110) 26477, 13669,  1295,   451,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Manipuri/Meitei Mayek/India
    { LOCALE_TAGS(180, 66,115) 26477, 13669,  1295,   452,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Manx/Latin/Isle of Man
    { LOCALE_TAGS(181, 66,167) 26518, 13669,  1299,   453,    0,    6,    0,    0, 3369, 3374, 3389,  6,  6,  5,  7,  5, 15, 12, }, // Maori/Latin/New Zealand
    { LOCALE_TAGS(182, 66, 49) 26660, 13813,  1299,   456,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mapuche/Latin/Chile
    { LOCALE_TAGS(183, 29,110) 26701, 13813,  1299,   457,    0,    6,  177,    0, 2780, 3401, 3414,  6,  6,  7,  7,  6, 13, 18, }, // Marathi/Devanagari/India
    { LOCALE_TAGS(185, 66,124) 27118, 13969,  1300,   460,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Masai/Latin/Kenya
    { LOCALE_TAGS(185, 66,230) 27159, 13969,  1300,   461,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Masai/Latin/Tanzania
    { LOCALE_TAGS(186,  4,112) 27159, 13969,  1300,   461,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mazanderani/Arabic/Iran
    { LOCALE_TAGS(188, 66,124) 27200, 13969,  1300,   462,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Meru/Latin/Kenya
    { LOCALE_TAGS(189, 66, 40) 27241, 13969,  1300,   463,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Meta/Latin/Cameroon
    { LOCALE_TAGS(190, 66, 41) 27282, 13969,  1300,   464,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mohawk/Latin/Canada
    { LOCALE_TAGS(191, 27,156) 27323, 13969,  1300,   465,    0,    6,    0,    0, 3432, 3440, 3457,  6,  6,  5,  7,  8, 17, 13, }, // Mongolian/Cyrillic/Mongolia
    { LOCALE_TAGS(191, 83, 50) 27740, 14113,  1300,   468,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mongolian/Mongolian/China
    { LOCALE_TAGS(191, 83,156) 27781, 14113,  1300,   469,    0,    6,   21,    0, 3470, 3476, 3494,  6,  6,  6,  7,  6, 18, 15, }, // Mongolian/Mongolian/Mongolia
    { LOCALE_TAGS(192, 66,150) 27783, 14122,  1300,   470,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Morisyen/Latin/Mauritius
    { LOCALE_TAGS(193, 66, 40) 27824, 14122,  1300,   471,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mundang/Latin/Cameroon
    { LOCALE_TAGS(194, 66,248) 27865, 14122,  1300,   472,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Muscogee/Latin/United States
    { LOCALE_TAGS(195, 66,162) 27906, 14122,  1300,   473,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nama/Latin/Namibia
    { LOCALE_TAGS(197, 66,248) 27947, 14122,  1300,   474,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Navajo/Latin/United States
    { LOCALE_TAGS(199, 29,164) 27988, 14122,  1300,   475,    0,    6,    0,    0, 2149, 3509, 3516,  6,  6,  5,  7,  6,  7,  7, }, // Nepali/Devanagari/Nepal
    { LOCALE_TAGS(199, 29,110) 28405, 14266,  1300,   478,    0,    6,    0,    0, 2149, 3509, 3516,  6,  6,  5,  7,  6,  7,  7, }, // Nepali/Devanagari/India
    { LOCALE_TAGS(201, 66, 40) 28405, 14267,  1301,   478,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ngiemboon/Latin/Cameroon
    { LOCALE_TAGS(202, 66, 40) 28446, 14267,  1301,   479,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ngomba/Latin/Cameroon
    { LOCALE_TAGS(203, 66,169) 28487, 14267,  1301,   480,    0,    6,    0,    0, 3523, 3530, 3543,  6,  6,  5,  7,  7, 13, 14, }, // Nigerian Pidgin/Latin/Nigeria
    { LOCALE_TAGS(204, 90,102) 28889, 14411,  1301,   483,    0,    6,  184,    0, 3557, 3565, 3565,  6,  6,  6,  7,  8, 20, 16, }, // Nko/Nko/Guinea
    { LOCALE_TAGS(205,  4,112) 29305, 14555,  1301,   486,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Northern Luri/Arabic/Iran
    { LOCALE_TAGS(205,  4,113) 29346, 14556,  1301,   487,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Northern Luri/Arabic/Iraq
    { LOCALE_TAGS(206, 66,175) 29346, 14556,  1301,   487,    0,   57,   56,   36, 3585, 3592, 3606,  6,  6,  5,  7,  7, 14, 13, }, // Northern Sami/Latin/Norway
    { LOCALE_TAGS(206, 66, 83) 29388, 14561,  1305,   488,    0,    6,   91,   36, 3585, 3619, 3631,  6,  6,  6,  7,  7, 12, 13, }, // Northern Sami/Latin/Finland
    { LOCALE_TAGS(206, 66,225) 29430, 14703,  1309,   491,    0,   57,   56,   36, 3585, 3592, 3606,  6,  6,  5,  7,  7, 14, 13, }, // Northern Sami/Latin/Sweden
    { LOCALE_TAGS(207, 66,216) 29430, 14703,  1309,   491,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Northern Sotho/Latin/South Africa
    { LOCALE_TAGS(208, 66,261) 29471, 14704,  1309,   492,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // North Ndebele/Latin/Zimbabwe
    { LOCALE_TAGS(209, 66,175) 29512, 14704,  1309,   493,    0,    6,    0,    0, 3644, 3659, 3673,  6,  6,  5,  7, 15, 14, 14, }, // Norwegian Bokmal/Latin/Norway
    { LOCALE_TAGS(209, 66,224) 29630, 14861,  1320,   497,    0,    6,    0,    0, 3644, 3659, 3673,  6,  6,  5,  7, 15, 14, 14, }, // Norwegian Bokmal/Latin/Svalbard and Jan Mayen
    { LOCALE_TAGS(210, 66,175) 29630, 14861,  1320,   497,    0,    6,    0,    0, 3644, 3659, 3687,  6,  6,  5,  7, 15, 14, 14, }, // Norwegian Nynorsk/Latin/Norway
    { LOCALE_TAGS(211, 66,219) 29749, 15018,  1331,   501,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nuer/Latin/South Sudan
    { LOCALE_TAGS(212, 66,142) 29790, 15018,  1331,   502,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nyanja/Latin/Malawi
    { LOCALE_TAGS(213, 66,243) 29831, 15018,  1331,   503,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nyankole/Latin/Uganda
    { LOCALE_TAGS(214, 66, 84) 29872, 15018,  1331,   504,    0,    6,   56,    0,  272, 3701, 3719,  6,  6,  5,  7,  9, 18, 16, }, // Occitan/Latin/France
    { LOCALE_TAGS(214, 66,220) 29942, 15161,  1331,   507,    0,    6,   56,    0,  272, 3735, 3756,  6,  6,  5,  7,  9, 21, 19, }, // Occitan/Latin/Spain
    { LOCALE_TAGS(215, 91,110) 30067, 15256,  1331,   510,    0,    6,    0,    0, 3775, 3781, 3795,  6,  6,  5,  7,  6, 14, 14, }, // Odia/Odia/India
    { LOCALE_TAGS(220, 66, 77) 30484, 15411,  1333,   513,    0,    6,    0,    0, 3809, 3820, 3844,  6,  6,  5,  7, 11, 24, 18, }, // Oromo/Latin/Ethiopia
    { LOCALE_TAGS(220, 66,124) 30525, 15558,  1340,   516,    0,    6,    0,    0, 3809, 3820, 3844,  6,  6,  5,  7, 11, 24, 18, }, // Oromo/Latin/Kenya
    { LOCALE_TAGS(221,101,248) 30525, 15558,  1340,   516,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Osage/Osage/United States
    { LOCALE_TAGS(222, 27, 90) 30566, 15558,  1340,   517,    0,    6,    0,    0, 3862,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Ossetic/Cyrillic/Georgia
    { LOCALE_TAGS(222, 27,193) 30566, 15558,  1340,   517,    0,    6,    0,    0, 3862,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Ossetic/Cyrillic/Russia
    { LOCALE_TAGS(226, 66, 62) 30610, 15564,  1340,   518,    0,    6,    0,    0, 3871,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Papiamento/Latin/Curacao
    { LOCALE_TAGS(226, 66, 13) 30659, 15571,  1340,   519,    0,    6,    0,    0, 3871,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Papiamento/Latin/Aruba
    { LOCALE_TAGS(227,  4,  1) 30659, 15571,  1340,   519,    0,    6,    0,    0, 3880, 3891, 3904,  6,  6,  5,  7, 11, 13, 14, }, // Pashto/Arabic/Afghanistan
    { LOCALE_TAGS(227,  4,178) 31076, 15715,  1340,   522,    0,    6,    0,    0, 3880, 3891, 3904,  6,  6,  5,  7, 11, 13, 14, }, // Pashto/Arabic/Pakistan
    { LOCALE_TAGS(228,  4,112) 31078, 15740,  1340,   522,   73,   87,  190,    0, 3918, 3924, 3935,  7,  7, 10,  7,  6, 11, 15, }, // Persian/Arabic/Iran
    { LOCALE_TAGS(228,  4,  1) 31495, 15893,  1340,   525,   73,   87,  190,    0, 3918, 3924, 3935,  7,  7, 10,  7,  6, 11, 15, }, // Persian/Arabic/Afghanistan
    { LOCALE_TAGS(230, 66,187) 31495, 15893,  1340,   525,    0,    6,    0,    0, 3950, 3958, 3979,  6,  6,  5,  7,  8, 21, 15, }, // Polish/Latin/Poland
    { LOCALE_TAGS(231, 66, 32) 31697, 16044,  1343,   528,    0,    6,    0,    0, 3994, 4004, 4022,  6,  6,  5,  7, 10, 18, 20, }, // Portuguese/Latin/Brazil
    { LOCALE_TAGS(231, 66,  7) 31859, 16200,  1346,   531,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Angola
    { LOCALE_TAGS(231, 66, 43) 31927, 16356,  1353,   534,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Cape Verde
    { LOCALE_TAGS(231, 66, 73) 31995, 16512,  1360,   537,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Equatorial Guinea
    { LOCALE_TAGS(231, 66,101) 32063, 16668,  1367,   540,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Guinea-Bissau
    { LOCALE_TAGS(231, 66,138) 32131, 16824,  1374,   543,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Luxembourg
    { LOCALE_TAGS(231, 66,139) 32199, 16980,  1381,   546,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Macao
    { LOCALE_TAGS(231, 66,160) 32267, 17136,  1388,   549,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Mozambique
    { LOCALE_TAGS(231, 66,188) 32335, 17292,  1395,   552,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Portugal
    { LOCALE_TAGS(231, 66,204) 32403, 17448,  1402,   555,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Sao Tome and Principe
    { LOCALE_TAGS(231, 66,226) 32471, 17604,  1409,   558,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Switzerland
    { LOCALE_TAGS(231, 66,232) 32539, 17760,  1416,   561,    0,    6,    0,    0,  271, 4042, 4042,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Timor-Leste
    { LOCALE_TAGS(232, 66,187) 32607, 17916,  1423,   564,    0,    6,    0,    0, 4059, 4068, 4084,  6,  6,  5,  7,  9, 16, 16, }, // Prussian/Latin/Poland
    { LOCALE_TAGS(233, 41,110) 32648, 17925,  1423,   565,    0,    6,    0,    0, 4100, 4107, 4120,  6,  6,  5,  7,  7, 13, 15, }, // Punjabi/Gurmukhi/India
    { LOCALE_TAGS(233,  4,178) 33065, 18078,  1424,   568,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Punjabi/Arabic/Pakistan
    { LOCALE_TAGS(234, 66,184) 33106, 18078,  1424,   569,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Quechua/Latin/Peru
    { LOCALE_TAGS(234, 66, 28) 33221, 18222,  1425,   572,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Quechua/Latin/Bolivia
    { LOCALE_TAGS(234, 66, 70) 33221, 18224,  1427,   572,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Quechua/Latin/Ecuador
    { LOCALE_TAGS(235, 66,192) 33221, 18226,  1429,   572,    0,    6,    0,    0, 4135, 4145, 4164,  6,  6,  5,  7, 10, 19, 18, }, // Romanian/Latin/Romania
    { LOCALE_TAGS(235, 66,154) 33333, 18377,  1433,   575,    0,    6,    0,    0, 4135, 4145, 4164,  6,  6,  5,  7, 10, 19, 18, }, // Romanian/Latin/Moldova
    { LOCALE_TAGS(236, 66,226) 33333, 18377,  1433,   575,    0,    6,    0,    0, 4182, 4190, 4205,  6,  6,  5,  7,  8, 15, 16, }, // Romansh/Latin/Switzerland
    { LOCALE_TAGS(237, 66,230) 33473, 18386,  1433,   577,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rombo/Latin/Tanzania
    { LOCALE_TAGS(238, 66, 38) 33514, 18386,  1433,   578,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rundi/Latin/Burundi
    { LOCALE_TAGS(239, 27,193) 33555, 18386,  1433,   579,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Russia
    { LOCALE_TAGS(239, 27, 22) 33972, 18543,  1433,   582,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Belarus
    { LOCALE_TAGS(239, 27,123) 33972, 18543,  1433,   582,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Kazakhstan
    { LOCALE_TAGS(239, 27,128) 33972, 18543,  1433,   582,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Kyrgyzstan
    { LOCALE_TAGS(239, 27,154) 33972, 18543,  1433,   582,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Moldova
    { LOCALE_TAGS(239, 27,244) 33972, 18543,  1433,   582,    0,    6,    0,    0,    0, 4221, 4242,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Ukraine
    { LOCALE_TAGS(240, 66,230) 33972, 18543,  1433,   582,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rwa/Latin/Tanzania
    { LOCALE_TAGS(241, 66, 74) 34013, 18543,  1433,   583,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Saho/Latin/Eritrea
    { LOCALE_TAGS(242, 27,193) 34054, 18543,  1433,   584,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sakha/Cyrillic/Russia
    { LOCALE_TAGS(243, 66,124) 34128, 18571,  1433,   585,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Samburu/Latin/Kenya
    { LOCALE_TAGS(245, 66, 46) 34169, 18571,  1433,   586,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sango/Latin/Central African Republic
    { LOCALE_TAGS(246, 66,230) 34210, 18571,  1433,   587,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sangu/Latin/Tanzania
    { LOCALE_TAGS(247, 29,110) 34251, 18571,  1433,   588,    0,    6,  200,    0, 4258, 4265, 4278,  6,  6, 12,  7,  7, 13, 11, }, // Sanskrit/Devanagari/India
    { LOCALE_TAGS(248, 93,110) 34292, 18580,  1433,   589,    0,    6,  212,    0, 4289, 4296, 4309,  6,  6,  8,  7,  7, 13, 15, }, // Santali/Ol Chiki/India
    { LOCALE_TAGS(248, 29,110) 34412, 18601,  1433,   590,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Santali/Devanagari/India
    { LOCALE_TAGS(249, 66,117) 34437, 18601,  1433,   591,    0,    6,    0,    0, 2382, 4324, 2404,  6,  6,  5,  7,  6, 17, 14, }, // Sardinian/Latin/Italy
    { LOCALE_TAGS(251, 66,160) 34547, 18758,  1440,   595,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sena/Latin/Mozambique
    { LOCALE_TAGS(252, 27,207) 34588, 18758,  1440,   596,    0,    6,    0,    0,    0, 4341, 4361,  6,  6,  5,  7,  2, 20, 15, }, // Serbian/Cyrillic/Serbia
    { LOCALE_TAGS(252, 27, 29) 35005, 18914,  1444,   599,    0,    6,    0,    0,    0, 4376, 4398,  6,  6,  5,  7,  2, 22, 17, }, // Serbian/Cyrillic/Bosnia and Herzegovina
    { LOCALE_TAGS(252, 27,126) 35028, 19056,  1448,   602,    0,    6,    0,    0,    0, 4341, 4361,  6,  6,  5,  7,  2, 20, 15, }, // Serbian/Cyrillic/Kosovo
    { LOCALE_TAGS(252, 27,157) 35028, 19056,  1448,   602,    0,    6,    0,    0,    0, 4341, 4361,  6,  6,  5,  7,  2, 20, 15, }, // Serbian/Cyrillic/Montenegro
    { LOCALE_TAGS(252, 66, 29) 35028, 19056,  1448,   602,    0,    6,    0,    0,    0,  624, 4415,  6,  6,  5,  7,  2, 22, 19, }, // Serbian/Latin/Bosnia and Herzegovina
    { LOCALE_TAGS(252, 66,126) 35445, 19212,  1452,   605,    0,    6,    0,    0,    0, 4434, 4454,  6,  6,  5,  7,  2, 20, 16, }, // Serbian/Latin/Kosovo
    { LOCALE_TAGS(252, 66,157) 35862, 19368,  1456,   608,    0,    6,    0,    0,    0, 4434, 4454,  6,  6,  5,  7,  2, 20, 16, }, // Serbian/Latin/Montenegro
    { LOCALE_TAGS(252, 66,207) 36279, 19524,  1460,   611,    0,    6,    0,    0,    0, 4434, 4454,  6,  6,  5,  7,  2, 20, 16, }, // Serbian/Latin/Serbia
    { LOCALE_TAGS(253, 66,230) 36696, 19680,  1464,   614,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Shambala/Latin/Tanzania
    { LOCALE_TAGS(254, 66,261) 36737, 19680,  1464,   615,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Shona/Latin/Zimbabwe
    { LOCALE_TAGS(255,141, 50) 36778, 19680,  1464,   616,    0,    6,  220,   12, 4470, 4474, 4480,  6,  6,  8,  6,  4,  6,  6, }, // Sichuan Yi/Yi/China
    { LOCALE_TAGS(256, 66,117) 36819, 19681,  1464,   617,    0,    6,    0,    0,    0, 4486, 4501,  6,  6,  5,  7,  2, 15, 15, }, // Sicilian/Latin/Italy
    { LOCALE_TAGS(257, 66, 77) 36942, 19826,  1472,   620,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sidamo/Latin/Ethiopia
    { LOCALE_TAGS(258, 66,187) 36983, 19826,  1472,   621,    0,    6,    0,    0, 3950, 4516, 4538,  6,  6,  5,  7,  8, 22, 16, }, // Silesian/Latin/Poland
    { LOCALE_TAGS(259,  4,178) 37193, 19948,  1475,   624,    0,    6,    0,    0, 4554, 4560, 4573,  6,  6,  5,  7,  6, 13, 15, }, // Sindhi/Arabic/Pakistan
    { LOCALE_TAGS(259, 29,110) 37610, 20092,  1475,   627,    0,    6,  149,    0, 4588, 4597, 4613,  6,  6,  8,  7,  9, 16, 16, }, // Sindhi/Devanagari/India
    { LOCALE_TAGS(260,119,221) 37651, 20101,  1475,   628,   45,   51,  228,    0, 4629, 4637, 4651,  6,  6, 10,  7,  8, 14, 17, }, // Sinhala/Sinhala/Sri Lanka
    { LOCALE_TAGS(261, 66, 83) 38068, 20246,  1475,   631,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Skolt Sami/Latin/Finland
    { LOCALE_TAGS(262, 66,212) 38109, 20246,  1475,   632,    0,    6,    0,    0, 1256,   83,   90,  6,  6,  5,  7, 15,  7,  7, }, // Slovak/Latin/Slovakia
    { LOCALE_TAGS(263, 66,213) 38281, 20403,  1476,   636,   45,   51,   21,    0, 4668, 4674, 4691,  6,  6,  6,  7,  6, 17, 14, }, // Slovenian/Latin/Slovenia
    { LOCALE_TAGS(264, 66,243) 38431, 20550,  1476,   639,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Soga/Latin/Uganda
    { LOCALE_TAGS(265, 66,215) 38472, 20550,  1476,   640,    0,    6,    0,    0, 4705, 4715, 4739,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Somalia
    { LOCALE_TAGS(265, 66, 67) 38832, 20702,  1477,   643,    0,    6,    0,    0, 4705, 4715, 4739,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Djibouti
    { LOCALE_TAGS(265, 66, 77) 38832, 20702,  1477,   643,    0,    6,    0,    0, 4705, 4715, 4739,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Ethiopia
    { LOCALE_TAGS(265, 66,124) 38832, 20702,  1477,   643,    0,    6,    0,    0, 4705, 4715, 4739,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Kenya
    { LOCALE_TAGS(266,  4,112) 38832, 20702,  1477,   643,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Kurdish/Arabic/Iran
    { LOCALE_TAGS(266,  4,113) 38873, 20702,  1477,   644,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Kurdish/Arabic/Iraq
    { LOCALE_TAGS(267, 66,225) 38873, 20702,  1477,   644,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Sami/Latin/Sweden
    { LOCALE_TAGS(267, 66,175) 38914, 20702,  1477,   645,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Sami/Latin/Norway
    { LOCALE_TAGS(268, 66,216) 38914, 20702,  1477,   645,    0,    6,    0,    0, 4762, 4769, 4787,  6,  6,  5,  7,  7, 18, 21, }, // Southern Sotho/Latin/South Africa
    { LOCALE_TAGS(268, 66,133) 38955, 20703,  1477,   646,    0,    6,    0,    0, 4762, 4769, 4787,  6,  6,  5,  7,  7, 18, 21, }, // Southern Sotho/Latin/Lesotho
    { LOCALE_TAGS(269, 66,216) 38955, 20703,  1477,   646,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // South Ndebele/Latin/South Africa
    { LOCALE_TAGS(270, 66,220) 38996, 20703,  1477,   647,    0,    6,    0,    0, 2285, 4808, 4830,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Spain
    { LOCALE_TAGS(270, 66, 11) 39195, 20858,  1481,   650,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Argentina
    { LOCALE_TAGS(270, 66, 24) 39206, 20879,  1487,   652,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Belize
    { LOCALE_TAGS(270, 66, 28) 39217, 20898,  1491,   654,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Bolivia
    { LOCALE_TAGS(270, 66, 32) 39228, 20918,  1496,   656,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Brazil
    { LOCALE_TAGS(270, 66, 42) 39239, 20937,  1500,   658,    0,    6,    0,    0, 2285, 4808, 4830,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Canary Islands
    { LOCALE_TAGS(270, 66, 47) 39239, 20937,  1500,   658,    0,    6,    0,    0, 2285, 4808, 4830,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Ceuta and Melilla
    { LOCALE_TAGS(270, 66, 49) 39239, 20937,  1500,   658,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Chile
    { LOCALE_TAGS(270, 66, 54) 39250, 20957,  1505,   660,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Colombia
    { LOCALE_TAGS(270, 66, 59) 39261, 20977,  1510,   662,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Costa Rica
    { LOCALE_TAGS(270, 66, 61) 39272, 20996,  1514,   664,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Cuba
    { LOCALE_TAGS(270, 66, 69) 39283, 21015,  1518,   666,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Dominican Republic
    { LOCALE_TAGS(270, 66, 70) 39294, 21034,  1522,   668,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Ecuador
    { LOCALE_TAGS(270, 66, 72) 39305, 21054,  1527,   670,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/El Salvador
    { LOCALE_TAGS(270, 66, 73) 39316, 21073,  1531,   672,    0,    6,    0,    0, 2285, 4808, 4830,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Equatorial Guinea
    { LOCALE_TAGS(270, 66, 99) 39316, 21073,  1531,   672,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Guatemala
    { LOCALE_TAGS(270, 66,106) 39327, 21092,  1535,   674,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Honduras
    { LOCALE_TAGS(270, 66,130) 39338, 21111,  1539,   676,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Latin America
    { LOCALE_TAGS(270, 66,152) 39349, 21130,  1543,   678,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Mexico
    { LOCALE_TAGS(270, 66,168) 39365, 21145,  1547,   680,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Nicaragua
    { LOCALE_TAGS(270, 66,181) 39376, 21164,  1551,   682,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Panama
    { LOCALE_TAGS(270, 66,183) 39387, 21183,  1555,   684,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Paraguay
    { LOCALE_TAGS(270, 66,184) 39398, 21202,  1559,   686,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Peru
    { LOCALE_TAGS(270, 66,185) 39409, 21222,  1564,   688,    0,    6,    0,    0, 2285, 4808, 4830,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Philippines
    { LOCALE_TAGS(270, 66,189) 39409, 21222,  1564,   688,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Puerto Rico
    { LOCALE_TAGS(270, 66,248) 39420, 21241,  1568,   690,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/United States
    { LOCALE_TAGS(270, 66,250) 39436, 21260,  1579,   693,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Uruguay
    { LOCALE_TAGS(270, 66,254) 39447, 21280,  1584,   695,    0,    6,    0,    0, 2285, 4853, 4872,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Venezuela
    { LOCALE_TAGS(271,135,159) 39458, 21300,  1589,   697,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Standard Moroccan Tamazight/Tifinagh/Morocco
    { LOCALE_TAGS(272, 66,111) 39499, 21300,  1589,   698,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sundanese/Latin/Indonesia
    { LOCALE_TAGS(273, 66,230) 39540, 21311,  1589,   699,    0,    6,   21,    0, 4892, 4901, 4921,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Tanzania
    { LOCALE_TAGS(273, 66, 57) 39586, 21458,  1589,   702,    0,    6,   21,    0, 4892, 4901, 4921,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Congo - Kinshasa
    { LOCALE_TAGS(273, 66,124) 39586, 21458,  1589,   702,    0,    6,   21,    0, 4892, 4901, 4921,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Kenya
    { LOCALE_TAGS(273, 66,243) 39606, 21523,  1589,   703,    0,    6,   21,    0, 4892, 4901, 4921,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Uganda
    { LOCALE_TAGS(274, 66,216) 39606, 21523,  1589,   703,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swati/Latin/South Africa
    { LOCALE_TAGS(274, 66, 76) 39647, 21523,  1589,   704,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swati/Latin/Eswatini
    { LOCALE_TAGS(275, 66,225) 39647, 21523,  1589,   704,    0,   57,    0,    0, 4940, 4945, 4959,  6,  6,  5,  7,  5, 14, 14, }, // Swedish/Latin/Sweden
    { LOCALE_TAGS(275, 66,  2) 39773, 21680,  1593,   708,    0,   57,    0,    0, 4940, 4945, 4959,  6,  6,  5,  7,  5, 14, 14, }, // Swedish/Latin/Aland Islands
    { LOCALE_TAGS(275, 66, 83) 39773, 21680,  1593,   708,    0,   57,    0,    0, 4940, 4945, 4959,  6,  6,  5,  7,  5, 14, 14, }, // Swedish/Latin/Finland
    { LOCALE_TAGS(276, 66,226) 39773, 21680,  1593,   708,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swiss German/Latin/Switzerland
    { LOCALE_TAGS(276, 66, 84) 39881, 21694,  1596,   709,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swiss German/Latin/France
    { LOCALE_TAGS(276, 66,136) 39881, 21694,  1596,   709,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swiss German/Latin/Liechtenstein
    { LOCALE_TAGS(277,123,113) 39881, 21694,  1596,   709,    0,    6,    0,    0, 4973, 4980, 4997,  6,  6,  5,  7,  7, 17, 17, }, // Syriac/Syriac/Iraq
    { LOCALE_TAGS(277,123,227) 40297, 21850,  1596,   712,    0,    6,    0,    0, 4973, 4980, 4997,  6,  6,  5,  7,  7, 17, 17, }, // Syriac/Syriac/Syria
    { LOCALE_TAGS(278,135,159) 40297, 21850,  1596,   712,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tachelhit/Tifinagh/Morocco
    { LOCALE_TAGS(278, 66,159) 40338, 21850,  1596,   713,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tachelhit/Latin/Morocco
    { LOCALE_TAGS(280,127,255) 40338, 21850,  1596,   713,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tai Dam/Tai Viet/Vietnam
    { LOCALE_TAGS(281, 66,124) 40379, 21850,  1596,   714,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Taita/Latin/Kenya
    { LOCALE_TAGS(282, 27,229) 40420, 21850,  1596,   715,    0,    6,    0,    0, 5014, 5022, 5041,  6,  6,  5,  7,  8, 19, 16, }, // Tajik/Cyrillic/Tajikistan
    { LOCALE_TAGS(283,129,110) 40836, 21994,  1596,   718,    0,    6,    0,    0, 5057, 5065, 5081,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/India
    { LOCALE_TAGS(283,129,143) 41253, 22150,  1597,   721,    0,    6,    0,    0, 5057, 5065, 5081,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/Malaysia
    { LOCALE_TAGS(283,129,210) 41253, 22153,  1600,   721,    0,    6,    0,    0, 5057, 5065, 5081,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/Singapore
    { LOCALE_TAGS(283,129,221) 41253, 22156,  1603,   721,    0,    6,    0,    0, 5057, 5065, 5081,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/Sri Lanka
    { LOCALE_TAGS(284, 66,228) 41253, 22156,  1603,   721,    0,    6,  238,    0, 5096,   83,   90,  6,  6,  5,  7,  8,  7,  7, }, // Taroko/Latin/Taiwan
    { LOCALE_TAGS(285, 66,170) 41305, 22165,  1612,   722,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tasawaq/Latin/Niger
    { LOCALE_TAGS(286, 27,193) 41346, 22165,  1612,   723,    0,    6,    0,    0, 5104, 5113, 5129,  6,  6,  5,  7,  9, 16, 15, }, // Tatar/Cyrillic/Russia
    { LOCALE_TAGS(287,131,110) 41761, 22313,  1612,   726,    0,    6,    0,    0, 5144, 5151, 5168,  6,  6,  5,  7,  7, 17, 21, }, // Telugu/Telugu/India
    { LOCALE_TAGS(288, 66,243) 42178, 22469,  1613,   729,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Teso/Latin/Uganda
    { LOCALE_TAGS(288, 66,124) 42219, 22469,  1613,   730,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Teso/Latin/Kenya
    { LOCALE_TAGS(289,133,231) 42219, 22469,  1613,   730,    0,    6,    0,    0, 5189, 5195, 5208,  6,  6,  5,  7,  6, 13, 12, }, // Thai/Thai/Thailand
    { LOCALE_TAGS(290,134, 50) 42636, 22626,  1613,   733,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tibetan/Tibetan/China
    { LOCALE_TAGS(290,134,110) 42677, 22626,  1613,   734,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tibetan/Tibetan/India
    { LOCALE_TAGS(291, 33, 74) 42677, 22626,  1613,   734,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tigre/Ethiopic/Eritrea
    { LOCALE_TAGS(292, 33, 77) 42718, 22626,  1613,   735,    0,    6,    0,    0, 5220, 5225, 5241,  6,  6,  5,  7,  5, 16, 13, }, // Tigrinya/Ethiopic/Ethiopia
    { LOCALE_TAGS(292, 33, 74) 43135, 22771,  1613,   738,    0,    6,    0,    0, 5220, 5225, 5241,  6,  6,  5,  7,  5, 16, 13, }, // Tigrinya/Ethiopic/Eritrea
    { LOCALE_TAGS(294, 66,182) 43135, 22771,  1613,   738,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tok Pisin/Latin/Papua New Guinea
    { LOCALE_TAGS(295, 66,235) 43176, 22771,  1613,   739,    0,    6,    0,    0, 5254, 5262, 5277,  6,  6,  5,  7,  8, 15, 14, }, // Tongan/Latin/Tonga
    { LOCALE_TAGS(296, 66,216) 43258, 22928,  1613,   743,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tsonga/Latin/South Africa
    { LOCALE_TAGS(297, 66,216) 43299, 22928,  1613,   744,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tswana/Latin/South Africa
    { LOCALE_TAGS(297, 66, 30) 43340, 22929,  1613,   745,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tswana/Latin/Botswana
    { LOCALE_TAGS(298, 66,239) 43340, 22929,  1613,   745,    0,    6,    0,    0, 5291, 5299, 5316,  6,  6,  5,  7,  8, 17, 12, }, // Turkish/Latin/Turkey
    { LOCALE_TAGS(298, 66, 63) 43487, 23086,  1613,   748,    0,    6,    0,    0, 5291, 5299, 5316,  6,  6,  5,  7,  8, 17, 12, }, // Turkish/Latin/Cyprus
    { LOCALE_TAGS(299, 66,240) 43487, 23086,  1613,   748,    0,    6,    0,    0, 5328, 5336, 5353,  6,  6,  5,  7,  8, 17, 16, }, // Turkmen/Latin/Turkmenistan
    { LOCALE_TAGS(301, 66,169) 43800, 23233,  1614,   751,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tyap/Latin/Nigeria
    { LOCALE_TAGS(303, 27,244) 43841, 23233,  1614,   752,    0,    6,    0,    0,  538, 5369, 5389,  6,  6,  5,  7,  7, 20, 15, }, // Ukrainian/Cyrillic/Ukraine
    { LOCALE_TAGS(304, 66, 91) 44258, 23384,  1614,   755,    0,    6,    0,    0, 5404, 5419, 5432,  6,  6,  5,  7, 15, 13, 12, }, // Upper Sorbian/Latin/Germany
    { LOCALE_TAGS(305,  4,178) 44377, 23528,  1617,   758,    0,    6,   21,    0, 4554, 5444, 5457,  6,  6,  6,  7,  6, 13, 15, }, // Urdu/Arabic/Pakistan
    { LOCALE_TAGS(305,  4,110) 44794, 23675,  1617,   761,    0,    6,    0,    0, 4554, 5444, 5472,  6,  6,  5,  7,  6, 13, 12, }, // Urdu/Arabic/India
    { LOCALE_TAGS(306,  4, 50) 44802, 23718,  1617,   761,    0,    6,    0,    0, 5484, 5492, 5510,  6,  6,  5,  7,  8, 18, 15, }, // Uyghur/Arabic/China
    { LOCALE_TAGS(307, 66,251) 44843, 23870,  1617,   764,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Uzbek/Latin/Uzbekistan
    { LOCALE_TAGS(307,  4,  1) 45132, 24014,  1617,   767,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Uzbek/Arabic/Afghanistan
    { LOCALE_TAGS(307, 27,251) 45172, 24015,  1617,   768,    0,    6,    0,    0, 5525, 5533, 5550,  6,  6,  5,  7,  8, 17, 17, }, // Uzbek/Cyrillic/Uzbekistan
    { LOCALE_TAGS(308,139,134) 45211, 24153,  1617,   771,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Vai/Vai/Liberia
    { LOCALE_TAGS(308, 66,134) 45252, 24153,  1617,   772,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Vai/Latin/Liberia
    { LOCALE_TAGS(309, 66,216) 45252, 24153,  1617,   772,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Venda/Latin/South Africa
    { LOCALE_TAGS(310, 66,255) 45293, 24153,  1617,   773,    0,    6,    0,    0, 5567, 5573, 5585,  6,  6,  5,  7,  6, 12, 13, }, // Vietnamese/Latin/Vietnam
    { LOCALE_TAGS(311, 66,258) 45347, 24310,  1624,   777,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Volapuk/Latin/world
    { LOCALE_TAGS(312, 66,230) 45388, 24310,  1624,   778,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Vunjo/Latin/Tanzania
    { LOCALE_TAGS(313, 66, 23) 45429, 24310,  1624,   779,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Walloon/Latin/Belgium
    { LOCALE_TAGS(314, 66,226) 45470, 24310,  1624,   780,    0,    6,    0,    0, 5598,   83,   90,  6,  6,  5,  7,  6,  7,  7, }, // Walser/Latin/Switzerland
    { LOCALE_TAGS(315, 66, 15) 45550, 24314,  1627,   781,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Warlpiri/Latin/Australia
    { LOCALE_TAGS(316, 66,246) 45591, 24314,  1627,   782,    0,    6,    0,    0, 5604, 5612, 5628,  6,  6,  5,  7,  8, 16, 12, }, // Welsh/Latin/United Kingdom
    { LOCALE_TAGS(317,  4,178) 45665, 24459,  1631,   785,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Pakistan
    { LOCALE_TAGS(317,  4,  1) 45745, 24464,  1631,   786,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Afghanistan
    { LOCALE_TAGS(317,  4,112) 45745, 24464,  1631,   786,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Iran
    { LOCALE_TAGS(317,  4,176) 45745, 24464,  1631,   786,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Oman
    { LOCALE_TAGS(317,  4,245) 45745, 24464,  1631,   786,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/United Arab Emirates
    { LOCALE_TAGS(318, 66,165) 45745, 24464,  1631,   786,    0,    6,    0,    0, 5640, 5647, 5663,  6,  6,  5,  7,  7, 16, 12, }, // Western Frisian/Latin/Netherlands
    { LOCALE_TAGS(319, 33, 77) 45864, 24620,  1634,   789,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Wolaytta/Ethiopic/Ethiopia
    { LOCALE_TAGS(320, 66,206) 45905, 24620,  1634,   790,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Wolof/Latin/Senegal
    { LOCALE_TAGS(321, 66,216) 45946, 24764,  1634,   793,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // Xhosa/Latin/South Africa
    { LOCALE_TAGS(322, 66, 40) 45988, 24908,  1634,   796,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Yangben/Latin/Cameroon
    { LOCALE_TAGS(323, 47,244) 46029, 24908,  1634,   797,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Yiddish/Hebrew/Ukraine
    { LOCALE_TAGS(324, 66,169) 46073, 24908,  1634,   798,    0,    6,  243,    0, 5675, 5682, 5696,  6,  6,  5,  7,  7, 14, 16, }, // Yoruba/Latin/Nigeria
    { LOCALE_TAGS(324, 66, 25) 46192, 25052,  1634,   801,    0,    6,  243,    0, 5675, 5682, 5712,  6,  6,  5,  7,  7, 14, 16, }, // Yoruba/Latin/Benin
    { LOCALE_TAGS(325, 66,170) 46213, 25140,  1634,   804,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Zarma/Latin/Niger
    { LOCALE_TAGS(326, 66, 50) 46254, 25140,  1634,   805,    0,    6,    0,    0, 5728, 5738, 5758,  6,  6,  5,  7, 10, 20, 17, }, // Zhuang/Latin/China
    { LOCALE_TAGS(327, 66,216) 46295, 25141,  1634,   806,    0,    6,    0,    0, 5775, 5792, 5814,  6,  6,  5,  7, 17, 22, 21, }, // Zulu/Latin/South Africa
    { LOCALE_TAGS(328, 66, 32) 46710, 25288,  1634,   809,    0,    6,    0,    0, 5835, 5845, 5854,  6,  6,  5,  7, 10,  9, 18, }, // Kaingang/Latin/Brazil
    { LOCALE_TAGS(329, 66, 32) 47121, 25442,  1637,   812,    0,    6,    0,    0, 5872, 5883, 5902,  6,  6,  5,  7, 11, 19, 23, }, // Nheengatu/Latin/Brazil
    { LOCALE_TAGS(329, 66, 54) 47514, 25596,  1640,   815,    0,    6,    0,    0, 5872, 5883, 5902,  6,  6,  5,  7, 11, 19, 23, }, // Nheengatu/Latin/Colombia
    { LOCALE_TAGS(329, 66,254) 47515, 25598,  1640,   815,    0,    6,    0,    0, 5872, 5883, 5902,  6,  6,  5,  7, 11, 19, 23, }, // Nheengatu/Latin/Venezuela
    { LOCALE_TAGS(330, 29,110) 47516, 25600,  1640,   815,    0,    6,    0,    0, 2149, 5925,   90,  6,  6,  5,  7,  6, 22,  7, }, // Haryanvi/Devanagari/India
    { LOCALE_TAGS(331, 66, 91) 47557, 25601,  1640,   816,    0,    6,    0,    0, 5947, 5954, 5970,  6,  6,  5,  7,  7, 16, 12, }, // Northern Frisian/Latin/Germany
    { LOCALE_TAGS(332, 29,110) 47680, 25745,  1640,   819,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rajasthani/Devanagari/India
    { LOCALE_TAGS(333, 27,193) 47721, 25746,  1640,   820,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Moksha/Cyrillic/Russia
    { LOCALE_TAGS(334, 66,258) 47762, 25746,  1640,   821,    0,    6,  248,    0, 5982, 5982, 5993,  6,  6, 11,  7, 11, 11, 21, }, // Toki Pona/Latin/world
    { LOCALE_TAGS(335, 66,214) 47803, 25747,  1640,   822,    0,    6,   21,    0, 6014, 6021, 6036,  6,  6,  6,  7,  7, 15, 14, }, // Pijin/Latin/Solomon Islands
    { LOCALE_TAGS(336, 66,169) 47844, 25748,  1640,   823,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Obolo/Latin/Nigeria
    { LOCALE_TAGS(337,  4,178) 47885, 25748,  1640,   824,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Baluchi/Arabic/Pakistan
    { LOCALE_TAGS(337, 66,178) 47926, 25785,  1640,   825,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Baluchi/Latin/Pakistan
    { LOCALE_TAGS(338, 66,117) 48203, 25929,  1640,   828,    0,   57,   56,    0, 6050, 6056, 6071,  6,  6,  5,  7,  6, 15, 13, }, // Ligurian/Latin/Italy
    { LOCALE_TAGS(339,142,161) 48304, 26072,  1640,   831,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rohingya/Hanifi/Myanmar
    { LOCALE_TAGS(339,142, 20) 48345, 26072,  1640,   832,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rohingya/Hanifi/Bangladesh
    { LOCALE_TAGS(340,  4,178) 48345, 26072,  1640,   832,    0,    6,    0,    0, 6084, 6090, 6103,  6,  6,  5,  7,  6, 13,  7, }, // Torwali/Arabic/Pakistan
    { LOCALE_TAGS(341, 66, 25) 48761, 26215,  1640,   835,    0,    6,  259,    0, 6110, 6124, 6145,  6,  6,  5,  7, 14, 21, 23, }, // Anii/Latin/Benin
    { LOCALE_TAGS(342, 29,110) 49165, 26363,  1641,   838,    0,    6,  264,    0, 6168, 6174, 6185,  6,  6, 15,  7,  6, 11, 26, }, // Kangri/Devanagari/India
    { LOCALE_TAGS(343, 66,117) 49581, 26506,  1641,   841,    0,    6,   56,    0, 2382, 6211, 6225,  6,  6,  5,  7,  6, 14, 13, }, // Venetian/Latin/Italy
    { LOCALE_TAGS(344, 66,110) 49812, 26650,  1641,   844,    0,    6,    0,    0, 6238, 6245, 6260,  6,  6,  5,  7,  7, 15, 14, }, // Kuvi/Latin/India
    { LOCALE_TAGS(344, 29,110) 50228, 26793,  1641,   847,    0,    6,    0,    0, 6274, 6281, 6295,  6,  6,  5,  7,  7, 14, 15, }, // Kuvi/Devanagari/India
    { LOCALE_TAGS(344, 91,110) 50643, 26936,  1641,   848,    0,    6,    0,    0, 6310, 6317, 6332,  6,  6,  5,  7,  7, 15, 14, }, // Kuvi/Odia/India
    { LOCALE_TAGS(344,131,110) 51058, 27079,  1641,   851,    0,    6,    0,    0, 5144, 5151, 5168,  6,  6,  5,  7,  7, 17, 21, }, // Kuvi/Telugu/India
    { LOCALE_TAGS(345, 27,251) 51473, 27222,  1641,   854,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kara-Kalpak/Cyrillic/Uzbekistan
    { LOCALE_TAGS(345, 66,251) 51514, 27222,  1641,   855,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kara-Kalpak/Latin/Uzbekistan
    { LOCALE_TAGS(346, 18, 41) 51514, 27222,  1641,   855,    0,    6,    0,    0, 6346, 6355, 6372,  6,  6,  5,  7,  9, 17, 14, }, // Swampy Cree/Canadian Aboriginal/Canada
    { LOCALE_TAGS(  0,  0,  0) 51560, 27223,  1641,   856,   94,   94,  279,   49, 6386, 6386, 6386,  0,  0,  0,  0,  0,  0,  0, } // Terminal row
}; // Locale/zone data

static inline constexpr char16_t hourFormatTable[] = {
0x2b, 0x48, 0x48, 0x3a, 0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x3a, 0x6d, 0x6d,
0x2b, 0x48, 0x48, 0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x6d, 0x6d, 0x2013, 0x48,
0x48, 0x3a, 0x6d, 0x6d, 0x20, 0x2d, 0x48, 0x48, 0x3a, 0x6d, 0x6d, 0x2b,
0x48, 0x3a, 0x6d, 0x6d, 0x2d, 0x48, 0x3a, 0x6d, 0x6d, 0x2b, 0x48, 0x48,
0x2e, 0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x2e, 0x6d, 0x6d, 0x2212, 0x48, 0x48,
0x3a, 0x6d, 0x6d, 0x2b, 0x48, 0x2e, 0x6d, 0x6d, 0x2d, 0x48, 0x2e, 0x6d,
0x6d, 0x200e, 0x2b, 0x48, 0x48, 0x3a, 0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x3a,
0x6d, 0x6d, 0x200e, 0x200e, 0x2212, 0x48, 0x48, 0x3a, 0x6d, 0x6d
};

static inline constexpr char16_t gmtFormatTable[] = {
0x47, 0x4d, 0x54, 0x25, 0x30, 0x1302, 0x20, 0x12a4, 0x121d, 0x20, 0x1272, 0x25,
0x30, 0x63a, 0x631, 0x64a, 0x646, 0x62a, 0x634, 0x25, 0x30, 0x47, 0x4d, 0x54,
0x20, 0x25, 0x30, 0x91c, 0x93f, 0x2e, 0x90f, 0x92e, 0x2e, 0x924, 0x93f, 0x20,
0x25, 0x30, 0x413, 0x440, 0x438, 0x43d, 0x443, 0x438, 0x447, 0x25, 0x30, 0x6af,
0x631, 0x6cc, 0x646, 0x6cc, 0x686, 0x20, 0x25, 0x30, 0x55, 0x54, 0x43, 0x25,
0x30, 0xf47, 0xf72, 0xf0b, 0xf68, 0xf7a, 0xf58, 0xf0b, 0xf4f, 0xf72, 0xf0b, 0x25,
0x30, 0xd801, 0xdc18, 0xd801, 0xdc23, 0xd801, 0xdc13, 0x20, 0x25, 0x30, 0xb7, 0xd801,
0xdc5c, 0xd801, 0xdc65, 0xd801, 0xdc51, 0x25, 0x30, 0x25, 0x30, 0x20, 0x47, 0x4d,
0x54, 0xd83a, 0xdd11, 0xd83a, 0xdd16, 0xd83a, 0xdd18, 0x25, 0x30, 0x47, 0x4d, 0x54,
0x25, 0x30, 0x200e, 0x54, 0x4d, 0x47, 0x25, 0x30, 0x4d, 0x41, 0x47, 0x25,
0x30, 0x4b, 0x4c, 0x47, 0x20, 0x25, 0x30, 0x91c, 0x940, 0x20, 0x90f, 0x92e,
0x20, 0x91f, 0x940, 0x20, 0x25, 0x30, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x1780, 0x179b, 0x20, 0x25, 0x30, 0x91c, 0x940, 0x90f, 0x92e, 0x91f, 0x940, 0x25,
0x30, 0xd1c, 0xd3f, 0xd0e, 0xd02, 0xd1f, 0xd3f, 0x20, 0x25, 0x30, 0x99c, 0x9bf,
0x20, 0x98f, 0x9ae, 0x20, 0x99f, 0x9bf, 0x20, 0x25, 0x30, 0x5b, 0x47, 0x4d,
0x54, 0x5d, 0x25, 0x30, 0x7dc, 0x7ed, 0x7d5, 0x7d6, 0x25, 0x30, 0x25, 0x30,
0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x648, 0x6cc, 0x686, 0x91c, 0x940, 0x2e, 0x90f,
0x92e, 0x2e, 0x91f, 0x940, 0x2e, 0x20, 0x25, 0x30, 0x1c61, 0x1c64, 0x1c6e, 0x1c62,
0x1c74, 0x1c64, 0x25, 0x30, 0xa2e7, 0xa0c5, 0xa395, 0xa3e6, 0xa12e, 0xa209, 0x25, 0x30,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb8, 0xdc0, 0xdda, 0x25, 0x30, 0x4a, 0x51,
0x47, 0x25, 0x30, 0x57, 0x41, 0x54, 0x25, 0x30, 0x74, 0x65, 0x6e, 0x70,
0x6f, 0x20, 0x55, 0x54, 0x43, 0x25, 0x30, 0x25, 0x30, 0x20, 0x47, 0x6b,
0x917, 0x94d, 0x930, 0x940, 0x970, 0x20, 0x92e, 0x948, 0x970, 0x20, 0x91f, 0x948,
0x970, 0x25, 0x30
};

static inline constexpr char16_t regionFormatTable[] = {
0x25, 0x30, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x25, 0x30, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x25,
0x30, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x25, 0x30, 0x20, 0x410, 0x430, 0x43c, 0x4ad, 0x430, 0x25,
0x30, 0x20, 0x410, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x410, 0x430, 0x43c, 0x4ad, 0x430, 0x25, 0x30, 0x20, 0x410, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x430, 0x43c, 0x4ad, 0x430, 0x25,
0x30, 0x20, 0x28, 0x2b, 0x30, 0x29, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x31,
0x29, 0x25, 0x30, 0x2d, 0x74, 0x79, 0x64, 0x25, 0x30, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x25, 0x30,
0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x42, 0x65,
0x72, 0x25b, 0x20, 0x25, 0x30, 0x42, 0x65, 0x72, 0x25b, 0x20, 0x73, 0x75,
0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x25, 0x30, 0x41, 0x77, 0x69, 0x61,
0x20, 0x62, 0x65, 0x72, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x3a, 0x20,
0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20,
0x1230, 0x12d3, 0x1275, 0x25, 0x30, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x25, 0x30, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x25,
0x30, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x25, 0x30, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x25, 0x30,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x30, 0x29, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x25, 0x30, 0x20, 0x28, 0x2b,
0x31, 0x29, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x25, 0x30, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x25, 0x30, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x25, 0x30, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x25, 0x30, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be,
0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x25, 0x30, 0x20, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x25,
0x30, 0x20, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x25, 0x30, 0x20, 0x28, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x29, 0x427, 0x430, 0x441, 0x3a, 0x20, 0x25, 0x30, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x3a, 0x20, 0x25, 0x30, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x25, 0x30, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x25, 0x30, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x92f, 0x25, 0x30, 0x20, 0x938, 0x92e, 0x25, 0x30, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x25, 0x30, 0x20, 0x938, 0x93e,
0x928, 0x928, 0x93f, 0x20, 0x938, 0x94b, 0x930, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x25, 0x30, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x25, 0x30,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x3a, 0x20, 0x25,
0x30, 0x65, 0x75, 0x72, 0x20, 0x25, 0x30, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x25, 0x30, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x2013,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x20, 0x2013, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x25, 0x30, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x25, 0x30, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x25, 0x30, 0x6642, 0x9593, 0x25, 0x30, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x25,
0x30, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x25, 0x30, 0x65f6, 0x95f4, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x2c, 0x20,
0x25, 0x30, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x2c, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x25, 0x30, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x25, 0x30, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x25, 0x30, 0x628, 0x6d5, 0x6a9, 0x627, 0x62a, 0x6cc, 0x20, 0x25, 0x30, 0x6a9,
0x627, 0x62a, 0x6cc, 0x20, 0x25, 0x30, 0x20, 0x641, 0x6d5, 0x631, 0x645, 0x6cc,
0x6a9, 0x627, 0x62a, 0x6cc, 0x20, 0x25, 0x30, 0x20, 0x647, 0x627, 0x648, 0x6cc,
0x646, 0x6d5, 0x25, 0x30, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x25, 0x30, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0x25,
0x30, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd33, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804,
0xdd34, 0x25, 0x30, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x25, 0x30, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x25,
0x30, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x25, 0x30, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x25, 0x30, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x25, 0x30, 0x20, 0x28, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x29,
0x25, 0x30, 0x20, 0x28, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x29, 0x25, 0x30, 0x20, 0x28, 0x43b,
0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x29, 0x25, 0x30, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x25, 0x30,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x5a, 0x69, 0x63, 0x6b, 0x20, 0x76, 0x75, 0x6e, 0x20,
0x25, 0x30, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x74, 0x74,
0x7a, 0x69, 0x63, 0x6b, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x25, 0x30, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b, 0x20, 0x76, 0x75,
0x6e, 0x20, 0x25, 0x30, 0x6f, 0x72, 0x61, 0x20, 0x3a, 0x20, 0x25, 0x30,
0x25, 0x30, 0x20, 0x28, 0x6f, 0x72, 0x61, 0x20, 0x75, 0x73, 0x75, 0x61,
0x6c, 0x65, 0x29, 0x25, 0x30, 0x20, 0x28, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x61, 0x74, 0x65, 0x29, 0x10d, 0x61, 0x73, 0x6f,
0x76, 0xe9, 0x20, 0x70, 0xe1, 0x73, 0x6d, 0x6f, 0x20, 0x25, 0x30, 0x25,
0x30, 0x2d, 0x74, 0x69, 0x64, 0x25, 0x30, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x25, 0x30, 0x2d, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x25, 0x30, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x25, 0x30, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x25, 0x30, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x93e, 0x902, 0x74, 0x69, 0x6a, 0x64, 0x20, 0x69, 0x6e, 0x20, 0x25, 0x30,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x20, 0x69, 0x6e, 0x20, 0x25, 0x30, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x20, 0x69, 0x6e, 0x20, 0x25, 0x30, 0x25, 0x30,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf0d, 0x25, 0x30, 0x20, 0xd801,
0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0x25, 0x30, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x25, 0x30, 0x20, 0xd801, 0xdc51, 0xd801, 0xdc72, 0xd801,
0xdc65, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x25, 0x30, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x28,
0x25, 0x30, 0x29, 0x25, 0x30, 0x20, 0x67, 0x61, 0x6d, 0x65, 0x25, 0x30,
0x20, 0x67, 0x61, 0x6d, 0x65, 0x20, 0x256, 0x6f, 0x256, 0x6f, 0x65, 0x61,
0x25, 0x30, 0x20, 0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d, 0x65, 0x263, 0x65,
0x79, 0x69, 0x263, 0x69, 0x25, 0x30, 0x20, 0x74, 0xed, 0xf0, 0x25, 0x30,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x25,
0x30, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x25, 0x30, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x6e,
0x67, 0x20, 0x25, 0x30, 0x61, 0x69, 0x6b, 0x61, 0x76, 0x79, 0xf6, 0x68,
0x79, 0x6b, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x28, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x29,
0x25, 0x30, 0x20, 0x28, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x29, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x3a, 0x20, 0x25, 0x30, 0x25,
0x30, 0x20, 0x28, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x29, 0x25, 0x30, 0x20, 0x28, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x29, 0x25, 0x30,
0x20, 0x28, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x29, 0x25, 0x30, 0x20, 0x28, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x29, 0x25, 0x30,
0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x25, 0x30, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x25, 0x30, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd36, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x3a, 0x20, 0x25, 0x30, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x3a, 0x20, 0x25, 0x30, 0x25,
0x30, 0x20, 0x42, 0x65, 0x25, 0x30, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b,
0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x25, 0x30,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x3a, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x3a, 0x20, 0x25,
0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x10d3, 0x10e0, 0x10dd,
0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x25, 0x30,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x25, 0x30, 0x20, 0x28, 0x4f, 0x72, 0x74, 0x73, 0x7a, 0x65, 0x69,
0x74, 0x29, 0x25, 0x30, 0x20, 0x28, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x29, 0x25, 0x30, 0x20, 0x28, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x29, 0x38f, 0x3c1, 0x3b1, 0x20,
0x28, 0x25, 0x30, 0x29, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x28, 0x25, 0x30, 0x29, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x28, 0x25, 0x30,
0x29, 0x25, 0x30, 0x20, 0xab8, 0xaae, 0xaaf, 0x25, 0x30, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0x25, 0x30, 0x20, 0xaa6, 0xabf, 0xab5,
0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0x25, 0x30, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x25, 0x30, 0x20, 0x44, 0x61, 0x69, 0x64, 0x61, 0x69, 0x74,
0x61, 0x63, 0x63, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x25, 0x30, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0x61, 0x6e, 0x61, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x25, 0x30, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x25, 0x30, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x25, 0x30, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x25, 0x30, 0x20, 0x938, 0x92e, 0x92f, 0x25, 0x30, 0x20, 0x69, 0x64,
0x151, 0x25, 0x30, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x25,
0x30, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x25,
0x30, 0x20, 0x28, 0x73, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x29, 0x25, 0x30, 0x20, 0x28, 0x73, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x29, 0x4f, 0x67, 0x65, 0x20, 0x25, 0x30, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x25, 0x30, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x25, 0x30, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x25, 0x30, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x25, 0x30, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x25,
0x30, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x20, 0x25,
0x30, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x25, 0x30,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x3a, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x590f, 0x6642, 0x9593, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x25, 0x30, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x25, 0x30, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x25, 0x30, 0x4f,
0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69,
0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x25, 0x30, 0x41, 0x6b, 0x75,
0x64, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x25, 0x30, 0x20, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x25, 0x30, 0x20, 0xcb8, 0xcae, 0xcaf, 0x25, 0x30, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0x25, 0x30, 0x20,
0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0x25, 0x30, 0x20, 0x648,
0x64e, 0x6a9, 0x6be, 0x25, 0x30, 0x20, 0x645, 0x639, 0x6cc, 0x672, 0x631, 0x6cc,
0x20, 0x648, 0x64e, 0x6a9, 0x6be, 0x25, 0x30, 0x20, 0x688, 0x6d2, 0x20, 0x644,
0x627, 0x626, 0x679, 0x20, 0x648, 0x64e, 0x6a9, 0x6be, 0x25, 0x30, 0x20, 0x935,
0x916, 0x25, 0x30, 0x20, 0x92e, 0x92f, 0x949, 0x930, 0x940, 0x20, 0x935, 0x916,
0x25, 0x30, 0x20, 0x921, 0x947, 0x20, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x935,
0x916, 0x25, 0x30, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x25, 0x30,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x25, 0x30, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x25, 0x30, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x25, 0x30, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x25, 0x30, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x20, 0x25, 0x30, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x20, 0x200b, 0x25, 0x30,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784,
0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x935,
0x947, 0x933, 0x25, 0x30, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x25, 0x30, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x25, 0x30, 0x20, 0x56, 0x65, 0x6c,
0x6c, 0x25, 0x30, 0x20, 0x50, 0x72, 0x6f, 0x6d, 0x61, 0x6e, 0x6e, 0x20,
0x56, 0x65, 0x6c, 0x6c, 0x25, 0x30, 0x20, 0x44, 0x69, 0x73, 0x2d, 0x75,
0x7a, 0x76, 0x61, 0x64, 0x64, 0x20, 0x76, 0x61, 0x63, 0x68, 0x6f, 0x76,
0x70, 0x20, 0x56, 0x65, 0x6c, 0x6c, 0x25, 0x30, 0x20, 0xc2dc, 0xac04, 0x25,
0x30, 0x20, 0xd45c, 0xc900, 0xc2dc, 0x25, 0x30, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x25, 0x30, 0x28, 0x79,
0x29, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x25, 0x30, 0x28, 0x79, 0x29,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x25, 0x30, 0x28, 0x79, 0x29, 0xea, 0x25,
0x30, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0xec0, 0xea7,
0xea5, 0xeb2, 0x20, 0x25, 0x30, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0x20, 0x25, 0x30, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0x20, 0x25, 0x30, 0x4c, 0x61,
0x69, 0x6b, 0x61, 0x20, 0x6a, 0x6f, 0x73, 0x6c, 0x61, 0x3a, 0x20, 0x25,
0x30, 0x25, 0x30, 0x3a, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x61, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x25, 0x30, 0x3a, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4e, 0x67, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x79, 0x61, 0x20, 0x25,
0x30, 0x4c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x3a, 0x20, 0x25, 0x30, 0x17d,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x3a, 0x20, 0x25, 0x30, 0x56, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x3a, 0x20, 0x25, 0x30, 0x43, 0x61,
0x73, 0x6f, 0x77, 0x65, 0x20, 0x70, 0x61, 0x73, 0x6d, 0x6f, 0x20, 0x25,
0x30, 0x25, 0x30, 0x20, 0x7a, 0x79, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x25, 0x30, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x25, 0x30, 0x2d, 0x54, 0x69, 0x65, 0x74,
0x25, 0x30, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74,
0x69, 0x65, 0x74, 0x25, 0x30, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x65, 0x74, 0x25, 0x30, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x25,
0x30, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x25, 0x30, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x25, 0x30,
0x25, 0x30, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0x25, 0x30, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0x25, 0x30, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x25, 0x30, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x25, 0x30,
0x648, 0x642, 0x62a, 0x648, 0x20, 0x25, 0x30, 0x126, 0x69, 0x6e, 0x20, 0x74,
0x61, 0x2019, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x126, 0x69, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x25, 0x30, 0x20, 0x99f,
0x9be, 0x987, 0x9ae, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x30, 0x29, 0x20, 0x9b7,
0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be,
0x987, 0x9ae, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x31, 0x29, 0x20, 0x9a6, 0x9c7,
0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x25, 0x30, 0x20,
0x57, 0x101, 0x25, 0x30, 0x20, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x25, 0x30, 0x20, 0x57, 0x101, 0x20, 0x41,
0x77, 0x61, 0x74, 0x65, 0x61, 0x25, 0x30, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x25, 0x30, 0x20, 0x938, 0x942, 0x930,
0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933,
0x25, 0x30, 0x2d, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x25, 0x30, 0x2d, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x25, 0x30, 0x2d, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x25, 0x30, 0x20, 0x1834, 0x1820, 0x182d, 0x25, 0x30, 0x20, 0x1830,
0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834,
0x1820, 0x182d, 0x25, 0x30, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x180e,
0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x966, 0x29,
0x25, 0x30, 0x20, 0x28, 0x2b, 0x967, 0x29, 0x25, 0x30, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x25, 0x30, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x25, 0x30, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x25, 0x30, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x25, 0x30, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x2d, 0x7e6,
0x7df, 0x7cd, 0x7e1, 0x7ca, 0x7f2, 0x7d3, 0x7ca, 0x7df, 0x7cc, 0x25, 0x30, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x64, 0xe1, 0x62, 0xe1, 0x6c,
0x61, 0x161, 0xe1, 0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x25, 0x30, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x2013, 0x20, 0x25,
0x30, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x2013,
0x20, 0x25, 0x30, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x2013, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x28, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x29, 0x25,
0x30, 0x20, 0x28, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x29, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30,
0x6f, 0x72, 0x61, 0x72, 0x69, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0xb38, 0xb2e,
0xb5f, 0x25, 0x30, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0x25, 0x30, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x25, 0x30, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x25, 0x30, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x25, 0x30, 0x62f, 0x20, 0x25, 0x30, 0x20, 0x67e, 0x647, 0x20,
0x648, 0x62e, 0x62a, 0x25, 0x30, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x62e, 0x62a, 0x25, 0x30, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648,
0x631, 0x681, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x642, 0x62a, 0x20, 0x25, 0x30,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x25, 0x30, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x25, 0x30, 0x63, 0x7a, 0x61, 0x73, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x25, 0x30, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x3a, 0x20,
0x25, 0x30, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x3a, 0x20, 0x25, 0x30, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x25, 0x30, 0x4b, 0x65, 0x72, 0x64, 0x101, 0x3a, 0x20, 0x25, 0x30,
0x5a, 0x113, 0x69, 0x6d, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101,
0x3a, 0x20, 0x25, 0x30, 0x44, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b,
0x65, 0x72, 0x64, 0x101, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0x25, 0x30, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0x25, 0x30, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e,
0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x25, 0x30,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x25, 0x30, 0x74, 0x65, 0x6d, 0x70, 0x3a, 0x20,
0x25, 0x30, 0x74, 0x65, 0x6d, 0x70, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x3a, 0x20, 0x25, 0x30, 0x74, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61,
0x20, 0x73, 0x74, 0x61, 0x64, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x25, 0x30, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x25, 0x30,
0x20, 0x938, 0x92e, 0x92f, 0x3a, 0x25, 0x30, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x938, 0x92e, 0x92f, 0x903, 0x25, 0x30, 0x20, 0x905, 0x92f, 0x93e,
0x92e, 0x938, 0x92e, 0x92f, 0x903, 0x25, 0x30, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a,
0x25, 0x30, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x25, 0x30, 0x20, 0x1c6b, 0x1c64, 0x1c71, 0x1c75, 0x1c6e, 0x1c72, 0x1c5f, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x25,
0x30, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x25, 0x30, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x25, 0x30, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x25, 0x30, 0xa12e, 0xa209, 0x25, 0x30,
0xa395, 0xa3e6, 0xa12e, 0xa209, 0x25, 0x30, 0xa0c5, 0xa3b8, 0xa12e, 0xa209, 0x25, 0x30,
0x20, 0x28, 0x75, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x29, 0x25, 0x30, 0x20, 0x28, 0x75, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x29, 0x25, 0x30, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x25, 0x30, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x25, 0x30, 0x20, 0x648, 0x642, 0x62a,
0x25, 0x30, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x25, 0x30, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x25, 0x30, 0x20, 0x935, 0x915, 0x93c, 0x94d, 0x924,
0x941, 0x25, 0x30, 0x20, 0x92e, 0x907, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935,
0x915, 0x93c, 0x94d, 0x924, 0x941, 0x25, 0x30, 0x20, 0x926, 0x940, 0x902, 0x952,
0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x25, 0x30, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x25, 0x30, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x25, 0x30, 0x20, 0xdaf, 0xdd2,
0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0x25, 0x30, 0x20, 0x10d, 0x61, 0x73, 0x25, 0x30, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x25,
0x30, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x25, 0x30, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x25, 0x30, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61,
0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x4e, 0x61, 0x6b, 0x6f, 0x25, 0x30, 0x20, 0x4e, 0x61, 0x6b, 0x6f,
0x20, 0x79, 0x61, 0x20, 0x54, 0x6c, 0x77, 0x61, 0x65, 0x6c, 0x6f, 0x25,
0x30, 0x20, 0x4e, 0x61, 0x6b, 0x6f, 0x20, 0x79, 0x61, 0x20, 0x4d, 0x6f,
0x74, 0x73, 0x68, 0x65, 0x68, 0x61, 0x72, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x72, 0x69, 0x6f, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x25, 0x30, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x77, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x25,
0x30, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61,
0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x25, 0x30, 0x25, 0x30, 0x74, 0x69,
0x64, 0x25, 0x30, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x29, 0x25, 0x30, 0x20, 0x28, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x29, 0x725, 0x715, 0x722, 0x710, 0x20, 0x25, 0x30,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x25, 0x30, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x25, 0x30, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x25, 0x30, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x25,
0x30, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0x25, 0x30, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0x25, 0x30, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca,
0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x25, 0x30, 0x20, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x25, 0x30, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x25, 0x30, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0x25, 0x30, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0x25, 0x30, 0x20, 0xc2a,
0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xe40, 0xe27, 0xe25, 0xe32, 0x25, 0x30, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0x25, 0x30,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0x25, 0x30,
0x130d, 0x12d8, 0x20, 0x25, 0x30, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x130a, 0x12dc, 0x20, 0x25, 0x30, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x25, 0x30, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8, 0x54, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x25, 0x30, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x25,
0x30, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x25, 0x30, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x25, 0x30, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x25, 0x30, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x25, 0x30, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x25, 0x30, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x25, 0x30,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x439, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x25, 0x30, 0x2c, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x439, 0x10d, 0x61, 0x73, 0x6f, 0x77, 0x65, 0x20, 0x70,
0x61, 0x73, 0x6d, 0x6f, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x7a, 0x79,
0x6d, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x25, 0x30, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x25, 0x30, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x25, 0x30, 0x20,
0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645,
0x25, 0x30, 0x20, 0x62f, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x25, 0x30, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x25, 0x30, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x25, 0x30, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x25, 0x30, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x25, 0x30, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x25, 0x30, 0x20, 0x43a, 0x443, 0x43d,
0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x47,
0x69, 0x1edd, 0x20, 0x25, 0x30, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x25, 0x30, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61,
0x20, 0x68, 0xe8, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x7a, 0x69, 0x74,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x25, 0x30, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x25, 0x30,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x25, 0x30,
0x25, 0x30, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x69, 0x64, 0x20, 0x25, 0x30, 0x5a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x20, 0x25, 0x30, 0xcc,
0x67, 0x62, 0xe0, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0xcc, 0x6c, 0xe0,
0x6e, 0xe0, 0x20, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x25, 0x30, 0x20, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd,
0x25, 0x30, 0x20, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa,
0x6d, 0x254, 0x6d, 0x254, 0x25, 0x30, 0x20, 0x53, 0x69, 0x7a, 0x67, 0x65,
0x6e, 0x68, 0x25, 0x30, 0x20, 0x42, 0x79, 0x61, 0x75, 0x68, 0x63, 0x75,
0x6e, 0x6a, 0x20, 0x53, 0x69, 0x7a, 0x67, 0x65, 0x6e, 0x68, 0x25, 0x30,
0x20, 0x59, 0x61, 0x6c, 0x69, 0x6e, 0x67, 0x20, 0x53, 0x69, 0x7a, 0x67,
0x65, 0x6e, 0x68, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x25, 0x30, 0x25, 0x30, 0x20, 0x69,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69,
0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x25, 0x30, 0x20, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d,
0x69, 0x6e, 0x69, 0xd3, 0x72, 0x61, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x25,
0x30, 0xd3, 0x72, 0x61, 0x20, 0x70, 0x1ebd, 0x20, 0x25, 0x30, 0x50, 0x72,
0x1ef9, 0x67, 0x20, 0x6b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x6b, 0x61,
0x72, 0x20, 0x25, 0x30, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x25, 0x30, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x3a, 0x20, 0x25, 0x30, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20,
0x28, 0x2b, 0x30, 0x29, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x923, 0x94d, 0x921,
0x930, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x25, 0x30, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x25, 0x30, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x25, 0x30, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x74, 0x65, 0x6e, 0x70, 0x6f, 0x20,
0x70, 0x69, 0x20, 0x25, 0x30, 0x74, 0x65, 0x6e, 0x70, 0x6f, 0x20, 0x73,
0x65, 0x6c, 0x69, 0x20, 0x73, 0x75, 0x6e, 0x6f, 0x20, 0x70, 0x69, 0x20,
0x25, 0x30, 0x25, 0x30, 0x20, 0x54, 0x61, 0x65, 0x6d, 0x25, 0x30, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x64, 0x20, 0x54, 0x61, 0x65, 0x6d,
0x25, 0x30, 0x20, 0x44, 0x65, 0x6c, 0x61, 0x65, 0x74, 0x20, 0x54, 0x61,
0x65, 0x6d, 0x6f, 0x61, 0x3a, 0x20, 0x25, 0x30, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x3a, 0x20, 0x25, 0x30, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x3a, 0x20, 0x25, 0x30,
0x25, 0x30, 0x20, 0x648, 0x64e, 0x62e, 0x25, 0x30, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x25, 0x30, 0x20, 0x62f, 0x6be,
0x627, 0x62a, 0x25, 0x30, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x25, 0x30, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x25, 0x30, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x25, 0x30, 0x20, 0x91f, 0x948, 0x92e, 0x25, 0x30, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x25, 0x30, 0x20, 0x927, 0x94d, 0x92f, 0x93e,
0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x25, 0x30, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x25, 0x30, 0x20, 0x1e0d,
0x61, 0x79, 0x6c, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x25, 0x30,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x25, 0x30, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x25, 0x30, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x25, 0x30,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0x25, 0x30, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0x25, 0x30, 0x20, 0xb21,
0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0x25, 0x30,
0x20, 0x1401, 0x1403, 0x1422, 0x1438, 0x14c2, 0x1420, 0x25, 0x30, 0x20, 0x142f, 0x152d,
0x146f, 0x1426, 0x1456, 0x140f, 0x1423, 0x20, 0x1401, 0x1403, 0x1422, 0x1438, 0x14c2, 0x1420,
0x25, 0x30, 0x20, 0x146e, 0x14ef, 0x1473, 0x1424, 0x20, 0x1401, 0x1403, 0x1422, 0x1438,
0x14c2, 0x1420
};

static inline constexpr char16_t fallbackFormatTable[] = {
0x25, 0x31, 0x20, 0x28, 0x25, 0x30, 0x29, 0x25, 0x31, 0x20, 0x25, 0x30,
0x25, 0x31, 0xff08, 0x25, 0x30, 0xff09, 0x25, 0x31, 0xf0d, 0x20, 0x28, 0x25,
0x30, 0xf0b, 0x29, 0x5b, 0x25, 0x31, 0x20, 0x28, 0x25, 0x30, 0x29, 0x5d,
0x25, 0x30, 0x20, 0x28, 0x25, 0x31, 0x29, 0x25, 0x31, 0x28, 0x25, 0x30,
0x29
};

static inline constexpr char16_t exemplarCityTable[] = {
0x41, 0x73, 0x6d, 0x61, 0x72, 0x61, 0x53, 0xe3, 0x6f, 0x20, 0x54, 0x6f,
0x6d, 0xe9, 0x41, 0x73, 0x75, 0x6e, 0x63, 0x69, 0xf3, 0x6e, 0x41, 0x74,
0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6e, 0x42, 0x61, 0x68, 0xed, 0x61, 0x20,
0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x43,
0x61, 0x6e, 0x63, 0xfa, 0x6e, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20,
0x4a, 0x75, 0xe1, 0x72, 0x65, 0x7a, 0x43, 0x75, 0x72, 0x61, 0xe7, 0x61,
0x6f, 0x4b, 0x6e, 0x6f, 0x78, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73,
0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74, 0x79, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x65, 0x76, 0x61, 0x79,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e,
0x63, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x69,
0x63, 0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75,
0x63, 0x6b, 0x79, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x20, 0x50, 0x72, 0x69,
0x6e, 0x63, 0x65, 0x2019, 0x73, 0x20, 0x51, 0x75, 0x61, 0x72, 0x74, 0x65,
0x72, 0x4d, 0xe9, 0x72, 0x69, 0x64, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x75,
0x75, 0x6b, 0x49, 0x74, 0x74, 0x6f, 0x71, 0x71, 0x6f, 0x72, 0x74, 0x6f,
0x6f, 0x72, 0x6d, 0x69, 0x69, 0x74, 0x53, 0x74, 0x2e, 0x20, 0x42, 0x61,
0x72, 0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x53, 0x74, 0x2e, 0x20,
0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73, 0x53, 0x74, 0x2e, 0x20, 0x4b, 0x69,
0x74, 0x74, 0x73, 0x53, 0x74, 0x2e, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61,
0x53, 0x74, 0x2e, 0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x74,
0x2e, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68,
0x20, 0x43, 0x69, 0x74, 0x79, 0x4b, 0x61, 0x74, 0x68, 0x6d, 0x61, 0x6e,
0x64, 0x75, 0x4b, 0x6f, 0x6c, 0x6b, 0x61, 0x74, 0x61, 0x4d, 0x61, 0x63,
0x61, 0x6f, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x59, 0x61,
0x6e, 0x67, 0x6f, 0x6e, 0x46, 0x61, 0x72, 0x6f, 0x65, 0x53, 0x74, 0x2e,
0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x4b, 0x79, 0x69, 0x76, 0x52,
0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x50,
0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x410, 0x431, 0x438, 0x45f, 0x430, 0x43d,
0x410, 0x43a, 0x43a, 0x440, 0x430, 0x410, 0x434, 0x434, 0x438, 0x441, 0x2d, 0x410,
0x431, 0x435, 0x431, 0x430, 0x410, 0x43b, 0x436, 0x438, 0x440, 0x410, 0x441, 0x43c,
0x435, 0x440, 0x430, 0x411, 0x430, 0x43c, 0x430, 0x43a, 0x43e, 0x411, 0x430, 0x43d,
0x433, 0x438, 0x411, 0x430, 0x43d, 0x436, 0x443, 0x43b, 0x411, 0x438, 0x441, 0x430,
0x443, 0x411, 0x43b, 0x430, 0x43d, 0x442, 0x430, 0x438, 0x440, 0x411, 0x440, 0x430,
0x437, 0x437, 0x430, 0x432, 0x438, 0x43b, 0x411, 0x443, 0x436, 0x443, 0x43c, 0x431,
0x443, 0x440, 0x430, 0x41a, 0x430, 0x438, 0x440, 0x41a, 0x430, 0x441, 0x430, 0x431,
0x43b, 0x430, 0x43d, 0x43a, 0x430, 0x421, 0x435, 0x443, 0x442, 0x430, 0x41a, 0x43e,
0x43d, 0x430, 0x43a, 0x440, 0x438, 0x414, 0x430, 0x43a, 0x430, 0x440, 0x414, 0x430,
0x440, 0x2d, 0x435, 0x441, 0x2d, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x40f, 0x438,
0x431, 0x443, 0x442, 0x438, 0x414, 0x443, 0x430, 0x43b, 0x430, 0x415, 0x43b, 0x2d,
0x410, 0x438, 0x443, 0x43d, 0x424, 0x440, 0x438, 0x442, 0x430, 0x443, 0x43d, 0x413,
0x430, 0x431, 0x43e, 0x440, 0x43e, 0x43d, 0x435, 0x4b2, 0x430, 0x440, 0x430, 0x440,
0x435, 0x418, 0x43e, 0x4b3, 0x430, 0x43d, 0x43d, 0x435, 0x441, 0x431, 0x443, 0x440,
0x433, 0x40f, 0x443, 0x431, 0x430, 0x41a, 0x430, 0x43c, 0x43f, 0x430, 0x43b, 0x430,
0x425, 0x430, 0x440, 0x442, 0x443, 0x43c, 0x41a, 0x438, 0x433, 0x430, 0x43b, 0x438,
0x41a, 0x438, 0x43d, 0x448, 0x430, 0x441, 0x430, 0x41b, 0x430, 0x433, 0x43e, 0x441,
0x41b, 0x438, 0x431, 0x440, 0x435, 0x432, 0x438, 0x43b, 0x41b, 0x43e, 0x43c, 0x435,
0x41b, 0x443, 0x430, 0x43d, 0x434, 0x430, 0x41b, 0x443, 0x431, 0x443, 0x43c, 0x431,
0x430, 0x448, 0x438, 0x41b, 0x443, 0x441, 0x430, 0x43a, 0x430, 0x41c, 0x430, 0x43b,
0x430, 0x431, 0x43e, 0x41c, 0x430, 0x43f, 0x443, 0x442, 0x443, 0x41c, 0x430, 0x441,
0x435, 0x440, 0x443, 0x41c, 0x431, 0x430, 0x431, 0x430, 0x43d, 0x435, 0x41c, 0x43e,
0x433, 0x430, 0x434, 0x438, 0x448, 0x43e, 0x41c, 0x43e, 0x43d, 0x440, 0x43e, 0x432,
0x438, 0x430, 0x41d, 0x430, 0x438, 0x440, 0x43e, 0x431, 0x438, 0x41d, 0x45f, 0x430,
0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x438, 0x430, 0x43c, 0x435, 0x438, 0x41d, 0x443,
0x430, 0x43a, 0x448, 0x43e, 0x442, 0x423, 0x430, 0x433, 0x430, 0x434, 0x443, 0x433,
0x443, 0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x2d, 0x41d, 0x43e, 0x432, 0x43e, 0x421,
0x430, 0x43d, 0x2d, 0x422, 0x43e, 0x43c, 0x435, 0x422, 0x440, 0x438, 0x43f, 0x43e,
0x43b, 0x438, 0x422, 0x443, 0x43d, 0x438, 0x441, 0x412, 0x438, 0x43d, 0x434, 0x445,
0x443, 0x43a, 0x410, 0x434, 0x430, 0x43a, 0x410, 0x43d, 0x43a, 0x43e, 0x440, 0x438,
0x45f, 0x410, 0x43d, 0x433, 0x438, 0x43b, 0x438, 0x430, 0x410, 0x43d, 0x442, 0x438,
0x433, 0x443, 0x430, 0x410, 0x440, 0x430, 0x433, 0x443, 0x430, 0x438, 0x43d, 0x430,
0x411, 0x443, 0x435, 0x43d, 0x43e, 0x441, 0x2d, 0x410, 0x438, 0x440, 0x435, 0x441,
0x41a, 0x430, 0x442, 0x430, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x41a, 0x43e, 0x440,
0x434, 0x43e, 0x432, 0x430, 0x416, 0x443, 0x436, 0x443, 0x438, 0x41b, 0x430, 0x2d,
0x420, 0x438, 0x43e, 0x445, 0x430, 0x41c, 0x435, 0x43d, 0x434, 0x43e, 0x441, 0x430,
0x420, 0x438, 0x43e, 0x2d, 0x413, 0x430, 0x43b, 0x435, 0x433, 0x43e, 0x441, 0x421,
0x430, 0x43b, 0x442, 0x430, 0x421, 0x430, 0x43d, 0x2d, 0x425, 0x443, 0x430, 0x43d,
0x421, 0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x438, 0x441, 0x422, 0x443, 0x43a, 0x443,
0x43c, 0x430, 0x43d, 0x423, 0x448, 0x443, 0x430, 0x438, 0x430, 0x410, 0x440, 0x443,
0x431, 0x430, 0x410, 0x441, 0x443, 0x43d, 0x441, 0x43e, 0x43d, 0x41a, 0x43e, 0x440,
0x430, 0x43b, 0x2d, 0x425, 0x430, 0x440, 0x431, 0x43e, 0x440, 0x411, 0x430, 0x438,
0x430, 0x411, 0x430, 0x438, 0x430, 0x2d, 0x434, 0x435, 0x2d, 0x411, 0x430, 0x43d,
0x434, 0x435, 0x440, 0x430, 0x441, 0x411, 0x430, 0x440, 0x431, 0x430, 0x434, 0x43e,
0x441, 0x411, 0x435, 0x43b, 0x435, 0x43d, 0x411, 0x435, 0x43b, 0x438, 0x437, 0x411,
0x43b, 0x430, 0x43d, 0x43a, 0x2d, 0x421, 0x430, 0x431, 0x43b, 0x43e, 0x43d, 0x411,
0x43e, 0x430, 0x2d, 0x412, 0x438, 0x441, 0x442, 0x430, 0x411, 0x43e, 0x433, 0x43e,
0x442, 0x430, 0x411, 0x43e, 0x438, 0x441, 0x435, 0x41a, 0x435, 0x438, 0x43c, 0x431,
0x440, 0x438, 0x45f, 0x2d, 0x411, 0x435, 0x438, 0x41a, 0x430, 0x43c, 0x43f, 0x443,
0x2d, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x438, 0x41a, 0x430, 0x43d, 0x43a, 0x443,
0x43d, 0x41a, 0x430, 0x440, 0x430, 0x43a, 0x430, 0x441, 0x41a, 0x430, 0x438, 0x435,
0x43d, 0x43d, 0x430, 0x410, 0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c,
0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x20, 0x41a, 0x430, 0x438, 0x43c, 0x430,
0x43d, 0x427, 0x438, 0x43a, 0x430, 0x433, 0x43e, 0x427, 0x438, 0x443, 0x430, 0x443,
0x430, 0x41a, 0x43e, 0x441, 0x442, 0x430, 0x2d, 0x420, 0x438, 0x43a, 0x430, 0x41a,
0x440, 0x435, 0x441, 0x442, 0x43e, 0x43d, 0x41a, 0x443, 0x438, 0x430, 0x431, 0x430,
0x41a, 0x438, 0x443, 0x440, 0x430, 0x441, 0x430, 0x43e, 0x414, 0x435, 0x43d, 0x43c,
0x430, 0x440, 0x43a, 0x441, 0x445, 0x430, 0x432, 0x43d, 0x414, 0x43e, 0x443, 0x441,
0x43e, 0x43d, 0x414, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x2d, 0x41a, 0x440, 0x438,
0x43a, 0x414, 0x435, 0x43d, 0x432, 0x435, 0x440, 0x414, 0x435, 0x442, 0x440, 0x43e,
0x438, 0x442, 0x414, 0x43e, 0x43c, 0x438, 0x43d, 0x438, 0x43a, 0x430, 0x415, 0x434,
0x43c, 0x43e, 0x43d, 0x442, 0x43e, 0x43d, 0x415, 0x438, 0x440, 0x443, 0x43d, 0x435,
0x43f, 0x435, 0x421, 0x430, 0x43b, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x424, 0x43e,
0x440, 0x442, 0x20, 0x41d, 0x435, 0x43b, 0x441, 0x43e, 0x43d, 0x424, 0x43e, 0x440,
0x442, 0x430, 0x43b, 0x435, 0x437, 0x430, 0x413, 0x43b, 0x435, 0x438, 0x441, 0x2d,
0x411, 0x435, 0x438, 0x413, 0x443, 0x441, 0x2d, 0x411, 0x435, 0x438, 0x413, 0x440,
0x430, 0x43d, 0x434, 0x2d, 0x422, 0x435, 0x440, 0x43a, 0x413, 0x440, 0x435, 0x43d,
0x430, 0x434, 0x430, 0x413, 0x432, 0x430, 0x434, 0x435, 0x43b, 0x443, 0x43f, 0x430,
0x413, 0x432, 0x430, 0x442, 0x435, 0x43c, 0x430, 0x43b, 0x430, 0x413, 0x443, 0x430,
0x438, 0x430, 0x43a, 0x438, 0x43b, 0x413, 0x430, 0x438, 0x430, 0x43d, 0x430, 0x4b2,
0x430, 0x43b, 0x438, 0x444, 0x430, 0x43a, 0x441, 0x413, 0x430, 0x432, 0x430, 0x43d,
0x430, 0x415, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b, 0x43e, 0x418, 0x43d, 0x434,
0x438, 0x430, 0x43d, 0x430, 0x43f, 0x43e, 0x43b, 0x438, 0x441, 0x41d, 0x43e, 0x43a,
0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x41c, 0x430,
0x440, 0x435, 0x43d, 0x433, 0x43e, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430,
0x43d, 0x430, 0x41f, 0x438, 0x442, 0x435, 0x440, 0x441, 0x431, 0x435, 0x440, 0x433,
0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x422, 0x435, 0x43b,
0x43b, 0x2d, 0x421, 0x438, 0x442, 0x438, 0x412, 0x435, 0x432, 0x435, 0x438, 0x2c,
0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x441,
0x435, 0x43d, 0x43d, 0x435, 0x441, 0x423, 0x438, 0x43d, 0x430, 0x43c, 0x430, 0x43a,
0x418, 0x43d, 0x443, 0x432, 0x438, 0x43a, 0x418, 0x43a, 0x430, 0x43b, 0x443, 0x438,
0x442, 0x418, 0x430, 0x43c, 0x430, 0x438, 0x43a, 0x430, 0x40f, 0x443, 0x43d, 0x43e,
0x41b, 0x443, 0x438, 0x441, 0x432, 0x438, 0x43b, 0x43b, 0x41c, 0x43e, 0x43d, 0x442,
0x438, 0x441, 0x435, 0x43b, 0x43b, 0x43e, 0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442,
0x443, 0x43a, 0x43a, 0x438, 0x41a, 0x440, 0x430, 0x43b, 0x435, 0x43d, 0x434, 0x435,
0x438, 0x43a, 0x41b, 0x430, 0x2d, 0x41f, 0x430, 0x441, 0x41b, 0x438, 0x43c, 0x430,
0x41b, 0x43e, 0x441, 0x2d, 0x410, 0x43d, 0x45f, 0x435, 0x43b, 0x435, 0x441, 0x41b,
0x43e, 0x443, 0x435, 0x440, 0x2d, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x2d, 0x41a,
0x443, 0x43e, 0x442, 0x435, 0x440, 0x41c, 0x430, 0x441, 0x435, 0x438, 0x43e, 0x41c,
0x430, 0x43d, 0x430, 0x433, 0x443, 0x430, 0x41c, 0x430, 0x43d, 0x430, 0x443, 0x441,
0x41c, 0x430, 0x440, 0x438, 0x433, 0x43e, 0x41c, 0x430, 0x440, 0x442, 0x438, 0x43d,
0x438, 0x43a, 0x430, 0x41c, 0x430, 0x442, 0x430, 0x43c, 0x43e, 0x440, 0x43e, 0x441,
0x41c, 0x430, 0x441, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x41c, 0x435, 0x43d, 0x43e,
0x43c, 0x438, 0x43d, 0x438, 0x41c, 0x435, 0x440, 0x438, 0x434, 0x430, 0x41c, 0x435,
0x442, 0x43b, 0x430, 0x43a, 0x430, 0x442, 0x43b, 0x430, 0x41c, 0x435, 0x445, 0x438,
0x43a, 0x43e, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x41c, 0x43e, 0x43d,
0x43a, 0x442, 0x43e, 0x43d, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x440, 0x440, 0x435,
0x438, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x432, 0x438, 0x434, 0x435, 0x43e, 0x41c,
0x43e, 0x43d, 0x442, 0x441, 0x435, 0x440, 0x440, 0x430, 0x442, 0x41d, 0x430, 0x441,
0x441, 0x430, 0x443, 0x41d, 0x438, 0x443, 0x2d, 0x418, 0x43e, 0x440, 0x43a, 0x41d,
0x43e, 0x43c, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x438, 0x430, 0x411, 0x43e, 0x438,
0x43b, 0x430, 0x2c, 0x20, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430,
0x2c, 0x20, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x438, 0x443, 0x2d, 0x421, 0x435, 0x438,
0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x443, 0x443, 0x43a,
0x41e, 0x445, 0x438, 0x43d, 0x430, 0x433, 0x430, 0x41f, 0x430, 0x43d, 0x430, 0x43c,
0x430, 0x41f, 0x430, 0x440, 0x430, 0x43c, 0x430, 0x440, 0x438, 0x431, 0x43e, 0x424,
0x438, 0x43d, 0x438, 0x43a, 0x441, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x2d,
0x41f, 0x440, 0x435, 0x43d, 0x441, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x444,
0x2d, 0x421, 0x43f, 0x435, 0x438, 0x43d, 0x41f, 0x43e, 0x440, 0x442, 0x443, 0x2d,
0x412, 0x435, 0x43b, 0x438, 0x443, 0x41f, 0x443, 0x435, 0x440, 0x442, 0x43e, 0x2d,
0x420, 0x438, 0x43a, 0x43e, 0x41f, 0x443, 0x43d, 0x442, 0x430, 0x2d, 0x410, 0x440,
0x435, 0x43d, 0x430, 0x441, 0x420, 0x430, 0x43d, 0x43a, 0x438, 0x43d, 0x2d, 0x418,
0x43d, 0x43b, 0x435, 0x442, 0x420, 0x435, 0x441, 0x438, 0x444, 0x438, 0x420, 0x435,
0x45f, 0x430, 0x438, 0x43d, 0x430, 0x420, 0x435, 0x437, 0x43e, 0x43b, 0x438, 0x443,
0x442, 0x420, 0x438, 0x443, 0x2d, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x443, 0x421,
0x430, 0x43d, 0x442, 0x430, 0x440, 0x435, 0x43d, 0x421, 0x430, 0x43d, 0x442, 0x438,
0x430, 0x433, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x43e, 0x2d, 0x414, 0x43e, 0x43c,
0x438, 0x43d, 0x433, 0x43e, 0x421, 0x430, 0x43d, 0x2d, 0x41f, 0x430, 0x443, 0x43b,
0x443, 0x421, 0x43a, 0x43e, 0x440, 0x441, 0x431, 0x438, 0x441, 0x443, 0x43d, 0x43d,
0x421, 0x438, 0x442, 0x43a, 0x430, 0x421, 0x435, 0x43d, 0x2d, 0x411, 0x430, 0x440,
0x442, 0x435, 0x43b, 0x435, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x40f,
0x43e, 0x43d, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41a, 0x438, 0x442, 0x441,
0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41b, 0x438, 0x443, 0x441, 0x438, 0x430, 0x421,
0x435, 0x43d, 0x442, 0x2d, 0x422, 0x43e, 0x43c, 0x430, 0x441, 0x421, 0x435, 0x43d,
0x442, 0x2d, 0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x442, 0x421, 0x432, 0x438,
0x444, 0x442, 0x2d, 0x41a, 0x435, 0x440, 0x440, 0x435, 0x43d, 0x442, 0x422, 0x435,
0x433, 0x443, 0x441, 0x438, 0x433, 0x430, 0x43b, 0x43f, 0x430, 0x422, 0x443, 0x43b,
0x435, 0x422, 0x438, 0x445, 0x443, 0x430, 0x43d, 0x430, 0x422, 0x43e, 0x440, 0x43e,
0x43d, 0x442, 0x43e, 0x422, 0x43e, 0x440, 0x442, 0x43e, 0x43b, 0x430, 0x412, 0x430,
0x43d, 0x43a, 0x443, 0x432, 0x435, 0x440, 0x423, 0x430, 0x438, 0x442, 0x445, 0x43e,
0x440, 0x441, 0x412, 0x438, 0x43d, 0x43d, 0x438, 0x43f, 0x435, 0x433, 0x418, 0x430,
0x43a, 0x443, 0x442, 0x430, 0x442, 0x41a, 0x435, 0x438, 0x441, 0x438, 0x414, 0x435,
0x438, 0x432, 0x438, 0x441, 0x414, 0x438, 0x443, 0x43c, 0x43e, 0x43d, 0x2d, 0x434,
0x2019, 0x418, 0x443, 0x440, 0x432, 0x438, 0x43b, 0x41c, 0x430, 0x43a, 0x43a, 0x443,
0x43e, 0x440, 0x438, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x41c, 0x430, 0x43a,
0x2d, 0x41c, 0x435, 0x440, 0x434, 0x43e, 0x41f, 0x430, 0x43b, 0x43c, 0x435, 0x440,
0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x421, 0x438, 0x43e, 0x432, 0x430, 0x422,
0x440, 0x43e, 0x43b, 0x43b, 0x410, 0x43c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430,
0x440, 0x430, 0x41b, 0x43e, 0x43d, 0x433, 0x438, 0x440, 0x410, 0x434, 0x435, 0x43d,
0x410, 0x43b, 0x43c, 0x430, 0x442, 0x44b, 0x410, 0x43c, 0x43c, 0x430, 0x43d, 0x410,
0x43d, 0x430, 0x434, 0x44b, 0x440, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x410, 0x43a,
0x442, 0x43e, 0x431, 0x435, 0x410, 0x448, 0x445, 0x430, 0x431, 0x430, 0x434, 0x410,
0x442, 0x44b, 0x440, 0x430, 0x443, 0x411, 0x430, 0x433, 0x434, 0x430, 0x434, 0x411,
0x430, 0x445, 0x440, 0x435, 0x438, 0x43d, 0x411, 0x430, 0x43a, 0x443, 0x411, 0x430,
0x43d, 0x433, 0x43a, 0x43e, 0x43a, 0x411, 0x430, 0x440, 0x43d, 0x430, 0x443, 0x43b,
0x411, 0x435, 0x438, 0x440, 0x443, 0x442, 0x411, 0x438, 0x448, 0x43a, 0x435, 0x43a,
0x411, 0x440, 0x443, 0x43d, 0x435, 0x438, 0x427, 0x438, 0x442, 0x430, 0x41a, 0x43e,
0x43b, 0x43e, 0x43c, 0x431, 0x43e, 0x414, 0x430, 0x43c, 0x430, 0x441, 0x43a, 0x414,
0x430, 0x43a, 0x43a, 0x430, 0x414, 0x438, 0x43b, 0x438, 0x414, 0x443, 0x431, 0x430,
0x438, 0x414, 0x443, 0x448, 0x430, 0x43d, 0x431, 0x435, 0x424, 0x430, 0x43c, 0x430,
0x433, 0x443, 0x441, 0x442, 0x430, 0x413, 0x430, 0x437, 0x430, 0x425, 0x435, 0x432,
0x440, 0x43e, 0x43d, 0x425, 0x43e, 0x448, 0x438, 0x43c, 0x438, 0x43d, 0x4b2, 0x43e,
0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x425, 0x43e, 0x432, 0x434, 0x418, 0x440, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x40f, 0x430, 0x43a, 0x430, 0x440, 0x442, 0x430, 0x40f,
0x430, 0x438, 0x430, 0x43f, 0x443, 0x440, 0x430, 0x418, 0x435, 0x440, 0x443, 0x441,
0x430, 0x43b, 0x438, 0x43c, 0x41a, 0x430, 0x431, 0x443, 0x43b, 0x41f, 0x435, 0x442,
0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a,
0x430, 0x43c, 0x447, 0x430, 0x442, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x41a, 0x430,
0x440, 0x430, 0x447, 0x438, 0x41a, 0x430, 0x442, 0x43c, 0x430, 0x43d, 0x434, 0x443,
0x425, 0x430, 0x43d, 0x434, 0x44b, 0x433, 0x430, 0x41a, 0x430, 0x43b, 0x43a, 0x443,
0x442, 0x442, 0x430, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x438, 0x430, 0x440,
0x441, 0x43a, 0x41a, 0x443, 0x430, 0x43b, 0x430, 0x2d, 0x41b, 0x443, 0x43c, 0x43f,
0x443, 0x440, 0x41a, 0x443, 0x447, 0x438, 0x43d, 0x433, 0x41a, 0x443, 0x432, 0x435,
0x438, 0x442, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x41c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x41c, 0x430, 0x43a, 0x430, 0x441, 0x430, 0x440, 0x41c, 0x430, 0x43d,
0x438, 0x43b, 0x430, 0x41c, 0x430, 0x441, 0x43a, 0x430, 0x442, 0x41d, 0x438, 0x43a,
0x43e, 0x441, 0x438, 0x430, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x443, 0x437, 0x43d,
0x435, 0x446, 0x43a, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440,
0x441, 0x43a, 0x41e, 0x43c, 0x441, 0x43a, 0x423, 0x440, 0x430, 0x43b, 0x441, 0x43a,
0x41f, 0x43d, 0x43e, 0x43c, 0x43f, 0x435, 0x43d, 0x41f, 0x43e, 0x43d, 0x442, 0x438,
0x430, 0x43d, 0x430, 0x43a, 0x41f, 0x445, 0x435, 0x43d, 0x438, 0x430, 0x43d, 0x41a,
0x430, 0x442, 0x430, 0x440, 0x41a, 0x43e, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x438,
0x41a, 0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x415, 0x440, 0x2d,
0x420, 0x438, 0x430, 0x434, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x43d, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x43a, 0x430,
0x43d, 0x434, 0x421, 0x435, 0x443, 0x43b, 0x428, 0x430, 0x43d, 0x445, 0x430, 0x438,
0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x421, 0x440, 0x435, 0x434,
0x43d, 0x435, 0x43a, 0x43e, 0x43b, 0x44b, 0x43c, 0x441, 0x43a, 0x422, 0x430, 0x438,
0x431, 0x435, 0x438, 0x422, 0x430, 0x448, 0x43a, 0x435, 0x43d, 0x442, 0x49a, 0x430,
0x440, 0x4ad, 0x422, 0x435, 0x433, 0x435, 0x440, 0x430, 0x43d, 0x422, 0x445, 0x438,
0x43c, 0x43f, 0x445, 0x443, 0x422, 0x43e, 0x43a, 0x438, 0x43e, 0x422, 0x43e, 0x43c,
0x441, 0x43a, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x423, 0x440, 0x443, 0x43c, 0x447, 0x438, 0x423, 0x441, 0x442, 0x2d, 0x41d, 0x435,
0x440, 0x430, 0x412, 0x435, 0x43d, 0x442, 0x438, 0x430, 0x43d, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x418, 0x430, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x418, 0x430, 0x43d, 0x433, 0x43e, 0x43d, 0x415, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x415, 0x440, 0x435,
0x432, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x411, 0x435, 0x440, 0x43c, 0x443,
0x434, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9,
0x430, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434,
0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412,
0x435, 0x440, 0x434, 0x435, 0x424, 0x430, 0x440, 0x435, 0x440, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41c, 0x430, 0x434,
0x435, 0x438, 0x440, 0x430, 0x420, 0x435, 0x438, 0x43a, 0x438, 0x430, 0x432, 0x438,
0x43a, 0x410, 0x43b, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x435,
0x43e, 0x440, 0x433, 0x438, 0x430, 0x418, 0x525, 0x448, 0x44c, 0x43e, 0x443, 0x20,
0x415, 0x43b, 0x435, 0x43d, 0x430, 0x20, 0x43b, 0x430, 0x434, 0x433, 0x44c, 0x44b,
0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x421, 0x442, 0x435, 0x43d, 0x43b,
0x438, 0x410, 0x434, 0x435, 0x43b, 0x430, 0x438, 0x434, 0x430, 0x411, 0x440, 0x438,
0x441, 0x431, 0x435, 0x43d, 0x411, 0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x2d, 0x425,
0x438, 0x43b, 0x43b, 0x414, 0x430, 0x440, 0x432, 0x438, 0x43d, 0x418, 0x443, 0x43a,
0x43b, 0x430, 0x425, 0x43e, 0x431, 0x430, 0x440, 0x442, 0x41b, 0x438, 0x43d, 0x434,
0x435, 0x43c, 0x430, 0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443,
0x41c, 0x435, 0x43b, 0x431, 0x443, 0x440, 0x43d, 0x41f, 0x435, 0x440, 0x442, 0x421,
0x438, 0x434, 0x43d, 0x435, 0x438, 0x410, 0x43c, 0x441, 0x442, 0x435, 0x440, 0x434,
0x430, 0x43c, 0x410, 0x43d, 0x434, 0x43e, 0x440, 0x440, 0x430, 0x410, 0x441, 0x442,
0x440, 0x430, 0x445, 0x430, 0x43d, 0x410, 0x444, 0x438, 0x43d, 0x411, 0x435, 0x43b,
0x433, 0x440, 0x430, 0x434, 0x411, 0x435, 0x440, 0x43b, 0x438, 0x43d, 0x411, 0x440,
0x430, 0x442, 0x438, 0x441, 0x43b, 0x430, 0x432, 0x430, 0x411, 0x440, 0x438, 0x443,
0x441, 0x441, 0x435, 0x43b, 0x411, 0x443, 0x445, 0x430, 0x440, 0x435, 0x441, 0x442,
0x411, 0x443, 0x434, 0x430, 0x43f, 0x435, 0x448, 0x442, 0x411, 0x438, 0x443, 0x437,
0x438, 0x43d, 0x433, 0x435, 0x43d, 0x2d, 0x425, 0x44b, 0x445, 0x44c, 0x442, 0x4d9,
0x438, 0x2d, 0x420, 0x435, 0x438, 0x43d, 0x435, 0x41a, 0x438, 0x448, 0x438, 0x43d,
0x435, 0x432, 0x41a, 0x43e, 0x43f, 0x435, 0x43d, 0x4b3, 0x430, 0x433, 0x435, 0x43d,
0x414, 0x443, 0x431, 0x43b, 0x438, 0x43d, 0x413, 0x438, 0x431, 0x440, 0x430, 0x43b,
0x442, 0x430, 0x440, 0x413, 0x435, 0x440, 0x43d, 0x441, 0x438, 0x425, 0x435, 0x43b,
0x441, 0x438, 0x43d, 0x43a, 0x438, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x41c,
0x435, 0x43d, 0x421, 0x442, 0x430, 0x43c, 0x431, 0x443, 0x43b, 0x40f, 0x435, 0x440,
0x441, 0x438, 0x41a, 0x430, 0x43b, 0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430,
0x434, 0x41a, 0x438, 0x440, 0x43e, 0x432, 0x41a, 0x438, 0x435, 0x432, 0x41b, 0x438,
0x441, 0x441, 0x430, 0x431, 0x43e, 0x43d, 0x41b, 0x438, 0x443, 0x431, 0x43b, 0x438,
0x430, 0x43d, 0x430, 0x41b, 0x43e, 0x43d, 0x434, 0x43e, 0x43d, 0x41b, 0x438, 0x443,
0x43a, 0x441, 0x435, 0x43c, 0x431, 0x443, 0x440, 0x433, 0x41c, 0x430, 0x434, 0x440,
0x438, 0x434, 0x41c, 0x430, 0x43b, 0x442, 0x430, 0x41c, 0x430, 0x440, 0x438, 0x435,
0x445, 0x430, 0x43c, 0x43d, 0x41c, 0x438, 0x43d, 0x441, 0x43a, 0x41c, 0x43e, 0x43d,
0x430, 0x43a, 0x43e, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x41e, 0x441, 0x43b,
0x43e, 0x41f, 0x430, 0x440, 0x438, 0x436, 0x41f, 0x43e, 0x434, 0x433, 0x43e, 0x440,
0x438, 0x446, 0x430, 0x41f, 0x440, 0x430, 0x433, 0x430, 0x420, 0x438, 0x433, 0x430,
0x420, 0x438, 0x43c, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x421, 0x430, 0x43d,
0x2d, 0x41c, 0x430, 0x440, 0x438, 0x43d, 0x43e, 0x421, 0x430, 0x440, 0x430, 0x435,
0x432, 0x43e, 0x421, 0x430, 0x440, 0x430, 0x442, 0x43e, 0x432, 0x421, 0x438, 0x43c,
0x444, 0x435, 0x440, 0x43e, 0x43f, 0x43e, 0x43b, 0x421, 0x43a, 0x43e, 0x43f, 0x435,
0x421, 0x43e, 0x444, 0x438, 0x430, 0x421, 0x442, 0x43e, 0x43a, 0x4b3, 0x43e, 0x43b,
0x43c, 0x422, 0x430, 0x43b, 0x43b, 0x438, 0x43d, 0x422, 0x438, 0x440, 0x430, 0x43d,
0x430, 0x423, 0x43b, 0x438, 0x430, 0x43d, 0x43e, 0x432, 0x441, 0x43a, 0x412, 0x430,
0x434, 0x443, 0x446, 0x412, 0x430, 0x442, 0x438, 0x43a, 0x430, 0x43d, 0x412, 0x435,
0x43d, 0x430, 0x412, 0x438, 0x43b, 0x43d, 0x438, 0x443, 0x441, 0x412, 0x43e, 0x43b,
0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x412, 0x430, 0x440, 0x448, 0x430, 0x432,
0x430, 0x417, 0x430, 0x433, 0x440, 0x435, 0x431, 0x426, 0x438, 0x443, 0x440, 0x438,
0x445, 0x410, 0x43d, 0x442, 0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x438, 0x432,
0x443, 0x427, 0x430, 0x433, 0x43e, 0x441, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20,
0x49a, 0x44c, 0x44b, 0x440, 0x441, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41a,
0x43e, 0x43c, 0x43e, 0x440, 0x41a, 0x435, 0x440, 0x433, 0x435, 0x43b, 0x435, 0x43d,
0x41c, 0x430, 0x435, 0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x41c, 0x430, 0x432,
0x440, 0x438, 0x43a, 0x438, 0x41c, 0x430, 0x438, 0x43e, 0x442, 0x442, 0x430, 0x420,
0x435, 0x438, 0x443, 0x43d, 0x43e, 0x43d, 0x410, 0x43f, 0x438, 0x430, 0x41e, 0x43a,
0x43b, 0x435, 0x43d, 0x434, 0x411, 0x443, 0x433, 0x435, 0x43d, 0x432, 0x438, 0x43b,
0x427, 0x430, 0x442, 0x435, 0x43c, 0x422, 0x440, 0x443, 0x43a, 0x430, 0x434, 0x2d,
0x445, 0x430, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x415, 0x444, 0x430, 0x442,
0x435, 0x424, 0x430, 0x43a, 0x430, 0x43e, 0x444, 0x43e, 0x424, 0x438, 0x45f, 0x438,
0x424, 0x443, 0x43d, 0x430, 0x444, 0x443, 0x442, 0x438, 0x413, 0x430, 0x43b, 0x430,
0x43f, 0x430, 0x433, 0x43e, 0x441, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d,
0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9,
0x430, 0x20, 0x413, 0x430, 0x43c, 0x431, 0x435, 0x413, 0x443, 0x430, 0x434, 0x430,
0x43b, 0x43a, 0x430, 0x43d, 0x430, 0x43b, 0x413, 0x443, 0x430, 0x43c, 0x41a, 0x430,
0x43d, 0x442, 0x43e, 0x43d, 0x41a, 0x438, 0x440, 0x438, 0x442, 0x438, 0x43c, 0x430,
0x442, 0x438, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435, 0x41a, 0x432, 0x430, 0x45f,
0x430, 0x43b, 0x435, 0x438, 0x43d, 0x41c, 0x430, 0x45f, 0x443, 0x440, 0x43e, 0x41c,
0x430, 0x440, 0x43a, 0x438, 0x437, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d,
0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9,
0x430, 0x20, 0x41c, 0x438, 0x434, 0x443, 0x435, 0x438, 0x41d, 0x430, 0x443, 0x440,
0x443, 0x41d, 0x438, 0x443, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a,
0x41d, 0x443, 0x43c, 0x435, 0x430, 0x41f, 0x430, 0x433, 0x43e, 0x2d, 0x41f, 0x430,
0x433, 0x43e, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x41f, 0x438, 0x442, 0x43a, 0x435,
0x440, 0x43d, 0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x438, 0x41f, 0x43e, 0x440, 0x442,
0x2d, 0x41c, 0x43e, 0x440, 0x441, 0x431, 0x438, 0x420, 0x430, 0x440, 0x43e, 0x442,
0x43e, 0x43d, 0x433, 0x430, 0x421, 0x430, 0x438, 0x43f, 0x430, 0x43d, 0x422, 0x430,
0x438, 0x442, 0x438, 0x422, 0x430, 0x440, 0x430, 0x432, 0x430, 0x422, 0x43e, 0x43d,
0x433, 0x430, 0x442, 0x430, 0x43f, 0x443, 0x423, 0x435, 0x438, 0x43a, 0x423, 0x43e,
0x43b, 0x43b, 0x438, 0x441, 0x41, 0x64, 0x64, 0x69, 0x73, 0x20, 0x41, 0x62,
0x65, 0x62, 0x61, 0x4b, 0x61, 0xef, 0x72, 0x6f, 0x44, 0x6a, 0x69, 0x62,
0x6f, 0x65, 0x74, 0x69, 0x4b, 0x61, 0x72, 0x74, 0x6f, 0x65, 0x6d, 0x4d,
0x6f, 0x67, 0x61, 0x64, 0x69, 0x73, 0x6a, 0x6f, 0x65, 0x42, 0x61, 0x68,
0x69, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x43,
0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x62, 0x61, 0x61, 0x69,
0x4b, 0x61, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x47, 0x6c, 0x61, 0x63, 0x65,
0x62, 0x61, 0x61, 0x69, 0x47, 0x6f, 0x6f, 0x73, 0x65, 0x62, 0x61, 0x61,
0x69, 0x4a, 0x61, 0x6d, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x6f, 0x73, 0x74, 0x61, 0x64, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e,
0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f,
0x6f, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x69,
0x6e, 0x74, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0x6f, 0x6c, 0x6f, 0x6d,
0x65, 0x75, 0x73, 0x53, 0x69, 0x6e, 0x74, 0x20, 0x4a, 0x6f, 0x68, 0x6e,
0x2019, 0x73, 0x53, 0x69, 0x6e, 0x74, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73,
0x53, 0x69, 0x6e, 0x74, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x69,
0x6e, 0x74, 0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x69, 0x6e,
0x74, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x57, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x41, 0x73, 0x6a, 0x63, 0x68, 0x61, 0x62, 0x61, 0x64,
0x42, 0x61, 0x67, 0x64, 0x61, 0x64, 0x42, 0x61, 0x68, 0x72, 0x65, 0x69,
0x6e, 0x42, 0x61, 0x6b, 0x6f, 0x65, 0x42, 0x65, 0x69, 0x72, 0x6f, 0x65,
0x74, 0x42, 0x69, 0x73, 0x6a, 0x6b, 0x65, 0x6b, 0x42, 0x72, 0x6f, 0x65,
0x6e, 0x65, 0x69, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x75, 0x73, 0x44,
0x6f, 0x65, 0x62, 0x61, 0x69, 0x48, 0x6f, 0x20, 0x54, 0x73, 0x6a, 0x69,
0x20, 0x4d, 0x69, 0x6e, 0x68, 0x73, 0x74, 0x61, 0x64, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73,
0x6b, 0x44, 0x6a, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61, 0x4b, 0x61, 0x62,
0x6f, 0x65, 0x6c, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x6b,
0x61, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x73, 0x6a, 0x69, 0x4b, 0x61, 0x74,
0x6d, 0x61, 0x6e, 0x64, 0x6f, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x4b, 0x6f, 0x65, 0x61, 0x6c, 0x61, 0x2d,
0x4c, 0x6f, 0x65, 0x6d, 0x70, 0x6f, 0x65, 0x72, 0x4b, 0x6f, 0x65, 0x77,
0x65, 0x69, 0x74, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x4d, 0x75, 0x73, 0x6b,
0x61, 0x74, 0x4b, 0x61, 0x74, 0x61, 0x72, 0x52, 0x69, 0x61, 0x61, 0x64,
0x53, 0x65, 0x6f, 0x65, 0x6c, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x65, 0x72, 0x54, 0x6f, 0x6b, 0x69, 0x6f, 0x57, 0x6c, 0x61, 0x64, 0x69,
0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74,
0x73, 0x6b, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x4b, 0x61, 0x6e, 0x61,
0x72, 0x69, 0x65, 0x4b, 0x61, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x53, 0x75, 0x69, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0xeb, 0x53, 0x69, 0x6e, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61,
0x41, 0x74, 0x68, 0x65, 0x6e, 0x65, 0x42, 0x65, 0x72, 0x6c, 0x79, 0x6e,
0x42, 0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x42, 0x6f, 0x65, 0x6b, 0x61,
0x72, 0x65, 0x73, 0x74, 0x42, 0x6f, 0x65, 0x64, 0x61, 0x70, 0x65, 0x73,
0x74, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x65, 0x6e, 0x45,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x6e, 0x4b, 0x69, 0xeb,
0x66, 0x4c, 0x69, 0x73, 0x73, 0x61, 0x62, 0x6f, 0x6e, 0x4c, 0x6f, 0x6e,
0x64, 0x65, 0x6e, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67,
0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x50, 0x61, 0x72, 0x79, 0x73, 0x50,
0x72, 0x61, 0x61, 0x67, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x66, 0x56,
0x61, 0x74, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x74, 0x61, 0x64, 0x57,
0x65, 0x6e, 0x65, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x57, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x75, 0x5a, 0xfc, 0x72, 0x69, 0x63,
0x68, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x65,
0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x76, 0x65, 0x50, 0x61, 0x61, 0x73,
0x46, 0x69, 0x64, 0x6a, 0x69, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c,
0x75, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x75, 0x72, 0x79, 0x4e, 0x6f,
0x75, 0x6d, 0xe9, 0x61, 0x4d, 0x61, 0x74, 0x61, 0x2d, 0x55, 0x74, 0x75,
0x41, 0x62, 0x69, 0x67, 0x79, 0x61, 0x6e, 0x41, 0x6b, 0x72, 0x61, 0x61,
0x186, 0x6c, 0x67, 0x79, 0x65, 0x73, 0x65, 0x42, 0x61, 0x6e, 0x64, 0x77,
0x75, 0x75, 0x42, 0x69, 0x73, 0x61, 0x77, 0x42, 0x6c, 0x61, 0x6e, 0x74,
0x61, 0x69, 0x42, 0x75, 0x64, 0x77, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61,
0x4b, 0x61, 0x69, 0x72, 0x6f, 0x4b, 0x61, 0x73, 0x61, 0x62, 0x6c, 0x61,
0x6e, 0x6b, 0x61, 0x4b, 0x79, 0x75, 0x74, 0x61, 0x4b, 0x254, 0x6e, 0x61,
0x6b, 0x72, 0x69, 0x44, 0x61, 0x6b, 0x61, 0x61, 0x47, 0x79, 0x69, 0x62,
0x75, 0x75, 0x74, 0x69, 0x44, 0x77, 0x75, 0x62, 0x61, 0x4b, 0x69, 0x6e,
0x68, 0x79, 0x61, 0x61, 0x68, 0x79, 0x61, 0x4c, 0x65, 0x67, 0x254, 0x73,
0x4e, 0x67, 0x79, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x57, 0x61, 0x67, 0x61,
0x64, 0x75, 0x67, 0x75, 0x41, 0x6e, 0x6b, 0x254, 0x72, 0x61, 0x67, 0x79,
0x69, 0x41, 0x6e, 0x67, 0x75, 0x69, 0x6c, 0x61, 0x4b, 0x61, 0x74, 0x61,
0x6d, 0x61, 0x61, 0x6b, 0x61, 0x4b, 0x254, 0x254, 0x64, 0x254, 0x62, 0x61,
0x44, 0x77, 0x75, 0x64, 0x77, 0x75, 0x69, 0x4c, 0x61, 0x20, 0x52, 0x69,
0x6f, 0x67, 0x79, 0x61, 0x4d, 0x25b, 0x6e, 0x64, 0x254, 0x73, 0x61, 0x52,
0x69, 0x6f, 0x20, 0x47, 0x61, 0x6c, 0x6c, 0x65, 0x67, 0x254, 0x73, 0x53,
0x61, 0x6e, 0x20, 0x44, 0x77, 0x75, 0x61, 0x6e, 0x54, 0x75, 0x6b, 0x75,
0x6d, 0x61, 0x6e, 0x42, 0x61, 0x61, 0x62, 0x61, 0x64, 0x6f, 0x73, 0x42,
0x25b, 0x6c, 0x25b, 0x6d, 0x42, 0x25b, 0x6c, 0x69, 0x73, 0x69, 0x42, 0x6c,
0x61, 0x6e, 0x63, 0x2d, 0x53, 0x61, 0x62, 0x6c, 0x254, 0x6e, 0x42, 0x254,
0x69, 0x73, 0x69, 0x4b, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x67, 0x79, 0x69,
0x20, 0x42, 0x61, 0x79, 0x4b, 0x61, 0x6d, 0x70, 0x6f, 0x20, 0x47, 0x72,
0x61, 0x6e, 0x64, 0x65, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x73, 0x4b,
0x61, 0x79, 0x69, 0x69, 0x6e, 0x69, 0x4b, 0x65, 0x6d, 0x61, 0x6e, 0x66,
0x6f, 0x4b, 0x79, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0x4b, 0x79, 0x69, 0x68,
0x75, 0x61, 0x68, 0x75, 0x61, 0x4b, 0x254, 0x73, 0x74, 0x61, 0x20, 0x52,
0x69, 0x6b, 0x61, 0x4b, 0x72, 0x25b, 0x73, 0x74, 0x6f, 0x6e, 0x4b, 0x75,
0x69, 0x61, 0x62, 0x61, 0x4b, 0x75, 0x72, 0x75, 0x6b, 0x61, 0x77, 0x44,
0x254, 0x254, 0x73, 0x6f, 0x6e, 0x44, 0x254, 0x254, 0x73, 0x6f, 0x6e, 0x20,
0x4b, 0x72, 0x65, 0x65, 0x6b, 0x44, 0x25b, 0x6e, 0x76, 0x61, 0x44, 0x65,
0x74, 0x72, 0x254, 0x69, 0x74, 0x44, 0x254, 0x6d, 0x65, 0x6e, 0x65, 0x6b,
0x61, 0x45, 0x64, 0x6d, 0x254, 0x6e, 0x74, 0x6f, 0x6e, 0x45, 0x6c, 0x20,
0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x254, 0x254, 0x46, 0x254, 0x74, 0x20,
0x4e, 0x25b, 0x6c, 0x73, 0x6f, 0x6e, 0x46, 0x254, 0x254, 0x74, 0x61, 0x6c,
0x25b, 0x73, 0x61, 0x47, 0x75, 0x75, 0x73, 0x20, 0x42, 0x61, 0x79, 0x47,
0x72, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x75, 0x6b, 0x47, 0x75, 0x77, 0x61,
0x64, 0x65, 0x6c, 0x75, 0x70, 0x47, 0x75, 0x77, 0x61, 0x74, 0x65, 0x6d,
0x61, 0x6c, 0x61, 0x47, 0x61, 0x79, 0x61, 0x6b, 0x77, 0x75, 0x69, 0x6c,
0x47, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x48, 0x25b, 0x6d, 0x254, 0x73, 0x69,
0x6c, 0x6f, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x70, 0x254, 0x6c,
0x69, 0x73, 0x50, 0x69, 0x74, 0x25b, 0x73, 0x62, 0x25b, 0x67, 0x79, 0x65,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c,
0x6c, 0x20, 0x53, 0x69, 0x74, 0x69, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x49, 0x6b, 0x61, 0x6c, 0x75,
0x69, 0x74, 0x47, 0x79, 0x61, 0x6d, 0x65, 0x6b, 0x61, 0x4d, 0x254, 0x6e,
0x74, 0x69, 0x73, 0x25b, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x25b, 0x6e, 0x74,
0x25b, 0x6b, 0x69, 0x4b, 0x72, 0x61, 0x6c, 0x25b, 0x6e, 0x67, 0x79, 0x69,
0x6b, 0x4c, 0x254, 0x73, 0x20, 0x41, 0x6e, 0x67, 0x79, 0x25b, 0x6c, 0x69,
0x73, 0x4c, 0x6f, 0x77, 0x61, 0x20, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x65,
0x20, 0x4b, 0x254, 0x74, 0x61, 0x4d, 0x61, 0x6b, 0x65, 0x69, 0x6f, 0x4d,
0x61, 0x72, 0x69, 0x67, 0x254, 0x74, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e,
0x69, 0x6b, 0x65, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x254, 0x72, 0x254, 0x73,
0x6f, 0x4d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x4d, 0x25b, 0x6e,
0x254, 0x6d, 0x69, 0x6e, 0x65, 0x65, 0x4d, 0x25b, 0x74, 0x6c, 0x61, 0x6b,
0x61, 0x74, 0x6c, 0x61, 0x4d, 0x25b, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20,
0x53, 0x69, 0x74, 0x69, 0x4d, 0x254, 0x6e, 0x6b, 0x74, 0x69, 0x6e, 0x4d,
0x254, 0x6e, 0x74, 0x69, 0x72, 0x69, 0x69, 0x4d, 0x61, 0x6e, 0x74, 0x73,
0x65, 0x72, 0x61, 0x74, 0x4e, 0x65, 0x77, 0x20, 0x59, 0x254, 0x6b, 0x42,
0x65, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x4e, 0x254, 0x66, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x53, 0x25b, 0x6e, 0x74, 0x61, 0x2c, 0x20, 0x4e,
0x254, 0x66, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77,
0x20, 0x53, 0x61, 0x6c, 0x25b, 0x6d, 0x2c, 0x20, 0x4e, 0x254, 0x66, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4f, 0x67, 0x79, 0x69, 0x6e, 0x61,
0x67, 0x61, 0x50, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x254,
0x46, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x53, 0x70, 0x61, 0x69, 0x6e, 0x20,
0x50, 0x254, 0x254, 0x74, 0x6f, 0x50, 0x254, 0x254, 0x74, 0x6f, 0x20, 0x56,
0x65, 0x6c, 0x68, 0x6f, 0x50, 0x75, 0x25b, 0x74, 0x6f, 0x20, 0x52, 0x69,
0x6b, 0x6f, 0x52, 0x25b, 0x73, 0x69, 0x66, 0x25b, 0x52, 0x25b, 0x67, 0x79,
0x69, 0x6e, 0x61, 0x52, 0x25b, 0x73, 0x254, 0x6c, 0x75, 0x74, 0x52, 0x69,
0x6f, 0x20, 0x42, 0x72, 0x61, 0x6e, 0x6b, 0x6f, 0x59, 0x69, 0x74, 0x6f,
0x6b, 0x254, 0x74, 0x75, 0x254, 0x6d, 0x65, 0x74, 0x65, 0x53, 0x74, 0x2e,
0x20, 0x42, 0x61, 0x61, 0x74, 0x69, 0x6c, 0x65, 0x6d, 0x69, 0x53, 0x77,
0x69, 0x66, 0x74, 0x20, 0x4b, 0x25b, 0x72, 0x25b, 0x6e, 0x74, 0x54, 0x65,
0x67, 0x75, 0x73, 0x69, 0x67, 0x61, 0x6c, 0x70, 0x61, 0x54, 0x69, 0x64,
0x77, 0x75, 0x61, 0x6e, 0x61, 0x54, 0x254, 0x254, 0x74, 0x6f, 0x6c, 0x61,
0x57, 0x69, 0x6e, 0x69, 0x70, 0x25b, 0x67, 0x4b, 0x61, 0x73, 0x69, 0x4d,
0x61, 0x6b, 0x61, 0x61, 0x72, 0x69, 0x4d, 0x254, 0x254, 0x73, 0x6f, 0x6e,
0x50, 0x61, 0x61, 0x6d, 0x61, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x54,
0x72, 0x254, 0x254, 0x56, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x41, 0x61, 0x6d,
0x61, 0x74, 0x69, 0x41, 0x6d, 0x61, 0x6e, 0x41, 0x6b, 0x74, 0x61, 0x75,
0x41, 0x6b, 0x74, 0x6f, 0x70, 0x25b, 0x42, 0x61, 0x6e, 0x6b, 0x254, 0x6b,
0x42, 0x25b, 0x25b, 0x72, 0x75, 0x74, 0x4b, 0x79, 0x69, 0x74, 0x61, 0x4b,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x6f, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b,
0x254, 0x73, 0x6f, 0x48, 0x25b, 0x62, 0x72, 0x254, 0x6e, 0x48, 0x6f, 0x20,
0x4b, 0x79, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x47, 0x79, 0x61, 0x6b, 0x61,
0x61, 0x74, 0x61, 0x47, 0x79, 0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61,
0x59, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x6d,
0x6b, 0x79, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x79,
0x69, 0x4b, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0x64, 0x75, 0x4b, 0x254, 0x254,
0x6b, 0x61, 0x74, 0x61, 0x4b, 0x75, 0x6b, 0x79, 0x69, 0x6e, 0x4d, 0x61,
0x6b, 0x61, 0x73, 0x61, 0x4e, 0x69, 0x6b, 0x6f, 0x73, 0x69, 0x61, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x254, 0x54, 0x25b, 0x25b, 0x72, 0x61, 0x6e,
0x54, 0x69, 0x6d, 0x70, 0x68, 0x75, 0x59, 0x75, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x61, 0x59, 0x75, 0x72, 0x79, 0x6d, 0x6b, 0x69, 0x56, 0x69,
0x65, 0x6e, 0x68, 0x79, 0x69, 0x61, 0x6e, 0x65, 0x59, 0x25b, 0x6b, 0x61,
0x74, 0x25b, 0x72, 0x69, 0x6e, 0x62, 0x25b, 0x67, 0x42, 0x25b, 0x6d, 0x75,
0x64, 0x61, 0x4b, 0x65, 0x70, 0x76, 0x25b, 0x64, 0x65, 0x52, 0x25b, 0x25b,
0x6b, 0x67, 0x79, 0x61, 0x76, 0x69, 0x6b, 0x47, 0x79, 0x254, 0x254, 0x67,
0x79, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x61, 0x66, 0x6f, 0x254, 0x53,
0x74, 0x61, 0x6e, 0x6c, 0x69, 0x42, 0x72, 0x254, 0x6b, 0x65, 0x6e, 0x20,
0x48, 0x69, 0x6c, 0x6c, 0x44, 0x61, 0x61, 0x77, 0x69, 0x6e, 0x45, 0x75,
0x6b, 0x6c, 0x61, 0x48, 0x254, 0x62, 0x61, 0x74, 0x4c, 0x254, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x4d, 0x25b, 0x25b, 0x62, 0x254, 0x6e, 0x50, 0x25b,
0x25b, 0x74, 0x53, 0x69, 0x64, 0x6e, 0x69, 0x41, 0x6d, 0x73, 0x74, 0x61,
0x64, 0x61, 0x6d, 0x41, 0x6e, 0x64, 0x254, 0x72, 0x61, 0x41, 0x74, 0x65,
0x6e, 0x65, 0x42, 0x25b, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x65, 0x42, 0x25b,
0x25b, 0x6c, 0x69, 0x6e, 0x42, 0x72, 0x25b, 0x73, 0x25b, 0x6c, 0x73, 0x65,
0x42, 0x75, 0x6b, 0x79, 0x61, 0x72, 0x25b, 0x73, 0x74, 0x42, 0x75, 0x64,
0x61, 0x70, 0x25b, 0x73, 0x68, 0x42, 0x75, 0x73, 0x69, 0x6e, 0x67, 0x79,
0x65, 0x4b, 0x79, 0x69, 0x73, 0x69, 0x6e, 0x61, 0x75, 0x4b, 0x254, 0x70,
0x25b, 0x68, 0x61, 0x6e, 0x67, 0x25b, 0x6e, 0x65, 0x44, 0x254, 0x62, 0x6c,
0x69, 0x6e, 0x47, 0x79, 0x65, 0x62, 0x72, 0x6f, 0x74, 0x61, 0x48, 0x25b,
0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x4a, 0x25b, 0x25b, 0x73, 0x69, 0x4c,
0x69, 0x73, 0x62, 0x254, 0x6e, 0x4c, 0x64, 0x77, 0x75, 0x62, 0x64, 0x77,
0x61, 0x6e, 0x61, 0x4c, 0x254, 0x6e, 0x64, 0x254, 0x6e, 0x4c, 0x25b, 0x73,
0x65, 0x6d, 0x62, 0x25b, 0x67, 0x4d, 0x254, 0x254, 0x74, 0x61, 0x4d, 0x6f,
0x6e, 0x61, 0x6b, 0x6f, 0x4d, 0x254, 0x73, 0x6b, 0x6f, 0x50, 0x6f, 0x64,
0x67, 0x6f, 0x72, 0x69, 0x6b, 0x61, 0x52, 0x6f, 0x6d, 0x61, 0x53, 0x61,
0x72, 0x61, 0x67, 0x79, 0x65, 0x76, 0x6f, 0x53, 0x6b, 0x254, 0x70, 0x67,
0x79, 0x65, 0x53, 0x254, 0x66, 0x69, 0x61, 0x53, 0x74, 0x254, 0x6b, 0x68,
0x254, 0x6d, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x56, 0x65, 0x25b,
0x6e, 0x61, 0x57, 0x254, 0x254, 0x73, 0x254, 0x254, 0x5a, 0x75, 0x72, 0x65,
0x6b, 0x79, 0x65, 0x4b, 0x79, 0x61, 0x67, 0x254, 0x73, 0x42, 0x75, 0x72,
0x6f, 0x6e, 0x79, 0x61, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x4b, 0x254,
0x6d, 0x254, 0x72, 0x254, 0x4b, 0x25b, 0x67, 0x75, 0x65, 0x6c, 0x25b, 0x6e,
0x4d, 0x254, 0x72, 0x69, 0x68, 0x79, 0x69, 0x254, 0x73, 0x6f, 0x4d, 0x61,
0x79, 0x6f, 0x74, 0x65, 0x41, 0x75, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4b,
0x79, 0x61, 0x74, 0x61, 0x6d, 0x4b, 0x79, 0x75, 0x75, 0x6b, 0x45, 0x61,
0x73, 0x74, 0x61, 0x46, 0x69, 0x67, 0x79, 0x69, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x254, 0x73, 0x47, 0x75, 0x61, 0x64, 0x61, 0x61, 0x6b,
0x61, 0x6e, 0x61, 0x61, 0x4b, 0x77, 0x61, 0x67, 0x79, 0x61, 0x6c, 0x65,
0x65, 0x6e, 0x65, 0x4d, 0x61, 0x67, 0x79, 0x75, 0x72, 0x6f, 0x4d, 0x61,
0x61, 0x6b, 0x65, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x254, 0x254, 0x66, 0x254,
0x6b, 0x50, 0x69, 0x74, 0x6b, 0x61, 0x69, 0x72, 0x6e, 0x65, 0x50, 0x254,
0x6e, 0x70, 0x65, 0x69, 0x50, 0x254, 0x74, 0x20, 0x4d, 0x6f, 0x72, 0x25b,
0x73, 0x62, 0x69, 0x41, 0x62, 0x69, 0x78, 0x68, 0x61, 0x6e, 0x41, 0x64,
0x69, 0x73, 0x2d, 0x41, 0x62, 0x61, 0x62, 0xeb, 0x41, 0x6c, 0x67, 0x6a,
0x65, 0x72, 0x41, 0x73, 0x6d, 0x61, 0x72, 0xeb, 0x42, 0x61, 0x6e, 0x78,
0x68, 0x75, 0x6c, 0x42, 0x69, 0x73, 0x61, 0x75, 0x42, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x72, 0xeb, 0x42, 0x72, 0x61, 0x7a, 0x61, 0x76, 0x69, 0x6c,
0x6c, 0xeb, 0x42, 0x75, 0x78, 0x68, 0x75, 0x6d, 0x62, 0x75, 0x72, 0xeb,
0x4b, 0x61, 0x6a, 0x72, 0x6f, 0x4b, 0x61, 0x7a, 0x61, 0x62, 0x6c, 0x61,
0x6e, 0x6b, 0xeb, 0x54, 0x68, 0x65, 0x75, 0x74, 0x61, 0x4b, 0x6f, 0x6e,
0x61, 0x6b, 0x72, 0x69, 0x44, 0x61, 0x72, 0x2d, 0x65, 0x73, 0x2d, 0x53,
0x61, 0x6c, 0x61, 0x6d, 0x58, 0x68, 0x69, 0x62, 0x75, 0x74, 0x69, 0x45,
0x6c, 0x20, 0x41, 0x6a, 0x75, 0x6e, 0x46, 0x72, 0x69, 0x74, 0x61, 0x75,
0x6e, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67,
0x58, 0x68, 0x75, 0x62, 0x61, 0x4b, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x4c,
0x69, 0x62, 0x72, 0x65, 0x76, 0x69, 0x6c, 0xeb, 0x4e, 0x69, 0x61, 0x6d,
0x65, 0x6a, 0x4e, 0x6f, 0x75, 0x61, 0x6b, 0x6f, 0x74, 0x55, 0x61, 0x67,
0xeb, 0x64, 0x75, 0x67, 0x75, 0x53, 0x61, 0x6f, 0x2d, 0x54, 0x6f, 0x6d,
0x65, 0x54, 0x75, 0x6e, 0x69, 0x7a, 0x56, 0x69, 0x6e, 0x74, 0x2019, 0x68,
0x75, 0x6b, 0x41, 0x6e, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x78, 0x68, 0x41,
0x6e, 0x67, 0x75, 0x69, 0x6c, 0xeb, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61,
0x6a, 0x61, 0x6e, 0x61, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x2d, 0x41,
0x6a, 0x72, 0x65, 0x73, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x72, 0x6b,
0x61, 0x4b, 0x6f, 0x72, 0x64, 0x6f, 0x62, 0x61, 0x48, 0x75, 0x68, 0x75,
0x69, 0x4c, 0x61, 0x20, 0x52, 0x69, 0x6f, 0x68, 0x61, 0x52, 0x69, 0x6f,
0x2d, 0x47, 0x61, 0x6c, 0x65, 0x67, 0x6f, 0x73, 0x53, 0x61, 0x6c, 0x74,
0xeb, 0x53, 0x61, 0x6e, 0x2d, 0x48, 0x75, 0x61, 0x6e, 0x53, 0x68, 0xeb,
0x6e, 0x2d, 0x4c, 0x75, 0x69, 0x73, 0x55, 0x73, 0x68, 0x75, 0x61, 0x6a,
0x61, 0x41, 0x72, 0x75, 0x62, 0xeb, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x69,
0x6f, 0x6e, 0x42, 0x61, 0x68, 0x69, 0x61, 0x2d, 0x42, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0xeb, 0x42, 0x6c,
0x61, 0x6e, 0x6b, 0x2d, 0x53, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x42, 0x6f,
0x61, 0x2d, 0x56, 0x69, 0x73, 0x74, 0x61, 0x42, 0x6f, 0x67, 0x6f, 0x74,
0xeb, 0x42, 0x6f, 0x69, 0x7a, 0xeb, 0x47, 0x6a, 0x69, 0x72, 0x69, 0x20,
0x69, 0x20, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x78, 0x68, 0x69, 0x74,
0x4b, 0x61, 0x6d, 0x70, 0x6f, 0x2d, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x65,
0x4b, 0x61, 0x6e, 0x6b, 0x75, 0x6e, 0x4b, 0x61, 0x6a, 0x65, 0x6e, 0xeb,
0x4b, 0x61, 0x6a, 0x6d, 0x61, 0x6e, 0xc7, 0x69, 0x6b, 0x61, 0x67, 0x6f,
0xc7, 0x69, 0x68, 0x61, 0x68, 0x75, 0x61, 0x53, 0x69, 0x75, 0x64, 0x61,
0x64, 0x2d, 0x48, 0x75, 0x61, 0x72, 0x65, 0x7a, 0x4b, 0x6f, 0x73, 0x74,
0x61, 0x2d, 0x52, 0x69, 0x6b, 0xeb, 0x4b, 0x72, 0x65, 0x73, 0x74, 0x6f,
0x6e, 0x4b, 0x75, 0x6a, 0x61, 0x62, 0x61, 0x4b, 0x75, 0x72, 0x61, 0x73,
0x61, 0x6f, 0x44, 0x65, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68, 0x61,
0x76, 0xeb, 0x6e, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x47, 0x6a, 0x69,
0x72, 0x69, 0x20, 0x69, 0x20, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x69,
0x74, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0xeb, 0x45, 0x6a, 0x72,
0x75, 0x6e, 0x65, 0x70, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72,
0x46, 0x6f, 0x72, 0x74, 0x2d, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x47,
0x6a, 0x69, 0x72, 0x69, 0x20, 0x69, 0x20, 0x4e, 0x67, 0x72, 0x69, 0x72,
0xeb, 0x47, 0x6a, 0x69, 0x72, 0x69, 0x20, 0x69, 0x20, 0x50, 0x61, 0x74,
0xeb, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x75, 0x20, 0x69, 0x20, 0x4d, 0x61,
0x64, 0x68, 0x47, 0x72, 0x61, 0x6e, 0x61, 0x64, 0xeb, 0x47, 0x75, 0x61,
0x64, 0x65, 0x6c, 0x75, 0x70, 0xeb, 0x47, 0x75, 0x61, 0x74, 0x65, 0x6d,
0x61, 0x6c, 0xeb, 0x47, 0x75, 0x61, 0x6a, 0x61, 0x6b, 0x75, 0x69, 0x6c,
0x47, 0x75, 0x61, 0x6a, 0x61, 0x6e, 0xeb, 0x48, 0x61, 0x6c, 0x69, 0x66,
0x61, 0x6b, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6e, 0xeb, 0x48, 0x65, 0x72,
0x6d, 0x6f, 0x73, 0x69, 0x6a, 0x6f, 0x4b, 0x6e, 0x6f, 0x6b, 0x73, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50, 0x65, 0x74, 0xeb,
0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x6c, 0x2d, 0x53, 0x69, 0x74, 0x69,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x65, 0x76,
0xeb, 0x6a, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56,
0x69, 0x6e, 0x63, 0x65, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x55, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x58, 0x68, 0x61, 0x6d,
0x61, 0x6a, 0x6b, 0xeb, 0x58, 0x68, 0x75, 0x6e, 0x6f, 0x75, 0x4c, 0x75,
0x69, 0x7a, 0x76, 0x69, 0x6c, 0xeb, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0xe7,
0x65, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x61, 0x6b, 0x69,
0x4c, 0x69, 0x6d, 0xeb, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x78, 0x68,
0x65, 0x6c, 0x6f, 0x73, 0x4c, 0x6f, 0x75, 0x65, 0x72, 0x2d, 0x50, 0x72,
0x69, 0x6e, 0x73, 0x69, 0x73, 0x2d, 0x4b, 0x75, 0x6f, 0x72, 0x74, 0xeb,
0x72, 0x4d, 0x61, 0x6b, 0x65, 0x6a, 0x6f, 0x4d, 0x61, 0x72, 0x74, 0x69,
0x6e, 0x69, 0x6b, 0xeb, 0x4d, 0x65, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x69,
0x4d, 0x65, 0x72, 0x69, 0x64, 0x61, 0x51, 0x79, 0x74, 0x65, 0x74, 0x69,
0x20, 0x69, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0xeb, 0x73, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4d, 0x6f, 0x6e, 0x6b, 0x74, 0x6f,
0x6e, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x72, 0x65, 0x6a, 0x4d, 0x6f,
0x6e, 0x74, 0x73, 0x65, 0x72, 0x61, 0x74, 0x4e, 0x61, 0x73, 0x61, 0x6f,
0x4e, 0x6a, 0x75, 0x2d, 0x4a, 0x6f, 0x72, 0x6b, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x6a, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x65, 0x20, 0x56, 0x65, 0x72, 0x69, 0x75,
0x74, 0x51, 0x65, 0x6e, 0x64, 0xeb, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x65, 0x20, 0x56, 0x65, 0x72, 0x69, 0x75, 0x74,
0x4e, 0x6a, 0x75, 0x2d, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x65, 0x20, 0x56, 0x65, 0x72, 0x69,
0x75, 0x74, 0x4e, 0x6a, 0x75, 0x6b, 0x4f, 0x68, 0x69, 0x6e, 0x61, 0x67,
0x61, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x2d,
0x6f, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x6f,
0x2d, 0x56, 0x65, 0x6c, 0x68, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d,
0x52, 0x69, 0x6b, 0x6f, 0x50, 0x75, 0x6e, 0x74, 0x61, 0x2d, 0x41, 0x72,
0x65, 0x6e, 0x61, 0x73, 0x47, 0x72, 0x79, 0x6b, 0x61, 0x20, 0x49, 0x6e,
0x6c, 0x65, 0x74, 0x52, 0x65, 0x78, 0x68, 0x69, 0x6e, 0x61, 0x52, 0x69,
0x6f, 0x2d, 0x42, 0x72, 0x61, 0x6e, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x74,
0x6f, 0x2d, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53, 0x61, 0x6f,
0x2d, 0x50, 0x61, 0x75, 0x6c, 0x6f, 0x49, 0x74, 0x6f, 0x6b, 0x6f, 0x72,
0x74, 0x75, 0x72, 0x6d, 0x69, 0x74, 0x53, 0x65, 0x6e, 0x2d, 0x42, 0x61,
0x72, 0x74, 0x65, 0x6c, 0x65, 0x6d, 0x69, 0x53, 0x68, 0xeb, 0x6e, 0x2d,
0x47, 0x6a, 0x6f, 0x6e, 0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x4b, 0x69, 0x74,
0x73, 0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x4c, 0x75, 0xe7, 0x69, 0x61, 0x53,
0x68, 0xeb, 0x6e, 0x2d, 0x54, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x68, 0xeb,
0x6e, 0x2d, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x52, 0x72, 0x79,
0x6d, 0x61, 0x20, 0x65, 0x20, 0x53, 0x68, 0x70, 0x65, 0x6a, 0x74, 0xeb,
0x44, 0x68, 0x75, 0x6c, 0x65, 0x54, 0x69, 0x68, 0x75, 0x61, 0x6e, 0x61,
0x54, 0x6f, 0x72, 0x74, 0x6f, 0x6c, 0xeb, 0x56, 0x61, 0x6e, 0x6b, 0x75,
0x76, 0x65, 0x72, 0x55, 0x61, 0x6a, 0x74, 0x2019, 0x68, 0x6f, 0x72, 0x73,
0x55, 0x69, 0x6e, 0x69, 0x70, 0x65, 0x67, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x61, 0x74, 0x4b, 0x65, 0x6a, 0x73, 0x69, 0x44, 0x65, 0x6a, 0x76, 0x69,
0x73, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0xeb, 0x4d, 0x65, 0x6b, 0x75, 0x61, 0x72, 0x69, 0x4d,
0x61, 0x75, 0x73, 0x6f, 0x6e, 0x4d, 0x65, 0x6b, 0x6d, 0x75, 0x72, 0x64,
0x6f, 0x52, 0x6f, 0x64, 0x68, 0x65, 0x72, 0xeb, 0x53, 0x6a, 0x6f, 0x75,
0x61, 0x4c, 0x6f, 0x6e, 0x67, 0x2019, 0x6a, 0xeb, 0x62, 0x6a, 0x65, 0x6e,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72,
0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x41, 0x74, 0x69, 0x72, 0x61, 0x75,
0x42, 0x61, 0x68, 0x72, 0x65, 0x6a, 0x6e, 0x42, 0x65, 0x6a, 0x72, 0x75,
0x74, 0xc7, 0x69, 0x74, 0x61, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x75, 0x73,
0x74, 0xeb, 0x48, 0x6f, 0x2d, 0xc7, 0x69, 0x2d, 0x4d, 0x69, 0x6e, 0x48,
0x6f, 0x6e, 0x67, 0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x58, 0x68, 0x61, 0x6b,
0x61, 0x72, 0x74, 0xeb, 0x58, 0x68, 0x61, 0x6a, 0x61, 0x70, 0x75, 0x72,
0xeb, 0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x6b, 0xeb, 0x4b, 0x61, 0x72,
0x61, 0xe7, 0x69, 0x4b, 0x61, 0x6e, 0x64, 0x69, 0x67, 0xeb, 0x4b, 0x61,
0x6c, 0x6b, 0x75, 0x74, 0xeb, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x2d, 0x4c,
0x75, 0x6d, 0x70, 0x75, 0x72, 0x4b, 0x75, 0xe7, 0x69, 0x6e, 0x67, 0x4b,
0x75, 0x76, 0x61, 0x6a, 0x74, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x4d, 0x61,
0x6b, 0x61, 0x73, 0x61, 0x72, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0xeb, 0x4e,
0x69, 0x6b, 0x6f, 0x7a, 0x69, 0x61, 0x50, 0x6e, 0x6f, 0x6d, 0x2d, 0x50,
0x65, 0x6e, 0x50, 0x65, 0x6e, 0x69, 0x61, 0x6e, 0x4b, 0x6f, 0x73, 0x74,
0x61, 0x6e, 0x61, 0x6a, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x52, 0x69, 0x61, 0x64, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e,
0x53, 0x65, 0x75, 0x6c, 0x53, 0x68, 0x61, 0x6e, 0x67, 0x61, 0x69, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x53, 0x72, 0x65, 0x64, 0x6e,
0x65, 0x6b, 0x6f, 0x6c, 0x69, 0x6d, 0x73, 0x6b, 0x54, 0x61, 0x6a, 0x70,
0x65, 0x6a, 0x54, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x54, 0x65, 0x68, 0x65,
0x72, 0x61, 0x6e, 0x54, 0x68, 0x69, 0x6d, 0x70, 0x75, 0x55, 0x6c, 0x61,
0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x56, 0x6a, 0x65, 0x6e, 0x74, 0x69,
0x61, 0x6e, 0xeb, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x52, 0x61,
0x6e, 0x67, 0x75, 0x6e, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x4a, 0x65, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x65, 0x4b,
0x65, 0x70, 0x69, 0x20, 0x69, 0x20, 0x47, 0x6a, 0x65, 0x6c, 0x62, 0xeb,
0x72, 0x52, 0x65, 0x69, 0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x58, 0x68,
0x6f, 0x72, 0x78, 0x68, 0x61, 0x20, 0x65, 0x20, 0x4a, 0x75, 0x67, 0x75,
0x74, 0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x45, 0x6c, 0x65, 0x6e, 0xeb, 0x53,
0x74, 0x65, 0x6e, 0x6c, 0x69, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x6a, 0x64,
0x65, 0x42, 0x72, 0x69, 0x73, 0x62, 0x65, 0x6a, 0x6e, 0x42, 0x72, 0x6f,
0x6b, 0xeb, 0x6e, 0x2d, 0x48, 0x69, 0x6c, 0x6c, 0x44, 0x61, 0x72, 0x76,
0x69, 0x6e, 0x4c, 0x69, 0x6e, 0x64, 0xeb, 0x6d, 0x65, 0x6e, 0x4c, 0x6f,
0x72, 0x64, 0x2d, 0x48, 0x6f, 0x75, 0x69, 0x4d, 0x65, 0x6c, 0x62, 0x75,
0x72, 0x6e, 0x50, 0xeb, 0x72, 0x74, 0x68, 0x53, 0x69, 0x64, 0x6e, 0x65,
0x6a, 0x41, 0x6e, 0x64, 0x6f, 0x72, 0x72, 0xeb, 0x41, 0x73, 0x74, 0x72,
0x61, 0x6b, 0x61, 0x6e, 0x41, 0x74, 0x68, 0x69, 0x6e, 0xeb, 0x42, 0x65,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c,
0x61, 0x76, 0xeb, 0x42, 0x72, 0x75, 0x6b, 0x73, 0x65, 0x6c, 0x42, 0x75,
0x6b, 0x75, 0x72, 0x65, 0x73, 0x68, 0x74, 0x4b, 0x69, 0x73, 0x68, 0x69,
0x6e, 0x65, 0x75, 0x47, 0x6a, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61,
0x72, 0x47, 0x65, 0x72, 0x6e, 0x73, 0x65, 0x6a, 0x49, 0x73, 0x68, 0x75,
0x6c, 0x6c, 0x69, 0x20, 0x69, 0x20, 0x4d, 0x61, 0x6e, 0x69, 0x74, 0x53,
0x74, 0x61, 0x6d, 0x62, 0x6f, 0x6c, 0x6c, 0x58, 0x68, 0x65, 0x72, 0x73,
0x65, 0x6a, 0x4b, 0x69, 0x65, 0x76, 0x4c, 0x69, 0x73, 0x62, 0x6f, 0x6e,
0xeb, 0x4c, 0x75, 0x62, 0x6a, 0x61, 0x6e, 0xeb, 0x4c, 0x6f, 0x6e, 0x64,
0xeb, 0x72, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67,
0x4d, 0x61, 0x6c, 0x74, 0xeb, 0x4d, 0x6f, 0x73, 0x6b, 0xeb, 0x50, 0x6f,
0x64, 0x67, 0x6f, 0x72, 0x69, 0x63, 0xeb, 0x50, 0x72, 0x61, 0x67, 0xeb,
0x52, 0x69, 0x67, 0xeb, 0x52, 0x6f, 0x6d, 0xeb, 0x53, 0x61, 0x6e, 0x2d,
0x4d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65,
0x76, 0xeb, 0x53, 0x68, 0x6b, 0x75, 0x70, 0x53, 0x6f, 0x66, 0x6a, 0x65,
0x53, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x54, 0x61, 0x6c, 0x69,
0x6e, 0x54, 0x69, 0x72, 0x61, 0x6e, 0xeb, 0x55, 0x6c, 0x6a, 0x61, 0x6e,
0x6f, 0x76, 0x73, 0x6b, 0x56, 0x6a, 0x65, 0x6e, 0xeb, 0x56, 0x61, 0x72,
0x73, 0x68, 0x61, 0x76, 0xeb, 0x5a, 0x79, 0x72, 0x69, 0x68, 0xc7, 0x61,
0x67, 0x6f, 0x73, 0x4b, 0x72, 0x69, 0x73, 0x68, 0x74, 0x6c, 0x69, 0x6e,
0x64, 0x6a, 0x65, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x65, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x65, 0x4d, 0x61, 0x6a, 0x6f, 0x74, 0xeb, 0x4f, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x42, 0x75, 0x6e, 0x67, 0x61, 0x6e, 0x76, 0x69,
0x6c, 0xeb, 0xc7, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0xc7, 0x75, 0x6b, 0x50,
0x61, 0x73, 0x68, 0x6b, 0xeb, 0x46, 0x69, 0x78, 0x68, 0x69, 0x47, 0x75,
0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x4b, 0x69, 0x72,
0x69, 0x74, 0x69, 0x6d, 0x61, 0x74, 0x4b, 0x6f, 0x73, 0x72, 0x65, 0x4b,
0x75, 0x61, 0x78, 0x68, 0x61, 0x6c, 0x65, 0x69, 0x6e, 0x4d, 0x61, 0x68,
0x75, 0x72, 0x6f, 0x4d, 0x61, 0x72, 0x6b, 0x65, 0x7a, 0x4d, 0x69, 0x64,
0x75, 0x65, 0x6a, 0x50, 0x61, 0x67, 0x6f, 0x2d, 0x50, 0x61, 0x67, 0x6f,
0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x50, 0x6f, 0x6e, 0x61, 0x70,
0x65, 0x69, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x4d, 0x6f, 0x72, 0x65, 0x73,
0x62, 0x69, 0x52, 0x61, 0x72, 0x6f, 0x74, 0x6f, 0x6e, 0x67, 0xeb, 0x54,
0x61, 0x72, 0x61, 0x76, 0xeb, 0x55, 0x65, 0x6a, 0x6b, 0x55, 0x6f, 0x6c,
0x6c, 0x69, 0x73, 0x12a0, 0x1262, 0x1303, 0x1295, 0x12a0, 0x12ad, 0x122b, 0x12a0, 0x12f2,
0x1235, 0x20, 0x12a0, 0x1260, 0x1263, 0x12a0, 0x120d, 0x1300, 0x122d, 0x1235, 0x12a0, 0x1235,
0x1218, 0x122b, 0x1263, 0x121b, 0x12ae, 0x1263, 0x1295, 0x1309, 0x12a2, 0x1263, 0x1295, 0x1301,
0x120d, 0x1262, 0x1233, 0x12a6, 0x1265, 0x120b, 0x1295, 0x1273, 0x12e8, 0x122d, 0x1265, 0x122b,
0x12db, 0x126a, 0x120d, 0x1261, 0x1301, 0x121d, 0x1265, 0x122b, 0x12ab, 0x12ed, 0x122e, 0x12ab,
0x12db, 0x1265, 0x120b, 0x1295, 0x12ab, 0x1232, 0x12a1, 0x1273, 0x12ae, 0x1293, 0x12ad, 0x122c,
0x12f3, 0x12ab, 0x122d, 0x12f3, 0x122c, 0x20, 0x1230, 0x120b, 0x121d, 0x1305, 0x1261, 0x1272,
0x12f1, 0x12a0, 0x120b, 0x12a4, 0x120d, 0x20, 0x12a0, 0x12e9, 0x12a0, 0x1295, 0x134d, 0x122a,
0x1273, 0x12cd, 0x1295, 0x130b, 0x1266, 0x122e, 0x1295, 0x1203, 0x122b, 0x122c, 0x1306, 0x1203,
0x1295, 0x1235, 0x1260, 0x122d, 0x130d, 0x1301, 0x1263, 0x12ab, 0x121d, 0x1353, 0x120b, 0x12ab,
0x122d, 0x1271, 0x121d, 0x12aa, 0x130b, 0x120a, 0x12aa, 0x1295, 0x123b, 0x1233, 0x120c, 0x130e,
0x1235, 0x120a, 0x1260, 0x122d, 0x126a, 0x120d, 0x120e, 0x121c, 0x1209, 0x12a0, 0x1295, 0x12f3,
0x1209, 0x1261, 0x121d, 0x1263, 0x123a, 0x1209, 0x1233, 0x12ab, 0x121b, 0x120b, 0x1261, 0x121b,
0x1351, 0x1271, 0x121b, 0x1234, 0x1229, 0x121d, 0x1263, 0x1263, 0x1294, 0x121e, 0x1243, 0x12f2,
0x1239, 0x121e, 0x1295, 0x122e, 0x126a, 0x12eb, 0x1293, 0x12ed, 0x122e, 0x1262, 0x1295, 0x1303,
0x121c, 0x1293, 0x1292, 0x12eb, 0x121c, 0x12ed, 0x1291, 0x12a0, 0x12ad, 0x127e, 0x1275, 0x12a1,
0x130b, 0x12f1, 0x1309, 0x1356, 0x122d, 0x1276, 0x20, 0x2d, 0x20, 0x1296, 0x126e, 0x1233,
0x12a6, 0x20, 0x1276, 0x121c, 0x1275, 0x122a, 0x1356, 0x120a, 0x1271, 0x1292, 0x12dd, 0x12ca,
0x1295, 0x12f5, 0x1206, 0x12ad, 0x12a0, 0x12f3, 0x12ad, 0x12a0, 0x1295, 0x12ae, 0x122b, 0x1305,
0x12a0, 0x1295, 0x1309, 0x12ed, 0x120b, 0x12a0, 0x1295, 0x1272, 0x1313, 0x12a0, 0x122b, 0x1309,
0x12e8, 0x1293, 0x1266, 0x1290, 0x1235, 0x20, 0x12a0, 0x12ed, 0x1228, 0x1235, 0x12ab, 0x1273,
0x121b, 0x122d, 0x12ab, 0x12ae, 0x122d, 0x12f6, 0x1263, 0x1301, 0x1301, 0x12ed, 0x120b, 0x20,
0x122a, 0x12a6, 0x1303, 0x121c, 0x1295, 0x12f6, 0x12db, 0x122a, 0x12ee, 0x20, 0x130b, 0x120c,
0x130e, 0x1235, 0x1233, 0x120d, 0x1273, 0x1233, 0x1295, 0x20, 0x1301, 0x12a0, 0x1295, 0x1233,
0x1295, 0x20, 0x120a, 0x12cd, 0x1235, 0x1271, 0x12a9, 0x121b, 0x1295, 0x12a1, 0x1239, 0x12a0,
0x12a2, 0x12a0, 0x1229, 0x1263, 0x12a0, 0x1231, 0x1295, 0x1232, 0x12ee, 0x1295, 0x12a0, 0x1272,
0x12ae, 0x12ab, 0x1295, 0x1263, 0x1202, 0x12a0, 0x1263, 0x1202, 0x12eb, 0x20, 0x1263, 0x1295,
0x12f0, 0x122b, 0x1235, 0x1263, 0x122d, 0x1264, 0x12f6, 0x1235, 0x1264, 0x1208, 0x121d, 0x1264,
0x120a, 0x12dd, 0x1265, 0x120b, 0x1295, 0x12ad, 0x2d, 0x20, 0x1233, 0x1265, 0x120e, 0x1295,
0x1266, 0x12a0, 0x20, 0x126a, 0x1235, 0x1273, 0x1266, 0x130e, 0x1273, 0x1266, 0x12ed, 0x12dd,
0x12ab, 0x121d, 0x1265, 0x122a, 0x1305, 0x20, 0x1264, 0x12ed, 0x12ab, 0x121d, 0x1356, 0x20,
0x130d, 0x122b, 0x1295, 0x12f4, 0x12ab, 0x1295, 0x12a9, 0x1295, 0x12ab, 0x122b, 0x12ab, 0x1235,
0x12ab, 0x12ed, 0x1295, 0x12a4, 0x12ab, 0x12ed, 0x121b, 0x1295, 0x127a, 0x12ab, 0x130e, 0x127a,
0x1201, 0x12cb, 0x12cd, 0x12a0, 0x1233, 0x12ee, 0x12f3, 0x12f5, 0x20, 0x1201, 0x12cb, 0x122c,
0x12dd, 0x12ae, 0x1235, 0x1273, 0x122a, 0x12ab, 0x12ad, 0x1228, 0x1235, 0x1270, 0x1295, 0x12a9,
0x12e8, 0x12a0, 0x1263, 0x12a9, 0x122b, 0x1233, 0x12ce, 0x12f3, 0x1295, 0x121b, 0x122d, 0x12ad,
0x123b, 0x126d, 0x1295, 0x12f3, 0x12cd, 0x1230, 0x1295, 0x12f3, 0x12cd, 0x1230, 0x1295, 0x20,
0x12ad, 0x122c, 0x12ad, 0x12f4, 0x1295, 0x1268, 0x122d, 0x12f2, 0x1275, 0x122e, 0x12ed, 0x1275,
0x12f6, 0x121c, 0x1292, 0x12ab, 0x12a4, 0x12f5, 0x1218, 0x1295, 0x1270, 0x1295, 0x12a2, 0x1229,
0x1294, 0x1355, 0x12a4, 0x120d, 0x1233, 0x120d, 0x126b, 0x12f6, 0x122d, 0x134e, 0x122d, 0x1275,
0x20, 0x1294, 0x120d, 0x1230, 0x1295, 0x134e, 0x122d, 0x1273, 0x120c, 0x12db, 0x130d, 0x120c,
0x1235, 0x20, 0x1264, 0x12ed, 0x1309, 0x12dd, 0x20, 0x1264, 0x12ed, 0x130d, 0x122b, 0x1295,
0x12f5, 0x20, 0x1270, 0x122d, 0x12ad, 0x130d, 0x122c, 0x1293, 0x12f3, 0x1315, 0x12f3, 0x1209,
0x1355, 0x1313, 0x1272, 0x121b, 0x120b, 0x1309, 0x12eb, 0x12a9, 0x12ed, 0x120d, 0x1309, 0x12eb,
0x1293, 0x1203, 0x120a, 0x134b, 0x12ad, 0x1235, 0x1203, 0x126b, 0x1293, 0x12b8, 0x122d, 0x121e,
0x12da, 0x120e, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x1356, 0x120a, 0x1235, 0x1296, 0x12ad,
0x1235, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x121b, 0x122c, 0x1295, 0x130e,
0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x1352, 0x1270, 0x122d, 0x1235, 0x1260,
0x122d, 0x130d, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x1274, 0x120d, 0x20,
0x12a8, 0x1270, 0x121b, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x126a, 0x126b,
0x12ed, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x126a, 0x1295, 0x127c, 0x1295,
0x1235, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x12ca, 0x1293, 0x121b, 0x12ad,
0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293, 0x12a2, 0x1291, 0x126a, 0x12ad, 0x12a2,
0x12b3, 0x120a, 0x12cd, 0x1275, 0x1303, 0x121b, 0x12ed, 0x12ab, 0x1301, 0x1292, 0x12e9, 0x120a,
0x12cd, 0x1235, 0x126a, 0x120d, 0x121e, 0x1295, 0x1272, 0x1234, 0x120e, 0x2c, 0x20, 0x12aa,
0x1295, 0x1273, 0x12aa, 0x12ad, 0x122b, 0x1208, 0x1295, 0x12f2, 0x12ed, 0x12ad, 0x120b, 0x20,
0x1353, 0x12dd, 0x120a, 0x121b, 0x120e, 0x1235, 0x20, 0x12a0, 0x1295, 0x1300, 0x1208, 0x1235,
0x12e8, 0x1273, 0x127d, 0x129b, 0x12cd, 0x20, 0x120d, 0x12d1, 0x120d, 0x20, 0x1229, 0x1265,
0x121c, 0x1232, 0x12a6, 0x121b, 0x1293, 0x1309, 0x12a0, 0x121b, 0x1293, 0x12a1, 0x1235, 0x121b,
0x122d, 0x130e, 0x1275, 0x121b, 0x122d, 0x1272, 0x1292, 0x12ad, 0x121b, 0x1273, 0x121e, 0x122e,
0x1235, 0x121b, 0x12db, 0x1275, 0x120b, 0x1295, 0x121c, 0x1296, 0x121a, 0x1292, 0x121c, 0x122a,
0x12f3, 0x1218, 0x1275, 0x120b, 0x12ab, 0x1275, 0x120b, 0x121c, 0x12ad, 0x1232, 0x12ae, 0x20,
0x12a8, 0x1270, 0x121b, 0x121a, 0x12ae, 0x12cd, 0x120e, 0x1295, 0x121e, 0x1295, 0x12ad, 0x1276,
0x1295, 0x121e, 0x1295, 0x1270, 0x122d, 0x122c, 0x12ed, 0x121e, 0x1295, 0x1274, 0x126a, 0x12f5,
0x12ee, 0x121e, 0x1295, 0x1275, 0x1234, 0x122b, 0x1275, 0x1293, 0x1233, 0x12cd, 0x1292, 0x12cd,
0x12ee, 0x122d, 0x12ad, 0x1296, 0x121c, 0x1296, 0x122e, 0x129b, 0x1264, 0x12a1, 0x120b, 0x1205,
0x2c, 0x20, 0x1230, 0x121c, 0x1295, 0x20, 0x12f3, 0x12ae, 0x1273, 0x1218, 0x12ab, 0x12a8,
0x1208, 0x129b, 0x2c, 0x20, 0x1230, 0x121c, 0x1295, 0x20, 0x12f3, 0x12ae, 0x1273, 0x12a0,
0x12f2, 0x1235, 0x20, 0x1233, 0x120c, 0x121d, 0x2c, 0x20, 0x1230, 0x121c, 0x1295, 0x20,
0x12f3, 0x12ae, 0x1273, 0x130b, 0x12f5, 0x1273, 0x1265, 0x12a6, 0x12ea, 0x1293, 0x130b, 0x1353,
0x1293, 0x121b, 0x1353, 0x122b, 0x121b, 0x122a, 0x1266, 0x134a, 0x1292, 0x12ad, 0x1235, 0x1356,
0x122d, 0x1275, 0x20, 0x12a6, 0x20, 0x1355, 0x122a, 0x1295, 0x1235, 0x12e8, 0x12a5, 0x1235,
0x1354, 0x1295, 0x20, 0x12c8, 0x12f0, 0x1265, 0x1354, 0x1275, 0x122e, 0x20, 0x126c, 0x120d,
0x1206, 0x1356, 0x122d, 0x1276, 0x122a, 0x12ae, 0x1351, 0x1295, 0x1273, 0x20, 0x12a0, 0x122c,
0x1293, 0x1235, 0x122b, 0x1295, 0x12aa, 0x1295, 0x20, 0x12a2, 0x1295, 0x120c, 0x1275, 0x1228,
0x1232, 0x134d, 0x1228, 0x1302, 0x1293, 0x122a, 0x12de, 0x120a, 0x12e9, 0x1275, 0x122a, 0x12ee,
0x20, 0x1265, 0x122b, 0x1295, 0x12ae, 0x1233, 0x1295, 0x1273, 0x122c, 0x121d, 0x1233, 0x1295,
0x1272, 0x12eb, 0x130e, 0x1233, 0x1295, 0x1276, 0x20, 0x12f6, 0x121a, 0x1295, 0x130e, 0x1233,
0x12a6, 0x20, 0x1356, 0x120e, 0x1235, 0x12ae, 0x122d, 0x1235, 0x1263, 0x12ed, 0x1230, 0x1295,
0x12f5, 0x1232, 0x1275, 0x12ab, 0x1245, 0x12f5, 0x1235, 0x1275, 0x20, 0x1264, 0x122d, 0x1270,
0x120e, 0x121c, 0x1245, 0x12f1, 0x1235, 0x20, 0x12ee, 0x1210, 0x1295, 0x1235, 0x1245, 0x12f1,
0x1235, 0x20, 0x12aa, 0x1275, 0x1235, 0x1245, 0x12f5, 0x1235, 0x1275, 0x20, 0x1209, 0x1232,
0x12eb, 0x1245, 0x12f1, 0x1235, 0x20, 0x1276, 0x121b, 0x1235, 0x1245, 0x12f1, 0x1235, 0x20,
0x126a, 0x1295, 0x1230, 0x1295, 0x1275, 0x12e8, 0x1210, 0x12cb, 0x120b, 0x20, 0x1308, 0x1295,
0x12d8, 0x1265, 0x1274, 0x1309, 0x1232, 0x130b, 0x120d, 0x1353, 0x1271, 0x120c, 0x1272, 0x1301,
0x12a0, 0x1293, 0x1276, 0x122e, 0x1295, 0x1276, 0x1276, 0x122d, 0x1276, 0x120b, 0x126b, 0x1295,
0x12ae, 0x1268, 0x122d, 0x128b, 0x12ed, 0x1275, 0x1206, 0x122d, 0x1235, 0x12ca, 0x1292, 0x1354,
0x130d, 0x12eb, 0x12a9, 0x1273, 0x1275, 0x12ab, 0x12da, 0x12ed, 0x12f3, 0x126a, 0x1235, 0x12f0,
0x121e, 0x1295, 0x1275, 0x20, 0x12f2, 0x12a1, 0x122d, 0x126a, 0x120d, 0x121b, 0x12a8, 0x122a,
0x1293, 0x12cd, 0x1230, 0x1295, 0x121b, 0x12ad, 0x1218, 0x122d, 0x12f6, 0x1353, 0x120d, 0x1218,
0x122d, 0x122e, 0x1274, 0x122b, 0x1235, 0x12ee, 0x12cb, 0x1275, 0x122e, 0x120d, 0x126d, 0x1235,
0x1276, 0x12ad, 0x120e, 0x1295, 0x130d, 0x12ed, 0x122d, 0x1263, 0x12e8, 0x1295, 0x12a4, 0x12f0,
0x1295, 0x12a0, 0x120d, 0x121b, 0x1275, 0x12ed, 0x12a0, 0x121b, 0x1295, 0x12a0, 0x1293, 0x12f5,
0x12ed, 0x122d, 0x12a0, 0x12a9, 0x1273, 0x12a1, 0x12a0, 0x12a9, 0x1276, 0x1264, 0x12a0, 0x123d,
0x130b, 0x1263, 0x1275, 0x12a0, 0x1275, 0x12ed, 0x122b, 0x12a1, 0x1263, 0x130d, 0x12f3, 0x12f5,
0x1263, 0x1205, 0x122c, 0x1295, 0x1263, 0x1295, 0x12ae, 0x12ad, 0x1263, 0x122d, 0x1293, 0x12a1,
0x120d, 0x1264, 0x1229, 0x1275, 0x1262, 0x123d, 0x12ac, 0x12ad, 0x1265, 0x1229, 0x1293, 0x12ed,
0x127a, 0x1273, 0x12ae, 0x120e, 0x121d, 0x1266, 0x12f0, 0x121b, 0x1235, 0x1246, 0x12f2, 0x120a,
0x12f1, 0x1263, 0x12ed, 0x12f0, 0x123b, 0x1295, 0x1264, 0x134b, 0x121b, 0x1309, 0x1235, 0x1273,
0x130b, 0x12db, 0x12ac, 0x1265, 0x122e, 0x1295, 0x1206, 0x20, 0x127a, 0x20, 0x121a, 0x1295,
0x1205, 0x20, 0x12a8, 0x1270, 0x121b, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295, 0x130d,
0x1206, 0x126d, 0x12f5, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x1303, 0x12ab, 0x122d,
0x1273, 0x1303, 0x12eb, 0x1351, 0x122b, 0x12a5, 0x12e8, 0x1229, 0x1233, 0x120c, 0x121d, 0x12ab,
0x1261, 0x120d, 0x12ab, 0x121d, 0x127b, 0x1275, 0x12ab, 0x12ab, 0x122b, 0x127a, 0x12ab, 0x1275,
0x121b, 0x1295, 0x12f1, 0x12ab, 0x1295, 0x12f2, 0x130b, 0x12ae, 0x120d, 0x12ab, 0x1273, 0x12ad,
0x122b, 0x1235, 0x1296, 0x12eb, 0x122d, 0x1235, 0x12ad, 0x12a9, 0x12cb, 0x120b, 0x20, 0x120b,
0x121d, 0x1351, 0x122d, 0x12a9, 0x127a, 0x1295, 0x130d, 0x12a9, 0x12cc, 0x1275, 0x121b, 0x12ab,
0x12a1, 0x121b, 0x130b, 0x12f3, 0x1295, 0x121b, 0x12ab, 0x1233, 0x122d, 0x121b, 0x1292, 0x120b,
0x1219, 0x1235, 0x12ab, 0x1275, 0x1292, 0x12ae, 0x1232, 0x12a0, 0x1296, 0x126e, 0x12a9, 0x1275,
0x12dd, 0x1294, 0x12ad, 0x1296, 0x126e, 0x1232, 0x1262, 0x122a, 0x1235, 0x12ad, 0x12a6, 0x121d,
0x1235, 0x12ad, 0x12a6, 0x122b, 0x120d, 0x134d, 0x1296, 0x121d, 0x20, 0x1354, 0x1295, 0x1205,
0x1356, 0x1295, 0x1272, 0x12a0, 0x1293, 0x12ad, 0x1355, 0x12ee, 0x1295, 0x130d, 0x12eb, 0x1295,
0x130d, 0x12b3, 0x1273, 0x122d, 0x12ae, 0x1235, 0x1273, 0x1293, 0x12ed, 0x12a9, 0x12ed, 0x12da,
0x120e, 0x122d, 0x12f3, 0x122a, 0x12eb, 0x12f5, 0x1233, 0x12ad, 0x1203, 0x120a, 0x1295, 0x1233,
0x121b, 0x122d, 0x12ab, 0x1295, 0x12f5, 0x1234, 0x12a6, 0x120d, 0x123b, 0x1295, 0x130b, 0x12ed,
0x1232, 0x1295, 0x130b, 0x1356, 0x122d, 0x1235, 0x122c, 0x12f5, 0x1294, 0x1235, 0x12ae, 0x120d,
0x121d, 0x1235, 0x12ad, 0x1273, 0x12ed, 0x1353, 0x12ed, 0x1273, 0x123d, 0x12ac, 0x1295, 0x1275,
0x1275, 0x1265, 0x120a, 0x1232, 0x1274, 0x1205, 0x122b, 0x1295, 0x1272, 0x121d, 0x1349, 0x1276,
0x12aa, 0x12ee, 0x1276, 0x121d, 0x1235, 0x12ad, 0x12a1, 0x120b, 0x12a0, 0x1295, 0x1263, 0x12a0,
0x1273, 0x122d, 0x12a1, 0x1229, 0x121d, 0x12aa, 0x12a1, 0x1235, 0x1275, 0x2d, 0x1294, 0x122b,
0x126c, 0x1295, 0x1272, 0x12a0, 0x1295, 0x126d, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad,
0x12eb, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x12eb, 0x1295, 0x130e, 0x1295, 0x12e8, 0x12ab, 0x1270,
0x122a, 0x1295, 0x1260, 0x122d, 0x130d, 0x12ed, 0x122c, 0x126b, 0x1295, 0x12a0, 0x12de, 0x1228,
0x1235, 0x1264, 0x122d, 0x1219, 0x12f3, 0x12ab, 0x1293, 0x122a, 0x12ac, 0x1355, 0x20, 0x126c,
0x122d, 0x12f0, 0x134b, 0x122e, 0x12a5, 0x121b, 0x12f4, 0x122b, 0x122c, 0x12ed, 0x12ad, 0x1303,
0x126a, 0x12ad, 0x12f0, 0x1261, 0x1265, 0x20, 0x1306, 0x122d, 0x1302, 0x12eb, 0x1245, 0x12f5,
0x1235, 0x1275, 0x20, 0x1204, 0x1208, 0x1293, 0x1235, 0x1273, 0x1295, 0x120c, 0x12ed, 0x12a0,
0x12f4, 0x120c, 0x12a5, 0x12f5, 0x1265, 0x122a, 0x1235, 0x1264, 0x1295, 0x1265, 0x122e, 0x12ad,
0x1295, 0x20, 0x1202, 0x120d, 0x12f3, 0x122d, 0x12ca, 0x1295, 0x12a1, 0x12ad, 0x120b, 0x1206,
0x1263, 0x122d, 0x1275, 0x120a, 0x1295, 0x12f5, 0x121b, 0x1295, 0x120e, 0x122d, 0x12f5, 0x20,
0x1206, 0x12ca, 0x121c, 0x120d, 0x1266, 0x122d, 0x1295, 0x1350, 0x122d, 0x12dd, 0x1232, 0x12f5,
0x1292, 0x12a0, 0x121d, 0x1235, 0x1270, 0x122d, 0x12f3, 0x121d, 0x12a0, 0x1295, 0x12f6, 0x122b,
0x12a0, 0x1235, 0x1275, 0x122b, 0x12ad, 0x1203, 0x1295, 0x12a0, 0x1274, 0x1295, 0x1235, 0x1264,
0x120d, 0x130d, 0x122c, 0x12f5, 0x1260, 0x122d, 0x120a, 0x1295, 0x1265, 0x122b, 0x1272, 0x1235,
0x120b, 0x126b, 0x1265, 0x122b, 0x1230, 0x120d, 0x1235, 0x1261, 0x12ab, 0x122c, 0x1235, 0x1275,
0x1261, 0x12f3, 0x1354, 0x1235, 0x1275, 0x1261, 0x1232, 0x1295, 0x1308, 0x1295, 0x127a, 0x1235,
0x1293, 0x12a1, 0x12ae, 0x1350, 0x1295, 0x1203, 0x1308, 0x1295, 0x12f0, 0x1265, 0x120a, 0x1295,
0x1302, 0x1265, 0x122b, 0x120d, 0x1273, 0x122d, 0x1309, 0x122d, 0x1290, 0x1232, 0x1204, 0x120d,
0x1232, 0x1295, 0x12aa, 0x12a0, 0x12ed, 0x1235, 0x120d, 0x20, 0x12a6, 0x134d, 0x20, 0x121b,
0x1295, 0x12a2, 0x1235, 0x1273, 0x1295, 0x1261, 0x120d, 0x1300, 0x122d, 0x1232, 0x12ab, 0x120a,
0x1292, 0x1295, 0x130d, 0x122b, 0x12f5, 0x12aa, 0x122e, 0x126d, 0x12aa, 0x12e8, 0x126d, 0x120a,
0x12dd, 0x1260, 0x1295, 0x120d, 0x1301, 0x1265, 0x120d, 0x1303, 0x1293, 0x1208, 0x1295, 0x12f0,
0x1295, 0x1209, 0x12ad, 0x1230, 0x121d, 0x1260, 0x122d, 0x130d, 0x121b, 0x12f5, 0x122a, 0x12f5,
0x121b, 0x120d, 0x1273, 0x121c, 0x122a, 0x1203, 0x121d, 0x1295, 0x121a, 0x1295, 0x1235, 0x12ad,
0x121e, 0x1293, 0x12ae, 0x121e, 0x1235, 0x12ae, 0x12a6, 0x1235, 0x120e, 0x1353, 0x122a, 0x1235,
0x1356, 0x12f5, 0x130e, 0x122a, 0x12ab, 0x1355, 0x122b, 0x130d, 0x122a, 0x130b, 0x122e, 0x121d,
0x1233, 0x121b, 0x122b, 0x1233, 0x1295, 0x20, 0x121b, 0x122a, 0x1296, 0x1233, 0x122a, 0x12e8,
0x126e, 0x1233, 0x122b, 0x1276, 0x126d, 0x1232, 0x121d, 0x1348, 0x122e, 0x1356, 0x120d, 0x1235,
0x12ae, 0x1355, 0x12e8, 0x1236, 0x134a, 0x12eb, 0x1235, 0x1276, 0x12ad, 0x1206, 0x120d, 0x121d,
0x1273, 0x120a, 0x1295, 0x1274, 0x122b, 0x1295, 0x12a1, 0x120d, 0x12eb, 0x1296, 0x126d, 0x1235,
0x12ad, 0x126b, 0x12f1, 0x12dd, 0x126b, 0x1272, 0x12ab, 0x1295, 0x126a, 0x12e8, 0x1293, 0x126a,
0x120a, 0x1290, 0x1235, 0x126e, 0x120d, 0x130e, 0x122b, 0x12f5, 0x12cb, 0x122d, 0x1236, 0x12cd,
0x12db, 0x130d, 0x122c, 0x1265, 0x12d9, 0x122a, 0x12ad, 0x12a0, 0x1295, 0x1273, 0x1293, 0x1293,
0x122a, 0x126e, 0x127b, 0x130e, 0x1235, 0x1308, 0x1293, 0x12ae, 0x12ae, 0x1235, 0x12ae, 0x121e,
0x122e, 0x12ac, 0x122d, 0x1309, 0x1208, 0x1295, 0x121b, 0x1204, 0x121b, 0x120d, 0x12f2, 0x1268,
0x121e, 0x122a, 0x123d, 0x12e8, 0x1235, 0x121b, 0x12ee, 0x1274, 0x122c, 0x12e9, 0x1292, 0x12e8,
0x1295, 0x12a0, 0x1352, 0x12a0, 0x12a6, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x1266, 0x130c, 0x12ed,
0x1295, 0x126a, 0x120d, 0x127b, 0x1273, 0x121d, 0x1279, 0x12ad, 0x134b, 0x1232, 0x12ab, 0x12a2,
0x134b, 0x1274, 0x134b, 0x12ab, 0x12a6, 0x134e, 0x134a, 0x1302, 0x1348, 0x1293, 0x1349, 0x1272,
0x130b, 0x120b, 0x1353, 0x130e, 0x1235, 0x130b, 0x121d, 0x1262, 0x12e8, 0x122d, 0x1309, 0x12cb,
0x12f3, 0x120d, 0x12ab, 0x1293, 0x120d, 0x1309, 0x12a0, 0x121d, 0x1206, 0x1296, 0x1209, 0x1209,
0x12a2, 0x1295, 0x12f0, 0x122d, 0x1260, 0x122a, 0x12aa, 0x122a, 0x1272, 0x121b, 0x1272, 0x12ae,
0x1235, 0x122c, 0x12ad, 0x12cb, 0x1303, 0x120a, 0x1295, 0x121b, 0x1301, 0x1229, 0x121b, 0x122d,
0x12ad, 0x12cc, 0x1233, 0x1235, 0x121a, 0x12f5, 0x12c8, 0x12ed, 0x1293, 0x12cd, 0x1229, 0x1292,
0x12cc, 0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x1293, 0x12a6, 0x121a, 0x12a0, 0x1353, 0x130e,
0x20, 0x1353, 0x130e, 0x1353, 0x120b, 0x12cd, 0x1352, 0x1275, 0x12a8, 0x12ed, 0x122d, 0x1295,
0x1356, 0x1205, 0x1295, 0x1354, 0x1356, 0x122d, 0x1275, 0x20, 0x121e, 0x122c, 0x1235, 0x1263,
0x12ed, 0x122b, 0x122e, 0x1276, 0x1295, 0x130b, 0x1234, 0x12ed, 0x1353, 0x1295, 0x1273, 0x1202,
0x1272, 0x1273, 0x122b, 0x12cb, 0x1276, 0x1295, 0x130b, 0x1273, 0x1351, 0x12cb, 0x12ac, 0x12cb,
0x120a, 0x1235, 0x623, 0x628, 0x64a, 0x62f, 0x62c, 0x627, 0x646, 0x623, 0x643, 0x631,
0x627, 0x623, 0x62f, 0x64a, 0x633, 0x20, 0x623, 0x628, 0x627, 0x628, 0x627, 0x627,
0x644, 0x62c, 0x632, 0x627, 0x626, 0x631, 0x623, 0x633, 0x645, 0x631, 0x629, 0x628,
0x627, 0x645, 0x627, 0x643, 0x648, 0x628, 0x627, 0x646, 0x63a, 0x648, 0x64a, 0x628,
0x627, 0x646, 0x62c, 0x648, 0x644, 0x628, 0x64a, 0x633, 0x627, 0x648, 0x628, 0x644,
0x627, 0x646, 0x62a, 0x627, 0x64a, 0x631, 0x628, 0x631, 0x627, 0x632, 0x627, 0x641,
0x64a, 0x644, 0x628, 0x648, 0x62c, 0x648, 0x645, 0x628, 0x648, 0x631, 0x627, 0x627,
0x644, 0x642, 0x627, 0x647, 0x631, 0x629, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x20,
0x627, 0x644, 0x628, 0x64a, 0x636, 0x627, 0x621, 0x633, 0x64a, 0x62a, 0x627, 0x643,
0x648, 0x646, 0x627, 0x643, 0x631, 0x64a, 0x62f, 0x627, 0x643, 0x627, 0x631, 0x62f,
0x627, 0x631, 0x20, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x62c, 0x64a, 0x628,
0x648, 0x62a, 0x64a, 0x62f, 0x648, 0x627, 0x644, 0x627, 0x627, 0x644, 0x639, 0x64a,
0x648, 0x646, 0x641, 0x631, 0x64a, 0x20, 0x62a, 0x627, 0x648, 0x646, 0x63a, 0x627,
0x628, 0x648, 0x631, 0x648, 0x646, 0x647, 0x631, 0x627, 0x631, 0x64a, 0x62c, 0x648,
0x647, 0x627, 0x646, 0x633, 0x628, 0x631, 0x63a, 0x62c, 0x648, 0x628, 0x627, 0x643,
0x627, 0x645, 0x628, 0x627, 0x644, 0x627, 0x627, 0x644, 0x62e, 0x631, 0x637, 0x648,
0x645, 0x643, 0x64a, 0x63a, 0x627, 0x644, 0x64a, 0x643, 0x64a, 0x646, 0x634, 0x627,
0x633, 0x627, 0x644, 0x627, 0x63a, 0x648, 0x633, 0x644, 0x64a, 0x628, 0x631, 0x641,
0x64a, 0x644, 0x644, 0x648, 0x645, 0x64a, 0x644, 0x648, 0x627, 0x646, 0x62f, 0x627,
0x644, 0x648, 0x645, 0x628, 0x628, 0x627, 0x634, 0x627, 0x644, 0x648, 0x633, 0x627,
0x643, 0x627, 0x645, 0x627, 0x644, 0x627, 0x628, 0x648, 0x645, 0x627, 0x628, 0x648,
0x62a, 0x648, 0x645, 0x627, 0x633, 0x64a, 0x631, 0x648, 0x645, 0x628, 0x627, 0x628,
0x627, 0x646, 0x64a, 0x645, 0x642, 0x62f, 0x64a, 0x634, 0x64a, 0x648, 0x645, 0x648,
0x646, 0x631, 0x648, 0x641, 0x64a, 0x627, 0x646, 0x64a, 0x631, 0x648, 0x628, 0x64a,
0x646, 0x62c, 0x627, 0x645, 0x64a, 0x646, 0x627, 0x646, 0x64a, 0x627, 0x645, 0x64a,
0x646, 0x648, 0x627, 0x643, 0x634, 0x648, 0x637, 0x648, 0x627, 0x63a, 0x627, 0x62f,
0x648, 0x63a, 0x648, 0x628, 0x648, 0x631, 0x62a, 0x648, 0x20, 0x646, 0x648, 0x641,
0x648, 0x633, 0x627, 0x648, 0x20, 0x62a, 0x648, 0x645, 0x64a, 0x637, 0x631, 0x627,
0x628, 0x644, 0x633, 0x62a, 0x648, 0x646, 0x633, 0x648, 0x64a, 0x646, 0x62f, 0x647,
0x648, 0x643, 0x623, 0x62f, 0x627, 0x643, 0x623, 0x646, 0x634, 0x648, 0x631, 0x627,
0x62c, 0x623, 0x646, 0x63a, 0x648, 0x64a, 0x644, 0x627, 0x623, 0x646, 0x62a, 0x64a,
0x63a, 0x648, 0x627, 0x623, 0x631, 0x648, 0x62c, 0x648, 0x627, 0x646, 0x64a, 0x627,
0x628, 0x648, 0x64a, 0x646, 0x648, 0x633, 0x20, 0x623, 0x64a, 0x631, 0x633, 0x643,
0x627, 0x62a, 0x627, 0x645, 0x627, 0x631, 0x643, 0x627, 0x643, 0x648, 0x631, 0x62f,
0x648, 0x628, 0x627, 0x62c, 0x648, 0x62c, 0x648, 0x644, 0x627, 0x20, 0x631, 0x64a,
0x648, 0x62c, 0x627, 0x645, 0x64a, 0x646, 0x62f, 0x648, 0x632, 0x627, 0x631, 0x64a,
0x648, 0x20, 0x62c, 0x627, 0x644, 0x64a, 0x64a, 0x648, 0x633, 0x633, 0x627, 0x644,
0x637, 0x627, 0x633, 0x627, 0x646, 0x20, 0x62e, 0x648, 0x627, 0x646, 0x633, 0x627,
0x646, 0x20, 0x644, 0x648, 0x64a, 0x633, 0x62a, 0x627, 0x643, 0x645, 0x627, 0x646,
0x623, 0x634, 0x648, 0x627, 0x623, 0x631, 0x648, 0x628, 0x627, 0x623, 0x633, 0x648,
0x646, 0x633, 0x64a, 0x648, 0x646, 0x643, 0x648, 0x631, 0x627, 0x644, 0x20, 0x647,
0x627, 0x631, 0x628, 0x631, 0x628, 0x627, 0x647, 0x64a, 0x627, 0x628, 0x627, 0x647,
0x64a, 0x627, 0x20, 0x628, 0x627, 0x646, 0x62f, 0x64a, 0x631, 0x627, 0x633, 0x628,
0x631, 0x628, 0x627, 0x62f, 0x648, 0x633, 0x628, 0x644, 0x645, 0x628, 0x644, 0x64a,
0x632, 0x628, 0x644, 0x627, 0x646, 0x643, 0x2d, 0x633, 0x627, 0x628, 0x644, 0x648,
0x646, 0x628, 0x627, 0x648, 0x20, 0x641, 0x64a, 0x633, 0x62a, 0x627, 0x628, 0x648,
0x63a, 0x648, 0x62a, 0x627, 0x628, 0x648, 0x64a, 0x633, 0x643, 0x627, 0x645, 0x628,
0x631, 0x62f, 0x64a, 0x62c, 0x20, 0x628, 0x627, 0x64a, 0x643, 0x648, 0x645, 0x628,
0x648, 0x20, 0x62c, 0x631, 0x627, 0x646, 0x62f, 0x643, 0x627, 0x646, 0x643, 0x648,
0x646, 0x643, 0x627, 0x631, 0x627, 0x643, 0x627, 0x633, 0x643, 0x627, 0x64a, 0x64a,
0x646, 0x643, 0x627, 0x64a, 0x645, 0x627, 0x646, 0x634, 0x64a, 0x643, 0x627, 0x63a,
0x648, 0x62a, 0x634, 0x64a, 0x648, 0x627, 0x648, 0x627, 0x633, 0x64a, 0x648, 0x62f,
0x627, 0x62f, 0x20, 0x62e, 0x648, 0x627, 0x631, 0x64a, 0x632, 0x643, 0x648, 0x633,
0x62a, 0x627, 0x631, 0x64a, 0x643, 0x627, 0x643, 0x631, 0x64a, 0x633, 0x62a, 0x648,
0x646, 0x643, 0x64a, 0x627, 0x628, 0x627, 0x643, 0x648, 0x631, 0x627, 0x633, 0x627,
0x648, 0x62f, 0x627, 0x646, 0x645, 0x631, 0x643, 0x20, 0x634, 0x627, 0x641, 0x646,
0x62f, 0x627, 0x648, 0x633, 0x627, 0x646, 0x62f, 0x627, 0x648, 0x633, 0x646, 0x20,
0x643, 0x631, 0x64a, 0x643, 0x62f, 0x646, 0x641, 0x631, 0x62f, 0x64a, 0x62a, 0x631,
0x648, 0x64a, 0x62a, 0x62f, 0x648, 0x645, 0x64a, 0x646, 0x64a, 0x643, 0x627, 0x627,
0x64a, 0x62f, 0x645, 0x648, 0x646, 0x62a, 0x648, 0x646, 0x627, 0x64a, 0x631, 0x648,
0x646, 0x628, 0x64a, 0x627, 0x644, 0x633, 0x644, 0x641, 0x627, 0x62f, 0x648, 0x631,
0x641, 0x648, 0x631, 0x62a, 0x20, 0x646, 0x64a, 0x644, 0x633, 0x648, 0x646, 0x641,
0x648, 0x631, 0x62a, 0x627, 0x644, 0x64a, 0x632, 0x627, 0x62c, 0x644, 0x627, 0x633,
0x20, 0x628, 0x627, 0x64a, 0x62c, 0x648, 0x633, 0x20, 0x628, 0x627, 0x64a, 0x63a,
0x631, 0x627, 0x646, 0x62f, 0x20, 0x62a, 0x631, 0x643, 0x63a, 0x631, 0x64a, 0x646,
0x627, 0x62f, 0x627, 0x63a, 0x648, 0x627, 0x62f, 0x644, 0x648, 0x628, 0x63a, 0x648,
0x627, 0x62a, 0x64a, 0x645, 0x627, 0x644, 0x627, 0x63a, 0x648, 0x627, 0x64a, 0x627,
0x643, 0x648, 0x64a, 0x644, 0x63a, 0x64a, 0x627, 0x646, 0x627, 0x647, 0x627, 0x644,
0x64a, 0x641, 0x627, 0x643, 0x633, 0x647, 0x627, 0x641, 0x627, 0x646, 0x627, 0x647,
0x64a, 0x631, 0x645, 0x648, 0x633, 0x64a, 0x644, 0x648, 0x625, 0x646, 0x62f, 0x64a,
0x627, 0x646, 0x627, 0x628, 0x648, 0x644, 0x64a, 0x633, 0x643, 0x648, 0x646, 0x643,
0x633, 0x645, 0x627, 0x631, 0x646, 0x62c, 0x648, 0x628, 0x64a, 0x62a, 0x631, 0x633,
0x628, 0x631, 0x63a, 0x645, 0x62f, 0x64a, 0x646, 0x629, 0x20, 0x62a, 0x644, 0x60c,
0x20, 0x625, 0x646, 0x62f, 0x64a, 0x627, 0x646, 0x627, 0x641, 0x64a, 0x641, 0x627,
0x64a, 0x641, 0x64a, 0x646, 0x633, 0x64a, 0x646, 0x633, 0x648, 0x64a, 0x646, 0x627,
0x645, 0x627, 0x643, 0x627, 0x64a, 0x646, 0x648, 0x641, 0x64a, 0x643, 0x627, 0x643,
0x648, 0x64a, 0x644, 0x62a, 0x62c, 0x627, 0x645, 0x627, 0x64a, 0x643, 0x627, 0x62c,
0x648, 0x646, 0x64a, 0x644, 0x648, 0x64a, 0x633, 0x20, 0x641, 0x64a, 0x644, 0x645,
0x648, 0x646, 0x62a, 0x64a, 0x633, 0x64a, 0x644, 0x648, 0x643, 0x631, 0x627, 0x644,
0x646, 0x62f, 0x64a, 0x643, 0x644, 0x627, 0x20, 0x628, 0x627, 0x632, 0x644, 0x64a,
0x645, 0x627, 0x644, 0x648, 0x633, 0x20, 0x627, 0x646, 0x62c, 0x644, 0x648, 0x633,
0x62d, 0x64a, 0x20, 0x627, 0x644, 0x623, 0x645, 0x64a, 0x631, 0x20, 0x627, 0x644,
0x633, 0x641, 0x644, 0x64a, 0x645, 0x627, 0x634, 0x64a, 0x648, 0x645, 0x627, 0x646,
0x627, 0x63a, 0x648, 0x627, 0x645, 0x627, 0x646, 0x627, 0x648, 0x633, 0x645, 0x627,
0x631, 0x64a, 0x63a, 0x648, 0x62a, 0x627, 0x644, 0x645, 0x627, 0x631, 0x62a, 0x64a,
0x646, 0x64a, 0x643, 0x645, 0x627, 0x62a, 0x627, 0x645, 0x648, 0x631, 0x648, 0x633,
0x645, 0x627, 0x632, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x645, 0x64a, 0x646, 0x648,
0x645, 0x64a, 0x646, 0x64a, 0x645, 0x64a, 0x631, 0x64a, 0x62f, 0x627, 0x645, 0x64a,
0x62a, 0x644, 0x627, 0x643, 0x627, 0x62a, 0x644, 0x627, 0x645, 0x643, 0x633, 0x64a,
0x643, 0x648, 0x20, 0x633, 0x64a, 0x62a, 0x64a, 0x645, 0x643, 0x648, 0x64a, 0x644,
0x648, 0x646, 0x648, 0x64a, 0x646, 0x643, 0x62a, 0x648, 0x646, 0x645, 0x648, 0x646,
0x62a, 0x64a, 0x631, 0x64a, 0x645, 0x648, 0x646, 0x62a, 0x641, 0x64a, 0x62f, 0x64a,
0x648, 0x645, 0x648, 0x646, 0x62a, 0x633, 0x64a, 0x631, 0x627, 0x62a, 0x646, 0x627,
0x633, 0x648, 0x646, 0x64a, 0x648, 0x64a, 0x648, 0x631, 0x643, 0x646, 0x648, 0x631,
0x648, 0x646, 0x627, 0x647, 0x628, 0x64a, 0x648, 0x644, 0x627, 0x60c, 0x20, 0x62f,
0x627, 0x643, 0x648, 0x62a, 0x627, 0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644,
0x64a, 0x629, 0x633, 0x646, 0x62a, 0x631, 0x646, 0x64a, 0x648, 0x20, 0x633, 0x627,
0x644, 0x64a, 0x645, 0x63a, 0x648, 0x62f, 0x62b, 0x627, 0x628, 0x623, 0x648, 0x62c,
0x64a, 0x646, 0x627, 0x62c, 0x627, 0x628, 0x646, 0x645, 0x627, 0x628, 0x627, 0x631,
0x627, 0x645, 0x627, 0x631, 0x64a, 0x628, 0x648, 0x641, 0x64a, 0x646, 0x643, 0x633,
0x628, 0x648, 0x631, 0x62a, 0x20, 0x623, 0x648, 0x20, 0x628, 0x631, 0x646, 0x633,
0x628, 0x648, 0x631, 0x62a, 0x20, 0x623, 0x648, 0x641, 0x20, 0x633, 0x628, 0x64a,
0x646, 0x628, 0x648, 0x631, 0x62a, 0x648, 0x20, 0x641, 0x64a, 0x644, 0x648, 0x628,
0x648, 0x631, 0x62a, 0x648, 0x631, 0x64a, 0x643, 0x648, 0x628, 0x648, 0x646, 0x62a,
0x627, 0x20, 0x623, 0x631, 0x64a, 0x646, 0x627, 0x632, 0x631, 0x627, 0x646, 0x643,
0x646, 0x20, 0x627, 0x646, 0x644, 0x62a, 0x631, 0x64a, 0x633, 0x64a, 0x641, 0x631,
0x64a, 0x62c, 0x64a, 0x646, 0x627, 0x631, 0x64a, 0x632, 0x648, 0x644, 0x648, 0x62a,
0x631, 0x64a, 0x648, 0x628, 0x631, 0x627, 0x646, 0x643, 0x648, 0x633, 0x627, 0x646,
0x62a, 0x627, 0x631, 0x64a, 0x645, 0x633, 0x627, 0x646, 0x62a, 0x64a, 0x627, 0x63a,
0x648, 0x633, 0x627, 0x646, 0x62a, 0x648, 0x20, 0x62f, 0x648, 0x645, 0x64a, 0x646,
0x63a, 0x648, 0x633, 0x627, 0x648, 0x20, 0x628, 0x627, 0x648, 0x644, 0x648, 0x633,
0x643, 0x648, 0x631, 0x633, 0x628, 0x64a, 0x633, 0x646, 0x62f, 0x633, 0x64a, 0x62a,
0x643, 0x627, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x628, 0x627, 0x631, 0x62a, 0x64a,
0x644, 0x64a, 0x645, 0x64a, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x62c, 0x648, 0x646,
0x633, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x643, 0x64a, 0x62a, 0x633, 0x633, 0x627,
0x646, 0x62a, 0x20, 0x644, 0x648, 0x634, 0x64a, 0x627, 0x633, 0x627, 0x646, 0x62a,
0x20, 0x62a, 0x648, 0x645, 0x627, 0x633, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x641,
0x646, 0x633, 0x646, 0x62a, 0x633, 0x648, 0x641, 0x62a, 0x20, 0x643, 0x627, 0x631,
0x646, 0x62a, 0x62a, 0x64a, 0x63a, 0x648, 0x633, 0x64a, 0x63a, 0x627, 0x644, 0x628,
0x627, 0x62b, 0x64a, 0x644, 0x62a, 0x64a, 0x62e, 0x648, 0x627, 0x646, 0x627, 0x62a,
0x648, 0x631, 0x648, 0x646, 0x62a, 0x648, 0x62a, 0x648, 0x631, 0x62a, 0x648, 0x644,
0x627, 0x641, 0x627, 0x646, 0x643, 0x648, 0x641, 0x631, 0x648, 0x627, 0x64a, 0x62a,
0x20, 0x647, 0x648, 0x631, 0x633, 0x648, 0x64a, 0x646, 0x64a, 0x628, 0x64a, 0x62c,
0x64a, 0x627, 0x643, 0x648, 0x62a, 0x627, 0x62a, 0x643, 0x627, 0x633, 0x627, 0x64a,
0x62f, 0x627, 0x641, 0x64a, 0x632, 0x62f, 0x64a, 0x20, 0x645, 0x648, 0x646, 0x62a,
0x20, 0x62f, 0x648, 0x20, 0x631, 0x648, 0x641, 0x64a, 0x644, 0x645, 0x627, 0x643,
0x648, 0x627, 0x631, 0x64a, 0x645, 0x627, 0x648, 0x633, 0x648, 0x646, 0x645, 0x627,
0x643, 0x20, 0x645, 0x648, 0x631, 0x62f, 0x648, 0x628, 0x627, 0x644, 0x645, 0x64a,
0x631, 0x631, 0x648, 0x62b, 0x64a, 0x631, 0x627, 0x633, 0x627, 0x64a, 0x648, 0x648,
0x627, 0x62a, 0x631, 0x648, 0x644, 0x641, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x644,
0x648, 0x646, 0x62c, 0x64a, 0x631, 0x628, 0x64a, 0x646, 0x639, 0x62f, 0x646, 0x623,
0x644, 0x645, 0x627, 0x62a, 0x64a, 0x639, 0x645, 0x651, 0x627, 0x646, 0x623, 0x646,
0x62f, 0x64a, 0x631, 0x623, 0x643, 0x62a, 0x627, 0x648, 0x623, 0x643, 0x62a, 0x648,
0x628, 0x639, 0x634, 0x642, 0x20, 0x622, 0x628, 0x627, 0x62f, 0x623, 0x62a, 0x64a,
0x631, 0x627, 0x648, 0x628, 0x63a, 0x62f, 0x627, 0x62f, 0x627, 0x644, 0x628, 0x62d,
0x631, 0x64a, 0x646, 0x628, 0x627, 0x646, 0x643, 0x648, 0x643, 0x628, 0x627, 0x631,
0x646, 0x627, 0x648, 0x644, 0x628, 0x64a, 0x631, 0x648, 0x62a, 0x628, 0x634, 0x643,
0x64a, 0x643, 0x628, 0x631, 0x648, 0x646, 0x627, 0x64a, 0x62a, 0x634, 0x64a, 0x62a,
0x627, 0x643, 0x648, 0x644, 0x648, 0x645, 0x628, 0x648, 0x62f, 0x645, 0x634, 0x642,
0x62f, 0x64a, 0x644, 0x64a, 0x62f, 0x628, 0x64a, 0x62f, 0x648, 0x634, 0x627, 0x646,
0x628, 0x64a, 0x641, 0x627, 0x645, 0x627, 0x63a, 0x648, 0x633, 0x62a, 0x627, 0x63a,
0x632, 0x629, 0x647, 0x64a, 0x628, 0x631, 0x648, 0x646, 0x20, 0x28, 0x645, 0x62f,
0x64a, 0x646, 0x629, 0x20, 0x627, 0x644, 0x62e, 0x644, 0x64a, 0x644, 0x29, 0x645,
0x62f, 0x64a, 0x646, 0x629, 0x20, 0x647, 0x648, 0x20, 0x62a, 0x634, 0x64a, 0x20,
0x645, 0x646, 0x629, 0x647, 0x648, 0x646, 0x63a, 0x20, 0x643, 0x648, 0x646, 0x63a,
0x647, 0x648, 0x641, 0x62f, 0x627, 0x64a, 0x631, 0x643, 0x64a, 0x62a, 0x633, 0x643,
0x62c, 0x627, 0x643, 0x631, 0x62a, 0x627, 0x62c, 0x627, 0x64a, 0x627, 0x628, 0x64a,
0x648, 0x631, 0x627, 0x627, 0x644, 0x642, 0x62f, 0x633, 0x643, 0x627, 0x628, 0x648,
0x644, 0x643, 0x627, 0x645, 0x62a, 0x634, 0x627, 0x62a, 0x643, 0x627, 0x643, 0x631,
0x627, 0x62a, 0x634, 0x64a, 0x643, 0x627, 0x62a, 0x645, 0x627, 0x646, 0x62f, 0x648,
0x62e, 0x627, 0x646, 0x62f, 0x64a, 0x62c, 0x627, 0x643, 0x627, 0x644, 0x643, 0x62a,
0x627, 0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643,
0x643, 0x648, 0x627, 0x644, 0x627, 0x20, 0x644, 0x627, 0x645, 0x628, 0x648, 0x631,
0x643, 0x64a, 0x634, 0x64a, 0x646, 0x62c, 0x627, 0x644, 0x643, 0x648, 0x64a, 0x62a,
0x645, 0x627, 0x643, 0x627, 0x648, 0x645, 0x62c, 0x627, 0x62f, 0x646, 0x645, 0x627,
0x643, 0x633, 0x627, 0x631, 0x645, 0x627, 0x646, 0x64a, 0x644, 0x627, 0x645, 0x633,
0x642, 0x637, 0x646, 0x64a, 0x642, 0x648, 0x633, 0x64a, 0x627, 0x646, 0x648, 0x641,
0x648, 0x643, 0x648, 0x632, 0x646, 0x62a, 0x633, 0x643, 0x646, 0x648, 0x641, 0x648,
0x633, 0x628, 0x64a, 0x631, 0x633, 0x643, 0x623, 0x648, 0x645, 0x633, 0x643, 0x623,
0x648, 0x631, 0x627, 0x644, 0x628, 0x646, 0x648, 0x645, 0x20, 0x628, 0x646, 0x647,
0x628, 0x648, 0x646, 0x62a, 0x64a, 0x627, 0x646, 0x643, 0x628, 0x64a, 0x648, 0x646,
0x63a, 0x20, 0x64a, 0x627, 0x646, 0x63a, 0x642, 0x637, 0x631, 0x642, 0x648, 0x633,
0x62a, 0x627, 0x646, 0x627, 0x64a, 0x643, 0x64a, 0x632, 0x64a, 0x644, 0x648, 0x631,
0x62f, 0x627, 0x627, 0x644, 0x631, 0x64a, 0x627, 0x636, 0x633, 0x643, 0x627, 0x644,
0x64a, 0x646, 0x633, 0x645, 0x631, 0x642, 0x646, 0x62f, 0x633, 0x648, 0x644, 0x634,
0x646, 0x63a, 0x647, 0x627, 0x64a, 0x633, 0x646, 0x63a, 0x627, 0x641, 0x648, 0x631,
0x629, 0x633, 0x631, 0x64a, 0x62f, 0x646, 0x643, 0x648, 0x644, 0x64a, 0x645, 0x633,
0x643, 0x62a, 0x627, 0x64a, 0x628, 0x64a, 0x647, 0x637, 0x634, 0x642, 0x646, 0x62f,
0x62a, 0x628, 0x644, 0x64a, 0x633, 0x64a, 0x637, 0x647, 0x631, 0x627, 0x646, 0x62a,
0x64a, 0x645, 0x641, 0x648, 0x637, 0x648, 0x643, 0x64a, 0x648, 0x62a, 0x648, 0x645,
0x633, 0x643, 0x622, 0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x627, 0x631, 0x623,
0x631, 0x648, 0x645, 0x643, 0x64a, 0x623, 0x648, 0x633, 0x62a, 0x20, 0x646, 0x64a,
0x631, 0x627, 0x641, 0x64a, 0x627, 0x646, 0x62a, 0x64a, 0x627, 0x646, 0x641, 0x644,
0x627, 0x62f, 0x64a, 0x641, 0x648, 0x633, 0x62a, 0x643, 0x64a, 0x627, 0x643, 0x62a,
0x633, 0x643, 0x631, 0x627, 0x646, 0x63a, 0x648, 0x646, 0x64a, 0x643, 0x627, 0x62a,
0x631, 0x646, 0x628, 0x64a, 0x631, 0x62c, 0x64a, 0x631, 0x64a, 0x641, 0x627, 0x646,
0x623, 0x632, 0x648, 0x631, 0x633, 0x628, 0x631, 0x645, 0x648, 0x62f, 0x627, 0x643,
0x646, 0x627, 0x631, 0x64a, 0x627, 0x644, 0x631, 0x623, 0x633, 0x20, 0x627, 0x644,
0x623, 0x62e, 0x636, 0x631, 0x641, 0x627, 0x631, 0x648, 0x645, 0x627, 0x62f, 0x64a,
0x631, 0x627, 0x631, 0x64a, 0x643, 0x64a, 0x627, 0x641, 0x64a, 0x643, 0x62c, 0x648,
0x631, 0x62c, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x62c, 0x646, 0x648, 0x628, 0x64a,
0x629, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x647, 0x64a, 0x644, 0x64a, 0x646, 0x627,
0x627, 0x633, 0x62a, 0x627, 0x646, 0x644, 0x64a, 0x623, 0x62f, 0x64a, 0x644, 0x64a,
0x62f, 0x628, 0x631, 0x633, 0x64a, 0x628, 0x627, 0x646, 0x628, 0x631, 0x648, 0x643,
0x646, 0x20, 0x647, 0x64a, 0x644, 0x62f, 0x627, 0x631, 0x648, 0x646, 0x623, 0x648,
0x643, 0x644, 0x627, 0x647, 0x648, 0x628, 0x627, 0x631, 0x62a, 0x644, 0x64a, 0x646,
0x62f, 0x645, 0x627, 0x646, 0x644, 0x648, 0x631, 0x62f, 0x20, 0x647, 0x627, 0x648,
0x645, 0x64a, 0x644, 0x628, 0x648, 0x631, 0x646, 0x628, 0x631, 0x62b, 0x627, 0x633,
0x64a, 0x62f, 0x646, 0x64a, 0x623, 0x645, 0x633, 0x62a, 0x631, 0x62f, 0x627, 0x645,
0x623, 0x646, 0x62f, 0x648, 0x631, 0x627, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x62e,
0x627, 0x646, 0x623, 0x62b, 0x64a, 0x646, 0x627, 0x628, 0x644, 0x63a, 0x631, 0x627,
0x62f, 0x628, 0x631, 0x644, 0x64a, 0x646, 0x628, 0x631, 0x627, 0x62a, 0x64a, 0x633,
0x644, 0x627, 0x641, 0x627, 0x628, 0x631, 0x648, 0x643, 0x633, 0x644, 0x628, 0x648,
0x62e, 0x627, 0x631, 0x633, 0x62a, 0x628, 0x648, 0x62f, 0x627, 0x628, 0x633, 0x62a,
0x628, 0x648, 0x633, 0x646, 0x63a, 0x646, 0x62a, 0x634, 0x64a, 0x633, 0x64a, 0x646,
0x627, 0x648, 0x643, 0x648, 0x628, 0x646, 0x647, 0x627, 0x63a, 0x646, 0x62f, 0x628,
0x644, 0x646, 0x62c, 0x628, 0x644, 0x20, 0x637, 0x627, 0x631, 0x642, 0x63a, 0x64a,
0x631, 0x646, 0x632, 0x64a, 0x647, 0x644, 0x633, 0x646, 0x643, 0x64a, 0x62c, 0x632,
0x64a, 0x631, 0x629, 0x20, 0x645, 0x627, 0x646, 0x625, 0x633, 0x637, 0x646, 0x628,
0x648, 0x644, 0x62c, 0x64a, 0x631, 0x633, 0x64a, 0x643, 0x627, 0x644, 0x64a, 0x646,
0x62c, 0x631, 0x627, 0x62f, 0x643, 0x64a, 0x631, 0x648, 0x641, 0x643, 0x64a, 0x64a,
0x641, 0x644, 0x634, 0x628, 0x648, 0x646, 0x629, 0x644, 0x64a, 0x648, 0x628, 0x644,
0x64a, 0x627, 0x646, 0x627, 0x644, 0x646, 0x62f, 0x646, 0x644, 0x648, 0x643, 0x633,
0x645, 0x628, 0x648, 0x631, 0x63a, 0x645, 0x62f, 0x631, 0x64a, 0x62f, 0x645, 0x627,
0x644, 0x637, 0x629, 0x645, 0x627, 0x631, 0x64a, 0x647, 0x627, 0x645, 0x646, 0x645,
0x64a, 0x646, 0x633, 0x643, 0x645, 0x648, 0x646, 0x627, 0x643, 0x648, 0x645, 0x648,
0x633, 0x643, 0x648, 0x623, 0x648, 0x633, 0x644, 0x648, 0x628, 0x627, 0x631, 0x64a,
0x633, 0x628, 0x648, 0x62f, 0x63a, 0x648, 0x631, 0x64a, 0x643, 0x627, 0x628, 0x631,
0x627, 0x63a, 0x631, 0x64a, 0x63a, 0x627, 0x633, 0x645, 0x631, 0x627, 0x621, 0x633,
0x627, 0x646, 0x20, 0x645, 0x627, 0x631, 0x64a, 0x646, 0x648, 0x633, 0x631, 0x627,
0x64a, 0x64a, 0x641, 0x648, 0x633, 0x627, 0x631, 0x627, 0x62a, 0x648, 0x641, 0x633,
0x64a, 0x645, 0x641, 0x631, 0x648, 0x628, 0x648, 0x644, 0x633, 0x643, 0x648, 0x628,
0x64a, 0x635, 0x648, 0x641, 0x64a, 0x627, 0x633, 0x62a, 0x648, 0x643, 0x647, 0x648,
0x644, 0x645, 0x62a, 0x627, 0x644, 0x64a, 0x646, 0x62a, 0x64a, 0x631, 0x627, 0x646,
0x627, 0x623, 0x648, 0x644, 0x64a, 0x627, 0x646, 0x648, 0x641, 0x633, 0x643, 0x641,
0x627, 0x62f, 0x648, 0x632, 0x627, 0x644, 0x641, 0x627, 0x62a, 0x64a, 0x643, 0x627,
0x646, 0x641, 0x64a, 0x64a, 0x646, 0x627, 0x641, 0x64a, 0x644, 0x646, 0x64a, 0x648,
0x633, 0x641, 0x648, 0x644, 0x648, 0x62c, 0x631, 0x627, 0x62f, 0x648, 0x627, 0x631,
0x633, 0x648, 0x632, 0x63a, 0x631, 0x628, 0x632, 0x64a, 0x648, 0x631, 0x62e, 0x623,
0x646, 0x62a, 0x627, 0x646, 0x627, 0x646, 0x627, 0x631, 0x64a, 0x641, 0x648, 0x62a,
0x634, 0x627, 0x63a, 0x648, 0x633, 0x643, 0x631, 0x64a, 0x633, 0x645, 0x627, 0x633,
0x643, 0x648, 0x643, 0x648, 0x633, 0x62c, 0x632, 0x631, 0x20, 0x627, 0x644, 0x642,
0x645, 0x631, 0x643, 0x64a, 0x631, 0x63a, 0x648, 0x64a, 0x644, 0x64a, 0x646, 0x645,
0x627, 0x647, 0x64a, 0x627, 0x644, 0x645, 0x627, 0x644, 0x62f, 0x64a, 0x641, 0x645,
0x648, 0x631, 0x64a, 0x634, 0x64a, 0x648, 0x633, 0x645, 0x627, 0x64a, 0x648, 0x62a,
0x631, 0x64a, 0x648, 0x646, 0x64a, 0x648, 0x646, 0x623, 0x628, 0x64a, 0x627, 0x623,
0x648, 0x643, 0x644, 0x627, 0x646, 0x62f, 0x628, 0x648, 0x63a, 0x627, 0x646, 0x641,
0x64a, 0x644, 0x62a, 0x634, 0x627, 0x62b, 0x627, 0x645, 0x627, 0x633, 0x62a, 0x631,
0x625, 0x64a, 0x641, 0x627, 0x62a, 0x641, 0x627, 0x643, 0x627, 0x648, 0x641, 0x648,
0x641, 0x64a, 0x62c, 0x64a, 0x641, 0x648, 0x646, 0x627, 0x641, 0x648, 0x62a, 0x64a,
0x62c, 0x644, 0x627, 0x628, 0x627, 0x62c, 0x648, 0x633, 0x62c, 0x627, 0x645, 0x628,
0x64a, 0x631, 0x63a, 0x648, 0x627, 0x62f, 0x627, 0x644, 0x643, 0x627, 0x646, 0x627,
0x644, 0x647, 0x648, 0x646, 0x648, 0x644, 0x648, 0x644, 0x648, 0x627, 0x646, 0x62f,
0x631, 0x628, 0x64a, 0x631, 0x62c, 0x643, 0x64a, 0x631, 0x64a, 0x62a, 0x64a, 0x20,
0x645, 0x627, 0x62a, 0x64a, 0x643, 0x648, 0x633, 0x631, 0x627, 0x643, 0x648, 0x627,
0x62c, 0x627, 0x644, 0x64a, 0x646, 0x645, 0x627, 0x62c, 0x648, 0x631, 0x648, 0x645,
0x627, 0x631, 0x643, 0x64a, 0x633, 0x627, 0x633, 0x645, 0x64a, 0x62f, 0x648, 0x627,
0x64a, 0x646, 0x627, 0x648, 0x631, 0x648, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644,
0x643, 0x646, 0x648, 0x645, 0x64a, 0x627, 0x628, 0x627, 0x63a, 0x648, 0x20, 0x628,
0x627, 0x63a, 0x648, 0x628, 0x627, 0x644, 0x627, 0x648, 0x628, 0x64a, 0x62a, 0x643,
0x64a, 0x631, 0x646, 0x628, 0x627, 0x646, 0x627, 0x628, 0x628, 0x648, 0x631, 0x20,
0x645, 0x648, 0x631, 0x633, 0x628, 0x64a, 0x631, 0x627, 0x631, 0x648, 0x62a, 0x648,
0x646, 0x63a, 0x627, 0x633, 0x627, 0x64a, 0x628, 0x627, 0x646, 0x62a, 0x627, 0x647,
0x64a, 0x62a, 0x64a, 0x62a, 0x627, 0x631, 0x627, 0x648, 0x627, 0x62a, 0x648, 0x646,
0x63a, 0x627, 0x62a, 0x627, 0x628, 0x648, 0x648, 0x627, 0x644, 0x64a, 0x633, 0x531,
0x562, 0x56b, 0x57b, 0x561, 0x576, 0x531, 0x584, 0x580, 0x561, 0x531, 0x564, 0x56b,
0x57d, 0x20, 0x531, 0x562, 0x565, 0x562, 0x561, 0x531, 0x56c, 0x56a, 0x56b, 0x580,
0x531, 0x57d, 0x574, 0x565, 0x580, 0x561, 0x532, 0x561, 0x574, 0x561, 0x56f, 0x578,
0x532, 0x561, 0x576, 0x563, 0x56b, 0x532, 0x561, 0x576, 0x56a, 0x578, 0x582, 0x56c,
0x532, 0x56b, 0x57d, 0x561, 0x578, 0x582, 0x532, 0x56c, 0x561, 0x576, 0x569, 0x561,
0x575, 0x580, 0x532, 0x580, 0x561, 0x566, 0x561, 0x57e, 0x56b, 0x56c, 0x532, 0x578,
0x582, 0x56a, 0x578, 0x582, 0x574, 0x562, 0x578, 0x582, 0x580, 0x561, 0x53f, 0x561,
0x570, 0x56b, 0x580, 0x565, 0x53f, 0x561, 0x57d, 0x561, 0x562, 0x56c, 0x561, 0x576,
0x56f, 0x561, 0x54d, 0x565, 0x578, 0x582, 0x57f, 0x561, 0x53f, 0x578, 0x576, 0x561,
0x56f, 0x580, 0x56b, 0x534, 0x561, 0x584, 0x561, 0x580, 0x534, 0x561, 0x580, 0x2d,
0x537, 0x57d, 0x2d, 0x54d, 0x561, 0x56c, 0x561, 0x574, 0x54b, 0x56b, 0x562, 0x578,
0x582, 0x569, 0x56b, 0x534, 0x578, 0x582, 0x561, 0x56c, 0x561, 0x537, 0x56c, 0x20,
0x531, 0x575, 0x578, 0x582, 0x576, 0x556, 0x580, 0x56b, 0x569, 0x561, 0x578, 0x582,
0x576, 0x533, 0x561, 0x562, 0x578, 0x580, 0x578, 0x576, 0x565, 0x540, 0x561, 0x580,
0x561, 0x580, 0x565, 0x545, 0x578, 0x570, 0x561, 0x576, 0x565, 0x57d, 0x562, 0x578,
0x582, 0x580, 0x563, 0x54b, 0x578, 0x582, 0x562, 0x561, 0x53f, 0x561, 0x574, 0x57a,
0x561, 0x56c, 0x561, 0x53d, 0x561, 0x580, 0x569, 0x578, 0x582, 0x574, 0x53f, 0x56b,
0x563, 0x561, 0x56c, 0x56b, 0x53f, 0x56b, 0x576, 0x577, 0x561, 0x57d, 0x561, 0x53c,
0x561, 0x563, 0x578, 0x57d, 0x53c, 0x56b, 0x562, 0x580, 0x587, 0x56b, 0x56c, 0x53c,
0x578, 0x574, 0x565, 0x53c, 0x578, 0x582, 0x561, 0x576, 0x564, 0x561, 0x53c, 0x578,
0x582, 0x562, 0x578, 0x582, 0x574, 0x562, 0x561, 0x577, 0x56b, 0x53c, 0x578, 0x582,
0x57d, 0x561, 0x56f, 0x561, 0x544, 0x561, 0x56c, 0x561, 0x562, 0x578, 0x544, 0x561,
0x57a, 0x578, 0x582, 0x57f, 0x578, 0x582, 0x544, 0x561, 0x57d, 0x565, 0x580, 0x578,
0x582, 0x544, 0x562, 0x561, 0x562, 0x561, 0x576, 0x565, 0x544, 0x578, 0x563, 0x561,
0x564, 0x56b, 0x577, 0x578, 0x544, 0x578, 0x576, 0x580, 0x578, 0x57e, 0x56b, 0x561,
0x546, 0x561, 0x575, 0x580, 0x578, 0x562, 0x56b, 0x546, 0x57b, 0x561, 0x574, 0x565,
0x576, 0x561, 0x546, 0x56b, 0x561, 0x574, 0x565, 0x575, 0x546, 0x578, 0x582, 0x561,
0x56f, 0x577, 0x578, 0x57f, 0x548, 0x582, 0x561, 0x563, 0x561, 0x564, 0x578, 0x582,
0x563, 0x578, 0x582, 0x54a, 0x578, 0x580, 0x57f, 0x578, 0x20, 0x546, 0x578, 0x57e,
0x578, 0x54d, 0x561, 0x576, 0x20, 0x54f, 0x578, 0x574, 0x565, 0x54f, 0x580, 0x56b,
0x57a, 0x578, 0x56c, 0x56b, 0x539, 0x578, 0x582, 0x576, 0x56b, 0x57d, 0x54e, 0x56b,
0x576, 0x564, 0x570, 0x578, 0x582, 0x56f, 0x531, 0x564, 0x561, 0x56f, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x531, 0x576, 0x584, 0x578, 0x580, 0x56b, 0x57b, 0x531, 0x576,
0x563, 0x578, 0x582, 0x56b, 0x56c, 0x561, 0x531, 0x576, 0x57f, 0x56b, 0x563, 0x578,
0x582, 0x561, 0x531, 0x580, 0x561, 0x563, 0x578, 0x582, 0x561, 0x56b, 0x576, 0x561,
0x532, 0x578, 0x582, 0x565, 0x576, 0x578, 0x57d, 0x20, 0x531, 0x575, 0x580, 0x565,
0x57d, 0x53f, 0x561, 0x57f, 0x561, 0x574, 0x561, 0x580, 0x56f, 0x561, 0x53f, 0x578,
0x580, 0x564, 0x578, 0x57e, 0x561, 0x53a, 0x578, 0x582, 0x56a, 0x578, 0x582, 0x575,
0x53c, 0x561, 0x20, 0x54c, 0x56b, 0x578, 0x56d, 0x561, 0x544, 0x565, 0x576, 0x564,
0x578, 0x57d, 0x561, 0x54c, 0x56b, 0x578, 0x20, 0x533, 0x561, 0x56c, 0x575, 0x565,
0x563, 0x578, 0x57d, 0x54d, 0x561, 0x56c, 0x57f, 0x561, 0x54d, 0x561, 0x576, 0x20,
0x53d, 0x578, 0x582, 0x561, 0x576, 0x54d, 0x561, 0x576, 0x20, 0x53c, 0x578, 0x582,
0x56b, 0x57d, 0x54f, 0x578, 0x582, 0x56f, 0x578, 0x582, 0x574, 0x561, 0x576, 0x548,
0x582, 0x577, 0x578, 0x582, 0x561, 0x575, 0x561, 0x531, 0x580, 0x578, 0x582, 0x562,
0x561, 0x531, 0x57d, 0x578, 0x582, 0x576, 0x57d, 0x575, 0x578, 0x576, 0x531, 0x57f,
0x56b, 0x56f, 0x578, 0x56f, 0x561, 0x576, 0x532, 0x561, 0x56b, 0x575, 0x561, 0x532,
0x561, 0x56b, 0x575, 0x561, 0x20, 0x532, 0x561, 0x576, 0x564, 0x565, 0x580, 0x561,
0x57d, 0x532, 0x561, 0x580, 0x562, 0x561, 0x564, 0x578, 0x57d, 0x532, 0x565, 0x56c,
0x565, 0x574, 0x532, 0x565, 0x56c, 0x56b, 0x566, 0x532, 0x56c, 0x561, 0x576, 0x56f,
0x2d, 0x54d, 0x561, 0x562, 0x56c, 0x578, 0x576, 0x532, 0x578, 0x561, 0x20, 0x54e,
0x56b, 0x57d, 0x57f, 0x561, 0x532, 0x578, 0x563, 0x578, 0x57f, 0x561, 0x532, 0x578,
0x575, 0x57d, 0x565, 0x554, 0x565, 0x574, 0x562, 0x580, 0x56b, 0x57b, 0x20, 0x532,
0x565, 0x575, 0x53f, 0x561, 0x574, 0x57a, 0x578, 0x582, 0x20, 0x533, 0x580, 0x561,
0x576, 0x564, 0x56b, 0x53f, 0x561, 0x576, 0x56f, 0x578, 0x582, 0x576, 0x53f, 0x561,
0x580, 0x561, 0x56f, 0x561, 0x57d, 0x53f, 0x561, 0x575, 0x565, 0x576, 0x53f, 0x561,
0x575, 0x574, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580,
0x549, 0x56b, 0x56f, 0x561, 0x563, 0x578, 0x549, 0x56b, 0x578, 0x582, 0x561, 0x578,
0x582, 0x561, 0x54d, 0x575, 0x578, 0x582, 0x564, 0x561, 0x564, 0x20, 0x53d, 0x578,
0x582, 0x561, 0x580, 0x565, 0x57d, 0x53f, 0x578, 0x57d, 0x57f, 0x561, 0x20, 0x54c,
0x56b, 0x56f, 0x561, 0x554, 0x580, 0x565, 0x57d, 0x569, 0x578, 0x576, 0x53f, 0x578,
0x582, 0x575, 0x561, 0x562, 0x561, 0x53f, 0x575, 0x578, 0x582, 0x580, 0x561, 0x57d,
0x561, 0x578, 0x534, 0x565, 0x576, 0x574, 0x561, 0x580, 0x584, 0x57d, 0x570, 0x561,
0x57e, 0x576, 0x534, 0x578, 0x578, 0x582, 0x57d, 0x578, 0x576, 0x534, 0x578, 0x578,
0x582, 0x57d, 0x578, 0x576, 0x20, 0x554, 0x580, 0x56b, 0x584, 0x534, 0x565, 0x576,
0x57e, 0x565, 0x580, 0x534, 0x565, 0x57f, 0x580, 0x578, 0x575, 0x569, 0x534, 0x578,
0x574, 0x56b, 0x576, 0x56b, 0x56f, 0x561, 0x537, 0x564, 0x574, 0x578, 0x576, 0x57f,
0x578, 0x576, 0x537, 0x575, 0x580, 0x578, 0x582, 0x576, 0x565, 0x57a, 0x565, 0x54d,
0x561, 0x56c, 0x57e, 0x561, 0x564, 0x578, 0x580, 0x556, 0x578, 0x580, 0x57f, 0x20,
0x546, 0x565, 0x56c, 0x57d, 0x578, 0x576, 0x556, 0x578, 0x580, 0x57f, 0x561, 0x56c,
0x565, 0x566, 0x561, 0x533, 0x56c, 0x565, 0x575, 0x57d, 0x20, 0x532, 0x565, 0x575,
0x533, 0x578, 0x582, 0x57d, 0x20, 0x532, 0x565, 0x575, 0x533, 0x580, 0x561, 0x576,
0x564, 0x20, 0x54f, 0x575, 0x578, 0x580, 0x584, 0x533, 0x580, 0x565, 0x576, 0x561,
0x564, 0x561, 0x533, 0x57e, 0x561, 0x564, 0x565, 0x56c, 0x578, 0x582, 0x57a, 0x561,
0x533, 0x57e, 0x561, 0x57f, 0x565, 0x574, 0x561, 0x56c, 0x561, 0x533, 0x578, 0x582,
0x561, 0x575, 0x561, 0x56f, 0x56b, 0x56c, 0x533, 0x561, 0x575, 0x561, 0x576, 0x561,
0x540, 0x561, 0x56c, 0x56b, 0x586, 0x561, 0x584, 0x57d, 0x540, 0x561, 0x57e, 0x561,
0x576, 0x561, 0x537, 0x580, 0x574, 0x578, 0x57d, 0x56b, 0x56c, 0x575, 0x578, 0x53b,
0x576, 0x564, 0x56b, 0x561, 0x576, 0x561, 0x57a, 0x578, 0x56c, 0x56b, 0x57d, 0x546,
0x578, 0x584, 0x57d, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561,
0x544, 0x561, 0x580, 0x565, 0x576, 0x563, 0x578, 0x2c, 0x20, 0x53b, 0x576, 0x564,
0x56b, 0x561, 0x576, 0x561, 0x54a, 0x565, 0x57f, 0x565, 0x580, 0x57d, 0x562, 0x578,
0x582, 0x580, 0x563, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561,
0x539, 0x565, 0x56c, 0x20, 0x54d, 0x56b, 0x569, 0x56b, 0x2c, 0x20, 0x53b, 0x576,
0x564, 0x56b, 0x561, 0x576, 0x561, 0x54e, 0x56b, 0x57e, 0x565, 0x575, 0x2c, 0x20,
0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561, 0x54e, 0x56b, 0x576, 0x57d, 0x565,
0x576, 0x57d, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561, 0x54e,
0x56b, 0x576, 0x561, 0x574, 0x561, 0x56f, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b,
0x561, 0x576, 0x561, 0x53b, 0x576, 0x578, 0x582, 0x57e, 0x56b, 0x56f, 0x53b, 0x56f,
0x561, 0x56c, 0x578, 0x582, 0x56b, 0x57f, 0x543, 0x561, 0x574, 0x561, 0x575, 0x56f,
0x561, 0x54b, 0x578, 0x582, 0x576, 0x578, 0x53c, 0x578, 0x582, 0x56b, 0x57d, 0x57e,
0x56b, 0x56c, 0x544, 0x578, 0x576, 0x57f, 0x56b, 0x57d, 0x565, 0x56c, 0x578, 0x2c,
0x20, 0x53f, 0x565, 0x576, 0x57f, 0x578, 0x582, 0x56f, 0x56b, 0x53f, 0x580, 0x561,
0x56c, 0x565, 0x576, 0x564, 0x565, 0x575, 0x56f, 0x53c, 0x561, 0x20, 0x54a, 0x561,
0x57d, 0x53c, 0x56b, 0x574, 0x561, 0x53c, 0x578, 0x57d, 0x20, 0x531, 0x576, 0x57b,
0x565, 0x56c, 0x565, 0x57d, 0x53c, 0x578, 0x578, 0x582, 0x565, 0x580, 0x20, 0x54a,
0x580, 0x56b, 0x576, 0x57d, 0x565, 0x57d, 0x20, 0x554, 0x57e, 0x578, 0x580, 0x569,
0x565, 0x580, 0x544, 0x561, 0x57d, 0x565, 0x575, 0x578, 0x544, 0x561, 0x576, 0x561,
0x563, 0x578, 0x582, 0x561, 0x544, 0x561, 0x576, 0x561, 0x578, 0x582, 0x57d, 0x544,
0x561, 0x580, 0x56b, 0x563, 0x578, 0x544, 0x561, 0x580, 0x57f, 0x56b, 0x576, 0x56b,
0x56f, 0x561, 0x544, 0x561, 0x57f, 0x561, 0x574, 0x578, 0x580, 0x578, 0x57d, 0x544,
0x561, 0x566, 0x561, 0x569, 0x56c, 0x561, 0x576, 0x544, 0x565, 0x576, 0x578, 0x574,
0x56b, 0x576, 0x56b, 0x544, 0x565, 0x580, 0x56b, 0x564, 0x561, 0x544, 0x565, 0x57f,
0x56c, 0x561, 0x56f, 0x561, 0x57f, 0x56c, 0x561, 0x544, 0x565, 0x56d, 0x56b, 0x56f,
0x578, 0x544, 0x56b, 0x584, 0x565, 0x56c, 0x578, 0x576, 0x544, 0x578, 0x576, 0x56f,
0x57f, 0x578, 0x576, 0x544, 0x578, 0x576, 0x57f, 0x565, 0x57c, 0x565, 0x575, 0x544,
0x578, 0x576, 0x57f, 0x565, 0x57e, 0x56b, 0x564, 0x565, 0x578, 0x544, 0x578, 0x576,
0x57d, 0x565, 0x57c, 0x561, 0x57f, 0x546, 0x561, 0x57d, 0x561, 0x578, 0x582, 0x546,
0x575, 0x578, 0x582, 0x20, 0x545, 0x578, 0x580, 0x584, 0x546, 0x578, 0x574, 0x546,
0x578, 0x580, 0x578, 0x576, 0x575, 0x561, 0x532, 0x578, 0x575, 0x56c, 0x561, 0x2c,
0x20, 0x540, 0x575, 0x578, 0x582, 0x57d, 0x56b, 0x57d, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x534, 0x561, 0x56f, 0x578, 0x57f, 0x561, 0x540, 0x575, 0x578, 0x582, 0x57d,
0x56b, 0x57d, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x534, 0x561, 0x56f, 0x578, 0x57f,
0x561, 0x20, 0x2d, 0x20, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x546,
0x575, 0x578, 0x582, 0x20, 0x54d, 0x561, 0x56c, 0x565, 0x574, 0x2c, 0x20, 0x540,
0x575, 0x578, 0x582, 0x57d, 0x56b, 0x57d, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x534,
0x561, 0x56f, 0x578, 0x57f, 0x561, 0x546, 0x578, 0x582, 0x578, 0x582, 0x56f, 0x555,
0x56d, 0x56b, 0x576, 0x561, 0x563, 0x561, 0x54a, 0x561, 0x576, 0x561, 0x574, 0x561,
0x54a, 0x561, 0x580, 0x561, 0x574, 0x561, 0x580, 0x56b, 0x562, 0x578, 0x556, 0x56b,
0x576, 0x56b, 0x584, 0x57d, 0x54a, 0x578, 0x580, 0x57f, 0x2d, 0x585, 0x2d, 0x54a,
0x580, 0x565, 0x576, 0x57d, 0x54a, 0x578, 0x580, 0x57f, 0x2d, 0x585, 0x586, 0x2d,
0x54d, 0x583, 0x565, 0x575, 0x576, 0x54a, 0x578, 0x580, 0x57f, 0x578, 0x582, 0x20,
0x54e, 0x565, 0x56c, 0x575, 0x578, 0x582, 0x54a, 0x578, 0x582, 0x565, 0x580, 0x57f,
0x578, 0x20, 0x54c, 0x56b, 0x56f, 0x578, 0x54a, 0x578, 0x582, 0x576, 0x57f, 0x561,
0x20, 0x531, 0x580, 0x565, 0x576, 0x561, 0x57d, 0x54c, 0x561, 0x576, 0x56f, 0x56b,
0x576, 0x20, 0x53b, 0x576, 0x56c, 0x565, 0x569, 0x54c, 0x565, 0x57d, 0x56b, 0x586,
0x56b, 0x54c, 0x565, 0x57b, 0x561, 0x575, 0x576, 0x561, 0x54c, 0x565, 0x566, 0x578,
0x56c, 0x575, 0x578, 0x582, 0x57f, 0x54c, 0x56b, 0x578, 0x582, 0x20, 0x532, 0x580,
0x561, 0x576, 0x56f, 0x578, 0x582, 0x54d, 0x561, 0x576, 0x57f, 0x561, 0x580, 0x565,
0x574, 0x54d, 0x561, 0x576, 0x57f, 0x575, 0x561, 0x563, 0x578, 0x54d, 0x561, 0x576,
0x57f, 0x578, 0x20, 0x534, 0x578, 0x574, 0x56b, 0x576, 0x563, 0x578, 0x54d, 0x561,
0x576, 0x20, 0x54a, 0x561, 0x578, 0x582, 0x56c, 0x578, 0x582, 0x54d, 0x56f, 0x578,
0x580, 0x57d, 0x562, 0x56b, 0x57d, 0x578, 0x582, 0x576, 0x54d, 0x56b, 0x57f, 0x56f,
0x561, 0x54d, 0x565, 0x576, 0x20, 0x532, 0x561, 0x580, 0x57f, 0x565, 0x56c, 0x574,
0x56b, 0x54d, 0x565, 0x576, 0x569, 0x20, 0x54b, 0x578, 0x576, 0x57d, 0x54d, 0x565,
0x576, 0x569, 0x20, 0x554, 0x56b, 0x57f, 0x57d, 0x54d, 0x565, 0x576, 0x569, 0x20,
0x53c, 0x575, 0x578, 0x582, 0x57d, 0x56b, 0x561, 0x54d, 0x565, 0x576, 0x569, 0x20,
0x539, 0x578, 0x574, 0x561, 0x57d, 0x54d, 0x565, 0x576, 0x569, 0x20, 0x54e, 0x56b,
0x576, 0x57d, 0x565, 0x576, 0x569, 0x54d, 0x57e, 0x56b, 0x586, 0x569, 0x20, 0x554,
0x561, 0x580, 0x565, 0x576, 0x569, 0x54f, 0x565, 0x563, 0x578, 0x582, 0x57d, 0x56b,
0x563, 0x561, 0x56c, 0x57a, 0x561, 0x54f, 0x578, 0x582, 0x56c, 0x565, 0x54f, 0x56b,
0x56d, 0x578, 0x582, 0x561, 0x576, 0x561, 0x54f, 0x578, 0x580, 0x578, 0x576, 0x57f,
0x578, 0x539, 0x578, 0x580, 0x569, 0x578, 0x56c, 0x561, 0x54e, 0x561, 0x576, 0x56f,
0x578, 0x582, 0x57e, 0x565, 0x580, 0x548, 0x582, 0x561, 0x575, 0x569, 0x570, 0x578,
0x580, 0x57d, 0x54e, 0x56b, 0x576, 0x56b, 0x57a, 0x565, 0x563, 0x545, 0x561, 0x56f,
0x578, 0x582, 0x57f, 0x561, 0x57f, 0x554, 0x565, 0x575, 0x57d, 0x56b, 0x534, 0x565,
0x575, 0x57e, 0x56b, 0x57d, 0x534, 0x575, 0x578, 0x582, 0x574, 0x578, 0x576, 0x20,
0x564, 0x2019, 0x545, 0x578, 0x582, 0x580, 0x57e, 0x56b, 0x56c, 0x544, 0x561, 0x56f,
0x56f, 0x578, 0x582, 0x578, 0x580, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x544,
0x578, 0x578, 0x582, 0x57d, 0x578, 0x576, 0x544, 0x561, 0x56f, 0x2d, 0x544, 0x565,
0x580, 0x564, 0x578, 0x54a, 0x561, 0x56c, 0x574, 0x565, 0x580, 0x54c, 0x578, 0x57f,
0x565, 0x580, 0x561, 0x54d, 0x575, 0x578, 0x57e, 0x561, 0x54f, 0x580, 0x578, 0x56c,
0x56c, 0x54e, 0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x53c, 0x578, 0x576, 0x563, 0x575,
0x56b, 0x580, 0x531, 0x564, 0x565, 0x576, 0x531, 0x56c, 0x574, 0x561, 0x569, 0x56b,
0x531, 0x574, 0x574, 0x561, 0x576, 0x531, 0x576, 0x561, 0x564, 0x56b, 0x580, 0x531,
0x56f, 0x57f, 0x561, 0x578, 0x582, 0x531, 0x56f, 0x57f, 0x578, 0x562, 0x565, 0x531,
0x577, 0x56d, 0x561, 0x562, 0x561, 0x564, 0x531, 0x57f, 0x56b, 0x580, 0x561, 0x578,
0x582, 0x532, 0x561, 0x572, 0x564, 0x561, 0x564, 0x532, 0x561, 0x570, 0x580, 0x565,
0x575, 0x576, 0x532, 0x561, 0x584, 0x578, 0x582, 0x532, 0x561, 0x576, 0x563, 0x56f,
0x578, 0x56f, 0x532, 0x561, 0x57c, 0x576, 0x561, 0x578, 0x582, 0x56c, 0x532, 0x565,
0x575, 0x580, 0x578, 0x582, 0x569, 0x532, 0x56b, 0x577, 0x584, 0x565, 0x56f, 0x532,
0x580, 0x578, 0x582, 0x576, 0x565, 0x575, 0x549, 0x56b, 0x57f, 0x561, 0x53f, 0x578,
0x56c, 0x578, 0x574, 0x562, 0x578, 0x534, 0x561, 0x574, 0x561, 0x57d, 0x56f, 0x578,
0x57d, 0x534, 0x561, 0x584, 0x584, 0x561, 0x534, 0x56b, 0x56c, 0x56b, 0x534, 0x578,
0x582, 0x562, 0x561, 0x575, 0x534, 0x578, 0x582, 0x577, 0x561, 0x576, 0x562, 0x565,
0x556, 0x561, 0x574, 0x561, 0x563, 0x578, 0x582, 0x57d, 0x57f, 0x561, 0x533, 0x561,
0x566, 0x561, 0x540, 0x565, 0x562, 0x580, 0x578, 0x576, 0x540, 0x578, 0x577, 0x56b,
0x574, 0x56b, 0x576, 0x540, 0x578, 0x576, 0x56f, 0x578, 0x576, 0x563, 0x540, 0x578,
0x57e, 0x564, 0x53b, 0x580, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x54b, 0x561,
0x56f, 0x561, 0x580, 0x57f, 0x561, 0x54b, 0x561, 0x575, 0x57a, 0x578, 0x582, 0x580,
0x561, 0x535, 0x580, 0x578, 0x582, 0x57d, 0x561, 0x572, 0x565, 0x574, 0x554, 0x561,
0x562, 0x578, 0x582, 0x56c, 0x53f, 0x561, 0x574, 0x579, 0x561, 0x57f, 0x56f, 0x561,
0x53f, 0x561, 0x580, 0x561, 0x579, 0x56b, 0x53f, 0x561, 0x57f, 0x574, 0x561, 0x576,
0x564, 0x578, 0x582, 0x53d, 0x561, 0x576, 0x564, 0x56b, 0x563, 0x561, 0x53f, 0x561,
0x56c, 0x56f, 0x578, 0x582, 0x57f, 0x561, 0x53f, 0x580, 0x561, 0x57d, 0x576, 0x578,
0x575, 0x561, 0x580, 0x57d, 0x56f, 0x53f, 0x578, 0x582, 0x561, 0x56c, 0x561, 0x20,
0x53c, 0x578, 0x582, 0x574, 0x57a, 0x578, 0x582, 0x580, 0x53f, 0x578, 0x582, 0x579,
0x56b, 0x576, 0x563, 0x554, 0x578, 0x582, 0x57e, 0x565, 0x575, 0x569, 0x544, 0x561,
0x56f, 0x561, 0x578, 0x544, 0x561, 0x563, 0x561, 0x564, 0x561, 0x576, 0x544, 0x561,
0x56f, 0x561, 0x57d, 0x561, 0x580, 0x544, 0x561, 0x576, 0x56b, 0x56c, 0x561, 0x544,
0x561, 0x57d, 0x56f, 0x561, 0x57f, 0x546, 0x56b, 0x56f, 0x578, 0x57d, 0x56b, 0x561,
0x546, 0x578, 0x57e, 0x578, 0x56f, 0x578, 0x582, 0x566, 0x576, 0x565, 0x581, 0x56f,
0x546, 0x578, 0x57e, 0x578, 0x57d, 0x56b, 0x562, 0x56b, 0x580, 0x57d, 0x56f, 0x555,
0x574, 0x57d, 0x56f, 0x548, 0x582, 0x580, 0x561, 0x56c, 0x57d, 0x56f, 0x54a, 0x576,
0x578, 0x574, 0x57a, 0x565, 0x576, 0x54a, 0x578, 0x576, 0x57f, 0x56b, 0x561, 0x576,
0x561, 0x56f, 0x553, 0x56d, 0x565, 0x576, 0x575, 0x561, 0x576, 0x53f, 0x561, 0x57f,
0x561, 0x580, 0x53f, 0x578, 0x57d, 0x57f, 0x561, 0x576, 0x561, 0x575, 0x53f, 0x56b,
0x566, 0x56b, 0x56c, 0x578, 0x580, 0x564, 0x561, 0x537, 0x580, 0x20, 0x54c, 0x56b,
0x561, 0x564, 0x54d, 0x561, 0x56d, 0x561, 0x56c, 0x56b, 0x576, 0x54d, 0x561, 0x574,
0x561, 0x580, 0x572, 0x561, 0x576, 0x564, 0x54d, 0x565, 0x578, 0x582, 0x56c, 0x547,
0x561, 0x576, 0x570, 0x561, 0x575, 0x54d, 0x56b, 0x576, 0x563, 0x561, 0x57a, 0x578,
0x582, 0x580, 0x54d, 0x580, 0x565, 0x564, 0x576, 0x565, 0x56f, 0x578, 0x56c, 0x56b,
0x574, 0x57d, 0x56f, 0x539, 0x561, 0x575, 0x57a, 0x565, 0x575, 0x54f, 0x561, 0x577,
0x584, 0x565, 0x576, 0x564, 0x539, 0x562, 0x56b, 0x56c, 0x56b, 0x57d, 0x56b, 0x539,
0x565, 0x570, 0x580, 0x561, 0x576, 0x54f, 0x570, 0x56b, 0x574, 0x57a, 0x570, 0x578,
0x582, 0x54f, 0x578, 0x56f, 0x56b, 0x578, 0x54f, 0x578, 0x574, 0x57d, 0x56f, 0x548,
0x582, 0x56c, 0x561, 0x576, 0x20, 0x532, 0x561, 0x57f, 0x578, 0x580, 0x548, 0x582,
0x580, 0x578, 0x582, 0x574, 0x579, 0x56b, 0x548, 0x582, 0x57d, 0x57f, 0x2d, 0x546,
0x565, 0x580, 0x561, 0x54e, 0x575, 0x565, 0x576, 0x57f, 0x575, 0x561, 0x576, 0x54e,
0x56c, 0x561, 0x564, 0x56b, 0x57e, 0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x545, 0x561,
0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x54c, 0x561, 0x576, 0x563, 0x578, 0x582,
0x576, 0x535, 0x56f, 0x561, 0x57f, 0x565, 0x580, 0x56b, 0x576, 0x562, 0x578, 0x582,
0x580, 0x563, 0x535, 0x580, 0x587, 0x561, 0x576, 0x531, 0x566, 0x578, 0x580, 0x575,
0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x532, 0x565,
0x580, 0x574, 0x578, 0x582, 0x564, 0x576, 0x565, 0x580, 0x53f, 0x561, 0x576, 0x561,
0x580, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580,
0x53f, 0x561, 0x562, 0x578, 0x20, 0x54e, 0x565, 0x580, 0x564, 0x565, 0x556, 0x561,
0x580, 0x565, 0x580, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576,
0x565, 0x580, 0x544, 0x561, 0x564, 0x565, 0x575, 0x580, 0x561, 0x54c, 0x565, 0x575,
0x56f, 0x575, 0x561, 0x57e, 0x56b, 0x56f, 0x540, 0x561, 0x580, 0x561, 0x57e, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x54b, 0x578, 0x580, 0x57b, 0x56b, 0x561, 0x54d, 0x578,
0x582, 0x580, 0x562, 0x20, 0x540, 0x565, 0x572, 0x56b, 0x576, 0x565, 0x56b, 0x20,
0x56f, 0x572, 0x566, 0x56b, 0x54d, 0x569, 0x565, 0x576, 0x56c, 0x56b, 0x531, 0x564,
0x565, 0x56c, 0x561, 0x56b, 0x564, 0x561, 0x532, 0x580, 0x56b, 0x57d, 0x562, 0x565,
0x576, 0x532, 0x580, 0x578, 0x584, 0x565, 0x576, 0x20, 0x540, 0x56b, 0x56c, 0x534,
0x561, 0x580, 0x57e, 0x56b, 0x576, 0x545, 0x578, 0x582, 0x56f, 0x56c, 0x561, 0x540,
0x578, 0x562, 0x561, 0x580, 0x57f, 0x53c, 0x56b, 0x576, 0x564, 0x565, 0x574, 0x561,
0x576, 0x53c, 0x578, 0x580, 0x564, 0x20, 0x540, 0x561, 0x578, 0x582, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x544, 0x565, 0x56c, 0x562, 0x578, 0x582, 0x580, 0x576, 0x54a,
0x565, 0x580, 0x569, 0x54d, 0x56b, 0x564, 0x576, 0x565, 0x575, 0x531, 0x574, 0x57d,
0x57f, 0x565, 0x580, 0x564, 0x561, 0x574, 0x531, 0x576, 0x564, 0x578, 0x580, 0x580,
0x561, 0x531, 0x57d, 0x57f, 0x580, 0x561, 0x56d, 0x561, 0x576, 0x531, 0x569, 0x565,
0x576, 0x584, 0x532, 0x565, 0x56c, 0x563, 0x580, 0x561, 0x564, 0x532, 0x565, 0x57c,
0x56c, 0x56b, 0x576, 0x532, 0x580, 0x561, 0x57f, 0x56b, 0x57d, 0x56c, 0x561, 0x57e,
0x561, 0x532, 0x580, 0x575, 0x578, 0x582, 0x57d, 0x565, 0x56c, 0x532, 0x578, 0x582,
0x56d, 0x561, 0x580, 0x565, 0x57d, 0x57f, 0x532, 0x578, 0x582, 0x564, 0x561, 0x57a,
0x565, 0x577, 0x57f, 0x532, 0x575, 0x578, 0x582, 0x566, 0x56b, 0x576, 0x563, 0x565,
0x576, 0x554, 0x56b, 0x577, 0x576, 0x587, 0x53f, 0x578, 0x57a, 0x565, 0x576, 0x570,
0x561, 0x563, 0x565, 0x576, 0x534, 0x578, 0x582, 0x562, 0x56c, 0x56b, 0x576, 0x54b,
0x56b, 0x562, 0x580, 0x561, 0x56c, 0x569, 0x561, 0x580, 0x533, 0x565, 0x580, 0x576,
0x57d, 0x56b, 0x540, 0x565, 0x56c, 0x57d, 0x56b, 0x576, 0x56f, 0x56b, 0x544, 0x565,
0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x54d, 0x57f, 0x561, 0x574, 0x562, 0x578,
0x582, 0x56c, 0x54b, 0x565, 0x580, 0x57d, 0x56b, 0x53f, 0x561, 0x56c, 0x56b, 0x576,
0x56b, 0x576, 0x563, 0x580, 0x561, 0x564, 0x53f, 0x56b, 0x580, 0x578, 0x57e, 0x53f,
0x56b, 0x587, 0x53c, 0x56b, 0x57d, 0x561, 0x562, 0x578, 0x576, 0x53c, 0x575, 0x578,
0x582, 0x562, 0x56c, 0x575, 0x561, 0x576, 0x561, 0x53c, 0x578, 0x576, 0x564, 0x578,
0x576, 0x53c, 0x575, 0x578, 0x582, 0x584, 0x57d, 0x565, 0x574, 0x562, 0x578, 0x582,
0x580, 0x563, 0x544, 0x561, 0x564, 0x580, 0x56b, 0x564, 0x544, 0x561, 0x56c, 0x569,
0x561, 0x544, 0x561, 0x580, 0x56b, 0x565, 0x570, 0x561, 0x574, 0x576, 0x544, 0x56b,
0x576, 0x57d, 0x56f, 0x544, 0x578, 0x576, 0x561, 0x56f, 0x578, 0x544, 0x578, 0x57d,
0x56f, 0x57e, 0x561, 0x555, 0x57d, 0x56c, 0x578, 0x553, 0x561, 0x580, 0x56b, 0x566,
0x54a, 0x578, 0x564, 0x563, 0x578, 0x580, 0x56b, 0x581, 0x561, 0x54a, 0x580, 0x561,
0x570, 0x561, 0x54c, 0x56b, 0x563, 0x561, 0x540, 0x57c, 0x578, 0x574, 0x54d, 0x561,
0x574, 0x561, 0x580, 0x561, 0x54d, 0x561, 0x576, 0x20, 0x544, 0x561, 0x580, 0x56b,
0x576, 0x578, 0x54d, 0x561, 0x580, 0x561, 0x587, 0x578, 0x54d, 0x561, 0x580, 0x561,
0x57f, 0x578, 0x57e, 0x54d, 0x56b, 0x574, 0x586, 0x565, 0x580, 0x578, 0x57a, 0x578,
0x56c, 0x54d, 0x56f, 0x578, 0x57a, 0x575, 0x565, 0x54d, 0x578, 0x586, 0x56b, 0x561,
0x54d, 0x57f, 0x578, 0x56f, 0x570, 0x578, 0x56c, 0x574, 0x54f, 0x561, 0x56c, 0x56c,
0x56b, 0x576, 0x54f, 0x56b, 0x580, 0x561, 0x576, 0x561, 0x548, 0x582, 0x56c, 0x575,
0x561, 0x576, 0x578, 0x57e, 0x57d, 0x56f, 0x54e, 0x561, 0x564, 0x578, 0x582, 0x581,
0x54e, 0x561, 0x57f, 0x56b, 0x56f, 0x561, 0x576, 0x54e, 0x56b, 0x565, 0x576, 0x576,
0x561, 0x54e, 0x56b, 0x56c, 0x576, 0x575, 0x578, 0x582, 0x57d, 0x54e, 0x578, 0x56c,
0x563, 0x578, 0x563, 0x580, 0x561, 0x564, 0x54e, 0x561, 0x580, 0x577, 0x561, 0x57e,
0x561, 0x536, 0x561, 0x563, 0x580, 0x565, 0x562, 0x551, 0x575, 0x578, 0x582, 0x580,
0x56b, 0x56d, 0x531, 0x576, 0x57f, 0x561, 0x576, 0x561, 0x576, 0x561, 0x580, 0x56b,
0x57e, 0x578, 0x582, 0x549, 0x561, 0x563, 0x578, 0x57d, 0x54d, 0x578, 0x582, 0x580,
0x562, 0x20, 0x53e, 0x576, 0x576, 0x564, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572,
0x566, 0x56b, 0x53f, 0x578, 0x56f, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x53f, 0x578, 0x574, 0x578, 0x580, 0x575,
0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x53f, 0x565,
0x580, 0x563, 0x565, 0x56c, 0x565, 0x576, 0x544, 0x561, 0x567, 0x544, 0x561, 0x56c,
0x564, 0x56b, 0x57e, 0x576, 0x565, 0x580, 0x544, 0x561, 0x57e, 0x580, 0x56b, 0x56f,
0x56b, 0x578, 0x57d, 0x544, 0x561, 0x575, 0x578, 0x569, 0x54c, 0x565, 0x575, 0x578,
0x582, 0x576, 0x56b, 0x578, 0x576, 0x531, 0x57a, 0x56b, 0x561, 0x555, 0x584, 0x56c,
0x565, 0x576, 0x564, 0x532, 0x578, 0x582, 0x563, 0x565, 0x576, 0x57e, 0x56b, 0x56c,
0x549, 0x561, 0x569, 0x565, 0x574, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x549, 0x578,
0x582, 0x578, 0x582, 0x56f, 0x536, 0x561, 0x57f, 0x56f, 0x56b, 0x20, 0x56f, 0x572,
0x566, 0x56b, 0x537, 0x586, 0x561, 0x57f, 0x565, 0x556, 0x561, 0x56f, 0x561, 0x578,
0x586, 0x578, 0x556, 0x56b, 0x57b, 0x56b, 0x556, 0x578, 0x582, 0x576, 0x561, 0x586,
0x578, 0x582, 0x57f, 0x56b, 0x533, 0x561, 0x56c, 0x561, 0x57a, 0x561, 0x563, 0x578,
0x57d, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580,
0x533, 0x561, 0x574, 0x562, 0x575, 0x565, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576,
0x565, 0x580, 0x533, 0x578, 0x582, 0x561, 0x564, 0x561, 0x56c, 0x56f, 0x561, 0x576,
0x561, 0x56c, 0x533, 0x578, 0x582, 0x561, 0x574, 0x540, 0x578, 0x576, 0x578, 0x56c,
0x578, 0x582, 0x56c, 0x578, 0x582, 0x537, 0x576, 0x564, 0x565, 0x580, 0x562, 0x565,
0x580, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x53f, 0x56b, 0x580, 0x56b, 0x57f,
0x56b, 0x574, 0x561, 0x57f, 0x56b, 0x53f, 0x578, 0x57d, 0x580, 0x561, 0x565, 0x554,
0x57e, 0x561, 0x57b, 0x561, 0x56c, 0x565, 0x575, 0x576, 0x544, 0x561, 0x57b, 0x578,
0x582, 0x580, 0x578, 0x544, 0x561, 0x580, 0x56f, 0x56b, 0x566, 0x575, 0x561, 0x576,
0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x544, 0x56b, 0x564, 0x578,
0x582, 0x565, 0x575, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x546, 0x561, 0x578, 0x582,
0x580, 0x578, 0x582, 0x546, 0x56b, 0x578, 0x582, 0x565, 0x546, 0x578, 0x580, 0x586,
0x578, 0x56c, 0x56f, 0x546, 0x578, 0x582, 0x574, 0x565, 0x561, 0x54a, 0x561, 0x563,
0x578, 0x20, 0x54a, 0x561, 0x563, 0x578, 0x54a, 0x561, 0x56c, 0x561, 0x578, 0x582,
0x54a, 0x56b, 0x57f, 0x56f, 0x565, 0x57c, 0x576, 0x54a, 0x578, 0x576, 0x57a, 0x565,
0x56b, 0x54a, 0x578, 0x580, 0x57f, 0x20, 0x544, 0x578, 0x580, 0x57d, 0x562, 0x56b,
0x54c, 0x561, 0x580, 0x578, 0x57f, 0x578, 0x576, 0x563, 0x561, 0x54d, 0x561, 0x575,
0x57a, 0x561, 0x576, 0x539, 0x561, 0x56b, 0x569, 0x56b, 0x54f, 0x561, 0x580, 0x561,
0x57e, 0x561, 0x54f, 0x578, 0x576, 0x563, 0x561, 0x57f, 0x561, 0x57a, 0x578, 0x582,
0x548, 0x582, 0x565, 0x575, 0x584, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x548, 0x582,
0x578, 0x56c, 0x56b, 0x57d, 0x986, 0x9ac, 0x9bf, 0x9a1, 0x99c, 0x9be, 0x9a8, 0x98f,
0x995, 0x9cd, 0x9f0, 0x9be, 0x986, 0x9a6, 0x9cd, 0x9a6, 0x9bf, 0x99b, 0x20, 0x986,
0x9ac, 0x9be, 0x9ac, 0x9be, 0x986, 0x9b2, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x9cd, 0x99b, 0x986, 0x9b8, 0x9cd, 0x9ae, 0x9be, 0x9f0, 0x9be, 0x9ac, 0x9be, 0x9ae,
0x9be, 0x995, 0x9cb, 0x9ac, 0x9be, 0x982, 0x997, 0x9c1, 0x987, 0x9ac, 0x9c7, 0x99e,
0x9cd, 0x99c, 0x9c1, 0x9b2, 0x9ac, 0x9bf, 0x99b, 0x9be, 0x989, 0x9ac, 0x9cd, 0x9b2,
0x9be, 0x9a3, 0x9cd, 0x99f, 0x9be, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9ac, 0x9cd, 0x9f0,
0x9be, 0x99c, 0x9be, 0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x9ac, 0x9c1, 0x99c, 0x9c1, 0x9ae,
0x9cd, 0x9ac, 0x9c1, 0x9f0, 0x9be, 0x995, 0x9be, 0x987, 0x9f0, 0x9cb, 0x995, 0x9be,
0x99b, 0x9be, 0x9ac, 0x9cd, 0x9b2, 0x9be, 0x982, 0x995, 0x9be, 0x99a, 0x9c7, 0x989,
0x99f, 0x9be, 0x995, 0x9cb, 0x9a8, 0x9be, 0x995, 0x9cd, 0x9f0, 0x9c0, 0x9a1, 0x9be,
0x995, 0x9be, 0x9f0, 0x9a1, 0x9be, 0x9f0, 0x20, 0x98f, 0x99b, 0x20, 0x99b, 0x9be,
0x9b2, 0x9be, 0x9ae, 0x99c, 0x9bf, 0x9ac, 0x9c1, 0x99f, 0x9bf, 0x9a1, 0x9c1, 0x9f1,
0x9be, 0x9b2, 0x9be, 0x98f, 0x9b2, 0x20, 0x986, 0x987, 0x9af, 0x9bc, 0x9c1, 0x9a8,
0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x99f, 0x9be, 0x989, 0x9a8, 0x997, 0x9be, 0x9ac, 0x9cb,
0x9f0, 0x9cb, 0x9a3, 0x9b9, 0x9be, 0x9f0, 0x9be, 0x9f0, 0x9c7, 0x99c, 0x9cb, 0x9b9,
0x9be, 0x9a8, 0x9cd, 0x9b8, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x99c, 0x9c1, 0x9ac,
0x9be, 0x995, 0x9be, 0x9ae, 0x9cd, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x996, 0x9be, 0x9f0,
0x9cd, 0x99f, 0x9c1, 0x9ae, 0x995, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9c0, 0x995, 0x9bf,
0x982, 0x99a, 0x9be, 0x99b, 0x9be, 0x9b2, 0x9be, 0x997, 0x9cb, 0x99b, 0x9b2, 0x9bf,
0x9ac, 0x9cd, 0x9f0, 0x9bf, 0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x9b2, 0x9cb, 0x9ae, 0x9b2,
0x9c1, 0x9f1, 0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9be, 0x9b2, 0x9c1, 0x9ac, 0x9c1, 0x9ae,
0x9cd, 0x9ac, 0x9be, 0x99a, 0x9bf, 0x9b2, 0x9c1, 0x99b, 0x9be, 0x995, 0x9be, 0x9ae,
0x9be, 0x9b2, 0x9be, 0x9ac, 0x9cb, 0x9ae, 0x9be, 0x9aa, 0x9c1, 0x99f, 0x9cb, 0x9ae,
0x9be, 0x99b, 0x9c7, 0x9f0, 0x9c1, 0x98f, 0x9ae, 0x9cd, 0x9ac, 0x9be, 0x9ac, 0x9c7,
0x9a8, 0x9ae, 0x9cb, 0x997, 0x9be, 0x9a6, 0x9bf, 0x99a, 0x9c1, 0x9ae, 0x9cb, 0x9a8,
0x9f0, 0x9cb, 0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x987, 0x9f0, 0x9cb,
0x9ac, 0x9bf, 0x9a8, 0x9c7, 0x9a6, 0x99c, 0x9be, 0x9ae, 0x9c7, 0x9a8, 0x9be, 0x9a8,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9ae, 0x9c0, 0x9a8, 0x9cc, 0x986, 0x995, 0x99a, 0x9cb,
0x99f, 0x994, 0x997, 0x9be, 0x9a1, 0x9cb, 0x997, 0x9cb, 0x9aa, 0x9cb, 0x9f0, 0x9cd,
0x99f, 0x9cb, 0x2d, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x99a, 0x9be, 0x993, 0x20, 0x99f,
0x9cb, 0x9ae, 0x9c7, 0x9a4, 0x9cd, 0x9f0, 0x9bf, 0x9aa, 0x9cb, 0x9b2, 0x9bf, 0x99f,
0x9c1, 0x9a8, 0x9bf, 0x99b, 0x9f1, 0x9bf, 0x9a3, 0x9cd, 0x9a1, 0x9b9, 0x9cb, 0x995,
0x986, 0x9a1, 0x9be, 0x995, 0x98f, 0x982, 0x995, 0x9cb, 0x9f0, 0x9c7, 0x99c, 0x98f,
0x9a8, 0x997, 0x9c1, 0x987, 0x9b2, 0x9be, 0x98f, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x997,
0x9c1, 0x9f1, 0x9be, 0x986, 0x9f0, 0x9be, 0x997, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9a8,
0x9be, 0x9ac, 0x9c1, 0x9a8, 0x99b, 0x20, 0x986, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd,
0x99b, 0x995, 0x9be, 0x99f, 0x9be, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x995, 0x9be, 0x995,
0x9cb, 0x9f0, 0x9cd, 0x9a1, 0x9cb, 0x9ac, 0x9be, 0x99c, 0x9c1, 0x99c, 0x9c1, 0x987,
0x9b2, 0x9be, 0x20, 0x9f0, 0x9bf, 0x993, 0x99c, 0x9be, 0x9ae, 0x9c7, 0x9a3, 0x9cd,
0x9a1, 0x9cb, 0x99c, 0x9be, 0x9f0, 0x9bf, 0x985, 0x2019, 0x20, 0x997, 0x9be, 0x9b2,
0x9c7, 0x997, 0x9cb, 0x99b, 0x99b, 0x9be, 0x9b2, 0x9cd, 0x99f, 0x9be, 0x99b, 0x9c7,
0x9a8, 0x20, 0x99c, 0x9c1, 0x9f1, 0x9be, 0x9a8, 0x99b, 0x9c7, 0x9a8, 0x20, 0x9b2,
0x9c1, 0x987, 0x99a, 0x99f, 0x9c1, 0x995, 0x9c1, 0x9ae, 0x9a8, 0x989, 0x99a, 0x9c1,
0x986, 0x987, 0x9af, 0x9bc, 0x9be, 0x986, 0x9f0, 0x9c1, 0x9ac, 0x9be, 0x986, 0x99b,
0x9c1, 0x9a8, 0x99a, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x986, 0x99f, 0x9bf, 0x995, 0x9cb,
0x995, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x9b9, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9ac, 0x9be,
0x9b9, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ac, 0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9c7,
0x9f0, 0x9be, 0x99b, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x9ac, 0x9be, 0x9a1, 0x9cb, 0x99a,
0x9ac, 0x9c7, 0x9b2, 0x9c7, 0x9ae, 0x9ac, 0x9c7, 0x9b2, 0x9bf, 0x99c, 0x9ac, 0x9cd,
0x9b2, 0x9be, 0x982, 0x995, 0x2d, 0x99b, 0x9c7, 0x9ac, 0x9b2, 0x9cb, 0x9a8, 0x9ac,
0x9cb, 0x9f1, 0x9be, 0x20, 0x9ad, 0x9bf, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x9ac, 0x9cb,
0x997, 0x9cb, 0x99f, 0x9be, 0x9ac, 0x2019, 0x987, 0x99c, 0x9c7, 0x995, 0x9c7, 0x9ae,
0x9ac, 0x9cd, 0x9f0, 0x9bf, 0x99c, 0x20, 0x989, 0x9aa, 0x9b8, 0x9be, 0x997, 0x9f0,
0x995, 0x9c7, 0x9ae, 0x9cd, 0x9aa, 0x9cb, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c7, 0x9a3,
0x9cd, 0x9a1, 0x9c7, 0x995, 0x9c7, 0x9a8, 0x995, 0x9c1, 0x9a8, 0x995, 0x9be, 0x9f0,
0x9be, 0x995, 0x9be, 0x99b, 0x995, 0x9c7, 0x9af, 0x9bc, 0x9c7, 0x9a8, 0x995, 0x9c7,
0x987, 0x9ae, 0x9c7, 0x9a8, 0x99a, 0x9bf, 0x995, 0x9be, 0x997, 0x9cb, 0x99a, 0x9bf,
0x9b9, 0x9c1, 0x986, 0x9b9, 0x9c1, 0x9f1, 0x9be, 0x99a, 0x9bf, 0x989, 0x9a1, 0x9c7,
0x2018, 0x9a1, 0x20, 0x99c, 0x9f0, 0x9c7, 0x99c, 0x995, 0x9cb, 0x9b7, 0x9cd, 0x99f,
0x9be, 0x20, 0x9f0, 0x9bf, 0x995, 0x9be, 0x995, 0x9cd, 0x9f0, 0x9c7, 0x9b7, 0x9cd,
0x99f, 0x9a8, 0x995, 0x9c1, 0x987, 0x986, 0x9ac, 0x9be, 0x995, 0x9c1, 0x9f0, 0x9be,
0x995, 0x9be, 0x993, 0x9a1, 0x9c7, 0x9a8, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x995, 0x9b6,
0x9cd, 0x9ac, 0x9a8, 0x9a1, 0x9be, 0x993, 0x99b, 0x9a8, 0x9a1, 0x9be, 0x993, 0x99b,
0x9a8, 0x20, 0x995, 0x9cd, 0x9f0, 0x9c0, 0x995, 0x9a1, 0x9c7, 0x9a8, 0x9ad, 0x9be,
0x9f0, 0x9a1, 0x9c7, 0x99f, 0x9cd, 0x9f0, 0x9cb, 0x987, 0x99f, 0x9a1, 0x9ae, 0x9bf,
0x9a8, 0x9bf, 0x995, 0x9be, 0x98f, 0x9a1, 0x9ae, 0x9a3, 0x9cd, 0x99f, 0x9a8, 0x987,
0x9f0, 0x9c1, 0x9a8, 0x9c7, 0x9aa, 0x9c7, 0x98f, 0x9b2, 0x20, 0x99b, 0x9c7, 0x9b2,
0x9ad, 0x9c7, 0x9a1, 0x2019, 0x9f0, 0x9ab, 0x2019, 0x9f0, 0x9cd, 0x99f, 0x20, 0x9a8,
0x9c7, 0x9b2, 0x99b, 0x9a8, 0x9ab, 0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9be, 0x9b2, 0x9c7,
0x99c, 0x9be, 0x997, 0x9cd, 0x9b2, 0x9c7, 0x99a, 0x20, 0x989, 0x9aa, 0x9b8, 0x9be,
0x997, 0x9f0, 0x997, 0x9c1, 0x99b, 0x20, 0x989, 0x9aa, 0x9b8, 0x9be, 0x997, 0x9f0,
0x997, 0x9cd, 0x9f0, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x20, 0x99f, 0x9be, 0x9f0, 0x9cd,
0x995, 0x997, 0x9cd, 0x9f0, 0x9c7, 0x9a8, 0x9be, 0x9a1, 0x9be, 0x997, 0x9c1, 0x9f1,
0x9be, 0x9a1, 0x9c7, 0x9b2, 0x9c1, 0x9aa, 0x997, 0x9c1, 0x9f1, 0x9be, 0x99f, 0x9c7,
0x9ae, 0x9be, 0x9b2, 0x9be, 0x997, 0x9be, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x987,
0x9b2, 0x997, 0x9be, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9b9, 0x9c7, 0x9b2, 0x9bf,
0x9ab, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9b9, 0x9be, 0x9ad, 0x9be, 0x9a8, 0x9be, 0x9b9,
0x9be, 0x9f0, 0x9cd, 0x9ae, 0x9cb, 0x99b, 0x9bf, 0x9b2, 0x9cd, 0x9b2, 0x9cb, 0x987,
0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9aa, 0x2019, 0x9b2,
0x9bf, 0x99b, 0x9a8, 0x995, 0x9cd, 0x9b8, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ae, 0x9be, 0x9f0, 0x9c7, 0x982, 0x997,
0x9cb, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8,
0x9be, 0x9aa, 0x9bf, 0x99f, 0x9be, 0x9f0, 0x9cd, 0x99b, 0x9ac, 0x9be, 0x9f0, 0x9cd,
0x997, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8,
0x9be, 0x99f, 0x9c7, 0x9b2, 0x20, 0x99a, 0x9bf, 0x99f, 0x9c0, 0x2c, 0x20, 0x987,
0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ad, 0x9c7, 0x9ad,
0x9c7, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8,
0x9be, 0x9ad, 0x9bf, 0x9a8, 0x99a, 0x9c7, 0x9a8, 0x9c7, 0x99b, 0x2c, 0x20, 0x987,
0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x989, 0x987, 0x9a8,
0x9be, 0x9ae, 0x9c7, 0x995, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x9be, 0x987, 0x9a8, 0x9c1, 0x9ad, 0x9bf, 0x995, 0x987, 0x995,
0x9be, 0x9b2, 0x9c1, 0x987, 0x99f, 0x99c, 0x9be, 0x9ae, 0x9be, 0x987, 0x995, 0x9be,
0x99c, 0x9c1, 0x9a8, 0x9c7, 0x989, 0x9b2, 0x9c1, 0x987, 0x99b, 0x9ad, 0x9bf, 0x9b2,
0x9c7, 0x9ae, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x99a, 0x9c7, 0x9b2, 0x9cb, 0x2c, 0x20,
0x995, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9c1, 0x995, 0x9c0, 0x995, 0x9cd, 0x9f0, 0x9be,
0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x99c, 0x9bf, 0x995, 0x9b2, 0x9be, 0x20,
0x9aa, 0x9be, 0x99c, 0x9b2, 0x9bf, 0x9ae, 0x9be, 0x9b2, 0x99b, 0x20, 0x98f, 0x99e,
0x9cd, 0x99c, 0x9c7, 0x9b2, 0x99b, 0x9cd, 0x9b2, 0x2019, 0x9f1, 0x9be, 0x9f0, 0x20,
0x9aa, 0x9cd, 0x9f0, 0x9bf, 0x9a8, 0x9cd, 0x9b8, 0x9c7, 0x99b, 0x20, 0x995, 0x9cb,
0x9f1, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x9ae, 0x9c7, 0x99a, 0x9bf, 0x993,
0x9ae, 0x9be, 0x9a8, 0x9be, 0x997, 0x9c1, 0x9f1, 0x9be, 0x9ae, 0x9be, 0x9a8, 0x9be,
0x99b, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x997, 0x9cb, 0x99f, 0x9ae, 0x9be, 0x9f0, 0x9cd,
0x99f, 0x9bf, 0x9a8, 0x9bf, 0x995, 0x9ae, 0x9be, 0x99f, 0x9be, 0x9ae, 0x9cb, 0x9f0,
0x9cb, 0x99b, 0x9ae, 0x9be, 0x99c, 0x9be, 0x99f, 0x9cd, 0x200c, 0x9b2, 0x9be, 0x9a8,
0x9ae, 0x9c7, 0x9a8, 0x9cb, 0x9ae, 0x9bf, 0x9a8, 0x9c0, 0x9ae, 0x9c7, 0x9f0, 0x9bf,
0x9a1, 0x9be, 0x9ae, 0x9c7, 0x99f, 0x9b2, 0x9be, 0x995, 0x9be, 0x99f, 0x9cd, 0x200c,
0x9b2, 0x9be, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9cb, 0x20, 0x99a,
0x9bf, 0x99f, 0x9c0, 0x9ae, 0x9bf, 0x995, 0x9bf, 0x989, 0x9f1, 0x9c7, 0x9b2, 0x9a8,
0x9ae, 0x9a8, 0x9cd, 0x200c, 0x995, 0x99f, 0x9a8, 0x9ae, 0x9a3, 0x9cd, 0x99f, 0x9c7,
0x9f0, 0x9c7, 0x9f0, 0x9c0, 0x9ae, 0x9a3, 0x9cd, 0x99f, 0x9c7, 0x9ad, 0x9bf, 0x9a1,
0x9bf, 0x985, 0x2019, 0x9ae, 0x9a3, 0x9cd, 0x99f, 0x99b, 0x9c7, 0x9f0, 0x9be, 0x99f,
0x9a8, 0x9be, 0x99b, 0x9be, 0x989, 0x9a8, 0x9bf, 0x989, 0x20, 0x987, 0x9af, 0x9bc,
0x9f0, 0x9cd, 0x995, 0x9a8, 0x9cb, 0x9f0, 0x9cb, 0x9a8, 0x9cd, 0x200c, 0x9b9, 0x9be,
0x9ac, 0x9c7, 0x989, 0x9b2, 0x9be, 0x9b9, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4,
0x9f0, 0x20, 0x9a1, 0x9be, 0x995, 0x9cb, 0x99f, 0x9be, 0x99a, 0x9c7, 0x9a3, 0x9cd,
0x99f, 0x9be, 0x9f0, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x9a1,
0x9be, 0x995, 0x9cb, 0x99f, 0x9be, 0x9a8, 0x9bf, 0x989, 0x20, 0x99b, 0x9be, 0x9b2,
0x9c7, 0x9ae, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x9a1, 0x9be,
0x995, 0x9cb, 0x99f, 0x9be, 0x9a8, 0x9c1, 0x995, 0x985, 0x2019, 0x99c, 0x9bf, 0x9a8,
0x9be, 0x997, 0x9be, 0x9aa, 0x9be, 0x9a8, 0x9be, 0x9ae, 0x9be, 0x9aa, 0x9be, 0x9f0,
0x9be, 0x9ae, 0x9be, 0x9f0, 0x9bf, 0x9ac, 0x9cb, 0x9ab, 0x9bf, 0x9a8, 0x9bf, 0x995,
0x9cd, 0x9b8, 0x9aa, 0x2019, 0x9f0, 0x9cd, 0x99f, 0x2d, 0x994, 0x2d, 0x9aa, 0x9cd,
0x9f0, 0x9bf, 0x9a8, 0x9cd, 0x9b8, 0x9aa, 0x2019, 0x9f0, 0x9cd, 0x99f, 0x20, 0x985,
0x9f1, 0x20, 0x9b8, 0x9cd, 0x9aa, 0x9c7, 0x987, 0x9a8, 0x9aa, 0x9cb, 0x9f0, 0x9cd,
0x99f, 0x9cb, 0x20, 0x9ad, 0x9c7, 0x9b2, 0x9cd, 0x200c, 0x9b9, 0x9cb, 0x9aa, 0x9c1,
0x9f1, 0x9c7, 0x9f0, 0x9cd, 0x99f, 0x9cb, 0x20, 0x9f0, 0x9bf, 0x995, 0x9cb, 0x9aa,
0x9c1, 0x9a3, 0x9cd, 0x99f, 0x9be, 0x20, 0x98f, 0x9f0, 0x9bf, 0x9a8, 0x9be, 0x99b,
0x9f0, 0x9c7, 0x982, 0x995, 0x9bf, 0x9a8, 0x20, 0x987, 0x9a8, 0x9b2, 0x9c7, 0x99f,
0x9f0, 0x9c7, 0x99a, 0x9be, 0x987, 0x9ab, 0x9f0, 0x9c7, 0x99c, 0x9bf, 0x9a8, 0x9be,
0x9f0, 0x9bf, 0x99c, 0x2019, 0x9b2, 0x9bf, 0x989, 0x99f, 0x9f0, 0x9bf, 0x985, 0x2019,
0x20, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x982, 0x995, 0x9cb, 0x99b, 0x9c7, 0x9a3, 0x9cd,
0x99f, 0x9be, 0x9f0, 0x9c7, 0x9ae, 0x99b, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x997, 0x9cb, 0x99b, 0x9be, 0x9a3, 0x9cd, 0x99f, 0x9cb, 0x20, 0x9a1,
0x9cb, 0x9ae, 0x9bf, 0x982, 0x997, 0x9cb, 0x99b, 0x9be, 0x993, 0x20, 0x9aa, 0x9be,
0x989, 0x9b2, 0x9cb, 0x987, 0x99f, 0x9cb, 0x995, 0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9cb,
0x9f0, 0x9ae, 0x9bf, 0x99f, 0x99b, 0x9bf, 0x99f, 0x9cd, 0x200c, 0x200c, 0x995, 0x9be,
0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x9a5,
0x9b2, 0x9c7, 0x9ae, 0x9c7, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x99c,
0x9a8, 0x99a, 0x9cd, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x995, 0x9bf,
0x99f, 0x99b, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9b2, 0x9c1, 0x99a,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9a5,
0x9ae, 0x9be, 0x99b, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9ad, 0x9bf,
0x9a8, 0x99a, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x99b, 0x9c1, 0x987, 0x9ab, 0x9cd, 0x99f,
0x20, 0x995, 0x9be, 0x9f0, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x99f, 0x9c7, 0x997, 0x9c1,
0x99a, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9cd, 0x9aa, 0x9be, 0x9a5, 0x9cd, 0x9af, 0x9c1,
0x9b2, 0x9c7, 0x9a4, 0x9bf, 0x99c, 0x9c1, 0x9f1, 0x9be, 0x9a8, 0x9be, 0x99f, 0x2019,
0x9f0, 0x2019, 0x9a3, 0x9cd, 0x99f, 0x9cb, 0x99f, 0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9cb,
0x9b2, 0x9be, 0x9ad, 0x9c7, 0x9a8, 0x995, 0x9c1, 0x9ad, 0x9c7, 0x9f0, 0x9b9, 0x9cb,
0x9f1, 0x9be, 0x987, 0x99f, 0x9b9, 0x9f0, 0x9cd, 0x99a, 0x9f1, 0x9bf, 0x9a8, 0x9bf,
0x9aa, 0x9c7, 0x997, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x99f, 0x995,
0x9c7, 0x99b, 0x9c0, 0x9a1, 0x9c7, 0x9ad, 0x9bf, 0x99b, 0x9a1, 0x9c1, 0x9ae, 0x9cb,
0x9a3, 0x9cd, 0x99f, 0x20, 0x9a1, 0x9bf, 0x20, 0x986, 0x9f0, 0x9cd, 0x9ad, 0x9bf,
0x9b2, 0x9f0, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x200c, 0x995, 0x9c1, 0x9f1, 0x9c7, 0x9f0,
0x9c0, 0x9ae, 0x9be, 0x989, 0x99b, 0x9a8, 0x9ae, 0x9c7, 0x995, 0x9ae, 0x9c1, 0x9f0,
0x9cd, 0x9a1, 0x9c1, 0x9aa, 0x9be, 0x9ae, 0x9f0, 0x9f0, 0x9cb, 0x9a5, 0x9c7, 0x9f0,
0x9be, 0x9b8, 0x9cd, 0x9ac, 0x9cb, 0x9f1, 0x9be, 0x99f, 0x9cd, 0x9f0, 0x9cb, 0x9b2,
0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x995, 0x9b2, 0x982, 0x997, 0x9c7, 0x987, 0x9f0,
0x9ac, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9a8, 0x986, 0x9a6, 0x9c7, 0x9a8, 0x986, 0x9b2,
0x9ae, 0x9be, 0x99f, 0x9bf, 0x986, 0x9ae, 0x9cd, 0x9ae, 0x9be, 0x9a8, 0x986, 0x9a8,
0x9be, 0x9a1, 0x9bf, 0x9f0, 0x98f, 0x995, 0x9cd, 0x9af, 0x9cb, 0x99f, 0x9cd, 0x9af,
0x9be, 0x993, 0x98f, 0x995, 0x9cd, 0x9af, 0x9cb, 0x99f, 0x9ac, 0x986, 0x9b6, 0x9cd,
0x9ac, 0x9cd, 0x997, 0x9be, 0x200c, 0x9ac, 0x9be, 0x99f, 0x98f, 0x99f, 0x9f0, 0x9be,
0x989, 0x9ac, 0x9be, 0x997, 0x9a6, 0x9be, 0x9a6, 0x9ac, 0x9be, 0x9b9, 0x9f0, 0x9c7,
0x987, 0x9a8, 0x9ac, 0x9c7, 0x982, 0x995, 0x995, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x9a8,
0x9c1, 0x9b2, 0x9ac, 0x9c7, 0x987, 0x9f0, 0x9c1, 0x99f, 0x9ac, 0x9bf, 0x9b6, 0x9cd,
0x9ac, 0x9cd, 0x200c, 0x995, 0x9c7, 0x995, 0x9ac, 0x9cd, 0x9f0, 0x9c1, 0x9a8, 0x9c7,
0x987, 0x99a, 0x9bf, 0x99f, 0x9be, 0x995, 0x9b2, 0x9ae, 0x9cd, 0x9ac, 0x9cb, 0x9a1,
0x9be, 0x9ae, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x99b, 0x9a2, 0x9be, 0x995, 0x9be,
0x9a1, 0x9bf, 0x9b2, 0x9bf, 0x9a1, 0x9c1, 0x9ac, 0x9be, 0x987, 0x9a1, 0x9c1, 0x99b,
0x9be, 0x9a8, 0x9ac, 0x9c7, 0x9ab, 0x9be, 0x9ae, 0x9be, 0x997, 0x9c1, 0x9b8, 0x9cd,
0x9a4, 0x9be, 0x997, 0x9be, 0x99c, 0x9be, 0x9b9, 0x9c7, 0x9ac, 0x9cd, 0x9f0, 0x9cb,
0x9a8, 0x9b9, 0x9cb, 0x20, 0x99a, 0x9bf, 0x20, 0x9ae, 0x9bf, 0x9a8, 0x20, 0x99a,
0x9bf, 0x99f, 0x9c0, 0x9b9, 0x982, 0x20, 0x995, 0x982, 0x9b9, 0x9cb, 0x9ad, 0x9cd,
0x200c, 0x9a1, 0x987, 0x9f0, 0x9cd, 0x995, 0x9c1, 0x9a4, 0x9cd, 0x9b8, 0x9cd, 0x995,
0x99c, 0x9be, 0x995, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x9be, 0x99c, 0x9be, 0x9af, 0x9bc,
0x9be, 0x9aa, 0x9c1, 0x9f0, 0x9be, 0x99c, 0x9c7, 0x9f0, 0x9c1, 0x99c, 0x9be, 0x9b2,
0x9c7, 0x9ae, 0x995, 0x9be, 0x9ac, 0x9c1, 0x9b2, 0x995, 0x9be, 0x9ae, 0x99a, 0x9a4,
0x9cd, 0x995, 0x9be, 0x995, 0x9f0, 0x9be, 0x99a, 0x9c0, 0x995, 0x9be, 0x9a0, 0x9ae,
0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9c1, 0x996, 0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x997,
0x9be, 0x995, 0x9b2, 0x995, 0x9be, 0x9a4, 0x9be, 0x995, 0x9cd, 0x9f0, 0x9be, 0x99b,
0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x995, 0x9c1,
0x9f1, 0x9be, 0x9b2, 0x9be, 0x9b2, 0x9c1, 0x9ae, 0x9aa, 0x9c1, 0x9f0, 0x995, 0x9c1,
0x99a, 0x9bf, 0x982, 0x995, 0x9c1, 0x9f1, 0x9c7, 0x987, 0x99f, 0x9ae, 0x9be, 0x995,
0x9be, 0x993, 0x9ae, 0x9be, 0x997, 0x9be, 0x9a6, 0x9be, 0x9a8, 0x9ae, 0x9be, 0x995,
0x9be, 0x99b, 0x9be, 0x9f0, 0x9ae, 0x9be, 0x9a8, 0x9bf, 0x9b2, 0x9be, 0x9ae, 0x9c1,
0x9b8, 0x9cd, 0x995, 0x9c7, 0x99f, 0x9a8, 0x9bf, 0x995, 0x9cb, 0x99b, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x995, 0x9c1, 0x99c, 0x9a8, 0x9c7, 0x9a4,
0x9cd, 0x9b8, 0x9cd, 0x995, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x99b, 0x9bf, 0x9ac, 0x9bf,
0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x985, 0x2019,
0x9f0, 0x9c7, 0x9b2, 0x9a8, 0x9cb, 0x9ae, 0x20, 0x9aa, 0x9c7, 0x9a8, 0x9cd, 0x200c,
0x9b9, 0x9aa, 0x9cb, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x982, 0x995,
0x9aa, 0x9cd, 0x9af, 0x982, 0x9af, 0x9bc, 0x9be, 0x982, 0x995, 0x9be, 0x99f, 0x9be,
0x9f0, 0x995, 0x2019, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9c0, 0x995, 0x9c7, 0x99c,
0x9bf, 0x9b2, 0x9cb, 0x9f0, 0x9cd, 0x9a1, 0x9be, 0x9f0, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a7, 0x99b, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x99b, 0x9be, 0x9ae, 0x9be,
0x9f0, 0x995, 0x9be, 0x9a3, 0x9cd, 0x9a1, 0x99b, 0x9bf, 0x989, 0x9b2, 0x99b, 0x9be,
0x982, 0x9b9, 0x9be, 0x987, 0x99b, 0x9bf, 0x982, 0x997, 0x9be, 0x9aa, 0x9c1, 0x9f0,
0x9b8, 0x9cd, 0x9f0, 0x9c7, 0x9a1, 0x9a8, 0x9bf, 0x995, 0x9cb, 0x9b2, 0x9bf, 0x9ae,
0x9b8, 0x9cd, 0x995, 0x99f, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x9a4, 0x9be, 0x99b,
0x995, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x99f, 0x9bf, 0x9ac, 0x9b2, 0x9bf, 0x99a, 0x9bf,
0x9a4, 0x9c7, 0x9b9, 0x9f0, 0x9be, 0x9a8, 0x9a5, 0x9bf, 0x9ae, 0x9cd, 0x9ab, 0x9c1,
0x99f, 0x995, 0x9bf, 0x985, 0x2019, 0x9a4, 0x9cb, 0x9ae, 0x9cd, 0x200c, 0x9b8, 0x9cd,
0x995, 0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x99f, 0x9be, 0x9f0, 0x989, 0x9f0,
0x9c1, 0x9ae, 0x9cd, 0x200c, 0x995, 0x9bf, 0x989, 0x9b7, 0x9cd, 0x99f, 0x2d, 0x9a8,
0x9c7, 0x9f0, 0x9be, 0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a1, 0x9bf, 0x9ad, 0x9cb,
0x9b7, 0x9cd, 0x99f, 0x995, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x9a4, 0x9cd, 0x9b8,
0x9cd, 0x995, 0x9af, 0x9bc, 0x9be, 0x999, 0x9cb, 0x9a8, 0x9af, 0x9bc, 0x9c7, 0x995,
0x9be, 0x99f, 0x9c7, 0x9f0, 0x9bf, 0x9a3, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x9af,
0x9bc, 0x9c7, 0x9f0, 0x9c7, 0x9ad, 0x9be, 0x9a8, 0x98f, 0x99c, 0x9cb, 0x9f0, 0x9c7,
0x99b, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x9ae, 0x9c1, 0x9a1, 0x9be, 0x995, 0x9c7, 0x9a8,
0x9c7, 0x9f0, 0x9c0, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9f0, 0x9cd, 0x9a6,
0x9c7, 0x9ab, 0x9be, 0x9f0, 0x9cb, 0x9ae, 0x9be, 0x9a1, 0x9c7, 0x9f0, 0x9be, 0x9f0,
0x9c7, 0x987, 0x995, 0x99c, 0x9be, 0x9ad, 0x9bf, 0x995, 0x9a6, 0x995, 0x9cd, 0x9b7,
0x9bf, 0x9a3, 0x20, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x99b,
0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9b9, 0x9c7, 0x9b2, 0x9c7, 0x9a8, 0x9be,
0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x200c, 0x9b2, 0x9c0, 0x98f, 0x9a1, 0x9bf,
0x9b2, 0x9c7, 0x987, 0x9a1, 0x9ac, 0x9cd, 0x9f0, 0x9bf, 0x99a, 0x9ac, 0x9c7, 0x9a8,
0x9ac, 0x9cd, 0x9f0, 0x9cb, 0x995, 0x9c7, 0x9a8, 0x20, 0x9b9, 0x9bf, 0x9b2, 0x9a1,
0x9be, 0x9f0, 0x989, 0x987, 0x9a8, 0x987, 0x989, 0x995, 0x9cd, 0x9b2, 0x9be, 0x9b9,
0x9cb, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x9b2, 0x9bf, 0x9a3, 0x9cd, 0x9a1, 0x9ae,
0x9be, 0x9a8, 0x9b2, 0x9f0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9ae, 0x9c7,
0x9b2, 0x9ac, 0x2019, 0x9f0, 0x9cd, 0x9a3, 0x9aa, 0x9be, 0x9f0, 0x9cd, 0x9a5, 0x99a,
0x9bf, 0x9a1, 0x9a8, 0x9c0, 0x986, 0x9ae, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x9a1,
0x9be, 0x9ae, 0x986, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x9f0, 0x9be, 0x986, 0x9b7, 0x9cd,
0x99f, 0x9cd, 0x9f0, 0x9be, 0x996, 0x9be, 0x9a8, 0x98f, 0x9a5, 0x9c7, 0x9a8, 0x9cd,
0x9b8, 0x9ac, 0x9c7, 0x9b2, 0x997, 0x9cd, 0x9f0, 0x9c7, 0x9a1, 0x9ac, 0x9be, 0x9f0,
0x9cd, 0x9b2, 0x9bf, 0x9a8, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x99f, 0x9bf, 0x99b, 0x9b2,
0x9be, 0x9ad, 0x9be, 0x9ac, 0x9cd, 0x9f0, 0x9c1, 0x99b, 0x9b2, 0x9c7, 0x99b, 0x9ac,
0x9c1, 0x99a, 0x9be, 0x9f0, 0x9c7, 0x9b7, 0x9cd, 0x99f, 0x9ac, 0x9c1, 0x9a1, 0x9be,
0x9aa, 0x9c7, 0x9b7, 0x9cd, 0x99f, 0x9ac, 0x9c1, 0x99b, 0x9bf, 0x9a8, 0x99c, 0x9c7,
0x9a8, 0x99a, 0x9bf, 0x99b, 0x9bf, 0x9a8, 0x9be, 0x989, 0x995, 0x9cb, 0x9aa, 0x9c7,
0x9a8, 0x9b9, 0x9be, 0x997, 0x9c7, 0x9a8, 0x9a1, 0x9be, 0x9ac, 0x9b2, 0x9bf, 0x9a8,
0x99c, 0x9bf, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x9b2, 0x9cd, 0x99f, 0x9f0, 0x997, 0x9cb,
0x9f0, 0x9c7, 0x9a8, 0x99a, 0x9bf, 0x9b9, 0x9c7, 0x9b2, 0x99b, 0x9bf, 0x982, 0x995,
0x9bf, 0x986, 0x987, 0x9b2, 0x20, 0x985, 0x9ab, 0x20, 0x9ae, 0x9c7, 0x9a8, 0x987,
0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9ac, 0x9c1, 0x9b2, 0x99c, 0x9be, 0x9f0, 0x9cd,
0x99a, 0x9bf, 0x995, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9bf, 0x9a8, 0x997, 0x9cd, 0x9f0,
0x9be, 0x9a1, 0x995, 0x9bf, 0x9f0, 0x9cb, 0x9ad, 0x995, 0x9bf, 0x9ad, 0x9b2, 0x9bf,
0x99b, 0x9ac, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9c1, 0x9ac, 0x9c7, 0x9b2, 0x99c, 0x9be,
0x9a8, 0x9be, 0x9b2, 0x9a3, 0x9cd, 0x9a1, 0x9a8, 0x9b2, 0x9be, 0x995, 0x9cd, 0x9b8,
0x9c7, 0x9ae, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x9ae, 0x9be, 0x9a6, 0x9cd, 0x9f0,
0x9bf, 0x9a6, 0x9ae, 0x9be, 0x9b2, 0x9cd, 0x99f, 0x9be, 0x9ae, 0x9be, 0x9f0, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9b9, 0x9c7, 0x9ae, 0x9ae, 0x9bf, 0x982, 0x9b8, 0x9cd, 0x995,
0x9ae, 0x9cb, 0x9a8, 0x9be, 0x995, 0x9cb, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9cb, 0x993,
0x9b8, 0x9cd, 0x9b2, 0x9cb, 0x9aa, 0x9c7, 0x9f0, 0x9bf, 0x99b, 0x9aa, 0x9cb, 0x9a1,
0x997, 0x9cb, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x997, 0x9f0,
0x9bf, 0x997, 0x9be, 0x99b, 0x9be, 0x9ae, 0x9be, 0x9f0, 0x9be, 0x99a, 0x9be, 0x9a8,
0x20, 0x9ae, 0x9be, 0x9f0, 0x9bf, 0x9a8, 0x9cb, 0x99b, 0x9be, 0x9f0, 0x9be, 0x99c,
0x9c7, 0x9ad, 0x9cb, 0x99b, 0x9be, 0x9f0, 0x9be, 0x99f, 0x9cb, 0x9ad, 0x99b, 0x9bf,
0x9ae, 0x9cd, 0x9ab, 0x9c7, 0x9f0, 0x9cb, 0x9aa, 0x9cb, 0x9b2, 0x9b8, 0x9cd, 0x995,
0x9cb, 0x9aa, 0x9cd, 0x99c, 0x9c7, 0x99b, 0x9cb, 0x9ab, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9b7, 0x9cd, 0x99f, 0x995, 0x9b9, 0x9cb, 0x9ae, 0x9a4, 0x9c7, 0x9b2, 0x9bf, 0x9a8,
0x99f, 0x9be, 0x987, 0x9f0, 0x9c7, 0x9a8, 0x989, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8,
0x9cb, 0x9ad, 0x9cd, 0x200c, 0x9b8, 0x9cd, 0x995, 0x9ad, 0x9be, 0x9a6, 0x9c1, 0x99c,
0x9ad, 0x9c7, 0x99f, 0x9bf, 0x995, 0x9be, 0x9a8, 0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9c7,
0x9a8, 0x9be, 0x9ad, 0x9bf, 0x9b2, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x99b, 0x9ad,
0x9cb, 0x9b2, 0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9f0, 0x9be, 0x9a1, 0x9f1, 0x9be,
0x9f0, 0x99b, 0x9be, 0x993, 0x99c, 0x9be, 0x997, 0x9cd, 0x9f0, 0x9c7, 0x9ac, 0x99c,
0x9c1, 0x9f0, 0x9bf, 0x996, 0x98f, 0x9a3, 0x9cd, 0x99f, 0x9be, 0x9a8, 0x9be, 0x9a8,
0x9be, 0x9f0, 0x9bf, 0x9ad, 0x9cb, 0x99a, 0x9be, 0x997, 0x9cb, 0x99b, 0x996, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x99f, 0x9ae, 0x9be, 0x99b, 0x995, 0x9cb, 0x995, 0x9cb,
0x99b, 0x995, 0x9cb, 0x9ae, 0x9cb, 0x9f0, 0x9cb, 0x995, 0x9c7, 0x9f0, 0x997, 0x9c1,
0x9b2, 0x9c7, 0x9a8, 0x9ae, 0x9be, 0x9b9, 0x9c7, 0x9ae, 0x9be, 0x9b2, 0x9a6, 0x9cd,
0x9ac, 0x9c0, 0x9aa, 0x9ae, 0x9f0, 0x9bf, 0x99b, 0x9be, 0x99b, 0x9ae, 0x9be, 0x9af,
0x9bc, 0x9cb, 0x99f, 0x9cd, 0x99f, 0x9c7, 0x9f0, 0x9bf, 0x987, 0x989, 0x9a8, 0x9bf,
0x9af, 0x9bc, 0x9a8, 0x986, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x985, 0x995, 0x9b2,
0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9ac, 0x9cb, 0x997, 0x9c7, 0x987, 0x9a8, 0x9ad, 0x9bf,
0x9b2, 0x9c7, 0x99a, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x99a, 0x9cd, 0x99a, 0x9c1, 0x995,
0x987, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x98f, 0x9ab, 0x9c7, 0x99f, 0x9ab, 0x9be,
0x995, 0x9be, 0x993, 0x9ab, 0x9cb, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x9ab, 0x9c1, 0x9a8,
0x9be, 0x9ab, 0x9c1, 0x99f, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9be, 0x9aa, 0x9be, 0x997,
0x9cb, 0x99b, 0x997, 0x9c7, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x997, 0x9c1, 0x9f1, 0x9be, 0x9a1, 0x9b2, 0x995, 0x9be, 0x9a8, 0x9be, 0x9b2, 0x98f,
0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x9ac, 0x9be, 0x9f0, 0x9c0, 0x995, 0x9bf, 0x9f0, 0x9bf,
0x99f, 0x9bf, 0x9ae, 0x9be, 0x99f, 0x9bf, 0x995, 0x9cb, 0x99b, 0x9f0, 0x9be, 0x9af,
0x9bc, 0x9c7, 0x995, 0x9cb, 0x9f1, 0x9be, 0x99c, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9ae,
0x9be, 0x99c, 0x9c1, 0x9f0, 0x9cb, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x995, 0x99b, 0x9be,
0x99b, 0x9ae, 0x9bf, 0x9a1, 0x9f1, 0x9c7, 0x9a8, 0x9be, 0x9f0, 0x9c1, 0x9a8, 0x9bf,
0x9af, 0x9bc, 0x9cb, 0x9a8, 0x2019, 0x9f0, 0x9ab, 0x2019, 0x995, 0x9a8, 0x989, 0x9ae,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9aa, 0x9be, 0x997, 0x9cb, 0x20, 0x9aa, 0x9be, 0x997,
0x9cb, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x989, 0x9aa, 0x9bf, 0x99f, 0x995, 0x9c7, 0x987,
0x9f0, 0x9cd, 0x9a3, 0x9aa, 0x9cb, 0x9a8, 0x9aa, 0x9c7, 0x987, 0x9aa, 0x2019, 0x9f0,
0x9cd, 0x99f, 0x20, 0x9ae, 0x9cb, 0x9f0, 0x9c7, 0x99b, 0x9cd, 0x200c, 0x9ac, 0x9bf,
0x9f0, 0x9be, 0x9f0, 0x9cb, 0x99f, 0x9cb, 0x999, 0x9cd, 0x997, 0x9be, 0x99b, 0x9be,
0x987, 0x9aa, 0x9be, 0x9a8, 0x9a4, 0x9be, 0x9b9, 0x9bf, 0x9a4, 0x9bf, 0x9a4, 0x9be,
0x9f0, 0x9be, 0x9f1, 0x9be, 0x99f, 0x982, 0x997, 0x9be, 0x99f, 0x9be, 0x9aa, 0x9c1,
0x9f1, 0x9c7, 0x995, 0x9f1, 0x9be, 0x9b2, 0x9bf, 0x99b, 0x41, 0x64, 0xed, 0x73,
0x20, 0x41, 0x62, 0x65, 0x62, 0x61, 0x41, 0x72, 0x78, 0x65, 0x6c, 0x45,
0x6c, 0x20, 0x43, 0x61, 0x69, 0x72, 0x75, 0x58, 0x69, 0x62, 0x75, 0x74,
0x69, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72,
0x67, 0x75, 0x1e24, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x4c, 0x6f, 0x6d, 0xe9,
0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x78, 0x75, 0x58, 0x61, 0x6d, 0x65,
0x6e, 0x61, 0x4e, 0x69, 0x61, 0x6d, 0xe9, 0x69, 0x4e, 0x75, 0x61, 0x6b,
0x78, 0x6f, 0x74, 0x55, 0x61, 0x67, 0x61, 0x64, 0x75, 0x67, 0xfa, 0x53,
0x61, 0x6e, 0x74, 0x75, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x54, 0x72, 0xed,
0x70, 0x6f, 0x6c, 0x69, 0x54, 0xfa, 0x6e, 0x65, 0x7a, 0x43, 0xf3, 0x72,
0x64, 0x6f, 0x62, 0x61, 0x54, 0x75, 0x63, 0x75, 0x6d, 0xe1, 0x6e, 0x42,
0x61, 0x68, 0xed, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61,
0x73, 0x42, 0x6f, 0x67, 0x6f, 0x74, 0xe1, 0x43, 0x61, 0x69, 0x6d, 0xe1,
0x6e, 0x47, 0x72, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x47, 0x75, 0x61, 0x64,
0x61, 0x6c, 0x75, 0x70, 0x65, 0x4c, 0x61, 0x20, 0x48, 0x61, 0x62, 0x61,
0x6e, 0x61, 0x58, 0x61, 0x6d, 0x61, 0x69, 0x63, 0x61, 0x4c, 0x61, 0x20,
0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x63, 0x61, 0x43, 0x69, 0x75,
0x64, 0xe1, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x75,
0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x4e, 0x75,
0x65, 0x76, 0x61, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x50, 0x61, 0x6e, 0x61, 0x6d, 0xe1, 0x50, 0x75, 0x65,
0x72, 0x74, 0x75, 0x20, 0x50, 0x72, 0xed, 0x6e, 0x63, 0x69, 0x70, 0x65,
0x50, 0x75, 0x65, 0x72, 0x74, 0x75, 0x20, 0x45, 0x73, 0x70, 0x61, 0xf1,
0x61, 0x50, 0x75, 0x65, 0x72, 0x74, 0x75, 0x20, 0x52, 0x69, 0x63, 0x75,
0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67,
0x75, 0x54, 0xf3, 0x72, 0x74, 0x6f, 0x6c, 0x61, 0x41, 0x6d, 0xe1, 0x6e,
0x42, 0x61, 0x67, 0x64, 0x61, 0x67, 0x42, 0x61, 0x1e25, 0x72, 0xe9, 0x69,
0x6e, 0x42, 0x61, 0x6b, 0xfa, 0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69, 0x44,
0x61, 0x6d, 0x61, 0x73, 0x63, 0x75, 0x44, 0x75, 0x78, 0x61, 0x6e, 0x62,
0xe9, 0x48, 0x65, 0x62, 0x72, 0xf3, 0x6e, 0x43, 0x69, 0x75, 0x64, 0xe1,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d,
0x69, 0x6e, 0x68, 0x1e24, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x58, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61, 0x58, 0x61, 0x79, 0x61, 0x70,
0x75, 0x72, 0x61, 0x58, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0xe9, 0x6e,
0x4b, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0x64, 0xfa, 0x43, 0x61, 0x6c, 0x63,
0x75, 0x74, 0x61, 0x4d, 0x61, 0x63, 0xe1, 0x75, 0x4d, 0x61, 0x67, 0x61,
0x64, 0xe1, 0x6e, 0x4d, 0x61, 0x73, 0x63, 0x61, 0x74, 0x65, 0x50, 0x79,
0x6f, 0x6e, 0x67, 0x20, 0x59, 0x61, 0x6e, 0x67, 0x52, 0x69, 0x79, 0x61,
0x64, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xed, 0x6e, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x63, 0x61, 0x6e, 0x64, 0x61, 0x53, 0x65, 0xfa, 0x6c, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x54, 0x65, 0x1e25, 0x72, 0xe1, 0x6e,
0x54, 0x6f, 0x6b, 0x69, 0x75, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0xe1,
0x74, 0x6f, 0x72, 0x56, 0x69, 0x65, 0x6e, 0x74, 0x69, 0xe1, 0x6e, 0x52,
0x61, 0x6e, 0x67, 0xfa, 0x6e, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x75, 0x4c, 0x65, 0x73, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x65,
0x73, 0x43, 0x61, 0x62, 0x75, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x49,
0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6f, 0x65, 0x52,
0x65, 0x69, 0x6b, 0x69, 0x61, 0x76, 0x69, 0x6b, 0x58, 0x65, 0x6f, 0x72,
0x78, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x53, 0x61,
0x6e, 0x74, 0x61, 0x20, 0x4c, 0x65, 0x6e, 0x61, 0x41, 0x64, 0x65, 0x6c,
0x61, 0x69, 0x64, 0x61, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0xe1, 0x6e,
0x41, 0x74, 0x65, 0x6e, 0x65, 0x73, 0x42, 0x65, 0x6c, 0x67, 0x72, 0xe1,
0x75, 0x42, 0x65, 0x72, 0x6c, 0xed, 0x6e, 0x42, 0x72, 0x75, 0x78, 0x65,
0x6c, 0x65, 0x73, 0x42, 0x75, 0x63, 0x61, 0x72, 0x65, 0x73, 0x74, 0x43,
0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x75, 0x65, 0x44, 0x75, 0x62,
0x6c, 0xed, 0x6e, 0x58, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72,
0x1e24, 0xe9, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x49, 0x73, 0x6c, 0x6c,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4b, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x6e, 0x67, 0x72, 0xe1, 0x75, 0x4c, 0x69, 0x73, 0x62, 0x6f,
0x61, 0x4c, 0x69, 0x75, 0x62, 0x6c, 0x69, 0x61, 0x6e, 0x61, 0x4c, 0x6f,
0x6e, 0x64, 0x72, 0x65, 0x73, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62, 0x75,
0x72, 0x67, 0x75, 0x4d, 0xf3, 0x6e, 0x61, 0x63, 0x75, 0x4d, 0x6f, 0x73,
0x63, 0xfa, 0x50, 0x61, 0x72, 0xed, 0x73, 0x50, 0x72, 0x61, 0x67, 0x61,
0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0xed, 0x6e, 0x53, 0x6f, 0x66,
0xed, 0x61, 0x45, 0x73, 0x74, 0x6f, 0x63, 0x6f, 0x6c, 0x6d, 0x75, 0x54,
0x69, 0x72, 0x61, 0x6e, 0x61, 0x45, 0x6c, 0x20, 0x56, 0x61, 0x74, 0x69,
0x63, 0x61, 0x6e, 0x75, 0x56, 0x69, 0x65, 0x6e, 0x61, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0xe1, 0x75, 0x56, 0x61, 0x72, 0x73, 0x6f, 0x76,
0x69, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x75, 0x52, 0x65,
0x75, 0x6e, 0x69, 0xf3, 0x6e, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x46,
0x69, 0x78, 0x69, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x41, 0x62, 0x69, 0x63, 0x61, 0x6e, 0x41, 0x6b, 0x6b, 0x72, 0x61, 0x18f,
0x64, 0x64, 0x69, 0x73, 0x20, 0x18f, 0x62, 0x259, 0x62, 0x259, 0x18f, 0x6c,
0x63, 0x259, 0x7a, 0x61, 0x69, 0x72, 0x18f, 0x73, 0x6d, 0x259, 0x72, 0x259,
0x42, 0x61, 0x6e, 0x71, 0x75, 0x69, 0x42, 0x61, 0x6e, 0x63, 0x75, 0x6c,
0x42, 0x72, 0x61, 0x7a, 0x7a, 0x61, 0x76, 0x69, 0x6c, 0x51, 0x61, 0x68,
0x69, 0x72, 0x259, 0x53, 0x65, 0x75, 0x74, 0x61, 0x44, 0x61, 0x72, 0x20,
0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x61, 0x6d, 0x43, 0x69, 0x62, 0x75,
0x74, 0x69, 0x44, 0x75, 0x61, 0x6c, 0x61, 0x18f, 0x6c, 0x20, 0x18f, 0x79,
0x75, 0x6e, 0x51, 0x61, 0x62, 0x6f, 0x72, 0x6f, 0x6e, 0x59, 0x6f, 0x68,
0x61, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x71, 0x58, 0x61, 0x72, 0x74,
0x75, 0x6d, 0x4b, 0x69, 0x71, 0x61, 0x6c, 0x69, 0x4b, 0x69, 0x6e, 0x15f,
0x61, 0x73, 0x61, 0x4c, 0x61, 0x71, 0x6f, 0x73, 0x4c, 0x75, 0x62, 0x75,
0x6d, 0x62, 0x61, 0x15f, 0x69, 0x4d, 0x6f, 0x71, 0x61, 0x64, 0x69, 0x15f,
0x75, 0x4e, 0x63, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x6f, 0x75, 0x61,
0x6b, 0x15f, 0x6f, 0x74, 0x55, 0x71, 0x61, 0x64, 0x75, 0x71, 0x75, 0x53,
0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6d, 0x56, 0x69, 0x6e, 0x64, 0x68, 0x75,
0x6b, 0x41, 0x6e, 0x6b, 0x6f, 0x72, 0x61, 0x63, 0x41, 0x6e, 0x67, 0x69,
0x6c, 0x79, 0x61, 0x41, 0x6e, 0x74, 0x69, 0x71, 0x75, 0x61, 0x41, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x6e, 0x61, 0x42, 0x75, 0x65, 0x6e, 0x6f,
0x73, 0x20, 0x41, 0x79, 0x72, 0x65, 0x73, 0x52, 0x69, 0x6f, 0x78, 0x61,
0x52, 0x69, 0x6f, 0x20, 0x51, 0x61, 0x6c, 0x79, 0x65, 0x71, 0x6f, 0x73,
0x53, 0x61, 0x6e, 0x20, 0x58, 0x75, 0x61, 0x6e, 0x55, 0x15f, 0x75, 0x61,
0x79, 0x61, 0x42, 0x61, 0x68, 0x69, 0x79, 0x61, 0x42, 0x6f, 0x71, 0x6f,
0x74, 0x61, 0x42, 0x6f, 0x79, 0x73, 0x65, 0x4b, 0x65, 0x6d, 0x62, 0x72,
0x69, 0x63, 0x20, 0x4b, 0xf6, 0x72, 0x66, 0x259, 0x7a, 0x69, 0x4b, 0x61,
0x6d, 0x70, 0x6f, 0x20, 0x51, 0x72, 0x61, 0x6e, 0x64, 0x65, 0x4b, 0x61,
0x79, 0x65, 0x6e, 0x4b, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0xc7, 0x69, 0x6b,
0x61, 0x71, 0x6f, 0xc7, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x4b,
0x6f, 0x73, 0x74, 0x61, 0x20, 0x52, 0x69, 0x6b, 0x61, 0x4b, 0x75, 0x79,
0x61, 0x62, 0x61, 0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x15f, 0x61,
0x76, 0x6e, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x20, 0x4b, 0x72, 0x69,
0x6b, 0x44, 0x65, 0x74, 0x72, 0x6f, 0x79, 0x74, 0x44, 0x6f, 0x6d, 0x69,
0x6e, 0x69, 0x6b, 0x61, 0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x64, 0x74, 0x6f,
0x6e, 0x130, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x51, 0x6c, 0x65, 0x79,
0x73, 0x20, 0x4b, 0xf6, 0x72, 0x66, 0x259, 0x7a, 0x69, 0x51, 0x75, 0x7a,
0x20, 0x4b, 0xf6, 0x72, 0x66, 0x259, 0x7a, 0x69, 0x51, 0x72, 0x61, 0x6e,
0x64, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x51, 0x72, 0x65, 0x6e, 0x61, 0x64,
0x61, 0x51, 0x76, 0x61, 0x64, 0x65, 0x6c, 0x75, 0x70, 0x61, 0x51, 0x76,
0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x51, 0x75, 0x61, 0x79, 0x61,
0x6b, 0x69, 0x6c, 0x51, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x48, 0x65, 0x72,
0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6f, 0x130, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4e, 0x6f, 0x6b, 0x73, 0x4d, 0x61,
0x72, 0x65, 0x6e, 0x71, 0x6f, 0x50, 0x69, 0x74, 0x65, 0x72, 0x73, 0x62,
0x75, 0x72, 0x71, 0x56, 0x69, 0x76, 0x65, 0x79, 0x56, 0x69, 0x6e, 0x73,
0x65, 0x6e, 0x56, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x130, 0x6e, 0x75,
0x76, 0x69, 0x6b, 0x130, 0x71, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x59, 0x61,
0x6d, 0x61, 0x79, 0x6b, 0x61, 0x43, 0x75, 0x6e, 0x6f, 0x4c, 0x75, 0x69,
0x73, 0x76, 0x69, 0x6c, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0xe7, 0x65, 0x6c,
0x6c, 0x6f, 0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x6b, 0x4c,
0x61, 0x20, 0x50, 0x61, 0x73, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x63,
0x65, 0x6c, 0x65, 0x73, 0x41, 0x15f, 0x61, 0x11f, 0x131, 0x20, 0x50, 0x72,
0x69, 0x6e, 0x73, 0x20, 0x4b, 0x76, 0x61, 0x72, 0x74, 0x61, 0x6c, 0x131,
0x4d, 0x61, 0x73, 0x65, 0x79, 0x6f, 0x4d, 0x61, 0x6e, 0x61, 0x71, 0x75,
0x61, 0x4d, 0x61, 0x72, 0x69, 0x71, 0x6f, 0x74, 0x4d, 0x61, 0x7a, 0x61,
0x6c, 0x74, 0x61, 0x6e, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x4d, 0x6f,
0x6e, 0x74, 0x65, 0x72, 0x65, 0x79, 0x4d, 0x6f, 0x6e, 0x73, 0x65, 0x72,
0x61, 0x74, 0x4e, 0x79, 0x75, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x4e, 0x6f,
0x6d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x42, 0x65, 0x75, 0x6c,
0x61, 0x68, 0x2c, 0x20, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x2c,
0x20, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x4e, 0x79, 0x75, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x4f, 0x63,
0x69, 0x6e, 0x61, 0x71, 0x61, 0x130, 0x73, 0x70, 0x61, 0x6e, 0x20, 0x4c,
0x69, 0x6d, 0x61, 0x6e, 0x131, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x56,
0x65, 0x6c, 0x79, 0x6f, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x52,
0x69, 0x6b, 0x6f, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x47, 0x69,
0x72, 0x69, 0x15f, 0x69, 0x52, 0x65, 0x73, 0x69, 0x66, 0x52, 0x65, 0x63,
0x69, 0x6e, 0x61, 0x52, 0x65, 0x7a, 0x6f, 0x6c, 0x79, 0x75, 0x74, 0x53,
0x61, 0x6e, 0x74, 0x79, 0x61, 0x71, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x6f,
0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x71, 0x6f, 0x53, 0x61, 0x6e, 0x20,
0x50, 0x61, 0x75, 0x6c, 0x6f, 0x53, 0x6b, 0x6f, 0x72, 0x65, 0x73, 0x62,
0x69, 0x73, 0x75, 0x6e, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x2d, 0x42, 0x61,
0x72, 0x74, 0x65, 0x6c, 0x65, 0x6d, 0x69, 0x53, 0x65, 0x6e, 0x74, 0x20,
0x43, 0x6f, 0x6e, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x4b, 0x69, 0x74, 0x73,
0x53, 0x61, 0x6e, 0x20, 0x4c, 0xfc, 0x73, 0x69, 0x79, 0x61, 0x53, 0x61,
0x6e, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x56,
0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x53, 0x76, 0x69, 0x66, 0x74, 0x20,
0x4b, 0x75, 0x72, 0x65, 0x6e, 0x74, 0x54, 0x65, 0x71, 0x75, 0x73, 0x69,
0x71, 0x61, 0x6c, 0x70, 0x61, 0x54, 0x75, 0x6c, 0x54, 0x69, 0x78, 0x75,
0x61, 0x6e, 0x61, 0x55, 0x61, 0x79, 0x74, 0x68, 0x6f, 0x72, 0x73, 0x56,
0x69, 0x6e, 0x6e, 0x69, 0x70, 0x65, 0x71, 0x4b, 0x65, 0x79, 0x73, 0x69,
0x44, 0x65, 0x79, 0x76, 0x69, 0x73, 0x4d, 0x61, 0x6b, 0x75, 0x61, 0x72,
0x69, 0x4d, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x4d, 0x61, 0x6b, 0x20, 0x4d,
0x75, 0x72, 0x64, 0x6f, 0x53, 0x79, 0x6f, 0x76, 0x61, 0x4c, 0x6f, 0x6e,
0x71, 0x79, 0x69, 0x72, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x131, 0x41, 0x6e,
0x61, 0x64, 0x131, 0x72, 0x41, 0x15f, 0x71, 0x61, 0x62, 0x61, 0x74, 0x41,
0x74, 0x131, 0x72, 0x61, 0x75, 0x42, 0x61, 0x11f, 0x64, 0x61, 0x64, 0x42,
0x259, 0x68, 0x72, 0x65, 0x79, 0x6e, 0x42, 0x61, 0x6b, 0x131, 0x42, 0x61,
0x6e, 0x71, 0x6b, 0x6f, 0x6b, 0x42, 0x65, 0x79, 0x72, 0x75, 0x74, 0x42,
0x69, 0x15f, 0x6b, 0x65, 0x6b, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x79, 0x44,
0x259, 0x6d, 0x259, 0x15f, 0x71, 0x44, 0x259, 0x6b, 0x6b, 0x259, 0x44, 0x75,
0x62, 0x61, 0x79, 0x44, 0xfc, 0x15f, 0x259, 0x6e, 0x62, 0x259, 0x46, 0x61,
0x6d, 0x61, 0x71, 0x75, 0x73, 0x74, 0x61, 0x51, 0x259, 0x7a, 0x61, 0x48,
0x6f, 0x20, 0x15e, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x48, 0x6f, 0x6e, 0x71,
0x20, 0x4b, 0x6f, 0x6e, 0x71, 0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x43, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61, 0x43, 0x61, 0x79, 0x61, 0x70,
0x75, 0x72, 0x61, 0x59, 0x65, 0x72, 0x75, 0x73, 0x259, 0x6c, 0x69, 0x6d,
0x4b, 0x61, 0x62, 0x69, 0x6c, 0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x6b,
0x61, 0x58, 0x61, 0x6e, 0x64, 0x131, 0x71, 0x61, 0x4b, 0x259, 0x6c, 0x6b,
0x259, 0x74, 0x259, 0x4b, 0x75, 0xe7, 0x69, 0x6e, 0x71, 0x4b, 0xfc, 0x76,
0x65, 0x79, 0x74, 0x4d, 0x61, 0x71, 0x61, 0x64, 0x61, 0x6e, 0x4e, 0x69,
0x6b, 0x6f, 0x73, 0x69, 0x79, 0x61, 0x50, 0x6e, 0x6f, 0x6d, 0x20, 0x50,
0x65, 0x6e, 0x50, 0x78, 0x65, 0x6e, 0x79, 0x61, 0x6e, 0x51, 0x131, 0x7a,
0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x53, 0x61, 0x78, 0x61, 0x6c, 0x69,
0x6e, 0x53, 0x259, 0x6d, 0x259, 0x72, 0x71, 0x259, 0x6e, 0x64, 0x15e, 0x61,
0x6e, 0x78, 0x61, 0x79, 0x53, 0x69, 0x6e, 0x71, 0x61, 0x70, 0x75, 0x72,
0x53, 0x72, 0x65, 0x64, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x131, 0x6d, 0x73,
0x6b, 0x54, 0x61, 0x79, 0x62, 0x65, 0x79, 0x44, 0x61, 0x15f, 0x6b, 0x259,
0x6e, 0x64, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x55,
0x72, 0x75, 0x6d, 0xe7, 0x69, 0x56, 0x79, 0x65, 0x6e, 0x74, 0x79, 0x61,
0x6e, 0x52, 0x61, 0x6e, 0x71, 0x75, 0x6e, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x71, 0x42, 0x65, 0x72, 0x6d,
0x75, 0x64, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x4b, 0x61,
0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x46, 0x61, 0x72, 0x65,
0x72, 0x4d, 0x61, 0x64, 0x65, 0x79, 0x72, 0x61, 0x52, 0x65, 0x79, 0x6b,
0x79, 0x61, 0x76, 0x69, 0x6b, 0x43, 0x259, 0x6e, 0x75, 0x62, 0x69, 0x20,
0x43, 0x6f, 0x72, 0x63, 0x69, 0x79, 0x61, 0x4d, 0xfc, 0x71, 0x259, 0x64,
0x64, 0x259, 0x73, 0x20, 0x59, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x72,
0x69, 0x73, 0x62, 0x65, 0x79, 0x6e, 0x59, 0x75, 0x6b, 0x6c, 0x61, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x61, 0x75, 0x50, 0x65, 0x72, 0x74, 0x53,
0x69, 0x64, 0x6e, 0x65, 0x79, 0x48, 0x259, 0x15f, 0x74, 0x259, 0x72, 0x78,
0x61, 0x6e, 0x41, 0x66, 0x69, 0x6e, 0x61, 0x42, 0x65, 0x6c, 0x71, 0x72,
0x61, 0x64, 0x42, 0x72, 0xfc, 0x73, 0x73, 0x65, 0x6c, 0x42, 0x75, 0x78,
0x61, 0x72, 0x65, 0x73, 0x74, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x15f,
0x74, 0x4b, 0x69, 0x15f, 0x69, 0x6e, 0x79, 0x6f, 0x76, 0x43, 0x259, 0x62,
0x259, 0x6c, 0x69, 0x2d, 0x54, 0x61, 0x72, 0x69, 0x71, 0x47, 0x65, 0x72,
0x6e, 0x7a, 0x65, 0x79, 0x4d, 0x65, 0x6e, 0x20, 0x41, 0x64, 0x61, 0x73,
0x131, 0x130, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x43, 0x65, 0x72,
0x73, 0x69, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x71, 0x72, 0x61,
0x64, 0x4b, 0x69, 0x79, 0x65, 0x76, 0x4c, 0x79, 0x75, 0x62, 0x6c, 0x79,
0x61, 0x6e, 0x61, 0x4c, 0xfc, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72,
0x71, 0x4d, 0x61, 0x72, 0x69, 0x68, 0x61, 0x6d, 0x4d, 0x6f, 0x73, 0x6b,
0x76, 0x61, 0x50, 0x6f, 0x64, 0x71, 0x6f, 0x72, 0x69, 0x74, 0x73, 0x61,
0x50, 0x72, 0x61, 0x71, 0x61, 0x52, 0x69, 0x71, 0x61, 0x53, 0x61, 0x72,
0x61, 0x79, 0x65, 0x76, 0x6f, 0x53, 0x6b, 0x6f, 0x70, 0x79, 0x65, 0x53,
0x6f, 0x66, 0x69, 0x79, 0x61, 0x54, 0x61, 0x6c, 0x6c, 0x69, 0x6e, 0x56,
0x61, 0x64, 0x75, 0x74, 0x73, 0x56, 0x79, 0x61, 0x6e, 0x61, 0x56, 0x69,
0x6c, 0x6e, 0xfc, 0x73, 0x56, 0x6f, 0x6c, 0x71, 0x6f, 0x71, 0x72, 0x61,
0x64, 0x56, 0x61, 0x72, 0x15f, 0x61, 0x76, 0x61, 0x5a, 0x61, 0x71, 0x72,
0x65, 0x62, 0x53, 0xfc, 0x72, 0x69, 0x78, 0xc7, 0x61, 0x71, 0x6f, 0x73,
0x4d, 0x69, 0x6c, 0x61, 0x64, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x6f, 0x4b,
0x65, 0x72, 0x67, 0x65, 0x6c, 0x65, 0x6e, 0x4d, 0x61, 0x76, 0x72, 0x69,
0x6b, 0x69, 0x4f, 0x6b, 0x6c, 0x259, 0x6e, 0x64, 0x42, 0x75, 0x71, 0x61,
0x6e, 0x76, 0x69, 0x6c, 0xc7, 0x61, 0x74, 0x61, 0x6d, 0xc7, 0x75, 0x75,
0x6b, 0x50, 0x61, 0x73, 0x78, 0x61, 0x46, 0x69, 0x63, 0x69, 0x51, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x71, 0x6f, 0x73, 0x51, 0x61, 0x6d, 0x62, 0x69,
0x79, 0x65, 0x72, 0x51, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e,
0x61, 0x6c, 0x51, 0x75, 0x61, 0x6d, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62,
0xf6, 0x72, 0x69, 0x4b, 0x69, 0x72, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x79, 0x65, 0x4b, 0x76, 0x61, 0x6a, 0x61, 0x6c,
0x65, 0x79, 0x6e, 0x4d, 0x61, 0x63, 0x75, 0x72, 0x6f, 0x4d, 0x61, 0x72,
0x6b, 0x65, 0x73, 0x61, 0x73, 0x4d, 0x69, 0x64, 0x76, 0x65, 0x79, 0x50,
0x61, 0x71, 0x6f, 0x20, 0x50, 0x61, 0x71, 0x6f, 0x50, 0x6f, 0x72, 0x74,
0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x69, 0x52, 0x61, 0x72, 0x6f,
0x74, 0x6f, 0x6e, 0x71, 0x61, 0x53, 0x61, 0x79, 0x70, 0x61, 0x6e, 0x54,
0x61, 0x72, 0x61, 0x76, 0x61, 0x54, 0x6f, 0x6e, 0x71, 0x61, 0x70, 0x61,
0x74, 0x75, 0x56, 0x65, 0x79, 0x6b, 0x986, 0x9ac, 0x9bf, 0x9a6, 0x99c, 0x9be,
0x9a8, 0x986, 0x995, 0x9cd, 0x9b0, 0x9be, 0x986, 0x9a6, 0x9cd, 0x9a6, 0x9bf, 0x9b8,
0x20, 0x986, 0x9ac, 0x9be, 0x9ac, 0x9be, 0x986, 0x9b2, 0x99c, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9b0, 0x9cd, 0x9b8, 0x985, 0x9cd, 0x9af, 0x9be, 0x9b8, 0x9ae, 0x9be, 0x9b0,
0x9be, 0x9b9, 0x9c1, 0x9ac, 0x9be, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x995, 0x9cb, 0x9ac,
0x9be, 0x999, 0x9cd, 0x997, 0x9c1, 0x987, 0x9ac, 0x9be, 0x99e, 0x9cd, 0x99c, 0x9c1,
0x9b2, 0x9ac, 0x9bf, 0x9b8, 0x9cb, 0x989, 0x9ac, 0x9cd, 0x9b2, 0x9cd, 0x9af, 0x9be,
0x9a8, 0x99f, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x99c,
0x9be, 0x9ad, 0x9bf, 0x9b2, 0x9bf, 0x9ac, 0x9c1, 0x99c, 0x9c1, 0x9ae, 0x9ac, 0x9c1,
0x9b0, 0x9b9, 0x9c1, 0x995, 0x9be, 0x9af, 0x9bc, 0x9b0, 0x9cb, 0x995, 0x9be, 0x9b8,
0x9be, 0x9ac, 0x9cd, 0x9b2, 0x9be, 0x999, 0x9cd, 0x995, 0x9be, 0x9b8, 0x9c7, 0x989,
0x99f, 0x9be, 0x995, 0x9a8, 0x9be, 0x995, 0x9cd, 0x9b0, 0x9bf, 0x9a1, 0x9be, 0x995,
0x9be, 0x9b0, 0x9a6, 0x9be, 0x9b0, 0x20, 0x98f, 0x9b8, 0x20, 0x9b8, 0x9be, 0x9b2,
0x9be, 0x9ae, 0x9a1, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b2, 0x9be, 0x98f, 0x9b2, 0x20,
0x986, 0x9b9, 0x987, 0x989, 0x9a8, 0x9ab, 0x9cd, 0x9b0, 0x9c0, 0x99f, 0x9be, 0x989,
0x9a8, 0x997, 0x9cd, 0x9af, 0x9be, 0x9ac, 0x9c1, 0x9b0, 0x9a8, 0x9bf, 0x9b9, 0x9be,
0x9b0, 0x9be, 0x9b0, 0x9c7, 0x99c, 0x9cb, 0x9b9, 0x9be, 0x9a8, 0x9c7, 0x9b8, 0x9ac,
0x9be, 0x9b0, 0x9cd, 0x997, 0x995, 0x9be, 0x9ae, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x996,
0x9be, 0x9b0, 0x9cd, 0x9a4, 0x9c1, 0x9ae, 0x995, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9bf,
0x995, 0x9bf, 0x9a8, 0x9b6, 0x9be, 0x9b8, 0x9be, 0x9b2, 0x9be, 0x997, 0x9cb, 0x9b8,
0x9b2, 0x9bf, 0x9ac, 0x9cd, 0x9b0, 0x9c1, 0x9ad, 0x9bf, 0x9b2, 0x9b2, 0x9cb, 0x9ae,
0x9c7, 0x9b2, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9be, 0x9b2, 0x9c1,
0x9ac, 0x9c1, 0x9ae, 0x9cd, 0x9ac, 0x9be, 0x9b6, 0x9bf, 0x9b2, 0x9c1, 0x9b8, 0x9be,
0x995, 0x9be, 0x9ae, 0x9be, 0x9aa, 0x9c1, 0x9a4, 0x9cb, 0x9ae, 0x9be, 0x9b9, 0x9b8,
0x9c1, 0x9b0, 0x9c1, 0x985, 0x9ae, 0x9ac, 0x9be, 0x9ac, 0x9be, 0x9a8, 0x9ae, 0x9be,
0x993, 0x997, 0x9be, 0x9a6, 0x9bf, 0x9b8, 0x9c1, 0x9ae, 0x9a8, 0x9b0, 0x9cb, 0x9ad,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x987, 0x9b0, 0x9cb, 0x9ac, 0x9bf, 0x98f,
0x9a8, 0x99c, 0x9ae, 0x9c7, 0x9a8, 0x9be, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9ae,
0x9c7, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x995, 0x9b6, 0x99f, 0x993, 0x9af, 0x9bc,
0x9be, 0x9b9, 0x997, 0x9c1, 0x9a1, 0x9cb, 0x997, 0x9c1, 0x9aa, 0x9cb, 0x9b0, 0x9cd,
0x99f, 0x9cb, 0x2d, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x9b8, 0x9be, 0x993, 0x20, 0x99f,
0x9cb, 0x9ae, 0x9a4, 0x9cd, 0x9b0, 0x9bf, 0x9aa, 0x9cb, 0x9b2, 0x9bf, 0x99f, 0x9bf,
0x989, 0x9a8, 0x9bf, 0x9b8, 0x989, 0x987, 0x9a8, 0x9a7, 0x9cb, 0x995, 0x986, 0x9a1,
0x995, 0x98f, 0x9a8, 0x995, 0x9cb, 0x9b0, 0x9c7, 0x99c, 0x985, 0x9cd, 0x9af, 0x9be,
0x999, 0x9cd, 0x997, 0x9c1, 0x987, 0x9b2, 0x9be, 0x985, 0x9cd, 0x9af, 0x9be, 0x9a8,
0x9cd, 0x99f, 0x9bf, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x986, 0x9b0, 0x9be, 0x997,
0x9c1, 0x9af, 0x9bc, 0x9be, 0x987, 0x9a8, 0x9be, 0x9ac, 0x9c1, 0x9af, 0x9bc, 0x9c7,
0x9a8, 0x9cb, 0x9b8, 0x20, 0x986, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x9c7, 0x9b8, 0x995,
0x9cd, 0x9af, 0x9be, 0x99f, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9cd, 0x995, 0x9be, 0x995,
0x9b0, 0x9cd, 0x9a1, 0x9cb, 0x9ac, 0x9be, 0x9b2, 0x9be, 0x20, 0x9b0, 0x9bf, 0x993,
0x9b9, 0x9be, 0x9ae, 0x9c7, 0x9a8, 0x9cd, 0x9a1, 0x9cb, 0x99c, 0x9be, 0x9b0, 0x9bf,
0x993, 0x20, 0x997, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x997, 0x9cb, 0x9b8, 0x9b8, 0x9cd,
0x9af, 0x9be, 0x9b2, 0x99f, 0x9be, 0x9b8, 0x9be, 0x9a8, 0x20, 0x99c, 0x9c1, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x9b8, 0x9be, 0x9a8, 0x20, 0x9b2, 0x9c1, 0x987, 0x9b8, 0x99f,
0x9c1, 0x995, 0x9c1, 0x9ae, 0x9be, 0x9a8, 0x989, 0x9b6, 0x9c1, 0x9af, 0x9bc, 0x9be,
0x987, 0x9af, 0x9bc, 0x9be, 0x98f, 0x9b0, 0x9c1, 0x9ac, 0x9be, 0x986, 0x9b8, 0x9c1,
0x9a8, 0x9b8, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x9ac, 0x9be, 0x9b9, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x20, 0x9ac, 0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9c7, 0x9b0, 0x9be, 0x9b8, 0x9ac,
0x9be, 0x9b0, 0x9cd, 0x9ac, 0x9be, 0x9a1, 0x9cb, 0x99c, 0x9ac, 0x9cd, 0x9b2, 0x9be,
0x999, 0x9cd, 0x995, 0x2d, 0x9b8, 0x9be, 0x9ac, 0x9cd, 0x9b2, 0x9cb, 0x9a8, 0x9ac,
0x9cb, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ad, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9ac,
0x9af, 0x9bc, 0x99c, 0x9bf, 0x995, 0x9c7, 0x9ae, 0x9ac, 0x9cd, 0x9b0, 0x9bf, 0x99c,
0x20, 0x9ac, 0x9c7, 0x995, 0x9be, 0x9ae, 0x9cd, 0x9aa, 0x9cb, 0x20, 0x997, 0x9cd,
0x9b0, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9c7, 0x995, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x995,
0x9c1, 0x9a8, 0x995, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x995, 0x9be, 0x9b8, 0x995,
0x9be, 0x9b9, 0x9c7, 0x9a8, 0x995, 0x9be, 0x9ae, 0x9c7, 0x9a8, 0x9b6, 0x9bf, 0x995,
0x9be, 0x997, 0x9cb, 0x99a, 0x9bf, 0x9b9, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b9, 0x9c1,
0x9af, 0x9bc, 0x9be, 0x9b8, 0x9bf, 0x989, 0x9a6, 0x9be, 0x9a6, 0x20, 0x99c, 0x9c1,
0x9af, 0x9bc, 0x9be, 0x9b0, 0x9c7, 0x99c, 0x995, 0x9cb, 0x9b8, 0x9cd, 0x99f, 0x9be,
0x9b0, 0x9bf, 0x995, 0x9be, 0x995, 0x9cd, 0x9b0, 0x9bf, 0x9b8, 0x9cd, 0x99f, 0x9be,
0x9a8, 0x995, 0x9c1, 0x987, 0x9af, 0x9bc, 0x9be, 0x9ac, 0x9be, 0x995, 0x9c1, 0x9b0,
0x9be, 0x9b8, 0x9be, 0x993, 0x9a1, 0x9be, 0x9a8, 0x9ae, 0x9be, 0x9b0, 0x9cd, 0x995,
0x9b6, 0x9cd, 0x9af, 0x9be, 0x9ad, 0x9a8, 0x9a1, 0x9b8, 0x9cb, 0x9a8, 0x9a1, 0x9b8,
0x9cb, 0x9a8, 0x20, 0x995, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9a1, 0x9c7, 0x9a8, 0x9ad,
0x9be, 0x9b0, 0x9a1, 0x9c7, 0x99f, 0x9cd, 0x9b0, 0x9cb, 0x987, 0x99f, 0x9a1, 0x9cb,
0x9ae, 0x9bf, 0x9a8, 0x9bf, 0x995, 0x9be, 0x98f, 0x9a1, 0x9ae, 0x9a8, 0x9cd, 0x99f,
0x9cb, 0x9a8, 0x986, 0x987, 0x9b0, 0x9c1, 0x9a8, 0x9c7, 0x9aa, 0x9c7, 0x98f, 0x9b2,
0x20, 0x9b8, 0x9be, 0x9b2, 0x9ad, 0x9be, 0x9a6, 0x9cb, 0x9b0, 0x9ab, 0x9cb, 0x9b0,
0x9cd, 0x99f, 0x20, 0x9a8, 0x9c7, 0x9b2, 0x9b8, 0x9a8, 0x9ab, 0x9cb, 0x9b0, 0x9cd,
0x99f, 0x9be, 0x9b2, 0x9c7, 0x99c, 0x9be, 0x997, 0x9cd, 0x9b2, 0x9be, 0x9b8, 0x20,
0x9ac, 0x9c7, 0x997, 0x9c1, 0x9b8, 0x20, 0x9ac, 0x9c7, 0x997, 0x9cd, 0x9b0, 0x9cd,
0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9a4, 0x9c1, 0x9b0, 0x9cd, 0x995, 0x997,
0x9cd, 0x9b0, 0x9c7, 0x9a8, 0x9be, 0x9a1, 0x9be, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be,
0x9a6, 0x9c7, 0x9b2, 0x9cb, 0x9aa, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a4, 0x9c7,
0x9ae, 0x9be, 0x9b2, 0x9be, 0x997, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x987, 0x9af, 0x9bc,
0x9be, 0x995, 0x9bf, 0x9b2, 0x9b9, 0x9cd, 0x9af, 0x9be, 0x9b2, 0x9bf, 0x9ab, 0x9cd,
0x9af, 0x9be, 0x995, 0x9cd, 0x9b8, 0x9b9, 0x9be, 0x9b0, 0x9ae, 0x9cb, 0x9b8, 0x9bf,
0x9b2, 0x9cd, 0x9b2, 0x9cb, 0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x9be, 0x9aa, 0x9cb, 0x9b2, 0x9bf, 0x9b8, 0x9a8, 0x995, 0x9cd, 0x9b8, 0x2c,
0x20, 0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ae,
0x9be, 0x9b0, 0x9c7, 0x999, 0x9cd, 0x997, 0x9cb, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd,
0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9aa, 0x9bf, 0x99f, 0x9be, 0x9b0,
0x9b0, 0x9cd, 0x9b8, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x997, 0x2c, 0x20, 0x987, 0x9a8,
0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x99f, 0x9c7, 0x9b2, 0x9b8,
0x9bf, 0x99f, 0x9bf, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9a8, 0x9be, 0x9ad, 0x9c7, 0x9ad, 0x9c7, 0x9af, 0x9bc, 0x2c, 0x20, 0x987,
0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ad, 0x9bf, 0x9a8,
0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x9a8, 0x9c7, 0x9b8, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd,
0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x989, 0x987, 0x9a8, 0x9be, 0x9ae,
0x9cd, 0x9af, 0x9be, 0x995, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x9be, 0x987, 0x9a8, 0x9c1, 0x9ad, 0x9cd, 0x9af, 0x9be, 0x995,
0x987, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b2, 0x9c1, 0x987, 0x99f, 0x99c, 0x9c1,
0x9a8, 0x9cb, 0x9b2, 0x9c1, 0x987, 0x9b8, 0x9ad, 0x9bf, 0x9b2, 0x9ae, 0x9a8, 0x9cd,
0x99f, 0x9bf, 0x99a, 0x9c7, 0x9b2, 0x9cb, 0x2c, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd,
0x99f, 0x9be, 0x995, 0x9bf, 0x995, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c7, 0x9a8, 0x9cd,
0x9a1, 0x9be, 0x99c, 0x9bf, 0x995, 0x9b2, 0x9b8, 0x20, 0x985, 0x9cd, 0x9af, 0x9be,
0x99e, 0x9cd, 0x99c, 0x9c7, 0x9b2, 0x9c7, 0x9b8, 0x9b2, 0x9cb, 0x9af, 0x9bc, 0x9be,
0x9b0, 0x20, 0x9aa, 0x9cd, 0x9b0, 0x9bf, 0x9a8, 0x9cd, 0x9b8, 0x9c7, 0x9b8, 0x20,
0x995, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x9ae, 0x9be,
0x9b8, 0x9c7, 0x9af, 0x9bc, 0x9cb, 0x9ae, 0x9be, 0x9a8, 0x9be, 0x997, 0x9c1, 0x9af,
0x9bc, 0x9be, 0x9ae, 0x9be, 0x9a8, 0x9be, 0x989, 0x9b8, 0x9ae, 0x9be, 0x9b0, 0x9bf,
0x997, 0x9cb, 0x9ae, 0x9be, 0x9b0, 0x99f, 0x9bf, 0x9a8, 0x9bf, 0x995, 0x9ae, 0x9be,
0x9a4, 0x9be, 0x9ae, 0x9cb, 0x9b0, 0x9b8, 0x9ae, 0x9be, 0x99c, 0x9be, 0x99f, 0x9b2,
0x9be, 0x9a8, 0x9ae, 0x9c7, 0x9a8, 0x9cb, 0x9ae, 0x9bf, 0x9a8, 0x9bf, 0x9ae, 0x9c7,
0x9b0, 0x9bf, 0x9a1, 0x9be, 0x9ae, 0x9c7, 0x99f, 0x9b2, 0x9be, 0x995, 0x9be, 0x99f,
0x9b2, 0x9be, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9cb, 0x20, 0x9b8,
0x9bf, 0x99f, 0x9bf, 0x9ae, 0x9bf, 0x995, 0x9c1, 0x9b2, 0x9a8, 0x9ae, 0x999, 0x9cd,
0x995, 0x99f, 0x9cb, 0x9a8, 0x9ae, 0x9a8, 0x9cd, 0x99f, 0x9c7, 0x9b0, 0x9bf, 0x9ae,
0x9a8, 0x9cd, 0x99f, 0x9c7, 0x9ad, 0x9bf, 0x9a1, 0x9bf, 0x993, 0x9ae, 0x9a8, 0x9cd,
0x9a4, 0x9b8, 0x9c7, 0x9b0, 0x9be, 0x9a4, 0x9a8, 0x9be, 0x9b8, 0x9be, 0x989, 0x9a8,
0x9bf, 0x989, 0x987, 0x9af, 0x9bc, 0x9b0, 0x9cd, 0x995, 0x9a8, 0x9b0, 0x9cb, 0x9a8,
0x9cd, 0x200c, 0x9b9, 0x9be, 0x9ac, 0x9c7, 0x989, 0x9b2, 0x9be, 0x2c, 0x20, 0x989,
0x9a4, 0x9cd, 0x9a4, 0x9b0, 0x20, 0x9a1, 0x9be, 0x995, 0x9cb, 0x99f, 0x9be, 0x9ae,
0x9a7, 0x9cd, 0x9af, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9b0, 0x20, 0x9a1,
0x9be, 0x995, 0x9cb, 0x99f, 0x9be, 0x9a8, 0x9bf, 0x989, 0x20, 0x9b8, 0x9be, 0x9b2,
0x9c7, 0x9ae, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9b0, 0x20, 0x9a1, 0x9be,
0x995, 0x9cb, 0x99f, 0x9be, 0x993, 0x99c, 0x9bf, 0x9a8, 0x9be, 0x997, 0x9be, 0x9aa,
0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x9ac, 0x9cb, 0x9aa,
0x9cb, 0x9b0, 0x9cd, 0x99f, 0x2d, 0x985, 0x9b9, 0x2d, 0x9aa, 0x9cd, 0x9b0, 0x9bf,
0x9a8, 0x9cd, 0x9b8, 0x9aa, 0x9cb, 0x9b0, 0x9cd, 0x99f, 0x20, 0x985, 0x9ab, 0x20,
0x9b8, 0x9cd, 0x9aa, 0x9c7, 0x9a8, 0x9aa, 0x9cb, 0x9b0, 0x9cd, 0x9a4, 0x9cb, 0x20,
0x9ad, 0x9c7, 0x9b2, 0x9cd, 0x200c, 0x9b9, 0x9cb, 0x9aa, 0x9c1, 0x9af, 0x9bc, 0x9c7,
0x9b0, 0x9cd, 0x9a4, 0x9cb, 0x20, 0x9b0, 0x9bf, 0x995, 0x9cb, 0x9aa, 0x9c1, 0x9a8,
0x9cd, 0x99f, 0x9be, 0x20, 0x986, 0x9b0, 0x9c7, 0x9a8, 0x9be, 0x9b8, 0x9b0, 0x200d,
0x9cd, 0x9af, 0x9be, 0x999, 0x9cd, 0x995, 0x9bf, 0x9a8, 0x20, 0x987, 0x9a8, 0x9b2,
0x9c7, 0x99f, 0x9b0, 0x9c7, 0x9b8, 0x9bf, 0x9ab, 0x9c7, 0x9b0, 0x9c7, 0x99c, 0x9bf,
0x9a8, 0x9be, 0x9b0, 0x9c7, 0x9b8, 0x9cb, 0x9b2, 0x9c1, 0x99f, 0x9b0, 0x9bf, 0x993,
0x20, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x999, 0x9cd, 0x995, 0x9cb, 0x9b8, 0x9c7, 0x9a8,
0x99f, 0x9c1, 0x9b0, 0x9c7, 0x9ae, 0x9b8, 0x9be, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x997, 0x9cb, 0x9b8, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x99f, 0x9cb,
0x20, 0x9a1, 0x9cb, 0x9ae, 0x9bf, 0x982, 0x997, 0x9cb, 0x9b8, 0x9be, 0x993, 0x20,
0x9aa, 0x9be, 0x989, 0x9b2, 0x9cb, 0x987, 0x99f, 0x9cd, 0x99f, 0x995, 0x9c1, 0x9af,
0x9bc, 0x9cb, 0x9b0, 0x99f, 0x9c1, 0x9b0, 0x9ae, 0x9bf, 0x99f, 0x9b6, 0x9bf, 0x99f,
0x995, 0x9be, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x9ac, 0x9be, 0x9b0, 0x9cd,
0x9a5, 0x9c7, 0x9b2, 0x9c7, 0x9ae, 0x9bf, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x20,
0x99c, 0x9a8, 0x9cd, 0x9b8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x995,
0x9bf, 0x99f, 0x9b8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x9b2, 0x9c1,
0x9b8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20,
0x9a5, 0x9ae, 0x9be, 0x9b8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x9ad,
0x9bf, 0x9a8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9b8, 0x9c1, 0x987, 0x9ab, 0x9cd,
0x99f, 0x20, 0x995, 0x9be, 0x9b0, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9a4, 0x9c7, 0x997,
0x9c1, 0x9b8, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9aa, 0x9be, 0x9a5, 0x9c1, 0x9b2, 0x9bf,
0x9a4, 0x9bf, 0x99c, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x99f, 0x9cb, 0x9b0,
0x9a8, 0x9cd, 0x99f, 0x9cb, 0x99f, 0x9b0, 0x99f, 0x9cb, 0x9b2, 0x9be, 0x9ad, 0x9cd,
0x9af, 0x9be, 0x999, 0x9cd, 0x995, 0x9c1, 0x9ad, 0x9be, 0x9b0, 0x9b9, 0x9cb, 0x9af,
0x9bc, 0x9be, 0x987, 0x99f, 0x9b9, 0x9b0, 0x9cd, 0x9b8, 0x989, 0x987, 0x9a8, 0x9bf,
0x9aa, 0x9c7, 0x997, 0x987, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x99f,
0x995, 0x9c7, 0x987, 0x9b8, 0x9bf, 0x9a1, 0x9c7, 0x9ad, 0x9bf, 0x9b8, 0x9a1, 0x9c1,
0x9ae, 0x9a8, 0x9cd, 0x99f, 0x20, 0x9a1, 0x9bf, 0x2019, 0x989, 0x9b0, 0x9ad, 0x9bf,
0x9b2, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x995, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9cd, 0x9af,
0x9be, 0x9b0, 0x9bf, 0x9ae, 0x9b8, 0x9cb, 0x9a8, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x995,
0x9ae, 0x9c1, 0x9b0, 0x9cd, 0x9a1, 0x9cb, 0x9aa, 0x9be, 0x9b2, 0x9ae, 0x9be, 0x9b0,
0x9b0, 0x9a5, 0x9c7, 0x9b0, 0x9be, 0x9b8, 0x9bf, 0x993, 0x9af, 0x9bc, 0x9be, 0x99f,
0x9cd, 0x9b0, 0x9b2, 0x9ad, 0x9b8, 0x9cd, 0x99f, 0x9cb, 0x995, 0x9b2, 0x99e, 0x9cd,
0x99c, 0x9bf, 0x9af, 0x9bc, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x981, 0x9a8, 0x986,
0x9b9, 0x9a6, 0x9c7, 0x9a8, 0x985, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9be, 0x9a1, 0x9bf,
0x9b0, 0x986, 0x995, 0x99f, 0x9be, 0x989, 0x986, 0x995, 0x99f, 0x9cb, 0x9ac, 0x9c7,
0x986, 0x9b6, 0x997, 0x9be, 0x9ac, 0x9be, 0x9a4, 0x985, 0x9a4, 0x9bf, 0x9b0, 0x9be,
0x989, 0x9ac, 0x9be, 0x9b9, 0x9be, 0x9b0, 0x9bf, 0x9a8, 0x9ac, 0x9cd, 0x9af, 0x9be,
0x982, 0x995, 0x995, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x9a8, 0x9cc, 0x9b2, 0x9ac, 0x9c7,
0x987, 0x9b0, 0x9c1, 0x99f, 0x9ac, 0x9bf, 0x9b6, 0x995, 0x9c7, 0x995, 0x9ac, 0x9cd,
0x9b0, 0x9c1, 0x9a8, 0x9c7, 0x987, 0x99a, 0x9bf, 0x9a4, 0x9be, 0x9a6, 0x9be, 0x9ae,
0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x9b8, 0x9a6, 0x9bf, 0x9b2, 0x9bf, 0x9a6, 0x9c1,
0x9ac, 0x9be, 0x987, 0x9a6, 0x9c1, 0x9b6, 0x9be, 0x9a8, 0x9ac, 0x9c7, 0x9ab, 0x9be,
0x9ae, 0x9be, 0x997, 0x9be, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9b9, 0x9c7, 0x9ac, 0x9cd,
0x9b0, 0x9cb, 0x9a8, 0x9b9, 0x9cb, 0x20, 0x99a, 0x9bf, 0x20, 0x9ae, 0x9bf, 0x9a8,
0x20, 0x9b6, 0x9b9, 0x9b0, 0x9b9, 0x9cb, 0x9ad, 0x9cd, 0x9a1, 0x987, 0x9b0, 0x995,
0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x99c, 0x9be, 0x995, 0x9be, 0x9b0, 0x9cd, 0x9a4,
0x9be, 0x99c, 0x9af, 0x9bc, 0x9be, 0x9aa, 0x9c1, 0x9b0, 0x9be, 0x99c, 0x9c7, 0x9b0,
0x9c1, 0x99c, 0x9be, 0x9b2, 0x9c7, 0x9ae, 0x995, 0x9be, 0x9ae, 0x99a, 0x9be, 0x99f,
0x995, 0x9be, 0x995, 0x9b0, 0x9be, 0x99a, 0x9bf, 0x995, 0x9be, 0x9a0, 0x9ae, 0x9be,
0x9a8, 0x9cd, 0x9a1, 0x9c1, 0x996, 0x9be, 0x9a8, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x997, 0x9be, 0x995, 0x9cb, 0x9b2, 0x995, 0x9be, 0x9a4, 0x9be, 0x995, 0x9cd, 0x9b0,
0x9be, 0x9b8, 0x9a8, 0x9cb, 0x987, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd, 0x9b8, 0x9cd,
0x995, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b2, 0x9be, 0x9b2, 0x9be, 0x9ae, 0x9aa,
0x9c1, 0x9b0, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x9a4, 0x9ae, 0x9cd, 0x9af, 0x9be,
0x995, 0x9be, 0x993, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x997, 0x9be, 0x9a1, 0x9be, 0x9a8,
0x9ae, 0x9be, 0x995, 0x9be, 0x9b8, 0x9b8, 0x9be, 0x9b0, 0x9ae, 0x9cd, 0x9af, 0x9be,
0x9a8, 0x9bf, 0x9b2, 0x9be, 0x9ae, 0x9be, 0x9b8, 0x995, 0x99f, 0x9a8, 0x9bf, 0x995,
0x9cb, 0x9b8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9ad, 0x995, 0x9c1, 0x9af, 0x9bc,
0x9c7, 0x9a4, 0x9b8, 0x9cd, 0x9a8, 0x995, 0x9a8, 0x9ad, 0x9cb, 0x9b8, 0x9bf, 0x9ac,
0x9bf, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x993, 0x9ae, 0x9cd, 0x9b8, 0x9cd, 0x995,
0x993, 0x9b0, 0x9be, 0x9b2, 0x9a8, 0x9ae, 0x20, 0x9aa, 0x9c7, 0x9a8, 0x9aa, 0x9a8,
0x9cd, 0x99f, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9cd, 0x9af, 0x9be, 0x995, 0x9aa,
0x9bf, 0x9af, 0x9bc, 0x982, 0x987, 0x9af, 0x9bc, 0x982, 0x995, 0x9be, 0x9a4, 0x9be,
0x9b0, 0x995, 0x9cb, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9a8, 0x9af, 0x9bc, 0x995, 0x9bf,
0x99c, 0x9bf, 0x9b2, 0x9b0, 0x9cd, 0x9a1, 0x9be, 0x9b0, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a7, 0x9b8, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9b8, 0x9ae, 0x9b0, 0x996,
0x9a8, 0x9cd, 0x9a6, 0x9b8, 0x9bf, 0x993, 0x9b2, 0x9b8, 0x9be, 0x982, 0x9b9, 0x9be,
0x987, 0x9b8, 0x9bf, 0x999, 0x9cd, 0x997, 0x9be, 0x9aa, 0x9c1, 0x9b0, 0x9b8, 0x9cd,
0x9b0, 0x9c7, 0x9a6, 0x9a8, 0x9c7, 0x995, 0x9cb, 0x9b2, 0x9af, 0x9bc, 0x9ae, 0x9b8,
0x9cd, 0x995, 0x9a4, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x9a4, 0x9be, 0x9b8, 0x996,
0x9a8, 0x9cd, 0x9a6, 0x9b8, 0x9bf, 0x9ac, 0x9bf, 0x9b2, 0x9bf, 0x9b6, 0x9bf, 0x9a4,
0x9c7, 0x9b9, 0x9c7, 0x9b0, 0x9be, 0x9a8, 0x99f, 0x9cb, 0x995, 0x9bf, 0x993, 0x9a4,
0x9cb, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x9a4,
0x9be, 0x9b0, 0x989, 0x9b0, 0x9c1, 0x9ae, 0x995, 0x9bf, 0x989, 0x9b8, 0x9cd, 0x9a4,
0x2d, 0x20, 0x9a8, 0x9c7, 0x9b0, 0x9be, 0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9a8,
0x9a4, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9a8, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a6, 0x9bf,
0x9ad, 0x9b8, 0x9cd, 0x9a4, 0x9cb, 0x995, 0x987, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1,
0x99f, 0x9b8, 0x9cd, 0x995, 0x9b0, 0x9c7, 0x999, 0x9cd, 0x997, 0x9c1, 0x9a8, 0x987,
0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x99f, 0x9c7, 0x9b0, 0x9bf, 0x9a8, 0x9ac, 0x9be,
0x9b0, 0x9cd, 0x997, 0x987, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x9be, 0x9ad, 0x9be, 0x9a8,
0x986, 0x99c, 0x9cb, 0x9b0, 0x9c7, 0x9b8, 0x9ac, 0x9be, 0x9b0, 0x9ae, 0x9c1, 0x9a1,
0x9be, 0x995, 0x9be, 0x9a8, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad,
0x9be, 0x9b0, 0x9cd, 0x9a6, 0x9ab, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9cb, 0x9ae, 0x9be,
0x9a1, 0x9c7, 0x987, 0x9b0, 0x9be, 0x9b0, 0x9bf, 0x995, 0x99c, 0x9be, 0x9ad, 0x9bf,
0x995, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x99c, 0x9b0, 0x9cd, 0x99c,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x9b9,
0x9c7, 0x9b2, 0x9c7, 0x9a8, 0x9be, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9a8, 0x9b2, 0x9c0,
0x98f, 0x9cd, 0x9af, 0x9be, 0x9a1, 0x9c7, 0x9b2, 0x9c7, 0x987, 0x9a1, 0x9ac, 0x9cd,
0x9b0, 0x9bf, 0x9b8, 0x9ac, 0x9c7, 0x9a8, 0x9ac, 0x9cd, 0x9b0, 0x9cb, 0x995, 0x9c7,
0x9a8, 0x20, 0x9b9, 0x9bf, 0x9b2, 0x9a1, 0x9be, 0x9b0, 0x989, 0x987, 0x9a8, 0x9b9,
0x9cb, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x99f, 0x9b2, 0x9bf, 0x9a8, 0x9a1, 0x9c7, 0x9ae,
0x9cd, 0x9af, 0x9be, 0x9a8, 0x9b2, 0x9b0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993,
0x9ae, 0x9c7, 0x9b2, 0x9ac, 0x9cb, 0x9b0, 0x9cd, 0x9a8, 0x9aa, 0x9be, 0x9b0, 0x9cd,
0x9a5, 0x9b8, 0x9bf, 0x9a1, 0x9a8, 0x9bf, 0x986, 0x9ae, 0x9b8, 0x9cd, 0x99f, 0x9be,
0x9b0, 0x9a1, 0x9be, 0x9ae, 0x985, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9cb,
0x9b0, 0x9be, 0x986, 0x9b8, 0x99f, 0x9cd, 0x9b0, 0x9be, 0x996, 0x9be, 0x9a8, 0x9ac,
0x9c7, 0x9b2, 0x997, 0x9cd, 0x9b0, 0x9c7, 0x9a1, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x9b2,
0x9bf, 0x9a8, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x9a4, 0x9bf, 0x9b8, 0x9cd, 0x9b2, 0x9be,
0x9ad, 0x9be, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9c7, 0x9b2, 0x9b8, 0x9ac, 0x9c1,
0x996, 0x9be, 0x9b0, 0x9c7, 0x9b8, 0x9cd, 0x99f, 0x9ac, 0x9c1, 0x9a1, 0x9be, 0x9aa,
0x9c7, 0x9b8, 0x9cd, 0x99f, 0x9ac, 0x9c1, 0x9b8, 0x9bf, 0x9a8, 0x997, 0x9c7, 0x9a8,
0x995, 0x9bf, 0x9b8, 0x9bf, 0x9a8, 0x9be, 0x9b9, 0x9c1, 0x995, 0x9cb, 0x9aa, 0x9c7,
0x9a8, 0x9b9, 0x9c7, 0x997, 0x9c7, 0x9a8, 0x99c, 0x9bf, 0x9ac, 0x9cd, 0x9b0, 0x9be,
0x9b2, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd,
0x9a8, 0x9b8, 0x9bf, 0x9b9, 0x9c7, 0x9b2, 0x9b8, 0x9bf, 0x999, 0x9cd, 0x995, 0x9bf,
0x986, 0x987, 0x9b2, 0x20, 0x985, 0x9ab, 0x20, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x9a8,
0x99c, 0x9be, 0x9b0, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9bf,
0x999, 0x997, 0x9cd, 0x9b0, 0x9be, 0x9a1, 0x995, 0x9bf, 0x9b0, 0x9cb, 0x9ad, 0x995,
0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9ad, 0x9b2, 0x9bf, 0x9b8, 0x9ac, 0x9a8, 0x9b2, 0x9c1,
0x9ac, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9b2, 0x9a8, 0x9cd, 0x9a1,
0x9a8, 0x9b2, 0x9c1, 0x995, 0x9cd, 0x9b8, 0x9c7, 0x9ae, 0x9ac, 0x9be, 0x9b0, 0x9cd,
0x997, 0x9ae, 0x9be, 0x9a6, 0x9cd, 0x9b0, 0x9bf, 0x9a6, 0x9ae, 0x9b0, 0x9bf, 0x9af,
0x9bc, 0x9c7, 0x9b9, 0x9be, 0x9ae, 0x9c7, 0x9a8, 0x9ae, 0x9bf, 0x9a8, 0x9cd, 0x9b8,
0x995, 0x985, 0x9b8, 0x9b2, 0x9cb, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9bf, 0x9b8,
0x9aa, 0x9a1, 0x997, 0x9b0, 0x9bf, 0x9a4, 0x9cd, 0x9b8, 0x9be, 0x9aa, 0x9cd, 0x9b0,
0x9be, 0x997, 0x9b0, 0x9bf, 0x997, 0x9be, 0x9b8, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9be,
0x9b8, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x9a8, 0x9cb, 0x9b8, 0x9be,
0x9b0, 0x9be, 0x99c, 0x9bf, 0x9ad, 0x9cb, 0x9b8, 0x9be, 0x9b0, 0x9be, 0x99f, 0x9cb,
0x9ad, 0x9b8, 0x9bf, 0x9ae, 0x9ab, 0x9c7, 0x9b0, 0x9cb, 0x9aa, 0x9cb, 0x9b2, 0x9b8,
0x9cd, 0x995, 0x9aa, 0x9af, 0x9bc, 0x9c7, 0x9b8, 0x9cb, 0x9ab, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9b8, 0x9cd, 0x99f, 0x995, 0x9b9, 0x9cb, 0x9ae, 0x9a4, 0x9be, 0x9b9, 0x9b2,
0x9bf, 0x9a8, 0x9a4, 0x9bf, 0x9b0, 0x9be, 0x9a8, 0x9be, 0x989, 0x9b2, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x9cb, 0x9ad, 0x9b8, 0x9cd, 0x995, 0x9ad, 0x9be, 0x99f, 0x9bf,
0x995, 0x9be, 0x9a8, 0x9ad, 0x9bf, 0x9b2, 0x9a8, 0x9bf, 0x993, 0x9b8, 0x9ad, 0x9cb,
0x9b2, 0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9b0, 0x9be, 0x9a6, 0x993, 0x9af, 0x9bc,
0x9be, 0x9b0, 0x9b6, 0x99c, 0x9be, 0x997, 0x9cd, 0x9b0, 0x9c7, 0x9ac, 0x99c, 0x9c1,
0x9b0, 0x9bf, 0x996, 0x986, 0x9a8, 0x9cd, 0x9a4, 0x9c1, 0x9a8, 0x9be, 0x9a8, 0x9be,
0x9b0, 0x9bf, 0x9ad, 0x9cb, 0x99b, 0x9be, 0x997, 0x9cb, 0x9b8, 0x995, 0x9cd, 0x9b0,
0x9bf, 0x9b8, 0x9ae, 0x9be, 0x9b8, 0x995, 0x9cb, 0x995, 0x9cb, 0x9b8, 0x995, 0x9cb,
0x9ae, 0x9cb, 0x9b0, 0x9cb, 0x995, 0x9be, 0x9b0, 0x9cd, 0x997, 0x9c1, 0x9b2, 0x9c7,
0x9a8, 0x9ae, 0x9b0, 0x9bf, 0x9b6, 0x9be, 0x9b8, 0x9ae, 0x9be, 0x9af, 0x9bc, 0x9cb,
0x9a4, 0x9cb, 0x9b0, 0x9bf, 0x987, 0x989, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x985,
0x995, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9ac, 0x9c1, 0x997, 0x9c7,
0x9a8, 0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x99a, 0x9cd, 0x9af, 0x9be, 0x9a0, 0x9be, 0x9ae,
0x987, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x987, 0x9ab, 0x9be, 0x9a4, 0x9c7, 0x9ab,
0x9cd, 0x9af, 0x9be, 0x995, 0x9be, 0x993, 0x9ab, 0x9cb, 0x997, 0x9cd, 0x9af, 0x9be,
0x9b2, 0x9be, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x997, 0x9cb, 0x9b8, 0x997, 0x9be, 0x9ae,
0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x997, 0x9cb, 0x9af, 0x9bc, 0x9be,
0x9a6, 0x9be, 0x9b2, 0x995, 0x9c1, 0x9a8, 0x9be, 0x9b2, 0x9b9, 0x9a8, 0x9cb, 0x9b2,
0x9c1, 0x9b2, 0x9c1, 0x98f, 0x9a8, 0x9cd, 0x9a1, 0x9be, 0x9b0, 0x9ac, 0x9be, 0x9b0,
0x9bf, 0x995, 0x9bf, 0x9b0, 0x9bf, 0x9a4, 0x9bf, 0x9ae, 0x9be, 0x9a4, 0x9bf, 0x995,
0x9cb, 0x9b8, 0x9b0, 0x9be, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x99c,
0x9be, 0x9b2, 0x9c7, 0x987, 0x9a8, 0x9ae, 0x9be, 0x99c, 0x9c1, 0x9b0, 0x9cb, 0x9ae,
0x9be, 0x9b0, 0x9cd, 0x995, 0x9c7, 0x9b8, 0x9be, 0x9b8, 0x9ae, 0x9bf, 0x9a1, 0x993,
0x9af, 0x9bc, 0x9c7, 0x9a8, 0x9be, 0x989, 0x9b0, 0x9c1, 0x9a8, 0x9bf, 0x989, 0x9af,
0x9bc, 0x9bf, 0x9a8, 0x9b0, 0x9ab, 0x995, 0x9a8, 0x9c1, 0x9ae, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x997, 0x9cb, 0x20, 0x9aa, 0x9cd, 0x9af, 0x9be,
0x997, 0x9cb, 0x9aa, 0x9bf, 0x99f, 0x995, 0x9c7, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd,
0x9a8, 0x9aa, 0x9cb, 0x9a8, 0x9be, 0x9aa, 0x9c7, 0x9aa, 0x9cb, 0x9b0, 0x9cd, 0x99f,
0x20, 0x9ae, 0x9cc, 0x9b0, 0x99c, 0x9ac, 0x9bf, 0x9b0, 0x9be, 0x9b0, 0x989, 0x9b9,
0x9a4, 0x9c1, 0x999, 0x9cd, 0x997, 0x9be, 0x9b8, 0x9be, 0x987, 0x9aa, 0x9be, 0x9a8,
0x99f, 0x9be, 0x9b0, 0x9be, 0x993, 0x9af, 0x9bc, 0x9be, 0x99f, 0x9cb, 0x999, 0x9cd,
0x997, 0x9be, 0x99f, 0x9be, 0x9aa, 0x9c1, 0x993, 0x9af, 0x9bc, 0x9c7, 0x995, 0x993,
0x9b2, 0x9bf, 0x9b8, 0x41, 0x6c, 0x6a, 0x65, 0x72, 0x42, 0x61, 0x6e, 0x67,
0x69, 0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x79, 0x44, 0x6a, 0x69, 0x62,
0x75, 0x74, 0x69, 0x41, 0x61, 0x69, 0x75, 0x6e, 0x4b, 0x68, 0x61, 0x72,
0x74, 0x75, 0x6d, 0x4d, 0x75, 0x71, 0x64, 0x69, 0x73, 0x68, 0x6f, 0x4e,
0x2019, 0x64, 0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x69, 0x61, 0x6d,
0x65, 0x69, 0x55, 0x61, 0x67, 0x61, 0x64, 0x75, 0x67, 0x75, 0x50, 0x6f,
0x72, 0x74, 0x6f, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x41, 0x69, 0x6e, 0x67,
0x69, 0x72, 0x61, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61, 0xed, 0x6e, 0x61,
0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x43, 0x75, 0x69, 0x61, 0x62, 0xe1,
0x45, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x70, 0xe9, 0x4d, 0x61, 0x63, 0x65,
0x69, 0xf3, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0x4d,
0x61, 0x7a, 0x61, 0x74, 0x6c, 0xe1, 0x6e, 0x4d, 0x65, 0x78, 0x69, 0x6b,
0x6f, 0x20, 0x48, 0x69, 0x72, 0x69, 0x61, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x75, 0x6e, 0x65, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x49,
0x70, 0x61, 0x72, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x49, 0x70, 0x61, 0x72, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x49, 0x70, 0x61, 0x72, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f, 0x66, 0x2d, 0x53,
0x70, 0x61, 0x69, 0x6e, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0xe9, 0x6d,
0x53, 0xe3, 0x6f, 0x20, 0x50, 0x61, 0x75, 0x6c, 0x6f, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d,
0x79, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x4a, 0x6f, 0x68, 0x6e, 0x2019,
0x73, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73,
0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x7a, 0x69, 0x61, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74,
0x51, 0x61, 0x61, 0x6e, 0x61, 0x61, 0x71, 0x41, 0x73, 0x67, 0x61, 0x62,
0x61, 0x74, 0x42, 0x69, 0x78, 0x6b, 0x65, 0x6b, 0x54, 0x78, 0x69, 0x74,
0x61, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x6f, 0x44, 0x75, 0x78, 0x61,
0x6e, 0x62, 0x65, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x4b, 0x61, 0x6d, 0x74,
0x78, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x78, 0x69,
0x4b, 0x68, 0x61, 0x6e, 0x64, 0x69, 0x67, 0x61, 0x4b, 0x61, 0x6c, 0x6b,
0x75, 0x74, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72,
0x73, 0x6b, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x74, 0x50, 0x69, 0x6f, 0x6e,
0x67, 0x69, 0x61, 0x6e, 0x67, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64, 0x61,
0x54, 0x61, 0x78, 0x6b, 0x65, 0x6e, 0x74, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0xdc, 0x72, 0xfc, 0x6d, 0x71, 0x69, 0x56,
0x69, 0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x45, 0x72, 0x65, 0x76, 0x61,
0x6e, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x61, 0x6b, 0x4b, 0x61, 0x6e, 0x61,
0x72, 0x69, 0x61, 0x6b, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x48, 0x65, 0x67, 0x6f, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6b, 0x53, 0x61, 0x6e,
0x74, 0x61, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x74, 0x65,
0x6e, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x42, 0x72,
0x75, 0x73, 0x65, 0x6c, 0x61, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x73,
0x74, 0x42, 0xfc, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x47, 0x75, 0x65,
0x72, 0x6e, 0x65, 0x73, 0x65, 0x79, 0x4d, 0x61, 0x6e, 0x20, 0x75, 0x68,
0x61, 0x72, 0x74, 0x65, 0x61, 0x4c, 0x75, 0x78, 0x65, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x6f, 0x4d, 0x61, 0x64, 0x72, 0x69, 0x6c, 0x4d, 0x6f, 0x73,
0x6b, 0x75, 0x45, 0x72, 0x72, 0x6f, 0x6d, 0x61, 0x55, 0x6c, 0x69, 0x61,
0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e,
0x6f, 0x20, 0x48, 0x69, 0x72, 0x69, 0x61, 0x4d, 0x61, 0x68, 0xe9, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x61, 0x6b, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x7a, 0x69, 0x6f, 0x50, 0x61, 0x7a, 0x6b, 0x6f, 0x20, 0x75, 0x68, 0x61,
0x72, 0x74, 0x65, 0x61, 0xc9, 0x66, 0x61, 0x74, 0xe9, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x61, 0x6b, 0x4d, 0x61, 0x72, 0x6b, 0x65,
0x73, 0x61, 0x6b, 0x410, 0x431, 0x456, 0x434, 0x436, 0x430, 0x43d, 0x410, 0x43a,
0x440, 0x430, 0x410, 0x434, 0x44b, 0x441, 0x2d, 0x410, 0x431, 0x435, 0x431, 0x430,
0x410, 0x43b, 0x436, 0x44b, 0x440, 0x410, 0x441, 0x43c, 0x430, 0x440, 0x430, 0x411,
0x430, 0x43d, 0x433, 0x456, 0x411, 0x430, 0x43d, 0x434, 0x436, 0x443, 0x43b, 0x411,
0x456, 0x441, 0x430, 0x443, 0x411, 0x43b, 0x430, 0x43d, 0x442, 0x430, 0x439, 0x440,
0x411, 0x440, 0x430, 0x437, 0x430, 0x432, 0x456, 0x43b, 0x44c, 0x41a, 0x430, 0x456,
0x440, 0x41a, 0x43e, 0x43d, 0x430, 0x43a, 0x440, 0x44b, 0x414, 0x430, 0x440, 0x2d,
0x44d, 0x441, 0x2d, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x414, 0x436, 0x44b, 0x431,
0x443, 0x446, 0x456, 0x42d, 0x43b, 0x44c, 0x2d, 0x410, 0x44e, 0x43d, 0x424, 0x440,
0x44b, 0x442, 0x430, 0x45e, 0x43d, 0x413, 0x430, 0x431, 0x430, 0x440, 0x43e, 0x43d,
0x44d, 0x425, 0x430, 0x440, 0x430, 0x440, 0x44d, 0x42f, 0x433, 0x430, 0x43d, 0x44d,
0x441, 0x431, 0x443, 0x440, 0x433, 0x414, 0x436, 0x443, 0x431, 0x430, 0x41a, 0x456,
0x433, 0x430, 0x43b, 0x456, 0x41a, 0x456, 0x43d, 0x448, 0x430, 0x441, 0x430, 0x41b,
0x430, 0x433, 0x430, 0x441, 0x41b, 0x456, 0x431, 0x440, 0x44d, 0x432, 0x456, 0x43b,
0x44c, 0x41b, 0x430, 0x43c, 0x44d, 0x41b, 0x443, 0x431, 0x443, 0x43c, 0x431, 0x430,
0x448, 0x44b, 0x41c, 0x430, 0x43b, 0x430, 0x431, 0x430, 0x41c, 0x431, 0x430, 0x431,
0x430, 0x43d, 0x44d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x44b, 0x448, 0x430, 0x41c,
0x430, 0x43d, 0x440, 0x43e, 0x432, 0x456, 0x44f, 0x41d, 0x430, 0x439, 0x440, 0x43e,
0x431, 0x456, 0x41d, 0x434, 0x436, 0x430, 0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x456,
0x44f, 0x43c, 0x435, 0x439, 0x41f, 0x43e, 0x440, 0x442, 0x430, 0x2d, 0x41d, 0x43e,
0x432, 0x430, 0x421, 0x430, 0x43d, 0x2d, 0x422, 0x430, 0x43c, 0x44d, 0x422, 0x440,
0x44b, 0x43f, 0x430, 0x43b, 0x456, 0x422, 0x443, 0x43d, 0x456, 0x441, 0x412, 0x456,
0x43d, 0x434, 0x445, 0x443, 0x43a, 0x410, 0x43d, 0x43a, 0x430, 0x440, 0x44b, 0x434,
0x436, 0x410, 0x43d, 0x433, 0x456, 0x43b, 0x44c, 0x44f, 0x410, 0x43d, 0x442, 0x44b,
0x433, 0x443, 0x430, 0x410, 0x440, 0x430, 0x433, 0x443, 0x430, 0x456, 0x43d, 0x430,
0x411, 0x443, 0x44d, 0x43d, 0x430, 0x441, 0x2d, 0x410, 0x439, 0x440, 0x44d, 0x441,
0x41a, 0x43e, 0x440, 0x434, 0x430, 0x432, 0x430, 0x416, 0x443, 0x436, 0x443, 0x439,
0x41b, 0x430, 0x2d, 0x420, 0x44b, 0x451, 0x445, 0x430, 0x420, 0x44b, 0x430, 0x2d,
0x413, 0x430, 0x43b, 0x44c, 0x435, 0x433, 0x430, 0x441, 0x421, 0x430, 0x43b, 0x44c,
0x442, 0x430, 0x421, 0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x456, 0x441, 0x423, 0x448,
0x443, 0x430, 0x44f, 0x410, 0x441, 0x443, 0x43d, 0x441, 0x44c, 0x451, 0x43d, 0x410,
0x442, 0x44b, 0x43a, 0x43e, 0x43a, 0x430, 0x43d, 0x411, 0x430, 0x456, 0x44f, 0x411,
0x430, 0x456, 0x44f, 0x2d, 0x434, 0x44d, 0x2d, 0x411, 0x430, 0x43d, 0x434, 0x44d,
0x440, 0x430, 0x441, 0x411, 0x430, 0x440, 0x431, 0x430, 0x434, 0x430, 0x441, 0x411,
0x435, 0x43b, 0x456, 0x437, 0x411, 0x43e, 0x430, 0x2d, 0x412, 0x456, 0x441, 0x442,
0x430, 0x411, 0x430, 0x433, 0x430, 0x442, 0x430, 0x411, 0x43e, 0x439, 0x441, 0x44d,
0x41a, 0x435, 0x43c, 0x431, 0x440, 0x44b, 0x434, 0x436, 0x2d, 0x411, 0x435, 0x439,
0x41a, 0x430, 0x43c, 0x43f, 0x443, 0x2d, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x44b,
0x41a, 0x430, 0x435, 0x43d, 0x430, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x430,
0x432, 0x44b, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x427, 0x44b,
0x43a, 0x430, 0x433, 0x430, 0x427, 0x44b, 0x45e, 0x430, 0x45e, 0x430, 0x421, 0x44c,
0x44e, 0x434, 0x430, 0x434, 0x2d, 0x425, 0x443, 0x430, 0x440, 0x44d, 0x441, 0x41a,
0x43e, 0x441, 0x442, 0x430, 0x2d, 0x420, 0x44b, 0x43a, 0x430, 0x41a, 0x440, 0x44d,
0x441, 0x442, 0x430, 0x43d, 0x41a, 0x443, 0x44f, 0x431, 0x430, 0x41a, 0x44e, 0x440,
0x430, 0x441, 0x430, 0x430, 0x414, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x43a, 0x441,
0x445, 0x430, 0x45e, 0x43d, 0x414, 0x43e, 0x45e, 0x441, 0x430, 0x43d, 0x414, 0x43e,
0x441, 0x430, 0x43d, 0x2d, 0x41a, 0x440, 0x44b, 0x43a, 0x414, 0x44d, 0x43d, 0x432,
0x435, 0x440, 0x414, 0x44d, 0x442, 0x440, 0x43e, 0x439, 0x442, 0x414, 0x430, 0x43c,
0x456, 0x43d, 0x456, 0x43a, 0x430, 0x42d, 0x434, 0x43c, 0x430, 0x43d, 0x442, 0x430,
0x43d, 0x42d, 0x439, 0x440, 0x443, 0x43d, 0x44d, 0x43f, 0x435, 0x421, 0x430, 0x43b,
0x44c, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x424, 0x43e, 0x440, 0x442, 0x2d, 0x41d,
0x435, 0x43b, 0x44c, 0x441, 0x430, 0x43d, 0x424, 0x430, 0x440, 0x442, 0x430, 0x43b,
0x435, 0x437, 0x430, 0x413, 0x43b, 0x44d, 0x439, 0x441, 0x2d, 0x411, 0x44d, 0x439,
0x413, 0x443, 0x441, 0x2d, 0x411, 0x44d, 0x439, 0x413, 0x440, 0x430, 0x43d, 0x434,
0x2d, 0x426, 0x451, 0x440, 0x43a, 0x413, 0x440, 0x44d, 0x43d, 0x430, 0x434, 0x430,
0x413, 0x432, 0x430, 0x434, 0x44d, 0x43b, 0x443, 0x43f, 0x430, 0x413, 0x432, 0x430,
0x442, 0x44d, 0x43c, 0x430, 0x43b, 0x430, 0x413, 0x443, 0x430, 0x44f, 0x43a, 0x456,
0x43b, 0x44c, 0x413, 0x430, 0x44f, 0x43d, 0x430, 0x413, 0x430, 0x43b, 0x456, 0x444,
0x430, 0x43a, 0x441, 0x42d, 0x440, 0x43c, 0x430, 0x441, 0x456, 0x43b, 0x44c, 0x44f,
0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x43f, 0x430, 0x43b, 0x456, 0x441,
0x41d, 0x43e, 0x43a, 0x441, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d,
0x430, 0x41c, 0x430, 0x440, 0x44d, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x406, 0x43d,
0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x41f, 0x456, 0x442, 0x44d, 0x440, 0x441, 0x431,
0x435, 0x440, 0x433, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430,
0x422, 0x44d, 0x43b, 0x20, 0x421, 0x456, 0x446, 0x456, 0x2c, 0x20, 0x406, 0x43d,
0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x412, 0x456, 0x432, 0x456, 0x2c, 0x20, 0x406,
0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x412, 0x456, 0x43d, 0x441, 0x435, 0x43d,
0x441, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x423, 0x456,
0x43d, 0x430, 0x43c, 0x430, 0x43a, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f,
0x43d, 0x430, 0x406, 0x43d, 0x443, 0x432, 0x456, 0x43a, 0x406, 0x43a, 0x430, 0x43b,
0x443, 0x456, 0x442, 0x42f, 0x43c, 0x430, 0x439, 0x43a, 0x430, 0x414, 0x436, 0x443,
0x43d, 0x430, 0x41b, 0x443, 0x456, 0x441, 0x432, 0x456, 0x43b, 0x41c, 0x430, 0x43d,
0x442, 0x44b, 0x441, 0x435, 0x43b, 0x430, 0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442,
0x443, 0x43a, 0x456, 0x41a, 0x440, 0x430, 0x43b, 0x435, 0x43d, 0x434, 0x44d, 0x439,
0x43a, 0x41b, 0x456, 0x43c, 0x430, 0x41b, 0x43e, 0x441, 0x2d, 0x410, 0x43d, 0x434,
0x436, 0x44d, 0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x45e, 0x44d, 0x440, 0x20, 0x41f,
0x440, 0x44b, 0x43d, 0x441, 0x456, 0x437, 0x20, 0x41a, 0x432, 0x43e, 0x442, 0x44d,
0x440, 0x41c, 0x430, 0x441, 0x435, 0x451, 0x41c, 0x430, 0x43d, 0x430, 0x45e, 0x441,
0x41c, 0x430, 0x440, 0x44b, 0x433, 0x43e, 0x41c, 0x430, 0x440, 0x446, 0x456, 0x43d,
0x456, 0x43a, 0x430, 0x41c, 0x430, 0x442, 0x430, 0x43c, 0x43e, 0x440, 0x430, 0x441,
0x41c, 0x435, 0x43d, 0x43e, 0x43c, 0x456, 0x43d, 0x456, 0x41c, 0x435, 0x440, 0x44b,
0x434, 0x430, 0x41c, 0x435, 0x445, 0x456, 0x43a, 0x430, 0x41c, 0x456, 0x43a, 0x435,
0x43b, 0x43e, 0x43d, 0x41c, 0x43e, 0x43d, 0x43a, 0x442, 0x430, 0x43d, 0x41c, 0x430,
0x43d, 0x442, 0x44d, 0x440, 0x44d, 0x439, 0x41c, 0x430, 0x43d, 0x442, 0x44d, 0x432,
0x456, 0x434, 0x44d, 0x430, 0x41c, 0x430, 0x43d, 0x442, 0x441, 0x435, 0x440, 0x430,
0x442, 0x41d, 0x430, 0x441, 0x430, 0x45e, 0x41d, 0x44c, 0x44e, 0x2d, 0x401, 0x440,
0x43a, 0x41d, 0x430, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x411, 0x2019, 0x44e, 0x43b,
0x430, 0x2c, 0x20, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x44f,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x44d,
0x440, 0x2c, 0x20, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x44f,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e, 0x2d, 0x421,
0x435, 0x439, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x41f, 0x430, 0x45e, 0x43d, 0x43e,
0x447, 0x43d, 0x430, 0x44f, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d,
0x443, 0x43a, 0x410, 0x445, 0x456, 0x43d, 0x430, 0x433, 0x430, 0x41f, 0x430, 0x440,
0x430, 0x43c, 0x430, 0x440, 0x44b, 0x431, 0x430, 0x424, 0x456, 0x43d, 0x456, 0x43a,
0x441, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x2d, 0x41f, 0x440, 0x44d, 0x43d,
0x441, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x444, 0x2d, 0x421, 0x43f, 0x435,
0x439, 0x43d, 0x41f, 0x43e, 0x440, 0x442, 0x443, 0x2d, 0x412, 0x435, 0x43b, 0x44c,
0x44e, 0x41f, 0x443, 0x44d, 0x440, 0x442, 0x430, 0x2d, 0x420, 0x44b, 0x43a, 0x430,
0x41f, 0x443, 0x43d, 0x442, 0x430, 0x2d, 0x410, 0x440, 0x44d, 0x43d, 0x430, 0x441,
0x420, 0x430, 0x43d, 0x43a, 0x456, 0x43d, 0x2d, 0x406, 0x43d, 0x43b, 0x435, 0x442,
0x420, 0x44d, 0x441, 0x456, 0x444, 0x456, 0x420, 0x44d, 0x434, 0x436, 0x430, 0x439,
0x43d, 0x430, 0x420, 0x44d, 0x437, 0x430, 0x43b, 0x44c, 0x44e, 0x442, 0x420, 0x44b,
0x443, 0x2d, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x443, 0x421, 0x430, 0x43d, 0x442,
0x430, 0x440, 0x44d, 0x43d, 0x421, 0x430, 0x43d, 0x442, 0x2019, 0x44f, 0x433, 0x430,
0x421, 0x430, 0x43d, 0x442, 0x430, 0x2d, 0x414, 0x430, 0x43c, 0x456, 0x43d, 0x433,
0x430, 0x421, 0x430, 0x43d, 0x2d, 0x41f, 0x430, 0x45e, 0x43b, 0x443, 0x406, 0x442,
0x430, 0x43a, 0x430, 0x440, 0x442, 0x430, 0x430, 0x440, 0x43c, 0x456, 0x442, 0x421,
0x456, 0x442, 0x43a, 0x430, 0x421, 0x435, 0x43d, 0x2d, 0x411, 0x430, 0x440, 0x442,
0x44d, 0x43b, 0x44c, 0x43c, 0x456, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x414, 0x436,
0x43e, 0x43d, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41a, 0x456, 0x442, 0x441,
0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41b, 0x44e, 0x441, 0x456, 0x44f, 0x421, 0x435,
0x43d, 0x442, 0x2d, 0x412, 0x456, 0x43d, 0x441, 0x435, 0x43d, 0x442, 0x421, 0x432,
0x456, 0x444, 0x442, 0x2d, 0x41a, 0x435, 0x440, 0x44d, 0x43d, 0x442, 0x422, 0x44d,
0x433, 0x443, 0x441, 0x456, 0x433, 0x430, 0x43b, 0x44c, 0x43f, 0x430, 0x422, 0x44b,
0x445, 0x443, 0x430, 0x43d, 0x430, 0x422, 0x430, 0x440, 0x43e, 0x43d, 0x442, 0x430,
0x422, 0x430, 0x440, 0x442, 0x43e, 0x43b, 0x430, 0x423, 0x430, 0x439, 0x442, 0x445,
0x43e, 0x440, 0x441, 0x412, 0x456, 0x43d, 0x456, 0x43f, 0x435, 0x433, 0x42f, 0x43a,
0x443, 0x442, 0x430, 0x442, 0x41a, 0x44d, 0x439, 0x441, 0x456, 0x414, 0x44d, 0x439,
0x432, 0x456, 0x441, 0x414, 0x437, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x414, 0x437,
0x44e, 0x440, 0x432, 0x456, 0x43b, 0x44c, 0x41c, 0x430, 0x43a, 0x443, 0x43e, 0x440,
0x44b, 0x41c, 0x43e, 0x45e, 0x441, 0x430, 0x43d, 0x41c, 0x430, 0x43a, 0x2d, 0x41c,
0x435, 0x440, 0x434, 0x430, 0x420, 0x43e, 0x442, 0x44d, 0x440, 0x430, 0x421, 0x451,
0x432, 0x430, 0x412, 0x430, 0x441, 0x442, 0x43e, 0x43a, 0x41b, 0x43e, 0x43d, 0x433,
0x439, 0x456, 0x440, 0x410, 0x434, 0x44d, 0x43d, 0x410, 0x43c, 0x430, 0x43d, 0x20,
0x28, 0x433, 0x43e, 0x440, 0x430, 0x434, 0x29, 0x410, 0x43a, 0x442, 0x430, 0x431,
0x44d, 0x410, 0x448, 0x433, 0x430, 0x431, 0x430, 0x442, 0x411, 0x430, 0x445, 0x440,
0x44d, 0x439, 0x43d, 0x411, 0x435, 0x439, 0x440, 0x443, 0x442, 0x411, 0x456, 0x448,
0x43a, 0x435, 0x43a, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x427, 0x44b, 0x442,
0x430, 0x41a, 0x430, 0x43b, 0x43e, 0x43c, 0x431, 0x430, 0x414, 0x44b, 0x43b, 0x456,
0x414, 0x443, 0x431, 0x430, 0x439, 0x414, 0x443, 0x448, 0x430, 0x43d, 0x431, 0x44d,
0x425, 0x435, 0x45e, 0x440, 0x43e, 0x43d, 0x425, 0x430, 0x448, 0x44b, 0x43c, 0x456,
0x43d, 0x413, 0x430, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x425, 0x43e, 0x45e, 0x434,
0x406, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x414, 0x436, 0x430, 0x43a, 0x430, 0x440,
0x442, 0x430, 0x414, 0x436, 0x430, 0x44f, 0x43f, 0x443, 0x440, 0x430, 0x406, 0x435,
0x440, 0x443, 0x441, 0x430, 0x43b, 0x456, 0x43c, 0x41a, 0x430, 0x440, 0x430, 0x447,
0x44b, 0x41a, 0x430, 0x43b, 0x44c, 0x43a, 0x443, 0x442, 0x430, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x430, 0x44f, 0x440, 0x441, 0x43a, 0x41a, 0x443, 0x447, 0x44b, 0x43d,
0x433, 0x41a, 0x443, 0x432, 0x435, 0x439, 0x442, 0x41c, 0x430, 0x43a, 0x430, 0x430,
0x41c, 0x430, 0x43d, 0x456, 0x43b, 0x430, 0x41d, 0x456, 0x43a, 0x430, 0x441, 0x456,
0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x43a, 0x443, 0x437, 0x43d, 0x435, 0x446, 0x43a,
0x41d, 0x43e, 0x432, 0x430, 0x441, 0x456, 0x431, 0x456, 0x440, 0x441, 0x43a, 0x423,
0x440, 0x430, 0x43b, 0x44c, 0x441, 0x43a, 0x41f, 0x43d, 0x430, 0x43c, 0x43f, 0x435,
0x43d, 0x44c, 0x41f, 0x430, 0x43d, 0x442, 0x44b, 0x44f, 0x43d, 0x430, 0x43a, 0x41f,
0x445, 0x435, 0x43d, 0x44c, 0x44f, 0x43d, 0x41a, 0x443, 0x441, 0x442, 0x430, 0x43d,
0x430, 0x439, 0x41a, 0x437, 0x44b, 0x43b, 0x2d, 0x410, 0x440, 0x434, 0x430, 0x42d,
0x440, 0x2d, 0x420, 0x44b, 0x44f, 0x434, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x456,
0x43d, 0x428, 0x430, 0x43d, 0x445, 0x430, 0x439, 0x421, 0x456, 0x43d, 0x433, 0x430,
0x43f, 0x443, 0x440, 0x421, 0x44f, 0x440, 0x44d, 0x434, 0x43d, 0x435, 0x43a, 0x430,
0x43b, 0x44b, 0x43c, 0x441, 0x43a, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x439, 0x422,
0x431, 0x456, 0x43b, 0x456, 0x441, 0x456, 0x422, 0x44d, 0x433, 0x435, 0x440, 0x430,
0x43d, 0x422, 0x445, 0x456, 0x43c, 0x43f, 0x445, 0x443, 0x422, 0x43e, 0x43a, 0x456,
0x430, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x430, 0x440, 0x423,
0x440, 0x443, 0x43c, 0x447, 0x44b, 0x423, 0x441, 0x446, 0x44c, 0x2d, 0x41d, 0x435,
0x440, 0x430, 0x412, 0x2019, 0x435, 0x43d, 0x446, 0x44c, 0x44f, 0x43d, 0x423, 0x43b,
0x430, 0x434, 0x437, 0x456, 0x432, 0x430, 0x441, 0x442, 0x43e, 0x43a, 0x42f, 0x43a,
0x443, 0x446, 0x43a, 0x420, 0x430, 0x43d, 0x433, 0x443, 0x43d, 0x415, 0x43a, 0x430,
0x446, 0x44f, 0x440, 0x44b, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x415, 0x440, 0x44d,
0x432, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x456, 0x44f, 0x20,
0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x411, 0x435, 0x440, 0x43c, 0x443,
0x434, 0x441, 0x43a, 0x456, 0x44f, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432,
0x44b, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x441, 0x43a, 0x456, 0x44f, 0x20, 0x430,
0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x41a, 0x430, 0x431, 0x430, 0x2d, 0x412,
0x435, 0x440, 0x434, 0x44d, 0x424, 0x430, 0x440, 0x44d, 0x440, 0x441, 0x43a, 0x456,
0x44f, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x41c, 0x430, 0x434,
0x44d, 0x439, 0x440, 0x430, 0x420, 0x44d, 0x439, 0x43a, 0x2019, 0x44f, 0x432, 0x456,
0x43a, 0x41f, 0x430, 0x45e, 0x434, 0x43d, 0x451, 0x432, 0x430, 0x44f, 0x20, 0x413,
0x435, 0x43e, 0x440, 0x433, 0x456, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x440, 0x430,
0x45e, 0x20, 0x421, 0x432, 0x44f, 0x442, 0x43e, 0x439, 0x20, 0x410, 0x43b, 0x435,
0x43d, 0x44b, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x421, 0x442, 0x44d, 0x43d, 0x43b,
0x456, 0x410, 0x434, 0x44d, 0x43b, 0x430, 0x456, 0x434, 0x430, 0x411, 0x440, 0x44b,
0x441, 0x431, 0x435, 0x43d, 0x411, 0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x2d, 0x425,
0x456, 0x43b, 0x414, 0x430, 0x440, 0x432, 0x456, 0x43d, 0x42e, 0x43a, 0x43b, 0x430,
0x41b, 0x456, 0x43d, 0x434, 0x44d, 0x43c, 0x430, 0x43d, 0x41c, 0x435, 0x43b, 0x44c,
0x431, 0x443, 0x440, 0x43d, 0x421, 0x456, 0x434, 0x43d, 0x435, 0x439, 0x410, 0x43c,
0x441, 0x442, 0x44d, 0x440, 0x434, 0x430, 0x43c, 0x410, 0x43d, 0x434, 0x43e, 0x440,
0x430, 0x410, 0x441, 0x442, 0x440, 0x430, 0x445, 0x430, 0x43d, 0x44c, 0x410, 0x444,
0x456, 0x43d, 0x44b, 0x411, 0x44f, 0x43b, 0x433, 0x440, 0x430, 0x434, 0x411, 0x435,
0x440, 0x43b, 0x456, 0x43d, 0x411, 0x440, 0x430, 0x446, 0x456, 0x441, 0x43b, 0x430,
0x432, 0x430, 0x411, 0x440, 0x443, 0x441, 0x435, 0x43b, 0x44c, 0x411, 0x443, 0x445,
0x430, 0x440, 0x44d, 0x441, 0x442, 0x411, 0x44e, 0x437, 0x456, 0x43d, 0x433, 0x435,
0x43d, 0x41a, 0x456, 0x448, 0x44b, 0x43d, 0x451, 0x45e, 0x41a, 0x430, 0x43f, 0x435,
0x43d, 0x433, 0x430, 0x433, 0x435, 0x43d, 0x414, 0x443, 0x431, 0x43b, 0x456, 0x43d,
0x413, 0x456, 0x431, 0x440, 0x430, 0x43b, 0x442, 0x430, 0x440, 0x413, 0x435, 0x440,
0x43d, 0x441, 0x456, 0x425, 0x435, 0x43b, 0x44c, 0x441, 0x456, 0x43d, 0x43a, 0x456,
0x412, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x45e, 0x20, 0x41c, 0x44d, 0x43d, 0x414,
0x436, 0x44d, 0x440, 0x441, 0x456, 0x41a, 0x430, 0x43b, 0x456, 0x43d, 0x456, 0x43d,
0x433, 0x440, 0x430, 0x434, 0x41a, 0x456, 0x440, 0x430, 0x45e, 0x41a, 0x456, 0x435,
0x45e, 0x41b, 0x456, 0x441, 0x430, 0x431, 0x43e, 0x43d, 0x41b, 0x44e, 0x431, 0x43b,
0x44f, 0x43d, 0x430, 0x41b, 0x43e, 0x43d, 0x434, 0x430, 0x43d, 0x41b, 0x44e, 0x43a,
0x441, 0x435, 0x43c, 0x431, 0x443, 0x440, 0x433, 0x41c, 0x430, 0x434, 0x440, 0x44b,
0x434, 0x41c, 0x430, 0x43b, 0x44c, 0x442, 0x430, 0x41c, 0x430, 0x440, 0x44b, 0x44f,
0x43d, 0x445, 0x430, 0x43c, 0x456, 0x43d, 0x430, 0x41c, 0x456, 0x43d, 0x441, 0x43a,
0x41c, 0x430, 0x43d, 0x430, 0x43a, 0x430, 0x41c, 0x430, 0x441, 0x43a, 0x432, 0x430,
0x41e, 0x441, 0x43b, 0x430, 0x41f, 0x430, 0x440, 0x44b, 0x436, 0x41f, 0x430, 0x434,
0x433, 0x43e, 0x440, 0x44b, 0x446, 0x430, 0x420, 0x44b, 0x433, 0x430, 0x420, 0x44b,
0x43c, 0x421, 0x430, 0x43d, 0x2d, 0x41c, 0x430, 0x440, 0x44b, 0x43d, 0x430, 0x421,
0x430, 0x440, 0x430, 0x435, 0x432, 0x430, 0x421, 0x430, 0x440, 0x430, 0x442, 0x430,
0x45e, 0x421, 0x456, 0x43c, 0x444, 0x435, 0x440, 0x43e, 0x43f, 0x430, 0x43b, 0x44c,
0x421, 0x43a, 0x43e, 0x43f, 0x2019, 0x435, 0x421, 0x430, 0x444, 0x456, 0x44f, 0x421,
0x442, 0x430, 0x43a, 0x433, 0x43e, 0x43b, 0x44c, 0x43c, 0x422, 0x430, 0x43b, 0x456,
0x43d, 0x422, 0x44b, 0x440, 0x430, 0x43d, 0x430, 0x423, 0x43b, 0x44c, 0x44f, 0x43d,
0x430, 0x45e, 0x441, 0x43a, 0x412, 0x430, 0x442, 0x44b, 0x43a, 0x430, 0x43d, 0x412,
0x456, 0x43b, 0x44c, 0x43d, 0x44e, 0x441, 0x412, 0x430, 0x43b, 0x433, 0x430, 0x433,
0x440, 0x430, 0x434, 0x417, 0x430, 0x433, 0x440, 0x430, 0x431, 0x426, 0x44e, 0x440,
0x44b, 0x445, 0x410, 0x43d, 0x442, 0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x44b,
0x432, 0x443, 0x427, 0x430, 0x433, 0x430, 0x441, 0x412, 0x43e, 0x441, 0x442, 0x440,
0x430, 0x45e, 0x20, 0x41a, 0x430, 0x43b, 0x44f, 0x434, 0x41a, 0x430, 0x43a, 0x43e,
0x441, 0x430, 0x432, 0x44b, 0x44f, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432,
0x44b, 0x41a, 0x430, 0x43c, 0x43e, 0x440, 0x441, 0x43a, 0x456, 0x44f, 0x20, 0x430,
0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x41c, 0x430, 0x44d, 0x41c, 0x430, 0x43b,
0x44c, 0x434, 0x44b, 0x432, 0x44b, 0x41c, 0x430, 0x45e, 0x440, 0x44b, 0x43a, 0x456,
0x439, 0x41c, 0x430, 0x451, 0x442, 0x430, 0x420, 0x44d, 0x44e, 0x43d, 0x44c, 0x451,
0x43d, 0x410, 0x43f, 0x456, 0x44f, 0x411, 0x443, 0x433, 0x435, 0x43d, 0x432, 0x456,
0x43b, 0x44c, 0x427, 0x430, 0x442, 0x44d, 0x43c, 0x412, 0x44f, 0x43b, 0x456, 0x43a,
0x430, 0x434, 0x43d, 0x44f, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x45e,
0x42d, 0x444, 0x430, 0x442, 0x44d, 0x424, 0x430, 0x43a, 0x430, 0x43e, 0x444, 0x430,
0x424, 0x456, 0x434, 0x436, 0x44b, 0x424, 0x443, 0x43d, 0x430, 0x444, 0x443, 0x446,
0x456, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x430, 0x441, 0x410, 0x441,
0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x413, 0x430, 0x43c, 0x431, 0x2019, 0x435,
0x413, 0x430, 0x43d, 0x430, 0x43b, 0x443, 0x43b, 0x443, 0x42d, 0x43d, 0x434, 0x44d,
0x440, 0x431, 0x435, 0x440, 0x44b, 0x41a, 0x456, 0x440, 0x44b, 0x446, 0x456, 0x43c,
0x430, 0x446, 0x456, 0x41a, 0x443, 0x441, 0x430, 0x456, 0x435, 0x41a, 0x432, 0x430,
0x434, 0x436, 0x430, 0x43b, 0x435, 0x439, 0x43d, 0x41c, 0x430, 0x434, 0x436, 0x443,
0x440, 0x430, 0x41c, 0x430, 0x440, 0x43a, 0x456, 0x437, 0x441, 0x43a, 0x456, 0x44f,
0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x41c, 0x456, 0x434, 0x443,
0x44d, 0x439, 0x41d, 0x456, 0x443, 0x44d, 0x41d, 0x43e, 0x440, 0x444, 0x430, 0x43b,
0x43a, 0x41f, 0x430, 0x433, 0x430, 0x2d, 0x41f, 0x430, 0x433, 0x430, 0x41f, 0x456,
0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x41f, 0x430, 0x43d, 0x43f, 0x435, 0x456, 0x41f,
0x43e, 0x440, 0x442, 0x2d, 0x41c, 0x43e, 0x440, 0x441, 0x431, 0x456, 0x420, 0x430,
0x440, 0x430, 0x442, 0x43e, 0x43d, 0x433, 0x430, 0x421, 0x430, 0x439, 0x43f, 0x430,
0x43d, 0x422, 0x430, 0x456, 0x446, 0x456, 0x422, 0x430, 0x43d, 0x433, 0x430, 0x442,
0x430, 0x43f, 0x443, 0x423, 0x44d, 0x439, 0x43a, 0x423, 0x43e, 0x43b, 0x456, 0x441,
0x905, 0x92c, 0x93f, 0x926, 0x91c, 0x93e, 0x928, 0x906, 0x915, 0x94d, 0x930, 0x93e,
0x90f, 0x926, 0x94d, 0x926, 0x93f, 0x938, 0x20, 0x906, 0x92c, 0x93e, 0x92c, 0x93e,
0x906, 0x932, 0x94d, 0x91c, 0x940, 0x90f, 0x930, 0x94d, 0x938, 0x906, 0x938, 0x94d,
0x92e, 0x93e, 0x930, 0x93e, 0x92c, 0x93e, 0x92e, 0x93e, 0x915, 0x2019, 0x92c, 0x93e,
0x902, 0x917, 0x942, 0x908, 0x92c, 0x928, 0x94d, 0x91c, 0x941, 0x932, 0x92c, 0x93f,
0x938, 0x94d, 0x938, 0x93e, 0x909, 0x92c, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x91f,
0x93e, 0x908, 0x930, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x94d, 0x91c, 0x93e, 0x92d,
0x940, 0x932, 0x92c, 0x941, 0x91c, 0x941, 0x92e, 0x94d, 0x92c, 0x941, 0x930, 0x93e,
0x915, 0x93e, 0x92f, 0x930, 0x2019, 0x915, 0x93e, 0x938, 0x93e, 0x92c, 0x94d, 0x932,
0x93e, 0x902, 0x915, 0x93e, 0x938, 0x947, 0x909, 0x924, 0x93e, 0x915, 0x928, 0x93e,
0x915, 0x94d, 0x930, 0x940, 0x921, 0x93e, 0x915, 0x93e, 0x930, 0x926, 0x93e, 0x930,
0x20, 0x90f, 0x938, 0x20, 0x938, 0x932, 0x93e, 0x92e, 0x91c, 0x93f, 0x92c, 0x94c,
0x91f, 0x940, 0x926, 0x941, 0x906, 0x932, 0x93e, 0x90f, 0x932, 0x20, 0x906, 0x907,
0x90a, 0x928, 0x92b, 0x94d, 0x930, 0x940, 0x91f, 0x93e, 0x909, 0x928, 0x917, 0x948,
0x92c, 0x94b, 0x930, 0x94b, 0x928, 0x939, 0x93e, 0x930, 0x93e, 0x930, 0x947, 0x91c,
0x939, 0x93e, 0x928, 0x94d, 0x938, 0x92c, 0x93e, 0x930, 0x94d, 0x917, 0x91c, 0x941,
0x92c, 0x93e, 0x915, 0x93e, 0x92e, 0x94d, 0x92a, 0x93e, 0x932, 0x93e, 0x916, 0x93e,
0x930, 0x94d, 0x924, 0x942, 0x92e, 0x915, 0x93f, 0x917, 0x93e, 0x932, 0x93f, 0x915,
0x93f, 0x928, 0x94d, 0x936, 0x93e, 0x938, 0x93e, 0x932, 0x93e, 0x917, 0x2019, 0x938,
0x932, 0x93f, 0x92c, 0x94d, 0x930, 0x947, 0x92d, 0x93f, 0x932, 0x932, 0x92e, 0x947,
0x932, 0x941, 0x906, 0x928, 0x94d, 0x921, 0x93e, 0x932, 0x941, 0x92c, 0x941, 0x92e,
0x94d, 0x92c, 0x93e, 0x936, 0x940, 0x932, 0x941, 0x938, 0x93e, 0x915, 0x93e, 0x92e,
0x93e, 0x932, 0x93e, 0x92c, 0x2019, 0x92e, 0x93e, 0x92a, 0x941, 0x91f, 0x2019, 0x92e,
0x93e, 0x938, 0x947, 0x930, 0x942, 0x92e, 0x94d, 0x92c, 0x93e, 0x92c, 0x93e, 0x928,
0x947, 0x92e, 0x917, 0x93e, 0x921, 0x93f, 0x936, 0x941, 0x92e, 0x928, 0x94d, 0x930,
0x2019, 0x92d, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x908, 0x930, 0x94c, 0x92c, 0x940,
0x928, 0x94d, 0x91c, 0x93e, 0x92e, 0x947, 0x928, 0x93e, 0x928, 0x940, 0x906, 0x92e,
0x947, 0x928, 0x94d, 0x935, 0x93e, 0x915, 0x936, 0x2019, 0x924, 0x90a, 0x917, 0x93e,
0x921, 0x941, 0x917, 0x941, 0x92a, 0x930, 0x94d, 0x91f, 0x2019, 0x2d, 0x928, 0x92d,
0x2019, 0x938, 0x93e, 0x935, 0x20, 0x91f, 0x92e, 0x947, 0x924, 0x94d, 0x930, 0x93f,
0x92a, 0x2019, 0x932, 0x940, 0x91f, 0x94d, 0x92f, 0x942, 0x928, 0x93f, 0x938, 0x935,
0x93f, 0x928, 0x94d, 0x926, 0x939, 0x2019, 0x915, 0x90f, 0x921, 0x947, 0x915, 0x90f,
0x902, 0x915, 0x930, 0x947, 0x91c, 0x90f, 0x902, 0x917, 0x94d, 0x935, 0x940, 0x932,
0x93e, 0x90f, 0x928, 0x94d, 0x91f, 0x93f, 0x917, 0x941, 0x935, 0x93e, 0x906, 0x930,
0x93e, 0x917, 0x941, 0x906, 0x908, 0x928, 0x93e, 0x92c, 0x941, 0x90f, 0x928, 0x2019,
0x938, 0x20, 0x90f, 0x930, 0x947, 0x938, 0x915, 0x93e, 0x91f, 0x93e, 0x92e, 0x93e,
0x930, 0x94d, 0x915, 0x93e, 0x915, 0x930, 0x94d, 0x921, 0x92c, 0x93e, 0x91c, 0x941,
0x91c, 0x941, 0x908, 0x932, 0x93e, 0x20, 0x930, 0x93f, 0x905, 0x91c, 0x93e, 0x92e,
0x947, 0x928, 0x94d, 0x921, 0x2019, 0x91c, 0x93e, 0x930, 0x93f, 0x905, 0x20, 0x917,
0x93e, 0x932, 0x947, 0x917, 0x938, 0x938, 0x93e, 0x932, 0x94d, 0x91f, 0x93e, 0x938,
0x93e, 0x928, 0x20, 0x939, 0x941, 0x935, 0x93e, 0x928, 0x938, 0x93e, 0x928, 0x20,
0x932, 0x941, 0x908, 0x938, 0x91f, 0x941, 0x915, 0x941, 0x92e, 0x93e, 0x928, 0x909,
0x936, 0x942, 0x906, 0x92f, 0x93f, 0x92f, 0x93e, 0x906, 0x930, 0x942, 0x92c, 0x93e,
0x906, 0x938, 0x941, 0x928, 0x938, 0x940, 0x905, 0x928, 0x906, 0x924, 0x93f, 0x915,
0x915, 0x93e, 0x928, 0x92c, 0x93e, 0x939, 0x93f, 0x92f, 0x93e, 0x92c, 0x93e, 0x939,
0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x93e, 0x928, 0x94d, 0x926, 0x947, 0x930, 0x93e,
0x938, 0x92c, 0x93e, 0x930, 0x94d, 0x92c, 0x93e, 0x921, 0x2019, 0x938, 0x92c, 0x947,
0x932, 0x947, 0x92e, 0x92c, 0x947, 0x932, 0x93f, 0x91c, 0x92c, 0x94d, 0x932, 0x93e,
0x902, 0x915, 0x2d, 0x20, 0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x928, 0x92c, 0x906,
0x20, 0x92d, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x92c, 0x917, 0x2019, 0x91f, 0x93e,
0x92c, 0x92f, 0x938, 0x93f, 0x915, 0x947, 0x92e, 0x94d, 0x92c, 0x94d, 0x930, 0x93f,
0x91c, 0x20, 0x917, 0x93e, 0x925, 0x94b, 0x928, 0x915, 0x947, 0x92e, 0x94d, 0x92a,
0x20, 0x917, 0x94d, 0x930, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x93e, 0x902, 0x915,
0x941, 0x928, 0x915, 0x93e, 0x930, 0x93e, 0x915, 0x93e, 0x938, 0x915, 0x93e, 0x92f,
0x947, 0x928, 0x915, 0x948, 0x92e, 0x947, 0x928, 0x936, 0x93f, 0x915, 0x93e, 0x917,
0x2019, 0x91a, 0x941, 0x939, 0x92d, 0x93e, 0x92d, 0x93e, 0x915, 0x938, 0x94d, 0x91f,
0x93e, 0x20, 0x930, 0x93f, 0x915, 0x93e, 0x915, 0x94d, 0x930, 0x947, 0x938, 0x94d,
0x91f, 0x928, 0x915, 0x941, 0x908, 0x906, 0x92c, 0x93e, 0x915, 0x942, 0x930, 0x93e,
0x938, 0x93e, 0x935, 0x921, 0x93e, 0x928, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x938,
0x94d, 0x92d, 0x93e, 0x928, 0x921, 0x93e, 0x935, 0x938, 0x2019, 0x928, 0x921, 0x93e,
0x935, 0x938, 0x2019, 0x928, 0x20, 0x915, 0x94d, 0x930, 0x940, 0x915, 0x921, 0x947,
0x928, 0x92d, 0x94b, 0x930, 0x921, 0x947, 0x91f, 0x94d, 0x930, 0x2019, 0x908, 0x924,
0x921, 0x92e, 0x93f, 0x928, 0x93f, 0x915, 0x93e, 0x90f, 0x921, 0x92e, 0x928, 0x94d,
0x91f, 0x928, 0x90f, 0x908, 0x930, 0x941, 0x928, 0x947, 0x92a, 0x947, 0x90f, 0x932,
0x20, 0x938, 0x93e, 0x932, 0x94d, 0x92d, 0x93e, 0x921, 0x930, 0x92b, 0x930, 0x94d,
0x924, 0x20, 0x928, 0x947, 0x932, 0x938, 0x928, 0x92b, 0x930, 0x94d, 0x91f, 0x93e,
0x932, 0x947, 0x91c, 0x93e, 0x917, 0x94d, 0x932, 0x947, 0x938, 0x20, 0x92c, 0x947,
0x907, 0x917, 0x942, 0x91c, 0x20, 0x92c, 0x947, 0x907, 0x917, 0x94d, 0x930, 0x947,
0x923, 0x94d, 0x921, 0x20, 0x924, 0x93e, 0x930, 0x94d, 0x915, 0x917, 0x94d, 0x930,
0x947, 0x928, 0x93e, 0x921, 0x93e, 0x917, 0x94d, 0x935, 0x93e, 0x926, 0x947, 0x932,
0x942, 0x92a, 0x917, 0x94d, 0x935, 0x93e, 0x91f, 0x947, 0x92e, 0x93e, 0x932, 0x93e,
0x917, 0x941, 0x906, 0x92f, 0x93e, 0x915, 0x93f, 0x932, 0x917, 0x941, 0x92f, 0x93e,
0x928, 0x93e, 0x939, 0x947, 0x932, 0x93f, 0x92b, 0x947, 0x915, 0x94d, 0x938, 0x939,
0x93e, 0x92d, 0x93e, 0x928, 0x93e, 0x939, 0x93e, 0x930, 0x94d, 0x92e, 0x2019, 0x938,
0x93f, 0x932, 0x94d, 0x932, 0x2019, 0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x92a, 0x932, 0x93f, 0x938, 0x928, 0x915, 0x94d, 0x938, 0x2c, 0x20,
0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92e, 0x93e, 0x930,
0x947, 0x919, 0x2019, 0x2c, 0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x92a, 0x93f, 0x924, 0x93e, 0x930, 0x94d, 0x938, 0x92c, 0x93e, 0x930,
0x94d, 0x917, 0x2c, 0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x924, 0x947, 0x932, 0x20, 0x928, 0x94b, 0x917, 0x94b, 0x930, 0x2c, 0x20,
0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92d, 0x947, 0x92d,
0x948, 0x2c, 0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e,
0x92d, 0x93e, 0x928, 0x938, 0x947, 0x928, 0x94d, 0x928, 0x947, 0x938, 0x2c, 0x20,
0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x928,
0x93e, 0x92e, 0x947, 0x915, 0x2c, 0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f,
0x93e, 0x928, 0x93e, 0x907, 0x928, 0x942, 0x935, 0x93f, 0x915, 0x907, 0x915, 0x93e,
0x932, 0x941, 0x908, 0x924, 0x91c, 0x93e, 0x92e, 0x93e, 0x908, 0x915, 0x93e, 0x91c,
0x941, 0x928, 0x94b, 0x932, 0x941, 0x908, 0x935, 0x940, 0x932, 0x92e, 0x928, 0x94d,
0x91f, 0x93f, 0x91b, 0x947, 0x932, 0x2019, 0x2c, 0x20, 0x915, 0x947, 0x928, 0x94d,
0x91f, 0x93e, 0x915, 0x940, 0x915, 0x94d, 0x930, 0x93e, 0x932, 0x947, 0x923, 0x91c,
0x940, 0x915, 0x932, 0x93e, 0x20, 0x92a, 0x93e, 0x91c, 0x932, 0x940, 0x92e, 0x93e,
0x932, 0x938, 0x20, 0x90f, 0x928, 0x94d, 0x91c, 0x947, 0x932, 0x94b, 0x938, 0x932,
0x935, 0x93e, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938, 0x94b,
0x938, 0x20, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x94d, 0x924, 0x93e, 0x930, 0x92e,
0x93e, 0x91a, 0x947, 0x908, 0x913, 0x92e, 0x928, 0x93e, 0x917, 0x941, 0x906, 0x92e,
0x928, 0x93e, 0x909, 0x938, 0x92e, 0x947, 0x930, 0x93f, 0x917, 0x924, 0x92e, 0x93e,
0x930, 0x94d, 0x91f, 0x940, 0x928, 0x940, 0x915, 0x92e, 0x947, 0x924, 0x93e, 0x92e,
0x930, 0x938, 0x92e, 0x93e, 0x91c, 0x93e, 0x91f, 0x932, 0x93e, 0x928, 0x92e, 0x947,
0x928, 0x2019, 0x92e, 0x940, 0x928, 0x940, 0x92e, 0x947, 0x930, 0x93f, 0x921, 0x93e,
0x92e, 0x947, 0x924, 0x94d, 0x932, 0x93e, 0x915, 0x93e, 0x924, 0x94d, 0x932, 0x93e,
0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x2019, 0x20, 0x938, 0x93f, 0x924,
0x940, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x2019, 0x928, 0x92e, 0x915,
0x94d, 0x91f, 0x2019, 0x928, 0x92e, 0x928, 0x94d, 0x91f, 0x947, 0x930, 0x94d, 0x930,
0x947, 0x92e, 0x928, 0x94d, 0x91f, 0x947, 0x92d, 0x93f, 0x921, 0x93f, 0x905, 0x92e,
0x928, 0x938, 0x947, 0x930, 0x94d, 0x930, 0x93e, 0x924, 0x928, 0x93e, 0x938, 0x94d,
0x938, 0x93e, 0x909, 0x928, 0x93f, 0x909, 0x20, 0x92f, 0x930, 0x94d, 0x915, 0x928,
0x930, 0x2019, 0x928, 0x94d, 0x939, 0x93e, 0x92c, 0x947, 0x90a, 0x932, 0x93e, 0x939,
0x2c, 0x20, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x921, 0x93e, 0x915, 0x2019, 0x924,
0x93e, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x94b, 0x930, 0x2c, 0x20, 0x938, 0x93e,
0x939, 0x93e, 0x20, 0x921, 0x93e, 0x915, 0x2019, 0x924, 0x93e, 0x928, 0x93f, 0x909,
0x20, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x938, 0x93e, 0x939, 0x93e,
0x20, 0x921, 0x93e, 0x915, 0x2019, 0x924, 0x93e, 0x928, 0x942, 0x915, 0x906, 0x91c,
0x93f, 0x928, 0x93e, 0x917, 0x93e, 0x92a, 0x93e, 0x928, 0x93e, 0x92e, 0x93e, 0x92a,
0x93e, 0x930, 0x93e, 0x92e, 0x93e, 0x930, 0x93f, 0x92c, 0x2019, 0x92b, 0x940, 0x928,
0x93f, 0x915, 0x94d, 0x938, 0x92a, 0x930, 0x94d, 0x91f, 0x2d, 0x906, 0x90a, 0x2d,
0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938, 0x92a, 0x930, 0x94d, 0x91f, 0x20,
0x905, 0x92b, 0x20, 0x938, 0x94d, 0x92a, 0x948, 0x928, 0x92a, 0x930, 0x94d, 0x91f,
0x2019, 0x20, 0x92d, 0x947, 0x932, 0x94d, 0x939, 0x2019, 0x92a, 0x941, 0x90f, 0x930,
0x94d, 0x91f, 0x2019, 0x20, 0x930, 0x93f, 0x915, 0x2019, 0x92a, 0x941, 0x928, 0x94d,
0x91f, 0x93e, 0x20, 0x906, 0x930, 0x947, 0x928, 0x93e, 0x938, 0x930, 0x947, 0x928,
0x915, 0x93f, 0x928, 0x20, 0x907, 0x928, 0x932, 0x947, 0x91f, 0x930, 0x947, 0x938,
0x93f, 0x92b, 0x947, 0x930, 0x947, 0x91c, 0x93f, 0x928, 0x93e, 0x930, 0x947, 0x91c,
0x932, 0x94d, 0x92f, 0x941, 0x924, 0x930, 0x93f, 0x905, 0x20, 0x92c, 0x94d, 0x930,
0x93e, 0x928, 0x94d, 0x915, 0x2019, 0x938, 0x93e, 0x928, 0x94d, 0x924, 0x93e, 0x930,
0x947, 0x92e, 0x938, 0x93e, 0x928, 0x94d, 0x924, 0x93f, 0x906, 0x917, 0x2019, 0x938,
0x93e, 0x928, 0x94d, 0x924, 0x94b, 0x20, 0x921, 0x92e, 0x93f, 0x917, 0x2019, 0x938,
0x93e, 0x935, 0x20, 0x92a, 0x93e, 0x935, 0x932, 0x2019, 0x908, 0x924, 0x94d, 0x924,
0x2019, 0x915, 0x94d, 0x935, 0x930, 0x94d, 0x924, 0x942, 0x92e, 0x940, 0x938, 0x93f,
0x924, 0x94d, 0x915, 0x93e, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x92c, 0x93e,
0x930, 0x94d, 0x925, 0x947, 0x932, 0x947, 0x92e, 0x940, 0x938, 0x948, 0x928, 0x94d,
0x91f, 0x20, 0x91c, 0x928, 0x94d, 0x938, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20,
0x915, 0x940, 0x924, 0x94d, 0x938, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x932,
0x941, 0x938, 0x93f, 0x92f, 0x93e, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x925,
0x92e, 0x93e, 0x938, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x92d, 0x93f, 0x928,
0x94d, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x938, 0x941, 0x908, 0x92b, 0x924, 0x20,
0x915, 0x93e, 0x930, 0x94d, 0x930, 0x947, 0x928, 0x94d, 0x924, 0x924, 0x947, 0x917,
0x941, 0x938, 0x93f, 0x917, 0x93e, 0x932, 0x94d, 0x92a, 0x93e, 0x925, 0x94d, 0x92f,
0x942, 0x932, 0x91f, 0x93f, 0x939, 0x941, 0x906, 0x928, 0x93e, 0x91f, 0x930, 0x928,
0x94d, 0x91f, 0x2019, 0x91f, 0x930, 0x94d, 0x924, 0x2019, 0x932, 0x93e, 0x92d, 0x947,
0x928, 0x915, 0x942, 0x92d, 0x930, 0x935, 0x94d, 0x939, 0x93e, 0x908, 0x924, 0x939,
0x930, 0x94d, 0x938, 0x935, 0x940, 0x928, 0x940, 0x92a, 0x947, 0x917, 0x92f, 0x93e,
0x915, 0x941, 0x924, 0x93e, 0x924, 0x915, 0x947, 0x938, 0x940, 0x921, 0x947, 0x92d,
0x93f, 0x938, 0x921, 0x941, 0x92e, 0x928, 0x94d, 0x924, 0x2d, 0x921, 0x940, 0x906,
0x930, 0x94d, 0x92d, 0x940, 0x932, 0x92e, 0x93e, 0x915, 0x94d, 0x935, 0x93e, 0x930,
0x940, 0x92e, 0x94c, 0x938, 0x928, 0x92e, 0x947, 0x915, 0x92e, 0x93e, 0x930, 0x94d,
0x921, 0x2019, 0x92a, 0x93e, 0x932, 0x92e, 0x93e, 0x930, 0x930, 0x925, 0x947, 0x930,
0x93e, 0x938, 0x940, 0x905, 0x935, 0x93e, 0x924, 0x94d, 0x930, 0x2019, 0x932, 0x92d,
0x938, 0x94d, 0x924, 0x2019, 0x915, 0x932, 0x902, 0x92f, 0x93e, 0x930, 0x92c, 0x94d,
0x92f, 0x947, 0x928, 0x90f, 0x921, 0x947, 0x928, 0x906, 0x932, 0x94d, 0x92e, 0x93e,
0x91f, 0x940, 0x906, 0x92e, 0x94d, 0x92e, 0x93e, 0x928, 0x906, 0x928, 0x93e, 0x921,
0x940, 0x930, 0x905, 0x915, 0x94d, 0x924, 0x93e, 0x909, 0x906, 0x915, 0x94d, 0x924,
0x2019, 0x92c, 0x947, 0x905, 0x936, 0x94d, 0x917, 0x93e, 0x92c, 0x93e, 0x924, 0x906,
0x924, 0x940, 0x930, 0x93e, 0x909, 0x92c, 0x93e, 0x918, 0x926, 0x93e, 0x926, 0x92c,
0x93e, 0x939, 0x930, 0x948, 0x928, 0x92c, 0x948, 0x902, 0x917, 0x915, 0x949, 0x915,
0x92c, 0x93e, 0x930, 0x94d, 0x928, 0x94c, 0x932, 0x92c, 0x948, 0x930, 0x941, 0x924,
0x92c, 0x93f, 0x936, 0x94d, 0x915, 0x947, 0x915, 0x92c, 0x94d, 0x930, 0x941, 0x928,
0x907, 0x91a, 0x940, 0x924, 0x93e, 0x915, 0x932, 0x92e, 0x94d, 0x92c, 0x2019, 0x926,
0x93e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x938, 0x927, 0x93e, 0x915, 0x93e,
0x921, 0x93f, 0x932, 0x940, 0x926, 0x941, 0x92c, 0x93e, 0x908, 0x926, 0x941, 0x936,
0x93e, 0x928, 0x94d, 0x92c, 0x947, 0x92b, 0x93e, 0x92e, 0x93e, 0x917, 0x941, 0x938,
0x94d, 0x924, 0x93e, 0x917, 0x93e, 0x91c, 0x93e, 0x939, 0x947, 0x92c, 0x94d, 0x930,
0x2019, 0x928, 0x938, 0x93e, 0x907, 0x917, 0x949, 0x928, 0x939, 0x902, 0x915, 0x902,
0x939, 0x2019, 0x92d, 0x94d, 0x926, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x91c, 0x93e, 0x915, 0x93e, 0x930, 0x94d, 0x924, 0x93e, 0x91c,
0x92f, 0x93e, 0x92a, 0x941, 0x930, 0x93e, 0x91c, 0x947, 0x930, 0x941, 0x91c, 0x93e,
0x932, 0x947, 0x92e, 0x915, 0x93e, 0x92c, 0x942, 0x932, 0x915, 0x93e, 0x92e, 0x91a,
0x93e, 0x91f, 0x915, 0x93e, 0x915, 0x930, 0x93e, 0x91a, 0x940, 0x915, 0x93e, 0x920,
0x92e, 0x93e, 0x928, 0x94d, 0x921, 0x941, 0x916, 0x93e, 0x928, 0x94d, 0x926, 0x940,
0x917, 0x93e, 0x915, 0x932, 0x915, 0x93e, 0x924, 0x93e, 0x915, 0x94d, 0x930, 0x93e,
0x938, 0x94d, 0x928, 0x2019, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x915,
0x94d, 0x935, 0x93e, 0x932, 0x93e, 0x20, 0x932, 0x92e, 0x94d, 0x92a, 0x941, 0x930,
0x915, 0x942, 0x91a, 0x93f, 0x902, 0x917, 0x915, 0x941, 0x935, 0x948, 0x924, 0x92e,
0x93e, 0x915, 0x93e, 0x90a, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x92e,
0x93e, 0x915, 0x93e, 0x938, 0x94d, 0x938, 0x93e, 0x930, 0x92e, 0x928, 0x940, 0x932,
0x93e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x947, 0x91f, 0x928, 0x93f, 0x915, 0x2019,
0x938, 0x93f, 0x92f, 0x93e, 0x928, 0x92d, 0x2019, 0x915, 0x941, 0x91c, 0x94d, 0x928,
0x947, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x928, 0x92d, 0x2019, 0x938, 0x93f, 0x92c,
0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x905, 0x92e, 0x94d, 0x938, 0x94d, 0x915,
0x905, 0x930, 0x93e, 0x932, 0x928, 0x949, 0x92e, 0x20, 0x92a, 0x947, 0x928, 0x92a,
0x94b, 0x902, 0x924, 0x93f, 0x906, 0x928, 0x93e, 0x915, 0x92a, 0x94d, 0x92f, 0x902,
0x92f, 0x93e, 0x902, 0x915, 0x93e, 0x924, 0x93e, 0x930, 0x915, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x948, 0x915, 0x940, 0x91c, 0x940, 0x932, 0x930, 0x94d, 0x921, 0x93e,
0x930, 0x93f, 0x92f, 0x93e, 0x927, 0x938, 0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928,
0x938, 0x92e, 0x930, 0x915, 0x93e, 0x928, 0x94d, 0x926, 0x938, 0x2019, 0x909, 0x932,
0x936, 0x93e, 0x902, 0x918, 0x93e, 0x908, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a,
0x941, 0x930, 0x938, 0x94d, 0x930, 0x947, 0x926, 0x928, 0x947, 0x915, 0x2019, 0x932,
0x93f, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x924, 0x93e, 0x908, 0x92a, 0x947, 0x907,
0x924, 0x93e, 0x936, 0x915, 0x947, 0x928, 0x94d, 0x924, 0x924, 0x93f, 0x92c, 0x93f,
0x932, 0x93f, 0x938, 0x940, 0x924, 0x947, 0x939, 0x930, 0x93e, 0x928, 0x925, 0x93f,
0x92e, 0x94d, 0x92b, 0x941, 0x91f, 0x2019, 0x915, 0x94d, 0x92f, 0x2019, 0x924, 0x92e,
0x94d, 0x938, 0x94d, 0x915, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x93e, 0x924, 0x93e,
0x930, 0x909, 0x930, 0x941, 0x92e, 0x915, 0x940, 0x906, 0x938, 0x94d, 0x924, 0x2d,
0x928, 0x947, 0x930, 0x93e, 0x935, 0x93f, 0x92f, 0x902, 0x938, 0x93f, 0x92f, 0x93e,
0x928, 0x92d, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x938, 0x94d, 0x924, 0x2019,
0x915, 0x92f, 0x93e, 0x915, 0x942, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x930, 0x902,
0x917, 0x942, 0x928, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x940, 0x928,
0x93e, 0x92c, 0x941, 0x930, 0x94d, 0x917, 0x92f, 0x947, 0x930, 0x947, 0x92d, 0x93e,
0x928, 0x906, 0x91c, 0x2019, 0x930, 0x94d, 0x938, 0x92c, 0x93e, 0x930, 0x94d, 0x92e,
0x941, 0x921, 0x93e, 0x915, 0x947, 0x928, 0x947, 0x930, 0x940, 0x915, 0x947, 0x92a,
0x20, 0x92d, 0x947, 0x930, 0x94d, 0x926, 0x947, 0x92b, 0x947, 0x930, 0x94c, 0x92e,
0x93e, 0x921, 0x948, 0x930, 0x93e, 0x930, 0x940, 0x915, 0x92f, 0x93e, 0x92d, 0x93f,
0x915, 0x916, 0x94b, 0x932, 0x93e, 0x20, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x939, 0x947, 0x932, 0x947, 0x928,
0x93e, 0x938, 0x94d, 0x91f, 0x947, 0x928, 0x932, 0x940, 0x90f, 0x921, 0x947, 0x932,
0x948, 0x921, 0x92c, 0x94d, 0x930, 0x940, 0x938, 0x92c, 0x947, 0x928, 0x92c, 0x94d,
0x930, 0x2019, 0x915, 0x947, 0x928, 0x20, 0x939, 0x93f, 0x932, 0x921, 0x93e, 0x930,
0x94d, 0x935, 0x940, 0x928, 0x92f, 0x942, 0x915, 0x94d, 0x932, 0x93e, 0x939, 0x92c,
0x93e, 0x930, 0x94d, 0x91f, 0x932, 0x93f, 0x923, 0x94d, 0x921, 0x92e, 0x947, 0x928,
0x932, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935, 0x93f, 0x92e, 0x947, 0x932,
0x92c, 0x94c, 0x930, 0x94d, 0x928, 0x92a, 0x94b, 0x930, 0x94d, 0x925, 0x938, 0x93f,
0x921, 0x928, 0x940, 0x910, 0x92e, 0x94d, 0x938, 0x94d, 0x91f, 0x930, 0x921, 0x93e,
0x92e, 0x906, 0x928, 0x94d, 0x921, 0x2019, 0x930, 0x93e, 0x906, 0x938, 0x94d, 0x924,
0x94d, 0x930, 0x93e, 0x916, 0x93e, 0x928, 0x90f, 0x925, 0x947, 0x928, 0x94d, 0x938,
0x92c, 0x947, 0x932, 0x917, 0x94d, 0x930, 0x947, 0x921, 0x92c, 0x93e, 0x930, 0x94d,
0x932, 0x93f, 0x928, 0x92c, 0x94d, 0x930, 0x93e, 0x924, 0x93f, 0x938, 0x94d, 0x932,
0x93e, 0x92d, 0x93e, 0x92c, 0x94d, 0x930, 0x941, 0x938, 0x94d, 0x938, 0x947, 0x932,
0x94d, 0x938, 0x92c, 0x941, 0x915, 0x93e, 0x930, 0x947, 0x938, 0x94d, 0x91f, 0x92c,
0x941, 0x926, 0x93e, 0x92a, 0x947, 0x938, 0x94d, 0x91f, 0x92c, 0x941, 0x938, 0x93f,
0x902, 0x917, 0x947, 0x928, 0x91a, 0x93f, 0x938, 0x940, 0x928, 0x94c, 0x915, 0x92a,
0x947, 0x928, 0x939, 0x947, 0x917, 0x947, 0x928, 0x921, 0x93e, 0x92c, 0x932, 0x93f,
0x928, 0x91c, 0x93f, 0x92c, 0x94d, 0x930, 0x93e, 0x932, 0x94d, 0x91f, 0x93e, 0x930,
0x917, 0x93e, 0x930, 0x94d, 0x928, 0x938, 0x940, 0x939, 0x947, 0x932, 0x94d, 0x938,
0x93f, 0x902, 0x915, 0x940, 0x906, 0x92f, 0x94b, 0x932, 0x20, 0x905, 0x92b, 0x20,
0x92e, 0x947, 0x928, 0x907, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x92c, 0x941, 0x932,
0x91c, 0x93e, 0x930, 0x94d, 0x938, 0x940, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x940,
0x902, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x915, 0x93f, 0x930, 0x2019, 0x92d, 0x915,
0x940, 0x92d, 0x932, 0x93f, 0x938, 0x94d, 0x92c, 0x928, 0x91c, 0x941, 0x92c, 0x94d,
0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x93e, 0x932, 0x928, 0x94d, 0x926, 0x928, 0x932,
0x93e, 0x915, 0x94d, 0x91c, 0x947, 0x92e, 0x94d, 0x92c, 0x93e, 0x930, 0x94d, 0x917,
0x92e, 0x93e, 0x926, 0x94d, 0x930, 0x93f, 0x926, 0x92e, 0x93e, 0x932, 0x94d, 0x91f,
0x93e, 0x92e, 0x93e, 0x930, 0x948, 0x939, 0x93e, 0x92e, 0x92e, 0x93f, 0x928, 0x94d,
0x938, 0x94d, 0x915, 0x92e, 0x928, 0x93e, 0x915, 0x2019, 0x92e, 0x938, 0x94d, 0x915,
0x94c, 0x905, 0x938, 0x94d, 0x932, 0x2019, 0x92a, 0x947, 0x930, 0x93f, 0x938, 0x92a,
0x926, 0x917, 0x2019, 0x930, 0x93f, 0x915, 0x93e, 0x92a, 0x94d, 0x930, 0x947, 0x917,
0x930, 0x93f, 0x917, 0x93e, 0x930, 0x2019, 0x92e, 0x938, 0x93e, 0x92e, 0x93e, 0x930,
0x93e, 0x938, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x930, 0x93f, 0x928, 0x2019, 0x938,
0x93e, 0x930, 0x93e, 0x91c, 0x947, 0x92d, 0x2019, 0x938, 0x93e, 0x930, 0x93e, 0x924,
0x2019, 0x92d, 0x938, 0x93f, 0x92e, 0x94d, 0x92b, 0x947, 0x930, 0x2019, 0x92a, 0x2019,
0x932, 0x938, 0x94d, 0x915, 0x2019, 0x92a, 0x94d, 0x91c, 0x947, 0x938, 0x92b, 0x93f,
0x92f, 0x93e, 0x938, 0x94d, 0x91f, 0x2019, 0x915, 0x939, 0x2019, 0x92e, 0x91f, 0x947,
0x932, 0x93f, 0x928, 0x91f, 0x93f, 0x930, 0x947, 0x928, 0x90a, 0x932, 0x94d, 0x92f,
0x93e, 0x928, 0x2019, 0x92d, 0x94d, 0x938, 0x94d, 0x915, 0x92d, 0x93e, 0x921, 0x941,
0x91c, 0x92d, 0x947, 0x91f, 0x93f, 0x915, 0x93e, 0x928, 0x92d, 0x93f, 0x92f, 0x947,
0x928, 0x94d, 0x928, 0x93e, 0x935, 0x93f, 0x932, 0x94d, 0x928, 0x93f, 0x909, 0x938,
0x92d, 0x932, 0x94d, 0x917, 0x2019, 0x917, 0x94d, 0x930, 0x947, 0x926, 0x935, 0x93e,
0x930, 0x938, 0x94c, 0x91c, 0x93e, 0x917, 0x94d, 0x930, 0x947, 0x92c, 0x91c, 0x941,
0x930, 0x93f, 0x916, 0x90f, 0x928, 0x94d, 0x91f, 0x93e, 0x928, 0x93e, 0x928, 0x93e,
0x930, 0x93f, 0x92d, 0x2019, 0x91a, 0x93e, 0x917, 0x2019, 0x938, 0x916, 0x94d, 0x930,
0x940, 0x938, 0x94d, 0x91f, 0x92e, 0x93e, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x915, 0x915, 0x2019, 0x938, 0x915, 0x92e, 0x2019, 0x930, 0x2019, 0x915, 0x947,
0x930, 0x94d, 0x917, 0x947, 0x932, 0x947, 0x928, 0x92e, 0x93e, 0x939, 0x947, 0x92e,
0x93e, 0x932, 0x926, 0x940, 0x92d, 0x94d, 0x938, 0x92e, 0x930, 0x93f, 0x936, 0x93f,
0x92f, 0x93e, 0x938, 0x92e, 0x93e, 0x92f, 0x2019, 0x924, 0x94d, 0x924, 0x947, 0x930,
0x947, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x2019, 0x928, 0x906, 0x92a, 0x93f, 0x92f,
0x93e, 0x914, 0x915, 0x932, 0x948, 0x923, 0x94d, 0x921, 0x92c, 0x941, 0x917, 0x948,
0x928, 0x92d, 0x940, 0x932, 0x91a, 0x93e, 0x925, 0x93e, 0x92e, 0x91a, 0x942, 0x915,
0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x90f, 0x92b, 0x93e,
0x91f, 0x947, 0x92b, 0x93e, 0x915, 0x93e, 0x905, 0x92b, 0x2019, 0x92b, 0x93f, 0x91c,
0x940, 0x92b, 0x941, 0x928, 0x93e, 0x92b, 0x942, 0x91f, 0x940, 0x917, 0x93e, 0x932,
0x93e, 0x92a, 0x93e, 0x917, 0x2019, 0x938, 0x917, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f,
0x92f, 0x947, 0x930, 0x917, 0x94d, 0x935, 0x93e, 0x921, 0x93e, 0x932, 0x915, 0x947,
0x928, 0x947, 0x932, 0x917, 0x941, 0x935, 0x93e, 0x92e, 0x939, 0x928, 0x932, 0x941,
0x932, 0x941, 0x90f, 0x928, 0x94d, 0x921, 0x93e, 0x930, 0x92c, 0x93e, 0x930, 0x940,
0x915, 0x93f, 0x930, 0x940, 0x924, 0x93f, 0x92e, 0x93e, 0x924, 0x93f, 0x915, 0x938,
0x94d, 0x930, 0x947, 0x915, 0x94d, 0x935, 0x93e, 0x91c, 0x93e, 0x932, 0x948, 0x928,
0x92e, 0x93e, 0x91c, 0x942, 0x930, 0x2019, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x947,
0x938, 0x93e, 0x938, 0x92e, 0x940, 0x921, 0x935, 0x947, 0x928, 0x93e, 0x90a, 0x930,
0x941, 0x928, 0x940, 0x90a, 0x90f, 0x928, 0x930, 0x92b, 0x2019, 0x915, 0x928, 0x942,
0x92e, 0x93f, 0x92f, 0x93e, 0x92a, 0x93e, 0x917, 0x2019, 0x20, 0x92a, 0x93e, 0x917,
0x2019, 0x92a, 0x93e, 0x932, 0x93e, 0x909, 0x92a, 0x93f, 0x91f, 0x915, 0x948, 0x930,
0x94d, 0x928, 0x92a, 0x928, 0x93e, 0x92a, 0x947, 0x92a, 0x930, 0x94d, 0x91f, 0x20,
0x92e, 0x930, 0x94d, 0x938, 0x92c, 0x940, 0x930, 0x93e, 0x930, 0x2019, 0x91f, 0x902,
0x917, 0x93e, 0x938, 0x93e, 0x907, 0x92a, 0x93e, 0x928, 0x91f, 0x93e, 0x939, 0x93f,
0x91f, 0x940, 0x924, 0x93e, 0x930, 0x93e, 0x935, 0x93e, 0x91f, 0x902, 0x917, 0x93e,
0x91f, 0x93e, 0x92a, 0x941, 0x935, 0x947, 0x915, 0x935, 0x93e, 0x932, 0x93f, 0x938,
0x4b, 0x61, 0x7a, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x61, 0x44, 0x17e,
0x69, 0x62, 0x75, 0x74, 0x69, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x161,
0x41, 0x6e, 0x67, 0x76, 0x69, 0x6c, 0x61, 0x41, 0x6e, 0x74, 0x69, 0x67,
0x76, 0x61, 0x43, 0x61, 0x6e, 0x63, 0x75, 0x6e, 0x4b, 0x6f, 0x73, 0x74,
0x61, 0x72, 0x69, 0x6b, 0x61, 0x47, 0x76, 0x61, 0x64, 0x61, 0x6c, 0x75,
0x70, 0x65, 0x47, 0x76, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x4a,
0x61, 0x6d, 0x61, 0x6a, 0x6b, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x2c, 0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c,
0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x2c, 0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x72,
0x69, 0x6b, 0x6f, 0x41, 0x6b, 0x75, 0x74, 0x6f, 0x62, 0x65, 0x41, 0x161,
0x68, 0x61, 0x62, 0x61, 0x64, 0x41, 0x74, 0x69, 0x72, 0x61, 0x6a, 0x42,
0x69, 0x161, 0x6b, 0x65, 0x6b, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x69,
0x44, 0x75, 0x161, 0x61, 0x6e, 0x62, 0x65, 0x48, 0x6f, 0x20, 0x160, 0x69,
0x20, 0x4d, 0x69, 0x6e, 0x44, 0x17e, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61,
0x44, 0x17e, 0x61, 0x6a, 0x61, 0x70, 0x75, 0x72, 0x61, 0x4a, 0x65, 0x72,
0x75, 0x7a, 0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74,
0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x10d, 0x69, 0x48, 0x61, 0x6e, 0x64,
0x69, 0x67, 0x61, 0x4b, 0x75, 0x10d, 0x69, 0x6e, 0x67, 0x4d, 0x61, 0x6b,
0x61, 0x75, 0x4e, 0x69, 0x6b, 0x6f, 0x7a, 0x69, 0x6a, 0x61, 0x4e, 0x6f,
0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x6a, 0x65, 0x63, 0x6b, 0x50, 0x6a,
0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x52, 0x69, 0x6a, 0x61, 0x64,
0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x160, 0x61, 0x6e, 0x67, 0x61,
0x6a, 0x54, 0x61, 0x161, 0x6b, 0x65, 0x6e, 0x74, 0x55, 0x72, 0x75, 0x6d,
0x10d, 0x69, 0x56, 0x69, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6a, 0x61, 0x6e,
0x41, 0x7a, 0x6f, 0x72, 0x69, 0x52, 0x65, 0x6a, 0x6b, 0x6a, 0x61, 0x76,
0x69, 0x6b, 0x53, 0x76, 0x65, 0x74, 0x61, 0x20, 0x48, 0x65, 0x6c, 0x65,
0x6e, 0x61, 0x41, 0x6e, 0x64, 0x6f, 0x72, 0x61, 0x41, 0x73, 0x74, 0x72,
0x61, 0x68, 0x61, 0x6e, 0x41, 0x74, 0x69, 0x6e, 0x61, 0x42, 0x72, 0x69,
0x73, 0x65, 0x6c, 0x42, 0x75, 0x6b, 0x75, 0x72, 0x65, 0x161, 0x74, 0x42,
0x75, 0x64, 0x69, 0x6d, 0x70, 0x65, 0x161, 0x74, 0x61, 0x4b, 0x69, 0x161,
0x69, 0x6e, 0x6a, 0x65, 0x76, 0x44, 0x61, 0x62, 0x6c, 0x69, 0x6e, 0x47,
0x65, 0x72, 0x6e, 0x7a, 0x69, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20,
0x4d, 0x61, 0x6e, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x6a, 0x69, 0x6e, 0x67,
0x72, 0x61, 0x64, 0x4b, 0x69, 0x6a, 0x65, 0x76, 0x4c, 0x69, 0x73, 0x61,
0x62, 0x6f, 0x6e, 0x50, 0x61, 0x72, 0x69, 0x7a, 0x52, 0x69, 0x6d, 0x53,
0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x6a, 0x53, 0x6b,
0x6f, 0x70, 0x6c, 0x6a, 0x65, 0x53, 0x6f, 0x66, 0x69, 0x6a, 0x61, 0x160,
0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x42, 0x65, 0x10d, 0x56, 0x61,
0x72, 0x161, 0x61, 0x76, 0x61, 0x43, 0x69, 0x72, 0x69, 0x68, 0x42, 0x6f,
0x17e, 0x69, 0x107, 0x6e, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x69, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x46, 0x69, 0x64, 0x17e,
0x69, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x410, 0x434, 0x438, 0x441, 0x20, 0x410,
0x431, 0x435, 0x431, 0x430, 0x411, 0x430, 0x43d, 0x433, 0x443, 0x438, 0x411, 0x438,
0x441, 0x430, 0x43e, 0x411, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x440, 0x411, 0x440,
0x430, 0x437, 0x430, 0x432, 0x438, 0x43b, 0x411, 0x443, 0x45f, 0x443, 0x43c, 0x431,
0x443, 0x440, 0x430, 0x41a, 0x430, 0x438, 0x440, 0x43e, 0x41a, 0x430, 0x437, 0x430,
0x431, 0x43b, 0x430, 0x43d, 0x43a, 0x430, 0x415, 0x43b, 0x20, 0x410, 0x458, 0x443,
0x43d, 0x425, 0x430, 0x440, 0x430, 0x440, 0x435, 0x408, 0x43e, 0x445, 0x430, 0x43d,
0x435, 0x441, 0x431, 0x443, 0x440, 0x433, 0x41a, 0x430, 0x440, 0x442, 0x443, 0x43c,
0x41b, 0x443, 0x43c, 0x443, 0x43c, 0x431, 0x430, 0x448, 0x438, 0x41c, 0x430, 0x43f,
0x443, 0x442, 0x43e, 0x41c, 0x43e, 0x43d, 0x440, 0x43e, 0x432, 0x438, 0x458, 0x430,
0x41d, 0x430, 0x458, 0x440, 0x43e, 0x431, 0x438, 0x41d, 0x438, 0x458, 0x430, 0x43c,
0x435, 0x458, 0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x43e,
0x421, 0x430, 0x43e, 0x20, 0x422, 0x43e, 0x43c, 0x435, 0x415, 0x43d, 0x43a, 0x43e,
0x440, 0x438, 0x45f, 0x410, 0x43d, 0x433, 0x432, 0x438, 0x43b, 0x430, 0x410, 0x43d,
0x442, 0x438, 0x433, 0x432, 0x430, 0x410, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458,
0x430, 0x43d, 0x430, 0x411, 0x443, 0x435, 0x43d, 0x43e, 0x441, 0x20, 0x410, 0x438,
0x440, 0x435, 0x441, 0x41a, 0x43e, 0x440, 0x434, 0x43e, 0x431, 0x430, 0x425, 0x443,
0x445, 0x443, 0x458, 0x41b, 0x430, 0x20, 0x420, 0x438, 0x43e, 0x445, 0x430, 0x41c,
0x435, 0x43d, 0x434, 0x43e, 0x437, 0x430, 0x420, 0x438, 0x43e, 0x20, 0x413, 0x430,
0x459, 0x435, 0x433, 0x43e, 0x441, 0x421, 0x430, 0x43d, 0x20, 0x425, 0x443, 0x430,
0x43d, 0x421, 0x430, 0x43d, 0x20, 0x41b, 0x443, 0x438, 0x423, 0x448, 0x443, 0x430,
0x438, 0x458, 0x430, 0x410, 0x441, 0x443, 0x43d, 0x441, 0x438, 0x43e, 0x43d, 0x410,
0x442, 0x438, 0x43a, 0x43e, 0x43a, 0x430, 0x43d, 0x411, 0x430, 0x438, 0x458, 0x430,
0x411, 0x430, 0x438, 0x458, 0x430, 0x20, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440,
0x430, 0x441, 0x411, 0x435, 0x43b, 0x435, 0x43c, 0x411, 0x435, 0x43b, 0x438, 0x437,
0x435, 0x411, 0x43b, 0x430, 0x43d, 0x43a, 0x2d, 0x421, 0x435, 0x458, 0x431, 0x43b,
0x43e, 0x43d, 0x411, 0x43e, 0x430, 0x20, 0x412, 0x438, 0x441, 0x442, 0x430, 0x411,
0x43e, 0x458, 0x437, 0x438, 0x41a, 0x435, 0x43c, 0x431, 0x440, 0x438, 0x45f, 0x20,
0x411, 0x435, 0x458, 0x41a, 0x430, 0x43c, 0x43f, 0x43e, 0x20, 0x413, 0x440, 0x430,
0x43d, 0x434, 0x435, 0x41a, 0x430, 0x458, 0x435, 0x43d, 0x41a, 0x430, 0x458, 0x43c,
0x430, 0x43d, 0x441, 0x43a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x427, 0x438, 0x432, 0x430, 0x432, 0x430, 0x41a, 0x43e, 0x441, 0x442, 0x430, 0x440,
0x438, 0x43a, 0x430, 0x41a, 0x443, 0x440, 0x430, 0x441, 0x430, 0x43e, 0x414, 0x430,
0x43d, 0x43c, 0x430, 0x440, 0x43a, 0x441, 0x445, 0x430, 0x433, 0x435, 0x43d, 0x414,
0x43e, 0x441, 0x43e, 0x43d, 0x414, 0x43e, 0x441, 0x43e, 0x43d, 0x20, 0x41a, 0x440,
0x438, 0x43a, 0x413, 0x43b, 0x435, 0x458, 0x441, 0x20, 0x411, 0x435, 0x458, 0x413,
0x443, 0x441, 0x20, 0x411, 0x435, 0x458, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x20,
0x422, 0x443, 0x440, 0x43a, 0x413, 0x432, 0x430, 0x434, 0x430, 0x43b, 0x443, 0x43f,
0x435, 0x413, 0x432, 0x430, 0x458, 0x430, 0x43a, 0x438, 0x43b, 0x413, 0x443, 0x430,
0x43d, 0x430, 0x425, 0x430, 0x43b, 0x438, 0x444, 0x430, 0x43a, 0x441, 0x425, 0x430,
0x432, 0x430, 0x43d, 0x430, 0x425, 0x435, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x459,
0x43e, 0x41a, 0x43d, 0x43e, 0x43a, 0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x458, 0x430, 0x43d, 0x430, 0x41c, 0x430, 0x440, 0x435, 0x43d, 0x433, 0x43e, 0x2c,
0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x41f, 0x435, 0x442,
0x435, 0x440, 0x441, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x418, 0x43d, 0x434,
0x438, 0x458, 0x430, 0x43d, 0x430, 0x422, 0x435, 0x43b, 0x20, 0x421, 0x438, 0x442,
0x438, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412,
0x435, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430,
0x43d, 0x430, 0x412, 0x438, 0x43d, 0x446, 0x435, 0x43d, 0x435, 0x441, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x430,
0x43c, 0x430, 0x43a, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d,
0x430, 0x418, 0x43a, 0x432, 0x430, 0x43b, 0x443, 0x438, 0x442, 0x408, 0x430, 0x43c,
0x430, 0x458, 0x43a, 0x430, 0x416, 0x443, 0x43d, 0x43e, 0x41c, 0x43e, 0x43d, 0x442,
0x438, 0x447, 0x435, 0x43b, 0x43e, 0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442, 0x430,
0x43a, 0x438, 0x41a, 0x440, 0x435, 0x43b, 0x435, 0x43d, 0x434, 0x438, 0x458, 0x43a,
0x41b, 0x430, 0x20, 0x41f, 0x430, 0x437, 0x41b, 0x43e, 0x441, 0x20, 0x410, 0x43d,
0x452, 0x435, 0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x443, 0x435, 0x440, 0x20, 0x41f,
0x440, 0x438, 0x43d, 0x441, 0x20, 0x41a, 0x432, 0x43e, 0x440, 0x442, 0x435, 0x440,
0x41c, 0x430, 0x441, 0x435, 0x458, 0x43e, 0x41c, 0x430, 0x43d, 0x430, 0x433, 0x432,
0x430, 0x41c, 0x430, 0x437, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x43a, 0x43e, 0x20, 0x421, 0x438, 0x442, 0x438, 0x41c, 0x43e, 0x43d,
0x442, 0x435, 0x440, 0x435, 0x458, 0x41c, 0x43e, 0x43d, 0x442, 0x441, 0x435, 0x440,
0x430, 0x442, 0x41d, 0x430, 0x441, 0x430, 0x443, 0x40a, 0x443, 0x458, 0x43e, 0x440,
0x43a, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x411, 0x438, 0x458, 0x443, 0x43b,
0x430, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x430, 0x440, 0x2c,
0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a,
0x43e, 0x442, 0x430, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x421, 0x430, 0x43b, 0x435,
0x43c, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x41f,
0x43e, 0x440, 0x442, 0x20, 0x43e, 0x444, 0x20, 0x421, 0x43f, 0x435, 0x458, 0x43d,
0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x20, 0x412, 0x435, 0x459, 0x43e, 0x41f, 0x43e,
0x440, 0x442, 0x43e, 0x440, 0x438, 0x43a, 0x43e, 0x41f, 0x443, 0x43d, 0x442, 0x430,
0x20, 0x410, 0x440, 0x435, 0x43d, 0x430, 0x441, 0x420, 0x430, 0x43d, 0x43a, 0x438,
0x43d, 0x20, 0x418, 0x43d, 0x43b, 0x435, 0x442, 0x420, 0x435, 0x441, 0x438, 0x444,
0x435, 0x420, 0x435, 0x433, 0x438, 0x43d, 0x430, 0x420, 0x435, 0x441, 0x43e, 0x43b,
0x443, 0x442, 0x420, 0x438, 0x43e, 0x20, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x43e,
0x421, 0x430, 0x43d, 0x442, 0x430, 0x440, 0x435, 0x43c, 0x421, 0x430, 0x43d, 0x442,
0x438, 0x458, 0x430, 0x433, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x43e, 0x20, 0x414,
0x43e, 0x43c, 0x438, 0x43d, 0x433, 0x43e, 0x421, 0x430, 0x43e, 0x20, 0x41f, 0x430,
0x43e, 0x43b, 0x43e, 0x418, 0x442, 0x43e, 0x43a, 0x43e, 0x440, 0x442, 0x43e, 0x440,
0x43c, 0x438, 0x442, 0x421, 0x432, 0x2e, 0x20, 0x411, 0x430, 0x440, 0x442, 0x43e,
0x43b, 0x43e, 0x43c, 0x65, 0x458, 0x421, 0x432, 0x2e, 0x20, 0x40f, 0x43e, 0x43d,
0x421, 0x435, 0x43d, 0x442, 0x20, 0x41a, 0x438, 0x442, 0x441, 0x421, 0x432, 0x2e,
0x20, 0x41b, 0x443, 0x446, 0x438, 0x458, 0x430, 0x421, 0x432, 0x2e, 0x20, 0x422,
0x43e, 0x43c, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x412, 0x438, 0x43d, 0x441,
0x435, 0x43d, 0x442, 0x421, 0x432, 0x438, 0x444, 0x442, 0x20, 0x41a, 0x443, 0x440,
0x435, 0x43d, 0x442, 0x412, 0x430, 0x458, 0x442, 0x445, 0x43e, 0x440, 0x441, 0x412,
0x438, 0x43d, 0x438, 0x43f, 0x435, 0x433, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x430,
0x442, 0x41a, 0x430, 0x441, 0x435, 0x458, 0x414, 0x435, 0x458, 0x432, 0x438, 0x441,
0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x423, 0x440, 0x432, 0x438,
0x43b, 0x41c, 0x435, 0x43a, 0x432, 0x43e, 0x440, 0x438, 0x41c, 0x43e, 0x441, 0x43e,
0x43d, 0x41c, 0x430, 0x43a, 0x43c, 0x443, 0x440, 0x434, 0x43e, 0x428, 0x43e, 0x432,
0x430, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x41b, 0x43e, 0x43d, 0x433, 0x458,
0x435, 0x440, 0x431, 0x458, 0x435, 0x43d, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x438,
0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x410, 0x43a, 0x443, 0x442, 0x43e, 0x431,
0x435, 0x410, 0x442, 0x438, 0x440, 0x430, 0x458, 0x411, 0x430, 0x43d, 0x43a, 0x43e,
0x43a, 0x411, 0x435, 0x458, 0x440, 0x443, 0x442, 0x411, 0x440, 0x443, 0x43d, 0x435,
0x458, 0x438, 0x425, 0x435, 0x431, 0x440, 0x43e, 0x43d, 0x425, 0x43e, 0x20, 0x428,
0x438, 0x20, 0x41c, 0x438, 0x43d, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e,
0x43d, 0x433, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x40f, 0x430, 0x458, 0x430,
0x43f, 0x443, 0x440, 0x430, 0x408, 0x435, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x438,
0x43c, 0x425, 0x430, 0x43d, 0x434, 0x438, 0x433, 0x430, 0x41a, 0x430, 0x43b, 0x43a,
0x443, 0x442, 0x430, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440,
0x441, 0x43a, 0x41a, 0x443, 0x430, 0x43b, 0x430, 0x20, 0x41b, 0x443, 0x43c, 0x43f,
0x443, 0x440, 0x41a, 0x443, 0x432, 0x430, 0x458, 0x442, 0x41c, 0x430, 0x43a, 0x430,
0x443, 0x41c, 0x443, 0x441, 0x43a, 0x430, 0x442, 0x41d, 0x438, 0x43a, 0x43e, 0x437,
0x438, 0x458, 0x430, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x443, 0x437, 0x45a, 0x435,
0x446, 0x43a, 0x41e, 0x440, 0x430, 0x43b, 0x41f, 0x43d, 0x43e, 0x43c, 0x20, 0x41f,
0x435, 0x43d, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430, 0x43d, 0x433, 0x41a,
0x43e, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x458, 0x41a, 0x438, 0x437, 0x438, 0x43b,
0x43e, 0x440, 0x434, 0x430, 0x420, 0x438, 0x458, 0x430, 0x434, 0x428, 0x430, 0x43d,
0x433, 0x430, 0x458, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x43a, 0x43e, 0x43b,
0x438, 0x43c, 0x441, 0x43a, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x422, 0x431,
0x438, 0x43b, 0x438, 0x441, 0x438, 0x422, 0x435, 0x445, 0x435, 0x440, 0x430, 0x43d,
0x422, 0x438, 0x43c, 0x43f, 0x443, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430,
0x442, 0x43e, 0x440, 0x423, 0x440, 0x443, 0x43c, 0x43a, 0x432, 0x438, 0x412, 0x438,
0x458, 0x435, 0x43d, 0x442, 0x438, 0x458, 0x430, 0x43d, 0x408, 0x430, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d,
0x431, 0x443, 0x440, 0x433, 0x408, 0x435, 0x440, 0x435, 0x432, 0x430, 0x43d, 0x410,
0x437, 0x43e, 0x440, 0x438, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x430, 0x41a,
0x430, 0x43d, 0x430, 0x440, 0x441, 0x43a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a,
0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x424, 0x430, 0x440, 0x441,
0x43a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x420, 0x435, 0x458,
0x43a, 0x458, 0x430, 0x432, 0x438, 0x43a, 0x408, 0x443, 0x436, 0x43d, 0x430, 0x20,
0x40f, 0x43e, 0x440, 0x45f, 0x438, 0x458, 0x430, 0x421, 0x432, 0x2e, 0x20, 0x408,
0x435, 0x43b, 0x435, 0x43d, 0x430, 0x411, 0x440, 0x438, 0x437, 0x431, 0x435, 0x458,
0x43d, 0x411, 0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x20, 0x425, 0x438, 0x43b, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x421, 0x438, 0x434, 0x43d, 0x435,
0x458, 0x410, 0x442, 0x438, 0x43d, 0x430, 0x411, 0x435, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x411, 0x440, 0x438, 0x441, 0x435, 0x43b, 0x411, 0x443, 0x43a, 0x443, 0x440,
0x435, 0x448, 0x442, 0x411, 0x443, 0x434, 0x438, 0x43c, 0x43f, 0x435, 0x448, 0x442,
0x430, 0x411, 0x443, 0x441, 0x438, 0x43d, 0x433, 0x435, 0x43d, 0x41a, 0x438, 0x448,
0x438, 0x45a, 0x435, 0x432, 0x41a, 0x43e, 0x43f, 0x435, 0x43d, 0x445, 0x430, 0x433,
0x435, 0x43d, 0x414, 0x430, 0x431, 0x43b, 0x438, 0x43d, 0x413, 0x435, 0x440, 0x43d,
0x437, 0x438, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x41c, 0x435, 0x43d,
0x418, 0x441, 0x442, 0x430, 0x43d, 0x431, 0x443, 0x43b, 0x41a, 0x430, 0x43b, 0x438,
0x45a, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x41a, 0x438, 0x458, 0x435, 0x432,
0x41b, 0x438, 0x441, 0x430, 0x431, 0x43e, 0x43d, 0x409, 0x443, 0x431, 0x459, 0x430,
0x43d, 0x430, 0x41b, 0x443, 0x43a, 0x441, 0x435, 0x43c, 0x431, 0x443, 0x440, 0x433,
0x41c, 0x430, 0x440, 0x438, 0x445, 0x430, 0x43c, 0x43d, 0x41f, 0x430, 0x440, 0x438,
0x437, 0x421, 0x430, 0x43d, 0x20, 0x41c, 0x430, 0x440, 0x438, 0x43d, 0x43e, 0x421,
0x430, 0x440, 0x430, 0x458, 0x435, 0x432, 0x43e, 0x421, 0x438, 0x43c, 0x444, 0x435,
0x440, 0x43e, 0x43f, 0x43e, 0x459, 0x421, 0x43a, 0x43e, 0x43f, 0x459, 0x435, 0x421,
0x43e, 0x444, 0x438, 0x458, 0x430, 0x421, 0x442, 0x43e, 0x43a, 0x445, 0x43e, 0x43b,
0x43c, 0x422, 0x430, 0x43b, 0x438, 0x43d, 0x423, 0x459, 0x430, 0x43d, 0x43e, 0x432,
0x441, 0x43a, 0x412, 0x430, 0x434, 0x443, 0x437, 0x411, 0x435, 0x447, 0x412, 0x438,
0x459, 0x43d, 0x443, 0x441, 0x426, 0x438, 0x440, 0x438, 0x445, 0x410, 0x43d, 0x442,
0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x438, 0x432, 0x43e, 0x411, 0x43e, 0x436,
0x438, 0x45b, 0x41a, 0x43e, 0x43c, 0x43e, 0x440, 0x43e, 0x41c, 0x430, 0x445, 0x435,
0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x438, 0x41c, 0x430, 0x443, 0x440, 0x438,
0x446, 0x438, 0x458, 0x443, 0x441, 0x41c, 0x430, 0x458, 0x43e, 0x442, 0x420, 0x435,
0x443, 0x43d, 0x438, 0x43e, 0x43d, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x41e, 0x43a,
0x43b, 0x430, 0x43d, 0x434, 0x41a, 0x430, 0x442, 0x445, 0x430, 0x43c, 0x423, 0x441,
0x43a, 0x440, 0x448, 0x45a, 0x435, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e,
0x413, 0x430, 0x43c, 0x431, 0x438, 0x458, 0x435, 0x413, 0x432, 0x430, 0x434, 0x430,
0x43b, 0x43a, 0x430, 0x43d, 0x430, 0x43b, 0x425, 0x43e, 0x43d, 0x43e, 0x43b, 0x443,
0x43b, 0x443, 0x415, 0x43d, 0x434, 0x435, 0x440, 0x431, 0x435, 0x440, 0x438, 0x41a,
0x43e, 0x448, 0x440, 0x435, 0x41a, 0x432, 0x430, 0x45f, 0x430, 0x43b, 0x435, 0x458,
0x438, 0x43d, 0x41c, 0x430, 0x458, 0x443, 0x440, 0x43e, 0x41c, 0x438, 0x434, 0x432,
0x435, 0x458, 0x41f, 0x430, 0x433, 0x43e, 0x20, 0x41f, 0x430, 0x433, 0x43e, 0x41f,
0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x41c, 0x43e,
0x440, 0x437, 0x431, 0x438, 0x421, 0x430, 0x458, 0x43f, 0x430, 0x43d, 0x422, 0x430,
0x445, 0x438, 0x442, 0x438, 0x412, 0x435, 0x458, 0x43a, 0x412, 0x430, 0x43b, 0x438,
0x441, 0x41, 0x64, 0x69, 0x73, 0x20, 0x41, 0x62, 0x65, 0x62, 0x61, 0x4b,
0x61, 0x65, 0x72, 0x6f, 0x44, 0x61, 0x72, 0x20, 0x65, 0x6c, 0x20, 0x42,
0x65, 0x69, 0x64, 0x61, 0x20, 0x28, 0x43, 0x61, 0x73, 0x61, 0x62, 0x6c,
0x61, 0x6e, 0x63, 0x61, 0x29, 0x4c, 0x61, 0x2bc, 0x59, 0x6f, 0x75, 0x6e,
0x4e, 0x2bc, 0x44, 0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x54, 0x61, 0x72,
0x61, 0x62, 0x75, 0x6c, 0x75, 0x73, 0x20, 0x28, 0x54, 0x72, 0x69, 0x70,
0x6f, 0x6c, 0x69, 0x29, 0x42, 0x65, 0x6c, 0xe9, 0x6d, 0x47, 0x77, 0x61,
0x64, 0x65, 0x6c, 0x6f, 0x75, 0x70, 0x4b, 0xea, 0x72, 0x20, 0x56, 0x65,
0x63, 0x2bc, 0x68, 0x69, 0x6b, 0x6f, 0x4e, 0x75, 0x75, 0x6b, 0x20, 0x28,
0x47, 0x6f, 0x64, 0x74, 0x68, 0xe5, 0x62, 0x29, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x4a, 0x6f, 0x68, 0x6e, 0x2bc, 0x73,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53,
0x61, 0x6e, 0x74, 0x20, 0x56, 0x69, 0x73, 0x61, 0x6e, 0x74, 0x51, 0xe2,
0x6e, 0xe2, 0x71, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2bc, 0x42, 0x61,
0x6b, 0x6f, 0x75, 0x42, 0x61, 0x79, 0x72, 0x75, 0x74, 0x54, 0x63, 0x68,
0x69, 0x74, 0x61, 0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x61, 0x4b, 0xea,
0x72, 0x20, 0x48, 0xf4, 0x2d, 0x43, 0x68, 0x69, 0x2d, 0x4d, 0x69, 0x6e,
0x68, 0x4b, 0x61, 0x62, 0x6f, 0x75, 0x6c, 0x4b, 0x61, 0x6d, 0x74, 0x63,
0x68, 0x61, 0x74, 0x6b, 0x61, 0x43, 0x61, 0x6c, 0x63, 0x75, 0x74, 0x74,
0x61, 0x4b, 0x6f, 0x77, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x73, 0x71, 0x61,
0x74, 0x4c, 0x65, 0x76, 0x6b, 0x6f, 0x73, 0xed, 0x61, 0x50, 0x68, 0x6e,
0x75, 0x6d, 0x20, 0x50, 0xe9, 0x6e, 0x68, 0x50, 0x2bc, 0x79, 0x6f, 0x6e,
0x67, 0x79, 0x61, 0x6e, 0x67, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x75, 0x72, 0x54, 0x6f, 0x73, 0x68, 0x6b, 0x65, 0x6e, 0x74, 0x56, 0x69,
0x61, 0x6e, 0x67, 0x63, 0x68, 0x61, 0x6e, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x7a, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x65, 0x7a, 0x4b,
0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x7a, 0x4b, 0x61, 0x62, 0x20, 0x47,
0x6c, 0x61, 0x73, 0x46, 0x61, 0x65, 0x72, 0x6f, 0x52, 0x65, 0x79, 0x6b,
0x6a, 0x61, 0x76, 0xed, 0x6b, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x61, 0x72, 0x20, 0x53, 0x75, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x44, 0x75, 0x6c, 0x65, 0x6e, 0x6e,
0x4a, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x47, 0x77, 0x65,
0x72, 0x6e, 0x65, 0x6e, 0x65, 0x7a, 0x4d, 0x61, 0x6e, 0x61, 0x76, 0x4a,
0x65, 0x72, 0x7a, 0x65, 0x6e, 0x65, 0x7a, 0x4c, 0x6f, 0x6e, 0x64, 0x72,
0x65, 0x7a, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x6f, 0x75, 0x72,
0x67, 0x4d, 0x61, 0x72, 0x6a, 0x65, 0x68, 0x61, 0x6d, 0x6e, 0x4d, 0x65,
0x6e, 0x73, 0x6b, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x50, 0x72, 0x61,
0x68, 0x61, 0x4b, 0x6f, 0x6b, 0x6f, 0x7a, 0x4b, 0x6f, 0x6d, 0x6f, 0x72,
0x65, 0x7a, 0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c, 0x65, 0x6e, 0x6e, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x7a, 0x4d, 0x6f, 0x72, 0x69, 0x73,
0x52, 0x65, 0xfc, 0x6e, 0x69, 0x6f, 0x6e, 0x45, 0x6e, 0x65, 0x7a, 0x20,
0x50, 0x61, 0x73, 0x6b, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x410, 0x431,
0x438, 0x434, 0x436, 0x430, 0x43d, 0x411, 0x43b, 0x430, 0x43d, 0x442, 0x430, 0x439,
0x44a, 0x440, 0x41a, 0x430, 0x439, 0x440, 0x43e, 0x414, 0x430, 0x440, 0x20, 0x435,
0x441, 0x20, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x414, 0x436, 0x438, 0x431, 0x443,
0x442, 0x438, 0x415, 0x43b, 0x20, 0x410, 0x44e, 0x43d, 0x424, 0x440, 0x438, 0x439,
0x442, 0x430, 0x443, 0x43d, 0x419, 0x43e, 0x445, 0x430, 0x43d, 0x435, 0x441, 0x431,
0x443, 0x440, 0x433, 0x41c, 0x43e, 0x433, 0x430, 0x434, 0x438, 0x448, 0x443, 0x41c,
0x43e, 0x43d, 0x440, 0x43e, 0x432, 0x438, 0x44f, 0x41d, 0x430, 0x439, 0x440, 0x43e,
0x431, 0x438, 0x41d, 0x438, 0x430, 0x43c, 0x435, 0x439, 0x410, 0x43d, 0x43a, 0x44a,
0x440, 0x438, 0x434, 0x436, 0x410, 0x43d, 0x433, 0x443, 0x438, 0x43b, 0x430, 0x410,
0x440, 0x430, 0x433, 0x443, 0x430, 0x439, 0x43d, 0x430, 0x411, 0x443, 0x435, 0x43d,
0x43e, 0x441, 0x20, 0x410, 0x439, 0x440, 0x435, 0x441, 0x425, 0x443, 0x445, 0x443,
0x439, 0x420, 0x438, 0x43e, 0x20, 0x413, 0x430, 0x43b, 0x435, 0x433, 0x43e, 0x441,
0x421, 0x430, 0x43d, 0x20, 0x41b, 0x443, 0x438, 0x441, 0x411, 0x430, 0x438, 0x44f,
0x411, 0x430, 0x438, 0x44f, 0x20, 0x434, 0x435, 0x20, 0x411, 0x430, 0x43d, 0x434,
0x435, 0x440, 0x430, 0x441, 0x411, 0x43b, 0x430, 0x43d, 0x2d, 0x421, 0x430, 0x431,
0x43b, 0x43e, 0x43d, 0x411, 0x43e, 0x439, 0x441, 0x438, 0x41a, 0x435, 0x439, 0x43c,
0x431, 0x440, 0x438, 0x434, 0x436, 0x20, 0x411, 0x435, 0x439, 0x41a, 0x430, 0x439,
0x435, 0x43d, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x43e, 0x432, 0x438, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x421, 0x438, 0x443, 0x434, 0x430,
0x434, 0x20, 0x425, 0x443, 0x430, 0x440, 0x435, 0x441, 0x41a, 0x43e, 0x441, 0x442,
0x430, 0x20, 0x420, 0x438, 0x43a, 0x430, 0x41a, 0x440, 0x435, 0x441, 0x442, 0x44a,
0x43d, 0x427, 0x443, 0x44f, 0x431, 0x430, 0x41a, 0x44e, 0x440, 0x430, 0x441, 0x430,
0x43e, 0x414, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x43a, 0x441, 0x445, 0x430, 0x432,
0x43d, 0x414, 0x43e, 0x443, 0x441, 0x44a, 0x43d, 0x414, 0x43e, 0x443, 0x441, 0x44a,
0x43d, 0x20, 0x41a, 0x440, 0x438, 0x439, 0x43a, 0x414, 0x435, 0x43d, 0x432, 0x44a,
0x440, 0x414, 0x435, 0x442, 0x440, 0x43e, 0x439, 0x442, 0x415, 0x434, 0x43c, 0x44a,
0x43d, 0x442, 0x44a, 0x43d, 0x415, 0x439, 0x440, 0x443, 0x43d, 0x435, 0x43f, 0x435,
0x424, 0x43e, 0x440, 0x442, 0x20, 0x41d, 0x435, 0x43b, 0x441, 0x44a, 0x43d, 0x413,
0x43b, 0x435, 0x439, 0x441, 0x20, 0x411, 0x435, 0x439, 0x413, 0x443, 0x441, 0x20,
0x411, 0x435, 0x439, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x20, 0x422, 0x44a, 0x440,
0x43a, 0x413, 0x443, 0x430, 0x44f, 0x43a, 0x438, 0x43b, 0x415, 0x440, 0x43c, 0x43e,
0x441, 0x438, 0x43b, 0x44c, 0x43e, 0x41f, 0x438, 0x442, 0x44a, 0x440, 0x441, 0x431,
0x44a, 0x440, 0x433, 0x412, 0x438, 0x432, 0x438, 0x412, 0x438, 0x43d, 0x441, 0x435,
0x43d, 0x441, 0x414, 0x436, 0x443, 0x43d, 0x43e, 0x41a, 0x440, 0x430, 0x43b, 0x435,
0x43d, 0x434, 0x435, 0x439, 0x43a, 0x41b, 0x430, 0x20, 0x41f, 0x430, 0x441, 0x41b,
0x43e, 0x441, 0x20, 0x410, 0x43d, 0x434, 0x436, 0x435, 0x43b, 0x438, 0x441, 0x41b,
0x43e, 0x443, 0x44a, 0x440, 0x20, 0x43f, 0x440, 0x438, 0x43d, 0x441, 0x435, 0x441,
0x20, 0x43a, 0x443, 0x43e, 0x442, 0x44a, 0x440, 0x41c, 0x430, 0x441, 0x435, 0x439,
0x43e, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x440, 0x435, 0x439, 0x41d, 0x44e, 0x20,
0x419, 0x43e, 0x440, 0x43a, 0x41d, 0x43e, 0x443, 0x43c, 0x41d, 0x43e, 0x440, 0x43e,
0x43d, 0x44f, 0x411, 0x44e, 0x43b, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x44a, 0x440,
0x41d, 0x44e, 0x20, 0x421, 0x435, 0x439, 0x43b, 0x44a, 0x43c, 0x41f, 0x43e, 0x440,
0x442, 0x20, 0x44a, 0x444, 0x20, 0x421, 0x43f, 0x435, 0x439, 0x43d, 0x41f, 0x43e,
0x440, 0x442, 0x43e, 0x20, 0x412, 0x435, 0x43b, 0x44c, 0x43e, 0x41f, 0x443, 0x435,
0x440, 0x442, 0x43e, 0x20, 0x420, 0x438, 0x43a, 0x43e, 0x420, 0x438, 0x434, 0x436,
0x430, 0x439, 0x43d, 0x430, 0x420, 0x435, 0x437, 0x43e, 0x43b, 0x44e, 0x442, 0x421,
0x430, 0x43d, 0x442, 0x44f, 0x433, 0x43e, 0x421, 0x430, 0x43e, 0x20, 0x41f, 0x430,
0x443, 0x43b, 0x43e, 0x421, 0x433, 0x43e, 0x440, 0x441, 0x431, 0x438, 0x441, 0x43e,
0x43d, 0x421, 0x435, 0x43d, 0x20, 0x411, 0x430, 0x440, 0x442, 0x435, 0x43b, 0x435,
0x43c, 0x438, 0x421, 0x435, 0x439, 0x43d, 0x442, 0x20, 0x414, 0x436, 0x43e, 0x43d,
0x441, 0x421, 0x435, 0x439, 0x43d, 0x442, 0x20, 0x41a, 0x438, 0x442, 0x441, 0x421,
0x435, 0x439, 0x43d, 0x442, 0x20, 0x41b, 0x443, 0x441, 0x438, 0x44f, 0x421, 0x435,
0x439, 0x43d, 0x442, 0x20, 0x422, 0x43e, 0x43c, 0x430, 0x441, 0x421, 0x435, 0x439,
0x43d, 0x442, 0x20, 0x412, 0x438, 0x43d, 0x441, 0x44a, 0x43d, 0x442, 0x421, 0x443,
0x438, 0x444, 0x442, 0x20, 0x41a, 0x44a, 0x440, 0x435, 0x43d, 0x442, 0x412, 0x430,
0x43d, 0x43a, 0x443, 0x432, 0x44a, 0x440, 0x423, 0x438, 0x43d, 0x438, 0x43f, 0x435,
0x433, 0x41a, 0x435, 0x439, 0x441, 0x438, 0x414, 0x435, 0x439, 0x432, 0x438, 0x441,
0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x414, 0x44e, 0x440, 0x432, 0x438, 0x43b,
0x41c, 0x43e, 0x443, 0x441, 0x44a, 0x43d, 0x41c, 0x430, 0x43a, 0x43c, 0x44a, 0x440,
0x434, 0x43e, 0x41f, 0x430, 0x43b, 0x43c, 0x44a, 0x440, 0x41b, 0x43e, 0x43d, 0x433,
0x438, 0x440, 0x431, 0x44e, 0x435, 0x43d, 0x410, 0x442, 0x44a, 0x440, 0x430, 0x443,
0x411, 0x430, 0x445, 0x440, 0x435, 0x439, 0x43d, 0x425, 0x43e, 0x43d, 0x43a, 0x43e,
0x43d, 0x433, 0x419, 0x435, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x438, 0x43c, 0x41a,
0x43e, 0x43b, 0x43a, 0x430, 0x442, 0x430, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x44f, 0x440, 0x441, 0x43a, 0x41d, 0x438, 0x43a, 0x43e, 0x437, 0x438, 0x44f, 0x410,
0x440, 0x430, 0x43b, 0x41f, 0x445, 0x435, 0x43d, 0x44f, 0x43d, 0x41a, 0x43e, 0x441,
0x442, 0x430, 0x43d, 0x430, 0x439, 0x41a, 0x44a, 0x437, 0x44a, 0x43b, 0x43e, 0x440,
0x434, 0x430, 0x420, 0x438, 0x44f, 0x434, 0x421, 0x440, 0x435, 0x434, 0x43d, 0x435,
0x43a, 0x43e, 0x43b, 0x438, 0x43c, 0x441, 0x43a, 0x422, 0x430, 0x439, 0x43f, 0x435,
0x412, 0x438, 0x435, 0x43d, 0x442, 0x44f, 0x43d, 0x42f, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x441, 0x43a,
0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x430, 0x43d,
0x430, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x438, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x424,
0x430, 0x440, 0x44c, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x41c, 0x430, 0x434, 0x435, 0x439, 0x440, 0x430, 0x420,
0x435, 0x439, 0x43a, 0x44f, 0x432, 0x438, 0x43a, 0x42e, 0x436, 0x43d, 0x430, 0x20,
0x414, 0x436, 0x43e, 0x440, 0x434, 0x436, 0x438, 0x44f, 0x421, 0x432, 0x435, 0x442,
0x430, 0x20, 0x415, 0x43b, 0x435, 0x43d, 0x430, 0x411, 0x440, 0x438, 0x437, 0x431,
0x435, 0x439, 0x43d, 0x411, 0x440, 0x43e, 0x443, 0x43a, 0x44a, 0x43d, 0x20, 0x425,
0x438, 0x43b, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x41c, 0x435,
0x43b, 0x431, 0x44a, 0x440, 0x43d, 0x41f, 0x44a, 0x440, 0x442, 0x421, 0x438, 0x434,
0x43d, 0x438, 0x411, 0x440, 0x44e, 0x43a, 0x441, 0x435, 0x43b, 0x411, 0x443, 0x43a,
0x443, 0x440, 0x435, 0x449, 0x411, 0x443, 0x434, 0x430, 0x43f, 0x435, 0x449, 0x430,
0x411, 0x44e, 0x437, 0x438, 0x43d, 0x433, 0x435, 0x43d, 0x414, 0x44a, 0x431, 0x43b,
0x438, 0x43d, 0x413, 0x44a, 0x440, 0x43d, 0x437, 0x438, 0x425, 0x435, 0x43b, 0x437,
0x438, 0x43d, 0x43a, 0x438, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20, 0x41c,
0x430, 0x43d, 0x414, 0x436, 0x44a, 0x440, 0x441, 0x438, 0x421, 0x43a, 0x43e, 0x43f,
0x438, 0x435, 0x421, 0x43e, 0x444, 0x438, 0x44f, 0x423, 0x43b, 0x44f, 0x43d, 0x43e,
0x432, 0x441, 0x43a, 0x412, 0x438, 0x435, 0x43d, 0x430, 0x412, 0x438, 0x43b, 0x43d,
0x44e, 0x441, 0x426, 0x44e, 0x440, 0x438, 0x445, 0x420, 0x43e, 0x436, 0x434, 0x435,
0x441, 0x442, 0x432, 0x43e, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x43e, 0x43c, 0x43e,
0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x439, 0x41c, 0x430, 0x439, 0x43e,
0x442, 0x420, 0x435, 0x44e, 0x43d, 0x438, 0x43e, 0x43d, 0x410, 0x43f, 0x438, 0x44f,
0x427, 0x430, 0x442, 0x430, 0x43c, 0x427, 0x443, 0x44e, 0x43a, 0x412, 0x435, 0x43b,
0x438, 0x43a, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x424, 0x438, 0x434, 0x436, 0x438, 0x413, 0x430, 0x43c, 0x431,
0x438, 0x435, 0x415, 0x43d, 0x434, 0x44a, 0x440, 0x431, 0x44a, 0x440, 0x438, 0x41a,
0x43e, 0x448, 0x440, 0x430, 0x439, 0x41a, 0x443, 0x430, 0x434, 0x436, 0x430, 0x43b,
0x438, 0x43d, 0x41c, 0x430, 0x434, 0x436, 0x443, 0x440, 0x43e, 0x41c, 0x430, 0x440,
0x43a, 0x438, 0x437, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x438, 0x41c, 0x438, 0x434, 0x443, 0x435, 0x439, 0x41f, 0x43e, 0x43d, 0x43f, 0x435,
0x439, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x41c, 0x43e, 0x440, 0x441, 0x431, 0x438,
0x422, 0x430, 0x440, 0x430, 0x443, 0x430, 0x423, 0x435, 0x439, 0x43a, 0x423, 0x43e,
0x43b, 0x438, 0x441, 0x1021, 0x102c, 0x1018, 0x102e, 0x1002, 0x103b, 0x1014, 0x103a, 0x1021,
0x1000, 0x103a, 0x1000, 0x101b, 0x102c, 0x1021, 0x102c, 0x1038, 0x1012, 0x1005, 0x103a, 0x20,
0x1021, 0x1018, 0x102c, 0x1018, 0x102c, 0x1021, 0x101a, 0x103a, 0x101c, 0x103a, 0x1002, 0x103b,
0x102e, 0x1038, 0x101b, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x1021, 0x102c, 0x1038, 0x1005,
0x103a, 0x1019, 0x102c, 0x1038, 0x101b, 0x102c, 0x1018, 0x102c, 0x1019, 0x102c, 0x1000, 0x102d,
0x102f, 0x1018, 0x102c, 0x1014, 0x103a, 0x1002, 0x102e, 0x1038, 0x1018, 0x1014, 0x103a, 0x1002,
0x103b, 0x102f, 0x1038, 0x101c, 0x103a, 0x1018, 0x102e, 0x1005, 0x102c, 0x1021, 0x102d, 0x102f,
0x1018, 0x101c, 0x1014, 0x103a, 0x1010, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x101b, 0x1032,
0x101c, 0x103a, 0x1018, 0x101b, 0x102c, 0x1038, 0x1007, 0x102c, 0x1017, 0x102e, 0x1038, 0x101c,
0x103a, 0x1018, 0x1030, 0x1002, 0x103b, 0x103d, 0x1014, 0x103a, 0x1018, 0x1030, 0x1038, 0x101b,
0x102c, 0x1000, 0x102d, 0x102f, 0x1004, 0x103a, 0x101b, 0x102d, 0x102f, 0x1000, 0x102c, 0x1006,
0x102c, 0x1018, 0x101c, 0x1014, 0x103a, 0x1000, 0x102c, 0x1006, 0x102e, 0x101b, 0x1030, 0x1038,
0x1010, 0x102c, 0x1000, 0x102d, 0x102f, 0x1014, 0x102c, 0x1000, 0x101b, 0x102e, 0x1038, 0x1012,
0x1000, 0x103b, 0x1000, 0x102c, 0x1038, 0x1012, 0x102b, 0x101b, 0x1000, 0x103a, 0x1005, 0x103a,
0x1006, 0x102c, 0x101c, 0x1019, 0x103a, 0x1002, 0x103b, 0x102e, 0x1018, 0x1030, 0x1010, 0x102e,
0x1012, 0x102d, 0x102f, 0x1021, 0x1030, 0x1021, 0x102c, 0x101c, 0x102c, 0x1021, 0x101a, 0x103a,
0x101c, 0x103a, 0x1021, 0x102c, 0x101a, 0x103d, 0x1014, 0x103a, 0x1038, 0x1016, 0x101b, 0x102e,
0x1038, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1038, 0x1014, 0x103a, 0x1002, 0x102b, 0x1018,
0x102c, 0x101b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1014, 0x102e, 0x101f, 0x102c, 0x101b, 0x102c,
0x1038, 0x101b, 0x1032, 0x101a, 0x103a, 0x1002, 0x103b, 0x102d, 0x102f, 0x101f, 0x1014, 0x103a,
0x1038, 0x1014, 0x1000, 0x103a, 0x1005, 0x1018, 0x1010, 0x103a, 0x1002, 0x103b, 0x102f, 0x1018,
0x102c, 0x1000, 0x1019, 0x103a, 0x1015, 0x102b, 0x101c, 0x102c, 0x1001, 0x102b, 0x1010, 0x102d,
0x102f, 0x1021, 0x1014, 0x103a, 0x1000, 0x102e, 0x1002, 0x102b, 0x1038, 0x101c, 0x102e, 0x1000,
0x1004, 0x103a, 0x101b, 0x103e, 0x102c, 0x1005, 0x102c, 0x101c, 0x102c, 0x1002, 0x102d, 0x102f,
0x1005, 0x103a, 0x101c, 0x102e, 0x1017, 0x101b, 0x102c, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a,
0x101c, 0x102d, 0x102f, 0x1019, 0x102e, 0x101c, 0x1030, 0x101d, 0x1019, 0x103a, 0x1012, 0x102b,
0x101c, 0x1030, 0x1018, 0x103d, 0x1014, 0x103a, 0x1038, 0x1018, 0x102c, 0x101b, 0x103e, 0x102e,
0x101c, 0x1030, 0x1005, 0x102c, 0x1000, 0x102c, 0x1019, 0x102c, 0x101c, 0x102c, 0x1018, 0x102d,
0x102f, 0x1019, 0x102c, 0x1015, 0x1030, 0x1038, 0x1010, 0x102d, 0x102f, 0x1019, 0x102c, 0x1006,
0x1030, 0x101b, 0x1030, 0x1038, 0x1018, 0x102c, 0x1038, 0x1018, 0x102c, 0x1014, 0x103a, 0x1038,
0x1019, 0x1031, 0x102c, 0x103a, 0x1002, 0x102b, 0x1012, 0x102e, 0x1038, 0x101e, 0x103b, 0x102f,
0x1038, 0x1019, 0x103d, 0x1014, 0x103a, 0x101b, 0x102d, 0x102f, 0x1038, 0x1017, 0x102e, 0x1038,
0x101a, 0x102c, 0x1038, 0x1014, 0x102d, 0x102f, 0x1004, 0x103a, 0x101b, 0x102d, 0x102f, 0x1018,
0x102e, 0x1021, 0x1004, 0x103a, 0x1002, 0x103b, 0x102c, 0x1019, 0x102e, 0x1014, 0x102c, 0x100a,
0x102c, 0x1019, 0x1032, 0x101a, 0x103a, 0x1014, 0x102d, 0x102f, 0x1021, 0x102c, 0x1037, 0x1001,
0x103a, 0x101b, 0x103e, 0x1031, 0x102c, 0x1037, 0x1010, 0x103a, 0x101d, 0x102b, 0x1002, 0x102b,
0x1012, 0x1030, 0x1038, 0x1002, 0x1030, 0x1015, 0x102d, 0x102f, 0x1010, 0x102d, 0x102f, 0x2d,
0x1014, 0x102d, 0x102f, 0x1017, 0x102d, 0x102f, 0x1006, 0x1031, 0x102c, 0x1004, 0x103a, 0x1010,
0x1030, 0x1019, 0x1031, 0x1038, 0x1011, 0x101b, 0x102e, 0x1015, 0x102d, 0x102f, 0x101c, 0x102e,
0x1010, 0x1030, 0x1014, 0x102e, 0x1005, 0x103a, 0x1017, 0x1004, 0x103a, 0x1038, 0x101f, 0x1030,
0x1038, 0x1001, 0x103a, 0x1021, 0x102c, 0x1012, 0x1001, 0x103b, 0x1021, 0x1014, 0x103a, 0x1000,
0x102c, 0x101b, 0x1031, 0x1037, 0x1002, 0x103b, 0x103a, 0x1021, 0x1014, 0x103a, 0x1002, 0x102e,
0x101c, 0x102c, 0x1021, 0x1014, 0x103a, 0x1010, 0x102e, 0x1002, 0x103d, 0x102b, 0x1021, 0x102c,
0x101b, 0x102c, 0x1002, 0x103d, 0x102b, 0x1021, 0x102e, 0x1014, 0x102c, 0x1017, 0x103b, 0x1030,
0x1014, 0x102d, 0x102f, 0x1021, 0x1031, 0x1038, 0x101b, 0x102d, 0x1005, 0x103a, 0x1000, 0x102c,
0x1010, 0x102c, 0x1019, 0x102c, 0x101b, 0x1000, 0x102c, 0x1000, 0x102d, 0x102f, 0x1012, 0x102d,
0x102f, 0x1018, 0x102c, 0x1002, 0x103b, 0x102f, 0x1002, 0x103b, 0x1031, 0x101c, 0x102c, 0x20,
0x101b, 0x102e, 0x101a, 0x102d, 0x102f, 0x101f, 0x102c, 0x1019, 0x1014, 0x103a, 0x1012, 0x102d,
0x102f, 0x1007, 0x102c, 0x101b, 0x102e, 0x101a, 0x102d, 0x102f, 0x20, 0x1002, 0x102b, 0x101c,
0x102e, 0x1002, 0x102d, 0x102f, 0x1005, 0x103a, 0x1006, 0x102c, 0x101c, 0x103a, 0x1010, 0x102c,
0x1006, 0x1014, 0x103a, 0x20, 0x1002, 0x103d, 0x1019, 0x103a, 0x1006, 0x1014, 0x103a, 0x20,
0x101c, 0x1030, 0x101d, 0x102e, 0x1005, 0x103a, 0x1010, 0x1030, 0x1000, 0x1030, 0x1019, 0x1014,
0x103a, 0x1025, 0x1006, 0x103d, 0x102c, 0x1021, 0x102e, 0x1021, 0x102c, 0x1021, 0x102c, 0x101b,
0x1030, 0x1038, 0x1017, 0x102c, 0x1021, 0x102c, 0x1006, 0x1030, 0x1005, 0x102e, 0x1021, 0x103d,
0x1014, 0x103a, 0x1038, 0x1021, 0x102c, 0x1010, 0x102e, 0x1000, 0x102d, 0x102f, 0x1000, 0x1014,
0x103a, 0x1018, 0x102c, 0x101f, 0x102e, 0x1021, 0x102c, 0x1018, 0x102c, 0x101f, 0x102e, 0x1021,
0x102c, 0x20, 0x1018, 0x1014, 0x103a, 0x1012, 0x101b, 0x1000, 0x103a, 0x1005, 0x103a, 0x1018,
0x102c, 0x1018, 0x1031, 0x1038, 0x1012, 0x102d, 0x102f, 0x1038, 0x1005, 0x103a, 0x1018, 0x102e,
0x101c, 0x1004, 0x103a, 0x1019, 0x103a, 0x1018, 0x101c, 0x102d, 0x1007, 0x103a, 0x1018, 0x101c,
0x103d, 0x1014, 0x103a, 0x1001, 0x103a, 0x2d, 0x1005, 0x102c, 0x1018, 0x101c, 0x103d, 0x1014,
0x103a, 0x1018, 0x102d, 0x102f, 0x1021, 0x102c, 0x1017, 0x102e, 0x1005, 0x103a, 0x1010, 0x102c,
0x1018, 0x102d, 0x102f, 0x1002, 0x102d, 0x102f, 0x1010, 0x102c, 0x1017, 0x103d, 0x102d, 0x102f,
0x1000, 0x103a, 0x1005, 0x102e, 0x1000, 0x102d, 0x1014, 0x103a, 0x1038, 0x1018, 0x101b, 0x1005,
0x103a, 0x1001, 0x103b, 0x103a, 0x20, 0x1018, 0x1031, 0x1038, 0x1000, 0x102d, 0x1019, 0x103a,
0x1015, 0x102d, 0x102f, 0x20, 0x1002, 0x101b, 0x1014, 0x103a, 0x1012, 0x102e, 0x1000, 0x1014,
0x103a, 0x1001, 0x1014, 0x103a, 0x1000, 0x102c, 0x101b, 0x102c, 0x1000, 0x102c, 0x1005, 0x103a,
0x1000, 0x1031, 0x100a, 0x1004, 0x103a, 0x1014, 0x103a, 0x1000, 0x1031, 0x1019, 0x1014, 0x103a,
0x1001, 0x103b, 0x102e, 0x1000, 0x102c, 0x1000, 0x102d, 0x102f, 0x1001, 0x103b, 0x102e, 0x101f,
0x1030, 0x1021, 0x102c, 0x101f, 0x1030, 0x1021, 0x102c, 0x1005, 0x102e, 0x101a, 0x1030, 0x1012,
0x1010, 0x103a, 0x1005, 0x103a, 0x20, 0x101f, 0x103d, 0x102c, 0x101b, 0x1000, 0x103a, 0x1005,
0x103a, 0x1000, 0x102d, 0x102f, 0x1037, 0x1005, 0x1010, 0x102c, 0x101b, 0x102e, 0x1000, 0x102c,
0x1000, 0x101b, 0x1005, 0x103a, 0x1005, 0x1010, 0x103d, 0x1014, 0x103a, 0x1000, 0x103d, 0x102e,
0x101b, 0x102c, 0x1018, 0x102c, 0x1000, 0x103b, 0x1030, 0x101b, 0x1031, 0x1038, 0x1000, 0x102d,
0x102f, 0x1038, 0x1012, 0x1014, 0x103a, 0x1019, 0x1010, 0x103a, 0x101b, 0x103e, 0x103a, 0x1017,
0x102c, 0x1014, 0x103a, 0x1012, 0x1031, 0x102b, 0x103a, 0x1006, 0x1014, 0x103a, 0x1012, 0x1031,
0x102b, 0x103a, 0x1006, 0x1014, 0x103a, 0x20, 0x1001, 0x101b, 0x102d, 0x1001, 0x103a, 0x1012,
0x1004, 0x103a, 0x1014, 0x103a, 0x1017, 0x102c, 0x1012, 0x1000, 0x103a, 0x1011, 0x101b, 0x103d,
0x102d, 0x102f, 0x1000, 0x103a, 0x1012, 0x102d, 0x102f, 0x1019, 0x102e, 0x1014, 0x102e, 0x1000,
0x102c, 0x1021, 0x1000, 0x103a, 0x1012, 0x103a, 0x1019, 0x103d, 0x1014, 0x103a, 0x1010, 0x1014,
0x103a, 0x1021, 0x102e, 0x101b, 0x1030, 0x1014, 0x102e, 0x1015, 0x102e, 0x1021, 0x101a, 0x103a,
0x101c, 0x103a, 0x1006, 0x102c, 0x1017, 0x1031, 0x1012, 0x102d, 0x102f, 0x1016, 0x102d, 0x102f,
0x1037, 0x1010, 0x103a, 0x20, 0x1014, 0x101a, 0x103a, 0x101c, 0x103a, 0x1006, 0x1004, 0x103a,
0x1016, 0x102d, 0x102f, 0x1037, 0x1010, 0x103a, 0x1010, 0x102c, 0x101c, 0x102e, 0x1007, 0x102c,
0x1002, 0x101c, 0x1032, 0x1005, 0x103a, 0x1018, 0x1031, 0x1038, 0x1002, 0x1030, 0x1038, 0x1005,
0x103a, 0x1018, 0x1031, 0x1038, 0x1002, 0x101b, 0x1014, 0x103a, 0x1012, 0x103a, 0x20, 0x1010,
0x1001, 0x103a, 0x1002, 0x101b, 0x102e, 0x1014, 0x1031, 0x1012, 0x102b, 0x1002, 0x103d, 0x102b,
0x1012, 0x102e, 0x101c, 0x102f, 0x1015, 0x103a, 0x1002, 0x103d, 0x102b, 0x1010, 0x102e, 0x1019,
0x102c, 0x101c, 0x102c, 0x1002, 0x103d, 0x102b, 0x101b, 0x102c, 0x1000, 0x103d, 0x102e, 0x1038,
0x101c, 0x103a, 0x1002, 0x102d, 0x102f, 0x1004, 0x103a, 0x101a, 0x102c, 0x1014, 0x102c, 0x101f,
0x101c, 0x102e, 0x1016, 0x1000, 0x103a, 0x1005, 0x103a, 0x101f, 0x102c, 0x1017, 0x102c, 0x1014,
0x102c, 0x101f, 0x102c, 0x1019, 0x102d, 0x102f, 0x1005, 0x103a, 0x1005, 0x102e, 0x101c, 0x102d,
0x102f, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038,
0x20, 0x1015, 0x102d, 0x102f, 0x101c, 0x102d, 0x1005, 0x103a, 0x1014, 0x1031, 0x102c, 0x1000,
0x103a, 0x1001, 0x103a, 0x1005, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e,
0x101a, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038, 0x1019, 0x102c, 0x101b, 0x1014, 0x103a, 0x1002,
0x102d, 0x102f, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038,
0x1014, 0x102c, 0x1038, 0x1015, 0x102e, 0x1010, 0x102c, 0x1005, 0x103a, 0x1018, 0x1010, 0x103a,
0x1001, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038,
0x1014, 0x102c, 0x1038, 0x1010, 0x1032, 0x101c, 0x103a, 0x1005, 0x102e, 0x1038, 0x1010, 0x102e,
0x1038, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038, 0x1014,
0x102c, 0x1038, 0x1017, 0x102e, 0x1017, 0x1032, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012,
0x102e, 0x101a, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038, 0x1017, 0x1004, 0x103a, 0x1006, 0x1004,
0x1037, 0x103a, 0x1005, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a,
0x102c, 0x1038, 0x1014, 0x102c, 0x1038, 0x101d, 0x102e, 0x1014, 0x102c, 0x1019, 0x1000, 0x103a,
0x1001, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038,
0x1014, 0x102c, 0x1038, 0x1021, 0x102e, 0x1014, 0x102f, 0x1017, 0x102e, 0x1001, 0x103a, 0x1021,
0x102e, 0x1000, 0x102c, 0x101c, 0x1030, 0x1021, 0x102e, 0x1010, 0x103a, 0x1002, 0x103b, 0x1019,
0x1031, 0x1000, 0x102c, 0x1002, 0x103b, 0x102f, 0x1014, 0x102d, 0x102f, 0x1021, 0x1031, 0x102c,
0x101c, 0x1030, 0x101d, 0x102e, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a, 0x1019, 0x103d, 0x1014,
0x103a, 0x1010, 0x102e, 0x1001, 0x103b, 0x101a, 0x103a, 0x101c, 0x102d, 0x102f, 0x104a, 0x20,
0x1000, 0x1004, 0x103a, 0x1010, 0x1015, 0x103a, 0x1000, 0x102e, 0x1001, 0x101b, 0x102c, 0x101c,
0x1032, 0x1014, 0x103a, 0x1038, 0x1012, 0x1005, 0x103a, 0x1001, 0x103b, 0x103a, 0x101c, 0x102c,
0x1015, 0x102b, 0x1007, 0x103a, 0x101c, 0x102e, 0x1019, 0x102c, 0x101c, 0x1031, 0x102c, 0x1037,
0x1005, 0x103a, 0x1021, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x101a, 0x103a, 0x101c, 0x102d,
0x1005, 0x103a, 0x101c, 0x102d, 0x102f, 0x1021, 0x102c, 0x1015, 0x101b, 0x1004, 0x1037, 0x103a,
0x1005, 0x103a, 0x20, 0x1000, 0x103d, 0x102c, 0x1010, 0x102c, 0x1019, 0x102c, 0x1006, 0x1032,
0x101e, 0x103d, 0x102c, 0x1019, 0x102c, 0x1014, 0x102c, 0x1002, 0x103d, 0x102b, 0x1019, 0x102c,
0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1005, 0x103a, 0x1019, 0x102c, 0x101b, 0x102e, 0x1002,
0x1031, 0x102b, 0x1037, 0x1019, 0x102c, 0x1010, 0x102e, 0x1014, 0x102e, 0x1002, 0x103a, 0x1019,
0x102c, 0x1010, 0x102c, 0x1019, 0x102d, 0x102f, 0x101b, 0x102d, 0x102f, 0x1005, 0x103a, 0x1019,
0x102c, 0x1007, 0x1010, 0x103a, 0x101c, 0x1014, 0x103a, 0x1019, 0x102e, 0x1014, 0x102d, 0x102f,
0x1019, 0x102e, 0x1014, 0x102e, 0x1038, 0x1019, 0x102e, 0x101b, 0x102e, 0x1012, 0x102b, 0x1019,
0x1000, 0x103a, 0x1010, 0x103a, 0x101c, 0x102c, 0x1000, 0x1000, 0x103a, 0x101c, 0x102c, 0x1019,
0x1000, 0x103a, 0x1000, 0x1006, 0x102e, 0x1000, 0x102d, 0x102f, 0x20, 0x1005, 0x102e, 0x1038,
0x1010, 0x102e, 0x1038, 0x1019, 0x102e, 0x1000, 0x103d, 0x102e, 0x101c, 0x103d, 0x1014, 0x103a,
0x1019, 0x103d, 0x1014, 0x103a, 0x1001, 0x103a, 0x1010, 0x103d, 0x1014, 0x103a, 0x1019, 0x103d,
0x1014, 0x103a, 0x1010, 0x1032, 0x101b, 0x1031, 0x1038, 0x1019, 0x103d, 0x1014, 0x103a, 0x1010,
0x102e, 0x1017, 0x102e, 0x1012, 0x102e, 0x1021, 0x102d, 0x102f, 0x1019, 0x103d, 0x1014, 0x1037,
0x103a, 0x28, 0x1010, 0x103a, 0x29, 0x1006, 0x1031, 0x1038, 0x101b, 0x1010, 0x103a, 0x1014,
0x102c, 0x1037, 0x1006, 0x1031, 0x102c, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x101a, 0x1031,
0x102c, 0x1000, 0x103a, 0x1014, 0x102d, 0x102f, 0x101b, 0x102d, 0x102f, 0x100a, 0x102c, 0x1017,
0x103c, 0x1030, 0x101c, 0x102c, 0x104a, 0x20, 0x1019, 0x103c, 0x1031, 0x102c, 0x1000, 0x103a,
0x1012, 0x102b, 0x1000, 0x102d, 0x102f, 0x1010, 0x102c, 0x1005, 0x1004, 0x103a, 0x1010, 0x102c,
0x104a, 0x20, 0x1019, 0x103c, 0x1031, 0x102c, 0x1000, 0x103a, 0x1012, 0x102b, 0x1000, 0x102d,
0x102f, 0x1010, 0x102c, 0x1014, 0x101a, 0x1030, 0x1038, 0x1006, 0x1031, 0x1038, 0x101c, 0x1019,
0x103a, 0x104a, 0x20, 0x1019, 0x103c, 0x1031, 0x102c, 0x1000, 0x103a, 0x1012, 0x102b, 0x1000,
0x102d, 0x102f, 0x1010, 0x102c, 0x1014, 0x102f, 0x1001, 0x103a, 0x1021, 0x102d, 0x102f, 0x1001,
0x102e, 0x1014, 0x1002, 0x102b, 0x1015, 0x1014, 0x102c, 0x1038, 0x1019, 0x102c, 0x1038, 0x1015,
0x102b, 0x101b, 0x102c, 0x1019, 0x102c, 0x101b, 0x102e, 0x1018, 0x102d, 0x102f, 0x1016, 0x102e,
0x1038, 0x1014, 0x1005, 0x103a, 0x1015, 0x102d, 0x102f, 0x1037, 0x1010, 0x103a, 0x2d, 0x1021,
0x102d, 0x102f, 0x2d, 0x1015, 0x101b, 0x1004, 0x1037, 0x103a, 0x1005, 0x103a, 0x1015, 0x102d,
0x102f, 0x1037, 0x1010, 0x103a, 0x20, 0x1021, 0x1031, 0x102c, 0x1037, 0x1016, 0x103a, 0x20,
0x1005, 0x1015, 0x102d, 0x1014, 0x103a, 0x1015, 0x102d, 0x102f, 0x1037, 0x1010, 0x103a, 0x1010,
0x102d, 0x102f, 0x20, 0x1017, 0x1032, 0x101c, 0x102e, 0x101a, 0x102d, 0x102f, 0x1015, 0x1031,
0x102b, 0x103a, 0x1010, 0x102d, 0x102f, 0x101b, 0x102e, 0x1000, 0x102d, 0x102f, 0x1015, 0x103d,
0x1014, 0x103a, 0x1010, 0x102c, 0x20, 0x1021, 0x101b, 0x102e, 0x1014, 0x102c, 0x1005, 0x103a,
0x101b, 0x1014, 0x103a, 0x1000, 0x1004, 0x103a, 0x20, 0x1021, 0x1004, 0x103a, 0x1038, 0x101c,
0x1000, 0x103a, 0x101f, 0x1031, 0x1005, 0x102e, 0x1016, 0x102e, 0x101c, 0x103a, 0x101b, 0x101a,
0x103a, 0x1002, 0x103b, 0x102e, 0x1014, 0x102c, 0x101b, 0x102e, 0x1006, 0x102d, 0x102f, 0x101c,
0x102f, 0x1015, 0x103a, 0x28, 0x1010, 0x103a, 0x29, 0x101b, 0x102e, 0x101a, 0x102d, 0x102f,
0x20, 0x1018, 0x101b, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x1006, 0x1014, 0x103a, 0x1010,
0x102c, 0x101b, 0x1019, 0x103a, 0x1006, 0x1014, 0x103a, 0x1010, 0x102e, 0x1021, 0x102c, 0x1002,
0x102d, 0x102f, 0x1006, 0x1014, 0x103a, 0x1010, 0x102d, 0x102f, 0x20, 0x1012, 0x102d, 0x102f,
0x1019, 0x1004, 0x103a, 0x1038, 0x1002, 0x102d, 0x102f, 0x1006, 0x1031, 0x102c, 0x103a, 0x1015,
0x102d, 0x102f, 0x101c, 0x102d, 0x102f, 0x1021, 0x1005, 0x103a, 0x1010, 0x102c, 0x1000, 0x102c,
0x20, 0x1010, 0x102d, 0x102f, 0x1038, 0x1019, 0x102d, 0x101b, 0x1000, 0x103a, 0x1010, 0x103a,
0x1005, 0x1005, 0x103a, 0x1000, 0x102c, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1018, 0x102c,
0x101e, 0x101a, 0x103a, 0x101c, 0x1019, 0x102e, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1002,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1000, 0x102d,
0x1005, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x101c, 0x1030, 0x1005, 0x102e, 0x101a,
0x102c, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x101e, 0x1031, 0x102c, 0x1019, 0x1010, 0x103a,
0x1005, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1017, 0x1004, 0x103a, 0x1038, 0x1006,
0x1004, 0x1037, 0x103a, 0x1005, 0x103d, 0x1010, 0x103a, 0x1016, 0x103a, 0x1010, 0x103a, 0x20,
0x1000, 0x102c, 0x1038, 0x101b, 0x1004, 0x1037, 0x103a, 0x1010, 0x102e, 0x1002, 0x1030, 0x1005,
0x102e, 0x1002, 0x101c, 0x103a, 0x1015, 0x102b, 0x101e, 0x102f, 0x101c, 0x102e, 0x1010, 0x102e,
0x1002, 0x103d, 0x102b, 0x1014, 0x102c, 0x1010, 0x102d, 0x102f, 0x101b, 0x1014, 0x103a, 0x1010,
0x102d, 0x102f, 0x1010, 0x1031, 0x102c, 0x1010, 0x102d, 0x102f, 0x101c, 0x102c, 0x1017, 0x1014,
0x103a, 0x1000, 0x1030, 0x1038, 0x1017, 0x102c, 0x1038, 0x101d, 0x102d, 0x102f, 0x1000, 0x103a,
0x28, 0x1010, 0x103a, 0x29, 0x101f, 0x102d, 0x102f, 0x1005, 0x103a, 0x101d, 0x102e, 0x1014,
0x102e, 0x1017, 0x1000, 0x103a, 0x1002, 0x103a, 0x101b, 0x102c, 0x1000, 0x102f, 0x1010, 0x1010,
0x103a, 0x1000, 0x1031, 0x1005, 0x102e, 0x1012, 0x1031, 0x1038, 0x1017, 0x1005, 0x103a, 0x1012,
0x1030, 0x1038, 0x1019, 0x1031, 0x102c, 0x1037, 0x1010, 0x103a, 0x20, 0x1012, 0x102b, 0x101b,
0x103a, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a, 0x1019, 0x1000, 0x103a, 0x1000, 0x103d, 0x101a,
0x103a, 0x101b, 0x102e, 0x1019, 0x1031, 0x102c, 0x103a, 0x1005, 0x103d, 0x1014, 0x103a, 0x1019,
0x1000, 0x103a, 0x1019, 0x1030, 0x1012, 0x102d, 0x102f, 0x1038, 0x1015, 0x102b, 0x101c, 0x103a,
0x1019, 0x102c, 0x101b, 0x102d, 0x102f, 0x101e, 0x102e, 0x101b, 0x102c, 0x101b, 0x103e, 0x102d,
0x102f, 0x1038, 0x101d, 0x102b, 0x1011, 0x101b, 0x102d, 0x102f, 0x101c, 0x103a, 0x1017, 0x102d,
0x102f, 0x1005, 0x103a, 0x1010, 0x102d, 0x102f, 0x1001, 0x103a, 0x101c, 0x1031, 0x102c, 0x1004,
0x103a, 0x101b, 0x102e, 0x101a, 0x102c, 0x1038, 0x1018, 0x101b, 0x1036, 0x1021, 0x102c, 0x1012,
0x1004, 0x103a, 0x1021, 0x1031, 0x102c, 0x103a, 0x1019, 0x102c, 0x1010, 0x102e, 0x1021, 0x102c,
0x1019, 0x102c, 0x1014, 0x103a, 0x1038, 0x1021, 0x1014, 0x103a, 0x1021, 0x102c, 0x1012, 0x102e,
0x1021, 0x102c, 0x1021, 0x1000, 0x103a, 0x1010, 0x102c, 0x1025, 0x102f, 0x1021, 0x102c, 0x1001,
0x103b, 0x1010, 0x1030, 0x1038, 0x1018, 0x102e, 0x1021, 0x102c, 0x101b, 0x103e, 0x103a, 0x1002,
0x102b, 0x1018, 0x102c, 0x1012, 0x103a, 0x1021, 0x102c, 0x1010, 0x102e, 0x101b, 0x1021, 0x1030,
0x1018, 0x1002, 0x1039, 0x1002, 0x1012, 0x1000, 0x103a, 0x1018, 0x102c, 0x101b, 0x102d, 0x1014,
0x103a, 0x1038, 0x1018, 0x102c, 0x1000, 0x1030, 0x1018, 0x1014, 0x103a, 0x1000, 0x1031, 0x102c,
0x1000, 0x103a, 0x1018, 0x101b, 0x103a, 0x1014, 0x102c, 0x1021, 0x1030, 0x1018, 0x1031, 0x101b,
0x103d, 0x1010, 0x103a, 0x1018, 0x102e, 0x101b, 0x103e, 0x103a, 0x1001, 0x1000, 0x103a, 0x1018,
0x101b, 0x1030, 0x1014, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1001, 0x103b, 0x102e, 0x1010,
0x102c, 0x1000, 0x102d, 0x102f, 0x101c, 0x1036, 0x1018, 0x102d, 0x102f, 0x1012, 0x1019, 0x102c,
0x1038, 0x1005, 0x1000, 0x1015, 0x103a, 0x1012, 0x1000, 0x103a, 0x1000, 0x102c, 0x1012, 0x1005,
0x103a, 0x101c, 0x102e, 0x1012, 0x1030, 0x1018, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1012,
0x1030, 0x101b, 0x103e, 0x1014, 0x103a, 0x1038, 0x1018, 0x1032, 0x1016, 0x102c, 0x1019, 0x102c,
0x1002, 0x1030, 0x1005, 0x1010, 0x102c, 0x1002, 0x102b, 0x1007, 0x102c, 0x101f, 0x102e, 0x1018,
0x101b, 0x103d, 0x1014, 0x103a, 0x101f, 0x102d, 0x102f, 0x1001, 0x103b, 0x102e, 0x1019, 0x1004,
0x103a, 0x1038, 0x1005, 0x102e, 0x1038, 0x1010, 0x102e, 0x1038, 0x101f, 0x1031, 0x102c, 0x1004,
0x103a, 0x1000, 0x1031, 0x102c, 0x1004, 0x103a, 0x101f, 0x102d, 0x102f, 0x1038, 0x1017, 0x103b,
0x1021, 0x102e, 0x101b, 0x1030, 0x1001, 0x1030, 0x1010, 0x103a, 0x1002, 0x103b, 0x1000, 0x102c,
0x1010, 0x102c, 0x1002, 0x103b, 0x102c, 0x101b, 0x102c, 0x1015, 0x1030, 0x101b, 0x102c, 0x1002,
0x103b, 0x1031, 0x101b, 0x102f, 0x1006, 0x101c, 0x1004, 0x103a, 0x1000, 0x102c, 0x1018, 0x1030,
0x1038, 0x101c, 0x103a, 0x1001, 0x1019, 0x103a, 0x1001, 0x103b, 0x102c, 0x1037, 0x1001, 0x102b,
0x1000, 0x101b, 0x102c, 0x1001, 0x103b, 0x102d, 0x1001, 0x1010, 0x103a, 0x1010, 0x1019, 0x1014,
0x103a, 0x1012, 0x1030, 0x1001, 0x1014, 0x103a, 0x1012, 0x102d, 0x102f, 0x1004, 0x103a, 0x1002,
0x102b, 0x1000, 0x102d, 0x102f, 0x101c, 0x103b, 0x1000, 0x1010, 0x103a, 0x1010, 0x102c, 0x1038,
0x1001, 0x101b, 0x102c, 0x1037, 0x1005, 0x103a, 0x1014, 0x102d, 0x102f, 0x101b, 0x102c, 0x1005,
0x103a, 0x1000, 0x103d, 0x102c, 0x101c, 0x102c, 0x101c, 0x1019, 0x103a, 0x1015, 0x1030, 0x1000,
0x1030, 0x1001, 0x103b, 0x1004, 0x103a, 0x1000, 0x1030, 0x101d, 0x102d, 0x1010, 0x103a, 0x1019,
0x1000, 0x102c, 0x1021, 0x102d, 0x102f, 0x1019, 0x102c, 0x1002, 0x102b, 0x1012, 0x1014, 0x103a,
0x1038, 0x1019, 0x1001, 0x1000, 0x103a, 0x1005, 0x103a, 0x1006, 0x102c, 0x1019, 0x1014, 0x102e,
0x101c, 0x102c, 0x1019, 0x1010, 0x103a, 0x1005, 0x103a, 0x1000, 0x1000, 0x103a, 0x1010, 0x103a,
0x1014, 0x102e, 0x1000, 0x102d, 0x102f, 0x101b, 0x103e, 0x102c, 0x1038, 0x1014, 0x102d, 0x102f,
0x1017, 0x102d, 0x102f, 0x1001, 0x1030, 0x1007, 0x103a, 0x1014, 0x1000, 0x103a, 0x1005, 0x103a,
0x1014, 0x102d, 0x102f, 0x1017, 0x102d, 0x102f, 0x1005, 0x1032, 0x1018, 0x102e, 0x1021, 0x1032,
0x101a, 0x103a, 0x1005, 0x103a, 0x1021, 0x103d, 0x1019, 0x103a, 0x1038, 0x1005, 0x103a, 0x1001,
0x103a, 0x1021, 0x1031, 0x102c, 0x103a, 0x101b, 0x1032, 0x101c, 0x103a, 0x1016, 0x1014, 0x103d,
0x1019, 0x103a, 0x1015, 0x1004, 0x103a, 0x1015, 0x103d, 0x1014, 0x103a, 0x1010, 0x102e, 0x1021,
0x102c, 0x1038, 0x1014, 0x102c, 0x1037, 0x1001, 0x103a, 0x1015, 0x103c, 0x102f, 0x1036, 0x101a,
0x1014, 0x103a, 0x1038, 0x1000, 0x1031, 0x102c, 0x1037, 0x1005, 0x103a, 0x1010, 0x1014, 0x1031,
0x1000, 0x102e, 0x1007, 0x101c, 0x1031, 0x102c, 0x103a, 0x1012, 0x102b, 0x101b, 0x102e, 0x101a,
0x102c, 0x1038, 0x1012, 0x103a, 0x1006, 0x102c, 0x1001, 0x102b, 0x101c, 0x1004, 0x103a, 0x1006,
0x1019, 0x103a, 0x1038, 0x1019, 0x102c, 0x1001, 0x1014, 0x103a, 0x1038, 0x1012, 0x103a, 0x1006,
0x102d, 0x102f, 0x1038, 0x101c, 0x103a, 0x101b, 0x103e, 0x1014, 0x103a, 0x101f, 0x102d, 0x102f,
0x1004, 0x103a, 0x1038, 0x1005, 0x1004, 0x103a, 0x1039, 0x1000, 0x102c, 0x1015, 0x1030, 0x1006,
0x101b, 0x1005, 0x103a, 0x1014, 0x1005, 0x103a, 0x1000, 0x102c, 0x101c, 0x1004, 0x103a, 0x1019,
0x103a, 0x1005, 0x103a, 0x1010, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1031, 0x1010, 0x102c,
0x101b, 0x103e, 0x103a, 0x1000, 0x1032, 0x1014, 0x1037, 0x103a, 0x1010, 0x1018, 0x102e, 0x1038,
0x101c, 0x102e, 0x1005, 0x102e, 0x1010, 0x102e, 0x101f, 0x102e, 0x101b, 0x1014, 0x103a, 0x1010,
0x1004, 0x103a, 0x1016, 0x1030, 0x1038, 0x1010, 0x102d, 0x102f, 0x1000, 0x103b, 0x102d, 0x102f,
0x1010, 0x103d, 0x1014, 0x103a, 0x1019, 0x103a, 0x1005, 0x103a, 0x1001, 0x103a, 0x1025, 0x101c,
0x1014, 0x103a, 0x1018, 0x102c, 0x1010, 0x1031, 0x102c, 0x1021, 0x1030, 0x101b, 0x102f, 0x1019,
0x103a, 0x1001, 0x103b, 0x102e, 0x1021, 0x1030, 0x1038, 0x1005, 0x103a, 0x20, 0x1014, 0x102e,
0x101b, 0x102c, 0x1038, 0x1017, 0x102e, 0x101a, 0x1004, 0x103a, 0x1000, 0x103b, 0x1014, 0x103a,
0x1038, 0x1017, 0x101c, 0x102c, 0x1012, 0x102e, 0x1017, 0x1031, 0x102c, 0x1037, 0x1005, 0x1010,
0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x101a, 0x1030, 0x1001, 0x1030, 0x1038, 0x1010, 0x103a,
0x1005, 0x103a, 0x101b, 0x1014, 0x103a, 0x1000, 0x102f, 0x1014, 0x103a, 0x101b, 0x101a, 0x103a,
0x1001, 0x102b, 0x1038, 0x1010, 0x102e, 0x101b, 0x1004, 0x103a, 0x1018, 0x102c, 0x1038, 0x1001,
0x103a, 0x101b, 0x1032, 0x101a, 0x103a, 0x101b, 0x1031, 0x1017, 0x1014, 0x103a, 0x1038, 0x1021,
0x1031, 0x1007, 0x102d, 0x102f, 0x1038, 0x101b, 0x102e, 0x1038, 0x1005, 0x103a, 0x1018, 0x102c,
0x1019, 0x103c, 0x1030, 0x1012, 0x102b, 0x1000, 0x1014, 0x1031, 0x101b, 0x102e, 0x1000, 0x102d,
0x1010, 0x103a, 0x1015, 0x103a, 0x20, 0x1017, 0x102c, 0x1012, 0x102e, 0x1016, 0x102b, 0x101b,
0x102d, 0x102f, 0x1019, 0x1012, 0x102e, 0x1038, 0x101b, 0x102c, 0x101b, 0x1031, 0x1038, 0x1000,
0x102e, 0x1017, 0x1005, 0x103a, 0x1001, 0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1002,
0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b, 0x102e, 0x101a, 0x102c, 0x1005, 0x102d, 0x1014,
0x1037, 0x103a, 0x101f, 0x101a, 0x103a, 0x101c, 0x101a, 0x103a, 0x1014, 0x102c, 0x1005, 0x1010,
0x1014, 0x103a, 0x101c, 0x1031, 0x1021, 0x1014, 0x1039, 0x1012, 0x102e, 0x101c, 0x102d, 0x1010,
0x103a, 0x1012, 0x103a, 0x1018, 0x101b, 0x1005, 0x1039, 0x1005, 0x1018, 0x102d, 0x1014, 0x103a,
0x1038, 0x1018, 0x101b, 0x102d, 0x102f, 0x1000, 0x1004, 0x103a, 0x101f, 0x102e, 0x1038, 0x101c,
0x103a, 0x1012, 0x102b, 0x101d, 0x1004, 0x103a, 0x101a, 0x1030, 0x1038, 0x1001, 0x101c, 0x102c,
0x101f, 0x102d, 0x102f, 0x1038, 0x1018, 0x102c, 0x1038, 0x1010, 0x103a, 0x101c, 0x1004, 0x103a,
0x1038, 0x1012, 0x102e, 0x1019, 0x1014, 0x103a, 0x1038, 0x101c, 0x1031, 0x102c, 0x1037, 0x1012,
0x103a, 0x20, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x1019, 0x1032, 0x101c, 0x103a, 0x1018,
0x102f, 0x1014, 0x103a, 0x1038, 0x1015, 0x102b, 0x1038, 0x101e, 0x103a, 0x1006, 0x1005, 0x103a,
0x1012, 0x1014, 0x102e, 0x1021, 0x1019, 0x103a, 0x1005, 0x1010, 0x102c, 0x1012, 0x1019, 0x103a,
0x1021, 0x1014, 0x103a, 0x1012, 0x102d, 0x102f, 0x101b, 0x102c, 0x1021, 0x102c, 0x1038, 0x1005,
0x103a, 0x1010, 0x101b, 0x1001, 0x1014, 0x103a, 0x1038, 0x1021, 0x1031, 0x101e, 0x1004, 0x103a,
0x1018, 0x1032, 0x101c, 0x103a, 0x1002, 0x101b, 0x102d, 0x1010, 0x103a, 0x1018, 0x102c, 0x101c,
0x1004, 0x103a, 0x1018, 0x101b, 0x102c, 0x1010, 0x102e, 0x1038, 0x1005, 0x103a, 0x101c, 0x102c,
0x1038, 0x1017, 0x102c, 0x1018, 0x101b, 0x1015, 0x103a, 0x1006, 0x1032, 0x101c, 0x103a, 0x1018,
0x1030, 0x1001, 0x102b, 0x101b, 0x1000, 0x103a, 0x1005, 0x103a, 0x1018, 0x1030, 0x1012, 0x102b,
0x1015, 0x1000, 0x103a, 0x1005, 0x103a, 0x1018, 0x1030, 0x101b, 0x103e, 0x1004, 0x103a, 0x1002,
0x103b, 0x1004, 0x103a, 0x1001, 0x103b, 0x102e, 0x101b, 0x103e, 0x102e, 0x1014, 0x102c, 0x1038,
0x1021, 0x1030, 0x1000, 0x102d, 0x102f, 0x1015, 0x1004, 0x103a, 0x101f, 0x1031, 0x1002, 0x1004,
0x103a, 0x1012, 0x1010, 0x103a, 0x1018, 0x101c, 0x1004, 0x103a, 0x1002, 0x103b, 0x102e, 0x1018,
0x101b, 0x1031, 0x102c, 0x103a, 0x101c, 0x103a, 0x1010, 0x102c, 0x1002, 0x103d, 0x1014, 0x103a,
0x1038, 0x1007, 0x102e, 0x101f, 0x1032, 0x101c, 0x103a, 0x1005, 0x1004, 0x103a, 0x1000, 0x102e,
0x1019, 0x1014, 0x103a, 0x1038, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1021, 0x1005,
0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x1018, 0x1030, 0x101c, 0x103a, 0x1002, 0x103b, 0x102c,
0x1005, 0x102e, 0x1001, 0x102b, 0x101c, 0x102e, 0x1014, 0x1004, 0x103a, 0x1002, 0x101b, 0x1000,
0x103a, 0x1001, 0x102e, 0x101b, 0x102d, 0x102f, 0x1017, 0x103a, 0x1001, 0x102e, 0x1038, 0x1021,
0x1000, 0x103a, 0x1016, 0x103a, 0x101c, 0x1005, 0x1039, 0x1005, 0x1018, 0x103d, 0x1014, 0x103a,
0x1038, 0x101c, 0x1030, 0x1018, 0x101c, 0x102e, 0x1021, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038,
0x101c, 0x1014, 0x103a, 0x1012, 0x1014, 0x103a, 0x101c, 0x1030, 0x1007, 0x1004, 0x103a, 0x1018,
0x1010, 0x103a, 0x1019, 0x1012, 0x101b, 0x1005, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a, 0x101c,
0x103a, 0x1010, 0x102c, 0x1019, 0x101b, 0x102e, 0x1021, 0x102c, 0x20, 0x101f, 0x102c, 0x1038,
0x1019, 0x1014, 0x103a, 0x1019, 0x1004, 0x103a, 0x1038, 0x1005, 0x1001, 0x103a, 0x1019, 0x102d,
0x102f, 0x1014, 0x102c, 0x1000, 0x102d, 0x102f, 0x1019, 0x1031, 0x102c, 0x103a, 0x1005, 0x1000,
0x102d, 0x102f, 0x1021, 0x1031, 0x102c, 0x103a, 0x1005, 0x101c, 0x102d, 0x102f, 0x1015, 0x1032,
0x101b, 0x1005, 0x103a, 0x1015, 0x1031, 0x102b, 0x1037, 0x1002, 0x102d, 0x102f, 0x101b, 0x102e,
0x1000, 0x102c, 0x1015, 0x101b, 0x1000, 0x103a, 0x1002, 0x103a, 0x101b, 0x102e, 0x1002, 0x102b,
0x101b, 0x1031, 0x102c, 0x1019, 0x1006, 0x102c, 0x1019, 0x102c, 0x1038, 0x101b, 0x102c, 0x1006,
0x1014, 0x103a, 0x1019, 0x101b, 0x102e, 0x1038, 0x1014, 0x102d, 0x102f, 0x1006, 0x102c, 0x101b,
0x102c, 0x101a, 0x1031, 0x1017, 0x102d, 0x102f, 0x1006, 0x102c, 0x101b, 0x102c, 0x1010, 0x102d,
0x102f, 0x1037, 0x1016, 0x103a, 0x1005, 0x1004, 0x103a, 0x1016, 0x102c, 0x101b, 0x102d, 0x102f,
0x1038, 0x1016, 0x102d, 0x102f, 0x1005, 0x1000, 0x1031, 0x102c, 0x1037, 0x1015, 0x103a, 0x101b,
0x102c, 0x1006, 0x102d, 0x102f, 0x1016, 0x102e, 0x1021, 0x102c, 0x1005, 0x1010, 0x1031, 0x102c,
0x1037, 0x101f, 0x102f, 0x1019, 0x103a, 0x1038, 0x1011, 0x102c, 0x1038, 0x101c, 0x1004, 0x103a,
0x1010, 0x102e, 0x101b, 0x102c, 0x1014, 0x102e, 0x1021, 0x1030, 0x101c, 0x102e, 0x101a, 0x102c,
0x1014, 0x102d, 0x102f, 0x1005, 0x1000, 0x1005, 0x103a, 0x1016, 0x103a, 0x1017, 0x102c, 0x1012,
0x1030, 0x1038, 0x1005, 0x103a, 0x1017, 0x102c, 0x1010, 0x102e, 0x1000, 0x1014, 0x103a, 0x1017,
0x102e, 0x101a, 0x1004, 0x103a, 0x1014, 0x102c, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a, 0x1014,
0x102e, 0x1021, 0x102d, 0x102f, 0x1038, 0x1005, 0x103a, 0x1017, 0x102d, 0x102f, 0x101c, 0x103a,
0x1002, 0x102d, 0x102f, 0x1002, 0x101b, 0x1000, 0x103a, 0x101d, 0x102b, 0x1006, 0x1031, 0x102c,
0x1007, 0x102c, 0x1002, 0x103a, 0x1002, 0x101b, 0x1000, 0x103a, 0x1018, 0x103a, 0x1007, 0x1030,
0x1038, 0x101b, 0x1005, 0x103a, 0x1001, 0x103b, 0x103a, 0x1021, 0x1014, 0x103a, 0x1010, 0x102c,
0x1014, 0x102c, 0x1014, 0x102c, 0x101b, 0x102e, 0x1038, 0x1018, 0x102d, 0x102f, 0x1001, 0x103b,
0x102c, 0x1002, 0x102d, 0x102f, 0x1005, 0x103a, 0x1001, 0x101b, 0x1005, 0x103a, 0x1005, 0x1019,
0x1010, 0x103a, 0x1000, 0x102d, 0x102f, 0x1000, 0x102d, 0x102f, 0x1038, 0x1005, 0x103a, 0x1000,
0x102d, 0x102f, 0x1019, 0x102d, 0x102f, 0x101b, 0x102d, 0x102f, 0x1001, 0x102b, 0x1002, 0x102b,
0x101c, 0x1014, 0x103a, 0x1019, 0x102c, 0x101f, 0x102e, 0x1019, 0x1031, 0x102c, 0x103a, 0x101c,
0x1012, 0x102d, 0x102f, 0x1000, 0x103a, 0x1019, 0x1031, 0x102c, 0x101b, 0x1005, 0x103a, 0x101b,
0x103e, 0x1019, 0x102c, 0x101a, 0x102d, 0x102f, 0x1010, 0x1032, 0x101b, 0x102e, 0x101a, 0x1030,
0x1014, 0x102e, 0x101a, 0x1014, 0x103a, 0x1021, 0x102c, 0x1038, 0x1015, 0x102e, 0x1021, 0x102c,
0x1038, 0x1021, 0x1031, 0x102c, 0x1037, 0x1000, 0x101c, 0x1014, 0x103a, 0x1018, 0x1030, 0x1002,
0x1014, 0x103a, 0x1017, 0x102e, 0x1038, 0x101c, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x1001,
0x103b, 0x102c, 0x1038, 0x101e, 0x1019, 0x103a, 0x1001, 0x103b, 0x102f, 0x1001, 0x103a, 0x1021,
0x102e, 0x1005, 0x1010, 0x102c, 0x1021, 0x102e, 0x1016, 0x102c, 0x1010, 0x102e, 0x1016, 0x102c,
0x1001, 0x102b, 0x1021, 0x102d, 0x102f, 0x1016, 0x102d, 0x102f, 0x1016, 0x102e, 0x1002, 0x103b,
0x102e, 0x1016, 0x1030, 0x1014, 0x102c, 0x1016, 0x1030, 0x1010, 0x102e, 0x1002, 0x102b, 0x101c,
0x102c, 0x1015, 0x102b, 0x1000, 0x1015, 0x103a, 0x1005, 0x103a, 0x1002, 0x1019, 0x103a, 0x1018,
0x102e, 0x101a, 0x102c, 0x1002, 0x103d, 0x102b, 0x1012, 0x102b, 0x1000, 0x1014, 0x1032, 0x101c,
0x103a, 0x1002, 0x1030, 0x1021, 0x1019, 0x103a, 0x101f, 0x102d, 0x102f, 0x1014, 0x102d, 0x102f,
0x101c, 0x1030, 0x101c, 0x1030, 0x1021, 0x1014, 0x103a, 0x1012, 0x102b, 0x1018, 0x1030, 0x101b,
0x102e, 0x1001, 0x101b, 0x102d, 0x1012, 0x102e, 0x1019, 0x1010, 0x102e, 0x1001, 0x102d, 0x102f,
0x1005, 0x103a, 0x101b, 0x102d, 0x102f, 0x1004, 0x103a, 0x1001, 0x103d, 0x102c, 0x1002, 0x103b,
0x102c, 0x101c, 0x102d, 0x1014, 0x103a, 0x1019, 0x102c, 0x1002, 0x103b, 0x1030, 0x1038, 0x101b,
0x102d, 0x102f, 0x1019, 0x102c, 0x1001, 0x1031, 0x1038, 0x1021, 0x1015, 0x103a, 0x1005, 0x103a,
0x1019, 0x1005, 0x103a, 0x1012, 0x103a, 0x101d, 0x1031, 0x1038, 0x1014, 0x102c, 0x1025, 0x1030,
0x101b, 0x1030, 0x1014, 0x102e, 0x1026, 0x1038, 0x1021, 0x1031, 0x1014, 0x1031, 0x102c, 0x103a,
0x1016, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x1014, 0x1030, 0x1019, 0x101a, 0x103a, 0x1021,
0x102c, 0x1015, 0x102b, 0x1002, 0x102d, 0x102f, 0x1015, 0x102b, 0x1002, 0x102d, 0x102f, 0x1015,
0x101c, 0x102c, 0x1021, 0x102d, 0x102f, 0x1015, 0x102b, 0x1010, 0x103a, 0x1000, 0x101a, 0x103a,
0x101b, 0x1004, 0x103a, 0x1016, 0x102d, 0x102f, 0x1014, 0x102c, 0x1016, 0x1032, 0x1021, 0x103a,
0x1016, 0x102d, 0x102f, 0x1037, 0x1010, 0x103a, 0x20, 0x1019, 0x102d, 0x102f, 0x101b, 0x1000,
0x103a, 0x1005, 0x103a, 0x1018, 0x102e, 0x101b, 0x102c, 0x101b, 0x102d, 0x102f, 0x1010, 0x103d,
0x1014, 0x103a, 0x1038, 0x1002, 0x102b, 0x1006, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1014,
0x103a, 0x1010, 0x101f, 0x102e, 0x1010, 0x102e, 0x1010, 0x102c, 0x101b, 0x102c, 0x101d, 0x102b,
0x1010, 0x103d, 0x1014, 0x103a, 0x1002, 0x102b, 0x1010, 0x102c, 0x1015, 0x102f, 0x101d, 0x102d,
0x1010, 0x103a, 0x1001, 0x103a, 0x101d, 0x1031, 0x102b, 0x101c, 0x102e, 0x1005, 0x103a, 0x963f,
0x6bd4, 0x8b93, 0x963f, 0x514b, 0x62c9, 0x963f, 0x8fea, 0x65af, 0x963f, 0x8c9d, 0x5df4, 0x963f,
0x723e, 0x53ca, 0x723e, 0x963f, 0x65af, 0x746a, 0x62c9, 0x5df4, 0x99ac, 0x79d1, 0x73ed, 0x5409,
0x73ed, 0x7af9, 0x6bd4, 0x7d39, 0x5e03, 0x862d, 0x592a, 0x723e, 0x5e03, 0x62c9, 0x67f4, 0x7dad,
0x723e, 0x5e03, 0x677e, 0x5e03, 0x62c9, 0x958b, 0x7f85, 0x5361, 0x85a9, 0x5e03, 0x862d, 0x5361,
0x4f11, 0x9054, 0x67ef, 0x90a3, 0x514b, 0x91cc, 0x9054, 0x5580, 0x723e, 0x6c99, 0x862d, 0x6e2f,
0x5409, 0x5e03, 0x5730, 0x675c, 0x963f, 0x62c9, 0x963f, 0x5c24, 0x6069, 0x81ea, 0x7531, 0x57ce,
0x5609, 0x67cf, 0x9686, 0x91cc, 0x54c8, 0x62c9, 0x96f7, 0x7d04, 0x7ff0, 0x5c3c, 0x65af, 0x5821,
0x6731, 0x5df4, 0x574e, 0x5e15, 0x62c9, 0x5580, 0x571f, 0x7a46, 0x57fa, 0x52a0, 0x5229, 0x91d1,
0x590f, 0x6c99, 0x62c9, 0x54e5, 0x65af, 0x81ea, 0x7531, 0x5e02, 0x6d1b, 0x7f8e, 0x7f85, 0x5b89,
0x9054, 0x76e7, 0x672c, 0x5df4, 0x5e0c, 0x8def, 0x6c99, 0x5361, 0x99ac, 0x62c9, 0x535a, 0x99ac,
0x666e, 0x6258, 0x99ac, 0x8cfd, 0x9b6f, 0x58a8, 0x5df4, 0x672c, 0x6469, 0x52a0, 0x8fea, 0x4f11,
0x8499, 0x7f85, 0x7dad, 0x4e9e, 0x5948, 0x6d1b, 0x6bd4, 0x6069, 0x8cc8, 0x6885, 0x7d0d, 0x5c3c,
0x4e9e, 0x7f8e, 0x8afe, 0x514b, 0x5c11, 0x74e6, 0x52a0, 0x675c, 0x53e4, 0x6ce2, 0x591a, 0x8afe,
0x4f5b, 0x8056, 0x591a, 0x7f8e, 0x7684, 0x9ece, 0x6ce2, 0x91cc, 0x7a81, 0x5c3c, 0x65af, 0x6eab,
0x5f97, 0x548c, 0x514b, 0x827e, 0x9054, 0x514b, 0x5b89, 0x514b, 0x62c9, 0x6cbb, 0x5b89, 0x5409,
0x62c9, 0x5b89, 0x5730, 0x5361, 0x963f, 0x62c9, 0x572d, 0x90a3, 0x5e03, 0x5b9c, 0x8afe, 0x65af,
0x827e, 0x5229, 0x65af, 0x5361, 0x5854, 0x99ac, 0x5361, 0x54e5, 0x591a, 0x83ef, 0x80e1, 0x80e1,
0x4f0a, 0x62c9, 0x7565, 0x54c8, 0x9580, 0x591a, 0x85a9, 0x91cc, 0x5967, 0x52a0, 0x8036, 0x6208,
0x65af, 0x85a9, 0x723e, 0x5854, 0x8056, 0x80e1, 0x5b89, 0x8056, 0x8def, 0x6613, 0x5410, 0x5eab,
0x66fc, 0x70cf, 0x65af, 0x61f7, 0x4e9e, 0x963f, 0x8def, 0x5df4, 0x4e9e, 0x677e, 0x68ee, 0x963f,
0x8482, 0x79d1, 0x80af, 0x5df4, 0x4f0a, 0x963f, 0x5df4, 0x4f0a, 0x4e9e, 0x73ed, 0x5fb7, 0x62c9,
0x65af, 0x5df4, 0x8c9d, 0x591a, 0x8c9d, 0x502b, 0x8c9d, 0x91cc, 0x65af, 0x767d, 0x6717, 0x85a9,
0x5e03, 0x9686, 0x4fdd, 0x7dad, 0x65af, 0x5854, 0x6ce2, 0x54e5, 0x5927, 0x6ce2, 0x5915, 0x528d,
0x6a4b, 0x7063, 0x683c, 0x862d, 0x5834, 0x574e, 0x6606, 0x5361, 0x62c9, 0x5361, 0x65af, 0x958b,
0x96f2, 0x958b, 0x66fc, 0x7fa4, 0x5cf6, 0x829d, 0x52a0, 0x54e5, 0x5947, 0x83ef, 0x83ef, 0x83ef,
0x96f7, 0x65af, 0x57ce, 0x54e5, 0x65af, 0x5927, 0x9ece, 0x52a0, 0x514b, 0x96f7, 0x65af, 0x9813,
0x53e4, 0x96c5, 0x5df4, 0x5eab, 0x62c9, 0x7d22, 0x4e39, 0x99ac, 0x6c99, 0x6587, 0x9053, 0x68ee,
0x9053, 0x68ee, 0x514b, 0x91cc, 0x514b, 0x4e39, 0x4f5b, 0x5e95, 0x7279, 0x5f8b, 0x591a, 0x660e,
0x5c3c, 0x52a0, 0x827e, 0x5fb7, 0x8499, 0x541e, 0x827e, 0x9b6f, 0x5167, 0x4f69, 0x85a9, 0x723e,
0x74e6, 0x591a, 0x7d0d, 0x723e, 0x905c, 0x5821, 0x798f, 0x5854, 0x529b, 0x838e, 0x683c, 0x96f7,
0x65af, 0x8c9d, 0x9d5d, 0x7063, 0x5927, 0x7279, 0x514b, 0x5cf6, 0x683c, 0x745e, 0x7d0d, 0x9054,
0x74dc, 0x5730, 0x6d1b, 0x666e, 0x74dc, 0x5730, 0x99ac, 0x62c9, 0x74dc, 0x4e9e, 0x57fa, 0x723e,
0x84cb, 0x4e9e, 0x90a3, 0x54c8, 0x91cc, 0x6cd5, 0x514b, 0x65af, 0x54c8, 0x74e6, 0x90a3, 0x57c3,
0x83ab, 0x897f, 0x7d04, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x6ce2, 0x91cc, 0x65af, 0x5370, 0x7b2c,
0x5b89, 0x90a3, 0x5dde, 0x8afe, 0x514b, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x99ac,
0x502b, 0x54e5, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x5f7c, 0x5f97, 0x5821, 0x5370, 0x7b2c,
0x5b89, 0x90a3, 0x5dde, 0x6cf0, 0x723e, 0x57ce, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x7dad,
0x5a01, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x6eab, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89,
0x90a3, 0x5dde, 0x5a01, 0x7d0d, 0x99ac, 0x514b, 0x4f0a, 0x5974, 0x7dad, 0x514b, 0x4f0a, 0x9b41,
0x7279, 0x7259, 0x8cb7, 0x52a0, 0x6731, 0x8afe, 0x8def, 0x6613, 0x65af, 0x7dad, 0x723e, 0x80af,
0x5854, 0x57fa, 0x5dde, 0x8499, 0x5730, 0x537b, 0x7f85, 0x514b, 0x62c9, 0x502b, 0x4ee3, 0x514b,
0x62c9, 0x5df4, 0x65af, 0x5229, 0x99ac, 0x6d1b, 0x6749, 0x78ef, 0x4e0b, 0x592a, 0x5b50, 0x5340,
0x99ac, 0x745f, 0x6b50, 0x99ac, 0x62ff, 0x74dc, 0x746a, 0x7459, 0x65af, 0x99ac, 0x91cc, 0x6208,
0x7279, 0x99ac, 0x4e01, 0x5c3c, 0x514b, 0x99ac, 0x5854, 0x83ab, 0x7f85, 0x65af, 0x99ac, 0x85a9,
0x7279, 0x862d, 0x7f8e, 0x8afe, 0x7c73, 0x5c3c, 0x6885, 0x91cc, 0x9054, 0x6885, 0x7279, 0x62c9,
0x5361, 0x7279, 0x62c9, 0x58a8, 0x897f, 0x54e5, 0x5e02, 0x5bc6, 0x555f, 0x5d19, 0x8499, 0x514b,
0x9813, 0x8499, 0x7279, 0x745e, 0x8499, 0x7279, 0x7dad, 0x591a, 0x8499, 0x54f2, 0x81d8, 0x62ff,
0x7d22, 0x7d10, 0x7d04, 0x8afe, 0x59c6, 0x8afe, 0x502b, 0x54c8, 0x5317, 0x9054, 0x79d1, 0x4ed6,
0x5dde, 0x5e03, 0x7531, 0x62c9, 0x5317, 0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x4e2d, 0x5fc3, 0x5317,
0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x7d10, 0x6c99, 0x502b, 0x52aa, 0x514b, 0x5967, 0x5e0c, 0x7d0d,
0x52a0, 0x5df4, 0x62ff, 0x99ac, 0x5df4, 0x62c9, 0x99ac, 0x5229, 0x6ce2, 0x9cf3, 0x51f0, 0x57ce,
0x592a, 0x5b50, 0x6e2f, 0x897f, 0x73ed, 0x7259, 0x6e2f, 0x7dad, 0x7559, 0x6e2f, 0x6ce2, 0x591a,
0x9ece, 0x5404, 0x84ec, 0x5854, 0x963f, 0x96f7, 0x7d0d, 0x65af, 0x862d, 0x4eca, 0x7063, 0x96f7,
0x897f, 0x975e, 0x91cc, 0x8cc8, 0x7d0d, 0x7f85, 0x65af, 0x9b6f, 0x7279, 0x91cc, 0x7d04, 0x5e03,
0x862d, 0x8056, 0x5854, 0x502b, 0x8056, 0x5730, 0x7259, 0x54e5, 0x8056, 0x591a, 0x660e, 0x54e5,
0x8056, 0x4fdd, 0x7f85, 0x4f0a, 0x6258, 0x79d1, 0x723e, 0x6258, 0x7c73, 0x7279, 0x932b, 0x7279,
0x5361, 0x8056, 0x5df4, 0x6258, 0x6d1b, 0x7e46, 0x5cf6, 0x8056, 0x7d04, 0x7ff0, 0x8056, 0x57fa,
0x8328, 0x8056, 0x9732, 0x897f, 0x4e9e, 0x8056, 0x6258, 0x99ac, 0x65af, 0x8056, 0x6587, 0x68ee,
0x65af, 0x5a01, 0x592b, 0x7279, 0x5361, 0x502b, 0x7279, 0x5fb7, 0x53e4, 0x65af, 0x52a0, 0x5df4,
0x675c, 0x91cc, 0x63d0, 0x83ef, 0x7d0d, 0x591a, 0x502b, 0x591a, 0x6258, 0x723e, 0x6258, 0x62c9,
0x6eab, 0x54e5, 0x83ef, 0x61f7, 0x7279, 0x970d, 0x65af, 0x6eab, 0x5c3c, 0x4f2f, 0x96c5, 0x5eab,
0x5854, 0x51f1, 0x897f, 0x6234, 0x7dad, 0x65af, 0x675c, 0x8499, 0x675c, 0x6bd4, 0x723e, 0x9ea5,
0x89ba, 0x7406, 0x83ab, 0x68ee, 0x9ea5, 0x514b, 0x9ed8, 0x591a, 0x5e15, 0x9ea5, 0x7f85, 0x745f,
0x62c9, 0x662d, 0x548c, 0x57fa, 0x5730, 0x7dbd, 0x723e, 0x6c83, 0x65af, 0x6258, 0x514b, 0x9686,
0x610f, 0x8033, 0x62dc, 0x6069, 0x4e9e, 0x4e01, 0x963f, 0x62c9, 0x6728, 0x5716, 0x5b89, 0x66fc,
0x963f, 0x90a3, 0x5e95, 0x963f, 0x514b, 0x5957, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x963f, 0x4ec0,
0x54c8, 0x5df4, 0x7279, 0x963f, 0x7279, 0x52de, 0x5df4, 0x683c, 0x9054, 0x5df4, 0x6797, 0x66fc,
0x8c37, 0x5df4, 0x723e, 0x7459, 0x723e, 0x8c9d, 0x9b6f, 0x7279, 0x6bd4, 0x4ec0, 0x51f1, 0x514b,
0x6c76, 0x840a, 0x8d64, 0x5854, 0x53ef, 0x502b, 0x5761, 0x5927, 0x99ac, 0x58eb, 0x9769, 0x9054,
0x5361, 0x5e1d, 0x529b, 0x675c, 0x62dc, 0x675c, 0x6851, 0x8c9d, 0x6cd5, 0x99ac, 0x53e4, 0x65af,
0x5854, 0x52a0, 0x85a9, 0x8d6b, 0x5e03, 0x9686, 0x80e1, 0x5fd7, 0x660e, 0x5e02, 0x4e2d, 0x83ef,
0x4eba, 0x6c11, 0x5171, 0x548c, 0x570b, 0x9999, 0x6e2f, 0x7279, 0x5225, 0x884c, 0x653f, 0x5340,
0x79d1, 0x5e03, 0x591a, 0x4f0a, 0x723e, 0x5eab, 0x6b21, 0x514b, 0x96c5, 0x52a0, 0x9054, 0x52a0,
0x4e9e, 0x5e03, 0x62c9, 0x8036, 0x8def, 0x6492, 0x51b7, 0x5580, 0x5e03, 0x723e, 0x582a, 0x5bdf,
0x52a0, 0x5580, 0x62c9, 0x86a9, 0x52a0, 0x5fb7, 0x6eff, 0x90fd, 0x582a, 0x5730, 0x52a0, 0x52a0,
0x723e, 0x5404, 0x7b54, 0x514b, 0x62c9, 0x65af, 0x8afe, 0x4e9e, 0x723e, 0x65af, 0x514b, 0x5409,
0x9686, 0x5761, 0x53e4, 0x6649, 0x79d1, 0x5a01, 0x7279, 0x4e2d, 0x83ef, 0x4eba, 0x6c11, 0x5171,
0x548c, 0x570b, 0x6fb3, 0x9580, 0x7279, 0x5225, 0x884c, 0x653f, 0x5340, 0x99ac, 0x52a0, 0x4e39,
0x99ac, 0x5361, 0x6c99, 0x723e, 0x99ac, 0x5c3c, 0x62c9, 0x99ac, 0x65af, 0x958b, 0x7279, 0x5c3c,
0x53e4, 0x897f, 0x4e9e, 0x65b0, 0x5eab, 0x8332, 0x6d85, 0x8328, 0x514b, 0x65b0, 0x897f, 0x4f2f,
0x5229, 0x4e9e, 0x9102, 0x6728, 0x65af, 0x514b, 0x70cf, 0x62c9, 0x723e, 0x91d1, 0x908a, 0x5764,
0x7538, 0x5e73, 0x58e4, 0x5361, 0x9054, 0x79d1, 0x65af, 0x5854, 0x5948, 0x514b, 0x5b5c, 0x52d2,
0x5967, 0x723e, 0x9054, 0x5229, 0x96c5, 0x5fb7, 0x5eab, 0x9801, 0x5cf6, 0x6492, 0x99ac, 0x723e,
0x7f55, 0x9996, 0x723e, 0x4e0a, 0x6d77, 0x65b0, 0x52a0, 0x5761, 0x4e2d, 0x79d1, 0x96f7, 0x59c6,
0x65af, 0x514b, 0x53f0, 0x5317, 0x5854, 0x4ec0, 0x5e72, 0x7b2c, 0x6bd4, 0x5229, 0x65af, 0x5fb7,
0x9ed1, 0x862d, 0x5ef7, 0x5e03, 0x6771, 0x4eac, 0x6258, 0x6728, 0x65af, 0x514b, 0x70cf, 0x862d,
0x5df4, 0x6258, 0x70cf, 0x9b6f, 0x6728, 0x9f4a, 0x70cf, 0x65af, 0x5167, 0x62c9, 0x6c38, 0x73cd,
0x6d77, 0x53c3, 0x5d34, 0x96c5, 0x5eab, 0x6b21, 0x514b, 0x4ef0, 0x5149, 0x8449, 0x5361, 0x6377,
0x6797, 0x5821, 0x8449, 0x91cc, 0x6eab, 0x4e9e, 0x901f, 0x723e, 0x7fa4, 0x5cf6, 0x767e, 0x6155,
0x9054, 0x52a0, 0x7d0d, 0x5229, 0x7dad, 0x5fb7, 0x89d2, 0x6cd5, 0x7f85, 0x7fa4, 0x5cf6, 0x99ac,
0x5f97, 0x62c9, 0x7fa4, 0x5cf6, 0x96f7, 0x514b, 0x96c5, 0x7dad, 0x514b, 0x5357, 0x55ac, 0x6cbb,
0x4e9e, 0x8056, 0x8d6b, 0x52d2, 0x62ff, 0x5cf6, 0x53f2, 0x5766, 0x5229, 0x963f, 0x5f97, 0x96f7,
0x5fb7, 0x5e03, 0x5229, 0x65af, 0x73ed, 0x5e03, 0x7f85, 0x80af, 0x5e0c, 0x723e, 0x9054, 0x723e,
0x6587, 0x5c24, 0x514b, 0x62c9, 0x8377, 0x5df4, 0x7279, 0x6797, 0x5fb7, 0x66fc, 0x8c6a, 0x52f3,
0x7235, 0x5cf6, 0x58a8, 0x723e, 0x672c, 0x4f2f, 0x65af, 0x96ea, 0x68a8, 0x963f, 0x59c6, 0x65af,
0x7279, 0x4e39, 0x5b89, 0x9053, 0x723e, 0x963f, 0x65af, 0x7279, 0x62c9, 0x7f55, 0x96c5, 0x5178,
0x8c9d, 0x723e, 0x683c, 0x52d2, 0x67cf, 0x6797, 0x5e03, 0x62c9, 0x63d0, 0x65af, 0x62c9, 0x74e6,
0x5e03, 0x9b6f, 0x585e, 0x723e, 0x5e03, 0x52a0, 0x52d2, 0x65af, 0x7279, 0x5e03, 0x9054, 0x4f69,
0x65af, 0x5e03, 0x8f9b, 0x6839, 0x5947, 0x897f, 0x7459, 0x54e5, 0x672c, 0x54c8, 0x6839, 0x90fd,
0x67cf, 0x6797, 0x76f4, 0x5e03, 0x7f85, 0x9640, 0x6839, 0x606f, 0x5cf6, 0x8d6b, 0x723e, 0x8f9b,
0x57fa, 0x66fc, 0x5cf6, 0x4f0a, 0x65af, 0x5766, 0x5821, 0x6fa4, 0x897f, 0x5cf6, 0x52a0, 0x91cc,
0x5be7, 0x683c, 0x52d2, 0x57fa, 0x6d1b, 0x592b, 0x57fa, 0x8f14, 0x91cc, 0x65af, 0x672c, 0x76e7,
0x6bd4, 0x5b89, 0x7d0d, 0x502b, 0x6566, 0x76e7, 0x68ee, 0x5821, 0x99ac, 0x5fb7, 0x91cc, 0x99ac,
0x723e, 0x4ed6, 0x746a, 0x9e97, 0x6e2f, 0x660e, 0x65af, 0x514b, 0x6469, 0x7d0d, 0x54e5, 0x83ab,
0x65af, 0x79d1, 0x5967, 0x65af, 0x9678, 0x5df4, 0x9ece, 0x6ce2, 0x591a, 0x91cc, 0x5bdf, 0x5e03,
0x62c9, 0x683c, 0x91cc, 0x52a0, 0x7f85, 0x99ac, 0x6c99, 0x99ac, 0x62c9, 0x8056, 0x99ac, 0x5229,
0x8afe, 0x585e, 0x62c9, 0x8036, 0x4f5b, 0x85a9, 0x62c9, 0x6258, 0x592b, 0x8f9b, 0x975e, 0x6d1b,
0x6d66, 0x53f2, 0x9ad8, 0x6bd4, 0x8036, 0x7d22, 0x83f2, 0x4e9e, 0x65af, 0x5fb7, 0x54e5, 0x723e,
0x6469, 0x5854, 0x6797, 0x5730, 0x62c9, 0x90a3, 0x70cf, 0x91cc, 0x63da, 0x8afe, 0x592b, 0x65af,
0x514b, 0x74e6, 0x90fd, 0x8332, 0x68b5, 0x8482, 0x5ca1, 0x7dad, 0x4e5f, 0x7d0d, 0x7dad, 0x723e,
0x7d10, 0x65af, 0x4f0f, 0x723e, 0x52a0, 0x683c, 0x52d2, 0x83ef, 0x6c99, 0x672d, 0x683c, 0x745e,
0x5e03, 0x8607, 0x9ece, 0x4e16, 0x5b89, 0x5854, 0x90a3, 0x90a3, 0x5229, 0x4f5b, 0x67e5, 0x6208,
0x65af, 0x8056, 0x8a95, 0x5cf6, 0x79d1, 0x79d1, 0x65af, 0x7fa4, 0x5cf6, 0x79d1, 0x6469, 0x7f85,
0x7fa4, 0x5cf6, 0x51f1, 0x723e, 0x84cb, 0x6717, 0x5cf6, 0x99ac, 0x57c3, 0x5cf6, 0x99ac, 0x723e,
0x5730, 0x592b, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x99ac, 0x7d04, 0x7279, 0x5cf6, 0x7559, 0x5c3c,
0x65fa, 0x5cf6, 0x963f, 0x76ae, 0x4e9e, 0x5967, 0x514b, 0x862d, 0x5e03, 0x5e72, 0x7dad, 0x723e,
0x67e5, 0x5766, 0x695a, 0x514b, 0x5fa9, 0x6d3b, 0x5cf6, 0x57c3, 0x6cd5, 0x7279, 0x6cd5, 0x8003,
0x798f, 0x6590, 0x6fdf, 0x5bcc, 0x90a3, 0x5bcc, 0x63d0, 0x52a0, 0x62c9, 0x5df4, 0x54e5, 0x7fa4,
0x5cf6, 0x7518, 0x6bd4, 0x723e, 0x7fa4, 0x5cf6, 0x74dc, 0x9054, 0x5eb7, 0x7d0d, 0x723e, 0x5cf6,
0x95dc, 0x5cf6, 0x6a80, 0x9999, 0x5c71, 0x6069, 0x5f97, 0x4f2f, 0x7406, 0x5cf6, 0x57fa, 0x91cc,
0x5730, 0x99ac, 0x5730, 0x5cf6, 0x79d1, 0x65af, 0x745e, 0x74dc, 0x52a0, 0x6797, 0x5cf6, 0x99ac,
0x6731, 0x8afe, 0x99ac, 0x53ef, 0x85a9, 0x65af, 0x5cf6, 0x4e2d, 0x9014, 0x5cf6, 0x8afe, 0x9b6f,
0x7d10, 0x57c3, 0x5cf6, 0x8afe, 0x798f, 0x514b, 0x8afe, 0x7f8e, 0x4e9e, 0x5df4, 0x54e5, 0x5df4,
0x54e5, 0x5e1b, 0x7409, 0x76ae, 0x7279, 0x80af, 0x7fa4, 0x5cf6, 0x6ce2, 0x7d0d, 0x4f69, 0x83ab,
0x58eb, 0x6bd4, 0x6e2f, 0x62c9, 0x7f85, 0x6e6f, 0x52a0, 0x585e, 0x73ed, 0x5927, 0x6eaa, 0x5730,
0x5854, 0x62c9, 0x74e6, 0x6771, 0x52a0, 0x5854, 0x5e03, 0x5cf6, 0x5a01, 0x514b, 0x74e6, 0x5229,
0x65af, 0x963f, 0x6bd4, 0x8ba9, 0x963f, 0x8fea, 0x65af, 0x963f, 0x8d1d, 0x5df4, 0x963f, 0x5c14,
0x53ca, 0x5c14, 0x963f, 0x65af, 0x739b, 0x62c9, 0x5df4, 0x9a6c, 0x79d1, 0x6bd4, 0x7ecd, 0x5e03,
0x5170, 0x592a, 0x5c14, 0x5e03, 0x62c9, 0x67f4, 0x7ef4, 0x5c14, 0x5f00, 0x7f57, 0x5361, 0x8428,
0x5e03, 0x5170, 0x5361, 0x4f11, 0x8fbe, 0x8fbe, 0x5580, 0x5c14, 0x6c99, 0x5170, 0x6e2f, 0x7ea6,
0x7ff0, 0x5c3c, 0x65af, 0x5821, 0x7f57, 0x5b89, 0x8fbe, 0x5362, 0x672c, 0x5df4, 0x5e0c, 0x9a6c,
0x62c9, 0x535a, 0x9a6c, 0x666e, 0x6258, 0x9a6c, 0x8d5b, 0x9c81, 0x8499, 0x7f57, 0x7ef4, 0x4e9a,
0x6069, 0x8d3e, 0x6885, 0x7eb3, 0x5c3c, 0x4e9a, 0x7f8e, 0x8bfa, 0x514b, 0x5c11, 0x6ce2, 0x591a,
0x8bfa, 0x4f5b, 0x5723, 0x591a, 0x7f8e, 0x6e29, 0x5f97, 0x548c, 0x514b, 0x827e, 0x8fbe, 0x514b,
0x5e03, 0x5b9c, 0x8bfa, 0x65af, 0x827e, 0x5229, 0x65af, 0x5361, 0x5854, 0x9a6c, 0x5361, 0x54e5,
0x591a, 0x534e, 0x95e8, 0x591a, 0x8428, 0x91cc, 0x5965, 0x52a0, 0x8036, 0x6208, 0x65af, 0x8428,
0x5c14, 0x5854, 0x5723, 0x80e1, 0x5b89, 0x5723, 0x8def, 0x6613, 0x5410, 0x5e93, 0x66fc, 0x4e4c,
0x65af, 0x6000, 0x4e9a, 0x4e9a, 0x677e, 0x68ee, 0x5df4, 0x4f0a, 0x4e9a, 0x73ed, 0x5fb7, 0x62c9,
0x65af, 0x5df4, 0x8d1d, 0x591a, 0x8d1d, 0x4f26, 0x8d1d, 0x91cc, 0x65af, 0x767d, 0x6717, 0x8428,
0x5e03, 0x9686, 0x4fdd, 0x7ef4, 0x65af, 0x5854, 0x5251, 0x6865, 0x6e7e, 0x683c, 0x5170, 0x573a,
0x5f00, 0x4e91, 0x5f00, 0x66fc, 0x7fa4, 0x5c9b, 0x5947, 0x534e, 0x534e, 0x534e, 0x96f7, 0x65af,
0x57ce, 0x514b, 0x96f7, 0x65af, 0x987f, 0x5e93, 0x62c9, 0x7d22, 0x4e39, 0x9a6c, 0x6c99, 0x6587,
0x827e, 0x9c81, 0x5185, 0x4f69, 0x8428, 0x5c14, 0x74e6, 0x591a, 0x7eb3, 0x5c14, 0x900a, 0x5821,
0x683c, 0x96f7, 0x65af, 0x8d1d, 0x9e45, 0x6e7e, 0x5927, 0x7279, 0x514b, 0x5c9b, 0x683c, 0x745e,
0x7eb3, 0x8fbe, 0x74dc, 0x5730, 0x9a6c, 0x62c9, 0x74dc, 0x4e9a, 0x57fa, 0x5c14, 0x76d6, 0x4e9a,
0x90a3, 0x57c3, 0x83ab, 0x897f, 0x7ea6, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x8bfa, 0x514b,
0x65af, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x9a6c, 0x4f26, 0x54e5, 0x5370, 0x7b2c, 0x5b89,
0x90a3, 0x5dde, 0x6cf0, 0x5c14, 0x57ce, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x7ef4, 0x5a01,
0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x6e29, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x90a3,
0x5dde, 0x5a01, 0x7eb3, 0x9a6c, 0x514b, 0x4f0a, 0x5974, 0x7ef4, 0x514b, 0x7259, 0x4e70, 0x52a0,
0x6731, 0x8bfa, 0x8def, 0x6613, 0x65af, 0x7ef4, 0x5c14, 0x80af, 0x5854, 0x57fa, 0x5dde, 0x8499,
0x5730, 0x5374, 0x7f57, 0x514b, 0x62c9, 0x4f26, 0x4ee3, 0x514b, 0x5229, 0x9a6c, 0x6d1b, 0x6749,
0x77f6, 0x4e0b, 0x592a, 0x5b50, 0x533a, 0x9a6c, 0x745f, 0x6b27, 0x9a6c, 0x62ff, 0x74dc, 0x739b,
0x7459, 0x65af, 0x9a6c, 0x91cc, 0x6208, 0x7279, 0x9a6c, 0x4e01, 0x5c3c, 0x514b, 0x9a6c, 0x5854,
0x83ab, 0x7f57, 0x65af, 0x9a6c, 0x8428, 0x7279, 0x5170, 0x7f8e, 0x8bfa, 0x7c73, 0x5c3c, 0x6885,
0x91cc, 0x8fbe, 0x5bc6, 0x542f, 0x4ed1, 0x8499, 0x514b, 0x987f, 0x8499, 0x7279, 0x7ef4, 0x591a,
0x8499, 0x54f2, 0x814a, 0x7ebd, 0x7ea6, 0x8bfa, 0x59c6, 0x8bfa, 0x4f26, 0x54c8, 0x5317, 0x8fbe,
0x79d1, 0x4ed6, 0x5dde, 0x5e03, 0x7531, 0x62c9, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde, 0x4e2d,
0x5fc3, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde, 0x7ebd, 0x6c99, 0x4f26, 0x5965, 0x5e0c, 0x7eb3,
0x52a0, 0x5df4, 0x62ff, 0x9a6c, 0x5df4, 0x62c9, 0x9a6c, 0x5229, 0x6ce2, 0x51e4, 0x51f0, 0x57ce,
0x7ef4, 0x7559, 0x6e2f, 0x84ec, 0x5854, 0x963f, 0x96f7, 0x7eb3, 0x65af, 0x5170, 0x4eca, 0x6e7e,
0x91cc, 0x8d3e, 0x7eb3, 0x7f57, 0x65af, 0x9c81, 0x7279, 0x91cc, 0x7ea6, 0x5e03, 0x5170, 0x5723,
0x5854, 0x4f26, 0x5723, 0x5730, 0x7259, 0x54e5, 0x5723, 0x591a, 0x660e, 0x54e5, 0x5723, 0x4fdd,
0x7f57, 0x4f0a, 0x6258, 0x79d1, 0x5c14, 0x6258, 0x7c73, 0x7279, 0x9521, 0x7279, 0x5361, 0x5723,
0x5df4, 0x6258, 0x6d1b, 0x7f2a, 0x5c9b, 0x5723, 0x7ea6, 0x7ff0, 0x5723, 0x57fa, 0x8328, 0x5723,
0x9732, 0x897f, 0x4e9a, 0x5723, 0x6258, 0x9a6c, 0x65af, 0x5723, 0x6587, 0x68ee, 0x65af, 0x5a01,
0x592b, 0x7279, 0x5361, 0x4f26, 0x7279, 0x63d0, 0x534e, 0x7eb3, 0x591a, 0x4f26, 0x591a, 0x6258,
0x5c14, 0x6258, 0x62c9, 0x6e29, 0x54e5, 0x534e, 0x6000, 0x7279, 0x970d, 0x65af, 0x6e29, 0x5c3c,
0x4f2f, 0x96c5, 0x5e93, 0x5854, 0x51ef, 0x897f, 0x6234, 0x7ef4, 0x65af, 0x675c, 0x8499, 0x675c,
0x6bd4, 0x5c14, 0x9ea6, 0x89c9, 0x7406, 0x9ea6, 0x514b, 0x9ed8, 0x591a, 0x5e15, 0x9ea6, 0x7f57,
0x745f, 0x62c9, 0x7ef0, 0x5c14, 0x4e9a, 0x4e01, 0x963f, 0x62c9, 0x6728, 0x56fe, 0x963f, 0x7279,
0x52b3, 0x5df4, 0x683c, 0x8fbe, 0x5df4, 0x5e93, 0x5df4, 0x5c14, 0x7459, 0x5c14, 0x8d1d, 0x9c81,
0x7279, 0x6bd4, 0x4ec0, 0x51ef, 0x514b, 0x6c76, 0x83b1, 0x53ef, 0x4f26, 0x5761, 0x5927, 0x9a6c,
0x58eb, 0x9769, 0x8fbe, 0x5361, 0x675c, 0x6851, 0x8d1d, 0x6cd5, 0x9a6c, 0x53e4, 0x65af, 0x5854,
0x52a0, 0x8428, 0x4e2d, 0x534e, 0x4eba, 0x6c11, 0x5171, 0x548c, 0x56fd, 0x9999, 0x6e2f, 0x7279,
0x522b, 0x884c, 0x653f, 0x533a, 0x4f0a, 0x5c14, 0x5e93, 0x6b21, 0x514b, 0x96c5, 0x52a0, 0x8fbe,
0x52a0, 0x4e9a, 0x5e03, 0x62c9, 0x5580, 0x5e03, 0x5c14, 0x52a0, 0x5fb7, 0x6ee1, 0x90fd, 0x52a0,
0x5c14, 0x5404, 0x7b54, 0x514b, 0x62c9, 0x65af, 0x8bfa, 0x4e9a, 0x5c14, 0x65af, 0x514b, 0x53e4,
0x664b, 0x4e2d, 0x534e, 0x4eba, 0x6c11, 0x5171, 0x548c, 0x56fd, 0x6fb3, 0x95e8, 0x7279, 0x522b,
0x884c, 0x653f, 0x533a, 0x9a6c, 0x52a0, 0x4e39, 0x9a6c, 0x5361, 0x6c99, 0x5c14, 0x9a6c, 0x5c3c,
0x62c9, 0x9a6c, 0x65af, 0x5f00, 0x7279, 0x5c3c, 0x53e4, 0x897f, 0x4e9a, 0x65b0, 0x5e93, 0x5179,
0x6d85, 0x8328, 0x514b, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9a, 0x4e4c, 0x62c9, 0x5c14, 0x91d1,
0x8fb9, 0x5361, 0x8fbe, 0x514b, 0x5b5c, 0x52d2, 0x5965, 0x5c14, 0x8fbe, 0x5e93, 0x9875, 0x5c9b,
0x6492, 0x9a6c, 0x5c14, 0x7f55, 0x9996, 0x5c14, 0x5fb7, 0x9ed1, 0x5170, 0x4e1c, 0x4eac, 0x4e4c,
0x5170, 0x5df4, 0x6258, 0x4e4c, 0x9c81, 0x6728, 0x9f50, 0x4e4c, 0x65af, 0x5185, 0x62c9, 0x6d77,
0x53c2, 0x5d34, 0x96c5, 0x5e93, 0x6b21, 0x514b, 0x53f6, 0x5361, 0x6377, 0x6797, 0x5821, 0x53f6,
0x91cc, 0x6e29, 0x4e9a, 0x901f, 0x5c14, 0x7fa4, 0x5c9b, 0x767e, 0x6155, 0x8fbe, 0x52a0, 0x7eb3,
0x5229, 0x7ef4, 0x5fb7, 0x89d2, 0x6cd5, 0x7f57, 0x7fa4, 0x5c9b, 0x9a6c, 0x5f97, 0x62c9, 0x7fa4,
0x5c9b, 0x96f7, 0x514b, 0x96c5, 0x7ef4, 0x514b, 0x5357, 0x4e54, 0x6cbb, 0x4e9a, 0x5723, 0x8d6b,
0x52d2, 0x62ff, 0x5c9b, 0x5e03, 0x7f57, 0x80af, 0x5e0c, 0x5c14, 0x8fbe, 0x5c14, 0x6587, 0x8c6a,
0x52cb, 0x7235, 0x5c9b, 0x58a8, 0x5c14, 0x672c, 0x5b89, 0x9053, 0x5c14, 0x8d1d, 0x5c14, 0x683c,
0x52d2, 0x5e03, 0x9c81, 0x585e, 0x5c14, 0x5e03, 0x8fbe, 0x4f69, 0x65af, 0x76f4, 0x5e03, 0x7f57,
0x9640, 0x6839, 0x606f, 0x5c9b, 0x8d6b, 0x5c14, 0x8f9b, 0x57fa, 0x66fc, 0x5c9b, 0x6cfd, 0x897f,
0x5c9b, 0x52a0, 0x91cc, 0x5b81, 0x683c, 0x52d2, 0x57fa, 0x8f85, 0x5362, 0x6bd4, 0x5b89, 0x7eb3,
0x4f26, 0x6566, 0x5362, 0x68ee, 0x5821, 0x9a6c, 0x5fb7, 0x91cc, 0x9a6c, 0x5c14, 0x4ed6, 0x739b,
0x4e3d, 0x6e2f, 0x6469, 0x7eb3, 0x54e5, 0x5965, 0x65af, 0x9646, 0x7f57, 0x9a6c, 0x6c99, 0x9a6c,
0x62c9, 0x5723, 0x9a6c, 0x5229, 0x8bfa, 0x8428, 0x62c9, 0x6258, 0x592b, 0x7d22, 0x83f2, 0x4e9a,
0x65af, 0x5fb7, 0x54e5, 0x5c14, 0x6469, 0x4e4c, 0x91cc, 0x626c, 0x8bfa, 0x592b, 0x65af, 0x514b,
0x74e6, 0x90fd, 0x5179, 0x68b5, 0x8482, 0x5188, 0x7ef4, 0x4e5f, 0x7eb3, 0x7ef4, 0x5c14, 0x7ebd,
0x65af, 0x4f0f, 0x5c14, 0x52a0, 0x683c, 0x52d2, 0x534e, 0x6c99, 0x82cf, 0x9ece, 0x4e16, 0x5723,
0x8bde, 0x5c9b, 0x79d1, 0x79d1, 0x65af, 0x7fa4, 0x5c9b, 0x79d1, 0x6469, 0x7f57, 0x7fa4, 0x5c9b,
0x51ef, 0x5c14, 0x76d6, 0x6717, 0x5c9b, 0x9a6c, 0x57c3, 0x5c9b, 0x9a6c, 0x5c14, 0x5730, 0x592b,
0x9a6c, 0x7ea6, 0x7279, 0x5c9b, 0x7559, 0x5c3c, 0x65fa, 0x5c9b, 0x963f, 0x76ae, 0x4e9a, 0x5965,
0x514b, 0x5170, 0x5e03, 0x5e72, 0x7ef4, 0x5c14, 0x590d, 0x6d3b, 0x5c9b, 0x6590, 0x6d4e, 0x52a0,
0x62c9, 0x5df4, 0x54e5, 0x7fa4, 0x5c9b, 0x7518, 0x6bd4, 0x5c14, 0x7fa4, 0x5c9b, 0x74dc, 0x8fbe,
0x5eb7, 0x7eb3, 0x5c14, 0x5c9b, 0x5173, 0x5c9b, 0x6069, 0x5f97, 0x4f2f, 0x7406, 0x5c9b, 0x57fa,
0x91cc, 0x5730, 0x9a6c, 0x5730, 0x5c9b, 0x74dc, 0x52a0, 0x6797, 0x5c9b, 0x9a6c, 0x6731, 0x8bfa,
0x9a6c, 0x53ef, 0x8428, 0x65af, 0x5c9b, 0x4e2d, 0x9014, 0x5c9b, 0x8bfa, 0x9c81, 0x7ebd, 0x57c3,
0x5c9b, 0x8bfa, 0x798f, 0x514b, 0x8bfa, 0x7f8e, 0x4e9a, 0x76ae, 0x7279, 0x80af, 0x7fa4, 0x5c9b,
0x6ce2, 0x7eb3, 0x4f69, 0x62c9, 0x7f57, 0x6c64, 0x52a0, 0x4e1c, 0x52a0, 0x5854, 0x5e03, 0x5c9b,
0x41, 0x6c, 0x67, 0x65, 0x72, 0x43, 0x61, 0x69, 0x72, 0x65, 0x2c, 0x20,
0x65, 0x6c, 0x61, 0x6c, 0x2d, 0x41, 0x61, 0x69, 0x75, 0x6e, 0x4d, 0x6f,
0x67, 0x61, 0x64, 0x69, 0x73, 0x63, 0x69, 0x6f, 0x4d, 0x6f, 0x6e, 0x72,
0xf2, 0x76, 0x69, 0x61, 0x4e, 0x2019, 0x44, 0x6a, 0x61, 0x6d, 0x65, 0x6e,
0x61, 0x52, 0xed, 0x6f, 0x20, 0x47, 0x61, 0x6c, 0x6c, 0x65, 0x67, 0x6f,
0x73, 0x42, 0x6c, 0x61, 0x6e, 0x63, 0x20, 0x53, 0x61, 0x62, 0x6c, 0x6f,
0x6e, 0x42, 0x6f, 0x67, 0x6f, 0x74, 0xe0, 0x43, 0x61, 0x69, 0x65, 0x6e,
0x61, 0x43, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x43, 0x69, 0x75, 0x74, 0x61,
0x74, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe8, 0x78, 0x69, 0x63, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x50, 0x61, 0x6e, 0x61, 0x6d,
0xe0, 0x53, 0x63, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x79, 0x73, 0x75, 0x6e,
0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61,
0x73, 0x41, 0x6e, 0xe0, 0x64, 0x69, 0x72, 0x41, 0x71, 0x74, 0x61, 0xfd,
0x41, 0x71, 0x74, 0xf3, 0x62, 0x65, 0x41, 0x15f, 0x67, 0x61, 0x62, 0x61,
0x74, 0x41, 0x74, 0x79, 0x72, 0x61, 0xfd, 0x42, 0x61, 0x72, 0x6e, 0x61,
0xfc, 0x6c, 0x54, 0x78, 0x69, 0x74, 0xe0, 0x44, 0x75, 0x69, 0x78, 0x61,
0x6e, 0x62, 0xe9, 0x48, 0x1ed3, 0x20, 0x43, 0x68, 0xed, 0x20, 0x4d, 0x69,
0x6e, 0x68, 0x4a, 0x61, 0x69, 0x70, 0x75, 0x72, 0x4e, 0x69, 0x63, 0xf2,
0x73, 0x69, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x51, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x69, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0xed, 0x6e, 0x53, 0x65, 0xfc, 0x6c, 0x54,
0x61, 0x69, 0x78, 0x6b, 0x65, 0x6e, 0x74, 0x54, 0xf2, 0x71, 0x75, 0x69,
0x6f, 0xdc, 0x72, 0xfc, 0x6d, 0x63, 0x68, 0x69, 0x55, 0x73, 0x74, 0x2019,
0x2d, 0x4e, 0x65, 0x72, 0x61, 0x49, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x49, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x42, 0x65, 0x72, 0x6d, 0x75,
0x64, 0x65, 0x73, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x61, 0x6e,
0xe0, 0x72, 0x69, 0x65, 0x73, 0x43, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72,
0x64, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0xe8, 0x72, 0x6f, 0x65,
0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x53, 0x75, 0x64, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0x61, 0x6e, 0x42,
0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0xb7, 0x6c, 0x65, 0x73, 0x43, 0x6f,
0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x75, 0x65, 0x6e, 0x48, 0xe8, 0x6c,
0x73, 0x69, 0x6e, 0x6b, 0x69, 0x4b, 0xed, 0x72, 0x6f, 0x76, 0x4b, 0xed,
0x69, 0x76, 0x4d, 0xf2, 0x6e, 0x61, 0x63, 0x6f, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x75, 0x53, 0x61, 0x72, 0xe0, 0x74, 0x6f, 0x76, 0x53, 0x69, 0x6d,
0x66, 0x65, 0x72, 0xf2, 0x70, 0x6f, 0x6c, 0x55, 0x6c, 0x69, 0xe0, 0x6e,
0x6f, 0x76, 0x73, 0x6b, 0x56, 0x61, 0x74, 0x69, 0x63, 0xe0, 0x56, 0xed,
0x6c, 0x6e, 0x69, 0x75, 0x73, 0x56, 0x61, 0x72, 0x73, 0xf2, 0x76, 0x69,
0x61, 0x5a, 0xfa, 0x72, 0x69, 0x63, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x65, 0x73, 0x54, 0x61, 0x68, 0x69, 0x74, 0xed, 0x53,
0x69, 0x79, 0x75, 0x64, 0x61, 0x64, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f,
0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0xd804, 0xdd03, 0xd804,
0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd03, 0xd804,
0xdd13, 0xd804, 0xdd33, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd1d, 0xd804, 0xdd1d, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd1d, 0xd804, 0xdd1f, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd0b, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd1a, 0xd804, 0xdd0e,
0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd25, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd0e, 0xd804, 0xdd34,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd20,
0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd25, 0xd804, 0xdd1d,
0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd01, 0xd804, 0xdd07, 0xd804, 0xdd25, 0xd804, 0xdd2c,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd07,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd25, 0xd804, 0xdd23,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd23,
0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd26, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804,
0xdd11, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd26, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804,
0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd07, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd1b, 0xd804, 0xdd23, 0xd804, 0xdd08, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804,
0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804,
0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd2a, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804,
0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804,
0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804,
0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd07, 0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804,
0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804,
0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd26, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804,
0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd1d, 0xd804, 0xdd1d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804,
0xdd09, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd2d, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804,
0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804,
0xdd34, 0xd804, 0xdd24, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804,
0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0x2d, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804,
0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd03, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd03, 0xd804, 0xdd0b, 0xd804,
0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804,
0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd03, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd03, 0xd804,
0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2d, 0xd804, 0xdd1a, 0xd804,
0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd11, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd07,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1d,
0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd23, 0x20, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804,
0xdd26, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd2e, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26, 0xd804,
0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd23, 0xd804, 0xdd2d, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd05, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2d,
0xd804, 0xdd20, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1d,
0xd804, 0xdd03, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804,
0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1d, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804,
0xdd20, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2c,
0xd804, 0xdd22, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd01, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x2d, 0xd804,
0xdd25, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd31, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd2e,
0xd804, 0xdd11, 0xd804, 0xdd1d, 0xd804, 0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd0e,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804,
0xdd2e, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd07, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd26,
0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd07,
0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2d,
0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1d, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd22,
0xd804, 0xdd25, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd13, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804,
0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2d, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804,
0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd18,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804,
0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd1c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804,
0xdd0e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd13,
0xd804, 0xdd09, 0xd804, 0xdd31, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2e,
0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd31, 0xd804, 0xdd16, 0xd804, 0xdd2c,
0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd26, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd1e, 0xd804, 0xdd1a,
0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd1f, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0x2c, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd11,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd11,
0xd804, 0xdd28, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd1e, 0xd804, 0xdd2c,
0xd804, 0xdd1e, 0xd804, 0xdd2c, 0xd804, 0xdd20, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd1e, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd31,
0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34,
0xd804, 0xdd0e, 0xd804, 0xdd1f, 0xd804, 0xdd2d, 0xd804, 0xdd07, 0xd804, 0xdd0e, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd2d, 0xd804, 0xdd2a, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0x2c, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd07,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd23, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804,
0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0e,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd23,
0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd20,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd20, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd09, 0xd804, 0xdd31, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd09,
0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd16,
0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd0e, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1f,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd24, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd0b, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804,
0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd2c, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804,
0xdd1a, 0xd804, 0xdd25, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd23, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd13,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd11, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd16,
0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20, 0x2c, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd11, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd25, 0xd804, 0xdd23,
0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd11, 0xd804, 0xdd1a, 0xd804,
0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd0e, 0xd804,
0xdd28, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd1b, 0xd804, 0xdd1a, 0xd804,
0xdd1f, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd1f, 0xd804,
0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd1c, 0xd804,
0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd34, 0x2d, 0xd804, 0xdd26, 0xd804, 0xdd27, 0x2d, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd25, 0xd804,
0xdd33, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1b,
0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd2e, 0x20, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1c, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd22, 0xd804, 0xdd2c,
0xd804, 0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd34,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1d, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804,
0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2a, 0xd804,
0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804,
0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd2e, 0x20, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd01,
0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804,
0xdd1b, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x2e, 0x20, 0xd804, 0xdd1d, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd17, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804,
0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd34, 0x2e, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x2e, 0x20, 0xd804, 0xdd23, 0xd804, 0xdd2a,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd17, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2d, 0xd804, 0xdd2a, 0xd804, 0xdd1c, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd33, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804,
0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd23, 0xd804,
0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd17, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804,
0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd11, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd1e, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd1e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd20, 0xd804, 0xdd2d, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804,
0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804,
0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd13, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd05,
0xd804, 0xdd20, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1b, 0xd804, 0xdd23,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd17, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd24,
0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd01, 0xd804, 0xdd09,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd26,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd26, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd13, 0xd804, 0xdd28,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd01, 0xd804, 0xdd07, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a,
0xd804, 0xdd2f, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd2d,
0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2c,
0xd804, 0xdd2d, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd07, 0xd804, 0xdd27,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd1d, 0xd804, 0xdd27,
0xd804, 0xdd27, 0xd804, 0xdd31, 0xd804, 0xdd31, 0xd804, 0xdd18, 0xd804, 0xdd1f, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd14, 0xd804, 0xdd07,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2a,
0xd804, 0xdd1d, 0xd804, 0xdd2d, 0xd804, 0xdd18, 0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd0e, 0xd804, 0xdd26,
0xd804, 0xdd2c, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd25, 0xd804,
0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804,
0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd01, 0xd804, 0xdd26,
0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd0e,
0xd804, 0xdd07, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd0e, 0xd804, 0xdd27,
0xd804, 0xdd20, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd0e, 0xd804, 0xdd2c,
0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804, 0xdd11, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd0c, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2a, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2a, 0xd804, 0xdd08, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd07, 0xd804, 0xdd27,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd18, 0xd804, 0xdd07, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd20,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd23, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd01, 0xd804, 0xdd07, 0xd804, 0xdd2a,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd13, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd07, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd23,
0xd804, 0xdd1f, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd34,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd07,
0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1d,
0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd01, 0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd19, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd08, 0xd804, 0xdd23,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd08, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd01, 0xd804, 0xdd0a,
0xd804, 0xdd2d, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09,
0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2c,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c,
0xd804, 0xdd16, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd08, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd1d,
0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd16,
0xd804, 0xdd2c, 0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd17, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2a,
0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2e,
0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd16, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd28, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0x20, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804,
0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0b, 0xd804,
0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd11, 0xd804, 0xdd2c, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd1e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804,
0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2a, 0xd804, 0xdd13, 0xd804,
0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804,
0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22,
0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd2d, 0xd804, 0xdd22,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd09,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd25, 0xd804,
0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd25, 0xd804, 0xdd33,
0xd804, 0xdd11, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c,
0xd804, 0xdd2d, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd07,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804,
0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804,
0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd26, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd23,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd1b, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd17, 0xd804, 0xdd27, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd33,
0xd804, 0xdd03, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd03, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd08, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd17,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd23,
0xd804, 0xdd1e, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd2c,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd08, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd13, 0xd804, 0xdd1b, 0xd804, 0xdd2c,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd1b,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd09,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd1b, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28,
0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd26, 0xd804, 0xdd2c,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd01, 0xd804, 0xdd07,
0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2d, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28,
0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd13,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1e,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1e,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd1d,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd13, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd23,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd26, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd33, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd1a,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd13,
0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07,
0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804, 0xdd22,
0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd22,
0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd22, 0xd804,
0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804,
0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804,
0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd21, 0xd804, 0xdd2c, 0xd804,
0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd25, 0xd804,
0xdd33, 0xd804, 0xdd11, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804,
0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804,
0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd18, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804,
0xdd1e, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804,
0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd24, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804,
0xdd08, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd1a, 0xd804, 0xdd1a, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804,
0xdd0d, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd21, 0xd804,
0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd0e,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd18,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd07,
0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd1f,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd20, 0xd804, 0xdd2e, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd1b,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd0c,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd17, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1c, 0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd1c,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1c,
0xd804, 0xdd2e, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd1c,
0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd1c, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd28,
0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd1b, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd09,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd18,
0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd31, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd2a,
0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd28,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd16,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd22,
0xd804, 0xdd20, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd31, 0xd804, 0xdd0e, 0xd804, 0xdd23,
0xd804, 0xdd2c, 0xd804, 0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd0e,
0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2e,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804,
0xdd1b, 0xd804, 0xdd23, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804,
0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804,
0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd2f, 0xd804, 0xdd22, 0xd804, 0xdd2c,
0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd01, 0xd804, 0xdd09, 0xd804, 0xdd25,
0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd26,
0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd24,
0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd01, 0xd804, 0xdd09, 0xd804, 0xdd11, 0xd804, 0xdd1b,
0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0x411, 0x440, 0x430, 0x437, 0x437, 0x430, 0x432, 0x438, 0x43b, 0x44c,
0x41a, 0x44a, 0x430, 0x445, 0x4c0, 0x438, 0x440, 0x430, 0x419, 0x43e, 0x445, 0x430,
0x43d, 0x43d, 0x435, 0x441, 0x431, 0x443, 0x440, 0x433, 0x41b, 0x438, 0x431, 0x440,
0x435, 0x432, 0x438, 0x43b, 0x44c, 0x410, 0x43d, 0x43a, 0x43e, 0x440, 0x438, 0x434,
0x436, 0x410, 0x43d, 0x433, 0x438, 0x43b, 0x44c, 0x44f, 0x411, 0x443, 0x44d, 0x43d,
0x43e, 0x441, 0x2d, 0x410, 0x439, 0x440, 0x435, 0x441, 0x420, 0x438, 0x43e, 0x2d,
0x413, 0x430, 0x43b, 0x44c, 0x435, 0x433, 0x43e, 0x441, 0x410, 0x441, 0x443, 0x43d,
0x441, 0x44c, 0x43e, 0x43d, 0x411, 0x430, 0x438, 0x44f, 0x2d, 0x434, 0x435, 0x2d,
0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x411, 0x43e, 0x439, 0x441,
0x435, 0x41a, 0x435, 0x439, 0x43c, 0x431, 0x440, 0x438, 0x434, 0x436, 0x2d, 0x411,
0x435, 0x439, 0x41a, 0x430, 0x439, 0x435, 0x43d, 0x43d, 0x430, 0x42d, 0x434, 0x43c,
0x43e, 0x43d, 0x442, 0x43e, 0x43d, 0x42d, 0x439, 0x440, 0x443, 0x43d, 0x435, 0x43f,
0x435, 0x424, 0x43e, 0x440, 0x442, 0x20, 0x41d, 0x435, 0x43b, 0x44c, 0x441, 0x43e,
0x43d, 0x413, 0x43b, 0x435, 0x439, 0x441, 0x2d, 0x411, 0x435, 0x439, 0x413, 0x443,
0x441, 0x2d, 0x411, 0x435, 0x439, 0x413, 0x443, 0x430, 0x44f, 0x43a, 0x438, 0x43b,
0x44c, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x413, 0x430, 0x43b, 0x438, 0x444,
0x430, 0x43a, 0x441, 0x42d, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b, 0x44c, 0x43e,
0x412, 0x438, 0x432, 0x44d, 0x439, 0x41b, 0x43e, 0x441, 0x2d, 0x410, 0x43d, 0x434,
0x436, 0x435, 0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x443, 0x44d, 0x440, 0x2d, 0x41f,
0x440, 0x438, 0x43d, 0x441, 0x435, 0x441, 0x2d, 0x41a, 0x443, 0x43e, 0x440, 0x442,
0x435, 0x440, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x440, 0x440, 0x435, 0x439, 0x41c,
0x43e, 0x43d, 0x441, 0x435, 0x440, 0x440, 0x430, 0x442, 0x41d, 0x44c, 0x44e, 0x2d,
0x419, 0x43e, 0x440, 0x43a, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x445, 0x430, 0x411,
0x43e, 0x439, 0x43b, 0x430, 0x2c, 0x20, 0x41a, 0x44a, 0x438, 0x43b, 0x431, 0x430,
0x441, 0x435, 0x434, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x42e,
0x43a, 0x44a, 0x2c, 0x20, 0x41a, 0x44a, 0x438, 0x43b, 0x431, 0x430, 0x441, 0x435,
0x434, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e,
0x2d, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x413, 0x43e, 0x442, 0x445, 0x43e, 0x431,
0x41f, 0x443, 0x44d, 0x440, 0x442, 0x43e, 0x2d, 0x420, 0x438, 0x43a, 0x43e, 0x420,
0x435, 0x434, 0x436, 0x430, 0x439, 0x43d, 0x430, 0x421, 0x430, 0x43d, 0x442, 0x44c,
0x44f, 0x433, 0x43e, 0x421, 0x435, 0x43d, 0x2d, 0x411, 0x430, 0x440, 0x442, 0x435,
0x43b, 0x44c, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41b, 0x44e, 0x441,
0x438, 0x421, 0x432, 0x438, 0x444, 0x442, 0x2d, 0x41a, 0x430, 0x440, 0x435, 0x43d,
0x442, 0x422, 0x435, 0x433, 0x443, 0x441, 0x438, 0x433, 0x430, 0x43b, 0x44c, 0x43f,
0x430, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e, 0x440, 0x432,
0x438, 0x43b, 0x44c, 0x422, 0x440, 0x43e, 0x43b, 0x43b, 0x44c, 0x41b, 0x43e, 0x43d,
0x433, 0x439, 0x438, 0x440, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430,
0x4c0, 0x430, 0x43c, 0x43c, 0x430, 0x43d, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440,
0x44c, 0x410, 0x442, 0x438, 0x440, 0x430, 0x443, 0x411, 0x430, 0x433, 0x4c0, 0x434,
0x430, 0x434, 0x411, 0x430, 0x43a, 0x43e, 0x445, 0x414, 0x438, 0x43c, 0x430, 0x448,
0x43a, 0x44a, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x414, 0x436, 0x430,
0x439, 0x43f, 0x443, 0x440, 0x41a, 0x44a, 0x443, 0x434, 0x441, 0x2d, 0x413, 0x4c0,
0x430, 0x43b, 0x430, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b,
0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441,
0x43a, 0x438, 0x41a, 0x430, 0x43b, 0x44c, 0x43a, 0x443, 0x442, 0x442, 0x430, 0x41f,
0x43d, 0x43e, 0x43c, 0x43f, 0x435, 0x43d, 0x44c, 0x42d, 0x440, 0x2d, 0x420, 0x438,
0x44f, 0x434, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x433, 0x4c0,
0x2d, 0x435, 0x422, 0x435, 0x445, 0x4c0, 0x440, 0x430, 0x43d, 0x423, 0x441, 0x442,
0x44c, 0x2d, 0x41d, 0x435, 0x440, 0x430, 0x412, 0x44c, 0x435, 0x43d, 0x442, 0x44c,
0x44f, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430,
0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434,
0x430, 0x448, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x430, 0x43d, 0x20, 0x433, 0x4c0,
0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x424, 0x430, 0x440, 0x435, 0x440,
0x438, 0x439, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430,
0x448, 0x41c, 0x430, 0x434, 0x435, 0x439, 0x440, 0x430, 0x2c, 0x20, 0x433, 0x4c0,
0x2d, 0x435, 0x420, 0x435, 0x439, 0x43a, 0x44c, 0x44f, 0x432, 0x438, 0x43a, 0x41a,
0x44a, 0x438, 0x43b, 0x431, 0x430, 0x20, 0x414, 0x436, 0x43e, 0x440, 0x434, 0x436,
0x438, 0x415, 0x437, 0x430, 0x447, 0x443, 0x20, 0x415, 0x43b, 0x435, 0x43d, 0x438,
0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x421, 0x442, 0x44d, 0x43d,
0x43b, 0x438, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x2c, 0x20,
0x433, 0x4c0, 0x2d, 0x435, 0x421, 0x438, 0x434, 0x43d, 0x435, 0x439, 0x410, 0x448,
0x442, 0x430, 0x440, 0x43a, 0x445, 0x43d, 0x435, 0x410, 0x444, 0x438, 0x43d, 0x430,
0x411, 0x440, 0x44e, 0x441, 0x441, 0x435, 0x43b, 0x44c, 0x411, 0x44e, 0x437, 0x438,
0x43d, 0x433, 0x435, 0x43d, 0x2d, 0x430, 0x43c, 0x2d, 0x425, 0x43e, 0x445, 0x440,
0x430, 0x439, 0x43d, 0x41a, 0x43e, 0x43f, 0x435, 0x43d, 0x433, 0x430, 0x433, 0x435,
0x43d, 0x425, 0x435, 0x43b, 0x44c, 0x441, 0x438, 0x43d, 0x43a, 0x438, 0x41c, 0x44d,
0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435, 0x414, 0x436, 0x435, 0x440, 0x441,
0x438, 0x421, 0x438, 0x43c, 0x444, 0x435, 0x440, 0x43e, 0x43f, 0x43e, 0x43b, 0x44c,
0x421, 0x43a, 0x43e, 0x43f, 0x44c, 0x435, 0x421, 0x442, 0x43e, 0x43a, 0x433, 0x43e,
0x43b, 0x44c, 0x43c, 0x423, 0x43b, 0x44c, 0x44f, 0x43d, 0x43e, 0x432, 0x441, 0x43a,
0x412, 0x438, 0x43b, 0x44c, 0x43d, 0x44e, 0x441, 0x4c0, 0x438, 0x439, 0x441, 0x430,
0x20, 0x43f, 0x430, 0x439, 0x445, 0x430, 0x43c, 0x430, 0x440, 0x20, 0x432, 0x438,
0x43d, 0x447, 0x443, 0x20, 0x434, 0x435, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439,
0x440, 0x435, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x438, 0x439, 0x43d, 0x2c, 0x20,
0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41a, 0x43e, 0x43c, 0x43e, 0x440, 0x438,
0x439, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448,
0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x430, 0x448, 0x41c, 0x430, 0x439,
0x43e, 0x440, 0x43a, 0x430, 0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x43e, 0x43d, 0x411,
0x443, 0x433, 0x435, 0x43d, 0x432, 0x438, 0x43b, 0x44c, 0x2c, 0x20, 0x433, 0x4c0,
0x2d, 0x435, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x2c, 0x20, 0x433, 0x4c0, 0x2d,
0x435, 0x422, 0x440, 0x443, 0x43a, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430,
0x448, 0x41c, 0x430, 0x440, 0x445, 0x438, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439,
0x440, 0x435, 0x43d, 0x430, 0x448, 0x42d, 0x444, 0x430, 0x442, 0x435, 0x413, 0x430,
0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x430, 0x43d, 0x20, 0x433, 0x4c0,
0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x413, 0x430, 0x43c, 0x431, 0x44c,
0x435, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x413, 0x43e,
0x43d, 0x43e, 0x43b, 0x443, 0x43b, 0x443, 0x42d, 0x43d, 0x434, 0x435, 0x440, 0x431,
0x435, 0x440, 0x438, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435, 0x41c, 0x430,
0x440, 0x43a, 0x438, 0x437, 0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d,
0x430, 0x448, 0x41c, 0x438, 0x434, 0x443, 0x44d, 0x439, 0x2c, 0x20, 0x433, 0x4c0,
0x2d, 0x43d, 0x430, 0x448, 0x41d, 0x438, 0x443, 0x44d, 0x41f, 0x43e, 0x43d, 0x430,
0x43f, 0x435, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x422, 0x430,
0x438, 0x442, 0x438, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x423,
0x44d, 0x439, 0x43a, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435, 0x13a0, 0x13c8, 0x13e3,
0x13c2, 0x13a0, 0x13c6, 0x13a1, 0x13d7, 0x13cd, 0x20, 0x13a0, 0x13c6, 0x13c6, 0x13a0, 0x13b5,
0x13e5, 0x13b5, 0x13a0, 0x13cf, 0x13b9, 0x13b3, 0x13c6, 0x13b9, 0x13aa, 0x13c7, 0x13c2, 0x13ab,
0x13eb, 0x13c6, 0x13c2, 0x13e7, 0x13b5, 0x13c7, 0x13cc, 0x13eb, 0x13c6, 0x13d8, 0x13b4, 0x13c6,
0x13cc, 0x13e9, 0x13b5, 0x13ca, 0x13e7, 0x13bb, 0x13ca, 0x13b3, 0x13a7, 0x13ef, 0x13b6, 0x13a4,
0x13c1, 0x13a6, 0x20, 0x13a6, 0x13b5, 0x13e6, 0x13d5, 0x13d1, 0x13d4, 0x13aa, 0x13be, 0x13c8,
0x13d3, 0x13a7, 0x13e9, 0x13d3, 0x20, 0x13a1, 0x13cf, 0x20, 0x13cc, 0x13b3, 0x13bb, 0x13e5,
0x13ca, 0x13d7, 0x13d9, 0x13a4, 0x13e9, 0x13b3, 0x13a1, 0x13b5, 0x20, 0x13a0, 0x13f1, 0x13f3,
0x13c2, 0x13a0, 0x13ce, 0x13c7, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13a6, 0x13c9, 0x13b6, 0x13c1,
0x13ad, 0x13b3, 0x13b4, 0x13e6, 0x13ad, 0x13c1, 0x13cd, 0x13ca, 0x13a6, 0x13e7, 0x13c6, 0x13a7,
0x13bb, 0x13c6, 0x13b3, 0x13a7, 0x13da, 0x13bb, 0x13a9, 0x13a6, 0x13b5, 0x13a8, 0x13c2, 0x13dd,
0x13cc, 0x13b3, 0x13aa, 0x13cf, 0x13b5, 0x13c7, 0x13eb, 0x13b5, 0x13b6, 0x13ba, 0x13b7, 0x13e9,
0x13c2, 0x13d3, 0x13b7, 0x13ca, 0x13c6, 0x13cf, 0x13b5, 0x13cc, 0x13a7, 0x13b9, 0x13b3, 0x13c9,
0x13b9, 0x13ca, 0x13d9, 0x13b9, 0x13ce, 0x13b5, 0x13c6, 0x13c7, 0x13c1, 0x13bc, 0x13a6, 0x13d7,
0x13e1, 0x13bc, 0x13c2, 0x13b6, 0x13eb, 0x13ef, 0x13be, 0x13f1, 0x13b6, 0x13c8, 0x13be, 0x13e3,
0x13ba, 0x13be, 0x13c2, 0x13ef, 0x13ba, 0x13be, 0x13ec, 0x13e3, 0x13d8, 0x13a3, 0x13e9, 0x13a6,
0x13da, 0x13ab, 0x13e5, 0x13f3, 0x13d7, 0x13d4, 0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x2d, 0x13c3,
0x13ec, 0x13cc, 0x13a3, 0x13d9, 0x13ba, 0x13df, 0x13c9, 0x13b5, 0x13da, 0x13c2, 0x13cf, 0x13ea,
0x13c4, 0x13d7, 0x13b0, 0x13a9, 0x13a0, 0x13d3, 0x13a7, 0x13a0, 0x13c2, 0x13aa, 0x13b4, 0x13e5,
0x13a0, 0x13c2, 0x13a9, 0x13b3, 0x13a4, 0x13ea, 0x13d8, 0x13a0, 0x13b3, 0x13ab, 0x13e9, 0x13f1,
0x13be, 0x13a3, 0x13cd, 0x13d3, 0x20, 0x13a4, 0x13c3, 0x13b4, 0x13a7, 0x13d4, 0x13b9, 0x13a7,
0x13aa, 0x13d9, 0x13c6, 0x13e7, 0x13e7, 0x13eb, 0x13b3, 0x20, 0x13b5, 0x13f2, 0x13ad, 0x13ba,
0x13be, 0x13d9, 0x13cc, 0x13a6, 0x13f0, 0x13aa, 0x13cf, 0x20, 0x13a4, 0x13ea, 0x13f4, 0x13cc,
0x13b3, 0x13d4, 0x13cc, 0x13c2, 0x20, 0x13e9, 0x13c2, 0x13a4, 0x13d3, 0x13c5, 0x13d7, 0x20,
0x13b7, 0x13eb, 0x13cf, 0x13da, 0x13ab, 0x13b9, 0x13c2, 0x13a4, 0x13d1, 0x13e9, 0x13ef, 0x13a0,
0x13b7, 0x13c6, 0x13a0, 0x13d1, 0x13cf, 0x13f2, 0x13c5, 0x13a0, 0x13d8, 0x13aa, 0x13a6, 0x13c2,
0x13c6, 0x13af, 0x13ef, 0x13c6, 0x13af, 0x13ef, 0x20, 0x13c6, 0x13c2, 0x13d5, 0x13b3, 0x13cf,
0x13c6, 0x13c7, 0x13d9, 0x13cd, 0x13c7, 0x13b4, 0x13bb, 0x13c7, 0x13b5, 0x13cd, 0x13dd, 0x13c2,
0x2d, 0x13cc, 0x13e0, 0x13c2, 0x13a3, 0x13cd, 0x13d3, 0x20, 0x13a0, 0x13aa, 0x13b5, 0x13f0,
0x13d7, 0x13c9, 0x13aa, 0x13d4, 0x13c9, 0x13f1, 0x13cf, 0x13a8, 0x13bb, 0x13c8, 0x13e5, 0x20,
0x13a1, 0x13c9, 0x13c4, 0x13b8, 0x13d7, 0x13a4, 0x13d4, 0x13be, 0x20, 0x13e0, 0x13a8, 0x13cf,
0x13a8, 0x13c2, 0x13ab, 0x13c2, 0x13a7, 0x13b3, 0x13a7, 0x13cf, 0x13a7, 0x13f0, 0x13c2, 0x13a8,
0x13a2, 0x13b9, 0x13c2, 0x13e5, 0x13a7, 0x13a9, 0x13e5, 0x13e9, 0x13e9, 0x13cf, 0x13f3, 0x13d3,
0x20, 0x13c6, 0x13a1, 0x13cf, 0x13aa, 0x13cd, 0x13d3, 0x13b5, 0x13a7, 0x13de, 0x13cd, 0x13d4,
0x13c2, 0x13e7, 0x13eb, 0x13c6, 0x13ab, 0x13b3, 0x13a8, 0x13a3, 0x13d5, 0x13be, 0x13b9, 0x13a7,
0x13cc, 0x13e9, 0x13c2, 0x13d3, 0x13cc, 0x13c2, 0x13d3, 0x13cc, 0x13c2, 0x20, 0x13a4, 0x13ea,
0x13f4, 0x13a6, 0x13b8, 0x13b3, 0x13d7, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13d7, 0x13e0, 0x13d8,
0x13d9, 0x13bb, 0x13c2, 0x13a7, 0x13a1, 0x13d7, 0x13b9, 0x13c2, 0x13d4, 0x13c2, 0x13a1, 0x13b7,
0x13c1, 0x13c7, 0x13a1, 0x13b5, 0x20, 0x13cc, 0x13b5, 0x13c6, 0x13d9, 0x13b5, 0x13d7, 0x13d0,
0x13f4, 0x20, 0x13c1, 0x13b5, 0x13cc, 0x13c2, 0x13ec, 0x13d4, 0x13b4, 0x13ce, 0x13de, 0x13cf,
0x20, 0x13a1, 0x13c9, 0x13c4, 0x13b8, 0x13d7, 0x13cc, 0x13cc, 0x20, 0x13a1, 0x13c9, 0x13c4,
0x13b8, 0x13d7, 0x13c4, 0x13ac, 0x13eb, 0x13f3, 0x13d2, 0x20, 0x13ac, 0x13be, 0x13cb, 0x13be,
0x13d3, 0x13e9, 0x13d3, 0x13b7, 0x13c7, 0x13e9, 0x13d4, 0x13b9, 0x13b3, 0x13c6, 0x13ef, 0x13a9,
0x13b5, 0x13a6, 0x13ef, 0x13be, 0x13ad, 0x13b5, 0x13e9, 0x13a7, 0x13cf, 0x13ad, 0x13e9, 0x13be,
0x13ae, 0x13bc, 0x13cf, 0x13b6, 0x13a2, 0x13c2, 0x13d7, 0x13a0, 0x13c2, 0x13a0, 0x13c9, 0x13b5,
0x13cf, 0x13c3, 0x13c8, 0x13cf, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13b9,
0x13b4, 0x13c2, 0x13aa, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13c8, 0x13d3,
0x13c8, 0x13a6, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13d6, 0x13b5, 0x20,
0x13a6, 0x13da, 0x13b2, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13ea, 0x13ea,
0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13eb, 0x13c2, 0x13ce, 0x13c1, 0x13cf,
0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13eb, 0x13be, 0x13b9, 0x13a9, 0x2c,
0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13a2, 0x13c4, 0x13eb, 0x13a9, 0x13a2, 0x13c6,
0x13b7, 0x13f1, 0x13d8, 0x13e3, 0x13ba, 0x13a2, 0x13a7, 0x13e7, 0x13c3, 0x13b7, 0x13eb, 0x13eb,
0x13b5, 0x13b9, 0x13c2, 0x13d4, 0x13ce, 0x13b6, 0x2c, 0x20, 0x13ac, 0x13c5, 0x13d3, 0x13a9,
0x13c6, 0x13b4, 0x13c2, 0x13d7, 0x13d9, 0x13af, 0x13be, 0x13cd, 0x13a9, 0x20, 0x13d7, 0x13c2,
0x13a7, 0x13bf, 0x13e9, 0x13d7, 0x13d9, 0x13af, 0x13a1, 0x13b3, 0x13d7, 0x20, 0x13d7, 0x13dc,
0x20, 0x13a4, 0x13ac, 0x13eb, 0x13f3, 0x13af, 0x20, 0x13a9, 0x13c4, 0x13d9, 0x13d7, 0x13b9,
0x13ce, 0x13f2, 0x13b9, 0x13be, 0x13c6, 0x13b9, 0x13be, 0x13a4, 0x13cf, 0x13b9, 0x13b5, 0x13a6,
0x13d8, 0x13b9, 0x13d8, 0x13c2, 0x13c7, 0x13b9, 0x13d4, 0x13bc, 0x13b6, 0x13cf, 0x13b9, 0x13cc,
0x13dd, 0x13c2, 0x13ba, 0x13c3, 0x13bb, 0x13c2, 0x13ba, 0x13b5, 0x13d3, 0x13ba, 0x13dd, 0x13a7,
0x13dd, 0x13a0, 0x13c2, 0x13cd, 0x13c6, 0x13c2, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13bb, 0x13a8,
0x13b6, 0x13c2, 0x13b9, 0x13be, 0x13d4, 0x13c2, 0x13bc, 0x13d6, 0x13b5, 0x13bc, 0x13c2, 0x13d6,
0x13eb, 0x13d5, 0x13f2, 0x13b9, 0x13c2, 0x13d8, 0x13cc, 0x13b3, 0x13d7, 0x13be, 0x13cc, 0x13eb,
0x13c4, 0x13ef, 0x13a9, 0x13c3, 0x13ba, 0x13c3, 0x13b6, 0x13be, 0x13ad, 0x13c7, 0x13f3, 0x13b3,
0x2c, 0x20, 0x13e7, 0x13f4, 0x13e2, 0x20, 0x13d3, 0x13aa, 0x13d4, 0x13a0, 0x13f0, 0x13df,
0x2c, 0x20, 0x13e7, 0x13f4, 0x13e2, 0x20, 0x13d3, 0x13aa, 0x13d4, 0x13a2, 0x13e4, 0x20,
0x13ce, 0x13b4, 0x13bb, 0x2c, 0x20, 0x13e7, 0x13f4, 0x13e2, 0x20, 0x13d3, 0x13aa, 0x13d4,
0x13c4, 0x13a9, 0x13a3, 0x13e5, 0x13be, 0x13a6, 0x13c6, 0x13be, 0x13b9, 0x13c6, 0x13b3, 0x13b9,
0x13b4, 0x13c9, 0x13e7, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x13af, 0x13e5, 0x13f3, 0x13d7, 0x13d4,
0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x2d, 0x13be, 0x13bf, 0x2d, 0x13a4, 0x13ac, 0x13eb, 0x13f3,
0x13af, 0x13e5, 0x13f3, 0x13d7, 0x13d4, 0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x20, 0x13cd, 0x13c6,
0x13c2, 0x13a8, 0x13cd, 0x13db, 0x13a4, 0x13ea, 0x13d8, 0x20, 0x13e5, 0x13f3, 0x13d7, 0x13d4,
0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x13c7, 0x13a1, 0x13d9, 0x20, 0x13b5, 0x13a2, 0x13aa, 0x13ca,
0x13d4, 0x20, 0x13a1, 0x13eb, 0x13be, 0x13cd, 0x13b4, 0x13c2, 0x13a9, 0x13c2, 0x20, 0x13a2,
0x13c2, 0x13b4, 0x13d8, 0x13b4, 0x13cf, 0x13ea, 0x13b4, 0x13a9, 0x13be, 0x13b4, 0x13d0, 0x13b7,
0x13d8, 0x13a4, 0x13c1, 0x13a6, 0x20, 0x13a4, 0x13ea, 0x13f4, 0x13cc, 0x13c2, 0x13d4, 0x13b4,
0x13bb, 0x13cc, 0x13c2, 0x13d8, 0x13ef, 0x13aa, 0x13cc, 0x13c2, 0x13d9, 0x20, 0x13d9, 0x13bb,
0x13aa, 0x13cc, 0x13a3, 0x20, 0x13c6, 0x13b6, 0x13a2, 0x13d9, 0x13aa, 0x13d9, 0x13bb, 0x13d8,
0x13cf, 0x13d8, 0x13a7, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13c6, 0x13b5, 0x13de, 0x13b4,
0x13bb, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13e3, 0x13c2, 0x20, 0x13a4, 0x13e4, 0x13b5,
0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13a9, 0x13d8, 0x13cf, 0x13a4, 0x13d3, 0x13c5, 0x13d8,
0x20, 0x13b7, 0x13cf, 0x13ef, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13d9, 0x13b9, 0x13cf,
0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13eb, 0x13c2, 0x13ce, 0x13d8, 0x13a0, 0x13ef, 0x13c4,
0x13b5, 0x20, 0x13a4, 0x13c3, 0x13b4, 0x13d6, 0x13ab, 0x13cf, 0x13a6, 0x13b5, 0x13c6, 0x13e1,
0x13b5, 0x13d8, 0x13f3, 0x13e9, 0x13be, 0x13d9, 0x13b3, 0x13be, 0x13d9, 0x13d9, 0x13d9, 0x13b3,
0x13ea, 0x13c2, 0x13ab, 0x13ea, 0x13b5, 0x13a4, 0x13c1, 0x13a6, 0x20, 0x13d0, 0x13c8, 0x13b5,
0x13eb, 0x13c2, 0x13c7, 0x13a9, 0x13ef, 0x13ab, 0x13d4, 0x13d8, 0x13d5, 0x13eb, 0x13cf, 0x13da,
0x13bc, 0x13be, 0x13d8, 0x2d, 0x13d7, 0x2019, 0x13a4, 0x13b5, 0x13eb, 0x13b5, 0x13b9, 0x13c7,
0x13b5, 0x13b9, 0x13cc, 0x13c2, 0x13bb, 0x13a6, 0x13bd, 0x13d9, 0x13c6, 0x13b5, 0x13ba, 0x13b5,
0x13b3, 0x13de, 0x13b3, 0x13cf, 0x13f2, 0x13e9, 0x13e0, 0x13b5, 0x13ec, 0x13cd, 0x13d9, 0x13a7,
0x13a6, 0x13c5, 0x13af, 0x13d3, 0x20, 0x13a4, 0x13d5, 0x13d8, 0x13f4, 0x13cc, 0x13d7, 0x13d2,
0x13a2, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13a1, 0x13d5, 0x13c2, 0x13a0, 0x13b5, 0x13b9, 0x13d8,
0x13a0, 0x13b9, 0x13c2, 0x13a0, 0x13be, 0x13d7, 0x13b5, 0x13a0, 0x13a6, 0x13d4, 0x13eb, 0x13a0,
0x13a6, 0x13d9, 0x13c7, 0x13a0, 0x13cd, 0x13a6, 0x13c6, 0x13d8, 0x13a0, 0x13d8, 0x13c6, 0x13a4,
0x13c6, 0x13a9, 0x13d3, 0x13d7, 0x13c6, 0x13ad, 0x13b4, 0x13a2, 0x13c2, 0x13c7, 0x13c2, 0x13a9,
0x13aa, 0x13a9, 0x13c6, 0x13be, 0x13a3, 0x13b5, 0x13c7, 0x13b7, 0x13d8, 0x13c7, 0x13cd, 0x13a8,
0x13a9, 0x13ca, 0x13be, 0x13a2, 0x13e5, 0x13d4, 0x13aa, 0x13b6, 0x13bb, 0x13c9, 0x13d3, 0x13b9,
0x13cd, 0x13ac, 0x13cf, 0x13da, 0x13c6, 0x13f1, 0x13da, 0x13dd, 0x13be, 0x13c7, 0x13c6, 0x13b9,
0x13ab, 0x13cd, 0x13d4, 0x13a6, 0x13cc, 0x13ae, 0x13c9, 0x13c2, 0x13b0, 0x20, 0x13e5, 0x20,
0x13bb, 0x13c2, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13b0, 0x13c2, 0x13a9, 0x20, 0x13aa, 0x13c2,
0x13a9, 0x13b0, 0x13e9, 0x13d7, 0x13a2, 0x13ab, 0x13e5, 0x13a7, 0x13e3, 0x13a7, 0x13d4, 0x13e3,
0x13ef, 0x13cb, 0x13b3, 0x13e4, 0x13b7, 0x13cc, 0x13b4, 0x13bb, 0x13a7, 0x13ca, 0x13b5, 0x13a7,
0x13bb, 0x13e3, 0x13a7, 0x13a7, 0x13b3, 0x13e5, 0x13a7, 0x13d8, 0x13b9, 0x13c2, 0x13da, 0x13a7,
0x13c2, 0x13d7, 0x13a6, 0x13aa, 0x13b5, 0x13a7, 0x13d4, 0x13dd, 0x13cd, 0x13c3, 0x13ef, 0x13cd,
0x13a7, 0x13ab, 0x13e9, 0x13b3, 0x20, 0x13b8, 0x13bb, 0x13cb, 0x13ab, 0x13e5, 0x13c2, 0x13a6,
0x13ab, 0x13ea, 0x13d8, 0x13b9, 0x13a7, 0x13a4, 0x13b9, 0x13a6, 0x13d3, 0x13c2, 0x13b9, 0x13a7,
0x13cc, 0x13b5, 0x13b9, 0x13c2, 0x13b3, 0x13bd, 0x13cd, 0x13a6, 0x13d8, 0x13c2, 0x13aa, 0x13cf,
0x13ef, 0x13c3, 0x13ec, 0x13ab, 0x13c1, 0x13d6, 0x13cd, 0x13a7, 0x13c3, 0x13ec, 0x13cf, 0x13c8,
0x13cd, 0x13a7, 0x13a3, 0x13bb, 0x13cd, 0x13a7, 0x13a3, 0x13b3, 0x13b5, 0x13bf, 0x13bb, 0x20,
0x13c7, 0x13c2, 0x13c6, 0x13c2, 0x13d8, 0x13a0, 0x13be, 0x13a9, 0x13c8, 0x13ef, 0x13c2, 0x13a9,
0x13f0, 0x13c2, 0x13a9, 0x13a7, 0x13d4, 0x13b5, 0x13a7, 0x13cd, 0x13d4, 0x13c1, 0x13a9, 0x13cf,
0x13b6, 0x13b3, 0x13d3, 0x13b5, 0x13ef, 0x13d7, 0x13cc, 0x13a7, 0x13b5, 0x13c2, 0x13cc, 0x13b9,
0x13a7, 0x13c2, 0x13d7, 0x13d0, 0x13b5, 0x13ce, 0x13c2, 0x13a6, 0x13ad, 0x13f1, 0x13cf, 0x13c2,
0x13a6, 0x13c9, 0x13b5, 0x13cd, 0x13b4, 0x13d7, 0x13c1, 0x13aa, 0x13b5, 0x13bb, 0x13cd, 0x13a7,
0x13d4, 0x13f1, 0x13c7, 0x13d4, 0x13cf, 0x13a8, 0x13c2, 0x13d8, 0x13d8, 0x13c8, 0x13b5, 0x13cf,
0x13d6, 0x13b3, 0x13c2, 0x13de, 0x13bb, 0x13e1, 0x13d9, 0x13a9, 0x13f2, 0x13d9, 0x13bb, 0x13cd,
0x13a7, 0x13a4, 0x13b3, 0x13c2, 0x13c6, 0x13d4, 0x13a4, 0x13b7, 0x13bb, 0x13a9, 0x13a4, 0x13cd,
0x13d4, 0x2d, 0x13c1, 0x13b3, 0x13eb, 0x13f0, 0x13c2, 0x13d8, 0x13ef, 0x13c1, 0x13ed, 0x13b3,
0x13d7, 0x13ec, 0x13cd, 0x13d9, 0x13a9, 0x13ef, 0x13ab, 0x13e5, 0x13a7, 0x13b3, 0x13c2, 0x13ab,
0x13c2, 0x13f0, 0x13a7, 0x13d6, 0x13b5, 0x13c2, 0x13ca, 0x13a6, 0x13f0, 0x13b4, 0x13ea, 0x13c2,
0x13a0, 0x13d0, 0x13b4, 0x13cf, 0x13c6, 0x13ca, 0x13d3, 0x13e5, 0x13cd, 0x13c6, 0x13a2, 0x13ac,
0x13be, 0x13d5, 0x13be, 0x20, 0x13a2, 0x13e4, 0x13f3, 0x13cd, 0x13d7, 0x13ea, 0x13b6, 0x13b9,
0x13d5, 0x13b3, 0x13b4, 0x13e3, 0x13eb, 0x13a9, 0x13e7, 0x13a6, 0x13be, 0x13ee, 0x20, 0x13e3,
0x13a0, 0x13e5, 0x13a2, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13ae, 0x13b5, 0x13be, 0x13cd,
0x13d5, 0x13c2, 0x13b5, 0x13a1, 0x13d5, 0x13b4, 0x13d7, 0x13c7, 0x13cd, 0x13c7, 0x13c2, 0x13a4,
0x13f2, 0x13e8, 0x13af, 0x20, 0x13a6, 0x13da, 0x13cf, 0x13d3, 0x13e9, 0x13c2, 0x13f3, 0x13dd,
0x13b0, 0x13c6, 0x13d8, 0x13b4, 0x13be, 0x13d5, 0x13b9, 0x13c2, 0x13a4, 0x13ac, 0x13eb, 0x13f3,
0x13af, 0x20, 0x13ad, 0x13eb, 0x13ba, 0x13b5, 0x13c9, 0x13c1, 0x13c7, 0x13b5, 0x13dd, 0x13cf,
0x13d7, 0x13c2, 0x13a0, 0x13bb, 0x13cd, 0x13d5, 0x13d3, 0x13bb, 0x13a0, 0x13c2, 0x13d9, 0x13e9,
0x13a0, 0x13dd, 0x13a7, 0x13c2, 0x13a0, 0x13d6, 0x13c2, 0x13cf, 0x13c7, 0x13b5, 0x13c7, 0x13d7,
0x13c7, 0x13b5, 0x13c2, 0x13c6, 0x13d8, 0x13cd, 0x13b3, 0x13e9, 0x13cb, 0x13ce, 0x13b5, 0x13cf,
0x13c7, 0x13e3, 0x13b4, 0x13cd, 0x13d7, 0x13ca, 0x13d3, 0x13c7, 0x13cd, 0x13d7, 0x13ca, 0x13cf,
0x13c2, 0x13a8, 0x13c2, 0x13e5, 0x13cf, 0x13be, 0x13eb, 0x13aa, 0x13c7, 0x13c2, 0x13ae, 0x13a8,
0x13c2, 0x13db, 0x13b5, 0x13c2, 0x13e5, 0x13c6, 0x13b5, 0x13d3, 0x13ac, 0x13c2, 0x13cf, 0x13ae,
0x13b5, 0x13cf, 0x13c2, 0x13a9, 0x13a4, 0x13cd, 0x13d7, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x20,
0x13be, 0x13bf, 0x20, 0x13a0, 0x13cd, 0x13a6, 0x13ef, 0x13a2, 0x13cd, 0x13d4, 0x13c2, 0x13ca,
0x13b5, 0x13e8, 0x13b5, 0x13cf, 0x13a7, 0x13b5, 0x13c2, 0x13c6, 0x13d7, 0x13a9, 0x13b6, 0x13e9,
0x13a9, 0x13eb, 0x13b5, 0x13cf, 0x13c9, 0x13c2, 0x13e7, 0x13e3, 0x13be, 0x13b8, 0x13c2, 0x13d3,
0x13c2, 0x13b8, 0x13a7, 0x13ce, 0x13cb, 0x13a9, 0x13b9, 0x13df, 0x13d7, 0x13b9, 0x13b5, 0x13d4,
0x13ba, 0x13b5, 0x13ad, 0x13bb, 0x13ba, 0x13c2, 0x13cd, 0x13a9, 0x13bc, 0x13be, 0x13aa, 0x13b9,
0x13cd, 0x13a6, 0x13eb, 0x13a3, 0x13cf, 0x13b6, 0x13c7, 0x13eb, 0x13cf, 0x13c9, 0x13aa, 0x13b5,
0x13a7, 0x13cc, 0x13b9, 0x13b3, 0x13b9, 0x13b5, 0x13c3, 0x13cc, 0x13b3, 0x13e4, 0x13ec, 0x13cc,
0x13c6, 0x13d9, 0x13a5, 0x13cf, 0x13bb, 0x13ea, 0x13b6, 0x13c9, 0x13b5, 0x13cd, 0x13aa, 0x13e4,
0x13d0, 0x13df, 0x13a0, 0x13cd, 0x13d3, 0x13a9, 0x13b0, 0x13bb, 0x13d4, 0x13b5, 0x13c2, 0x13d8,
0x13b3, 0x13be, 0x13a4, 0x13b5, 0x13ef, 0x13c3, 0x13ec, 0x13cd, 0x13a7, 0x13e9, 0x13da, 0x13cf,
0x13a0, 0x13e5, 0x13b3, 0x13c1, 0x13e0, 0x13eb, 0x13f0, 0x13be, 0x13eb, 0x13b5, 0x13c2, 0x13f4,
0x13cf, 0x13ec, 0x13b6, 0x13aa, 0x13dd, 0x13d7, 0x13d3, 0x13bf, 0x13e9, 0x20, 0x13a4, 0x13aa,
0x13b2, 0x13a9, 0x13cc, 0x13c7, 0x13c8, 0x13d1, 0x13b5, 0x13e5, 0x13a0, 0x13c2, 0x13d4, 0x13be,
0x13be, 0x13b5, 0x13ec, 0x13e3, 0x13aa, 0x13cf, 0x13d3, 0x13c2, 0x13cd, 0x13d3, 0x13f2, 0x13af,
0x13b2, 0x13aa, 0x13aa, 0x13cd, 0x13aa, 0x13bc, 0x13b3, 0x13ac, 0x13b5, 0x13ab, 0x13f0, 0x13b4,
0x13c2, 0x13b9, 0x13ae, 0x13b9, 0x13b5, 0x13d7, 0x13eb, 0x13cd, 0x13bc, 0x13b5, 0x13cf, 0x13a5,
0x13cd, 0x13ba, 0x13ef, 0x13d6, 0x13b4, 0x13f3, 0x13c2, 0x13a0, 0x13c2, 0x13a0, 0x13c8, 0x13a0,
0x13a0, 0x13a6, 0x13b3, 0x13be, 0x13d7, 0x13ca, 0x13a8, 0x13c2, 0x13eb, 0x13b5, 0x13e3, 0x13dd,
0x13bb, 0x13e7, 0x13a9, 0x13e5, 0x13cc, 0x13d5, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x13a1, 0x13e9,
0x13d6, 0x13e9, 0x13a7, 0x13a3, 0x13ec, 0x13eb, 0x13e5, 0x13e1, 0x13be, 0x13e1, 0x13d8, 0x13a1,
0x13c6, 0x20, 0x13d3, 0x13a6, 0x13cf, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x13a6,
0x13bb, 0x13c7, 0x13b5, 0x13c6, 0x13d3, 0x13b5, 0x13a7, 0x13be, 0x13b5, 0x13c6, 0x13bb, 0x13ad,
0x13c3, 0x13b7, 0x13b7, 0x13a1, 0x13c2, 0x13c7, 0x13b5, 0x13a9, 0x13b5, 0x13d8, 0x13b9, 0x13d8,
0x13aa, 0x13cd, 0x13b4, 0x13c6, 0x13e3, 0x13b4, 0x13a2, 0x13c2, 0x13b9, 0x13e7, 0x13b6, 0x13b9,
0x13b5, 0x13a8, 0x13cc, 0x13cf, 0x13a0, 0x13f0, 0x13df, 0x13f4, 0x13da, 0x13c3, 0x13a4, 0x13b7,
0x13c3, 0x13b5, 0x13ec, 0x13b5, 0x13a9, 0x13c3, 0x13a4, 0x13ba, 0x13a0, 0x13c6, 0x13aa, 0x20,
0x13c6, 0x13aa, 0x13c6, 0x13b4, 0x13a0, 0x13eb, 0x13c8, 0x13a7, 0x13b5, 0x13c2, 0x13a4, 0x13c9,
0x13c2, 0x13c7, 0x13e5, 0x13f3, 0x13d7, 0x13d4, 0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x20, 0x13bc,
0x13b5, 0x13cd, 0x13c8, 0x13b3, 0x13b6, 0x13d9, 0x13c2, 0x13a6, 0x13cc, 0x13f1, 0x13c6, 0x13c2,
0x13d4, 0x13af, 0x13d8, 0x13d4, 0x13b3, 0x13e9, 0x13d9, 0x13be, 0x13a6, 0x13d4, 0x13ca, 0x13a4,
0x13f0, 0x13e8, 0x13e9, 0x13b5, 0x13cd, 0x4e9a, 0x7684, 0x65af, 0x4e9a, 0x8d1d, 0x5df4, 0x963f,
0x65af, 0x9a6c, 0x62c9, 0x73ed, 0x73e0, 0x5c14, 0x5e03, 0x743c, 0x5e03, 0x62c9, 0x79d1, 0x7eb3,
0x514b, 0x91cc, 0x8fbe, 0x7d2f, 0x65af, 0x8428, 0x62c9, 0x59c6, 0x5409, 0x5e03, 0x63d0, 0x5f17,
0x91cc, 0x6566, 0x54c8, 0x535a, 0x7f57, 0x5185, 0x7ea6, 0x7ff0, 0x5185, 0x65af, 0x5821, 0x91d1,
0x6c99, 0x8428, 0x62c9, 0x5404, 0x65af, 0x5229, 0x4f2f, 0x7ef4, 0x5c14, 0x5362, 0x8428, 0x5361,
0x9a6c, 0x585e, 0x5362, 0x59c6, 0x5df4, 0x5df4, 0x7eb3, 0x6469, 0x52a0, 0x8fea, 0x6c99, 0x5185,
0x7f57, 0x6bd5, 0x52aa, 0x74e6, 0x514b, 0x8096, 0x7279, 0x6ce2, 0x591a, 0x8bfa, 0x4f0f, 0x57c3,
0x8fbe, 0x514b, 0x5b89, 0x514b, 0x96f7, 0x5947, 0x5b89, 0x572d, 0x62c9, 0x5b89, 0x63d0, 0x74dc,
0x963f, 0x62c9, 0x74dc, 0x4f0a, 0x7eb3, 0x79d1, 0x5c14, 0x591a, 0x74e6, 0x62c9, 0x91cc, 0x5965,
0x54c8, 0x5723, 0x8def, 0x6613, 0x65af, 0x56fe, 0x5e93, 0x66fc, 0x963f, 0x9c81, 0x5df4, 0x5df4,
0x5df4, 0x591a, 0x65af, 0x4f2f, 0x5229, 0x5179, 0x5e03, 0x5170, 0x514b, 0x8428, 0x5e03, 0x9686,
0x535a, 0x963f, 0x7ef4, 0x65af, 0x5854, 0x535a, 0x4f0a, 0x897f, 0x5927, 0x574e, 0x666e, 0x52a0,
0x62c9, 0x52a0, 0x65af, 0x5361, 0x5bb4, 0x5947, 0x74e6, 0x74e6, 0x54e5, 0x65af, 0x8fbe, 0x9ece,
0x52a0, 0x5e93, 0x4e9a, 0x5df4, 0x591a, 0x7c73, 0x5c3c, 0x52a0, 0x57c3, 0x5fb7, 0x8499, 0x987f,
0x4f9d, 0x4f26, 0x5c3c, 0x8d1d, 0x798f, 0x5854, 0x96f7, 0x8428, 0x683c, 0x83b1, 0x65af, 0x8d1d,
0x53e4, 0x65af, 0x6e7e, 0x683c, 0x6797, 0x7eb3, 0x8fbe, 0x74dc, 0x5fb7, 0x7f57, 0x666e, 0x5371,
0x5730, 0x9a6c, 0x62c9, 0x572d, 0x4e9a, 0x90a3, 0x54c8, 0x5229, 0x6cd5, 0x514b, 0x65af, 0x5370,
0x7b2c, 0x5b89, 0x7eb3, 0x6ce2, 0x5229, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x8bfa,
0x514b, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x9a6c, 0x4f26, 0x6208, 0x5370, 0x7b2c,
0x5b89, 0x7eb3, 0x5dde, 0x5f7c, 0x5f97, 0x65af, 0x5821, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde,
0x7279, 0x5c14, 0x57ce, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x7ef4, 0x7ef4, 0x5e02, 0x5370,
0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x6e29, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde,
0x5a01, 0x7eb3, 0x9a6c, 0x514b, 0x4f0a, 0x52aa, 0x7ef4, 0x514b, 0x80af, 0x5854, 0x57fa, 0x5dde,
0x8499, 0x8482, 0x585e, 0x6d1b, 0x9a6c, 0x585e, 0x7ea6, 0x9a6c, 0x90a3, 0x74dc, 0x9a6c, 0x7459,
0x65af, 0x9a6c, 0x63d0, 0x5c3c, 0x514b, 0x6885, 0x8bfa, 0x7c73, 0x5c3c, 0x58a8, 0x897f, 0x54e5,
0x57ce, 0x5bc6, 0x514b, 0x9686, 0x8499, 0x7279, 0x96f7, 0x8499, 0x5f97, 0x7ef4, 0x7684, 0x4e9a,
0x8499, 0x7279, 0x585e, 0x62c9, 0x7279, 0x62ff, 0x9a9a, 0x6d1b, 0x7f57, 0x5c3c, 0x4e9a, 0x5317,
0x8fbe, 0x79d1, 0x4ed6, 0x5dde, 0x6bd4, 0x5c24, 0x62c9, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde,
0x7533, 0x7279, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde, 0x65b0, 0x585e, 0x52d2, 0x59c6, 0x5e15,
0x62c9, 0x9a6c, 0x91cc, 0x535a, 0x6ce2, 0x591a, 0x97e6, 0x67f3, 0x7d2f, 0x897f, 0x8153, 0x96f7,
0x7d22, 0x5362, 0x7279, 0x91cc, 0x5965, 0x5e03, 0x90ce, 0x5e93, 0x5723, 0x5730, 0x4e9a, 0x54e5,
0x5723, 0x591a, 0x660e, 0x5404, 0x65af, 0x79d1, 0x5217, 0x65af, 0x6bd4, 0x6851, 0x5fb7, 0x5723,
0x5df4, 0x6cf0, 0x52d2, 0x7c73, 0x5c9b, 0x5723, 0x7ea6, 0x7ff0, 0x65af, 0x5723, 0x5362, 0x897f,
0x4e9a, 0x5723, 0x6587, 0x68ee, 0x7279, 0x7279, 0x53e4, 0x897f, 0x52a0, 0x5c14, 0x5df4, 0x56fe,
0x52d2, 0x8482, 0x534e, 0x7eb3, 0x4e9a, 0x5e93, 0x5854, 0x7279, 0x5361, 0x585e, 0x8fea, 0x8499,
0xb7, 0x8fea, 0x7ef4, 0x5c14, 0x9ea6, 0x683c, 0x7406, 0x5e15, 0x5c14, 0x9ed8, 0x7279, 0x7f57,
0x5c14, 0x6717, 0x4f0a, 0x5c14, 0x57ce, 0x963f, 0x7eb3, 0x5fb7, 0x5c14, 0x963f, 0x514b, 0x6258,
0x522b, 0x963f, 0x4ec0, 0x54c8, 0x5df4, 0x5fb7, 0x6587, 0x83b1, 0x79d1, 0x4f26, 0x5761, 0x8fea,
0x62dc, 0x675c, 0x5c1a, 0x522b, 0x52a0, 0x6c99, 0x5e0c, 0x4f2f, 0x4f26, 0x4f0a, 0x5c14, 0x5e93,
0x8328, 0x514b, 0x67e5, 0x4e9a, 0x666e, 0x62c9, 0x5361, 0x62c9, 0x5947, 0x6c49, 0x5fb7, 0x52a0,
0x671b, 0x52a0, 0x9521, 0x9a6c, 0x65af, 0x5580, 0x7279, 0x5c3c, 0x79d1, 0x897f, 0x4e9a, 0x5361,
0x5854, 0x5c14, 0x5e93, 0x65af, 0x5854, 0x5948, 0x514b, 0x5b5c, 0x6d1b, 0x5c14, 0x8fbe, 0x5229,
0x96c5, 0x5f97, 0x8428, 0x54c8, 0x6797, 0x4e07, 0x8c61, 0x7b26, 0x62c9, 0x8fea, 0x6c83, 0x65af,
0x6258, 0x514b, 0x96c5, 0x5e93, 0x8328, 0x514b, 0x53f6, 0x5361, 0x6377, 0x7433, 0x5821, 0x57c3,
0x91cc, 0x6e29, 0x767e, 0x6155, 0x5927, 0x52a0, 0x90a3, 0x5229, 0x4f5b, 0x5f97, 0x89d2, 0x9a6c,
0x5fb7, 0x62c9, 0x96f7, 0x514b, 0x96c5, 0x672a, 0x514b, 0x65af, 0x5766, 0x5229, 0x963f, 0x5fb7,
0x83b1, 0x5fb7, 0x5e03, 0x91cc, 0x65af, 0x73ed, 0x970d, 0x5df4, 0x7279, 0x73c0, 0x65af, 0x6089,
0x5c3c, 0x8d1d, 0x5c14, 0x683c, 0x83b1, 0x5fb7, 0x5e03, 0x62c9, 0x8fea, 0x65af, 0x62c9, 0x53d1,
0x57fa, 0x5e0c, 0x8bb7, 0x4e4c, 0x6839, 0x897f, 0x5c9b, 0x9a6c, 0x6069, 0x5c9b, 0x4f0a, 0x65af,
0x5766, 0x5e03, 0x5c14, 0x5362, 0x5e03, 0x5c14, 0x96c5, 0x90a3, 0x9a6c, 0x8033, 0x4ed6, 0x6ce2,
0x5fb7, 0x6208, 0x91cc, 0x5bdf, 0x8428, 0x9a6c, 0x62c9, 0x5723, 0x9a6c, 0x529b, 0x8bfa, 0x8428,
0x62c9, 0x70ed, 0x7a9d, 0x8f9b, 0x83f2, 0x7f57, 0x6ce2, 0x5c14, 0x65af, 0x79d1, 0x666e, 0x91cc,
0x7d22, 0x975e, 0x4e9a, 0x74e6, 0x675c, 0x5179, 0x8428, 0x683c, 0x52d2, 0x5e03, 0x53ef, 0x53ef,
0x65af, 0x9a6c, 0x5c14, 0x4ee3, 0x592b, 0x6bdb, 0x91cc, 0x6c42, 0x65af, 0x7559, 0x5c3c, 0x6c6a,
0x67e5, 0x5854, 0x59c6, 0x7279, 0x9c81, 0x514b, 0x7fa4, 0x5c9b, 0x590d, 0x6d3b, 0x8282, 0x5c9b,
0x5bcc, 0x7eb3, 0x5bcc, 0x63d0, 0x79d1, 0x9686, 0x7fa4, 0x5c9b, 0x74dc, 0x8fbe, 0x5c14, 0x5361,
0x7eb3, 0x5c14, 0x6069, 0x5fb7, 0x4f2f, 0x91cc, 0x5e93, 0x8d5b, 0x57c3, 0x5938, 0x8d3e, 0x6797,
0x9a6c, 0x6731, 0x7f57, 0x9a6c, 0x514b, 0x8428, 0x65af, 0x7459, 0x9c81, 0x52aa, 0x7f8e, 0x963f,
0x5e15, 0x679c, 0x5e15, 0x679c, 0x5e15, 0x52b3, 0x76ae, 0x7279, 0x51ef, 0x6069, 0x6ce2, 0x7eb3,
0x4f69, 0x5c9b, 0x83ab, 0x5c14, 0x5179, 0x6bd4, 0x6e2f, 0x5854, 0x5e0c, 0x63d0, 0x65af, 0x8003,
0x65af, 0x4f2f, 0x677e, 0x5fb7, 0x963f, 0x6bd4, 0x8d0a, 0x963f, 0x65af, 0x99ac, 0x62c9, 0x73ed,
0x57fa, 0x79d1, 0x7d0d, 0x514b, 0x91cc, 0x9054, 0x7d2f, 0x65af, 0x85a9, 0x62c9, 0x59c6, 0x7d04,
0x7ff0, 0x5167, 0x65af, 0x5821, 0x91d1, 0x6c99, 0x85a9, 0x5229, 0x5e03, 0x7dad, 0x76e7, 0x5b89,
0x9054, 0x76e7, 0x85a9, 0x5361, 0x99ac, 0x585e, 0x9b6f, 0x5167, 0x7f85, 0x7562, 0x6eab, 0x7279,
0x548c, 0x514b, 0x57c3, 0x9054, 0x514b, 0x963f, 0x62c9, 0x74dc, 0x4f0a, 0x7d0d, 0x79d1, 0x723e,
0x591a, 0x74e6, 0x62c9, 0x91cc, 0x5967, 0x54c8, 0x963f, 0x9b6f, 0x5df4, 0x963f, 0x677e, 0x68ee,
0x5df4, 0x5e0c, 0x96c5, 0x5df4, 0x4f0a, 0x4e9e, 0x5fb7, 0x73ed, 0x5fb7, 0x62c9, 0x65af, 0x4f2f,
0x5229, 0x8332, 0x535a, 0x963f, 0x7dad, 0x65af, 0x5854, 0x829d, 0x5a03, 0x5a03, 0x54e5, 0x65af,
0x9054, 0x9ece, 0x52a0, 0x5eab, 0x4e9e, 0x5df4, 0x9053, 0x68ee, 0x7063, 0x611b, 0x6c11, 0x9813,
0x798f, 0x5854, 0x840a, 0x85a9, 0x683c, 0x840a, 0x65af, 0x7063, 0x683c, 0x6797, 0x7d0d, 0x9054,
0x74dc, 0x5fb7, 0x7f85, 0x666e, 0x5371, 0x5730, 0x99ac, 0x62c9, 0x572d, 0x4e9e, 0x90a3, 0x590f,
0x7063, 0x62ff, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x6ce2, 0x5229, 0x65af, 0x5370, 0x7b2c, 0x5b89,
0x7d0d, 0x5dde, 0x8afe, 0x514b, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x99ac, 0x502b,
0x54e5, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x5f7c, 0x5f97, 0x5821, 0x5370, 0x7b2c, 0x5b89,
0x7d0d, 0x5dde, 0x7279, 0x723e, 0x57ce, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x97cb, 0x97cb,
0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x6eab, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7d0d,
0x5dde, 0x5a01, 0x7d0d, 0x99ac, 0x514b, 0x4f0a, 0x52aa, 0x7dad, 0x514b, 0x8def, 0x6613, 0x7dad,
0x723e, 0x99ac, 0x585e, 0x7d04, 0x99ac, 0x90a3, 0x74dc, 0x99ac, 0x63d0, 0x5c3c, 0x514b, 0x6885,
0x8afe, 0x7c73, 0x5c3c, 0x8499, 0x5f97, 0x7dad, 0x7684, 0x4e9e, 0x8499, 0x585e, 0x62c9, 0x7279,
0x5cf6, 0x62ff, 0x9a37, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x5317, 0x9054, 0x79d1, 0x4ed6, 0x5dde,
0x6bd4, 0x5c24, 0x62c9, 0x5317, 0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x4e2d, 0x5fc3, 0x5e02, 0x5317,
0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x65b0, 0x85a9, 0x52d2, 0x59c6, 0x97cb, 0x67f3, 0x6e2f, 0x91cc,
0x7d04, 0x5e03, 0x862d, 0x79d1, 0x8056, 0x5730, 0x4e9e, 0x54e5, 0x8056, 0x591a, 0x660e, 0x5404,
0x8056, 0x5df4, 0x6cf0, 0x52d2, 0x7c73, 0x8056, 0x7d04, 0x7ff0, 0x65af, 0x8056, 0x76e7, 0x897f,
0x4e9e, 0x8056, 0x6587, 0x68ee, 0x7279, 0x7279, 0x53e4, 0x897f, 0x52a0, 0x723e, 0x5df4, 0x5716,
0x52d2, 0x8482, 0x83ef, 0x7d0d, 0x767d, 0x99ac, 0x5e02, 0x4e9e, 0x5eab, 0x5854, 0x7279, 0x51f1,
0x897f, 0x7ad9, 0x6234, 0x7dad, 0x65af, 0x7ad9, 0x675c, 0x8499, 0x8fea, 0x7dad, 0x723e, 0x7ad9,
0x9ea5, 0x5938, 0x91cc, 0x83ab, 0x68ee, 0x7ad9, 0x9ea5, 0x514b, 0x9ed8, 0x591a, 0x7ad9, 0x5e15,
0x723e, 0x9ed8, 0x7f85, 0x745f, 0x62c9, 0x7ad9, 0x662d, 0x548c, 0x7ad9, 0x7279, 0x7f85, 0x723e,
0x7ad9, 0x6717, 0x4f0a, 0x723e, 0x57ce, 0x963f, 0x7d0d, 0x5fb7, 0x723e, 0x6bd4, 0x65af, 0x51f1,
0x514b, 0x79d1, 0x502b, 0x5761, 0x675c, 0x5c1a, 0x5225, 0x5e0c, 0x4f2f, 0x4f96, 0x4f0a, 0x723e,
0x5eab, 0x8328, 0x514b, 0x67e5, 0x96c5, 0x666e, 0x62c9, 0x932b, 0x6c5f, 0x99ac, 0x65af, 0x5580,
0x7279, 0x5c3c, 0x79d1, 0x897f, 0x4e9e, 0x5361, 0x5854, 0x723e, 0x96c5, 0x5eab, 0x8328, 0x514b,
0x8449, 0x5361, 0x6377, 0x7433, 0x5821, 0x99ac, 0x5fb7, 0x62c9, 0x5cf6, 0x53f2, 0x4e39, 0x5229,
0x963f, 0x5fb7, 0x840a, 0x5fb7, 0x5e03, 0x91cc, 0x65af, 0x672c, 0x8377, 0x4f2f, 0x7279, 0x8c9d,
0x723e, 0x683c, 0x840a, 0x5fb7, 0x4f2f, 0x62c9, 0x7b2c, 0x65af, 0x62c9, 0x74e6, 0x57fa, 0x5e0c,
0x8a25, 0x70cf, 0x6839, 0x897f, 0x5cf6, 0x99ac, 0x6069, 0x5cf6, 0x76e7, 0x5e03, 0x723e, 0x96c5,
0x90a3, 0x99ac, 0x8033, 0x4ed6, 0x85a9, 0x62c9, 0x71b1, 0x7aa9, 0x83ef, 0x675c, 0x8332, 0x85a9,
0x683c, 0x52d2, 0x5e03, 0x67e5, 0x6208, 0x65af, 0x7fa4, 0x5cf6, 0x53ef, 0x53ef, 0x65af, 0x7fa4,
0x5cf6, 0x51f1, 0x723e, 0x84cb, 0x6717, 0x7fa4, 0x5cf6, 0x99ac, 0x723e, 0x4ee3, 0x592b, 0x6bdb,
0x91cc, 0x88d8, 0x65af, 0x5e03, 0x5e72, 0x7dad, 0x723e, 0x5cf6, 0x6cd5, 0x8003, 0x798f, 0x74b0,
0x7901, 0x74dc, 0x9054, 0x723e, 0x5361, 0x7d0d, 0x723e, 0x5cf6, 0x6069, 0x5fb7, 0x4f2f, 0x91cc,
0x5cf6, 0x79d1, 0x65af, 0x96f7, 0x74dc, 0x52a0, 0x6797, 0x74b0, 0x7901, 0x99ac, 0x4e45, 0x7f85,
0x99ac, 0x514b, 0x85a9, 0x65af, 0x7fa4, 0x5cf6, 0x7459, 0x9b6f, 0x76ae, 0x7279, 0x5eb7, 0x7fa4,
0x5cf6, 0x6e6f, 0x52a0, 0x5854, 0x5e03, 0x5cf6, 0x5a01, 0x514b, 0x5cf6, 0x5b89, 0x594e, 0x62c9,
0x8377, 0x5c6c, 0x963f, 0x9b6f, 0x5df4, 0x591a, 0x7c73, 0x5c3c, 0x514b, 0x5eab, 0x65af, 0x5854,
0x5948, 0x57fa, 0x897f, 0x7d10, 0x5b89, 0x5854, 0x90a3, 0x90a3, 0x5229, 0x5f17, 0x430, 0x486,
0x43b, 0x43c, 0x430, 0x442, 0x44b, 0x300, 0x430, 0x486, 0x43d, 0x430, 0x301, 0x434,
0x44b, 0x440, 0x44c, 0x430, 0x486, 0x43a, 0x442, 0x430, 0x301, 0xa64b, 0x430, 0x486,
0x43a, 0x442, 0x44e, 0x301, 0x431, 0x438, 0x43d, 0x441, 0x43a, 0x44a, 0x431, 0x438,
0x448, 0x43a, 0x435, 0x301, 0x43a, 0x44a, 0x447, 0x438, 0x442, 0x430, 0x300, 0x438,
0x486, 0x440, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x44a, 0x43f, 0x435, 0x442,
0x440, 0x43e, 0x43f, 0x430, 0x301, 0x475, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x44a,
0x5f, 0x43a, 0x430, 0x43c, 0x447, 0x430, 0x301, 0x442, 0x441, 0x43a, 0x457, 0x439,
0x445, 0x430, 0x301, 0x43d, 0x434, 0x44b, 0x433, 0x430, 0x43a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x467, 0x301, 0x440, 0x441, 0x43a, 0x44a, 0x43c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x301, 0x43d, 0x44a, 0x43d, 0x43e, 0x432, 0x43e, 0x43a, 0xa64b, 0x437,
0x43d, 0x435, 0x301, 0x446, 0x43a, 0x44a, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x301, 0x440, 0x441, 0x43a, 0x44a, 0x47b, 0x486, 0x301, 0x43c, 0x441,
0x43a, 0x44a, 0x1c82, 0x443, 0x486, 0x440, 0x430, 0x301, 0x43b, 0x44c, 0x441, 0x43a,
0x44a, 0x43a, 0x44b, 0x437, 0x44b, 0x43b, 0x47b, 0x440, 0x434, 0x430, 0x300, 0x441,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x301, 0x43d, 0x44a, 0x441, 0x440, 0x435, 0x434,
0x43d, 0x435, 0x43a, 0x43e, 0x43b, 0x44b, 0x301, 0x43c, 0x441, 0x43a, 0x44a, 0x1c82,
0x443, 0x486, 0x301, 0x441, 0x442, 0x44c, 0x5f, 0x43d, 0x435, 0x301, 0x440, 0x430,
0x432, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x301, 0x43a,
0x44a, 0xa657, 0x486, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x44a, 0x454, 0x486,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x457, 0x43d, 0x431, 0xa64b, 0x301, 0x440, 0x433,
0x44a, 0x43a, 0x430, 0x43b, 0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x301,
0x434, 0x44a, 0x43a, 0x456, 0x301, 0x435, 0x432, 0x44a, 0x43c, 0x438, 0x301, 0x43d,
0x441, 0x43a, 0x44a, 0x43c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x300, 0x441, 0x430,
0x43c, 0x430, 0x301, 0x440, 0x430, 0x441, 0x477, 0x43c, 0x444, 0x435, 0x440, 0xa64b,
0x301, 0x43f, 0x43e, 0x43b, 0x44c, 0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x301, 0x434, 0x44a, 0x410, 0x441, 0x43c, 0x44d, 0x440, 0x430, 0x421, 0x430,
0x43d, 0x2d, 0x422, 0x43e, 0x43c, 0x435, 0x20, 0x421, 0x430, 0x43d, 0x2d, 0x422,
0x43e, 0x43c, 0x435, 0x421, 0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x438, 0x441, 0x20,
0x421, 0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x438, 0x441, 0x41a, 0x430, 0x439, 0x43c,
0x430, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c,
0x421, 0x44c, 0x44e, 0x434, 0x430, 0x434, 0x2d, 0x425, 0x443, 0x430, 0x440, 0x435,
0x441, 0x422, 0x435, 0x43b, 0x43b, 0x2d, 0x421, 0x438, 0x442, 0x438, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x435, 0x432, 0x435, 0x439,
0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d,
0x441, 0x435, 0x43d, 0x43d, 0x435, 0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x430, 0x43d, 0x430, 0x423, 0x438, 0x43d, 0x430, 0x43c, 0x430, 0x43a, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x41b, 0x43e, 0x443, 0x44d, 0x440,
0x2d, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x2d, 0x41a, 0x443, 0x43e, 0x442, 0x435,
0x440, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x411, 0x43e, 0x439, 0x43b,
0x430, 0x2c, 0x20, 0x4aa, 0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20, 0x414, 0x430,
0x43a, 0x43e, 0x442, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x2c, 0x20, 0x4aa,
0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430,
0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x435, 0x439, 0x43b, 0x435, 0x43c, 0x2c, 0x20,
0x4aa, 0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442,
0x430, 0x420, 0x435, 0x437, 0x43e, 0x43b, 0x44c, 0x44e, 0x442, 0x421, 0x430, 0x43d,
0x2d, 0x41f, 0x430, 0x443, 0x43b, 0x443, 0x20, 0x421, 0x430, 0x43d, 0x2d, 0x41f,
0x430, 0x443, 0x43b, 0x443, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41b, 0x44e, 0x441,
0x438, 0x44f, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e, 0x440,
0x432, 0x438, 0x43b, 0x44c, 0x20, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434,
0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x44c, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x42f, 0x43d, 0x433,
0x43e, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432,
0x4d7, 0x441, 0x435, 0x43c, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x20, 0x443,
0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x41a, 0x430, 0x43d, 0x430,
0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x424,
0x430, 0x440, 0x435, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441,
0x435, 0x43c, 0x41a, 0x4d1, 0x43d, 0x442, 0x4d1, 0x440, 0x20, 0x413, 0x435, 0x43e,
0x440, 0x433, 0x438, 0x421, 0x4d1, 0x432, 0x430, 0x43f, 0x43b, 0x4d1, 0x20, 0x415,
0x43b, 0x435, 0x43d, 0x430, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x410,
0x444, 0x438, 0x43d, 0x441, 0x435, 0x43c, 0x41c, 0x44d, 0x43d, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x41c, 0x443, 0x441, 0x43a, 0x430, 0x432, 0x420, 0x430,
0x448, 0x442, 0x430, 0x432, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x41a,
0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441,
0x435, 0x43c, 0x41a, 0x43e, 0x43c, 0x43e, 0x440, 0x441, 0x435, 0x43c, 0x41c, 0x430,
0x43b, 0x44c, 0x434, 0x438, 0x432, 0x441, 0x435, 0x43c, 0x41c, 0x430, 0x432, 0x440,
0x438, 0x43a, 0x438, 0x439, 0x41c, 0x430, 0x439, 0x43e, 0x442, 0x442, 0x430, 0x41c,
0x4d1, 0x43d, 0x43a, 0x443, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7,
0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x413, 0x430, 0x43c, 0x431, 0x44c,
0x435, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x41c,
0x430, 0x440, 0x43a, 0x438, 0x437, 0x430, 0x441, 0x20, 0x443, 0x442, 0x440, 0x430,
0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d,
0x4b, 0x61, 0x69, 0x6a, 0x72, 0x6f, 0x4c, 0x61, 0x61, 0x6a, 0x6f, 0x73,
0x57, 0x69, 0x6e, 0x64, 0x68, 0x75, 0x6b, 0x4b, 0x6f, 0xdf, 0x74, 0x61,
0x72, 0x69, 0x63, 0x6b, 0x61, 0x4a, 0x75, 0x61, 0x79, 0x61, 0x71, 0x75,
0x69, 0x6c, 0x4b, 0x6e, 0x6f, 0x78, 0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f,
0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50,
0x65, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x20, 0x65, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x6c,
0x20, 0x43, 0x69, 0x74, 0x79, 0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x56, 0x65, 0x76, 0x61, 0x79, 0x20, 0x65, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x63,
0x65, 0x6e, 0x6e, 0x65, 0x73, 0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63, 0x20,
0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x6f,
0x6e, 0x74, 0x69, 0x63, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x65, 0x6e, 0x20,
0x4b, 0x65, 0x6e, 0x74, 0x75, 0x63, 0x6b, 0x79, 0x53, 0x63, 0x68, 0x74,
0x61, 0x64, 0x74, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x20, 0x65, 0x6e, 0x20, 0x4e, 0x6f, 0x6f, 0x64,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x5a, 0x65, 0x6e, 0x74, 0x65,
0x72, 0x20, 0x65, 0x6e, 0x20, 0x4e, 0x6f, 0x6f, 0x64, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x75, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x20, 0x65, 0x6e, 0x20, 0x4e, 0x6f, 0x6f, 0x64, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x5a, 0x69, 0x6e, 0x74, 0x20, 0x42, 0x61, 0x72,
0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x5a, 0x69, 0x6e, 0x74, 0x20,
0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73, 0x5a, 0x69, 0x6e, 0x74, 0x20, 0x4b,
0x69, 0x74, 0x74, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75,
0x74, 0x73, 0x63, 0x68, 0x69, 0x69, 0x6a, 0x61, 0x5a, 0x69, 0x6e, 0x74,
0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x5a, 0x69, 0x6e, 0x74, 0x20,
0x56, 0x69, 0x6e, 0x74, 0x73, 0x63, 0x68, 0xe4, 0x6e, 0x74, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x7a, 0x6a, 0x6f, 0x68,
0x6e, 0x53, 0x63, 0x68, 0x6f, 0x6f, 0x77, 0x61, 0x2d, 0x53, 0x63, 0x68,
0x74, 0x61, 0x7a, 0x6a, 0x6f, 0x68, 0x6e, 0x20, 0x6f, 0x70, 0x20, 0x64,
0xe4, 0x20, 0x45, 0x6e, 0x73, 0x65, 0x6c, 0x20, 0x4f, 0x6e, 0x6a, 0x75,
0x6c, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x41, 0x73, 0x73, 0x68, 0x67,
0x61, 0x62, 0x61, 0x74, 0x42, 0x69, 0x73, 0x63, 0x68, 0x6b, 0x65, 0x6b,
0x44, 0x75, 0x73, 0x63, 0x68, 0x61, 0x6e, 0x62, 0x65, 0x4a, 0x61, 0x61,
0x73, 0x61, 0x53, 0x69, 0x67, 0x6f, 0x6e, 0x67, 0x20, 0x28, 0x48, 0x6f,
0x2d, 0x54, 0x73, 0x63, 0x68, 0x69, 0x2d, 0x4d, 0x69, 0x6e, 0x68, 0x2d,
0x53, 0x63, 0x68, 0x74, 0x61, 0x64, 0x74, 0x29, 0x4b, 0x61, 0x6d, 0x73,
0x63, 0x68, 0x61, 0x74, 0x74, 0x6b, 0x61, 0x4e, 0x69, 0x6b, 0x6f, 0x73,
0x69, 0x6a, 0x61, 0x50, 0x6e, 0x6f, 0x6d, 0x20, 0x50, 0x65, 0x6e, 0x68,
0x51, 0x79, 0x73, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x53, 0x63, 0x68,
0x61, 0x6e, 0x67, 0x68, 0x61, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x6a, 0x61,
0x70, 0x75, 0x75, 0x72, 0x54, 0x61, 0x73, 0x63, 0x68, 0x6b, 0x65, 0x6e,
0x74, 0x54, 0x69, 0x62, 0x6c, 0x69, 0x73, 0x55, 0x72, 0x72, 0x75, 0x6d,
0x70, 0x74, 0x73, 0x63, 0x68, 0x69, 0x52, 0x61, 0x6e, 0x6a, 0x75, 0x6e,
0x45, 0x72, 0x69, 0x77, 0x61, 0x6e, 0x41, 0x7a, 0x6f, 0x6f, 0x72, 0x65,
0x4b, 0x61, 0x6e, 0x6e, 0x61, 0x61, 0x72, 0x65, 0x4b, 0x61, 0x70, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x46, 0xe4, 0x72, 0xf6, 0x72, 0x5a, 0x69,
0x6e, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x65, 0x6c,
0x6a, 0x72, 0x61, 0x64, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x72,
0x65, 0x6e, 0x4a, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x61, 0x4a,
0xf6, 0xf6, 0x6e, 0x73, 0x65, 0x69, 0x45, 0x6e, 0x73, 0x65, 0x6c, 0x20,
0x4d, 0xe4, 0x6e, 0x49, 0x73, 0x74, 0x61, 0x6d, 0x62, 0x75, 0x6c, 0x4a,
0xf6, 0xf6, 0x73, 0x65, 0x69, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x6e, 0x69,
0x6e, 0x6a, 0x72, 0x61, 0x61, 0x64, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62,
0x75, 0x72, 0x73, 0x63, 0x68, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x52,
0x69, 0x69, 0x6a, 0x61, 0x54, 0x69, 0x72, 0x61, 0x61, 0x6e, 0x61, 0x64,
0x65, 0x72, 0x20, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x68, 0x6e, 0x57,
0x69, 0x65, 0x6e, 0x57, 0x6f, 0x6c, 0x6a, 0x6f, 0x6a, 0x72, 0x61, 0x64,
0x57, 0x61, 0x72, 0x73, 0x63, 0x68, 0x61, 0x75, 0x53, 0x61, 0x67, 0x72,
0x65, 0x62, 0x5a, 0xfc, 0x72, 0x65, 0x73, 0x63, 0x68, 0x4d, 0x61, 0x6c,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x4a, 0x61, 0x6c, 0x61, 0x70, 0x70, 0x61,
0x6a, 0x6f, 0x73, 0x4a, 0x75, 0x61, 0x6d, 0x48, 0x6f, 0x6e, 0x75, 0x6c,
0x75, 0x6c, 0x75, 0x41, 0x6c, 0x17e, 0x69, 0x72, 0x47, 0x76, 0x61, 0x6a,
0x61, 0x6e, 0x61, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x64, 0x65,
0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x41, 0x6c, 0x6d, 0x61, 0x20,
0x41, 0x74, 0x61, 0x41, 0x161, 0x67, 0x61, 0x62, 0x61, 0x74, 0x10c, 0x69,
0x74, 0x61, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x69, 0x46, 0x65, 0x72,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x63, 0x69, 0x4a,
0x75, 0x17e, 0x6e, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x6a,
0x61, 0x42, 0x72, 0x75, 0x78, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x4f, 0x74,
0x6f, 0x6b, 0x20, 0x4d, 0x61, 0x6e, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x17e,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a, 0x65,
0x41, 0x62, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x41, 0x6c, 0x17e, 0xed, 0x72,
0x4b, 0xe1, 0x68, 0x69, 0x72, 0x61, 0x43, 0x68, 0x61, 0x72, 0x74, 0xfa,
0x6d, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x161, 0x75, 0x4e, 0x64, 0x17e,
0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x75, 0x61, 0x6b, 0x161, 0x6f, 0x74,
0x74, 0x53, 0x76, 0x61, 0x74, 0xfd, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x161,
0x54, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4b, 0x61, 0x6a, 0x6d,
0x61, 0x6e, 0x73, 0x6b, 0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x79, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x53, 0x65, 0x76,
0x65, 0x72, 0x6e, 0xed, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43,
0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72,
0x6e, 0xed, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77,
0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65,
0x72, 0x6e, 0xed, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x76,
0x61, 0x74, 0xfd, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d,
0x11b, 0x6a, 0x53, 0x76, 0x61, 0x74, 0xfd, 0x20, 0x4b, 0x72, 0x79, 0x161,
0x74, 0x6f, 0x66, 0x53, 0x76, 0x61, 0x74, 0xe1, 0x20, 0x4c, 0x75, 0x63,
0x69, 0x65, 0x53, 0x76, 0x61, 0x74, 0xfd, 0x20, 0x54, 0x6f, 0x6d, 0xe1,
0x161, 0x20, 0x28, 0x4b, 0x61, 0x72, 0x69, 0x62, 0x69, 0x6b, 0x29, 0x53,
0x76, 0x61, 0x74, 0xfd, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x63,
0x41, 0x6d, 0x6d, 0xe1, 0x6e, 0x41, 0x161, 0x63, 0x68, 0x61, 0x62, 0x61,
0x64, 0x42, 0x61, 0x67, 0x64, 0xe1, 0x64, 0x42, 0x61, 0x68, 0x72, 0x61,
0x6a, 0x6e, 0x42, 0x65, 0x6a, 0x72, 0xfa, 0x74, 0x44, 0x61, 0x6d, 0x61,
0x161, 0x65, 0x6b, 0x44, 0x68, 0xe1, 0x6b, 0x61, 0x44, 0x75, 0x62, 0x61,
0x6a, 0x48, 0x6f, 0x20, 0x10c, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x6f, 0x76,
0x6f, 0x20, 0x6d, 0x11b, 0x73, 0x74, 0x6f, 0x4a, 0x65, 0x72, 0x75, 0x7a,
0x61, 0x6c, 0xe9, 0x6d, 0x4b, 0xe1, 0x62, 0x75, 0x6c, 0x4b, 0x61, 0x72,
0xe1, 0x10d, 0xed, 0x4b, 0xe1, 0x74, 0x68, 0x6d, 0xe1, 0x6e, 0x64, 0xfa,
0x43, 0x68, 0x61, 0x6e, 0x64, 0x79, 0x67, 0x61, 0x4b, 0x61, 0x6c, 0x6b,
0x61, 0x74, 0x61, 0x4e, 0x69, 0x6b, 0xf3, 0x73, 0x69, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x11b, 0x63, 0x6b, 0x55, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x50, 0x68, 0x6e, 0x6f, 0x6d, 0x70, 0x65, 0x6e, 0x68,
0x50, 0x63, 0x68, 0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x52,
0x69, 0x6a, 0xe1, 0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x53, 0x6f, 0x75, 0x6c, 0x160, 0x61, 0x6e, 0x67, 0x68, 0x61, 0x6a, 0x53,
0x72, 0x65, 0x64, 0x6e, 0x11b, 0x6b, 0x6f, 0x6c, 0x79, 0x6d, 0x73, 0x6b,
0x54, 0x63, 0x68, 0x61, 0x6a, 0x2d, 0x70, 0x65, 0x6a, 0x54, 0x65, 0x68,
0x65, 0x72, 0xe1, 0x6e, 0x54, 0x68, 0x69, 0x6d, 0x62, 0xfa, 0x55, 0x6c,
0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x11b, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x7a, 0x6f, 0x72,
0x73, 0x6b, 0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x42,
0x65, 0x72, 0x6d, 0x75, 0x64, 0x79, 0x4b, 0x61, 0x6e, 0xe1, 0x72, 0x73,
0x6b, 0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x4b, 0x61,
0x70, 0x76, 0x65, 0x72, 0x64, 0x79, 0x46, 0x61, 0x65, 0x72, 0x73, 0x6b,
0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x4a, 0x69, 0x17e,
0x6e, 0xed, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x53, 0x76,
0x61, 0x74, 0xe1, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x73,
0x74, 0x72, 0x61, 0x63, 0x68, 0x61, 0x148, 0x41, 0x74, 0x68, 0xe9, 0x6e,
0x79, 0x42, 0x11b, 0x6c, 0x65, 0x68, 0x72, 0x61, 0x64, 0x42, 0x75, 0x6b,
0x75, 0x72, 0x65, 0x161, 0x165, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x161,
0x165, 0x4b, 0x69, 0x161, 0x69, 0x6e, 0x11b, 0x76, 0x4b, 0x6f, 0x64, 0x61,
0x148, 0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x79, 0x4f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x20, 0x4d, 0x61, 0x6e, 0x4b, 0x79, 0x6a, 0x65, 0x76,
0x4c, 0x75, 0x62, 0x6c, 0x61, 0x148, 0x4c, 0x6f, 0x6e, 0x64, 0xfd, 0x6e,
0x4c, 0x75, 0x63, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x6b, 0x50, 0x61, 0x159,
0xed, 0x17e, 0x158, 0xed, 0x6d, 0x53, 0x6f, 0x66, 0x69, 0x65, 0x56, 0x61,
0x74, 0x69, 0x6b, 0xe1, 0x6e, 0x56, 0xed, 0x64, 0x65, 0x148, 0x5a, 0xe1,
0x68, 0x159, 0x65, 0x62, 0x43, 0x75, 0x72, 0x79, 0x63, 0x68, 0x56, 0xe1,
0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0xe9, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x79, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x79, 0x4b, 0x65,
0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x76, 0x79, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69,
0x76, 0x79, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x75, 0x73, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xe9, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x79, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x6b,
0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x56, 0x65, 0x6c,
0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x47, 0x61, 0x6c, 0x61, 0x70, 0xe1, 0x67, 0x79, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x6f, 0x76, 0x79, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x79, 0x4d, 0x61, 0x72, 0x6b, 0xe9, 0x7a, 0x79,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x76, 0x79, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x41, 0x6c, 0x67, 0x69, 0x65,
0x72, 0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0xf8, 0x65, 0x72, 0x6e, 0x65,
0x41, 0x73, 0x6a, 0x6b, 0x68, 0x61, 0x62, 0x61, 0x64, 0x44, 0x75, 0x73,
0x6a, 0x61, 0x6e, 0x62, 0x65, 0x4b, 0x61, 0x6d, 0x74, 0x6a, 0x61, 0x74,
0x6b, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x73, 0x6e, 0x65, 0x74,
0x73, 0x6b, 0x54, 0x61, 0x73, 0x6a, 0x6b, 0x65, 0x6e, 0x74, 0x52, 0x61,
0x6e, 0x67, 0x6f, 0x6f, 0x6e, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x72, 0x6e,
0x65, 0x44, 0x65, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x73, 0x6b,
0x65, 0x20, 0xd8, 0x65, 0x72, 0x46, 0xe6, 0x72, 0xf8, 0x65, 0x72, 0x6e,
0x65, 0x4b, 0xf8, 0x62, 0x65, 0x6e, 0x68, 0x61, 0x76, 0x6e, 0x56, 0x61,
0x74, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x74, 0x57, 0x61, 0x72, 0x73, 0x7a,
0x61, 0x77, 0x61, 0x4a, 0x75, 0x6c, 0x65, 0xf8, 0x65, 0x6e, 0x43, 0x6f,
0x6d, 0x6f, 0x72, 0x65, 0x72, 0x6e, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x65, 0x72, 0x6e, 0x65, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x65,
0x6e, 0x43, 0x61, 0xef, 0x72, 0x6f, 0x4b, 0x68, 0x61, 0x72, 0x74, 0x6f,
0x65, 0x6d, 0x53, 0x61, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x42, 0x65,
0x6e, 0x65, 0x64, 0x65, 0x6e, 0x20, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x65,
0x6e, 0x20, 0x4b, 0x77, 0x61, 0x72, 0x74, 0x69, 0x65, 0x72, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x6f, 0x2d, 0x53, 0x74, 0x61, 0x64, 0x41, 0x6c, 0x6d,
0x61, 0x2d, 0x41, 0x74, 0x61, 0x41, 0x74, 0x131, 0x72, 0x61, 0x77, 0x44,
0x6f, 0x65, 0x73, 0x6a, 0x61, 0x6e, 0x62, 0x65, 0x48, 0x6f, 0x20, 0x43,
0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x73, 0x74, 0x61, 0x64, 0x4d,
0x61, 0x6e, 0x69, 0x6c, 0x6c, 0x61, 0x53, 0x6a, 0x61, 0x6e, 0x67, 0x68,
0x61, 0x69, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x65,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x43,
0x61, 0x6e, 0x61, 0x72, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x45, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x4b, 0x61, 0x61, 0x70, 0x76, 0x65,
0x72, 0x64, 0x69, 0xeb, 0x46, 0x61, 0x65, 0x72, 0xf6, 0x65, 0x72, 0x5a,
0x75, 0x69, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x53,
0x69, 0x6e, 0x74, 0x2d, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x65,
0x6c, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x42, 0x65, 0x72, 0x6c, 0x69, 0x6a,
0x6e, 0x49, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x6f, 0x65, 0x6c, 0x50, 0x61,
0x72, 0x69, 0x6a, 0x73, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61, 0x61, 0x6e,
0x73, 0x74, 0x61, 0x64, 0x57, 0x65, 0x6e, 0x65, 0x6e, 0x43, 0x68, 0x61,
0x67, 0x6f, 0x73, 0x61, 0x72, 0x63, 0x68, 0x69, 0x70, 0x65, 0x6c, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x6e, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x50,
0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0xce, 0x6c, 0x65,
0x73, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x6e, 0xf68, 0xf7a, 0xf40, 0xf0b, 0xf40, 0xfb2, 0xf68, 0xf0b, 0xf4c, 0xf72,
0xf66, 0xf0b, 0x20, 0xf68, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf56, 0xfb7, 0xf68, 0xf63,
0xf0b, 0xf47, 0xf72, 0xf0b, 0xf61, 0xf66, 0xf68, 0xf71, 0xf66, 0xf0b, 0xf58, 0xf0b,
0xf62, 0xf56, 0xfb7, 0xf0b, 0xf58, 0xf0b, 0xf40, 0xf7c, 0xf56, 0xfb7, 0xf44, 0xf0b,
0xf42, 0xf72, 0xf56, 0xfb7, 0xf71, 0xf53, 0xf0b, 0xf47, 0xf74, 0xf63, 0xf56, 0xfb7,
0xf72, 0xf0b, 0xf66, 0xf71, 0xf60, 0xf74, 0xf56, 0xfb7, 0xfb2, 0xf0b, 0xf5b, 0xf0b,
0xf56, 0xf72, 0xf63, 0xf56, 0xfb7, 0xf74, 0xf0b, 0xf47, 0xf74, 0xf58, 0xf0b, 0xf56,
0xfb7, 0xf74, 0xf0b, 0xf62, 0xf40, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf62, 0xf7c, 0xf40,
0xf7c, 0xf0b, 0xf53, 0xf40, 0xf0b, 0xf62, 0xf72, 0xf4c, 0xf71, 0xf0b, 0xf40, 0xf62,
0xf47, 0xf72, 0xf0b, 0xf56, 0xfb7, 0xf74, 0xf0b, 0xf4a, 0xf72, 0xf42, 0xfb7, 0xf0b,
0xf56, 0xfb7, 0xf7c, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf53, 0xf7a, 0xf67, 0xf0b, 0xf62,
0xf71, 0xf0b, 0xf62, 0xf7a, 0xf40, 0xf7a, 0xf58, 0xf0b, 0xf54, 0xf0b, 0xf63, 0xf41,
0xf71, 0xf62, 0xf0b, 0xf4a, 0xf74, 0xf58, 0xf40, 0xf72, 0xf0b, 0xf42, 0xfb7, 0xf0b,
0xf63, 0xf72, 0xf40, 0xf72, 0xf53, 0xf0b, 0xf64, 0xf0b, 0xf66, 0xf63, 0xf72, 0xf0b,
0xf56, 0xfb7, 0xfb2, 0xf0b, 0xf56, 0xf72, 0xf63, 0xf63, 0xf7c, 0xf0b, 0xf58, 0xf7a,
0xf63, 0xf74, 0xf0b, 0xf5d, 0xf7a, 0xf53, 0xf0b, 0xf4c, 0xf63, 0xf74, 0xf0b, 0xf66,
0xf71, 0xf0b, 0xf40, 0xf58, 0xf0b, 0xf63, 0xf0b, 0xf56, 0xfb7, 0xf7c, 0xf58, 0xf0b,
0xf54, 0xf74, 0xf0b, 0xf4f, 0xf7c, 0xf58, 0xf0b, 0xf66, 0xf0b, 0xf62, 0xf74, 0xf60,
0xf74, 0xf58, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf53, 0xf72, 0xf58,
0xf7c, 0xf0b, 0xf42, 0xf71, 0xf0b, 0xf4c, 0xf72, 0xf0b, 0xf64, 0xf74, 0xf58, 0xf71,
0xf53, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf56, 0xf72, 0xf0b, 0xf61, 0xf53, 0xf71, 0xf60,
0xf72, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf56, 0xf72, 0xf53, 0xf72, 0xf0b, 0xf61, 0xf0b,
0xf58, 0xf7a, 0xf53, 0xf74, 0xf0b, 0xf68, 0xf40, 0xf0b, 0xf64, 0xf60, 0xf7c, 0xf4a,
0xf5d, 0xf0b, 0xf42, 0xf71, 0xf0b, 0xf4c, 0xf74, 0xf0b, 0xf42, 0xf74, 0xf66, 0xf5d,
0xf0b, 0x20, 0xf4a, 0xf7c, 0xf0b, 0xf58, 0xf7a, 0xf4f, 0xfb2, 0xf72, 0xf0b, 0xf54,
0xf7c, 0xf0b, 0xf63, 0xf72, 0xf4a, 0xf74, 0xf0b, 0xf53, 0xf72, 0xf66, 0xf84, 0xf5d,
0xf72, 0xf53, 0xf4c, 0xf0b, 0xf67, 0xf74, 0xf40, 0xf68, 0xf53, 0xf0b, 0xf4a, 0xf72,
0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf0b, 0xf56, 0xfb7, 0xf74, 0xf60, 0xf72, 0xf0b,
0xf53, 0xf7c, 0xf66, 0xf0b, 0x20, 0xf68, 0xf60, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf66,
0xf68, 0xf0b, 0xf66, 0xf74, 0xf53, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf60, 0xf7c, 0xf53,
0xf68, 0xf0b, 0xf4f, 0xf72, 0xf0b, 0xf40, 0xf7c, 0xf0b, 0xf40, 0xf71, 0xf53, 0xf56,
0xfb7, 0xf62, 0xf0b, 0xf56, 0xf7a, 0xf0b, 0xf4c, 0xf7c, 0xf66, 0xf56, 0xfb7, 0xf7a,
0xf0b, 0xf63, 0xf72, 0xf5b, 0xf56, 0xfb7, 0xf7c, 0xf0b, 0xf42, 0xf0b, 0xf4a, 0xf40,
0xf7a, 0xf58, 0xf0b, 0xf56, 0xf72, 0xf62, 0xf72, 0xf51, 0xf46, 0xf0b, 0x20, 0xf56,
0xf60, 0xf7a, 0xf40, 0xf0b, 0xf62, 0xf0b, 0xf40, 0xf71, 0xf66, 0xf45, 0xf72, 0xf0b,
0xf40, 0xf71, 0xf0b, 0xf42, 0xf7c, 0xf40, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xf0b, 0xf62,
0xf72, 0xf0b, 0xf40, 0xf51, 0xf60, 0xf74, 0xf0b, 0xf66, 0xf71, 0xf7c, 0xf53, 0xf51,
0xf60, 0xf74, 0xf0b, 0xf66, 0xf71, 0xf7c, 0xf53, 0xf0b, 0x20, 0xf40, 0xf72, 0xf62,
0xf72, 0xf40, 0xf4c, 0xf7c, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf40,
0xf68, 0xf7a, 0xf4c, 0xf0b, 0xf58, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf4a, 0xf7c, 0xf53,
0xf68, 0xf7a, 0xf63, 0xf0b, 0x20, 0xf66, 0xf71, 0xf63, 0xf0b, 0xf56, 0xf0b, 0xf4c,
0xf7c, 0xf42, 0xf71, 0xf74, 0xf66, 0xf72, 0xf0b, 0x20, 0xf56, 0xf60, 0xf7a, 0xf42,
0xfb7, 0xf7c, 0xf0b, 0xf4a, 0xf0b, 0xf58, 0xf0b, 0xf63, 0xf42, 0xfb7, 0xf0b, 0xf61,
0xf0b, 0xf53, 0xf67, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf55, 0xf7a, 0xf42, 0xf66, 0xf72,
0xf68, 0xf72, 0xf0b, 0xf53, 0xf74, 0xf0b, 0xf5d, 0xf72, 0xf40, 0xf47, 0xf71, 0xf0b,
0xf58, 0xf60, 0xf7a, 0xf0b, 0xf40, 0xf63, 0xf0b, 0xf54, 0xf71, 0xf5b, 0xf84, 0xf63,
0xf72, 0xf0b, 0xf58, 0xf58, 0xf7a, 0xf40, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf40, 0xf7c,
0xf0b, 0x20, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf72, 0xf58, 0xf71, 0xf7c, 0xf44, 0xf40,
0xf0b, 0xf4a, 0xf7c, 0xf53, 0xf58, 0xf7c, 0xf53, 0xf0b, 0xf4a, 0xf7a, 0xf0b, 0xf56,
0xfb7, 0xf72, 0xf0b, 0xf4c, 0xf72, 0xf60, 0xf7c, 0xf53, 0xf0b, 0xf66, 0xf60, 0xf7c,
0xf54, 0xf0b, 0xf53, 0xf0b, 0xf58, 0xf54, 0xf0b, 0xf62, 0xf0b, 0xf58, 0xf0b, 0xf62,
0xf72, 0xf0b, 0xf56, 0xfb7, 0xf7c, 0xf62, 0xf7a, 0xf53, 0xf0b, 0xf40, 0xf72, 0xf53,
0xf0b, 0x20, 0xf68, 0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf4a, 0xf62, 0xf72, 0xf0b,
0xf47, 0xf72, 0xf0b, 0xf53, 0xf62, 0xf72, 0xf0b, 0xf66, 0xf7c, 0xf0b, 0xf63, 0xf72,
0xf60, 0xf74, 0xf4a, 0xf66, 0xf71, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf61, 0xf0b,
0xf42, 0xfb7, 0xf7c, 0xf66, 0xf71, 0xf53, 0xf0b, 0xf4a, 0xf7c, 0xf0b, 0x20, 0xf4c,
0xf7c, 0xf0b, 0xf58, 0xf72, 0xf44, 0xf0b, 0xf42, 0xfb7, 0xf7c, 0xf68, 0xf72, 0xf66,
0xf72, 0xf0b, 0xf4a, 0xf7a, 0xf4a, 0xf0b, 0x20, 0xf47, 0xf71, 0xf7c, 0xf53, 0xf66,
0xf72, 0xf0b, 0xf66, 0xf74, 0xf0b, 0xf68, 0xf72, 0xf55, 0xf4a, 0xf0b, 0xf40, 0xf0b,
0xf62, 0xf7a, 0xf53, 0xf4a, 0xf4a, 0xf7a, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf66, 0xf72,
0xf0b, 0xf42, 0xf71, 0xf63, 0xf0b, 0xf54, 0xf4f, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf5d,
0xf0b, 0xf53, 0xf4a, 0xf7c, 0xf0b, 0xf62, 0xf7c, 0xf53, 0xf0b, 0xf4a, 0xf7c, 0xf61,
0xf0b, 0xf40, 0xf74, 0xf0b, 0xf4f, 0xf4f, 0xf4c, 0xf7a, 0xf0b, 0xf5d, 0xf72, 0xf66,
0xf0d, 0xf58, 0xf60, 0xf74, 0xf0b, 0xf66, 0xf71, 0xf7c, 0xf53, 0xf55, 0xf71, 0xf63,
0xf0b, 0xf58, 0xf62, 0xf62, 0xf71, 0xf7c, 0xf0b, 0xf50, 0xf7a, 0xf0b, 0xf62, 0xf5d,
0xf7c, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf7c, 0xf40, 0xf56, 0xfb7, 0xf0b, 0xf62, 0xf7a,
0xf53, 0xf0b, 0xf56, 0xfb7, 0xfb2, 0xf74, 0xf0b, 0xf53, 0xf60, 0xf72, 0xf40, 0xf7c,
0xf0b, 0xf63, 0xf7c, 0xf58, 0xf0b, 0xf56, 0xfb7, 0xf7c, 0xf47, 0xf71, 0xf0b, 0xf40,
0xf62, 0xf0b, 0xf4a, 0xf47, 0xf0b, 0xf61, 0xf0b, 0xf54, 0xf74, 0xf0b, 0xf62, 0xf40,
0xf0b, 0xf56, 0xfb7, 0xf74, 0xf63, 0xf40, 0xf0b, 0xf62, 0xf0b, 0xf45, 0xf72, 0xf40,
0xf71, 0xf4a, 0xf0b, 0xf58, 0xf71, 0xf53, 0xf0b, 0xf4c, 0xf74, 0xf40, 0xf71, 0xf7c,
0xf63, 0xf0b, 0xf40, 0xf0b, 0xf4a, 0xf40, 0xf7c, 0xf0b, 0xf63, 0xf0b, 0x20, 0xf63,
0xf58, 0xf0b, 0xf54, 0xf74, 0xf62, 0xf54, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf4a, 0xf60,
0xf7a, 0xf0b, 0xf53, 0xf40, 0xf4a, 0xf7a, 0xf0b, 0xf62, 0xf71, 0xf53, 0xf50, 0xf72,
0xf58, 0xf0b, 0xf55, 0xf74, 0xf42, 0xf56, 0xf72, 0xf0b, 0xf61, 0xf71, 0xf53, 0xf0b,
0xf4a, 0xf72, 0xf0b, 0xf61, 0xf71, 0xf53, 0xf62, 0xf44, 0xf0b, 0xf42, 0xfb7, 0xf74,
0xf53, 0xf56, 0xf62, 0xf0b, 0xf58, 0xf74, 0xf0b, 0xf51, 0xf40, 0xf7a, 0xf54, 0xf0b,
0xf56, 0xf71, 0xf4c, 0xf55, 0xf71, 0xf60, 0xf7a, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf62,
0xf7a, 0xf0b, 0xf41, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf56, 0xf72, 0xf40, 0xf66, 0xf71,
0xf60, 0xf74, 0xf50, 0xf0b, 0xf47, 0xf7c, 0xf0b, 0xf47, 0xf71, 0xf68, 0xf58, 0xf66,
0xf0b, 0xf4a, 0xf62, 0xf0b, 0xf4c, 0xf58, 0xf68, 0xf71, 0xf53, 0xf0b, 0xf51, 0xf7c,
0xf0b, 0xf62, 0xf68, 0xf7a, 0xf0b, 0xf50, 0xf7a, 0xf53, 0xf66, 0xf0b, 0xf56, 0xfb7,
0xf62, 0xf0b, 0xf63, 0xf72, 0xf53, 0xf56, 0xfb2, 0xf0b, 0xf66, 0xf7a, 0xf63, 0xf66,
0xf56, 0xfb7, 0xf74, 0xf0b, 0xf40, 0xf0b, 0xf62, 0xf7a, 0xf66, 0xf4a, 0xf56, 0xfb7,
0xf74, 0xf0b, 0xf4c, 0xf0b, 0xf54, 0xf7a, 0xf66, 0xf4a, 0xf40, 0xf72, 0xf0b, 0xf64,
0xf72, 0xf0b, 0xf53, 0xf71, 0xf5d, 0xf40, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b,
0xf67, 0xf7a, 0xf0b, 0xf42, 0xf7a, 0xf53, 0xf4c, 0xf56, 0xf0b, 0xf63, 0xf72, 0xf53,
0xf67, 0xf7a, 0xf63, 0xf0b, 0xf66, 0xf72, 0xf44, 0xf0b, 0xf40, 0xf72, 0xf63, 0xf71,
0xf53, 0xf0b, 0xf4c, 0xf71, 0xf53, 0xf63, 0xf42, 0xf0b, 0xf66, 0xf58, 0xf0b, 0xf56,
0xfb7, 0xf40, 0xf58, 0xf0b, 0xf4c, 0xf7a, 0xf0b, 0xf62, 0xf7a, 0xf4c, 0xf58, 0xf72,
0xf53, 0xf66, 0xf40, 0xf58, 0xf7c, 0xf0b, 0xf53, 0xf0b, 0xf40, 0xf7c, 0xf58, 0xf71,
0xf7c, 0xf66, 0xf0b, 0xf40, 0xf7c, 0xf68, 0xf7c, 0xf66, 0xf0b, 0xf63, 0xf7c, 0xf54,
0xf7a, 0xf0b, 0xf62, 0xf72, 0xf66, 0xf54, 0xfb2, 0xf42, 0xf62, 0xf72, 0xf0b, 0xf42,
0xfb7, 0xf66, 0xf0b, 0xf62, 0xf0b, 0xf61, 0xf7a, 0xf0b, 0xf56, 0xf7c, 0xf66, 0xf7c,
0xf0b, 0xf55, 0xf72, 0xf0b, 0xf61, 0xf68, 0xf72, 0xf66, 0xf0b, 0xf4a, 0xf7c, 0xf40,
0xf0b, 0xf67, 0xf7c, 0xf63, 0xf58, 0xf4a, 0xf71, 0xf0b, 0xf63, 0xf71, 0xf72, 0xf53,
0xf4a, 0xf0b, 0xf62, 0xf0b, 0xf53, 0xf72, 0xf56, 0xf0b, 0xf51, 0xf74, 0xf51, 0xf66,
0xf5d, 0xf72, 0xf0b, 0xf61, 0xf7a, 0xf0b, 0xf53, 0xf56, 0xf72, 0xf63, 0xf0b, 0xf53,
0xf72, 0xf0b, 0xf61, 0xf71, 0xf66, 0xf5d, 0xf62, 0xf0b, 0xf66, 0xf7c, 0xf5b, 0xf74,
0xf0b, 0xf62, 0xf72, 0xf46, 0xf68, 0xf71, 0xf53, 0xf0b, 0xf4f, 0xf0b, 0xf53, 0xf0b,
0xf53, 0xf0b, 0xf51, 0xf72, 0xf0b, 0xf56, 0xf7c, 0xf40, 0xf72, 0xf62, 0xf72, 0xf66,
0xf4a, 0xf0b, 0xf58, 0xf66, 0xf72, 0xf40, 0xf7c, 0xf0b, 0xf40, 0xf7c, 0xf66, 0xf72,
0xf40, 0xf7c, 0xf0b, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf7c, 0xf40, 0xf72, 0xf62, 0xf0b,
0xf42, 0xf74, 0xf0b, 0xf63, 0xf72, 0xf53, 0xf58, 0xf71, 0xf63, 0xf0b, 0xf51, 0xf72,
0xf56, 0xf66, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf64, 0xf71, 0xf66, 0xf68,
0xf0b, 0xf54, 0xf72, 0xf0b, 0xf61, 0xf55, 0xf0b, 0xf40, 0xf60, 0xf7c, 0xf0b, 0xf55,
0xf7c, 0xf55, 0xf74, 0xf0b, 0xf53, 0xf0b, 0xf55, 0xf74, 0xf0b, 0xf4a, 0xf72, 0xf42,
0xf7a, 0xf58, 0xf0b, 0xf56, 0xf72, 0xf61, 0xf62, 0xf58, 0xf7a, 0xf0b, 0xf47, 0xf74,
0xf0b, 0xf62, 0xf7c, 0xf58, 0xf72, 0xf4c, 0xf0b, 0xf5d, 0xf7a, 0xf53, 0xf71, 0xf5d,
0xf0b, 0xf62, 0xf74, 0xf54, 0xf0b, 0xf42, 0xf7c, 0xf0b, 0x20, 0xf54, 0xf0b, 0xf42,
0xf7c, 0xf54, 0xf0b, 0xf63, 0xf71, 0xf5d, 0xf62, 0xf71, 0xf0b, 0xf62, 0xf7c, 0xf0b,
0xf0b, 0xf4f, 0xf7c, 0xf44, 0xf0b, 0xf42, 0xf66, 0xf71, 0xf60, 0xf7a, 0xf0b, 0xf55,
0xf71, 0xf53, 0xf4f, 0xf7c, 0xf44, 0xf0b, 0xf42, 0xf0b, 0xf4f, 0xf0b, 0xf54, 0xf74,
0xf5d, 0xf7a, 0xf40, 0xd801, 0xdc08, 0xd801, 0xdc3c, 0xd801, 0xdc30, 0xd801, 0xdc3f, 0xd801,
0xdc01, 0xd801, 0xdc4d, 0xd801, 0xdc3f, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc2e, 0xd801,
0xdc3e, 0xd801, 0xdc12, 0xd801, 0xdc31, 0xd801, 0xdc2e, 0xd801, 0xdc46, 0xd801, 0xdc28, 0xd801,
0xdc1f, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0xd801, 0xdc2a, 0xd801, 0xdc40, 0xd801, 0xdc2c, 0xd801,
0xdc14, 0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc42, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801,
0xdc14, 0xd801, 0xdc32, 0xd801, 0xdc3b, 0xd801, 0xdc49, 0xd801, 0xdc31, 0xd801, 0xdc2e, 0xd801,
0xdc3b, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc32, 0xd801,
0xdc4c, 0xd801, 0xdc30, 0xd801, 0xdc39, 0xd801, 0xdc2c, 0xd801, 0xdc4a, 0xd801, 0xdc32, 0xd801,
0xdc45, 0xd801, 0xdc24, 0xd801, 0xdc2a, 0xd801, 0xdc3f, 0xd801, 0xdc45, 0x2c, 0x20, 0xd801,
0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801,
0xdc32, 0xd801, 0xdc23, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc2f, 0xd801, 0xdc4d, 0xd801,
0xdc40, 0xd801, 0xdc2c, 0x2c, 0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801,
0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc11, 0xd801, 0xdc28, 0xd801,
0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc46, 0xd801, 0xdc3a, 0xd801, 0xdc32, 0xd801,
0xdc49, 0xd801, 0xdc40, 0x2c, 0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801,
0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc13, 0xd801, 0xdc2f, 0xd801,
0xdc4a, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc2e, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0x2c, 0x20,
0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c,
0xd801, 0xdc32, 0xd801, 0xdc1a, 0xd801, 0xdc2f, 0xd801, 0xdc42, 0xd801, 0xdc29, 0x2c, 0x20,
0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c,
0xd801, 0xdc32, 0xd801, 0xdc1a, 0xd801, 0xdc2e, 0xd801, 0xdc4c, 0xd801, 0xdc45, 0xd801, 0xdc2f,
0xd801, 0xdc4c, 0xd801, 0xdc46, 0x2c, 0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c,
0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc0e, 0xd801, 0xdc2e,
0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc4b, 0xd801, 0xdc30, 0xd801, 0xdc3f, 0x2c, 0x20,
0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c,
0xd801, 0xdc32, 0xd801, 0xdc16, 0xd801, 0xdc2d, 0xd801, 0xdc4c, 0xd801, 0xdc2c, 0xd801, 0xdc22,
0xd801, 0xdc2d, 0xd801, 0xdc36, 0xd801, 0xdc28, 0xd801, 0xdc42, 0xd801, 0xdc2e, 0xd801, 0xdc4a,
0xd801, 0xdc23, 0xd801, 0xdc2a, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc45,
0xd801, 0xdc2f, 0xd801, 0xdc4a, 0xd801, 0xdc2c, 0x2c, 0x20, 0xd801, 0xdc17, 0xd801, 0xdc32,
0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc3f, 0xd801, 0xdc28, 0xd801, 0xdc22,
0xd801, 0xdc31, 0xd801, 0xdc45, 0x20, 0xd801, 0xdc08, 0xd801, 0xdc4c, 0xd801, 0xdc3e, 0xd801,
0xdc32, 0xd801, 0xdc4a, 0xd801, 0xdc45, 0xd801, 0xdc23, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801,
0xdc2a, 0xd801, 0xdc4b, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc28, 0xd801, 0xdc24, 0xd801,
0xdc2d, 0x20, 0xd801, 0xdc0f, 0xd801, 0xdc31, 0xd801, 0xdc49, 0xd801, 0xdc3f, 0xd801, 0xdc24,
0xd801, 0xdc2c, 0xd801, 0xdc4b, 0xd801, 0xdc1d, 0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc3b,
0xd801, 0xdc32, 0xd801, 0xdc49, 0x2c, 0x20, 0xd801, 0xdc24, 0xd801, 0xdc31, 0xd801, 0xdc49,
0xd801, 0xdc43, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc32, 0xd801, 0xdc3f, 0xd801, 0xdc2c, 0xd801,
0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc24, 0xd801, 0xdc2d, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc29,
0xd801, 0xdc4a, 0xd801, 0xdc32, 0xd801, 0xdc4b, 0x2c, 0x20, 0xd801, 0xdc24, 0xd801, 0xdc31,
0xd801, 0xdc49, 0xd801, 0xdc43, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc32, 0xd801, 0xdc3f, 0xd801,
0xdc2c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc19, 0xd801, 0xdc28, 0xd801, 0xdc4c, 0xd801,
0xdc2e, 0xd801, 0xdc3f, 0xd801, 0xdc45, 0xd801, 0xdc0f, 0xd801, 0xdc30, 0xd801, 0xdc3f, 0xd801,
0xdc2d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc3b, 0xd801, 0xdc10, 0xd801, 0xdc2a, 0xd801,
0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc4a, 0xd801, 0xdc2d, 0xd801, 0xdc4a, 0xd801, 0xdc2d, 0xd801,
0xdc23, 0xd801, 0xdc2e, 0xd801, 0xdc3c, 0xd801, 0xdc36, 0xd801, 0xdc29, 0xd801, 0xdc0e, 0xd801,
0xdc29, 0xd801, 0xdc3f, 0x53, 0x74, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9,
0x6c, 0x65, 0x6d, 0x79, 0x53, 0x74, 0x20, 0x4a, 0x6f, 0x68, 0x6e, 0x2019,
0x73, 0x53, 0x74, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73, 0x53, 0x74, 0x20,
0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x74, 0x20, 0x54, 0x68, 0x6f, 0x6d,
0x61, 0x73, 0x53, 0x74, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74,
0x53, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x428, 0x438, 0x43a,
0x430, 0x433, 0x43e, 0x41b, 0x43e, 0x43d, 0x433, 0x439, 0x438, 0x440, 0x431, 0x44e,
0x435, 0x43d, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x439, 0x422, 0x430, 0x439, 0x43f,
0x435, 0x439, 0x422, 0x43e, 0x43c, 0x441, 0x43a, 0x43e, 0x439, 0x423, 0x43b, 0x430,
0x43d, 0x2d, 0x431, 0x430, 0x442, 0x43e, 0x440, 0x410, 0x444, 0x438, 0x43d, 0x44b,
0x41c, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x439, 0x41c, 0x43e, 0x441, 0x43a, 0x43e,
0x432, 0x41, 0x62, 0x69, 0x11d, 0x61, 0x6e, 0x6f, 0x41, 0x6b, 0x72, 0x61,
0x6f, 0x41, 0x64, 0x69, 0x73, 0x61, 0x62, 0x65, 0x62, 0x6f, 0x41, 0x6c,
0x11d, 0x65, 0x72, 0x6f, 0x41, 0x73, 0x6d, 0x65, 0x72, 0x6f, 0x42, 0x61,
0x6e, 0x67, 0x6f, 0x42, 0x61, 0x6e, 0x6a, 0x75, 0x6c, 0x6f, 0x42, 0x69,
0x73, 0x61, 0x16d, 0x6f, 0x42, 0x72, 0x61, 0x7a, 0x61, 0x76, 0x69, 0x6c,
0x6f, 0x42, 0x75, 0x135, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x6f, 0x4b, 0x61,
0x7a, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x6f, 0x43, 0x65, 0x16d, 0x74,
0x6f, 0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x69, 0x6f, 0x44, 0x61, 0x6b,
0x61, 0x72, 0x6f, 0x44, 0x61, 0x72, 0x65, 0x73, 0x61, 0x6c, 0x61, 0x6d,
0x6f, 0x11c, 0x69, 0x62, 0x75, 0x74, 0x69, 0x6f, 0x44, 0x75, 0x61, 0x6c,
0x61, 0x6f, 0x41, 0x6a, 0x75, 0x6e, 0x6f, 0x46, 0x72, 0x69, 0x74, 0x61,
0x16d, 0x6e, 0x6f, 0x47, 0x61, 0x62, 0x6f, 0x72, 0x6f, 0x6e, 0x6f, 0x48,
0x61, 0x72, 0x61, 0x72, 0x65, 0x6f, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x65,
0x73, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x11c, 0x75, 0x62, 0x61, 0x6f, 0x4b,
0x61, 0x6d, 0x70, 0x61, 0x6c, 0x6f, 0x124, 0x61, 0x72, 0x74, 0x75, 0x6d,
0x6f, 0x4b, 0x69, 0x67, 0x61, 0x6c, 0x6f, 0x4b, 0x69, 0x6e, 0x15d, 0x61,
0x73, 0x6f, 0x4c, 0x61, 0x67, 0x6f, 0x73, 0x6f, 0x4c, 0x69, 0x62, 0x72,
0x65, 0x76, 0x69, 0x6c, 0x6f, 0x4c, 0x6f, 0x6d, 0x65, 0x6f, 0x4c, 0x75,
0x61, 0x6e, 0x64, 0x6f, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62, 0x61, 0x15d,
0x6f, 0x4c, 0x75, 0x73, 0x61, 0x6b, 0x6f, 0x4d, 0x61, 0x73, 0x65, 0x72,
0x75, 0x6f, 0x4d, 0x62, 0x61, 0x62, 0x61, 0x6e, 0x6f, 0x4d, 0x6f, 0x67,
0x61, 0x64, 0x69, 0x15d, 0x6f, 0x4d, 0x6f, 0x6e, 0x72, 0x6f, 0x76, 0x69,
0x6f, 0x4e, 0x61, 0x6a, 0x72, 0x6f, 0x62, 0x69, 0x6f, 0x4e, 0x69, 0x11d,
0x61, 0x6d, 0x65, 0x6e, 0x6f, 0x4e, 0x69, 0x61, 0x6d, 0x65, 0x6a, 0x6f,
0x4e, 0x75, 0x61, 0x6b, 0x15d, 0x6f, 0x74, 0x6f, 0x56, 0x61, 0x67, 0x61,
0x64, 0x75, 0x67, 0x75, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x6e, 0x6f,
0x76, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x6d, 0x65, 0x6f, 0x54, 0x72,
0x69, 0x70, 0x6f, 0x6c, 0x6f, 0x54, 0x75, 0x6e, 0x69, 0x7a, 0x6f, 0x56,
0x69, 0x6e, 0x64, 0x68, 0x75, 0x6b, 0x6f, 0x41, 0x6e, 0x67, 0x76, 0x69,
0x6c, 0x6f, 0x41, 0x6e, 0x74, 0x69, 0x67, 0x76, 0x6f, 0x42, 0x6f, 0x6e,
0x61, 0x65, 0x72, 0x6f, 0x4b, 0x6f, 0x72, 0x64, 0x6f, 0x62, 0x6f, 0x53,
0x61, 0x6c, 0x74, 0x61, 0x75, 0x72, 0x62, 0x6f, 0x41, 0x72, 0x75, 0x62,
0x6f, 0x41, 0x73, 0x75, 0x6e, 0x63, 0x69, 0x6f, 0x6e, 0x6f, 0x42, 0x61,
0x68, 0x69, 0x6f, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64, 0x6f, 0x42, 0x65,
0x6c, 0x65, 0x6d, 0x6f, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0x6f, 0x42, 0x6f,
0x67, 0x6f, 0x74, 0x6f, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x11d, 0x61,
0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x4b, 0x61, 0x6e, 0x6b, 0x75, 0x6e,
0x6f, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x73, 0x6f, 0x4b, 0x61, 0x6a,
0x65, 0x6e, 0x6f, 0x4b, 0x61, 0x6a, 0x6d, 0x61, 0x6e, 0x6f, 0x6a, 0x108,
0x69, 0x6b, 0x61, 0x67, 0x6f, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x72, 0x69,
0x6b, 0x6f, 0x4b, 0x75, 0x72, 0x61, 0x63, 0x61, 0x6f, 0x44, 0x65, 0x6e,
0x76, 0x65, 0x72, 0x6f, 0x44, 0x65, 0x74, 0x72, 0x6f, 0x6a, 0x74, 0x6f,
0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x6f, 0x45, 0x64, 0x6d, 0x6f,
0x6e, 0x74, 0x6f, 0x6e, 0x6f, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f,
0x72, 0x6f, 0x46, 0x6f, 0x72, 0x74, 0x61, 0x6c, 0x65, 0x7a, 0x6f, 0x47,
0x72, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6f, 0x47,
0x72, 0x65, 0x6e, 0x61, 0x64, 0x6f, 0x47, 0x76, 0x61, 0x64, 0x65, 0x6c,
0x75, 0x70, 0x6f, 0x47, 0x76, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x6f,
0x47, 0x75, 0x6a, 0x61, 0x6e, 0x6f, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61,
0x6b, 0x73, 0x6f, 0x48, 0x61, 0x76, 0x61, 0x6e, 0x6f, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x6f, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x4b, 0x6e, 0x6f, 0x78, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x4d, 0x61, 0x72, 0x65,
0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x6f, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x54, 0x65, 0x6c,
0x6c, 0x20, 0x43, 0x69, 0x74, 0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x6f, 0x56, 0x65, 0x76, 0x61, 0x79, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x56, 0x69, 0x6e, 0x63, 0x65,
0x6e, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x6f, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x49, 0x6b, 0x61, 0x6c, 0x75,
0x69, 0x74, 0x6f, 0x4a, 0x61, 0x6d, 0x61, 0x6a, 0x6b, 0x6f, 0x4d, 0x6f,
0x6e, 0x74, 0x69, 0x63, 0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65,
0x6e, 0x74, 0x75, 0x6b, 0x69, 0x6f, 0x4c, 0x69, 0x6d, 0x6f, 0x4c, 0x6f,
0x73, 0x61, 0x6e, 0x11d, 0x65, 0x6c, 0x65, 0x73, 0x6f, 0x4d, 0x61, 0x6e,
0x61, 0x67, 0x76, 0x6f, 0x4d, 0x61, 0x6e, 0x61, 0x16d, 0x73, 0x6f, 0x4d,
0x61, 0x72, 0x69, 0x67, 0x6f, 0x74, 0x6f, 0x4d, 0x61, 0x72, 0x74, 0x69,
0x6e, 0x69, 0x6b, 0x6f, 0x4d, 0x65, 0x72, 0x69, 0x64, 0x6f, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x75, 0x72, 0x62, 0x6f, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x6f, 0x4d, 0x6f, 0x6e, 0x6b, 0x74, 0x6f, 0x6e, 0x6f,
0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x65, 0x6a, 0x6f, 0x4d, 0x6f, 0x6e,
0x63, 0x65, 0x72, 0x61, 0x74, 0x6f, 0x4e, 0x61, 0x73, 0x61, 0x16d, 0x6f,
0x4e, 0x6f, 0x76, 0x6a, 0x6f, 0x72, 0x6b, 0x6f, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x6f, 0x43,
0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x61,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x6f, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x6f, 0x4e, 0x75, 0x6b, 0x6f, 0x50, 0x61,
0x6e, 0x61, 0x6d, 0x6f, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x6f, 0x50,
0x6f, 0x72, 0x74, 0x6f, 0x70, 0x72, 0x69, 0x6e, 0x63, 0x6f, 0x50, 0x6f,
0x72, 0x74, 0x6f, 0x73, 0x70, 0x65, 0x6e, 0x6f, 0x50, 0x75, 0x65, 0x72,
0x74, 0x6f, 0x72, 0x69, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61,
0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53, 0x61, 0x6e, 0x70,
0x61, 0x16d, 0x6c, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x42,
0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x6f, 0x53, 0x61, 0x6e,
0x6b, 0x74, 0x61, 0x20, 0x4b, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x66, 0x6f,
0x72, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x63,
0x69, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x54, 0x6f, 0x6d,
0x61, 0x73, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x56, 0x69,
0x6e, 0x63, 0x65, 0x6e, 0x74, 0x6f, 0x54, 0x65, 0x67, 0x75, 0x63, 0x69,
0x67, 0x61, 0x6c, 0x70, 0x6f, 0x56, 0x61, 0x6e, 0x6b, 0x75, 0x76, 0x65,
0x72, 0x6f, 0x56, 0x69, 0x6e, 0x69, 0x70, 0x65, 0x67, 0x6f, 0x4d, 0x61,
0x6b, 0x76, 0x6f, 0x72, 0x61, 0x20, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f,
0x53, 0x68, 0x6f, 0x77, 0x61, 0x4c, 0x6f, 0x6e, 0x67, 0x6a, 0x65, 0x72,
0x75, 0x72, 0x62, 0x6f, 0x41, 0x64, 0x65, 0x6e, 0x6f, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x6f, 0x41, 0x6d, 0x61, 0x6e, 0x6f, 0x41, 0x15d, 0x125, 0x61,
0x62, 0x61, 0x64, 0x6f, 0x42, 0x61, 0x67, 0x64, 0x61, 0x64, 0x6f, 0x42,
0x61, 0x72, 0x65, 0x6a, 0x6e, 0x6f, 0x42, 0x61, 0x6b, 0x75, 0x6f, 0x42,
0x61, 0x6e, 0x6b, 0x6f, 0x6b, 0x6f, 0x42, 0x65, 0x6a, 0x72, 0x75, 0x74,
0x6f, 0x42, 0x69, 0x15d, 0x6b, 0x65, 0x6b, 0x6f, 0x42, 0x72, 0x75, 0x6e,
0x65, 0x6a, 0x6f, 0x108, 0x69, 0x74, 0x61, 0x44, 0x69, 0x6c, 0x6f, 0x44,
0x75, 0x62, 0x61, 0x6a, 0x75, 0x72, 0x62, 0x6f, 0x44, 0x75, 0x15d, 0x61,
0x6e, 0x62, 0x65, 0x6f, 0x47, 0x61, 0x7a, 0x61, 0x6f, 0x48, 0x65, 0x62,
0x72, 0x6f, 0x6e, 0x6f, 0x48, 0x6f, 0x109, 0x69, 0x6d, 0x69, 0x6e, 0x6f,
0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x6f, 0x124, 0x6f, 0x76, 0x64,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x11c, 0x61, 0x6b, 0x61,
0x72, 0x74, 0x6f, 0x4a, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d,
0x6f, 0x4b, 0x61, 0x62, 0x75, 0x6c, 0x6f, 0x4b, 0x61, 0x6d, 0x109, 0x61,
0x74, 0x6b, 0x6f, 0x4b, 0x61, 0x72, 0x61, 0x109, 0x69, 0x6f, 0x4b, 0x61,
0x74, 0x6d, 0x61, 0x6e, 0x64, 0x75, 0x6f, 0x124, 0x61, 0x6e, 0x64, 0x69,
0x67, 0x61, 0x4b, 0x6f, 0x6c, 0x6b, 0x61, 0x74, 0x6f, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x4b, 0x75, 0x61,
0x6c, 0x61, 0x6c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x6f, 0x4b, 0x75, 0x76,
0x61, 0x6a, 0x74, 0x6f, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x6f, 0x4d, 0x61,
0x73, 0x6b, 0x61, 0x74, 0x6f, 0x4e, 0x69, 0x6b, 0x6f, 0x7a, 0x69, 0x6f,
0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x65, 0x63, 0x6b, 0x6f,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f,
0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x4f, 0x72, 0x61, 0x6c, 0x6f, 0x50, 0x6e,
0x6f, 0x6d, 0x70, 0x65, 0x6e, 0x6f, 0x50, 0x6a, 0x6f, 0x6e, 0x67, 0x6a,
0x61, 0x6e, 0x67, 0x6f, 0x4b, 0x61, 0x74, 0x61, 0x72, 0x6f, 0x4b, 0x69,
0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x6f, 0x52, 0x69, 0x61, 0x64, 0x6f,
0x53, 0x61, 0x125, 0x61, 0x6c, 0x65, 0x6e, 0x6f, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x6b, 0x61, 0x6e, 0x64, 0x6f, 0x53, 0x65, 0x75, 0x6c, 0x6f, 0x15c,
0x61, 0x6e, 0x68, 0x61, 0x6a, 0x6f, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x75, 0x72, 0x6f, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6f, 0x54, 0x61, 0x15d,
0x6b, 0x65, 0x6e, 0x74, 0x6f, 0x54, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x6f,
0x54, 0x65, 0x68, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x54, 0x69, 0x6d, 0x62,
0x75, 0x6f, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x6f,
0x55, 0x72, 0x75, 0x6d, 0x109, 0x69, 0x6f, 0x55, 0x73, 0x74, 0x6e, 0x65,
0x72, 0x6f, 0x56, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x6f, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x52, 0x61, 0x6e, 0x67, 0x75,
0x6e, 0x6f, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x45, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x6f, 0x41,
0x63, 0x6f, 0x72, 0x6f, 0x6a, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x6f,
0x6a, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x6f, 0x6a, 0x4b, 0x61, 0x62,
0x6f, 0x76, 0x65, 0x72, 0x64, 0x6f, 0x46, 0x65, 0x72, 0x6f, 0x6f, 0x6a,
0x4d, 0x61, 0x64, 0x65, 0x6a, 0x72, 0x6f, 0x52, 0x65, 0x6a, 0x6b, 0x6a,
0x61, 0x76, 0x69, 0x6b, 0x6f, 0x53, 0x75, 0x64, 0x2d, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x48,
0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x53, 0x74, 0x61, 0x6e, 0x6c, 0x65, 0x6a,
0x6f, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x6a, 0x64, 0x6f, 0x42, 0x72, 0x69,
0x73, 0x62, 0x61, 0x6e, 0x6f, 0x44, 0x61, 0x72, 0x76, 0x69, 0x6e, 0x6f,
0x48, 0x6f, 0x62, 0x61, 0x72, 0x74, 0x6f, 0x4d, 0x65, 0x6c, 0x62, 0x75,
0x72, 0x6e, 0x6f, 0x50, 0x65, 0x72, 0x74, 0x6f, 0x53, 0x69, 0x64, 0x6e,
0x65, 0x6a, 0x6f, 0x41, 0x6d, 0x73, 0x74, 0x65, 0x72, 0x64, 0x61, 0x6d,
0x6f, 0x41, 0x6e, 0x64, 0x6f, 0x72, 0x6f, 0x41, 0x73, 0x74, 0x72, 0x61,
0x125, 0x61, 0x6e, 0x6f, 0x41, 0x74, 0x65, 0x6e, 0x6f, 0x42, 0x65, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x6f, 0x42, 0x65, 0x72, 0x6c, 0x69, 0x6e, 0x6f,
0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61, 0x76, 0x6f, 0x42, 0x72,
0x75, 0x73, 0x65, 0x6c, 0x6f, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x15d,
0x74, 0x6f, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x15d, 0x74, 0x6f, 0x42,
0xfc, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x20, 0x61, 0x6d, 0x20, 0x48,
0x6f, 0x63, 0x68, 0x72, 0x68, 0x65, 0x69, 0x6e, 0x4b, 0x69, 0x15d, 0x69,
0x6e, 0x65, 0x76, 0x6f, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x67,
0x6f, 0x44, 0x75, 0x62, 0x6c, 0x69, 0x6e, 0x6f, 0x11c, 0x69, 0x62, 0x72,
0x61, 0x6c, 0x74, 0x61, 0x72, 0x6f, 0x47, 0x65, 0x72, 0x6e, 0x65, 0x7a,
0x65, 0x6a, 0x6f, 0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x6f, 0x4d,
0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x49, 0x73, 0x74,
0x61, 0x6e, 0x62, 0x75, 0x6c, 0x6f, 0x134, 0x65, 0x72, 0x7a, 0x65, 0x6a,
0x6f, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64,
0x6f, 0x4b, 0x69, 0x65, 0x76, 0x6f, 0x4c, 0x69, 0x73, 0x62, 0x6f, 0x6e,
0x6f, 0x4c, 0x6a, 0x75, 0x62, 0x6c, 0x6a, 0x61, 0x6e, 0x6f, 0x4c, 0x6f,
0x6e, 0x64, 0x6f, 0x6e, 0x6f, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x4d, 0x61, 0x64, 0x72, 0x69, 0x64, 0x6f, 0x4d,
0x61, 0x6c, 0x74, 0x6f, 0x4d, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x4d, 0x6f,
0x73, 0x6b, 0x76, 0x6f, 0x50, 0x61, 0x72, 0x69, 0x7a, 0x6f, 0x50, 0x6f,
0x64, 0x67, 0x6f, 0x72, 0x69, 0x63, 0x6f, 0x50, 0x72, 0x61, 0x67, 0x6f,
0x52, 0x69, 0x67, 0x6f, 0x52, 0x6f, 0x6d, 0x6f, 0x53, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x69, 0x6e, 0x6f, 0x53, 0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f,
0x70, 0x6f, 0x6c, 0x6f, 0x53, 0x6b, 0x6f, 0x70, 0x6a, 0x6f, 0x53, 0x6f,
0x66, 0x69, 0x6f, 0x53, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x6f,
0x54, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x54, 0x69, 0x72, 0x61, 0x6e, 0x6f,
0x55, 0x6c, 0x6a, 0x61, 0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x56, 0x61,
0x64, 0x75, 0x7a, 0x6f, 0x56, 0x69, 0x65, 0x6e, 0x6f, 0x56, 0x69, 0x6c,
0x6e, 0x6f, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x56, 0x61, 0x72, 0x73, 0x6f, 0x76, 0x69, 0x6f, 0x5a, 0x61, 0x67, 0x72,
0x65, 0x62, 0x6f, 0x5a, 0x75, 0x72, 0x69, 0x6b, 0x6f, 0x108, 0x61, 0x67,
0x6f, 0x73, 0x6f, 0x6a, 0x4b, 0x72, 0x69, 0x73, 0x74, 0x6e, 0x61, 0x73,
0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x4b, 0x6f, 0x6b, 0x6f, 0x73,
0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x6a, 0x4b, 0x6f, 0x6d, 0x6f, 0x72,
0x6f, 0x6a, 0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x6a,
0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x6f, 0x6a, 0x4d, 0x61, 0x16d, 0x72,
0x69, 0x63, 0x69, 0x6f, 0x4d, 0x61, 0x6a, 0x6f, 0x74, 0x6f, 0x52, 0x65,
0x75, 0x6e, 0x69, 0x6f, 0x41, 0x70, 0x69, 0x6f, 0x41, 0x16d, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x6f, 0x108, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6f, 0x108,
0x75, 0x75, 0x6b, 0x50, 0x61, 0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c,
0x6f, 0x46, 0x69, 0x11d, 0x69, 0x6f, 0x46, 0x75, 0x6e, 0x61, 0x66, 0x75,
0x74, 0x69, 0x6f, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x6a,
0x47, 0x76, 0x61, 0x6d, 0x6f, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x69,
0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x6a, 0x4d, 0x69, 0x64, 0x76, 0x65, 0x6a,
0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x6a, 0x4e, 0x61, 0x75, 0x72, 0x6f,
0x4e, 0x69, 0x75, 0x6f, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69,
0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x4e, 0x75, 0x6d, 0x65, 0x6f, 0x50, 0x61,
0x67, 0x6f, 0x70, 0x61, 0x67, 0x6f, 0x50, 0x61, 0x6c, 0x61, 0x16d, 0x6f,
0x50, 0x69, 0x74, 0x6b, 0x61, 0x72, 0x6e, 0x61, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x6f, 0x52, 0x61, 0x72, 0x6f, 0x74, 0x6f, 0x6e, 0x67, 0x6f,
0x53, 0x61, 0x69, 0x70, 0x61, 0x6e, 0x6f, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x6f, 0x54, 0x61, 0x72, 0x61, 0x76, 0x6f, 0x56, 0x65, 0x6a, 0x6b,
0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x6f,
0x41, 0x6c, 0x17e, 0x69, 0x69, 0x72, 0x45, 0x6c, 0x20, 0x41, 0x61, 0x69,
0xfa, 0x6e, 0x48, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x48, 0x61, 0x76, 0x61,
0x6e, 0x6e, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x50,
0xf5, 0x68, 0x6a, 0x61, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43,
0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x50, 0xf5, 0x68, 0x6a, 0x61,
0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x50, 0xf5, 0x68, 0x6a, 0x61, 0x2d,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0xf5,
0x41, 0x6e, 0x61, 0x64, 0xf5, 0x72, 0x41, 0x6b, 0x74, 0xf6, 0x62, 0x65,
0x41, 0x74, 0xf5, 0x72, 0x61, 0x75, 0x42, 0x61, 0x6b, 0x75, 0x75, 0x54,
0x161, 0x69, 0x74, 0x61, 0x48, 0xf4, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d,
0x69, 0x6e, 0x68, 0x4a, 0x65, 0x72, 0x75, 0x75, 0x73, 0x61, 0x6c, 0x65,
0x6d, 0x6d, 0x4b, 0x61, 0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b, 0x61, 0x48,
0x61, 0x6e, 0x64, 0xf5, 0x67, 0x61, 0x4b, 0x75, 0x76, 0x65, 0x69, 0x74,
0x4d, 0x61, 0x73, 0x71, 0x61, 0x163, 0x4b, 0xf5, 0x7a, 0xf5, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x41, 0x72, 0x2d, 0x52, 0x69, 0x79, 0x101, 0x1e11, 0x53,
0x61, 0x68, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x72, 0x65, 0x64, 0x6e,
0x65, 0x6b, 0x6f, 0x6c, 0xf5, 0x6d, 0x73, 0x6b, 0x54, 0x68, 0x62, 0x69,
0x6c, 0x69, 0x73, 0x69, 0x54, 0x14d, 0x6b, 0x79, 0x14d, 0x41, 0x73, 0x73,
0x6f, 0x6f, 0x72, 0x69, 0x64, 0x4b, 0x61, 0x6e, 0x61, 0x61, 0x72, 0x69,
0x20, 0x73, 0x61, 0x61, 0x72, 0x65, 0x64, 0x52, 0x6f, 0x68, 0x65, 0x6e,
0x65, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x61, 0x72, 0x65, 0x64, 0x46, 0xe4,
0xe4, 0x72, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x65, 0x64, 0x4c, 0xf5,
0x75, 0x6e, 0x61, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x41,
0x74, 0x65, 0x65, 0x6e, 0x61, 0x42, 0x65, 0x72, 0x6c, 0x69, 0x69, 0x6e,
0x43, 0x68, 0x69, 0x219, 0x69, 0x6e, 0x103, 0x75, 0x4b, 0x6f, 0x70, 0x65,
0x6e, 0x68, 0x61, 0x61, 0x67, 0x65, 0x6e, 0x48, 0x65, 0x6c, 0x73, 0x69,
0x6e, 0x67, 0x69, 0x4d, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72,
0x4b, 0x69, 0x69, 0x65, 0x76, 0x4d, 0x61, 0x61, 0x72, 0x69, 0x61, 0x6e,
0x68, 0x61, 0x6d, 0x69, 0x6e, 0x61, 0x50, 0x61, 0x72, 0x69, 0x69, 0x73,
0x52, 0x69, 0x69, 0x61, 0x52, 0x6f, 0x6f, 0x6d, 0x61, 0x56, 0x69, 0x69,
0x6e, 0x56, 0x61, 0x72, 0x73, 0x73, 0x61, 0x76, 0x69, 0x4a, 0xf5, 0x75,
0x6c, 0x75, 0x73, 0x61, 0x61, 0x72, 0x4b, 0x6f, 0x6f, 0x6b, 0x6f, 0x73,
0x73, 0x61, 0x61, 0x72, 0x65, 0x64, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x69,
0x76, 0x69, 0x64, 0x4c, 0x69, 0x68, 0x61, 0x76, 0xf5, 0x74, 0x74, 0x65,
0x73, 0x61, 0x61, 0x72, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x69, 0x73, 0x61,
0x61, 0x72, 0x65, 0x64, 0x42, 0x65, 0x6c, 0x61, 0x75, 0x4b, 0x61, 0x74,
0x68, 0x6d, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x42, 0x65, 0x6c,
0x69, 0x73, 0x42, 0x61, 0x72, 0x65, 0x69, 0x6e, 0x47, 0x61, 0x73, 0x61,
0x4b, 0x75, 0x76, 0x61, 0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x72,
0x6e, 0x61, 0x72, 0x47, 0x72, 0xf8, 0x6e, 0x68, 0xf8, 0x76, 0x64, 0x61,
0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x46, 0xf8, 0x72, 0x6f, 0x79,
0x61, 0x72, 0x53, 0x75, 0xf0, 0x75, 0x72, 0x67, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x4b, 0x65, 0x79,
0x70, 0x6d, 0x61, 0x6e, 0x6e, 0x61, 0x68, 0x61, 0x76, 0x6e, 0x4c, 0x75,
0x6b, 0x73, 0x65, 0x6d, 0x62, 0x6f, 0x72, 0x67, 0x53, 0x74, 0x6f, 0x6b,
0x6b, 0x68, 0xf3, 0x6c, 0x6d, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e,
0x69, 0xf0, 0x56, 0x61, 0x72, 0x73, 0x6a, 0x61, 0x76, 0x61, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x4d,
0xf3, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4c, 0x75, 0x6e, 0x67, 0x73,
0x6f, 0x64, 0x20, 0x6e, 0x67, 0x20, 0x4a, 0x75, 0xe1, 0x72, 0x65, 0x7a,
0x4c, 0x75, 0x6e, 0x67, 0x73, 0x6f, 0x64, 0x20, 0x6e, 0x67, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x6f, 0x50, 0x75, 0x77, 0x65, 0x72, 0x74, 0x6f,
0x20, 0x6e, 0x67, 0x20, 0x45, 0x73, 0x70, 0x61, 0x6e, 0x79, 0x61, 0x4d,
0x61, 0x6b, 0x69, 0x70, 0x6f, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4c, 0x6f,
0x6f, 0x6b, 0x20, 0x6e, 0x67, 0x20, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e,
0x4c, 0x75, 0x6e, 0x67, 0x73, 0x6f, 0x64, 0x20, 0x6e, 0x67, 0x20, 0x48,
0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x50, 0x6f, 0x68, 0x6a, 0x6f, 0x69,
0x73, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x2c, 0x20, 0x50, 0x6f, 0x68, 0x6a, 0x6f, 0x69, 0x73, 0x2d,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x50, 0x6f, 0x68, 0x6a, 0x6f, 0x69, 0x73,
0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x69,
0x61, 0x67, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x4d, 0x61, 0x63, 0x71, 0x75, 0x61, 0x72, 0x69, 0x65, 0x6e, 0x73, 0x61,
0x61, 0x72, 0x69, 0x41, 0x71, 0x74, 0x61, 0x77, 0x44, 0x61, 0x6d, 0x61,
0x73, 0x6b, 0x6f, 0x73, 0x48, 0x1ed3, 0x20, 0x43, 0x68, 0xed, 0x20, 0x4d,
0x69, 0x6e, 0x68, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x75, 0x70, 0x75, 0x6e,
0x6b, 0x69, 0x48, 0x61, 0x6e, 0x64, 0x79, 0x67, 0x61, 0x4b, 0x61, 0x6c,
0x6b, 0x75, 0x74, 0x74, 0x61, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61,
0x69, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x74, 0x4b, 0x61, 0x6e, 0x61, 0x72,
0x69, 0x61, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x46, 0xe4, 0x72,
0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x45, 0x74, 0x65, 0x6c, 0xe4, 0x2d,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x42, 0x65, 0x72, 0x6c, 0x69,
0x69, 0x6e, 0x69, 0x42, 0x72, 0x79, 0x73, 0x73, 0x65, 0x6c, 0x43, 0x68,
0x69, 0x15f, 0x69, 0x6e, 0x103, 0x75, 0x4b, 0xf6, 0xf6, 0x70, 0x65, 0x6e,
0x68, 0x61, 0x6d, 0x69, 0x6e, 0x61, 0x4d, 0x61, 0x6e, 0x73, 0x61, 0x61,
0x72, 0x69, 0x4b, 0x69, 0x6f, 0x76, 0x61, 0x4c, 0x6f, 0x6e, 0x74, 0x6f,
0x6f, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x50, 0x61, 0x72, 0x69,
0x69, 0x73, 0x69, 0x52, 0x69, 0x69, 0x6b, 0x61, 0x54, 0x75, 0x6b, 0x68,
0x6f, 0x6c, 0x6d, 0x61, 0x54, 0x61, 0x6c, 0x6c, 0x69, 0x6e, 0x6e, 0x61,
0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x69, 0x56, 0x69, 0x6c,
0x6e, 0x61, 0x56, 0x61, 0x72, 0x73, 0x6f, 0x76, 0x61, 0x4a, 0x6f, 0x75,
0x6c, 0x75, 0x73, 0x61, 0x61, 0x72, 0x69, 0x4b, 0x6f, 0x6f, 0x6b, 0x6f,
0x73, 0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x4b, 0x6f, 0x6d, 0x6f, 0x72,
0x69, 0x74, 0x4b, 0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e, 0x73,
0x61, 0x61, 0x72, 0x65, 0x74, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69,
0x76, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x61,
0x61, 0x72, 0x65, 0x74, 0x50, 0xe4, 0xe4, 0x73, 0x69, 0xe4, 0x69, 0x73,
0x73, 0x61, 0x61, 0x72, 0x69, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65,
0x73, 0x61, 0x73, 0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x4d, 0x69, 0x64,
0x77, 0x61, 0x79, 0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x41, 0x64, 0x64,
0x69, 0x73, 0x2d, 0x41, 0x62, 0x65, 0x62, 0x61, 0x4c, 0x65, 0x20, 0x43,
0x61, 0x69, 0x72, 0x65, 0x4c, 0x61, 0xe2, 0x79, 0x6f, 0x75, 0x6e, 0x65,
0x54, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x69, 0x20, 0x28, 0x4c, 0x69, 0x62,
0x79, 0x65, 0x29, 0x55, 0x73, 0x68, 0x75, 0x61, 0xef, 0x61, 0x42, 0x61,
0x68, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x61, 0x73, 0x4c, 0x61, 0x20, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64,
0x65, 0x43, 0x61, 0xef, 0x6d, 0x61, 0x6e, 0x73, 0x44, 0xe9, 0x74, 0x72,
0x6f, 0x69, 0x74, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x71, 0x75, 0x65,
0x47, 0x72, 0x65, 0x6e, 0x61, 0x64, 0x65, 0x4c, 0x61, 0x20, 0x48, 0x61,
0x76, 0x61, 0x6e, 0x65, 0x4b, 0x6e, 0x6f, 0x78, 0x20, 0x5b, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67,
0x6f, 0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d, 0x50,
0x65, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x20, 0x5b, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d, 0x54, 0x65, 0x6c, 0x6c, 0x20,
0x43, 0x69, 0x74, 0x79, 0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x5d, 0x56, 0x65, 0x76, 0x61, 0x79, 0x20, 0x5b, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x5d, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e,
0x65, 0x73, 0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d,
0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63, 0x20, 0x5b, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x5d, 0x4a, 0x61, 0x6d, 0x61, 0xef, 0x71, 0x75,
0x65, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x63, 0x65, 0x6c, 0x6c, 0x6f, 0x20,
0x5b, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x63, 0x6b, 0x79, 0x5d, 0x4d, 0x61,
0x6e, 0x61, 0x6f, 0x73, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x20, 0x28,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x75, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x29, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x20, 0x28, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x75, 0x20, 0x4e, 0x6f, 0x72,
0x64, 0x29, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x20,
0x28, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x75, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x29, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x64, 0x2019, 0x45,
0x73, 0x70, 0x61, 0x67, 0x6e, 0x65, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20,
0x52, 0x69, 0x63, 0x6f, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x44, 0x6f,
0x6d, 0x69, 0x6e, 0x67, 0x75, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x4a, 0x65, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72,
0x65, 0x2d, 0x4e, 0x65, 0x75, 0x76, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x70, 0x68, 0x65, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x65, 0x2d, 0x4c, 0x75, 0x63, 0x69, 0x65, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74,
0x54, 0x68, 0x75, 0x6c, 0xe9, 0x41, 0x6b, 0x74, 0x61, 0x6f, 0x75, 0x41,
0x6b, 0x74, 0x69, 0x6f, 0x75, 0x62, 0x69, 0x6e, 0x73, 0x6b, 0x41, 0x63,
0x68, 0x67, 0x61, 0x62, 0x61, 0x74, 0x41, 0x74, 0x79, 0x72, 0x61, 0x6f,
0x75, 0x42, 0x61, 0x68, 0x72, 0x65, 0xef, 0x6e, 0x42, 0x65, 0x79, 0x72,
0x6f, 0x75, 0x74, 0x68, 0x42, 0x69, 0x63, 0x68, 0x6b, 0x65, 0x6b, 0x44,
0x75, 0x62, 0x61, 0xef, 0x44, 0x6f, 0x75, 0x63, 0x68, 0x61, 0x6e, 0x62,
0xe9, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x6f, 0x75, 0x73, 0x74, 0x65, 0x48,
0xe9, 0x62, 0x72, 0x6f, 0x6e, 0x48, 0xf4, 0x2d, 0x43, 0x68, 0x69, 0x2d,
0x4d, 0x69, 0x6e, 0x68, 0x2d, 0x56, 0x69, 0x6c, 0x6c, 0x65, 0x49, 0x72,
0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x4a, 0xe9, 0x72, 0x75, 0x73, 0x61,
0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0x64, 0x6f, 0x75,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xef, 0x61, 0x72, 0x73, 0x6b, 0x4b,
0x6f, 0x77, 0x65, 0xef, 0x74, 0x4d, 0x61, 0x63, 0x61, 0x73, 0x73, 0x61,
0x72, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x6c, 0x65, 0x4e, 0x69, 0x63, 0x6f,
0x73, 0x69, 0x65, 0x4f, 0x75, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x4b, 0x6f,
0x73, 0x74, 0x61, 0x6e, 0x61, 0xef, 0x4b, 0x7a, 0x79, 0x6c, 0x20, 0x4f,
0x72, 0x64, 0x61, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x63, 0x61, 0x6e, 0x64, 0x65, 0x53, 0xe9,
0x6f, 0x75, 0x6c, 0x54, 0x61, 0x63, 0x68, 0x6b, 0x65, 0x6e, 0x74, 0x54,
0x62, 0x69, 0x6c, 0x69, 0x73, 0x73, 0x69, 0x54, 0xe9, 0x68, 0xe9, 0x72,
0x61, 0x6e, 0x4f, 0x75, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x52, 0x61, 0x6e,
0x67, 0x6f, 0x75, 0x6e, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x6f, 0x75, 0x72, 0x67, 0xce, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x61,
0x6e, 0x61, 0x72, 0x69, 0x65, 0x73, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65,
0x72, 0x74, 0xce, 0x6c, 0x65, 0x73, 0x20, 0x46, 0xe9, 0x72, 0x6f, 0xe9,
0x4d, 0x61, 0x64, 0xe8, 0x72, 0x65, 0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69,
0x65, 0x20, 0x64, 0x75, 0x20, 0x53, 0x75, 0x64, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x65, 0x2d, 0x48, 0xe9, 0x6c, 0xe8, 0x6e, 0x65, 0x41, 0x64, 0xe9,
0x6c, 0x61, 0xef, 0x64, 0x65, 0x41, 0x6e, 0x64, 0x6f, 0x72, 0x72, 0x65,
0x41, 0x74, 0x68, 0xe8, 0x6e, 0x65, 0x73, 0xce, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x69, 0x73, 0x62, 0x6f, 0x6e, 0x6e,
0x65, 0x4d, 0x61, 0x6c, 0x74, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x4d, 0x61, 0x72, 0x69, 0x6e, 0x4f, 0x75, 0x6c, 0x69, 0x61, 0x6e, 0x6f,
0x76, 0x73, 0x6b, 0x4c, 0x65, 0x20, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61,
0x6e, 0x56, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x56, 0x61, 0x72, 0x73, 0x6f,
0x76, 0x69, 0x65, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x65, 0x73, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x65, 0x4c, 0x61, 0x20, 0x52, 0xe9, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0xce, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe2,
0x71, 0x75, 0x65, 0x73, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x69, 0x73, 0x65,
0x73, 0x50, 0x61, 0x6c, 0x61, 0x6f, 0x73, 0x4e, 0x64, 0x6a, 0x61, 0x6d,
0x65, 0x6e, 0x61, 0x54, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x69, 0x20, 0x5b,
0x4c, 0x69, 0x62, 0x79, 0x65, 0x5d, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64,
0x65, 0x20, 0x28, 0x4c, 0x61, 0x29, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43,
0x61, 0xef, 0x6d, 0x61, 0x6e, 0x73, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x75, 0x20,
0x4e, 0x6f, 0x72, 0x64, 0x5d, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x20,
0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x75, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x5d, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x75,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x5d, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x70, 0x68, 0x65, 0x2d, 0x65,
0x74, 0x2d, 0x4e, 0x69, 0xe9, 0x76, 0xe8, 0x73, 0x44, 0x61, 0x63, 0x63,
0x61, 0x54, 0x68, 0x69, 0x6d, 0x70, 0x68, 0x6f, 0x75, 0xee, 0x6c, 0x65,
0x73, 0x20, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x73, 0xee, 0x6c,
0x65, 0x73, 0x20, 0x46, 0xe9, 0x72, 0x6f, 0xe9, 0xee, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x47, 0x6e, 0x6f, 0x76, 0x65, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x53, 0x61,
0x6e, 0x20, 0x50, 0x61, 0x75, 0x6c, 0x69, 0x20, 0x64, 0x61, 0x6c, 0x20,
0x42, 0x72, 0x61, 0x73, 0xee, 0x6c, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x73,
0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x69, 0x73, 0x42, 0x65, 0x6c, 0x67,
0x72, 0xe2, 0x74, 0x4c, 0x69, 0x73, 0x62, 0x6f, 0x6e, 0x65, 0x4c, 0x75,
0x62, 0x69, 0x61, 0x6e, 0x65, 0x4c, 0x75, 0x73, 0x73, 0x65, 0x6d, 0x62,
0x75, 0x72, 0x63, 0x4d, 0x6f, 0x73, 0x63, 0x68, 0x65, 0x50, 0x72, 0x61,
0x67, 0x68, 0x65, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e,
0x56, 0x69, 0x65, 0x6e, 0x65, 0xd83a, 0xdd00, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd00, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd00, 0xd83a,
0xdd24, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd00, 0xd83a, 0xdd27, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a,
0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a,
0xdd04, 0xd83a, 0xdd2d, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a,
0xdd04, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a,
0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd04, 0xd83a,
0xdd35, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a,
0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x2d, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd14, 0xd83a, 0xdd2d,
0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd01,
0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd02,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd38, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd14, 0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd1d, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd25, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd22,
0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd02, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd26,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24,
0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd02, 0xd83a, 0xdd35,
0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd02, 0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd43, 0xd83a, 0xdd2d, 0xd83a, 0xdd02, 0xd83a, 0xdd35, 0xd83a, 0xdd27,
0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22,
0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd03, 0xd83a, 0xdd22,
0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd10, 0xd83a, 0xdd4b,
0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd2b, 0xd83a, 0xdd03, 0xd83a, 0xdd35, 0xd83a, 0xdd39, 0xd83a, 0xdd46, 0xd83a, 0xdd22,
0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a,
0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd10,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d,
0xd83a, 0xdd10, 0xd83a, 0xdd4b, 0xd83a, 0xdd14, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd33, 0xd83a, 0xdd43, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3c,
0xd83a, 0xdd0f, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd34,
0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1a,
0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd1a,
0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd0f,
0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd33,
0xd83a, 0xdd00, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd00, 0xd83a, 0xdd32,
0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd36,
0xd83a, 0xdd35, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3a,
0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd04,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0x2d, 0xd83a,
0xdd09, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd14, 0xd83a, 0xdd35, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a,
0xdd34, 0xd83a, 0xdd02, 0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd2e, 0xd83a, 0xdd38, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd08,
0xd83a, 0xdd2d, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd2b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a,
0xdd24, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0x2d,
0xd83a, 0xdd1d, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0x2d, 0xd83a, 0xdd02, 0xd83a, 0xdd35, 0xd83a,
0xdd31, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd13, 0xd83a, 0xdd27, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd36, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a,
0xdd22, 0xd83a, 0xdd00, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd3c, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd04,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd23,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2b,
0xd83a, 0xdd25, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd41, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd04, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0x2d,
0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd04,
0xd83a, 0xdd2e, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd04,
0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd41, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11,
0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd36, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd18,
0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd25, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd15, 0xd83a, 0xdd0b,
0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0x20, 0xd83a,
0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a,
0xdd43, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3e, 0xd83a, 0xdd32, 0xd83a, 0xdd01, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd01, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0x2d, 0xd83a, 0xdd11,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd01, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c,
0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd3c, 0xd83a, 0xdd01, 0xd83a, 0xdd2e,
0xd83a, 0xdd25, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd09, 0xd83a, 0xdd23, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd09, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd2b, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd09, 0xd83a, 0xdd24,
0x2d, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a,
0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd27,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0x3b, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd41, 0xd83a,
0xdd22, 0xd83a, 0xdd18, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0x2d, 0xd83a, 0xdd04,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd18, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd27,
0x2d, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd18, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0x2d, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd2a,
0xd83a, 0xdd33, 0xd83a, 0xdd18, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31,
0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd28,
0xd83a, 0xdd46, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31,
0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd22, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd33,
0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd2c, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd2e,
0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd33,
0xd83a, 0xdd27, 0xd83a, 0xdd35, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd23, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e,
0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd06, 0xd83a, 0xdd2b,
0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd1a, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0x2d, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a,
0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd2b, 0xd83a,
0xdd34, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd1c, 0xd83a,
0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0x2c,
0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd0f, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0x2c, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a,
0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd0b, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd14, 0xd83a,
0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd14, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd02, 0xd83a, 0xdd35, 0xd83a, 0xdd2d, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a,
0xdd24, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a,
0xdd37, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x2c, 0x20, 0xd83a,
0xdd11, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd33, 0xd83a, 0xdd02, 0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd41, 0xd83a, 0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd22,
0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd3a, 0xd83a,
0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd02, 0xd83a,
0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0x20, 0xd83a,
0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a,
0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd03, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd27, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a,
0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a,
0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a,
0xdd2b, 0xd83a, 0xdd3c, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd3c, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x20, 0xd83a, 0xdd05,
0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd2b,
0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd03,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd34, 0xd83a, 0xdd03, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd23,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd34, 0xd83a, 0xdd3c, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd12, 0xd83a,
0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a,
0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0x2c, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd2c, 0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0x2c, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c,
0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd05,
0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd25, 0x2c, 0x20, 0xd83a, 0xdd10,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a,
0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd10, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd0c, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd3c, 0x20, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a,
0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd24,
0xd83a, 0xdd38, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a,
0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd06, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0x2d,
0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd41,
0xd83a, 0xdd08, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd32,
0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd3c, 0xd83a,
0xdd08, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd2c, 0xd83a, 0xdd2d, 0xd83a,
0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd2e, 0x2d, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33,
0xd83a, 0xdd2e, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd05, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd3a,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd0b, 0xd83a, 0xdd3c, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0xd83a, 0xdd33,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd25, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd05, 0xd83a, 0xdd2d,
0xd83a, 0xdd3c, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a,
0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd25, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a,
0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0x2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a,
0xdd27, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd02,
0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a,
0xdd3c, 0x2d, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd2c, 0xd83a, 0xdd3c,
0x2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd28, 0xd83a,
0xdd22, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd1a, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a,
0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd0f, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd3c, 0xd83a,
0xdd38, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd0f, 0xd83a,
0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd3a, 0xd83a, 0xdd12, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd27, 0xd83a,
0xdd2b, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd27, 0xd83a, 0xdd01, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd3c, 0xd83a, 0xdd35, 0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3e,
0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33,
0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2e,
0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd08, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd35,
0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd36,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd35, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd31, 0xd83a, 0xdd35, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd00, 0xd83a, 0xdd27,
0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd35,
0xd83a, 0xdd00, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd23, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd38,
0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd04,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd2d,
0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd34,
0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd22,
0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd39,
0xd83a, 0xdd22, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd01,
0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd2b,
0xd83a, 0xdd0a, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd35,
0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd1d, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd38,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x2d,
0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd32, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0x20, 0xd83a, 0xdd11,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c,
0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd14,
0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd14, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd28,
0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd17, 0xd83a, 0xdd35, 0xd83a, 0xdd23,
0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd26,
0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd37,
0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd37, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd1d, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2e,
0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd32, 0xd83a, 0xdd2e,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd33,
0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd37, 0xd83a,
0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a,
0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd03, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a,
0xdd35, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a,
0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd41, 0xd83a, 0xdd32, 0xd83a, 0xdd2b, 0xd83a,
0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd10, 0xd83a,
0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd26, 0xd83a,
0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd0c, 0xd83a,
0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd13, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd2e, 0xd83a, 0xdd25, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd06,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd17,
0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd34,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd41, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd3f,
0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd31,
0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd38,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd05,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd23, 0xd83a, 0xdd33, 0xd83a, 0xdd2e,
0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd1a,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd1a,
0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0xd83a, 0xdd35, 0xd83a, 0xdd1a, 0xd83a, 0xdd2d, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd2b,
0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd1a, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd1a,
0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd1a,
0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd13, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd22,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd13, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd25, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd13, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd37, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd12, 0xd83a,
0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd08, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd12, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd12, 0xd83a,
0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd0a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd08, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2c, 0x2d, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd16, 0xd83a, 0xdd2b,
0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd05,
0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd24, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd00, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd34, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd04, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd2d,
0xd83a, 0xdd32, 0x2d, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a,
0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd31, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd13, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd16, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0x2d,
0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2b, 0xd83a, 0xdd03, 0xd83a, 0xdd2b,
0xd83a, 0xdd24, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32,
0xd83a, 0xdd35, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd32, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd00,
0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd00, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3f,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd24,
0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd23,
0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd27, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd04,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2b,
0xd83a, 0xdd24, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd04, 0xd83a, 0xdd35,
0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd36,
0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2e, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd3a, 0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd35,
0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd14, 0xd83a, 0xdd2d, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd14, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd2d,
0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a, 0xdd24,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd05,
0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x2d, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd0b, 0xd83a,
0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a,
0xdd24, 0xd83a, 0xdd14, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd02, 0xd83a,
0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a,
0xdd0b, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a,
0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd02, 0xd83a, 0xdd2d, 0xd83a,
0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a,
0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a,
0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a,
0xdd24, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd38, 0xd83a,
0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd0c, 0xd83a,
0xdd27, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd38, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a,
0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3a, 0xd83a,
0xdd2d, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd08, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a,
0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0x2d, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32,
0xd83a, 0xdd2e, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd34,
0xd83a, 0xdd2b, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd3e,
0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd2c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd28, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd05, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd33,
0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd1a, 0xd83a, 0xdd22,
0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd1a,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd13,
0xd83a, 0xdd24, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd1c, 0xd83a, 0xdd22,
0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd1c,
0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd32, 0xd83a, 0xdd35,
0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd3a,
0xd83a, 0xdd2e, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd23,
0xd83a, 0xdd0f, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd1f,
0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd26, 0xd83a, 0xdd35,
0xd83a, 0xdd1f, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd00,
0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd27,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd25,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd25,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd34,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd08, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd0c, 0xd83a, 0xdd33, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd2d, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd15, 0xd83a, 0xdd22,
0xd83a, 0xdd43, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd15, 0xd83a, 0xdd35,
0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd09, 0xd83a, 0xdd2c, 0xd83a, 0xdd22, 0xd83a, 0xdd3c,
0xd83a, 0xdd35, 0xd83a, 0xdd0a, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd2c,
0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd18, 0xd83a, 0xdd22,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31,
0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd09, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd11,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd11,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd36,
0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd27, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd31, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd31, 0xd83a, 0xdd2d, 0xd83a, 0xdd10,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33,
0xd83a, 0xdd35, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0x2d, 0xd83a,
0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a,
0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a,
0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0x2d, 0xd83a, 0xdd03,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd2d,
0xd83a, 0xdd08, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd34,
0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd1a, 0xd83a, 0xdd22,
0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd1a,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22,
0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd0f, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd0f, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0x41, 0x64, 0x64, 0x69,
0x73, 0x20, 0x41, 0x62, 0xe4, 0x62, 0x61, 0x41, 0x73, 0x6d, 0x61, 0x72,
0xe0, 0x44, 0xe0, 0x72, 0x20, 0x61, 0x73, 0x2d, 0x53, 0x61, 0x6c, 0xe0,
0x6d, 0x44, 0x69, 0x6f, 0x62, 0xf9, 0x74, 0x61, 0x69, 0x64, 0x68, 0x48,
0x61, 0x6e, 0x6e, 0x73, 0x61, 0x62, 0x6f, 0x72, 0x67, 0x68, 0x4e, 0x2019,
0x44, 0x6a, 0x61, 0x6d, 0xe9, 0x6e, 0x61, 0x41, 0x6e, 0x67, 0x75, 0x69,
0x6c, 0x6c, 0x69, 0x61, 0x41, 0x69, 0x6e, 0x74, 0xec, 0x6f, 0x67, 0x61,
0x41, 0x72, 0xf9, 0x62, 0x61, 0x41, 0x2019, 0x20, 0x42, 0x68, 0x65, 0x69,
0x6c, 0xec, 0x73, 0x43, 0x61, 0x69, 0x6d, 0x65, 0x61, 0x6e, 0x43, 0x6f,
0x73, 0x74, 0x61, 0x20, 0x52, 0xec, 0x63, 0x65, 0x61, 0x44, 0x6f, 0x69,
0x6d, 0x69, 0x6e, 0x69, 0x63, 0x65, 0x61, 0x41, 0x6e, 0x20, 0x53, 0x61,
0x6c, 0x62, 0x68, 0x61, 0x64, 0x6f, 0x72, 0x47, 0x6c, 0x61, 0x73, 0x62,
0x61, 0x69, 0x64, 0x68, 0x41, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x63, 0x20,
0x4d, 0x68, 0xf2, 0x72, 0x47, 0x72, 0x65, 0x61, 0x6e, 0xe0, 0x64, 0x61,
0x47, 0x75, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x6c, 0x61, 0x47, 0x75, 0x69,
0x64, 0x68, 0x65, 0xe0, 0x6e, 0x61, 0x49, 0x6e, 0x75, 0x75, 0x76, 0x69,
0x6b, 0x44, 0x69, 0x61, 0x6d, 0x65, 0x75, 0x67, 0x61, 0x4d, 0x61, 0x69,
0x72, 0x74, 0x69, 0x6e, 0x69, 0x63, 0x43, 0x61, 0x74, 0x68, 0x61, 0x69,
0x72, 0x20, 0x4d, 0x68, 0x65, 0x61, 0x67, 0x73, 0x61, 0x67, 0x6f, 0x4d,
0x6f, 0x6e, 0x74, 0x73, 0x61, 0x72, 0x61, 0x74, 0x4e, 0x75, 0x61, 0x64,
0x68, 0x20, 0x45, 0x61, 0x62, 0x68, 0x72, 0x61, 0x63, 0x50, 0x6f, 0x72,
0x74, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x70, 0xe0, 0x69, 0x6e, 0x6e, 0x65,
0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x52, 0xec, 0x63, 0x65, 0x6f, 0x4b,
0x61, 0x6e, 0x67, 0x69, 0x71, 0x6c, 0x69, 0x6e, 0x69, 0x71, 0x51, 0x61,
0x75, 0x73, 0x75, 0x69, 0x74, 0x74, 0x75, 0x71, 0x4e, 0x61, 0x6f, 0x6d,
0x68, 0x20, 0x43, 0x72, 0xec, 0x73, 0x74, 0x65, 0x61, 0x6e, 0x4e, 0x61,
0x6f, 0x6d, 0x68, 0x20, 0x4c, 0xf9, 0x69, 0x73, 0x65, 0x61, 0x4e, 0x61,
0x6f, 0x6d, 0x68, 0x20, 0x42, 0x68, 0x69, 0x6f, 0x6e, 0x73, 0x61, 0x6e,
0x74, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x69, 0x63,
0x47, 0x75, 0x61, 0x69, 0x72, 0x65, 0x41, 0x6d, 0x6d, 0x101, 0x6e, 0x42,
0x61, 0x67, 0x68, 0x64, 0x101, 0x64, 0x42, 0x61, 0x63, 0x68, 0x72, 0x61,
0x69, 0x6e, 0x42, 0x72, 0xf9, 0x6e, 0x61, 0x69, 0x67, 0x68, 0x47, 0xe0,
0x73, 0x61, 0x43, 0x61, 0x74, 0x68, 0x61, 0x69, 0x72, 0x20, 0x48, 0x6f,
0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x49, 0x65, 0x72,
0x75, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x4b, 0x61, 0x72, 0x101, 0x63, 0x12b,
0x43, 0x75, 0x69, 0x62, 0x68, 0xe8, 0x69, 0x74, 0x4d, 0x61, 0x63, 0xe0,
0x74, 0x68, 0x75, 0x50, 0x79, 0x65, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e,
0x67, 0x43, 0x61, 0x74, 0x61, 0x72, 0x51, 0x6f, 0x73, 0x74, 0x61, 0x6e,
0x61, 0x131, 0x53, 0x69, 0x6e, 0x67, 0x65, 0x61, 0x70, 0xf2, 0x72, 0x54,
0x6f, 0x161, 0x6b, 0x65, 0x6e, 0x74, 0x54, 0x2019, 0x62, 0x69, 0x6c, 0x69,
0x73, 0x69, 0x54, 0x65, 0x68, 0x72, 0x101, 0x6e, 0x56, 0x69, 0x61, 0x6e,
0x67, 0x20, 0x43, 0x68, 0x61, 0x6e, 0x50, 0x6f, 0x6e, 0x74, 0x61, 0x20,
0x44, 0x65, 0x6c, 0x67, 0x61, 0x64, 0x61, 0x42, 0x65, 0x61, 0x72, 0x6d,
0xf9, 0x64, 0x61, 0x4e, 0x61, 0x20, 0x68, 0x2d, 0x45, 0x69, 0x6c, 0x65,
0x61, 0x6e, 0x61, 0x6e, 0x20, 0x43, 0x61, 0x6e, 0xe0, 0x72, 0x61, 0x63,
0x68, 0x41, 0x6e, 0x20, 0x43, 0x65, 0x61, 0x70, 0x20, 0x55, 0x61, 0x69,
0x6e, 0x65, 0x46, 0xe0, 0x72, 0x6f, 0x53, 0x65, 0xf2, 0x72, 0x73, 0x65,
0x61, 0x20, 0x61, 0x20, 0x44, 0x65, 0x61, 0x73, 0x45, 0x69, 0x6c, 0x65,
0x61, 0x6e, 0x20, 0x4e, 0x61, 0x6f, 0x6d, 0x68, 0x20, 0x45, 0x69, 0x6c,
0x69, 0x64, 0x68, 0x41, 0x6e, 0x20, 0xc0, 0x69, 0x74, 0x68, 0x6e, 0x65,
0x41, 0x2019, 0x20, 0x42, 0x68, 0x72, 0x75, 0x69, 0x73, 0x65, 0x61, 0x6c,
0x42, 0x65, 0x69, 0x72, 0x62, 0x68, 0x42, 0x61, 0x69, 0x6c, 0x65, 0x20,
0xc0, 0x74, 0x68, 0x61, 0x20, 0x43, 0x6c, 0x69, 0x61, 0x74, 0x68, 0x44,
0x69, 0x6f, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x47, 0x65, 0xe0,
0x72, 0x6e, 0x73, 0x61, 0x69, 0x64, 0x68, 0x45, 0x69, 0x6c, 0x65, 0x61,
0x6e, 0x20, 0x4d, 0x68, 0x61, 0x6e, 0x61, 0x69, 0x6e, 0x6e, 0x44, 0x65,
0xe0, 0x72, 0x73, 0x61, 0x69, 0x64, 0x68, 0x44, 0xf9, 0x6e, 0x20, 0xc8,
0x69, 0x64, 0x65, 0x61, 0x6e, 0x6e, 0x2f, 0x4c, 0x75, 0x6e, 0x6e, 0x61,
0x69, 0x6e, 0x6e, 0x4c, 0x75, 0x67, 0x73, 0x61, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x4d, 0x6f, 0x73, 0x67, 0x6f, 0x50, 0x61, 0x72, 0x61, 0x73, 0x50,
0x72, 0xe0, 0x67, 0x52, 0x12b, 0x67, 0x61, 0x41, 0x6e, 0x20, 0x52, 0xf2,
0x69, 0x6d, 0x68, 0x41, 0x2019, 0x20, 0x42, 0x68, 0x61, 0x74, 0x61, 0x63,
0x61, 0x6e, 0x4e, 0x6f, 0x6c, 0x6c, 0x61, 0x69, 0x67, 0x4e, 0x61, 0x20,
0x68, 0x2d, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d,
0x68, 0x61, 0x6c, 0x61, 0x64, 0x61, 0x69, 0x62, 0x68, 0x4e, 0x61, 0x20,
0x68, 0x2d, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d,
0x68, 0x6f, 0x69, 0x72, 0x69, 0x73, 0x65, 0x61, 0x73, 0x52, 0x61, 0x70,
0x61, 0x20, 0x4e, 0x75, 0x69, 0x46, 0xec, 0x64, 0x69, 0x4d, 0x61, 0x6e,
0x67, 0x61, 0x72, 0x65, 0x76, 0x61, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e,
0x61, 0x6e, 0x20, 0x61, 0x2019, 0x20, 0x4d, 0x68, 0x61, 0x72, 0x63, 0x61,
0x69, 0x73, 0x4e, 0x61, 0x62, 0x68, 0x72, 0x75, 0x50, 0x61, 0x6c, 0x61,
0x62, 0x68, 0x50, 0x65, 0x69, 0x74, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68,
0xe0, 0x69, 0x72, 0x6e, 0x55, 0x61, 0x6c, 0x6c, 0x61, 0x73, 0x4e, 0x6f,
0x75, 0x61, 0x6b, 0x73, 0x68, 0x6f, 0x74, 0x74, 0x41, 0x6e, 0x6b, 0x6f,
0x72, 0x61, 0x67, 0x65, 0x4b, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67,
0x65, 0x20, 0x14a, 0x73, 0x68, 0x254, 0x6e, 0x69, 0x6e, 0x65, 0x20, 0x42,
0x69, 0x62, 0x69, 0x6f, 0x6f, 0x54, 0x73, 0x69, 0x6b, 0x61, 0x67, 0x6f,
0x54, 0x73, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x44, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x20, 0x4b, 0x70, 0x61, 0x61, 0x6b, 0x70, 0x6f, 0x20,
0x42, 0x69, 0x62, 0x69, 0x6f, 0x6f, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e,
0x20, 0x4d, 0x254, 0x254, 0x47, 0x6c, 0x61, 0x73, 0x65, 0x20, 0x14a, 0x73,
0x68, 0x254, 0x6e, 0x69, 0x6e, 0x65, 0x20, 0x42, 0x69, 0x62, 0x69, 0x6f,
0x6f, 0x47, 0x6f, 0x6f, 0x73, 0x65, 0x20, 0x14a, 0x73, 0x68, 0x254, 0x6e,
0x69, 0x6e, 0x65, 0x20, 0x42, 0x69, 0x62, 0x69, 0x6f, 0x6f, 0x54, 0x75,
0x72, 0x6b, 0x65, 0x20, 0x57, 0x75, 0x6c, 0x75, 0x48, 0x61, 0x6c, 0x69,
0x66, 0x61, 0x73, 0x4b, 0x6e, 0x6f, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x4f, 0x73, 0x68, 0x65, 0x6b, 0x75, 0x20, 0x4d,
0x61, 0x14b, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56,
0x69, 0x6e, 0x73, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73, 0x65,
0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b, 0x79,
0x41, 0x62, 0x6c, 0x61, 0x64, 0x65, 0x20, 0x53, 0x68, 0x129, 0x61, 0x20,
0x4e, 0x69, 0x20, 0x59, 0x254, 0x254, 0x20, 0x4a, 0x254, 0x254, 0x20, 0x4d,
0x6c, 0x69, 0x4d, 0x65, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x4d, 0x61, 0x14b,
0x4d, 0x61, 0x14b, 0x74, 0x65, 0x14b, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x62, 0x25b, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x62,
0x25b, 0x41, 0x62, 0x6c, 0x61, 0x64, 0x65, 0x69, 0x20, 0x41, 0x6c, 0x25b,
0x6a, 0x69, 0x61, 0x64, 0x61, 0x61, 0x6d, 0x254, 0x68, 0x65, 0x53, 0x70,
0x61, 0x69, 0x6e, 0x20, 0x4c, 0x25b, 0x6a, 0x69, 0x61, 0x64, 0x61, 0x61,
0x6d, 0x254, 0x68, 0x65, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x14a,
0x73, 0x68, 0x254, 0x6e, 0x69, 0x6e, 0x65, 0x49, 0x74, 0x74, 0x6f, 0x6b,
0x6f, 0x72, 0x74, 0x6f, 0x6f, 0x72, 0x6d, 0x69, 0x69, 0x74, 0x53, 0x74,
0x2e, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x61, 0x53, 0x74, 0x2e, 0x20, 0x56,
0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x53, 0x77, 0x69, 0x66, 0x74, 0x20,
0x4b, 0x61, 0x72, 0x25b, 0x6e, 0x74, 0x56, 0x61, 0x6e, 0x6b, 0x6f, 0x75,
0x76, 0x65, 0x72, 0x4f, 0x6b, 0x70, 0x254, 0x14b, 0x254, 0x20, 0x59, 0x25b,
0x14b, 0x4b, 0x61, 0x73, 0x65, 0x79, 0x4d, 0x61, 0x6b, 0x77, 0x61, 0x72,
0x69, 0x65, 0x4d, 0x61, 0x6b, 0x4d, 0x75, 0x72, 0x64, 0x6f, 0x48, 0x6f,
0x20, 0x54, 0x73, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x20, 0x4d, 0x61,
0x14b, 0x74, 0x69, 0x61, 0x73, 0x65, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x73,
0x69, 0x4b, 0x75, 0x74, 0x73, 0x69, 0x6e, 0x67, 0x55, 0x72, 0x75, 0x6d,
0x6b, 0x69, 0x14a, 0x6d, 0x65, 0x14b, 0x6d, 0x65, 0x20, 0x4e, 0x69, 0x20,
0x45, 0x6b, 0x75, 0x6d, 0x254, 0x4e, 0x75, 0x14b, 0x74, 0x73, 0x254, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x54, 0x73, 0x61, 0x67, 0x6f, 0x73, 0x4b, 0x72,
0x69, 0x73, 0x6d, 0x61, 0x73, 0x186, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x54,
0x73, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x54, 0x73, 0x75, 0x75, 0x6b, 0x4d,
0x61, 0x72, 0x6b, 0x77, 0x65, 0x73, 0x61, 0x73, 0x41, 0x63, 0x72, 0x61,
0x41, 0x6c, 0x78, 0x65, 0x72, 0x42, 0x61, 0x6d, 0x61, 0x63, 0x6f, 0x4f,
0x20, 0x43, 0x61, 0x69, 0x72, 0x6f, 0x4f, 0x20, 0x41, 0x69, 0xfa, 0x6e,
0x58, 0x6f, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x4b, 0x68, 0x61, 0x72, 0x74, 0xfa, 0x6e, 0x4c, 0x75, 0x73, 0x61, 0x63,
0x61, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x78, 0x6f, 0x53, 0x61, 0x6e,
0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x54, 0x75, 0x6e, 0x65, 0x73, 0x42, 0x61,
0xed, 0x61, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x61, 0x69, 0x6d,
0xe1, 0x6e, 0x4f, 0x20, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72,
0x47, 0xfc, 0x69, 0x61, 0x6e, 0x61, 0x41, 0x20, 0x48, 0x61, 0x62, 0x61,
0x6e, 0x61, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0xe1, 0x70, 0x6f, 0x6c,
0x69, 0x73, 0x41, 0x20, 0x50, 0x61, 0x7a, 0x4f, 0x73, 0x20, 0xc1, 0x6e,
0x78, 0x65, 0x6c, 0x65, 0x73, 0x43, 0x69, 0x64, 0x61, 0x64, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x42, 0x65, 0x75,
0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x63, 0x6f, 0x74, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x63, 0x6f, 0x74, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x63, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x20, 0x50, 0x72, 0xed, 0x6e, 0x63, 0x69, 0x70, 0x65, 0x50,
0x6f, 0x72, 0x74, 0x6f, 0x20, 0x45, 0x73, 0x70, 0x61, 0xf1, 0x61, 0x52,
0xed, 0x6f, 0x20, 0x42, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x53, 0x61, 0x6e,
0x74, 0x61, 0x20, 0x4c, 0x75, 0x63, 0xed, 0x61, 0x53, 0x61, 0x6e, 0x20,
0x56, 0x69, 0x63, 0x65, 0x6e, 0x74, 0x65, 0x41, 0x64, 0xe9, 0x6e, 0x41,
0x63, 0x68, 0x6b, 0x68, 0x61, 0x62, 0x61, 0x64, 0x42, 0x61, 0x63, 0xfa,
0x43, 0x68, 0x69, 0x74, 0xe1, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x63, 0x6f,
0x49, 0x61, 0x63, 0x61, 0x72, 0x74, 0x61, 0x43, 0x61, 0x62, 0x75, 0x6c,
0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x6f, 0x49, 0x65, 0x72, 0x65, 0x76, 0xe1, 0x6e, 0x49, 0x6c, 0x6c, 0x61,
0x73, 0x20, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x61, 0x73, 0x49, 0x6c,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x61, 0x73,
0x52, 0x65, 0x69, 0x71, 0x75, 0x69, 0x61, 0x76, 0x69, 0x6b, 0x58, 0x65,
0x6f, 0x72, 0x78, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x72,
0xc1, 0x6d, 0x73, 0x74, 0x65, 0x72, 0x64, 0x61, 0x6d, 0x41, 0x73, 0x74,
0x72, 0x61, 0x6b, 0xe1, 0x6e, 0x42, 0x72, 0x75, 0x78, 0x65, 0x6c, 0x61,
0x73, 0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x71, 0x75, 0x69, 0x49, 0x6c,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x75, 0x78,
0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4d, 0xf3, 0x6e, 0x61, 0x63,
0x6f, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x61, 0x53, 0x61, 0x72, 0x61,
0x69, 0x65, 0x76, 0x6f, 0x53, 0x69, 0x6d, 0x66, 0x65, 0x72, 0xf3, 0x70,
0x6f, 0x6c, 0x45, 0x73, 0x74, 0x6f, 0x63, 0x6f, 0x6c, 0x6d, 0x6f, 0x56,
0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x49, 0x6c, 0x6c, 0x61, 0x20,
0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x61, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69,
0x6f, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73,
0x63, 0x75, 0x61, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x47, 0x61, 0x6c,
0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75,
0x6c, 0xfa, 0x10d0, 0x10d1, 0x10d8, 0x10ef, 0x10d0, 0x10dc, 0x10d8, 0x10d0, 0x10d9, 0x10e0,
0x10d0, 0x10d0, 0x10d3, 0x10d8, 0x10e1, 0x2d, 0x10d0, 0x10d1, 0x10d4, 0x10d1, 0x10d0, 0x10d0,
0x10da, 0x10df, 0x10d8, 0x10e0, 0x10d8, 0x10d0, 0x10e1, 0x10db, 0x10d0, 0x10e0, 0x10d0, 0x10d1,
0x10d0, 0x10db, 0x10d0, 0x10d9, 0x10dd, 0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10d8, 0x10d1, 0x10d0,
0x10dc, 0x10ef, 0x10e3, 0x10da, 0x10d8, 0x10d1, 0x10d8, 0x10e1, 0x10d0, 0x10e3, 0x10d1, 0x10da,
0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10e0, 0x10d8, 0x10d1, 0x10e0, 0x10d0, 0x10d6, 0x10d0, 0x10d5,
0x10d8, 0x10da, 0x10d8, 0x10d1, 0x10e3, 0x10ef, 0x10e3, 0x10db, 0x10d1, 0x10e3, 0x10e0, 0x10d0,
0x10d9, 0x10d0, 0x10d8, 0x10e0, 0x10dd, 0x10d9, 0x10d0, 0x10e1, 0x10d0, 0x10d1, 0x10da, 0x10d0,
0x10dc, 0x10d9, 0x10d0, 0x10e1, 0x10d4, 0x10e3, 0x10e2, 0x10d0, 0x10d9, 0x10dd, 0x10dc, 0x10d0,
0x10d9, 0x10e0, 0x10d8, 0x10d3, 0x10d0, 0x10d9, 0x10d0, 0x10e0, 0x10d8, 0x10d3, 0x10d0, 0x10e0,
0x2d, 0x10d4, 0x10e1, 0x2d, 0x10e1, 0x10d0, 0x10da, 0x10d0, 0x10db, 0x10d8, 0x10ef, 0x10d8,
0x10d1, 0x10e3, 0x10e2, 0x10d8, 0x10d3, 0x10e3, 0x10d0, 0x10da, 0x10d0, 0x10d4, 0x10da, 0x2d,
0x10d0, 0x10d0, 0x10d8, 0x10e3, 0x10dc, 0x10d8, 0x10e4, 0x10e0, 0x10d8, 0x10e2, 0x10d0, 0x10e3,
0x10dc, 0x10d8, 0x10d2, 0x10d0, 0x10d1, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10d4, 0x10f0, 0x10d0,
0x10e0, 0x10d0, 0x10e0, 0x10d4, 0x10d8, 0x10dd, 0x10f0, 0x10d0, 0x10dc, 0x10d4, 0x10e1, 0x10d1,
0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10ef, 0x10e3, 0x10d1, 0x10d0, 0x10d9, 0x10d0, 0x10db, 0x10de,
0x10d0, 0x10da, 0x10d0, 0x10ee, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10db, 0x10d8, 0x10d9, 0x10d8,
0x10d2, 0x10d0, 0x10da, 0x10d8, 0x10d9, 0x10d8, 0x10dc, 0x10e8, 0x10d0, 0x10e1, 0x10d0, 0x10da,
0x10d0, 0x10d2, 0x10dd, 0x10e1, 0x10d8, 0x10da, 0x10d8, 0x10d1, 0x10e0, 0x10d4, 0x10d5, 0x10d8,
0x10da, 0x10d8, 0x10da, 0x10dd, 0x10db, 0x10d4, 0x10da, 0x10e3, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10da, 0x10e3, 0x10d1, 0x10e3, 0x10db, 0x10d1, 0x10d0, 0x10e8, 0x10d8, 0x10da, 0x10e3, 0x10e1,
0x10d0, 0x10d9, 0x10d0, 0x10db, 0x10d0, 0x10da, 0x10d0, 0x10d1, 0x10dd, 0x10db, 0x10d0, 0x10de,
0x10e3, 0x10e2, 0x10dd, 0x10db, 0x10d0, 0x10e1, 0x10d4, 0x10e0, 0x10e3, 0x10db, 0x10d1, 0x10d0,
0x10d1, 0x10d0, 0x10dc, 0x10d4, 0x10db, 0x10dd, 0x10d2, 0x10d0, 0x10d3, 0x10d8, 0x10e8, 0x10e3,
0x10db, 0x10dd, 0x10dc, 0x10e0, 0x10dd, 0x10d5, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d8, 0x10e0,
0x10dd, 0x10d1, 0x10d8, 0x10dc, 0x10ef, 0x10d0, 0x10db, 0x10d4, 0x10dc, 0x10d0, 0x10dc, 0x10d8,
0x10d0, 0x10db, 0x10d4, 0x10d8, 0x10dc, 0x10e3, 0x10d0, 0x10d9, 0x10e8, 0x10dd, 0x10e2, 0x10d8,
0x10e3, 0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10e3, 0x10d2, 0x10e3, 0x10de, 0x10dd, 0x10e0, 0x10e2,
0x10dd, 0x2d, 0x10dc, 0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10e2, 0x10dd,
0x10db, 0x10d4, 0x10e2, 0x10e0, 0x10d8, 0x10de, 0x10dd, 0x10da, 0x10d8, 0x10e2, 0x10e3, 0x10dc,
0x10d8, 0x10e1, 0x10d8, 0x10d5, 0x10d8, 0x10dc, 0x10d3, 0x10ee, 0x10e3, 0x10d9, 0x10d8, 0x10d0,
0x10d3, 0x10d0, 0x10d9, 0x10d8, 0x10d4, 0x10dc, 0x10e5, 0x10dd, 0x10e0, 0x10d0, 0x10ef, 0x10d8,
0x10d0, 0x10dc, 0x10d2, 0x10d8, 0x10da, 0x10d8, 0x10d0, 0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10d2,
0x10e3, 0x10d0, 0x10d0, 0x10e0, 0x10d0, 0x10d2, 0x10e3, 0x10d0, 0x10d8, 0x10dc, 0x10d0, 0x10d1,
0x10e3, 0x10d4, 0x10dc, 0x10dd, 0x10e1, 0x2d, 0x10d0, 0x10d8, 0x10e0, 0x10d4, 0x10e1, 0x10d8,
0x10d9, 0x10d0, 0x10e2, 0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10d9, 0x10d0, 0x10d9, 0x10dd, 0x10e0,
0x10d3, 0x10dd, 0x10d1, 0x10d0, 0x10ef, 0x10e3, 0x10ef, 0x10e3, 0x10d8, 0x10da, 0x10d0, 0x20,
0x10e0, 0x10d8, 0x10dd, 0x10ee, 0x10d0, 0x10db, 0x10d4, 0x10dc, 0x10d3, 0x10dd, 0x10d6, 0x10d0,
0x10e0, 0x10d8, 0x10dd, 0x20, 0x10d2, 0x10d0, 0x10da, 0x10d4, 0x10d2, 0x10dd, 0x10e1, 0x10d8,
0x10e1, 0x10d0, 0x10da, 0x10e2, 0x10d0, 0x10e1, 0x10d0, 0x10dc, 0x20, 0x10ee, 0x10e3, 0x10d0,
0x10dc, 0x10d8, 0x10e1, 0x10d0, 0x10dc, 0x2d, 0x10da, 0x10e3, 0x10d8, 0x10e1, 0x10d8, 0x10e2,
0x10e3, 0x10d9, 0x10e3, 0x10db, 0x10d0, 0x10dc, 0x10d8, 0x10e3, 0x10e8, 0x10e3, 0x10d0, 0x10d8,
0x10d0, 0x10d0, 0x10e0, 0x10e3, 0x10d1, 0x10d0, 0x10d0, 0x10e1, 0x10e3, 0x10dc, 0x10e1, 0x10d8,
0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10e2, 0x10d8, 0x10d9, 0x10dd, 0x10d9, 0x10d0, 0x10dc, 0x10d8,
0x10d1, 0x10d0, 0x10d8, 0x10d0, 0x10d1, 0x10d0, 0x10f0, 0x10d8, 0x10d0, 0x20, 0x10d1, 0x10d0,
0x10dc, 0x10d3, 0x10d4, 0x10e0, 0x10d0, 0x10e1, 0x10d8, 0x10d1, 0x10d0, 0x10e0, 0x10d1, 0x10d0,
0x10d3, 0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d4, 0x10da, 0x10d4, 0x10db, 0x10d8, 0x10d1, 0x10d4,
0x10da, 0x10d8, 0x10d6, 0x10d8, 0x10d1, 0x10da, 0x10d0, 0x10dc, 0x2d, 0x10e1, 0x10d0, 0x10d1,
0x10da, 0x10dd, 0x10dc, 0x10d8, 0x10d1, 0x10dd, 0x10d0, 0x20, 0x10d5, 0x10d8, 0x10e1, 0x10e2,
0x10d0, 0x10d1, 0x10dd, 0x10d2, 0x10dd, 0x10e2, 0x10d0, 0x10d1, 0x10e3, 0x10d0, 0x10e1, 0x10d8,
0x10d9, 0x10d4, 0x10db, 0x10d1, 0x10e0, 0x10d8, 0x10ef, 0x20, 0x10d1, 0x10d4, 0x10d8, 0x10d9,
0x10d0, 0x10db, 0x10de, 0x10dd, 0x20, 0x10d2, 0x10e0, 0x10d0, 0x10dc, 0x10d3, 0x10d4, 0x10d9,
0x10d0, 0x10dc, 0x10d9, 0x10e3, 0x10dc, 0x10d8, 0x10d9, 0x10d0, 0x10e0, 0x10d0, 0x10d9, 0x10d0,
0x10e1, 0x10d8, 0x10d9, 0x10d0, 0x10d8, 0x10d4, 0x10dc, 0x10d0, 0x10d9, 0x10d0, 0x10d8, 0x10db,
0x10d0, 0x10dc, 0x10d8, 0x10e9, 0x10d8, 0x10d9, 0x10d0, 0x10d2, 0x10dd, 0x10e9, 0x10d8, 0x10f0,
0x10e3, 0x10d0, 0x10f0, 0x10e3, 0x10d0, 0x10e1, 0x10d8, 0x10e3, 0x10d3, 0x10d0, 0x10d3, 0x2d,
0x10ee, 0x10e3, 0x10d0, 0x10e0, 0x10d4, 0x10e1, 0x10d8, 0x10d9, 0x10dd, 0x10e1, 0x10e2, 0x10d0,
0x2d, 0x10e0, 0x10d8, 0x10d9, 0x10d0, 0x10d9, 0x10e0, 0x10d4, 0x10e1, 0x10e2, 0x10dd, 0x10dc,
0x10d8, 0x10d9, 0x10e3, 0x10d8, 0x10d0, 0x10d1, 0x10d0, 0x10d9, 0x10d8, 0x10e3, 0x10e0, 0x10d0,
0x10e1, 0x10d0, 0x10dd, 0x10d3, 0x10d4, 0x10dc, 0x10db, 0x10d0, 0x10e0, 0x10d9, 0x10e8, 0x10d0,
0x10d5, 0x10dc, 0x10d8, 0x10d3, 0x10dd, 0x10e3, 0x10e1, 0x10dd, 0x10dc, 0x10d8, 0x10d3, 0x10dd,
0x10e3, 0x10e1, 0x10dd, 0x10dc, 0x20, 0x10e5, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10d3, 0x10d4,
0x10dc, 0x10d5, 0x10d4, 0x10e0, 0x10d8, 0x10d3, 0x10d4, 0x10e2, 0x10e0, 0x10dd, 0x10d8, 0x10e2,
0x10d8, 0x10d3, 0x10dd, 0x10db, 0x10d8, 0x10dc, 0x10d8, 0x10d9, 0x10d0, 0x10d4, 0x10d3, 0x10db,
0x10dd, 0x10dc, 0x10e2, 0x10dd, 0x10dc, 0x10d8, 0x10d4, 0x10d8, 0x10e0, 0x10e3, 0x10dc, 0x10d4,
0x10de, 0x10d4, 0x10e1, 0x10d0, 0x10da, 0x10d5, 0x10d0, 0x10d3, 0x10dd, 0x10e0, 0x10d8, 0x10e4,
0x10dd, 0x10e0, 0x10e2, 0x2d, 0x10dc, 0x10d4, 0x10da, 0x10e1, 0x10dd, 0x10dc, 0x10d8, 0x10e4,
0x10dd, 0x10e0, 0x10e2, 0x10d0, 0x10da, 0x10d4, 0x10d6, 0x10d0, 0x10d2, 0x10da, 0x10d4, 0x10d8,
0x10e1, 0x20, 0x10d1, 0x10d4, 0x10d8, 0x10d2, 0x10e3, 0x10d6, 0x20, 0x10d1, 0x10d4, 0x10d8,
0x10d2, 0x10e0, 0x10d0, 0x10dc, 0x10d3, 0x2d, 0x10e2, 0x10e3, 0x10e0, 0x10d9, 0x10d8, 0x10d2,
0x10e0, 0x10d4, 0x10dc, 0x10d0, 0x10d3, 0x10d0, 0x10d2, 0x10d5, 0x10d0, 0x10d3, 0x10d4, 0x10da,
0x10e3, 0x10de, 0x10d0, 0x10d2, 0x10d5, 0x10d0, 0x10e2, 0x10d4, 0x10db, 0x10d0, 0x10da, 0x10d0,
0x10d2, 0x10e3, 0x10d0, 0x10d8, 0x10d0, 0x10d9, 0x10d8, 0x10da, 0x10d8, 0x10d2, 0x10d0, 0x10d8,
0x10d0, 0x10dc, 0x10d0, 0x10f0, 0x10d0, 0x10da, 0x10d8, 0x10e4, 0x10d0, 0x10e5, 0x10e1, 0x10d8,
0x10f0, 0x10d0, 0x10d5, 0x10d0, 0x10dc, 0x10d0, 0x10f0, 0x10d4, 0x10e0, 0x10db, 0x10dd, 0x10e1,
0x10d8, 0x10da, 0x10dd, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10de, 0x10dd,
0x10da, 0x10d8, 0x10e1, 0x10d8, 0x10dc, 0x10dd, 0x10e5, 0x10e1, 0x10d8, 0x2c, 0x20, 0x10d8,
0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10d4, 0x10dc, 0x10d2,
0x10dd, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10de, 0x10d8,
0x10e2, 0x10d4, 0x10e0, 0x10e1, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x2c, 0x20, 0x10d8,
0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d7, 0x10d4, 0x10da, 0x20, 0x10e1, 0x10d8,
0x10d7, 0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d5,
0x10d8, 0x10d5, 0x10d4, 0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc,
0x10d0, 0x10d5, 0x10d8, 0x10dc, 0x10e1, 0x10d4, 0x10dc, 0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc,
0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10e3, 0x10d8, 0x10dc, 0x10d4, 0x10db, 0x10d0, 0x10d9,
0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d8, 0x10dc,
0x10e3, 0x10d5, 0x10d8, 0x10d9, 0x10d8, 0x10d8, 0x10e5, 0x10d0, 0x10da, 0x10e3, 0x10d8, 0x10e2,
0x10d8, 0x10d8, 0x10d0, 0x10db, 0x10d0, 0x10d8, 0x10d9, 0x10d0, 0x10ef, 0x10e3, 0x10dc, 0x10dd,
0x10da, 0x10e3, 0x10d8, 0x10e1, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10db, 0x10dd, 0x10dc, 0x10e2,
0x10d8, 0x10e9, 0x10d4, 0x10da, 0x10dd, 0x2c, 0x20, 0x10d9, 0x10d4, 0x10dc, 0x10e2, 0x10e3,
0x10d9, 0x10d8, 0x10d9, 0x10e0, 0x10d0, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10d9,
0x10d8, 0x10da, 0x10d0, 0x2d, 0x10de, 0x10d0, 0x10d6, 0x10d8, 0x10da, 0x10d8, 0x10db, 0x10d0,
0x10da, 0x10dd, 0x10e1, 0x2d, 0x10d0, 0x10dc, 0x10ef, 0x10d4, 0x10da, 0x10d4, 0x10e1, 0x10d8,
0x10da, 0x10dd, 0x10d4, 0x10e0, 0x20, 0x10de, 0x10e0, 0x10d8, 0x10dc, 0x10ea, 0x20, 0x10d9,
0x10d5, 0x10dd, 0x10e0, 0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10db, 0x10d0, 0x10e1, 0x10d4, 0x10d8,
0x10dd, 0x10db, 0x10d0, 0x10dc, 0x10d0, 0x10d2, 0x10e3, 0x10d0, 0x10db, 0x10d0, 0x10dc, 0x10d0,
0x10e3, 0x10e1, 0x10d8, 0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10d2, 0x10dd, 0x10db, 0x10d0, 0x10e0,
0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10d9, 0x10d8, 0x10db, 0x10d0, 0x10e2, 0x10d0, 0x10db, 0x10dd,
0x10e0, 0x10dd, 0x10e1, 0x10d8, 0x10db, 0x10d0, 0x10d6, 0x10d0, 0x10e2, 0x10da, 0x10d0, 0x10dc,
0x10d8, 0x10db, 0x10d4, 0x10dc, 0x10dd, 0x10db, 0x10d8, 0x10dc, 0x10d8, 0x10db, 0x10d4, 0x10e0,
0x10d8, 0x10d3, 0x10d0, 0x10db, 0x10d4, 0x10e2, 0x10da, 0x10d0, 0x10d9, 0x10d0, 0x10e2, 0x10da,
0x10d0, 0x10db, 0x10d4, 0x10ee, 0x10d8, 0x10d9, 0x10dd, 0x10db, 0x10d8, 0x10d9, 0x10d4, 0x10da,
0x10dd, 0x10dc, 0x10d8, 0x10db, 0x10dd, 0x10dc, 0x10e5, 0x10e2, 0x10dd, 0x10dc, 0x10d8, 0x10db,
0x10dd, 0x10dc, 0x10e2, 0x10d4, 0x10e0, 0x10d4, 0x10d8, 0x10db, 0x10dd, 0x10dc, 0x10e2, 0x10d4,
0x10d5, 0x10d8, 0x10d3, 0x10d4, 0x10dd, 0x10db, 0x10dd, 0x10dc, 0x10e1, 0x10d4, 0x10e0, 0x10d0,
0x10e2, 0x10d8, 0x10dc, 0x10d0, 0x10e1, 0x10d0, 0x10e3, 0x10dc, 0x10d8, 0x10e3, 0x2d, 0x10d8,
0x10dd, 0x10e0, 0x10d9, 0x10d8, 0x10dc, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10d1,
0x10d8, 0x10e3, 0x10da, 0x10d0, 0x2c, 0x20, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd,
0x10d4, 0x10d7, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x10d9, 0x10dd, 0x10e2, 0x10d0, 0x10ea, 0x10d4,
0x10dc, 0x10e2, 0x10e0, 0x10d8, 0x2c, 0x20, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd,
0x10d4, 0x10d7, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x10d9, 0x10dd, 0x10e2, 0x10d0, 0x10dc, 0x10d8,
0x10e3, 0x2d, 0x10e1, 0x10d0, 0x10da, 0x10d4, 0x10db, 0x10d8, 0x2c, 0x20, 0x10e9, 0x10e0,
0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x10d9, 0x10dd,
0x10e2, 0x10d0, 0x10d2, 0x10dd, 0x10d3, 0x10d7, 0x10d4, 0x10d1, 0x10d8, 0x10dd, 0x10ee, 0x10d8,
0x10dc, 0x10d0, 0x10d2, 0x10d0, 0x10de, 0x10d0, 0x10dc, 0x10d0, 0x10db, 0x10d0, 0x10de, 0x10d0,
0x10e0, 0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10d1, 0x10dd, 0x10e4, 0x10d4, 0x10dc, 0x10d8,
0x10e5, 0x10e1, 0x10d8, 0x10de, 0x10dd, 0x10e0, 0x10e2, 0x2d, 0x10d0, 0x2d, 0x10de, 0x10e0,
0x10d8, 0x10dc, 0x10e1, 0x10d8, 0x10de, 0x10dd, 0x10e0, 0x10e2, 0x2d, 0x10dd, 0x10d5, 0x2d,
0x10e1, 0x10de, 0x10d4, 0x10d8, 0x10dc, 0x10d8, 0x10de, 0x10dd, 0x10e0, 0x10e2, 0x10e3, 0x2d,
0x10d5, 0x10d4, 0x10da, 0x10d8, 0x10dd, 0x10de, 0x10e3, 0x10d4, 0x10e0, 0x10e2, 0x10dd, 0x2d,
0x10e0, 0x10d8, 0x10d9, 0x10dd, 0x10de, 0x10e3, 0x10dc, 0x10e2, 0x10d0, 0x2d, 0x10d0, 0x10e0,
0x10d4, 0x10dc, 0x10d0, 0x10e1, 0x10d8, 0x10e0, 0x10d4, 0x10dc, 0x10d9, 0x10d8, 0x10dc, 0x20,
0x10d8, 0x10dc, 0x10da, 0x10d4, 0x10e2, 0x10d8, 0x10e0, 0x10d4, 0x10ea, 0x10d8, 0x10e4, 0x10d4,
0x10e0, 0x10d4, 0x10ef, 0x10d8, 0x10dc, 0x10d0, 0x10e0, 0x10d4, 0x10d6, 0x10dd, 0x10da, 0x10e3,
0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10dd, 0x20, 0x10d1, 0x10e0, 0x10d0, 0x10dc, 0x10d9, 0x10dd,
0x10e1, 0x10d0, 0x10dc, 0x10e2, 0x10d0, 0x10e0, 0x10d4, 0x10db, 0x10d8, 0x10e1, 0x10d0, 0x10dc,
0x10e2, 0x10d8, 0x10d0, 0x10d2, 0x10dd, 0x10e1, 0x10d0, 0x10dc, 0x10e2, 0x10dd, 0x2d, 0x10d3,
0x10dd, 0x10db, 0x10d8, 0x10dc, 0x10d2, 0x10dd, 0x10e1, 0x10d0, 0x10dc, 0x2d, 0x10de, 0x10d0,
0x10e3, 0x10da, 0x10e3, 0x10e1, 0x10d9, 0x10dd, 0x10e0, 0x10e1, 0x10d1, 0x10d8, 0x10e1, 0x10e3,
0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x10d8, 0x10e2, 0x10d9, 0x10d0, 0x10e1, 0x10d4, 0x10dc, 0x2d,
0x10d1, 0x10d0, 0x10e0, 0x10d7, 0x10d4, 0x10da, 0x10d4, 0x10db, 0x10d8, 0x10e1, 0x10d4, 0x10dc,
0x10e2, 0x2d, 0x10ef, 0x10dd, 0x10dc, 0x10e1, 0x10d8, 0x10e1, 0x10d4, 0x10dc, 0x10e2, 0x2d,
0x10d9, 0x10d8, 0x10e2, 0x10e1, 0x10d8, 0x10e1, 0x10d4, 0x10dc, 0x10e2, 0x2d, 0x10da, 0x10e3,
0x10e1, 0x10d8, 0x10d0, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10e2, 0x10dd, 0x10db, 0x10d0, 0x10e1,
0x10d8, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10d5, 0x10d8, 0x10dc, 0x10e1, 0x10d4, 0x10dc, 0x10e2,
0x10d8, 0x10e1, 0x10d5, 0x10d8, 0x10e4, 0x10e2, 0x20, 0x10d9, 0x10d0, 0x10e0, 0x10d4, 0x10dc,
0x10e2, 0x10d8, 0x10e2, 0x10d4, 0x10d2, 0x10e3, 0x10e9, 0x10d8, 0x10d2, 0x10d0, 0x10da, 0x10de,
0x10d0, 0x10d7, 0x10e3, 0x10da, 0x10d4, 0x10e2, 0x10d8, 0x10ee, 0x10e3, 0x10d0, 0x10dc, 0x10d0,
0x10e2, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10e2, 0x10dd, 0x10e2, 0x10dd, 0x10e0, 0x10e2, 0x10dd,
0x10da, 0x10d0, 0x10d5, 0x10d0, 0x10dc, 0x10d9, 0x10e3, 0x10d5, 0x10d4, 0x10e0, 0x10d8, 0x10e3,
0x10d0, 0x10d8, 0x10d7, 0x10f0, 0x10dd, 0x10e0, 0x10e1, 0x10d8, 0x10e3, 0x10d8, 0x10dc, 0x10d8,
0x10de, 0x10d4, 0x10d2, 0x10d8, 0x10d8, 0x10d0, 0x10d9, 0x10e3, 0x10e2, 0x10d0, 0x10e2, 0x10d8,
0x10d9, 0x10d4, 0x10d8, 0x10e1, 0x10d8, 0x10d3, 0x10d4, 0x10d5, 0x10d8, 0x10e1, 0x10d8, 0x10d3,
0x10d8, 0x10e3, 0x10db, 0x10dd, 0x10dc, 0x20, 0x10d3, 0x2019, 0x10e3, 0x10e0, 0x10d5, 0x10d8,
0x10da, 0x10d8, 0x10db, 0x10d4, 0x10e5, 0x10d5, 0x10dd, 0x10e0, 0x10d8, 0x10db, 0x10dd, 0x10e3,
0x10e1, 0x10dd, 0x10dc, 0x10d8, 0x10db, 0x10d0, 0x10d9, 0x10db, 0x10e3, 0x10e0, 0x10d3, 0x10dd,
0x10de, 0x10d0, 0x10da, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10e0, 0x10dd, 0x10d7, 0x10d4, 0x10e0,
0x10d0, 0x10e1, 0x10d8, 0x10dd, 0x10e3, 0x10d0, 0x10e2, 0x10e0, 0x10dd, 0x10da, 0x10d8, 0x10d5,
0x10dd, 0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10da, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e0,
0x10d1, 0x10d8, 0x10d4, 0x10dc, 0x10d8, 0x10d0, 0x10d3, 0x10d4, 0x10dc, 0x10d8, 0x10d0, 0x10da,
0x10db, 0x10d0, 0x10e2, 0x10d8, 0x10d0, 0x10db, 0x10d0, 0x10dc, 0x10d8, 0x10d0, 0x10dc, 0x10d0,
0x10d3, 0x10d8, 0x10e0, 0x10d8, 0x10d0, 0x10e5, 0x10e2, 0x10d0, 0x10e3, 0x10d0, 0x10e5, 0x10e2,
0x10dd, 0x10d1, 0x10d4, 0x10d0, 0x10e8, 0x10ee, 0x10d0, 0x10d1, 0x10d0, 0x10d3, 0x10d8, 0x10d0,
0x10e2, 0x10d8, 0x10e0, 0x10d0, 0x10e3, 0x10d1, 0x10d0, 0x10e6, 0x10d3, 0x10d0, 0x10d3, 0x10d8,
0x10d1, 0x10d0, 0x10f0, 0x10e0, 0x10d4, 0x10d8, 0x10dc, 0x10d8, 0x10d1, 0x10d0, 0x10e5, 0x10dd,
0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10d9, 0x10dd, 0x10d9, 0x10d8, 0x10d1, 0x10d0, 0x10e0, 0x10dc,
0x10d0, 0x10e3, 0x10da, 0x10d8, 0x10d1, 0x10d4, 0x10d8, 0x10e0, 0x10e3, 0x10d7, 0x10d8, 0x10d1,
0x10d8, 0x10e8, 0x10d9, 0x10d4, 0x10d9, 0x10d8, 0x10d1, 0x10e0, 0x10e3, 0x10dc, 0x10d4, 0x10d8,
0x10e9, 0x10d8, 0x10e2, 0x10d0, 0x10d9, 0x10dd, 0x10da, 0x10dd, 0x10db, 0x10d1, 0x10dd, 0x10d3,
0x10d0, 0x10db, 0x10d0, 0x10e1, 0x10d9, 0x10d8, 0x10d3, 0x10d8, 0x10da, 0x10d8, 0x10d3, 0x10e3,
0x10d1, 0x10d0, 0x10d8, 0x10d3, 0x10e3, 0x10e8, 0x10d0, 0x10dc, 0x10d1, 0x10d4, 0x10e4, 0x10d0,
0x10db, 0x10d0, 0x10d2, 0x10e3, 0x10e1, 0x10e2, 0x10d0, 0x10d2, 0x10d0, 0x10d6, 0x10d0, 0x10f0,
0x10d4, 0x10d1, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10f0, 0x10dd, 0x10e9, 0x10d8, 0x10db, 0x10d8,
0x10dc, 0x10d8, 0x10f0, 0x10dd, 0x10dc, 0x10d9, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10f0, 0x10dd,
0x10d5, 0x10d3, 0x10d8, 0x10d8, 0x10e0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10ef,
0x10d0, 0x10d9, 0x10d0, 0x10e0, 0x10e2, 0x10d0, 0x10ef, 0x10d0, 0x10d8, 0x10d0, 0x10de, 0x10e3,
0x10e0, 0x10d0, 0x10d8, 0x10d4, 0x10e0, 0x10e3, 0x10e1, 0x10d0, 0x10da, 0x10d8, 0x10db, 0x10d8,
0x10e5, 0x10d0, 0x10d1, 0x10e3, 0x10da, 0x10d8, 0x10d9, 0x10d0, 0x10db, 0x10e9, 0x10d0, 0x10e2,
0x10d9, 0x10d0, 0x10d9, 0x10d0, 0x10e0, 0x10d0, 0x10e9, 0x10d8, 0x10d9, 0x10d0, 0x10e2, 0x10db,
0x10d0, 0x10dc, 0x10d3, 0x10e3, 0x10ee, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d2, 0x10d0, 0x10d9,
0x10d0, 0x10da, 0x10d9, 0x10e3, 0x10e2, 0x10d0, 0x10d9, 0x10e0, 0x10d0, 0x10e1, 0x10dc, 0x10dd,
0x10d8, 0x10d0, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10d9, 0x10e3, 0x10d0, 0x10da, 0x10d0, 0x2d,
0x10da, 0x10e3, 0x10db, 0x10de, 0x10e3, 0x10e0, 0x10d8, 0x10d9, 0x10e3, 0x10e9, 0x10d8, 0x10dc,
0x10d2, 0x10d8, 0x10e5, 0x10e3, 0x10d5, 0x10d4, 0x10d8, 0x10d7, 0x10d8, 0x10db, 0x10d0, 0x10d9,
0x10d0, 0x10dd, 0x10db, 0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10d0, 0x10dc, 0x10d8, 0x10db, 0x10d0,
0x10d9, 0x10d0, 0x10e1, 0x10d0, 0x10e0, 0x10d8, 0x10db, 0x10d0, 0x10dc, 0x10d8, 0x10da, 0x10d0,
0x10db, 0x10e3, 0x10e1, 0x10d9, 0x10d0, 0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10d9, 0x10dd, 0x10e1,
0x10d8, 0x10d0, 0x10dc, 0x10dd, 0x10d5, 0x10dd, 0x10d9, 0x10e3, 0x10d6, 0x10dc, 0x10d4, 0x10ea,
0x10d9, 0x10d8, 0x10dc, 0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d8, 0x10e0, 0x10e1,
0x10d9, 0x10d8, 0x10dd, 0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10dd, 0x10e0, 0x10d0, 0x10da, 0x10d8,
0x10de, 0x10dc, 0x10dd, 0x10db, 0x10de, 0x10d4, 0x10dc, 0x10d8, 0x10de, 0x10dd, 0x10dc, 0x10e2,
0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d9, 0x10d8, 0x10e4, 0x10ee, 0x10d4, 0x10dc, 0x10d8, 0x10d0,
0x10dc, 0x10d8, 0x10d9, 0x10d0, 0x10e2, 0x10d0, 0x10e0, 0x10d8, 0x10d9, 0x10dd, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d0, 0x10d8, 0x10e7, 0x10d8, 0x10d6, 0x10d8, 0x10da, 0x10dd, 0x10e0, 0x10d3,
0x10d0, 0x10d4, 0x10e0, 0x2d, 0x10e0, 0x10d8, 0x10d0, 0x10d3, 0x10d8, 0x10e1, 0x10d0, 0x10ee,
0x10d0, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10e7, 0x10d0,
0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x10d4, 0x10e3, 0x10da, 0x10d8, 0x10e8, 0x10d0, 0x10dc, 0x10ee,
0x10d0, 0x10d8, 0x10e1, 0x10d8, 0x10dc, 0x10d2, 0x10d0, 0x10de, 0x10e3, 0x10e0, 0x10d8, 0x10e1,
0x10e0, 0x10d4, 0x10d3, 0x10dc, 0x10d4, 0x10d9, 0x10dd, 0x10da, 0x10d8, 0x10db, 0x10e1, 0x10d9,
0x10d8, 0x10e2, 0x10d0, 0x10d8, 0x10de, 0x10d4, 0x10d8, 0x10e2, 0x10d0, 0x10e8, 0x10d9, 0x10d4,
0x10dc, 0x10e2, 0x10d8, 0x10d7, 0x10d1, 0x10d8, 0x10da, 0x10d8, 0x10e1, 0x10d8, 0x10d7, 0x10d4,
0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d8, 0x10d7, 0x10ee, 0x10d8, 0x10db, 0x10e4, 0x10ee, 0x10e3,
0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10dd, 0x10e2, 0x10dd, 0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10e3,
0x10da, 0x10d0, 0x10dc, 0x10d1, 0x10d0, 0x10e2, 0x10d0, 0x10e0, 0x10d8, 0x10e3, 0x10e0, 0x10e3,
0x10db, 0x10e5, 0x10d8, 0x10e3, 0x10e1, 0x10e2, 0x2d, 0x10dc, 0x10d4, 0x10e0, 0x10d0, 0x10d5,
0x10d8, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10d5, 0x10da, 0x10d0, 0x10d3,
0x10d8, 0x10d5, 0x10dd, 0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10d8, 0x10d0, 0x10d9, 0x10e3,
0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d2, 0x10e3, 0x10dc, 0x10d8, 0x10d4,
0x10d9, 0x10d0, 0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10dc, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8,
0x10d4, 0x10e0, 0x10d4, 0x10d5, 0x10d0, 0x10dc, 0x10d8, 0x10d0, 0x10d6, 0x10dd, 0x10e0, 0x10d8,
0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10d1,
0x10d4, 0x10e0, 0x10db, 0x10e3, 0x10d3, 0x10d0, 0x10d9, 0x10d0, 0x10dc, 0x10d0, 0x10e0, 0x10d8,
0x10d9, 0x10d0, 0x10d1, 0x10dd, 0x2d, 0x10d5, 0x10d4, 0x10e0, 0x10d3, 0x10d4, 0x10e4, 0x10d0,
0x10e0, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da,
0x10d4, 0x10d1, 0x10d8, 0x10db, 0x10d0, 0x10d3, 0x10d4, 0x10d8, 0x10e0, 0x10d0, 0x10e0, 0x10d4,
0x10d8, 0x10d9, 0x10d8, 0x10d0, 0x10d5, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x10d0, 0x10db, 0x10ee,
0x10e0, 0x10d4, 0x10d7, 0x20, 0x10ef, 0x10dd, 0x10e0, 0x10ef, 0x10d8, 0x10d0, 0x10ec, 0x10db,
0x2e, 0x20, 0x10d4, 0x10da, 0x10d4, 0x10dc, 0x10d4, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc,
0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x10e2, 0x10d4, 0x10dc, 0x10da, 0x10d8, 0x10d0, 0x10d3,
0x10d4, 0x10da, 0x10d0, 0x10d8, 0x10d3, 0x10d0, 0x10d1, 0x10e0, 0x10d8, 0x10e1, 0x10d1, 0x10d4,
0x10d8, 0x10dc, 0x10d8, 0x10d1, 0x10e0, 0x10dd, 0x10e3, 0x10d9, 0x10d4, 0x10dc, 0x20, 0x10f0,
0x10d8, 0x10da, 0x10d8, 0x10d3, 0x10d0, 0x10e0, 0x10d5, 0x10d8, 0x10dc, 0x10d8, 0x10d4, 0x10e3,
0x10d9, 0x10da, 0x10d0, 0x10f0, 0x10dd, 0x10d1, 0x10d0, 0x10e0, 0x10e2, 0x10d8, 0x10da, 0x10d8,
0x10dc, 0x10d3, 0x10db, 0x10d0, 0x10dc, 0x10d8, 0x10da, 0x10dd, 0x10e0, 0x10d3, 0x20, 0x10f0,
0x10dd, 0x10e3, 0x10d8, 0x10db, 0x10d4, 0x10da, 0x10d1, 0x10e3, 0x10e0, 0x10dc, 0x10d8, 0x10de,
0x10d4, 0x10e0, 0x10d7, 0x10d8, 0x10e1, 0x10d8, 0x10d3, 0x10dc, 0x10d4, 0x10d8, 0x10d0, 0x10db,
0x10e1, 0x10e2, 0x10d4, 0x10e0, 0x10d3, 0x10d0, 0x10db, 0x10d8, 0x10d0, 0x10dc, 0x10d3, 0x10dd,
0x10e0, 0x10d0, 0x10d0, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10ee, 0x10d0, 0x10dc, 0x10d8, 0x10d0,
0x10d7, 0x10d4, 0x10dc, 0x10d8, 0x10d1, 0x10d4, 0x10da, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8,
0x10d1, 0x10d4, 0x10e0, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10d1, 0x10e0, 0x10d0, 0x10e2, 0x10d8,
0x10e1, 0x10da, 0x10d0, 0x10d5, 0x10d0, 0x10d1, 0x10e0, 0x10d8, 0x10e3, 0x10e1, 0x10d4, 0x10da,
0x10d8, 0x10d1, 0x10e3, 0x10e5, 0x10d0, 0x10e0, 0x10d4, 0x10e1, 0x10e2, 0x10d8, 0x10d1, 0x10e3,
0x10d3, 0x10d0, 0x10de, 0x10d4, 0x10e8, 0x10e2, 0x10d8, 0x10d1, 0x10e3, 0x10e1, 0x10d8, 0x10dc,
0x10d2, 0x10d4, 0x10dc, 0x10d8, 0x10d9, 0x10d8, 0x10e8, 0x10d8, 0x10dc, 0x10d8, 0x10dd, 0x10d5,
0x10d8, 0x10d9, 0x10dd, 0x10de, 0x10d4, 0x10dc, 0x10f0, 0x10d0, 0x10d2, 0x10d4, 0x10dc, 0x10d8,
0x10d3, 0x10e3, 0x10d1, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10d2, 0x10d8, 0x10d1, 0x10e0, 0x10d0,
0x10da, 0x10e2, 0x10d0, 0x10e0, 0x10d8, 0x10d2, 0x10d4, 0x10e0, 0x10dc, 0x10e1, 0x10d8, 0x10f0,
0x10d4, 0x10da, 0x10e1, 0x10d8, 0x10dc, 0x10d9, 0x10d8, 0x10db, 0x10d4, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x10e2, 0x10d0, 0x10db,
0x10d1, 0x10dd, 0x10da, 0x10d8, 0x10ef, 0x10d4, 0x10e0, 0x10e1, 0x10d8, 0x10d9, 0x10d0, 0x10da,
0x10d8, 0x10dc, 0x10d8, 0x10dc, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10d9, 0x10d8, 0x10e0,
0x10dd, 0x10d5, 0x10d8, 0x10d9, 0x10d8, 0x10d4, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10e1, 0x10d0,
0x10d1, 0x10dd, 0x10dc, 0x10d8, 0x10da, 0x10d8, 0x10e3, 0x10d1, 0x10da, 0x10d8, 0x10d0, 0x10dc,
0x10d0, 0x10da, 0x10dd, 0x10dc, 0x10d3, 0x10dd, 0x10dc, 0x10d8, 0x10da, 0x10e3, 0x10e5, 0x10e1,
0x10d4, 0x10db, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10db, 0x10d0, 0x10d3, 0x10e0, 0x10d8,
0x10d3, 0x10d8, 0x10db, 0x10d0, 0x10da, 0x10e2, 0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10f0,
0x10d0, 0x10db, 0x10dc, 0x10d8, 0x10db, 0x10d8, 0x10dc, 0x10e1, 0x10d9, 0x10d8, 0x10db, 0x10dd,
0x10dc, 0x10d0, 0x10d9, 0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd, 0x10d5, 0x10d8, 0x10dd,
0x10e1, 0x10da, 0x10dd, 0x10de, 0x10d0, 0x10e0, 0x10d8, 0x10d6, 0x10d8, 0x10de, 0x10dd, 0x10d3,
0x10d2, 0x10dd, 0x10e0, 0x10d8, 0x10ea, 0x10d0, 0x10de, 0x10e0, 0x10d0, 0x10e6, 0x10d0, 0x10e0,
0x10d8, 0x10d2, 0x10d0, 0x10e0, 0x10dd, 0x10db, 0x10d8, 0x10e1, 0x10d0, 0x10db, 0x10d0, 0x10e0,
0x10d0, 0x10e1, 0x10d0, 0x10dc, 0x2d, 0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10dc, 0x10dd, 0x10e1,
0x10d0, 0x10e0, 0x10d0, 0x10d4, 0x10d5, 0x10dd, 0x10e1, 0x10d0, 0x10e0, 0x10d0, 0x10e2, 0x10dd,
0x10d5, 0x10d8, 0x10e1, 0x10d8, 0x10db, 0x10e4, 0x10d4, 0x10e0, 0x10dd, 0x10de, 0x10dd, 0x10da,
0x10d8, 0x10e1, 0x10d9, 0x10dd, 0x10de, 0x10d8, 0x10d4, 0x10e1, 0x10dd, 0x10e4, 0x10d8, 0x10d0,
0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10f0, 0x10dd, 0x10da, 0x10db, 0x10d8, 0x10e2, 0x10d0, 0x10da,
0x10d8, 0x10dc, 0x10d8, 0x10e2, 0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d0, 0x10e3, 0x10da, 0x10d8,
0x10d0, 0x10dc, 0x10dd, 0x10d5, 0x10e1, 0x10d9, 0x10d8, 0x10d5, 0x10d0, 0x10d3, 0x10e3, 0x10ea,
0x10d8, 0x10d5, 0x10d0, 0x10e2, 0x10d8, 0x10d9, 0x10d0, 0x10dc, 0x10d8, 0x10d5, 0x10d4, 0x10dc,
0x10d0, 0x10d5, 0x10d8, 0x10da, 0x10dc, 0x10d8, 0x10e3, 0x10e1, 0x10d8, 0x10d5, 0x10dd, 0x10da,
0x10d2, 0x10dd, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10d5, 0x10d0, 0x10e0, 0x10e8, 0x10d0,
0x10d5, 0x10d0, 0x10d6, 0x10d0, 0x10d2, 0x10e0, 0x10d4, 0x10d1, 0x10d8, 0x10ea, 0x10d8, 0x10e3,
0x10e0, 0x10d8, 0x10ee, 0x10d8, 0x10d0, 0x10dc, 0x10e2, 0x10d0, 0x10dc, 0x10d0, 0x10dc, 0x10d0,
0x10e0, 0x10d8, 0x10d5, 0x10e3, 0x10e9, 0x10d0, 0x10d2, 0x10dd, 0x10e1, 0x10d8, 0x10e8, 0x10dd,
0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e5,
0x10dd, 0x10e5, 0x10dd, 0x10e1, 0x10d8, 0x10d9, 0x10dd, 0x10db, 0x10dd, 0x10e0, 0x10dd, 0x10d9,
0x10d4, 0x10e0, 0x10d2, 0x10d4, 0x10da, 0x10d4, 0x10dc, 0x10d8, 0x10db, 0x10d0, 0x10f0, 0x10d4,
0x10db, 0x10d0, 0x10da, 0x10d3, 0x10d8, 0x10d5, 0x10d4, 0x10d1, 0x10d8, 0x10db, 0x10d0, 0x10d5,
0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10db, 0x10d0, 0x10d8, 0x10dd, 0x10e2, 0x10d8, 0x10e0, 0x10d4,
0x10d8, 0x10e3, 0x10dc, 0x10d8, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10de, 0x10d8, 0x10d0, 0x10dd,
0x10d9, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10d1, 0x10e3, 0x10d2, 0x10d4, 0x10dc, 0x10d5,
0x10d8, 0x10da, 0x10d8, 0x10e9, 0x10d0, 0x10d7, 0x10d0, 0x10db, 0x10d8, 0x10e9, 0x10e3, 0x10e3,
0x10d9, 0x10d8, 0x10d8, 0x10e1, 0x10d7, 0x10d4, 0x10e0, 0x10d8, 0x10d4, 0x10e4, 0x10d0, 0x10e2,
0x10d4, 0x10e4, 0x10d0, 0x10d9, 0x10d0, 0x10dd, 0x10e4, 0x10dd, 0x10e4, 0x10d8, 0x10ef, 0x10d8,
0x10e4, 0x10e3, 0x10dc, 0x10d0, 0x10e4, 0x10e3, 0x10e2, 0x10d8, 0x10d2, 0x10d0, 0x10da, 0x10d0,
0x10de, 0x10d0, 0x10d2, 0x10dd, 0x10e1, 0x10d8, 0x10d2, 0x10d0, 0x10db, 0x10d1, 0x10d8, 0x10d4,
0x10e0, 0x10d8, 0x10d2, 0x10d5, 0x10d0, 0x10d3, 0x10d0, 0x10da, 0x10d9, 0x10d0, 0x10dc, 0x10d0,
0x10da, 0x10d8, 0x10d2, 0x10e3, 0x10d0, 0x10db, 0x10d8, 0x10f0, 0x10dd, 0x10dc, 0x10dd, 0x10da,
0x10e3, 0x10da, 0x10e3, 0x10d4, 0x10dc, 0x10d3, 0x10d4, 0x10e0, 0x10d1, 0x10e3, 0x10e0, 0x10d8,
0x10d9, 0x10d8, 0x10e0, 0x10d8, 0x10e2, 0x10d8, 0x10db, 0x10d0, 0x10e2, 0x10d8, 0x10d9, 0x10dd,
0x10e1, 0x10e0, 0x10d0, 0x10d4, 0x10d9, 0x10d5, 0x10d0, 0x10ef, 0x10d0, 0x10da, 0x10d4, 0x10d8,
0x10dc, 0x10d8, 0x10db, 0x10d0, 0x10ee, 0x10e3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10e0, 0x10d9,
0x10d4, 0x10d6, 0x10d0, 0x10e1, 0x10d8, 0x10db, 0x10d8, 0x10d3, 0x10e3, 0x10d4, 0x10d8, 0x10dc,
0x10d0, 0x10e3, 0x10e0, 0x10e3, 0x10dc, 0x10d8, 0x10e3, 0x10d4, 0x10dc, 0x10dd, 0x10e0, 0x10e4,
0x10dd, 0x10da, 0x10d9, 0x10d8, 0x10dc, 0x10e3, 0x10db, 0x10d4, 0x10d0, 0x10de, 0x10d0, 0x10d2,
0x10dd, 0x2d, 0x10de, 0x10d0, 0x10d2, 0x10dd, 0x10de, 0x10d0, 0x10da, 0x10d0, 0x10e3, 0x10de,
0x10d8, 0x10e2, 0x10d9, 0x10d4, 0x10e0, 0x10dc, 0x10d8, 0x10de, 0x10dd, 0x10dc, 0x10de, 0x10d4,
0x10d8, 0x10de, 0x10dd, 0x10e0, 0x10e2, 0x20, 0x10db, 0x10dd, 0x10e0, 0x10e1, 0x10d1, 0x10d8,
0x10e0, 0x10d0, 0x10e0, 0x10dd, 0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d0, 0x10e1, 0x10d0, 0x10d8,
0x10de, 0x10d0, 0x10dc, 0x10d8, 0x10e2, 0x10d0, 0x10d8, 0x10e2, 0x10d8, 0x10e2, 0x10d0, 0x10e0,
0x10d0, 0x10e3, 0x10d0, 0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d0, 0x10e2, 0x10d0, 0x10de, 0x10e3,
0x10e3, 0x10d4, 0x10d8, 0x10d9, 0x10d8, 0x10d5, 0x10d0, 0x10da, 0x10d8, 0x10e1, 0x10d8, 0x44,
0x61, 0x72, 0x65, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x44, 0x73, 0x63,
0x68, 0x69, 0x62, 0x75, 0x74, 0x69, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69,
0x73, 0x63, 0x68, 0x75, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x69, 0x6e,
0x73, 0x65, 0x6c, 0x6e, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x2d, 0x53,
0x74, 0x61, 0x64, 0x74, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x54, 0x73, 0x63,
0x68, 0x69, 0x74, 0x61, 0x48, 0x6f, 0x2d, 0x43, 0x68, 0x69, 0x2d, 0x4d,
0x69, 0x6e, 0x68, 0x2d, 0x53, 0x74, 0x61, 0x64, 0x74, 0x43, 0x68, 0x6f,
0x77, 0x64, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b,
0x61, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x73, 0x63, 0x68, 0x69, 0x4e, 0x6f,
0x77, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x65, 0x74, 0x73, 0x6b, 0x4e, 0x6f,
0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x50, 0x6a, 0xf6,
0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x54, 0x61, 0x69, 0x70, 0x65, 0x68,
0x54, 0x69, 0x66, 0x6c, 0x69, 0x73, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x65,
0x6e, 0x46, 0xe4, 0x72, 0xf6, 0x65, 0x72, 0x52, 0x65, 0x79, 0x6b, 0xad,
0x6a, 0x61, 0xad, 0x76, 0xed, 0x6b, 0x53, 0xfc, 0x64, 0x67, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x65, 0x6e, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0x68,
0x61, 0x6e, 0x4b, 0x69, 0x72, 0x6f, 0x77, 0x4b, 0x69, 0x65, 0x77, 0x53,
0x61, 0x72, 0x61, 0x74, 0x6f, 0x77, 0x55, 0x6c, 0x6a, 0x61, 0x6e, 0x6f,
0x77, 0x73, 0x6b, 0x57, 0x65, 0x69, 0x68, 0x6e, 0x61, 0x63, 0x68, 0x74,
0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x65,
0x6e, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x4f, 0x73,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x46, 0x69, 0x64, 0x73,
0x63, 0x68, 0x69, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76, 0x391, 0x3bc,
0x3c0, 0x3b9, 0x3c4, 0x3b6, 0x3ac, 0x3bd, 0x386, 0x3ba, 0x3c1, 0x3b1, 0x391, 0x3bd,
0x3c4, 0x3af, 0x3c2, 0x20, 0x391, 0x3bc, 0x3c0, 0x3ad, 0x3bc, 0x3c0, 0x3b1, 0x391,
0x3bb, 0x3b3, 0x3ad, 0x3c1, 0x3b9, 0x391, 0x3c3, 0x3bc, 0x3ac, 0x3c1, 0x3b1, 0x39c,
0x3c0, 0x3b1, 0x3bc, 0x3ac, 0x3ba, 0x3bf, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba,
0x3af, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b6, 0x3bf, 0x3cd, 0x3bb, 0x39c, 0x3c0, 0x3b9,
0x3c3, 0x3ac, 0x3bf, 0x3c5, 0x39c, 0x3c0, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3ac, 0x3b9,
0x3c1, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3b6, 0x3b1, 0x3b2, 0x3af, 0x3bb, 0x39c, 0x3c0,
0x3bf, 0x3c5, 0x3b6, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3bf, 0x3cd, 0x3c1, 0x3b1, 0x39a,
0x3ac, 0x3b9, 0x3c1, 0x3bf, 0x39a, 0x3b1, 0x3b6, 0x3b1, 0x3bc, 0x3c0, 0x3bb, 0x3ac,
0x3bd, 0x3ba, 0x3b1, 0x398, 0x3ad, 0x3bf, 0x3c5, 0x3c4, 0x3b1, 0x39a, 0x3cc, 0x3bd,
0x3b1, 0x3ba, 0x3c1, 0x3b9, 0x39d, 0x3c4, 0x3b1, 0x3ba, 0x3ac, 0x3c1, 0x39d, 0x3c4,
0x3b1, 0x3c1, 0x20, 0x3b5, 0x3c2, 0x20, 0x3a3, 0x3b1, 0x3bb, 0x3ac, 0x3bc, 0x3a4,
0x3b6, 0x3b9, 0x3bc, 0x3c0, 0x3bf, 0x3c5, 0x3c4, 0x3af, 0x39d, 0x3c4, 0x3bf, 0x3c5,
0x3ac, 0x3bb, 0x3b1, 0x395, 0x3bb, 0x20, 0x391, 0x3b3, 0x3b9, 0x3bf, 0x3cd, 0x3bd,
0x3a6, 0x3c1, 0x3af, 0x3c4, 0x3b1, 0x3bf, 0x3c5, 0x3bd, 0x393, 0x3ba, 0x3b1, 0x3bc,
0x3c0, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b5, 0x3a7, 0x3b1, 0x3c1, 0x3ac, 0x3c1, 0x3b5,
0x393, 0x3b9, 0x3bf, 0x3c7, 0x3ac, 0x3bd, 0x3b5, 0x3c3, 0x3bc, 0x3c0, 0x3bf, 0x3c5,
0x3c1, 0x3b3, 0x3ba, 0x3a4, 0x3b6, 0x3bf, 0x3cd, 0x3bc, 0x3c0, 0x3b1, 0x39a, 0x3b1,
0x3bc, 0x3c0, 0x3ac, 0x3bb, 0x3b1, 0x3a7, 0x3b1, 0x3c1, 0x3c4, 0x3bf, 0x3cd, 0x3bc,
0x39a, 0x3b9, 0x3b3, 0x3ba, 0x3ac, 0x3bb, 0x3b9, 0x39a, 0x3b9, 0x3bd, 0x3c3, 0x3ac,
0x3c3, 0x3b1, 0x39b, 0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x39b, 0x3b9, 0x3bc, 0x3c0,
0x3c1, 0x3b5, 0x3b2, 0x3af, 0x3bb, 0x39b, 0x3bf, 0x3bc, 0x3ad, 0x39b, 0x3bf, 0x3c5,
0x3ac, 0x3bd, 0x3c4, 0x3b1, 0x39b, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3bf, 0x3c5, 0x3bc,
0x3c0, 0x3ac, 0x3c3, 0x3b9, 0x39b, 0x3bf, 0x3c5, 0x3b6, 0x3ac, 0x3ba, 0x3b1, 0x39c,
0x3b1, 0x3bb, 0x3ac, 0x3bc, 0x3c0, 0x3bf, 0x39c, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3c4,
0x3bf, 0x39c, 0x3b1, 0x3c3, 0x3ad, 0x3c1, 0x3bf, 0x3c5, 0x39c, 0x3c0, 0x3b1, 0x3bc,
0x3c0, 0x3ac, 0x3bd, 0x3b5, 0x39c, 0x3bf, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3af,
0x3c3, 0x3bf, 0x3c5, 0x39c, 0x3bf, 0x3bd, 0x3c1, 0x3cc, 0x3b2, 0x3b9, 0x3b1, 0x39d,
0x3b1, 0x3ca, 0x3c1, 0x3cc, 0x3bc, 0x3c0, 0x3b9, 0x39d, 0x3c4, 0x3b6, 0x3b1, 0x3bc,
0x3ad, 0x3bd, 0x3b1, 0x39d, 0x3b9, 0x3b1, 0x3bc, 0x3ad, 0x3b9, 0x39d, 0x3bf, 0x3c5,
0x3b1, 0x3ba, 0x3c3, 0x3cc, 0x3c4, 0x39f, 0x3c5, 0x3b1, 0x3b3, 0x3ba, 0x3b1, 0x3bd,
0x3c4, 0x3bf, 0x3cd, 0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3a0, 0x3cc, 0x3c1, 0x3c4, 0x3bf,
0x2d, 0x39d, 0x3cc, 0x3b2, 0x3bf, 0x3a3, 0x3ac, 0x3bf, 0x20, 0x3a4, 0x3bf, 0x3bc,
0x3ad, 0x3a4, 0x3c1, 0x3af, 0x3c0, 0x3bf, 0x3bb, 0x3b7, 0x3a4, 0x3cd, 0x3bd, 0x3b9,
0x3b4, 0x3b1, 0x392, 0x3af, 0x3bd, 0x3c4, 0x3c7, 0x3bf, 0x3c5, 0x3ba, 0x386, 0x3bd,
0x3c4, 0x3b1, 0x3ba, 0x386, 0x3bd, 0x3ba, 0x3bf, 0x3c1, 0x3b1, 0x3c4, 0x3b6, 0x391,
0x3bd, 0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3af, 0x3bb, 0x3b1, 0x391, 0x3bd, 0x3c4, 0x3af,
0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3b1, 0x391, 0x3c1, 0x3b1, 0x3b3, 0x3ba, 0x3bf, 0x3c5,
0x3ac, 0x3b9, 0x3bd, 0x3b1, 0x39c, 0x3c0, 0x3bf, 0x3c5, 0x3ad, 0x3bd, 0x3bf, 0x3c2,
0x20, 0x386, 0x3b9, 0x3c1, 0x3b5, 0x3c2, 0x39a, 0x3b1, 0x3c4, 0x3b1, 0x3bc, 0x3ac,
0x3c1, 0x3ba, 0x3b1, 0x39a, 0x3cc, 0x3c1, 0x3b4, 0x3bf, 0x3b2, 0x3b1, 0x3a7, 0x3bf,
0x3c5, 0x3c7, 0x3bf, 0x3cd, 0x3b9, 0x39b, 0x3b1, 0x20, 0x3a1, 0x3b9, 0x3cc, 0x3c7,
0x3b1, 0x39c, 0x3b5, 0x3bd, 0x3c4, 0x3cc, 0x3b6, 0x3b1, 0x3a1, 0x3af, 0x3bf, 0x20,
0x393, 0x3ba, 0x3b1, 0x3b3, 0x3b9, 0x3ad, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x3a3, 0x3ac,
0x3bb, 0x3c4, 0x3b1, 0x3a3, 0x3b1, 0x3bd, 0x20, 0x3a7, 0x3bf, 0x3c5, 0x3ac, 0x3bd,
0x3a3, 0x3b1, 0x3bd, 0x20, 0x39b, 0x3bf, 0x3cd, 0x3b9, 0x3c2, 0x3a4, 0x3bf, 0x3c5,
0x3ba, 0x3bf, 0x3c5, 0x3bc, 0x3ac, 0x3bd, 0x39f, 0x3c5, 0x3c3, 0x3bf, 0x3c5, 0x3ac,
0x3b9, 0x3b1, 0x391, 0x3c1, 0x3bf, 0x3cd, 0x3bc, 0x3c0, 0x3b1, 0x391, 0x3c3, 0x3bf,
0x3c5, 0x3bd, 0x3c3, 0x3b9, 0x3cc, 0x3bd, 0x391, 0x3c4, 0x3b9, 0x3ba, 0x3cc, 0x3ba,
0x3b1, 0x3bd, 0x39c, 0x3c0, 0x3b1, 0x390, 0x3b1, 0x39c, 0x3c0, 0x3b1, 0x390, 0x3b1,
0x20, 0x3bd, 0x3c4, 0x3b5, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3c4, 0x3ad, 0x3c1,
0x3b1, 0x3c2, 0x39c, 0x3c0, 0x3b1, 0x3c1, 0x3bc, 0x3c0, 0x3ad, 0x3b9, 0x3bd, 0x3c4,
0x3bf, 0x3c2, 0x39c, 0x3c0, 0x3b5, 0x3bb, 0x3ad, 0x3bc, 0x39c, 0x3c0, 0x3b5, 0x3bb,
0x3af, 0x3b6, 0x39c, 0x3c0, 0x3bb, 0x3b1, 0x3bd, 0x20, 0x3a3, 0x3b1, 0x3bc, 0x3c0,
0x3bb, 0x3cc, 0x3bd, 0x39c, 0x3c0, 0x3cc, 0x3b1, 0x20, 0x392, 0x3af, 0x3c3, 0x3c4,
0x3b1, 0x39c, 0x3c0, 0x3bf, 0x3b3, 0x3ba, 0x3bf, 0x3c4, 0x3ac, 0x39c, 0x3c0, 0x3cc,
0x3b9, 0x3b6, 0x3b9, 0x39a, 0x3ad, 0x3bc, 0x3c0, 0x3c1, 0x3b9, 0x3c4, 0x3b6, 0x20,
0x39c, 0x3c0, 0x3ad, 0x3b9, 0x39a, 0x3ac, 0x3bc, 0x3c0, 0x3bf, 0x20, 0x393, 0x3ba,
0x3c1, 0x3ac, 0x3bd, 0x3c4, 0x3b5, 0x39a, 0x3b1, 0x3bd, 0x3ba, 0x3bf, 0x3cd, 0x3bd,
0x39a, 0x3b1, 0x3c1, 0x3ac, 0x3ba, 0x3b1, 0x3c2, 0x39a, 0x3b1, 0x3b3, 0x3b9, 0x3ad,
0x3bd, 0x39a, 0x3ad, 0x3b9, 0x3bc, 0x3b1, 0x3bd, 0x3a3, 0x3b9, 0x3ba, 0x3ac, 0x3b3,
0x3bf, 0x3a4, 0x3c3, 0x3b9, 0x3bf, 0x3c5, 0x3ac, 0x3bf, 0x3c5, 0x3b1, 0x3a3, 0x3b9,
0x3bf, 0x3c5, 0x3b4, 0x3ac, 0x3b4, 0x20, 0x3a7, 0x3bf, 0x3c5, 0x3ac, 0x3c1, 0x3b5,
0x3c2, 0x39a, 0x3cc, 0x3c3, 0x3c4, 0x3b1, 0x20, 0x3a1, 0x3af, 0x3ba, 0x3b1, 0x39a,
0x3c1, 0x3ad, 0x3c3, 0x3c4, 0x3bf, 0x3bd, 0x39a, 0x3bf, 0x3c5, 0x3b9, 0x3b1, 0x3bc,
0x3c0, 0x3ac, 0x39a, 0x3bf, 0x3c5, 0x3c1, 0x3b1, 0x3c3, 0x3ac, 0x3bf, 0x39d, 0x3c4,
0x3b1, 0x3bd, 0x3bc, 0x3b1, 0x3c1, 0x3ba, 0x3c3, 0x3ac, 0x3b2, 0x3bd, 0x39d, 0x3c4,
0x3cc, 0x3c3, 0x3bf, 0x3bd, 0x39d, 0x3c4, 0x3cc, 0x3c3, 0x3bf, 0x3bd, 0x20, 0x39a,
0x3c1, 0x3b9, 0x3ba, 0x39d, 0x3c4, 0x3ad, 0x3bd, 0x3b2, 0x3b5, 0x3c1, 0x39d, 0x3c4,
0x3b9, 0x3c4, 0x3c1, 0x3cc, 0x3b9, 0x3c4, 0x39d, 0x3c4, 0x3bf, 0x3bc, 0x3af, 0x3bd,
0x3b9, 0x3ba, 0x3b1, 0x388, 0x3bd, 0x3c4, 0x3bc, 0x3bf, 0x3bd, 0x3c4, 0x3bf, 0x3bd,
0x395, 0x3ca, 0x3c1, 0x3bf, 0x3c5, 0x3bd, 0x3b5, 0x3c0, 0x3ad, 0x395, 0x3bb, 0x20,
0x3a3, 0x3b1, 0x3bb, 0x3b2, 0x3b1, 0x3b4, 0x3cc, 0x3c1, 0x3a6, 0x3bf, 0x3c1, 0x3c4,
0x20, 0x39d, 0x3ad, 0x3bb, 0x3c3, 0x3bf, 0x3bd, 0x3a6, 0x3bf, 0x3c1, 0x3c4, 0x3b1,
0x3bb, 0x3ad, 0x3b6, 0x3b1, 0x393, 0x3ba, 0x3bb, 0x3ad, 0x3b9, 0x3c2, 0x20, 0x39c,
0x3c0, 0x3ad, 0x3b9, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3c2, 0x20, 0x39c, 0x3c0, 0x3ad,
0x3b9, 0x393, 0x3ba, 0x3c1, 0x3b1, 0x3bd, 0x3c4, 0x20, 0x3a4, 0x3bf, 0x3c5, 0x3c1,
0x3ba, 0x393, 0x3c1, 0x3b5, 0x3bd, 0x3ac, 0x3b4, 0x3b1, 0x393, 0x3bf, 0x3c5, 0x3b1,
0x3b4, 0x3b5, 0x3bb, 0x3bf, 0x3cd, 0x3c0, 0x3b7, 0x393, 0x3bf, 0x3c5, 0x3b1, 0x3c4,
0x3b5, 0x3bc, 0x3ac, 0x3bb, 0x3b1, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3b1, 0x3b3, 0x3b9,
0x3b1, 0x3ba, 0x3af, 0x3bb, 0x393, 0x3bf, 0x3c5, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x3a7,
0x3ac, 0x3bb, 0x3b9, 0x3c6, 0x3b1, 0x3be, 0x391, 0x3b2, 0x3ac, 0x3bd, 0x3b1, 0x395,
0x3c1, 0x3bc, 0x3bf, 0x3c3, 0x3af, 0x3b3, 0x3b9, 0x3bf, 0x399, 0x3bd, 0x3c4, 0x3b9,
0x3b1, 0x3bd, 0x3ac, 0x3c0, 0x3bf, 0x3bb, 0x3b9, 0x3c2, 0x39d, 0x3bf, 0x3be, 0x2c,
0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x39c, 0x3b1, 0x3c1, 0x3ad,
0x3bd, 0x3b3, 0x3ba, 0x3bf, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd,
0x3b1, 0x3a0, 0x3af, 0x3c4, 0x3b5, 0x3c1, 0x3c3, 0x3bc, 0x3c0, 0x3b5, 0x3c1, 0x3b3,
0x3ba, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x3a4, 0x3b5,
0x3bb, 0x20, 0x3a3, 0x3af, 0x3c4, 0x3b9, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9,
0x3ac, 0x3bd, 0x3b1, 0x392, 0x3b9, 0x3b2, 0x3ad, 0x3b9, 0x2c, 0x20, 0x399, 0x3bd,
0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x392, 0x3b1, 0x3bd, 0x3c3, 0x3ad, 0x3bd, 0x2c,
0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x393, 0x3bf, 0x3c5, 0x3af,
0x3bd, 0x3b1, 0x3bc, 0x3b1, 0x3ba, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac,
0x3bd, 0x3b1, 0x399, 0x3bd, 0x3bf, 0x3cd, 0x3b2, 0x3b9, 0x3ba, 0x399, 0x3ba, 0x3b1,
0x3bb, 0x3bf, 0x3cd, 0x3b9, 0x3c4, 0x3a4, 0x3b6, 0x3b1, 0x3bc, 0x3ac, 0x3b9, 0x3ba,
0x3b1, 0x3a4, 0x3b6, 0x3bf, 0x3cd, 0x3bd, 0x3bf, 0x39b, 0x3bf, 0x3cd, 0x3b9, 0x3b2,
0x3b9, 0x3bb, 0x39c, 0x3bf, 0x3bd, 0x3c4, 0x3b9, 0x3c4, 0x3c3, 0x3ad, 0x3bb, 0x3bf,
0x2c, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3ac, 0x3ba, 0x3b9, 0x39a, 0x3c1, 0x3ac,
0x3bb, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3ba, 0x39b, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3b6,
0x39b, 0x3af, 0x3bc, 0x3b1, 0x39b, 0x3bf, 0x3c2, 0x20, 0x386, 0x3bd, 0x3c4, 0x3b6,
0x3b5, 0x3bb, 0x3b5, 0x3c2, 0x39c, 0x3b1, 0x3c3, 0x3b5, 0x3ca, 0x3cc, 0x39c, 0x3b1,
0x3bd, 0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3b1, 0x39c, 0x3b1, 0x3bd, 0x3ac, 0x3bf,
0x3c5, 0x3c2, 0x39c, 0x3b1, 0x3c1, 0x3b9, 0x3b3, 0x3ba, 0x3cc, 0x39c, 0x3b1, 0x3c1,
0x3c4, 0x3b9, 0x3bd, 0x3af, 0x3ba, 0x3b1, 0x39c, 0x3b1, 0x3c4, 0x3b1, 0x3bc, 0x3cc,
0x3c1, 0x3bf, 0x3c2, 0x39c, 0x3b1, 0x3b6, 0x3b1, 0x3c4, 0x3bb, 0x3ac, 0x3bd, 0x39c,
0x3b5, 0x3bd, 0x3bf, 0x3bc, 0x3af, 0x3bd, 0x3b5, 0x39c, 0x3ad, 0x3c1, 0x3b9, 0x3b4,
0x3b1, 0x39c, 0x3b5, 0x3c4, 0x3bb, 0x3b1, 0x3ba, 0x3ac, 0x3c4, 0x3bb, 0x3b1, 0x3a0,
0x3cc, 0x3bb, 0x3b7, 0x20, 0x3c4, 0x3bf, 0x3c5, 0x20, 0x39c, 0x3b5, 0x3be, 0x3b9,
0x3ba, 0x3bf, 0x3cd, 0x39c, 0x3b9, 0x3ba, 0x3b5, 0x3bb, 0x3cc, 0x3bd, 0x39c, 0x3cc,
0x3bd, 0x3ba, 0x3c4, 0x3bf, 0x3bd, 0x39c, 0x3bf, 0x3bd, 0x3c4, 0x3b5, 0x3c1, 0x3ad,
0x3b9, 0x39c, 0x3bf, 0x3bd, 0x3c4, 0x3b5, 0x3b2, 0x3b9, 0x3b4, 0x3ad, 0x3bf, 0x39c,
0x3bf, 0x3bd, 0x3c3, 0x3b5, 0x3c1, 0x3ac, 0x3c4, 0x39d, 0x3b1, 0x3c3, 0x3ac, 0x3bf,
0x3c5, 0x39d, 0x3ad, 0x3b1, 0x20, 0x3a5, 0x3cc, 0x3c1, 0x3ba, 0x3b7, 0x39d, 0x3cc,
0x3bc, 0x3b5, 0x39d, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b9, 0x3b1, 0x39c, 0x3c0, 0x3ad,
0x3bf, 0x3c5, 0x3bb, 0x3b1, 0x2c, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1,
0x20, 0x39d, 0x3c4, 0x3b1, 0x3ba, 0x3cc, 0x3c4, 0x3b1, 0x3a3, 0x3ad, 0x3bd, 0x3c4,
0x3b5, 0x3c1, 0x2c, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x20, 0x39d,
0x3c4, 0x3b1, 0x3ba, 0x3cc, 0x3c4, 0x3b1, 0x39d, 0x3b9, 0x3bf, 0x3c5, 0x20, 0x3a3,
0x3ad, 0x3b9, 0x3bb, 0x3b5, 0x3bc, 0x2c, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9,
0x3b1, 0x20, 0x39d, 0x3c4, 0x3b1, 0x3ba, 0x3cc, 0x3c4, 0x3b1, 0x39d, 0x3bf, 0x3c5,
0x3bf, 0x3cd, 0x3ba, 0x39f, 0x3c7, 0x3b9, 0x3bd, 0x3ac, 0x3b3, 0x3ba, 0x3b1, 0x3a0,
0x3b1, 0x3bd, 0x3b1, 0x3bc, 0x3ac, 0x3c2, 0x3a0, 0x3b1, 0x3c1, 0x3b1, 0x3bc, 0x3b1,
0x3c1, 0x3af, 0x3bc, 0x3c0, 0x3bf, 0x3a6, 0x3bf, 0x3af, 0x3bd, 0x3b9, 0x3be, 0x3a0,
0x3bf, 0x3c1, 0x3c4, 0x2d, 0x3bf, 0x2d, 0x3a0, 0x3c1, 0x3b5, 0x3bd, 0x3c2, 0x3a0,
0x3bf, 0x3c1, 0x3c4, 0x20, 0x3bf, 0x3c6, 0x20, 0x3a3, 0x3c0, 0x3ad, 0x3b9, 0x3bd,
0x3a0, 0x3cc, 0x3c1, 0x3c4, 0x3bf, 0x20, 0x392, 0x3ad, 0x3bb, 0x3b9, 0x3bf, 0x3a0,
0x3bf, 0x3c5, 0x3ad, 0x3c1, 0x3c4, 0x3bf, 0x20, 0x3a1, 0x3af, 0x3ba, 0x3bf, 0x3a0,
0x3bf, 0x3cd, 0x3bd, 0x3c4, 0x3b1, 0x20, 0x391, 0x3c1, 0x3ad, 0x3bd, 0x3b1, 0x3c2,
0x3a1, 0x3ac, 0x3bd, 0x3ba, 0x3b9, 0x3bd, 0x20, 0x38a, 0x3bd, 0x3bb, 0x3b5, 0x3c4,
0x3a1, 0x3b5, 0x3c3, 0x3af, 0x3c6, 0x3b5, 0x3a1, 0x3b5, 0x3c4, 0x3b6, 0x3ac, 0x3b9,
0x3bd, 0x3b1, 0x3a1, 0x3ad, 0x3b6, 0x3bf, 0x3bb, 0x3bf, 0x3c5, 0x3c4, 0x3a1, 0x3af,
0x3bf, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3ac, 0x3bd, 0x3ba, 0x3bf, 0x3a3, 0x3b1, 0x3bd,
0x3c4, 0x3b1, 0x3c1, 0x3ad, 0x3bc, 0x3a3, 0x3b1, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3b3,
0x3ba, 0x3bf, 0x386, 0x3b3, 0x3b9, 0x3bf, 0x3c2, 0x20, 0x394, 0x3bf, 0x3bc, 0x3af,
0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3c2, 0x3a3, 0x3ac, 0x3bf, 0x20, 0x3a0, 0x3ac, 0x3bf,
0x3bb, 0x3bf, 0x3a3, 0x3ba, 0x3bf, 0x3c1, 0x3b5, 0x3c3, 0x3bc, 0x3c0, 0x3af, 0x3c3,
0x3bf, 0x3c5, 0x3bd, 0x3c4, 0x3a3, 0x3af, 0x3c4, 0x3ba, 0x3b1, 0x386, 0x3b3, 0x3b9,
0x3bf, 0x3c2, 0x20, 0x392, 0x3b1, 0x3c1, 0x3b8, 0x3bf, 0x3bb, 0x3bf, 0x3bc, 0x3b1,
0x3af, 0x3bf, 0x3c2, 0x3a3, 0x3b5, 0x3bd, 0x20, 0x3a4, 0x3b6, 0x3bf, 0x3bd, 0x3c2,
0x3a3, 0x3b5, 0x3bd, 0x20, 0x39a, 0x3b9, 0x3c4, 0x3c2, 0x391, 0x3b3, 0x3af, 0x3b1,
0x20, 0x39b, 0x3bf, 0x3c5, 0x3ba, 0x3af, 0x3b1, 0x386, 0x3b3, 0x3b9, 0x3bf, 0x3c2,
0x20, 0x398, 0x3c9, 0x3bc, 0x3ac, 0x3c2, 0x386, 0x3b3, 0x3b9, 0x3bf, 0x3c2, 0x20,
0x392, 0x3b9, 0x3ba, 0x3ad, 0x3bd, 0x3c4, 0x3b9, 0x3bf, 0x3c2, 0x3a3, 0x3bf, 0x3c5,
0x3af, 0x3c6, 0x3c4, 0x20, 0x39a, 0x3ac, 0x3c1, 0x3b5, 0x3bd, 0x3c4, 0x3a4, 0x3b5,
0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3c3, 0x3b9, 0x3b3, 0x3ba, 0x3ac, 0x3bb, 0x3c0, 0x3b1,
0x398, 0x3bf, 0x3cd, 0x3bb, 0x3b7, 0x3a4, 0x3b9, 0x3c7, 0x3bf, 0x3c5, 0x3ac, 0x3bd,
0x3b1, 0x3a4, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3c4, 0x3bf, 0x3a4, 0x3bf, 0x3c1, 0x3c4,
0x3cc, 0x3bb, 0x3b1, 0x392, 0x3b1, 0x3bd, 0x3ba, 0x3bf, 0x3cd, 0x3b2, 0x3b5, 0x3c1,
0x393, 0x3bf, 0x3c5, 0x3ac, 0x3b9, 0x3c4, 0x3c7, 0x3bf, 0x3c1, 0x3c2, 0x393, 0x3bf,
0x3c5, 0x3af, 0x3bd, 0x3b9, 0x3c0, 0x3b5, 0x3b3, 0x3ba, 0x393, 0x3b9, 0x3ac, 0x3ba,
0x3bf, 0x3c5, 0x3c4, 0x3b1, 0x3c4, 0x39a, 0x3ac, 0x3c3, 0x3b5, 0x3ca, 0x39d, 0x3c4,
0x3ad, 0x3b9, 0x3b2, 0x3b9, 0x3c2, 0x39d, 0x3c4, 0x3b9, 0x3bc, 0x3cc, 0x3bd, 0x20,
0x3bd, 0x3c4, 0x2019, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3b2, 0x3af, 0x3bb, 0x39c, 0x3b1,
0x3ba, 0x3bf, 0x3c5, 0x3ac, 0x3c1, 0x3b9, 0x39c, 0x3cc, 0x3c3, 0x3bf, 0x3bd, 0x39c,
0x3b1, 0x3ba, 0x3bc, 0x3ad, 0x3c1, 0x3bd, 0x3c4, 0x3bf, 0x3a0, 0x3ac, 0x3bb, 0x3bc,
0x3b5, 0x3c1, 0x3a1, 0x3cc, 0x3b8, 0x3b5, 0x3c1, 0x3b1, 0x3a3, 0x3af, 0x3bf, 0x3b2,
0x3b1, 0x3a4, 0x3c1, 0x3bf, 0x3bb, 0x392, 0x3cc, 0x3c3, 0x3c4, 0x3bf, 0x3ba, 0x39b,
0x3cc, 0x3bd, 0x3b3, 0x3ba, 0x3b9, 0x3b5, 0x3b1, 0x3c1, 0x3bc, 0x3c0, 0x3b9, 0x3b5,
0x3bd, 0x386, 0x3bd, 0x3c4, 0x3b5, 0x3bd, 0x391, 0x3bb, 0x3bc, 0x3ac, 0x3c4, 0x3b9,
0x391, 0x3bc, 0x3bc, 0x3ac, 0x3bd, 0x391, 0x3bd, 0x3b1, 0x3bd, 0x3c4, 0x3af, 0x3c1,
0x391, 0x3ba, 0x3c4, 0x3ac, 0x3bf, 0x3c5, 0x391, 0x3ba, 0x3c4, 0x3cc, 0x3bc, 0x3c0,
0x3b5, 0x391, 0x3c3, 0x3c7, 0x3b1, 0x3bc, 0x3c0, 0x3ac, 0x3c4, 0x391, 0x3c4, 0x3b9,
0x3c1, 0x3ac, 0x3bf, 0x3c5, 0x392, 0x3b1, 0x3b3, 0x3b4, 0x3ac, 0x3c4, 0x3b7, 0x39c,
0x3c0, 0x3b1, 0x3c7, 0x3c1, 0x3ad, 0x3b9, 0x3bd, 0x39c, 0x3c0, 0x3b1, 0x3ba, 0x3bf,
0x3cd, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3cc, 0x3ba, 0x39c, 0x3c0, 0x3b1,
0x3c1, 0x3bd, 0x3b1, 0x3bf, 0x3cd, 0x3bb, 0x392, 0x3c5, 0x3c1, 0x3b7, 0x3c4, 0x3cc,
0x3c2, 0x39c, 0x3c0, 0x3b9, 0x3c3, 0x3ba, 0x3ad, 0x3ba, 0x39c, 0x3c0, 0x3c1, 0x3bf,
0x3c5, 0x3bd, 0x3ad, 0x3b9, 0x3a4, 0x3c3, 0x3b9, 0x3c4, 0x3ac, 0x39a, 0x3bf, 0x3bb,
0x3cc, 0x3bc, 0x3c0, 0x3bf, 0x394, 0x3b1, 0x3bc, 0x3b1, 0x3c3, 0x3ba, 0x3cc, 0x3c2,
0x39d, 0x3c4, 0x3ac, 0x3ba, 0x3b1, 0x39d, 0x3c4, 0x3af, 0x3bb, 0x3b9, 0x39d, 0x3c4,
0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3ac, 0x3b9, 0x39d, 0x3c4, 0x3bf, 0x3c5, 0x3c3, 0x3ac,
0x3bd, 0x3bc, 0x3c0, 0x3b5, 0x391, 0x3bc, 0x3bc, 0x3cc, 0x3c7, 0x3c9, 0x3c3, 0x3c4,
0x3bf, 0x3c2, 0x393, 0x3ac, 0x3b6, 0x3b1, 0x3a7, 0x3b5, 0x3b2, 0x3c1, 0x3ce, 0x3bd,
0x3b1, 0x3a0, 0x3cc, 0x3bb, 0x3b7, 0x20, 0x3a7, 0x3bf, 0x20, 0x3a4, 0x3c3, 0x3b9,
0x20, 0x39c, 0x3b9, 0x3bd, 0x3c7, 0x3a7, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x20, 0x39a,
0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x3a7, 0x3bf, 0x3b2, 0x3bd, 0x3c4, 0x399, 0x3c1, 0x3ba,
0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x3a4, 0x3b6, 0x3b1, 0x3ba, 0x3ac, 0x3c1, 0x3c4,
0x3b1, 0x3a4, 0x3b6, 0x3b1, 0x3b3, 0x3b9, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3c1, 0x3b1,
0x399, 0x3b5, 0x3c1, 0x3bf, 0x3c5, 0x3c3, 0x3b1, 0x3bb, 0x3ae, 0x3bc, 0x39a, 0x3b1,
0x3bc, 0x3c0, 0x3bf, 0x3cd, 0x3bb, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3, 0x3ac, 0x3c4,
0x3ba, 0x3b1, 0x39a, 0x3b1, 0x3c1, 0x3ac, 0x3c4, 0x3c3, 0x3b9, 0x39a, 0x3b1, 0x3c4,
0x3bc, 0x3b1, 0x3bd, 0x3c4, 0x3bf, 0x3cd, 0x3a7, 0x3b1, 0x3bd, 0x3c4, 0x3af, 0x3b3,
0x3ba, 0x3b1, 0x39a, 0x3b1, 0x3bb, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3b1, 0x39a, 0x3c1,
0x3b1, 0x3c3, 0x3bd, 0x3bf, 0x3b3, 0x3b9, 0x3ac, 0x3c1, 0x3c3, 0x3ba, 0x39a, 0x3bf,
0x3c5, 0x3ac, 0x3bb, 0x3b1, 0x20, 0x39b, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3bf, 0x3cd,
0x3c1, 0x39a, 0x3bf, 0x3c5, 0x3c4, 0x3c3, 0x3af, 0x3bd, 0x3b3, 0x3ba, 0x39a, 0x3bf,
0x3c5, 0x3b2, 0x3ad, 0x3b9, 0x3c4, 0x39c, 0x3b1, 0x3ba, 0x3ac, 0x3bf, 0x39c, 0x3b1,
0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3ac, 0x3bd, 0x39c, 0x3b1, 0x3ba, 0x3b1, 0x3c3,
0x3ac, 0x3c1, 0x39c, 0x3b1, 0x3bd, 0x3af, 0x3bb, 0x3b1, 0x39c, 0x3b1, 0x3c3, 0x3ba,
0x3ac, 0x3c4, 0x39b, 0x3b5, 0x3c5, 0x3ba, 0x3c9, 0x3c3, 0x3af, 0x3b1, 0x39d, 0x3bf,
0x3b2, 0x3bf, 0x3ba, 0x3bf, 0x3c5, 0x3b6, 0x3bd, 0x3ad, 0x3c4, 0x3c3, 0x3ba, 0x39d,
0x3bf, 0x3b2, 0x3bf, 0x3c3, 0x3b9, 0x3bc, 0x3c0, 0x3af, 0x3c1, 0x3c3, 0x3ba, 0x39f,
0x3bc, 0x3c3, 0x3ba, 0x39f, 0x3c1, 0x3ac, 0x3bb, 0x3a0, 0x3bd, 0x3bf, 0x3bc, 0x20,
0x3a0, 0x3b5, 0x3bd, 0x3c7, 0x3a0, 0x3cc, 0x3bd, 0x3c4, 0x3b9, 0x3b1, 0x3bd, 0x3b1,
0x3ba, 0x3a0, 0x3b9, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x3b3, 0x3b9, 0x3ac, 0x3bd, 0x3b3,
0x3ba, 0x39a, 0x3b1, 0x3c4, 0x3ac, 0x3c1, 0x39a, 0x3bf, 0x3c3, 0x3c4, 0x3b1, 0x3bd,
0x3ac, 0x3b9, 0x39a, 0x3b9, 0x3b6, 0x3b9, 0x3bb, 0x3bf, 0x3c1, 0x3bd, 0x3c4, 0x3ac,
0x3a1, 0x3b9, 0x3ac, 0x3bd, 0x3c4, 0x3a3, 0x3b1, 0x3c7, 0x3b1, 0x3bb, 0x3af, 0x3bd,
0x3b7, 0x3a3, 0x3b1, 0x3bc, 0x3b1, 0x3c1, 0x3ba, 0x3ac, 0x3bd, 0x3b4, 0x3b7, 0x3a3,
0x3b5, 0x3bf, 0x3cd, 0x3bb, 0x3a3, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3ac, 0x3b7, 0x3a3,
0x3b9, 0x3b3, 0x3ba, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3c1, 0x3b7, 0x3a3, 0x3c1, 0x3b5,
0x3bd, 0x3c4, 0x3bd, 0x3b5, 0x3ba, 0x3bf, 0x3bb, 0x3af, 0x3bc, 0x3c3, 0x3ba, 0x3a4,
0x3b1, 0x3ca, 0x3c0, 0x3ad, 0x3b9, 0x3a4, 0x3b1, 0x3c3, 0x3ba, 0x3ad, 0x3bd, 0x3b4,
0x3b7, 0x3a4, 0x3b9, 0x3c6, 0x3bb, 0x3af, 0x3b4, 0x3b1, 0x3a4, 0x3b5, 0x3c7, 0x3b5,
0x3c1, 0x3ac, 0x3bd, 0x3b7, 0x398, 0x3af, 0x3bc, 0x3c6, 0x3bf, 0x3c5, 0x3a4, 0x3cc,
0x3ba, 0x3b9, 0x3bf, 0x3a4, 0x3bf, 0x3bc, 0x3c3, 0x3ba, 0x39f, 0x3c5, 0x3bb, 0x3ac,
0x3bd, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x39f, 0x3c5, 0x3c1, 0x3bf,
0x3cd, 0x3bc, 0x3c4, 0x3c3, 0x3b9, 0x39f, 0x3c5, 0x3c3, 0x3c4, 0x2d, 0x39d, 0x3b5,
0x3c1, 0x3ac, 0x392, 0x3b9, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x392, 0x3bb,
0x3b1, 0x3b4, 0x3b9, 0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc, 0x3ba, 0x393, 0x3b9, 0x3b1,
0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x3a1, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3bf,
0x3cd, 0x3bd, 0x391, 0x3b9, 0x3ba, 0x3b1, 0x3c4, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3bf,
0x3cd, 0x3c0, 0x3bf, 0x3bb, 0x3b7, 0x395, 0x3c1, 0x3b5, 0x3b2, 0x3ac, 0x3bd, 0x391,
0x3b6, 0x3cc, 0x3c1, 0x3b5, 0x3c2, 0x392, 0x3b5, 0x3c1, 0x3bc, 0x3bf, 0x3cd, 0x3b4,
0x3b5, 0x3c2, 0x39a, 0x3b1, 0x3bd, 0x3ac, 0x3c1, 0x3b9, 0x3b1, 0x3a0, 0x3c1, 0x3ac,
0x3c3, 0x3b9, 0x3bd, 0x3bf, 0x20, 0x391, 0x3ba, 0x3c1, 0x3c9, 0x3c4, 0x3ae, 0x3c1,
0x3b9, 0x3bf, 0x3a6, 0x3b5, 0x3c1, 0x3cc, 0x3b5, 0x3c2, 0x39c, 0x3b1, 0x3b4, 0x3ad,
0x3c1, 0x3b1, 0x3a1, 0x3ad, 0x3c5, 0x3ba, 0x3b9, 0x3b1, 0x3b2, 0x3b9, 0x3ba, 0x39d,
0x3cc, 0x3c4, 0x3b9, 0x3b1, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af, 0x3b1,
0x391, 0x3b3, 0x2e, 0x20, 0x395, 0x3bb, 0x3ad, 0x3bd, 0x3b7, 0x3a3, 0x3c4, 0x3ac,
0x3bd, 0x3bb, 0x3b5, 0x3ca, 0x391, 0x3b4, 0x3b5, 0x3bb, 0x3b1, 0x390, 0x3b4, 0x3b1,
0x39c, 0x3c0, 0x3c1, 0x3b9, 0x3c3, 0x3bc, 0x3c0, 0x3ad, 0x3b9, 0x3bd, 0x39c, 0x3c0,
0x3c1, 0x3cc, 0x3ba, 0x3b5, 0x3bd, 0x20, 0x3a7, 0x3b9, 0x3bb, 0x39d, 0x3c4, 0x3ac,
0x3c1, 0x3b3, 0x3bf, 0x3c5, 0x3b9, 0x3bd, 0x393, 0x3b9, 0x3bf, 0x3cd, 0x3ba, 0x3bb,
0x3b1, 0x3a7, 0x3cc, 0x3bc, 0x3c0, 0x3b1, 0x3c1, 0x3c4, 0x39b, 0x3af, 0x3bd, 0x3c4,
0x3b5, 0x3bc, 0x3b1, 0x3bd, 0x39b, 0x3bf, 0x3c1, 0x3bd, 0x3c4, 0x20, 0x3a7, 0x3ac,
0x3bf, 0x3c5, 0x39c, 0x3b5, 0x3bb, 0x3b2, 0x3bf, 0x3cd, 0x3c1, 0x3bd, 0x3b7, 0x3a0,
0x3b5, 0x3c1, 0x3b8, 0x3a3, 0x3af, 0x3b4, 0x3bd, 0x3b5, 0x3ca, 0x386, 0x3bc, 0x3c3,
0x3c4, 0x3b5, 0x3c1, 0x3bd, 0x3c4, 0x3b1, 0x3bc, 0x391, 0x3bd, 0x3b4, 0x3cc, 0x3c1,
0x3b1, 0x391, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3c7, 0x3ac, 0x3bd, 0x391, 0x3b8, 0x3ae,
0x3bd, 0x3b1, 0x392, 0x3b5, 0x3bb, 0x3b9, 0x3b3, 0x3c1, 0x3ac, 0x3b4, 0x3b9, 0x392,
0x3b5, 0x3c1, 0x3bf, 0x3bb, 0x3af, 0x3bd, 0x3bf, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3c4,
0x3b9, 0x3c3, 0x3bb, 0x3ac, 0x3b2, 0x3b1, 0x392, 0x3c1, 0x3c5, 0x3be, 0x3ad, 0x3bb,
0x3bb, 0x3b5, 0x3c2, 0x392, 0x3bf, 0x3c5, 0x3ba, 0x3bf, 0x3c5, 0x3c1, 0x3ad, 0x3c3,
0x3c4, 0x3b9, 0x392, 0x3bf, 0x3c5, 0x3b4, 0x3b1, 0x3c0, 0x3ad, 0x3c3, 0x3c4, 0x3b7,
0x39c, 0x3c0, 0x3af, 0x3c3, 0x3b9, 0x3bd, 0x3b3, 0x3ba, 0x3b5, 0x3bd, 0x39a, 0x3b9,
0x3c3, 0x3b9, 0x3bd, 0x3ac, 0x3bf, 0x3c5, 0x39a, 0x3bf, 0x3c0, 0x3b5, 0x3b3, 0x3c7,
0x3ac, 0x3b3, 0x3b7, 0x394, 0x3bf, 0x3c5, 0x3b2, 0x3bb, 0x3af, 0x3bd, 0x3bf, 0x393,
0x3b9, 0x3b2, 0x3c1, 0x3b1, 0x3bb, 0x3c4, 0x3ac, 0x3c1, 0x393, 0x3ba, 0x3ad, 0x3c1,
0x3bd, 0x3b6, 0x3b9, 0x395, 0x3bb, 0x3c3, 0x3af, 0x3bd, 0x3ba, 0x3b9, 0x39d, 0x3ae,
0x3c3, 0x3bf, 0x3c2, 0x20, 0x3c4, 0x3bf, 0x3c5, 0x20, 0x39c, 0x3b1, 0x3bd, 0x39a,
0x3c9, 0x3bd, 0x3c3, 0x3c4, 0x3b1, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3bf, 0x3cd, 0x3c0,
0x3bf, 0x3bb, 0x3b7, 0x3a4, 0x3b6, 0x3ad, 0x3c1, 0x3c3, 0x3b5, 0x3ca, 0x39a, 0x3b1,
0x3bb, 0x3af, 0x3bd, 0x3b9, 0x3bd, 0x3b3, 0x3ba, 0x3c1, 0x3b1, 0x3bd, 0x3c4, 0x39a,
0x3af, 0x3c1, 0x3bf, 0x3c6, 0x39a, 0x3af, 0x3b5, 0x3b2, 0x3bf, 0x39b, 0x3b9, 0x3c3,
0x3b1, 0x3b2, 0x3cc, 0x3bd, 0x3b1, 0x39b, 0x3b9, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3bb,
0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x39b, 0x3bf, 0x3bd, 0x3b4, 0x3af, 0x3bd, 0x3bf, 0x39b,
0x3bf, 0x3c5, 0x3be, 0x3b5, 0x3bc, 0x3b2, 0x3bf, 0x3cd, 0x3c1, 0x3b3, 0x3bf, 0x39c,
0x3b1, 0x3b4, 0x3c1, 0x3af, 0x3c4, 0x3b7, 0x39c, 0x3ac, 0x3bb, 0x3c4, 0x3b1, 0x39c,
0x3ac, 0x3c1, 0x3b9, 0x3b5, 0x3c7, 0x3b1, 0x3bc, 0x3bd, 0x39c, 0x3b9, 0x3bd, 0x3c3,
0x3ba, 0x39c, 0x3bf, 0x3bd, 0x3b1, 0x3ba, 0x3cc, 0x39c, 0x3cc, 0x3c3, 0x3c7, 0x3b1,
0x38c, 0x3c3, 0x3bb, 0x3bf, 0x3a0, 0x3b1, 0x3c1, 0x3af, 0x3c3, 0x3b9, 0x3a0, 0x3bf,
0x3bd, 0x3c4, 0x3b3, 0x3ba, 0x3cc, 0x3c1, 0x3b9, 0x3c4, 0x3c3, 0x3b1, 0x3a0, 0x3c1,
0x3ac, 0x3b3, 0x3b1, 0x3a1, 0x3af, 0x3b3, 0x3b1, 0x3a1, 0x3ce, 0x3bc, 0x3b7, 0x3a3,
0x3b1, 0x3bc, 0x3ac, 0x3c1, 0x3b1, 0x386, 0x3b3, 0x3b9, 0x3bf, 0x3c2, 0x20, 0x39c,
0x3b1, 0x3c1, 0x3af, 0x3bd, 0x3bf, 0x3c2, 0x3a3, 0x3b1, 0x3c1, 0x3ac, 0x3b3, 0x3b5,
0x3b2, 0x3bf, 0x3a3, 0x3b1, 0x3c1, 0x3ac, 0x3c4, 0x3bf, 0x3c6, 0x3a3, 0x3c5, 0x3bc,
0x3c6, 0x3b5, 0x3c1, 0x3cc, 0x3c0, 0x3bf, 0x3bb, 0x3b7, 0x3a3, 0x3ba, 0x3cc, 0x3c0,
0x3b9, 0x3b1, 0x3a3, 0x3cc, 0x3c6, 0x3b9, 0x3b1, 0x3a3, 0x3c4, 0x3bf, 0x3ba, 0x3c7,
0x3cc, 0x3bb, 0x3bc, 0x3b7, 0x3a4, 0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3a4, 0x3af, 0x3c1,
0x3b1, 0x3bd, 0x3b1, 0x39f, 0x3c5, 0x3bb, 0x3b9, 0x3ac, 0x3bd, 0x3bf, 0x3c6, 0x3c3,
0x3ba, 0x392, 0x3b1, 0x3bd, 0x3c4, 0x3bf, 0x3cd, 0x3b6, 0x392, 0x3b1, 0x3c4, 0x3b9,
0x3ba, 0x3b1, 0x3bd, 0x3cc, 0x392, 0x3b9, 0x3ad, 0x3bd, 0x3bd, 0x3b7, 0x392, 0x3af,
0x3bb, 0x3bd, 0x3b9, 0x3bf, 0x3c5, 0x3c2, 0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba, 0x3bf,
0x3ba, 0x3c1, 0x3b1, 0x3bd, 0x3c4, 0x392, 0x3b1, 0x3c1, 0x3c3, 0x3bf, 0x3b2, 0x3af,
0x3b1, 0x396, 0x3ac, 0x3b3, 0x3ba, 0x3c1, 0x3b5, 0x3bc, 0x3c0, 0x396, 0x3c5, 0x3c1,
0x3af, 0x3c7, 0x3b7, 0x391, 0x3bd, 0x3c4, 0x3b1, 0x3bd, 0x3b1, 0x3bd, 0x3b1, 0x3c1,
0x3af, 0x3b2, 0x3bf, 0x3a4, 0x3c3, 0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x39d, 0x3ae,
0x3c3, 0x3bf, 0x3c2, 0x20, 0x3a7, 0x3c1, 0x3b9, 0x3c3, 0x3c4, 0x3bf, 0x3c5, 0x3b3,
0x3ad, 0x3bd, 0x3bd, 0x3c9, 0x3bd, 0x39a, 0x3cc, 0x3ba, 0x3bf, 0x3c2, 0x39a, 0x3bf,
0x3bc, 0x3cc, 0x3c1, 0x3bf, 0x39a, 0x3b5, 0x3c1, 0x3b3, 0x3ba, 0x3b5, 0x3bb, 0x3ad,
0x3bd, 0x39c, 0x3ac, 0x3c7, 0x3b5, 0x39c, 0x3b1, 0x3bb, 0x3b4, 0x3af, 0x3b2, 0x3b5,
0x3c2, 0x39c, 0x3b1, 0x3c5, 0x3c1, 0x3af, 0x3ba, 0x3b9, 0x3bf, 0x3c2, 0x39c, 0x3b1,
0x3b3, 0x3b9, 0x3cc, 0x3c4, 0x3a1, 0x3b5, 0x3ca, 0x3bd, 0x3b9, 0x3cc, 0x3bd, 0x391,
0x3c0, 0x3af, 0x3b1, 0x38c, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x39c, 0x3c0, 0x3bf,
0x3c5, 0x3b3, 0x3ba, 0x3b5, 0x3bd, 0x3b2, 0x3af, 0x3bb, 0x3a4, 0x3c3, 0x3ac, 0x3c4,
0x3b1, 0x3bc, 0x3a4, 0x3c3, 0x3bf, 0x3c5, 0x3ba, 0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c2,
0x20, 0x3a0, 0x3ac, 0x3c3, 0x3c7, 0x3b1, 0x395, 0x3c6, 0x3ac, 0x3c4, 0x3b5, 0x3a6,
0x3b1, 0x3ba, 0x3b1, 0x3cc, 0x3c6, 0x3bf, 0x3a6, 0x3af, 0x3c4, 0x3b6, 0x3b9, 0x3a6,
0x3bf, 0x3c5, 0x3bd, 0x3b1, 0x3c6, 0x3bf, 0x3cd, 0x3c4, 0x3b9, 0x393, 0x3ba, 0x3b1,
0x3bb, 0x3ac, 0x3c0, 0x3b1, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x393, 0x3ba, 0x3ac, 0x3bc,
0x3c0, 0x3b9, 0x3b5, 0x3c1, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3b1, 0x3bd, 0x3c4, 0x3b1,
0x3bb, 0x3ba, 0x3b1, 0x3bd, 0x3ac, 0x3bb, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3ac, 0x3bc,
0x3a7, 0x3bf, 0x3bd, 0x3bf, 0x3bb, 0x3bf, 0x3c5, 0x3bb, 0x3bf, 0x3cd, 0x388, 0x3bd,
0x3c4, 0x3b5, 0x3c1, 0x3bc, 0x3c0, 0x3b5, 0x3c1, 0x3b9, 0x39a, 0x3b9, 0x3c1, 0x3b9,
0x3c4, 0x3b9, 0x3bc, 0x3ac, 0x3c4, 0x3b9, 0x39a, 0x3cc, 0x3c3, 0x3c1, 0x3b1, 0x3ca,
0x39a, 0x3bf, 0x3c5, 0x3b1, 0x3c4, 0x3b6, 0x3b1, 0x3bb, 0x3ad, 0x3b9, 0x3bd, 0x39c,
0x3b1, 0x3c4, 0x3b6, 0x3bf, 0x3cd, 0x3c1, 0x3bf, 0x39c, 0x3b1, 0x3c1, 0x3ba, 0x3ad,
0x3b6, 0x3b1, 0x3c2, 0x39c, 0x3af, 0x3bd, 0x3c4, 0x3b3, 0x3bf, 0x3c5, 0x3b5, 0x3ca,
0x39d, 0x3b1, 0x3bf, 0x3cd, 0x3c1, 0x3bf, 0x3c5, 0x39d, 0x3b9, 0x3bf, 0x3cd, 0x3b5,
0x39d, 0x3cc, 0x3c1, 0x3c6, 0x3bf, 0x3bb, 0x3ba, 0x39d, 0x3bf, 0x3c5, 0x3bc, 0x3ad,
0x3b1, 0x3a0, 0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x20, 0x3a0, 0x3ac, 0x3b3, 0x3ba, 0x3bf,
0x3a0, 0x3b1, 0x3bb, 0x3ac, 0x3bf, 0x3c5, 0x3a0, 0x3af, 0x3c4, 0x3ba, 0x3b5, 0x3c1,
0x3bd, 0x3a0, 0x3bf, 0x3bd, 0x3ac, 0x3c0, 0x3b5, 0x3a0, 0x3bf, 0x3c1, 0x3c4, 0x20,
0x39c, 0x3cc, 0x3c1, 0x3b5, 0x3c3, 0x3bc, 0x3c0, 0x3b9, 0x3a1, 0x3b1, 0x3c1, 0x3bf,
0x3c4, 0x3cc, 0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x3a3, 0x3b1, 0x3ca, 0x3c0, 0x3ac, 0x3bd,
0x3a4, 0x3b1, 0x3ca, 0x3c4, 0x3ae, 0x3a4, 0x3b1, 0x3c1, 0x3ac, 0x3bf, 0x3c5, 0x3b1,
0x3a4, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x3c4, 0x3ac, 0x3c0, 0x3bf, 0x3c5, 0x393,
0x3bf, 0x3c5, 0x3ad, 0x3b9, 0x3ba, 0x393, 0x3bf, 0x3c5, 0x3ac, 0x3bb, 0x3b9, 0x3c2,
0xa86, 0xaac, 0xabf, 0xaa6, 0xacd, 0xa9c, 0xabe, 0xaa8, 0xa8d, 0xa95, 0xab0, 0xabe,
0xa8f, 0xaa1, 0xabf, 0xab8, 0x20, 0xa85, 0xaac, 0xabe, 0xaac, 0xabe, 0xa85, 0xab2,
0xacd, 0xa9c, 0xac0, 0xaaf, 0xab0, 0xacd, 0xab8, 0xa85, 0xab8, 0xacd, 0xaae, 0xabe,
0xab0, 0xabe, 0xaac, 0xaae, 0xac7, 0xa95, 0xacb, 0xaac, 0xabe, 0xa82, 0xa97, 0xac1,
0xa88, 0xaac, 0xabe, 0xa82, 0xa9c, 0xac1, 0xab2, 0xaac, 0xabf, 0xab8, 0xabe, 0xa89,
0xaac, 0xacd, 0xab2, 0xac7, 0xa82, 0xa9f, 0xac0, 0xab0, 0xaac, 0xacd, 0xab0, 0xabe,
0xa9d, 0xabe, 0xab5, 0xabf, 0xab2, 0xac7, 0xaac, 0xac1, 0xa9c, 0xaae, 0xacd, 0xaac,
0xac1, 0xab0, 0xabe, 0xa95, 0xac8, 0xab0, 0xacb, 0xa95, 0xabe, 0xab8, 0xabe, 0xaac,
0xacd, 0xab2, 0xabe, 0xa82, 0xa95, 0xabe, 0xa95, 0xacd, 0xab5, 0xac7, 0xa9f, 0xabe,
0xa95, 0xacb, 0xaa8, 0xac5, 0xa95, 0xacd, 0xab0, 0xac0, 0xaa1, 0xa95, 0xabe, 0xab0,
0xaa6, 0xabe, 0xab0, 0x20, 0xa8f, 0xab8, 0x20, 0xab8, 0xab2, 0xabe, 0xaae, 0xa9c,
0xac0, 0xaac, 0xac1, 0xa9f, 0xac0, 0xaa1, 0xacc, 0xa86, 0xab2, 0xabe, 0xa8f, 0xab2,
0x20, 0xa90, 0xa89, 0xaa8, 0xaab, 0xacd, 0xab0, 0xac0, 0xa9f, 0xabe, 0xa89, 0xaa8,
0xa97, 0xac5, 0xaac, 0xacb, 0xab0, 0xacb, 0xaa8, 0xab9, 0xab0, 0xabe, 0xab0, 0xac7,
0xa9c, 0xacb, 0xab9, 0xabe, 0xaa8, 0xacd, 0xab8, 0xaac, 0xab0, 0xacd, 0xa97, 0xa9c,
0xac1, 0xaac, 0xabe, 0xa95, 0xaae, 0xacd, 0xaaa, 0xabe, 0xab2, 0xabe, 0xa96, 0xabe,
0xab0, 0xacd, 0xa9f, 0xac2, 0xaae, 0xa95, 0xabf, 0xa97, 0xabe, 0xab2, 0xac0, 0xa95,
0xabf, 0xa82, 0xab6, 0xabe, 0xab8, 0xabe, 0xab2, 0xabe, 0xa97, 0xacb, 0xab8, 0xab2,
0xabf, 0xaac, 0xacd, 0xab0, 0xac7, 0xab5, 0xabf, 0xab2, 0xac7, 0xab2, 0xacb, 0xaae,
0xab2, 0xacd, 0xaaf, 0xac1, 0xa8f, 0xaa8, 0xacd, 0xaa1, 0xabe, 0xab2, 0xac1, 0xaac,
0xac1, 0xaae, 0xacd, 0xaac, 0xabe, 0xab6, 0xac0, 0xab2, 0xac1, 0xab8, 0xabe, 0xa95,
0xabe, 0xaae, 0xab2, 0xabe, 0xaac, 0xacb, 0xaae, 0xabe, 0xaaa, 0xac1, 0xa9f, 0xacb,
0xaae, 0xab8, 0xac7, 0xab0, 0xac1, 0xa85, 0xaae, 0xacd, 0xaac, 0xaac, 0xabe, 0xaa8,
0xaae, 0xacb, 0xa97, 0xabe, 0xaa1, 0xac0, 0xab6, 0xac1, 0xaae, 0xacb, 0xa82, 0xab0,
0xacb, 0xab5, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xac8, 0xab0, 0xacb, 0xaac, 0xac0, 0xa85,
0xaa8, 0xacd, 0xa9c, 0xaae, 0xac7, 0xaa8, 0xaa8, 0xabf, 0xaaf, 0xabe, 0xaae, 0xac0,
0xaa8, 0xacc, 0xa95, 0xa9a, 0xacb, 0xa9f, 0xa94, 0xa86, 0xa97, 0xabe, 0xaa6, 0xacc,
0xa97, 0xacc, 0xaaa, 0xacb, 0xab0, 0xacd, 0xa9f, 0xacb, 0x2d, 0xaa8, 0xacb, 0xab5,
0xacb, 0xab8, 0xabe, 0xa93, 0x20, 0xa9f, 0xac9, 0xaae, 0xa9f, 0xacd, 0xab0, 0xac0,
0xaaa, 0xacb, 0xab2, 0xac0, 0xa9f, 0xacd, 0xaaf, 0xac1, 0xaa8, 0xabf, 0xab8, 0xab5,
0xabf, 0xa82, 0xaa1, 0xab9, 0xac9, 0xa95, 0xa85, 0xaa6, 0xa95, 0xa8f, 0xaa8, 0xacd,
0xa95, 0xacb, 0xab0, 0xac7, 0xa9c, 0xa8d, 0xa82, 0xa97, 0xacd, 0xab5, 0xabf, 0xab2,
0xabe, 0xa8d, 0xaa8, 0xacd, 0xa9f, 0xabf, 0xa97, 0xac1, 0xa86, 0xa85, 0xab0, 0xabe,
0xa97, 0xac1, 0xa87, 0xaa8, 0xabe, 0xaac, 0xacd, 0xaaf, 0xac1, 0xaa8, 0xab8, 0x20,
0xa8f, 0xaaf, 0xab0, 0xacd, 0xab8, 0xa95, 0xabe, 0xaa4, 0xabe, 0xaae, 0xabe, 0xab0,
0xacd, 0xa95, 0xabe, 0xa95, 0xacb, 0xab0, 0xacd, 0xaa1, 0xacb, 0xaac, 0xabe, 0xab9,
0xac1, 0xab9, 0xac1, 0xa87, 0xab2, 0xabe, 0x20, 0xab0, 0xabf, 0xa93, 0xab9, 0xabe,
0xaae, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacb, 0xa9d, 0xabe, 0xab0, 0xabf, 0xa93, 0x20,
0xa97, 0xac7, 0xab2, 0xac7, 0xa97, 0xacb, 0xab8, 0xab8, 0xabe, 0xab2, 0xacd, 0xa9f,
0xabe, 0xab8, 0xac7, 0xaa8, 0x20, 0xa9c, 0xac1, 0xa8f, 0xaa8, 0xab8, 0xac7, 0xaa8,
0x20, 0xab2, 0xac1, 0xa87, 0xab8, 0xaa4, 0xac1, 0xa95, 0xac1, 0xaae, 0xabe, 0xaa8,
0xa89, 0xab6, 0xabe, 0xaaf, 0xabe, 0xa85, 0xab0, 0xac1, 0xaac, 0xabe, 0xa8f, 0xab8,
0xaa8, 0xacd, 0xab6, 0xabf, 0xaaf, 0xaa8, 0xa8f, 0xa9f, 0xabf, 0xa95, 0xacb, 0xa95,
0xac7, 0xaa8, 0xaac, 0xabe, 0xab9, 0xabf, 0xaaf, 0xabe, 0xaac, 0xab9, 0xabf, 0xaaf,
0xabe, 0x20, 0xaac, 0xabe, 0xa82, 0xaa6, 0xacd, 0xab0, 0xabe, 0xab8, 0xaac, 0xabe,
0xab0, 0xaac, 0xabe, 0xaa1, 0xacb, 0xab8, 0xaac, 0xac7, 0xab2, 0xac7, 0xaae, 0xaac,
0xac7, 0xab2, 0xac0, 0xa9d, 0xaac, 0xacd, 0xab2, 0xabe, 0xa82, 0xa95, 0x2d, 0xab8,
0xac7, 0xaac, 0xab2, 0xacb, 0xaa8, 0xaac, 0xacb, 0xa86, 0x20, 0xab5, 0xabf, 0xab8,
0xacd, 0xa9f, 0xabe, 0xaac, 0xacb, 0xa97, 0xacb, 0xa9f, 0xabe, 0xaac, 0xacb, 0xa87,
0xa9d, 0xa95, 0xac7, 0xaae, 0xacd, 0xaac, 0xacd, 0xab0, 0xabf, 0xa9c, 0x20, 0xaac,
0xac7, 0xa95, 0xabe, 0xaae, 0xacd, 0xaaa, 0xacb, 0x20, 0xa97, 0xacd, 0xab0, 0xabe,
0xa82, 0xaa1, 0xa95, 0xabe, 0xaa8, 0xacd, 0xa95, 0xac1, 0xaa8, 0xa95, 0xac5, 0xab0,
0xabe, 0xa95, 0xab8, 0xa95, 0xabe, 0xaaf, 0xac7, 0xaa8, 0xac7, 0xa95, 0xac7, 0xaae,
0xac7, 0xaa8, 0xab6, 0xabf, 0xa95, 0xabe, 0xa97, 0xacb, 0xa9a, 0xabf, 0xab9, 0xac1,
0xa86, 0xab9, 0xac1, 0xa86, 0xab8, 0xabf, 0xaaf, 0xac1, 0xaa6, 0xabe, 0xaa6, 0x20,
0xab9, 0xac1, 0xa86, 0xab0, 0xac7, 0xa9d, 0xa95, 0xacb, 0xab8, 0xacd, 0xa9f, 0xabe,
0x20, 0xab0, 0xabf, 0xa95, 0xabe, 0xa95, 0xacd, 0xab0, 0xac7, 0xab8, 0xacd, 0xa9f,
0xaa8, 0xa95, 0xacd, 0xaaf, 0xac1, 0xa8f, 0xaac, 0xabe, 0xa95, 0xac1, 0xab0, 0xabe,
0xa95, 0xabe, 0xa93, 0xaa1, 0xac7, 0xaa8, 0xaae, 0xabe, 0xab0, 0xacd, 0xa95, 0xab6,
0xac9, 0xaa8, 0xaa1, 0xacc, 0xab8, 0xaa8, 0xaa1, 0xacc, 0xab8, 0xaa8, 0x20, 0xa95,
0xacd, 0xab0, 0xabf, 0xa95, 0xaa6, 0xac7, 0xaa8, 0xacd, 0xab5, 0xab0, 0xaa1, 0xac7,
0xa9f, 0xacd, 0xab0, 0xacb, 0xa87, 0xa9f, 0xaa1, 0xacb, 0xaae, 0xabf, 0xaa8, 0xabf,
0xa95, 0xabe, 0xa8f, 0xaa1, 0xaae, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xaa8, 0xa87, 0xab0,
0xac1, 0xaa8, 0xac7, 0xaaa, 0xa8f, 0xab2, 0x20, 0xab8, 0xac7, 0xab2, 0xacd, 0xab5,
0xabe, 0xaa1, 0xacb, 0xab0, 0xaab, 0xacb, 0xab0, 0xacd, 0xa9f, 0x20, 0xaa8, 0xac7,
0xab2, 0xacd, 0xab8, 0xaa8, 0xaab, 0xacb, 0xab0, 0xacd, 0xa9f, 0xabe, 0xab2, 0xac7,
0xa9d, 0xabe, 0xa97, 0xacd, 0xab2, 0xac7, 0xab8, 0x20, 0xaac, 0xac7, 0xa97, 0xac2,
0xab8, 0x20, 0xaac, 0xac7, 0xa97, 0xacd, 0xab0, 0xabe, 0xaa8, 0xacd, 0xaa1, 0x20,
0xa9f, 0xab0, 0xacd, 0xa95, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa8, 0xac7, 0xaa1, 0xabe,
0xa97, 0xacd, 0xab5, 0xabe, 0xaa1, 0xac7, 0xab2, 0xacb, 0xaaa, 0xa97, 0xacd, 0xab5,
0xabe, 0xa9f, 0xac7, 0xaae, 0xabe, 0xab2, 0xabe, 0xa97, 0xac1, 0xaaf, 0xabe, 0xa95,
0xacd, 0xab5, 0xabf, 0xab2, 0xa97, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xab9, 0xac7, 0xab2,
0xabf, 0xaab, 0xac7, 0xa95, 0xacd, 0xab8, 0xab9, 0xab5, 0xabe, 0xaa8, 0xabe, 0xa8f,
0xab0, 0xaae, 0xacb, 0xab8, 0xabf, 0xaaf, 0xacb, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf,
0xaaf, 0xabe, 0xaa8, 0xabe, 0xaaa, 0xacb, 0xab2, 0xabf, 0xab8, 0xaa8, 0xacb, 0xa95,
0xacd, 0xab8, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8,
0xabe, 0xaae, 0xac7, 0xab0, 0xac7, 0xa82, 0xa97, 0xacb, 0x2c, 0x20, 0xa87, 0xaa8,
0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xaaa, 0xabf, 0xa9f, 0xab0, 0xacd,
0xab8, 0xaac, 0xab0, 0xacd, 0xa97, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf,
0xaaf, 0xabe, 0xaa8, 0xabe, 0xa9f, 0xac7, 0xab2, 0x20, 0xab8, 0xabf, 0xa9f, 0xac0,
0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xab5,
0xac7, 0xab5, 0xac8, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe,
0xaa8, 0xabe, 0xab5, 0xabf, 0xaa8, 0xacd, 0xab8, 0xac7, 0xaa8, 0xacd, 0xab8, 0x2c,
0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xab5, 0xabf,
0xaa8, 0xabe, 0xaae, 0xac7, 0xa95, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf,
0xaaf, 0xabe, 0xaa8, 0xabe, 0xa87, 0xaa8, 0xac1, 0xab5, 0xabf, 0xa95, 0xa87, 0xa95,
0xabe, 0xab2, 0xac1, 0xa87, 0xaa4, 0xa9c, 0xaae, 0xac8, 0xa95, 0xabe, 0xa9c, 0xac1,
0xaa8, 0xac7, 0xa87, 0xab2, 0xac2, 0xa87, 0xab5, 0xabf, 0xab2, 0xaae, 0xacb, 0xaa8,
0xacd, 0xa9f, 0xac0, 0xab8, 0xac7, 0xab2, 0xacb, 0x2c, 0x20, 0xa95, 0xac7, 0xaa8,
0xacd, 0xa9f, 0xac1, 0xa95, 0xac0, 0xa95, 0xacd, 0xab0, 0xabe, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0xabf, 0xa95, 0xab2, 0xabe, 0x20, 0xaaa, 0xac7, 0xa9d, 0xab2, 0xabf,
0xaae, 0xabe, 0xab2, 0xacb, 0xab8, 0x20, 0xa8f, 0xa82, 0xa9c, 0xac7, 0xab2, 0xac7,
0xab8, 0xab2, 0xacb, 0xa85, 0xab0, 0x20, 0xaaa, 0xacd, 0xab0, 0xabf, 0xaa8, 0xacd,
0xab8, 0xaa8, 0xac1, 0xa82, 0x20, 0xa95, 0xacd, 0xab5, 0xabe, 0xab0, 0xacd, 0xa9f,
0xab0, 0xaae, 0xac7, 0xab8, 0xabf, 0xa93, 0xaae, 0xabe, 0xaa8, 0xabe, 0xa97, 0xac1,
0xa86, 0xaae, 0xaa8, 0xacc, 0xab8, 0xaae, 0xac7, 0xab0, 0xac0, 0xa97, 0xacb, 0xa9f,
0xaae, 0xabe, 0xab0, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xac0, 0xa95, 0xaae, 0xabe, 0xaa4,
0xabe, 0xaae, 0xacb, 0xab0, 0xacb, 0xab8, 0xaae, 0xa9d, 0xaa4, 0xacd, 0xab2, 0xabe,
0xaa8, 0xaae, 0xac7, 0xaa8, 0xacb, 0xaae, 0xabf, 0xaa8, 0xac0, 0xaae, 0xac7, 0xab0,
0xabf, 0xaa6, 0xabe, 0xaae, 0xac7, 0xa9f, 0xacd, 0xab2, 0xabe, 0xa95, 0xabe, 0xa9f,
0xacd, 0xab2, 0xabe, 0xaae, 0xac7, 0xa95, 0xacd, 0xab8, 0xabf, 0xa95, 0xacb, 0x20,
0xab8, 0xabf, 0xa9f, 0xac0, 0xaae, 0xabf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xab2, 0xacb,
0xaa8, 0xaae, 0xacb, 0xaa8, 0xacd, 0xa95, 0xa9f, 0xaa8, 0xaae, 0xacb, 0xaa8, 0xacd,
0xaa4, 0xacb, 0xab0, 0xac7, 0xaae, 0xac9, 0xa82, 0xa9f, 0xac7, 0xab5, 0xabf, 0xaa1,
0xabf, 0xa93, 0xaae, 0xacb, 0xa82, 0xa9f, 0xab8, 0xac7, 0xab0, 0xabe, 0xaa4, 0xaa8,
0xabe, 0xab8, 0xabe, 0xa89, 0xaa8, 0xacd, 0xaaf, 0xac2, 0xaaf, 0xacb, 0xab0, 0xacd,
0xa95, 0xaa8, 0xacb, 0xab0, 0xacb, 0xaa8, 0xacd, 0xab9, 0xabe, 0xaac, 0xabf, 0xaaf,
0xac1, 0xab2, 0xabe, 0xab9, 0x2c, 0x20, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20,
0xaa1, 0xac7, 0xa95, 0xacb, 0xa9f, 0xabe, 0xab8, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xab0,
0x2c, 0x20, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xaa1, 0xac7, 0xa95, 0xacb,
0xa9f, 0xabe, 0xaa8, 0xacd, 0xaaf, 0xac1, 0x20, 0xab8, 0xac7, 0xab2, 0xaae, 0x2c,
0x20, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xaa1, 0xac7, 0xa95, 0xacb, 0xa9f,
0xabe, 0xaa8, 0xac2, 0xa95, 0xa93, 0xa9c, 0xabf, 0xaa8, 0xabe, 0xa97, 0xabe, 0xaaa,
0xaa8, 0xabe, 0xaae, 0xabe, 0xaaa, 0xac7, 0xab0, 0xabe, 0xaae, 0xac7, 0xab0, 0xabf,
0xaac, 0xacb, 0xaab, 0xacb, 0xaa8, 0xabf, 0xa95, 0xacd, 0xab8, 0xaaa, 0xacb, 0xab0,
0xacd, 0xa9f, 0x2d, 0xa93, 0x2d, 0xaaa, 0xacd, 0xab0, 0xabf, 0xaa8, 0xacd, 0xab8,
0xaaa, 0xacb, 0xab0, 0xacd, 0xa9f, 0x20, 0xa91, 0xaab, 0x20, 0xab8, 0xacd, 0xaaa,
0xac7, 0xaa8, 0xaaa, 0xacb, 0xab0, 0xacd, 0xa9f, 0xacb, 0x20, 0xab5, 0xac7, 0xab2,
0xacd, 0xab9, 0xacb, 0xaaa, 0xacd, 0xaaf, 0xac1, 0xa85, 0xab0, 0xacd, 0xa9f, 0xacb,
0x20, 0xab0, 0xabf, 0xa95, 0xacb, 0xaaa, 0xac1, 0xaa8, 0xacd, 0xa9f, 0xabe, 0x20,
0xa8f, 0xab0, 0xac0, 0xaa8, 0xabe, 0xa9d, 0xab0, 0xac7, 0xa82, 0xa95, 0xabf, 0xaa8,
0x20, 0xa87, 0xaa8, 0xacd, 0xab2, 0xac7, 0xa9f, 0xab0, 0xac7, 0xab8, 0xac0, 0xaab,
0xab0, 0xac7, 0xa9c, 0xac0, 0xaa8, 0xabe, 0xab0, 0xac7, 0xa9d, 0xacb, 0xab2, 0xac2,
0xa9f, 0xab0, 0xabf, 0xaaf, 0xacb, 0x20, 0xaac, 0xacd, 0xab0, 0xabe, 0xa82, 0xa95,
0xacb, 0xab8, 0xac7, 0xaa8, 0xacd, 0xaa4, 0xabe, 0xab0, 0xac7, 0xaae, 0xab8, 0xabe,
0xa82, 0xa9f, 0xabf, 0xaaf, 0xabe, 0xa97, 0xacb, 0xab8, 0xac7, 0xa82, 0xa9f, 0xacb,
0x20, 0xaa1, 0xacb, 0xaae, 0xabf, 0xa82, 0xa97, 0xacb, 0xab8, 0xabe, 0xa93, 0x20,
0xaaa, 0xabe, 0xa89, 0xab2, 0xacb, 0xa87, 0xaa4, 0xacb, 0xa95, 0xacb, 0xab0, 0xacd,
0xa9f, 0xacb, 0xab0, 0xacb, 0xaae, 0xabf, 0xa9f, 0xab8, 0xabf, 0xa9f, 0xacd, 0xa95,
0xabe, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xaac, 0xabe, 0xab0, 0xacd, 0xaa5, 0xac7,
0xab2, 0xac7, 0xaae, 0xac0, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xa9c, 0xacd, 0xab9,
0xacb, 0xaa8, 0xacd, 0xab8, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xa95, 0xac0, 0xa9f,
0xacd, 0xab8, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xab2, 0xac1, 0xab8, 0xabf, 0xaaf,
0xabe, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xaa5, 0xacb, 0xaae, 0xab8, 0xab8, 0xac7,
0xa82, 0xa9f, 0x20, 0xab5, 0xabf, 0xaa8, 0xacd, 0xab8, 0xac7, 0xaa8, 0xacd, 0xa9f,
0xab8, 0xacd, 0xab5, 0xabf, 0xaab, 0xacd, 0xa9f, 0x20, 0xa95, 0xab0, 0xa82, 0xa9f,
0xaa4, 0xac7, 0xa97, 0xac1, 0xab8, 0xabf, 0xa97, 0xab2, 0xacd, 0xaaa, 0xabe, 0xaa5,
0xac1, 0xab2, 0xac7, 0xaa4, 0xabf, 0xa9c, 0xac1, 0xa86, 0xaa8, 0xabe, 0xa9f, 0xacb,
0xab0, 0xaa8, 0xacd, 0xa9f, 0xacb, 0xa9f, 0xacb, 0xab0, 0xacd, 0xa9f, 0xacb, 0xab2,
0xabe, 0xab5, 0xac7, 0xaa8, 0xacd, 0xa95, 0xac1, 0xab5, 0xab0, 0xab5, 0xacd, 0xab9,
0xabe, 0xa87, 0xa9f, 0xab9, 0xacb, 0xab0, 0xacd, 0xab8, 0xab5, 0xabf, 0xaa8, 0xacd,
0xaa8, 0xabf, 0xaaa, 0xac7, 0xa97, 0xaaf, 0xa95, 0xac1, 0xaa4, 0xaa4, 0xa95, 0xac5,
0xab8, 0xac0, 0xaa1, 0xac7, 0xab5, 0xabf, 0xab8, 0xaa6, 0xac1, 0xaae, 0xacb, 0xaa8,
0xacd, 0xaa4, 0x20, 0xaa6, 0xac0, 0x2018, 0xa89, 0xab0, 0xacd, 0xab5, 0xabf, 0xab2,
0xaae, 0xac7, 0xa95, 0xacd, 0xab5, 0xac7, 0xab0, 0xac0, 0xaae, 0xacb, 0xab8, 0xaa8,
0xaae, 0xac5, 0xa95, 0xaae, 0xab0, 0xacd, 0xaa1, 0xacb, 0xaaa, 0xabe, 0xab2, 0xacd,
0xaae, 0xab0, 0xab0, 0xacb, 0xaa5, 0xac7, 0xab0, 0xabe, 0xab8, 0xacd, 0xaaf, 0xacb,
0xab5, 0xabe, 0xa9f, 0xacd, 0xab0, 0xacb, 0xab2, 0xab5, 0xacb, 0xab8, 0xacd, 0xa9f,
0xac9, 0xa95, 0xab2, 0xacb, 0xa82, 0xa97, 0xa87, 0xaaf, 0xab0, 0xaac, 0xabf, 0xaaf,
0xac7, 0xaa8, 0xa8f, 0xaa6, 0xac7, 0xaa8, 0xa85, 0xab2, 0xacd, 0xaae, 0xabe, 0xa9f,
0xac0, 0xa85, 0xaae, 0xacd, 0xaae, 0xabe, 0xaa8, 0xa85, 0xaa8, 0xaa6, 0xabf, 0xab0,
0xa85, 0xa95, 0xa9f, 0xabe, 0xa89, 0xa8d, 0xa95, 0xacd, 0xa9f, 0xacb, 0xaac, 0xa85,
0xab6, 0xa97, 0xabe, 0xaac, 0xa9f, 0xa85, 0xaa4, 0xacd, 0xaaf, 0xabe, 0xab0, 0xac1,
0xaac, 0xa97, 0xaa6, 0xabe, 0xaa6, 0xaac, 0xac7, 0xab9, 0xab0, 0xac0, 0xaa8, 0xaac,
0xac7, 0xa82, 0xa97, 0xa95, 0xacb, 0xa95, 0xaac, 0xabe, 0xab0, 0xaa8, 0xacc, 0xab2,
0xaac, 0xac8, 0xab0, 0xac1, 0xaa4, 0xaac, 0xabf, 0xab6, 0xa95, 0xac7, 0xa95, 0xaac,
0xacd, 0xab0, 0xac1, 0xaa8, 0xac7, 0xa87, 0xa9a, 0xabf, 0xaa4, 0xabe, 0xa95, 0xacb,
0xab2, 0xa82, 0xaac, 0xacb, 0xaa6, 0xaae, 0xabe, 0xab8, 0xacd, 0xa95, 0xab8, 0xaa2,
0xabe, 0xa95, 0xabe, 0xaa6, 0xabf, 0xab2, 0xabf, 0xaa6, 0xac1, 0xaac, 0xa88, 0xaa6,
0xac1, 0xab6, 0xabe, 0xaae, 0xacd, 0xaac, 0xac7, 0xaab, 0xabe, 0xaae, 0xabe, 0xa97,
0xac1, 0xab8, 0xacd, 0xaa4, 0xabe, 0xa97, 0xabe, 0xa9d, 0xabe, 0xab9, 0xac7, 0xaac,
0xacd, 0xab0, 0xacb, 0xaa8, 0xab9, 0xacb, 0x20, 0xa9a, 0xac0, 0x20, 0xaae, 0xac0,
0xaa8, 0x20, 0xab8, 0xabf, 0xa9f, 0xac0, 0xab9, 0xacb, 0xa82, 0xa97, 0xa95, 0xacb,
0xa82, 0xa97, 0xab9, 0xacb, 0xab5, 0xacd, 0xaa1, 0xa87, 0xab0, 0xacd, 0xa95, 0xac1,
0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0xa9c, 0xa95, 0xabe, 0xab0, 0xacd, 0xaa4, 0xabe,
0xa9c, 0xaaf, 0xabe, 0xaaa, 0xac1, 0xab0, 0xabe, 0xa9c, 0xac7, 0xab0, 0xac1, 0xab8,
0xab2, 0xac7, 0xaae, 0xa95, 0xabe, 0xaac, 0xac1, 0xab2, 0xa95, 0xabe, 0xaae, 0xa9a,
0xa9f, 0xacd, 0xa95, 0xabe, 0xa95, 0xab0, 0xabe, 0xa9a, 0xac0, 0xa95, 0xabe, 0xaa0,
0xaae, 0xa82, 0xaa1, 0xac1, 0xa96, 0xa82, 0xaa1, 0xabf, 0xa97, 0xabe, 0xa95, 0xacb,
0xab2, 0xa95, 0xabe, 0xaa4, 0xabe, 0xa95, 0xacd, 0xab0, 0xabf, 0xab8, 0xacd, 0xaa8,
0xacb, 0xaaf, 0xabe, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0xa95, 0xac1, 0xa86, 0xab2,
0xabe, 0x20, 0xab2, 0xac1, 0xaae, 0xacd, 0xaaa, 0xac1, 0xab0, 0xa95, 0xac1, 0xa9a,
0xabf, 0xa82, 0xa97, 0xa95, 0xac1, 0xab5, 0xac8, 0xaa4, 0xaae, 0xa95, 0xabe, 0xa89,
0xaae, 0xac7, 0xa97, 0xabe, 0xaa1, 0xaa8, 0xaae, 0xabe, 0xa95, 0xab8, 0xacd, 0xab8,
0xab0, 0xaae, 0xaa8, 0xabf, 0xab2, 0xabe, 0xaae, 0xab8, 0xacd, 0xa95, 0xaa4, 0xaa8,
0xabf, 0xa95, 0xacb, 0xab8, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xacb, 0xab5, 0xacb, 0xa95,
0xac1, 0xa9d, 0xacd, 0xaa8, 0xac7, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0xaa8, 0xacb,
0xab5, 0xacb, 0xab8, 0xac0, 0xab0, 0xacd, 0xaac, 0xabf, 0xab0, 0xacd, 0xab8, 0xacd,
0xa95, 0xa93, 0xaae, 0xacd, 0xab8, 0xacd, 0xa95, 0xa93, 0xab0, 0xab2, 0xaab, 0xacd,
0xaa8, 0xacb, 0xaae, 0x20, 0xaaa, 0xac7, 0xaa8, 0xacd, 0xab9, 0xaaa, 0xacb, 0xaa8,
0xacd, 0xa9f, 0xabf, 0xaaf, 0xaa8, 0xac7, 0xa95, 0xaaa, 0xacd, 0xaaf, 0xacb, 0xa82,
0xa97, 0xaaf, 0xabe, 0xa82, 0xa97, 0xa95, 0xaa4, 0xabe, 0xab0, 0xa95, 0xacb, 0xab8,
0xacd, 0xa9f, 0xabe, 0xaa8, 0xac7, 0xa95, 0xabf, 0xa9d, 0xabf, 0xab2, 0xacb, 0xab0,
0xacd, 0xaa1, 0xabe, 0xab0, 0xabf, 0xaaf, 0xabe, 0xaa7, 0xab8, 0xa96, 0xabe, 0xab2,
0xabf, 0xaa8, 0xab8, 0xaae, 0xab0, 0xa95, 0xaa8, 0xacd, 0xaa1, 0xab8, 0xac0, 0xa93,
0xab2, 0xab6, 0xabe, 0xa82, 0xa98, 0xabe, 0xa88, 0xab8, 0xabf, 0xa82, 0xa97, 0xabe,
0xaaa, 0xac1, 0xab0, 0xab8, 0xacd, 0xab0, 0xac7, 0xaa1, 0xacd, 0xaa8, 0xac7, 0xa95,
0xacb, 0xab2, 0xac0, 0xaae, 0xacd, 0xab8, 0xacd, 0xa95, 0xaa4, 0xabe, 0xa87, 0xaaa,
0xac7, 0xa87, 0xaa4, 0xabe, 0xab6, 0xa95, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xaac, 0xabf,
0xab2, 0xabf, 0xab8, 0xabf, 0xaa4, 0xac7, 0xab9, 0xab0, 0xabe, 0xaa8, 0xaa5, 0xac0,
0xaae, 0xacd, 0xaab, 0xac1, 0xa9f, 0xacb, 0xa95, 0xacd, 0xaaf, 0xacb, 0xaa4, 0xacb,
0xaae, 0xab8, 0xacd, 0xa95, 0xa89, 0xab2, 0xabe, 0xa82, 0xaac, 0xabe, 0xaa4, 0xab0,
0xa89, 0xab0, 0xac1, 0xaae, 0xacd, 0xa95, 0xac0, 0xa89, 0xab8, 0xacd, 0xaa4, 0x2d,
0xaa8, 0xac7, 0xab0, 0xabe, 0xab5, 0xabf, 0xaaf, 0xac7, 0xaa8, 0xa9f, 0xabf, 0xaaf,
0xac7, 0xaa8, 0xab5, 0xacd, 0xab2, 0xac7, 0xaa1, 0xabf, 0xab5, 0xacb, 0xab8, 0xacd,
0xa9f, 0xacb, 0xa95, 0xaaf, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xa95, 0xab0, 0xa82,
0xa97, 0xac2, 0xaa8, 0xaaf, 0xac7, 0xa95, 0xabe, 0xaa4, 0xac7, 0xab0, 0xabf, 0xaa8,
0xaac, 0xab0, 0xacd, 0xa97, 0xaaf, 0xac7, 0xab0, 0xac7, 0xab5, 0xabe, 0xaa8, 0xa8f,
0xa9d, 0xacb, 0xab0, 0xac7, 0xa9d, 0xaac, 0xab0, 0xacd, 0xaae, 0xac1, 0xaa1, 0xabe,
0xa95, 0xac5, 0xaa8, 0xac7, 0xab0, 0xac0, 0xa95, 0xac5, 0xaaa, 0x20, 0xab5, 0xab0,
0xacd, 0xaa1, 0xac7, 0xaab, 0xac7, 0xab0, 0xacb, 0xaae, 0xaa1, 0xabf, 0xab0, 0xabe,
0xab0, 0xac5, 0xa95, 0xaaf, 0xabe, 0xab5, 0xabf, 0xa95, 0xaa6, 0xa95, 0xacd, 0xab7,
0xabf, 0xaa3, 0x20, 0xa9c, 0xacd, 0xaaf, 0xacb, 0xab0, 0xacd, 0xa9c, 0xabf, 0xaaf,
0xabe, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xab9, 0xac7, 0xab2, 0xac7, 0xaa8, 0xabe,
0xab8, 0xacd, 0xa9f, 0xac7, 0xaa8, 0xab2, 0xac0, 0xa8f, 0xaa1, 0xabf, 0xab2, 0xac7,
0xa87, 0xaa1, 0xaac, 0xacd, 0xab0, 0xabf, 0xab8, 0xaac, 0xac7, 0xaa8, 0xaac, 0xacd,
0xab0, 0xacb, 0xa95, 0xaa8, 0x20, 0xab9, 0xabf, 0xab2, 0xaa1, 0xabe, 0xab0, 0xacd,
0xab5, 0xabf, 0xaa8, 0xa89, 0xa95, 0xacd, 0xab2, 0xabe, 0xab9, 0xacb, 0xaac, 0xabe,
0xab0, 0xacd, 0xa9f, 0xab2, 0xabf, 0xaa8, 0xacd, 0xaa1, 0xac7, 0xaae, 0xac7, 0xaa8,
0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0x20, 0xab9, 0xacb, 0xab5, 0xaae, 0xac7, 0xab2,
0xaac, 0xacb, 0xab0, 0xacd, 0xaa8, 0xaaa, 0xab0, 0xacd, 0xaa5, 0xab8, 0xabf, 0xaa1,
0xaa8, 0xac0, 0xa8d, 0xaae, 0xacd, 0xab8, 0xacd, 0xa9f, 0xab0, 0xaa1, 0xac5, 0xaae,
0xa8d, 0xa82, 0xaa1, 0xacb, 0xab0, 0xabe, 0xa86, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0,
0xa96, 0xaa8, 0xa8f, 0xaa5, 0xac7, 0xa82, 0xab8, 0xaac, 0xac7, 0xab2, 0xa97, 0xacd,
0xab0, 0xac7, 0xaa1, 0xaac, 0xab0, 0xacd, 0xab2, 0xabf, 0xaa8, 0xaac, 0xacd, 0xab0,
0xac7, 0xa9f, 0xabf, 0xab8, 0xab2, 0xabe, 0xab5, 0xabe, 0xaac, 0xacd, 0xab0, 0xab8,
0xac7, 0xab2, 0xacd, 0xab8, 0xaac, 0xac1, 0xa95, 0xab0, 0xac7, 0xab8, 0xacd, 0xa9f,
0xaac, 0xac1, 0xaa1, 0xabe, 0xaaa, 0xac7, 0xab8, 0xacd, 0xa9f, 0xaac, 0xabf, 0xa9d,
0xac7, 0xaa8, 0xacd, 0xa97, 0xaa8, 0xa9a, 0xabf, 0xab8, 0xac0, 0xaa8, 0xabe, 0xa89,
0xa95, 0xacb, 0xaaa, 0xaa8, 0xab9, 0xac7, 0xa97, 0xaa8, 0xaa1, 0xaac, 0xab2, 0xabf,
0xaa8, 0xa9c, 0xabf, 0xaac, 0xacd, 0xab0, 0xabe, 0xab2, 0xacd, 0xa9f, 0xab0, 0xa97,
0xab0, 0xacd, 0xaa8, 0xa9d, 0xac0, 0xab9, 0xac7, 0xab2, 0xab8, 0xabf, 0xa82, 0xa95,
0xac0, 0xa86, 0xa87, 0xab2, 0x20, 0xa91, 0xaab, 0x20, 0xaae, 0xac7, 0xaa8, 0xa87,
0xab8, 0xacd, 0xaa4, 0xabe, 0xa82, 0xaac, 0xac1, 0xab2, 0xa9c, 0xab0, 0xacd, 0xab8,
0xac0, 0xa95, 0xab2, 0xabf, 0xaa8, 0xabf, 0xaa8, 0xacd, 0xa97, 0xacd, 0xab0, 0xac7,
0xaa1, 0xa95, 0xac0, 0xab0, 0xacb, 0xab5, 0xa95, 0xac8, 0xab5, 0xab2, 0xabf, 0xab8,
0xacd, 0xaac, 0xaa8, 0xab2, 0xac1, 0xaac, 0xacd, 0xab2, 0xabf, 0xaaf, 0xabe, 0xaa8,
0xabe, 0xab2, 0xa82, 0xaa1, 0xaa8, 0xab2, 0xa95, 0xacd, 0xa9d, 0xaae, 0xaac, 0xab0,
0xacd, 0xa97, 0xaae, 0xac7, 0xaa1, 0xacd, 0xab0, 0xabf, 0xaa1, 0xaae, 0xabe, 0xab2,
0xacd, 0xa9f, 0xabe, 0xaae, 0xac7, 0xab0, 0xabf, 0xab9, 0xac7, 0xaae, 0xaae, 0xabf,
0xa82, 0xab8, 0xacd, 0xa95, 0xaae, 0xacb, 0xaa8, 0xabe, 0xa95, 0xacb, 0xaae, 0xacb,
0xab8, 0xacd, 0xa95, 0xacb, 0xa91, 0xab8, 0xacd, 0xab2, 0xacb, 0xaaa, 0xac7, 0xab0,
0xabf, 0xab8, 0xaaa, 0xacb, 0xaa1, 0xa97, 0xacb, 0xab0, 0xabf, 0xa95, 0xabe, 0xaaa,
0xacd, 0xab0, 0xabe, 0xa97, 0xab0, 0xabf, 0xa97, 0xabe, 0xab8, 0xaae, 0xabe, 0xab0,
0xabe, 0xab8, 0xac5, 0xaa8, 0x20, 0xaae, 0xac7, 0xab0, 0xabf, 0xaa8, 0xacb, 0xab8,
0xabe, 0xab0, 0xabe, 0xa9c, 0xac7, 0xab5, 0xacb, 0xab8, 0xabe, 0xab0, 0xabe, 0xa9f,
0xacb, 0xab5, 0xab8, 0xabf, 0xaae, 0xacd, 0xaab, 0xac7, 0xab0, 0xacb, 0xaaa, 0xacb,
0xab2, 0xab8, 0xacd, 0xa95, 0xacb, 0xaaa, 0xaaf, 0xac7, 0xab8, 0xacb, 0xaab, 0xabf,
0xaaf, 0xabe, 0xab8, 0xacd, 0xa9f, 0xacb, 0xa95, 0xab9, 0xac9, 0xaae, 0xaa4, 0xab2,
0xacd, 0xab2, 0xac0, 0xaa8, 0xaa4, 0xabf, 0xab0, 0xabe, 0xaa8, 0xac7, 0xa89, 0xab2,
0xac7, 0xaa8, 0xacb, 0xab5, 0xacd, 0xab8, 0xacd, 0xa95, 0xab5, 0xac5, 0xaa6, 0xac1,
0xa9d, 0xab5, 0xac7, 0xa9f, 0xabf, 0xa95, 0xaa8, 0xab5, 0xabf, 0xaaf, 0xac7, 0xaa8,
0xabe, 0xab5, 0xabf, 0xab2, 0xacd, 0xaa8, 0xabf, 0xa85, 0xab8, 0xab5, 0xacb, 0xab2,
0xacd, 0xa97, 0xacb, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1, 0xab5, 0xacb, 0xab0, 0xab8,
0xac9, 0xa9d, 0xac7, 0xa97, 0xab0, 0xac7, 0xaac, 0xa9d, 0xac1, 0xab0, 0xac0, 0xa95,
0xa85, 0xa82, 0xaa4, 0xabe, 0xaa8, 0xabe, 0xaa8, 0xabe, 0xab0, 0xabf, 0xab5, 0xacb,
0xa9a, 0xabe, 0xa97, 0xacb, 0xab8, 0xa95, 0xacd, 0xab0, 0xabf, 0xab8, 0xaae, 0xab8,
0xa95, 0xacb, 0xa95, 0xacb, 0xa9d, 0xa95, 0xacb, 0xaae, 0xacb, 0xab0, 0xacb, 0xa95,
0xac7, 0xab0, 0xa97, 0xacd, 0xab5, 0xac7, 0xab2, 0xabf, 0xaa8, 0xaae, 0xabe, 0xab9,
0xac7, 0xaae, 0xabe, 0xab2, 0xaa6, 0xabf, 0xab5, 0xacd, 0xab8, 0xaae, 0xacb, 0xab0,
0xabf, 0xab6, 0xabf, 0xaaf, 0xab8, 0xaae, 0xac7, 0xaaf, 0xacb, 0xa9f, 0xab0, 0xac0,
0xaaf, 0xac1, 0xaa8, 0xabf, 0xaaf, 0xaa8, 0xa8f, 0xaaa, 0xabf, 0xaaf, 0xabe, 0xa91,
0xa95, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xaac, 0xacc, 0xa97, 0xac7, 0xa88, 0xaa8,
0xab5, 0xabf, 0xab2, 0xac7, 0xa9a, 0xaa5, 0xaae, 0xa9a, 0xac1, 0xa95, 0xa87, 0xab8,
0xacd, 0xa9f, 0xab0, 0xa8f, 0xaab, 0xac7, 0xa9f, 0xaab, 0xabe, 0xa95, 0xabe, 0xa93,
0xaab, 0xacb, 0xaab, 0xac0, 0xa9c, 0xac0, 0xaab, 0xac1, 0xaa8, 0xabe, 0xaab, 0xac1,
0xa9f, 0xac0, 0xa97, 0xabe, 0xab2, 0xabe, 0xaaa, 0xac7, 0xa97, 0xacb, 0xab8, 0xa97,
0xac7, 0xaae, 0xacd, 0xaac, 0xabf, 0xaaf, 0xab0, 0xa97, 0xacc, 0xaa1, 0xab2, 0xa95,
0xac5, 0xaa8, 0xab2, 0xa97, 0xacd, 0xab5, 0xabe, 0xaae, 0xab9, 0xacb, 0xaa8, 0xacb,
0xab2, 0xac1, 0xab2, 0xac1, 0xa8f, 0xa82, 0xaa1, 0xab0, 0xacd, 0xaac, 0xab0, 0xac0,
0xa95, 0xabf, 0xab0, 0xabf, 0xaa4, 0xabf, 0xaae, 0xabe, 0xaa4, 0xac0, 0xa95, 0xacb,
0xab8, 0xab0, 0xac7, 0xa95, 0xacd, 0xab5, 0xabe, 0xa9c, 0xabe, 0xab2, 0xac0, 0xaa8,
0xaae, 0xabe, 0xa9c, 0xac1, 0xab0, 0xacb, 0xaae, 0xabe, 0xab0, 0xacd, 0xa95, 0xac0,
0xab8, 0xabe, 0xab8, 0xaae, 0xac0, 0xaa1, 0xab5, 0xac7, 0xaa8, 0xacc, 0xab0, 0xac1,
0xaa8, 0xac0, 0xaaf, 0xac1, 0xaa8, 0xac9, 0xab0, 0xaab, 0xac9, 0xa95, 0xaa8, 0xacc,
0xaae, 0xac0, 0xa86, 0xaaa, 0xac7, 0xa97, 0xacb, 0x20, 0xaaa, 0xac7, 0xa97, 0xacb,
0xaaa, 0xab2, 0xabe, 0xa89, 0xaaa, 0xac0, 0xa9f, 0xa95, 0xac8, 0xab0, 0xacd, 0xaa8,
0xaaa, 0xacb, 0xaa8, 0xacd, 0xaaa, 0xac7, 0xaaa, 0xacb, 0xab0, 0xacd, 0xa9f, 0x20,
0xaae, 0xacb, 0xab0, 0xacd, 0xab8, 0xacd, 0xaac, 0xac0, 0xab0, 0xac7, 0xab0, 0xacb,
0xa9f, 0xacb, 0xa82, 0xa97, 0xabe, 0xab8, 0xabe, 0xa87, 0xaaa, 0xaa8, 0xaa4, 0xabe,
0xab9, 0xac0, 0xaa4, 0xac0, 0xaa4, 0xabe, 0xab0, 0xabe, 0xab5, 0xabe, 0xa9f, 0xac9,
0xa82, 0xa97, 0xabe, 0xa9f, 0xabe, 0xaaa, 0xac1, 0xab5, 0xac7, 0xa95, 0xab5, 0xabe,
0xab2, 0xabf, 0xab8, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x41,
0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x41, 0x72,
0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20,
0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x5d0, 0x5d1, 0x5d9, 0x5d2, 0x5f3, 0x5d0, 0x5df, 0x5d0, 0x5e7,
0x5e8, 0x5d4, 0x5d0, 0x5d3, 0x5d9, 0x5e1, 0x20, 0x5d0, 0x5d1, 0x5d1, 0x5d4, 0x5d0,
0x5dc, 0x5d2, 0x5f3, 0x5d9, 0x5e8, 0x5d0, 0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5d1, 0x5de,
0x5d0, 0x5e7, 0x5d5, 0x5d1, 0x5e0, 0x5d2, 0x5d5, 0x5d0, 0x5d9, 0x5d1, 0x5e0, 0x5d2,
0x5f3, 0x5d5, 0x5dc, 0x5d1, 0x5d9, 0x5e1, 0x5d0, 0x5d5, 0x5d1, 0x5dc, 0x5e0, 0x5d8,
0x5d9, 0x5e8, 0x5d1, 0x5e8, 0x5d6, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5d1, 0x5d5, 0x5d2,
0x5f3, 0x5d5, 0x5de, 0x5d1, 0x5d5, 0x5e8, 0x5d4, 0x5e7, 0x5d4, 0x5d9, 0x5e8, 0x5e7,
0x5d6, 0x5d1, 0x5dc, 0x5e0, 0x5e7, 0x5d4, 0x5e1, 0x5d0, 0x5d5, 0x5d8, 0x5d4, 0x5e7,
0x5d5, 0x5e0, 0x5d0, 0x5e7, 0x5e8, 0x5d9, 0x5d3, 0x5e7, 0x5d0, 0x5e8, 0x5d3, 0x5d0,
0x5e8, 0x20, 0x5d0, 0x2d, 0x5e1, 0x5dc, 0x5d0, 0x5dd, 0x5d2, 0x5f3, 0x5d9, 0x5d1,
0x5d5, 0x5d8, 0x5d9, 0x5d3, 0x5d5, 0x5d0, 0x5dc, 0x5d4, 0x5d0, 0x5dc, 0x20, 0x5e2,
0x5d9, 0x5d5, 0x5df, 0x5e4, 0x5e8, 0x5d9, 0x5d8, 0x5d0, 0x5d5, 0x5df, 0x5d2, 0x5d1,
0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d4, 0x5d4, 0x5e8, 0x5d0, 0x5e8, 0x5d4, 0x5d9, 0x5d5,
0x5d4, 0x5e0, 0x5e1, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x5d2, 0x5f3, 0x5d5, 0x5d1, 0x5d4,
0x5e7, 0x5de, 0x5e4, 0x5d0, 0x5dc, 0x5d4, 0x5d7, 0x5e8, 0x5d8, 0x5d5, 0x5dd, 0x5e7,
0x5d9, 0x5d2, 0x5dc, 0x5d9, 0x5e7, 0x5d9, 0x5e0, 0x5e9, 0x5e1, 0x5d4, 0x5dc, 0x5d0,
0x5d2, 0x5d5, 0x5e1, 0x5dc, 0x5d9, 0x5d1, 0x5e8, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5dc,
0x5d5, 0x5de, 0x5d4, 0x5dc, 0x5d5, 0x5d0, 0x5e0, 0x5d3, 0x5d4, 0x5dc, 0x5d5, 0x5d1,
0x5d5, 0x5de, 0x5d1, 0x5d0, 0x5e9, 0x5d9, 0x5dc, 0x5d5, 0x5e1, 0x5e7, 0x5d4, 0x5de,
0x5dc, 0x5d1, 0x5d5, 0x5de, 0x5d0, 0x5e4, 0x5d5, 0x5d8, 0x5d5, 0x5de, 0x5e1, 0x5e8,
0x5d5, 0x5de, 0x5d1, 0x5d1, 0x5e0, 0x5d4, 0x5de, 0x5d5, 0x5d2, 0x5d3, 0x5d9, 0x5e9,
0x5d5, 0x5de, 0x5d5, 0x5e0, 0x5e8, 0x5d5, 0x5d1, 0x5d9, 0x5d4, 0x5e0, 0x5d9, 0x5d9,
0x5e8, 0x5d5, 0x5d1, 0x5d9, 0x5e0, 0x5d2, 0x5f3, 0x5de, 0x5e0, 0x5d4, 0x5e0, 0x5d9,
0x5d0, 0x5de, 0x5d9, 0x5d9, 0x5e0, 0x5d5, 0x5d0, 0x5e7, 0x5e6, 0x5f3, 0x5d5, 0x5d8,
0x5d5, 0x5d5, 0x5d0, 0x5d2, 0x5d0, 0x5d3, 0x5d5, 0x5d2, 0x5d5, 0x5e4, 0x5d5, 0x5e8,
0x5d8, 0x5d5, 0x20, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e1, 0x5d0, 0x5d5, 0x20, 0x5d8,
0x5d5, 0x5de, 0x5d4, 0x5d8, 0x5e8, 0x5d9, 0x5e4, 0x5d5, 0x5dc, 0x5d9, 0x5ea, 0x5d5,
0x5e0, 0x5d9, 0x5e1, 0x5d5, 0x5d9, 0x5e0, 0x5d3, 0x5d4, 0x5d5, 0x5e7, 0x5d0, 0x5d3,
0x5d0, 0x5e7, 0x5d0, 0x5e0, 0x5e7, 0x5d5, 0x5e8, 0x5d2, 0x5f3, 0x5d0, 0x5e0, 0x5d2,
0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5d4, 0x5d0, 0x5e0, 0x5d8, 0x5d9, 0x5d2, 0x5d5, 0x5d0,
0x5d4, 0x5d0, 0x5e8, 0x5d0, 0x5d2, 0x5d5, 0x5d0, 0x5d9, 0x5e0, 0x5d4, 0x5d1, 0x5d5,
0x5d0, 0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x5d8,
0x5de, 0x5e8, 0x5e7, 0x5d4, 0x5e7, 0x5d5, 0x5e8, 0x5d3, 0x5d5, 0x5d1, 0x5d4, 0x5d7,
0x5d5, 0x5d7, 0x5d5, 0x5d9, 0x5dc, 0x5d4, 0x20, 0x5e8, 0x5d9, 0x5d5, 0x5d7, 0x5d4,
0x5de, 0x5e0, 0x5d3, 0x5d5, 0x5d6, 0x5d4, 0x5e8, 0x5d9, 0x5d5, 0x20, 0x5d2, 0x5d0,
0x5d9, 0x5d9, 0x5d2, 0x5d5, 0x5e1, 0x5e1, 0x5dc, 0x5d8, 0x5d4, 0x5e1, 0x5df, 0x20,
0x5d7, 0x5d5, 0x5d0, 0x5df, 0x5e1, 0x5df, 0x20, 0x5dc, 0x5d5, 0x5d0, 0x5d9, 0x5e1,
0x5d8, 0x5d5, 0x5e7, 0x5d5, 0x5de, 0x5df, 0x5d0, 0x5d5, 0x5e9, 0x5d5, 0x5d5, 0x5d0,
0x5d9, 0x5d4, 0x5d0, 0x5e8, 0x5d5, 0x5d1, 0x5d4, 0x5d0, 0x5e1, 0x5d5, 0x5e0, 0x5e1,
0x5d9, 0x5d5, 0x5df, 0x5d0, 0x5d8, 0x5d9, 0x5e7, 0x5d5, 0x5e7, 0x5df, 0x5d1, 0x5d0,
0x5d4, 0x5d9, 0x5d4, 0x5d1, 0x5d0, 0x5d4, 0x5d9, 0x5d4, 0x20, 0x5d1, 0x5e0, 0x5d3,
0x5e8, 0x5e1, 0x5d1, 0x5e8, 0x5d1, 0x5d3, 0x5d5, 0x5e1, 0x5d1, 0x5dc, 0x5dd, 0x5d1,
0x5dc, 0x5d9, 0x5d6, 0x5d1, 0x5dc, 0x5d0, 0x5df, 0x2d, 0x5e1, 0x5d1, 0x5dc, 0x5d5,
0x5df, 0x5d1, 0x5d5, 0x5d0, 0x5d4, 0x20, 0x5d5, 0x5d9, 0x5e1, 0x5d8, 0x5d4, 0x5d1,
0x5d5, 0x5d2, 0x5d5, 0x5d8, 0x5d4, 0x5d1, 0x5d5, 0x5d9, 0x5e1, 0x5d9, 0x5e7, 0x5d9,
0x5d9, 0x5de, 0x5d1, 0x5e8, 0x5d9, 0x5d3, 0x5d2, 0x5f3, 0x20, 0x5d1, 0x5d9, 0x5d9,
0x5e7, 0x5de, 0x5e4, 0x5d5, 0x20, 0x5d2, 0x5e8, 0x5e0, 0x5d3, 0x5d4, 0x5e7, 0x5e0,
0x5e7, 0x5d5, 0x5df, 0x5e7, 0x5e8, 0x5d0, 0x5e7, 0x5e1, 0x5e7, 0x5d0, 0x5d9, 0x5d9,
0x5df, 0x5e7, 0x5d9, 0x5d9, 0x5de, 0x5df, 0x5e9, 0x5d9, 0x5e7, 0x5d2, 0x5d5, 0x5e6,
0x5f3, 0x5d9, 0x5d5, 0x5d5, 0x5d0, 0x5d5, 0x5d5, 0x5d4, 0x5e1, 0x5d9, 0x5d5, 0x5d3,
0x5d3, 0x20, 0x5d7, 0x5d5, 0x5d0, 0x5e8, 0x5e1, 0x5e7, 0x5d5, 0x5e1, 0x5d8, 0x5d4,
0x20, 0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x5e7, 0x5e8, 0x5e1, 0x5d8, 0x5d5, 0x5df, 0x5e7,
0x5d5, 0x5d9, 0x5d0, 0x5d1, 0x5d4, 0x5e7, 0x5d5, 0x5e8, 0x5d0, 0x5e1, 0x5d0, 0x5d5,
0x5d3, 0x5e0, 0x5de, 0x5e8, 0x5e7, 0x5e1, 0x5d4, 0x5d5, 0x5d5, 0x5df, 0x5d3, 0x5d5,
0x5e1, 0x5d5, 0x5df, 0x5d3, 0x5d5, 0x5e1, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5e8, 0x5d9,
0x5e7, 0x5d3, 0x5e0, 0x5d5, 0x5d5, 0x5e8, 0x5d3, 0x5d8, 0x5e8, 0x5d5, 0x5d9, 0x5d8,
0x5d3, 0x5d5, 0x5de, 0x5d9, 0x5e0, 0x5d9, 0x5e7, 0x5d4, 0x5d0, 0x5d3, 0x5de, 0x5d5,
0x5e0, 0x5d8, 0x5d5, 0x5df, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e0, 0x5e4, 0x5d9, 0x5d0,
0x5dc, 0x20, 0x5e1, 0x5dc, 0x5d1, 0x5d3, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e8, 0x5d8,
0x20, 0x5e0, 0x5dc, 0x5e1, 0x5d5, 0x5df, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x5d0, 0x5dc,
0x5d6, 0x5d4, 0x5d2, 0x5dc, 0x5d9, 0x5d9, 0x5e1, 0x20, 0x5d1, 0x5d9, 0x5d9, 0x5d2,
0x5d5, 0x5e1, 0x20, 0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5e8, 0x5e0, 0x5d3, 0x20, 0x5d8,
0x5d5, 0x5e8, 0x5e7, 0x5d2, 0x5d5, 0x5d0, 0x5d3, 0x5dc, 0x5d5, 0x5e4, 0x5d2, 0x5d5,
0x5d0, 0x5d8, 0x5de, 0x5dc, 0x5d4, 0x5d2, 0x5d5, 0x5d0, 0x5d9, 0x5d0, 0x5e7, 0x5d9,
0x5dc, 0x5d2, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d4, 0x5dc, 0x5d9, 0x5e4, 0x5e7, 0x5e1,
0x5d4, 0x5d5, 0x5d5, 0x5d0, 0x5e0, 0x5d4, 0x5d4, 0x5e8, 0x5de, 0x5d5, 0x5e1, 0x5d9,
0x5d5, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5e4, 0x5d5, 0x5dc, 0x5d9,
0x5e1, 0x5e0, 0x5d5, 0x5e7, 0x5e1, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9,
0x5d0, 0x5e0, 0x5d4, 0x5de, 0x5e8, 0x5e0, 0x5d2, 0x5d5, 0x2c, 0x20, 0x5d0, 0x5d9,
0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5e4, 0x5d9, 0x5d8, 0x5e8, 0x5e1, 0x5d1,
0x5d5, 0x5e8, 0x5d2, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0,
0x5d4, 0x5d8, 0x5dc, 0x20, 0x5e1, 0x5d9, 0x5d8, 0x5d9, 0x2c, 0x20, 0x5d0, 0x5d9,
0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d5, 0x5d9, 0x5d5, 0x5d5, 0x5d0, 0x5d9,
0x5d9, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d5,
0x5d9, 0x5e0, 0x5e1, 0x5e0, 0x5e1, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9,
0x5d0, 0x5e0, 0x5d4, 0x5d5, 0x5d9, 0x5e0, 0x5de, 0x5d0, 0x5e7, 0x2c, 0x20, 0x5d0,
0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d0, 0x5d9, 0x5e0, 0x5d5, 0x5d5,
0x5d9, 0x5e7, 0x5d0, 0x5d9, 0x5e7, 0x5dc, 0x5d5, 0x5d0, 0x5d9, 0x5d8, 0x5d2, 0x5f3,
0x5de, 0x5d9, 0x5d9, 0x5e7, 0x5d4, 0x5d2, 0x5f3, 0x5d5, 0x5e0, 0x5d5, 0x5dc, 0x5d5,
0x5d0, 0x5d9, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5de, 0x5d5, 0x5e0, 0x5d8, 0x5d9, 0x5e6,
0x5f3, 0x5dc, 0x5d5, 0x2c, 0x20, 0x5e7, 0x5e0, 0x5d8, 0x5d0, 0x5e7, 0x5d9, 0x5e7,
0x5e8, 0x5dc, 0x5e0, 0x5d3, 0x5d9, 0x5d9, 0x5e7, 0x5dc, 0x5d4, 0x20, 0x5e4, 0x5d0,
0x5e1, 0x5dc, 0x5d9, 0x5de, 0x5d4, 0x5dc, 0x5d5, 0x5e1, 0x20, 0x5d0, 0x5e0, 0x5d2,
0x5f3, 0x5dc, 0x5e1, 0x5dc, 0x5d5, 0x5d0, 0x5d5, 0x5d5, 0x5e8, 0x20, 0x5e4, 0x5e8,
0x5d9, 0x5e0, 0x5e1, 0x5e1, 0x20, 0x5e7, 0x5d5, 0x5d5, 0x5e8, 0x5d8, 0x5e8, 0x5de,
0x5e1, 0x5d9, 0x5d9, 0x5d0, 0x5d5, 0x5de, 0x5e0, 0x5d2, 0x5d5, 0x5d0, 0x5d4, 0x5de,
0x5e0, 0x5d0, 0x5d5, 0x5e1, 0x5de, 0x5e8, 0x5d9, 0x5d2, 0x5d5, 0x5de, 0x5e8, 0x5d8,
0x5d9, 0x5e0, 0x5d9, 0x5e7, 0x5de, 0x5d8, 0x5de, 0x5d5, 0x5e8, 0x5d5, 0x5e1, 0x5de,
0x5d6, 0x5d8, 0x5dc, 0x5df, 0x5de, 0x5e0, 0x5d5, 0x5de, 0x5d9, 0x5e0, 0x5d9, 0x5de,
0x5e8, 0x5d9, 0x5d3, 0x5d4, 0x5de, 0x5d8, 0x5dc, 0x5e7, 0x5d8, 0x5dc, 0x5d4, 0x5de,
0x5e7, 0x5e1, 0x5d9, 0x5e7, 0x5d5, 0x20, 0x5e1, 0x5d9, 0x5d8, 0x5d9, 0x5de, 0x5d9,
0x5e7, 0x5dc, 0x5d5, 0x5df, 0x5de, 0x5d5, 0x5e0, 0x5e7, 0x5d8, 0x5d5, 0x5df, 0x5de,
0x5d5, 0x5e0, 0x5d8, 0x5e8, 0x5d9, 0x5d9, 0x5de, 0x5d5, 0x5e0, 0x5d8, 0x5d5, 0x5d5,
0x5d9, 0x5d3, 0x5d0, 0x5d5, 0x5de, 0x5d5, 0x5e0, 0x5e1, 0x5e8, 0x5d0, 0x5d8, 0x5e0,
0x5e1, 0x5d0, 0x5d5, 0x5e0, 0x5d9, 0x5d5, 0x20, 0x5d9, 0x5d5, 0x5e8, 0x5e7, 0x5e0,
0x5d5, 0x5dd, 0x5e0, 0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d4, 0x5d1, 0x5d9, 0x5d5,
0x5dc, 0x5d4, 0x2c, 0x20, 0x5d3, 0x5e7, 0x5d5, 0x5d8, 0x5d4, 0x20, 0x5d4, 0x5e6,
0x5e4, 0x5d5, 0x5e0, 0x5d9, 0x5ea, 0x5e1, 0x5e0, 0x5d8, 0x5e8, 0x2c, 0x20, 0x5d3,
0x5e7, 0x5d5, 0x5d8, 0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e4, 0x5d5, 0x5e0, 0x5d9, 0x5ea,
0x5e0, 0x5d9, 0x5d5, 0x20, 0x5e1, 0x5d9, 0x5d9, 0x5dc, 0x5dd, 0x2c, 0x20, 0x5d3,
0x5e7, 0x5d5, 0x5d8, 0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e4, 0x5d5, 0x5e0, 0x5d9, 0x5ea,
0x5e0, 0x5d5, 0x5d0, 0x5d5, 0x5e7, 0x5d0, 0x5d5, 0x5d2, 0x5f3, 0x5d9, 0x5e0, 0x5d0,
0x5d2, 0x5d4, 0x5e4, 0x5e0, 0x5de, 0x5d4, 0x5e4, 0x5e8, 0x5de, 0x5e8, 0x5d9, 0x5d1,
0x5d5, 0x5e4, 0x5d9, 0x5e0, 0x5d9, 0x5e7, 0x5e1, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x20,
0x5d0, 0x5d5, 0x20, 0x5e4, 0x5e8, 0x5d0, 0x5e0, 0x5e1, 0x5e4, 0x5d5, 0x5e8, 0x5d8,
0x20, 0x5d0, 0x5d5, 0x5e3, 0x20, 0x5e1, 0x5e4, 0x5d9, 0x5d9, 0x5df, 0x5e4, 0x5d5,
0x5e8, 0x5d8, 0x5d5, 0x20, 0x5d5, 0x5dc, 0x5d9, 0x5d5, 0x5e4, 0x5d5, 0x5d0, 0x5e8,
0x5d8, 0x5d5, 0x20, 0x5e8, 0x5d9, 0x5e7, 0x5d5, 0x5e4, 0x5d5, 0x5e0, 0x5d8, 0x5d4,
0x20, 0x5d0, 0x5e8, 0x5e0, 0x5e1, 0x5e8, 0x5e0, 0x5e7, 0x5d9, 0x5df, 0x20, 0x5d0,
0x5d9, 0x5e0, 0x5dc, 0x5d8, 0x5e8, 0x5e1, 0x5d9, 0x5e4, 0x5d4, 0x5e8, 0x5d2, 0x5f3,
0x5d9, 0x5e0, 0x5d4, 0x5e8, 0x5d6, 0x5d5, 0x5dc, 0x5d5, 0x5d8, 0x5e8, 0x5d9, 0x5d5,
0x20, 0x5d1, 0x5e8, 0x5e0, 0x5e7, 0x5d5, 0x5e1, 0x5e0, 0x5d8, 0x5e8, 0x5dd, 0x5e1,
0x5e0, 0x5d8, 0x5d9, 0x5d0, 0x5d2, 0x5d5, 0x5e1, 0x5e0, 0x5d8, 0x5d5, 0x20, 0x5d3,
0x5d5, 0x5de, 0x5d9, 0x5e0, 0x5d2, 0x5d5, 0x5e1, 0x5d0, 0x5d5, 0x20, 0x5e4, 0x5d0,
0x5d5, 0x5dc, 0x5d5, 0x5e1, 0x5e7, 0x5d5, 0x5e8, 0x5e1, 0x5d1, 0x5d9, 0x5e1, 0x5d5,
0x5e0, 0x5d3, 0x5e1, 0x5d9, 0x5d8, 0x5e7, 0x5d4, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5d1,
0x5e8, 0x5ea, 0x5dc, 0x5de, 0x5d9, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5d2, 0x5f3, 0x5d5,
0x5e0, 0x5e1, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5e7, 0x5d9, 0x5d8, 0x5e1, 0x5e1, 0x5e0,
0x5d8, 0x20, 0x5dc, 0x5d5, 0x5e1, 0x5d9, 0x5d4, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5ea,
0x5d5, 0x5de, 0x5d0, 0x5e1, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5d5, 0x5d9, 0x5e0, 0x5e1,
0x5e0, 0x5d8, 0x5e1, 0x5d5, 0x5d5, 0x5d9, 0x5e4, 0x5d8, 0x20, 0x5e7, 0x5e8, 0x5e0,
0x5d8, 0x5d8, 0x5d2, 0x5d5, 0x5e1, 0x5d9, 0x5d2, 0x5dc, 0x5e4, 0x5d4, 0x5ea, 0x5d5,
0x5dc, 0x5d4, 0x5d8, 0x5d9, 0x5d7, 0x5d5, 0x5d0, 0x5e0, 0x5d4, 0x5d8, 0x5d5, 0x5e8,
0x5d5, 0x5e0, 0x5d8, 0x5d5, 0x5d8, 0x5d5, 0x5e8, 0x5d8, 0x5d5, 0x5dc, 0x5d4, 0x5d5,
0x5e0, 0x5e7, 0x5d5, 0x5d1, 0x5e8, 0x5d5, 0x5d5, 0x5d9, 0x5d9, 0x5d8, 0x5d4, 0x5d5,
0x5e8, 0x5e1, 0x5d5, 0x5d5, 0x5d9, 0x5e0, 0x5d9, 0x5e4, 0x5d2, 0x5d9, 0x5e7, 0x5d5,
0x5d8, 0x5d0, 0x5d8, 0x5e7, 0x5d9, 0x5d9, 0x5e1, 0x5d9, 0x5d3, 0x5d9, 0x5d5, 0x5d5,
0x5d9, 0x5e1, 0x5d3, 0x5d5, 0x5de, 0x5d5, 0x5df, 0x20, 0x5d3, 0x5f3, 0x5d0, 0x5d5,
0x5e8, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5de, 0x5e7, 0x5d5, 0x5d5, 0x5d0, 0x5e8, 0x5d9,
0x5de, 0x5d5, 0x5e1, 0x5d5, 0x5df, 0x5de, 0x5e7, 0x2d, 0x5de, 0x5e8, 0x5d3, 0x5d5,
0x5e4, 0x5d0, 0x5dc, 0x5de, 0x5e8, 0x5e8, 0x5d5, 0x5ea, 0x5f3, 0x5e8, 0x5d4, 0x5e1,
0x5d9, 0x5d9, 0x5d5, 0x5d5, 0x5d4, 0x5d8, 0x5e8, 0x5d5, 0x5dc, 0x5d5, 0x5d5, 0x5e1,
0x5d8, 0x5d5, 0x5e7, 0x5dc, 0x5d5, 0x5e0, 0x5d2, 0x5d9, 0x5d9, 0x5e8, 0x5d1, 0x5d9,
0x5df, 0x5e2, 0x5d3, 0x5df, 0x5d0, 0x5dc, 0x5de, 0x5d0, 0x5d8, 0x5d9, 0x5e2, 0x5de,
0x5d0, 0x5df, 0x5d0, 0x5e0, 0x5d3, 0x5d9, 0x5e8, 0x5d0, 0x5e7, 0x5d8, 0x5d0, 0x5d5,
0x5d0, 0x5e7, 0x5d8, 0x5d5, 0x5d1, 0x5d4, 0x5d0, 0x5e9, 0x5d2, 0x5d1, 0x5d8, 0x5d0,
0x5d8, 0x5d9, 0x5e8, 0x5d0, 0x5d5, 0x5d1, 0x5d2, 0x5d3, 0x5d3, 0x5d1, 0x5d7, 0x5e8,
0x5d9, 0x5d9, 0x5df, 0x5d1, 0x5d0, 0x5e7, 0x5d5, 0x5d1, 0x5e0, 0x5d2, 0x5e7, 0x5d5,
0x5e7, 0x5d1, 0x5e8, 0x5e0, 0x5d0, 0x5d5, 0x5dc, 0x5d1, 0x5d9, 0x5d9, 0x5e8, 0x5d5,
0x5ea, 0x5d1, 0x5d9, 0x5e9, 0x5e7, 0x5e7, 0x5d1, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d9,
0x5e6, 0x5f3, 0x5d9, 0x5d8, 0x5d4, 0x5e7, 0x5d5, 0x5dc, 0x5d5, 0x5de, 0x5d1, 0x5d5,
0x5d3, 0x5de, 0x5e9, 0x5e7, 0x5d3, 0x5d0, 0x5e7, 0x5d4, 0x5d3, 0x5d9, 0x5dc, 0x5d9,
0x5d3, 0x5d5, 0x5d1, 0x5d0, 0x5d9, 0x5d3, 0x5d5, 0x5e9, 0x5e0, 0x5d1, 0x5d4, 0x5e4,
0x5de, 0x5d2, 0x5d5, 0x5e1, 0x5d8, 0x5d4, 0x5e2, 0x5d6, 0x5d4, 0x5d7, 0x5d1, 0x5e8,
0x5d5, 0x5df, 0x5d4, 0x5d5, 0x20, 0x5e6, 0x5f3, 0x5d9, 0x20, 0x5de, 0x5d9, 0x5df,
0x20, 0x5e1, 0x5d9, 0x5d8, 0x5d9, 0x5d4, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5,
0x5e0, 0x5d2, 0x5d7, 0x5d5, 0x5d1, 0x5d3, 0x5d0, 0x5d9, 0x5e8, 0x5e7, 0x5d5, 0x5d8,
0x5e1, 0x5e7, 0x5d2, 0x5f3, 0x5e7, 0x5e8, 0x5d8, 0x5d4, 0x5d2, 0x5f3, 0x5d0, 0x5d9,
0x5d0, 0x5e4, 0x5d5, 0x5e8, 0x5d4, 0x5d9, 0x5e8, 0x5d5, 0x5e9, 0x5dc, 0x5d9, 0x5dd,
0x5e7, 0x5d0, 0x5d1, 0x5d5, 0x5dc, 0x5e7, 0x5de, 0x5e6, 0x5f3, 0x5d8, 0x5e7, 0x5d4,
0x5e7, 0x5e8, 0x5d0, 0x5e6, 0x5f3, 0x5d9, 0x5e7, 0x5d8, 0x5de, 0x5e0, 0x5d3, 0x5d5,
0x5d7, 0x5e0, 0x5d3, 0x5d9, 0x5d2, 0x5d4, 0x5e7, 0x5d5, 0x5dc, 0x5e7, 0x5d8, 0x5d4,
0x5e7, 0x5e8, 0x5e1, 0x5e0, 0x5d5, 0x5d9, 0x5d0, 0x5e8, 0x5e1, 0x5e7, 0x5e7, 0x5d5,
0x5d0, 0x5dc, 0x5d4, 0x20, 0x5dc, 0x5d5, 0x5de, 0x5e4, 0x5d5, 0x5e8, 0x5e7, 0x5d5,
0x5e6, 0x5f3, 0x5d9, 0x5e0, 0x5d2, 0x5db, 0x5d5, 0x5d5, 0x5d9, 0x5ea, 0x5de, 0x5e7,
0x5d0, 0x5d5, 0x5de, 0x5d2, 0x5d3, 0x5df, 0x5de, 0x5d0, 0x5e7, 0x5d0, 0x5e1, 0x5d0,
0x5e8, 0x5de, 0x5e0, 0x5d9, 0x5dc, 0x5d4, 0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d8, 0x5e0,
0x5d9, 0x5e7, 0x5d5, 0x5e1, 0x5d9, 0x5d4, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e7, 0x5d5,
0x5d6, 0x5e0, 0x5d8, 0x5e1, 0x5e7, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e1, 0x5d9, 0x5d1,
0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x5d0, 0x5d5, 0x5de, 0x5e1, 0x5e7, 0x5d0, 0x5d5, 0x5e8,
0x5dc, 0x5e4, 0x5e0, 0x5d5, 0x5dd, 0x20, 0x5e4, 0x5df, 0x5e4, 0x5d5, 0x5e0, 0x5d8,
0x5d9, 0x5d0, 0x5e0, 0x5e7, 0x5e4, 0x5d9, 0x5d5, 0x5e0, 0x5d2, 0x5d9, 0x5d0, 0x5e0,
0x5d2, 0x5e7, 0x5d8, 0x5d0, 0x5e8, 0x5e7, 0x5d5, 0x5e1, 0x5d8, 0x5e0, 0x5d0, 0x5d9,
0x5e7, 0x5d9, 0x5d6, 0x5d9, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x5e8, 0x5d9, 0x5d0,
0x5d3, 0x5e1, 0x5d7, 0x5dc, 0x5d9, 0x5df, 0x5e1, 0x5de, 0x5e8, 0x5e7, 0x5e0, 0x5d3,
0x5e1, 0x5d9, 0x5d0, 0x5d5, 0x5dc, 0x5e9, 0x5e0, 0x5d7, 0x5d0, 0x5d9, 0x5e1, 0x5d9,
0x5e0, 0x5d2, 0x5e4, 0x5d5, 0x5e8, 0x5e1, 0x5e8, 0x5d3, 0x5e0, 0x5d9, 0x5d9, 0x5e7,
0x5d5, 0x5dc, 0x5d9, 0x5de, 0x5e1, 0x5e7, 0x5d8, 0x5d0, 0x5d9, 0x5e4, 0x5d9, 0x5d9,
0x5d8, 0x5e9, 0x5e7, 0x5e0, 0x5d8, 0x5d8, 0x5d1, 0x5d9, 0x5dc, 0x5d9, 0x5e1, 0x5d9,
0x5d8, 0x5d4, 0x5e8, 0x5df, 0x5d8, 0x5d4, 0x5d9, 0x5de, 0x5e4, 0x5d4, 0x5d5, 0x5d8,
0x5d5, 0x5e7, 0x5d9, 0x5d5, 0x5d8, 0x5d5, 0x5de, 0x5e1, 0x5e7, 0x5d0, 0x5d5, 0x5dc,
0x5d0, 0x5df, 0x20, 0x5d1, 0x5d0, 0x5d8, 0x5d5, 0x5e8, 0x5d0, 0x5d5, 0x5e8, 0x5d5,
0x5de, 0x5e7, 0x5d9, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x2d, 0x5e0, 0x5e8, 0x5d4, 0x5d5,
0x5d9, 0x5d9, 0x5e0, 0x5d8, 0x5d9, 0x5d0, 0x5df, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5,
0x5d5, 0x5e1, 0x5d8, 0x5d5, 0x5e7, 0x5d9, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x5e8,
0x5e0, 0x5d2, 0x5d5, 0x5df, 0x5d9, 0x5e7, 0x5d8, 0x5e8, 0x5d9, 0x5e0, 0x5d1, 0x5d5,
0x5e8, 0x5d2, 0x5d9, 0x5e8, 0x5d5, 0x5d5, 0x5d0, 0x5df, 0x5d4, 0x5d0, 0x5d9, 0x5d9,
0x5dd, 0x20, 0x5d4, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x5d9, 0x5d9, 0x5dd, 0x5d1, 0x5e8,
0x5de, 0x5d5, 0x5d3, 0x5d4, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5e7,
0x5e0, 0x5e8, 0x5d9, 0x5d9, 0x5dd, 0x5db, 0x5e3, 0x20, 0x5d5, 0x5e8, 0x5d3, 0x5d4,
0x5e4, 0x5d0, 0x5e8, 0x5d5, 0x5de, 0x5d3, 0x5d9, 0x5d9, 0x5e8, 0x5d4, 0x5e8, 0x5d9,
0x5d9, 0x5e7, 0x5d9, 0x5d0, 0x5d5, 0x5d5, 0x5d9, 0x5e7, 0x5d3, 0x5e8, 0x5d5, 0x5dd,
0x20, 0x5d2, 0x5f3, 0x5d5, 0x5e8, 0x5d2, 0x5f3, 0x5d9, 0x5d4, 0x5e1, 0x5e0, 0x5d8,
0x20, 0x5d4, 0x5dc, 0x5e0, 0x5d4, 0x5e1, 0x5d8, 0x5e0, 0x5dc, 0x5d9, 0x5d0, 0x5d3,
0x5dc, 0x5d9, 0x5d9, 0x5d3, 0x5d1, 0x5e8, 0x5d9, 0x5e1, 0x5d1, 0x5d9, 0x5d9, 0x5df,
0x5d1, 0x5e8, 0x5d5, 0x5e7, 0x5df, 0x20, 0x5d4, 0x5d9, 0x5dc, 0x5d3, 0x5e8, 0x5d5,
0x5d5, 0x5d9, 0x5df, 0x5d9, 0x5d5, 0x5e7, 0x5dc, 0x5d4, 0x5d4, 0x5d5, 0x5d1, 0x5e8,
0x5d8, 0x5dc, 0x5d9, 0x5e0, 0x5d3, 0x5de, 0x5df, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5dc,
0x5d5, 0x5e8, 0x5d3, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x5de, 0x5dc, 0x5d1, 0x5d5, 0x5e8,
0x5df, 0x5e4, 0x5e8, 0x5ea, 0x5f3, 0x5e1, 0x5d9, 0x5d3, 0x5e0, 0x5d9, 0x5d0, 0x5de,
0x5e1, 0x5d8, 0x5e8, 0x5d3, 0x5dd, 0x5d0, 0x5e0, 0x5d3, 0x5d5, 0x5e8, 0x5d4, 0x5d0,
0x5e1, 0x5d8, 0x5e8, 0x5d7, 0x5df, 0x5d0, 0x5ea, 0x5d5, 0x5e0, 0x5d4, 0x5d1, 0x5dc,
0x5d2, 0x5e8, 0x5d3, 0x5d1, 0x5e8, 0x5dc, 0x5d9, 0x5df, 0x5d1, 0x5e8, 0x5d8, 0x5d9,
0x5e1, 0x5dc, 0x5d1, 0x5d4, 0x5d1, 0x5e8, 0x5d9, 0x5e1, 0x5dc, 0x5d1, 0x5d5, 0x5e7,
0x5e8, 0x5e9, 0x5d8, 0x5d1, 0x5d5, 0x5d3, 0x5e4, 0x5e9, 0x5d8, 0x5d1, 0x5d9, 0x5d6,
0x5d9, 0x5e0, 0x5d2, 0x5df, 0x5e7, 0x5d9, 0x5e9, 0x5d9, 0x5e0, 0x5d1, 0x5e7, 0x5d5,
0x5e4, 0x5e0, 0x5d4, 0x5d2, 0x5df, 0x5d3, 0x5d1, 0x5dc, 0x5d9, 0x5df, 0x5d2, 0x5d9,
0x5d1, 0x5e8, 0x5dc, 0x5d8, 0x5e8, 0x5d2, 0x5e8, 0x5e0, 0x5d6, 0x5d9, 0x5d4, 0x5dc,
0x5e1, 0x5d9, 0x5e0, 0x5e7, 0x5d9, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5de, 0x5d0, 0x5df,
0x5d0, 0x5d9, 0x5e1, 0x5d8, 0x5e0, 0x5d1, 0x5d5, 0x5dc, 0x5d2, 0x5f3, 0x5e8, 0x5d6,
0x5d9, 0x5e7, 0x5dc, 0x5d9, 0x5e0, 0x5d9, 0x5e0, 0x5d2, 0x5e8, 0x5d3, 0x5e7, 0x5d9,
0x5e8, 0x5d5, 0x5d1, 0x5e7, 0x5d9, 0x5d9, 0x5d1, 0x5dc, 0x5d9, 0x5e1, 0x5d1, 0x5d5,
0x5df, 0x5dc, 0x5d5, 0x5d1, 0x5dc, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5dc, 0x5d5, 0x5e0,
0x5d3, 0x5d5, 0x5df, 0x5dc, 0x5d5, 0x5e7, 0x5e1, 0x5de, 0x5d1, 0x5d5, 0x5e8, 0x5d2,
0x5de, 0x5d3, 0x5e8, 0x5d9, 0x5d3, 0x5de, 0x5dc, 0x5d8, 0x5d4, 0x5de, 0x5e8, 0x5d9,
0x5d9, 0x5d4, 0x5d0, 0x5de, 0x5df, 0x5de, 0x5d9, 0x5e0, 0x5e1, 0x5e7, 0x5de, 0x5d5,
0x5e0, 0x5e7, 0x5d5, 0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d1, 0x5d4, 0x5d0, 0x5d5, 0x5e1,
0x5dc, 0x5d5, 0x5e4, 0x5e8, 0x5d9, 0x5d6, 0x5e4, 0x5d5, 0x5d3, 0x5d2, 0x5d5, 0x5e8,
0x5d9, 0x5e6, 0x5d4, 0x5e4, 0x5e8, 0x5d0, 0x5d2, 0x5e8, 0x5d9, 0x5d2, 0x5d4, 0x5e8,
0x5d5, 0x5de, 0x5d0, 0x5e1, 0x5df, 0x20, 0x5de, 0x5e8, 0x5d9, 0x5e0, 0x5d5, 0x5e1,
0x5e8, 0x5d9, 0x5d9, 0x5d1, 0x5d5, 0x5e1, 0x5e8, 0x5d0, 0x5d8, 0x5d5, 0x5d1, 0x5e1,
0x5d9, 0x5de, 0x5e4, 0x5e8, 0x5d5, 0x5e4, 0x5d5, 0x5dc, 0x5e1, 0x5e7, 0x5d5, 0x5e4,
0x5d9, 0x5d4, 0x5e1, 0x5d5, 0x5e4, 0x5d9, 0x5d4, 0x5e9, 0x5d8, 0x5d5, 0x5e7, 0x5d4,
0x5d5, 0x5dc, 0x5dd, 0x5d8, 0x5d0, 0x5dc, 0x5d9, 0x5df, 0x5d8, 0x5d9, 0x5e8, 0x5e0,
0x5d4, 0x5d0, 0x5d5, 0x5dc, 0x5d9, 0x5d0, 0x5e0, 0x5d5, 0x5d1, 0x5e1, 0x5e7, 0x5d5,
0x5d0, 0x5d3, 0x5d5, 0x5e5, 0x5d4, 0x5d5, 0x5d5, 0x5ea, 0x5d9, 0x5e7, 0x5df, 0x5d5,
0x5d9, 0x5e0, 0x5d4, 0x5d5, 0x5d9, 0x5dc, 0x5e0, 0x5d4, 0x5d5, 0x5d5, 0x5dc, 0x5d2,
0x5d5, 0x5d2, 0x5e8, 0x5d3, 0x5d5, 0x5e8, 0x5e9, 0x5d4, 0x5d6, 0x5d0, 0x5d2, 0x5e8,
0x5d1, 0x5e6, 0x5d9, 0x5e8, 0x5d9, 0x5da, 0x5d0, 0x5e0, 0x5d8, 0x5e0, 0x5e0, 0x5e8,
0x5d9, 0x5d1, 0x5d5, 0x5e6, 0x5f3, 0x5d0, 0x5d2, 0x5d5, 0x5e1, 0x5d4, 0x5d0, 0x5d9,
0x20, 0x5db, 0x5e8, 0x5d9, 0x5e1, 0x5d8, 0x5de, 0x5e1, 0x5e7, 0x5d5, 0x5e7, 0x5d5,
0x5e1, 0x5e7, 0x5d5, 0x5de, 0x5d5, 0x5e8, 0x5d5, 0x5e7, 0x5e8, 0x5d2, 0x5d5, 0x5d5,
0x5dc, 0x5df, 0x5de, 0x5d4, 0x5d0, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4,
0x5de, 0x5dc, 0x5d3, 0x5d9, 0x5d1, 0x5d9, 0x5d9, 0x5dd, 0x5de, 0x5d0, 0x5d5, 0x5e8,
0x5d9, 0x5e6, 0x5d9, 0x5d5, 0x5e1, 0x5de, 0x5d0, 0x5d9, 0x5d5, 0x5d8, 0x5e8, 0x5d0,
0x5d5, 0x5e0, 0x5d9, 0x5d5, 0x5df, 0x5d0, 0x5e4, 0x5d9, 0x5d4, 0x5d0, 0x5d5, 0x5e7,
0x5dc, 0x5e0, 0x5d3, 0x5d1, 0x5d5, 0x5d2, 0x5e0, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5e6,
0x5f3, 0x5d0, 0x5d8, 0x5d4, 0x5d0, 0x5dd, 0x5e6, 0x5f3, 0x5d5, 0x5e7, 0x5d0, 0x5d9,
0x20, 0x5d4, 0x5e4, 0x5e1, 0x5d7, 0x5d0, 0x5d0, 0x5e4, 0x5d8, 0x5d4, 0x5e4, 0x5e7,
0x5d0, 0x5d5, 0x5e4, 0x5d5, 0x5e4, 0x5d9, 0x5d2, 0x5f3, 0x5d9, 0x5e4, 0x5d5, 0x5e0,
0x5e4, 0x5d5, 0x5d8, 0x5d9, 0x5d2, 0x5dc, 0x5e4, 0x5d0, 0x5d2, 0x5d5, 0x5e1, 0x5d0,
0x5d9, 0x5d9, 0x20, 0x5d2, 0x5de, 0x5d1, 0x5d9, 0x5d9, 0x5d4, 0x5d2, 0x5d5, 0x5d5,
0x5d3, 0x5dc, 0x5e7, 0x5e0, 0x5d0, 0x5dc, 0x5d2, 0x5d5, 0x5d0, 0x5dd, 0x5d4, 0x5d5,
0x5e0, 0x5d5, 0x5dc, 0x5d5, 0x5dc, 0x5d5, 0x5d0, 0x5e0, 0x5d3, 0x5e8, 0x5d1, 0x5d5,
0x5e8, 0x5d9, 0x5e7, 0x5d9, 0x5e8, 0x5d9, 0x5d8, 0x5d9, 0x5de, 0x5d0, 0x5d8, 0x5d9,
0x5e7, 0x5d5, 0x5e1, 0x5e8, 0x5d4, 0x5e7, 0x5d5, 0x5d5, 0x5d0, 0x5d2, 0x5f3, 0x5dc,
0x5d9, 0x5d9, 0x5df, 0x5de, 0x5d0, 0x5d2, 0x5f3, 0x5d5, 0x5e8, 0x5d5, 0x5d0, 0x5d9,
0x5d9, 0x20, 0x5de, 0x5e8, 0x5e7, 0x5d9, 0x5d6, 0x5de, 0x5d9, 0x5d3, 0x5d5, 0x5d5,
0x5d9, 0x5d9, 0x5e0, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d5, 0x5d0, 0x5d4,
0x5e0, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e7, 0x5e0, 0x5d5, 0x5de, 0x5d0, 0x5d4, 0x5e4,
0x5d0, 0x5d2, 0x5d5, 0x20, 0x5e4, 0x5d0, 0x5d2, 0x5d5, 0x5e4, 0x5dc, 0x5d0, 0x5d5,
0x5e4, 0x5d9, 0x5d8, 0x5e7, 0x5e8, 0x5df, 0x5e4, 0x5d5, 0x5e0, 0x5e4, 0x5d9, 0x5d9,
0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x20, 0x5de, 0x5d5, 0x5e8, 0x5e1, 0x5d1, 0x5d9, 0x5e8,
0x5d0, 0x5e8, 0x5d5, 0x5d8, 0x5d5, 0x5e0, 0x5d2, 0x5d4, 0x5e1, 0x5d0, 0x5d9, 0x5e4,
0x5d0, 0x5df, 0x5d8, 0x5d4, 0x5d9, 0x5d8, 0x5d9, 0x5d8, 0x5d0, 0x5e8, 0x5d0, 0x5d5,
0x5d5, 0x5d4, 0x5d8, 0x5d5, 0x5e0, 0x5d2, 0x5d8, 0x5d0, 0x5e4, 0x5d5, 0x5d5, 0x5d9,
0x5d9, 0x5e7, 0x5d5, 0x5d5, 0x5dc, 0x5d9, 0x5e1, 0x90f, 0x915, 0x94d, 0x930, 0x93e,
0x905, 0x926, 0x940, 0x938, 0x20, 0x905, 0x92c, 0x93e, 0x92c, 0x93e, 0x905, 0x932,
0x94d, 0x91c, 0x940, 0x92f, 0x930, 0x94d, 0x938, 0x905, 0x938, 0x94d, 0x92e, 0x93e,
0x930, 0x93e, 0x92c, 0x93e, 0x92e, 0x93e, 0x915, 0x94b, 0x92c, 0x93e, 0x902, 0x917,
0x941, 0x907, 0x92c, 0x948, 0x902, 0x91c, 0x941, 0x932, 0x92c, 0x93f, 0x938, 0x93e,
0x90a, 0x92c, 0x94d, 0x932, 0x93e, 0x902, 0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d,
0x930, 0x93e, 0x91c, 0x93c, 0x93e, 0x935, 0x93f, 0x932, 0x947, 0x92c, 0x941, 0x91c,
0x941, 0x902, 0x92c, 0x942, 0x930, 0x93e, 0x915, 0x93e, 0x92f, 0x930, 0x94b, 0x938,
0x947, 0x909, 0x91f, 0x93e, 0x915, 0x94b, 0x928, 0x93e, 0x915, 0x94d, 0x930, 0x940,
0x921, 0x915, 0x93e, 0x930, 0x926, 0x93e, 0x930, 0x20, 0x905, 0x938, 0x20, 0x938,
0x932, 0x93e, 0x92e, 0x91c, 0x93f, 0x92c, 0x942, 0x924, 0x940, 0x921, 0x942, 0x906,
0x932, 0x93e, 0x905, 0x932, 0x20, 0x906, 0x907, 0x92f, 0x942, 0x928, 0x92b, 0x93c,
0x94d, 0x930, 0x940, 0x91f, 0x93e, 0x909, 0x928, 0x917, 0x93e, 0x92c, 0x94b, 0x930,
0x94b, 0x928, 0x939, 0x930, 0x93e, 0x930, 0x947, 0x91c, 0x94b, 0x939, 0x93e, 0x902,
0x938, 0x92c, 0x930, 0x94d, 0x917, 0x915, 0x902, 0x92a, 0x93e, 0x932, 0x93e, 0x915,
0x93f, 0x917, 0x93e, 0x932, 0x940, 0x915, 0x93f, 0x902, 0x936, 0x93e, 0x938, 0x93e,
0x932, 0x93e, 0x917, 0x94b, 0x938, 0x932, 0x93f, 0x92c, 0x94d, 0x930, 0x947, 0x935,
0x93f, 0x932, 0x947, 0x932, 0x94b, 0x92e, 0x932, 0x941, 0x906, 0x902, 0x921, 0x93e,
0x932, 0x941, 0x92c, 0x941, 0x92e, 0x92c, 0x93e, 0x936, 0x940, 0x92e, 0x932, 0x93e,
0x92c, 0x94b, 0x92e, 0x93e, 0x92a, 0x941, 0x91f, 0x94b, 0x90f, 0x92e, 0x92c, 0x93e,
0x92c, 0x947, 0x928, 0x92e, 0x94b, 0x917, 0x93e, 0x926, 0x93f, 0x936, 0x941, 0x92e,
0x94b, 0x928, 0x930, 0x94b, 0x935, 0x93f, 0x92f, 0x93e, 0x928, 0x948, 0x930, 0x94b,
0x92c, 0x940, 0x928, 0x947, 0x926, 0x94d, 0x91c, 0x93e, 0x92e, 0x940, 0x928, 0x93e,
0x928, 0x93f, 0x92f, 0x93e, 0x92e, 0x940, 0x928, 0x94c, 0x906, 0x915, 0x936, 0x949,
0x91f, 0x914, 0x917, 0x93e, 0x921, 0x94b, 0x917, 0x942, 0x92a, 0x94b, 0x930, 0x94d,
0x91f, 0x94b, 0x2d, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93e, 0x913, 0x20, 0x91f,
0x94b, 0x92e, 0x924, 0x94d, 0x930, 0x93f, 0x92a, 0x94b, 0x932, 0x940, 0x935, 0x93f,
0x902, 0x921, 0x939, 0x94b, 0x915, 0x905, 0x921, 0x915, 0x90f, 0x902, 0x917, 0x94d,
0x935, 0x93f, 0x932, 0x93e, 0x90f, 0x902, 0x91f, 0x940, 0x917, 0x941, 0x906, 0x906,
0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x928, 0x93e, 0x92c, 0x94d, 0x92f, 0x942,
0x928, 0x938, 0x20, 0x906, 0x92f, 0x930, 0x938, 0x915, 0x94b, 0x930, 0x94d, 0x921,
0x94b, 0x92c, 0x93e, 0x91c, 0x941, 0x91c, 0x94b, 0x90f, 0x932, 0x93e, 0x20, 0x930,
0x93f, 0x913, 0x91c, 0x93e, 0x92e, 0x947, 0x902, 0x921, 0x94b, 0x91c, 0x93c, 0x93e,
0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x917, 0x93e, 0x932, 0x947, 0x917, 0x94b, 0x938,
0x938, 0x948, 0x928, 0x20, 0x939, 0x94d, 0x935, 0x93e, 0x928, 0x938, 0x948, 0x928,
0x20, 0x932, 0x942, 0x908, 0x91f, 0x94b, 0x915, 0x942, 0x92e, 0x928, 0x909, 0x936,
0x941, 0x906, 0x907, 0x92f, 0x93e, 0x905, 0x930, 0x942, 0x92c, 0x93e, 0x90f, 0x938,
0x928, 0x936, 0x93f, 0x92f, 0x949, 0x928, 0x905, 0x91f, 0x93f, 0x915, 0x94b, 0x915,
0x93e, 0x928, 0x92c, 0x939, 0x93f, 0x92f, 0x93e, 0x92c, 0x947, 0x939, 0x93f, 0x92f,
0x93e, 0x20, 0x92c, 0x93e, 0x902, 0x921, 0x947, 0x930, 0x93e, 0x938, 0x92c, 0x93e,
0x930, 0x92c, 0x93e, 0x921, 0x94b, 0x938, 0x92c, 0x947, 0x932, 0x940, 0x91c, 0x93c,
0x92c, 0x94d, 0x932, 0x93e, 0x902, 0x915, 0x2d, 0x938, 0x947, 0x92c, 0x932, 0x94b,
0x928, 0x92c, 0x94b, 0x906, 0x20, 0x935, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x92c,
0x94b, 0x917, 0x94b, 0x91f, 0x93e, 0x92c, 0x949, 0x907, 0x938, 0x940, 0x915, 0x948,
0x92e, 0x94d, 0x92c, 0x94d, 0x930, 0x93f, 0x91c, 0x20, 0x916, 0x93e, 0x921, 0x93c,
0x940, 0x915, 0x948, 0x902, 0x92a, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x93e, 0x902,
0x921, 0x947, 0x915, 0x948, 0x928, 0x915, 0x941, 0x928, 0x915, 0x93e, 0x930, 0x93e,
0x915, 0x938, 0x936, 0x93f, 0x915, 0x93e, 0x917, 0x94b, 0x91a, 0x93f, 0x939, 0x941,
0x906, 0x939, 0x941, 0x906, 0x938, 0x94d, 0x92f, 0x941, 0x926, 0x93e, 0x926, 0x20,
0x939, 0x94d, 0x935, 0x93e, 0x930, 0x947, 0x91c, 0x93c, 0x915, 0x94b, 0x938, 0x94d,
0x91f, 0x93e, 0x20, 0x930, 0x93f, 0x915, 0x93e, 0x915, 0x94d, 0x92f, 0x942, 0x906,
0x92c, 0x93e, 0x915, 0x941, 0x930, 0x93e, 0x915, 0x93e, 0x913, 0x921, 0x947, 0x928,
0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x936, 0x949, 0x928, 0x921, 0x949, 0x938, 0x928,
0x921, 0x949, 0x938, 0x928, 0x20, 0x915, 0x94d, 0x930, 0x940, 0x915, 0x921, 0x947,
0x928, 0x935, 0x930, 0x921, 0x947, 0x91f, 0x94d, 0x930, 0x949, 0x92f, 0x91f, 0x921,
0x94b, 0x92e, 0x93f, 0x928, 0x93f, 0x915, 0x93e, 0x90f, 0x921, 0x92e, 0x902, 0x91f,
0x928, 0x905, 0x932, 0x20, 0x938, 0x932, 0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930,
0x92b, 0x93c, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x928, 0x947, 0x932, 0x94d, 0x938,
0x928, 0x92b, 0x93c, 0x94b, 0x930, 0x94d, 0x91f, 0x93e, 0x932, 0x947, 0x91c, 0x93c,
0x93e, 0x917, 0x94d, 0x932, 0x947, 0x938, 0x20, 0x916, 0x93e, 0x921, 0x93c, 0x940,
0x917, 0x942, 0x938, 0x20, 0x916, 0x93e, 0x921, 0x93c, 0x940, 0x917, 0x94d, 0x930,
0x93e, 0x902, 0x921, 0x20, 0x91f, 0x930, 0x94d, 0x915, 0x917, 0x94d, 0x935, 0x93e,
0x921, 0x947, 0x932, 0x94b, 0x92a, 0x917, 0x94d, 0x935, 0x93e, 0x92f, 0x93e, 0x915,
0x940, 0x932, 0x917, 0x92f, 0x93e, 0x928, 0x93e, 0x939, 0x947, 0x932, 0x93f, 0x92b,
0x93c, 0x948, 0x915, 0x94d, 0x938, 0x939, 0x935, 0x93e, 0x928, 0x93e, 0x939, 0x930,
0x94d, 0x92e, 0x94b, 0x938, 0x93f, 0x932, 0x94d, 0x932, 0x94b, 0x907, 0x902, 0x921,
0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92a, 0x94b, 0x932, 0x93f, 0x938, 0x928, 0x94c,
0x915, 0x94d, 0x938, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x92e, 0x93e, 0x930, 0x947, 0x902, 0x917, 0x94b, 0x2c, 0x20, 0x907, 0x902,
0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92a, 0x940, 0x91f, 0x930, 0x94d, 0x938,
0x92c, 0x930, 0x94d, 0x917, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x91f, 0x947, 0x932, 0x20, 0x938, 0x93f, 0x91f, 0x940, 0x2c, 0x20,
0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x947, 0x935, 0x947,
0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f,
0x902, 0x938, 0x947, 0x928, 0x947, 0x938, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f,
0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x928, 0x93e, 0x92e, 0x947, 0x915, 0x2c,
0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x915, 0x93e,
0x932, 0x941, 0x908, 0x91f, 0x91c, 0x92e, 0x948, 0x915, 0x93e, 0x91c, 0x94d, 0x92f,
0x942, 0x928, 0x93e, 0x909, 0x932, 0x941, 0x907, 0x938, 0x935, 0x93f, 0x932, 0x947,
0x92e, 0x94b, 0x902, 0x91f, 0x940, 0x938, 0x947, 0x932, 0x94b, 0x2c, 0x20, 0x915,
0x947, 0x902, 0x91f, 0x915, 0x940, 0x915, 0x94d, 0x930, 0x93e, 0x932, 0x948, 0x902,
0x921, 0x93f, 0x91c, 0x93f, 0x915, 0x932, 0x93e, 0x20, 0x92a, 0x93e, 0x91c, 0x93c,
0x932, 0x949, 0x938, 0x20, 0x90f, 0x902, 0x91c, 0x93f, 0x932, 0x94d, 0x938, 0x932,
0x94b, 0x905, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x902, 0x938, 0x947, 0x938,
0x20, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x94d, 0x91f, 0x930, 0x92e, 0x947, 0x938,
0x940, 0x913, 0x92e, 0x93e, 0x928, 0x93e, 0x917, 0x941, 0x906, 0x92e, 0x928, 0x94c,
0x938, 0x92e, 0x948, 0x930, 0x940, 0x917, 0x94b, 0x91f, 0x92e, 0x93e, 0x930, 0x94d,
0x91f, 0x93f, 0x928, 0x93f, 0x915, 0x92e, 0x93e, 0x91f, 0x93e, 0x92e, 0x94b, 0x930,
0x94b, 0x938, 0x92e, 0x93e, 0x91c, 0x93c, 0x93e, 0x91f, 0x932, 0x93e, 0x928, 0x92e,
0x947, 0x928, 0x94b, 0x92e, 0x93f, 0x928, 0x940, 0x92e, 0x947, 0x91f, 0x94d, 0x932,
0x947, 0x915, 0x93e, 0x91f, 0x94d, 0x932, 0x93e, 0x92e, 0x947, 0x915, 0x94d, 0x938,
0x93f, 0x915, 0x94b, 0x20, 0x938, 0x93f, 0x91f, 0x940, 0x92e, 0x93f, 0x915, 0x947,
0x932, 0x949, 0x928, 0x92e, 0x94b, 0x902, 0x915, 0x91f, 0x928, 0x92e, 0x94b, 0x902,
0x91f, 0x947, 0x930, 0x947, 0x930, 0x940, 0x92e, 0x94b, 0x902, 0x91f, 0x947, 0x935,
0x940, 0x921, 0x93f, 0x92f, 0x94b, 0x92e, 0x94b, 0x902, 0x91f, 0x938, 0x947, 0x930,
0x93e, 0x924, 0x928, 0x93e, 0x938, 0x93e, 0x909, 0x928, 0x94d, 0x92f, 0x942, 0x92f,
0x949, 0x930, 0x94d, 0x915, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e,
0x92c, 0x94d, 0x92f, 0x942, 0x932, 0x93e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x92e, 0x927, 0x94d, 0x92f,
0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x926, 0x93e, 0x915,
0x94b, 0x924, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x938, 0x93e, 0x932, 0x947,
0x92e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x921, 0x915,
0x94b, 0x91f, 0x93e, 0x928, 0x941, 0x915, 0x913, 0x916, 0x93e, 0x91c, 0x940, 0x928,
0x93e, 0x917, 0x93e, 0x92a, 0x928, 0x93e, 0x92e, 0x93e, 0x92a, 0x93e, 0x930, 0x93e,
0x92e, 0x93e, 0x930, 0x93f, 0x92c, 0x94b, 0x92b, 0x93c, 0x940, 0x928, 0x93f, 0x915,
0x94d, 0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x2d, 0x911, 0x2d, 0x92a, 0x94d,
0x930, 0x93f, 0x902, 0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x911, 0x92b,
0x93c, 0x20, 0x938, 0x94d, 0x92a, 0x947, 0x928, 0x92a, 0x94b, 0x930, 0x94d, 0x91f,
0x94b, 0x20, 0x935, 0x947, 0x932, 0x94d, 0x939, 0x94b, 0x92a, 0x94b, 0x930, 0x94d,
0x91f, 0x94b, 0x20, 0x930, 0x93f, 0x915, 0x94b, 0x92a, 0x941, 0x902, 0x91f, 0x93e,
0x20, 0x90f, 0x930, 0x93f, 0x928, 0x93e, 0x938, 0x930, 0x947, 0x902, 0x915, 0x93f,
0x928, 0x20, 0x907, 0x928, 0x932, 0x947, 0x91f, 0x930, 0x947, 0x938, 0x93e, 0x907,
0x92b, 0x93c, 0x930, 0x93f, 0x938, 0x94b, 0x932, 0x94d, 0x92f, 0x942, 0x91f, 0x930,
0x93f, 0x92f, 0x94b, 0x20, 0x92c, 0x94d, 0x930, 0x93e, 0x902, 0x915, 0x94b, 0x938,
0x948, 0x902, 0x91f, 0x93e, 0x930, 0x947, 0x92e, 0x938, 0x948, 0x902, 0x91f, 0x93f,
0x92f, 0x93e, 0x917, 0x94b, 0x938, 0x947, 0x902, 0x91f, 0x94b, 0x20, 0x921, 0x94b,
0x92e, 0x93f, 0x902, 0x917, 0x94b, 0x938, 0x93e, 0x913, 0x20, 0x92a, 0x93e, 0x909,
0x932, 0x94b, 0x907, 0x91f, 0x94b, 0x915, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x930,
0x92e, 0x93f, 0x91f, 0x938, 0x93f, 0x91f, 0x94d, 0x915, 0x93e, 0x938, 0x947, 0x902,
0x91f, 0x20, 0x92c, 0x93e, 0x930, 0x94d, 0x925, 0x947, 0x932, 0x947, 0x92e, 0x940,
0x938, 0x947, 0x902, 0x91f, 0x20, 0x91c, 0x94b, 0x902, 0x938, 0x938, 0x947, 0x902,
0x91f, 0x20, 0x915, 0x93f, 0x91f, 0x94d, 0x938, 0x938, 0x947, 0x902, 0x91f, 0x20,
0x932, 0x942, 0x938, 0x93f, 0x92f, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x925,
0x949, 0x92e, 0x938, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x935, 0x93f, 0x902, 0x938,
0x947, 0x902, 0x91f, 0x938, 0x94d, 0x935, 0x93f, 0x92b, 0x93c, 0x94d, 0x91f, 0x20,
0x915, 0x930, 0x902, 0x91f, 0x91f, 0x947, 0x917, 0x941, 0x938, 0x93f, 0x917, 0x932,
0x94d, 0x92a, 0x93e, 0x925, 0x94d, 0x92f, 0x942, 0x932, 0x947, 0x924, 0x93f, 0x91c,
0x941, 0x906, 0x928, 0x93e, 0x91f, 0x94b, 0x930, 0x902, 0x91f, 0x94b, 0x91f, 0x94b,
0x930, 0x94d, 0x91f, 0x94b, 0x932, 0x93e, 0x935, 0x948, 0x902, 0x915, 0x942, 0x935,
0x930, 0x935, 0x94d, 0x939, 0x93e, 0x907, 0x91f, 0x939, 0x94b, 0x930, 0x94d, 0x938,
0x935, 0x93f, 0x928, 0x940, 0x92a, 0x947, 0x917, 0x92f, 0x93e, 0x915, 0x942, 0x91f,
0x93e, 0x91f, 0x921, 0x947, 0x935, 0x93f, 0x938, 0x921, 0x94d, 0x92f, 0x942, 0x92e,
0x94b, 0x902, 0x91f, 0x20, 0x921, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x935, 0x93f,
0x932, 0x947, 0x92e, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x940, 0x92e, 0x949, 0x938,
0x928, 0x92e, 0x948, 0x915, 0x92e, 0x941, 0x930, 0x94d, 0x921, 0x94b, 0x92a, 0x949,
0x92e, 0x930, 0x930, 0x94b, 0x925, 0x947, 0x930, 0x93e, 0x938, 0x94d, 0x92f, 0x94b,
0x935, 0x93e, 0x91f, 0x94d, 0x930, 0x94b, 0x932, 0x935, 0x94b, 0x938, 0x94d, 0x924,
0x94b, 0x915, 0x932, 0x949, 0x928, 0x94d, 0x917, 0x908, 0x92f, 0x930, 0x92c, 0x93e,
0x92f, 0x947, 0x928, 0x906, 0x926, 0x947, 0x928, 0x905, 0x932, 0x94d, 0x92e, 0x93e,
0x91f, 0x940, 0x905, 0x92e, 0x94d, 0x92e, 0x93e, 0x928, 0x905, 0x928, 0x93e, 0x921,
0x93f, 0x930, 0x905, 0x915, 0x94d, 0x924, 0x94b, 0x92c, 0x90f, 0x924, 0x930, 0x93e,
0x909, 0x92c, 0x917, 0x926, 0x93e, 0x926, 0x92c, 0x939, 0x930, 0x940, 0x928, 0x92c,
0x948, 0x902, 0x915, 0x949, 0x915, 0x92c, 0x930, 0x94d, 0x928, 0x94b, 0x932, 0x92c,
0x947, 0x930, 0x941, 0x924, 0x92c, 0x94d, 0x930, 0x942, 0x928, 0x947, 0x908, 0x924,
0x94d, 0x936, 0x93f, 0x924, 0x93e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x94b, 0x926,
0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x938, 0x922, 0x93e, 0x915, 0x93e, 0x921, 0x93f,
0x932, 0x93f, 0x926, 0x941, 0x92c, 0x908, 0x926, 0x941, 0x936, 0x93e, 0x902, 0x92c,
0x947, 0x92b, 0x93c, 0x93e, 0x92e, 0x93e, 0x917, 0x941, 0x938, 0x94d, 0x924, 0x93e,
0x917, 0x93e, 0x91c, 0x93c, 0x93e, 0x939, 0x947, 0x92c, 0x94d, 0x930, 0x94b, 0x928,
0x939, 0x94b, 0x20, 0x91a, 0x940, 0x20, 0x92e, 0x93f, 0x928, 0x94d, 0x939, 0x20,
0x938, 0x93f, 0x91f, 0x940, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901,
0x917, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924,
0x94d, 0x938, 0x94d, 0x915, 0x91c, 0x915, 0x93e, 0x930, 0x94d, 0x924, 0x93e, 0x92f,
0x930, 0x942, 0x936, 0x932, 0x92e, 0x915, 0x93e, 0x92c, 0x941, 0x932, 0x915, 0x92e,
0x91a, 0x924, 0x94d, 0x915, 0x93e, 0x915, 0x93e, 0x920, 0x92e, 0x93e, 0x902, 0x921,
0x942, 0x916, 0x93e, 0x902, 0x921, 0x93f, 0x917, 0x93e, 0x915, 0x94b, 0x932, 0x915,
0x93e, 0x924, 0x93e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f,
0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x915, 0x941, 0x906, 0x932, 0x93e, 0x932,
0x902, 0x92a, 0x941, 0x930, 0x92e, 0x915, 0x93e, 0x90a, 0x92e, 0x915, 0x938, 0x94d,
0x938, 0x930, 0x92e, 0x938, 0x94d, 0x915, 0x91f, 0x928, 0x93f, 0x915, 0x94b, 0x938,
0x93f, 0x92f, 0x93e, 0x928, 0x94b, 0x935, 0x94b, 0x915, 0x941, 0x91c, 0x93c, 0x94d,
0x928, 0x947, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x928, 0x94b, 0x935, 0x94b, 0x938,
0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x913, 0x92e, 0x94d, 0x938,
0x94d, 0x915, 0x913, 0x930, 0x932, 0x928, 0x949, 0x92e, 0x20, 0x92a, 0x947, 0x928,
0x94d, 0x939, 0x92a, 0x94b, 0x902, 0x91f, 0x940, 0x92f, 0x93e, 0x902, 0x915, 0x92a,
0x94d, 0x92f, 0x94b, 0x902, 0x917, 0x92f, 0x93e, 0x902, 0x917, 0x915, 0x924, 0x930,
0x915, 0x94b, 0x938, 0x94d, 0x91f, 0x93e, 0x928, 0x947, 0x915, 0x947, 0x91c, 0x93c,
0x947, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x930, 0x93f, 0x92f, 0x93e, 0x926,
0x938, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x938, 0x92e, 0x930, 0x915, 0x902, 0x926,
0x938, 0x93f, 0x913, 0x932, 0x936, 0x902, 0x918, 0x93e, 0x908, 0x938, 0x94d, 0x930,
0x947, 0x926, 0x94d, 0x928, 0x93f, 0x915, 0x94b, 0x932, 0x93f, 0x92e, 0x938, 0x94d,
0x915, 0x924, 0x93e, 0x908, 0x92a, 0x947, 0x908, 0x924, 0x93e, 0x936, 0x915, 0x902,
0x926, 0x91f, 0x92c, 0x93f, 0x932, 0x93f, 0x938, 0x940, 0x925, 0x93f, 0x902, 0x92a,
0x942, 0x91f, 0x94b, 0x915, 0x94d, 0x92f, 0x94b, 0x924, 0x94b, 0x92e, 0x94d, 0x938,
0x94d, 0x915, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x93e, 0x924, 0x930, 0x909, 0x930,
0x942, 0x92e, 0x94d, 0x915, 0x940, 0x92f, 0x942, 0x938, 0x94d, 0x91f, 0x2013, 0x928,
0x947, 0x930, 0x93e, 0x935, 0x93f, 0x90f, 0x928, 0x924, 0x93f, 0x92f, 0x93e, 0x928,
0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b,
0x915, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930,
0x94d, 0x917, 0x92f, 0x947, 0x930, 0x947, 0x935, 0x93e, 0x928, 0x905, 0x91c, 0x93c,
0x94b, 0x930, 0x947, 0x938, 0x92c, 0x930, 0x92e, 0x942, 0x921, 0x93e, 0x915, 0x948,
0x928, 0x947, 0x930, 0x940, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921,
0x92b, 0x93c, 0x948, 0x930, 0x94b, 0x92e, 0x921, 0x947, 0x930, 0x93e, 0x930, 0x947,
0x915, 0x94d, 0x92f, 0x93e, 0x935, 0x93f, 0x915, 0x926, 0x915, 0x94d, 0x937, 0x93f,
0x923, 0x20, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x938, 0x947,
0x902, 0x91f, 0x20, 0x939, 0x947, 0x932, 0x947, 0x928, 0x93e, 0x938, 0x94d, 0x91f,
0x948, 0x928, 0x932, 0x940, 0x90f, 0x921, 0x93f, 0x932, 0x947, 0x921, 0x92c, 0x94d,
0x930, 0x93f, 0x938, 0x94d, 0x92c, 0x928, 0x92c, 0x94d, 0x930, 0x94b, 0x915, 0x928,
0x20, 0x939, 0x93f, 0x932, 0x921, 0x93e, 0x930, 0x94d, 0x935, 0x93f, 0x928, 0x939,
0x94b, 0x92c, 0x93e, 0x930, 0x94d, 0x91f, 0x932, 0x93f, 0x902, 0x921, 0x947, 0x92e,
0x93e, 0x928, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947,
0x92e, 0x947, 0x932, 0x92c, 0x94b, 0x930, 0x94d, 0x928, 0x92a, 0x930, 0x94d, 0x925,
0x90f, 0x92e, 0x94d, 0x938, 0x94d, 0x91f, 0x930, 0x94d, 0x921, 0x92e, 0x905, 0x902,
0x921, 0x94b, 0x930, 0x93e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x93e, 0x916,
0x93e, 0x928, 0x90f, 0x925, 0x947, 0x902, 0x938, 0x92c, 0x930, 0x94d, 0x932, 0x93f,
0x928, 0x92c, 0x94d, 0x930, 0x93e, 0x924, 0x93f, 0x938, 0x94d, 0x932, 0x93e, 0x935,
0x93e, 0x92c, 0x94d, 0x930, 0x942, 0x938, 0x947, 0x932, 0x94d, 0x938, 0x92c, 0x941,
0x916, 0x93c, 0x93e, 0x930, 0x947, 0x938, 0x94d, 0x91f, 0x92c, 0x941, 0x921, 0x93e,
0x92a, 0x947, 0x938, 0x94d, 0x91f, 0x92c, 0x94d, 0x92f, 0x942, 0x938, 0x93f, 0x928,
0x91c, 0x947, 0x928, 0x91a, 0x93f, 0x938, 0x940, 0x928, 0x93e, 0x909, 0x915, 0x94b,
0x92a, 0x947, 0x928, 0x939, 0x947, 0x917, 0x928, 0x921, 0x92c, 0x932, 0x93f, 0x928,
0x91c, 0x93f, 0x92c, 0x94d, 0x930, 0x93e, 0x932, 0x94d, 0x91f, 0x930, 0x917, 0x930,
0x94d, 0x928, 0x938, 0x940, 0x939, 0x947, 0x932, 0x938, 0x93f, 0x902, 0x915, 0x940,
0x906, 0x907, 0x932, 0x20, 0x911, 0x92b, 0x93c, 0x20, 0x92e, 0x948, 0x928, 0x907,
0x938, 0x94d, 0x924, 0x93e, 0x902, 0x92c, 0x941, 0x932, 0x91c, 0x930, 0x94d, 0x938,
0x940, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x93f, 0x928, 0x917, 0x94d, 0x930, 0x93e,
0x921, 0x915, 0x93f, 0x930, 0x94b, 0x935, 0x915, 0x940, 0x935, 0x932, 0x94d, 0x92f,
0x942, 0x92c, 0x947, 0x932, 0x91c, 0x93e, 0x928, 0x93e, 0x932, 0x902, 0x926, 0x928,
0x932, 0x915, 0x94d, 0x91c, 0x93c, 0x92e, 0x92c, 0x930, 0x94d, 0x917, 0x92e, 0x948,
0x921, 0x94d, 0x930, 0x93f, 0x921, 0x92e, 0x93e, 0x930, 0x93f, 0x92f, 0x93e, 0x939,
0x948, 0x92e, 0x92e, 0x93f, 0x902, 0x938, 0x94d, 0x915, 0x92e, 0x94b, 0x928, 0x93e,
0x915, 0x94b, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x913, 0x938, 0x94d, 0x932,
0x94b, 0x92a, 0x94b, 0x921, 0x94d, 0x917, 0x94b, 0x930, 0x93f, 0x915, 0x93e, 0x92a,
0x94d, 0x930, 0x93e, 0x917, 0x930, 0x940, 0x917, 0x93e, 0x938, 0x92e, 0x93e, 0x930,
0x93e, 0x938, 0x948, 0x928, 0x20, 0x92e, 0x93e, 0x930, 0x940, 0x928, 0x94b, 0x938,
0x93e, 0x930, 0x93e, 0x91c, 0x947, 0x935, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x924,
0x94b, 0x935, 0x938, 0x93f, 0x92e, 0x94d, 0x92b, 0x93c, 0x947, 0x930, 0x94b, 0x92a,
0x94b, 0x932, 0x938, 0x94d, 0x915, 0x94b, 0x92a, 0x94d, 0x91c, 0x947, 0x938, 0x94b,
0x92b, 0x93c, 0x93f, 0x92f, 0x93e, 0x938, 0x94d, 0x91f, 0x949, 0x915, 0x939, 0x94b,
0x92e, 0x924, 0x947, 0x932, 0x93f, 0x928, 0x91f, 0x93e, 0x907, 0x930, 0x947, 0x928,
0x909, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94b, 0x935, 0x94d, 0x938, 0x94d, 0x915,
0x935, 0x93e, 0x926, 0x941, 0x91c, 0x93c, 0x935, 0x947, 0x91f, 0x93f, 0x915, 0x928,
0x935, 0x93f, 0x90f, 0x928, 0x93e, 0x935, 0x93f, 0x932, 0x94d, 0x928, 0x93f, 0x92f,
0x938, 0x935, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921,
0x935, 0x949, 0x930, 0x938, 0x949, 0x91c, 0x93c, 0x93e, 0x917, 0x94d, 0x930, 0x947,
0x92c, 0x91c, 0x93c, 0x94d, 0x92f, 0x942, 0x930, 0x93f, 0x916, 0x93c, 0x90f, 0x902,
0x91f, 0x93e, 0x928, 0x93e, 0x928, 0x930, 0x940, 0x935, 0x94b, 0x936, 0x93e, 0x917,
0x94b, 0x938, 0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938, 0x915, 0x94b, 0x915,
0x94b, 0x938, 0x915, 0x94b, 0x92e, 0x94b, 0x930, 0x94b, 0x915, 0x930, 0x917, 0x941,
0x932, 0x947, 0x928, 0x92e, 0x93e, 0x932, 0x926, 0x940, 0x935, 0x92e, 0x949, 0x930,
0x940, 0x936, 0x938, 0x92e, 0x93e, 0x92f, 0x94b, 0x924, 0x94d, 0x924, 0x947, 0x930,
0x940, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928, 0x90f, 0x92a, 0x93f, 0x92f, 0x93e,
0x911, 0x915, 0x932, 0x948, 0x902, 0x921, 0x92c, 0x94b, 0x917, 0x928, 0x935, 0x93f,
0x932, 0x947, 0x91a, 0x948, 0x925, 0x92e, 0x91a, 0x915, 0x908, 0x938, 0x94d, 0x91f,
0x930, 0x90f, 0x92b, 0x93c, 0x947, 0x91f, 0x92b, 0x93c, 0x93e, 0x915, 0x93e, 0x913,
0x92b, 0x93c, 0x94b, 0x92b, 0x93c, 0x93f, 0x91c, 0x940, 0x92b, 0x93c, 0x94d, 0x92f,
0x942, 0x928, 0x93e, 0x92b, 0x93c, 0x941, 0x91f, 0x940, 0x917, 0x947, 0x932, 0x93e,
0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x917, 0x948, 0x902, 0x92c, 0x93f, 0x92f, 0x930,
0x917, 0x94d, 0x935, 0x93e, 0x921, 0x932, 0x915, 0x928, 0x93e, 0x932, 0x939, 0x94b,
0x928, 0x94b, 0x932, 0x941, 0x932, 0x941, 0x90f, 0x902, 0x921, 0x930, 0x92c, 0x930,
0x940, 0x915, 0x94b, 0x938, 0x930, 0x93e, 0x90f, 0x915, 0x94d, 0x935, 0x93e, 0x91c,
0x93c, 0x93e, 0x932, 0x940, 0x928, 0x92e, 0x93e, 0x91c, 0x941, 0x930, 0x94b, 0x92e,
0x93e, 0x930, 0x94d, 0x915, 0x947, 0x938, 0x938, 0x92e, 0x93f, 0x921, 0x935, 0x947,
0x928, 0x94c, 0x930, 0x941, 0x928, 0x940, 0x92f, 0x942, 0x928, 0x949, 0x930, 0x92b,
0x93c, 0x949, 0x915, 0x928, 0x94c, 0x92e, 0x93f, 0x92f, 0x93e, 0x92a, 0x93e, 0x917,
0x94b, 0x20, 0x92a, 0x93e, 0x917, 0x94b, 0x92a, 0x932, 0x93e, 0x90a, 0x92a, 0x94b,
0x928, 0x92a, 0x947, 0x908, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x92e, 0x94b,
0x930, 0x947, 0x938, 0x94d, 0x92c, 0x940, 0x930, 0x93e, 0x930, 0x94b, 0x91f, 0x94b,
0x902, 0x917, 0x93e, 0x938, 0x93e, 0x92f, 0x92a, 0x93e, 0x928, 0x924, 0x93e, 0x939,
0x93f, 0x924, 0x940, 0x91f, 0x93e, 0x930, 0x93e, 0x935, 0x93e, 0x91f, 0x94b, 0x902,
0x917, 0x93e, 0x91f, 0x93e, 0x92a, 0x942, 0x41, 0x73, 0x6d, 0x65, 0x72, 0x61,
0x53, 0x61, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0x65, 0x43, 0x6f, 0x72, 0x61,
0x6c, 0x20, 0x48, 0x61, 0x72, 0x62, 0x6f, 0x75, 0x72, 0x43, 0x69, 0x75,
0x64, 0x61, 0x64, 0x20, 0x4a, 0x75, 0x61, 0x72, 0x65, 0x7a, 0x43, 0x75,
0x72, 0x61, 0x63, 0x61, 0x6f, 0x53, 0x74, 0x20, 0x42, 0x61, 0x72, 0x74,
0x68, 0x65, 0x6c, 0x65, 0x6d, 0x79, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x44, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x41, 0x71, 0x74, 0x61,
0x75, 0x53, 0x61, 0x69, 0x67, 0x6f, 0x6e, 0x51, 0x6f, 0x73, 0x74, 0x61,
0x6e, 0x61, 0x79, 0x46, 0x61, 0x65, 0x72, 0x6f, 0x65, 0x52, 0x65, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x54, 0x72, 0x75, 0x6b, 0x41, 0x64, 0x64, 0x69,
0x73, 0x7a, 0x2d, 0x41, 0x62, 0x65, 0x62, 0x61, 0x41, 0x6c, 0x67, 0xed,
0x72, 0x41, 0x73, 0x7a, 0x6d, 0x61, 0x72, 0x61, 0x4b, 0x61, 0x69, 0x72,
0xf3, 0x44, 0x61, 0x72, 0x20, 0x65, 0x73, 0x2d, 0x53, 0x61, 0x6c, 0x61,
0x61, 0x6d, 0x44, 0x7a, 0x73, 0x69, 0x62, 0x75, 0x74, 0x69, 0x45, 0x6c,
0x2d, 0x41, 0x6a, 0xfa, 0x6e, 0x4b, 0x61, 0x72, 0x74, 0xfa, 0x6d, 0x4d,
0x61, 0x6c, 0x61, 0x62, 0xf3, 0x54, 0x75, 0x6e, 0x69, 0x73, 0x7a, 0x4b,
0x61, 0x6a, 0x6d, 0xe1, 0x6e, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0xf3, 0x76, 0xe1, 0x72, 0x6f,
0x73, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0xc9, 0x73, 0x7a,
0x61, 0x6b, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e,
0x74, 0x65, 0x72, 0x2c, 0x20, 0xc9, 0x73, 0x7a, 0x61, 0x6b, 0x2d, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0xc9, 0x73, 0x7a, 0x61, 0x6b, 0x2d, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x56, 0x6f, 0x73, 0x7a, 0x74, 0x6f, 0x6b, 0xc1,
0x64, 0x65, 0x6e, 0x42, 0x69, 0x73, 0x6b, 0x65, 0x6b, 0x43, 0x73, 0x69,
0x74, 0x61, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x7a, 0x6b, 0x75, 0x73, 0x7a,
0x44, 0x61, 0x6b, 0x6b, 0x61, 0x44, 0x75, 0x73, 0x61, 0x6e, 0x62, 0x65,
0x47, 0xe1, 0x7a, 0x61, 0x48, 0x6f, 0x20, 0x53, 0x69, 0x20, 0x4d, 0x69,
0x6e, 0x68, 0x2d, 0x76, 0xe1, 0x72, 0x6f, 0x73, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x7a, 0x6b, 0x4a, 0x65, 0x72, 0x75, 0x7a, 0x73, 0xe1, 0x6c,
0x65, 0x6d, 0x4b, 0x61, 0x6d, 0x63, 0x73, 0x61, 0x74, 0x6b, 0x61, 0x4b,
0x61, 0x72, 0x61, 0x63, 0x73, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x7a, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x7a, 0x6b, 0x4b, 0x75, 0x63, 0x73, 0x65,
0x6e, 0x67, 0x4d, 0x61, 0x6b, 0x61, 0xf3, 0x4d, 0x61, 0x73, 0x7a, 0x6b,
0x61, 0x74, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x79, 0x65,
0x63, 0x6b, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x7a, 0x69, 0x62, 0x69, 0x72,
0x73, 0x7a, 0x6b, 0x4f, 0x6d, 0x73, 0x7a, 0x6b, 0x50, 0x68, 0x65, 0x6e,
0x6a, 0x61, 0x6e, 0x4b, 0x6f, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x61, 0x6a,
0x53, 0x7a, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x7a, 0x61, 0x6d,
0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64, 0x53, 0x7a, 0xf6, 0x75, 0x6c, 0x53,
0x61, 0x6e, 0x67, 0x68, 0x61, 0x6a, 0x53, 0x7a, 0x69, 0x6e, 0x67, 0x61,
0x70, 0xfa, 0x72, 0x53, 0x7a, 0x72, 0x65, 0x64, 0x6e, 0x65, 0x6b, 0x6f,
0x6c, 0x69, 0x6d, 0x73, 0x7a, 0x6b, 0x54, 0x61, 0x73, 0x6b, 0x65, 0x6e,
0x74, 0x54, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x7a, 0x69, 0x54, 0x69, 0x6d,
0x70, 0x75, 0x54, 0x6f, 0x6b, 0x69, 0xf3, 0x54, 0x6f, 0x6d, 0x73, 0x7a,
0x6b, 0x55, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x6f, 0x72, 0xdc, 0x72,
0xfc, 0x6d, 0x63, 0x73, 0x69, 0x55, 0x73, 0x7a, 0x74, 0x79, 0x2d, 0x4e,
0x79, 0x65, 0x72, 0x61, 0x56, 0x6c, 0x61, 0x67, 0x79, 0x69, 0x76, 0x6f,
0x73, 0x7a, 0x74, 0x6f, 0x6b, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x7a,
0x6b, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x79, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x4a, 0x65, 0x72, 0x65, 0x76, 0xe1, 0x6e, 0x41, 0x7a,
0x6f, 0x72, 0x69, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x4b, 0x61, 0x6e, 0xe1, 0x72, 0x69, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x65, 0x6b, 0x5a, 0xf6, 0x6c, 0x64, 0x2d, 0x66, 0x6f, 0x6b, 0x69,
0x20, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x46, 0x65, 0x72,
0xf6, 0x65, 0x72, 0x44, 0xe9, 0x6c, 0x69, 0x2d, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x53, 0x7a, 0x65, 0x6e, 0x74, 0x20, 0x49, 0x6c, 0x6f,
0x6e, 0x61, 0x41, 0x6d, 0x73, 0x7a, 0x74, 0x65, 0x72, 0x64, 0x61, 0x6d,
0x41, 0x73, 0x7a, 0x74, 0x72, 0x61, 0x68, 0xe1, 0x6e, 0x42, 0x65, 0x6c,
0x67, 0x72, 0xe1, 0x64, 0x50, 0x6f, 0x7a, 0x73, 0x6f, 0x6e, 0x79, 0x42,
0x72, 0xfc, 0x73, 0x73, 0x7a, 0x65, 0x6c, 0x4b, 0x6f, 0x70, 0x70, 0x65,
0x6e, 0x68, 0xe1, 0x67, 0x61, 0x47, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74,
0xe1, 0x72, 0x4d, 0x61, 0x6e, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x49, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x4b, 0x61, 0x6c,
0x69, 0x6e, 0x79, 0x69, 0x6e, 0x67, 0x72, 0xe1, 0x64, 0x4c, 0x69, 0x73,
0x73, 0x7a, 0x61, 0x62, 0x6f, 0x6e, 0x4d, 0xe1, 0x6c, 0x74, 0x61, 0x4d,
0x69, 0x6e, 0x73, 0x7a, 0x6b, 0x4d, 0x6f, 0x73, 0x7a, 0x6b, 0x76, 0x61,
0x50, 0xe1, 0x72, 0x69, 0x7a, 0x73, 0x50, 0x72, 0xe1, 0x67, 0x61, 0x52,
0xf3, 0x6d, 0x61, 0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x7a,
0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0xf3, 0x53, 0x7a, 0x61, 0x72, 0x61,
0x74, 0x6f, 0x76, 0x53, 0x7a, 0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70,
0x6f, 0x6c, 0x53, 0x7a, 0x6b, 0x6f, 0x70, 0x6a, 0x65, 0x53, 0x7a, 0xf3,
0x66, 0x69, 0x61, 0x55, 0x6c, 0x6a, 0x61, 0x6e, 0x6f, 0x76, 0x73, 0x7a,
0x6b, 0x42, 0xe9, 0x63, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0xe1, 0x64, 0x56, 0x61, 0x72, 0x73, 0xf3, 0x5a, 0xe1, 0x67, 0x72, 0xe1,
0x62, 0x4b, 0x61, 0x72, 0xe1, 0x63, 0x73, 0x6f, 0x6e, 0x79, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x4b, 0xf3, 0x6b, 0x75, 0x73, 0x7a, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0xf3,
0x4d, 0x61, 0x6c, 0x64, 0xed, 0x76, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x65, 0x6b, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x48, 0xfa, 0x73, 0x76, 0xe9,
0x74, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x46, 0x69,
0x64, 0x7a, 0x73, 0x69, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f,
0x73, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x74, 0x69,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x4b, 0x77,
0x61, 0x6a, 0x61, 0x6c, 0x65, 0x69, 0x6e, 0x2d, 0x7a, 0xe1, 0x74, 0x6f,
0x6e, 0x79, 0x4d, 0x61, 0x6a, 0x75, 0x72, 0x6f, 0x2d, 0x7a, 0xe1, 0x74,
0x6f, 0x6e, 0x79, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x4d, 0x69, 0x64,
0x77, 0x61, 0x79, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x2d, 0x73, 0x7a, 0x69,
0x67, 0x65, 0x74, 0x65, 0x6b, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x57, 0x61, 0x6b, 0x65,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x41, 0x6c, 0x67, 0x65, 0x69,
0x72, 0x73, 0x62, 0x6f, 0x72, 0x67, 0x42, 0x69, 0x73, 0x73, 0xe1, 0x4b,
0x61, 0xed, 0x72, 0xf3, 0x44, 0x6a, 0x69, 0x62, 0xfa, 0x74, 0xed, 0x4a,
0xf3, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x61, 0x72, 0x62, 0x6f, 0x72,
0x67, 0x53, 0x61, 0xf3, 0x20, 0x54, 0xf3, 0x6d, 0x65, 0x54, 0x72, 0xed,
0x70, 0xf3, 0x6c, 0xed, 0x54, 0xfa, 0x6e, 0x69, 0x73, 0x62, 0x6f, 0x72,
0x67, 0x41, 0x6e, 0x67, 0x76, 0x69, 0x6c, 0x6c, 0x61, 0x41, 0x6e, 0x74,
0xed, 0x67, 0x76, 0x61, 0x41, 0x72, 0xfa, 0x62, 0x61, 0x42, 0x65, 0x6c,
0xed, 0x73, 0x4b, 0x61, 0x6e, 0x6b, 0xfa, 0x6e, 0x43, 0x61, 0x79, 0x6d,
0x61, 0x6e, 0x2d, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x4b, 0x6f, 0x73, 0x74,
0x61, 0x72, 0xed, 0x6b, 0x61, 0x44, 0xf3, 0x6d, 0x69, 0x6e, 0xed, 0x6b,
0x61, 0x47, 0x76, 0x61, 0x64, 0x65, 0x6c, 0xfa, 0x70, 0x47, 0x76, 0xe6,
0x6a, 0x61, 0x6e, 0x61, 0x4a, 0x61, 0x6d, 0x61, 0xed, 0x6b, 0x61, 0x4d,
0x61, 0x72, 0x74, 0x69, 0x6e, 0xed, 0x6b, 0x4d, 0x65, 0x78, 0xed, 0x6b,
0xf3, 0x62, 0x6f, 0x72, 0x67, 0x50, 0xfa, 0x65, 0x72, 0x74, 0xf3, 0x20,
0x52, 0xed, 0x6b, 0xf3, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x42,
0x61, 0x72, 0x74, 0xf3, 0x6c, 0xf3, 0x6d, 0x65, 0x75, 0x73, 0x61, 0x72,
0x65, 0x79, 0x6a, 0x61, 0x72, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20,
0x4b, 0x69, 0x74, 0x74, 0x73, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20,
0x4c, 0xfa, 0x73, 0xed, 0x61, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20,
0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69,
0x20, 0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x54, 0x6f, 0x72, 0x74,
0xf3, 0x6c, 0x61, 0x42, 0x65, 0x69, 0x72, 0xfa, 0x74, 0x42, 0x72, 0xfa,
0x6e, 0x65, 0x69, 0x4b, 0xf3, 0x6c, 0x6f, 0x6d, 0x62, 0xf3, 0x48, 0x6f,
0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x2d, 0x62, 0x6f,
0x72, 0x67, 0x4a, 0x65, 0x72, 0xfa, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x4b,
0x61, 0x62, 0xfa, 0x6c, 0x4b, 0x61, 0x6c, 0x6b, 0xfa, 0x74, 0x74, 0x61,
0x4b, 0xfa, 0x61, 0x6c, 0x61, 0x20, 0x4c, 0xfa, 0x6d, 0x70, 0xfa, 0x72,
0x4b, 0xfa, 0x76, 0x65, 0x69, 0x74, 0x4e, 0xed, 0x6b, 0xf3, 0x73, 0xed,
0x61, 0x52, 0xed, 0x6a, 0x61, 0x64, 0x53, 0x6a, 0x61, 0x6e, 0x67, 0x68,
0xe6, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0xfa, 0x72, 0x54, 0xf3, 0x6b,
0xfd, 0xf3, 0xda, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x42, 0x65, 0x72,
0x6d, 0xfa, 0x64, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0xed, 0x65, 0x79,
0x6a, 0x61, 0x72, 0x47, 0x72, 0xe6, 0x6e, 0x68, 0xf6, 0x66, 0xf0, 0x61,
0x65, 0x79, 0x6a, 0x61, 0x72, 0x46, 0xe6, 0x72, 0x65, 0x79, 0x6a, 0x61,
0x72, 0x53, 0x75, 0xf0, 0x75, 0x72, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67,
0xed, 0x61, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x48, 0x65, 0x6c,
0x65, 0x6e, 0x61, 0x41, 0xfe, 0x65, 0x6e, 0x61, 0x42, 0xfa, 0x6b, 0x61,
0x72, 0x65, 0x73, 0x74, 0x42, 0xfa, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74,
0x4b, 0x61, 0x75, 0x70, 0x6d, 0x61, 0x6e, 0x6e, 0x61, 0x68, 0xf6, 0x66,
0x6e, 0x47, 0xed, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x4d, 0xf6,
0x6e, 0x49, 0x73, 0x74, 0x61, 0x6e, 0x62, 0xfa, 0x6c, 0x4b, 0xe6, 0x6e,
0x75, 0x67, 0x61, 0x72, 0xf0, 0x75, 0x72, 0x4c, 0x75, 0x6e, 0x64, 0xfa,
0x6e, 0x69, 0x72, 0x4c, 0xfa, 0x78, 0x65, 0x6d, 0x62, 0x6f, 0x72, 0x67,
0x4d, 0x61, 0x64, 0x72, 0xed, 0x64, 0x4d, 0x61, 0x72, 0xed, 0x75, 0x68,
0xf6, 0x66, 0x6e, 0x4d, 0xf3, 0x6e, 0x61, 0x6b, 0xf3, 0x4f, 0x73, 0x6c,
0xf3, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0xed, 0x6e, 0xf3, 0x53,
0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0xf3, 0x53, 0xf3, 0x66, 0xed, 0x61,
0x53, 0x74, 0x6f, 0x6b, 0x6b, 0x68, 0xf3, 0x6c, 0x6d, 0x75, 0x72, 0x54,
0xed, 0x72, 0x61, 0x6e, 0x61, 0x56, 0x61, 0x74, 0xed, 0x6b, 0x61, 0x6e,
0x69, 0xf0, 0x56, 0xed, 0x6e, 0x56, 0x69, 0x6c, 0x6e, 0xed, 0x75, 0x73,
0x56, 0x61, 0x72, 0x73, 0x6a, 0xe1, 0x4a, 0xf3, 0x6c, 0x61, 0x65, 0x79,
0x4b, 0xf3, 0x6b, 0x6f, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x4d, 0x61,
0x6c, 0x64, 0xed, 0x76, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x4d, 0xe1, 0x72,
0x69, 0x74, 0xed, 0x75, 0x73, 0x50, 0xe1, 0x73, 0x6b, 0x61, 0x65, 0x79,
0x6a, 0x61, 0x46, 0xed, 0x64, 0x6a, 0xed, 0x4d, 0x61, 0x72, 0x71, 0x75,
0x65, 0x73, 0x61, 0x73, 0x2d, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x4e, 0xe1,
0x72, 0xfa, 0x50, 0x61, 0x6c, 0xe1, 0x54, 0x61, 0x68, 0xed, 0x74, 0xed,
0x41, 0x6c, 0x6a, 0x69, 0x72, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x55, 0x74, 0x61, 0x72,
0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x4e, 0x65, 0x77,
0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x61, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x41, 0x74, 0x68,
0x65, 0x6e, 0x61, 0x4b, 0x69, 0x73, 0x68, 0x69, 0x6e, 0x65, 0x76, 0x50,
0x75, 0x6c, 0x61, 0x75, 0x20, 0x4d, 0x61, 0x6e, 0x4d, 0x6f, 0x73, 0x6b,
0x77, 0x61, 0x57, 0x61, 0x72, 0x73, 0x61, 0x77, 0x61, 0x4d, 0x61, 0x6c,
0x61, 0x64, 0x65, 0x77, 0x61, 0x43, 0x69, 0x74, 0x61, 0x74, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x4e, 0x6f, 0x76,
0x65, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x53, 0x61, 0x6e, 0x63, 0x74, 0x65,
0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x6f,
0x53, 0x61, 0x6e, 0x63, 0x74, 0x65, 0x20, 0x4a, 0x6f, 0x68, 0x61, 0x6e,
0x6e, 0x65, 0x73, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61,
0x6e, 0x6f, 0x76, 0x61, 0x53, 0x61, 0x6e, 0x63, 0x74, 0x65, 0x20, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x70, 0x68, 0x6f, 0x72, 0x6f, 0x53,
0x61, 0x6e, 0x63, 0x74, 0x65, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53,
0x61, 0x6e, 0x63, 0x74, 0x65, 0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73,
0x53, 0x61, 0x6e, 0x63, 0x74, 0x65, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65,
0x6e, 0x74, 0x65, 0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x41, 0x74, 0x68, 0x65, 0x6e, 0x61, 0x73, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x69, 0x73,
0x62, 0x6f, 0x6e, 0x61, 0x4e, 0x61, 0x74, 0x61, 0x6c, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x6f, 0x4d, 0x61, 0x79, 0x6f, 0x74, 0x74, 0x61,
0x50, 0x61, 0x73, 0x63, 0x68, 0x61, 0x44, 0x61, 0x72, 0x2d, 0x65, 0x73,
0x2d, 0x53, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x45, 0x6c, 0x2d, 0x41, 0x61,
0x69, 0x75, 0x6e, 0x53, 0xe3, 0x6f, 0x2d, 0x54, 0x6f, 0x6d, 0xe9, 0x42,
0x75, 0x65, 0x6e, 0x6f, 0x73, 0x2d, 0x41, 0x69, 0x72, 0x65, 0x73, 0x4c,
0x61, 0x2d, 0x52, 0x69, 0x6f, 0x6a, 0x61, 0x52, 0x69, 0x6f, 0x2d, 0x47,
0x61, 0x6c, 0x6c, 0x65, 0x67, 0x6f, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x2d,
0x4a, 0x75, 0x61, 0x6e, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x4c, 0x75, 0x69,
0x73, 0x62, 0x61, 0x79, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x61, 0x73, 0x43, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67,
0x65, 0x2d, 0x42, 0x61, 0x79, 0x43, 0x61, 0x6d, 0x70, 0x6f, 0x2d, 0x47,
0x72, 0x61, 0x6e, 0x64, 0x65, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x2d,
0x4a, 0x75, 0x61, 0x72, 0x65, 0x7a, 0x43, 0x6f, 0x73, 0x74, 0x61, 0x2d,
0x52, 0x69, 0x63, 0x61, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x2d, 0x43,
0x72, 0x65, 0x65, 0x6b, 0x45, 0x6c, 0x2d, 0x53, 0x61, 0x6c, 0x76, 0x61,
0x64, 0x6f, 0x72, 0x47, 0x6c, 0x61, 0x63, 0x65, 0x2d, 0x42, 0x61, 0x79,
0x47, 0x6f, 0x6f, 0x73, 0x65, 0x2d, 0x42, 0x61, 0x79, 0x47, 0x72, 0x61,
0x6e, 0x64, 0x2d, 0x54, 0x75, 0x72, 0x6b, 0x47, 0x75, 0x61, 0x64, 0x65,
0x6c, 0x75, 0x70, 0x61, 0x54, 0x65, 0x6c, 0x6c, 0x2d, 0x43, 0x69, 0x74,
0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4c, 0x61,
0x2d, 0x50, 0x61, 0x7a, 0x4c, 0x6f, 0x73, 0x2d, 0x41, 0x6e, 0x67, 0x65,
0x6c, 0x65, 0x73, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x2d, 0x50, 0x72, 0x69,
0x6e, 0x63, 0x65, 0x2019, 0x73, 0x2d, 0x51, 0x75, 0x61, 0x72, 0x74, 0x65,
0x72, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x43, 0x69, 0x74,
0xe9, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4e, 0x6f, 0x72,
0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d,
0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d, 0x52, 0x69, 0x63, 0x6f, 0x52,
0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x2d, 0x49, 0x6e, 0x6c, 0x65, 0x74, 0x52,
0x69, 0x6f, 0x2d, 0x42, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x53, 0x61, 0x6e,
0x2d, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53, 0xe3, 0x6f, 0x2d,
0x50, 0x61, 0x75, 0x6c, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x42, 0x61,
0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x6f, 0x53, 0x74, 0x2e, 0x2d,
0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73, 0x53, 0x74, 0x2e, 0x2d, 0x4b, 0x69,
0x74, 0x74, 0x73, 0x53, 0x74, 0x2e, 0x2d, 0x4c, 0x75, 0x63, 0x69, 0x61,
0x53, 0x74, 0x2e, 0x2d, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x74,
0x2e, 0x2d, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x53, 0x77, 0x69,
0x66, 0x74, 0x2d, 0x43, 0x75, 0x72, 0x72, 0x65, 0x6e, 0x74, 0x50, 0x68,
0x6e, 0x6f, 0x6d, 0x2d, 0x50, 0x65, 0x6e, 0x68, 0x43, 0x61, 0x70, 0x2d,
0x56, 0x65, 0x72, 0x64, 0x69, 0x53, 0x75, 0x64, 0x2d, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x48, 0x65, 0x6c,
0x65, 0x6e, 0x61, 0x42, 0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x2d, 0x48, 0x69,
0x6c, 0x6c, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x48, 0x6f, 0x77, 0x65, 0x42, 0x75, 0x63, 0x75, 0x72, 0x65, 0x73,
0x74, 0x69, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x6e, 0x4d, 0x6f, 0x73, 0x63, 0x76, 0x61, 0x50, 0x61, 0x73, 0x63,
0x61, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62,
0x79, 0x43, 0x61, 0x74, 0x68, 0x61, 0x69, 0x72, 0x20, 0x6e, 0x61, 0x20,
0x68, 0x41, 0x69, 0x6c, 0x67, 0xe9, 0x69, 0x72, 0x65, 0x43, 0x61, 0x69,
0x72, 0x65, 0x6f, 0x43, 0x6f, 0x6e, 0x61, 0x63, 0x72, 0x61, 0xed, 0x44,
0x61, 0x63, 0xe1, 0x72, 0x44, 0xe1, 0x72, 0x61, 0x73, 0x61, 0x6c, 0xe1,
0x6d, 0x4c, 0xe1, 0xfa, 0x69, 0x6e, 0x65, 0x43, 0x61, 0x72, 0x74, 0xfa,
0x6d, 0x4d, 0x61, 0x70, 0xfa, 0x74, 0xf3, 0x4d, 0x6f, 0x67, 0x61, 0x69,
0x64, 0x69, 0x73, 0x69, 0xfa, 0x4e, 0x75, 0x61, 0x63, 0x73, 0x61, 0x74,
0x54, 0x72, 0x69, 0x70, 0x69, 0x6c, 0xed, 0x54, 0xfa, 0x69, 0x6e, 0x69,
0x73, 0x41, 0x6e, 0x67, 0x61, 0xed, 0x6c, 0x65, 0x42, 0x61, 0x72, 0x62,
0x61, 0x64, 0xf3, 0x73, 0x61, 0x6e, 0x20, 0x42, 0x68, 0x65, 0x69, 0x6c,
0xed, 0x73, 0x42, 0x6f, 0x67, 0x61, 0x74, 0xe1, 0x43, 0xf3, 0x73, 0x74,
0x61, 0x20, 0x52, 0xed, 0x63, 0x65, 0x43, 0xfa, 0x72, 0x61, 0x73, 0xf3,
0x61, 0x6e, 0x20, 0x74, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0xf3, 0x69,
0x72, 0x47, 0x72, 0x65, 0x61, 0x6e, 0xe1, 0x64, 0x61, 0x47, 0x75, 0x61,
0x64, 0x61, 0x6c, 0xfa, 0x69, 0x70, 0x61, 0x6e, 0x20, 0x47, 0x68, 0x75,
0xe1, 0x69, 0x6e, 0x49, 0x61, 0x6d, 0xe1, 0x69, 0x63, 0x65, 0x4c, 0xed,
0x6f, 0x6d, 0x61, 0x43, 0x61, 0x74, 0x68, 0x61, 0x69, 0x72, 0x20, 0x4d,
0x68, 0x65, 0x69, 0x63, 0x73, 0x69, 0x63, 0x65, 0x6f, 0x4e, 0x75, 0x61,
0x2d, 0x45, 0x61, 0x62, 0x68, 0x72, 0x61, 0x63, 0x50, 0xf3, 0x72, 0x74,
0xf3, 0x20, 0x52, 0xed, 0x63, 0x65, 0x53, 0x61, 0x6e, 0x20, 0x43, 0x72,
0xed, 0x6f, 0x73, 0x74, 0xf3, 0x69, 0x72, 0x53, 0x61, 0x6e, 0x20, 0x55,
0x69, 0x6e, 0x73, 0x65, 0x61, 0x6e, 0x6e, 0x49, 0x6e, 0x69, 0x73, 0x20,
0x54, 0x75, 0x69, 0x6c, 0x65, 0x4d, 0x61, 0x63, 0x20, 0x47, 0x75, 0x61,
0x69, 0x72, 0x65, 0xc1, 0x69, 0x64, 0x69, 0x6e, 0x42, 0x61, 0x69, 0x72,
0xe9, 0x69, 0x6e, 0x42, 0x61, 0x6b, 0x69, 0x42, 0xe9, 0x69, 0x72, 0x69,
0xfa, 0x74, 0x42, 0x72, 0xfa, 0x69, 0x6e, 0xe9, 0x61, 0x6e, 0x20, 0x44,
0x61, 0x6d, 0x61, 0x69, 0x73, 0x63, 0x48, 0x65, 0x61, 0x62, 0x72, 0xf3,
0x6e, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x43, 0x6f, 0x6e, 0x67, 0x49, 0x61,
0x63, 0xe1, 0x72, 0x74, 0x61, 0x49, 0x61, 0x72, 0xfa, 0x73, 0x61, 0x69,
0x6c, 0xe9, 0x69, 0x6d, 0x43, 0x61, 0x62, 0xfa, 0x6c, 0x43, 0x61, 0x6c,
0x63, 0xfa, 0x74, 0x61, 0x43, 0x75, 0xe1, 0x69, 0x74, 0x4d, 0x61, 0x63,
0x61, 0x73, 0x61, 0x72, 0x4d, 0x61, 0x69, 0x6e, 0x69, 0x6c, 0x65, 0x61,
0x6e, 0x20, 0x4e, 0x69, 0x6f, 0x63, 0xf3, 0x69, 0x73, 0x53, 0xfa, 0x6c,
0x53, 0x68, 0x61, 0x6e, 0x67, 0x2d, 0x68, 0x61, 0x69, 0x53, 0x69, 0x6e,
0x67, 0x65, 0x61, 0x70, 0xf3, 0x72, 0x54, 0xf3, 0x69, 0x63, 0x65, 0x6f,
0x49, 0x61, 0x63, 0xfa, 0x74, 0x73, 0x63, 0x45, 0x69, 0x72, 0x65, 0x61,
0x76, 0xe1, 0x6e, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0xf3, 0x69, 0x72,
0x42, 0x65, 0x69, 0x72, 0x6d, 0x69, 0xfa, 0x64, 0x61, 0x6e, 0x61, 0x20,
0x68, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61,
0x6e, 0xe1, 0x72, 0x61, 0x63, 0x68, 0x61, 0x52, 0x69, 0x6e, 0x6e, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e,
0x20, 0x46, 0x68, 0x61, 0x72, 0xf3, 0x4d, 0x61, 0x69, 0x64, 0xe9, 0x61,
0x72, 0x61, 0x52, 0xe9, 0x69, 0x63, 0x69, 0x76, 0xed, 0x63, 0x61, 0x6e,
0x20, 0x74, 0x53, 0x65, 0x6f, 0x69, 0x72, 0x73, 0x69, 0x61, 0x20, 0x54,
0x68, 0x65, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x48, 0xe9, 0x69, 0x6c,
0x69, 0x6e, 0x41, 0x6d, 0x73, 0x74, 0x61, 0x72, 0x64, 0x61, 0x6d, 0x41,
0x6e, 0x64, 0xf3, 0x72, 0x61, 0x61, 0x6e, 0x20, 0x41, 0x73, 0x74, 0x72,
0x61, 0x63, 0xe1, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x74, 0x68,
0x69, 0x6e, 0x42, 0xe9, 0x61, 0x6c, 0x67, 0x72, 0xe1, 0x64, 0x42, 0x65,
0x69, 0x72, 0x6c, 0xed, 0x6e, 0x61, 0x6e, 0x20, 0x42, 0x68, 0x72, 0x61,
0x74, 0x61, 0x73, 0x6c, 0xe1, 0x69, 0x76, 0x61, 0x6e, 0x20, 0x42, 0x68,
0x72, 0x75, 0x69, 0x73, 0xe9, 0x69, 0x6c, 0x42, 0xfa, 0x63, 0x61, 0x69,
0x72, 0x69, 0x73, 0x74, 0x42, 0xfa, 0x64, 0x61, 0x69, 0x70, 0x65, 0x69,
0x73, 0x74, 0x43, 0xed, 0x73, 0x69, 0x6e, 0x65, 0xe1, 0x43, 0xf3, 0x62,
0x61, 0x6e, 0x68, 0xe1, 0x76, 0x61, 0x6e, 0x42, 0x61, 0x69, 0x6c, 0x65,
0x20, 0xc1, 0x74, 0x68, 0x61, 0x20, 0x43, 0x6c, 0x69, 0x61, 0x74, 0x68,
0x47, 0x69, 0x6f, 0x62, 0x72, 0xe1, 0x6c, 0x74, 0x61, 0x72, 0x47, 0x65,
0x61, 0x6e, 0x73, 0x61, 0xed, 0x48, 0x65, 0x69, 0x6c, 0x73, 0x69, 0x6e,
0x63, 0xed, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x68, 0x61,
0x6e, 0x61, 0x6e, 0x6e, 0x49, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x62, 0xfa,
0x6c, 0x47, 0x65, 0x69, 0x72, 0x73, 0xed, 0x43, 0xed, 0x76, 0x4c, 0x69,
0x6f, 0x73, 0x70, 0xf3, 0x69, 0x6e, 0x4c, 0x69, 0xfa, 0x69, 0x62, 0x6c,
0x65, 0xe1, 0x6e, 0x61, 0x4c, 0x6f, 0x6e, 0x64, 0x61, 0x69, 0x6e, 0x4c,
0x75, 0x63, 0x73, 0x61, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x4d, 0x61, 0x69,
0x64, 0x72, 0x69, 0x64, 0x4d, 0x69, 0x6f, 0x6e, 0x73, 0x63, 0x4d, 0x6f,
0x6e, 0x61, 0x63, 0xf3, 0x4d, 0x6f, 0x73, 0x63, 0xf3, 0x50, 0xe1, 0x72,
0x61, 0x73, 0x50, 0x6f, 0x64, 0x67, 0x61, 0x69, 0x72, 0xed, 0x74, 0x73,
0x65, 0x52, 0xed, 0x67, 0x65, 0x61, 0x6e, 0x20, 0x52, 0xf3, 0x69, 0x6d,
0x68, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x69, 0x72, 0xed, 0x6e, 0x65,
0x53, 0x61, 0x69, 0x72, 0xe9, 0x61, 0x76, 0xf3, 0x53, 0x63, 0xf3, 0x69,
0x70, 0xe9, 0x53, 0xf3, 0x69, 0x66, 0x69, 0x61, 0x53, 0x74, 0xf3, 0x63,
0xf3, 0x6c, 0x6d, 0x54, 0x61, 0x69, 0x6c, 0x6c, 0x69, 0x6e, 0x6e, 0x54,
0x69, 0x6f, 0x72, 0xe1, 0x6e, 0x61, 0x56, 0x61, 0x64, 0xfa, 0x73, 0x61,
0x6e, 0x20, 0x56, 0x61, 0x74, 0x61, 0x63, 0xe1, 0x69, 0x6e, 0x56, 0x69,
0x6c, 0x6e, 0x69, 0x61, 0x73, 0x56, 0xe1, 0x72, 0x73, 0xe1, 0x53, 0xe1,
0x67, 0x72, 0x61, 0x62, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61,
0x69, 0x72, 0xed, 0x76, 0x65, 0x6f, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e,
0x20, 0x6e, 0x61, 0x20, 0x4e, 0x6f, 0x6c, 0x6c, 0x61, 0x67, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x6f, 0x6d, 0xf3, 0x72,
0x61, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x4d, 0x68, 0x61,
0x69, 0x6c, 0x64, 0xed, 0x76, 0x65, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e,
0x20, 0x4d, 0x68, 0x75, 0x69, 0x72, 0xed, 0x73, 0x46, 0x69, 0x64, 0x73,
0xed, 0x6e, 0x61, 0x20, 0x68, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e,
0x20, 0x4d, 0x68, 0x61, 0x72, 0x63, 0x61, 0x73, 0x61, 0x63, 0x68, 0x61,
0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x4d, 0x69, 0x64, 0x77,
0x61, 0x79, 0x54, 0x61, 0x69, 0x68, 0xed, 0x74, 0xed, 0x56, 0x61, 0x69,
0x6c, 0xed, 0x73, 0x41, 0x6c, 0x67, 0x65, 0x72, 0x69, 0x49, 0x6c, 0x20,
0x43, 0x61, 0x69, 0x72, 0x6f, 0x47, 0x69, 0x62, 0x75, 0x74, 0x69, 0x45,
0x6c, 0x20, 0x41, 0x79, 0x75, 0x6e, 0x47, 0x69, 0x75, 0x62, 0x61, 0x54,
0x75, 0x6e, 0x69, 0x73, 0x69, 0x43, 0x61, 0x69, 0x65, 0x6e, 0x6e, 0x61,
0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x75, 0x70, 0x61, 0x4c, 0x2019, 0x41,
0x76, 0x61, 0x6e, 0x61, 0x47, 0x69, 0x61, 0x6d, 0x61, 0x69, 0x63, 0x61,
0x43, 0x69, 0x74, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4d, 0x65,
0x73, 0x73, 0x69, 0x63, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x69,
0x63, 0x6f, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x6f, 0x6c, 0x6f, 0x53,
0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x2019, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x6f, 0x73,
0x74, 0x61, 0x47, 0x69, 0x61, 0x63, 0x61, 0x72, 0x74, 0x61, 0x47, 0x65,
0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x65, 0x41, 0x7a, 0x7a,
0x6f, 0x72, 0x72, 0x65, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0xe6,
0x72, 0x20, 0xd8, 0x65, 0x72, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x64, 0x53, 0x61, 0x6e, 0x74,
0x2019, 0x45, 0x6c, 0x65, 0x6e, 0x61, 0x43, 0x6f, 0x70, 0x65, 0x6e, 0x61,
0x67, 0x68, 0x65, 0x6e, 0x47, 0x69, 0x62, 0x69, 0x6c, 0x74, 0x65, 0x72,
0x72, 0x61, 0x49, 0x73, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d,
0x61, 0x6e, 0x4c, 0x75, 0x62, 0x69, 0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x6e,
0x64, 0x72, 0x61, 0x4c, 0x75, 0x73, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x6f, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x50, 0x61, 0x72, 0x69, 0x67,
0x69, 0x53, 0x69, 0x6e, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x69,
0x53, 0x74, 0x6f, 0x63, 0x63, 0x6f, 0x6c, 0x6d, 0x61, 0x43, 0x69, 0x74,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x56, 0x61, 0x74, 0x69, 0x63,
0x61, 0x6e, 0x6f, 0x56, 0x61, 0x72, 0x73, 0x61, 0x76, 0x69, 0x61, 0x5a,
0x61, 0x67, 0x61, 0x62, 0x72, 0x69, 0x61, 0x5a, 0x75, 0x72, 0x69, 0x67,
0x6f, 0x4c, 0x61, 0x20, 0x52, 0x69, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65,
0x46, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x73, 0x69,
0x30a2, 0x30d3, 0x30b8, 0x30e3, 0x30f3, 0x30a2, 0x30af, 0x30e9, 0x30a2, 0x30b8, 0x30b9, 0x30a2,
0x30d9, 0x30d0, 0x30a2, 0x30eb, 0x30b8, 0x30a7, 0x30a2, 0x30b9, 0x30de, 0x30e9, 0x30d0, 0x30de,
0x30b3, 0x30d0, 0x30f3, 0x30ae, 0x30d0, 0x30f3, 0x30b8, 0x30e5, 0x30fc, 0x30eb, 0x30d3, 0x30b5,
0x30a6, 0x30d6, 0x30e9, 0x30f3, 0x30bf, 0x30a4, 0x30e4, 0x30d6, 0x30e9, 0x30b6, 0x30d3, 0x30eb,
0x30d6, 0x30b8, 0x30e5, 0x30f3, 0x30d6, 0x30e9, 0x30ab, 0x30a4, 0x30ed, 0x30ab, 0x30b5, 0x30d6,
0x30e9, 0x30f3, 0x30ab, 0x30bb, 0x30a6, 0x30bf, 0x30b3, 0x30ca, 0x30af, 0x30ea, 0x30c0, 0x30ab,
0x30fc, 0x30eb, 0x30c0, 0x30eb, 0x30a8, 0x30b9, 0x30b5, 0x30e9, 0x30fc, 0x30e0, 0x30b8, 0x30d6,
0x30c1, 0x30c9, 0x30a5, 0x30a2, 0x30e9, 0x30a2, 0x30a4, 0x30a6, 0x30f3, 0x30d5, 0x30ea, 0x30fc,
0x30bf, 0x30a6, 0x30f3, 0x30cf, 0x30dc, 0x30ed, 0x30fc, 0x30cd, 0x30cf, 0x30e9, 0x30ec, 0x30e8,
0x30cf, 0x30cd, 0x30b9, 0x30d6, 0x30eb, 0x30b0, 0x30b8, 0x30e5, 0x30d0, 0x30ab, 0x30f3, 0x30d1,
0x30e9, 0x30cf, 0x30eb, 0x30c4, 0x30fc, 0x30e0, 0x30ad, 0x30ac, 0x30ea, 0x30ad, 0x30f3, 0x30b7,
0x30e3, 0x30b5, 0x30e9, 0x30b4, 0x30b9, 0x30ea, 0x30fc, 0x30d6, 0x30eb, 0x30f4, 0x30a3, 0x30eb,
0x30ed, 0x30e1, 0x30eb, 0x30a2, 0x30f3, 0x30c0, 0x30eb, 0x30d6, 0x30f3, 0x30d0, 0x30b7, 0x30eb,
0x30b5, 0x30ab, 0x30de, 0x30e9, 0x30dc, 0x30de, 0x30d7, 0x30c8, 0x30de, 0x30bb, 0x30eb, 0x30e0,
0x30d0, 0x30d0, 0x30fc, 0x30cd, 0x30e2, 0x30ac, 0x30c7, 0x30a3, 0x30b7, 0x30aa, 0x30e2, 0x30f3,
0x30ed, 0x30d3, 0x30a2, 0x30ca, 0x30a4, 0x30ed, 0x30d3, 0x30f3, 0x30b8, 0x30e3, 0x30e1, 0x30ca,
0x30cb, 0x30a2, 0x30e1, 0x30cc, 0x30a2, 0x30af, 0x30b7, 0x30e7, 0x30c3, 0x30c8, 0x30ef, 0x30ac,
0x30c9, 0x30a5, 0x30b0, 0x30fc, 0x30dd, 0x30eb, 0x30c8, 0x30ce, 0x30dc, 0x30b5, 0x30f3, 0x30c8,
0x30e1, 0x30c8, 0x30ea, 0x30dd, 0x30ea, 0x30c1, 0x30e5, 0x30cb, 0x30b9, 0x30a6, 0x30a3, 0x30f3,
0x30c8, 0x30d5, 0x30c3, 0x30af, 0x30a2, 0x30c0, 0x30c3, 0x30af, 0x30a2, 0x30f3, 0x30ab, 0x30ec,
0x30c3, 0x30b8, 0x30a2, 0x30f3, 0x30ae, 0x30e9, 0x30a2, 0x30f3, 0x30c6, 0x30a3, 0x30b0, 0x30a2,
0x30a2, 0x30e9, 0x30b0, 0x30a1, 0x30a4, 0x30ca, 0x30d6, 0x30a8, 0x30ce, 0x30b9, 0x30a2, 0x30a4,
0x30ec, 0x30b9, 0x30ab, 0x30bf, 0x30de, 0x30eb, 0x30ab, 0x30b3, 0x30eb, 0x30c9, 0x30d0, 0x30d5,
0x30d5, 0x30a4, 0x30e9, 0x30ea, 0x30aa, 0x30cf, 0x30e1, 0x30f3, 0x30c9, 0x30fc, 0x30b5, 0x30ea,
0x30aa, 0x30ac, 0x30b8, 0x30a7, 0x30b4, 0x30b9, 0x30b5, 0x30eb, 0x30bf, 0x30b5, 0x30f3, 0x30d5,
0x30a1, 0x30f3, 0x30b5, 0x30f3, 0x30eb, 0x30a4, 0x30b9, 0x30c8, 0x30a5, 0x30af, 0x30de, 0x30f3,
0x30a6, 0x30b7, 0x30e5, 0x30a2, 0x30a4, 0x30a2, 0x30a2, 0x30eb, 0x30d0, 0x30a2, 0x30b9, 0x30f3,
0x30b7, 0x30aa, 0x30f3, 0x30a2, 0x30c6, 0x30a3, 0x30b3, 0x30ab, 0x30f3, 0x30d0, 0x30a4, 0x30fc,
0x30a2, 0x30d0, 0x30a4, 0x30a2, 0x30d0, 0x30f3, 0x30c7, 0x30e9, 0x30d0, 0x30eb, 0x30d0, 0x30c9,
0x30b9, 0x30d9, 0x30ec, 0x30f3, 0x30d9, 0x30ea, 0x30fc, 0x30ba, 0x30d6, 0x30e9, 0x30f3, 0x30fb,
0x30b5, 0x30d6, 0x30ed, 0x30f3, 0x30dc, 0x30a2, 0x30d3, 0x30b9, 0x30bf, 0x30dc, 0x30b4, 0x30bf,
0x30dc, 0x30a4, 0x30b7, 0x30b1, 0x30f3, 0x30d6, 0x30ea, 0x30c3, 0x30b8, 0x30d9, 0x30a4, 0x30ab,
0x30f3, 0x30dd, 0x30b0, 0x30e9, 0x30f3, 0x30c7, 0x30ab, 0x30f3, 0x30af, 0x30f3, 0x30ab, 0x30e9,
0x30ab, 0x30b9, 0x30ab, 0x30a4, 0x30a8, 0x30f3, 0x30cc, 0x30b1, 0x30a4, 0x30de, 0x30f3, 0x30b7,
0x30ab, 0x30b4, 0x30c1, 0x30ef, 0x30ef, 0x30b7, 0x30a6, 0x30c0, 0x30fc, 0x30fb, 0x30d5, 0x30a2,
0x30ec, 0x30b9, 0x30b3, 0x30b9, 0x30bf, 0x30ea, 0x30ab, 0x30af, 0x30ec, 0x30b9, 0x30c8, 0x30f3,
0x30af, 0x30a4, 0x30a2, 0x30d0, 0x30ad, 0x30e5, 0x30e9, 0x30bd, 0x30fc, 0x30c7, 0x30f3, 0x30de,
0x30fc, 0x30af, 0x30b7, 0x30e3, 0x30a6, 0x30f3, 0x30c9, 0x30fc, 0x30bd, 0x30f3, 0x30c9, 0x30fc,
0x30bd, 0x30f3, 0x30af, 0x30ea, 0x30fc, 0x30af, 0x30c7, 0x30f3, 0x30d0, 0x30fc, 0x30c7, 0x30c8,
0x30ed, 0x30a4, 0x30c8, 0x30c9, 0x30df, 0x30cb, 0x30ab, 0x30a8, 0x30c9, 0x30e2, 0x30f3, 0x30c8,
0x30f3, 0x30a8, 0x30a4, 0x30eb, 0x30cd, 0x30da, 0x30a8, 0x30eb, 0x30b5, 0x30eb, 0x30d0, 0x30c9,
0x30eb, 0x30d5, 0x30a9, 0x30fc, 0x30c8, 0x30cd, 0x30eb, 0x30bd, 0x30f3, 0x30d5, 0x30a9, 0x30eb,
0x30bf, 0x30ec, 0x30b6, 0x30b0, 0x30ec, 0x30fc, 0x30b9, 0x30d9, 0x30a4, 0x30b0, 0x30fc, 0x30b9,
0x30d9, 0x30a4, 0x30b0, 0x30e9, 0x30f3, 0x30c9, 0x30bf, 0x30fc, 0x30af, 0x30b0, 0x30ec, 0x30ca,
0x30c0, 0x30b0, 0x30a2, 0x30c9, 0x30eb, 0x30fc, 0x30d7, 0x30b0, 0x30a2, 0x30c6, 0x30de, 0x30e9,
0x30b0, 0x30a2, 0x30e4, 0x30ad, 0x30eb, 0x30ac, 0x30a4, 0x30a2, 0x30ca, 0x30cf, 0x30ea, 0x30d5,
0x30a1, 0x30af, 0x30b9, 0x30cf, 0x30d0, 0x30ca, 0x30a8, 0x30eb, 0x30e2, 0x30b7, 0x30e8, 0x30a4,
0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x30dd, 0x30ea, 0x30b9, 0x30a4, 0x30f3, 0x30c7, 0x30a3,
0x30a2, 0x30ca, 0x5dde, 0x30ce, 0x30c3, 0x30af, 0x30b9, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2,
0x30ca, 0x5dde, 0x30de, 0x30ec, 0x30f3, 0x30b4, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca,
0x5dde, 0x30d4, 0x30fc, 0x30bf, 0x30fc, 0x30ba, 0x30d0, 0x30fc, 0x30b0, 0x30a4, 0x30f3, 0x30c7,
0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30c6, 0x30eb, 0x30b7, 0x30c6, 0x30a3, 0x30a4, 0x30f3, 0x30c7,
0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30d3, 0x30d9, 0x30fc, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2,
0x30ca, 0x5dde, 0x30d3, 0x30f3, 0x30bb, 0x30f3, 0x30b9, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2,
0x30ca, 0x5dde, 0x30a6, 0x30a3, 0x30ca, 0x30de, 0x30c3, 0x30af, 0x30a4, 0x30cc, 0x30f4, 0x30a3,
0x30af, 0x30a4, 0x30ab, 0x30eb, 0x30a4, 0x30c3, 0x30c8, 0x30b8, 0x30e3, 0x30de, 0x30a4, 0x30ab,
0x30b8, 0x30e5, 0x30ce, 0x30fc, 0x30eb, 0x30a4, 0x30d3, 0x30eb, 0x30b1, 0x30f3, 0x30bf, 0x30c3,
0x30ad, 0x30fc, 0x5dde, 0x30e2, 0x30f3, 0x30c6, 0x30a3, 0x30c1, 0x30a7, 0x30ed, 0x30af, 0x30e9,
0x30ec, 0x30f3, 0x30c0, 0x30a4, 0x30af, 0x30e9, 0x30d1, 0x30b9, 0x30ea, 0x30de, 0x30ed, 0x30b5,
0x30f3, 0x30bc, 0x30eb, 0x30b9, 0x30ed, 0x30fc, 0x30ef, 0x30fc, 0x30fb, 0x30d7, 0x30ea, 0x30f3,
0x30bb, 0x30ba, 0x30fb, 0x30af, 0x30a6, 0x30a9, 0x30fc, 0x30bf, 0x30fc, 0x30de, 0x30bb, 0x30a4,
0x30aa, 0x30de, 0x30ca, 0x30b0, 0x30a2, 0x30de, 0x30ca, 0x30a6, 0x30b9, 0x30de, 0x30ea, 0x30b4,
0x30de, 0x30eb, 0x30c6, 0x30a3, 0x30cb, 0x30fc, 0x30af, 0x30de, 0x30bf, 0x30e2, 0x30ed, 0x30b9,
0x30de, 0x30b5, 0x30c8, 0x30e9, 0x30f3, 0x30e1, 0x30ce, 0x30df, 0x30cb, 0x30fc, 0x30e1, 0x30ea,
0x30c0, 0x30e1, 0x30c8, 0x30e9, 0x30ab, 0x30c8, 0x30e9, 0x30e1, 0x30ad, 0x30b7, 0x30b3, 0x30b7,
0x30c6, 0x30a3, 0x30fc, 0x30df, 0x30af, 0x30ed, 0x30f3, 0x5cf6, 0x30e2, 0x30f3, 0x30af, 0x30c8,
0x30f3, 0x30e2, 0x30f3, 0x30c6, 0x30ec, 0x30a4, 0x30e2, 0x30f3, 0x30c6, 0x30d3, 0x30c7, 0x30aa,
0x30e2, 0x30f3, 0x30c8, 0x30bb, 0x30e9, 0x30c8, 0x30ca, 0x30c3, 0x30bd, 0x30fc, 0x30cb, 0x30e5,
0x30fc, 0x30e8, 0x30fc, 0x30af, 0x30ce, 0x30fc, 0x30e0, 0x30ce, 0x30ed, 0x30fc, 0x30cb, 0x30e3,
0x30ce, 0x30fc, 0x30b9, 0x30c0, 0x30b3, 0x30bf, 0x5dde, 0x30d3, 0x30e5, 0x30fc, 0x30e9, 0x30fc,
0x30ce, 0x30fc, 0x30b9, 0x30c0, 0x30b3, 0x30bf, 0x5dde, 0x30bb, 0x30f3, 0x30bf, 0x30fc, 0x30ce,
0x30fc, 0x30b9, 0x30c0, 0x30b3, 0x30bf, 0x5dde, 0x30cb, 0x30e5, 0x30fc, 0x30bb, 0x30fc, 0x30e9,
0x30e0, 0x30cc, 0x30fc, 0x30af, 0x30aa, 0x30d2, 0x30ca, 0x30ac, 0x30d1, 0x30ca, 0x30de, 0x30d1,
0x30e9, 0x30de, 0x30ea, 0x30dc, 0x30d5, 0x30a7, 0x30cb, 0x30c3, 0x30af, 0x30b9, 0x30dd, 0x30eb,
0x30c8, 0x30fc, 0x30d7, 0x30e9, 0x30f3, 0x30b9, 0x30dd, 0x30fc, 0x30c8, 0x30aa, 0x30d6, 0x30b9,
0x30da, 0x30a4, 0x30f3, 0x30dd, 0x30eb, 0x30c8, 0x30d9, 0x30fc, 0x30ea, 0x30e7, 0x30d7, 0x30a8,
0x30eb, 0x30c8, 0x30ea, 0x30b3, 0x30d7, 0x30f3, 0x30bf, 0x30a2, 0x30ec, 0x30ca, 0x30b9, 0x30e9,
0x30f3, 0x30ad, 0x30f3, 0x30a4, 0x30f3, 0x30ec, 0x30c3, 0x30c8, 0x30ec, 0x30b7, 0x30d5, 0x30a7,
0x30ec, 0x30b8, 0x30e3, 0x30a4, 0x30ca, 0x30ec, 0x30be, 0x30ea, 0x30e5, 0x30fc, 0x30c8, 0x30ea,
0x30aa, 0x30d6, 0x30e9, 0x30f3, 0x30b3, 0x30b5, 0x30f3, 0x30bf, 0x30ec, 0x30e0, 0x30b5, 0x30f3,
0x30c1, 0x30a2, 0x30b4, 0x30b5, 0x30f3, 0x30c8, 0x30c9, 0x30df, 0x30f3, 0x30b4, 0x30b5, 0x30f3,
0x30d1, 0x30a6, 0x30ed, 0x30a4, 0x30c8, 0x30b3, 0x30eb, 0x30c8, 0x30eb, 0x30df, 0x30c3, 0x30c8,
0x30b7, 0x30c8, 0x30ab, 0x30b5, 0x30f3, 0x30fb, 0x30d0, 0x30eb, 0x30c6, 0x30eb, 0x30df, 0x30fc,
0x30bb, 0x30f3, 0x30c8, 0x30b8, 0x30e7, 0x30f3, 0x30ba, 0x30bb, 0x30f3, 0x30c8, 0x30af, 0x30ea,
0x30b9, 0x30c8, 0x30d5, 0x30a1, 0x30fc, 0x30bb, 0x30f3, 0x30c8, 0x30eb, 0x30b7, 0x30a2, 0x30bb,
0x30f3, 0x30c8, 0x30c8, 0x30fc, 0x30de, 0x30b9, 0x30bb, 0x30f3, 0x30c8, 0x30d3, 0x30f3, 0x30bb,
0x30f3, 0x30c8, 0x30b9, 0x30a6, 0x30a3, 0x30d5, 0x30c8, 0x30ab, 0x30ec, 0x30f3, 0x30c8, 0x30c6,
0x30b0, 0x30b7, 0x30ac, 0x30eb, 0x30d1, 0x30c1, 0x30e5, 0x30fc, 0x30ec, 0x30c6, 0x30a3, 0x30d5,
0x30a2, 0x30ca, 0x30c8, 0x30ed, 0x30f3, 0x30c8, 0x30c8, 0x30eb, 0x30c8, 0x30fc, 0x30e9, 0x30d0,
0x30f3, 0x30af, 0x30fc, 0x30d0, 0x30fc, 0x30db, 0x30ef, 0x30a4, 0x30c8, 0x30db, 0x30fc, 0x30b9,
0x30a6, 0x30a3, 0x30cb, 0x30da, 0x30b0, 0x30e4, 0x30af, 0x30bf, 0x30c3, 0x30c8, 0x30b1, 0x30fc,
0x30b7, 0x30fc, 0x57fa, 0x5730, 0x30c7, 0x30fc, 0x30d3, 0x30b9, 0x57fa, 0x5730, 0x30c7, 0x30e5,
0x30e2, 0x30f3, 0x30fb, 0x30c7, 0x30e5, 0x30eb, 0x30f4, 0x30a3, 0x30eb, 0x57fa, 0x5730, 0x30de,
0x30c3, 0x30b3, 0x30ea, 0x30fc, 0x30e2, 0x30fc, 0x30bd, 0x30f3, 0x57fa, 0x5730, 0x30de, 0x30af,
0x30de, 0x30fc, 0x30c9, 0x57fa, 0x5730, 0x30d1, 0x30fc, 0x30de, 0x30fc, 0x57fa, 0x5730, 0x30ed,
0x30bc, 0x30e9, 0x57fa, 0x5730, 0x30c8, 0x30ed, 0x30eb, 0x57fa, 0x5730, 0x30dc, 0x30b9, 0x30c8,
0x30fc, 0x30af, 0x57fa, 0x5730, 0x30ed, 0x30f3, 0x30b0, 0x30a4, 0x30a7, 0x30fc, 0x30eb, 0x30d3,
0x30fc, 0x30f3, 0x30a2, 0x30c7, 0x30f3, 0x30a2, 0x30eb, 0x30de, 0x30c8, 0x30a4, 0x30a2, 0x30f3,
0x30de, 0x30f3, 0x30a2, 0x30ca, 0x30c7, 0x30a3, 0x30ea, 0x30a2, 0x30af, 0x30bf, 0x30a6, 0x30a2,
0x30af, 0x30c8, 0x30d9, 0x30a2, 0x30b7, 0x30ac, 0x30d0, 0x30fc, 0x30c9, 0x30a2, 0x30c6, 0x30a3,
0x30e9, 0x30a6, 0x30d0, 0x30b0, 0x30c0, 0x30c3, 0x30c9, 0x30d0, 0x30fc, 0x30ec, 0x30fc, 0x30f3,
0x30d0, 0x30af, 0x30fc, 0x30d0, 0x30f3, 0x30b3, 0x30af, 0x30d0, 0x30eb, 0x30ca, 0x30a6, 0x30eb,
0x30d9, 0x30a4, 0x30eb, 0x30fc, 0x30c8, 0x30d3, 0x30b7, 0x30e5, 0x30b1, 0x30af, 0x30d6, 0x30eb,
0x30cd, 0x30a4, 0x30c1, 0x30bf, 0x30b3, 0x30ed, 0x30f3, 0x30dc, 0x30c0, 0x30de, 0x30b9, 0x30ab,
0x30b9, 0x30c0, 0x30c3, 0x30ab, 0x30c9, 0x30d0, 0x30a4, 0x30c9, 0x30a5, 0x30b7, 0x30e3, 0x30f3,
0x30d9, 0x30d5, 0x30a1, 0x30de, 0x30b0, 0x30b9, 0x30bf, 0x30ac, 0x30b6, 0x30d8, 0x30d6, 0x30ed,
0x30f3, 0x30db, 0x30fc, 0x30c1, 0x30df, 0x30f3, 0x30db, 0x30d6, 0x30c9, 0x30a4, 0x30eb, 0x30af,
0x30fc, 0x30c4, 0x30af, 0x30b8, 0x30e3, 0x30ab, 0x30eb, 0x30bf, 0x30b8, 0x30e3, 0x30e4, 0x30d7,
0x30e9, 0x30a8, 0x30eb, 0x30b5, 0x30ec, 0x30e0, 0x30ab, 0x30d6, 0x30fc, 0x30eb, 0x30ab, 0x30e0,
0x30c1, 0x30e3, 0x30c3, 0x30ab, 0x30ab, 0x30e9, 0x30c1, 0x30ab, 0x30c8, 0x30de, 0x30f3, 0x30ba,
0x30cf, 0x30f3, 0x30c9, 0x30a5, 0x30a4, 0x30ac, 0x30b3, 0x30eb, 0x30ab, 0x30bf, 0x30af, 0x30e9,
0x30b9, 0x30ce, 0x30e4, 0x30eb, 0x30b9, 0x30af, 0x30af, 0x30a2, 0x30e9, 0x30eb, 0x30f3, 0x30d7,
0x30fc, 0x30eb, 0x30af, 0x30c1, 0x30f3, 0x30af, 0x30a6, 0x30a7, 0x30fc, 0x30c8, 0x30de, 0x30ab,
0x30aa, 0x30de, 0x30ac, 0x30c0, 0x30f3, 0x30de, 0x30ab, 0x30c3, 0x30b5, 0x30eb, 0x30de, 0x30cb,
0x30e9, 0x30de, 0x30b9, 0x30ab, 0x30c3, 0x30c8, 0x30cb, 0x30b3, 0x30b7, 0x30a2, 0x30ce, 0x30f4,
0x30a9, 0x30af, 0x30ba, 0x30cd, 0x30c4, 0x30af, 0x30ce, 0x30f4, 0x30a9, 0x30b7, 0x30d3, 0x30eb,
0x30b9, 0x30af, 0x30aa, 0x30e0, 0x30b9, 0x30af, 0x30aa, 0x30e9, 0x30eb, 0x30d7, 0x30ce, 0x30f3,
0x30da, 0x30f3, 0x30dd, 0x30f3, 0x30c6, 0x30a3, 0x30a2, 0x30ca, 0x30c3, 0x30af, 0x5e73, 0x58cc,
0x30ab, 0x30bf, 0x30fc, 0x30eb, 0x30b3, 0x30b9, 0x30bf, 0x30ca, 0x30a4, 0x30af, 0x30ba, 0x30ed,
0x30eb, 0x30c0, 0x30ea, 0x30e4, 0x30c9, 0x30b5, 0x30cf, 0x30ea, 0x30f3, 0x30b5, 0x30de, 0x30eb,
0x30ab, 0x30f3, 0x30c9, 0x30bd, 0x30a6, 0x30eb, 0x30b7, 0x30f3, 0x30ac, 0x30dd, 0x30fc, 0x30eb,
0x30b9, 0x30ec, 0x30c9, 0x30cd, 0x30b3, 0x30ea, 0x30e0, 0x30b9, 0x30af, 0x30bf, 0x30b7, 0x30b1,
0x30f3, 0x30c8, 0x30c8, 0x30d3, 0x30ea, 0x30b7, 0x30c6, 0x30d8, 0x30e9, 0x30f3, 0x30c6, 0x30a3,
0x30f3, 0x30d7, 0x30fc, 0x30c8, 0x30e0, 0x30b9, 0x30af, 0x30a6, 0x30e9, 0x30f3, 0x30d0, 0x30fc,
0x30c8, 0x30eb, 0x30a6, 0x30eb, 0x30e0, 0x30c1, 0x30a6, 0x30b9, 0x30c1, 0x30cd, 0x30e9, 0x30d3,
0x30a8, 0x30f3, 0x30c1, 0x30e3, 0x30f3, 0x30a6, 0x30e9, 0x30b8, 0x30aa, 0x30b9, 0x30c8, 0x30af,
0x30e4, 0x30af, 0x30fc, 0x30c4, 0x30af, 0x30e4, 0x30f3, 0x30b4, 0x30f3, 0x30a8, 0x30ab, 0x30c6,
0x30ea, 0x30f3, 0x30d6, 0x30eb, 0x30b0, 0x30a8, 0x30ec, 0x30d0, 0x30f3, 0x30a2, 0x30be, 0x30ec,
0x30b9, 0x30d0, 0x30df, 0x30e5, 0x30fc, 0x30c0, 0x30ab, 0x30ca, 0x30ea, 0x30a2, 0x30ab, 0x30fc,
0x30dc, 0x30d9, 0x30eb, 0x30c7, 0x30d5, 0x30a7, 0x30ed, 0x30fc, 0x30de, 0x30c7, 0x30a4, 0x30e9,
0x30ec, 0x30a4, 0x30ad, 0x30e3, 0x30d3, 0x30af, 0x30b5, 0x30a6, 0x30b9, 0x30b8, 0x30e7, 0x30fc,
0x30b8, 0x30a2, 0x30bb, 0x30f3, 0x30c8, 0x30d8, 0x30ec, 0x30ca, 0x30b9, 0x30bf, 0x30f3, 0x30ec,
0x30fc, 0x30a2, 0x30c7, 0x30ec, 0x30fc, 0x30c9, 0x30d6, 0x30ea, 0x30b9, 0x30d9, 0x30f3, 0x30d6,
0x30ed, 0x30fc, 0x30af, 0x30f3, 0x30d2, 0x30eb, 0x30c0, 0x30fc, 0x30a6, 0x30a3, 0x30f3, 0x30e6,
0x30fc, 0x30af, 0x30e9, 0x30db, 0x30d0, 0x30fc, 0x30c8, 0x30ea, 0x30f3, 0x30c7, 0x30de, 0x30f3,
0x30ed, 0x30fc, 0x30c9, 0x30cf, 0x30a6, 0x30e1, 0x30eb, 0x30dc, 0x30eb, 0x30f3, 0x30d1, 0x30fc,
0x30b9, 0x30b7, 0x30c9, 0x30cb, 0x30fc, 0x30a2, 0x30e0, 0x30b9, 0x30c6, 0x30eb, 0x30c0, 0x30e0,
0x30a2, 0x30f3, 0x30c9, 0x30e9, 0x30a2, 0x30b9, 0x30c8, 0x30e9, 0x30cf, 0x30f3, 0x30a2, 0x30c6,
0x30cd, 0x30d9, 0x30aa, 0x30b0, 0x30e9, 0x30fc, 0x30c9, 0x30d9, 0x30eb, 0x30ea, 0x30f3, 0x30d6,
0x30e9, 0x30c1, 0x30b9, 0x30e9, 0x30d0, 0x30d6, 0x30ea, 0x30e5, 0x30c3, 0x30bb, 0x30eb, 0x30d6,
0x30ab, 0x30ec, 0x30b9, 0x30c8, 0x30d6, 0x30c0, 0x30da, 0x30b9, 0x30c8, 0x30d3, 0x30e5, 0x30fc,
0x30b8, 0x30f3, 0x30b2, 0x30f3, 0x30ad, 0x30b7, 0x30ca, 0x30a6, 0x30b3, 0x30da, 0x30f3, 0x30cf,
0x30fc, 0x30b2, 0x30f3, 0x30c0, 0x30d6, 0x30ea, 0x30f3, 0x30b8, 0x30d6, 0x30e9, 0x30eb, 0x30bf,
0x30eb, 0x30ac, 0x30fc, 0x30f3, 0x30b8, 0x30fc, 0x30d8, 0x30eb, 0x30b7, 0x30f3, 0x30ad, 0x30de,
0x30f3, 0x5cf6, 0x30a4, 0x30b9, 0x30bf, 0x30f3, 0x30d6, 0x30fc, 0x30eb, 0x30b8, 0x30e3, 0x30fc,
0x30b8, 0x30fc, 0x30ab, 0x30ea, 0x30fc, 0x30cb, 0x30f3, 0x30b0, 0x30e9, 0x30fc, 0x30c9, 0x30ad,
0x30fc, 0x30ed, 0x30d5, 0x30ad, 0x30fc, 0x30a6, 0x30ea, 0x30b9, 0x30dc, 0x30f3, 0x30ea, 0x30e5,
0x30d6, 0x30ea, 0x30e3, 0x30ca, 0x30ed, 0x30f3, 0x30c9, 0x30f3, 0x30eb, 0x30af, 0x30bb, 0x30f3,
0x30d6, 0x30eb, 0x30af, 0x30de, 0x30c9, 0x30ea, 0x30fc, 0x30c9, 0x30de, 0x30eb, 0x30bf, 0x30de,
0x30ea, 0x30a8, 0x30cf, 0x30e0, 0x30f3, 0x30df, 0x30f3, 0x30b9, 0x30af, 0x30e2, 0x30ca, 0x30b3,
0x30e2, 0x30b9, 0x30af, 0x30ef, 0x30aa, 0x30b9, 0x30ed, 0x30d1, 0x30ea, 0x30dd, 0x30c9, 0x30b4,
0x30ea, 0x30c4, 0x30a1, 0x30d7, 0x30e9, 0x30cf, 0x30ea, 0x30ac, 0x30b5, 0x30de, 0x30e9, 0x30b5,
0x30f3, 0x30de, 0x30ea, 0x30ce, 0x30b5, 0x30e9, 0x30a8, 0x30dc, 0x30b5, 0x30e9, 0x30c8, 0x30d5,
0x30b7, 0x30f3, 0x30d5, 0x30a7, 0x30ed, 0x30dd, 0x30ea, 0x30b9, 0x30b3, 0x30d4, 0x30a8, 0x30bd,
0x30d5, 0x30a3, 0x30a2, 0x30b9, 0x30c8, 0x30c3, 0x30af, 0x30db, 0x30eb, 0x30e0, 0x30bf, 0x30ea,
0x30f3, 0x30c6, 0x30a3, 0x30e9, 0x30ca, 0x30a6, 0x30ea, 0x30e4, 0x30ce, 0x30d5, 0x30b9, 0x30af,
0x30d5, 0x30a1, 0x30c9, 0x30a5, 0x30fc, 0x30c4, 0x30d0, 0x30c1, 0x30ab, 0x30f3, 0x30a6, 0x30a3,
0x30fc, 0x30f3, 0x30f4, 0x30a3, 0x30ea, 0x30cb, 0x30e5, 0x30b9, 0x30dc, 0x30eb, 0x30b4, 0x30b0,
0x30e9, 0x30fc, 0x30c9, 0x30ef, 0x30eb, 0x30b7, 0x30e3, 0x30ef, 0x30b6, 0x30b0, 0x30ec, 0x30d6,
0x30c1, 0x30e5, 0x30fc, 0x30ea, 0x30c3, 0x30d2, 0x30a2, 0x30f3, 0x30bf, 0x30ca, 0x30ca, 0x30ea,
0x30dc, 0x30c1, 0x30e3, 0x30b4, 0x30b9, 0x30af, 0x30ea, 0x30b9, 0x30de, 0x30b9, 0x5cf6, 0x30b3,
0x30b3, 0x30b9, 0x8af8, 0x5cf6, 0x30b3, 0x30e2, 0x30ed, 0x30b1, 0x30eb, 0x30b2, 0x30ec, 0x30f3,
0x8af8, 0x5cf6, 0x30de, 0x30d8, 0x30e2, 0x30eb, 0x30c7, 0x30a3, 0x30d6, 0x30e2, 0x30fc, 0x30ea,
0x30b7, 0x30e3, 0x30b9, 0x30de, 0x30e8, 0x30c3, 0x30c8, 0x30ec, 0x30e6, 0x30cb, 0x30aa, 0x30f3,
0x30a2, 0x30d4, 0x30a2, 0x30aa, 0x30fc, 0x30af, 0x30e9, 0x30f3, 0x30c9, 0x30d6, 0x30fc, 0x30b2,
0x30f3, 0x30d3, 0x30eb, 0x30c1, 0x30e3, 0x30bf, 0x30e0, 0x30c1, 0x30e5, 0x30fc, 0x30af, 0x30a4,
0x30fc, 0x30b9, 0x30bf, 0x30fc, 0x5cf6, 0x30a8, 0x30d5, 0x30a7, 0x30c6, 0x5cf6, 0x30d5, 0x30a1,
0x30ab, 0x30aa, 0x30d5, 0x30a9, 0x30d5, 0x30a3, 0x30b8, 0x30fc, 0x30d5, 0x30ca, 0x30d5, 0x30c6,
0x30a3, 0x30ac, 0x30e9, 0x30d1, 0x30b4, 0x30b9, 0x30ac, 0x30f3, 0x30d3, 0x30a8, 0x8af8, 0x5cf6,
0x30ac, 0x30c0, 0x30eb, 0x30ab, 0x30ca, 0x30eb, 0x30b0, 0x30a2, 0x30e0, 0x30db, 0x30ce, 0x30eb,
0x30eb, 0x30a8, 0x30f3, 0x30c0, 0x30fc, 0x30d9, 0x30ea, 0x30fc, 0x5cf6, 0x30ad, 0x30ea, 0x30b9,
0x30a3, 0x30de, 0x30b9, 0x30a3, 0x5cf6, 0x30b3, 0x30b9, 0x30e9, 0x30a8, 0x30af, 0x30a7, 0x30bc,
0x30ea, 0x30f3, 0x30de, 0x30b8, 0x30e5, 0x30ed, 0x30de, 0x30eb, 0x30ad, 0x30fc, 0x30ba, 0x30df,
0x30c3, 0x30c9, 0x30a6, 0x30a7, 0x30fc, 0x5cf6, 0x30cb, 0x30a6, 0x30a8, 0x30ce, 0x30fc, 0x30d5,
0x30a9, 0x30fc, 0x30af, 0x5cf6, 0x30cc, 0x30e1, 0x30a2, 0x30d1, 0x30b4, 0x30d1, 0x30b4, 0x30d1,
0x30e9, 0x30aa, 0x30d4, 0x30c8, 0x30b1, 0x30a2, 0x30f3, 0x8af8, 0x5cf6, 0x30dd, 0x30f3, 0x30da,
0x30a4, 0x5cf6, 0x30dd, 0x30fc, 0x30c8, 0x30e2, 0x30ec, 0x30b9, 0x30d3, 0x30fc, 0x30e9, 0x30ed,
0x30c8, 0x30f3, 0x30ac, 0x30b5, 0x30a4, 0x30d1, 0x30f3, 0x30bf, 0x30d2, 0x30c1, 0x30bf, 0x30e9,
0x30ef, 0x30c8, 0x30f3, 0x30ac, 0x30bf, 0x30d7, 0x30a6, 0x30a7, 0x30fc, 0x30af, 0x5cf6, 0x30a6,
0x30a9, 0x30ea, 0x30b9, 0x8af8, 0x5cf6, 0x4d, 0x65, 0x6e, 0x64, 0x6f, 0x73, 0x61,
0x42, 0x65, 0x6c, 0x69, 0x73, 0x65, 0x54, 0x65, 0x6c, 0x75, 0x6b, 0x20,
0x43, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x4b, 0x61, 0x79,
0x65, 0x6e, 0x6e, 0x65, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x4a,
0x75, 0xe1, 0x72, 0x65, 0x73, 0x42, 0x65, 0x6e, 0x74, 0x65, 0x6e, 0x67,
0x20, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x54, 0x65, 0x6c, 0x75, 0x6b,
0x20, 0x47, 0x6c, 0x61, 0x63, 0x65, 0x54, 0x65, 0x6c, 0x75, 0x6b, 0x20,
0x47, 0x6f, 0x6f, 0x73, 0x65, 0x4b, 0x75, 0x74, 0x68, 0x6f, 0x20, 0x4d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4c, 0x6f, 0x72,
0x5d, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x5b, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x5d, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72, 0x20, 0x5b, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x5d, 0x50, 0x61, 0x6c, 0x61,
0x62, 0x75, 0x68, 0x61, 0x6e, 0x20, 0x53, 0x70, 0x61, 0x6e, 0x79, 0x6f,
0x6c, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68,
0x65, 0x6c, 0x65, 0x6d, 0x79, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4a,
0x6f, 0x68, 0x6e, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4b, 0x69, 0x74,
0x74, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x54, 0x68, 0x6f, 0x6d,
0x61, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x56, 0x69, 0x6e, 0x63,
0x65, 0x6e, 0x74, 0x41, 0x72, 0x75, 0x73, 0x20, 0x42, 0x61, 0x6e, 0x74,
0x65, 0x72, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x4b, 0x69,
0x64, 0x75, 0x6c, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4d, 0x61, 0x6e, 0x4c,
0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x6b, 0x4b, 0x68, 0x61,
0x67, 0x6f, 0x73, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x50, 0x65, 0x6c,
0x61, 0x62, 0x75, 0x68, 0x61, 0x6e, 0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73,
0x62, 0x79, 0x41, 0x64, 0x61, 0x6b, 0x69, 0x41, 0x6e, 0x6b, 0x6f, 0x72,
0x61, 0x6a, 0x69, 0x41, 0x6e, 0x67, 0x69, 0x6c, 0x61, 0x42, 0x61, 0xed,
0x61, 0x20, 0x64, 0x69, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61,
0x73, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64, 0x75, 0x73, 0x42, 0x65, 0x6c,
0x69, 0x7a, 0x69, 0x58, 0x69, 0x6b, 0x61, 0x67, 0x75, 0x58, 0x69, 0x75,
0xe1, 0x75, 0x61, 0x4b, 0x72, 0xe9, 0x73, 0x74, 0x6f, 0x6e, 0x4b, 0x75,
0x72, 0x61, 0x73, 0x61, 0x75, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x75,
0x70, 0x69, 0x47, 0x75, 0x61, 0x74, 0x69, 0x6d, 0x61, 0x6c, 0x61, 0x4d,
0x61, 0x6e, 0xe1, 0x67, 0x75, 0x61, 0x53, 0x69, 0x64, 0x61, 0x64, 0x69,
0x20, 0x64, 0x69, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x6b, 0x75, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x49, 0x6f, 0x72, 0x6b, 0x69, 0x42, 0x65, 0x75, 0x6c,
0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x69, 0x43, 0x65, 0x6e, 0x74, 0x65,
0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x69,
0x20, 0x4e, 0x6f, 0x72, 0x74, 0x69, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20,
0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x69, 0x50, 0x6f, 0x72, 0x74,
0x75, 0x20, 0x64, 0x69, 0x20, 0x53, 0x70, 0x61, 0x6e, 0x68, 0x61, 0x50,
0x6f, 0x72, 0x74, 0x75, 0x20, 0x52, 0x69, 0x6b, 0x75, 0x53, 0x61, 0x6e,
0x74, 0x75, 0x20, 0x44, 0x75, 0x6d, 0x69, 0x6e, 0x67, 0x75, 0x53, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x75,
0x53, 0x61, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x76, 0x61,
0x6e, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0xfa, 0x73, 0x69, 0x61,
0x53, 0x61, 0x6e, 0x20, 0x42, 0x69, 0x73, 0x65, 0x6e, 0x74, 0x69, 0x54,
0x6f, 0x72, 0x6f, 0x6e, 0x74, 0x75, 0x55, 0xed, 0x6e, 0x69, 0x70, 0x65,
0x67, 0x41, 0x62, 0x69, 0x1e7, 0x61, 0x6e, 0x41, 0x6b, 0x1e5b, 0x61, 0x41,
0x64, 0x69, 0x73, 0x20, 0x41, 0x62, 0x61, 0x62, 0x61, 0x4c, 0x65, 0x7a,
0x7a, 0x61, 0x79, 0x65, 0x72, 0x42, 0x61, 0x6d, 0x61, 0x6b, 0x75, 0x42,
0x72, 0x61, 0x1e93, 0x61, 0x76, 0x69, 0x6c, 0x4c, 0x71, 0x61, 0x68, 0x69,
0x72, 0x61, 0x4b, 0x61, 0x1e93, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x61,
0x53, 0x65, 0x62, 0x74, 0x61, 0x4b, 0x75, 0x6e, 0x61, 0x6b, 0x72, 0x69,
0x44, 0x61, 0x72, 0x20, 0x53, 0x61, 0x6c, 0x61, 0x6d, 0x1e6, 0x69, 0x62,
0x75, 0x74, 0x69, 0x44, 0x77, 0x61, 0x6c, 0x61, 0x4c, 0x65, 0x25b, 0x79,
0x75, 0x6e, 0x46, 0x72, 0x69, 0x1e6d, 0x61, 0x77, 0x6e, 0x47, 0x61, 0x62,
0x75, 0x72, 0x75, 0x6e, 0x48, 0x61, 0x72, 0x61, 0x72, 0x69, 0x4a, 0x75,
0x68, 0x61, 0x6e, 0x69, 0x73, 0x62, 0x75, 0x72, 0x67, 0x58, 0x61, 0x1e5b,
0x74, 0x75, 0x6d, 0x4b, 0x69, 0x6e, 0x63, 0x61, 0x1e63, 0x61, 0x4c, 0x61,
0x67, 0x75, 0x73, 0x4c, 0x69, 0x62, 0x65, 0x72, 0x76, 0x69, 0x6c, 0x4c,
0x75, 0x6d, 0x69, 0x4c, 0x77, 0x61, 0x6e, 0x64, 0x61, 0x4c, 0x75, 0x62,
0x75, 0x6d, 0x62, 0x61, 0x63, 0x69, 0x4c, 0x75, 0x7a, 0x61, 0x6b, 0x61,
0x4d, 0x61, 0x6c, 0x61, 0x62, 0x75, 0x4d, 0x61, 0x70, 0x75, 0x1e6d, 0x75,
0x4d, 0x61, 0x73, 0x69, 0x72, 0x75, 0x4d, 0x75, 0x71, 0x61, 0x64, 0x69,
0x63, 0x75, 0x4d, 0x75, 0x6e, 0x1e5b, 0x75, 0x76, 0x79, 0x61, 0x4e, 0x61,
0x79, 0x72, 0x75, 0x62, 0x69, 0x4e, 0x1e7, 0x61, 0x6d, 0x69, 0x6e, 0x61,
0x4e, 0x79, 0x61, 0x6d, 0x65, 0x79, 0x4e, 0x77, 0x61, 0x71, 0x63, 0x75,
0x1e6d, 0x50, 0x75, 0x1e5b, 0x74, 0x75, 0x20, 0x4e, 0x75, 0x76, 0x75, 0x53,
0x61, 0x77, 0x20, 0x1e6c, 0x75, 0x6d, 0x69, 0x54, 0x72, 0x61, 0x62, 0x6c,
0x75, 0x73, 0x57, 0x69, 0x6e, 0x64, 0x68, 0x65, 0x6b, 0x41, 0x6e, 0x6b,
0x75, 0x72, 0x61, 0x6a, 0x41, 0x6e, 0x74, 0x69, 0x67, 0x61, 0x4b, 0x75,
0x72, 0x64, 0x75, 0x62, 0x61, 0x4d, 0x69, 0x6e, 0x64, 0x75, 0x1e93, 0x61,
0x53, 0x61, 0x6e, 0x20, 0x58, 0x77, 0x61, 0x6e, 0x53, 0x61, 0x6e, 0x20,
0x4c, 0x77, 0x69, 0x73, 0x55, 0x63, 0x77, 0x61, 0x79, 0x61, 0x41, 0x73,
0x75, 0x6e, 0x73, 0x79, 0x75, 0x6e, 0x41, 0x74, 0x69, 0x6b, 0x75, 0x6b,
0x61, 0x6e, 0x42, 0x61, 0x68, 0x79, 0x61, 0x42, 0x61, 0x68, 0x79, 0x61,
0x20, 0x42, 0x61, 0x6e, 0x64, 0x69, 0x72, 0x61, 0x73, 0x42, 0x69, 0x6c,
0x69, 0x7a, 0x42, 0x6c, 0x75, 0x6e, 0x2d, 0x53, 0x61, 0x62, 0x6c, 0x75,
0x6e, 0x42, 0x75, 0x67, 0x75, 0x1e6d, 0x61, 0x42, 0x77, 0x61, 0x1e93, 0x4b,
0x61, 0x6d, 0x62, 0x72, 0x69, 0x1e7, 0x20, 0x42, 0x61, 0x79, 0x4b, 0x75,
0x6d, 0x70, 0x75, 0x20, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x69, 0x4b, 0x61,
0x79, 0x61, 0x6e, 0x43, 0x69, 0x6b, 0x61, 0x67, 0x75, 0x43, 0x69, 0x77,
0x61, 0x77, 0x61, 0x4b, 0x75, 0x73, 0x74, 0x61, 0x20, 0x52, 0x69, 0x6b,
0x61, 0x4b, 0x72, 0x69, 0x73, 0x74, 0x75, 0x6e, 0x4b, 0x75, 0x72, 0x61,
0x10d, 0x61, 0x77, 0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x63, 0x61,
0x76, 0x65, 0x6e, 0x44, 0x61, 0x77, 0x73, 0x75, 0x6e, 0x44, 0x61, 0x77,
0x73, 0x75, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x6b, 0x44, 0x69, 0x74, 0x72,
0x77, 0x61, 0x44, 0x75, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x49, 0x64, 0x6d,
0x75, 0x6e, 0x74, 0x75, 0x6e, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x75,
0x72, 0x46, 0x75, 0x1e5b, 0x74, 0x20, 0x4e, 0x69, 0x6c, 0x1e63, 0x75, 0x6e,
0x46, 0x75, 0x72, 0x1e6d, 0x61, 0x6c, 0x69, 0x7a, 0x61, 0x47, 0x72, 0x75,
0x6e, 0x61, 0x64, 0x61, 0x47, 0x77, 0x61, 0x64, 0x61, 0x6c, 0x75, 0x70,
0x69, 0x47, 0x77, 0x61, 0x74, 0x69, 0x6d, 0x61, 0x6c, 0x61, 0x47, 0x61,
0x79, 0x61, 0x6b, 0x69, 0x6c, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x4a, 0x61,
0x6d, 0x61, 0x79, 0x6b, 0x61, 0x4d, 0x61, 0x6e, 0x61, 0x67, 0x61, 0x4d,
0x61, 0x72, 0x69, 0x67, 0x75, 0x1e6d, 0x4e, 0x75, 0x1e5b, 0x75, 0x6e, 0x68,
0x61, 0x50, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x75, 0x50,
0x75, 0x72, 0x74, 0x75, 0x20, 0x56, 0x69, 0x6c, 0x68, 0x75, 0x52, 0x69,
0x73, 0x69, 0x66, 0x52, 0x79, 0x75, 0x20, 0x42, 0x72, 0x61, 0x6e, 0x6b,
0x75, 0x53, 0x61, 0x6e, 0x74, 0x79, 0x61, 0x67, 0x75, 0x53, 0x61, 0x77,
0x20, 0x50, 0x61, 0x77, 0x6c, 0x75, 0x4d, 0x61, 0x6b, 0x61, 0x72, 0x69,
0x56, 0x75, 0x73, 0x1e6d, 0x75, 0x6b, 0x4c, 0x75, 0x6e, 0x67, 0x79, 0x65,
0x72, 0x62, 0x79, 0x65, 0x6e, 0x190, 0x61, 0x64, 0x65, 0x6e, 0x190, 0x65,
0x6d, 0x6d, 0x61, 0x6e, 0x42, 0x65, 0x263, 0x64, 0x61, 0x64, 0x42, 0x61,
0x1e25, 0x72, 0x69, 0x6e, 0x42, 0x61, 0x6e, 0x67, 0x6b, 0x75, 0x6b, 0x42,
0x72, 0x75, 0x6e, 0x61, 0x79, 0x4b, 0x75, 0x6c, 0x75, 0x6d, 0x62, 0x75,
0x194, 0x65, 0x7a, 0x7a, 0x61, 0x48, 0x65, 0x62, 0x72, 0x75, 0x6e, 0x48,
0x75, 0x6e, 0x67, 0x20, 0x4b, 0x75, 0x6e, 0x67, 0x48, 0x75, 0x76, 0x64,
0x4a, 0x69, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x72,
0x61, 0x63, 0x69, 0x4b, 0x75, 0x6c, 0x6b, 0x61, 0x74, 0x61, 0x4b, 0x77,
0x61, 0x6c, 0x61, 0x20, 0x4c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x4b, 0x77,
0x69, 0x74, 0x4d, 0x61, 0x6b, 0x61, 0x77, 0x4d, 0x75, 0x73, 0x71, 0x61,
0x1e6d, 0x4e, 0x69, 0x6b, 0x75, 0x73, 0x79, 0x61, 0x50, 0x79, 0x75, 0x6e,
0x67, 0x79, 0x75, 0x6e, 0x67, 0x51, 0x61, 0x1e6d, 0x61, 0x72, 0x4b, 0x75,
0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x52, 0x69, 0x79, 0x61, 0x1e0d, 0x53,
0x69, 0x79, 0x75, 0x6c, 0x43, 0x61, 0x6e, 0x67, 0x61, 0x79, 0x53, 0x61,
0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x54, 0x61, 0x79, 0x70, 0x61, 0x79,
0x54, 0x75, 0x6b, 0x79, 0x75, 0x59, 0x65, 0x72, 0x76, 0x61, 0x6e, 0x41,
0x1e93, 0x75, 0x72, 0x69, 0x73, 0x42, 0x69, 0x72, 0x6d, 0x75, 0x64, 0x54,
0x69, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x4b, 0x61,
0x6e, 0x61, 0x72, 0x69, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x69, 0x72, 0x46,
0x61, 0x72, 0x77, 0x69, 0x4d, 0x61, 0x64, 0x69, 0x72, 0x61, 0x4a, 0x79,
0x75, 0x72, 0x6a, 0x79, 0x61, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x1e93, 0x75,
0x6c, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x49, 0x6c, 0x69, 0x6e, 0x61, 0x41,
0x64, 0x69, 0x6c, 0x61, 0x79, 0x69, 0x64, 0x4d, 0x61, 0x6c, 0x62, 0x75,
0x72, 0x6e, 0x41, 0x6d, 0x65, 0x73, 0x74, 0x69, 0x72, 0x64, 0x61, 0x6d,
0x41, 0x6e, 0x64, 0x75, 0x72, 0x61, 0x42, 0x69, 0x6c, 0x67, 0x72, 0x61,
0x64, 0x4b, 0x75, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x65, 0x6e, 0x44,
0x75, 0x62, 0x6c, 0x61, 0x6e, 0x4a, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x1e6d,
0x61, 0x72, 0x47, 0x69, 0x72, 0x6e, 0x69, 0x7a, 0x65, 0x79, 0x48, 0x69,
0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x54, 0x69, 0x67, 0x7a, 0x69, 0x72,
0x74, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x6e, 0x49, 0x73, 0x1e6d, 0x61, 0x6e,
0x62, 0x75, 0x6c, 0x4a, 0x69, 0x72, 0x7a, 0x65, 0x79, 0x4b, 0x79, 0x65,
0x76, 0x4c, 0x69, 0x7a, 0x62, 0x75, 0x6e, 0x4c, 0x75, 0x6e, 0x64, 0x75,
0x6e, 0x4d, 0x61, 0x72, 0x69, 0x68, 0x61, 0x6d, 0x6e, 0x4d, 0x75, 0x6e,
0x61, 0x6b, 0x75, 0x4d, 0x75, 0x73, 0x6b, 0x75, 0x55, 0x1e63, 0x6c, 0x75,
0x52, 0x75, 0x6d, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e,
0x75, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x69, 0x76, 0x75, 0x53, 0x6f, 0x66,
0x79, 0x61, 0x53, 0x1e6d, 0x75, 0x6b, 0x68, 0x75, 0x6c, 0x6d, 0x56, 0x79,
0x65, 0x6e, 0x6e, 0x61, 0x56, 0x69, 0x6c, 0x6e, 0x79, 0x75, 0x73, 0x5a,
0x75, 0x72, 0x69, 0x78, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61,
0x72, 0x69, 0x76, 0x75, 0x43, 0x61, 0x67, 0x75, 0x73, 0x4b, 0x75, 0x6b,
0x75, 0x4b, 0x75, 0x6d, 0x75, 0x1e5b, 0x4b, 0x65, 0x72, 0x67, 0x6c, 0x65,
0x6e, 0x4d, 0x61, 0x68, 0x69, 0x4d, 0x75, 0x72, 0x69, 0x73, 0x4d, 0x61,
0x79, 0x75, 0x1e6d, 0x52, 0x69, 0x79, 0x75, 0x6e, 0x79, 0x75, 0x6e, 0x41,
0x70, 0x79, 0x61, 0x55, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x43, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x75, 0x73,
0x47, 0x61, 0x6d, 0x62, 0x79, 0x65, 0x72, 0x4e, 0x69, 0x79, 0x75, 0x4e,
0x75, 0x72, 0x66, 0x75, 0x6c, 0x6b, 0xc85, 0xcac, 0xcbf, 0xca6, 0xccd, 0x200c,
0xc9c, 0xca8, 0xccd, 0xc85, 0xc95, 0xccd, 0xcb0, 0xcbe, 0xc86, 0xca1, 0xcbf, 0xcb8,
0xccd, 0x20, 0xc85, 0xcac, 0xcbe, 0xcac, 0xcbe, 0xc85, 0xcb2, 0xccd, 0xc9c, 0xcc0,
0xcb0, 0xccd, 0xcb8, 0xccd, 0xc85, 0xcb8, 0xccd, 0xcae, 0xcbe, 0xcb0, 0xcbe, 0xcac,
0xcae, 0xcbe, 0xc95, 0xcca, 0xcac, 0xcbe, 0xc82, 0xc97, 0xcbf, 0xcac, 0xc82, 0xc9c,
0xcc1, 0xcb2, 0xccd, 0xcac, 0xcbf, 0xcb8, 0xccd, 0xcb8, 0xcbe, 0xcb5, 0xccd, 0xcac,
0xccd, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc9f, 0xcc8, 0xcb0, 0xccd, 0xcac, 0xccd,
0xcb0, 0xcc6, 0xc9c, 0xcbe, 0xcb5, 0xcc0, 0xcb2, 0xccd, 0xcac, 0xcc1, 0xc9c, 0xcc1,
0xc82, 0xcac, 0xcc1, 0xcb0, 0xcbe, 0xc95, 0xcc8, 0xcb0, 0xccb, 0xc95, 0xccd, 0xcaf,
0xcbe, 0xcb8, 0xcbe, 0xcac, 0xccd, 0xcb2, 0xcbe, 0xc82, 0xc95, 0xcbe, 0xcb8, 0xcc6,
0xcaf, 0xcc1, 0xc9f, 0xcbe, 0xc95, 0xcca, 0xca8, 0xcbe, 0xc95, 0xccd, 0xcb0, 0xcbf,
0xca1, 0xcbe, 0xc95, 0xcb0, 0xccd, 0xca1, 0xcbe, 0xcb0, 0xccd, 0x20, 0xc8e, 0xcb8,
0xccd, 0x20, 0xcb8, 0xcb2, 0xcbe, 0xcae, 0xccd, 0xc9c, 0xcbf, 0xcac, 0xcc2, 0xc9f,
0xcbf, 0xca6, 0xccc, 0xcb2, 0xcbe, 0xc8e, 0xcb2, 0xccd, 0x20, 0xc94, 0xca8, 0xccd,
0xcab, 0xccd, 0xcb0, 0xcc0, 0xc9f, 0xccc, 0xca8, 0xccd, 0xc97, 0xccd, 0xcaf, 0xcbe,
0xcac, 0xcb0, 0xccb, 0xca8, 0xcc6, 0xcb9, 0xcb0, 0xcbe, 0xcb0, 0xcc6, 0xc9c, 0xccb,
0xcb9, 0xcbe, 0xca8, 0xccd, 0xcb8, 0xccd, 0x200c, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd,
0xc9c, 0xcc1, 0xcac, 0xcbe, 0xc95, 0xc82, 0xcaa, 0xcbe, 0xcb2, 0xcbe, 0xc96, 0xcb0,
0xccd, 0xc9f, 0xccb, 0xcae, 0xccd, 0xc95, 0xcbf, 0xc97, 0xcbe, 0xcb2, 0xcbf, 0xc95,
0xcbf, 0xca8, 0xccd, 0xcb7, 0xcbe, 0xcb8, 0xcbe, 0xcb2, 0xcbe, 0xc97, 0xccb, 0xcb8,
0xccd, 0xcb2, 0xcbf, 0xcac, 0xccd, 0xcb0, 0xcc6, 0xcb5, 0xcc0, 0xcb2, 0xccd, 0xcb2,
0xccb, 0xcae, 0xccd, 0xcb2, 0xcc1, 0xcb5, 0xcbe, 0xc82, 0xca1, 0xcbe, 0xcb2, 0xcc1,
0xcac, 0xcc1, 0xc82, 0xcac, 0xcbe, 0xcb6, 0xcbf, 0xcb2, 0xcc1, 0xcb8, 0xcbe, 0xc95,
0xcbe, 0xcae, 0xcb2, 0xcbe, 0xcac, 0xccb, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xcaa, 0xcc1,
0xc9f, 0xcca, 0xcae, 0xcb8, 0xcc7, 0xcb0, 0xcc1, 0xc85, 0xc82, 0xcac, 0xcac, 0xcbe,
0xca8, 0xcc6, 0xcae, 0xcca, 0xc97, 0xcbe, 0xca6, 0xcbf, 0xcb6, 0xcc1, 0xcae, 0xcca,
0xca8, 0xccd, 0xcb0, 0xccb, 0xcb5, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcc8, 0xcb0, 0xccb,
0xcac, 0xcbf, 0xca1, 0xccd, 0xc9c, 0xcae, 0xcc7, 0xca8, 0xcbe, 0xca8, 0xcbf, 0xcaf,
0xcbe, 0xcae, 0xcc6, 0xca8, 0xccc, 0xc95, 0xccd, 0x200c, 0xc9a, 0xccb, 0xc9f, 0xccd,
0xc9f, 0xccd, 0xc94, 0xc97, 0xcbe, 0xca1, 0xccb, 0xc97, 0xcc1, 0xcaa, 0xccb, 0xcb0,
0xccd, 0xc9f, 0xccb, 0x2d, 0xca8, 0xccb, 0xcb5, 0xccb, 0xcb8, 0xcbe, 0xcb5, 0xccd,
0x20, 0xc9f, 0xcca, 0xcae, 0xcbf, 0xc9f, 0xccd, 0xcb0, 0xcc8, 0xcaa, 0xccb, 0xcb2,
0xcbf, 0xc9f, 0xcc1, 0xca8, 0xcc0, 0xcb8, 0xccd, 0xcb5, 0xcbf, 0xc82, 0xca1, 0xccd,
0xcb9, 0xccb, 0xc95, 0xccd, 0xc85, 0xca1, 0xc95, 0xccd, 0x200c, 0x200c, 0xc86, 0xc82,
0xc95, 0xcb0, 0xccb, 0xc9c, 0xccd, 0xc86, 0xc82, 0xc97, 0xccd, 0xcb5, 0xcbf, 0xcb2,
0xccd, 0xcb2, 0xcbe, 0xc86, 0xc82, 0xc9f, 0xcbf, 0xc97, 0xcc1, 0xcb5, 0xcbe, 0xc85,
0xcb0, 0xcbe, 0xc97, 0xcc1, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcac, 0xccd, 0xcaf, 0xcc2,
0xca8, 0xcb8, 0xccd, 0x20, 0xc90, 0xcb0, 0xcbf, 0xcb8, 0xccd, 0xc95, 0xc9f, 0xcae,
0xcbe, 0xcb0, 0xccd, 0xc95, 0xc95, 0xcca, 0xcb0, 0xccd, 0xca1, 0xcca, 0xcac, 0xcbe,
0xc9c, 0xcc1, 0xc9c, 0xcc8, 0xcb2, 0xcbe, 0x20, 0xcb0, 0xcbf, 0xcaf, 0xccb, 0xc9c,
0xcbe, 0xcae, 0xcc6, 0xc82, 0xca1, 0xcca, 0xc9c, 0xcbe, 0xcb0, 0xcbf, 0xcaf, 0xccb,
0x20, 0xc97, 0xcb2, 0xccd, 0xcb2, 0xcc6, 0xc97, 0xcca, 0xcb8, 0xccd, 0xcb8, 0xcbe,
0xcb2, 0xccd, 0xc9f, 0xcbe, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd, 0x20, 0xc9c,
0xcc1, 0xcb5, 0xcbe, 0xca8, 0xccd, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd, 0x20,
0xcb2, 0xcc2, 0xcaf, 0xcc0, 0xcb8, 0xccd, 0xc9f, 0xcc1, 0xc95, 0xcc1, 0xcae, 0xcbe,
0xca8, 0xccd, 0xc89, 0xcb6, 0xccd, 0xcb5, 0xcbe, 0xcaf, 0xc85, 0xcb0, 0xcc1, 0xcac,
0xcbe, 0xc85, 0xcb8, 0xcc1, 0xca8, 0xccd, 0xcb8, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xccd,
0xc85, 0xc9f, 0xcbf, 0xc95, 0xcca, 0xc95, 0xcbe, 0xca8, 0xccd, 0xcac, 0xcb9, 0xcc0,
0xcaf, 0xcbe, 0xcac, 0xcbe, 0xcb9, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcac, 0xc82, 0xca1,
0xcc7, 0xcb0, 0xcbe, 0xcb8, 0xccd, 0xcac, 0xcbe, 0xcb0, 0xccd, 0xcac, 0xca1, 0xccb,
0xcb8, 0xccd, 0xcac, 0xcc7, 0xcb2, 0xcc6, 0xc82, 0xcac, 0xcc6, 0xcb2, 0xcc0, 0xc9c,
0xccd, 0xcac, 0xccd, 0xcb2, 0xcbe, 0xc82, 0xc95, 0xccd, 0x2d, 0xcb8, 0xcbe, 0xcac,
0xccd, 0xcb2, 0xca8, 0xccd, 0xcac, 0xccb, 0xcb5, 0xcbe, 0x20, 0xcb5, 0xcbf, 0xcb8,
0xccd, 0xc9f, 0xcac, 0xcca, 0xc97, 0xccb, 0xc9f, 0xcac, 0xccd, 0xcb5, 0xcbe, 0xcb8,
0xcbf, 0xc95, 0xcc7, 0xcae, 0xccd, 0x200c, 0xcac, 0xccd, 0xcb0, 0xcbf, 0xca1, 0xccd,
0xc9c, 0xccd, 0x20, 0xcac, 0xcc7, 0xc95, 0xcbe, 0xc82, 0xcaa, 0xcca, 0x20, 0xc97,
0xccd, 0xcb0, 0xcbe, 0xc82, 0xca1, 0xcc6, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc95,
0xca8, 0xccd, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xcb0, 0xc95, 0xcbe, 0xcb8, 0xccd, 0xc95,
0xcc6, 0xcaf, 0xcc6, 0xca8, 0xcbf, 0xc95, 0xcc7, 0xcae, 0xca8, 0xccd, 0xc9a, 0xcbf,
0xc95, 0xcbe, 0xc97, 0xccb, 0xc9a, 0xcbf, 0xcb5, 0xcbe, 0xcb5, 0xcb8, 0xcbf, 0xcaf,
0xcc1, 0xca1, 0xcbe, 0xca1, 0xccd, 0x20, 0xcb5, 0xcbe, 0xcb0, 0xcc6, 0xc9d, 0xccd,
0xc95, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xcbe, 0x20, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0xc95,
0xccd, 0xcb0, 0xcc6, 0xcb8, 0xccd, 0xc9f, 0xca8, 0xccd, 0xc95, 0xccd, 0xcaf, 0xcc2,
0xc87, 0xcac, 0xcbe, 0xc95, 0xcc1, 0xcb0, 0xcbe, 0xc95, 0xcbe, 0xcb5, 0xccd, 0xca1,
0xca8, 0xccd, 0x200c, 0xcae, 0xcbe, 0xcb0, 0xccd, 0xc95, 0xccd, 0x200c, 0xcb6, 0xccd,
0xcb5, 0xcbe, 0xca8, 0xccd, 0xca1, 0xcbe, 0xcb8, 0xca8, 0xccd, 0xca1, 0xcbe, 0xcb8,
0xca8, 0xccd, 0x20, 0xc95, 0xccd, 0xcb0, 0xcc0, 0xc95, 0xccd, 0xca1, 0xcc6, 0xca8,
0xccd, 0xcb5, 0xcb0, 0xccd, 0xca1, 0xcc6, 0xc9f, 0xccd, 0xcb0, 0xcbe, 0xcaf, 0xccd,
0xc9f, 0xccd, 0xca1, 0xcca, 0xcae, 0xcbf, 0xca8, 0xcbf, 0xc95, 0xcbe, 0xc8e, 0xca1,
0xccd, 0xcae, 0xc82, 0xc9f, 0xca8, 0xccd, 0x200c, 0xc87, 0xcb0, 0xcc1, 0xca8, 0xcbf,
0xcaa, 0xcc6, 0xc8e, 0xcb2, 0xccd, 0x20, 0xcb8, 0xcbe, 0xcb2, 0xccd, 0xcb5, 0xca1,
0xccb, 0xcb0, 0xccd, 0xcab, 0xccb, 0xcb0, 0xccd, 0xc9f, 0xccd, 0x20, 0xca8, 0xcc6,
0xcb2, 0xccd, 0xcb8, 0xca8, 0xccd, 0xcab, 0xcca, 0xcb0, 0xccd, 0xc9f, 0xcb2, 0xcc6,
0xc9c, 0xcbe, 0xc97, 0xccd, 0xcb2, 0xcc7, 0xcb8, 0xccd, 0x20, 0xcac, 0xcc7, 0xc97,
0xcc2, 0xcb8, 0xccd, 0x20, 0xcac, 0xcc7, 0xc97, 0xccd, 0xcb0, 0xccd, 0xcaf, 0xcbe,
0xc82, 0xca1, 0xccd, 0x20, 0xc9f, 0xcb0, 0xccd, 0xc95, 0xccd, 0xc97, 0xccd, 0xcb0,
0xcc6, 0xca8, 0xcbe, 0xca1, 0xc97, 0xccd, 0xcb5, 0xcbe, 0xca1, 0xcc6, 0xcb2, 0xccb,
0xcaa, 0xccd, 0xc97, 0xccd, 0xcb5, 0xcbe, 0xc9f, 0xcc6, 0xcae, 0xcbe, 0xcb2, 0xcbe,
0xc97, 0xcc1, 0xcaf, 0xcbe, 0xc95, 0xccd, 0xcb5, 0xcbf, 0xcb2, 0xccd, 0xc97, 0xcaf,
0xcbe, 0xca8, 0xcbe, 0xcb9, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcbf, 0xcab, 0xcc6, 0xc95,
0xccd, 0xcb8, 0xccd, 0xcb9, 0xcb5, 0xcbe, 0xca8, 0xcbe, 0xcb9, 0xcb0, 0xccd, 0xcae,
0xcca, 0xcb8, 0xcbf, 0xcb2, 0xccd, 0xcb2, 0xccb, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf,
0xcbe, 0xca8, 0xcbe, 0xcaa, 0xcca, 0xcb2, 0xcbf, 0xcb8, 0xccd, 0xca8, 0xcbe, 0xc95,
0xccd, 0xcb8, 0xccd, 0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8,
0xcbe, 0xcae, 0xcb0, 0xcc6, 0xc82, 0xc97, 0xcca, 0x2c, 0x20, 0xc87, 0xc82, 0xca1,
0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcaa, 0xcc0, 0xc9f, 0xcb0, 0xccd, 0xcb8, 0xccd,
0x200c, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd, 0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf,
0xcaf, 0xcbe, 0xca8, 0xcbe, 0xc9f, 0xcc6, 0xcb2, 0xccd, 0x20, 0xcb8, 0xcbf, 0xc9f,
0xcbf, 0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcb5,
0xcc7, 0xcb5, 0xcbe, 0xcaf, 0xccd, 0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf,
0xcbe, 0xca8, 0xcbe, 0xcb5, 0xcbf, 0xca8, 0xccd, 0x200c, 0xcb8, 0xcc6, 0xca8, 0xccd,
0xca8, 0xcc0, 0xcb8, 0xccd, 0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe,
0xca8, 0xcbe, 0xcb5, 0xcbf, 0xca8, 0xcbe, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd,
0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xc87, 0xca8,
0xcc1, 0xcb5, 0xcbf, 0xc95, 0xccd, 0xc88, 0xc95, 0xccd, 0xcb5, 0xcbe, 0xcb2, 0xcbf,
0xc9f, 0xccd, 0xc9c, 0xcae, 0xcc8, 0xc95, 0xcbe, 0xc9c, 0xcc1, 0xca8, 0xcc7, 0xcb5,
0xccd, 0xcb2, 0xcc2, 0xcaf, 0xcc0, 0xcb8, 0xccd, 0x200c, 0xcb5, 0xcbf, 0xcb2, 0xccd,
0xcb2, 0xcc6, 0xcae, 0xcca, 0xc82, 0xc9f, 0xcbf, 0xcb8, 0xcc6, 0xcb2, 0xccd, 0xcb2,
0xccb, 0x2c, 0x20, 0xc95, 0xcc6, 0xc82, 0xc9f, 0xcc1, 0xc95, 0xcbf, 0xc95, 0xccd,
0xcb0, 0xcc6, 0xcb2, 0xcc6, 0xc82, 0xca1, 0xcbf, 0xc9c, 0xccd, 0xc95, 0xccd, 0xcb2,
0xcbe, 0x20, 0xcaa, 0xcc6, 0xc9c, 0xccd, 0xcb2, 0xcbf, 0xcae, 0xcbe, 0xcb2, 0xcbe,
0xcb8, 0xccd, 0x20, 0xc8f, 0xc82, 0xc9c, 0xcb2, 0xcc0, 0xcb8, 0xccd, 0xcb2, 0xccb,
0xcb5, 0xcb0, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcbf, 0xca8, 0xccd, 0xcb8, 0xcb8,
0xccd, 0x20, 0xc95, 0xccd, 0xcb5, 0xcbe, 0xcb0, 0xccd, 0xc9f, 0xcb0, 0xccd, 0xcae,
0xcc6, 0xcb8, 0xcc0, 0xcaf, 0xcca, 0xcae, 0xca8, 0xcbe, 0xc97, 0xcc1, 0xcb5, 0xcbe,
0xcae, 0xca8, 0xcbe, 0xcb5, 0xccd, 0xcb8, 0xccd, 0xcae, 0xcb0, 0xcbf, 0xc97, 0xcca,
0xcae, 0xcbe, 0xcb0, 0xccd, 0xc9f, 0xcbf, 0xca8, 0xcbf, 0xc95, 0xccd, 0xcae, 0xc9f,
0xcbe, 0xcae, 0xccb, 0xcb0, 0xcb8, 0xccd, 0xcae, 0xc9c, 0xc9f, 0xccd, 0xcb2, 0xcbe,
0xca8, 0xccd, 0xcae, 0xcc6, 0xca8, 0xcca, 0xcae, 0xcbf, 0xca8, 0xcc0, 0xcae, 0xcc6,
0xcb0, 0xcbf, 0xca6, 0xcbe, 0xcae, 0xcc6, 0xc9f, 0xccd, 0xcb2, 0xcbe, 0xc95, 0xcbe,
0xc9f, 0xccd, 0xcb2, 0xcbe, 0xcae, 0xcc6, 0xc95, 0xccd, 0xcb8, 0xcbf, 0xc95, 0xccb,
0x20, 0xca8, 0xc97, 0xcb0, 0xcae, 0xcbf, 0xc95, 0xcc6, 0xcb2, 0xcbe, 0xca8, 0xccd,
0xcae, 0xcbe, 0xc82, 0xc95, 0xccd, 0xc9f, 0xca8, 0xccd, 0xcae, 0xcbe, 0xc82, 0xc9f,
0xcc6, 0xcb0, 0xccd, 0xcb0, 0xcbf, 0xcae, 0xcbe, 0xc82, 0xc9f, 0xcbf, 0xcb5, 0xcc0,
0xca1, 0xcbf, 0xcaf, 0xcca, 0xcae, 0xcbe, 0xc82, 0xc9f, 0xccd, 0x200c, 0xcb8, 0xcc6,
0xcb0, 0xcc7, 0xc9f, 0xccd, 0xca8, 0xcb8, 0xccd, 0xcb8, 0xccc, 0xcb5, 0xccd, 0xca8,
0xccd, 0xcaf, 0xcc2, 0xcaf, 0xcbe, 0xcb0, 0xccd, 0xc95, 0xccd, 0xca8, 0xccb, 0xcae,
0xccd, 0xca8, 0xcca, 0xcb0, 0xcca, 0xca8, 0xccd, 0xcb9, 0xcbe, 0xcac, 0xccd, 0xcaf,
0xcc2, 0xcb2, 0xcbe, 0x2c, 0x20, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xca1,
0xc95, 0xcca, 0xc9f, 0xcb8, 0xcc6, 0xc82, 0xc9f, 0xcb0, 0xccd, 0x2c, 0x20, 0xc89,
0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xca1, 0xc95, 0xcca, 0xc9f, 0xca8, 0xccd, 0xcaf,
0xcc2, 0x20, 0xcb8, 0xcb2, 0xcc7, 0xc82, 0x2c, 0x20, 0xc89, 0xca4, 0xccd, 0xca4,
0xcb0, 0x20, 0xca1, 0xc95, 0xcca, 0xc9f, 0xca8, 0xcc2, 0xc95, 0xccd, 0xc93, 0xc9c,
0xcbf, 0xca8, 0xcbe, 0xc97, 0xcbe, 0xcaa, 0xca8, 0xcbe, 0xcae, 0xcbe, 0xcaa, 0xcb0,
0xcae, 0xcbe, 0xcb0, 0xcbf, 0xcac, 0xccb, 0xcab, 0xcbf, 0xca8, 0xcbf, 0xc95, 0xccd,
0xcb8, 0xccd, 0xcaa, 0xccb, 0xcb0, 0xccd, 0xc9f, 0xccd, 0x2d, 0xc92, 0x2d, 0xcaa,
0xccd, 0xcb0, 0xcbf, 0xca8, 0xccd, 0xcb8, 0xccd, 0xcaa, 0xccb, 0xcb0, 0xccd, 0xc9f,
0xccd, 0x20, 0xc86, 0xcab, 0xccd, 0x20, 0xcb8, 0xccd, 0xcaa, 0xcc7, 0xca8, 0xccd,
0xcaa, 0xccb, 0xcb0, 0xccd, 0xc9f, 0xcca, 0x20, 0xcb5, 0xcc6, 0xcb2, 0xccd, 0xcb9,
0xccb, 0xcaa, 0xccd, 0xcaf, 0xcc1, 0xcb0, 0xccd, 0xc9f, 0xccb, 0x20, 0xcb0, 0xcbf,
0xc95, 0xcca, 0xcaa, 0xcc1, 0xc82, 0xca4, 0xcbe, 0x20, 0xc85, 0xcb0, 0xcc7, 0xca8,
0xcbe, 0xcb8, 0xccd, 0xcb0, 0xcbe, 0xc82, 0xc95, 0xcbf, 0xca8, 0xccd, 0x20, 0xc87,
0xca8, 0xccd, 0x200c, 0xcb2, 0xcc6, 0xc9f, 0xccd, 0xcb0, 0xcc6, 0xcb8, 0xcbf, 0xcab,
0xcbf, 0xcb0, 0xcc6, 0xc9c, 0xcbf, 0xca8, 0xcbe, 0xcb0, 0xcc6, 0xcb8, 0xcca, 0xcb2,
0xccd, 0xcaf, 0xcc2, 0xc9f, 0xccd, 0xcb0, 0xcbf, 0xcaf, 0xccb, 0x20, 0xcac, 0xccd,
0xcb0, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc95, 0xcca, 0xcb8, 0xcbe, 0xc82, 0xc9f, 0xcb0,
0xcc6, 0xcae, 0xccd, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc9f, 0xcbf, 0xcaf, 0xcbe,
0xc97, 0xcca, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc9f, 0xccb, 0x20, 0xca1, 0xcca,
0xcae, 0xcbf, 0xc82, 0xc97, 0xcca, 0xcb8, 0xcbe, 0xcb5, 0xccd, 0x20, 0xcaa, 0xcbe,
0xcb2, 0xcca, 0xc87, 0xc9f, 0xccd, 0xc9f, 0xcca, 0xc95, 0xccd, 0xc95, 0xcca, 0xcb0,
0xccd, 0xc9f, 0xcc2, 0xcae, 0xcbf, 0xcaf, 0xcc8, 0xc9f, 0xccd, 0xcb8, 0xcbf, 0xca4,
0xccd, 0xc95, 0xcbe, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcac, 0xcbe, 0xcb0,
0xccd, 0xca4, 0xcc6, 0xcb2, 0xcc6, 0xcae, 0xcbf, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd,
0x20, 0xc9c, 0xcbe, 0xca8, 0xccd, 0xcb8, 0xccd, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd,
0x20, 0xc95, 0xcbf, 0xc9f, 0xccd, 0xcb8, 0xccd, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd,
0x20, 0xcb2, 0xcc2, 0xcb8, 0xcbf, 0xcaf, 0xcbe, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd,
0x20, 0xca5, 0xcbe, 0xcae, 0xcb8, 0xccd, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20,
0xcb5, 0xcbf, 0xca8, 0xccd, 0xcb8, 0xcc6, 0xc82, 0xc9f, 0xccd, 0xcb8, 0xccd, 0xcb5,
0xcbf, 0xcab, 0xccd, 0xc9f, 0xccd, 0x20, 0xc95, 0xcb0, 0xcc6, 0xc82, 0xc9f, 0xccd,
0xca4, 0xcc6, 0xc97, 0xcc1, 0xcb8, 0xcbf, 0xc97, 0xcb2, 0xccd, 0xcaa, 0xcbe, 0xca5,
0xcc2, 0xcb2, 0xcc6, 0xca4, 0xcbf, 0xc9c, 0xccd, 0xcb5, 0xcbe, 0xca8, 0xcbe, 0xc9f,
0xcca, 0xcb0, 0xcca, 0xc82, 0xc9f, 0xccb, 0xc9f, 0xcbe, 0xcb0, 0xccd, 0xc9f, 0xcca,
0xcb2, 0xcbe, 0xcb5, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc95, 0xcca, 0xcb5, 0xcb0, 0xccd,
0x200c, 0xcb5, 0xcc8, 0xc9f, 0xccd, 0x200c, 0xcb9, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xccd,
0xcb5, 0xcbf, 0xca8, 0xccd, 0xca8, 0xcbf, 0xcaa, 0xcc6, 0xc97, 0xccd, 0xcaf, 0xcbe,
0xc95, 0xcc1, 0xc9f, 0xcbe, 0xc9f, 0xccd, 0xc95, 0xcc7, 0xcb8, 0xcbf, 0xca1, 0xcc7,
0xcb5, 0xcc0, 0xcb8, 0xccd, 0xca1, 0xcc1, 0xcae, 0xc82, 0xc9f, 0xccd, 0x20, 0xca1,
0xcbf, 0x20, 0xc89, 0xcb0, 0xccd, 0xcb5, 0xcbf, 0xcb2, 0xcc6, 0xcae, 0xccd, 0xcaf,
0xcbe, 0xc95, 0xccd, 0xcb5, 0xcc6, 0xcb0, 0xcbf, 0xcae, 0xcbe, 0xcb5, 0xccd, 0xcb8,
0xca8, 0xccd, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd, 0x200d, 0xcae, 0xcc1, 0xcb0,
0xccd, 0xca1, 0xcca, 0xcaa, 0xcbe, 0xcb2, 0xccd, 0xcae, 0xcb0, 0xccd, 0xcb0, 0xcca,
0xca5, 0xcc7, 0xcb0, 0xcbe, 0xcb8, 0xccd, 0xcaf, 0xccb, 0xcb5, 0xcbe, 0xc9f, 0xccd,
0xcb0, 0xcbe, 0xcb2, 0xccd, 0xcb5, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xccb, 0xc95, 0xccd,
0xcb2, 0xcbe, 0xc82, 0xc97, 0xccd, 0xcaf, 0xcc0, 0xcb0, 0xccd, 0xcac, 0xcc6, 0xca8,
0xccd, 0xc8f, 0xca1, 0xca8, 0xccd, 0xc85, 0xcb2, 0xccd, 0xcae, 0xcbe, 0xc9f, 0xcbf,
0xc85, 0xcae, 0xccd, 0xcae, 0xcbe, 0xca8, 0xccd, 0xc85, 0xca8, 0xca6, 0xccd, 0xcaf,
0xcb0, 0xccd, 0xc85, 0xc95, 0xccd, 0xca4, 0xcbe, 0xcb5, 0xccd, 0xc85, 0xc95, 0xccd,
0xc9f, 0xccb, 0xcac, 0xcc6, 0xc85, 0xcb6, 0xccd, 0xc97, 0xcac, 0xcbe, 0xca4, 0xccd,
0xc85, 0xc9f, 0xccd, 0xcb0, 0xcbe, 0xcb5, 0xccd, 0xcac, 0xcbe, 0xc97, 0xccd, 0xca6,
0xcbe, 0xca6, 0xccd, 0xcac, 0xcb9, 0xccd, 0xcb0, 0xcc7, 0xca8, 0xccd, 0xcac, 0xc95,
0xcc1, 0xcac, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc95, 0xcbe, 0xc95, 0xccd, 0xcac, 0xcbe,
0xcb0, 0xccd, 0xca8, 0xccc, 0xcb2, 0xccd, 0xcac, 0xcc0, 0xcb0, 0xc9f, 0xccd, 0xcac,
0xcbf, 0xcb6, 0xccd, 0xc95, 0xcc6, 0xc95, 0xccd, 0xcac, 0xccd, 0xcb0, 0xcc2, 0xca8,
0xcbf, 0xc9a, 0xcbf, 0xc9f, 0xc95, 0xcca, 0xcb2, 0xc82, 0xcac, 0xcca, 0xca1, 0xcae,
0xcbe, 0xcb8, 0xccd, 0xc95, 0xcb8, 0xccd, 0xca2, 0xcbe, 0xc95, 0xcbe, 0xca6, 0xcbf,
0xcb2, 0xcbf, 0xca6, 0xcc1, 0xcac, 0xcc8, 0xca6, 0xcc1, 0xcb6, 0xcbe, 0xc82, 0xcac,
0xcc6, 0xcab, 0xcae, 0xcbe, 0xc97, 0xcc1, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xc97, 0xcbe,
0xc9c, 0xcbe, 0xcb9, 0xcc6, 0xcac, 0xccd, 0xcb0, 0xcbe, 0xca8, 0xccd, 0xcb9, 0xcca,
0x20, 0xc9a, 0xcbf, 0x20, 0xcae, 0xcbf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcbf, 0xc9f,
0xcbf, 0xcb9, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xc95, 0xcbe, 0xc82, 0xc97, 0xccd,
0xcb9, 0xccb, 0xcb5, 0xccd, 0xca1, 0xccd, 0xc87, 0xcb0, 0xc95, 0xcc1, 0xc9f, 0xccd,
0xcb8, 0xccd, 0xc95, 0xccd, 0xc9c, 0xc95, 0xcbe, 0xcb0, 0xccd, 0xca4, 0xcbe, 0xc9c,
0xcaf, 0xcaa, 0xcc1, 0xcb0, 0xc9c, 0xcc6, 0xcb0, 0xcc1, 0xcb8, 0xcb2, 0xcc6, 0xcae,
0xccd, 0xc95, 0xcbe, 0xcac, 0xcc2, 0xcb2, 0xccd, 0xc95, 0xc82, 0xc9a, 0xc9f, 0xccd,
0xc95, 0xccd, 0xc95, 0xcb0, 0xcbe, 0xc9a, 0xcbf, 0xc95, 0xca0, 0xccd, 0xcae, 0xc82,
0xca1, 0xcc1, 0xc96, 0xc82, 0xca1, 0xccd, 0xcaf, 0xcbe, 0xc97, 0xcbe, 0xc95, 0xcca,
0xcb2, 0xccd, 0xc95, 0xca4, 0xccd, 0xca4, 0xcbe, 0xc95, 0xccd, 0xcb0, 0xcbe, 0xcb8,
0xccd, 0xca8, 0xcca, 0xcaf, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0xc95,
0xccc, 0xcb2, 0xcbe, 0x20, 0xcb2, 0xc82, 0xcaa, 0xcc1, 0xcb0, 0xc95, 0xcc1, 0xc9a,
0xcbf, 0xc82, 0xc97, 0xccd, 0xc95, 0xcc1, 0xcb5, 0xcc8, 0xca4, 0xccd, 0xcae, 0xc95,
0xcbe, 0xcb5, 0xcc1, 0xcae, 0xc97, 0xca6, 0xca8, 0xccd, 0x200c, 0xcae, 0xcc6, 0xc95,
0xcbe, 0xcb8, 0xccd, 0xcb8, 0xcb0, 0xccd, 0xcae, 0xca8, 0xcbf, 0xcb2, 0xcbe, 0xcae,
0xcb8, 0xccd, 0xc95, 0xc9f, 0xccd, 0xca8, 0xcbf, 0xc95, 0xccb, 0xcb8, 0xcbf, 0xcaf,
0xcbe, 0xca8, 0xccb, 0xcb5, 0xccb, 0xc95, 0xcc1, 0xc9c, 0xcc6, 0xca8, 0xcca, 0xcb5,
0xccb, 0xcb8, 0xcbf, 0xcac, 0xcbf, 0xcb8, 0xccd, 0xc95, 0xccd, 0xc92, 0xcae, 0xccd,
0xcb8, 0xccd, 0xc95, 0xccd, 0xc92, 0xcb0, 0xcb2, 0xccd, 0xca8, 0xccb, 0xcae, 0xccd,
0x20, 0xcaa, 0xcc6, 0xca8, 0xccd, 0xcaa, 0xcca, 0xc82, 0xc9f, 0xcbf, 0xcaf, 0xcbe,
0xca8, 0xc95, 0xccd, 0xcaa, 0xccd, 0xcaf, 0xcca, 0xc82, 0xc97, 0xccd, 0x200d, 0xcaf,
0xcbe, 0xc82, 0xc97, 0xccd, 0xc96, 0xca4, 0xcbe, 0xcb0, 0xccd, 0xc95, 0xcca, 0xcb8,
0xccd, 0xc9f, 0xca8, 0xcaf, 0xccd, 0xc95, 0xcbf, 0xc9c, 0xcc8, 0xcb2, 0xccb, 0xcb0,
0xccd, 0xca6, 0xcb0, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0xccd, 0xcb8, 0xc96, 0xcbe, 0xcb2,
0xcbf, 0xca8, 0xccd, 0xcb8, 0xcae, 0xcb0, 0xc96, 0xc82, 0xca1, 0xcb8, 0xcbf, 0xcaf,
0xccb, 0xcb2, 0xccd, 0xcb6, 0xcbe, 0xc82, 0xc98, 0xcc8, 0xcb8, 0xcbf, 0xc82, 0xc97,
0xcaa, 0xcc1, 0xcb0, 0xccd, 0xcb8, 0xccd, 0xcb0, 0xcc7, 0xca6, 0xccd, 0xca8, 0xcc7,
0xc95, 0xccb, 0xcb2, 0xcc0, 0xcae, 0xcb8, 0xccd, 0xc95, 0xca4, 0xcc8, 0xcaa, 0xcc6,
0xca4, 0xcbe, 0xcb6, 0xccd, 0xc95, 0xcc6, 0xc82, 0xc9f, 0xccd, 0xc9f, 0xcbf, 0xcac,
0xcbf, 0xcb2, 0xcbf, 0xcb8, 0xcbf, 0xca4, 0xcc6, 0xcb9, 0xccd, 0xcb0, 0xcbe, 0xca8,
0xccd, 0xca5, 0xcbf, 0xc82, 0xcaa, 0xcc1, 0xc9f, 0xccb, 0xc95, 0xcbf, 0xcaf, 0xcca,
0xc9f, 0xccb, 0xcae, 0xccd, 0x200c, 0xcb8, 0xccd, 0xc95, 0xccd, 0xc89, 0xcb2, 0xcbe,
0xca8, 0xccd, 0x20, 0xcac, 0xca4, 0xcca, 0xcb0, 0xccd, 0xc89, 0xcb0, 0xcc1, 0xcae,
0xccd, 0x200d, 0xc95, 0xcbf, 0xc89, 0xcb8, 0xccd, 0xca4, 0xccd, 0x2d, 0xca8, 0xcc6,
0xcb0, 0xcbe, 0xcb5, 0xcbf, 0xcaf, 0xcc6, 0xc82, 0xc9f, 0xcbf, 0xcaf, 0xcbe, 0xca8,
0xccd, 0xcb5, 0xccd, 0xcb2, 0xcbe, 0xca6, 0xcbf, 0xcb5, 0xcca, 0xcb8, 0xccd, 0xc9f,
0xcbe, 0xc95, 0xccd, 0x200c, 0xcaf, 0xc95, 0xcc1, 0xc9f, 0xccd, 0xcb8, 0xccd, 0xc95,
0xccd, 0xcaf, 0xcbe, 0xc82, 0xc97, 0xcca, 0xca8, 0xccd, 0xc8e, 0xc95, 0xccd, 0xcaf,
0xcbe, 0xc9f, 0xcc6, 0xcb0, 0xcbf, 0xca8, 0xccd, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd,
0xcaf, 0xcc6, 0xcb0, 0xcc6, 0xcb5, 0xcbe, 0xca8, 0xccd, 0xc85, 0xc9d, 0xccb, 0xcb0,
0xcbf, 0xcb8, 0xccd, 0xcac, 0xcb0, 0xccd, 0xcae, 0xcc1, 0xca1, 0xcbe, 0xc95, 0xccd,
0xcaf, 0xcbe, 0xca8, 0xcb0, 0xcbf, 0xc95, 0xcc7, 0xcaa, 0xccd, 0x20, 0xcb5, 0xcc6,
0xcb0, 0xccd, 0xca1, 0xccd, 0xcab, 0xccd, 0xcaf, 0xcbe, 0xcb0, 0xcca, 0xcae, 0xcc6,
0xca1, 0xcbf, 0xcb0, 0xcbe, 0xcb0, 0xcc8, 0xc95, 0xccd, 0xc9c, 0xcbe, 0xcb5, 0xcbf,
0xc95, 0xccd, 0xca6, 0xc95, 0xccd, 0xcb7, 0xcbf, 0xca3, 0x20, 0xc9c, 0xcbe, 0xcb0,
0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcb9,
0xcc6, 0xcb2, 0xcc6, 0xca8, 0xcbe, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcaf, 0xcbe, 0xca8,
0xccd, 0xcb2, 0xcbf, 0xc85, 0xca1, 0xcbf, 0xcb2, 0xcc7, 0xca1, 0xccd, 0xcac, 0xccd,
0xcb0, 0xcbf, 0xcb8, 0xccd, 0xcac, 0xcc7, 0xca8, 0xccd, 0x200c, 0xcac, 0xccd, 0xcb0,
0xcca, 0xc95, 0xca8, 0xccd, 0x20, 0xcb9, 0xcbf, 0xcb2, 0xccd, 0xca1, 0xcbe, 0xcb0,
0xccd, 0xcb5, 0xcbf, 0xca8, 0xccd, 0xcaf, 0xcc1, 0xc95, 0xccd, 0xcb2, 0xcbe, 0xcb9,
0xccb, 0xcac, 0xcbe, 0xcb0, 0xccd, 0xc9f, 0xccd, 0x200c, 0xcb2, 0xcbf, 0xc82, 0xca1,
0xcbf, 0xcae, 0xcbe, 0xca8, 0xccd, 0xcb2, 0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20,
0xcb9, 0xccb, 0xcb5, 0xccd, 0xcae, 0xcc6, 0xcb2, 0xccd, 0xcac, 0xcb0, 0xccd, 0xca8,
0xccd, 0xcaa, 0xcb0, 0xccd, 0xca5, 0xccd, 0xcb8, 0xcbf, 0xca1, 0xccd, 0xca8, 0xcbf,
0xc86, 0xcae, 0xccd, 0x200c, 0xcb8, 0xccd, 0xc9f, 0xcc6, 0xcb0, 0xccd, 0x200c, 0xca1,
0xccd, 0xcaf, 0xcbe, 0xc82, 0xc85, 0xc82, 0xca1, 0xccb, 0xcb0, 0xcbe, 0xc86, 0xcb8,
0xccd, 0xc9f, 0xccd, 0xcb0, 0xcbe, 0xc96, 0xcbe, 0xca8, 0xccd, 0xc85, 0xca5, 0xcc6,
0xca8, 0xccd, 0xcb8, 0xccd, 0xcac, 0xcc6, 0xcb2, 0xccd, 0x200c, 0xc97, 0xccd, 0xcb0,
0xcc7, 0xca1, 0xccd, 0xcac, 0xcb0, 0xccd, 0xcb2, 0xcbf, 0xca8, 0xccd, 0xcac, 0xccd,
0xcb0, 0xcbe, 0xc9f, 0xcbf, 0xcb8, 0xccd, 0xcb2, 0xcbe, 0xcb5, 0xcac, 0xccd, 0xcb0,
0xcb8, 0xccd, 0xcb8, 0xcc6, 0xcb2, 0xccd, 0xcb8, 0xccd, 0xcac, 0xcc1, 0xc95, 0xcbe,
0xcb0, 0xcc6, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcac, 0xcc1, 0xca1, 0xcbe, 0xcaa, 0xcc6,
0xcb8, 0xccd, 0xc9f, 0xccd, 0xcac, 0xcc1, 0xcb8, 0xcbf, 0xc82, 0xc9c, 0xcc6, 0xca8,
0xccd, 0xc9a, 0xcbf, 0xcb8, 0xcbf, 0xca8, 0xcbe, 0xcb5, 0xccd, 0xc95, 0xccb, 0xcaa,
0xcc6, 0xca8, 0xccd, 0x200c, 0xcb9, 0xcc7, 0xc97, 0xca8, 0xccd, 0xca1, 0xcc1, 0xcac,
0xccd, 0xcb2, 0xcbf, 0xca8, 0xccd, 0xc9c, 0xcbf, 0xcac, 0xccd, 0xcb0, 0xcbe, 0xcb2,
0xccd, 0xc9f, 0xcb0, 0xccd, 0xc97, 0xccd, 0xcaf, 0xcc1, 0xcb0, 0xccd, 0xca8, 0xccd,
0x200d, 0xcb8, 0xcc6, 0xcb9, 0xcc6, 0xcb2, 0xccd, 0xcb8, 0xcbf, 0xc82, 0xc95, 0xcbf,
0xc90, 0xcb2, 0xccd, 0x20, 0xc86, 0xcab, 0xccd, 0x20, 0xcae, 0xccd, 0xcaf, 0xcbe,
0xca8, 0xccd, 0xc87, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x200c, 0xcac, 0xcc1,
0xcb2, 0xccd, 0xc9c, 0xcc6, 0xcb0, 0xccd, 0xcb8, 0xcbf, 0xc95, 0xcb2, 0xcbf, 0xca8,
0xcbf, 0xca8, 0xccd, 0x200c, 0xc97, 0xccd, 0xcb0, 0xcbe, 0xca1, 0xccd, 0xc95, 0xcbf,
0xcb0, 0xcbe, 0xcb5, 0xccd, 0xc95, 0xcbf, 0xcb5, 0xccd, 0xcb2, 0xcbf, 0xcb8, 0xccd,
0xcac, 0xca8, 0xccd, 0xcb2, 0xccd, 0xcaf, 0xcc2, 0xcac, 0xccd, 0xcb2, 0xccd, 0xcaf,
0xcbe, 0xca8, 0xcbe, 0xcb2, 0xc82, 0xca1, 0xca8, 0xccd, 0xcb2, 0xc95, 0xccd, 0xcb8,
0xcc6, 0xc82, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xca1,
0xccd, 0xcb0, 0xcbf, 0xca1, 0xccd, 0xcae, 0xcbe, 0xcb2, 0xccd, 0xc9f, 0xcbe, 0xcae,
0xcb0, 0xcbf, 0xcb9, 0xcae, 0xccd, 0xca8, 0xccd, 0xcae, 0xcbf, 0xca8, 0xccd, 0x200c,
0xcb8, 0xccd, 0xc95, 0xccd, 0xcae, 0xcca, 0xca8, 0xcbe, 0xc95, 0xcca, 0xcae, 0xcbe,
0xcb8, 0xccd, 0xc95, 0xccb, 0xc93, 0xcb8, 0xccd, 0xcb2, 0xccb, 0xcaa, 0xccd, 0xcaf,
0xcbe, 0xcb0, 0xcbf, 0xcb8, 0xccd, 0xcaa, 0xcca, 0xca1, 0xccd, 0xc97, 0xcca, 0xcb0,
0xcbf, 0xc95, 0xcaa, 0xccd, 0xcb0, 0xcbe, 0xc97, 0xccd, 0xcb5, 0xcc6, 0xcb0, 0xcbf,
0xc97, 0xcbe, 0xcb0, 0xccb, 0xcae, 0xccd, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd,
0x20, 0xcae, 0xcb0, 0xcc0, 0xca8, 0xcca, 0xcb8, 0xcb0, 0xcaf, 0xcc7, 0xcb5, 0xcca,
0xcb8, 0xcb0, 0xcbe, 0xc9f, 0xccb, 0xcb5, 0xccd, 0xcb8, 0xcbf, 0xcae, 0xccd, 0xcab,
0xcc6, 0xcb0, 0xccb, 0xcaa, 0xccb, 0xcb2, 0xccd, 0xcb8, 0xccd, 0xc95, 0xcbe, 0xcaa,
0xcbf, 0xcaf, 0xcbe, 0xcb8, 0xccb, 0xcab, 0xcbf, 0xcaf, 0xcbe, 0xcb8, 0xccd, 0xc9f,
0xcbe, 0xc95, 0xccd, 0x200d, 0xcb9, 0xccb, 0xcae, 0xccd, 0xc9f, 0xccd, 0xcaf, 0xcbe,
0xcb2, 0xcbf, 0xca8, 0xccd, 0xc9f, 0xcbf, 0xcb0, 0xcbe, 0xca8, 0xcc6, 0xc89, 0xcb2,
0xccd, 0xcaf, 0xcbe, 0xca8, 0xcca, 0xcb5, 0xcb8, 0xccd, 0xc95, 0xccd, 0xcb5, 0xca1,
0xcc2, 0xc9c, 0xccd, 0xcb5, 0xccd, 0xcaf, 0xcbe, 0xc9f, 0xcbf, 0xc95, 0xca8, 0xccd,
0xcb5, 0xcbf, 0xcaf, 0xcc6, 0xca8, 0xccd, 0xca8, 0xcbe, 0xcb5, 0xcbf, 0xcb2, 0xccd,
0xca8, 0xcbf, 0xcaf, 0xcb8, 0xccd, 0xcb5, 0xcca, 0xcb2, 0xccd, 0xc97, 0xcca, 0xc97,
0xccd, 0xcb0, 0xcbe, 0xca1, 0xccd, 0xcb5, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xcbe, 0xc9c,
0xcbe, 0xc97, 0xccd, 0xcb0, 0xcc6, 0xcac, 0xccd, 0x200c, 0xc9c, 0xccd, 0xcaf, 0xcc2,
0xcb0, 0xcbf, 0xc95, 0xccd, 0xc85, 0xc82, 0xc9f, 0xcbe, 0xca8, 0xca8, 0xcbe, 0xcb0,
0xcbf, 0xcb5, 0xcca, 0xc9a, 0xcbe, 0xc97, 0xcca, 0xcb8, 0xccd, 0xc95, 0xccd, 0xcb0,
0xcbf, 0xcb8, 0xccd, 0xcae, 0xcb8, 0xccd, 0xc95, 0xcca, 0xc95, 0xccb, 0xcb8, 0xccd,
0xc95, 0xcca, 0xcae, 0xccb, 0xcb0, 0xcca, 0xc95, 0xcc6, 0xcb0, 0xccd, 0xc97, 0xccd,
0xcaf, 0xcc1, 0xcb2, 0xcc6, 0xca8, 0xccd, 0xcae, 0xcbe, 0xcb9, 0xcc6, 0xcae, 0xcbe,
0xcb2, 0xccd, 0xca1, 0xcc0, 0xcb5, 0xccd, 0xcb8, 0xccd, 0xcae, 0xcbe, 0xcb0, 0xcbf,
0xcb7, 0xcb8, 0xccd, 0xcae, 0xcaf, 0xcca, 0xc9f, 0xccd, 0xc9f, 0xcc6, 0xcb0, 0xcc0,
0xcaf, 0xcc2, 0xca8, 0xcbf, 0xcaf, 0xca8, 0xccd, 0xc85, 0xcaa, 0xcbf, 0xcaf, 0xcbe,
0xc86, 0xc95, 0xccd, 0x20, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0xcac,
0xccc, 0xc97, 0xcc6, 0xca8, 0xccd, 0x200d, 0xcb5, 0xcbf, 0xcb2, 0xccd, 0xcb2, 0xcc6,
0xc9a, 0xca5, 0xcbe, 0xcae, 0xccd, 0xc9a, 0xcc1, 0xc95, 0xccd, 0xc88, 0xcb8, 0xccd,
0xc9f, 0xcb0, 0xccd, 0xc87, 0xcab, 0xcc7, 0xc9f, 0xccd, 0xcab, 0xc95, 0xccb, 0xcab,
0xccb, 0xcab, 0xcbf, 0xc9c, 0xcbf, 0xcab, 0xcc1, 0xca8, 0xcbe, 0xcab, 0xcc1, 0xc9f,
0xcbf, 0xc97, 0xcb2, 0xcbe, 0xcaa, 0xc97, 0xccb, 0xcb8, 0xccd, 0xc97, 0xccd, 0xcaf,
0xcbe, 0xc82, 0xcac, 0xcbf, 0xcaf, 0xcb0, 0xccd, 0xc97, 0xccd, 0xcb5, 0xcbe, 0xca1,
0xcb2, 0xccd, 0xc95, 0xcc6, 0xca8, 0xcbe, 0xcb2, 0xccd, 0xc97, 0xccd, 0xcb5, 0xcbe,
0xcae, 0xccd, 0xcb9, 0xcca, 0xca8, 0xcb2, 0xcc1, 0xcb2, 0xcc1, 0xc8e, 0xc82, 0xca1,
0xcb0, 0xccd, 0xcac, 0xcb0, 0xcbf, 0xc95, 0xcbf, 0xcb0, 0xcbf, 0xca4, 0xcbf, 0xcae,
0xca4, 0xcbf, 0xc95, 0xccb, 0xcb8, 0xccd, 0xcb0, 0xcc6, 0xc95, 0xccd, 0xcb5, 0xcbe,
0xc9c, 0xcb2, 0xcc7, 0xca8, 0xccd, 0xcae, 0xc9c, 0xcc1, 0xcb0, 0xcca, 0xcae, 0xcbe,
0xcb0, 0xccd, 0xc95, 0xccd, 0xcaf, 0xcc2, 0xcb8, 0xcb8, 0xccd, 0xcae, 0xcbf, 0xca1,
0xccd, 0x200c, 0xcb5, 0xcc7, 0xca8, 0xccc, 0xcb0, 0xcc1, 0xca8, 0xcbf, 0xcaf, 0xcc1,
0xca8, 0xcbe, 0xcb0, 0xccd, 0xcab, 0xccb, 0xc95, 0xccd, 0xca8, 0xccd, 0xcaf, 0xcc2,
0xcae, 0xcbf, 0xcaf, 0xcbe, 0xcaa, 0xcbe, 0xc97, 0xcca, 0x20, 0xcaa, 0xcbe, 0xc97,
0xcca, 0xcaa, 0xcbe, 0xcb2, 0xcbe, 0xcb5, 0xccd, 0xcaa, 0xcbf, 0xc9f, 0xccd, 0x200c,
0xc95, 0xcc8, 0xcb0, 0xccd, 0xca8, 0xccd, 0xcaa, 0xccb, 0xca8, 0xccd, 0x200c, 0xcaa,
0xcc6, 0xcaa, 0xccb, 0xcb0, 0xccd, 0xc9f, 0xccd, 0x20, 0xcae, 0xccb, 0xcb0, 0xcbf,
0xcb8, 0xccd, 0x200c, 0xcac, 0xcc8, 0xcb0, 0xcb0, 0xccb, 0xc9f, 0xccb, 0xc82, 0xc97,
0xcbe, 0xcb8, 0xcc8, 0xcaa, 0xca8, 0xccd, 0xca4, 0xcb9, 0xcbf, 0xca4, 0xcbf, 0xca4,
0xcb0, 0xcbe, 0xcb5, 0xcbe, 0xc9f, 0xcca, 0xc82, 0xc97, 0xcbe, 0xc9f, 0xcbe, 0xcaa,
0xcc1, 0xcb5, 0xcc7, 0xc95, 0xccd, 0xcb5, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcbf, 0xcb8,
0xccd, 0x639, 0x627, 0x628, 0x650, 0x62f, 0x62c, 0x627, 0x646, 0x627, 0x620, 0x6a9,
0x631, 0x627, 0x639, 0x62f, 0x6cc, 0x633, 0x20, 0x627, 0x628, 0x627, 0x628, 0x627,
0x627, 0x64e, 0x644, 0x62c, 0x6cc, 0x656, 0x631, 0x650, 0x6cc, 0x627, 0x627, 0x64e,
0x633, 0x645, 0x6cc, 0x631, 0x627, 0x628, 0x645, 0x627, 0x6a9, 0x648, 0x628, 0x620,
0x646, 0x6af, 0x648, 0x657, 0x6cc, 0x6cc, 0x628, 0x64e, 0x646, 0x62c, 0x648, 0x657,
0x644, 0x628, 0x650, 0x633, 0x627, 0x624, 0x6ba, 0x628, 0x644, 0x627, 0x646, 0x62a,
0x64e, 0x6cc, 0x631, 0x628, 0x631, 0x627, 0x632, 0x627, 0x648, 0x650, 0x644, 0x628,
0x64f, 0x62c, 0x64f, 0x645, 0x628, 0x64f, 0x631, 0x627, 0x6a9, 0x64e, 0x6cc, 0x631,
0x648, 0x6a9, 0x620, 0x633, 0x627, 0x628, 0x644, 0x620, 0x646, 0x6a9, 0x627, 0x6a9,
0x6cc, 0x648, 0x657, 0x679, 0x627, 0x6a9, 0x648, 0x646, 0x627, 0x6a9, 0x631, 0x6cc,
0x62f, 0x64e, 0x6a9, 0x627, 0x631, 0x62f, 0x627, 0x631, 0x627, 0x644, 0x633, 0x644,
0x627, 0x645, 0x688, 0x650, 0x62c, 0x6cc, 0x628, 0x648, 0x679, 0x6cc, 0x688, 0x648,
0x648, 0x644, 0x627, 0x627, 0x644, 0x20, 0x639, 0x6cc, 0x648, 0x646, 0x641, 0x631,
0x6cc, 0x20, 0x679, 0x627, 0x648, 0x64f, 0x646, 0x6af, 0x620, 0x628, 0x648, 0x631,
0x648, 0x646, 0x6c1, 0x64e, 0x631, 0x627, 0x631, 0x6cc, 0x6d2, 0x62c, 0x648, 0x6c1,
0x627, 0x646, 0x633, 0x628, 0x631, 0x6af, 0x6a9, 0x64e, 0x645, 0x67e, 0x627, 0x644,
0x627, 0x6a9, 0x6be, 0x627, 0x631, 0x62a, 0x648, 0x645, 0x6a9, 0x650, 0x6af, 0x627,
0x644, 0x6cc, 0x6a9, 0x650, 0x646, 0x634, 0x627, 0x633, 0x627, 0x644, 0x6cc, 0x6af,
0x648, 0x633, 0x644, 0x650, 0x628, 0x64e, 0x631, 0x648, 0x650, 0x644, 0x644, 0x6cc,
0x648, 0x627, 0x646, 0x688, 0x627, 0x644, 0x648, 0x628, 0x645, 0x20, 0x628, 0x627,
0x634, 0x6cc, 0x644, 0x648, 0x633, 0x627, 0x6a9, 0x627, 0x645, 0x64e, 0x67e, 0x648,
0x657, 0x679, 0x648, 0x645, 0x64e, 0x633, 0x6cc, 0x631, 0x648, 0x657, 0x645, 0x627,
0x628, 0x6cc, 0x646, 0x645, 0x648, 0x6af, 0x627, 0x62f, 0x650, 0x634, 0x648, 0x657,
0x645, 0x648, 0x646, 0x631, 0x648, 0x648, 0x650, 0x6cc, 0x627, 0x646, 0x6cc, 0x631,
0x648, 0x628, 0x6cc, 0x62c, 0x64e, 0x645, 0x6cc, 0x656, 0x646, 0x627, 0x646, 0x64e,
0x6cc, 0x645, 0x6cc, 0x6d2, 0x646, 0x648, 0x648, 0x627, 0x6a9, 0x686, 0x6be, 0x648,
0x62a, 0x627, 0x648, 0x622, 0x6af, 0x62f, 0x648, 0x6af, 0x648, 0x67e, 0x648, 0x631,
0x679, 0x648, 0x20, 0x646, 0x648, 0x648, 0x648, 0x633, 0x627, 0x648, 0x20, 0x679,
0x648, 0x645, 0x62a, 0x631, 0x67e, 0x648, 0x644, 0x6cc, 0x679, 0x648, 0x657, 0x646,
0x650, 0x633, 0x648, 0x650, 0x646, 0x688, 0x6c1, 0x648, 0x6a9, 0x627, 0x62f, 0x6a9,
0x627, 0x64e, 0x646, 0x6a9, 0x648, 0x631, 0x627, 0x62c, 0x627, 0x620, 0x646, 0x6af,
0x650, 0x648, 0x644, 0x627, 0x627, 0x620, 0x646, 0x679, 0x650, 0x6af, 0x64f, 0x648,
0x627, 0x627, 0x620, 0x631, 0x6cc, 0x6af, 0x64f, 0x648, 0x6cc, 0x646, 0x627, 0x628,
0x6cc, 0x648, 0x657, 0x646, 0x64e, 0x633, 0x20, 0x622, 0x6cc, 0x631, 0x64e, 0x633,
0x6a9, 0x6cc, 0x679, 0x627, 0x645, 0x627, 0x631, 0x6a9, 0x627, 0x6a9, 0x648, 0x631,
0x691, 0x648, 0x628, 0x627, 0x62c, 0x64f, 0x62c, 0x648, 0x6cc, 0x6d2, 0x644, 0x627,
0x20, 0x631, 0x650, 0x6cc, 0x648, 0x62c, 0x627, 0x645, 0x620, 0x646, 0x691, 0x648,
0x632, 0x627, 0x631, 0x650, 0x6cc, 0x648, 0x20, 0x6af, 0x627, 0x644, 0x6cc, 0x6af,
0x648, 0x633, 0x633, 0x627, 0x644, 0x679, 0x627, 0x633, 0x6cc, 0x646, 0x20, 0x62c,
0x64f, 0x648, 0x627, 0x6ba, 0x633, 0x6cc, 0x646, 0x20, 0x644, 0x648, 0x657, 0x6cc,
0x650, 0x633, 0x679, 0x648, 0x6a9, 0x648, 0x657, 0x645, 0x64e, 0x646, 0x627, 0x64f,
0x634, 0x64f, 0x648, 0x657, 0x627, 0x6cc, 0x6cc, 0x627, 0x627, 0x64e, 0x631, 0x648,
0x657, 0x628, 0x627, 0x627, 0x64e, 0x633, 0x64f, 0x646, 0x686, 0x650, 0x6cc, 0x64e,
0x646, 0x627, 0x679, 0x6cc, 0x20, 0x6a9, 0x648, 0x6a9, 0x646, 0x64d, 0x628, 0x64e,
0x6c1, 0x650, 0x6cc, 0x627, 0x628, 0x627, 0x6c1, 0x6cc, 0x627, 0x20, 0x628, 0x646,
0x62f, 0x627, 0x631, 0x633, 0x628, 0x627, 0x631, 0x628, 0x627, 0x688, 0x648, 0x633,
0x628, 0x620, 0x644, 0x620, 0x645, 0x628, 0x620, 0x644, 0x6cc, 0x656, 0x632, 0x628,
0x644, 0x627, 0x646, 0x6a9, 0x20, 0x633, 0x620, 0x628, 0x644, 0x64e, 0x646, 0x628,
0x648, 0x622, 0x20, 0x648, 0x650, 0x633, 0x679, 0x627, 0x628, 0x648, 0x6af, 0x648,
0x679, 0x627, 0x628, 0x648, 0x6cc, 0x650, 0x633, 0x6a9, 0x6cc, 0x645, 0x628, 0x631,
0x650, 0x62c, 0x20, 0x62e, 0x644, 0x6cc, 0x656, 0x62c, 0x6a9, 0x620, 0x645, 0x67e,
0x648, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x691, 0x6a9, 0x6cc, 0x646, 0x6a9, 0x64e,
0x646, 0x6a9, 0x6cc, 0x631, 0x64e, 0x6a9, 0x64e, 0x633, 0x6a9, 0x64e, 0x6cc, 0x6cc,
0x646, 0x6a9, 0x6cc, 0x645, 0x64e, 0x646, 0x634, 0x650, 0x6a9, 0x627, 0x6af, 0x648,
0x686, 0x650, 0x6c1, 0x64f, 0x648, 0x627, 0x20, 0x6c1, 0x64f, 0x648, 0x627, 0x6a9,
0x648, 0x633, 0x679, 0x627, 0x20, 0x631, 0x6cc, 0x656, 0x6a9, 0x627, 0x6a9, 0x631,
0x6cc, 0x633, 0x679, 0x646, 0x6a9, 0x648, 0x657, 0x6cc, 0x627, 0x628, 0x627, 0x6a9,
0x6cc, 0x648, 0x657, 0x631, 0x627, 0x6a9, 0x627, 0x627, 0x648, 0x688, 0x646, 0x645,
0x627, 0x631, 0x6a9, 0x20, 0x634, 0x627, 0x648, 0x646, 0x688, 0x627, 0x648, 0x633,
0x646, 0x688, 0x648, 0x627, 0x633, 0x64e, 0x646, 0x20, 0x6a9, 0x631, 0x6cc, 0x656,
0x6a9, 0x688, 0x6cc, 0x646, 0x648, 0x64e, 0x631, 0x688, 0x6cc, 0x679, 0x631, 0x627,
0x6cc, 0x650, 0x679, 0x688, 0x648, 0x645, 0x650, 0x646, 0x650, 0x6a9, 0x627, 0x627,
0x620, 0x688, 0x645, 0x64e, 0x646, 0x679, 0x64e, 0x646, 0x627, 0x6cc, 0x656, 0x631,
0x648, 0x657, 0x646, 0x6cc, 0x67e, 0x627, 0x6cc, 0x644, 0x20, 0x633, 0x64e, 0x644,
0x648, 0x6cc, 0x62f, 0x64e, 0x631, 0x641, 0x648, 0x631, 0x679, 0x20, 0x646, 0x6cc,
0x644, 0x633, 0x646, 0x641, 0x648, 0x631, 0x679, 0x6cc, 0x644, 0x6cc, 0x632, 0x627,
0x6af, 0x644, 0x6cc, 0x633, 0x20, 0x62e, 0x644, 0x6cc, 0x656, 0x62c, 0x6af, 0x648,
0x657, 0x633, 0x20, 0x62e, 0x644, 0x6cc, 0x656, 0x62c, 0x6af, 0x631, 0x6cc, 0x646,
0x688, 0x20, 0x62a, 0x654, 0x631, 0x6a9, 0x6af, 0x631, 0x6cc, 0x646, 0x627, 0x688,
0x627, 0x6af, 0x648, 0x691, 0x644, 0x648, 0x67e, 0x6af, 0x648, 0x627, 0x679, 0x6cc,
0x645, 0x627, 0x644, 0x627, 0x6af, 0x648, 0x627, 0x6cc, 0x627, 0x6a9, 0x650, 0x648,
0x64e, 0x644, 0x6af, 0x64f, 0x6cc, 0x627, 0x646, 0x627, 0x62d, 0x6cc, 0x644, 0x650,
0x641, 0x620, 0x6a9, 0x633, 0x62d, 0x648, 0x627, 0x646, 0x627, 0x6c2, 0x631, 0x645,
0x648, 0x633, 0x650, 0x644, 0x648, 0x627, 0x650, 0x646, 0x688, 0x6cc, 0x64e, 0x646,
0x20, 0x67e, 0x648, 0x644, 0x650, 0x633, 0x646, 0x648, 0x6a9, 0x633, 0x645, 0x6cc,
0x631, 0x646, 0x6af, 0x648, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646,
0x627, 0x67e, 0x650, 0x679, 0x633, 0x20, 0x628, 0x654, 0x631, 0x6af, 0x60c, 0x20,
0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x679, 0x6cc, 0x644, 0x20, 0x633,
0x650, 0x679, 0x6cc, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627,
0x648, 0x6cc, 0x648, 0x6cc, 0x6d2, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627,
0x646, 0x627, 0x648, 0x650, 0x646, 0x633, 0x6cc, 0x646, 0x6cc, 0x633, 0x60c, 0x20,
0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x650, 0x646, 0x6cc, 0x645,
0x6cc, 0x6a9, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x627,
0x650, 0x646, 0x648, 0x657, 0x648, 0x650, 0x6a9, 0x627, 0x650, 0x642, 0x627, 0x644,
0x6cc, 0x648, 0x657, 0x6cc, 0x650, 0x62a, 0x62c, 0x64e, 0x645, 0x64e, 0x6cc, 0x6a9,
0x627, 0x62c, 0x648, 0x657, 0x646, 0x6cc, 0x644, 0x648, 0x657, 0x6cc, 0x650, 0x633,
0x20, 0x648, 0x650, 0x644, 0x645, 0x648, 0x646, 0x679, 0x650, 0x633, 0x6cc, 0x644,
0x648, 0x60c, 0x20, 0x6a9, 0x6cc, 0x646, 0x679, 0x648, 0x6a9, 0x6cc, 0x6a9, 0x631,
0x627, 0x644, 0x6cc, 0x646, 0x688, 0x650, 0x6a9, 0x644, 0x627, 0x20, 0x67e, 0x627,
0x632, 0x644, 0x650, 0x645, 0x627, 0x644, 0x627, 0x633, 0x20, 0x627, 0x6cc, 0x646,
0x62c, 0x655, 0x644, 0x632, 0x644, 0x648, 0x648, 0x631, 0x20, 0x67e, 0x631, 0x646,
0x633, 0x20, 0x6a9, 0x648, 0x627, 0x631, 0x679, 0x631, 0x645, 0x6cc, 0x633, 0x650,
0x6cc, 0x648, 0x648, 0x645, 0x64e, 0x646, 0x627, 0x6af, 0x648, 0x627, 0x645, 0x64e,
0x646, 0x648, 0x633, 0x645, 0x6cc, 0x631, 0x6cc, 0x6af, 0x648, 0x679, 0x645, 0x627,
0x631, 0x679, 0x650, 0x646, 0x650, 0x6a9, 0x645, 0x6cc, 0x679, 0x627, 0x645, 0x648,
0x631, 0x648, 0x633, 0x645, 0x64e, 0x632, 0x64e, 0x679, 0x644, 0x627, 0x646, 0x645,
0x6cc, 0x646, 0x648, 0x645, 0x650, 0x646, 0x6cc, 0x645, 0x6cc, 0x631, 0x650, 0x688,
0x627, 0x645, 0x6cc, 0x679, 0x644, 0x627, 0x20, 0x6a9, 0x627, 0x679, 0x644, 0x627,
0x645, 0x6cc, 0x6a9, 0x633, 0x650, 0x6a9, 0x648, 0x20, 0x633, 0x650, 0x679, 0x6cc,
0x645, 0x650, 0x6a9, 0x6cc, 0x656, 0x644, 0x64e, 0x646, 0x645, 0x648, 0x646, 0x6a9,
0x679, 0x655, 0x646, 0x645, 0x648, 0x679, 0x6cc, 0x631, 0x6cc, 0x645, 0x648, 0x646,
0x679, 0x6cc, 0x648, 0x650, 0x688, 0x6cc, 0x648, 0x645, 0x648, 0x646, 0x698, 0x6cc,
0x631, 0x627, 0x62a, 0x646, 0x633, 0x627, 0x624, 0x6ba, 0x646, 0x650, 0x648, 0x20,
0x6cc, 0x627, 0x631, 0x6a9, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6c1, 0x627, 0x628,
0x6cc, 0x648, 0x644, 0x627, 0x6c1, 0x60c, 0x20, 0x634, 0x64f, 0x645, 0x627, 0x644,
0x6cc, 0x20, 0x688, 0x6cc, 0x6a9, 0x648, 0x679, 0x627, 0x645, 0x64e, 0x631, 0x6a9,
0x632, 0x6cc, 0x20, 0x62c, 0x646, 0x648, 0x657, 0x628, 0x6cc, 0x20, 0x688, 0x6a9,
0x648, 0x679, 0x627, 0x646, 0x648, 0x648, 0x20, 0x633, 0x6cc, 0x644, 0x655, 0x645,
0x60c, 0x20, 0x634, 0x64f, 0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688, 0x6cc, 0x6a9,
0x648, 0x679, 0x627, 0x646, 0x648, 0x657, 0x6a9, 0x627, 0x648, 0x62c, 0x6cc, 0x20,
0x646, 0x627, 0x6af, 0x627, 0x67e, 0x64e, 0x646, 0x627, 0x645, 0x627, 0x67e, 0x64e,
0x631, 0x627, 0x645, 0x627, 0x631, 0x650, 0x628, 0x648, 0x67e, 0x6be, 0x650, 0x646,
0x650, 0x6a9, 0x633, 0x67e, 0x648, 0x679, 0x20, 0x622, 0x624, 0x6ba, 0x20, 0x67e,
0x631, 0x650, 0x646, 0x633, 0x67e, 0x648, 0x679, 0x20, 0x622, 0x641, 0x20, 0x633,
0x67e, 0x6cc, 0x646, 0x67e, 0x648, 0x679, 0x648, 0x20, 0x648, 0x620, 0x644, 0x6c1,
0x648, 0x67e, 0x6cc, 0x656, 0x679, 0x648, 0x20, 0x631, 0x650, 0x6a9, 0x648, 0x67e,
0x648, 0x646, 0x679, 0x627, 0x20, 0x627, 0x631, 0x6cc, 0x646, 0x627, 0x633, 0x631,
0x6cc, 0x646, 0x6a9, 0x650, 0x646, 0x20, 0x627, 0x650, 0x646, 0x644, 0x620, 0x679,
0x631, 0x6cc, 0x633, 0x6cc, 0x641, 0x631, 0x620, 0x62c, 0x6cc, 0x656, 0x646, 0x627,
0x631, 0x650, 0x633, 0x648, 0x644, 0x6cc, 0x648, 0x657, 0x679, 0x631, 0x650, 0x6cc,
0x648, 0x20, 0x628, 0x631, 0x627, 0x646, 0x6a9, 0x648, 0x633, 0x627, 0x646, 0x62a,
0x627, 0x631, 0x6cc, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x6cc, 0x627, 0x6af, 0x648,
0x633, 0x620, 0x646, 0x679, 0x648, 0x20, 0x691, 0x648, 0x645, 0x650, 0x646, 0x6af,
0x648, 0x633, 0x627, 0x624, 0x20, 0x67e, 0x627, 0x644, 0x648, 0x633, 0x6a9, 0x648,
0x631, 0x655, 0x633, 0x628, 0x627, 0x6d2, 0x633, 0x64e, 0x646, 0x691, 0x633, 0x650,
0x679, 0x6a9, 0x627, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x628, 0x627, 0x631, 0x62a,
0x6be, 0x6cc, 0x644, 0x645, 0x6cc, 0x633, 0x620, 0x646, 0x679, 0x20, 0x62c, 0x648,
0x646, 0x633, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x6a9, 0x650, 0x679, 0x633, 0x633,
0x620, 0x646, 0x679, 0x20, 0x644, 0x648, 0x657, 0x633, 0x650, 0x6cc, 0x627, 0x633,
0x6cc, 0x646, 0x679, 0x20, 0x62a, 0x6be, 0x627, 0x645, 0x64e, 0x633, 0x633, 0x6cc,
0x646, 0x679, 0x20, 0x648, 0x650, 0x646, 0x633, 0x6cc, 0x646, 0x679, 0x633, 0x655,
0x648, 0x650, 0x641, 0x679, 0x20, 0x6a9, 0x64e, 0x631, 0x64e, 0x646, 0x679, 0x679,
0x6cc, 0x6af, 0x648, 0x633, 0x6cc, 0x20, 0x6af, 0x627, 0x644, 0x67e, 0x627, 0x62a,
0x6be, 0x6cc, 0x648, 0x657, 0x644, 0x6d2, 0x62a, 0x650, 0x62c, 0x64f, 0x648, 0x627,
0x646, 0x627, 0x679, 0x648, 0x631, 0x648, 0x646, 0x679, 0x648, 0x679, 0x627, 0x631,
0x679, 0x648, 0x644, 0x627, 0x648, 0x620, 0x646, 0x6a9, 0x648, 0x648, 0x64e, 0x631,
0x648, 0x627, 0x6cc, 0x650, 0x679, 0x20, 0x6c1, 0x627, 0x631, 0x655, 0x633, 0x648,
0x650, 0x646, 0x650, 0x67e, 0x620, 0x6af, 0x6cc, 0x6a9, 0x648, 0x657, 0x62a, 0x627,
0x62a, 0x6a9, 0x6cc, 0x633, 0x6cc, 0x688, 0x6cc, 0x648, 0x650, 0x633, 0x688, 0x64f,
0x645, 0x648, 0x646, 0x679, 0x20, 0x688, 0x20, 0x627, 0x64e, 0x631, 0x648, 0x650,
0x644, 0x645, 0x627, 0x633, 0x64e, 0x646, 0x645, 0x620, 0x6a9, 0x20, 0x645, 0x672,
0x691, 0x648, 0x67e, 0x627, 0x645, 0x64e, 0x631, 0x631, 0x648, 0x62a, 0x6be, 0x6cc,
0x631, 0x627, 0x633, 0x6cc, 0x648, 0x648, 0x627, 0x648, 0x648, 0x633, 0x62a, 0x648,
0x6a9, 0x644, 0x648, 0x646, 0x6af, 0x20, 0x6cc, 0x6cc, 0x626, 0x631, 0x20, 0x628,
0x626, 0x6cc, 0x646, 0x627, 0x6cc, 0x688, 0x655, 0x646, 0x627, 0x64e, 0x644, 0x645,
0x627, 0x679, 0x6cc, 0x627, 0x64e, 0x645, 0x627, 0x646, 0x627, 0x64e, 0x646, 0x64e,
0x62f, 0x6cc, 0x631, 0x627, 0x6a9, 0x679, 0x648, 0x627, 0x64e, 0x642, 0x679, 0x648,
0x628, 0x627, 0x64e, 0x634, 0x6af, 0x64e, 0x628, 0x64e, 0x62a, 0x627, 0x679, 0x6cc,
0x631, 0x648, 0x628, 0x620, 0x6c1, 0x631, 0x6cc, 0x656, 0x646, 0x628, 0x627, 0x642,
0x648, 0x657, 0x628, 0x6cc, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x6a9, 0x628, 0x631,
0x646, 0x648, 0x644, 0x628, 0x6cc, 0x631, 0x679, 0x628, 0x650, 0x634, 0x6a9, 0x6cc,
0x6a9, 0x628, 0x631, 0x648, 0x657, 0x646, 0x64e, 0x6d2, 0x686, 0x6cc, 0x679, 0x627,
0x6a9, 0x648, 0x644, 0x64e, 0x645, 0x628, 0x648, 0x62f, 0x64e, 0x645, 0x64e, 0x633,
0x6a9, 0x64e, 0x633, 0x688, 0x6be, 0x627, 0x6a9, 0x627, 0x62f, 0x650, 0x644, 0x6cc,
0x62f, 0x64f, 0x628, 0x626, 0x6cc, 0x62f, 0x64f, 0x634, 0x627, 0x646, 0x628, 0x6cc,
0x6d2, 0x641, 0x627, 0x645, 0x627, 0x20, 0x6af, 0x648, 0x633, 0x679, 0x627, 0x63a,
0x632, 0x6c1, 0x6c1, 0x6cc, 0x628, 0x631, 0x648, 0x646, 0x633, 0x6cc, 0x6af, 0x64e,
0x646, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af, 0x62d, 0x648,
0x648, 0x691, 0x627, 0x650, 0x631, 0x6a9, 0x64f, 0x633, 0x6a9, 0x62c, 0x6a9, 0x627,
0x631, 0x62a, 0x627, 0x62c, 0x64e, 0x6cc, 0x627, 0x67e, 0x648, 0x657, 0x631, 0x627,
0x6cc, 0x631, 0x648, 0x634, 0x644, 0x645, 0x642, 0x627, 0x628, 0x64f, 0x644, 0x6a9,
0x64e, 0x645, 0x686, 0x6be, 0x679, 0x6a9, 0x627, 0x6a9, 0x64e, 0x631, 0x627, 0x686,
0x6cc, 0x6a9, 0x627, 0x679, 0x6be, 0x645, 0x64e, 0x646, 0x688, 0x648, 0x657, 0x6a9,
0x6be, 0x627, 0x646, 0x688, 0x6cc, 0x6af, 0x627, 0x6a9, 0x64e, 0x644, 0x655, 0x6a9,
0x64e, 0x62a, 0x627, 0x6a9, 0x631, 0x646, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631,
0x633, 0x6a9, 0x6a9, 0x648, 0x644, 0x627, 0x644, 0x64e, 0x645, 0x67e, 0x648, 0x657,
0x631, 0x6a9, 0x64f, 0x686, 0x650, 0x646, 0x6af, 0x6a9, 0x64f, 0x648, 0x6cc, 0x62a,
0x645, 0x6a9, 0x648, 0x645, 0x64e, 0x6af, 0x627, 0x62f, 0x64e, 0x646, 0x645, 0x64e,
0x6a9, 0x64e, 0x633, 0x64e, 0x631, 0x645, 0x64e, 0x646, 0x6cc, 0x656, 0x644, 0x627,
0x646, 0x650, 0x6a9, 0x648, 0x633, 0x650, 0x6cc, 0x627, 0x646, 0x648, 0x648, 0x648,
0x6a9, 0x64f, 0x632, 0x646, 0x6cc, 0x679, 0x633, 0x6a9, 0x646, 0x648, 0x648, 0x648,
0x633, 0x650, 0x628, 0x650, 0x631, 0x633, 0x6a9, 0x627, 0x648, 0x645, 0x633, 0x6a9,
0x627, 0x648, 0x631, 0x64e, 0x644, 0x646, 0x648, 0x645, 0x20, 0x67e, 0x620, 0x646,
0x6c1, 0x6c1, 0x67e, 0x648, 0x646, 0x62a, 0x650, 0x639, 0x627, 0x646, 0x6a9, 0x67e,
0x6cc, 0x648, 0x646, 0x6af, 0x6cc, 0x627, 0x646, 0x6af, 0x6a9, 0x648, 0x633, 0x679,
0x627, 0x646, 0x6d2, 0x642, 0x620, 0x632, 0x620, 0x644, 0x648, 0x691, 0x627, 0x631,
0x6cc, 0x627, 0x636, 0x633, 0x64e, 0x6a9, 0x6be, 0x627, 0x644, 0x650, 0x646, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x6a9, 0x64e, 0x646, 0x62f, 0x633, 0x6cc, 0x648, 0x644,
0x634, 0x627, 0x646, 0x6af, 0x6d2, 0x633, 0x650, 0x646, 0x6af, 0x627, 0x67e, 0x648,
0x631, 0x633, 0x631, 0x6cc, 0x688, 0x646, 0x6cc, 0x6a9, 0x648, 0x644, 0x6cc, 0x645,
0x633, 0x6a9, 0x62a, 0x64e, 0x6cc, 0x67e, 0x6cc, 0x6d2, 0x62a, 0x627, 0x634, 0x6a9,
0x6cc, 0x646, 0x679, 0x628, 0x650, 0x644, 0x650, 0x633, 0x6cc, 0x62a, 0x620, 0x6c1,
0x631, 0x627, 0x646, 0x62a, 0x6be, 0x650, 0x645, 0x67e, 0x648, 0x657, 0x679, 0x648,
0x6a9, 0x6cc, 0x648, 0x679, 0x648, 0x645, 0x633, 0x6a9, 0x627, 0x648, 0x644, 0x646,
0x20, 0x628, 0x627, 0x679, 0x631, 0x627, 0x64f, 0x631, 0x648, 0x645, 0x642, 0x6cc,
0x627, 0x648, 0x633, 0x679, 0x2d, 0x646, 0x6cc, 0x631, 0x627, 0x648, 0x650, 0x6cc,
0x64e, 0x646, 0x62a, 0x650, 0x6cc, 0x6cc, 0x646, 0x644, 0x627, 0x62f, 0x650, 0x648,
0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x6cc, 0x6a9, 0x648, 0x633, 0x6a9, 0x631, 0x646,
0x6af, 0x648, 0x657, 0x646, 0x6cc, 0x64e, 0x6a9, 0x627, 0x62a, 0x650, 0x631, 0x650,
0x646, 0x628, 0x654, 0x631, 0x6af, 0x6cc, 0x6d2, 0x631, 0x6cc, 0x648, 0x64e, 0x646,
0x627, 0x64e, 0x632, 0x648, 0x631, 0x64e, 0x633, 0x628, 0x631, 0x645, 0x648, 0x657,
0x691, 0x627, 0x6a9, 0x620, 0x646, 0x64e, 0x631, 0x6cc, 0x6a9, 0x6cc, 0x67e, 0x20,
0x624, 0x631, 0x691, 0x6d2, 0x641, 0x620, 0x631, 0x648, 0x645, 0x64e, 0x688, 0x6cc,
0x656, 0x631, 0x627, 0x631, 0x620, 0x6a9, 0x6cc, 0x627, 0x648, 0x650, 0x6a9, 0x62c,
0x646, 0x648, 0x628, 0x6cc, 0x20, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x633,
0x6cc, 0x646, 0x679, 0x20, 0x6c1, 0x6cc, 0x644, 0x650, 0x646, 0x627, 0x633, 0x679,
0x6cc, 0x646, 0x644, 0x6cc, 0x627, 0x620, 0x691, 0x650, 0x644, 0x6cc, 0x691, 0x628,
0x631, 0x633, 0x628, 0x6cc, 0x646, 0x628, 0x631, 0x648, 0x6a9, 0x655, 0x646, 0x20,
0x6c1, 0x650, 0x644, 0x688, 0x627, 0x631, 0x648, 0x650, 0x646, 0x6cc, 0x648, 0x657,
0x6a9, 0x644, 0x627, 0x62d, 0x6c4, 0x628, 0x672, 0x679, 0x644, 0x650, 0x646, 0x691,
0x6cc, 0x645, 0x627, 0x646, 0x644, 0x639, 0x627, 0x691, 0x20, 0x6c1, 0x648, 0x648,
0x6d2, 0x645, 0x620, 0x644, 0x628, 0x639, 0x627, 0x631, 0x646, 0x67e, 0x654, 0x631,
0x62a, 0x6be, 0x633, 0x650, 0x691, 0x646, 0x6cc, 0x627, 0x6cc, 0x645, 0x633, 0x679,
0x64e, 0x631, 0x688, 0x6cc, 0x645, 0x627, 0x64e, 0x646, 0x688, 0x648, 0x631, 0x627,
0x627, 0x6cc, 0x633, 0x679, 0x631, 0x627, 0x20, 0x6a9, 0x6be, 0x627, 0x646, 0x627,
0x620, 0x62a, 0x6be, 0x655, 0x646, 0x633, 0x628, 0x6cc, 0x644, 0x6af, 0x631, 0x6cc,
0x688, 0x628, 0x654, 0x631, 0x644, 0x650, 0x646, 0x628, 0x631, 0x6cc, 0x679, 0x6cc,
0x633, 0x644, 0x627, 0x648, 0x627, 0x628, 0x631, 0x633, 0x655, 0x644, 0x633, 0x628,
0x64e, 0x686, 0x627, 0x631, 0x6cc, 0x633, 0x679, 0x628, 0x64f, 0x688, 0x627, 0x67e,
0x6cc, 0x633, 0x679, 0x628, 0x648, 0x633, 0x650, 0x646, 0x6af, 0x6cc, 0x646, 0x686,
0x650, 0x633, 0x6cc, 0x656, 0x646, 0x648, 0x6a9, 0x648, 0x67e, 0x646, 0x20, 0x6c1,
0x6cc, 0x6af, 0x646, 0x688, 0x64e, 0x628, 0x644, 0x650, 0x646, 0x6af, 0x650, 0x628,
0x631, 0x627, 0x644, 0x679, 0x64e, 0x631, 0x6af, 0x648, 0x6cc, 0x631, 0x646, 0x633,
0x6d2, 0x62d, 0x620, 0x644, 0x633, 0x650, 0x646, 0x6a9, 0x6cc, 0x622, 0x626, 0x644,
0x20, 0x622, 0x641, 0x20, 0x645, 0x6cc, 0x646, 0x627, 0x650, 0x633, 0x62a, 0x627,
0x646, 0x628, 0x64f, 0x644, 0x62c, 0x631, 0x633, 0x6cc, 0x6a9, 0x64e, 0x644, 0x650,
0x646, 0x627, 0x650, 0x646, 0x6af, 0x631, 0x64e, 0x62f, 0x6a9, 0x6cc, 0x631, 0x648,
0x6a9, 0x6cc, 0x656, 0x648, 0x644, 0x650, 0x633, 0x628, 0x64e, 0x646, 0x644, 0x650,
0x6cc, 0x648, 0x657, 0x628, 0x20, 0x644, 0x650, 0x6cc, 0x627, 0x646, 0x627, 0x644,
0x64e, 0x646, 0x62f, 0x64e, 0x646, 0x644, 0x64e, 0x6a9, 0x632, 0x655, 0x645, 0x628,
0x654, 0x631, 0x6af, 0x645, 0x6cc, 0x688, 0x631, 0x6cc, 0x688, 0x645, 0x627, 0x644,
0x679, 0x627, 0x645, 0x6cc, 0x631, 0x6cc, 0x20, 0x6c1, 0x6cc, 0x645, 0x645, 0x650,
0x646, 0x633, 0x6a9, 0x645, 0x648, 0x646, 0x627, 0x6a9, 0x648, 0x645, 0x627, 0x633,
0x6a9, 0x648, 0x627, 0x648, 0x633, 0x644, 0x648, 0x67e, 0x6cc, 0x631, 0x650, 0x633,
0x67e, 0x648, 0x688, 0x6af, 0x648, 0x631, 0x6cc, 0x6a9, 0x627, 0x67e, 0x631, 0x627,
0x6af, 0x631, 0x650, 0x6af, 0x627, 0x633, 0x64e, 0x645, 0x627, 0x631, 0x627, 0x633,
0x6cc, 0x646, 0x20, 0x645, 0x631, 0x6cc, 0x646, 0x648, 0x633, 0x627, 0x631, 0x627,
0x20, 0x62c, 0x6cc, 0x648, 0x648, 0x633, 0x627, 0x631, 0x627, 0x679, 0x648, 0x633,
0x650, 0x645, 0x641, 0x6cc, 0x631, 0x648, 0x67e, 0x648, 0x644, 0x633, 0x650, 0x6a9,
0x648, 0x67e, 0x6cc, 0x6d2, 0x633, 0x648, 0x641, 0x650, 0x6cc, 0x627, 0x633, 0x679,
0x627, 0x6a9, 0x20, 0x6c1, 0x648, 0x644, 0x645, 0x679, 0x620, 0x644, 0x650, 0x646,
0x679, 0x650, 0x631, 0x6cc, 0x646, 0x627, 0x648, 0x644, 0x6cc, 0x627, 0x646, 0x648,
0x648, 0x633, 0x6a9, 0x648, 0x627, 0x62f, 0x64f, 0x632, 0x648, 0x6cc, 0x679, 0x6cc,
0x6a9, 0x646, 0x648, 0x650, 0x6cc, 0x64e, 0x646, 0x646, 0x627, 0x648, 0x650, 0x644,
0x650, 0x646, 0x650, 0x6cc, 0x64e, 0x633, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af,
0x631, 0x64e, 0x62f, 0x648, 0x627, 0x631, 0x633, 0x627, 0x632, 0x6af, 0x631, 0x6cc,
0x628, 0x632, 0x6cc, 0x648, 0x657, 0x631, 0x650, 0x6a9, 0x627, 0x620, 0x646, 0x679,
0x646, 0x627, 0x646, 0x631, 0x650, 0x648, 0x648, 0x686, 0x627, 0x6af, 0x648, 0x633,
0x6a9, 0x631, 0x650, 0x633, 0x645, 0x64e, 0x633, 0x6a9, 0x648, 0x6a9, 0x648, 0x633,
0x6a9, 0x648, 0x645, 0x648, 0x631, 0x648, 0x6a9, 0x6cc, 0x631, 0x6af, 0x648, 0x644,
0x650, 0x646, 0x645, 0x627, 0x6c1, 0x6cc, 0x6d2, 0x645, 0x627, 0x644, 0x62f, 0x6cc,
0x656, 0x648, 0x632, 0x645, 0x648, 0x631, 0x650, 0x634, 0x6cc, 0x633, 0x645, 0x6cc,
0x648, 0x679, 0x631, 0x650, 0x6cc, 0x648, 0x657, 0x646, 0x6cc, 0x64e, 0x646, 0x622,
0x67e, 0x650, 0x6cc, 0x627, 0x622, 0x6a9, 0x644, 0x6cc, 0x646, 0x688, 0x686, 0x64e,
0x62a, 0x6be, 0x64e, 0x645, 0x679, 0x631, 0x6a9, 0x627, 0x6cc, 0x656, 0x633, 0x679,
0x64e, 0x631, 0x627, 0x6cc, 0x641, 0x627, 0x62a, 0x6cc, 0x6d2, 0x641, 0x64e, 0x6a9,
0x64e, 0x648, 0x641, 0x648, 0x641, 0x650, 0x62c, 0x6cc, 0x641, 0x64f, 0x646, 0x627,
0x641, 0x648, 0x657, 0x62a, 0x6cc, 0x6af, 0x620, 0x644, 0x64e, 0x67e, 0x6af, 0x648,
0x633, 0x6af, 0x6cc, 0x645, 0x628, 0x650, 0x6cc, 0x64e, 0x631, 0x6af, 0x64f, 0x648,
0x627, 0x62f, 0x64e, 0x644, 0x686, 0x64e, 0x646, 0x64e, 0x644, 0x6c1, 0x648, 0x646,
0x648, 0x644, 0x648, 0x20, 0x644, 0x648, 0x627, 0x620, 0x646, 0x691, 0x631, 0x628,
0x6cc, 0x631, 0x6cc, 0x6a9, 0x650, 0x631, 0x650, 0x62a, 0x650, 0x645, 0x627, 0x62a,
0x6cc, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x6d2, 0x6a9, 0x64f, 0x648, 0x62c, 0x644,
0x6cc, 0x646, 0x645, 0x64e, 0x62c, 0x648, 0x657, 0x631, 0x648, 0x645, 0x627, 0x631,
0x6a9, 0x6cc, 0x633, 0x64e, 0x633, 0x645, 0x650, 0x688, 0x648, 0x6cc, 0x6d2, 0x646,
0x627, 0x648, 0x631, 0x648, 0x657, 0x646, 0x648, 0x631, 0x641, 0x648, 0x6a9, 0x646,
0x648, 0x645, 0x6cc, 0x6cc, 0x67e, 0x6cc, 0x6af, 0x648, 0x20, 0x67e, 0x6cc, 0x6af,
0x648, 0x67e, 0x64e, 0x644, 0x627, 0x648, 0x67e, 0x650, 0x679, 0x6a9, 0x6cc, 0x631,
0x64e, 0x646, 0x67e, 0x648, 0x646, 0x627, 0x67e, 0x6d2, 0x67e, 0x648, 0x679, 0x20,
0x645, 0x648, 0x631, 0x633, 0x628, 0x6cc, 0x631, 0x627, 0x631, 0x648, 0x679, 0x648,
0x646, 0x6af, 0x627, 0x633, 0x64e, 0x6cc, 0x67e, 0x627, 0x646, 0x62a, 0x64e, 0x6c1,
0x650, 0x62a, 0x6cc, 0x679, 0x64e, 0x631, 0x627, 0x648, 0x627, 0x62a, 0x648, 0x646,
0x6af, 0x627, 0x62a, 0x64e, 0x67e, 0x648, 0x657, 0x648, 0x6cc, 0x6a9, 0x648, 0x6cc,
0x644, 0x650, 0x633, 0x423, 0x448, 0x443, 0x430, 0x439, 0x44f, 0x41a, 0x435, 0x43c,
0x431, 0x440, 0x438, 0x434, 0x436, 0x2d, 0x411, 0x435, 0x439, 0x41a, 0x430, 0x439,
0x43c, 0x430, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x414, 0x43e, 0x441, 0x43e, 0x43d, 0x2d, 0x41a, 0x440, 0x438, 0x43a, 0x424, 0x43e,
0x440, 0x442, 0x2d, 0x41d, 0x435, 0x43b, 0x44c, 0x441, 0x43e, 0x43d, 0x412, 0x438,
0x432, 0x435, 0x439, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430,
0x411, 0x43e, 0x439, 0x43b, 0x430, 0x2c, 0x20, 0x421, 0x43e, 0x43b, 0x442, 0x4af,
0x441, 0x442, 0x456, 0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x421,
0x435, 0x43d, 0x442, 0x435, 0x440, 0x2c, 0x20, 0x421, 0x43e, 0x43b, 0x442, 0x4af,
0x441, 0x442, 0x456, 0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d,
0x44c, 0x44e, 0x20, 0x421, 0x435, 0x439, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x421,
0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x414, 0x430, 0x43a,
0x43e, 0x442, 0x430, 0x418, 0x43b, 0x43b, 0x43e, 0x43a, 0x43a, 0x43e, 0x440, 0x442,
0x43e, 0x43e, 0x440, 0x43c, 0x438, 0x443, 0x442, 0x421, 0x443, 0x438, 0x444, 0x442,
0x2d, 0x41a, 0x430, 0x440, 0x440, 0x435, 0x43d, 0x442, 0x414, 0x44d, 0x439, 0x432,
0x438, 0x441, 0x410, 0x49b, 0x442, 0x430, 0x443, 0x410, 0x49b, 0x442, 0x4e9, 0x431,
0x435, 0x411, 0x430, 0x493, 0x434, 0x430, 0x442, 0x411, 0x430, 0x440, 0x43d, 0x430,
0x443, 0x44b, 0x43b, 0x41d, 0x438, 0x43a, 0x43e, 0x441, 0x438, 0x44f, 0x41d, 0x43e,
0x432, 0x43e, 0x441, 0x456, 0x431, 0x456, 0x440, 0x41e, 0x43c, 0x431, 0x44b, 0x49a,
0x43e, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x439, 0x49a, 0x44b, 0x437, 0x44b, 0x43b,
0x43e, 0x440, 0x434, 0x430, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x49b, 0x430, 0x43d,
0x434, 0x4b0, 0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x440, 0x4ae, 0x440,
0x456, 0x43c, 0x448, 0x456, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x41a, 0x430, 0x43d, 0x430,
0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x424, 0x430,
0x440, 0x435, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x41e, 0x4a3, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x413, 0x435, 0x43e,
0x440, 0x433, 0x438, 0x44f, 0x4d8, 0x443, 0x43b, 0x438, 0x435, 0x20, 0x415, 0x43b,
0x435, 0x43d, 0x430, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x41c, 0x44d, 0x43d,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x421, 0x442, 0x430, 0x43c, 0x431, 0x4b1,
0x43b, 0x41c, 0x4d9, 0x441, 0x43a, 0x435, 0x443, 0x420, 0x43e, 0x436, 0x434, 0x435,
0x441, 0x442, 0x432, 0x43e, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x41a, 0x43e,
0x43a, 0x43e, 0x441, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x41a, 0x43e, 0x43c, 0x43e, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x44b, 0x41a, 0x443, 0x441, 0x430, 0x438, 0x435, 0x41c,
0x430, 0x440, 0x43a, 0x438, 0x437, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x627, 0x628, 0x64a, 0x62f, 0x62c, 0x627, 0x646, 0x627, 0x643, 0x643,
0x631, 0x627, 0x627, 0x62f, 0x62f, 0x64a, 0x633, 0x2d, 0x627, 0x628, 0x6d5, 0x628,
0x627, 0x627, 0x644, 0x62c, 0x64a, 0x631, 0x627, 0x633, 0x645, 0x627, 0x631, 0x627,
0x628, 0x627, 0x646, 0x6af, 0x64a, 0x628, 0x627, 0x646, 0x62c, 0x6cb, 0x644, 0x628,
0x64a, 0x633, 0x627, 0x6cb, 0x628, 0x631, 0x627, 0x632, 0x632, 0x627, 0x6c6, 0x64a,
0x644, 0x628, 0x6cb, 0x62c, 0x6cb, 0x645, 0x628, 0x6cb, 0x631, 0x627, 0x643, 0x627,
0x64a, 0x631, 0x643, 0x627, 0x633, 0x627, 0x628, 0x644, 0x627, 0x646, 0x643, 0x627,
0x633, 0x6d5, 0x6cb, 0x62a, 0x627, 0x62f, 0x627, 0x631, 0x2d, 0x6d5, 0x633, 0x2d,
0x633, 0x627, 0x644, 0x627, 0x645, 0x62c, 0x64a, 0x628, 0x6cb, 0x62a, 0x64a, 0x62f,
0x6cb, 0x627, 0x644, 0x627, 0x6d5, 0x644, 0x2d, 0x627, 0x64a, 0x6cb, 0x646, 0x641,
0x631, 0x64a, 0x62a, 0x627, 0x6cb, 0x646, 0x6af, 0x627, 0x628, 0x648, 0x631, 0x648,
0x646, 0x6d5, 0x62d, 0x627, 0x631, 0x627, 0x631, 0x6d5, 0x64a, 0x648, 0x62d, 0x627,
0x646, 0x646, 0x6d5, 0x633, 0x628, 0x6cb, 0x631, 0x6af, 0x62c, 0x6cb, 0x628, 0x627,
0x643, 0x627, 0x645, 0x67e, 0x627, 0x644, 0x627, 0x62d, 0x627, 0x631, 0x62a, 0x6cb,
0x645, 0x643, 0x64a, 0x6af, 0x627, 0x644, 0x64a, 0x644, 0x627, 0x6af, 0x648, 0x633,
0x644, 0x64a, 0x628, 0x631, 0x6d5, 0x6c6, 0x64a, 0x644, 0x644, 0x648, 0x645, 0x6d5,
0x644, 0x6cb, 0x627, 0x646, 0x62f, 0x627, 0x644, 0x6cb, 0x628, 0x6cb, 0x645, 0x628,
0x627, 0x634, 0x64a, 0x644, 0x6cb, 0x633, 0x627, 0x643, 0x627, 0x645, 0x627, 0x67e,
0x6cb, 0x62a, 0x6cb, 0x645, 0x627, 0x633, 0x6d5, 0x631, 0x6cb, 0x645, 0x628, 0x627,
0x628, 0x6d5, 0x646, 0x6d5, 0x645, 0x648, 0x6af, 0x627, 0x62f, 0x64a, 0x634, 0x6cb,
0x645, 0x648, 0x646, 0x631, 0x648, 0x6c6, 0x64a, 0x627, 0x646, 0x627, 0x64a, 0x631,
0x648, 0x628, 0x64a, 0x646, 0x62f, 0x62c, 0x627, 0x645, 0x6d5, 0x646, 0x627, 0x646,
0x64a, 0x627, 0x645, 0x6d5, 0x64a, 0x646, 0x6cb, 0x627, 0x643, 0x634, 0x648, 0x62a,
0x6cb, 0x627, 0x6af, 0x627, 0x62f, 0x6cb, 0x6af, 0x6cb, 0x67e, 0x648, 0x631, 0x62a,
0x648, 0x2d, 0x646, 0x648, 0x6c6, 0x648, 0x633, 0x627, 0x646, 0x2d, 0x62a, 0x648,
0x645, 0x6d5, 0x62a, 0x631, 0x64a, 0x67e, 0x648, 0x644, 0x64a, 0x62a, 0x6cb, 0x646,
0x64a, 0x633, 0x6c6, 0x64a, 0x646, 0x62f, 0x62d, 0x6cb, 0x643, 0x627, 0x62f, 0x627,
0x643, 0x627, 0x646, 0x643, 0x648, 0x631, 0x64a, 0x62c, 0x627, 0x646, 0x6af, 0x64a,
0x644, 0x64a, 0x627, 0x627, 0x646, 0x62a, 0x64a, 0x6af, 0x6cb, 0x627, 0x627, 0x631,
0x627, 0x6af, 0x6cb, 0x627, 0x64a, 0x646, 0x627, 0x628, 0x6cb, 0x6d5, 0x646, 0x648,
0x633, 0x2d, 0x627, 0x64a, 0x631, 0x648, 0x633, 0x62c, 0x6cb, 0x62c, 0x6cb, 0x64a,
0x644, 0x627, 0x2d, 0x631, 0x64a, 0x648, 0x62d, 0x627, 0x645, 0x6d5, 0x646, 0x62f,
0x648, 0x632, 0x627, 0x631, 0x64a, 0x648, 0x2d, 0x6af, 0x627, 0x644, 0x6d5, 0x6af,
0x648, 0x633, 0x633, 0x627, 0x644, 0x62a, 0x627, 0x633, 0x627, 0x646, 0x2d, 0x62d,
0x6cb, 0x627, 0x646, 0x633, 0x627, 0x646, 0x2d, 0x644, 0x6cb, 0x64a, 0x633, 0x62a,
0x6cb, 0x643, 0x6cb, 0x645, 0x627, 0x646, 0x6cb, 0x634, 0x6cb, 0x627, 0x64a, 0x627,
0x627, 0x631, 0x6cb, 0x628, 0x627, 0x627, 0x633, 0x6cb, 0x646, 0x633, 0x64a, 0x648,
0x646, 0x627, 0x62a, 0x64a, 0x643, 0x648, 0x643, 0x627, 0x646, 0x628, 0x627, 0x64a,
0x627, 0x628, 0x627, 0x64a, 0x627, 0x2d, 0x62f, 0x6d5, 0x2d, 0x628, 0x627, 0x646,
0x62f, 0x6d5, 0x631, 0x627, 0x633, 0x628, 0x627, 0x631, 0x628, 0x627, 0x62f, 0x648,
0x633, 0x628, 0x6d5, 0x644, 0x6d5, 0x645, 0x628, 0x6d5, 0x644, 0x64a, 0x632, 0x628,
0x627, 0x648, 0x2d, 0x6c6, 0x64a, 0x633, 0x62a, 0x627, 0x628, 0x648, 0x6af, 0x648,
0x62a, 0x627, 0x628, 0x648, 0x64a, 0x633, 0x6d5, 0x643, 0x6d5, 0x645, 0x628, 0x631,
0x64a, 0x62c, 0x2d, 0x628, 0x6d5, 0x64a, 0x643, 0x627, 0x645, 0x67e, 0x6cb, 0x2d,
0x6af, 0x631, 0x627, 0x646, 0x62f, 0x6d5, 0x643, 0x627, 0x646, 0x643, 0x6cb, 0x646,
0x643, 0x627, 0x64a, 0x6d5, 0x646, 0x646, 0x627, 0x643, 0x627, 0x64a, 0x645, 0x627,
0x646, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x686, 0x64a,
0x643, 0x627, 0x6af, 0x648, 0x686, 0x64a, 0x6cb, 0x627, 0x6cb, 0x627, 0x633, 0x64a,
0x6cb, 0x62f, 0x627, 0x62f, 0x2d, 0x62d, 0x6cb, 0x627, 0x631, 0x6d5, 0x633, 0x643,
0x648, 0x633, 0x62a, 0x627, 0x2d, 0x631, 0x64a, 0x643, 0x627, 0x643, 0x631, 0x6d5,
0x633, 0x62a, 0x648, 0x646, 0x643, 0x6cb, 0x64a, 0x627, 0x628, 0x627, 0x643, 0x64a,
0x6cb, 0x631, 0x627, 0x633, 0x627, 0x648, 0x62f, 0x627, 0x646, 0x645, 0x627, 0x631,
0x643, 0x633, 0x62d, 0x627, 0x6c6, 0x646, 0x62f, 0x648, 0x6cb, 0x633, 0x648, 0x646,
0x62f, 0x648, 0x6cb, 0x633, 0x648, 0x646, 0x2d, 0x643, 0x631, 0x64a, 0x643, 0x62f,
0x6d5, 0x646, 0x6c6, 0x6d5, 0x631, 0x62f, 0x6d5, 0x62a, 0x631, 0x648, 0x64a, 0x62f,
0x6d5, 0x62f, 0x645, 0x648, 0x646, 0x62a, 0x648, 0x646, 0x6d5, 0x64a, 0x631, 0x6cb,
0x646, 0x6d5, 0x67e, 0x6d5, 0x633, 0x627, 0x644, 0x6c6, 0x627, 0x62f, 0x648, 0x631,
0x641, 0x648, 0x631, 0x62a, 0x2d, 0x646, 0x6d5, 0x644, 0x633, 0x648, 0x646, 0x641,
0x648, 0x631, 0x62a, 0x627, 0x644, 0x6d5, 0x632, 0x627, 0x6af, 0x644, 0x6d5, 0x64a,
0x633, 0x2d, 0x628, 0x6d5, 0x64a, 0x6af, 0x6cb, 0x633, 0x2d, 0x628, 0x6d5, 0x64a,
0x6af, 0x631, 0x627, 0x646, 0x62f, 0x2d, 0x62a, 0x6d5, 0x631, 0x643, 0x6af, 0x631,
0x6d5, 0x646, 0x627, 0x62f, 0x627, 0x6af, 0x6c6, 0x627, 0x62f, 0x6d5, 0x644, 0x6c7,
0x67e, 0x627, 0x6af, 0x6c6, 0x627, 0x62a, 0x6d5, 0x645, 0x627, 0x644, 0x627, 0x6af,
0x6cb, 0x627, 0x64a, 0x627, 0x643, 0x64a, 0x644, 0x6af, 0x627, 0x64a, 0x627, 0x646,
0x627, 0x6af, 0x627, 0x644, 0x64a, 0x641, 0x627, 0x643, 0x633, 0x6af, 0x627, 0x6c6,
0x627, 0x646, 0x627, 0x6d5, 0x631, 0x645, 0x648, 0x633, 0x64a, 0x644, 0x648, 0x64a,
0x646, 0x62f, 0x64a, 0x627, 0x646, 0x627, 0x67e, 0x648, 0x644, 0x64a, 0x633, 0x646,
0x648, 0x643, 0x633, 0x60c, 0x20, 0x64a, 0x646, 0x62f, 0x64a, 0x627, 0x646, 0x627,
0x645, 0x627, 0x631, 0x6d5, 0x646, 0x6af, 0x648, 0x60c, 0x20, 0x64a, 0x646, 0x62f,
0x64a, 0x627, 0x646, 0x627, 0x67e, 0x64a, 0x62a, 0x6d5, 0x631, 0x633, 0x628, 0x6d5,
0x631, 0x6af, 0x60c, 0x20, 0x64a, 0x646, 0x62f, 0x64a, 0x627, 0x646, 0x627, 0x62a,
0x6d5, 0x644, 0x644, 0x2d, 0x633, 0x64a, 0x62a, 0x64a, 0x60c, 0x20, 0x64a, 0x646,
0x62f, 0x64a, 0x627, 0x646, 0x627, 0x6c6, 0x64a, 0x6c6, 0x6d5, 0x64a, 0x60c, 0x20,
0x64a, 0x646, 0x62f, 0x64a, 0x627, 0x646, 0x627, 0x6c6, 0x64a, 0x646, 0x633, 0x6d5,
0x646, 0x646, 0x6d5, 0x633, 0x60c, 0x20, 0x64a, 0x646, 0x62f, 0x64a, 0x627, 0x646,
0x627, 0x6cb, 0x64a, 0x646, 0x627, 0x645, 0x627, 0x643, 0x60c, 0x20, 0x64a, 0x646,
0x62f, 0x64a, 0x627, 0x646, 0x627, 0x64a, 0x646, 0x6cb, 0x6c6, 0x64a, 0x643, 0x64a,
0x643, 0x627, 0x644, 0x6cb, 0x64a, 0x62a, 0x64a, 0x627, 0x645, 0x627, 0x64a, 0x643,
0x627, 0x62c, 0x6cb, 0x646, 0x648, 0x644, 0x6cb, 0x64a, 0x633, 0x6c6, 0x64a, 0x644,
0x645, 0x648, 0x646, 0x62a, 0x64a, 0x633, 0x6d5, 0x644, 0x644, 0x648, 0x60c, 0x20,
0x643, 0x6d5, 0x646, 0x62a, 0x6cb, 0x643, 0x643, 0x64a, 0x643, 0x631, 0x627, 0x644,
0x6d5, 0x646, 0x62f, 0x6d5, 0x64a, 0x643, 0x644, 0x627, 0x2d, 0x67e, 0x627, 0x633,
0x644, 0x648, 0x633, 0x2d, 0x627, 0x646, 0x62c, 0x6d5, 0x644, 0x6d5, 0x633, 0x644,
0x648, 0x6cb, 0x6d5, 0x631, 0x2d, 0x67e, 0x631, 0x64a, 0x646, 0x633, 0x2d, 0x643,
0x6cb, 0x648, 0x62a, 0x6d5, 0x631, 0x645, 0x627, 0x633, 0x6d5, 0x64a, 0x648, 0x645,
0x627, 0x646, 0x627, 0x6af, 0x6cb, 0x627, 0x645, 0x627, 0x646, 0x627, 0x6cb, 0x633,
0x645, 0x627, 0x631, 0x64a, 0x6af, 0x648, 0x645, 0x627, 0x631, 0x62a, 0x64a, 0x646,
0x64a, 0x643, 0x627, 0x645, 0x627, 0x633, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x645,
0x6d5, 0x646, 0x648, 0x645, 0x64a, 0x646, 0x64a, 0x645, 0x6d5, 0x631, 0x64a, 0x62f,
0x627, 0x645, 0x6d5, 0x62a, 0x644, 0x627, 0x643, 0x627, 0x62a, 0x644, 0x627, 0x645,
0x6d5, 0x62d, 0x64a, 0x643, 0x648, 0x645, 0x64a, 0x643, 0x6d5, 0x644, 0x648, 0x646,
0x645, 0x648, 0x646, 0x643, 0x62a, 0x648, 0x646, 0x645, 0x648, 0x646, 0x62a, 0x6d5,
0x631, 0x631, 0x6d5, 0x64a, 0x645, 0x648, 0x646, 0x62a, 0x6d5, 0x6c6, 0x64a, 0x62f,
0x6d5, 0x648, 0x645, 0x648, 0x646, 0x62a, 0x633, 0x6d5, 0x631, 0x631, 0x627, 0x62a,
0x646, 0x627, 0x633, 0x633, 0x627, 0x6cb, 0x646, 0x64a, 0x6cb, 0x2d, 0x64a, 0x648,
0x631, 0x643, 0x646, 0x648, 0x631, 0x648, 0x646, 0x64a, 0x627, 0x628, 0x648, 0x64a,
0x644, 0x627, 0x60c, 0x20, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649,
0x643, 0x20, 0x62f, 0x627, 0x643, 0x648, 0x62a, 0x627, 0x633, 0x6d5, 0x646, 0x62a,
0x6d5, 0x631, 0x60c, 0x20, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649,
0x643, 0x20, 0x62f, 0x627, 0x643, 0x648, 0x62a, 0x627, 0x646, 0x64a, 0x6cb, 0x2d,
0x633, 0x6d5, 0x64a, 0x644, 0x6d5, 0x645, 0x60c, 0x20, 0x633, 0x648, 0x644, 0x62a,
0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x62f, 0x627, 0x643, 0x648, 0x62a, 0x627,
0x646, 0x6cb, 0x6cb, 0x643, 0x648, 0x62d, 0x64a, 0x646, 0x627, 0x6af, 0x627, 0x67e,
0x627, 0x646, 0x627, 0x645, 0x627, 0x67e, 0x627, 0x631, 0x627, 0x645, 0x627, 0x631,
0x64a, 0x628, 0x648, 0x641, 0x64a, 0x646, 0x64a, 0x643, 0x633, 0x67e, 0x648, 0x631,
0x62a, 0x2d, 0x648, 0x641, 0x2d, 0x67e, 0x631, 0x6d5, 0x646, 0x633, 0x67e, 0x648,
0x631, 0x62a, 0x2d, 0x648, 0x641, 0x2d, 0x633, 0x67e, 0x627, 0x64a, 0x646, 0x67e,
0x648, 0x631, 0x62a, 0x648, 0x2d, 0x6c6, 0x6d5, 0x644, 0x64a, 0x6cb, 0x67e, 0x6cb,
0x6d5, 0x631, 0x62a, 0x648, 0x2d, 0x631, 0x64a, 0x643, 0x648, 0x67e, 0x6cb, 0x646,
0x62a, 0x627, 0x2d, 0x627, 0x631, 0x6d5, 0x646, 0x627, 0x633, 0x631, 0x627, 0x646,
0x643, 0x64a, 0x646, 0x2d, 0x64a, 0x646, 0x644, 0x6d5, 0x62a, 0x631, 0x6d5, 0x633,
0x64a, 0x641, 0x64a, 0x631, 0x6d5, 0x62c, 0x627, 0x64a, 0x646, 0x627, 0x631, 0x6d5,
0x632, 0x648, 0x644, 0x64a, 0x6cb, 0x62a, 0x631, 0x64a, 0x6cb, 0x2d, 0x628, 0x631,
0x627, 0x646, 0x643, 0x6cb, 0x633, 0x627, 0x646, 0x62a, 0x627, 0x631, 0x6d5, 0x646,
0x633, 0x627, 0x646, 0x62a, 0x64a, 0x627, 0x6af, 0x648, 0x633, 0x627, 0x646, 0x62a,
0x648, 0x2d, 0x62f, 0x648, 0x645, 0x64a, 0x646, 0x6af, 0x648, 0x633, 0x627, 0x646,
0x2d, 0x67e, 0x6cb, 0x627, 0x644, 0x6cb, 0x64a, 0x644, 0x644, 0x648, 0x643, 0x643,
0x648, 0x631, 0x62a, 0x648, 0x648, 0x631, 0x645, 0x64a, 0x6cb, 0x62a, 0x633, 0x6d5,
0x646, 0x2d, 0x628, 0x627, 0x631, 0x62a, 0x6d5, 0x644, 0x6d5, 0x645, 0x64a, 0x633,
0x6d5, 0x646, 0x62a, 0x2d, 0x62c, 0x648, 0x646, 0x633, 0x633, 0x6d5, 0x646, 0x62a,
0x2d, 0x643, 0x64a, 0x62a, 0x633, 0x633, 0x6d5, 0x646, 0x62a, 0x2d, 0x644, 0x64a,
0x6cb, 0x633, 0x64a, 0x627, 0x633, 0x6d5, 0x646, 0x62a, 0x2d, 0x62a, 0x648, 0x645,
0x627, 0x633, 0x633, 0x6d5, 0x646, 0x62a, 0x2d, 0x6c6, 0x64a, 0x646, 0x633, 0x6d5,
0x646, 0x62a, 0x633, 0x6cb, 0x64a, 0x627, 0x62a, 0x2d, 0x643, 0x627, 0x631, 0x631,
0x6d5, 0x646, 0x62a, 0x62a, 0x6d5, 0x6af, 0x6cb, 0x633, 0x64a, 0x6af, 0x627, 0x644,
0x67e, 0x627, 0x62a, 0x6cb, 0x644, 0x6d5, 0x62a, 0x64a, 0x62d, 0x6cb, 0x627, 0x646,
0x627, 0x6c6, 0x627, 0x646, 0x643, 0x6cb, 0x6c6, 0x6d5, 0x631, 0x6cb, 0x627, 0x64a,
0x62a, 0x62d, 0x648, 0x631, 0x633, 0x6c6, 0x64a, 0x646, 0x646, 0x64a, 0x67e, 0x6d5,
0x6af, 0x64a, 0x627, 0x643, 0x6cb, 0x62a, 0x627, 0x62a, 0x643, 0x6d5, 0x64a, 0x633,
0x64a, 0x62f, 0x6d5, 0x64a, 0x6c6, 0x64a, 0x633, 0x62f, 0x64a, 0x6cb, 0x645, 0x648,
0x646, 0x2d, 0x62f, 0x64a, 0x6cb, 0x631, 0x6c6, 0x64a, 0x644, 0x645, 0x627, 0x643,
0x643, 0x6cb, 0x648, 0x631, 0x64a, 0x645, 0x648, 0x6cb, 0x633, 0x648, 0x646, 0x645,
0x627, 0x643, 0x2d, 0x645, 0x6d5, 0x631, 0x62f, 0x648, 0x67e, 0x627, 0x644, 0x645,
0x6d5, 0x631, 0x631, 0x648, 0x62a, 0x6d5, 0x631, 0x627, 0x633, 0x6d5, 0x6c6, 0x627,
0x6c6, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x644, 0x648, 0x646, 0x6af, 0x64a, 0x631,
0x627, 0x62f, 0x6d5, 0x646, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x649, 0x627, 0x645,
0x645, 0x627, 0x646, 0x627, 0x646, 0x627, 0x62f, 0x649, 0x631, 0x627, 0x642, 0x62a,
0x627, 0x6cb, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x6d5, 0x627, 0x634, 0x62d, 0x627,
0x628, 0x627, 0x62a, 0x627, 0x62a, 0x649, 0x631, 0x627, 0x6cb, 0x628, 0x627, 0x6af,
0x62f, 0x627, 0x62a, 0x628, 0x627, 0x62d, 0x631, 0x6d5, 0x64a, 0x646, 0x628, 0x627,
0x643, 0x6cb, 0x628, 0x627, 0x6ad, 0x643, 0x648, 0x643, 0x628, 0x627, 0x631, 0x646,
0x627, 0x6cb, 0x649, 0x644, 0x628, 0x6d5, 0x64a, 0x631, 0x6cb, 0x62a, 0x628, 0x649,
0x634, 0x643, 0x6d5, 0x643, 0x628, 0x631, 0x6cb, 0x646, 0x6d5, 0x64a, 0x686, 0x64a,
0x62a, 0x627, 0x62f, 0x627, 0x645, 0x627, 0x633, 0x643, 0x62f, 0x627, 0x643, 0x643,
0x627, 0x62f, 0x6cb, 0x628, 0x627, 0x64a, 0x62f, 0x6cb, 0x634, 0x627, 0x646, 0x628,
0x6d5, 0x641, 0x627, 0x645, 0x627, 0x6af, 0x6cb, 0x633, 0x62a, 0x627, 0x6af, 0x627,
0x632, 0x627, 0x62d, 0x6d5, 0x6c6, 0x631, 0x648, 0x646, 0x62d, 0x648, 0x634, 0x64a,
0x645, 0x64a, 0x646, 0x62d, 0x648, 0x6ad, 0x643, 0x648, 0x6ad, 0x62d, 0x648, 0x6c6,
0x62f, 0x64a, 0x631, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x62c, 0x627, 0x643, 0x627,
0x631, 0x62a, 0x627, 0x62c, 0x627, 0x64a, 0x627, 0x67e, 0x6cb, 0x631, 0x627, 0x64a,
0x6d5, 0x631, 0x6cb, 0x633, 0x627, 0x644, 0x64a, 0x645, 0x643, 0x627, 0x628, 0x6cb,
0x644, 0x643, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x643, 0x627, 0x643, 0x627, 0x631,
0x627, 0x686, 0x627, 0x643, 0x627, 0x62a, 0x645, 0x627, 0x646, 0x62f, 0x6cb, 0x62d,
0x627, 0x646, 0x62f, 0x649, 0x6af, 0x627, 0x643, 0x627, 0x644, 0x643, 0x6cb, 0x62a,
0x627, 0x643, 0x6cb, 0x627, 0x644, 0x627, 0x2d, 0x644, 0x6cb, 0x645, 0x67e, 0x6cb,
0x631, 0x643, 0x6cb, 0x686, 0x64a, 0x6ad, 0x643, 0x6cb, 0x6c6, 0x6d5, 0x64a, 0x62a,
0x645, 0x627, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x645, 0x627, 0x643, 0x627, 0x633,
0x627, 0x631, 0x645, 0x627, 0x633, 0x643, 0x627, 0x62a, 0x646, 0x64a, 0x643, 0x648,
0x633, 0x64a, 0x627, 0x646, 0x648, 0x6c6, 0x648, 0x643, 0x6cb, 0x632, 0x646, 0x6d5,
0x62a, 0x633, 0x643, 0x62c, 0x627, 0x6ad, 0x627, 0x633, 0x649, 0x628, 0x649, 0x631,
0x648, 0x645, 0x628, 0x649, 0x67e, 0x646, 0x648, 0x645, 0x67e, 0x6d5, 0x646, 0x67e,
0x648, 0x646, 0x62a, 0x64a, 0x627, 0x646, 0x627, 0x643, 0x67e, 0x62d, 0x6d5, 0x646,
0x64a, 0x627, 0x646, 0x643, 0x627, 0x62a, 0x627, 0x631, 0x642, 0x649, 0x632, 0x649,
0x644, 0x648, 0x631, 0x62f, 0x627, 0x6d5, 0x631, 0x2d, 0x631, 0x64a, 0x627, 0x62f,
0x633, 0x627, 0x62d, 0x627, 0x644, 0x64a, 0x646, 0x633, 0x627, 0x645, 0x627, 0x631,
0x642, 0x627, 0x646, 0x62f, 0x633, 0x6d5, 0x6cb, 0x644, 0x634, 0x627, 0x646, 0x62d,
0x627, 0x64a, 0x633, 0x64a, 0x646, 0x6af, 0x627, 0x67e, 0x6cb, 0x631, 0x633, 0x631,
0x6d5, 0x62f, 0x646, 0x6d5, 0x643, 0x648, 0x644, 0x649, 0x645, 0x633, 0x643, 0x62a,
0x627, 0x64a, 0x628, 0x6d5, 0x64a, 0x62a, 0x627, 0x634, 0x643, 0x6d5, 0x646, 0x62a,
0x62a, 0x628, 0x64a, 0x644, 0x64a, 0x633, 0x64a, 0x62a, 0x6d5, 0x6af, 0x6d5, 0x631,
0x627, 0x646, 0x62a, 0x62d, 0x64a, 0x645, 0x67e, 0x62d, 0x6cb, 0x62a, 0x648, 0x643,
0x64a, 0x648, 0x62a, 0x6cb, 0x645, 0x6d5, 0x646, 0x6c7, 0x644, 0x627, 0x646, 0x628,
0x627, 0x62a, 0x649, 0x631, 0x621, 0x6c7, 0x631, 0x649, 0x645, 0x634, 0x649, 0x6cb,
0x633, 0x62a, 0x2d, 0x646, 0x6d5, 0x631, 0x627, 0x6c6, 0x6d5, 0x646, 0x62a, 0x64a,
0x627, 0x646, 0x6c6, 0x644, 0x627, 0x62f, 0x64a, 0x6c6, 0x627, 0x633, 0x62a, 0x648,
0x643, 0x64a, 0x627, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x64a, 0x627, 0x646, 0x6af,
0x648, 0x646, 0x6d5, 0x643, 0x627, 0x62a, 0x6d5, 0x631, 0x64a, 0x646, 0x628, 0x6cb,
0x631, 0x6af, 0x64a, 0x6d5, 0x631, 0x6d5, 0x6c6, 0x627, 0x646, 0x627, 0x632, 0x648,
0x631, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x628, 0x6d5,
0x631, 0x645, 0x6cb, 0x62f, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631,
0x649, 0x643, 0x627, 0x646, 0x627, 0x631, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f,
0x627, 0x631, 0x649, 0x643, 0x627, 0x628, 0x648, 0x2d, 0x6c6, 0x6d5, 0x631, 0x62f,
0x6d5, 0x641, 0x627, 0x631, 0x6d5, 0x631, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f,
0x627, 0x631, 0x649, 0x645, 0x627, 0x62f, 0x6d5, 0x64a, 0x631, 0x627, 0x631, 0x6d5,
0x64a, 0x643, 0x64a, 0x627, 0x6c6, 0x64a, 0x643, 0x648, 0x6ad, 0x62a, 0x6c7, 0x633,
0x62a, 0x649, 0x643, 0x20, 0x6af, 0x6d5, 0x648, 0x631, 0x6af, 0x64a, 0x627, 0x627,
0x6cb, 0x644, 0x64a, 0x6d5, 0x20, 0x6d5, 0x644, 0x6d5, 0x646, 0x627, 0x20, 0x627,
0x631, 0x627, 0x644, 0x649, 0x633, 0x62a, 0x6d5, 0x646, 0x644, 0x64a, 0x627, 0x62f,
0x6d5, 0x644, 0x627, 0x64a, 0x62f, 0x627, 0x628, 0x631, 0x64a, 0x633, 0x628, 0x6d5,
0x646, 0x628, 0x631, 0x648, 0x643, 0x6d5, 0x646, 0x2d, 0x62d, 0x64a, 0x644, 0x62f,
0x627, 0x631, 0x6c6, 0x64a, 0x646, 0x64a, 0x6cb, 0x643, 0x644, 0x627, 0x62d, 0x648,
0x628, 0x627, 0x631, 0x62a, 0x644, 0x64a, 0x646, 0x62f, 0x6d5, 0x645, 0x627, 0x646,
0x644, 0x648, 0x631, 0x62f, 0x2d, 0x62d, 0x627, 0x6cb, 0x645, 0x6d5, 0x644, 0x628,
0x6cb, 0x631, 0x646, 0x67e, 0x6d5, 0x631, 0x62a, 0x633, 0x64a, 0x62f, 0x646, 0x6d5,
0x64a, 0x627, 0x645, 0x633, 0x62a, 0x6d5, 0x631, 0x62f, 0x627, 0x645, 0x627, 0x646,
0x62f, 0x648, 0x631, 0x631, 0x627, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x62d, 0x627,
0x646, 0x627, 0x641, 0x64a, 0x646, 0x627, 0x628, 0x6d5, 0x644, 0x6af, 0x631, 0x627,
0x62f, 0x628, 0x6d5, 0x631, 0x644, 0x64a, 0x646, 0x628, 0x631, 0x627, 0x62a, 0x64a,
0x633, 0x644, 0x627, 0x6c6, 0x627, 0x628, 0x631, 0x64a, 0x6cb, 0x633, 0x6d5, 0x644,
0x628, 0x6cb, 0x62d, 0x627, 0x631, 0x6d5, 0x633, 0x62a, 0x628, 0x6cb, 0x62f, 0x627,
0x67e, 0x6d5, 0x634, 0x62a, 0x628, 0x64a, 0x6cb, 0x632, 0x64a, 0x646, 0x6af, 0x6d5,
0x646, 0x643, 0x64a, 0x634, 0x64a, 0x646, 0x6d5, 0x6c6, 0x643, 0x648, 0x67e, 0x6d5,
0x646, 0x6af, 0x627, 0x6af, 0x6d5, 0x646, 0x62f, 0x6cb, 0x628, 0x644, 0x64a, 0x646,
0x6af, 0x64a, 0x628, 0x631, 0x627, 0x644, 0x62a, 0x627, 0x644, 0x6af, 0x6d5, 0x631,
0x646, 0x633, 0x64a, 0x62d, 0x6d5, 0x644, 0x633, 0x64a, 0x646, 0x643, 0x64a, 0x645,
0x6d5, 0x646, 0x20, 0x627, 0x631, 0x627, 0x644, 0x649, 0x633, 0x62a, 0x627, 0x645,
0x628, 0x6c7, 0x644, 0x62c, 0x6d5, 0x631, 0x633, 0x6d5, 0x64a, 0x643, 0x627, 0x644,
0x64a, 0x646, 0x64a, 0x646, 0x6af, 0x631, 0x627, 0x62f, 0x643, 0x64a, 0x631, 0x648,
0x6c6, 0x643, 0x64a, 0x6d5, 0x6c6, 0x644, 0x64a, 0x633, 0x627, 0x628, 0x648, 0x646,
0x644, 0x64a, 0x6cb, 0x628, 0x644, 0x64a, 0x627, 0x646, 0x627, 0x644, 0x648, 0x646,
0x62f, 0x648, 0x646, 0x644, 0x64a, 0x6cb, 0x643, 0x633, 0x6d5, 0x645, 0x628, 0x6cb,
0x631, 0x6af, 0x645, 0x627, 0x62f, 0x631, 0x64a, 0x62f, 0x645, 0x627, 0x644, 0x62a,
0x627, 0x645, 0x627, 0x631, 0x64a, 0x6d5, 0x62d, 0x627, 0x645, 0x646, 0x645, 0x627,
0x633, 0x643, 0x6d5, 0x6cb, 0x67e, 0x627, 0x631, 0x64a, 0x62c, 0x67e, 0x648, 0x62f,
0x6af, 0x648, 0x631, 0x64a, 0x633, 0x627, 0x67e, 0x631, 0x627, 0x6af, 0x627, 0x631,
0x64a, 0x6af, 0x627, 0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x633, 0x627, 0x646,
0x2d, 0x645, 0x627, 0x631, 0x64a, 0x646, 0x648, 0x633, 0x627, 0x631, 0x627, 0x6d5,
0x6c6, 0x648, 0x633, 0x627, 0x631, 0x649, 0x62a, 0x627, 0x6cb, 0x633, 0x64a, 0x645,
0x641, 0x6d5, 0x631, 0x648, 0x644, 0x633, 0x643, 0x648, 0x67e, 0x64a, 0x6d5, 0x633,
0x648, 0x641, 0x64a, 0x627, 0x633, 0x62a, 0x648, 0x643, 0x6af, 0x648, 0x644, 0x649,
0x645, 0x62a, 0x627, 0x644, 0x644, 0x64a, 0x646, 0x6cb, 0x644, 0x64a, 0x627, 0x646,
0x648, 0x6c6, 0x633, 0x643, 0x6c6, 0x627, 0x62f, 0x6cb, 0x633, 0x6c6, 0x627, 0x62a,
0x64a, 0x643, 0x627, 0x646, 0x6c6, 0x6d5, 0x646, 0x627, 0x6c6, 0x64a, 0x646, 0x64a,
0x6cb, 0x633, 0x6c6, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x6c6,
0x627, 0x631, 0x634, 0x627, 0x6c6, 0x627, 0x632, 0x627, 0x6af, 0x631, 0x6d5, 0x628,
0x633, 0x64a, 0x6cb, 0x631, 0x64a, 0x62d, 0x627, 0x646, 0x62a, 0x627, 0x646, 0x627,
0x646, 0x627, 0x631, 0x64a, 0x6c6, 0x6cb, 0x643, 0x631, 0x64a, 0x633, 0x62a, 0x645,
0x627, 0x633, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x643,
0x648, 0x643, 0x648, 0x633, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631,
0x649, 0x643, 0x648, 0x645, 0x648, 0x631, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f,
0x627, 0x631, 0x649, 0x643, 0x6d5, 0x631, 0x6af, 0x6d5, 0x644, 0x6d5, 0x646, 0x645,
0x627, 0x6d5, 0x645, 0x627, 0x644, 0x62f, 0x64a, 0x6c6, 0x20, 0x627, 0x631, 0x627,
0x644, 0x62f, 0x627, 0x631, 0x649, 0x645, 0x627, 0x6c6, 0x631, 0x64a, 0x643, 0x64a,
0x645, 0x627, 0x64a, 0x648, 0x62a, 0x62a, 0x627, 0x631, 0x6d5, 0x64a, 0x6cb, 0x646,
0x64a, 0x648, 0x646, 0x627, 0x67e, 0x64a, 0x627, 0x648, 0x643, 0x644, 0x6d5, 0x646,
0x62f, 0x628, 0x6cb, 0x6af, 0x6d5, 0x646, 0x6c6, 0x64a, 0x644, 0x686, 0x627, 0x62a,
0x6d5, 0x645, 0x62a, 0x631, 0x6cb, 0x643, 0x67e, 0x627, 0x633, 0x62d, 0x627, 0x20,
0x627, 0x631, 0x627, 0x644, 0x649, 0x6d5, 0x641, 0x627, 0x62a, 0x6d5, 0x641, 0x6cb,
0x646, 0x627, 0x641, 0x6cb, 0x62a, 0x64a, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627,
0x6af, 0x648, 0x633, 0x6af, 0x627, 0x645, 0x628, 0x64a, 0x6d5, 0x6af, 0x6cb, 0x627,
0x62f, 0x627, 0x644, 0x643, 0x627, 0x646, 0x627, 0x644, 0x643, 0x627, 0x646, 0x62a,
0x648, 0x646, 0x643, 0x64a, 0x631, 0x64a, 0x62a, 0x64a, 0x645, 0x627, 0x62a, 0x64a,
0x643, 0x6cb, 0x633, 0x627, 0x64a, 0x6d5, 0x643, 0x6c6, 0x627, 0x62c, 0x627, 0x644,
0x6d5, 0x64a, 0x649, 0x646, 0x645, 0x627, 0x62c, 0x6cb, 0x631, 0x648, 0x645, 0x627,
0x631, 0x643, 0x64a, 0x632, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631,
0x649, 0x645, 0x64a, 0x62f, 0x6cb, 0x6d5, 0x64a, 0x646, 0x627, 0x6cb, 0x631, 0x6cb,
0x646, 0x64a, 0x6cb, 0x6d5, 0x646, 0x6cb, 0x645, 0x6d5, 0x627, 0x67e, 0x627, 0x6af,
0x648, 0x2d, 0x67e, 0x627, 0x6af, 0x648, 0x67e, 0x627, 0x644, 0x627, 0x6cb, 0x67e,
0x64a, 0x62a, 0x643, 0x6d5, 0x631, 0x646, 0x67e, 0x648, 0x646, 0x627, 0x67e, 0x6d5,
0x67e, 0x648, 0x631, 0x62a, 0x2d, 0x645, 0x648, 0x631, 0x633, 0x628, 0x64a, 0x631,
0x627, 0x631, 0x648, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x633, 0x627, 0x64a, 0x67e,
0x627, 0x646, 0x62a, 0x627, 0x64a, 0x62a, 0x64a, 0x62a, 0x627, 0x631, 0x627, 0x6cb,
0x627, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x62a, 0x627, 0x67e, 0x6cb, 0x6cb, 0x6d5,
0x64a, 0x643, 0x6cb, 0x648, 0x644, 0x644, 0x64a, 0x633, 0x17a2, 0x17b6, 0x1794, 0x17ca,
0x17b8, 0x178a, 0x17d2, 0x1799, 0x17b6, 0x1793, 0x17a2, 0x17b6, 0x1780, 0x17d2, 0x179a, 0x17b6,
0x17a2, 0x17b6, 0x178c, 0x17b8, 0x179f, 0x17a2, 0x17b6, 0x1794, 0x17c1, 0x1794, 0x17b6, 0x17a2,
0x17b6, 0x179b, 0x17cb, 0x17a0, 0x17d2, 0x179f, 0x17c1, 0x17a2, 0x17b6, 0x179f, 0x17d2, 0x1798,
0x17b6, 0x179a, 0x17c9, 0x17b6, 0x1794, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17bc, 0x1794,
0x1784, 0x17cb, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x1794, 0x1784, 0x17cb, 0x17a0, 0x17d2, 0x179f,
0x17ca, 0x17bb, 0x179b, 0x1794, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x179f, 0x17bc, 0x1794, 0x17d2,
0x179b, 0x17b6, 0x17c6, 0x1784, 0x1791, 0x17b8, 0x179a, 0x17c9, 0x17c1, 0x1794, 0x17d2, 0x179a,
0x17b6, 0x179f, 0x17d2, 0x17a0, 0x17b6, 0x179c, 0x17b8, 0x179b, 0x1794, 0x17ca, 0x17bc, 0x1799,
0x17c9, 0x17b6, 0x17c6, 0x1794, 0x17ca, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x1782, 0x17c2, 0x179a,
0x1780, 0x17b6, 0x179f, 0x17b6, 0x1794, 0x17d2, 0x179b, 0x17b6, 0x17c6, 0x1784, 0x1780, 0x17b6,
0x1788, 0x17c0, 0x178f, 0x17b6, 0x1780, 0x17bc, 0x178e, 0x17b6, 0x1782, 0x17d2, 0x179a, 0x17b8,
0x178a, 0x17b6, 0x1780, 0x17b6, 0x178a, 0x17b6, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17b6, 0x17a1,
0x17b6, 0x17c6, 0x1787, 0x17b8, 0x1794, 0x17ca, 0x17bc, 0x1791, 0x17b8, 0x178c, 0x17bc, 0x17a2,
0x17ca, 0x17bc, 0x17a1, 0x17b6, 0x17a2, 0x17c2, 0x179b, 0x200b, 0x17a2, 0x17ca, 0x17bb, 0x1799,
0x1789, 0x17c0, 0x1793, 0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b8, 0x1790, 0x17c4, 0x1793,
0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1794, 0x17bc, 0x179a, 0x17c9, 0x17bc, 0x1793, 0x17a0, 0x17b6,
0x179a, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17c1, 0x1787, 0x17bc, 0x17a0, 0x17b6, 0x1793, 0x178e,
0x17c1, 0x1794, 0x17ca, 0x17ba, 0x17a0, 0x17d2, 0x1782, 0x1787, 0x17bc, 0x1794, 0x17b6, 0x1780,
0x17c6, 0x1794, 0x17c9, 0x17b6, 0x17a1, 0x17b6, 0x1781, 0x17b6, 0x1791, 0x17bb, 0x17c6, 0x1782,
0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x179b, 0x17b8, 0x1782, 0x17b8, 0x1793, 0x179f, 0x17d2,
0x17a0, 0x17b6, 0x179f, 0x17b6, 0x17a1, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17bc, 0x179f, 0x179b,
0x17b8, 0x1794, 0x17d2, 0x179a, 0x17b8, 0x179c, 0x17b8, 0x179b, 0x17a1, 0x17bc, 0x1798, 0x17c9,
0x17c1, 0x179b, 0x17bc, 0x17a2, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179b, 0x17bc, 0x1794, 0x17ca,
0x17bb, 0x1798, 0x1794, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17bc, 0x179f, 0x17b6, 0x1780,
0x17b6, 0x1798, 0x17c9, 0x17b6, 0x17a1, 0x17b6, 0x1794, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x1796,
0x17bb, 0x1799, 0x178f, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x179f, 0x17c1, 0x179a, 0x17bb, 0x1799,
0x1794, 0x17b6, 0x1794, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17b6,
0x178c, 0x17b8, 0x179f, 0x17d2, 0x1799, 0x17bc, 0x1798, 0x17c9, 0x17bb, 0x1784, 0x179a, 0x17c9,
0x17bc, 0x179c, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x178e, 0x17c3, 0x179a, 0x17c9, 0x17bc, 0x1794,
0x17ca, 0x17b8, 0x178a, 0x17d2, 0x1799, 0x17b6, 0x1798, 0x17c1, 0x178e, 0x17b6, 0x1793, 0x17b8,
0x1789, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c1, 0x178e, 0x17bd, 0x1780, 0x179f, 0x17bb, 0x178f,
0x17a2, 0x17c5, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x178c, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17bc,
0x1796, 0x17d0, 0x179a, 0x178f, 0x17bc, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17c5,
0x178f, 0x17bc, 0x1798, 0x17c9, 0x17c1, 0x1791, 0x17d2, 0x179a, 0x17b8, 0x1794, 0x17c9, 0x17bc,
0x179b, 0x17b8, 0x1791, 0x17bb, 0x1799, 0x1793, 0x17b8, 0x179f, 0x179c, 0x17b8, 0x1793, 0x178a,
0x17bb, 0x1780, 0x17a2, 0x17b6, 0x178a, 0x17b6, 0x1780, 0x17a2, 0x1784, 0x17cb, 0x1785, 0x17bc,
0x179a, 0x17c9, 0x17b6, 0x1780, 0x17a2, 0x1784, 0x17cb, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x17a1,
0x17b6, 0x17a2, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x17a2, 0x17b6,
0x179a, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17bb, 0x1799, 0x178e, 0x17b6, 0x1794, 0x17ca,
0x17bb, 0x1799, 0x178e, 0x17bc, 0x179f, 0x17ca, 0x17c2, 0x1780, 0x17b6, 0x178f, 0x17b6, 0x1798,
0x17c9, 0x17b6, 0x1780, 0x17b6, 0x1780, 0x17bc, 0x178a, 0x17bc, 0x1794, 0x17b6, 0x179f, 0x17bc,
0x179f, 0x17bb, 0x1799, 0x17a1, 0x17b6, 0x179a, 0x17b8, 0x1799, 0x17c9, 0x17bc, 0x17a0, 0x17b6,
0x1798, 0x17c9, 0x17c2, 0x1793, 0x178c, 0x17bc, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x179a, 0x17b8,
0x1799, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x17a1, 0x17c1, 0x17a0, 0x17d2, 0x1782,
0x17bc, 0x179f, 0x179f, 0x17b6, 0x179b, 0x17cb, 0x178f, 0x17b6, 0x179f, 0x17b6, 0x17c6, 0x1784,
0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x179b, 0x17bc, 0x17a2,
0x17ca, 0x17b8, 0x179f, 0x1791, 0x17bc, 0x1782, 0x17bc, 0x1798, 0x17c9, 0x1784, 0x17cb, 0x17a2,
0x17ca, 0x17bb, 0x1799, 0x179f, 0x17bc, 0x17a2, 0x17c3, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17bc,
0x1794, 0x17b6, 0x17a2, 0x17b6, 0x179f, 0x1784, 0x17cb, 0x179f, 0x17d2, 0x1799, 0x17bb, 0x1784,
0x17a2, 0x17b6, 0x1791, 0x17b8, 0x1780, 0x17bc, 0x1780, 0x17b6, 0x1793, 0x1794, 0x17b6, 0x17a0,
0x17c0, 0x1794, 0x17b6, 0x17a0, 0x17c0, 0x1794, 0x1784, 0x17cb, 0x178a, 0x17c1, 0x179a, 0x17c9,
0x17b6, 0x179f, 0x1794, 0x17b6, 0x1794, 0x17b6, 0x178a, 0x17bb, 0x179f, 0x1794, 0x17c1, 0x17a1,
0x17b9, 0x1798, 0x1794, 0x17c1, 0x179b, 0x17b8, 0x1794, 0x17d2, 0x179b, 0x1784, 0x17cb, 0x200b,
0x179f, 0x17b6, 0x1794, 0x17cb, 0x17a1, 0x17bb, 0x1784, 0x1794, 0x17c5, 0x200b, 0x179c, 0x17b8,
0x179f, 0x17d2, 0x178f, 0x17b6, 0x1794, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x178f, 0x17b6,
0x1794, 0x17ca, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x1781, 0x17c1, 0x1798, 0x1794, 0x17d2, 0x179a,
0x17b8, 0x178a, 0x1794, 0x17c1, 0x1781, 0x17c1, 0x1798, 0x1794, 0x17c9, 0x17bc, 0x17a0, 0x17d2,
0x1782, 0x17d2, 0x179a, 0x1784, 0x17cb, 0x200b, 0x1781, 0x17b6, 0x1793, 0x1781, 0x17bb, 0x1793,
0x1780, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x1780, 0x17b6, 0x179f, 0x1780, 0x17b6, 0x1799, 0x17c9,
0x17c1, 0x1793, 0x1780, 0x17c3, 0x1798, 0x17c2, 0x1793, 0x179f, 0x17ca, 0x17b8, 0x1780, 0x17b6,
0x17a0, 0x17d2, 0x1782, 0x17c4, 0x1787, 0x17b8, 0x17a0, 0x17bd, 0x17a0, 0x17bd, 0x179f, 0x17ca,
0x17b8, 0x17a2, 0x17ca, 0x17bc, 0x178a, 0x17b6, 0x178a, 0x20, 0x17a0, 0x17ca, 0x17bc, 0x17a2,
0x17b6, 0x179a, 0x17c9, 0x17c1, 0x179f, 0x1780, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17b6, 0x179a,
0x17b8, 0x1780, 0x17b6, 0x1780, 0x17d2, 0x179a, 0x17c1, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1793,
0x1782, 0x17bb, 0x1799, 0x17a2, 0x17b6, 0x1794, 0x17b6, 0x1780, 0x17bc, 0x179a, 0x17c9, 0x17b6,
0x1780, 0x17c5, 0x178a, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x179f, 0x17d2, 0x17a0,
0x17b6, 0x179c, 0x17b7, 0x1793, 0x178a, 0x17b6, 0x179c, 0x179f, 0x17bb, 0x1793, 0x178a, 0x17b6,
0x179c, 0x179f, 0x17bb, 0x1793, 0x200b, 0x1780, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x178a, 0x17c2,
0x1793, 0x179c, 0x17be, 0x178a, 0x17c1, 0x178f, 0x17d2, 0x179a, 0x17bc, 0x17a2, 0x17ca, 0x17b8,
0x178f, 0x178a, 0x17bc, 0x1798, 0x17b8, 0x1793, 0x17b8, 0x1780, 0x17a2, 0x17c2, 0x178f, 0x1798,
0x17c9, 0x17bb, 0x1784, 0x178f, 0x17bb, 0x1784, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x17bb,
0x1799, 0x178e, 0x17c1, 0x1797, 0x17b8, 0x17a2, 0x17c2, 0x179b, 0x179f, 0x17b6, 0x179b, 0x17cb,
0x179c, 0x17c9, 0x17b6, 0x178c, 0x17d0, 0x179a, 0x17a0, 0x17d2, 0x179c, 0x178f, 0x178e, 0x17c1,
0x179b, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x17a0, 0x17d2, 0x179c, 0x17d0, 0x179a, 0x178f, 0x17b6,
0x17a1, 0x17c1, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x179a, 0x1780, 0x17d2, 0x179b, 0x17b6, 0x179f,
0x200b, 0x1794, 0x17c1, 0x1780, 0x17bc, 0x179f, 0x17c1, 0x1794, 0x17c1, 0x17a0, 0x17d2, 0x1782,
0x17d2, 0x179a, 0x17c1, 0x1793, 0x200b, 0x1791, 0x17bc, 0x1780, 0x17a0, 0x17d2, 0x1782, 0x17d2,
0x179a, 0x17be, 0x178e, 0x17b6, 0x178a, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x178a, 0x17c1, 0x179b,
0x17bc, 0x1794, 0x17c9, 0x17c1, 0x1780, 0x17d2, 0x179c, 0x17b6, 0x178f, 0x17c1, 0x1798, 0x17c9,
0x17b6, 0x17a1, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17bb, 0x1799, 0x17c9, 0x17b6, 0x1782, 0x17b8,
0x179b, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x1793, 0x17a0, 0x17b6, 0x179b,
0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b6, 0x1780, 0x17a1, 0x17b6, 0x17a0, 0x17b6, 0x179c, 0x17c9,
0x17b6, 0x1793, 0x17a0, 0x17ca, 0x17c2, 0x1798, 0x17c9, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x17a1,
0x17bc, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6, 0x1794, 0x17c9, 0x17bc, 0x179b,
0x17b8, 0x179f, 0x178e, 0x17bb, 0x1780, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e,
0x17b6, 0x1798, 0x17c9, 0x17b6, 0x179a, 0x17c1, 0x1793, 0x1780, 0x17bc, 0x20, 0x17a5, 0x178e,
0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6, 0x1797, 0x17b8, 0x1792, 0x17be, 0x200b, 0x1794, 0x17ca,
0x17b9, 0x1780, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6, 0x1791, 0x17b8,
0x1780, 0x17d2, 0x179a, 0x17bb, 0x1784, 0x1790, 0x17c2, 0x179b, 0x20, 0x17a5, 0x178e, 0x17d2,
0x178c, 0x17b6, 0x178e, 0x17b6, 0x179c, 0x17c1, 0x179c, 0x17c9, 0x17c3, 0x20, 0x17a5, 0x178e,
0x17d2, 0x178c, 0x17b6, 0x1799, 0x17c9, 0x17b6, 0x178e, 0x17b6, 0x179c, 0x17b7, 0x1793, 0x179f,
0x17c1, 0x178e, 0x17c1, 0x179f, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6,
0x179c, 0x17b8, 0x178e, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17cb, 0x20, 0x17a5, 0x178e,
0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6, 0x17a2, 0x17ca, 0x17b8, 0x1793, 0x17bc, 0x179c, 0x17b8,
0x1780, 0x17a2, 0x17ca, 0x17b8, 0x1780, 0x17b6, 0x17a1, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x178f,
0x17a0, 0x17d2, 0x179f, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x17a2, 0x17ca, 0x17b8, 0x1780, 0x17a0,
0x17d2, 0x1799, 0x17bc, 0x1793, 0x17c0, 0x179b, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x179c, 0x17b8,
0x179b, 0x1798, 0x17c9, 0x1784, 0x1791, 0x17b8, 0x179f, 0x17c1, 0x17a1, 0x17bc, 0x20, 0x1780,
0x17c1, 0x1793, 0x1791, 0x17bc, 0x1782, 0x17b8, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17c2,
0x1793, 0x178c, 0x17b8, 0x1780, 0x17a1, 0x17b6, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x179b, 0x17b8,
0x1798, 0x17c9, 0x17b6, 0x17a1, 0x17bc, 0x17a2, 0x1784, 0x17cb, 0x17a0, 0x17d2, 0x1782, 0x17b7,
0x17a1, 0x17c1, 0x179f, 0x17a1, 0x17bc, 0x179c, 0x17be, 0x200b, 0x1796, 0x17d2, 0x179a, 0x17b8,
0x1793, 0x200b, 0x17a0, 0x17d2, 0x1782, 0x17d0, 0x179a, 0x1791, 0x17d0, 0x179a, 0x1798, 0x17c9,
0x17b6, 0x1780, 0x179f, 0x17c1, 0x17a2, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x178e, 0x17b6, 0x17a0,
0x17d2, 0x1782, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x178e, 0x17b6, 0x17a2, 0x17ca, 0x17bc, 0x179f,
0x1798, 0x17c9, 0x17b6, 0x179a, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17bb, 0x178f, 0x1798, 0x17c9,
0x17b6, 0x1791, 0x17b8, 0x1793, 0x17b8, 0x1780, 0x1798, 0x17c9, 0x17b6, 0x178f, 0x17b6, 0x1798,
0x17c9, 0x17bc, 0x179a, 0x17c9, 0x17bc, 0x179f, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f,
0x17b6, 0x178f, 0x17a1, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c1, 0x178e, 0x17bc, 0x1798, 0x17b8,
0x1793, 0x17b8, 0x1798, 0x17c9, 0x17c1, 0x179a, 0x17b8, 0x178a, 0x17b6, 0x1798, 0x17c1, 0x178f,
0x17d2, 0x179b, 0x17b6, 0x20, 0x1780, 0x17b6, 0x178f, 0x17d2, 0x179b, 0x17b6, 0x1791, 0x17b8,
0x1780, 0x17d2, 0x179a, 0x17bb, 0x1784, 0x1798, 0x17c9, 0x17b7, 0x1785, 0x179f, 0x17ca, 0x17b8,
0x1780, 0x17bc, 0x1798, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x17a1, 0x17bb, 0x1784, 0x1798,
0x17c9, 0x17bb, 0x1784, 0x178f, 0x17bb, 0x1793, 0x1798, 0x17c9, 0x17bb, 0x1793, 0x178f, 0x17c1,
0x179a, 0x17c9, 0x17c1, 0x1798, 0x17c9, 0x17bb, 0x1784, 0x178f, 0x17c1, 0x179c, 0x17b8, 0x178a,
0x17c1, 0x17a2, 0x17bc, 0x1798, 0x17c9, 0x17bb, 0x1784, 0x179f, 0x17ca, 0x17c2, 0x179a, 0x17c9,
0x17b6, 0x178f, 0x200b, 0x178e, 0x17b6, 0x179f, 0x17d2, 0x179f, 0x17bc, 0x1789, 0x17bc, 0x179c,
0x1799, 0x17c9, 0x1780, 0x178e, 0x17bc, 0x179a, 0x17c9, 0x17bb, 0x1793, 0x1789, 0x17c9, 0x17b6,
0x1794, 0x17ca, 0x17ba, 0x17a1, 0x17b6, 0x20, 0x178a, 0x17b6, 0x1780, 0x17bc, 0x178f, 0x17b6,
0x200b, 0x1781, 0x17b6, 0x1784, 0x1787, 0x17be, 0x1784, 0x179f, 0x17b7, 0x1793, 0x1792, 0x17ba,
0x20, 0x178a, 0x17b6, 0x1780, 0x17bc, 0x178f, 0x17b6, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x1787, 0x17be, 0x1784, 0x1789, 0x17bc, 0x179c, 0x200b, 0x179f, 0x17b6, 0x17a1, 0x17b9, 0x1798,
0x20, 0x178a, 0x17b6, 0x1780, 0x17bc, 0x178f, 0x17b6, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x1787, 0x17be, 0x1784, 0x1793, 0x17bc, 0x1780, 0x17a2, 0x17bc, 0x1787, 0x17b8, 0x178e, 0x17b6,
0x17a0, 0x17d2, 0x1780, 0x17b6, 0x1794, 0x17c9, 0x17b6, 0x178e, 0x17b6, 0x1798, 0x17c9, 0x17b6,
0x1794, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x179a, 0x17b8, 0x1794,
0x17bc, 0x1795, 0x17bc, 0x1793, 0x17b8, 0x1780, 0x1796, 0x17d0, 0x179a, 0x17a2, 0x17bc, 0x1794,
0x17d2, 0x179a, 0x17c9, 0x17b6, 0x17c6, 0x1784, 0x1796, 0x17d0, 0x179a, 0x17a2, 0x17c1, 0x179f,
0x17d2, 0x1794, 0x17c9, 0x17b6, 0x1789, 0x1796, 0x17d0, 0x179a, 0x178f, 0x17bc, 0x200b, 0x179c,
0x17b7, 0x179b, 0x17a0, 0x17bc, 0x1796, 0x17d0, 0x179a, 0x178f, 0x17bc, 0x179a, 0x17b8, 0x1780,
0x17bc, 0x1796, 0x17bb, 0x1793, 0x178f, 0x17b6, 0x20, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17c1,
0x178e, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x1793, 0x17cb, 0x1783, 0x17b8, 0x1793, 0x200b, 0x17a2,
0x17ca, 0x17b8, 0x1793, 0x17a1, 0x17b7, 0x178f, 0x179a, 0x17c9, 0x17c1, 0x179f, 0x17ca, 0x17b8,
0x17a0, 0x17d2, 0x179c, 0x17b8, 0x179a, 0x17c9, 0x17c1, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x178e,
0x17b6, 0x179a, 0x17c9, 0x17c1, 0x200b, 0x179f, 0x17ca, 0x17bc, 0x17a1, 0x17bc, 0x178f, 0x179a,
0x17b8, 0x1799, 0x17c9, 0x17bc, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x1793, 0x1780, 0x17bc, 0x179f,
0x17b6, 0x1793, 0x17cb, 0x178f, 0x17b6, 0x179a, 0x17b9, 0x1798, 0x179f, 0x17b6, 0x1793, 0x17cb,
0x1791, 0x17b8, 0x17a2, 0x17c1, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x179f, 0x17b6, 0x1793, 0x17cb,
0x178a, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x17c6, 0x1784, 0x179f, 0x17c5, 0x200b, 0x1794, 0x17c9,
0x17bc, 0x17a1, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x178f, 0x17bc, 0x1782, 0x17d0, 0x179a, 0x178f,
0x17bc, 0x1798, 0x17b8, 0x178f, 0x179f, 0x17ca, 0x17b8, 0x178f, 0x1780, 0x17b6, 0x179f, 0x17b6,
0x17c6, 0x1784, 0x200b, 0x1794, 0x17b6, 0x1792, 0x17b8, 0x179b, 0x17b8, 0x1798, 0x17c9, 0x17b8,
0x179f, 0x17b6, 0x17c6, 0x1784, 0x200b, 0x1785, 0x1793, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x200b,
0x1783, 0x17b8, 0x178f, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x200b, 0x179b, 0x17bc, 0x179f, 0x17c0,
0x179f, 0x17b6, 0x17c6, 0x1784, 0x200b, 0x178f, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x179f, 0x17cb,
0x179f, 0x17b6, 0x17c6, 0x1784, 0x200b, 0x179c, 0x17b8, 0x1793, 0x179f, 0x17c1, 0x1793, 0x179f,
0x17d2, 0x179c, 0x17b8, 0x178f, 0x200b, 0x1781, 0x17ba, 0x179a, 0x17b7, 0x1793, 0x178f, 0x17c1,
0x17a0, 0x17d2, 0x1782, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x179b,
0x17cb, 0x1794, 0x17c9, 0x17b6, 0x1792, 0x17bb, 0x17a1, 0x17c1, 0x1791, 0x17b8, 0x1799, 0x17d2,
0x1799, 0x17bc, 0x17a2, 0x17b6, 0x178e, 0x17b6, 0x178f, 0x17bc, 0x179a, 0x17c9, 0x17bb, 0x1793,
0x178f, 0x17bc, 0x178f, 0x17bc, 0x178f, 0x17bc, 0x17a1, 0x17b6, 0x179c, 0x17c9, 0x17b6, 0x1793,
0x17cb, 0x1780, 0x17bc, 0x179c, 0x17be, 0x179c, 0x17c9, 0x17b6, 0x1799, 0x17a0, 0x179f, 0x179c,
0x17b8, 0x1793, 0x17b8, 0x1797, 0x17b7, 0x1780, 0x1799, 0x17c9, 0x17b6, 0x1782, 0x17bc, 0x178f,
0x17b6, 0x178f, 0x1780, 0x17b6, 0x179f, 0x17b8, 0x178a, 0x17b6, 0x179c, 0x17b8, 0x179f, 0x178c,
0x17bb, 0x1799, 0x1798, 0x17c9, 0x17bb, 0x1784, 0x178a, 0x17ba, 0x17a2, 0x17ca, 0x17bb, 0x1799,
0x179c, 0x17b8, 0x179b, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x1781, 0x17d2, 0x179c, 0x17b6, 0x179a,
0x17b8, 0x1798, 0x17c9, 0x17c5, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1780,
0x17cb, 0x1798, 0x17bb, 0x1799, 0x178a, 0x17bc, 0x1795, 0x17b6, 0x179b, 0x17cb, 0x1798, 0x17d0,
0x179a, 0x179a, 0x17c9, 0x17bc, 0x1792, 0x17b8, 0x179a, 0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8,
0x1799, 0x17c9, 0x17bc, 0x179c, 0x17c9, 0x17b6, 0x178f, 0x17d2, 0x179a, 0x17bc, 0x179b, 0x179c,
0x17c9, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x17a1, 0x17bb, 0x1784, 0x200b, 0x1799,
0x17c9, 0x17b6, 0x200b, 0x1794, 0x17ca, 0x17b8, 0x1799, 0x17c1, 0x1793, 0x17a2, 0x17b6, 0x178a,
0x17c2, 0x1793, 0x17a2, 0x17b6, 0x179b, 0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1791, 0x17b8, 0x17a2,
0x17b6, 0x1798, 0x17c9, 0x17b6, 0x1793, 0x17cb, 0x17a2, 0x17b6, 0x178e, 0x17b6, 0x178c, 0x17b8,
0x17a2, 0x17b6, 0x1780, 0x1791, 0x17bc, 0x17a2, 0x17b6, 0x1780, 0x1791, 0x17bc, 0x1794, 0x17ca,
0x17b8, 0x17a2, 0x17b6, 0x179f, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1794, 0x17b6, 0x178f, 0x17a2,
0x17b6, 0x1791, 0x17b8, 0x179a, 0x17c9, 0x17bc, 0x1794, 0x17b6, 0x1780, 0x178a, 0x17b6, 0x178a,
0x1794, 0x17b6, 0x179a, 0x17c9, 0x17c2, 0x1793, 0x1794, 0x17b6, 0x1782, 0x17bc, 0x1794, 0x17b6,
0x1784, 0x1780, 0x1780, 0x1794, 0x17b6, 0x1793, 0x17bc, 0x179b, 0x1794, 0x17c1, 0x179a, 0x17bc,
0x178f, 0x1794, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1780, 0x17c1, 0x1780, 0x1796, 0x17d2, 0x179a,
0x17bb, 0x1799, 0x178e, 0x17c1, 0x1788, 0x17b8, 0x178f, 0x17b6, 0x1780, 0x17bc, 0x17a1, 0x17bb,
0x17c6, 0x1794, 0x17bc, 0x178a, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x179f, 0x178a, 0x17b6, 0x1780,
0x17d2, 0x1780, 0x17b6, 0x178a, 0x17c1, 0x179b, 0x17b8, 0x178c, 0x17bb, 0x1799, 0x1794, 0x17c3,
0x178c, 0x17bc, 0x179f, 0x17d2, 0x17a0, 0x17b6, 0x1793, 0x17cb, 0x1794, 0x17c1, 0x17a0, 0x17d2,
0x179c, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17bb, 0x179f, 0x17d2,
0x178f, 0x17b6, 0x17a0, 0x17c1, 0x1794, 0x179a, 0x17bb, 0x1793, 0x17a0, 0x17bc, 0x1787, 0x17b8,
0x1798, 0x17b8, 0x1789, 0x17a0, 0x17bb, 0x1784, 0x1780, 0x17bb, 0x1784, 0x17a0, 0x17bc, 0x179c,
0x17a2, 0x17c0, 0x179a, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x17a0,
0x17d2, 0x179f, 0x17b6, 0x1780, 0x17b6, 0x179a, 0x178f, 0x17b6, 0x1785, 0x17b6, 0x1799, 0x17c9,
0x17b6, 0x1797, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17c1, 0x179a,
0x17bb, 0x1799, 0x179f, 0x17b6, 0x17a1, 0x17b7, 0x1798, 0x1780, 0x17b6, 0x1794, 0x17ca, 0x17bb,
0x179b, 0x1780, 0x17b6, 0x17c6, 0x1786, 0x17b6, 0x178f, 0x1780, 0x17b6, 0x1780, 0x17b6, 0x179a,
0x17c9, 0x17b6, 0x1787, 0x17b8, 0x1780, 0x17b6, 0x178f, 0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1793,
0x17cb, 0x178c, 0x17bc, 0x1781, 0x17b6, 0x1793, 0x17cb, 0x178c, 0x17b8, 0x17a0, 0x17d2, 0x1782,
0x17b6, 0x1780, 0x17bb, 0x179b, 0x1780, 0x17b6, 0x178f, 0x17b6, 0x1780, 0x17d2, 0x179a, 0x17b6,
0x179f, 0x1793, 0x17bb, 0x1799, 0x17c9, 0x17b6, 0x1780, 0x1780, 0x17bc, 0x17a1, 0x17b6, 0x17a1,
0x17b6, 0x17c6, 0x1796, 0x17bd, 0x179a, 0x1783, 0x17ba, 0x178f, 0x1785, 0x17b8, 0x1784, 0x1780,
0x17bc, 0x179c, 0x17c9, 0x17c2, 0x178f, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17b6, 0x179c, 0x1798,
0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x178a, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17b6,
0x1780, 0x17b6, 0x179f, 0x179f, 0x17b6, 0x179a, 0x1798, 0x17c9, 0x17b6, 0x1793, 0x17b8, 0x179b,
0x1798, 0x17c9, 0x17b6, 0x179f, 0x17d2, 0x1780, 0x17b6, 0x178f, 0x1793, 0x17b8, 0x1780, 0x17bc,
0x179f, 0x17ca, 0x17b8, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x1783, 0x17bc, 0x179f, 0x178e,
0x17c1, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc,
0x179f, 0x17ca, 0x17b8, 0x1794, 0x17ca, 0x17ba, 0x1780, 0x17a2, 0x17bc, 0x1798, 0x179f, 0x17d2,
0x1780, 0x17b7, 0x17cd, 0x17a2, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x179b, 0x17cb, 0x1797, 0x17d2,
0x1793, 0x17c6, 0x1796, 0x17c1, 0x1789, 0x1794, 0x17c9, 0x17bb, 0x1793, 0x1791, 0x17b8, 0x17a2,
0x17b6, 0x178e, 0x17b6, 0x1780, 0x17cb, 0x1796, 0x17d2, 0x1799, 0x17bb, 0x1784, 0x1799, 0x17c9,
0x17b6, 0x1784, 0x1780, 0x17bc, 0x179f, 0x17d2, 0x178a, 0x17c1, 0x178e, 0x17c1, 0x1782, 0x17b8,
0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17b8, 0x17a1, 0x17bc, 0x178a, 0x17b6, 0x179a, 0x17b8, 0x1799,
0x17c9, 0x17b6, 0x178a, 0x179f, 0x17b6, 0x1781, 0x17b6, 0x179b, 0x17b8, 0x1793, 0x179f, 0x17b6,
0x1798, 0x17c9, 0x17b6, 0x1780, 0x17b6, 0x1793, 0x17cb, 0x179f, 0x17c1, 0x17a2, 0x17ca, 0x17bc,
0x179b, 0x179f, 0x17c0, 0x1784, 0x17a0, 0x17c3, 0x179f, 0x17b7, 0x1784, 0x17d2, 0x17a0, 0x1794,
0x17bc, 0x179a, 0x17b8, 0x179f, 0x17d2, 0x179a, 0x17b8, 0x178f, 0x1793, 0x17b8, 0x1780, 0x17bc,
0x179b, 0x17b8, 0x1798, 0x178f, 0x17c3, 0x1794, 0x17c9, 0x17b7, 0x178f, 0x17b6, 0x179f, 0x17d2,
0x1780, 0x17b7, 0x1793, 0x1794, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x179f, 0x17ca, 0x17b8, 0x178f,
0x17c1, 0x17a0, 0x17c1, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x1792, 0x17b8, 0x1798, 0x1797, 0x17bc,
0x178f, 0x17bc, 0x1780, 0x17d2, 0x1799, 0x17bc, 0x1790, 0x1798, 0x179f, 0x17d2, 0x1780, 0x17b7,
0x17cd, 0x17a2, 0x17ca, 0x17bc, 0x17a1, 0x1784, 0x17cb, 0x1794, 0x17b6, 0x1791, 0x17d0, 0x179a,
0x17a2, 0x17ca, 0x17bc, 0x179a, 0x17b6, 0x17c6, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x17a2, 0x17bc,
0x179f, 0x17d2, 0x178f, 0x178e, 0x17c1, 0x179a, 0x17c9, 0x17b6, 0x179c, 0x17c0, 0x1784, 0x1785,
0x1793, 0x17d2, 0x1791, 0x1793, 0x17cd, 0x179c, 0x17c9, 0x17d2, 0x179b, 0x17b6, 0x178a, 0x17b8,
0x179c, 0x17c9, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1799, 0x17c9, 0x17b6, 0x1782,
0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x17a0,
0x17d2, 0x1782, 0x17bc, 0x1793, 0x1799, 0x17c9, 0x17c1, 0x1780, 0x17b6, 0x1792, 0x17ba, 0x179a,
0x17b7, 0x1793, 0x1794, 0x17ca, 0x17be, 0x1780, 0x1799, 0x17c9, 0x17c1, 0x179a, 0x17c9, 0x17c1,
0x179c, 0x17c9, 0x17b6, 0x1793, 0x17cb, 0x17a2, 0x17b6, 0x179f, 0x17bc, 0x179a, 0x17c9, 0x17c1,
0x179f, 0x1794, 0x17c9, 0x17ba, 0x1798, 0x17bb, 0x1799, 0x178a, 0x17b6, 0x1780, 0x17b6, 0x178e,
0x17b6, 0x179a, 0x17b8, 0x1780, 0x17b6, 0x1794, 0x17cb, 0x179c, 0x17c2, 0x179a, 0x17a0, 0x17d2,
0x179c, 0x17b6, 0x179a, 0x17c9, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x178a, 0x17c0, 0x179a, 0x17c9,
0x17b6, 0x179a, 0x17c9, 0x17c3, 0x1780, 0x17d2, 0x1799, 0x17b6, 0x179c, 0x17b7, 0x1780, 0x17a0,
0x17d2, 0x179f, 0x1780, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17b8, 0x200b, 0x1781, 0x17b6, 0x1784,
0x200b, 0x178f, 0x17d2, 0x1794, 0x17bc, 0x1784, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x17a0, 0x17c1,
0x17a1, 0x17c1, 0x178e, 0x17b6, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x179b, 0x17b8, 0x17a2,
0x17b6, 0x178a, 0x17c1, 0x17a1, 0x17c2, 0x178f, 0x1794, 0x17d2, 0x179a, 0x17b8, 0x179f, 0x1794,
0x17ca, 0x17c2, 0x1793, 0x1794, 0x17d2, 0x179a, 0x17bc, 0x1780, 0x1781, 0x17b7, 0x1793, 0x17a0,
0x17b8, 0x179b, 0x178a, 0x17b6, 0x179c, 0x17b8, 0x1793, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x1780,
0x17d2, 0x179b, 0x17b6, 0x17a0, 0x17bc, 0x1794, 0x17b6, 0x178f, 0x179b, 0x17b8, 0x1793, 0x178c,
0x17b8, 0x1798, 0x17c9, 0x17c2, 0x1793, 0x17a1, 0x178f, 0x17a0, 0x17bc, 0x179c, 0x17c9, 0x17c1,
0x1798, 0x17c9, 0x17c1, 0x179b, 0x1794, 0x17ca, 0x1793, 0x1797, 0x17ba, 0x1792, 0x179f, 0x17ca,
0x17b8, 0x178a, 0x1793, 0x17b8, 0x17a2, 0x17b6, 0x17c6, 0x179f, 0x17d2, 0x1791, 0x17c2, 0x178a,
0x17b6, 0x17c6, 0x17a2, 0x1784, 0x17cb, 0x178a, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x17a2, 0x17b6,
0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x1781, 0x17b6, 0x1793, 0x17cb, 0x17a2, 0x17b6,
0x1790, 0x17c2, 0x1793, 0x1794, 0x17c2, 0x179b, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x178a, 0x1794,
0x17c2, 0x179a, 0x17a1, 0x17b6, 0x17c6, 0x1784, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x1791, 0x17b8,
0x179f, 0x17d2, 0x179b, 0x17b6, 0x179c, 0x17c9, 0x17b6, 0x1796, 0x17d2, 0x179a, 0x17bb, 0x1785,
0x179f, 0x17c2, 0x179b, 0x1794, 0x17ca, 0x17bb, 0x1799, 0x1780, 0x17b6, 0x179a, 0x17c9, 0x17c1,
0x179f, 0x1794, 0x17ca, 0x17bb, 0x1799, 0x178a, 0x17b6, 0x1794, 0x17c9, 0x17c2, 0x179f, 0x1794,
0x17ca, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17c2, 0x1793, 0x1787,
0x17b8, 0x179f, 0x17ca, 0x17b8, 0x178e, 0x17bc, 0x1780, 0x17bc, 0x1794, 0x17c9, 0x17b7, 0x1793,
0x17a0, 0x17b6, 0x1780, 0x178c, 0x17bb, 0x1799, 0x1794, 0x17d2, 0x179b, 0x17b6, 0x17c6, 0x1784,
0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17b8, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17cb, 0x178f,
0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17c1, 0x1793, 0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17c2, 0x179b,
0x179f, 0x17ca, 0x17b7, 0x1793, 0x1782, 0x17b8, 0x17a2, 0x17c2, 0x179b, 0x17a2, 0x17bb, 0x17a0,
0x17d2, 0x179c, 0x1798, 0x17c9, 0x17c2, 0x1793, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x1794, 0x17ca, 0x17bb, 0x179b, 0x1787, 0x17c1, 0x179f, 0x17ca, 0x17b8, 0x1780,
0x17b6, 0x179b, 0x17b8, 0x1793, 0x17b8, 0x1784, 0x17a0, 0x17d2, 0x179a, 0x17d2, 0x1782, 0x17b6,
0x178a, 0x1782, 0x17b8, 0x179a, 0x17c9, 0x17bc, 0x179c, 0x1780, 0x17c0, 0x179c, 0x179b, 0x17b8,
0x179f, 0x1794, 0x17c4, 0x1793, 0x1787, 0x17bc, 0x1794, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x178e,
0x17b6, 0x17a1, 0x17bb, 0x1784, 0x178a, 0x17cd, 0x179b, 0x17bb, 0x1785, 0x179f, 0x17c6, 0x1794,
0x17bd, 0x1798, 0x17c9, 0x17b6, 0x178c, 0x17d2, 0x179a, 0x17b8, 0x178a, 0x1798, 0x17c9, 0x17b6,
0x179b, 0x17cb, 0x178f, 0x17cd, 0x1798, 0x17c9, 0x17b6, 0x179a, 0x17c0, 0x17a0, 0x17b6, 0x17c6,
0x1798, 0x17b8, 0x1793, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1798, 0x17c9, 0x17bc, 0x178e,
0x17b6, 0x1780, 0x17bc, 0x1798, 0x17bc, 0x179f, 0x17d2, 0x1782, 0x17bc, 0x17a2, 0x17bc, 0x179f,
0x17d2, 0x179b, 0x17bc, 0x1794, 0x17c9, 0x17b6, 0x179a, 0x17b8, 0x179f, 0x1794, 0x17c9, 0x17bc,
0x178a, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x179a, 0x17b8, 0x1780, 0x17b6, 0x1794, 0x17d2, 0x179a,
0x17b6, 0x1780, 0x179a, 0x17b8, 0x17a0, 0x17d2, 0x1780, 0x17b6, 0x179f, 0x17b6, 0x1798, 0x17c9,
0x17b6, 0x179a, 0x17c9, 0x17b6, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x1798, 0x17c9, 0x17b6, 0x179a,
0x17c9, 0x17b6, 0x17c6, 0x1784, 0x179f, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x1799, 0x17c9, 0x17c1,
0x179c, 0x17c9, 0x17bc, 0x179f, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x178f, 0x17b6, 0x179c, 0x179f,
0x17ca, 0x17b8, 0x1798, 0x17a0, 0x17d2, 0x179c, 0x17be, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x17c9,
0x17bc, 0x179b, 0x179f, 0x17d2, 0x1780, 0x17bb, 0x1794, 0x1787, 0x17c1, 0x179f, 0x17bc, 0x17a0,
0x17ca, 0x17d2, 0x179c, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780,
0x1781, 0x17bb, 0x179b, 0x178f, 0x17b6, 0x179b, 0x17b7, 0x1793, 0x1791, 0x17b8, 0x179a, 0x17c9,
0x17b6, 0x178e, 0x17c1, 0x17a2, 0x17bb, 0x179b, 0x1799, 0x17c9, 0x17b6, 0x178e, 0x17bc, 0x179c,
0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x179c, 0x17c9, 0x17b6, 0x178c, 0x17bc, 0x179f, 0x179c,
0x17c9, 0x17b6, 0x1791, 0x17b8, 0x1780, 0x1784, 0x17cb, 0x179c, 0x17b8, 0x1799, 0x17c2, 0x1793,
0x179c, 0x17b8, 0x179b, 0x1793, 0x17b8, 0x1789, 0x17bc, 0x179f, 0x179c, 0x17c9, 0x17bb, 0x179b,
0x17a0, 0x17d2, 0x1782, 0x17c4, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b6, 0x178a, 0x179c,
0x17c9, 0x17b6, 0x179f, 0x17bc, 0x179c, 0x17b8, 0x179f, 0x17b6, 0x1782, 0x17d2, 0x179a, 0x17c2,
0x1794, 0x17a0, 0x17ca, 0x17d2, 0x179f, 0x17bc, 0x179a, 0x17b8, 0x1785, 0x17a2, 0x1784, 0x17cb,
0x178f, 0x17b6, 0x178e, 0x17b6, 0x178e, 0x17b6, 0x179a, 0x17b8, 0x179c, 0x1782, 0x17d2, 0x179a,
0x17b8, 0x179f, 0x17d2, 0x1798, 0x17b6, 0x179f, 0x1780, 0x17bc, 0x1780, 0x17bc, 0x179f, 0x1780,
0x17bc, 0x1798, 0x17d0, 0x179a, 0x1783, 0x17ba, 0x17a0, 0x17d2, 0x1782, 0x17bb, 0x1799, 0x17a1,
0x17c2, 0x1793, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17c1, 0x1798, 0x17c9, 0x17b6, 0x179b, 0x17cb,
0x178c, 0x17b8, 0x179c, 0x1798, 0x17c9, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17b6,
0x1799, 0x17bb, 0x178f, 0x179a, 0x17c1, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x17c9, 0x17bb, 0x1784,
0x17a2, 0x17b6, 0x1796, 0x17b8, 0x17a2, 0x1780, 0x17a1, 0x17c2, 0x1793, 0x1794, 0x17bc, 0x17a0,
0x17d2, 0x1782, 0x17c2, 0x1793, 0x179c, 0x17b8, 0x179b, 0x1785, 0x17b6, 0x1790, 0x17b6, 0x17c6,
0x1788, 0x17bc, 0x17a2, 0x17bb, 0x1780, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1791, 0x17d0,
0x179a, 0x17a2, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b6, 0x178f, 0x17c1, 0x17a0, 0x17d2,
0x179c, 0x17b6, 0x1780, 0x17c5, 0x17a0, 0x17d2, 0x179c, 0x17bc, 0x17a0, 0x17d2, 0x179c, 0x17b8,
0x1787, 0x17b8, 0x17a0, 0x17ca, 0x17d2, 0x179c, 0x17bc, 0x178e, 0x17b6, 0x17a0, 0x17ca, 0x17d2,
0x179c, 0x17bc, 0x1791, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x17a1, 0x17b6, 0x1794, 0x17c9,
0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x179f, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x17c6, 0x1794,
0x17c0, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x178a, 0x17b6, 0x179b, 0x17cb, 0x1781, 0x17b6, 0x178e,
0x17b6, 0x179b, 0x17cb, 0x17a0, 0x17bc, 0x178e, 0x17bc, 0x179b, 0x17bc, 0x179b, 0x17bc, 0x17a2,
0x17ca, 0x17b8, 0x1793, 0x178a, 0x17ba, 0x1794, 0x17ca, 0x17bc, 0x179a, 0x17b8, 0x1782, 0x17b7,
0x179a, 0x17b8, 0x1791, 0x17b8, 0x1798, 0x17c9, 0x17b6, 0x1791, 0x17b7, 0x1780, 0x17bc, 0x179f,
0x17d2, 0x179a, 0x17c2, 0x1780, 0x17d2, 0x179c, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x179b,
0x17c0, 0x1793, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17bc, 0x179a, 0x17c9, 0x17bc,
0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x179f, 0x17b6, 0x179f, 0x1798, 0x17b8,
0x178f, 0x179c, 0x17c9, 0x17c1, 0x178e, 0x17bc, 0x179a, 0x17bc, 0x1793, 0x17b8, 0x179c, 0x17c9,
0x17c1, 0x178e, 0x17d0, 0x179a, 0x17a0, 0x17d2, 0x179c, 0x17bb, 0x1780, 0x178e, 0x17bc, 0x1798,
0x17c0, 0x1794, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x20, 0x1794, 0x17c9, 0x17b6,
0x17a0, 0x17d2, 0x1782, 0x17c4, 0x1794, 0x17c9, 0x17b6, 0x17a1, 0x17c5, 0x1797, 0x17b8, 0x178f,
0x1781, 0x17c3, 0x179a, 0x17c9, 0x17b7, 0x1793, 0x1795, 0x17bb, 0x1793, 0x1794, 0x17c9, 0x17b8,
0x1796, 0x17d0, 0x179a, 0x1798, 0x17c9, 0x17bc, 0x179a, 0x17c1, 0x179f, 0x1794, 0x17ca, 0x17b8,
0x179a, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17bc, 0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782,
0x17b6, 0x179f, 0x17c3, 0x1794, 0x17c9, 0x17b6, 0x1793, 0x178f, 0x17b6, 0x17a0, 0x17b8, 0x1791,
0x17b8, 0x178f, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x179c, 0x17c9, 0x17b6, 0x178f, 0x17bb, 0x1784,
0x17a0, 0x17d2, 0x1782, 0x17b6, 0x178f, 0x17b6, 0x1796, 0x17bc, 0x179c, 0x17c9, 0x17b6, 0x1782,
0x17b8, 0x179c, 0x17c9, 0x17b6, 0x179b, 0x17b8, 0x179f, 0x906, 0x915, 0x930, 0x93e, 0x906,
0x926, 0x940, 0x938, 0x20, 0x905, 0x92c, 0x93e, 0x92c, 0x93e, 0x906, 0x932, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x92c, 0x93e, 0x902, 0x917, 0x940,
0x92c, 0x93e, 0x928, 0x91c, 0x941, 0x932, 0x92c, 0x93f, 0x938, 0x93e, 0x909, 0x92c,
0x94d, 0x932, 0x93e, 0x928, 0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d, 0x930, 0x93e,
0x91d, 0x93e, 0x935, 0x93f, 0x932, 0x947, 0x92c, 0x941, 0x91c, 0x941, 0x92e, 0x92c,
0x941, 0x930, 0x93e, 0x915, 0x94b, 0x928, 0x93e, 0x915, 0x94d, 0x930, 0x93e, 0x926,
0x93e, 0x930, 0x20, 0x907, 0x20, 0x938, 0x932, 0x93e, 0x92e, 0x921, 0x94c, 0x906,
0x932, 0x93e, 0x905, 0x932, 0x20, 0x906, 0x92f, 0x941, 0x928, 0x92b, 0x94d, 0x930,
0x93f, 0x91f, 0x93e, 0x935, 0x928, 0x91c, 0x94b, 0x939, 0x93e, 0x928, 0x94d, 0x938,
0x92c, 0x930, 0x94d, 0x917, 0x916, 0x93e, 0x930, 0x94d, 0x91f, 0x94b, 0x92e, 0x915,
0x93f, 0x928, 0x936, 0x93e, 0x938, 0x93e, 0x92e, 0x93e, 0x938, 0x947, 0x930, 0x941,
0x92e, 0x94b, 0x917, 0x93e, 0x926, 0x93f, 0x936, 0x942, 0x928, 0x93e, 0x92f, 0x930,
0x94b, 0x92c, 0x940, 0x90f, 0x928, 0x91c, 0x93e, 0x92e, 0x947, 0x928, 0x93e, 0x928,
0x935, 0x93e, 0x915, 0x938, 0x941, 0x924, 0x913, 0x917, 0x93e, 0x926, 0x94c, 0x917,
0x94b, 0x91f, 0x941, 0x928, 0x93f, 0x938, 0x905, 0x921, 0x93e, 0x915, 0x910, 0x902,
0x915, 0x930, 0x91c, 0x905, 0x902, 0x917, 0x941, 0x932, 0x93e, 0x90f, 0x901, 0x91f,
0x93f, 0x917, 0x941, 0x906, 0x905, 0x930, 0x917, 0x941, 0x92f, 0x947, 0x928, 0x93e,
0x92c, 0x94d, 0x92f, 0x942, 0x928, 0x938, 0x20, 0x906, 0x92f, 0x930, 0x94d, 0x938,
0x915, 0x93e, 0x91f, 0x93e, 0x92e, 0x93e, 0x930, 0x915, 0x93e, 0x915, 0x94b, 0x930,
0x921, 0x94b, 0x92c, 0x93e, 0x91c, 0x941, 0x91c, 0x947, 0x92f, 0x932, 0x93e, 0x20,
0x930, 0x93f, 0x92f, 0x94b, 0x91c, 0x93e, 0x92e, 0x947, 0x928, 0x94d, 0x921, 0x94b,
0x91d, 0x93e, 0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x917, 0x932, 0x947, 0x917, 0x949,
0x938, 0x938, 0x93e, 0x928, 0x20, 0x91c, 0x941, 0x906, 0x928, 0x91f, 0x941, 0x915,
0x941, 0x92e, 0x945, 0x928, 0x905, 0x930, 0x941, 0x92c, 0x93e, 0x905, 0x938, 0x902,
0x938, 0x93f, 0x92f, 0x928, 0x905, 0x924, 0x93f, 0x915, 0x94b, 0x915, 0x949, 0x928,
0x92c, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x902, 0x926, 0x947, 0x930, 0x938,
0x92c, 0x947, 0x932, 0x93f, 0x91d, 0x92c, 0x94d, 0x932, 0x93e, 0x902, 0x915, 0x2d,
0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x94b, 0x928, 0x92c, 0x93e, 0x913, 0x20, 0x935,
0x93f, 0x938, 0x94d, 0x91f, 0x93e, 0x92c, 0x94b, 0x92f, 0x938, 0x915, 0x947, 0x902,
0x92c, 0x94d, 0x930, 0x93f, 0x91c, 0x20, 0x92c, 0x947, 0x915, 0x945, 0x92e, 0x94d,
0x92a, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x93e, 0x902, 0x921, 0x915, 0x93e, 0x928,
0x915, 0x941, 0x928, 0x915, 0x930, 0x93e, 0x915, 0x938, 0x915, 0x947, 0x907, 0x928,
0x938, 0x93f, 0x909, 0x926, 0x93e, 0x926, 0x20, 0x91c, 0x941, 0x906, 0x930, 0x947,
0x91c, 0x915, 0x94b, 0x938, 0x94d, 0x924, 0x93e, 0x20, 0x930, 0x93f, 0x915, 0x93e,
0x915, 0x94d, 0x930, 0x947, 0x938, 0x94d, 0x91f, 0x94b, 0x928, 0x915, 0x941, 0x92f,
0x92f, 0x92c, 0x93e, 0x921, 0x947, 0x928, 0x94d, 0x92e, 0x93e, 0x930, 0x94d, 0x915,
0x936, 0x949, 0x928, 0x921, 0x94c, 0x938, 0x928, 0x921, 0x94c, 0x938, 0x928, 0x20,
0x915, 0x94d, 0x930, 0x93f, 0x915, 0x921, 0x947, 0x928, 0x94d, 0x935, 0x947, 0x930,
0x921, 0x947, 0x91f, 0x94d, 0x930, 0x949, 0x908, 0x91f, 0x921, 0x94b, 0x92e, 0x93f,
0x928, 0x940, 0x915, 0x93e, 0x90d, 0x921, 0x92e, 0x94b, 0x902, 0x91f, 0x928, 0x90f,
0x932, 0x20, 0x938, 0x93e, 0x932, 0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930, 0x92b,
0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x928, 0x947, 0x932, 0x94d, 0x938, 0x928, 0x92b,
0x94b, 0x930, 0x94d, 0x91f, 0x93e, 0x932, 0x947, 0x91d, 0x93e, 0x917, 0x942, 0x938,
0x20, 0x92c, 0x947, 0x917, 0x94d, 0x930, 0x901, 0x921, 0x20, 0x924, 0x941, 0x930,
0x94d, 0x915, 0x917, 0x94d, 0x930, 0x947, 0x928, 0x945, 0x921, 0x93e, 0x917, 0x94d,
0x935, 0x93e, 0x92f, 0x947, 0x915, 0x93f, 0x932, 0x939, 0x947, 0x932, 0x93f, 0x92b,
0x945, 0x915, 0x94d, 0x938, 0x939, 0x930, 0x94d, 0x92e, 0x94b, 0x938, 0x93f, 0x932,
0x94b, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92a, 0x94b, 0x932,
0x940, 0x938, 0x928, 0x949, 0x915, 0x94d, 0x938, 0x2c, 0x20, 0x907, 0x902, 0x921,
0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92a, 0x93f, 0x91f, 0x930, 0x94d, 0x938, 0x92c,
0x930, 0x94d, 0x917, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x935, 0x947, 0x935, 0x92f, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f,
0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x902, 0x938, 0x947, 0x928, 0x938, 0x2c, 0x20,
0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x928, 0x93e,
0x92e, 0x945, 0x915, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x907, 0x928, 0x941, 0x935, 0x93f, 0x915, 0x907, 0x915, 0x93e, 0x932, 0x941,
0x92f, 0x940, 0x91f, 0x91c, 0x942, 0x928, 0x94b, 0x932, 0x941, 0x908, 0x91c, 0x935,
0x93f, 0x932, 0x947, 0x92e, 0x94b, 0x902, 0x91f, 0x940, 0x938, 0x945, 0x932, 0x94b,
0x2c, 0x20, 0x915, 0x947, 0x902, 0x91f, 0x941, 0x915, 0x93e, 0x92f, 0x915, 0x94d,
0x930, 0x93e, 0x932, 0x947, 0x928, 0x921, 0x940, 0x915, 0x932, 0x93e, 0x20, 0x92a,
0x93e, 0x91d, 0x932, 0x93f, 0x92e, 0x93e, 0x932, 0x949, 0x938, 0x20, 0x90f, 0x902,
0x91c, 0x932, 0x940, 0x938, 0x932, 0x94b, 0x935, 0x930, 0x20, 0x92a, 0x94d, 0x930,
0x93f, 0x902, 0x938, 0x947, 0x938, 0x20, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x94d,
0x91f, 0x930, 0x92e, 0x93e, 0x938, 0x93f, 0x92f, 0x94b, 0x92e, 0x945, 0x928, 0x93e,
0x917, 0x94d, 0x935, 0x93e, 0x92e, 0x947, 0x930, 0x940, 0x917, 0x949, 0x91f, 0x92e,
0x93e, 0x91d, 0x93e, 0x91f, 0x94d, 0x932, 0x93e, 0x928, 0x92e, 0x947, 0x91f, 0x94d,
0x932, 0x93e, 0x915, 0x93e, 0x91f, 0x94d, 0x932, 0x93e, 0x92e, 0x947, 0x915, 0x94d,
0x938, 0x93f, 0x915, 0x94b, 0x20, 0x936, 0x93e, 0x930, 0x92e, 0x93f, 0x915, 0x94d,
0x935, 0x947, 0x932, 0x94b, 0x928, 0x92e, 0x94b, 0x902, 0x915, 0x94d, 0x91f, 0x949,
0x928, 0x92e, 0x94b, 0x902, 0x91f, 0x947, 0x930, 0x940, 0x92e, 0x94b, 0x902, 0x91f,
0x947, 0x935, 0x94d, 0x939, 0x93f, 0x921, 0x93f, 0x92f, 0x94b, 0x92e, 0x949, 0x928,
0x94d, 0x91f, 0x938, 0x947, 0x930, 0x93e, 0x91f, 0x928, 0x938, 0x93e, 0x90a, 0x928,
0x94d, 0x92f, 0x941, 0x20, 0x92f, 0x949, 0x930, 0x94d, 0x915, 0x928, 0x94b, 0x92e,
0x947, 0x92c, 0x94d, 0x92f, 0x942, 0x932, 0x94d, 0x932, 0x93e, 0x939, 0x2c, 0x20,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e,
0x92e, 0x927, 0x94d, 0x92f, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20,
0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x928, 0x94d, 0x92f, 0x941, 0x20, 0x938,
0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20,
0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x913, 0x91c, 0x93f, 0x928, 0x93e, 0x917,
0x93e, 0x92a, 0x930, 0x93e, 0x92e, 0x93e, 0x930, 0x93f, 0x92c, 0x94b, 0x92b, 0x93f,
0x928, 0x93f, 0x915, 0x94d, 0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x2d, 0x914,
0x2d, 0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938, 0x92a, 0x94b, 0x930, 0x94d,
0x91f, 0x20, 0x911, 0x92b, 0x20, 0x938, 0x94d, 0x92a, 0x947, 0x928, 0x92a, 0x93f,
0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x930, 0x93f, 0x915, 0x94b, 0x92a, 0x941, 0x902,
0x91f, 0x93e, 0x20, 0x905, 0x930, 0x947, 0x928, 0x93e, 0x938, 0x930, 0x901, 0x915,
0x93f, 0x928, 0x20, 0x907, 0x928, 0x932, 0x945, 0x91f, 0x930, 0x947, 0x938, 0x940,
0x92b, 0x930, 0x947, 0x91c, 0x940, 0x928, 0x93e, 0x930, 0x93f, 0x938, 0x94b, 0x932,
0x94d, 0x92f, 0x941, 0x91f, 0x938, 0x93e, 0x902, 0x924, 0x93e, 0x930, 0x947, 0x92e,
0x938, 0x901, 0x91f, 0x93f, 0x92f, 0x93e, 0x917, 0x94b, 0x938, 0x93e, 0x902, 0x924,
0x94b, 0x20, 0x921, 0x94b, 0x92e, 0x93f, 0x902, 0x917, 0x94b, 0x938, 0x93e, 0x913,
0x20, 0x92a, 0x93e, 0x935, 0x932, 0x94b, 0x907, 0x91f, 0x94b, 0x915, 0x949, 0x930,
0x91f, 0x949, 0x930, 0x94d, 0x92e, 0x940, 0x91f, 0x938, 0x947, 0x902, 0x91f, 0x20,
0x91c, 0x949, 0x928, 0x94d, 0x938, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x932, 0x941,
0x938, 0x940, 0x92f, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x935, 0x93f, 0x928,
0x94d, 0x938, 0x947, 0x902, 0x91f, 0x938, 0x94d, 0x935, 0x93f, 0x92b, 0x94d, 0x91f,
0x20, 0x915, 0x930, 0x902, 0x91f, 0x924, 0x947, 0x917, 0x941, 0x938, 0x93f, 0x917,
0x932, 0x94d, 0x92a, 0x93e, 0x925, 0x941, 0x932, 0x947, 0x91f, 0x949, 0x930, 0x902,
0x91f, 0x94b, 0x91f, 0x94b, 0x930, 0x91f, 0x94b, 0x932, 0x93e, 0x935, 0x94d, 0x939,
0x901, 0x915, 0x941, 0x935, 0x94d, 0x939, 0x930, 0x935, 0x94d, 0x939, 0x93e, 0x92f,
0x91f, 0x939, 0x949, 0x930, 0x94d, 0x938, 0x92f, 0x93e, 0x915, 0x941, 0x91f, 0x93e,
0x91f, 0x921, 0x947, 0x935, 0x94d, 0x939, 0x940, 0x938, 0x926, 0x94d, 0x92f, 0x941,
0x92e, 0x93e, 0x901, 0x20, 0x926, 0x942, 0x930, 0x935, 0x940, 0x92f, 0x92e, 0x945,
0x915, 0x94d, 0x935, 0x947, 0x930, 0x940, 0x92e, 0x945, 0x915, 0x92e, 0x941, 0x930,
0x94d, 0x926, 0x94b, 0x92a, 0x93e, 0x932, 0x94d, 0x92e, 0x947, 0x930, 0x932, 0x94b,
0x902, 0x917, 0x92f, 0x947, 0x905, 0x930, 0x92c, 0x94d, 0x92f, 0x947, 0x928, 0x90d,
0x921, 0x928, 0x905, 0x928, 0x93e, 0x926, 0x93f, 0x930, 0x90d, 0x915, 0x94d, 0x91f,
0x94b, 0x905, 0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x905, 0x936, 0x917, 0x93e,
0x92c, 0x93e, 0x91f, 0x905, 0x924, 0x94d, 0x92f, 0x93e, 0x930, 0x94b, 0x92c, 0x947,
0x939, 0x930, 0x947, 0x928, 0x92c, 0x901, 0x915, 0x949, 0x915, 0x92c, 0x930, 0x928,
0x94c, 0x932, 0x92c, 0x947, 0x930, 0x942, 0x924, 0x91a, 0x93f, 0x91f, 0x93e, 0x926,
0x92e, 0x938, 0x94d, 0x915, 0x938, 0x926, 0x93f, 0x932, 0x940, 0x926, 0x941, 0x92c,
0x92f, 0x92b, 0x92e, 0x93e, 0x917, 0x941, 0x938, 0x94d, 0x924, 0x93e, 0x917, 0x93e,
0x91d, 0x93e, 0x939, 0x947, 0x92c, 0x94d, 0x930, 0x949, 0x928, 0x939, 0x94b, 0x935,
0x94d, 0x939, 0x921, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x91f, 0x938, 0x94d, 0x915,
0x91c, 0x93e, 0x92f, 0x93e, 0x92a, 0x941, 0x930, 0x93e, 0x91c, 0x947, 0x930, 0x941,
0x938, 0x93e, 0x932, 0x947, 0x92e, 0x915, 0x93e, 0x92e, 0x91a, 0x93e, 0x91f, 0x94d,
0x915, 0x93e, 0x916, 0x93e, 0x93c, 0x902, 0x921, 0x93f, 0x917, 0x93e, 0x915, 0x941,
0x906, 0x932, 0x93e, 0x932, 0x92e, 0x94d, 0x92a, 0x942, 0x930, 0x915, 0x941, 0x91a,
0x940, 0x902, 0x917, 0x915, 0x941, 0x935, 0x947, 0x924, 0x92e, 0x915, 0x93e, 0x935,
0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x92e, 0x915, 0x93e, 0x938, 0x94d, 0x938,
0x93e, 0x930, 0x92e, 0x928, 0x93f, 0x932, 0x93e, 0x928, 0x94b, 0x935, 0x94b, 0x915,
0x941, 0x91d, 0x928, 0x947, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x92b, 0x94d, 0x928,
0x94b, 0x92e, 0x20, 0x92a, 0x947, 0x928, 0x94d, 0x939, 0x92a, 0x94b, 0x923, 0x94d,
0x91f, 0x940, 0x905, 0x928, 0x93e, 0x915, 0x915, 0x924, 0x93e, 0x930, 0x915, 0x94b,
0x938, 0x94d, 0x91f, 0x93e, 0x928, 0x93e, 0x92f, 0x915, 0x94d, 0x92f, 0x94d, 0x91c,
0x94d, 0x92f, 0x932, 0x949, 0x930, 0x921, 0x93e, 0x938, 0x916, 0x932, 0x93f, 0x928,
0x936, 0x93e, 0x902, 0x918, 0x93e, 0x92f, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a,
0x942, 0x930, 0x938, 0x94d, 0x930, 0x947, 0x926, 0x928, 0x947, 0x915, 0x949, 0x932,
0x92f, 0x94d, 0x92e, 0x938, 0x94d, 0x915, 0x924, 0x948, 0x92a, 0x947, 0x908, 0x924,
0x93e, 0x936, 0x94d, 0x915, 0x902, 0x926, 0x924, 0x941, 0x92c, 0x932, 0x940, 0x938,
0x940, 0x925, 0x93f, 0x92e, 0x94d, 0x92b, 0x942, 0x924, 0x94b, 0x915, 0x94d, 0x92f,
0x94b, 0x91f, 0x949, 0x92e, 0x938, 0x94d, 0x915, 0x909, 0x932, 0x93e, 0x928, 0x92c,
0x924, 0x93e, 0x930, 0x909, 0x930, 0x941, 0x92e, 0x94d, 0x915, 0x940, 0x909, 0x938,
0x94d, 0x924, 0x20, 0x928, 0x947, 0x930, 0x93e, 0x935, 0x94d, 0x939, 0x93f, 0x90f,
0x928, 0x94d, 0x91f, 0x93f, 0x92f, 0x928, 0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938,
0x94d, 0x915, 0x92f, 0x93e, 0x928, 0x917, 0x949, 0x928, 0x905, 0x91d, 0x94b, 0x930,
0x947, 0x938, 0x92c, 0x930, 0x94d, 0x92e, 0x941, 0x921, 0x93e, 0x915, 0x945, 0x928,
0x930, 0x940, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x926, 0x947, 0x92b,
0x948, 0x930, 0x94b, 0x92e, 0x926, 0x93f, 0x930, 0x93e, 0x930, 0x93f, 0x915, 0x94d,
0x91c, 0x947, 0x935, 0x93f, 0x915, 0x938, 0x947, 0x902, 0x2e, 0x20, 0x939, 0x947,
0x932, 0x947, 0x928, 0x93e, 0x938, 0x94d, 0x91f, 0x947, 0x928, 0x94d, 0x932, 0x947,
0x92c, 0x94d, 0x930, 0x93f, 0x938, 0x94d, 0x92c, 0x947, 0x928, 0x909, 0x915, 0x94d,
0x932, 0x93e, 0x932, 0x93f, 0x921, 0x947, 0x92e, 0x945, 0x928, 0x92e, 0x947, 0x932,
0x92c, 0x930, 0x94d, 0x928, 0x90d, 0x92e, 0x94d, 0x938, 0x94d, 0x91f, 0x930, 0x921,
0x945, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x916, 0x928, 0x905, 0x925,
0x947, 0x928, 0x94d, 0x938, 0x92c, 0x94d, 0x930, 0x93e, 0x91f, 0x93f, 0x938, 0x94d,
0x932, 0x93e, 0x935, 0x94d, 0x939, 0x93e, 0x92c, 0x94d, 0x930, 0x941, 0x938, 0x947,
0x932, 0x94d, 0x938, 0x92c, 0x941, 0x916, 0x93e, 0x930, 0x947, 0x938, 0x94d, 0x91f,
0x92c, 0x941, 0x938, 0x93f, 0x902, 0x91c, 0x947, 0x928, 0x91a, 0x93f, 0x938, 0x940,
0x928, 0x93e, 0x90a, 0x915, 0x94b, 0x92a, 0x928, 0x939, 0x947, 0x917, 0x928, 0x921,
0x92c, 0x94d, 0x932, 0x93f, 0x928, 0x907, 0x938, 0x932, 0x947, 0x20, 0x911, 0x92b,
0x20, 0x92e, 0x945, 0x928, 0x907, 0x938, 0x94d, 0x924, 0x902, 0x92c, 0x942, 0x932,
0x915, 0x93e, 0x932, 0x93f, 0x928, 0x93f, 0x928, 0x917, 0x94d, 0x930, 0x93e, 0x921,
0x91c, 0x941, 0x92c, 0x94d, 0x932, 0x91c, 0x93e, 0x928, 0x93e, 0x932, 0x902, 0x921,
0x928, 0x932, 0x915, 0x94d, 0x938, 0x947, 0x92e, 0x92c, 0x930, 0x94d, 0x917, 0x92e,
0x93e, 0x926, 0x94d, 0x930, 0x940, 0x926, 0x92e, 0x945, 0x930, 0x940, 0x939, 0x93e,
0x92e, 0x94d, 0x928, 0x92e, 0x94b, 0x928, 0x945, 0x915, 0x94b, 0x92a, 0x945, 0x930,
0x93f, 0x938, 0x92a, 0x949, 0x921, 0x917, 0x94b, 0x930, 0x93f, 0x915, 0x93e, 0x938,
0x945, 0x928, 0x20, 0x92e, 0x93e, 0x930, 0x940, 0x928, 0x94b, 0x938, 0x93e, 0x930,
0x93e, 0x91c, 0x93f, 0x935, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x91f, 0x94b, 0x935,
0x938, 0x93f, 0x92e, 0x92b, 0x930, 0x94b, 0x92a, 0x94b, 0x932, 0x938, 0x94b, 0x92b,
0x93f, 0x92f, 0x93e, 0x91f, 0x945, 0x932, 0x93f, 0x928, 0x924, 0x93f, 0x930, 0x93e,
0x928, 0x93e, 0x935, 0x921, 0x941, 0x91c, 0x935, 0x945, 0x91f, 0x93f, 0x915, 0x928,
0x935, 0x93f, 0x92f, 0x947, 0x928, 0x93e, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d,
0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x935, 0x93e, 0x930, 0x938, 0x949,
0x91d, 0x93e, 0x917, 0x94d, 0x930, 0x947, 0x92c, 0x91d, 0x941, 0x930, 0x93f, 0x91a,
0x905, 0x902, 0x91f, 0x93e, 0x928, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x935, 0x94b,
0x91a, 0x93e, 0x917, 0x94b, 0x938, 0x915, 0x947, 0x930, 0x917, 0x941, 0x932, 0x947,
0x928, 0x92e, 0x949, 0x930, 0x93f, 0x936, 0x938, 0x92e, 0x947, 0x92f, 0x94b, 0x91f,
0x930, 0x93f, 0x92f, 0x941, 0x928, 0x93f, 0x92f, 0x928, 0x905, 0x92a, 0x93f, 0x92f,
0x93e, 0x911, 0x915, 0x932, 0x901, 0x921, 0x92c, 0x94b, 0x917, 0x947, 0x928, 0x935,
0x93f, 0x932, 0x947, 0x91a, 0x945, 0x925, 0x92e, 0x91a, 0x941, 0x915, 0x907, 0x92b,
0x947, 0x91f, 0x92b, 0x93e, 0x915, 0x93e, 0x913, 0x92b, 0x94b, 0x92b, 0x941, 0x928,
0x93e, 0x92b, 0x941, 0x91f, 0x940, 0x917, 0x93e, 0x932, 0x93e, 0x92a, 0x93e, 0x917,
0x94b, 0x938, 0x917, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x917, 0x94d,
0x935, 0x93e, 0x926, 0x93e, 0x932, 0x915, 0x945, 0x928, 0x93e, 0x932, 0x907, 0x902,
0x921, 0x930, 0x92c, 0x930, 0x940, 0x915, 0x93f, 0x930, 0x93f, 0x924, 0x93f, 0x92e,
0x93e, 0x924, 0x940, 0x915, 0x94b, 0x938, 0x930, 0x947, 0x915, 0x94d, 0x935, 0x93e,
0x91c, 0x932, 0x947, 0x928, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93f, 0x938, 0x938,
0x928, 0x93e, 0x935, 0x930, 0x942, 0x928, 0x949, 0x930, 0x92b, 0x94b, 0x915, 0x928,
0x942, 0x92e, 0x947, 0x906, 0x92a, 0x93f, 0x91f, 0x915, 0x945, 0x930, 0x928, 0x92a,
0x94b, 0x939, 0x94d, 0x928, 0x92a, 0x947, 0x908, 0x92a, 0x94b, 0x930, 0x94d, 0x91f,
0x20, 0x92e, 0x949, 0x930, 0x947, 0x938, 0x94d, 0x92c, 0x940, 0x924, 0x93e, 0x930,
0x935, 0x93e, 0x935, 0x948, 0x915, 0xc544, 0xbe44, 0xc7a5, 0xc544, 0xd06c, 0xb77c, 0xc544,
0xb514, 0xc2a4, 0xc544, 0xbc14, 0xbc14, 0xc54c, 0xc81c, 0xc544, 0xc2a4, 0xba54, 0xb77c, 0xbc14,
0xb9c8, 0xcf54, 0xbc29, 0xae30, 0xbc18, 0xc904, 0xbe44, 0xc0ac, 0xc6b0, 0xbe14, 0xb79c, 0xd0c0,
0xc774, 0xc5b4, 0xbe0c, 0xb77c, 0xc790, 0xbe4c, 0xbd80, 0xc90c, 0xbd80, 0xb77c, 0xce74, 0xc774,
0xb85c, 0xce74, 0xc0ac, 0xbe14, 0xb791, 0xce74, 0xc138, 0xc6b0, 0xd0c0, 0xcf54, 0xb098, 0xd06c,
0xb9ac, 0xb2e4, 0xce74, 0xb974, 0xb2e4, 0xb974, 0xc5d0, 0xc2a4, 0xc0b4, 0xb78c, 0xc9c0, 0xbd80,
0xd2f0, 0xb450, 0xc54c, 0xb77c, 0xc5d8, 0xc544, 0xc774, 0xc6b4, 0xd504, 0xb9ac, 0xd0c0, 0xc6b4,
0xac00, 0xbcf4, 0xb85c, 0xb124, 0xd558, 0xb77c, 0xb808, 0xc694, 0xd558, 0xb124, 0xc2a4, 0xbc84,
0xadf8, 0xc8fc, 0xbc14, 0xce84, 0xd314, 0xb77c, 0xce74, 0xb974, 0xd23c, 0xd0a4, 0xac08, 0xb9ac,
0xd0a8, 0xc0e4, 0xc0ac, 0xb77c, 0xace0, 0xc2a4, 0xb9ac, 0xbe0c, 0xb974, 0xbe4c, 0xb85c, 0xba54,
0xb8e8, 0xc548, 0xb2e4, 0xb8e8, 0xbd90, 0xbc14, 0xc2dc, 0xb8e8, 0xc0ac, 0xce74, 0xb9d0, 0xb77c,
0xbcf4, 0xb9c8, 0xd478, 0xd1a0, 0xb9c8, 0xc138, 0xb8e8, 0xc74c, 0xbc14, 0xbc14, 0xb124, 0xbaa8,
0xac00, 0xb514, 0xc288, 0xbaac, 0xb85c, 0xbe44, 0xc544, 0xb098, 0xc774, 0xb85c, 0xbe44, 0xc5d4,
0xc790, 0xba54, 0xb098, 0xb2c8, 0xc544, 0xba54, 0xb204, 0xc545, 0xc1fc, 0xd2b8, 0xc640, 0xac00,
0xb450, 0xad6c, 0xd3ec, 0xb974, 0xd1a0, 0xb178, 0xbcf4, 0xc0c1, 0xd22c, 0xba54, 0xd2b8, 0xb9ac,
0xd3f4, 0xb9ac, 0xd280, 0xb2c8, 0xc2a4, 0xbe48, 0xd2b8, 0xd6c4, 0xd06c, 0xc5d0, 0xc774, 0xb2e5,
0xc575, 0xcee4, 0xb9ac, 0xc9c0, 0xc559, 0xadc8, 0xb77c, 0xc548, 0xd2f0, 0xacfc, 0xc544, 0xb77c,
0xacfc, 0xc774, 0xb098, 0xbd80, 0xc5d0, 0xb178, 0xc2a4, 0x20, 0xc544, 0xc774, 0xb808, 0xc2a4,
0xce74, 0xd0c0, 0xb9c8, 0xb974, 0xce74, 0xcf54, 0xb974, 0xb3c4, 0xbc14, 0xd6c4, 0xd6c4, 0xc774,
0xb77c, 0x20, 0xb9ac, 0xc624, 0xd558, 0xba58, 0xb3c4, 0xc0ac, 0xb9ac, 0xc624, 0x20, 0xac00,
0xc608, 0xace0, 0xc2a4, 0xc0b4, 0xd0c0, 0xc0b0, 0xd6c4, 0xc548, 0xc0b0, 0xb8e8, 0xc774, 0xc2a4,
0xd22c, 0xcfe0, 0xb9cc, 0xc6b0, 0xc218, 0xc544, 0xc774, 0xc544, 0xc544, 0xb8e8, 0xbc14, 0xc544,
0xc21c, 0xc2dc, 0xc628, 0xcf54, 0xb784, 0xd558, 0xbc84, 0xbc14, 0xd788, 0xc544, 0xbc14, 0xc774,
0xc544, 0x20, 0xbc18, 0xb370, 0xb77c, 0xc2a4, 0xbc14, 0xbca0, 0xc774, 0xb3c4, 0xc2a4, 0xbca8,
0xb818, 0xbca8, 0xb9ac, 0xc988, 0xbe14, 0xb791, 0x20, 0xc0ac, 0xbe14, 0xb871, 0xbcf4, 0xc544,
0x20, 0xbe44, 0xc2a4, 0xd0c0, 0xbcf4, 0xace0, 0xd0c0, 0xbcf4, 0xc774, 0xc2dc, 0xcf00, 0xc784,
0xbe0c, 0xb9ac, 0xc9c0, 0x20, 0xbca0, 0xc774, 0xce84, 0xd3ec, 0x20, 0xadf8, 0xb780, 0xb370,
0xce78, 0xcfe4, 0xce74, 0xb77c, 0xce74, 0xc2a4, 0xce74, 0xc60c, 0xcf00, 0xc774, 0xb9e8, 0xc2dc,
0xce74, 0xace0, 0xce58, 0xc640, 0xc640, 0xc2dc, 0xc6b0, 0xb2e4, 0xb4dc, 0xd6c4, 0xc544, 0xb808,
0xc2a4, 0xcf54, 0xc2a4, 0xd0c0, 0xb9ac, 0xce74, 0xd06c, 0xb808, 0xc2a4, 0xd1a4, 0xcfe0, 0xc774,
0xc544, 0xbc14, 0xd034, 0xb77c, 0xc18c, 0xb374, 0xb9c8, 0xd06c, 0xc0e4, 0xbe10, 0xb3c4, 0xc2a8,
0xb3c4, 0xc2a8, 0xd06c, 0xb9ad, 0xb374, 0xbc84, 0xb514, 0xd2b8, 0xb85c, 0xc774, 0xd2b8, 0xb3c4,
0xbbf8, 0xb2c8, 0xce74, 0xc5d0, 0xb4dc, 0xba3c, 0xd134, 0xc544, 0xc774, 0xb8e8, 0xb124, 0xd398,
0xc5d8, 0xc0b4, 0xbc14, 0xb3c4, 0xb974, 0xd3ec, 0xd2b8, 0x20, 0xb12c, 0xc2a8, 0xd3ec, 0xb974,
0xd0c8, 0xb808, 0xc790, 0xae00, 0xb77c, 0xc2a4, 0xbca0, 0xc774, 0xad6c, 0xc988, 0xbca0, 0xc774,
0xadf8, 0xb79c, 0xb4dc, 0x20, 0xd130, 0xd06c, 0xadf8, 0xb808, 0xb098, 0xb2e4, 0xacfc, 0xb4e4,
0xb8e8, 0xd504, 0xacfc, 0xd14c, 0xb9d0, 0xb77c, 0xacfc, 0xc57c, 0xd0ac, 0xac00, 0xc774, 0xc544,
0xb098, 0xd57c, 0xb9ac, 0xd329, 0xc2a4, 0xd558, 0xbc14, 0xb098, 0xc5d0, 0xb974, 0xbaa8, 0xc2dc,
0xc694, 0xc778, 0xb514, 0xc560, 0xb098, 0xd3f4, 0xb9ac, 0xc2a4, 0xc778, 0xb514, 0xc560, 0xb098,
0xc8fc, 0x2c, 0x20, 0xb179, 0xc2a4, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20,
0xba38, 0xb81d, 0xace0, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20, 0xd53c, 0xce20,
0xbc84, 0xadf8, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20, 0xd154, 0xc2dc, 0xd2f0,
0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20, 0xbe44, 0xbe44, 0xc778, 0xb514, 0xc560,
0xb098, 0xc8fc, 0x2c, 0x20, 0xbe48, 0xc13c, 0xc2a4, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc,
0x2c, 0x20, 0xc704, 0xb108, 0xb9e5, 0xc774, 0xb204, 0xbe45, 0xc774, 0xce7c, 0xb8e8, 0xc774,
0xd2b8, 0xc790, 0xba54, 0xc774, 0xce74, 0xc8fc, 0xb178, 0xb8e8, 0xc774, 0xbe4c, 0xcf04, 0xd130,
0xd0a4, 0xc8fc, 0x2c, 0x20, 0xbaac, 0xd2f0, 0xccbc, 0xb85c, 0xd06c, 0xb77c, 0xb80c, 0xb514,
0xc988, 0xd06c, 0xb77c, 0xd30c, 0xc2a4, 0xb9ac, 0xb9c8, 0xb85c, 0xc2a4, 0xc564, 0xc824, 0xb808,
0xc2a4, 0xb85c, 0xc6cc, 0x20, 0xd504, 0xb9b0, 0xc2a4, 0x20, 0xcffc, 0xd130, 0xb9c8, 0xc138,
0xc774, 0xc624, 0xb9c8, 0xb098, 0xacfc, 0xb9c8, 0xb098, 0xc6b0, 0xc2a4, 0xb9c8, 0xb9ac, 0xacf3,
0xb9c8, 0xd2f0, 0xb2c8, 0xd06c, 0xb9c8, 0xd0c0, 0xbaa8, 0xb85c, 0xc2a4, 0xb9c8, 0xc0ac, 0xd2c0,
0xb780, 0xba54, 0xb178, 0xbbf8, 0xb2c8, 0xba54, 0xb9ac, 0xb2e4, 0xba54, 0xd2c0, 0xb77c, 0xce74,
0xd2c0, 0xb77c, 0xba55, 0xc2dc, 0xcf54, 0x20, 0xc2dc, 0xd2f0, 0xbbf8, 0xd074, 0xb871, 0xbabd,
0xd134, 0xbaac, 0xd14c, 0xb808, 0xc774, 0xbaac, 0xd14c, 0xbe44, 0xb370, 0xc624, 0xbaac, 0xc138,
0xb77c, 0xd2b8, 0xb098, 0xc18c, 0xb274, 0xc695, 0xb188, 0xb178, 0xb871, 0xc57c, 0xb178, 0xc2a4,
0xb2e4, 0xcf54, 0xd0c0, 0xc8fc, 0x2c, 0x20, 0xbca0, 0xb77c, 0xc911, 0xbd80, 0x2c, 0x20,
0xb178, 0xc2a4, 0xb2e4, 0xcf54, 0xd0c0, 0xb178, 0xc2a4, 0xb2e4, 0xcf54, 0xd0c0, 0xc8fc, 0x2c,
0x20, 0xb274, 0xc0b4, 0xb818, 0xace0, 0xb4dc, 0xd638, 0xd504, 0xc624, 0xd788, 0xb098, 0xac00,
0xd30c, 0xb098, 0xb9c8, 0xd30c, 0xb77c, 0xb9c8, 0xb9ac, 0xbcf4, 0xd53c, 0xb2c9, 0xc2a4, 0xd3ec,
0xb974, 0xd1a0, 0xd504, 0xb7ad, 0xc2a4, 0xd3ec, 0xd2b8, 0xc624, 0xbe0c, 0xc2a4, 0xd398, 0xc778,
0xd3ec, 0xb974, 0xd1a0, 0xbca8, 0xb8cc, 0xd478, 0xc5d0, 0xb974, 0xd1a0, 0xb9ac, 0xcf54, 0xd47c,
0xd0c0, 0xc544, 0xb808, 0xb098, 0xc2a4, 0xb7ad, 0xd0b9, 0x20, 0xc778, 0xb81b, 0xb808, 0xc2dc,
0xd398, 0xb9ac, 0xc790, 0xc774, 0xb098, 0xb9ac, 0xc878, 0xb8e8, 0xd2b8, 0xd788, 0xc6b0, 0x20,
0xbe0c, 0xb791, 0xcfe0, 0xc0b0, 0xd0c0, 0xb818, 0xc0b0, 0xd2f0, 0xc544, 0xace0, 0xc0b0, 0xd1a0,
0xb3c4, 0xbc0d, 0xace0, 0xc0c1, 0xd30c, 0xc6b8, 0xb8e8, 0xc2a4, 0xcf54, 0xb808, 0xc2a4, 0xbc14,
0xc774, 0xc120, 0xb4dc, 0xc2ef, 0xce74, 0xc0dd, 0xbc14, 0xb974, 0xd154, 0xb808, 0xbbf8, 0xc138,
0xc778, 0xd2b8, 0xc874, 0xc2a4, 0xc138, 0xc778, 0xd2b8, 0xd0a4, 0xce20, 0xc138, 0xc778, 0xd2b8,
0xb8e8, 0xc2dc, 0xc544, 0xc138, 0xc778, 0xd2b8, 0xd1a0, 0xb9c8, 0xc2a4, 0xc138, 0xc778, 0xd2b8,
0xbe48, 0xc13c, 0xd2b8, 0xc2a4, 0xc704, 0xd504, 0xd2b8, 0xcee4, 0xb7f0, 0xd2b8, 0xd14c, 0xad6c,
0xc2dc, 0xac08, 0xd30c, 0xd234, 0xb808, 0xd2f0, 0xd6c4, 0xc544, 0xb098, 0xd1a0, 0xb860, 0xd1a0,
0xd1a0, 0xb974, 0xd1a8, 0xb77c, 0xbca4, 0xcfe0, 0xbc84, 0xd654, 0xc774, 0xd2b8, 0xd638, 0xc2a4,
0xc704, 0xb2c8, 0xd399, 0xc57c, 0xcfe0, 0xd0c0, 0xd2b8, 0xcf00, 0xc774, 0xc2dc, 0xb370, 0xc774,
0xbe44, 0xc2a4, 0xb4a4, 0xbabd, 0x20, 0xb4a4, 0xb974, 0xbe4c, 0xb9e5, 0xcffc, 0xb9ac, 0xbaa8,
0xc2a8, 0xb9e5, 0xba38, 0xb3c4, 0xd30c, 0xba38, 0xb85c, 0xb370, 0xb77c, 0xc1fc, 0xc640, 0xd2b8,
0xb864, 0xbcf4, 0xc2a4, 0xd1a0, 0xd06c, 0xb871, 0xc774, 0xc5b4, 0xbe44, 0xc5d4, 0xc544, 0xb374,
0xc54c, 0xb9c8, 0xd2f0, 0xc554, 0xb9cc, 0xc544, 0xb098, 0xb514, 0xb9ac, 0xc544, 0xd06c, 0xd0c0,
0xc6b0, 0xc545, 0xd1a0, 0xbe0c, 0xc544, 0xc288, 0xd558, 0xbc14, 0xd2b8, 0xc544, 0xd2f0, 0xb77c,
0xc6b0, 0xbc14, 0xadf8, 0xb2e4, 0xb4dc, 0xbc14, 0xb808, 0xc778, 0xbc14, 0xcfe0, 0xbc29, 0xcf55,
0xbc14, 0xb974, 0xb098, 0xc6b8, 0xbca0, 0xc774, 0xb8e8, 0xd2b8, 0xbe44, 0xc288, 0xcf00, 0xd06c,
0xbe0c, 0xb8e8, 0xb098, 0xc774, 0xce58, 0xd0c0, 0xcf5c, 0xb86c, 0xbcf4, 0xb2e4, 0xb9c8, 0xc2a4,
0xcfe0, 0xc2a4, 0xb51c, 0xb9ac, 0xb450, 0xbc14, 0xc774, 0xb450, 0xc0e8, 0xbca0, 0xd30c, 0xb9c8,
0xad6c, 0xc2a4, 0xd0c0, 0xac00, 0xc790, 0xd5e4, 0xbe0c, 0xb860, 0xc0ac, 0xc774, 0xacf5, 0xd64d,
0xcf69, 0xd638, 0xbe0c, 0xb4dc, 0xc774, 0xb974, 0xcfe0, 0xce20, 0xd06c, 0xc790, 0xce74, 0xb974,
0xd0c0, 0xc790, 0xc57c, 0xd478, 0xb77c, 0xc608, 0xb8e8, 0xc0b4, 0xb818, 0xce74, 0xbd88, 0xce84,
0xcc28, 0xce74, 0xce74, 0xb77c, 0xce58, 0xce74, 0xd2b8, 0xb9cc, 0xb450, 0xd55c, 0xb514, 0xac00,
0xcf5c, 0xce74, 0xd0c0, 0xd06c, 0xb77c, 0xc2a4, 0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c, 0xcfe0,
0xc54c, 0xb77c, 0xb8f8, 0xd478, 0xb974, 0xcfe0, 0xce6d, 0xcfe0, 0xc6e8, 0xc774, 0xd2b8, 0xb9c8,
0xce74, 0xc624, 0xb9c8, 0xac00, 0xb2e8, 0xb9c8, 0xce74, 0xc0ac, 0xb974, 0xb9c8, 0xb2d0, 0xb77c,
0xbb34, 0xc2a4, 0xce74, 0xd2b8, 0xb2c8, 0xcf54, 0xc2dc, 0xc544, 0xb178, 0xbcf4, 0xcfe0, 0xc988,
0xb124, 0xce20, 0xd06c, 0xb178, 0xbcf4, 0xc2dc, 0xbe44, 0xb974, 0xc2a4, 0xd06c, 0xc634, 0xc2a4,
0xd06c, 0xc624, 0xb784, 0xd504, 0xb188, 0xd39c, 0xd3f0, 0xd2f0, 0xc544, 0xb098, 0xd06c, 0xd3c9,
0xc591, 0xce74, 0xd0c0, 0xb974, 0xcf54, 0xc2a4, 0xd0c0, 0xb098, 0xc774, 0xd0a4, 0xc9c8, 0xb85c,
0xb974, 0xb2e4, 0xb9ac, 0xc57c, 0xb4dc, 0xc0ac, 0xd560, 0xb9b0, 0xc0ac, 0xb9c8, 0xb974, 0xce78,
0xd2b8, 0xc11c, 0xc6b8, 0xc0c1, 0xd558, 0xc774, 0xc2f1, 0xac00, 0xd3ec, 0xb974, 0xc2a4, 0xb808,
0xb4dc, 0xb124, 0xcf5c, 0xb9bc, 0xc2a4, 0xd06c, 0xd0c0, 0xc774, 0xbca0, 0xc774, 0xd0c0, 0xc288,
0xcf04, 0xd2b8, 0xd2b8, 0xbe4c, 0xb9ac, 0xc2dc, 0xd14c, 0xd5e4, 0xb780, 0xd300, 0xbd80, 0xb3c4,
0xcfc4, 0xd1b0, 0xc2a4, 0xd06c, 0xc6b8, 0xb780, 0xbc14, 0xd1a0, 0xb974, 0xc6b0, 0xb8e8, 0xbb34,
0xce58, 0xc6b0, 0xc2a4, 0xd2f0, 0xb124, 0xb77c, 0xbe44, 0xc5d4, 0xd2f0, 0xc548, 0xbe14, 0xb77c,
0xb514, 0xbcf4, 0xc2a4, 0xd1a0, 0xd06c, 0xc57c, 0xcfe0, 0xce20, 0xd06c, 0xb791, 0xad70, 0xc608,
0xce74, 0xd14c, 0xb9b0, 0xbd80, 0xb974, 0xd06c, 0xc608, 0xb808, 0xbc18, 0xc544, 0xc870, 0xb808,
0xc2a4, 0xbc84, 0xbba4, 0xb2e4, 0xce74, 0xb098, 0xb9ac, 0xc544, 0x20, 0xc81c, 0xb3c4, 0xce74,
0xbcf4, 0x20, 0xbca0, 0xb974, 0xb370, 0xd398, 0xb85c, 0x20, 0xc81c, 0xb3c4, 0xb9c8, 0xb370,
0xc774, 0xb77c, 0xb808, 0xc774, 0xceac, 0xbe44, 0xd06c, 0xc0ac, 0xc6b0, 0xc2a4, 0xc870, 0xc9c0,
0xc544, 0xc138, 0xc778, 0xd2b8, 0x20, 0xd5ec, 0xb808, 0xb098, 0xc2a4, 0xd0e0, 0xb9ac, 0xc560,
0xb4e4, 0xb808, 0xc774, 0xb4dc, 0xbe0c, 0xb9ac, 0xc2a4, 0xbca0, 0xc778, 0xbe0c, 0xb85c, 0xcee8,
0xd790, 0xb2e4, 0xc708, 0xc720, 0xd074, 0xb77c, 0xd638, 0xbc14, 0xd2b8, 0xb9b0, 0xb370, 0xb9cc,
0xb85c, 0xb4dc, 0x20, 0xd558, 0xc6b0, 0xba5c, 0xbc84, 0xb978, 0xd37c, 0xc2a4, 0xc2dc, 0xb4dc,
0xb2c8, 0xc554, 0xc2a4, 0xd14c, 0xb974, 0xb2f4, 0xc548, 0xb3c4, 0xb77c, 0xc544, 0xc2a4, 0xd2b8,
0xb77c, 0xd55c, 0xc544, 0xd14c, 0xb124, 0xbca0, 0xc624, 0xadf8, 0xb77c, 0xb4dc, 0xbca0, 0xb97c,
0xb9b0, 0xbe0c, 0xb77c, 0xd2f0, 0xc2ac, 0xb77c, 0xbc14, 0xbe0c, 0xb93c, 0xc140, 0xbd80, 0xcfe0,
0xb808, 0xc288, 0xd2f0, 0xbd80, 0xb2e4, 0xd398, 0xc2a4, 0xd2b8, 0xbdd4, 0xc9c0, 0xac90, 0xd0a4,
0xc2dc, 0xb098, 0xc6b0, 0xcf54, 0xd39c, 0xd558, 0xac90, 0xb354, 0xbe14, 0xb9b0, 0xc9c0, 0xbe0c,
0xb864, 0xd130, 0xac74, 0xc9c0, 0xd5ec, 0xc2f1, 0xd0a4, 0xb9e8, 0xc12c, 0xc774, 0xc2a4, 0xd0c4,
0xbd88, 0xc800, 0xc9c0, 0xce7c, 0xb9ac, 0xb2cc, 0xadf8, 0xb77c, 0xb4dc, 0xd0a4, 0xb85c, 0xd504,
0xd0a4, 0xc608, 0xd504, 0xb9ac, 0xc2a4, 0xbcf8, 0xb958, 0xbe14, 0xb7b4, 0xb098, 0xb7f0, 0xb358,
0xb8e9, 0xc148, 0xbd80, 0xb974, 0xd06c, 0xb9c8, 0xb4dc, 0xb9ac, 0xb4dc, 0xbab0, 0xd0c0, 0xb9c8,
0xb9ac, 0xc5d0, 0xd568, 0xbbfc, 0xc2a4, 0xd06c, 0xbaa8, 0xb098, 0xcf54, 0xbaa8, 0xc2a4, 0xd06c,
0xbc14, 0xc624, 0xc2ac, 0xb85c, 0xd30c, 0xb9ac, 0xd3ec, 0xb4dc, 0xace0, 0xb9ac, 0xcc28, 0xd504,
0xb77c, 0xd558, 0xb9ac, 0xac00, 0xb85c, 0xb9c8, 0xc0ac, 0xb9c8, 0xb77c, 0xc0b0, 0xb9c8, 0xb9ac,
0xb178, 0xc0ac, 0xb77c, 0xc608, 0xbcf4, 0xc0ac, 0xb77c, 0xd1a0, 0xd504, 0xc2ec, 0xd398, 0xb85c,
0xd3f4, 0xc2a4, 0xcf54, 0xd398, 0xc18c, 0xd53c, 0xc544, 0xc2a4, 0xd1a1, 0xd640, 0xb984, 0xd0c8,
0xb9b0, 0xd2f0, 0xb77c, 0xb098, 0xc6b8, 0xb9ac, 0xc57c, 0xb178, 0xd504, 0xc2a4, 0xd06c, 0xd30c,
0xb450, 0xce20, 0xbc14, 0xd2f0, 0xce78, 0xbe44, 0xc5d4, 0xb098, 0xbe4c, 0xb2c8, 0xc6b0, 0xc2a4,
0xbcfc, 0xace0, 0xadf8, 0xb77c, 0xd2b8, 0xbc14, 0xb974, 0xc0e4, 0xbc14, 0xc790, 0xadf8, 0xb808,
0xbe0c, 0xcde8, 0xb9ac, 0xd788, 0xc548, 0xd0c0, 0xb098, 0xb098, 0xb9ac, 0xbcf4, 0xcc28, 0xace0,
0xc2a4, 0xd06c, 0xb9ac, 0xc2a4, 0xb9c8, 0xc2a4, 0xcf54, 0xcf54, 0xc2a4, 0xcf54, 0xbaa8, 0xb85c,
0xcf00, 0xb974, 0xcf08, 0xb80c, 0xb9c8, 0xd5e4, 0xbab0, 0xb514, 0xbe0c, 0xbaa8, 0xb9ac, 0xc154,
0xc2a4, 0xba54, 0xc694, 0xd2b8, 0xb808, 0xc704, 0xb2c8, 0xc639, 0xc544, 0xd53c, 0xc544, 0xc624,
0xd074, 0xb79c, 0xb4dc, 0xbd80, 0xac31, 0xbe4c, 0xcc44, 0xd140, 0xd2b8, 0xb8e8, 0xd06c, 0xc774,
0xc2a4, 0xd130, 0x20, 0xc12c, 0xc5d0, 0xd30c, 0xd14c, 0xd30c, 0xce74, 0xc624, 0xd478, 0xd53c,
0xc9c0, 0xd478, 0xb098, 0xd478, 0xd2f0, 0xac08, 0xb77c, 0xd30c, 0xace0, 0xc2a4, 0xac10, 0xbe44,
0xc5b4, 0xacfc, 0xb2ec, 0xce74, 0xb0a0, 0xad0c, 0xd638, 0xb180, 0xb8f0, 0xb8e8, 0xc5d4, 0xb354,
0xbca0, 0xb9ac, 0xd0a4, 0xb9ac, 0xd2f0, 0xb9c8, 0xd2f0, 0xcf54, 0xc2a4, 0xb808, 0xcf70, 0xc798,
0xb80c, 0xb9c8, 0xc8fc, 0xb85c, 0xb9c8, 0xd018, 0xc0ac, 0xc2a4, 0xbbf8, 0xb4dc, 0xc6e8, 0xc774,
0xb098, 0xc6b0, 0xb8e8, 0xb2c8, 0xc6b0, 0xc5d0, 0xb178, 0xd37d, 0xb204, 0xba54, 0xc544, 0xd30c,
0xace0, 0xd30c, 0xace0, 0xd314, 0xb77c, 0xc6b0, 0xd54f, 0xcf00, 0xc5b8, 0xd3ec, 0xb098, 0xd398,
0xd3ec, 0xd2b8, 0xbaa8, 0xb974, 0xc988, 0xbe44, 0xb77c, 0xb85c, 0xd1b5, 0xac00, 0xc0ac, 0xc774,
0xd310, 0xd0c0, 0xd788, 0xd2f0, 0xd0c0, 0xb77c, 0xc640, 0xd1b5, 0xac00, 0xd0c0, 0xd478, 0xc6e8,
0xc774, 0xd06c, 0xc6d4, 0xb9ac, 0xc2a4, 0x43, 0x65, 0x7a, 0x61, 0x79, 0xee, 0x72,
0x51, 0x61, 0x68, 0xee, 0x72, 0x65, 0x53, 0x65, 0x70, 0x74, 0x65, 0x4b,
0x6f, 0x6e, 0x61, 0x6b, 0x72, 0xee, 0x44, 0x61, 0x72, 0x75, 0x73, 0x65,
0x6c, 0x61, 0x6d, 0x43, 0x69, 0x62, 0xfb, 0x74, 0xee, 0x58, 0x61, 0x72,
0x74, 0xfb, 0x6d, 0x4b, 0xee, 0x67, 0x61, 0x6c, 0xee, 0x4b, 0xee, 0x6e,
0x15f, 0x61, 0x73, 0x61, 0x4c, 0xee, 0x62, 0x72, 0x65, 0x76, 0xee, 0x6c,
0x4d, 0x61, 0x73, 0x65, 0x72, 0xfb, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0xee,
0x15f, 0xfb, 0x4e, 0x61, 0xee, 0x72, 0x6f, 0x62, 0xee, 0x54, 0x72, 0x61,
0x62, 0x6c, 0xfb, 0x73, 0x54, 0xfb, 0x6e, 0x69, 0x73, 0x41, 0x72, 0xfb,
0x62, 0x61, 0x42, 0x61, 0x68, 0xee, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x61, 0x73, 0x42, 0x65, 0x6c, 0xee, 0x7a, 0x65, 0x43, 0x61,
0x6e, 0x63, 0xfb, 0x6e, 0x43, 0x69, 0xfb, 0x64, 0x61, 0x64, 0x20, 0x4a,
0x75, 0x61, 0x72, 0x65, 0x7a, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x20, 0x52,
0xee, 0x6b, 0x61, 0x44, 0x6f, 0x6d, 0xee, 0x6e, 0xee, 0x6b, 0x61, 0x4a,
0x61, 0x6d, 0x61, 0xee, 0x6b, 0x61, 0x4d, 0x65, 0x72, 0xee, 0x64, 0x61,
0x42, 0x65, 0xfb, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x4e, 0x65, 0x77, 0x20,
0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x20, 0x52, 0xee, 0x6b, 0x6f, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0x65, 0x6c, 0x65, 0x6d, 0x79, 0x44,
0x61, 0x76, 0xee, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0xee, 0x45, 0x6d,
0x61, 0x6e, 0x45, 0x15f, 0x71, 0x61, 0x62, 0x61, 0x74, 0x41, 0x74, 0xee,
0x72, 0x61, 0x76, 0x42, 0x65, 0x78, 0x64, 0x61, 0x42, 0x65, 0x68, 0x72,
0x65, 0x79, 0x6e, 0x42, 0x61, 0x6b, 0xfb, 0x42, 0x65, 0x79, 0x72, 0xfb,
0x74, 0x42, 0xee, 0x15f, 0x6b, 0x65, 0x6b, 0x42, 0x72, 0xfb, 0x6e, 0x65,
0x79, 0xc7, 0xee, 0x74, 0x61, 0x15e, 0x61, 0x6d, 0x44, 0xee, 0x6c, 0xee,
0x44, 0xfb, 0x62, 0x61, 0xee, 0x44, 0x75, 0x15f, 0x65, 0x6e, 0x62, 0x65,
0x58, 0x65, 0x7a, 0x65, 0x42, 0x61, 0x6a, 0x61, 0x72, 0xea, 0x20, 0x48,
0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0xea, 0x43,
0x61, 0x79, 0x61, 0x70, 0xfb, 0x72, 0x61, 0x51, 0x75, 0x64, 0x73, 0x4b,
0x61, 0x62, 0xfb, 0x6c, 0x4b, 0x61, 0x72, 0x61, 0xe7, 0xee, 0x4b, 0x61,
0x74, 0x6d, 0x61, 0x6e, 0x64, 0xfb, 0x58, 0x61, 0x6e, 0x64, 0xee, 0x67,
0x61, 0x4b, 0xfb, 0x61, 0x6c, 0x61, 0x20, 0x4c, 0xfb, 0x6d, 0x70, 0xfb,
0x72, 0x4b, 0xfb, 0xe7, 0xee, 0x6e, 0x67, 0x4b, 0x75, 0x77, 0x65, 0x79,
0x74, 0x4d, 0x61, 0x6e, 0xee, 0x6c, 0x61, 0x4c, 0x65, 0x66, 0x6b, 0x6f,
0x15f, 0x65, 0x51, 0x65, 0x74, 0x65, 0x72, 0x51, 0x69, 0x7a, 0x69, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0x53,
0x65, 0x6d, 0x65, 0x72, 0x6b, 0x61, 0x6e, 0x64, 0x53, 0x65, 0xfb, 0x6c,
0x15e, 0x61, 0x6e, 0x67, 0x68, 0x61, 0x79, 0x53, 0xee, 0x6e, 0x67, 0x61,
0x70, 0xfb, 0x72, 0x54, 0x61, 0xee, 0x70, 0x65, 0xee, 0x54, 0x61, 0x15f,
0x6b, 0x65, 0x6e, 0x74, 0x54, 0x69, 0x66, 0x6c, 0xee, 0x73, 0xdb, 0x6c,
0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0xdb, 0x72, 0xfb, 0x6d, 0xe7,
0xee, 0xdb, 0x73, 0x74, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x52, 0x65, 0x77,
0x61, 0x6e, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0xea, 0x42, 0x65, 0x72, 0x6d, 0xfb, 0x64, 0x61, 0x47, 0x69,
0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x79,
0x61, 0x79, 0xea, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x79, 0x61,
0x20, 0x42, 0x61, 0x15f, 0xfb, 0x72, 0x53, 0xee, 0x64, 0x6e, 0x65, 0x79,
0x41, 0x73, 0x74, 0x72, 0x61, 0x78, 0x61, 0x6e, 0x41, 0x74, 0xee, 0x6e,
0x61, 0x42, 0x65, 0x72, 0x6c, 0xee, 0x6e, 0x42, 0x75, 0x6b, 0x72, 0x65,
0x15f, 0x42, 0xfb, 0x64, 0x61, 0x70, 0x65, 0x15f, 0x74, 0x42, 0xfb, 0x73,
0xee, 0x6e, 0x67, 0x65, 0x6e, 0x4b, 0x69, 0x15f, 0xee, 0x6e, 0x65, 0x77,
0x44, 0xfb, 0x62, 0x6c, 0xee, 0x6e, 0x43, 0x65, 0x62, 0x65, 0x6c, 0xee,
0x74, 0x61, 0x72, 0x69, 0x71, 0x48, 0x65, 0x6c, 0x73, 0xee, 0x6e, 0x6b,
0xee, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x4d, 0x61, 0x6e, 0xea,
0x53, 0x74, 0x65, 0x6e, 0x62, 0x6f, 0x6c, 0x4b, 0x61, 0x6c, 0xee, 0x6e,
0xee, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x4b, 0xee, 0x72, 0x6f, 0x76, 0x4b,
0xee, 0x65, 0x76, 0x4c, 0xee, 0x7a, 0x62, 0x6f, 0x6e, 0x4c, 0x75, 0x6b,
0x73, 0x65, 0x6d, 0x62, 0xfb, 0x72, 0x67, 0x4d, 0x61, 0x64, 0x72, 0xee,
0x64, 0x4d, 0xee, 0x6e, 0x73, 0x6b, 0x50, 0x61, 0x72, 0xee, 0x73, 0x50,
0x6f, 0x64, 0x67, 0x6f, 0x72, 0xee, 0x6b, 0x61, 0x52, 0xee, 0x67, 0x61,
0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0xee, 0x6e, 0x6f, 0x53, 0x61,
0x72, 0x61, 0x79, 0x62, 0x6f, 0x73, 0x6e, 0x61, 0x55, 0x73, 0x6b, 0x75,
0x70, 0x54, 0x61, 0x6c, 0xee, 0x6e, 0x54, 0xee, 0x72, 0x61, 0x6e, 0x56,
0x61, 0x74, 0xee, 0x6b, 0x61, 0x6e, 0x56, 0x69, 0x79, 0x61, 0x6e, 0x61,
0x56, 0xee, 0x6c, 0x6e, 0xfb, 0x73, 0x57, 0x61, 0x72, 0x15f, 0x6f, 0x76,
0x61, 0x5a, 0xfb, 0x72, 0xee, 0x68, 0x4d, 0x61, 0x6c, 0x64, 0xee, 0x76,
0x46, 0xee, 0x6a, 0xee, 0x54, 0x61, 0x68, 0xee, 0x74, 0xee, 0x411, 0x430,
0x433, 0x438, 0x411, 0x440, 0x430, 0x437, 0x437, 0x430, 0x432, 0x438, 0x43b, 0x43b,
0x414, 0x430, 0x440, 0x20, 0x44d, 0x441, 0x20, 0x421, 0x430, 0x43b, 0x430, 0x430,
0x43c, 0x416, 0x438, 0x431, 0x443, 0x442, 0x438, 0x42d, 0x43b, 0x20, 0x410, 0x439,
0x443, 0x43d, 0x416, 0x443, 0x431, 0x430, 0x41b, 0x438, 0x431, 0x440, 0x435, 0x432,
0x438, 0x43b, 0x43b, 0x41d, 0x436, 0x430, 0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x439,
0x430, 0x43c, 0x435, 0x423, 0x438, 0x43d, 0x434, 0x445, 0x443, 0x43a, 0x410, 0x43d,
0x43a, 0x43e, 0x440, 0x438, 0x436, 0x420, 0x438, 0x43e, 0x20, 0x413, 0x430, 0x43b,
0x43b, 0x435, 0x433, 0x43e, 0x441, 0x423, 0x448, 0x443, 0x430, 0x439, 0x430, 0x411,
0x430, 0x439, 0x438, 0x430, 0x411, 0x430, 0x438, 0x439, 0x430, 0x20, 0x411, 0x430,
0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x41a, 0x44d, 0x43c, 0x431, 0x440, 0x438,
0x436, 0x20, 0x411, 0x435, 0x439, 0x427, 0x438, 0x445, 0x443, 0x430, 0x445, 0x443,
0x430, 0x41a, 0x443, 0x439, 0x430, 0x431, 0x430, 0x414, 0x43e, 0x443, 0x441, 0x43e,
0x43d, 0x20, 0x413, 0x440, 0x435, 0x43a, 0x42d, 0x43b, 0x20, 0x421, 0x430, 0x43b,
0x432, 0x430, 0x434, 0x43e, 0x440, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x20, 0x422,
0x4af, 0x440, 0x43a, 0x413, 0x432, 0x430, 0x434, 0x435, 0x43b, 0x443, 0x43f, 0x435,
0x413, 0x443, 0x430, 0x439, 0x430, 0x43a, 0x438, 0x43b, 0x413, 0x443, 0x439, 0x430,
0x43d, 0x430, 0x42d, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b, 0x43b, 0x43e, 0x41f,
0x438, 0x442, 0x435, 0x440, 0x441, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x418,
0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x422, 0x435, 0x43b, 0x43b, 0x20, 0x421,
0x438, 0x442, 0x438, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430,
0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x430, 0x43d, 0x430, 0x41b, 0x443, 0x438, 0x437, 0x432, 0x438, 0x43b, 0x43b, 0x41c,
0x43e, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x43b, 0x43b, 0x43e, 0x2c, 0x20, 0x41a,
0x435, 0x43d, 0x442, 0x443, 0x43a, 0x43a, 0x438, 0x41b, 0x43e, 0x441, 0x2d, 0x410,
0x43d, 0x436, 0x435, 0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x443, 0x44d, 0x440, 0x20,
0x41f, 0x440, 0x438, 0x43d, 0x441, 0x435, 0x441, 0x20, 0x41a, 0x443, 0x430, 0x442,
0x435, 0x41c, 0x435, 0x445, 0x438, 0x43a, 0x43e, 0x20, 0x448, 0x430, 0x430, 0x440,
0x44b, 0x41c, 0x43e, 0x43d, 0x441, 0x435, 0x440, 0x430, 0x442, 0x411, 0x435, 0x443,
0x43b, 0x430, 0x2c, 0x20, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20,
0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x43d, 0x44b, 0x43d, 0x20, 0x431, 0x43e,
0x440, 0x431, 0x43e, 0x440, 0x443, 0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x430, 0x43b,
0x435, 0x43c, 0x2c, 0x20, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x43e, 0x444,
0x20, 0x421, 0x43f, 0x435, 0x439, 0x43d, 0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x20,
0x412, 0x435, 0x43b, 0x438, 0x43e, 0x420, 0x44d, 0x43d, 0x43a, 0x438, 0x43d, 0x20,
0x418, 0x43d, 0x43b, 0x435, 0x442, 0x420, 0x435, 0x437, 0x43e, 0x43b, 0x443, 0x442,
0x418, 0x442, 0x442, 0x43e, 0x43a, 0x43a, 0x43e, 0x440, 0x442, 0x43e, 0x43e, 0x440,
0x43c, 0x438, 0x438, 0x442, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x411, 0x430, 0x440,
0x442, 0x435, 0x43b, 0x435, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x416,
0x43e, 0x43d, 0x441, 0x421, 0x430, 0x43d, 0x442, 0x430, 0x20, 0x41b, 0x44e, 0x447,
0x438, 0x44f, 0x421, 0x432, 0x438, 0x444, 0x442, 0x20, 0x41a, 0x430, 0x440, 0x440,
0x435, 0x43d, 0x442, 0x423, 0x438, 0x43d, 0x43d, 0x438, 0x43f, 0x435, 0x433, 0x414,
0x44d, 0x432, 0x438, 0x441, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2d,
0x423, 0x440, 0x432, 0x438, 0x43b, 0x41c, 0x430, 0x43a, 0x41c, 0x451, 0x440, 0x434,
0x43e, 0x421, 0x430, 0x43e, 0x430, 0x41b, 0x43e, 0x43d, 0x433, 0x439, 0x435, 0x440,
0x431, 0x438, 0x435, 0x43d, 0x416, 0x430, 0x43a, 0x430, 0x440, 0x442, 0x430, 0x416,
0x430, 0x439, 0x430, 0x43f, 0x443, 0x440, 0x430, 0x41a, 0x430, 0x43d, 0x434, 0x44b,
0x433, 0x430, 0x41a, 0x443, 0x447, 0x438, 0x4a3, 0x41c, 0x430, 0x43a, 0x430, 0x441,
0x441, 0x430, 0x440, 0x420, 0x438, 0x439, 0x430, 0x434, 0x4ae, 0x440, 0x4af, 0x43c,
0x447, 0x4af, 0x412, 0x438, 0x435, 0x43d, 0x442, 0x438, 0x430, 0x43d, 0x41a, 0x430,
0x43d, 0x430, 0x440, 0x438, 0x41a, 0x430, 0x43f, 0x435, 0x20, 0x412, 0x435, 0x440,
0x434, 0x435, 0x422, 0x4af, 0x448, 0x442, 0x4af, 0x43a, 0x20, 0x416, 0x43e, 0x440,
0x436, 0x438, 0x44f, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x415, 0x43b, 0x435, 0x43d,
0x430, 0x411, 0x440, 0x43e, 0x443, 0x43a, 0x435, 0x43d, 0x20, 0x425, 0x438, 0x43b,
0x411, 0x443, 0x437, 0x438, 0x43d, 0x433, 0x435, 0x43d, 0x416, 0x435, 0x440, 0x441,
0x438, 0x421, 0x43a, 0x43e, 0x43f, 0x439, 0x435, 0x424, 0x430, 0x434, 0x443, 0x446,
0x41a, 0x440, 0x438, 0x441, 0x43c, 0x430, 0x441, 0x41e, 0x43a, 0x43b, 0x44d, 0x43d,
0x434, 0x427, 0x443, 0x443, 0x43a, 0x418, 0x441, 0x442, 0x435, 0x440, 0x424, 0x438,
0x436, 0x438, 0x41a, 0x43e, 0x437, 0x440, 0x430, 0x435, 0x41a, 0x443, 0x430, 0x436,
0x430, 0x43b, 0x435, 0x439, 0x43d, 0x41c, 0x430, 0x436, 0x443, 0x440, 0x43e, 0x41c,
0x430, 0x440, 0x43a, 0x435, 0x437, 0x430, 0x441, 0x41f, 0x43e, 0x440, 0x442, 0x20,
0x41c, 0x43e, 0x44d, 0x440, 0x441, 0x431, 0x438, 0x420, 0x430, 0x433, 0x43e, 0x442,
0x43e, 0x43d, 0x433, 0x430, 0xead, 0xeb2, 0xe9a, 0xeb4, 0xe94, 0xec1, 0xe88, 0xe99,
0xead, 0xeb1, 0xe81, 0xe84, 0xea3, 0xeb2, 0xec1, 0xead, 0xe94, 0xe94, 0xeb4, 0xeaa,
0x20, 0xead, 0xeb0, 0xe9a, 0xeb2, 0xe9a, 0xeb2, 0xec1, 0xead, 0xea7, 0xec0, 0xe88,
0xe8d, 0xead, 0xeb1, 0xeaa, 0xea1, 0xeb2, 0xea3, 0xeb2, 0xe9a, 0xeb2, 0xea1, 0xeb2,
0xec2, 0xe81, 0xe9a, 0xeb1, 0xe87, 0xe81, 0xeb8, 0xe8d, 0xe9a, 0xeb2, 0xe99, 0xe88,
0xeb9, 0xe9a, 0xeb4, 0xeaa, 0xec0, 0xe8a, 0xebb, 0xeb2, 0xec1, 0xe9a, 0xea5, 0xe99,
0xec4, 0xe97, 0xea3, 0xecc, 0xe9a, 0xea3, 0xeb2, 0xe8a, 0xeb2, 0xea7, 0xeb4, 0xea7,
0xe9a, 0xeb9, 0xe88, 0xeb9, 0xea1, 0xe9a, 0xeb9, 0xea3, 0xeb2, 0xec4, 0xe84, 0xec2,
0xea3, 0xe84, 0xeb2, 0xe8a, 0xeb2, 0xe9a, 0xea5, 0xeb1, 0xe87, 0xe81, 0xeb2, 0xe8a,
0xeb5, 0xea7, 0xe95, 0xeb2, 0xec2, 0xe84, 0xe99, 0xeb2, 0xe84, 0xea3, 0xeb5, 0xe94,
0xeb2, 0xe81, 0xeb2, 0xe94, 0xeb2, 0xec0, 0xead, 0xeaa, 0xeaa, 0xeb0, 0xea5, 0xeb2,
0xea1, 0xe88, 0xeb5, 0xe9a, 0xeb9, 0xe95, 0xeb4, 0xe94, 0xeb9, 0xead, 0xeb2, 0xea5,
0xeb2, 0xec0, 0xead, 0xea7, 0x20, 0xead, 0xeb2, 0xea2, 0xeb9, 0xe99, 0xe9f, 0xea3,
0xeb5, 0xe97, 0xeb2, 0xea7, 0xe81, 0xeb2, 0xe9a, 0xecd, 0xec2, 0xea3, 0xe99, 0xeae,
0xeb2, 0xea3, 0xeb2, 0xec0, 0xea3, 0xec2, 0xe88, 0xeae, 0xeb1, 0xe99, 0xec0, 0xe99,
0xe94, 0xeaa, 0xec0, 0xe9a, 0xeb5, 0xe81, 0xe88, 0xeb9, 0xe9a, 0xeb2, 0xe84, 0xeb3,
0xe9b, 0xeb2, 0xea5, 0xeb2, 0xe84, 0xeb2, 0xe97, 0xeb9, 0xea1, 0xe84, 0xeb5, 0xe81,
0xeb2, 0xea5, 0xeb5, 0xe81, 0xeb4, 0xe99, 0xe8a, 0xeb2, 0xe8a, 0xeb2, 0xea5, 0xeb2,
0xec2, 0xe81, 0xeaa, 0xea5, 0xeb4, 0xec0, 0xe9a, 0xebc, 0xeb5, 0xea7, 0xeb4, 0xea7,
0xec2, 0xea5, 0xec0, 0xea1, 0xea5, 0xea7, 0xe99, 0xe94, 0xeb2, 0xea5, 0xeb9, 0xe9a,
0xeb3, 0xe9a, 0xeb2, 0xe8a, 0xeb4, 0xea5, 0xeb9, 0xe8a, 0xeb2, 0xe81, 0xeb2, 0xea1,
0xeb2, 0xea5, 0xeb2, 0xec2, 0xe9a, 0xea1, 0xeb2, 0xe9b, 0xeb9, 0xec2, 0xe95, 0xea1,
0xeb2, 0xec0, 0xe8a, 0xea3, 0xeb9, 0xead, 0xeb3, 0xe9a, 0xeb2, 0xe9a, 0xeb2, 0xec0,
0xe99, 0xec2, 0xea1, 0xe81, 0xeb2, 0xe94, 0xeb5, 0xe8a, 0xeb9, 0xea1, 0xead, 0xe99,
0xec2, 0xea3, 0xec0, 0xea7, 0xe8d, 0xec4, 0xe99, 0xec2, 0xea3, 0xe9a, 0xeb5, 0xec0,
0xead, 0xe99, 0xe88, 0xeb2, 0xec0, 0xea1, 0xe99, 0xec8, 0xeb2, 0xe99, 0xeb5, 0xead,
0xeb2, 0xec0, 0xea1, 0xe99, 0xeb9, 0xec1, 0xead, 0xe81, 0xe8a, 0xead, 0xe94, 0xead,
0xeb9, 0xe81, 0xeb2, 0xe94, 0xeb9, 0xe81, 0xeb9, 0xe9b, 0xecd, 0xec2, 0xe95, 0x2d,
0xec2, 0xe99, 0xec2, 0xea7, 0xe8a, 0xeb2, 0xea7, 0xec2, 0xe95, 0xec0, 0xea1, 0xe97,
0xea3, 0xeb4, 0xec2, 0xe9e, 0xea5, 0xeb4, 0xe95, 0xeb9, 0xe99, 0xeb4, 0xeaa, 0xea7,
0xeb5, 0xe99, 0xeae, 0xeb9, 0xe94, 0xead, 0xeb2, 0xec1, 0xe94, 0xe81, 0xec1, 0xead,
0xe99, 0xec0, 0xe84, 0xeb5, 0xec0, 0xea3, 0xe81, 0xec1, 0xead, 0xe99, 0xe81, 0xeb4,
0xe99, 0xea5, 0xeb2, 0xec1, 0xead, 0xe99, 0xe97, 0xeb4, 0xe81, 0xebb, 0xea7, 0xead,
0xeb2, 0xea3, 0xeb2, 0xe81, 0xebb, 0xea7, 0xe99, 0xeb2, 0xe9a, 0xebb, 0xea7, 0xec2,
0xe99, 0xeaa, 0x20, 0xec4, 0xead, 0xec0, 0xea3, 0xeaa, 0xe84, 0xeb2, 0xe95, 0xeb2,
0xea1, 0xeb2, 0xe81, 0xeb2, 0xe84, 0xecd, 0xec2, 0xe94, 0xe9a, 0xeb2, 0xe88, 0xeb9,
0xe88, 0xeb8, 0xe8d, 0xea5, 0xeb2, 0x20, 0xea3, 0xeb4, 0xec2, 0xead, 0xe88, 0xeb2,
0xec0, 0xea1, 0xeb1, 0xe99, 0xec2, 0xe94, 0xe8a, 0xec8, 0xeb2, 0xea3, 0xeb4, 0xec2,
0xe81, 0x20, 0xec1, 0xe81, 0xea7, 0xec0, 0xea5, 0xe81, 0xead, 0xeaa, 0xe8a, 0xeb2,
0xe99, 0xe95, 0xeb2, 0xec1, 0xe8a, 0xe99, 0xeae, 0xea7, 0xe99, 0xec1, 0xe8a, 0xe99,
0xea5, 0xeb8, 0xe8d, 0xeaa, 0xecc, 0xe95, 0xeb9, 0xe84, 0xeb9, 0xec1, 0xea1, 0xe99,
0xead, 0xeb9, 0xe8a, 0xeb9, 0xec0, 0xead, 0xe8d, 0xead, 0xeb2, 0xea3, 0xeb9, 0xe9a,
0xeb2, 0xead, 0xeb0, 0xe8a, 0xeb8, 0xe99, 0xe8a, 0xeb4, 0xead, 0xead, 0xe87, 0xead,
0xeb2, 0xe97, 0xeb4, 0xec2, 0xe84, 0xe84, 0xeb2, 0xe99, 0xe9a, 0xeb2, 0xec0, 0xea2,
0xe8d, 0xe9a, 0xeb2, 0xec0, 0xeae, 0xe8d, 0x20, 0xec1, 0xe9a, 0xe99, 0xec0, 0xe94,
0xeb5, 0xea3, 0xeb2, 0xeaa, 0xe9a, 0xeb2, 0xec0, 0xe9a, 0xe94, 0xead, 0xeaa, 0xe9a,
0xeb5, 0xec0, 0xea5, 0xea1, 0xec0, 0xe9a, 0xea5, 0xeb5, 0xe8a, 0xe9a, 0xea5, 0xeb2,
0xe99, 0x2d, 0xe8a, 0xeb2, 0xe9a, 0xea5, 0xead, 0xe99, 0xe9a, 0xebb, 0xea7, 0x20,
0xea7, 0xeb4, 0xeaa, 0xe95, 0xeb2, 0xec2, 0xe9a, 0xec2, 0xe81, 0xe95, 0xeb2, 0xe9a,
0xead, 0xe8d, 0xe8a, 0xecc, 0xec1, 0xe84, 0xea1, 0xe9a, 0xea3, 0xeb4, 0xe94, 0x20,
0xec0, 0xe9a, 0xe81, 0xeb1, 0xe87, 0xe9b, 0xeb9, 0xea3, 0xeb1, 0xe87, 0xe88, 0xeb5,
0xec1, 0xe84, 0xe99, 0xe81, 0xeb8, 0xe99, 0xe84, 0xeb2, 0xea3, 0xeb2, 0xe81, 0xeb1,
0xeaa, 0xe84, 0xeb2, 0xec0, 0xea2, 0xe99, 0xec0, 0xe84, 0xec1, 0xea1, 0xe99, 0xe8a,
0xeb4, 0xe84, 0xeb2, 0xec2, 0xe81, 0xe8a, 0xeb4, 0xea7, 0xeb2, 0xea7, 0xeb2, 0xe8a,
0xeb5, 0xead, 0xeb9, 0xe94, 0xeb1, 0xe94, 0x20, 0xe84, 0xebb, 0xea7, 0xec0, 0xea3,
0xe8a, 0xe84, 0xead, 0xeaa, 0xe95, 0xeb2, 0xea3, 0xeb4, 0xe81, 0xeb2, 0xec0, 0xe84,
0xea3, 0xeaa, 0xe95, 0xeb1, 0xe99, 0xe81, 0xeb8, 0xea2, 0xeb2, 0xe9a, 0xeb2, 0xe81,
0xeb7, 0xea3, 0xeb2, 0xec0, 0xe8a, 0xebb, 0xeb2, 0xec0, 0xe94, 0xe99, 0xea1, 0xeb2,
0xe81, 0xec1, 0xe8a, 0xe99, 0xe94, 0xecd, 0xeaa, 0xeb1, 0xe99, 0xe94, 0xecd, 0xeaa,
0xeb1, 0xe99, 0x20, 0xe84, 0xea3, 0xeb5, 0xe81, 0xec0, 0xe94, 0xeb1, 0xe99, 0xec0,
0xea7, 0xeb5, 0xe94, 0xeb5, 0xe97, 0xea3, 0xead, 0xe8d, 0xec2, 0xe94, 0xea1, 0xeb4,
0xe99, 0xeb4, 0xe81, 0xeb2, 0xec0, 0xead, 0xe94, 0xea1, 0xead, 0xe99, 0xe95, 0xead,
0xe99, 0xec0, 0xead, 0xea3, 0xeb8, 0xec0, 0xe99, 0xe9b, 0xeb5, 0xec0, 0xead, 0xea7,
0x20, 0xe8a, 0xeb2, 0xea7, 0xeb2, 0xe94, 0xecd, 0xe9f, 0xead, 0xe94, 0x20, 0xec0,
0xe99, 0xea7, 0xeaa, 0xeb1, 0xe99, 0xe9f, 0xecd, 0xe95, 0xeb2, 0xec0, 0xea5, 0xe8a,
0xeb2, 0xec0, 0xe81, 0xea5, 0xe94, 0xec0, 0xe9a, 0xe81, 0xeb9, 0xeaa, 0xec0, 0xe9a,
0xec1, 0xe81, 0xea3, 0xe99, 0x20, 0xec0, 0xe97, 0xeb5, 0xe81, 0xec0, 0xe81, 0xea3,
0xe99, 0xeb2, 0xe94, 0xeb2, 0xe81, 0xeb2, 0xea7, 0xec0, 0xe94, 0xea5, 0xeb9, 0xe9a,
0xe81, 0xebb, 0xea7, 0xec0, 0xe95, 0xea1, 0xeb2, 0xea5, 0xeb2, 0xe81, 0xebb, 0xea7,
0xea2, 0xeb2, 0xe81, 0xeb4, 0xea5, 0xe81, 0xeb9, 0xea2, 0xeb2, 0xe99, 0xeb2, 0xeae,
0xeb2, 0xea5, 0xeb4, 0xec1, 0xe9f, 0xeb1, 0xe81, 0xeae, 0xeb2, 0xea7, 0xeb2, 0xe99,
0xeb2, 0xec0, 0xead, 0xec2, 0xea1, 0xe8a, 0xeb4, 0xec2, 0xea2, 0xead, 0xeb4, 0xe99,
0xe94, 0xeb5, 0xec0, 0xead, 0xe99, 0xeb2, 0xec2, 0xe9e, 0xea5, 0xeb4, 0xeaa, 0xe99,
0xecb, 0xead, 0xe81, 0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99,
0xeb2, 0xea1, 0xeb2, 0xec0, 0xea3, 0xe99, 0xec2, 0xe81, 0x2c, 0x20, 0xead, 0xeb4,
0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xe9b, 0xeb5, 0xec0, 0xe95, 0xeb5, 0xeaa,
0xec0, 0xe9b, 0xeb5, 0xe81, 0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d,
0xe99, 0xeb2, 0xec0, 0xe97, 0xea7, 0x20, 0xe8a, 0xeb4, 0xe95, 0xeb5, 0x2c, 0x20,
0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xea7, 0xeb5, 0xec0, 0xea7,
0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xea7, 0xeb4,
0xe99, 0xec0, 0xe8a, 0xe99, 0xec0, 0xe99, 0xeaa, 0x2c, 0x20, 0xead, 0xeb4, 0xe99,
0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xea7, 0xeb4, 0xe99, 0xeb2, 0xec1, 0xea1, 0xeb1,
0xe81, 0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xead,
0xeb5, 0xe99, 0xeb9, 0xea7, 0xeb4, 0xe81, 0xead, 0xeb5, 0xe81, 0xebb, 0xea7, 0xea5,
0xeb4, 0xe94, 0xe88, 0xeb2, 0xec4, 0xea1, 0xe81, 0xeb2, 0xe88, 0xeb9, 0xec2, 0xe99,
0xeab, 0xea5, 0xeb8, 0xe8d, 0xea7, 0xeb4, 0xea7, 0xea1, 0xead, 0xe99, 0xe95, 0xeb5,
0xec0, 0xe8a, 0xea7, 0xec2, 0xea5, 0x2c, 0x20, 0xec0, 0xe84, 0xe99, 0xe97, 0xeb1,
0xe81, 0xe81, 0xeb5, 0xe84, 0xea3, 0xeb2, 0xec0, 0xea5, 0xe99, 0xe94, 0xeb4, 0xe81,
0xea5, 0xeb2, 0xe9b, 0xeb2, 0xeaa, 0xea5, 0xeb5, 0xea1, 0xeb2, 0xea5, 0xead, 0xeaa,
0xec1, 0xead, 0xe99, 0xec0, 0xe88, 0xea5, 0xeb5, 0xeaa, 0xec2, 0xea5, 0xec0, 0xea7,
0xeb5, 0x20, 0xe9e, 0xea3, 0xeb4, 0xe99, 0xe8a, 0xecc, 0x20, 0xe84, 0xea7, 0xe94,
0xec0, 0xe95, 0xeb5, 0xea1, 0xeb2, 0xec0, 0xe8a, 0xec2, 0xead, 0xea1, 0xeb2, 0xe99,
0xeb2, 0xe81, 0xebb, 0xea7, 0xea1, 0xeb2, 0xec0, 0xe99, 0xebb, 0xeb2, 0xeaa, 0xecc,
0xea1, 0xeb2, 0xea3, 0xeb5, 0xec2, 0xe81, 0xe94, 0xea1, 0xeb2, 0xe97, 0xeb5, 0xe99,
0xeb4, 0xe81, 0xea1, 0xeb2, 0xe95, 0xeb2, 0xec2, 0xea1, 0xea3, 0xead, 0xeaa, 0xea1,
0xeb2, 0xe8a, 0xeb2, 0xe94, 0xea5, 0xeb2, 0xe99, 0xec0, 0xea1, 0xec2, 0xe99, 0xea1,
0xeb4, 0xe99, 0xeb5, 0xec0, 0xea1, 0xeb5, 0xea3, 0xeb4, 0xe94, 0xeb2, 0xec0, 0xea1,
0xeb1, 0xe94, 0xea5, 0xeb2, 0xe81, 0xeb1, 0xe94, 0xea5, 0xeb2, 0xec0, 0xea1, 0xeb1,
0xe81, 0xe8a, 0xeb4, 0xec2, 0xe81, 0x20, 0xe8a, 0xeb4, 0xe95, 0xeb5, 0xea1, 0xeb4,
0xe81, 0xebb, 0xea7, 0xea5, 0xead, 0xe99, 0xea1, 0xead, 0xe99, 0xe95, 0xeb1, 0xe99,
0xea1, 0xead, 0xe99, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea3, 0xe8d, 0xea1, 0xead, 0xe99,
0xec0, 0xe95, 0xea7, 0xeb4, 0xec0, 0xe94, 0xec2, 0xead, 0xea1, 0xead, 0xe99, 0xec0,
0xe8a, 0xeb5, 0xea3, 0xeb1, 0xe94, 0xec1, 0xe99, 0xeaa, 0xe8a, 0xecd, 0xe99, 0xeb4,
0xea7, 0xea2, 0xead, 0xe81, 0xe99, 0xead, 0xea1, 0xe99, 0xecd, 0xea3, 0xead, 0xe99,
0xeae, 0xeb2, 0xec0, 0xe9a, 0xeb5, 0xea5, 0xeb2, 0x2c, 0x20, 0xe99, 0xead, 0xe94,
0x20, 0xe94, 0xeb2, 0xec2, 0xe81, 0xe95, 0xeb2, 0xec0, 0xe8a, 0xeb1, 0xe99, 0xec0,
0xe88, 0xeb5, 0x2c, 0x20, 0xe99, 0xead, 0xe94, 0x20, 0xe94, 0xeb2, 0xec2, 0xe81,
0xe95, 0xeb2, 0xe99, 0xeb4, 0xea7, 0x20, 0xe8a, 0xeb2, 0xec0, 0xea5, 0xea1, 0x2c,
0x20, 0xe99, 0xead, 0xe94, 0x20, 0xe94, 0xeb2, 0xec2, 0xe81, 0xe95, 0xeb2, 0xe99,
0xeb9, 0xe81, 0xec2, 0xead, 0xe88, 0xeb4, 0xe99, 0xeb2, 0xe81, 0xeb2, 0xe9e, 0xeb2,
0xe99, 0xeb2, 0xea1, 0xeb2, 0xe9e, 0xeb2, 0xea3, 0xeb2, 0xea1, 0xeb2, 0xea3, 0xeb4,
0xec2, 0xe9a, 0xe9f, 0xeb4, 0xe99, 0xeb4, 0xe81, 0xe9b, 0xecd, 0xec2, 0xe95, 0xec1,
0xe9b, 0xea3, 0xe87, 0xe9e, 0xead, 0xe94, 0xead, 0xead, 0xe9a, 0xeaa, 0xeb0, 0xec0,
0xe9b, 0xe99, 0xe9b, 0xecd, 0xe95, 0xeb9, 0x20, 0xec0, 0xea7, 0xea7, 0xea2, 0xeb9,
0xec0, 0xe9b, 0xeb5, 0xec2, 0xe97, 0xea3, 0xeb4, 0xec2, 0xe81, 0xe9e, 0xeb8, 0xe99,
0xe97, 0xeb2, 0x20, 0xead, 0xeb0, 0xea3, 0xeb5, 0xe99, 0xeb2, 0xeaa, 0xecc, 0xec1,
0xea3, 0xe99, 0xe81, 0xeb4, 0xe99, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xea5, 0xeb1,
0xe94, 0xec0, 0xea3, 0xe8a, 0xeb4, 0xec0, 0xe9f, 0xec0, 0xea3, 0xe88, 0xeb4, 0xe99,
0xeb2, 0xec0, 0xea3, 0xec2, 0xe8a, 0xea5, 0xeb8, 0xe94, 0xea3, 0xeb4, 0xec2, 0xead,
0x20, 0xe9a, 0xea3, 0xeb1, 0xe99, 0xec2, 0xe81, 0xe8a, 0xeb1, 0xe99, 0xe95, 0xeb2,
0xec0, 0xea3, 0xea1, 0xe8a, 0xeb1, 0xe99, 0xe95, 0xeb5, 0xead, 0xeb2, 0xec2, 0xe81,
0xe8a, 0xeb2, 0xe99, 0xec2, 0xe95, 0xec2, 0xe94, 0xea1, 0xeb4, 0xe99, 0xec2, 0xe81,
0xec0, 0xe8a, 0xebb, 0xeb2, 0x20, 0xec0, 0xe9b, 0xebb, 0xeb2, 0xec2, 0xea5, 0xead,
0xeb4, 0xec2, 0xe95, 0xe84, 0xecd, 0xe97, 0xebb, 0xea7, 0xea1, 0xeb4, 0xe94, 0xe8a,
0xeb4, 0xe94, 0xe81, 0xeb2, 0xec0, 0xe8a, 0xe99, 0xe9a, 0xeb2, 0xec0, 0xe97, 0xec0,
0xea5, 0xea1, 0xeb5, 0xec0, 0xe8a, 0xe99, 0x20, 0xe88, 0xead, 0xe99, 0xec0, 0xe8a,
0xe99, 0x20, 0xe84, 0xeb4, 0xe94, 0xeaa, 0xecc, 0xec0, 0xe8a, 0xe99, 0x20, 0xea5,
0xeb9, 0xec0, 0xe8a, 0xe8d, 0xec0, 0xe8a, 0xe99, 0x20, 0xec2, 0xe97, 0xea1, 0xeb1,
0xeaa, 0xec0, 0xe8a, 0xe99, 0x20, 0xea7, 0xeb4, 0xe99, 0xec0, 0xe8a, 0xe99, 0xeaa,
0xeb0, 0xea7, 0xeb4, 0xe9f, 0x20, 0xec0, 0xe84, 0xeb5, 0xec0, 0xea3, 0xe99, 0xec0,
0xe95, 0xe81, 0xeb9, 0xe8a, 0xeb5, 0xe81, 0xeb2, 0xe99, 0xe9b, 0xeb2, 0xe97, 0xeb9,
0xec0, 0xea5, 0xe97, 0xeb5, 0xe88, 0xebb, 0xea7, 0xe99, 0xeb2, 0xec2, 0xe97, 0xea3,
0xead, 0xe99, 0xec2, 0xe95, 0xe97, 0xecd, 0xec2, 0xe95, 0xea5, 0xeb2, 0xec1, 0xea7,
0xe99, 0xe84, 0xeb9, 0xec0, 0xea7, 0xeb5, 0xec4, 0xea7, 0xe97, 0xecc, 0xeae, 0xead,
0xeaa, 0xea7, 0xeb4, 0xe99, 0xe99, 0xeb5, 0xec0, 0xe9e, 0xe81, 0xea2, 0xeb2, 0xe84,
0xeb9, 0xe95, 0xeb1, 0xe94, 0xec0, 0xe84, 0xe8a, 0xeb5, 0xe94, 0xeb2, 0xea7, 0xeb5,
0xeaa, 0xe94, 0xeb9, 0xea1, 0xead, 0xe99, 0xe94, 0xeb5, 0xe8d, 0xeb9, 0xea7, 0xeb5,
0xea7, 0xeaa, 0xecc, 0xec1, 0xea1, 0xeb1, 0xe81, 0xe84, 0xebb, 0xea7, 0xea3, 0xeb5,
0xec0, 0xea1, 0xebb, 0xeb2, 0xe8a, 0xeb1, 0xe99, 0xec1, 0xea1, 0xeb1, 0xe81, 0xea1,
0xebb, 0xea7, 0xec2, 0xe94, 0xe9e, 0xeb2, 0xea1, 0xec0, 0xea1, 0xeb5, 0xec2, 0xea3,
0xec0, 0xe97, 0xea3, 0xeb2, 0xec0, 0xe8a, 0xe8d, 0xea7, 0xeb2, 0xec2, 0xe97, 0xea3,
0xea5, 0xecc, 0xea7, 0xecd, 0xeaa, 0xeb0, 0xe95, 0xead, 0xe81, 0xea5, 0xead, 0xe87,
0xec0, 0xea2, 0xe8d, 0xe9a, 0xebd, 0xe99, 0xec0, 0xead, 0xec0, 0xe94, 0xeb1, 0xe99,
0xead, 0xecd, 0xea1, 0xeb2, 0xe95, 0xeb5, 0xead, 0xeb3, 0xea1, 0xeb2, 0xe99, 0xead,
0xeb2, 0xe99, 0xeb2, 0xe94, 0xeb5, 0xead, 0xeb1, 0xe94, 0xe95, 0xeb2, 0xead, 0xeb9,
0xead, 0xeb1, 0xe94, 0xec2, 0xe97, 0xe9a, 0xeb5, 0xead, 0xeb2, 0xe94, 0xe8a, 0xecc,
0xe81, 0xeb2, 0xe9a, 0xeb1, 0xe94, 0xead, 0xeb1, 0xe94, 0xe97, 0xeb5, 0xec0, 0xea3,
0xebb, 0xeb2, 0xec1, 0xe9a, 0xe81, 0xec1, 0xe94, 0xe94, 0xe9a, 0xeb2, 0xec0, 0xea3,
0xe99, 0xe9a, 0xeb2, 0xe81, 0xeb9, 0xe9a, 0xeb2, 0xe87, 0xe81, 0xead, 0xe81, 0xe9a,
0xeb2, 0xe99, 0xebb, 0xea7, 0xec0, 0xe9a, 0xea3, 0xeb8, 0xe94, 0xe9a, 0xeb4, 0xe94,
0xe8a, 0xecc, 0xec0, 0xe84, 0xe81, 0xe9a, 0xea3, 0xeb9, 0xec4, 0xe99, 0xe8a, 0xeb5,
0xe95, 0xec8, 0xeb2, 0xec2, 0xe84, 0xea5, 0xeb3, 0xec2, 0xe9a, 0xe94, 0xeb2, 0xea1,
0xeb2, 0xeaa, 0xe84, 0xeb1, 0xeaa, 0xe94, 0xeb2, 0xeab, 0xecc, 0xe81, 0xeb2, 0xe94,
0xeb5, 0xea5, 0xeb5, 0xe94, 0xeb9, 0xec4, 0xe9a, 0xe94, 0xeb9, 0xe8a, 0xeb2, 0xe99,
0xec0, 0xe9a, 0xe9f, 0xeb2, 0xea1, 0xeb2, 0xe81, 0xeb1, 0xeaa, 0xe97, 0xeb2, 0xe81,
0xeb2, 0xe8a, 0xeb2, 0xec0, 0xeae, 0xe9a, 0xea3, 0xead, 0xe99, 0xec2, 0xeae, 0xe88,
0xeb4, 0xea1, 0xeb4, 0xe99, 0xeae, 0xec8, 0xead, 0xe87, 0xe81, 0xebb, 0xe87, 0xeae,
0xead, 0xe9a, 0xead, 0xeb5, 0xe84, 0xeb8, 0xeaa, 0xe84, 0xecc, 0xe88, 0xeb2, 0xe81,
0xeb2, 0xe95, 0xeb2, 0xe88, 0xeb2, 0xea2, 0xeb2, 0xe9b, 0xeb9, 0xea3, 0xeb0, 0xec0,
0xea2, 0xea3, 0xeb9, 0xe8a, 0xeb2, 0xec0, 0xea5, 0xea1, 0xe84, 0xeb2, 0xe9a, 0xeb9,
0xe84, 0xeb2, 0xea1, 0xe8a, 0xeb2, 0xe94, 0xe81, 0xeb2, 0xe81, 0xeb2, 0xea3, 0xeb2,
0xe88, 0xeb5, 0xe84, 0xeb1, 0xe94, 0xea1, 0xeb1, 0xe99, 0xe94, 0xeb9, 0xec1, 0xe84,
0xe99, 0xe94, 0xeb5, 0xe81, 0xeb2, 0xec2, 0xe81, 0xea5, 0xe81, 0xeb2, 0xe95, 0xeb2,
0xe84, 0xea3, 0xeb1, 0xeaa, 0xec2, 0xe99, 0xea2, 0xeb2, 0xeaa, 0xe84, 0xecc, 0xe81,
0xebb, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb3, 0xec0, 0xe9b, 0xeb5, 0xe81, 0xeb9, 0xe8a,
0xeb5, 0xe87, 0xe84, 0xeb9, 0xec0, 0xea7, 0xe94, 0xea1, 0xeb2, 0xec0, 0xe81, 0xebb,
0xeb2, 0xea1, 0xeb2, 0xe81, 0xeb2, 0xec1, 0xe94, 0xe99, 0xea1, 0xeb2, 0xe81, 0xeb2,
0xeaa, 0xe8a, 0xeb2, 0xea1, 0xeb2, 0xe99, 0xeb5, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xeaa,
0xe84, 0xeb1, 0xe94, 0xe99, 0xeb4, 0xec2, 0xe84, 0xec0, 0xe8a, 0xe8d, 0xec2, 0xe99,
0xec2, 0xea7, 0xe84, 0xeb8, 0xeaa, 0xec0, 0xe99, 0xeb1, 0xe94, 0xec2, 0xe99, 0xec2,
0xea7, 0xe8a, 0xeb5, 0xe9a, 0xeb5, 0xeaa, 0xe84, 0xecc, 0xead, 0xead, 0xea1, 0xeaa,
0xe84, 0xecc, 0xead, 0xecd, 0xea3, 0xecd, 0xe9e, 0xeb0, 0xe99, 0xebb, 0xea1, 0xec0,
0xe9b, 0xeb1, 0xe99, 0xe9e, 0xead, 0xe99, 0xec0, 0xe97, 0xe8d, 0xe99, 0xeb1, 0xe81,
0xe9b, 0xebd, 0xe87, 0xea2, 0xeb2, 0xe87, 0xe81, 0xeb2, 0xe95, 0xeb2, 0xea3, 0xecc,
0xe84, 0xead, 0xeaa, 0xe95, 0xeb2, 0xec0, 0xe99, 0xec4, 0xe84, 0xe8a, 0xeb5, 0xea5,
0xecd, 0xe94, 0xeb2, 0xea3, 0xeb5, 0xea2, 0xeb2, 0xe94, 0xe8a, 0xeb2, 0xe84, 0xeb2,
0xea5, 0xeb4, 0xe99, 0xe8a, 0xeb2, 0xea1, 0xeb2, 0xec1, 0xe84, 0xe99, 0xec2, 0xe8a,
0xea5, 0xecc, 0xe8a, 0xec8, 0xebd, 0xe87, 0xec4, 0xeae, 0xec9, 0xeaa, 0xeb4, 0xe87,
0xe81, 0xeb0, 0xec2, 0xe9b, 0xeaa, 0xea3, 0xeb4, 0xea5, 0xec0, 0xe99, 0xeb1, 0xe81,
0xec2, 0xe81, 0xea5, 0xeb5, 0xe94, 0xec4, 0xe97, 0xec0, 0xe9b, 0xe97, 0xeb2, 0xe94,
0xeaa, 0xecc, 0xec0, 0xe84, 0xe99, 0xe97, 0xeb4, 0xe9a, 0xeb5, 0xea5, 0xeb5, 0xe8a,
0xeb5, 0xec0, 0xe95, 0xea3, 0xeb2, 0xe99, 0xe97, 0xeb4, 0xea1, 0xe9e, 0xeb9, 0xec2,
0xe95, 0xe81, 0xebd, 0xea7, 0xe97, 0xead, 0xea1, 0xeaa, 0xe81, 0xecc, 0xead, 0xeb9,
0xea5, 0xeb2, 0xe99, 0xe9a, 0xeb2, 0xe95, 0xeb2, 0xea3, 0xecc, 0xead, 0xeb9, 0xea3,
0xeb8, 0xea1, 0xe8a, 0xeb5, 0xead, 0xeb8, 0xeaa, 0x20, 0xec0, 0xe99, 0xea3, 0xeb2,
0xea7, 0xebd, 0xe87, 0xe88, 0xeb1, 0xe99, 0xea7, 0xeb5, 0xea5, 0xeb2, 0xe94, 0xeb4,
0xec2, 0xea7, 0xeaa, 0xe95, 0xead, 0xe81, 0xea2, 0xeb2, 0xe84, 0xeb8, 0xe97, 0xe8a,
0xecc, 0xea2, 0xeb2, 0xe87, 0xe81, 0xead, 0xe99, 0xec0, 0xea2, 0xe84, 0xeb2, 0xec0,
0xe95, 0xeb5, 0xea3, 0xeb4, 0xe99, 0xec0, 0xe9a, 0xeb5, 0xe81, 0xec0, 0xe8d, 0xec0,
0xea3, 0xea7, 0xeb2, 0xe99, 0xead, 0xeb2, 0x200b, 0xec2, 0xe8a, 0x200b, 0xec0, 0xea3,
0x200b, 0xeaa, 0xec0, 0xe9a, 0xeb5, 0xea1, 0xeb4, 0xea7, 0xe94, 0xeb2, 0xe84, 0xeb2,
0xe99, 0xeb2, 0xea3, 0xeb5, 0xec0, 0xe84, 0xe9a, 0xec0, 0xea7, 0xeb5, 0xe94, 0xec1,
0xe9f, 0xec2, 0xea3, 0xea1, 0xeb2, 0xec0, 0xe94, 0xea3, 0xeb2, 0xec0, 0xea3, 0xe81,
0xea2, 0xeb2, 0xea7, 0xeb4, 0xe81, 0xec0, 0xe8a, 0xebb, 0xeb2, 0xe88, 0xecd, 0xec0,
0xe88, 0xe8d, 0xec0, 0xe8a, 0xe99, 0x20, 0xec0, 0xeae, 0xec0, 0xea5, 0xe99, 0xeb2,
0xeaa, 0xeb0, 0xec1, 0xe95, 0xe99, 0xec0, 0xea5, 0xec0, 0xead, 0xe94, 0xeb5, 0xec1,
0xea5, 0xe94, 0xe9a, 0xea3, 0xeb4, 0xeaa, 0xec0, 0xe9a, 0xe99, 0xec2, 0xe9a, 0xea3,
0xe81, 0xec0, 0xe84, 0xe99, 0x20, 0xeae, 0xeb4, 0xea7, 0xe94, 0xeb2, 0xea7, 0xeb4,
0xe99, 0xea2, 0xeb9, 0xe84, 0xea5, 0xeb2, 0xec2, 0xeae, 0xe9a, 0xeb2, 0xe94, 0xea5,
0xeb4, 0xe99, 0xe94, 0xeb5, 0xec1, 0xea1, 0xe99, 0xec2, 0xea5, 0xe94, 0xec2, 0xeae,
0xea7, 0xeb5, 0xec0, 0xea1, 0xea7, 0xec0, 0xe9a, 0xeb4, 0xe99, 0xec0, 0xe9e, 0xeb5,
0xe94, 0xe8a, 0xeb4, 0xe94, 0xe99, 0xeb5, 0xead, 0xeb2, 0xea1, 0xeaa, 0xec0, 0xe95,
0xeb5, 0xe94, 0xeb3, 0xead, 0xeb4, 0xe99, 0xe94, 0xecd, 0xea3, 0xeb2, 0xead, 0xeb2,
0xeaa, 0xe95, 0xea3, 0xeb2, 0xe84, 0xeb2, 0xe99, 0xec0, 0xead, 0xec0, 0xe97, 0xe99,
0xeaa, 0xecc, 0xec0, 0xe9a, 0xea7, 0xec0, 0xe81, 0xea3, 0xe94, 0xec0, 0xe9a, 0xeb5,
0xea5, 0xeb4, 0xe99, 0xe9a, 0xea3, 0xeb2, 0xe97, 0xeb4, 0xeaa, 0xea5, 0xeb2, 0xea7,
0xeb2, 0xe9a, 0xea3, 0xeb1, 0xeaa, 0xec0, 0xe8a, 0xea7, 0xe9a, 0xeb9, 0xe84, 0xeb2,
0xec0, 0xea3, 0xeaa, 0xe95, 0xecc, 0xe9a, 0xeb9, 0xe94, 0xeb2, 0xec0, 0xe9b, 0xeaa,
0xe95, 0xecc, 0xe9a, 0xeb1, 0xe94, 0xe8a, 0xeb4, 0xe99, 0xec0, 0xe81, 0xe99, 0xe84,
0xeb5, 0xe8a, 0xeb5, 0xec0, 0xe99, 0xebb, 0xeb2, 0xec2, 0xe84, 0xec0, 0xe9b, 0xe99,
0xec0, 0xeae, 0xec0, 0xe81, 0xe99, 0xe94, 0xeb1, 0xe9a, 0xe9a, 0xea5, 0xeb4, 0xe99,
0xe81, 0xeb4, 0xe9a, 0xea3, 0xead, 0xea5, 0xe97, 0xeb2, 0xec0, 0xe81, 0xeb5, 0xe99,
0xe8a, 0xeb5, 0xec0, 0xeae, 0xea7, 0xe8a, 0xeb4, 0xe99, 0xe81, 0xeb4, 0xec0, 0xe81,
0xeb2, 0xeb0, 0xec1, 0xea1, 0xe99, 0xead, 0xeb4, 0xeaa, 0xe95, 0xeb1, 0xe99, 0xe9a,
0xeb9, 0xea5, 0xecc, 0xec0, 0xe88, 0xeb5, 0xe8a, 0xeb5, 0xe84, 0xeb2, 0xea5, 0xeb4,
0xe99, 0xeb4, 0xe99, 0xe81, 0xea3, 0xeb2, 0xe94, 0xe81, 0xeb4, 0xec2, 0xea3, 0xe9a,
0xe82, 0xebd, 0xe9a, 0xea5, 0xeb4, 0xeaa, 0xe9a, 0xead, 0xe99, 0xea5, 0xeb9, 0xe9a,
0xea5, 0xeb5, 0xe8d, 0xeb2, 0xe99, 0xeb2, 0xea5, 0xead, 0xe99, 0xe94, 0xead, 0xe99,
0xea5, 0xeb1, 0xe81, 0xec0, 0xe8a, 0xeb1, 0xea1, 0xec0, 0xe9a, 0xeb5, 0xe81, 0xea1,
0xeb2, 0xe94, 0xea3, 0xeb4, 0xe94, 0xea1, 0xead, 0xea5, 0xe95, 0xeb2, 0xea1, 0xeb2,
0xea3, 0xeb5, 0xec1, 0xeae, 0xea1, 0xe99, 0xecc, 0xea1, 0xeb4, 0xe99, 0xeaa, 0xe81,
0xecc, 0xec2, 0xea1, 0xe99, 0xeb2, 0xec2, 0xe84, 0xea1, 0xead, 0xeaa, 0xec2, 0xe84,
0xead, 0xead, 0xeaa, 0xec2, 0xea5, 0xe9b, 0xeb2, 0xea3, 0xeb5, 0xeaa, 0xe9e, 0xead,
0xe94, 0xe81, 0xecd, 0xea3, 0xeb5, 0xe81, 0xeb2, 0xe9b, 0xea3, 0xeb2, 0xe81, 0xe8a,
0xeb2, 0xea1, 0xeb2, 0xea3, 0xeb2, 0xe8a, 0xeb2, 0xe99, 0xea1, 0xeb2, 0xea3, 0xeb5,
0xec2, 0xe99, 0xe8a, 0xeb2, 0xea3, 0xeb2, 0xec0, 0xea2, 0xec2, 0xea7, 0xe8a, 0xeb2,
0xea3, 0xeb2, 0xe97, 0xead, 0xe9a, 0xe8a, 0xeb5, 0xea1, 0xec0, 0xe9f, 0xec2, 0xea5,
0xec2, 0xe9b, 0xeaa, 0xeb0, 0xec2, 0xe81, 0xec0, 0xe9b, 0xe8d, 0xec2, 0xe8a, 0xec0,
0xe9f, 0xe8d, 0xeaa, 0xeb0, 0xe95, 0xead, 0xe81, 0xec2, 0xeae, 0xea1, 0xe97, 0xeb2,
0xea5, 0xea5, 0xeb4, 0xe99, 0xe99, 0xecc, 0xe97, 0xeb4, 0xea3, 0xeb2, 0xe99, 0xead,
0xeb9, 0xea5, 0xeb5, 0xead, 0xeb2, 0xe99, 0xead, 0xe9a, 0xeaa, 0xe84, 0xecc, 0xea7,
0xeb2, 0xe94, 0xeb2, 0xe8a, 0xea7, 0xeb2, 0xe95, 0xeb4, 0xe81, 0xeb1, 0xe99, 0xea7,
0xebd, 0xe99, 0xe99, 0xeb2, 0xea7, 0xeb4, 0xea5, 0xe99, 0xeb4, 0xead, 0xeb8, 0xeaa,
0xea7, 0xead, 0xea5, 0xec2, 0xe81, 0xe81, 0xea3, 0xeb2, 0xe94, 0xea7, 0xecd, 0xe8a,
0xecd, 0xe8a, 0xeb2, 0xec0, 0xe81, 0xea3, 0xe9a, 0xe8a, 0xeb9, 0xea3, 0xeb4, 0xe81,
0xead, 0xeb1, 0xe99, 0xe95, 0xeb2, 0xe99, 0xeb2, 0xe99, 0xeb2, 0xea3, 0xeb4, 0xec2,
0xea7, 0xe8a, 0xeb2, 0xec2, 0xe81, 0xeaa, 0xe84, 0xea3, 0xeb4, 0xe94, 0xeaa, 0xeb0,
0xea1, 0xeb2, 0xe94, 0xec2, 0xe84, 0xec2, 0xe84, 0xeaa, 0xec2, 0xe84, 0xec2, 0xea1,
0xec2, 0xea3, 0xec1, 0xe81, 0xec0, 0xe81, 0xeb5, 0xec0, 0xea5, 0xe99, 0xea1, 0xeb2,
0xec0, 0xeae, 0xea1, 0xeb1, 0xea5, 0xe94, 0xeb5, 0xe9f, 0xec0, 0xea1, 0xebb, 0xeb2,
0xea3, 0xeb4, 0xe97, 0xebd, 0xeaa, 0xea1, 0xeb2, 0xea2, 0xeb1, 0xead, 0xe94, 0xec0,
0xe95, 0xec0, 0xea3, 0xead, 0xeb9, 0xe99, 0xeb4, 0xea2, 0xebb, 0xe87, 0xec0, 0xead,
0xe9b, 0xeb5, 0xead, 0xeb2, 0xead, 0xeb1, 0xe81, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7,
0xea5, 0xeb2, 0xe95, 0xeb2, 0xea1, 0xec0, 0xe82, 0xe94, 0xe9a, 0xeb9, 0xe99, 0xe81,
0xebd, 0xe99, 0xea7, 0xeb4, 0xea7, 0xec1, 0xe8a, 0xec1, 0xe97, 0xea1, 0xe88, 0xeb1,
0xe81, 0xec0, 0xead, 0xeaa, 0xec0, 0xe95, 0xeb5, 0xec0, 0xead, 0xe9f, 0xeb2, 0xec0,
0xe95, 0xe9f, 0xeb2, 0xe81, 0xeb2, 0xea7, 0xec2, 0xe9f, 0xe9f, 0xeb9, 0xe88, 0xeb4,
0xe9f, 0xeb9, 0xe99, 0xeb0, 0xe9f, 0xeb9, 0xe95, 0xeb4, 0xe81, 0xeb2, 0xea5, 0xeb2,
0xe9b, 0xeb2, 0xe81, 0xead, 0xeaa, 0xec1, 0xe81, 0xea1, 0xec0, 0xe9a, 0xe8d, 0xe81,
0xebb, 0xea7, 0xe94, 0xeb1, 0xea5, 0xe84, 0xeb0, 0xec1, 0xe99, 0xea5, 0xe81, 0xea7,
0xea1, 0xec2, 0xeae, 0xec2, 0xe99, 0xea5, 0xeb9, 0xea5, 0xeb9, 0xec0, 0xead, 0xeb1,
0xe99, 0xec0, 0xe94, 0xeb5, 0xec0, 0xe9a, 0xeb5, 0xea5, 0xeb5, 0xe84, 0xeb5, 0xea3,
0xeb4, 0xe97, 0xeb5, 0xea1, 0xeb2, 0xe95, 0xeb5, 0xe84, 0xecd, 0xeaa, 0xec1, 0xea3,
0xe84, 0xea7, 0xeb2, 0xe88, 0xeb2, 0xec0, 0xea5, 0xe99, 0xea1, 0xeb2, 0xe88, 0xeb9,
0xec2, 0xea3, 0xea1, 0xeb2, 0xe84, 0xeb4, 0xea7, 0xe8a, 0xeb2, 0xeaa, 0xea1, 0xeb4,
0xe94, 0xec0, 0xea7, 0xe99, 0xeb2, 0xead, 0xeb9, 0xea3, 0xeb9, 0xe99, 0xeb5, 0xead,
0xeb9, 0xec0, 0xead, 0xe99, 0xecd, 0xe9f, 0xead, 0xea5, 0xecc, 0xe81, 0xe99, 0xeb9,
0xec0, 0xea1, 0xead, 0xeb2, 0xe9b, 0xeb2, 0xec2, 0xe81, 0xe9b, 0xeb2, 0xec2, 0xe81,
0xe9b, 0xeb2, 0xec0, 0xea5, 0xebb, 0xeb2, 0xe9e, 0xeb4, 0xe94, 0xec1, 0xe84, 0xe99,
0xe9b, 0xebb, 0xe99, 0xec0, 0xe9b, 0xeb5, 0xe8d, 0xe9e, 0xead, 0xe94, 0x20, 0xea1,
0xecd, 0xec0, 0xea3, 0xeaa, 0xe9a, 0xeb5, 0xea3, 0xeb2, 0xec2, 0xea3, 0xe95, 0xead,
0xe87, 0xe81, 0xeb2, 0xec4, 0xe8a, 0xe9b, 0xeb2, 0xe99, 0xe97, 0xeb2, 0xeae, 0xeb5,
0xe95, 0xeb4, 0xe95, 0xeb2, 0xea3, 0xeb2, 0xea7, 0xeb2, 0xe95, 0xead, 0xe87, 0xe81,
0xeb2, 0xe95, 0xeb2, 0xe9b, 0xeb9, 0xec0, 0xea7, 0xe81, 0xea7, 0xeb2, 0xea5, 0xeb4,
0xe94, 0x41, 0x62, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x61, 0x41, 0x64, 0x69,
0x73, 0x61, 0x62, 0x65, 0x62, 0x61, 0x41, 0x6c, 0x17e, 0x12b, 0x72, 0x61,
0x42, 0x61, 0x6e, 0x64, 0x17e, 0x75, 0x6c, 0x61, 0x42, 0x69, 0x73, 0x61,
0x76, 0x61, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x72, 0x61, 0x42,
0x72, 0x61, 0x7a, 0x61, 0x76, 0x69, 0x6c, 0x61, 0x42, 0x75, 0x17e, 0x75,
0x6d, 0x62, 0x75, 0x72, 0x61, 0x4b, 0x61, 0x69, 0x72, 0x61, 0x44, 0x61,
0x6b, 0x61, 0x72, 0x61, 0x44, 0x101, 0x72, 0x65, 0x73, 0x61, 0x6c, 0x101,
0x6d, 0x61, 0x44, 0x17e, 0x69, 0x62, 0x75, 0x74, 0x69, 0x6a, 0x61, 0x41,
0x6a, 0x16b, 0x6e, 0x61, 0x46, 0x72, 0x12b, 0x74, 0x61, 0x75, 0x6e, 0x61,
0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67,
0x61, 0x44, 0x17e, 0x16b, 0x62, 0x61, 0x48, 0x61, 0x72, 0x74, 0x16b, 0x6d,
0x61, 0x4b, 0x69, 0x6e, 0x161, 0x61, 0x73, 0x61, 0x4c, 0x61, 0x67, 0x6f,
0x73, 0x61, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x76, 0x69, 0x6c, 0x61, 0x4c,
0x75, 0x62, 0x75, 0x6d, 0x62, 0x61, 0x161, 0x69, 0x4d, 0x61, 0x70, 0x75,
0x74, 0x75, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x12b, 0x161, 0x6f, 0x4d, 0x6f,
0x6e, 0x72, 0x6f, 0x76, 0x69, 0x6a, 0x61, 0x4e, 0x69, 0x61, 0x6d, 0x65,
0x6a, 0x61, 0x4e, 0x75, 0x61, 0x6b, 0x161, 0x6f, 0x74, 0x61, 0x54, 0x72,
0x69, 0x70, 0x6f, 0x6c, 0x65, 0x54, 0x75, 0x6e, 0x69, 0x73, 0x61, 0x56,
0x69, 0x6e, 0x64, 0x68, 0x75, 0x6b, 0x61, 0x41, 0x64, 0x61, 0x6b, 0x61,
0x41, 0x6e, 0x6b, 0x75, 0x72, 0x101, 0x17e, 0x61, 0x41, 0x6e, 0x67, 0x69,
0x6c, 0x6a, 0x61, 0x41, 0x72, 0x61, 0x67, 0x76, 0x61, 0x69, 0x6e, 0x61,
0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x61, 0x69, 0x72, 0x65, 0x73, 0x61,
0x4b, 0x6f, 0x72, 0x64, 0x6f, 0x76, 0x61, 0x48, 0x75, 0x68, 0x75, 0x6a,
0x61, 0x4c, 0x61, 0x72, 0x6a, 0x6f, 0x68, 0x61, 0x52, 0x69, 0x6f, 0x67,
0x61, 0x6c, 0x6a, 0x65, 0x67, 0x6f, 0x73, 0x61, 0x53, 0x61, 0x6e, 0x68,
0x75, 0x61, 0x6e, 0x61, 0x53, 0x61, 0x6e, 0x6c, 0x75, 0x69, 0x73, 0x61,
0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e, 0x61, 0x55, 0x161, 0x75, 0x61,
0x6a, 0x61, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x6a, 0x6f, 0x6e, 0x61, 0x41,
0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6e, 0x61, 0x42, 0x61, 0x69, 0x6a,
0x61, 0x42, 0x61, 0x6a, 0x61, 0x62, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61,
0x73, 0x61, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64, 0x6f, 0x73, 0x61, 0x42,
0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0x61, 0x42,
0x6c, 0x61, 0x6e, 0x73, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x61, 0x42, 0x6f,
0x61, 0x76, 0x69, 0x73, 0x74, 0x61, 0x42, 0x6f, 0x69, 0x73, 0x69, 0x73,
0x69, 0x74, 0x69, 0x6a, 0x61, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x64,
0x17e, 0x62, 0x65, 0x6a, 0x61, 0x4b, 0x61, 0x6d, 0x70, 0x75, 0x67, 0x72,
0x61, 0x6e, 0x64, 0x69, 0x4b, 0x61, 0x6e, 0x6b, 0x75, 0x6e, 0x61, 0x4b,
0x61, 0x72, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x4b, 0x61, 0x6a, 0x65, 0x6e,
0x6e, 0x61, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x75, 0x20, 0x73, 0x61,
0x6c, 0x61, 0x73, 0x10c, 0x69, 0x6b, 0x101, 0x67, 0x61, 0x10c, 0x69, 0x76,
0x61, 0x76, 0x61, 0x48, 0x75, 0x61, 0x72, 0x65, 0x73, 0x61, 0x4b, 0x72,
0x65, 0x73, 0x74, 0x6f, 0x6e, 0x61, 0x4b, 0x69, 0x72, 0x61, 0x73, 0x61,
0x6f, 0x44, 0x65, 0x6e, 0x6d, 0x101, 0x72, 0x6b, 0x161, 0x61, 0x76, 0x6e,
0x61, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x61, 0x44, 0x6f, 0x75, 0x73,
0x6f, 0x6e, 0x6b, 0x72, 0x12b, 0x6b, 0x61, 0x44, 0x65, 0x6e, 0x76, 0x65,
0x72, 0x61, 0x44, 0x65, 0x74, 0x72, 0x6f, 0x69, 0x74, 0x61, 0x45, 0x64,
0x6d, 0x6f, 0x6e, 0x74, 0x6f, 0x6e, 0x61, 0x53, 0x61, 0x6c, 0x76, 0x61,
0x64, 0x6f, 0x72, 0x61, 0x46, 0x6f, 0x72, 0x74, 0x6e, 0x65, 0x6c, 0x73,
0x6f, 0x6e, 0x61, 0x47, 0x6c, 0x65, 0x69, 0x73, 0x62, 0x65, 0x6a, 0x61,
0x47, 0x16b, 0x73, 0x62, 0x65, 0x6a, 0x61, 0x47, 0x72, 0x61, 0x6e, 0x64,
0x74, 0x6b, 0x113, 0x72, 0x6b, 0x61, 0x47, 0x72, 0x65, 0x6e, 0x101, 0x64,
0x61, 0x47, 0x76, 0x61, 0x64, 0x65, 0x6c, 0x75, 0x70, 0x61, 0x47, 0x76,
0x61, 0x6a, 0x61, 0x6b, 0x69, 0x6c, 0x61, 0x47, 0x61, 0x6a, 0x101, 0x6e,
0x61, 0x48, 0x65, 0x6c, 0x69, 0x66, 0x65, 0x6b, 0x73, 0x61, 0x45, 0x72,
0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6a, 0x6f, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x61, 0x4e, 0x6f, 0x6b, 0x73,
0x61, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61, 0x4d, 0x61,
0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101,
0x6e, 0x61, 0x50, 0x12b, 0x74, 0x65, 0x72, 0x73, 0x62, 0x113, 0x72, 0x67,
0x61, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61, 0x54, 0x65,
0x6c, 0x73, 0x69, 0x74, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x101, 0x6e, 0x61, 0x56, 0x12b, 0x76, 0x65, 0x6a, 0x61, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x73, 0x65,
0x6e, 0x73, 0x61, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61,
0x56, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x61, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x101, 0x6e, 0x61, 0x49, 0x6e, 0x75, 0x76, 0x69, 0x6b, 0x61,
0x49, 0x6b, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x61, 0x44, 0x17e, 0x75, 0x6e,
0x6f, 0x4c, 0x16b, 0x69, 0x76, 0x69, 0x6c, 0x61, 0x4d, 0x6f, 0x6e, 0x74,
0x69, 0x10d, 0x65, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75,
0x6b, 0x69, 0x4b, 0x72, 0x101, 0x6c, 0x65, 0x6e, 0x64, 0x65, 0x69, 0x6b,
0x61, 0x4c, 0x61, 0x70, 0x61, 0x73, 0x61, 0x4c, 0x6f, 0x73, 0x61, 0x6e,
0x64, 0x17e, 0x65, 0x6c, 0x6f, 0x73, 0x61, 0x4c, 0x6f, 0x75, 0x65, 0x72,
0x70, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x73, 0x6b, 0x76, 0x6f, 0x74, 0x65,
0x72, 0x61, 0x4d, 0x61, 0x73, 0x65, 0x6a, 0x6f, 0x4d, 0x61, 0x6e, 0x61,
0x67, 0x76, 0x61, 0x4d, 0x61, 0x6e, 0x61, 0x75, 0x73, 0x61, 0x4d, 0x65,
0x72, 0x69, 0x67, 0x6f, 0x74, 0x61, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x6f,
0x72, 0x6f, 0x73, 0x61, 0x4d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e,
0x61, 0x4d, 0x65, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x12b, 0x4d, 0x65, 0x68,
0x69, 0x6b, 0x6f, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x61, 0x4d,
0x6f, 0x6e, 0x6b, 0x74, 0x6f, 0x6e, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x65,
0x72, 0x72, 0x65, 0x6a, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x73, 0x65, 0x72,
0x72, 0x61, 0x74, 0x61, 0x4e, 0x61, 0x73, 0x6f, 0x145, 0x75, 0x6a, 0x6f,
0x72, 0x6b, 0x61, 0x4e, 0x6f, 0x6d, 0x61, 0x4e, 0x6f, 0x72, 0x6f, 0x146,
0x61, 0x42, 0x6a, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x5a, 0x69, 0x65, 0x6d,
0x65, 0x13c, 0x64, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x61, 0x2c, 0x20, 0x5a, 0x69, 0x65, 0x6d, 0x65, 0x13c, 0x64,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x145, 0x16b, 0x73, 0x65, 0x69, 0x6c, 0x65,
0x6d, 0x61, 0x2c, 0x20, 0x5a, 0x69, 0x65, 0x6d, 0x65, 0x13c, 0x64, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x16b, 0x6b, 0x61, 0x46, 0x12b, 0x6e, 0x69,
0x6b, 0x73, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x70, 0x72, 0x65, 0x6e,
0x73, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x66, 0x73, 0x70, 0x65, 0x69,
0x6e, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x76, 0x65, 0x13c, 0x75, 0x50,
0x75, 0x6e, 0x74, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x61, 0x73, 0x61, 0x52,
0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x69, 0x6e, 0x6c, 0x65, 0x74, 0x61, 0x52,
0x65, 0x73, 0x69, 0x66, 0x69, 0x52, 0x69, 0x64, 0x17e, 0x61, 0x69, 0x6e,
0x61, 0x52, 0x65, 0x7a, 0x6f, 0x6c, 0x16b, 0x74, 0x61, 0x52, 0x69, 0x6f,
0x62, 0x72, 0x61, 0x6e, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72,
0x65, 0x6e, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x6a, 0x61, 0x67, 0x6f, 0x53,
0x61, 0x6e, 0x74, 0x6f, 0x64, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53,
0x61, 0x6e, 0x70, 0x61, 0x75, 0x6c, 0x75, 0x49, 0x74, 0x6f, 0x6b, 0x6f,
0x72, 0x74, 0x6f, 0x72, 0x6d, 0x69, 0x74, 0x61, 0x53, 0x65, 0x6e, 0x62,
0x61, 0x72, 0x74, 0x65, 0x6c, 0x6d, 0x12b, 0x53, 0x65, 0x6e, 0x74, 0x64,
0x17e, 0x6f, 0x6e, 0x73, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x6b, 0x69, 0x74,
0x73, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x6c, 0x16b, 0x73, 0x69, 0x6a, 0x61,
0x53, 0x65, 0x6e, 0x74, 0x6f, 0x6d, 0x61, 0x73, 0x61, 0x53, 0x65, 0x6e,
0x74, 0x76, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x61, 0x53, 0x76, 0x69,
0x66, 0x74, 0x6b, 0x61, 0x72, 0x65, 0x6e, 0x74, 0x61, 0x54, 0x75, 0x6c,
0x65, 0x56, 0x61, 0x6e, 0x6b, 0x16b, 0x76, 0x65, 0x72, 0x61, 0x56, 0x61,
0x69, 0x74, 0x68, 0x6f, 0x72, 0x73, 0x61, 0x56, 0x69, 0x6e, 0x69, 0x70,
0x65, 0x67, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x74, 0x61, 0x4b,
0x65, 0x69, 0x73, 0x69, 0x44, 0x65, 0x69, 0x76, 0x69, 0x73, 0x61, 0x44,
0x69, 0x6d, 0x6f, 0x6e, 0x64, 0x69, 0x72, 0x76, 0x69, 0x6c, 0x61, 0x4d,
0x61, 0x6b, 0x76, 0x6f, 0x72, 0x69, 0x4d, 0x6f, 0x73, 0x6f, 0x6e, 0x61,
0x4d, 0x61, 0x6b, 0x6d, 0x65, 0x72, 0x64, 0x6f, 0x50, 0x101, 0x6c, 0x6d,
0x65, 0x72, 0x61, 0x160, 0x6f, 0x76, 0x61, 0x54, 0x72, 0x6f, 0x6c, 0x6c,
0x61, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x4c, 0x6f, 0x6e, 0x67,
0x6a, 0x113, 0x72, 0x62, 0x12b, 0x65, 0x6e, 0x65, 0x41, 0x64, 0x65, 0x6e,
0x61, 0x41, 0x6d, 0x6d, 0x101, 0x6e, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x69,
0x72, 0x61, 0x41, 0x6b, 0x74, 0x65, 0x62, 0x65, 0x41, 0x161, 0x67, 0x61,
0x62, 0x61, 0x74, 0x61, 0x42, 0x61, 0x67, 0x64, 0x101, 0x64, 0x65, 0x42,
0x61, 0x68, 0x72, 0x65, 0x69, 0x6e, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6b,
0x6f, 0x6b, 0x61, 0x42, 0x61, 0x72, 0x6e, 0x61, 0x75, 0x6c, 0x61, 0x42,
0x65, 0x69, 0x72, 0x16b, 0x74, 0x61, 0x42, 0x69, 0x161, 0x6b, 0x65, 0x6b,
0x61, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x61, 0x44, 0x61, 0x6d, 0x61,
0x73, 0x6b, 0x61, 0x44, 0x75, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x48, 0x65,
0x62, 0x72, 0x6f, 0x6e, 0x61, 0x48, 0x6f, 0x161, 0x69, 0x6d, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x76,
0x64, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x4a, 0x65,
0x72, 0x75, 0x7a, 0x61, 0x6c, 0x65, 0x6d, 0x65, 0x4b, 0x61, 0x62, 0x75,
0x6c, 0x61, 0x4b, 0x61, 0x72, 0x101, 0x10d, 0x69, 0x4b, 0x61, 0x6c, 0x6b,
0x101, 0x74, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x61, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x6c, 0x75, 0x6d, 0x70,
0x75, 0x72, 0x61, 0x4b, 0x75, 0x10d, 0x69, 0x6e, 0x61, 0x4b, 0x75, 0x76,
0x65, 0x69, 0x74, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61,
0x4d, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x72, 0x61, 0x4d, 0x61, 0x73, 0x6b,
0x61, 0x74, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x146, 0x65,
0x63, 0x6b, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x61, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x6c,
0x61, 0x50, 0x6e, 0x6f, 0x6d, 0x70, 0x65, 0x146, 0x61, 0x50, 0x6f, 0x6e,
0x74, 0x69, 0x61, 0x6e, 0x61, 0x6b, 0x61, 0x50, 0x68, 0x65, 0x6e, 0x6a,
0x61, 0x6e, 0x61, 0x4b, 0x61, 0x74, 0x61, 0x72, 0x61, 0x4b, 0x6f, 0x73,
0x74, 0x61, 0x6e, 0x61, 0x6a, 0x61, 0x52, 0x69, 0x6a, 0x101, 0x64, 0x61,
0x53, 0x61, 0x68, 0x61, 0x6c, 0x12b, 0x6e, 0x61, 0x53, 0x65, 0x75, 0x6c,
0x61, 0x160, 0x61, 0x6e, 0x68, 0x61, 0x6a, 0x61, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x16b, 0x72, 0x61, 0x53, 0x72, 0x65, 0x64, 0x146, 0x65, 0x6b,
0x6f, 0x6c, 0x69, 0x6d, 0x73, 0x6b, 0x61, 0x54, 0x61, 0x69, 0x62, 0x65,
0x69, 0x54, 0x61, 0x161, 0x6b, 0x65, 0x6e, 0x74, 0x61, 0x54, 0x65, 0x68,
0x65, 0x72, 0x101, 0x6e, 0x61, 0x54, 0x6f, 0x6b, 0x69, 0x6a, 0x61, 0x54,
0x6f, 0x6d, 0x73, 0x6b, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74,
0x6f, 0x72, 0x61, 0x55, 0x73, 0x74, 0x6a, 0x146, 0x65, 0x72, 0x61, 0x56,
0x6a, 0x65, 0x6e, 0x10d, 0x61, 0x6e, 0x61, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x61, 0x52, 0x61, 0x6e, 0x67, 0x75, 0x6e, 0x61, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61,
0x45, 0x72, 0x65, 0x76, 0x101, 0x6e, 0x61, 0x41, 0x7a, 0x6f, 0x72, 0x75,
0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x4b, 0x61, 0x6e, 0x101, 0x72, 0x69,
0x6a, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x4b, 0x61, 0x62, 0x6f,
0x76, 0x65, 0x72, 0x64, 0x65, 0x46, 0x113, 0x72, 0x75, 0x20, 0x73, 0x61,
0x6c, 0x61, 0x73, 0x52, 0x65, 0x69, 0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b,
0x61, 0x44, 0x69, 0x65, 0x6e, 0x76, 0x69, 0x64, 0x64, 0x17e, 0x6f, 0x72,
0x64, 0x17e, 0x69, 0x6a, 0x61, 0x53, 0x76, 0x2e, 0x48, 0x65, 0x6c, 0x113,
0x6e, 0x61, 0x73, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x42, 0x72, 0x69, 0x73,
0x62, 0x65, 0x6e, 0x61, 0x42, 0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x68, 0x69,
0x6c, 0x61, 0x44, 0x101, 0x72, 0x76, 0x69, 0x6e, 0x61, 0x4a, 0x75, 0x6b,
0x6c, 0x61, 0x48, 0x6f, 0x62, 0x101, 0x72, 0x74, 0x61, 0x4c, 0x69, 0x6e,
0x64, 0x65, 0x6d, 0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x68, 0x61,
0x76, 0x61, 0x4d, 0x65, 0x6c, 0x62, 0x75, 0x72, 0x6e, 0x61, 0x50, 0x113,
0x72, 0x74, 0x61, 0x53, 0x69, 0x64, 0x6e, 0x65, 0x6a, 0x61, 0x41, 0x6d,
0x73, 0x74, 0x65, 0x72, 0x64, 0x61, 0x6d, 0x61, 0x41, 0x73, 0x74, 0x72,
0x61, 0x68, 0x61, 0x146, 0x61, 0x41, 0x74, 0x113, 0x6e, 0x61, 0x73, 0x42,
0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x61, 0x42, 0x65, 0x72, 0x6c, 0x12b,
0x6e, 0x65, 0x42, 0x72, 0x69, 0x73, 0x65, 0x6c, 0x65, 0x42, 0x75, 0x6b,
0x61, 0x72, 0x65, 0x73, 0x74, 0x65, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65,
0x161, 0x74, 0x61, 0x42, 0x12b, 0x7a, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x65,
0x4b, 0x69, 0x161, 0x69, 0x146, 0x65, 0x76, 0x61, 0x4b, 0x6f, 0x70, 0x65,
0x6e, 0x68, 0x101, 0x67, 0x65, 0x6e, 0x61, 0x44, 0x75, 0x62, 0x6c, 0x69,
0x6e, 0x61, 0x47, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x101, 0x72, 0x73,
0x47, 0x113, 0x72, 0x6e, 0x73, 0x69, 0x6a, 0x61, 0x4d, 0x65, 0x6e, 0x61,
0x73, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x53, 0x74, 0x61, 0x6d, 0x62, 0x75,
0x6c, 0x61, 0x44, 0x17e, 0x113, 0x72, 0x73, 0x69, 0x6a, 0x61, 0x4b, 0x61,
0x13c, 0x69, 0x146, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x61, 0x4b, 0x69,
0x72, 0x6f, 0x76, 0x61, 0x4b, 0x69, 0x6a, 0x65, 0x76, 0x61, 0x4c, 0x69,
0x73, 0x61, 0x62, 0x6f, 0x6e, 0x61, 0x13b, 0x75, 0x62, 0x13c, 0x61, 0x6e,
0x61, 0x4c, 0x6f, 0x6e, 0x64, 0x6f, 0x6e, 0x61, 0x4c, 0x75, 0x6b, 0x73,
0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x61, 0x4d, 0x61, 0x64, 0x72, 0x69,
0x64, 0x65, 0x4d, 0x61, 0x72, 0x69, 0x65, 0x68, 0x61, 0x6d, 0x6e, 0x61,
0x4d, 0x69, 0x6e, 0x73, 0x6b, 0x61, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x76,
0x61, 0x50, 0x61, 0x72, 0x12b, 0x7a, 0x65, 0x50, 0x72, 0x101, 0x67, 0x61,
0x53, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x12b, 0x6e, 0x6f, 0x53, 0x61, 0x72,
0x61, 0x6a, 0x65, 0x76, 0x61, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76,
0x61, 0x53, 0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x65,
0x53, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x61, 0x54, 0x61, 0x6c,
0x6c, 0x69, 0x6e, 0x61, 0x54, 0x69, 0x72, 0x101, 0x6e, 0x61, 0x55, 0x13c,
0x6a, 0x61, 0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x61, 0x56, 0x61, 0x64, 0x75,
0x63, 0x61, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x101, 0x6e, 0x73, 0x56, 0x12b,
0x6e, 0x65, 0x56, 0x69, 0x13c, 0x146, 0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x61, 0x5a, 0x61, 0x67, 0x72, 0x65, 0x62, 0x61,
0x43, 0x12b, 0x72, 0x69, 0x68, 0x65, 0x10c, 0x61, 0x67, 0x6f, 0x73, 0x61,
0x5a, 0x69, 0x65, 0x6d, 0x73, 0x76, 0x113, 0x74, 0x6b, 0x75, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x75, 0x20, 0x28, 0x4b,
0x12b, 0x6c, 0x69, 0x6e, 0x67, 0x61, 0x29, 0x20, 0x73, 0x61, 0x6c, 0x61,
0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x61, 0x73, 0x4b, 0x65, 0x72, 0x67, 0x65,
0x6c, 0x113, 0x6e, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x4d, 0x61, 0x65,
0x4d, 0x61, 0x6c, 0x64, 0x12b, 0x76, 0x69, 0x6a, 0x61, 0x4d, 0x61, 0x75,
0x72, 0x12b, 0x63, 0x69, 0x6a, 0x61, 0x4d, 0x61, 0x6a, 0x6f, 0x74, 0x61,
0x52, 0x65, 0x69, 0x6e, 0x6a, 0x6f, 0x6e, 0x61, 0x41, 0x70, 0x69, 0x6a,
0x61, 0x4f, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x61, 0x42, 0x75, 0x67, 0x65,
0x6e, 0x76, 0x69, 0x6c, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x10c, 0x65,
0x74, 0x65, 0x6d, 0x61, 0x10c, 0x16b, 0x6b, 0x61, 0x4c, 0x69, 0x65, 0x6c,
0x64, 0x69, 0x65, 0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x47, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73,
0x47, 0x61, 0x6d, 0x62, 0x6a, 0x113, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73,
0x47, 0x76, 0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x61,
0x47, 0x75, 0x61, 0x6d, 0x61, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x65,
0x72, 0x69, 0x6a, 0x61, 0x4b, 0x69, 0x72, 0x69, 0x73, 0x69, 0x6d, 0x61,
0x73, 0x69, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x6a, 0x61, 0x4b, 0x76, 0x61,
0x64, 0x17e, 0x61, 0x6c, 0x65, 0x69, 0x6e, 0x61, 0x4d, 0x61, 0x64, 0x17e,
0x75, 0x72, 0x6f, 0x4d, 0x61, 0x72, 0x137, 0x12b, 0x7a, 0x61, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x73, 0x4d, 0x69, 0x64, 0x76, 0x65, 0x6a, 0x73, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x61, 0x4e, 0x75, 0x6d, 0x65, 0x61,
0x50, 0x69, 0x74, 0x6b, 0x113, 0x72, 0x6e, 0x61, 0x50, 0x6f, 0x6e, 0x70,
0x65, 0x6a, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6d, 0x6f, 0x72, 0x73, 0x62,
0x69, 0x53, 0x61, 0x69, 0x70, 0x61, 0x6e, 0x61, 0x54, 0x61, 0x69, 0x74,
0x69, 0x56, 0x65, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x56,
0x6f, 0x6c, 0x69, 0x73, 0x61, 0x41, 0x62, 0x69, 0x64, 0x17e, 0x61, 0x6e,
0x61, 0x73, 0x41, 0x6c, 0x17e, 0x79, 0x72, 0x61, 0x73, 0x42, 0x61, 0x6d,
0x61, 0x6b, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x69, 0x73, 0x42, 0x61,
0x6e, 0x64, 0x17e, 0x75, 0x6c, 0x69, 0x73, 0x42, 0x6c, 0x61, 0x6e, 0x74,
0x61, 0x69, 0x72, 0x61, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x61, 0x76, 0x69,
0x6c, 0x69, 0x73, 0x42, 0x75, 0x17e, 0x75, 0x6d, 0x62, 0x16b, 0x72, 0x61,
0x4b, 0x61, 0x69, 0x72, 0x61, 0x73, 0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72,
0x69, 0x73, 0x44, 0x61, 0x6b, 0x61, 0x72, 0x61, 0x73, 0x44, 0x61, 0x72,
0x20, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x73, 0x44,
0x17e, 0x69, 0x62, 0x75, 0x74, 0x69, 0x73, 0x61, 0x6c, 0x20, 0x41, 0x6a,
0x16b, 0x6e, 0x61, 0x73, 0x46, 0x72, 0x69, 0x74, 0x61, 0x75, 0x6e, 0x61,
0x73, 0x47, 0x61, 0x62, 0x6f, 0x72, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x61,
0x72, 0x61, 0x72, 0x117, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x62,
0x75, 0x72, 0x67, 0x61, 0x73, 0x44, 0x17e, 0x75, 0x62, 0x61, 0x43, 0x68,
0x61, 0x72, 0x74, 0x75, 0x6d, 0x61, 0x73, 0x4b, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x73, 0x4c, 0x61, 0x67, 0x6f, 0x73, 0x61, 0x73, 0x4c, 0x69, 0x62,
0x72, 0x65, 0x76, 0x69, 0x6c, 0x69, 0x73, 0x4c, 0x6f, 0x6d, 0x117, 0x4c,
0x75, 0x62, 0x75, 0x6d, 0x62, 0x61, 0x161, 0x69, 0x73, 0x4d, 0x61, 0x6c,
0x61, 0x62, 0x61, 0x73, 0x4d, 0x61, 0x70, 0x75, 0x74, 0x61, 0x73, 0x4d,
0x62, 0x61, 0x62, 0x61, 0x6e, 0x117, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69,
0x161, 0x61, 0x73, 0x4e, 0x61, 0x69, 0x72, 0x6f, 0x62, 0x69, 0x73, 0x4e,
0x69, 0x61, 0x6d, 0x117, 0x6a, 0x75, 0x73, 0x4e, 0x75, 0x61, 0x6b, 0x161,
0x6f, 0x74, 0x61, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x4e, 0x6f,
0x76, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6d, 0x117, 0x54,
0x75, 0x6e, 0x69, 0x73, 0x61, 0x73, 0x56, 0x69, 0x6e, 0x64, 0x68, 0x75,
0x6b, 0x61, 0x73, 0x45, 0x69, 0x64, 0x61, 0x6b, 0x61, 0x73, 0x41, 0x6e,
0x6b, 0x6f, 0x72, 0x69, 0x64, 0x17e, 0x61, 0x73, 0x41, 0x6e, 0x67, 0x69,
0x6c, 0x69, 0x6a, 0x61, 0x41, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61,
0x6e, 0x61, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x20, 0x41, 0x69, 0x72,
0x117, 0x73, 0x43, 0x68, 0x75, 0x63, 0x68, 0x75, 0x6a, 0x75, 0x73, 0x4c,
0x61, 0x20, 0x52, 0x69, 0x6f, 0x63, 0x68, 0x61, 0x52, 0x69, 0x6f, 0x20,
0x47, 0x61, 0x6c, 0x6a, 0x65, 0x67, 0x6f, 0x73, 0x61, 0x73, 0x53, 0x61,
0x6c, 0x74, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x43, 0x68, 0x75, 0x61,
0x6e, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x4c, 0x75, 0x69, 0x73,
0x61, 0x73, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x41,
0x73, 0x75, 0x6e, 0x73, 0x6a, 0x6f, 0x6e, 0x61, 0x73, 0x41, 0x74, 0x69,
0x6b, 0x6f, 0x6b, 0x61, 0x6e, 0x61, 0x73, 0x42, 0x61, 0x68, 0x69, 0x61,
0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x61, 0x73, 0x42,
0x61, 0x72, 0x62, 0x61, 0x64, 0x6f, 0x73, 0x61, 0x73, 0x42, 0x65, 0x6c,
0x65, 0x6e, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0x61, 0x73, 0x42,
0x6c, 0x61, 0x6e, 0x10d, 0x20, 0x53, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x61,
0x73, 0x42, 0x75, 0x61, 0x20, 0x56, 0x69, 0x73, 0x74, 0x61, 0x4b, 0x65,
0x69, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x17e, 0x20, 0x42, 0x117, 0x6a, 0x75,
0x73, 0x4b, 0x61, 0x6d, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e, 0x64,
0x117, 0x4b, 0x61, 0x6e, 0x6b, 0x16b, 0x6e, 0x61, 0x73, 0x4b, 0x61, 0x72,
0x61, 0x6b, 0x61, 0x73, 0x61, 0x73, 0x4b, 0x61, 0x6a, 0x65, 0x6e, 0x61,
0x73, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x10c, 0x69, 0x6b,
0x61, 0x67, 0x61, 0x10c, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x53,
0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x43, 0x68, 0x75, 0x61, 0x72, 0x65,
0x73, 0x61, 0x73, 0x4b, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x6e, 0x61, 0x73,
0x4b, 0x69, 0x75, 0x72, 0x61, 0x73, 0x61, 0x6f, 0x44, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x6b, 0x73, 0x68, 0x61, 0x75, 0x6e, 0x61, 0x73, 0x44, 0x6f,
0x73, 0x6f, 0x6e, 0x61, 0x73, 0x44, 0x6f, 0x73, 0x6f, 0x6e, 0x20, 0x4b,
0x72, 0x69, 0x6b, 0x61, 0x73, 0x44, 0x65, 0x6e, 0x76, 0x65, 0x72, 0x69,
0x73, 0x44, 0x65, 0x74, 0x72, 0x6f, 0x69, 0x74, 0x61, 0x73, 0x45, 0x64,
0x6d, 0x6f, 0x6e, 0x74, 0x6f, 0x6e, 0x61, 0x73, 0x45, 0x69, 0x72, 0x75,
0x6e, 0x65, 0x70, 0x117, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72,
0x61, 0x73, 0x46, 0x6f, 0x72, 0x74, 0x20, 0x4e, 0x65, 0x6c, 0x73, 0x6f,
0x6e, 0x61, 0x73, 0x47, 0x6c, 0x65, 0x69, 0x73, 0x20, 0x42, 0x117, 0x6a,
0x75, 0x73, 0x47, 0x75, 0x73, 0x20, 0x42, 0x117, 0x6a, 0x75, 0x73, 0x47,
0x72, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x65, 0x72, 0x6b, 0x61, 0x73, 0x47,
0x76, 0x61, 0x64, 0x61, 0x6c, 0x75, 0x70, 0x117, 0x47, 0x76, 0x61, 0x6a,
0x61, 0x6b, 0x69, 0x6c, 0x69, 0x73, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61,
0x6b, 0x73, 0x61, 0x73, 0x48, 0x65, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c,
0x6a, 0x61, 0x73, 0x4e, 0x6f, 0x6b, 0x73, 0x61, 0x73, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67,
0x61, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50,
0x69, 0x74, 0x65, 0x72, 0x73, 0x62, 0x65, 0x72, 0x67, 0x61, 0x73, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x20,
0x53, 0x69, 0x74, 0x69, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x56, 0x69, 0x76, 0x69, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x61, 0x73,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e,
0x61, 0x6d, 0x61, 0x6b, 0x61, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x49, 0x6e, 0x75, 0x76, 0x69, 0x6b, 0x61, 0x73, 0x49,
0x6b, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x61, 0x73, 0x44, 0x17e, 0x75, 0x6e,
0x61, 0x73, 0x4c, 0x75, 0x69, 0x73, 0x76, 0x69, 0x6c, 0x69, 0x73, 0x4d,
0x6f, 0x6e, 0x74, 0x69, 0x73, 0x65, 0x6c, 0x61, 0x73, 0x2c, 0x20, 0x4b,
0x65, 0x6e, 0x74, 0x75, 0x6b, 0x69, 0x73, 0x4b, 0x72, 0x61, 0x6c, 0x65,
0x6e, 0x64, 0x65, 0x69, 0x6b, 0x61, 0x73, 0x4c, 0x61, 0x20, 0x50, 0x61,
0x73, 0x61, 0x73, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x64, 0x17e, 0x65,
0x6c, 0x61, 0x73, 0x17d, 0x65, 0x6d, 0x75, 0x74, 0x69, 0x6e, 0x69, 0x73,
0x20, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x73, 0x20, 0x4b, 0x76, 0x6f,
0x72, 0x74, 0x65, 0x72, 0x69, 0x73, 0x4d, 0x61, 0x6e, 0x61, 0x75, 0x73,
0x61, 0x73, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x73, 0x61,
0x73, 0x4d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x61, 0x73, 0x4d,
0x65, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x73, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x61, 0x73, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x61,
0x73, 0x4d, 0x6f, 0x6e, 0x6b, 0x74, 0x6f, 0x6e, 0x61, 0x73, 0x4d, 0x6f,
0x6e, 0x74, 0x65, 0x72, 0x117, 0x6a, 0x75, 0x73, 0x4d, 0x6f, 0x6e, 0x74,
0x65, 0x76, 0x69, 0x64, 0x117, 0x6a, 0x61, 0x73, 0x4d, 0x6f, 0x6e, 0x74,
0x73, 0x65, 0x72, 0x61, 0x74, 0x61, 0x73, 0x4e, 0x61, 0x73, 0x61, 0x75,
0x4e, 0x69, 0x75, 0x6a, 0x6f, 0x72, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x6d,
0x61, 0x73, 0x42, 0x6a, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x160, 0x69, 0x61,
0x75, 0x72, 0x117, 0x73, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53,
0x65, 0x6e, 0x74, 0x65, 0x72, 0x69, 0x73, 0x2c, 0x20, 0x160, 0x69, 0x61,
0x75, 0x72, 0x117, 0x73, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x69, 0x75, 0x20, 0x53, 0x65, 0x69, 0x6c, 0x65, 0x6d, 0x61, 0x73, 0x2c,
0x20, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x4e, 0x16b, 0x6b, 0x61, 0x73, 0x4f, 0x63, 0x68, 0x69,
0x6e, 0x61, 0x67, 0x61, 0x50, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72, 0x69,
0x62, 0x61, 0x73, 0x46, 0x69, 0x6e, 0x69, 0x6b, 0x73, 0x61, 0x73, 0x50,
0x6f, 0x72, 0x74, 0x20, 0x6f, 0x20, 0x50, 0x72, 0x65, 0x6e, 0x73, 0x61,
0x73, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x6f, 0x66, 0x20, 0x53, 0x70, 0x65,
0x69, 0x6e, 0x61, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x56, 0x65,
0x6c, 0x6a, 0x61, 0x73, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x52,
0x69, 0x6b, 0x61, 0x73, 0x50, 0x75, 0x6e, 0x74, 0x61, 0x20, 0x41, 0x72,
0x65, 0x6e, 0x61, 0x73, 0x61, 0x73, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e,
0x20, 0x49, 0x6e, 0x6c, 0x65, 0x74, 0x61, 0x73, 0x52, 0x65, 0x73, 0x69,
0x66, 0x117, 0x52, 0x65, 0x64, 0x17e, 0x69, 0x6e, 0x61, 0x52, 0x65, 0x73,
0x6f, 0x6c, 0x75, 0x74, 0x61, 0x73, 0x52, 0x69, 0x6f, 0x20, 0x42, 0x72,
0x61, 0x6e, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x65,
0x6e, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x6a, 0x61, 0x67, 0x61, 0x73,
0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67,
0x61, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x75, 0x6c, 0x61, 0x73,
0x49, 0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x6d, 0x69, 0x74,
0x61, 0x73, 0x53, 0x65, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x74, 0x65, 0x6c,
0x65, 0x6d, 0x69, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x44, 0x17e, 0x6f, 0x6e,
0x73, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x4b, 0x69, 0x74, 0x73,
0x61, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x6a,
0x61, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x73, 0x61,
0x73, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e,
0x74, 0x61, 0x73, 0x53, 0x76, 0x69, 0x66, 0x74, 0x20, 0x4b, 0x61, 0x72,
0x65, 0x6e, 0x74, 0x61, 0x73, 0x4b, 0x61, 0x6e, 0x61, 0x6b, 0x61, 0x73,
0x54, 0x69, 0x63, 0x68, 0x75, 0x61, 0x6e, 0x61, 0x54, 0x6f, 0x72, 0x6f,
0x6e, 0x74, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x6b, 0x75, 0x76, 0x65, 0x72,
0x69, 0x73, 0x56, 0x61, 0x69, 0x74, 0x68, 0x6f, 0x72, 0x73, 0x61, 0x73,
0x56, 0x69, 0x6e, 0x69, 0x70, 0x65, 0x67, 0x61, 0x73, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x61, 0x74, 0x61, 0x73, 0x4b, 0x65, 0x69, 0x73, 0x69, 0x73,
0x44, 0x65, 0x69, 0x76, 0x69, 0x73, 0x61, 0x73, 0x44, 0x69, 0x75, 0x6d,
0x6f, 0x6e, 0x61, 0x73, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x69, 0x73, 0x4d, 0x61, 0x6b, 0x76, 0x6f, 0x72, 0x69, 0x73, 0x4d, 0x6f,
0x73, 0x6f, 0x6e, 0x61, 0x73, 0x4d, 0x61, 0x6b, 0x6d, 0x65, 0x72, 0x64,
0x61, 0x73, 0x50, 0x61, 0x6c, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x53, 0x69,
0x6f, 0x76, 0x61, 0x54, 0x72, 0x6f, 0x6c, 0x69, 0x73, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x61, 0x73, 0x4c, 0x6f, 0x6e, 0x67, 0x6a, 0x79, 0x72,
0x62, 0x69, 0x65, 0x6e, 0x61, 0x73, 0x41, 0x64, 0x65, 0x6e, 0x61, 0x73,
0x41, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x69, 0x73, 0x41, 0x6b, 0x74, 0x69, 0x75, 0x62, 0x69, 0x6e, 0x73, 0x6b,
0x61, 0x73, 0x41, 0x161, 0x63, 0x68, 0x61, 0x62, 0x61, 0x64, 0x61, 0x73,
0x42, 0x61, 0x67, 0x64, 0x61, 0x64, 0x61, 0x73, 0x42, 0x61, 0x68, 0x72,
0x65, 0x69, 0x6e, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x6b, 0x6f, 0x6b, 0x61,
0x73, 0x42, 0x61, 0x72, 0x6e, 0x61, 0x75, 0x6c, 0x61, 0x73, 0x42, 0x65,
0x69, 0x72, 0x75, 0x74, 0x61, 0x73, 0x42, 0x69, 0x161, 0x6b, 0x65, 0x6b,
0x61, 0x73, 0x42, 0x72, 0x75, 0x6e, 0x117, 0x6a, 0x75, 0x73, 0x4b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x61, 0x73, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b,
0x61, 0x73, 0x44, 0x69, 0x6c, 0x69, 0x73, 0x44, 0x75, 0x62, 0x61, 0x6a,
0x75, 0x73, 0x44, 0x75, 0x161, 0x61, 0x6e, 0x62, 0x117, 0x47, 0x61, 0x7a,
0x6f, 0x73, 0x20, 0x72, 0x75, 0x6f, 0x17e, 0x61, 0x73, 0x48, 0x65, 0x62,
0x72, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x161, 0x69, 0x6d, 0x69, 0x6e,
0x61, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x48,
0x6f, 0x76, 0x64, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x61, 0x73, 0x4a, 0x65, 0x72, 0x75, 0x7a, 0x61, 0x6c, 0x117, 0x4b, 0x61,
0x62, 0x75, 0x6c, 0x61, 0x73, 0x4b, 0x61, 0x6d, 0x10d, 0x69, 0x61, 0x74,
0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x10d, 0x69, 0x73, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x4b, 0x76,
0x61, 0x6c, 0x61, 0x20, 0x4c, 0x75, 0x6d, 0x70, 0x16b, 0x72, 0x61, 0x73,
0x4b, 0x75, 0x10d, 0x69, 0x6e, 0x67, 0x61, 0x73, 0x4b, 0x75, 0x76, 0x65,
0x69, 0x74, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61,
0x73, 0x4d, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x4d, 0x61,
0x73, 0x6b, 0x61, 0x74, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75,
0x7a, 0x6e, 0x65, 0x63, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b,
0x61, 0x73, 0x55, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x61, 0x73, 0x50, 0x6e,
0x6f, 0x6d, 0x70, 0x65, 0x6e, 0x69, 0x73, 0x50, 0x6f, 0x6e, 0x74, 0x69,
0x61, 0x6e, 0x61, 0x6b, 0x61, 0x73, 0x50, 0x63, 0x68, 0x65, 0x6e, 0x6a,
0x61, 0x6e, 0x61, 0x73, 0x4b, 0x61, 0x74, 0x61, 0x72, 0x61, 0x73, 0x4b,
0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x6a, 0x75, 0x73, 0x4b, 0x7a, 0x79,
0x6c, 0x2d, 0x4f, 0x72, 0x64, 0x61, 0x52, 0x69, 0x6a, 0x61, 0x64, 0x61,
0x73, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x73, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64, 0x61, 0x73, 0x53, 0x65,
0x75, 0x6c, 0x61, 0x73, 0x160, 0x61, 0x6e, 0x63, 0x68, 0x61, 0x6a, 0x75,
0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x16b, 0x72, 0x61, 0x73, 0x53,
0x72, 0x65, 0x64, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x79, 0x6d, 0x73, 0x6b,
0x61, 0x73, 0x54, 0x61, 0x69, 0x70, 0x117, 0x6a, 0x75, 0x73, 0x54, 0x61,
0x161, 0x6b, 0x65, 0x6e, 0x74, 0x61, 0x73, 0x54, 0x62, 0x69, 0x6c, 0x69,
0x73, 0x69, 0x73, 0x54, 0x65, 0x68, 0x65, 0x72, 0x61, 0x6e, 0x61, 0x73,
0x54, 0x6f, 0x6b, 0x69, 0x6a, 0x61, 0x73, 0x54, 0x6f, 0x6d, 0x73, 0x6b,
0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x61, 0x73, 0x55, 0x73, 0x74, 0x20, 0x4e, 0x65, 0x72, 0x61, 0x56, 0x69,
0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x52, 0x61, 0x6e, 0x67, 0x16b, 0x6e,
0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x61, 0x73, 0x4a, 0x65, 0x72, 0x65, 0x76, 0x61, 0x6e,
0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x69, 0x4b, 0x61, 0x6e, 0x61,
0x72, 0x173, 0x20, 0x73, 0x61, 0x6c, 0x6f, 0x73, 0x17d, 0x61, 0x6c, 0x69,
0x61, 0x73, 0x69, 0x73, 0x20, 0x4b, 0x79, 0x161, 0x75, 0x6c, 0x79, 0x73,
0x46, 0x61, 0x72, 0x65, 0x72, 0x61, 0x69, 0x52, 0x65, 0x69, 0x6b, 0x6a,
0x61, 0x76, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x69, 0x65, 0x74, 0x173, 0x20,
0x44, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x53, 0x65, 0x6e,
0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x53, 0x74, 0x65, 0x6e,
0x6c, 0x69, 0x73, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x69, 0x64, 0x117, 0x42,
0x72, 0x69, 0x73, 0x62, 0x61, 0x6e, 0x61, 0x73, 0x42, 0x72, 0x6f, 0x6b,
0x65, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x69, 0x73, 0x44, 0x61, 0x72, 0x76,
0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x62, 0x61, 0x72, 0x74, 0x61, 0x73,
0x4c, 0x69, 0x6e, 0x64, 0x65, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x4c, 0x6f,
0x72, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61,
0x4d, 0x65, 0x6c, 0x62, 0x75, 0x72, 0x6e, 0x61, 0x73, 0x50, 0x65, 0x72,
0x74, 0x61, 0x73, 0x53, 0x69, 0x64, 0x6e, 0x117, 0x6a, 0x75, 0x73, 0x41,
0x6d, 0x73, 0x74, 0x65, 0x72, 0x64, 0x61, 0x6d, 0x61, 0x73, 0x41, 0x73,
0x74, 0x72, 0x61, 0x63, 0x68, 0x61, 0x6e, 0x117, 0x41, 0x74, 0x117, 0x6e,
0x61, 0x69, 0x42, 0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x61, 0x73, 0x42,
0x65, 0x72, 0x6c, 0x79, 0x6e, 0x61, 0x73, 0x42, 0x72, 0x69, 0x75, 0x73,
0x65, 0x6c, 0x69, 0x73, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x161, 0x74,
0x61, 0x73, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x161, 0x74, 0x61, 0x73,
0x42, 0x69, 0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x61, 0x73, 0x4b,
0x69, 0x161, 0x69, 0x6e, 0x69, 0x6f, 0x76, 0x61, 0x73, 0x4b, 0x6f, 0x70,
0x65, 0x6e, 0x68, 0x61, 0x67, 0x61, 0x44, 0x75, 0x62, 0x6c, 0x69, 0x6e,
0x61, 0x73, 0x47, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x61,
0x73, 0x47, 0x65, 0x72, 0x6e, 0x73, 0x69, 0x73, 0x48, 0x65, 0x6c, 0x73,
0x69, 0x6e, 0x6b, 0x69, 0x73, 0x4d, 0x65, 0x6e, 0x6f, 0x20, 0x73, 0x61,
0x6c, 0x61, 0x53, 0x74, 0x61, 0x6d, 0x62, 0x75, 0x6c, 0x61, 0x73, 0x44,
0x17e, 0x65, 0x72, 0x73, 0x69, 0x73, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69,
0x6e, 0x67, 0x72, 0x61, 0x64, 0x61, 0x73, 0x4b, 0x69, 0x72, 0x6f, 0x76,
0x61, 0x73, 0x4b, 0x69, 0x6a, 0x65, 0x76, 0x61, 0x73, 0x4c, 0x6f, 0x6e,
0x64, 0x6f, 0x6e, 0x61, 0x73, 0x4c, 0x69, 0x75, 0x6b, 0x73, 0x65, 0x6d,
0x62, 0x75, 0x72, 0x67, 0x61, 0x73, 0x4d, 0x61, 0x64, 0x72, 0x69, 0x64,
0x61, 0x73, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x68, 0x61, 0x6d, 0x69,
0x6e, 0x61, 0x4d, 0x69, 0x6e, 0x73, 0x6b, 0x61, 0x73, 0x4d, 0x6f, 0x6e,
0x61, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x76, 0x61, 0x4f, 0x73,
0x6c, 0x61, 0x73, 0x50, 0x61, 0x72, 0x79, 0x17e, 0x69, 0x75, 0x73, 0x52,
0x79, 0x67, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e,
0x61, 0x73, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0x61, 0x73, 0x53,
0x61, 0x72, 0x61, 0x74, 0x6f, 0x76, 0x61, 0x73, 0x53, 0x69, 0x6d, 0x66,
0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x53, 0x6b, 0x6f, 0x70,
0x6a, 0x117, 0x53, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x61, 0x73,
0x54, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x73, 0x55, 0x6c, 0x6a, 0x61, 0x6e,
0x6f, 0x76, 0x73, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x64, 0x75, 0x63, 0x61,
0x73, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x61, 0x73, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x73, 0x56, 0x61, 0x72,
0x161, 0x75, 0x76, 0x61, 0x5a, 0x61, 0x67, 0x72, 0x65, 0x62, 0x61, 0x73,
0x43, 0x69, 0x75, 0x72, 0x69, 0x63, 0x68, 0x61, 0x73, 0x41, 0x6e, 0x74,
0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x76, 0x61, 0x73, 0x10c, 0x61,
0x67, 0x6f, 0x73, 0x61, 0x73, 0x4b, 0x61, 0x6c, 0x117, 0x64, 0x173, 0x20,
0x53, 0x61, 0x6c, 0x61, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x173, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c, 0x65, 0x6e, 0x61,
0x73, 0x4d, 0x61, 0x68, 0x117, 0x4d, 0x61, 0x6c, 0x64, 0x79, 0x76, 0x61,
0x69, 0x4d, 0x61, 0x6a, 0x6f, 0x74, 0x61, 0x73, 0x52, 0x65, 0x75, 0x6e,
0x6a, 0x6f, 0x6e, 0x61, 0x73, 0x4f, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x73, 0x42, 0x75, 0x67, 0x65, 0x6e, 0x76, 0x69, 0x6c, 0x69, 0x73, 0x10c,
0x61, 0x74, 0x61, 0x6d, 0x61, 0x73, 0x10c, 0x75, 0x6b, 0x61, 0x73, 0x56,
0x65, 0x6c, 0x79, 0x6b, 0x173, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x45, 0x66,
0x61, 0x74, 0x117, 0x46, 0x61, 0x6b, 0x61, 0x6f, 0x66, 0x61, 0x73, 0x46,
0x69, 0x64, 0x17e, 0x69, 0x73, 0x46, 0x75, 0x6e, 0x61, 0x66, 0x75, 0x74,
0x69, 0x73, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x61, 0x69, 0x47,
0x61, 0x6d, 0x62, 0x79, 0x72, 0x61, 0x73, 0x47, 0x76, 0x61, 0x64, 0x61,
0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x69, 0x73, 0x47, 0x75, 0x61, 0x6d,
0x61, 0x73, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x75, 0x72, 0x69, 0x73,
0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x73, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x6a, 0x117, 0x4b, 0x76, 0x61, 0x64, 0x17e, 0x61,
0x6c, 0x65, 0x69, 0x6e, 0x61, 0x73, 0x4d, 0x61, 0x64, 0x17e, 0x16b, 0x72,
0x61, 0x73, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x6f, 0x20, 0x73, 0x61,
0x6c, 0x6f, 0x73, 0x4d, 0x69, 0x64, 0x76, 0x117, 0x6a, 0x75, 0x73, 0x4e,
0x69, 0x75, 0x6a, 0x117, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x61,
0x73, 0x4e, 0x75, 0x6d, 0x117, 0x6a, 0x61, 0x50, 0x61, 0x67, 0x6f, 0x20,
0x50, 0x61, 0x67, 0x61, 0x73, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e,
0x6f, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x117,
0x50, 0x6f, 0x72, 0x74, 0x20, 0x4d, 0x6f, 0x72, 0x73, 0x62, 0x69, 0x73,
0x53, 0x61, 0x69, 0x70, 0x61, 0x6e, 0x61, 0x73, 0x54, 0x61, 0x69, 0x74,
0x69, 0x73, 0x56, 0x65, 0x69, 0x6b, 0x6f, 0x20, 0x73, 0x61, 0x6c, 0x61,
0x56, 0x6f, 0x6c, 0x69, 0x73, 0x61, 0x73, 0x4b, 0x61, 0x69, 0x6d, 0x61,
0x6e, 0x69, 0x73, 0x6b, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x4d, 0x65,
0x78, 0x69, 0x6b, 0x6f, 0x2d, 0x6d, 0x11b, 0x73, 0x74, 0x6f, 0x53, 0x74,
0x2e, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x44, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x48, 0x6f, 0x2d, 0x43, 0x68, 0x69, 0x2d, 0x4d, 0x69,
0x6e, 0x68, 0x2d, 0x6d, 0x11b, 0x73, 0x74, 0x6f, 0x50, 0x6f, 0x72, 0x74,
0x20, 0x4e, 0x75, 0x6d, 0x62, 0x61, 0x79, 0x4e, 0x6f, 0x77, 0x6f, 0x6b,
0x75, 0x7a, 0x6e, 0x6a, 0x65, 0x74, 0x73, 0x6b, 0x4a, 0x65, 0x72, 0x65,
0x77, 0x61, 0x6e, 0x41, 0x63, 0x6f, 0x72, 0x79, 0x46, 0xe4, 0x72, 0xf6,
0x6a, 0x65, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x64, 0x6e, 0x6a, 0x6f,
0x77, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x61,
0x42, 0x11b, 0x142, 0x6f, 0x67, 0x72, 0x6f, 0x64, 0x4b, 0x69, 0x161, 0x69,
0x6e, 0x61, 0x75, 0x57, 0x69, 0x6c, 0x6e, 0x61, 0x57, 0x61, 0x72, 0x161,
0x61, 0x77, 0x61, 0x47, 0xf3, 0x64, 0x6f, 0x77, 0x6e, 0x65, 0x20, 0x6b,
0x75, 0x70, 0x79, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x77, 0x79, 0x4a,
0x61, 0x74, 0x161, 0x6f, 0x77, 0x6e, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79,
0x4b, 0x61, 0x6d, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x72, 0x67, 0x4b,
0x61, 0x69, 0x6d, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x65,
0x72, 0x67, 0x45, 0x72, 0x69, 0x76, 0x61, 0x6e, 0x4b, 0x61, 0x6e, 0x61,
0x72, 0x65, 0x73, 0x63, 0x68, 0x20, 0x49, 0x6e, 0x73, 0x65, 0x6c, 0x65,
0x6e, 0x42, 0x72, 0xe9, 0x69, 0x73, 0x73, 0x65, 0x6c, 0x4b, 0x69, 0x73,
0x63, 0x68, 0x69, 0x6e, 0x61, 0x75, 0x4c, 0xeb, 0x74, 0x7a, 0x65, 0x62,
0x75, 0x65, 0x72, 0x67, 0x52, 0x6f, 0x75, 0x6d, 0x5a, 0xfc, 0x72, 0x65,
0x63, 0x68, 0x43, 0x68, 0x72, 0xeb, 0x73, 0x63, 0x68, 0x74, 0x64, 0x61,
0x67, 0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x4f, 0x75, 0x73, 0x63, 0x68,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x411, 0x430, 0x43d, 0x45f,
0x443, 0x43b, 0x411, 0x43b, 0x430, 0x43d, 0x442, 0x430, 0x458, 0x440, 0x41b, 0x438,
0x431, 0x440, 0x432, 0x438, 0x43b, 0x421, 0x430, 0x443, 0x43d, 0x20, 0x422, 0x43e,
0x43c, 0x435, 0x410, 0x43d, 0x442, 0x438, 0x433, 0x430, 0x423, 0x448, 0x443, 0x430,
0x458, 0x430, 0x41a, 0x43e, 0x440, 0x430, 0x43b, 0x20, 0x425, 0x430, 0x440, 0x431,
0x43e, 0x440, 0x411, 0x430, 0x445, 0x438, 0x430, 0x411, 0x430, 0x445, 0x438, 0x458,
0x430, 0x20, 0x434, 0x435, 0x20, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430,
0x441, 0x41a, 0x430, 0x458, 0x435, 0x43d, 0x430, 0x41a, 0x430, 0x458, 0x43c, 0x430,
0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x421, 0x438, 0x443, 0x434, 0x430, 0x434, 0x20, 0x425, 0x443, 0x430, 0x440, 0x435,
0x437, 0x41a, 0x443, 0x458, 0x430, 0x431, 0x430, 0x414, 0x430, 0x43d, 0x43c, 0x430,
0x440, 0x43a, 0x448, 0x430, 0x43d, 0x415, 0x43b, 0x20, 0x421, 0x430, 0x43b, 0x432,
0x430, 0x434, 0x43e, 0x440, 0x413, 0x432, 0x430, 0x458, 0x430, 0x43d, 0x430, 0x415,
0x440, 0x43c, 0x43e, 0x441, 0x438, 0x459, 0x43e, 0x418, 0x43d, 0x434, 0x438, 0x458,
0x430, 0x43d, 0x430, 0x43f, 0x43e, 0x43b, 0x438, 0x441, 0x41d, 0x43e, 0x43a, 0x441,
0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x41f, 0x438,
0x442, 0x435, 0x440, 0x441, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x418, 0x43d,
0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x432, 0x435, 0x458, 0x2c,
0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412, 0x435, 0x43d,
0x441, 0x435, 0x43d, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d,
0x430, 0x41a, 0x440, 0x430, 0x43b, 0x435, 0x43d, 0x434, 0x430, 0x458, 0x43a, 0x41b,
0x43e, 0x441, 0x20, 0x410, 0x43d, 0x45f, 0x435, 0x43b, 0x435, 0x441, 0x414, 0x43e,
0x43b, 0x435, 0x43d, 0x20, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x20, 0x43a, 0x432,
0x43e, 0x440, 0x442, 0x440, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x430, 0x43d, 0x411,
0x458, 0x443, 0x43b, 0x430, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x40a, 0x443, 0x20, 0x421,
0x430, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41f, 0x43e, 0x440, 0x442,
0x20, 0x43e, 0x20, 0x41f, 0x440, 0x435, 0x43d, 0x441, 0x41f, 0x43e, 0x440, 0x442,
0x20, 0x43e, 0x432, 0x20, 0x421, 0x43f, 0x435, 0x458, 0x43d, 0x420, 0x435, 0x43d,
0x43a, 0x438, 0x43d, 0x20, 0x418, 0x43d, 0x43b, 0x435, 0x442, 0x420, 0x435, 0x45f,
0x430, 0x458, 0x43d, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x411, 0x430, 0x440,
0x442, 0x43e, 0x43b, 0x43e, 0x43c, 0x435, 0x458, 0x421, 0x435, 0x43d, 0x442, 0x20,
0x40f, 0x43e, 0x43d, 0x441, 0x421, 0x432, 0x435, 0x442, 0x438, 0x20, 0x41a, 0x438,
0x442, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x41b, 0x443, 0x441, 0x438, 0x458,
0x430, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x422, 0x43e, 0x43c, 0x430, 0x441, 0x41a,
0x435, 0x458, 0x441, 0x438, 0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x414, 0x438,
0x440, 0x432, 0x438, 0x43b, 0x41c, 0x430, 0x43a, 0x432, 0x43e, 0x440, 0x438, 0x421,
0x430, 0x458, 0x43e, 0x432, 0x430, 0x41b, 0x43e, 0x43d, 0x433, 0x458, 0x435, 0x440,
0x431, 0x438, 0x458, 0x435, 0x43d, 0x410, 0x448, 0x43a, 0x430, 0x431, 0x430, 0x434,
0x415, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x438, 0x43c, 0x41f, 0x43e, 0x43d, 0x442,
0x438, 0x458, 0x430, 0x43d, 0x430, 0x43a, 0x421, 0x430, 0x43a, 0x430, 0x43b, 0x438,
0x43d, 0x412, 0x438, 0x435, 0x43d, 0x442, 0x438, 0x458, 0x430, 0x43d, 0x410, 0x437,
0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x438, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x438, 0x41a, 0x430, 0x43d, 0x430,
0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x424, 0x430, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x438, 0x408, 0x443, 0x43a, 0x43b, 0x430, 0x411, 0x438, 0x437, 0x438, 0x43d,
0x433, 0x435, 0x43d, 0x41a, 0x438, 0x448, 0x438, 0x43d, 0x430, 0x443, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x20, 0x41c, 0x430, 0x43d, 0x40f, 0x435, 0x440, 0x437,
0x438, 0x41c, 0x430, 0x440, 0x438, 0x458, 0x435, 0x445, 0x430, 0x43c, 0x435, 0x43d,
0x421, 0x43a, 0x43e, 0x43f, 0x458, 0x435, 0x423, 0x43b, 0x458, 0x430, 0x43d, 0x43e,
0x432, 0x441, 0x43a, 0x412, 0x438, 0x43b, 0x43d, 0x443, 0x441, 0x411, 0x43e, 0x436,
0x438, 0x45c, 0x435, 0x43d, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x41a,
0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x438, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x443, 0x441,
0x420, 0x435, 0x458, 0x443, 0x43d, 0x438, 0x43e, 0x43d, 0x411, 0x443, 0x433, 0x430,
0x43d, 0x432, 0x438, 0x43b, 0x427, 0x443, 0x43a, 0x412, 0x435, 0x43b, 0x438, 0x433,
0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x413, 0x432, 0x430, 0x43c, 0x41a, 0x432, 0x430, 0x45f, 0x430, 0x43b, 0x435,
0x458, 0x43d, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x441, 0x43a, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440,
0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x458, 0x905, 0x92c, 0x93f, 0x91c, 0x93e, 0x928,
0x905, 0x915, 0x930, 0x93e, 0x905, 0x926, 0x93f, 0x938, 0x20, 0x905, 0x92c, 0x93e,
0x92c, 0x93e, 0x905, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x92c, 0x92e, 0x93e, 0x915,
0x94b, 0x92c, 0x902, 0x917, 0x941, 0x908, 0x92c, 0x902, 0x91c, 0x941, 0x932, 0x92c,
0x94d, 0x932, 0x947, 0x902, 0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d, 0x930, 0x947,
0x91c, 0x93e, 0x935, 0x93f, 0x932, 0x915, 0x93e, 0x939, 0x93f, 0x930, 0x93e, 0x938,
0x93f, 0x909, 0x924, 0x93e, 0x921, 0x93e, 0x930, 0x947, 0x20, 0x938, 0x932, 0x93e,
0x92e, 0x921, 0x93e, 0x909, 0x932, 0x93e, 0x90f, 0x932, 0x20, 0x910, 0x92f, 0x942,
0x928, 0x917, 0x947, 0x92c, 0x94b, 0x930, 0x94b, 0x928, 0x91c, 0x94b, 0x939, 0x93e,
0x928, 0x93f, 0x938, 0x92c, 0x930, 0x94d, 0x917, 0x915, 0x92e, 0x92a, 0x93e, 0x932,
0x93e, 0x916, 0x93e, 0x930, 0x924, 0x94c, 0x92e, 0x92e, 0x92a, 0x942, 0x924, 0x94b,
0x92e, 0x938, 0x948, 0x930, 0x942, 0x92e, 0x947, 0x902, 0x92c, 0x93e, 0x92c, 0x93e,
0x928, 0x947, 0x928, 0x947, 0x91c, 0x947, 0x92e, 0x940, 0x928, 0x93e, 0x928, 0x93f,
0x92f, 0x93e, 0x92e, 0x947, 0x92f, 0x928, 0x94c, 0x915, 0x91a, 0x94b, 0x91f, 0x914,
0x917, 0x93e, 0x921, 0x941, 0x917, 0x942, 0x92c, 0x93f, 0x902, 0x921, 0x94b, 0x90f,
0x915, 0x905, 0x926, 0x915, 0x90f, 0x902, 0x915, 0x94b, 0x930, 0x947, 0x91c, 0x90f,
0x902, 0x91f, 0x93f, 0x917, 0x941, 0x906, 0x905, 0x930, 0x93e, 0x917, 0x941, 0x90f,
0x928, 0x93e, 0x91c, 0x941, 0x91c, 0x941, 0x92f, 0x92e, 0x947, 0x902, 0x921, 0x94b,
0x91c, 0x93e, 0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x917, 0x948, 0x932, 0x917, 0x938,
0x938, 0x93e, 0x932, 0x91f, 0x93e, 0x938, 0x928, 0x20, 0x91c, 0x941, 0x906, 0x928,
0x938, 0x928, 0x20, 0x932, 0x942, 0x908, 0x938, 0x909, 0x936, 0x941, 0x92f, 0x93e,
0x92f, 0x93e, 0x905, 0x938, 0x941, 0x928, 0x936, 0x93f, 0x92f, 0x928, 0x905, 0x91f,
0x93f, 0x915, 0x94b, 0x915, 0x928, 0x92c, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x92c,
0x902, 0x921, 0x947, 0x930, 0x93e, 0x938, 0x92c, 0x93e, 0x930, 0x92c, 0x921, 0x94b,
0x938, 0x92c, 0x948, 0x932, 0x948, 0x92e, 0x92c, 0x94d, 0x932, 0x948, 0x902, 0x915,
0x2d, 0x938, 0x92c, 0x932, 0x94b, 0x928, 0x92c, 0x94b, 0x906, 0x20, 0x92c, 0x93f,
0x938, 0x94d, 0x91f, 0x93e, 0x92c, 0x94b, 0x92f, 0x91c, 0x940, 0x915, 0x948, 0x92e,
0x94d, 0x92c, 0x94d, 0x930, 0x940, 0x91c, 0x20, 0x92c, 0x947, 0x915, 0x947, 0x92e,
0x94d, 0x92a, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x947, 0x928, 0x921, 0x93e, 0x915,
0x941, 0x92f, 0x93e, 0x92c, 0x93e, 0x915, 0x94d, 0x92f, 0x942, 0x930, 0x93e, 0x938,
0x93e, 0x913, 0x926, 0x92e, 0x930, 0x94d, 0x915, 0x936, 0x928, 0x921, 0x93e, 0x909,
0x938, 0x928, 0x921, 0x93e, 0x909, 0x938, 0x928, 0x20, 0x915, 0x94d, 0x930, 0x940,
0x915, 0x921, 0x947, 0x928, 0x92c, 0x930, 0x921, 0x947, 0x91f, 0x94d, 0x930, 0x93e,
0x92f, 0x91f, 0x90f, 0x921, 0x92e, 0x94b, 0x902, 0x91f, 0x94b, 0x928, 0x908, 0x930,
0x942, 0x928, 0x947, 0x92a, 0x947, 0x92b, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x928,
0x947, 0x932, 0x938, 0x928, 0x92b, 0x94b, 0x930, 0x94d, 0x91f, 0x93e, 0x932, 0x947,
0x91c, 0x93e, 0x917, 0x94d, 0x935, 0x93e, 0x921, 0x947, 0x932, 0x942, 0x92a, 0x917,
0x941, 0x906, 0x92f, 0x93e, 0x915, 0x94d, 0x935, 0x93f, 0x932, 0x939, 0x930, 0x92e,
0x94b, 0x938, 0x93f, 0x932, 0x94b, 0x928, 0x94b, 0x915, 0x94d, 0x938, 0x2c, 0x20,
0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92e, 0x947, 0x930, 0x948,
0x902, 0x917, 0x94b, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x92c, 0x940, 0x92c, 0x947, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f,
0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x928, 0x938, 0x947, 0x902, 0x938, 0x2c, 0x20,
0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92c, 0x93f, 0x928, 0x93e,
0x92e, 0x948, 0x915, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x907, 0x928, 0x941, 0x92c, 0x93f, 0x915, 0x907, 0x915, 0x93e, 0x932, 0x935,
0x93f, 0x91f, 0x91c, 0x941, 0x928, 0x947, 0x909, 0x92e, 0x94b, 0x902, 0x91f, 0x940,
0x91a, 0x947, 0x932, 0x94b, 0x2c, 0x20, 0x915, 0x947, 0x902, 0x91f, 0x941, 0x915,
0x940, 0x915, 0x947, 0x932, 0x947, 0x902, 0x921, 0x93f, 0x91c, 0x94d, 0x915, 0x932,
0x93e, 0x938, 0x20, 0x90f, 0x902, 0x91c, 0x93f, 0x932, 0x93f, 0x938, 0x932, 0x94b,
0x905, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x902, 0x938, 0x947, 0x938, 0x20,
0x915, 0x94d, 0x935, 0x93e, 0x91f, 0x930, 0x92e, 0x938, 0x940, 0x92f, 0x94b, 0x92e,
0x93e, 0x930, 0x940, 0x917, 0x94c, 0x91f, 0x92e, 0x91c, 0x93e, 0x91f, 0x932, 0x93e,
0x928, 0x92e, 0x948, 0x928, 0x94b, 0x92e, 0x93f, 0x928, 0x940, 0x92e, 0x948, 0x930,
0x93f, 0x921, 0x93e, 0x92e, 0x948, 0x91f, 0x932, 0x93e, 0x915, 0x93e, 0x91f, 0x932,
0x93e, 0x92e, 0x948, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x938, 0x93f,
0x91f, 0x940, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x92e,
0x93e, 0x928, 0x94d, 0x915, 0x91f, 0x93e, 0x928, 0x92e, 0x93e, 0x928, 0x91f, 0x947,
0x948, 0x930, 0x940, 0x92e, 0x94b, 0x902, 0x91f, 0x948, 0x935, 0x93f, 0x921, 0x93f,
0x92f, 0x94b, 0x928, 0x938, 0x93e, 0x909, 0x928, 0x94d, 0x92f, 0x942, 0x92f, 0x93e,
0x930, 0x94d, 0x915, 0x92c, 0x93f, 0x92f, 0x942, 0x932, 0x93e, 0x2c, 0x20, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x938, 0x947,
0x902, 0x91f, 0x930, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921,
0x915, 0x94b, 0x91f, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x938, 0x932, 0x947,
0x92e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x915, 0x94b,
0x91f, 0x93e, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x906, 0x909, 0x20, 0x92a,
0x94d, 0x930, 0x93f, 0x902, 0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x906,
0x92b, 0x20, 0x938, 0x94d, 0x92a, 0x947, 0x928, 0x92a, 0x94b, 0x930, 0x94d, 0x91f,
0x94b, 0x20, 0x92c, 0x947, 0x932, 0x94d, 0x939, 0x94b, 0x92a, 0x941, 0x902, 0x91f,
0x93e, 0x20, 0x905, 0x930, 0x947, 0x928, 0x93e, 0x91c, 0x930, 0x93f, 0x938, 0x940,
0x92b, 0x930, 0x948, 0x917, 0x93f, 0x928, 0x93e, 0x930, 0x947, 0x938, 0x93e, 0x932,
0x942, 0x924, 0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x92c, 0x94d, 0x930, 0x947, 0x902,
0x915, 0x94b, 0x938, 0x947, 0x902, 0x91f, 0x94b, 0x20, 0x921, 0x94b, 0x92e, 0x93f,
0x928, 0x917, 0x94b, 0x907, 0x91f, 0x94b, 0x915, 0x94b, 0x930, 0x94d, 0x91f, 0x942,
0x930, 0x92e, 0x93f, 0x91f, 0x938, 0x93f, 0x924, 0x915, 0x93e, 0x938, 0x947, 0x902,
0x91f, 0x20, 0x92c, 0x930, 0x94d, 0x925, 0x947, 0x932, 0x92e, 0x940, 0x938, 0x947,
0x902, 0x91f, 0x20, 0x91c, 0x93e, 0x928, 0x94d, 0x938, 0x938, 0x947, 0x902, 0x91f,
0x20, 0x932, 0x942, 0x936, 0x93f, 0x92f, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20,
0x925, 0x94b, 0x92e, 0x938, 0x938, 0x94d, 0x935, 0x93f, 0x92b, 0x94d, 0x91f, 0x20,
0x915, 0x930, 0x947, 0x902, 0x91f, 0x924, 0x941, 0x932, 0x947, 0x92c, 0x948, 0x902,
0x915, 0x941, 0x92c, 0x930, 0x935, 0x93e, 0x907, 0x91f, 0x939, 0x93e, 0x930, 0x94d,
0x938, 0x92c, 0x93f, 0x928, 0x93f, 0x92a, 0x948, 0x917, 0x92f, 0x915, 0x941, 0x91f,
0x93e, 0x91f, 0x92e, 0x948, 0x915, 0x94d, 0x935, 0x948, 0x930, 0x940, 0x932, 0x93e,
0x928, 0x94d, 0x917, 0x93f, 0x92f, 0x930, 0x92c, 0x947, 0x928, 0x905, 0x926, 0x928,
0x905, 0x932, 0x94d, 0x92e, 0x93e, 0x924, 0x940, 0x90f, 0x928, 0x93e, 0x921, 0x93f,
0x930, 0x905, 0x924, 0x93f, 0x930, 0x93e, 0x909, 0x92c, 0x948, 0x902, 0x915, 0x93e,
0x915, 0x91a, 0x93f, 0x924, 0x93e, 0x915, 0x94b, 0x932, 0x92e, 0x94d, 0x92c, 0x94b,
0x926, 0x92e, 0x93f, 0x936, 0x94d, 0x915, 0x939, 0x948, 0x92c, 0x94d, 0x930, 0x93e,
0x928, 0x939, 0x93e, 0x902, 0x917, 0x20, 0x915, 0x93e, 0x902, 0x917, 0x939, 0x94b,
0x92c, 0x94d, 0x926, 0x907, 0x930, 0x915, 0x941, 0x938, 0x94d, 0x915, 0x91c, 0x947,
0x930, 0x942, 0x938, 0x947, 0x932, 0x92e, 0x915, 0x92e, 0x91a, 0x91f, 0x915, 0x93e,
0x916, 0x93e, 0x902, 0x927, 0x94d, 0x92f, 0x917, 0x93e, 0x915, 0x94d, 0x930, 0x947,
0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x915,
0x941, 0x906, 0x932, 0x93e, 0x932, 0x92e, 0x94d, 0x92a, 0x941, 0x930, 0x915, 0x941,
0x91a, 0x93f, 0x902, 0x917, 0x92e, 0x915, 0x93e, 0x938, 0x93e, 0x930, 0x928, 0x93f,
0x915, 0x94b, 0x936, 0x93f, 0x92f, 0x93e, 0x915, 0x94b, 0x92c, 0x94b, 0x915, 0x941,
0x91c, 0x928, 0x947, 0x91f, 0x94d, 0x938, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x93f,
0x92c, 0x93f, 0x930, 0x938, 0x94d, 0x915, 0x913, 0x92e, 0x938, 0x94d, 0x915, 0x913,
0x930, 0x93e, 0x932, 0x92a, 0x94b, 0x902, 0x91f, 0x93f, 0x92f, 0x93e, 0x902, 0x915,
0x915, 0x94b, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947, 0x915, 0x93f, 0x91c, 0x93f,
0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x938, 0x93e, 0x915, 0x932, 0x93f, 0x928,
0x938, 0x93f, 0x92f, 0x94b, 0x932, 0x936, 0x94d, 0x930, 0x947, 0x921, 0x928, 0x947,
0x915, 0x94b, 0x932, 0x93f, 0x92e, 0x93f, 0x938, 0x94d, 0x915, 0x924, 0x93e, 0x907,
0x92a, 0x947, 0x908, 0x91f, 0x92c, 0x93f, 0x932, 0x93f, 0x938, 0x93f, 0x925, 0x93f,
0x92e, 0x94d, 0x92a, 0x942, 0x91f, 0x94b, 0x92e, 0x94d, 0x938, 0x909, 0x932, 0x928,
0x92c, 0x91f, 0x94b, 0x930, 0x909, 0x930, 0x942, 0x92e, 0x915, 0x940, 0x909, 0x938,
0x94d, 0x924, 0x2d, 0x928, 0x947, 0x930, 0x93e, 0x92c, 0x93f, 0x92f, 0x902, 0x91f,
0x93f, 0x92f, 0x93e, 0x928, 0x92c, 0x94d, 0x932, 0x93e, 0x921, 0x93f, 0x92c, 0x94b,
0x938, 0x94d, 0x924, 0x94b, 0x915, 0x92f, 0x941, 0x915, 0x941, 0x924, 0x94d, 0x938,
0x94d, 0x915, 0x92f, 0x93e, 0x902, 0x917, 0x941, 0x928, 0x92f, 0x948, 0x915, 0x947,
0x91f, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x92f, 0x948, 0x930,
0x947, 0x92c, 0x928, 0x90f, 0x91c, 0x94b, 0x930, 0x94d, 0x938, 0x915, 0x947, 0x92a,
0x20, 0x92c, 0x930, 0x94d, 0x921, 0x92b, 0x930, 0x93e, 0x913, 0x92e, 0x926, 0x948,
0x930, 0x93e, 0x930, 0x93f, 0x915, 0x91c, 0x947, 0x935, 0x93f, 0x915, 0x926, 0x915,
0x94d, 0x937, 0x93f, 0x923, 0x20, 0x91c, 0x93e, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x92c, 0x94d, 0x930, 0x93f, 0x938, 0x92c, 0x947, 0x928, 0x92c, 0x94d, 0x930,
0x94b, 0x915, 0x947, 0x928, 0x20, 0x939, 0x93f, 0x932, 0x921, 0x93e, 0x930, 0x94d,
0x92c, 0x93f, 0x928, 0x907, 0x92f, 0x942, 0x915, 0x932, 0x93e, 0x939, 0x94b, 0x92c,
0x930, 0x94d, 0x91f, 0x932, 0x93f, 0x902, 0x921, 0x92e, 0x948, 0x928, 0x932, 0x93e,
0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x92c, 0x947, 0x90f, 0x92e, 0x94d, 0x938,
0x91f, 0x930, 0x94d, 0x921, 0x92e, 0x90f, 0x902, 0x921, 0x94b, 0x930, 0x93e, 0x905,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x93e, 0x915, 0x93e, 0x928, 0x92c, 0x93e, 0x930,
0x93f, 0x924, 0x938, 0x94d, 0x932, 0x93e, 0x935, 0x93e, 0x92c, 0x94d, 0x930, 0x938,
0x947, 0x932, 0x94d, 0x938, 0x92c, 0x938, 0x93f, 0x902, 0x917, 0x947, 0x928, 0x91a,
0x93f, 0x938, 0x93f, 0x928, 0x93e, 0x90a, 0x915, 0x94b, 0x92a, 0x947, 0x928, 0x939,
0x947, 0x917, 0x947, 0x928, 0x917, 0x930, 0x94d, 0x928, 0x91c, 0x940, 0x939, 0x948,
0x932, 0x938, 0x93f, 0x902, 0x915, 0x940, 0x906, 0x907, 0x91c, 0x932, 0x20, 0x906,
0x92b, 0x20, 0x92e, 0x948, 0x928, 0x907, 0x938, 0x94d, 0x924, 0x93e, 0x92e, 0x94d,
0x92c, 0x941, 0x932, 0x915, 0x932, 0x93f, 0x928, 0x93f, 0x928, 0x917, 0x94d, 0x930,
0x947, 0x921, 0x932, 0x93f, 0x938, 0x92c, 0x928, 0x932, 0x941, 0x92c, 0x932, 0x93f,
0x92f, 0x93e, 0x928, 0x93e, 0x932, 0x917, 0x94d, 0x91c, 0x92e, 0x92c, 0x930, 0x94d,
0x917, 0x92e, 0x947, 0x921, 0x94d, 0x930, 0x93f, 0x921, 0x92e, 0x93e, 0x932, 0x91f,
0x93e, 0x92e, 0x947, 0x930, 0x93f, 0x92f, 0x947, 0x939, 0x92e, 0x92e, 0x93f, 0x928,
0x94d, 0x938, 0x915, 0x92e, 0x94b, 0x928, 0x947, 0x915, 0x94b, 0x92e, 0x93e, 0x938,
0x94d, 0x915, 0x94b, 0x92a, 0x94b, 0x921, 0x917, 0x94b, 0x930, 0x93f, 0x936, 0x93e,
0x938, 0x948, 0x928, 0x20, 0x92e, 0x930, 0x940, 0x928, 0x94b, 0x938, 0x930, 0x93e,
0x91c, 0x947, 0x92c, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x924, 0x94b, 0x92c, 0x938,
0x93f, 0x92e, 0x94d, 0x92b, 0x947, 0x930, 0x94b, 0x92a, 0x94b, 0x932, 0x938, 0x94d,
0x915, 0x94b, 0x92a, 0x92f, 0x947, 0x938, 0x94d, 0x91f, 0x93e, 0x915, 0x939, 0x94b,
0x92e, 0x91f, 0x93e, 0x932, 0x93f, 0x928, 0x91f, 0x93f, 0x930, 0x93e, 0x928, 0x947,
0x909, 0x932, 0x94d, 0x92f, 0x947, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x94d, 0x915,
0x92c, 0x947, 0x926, 0x941, 0x91c, 0x92c, 0x947, 0x91f, 0x93f, 0x915, 0x928, 0x92c,
0x93f, 0x90f, 0x928, 0x93e, 0x92c, 0x93f, 0x932, 0x928, 0x93f, 0x909, 0x938, 0x92c,
0x94b, 0x932, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x92c, 0x93e, 0x930,
0x938, 0x93e, 0x91c, 0x917, 0x930, 0x948, 0x92c, 0x91c, 0x94d, 0x92f, 0x942, 0x930,
0x93f, 0x916, 0x905, 0x902, 0x924, 0x93e, 0x928, 0x93e, 0x928, 0x947, 0x930, 0x93f,
0x935, 0x94b, 0x91a, 0x917, 0x94b, 0x938, 0x915, 0x948, 0x930, 0x917, 0x941, 0x90f,
0x932, 0x947, 0x928, 0x92e, 0x93e, 0x930, 0x940, 0x936, 0x938, 0x92e, 0x92f, 0x94b,
0x924, 0x947, 0x930, 0x93f, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928, 0x906, 0x915,
0x932, 0x948, 0x902, 0x921, 0x91a, 0x947, 0x925, 0x92e, 0x917, 0x948, 0x932, 0x93e,
0x92a, 0x948, 0x917, 0x94b, 0x938, 0x928, 0x93e, 0x930, 0x92b, 0x93e, 0x915, 0xd05,
0xd2c, 0xd3f, 0xd26, 0xd4d, 0x200c, 0xd1c, 0xd3e, 0xd7b, 0x200c, 0xd06, 0xd15, 0xd4d,
0xd30, 0xd05, 0xd21, 0xd3f, 0xd38, 0xd4d, 0x20, 0xd05, 0xd2c, 0xd3e, 0xd2c, 0xd05,
0xd7e, 0xd1c, 0xd3f, 0xd2f, 0xd47, 0xd34, 0xd4d, 0x200c, 0xd38, 0xd4d, 0xd05, 0xd38,
0xd4d, 0x200d, 0xd2e, 0xd3e, 0xd30, 0xd2c, 0xd2e, 0xd3e, 0xd15, 0xd4b, 0xd2c, 0xd02,
0xd17, 0xd41, 0xd2f, 0xd3f, 0xd2c, 0xd1e, 0xd4d, 0xd1a, 0xd41, 0xd7d, 0xd2c, 0xd3f,
0xd38, 0xd4d, 0xd38, 0xd3e, 0xd35, 0xd41, 0xd2c, 0xd4d, 0xd32, 0xd3e, 0xd23, 0xd4d,
0xd1f, 0xd46, 0xd2f, 0xd7c, 0x200c, 0xd2c, 0xd4d, 0xd30, 0xd3e, 0xd38, 0xd35, 0xd3f,
0xd7d, 0xd2c, 0xd41, 0xd1c, 0xd41, 0xd02, 0x200c, 0xd2c, 0xd41, 0xd30, 0xd15, 0xd46,
0xd2f, 0xd4d, 0x200c, 0xd31, 0xd4b, 0xd15, 0xd3e, 0xd38, 0xd2c, 0xd4d, 0xd32, 0xd3e,
0xd19, 0xd4d, 0xd15, 0xd15, 0xd4d, 0xd2f, 0xd42, 0xd1f, 0xd4d, 0xd1f, 0xd15, 0xd4a,
0xd23, 0xd3e, 0xd15, 0xd4d, 0xd30, 0xd3f, 0xd21, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd7c,
0x200c, 0xd26, 0xd3e, 0xd7c, 0x20, 0xd0e, 0xd38, 0xd4d, 0x20, 0xd38, 0xd32, 0xd3e,
0xd02, 0xd26, 0xd3f, 0xd1c, 0xd3f, 0xd2c, 0xd57, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd21,
0xd57, 0xd32, 0xd0e, 0xd7d, 0x200c, 0x20, 0xd10, 0xd2f, 0xd41, 0xd7b, 0x200c, 0xd2b,
0xd4d, 0xd30, 0xd40, 0xd1f, 0xd57, 0xd7a, 0xd17, 0xd3e, 0xd2c, 0xd31, 0xd4b, 0xd7a,
0xd39, 0xd30, 0xd3e, 0xd30, 0xd46, 0xd1c, 0xd4b, 0xd39, 0xd28, 0xd4d, 0xd28, 0xd3e,
0xd38, 0xd4d, 0xd2c, 0xd7c, 0x200c, 0xd17, 0xd4d, 0xd1c, 0xd41, 0xd2c, 0xd15, 0xd2e,
0xd4d, 0xd2a, 0xd3e, 0xd32, 0xd16, 0xd3e, 0xd7c, 0x200c, 0xd24, 0xd4c, 0xd02, 0xd15,
0xd3f, 0xd17, 0xd3e, 0xd32, 0xd3f, 0xd15, 0xd3f, 0xd7b, 0xd37, 0xd3e, 0xd38, 0xd32,
0xd3e, 0xd17, 0xd4b, 0xd38, 0xd4d, 0xd32, 0xd3f, 0xd2c, 0xd4d, 0xd30, 0xd46, 0xd35,
0xd3f, 0xd32, 0xd4d, 0xd32, 0xd32, 0xd4b, 0xd02, 0xd32, 0xd41, 0xd35, 0xd3e, 0xd7b,
0xd21, 0xd32, 0xd42, 0xd2c, 0xd41, 0xd02, 0xd2c, 0xd3e, 0xd37, 0xd3f, 0xd32, 0xd41,
0xd38, 0xd3e, 0xd15, 0xd2e, 0xd32, 0xd3e, 0xd2c, 0xd4b, 0xd2e, 0xd3e, 0xd2a, 0xd4d,
0xd2f, 0xd41, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0xd2e, 0xd38, 0xd47, 0xd31, 0xd41, 0xd2e,
0xd2c, 0xd3e, 0xd2c, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd7b, 0x200c, 0xd2e, 0xd4a, 0xd17,
0xd3e, 0xd26, 0xd3f, 0xd37, 0xd41, 0xd2e, 0xd7a, 0x200c, 0xd31, 0xd4b, 0xd35, 0xd3f,
0xd2f, 0xd28, 0xd2f, 0xd4d, 0x200c, 0xd31, 0xd4b, 0xd2c, 0xd3f, 0xd1c, 0xd2e, 0xd46,
0xd28, 0xd28, 0xd3f, 0xd2f, 0xd3e, 0xd2e, 0xd3f, 0xd28, 0xd57, 0xd35, 0xd3e, 0xd15,
0xd4d, 0x200c, 0xd37, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd14, 0xd17, 0xd3e, 0xd26,
0xd57, 0xd17, 0xd57, 0xd2a, 0xd4b, 0xd7c, 0x200c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0x2d,
0xd28, 0xd4b, 0xd35, 0xd4b, 0xd38, 0xd3e, 0xd35, 0xd4b, 0x20, 0xd1f, 0xd4b, 0xd02,
0x200c, 0xd1f, 0xd4d, 0xd30, 0xd3f, 0xd2a, 0xd4a, 0xd33, 0xd3f, 0xd1f, 0xd4d, 0xd2f,
0xd42, 0xd23, 0xd3f, 0xd38, 0xd4d, 0xd35, 0xd3f, 0xd7b, 0xd21, 0xd4d, 0x200c, 0xd39,
0xd4b, 0xd15, 0xd4d, 0xd05, 0xd21, 0xd3e, 0xd15, 0xd4d, 0xd06, 0xd19, 0xd4d, 0xd15,
0xd31, 0xd47, 0xd1c, 0xd4d, 0xd06, 0xd7b, 0xd17, 0xd4d, 0xd35, 0xd3f, 0xd32, 0xd4d,
0xd32, 0xd06, 0xd7b, 0xd31, 0xd3f, 0xd17, 0xd4d, 0xd35, 0xd05, 0xd31, 0xd17, 0xd4d,
0xd35, 0xd48, 0xd28, 0xd2c, 0xd4d, 0xd2f, 0xd42, 0xd23, 0xd38, 0xd4d, 0x20, 0xd10,
0xd31, 0xd3f, 0xd38, 0xd4d, 0x200d, 0xd15, 0xd4d, 0xd2f, 0xd3e, 0xd31, 0xd4d, 0xd31,
0xd2e, 0xd3e, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd15, 0xd4b, 0xd7c, 0xd21, 0xd4b, 0xd2c,
0xd1c, 0xd41, 0xd1c, 0xd41, 0xd2f, 0xd4d, 0xd32, 0xd3e, 0x20, 0xd31, 0xd3f, 0xd2f,
0xd4b, 0xd1c, 0xd2e, 0xd46, 0xd7b, 0xd21, 0xd4b, 0xd38, 0xd31, 0xd3f, 0xd2f, 0xd4b,
0x20, 0xd17, 0xd4d, 0xd2f, 0xd3e, 0xd32, 0xd17, 0xd4b, 0xd38, 0xd4d, 0xd38, 0xd3e,
0xd7d, 0xd1f, 0xd4d, 0xd1f, 0xd38, 0xd3e, 0xd7b, 0x20, 0xd1c, 0xd41, 0xd35, 0xd3e,
0xd7b, 0xd38, 0xd3e, 0xd7b, 0x20, 0xd32, 0xd42, 0xd2f, 0xd3f, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd41, 0xd15, 0xd41, 0xd2e, 0xd3e, 0xd7b, 0xd09, 0xd37, 0xd3f, 0xd2f,
0xd05, 0xd31, 0xd42, 0xd2c, 0xd05, 0xd38, 0xd7b, 0x200c, 0xd37, 0xd4d, 0xd2f, 0xd7b,
0x200c, 0xd0f, 0xd31, 0xd4d, 0xd31, 0xd3f, 0xd15, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd7a,
0xd2c, 0xd39, 0xd3f, 0xd2f, 0xd2c, 0xd39, 0xd3f, 0xd2f, 0x20, 0xd2c, 0xd7b, 0xd21,
0xd3e, 0xd30, 0xd38, 0xd4d, 0xd2c, 0xd3e, 0xd7c, 0xd2c, 0xd21, 0xd4b, 0xd38, 0xd4d,
0xd2c, 0xd46, 0xd32, 0xd02, 0xd2c, 0xd46, 0xd32, 0xd40, 0xd38, 0xd4d, 0xd2c, 0xd4d,
0xd32, 0xd3e, 0xd19, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd3e, 0xd2c, 0xd4d, 0xd32,
0xd4b, 0xd7a, 0xd2c, 0xd4b, 0xd35, 0x20, 0xd35, 0xd3f, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd2c, 0xd4a, 0xd17, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd2c, 0xd4a, 0xd2f, 0xd4d,
0xd38, 0xd3f, 0xd15, 0xd47, 0xd02, 0xd2c, 0xd4d, 0xd30, 0xd3f, 0xd21, 0xd4d, 0x200c,
0xd1c, 0xd4d, 0x20, 0xd2c, 0xd47, 0xd15, 0xd4d, 0xd2f, 0xd3e, 0xd2e, 0xd4d, 0xd2a,
0xd4b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd7b, 0xd21, 0xd46, 0xd15, 0xd3e, 0xd7b,
0xd15, 0xd42, 0xd7a, 0xd15, 0xd30, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd38, 0xd4d, 0xd15,
0xd2f, 0xd40, 0xd7b, 0x200c, 0xd15, 0xd47, 0xd2e, 0xd3e, 0xd7b, 0xd1a, 0xd3f, 0xd15,
0xd4d, 0xd15, 0xd3e, 0xd17, 0xd4b, 0xd1a, 0xd3f, 0xd39, 0xd4d, 0xd35, 0xd3e, 0xd39,
0xd38, 0xd3f, 0xd2f, 0xd41, 0xd21, 0xd3e, 0xd21, 0xd4d, 0x20, 0xd39, 0xd41, 0xd35,
0xd3e, 0xd30, 0xd38, 0xd4d, 0xd15, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0x20,
0xd31, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd15, 0xd4d, 0xd30, 0xd46, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd7a, 0xd15, 0xd41, 0xd2f, 0xd4d, 0x200c, 0xd2c, 0xd15, 0xd41, 0xd31,
0xd3e, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd35, 0xd4b, 0xd21, 0xd3e, 0xd7b, 0xd2e, 0xd3e,
0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd37, 0xd3e, 0xd35, 0xd4d, 0xd7b, 0xd21, 0xd4b,
0xd35, 0xd4d, 0xd38, 0xd7a, 0xd21, 0xd4b, 0xd35, 0xd4d, 0xd38, 0xd7a, 0x20, 0xd15,
0xd4d, 0xd30, 0xd40, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd21, 0xd46, 0xd7b, 0x200c, 0xd35,
0xd7c, 0xd21, 0xd46, 0xd1f, 0xd4d, 0xd30, 0xd4b, 0xd2f, 0xd3f, 0xd31, 0xd4d, 0xd31,
0xd4d, 0xd21, 0xd4a, 0xd2e, 0xd3f, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd0e, 0xd21,
0xd4d, 0xd2e, 0xd4b, 0xd7a, 0xd1f, 0xd7a, 0xd2f, 0xd46, 0xd31, 0xd41, 0xd28, 0xd40,
0xd2a, 0xd4d, 0xd2a, 0xd46, 0xd0e, 0xd7d, 0x20, 0xd38, 0xd3e, 0xd7d, 0x200c, 0xd35,
0xd26, 0xd4b, 0xd7c, 0xd2b, 0xd4b, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0x20, 0xd28,
0xd46, 0xd7d, 0xd38, 0xd7a, 0xd2b, 0xd4b, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd32, 0xd40,
0xd38, 0xd17, 0xd4d, 0xd32, 0xd46, 0xd2f, 0xd4d, 0xd38, 0xd4d, 0x20, 0xd2c, 0xd47,
0xd17, 0xd42, 0xd38, 0xd4d, 0x20, 0xd2c, 0xd47, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd7b,
0xd21, 0xd4d, 0x20, 0xd1f, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd17, 0xd4d, 0xd30,
0xd28, 0xd47, 0xd21, 0xd17, 0xd4d, 0xd35, 0xd3e, 0xd21, 0xd32, 0xd42, 0xd2a, 0xd4d,
0xd2a, 0xd4d, 0xd17, 0xd4d, 0xd35, 0xd3e, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd2e, 0xd3e,
0xd32, 0xd17, 0xd41, 0xd35, 0xd3e, 0xd2f, 0xd15, 0xd4d, 0xd35, 0xd3f, 0xd7d, 0xd17,
0xd2f, 0xd3e, 0xd28, 0xd39, 0xd3e, 0xd32, 0xd3f, 0xd2b, 0xd3e, 0xd15, 0xd4d, 0xd38,
0xd4d, 0xd39, 0xd35, 0xd3e, 0xd28, 0xd39, 0xd46, 0xd7c, 0xd2e, 0xd4b, 0xd38, 0xd3f,
0xd32, 0xd4d, 0xd32, 0xd4b, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd3e,
0xd2a, 0xd4b, 0xd32, 0xd40, 0xd38, 0xd4d, 0xd28, 0xd4b, 0xd15, 0xd4d, 0xd38, 0xd4d,
0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd2e, 0xd30, 0xd46,
0xd19, 0xd4d, 0xd15, 0xd4b, 0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e,
0xd28, 0xd2a, 0xd40, 0xd31, 0xd4d, 0xd31, 0xd47, 0xd34, 0xd4d, 0xd38, 0xd4d, 0xd2c,
0xd7c, 0xd17, 0xd4d, 0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28,
0xd31, 0xd4d, 0xd31, 0xd46, 0xd7d, 0x20, 0xd38, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd3f,
0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd35, 0xd3f, 0xd35,
0xd46, 0xd2f, 0xd4d, 0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28,
0xd35, 0xd3f, 0xd7b, 0xd38, 0xd46, 0xd7b, 0xd38, 0xd4d, 0x2c, 0x20, 0xd07, 0xd7b,
0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd35, 0xd3f, 0xd28, 0xd3e, 0xd2e, 0xd3e, 0xd15,
0xd4d, 0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd07, 0xd28,
0xd41, 0xd35, 0xd3f, 0xd15, 0xd4d, 0xd07, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd24, 0xd4d,
0xd1c, 0xd2e, 0xd48, 0xd15, 0xd4d, 0xd15, 0xd1c, 0xd42, 0xd28, 0xd4b, 0xd32, 0xd42,
0xd2f, 0xd3f, 0xd38, 0xd4d, 0x200c, 0xd35, 0xd3f, 0xd32, 0xd4d, 0xd32, 0xd46, 0xd2e,
0xd4b, 0xd23, 0xd4d, 0xd1f, 0xd3f, 0xd38, 0xd46, 0xd32, 0xd4d, 0xd32, 0xd4b, 0x2c,
0x20, 0xd15, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd15, 0xd3e,
0xd7c, 0xd32, 0xd7b, 0xd21, 0xd3f, 0xd1c, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd32,
0xd3e, 0x20, 0xd2a, 0xd3e, 0xd38, 0xd4d, 0xd32, 0xd3f, 0xd2e, 0xd32, 0xd4b, 0xd38,
0xd4d, 0x20, 0xd0e, 0xd2f, 0xd4d, 0xd1e, 0xd4d, 0xd1a, 0xd32, 0xd38, 0xd4d, 0xd32,
0xd4b, 0xd35, 0xd7c, 0x20, 0xd2a, 0xd4d, 0xd30, 0xd3f, 0xd7b, 0xd38, 0xd38, 0xd4d,
0x20, 0xd15, 0xd4d, 0xd35, 0xd3e, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd7c, 0xd2e, 0xd3e,
0xd38, 0xd3f, 0xd2f, 0xd4b, 0xd2e, 0xd28, 0xd3e, 0xd17, 0xd4d, 0xd35, 0xd2e, 0xd28,
0xd57, 0xd38, 0xd4d, 0xd2e, 0xd3e, 0xd30, 0xd3f, 0xd17, 0xd4b, 0xd2e, 0xd3e, 0xd7c,
0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd2e, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd2e, 0xd4a, 0xd31, 0xd4b, 0xd38, 0xd4d, 0xd2e, 0xd38, 0xd31, 0xd4d, 0xd31,
0xd4d, 0x200c, 0xd32, 0xd3e, 0xd7b, 0xd2e, 0xd46, 0xd28, 0xd4b, 0xd2e, 0xd3f, 0xd28,
0xd40, 0xd2e, 0xd46, 0xd30, 0xd3f, 0xd21, 0xd2e, 0xd46, 0xd21, 0xd4d, 0x200c, 0xd32,
0xd15, 0xd1f, 0xd4d, 0x200c, 0xd32, 0xd46, 0xd2e, 0xd46, 0xd15, 0xd4d, 0xd38, 0xd3f,
0xd15, 0xd4d, 0xd15, 0xd4b, 0x20, 0xd38, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd3f, 0xd2e,
0xd3f, 0xd15, 0xd4d, 0xd35, 0xd32, 0xd7b, 0xd2e, 0xd4b, 0xd02, 0xd17, 0xd4d, 0x200c,
0xd1f, 0xd7b, 0xd2e, 0xd4b, 0xd23, 0xd4d, 0xd1f, 0xd46, 0xd31, 0xd3f, 0xd2e, 0xd4a,
0xd23, 0xd4d, 0xd1f, 0xd46, 0x200c, 0xd35, 0xd40, 0xd21, 0xd3f, 0xd2f, 0xd4b, 0xd2e,
0xd4a, 0xd23, 0xd4d, 0xd1f, 0xd46, 0xd38, 0xd30, 0xd24, 0xd4d, 0xd24, 0xd4d, 0xd28,
0xd3e, 0xd38, 0xd57, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd2f, 0xd4b, 0xd7c, 0xd15, 0xd4d,
0xd15, 0xd4d, 0xd28, 0xd4b, 0xd02, 0xd28, 0xd4a, 0xd31, 0xd4b, 0xd28, 0xd2c, 0xd4d,
0xd2f, 0xd42, 0xd32, 0x2c, 0x20, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd21, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd38, 0xd46, 0xd28,
0xd4d, 0xd31, 0xd7c, 0x2c, 0x20, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd21, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd28, 0xd4d, 0xd2f,
0xd42, 0x20, 0xd38, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd32, 0xd02, 0x2c, 0x20, 0xd35,
0xd1f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd21, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd4b,
0xd1f, 0xd4d, 0xd1f, 0xd28, 0xd42, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd12, 0xd1c, 0xd3f,
0xd7b, 0xd17, 0xd2a, 0xd28, 0xd3e, 0xd2e, 0xd2a, 0xd30, 0xd3e, 0xd2e, 0xd3e, 0xd30,
0xd3f, 0xd2c, 0xd4b, 0xd2b, 0xd40, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd38, 0xd4d, 0xd2a,
0xd4b, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0xd2a, 0xd4d, 0xd30, 0xd3f, 0xd7b, 0xd38,
0xd4d, 0xd2a, 0xd4b, 0xd7c, 0x200c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0x20, 0xd13, 0xd2b,
0xd4d, 0x20, 0xd38, 0xd4d, 0x200c, 0xd2a, 0xd46, 0xd2f, 0xd3f, 0xd7b, 0x200c, 0xd2a,
0xd4b, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0x20, 0xd35, 0xd46, 0xd32, 0xd4d, 0xd32,
0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd42, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0x20, 0xd31,
0xd3f, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd2a, 0xd41, 0xd28, 0xd4d, 0xd31, 0x20, 0xd05,
0xd30, 0xd40, 0xd28, 0xd38, 0xd4d, 0xd31, 0xd3e, 0xd19, 0xd4d, 0xd15, 0xd3f, 0xd7b,
0x20, 0xd07, 0xd7b, 0xd32, 0xd46, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd46, 0xd38,
0xd40, 0xd2b, 0xd46, 0xd31, 0xd3f, 0xd1c, 0xd40, 0xd28, 0xd31, 0xd46, 0xd38, 0xd32,
0xd4d, 0xd2f, 0xd42, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd31, 0xd3f, 0xd2f, 0xd4b, 0x20,
0xd2c, 0xd4d, 0xd30, 0xd3e, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd38, 0xd3e, 0xd28, 0xd4d,
0xd31, 0xd31, 0xd46, 0xd02, 0xd38, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd3f, 0xd2f, 0xd3e,
0xd17, 0xd4b, 0xd38, 0xd3e, 0xd28, 0xd4d, 0xd24, 0xd4b, 0x20, 0xd21, 0xd4b, 0xd2e,
0xd3f, 0xd02, 0xd17, 0xd4b, 0xd38, 0xd3e, 0xd35, 0xd4b, 0xd2a, 0xd4b, 0xd33, 0xd4b,
0xd07, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd35, 0xd3e, 0xd16, 0xd4d, 0xd35, 0xd3e, 0xd7c,
0xd1f, 0xd42, 0xd7c, 0xd2e, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd38, 0xd3f, 0xd31,
0xd4d, 0xd31, 0xd4d, 0x200c, 0xd15, 0xd3e, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d,
0x20, 0xd2c, 0xd7c, 0xd24, 0xd4d, 0xd24, 0xd32, 0xd46, 0xd2e, 0xd3f, 0xd38, 0xd46,
0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd1c, 0xd4b, 0xd7a, 0xd38, 0xd4d, 0xd38, 0xd46,
0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd15, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd38,
0xd4d, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd32, 0xd42, 0xd38, 0xd3f,
0xd2f, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd24, 0xd4b, 0xd2e, 0xd38,
0xd4d, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd35, 0xd3f, 0xd7b, 0xd38,
0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0xd38, 0xd4d, 0xd35, 0xd3f, 0xd2b, 0xd4d, 0x200c,
0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd15, 0xd31, 0xd28, 0xd4d, 0xd31, 0xd4d, 0xd1f,
0xd46, 0xd17, 0xd42, 0xd38, 0xd3f, 0xd17, 0xd3e, 0xd7d, 0xd2a, 0xd24, 0xd42, 0xd33,
0xd3f, 0xd24, 0xd3f, 0xd2f, 0xd41, 0xd35, 0xd3e, 0xd28, 0xd1f, 0xd4a, 0xd31, 0xd28,
0xd4d, 0xd31, 0xd4b, 0xd1f, 0xd4b, 0xd7c, 0x200c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0xd33,
0xd35, 0xd3e, 0xd7b, 0x200c, 0xd15, 0xd42, 0xd35, 0xd7c, 0xd35, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0xd39, 0xd4b, 0xd34, 0xd4d, 0xd38, 0xd4d, 0xd35, 0xd3f, 0xd28, 0xd4d,
0xd28, 0xd3f, 0xd2a, 0xd46, 0xd17, 0xd4d, 0xd2f, 0xd3e, 0xd15, 0xd41, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd21, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd35,
0xd3f, 0xd38, 0xd4d, 0xd21, 0xd4d, 0xd2f, 0xd42, 0xd2e, 0xd23, 0xd4d, 0xd1f, 0xd4d,
0x20, 0xd21, 0xd3f, 0x20, 0xd2f, 0xd41, 0xd7c, 0xd35, 0xd3f, 0xd7d, 0xd2e, 0xd15,
0xd4d, 0xd35, 0xd2f, 0xd31, 0xd3f, 0xd2e, 0xd3e, 0xd35, 0xd4d, 0xd38, 0xd7a, 0xd2e,
0xd3e, 0xd15, 0xd4d, 0xd2e, 0xd7c, 0xd21, 0xd4b, 0xd2a, 0xd3e, 0xd2e, 0xd7c, 0xd31,
0xd4a, 0xd24, 0xd40, 0xd31, 0xd38, 0xd4d, 0xd2f, 0xd4b, 0xd35, 0xd1f, 0xd4d, 0xd30,
0xd4b, 0xd7e, 0xd35, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d,
0xd32, 0xd02, 0xd17, 0xd4d, 0x200d, 0xd2f, 0xd46, 0xd7c, 0xd2c, 0xd3f, 0xd7b, 0xd0f,
0xd26, 0xd46, 0xd7b, 0xd05, 0xd7d, 0xd2e, 0xd3e, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd05,
0xd2e, 0xd4d, 0xd2e, 0xd3e, 0xd7b, 0x200c, 0xd05, 0xd28, 0xd3e, 0xd21, 0xd3f, 0xd7c,
0xd05, 0xd15, 0xd4d, 0xd24, 0xd57, 0xd05, 0xd16, 0xd4d, 0x200c, 0xd24, 0xd4b, 0xd2c,
0xd4d, 0xd06, 0xd37, 0xd4d, 0x200c, 0xd17, 0xd3e, 0xd2c, 0xd1f, 0xd4d, 0xd1f, 0xd4d,
0xd05, 0xd31, 0xd4d, 0xd31, 0xd3f, 0xd31, 0xd4b, 0xd2c, 0xd3e, 0xd17, 0xd4d, 0x200c,
0xd26, 0xd3e, 0xd26, 0xd4d, 0xd2c, 0xd39, 0xd4d, 0xd31, 0xd3f, 0xd7b, 0xd2c, 0xd3e,
0xd15, 0xd4d, 0xd15, 0xd41, 0xd2c, 0xd3e, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd15, 0xd4d,
0xd15, 0xd4d, 0xd2c, 0xd7c, 0xd23, 0xd4b, 0xd7d, 0xd2c, 0xd46, 0xd2f, 0xd4d, 0x200c,
0xd31, 0xd42, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd2c, 0xd3f, 0xd37, 0xd4d, 0x200c, 0xd15,
0xd47, 0xd15, 0xd4d, 0xd2c, 0xd4d, 0xd30, 0xd42, 0xd23, 0xd48, 0xd1a, 0xd40, 0xd31,
0xd4d, 0xd31, 0xd15, 0xd4a, 0xd33, 0xd02, 0x200c, 0xd2c, 0xd4b, 0xd26, 0xd2e, 0xd3e,
0xd38, 0xd4d, 0xd15, 0xd38, 0xd4d, 0xd27, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd26, 0xd3f,
0xd32, 0xd3f, 0xd26, 0xd41, 0xd2c, 0xd3e, 0xd2f, 0xd4d, 0xd26, 0xd41, 0xd37, 0xd7b,
0x200c, 0xd2c, 0xd46, 0xd2b, 0xd3e, 0xd2e, 0xd17, 0xd41, 0xd38, 0xd4d, 0x200c, 0xd31,
0xd4d, 0xd31, 0xd17, 0xd3e, 0xd38, 0xd39, 0xd46, 0xd2c, 0xd4d, 0x200c, 0xd31, 0xd4b,
0xd7a, 0xd39, 0xd4b, 0x20, 0xd1a, 0xd3f, 0x20, 0xd2e, 0xd3f, 0xd7b, 0x20, 0xd38,
0xd3f, 0xd31, 0xd4d, 0xd31, 0xd3f, 0xd39, 0xd4b, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd02,
0xd17, 0xd4d, 0xd39, 0xd4b, 0xd21, 0xd4d, 0xd07, 0xd7c, 0xd15, 0xd38, 0xd4d, 0xd15,
0xd4d, 0xd1c, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd7c, 0xd24, 0xd4d, 0xd24, 0xd1c, 0xd2f,
0xd2a, 0xd41, 0xd30, 0xd1c, 0xd46, 0xd31, 0xd41, 0xd38, 0xd32, 0xd47, 0xd02, 0xd15,
0xd3e, 0xd2c, 0xd42, 0xd7e, 0xd15, 0xd3e, 0xd02, 0xd1a, 0xd1f, 0xd4d, 0xd15, 0xd15,
0xd31, 0xd3e, 0xd1a, 0xd4d, 0xd1a, 0xd3f, 0xd15, 0xd3e, 0xd20, 0xd4d, 0x200c, 0xd2e,
0xd23, 0xd4d, 0xd21, 0xd41, 0xd15, 0xd3e, 0xd7b, 0xd21, 0xd3f, 0xd17, 0xd15, 0xd4a,
0xd7d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd24, 0xd4d, 0xd24, 0xd15, 0xd4d, 0xd30, 0xd3e,
0xd38, 0xd4d, 0xd28, 0xd4b, 0xd2f, 0xd3e, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd15, 0xd4d,
0xd35, 0xd3e, 0xd32, 0xd32, 0xd02, 0x200c, 0xd2a, 0xd42, 0xd7c, 0x200c, 0x200c, 0xd15,
0xd41, 0xd1a, 0xd3f, 0xd19, 0xd4d, 0xd15, 0xd41, 0xd35, 0xd48, 0xd24, 0xd4d, 0xd24,
0xd4d, 0xd2e, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd35, 0xd41, 0xd2e, 0xd17, 0xd21, 0xd3e,
0xd7b, 0xd2e, 0xd15, 0xd38, 0xd4d, 0xd38, 0xd7c, 0xd2e, 0xd28, 0xd3f, 0xd32, 0xd2e,
0xd38, 0xd4d, 0xd15, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd15,
0xd4b, 0xd37, 0xd4d, 0xd2f, 0xd28, 0xd4b, 0xd35, 0xd4b, 0xd15, 0xd41, 0xd38, 0xd46,
0xd7b, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd28, 0xd4a, 0xd35, 0xd4b, 0xd38, 0xd3f,
0xd2c, 0xd3f, 0xd7c, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd12, 0xd02, 0xd38, 0xd4d, 0xd15,
0xd4d, 0xd13, 0xd31, 0xd7d, 0xd2b, 0xd46, 0xd28, 0xd4b, 0xd02, 0x20, 0xd2a, 0xd46,
0xd7b, 0xd2a, 0xd4a, 0xd28, 0xd4d, 0xd31, 0xd3f, 0xd2f, 0xd3e, 0xd28, 0xd15, 0xd4d,
0xd2a, 0xd4d, 0xd2f, 0xd4b, 0xd02, 0xd17, 0xd4d, 0x200c, 0xd2f, 0xd3e, 0xd02, 0xd17,
0xd4d, 0xd16, 0xd24, 0xd4d, 0xd24, 0xd7c, 0xd15, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd28, 0xd47, 0xd16, 0xd3f, 0xd38, 0xd3f, 0xd32, 0xd4b, 0xd7c, 0xd21, 0xd31,
0xd3f, 0xd2f, 0xd3e, 0xd26, 0xd4d, 0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0xd38,
0xd2e, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd28, 0xd4d, 0xd26, 0xd4d, 0xd38, 0xd4b, 0xd7e,
0xd37, 0xd3e, 0xd19, 0xd4d, 0x200c, 0xd39, 0xd3e, 0xd2f, 0xd3f, 0xd38, 0xd3f, 0xd02,
0xd17, 0xd2a, 0xd4d, 0xd2a, 0xd42, 0xd7c, 0xd38, 0xd4d, 0xd30, 0xd3f, 0xd21, 0xd4d,
0x200c, 0xd28, 0xd3f, 0xd15, 0xd4b, 0xd33, 0xd3f, 0xd02, 0xd38, 0xd4d, 0xd15, 0xd4d,
0xd24, 0xd3e, 0xd2f, 0xd4d, 0x200c, 0xd2a, 0xd46, 0xd2f, 0xd4d, 0xd24, 0xd3e, 0xd37,
0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd28, 0xd4d, 0xd31, 0xd4d, 0xd24, 0xd3f, 0xd2c,
0xd3f, 0xd32, 0xd3f, 0xd38, 0xd3f, 0xd1f, 0xd46, 0xd39, 0xd4d, 0x200c, 0xd31, 0xd3e,
0xd7b, 0x200c, 0xd24, 0xd3f, 0xd02, 0xd2b, 0xd41, 0xd1f, 0xd4b, 0xd15, 0xd4d, 0xd15,
0xd3f, 0xd2f, 0xd4b, 0xd1f, 0xd4b, 0xd02, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd09, 0xd32,
0xd3e, 0xd7b, 0xd2c, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd7c, 0xd09, 0xd31, 0xd41, 0xd02,
0xd16, 0xd3f, 0xd2f, 0xd41, 0xd38, 0xd4d, 0x2d, 0xd28, 0xd47, 0xd30, 0xd35, 0xd46,
0xd28, 0xd4d, 0xd31, 0xd3f, 0xd2f, 0xd3e, 0xd7b, 0xd35, 0xd4d, 0xd33, 0xd3e, 0xd21,
0xd3f, 0xd35, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0xd2f,
0xd3e, 0xd15, 0xd4d, 0xd15, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd38, 0xd4d, 0x200c, 0xd15,
0xd4d, 0xd31, 0xd19, 0xd4d, 0xd15, 0xd42, 0xd7a, 0x200c, 0xd2f, 0xd3e, 0xd15, 0xd3e,
0xd31, 0xd4d, 0xd31, 0xd46, 0xd31, 0xd3f, 0xd7b, 0xd2c, 0xd7c, 0xd17, 0xd4d, 0xd2f,
0xd47, 0xd30, 0xd35, 0xd7b, 0x200c, 0xd05, 0xd38, 0xd4b, 0xd31, 0xd38, 0xd4d, 0xd2c,
0xd7c, 0xd2e, 0xd41, 0xd21, 0xd15, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd31, 0xd3f, 0xd15,
0xd47, 0xd2a, 0xd4d, 0x20, 0xd35, 0xd46, 0xd7c, 0xd26, 0xd46, 0xd2b, 0xd46, 0xd31,
0xd4b, 0xd2e, 0xd21, 0xd46, 0xd2f, 0xd4d, 0xd31, 0xd31, 0xd47, 0xd2f, 0xd4d, 0x200c,
0xd1c, 0xd3e, 0xd35, 0xd3f, 0xd15, 0xd4d, 0xd26, 0xd15, 0xd4d, 0xd37, 0xd3f, 0xd23,
0x20, 0xd1c, 0xd4b, 0xd7c, 0xd1c, 0xd3f, 0xd2f, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd39, 0xd46, 0xd32, 0xd46, 0xd28, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0x200c, 0xd32, 0xd3f, 0xd05, 0xd21, 0xd3f, 0xd32, 0xd46, 0xd2f, 0xd4d,
0x200c, 0xd21, 0xd4d, 0xd2c, 0xd4d, 0xd30, 0xd3f, 0xd38, 0xd4d, 0x200c, 0xd2c, 0xd46,
0xd2f, 0xd3f, 0xd7b, 0xd2c, 0xd4d, 0xd30, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd7a, 0x20,
0xd39, 0xd3f, 0xd7d, 0xd21, 0xd3e, 0xd7c, 0xd35, 0xd3f, 0xd7b, 0xd2f, 0xd42, 0xd15,
0xd4d, 0xd32, 0xd39, 0xd4a, 0xd2c, 0xd3e, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd32,
0xd3f, 0xd7b, 0xd21, 0xd46, 0xd2e, 0xd3e, 0xd7b, 0xd32, 0xd4b, 0xd21, 0xd4d, 0x20,
0xd39, 0xd4b, 0xd35, 0xd4d, 0xd2e, 0xd46, 0xd7d, 0xd2c, 0xd7a, 0xd2a, 0xd46, 0xd7c,
0xd24, 0xd4d, 0xd24, 0xd4d, 0xd38, 0xd3f, 0xd21, 0xd4d, 0xd28, 0xd3f, 0xd06, 0xd02,
0x200c, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd7c, 0x200c, 0xd21, 0xd3e, 0xd02, 0xd05,
0xd23, 0xd4d, 0xd1f, 0xd4b, 0xd31, 0xd05, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30,
0xd16, 0xd3e, 0xd7b, 0xd0f, 0xd24, 0xd7b, 0x200c, 0xd38, 0xd4d, 0xd2c, 0xd46, 0xd7d,
0xd17, 0xd4d, 0xd30, 0xd47, 0xd21, 0xd4d, 0xd2c, 0xd46, 0xd7c, 0x200c, 0xd32, 0xd3f,
0xd7b, 0x200c, 0xd2c, 0xd4d, 0xd30, 0xd3e, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd38, 0xd4d,
0x200d, 0xd32, 0xd3e, 0xd35, 0xd2c, 0xd4d, 0xd30, 0xd38, 0xd4d, 0xd38, 0xd7d, 0x200c,
0xd38, 0xd4d, 0xd2c, 0xd41, 0xd1a, 0xd4d, 0xd1a, 0xd3e, 0xd31, 0xd46, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd4d, 0xd2c, 0xd41, 0xd21, 0xd3e, 0xd2a, 0xd46, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd4d, 0xd2c, 0xd41, 0xd38, 0xd3f, 0xd7b, 0xd1c, 0xd7b, 0xd1a,
0xd3f, 0xd38, 0xd3f, 0xd28, 0xd3e, 0xd35, 0xd41, 0xd15, 0xd4b, 0xd2a, 0xd4d, 0xd2a,
0xd7b, 0x200c, 0xd39, 0xd47, 0xd17, 0xd7b, 0x200c, 0xd21, 0xd2c, 0xd4d, 0xd32, 0xd3f,
0xd7b, 0xd1c, 0xd3f, 0xd2c, 0xd4d, 0xd30, 0xd3e, 0xd7e, 0xd1f, 0xd4d, 0xd1f, 0xd7c,
0xd17, 0xd47, 0xd7a, 0xd38, 0xd47, 0xd39, 0xd46, 0xd7d, 0x200c, 0xd38, 0xd3f, 0xd19,
0xd4d, 0xd15, 0xd3f, 0xd10, 0xd7d, 0x200c, 0x20, 0xd13, 0xd2b, 0xd4d, 0x20, 0xd2e,
0xd3e, 0xd7b, 0x200c, 0xd07, 0xd38, 0xd4d, 0xd24, 0xd3e, 0xd02, 0x200c, 0xd2c, 0xd41,
0xd7e, 0x200c, 0xd1c, 0xd47, 0xd34, 0xd4d, 0x200c, 0xd38, 0xd3f, 0xd15, 0xd32, 0xd3f,
0xd28, 0xd3f, 0xd7b, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d, 0xd15, 0xd3f, 0xd31,
0xd4b, 0xd15, 0xd40, 0xd35, 0xd4d, 0xd32, 0xd3f, 0xd38, 0xd4d, 0x200c, 0xd2c, 0xd7a,
0x200c, 0xd32, 0xd41, 0xd2c, 0xd4d, 0x200d, 0xd32, 0xd3f, 0xd2f, 0xd3e, 0xd28, 0xd32,
0xd23, 0xd4d, 0xd1f, 0xd7b, 0x200c, 0xd32, 0xd15, 0xd4d, 0x200c, 0xd38, 0xd02, 0xd2c,
0xd7c, 0xd17, 0xd4d, 0xd2e, 0xd3e, 0xd21, 0xd4d, 0xd30, 0xd3f, 0xd21, 0xd4d, 0xd2e,
0xd3e, 0xd7e, 0xd1f, 0xd4d, 0xd1f, 0xd2e, 0xd30, 0xd3f, 0xd2f, 0xd3e, 0xd39, 0xd3e,
0xd2e, 0xd7b, 0xd2e, 0xd3f, 0xd7b, 0x200c, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd2e, 0xd4a,
0xd23, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4b,
0xd13, 0xd38, 0xd4d, 0xd32, 0xd4b, 0xd2a, 0xd3e, 0xd30, 0xd40, 0xd38, 0xd4d, 0xd2a,
0xd4a, 0xd21, 0xd4d, 0x200d, 0xd17, 0xd4a, 0xd31, 0xd3f, 0xd38, 0xd2a, 0xd4d, 0xd30,
0xd3e, 0xd17, 0xd4d, 0xd31, 0xd4b, 0xd02, 0xd38, 0xd2e, 0xd3e, 0xd31, 0xd38, 0xd3e,
0xd7b, 0x20, 0xd2e, 0xd3e, 0xd30, 0xd3f, 0xd28, 0xd4b, 0xd38, 0xd30, 0xd2f, 0xd47,
0xd35, 0xd4b, 0xd38, 0xd30, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd4b, 0xd35, 0xd4d, 0xd38,
0xd3f, 0xd02, 0xd2b, 0xd46, 0xd31, 0xd4b, 0xd2a, 0xd4b, 0xd7e, 0xd38, 0xd4d, 0xd15,
0xd4b, 0xd2a, 0xd4d, 0xd2a, 0xd3f, 0xd2f, 0xd46, 0xd38, 0xd4b, 0xd2b, 0xd3f, 0xd2f,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd39, 0xd4b,
0xd02, 0xd1f, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0x200c, 0xd1f, 0xd3f, 0xd30, 0xd3e, 0xd28,
0xd46, 0xd09, 0xd32, 0xd4d, 0xd32, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd4b, 0xd35, 0xd4d,
0x200c, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd35, 0xd3e, 0xd26, 0xd41, 0xd38, 0xd4d,
0xd35, 0xd24, 0xd4d, 0xd24, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd7b, 0xd35, 0xd3f,
0xd2f, 0xd28, 0xd4d, 0xd28, 0xd35, 0xd3f, 0xd7d, 0x200c, 0xd28, 0xd3f, 0xd2f, 0xd38,
0xd4d, 0xd35, 0xd4b, 0xd7e, 0xd17, 0xd4b, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d,
0xd35, 0xd3e, 0xd7c, 0xd38, 0xd4b, 0xd38, 0xd3e, 0xd15, 0xd4d, 0xd30, 0xd46, 0xd2c,
0xd4d, 0xd38, 0xd42, 0xd31, 0xd3f, 0xd1a, 0xd4d, 0xd1a, 0xd4d, 0xd05, 0xd7b, 0xd31,
0xd3e, 0xd28, 0xd28, 0xd31, 0xd3f, 0xd35, 0xd4b, 0xd1a, 0xd3e, 0xd17, 0xd4b, 0xd38,
0xd4d, 0xd15, 0xd4d, 0xd30, 0xd3f, 0xd38, 0xd4d, 0xd24, 0xd41, 0xd2e, 0xd38, 0xd4d,
0xd15, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4a, 0xd2e, 0xd4b,
0xd31, 0xd4b, 0xd15, 0xd46, 0xd7c, 0xd17, 0xd41, 0xd32, 0xd46, 0xd7b, 0xd2e, 0xd3e,
0xd39, 0xd3f, 0xd2e, 0xd3e, 0xd32, 0xd3f, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d,
0xd2e, 0xd57, 0xd31, 0xd40, 0xd37, 0xd4d, 0xd2f, 0xd38, 0xd4d, 0xd2e, 0xd2f, 0xd4b,
0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd31, 0xd40, 0xd2f, 0xd42, 0xd23, 0xd3f, 0xd2f, 0xd7b,
0xd06, 0xd2a, 0xd3f, 0xd2f, 0xd13, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c, 0xd32, 0xd3e,
0xd28, 0xd4d, 0xd31, 0xd4d, 0xd2c, 0xd4b, 0xd17, 0xd7a, 0xd35, 0xd3f, 0xd32, 0xd4d,
0xd32, 0xd46, 0xd1a, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd02, 0xd1a, 0xd15, 0xd4d, 0xd08,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd7c, 0xd07, 0xd2b, 0xd3e, 0xd24, 0xd46, 0xd2b,
0xd15, 0xd4d, 0xd15, 0xd3e, 0xd35, 0xd4b, 0xd2b, 0xd4b, 0xd2b, 0xd3f, 0xd1c, 0xd3f,
0xd2b, 0xd41, 0xd23, 0xd3e, 0xd2b, 0xd41, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd17, 0xd3e,
0xd32, 0xd2a, 0xd4d, 0xd2a, 0xd3e, 0xd17, 0xd4b, 0xd38, 0xd4d, 0xd17, 0xd3e, 0xd2e,
0xd4d, 0xd2a, 0xd3f, 0xd2f, 0xd7c, 0xd17, 0xd4d, 0xd35, 0xd3e, 0xd21, 0xd7d, 0xd15,
0xd28, 0xd3e, 0xd7d, 0xd17, 0xd4d, 0xd35, 0xd3e, 0xd02, 0xd39, 0xd4b, 0xd23, 0xd32,
0xd42, 0xd32, 0xd41, 0xd0e, 0xd7b, 0xd21, 0xd2c, 0xd31, 0xd3f, 0xd15, 0xd3f, 0xd30,
0xd3f, 0xd31, 0xd4d, 0xd31, 0xd3f, 0xd2e, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd3f, 0xd15,
0xd4a, 0xd38, 0xd4d, 0xd30, 0xd47, 0xd15, 0xd4d, 0xd35, 0xd3e, 0xd1c, 0xd32, 0xd46,
0xd2f, 0xd4d, 0xd7b, 0xd2e, 0xd1c, 0xd42, 0xd31, 0xd4b, 0xd2e, 0xd3e, 0xd7c, 0xd15,
0xd4d, 0xd2f, 0xd41, 0xd38, 0xd3e, 0xd38, 0xd4d, 0xd2e, 0xd3f, 0xd21, 0xd4d, 0x200c,
0xd35, 0xd47, 0xd28, 0xd57, 0xd31, 0xd41, 0xd28, 0xd3f, 0xd2f, 0xd41, 0xd28, 0xd4b,
0xd7c, 0x200c, 0xd2b, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd28, 0xd4b, 0xd2e, 0xd3f,
0xd2f, 0xd2a, 0xd3e, 0xd17, 0xd4b, 0x20, 0xd2a, 0xd3e, 0xd17, 0xd4b, 0xd2a, 0xd32,
0xd3e, 0xd35, 0xd41, 0xd2a, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd15, 0xd2f, 0xd3f,
0xd7b, 0x200c, 0xd2a, 0xd4b, 0xd7a, 0xd2a, 0xd46, 0xd2a, 0xd4b, 0xd7c, 0xd1f, 0xd4d,
0xd1f, 0xd4d, 0x20, 0xd2e, 0xd4b, 0xd34, 0xd4d, 0x200c, 0xd38, 0xd4d, 0xd2c, 0xd48,
0xd31, 0xd3e, 0xd30, 0xd4b, 0xd1f, 0xd4b, 0xd02, 0xd17, 0xd38, 0xd46, 0xd2f, 0xd4d,
0x200c, 0xd2a, 0xd4d, 0xd2a, 0xd3e, 0xd7b, 0x200c, 0xd24, 0xd3e, 0xd39, 0xd3f, 0xd24,
0xd3f, 0xd24, 0xd30, 0xd3e, 0xd35, 0xd1f, 0xd4b, 0xd02, 0xd17, 0xd3e, 0xd1f, 0xd3e,
0xd2a, 0xd41, 0xd35, 0xd46, 0xd2f, 0xd4d, 0xd15, 0xd4d, 0xd35, 0xd3e, 0xd32, 0xd4d,
0xd32, 0xd3f, 0xd38, 0xd4d, 0x4b, 0x61, 0x68, 0x65, 0x72, 0x61, 0x68, 0x44,
0x61, 0x6d, 0x73, 0x79, 0x69, 0x6b, 0x42, 0x61, 0x69, 0x74, 0x75, 0x6c,
0x6d, 0x75, 0x71, 0x61, 0x64, 0x64, 0x69, 0x73, 0x633, 0x64a, 0x6a0, 0x627,
0x6a4, 0x648, 0x631, 0x627, 0x6c, 0x2d, 0x41, 0x6c, 0x121, 0x69, 0x65, 0x72,
0x69, 0x6c, 0x2d, 0x47, 0x77, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61,
0x69, 0x6c, 0x2d, 0x120, 0x61, 0x6d, 0x61, 0x6a, 0x6b, 0x61, 0x120, 0x65,
0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x69, 0x6c, 0x2d, 0x42,
0x65, 0x6c, 0x74, 0x20, 0x74, 0x61, 0x6c, 0x2d, 0x4b, 0x75, 0x77, 0x61,
0x6a, 0x74, 0x69, 0x6c, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x74, 0x61, 0x6e, 0x2d, 0x4e, 0x6f, 0x66, 0x73, 0x69, 0x6e, 0x68,
0x61, 0x72, 0x41, 0x74, 0x65, 0x6e, 0x69, 0x42, 0x72, 0x75, 0x73, 0x73,
0x65, 0x6c, 0x6c, 0x120, 0x69, 0x62, 0x69, 0x6c, 0x74, 0xe0, 0x4c, 0x69,
0x17c, 0x62, 0x6f, 0x6e, 0x61, 0x69, 0x6c, 0x2d, 0x4c, 0x75, 0x73, 0x73,
0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x75, 0x56, 0x61, 0x6c, 0x6c, 0x65,
0x74, 0x74, 0x61, 0x50, 0x61, 0x72, 0x69, 0x121, 0x69, 0x52, 0x75, 0x6d,
0x61, 0x53, 0x74, 0x6f, 0x6b, 0x6b, 0x6f, 0x6c, 0x6d, 0x61, 0x69, 0x6c,
0x2d, 0x42, 0x65, 0x6c, 0x74, 0x20, 0x74, 0x61, 0x6c, 0x2d, 0x56, 0x61,
0x74, 0x69, 0x6b, 0x61, 0x6e, 0x56, 0x6a, 0x65, 0x6e, 0x6e, 0x61, 0x56,
0x61, 0x72, 0x73, 0x61, 0x76, 0x6a, 0x61, 0x69, 0x6c, 0x2d, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x69, 0x4e, 0x67, 0x101, 0x69, 0x72, 0x6f, 0x70,
0x69, 0x54, 0x69, 0x72, 0x69, 0x70, 0x6f, 0x72, 0x12b, 0x54, 0x16b, 0x6e,
0x12b, 0x68, 0x69, 0x54, 0x65, 0x20, 0x4d, 0x6f, 0x74, 0x75, 0x20, 0x4e,
0x65, 0x68, 0x65, 0x50, 0x61, 0x70, 0x61, 0x74, 0x6f, 0x68, 0x65, 0x50,
0x113, 0x72, 0x69, 0x68, 0x69, 0x4b, 0x65, 0x6d, 0x75, 0x72, 0x65, 0x74,
0x69, 0x20, 0x50, 0x65, 0x69, 0x4b, 0x101, 0x6d, 0x61, 0x6e, 0x61, 0x48,
0x69, 0x6b, 0x101, 0x6b, 0x6f, 0x4b, 0x6f, 0x74, 0x6f, 0x20, 0x52, 0x69,
0x6b, 0x61, 0x54, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0x57, 0x68,
0x61, 0x6b, 0x61, 0x6f, 0x72, 0x61, 0x4b, 0x75, 0x69, 0x68, 0x69, 0x20,
0x50, 0x65, 0x69, 0x54, 0x101, 0x6b, 0x6f, 0x72, 0x75, 0x20, 0x4e, 0x75,
0x69, 0x4b, 0x65, 0x72, 0x65, 0x6e, 0x61, 0x74, 0x61, 0x4b, 0x75, 0x61,
0x74, 0x61, 0x6d, 0x101, 0x72, 0x61, 0x48, 0x61, 0x77, 0x68, 0x101, 0x6e,
0x61, 0x48, 0x65, 0x6d, 0x65, 0x69, 0x6b, 0x61, 0x4e, 0x67, 0x101, 0x20,
0x41, 0x6e, 0x61, 0x68, 0x65, 0x72, 0x61, 0x4d, 0x101, 0x74, 0x69, 0x6e,
0x69, 0x6b, 0x69, 0x4d, 0x113, 0x68, 0x69, 0x6b, 0x6f, 0x20, 0x54, 0x101,
0x6f, 0x6e, 0x65, 0x6e, 0x75, 0x69, 0x54, 0x65, 0x20, 0x100, 0x70, 0x6f,
0x72, 0x6f, 0x20, 0x4e, 0x75, 0x69, 0x50, 0x65, 0x74, 0x61, 0x20, 0x52,
0x69, 0x6b, 0x6f, 0x48, 0x61, 0x74, 0x6f, 0x20, 0x48, 0x6f, 0x6e, 0x65,
0x48, 0x61, 0x74, 0x6f, 0x20, 0x52, 0x75, 0x69, 0x68, 0x69, 0x61, 0x48,
0x61, 0x74, 0x6f, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x74, 0x69, 0x48, 0x61,
0x74, 0x6f, 0x20, 0x57, 0x113, 0x6e, 0x65, 0x74, 0x69, 0x54, 0x101, 0x72,
0x6f, 0x74, 0x6f, 0x54, 0x65, 0x20, 0x57, 0x68, 0x61, 0x6e, 0x67, 0x61,
0x2d, 0x61, 0x2d, 0x4b, 0x69, 0x77, 0x61, 0x52, 0x113, 0x77, 0x65, 0x74,
0x69, 0x4d, 0x61, 0x6b, 0x6f, 0x61, 0x72, 0x65, 0x50, 0x101, 0x6b, 0x61,
0x74, 0x61, 0x74, 0x61, 0x50, 0x101, 0x72, 0x65, 0x69, 0x6e, 0x61, 0x50,
0x61, 0x6e, 0x67, 0x61, 0x6b, 0x6f, 0x6b, 0x6f, 0x50, 0x6f, 0x72, 0x6f,
0x6e, 0x61, 0x69, 0x54, 0x75, 0x70, 0x61, 0x65, 0x4b, 0x101, 0x68, 0x61,
0x48, 0x6f, 0x6e, 0x67, 0x69, 0x70, 0x75, 0x61, 0x54, 0x69, 0x61, 0x6b,
0x101, 0x74, 0x61, 0x48, 0x69, 0x72, 0x75, 0x68, 0x101, 0x72, 0x61, 0x6d,
0x61, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x72, 0x16b, 0x4b, 0x75, 0x61,
0x72, 0x61, 0x20, 0x52, 0x16b, 0x70, 0x61, 0x4b, 0x16b, 0x77, 0x65, 0x69,
0x74, 0x69, 0x4d, 0x61, 0x6e, 0x69, 0x72, 0x61, 0x50, 0x65, 0x6e, 0x6f,
0x6d, 0x61, 0x20, 0x50, 0x65, 0x6e, 0x61, 0x4b, 0x61, 0x74, 0x101, 0x52,
0x69, 0x61, 0x74, 0x61, 0x48, 0x6f, 0x75, 0x72, 0x61, 0x48, 0x61, 0x6e,
0x67, 0x61, 0x68, 0x61, 0x69, 0x48, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x61, 0x54, 0x65, 0x72, 0x101, 0x6e, 0x61, 0x54, 0x14d, 0x6b, 0x69, 0x6f,
0x50, 0x101, 0x6d, 0x75, 0x72, 0x61, 0x54, 0x65, 0x20, 0x4b, 0x16b, 0x72,
0x61, 0x65, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x6d, 0x61, 0x74, 0x6f, 0x48,
0x14d, 0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x54, 0x65, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x48, 0x61, 0x74, 0x6f, 0x20, 0x48, 0x113, 0x72,
0x65, 0x6e, 0x61, 0x41, 0x74, 0x69, 0x72, 0x65, 0x69, 0x72, 0x61, 0x50,
0x69, 0x72, 0x69, 0x70, 0x61, 0x6e, 0x65, 0x54, 0x101, 0x77, 0x69, 0x6e,
0x69, 0x48, 0x6f, 0x70, 0x61, 0x74, 0x101, 0x6f, 0x6e, 0x65, 0x50, 0x6f,
0x69, 0x70, 0x69, 0x72, 0x69, 0x70, 0x69, 0x50, 0x113, 0x74, 0x69, 0x61,
0x50, 0x6f, 0x69, 0x68, 0x101, 0x6b, 0x65, 0x6e, 0x61, 0x50, 0x101, 0x70,
0x75, 0x6e, 0x69, 0x101, 0x6d, 0x69, 0x74, 0x61, 0x41, 0x6e, 0x61, 0x74,
0x14d, 0x72, 0x61, 0x100, 0x74, 0x65, 0x6e, 0x65, 0x50, 0x65, 0x61, 0x72,
0x12b, 0x6e, 0x69, 0x50, 0x61, 0x72, 0x75, 0x68, 0x69, 0x50, 0x75, 0x74,
0x61, 0x70, 0x113, 0x68, 0x69, 0x4b, 0x6f, 0x70, 0x65, 0x68, 0x65, 0x69,
0x6b, 0x61, 0x6e, 0x61, 0x54, 0x61, 0x70, 0x75, 0x72, 0x69, 0x6e, 0x69,
0x48, 0x113, 0x72, 0x69, 0x6b, 0x69, 0x54, 0x65, 0x20, 0x4d, 0x6f, 0x75,
0x74, 0x65, 0x72, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x61, 0x49, 0x74, 0x61,
0x70, 0x16b, 0x72, 0x75, 0x52, 0x12b, 0x70, 0x65, 0x6e, 0x65, 0x52, 0x101,
0x6e, 0x61, 0x6e, 0x61, 0x52, 0x61, 0x6b, 0x61, 0x70, 0x75, 0x14d, 0x4d,
0x101, 0x74, 0x69, 0x72, 0x69, 0x4d, 0x101, 0x72, 0x61, 0x74, 0x61, 0x4d,
0x6f, 0x6e, 0x101, 0x6b, 0x6f, 0x4d, 0x6f, 0x68, 0x69, 0x6b, 0x61, 0x75,
0x14c, 0x68, 0x6f, 0x72, 0x6f, 0x50, 0x61, 0x72, 0x101, 0x6b, 0x61, 0x52,
0x14d, 0x6d, 0x61, 0x48, 0x61, 0x6e, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x69,
0x6e, 0x6f, 0x54, 0x6f, 0x6b, 0x6f, 0x6f, 0x6d, 0x6f, 0x54, 0x65, 0x20,
0x50, 0x6f, 0x68, 0x6f, 0x2d, 0x6f, 0x2d, 0x50, 0x69, 0x74, 0x61, 0x57,
0x68, 0x69, 0x65, 0x6e, 0x61, 0x48, 0x16b, 0x72, 0x69, 0x6b, 0x61, 0x4b,
0x69, 0x72, 0x69, 0x68, 0x69, 0x6d, 0x65, 0x74, 0x65, 0x4b, 0x6f, 0x6b,
0x6f, 0x68, 0x69, 0x4d, 0x101, 0x72, 0x61, 0x74, 0x69, 0x72, 0x69, 0x4d,
0x61, 0x72, 0x69, 0x68, 0x69, 0x100, 0x70, 0x69, 0x61, 0x54, 0x101, 0x6d,
0x61, 0x6b, 0x69, 0x20, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x72, 0x61, 0x75,
0x52, 0x113, 0x6b, 0x6f, 0x68, 0x75, 0x57, 0x68, 0x12b, 0x74, 0x12b, 0x4b,
0x75, 0x61, 0x6d, 0x61, 0x4e, 0x14d, 0x70, 0x6f, 0x6b, 0x65, 0x4e, 0x16b,
0x6d, 0x65, 0x61, 0x50, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x6e,
0x67, 0x6f, 0x50, 0x101, 0x72, 0x61, 0x75, 0x50, 0x6f, 0x74, 0x61, 0x20,
0x4d, 0x6f, 0x61, 0x68, 0x69, 0x70, 0x69, 0x57, 0x101, 0x72, 0x69, 0x68,
0x69, 0x905, 0x92c, 0x93f, 0x926, 0x94d, 0x91c, 0x93e, 0x928, 0x905, 0x200d, 0x945,
0x915, 0x94d, 0x930, 0x93e, 0x906, 0x926, 0x93f, 0x938, 0x20, 0x905, 0x92c, 0x93e,
0x92c, 0x93e, 0x905, 0x932, 0x94d, 0x91c, 0x93f, 0x905, 0x930, 0x94d, 0x938, 0x90f,
0x938, 0x94d, 0x92e, 0x93e, 0x930, 0x93e, 0x92c, 0x93e, 0x92e, 0x915, 0x94b, 0x92c,
0x93e, 0x902, 0x917, 0x941, 0x908, 0x92c, 0x94d, 0x932, 0x901, 0x91f, 0x93e, 0x92f,
0x930, 0x92c, 0x94d, 0x930, 0x93e, 0x91d, 0x93e, 0x935, 0x94d, 0x939, 0x93f, 0x932,
0x947, 0x92c, 0x941, 0x91c, 0x941, 0x902, 0x92c, 0x941, 0x930, 0x93e, 0x915, 0x948,
0x930, 0x94b, 0x915, 0x945, 0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x93e, 0x928, 0x94d,
0x915, 0x93e, 0x915, 0x94b, 0x928, 0x93e, 0x915, 0x94d, 0x930, 0x93f, 0x921, 0x915,
0x930, 0x926, 0x93e, 0x930, 0x20, 0x90f, 0x20, 0x938, 0x932, 0x93e, 0x92e, 0x926,
0x94b, 0x909, 0x906, 0x932, 0x93e, 0x90f, 0x932, 0x20, 0x910, 0x909, 0x928, 0x916,
0x93e, 0x930, 0x94d, 0x91f, 0x941, 0x92e, 0x915, 0x940, 0x917, 0x93e, 0x932, 0x940,
0x932, 0x93f, 0x92c, 0x930, 0x935, 0x94d, 0x939, 0x93f, 0x932, 0x932, 0x941, 0x92c,
0x902, 0x92c, 0x93e, 0x936, 0x940, 0x92e, 0x938, 0x947, 0x930, 0x941, 0x905, 0x902,
0x92c, 0x93e, 0x92c, 0x93e, 0x928, 0x947, 0x92e, 0x94b, 0x928, 0x930, 0x94b, 0x935,
0x94d, 0x939, 0x93f, 0x92f, 0x93e, 0x907, 0x902, 0x91c, 0x93e, 0x92e, 0x947, 0x928,
0x93e, 0x928, 0x941, 0x935, 0x93e, 0x915, 0x938, 0x941, 0x924, 0x935, 0x93e, 0x917,
0x921, 0x942, 0x917, 0x942, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x2d, 0x928,
0x94b, 0x935, 0x94d, 0x939, 0x94b, 0x938, 0x93e, 0x913, 0x20, 0x924, 0x94b, 0x92e,
0x947, 0x91f, 0x92f, 0x942, 0x928, 0x93f, 0x938, 0x935, 0x93f, 0x902, 0x921, 0x939,
0x94b, 0x90f, 0x915, 0x905, 0x921, 0x945, 0x915, 0x905, 0x901, 0x915, 0x930, 0x947,
0x91c, 0x905, 0x901, 0x917, 0x94d, 0x935, 0x93f, 0x932, 0x93e, 0x905, 0x901, 0x91f,
0x93f, 0x917, 0x94d, 0x935, 0x93e, 0x905, 0x930, 0x93e, 0x917, 0x93e, 0x92f, 0x928,
0x93e, 0x92c, 0x94d, 0x92f, 0x941, 0x928, 0x94b, 0x938, 0x20, 0x906, 0x92f, 0x930,
0x94d, 0x938, 0x915, 0x945, 0x91f, 0x93e, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93e,
0x915, 0x949, 0x930, 0x94d, 0x921, 0x94b, 0x92c, 0x93e, 0x92e, 0x947, 0x902, 0x926,
0x94b, 0x91d, 0x93e, 0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x917, 0x945, 0x932, 0x947,
0x917, 0x949, 0x938, 0x938, 0x949, 0x932, 0x94d, 0x924, 0x93e, 0x938, 0x93e, 0x928,
0x20, 0x932, 0x94d, 0x92f, 0x941, 0x907, 0x938, 0x909, 0x938, 0x94d, 0x935, 0x93e,
0x907, 0x92f, 0x93e, 0x906, 0x938, 0x941, 0x928, 0x94d, 0x938, 0x93f, 0x92f, 0x94b,
0x928, 0x905, 0x945, 0x91f, 0x93f, 0x915, 0x94b, 0x915, 0x928, 0x92c, 0x93e, 0x939,
0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x93e, 0x902, 0x926, 0x947, 0x930, 0x93e, 0x938,
0x92c, 0x93e, 0x930, 0x94d, 0x92c, 0x93e, 0x921, 0x94b, 0x938, 0x92c, 0x947, 0x932,
0x93f, 0x91d, 0x947, 0x92c, 0x94d, 0x932, 0x93e, 0x902, 0x915, 0x20, 0x938, 0x945,
0x92c, 0x932, 0x94b, 0x928, 0x92c, 0x94b, 0x906, 0x20, 0x935, 0x94d, 0x939, 0x93f,
0x938, 0x94d, 0x91f, 0x93e, 0x92c, 0x94b, 0x907, 0x938, 0x940, 0x915, 0x947, 0x902,
0x92c, 0x94d, 0x930, 0x93f, 0x91c, 0x20, 0x909, 0x92a, 0x938, 0x93e, 0x917, 0x930,
0x915, 0x945, 0x92e, 0x94d, 0x92a, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x901, 0x921,
0x947, 0x915, 0x930, 0x93e, 0x915, 0x93e, 0x938, 0x915, 0x93e, 0x92f, 0x947, 0x928,
0x947, 0x915, 0x947, 0x92e, 0x928, 0x938, 0x93f, 0x909, 0x926, 0x93e, 0x926, 0x20,
0x939, 0x941, 0x906, 0x930, 0x947, 0x91d, 0x915, 0x94d, 0x92f, 0x941, 0x930, 0x93e,
0x938, 0x93e, 0x913, 0x921, 0x947, 0x928, 0x94d, 0x935, 0x94d, 0x939, 0x930, 0x921,
0x947, 0x91f, 0x94d, 0x930, 0x949, 0x907, 0x91f, 0x90f, 0x921, 0x92e, 0x93e, 0x901,
0x91f, 0x928, 0x92f, 0x942, 0x930, 0x941, 0x928, 0x940, 0x92a, 0x947, 0x917, 0x94d,
0x932, 0x947, 0x938, 0x20, 0x909, 0x92a, 0x938, 0x93e, 0x917, 0x930, 0x917, 0x942,
0x938, 0x20, 0x909, 0x92a, 0x938, 0x93e, 0x917, 0x930, 0x917, 0x94d, 0x930, 0x901,
0x921, 0x20, 0x91f, 0x930, 0x94d, 0x915, 0x917, 0x94d, 0x930, 0x947, 0x928, 0x947,
0x921, 0x93e, 0x917, 0x94d, 0x935, 0x93e, 0x921, 0x947, 0x932, 0x94b, 0x909, 0x92a,
0x947, 0x917, 0x92f, 0x93e, 0x915, 0x94d, 0x935, 0x93f, 0x932, 0x939, 0x945, 0x932,
0x93f, 0x92b, 0x945, 0x915, 0x94d, 0x938, 0x935, 0x947, 0x935, 0x93e, 0x92f, 0x2d,
0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x928, 0x92e,
0x945, 0x915, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e,
0x907, 0x915, 0x93e, 0x932, 0x941, 0x907, 0x924, 0x91c, 0x94d, 0x92f, 0x942, 0x928,
0x94c, 0x932, 0x94d, 0x92f, 0x941, 0x907, 0x938, 0x935, 0x94d, 0x939, 0x93f, 0x932,
0x92e, 0x93e, 0x901, 0x91f, 0x93f, 0x938, 0x947, 0x932, 0x94b, 0x2c, 0x20, 0x915,
0x947, 0x902, 0x91f, 0x941, 0x915, 0x940, 0x915, 0x94d, 0x930, 0x93e, 0x932, 0x947,
0x902, 0x926, 0x93f, 0x91c, 0x93f, 0x915, 0x932, 0x949, 0x938, 0x20, 0x90f, 0x902,
0x91c, 0x947, 0x932, 0x93f, 0x938, 0x932, 0x94b, 0x905, 0x930, 0x20, 0x92a, 0x94d,
0x930, 0x93f, 0x928, 0x94d, 0x938, 0x938, 0x20, 0x915, 0x94d, 0x935, 0x93e, 0x930,
0x94d, 0x91f, 0x930, 0x92e, 0x947, 0x938, 0x947, 0x907, 0x913, 0x92e, 0x947, 0x930,
0x940, 0x917, 0x94b, 0x91f, 0x92e, 0x93e, 0x91d, 0x93e, 0x924, 0x932, 0x93e, 0x928,
0x92e, 0x947, 0x924, 0x932, 0x93e, 0x915, 0x924, 0x932, 0x93e, 0x92e, 0x93e, 0x901,
0x915, 0x91f, 0x928, 0x92e, 0x949, 0x928, 0x94d, 0x91f, 0x947, 0x930, 0x947, 0x92e,
0x94b, 0x928, 0x94d, 0x91f, 0x947, 0x935, 0x94d, 0x939, 0x93f, 0x921, 0x93f, 0x92f,
0x94b, 0x92e, 0x93e, 0x901, 0x928, 0x94d, 0x91f, 0x938, 0x947, 0x930, 0x93e, 0x924,
0x92c, 0x94d, 0x92f, 0x941, 0x932, 0x93e, 0x939, 0x2c, 0x20, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x92e, 0x927, 0x94d, 0x92f,
0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f,
0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c,
0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e,
0x92a, 0x93e, 0x930, 0x92e, 0x930, 0x940, 0x92c, 0x94b, 0x92b, 0x949, 0x928, 0x93f,
0x915, 0x94d, 0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x2d, 0x914, 0x2d, 0x92a,
0x94d, 0x930, 0x93f, 0x902, 0x938, 0x92a, 0x94d, 0x92f, 0x941, 0x90f, 0x930, 0x94d,
0x924, 0x94b, 0x20, 0x930, 0x93f, 0x915, 0x94b, 0x92a, 0x941, 0x902, 0x924, 0x93e,
0x20, 0x905, 0x930, 0x940, 0x928, 0x93e, 0x938, 0x930, 0x945, 0x928, 0x915, 0x93f,
0x928, 0x20, 0x907, 0x928, 0x932, 0x947, 0x91f, 0x930, 0x947, 0x91c, 0x94b, 0x932,
0x94d, 0x92f, 0x942, 0x91f, 0x938, 0x901, 0x91f, 0x93e, 0x930, 0x947, 0x92e, 0x938,
0x945, 0x928, 0x94d, 0x91f, 0x93f, 0x90f, 0x917, 0x94b, 0x938, 0x945, 0x928, 0x94d,
0x91f, 0x94b, 0x20, 0x921, 0x94b, 0x92e, 0x93f, 0x902, 0x917, 0x94b, 0x938, 0x93f,
0x91f, 0x915, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x932, 0x941, 0x938, 0x93f,
0x92f, 0x93e, 0x91f, 0x947, 0x917, 0x941, 0x938, 0x93f, 0x917, 0x93e, 0x932, 0x94d,
0x92a, 0x93e, 0x91f, 0x94b, 0x930, 0x94b, 0x902, 0x91f, 0x94b, 0x935, 0x94d, 0x939,
0x945, 0x928, 0x915, 0x941, 0x935, 0x94d, 0x939, 0x930, 0x935, 0x94d, 0x939, 0x93e,
0x907, 0x91f, 0x939, 0x949, 0x930, 0x94d, 0x938, 0x92f, 0x915, 0x941, 0x924, 0x93e,
0x924, 0x915, 0x945, 0x938, 0x947, 0x921, 0x947, 0x935, 0x94d, 0x939, 0x93f, 0x938,
0x921, 0x94d, 0x92f, 0x941, 0x92e, 0x949, 0x928, 0x94d, 0x91f, 0x20, 0x921, 0x94d,
0x92f, 0x941, 0x930, 0x94d, 0x935, 0x93f, 0x932, 0x92e, 0x945, 0x915, 0x94d, 0x935,
0x93e, 0x92f, 0x930, 0x92e, 0x945, 0x915, 0x94d, 0x92e, 0x941, 0x930, 0x921, 0x94b,
0x92a, 0x93e, 0x92e, 0x947, 0x930, 0x935, 0x94b, 0x938, 0x94d, 0x91f, 0x94b, 0x915,
0x932, 0x93e, 0x901, 0x917, 0x907, 0x92f, 0x930, 0x92c, 0x940, 0x92f, 0x947, 0x928,
0x90f, 0x928, 0x93e, 0x921, 0x940, 0x92f, 0x930, 0x905, 0x200d, 0x945, 0x915, 0x94d,
0x91f, 0x94c, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x905,
0x924, 0x93f, 0x930, 0x949, 0x92c, 0x939, 0x93e, 0x930, 0x93f, 0x928, 0x92c, 0x930,
0x94d, 0x928, 0x94c, 0x932, 0x92c, 0x94d, 0x930, 0x941, 0x928, 0x947, 0x908, 0x92b,
0x945, 0x92e, 0x917, 0x941, 0x938, 0x94d, 0x924, 0x93e, 0x939, 0x94b, 0x20, 0x91a,
0x93f, 0x20, 0x92e, 0x93f, 0x928, 0x94d, 0x939, 0x20, 0x936, 0x939, 0x930, 0x939,
0x93e, 0x901, 0x917, 0x915, 0x93e, 0x901, 0x917, 0x91c, 0x947, 0x930, 0x941, 0x938,
0x932, 0x947, 0x92e, 0x915, 0x945, 0x92e, 0x91a, 0x93e, 0x91f, 0x915, 0x93e, 0x916,
0x902, 0x926, 0x93f, 0x917, 0x93e, 0x915, 0x94d, 0x935, 0x93e, 0x932, 0x93e, 0x932,
0x902, 0x92a, 0x942, 0x930, 0x92e, 0x947, 0x917, 0x93e, 0x921, 0x928, 0x92e, 0x915,
0x938, 0x94d, 0x938, 0x93e, 0x930, 0x928, 0x94b, 0x935, 0x94b, 0x915, 0x941, 0x91d,
0x94d, 0x928, 0x947, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x92a, 0x94d, 0x928, 0x94b,
0x92e, 0x20, 0x92a, 0x947, 0x928, 0x94d, 0x939, 0x92a, 0x93e, 0x901, 0x91f, 0x93f,
0x92f, 0x93e, 0x928, 0x93e, 0x915, 0x915, 0x93f, 0x91d, 0x940, 0x932, 0x94b, 0x930,
0x94d, 0x921, 0x93e, 0x938, 0x947, 0x909, 0x932, 0x938, 0x94d, 0x930, 0x947, 0x926,
0x928, 0x947, 0x915, 0x94b, 0x932, 0x940, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x925,
0x93f, 0x902, 0x92b, 0x942, 0x91f, 0x94b, 0x915, 0x93f, 0x92f, 0x94b, 0x924, 0x94b,
0x92e, 0x938, 0x915, 0x935, 0x94d, 0x939, 0x93f, 0x90f, 0x928, 0x94d, 0x91f, 0x93e,
0x907, 0x928, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d,
0x91f, 0x94b, 0x915, 0x92f, 0x947, 0x915, 0x94d, 0x924, 0x947, 0x930, 0x93f, 0x928,
0x92c, 0x930, 0x94d, 0x917, 0x92f, 0x947, 0x930, 0x947, 0x935, 0x928, 0x915, 0x947,
0x92a, 0x20, 0x935, 0x94d, 0x939, 0x930, 0x94d, 0x921, 0x947, 0x92b, 0x945, 0x930,
0x94b, 0x92e, 0x921, 0x940, 0x92f, 0x930, 0x93e, 0x930, 0x947, 0x92f, 0x915, 0x94d,
0x91c, 0x93e, 0x935, 0x93f, 0x915, 0x938, 0x947, 0x902, 0x91f, 0x2e, 0x20, 0x939,
0x947, 0x932, 0x947, 0x928, 0x93e, 0x938, 0x94d, 0x91f, 0x945, 0x928, 0x932, 0x947,
0x90f, 0x921, 0x932, 0x947, 0x921, 0x932, 0x93f, 0x902, 0x921, 0x947, 0x92e, 0x928,
0x905, 0x200d, 0x945, 0x92e, 0x938, 0x94d, 0x91f, 0x930, 0x921, 0x945, 0x92e, 0x905,
0x901, 0x921, 0x94b, 0x930, 0x93e, 0x92c, 0x94d, 0x930, 0x93e, 0x924, 0x93f, 0x938,
0x94d, 0x932, 0x93e, 0x935, 0x94d, 0x939, 0x93e, 0x91a, 0x93f, 0x938, 0x93f, 0x928,
0x94c, 0x917, 0x94d, 0x935, 0x947, 0x930, 0x94d, 0x928, 0x938, 0x947, 0x906, 0x92f,
0x932, 0x20, 0x911, 0x92b, 0x20, 0x92e, 0x945, 0x928, 0x915, 0x932, 0x93f, 0x928,
0x93f, 0x928, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x932, 0x941, 0x92c, 0x94d, 0x932,
0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x932, 0x915, 0x94d, 0x91d, 0x947, 0x902, 0x92c,
0x930, 0x94d, 0x917, 0x92e, 0x930, 0x93f, 0x92f, 0x947, 0x939, 0x93e, 0x92e, 0x947,
0x928, 0x911, 0x938, 0x94d, 0x932, 0x94b, 0x938, 0x945, 0x928, 0x20, 0x92e, 0x930,
0x93f, 0x928, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x91c, 0x947, 0x935, 0x94d, 0x939,
0x94b, 0x938, 0x93f, 0x92e, 0x94d, 0x92b, 0x930, 0x94b, 0x92a, 0x94b, 0x932, 0x938,
0x94d, 0x915, 0x949, 0x92a, 0x94d, 0x91c, 0x947, 0x924, 0x93e, 0x932, 0x93f, 0x928,
0x935, 0x921, 0x942, 0x91d, 0x935, 0x94d, 0x939, 0x945, 0x91f, 0x93f, 0x915, 0x928,
0x935, 0x94d, 0x939, 0x93f, 0x90f, 0x928, 0x94d, 0x928, 0x93e, 0x935, 0x93f, 0x932,
0x928, 0x93f, 0x92f, 0x938, 0x935, 0x949, 0x930, 0x94d, 0x938, 0x93e, 0x91d, 0x945,
0x917, 0x94d, 0x930, 0x947, 0x92c, 0x91d, 0x941, 0x930, 0x93f, 0x915, 0x916, 0x94d,
0x930, 0x93f, 0x938, 0x92e, 0x938, 0x915, 0x930, 0x917, 0x94d, 0x92f, 0x942, 0x932,
0x947, 0x928, 0x92e, 0x93e, 0x92f, 0x94b, 0x91f, 0x94d, 0x91f, 0x947, 0x911, 0x915,
0x932, 0x902, 0x921, 0x92c, 0x949, 0x917, 0x945, 0x928, 0x935, 0x94d, 0x939, 0x93f,
0x932, 0x91a, 0x945, 0x91f, 0x939, 0x945, 0x92e, 0x92b, 0x941, 0x928, 0x93e, 0x92b,
0x941, 0x924, 0x940, 0x917, 0x945, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938,
0x917, 0x945, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x939, 0x94b, 0x928, 0x94b,
0x932, 0x941, 0x932, 0x942, 0x915, 0x94b, 0x936, 0x93e, 0x92f, 0x915, 0x94d, 0x935,
0x93e, 0x91c, 0x93e, 0x932, 0x947, 0x908, 0x928, 0x92e, 0x91c, 0x941, 0x930, 0x94b,
0x92e, 0x93e, 0x930, 0x915, 0x94d, 0x935, 0x93f, 0x938, 0x93e, 0x938, 0x928, 0x909,
0x930, 0x941, 0x928, 0x94d, 0x92f, 0x941, 0x90f, 0x928, 0x94c, 0x92e, 0x93f, 0x906,
0x92a, 0x945, 0x917, 0x94b, 0x20, 0x92a, 0x945, 0x917, 0x94b, 0x92a, 0x93f, 0x91f,
0x915, 0x947, 0x930, 0x94d, 0x928, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x92e,
0x94b, 0x930, 0x947, 0x938, 0x94d, 0x92c, 0x947, 0x938, 0x948, 0x92a, 0x93e, 0x928,
0x410, 0x431, 0x438, 0x436, 0x430, 0x43d, 0x410, 0x434, 0x434, 0x438, 0x441, 0x2d,
0x41, 0x431, 0x435, 0x431, 0x430, 0x411, 0x430, 0x43d, 0x433, 0x443, 0x439, 0x411,
0x438, 0x441, 0x441, 0x430, 0x443, 0x410, 0x43d, 0x43a, 0x43e, 0x440, 0x430, 0x436,
0x410, 0x440, 0x430, 0x433, 0x443, 0x430, 0x44f, 0x43d, 0x430, 0x411, 0x430, 0x439,
0x44f, 0x411, 0x430, 0x445, 0x44c, 0x44f, 0x20, 0x411, 0x430, 0x43d, 0x434, 0x435,
0x440, 0x430, 0x441, 0x41a, 0x44d, 0x43c, 0x431, 0x440, 0x438, 0x436, 0x438, 0x439,
0x43d, 0x20, 0x431, 0x443, 0x43b, 0x430, 0x43d, 0x41a, 0x430, 0x43c, 0x43f, 0x43e,
0x2d, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x435, 0x421, 0x44c, 0x44e, 0x434, 0x430,
0x434, 0x20, 0x425, 0x443, 0x430, 0x440, 0x435, 0x441, 0x414, 0x430, 0x43d, 0x43c,
0x430, 0x440, 0x43a, 0x448, 0x430, 0x432, 0x43d, 0x414, 0x43e, 0x443, 0x441, 0x43e,
0x43d, 0x20, 0x41a, 0x440, 0x438, 0x43a, 0x42d, 0x43b, 0x44c, 0x20, 0x421, 0x430,
0x43b, 0x44c, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x424, 0x43e, 0x440, 0x442, 0x20,
0x41d, 0x44d, 0x43b, 0x44c, 0x441, 0x43e, 0x43d, 0x413, 0x43b, 0x435, 0x439, 0x441,
0x20, 0x431, 0x443, 0x43b, 0x430, 0x43d, 0x413, 0x443, 0x443, 0x441, 0x20, 0x431,
0x443, 0x43b, 0x430, 0x43d, 0x425, 0x435, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b,
0x43b, 0x43e, 0x422, 0x435, 0x43b, 0x20, 0x421, 0x438, 0x442, 0x438, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x441, 0x435,
0x43d, 0x435, 0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430,
0x412, 0x438, 0x43d, 0x430, 0x43c, 0x430, 0x43a, 0x2c, 0x20, 0x418, 0x43d, 0x434,
0x438, 0x430, 0x43d, 0x430, 0x41c, 0x43e, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x43b,
0x43b, 0x43e, 0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442, 0x430, 0x43a, 0x438, 0x41a,
0x440, 0x430, 0x43b, 0x435, 0x43d, 0x434, 0x438, 0x43a, 0x41b, 0x43e, 0x432, 0x435,
0x440, 0x20, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x435, 0x441, 0x20, 0x41a, 0x432,
0x430, 0x440, 0x442, 0x435, 0x440, 0x41c, 0x430, 0x440, 0x438, 0x433, 0x43e, 0x442,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x430, 0x411, 0x438, 0x43b, 0x430, 0x2c, 0x20,
0x425, 0x43e, 0x439, 0x434, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x422,
0x4e9, 0x432, 0x2c, 0x20, 0x425, 0x43e, 0x439, 0x434, 0x20, 0x414, 0x430, 0x43a,
0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x430, 0x43b, 0x435, 0x43c,
0x2c, 0x20, 0x425, 0x43e, 0x439, 0x434, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442,
0x430, 0x41d, 0x4af, 0x4af, 0x43a, 0x41e, 0x436, 0x438, 0x43d, 0x430, 0x433, 0x430,
0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x2d, 0x41f, 0x440, 0x438, 0x43d, 0x441,
0x418, 0x441, 0x43f, 0x430, 0x43d, 0x438, 0x439, 0x20, 0x431, 0x43e, 0x43e, 0x43c,
0x442, 0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x2d, 0x412, 0x435, 0x43b, 0x44c, 0x44e,
0x420, 0x438, 0x43e, 0x2d, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x43e, 0x421, 0x435,
0x43d, 0x442, 0x2d, 0x411, 0x430, 0x440, 0x442, 0x435, 0x43b, 0x44c, 0x43c, 0x438,
0x421, 0x435, 0x43d, 0x442, 0x2d, 0x416, 0x43e, 0x43d, 0x441, 0x421, 0x435, 0x43d,
0x442, 0x2d, 0x41a, 0x438, 0x442, 0x442, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x20,
0x41b, 0x44e, 0x441, 0x438, 0x430, 0x423, 0x430, 0x439, 0x442, 0x445, 0x43e, 0x440,
0x437, 0x41a, 0x44d, 0x441, 0x435, 0x439, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20,
0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x44c, 0x41c, 0x430, 0x43a, 0x2d,
0x41c, 0x4e9, 0x440, 0x434, 0x43e, 0x41b, 0x43e, 0x43d, 0x433, 0x438, 0x440, 0x431,
0x430, 0x439, 0x435, 0x43d, 0x410, 0x43a, 0x442, 0x4e9, 0x431, 0x435, 0x425, 0x43e,
0x20, 0x428, 0x438, 0x20, 0x41c, 0x438, 0x43d, 0x20, 0x445, 0x43e, 0x442, 0x42d,
0x440, 0x445, 0x4af, 0x4af, 0x416, 0x430, 0x439, 0x43f, 0x443, 0x440, 0x41f, 0x451,
0x43d, 0x44c, 0x44f, 0x43d, 0x421, 0x4e9, 0x4af, 0x43b, 0x423, 0x43b, 0x430, 0x430,
0x43d, 0x431, 0x430, 0x430, 0x442, 0x430, 0x440, 0x4ae, 0x440, 0x4af, 0x43c, 0x447,
0x438, 0x4e8, 0x43c, 0x43d, 0x4e9, 0x434, 0x20, 0x416, 0x43e, 0x440, 0x436, 0x438,
0x430, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x425, 0x435, 0x43b, 0x435, 0x43d, 0x430,
0x411, 0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x20, 0x425, 0x438, 0x43b, 0x43b, 0x415,
0x443, 0x43a, 0x43b, 0x430, 0x41b, 0x438, 0x43d, 0x434, 0x435, 0x43c, 0x44d, 0x43d,
0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x41f, 0x435, 0x440, 0x441,
0x41a, 0x438, 0x448, 0x438, 0x43d, 0x451, 0x432, 0x41c, 0x44d, 0x43d, 0x20, 0x410,
0x440, 0x430, 0x43b, 0x41b, 0x438, 0x441, 0x431, 0x43e, 0x43d, 0x41f, 0x430, 0x440,
0x438, 0x441, 0x420, 0x43e, 0x43c, 0x41e, 0x443, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x427, 0x4af, 0x4af, 0x43a, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x440, 0x41a,
0x43e, 0x441, 0x440, 0x44d, 0x41a, 0x432, 0x430, 0x436, 0x430, 0x43b, 0x435, 0x439,
0x43d, 0x423, 0x44d, 0x43a, 0x182c, 0x1823, 0x182a, 0x1833, 0x1823, 0x1823, 0x182f, 0x1820,
0x182d, 0x1820, 0x1828, 0x182a, 0x1820, 0x182d, 0x1820, 0x1833, 0x1823, 0x1837, 0x905, 0x935,
0x93f, 0x921, 0x94d, 0x91c, 0x93e, 0x928, 0x905, 0x915, 0x94d, 0x930, 0x93e, 0x90f,
0x921, 0x94d, 0x921, 0x93f, 0x938, 0x20, 0x906, 0x92c, 0x93e, 0x92c, 0x93e, 0x905,
0x932, 0x94d, 0x91c, 0x93f, 0x92f, 0x930, 0x94d, 0x938, 0x92c, 0x94b, 0x92e, 0x93e,
0x915, 0x94b, 0x92c, 0x93e, 0x919, 0x94d, 0x917, 0x941, 0x935, 0x940, 0x92c, 0x94d,
0x932, 0x93e, 0x928, 0x94d, 0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d, 0x930, 0x93e,
0x91c, 0x93e, 0x92d, 0x93f, 0x932, 0x94d, 0x932, 0x947, 0x915, 0x93e, 0x907, 0x930,
0x94b, 0x915, 0x93e, 0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x915,
0x93e, 0x921, 0x93e, 0x930, 0x20, 0x90f, 0x938, 0x94d, 0x20, 0x938, 0x932, 0x93e,
0x92e, 0x91c, 0x93f, 0x92c, 0x94c, 0x902, 0x91f, 0x940, 0x921, 0x94b, 0x909, 0x906,
0x932, 0x93e, 0x90f, 0x932, 0x94d, 0x20, 0x906, 0x907, 0x92f, 0x941, 0x928, 0x92b,
0x94d, 0x930, 0x93f, 0x91f, 0x93e, 0x909, 0x928, 0x917, 0x93e, 0x935, 0x94b, 0x930,
0x94b, 0x928, 0x91c, 0x94b, 0x939, 0x93e, 0x928, 0x947, 0x938, 0x935, 0x930, 0x94d,
0x917, 0x916, 0x93e, 0x930, 0x94d, 0x91f, 0x909, 0x92e, 0x932, 0x93f, 0x92c, 0x94d,
0x930, 0x947, 0x92d, 0x93f, 0x932, 0x94d, 0x932, 0x947, 0x932, 0x941, 0x935, 0x93e,
0x928, 0x94d, 0x921, 0x93e, 0x932, 0x941, 0x92c, 0x941, 0x92e, 0x94d, 0x92c, 0x93e,
0x938, 0x940, 0x92e, 0x93e, 0x932, 0x93e, 0x92c, 0x94b, 0x92e, 0x93e, 0x92a, 0x941,
0x924, 0x94b, 0x92e, 0x94b, 0x917, 0x93e, 0x921, 0x93f, 0x936, 0x941, 0x92e, 0x94b,
0x928, 0x94d, 0x930, 0x94b, 0x92d, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x930,
0x94b, 0x92c, 0x940, 0x90f, 0x928, 0x94d, 0x200c, 0x91c, 0x93e, 0x92e, 0x947, 0x928,
0x93e, 0x928, 0x93e, 0x92f, 0x92e, 0x947, 0x928, 0x94b, 0x909, 0x906, 0x915, 0x91a,
0x94b, 0x91f, 0x914, 0x906, 0x917, 0x93e, 0x921, 0x94c, 0x917, 0x94c, 0x92a, 0x94b,
0x930, 0x94d, 0x91f, 0x94b, 0x2d, 0x928, 0x94b, 0x92d, 0x94b, 0x938, 0x93e, 0x913,
0x20, 0x91f, 0x94b, 0x92e, 0x947, 0x935, 0x93f, 0x928, 0x94d, 0x921, 0x939, 0x94b,
0x90f, 0x915, 0x906, 0x921, 0x93e, 0x915, 0x90f, 0x919, 0x94d, 0x915, 0x94b, 0x930,
0x947, 0x91c, 0x90f, 0x919, 0x94d, 0x917, 0x94d, 0x935, 0x93f, 0x932, 0x94d, 0x932,
0x93e, 0x906, 0x930, 0x93e, 0x917, 0x941, 0x935, 0x93e, 0x928, 0x93e, 0x92c, 0x94d,
0x92f, 0x928, 0x947, 0x936, 0x20, 0x906, 0x92f, 0x930, 0x94d, 0x938, 0x915, 0x93e,
0x924, 0x93e, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93e, 0x915, 0x94b, 0x930, 0x921,
0x94b, 0x935, 0x93e, 0x92e, 0x947, 0x928, 0x94d, 0x921, 0x94b, 0x91c, 0x93e, 0x930,
0x93f, 0x92f, 0x94b, 0x20, 0x917, 0x94d, 0x92f, 0x93e, 0x932, 0x947, 0x917, 0x94b,
0x938, 0x938, 0x93e, 0x928, 0x20, 0x91c, 0x941, 0x935, 0x93e, 0x928, 0x938, 0x93e,
0x928, 0x20, 0x932, 0x941, 0x907, 0x938, 0x909, 0x936, 0x941, 0x906, 0x907, 0x906,
0x905, 0x938, 0x928, 0x94d, 0x938, 0x93f, 0x92f, 0x928, 0x90f, 0x91f, 0x93f, 0x915,
0x94b, 0x915, 0x93e, 0x928, 0x92c, 0x93e, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x92c,
0x93e, 0x928, 0x94d, 0x921, 0x947, 0x930, 0x93e, 0x936, 0x92c, 0x94d, 0x932, 0x93e,
0x928, 0x94d, 0x915, 0x2d, 0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x94b, 0x928, 0x92c,
0x94b, 0x906, 0x20, 0x92d, 0x93f, 0x937, 0x94d, 0x91f, 0x93e, 0x92c, 0x94b, 0x907,
0x91c, 0x915, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92c, 0x94d, 0x930, 0x93f, 0x91c,
0x20, 0x92c, 0x947, 0x915, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92a, 0x94b, 0x20,
0x917, 0x94d, 0x930, 0x93e, 0x928, 0x94d, 0x921, 0x947, 0x915, 0x93e, 0x92f, 0x947,
0x928, 0x94d, 0x928, 0x947, 0x915, 0x947, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x91a,
0x93f, 0x939, 0x941, 0x935, 0x93e, 0x939, 0x941, 0x935, 0x93e, 0x91c, 0x941, 0x906,
0x930, 0x947, 0x91c, 0x20, 0x938, 0x939, 0x930, 0x915, 0x94b, 0x937, 0x94d, 0x91f,
0x93e, 0x20, 0x930, 0x93f, 0x915, 0x93e, 0x915, 0x94d, 0x92f, 0x941, 0x907, 0x906,
0x92c, 0x93e, 0x921, 0x93e, 0x928, 0x94d, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x936,
0x93e, 0x92d, 0x928, 0x921, 0x938, 0x928, 0x921, 0x938, 0x928, 0x20, 0x915, 0x94d,
0x930, 0x93f, 0x915, 0x921, 0x947, 0x928, 0x94d, 0x92d, 0x930, 0x921, 0x93f, 0x91f,
0x94d, 0x930, 0x94b, 0x907, 0x91f, 0x90f, 0x921, 0x94d, 0x92e, 0x94b, 0x928, 0x94d,
0x91f, 0x928, 0x906, 0x907, 0x930, 0x928, 0x947, 0x92a, 0x947, 0x90f, 0x932, 0x94d,
0x20, 0x938, 0x93e, 0x932, 0x94d, 0x92d, 0x93e, 0x921, 0x94b, 0x930, 0x917, 0x941,
0x91c, 0x20, 0x92c, 0x947, 0x917, 0x94d, 0x930, 0x93e, 0x928, 0x94d, 0x921, 0x20,
0x91f, 0x930, 0x94d, 0x915, 0x917, 0x94d, 0x935, 0x93e, 0x921, 0x947, 0x932, 0x941,
0x92a, 0x917, 0x941, 0x92f, 0x93e, 0x915, 0x94d, 0x935, 0x93f, 0x932, 0x939, 0x94d,
0x92f, 0x93e, 0x932, 0x93f, 0x92b, 0x94d, 0x92f, 0x93e, 0x915, 0x94d, 0x938, 0x939,
0x92d, 0x93e, 0x928, 0x93e, 0x939, 0x947, 0x930, 0x94d, 0x92e, 0x94b, 0x938, 0x93f,
0x932, 0x94d, 0x932, 0x94b, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x92a, 0x94b, 0x932, 0x93f, 0x938, 0x928, 0x94b, 0x915, 0x94d, 0x938, 0x20,
0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92e, 0x93e, 0x930,
0x947, 0x928, 0x94d, 0x917, 0x94b, 0x2c, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f,
0x92f, 0x93e, 0x928, 0x93e, 0x92a, 0x93f, 0x91f, 0x930, 0x94d, 0x938, 0x92c, 0x930,
0x94d, 0x917, 0x2c, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x91f, 0x947, 0x932, 0x20, 0x938, 0x93f, 0x91f, 0x940, 0x2c, 0x20, 0x907,
0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92d, 0x947, 0x92d, 0x947,
0x2c, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92d,
0x93f, 0x928, 0x94d, 0x938, 0x947, 0x928, 0x94d, 0x938, 0x935, 0x93f, 0x928, 0x93e,
0x92e, 0x93e, 0x915, 0x2c, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x907, 0x928, 0x941, 0x92d, 0x93f, 0x915, 0x907, 0x915, 0x94d, 0x935,
0x93e, 0x932, 0x941, 0x907, 0x91f, 0x91c, 0x92e, 0x93e, 0x907, 0x915, 0x93e, 0x91c,
0x941, 0x928, 0x93f, 0x909, 0x932, 0x941, 0x907, 0x938, 0x92d, 0x93f, 0x932, 0x94d,
0x932, 0x947, 0x92e, 0x94b, 0x928, 0x94d, 0x91f, 0x93f, 0x938, 0x947, 0x932, 0x94d,
0x932, 0x94b, 0x2c, 0x915, 0x947, 0x928, 0x94d, 0x91f, 0x915, 0x940, 0x915, 0x93e,
0x932, 0x947, 0x928, 0x94d, 0x926, 0x947, 0x91c, 0x93f, 0x915, 0x932, 0x938, 0x94d,
0x20, 0x90f, 0x928, 0x94d, 0x91c, 0x947, 0x932, 0x938, 0x932, 0x94b, 0x905, 0x930,
0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938, 0x94d, 0x938, 0x20, 0x915,
0x94d, 0x935, 0x93e, 0x930, 0x94d, 0x91f, 0x930, 0x92e, 0x93e, 0x938, 0x947, 0x907,
0x913, 0x92e, 0x93e, 0x928, 0x93e, 0x917, 0x941, 0x935, 0x93e, 0x92e, 0x93e, 0x928,
0x93e, 0x909, 0x938, 0x92e, 0x94d, 0x92f, 0x93e, 0x930, 0x93f, 0x917, 0x91f, 0x92e,
0x93e, 0x91f, 0x94d, 0x924, 0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x938, 0x94d, 0x92e,
0x93e, 0x91c, 0x93e, 0x91f, 0x94d, 0x932, 0x93e, 0x928, 0x92e, 0x947, 0x91f, 0x94d,
0x932, 0x93e, 0x915, 0x94d, 0x91f, 0x932, 0x93e, 0x92e, 0x93f, 0x915, 0x94d, 0x935,
0x93f, 0x932, 0x928, 0x92e, 0x94b, 0x928, 0x94d, 0x915, 0x91f, 0x928, 0x92e, 0x94b,
0x928, 0x94d, 0x91f, 0x947, 0x930, 0x94d, 0x930, 0x947, 0x92e, 0x94b, 0x928, 0x94d,
0x91f, 0x947, 0x92d, 0x93f, 0x921, 0x93f, 0x92f, 0x94b, 0x92e, 0x928, 0x94d, 0x91f,
0x938, 0x947, 0x930, 0x94d, 0x930, 0x93e, 0x91f, 0x928, 0x94d, 0x92f, 0x941, 0x92f,
0x94b, 0x930, 0x94d, 0x915, 0x92c, 0x947, 0x909, 0x932, 0x93e, 0x2c, 0x20, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x2c,
0x20, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x928, 0x92f, 0x93e, 0x901,
0x20, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x92a, 0x93e, 0x930, 0x93e,
0x92e, 0x93e, 0x930, 0x93f, 0x935, 0x94b, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x2d,
0x905, 0x909, 0x2d, 0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938, 0x92a, 0x94b,
0x930, 0x94d, 0x91f, 0x20, 0x905, 0x92b, 0x94d, 0x20, 0x938, 0x94d, 0x92a, 0x947,
0x928, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x92d, 0x947, 0x932, 0x94d,
0x939, 0x94b, 0x92a, 0x94d, 0x92f, 0x941, 0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x930,
0x93f, 0x915, 0x94b, 0x92a, 0x941, 0x928, 0x94d, 0x91f, 0x93e, 0x20, 0x905, 0x930,
0x93f, 0x928, 0x93e, 0x91c, 0x930, 0x93e, 0x928, 0x94d, 0x915, 0x93f, 0x928, 0x20,
0x907, 0x928, 0x94d, 0x932, 0x947, 0x91f, 0x930, 0x93f, 0x938, 0x93e, 0x907, 0x92b,
0x930, 0x93f, 0x91c, 0x94b, 0x932, 0x941, 0x91f, 0x930, 0x93f, 0x92f, 0x94b, 0x20,
0x92c, 0x94d, 0x930, 0x93e, 0x928, 0x94d, 0x915, 0x94b, 0x938, 0x93e, 0x928, 0x94d,
0x91f, 0x93e, 0x930, 0x947, 0x92e, 0x938, 0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x906,
0x917, 0x94b, 0x938, 0x93e, 0x928, 0x94d, 0x91f, 0x94b, 0x20, 0x921, 0x94b, 0x92e,
0x93f, 0x919, 0x94d, 0x917, 0x94b, 0x908, 0x91f, 0x94d, 0x91f, 0x94b, 0x915, 0x94d,
0x915, 0x94b, 0x930, 0x91f, 0x942, 0x930, 0x94d, 0x92e, 0x93f, 0x91f, 0x938, 0x947,
0x928, 0x94d, 0x91f, 0x20, 0x92c, 0x93e, 0x930, 0x94d, 0x925, 0x947, 0x932, 0x947,
0x92e, 0x940, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x91c, 0x94b, 0x928, 0x94d,
0x938, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x94d, 0x20, 0x915, 0x93f, 0x91f, 0x94d,
0x938, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x932, 0x941, 0x938, 0x93f, 0x92f,
0x93e, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x925, 0x94b, 0x92e, 0x938, 0x938,
0x947, 0x928, 0x94d, 0x91f, 0x20, 0x92d, 0x93f, 0x928, 0x94d, 0x938, 0x947, 0x928,
0x94d, 0x91f, 0x938, 0x94d, 0x935, 0x93f, 0x92b, 0x94d, 0x91f, 0x20, 0x915, 0x930,
0x947, 0x928, 0x94d, 0x91f, 0x91f, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x91f, 0x94b,
0x91f, 0x93e, 0x930, 0x94d, 0x91f, 0x94b, 0x932, 0x93e, 0x92d, 0x94d, 0x92f, 0x93e,
0x928, 0x915, 0x94b, 0x92d, 0x930, 0x939, 0x94d, 0x935, 0x93e, 0x907, 0x91f, 0x939,
0x930, 0x94d, 0x938, 0x935, 0x93f, 0x928, 0x94d, 0x928, 0x93f, 0x92a, 0x947, 0x917,
0x915, 0x947, 0x91c, 0x947, 0x926, 0x941, 0x92e, 0x94b, 0x928, 0x94d, 0x91f, 0x20,
0x921, 0x93f, 0x20, 0x909, 0x930, 0x94d, 0x92d, 0x947, 0x932, 0x94d, 0x932, 0x947,
0x92e, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x93f, 0x908, 0x92e, 0x93e, 0x909, 0x938,
0x928, 0x92e, 0x93e, 0x915, 0x92e, 0x941, 0x930, 0x94d, 0x921, 0x94b, 0x938, 0x93f,
0x913, 0x906, 0x92d, 0x93e, 0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x932, 0x919, 0x92f,
0x93f, 0x905, 0x930, 0x92c, 0x93e, 0x907, 0x90f, 0x928, 0x906, 0x928, 0x93e, 0x921,
0x93f, 0x92f, 0x930, 0x906, 0x915, 0x94d, 0x91f, 0x93e, 0x909, 0x906, 0x915, 0x94d,
0x91f, 0x94b, 0x92c, 0x905, 0x938, 0x94d, 0x915, 0x93e, 0x92c, 0x93e, 0x91f, 0x905,
0x91f, 0x93f, 0x930, 0x93e, 0x909, 0x92c, 0x939, 0x930, 0x93e, 0x908, 0x928, 0x92c,
0x948, 0x902, 0x915, 0x915, 0x92c, 0x947, 0x908, 0x930, 0x941, 0x91f, 0x92c, 0x93f,
0x938, 0x915, 0x947, 0x915, 0x94d, 0x92c, 0x94d, 0x930, 0x941, 0x928, 0x93e, 0x907,
0x926, 0x93e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x938, 0x94d, 0x926, 0x93f, 0x932,
0x94d, 0x932, 0x940, 0x926, 0x941, 0x92c, 0x939, 0x940, 0x926, 0x938, 0x94d, 0x938,
0x93e, 0x928, 0x94d, 0x92c, 0x947, 0x939, 0x93f, 0x92c, 0x94d, 0x930, 0x94b, 0x928,
0x939, 0x94b, 0x20, 0x91a, 0x940, 0x20, 0x92e, 0x93f, 0x928, 0x94d, 0x939, 0x20,
0x936, 0x939, 0x930, 0x939, 0x919, 0x915, 0x919, 0x939, 0x94b, 0x92d, 0x94d, 0x921,
0x915, 0x93e, 0x92e, 0x91a, 0x91f, 0x94d, 0x915, 0x93e, 0x915, 0x93e, 0x920, 0x92e,
0x93e, 0x923, 0x94d, 0x921, 0x94c, 0x902, 0x916, 0x93e, 0x928, 0x94d, 0x926, 0x93f,
0x917, 0x93e, 0x915, 0x941, 0x906, 0x20, 0x932, 0x93e, 0x92e, 0x94d, 0x92a, 0x941,
0x930, 0x915, 0x941, 0x91a, 0x93f, 0x919, 0x92e, 0x915, 0x93e, 0x909, 0x92e, 0x93e,
0x917, 0x93e, 0x921, 0x93e, 0x928, 0x92e, 0x938, 0x94d, 0x915, 0x94d, 0x92f, 0x93e,
0x91f, 0x928, 0x947, 0x92d, 0x94b, 0x915, 0x941, 0x91c, 0x928, 0x947, 0x938, 0x94d,
0x915, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x92b, 0x947, 0x928, 0x94b, 0x92e, 0x20, 0x92b, 0x947, 0x928, 0x92a,
0x94b, 0x928, 0x94d, 0x91f, 0x93f, 0x906, 0x928, 0x93e, 0x915, 0x92a, 0x94d, 0x92f,
0x94b, 0x919, 0x92f, 0x93e, 0x919, 0x915, 0x938, 0x94d, 0x91f, 0x93e, 0x928, 0x947,
0x938, 0x92e, 0x93e, 0x930, 0x915, 0x923, 0x94d, 0x921, 0x938, 0x93e, 0x928, 0x94d,
0x918, 0x93e, 0x908, 0x938, 0x94d, 0x930, 0x947, 0x926, 0x94d, 0x928, 0x93f, 0x915,
0x94b, 0x932, 0x93f, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x924, 0x93e, 0x938, 0x94d,
0x915, 0x947, 0x928, 0x94d, 0x91f, 0x924, 0x947, 0x939, 0x947, 0x930, 0x93e, 0x928,
0x925, 0x93f, 0x92e, 0x94d, 0x92a, 0x941, 0x91f, 0x94b, 0x92e, 0x94d, 0x938, 0x94d,
0x915, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x91f, 0x93e, 0x930, 0x909, 0x938, 0x94d,
0x91f, 0x2d, 0x928, 0x947, 0x930, 0x93e, 0x92d, 0x93f, 0x92f, 0x928, 0x94d, 0x924,
0x93f, 0x928, 0x92d, 0x94d, 0x932, 0x93e, 0x921, 0x93f, 0x92d, 0x93e, 0x938, 0x94d,
0x91f, 0x94b, 0x915, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915,
0x930, 0x93e, 0x928, 0x94d, 0x917, 0x941, 0x928, 0x90f, 0x915, 0x91f, 0x930, 0x93f,
0x928, 0x92c, 0x941, 0x930, 0x94d, 0x917, 0x906, 0x91c, 0x94b, 0x930, 0x94d, 0x938,
0x915, 0x94d, 0x92f, 0x93e, 0x928, 0x93e, 0x930, 0x940, 0x915, 0x947, 0x92a, 0x20,
0x92d, 0x930, 0x94d, 0x921, 0x947, 0x92b, 0x93e, 0x930, 0x94b, 0x907, 0x92e, 0x93e,
0x921, 0x947, 0x907, 0x930, 0x93e, 0x930, 0x947, 0x915, 0x94d, 0x91c, 0x93e, 0x92d,
0x93f, 0x915, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x91c, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x939, 0x947,
0x932, 0x947, 0x928, 0x93e, 0x938, 0x94d, 0x91f, 0x94d, 0x92f, 0x93e, 0x928, 0x94d,
0x932, 0x940, 0x90f, 0x921, 0x947, 0x932, 0x947, 0x921, 0x907, 0x92f, 0x941, 0x915,
0x94d, 0x932, 0x93e, 0x939, 0x94b, 0x92d, 0x93e, 0x930, 0x94d, 0x91f, 0x932, 0x93f,
0x928, 0x94d, 0x921, 0x947, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x932, 0x930, 0x94d,
0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x92e, 0x947, 0x932, 0x94d, 0x92c, 0x930,
0x94d, 0x928, 0x938, 0x93f, 0x921, 0x94d, 0x928, 0x940, 0x90f, 0x92e, 0x94d, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x94d, 0x921, 0x92e, 0x906, 0x928, 0x94d, 0x921, 0x94b,
0x930, 0x94d, 0x930, 0x93e, 0x905, 0x938, 0x94d, 0x924, 0x94d, 0x930, 0x916, 0x93e,
0x928, 0x92c, 0x94d, 0x930, 0x93e, 0x91f, 0x93f, 0x938, 0x94d, 0x932, 0x93e, 0x92d,
0x93e, 0x935, 0x941, 0x91a, 0x93e, 0x930, 0x947, 0x938, 0x94d, 0x91f, 0x92c, 0x941,
0x938, 0x93f, 0x928, 0x94d, 0x928, 0x917, 0x928, 0x91a, 0x93f, 0x938, 0x93f, 0x928,
0x93e, 0x909, 0x915, 0x94b, 0x92a, 0x947, 0x928, 0x939, 0x94d, 0x92f, 0x93e, 0x917,
0x928, 0x917, 0x941, 0x90f, 0x930, 0x94d, 0x928, 0x938, 0x947, 0x939, 0x947, 0x932,
0x94d, 0x938, 0x93f, 0x928, 0x94d, 0x915, 0x940, 0x907, 0x91c, 0x932, 0x20, 0x905,
0x92b, 0x94d, 0x20, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x908, 0x938, 0x94d, 0x91f,
0x93e, 0x928, 0x92c, 0x941, 0x932, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x93f, 0x928,
0x917, 0x94d, 0x930, 0x93e, 0x926, 0x915, 0x93f, 0x930, 0x94b, 0x92d, 0x915, 0x93f,
0x92d, 0x932, 0x93f, 0x938, 0x94d, 0x92c, 0x94b, 0x928, 0x932, 0x91c, 0x941, 0x92c,
0x93f, 0x91c, 0x93e, 0x928, 0x93e, 0x932, 0x923, 0x94d, 0x921, 0x928, 0x932, 0x915,
0x94d, 0x91c, 0x947, 0x92e, 0x94d, 0x935, 0x930, 0x94d, 0x917, 0x92e, 0x94d, 0x92f,
0x93e, 0x921, 0x94d, 0x930, 0x93f, 0x921, 0x92e, 0x94d, 0x92f, 0x93e, 0x930, 0x93f,
0x939, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x928, 0x92e, 0x938, 0x94d, 0x915, 0x94b,
0x92a, 0x921, 0x94d, 0x917, 0x94b, 0x930, 0x93f, 0x915, 0x93e, 0x938, 0x93e, 0x928,
0x20, 0x92e, 0x93e, 0x930, 0x93f, 0x928, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x91c,
0x947, 0x92d, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x91f, 0x94b, 0x92d, 0x938, 0x94d,
0x915, 0x94b, 0x92a, 0x91c, 0x947, 0x938, 0x94d, 0x91f, 0x915, 0x939, 0x94b, 0x932,
0x94d, 0x92e, 0x924, 0x93e, 0x932, 0x94d, 0x932, 0x93f, 0x928, 0x909, 0x932, 0x94d,
0x92f, 0x93e, 0x928, 0x94b, 0x92d, 0x94d, 0x938, 0x94d, 0x915, 0x92d, 0x94d, 0x92f,
0x93e, 0x91f, 0x93f, 0x915, 0x928, 0x92d, 0x93f, 0x92f, 0x928, 0x93e, 0x92d, 0x93f,
0x932, 0x94d, 0x928, 0x93f, 0x905, 0x938, 0x92d, 0x94b, 0x932, 0x94d, 0x917, 0x94b,
0x917, 0x94d, 0x930, 0x93e, 0x926, 0x935, 0x93e, 0x930, 0x938, 0x905, 0x91c, 0x941,
0x930, 0x93f, 0x915, 0x905, 0x928, 0x94d, 0x91f, 0x93e, 0x928, 0x93e, 0x930, 0x93f,
0x92d, 0x94b, 0x916, 0x94d, 0x930, 0x93f, 0x938, 0x94d, 0x91f, 0x92e, 0x938, 0x915,
0x947, 0x930, 0x94d, 0x917, 0x941, 0x90f, 0x932, 0x947, 0x928, 0x92e, 0x93e, 0x932,
0x94d, 0x926, 0x93f, 0x92d, 0x94d, 0x938, 0x92e, 0x909, 0x930, 0x93f, 0x91f, 0x93f,
0x905, 0x938, 0x905, 0x915, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x92c,
0x941, 0x917, 0x947, 0x928, 0x92d, 0x93f, 0x932, 0x94d, 0x932, 0x947, 0x907, 0x938,
0x94d, 0x91f, 0x930, 0x908, 0x92b, 0x93e, 0x924, 0x947, 0x92b, 0x941, 0x928, 0x93e,
0x92b, 0x941, 0x924, 0x93f, 0x917, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938,
0x917, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x917, 0x941,
0x905, 0x921, 0x93e, 0x932, 0x915, 0x928, 0x93e, 0x932, 0x90f, 0x928, 0x94d, 0x921,
0x930, 0x92c, 0x930, 0x940, 0x915, 0x93f, 0x930, 0x93f, 0x924, 0x93f, 0x92e, 0x93e,
0x91f, 0x940, 0x915, 0x94b, 0x938, 0x94d, 0x930, 0x93e, 0x90f, 0x915, 0x94d, 0x935,
0x93e, 0x91c, 0x93e, 0x932, 0x947, 0x907, 0x928, 0x928, 0x93e, 0x909, 0x930, 0x941,
0x928, 0x93f, 0x909, 0x908, 0x928, 0x930, 0x92b, 0x94b, 0x932, 0x94d, 0x915, 0x928,
0x94b, 0x909, 0x92e, 0x947, 0x905, 0x92a, 0x932, 0x93e, 0x909, 0x92a, 0x93f, 0x924,
0x915, 0x93e, 0x908, 0x930, 0x928, 0x92a, 0x94b, 0x928, 0x93e, 0x92a, 0x947, 0x930,
0x93e, 0x913, 0x924, 0x94b, 0x902, 0x917, 0x93e, 0x938, 0x93e, 0x908, 0x92a, 0x928,
0x924, 0x930, 0x935, 0x93e, 0x91f, 0x902, 0x917, 0x93e, 0x924, 0x92a, 0x941, 0xc1,
0x62, 0x69, 0x6a, 0x61, 0x6e, 0x41, 0x6b, 0x72, 0xe1, 0x41, 0x64, 0xed,
0x20, 0x41, 0x62, 0xe1, 0x62, 0x61, 0x41, 0x6c, 0x6a, 0xed, 0x1eb9, 0x7a,
0x41, 0x73, 0x6d, 0xe1, 0x72, 0x61, 0x42, 0x61, 0x6d, 0xe1, 0x6b, 0x6f,
0x42, 0x61, 0x6e, 0x67, 0xfa, 0x69, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0xe1,
0x79, 0x61, 0x42, 0x72, 0xe1, 0x7a, 0x61, 0x76, 0x69, 0x6c, 0x42, 0x75,
0x6a, 0x75, 0x6d, 0x62, 0xfa, 0x72, 0x61, 0x4b, 0x61, 0xed, 0x72, 0x6f,
0x4b, 0x61, 0x73, 0x61, 0x62, 0x6c, 0xe1, 0x6e, 0x6b, 0x61, 0x53, 0x1eb9,
0xfa, 0x74, 0x61, 0x4b, 0x1ecd, 0x6e, 0xe1, 0x6b, 0x72, 0x69, 0x44, 0x61,
0x6b, 0xe1, 0x72, 0x44, 0x61, 0x72, 0x20, 0x1eb9, 0x301, 0x73, 0x20, 0x53,
0x61, 0x6c, 0x61, 0x6d, 0x4a, 0x69, 0x62, 0xfa, 0x74, 0x69, 0x44, 0x75,
0xe1, 0x6c, 0x61, 0x1eb8, 0x6c, 0x20, 0x41, 0x69, 0x75, 0x6e, 0x46, 0x72,
0xed, 0x74, 0x61, 0x75, 0x6e, 0x48, 0xe1, 0x62, 0x6f, 0x72, 0xf3, 0x6e,
0x1eb9, 0x48, 0x61, 0x72, 0xe1, 0x72, 0x1eb9, 0x4a, 0x6f, 0x68, 0xe1, 0x6e,
0xed, 0x73, 0x62, 0x1ecd, 0x67, 0x4a, 0xfa, 0x62, 0x61, 0x4b, 0x61, 0x6d,
0x70, 0xe1, 0x6c, 0x61, 0x4b, 0x69, 0x67, 0xe1, 0x6c, 0x69, 0x4b, 0x69,
0x6e, 0x73, 0x68, 0xe1, 0x73, 0x68, 0x61, 0x4c, 0xe9, 0x67, 0x6f, 0x73,
0x4c, 0xed, 0x62, 0x72, 0x1eb9, 0x76, 0x69, 0x6c, 0x4c, 0xf3, 0x6d, 0x1eb9,
0x4c, 0x75, 0xe1, 0x6e, 0x64, 0x61, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62,
0xe1, 0x73, 0x68, 0x69, 0x4c, 0x75, 0x73, 0xe1, 0x6b, 0x61, 0x4d, 0x61,
0x6c, 0xe1, 0x62, 0x6f, 0x4d, 0x61, 0x70, 0xfa, 0x74, 0x6f, 0x4d, 0x61,
0x73, 0x1eb9, 0x301, 0x72, 0x75, 0x4d, 0x62, 0x61, 0x62, 0xe1, 0x6e, 0x1eb9,
0x4d, 0x1ecd, 0x67, 0xe1, 0x64, 0xed, 0x73, 0x68, 0x75, 0x4d, 0x6f, 0x6e,
0x72, 0xf3, 0x76, 0x69, 0x61, 0x4e, 0x61, 0xed, 0x72, 0xf3, 0x62, 0x69,
0x4e, 0x6a, 0x61, 0x6d, 0x1eb9, 0x301, 0x6e, 0x61, 0x4e, 0x69, 0xe1, 0x6d,
0x65, 0x4e, 0x6f, 0x75, 0xe1, 0x6b, 0x73, 0x68, 0x1ecd, 0x74, 0x4f, 0x75,
0x61, 0x67, 0x61, 0x64, 0xfa, 0x67, 0x75, 0x50, 0x1ecd, 0x74, 0x6f, 0x2d,
0x4e, 0xf3, 0x76, 0x6f, 0x53, 0x61, 0x6f, 0x20, 0x54, 0xf3, 0x6d, 0x1eb9,
0x54, 0x72, 0xed, 0x70, 0x1ecd, 0x301, 0x6c, 0x69, 0x57, 0xed, 0x6e, 0x64,
0x68, 0x6f, 0x6b, 0xc1, 0x64, 0x61, 0x6b, 0xc1, 0x6e, 0x6b, 0x1ecd, 0x301,
0x72, 0x65, 0x6a, 0x41, 0x6e, 0x67, 0x77, 0xed, 0x6c, 0x61, 0x41, 0x6e,
0x74, 0xed, 0x67, 0x77, 0x75, 0x61, 0x41, 0x72, 0x61, 0x67, 0x77, 0x75,
0x61, 0xed, 0x6e, 0x61, 0x42, 0x75, 0x1eb9, 0x6e, 0x6f, 0x73, 0x20, 0x41,
0xed, 0x72, 0x1eb9, 0x73, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0xe1, 0x6b, 0x61,
0x4b, 0xf3, 0x72, 0x64, 0x6f, 0x62, 0x61, 0x4c, 0x61, 0x20, 0x52, 0x69,
0xf3, 0x6b, 0x68, 0x61, 0x4d, 0x1eb9, 0x6e, 0x64, 0xf3, 0x7a, 0x61, 0x52,
0x69, 0xf3, 0x20, 0x47, 0x61, 0x6c, 0x1eb9, 0x301, 0x67, 0x1ecd, 0x73, 0x53,
0xe1, 0x6c, 0x74, 0x61, 0x53, 0xe1, 0x6e, 0x20, 0x48, 0x77, 0xe1, 0x6e,
0x53, 0xe1, 0x6e, 0x20, 0x4c, 0x75, 0x69, 0x73, 0x54, 0xfa, 0x6b, 0xfa,
0x6d, 0x61, 0x6e, 0x55, 0x73, 0x75, 0xe1, 0x79, 0x61, 0x41, 0x73, 0x75,
0x6e, 0x73, 0x69, 0x1ecd, 0x6e, 0x41, 0x74, 0xed, 0x6b, 0xf3, 0x6b, 0x61,
0x6e, 0x42, 0x61, 0x68, 0xed, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x1eb9,
0x301, 0x72, 0x61, 0x73, 0x42, 0x61, 0x72, 0x62, 0xe9, 0x64, 0x1ecd, 0x73,
0x42, 0x1eb9, 0x6c, 0x1eb9, 0x6d, 0x42, 0x1eb9, 0x6c, 0x69, 0x7a, 0x42, 0x6c,
0x61, 0x6e, 0x6b, 0x2d, 0x53, 0xe1, 0x62, 0x6c, 0x1ecd, 0x6e, 0x42, 0xf3,
0x61, 0x20, 0x56, 0xed, 0x73, 0x74, 0x61, 0x42, 0x1ecd, 0xed, 0x73, 0x69,
0x4b, 0xe9, 0x6d, 0x62, 0x72, 0xed, 0x6a, 0x20, 0x42, 0xe8, 0x4b, 0x61,
0x6d, 0x70, 0xf3, 0x20, 0x47, 0x72, 0xe1, 0x6e, 0x64, 0x1eb9, 0x4b, 0x61,
0x72, 0xe1, 0x6b, 0x61, 0x73, 0x4b, 0x61, 0x79, 0x1eb9, 0x6e, 0x4b, 0xe9,
0x6d, 0x61, 0x6e, 0x43, 0x68, 0x69, 0x6b, 0xe1, 0x67, 0x6f, 0x43, 0x68,
0x69, 0x77, 0x75, 0xe1, 0x77, 0x75, 0x61, 0x4b, 0xf3, 0x73, 0x74, 0x61,
0x20, 0x52, 0xed, 0x6b, 0x61, 0x4b, 0x72, 0x1eb9, 0x301, 0x73, 0x74, 0x1ecd,
0x6e, 0x4b, 0xfa, 0x79, 0xe1, 0x62, 0x61, 0x61, 0x4b, 0x69, 0x75, 0x72,
0xe1, 0x73, 0x61, 0x6f, 0x44, 0x61, 0x6e, 0x6d, 0xe1, 0x6b, 0x73, 0x68,
0xe1, 0x76, 0x75, 0x6e, 0x44, 0x1ecd, 0x301, 0x73, 0x1ecd, 0x6e, 0x44, 0x1ecd,
0x301, 0x73, 0x1ecd, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x6b, 0x44, 0x1eb9, 0x301,
0x6e, 0x76, 0x61, 0x44, 0x69, 0x74, 0x72, 0x1ecd, 0x69, 0x74, 0x44, 0x1ecd,
0x6d, 0xed, 0x6e, 0xed, 0x6b, 0x61, 0x1eb8, 0x301, 0x64, 0x6d, 0x1ecd, 0x301,
0x6e, 0x74, 0x1ecd, 0x6e, 0x1eb8, 0x69, 0x72, 0x75, 0x6e, 0x1eb9, 0x70, 0x1eb9,
0x45, 0x6c, 0x20, 0x53, 0xe1, 0x6c, 0x76, 0xe1, 0x64, 0x1ecd, 0x46, 0x1ecd,
0x74, 0x20, 0x4e, 0x1eb9, 0x301, 0x6c, 0x73, 0x6f, 0x6e, 0x46, 0x1ecd, 0x74,
0x61, 0x6c, 0x1eb9, 0x301, 0x7a, 0x61, 0x47, 0x6c, 0xe1, 0x73, 0x20, 0x42,
0xe8, 0x47, 0xfa, 0x7a, 0x20, 0x42, 0xe8, 0x47, 0x72, 0xe1, 0x6e, 0x64,
0x20, 0x54, 0x1ecd, 0x6b, 0x47, 0x72, 0x1eb9, 0x6e, 0xe9, 0x64, 0x61, 0x47,
0x75, 0x61, 0x64, 0x61, 0x6c, 0xfa, 0x70, 0x1eb9, 0x47, 0x75, 0xe1, 0x74,
0x1eb9, 0x6d, 0xe1, 0x6c, 0x61, 0x47, 0x75, 0xe1, 0x79, 0x61, 0x6b, 0x69,
0x6c, 0x47, 0x61, 0x79, 0xe1, 0x6e, 0x61, 0x48, 0xe1, 0x6c, 0xed, 0x66,
0x61, 0x6b, 0x73, 0x48, 0x61, 0x76, 0xe1, 0x6e, 0x61, 0x48, 0x1eb9, 0x301,
0x6d, 0xf3, 0x73, 0xed, 0x6c, 0x6f, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e,
0xe1, 0x70, 0x1ecd, 0x301, 0x6c, 0x69, 0x73, 0x4e, 0x1ecd, 0x6b, 0x73, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x1eb9,
0x301, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e,
0x61, 0x50, 0xed, 0x74, 0xe1, 0x73, 0x62, 0x1ecd, 0x67, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61, 0x54, 0x1eb9, 0x301, 0x6c, 0x20, 0x53,
0xed, 0x74, 0x69, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61,
0x56, 0x1eb9, 0x301, 0x76, 0x65, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1,
0x6e, 0x61, 0x56, 0xed, 0x6e, 0x73, 0x1eb9, 0x6e, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0xe1, 0x6e, 0x61, 0x57, 0xed, 0x6e, 0xe1, 0x6d, 0x61, 0x6b,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61, 0x49, 0x6e, 0xfa,
0x76, 0x69, 0x6b, 0x49, 0x6b, 0xe1, 0x6c, 0x75, 0x69, 0x74, 0x4a, 0xfa,
0x6e, 0x6f, 0x4c, 0x75, 0xed, 0x76, 0x69, 0x6c, 0x4d, 0x1ecd, 0x6e, 0x74,
0x1eb9, 0x63, 0x68, 0x1eb9, 0x301, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x1eb9, 0x301,
0x6e, 0x74, 0x1ecd, 0x301, 0x6b, 0x69, 0x4b, 0x72, 0xe1, 0x6c, 0x1eb9, 0x301,
0x6e, 0x64, 0x69, 0x6a, 0x6b, 0x4c, 0xed, 0x6d, 0x61, 0x4c, 0x1ecd, 0x73,
0x20, 0xc1, 0x6e, 0x6a, 0x1eb9, 0x301, 0x6c, 0x69, 0x73, 0x4c, 0xf3, 0x77,
0xe1, 0x20, 0x50, 0x72, 0xed, 0x6e, 0x73, 0x20, 0x49, 0x6d, 0x20, 0x4b,
0x77, 0x1ecd, 0x301, 0x74, 0x61, 0x4d, 0x61, 0x73, 0x1eb9, 0x301, 0x69, 0x6f,
0x4d, 0x61, 0x6e, 0xe1, 0x75, 0x73, 0x4d, 0xe1, 0x72, 0xed, 0x67, 0x1ecd,
0x74, 0x4d, 0x61, 0x74, 0xed, 0x6e, 0x69, 0x6b, 0x4d, 0xe1, 0x74, 0x61,
0x6d, 0xf3, 0x72, 0x6f, 0x73, 0x4d, 0x61, 0x7a, 0xe1, 0x74, 0x6c, 0x61,
0x6e, 0x4d, 0x1eb9, 0x6e, 0x1ecd, 0x301, 0x6d, 0x69, 0x6e, 0x69, 0x69, 0x4d,
0x1eb9, 0x301, 0x72, 0xed, 0x64, 0x61, 0x4d, 0x1eb9, 0x74, 0x6c, 0x61, 0x6b,
0xe1, 0x74, 0x6c, 0x61, 0x4d, 0x1eb9, 0x301, 0x6b, 0x73, 0xed, 0x6b, 0xf3,
0x20, 0x53, 0xed, 0x74, 0x69, 0x4d, 0xed, 0x6b, 0x1eb9, 0x6c, 0x1ecd, 0x6e,
0x4d, 0x1ecd, 0x301, 0x6e, 0x6b, 0x74, 0x1ecd, 0x6e, 0x4d, 0x1ecd, 0x6e, 0x74,
0x1eb9, 0x72, 0x1eb9, 0x1eb9, 0x4d, 0x1ecd, 0x6e, 0x74, 0x1eb9, 0x76, 0x69, 0x64,
0x1eb9, 0x6f, 0x4d, 0x1ecd, 0x6e, 0x74, 0x73, 0x1eb9, 0x72, 0x61, 0x74, 0x4e,
0xe1, 0x73, 0x73, 0x1ecd, 0x75, 0x4e, 0x69, 0xfa, 0x20, 0x59, 0x1ecd, 0x6b,
0x4e, 0x6f, 0x6f, 0x6d, 0x4e, 0x1ecd, 0x72, 0xf3, 0x6e, 0x69, 0x61, 0x42,
0x69, 0xfa, 0x6c, 0x61, 0x2c, 0x20, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x44,
0x61, 0x6b, 0xf3, 0x74, 0x61, 0x53, 0x1eb9, 0x301, 0x6e, 0x74, 0x61, 0x2c,
0x20, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x44, 0x61, 0x6b, 0xf3, 0x74, 0x61,
0x4e, 0x69, 0xfa, 0x20, 0x53, 0xe9, 0x6c, 0x1eb9, 0x6d, 0x2c, 0x20, 0x4e,
0x1ecd, 0x301, 0x74, 0x20, 0x44, 0x61, 0x6b, 0xf3, 0x74, 0x61, 0x4f, 0x6b,
0x68, 0x69, 0x6e, 0xe1, 0x67, 0x61, 0x50, 0xe1, 0x6e, 0xe1, 0x6d, 0x61,
0x50, 0xe1, 0x72, 0xe1, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x6f, 0x46, 0xed,
0x6e, 0x69, 0x6b, 0x73, 0x50, 0x1ecd, 0x74, 0x2d, 0x6f, 0x2d, 0x50, 0x72,
0x69, 0x6e, 0x73, 0x50, 0x1ecd, 0x301, 0x74, 0x20, 0x1ecd, 0x66, 0x20, 0x53,
0x70, 0x65, 0x6e, 0x50, 0x1ecd, 0x74, 0x6f, 0x20, 0x56, 0x1eb9, 0x301, 0x6c,
0x68, 0x6f, 0x50, 0x75, 0x1eb9, 0x301, 0x72, 0x74, 0x6f, 0x20, 0x52, 0xed,
0x6b, 0x1ecd, 0x50, 0xfa, 0x6e, 0x74, 0xe1, 0x20, 0x41, 0x72, 0x1eb9, 0x301,
0x6e, 0x61, 0x73, 0x52, 0xe1, 0x6e, 0x6b, 0xed, 0x6e, 0x20, 0xcd, 0x6e,
0x6c, 0x1eb9, 0x74, 0x52, 0x1eb9, 0x73, 0xed, 0x66, 0x1eb9, 0x52, 0x1eb9, 0x6a,
0xed, 0x6e, 0x61, 0x52, 0x1eb9, 0x301, 0x7a, 0xf3, 0x6c, 0x75, 0x74, 0x52,
0x69, 0xf3, 0x20, 0x42, 0x72, 0xe1, 0x6e, 0x6b, 0x6f, 0x53, 0x61, 0x6e,
0x74, 0x61, 0x72, 0x1eb9, 0x6d, 0x53, 0x61, 0x6e, 0x74, 0x69, 0xe1, 0x67,
0x6f, 0x53, 0xe1, 0x6e, 0x74, 0xf3, 0x20, 0x44, 0x6f, 0x6d, 0xed, 0x6e,
0x67, 0x6f, 0x53, 0x61, 0x6f, 0x20, 0x50, 0x61, 0xfa, 0x6c, 0x6f, 0x49,
0x74, 0xf3, 0x6b, 0x6f, 0x74, 0xfa, 0x72, 0x6d, 0x69, 0x74, 0x53, 0x65,
0x6e, 0x74, 0x20, 0x42, 0x61, 0x74, 0x1eb9, 0x6c, 0x1eb9, 0x301, 0x6d, 0x69,
0x53, 0x65, 0x6e, 0x74, 0x20, 0x4a, 0x1ecd, 0x6e, 0x53, 0x65, 0x6e, 0x74,
0x20, 0x4c, 0xfa, 0x73, 0x68, 0x69, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x20,
0x54, 0x1ecd, 0x6d, 0x1ecd, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x56, 0xed,
0x6e, 0x73, 0x1eb9, 0x6e, 0x74, 0x53, 0x77, 0xed, 0x66, 0x74, 0x20, 0x4b,
0x1ecd, 0x301, 0x72, 0x1eb9, 0x6e, 0x74, 0x54, 0x1eb9, 0x67, 0xfa, 0x73, 0x69,
0x67, 0xe1, 0x6c, 0x70, 0x61, 0x54, 0xfa, 0x6c, 0x69, 0x54, 0x69, 0x6b,
0x68, 0x75, 0xe1, 0x6e, 0x61, 0x54, 0x6f, 0x72, 0xf3, 0x6e, 0x74, 0x6f,
0x54, 0x1ecd, 0x74, 0xf3, 0x6c, 0x61, 0x56, 0x61, 0x6e, 0x6b, 0xfa, 0x76,
0x61, 0x57, 0x61, 0xed, 0x74, 0x68, 0x1ecd, 0x73, 0x57, 0xed, 0x6e, 0xed,
0x70, 0x1eb9, 0x67, 0x59, 0x61, 0x6b, 0xfa, 0x74, 0x61, 0x74, 0x4b, 0xe9,
0x73, 0x69, 0x44, 0xe9, 0x76, 0x69, 0x73, 0x44, 0x69, 0xfa, 0x6d, 0x1ecd,
0x301, 0x6e, 0x2d, 0x64, 0x2019, 0x55, 0x76, 0x69, 0x6c, 0x4d, 0x61, 0x6b,
0x77, 0x75, 0xe9, 0xed, 0x4d, 0x1ecd, 0x301, 0x73, 0x1ecd, 0x6e, 0x4d, 0x63,
0x4d, 0x1ecd, 0x301, 0x64, 0x6f, 0x50, 0xe1, 0x6d, 0x61, 0x52, 0x6f, 0x74,
0x1eb9, 0x301, 0x72, 0x61, 0x53, 0x69, 0xf3, 0x77, 0x61, 0x56, 0x1ecd, 0x301,
0x73, 0x74, 0x1ecd, 0x6b, 0x4c, 0x1ecd, 0x6e, 0x67, 0x79, 0x1eb9, 0x301, 0x61,
0x62, 0x69, 0x1eb9, 0x6e, 0xc9, 0x64, 0x1eb9, 0x6e, 0xc1, 0x6c, 0x6d, 0xe1,
0x74, 0x69, 0xc1, 0x6e, 0x61, 0x64, 0x69, 0x61, 0x72, 0x41, 0x6b, 0x74,
0xe1, 0x75, 0x41, 0x6b, 0x74, 0xf3, 0x62, 0x1eb9, 0xc1, 0x73, 0x68, 0x67,
0x61, 0x62, 0x61, 0x74, 0xc1, 0x74, 0xed, 0x72, 0x61, 0x75, 0x42, 0xe1,
0x67, 0x64, 0x61, 0x64, 0x42, 0x61, 0x68, 0x72, 0xe9, 0x6e, 0x42, 0xe1,
0x6b, 0x75, 0x42, 0xe1, 0x6e, 0x6b, 0x1ecd, 0x6b, 0x42, 0xe1, 0x72, 0x6e,
0xe1, 0x75, 0x6c, 0x42, 0x1eb9, 0x72, 0x75, 0x74, 0x42, 0x69, 0x73, 0x68,
0x6b, 0x1eb9, 0x6b, 0x42, 0x72, 0x75, 0x6e, 0x1eb9, 0x69, 0x43, 0x68, 0xed,
0x74, 0xe1, 0x4b, 0x6f, 0x6c, 0xf3, 0x6d, 0x62, 0x6f, 0x44, 0x61, 0x6d,
0xe1, 0x73, 0x6b, 0x1ecd, 0x73, 0x44, 0xe1, 0x6b, 0x61, 0x44, 0xed, 0x6c,
0x69, 0x44, 0x75, 0x73, 0x68, 0xe1, 0x6e, 0x62, 0x1eb9, 0x46, 0x61, 0x67,
0xfa, 0x73, 0x74, 0x61, 0x48, 0x1eb9, 0x301, 0x62, 0x72, 0x1ecd, 0x6e, 0x48,
0xf3, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0xed, 0x6e, 0x20, 0x53, 0xed,
0x74, 0x69, 0x48, 0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd, 0x6e, 0x67, 0x49,
0x72, 0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x4a, 0x61, 0x6b, 0xe1, 0x74, 0x61,
0x4a, 0x61, 0x79, 0x61, 0x70, 0xfa, 0x72, 0x61, 0x4a, 0x1eb9, 0x72, 0xfa,
0x73, 0xe1, 0x6c, 0x1eb9, 0x6d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0xe1, 0x74,
0x6b, 0xe9, 0x4b, 0x61, 0x72, 0xe1, 0x63, 0x68, 0x69, 0x4b, 0x61, 0x74,
0x6d, 0xe1, 0x6e, 0x64, 0x75, 0x4b, 0x61, 0x6e, 0x64, 0xed, 0x67, 0x61,
0x4b, 0x6f, 0x6c, 0x6b, 0xe1, 0x74, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x73, 0x6b, 0x4b, 0x75, 0xe1, 0x6c, 0x61, 0x20, 0x4c,
0xfa, 0x6d, 0x70, 0x1ecd, 0x4b, 0x75, 0x77, 0x65, 0x74, 0x4d, 0x61, 0x6b,
0xe1, 0x6f, 0x4d, 0xe1, 0x67, 0xe1, 0x64, 0x61, 0x6e, 0x4d, 0x61, 0x6b,
0xe1, 0x73, 0x61, 0x4d, 0x61, 0x6e, 0xed, 0x6c, 0x61, 0x4d, 0xfa, 0x73,
0x6b, 0x61, 0x74, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x1eb9,
0x301, 0x73, 0x6b, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73,
0x6b, 0x1ecc, 0x6d, 0x73, 0x6b, 0x1ecc, 0x301, 0x72, 0x61, 0x6c, 0x46, 0x6e,
0x1ecd, 0x301, 0x6d, 0x20, 0x50, 0x1eb9, 0x6e, 0x50, 0x1ecd, 0x6e, 0x74, 0x69,
0xe1, 0x6e, 0x61, 0x6b, 0x50, 0x69, 0x1ecd, 0x6e, 0x67, 0x79, 0x61, 0x6e,
0x67, 0x4b, 0xe1, 0x74, 0x61, 0x4b, 0x6f, 0x73, 0x74, 0xe1, 0x6e, 0x61,
0x69, 0x4b, 0xed, 0x7a, 0x69, 0x6c, 0x1ecd, 0x301, 0x64, 0x61, 0x52, 0x69,
0x79, 0xe1, 0x64, 0x53, 0xe1, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53,
0xe1, 0x6d, 0xe1, 0x6b, 0x61, 0x6e, 0x64, 0x53, 0x6f, 0x6c, 0x53, 0x68,
0xe1, 0x6e, 0x67, 0x68, 0x61, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x1ecd, 0x53, 0x72, 0x1eb9, 0x301, 0x64, 0x6e, 0x1eb9, 0x6b, 0x6f, 0x6c, 0x69,
0x6d, 0x73, 0x6b, 0x54, 0x61, 0x69, 0x70, 0x1eb9, 0x69, 0x54, 0xe1, 0x73,
0x68, 0x6b, 0x1eb9, 0x6e, 0x74, 0x54, 0x69, 0x62, 0x6c, 0xed, 0x73, 0x69,
0x54, 0x1eb9, 0x72, 0x61, 0x6e, 0x54, 0xed, 0x6d, 0x66, 0x75, 0x54, 0xf3,
0x6b, 0x79, 0x6f, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x59,
0x75, 0x72, 0x1ecd, 0x301, 0x6d, 0x6b, 0x69, 0x55, 0x73, 0x74, 0x2d, 0x4e,
0x1eb9, 0x301, 0x72, 0x61, 0x56, 0x69, 0x1eb9, 0x301, 0x6e, 0x74, 0x69, 0x1eb9,
0x6e, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x1ecd, 0x73, 0x74, 0x1ecd, 0x6b,
0x59, 0xe9, 0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x59, 0x61, 0x6e, 0x67, 0x1ecd,
0x6e, 0x59, 0x1eb9, 0x6b, 0x65, 0x74, 0x1eb9, 0x72, 0xed, 0x6e, 0x62, 0x75,
0x67, 0x59, 0x1eb9, 0x72, 0x1eb9, 0x301, 0x76, 0x61, 0x6e, 0x41, 0x7a, 0x1ecd,
0x7a, 0x42, 0x1eb9, 0x6d, 0x69, 0xfa, 0x64, 0x61, 0x4b, 0x65, 0x6e, 0x65,
0x72, 0xed, 0x4b, 0xe9, 0x70, 0x20, 0x56, 0x1eb9, 0x64, 0x46, 0xe1, 0x72,
0xf3, 0x69, 0x73, 0x4d, 0x61, 0x64, 0xed, 0x72, 0x61, 0x52, 0x1eb9, 0x301,
0x6b, 0x6a, 0xe1, 0x76, 0x69, 0x6b, 0x53, 0x61, 0xfa, 0x74, 0x20, 0x4a,
0x1ecd, 0x301, 0x6a, 0x69, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x48, 0x1eb9,
0x6c, 0x1eb9, 0x301, 0x6e, 0x61, 0x53, 0x74, 0xe1, 0x6e, 0x6c, 0x69, 0x41,
0x64, 0x6c, 0x65, 0x69, 0x64, 0x42, 0x72, 0xed, 0x73, 0x62, 0x65, 0x6e,
0x42, 0x72, 0xf3, 0x6b, 0xfa, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x44, 0xe1,
0x72, 0x77, 0x69, 0x6e, 0x59, 0xfa, 0x6b, 0x6c, 0x61, 0x48, 0xf3, 0x62,
0x61, 0x74, 0x4c, 0xed, 0x6e, 0x64, 0x1eb9, 0x6d, 0x61, 0x6e, 0x4c, 0x1ecd,
0x64, 0x20, 0x48, 0x61, 0xfa, 0x4d, 0x1eb9, 0x301, 0x6c, 0x62, 0x1ecd, 0x6e,
0x50, 0x1eb9, 0x72, 0x74, 0x53, 0xed, 0x64, 0x6e, 0x69, 0xc1, 0x6d, 0x73,
0x74, 0xe1, 0x64, 0x61, 0x6d, 0x41, 0x6e, 0x64, 0x1ecd, 0x301, 0x72, 0x61,
0xc1, 0x73, 0x74, 0x72, 0x61, 0x68, 0xe1, 0x6e, 0xc1, 0x74, 0x1eb9, 0x6e,
0x73, 0x42, 0x1eb9, 0x301, 0x6c, 0x67, 0x72, 0xe9, 0x64, 0x42, 0x1eb9, 0x6c,
0x69, 0x6e, 0x42, 0x72, 0x61, 0x74, 0xed, 0x73, 0x6c, 0xe1, 0x76, 0x61,
0x42, 0x72, 0xfa, 0x73, 0x75, 0x6c, 0x73, 0x42, 0xfa, 0x6b, 0xe1, 0x72,
0x1eb9, 0x73, 0x74, 0x42, 0xfa, 0x64, 0xe1, 0x70, 0x1eb9, 0x73, 0x74, 0x42,
0x75, 0x73, 0xed, 0x6e, 0x67, 0x1eb9, 0x6e, 0x43, 0x68, 0x69, 0x73, 0x69,
0x6e, 0xe1, 0x75, 0x4b, 0x1ecd, 0x70, 0x1eb9, 0x6e, 0x68, 0xe1, 0x67, 0x1eb9,
0x6e, 0x44, 0x1ecd, 0x301, 0x62, 0x6c, 0x69, 0x6e, 0x4a, 0x69, 0x62, 0x72,
0x1ecd, 0x301, 0x6c, 0x74, 0x61, 0x47, 0x75, 0x1eb9, 0x6e, 0x7a, 0x69, 0x48,
0x1eb9, 0x6c, 0x73, 0xed, 0x6e, 0x6b, 0x69, 0x41, 0xed, 0x6c, 0x20, 0x1ecd,
0x66, 0x20, 0x4d, 0x61, 0x6e, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x62, 0x75,
0x6c, 0x4a, 0x1eb9, 0x301, 0x73, 0x69, 0x4b, 0x61, 0x6c, 0xed, 0x6e, 0xed,
0x6e, 0x67, 0x72, 0x61, 0x64, 0x4b, 0x69, 0x72, 0x1ecd, 0x76, 0x4b, 0x69,
0x1eb9, 0x76, 0x4c, 0xed, 0x73, 0x62, 0x1ecd, 0x6e, 0x4c, 0x75, 0x62, 0x6c,
0x69, 0xe1, 0x6e, 0x61, 0x4c, 0x1ecd, 0x301, 0x6e, 0x64, 0x1ecd, 0x6e, 0x4c,
0x1ecd, 0x301, 0x6b, 0x73, 0x1eb9, 0x301, 0x6d, 0x62, 0x1ecd, 0x67, 0x4d, 0x1ecd,
0x301, 0x6c, 0x74, 0x61, 0x4d, 0x61, 0x72, 0xed, 0x61, 0x68, 0xe1, 0x6d,
0x6e, 0x4d, 0x1ecd, 0x301, 0x6e, 0xe1, 0x6b, 0x6f, 0x4d, 0x1ecd, 0x301, 0x73,
0x6b, 0x6f, 0x1ecc, 0x301, 0x73, 0x6c, 0x6f, 0x50, 0xe1, 0x72, 0x69, 0x73,
0x50, 0x1ecd, 0x301, 0x6a, 0xf3, 0x72, 0xed, 0x6b, 0x61, 0x52, 0xed, 0x67,
0x61, 0x53, 0x61, 0x6d, 0xe1, 0x72, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x4d,
0x61, 0x72, 0xed, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x79, 0x1eb9, 0x301,
0x76, 0x6f, 0x53, 0xe1, 0x72, 0xe1, 0x74, 0x6f, 0x76, 0x53, 0xed, 0x6d,
0x66, 0x1eb9, 0x72, 0xf3, 0x70, 0x6f, 0x6c, 0x53, 0x6b, 0x1ecd, 0x301, 0x70,
0x79, 0x1eb9, 0x53, 0x74, 0x1ecd, 0x301, 0x6b, 0x68, 0x1ecd, 0x6d, 0x54, 0xe1,
0x6c, 0x69, 0x6e, 0x54, 0x69, 0x72, 0xe1, 0x6e, 0x1eb9, 0x55, 0x6c, 0x69,
0xe1, 0x6e, 0x1ecd, 0x76, 0x73, 0x6b, 0x56, 0xe1, 0x74, 0xed, 0x6b, 0x61,
0x6e, 0x56, 0x69, 0x1eb9, 0x301, 0x6e, 0x61, 0x56, 0x6f, 0x6c, 0x76, 0xf3,
0x67, 0x72, 0x61, 0x64, 0x57, 0x1ecd, 0x301, 0x73, 0x1ecd, 0x5a, 0xe1, 0x67,
0x72, 0x1eb9, 0x62, 0x5a, 0xfa, 0x72, 0x69, 0x6b, 0x41, 0x6e, 0x74, 0xe1,
0x6e, 0xe1, 0x6e, 0x61, 0x72, 0xed, 0x76, 0x6f, 0x43, 0x68, 0xe1, 0x67,
0x1ecd, 0x73, 0x4b, 0x72, 0xed, 0x73, 0x6d, 0x61, 0x73, 0x4b, 0x1ecd, 0x301,
0x6d, 0x1ecd, 0x301, 0x72, 0x6f, 0x73, 0x4b, 0x1eb9, 0x301, 0x72, 0x67, 0xfa,
0x6c, 0x1eb9, 0x6e, 0x4d, 0x61, 0x68, 0x1eb9, 0x301, 0x4d, 0x1ecd, 0x301, 0x6c,
0x64, 0x69, 0x76, 0x73, 0x4d, 0x1ecd, 0x72, 0xed, 0x73, 0x68, 0x1ecd, 0x73,
0x4d, 0x65, 0x79, 0x1ecd, 0x74, 0x52, 0x69, 0x79, 0xfa, 0x6e, 0x69, 0x1ecd,
0x6e, 0xc1, 0x70, 0x69, 0x61, 0x1ecc, 0x301, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x42, 0x75, 0x67, 0x65, 0x6e, 0x76, 0xed, 0x6c, 0x69, 0x61, 0x43, 0x68,
0xe1, 0x74, 0x61, 0x6d, 0x43, 0x68, 0x75, 0x6b, 0xcd, 0x73, 0x74, 0x61,
0x1eb8, 0x66, 0xe1, 0x74, 0x1eb9, 0x46, 0x61, 0x6b, 0xe1, 0xf3, 0x66, 0x6f,
0x46, 0xed, 0x6a, 0x69, 0x46, 0x75, 0x6e, 0x61, 0x66, 0xfa, 0x74, 0x69,
0x47, 0x61, 0x6c, 0xe1, 0x70, 0xe1, 0x67, 0x1ecd, 0x73, 0x47, 0xe1, 0x6d,
0x62, 0x69, 0x1eb9, 0x72, 0x47, 0x75, 0xe1, 0x64, 0xe1, 0x6c, 0x6b, 0x61,
0x6e, 0x61, 0x6c, 0x1eb8, 0x301, 0x6e, 0x64, 0xe1, 0x62, 0x1eb9, 0x301, 0x72,
0x69, 0x4b, 0x72, 0x69, 0x74, 0xed, 0x6d, 0xe1, 0x74, 0x69, 0x4b, 0x1ecd,
0x301, 0x73, 0x72, 0x61, 0x1eb9, 0x4b, 0x77, 0xe1, 0x6a, 0x61, 0x6c, 0x1eb9,
0x6e, 0x4d, 0x61, 0x6a, 0xfa, 0x72, 0x6f, 0x4d, 0x61, 0x6b, 0x77, 0x1eb9,
0x301, 0x73, 0x61, 0x73, 0x4d, 0xed, 0x64, 0x77, 0xe8, 0x4e, 0x61, 0xfa,
0x72, 0x75, 0x4e, 0x1ecd, 0x301, 0x72, 0x66, 0x1ecd, 0x301, 0x6c, 0x6b, 0x4e,
0xfa, 0x6d, 0x1eb9, 0x301, 0x61, 0x50, 0xe1, 0x67, 0x6f, 0x20, 0x50, 0xe1,
0x67, 0x6f, 0x50, 0x61, 0x6c, 0xe1, 0x75, 0x50, 0xed, 0x74, 0x6b, 0x61,
0x6e, 0x50, 0x1ecd, 0x6e, 0x70, 0x1eb9, 0x301, 0x69, 0x50, 0x1ecd, 0x74, 0x20,
0x4d, 0x1ecd, 0x72, 0x1eb9, 0x301, 0x73, 0x62, 0x69, 0x52, 0x61, 0x72, 0x61,
0x74, 0xf3, 0x6e, 0x67, 0x61, 0x54, 0x61, 0x68, 0xed, 0x74, 0x69, 0x54,
0x61, 0x72, 0xe1, 0x77, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0xe1,
0x70, 0x75, 0x57, 0x65, 0x6b, 0x57, 0xe1, 0x6c, 0x69, 0x7ca, 0x7d3, 0x7cc,
0x7d6, 0x7ca, 0x7f2, 0x7ec, 0x7ca, 0x7de, 0x7d9, 0x7ca, 0x7eb, 0x7ca, 0x7ec, 0x7d8,
0x7cc, 0x7eb, 0x20, 0x7db, 0x7ca, 0x7ec, 0x7d3, 0x7cb, 0x7d3, 0x7ca, 0x7eb, 0x7ca,
0x7df, 0x7d1, 0x7d6, 0x7ed, 0x7cb, 0x7ca, 0x7db, 0x7d1, 0x7e1, 0x7d9, 0x7ca, 0x7eb,
0x7d3, 0x7e1, 0x7ca, 0x7ec, 0x7de, 0x7d0, 0x7eb, 0x7d3, 0x7ca, 0x7f2, 0x7dc, 0x7ed,
0x7cc, 0x7eb, 0x7d3, 0x7ca, 0x7f2, 0x7d6, 0x7ce, 0x7df, 0x7d3, 0x7cc, 0x7db, 0x7ca,
0x7e5, 0x7cf, 0x7eb, 0x7d3, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7ef, 0x7d9,
0x7d3, 0x7d9, 0x7d6, 0x7ed, 0x7ca, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7d3, 0x7ce, 0x7d6,
0x7ed, 0x7ce, 0x7f2, 0x7d3, 0x7ce, 0x7d9, 0x7ca, 0x7eb, 0x7de, 0x7cd, 0x7ef, 0x7d9,
0x7de, 0x7db, 0x7ca, 0x7d3, 0x7ce, 0x7df, 0x7ca, 0x7f2, 0x7de, 0x7ca, 0x7eb, 0x7db,
0x7cb, 0x7e5, 0x7ce, 0x7d5, 0x7ca, 0x7eb, 0x7de, 0x7d0, 0x7e3, 0x7ca, 0x7de, 0x7d9,
0x7cc, 0x7eb, 0x7d8, 0x7ca, 0x7ec, 0x7de, 0x7ca, 0x7ef, 0x7d9, 0x7ce, 0x7eb, 0x7d8,
0x7ca, 0x7ef, 0x7d9, 0x2d, 0x7db, 0x7ca, 0x7ec, 0x7df, 0x7ca, 0x7ef, 0x7e1, 0x7d6,
0x7cc, 0x7d3, 0x7ce, 0x7d5, 0x7cc, 0x7eb, 0x7d8, 0x7ce, 0x7e5, 0x7df, 0x7ca, 0x7eb,
0x7df, 0x7ca, 0x7ef, 0x7e6, 0x7ce, 0x7e3, 0x7dd, 0x7d9, 0x7cc, 0x7d5, 0x7d0, 0x7f2,
0x7ec, 0x7dc, 0x7ed, 0x7ca, 0x7d3, 0x7cf, 0x7d9, 0x7d0, 0x7e3, 0x7e4, 0x7d9, 0x7ca,
0x7d9, 0x7cb, 0x7eb, 0x7d6, 0x7ed, 0x7ce, 0x7e5, 0x7ca, 0x7e3, 0x7cd, 0x7db, 0x7d1,
0x7d3, 0x7ce, 0x7ef, 0x7d9, 0x7d6, 0x7ce, 0x7d3, 0x7ca, 0x7eb, 0x7de, 0x7ca, 0x7f2,
0x7d4, 0x7df, 0x7ca, 0x7eb, 0x7de, 0x7ca, 0x7d9, 0x7d1, 0x7d5, 0x7ce, 0x7e1, 0x7de,
0x7cc, 0x7dc, 0x7ed, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x7de, 0x7cc, 0x7f2, 0x7db, 0x7ca,
0x7db, 0x7ca, 0x7eb, 0x7df, 0x7cb, 0x7dc, 0x7ed, 0x7d0, 0x7db, 0x7df, 0x7cc, 0x7d3,
0x7d9, 0x7cb, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7df, 0x7cf, 0x7e1, 0x7cb, 0x7eb, 0x7df,
0x7ce, 0x7e5, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7eb, 0x7df, 0x7ce, 0x7d3, 0x7ce, 0x7f2,
0x7d3, 0x7ca, 0x7db, 0x7cc, 0x7eb, 0x7df, 0x7ce, 0x7db, 0x7de, 0x7ca, 0x7eb, 0x7e1,
0x7df, 0x7ca, 0x7d3, 0x7cf, 0x7eb, 0x7e1, 0x7ca, 0x7d4, 0x7ce, 0x7d5, 0x7cf, 0x7eb,
0x7e1, 0x7ca, 0x7db, 0x7cb, 0x7d9, 0x7ce, 0x7eb, 0x7d2, 0x7d3, 0x7ca, 0x7d3, 0x7ca,
0x7e3, 0x7e1, 0x7cf, 0x7dc, 0x7ed, 0x7ca, 0x7d8, 0x7cc, 0x7db, 0x7cf, 0x7eb, 0x7e1,
0x7cf, 0x7ec, 0x7d9, 0x7cf, 0x7dd, 0x7ed, 0x7cc, 0x7e6, 0x7ca, 0x7eb, 0x7e3, 0x7ca,
0x7e6, 0x7d9, 0x7cf, 0x7d3, 0x7cc, 0x7eb, 0x7d2, 0x7d6, 0x7ca, 0x7e1, 0x7cb, 0x7e3,
0x7ca, 0x7eb, 0x7e2, 0x7ca, 0x7e1, 0x7cb, 0x7eb, 0x7e3, 0x7ce, 0x7e5, 0x7ca, 0x7de,
0x7d1, 0x7db, 0x7d0, 0x7d5, 0x7e5, 0x7dc, 0x7ed, 0x7ca, 0x7ec, 0x7d8, 0x7dc, 0x7ed,
0x7ce, 0x7eb, 0x7d4, 0x7d0, 0x7d9, 0x7d1, 0x7d5, 0x7cf, 0x7eb, 0x2d, 0x7e3, 0x7dd,
0x7ed, 0x7cf, 0x7eb, 0x7db, 0x7ca, 0x7cf, 0x2d, 0x7d5, 0x7cf, 0x7e1, 0x7cb, 0x7eb,
0x7d5, 0x7d9, 0x7cc, 0x7d4, 0x7df, 0x7cc, 0x7eb, 0x20, 0x28, 0x20, 0x7df, 0x7d3,
0x7cc, 0x7eb, 0x20, 0x29, 0x7d5, 0x7ce, 0x7e3, 0x7cc, 0x7db, 0x7e5, 0x7cc, 0x7f2,
0x7d8, 0x7d0, 0x7de, 0x7ca, 0x7d8, 0x7ca, 0x7de, 0x7ca, 0x7f2, 0x7de, 0x7cf, 0x7d9,
0x7ca, 0x7d6, 0x7ed, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7cc, 0x7df, 0x7ca, 0x7eb, 0x7ca,
0x7f2, 0x7d5, 0x7cc, 0x7dc, 0x7ed, 0x7ca, 0x7eb, 0x7ca, 0x7d9, 0x7ca, 0x7dc, 0x7ed,
0x7ce, 0x7e5, 0x7ca, 0x7e6, 0x7cc, 0x7e3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d3, 0x7cc, 0x7e6, 0x7e3, 0x7cf, 0x7db, 0x7cd,
0x7d9, 0x7de, 0x7d5, 0x7ca, 0x7e1, 0x7ca, 0x7d9, 0x7d1, 0x7de, 0x7ca, 0x7eb, 0x7de,
0x7d0, 0x7d9, 0x7d1, 0x7d8, 0x7cf, 0x7d3, 0x7ca, 0x7eb, 0x7d6, 0x7ce, 0x7d6, 0x7ce,
0x7e6, 0x7cc, 0x7eb, 0x7df, 0x7ca, 0x20, 0x7d9, 0x7cc, 0x7e6, 0x7cf, 0x7d6, 0x7ca,
0x7eb, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7cf, 0x7d6, 0x7ed, 0x7ca, 0x7d9, 0x7cc, 0x7e6,
0x7cf, 0x7eb, 0x20, 0x7dc, 0x7ed, 0x7ca, 0x7df, 0x7df, 0x7cb, 0x7dc, 0x7ed, 0x7d0,
0x7db, 0x7db, 0x7ca, 0x7df, 0x7d1, 0x7d5, 0x7ca, 0x7eb, 0x7db, 0x7ca, 0x7f2, 0x7eb,
0x20, 0x7d6, 0x7ce, 0x7e5, 0x7ca, 0x7f2, 0x7eb, 0x7db, 0x7ca, 0x7f2, 0x7eb, 0x20,
0x7df, 0x7ce, 0x7e5, 0x7cc, 0x7db, 0x7d5, 0x7de, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7eb,
0x7ce, 0x7db, 0x7d1, 0x7e5, 0x7e6, 0x7ca, 0x7eb, 0x7ca, 0x7d9, 0x7ce, 0x7d3, 0x7ca,
0x7eb, 0x7ca, 0x7db, 0x7ce, 0x7f2, 0x7db, 0x7cc, 0x7e6, 0x7d0, 0x7f2, 0x7eb, 0x7ca,
0x7d5, 0x7cc, 0x7de, 0x7cf, 0x7de, 0x7ca, 0x7f2, 0x7eb, 0x7d3, 0x7ca, 0x7e4, 0x7cc,
0x7e6, 0x7ca, 0x7eb, 0x7d3, 0x7ca, 0x7e4, 0x7cc, 0x7e6, 0x7ca, 0x20, 0x7d8, 0x7cb,
0x7ec, 0x20, 0x7d3, 0x7ca, 0x7f2, 0x7d8, 0x7cb, 0x7d9, 0x7ca, 0x7eb, 0x7d3, 0x7ca,
0x7d9, 0x7d1, 0x7d3, 0x7d8, 0x7d3, 0x7df, 0x7cb, 0x7e1, 0x7d3, 0x7cb, 0x7df, 0x7cb,
0x7cc, 0x7d6, 0x7ed, 0x7ca, 0x7eb, 0x7d3, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7db, 0x2d,
0x7db, 0x7ca, 0x7d3, 0x7d1, 0x7df, 0x7d0, 0x7f2, 0x7eb, 0x7d3, 0x7cf, 0x7e5, 0x7ca,
0x7eb, 0x20, 0x7dd, 0x7ed, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7eb, 0x7d3, 0x7dc,
0x7ed, 0x7cf, 0x7d5, 0x7ca, 0x7eb, 0x7d3, 0x7ce, 0x7e5, 0x7ca, 0x7db, 0x7de, 0x7ca,
0x7f2, 0x7d3, 0x7d9, 0x7cc, 0x7d8, 0x7d6, 0x20, 0x7d3, 0x7ca, 0x7e6, 0x7de, 0x7ca,
0x7f2, 0x7d4, 0x7cf, 0x7eb, 0x20, 0x7dc, 0x7ed, 0x7d1, 0x7d9, 0x7ca, 0x7f2, 0x7d8,
0x7de, 0x7ca, 0x7f2, 0x7de, 0x7ce, 0x7f2, 0x7eb, 0x7de, 0x7d9, 0x7ca, 0x7de, 0x7ca,
0x7db, 0x7de, 0x7ca, 0x7e6, 0x7d1, 0x7e3, 0x7cb, 0x7eb, 0x7de, 0x7ca, 0x7e6, 0x7cc,
0x7e1, 0x7ca, 0x7f2, 0x7eb, 0x7db, 0x7cc, 0x7de, 0x7ca, 0x7dc, 0x7ed, 0x7cf, 0x7eb,
0x7db, 0x7cc, 0x7e4, 0x7ce, 0x7e5, 0x7ca, 0x7e4, 0x7ce, 0x7eb, 0x7db, 0x7cc, 0x7e6,
0x7ce, 0x7d8, 0x7ca, 0x7d8, 0x20, 0x7d6, 0x7ce, 0x7e5, 0x7ca, 0x7d9, 0x7cb, 0x7eb,
0x7de, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7d9, 0x7cc, 0x7de, 0x7ca, 0x7eb, 0x7de,
0x7d9, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7d0, 0x7f2, 0x7eb, 0x7de, 0x7ce, 0x7e6, 0x7cc,
0x7e6, 0x7d3, 0x7ca, 0x7eb, 0x7de, 0x7ce, 0x7d9, 0x7db, 0x7ca, 0x7e5, 0x7cf, 0x7eb,
0x7d8, 0x7ca, 0x7f2, 0x7e1, 0x7ca, 0x7d9, 0x7de, 0x7db, 0x7d1, 0x7e4, 0x7ca, 0x7dd,
0x7ed, 0x7e3, 0x7d8, 0x7ca, 0x7e5, 0x7ce, 0x7db, 0x7d0, 0x7f2, 0x7eb, 0x7d8, 0x7ca,
0x7e5, 0x7ce, 0x7db, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7de, 0x7d9, 0x7cc, 0x7de, 0x7d8,
0x7ca, 0x7f2, 0x7dd, 0x7ed, 0x7cd, 0x7d9, 0x7d8, 0x7cb, 0x7d5, 0x7d9, 0x7cf, 0x7e5,
0x7ca, 0x7d8, 0x7cf, 0x7e1, 0x7cb, 0x7e3, 0x7cc, 0x7de, 0x7cb, 0x7d8, 0x7d1, 0x7e1,
0x7d0, 0x7f2, 0x7d5, 0x7d0, 0x7f2, 0x7eb, 0x7cb, 0x7d9, 0x7ce, 0x7e3, 0x7d4, 0x7cb,
0x7db, 0x7ca, 0x7df, 0x7d1, 0x7dd, 0x7ed, 0x7ca, 0x7d8, 0x7d0, 0x7d9, 0x7dd, 0x7d0,
0x7d9, 0x7d5, 0x20, 0x7e3, 0x7cb, 0x7df, 0x7d1, 0x7db, 0x7d0, 0x7f2, 0x7eb, 0x7dd,
0x7d0, 0x7d9, 0x7d1, 0x7d5, 0x7ca, 0x7df, 0x7cb, 0x7d6, 0x7ed, 0x7ca, 0x7eb, 0x7dc,
0x7ed, 0x7d1, 0x7df, 0x7ca, 0x7db, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x7dc, 0x7ed, 0x7cf,
0x7db, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x7dc, 0x7ed, 0x7d1, 0x7d9, 0x7ca,
0x7f2, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7d9, 0x7de, 0x7dc, 0x7ed, 0x7d9, 0x7cb, 0x7e3,
0x7d8, 0x7ca, 0x7eb, 0x7dc, 0x7ed, 0x7d1, 0x7e5, 0x7ca, 0x7d8, 0x7cc, 0x7df, 0x7ce,
0x7d4, 0x7dc, 0x7ed, 0x7ca, 0x7ec, 0x7d5, 0x7cc, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7df,
0x7ca, 0x7eb, 0x7dc, 0x7ed, 0x7ce, 0x7e5, 0x7e6, 0x7ca, 0x7de, 0x7cc, 0x7df, 0x7dc,
0x7ed, 0x7ce, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x7e4, 0x7ca, 0x7df, 0x7cc, 0x7dd, 0x7ca,
0x7de, 0x7db, 0x7e4, 0x7dd, 0x7ed, 0x7ca, 0x7e3, 0x7ca, 0x7eb, 0x7e4, 0x7cb, 0x7d9,
0x7d1, 0x7e1, 0x7cf, 0x7db, 0x7cc, 0x7df, 0x7cf, 0x7eb, 0x7cc, 0x7f2, 0x7d8, 0x7cc,
0x7e6, 0x7ca, 0x7e3, 0x7ca, 0x7d4, 0x7cf, 0x7df, 0x7cc, 0x7db, 0x7de, 0x7d1, 0x7e3,
0x7cf, 0x7de, 0x7db, 0x20, 0x5b, 0x7cc, 0x7f2, 0x7d8, 0x7cc, 0x7e6, 0x7e3, 0x7ca,
0x7eb, 0x5d, 0x7e1, 0x7ca, 0x7d9, 0x7cc, 0x7f2, 0x7dc, 0x7ed, 0x7cf, 0x7eb, 0x20,
0x5b, 0x7cc, 0x7f2, 0x7d8, 0x7cc, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x5d, 0x7d4, 0x7cb,
0x7d5, 0x7cd, 0x7d9, 0x7db, 0x7d1, 0x7d3, 0x7ce, 0x7d9, 0x7dc, 0x7ed, 0x20, 0x5b,
0x7cc, 0x7f2, 0x7d8, 0x7cc, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x5d, 0x7d5, 0x7cd, 0x7df,
0x20, 0x7db, 0x7d5, 0x7cc, 0x7eb, 0x20, 0x5b, 0x7cc, 0x7f2, 0x7d8, 0x7cc, 0x7e6,
0x7e3, 0x7ca, 0x7eb, 0x5d, 0x7dd, 0x7ed, 0x7cb, 0x7dd, 0x7ed, 0x7ca, 0x7e6, 0x20,
0x5b, 0x7cc, 0x7f2, 0x7d8, 0x7cc, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x5d, 0x7dd, 0x7ed,
0x7cd, 0x7f2, 0x7db, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x5b, 0x7cc, 0x7f2, 0x7d8, 0x7cc,
0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x5d, 0x7e5, 0x7cc, 0x7e3, 0x7e1, 0x7ca, 0x7de, 0x20,
0x5b, 0x7cc, 0x7f2, 0x7d8, 0x7cc, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x5d, 0x7cc, 0x7e3,
0x7ce, 0x7dd, 0x7ed, 0x7cc, 0x7de, 0x7cc, 0x7de, 0x7ca, 0x7df, 0x7ce, 0x7e6, 0x7cc,
0x7d5, 0x7d6, 0x7e1, 0x7ca, 0x7e6, 0x7cc, 0x7de, 0x7ca, 0x7eb, 0x7d6, 0x7ce, 0x7e3,
0x7cb, 0x7e5, 0x7cf, 0x7eb, 0x7df, 0x7ce, 0x7e6, 0x7cc, 0x7db, 0x7dd, 0x7ed, 0x7cc,
0x7df, 0x7e1, 0x7d0, 0x7f2, 0x7d5, 0x7cc, 0x7db, 0x7cb, 0x7df, 0x7cf, 0x7ec, 0x20,
0x5b, 0x7de, 0x7ca, 0x7f2, 0x7d5, 0x7ce, 0x7de, 0x7cc, 0x7eb, 0x5d, 0x7de, 0x7d1,
0x7d9, 0x7ca, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x7cc, 0x7d6, 0x7de, 0x7df, 0x7ca, 0x20,
0x7d4, 0x7ca, 0x7d6, 0x7ed, 0x7df, 0x7cc, 0x7e1, 0x7ca, 0x7eb, 0x7df, 0x7d0, 0x7db,
0x20, 0x7ca, 0x7f2, 0x7d6, 0x7ed, 0x7cb, 0x7df, 0x7cd, 0x7db, 0x7df, 0x7cf, 0x7e5,
0x7cd, 0x20, 0x7d4, 0x7d1, 0x7d9, 0x7cd, 0x7f2, 0x7db, 0x20, 0x7de, 0x7ca, 0x7d9,
0x7d1, 0x7d5, 0x7cb, 0x7eb, 0x7e1, 0x7ca, 0x7db, 0x7cb, 0x7e6, 0x7cf, 0x7eb, 0x7e1,
0x7e3, 0x7ca, 0x7dc, 0x7ed, 0x7ce, 0x7e5, 0x7ca, 0x7eb, 0x7e1, 0x7e3, 0x7ca, 0x7e5,
0x7d0, 0x7db, 0x7e1, 0x7ca, 0x7d9, 0x7cc, 0x7dc, 0x7ed, 0x7d0, 0x7d5, 0x7e1, 0x7ca,
0x7d9, 0x7d1, 0x7d5, 0x7e3, 0x7cc, 0x7de, 0x7e1, 0x7d5, 0x7ca, 0x7e1, 0x7d9, 0x7cf,
0x7db, 0x7e1, 0x7d6, 0x7ed, 0x7ca, 0x7d5, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7eb, 0x7e1,
0x7cb, 0x7e3, 0x7cf, 0x7e1, 0x7cc, 0x7f2, 0x7eb, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7d8,
0x7ca, 0x7eb, 0x7e1, 0x7cb, 0x7d5, 0x7d1, 0x7df, 0x7de, 0x7ca, 0x7e6, 0x7d1, 0x7df,
0x7ca, 0x7eb, 0x7e1, 0x7cd, 0x7de, 0x7db, 0x7cc, 0x7de, 0x7cf, 0x7eb, 0x7e1, 0x7cc,
0x7de, 0x7cb, 0x7df, 0x7d0, 0x7f2, 0x7eb, 0x7e1, 0x7d0, 0x7f2, 0x7de, 0x7d1, 0x7d5,
0x7d0, 0x7f2, 0x7eb, 0x7e1, 0x7d0, 0x7f2, 0x7d5, 0x7cb, 0x7da, 0x7cb, 0x7e6, 0x7e1,
0x7d0, 0x7f2, 0x7d5, 0x7cb, 0x7dd, 0x7ed, 0x7cc, 0x7d8, 0x7cb, 0x7e5, 0x7cf, 0x7eb,
0x7e1, 0x7d0, 0x7f2, 0x7d5, 0x7db, 0x7d1, 0x7da, 0x7ca, 0x7d5, 0x7e3, 0x7ca, 0x7db,
0x7ca, 0x7e5, 0x7cf, 0x7eb, 0x7e3, 0x7cb, 0x7e5, 0x20, 0x7e6, 0x7d0, 0x7d9, 0x7de,
0x7e3, 0x7cf, 0x7e1, 0x7cb, 0x7eb, 0x7e3, 0x7d0, 0x7d9, 0x7d0, 0x7f2, 0x7e4, 0x7ca,
0x7f2, 0x7eb, 0x7d3, 0x7cb, 0x7e6, 0x7ce, 0x7df, 0x7ca, 0x7eb, 0x20, 0x28, 0x7de,
0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7ee, 0x20, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7d5, 0x7ca,
0x7eb, 0x29, 0x7db, 0x7ca, 0x7f2, 0x7d5, 0x7d9, 0x7cc, 0x7eb, 0x20, 0x28, 0x7de,
0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7ee, 0x20, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7d5, 0x7ca,
0x7eb, 0x29, 0x7e3, 0x7cb, 0x7e5, 0x20, 0x7db, 0x7ca, 0x7df, 0x7cb, 0x7e1, 0x20,
0x28, 0x7de, 0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7ee, 0x20, 0x7d8, 0x7ca, 0x7de, 0x7cf,
0x7d5, 0x7ca, 0x7eb, 0x29, 0x7e3, 0x7ce, 0x7ef, 0x7de, 0x7cf, 0x7d6, 0x7cc, 0x7e3,
0x7dc, 0x7ed, 0x7ca, 0x7eb, 0x7d4, 0x7e3, 0x7ca, 0x7e1, 0x7ca, 0x7ec, 0x7d4, 0x7d9,
0x7ca, 0x7e1, 0x7ca, 0x7d9, 0x7cc, 0x7d3, 0x7cf, 0x7eb, 0x7dd, 0x7cf, 0x7cb, 0x7e3,
0x7cc, 0x7de, 0x7db, 0x7d4, 0x7d0, 0x7d9, 0x7d5, 0x2d, 0x7cf, 0x2d, 0x7d4, 0x7d1,
0x7d9, 0x7cd, 0x7f2, 0x7db, 0x7d4, 0x7d0, 0x7d9, 0x7d5, 0x20, 0x7ca, 0x7db, 0x7d1,
0x7d4, 0x7ca, 0x7f2, 0x7eb, 0x7d4, 0x7d0, 0x7d9, 0x7d1, 0x7d5, 0x7cf, 0x7eb, 0x20,
0x7dd, 0x7ed, 0x7cb, 0x7df, 0x7cf, 0x7eb, 0x7d4, 0x7d0, 0x7d9, 0x7d5, 0x2d, 0x7cf,
0x2d, 0x7d9, 0x7cc, 0x7de, 0x7cf, 0x7eb, 0x7d4, 0x7ce, 0x7f2, 0x7d5, 0x7ca, 0x7eb,
0x20, 0x7ca, 0x7d9, 0x7cb, 0x7e3, 0x7ca, 0x7eb, 0x7d9, 0x7ca, 0x7f2, 0x7de, 0x7cc,
0x7f2, 0x7eb, 0x20, 0x7cc, 0x7f2, 0x7df, 0x7cd, 0x7d5, 0x7d9, 0x7cb, 0x7db, 0x7cc,
0x7dd, 0x7d9, 0x7cb, 0x7d6, 0x7ed, 0x7cc, 0x7e3, 0x7ca, 0x7eb, 0x7d9, 0x7cb, 0x7d6,
0x7ed, 0x7cf, 0x7df, 0x7ce, 0x7d5, 0x7d9, 0x7cc, 0x7e6, 0x7cf, 0x7eb, 0x20, 0x7d3,
0x7d1, 0x7d9, 0x7ca, 0x7f2, 0x7de, 0x7cf, 0x7eb, 0x7db, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7d9, 0x7cd, 0x7e1, 0x7db, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7e6, 0x7ca, 0x7dc, 0x7ed,
0x7cf, 0x7eb, 0x7db, 0x7ca, 0x7f2, 0x7d5, 0x7cf, 0x2d, 0x7d8, 0x7cf, 0x7e1, 0x7cc,
0x7f2, 0x7dc, 0x7ed, 0x7db, 0x7ca, 0x7f2, 0x7d4, 0x7ca, 0x7e5, 0x7df, 0x7cf, 0x7eb,
0x7cc, 0x7d5, 0x7d5, 0x7cf, 0x7de, 0x7de, 0x7cf, 0x7d9, 0x7d1, 0x7d5, 0x7cf, 0x7d9,
0x7d1, 0x7df, 0x7dc, 0x7d5, 0x7db, 0x7cc, 0x7d5, 0x7d1, 0x7de, 0x7ca, 0x7eb, 0x7db,
0x7cd, 0x7f2, 0x2d, 0x7d3, 0x7ca, 0x7d9, 0x7d5, 0x7d1, 0x7e4, 0x7df, 0x7cb, 0x7e1,
0x7cc, 0x7eb, 0x7db, 0x7cd, 0x7f2, 0x2d, 0x7d6, 0x7ed, 0x7ca, 0x7f2, 0x20, 0x7d5,
0x7cb, 0x20, 0x7d5, 0x7cd, 0x7da, 0x2d, 0x7e3, 0x7cb, 0x7dd, 0x7ed, 0x7db, 0x7cd,
0x7f2, 0x2d, 0x7de, 0x7d9, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7d0, 0x7dd, 0x7db, 0x7cd,
0x2d, 0x7df, 0x7ce, 0x7e5, 0x7cc, 0x7db, 0x7db, 0x7cd, 0x7f2, 0x2d, 0x7d5, 0x7cf,
0x7e1, 0x7ca, 0x7db, 0x7db, 0x7cd, 0x7f2, 0x2d, 0x7dd, 0x7ed, 0x7cd, 0x7f2, 0x7db,
0x7ca, 0x7f2, 0x7eb, 0x7db, 0x7e5, 0x7cc, 0x7dd, 0x20, 0x7de, 0x7ce, 0x7d9, 0x7ca,
0x7f2, 0x7eb, 0x7d5, 0x7cb, 0x7dc, 0x7ed, 0x7ce, 0x7db, 0x7cc, 0x7dc, 0x7ed, 0x7ca,
0x7df, 0x7d1, 0x7d4, 0x7ca, 0x7eb, 0x7d5, 0x7ce, 0x7df, 0x7cb, 0x7eb, 0x7d5, 0x7cc,
0x7d6, 0x7ce, 0x7e5, 0x7e3, 0x7ca, 0x7eb, 0x7d5, 0x7cf, 0x7d9, 0x7cf, 0x7f2, 0x7d5,
0x7cf, 0x7eb, 0x7d5, 0x7cf, 0x7d9, 0x7d1, 0x7d5, 0x7cf, 0x7df, 0x7ca, 0x7eb, 0x7dd,
0x7ed, 0x7ca, 0x7f2, 0x7de, 0x7ce, 0x7dd, 0x7ed, 0x7cd, 0x7d9, 0x7e5, 0x7cc, 0x7d5,
0x7cb, 0x7e4, 0x7d0, 0x7d9, 0x7db, 0x7e5, 0x7cc, 0x7f2, 0x7e3, 0x7cc, 0x7d4, 0x7cb,
0x7dc, 0x7ed, 0x7e6, 0x7ca, 0x7de, 0x7ce, 0x7d5, 0x7ca, 0x7eb, 0x7de, 0x7ca, 0x7db,
0x7cb, 0x7e6, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7cc, 0x7db, 0x7d8, 0x7ce, 0x7e1, 0x7d0, 0x7f2,
0x2d, 0x7ce, 0x7d9, 0x7d1, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7e1, 0x7de, 0x7ca, 0x7d9,
0x7cc, 0x7eb, 0x7e1, 0x7ca, 0x7e5, 0x7d1, 0x7db, 0x7d0, 0x7f2, 0x7eb, 0x7e1, 0x7de,
0x7d1, 0x7e1, 0x7ce, 0x7d9, 0x7d1, 0x7d8, 0x7cf, 0x7eb, 0x7d4, 0x7ca, 0x7df, 0x7d1,
0x7e1, 0x7cd, 0x7d9, 0x7d9, 0x7cf, 0x7d5, 0x7cb, 0x7d9, 0x7ca, 0x7eb, 0x7db, 0x7cf,
0x7e5, 0x7ca, 0x7eb, 0x7d5, 0x7d9, 0x7cf, 0x7df, 0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d1,
0x7d5, 0x7cf, 0x7de, 0x7df, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x7cc, 0x7cb, 0x7e6, 0x7ca,
0x7d9, 0x7d1, 0x7d3, 0x7cc, 0x7e6, 0x7cd, 0x7f2, 0x7eb, 0x7ca, 0x7d8, 0x7cd, 0x7f2,
0x7eb, 0x7ca, 0x7df, 0x7d1, 0x7e1, 0x7ca, 0x20, 0x7ca, 0x7d5, 0x7ca, 0x7eb, 0x7ca,
0x7e1, 0x7e1, 0x7ca, 0x7f2, 0x7eb, 0x7ca, 0x7e3, 0x7ca, 0x7d8, 0x7cc, 0x7d9, 0x7ca,
0x7de, 0x7d1, 0x7d5, 0x7ca, 0x7e5, 0x7ce, 0x7eb, 0x7ca, 0x7de, 0x7d1, 0x7d5, 0x7cf,
0x7d3, 0x7cc, 0x7f2, 0x7db, 0x7de, 0x7ca, 0x7db, 0x7d1, 0x7dc, 0x7ed, 0x7ca, 0x7d3,
0x7ca, 0x7d5, 0x7ca, 0x7d5, 0x7cc, 0x7d9, 0x7ca, 0x7ce, 0x7eb, 0x7d3, 0x7ca, 0x7dc,
0x7ed, 0x7d1, 0x7d8, 0x7ca, 0x7d8, 0x7d3, 0x7e4, 0x7ca, 0x7d9, 0x7cd, 0x7f2, 0x7eb,
0x7d3, 0x7ca, 0x7de, 0x7ce, 0x7eb, 0x7d3, 0x7ca, 0x7f2, 0x7de, 0x7d0, 0x7de, 0x7d3,
0x7ca, 0x7d9, 0x7d1, 0x7e3, 0x7d0, 0x7df, 0x7d3, 0x7cb, 0x7e6, 0x7cc, 0x7d9, 0x7ce,
0x7d5, 0x7d3, 0x7cc, 0x7d7, 0x7cd, 0x7de, 0x7d3, 0x7d9, 0x7ce, 0x7e3, 0x7cb, 0x7e6,
0x7cc, 0x7eb, 0x7d7, 0x7cc, 0x7d5, 0x7ca, 0x7eb, 0x7de, 0x7cf, 0x7df, 0x7cf, 0x7f2,
0x7d3, 0x7cf, 0x7eb, 0x7d8, 0x7e1, 0x7ca, 0x7db, 0x7d8, 0x7de, 0x7ca, 0x7eb, 0x7d8,
0x7df, 0x7cc, 0x7eb, 0x7d8, 0x7ce, 0x7ec, 0x7d3, 0x7ca, 0x7e6, 0x7cc, 0x7ec, 0x7d8,
0x7ce, 0x7db, 0x7ca, 0x7f2, 0x7d3, 0x7cb, 0x7eb, 0x7dd, 0x7e1, 0x7ca, 0x7dc, 0x7ed,
0x7ce, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7eb, 0x7dc, 0x7ed, 0x7ca, 0x7d6, 0x7ed, 0x7ca,
0x7eb, 0x7e4, 0x7cb, 0x7d3, 0x7d1, 0x7d9, 0x7d0, 0x7f2, 0x7eb, 0x7e4, 0x7d0, 0x7ef,
0x2d, 0x7d7, 0x7cc, 0x2d, 0x7e1, 0x7cc, 0x7f2, 0x2d, 0x7d8, 0x7ce, 0x7f1, 0x7e4,
0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x20, 0x7de, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x7e4, 0x7cf,
0x7dd, 0x7ed, 0x7d8, 0x7cc, 0x7d9, 0x7d1, 0x7de, 0x7ce, 0x7d5, 0x7d1, 0x7d8, 0x7de,
0x7cc, 0x7eb, 0x7d6, 0x7de, 0x7ca, 0x7d9, 0x7d1, 0x7d5, 0x7ca, 0x7eb, 0x7d6, 0x7e6,
0x7ca, 0x7d4, 0x7ce, 0x7d9, 0x7ca, 0x7eb, 0x7d6, 0x7ed, 0x7cb, 0x7d9, 0x7ce, 0x7d6,
0x7ed, 0x7ca, 0x7df, 0x7cd, 0x7e1, 0x7de, 0x7ca, 0x7d3, 0x7ce, 0x7df, 0x7de, 0x7ca,
0x7f2, 0x7e6, 0x7d1, 0x7d7, 0x7ca, 0x7d5, 0x7d1, 0x7de, 0x7cc, 0x7eb, 0x7de, 0x7d9,
0x7ca, 0x7db, 0x7cc, 0x7eb, 0x7de, 0x7ca, 0x7d5, 0x7d1, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ce, 0x7eb, 0x7de, 0x7ed, 0x7ca, 0x7f2, 0x7d8, 0x7cc, 0x7dc, 0x7ed, 0x7ca, 0x7eb,
0x7de, 0x7ca, 0x7df, 0x7d1, 0x7de, 0x7ce, 0x7d5, 0x7ca, 0x7eb, 0x7de, 0x7d9, 0x7ca,
0x7db, 0x7d1, 0x7df, 0x7cf, 0x7e6, 0x7d1, 0x7e6, 0x7ca, 0x7d9, 0x7d1, 0x7db, 0x7de,
0x7cc, 0x7eb, 0x7de, 0x7ce, 0x7e5, 0x7df, 0x7ca, 0x20, 0x7df, 0x7ce, 0x7f2, 0x7d4,
0x7ce, 0x7d9, 0x7de, 0x7ce, 0x7d7, 0x7cc, 0x7f2, 0x7dc, 0x7ed, 0x7de, 0x7ce, 0x7e5,
0x7cd, 0x7e6, 0x7cc, 0x7d5, 0x7e1, 0x7de, 0x7ca, 0x7e5, 0x7cf, 0x7eb, 0x7e1, 0x7dc,
0x7ed, 0x7ca, 0x7d8, 0x7ca, 0x7f2, 0x7eb, 0x7e1, 0x7de, 0x7ca, 0x7db, 0x7ca, 0x7d9,
0x7e1, 0x7ca, 0x7e3, 0x7cc, 0x7df, 0x7e1, 0x7ca, 0x7db, 0x7d1, 0x7de, 0x7ca, 0x7d5,
0x7e3, 0x7cc, 0x7de, 0x7cf, 0x7db, 0x7cc, 0x7e6, 0x7ca, 0x7eb, 0x7e3, 0x7dd, 0x7ed,
0x7cf, 0x7de, 0x7ce, 0x7d6, 0x7ed, 0x7e3, 0x7cd, 0x7d5, 0x7d1, 0x7db, 0x7de, 0x7cc,
0x7eb, 0x7e3, 0x7dd, 0x7ed, 0x7cf, 0x7de, 0x7db, 0x7d3, 0x7cc, 0x7d9, 0x7d1, 0x7db,
0x7de, 0x7cc, 0x7eb, 0x7cf, 0x7e1, 0x7d1, 0x7db, 0x7de, 0x7cf, 0x7d9, 0x7ca, 0x7df,
0x7d4, 0x7d1, 0x7e3, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d4, 0x7cd, 0x7f2, 0x7eb, 0x7d4,
0x7d0, 0x7f2, 0x7d5, 0x7cc, 0x7e6, 0x7ca, 0x7e3, 0x7ca, 0x7de, 0x7d4, 0x7cc, 0x7e6,
0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x7e6, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7de, 0x7d5, 0x7ca,
0x7d9, 0x7de, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7e3, 0x7ca, 0x7e6, 0x7de, 0x7d6, 0x7ed,
0x7cc, 0x7df, 0x7cf, 0x7d9, 0x7d1, 0x7d8, 0x7ca, 0x7eb, 0x7d9, 0x7cc, 0x7e6, 0x7ca,
0x7d8, 0x7db, 0x7de, 0x7ca, 0x7df, 0x7cc, 0x7e3, 0x7db, 0x7e1, 0x7ca, 0x7d9, 0x7d1,
0x7de, 0x7ca, 0x7f2, 0x7d8, 0x7db, 0x7cb, 0x7ce, 0x7df, 0x7db, 0x7ca, 0x7f2, 0x7dc,
0x7ed, 0x7ca, 0x7e6, 0x7cc, 0x7eb, 0x7db, 0x7cd, 0x7f2, 0x7dc, 0x7ed, 0x7ca, 0x7d4,
0x7ce, 0x7d9, 0x7db, 0x7d9, 0x7cb, 0x7d8, 0x7d1, 0x7e3, 0x7cf, 0x7de, 0x7d0, 0x7df,
0x7cc, 0x7e1, 0x7d1, 0x7db, 0x7de, 0x7cc, 0x7eb, 0x7d5, 0x7ca, 0x7e6, 0x7cc, 0x7d4,
0x7cb, 0x7eb, 0x7d5, 0x7ca, 0x7db, 0x7d7, 0x7d1, 0x7de, 0x7cd, 0x7f2, 0x7d5, 0x7d5,
0x7d3, 0x7cc, 0x7df, 0x7db, 0x7cc, 0x7eb, 0x7d5, 0x7e4, 0x7cb, 0x7d9, 0x7ca, 0x7f2,
0x7eb, 0x7d5, 0x7cc, 0x7f2, 0x7d4, 0x7ce, 0x7eb, 0x7d5, 0x7cf, 0x7de, 0x7cc, 0x7e6,
0x7cf, 0x7eb, 0x7d5, 0x7cf, 0x7e1, 0x7d1, 0x7db, 0x7de, 0x7ce, 0x7df, 0x7ca, 0x7f2,
0x7d3, 0x7ca, 0x7d5, 0x7d0, 0x7d9, 0x7ce, 0x7d9, 0x7ce, 0x7f2, 0x7de, 0x7cc, 0x7eb,
0x7ce, 0x7db, 0x7d5, 0x2d, 0x7e3, 0x7cb, 0x7d9, 0x7ca, 0x7eb, 0x7dd, 0x7ed, 0x7cc,
0x7e6, 0x7cd, 0x7d5, 0x7cc, 0x7e3, 0x7ca, 0x7e1, 0x7dd, 0x7ed, 0x7d1, 0x7df, 0x7ca,
0x7d8, 0x7cc, 0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d0, 0x7de, 0x7e6, 0x7ca,
0x7de, 0x7ce, 0x7d5, 0x7d1, 0x7db, 0x7de, 0x7d9, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7ce,
0x7f2, 0x7eb, 0x7e6, 0x7cb, 0x7de, 0x7ca, 0x7d5, 0x7cb, 0x7d9, 0x7cc, 0x7f2, 0x7d3,
0x7ce, 0x7d9, 0x7dc, 0x7ed, 0x7e6, 0x7d9, 0x7cb, 0x7dd, 0x7ed, 0x7ca, 0x7f2, 0x7eb,
0x7ca, 0x7d7, 0x7ed, 0x7cf, 0x7d9, 0x7cd, 0x7db, 0x7d3, 0x7cd, 0x7d9, 0x7f8, 0x7e1,
0x7ce, 0x7d8, 0x7cd, 0x7db, 0x7de, 0x7e3, 0x7ca, 0x7d9, 0x7cc, 0x7eb, 0x7dc, 0x7e6,
0x7cb, 0x7de, 0x7ce, 0x7f2, 0x7eb, 0x2d, 0x7dd, 0x7d9, 0x7cc, 0x7db, 0x7cc, 0x7eb,
0x7dd, 0x7ca, 0x7d9, 0x7cf, 0x7cb, 0x7eb, 0x7e1, 0x7ca, 0x7d8, 0x7cd, 0x7d9, 0x7d9,
0x7cb, 0x7e6, 0x7d1, 0x7de, 0x7cc, 0x7e6, 0x7ca, 0x7dd, 0x7ed, 0x7cd, 0x7de, 0x7de,
0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7ee, 0x20, 0x7d6, 0x7ed, 0x7d0, 0x7d9, 0x7d1, 0x7d6,
0x7ed, 0x7cc, 0x7df, 0x7ca, 0x7eb, 0x7db, 0x7cd, 0x7f2, 0x7d5, 0x20, 0x7e4, 0x7cb,
0x7df, 0x7cd, 0x7e3, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7df, 0x7cb, 0x7e6, 0x7ca,
0x7d8, 0x7cb, 0x7df, 0x7ca, 0x7e6, 0x7d8, 0x7d3, 0x7d1, 0x7d9, 0x7cc, 0x7db, 0x7d1,
0x7d3, 0x7ca, 0x7f2, 0x7eb, 0x7d3, 0x7d1, 0x7d9, 0x7cf, 0x7de, 0x7cd, 0x7f2, 0x7eb,
0x20, 0x7e4, 0x7cc, 0x7df, 0x7d8, 0x7ca, 0x7d9, 0x7d1, 0x7e5, 0x7cc, 0x7f2, 0x7eb,
0x7cb, 0x7de, 0x7d1, 0x7df, 0x7ca, 0x7eb, 0x7e4, 0x7cf, 0x7d3, 0x7ca, 0x7d9, 0x7d5,
0x7df, 0x7cc, 0x7f2, 0x7d8, 0x7cb, 0x7e1, 0x7ca, 0x7f2, 0x7eb, 0x7df, 0x7d0, 0x7d9,
0x7d8, 0x20, 0x7e4, 0x7cf, 0x7e5, 0x7cb, 0x7eb, 0x7e1, 0x7cb, 0x7df, 0x7d1, 0x7d3,
0x7ce, 0x7d9, 0x7e3, 0x7d4, 0x7cd, 0x7d9, 0x7d5, 0x7db, 0x7cc, 0x7d8, 0x7d1, 0x7e3,
0x7cb, 0x7eb, 0x7ca, 0x7e1, 0x7db, 0x7d1, 0x7d5, 0x7cd, 0x7d9, 0x7d1, 0x7d8, 0x7ca,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7cf, 0x7d9, 0x7cb, 0x7eb, 0x7ca, 0x7db, 0x7d1, 0x7d5,
0x7d9, 0x7ca, 0x7de, 0x7d1, 0x7e4, 0x7ca, 0x7f2, 0x7eb, 0x7ca, 0x7d5, 0x7cd, 0x7e3,
0x7d3, 0x7cd, 0x7df, 0x7d1, 0x7dc, 0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x7d3, 0x7cd, 0x7d9,
0x7d1, 0x7df, 0x7cd, 0x7f2, 0x7eb, 0x7d3, 0x7d1, 0x7d9, 0x7ca, 0x7d5, 0x7cc, 0x7db,
0x7d1, 0x7df, 0x7dd, 0x7ed, 0x7ca, 0x7eb, 0x7d3, 0x7d1, 0x7d9, 0x7ce, 0x7de, 0x7d1,
0x7db, 0x7cd, 0x7df, 0x7d3, 0x7ce, 0x7de, 0x7ca, 0x7d9, 0x7cd, 0x7db, 0x7d3, 0x7ce,
0x7d8, 0x7ca, 0x7d3, 0x7cd, 0x7db, 0x7d5, 0x7d3, 0x7ce, 0x7db, 0x7cd, 0x7f2, 0x7dc,
0x7ed, 0x7cd, 0x7f2, 0x7eb, 0x7d7, 0x7db, 0x7cc, 0x7e3, 0x7ca, 0x7e5, 0x7de, 0x7cf,
0x7d4, 0x7cb, 0x7e3, 0x7ca, 0x7dc, 0x7ed, 0x7d8, 0x7ce, 0x7d3, 0x7d1, 0x7df, 0x7cd,
0x7eb, 0x7d6, 0x7ed, 0x7cc, 0x7d3, 0x7d9, 0x7ca, 0x7df, 0x7d1, 0x7d5, 0x7ca, 0x7eb,
0x7dc, 0x7ed, 0x7cd, 0x7d9, 0x7d1, 0x7e3, 0x7db, 0x7cb, 0x7e6, 0x7e4, 0x7cd, 0x7df,
0x7d1, 0x7db, 0x7cc, 0x7f2, 0x7de, 0x7cc, 0x7eb, 0x7e1, 0x7ca, 0x7f2, 0x7eb, 0x20,
0x7d5, 0x7cc, 0x7f2, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7d3, 0x7ce, 0x7df,
0x7d6, 0x7cb, 0x7d9, 0x7d1, 0x7db, 0x7cc, 0x7eb, 0x7de, 0x7ca, 0x7df, 0x7cc, 0x7e3,
0x7cc, 0x7f2, 0x7dc, 0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x7de, 0x7cc, 0x7d9, 0x7d0, 0x7dd,
0x7ed, 0x7de, 0x7cc, 0x7e6, 0x7cd, 0x7dd, 0x7ed, 0x7df, 0x7cc, 0x7db, 0x7d1, 0x7d3,
0x7d0, 0x7f2, 0x7eb, 0x7df, 0x7d1, 0x7d6, 0x7ce, 0x7d3, 0x7d1, 0x7d6, 0x7e3, 0x7ca,
0x7eb, 0x7df, 0x7d0, 0x7f2, 0x7d8, 0x7d9, 0x7cd, 0x7db, 0x7df, 0x7ce, 0x7de, 0x7d1,
0x7db, 0x7ca, 0x7f2, 0x7d3, 0x7ce, 0x7d9, 0x7dc, 0x7ed, 0x7e1, 0x7ca, 0x7d8, 0x7d9,
0x7cc, 0x7d8, 0x7e1, 0x7ca, 0x7df, 0x7d1, 0x7d5, 0x7e1, 0x7ca, 0x7d9, 0x7cc, 0x7e4,
0x7ca, 0x7e1, 0x7e1, 0x7cc, 0x7f2, 0x7db, 0x7de, 0x7e1, 0x7cf, 0x7e3, 0x7ca, 0x7de,
0x7cf, 0x7eb, 0x7e1, 0x7cf, 0x7db, 0x7d1, 0x7de, 0x7ce, 0x7eb, 0x7cf, 0x7db, 0x7d1,
0x7df, 0x7cf, 0x7eb, 0x7d4, 0x7ca, 0x7ec, 0x7d9, 0x7cc, 0x7eb, 0x7d4, 0x7cf, 0x7d8,
0x7dc, 0x7ed, 0x7e2, 0x7d9, 0x7cc, 0x7de, 0x7ca, 0x7eb, 0x7d4, 0x7d9, 0x7ca, 0x7dc,
0x7ed, 0x7d9, 0x7cc, 0x7dc, 0x7ed, 0x7ca, 0x7eb, 0x7d9, 0x7d0, 0x7e1, 0x7db, 0x7e1,
0x7ca, 0x7d9, 0x7ca, 0x7eb, 0x7db, 0x7cd, 0x7f2, 0x2d, 0x7e1, 0x7ca, 0x7d9, 0x7cc,
0x7e6, 0x7cf, 0x7eb, 0x7db, 0x7d9, 0x7ca, 0x7d6, 0x7ed, 0x7cb, 0x7dd, 0x7ed, 0x7cf,
0x7eb, 0x7db, 0x7d9, 0x7ca, 0x7d5, 0x7d0, 0x7dd, 0x7ed, 0x7db, 0x7cc, 0x7f2, 0x7dd,
0x7cb, 0x7d9, 0x7d4, 0x7cf, 0x7df, 0x7db, 0x7d1, 0x7de, 0x7cf, 0x7d4, 0x7d1, 0x7d6,
0x7cb, 0x7eb, 0x7db, 0x7cf, 0x7dd, 0x7cc, 0x7e6, 0x7ca, 0x7eb, 0x7db, 0x7d1, 0x7d5,
0x7cf, 0x7de, 0x7d0, 0x7df, 0x7e1, 0x7d5, 0x7ca, 0x7df, 0x7cc, 0x7f2, 0x7e3, 0x7d5,
0x7cc, 0x7d9, 0x7e3, 0x7ca, 0x7eb, 0x7ce, 0x7df, 0x7cc, 0x7e6, 0x7ca, 0x7e3, 0x7cf,
0x7dd, 0x7ed, 0x7d1, 0x7db, 0x7de, 0x7dd, 0x7ed, 0x7ee, 0x7ca, 0x7d8, 0x7ce, 0x7d6,
0x7ed, 0x7dd, 0x7ed, 0x7ca, 0x7d5, 0x7cc, 0x7de, 0x7ca, 0x7f2, 0x7eb, 0x7dd, 0x7ed,
0x7cc, 0x7e6, 0x7cd, 0x7e3, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7d1, 0x7e3, 0x7cc, 0x7e6,
0x7ce, 0x7db, 0x7dd, 0x7ed, 0x7cf, 0x7df, 0x7dc, 0x7ed, 0x7cf, 0x7dc, 0x7ed, 0x7d9,
0x7ca, 0x7d8, 0x7dd, 0x7ed, 0x7ca, 0x7d9, 0x7d1, 0x7db, 0x7cf, 0x7dd, 0x7ed, 0x7cc,
0x7eb, 0x7d7, 0x7ed, 0x7ca, 0x7dc, 0x7ed, 0x7d9, 0x7cd, 0x7d3, 0x7d6, 0x7ed, 0x7ce,
0x7d9, 0x7cc, 0x7de, 0x7ca, 0x7f2, 0x7d5, 0x7e3, 0x7ca, 0x7e3, 0x7ca, 0x7d9, 0x7cc,
0x7dd, 0x7ed, 0x7cf, 0x7eb, 0x7db, 0x7ca, 0x7dc, 0x7ed, 0x7d0, 0x7db, 0x7de, 0x7d9,
0x7cc, 0x7db, 0x7d1, 0x7e1, 0x7ca, 0x7db, 0x7de, 0x7cf, 0x7de, 0x7cf, 0x7db, 0x7de,
0x7d0, 0x7e1, 0x7d0, 0x7ef, 0x7d9, 0x7de, 0x7cd, 0x7d9, 0x7d1, 0x7dc, 0x7ed, 0x7cb,
0x7df, 0x7cd, 0x7f2, 0x7eb, 0x7e1, 0x7ca, 0x7e4, 0x7cb, 0x7eb, 0x7e1, 0x7ca, 0x7df,
0x7d1, 0x7d8, 0x7cc, 0x7dd, 0x7ed, 0x7e1, 0x7cf, 0x7d9, 0x7cc, 0x7db, 0x7e1, 0x7ca,
0x7e6, 0x7d0, 0x7d5, 0x7d9, 0x7cb, 0x7e3, 0x7cc, 0x7e6, 0x7d0, 0x7f2, 0x7eb, 0x7ca,
0x7d4, 0x7cc, 0x7e6, 0x7ca, 0x7eb, 0x7cf, 0x7de, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8,
0x7d3, 0x7ce, 0x7dc, 0x7ed, 0x7cd, 0x7f2, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7d7, 0x7d5,
0x7ca, 0x7e1, 0x7d7, 0x7ce, 0x7ef, 0x7de, 0x7cd, 0x7db, 0x7d1, 0x7d5, 0x7cd, 0x7d9,
0x7cb, 0x7dd, 0x7ca, 0x7d5, 0x7cb, 0x7eb, 0x7dd, 0x7de, 0x7ca, 0x7cf, 0x7dd, 0x7cf,
0x7eb, 0x7dd, 0x7d6, 0x7cc, 0x7eb, 0x7dd, 0x7ce, 0x7e3, 0x7ca, 0x7dd, 0x7ce, 0x7d5,
0x7cc, 0x7eb, 0x7dc, 0x7ed, 0x7df, 0x7ca, 0x7d4, 0x7ca, 0x7dc, 0x7ed, 0x7d0, 0x7db,
0x7dc, 0x7ed, 0x7ca, 0x7f2, 0x7d3, 0x7cc, 0x7e6, 0x7cb, 0x7d9, 0x7dc, 0x7ed, 0x7ce,
0x7e5, 0x7d8, 0x7ca, 0x7df, 0x7d1, 0x7de, 0x7e3, 0x7ca, 0x7df, 0x7dc, 0x7ed, 0x7ce,
0x7e5, 0x7ca, 0x7f2, 0x7eb, 0x7de, 0x7ca, 0x7f2, 0x7d5, 0x7d0, 0x7f2, 0x7eb, 0x7de,
0x7d9, 0x7cc, 0x7d5, 0x7cc, 0x7e1, 0x7ca, 0x7d5, 0x7cc, 0x7eb, 0x7de, 0x7d0, 0x7db,
0x2e, 0x7d9, 0x7ca, 0x7eb, 0x7de, 0x7d1, 0x7e5, 0x7d6, 0x7ca, 0x7df, 0x7cb, 0x7e6,
0x7cc, 0x7f2, 0x7eb, 0x7e1, 0x7ca, 0x7d6, 0x7ce, 0x7d9, 0x7cf, 0x7eb, 0x7e1, 0x7ca,
0x7d9, 0x7de, 0x7cc, 0x7db, 0x7e1, 0x7cc, 0x7d8, 0x7d1, 0x7e5, 0x7ca, 0x7e6, 0x7e3,
0x7ca, 0x7e5, 0x7d9, 0x7ce, 0x7eb, 0x7e3, 0x7cc, 0x7e6, 0x7ce, 0x7cb, 0x7eb, 0x7e3,
0x7d0, 0x7d9, 0x7d1, 0x7dd, 0x7d0, 0x7df, 0x7de, 0x7e3, 0x7ce, 0x7e1, 0x7cb, 0x7e6,
0x7ca, 0x7eb, 0x7d4, 0x7ca, 0x7dc, 0x7ed, 0x7cf, 0x7eb, 0x20, 0x7e1, 0x7cf, 0x7dc,
0x7ed, 0x7cf, 0x7eb, 0x7d4, 0x7df, 0x7ca, 0x7cf, 0x7db, 0x7d4, 0x7cc, 0x7d5, 0x7de,
0x7cd, 0x7d9, 0x7e3, 0x7dd, 0x7cf, 0x7f2, 0x7d4, 0x7cb, 0x7e6, 0x7cc, 0x7eb, 0x7e1,
0x7cf, 0x7d9, 0x7cd, 0x7db, 0x7d1, 0x7d3, 0x7cc, 0x7eb, 0x20, 0x7d8, 0x7ca, 0x7f2,
0x7de, 0x7ca, 0x7f2, 0x7d9, 0x7ca, 0x7d9, 0x7cf, 0x7d5, 0x7cf, 0x7f2, 0x7dc, 0x7ed,
0x7ca, 0x7eb, 0x7db, 0x7cd, 0x7f2, 0x7d4, 0x7ca, 0x7f2, 0x7eb, 0x7d5, 0x7ca, 0x7e4,
0x7d5, 0x7cc, 0x7eb, 0x7d5, 0x7d9, 0x7ca, 0x7e5, 0x7ca, 0x7eb, 0x7d5, 0x7cf, 0x7f2,
0x7dc, 0x7ed, 0x7d5, 0x7ca, 0x7d4, 0x7ce, 0x7eb, 0x7e5, 0x7ca, 0x7de, 0x7cb, 0x7eb,
0x7e5, 0x7ca, 0x7df, 0x7df, 0x7cc, 0x7db, 0x44, 0xe1, 0x6e, 0x6d, 0xe1, 0x72,
0x6b, 0x6b, 0x75, 0x68, 0xe1, 0x6d, 0x6d, 0x61, 0x6e, 0x42, 0x65, 0x75,
0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x76, 0x76, 0x69, 0x2d, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x47, 0x75, 0x6f, 0x76, 0x64, 0x64, 0xe1,
0x161, 0x2c, 0x20, 0x44, 0x61, 0x76, 0x76, 0x69, 0x2d, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d,
0x2c, 0x20, 0x44, 0x61, 0x76, 0x76, 0x69, 0x2d, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x4e, 0x6f, 0x76, 0x6f,
0x6b, 0x75, 0x73, 0x6e, 0x65, 0x63, 0x6b, 0x55, 0x6c, 0x61, 0x6e, 0x2d,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x41,
0x7a, 0x6f, 0x72, 0x61, 0x74, 0x4b, 0x61, 0x6e, 0xe1, 0x72, 0x69, 0x61,
0x4c, 0x75, 0x6c, 0x6c, 0x69, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x48, 0x65, 0x6c, 0x73, 0x73, 0x65, 0x74, 0x4d, 0x61, 0x6e, 0x73,
0x75, 0x6f, 0x6c, 0x75, 0x4a, 0x75, 0x6f, 0x76, 0x6c, 0x61, 0x73, 0x75,
0x6f, 0x6c, 0x75, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69, 0x76, 0x76,
0x61, 0x74, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x61,
0x74, 0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0xf8, 0x79, 0x65, 0x6e, 0x65,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x20, 0x62, 0x79, 0x54, 0x73, 0x6a,
0x69, 0x74, 0x61, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69,
0x6e, 0x68, 0x2d, 0x62, 0x79, 0x65, 0x6e, 0x4a, 0x61, 0x6a, 0x61, 0x70,
0x75, 0x72, 0x61, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x6e, 0x65, 0x4b, 0x61,
0x6e, 0x61, 0x72, 0x69, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x4b, 0x61, 0x70,
0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x46, 0xe6, 0x72, 0xf8, 0x79,
0x65, 0x6e, 0x65, 0x53, 0xf8, 0x72, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x42, 0x75, 0x63, 0x75, 0x72, 0x65, 0x219, 0x74, 0x69, 0x48,
0x65, 0x6c, 0x73, 0x69, 0x6e, 0x67, 0x66, 0x6f, 0x72, 0x73, 0x56, 0x61,
0x74, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x74, 0x61, 0x74, 0x65, 0x6e, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0xf8, 0x79, 0x61, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x4b, 0x6f, 0x6d,
0x6f, 0x72, 0x65, 0x6e, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65,
0x6e, 0x65, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61, 0x47, 0x61,
0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf8, 0x79, 0x61, 0x43, 0x61,
0x79, 0x6d, 0x61, 0x6e, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x43, 0x75, 0x69,
0x61, 0x62, 0x61, 0x41, 0x73, 0x6a, 0x67, 0x61, 0x62, 0x61, 0x74, 0x59,
0x61, 0x6e, 0x67, 0xf4, 0x6e, 0x41, 0x73, 0x6f, 0x72, 0x61, 0x6e, 0x65,
0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x46,
0xe6, 0x72, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x4b, 0x6f, 0x6b, 0x6f, 0x73,
0xf8, 0x79, 0x61, 0x6e, 0x65, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x61, 0x6e,
0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x61, 0x6e, 0x65, 0x47, 0x61,
0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65,
0x41, 0x72, 0x67, 0x69, 0xe8, 0x72, 0x73, 0x4a, 0x69, 0x62, 0x6f, 0x74,
0x69, 0x44, 0x6f, 0x61, 0x6c, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6b, 0xf2,
0x6b, 0x4b, 0x61, 0x62, 0x6f, 0x6c, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0x69,
0x6e, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x41, 0xe7, 0xf2, 0x72, 0x65, 0x73, 0x41, 0x64, 0x65, 0x6c, 0x61,
0xef, 0x64, 0x61, 0x41, 0x6e, 0x64, 0xf2, 0x72, 0x72, 0x61, 0x42, 0x72,
0x75, 0x73, 0x73, 0xe8, 0x6c, 0x61, 0x73, 0x42, 0x75, 0x63, 0x61, 0x72,
0xe8, 0x73, 0x74, 0x42, 0x75, 0x64, 0x61, 0x70, 0xe8, 0x73, 0x74, 0x4c,
0x75, 0x78, 0x65, 0x6d, 0x62, 0x6f, 0x72, 0x67, 0x4d, 0xf3, 0x6e, 0x65,
0x67, 0x75, 0x65, 0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75, 0x4d, 0x61, 0x6c,
0x76, 0x69, 0x6e, 0x65, 0x73, 0x41, 0x62, 0x69, 0x79, 0x61, 0x6e, 0x41,
0x72, 0x67, 0x65, 0x6c, 0x45, 0x74, 0x68, 0x20, 0x43, 0x61, 0x69, 0x72,
0x6f, 0x59, 0x69, 0x62, 0x75, 0x74, 0x69, 0x4a, 0x61, 0x72, 0x74, 0x75,
0x6d, 0x59, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x75, 0x61, 0x6b, 0x63,
0x68, 0x6f, 0x74, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x54, 0x6f, 0x6d, 0xe9,
0x43, 0xf2, 0x64, 0x6f, 0x62, 0x61, 0x42, 0x65, 0x6c, 0xe9, 0x6e, 0x42,
0x65, 0x6c, 0x69, 0x63, 0x65, 0x43, 0x61, 0x79, 0x65, 0x6e, 0x61, 0x43,
0xf2, 0x73, 0x74, 0x61, 0x20, 0x52, 0x69, 0x63, 0x61, 0x47, 0x72, 0x61,
0x6e, 0x20, 0x54, 0x75, 0x72, 0x63, 0x61, 0x45, 0x72, 0x61, 0x20, 0x48,
0x61, 0x76, 0x61, 0x6e, 0x61, 0x4d, 0x6d, 0x4d, 0x61, 0x6e, 0x61, 0x6f,
0x73, 0x43, 0x69, 0x75, 0x74, 0x61, 0x74, 0x20, 0x64, 0x65, 0x20, 0x4d,
0xe9, 0x78, 0x69, 0x63, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x59, 0x6f, 0x72,
0x6b, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x4e, 0xf2, 0x72,
0x64, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x4e, 0xf2, 0x72,
0x64, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20,
0x4e, 0xf2, 0x72, 0x64, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x50,
0x72, 0xed, 0x6e, 0x63, 0x69, 0x70, 0x65, 0x50, 0x75, 0x65, 0x72, 0x74,
0x6f, 0x20, 0x45, 0x73, 0x70, 0x61, 0x6e, 0x68, 0x61, 0x53, 0x61, 0x6f,
0x20, 0x70, 0x61, 0x75, 0x6c, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x42,
0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0xe9, 0x53, 0x61, 0x6e, 0x74,
0x20, 0x4a, 0x6f, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x6e, 0xf2, 0x76, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x43,
0x72, 0x69, 0x73, 0x74, 0xf2, 0x62, 0x61, 0x6c, 0x53, 0x61, 0x6e, 0x74,
0x20, 0x56, 0x69, 0x63, 0x65, 0x6e, 0x74, 0x54, 0xf2, 0x72, 0x74, 0x6f,
0x6c, 0x61, 0x41, 0x6e, 0xe1, 0x64, 0x79, 0x72, 0x41, 0x73, 0x6a, 0x61,
0x62, 0x61, 0x64, 0x42, 0x61, 0x72, 0xe9, 0x69, 0x6e, 0x42, 0x61, 0x72,
0x6e, 0x61, 0xfa, 0x6c, 0x44, 0x75, 0x73, 0x61, 0x6d, 0x62, 0xe9, 0x43,
0x69, 0x75, 0x74, 0x61, 0x74, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69,
0x20, 0x4d, 0x69, 0x6e, 0x68, 0x4a, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c,
0x65, 0x6e, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0xed, 0x6e, 0x53, 0x65, 0x6f,
0x6c, 0x54, 0x69, 0x66, 0x6c, 0x69, 0x65, 0x73, 0x45, 0x72, 0x65, 0x76,
0xe1, 0x6e, 0x49, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6f,
0x65, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x74,
0x68, 0x20, 0x53, 0x75, 0x64, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x45,
0x6c, 0x65, 0x6e, 0x61, 0x53, 0xed, 0x64, 0x6e, 0x65, 0x79, 0x49, 0x73,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x45, 0x73, 0x74,
0x61, 0x6d, 0x62, 0x75, 0x6c, 0x4c, 0xf2, 0x6e, 0x64, 0x72, 0x65, 0x73,
0x53, 0x61, 0x6e, 0x74, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x53,
0x61, 0x72, 0xe1, 0x74, 0x6f, 0x76, 0x53, 0x6b, 0x6f, 0x70, 0x69, 0x65,
0x55, 0x6c, 0x69, 0xe1, 0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x45, 0x74, 0x68,
0x20, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x5a, 0xfa, 0x72, 0x69,
0x63, 0x68, 0x4e, 0x61, 0x64, 0x61, 0x75, 0x49, 0x73, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x46, 0x69, 0x79,
0x69, 0xb06, 0xb2c, 0xb3f, 0xb26, 0xb1c, 0xb3e, 0xb28, 0xb06, 0xb15, 0xb3e, 0xb30,
0xb3e, 0xb06, 0xb26, 0xb3f, 0xb38, 0xb4d, 0x200c, 0x20, 0xb06, 0xb2c, 0xb3e, 0xb2c,
0xb3e, 0xb05, 0xb32, 0xb1c, 0xb3f, 0xb5f, 0xb30, 0xb4d, 0xb38, 0xb05, 0xb38, 0xb2e,
0xb30, 0xb3e, 0xb2c, 0xb3e, 0xb2e, 0xb3e, 0xb15, 0xb4b, 0xb2c, 0xb3e, 0xb19, 0xb4d,
0xb17, 0xb41, 0xb07, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1c, 0xb41, 0xb32, 0xb2c, 0xb3f,
0xb38, 0xb3e, 0xb09, 0xb2c, 0xb4d, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3e, 0xb5f,
0xb3e, 0xb30, 0xb4d, 0x200c, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb1c, 0xb3e, 0xb2d, 0xb3f,
0xb32, 0xb4d, 0xb32, 0xb47, 0xb2c, 0xb41, 0xb1c, 0xb41, 0xb2e, 0xb4d, 0xb2c, 0xb41,
0xb30, 0xb3e, 0xb15, 0xb3e, 0xb07, 0xb30, 0xb4b, 0xb15, 0xb3e, 0xb38, 0xb3e, 0xb2c,
0xb4d, 0xb32, 0xb3e, 0xb19, 0xb4d, 0xb15, 0xb3e, 0xb38, 0xb47, 0xb09, 0xb1f, 0xb3e,
0xb15, 0xb4b, 0xb28, 0xb3e, 0xb15, 0xb4d, 0xb30, 0xb3f, 0xb21, 0xb15, 0xb3e, 0xb30,
0xb21, 0xb30, 0xb4d, 0x200c, 0x20, 0xb07, 0xb38, 0xb4d, 0x200c, 0x20, 0xb38, 0xb3e,
0xb32, 0xb3e, 0xb2e, 0xb1c, 0xb3f, 0xb2c, 0xb4b, 0xb1f, 0xb3f, 0xb21, 0xb09, 0xb32,
0xb3e, 0xb0f, 0xb32, 0xb4d, 0x200c, 0x20, 0xb0f, 0xb5f, 0xb28, 0xb4d, 0xb2b, 0xb4d,
0xb30, 0xb3f, 0xb1f, 0xb3e, 0xb09, 0xb28, 0xb4d, 0x200c, 0xb17, 0xb3e, 0xb2c, 0xb4b,
0xb30, 0xb4d, 0xb23, 0xb4d, 0xb23, 0xb39, 0xb30, 0xb3e, 0xb30, 0xb47, 0xb1c, 0xb4b,
0xb39, 0xb3e, 0xb28, 0xb4d, 0xb38, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb1c, 0xb41, 0xb2c,
0xb3e, 0xb15, 0xb3e, 0xb2e, 0xb4d, 0xb2a, 0xb3e, 0xb32, 0xb3e, 0xb16, 0xb30, 0xb1f,
0xb4c, 0xb2e, 0xb4d, 0x200c, 0xb15, 0xb3f, 0xb17, 0xb3e, 0xb32, 0xb3f, 0xb15, 0xb3f,
0xb28, 0xb38, 0xb3e, 0xb38, 0xb4d, 0x200c, 0xb32, 0xb3e, 0xb17, 0xb4b, 0xb38, 0xb4d,
0x200c, 0xb32, 0xb3f, 0xb2c, 0xb4d, 0xb30, 0xb47, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32,
0xb47, 0xb32, 0xb4b, 0xb2e, 0xb4d, 0x200c, 0xb32, 0xb41, 0xb06, 0xb23, 0xb4d, 0xb21,
0xb3e, 0xb32, 0xb41, 0xb2c, 0xb41, 0xb2e, 0xb4d, 0xb2c, 0xb3e, 0xb36, 0xb3f, 0xb32,
0xb41, 0xb38, 0xb3e, 0xb15, 0xb3e, 0xb2e, 0xb3e, 0xb32, 0xb3e, 0xb2c, 0xb4b, 0xb2e,
0xb3e, 0xb2a, 0xb41, 0xb24, 0xb4b, 0xb2e, 0xb47, 0xb38, 0xb47, 0xb30, 0xb41, 0xb2c,
0xb3e, 0xb2c, 0xb47, 0xb28, 0xb4d, 0x200c, 0xb2e, 0xb4b, 0xb17, 0xb3e, 0xb26, 0xb3f,
0xb36, 0xb41, 0xb2e, 0xb28, 0xb30, 0xb4b, 0xb2d, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb07,
0xb30, 0xb4b, 0xb2c, 0xb3f, 0xb1c, 0xb3e, 0xb2e, 0xb47, 0xb28, 0xb3e, 0xb28, 0xb3f,
0xb06, 0xb2e, 0xb3f, 0xb28, 0xb4c, 0xb15, 0xb3e, 0xb1a, 0xb4b, 0xb1f, 0xb05, 0xb09,
0xb17, 0xb3e, 0xb21, 0xb09, 0xb17, 0xb09, 0xb2a, 0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb4b,
0x2d, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb38, 0xb3e, 0xb13, 0x20, 0xb1f, 0xb4b, 0xb2e,
0xb47, 0xb24, 0xb4d, 0xb30, 0xb3f, 0xb2a, 0xb4b, 0xb32, 0xb3f, 0xb1f, 0xb41, 0xb28,
0xb3f, 0xb38, 0xb4d, 0x200c, 0xb71, 0xb3f, 0xb23, 0xb4d, 0xb21, 0xb39, 0xb4b, 0xb0f,
0xb15, 0xb4d, 0xb06, 0xb21, 0xb3e, 0xb15, 0xb4d, 0xb06, 0xb19, 0xb4d, 0xb15, 0xb30,
0xb47, 0xb1c, 0xb4d, 0xb06, 0xb19, 0xb4d, 0xb17, 0xb41, 0xb07, 0xb32, 0xb3e, 0xb06,
0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb17, 0xb41, 0xb06, 0xb06, 0xb30, 0xb3e, 0xb17, 0xb41,
0xb06, 0xb28, 0xb3e, 0xb2c, 0xb41, 0xb0f, 0xb28, 0xb38, 0xb4d, 0x200c, 0x20, 0xb0f,
0xb30, 0xb3f, 0xb38, 0xb4d, 0x200c, 0xb15, 0xb3e, 0xb1f, 0xb3e, 0xb2e, 0xb3e, 0xb30,
0xb4d, 0xb15, 0xb3e, 0xb15, 0xb4b, 0xb30, 0xb4d, 0xb21, 0xb4b, 0xb2c, 0xb3e, 0xb1c,
0xb41, 0xb1c, 0xb41, 0xb07, 0xb32, 0xb3e, 0x20, 0xb30, 0xb3f, 0xb13, 0xb1c, 0xb3e,
0xb2e, 0xb47, 0xb23, 0xb4d, 0xb21, 0xb4b, 0xb1c, 0xb3e, 0xb30, 0xb3f, 0xb13, 0x20,
0xb17, 0xb3e, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb17, 0xb38, 0xb4d, 0x200c, 0xb38, 0xb32,
0xb4d, 0xb1f, 0xb3e, 0xb38, 0xb3e, 0xb28, 0xb4d, 0x200c, 0x20, 0xb1c, 0xb41, 0xb06,
0xb28, 0xb38, 0xb3e, 0xb28, 0xb4d, 0x200c, 0x20, 0xb32, 0xb41, 0xb07, 0xb38, 0xb4d,
0x200c, 0xb1f, 0xb41, 0xb15, 0xb41, 0xb2e, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb09, 0xb36,
0xb41, 0xb5f, 0xb3e, 0xb06, 0xb30, 0xb41, 0xb2c, 0xb3e, 0xb06, 0xb38, 0xb28, 0xb38,
0xb3f, 0xb05, 0xb28, 0xb4d, 0x200c, 0xb06, 0xb1f, 0xb3f, 0xb15, 0xb4b, 0xb15, 0xb3e,
0xb28, 0xb4d, 0xb2c, 0xb3e, 0xb39, 0xb3f, 0xb06, 0xb2c, 0xb3e, 0xb39, 0xb3f, 0xb06,
0x20, 0xb2c, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb47, 0xb30, 0xb3e, 0xb38,
0xb4d, 0xb2c, 0xb3e, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb21, 0xb4b, 0xb38, 0xb4d, 0xb2c,
0xb47, 0xb32, 0xb47, 0xb2e, 0xb2c, 0xb47, 0xb32, 0xb3f, 0xb1c, 0xb47, 0xb2c, 0xb4d,
0xb32, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x2d, 0xb38, 0xb3e, 0xb2c, 0xb32, 0xb4b, 0xb28,
0xb4d, 0xb2c, 0xb4b, 0xb06, 0x20, 0xb2d, 0xb3f, 0xb37, 0xb4d, 0xb1f, 0xb3e, 0xb2c,
0xb4b, 0xb17, 0xb4b, 0xb1f, 0xb3e, 0xb2c, 0xb4b, 0xb07, 0xb38, 0xb47, 0xb15, 0xb47,
0xb2e, 0xb4d, 0xb35, 0xb4d, 0xb30, 0xb3f, 0xb1c, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb15,
0xb3e, 0xb2e, 0xb4d, 0xb2a, 0xb4b, 0x20, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb23, 0xb4d,
0xb21, 0xb47, 0xb15, 0xb3e, 0xb28, 0xb15, 0xb41, 0xb28, 0xb4d, 0xb15, 0xb3e, 0xb30,
0xb15, 0xb3e, 0xb38, 0xb4d, 0x200c, 0xb15, 0xb47, 0xb5f, 0xb47, 0xb28, 0xb4d, 0xb28,
0xb3f, 0xb15, 0xb3e, 0xb5f, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb28, 0xb4d, 0xb1a, 0xb3f,
0xb15, 0xb3e, 0xb17, 0xb4b, 0xb1a, 0xb3f, 0xb39, 0xb41, 0xb06, 0xb39, 0xb41, 0xb06,
0xb38, 0xb3f, 0xb09, 0xb21, 0xb3e, 0xb21, 0xb4d, 0x20, 0xb1c, 0xb41, 0xb06, 0xb30,
0xb47, 0xb1c, 0xb4d, 0xb15, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb3e, 0x20, 0xb30, 0xb3f,
0xb15, 0xb3e, 0xb15, 0xb4d, 0xb30, 0xb47, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb28, 0xb4d,
0xb15, 0xb41, 0xb07, 0xb2c, 0xb3e, 0xb15, 0xb41, 0xb30, 0xb3e, 0xb15, 0xb4b, 0xb21,
0xb3e, 0xb28, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb38, 0xb3e, 0xb2d, 0xb28, 0xb4d,
0xb21, 0xb38, 0xb28, 0xb4d, 0x200c, 0xb21, 0xb71, 0xb38, 0xb28, 0xb4d, 0x20, 0xb15,
0xb4d, 0xb30, 0xb3f, 0xb15, 0xb4d, 0xb21, 0xb47, 0xb28, 0xb2d, 0xb3f, 0xb30, 0xb4d,
0xb21, 0xb47, 0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb07, 0xb1f, 0xb4d, 0xb21, 0xb4b, 0xb2e,
0xb3f, 0xb28, 0xb3f, 0xb15, 0xb3e, 0xb0f, 0xb21, 0xb2e, 0xb4b, 0xb28, 0xb1f, 0xb4b,
0xb28, 0xb4d, 0xb07, 0xb30, 0xb41, 0xb28, 0xb47, 0xb2a, 0xb47, 0xb0f, 0xb32, 0xb4d,
0x20, 0xb38, 0xb3e, 0xb32, 0xb2d, 0xb3e, 0xb21, 0xb4b, 0xb30, 0xb4d, 0xb2b, 0xb4b,
0xb30, 0xb4d, 0xb1f, 0xb4d, 0x20, 0xb28, 0xb47, 0xb32, 0xb38, 0xb28, 0xb4d, 0xb2b,
0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb47, 0xb32, 0xb47, 0xb1c, 0xb3e, 0xb17, 0xb4d, 0xb32,
0xb3e, 0xb38, 0xb47, 0x20, 0xb2c, 0xb47, 0xb17, 0xb41, 0xb38, 0xb4d, 0x20, 0xb2c,
0xb47, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb1f, 0xb30,
0xb4d, 0xb15, 0xb4d, 0xb17, 0xb4d, 0xb30, 0xb47, 0xb28, 0xb3e, 0xb21, 0xb3e, 0xb17,
0xb41, 0xb06, 0xb21, 0xb47, 0xb32, 0xb4b, 0xb2a, 0xb4d, 0x200c, 0xb17, 0xb41, 0xb06,
0xb1f, 0xb47, 0xb2e, 0xb3e, 0xb32, 0xb3e, 0xb17, 0xb41, 0xb5f, 0xb3e, 0xb15, 0xb4d,
0xb71, 0xb3f, 0xb32, 0xb17, 0xb41, 0xb0f, 0xb28, 0xb3e, 0xb39, 0xb3e, 0xb32, 0xb3f,
0xb2b, 0xb4d, 0xb5f, 0xb3e, 0xb15, 0xb4d, 0xb38, 0xb4d, 0xb39, 0xb3e, 0xb2d, 0xb3e,
0xb28, 0xb3e, 0xb39, 0xb47, 0xb30, 0xb2e, 0xb4b, 0xb38, 0xb3f, 0xb32, 0xb4b, 0xb07,
0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2a, 0xb4b, 0xb32, 0xb3f, 0xb38,
0xb4d, 0xb15, 0xb4d, 0xb28, 0xb4b, 0xb15, 0xb4d, 0xb38, 0x2c, 0x20, 0xb07, 0xb23,
0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb47, 0xb28, 0xb17,
0xb4b, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2a,
0xb47, 0xb1f, 0xb47, 0xb30, 0xb4d, 0xb38, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb4d, 0x2c,
0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb1f, 0xb47, 0xb32,
0xb4d, 0x20, 0xb38, 0xb3f, 0xb1f, 0xb3f, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21,
0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2d, 0xb47, 0xb2d, 0xb3e, 0xb5f, 0x2c, 0x20, 0xb07,
0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2d, 0xb3f, 0xb28, 0xb38, 0xb47,
0xb28, 0xb4d, 0xb28, 0xb47, 0xb38, 0xb4d, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21,
0xb3f, 0xb06, 0xb28, 0xb3e, 0xb71, 0xb3f, 0xb28, 0xb3e, 0xb2e, 0xb3e, 0xb15, 0xb4d,
0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb07, 0xb28,
0xb41, 0xb2d, 0xb3f, 0xb15, 0xb4d, 0xb07, 0xb15, 0xb4d, 0xb35, 0xb3e, 0xb32, 0xb41,
0xb07, 0xb1f, 0xb4d, 0xb1c, 0xb3e, 0xb2e, 0xb3e, 0xb07, 0xb15, 0xb3e, 0xb1c, 0xb41,
0xb28, 0xb47, 0xb06, 0xb09, 0xb32, 0xb41, 0xb07, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32,
0xb47, 0xb2e, 0xb4b, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb38, 0xb47, 0xb32, 0xb4b, 0x2c,
0x20, 0xb15, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb09, 0xb15, 0xb3f, 0xb15, 0xb47, 0xb15,
0xb4d, 0xb30, 0xb3e, 0xb32, 0xb47, 0xb23, 0xb4d, 0xb21, 0xb1c, 0xb3f, 0xb15, 0xb32,
0xb3e, 0x20, 0xb2a, 0xb3e, 0xb1c, 0xb4d, 0x200c, 0xb32, 0xb3f, 0xb2e, 0xb3e, 0xb32,
0xb38, 0xb4d, 0x20, 0xb0f, 0xb1e, 0xb4d, 0xb1c, 0xb47, 0xb32, 0xb47, 0xb38, 0xb4d,
0xb28, 0xb3f, 0xb2e, 0xb4d, 0xb28, 0x20, 0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb4d,
0xb38, 0x2019, 0xb30, 0x20, 0xb15, 0xb4d, 0xb35, 0xb3e, 0xb1f, 0xb30, 0xb4d, 0xb2e,
0xb3e, 0xb38, 0xb3f, 0xb13, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb17, 0xb41, 0xb06, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb09, 0xb38, 0xb4d, 0x200c, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb17,
0xb4b, 0xb1f, 0xb4d, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3f, 0xb15,
0xb4d, 0xb5f, 0xb41, 0xb2e, 0xb3e, 0xb1f, 0xb3e, 0xb2e, 0xb4b, 0xb30, 0xb4b, 0xb38,
0xb4d, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb1f, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb2e, 0xb47,
0xb28, 0xb4b, 0xb2e, 0xb3f, 0xb28, 0xb3f, 0xb2e, 0xb47, 0xb30, 0xb3f, 0xb21, 0xb3e,
0xb2e, 0xb3e, 0xb1f, 0xb32, 0xb3e, 0xb15, 0xb3e, 0xb1f, 0xb32, 0xb3e, 0xb2e, 0xb47,
0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb4b, 0x20, 0xb38, 0xb3f, 0xb1f, 0xb3f, 0xb2e,
0xb3f, 0xb15, 0xb4d, 0xb35, 0xb47, 0xb32, 0xb4b, 0xb28, 0xb4d, 0xb2e, 0xb3e, 0xb28,
0xb15, 0xb1f, 0xb4b, 0xb28, 0xb4d, 0xb2e, 0xb4b, 0xb28, 0xb1f, 0xb47, 0xb30, 0xb3f,
0xb0f, 0xb2e, 0xb23, 0xb4d, 0xb1f, 0xb47, 0xb2d, 0xb3f, 0xb21, 0xb3f, 0xb13, 0xb2e,
0xb4b, 0xb28, 0xb1f, 0xb38, 0xb47, 0xb30, 0xb30, 0xb3e, 0xb1f, 0xb4d, 0xb28, 0xb3e,
0xb38, 0xb3e, 0xb09, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb5f, 0xb4b, 0xb30, 0xb4d,
0xb15, 0xb4d, 0xb28, 0xb4b, 0xb2e, 0xb47, 0xb28, 0xb4b, 0xb30, 0xb4b, 0xb28, 0xb4d,
0xb28, 0xb3e, 0xb2c, 0xb47, 0xb09, 0xb32, 0xb3e, 0xb39, 0x2c, 0x20, 0xb09, 0xb24,
0xb4d, 0xb24, 0xb30, 0x20, 0xb21, 0xb3e, 0xb15, 0xb4b, 0xb1f, 0xb3e, 0xb15, 0xb47,
0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0x2c, 0x20, 0xb09, 0xb24, 0xb4d, 0xb24, 0xb30,
0x20, 0xb21, 0xb3e, 0xb15, 0xb4b, 0xb1f, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20,
0xb38, 0xb3e, 0xb32, 0xb47, 0xb2e, 0xb4d, 0x2c, 0x20, 0xb09, 0xb24, 0xb4d, 0xb24,
0xb30, 0x20, 0xb21, 0xb3e, 0xb15, 0xb4b, 0xb1f, 0xb3e, 0xb28, 0xb41, 0xb09, 0xb15,
0xb4d, 0xb13, 0xb1c, 0xb3f, 0xb28, 0xb3e, 0xb17, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0xb3e,
0xb2e, 0xb3e, 0xb2a, 0xb3e, 0xb30, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb2c, 0xb4b,
0xb2b, 0xb4b, 0xb07, 0xb28, 0xb3f, 0xb15, 0xb4d, 0xb38, 0xb2a, 0xb4b, 0xb30, 0xb4d,
0xb1f, 0xb4d, 0x2d, 0xb0f, 0xb5f, 0xb41, 0x2d, 0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb28,
0xb4d, 0xb38, 0xb2a, 0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x20, 0xb05, 0xb2b, 0xb4d,
0x20, 0xb38, 0xb4d, 0xb2a, 0xb47, 0xb28, 0xb4d, 0xb2a, 0xb4b, 0xb30, 0xb4d, 0xb1f,
0xb4b, 0x20, 0xb2d, 0xb47, 0xb32, 0xb4b, 0xb2a, 0xb41, 0xb0f, 0xb30, 0xb4d, 0xb24,
0xb4b, 0x20, 0xb30, 0xb3f, 0xb15, 0xb4b, 0xb2a, 0xb41, 0xb23, 0xb4d, 0xb1f, 0xb3e,
0x20, 0xb0f, 0xb30, 0xb3f, 0xb28, 0xb3e, 0xb38, 0xb4d, 0x200c, 0xb30, 0xb3e, 0xb28,
0xb15, 0xb3f, 0xb28, 0xb4d, 0x20, 0xb07, 0xb28, 0xb32, 0xb47, 0xb1f, 0xb4d, 0xb30,
0xb47, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb30, 0xb47, 0xb17, 0xb3f, 0xb28, 0xb3e, 0xb30,
0xb3f, 0xb38, 0xb4b, 0xb32, 0xb41, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb13, 0x20, 0xb2c,
0xb4d, 0xb30, 0xb3e, 0xb19, 0xb4d, 0xb15, 0xb4b, 0xb38, 0xb3e, 0xb23, 0xb4d, 0xb1f,
0xb3e, 0xb30, 0xb47, 0xb2e, 0xb4d, 0x200c, 0xb38, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3f,
0xb06, 0xb17, 0xb4b, 0xb38, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb4b, 0x20, 0xb21, 0xb4b,
0xb2e, 0xb3f, 0xb19, 0xb4d, 0xb17, 0xb4b, 0xb38, 0xb3e, 0xb13, 0x20, 0xb2a, 0xb3e,
0xb13, 0xb32, 0xb4b, 0xb07, 0xb1f, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0xb35, 0xb4b,
0xb30, 0xb1f, 0xb4b, 0xb30, 0xb2e, 0xb3f, 0xb1f, 0xb4d, 0xb38, 0xb3f, 0xb1f, 0xb15,
0xb3e, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20, 0xb2c, 0xb3e, 0xb30,
0xb4d, 0xb25, 0xb47, 0xb32, 0xb47, 0xb2e, 0xb3f, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f,
0xb4d, 0x2e, 0x20, 0xb1c, 0xb28, 0xb38, 0xb4d, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f,
0xb4d, 0x20, 0xb15, 0xb3f, 0xb1f, 0xb4d, 0xb38, 0xb4d, 0xb38, 0xb47, 0xb23, 0xb4d,
0xb1f, 0xb4d, 0x2e, 0x20, 0xb32, 0xb41, 0xb38, 0xb3f, 0xb06, 0xb38, 0xb47, 0xb23,
0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20, 0xb25, 0xb4b, 0xb2e, 0xb3e, 0xb38, 0xb4d, 0xb38,
0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20, 0xb2d, 0xb3f, 0xb28, 0xb38, 0xb47,
0xb23, 0xb4d, 0xb1f, 0xb4d, 0xb38, 0xb41, 0xb07, 0xb2b, 0xb4d, 0xb1f, 0xb4d, 0x20,
0xb15, 0xb30, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0xb1f, 0xb47, 0xb17, 0xb41, 0xb38,
0xb3f, 0xb17, 0xb3e, 0xb32, 0xb2a, 0xb3e, 0xb25, 0xb41, 0xb32, 0xb47, 0xb24, 0xb3f,
0xb1c, 0xb41, 0xb06, 0xb28, 0xb3e, 0xb1f, 0xb4b, 0xb30, 0xb4b, 0xb23, 0xb4d, 0xb1f,
0xb4b, 0xb1f, 0xb4b, 0xb30, 0xb1f, 0xb4b, 0xb32, 0xb3e, 0xb2d, 0xb3e, 0xb19, 0xb4d,
0xb15, 0xb41, 0xb2d, 0xb30, 0xb4d, 0xb39, 0xb4d, 0xb35, 0xb3e, 0xb07, 0xb1f, 0xb39,
0xb30, 0xb4d, 0xb38, 0xb4d, 0xb71, 0xb3f, 0xb28, 0xb3f, 0xb2a, 0xb47, 0xb17, 0xb4d,
0xb5f, 0xb3e, 0xb15, 0xb41, 0xb1f, 0xb3e, 0xb1f, 0xb4d, 0xb21, 0xb47, 0xb2d, 0xb3f,
0xb38, 0xb4d, 0x200c, 0xb21, 0xb41, 0xb2e, 0xb4b, 0xb23, 0xb4d, 0xb1f, 0x20, 0xb21,
0xb3f, 0x2018, 0xb09, 0xb30, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb2e, 0xb3e,
0xb15, 0xb4d, 0xb71, 0xb47, 0xb30, 0xb40, 0xb2e, 0xb3e, 0xb71, 0xb38, 0xb28, 0xb4d,
0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb15, 0xb4d, 0x200c, 0xb2e, 0xb41, 0xb30, 0xb4d, 0xb21,
0xb4b, 0xb2a, 0xb3e, 0xb32, 0xb2e, 0xb30, 0xb4d, 0x200c, 0xb30, 0xb4b, 0xb25, 0xb47,
0xb30, 0xb3e, 0xb38, 0xb4b, 0xb71, 0xb3e, 0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb32, 0xb4d,
0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x200c, 0xb32, 0xb19, 0xb4d,
0xb17, 0xb5f, 0xb47, 0xb06, 0xb30, 0xb2c, 0xb5f, 0xb47, 0xb28, 0xb4d, 0xb0f, 0xb21,
0xb47, 0xb28, 0xb4d, 0x200c, 0xb05, 0xb32, 0xb2e, 0xb3e, 0xb1f, 0xb3f, 0xb05, 0xb2e,
0xb4d, 0xb2e, 0xb3e, 0xb28, 0xb06, 0xb28, 0xb3e, 0xb21, 0xb5f, 0xb3e, 0xb30, 0xb4d,
0xb06, 0xb15, 0xb1f, 0xb3e, 0xb09, 0xb06, 0xb15, 0xb1f, 0xb4b, 0xb2c, 0xb4d, 0x200c,
0xb06, 0xb36, 0xb4d, 0x200c, 0xb17, 0xb3e, 0xb2c, 0xb4b, 0xb1f, 0xb4d, 0x200c, 0xb05,
0xb24, 0xb3f, 0xb30, 0xb3e, 0xb09, 0xb2c, 0xb3e, 0xb17, 0xb26, 0xb3e, 0xb26, 0xb4d,
0x200c, 0xb2c, 0xb3e, 0xb39, 0xb3e, 0xb30, 0xb3f, 0xb28, 0xb4d, 0xb2c, 0xb4d, 0xb5f,
0xb3e, 0xb19, 0xb4d, 0xb17, 0xb15, 0xb15, 0xb4d, 0x200c, 0xb2c, 0xb3e, 0xb30, 0xb28,
0xb3e, 0xb09, 0xb32, 0xb4d, 0xb2c, 0xb40, 0xb30, 0xb41, 0xb1f, 0xb4d, 0x200c, 0xb2c,
0xb3f, 0xb36, 0xb15, 0xb47, 0xb15, 0xb4d, 0x200c, 0xb2c, 0xb4d, 0xb30, 0xb41, 0xb28,
0xb47, 0xb07, 0xb1a, 0xb3f, 0xb1f, 0xb3e, 0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb4b,
0xb21, 0xb3e, 0xb2e, 0xb3e, 0xb38, 0xb15, 0xb38, 0xb4d, 0x200c, 0xb22, 0xb3e, 0xb15,
0xb3e, 0xb26, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb40, 0xb26, 0xb41, 0xb2c, 0xb3e, 0xb07,
0xb26, 0xb41, 0xb36, 0xb3e, 0xb28, 0xb2c, 0xb47, 0xb2b, 0xb3e, 0xb2e, 0xb3e, 0xb17,
0xb41, 0xb37, 0xb4d, 0xb1f, 0xb3e, 0xb17, 0xb3e, 0xb1c, 0xb3e, 0xb39, 0xb47, 0xb2c,
0xb4d, 0xb30, 0xb28, 0xb4d, 0x200c, 0xb39, 0xb4b, 0x20, 0xb1a, 0xb3f, 0x20, 0xb2e,
0xb3f, 0xb28, 0xb4d, 0x200c, 0x20, 0xb38, 0xb3f, 0xb1f, 0xb3f, 0xb39, 0xb02, 0x20,
0xb15, 0xb02, 0xb39, 0xb4b, 0xb2d, 0xb21, 0xb4d, 0x200c, 0xb07, 0xb30, 0xb15, 0xb41,
0xb1f, 0xb38, 0xb4d, 0xb15, 0xb1c, 0xb3e, 0xb15, 0xb30, 0xb4d, 0xb24, 0xb4d, 0xb24,
0xb3e, 0xb1c, 0xb5f, 0xb2a, 0xb41, 0xb30, 0xb3e, 0xb1c, 0xb47, 0xb30, 0xb41, 0xb1c,
0xb47, 0xb32, 0xb2e, 0xb15, 0xb3e, 0xb2c, 0xb41, 0xb32, 0xb15, 0xb3e, 0xb2e, 0xb1a,
0xb3e, 0xb1f, 0xb15, 0xb3e, 0xb15, 0xb30, 0xb3e, 0xb1a, 0xb3f, 0xb15, 0xb3e, 0xb20,
0xb2e, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb41, 0xb16, 0xb3e, 0xb28, 0xb21, 0xb4d, 0xb5f,
0xb3e, 0xb17, 0xb3e, 0xb15, 0xb4b, 0xb32, 0xb15, 0xb3e, 0xb24, 0xb3e, 0xb15, 0xb4d,
0xb30, 0xb3e, 0xb38, 0xb28, 0xb4b, 0xb5f, 0xb3e, 0xb30, 0xb38, 0xb4d, 0xb15, 0xb15,
0xb4d, 0xb71, 0xb3e, 0xb32, 0xb3e, 0xb32, 0xb2e, 0xb4d, 0xb2a, 0xb41, 0xb30, 0xb15,
0xb41, 0xb1a, 0xb3f, 0xb02, 0xb15, 0xb41, 0xb0f, 0xb24, 0xb2e, 0xb3e, 0xb15, 0xb3e,
0xb09, 0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb21, 0xb3e, 0xb28, 0xb4d, 0xb2e, 0xb3e, 0xb15,
0xb3e, 0xb38, 0xb3e, 0xb30, 0xb4d, 0x200c, 0xb2e, 0xb3e, 0xb28, 0xb3f, 0xb32, 0xb3e,
0xb2e, 0xb38, 0xb4d, 0xb15, 0xb3e, 0xb1f, 0xb4d, 0x200c, 0xb28, 0xb3f, 0xb15, 0xb4b,
0xb38, 0xb3f, 0xb06, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb15, 0xb41, 0xb1c, 0xb28, 0xb47,
0xb1f, 0xb38, 0xb4d, 0xb15, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb38, 0xb3f, 0xb2c, 0xb3f,
0xb30, 0xb38, 0xb4d, 0xb15, 0xb13, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb13, 0xb30, 0xb3e,
0xb32, 0xb4d, 0x200c, 0xb2b, 0xb28, 0xb4b, 0xb2e, 0xb4d, 0x200c, 0x20, 0xb2a, 0xb47,
0xb28, 0xb39, 0xb2a, 0xb4b, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb15,
0xb4d, 0x200c, 0xb2a, 0xb4b, 0xb5f, 0xb19, 0xb4d, 0xb17, 0xb5f, 0xb3e, 0xb19, 0xb4d,
0xb17, 0xb15, 0xb24, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb28,
0xb47, 0xb15, 0xb40, 0xb1c, 0xb3f, 0xb32, 0xb4b, 0xb30, 0xb4d, 0xb21, 0xb3e, 0xb30,
0xb3f, 0xb06, 0xb26, 0xb38, 0xb16, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0xb38, 0xb2e,
0xb30, 0xb15, 0xb28, 0xb4d, 0xb26, 0xb38, 0xb3f, 0xb13, 0xb32, 0xb38, 0xb3e, 0xb02,
0xb18, 0xb3e, 0xb07, 0xb38, 0xb3f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb2a, 0xb41, 0xb30,
0xb4d, 0x200c, 0xb38, 0xb4d, 0xb30, 0xb47, 0xb21, 0xb28, 0xb47, 0xb15, 0xb4b, 0xb32,
0xb5f, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47, 0xb07, 0xb24,
0xb3e, 0xb36, 0xb15, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb1f, 0xb3f, 0xb2c, 0xb3f, 0xb32,
0xb3f, 0xb38, 0xb3f, 0xb24, 0xb47, 0xb39, 0xb47, 0xb30, 0xb3e, 0xb28, 0xb4d, 0xb25,
0xb3f, 0xb2e, 0xb4d, 0xb2a, 0xb41, 0xb1f, 0xb4b, 0xb15, 0xb3f, 0xb13, 0xb1f, 0xb4b,
0xb2e, 0xb38, 0xb4d, 0xb15, 0xb09, 0xb32, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb2c, 0xb3e,
0xb1f, 0xb30, 0xb4d, 0x200c, 0xb09, 0xb30, 0xb41, 0xb2e, 0xb15, 0xb3f, 0xb5f, 0xb41,
0xb37, 0xb4d, 0xb1f, 0x2d, 0xb28, 0xb47, 0xb30, 0xb3e, 0xb2d, 0xb3f, 0xb0f, 0xb23,
0xb4d, 0xb1f, 0xb3f, 0xb0f, 0xb28, 0xb4d, 0x200c, 0xb2d, 0xb4d, 0xb32, 0xb3e, 0xb21,
0xb3f, 0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0xb5f, 0xb3e, 0xb15,
0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb5f, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb41, 0xb28,
0xb4d, 0x200c, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c,
0xb30, 0xb4d, 0xb17, 0xb4d, 0xb5f, 0xb47, 0xb30, 0xb47, 0xb2d, 0xb3e, 0xb28, 0xb4d,
0xb06, 0xb1c, 0xb4b, 0xb30, 0xb47, 0xb38, 0xb4d, 0xb2c, 0xb30, 0xb4d, 0xb2e, 0xb41,
0xb21, 0xb3e, 0xb15, 0xb3e, 0xb28, 0xb3e, 0xb30, 0xb47, 0xb15, 0xb47, 0xb2a, 0xb4d,
0x200c, 0x20, 0xb2d, 0xb30, 0xb4d, 0xb26, 0xb47, 0xb2b, 0xb3e, 0xb30, 0xb4b, 0xb0f,
0xb2e, 0xb3e, 0xb21, 0xb47, 0xb07, 0xb30, 0xb3e, 0xb30, 0xb47, 0xb5f, 0xb15, 0xb1c,
0xb3e, 0xb2d, 0xb3f, 0xb15, 0xb4d, 0xb26, 0xb15, 0xb4d, 0xb37, 0xb3f, 0xb23, 0x20,
0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb06, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d,
0x200c, 0x20, 0xb39, 0xb47, 0xb32, 0xb3f, 0xb28, 0xb3e, 0xb37, 0xb4d, 0xb1f, 0xb3e,
0xb28, 0xb32, 0xb3f, 0xb06, 0xb21, 0xb3f, 0xb32, 0xb47, 0xb21, 0xb4d, 0x200c, 0xb2c,
0xb4d, 0xb30, 0xb3f, 0xb38, 0xb2c, 0xb47, 0xb28, 0xb4d, 0x200c, 0xb2c, 0xb4d, 0xb30,
0xb4b, 0xb15, 0xb47, 0xb28, 0xb4d, 0x200c, 0x20, 0xb39, 0xb3f, 0xb32, 0xb4d, 0x200c,
0xb21, 0xb3e, 0xb30, 0xb71, 0xb3f, 0xb28, 0xb4d, 0x200c, 0xb5f, 0xb41, 0xb15, 0xb32,
0xb3e, 0xb39, 0xb4b, 0xb2c, 0xb3e, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x200c, 0xb32, 0xb3f,
0xb23, 0xb4d, 0xb21, 0xb47, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb32,
0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb4b, 0xb71, 0xb47, 0xb2e, 0xb47, 0xb32, 0xb2c,
0xb4b, 0xb30, 0xb4d, 0xb23, 0xb4d, 0xb23, 0xb2a, 0xb30, 0xb4d, 0xb25, 0xb4d, 0x200c,
0xb38, 0xb3f, 0xb21, 0xb28, 0xb40, 0xb06, 0xb2e, 0xb37, 0xb4d, 0xb1f, 0xb30, 0xb21,
0xb3e, 0xb2e, 0xb4d, 0xb06, 0xb28, 0xb21, 0xb4b, 0xb30, 0xb3e, 0xb06, 0xb37, 0xb4d,
0xb1f, 0xb4d, 0xb30, 0xb3e, 0xb16, 0xb3e, 0xb28, 0xb4d, 0xb0f, 0xb25, 0xb47, 0xb28,
0xb4d, 0xb38, 0xb2c, 0xb47, 0xb32, 0xb17, 0xb4d, 0xb30, 0xb47, 0xb21, 0xb4d, 0x200c,
0xb2c, 0xb30, 0xb4d, 0xb32, 0xb3f, 0xb28, 0xb4d, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb1f,
0xb3f, 0xb38, 0xb32, 0xb3e, 0xb2d, 0xb3e, 0xb2c, 0xb4d, 0xb30, 0xb38, 0xb32, 0xb4d,
0xb38, 0xb4d, 0xb2c, 0xb41, 0xb1a, 0xb3e, 0xb30, 0xb47, 0xb37, 0xb4d, 0xb1f, 0xb4d,
0xb2c, 0xb41, 0xb21, 0xb3e, 0xb2a, 0xb47, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb2c, 0xb41,
0xb38, 0xb3f, 0xb28, 0xb17, 0xb47, 0xb28, 0xb4d, 0xb1a, 0xb3f, 0xb38, 0xb3f, 0xb28,
0xb4c, 0xb15, 0xb4b, 0xb2a, 0xb47, 0xb28, 0xb39, 0xb3e, 0xb17, 0xb47, 0xb28, 0xb4d,
0xb21, 0xb2c, 0xb32, 0xb3f, 0xb28, 0xb4d, 0xb1c, 0xb3f, 0xb2c, 0xb4d, 0xb30, 0xb3e,
0xb32, 0xb1f, 0xb30, 0xb4d, 0x200c, 0xb17, 0xb41, 0xb0f, 0xb30, 0xb28, 0xb38, 0xb47,
0xb39, 0xb47, 0xb32, 0xb38, 0xb3f, 0xb28, 0xb15, 0xb3f, 0xb07, 0xb38, 0xb32, 0xb47,
0x20, 0xb05, 0xb2b, 0xb4d, 0x20, 0xb2e, 0xb47, 0xb28, 0xb4d, 0xb07, 0xb38, 0xb4d,
0xb24, 0xb3e, 0xb28, 0xb2c, 0xb41, 0xb32, 0xb1c, 0xb30, 0xb4d, 0xb38, 0xb3f, 0xb15,
0xb3e, 0xb32, 0xb3f, 0xb28, 0xb3f, 0xb28, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d,
0xb15, 0xb3f, 0xb30, 0xb4b, 0xb2d, 0xb15, 0xb3f, 0xb0f, 0xb2d, 0xb4d, 0xb32, 0xb3f,
0xb38, 0xb2c, 0xb4b, 0xb28, 0xb4d, 0xb32, 0xb1c, 0xb41, 0xb2c, 0xb4d, 0xb32, 0xb1c,
0xb3e, 0xb28, 0xb4d, 0xb32, 0xb23, 0xb4d, 0xb21, 0xb28, 0xb4d, 0xb32, 0xb15, 0xb4d,
0xb38, 0xb2e, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb2e, 0xb3e, 0xb21, 0xb4d, 0xb30, 0xb3f,
0xb21, 0xb4d, 0xb2e, 0xb3e, 0xb32, 0xb1f, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb0f,
0xb39, 0xb3e, 0xb2e, 0xb28, 0xb4d, 0xb2e, 0xb3f, 0xb28, 0xb38, 0xb4d, 0xb15, 0xb2e,
0xb4b, 0xb28, 0xb3e, 0xb15, 0xb4b, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb4b, 0xb13, 0xb38,
0xb32, 0xb4b, 0xb2a, 0xb47, 0xb30, 0xb3f, 0xb38, 0xb4d, 0xb2a, 0xb4b, 0xb21, 0xb17,
0xb4b, 0xb30, 0xb3f, 0xb15, 0xb3e, 0xb2a, 0xb4d, 0xb30, 0xb3e, 0xb17, 0xb4d, 0xb30,
0xb3f, 0xb17, 0xb3e, 0xb30, 0xb4b, 0xb2e, 0xb4d, 0xb38, 0xb3e, 0xb2e, 0xb3e, 0xb30,
0xb3e, 0xb38, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb28, 0xb4b,
0xb38, 0xb3e, 0xb30, 0xb3e, 0xb1c, 0xb47, 0xb2d, 0xb4b, 0xb38, 0xb3e, 0xb30, 0xb3e,
0xb1f, 0xb4b, 0xb2d, 0xb4d, 0xb38, 0xb3f, 0xb2e, 0xb2b, 0xb47, 0xb30, 0xb4b, 0xb2a,
0xb4b, 0xb32, 0xb4d, 0xb38, 0xb4d, 0xb15, 0xb4b, 0xb2a, 0xb1c, 0xb47, 0xb38, 0xb4b,
0xb2b, 0xb3f, 0xb06, 0xb37, 0xb4d, 0xb1f, 0xb15, 0xb39, 0xb4b, 0xb2e, 0xb4d, 0x200c,
0xb1f, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0xb28, 0xb1f, 0xb3f, 0xb30, 0xb3e, 0xb28,
0xb47, 0xb5f, 0xb41, 0xb32, 0xb5f, 0xb3e, 0xb28, 0xb4b, 0xb2d, 0xb38, 0xb4d, 0xb15,
0xb2d, 0xb3e, 0xb21, 0xb41, 0xb1c, 0xb2d, 0xb3e, 0xb1f, 0xb3f, 0xb15, 0xb3e, 0xb28,
0xb4d, 0xb2d, 0xb3f, 0xb0f, 0xb28, 0xb3e, 0xb2d, 0xb3f, 0xb32, 0xb28, 0xb3f, 0xb09,
0xb38, 0xb4d, 0xb2d, 0xb4b, 0xb32, 0xb17, 0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21,
0xb4d, 0xb71, 0xb3e, 0xb30, 0xb38, 0xb71, 0xb3e, 0xb1c, 0xb3e, 0xb17, 0xb4d, 0xb30,
0xb47, 0xb2c, 0xb4d, 0xb1c, 0xb41, 0xb30, 0xb3f, 0xb15, 0xb4d, 0xb06, 0xb23, 0xb4d,
0xb1f, 0xb3e, 0xb28, 0xb3e, 0xb28, 0xb3e, 0xb30, 0xb3f, 0xb2d, 0xb4b, 0xb1a, 0xb3e,
0xb17, 0xb4b, 0xb38, 0xb4d, 0x200c, 0xb16, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb1f,
0xb2e, 0xb3e, 0xb38, 0xb15, 0xb4b, 0xb15, 0xb4b, 0xb38, 0xb4d, 0x200c, 0xb15, 0xb4b,
0xb2e, 0xb4b, 0xb30, 0xb4b, 0xb15, 0xb47, 0xb30, 0xb41, 0xb17, 0xb47, 0xb32, 0xb47,
0xb28, 0xb2e, 0xb3e, 0xb39, 0xb47, 0xb2e, 0xb3e, 0xb33, 0xb26, 0xb4d, 0xb71, 0xb40,
0xb2a, 0xb2e, 0xb4c, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0xb2e, 0xb3e, 0xb5f, 0xb4b,
0xb1f, 0xb47, 0xb30, 0xb3f, 0xb5f, 0xb41, 0xb28, 0xb3f, 0xb05, 0xb28, 0xb4d, 0x200c,
0xb06, 0xb2a, 0xb3f, 0xb06, 0xb05, 0xb15, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb2c,
0xb4c, 0xb17, 0xb47, 0xb28, 0xb4d, 0x200c, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47,
0xb1a, 0xb3e, 0xb25, 0xb3e, 0xb2e, 0xb4d, 0x200c, 0xb1a, 0xb42, 0xb15, 0xb4d, 0x200c,
0xb07, 0xb37, 0xb4d, 0xb1f, 0xb30, 0xb4d, 0x200c, 0xb07, 0xb2b, 0xb47, 0xb1f, 0xb4d,
0x200c, 0xb2b, 0xb3e, 0xb15, 0xb3e, 0xb13, 0xb2b, 0xb4b, 0xb2b, 0xb3f, 0xb1c, 0xb3f,
0xb2b, 0xb41, 0xb28, 0xb3e, 0xb2b, 0xb41, 0xb1f, 0xb3f, 0xb17, 0xb3e, 0xb32, 0xb3e,
0xb2a, 0xb3e, 0xb17, 0xb4b, 0xb38, 0xb17, 0xb3e, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb5f,
0xb30, 0xb4d, 0x200c, 0xb17, 0xb41, 0xb06, 0xb21, 0xb3e, 0xb32, 0xb15, 0xb3e, 0xb28,
0xb3e, 0xb32, 0xb39, 0xb4b, 0xb28, 0xb4b, 0xb32, 0xb41, 0xb32, 0xb41, 0xb0f, 0xb23,
0xb4d, 0xb21, 0xb47, 0xb30, 0xb2c, 0xb41, 0xb30, 0xb3f, 0xb15, 0xb3f, 0xb30, 0xb3f,
0xb24, 0xb3f, 0xb2e, 0xb3e, 0xb1f, 0xb3f, 0xb15, 0xb4b, 0xb38, 0xb30, 0xb47, 0xb07,
0xb15, 0xb4d, 0xb71, 0xb3e, 0xb1c, 0xb3e, 0xb32, 0xb47, 0xb07, 0xb28, 0xb4d, 0xb2e,
0xb3e, 0xb1c, 0xb41, 0xb30, 0xb4b, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb4d, 0xb71,
0xb47, 0xb38, 0xb3e, 0xb38, 0xb4d, 0x200c, 0xb2e, 0xb3f, 0xb21, 0xb4d, 0x200c, 0xb71,
0xb47, 0xb28, 0xb3e, 0xb09, 0xb30, 0xb41, 0xb28, 0xb3f, 0xb5f, 0xb42, 0xb28, 0xb30,
0xb2b, 0xb4b, 0xb15, 0xb4d, 0x200c, 0xb28, 0xb4c, 0xb2e, 0xb3f, 0xb5f, 0xb2a, 0xb3e,
0xb17, 0xb4b, 0x20, 0xb2a, 0xb3e, 0xb17, 0xb4b, 0xb2a, 0xb3e, 0xb32, 0xb3e, 0xb09,
0xb2a, 0xb3f, 0xb1f, 0xb15, 0xb3e, 0xb30, 0xb3f, 0xb28, 0xb4d, 0x200c, 0xb2a, 0xb4b,
0xb39, 0xb28, 0xb2a, 0xb47, 0xb07, 0xb2a, 0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x200c,
0x20, 0xb2e, 0xb4b, 0xb30, 0xb47, 0xb38, 0xb2c, 0xb3f, 0xb30, 0xb3e, 0xb30, 0xb4b,
0xb1f, 0xb4b, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb38, 0xb3e, 0xb07, 0xb2a, 0xb3e, 0xb28,
0xb4d, 0xb24, 0xb3e, 0xb39, 0xb3f, 0xb24, 0xb3f, 0xb24, 0xb3e, 0xb30, 0xb71, 0xb3e,
0xb1f, 0xb4b, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb1f, 0xb3e, 0xb2a, 0xb41, 0xb71, 0xb47,
0xb15, 0xb4d, 0xb71, 0xb3e, 0xb32, 0xb3f, 0xb38, 0xb4d, 0x200c, 0x41c, 0x4d5, 0x441,
0x43a, 0x443, 0x44b, 0x4b, 0xf2, 0x72, 0x73, 0x6f, 0x75, 0x52, 0x65, 0x70,
0xfa, 0x62, 0x6c, 0x6b, 0x61, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69,
0x63, 0x61, 0x6e, 0x61, 0x48, 0x65, 0x72, 0xfa, 0x73, 0x61, 0x6c, 0xe8,
0x6d, 0x41, 0x7a, 0xf3, 0x72, 0x65, 0x73, 0x4b, 0x61, 0x62, 0x6f, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x74, 0xe9, 0x6e, 0x61, 0x73, 0x4c,
0xf2, 0x6e, 0x64, 0x65, 0x6e, 0x627, 0x628, 0x64a, 0x62c, 0x627, 0x646, 0x627,
0x6a9, 0x631, 0x627, 0x627, 0x62f, 0x64a, 0x633, 0x20, 0x627, 0x628, 0x627, 0x628,
0x627, 0x627, 0x644, 0x62c, 0x64a, 0x64a, 0x631, 0x632, 0x627, 0x633, 0x645, 0x627,
0x631, 0x647, 0x628, 0x627, 0x645, 0x6cc, 0x6a9, 0x648, 0x628, 0x627, 0x646, 0x6ab,
0x648, 0x64a, 0x628, 0x633, 0x627, 0x624, 0x628, 0x644, 0x646, 0x67c, 0x627, 0x64a,
0x631, 0x628, 0x631, 0x627, 0x632, 0x627, 0x648, 0x64a, 0x644, 0x628, 0x62c, 0x648,
0x6ab, 0x648, 0x631, 0x627, 0x642, 0x627, 0x647, 0x631, 0x647, 0x6a9, 0x627, 0x633,
0x627, 0x628, 0x644, 0x627, 0x646, 0x6a9, 0x627, 0x633, 0x64a, 0x648, 0x67c, 0x627,
0x6a9, 0x648, 0x646, 0x6a9, 0x631, 0x64a, 0x689, 0x627, 0x6a9, 0x627, 0x631, 0x62c,
0x628, 0x648, 0x62a, 0x64a, 0x627, 0x644, 0x6cc, 0x648, 0x646, 0x641, 0x631, 0x6cc,
0x67c, 0x648, 0x646, 0x6ab, 0x627, 0x628, 0x631, 0x648, 0x646, 0x647, 0x631, 0x627,
0x631, 0x6d0, 0x62c, 0x648, 0x647, 0x627, 0x646, 0x633, 0x628, 0x631, 0x6ab, 0x6a9,
0x645, 0x67e, 0x627, 0x644, 0x647, 0x62e, 0x631, 0x62a, 0x648, 0x645, 0x6a9, 0x64a,
0x6af, 0x627, 0x644, 0x64a, 0x6a9, 0x64a, 0x646, 0x634, 0x627, 0x633, 0x627, 0x644,
0x6cc, 0x628, 0x631, 0x6cc, 0x644, 0x644, 0x648, 0x648, 0x645, 0x644, 0x648, 0x646,
0x62f, 0x647, 0x644, 0x628, 0x648, 0x628, 0x627, 0x634, 0x64a, 0x645, 0x627, 0x67e,
0x648, 0x67c, 0x648, 0x645, 0x633, 0x64a, 0x631, 0x648, 0x645, 0x628, 0x627, 0x628,
0x627, 0x646, 0x6d0, 0x645, 0x648, 0x6af, 0x62f, 0x64a, 0x634, 0x648, 0x645, 0x648,
0x646, 0x631, 0x648, 0x641, 0x6cc, 0x627, 0x646, 0x62c, 0x627, 0x645, 0x6cc, 0x646,
0x627, 0x646, 0x6cc, 0x645, 0x64a, 0x646, 0x648, 0x6a9, 0x648, 0x686, 0x67c, 0x627,
0x648, 0x6ab, 0x648, 0x689, 0x648, 0x6ab, 0x648, 0x67e, 0x648, 0x631, 0x67c, 0x648,
0x20, 0x2d, 0x20, 0x646, 0x648, 0x648, 0x633, 0x627, 0x648, 0x20, 0x67c, 0x648,
0x645, 0x648, 0x6cc, 0x646, 0x647, 0x648, 0x6a9, 0x627, 0x62f, 0x627, 0x6a9, 0x627,
0x6cc, 0x646, 0x6a9, 0x631, 0x6cc, 0x62c, 0x627, 0x646, 0x6ab, 0x6cc, 0x644, 0x627,
0x627, 0x646, 0x67c, 0x64a, 0x6ab, 0x627, 0x631, 0x6ab, 0x6cc, 0x646, 0x6cc, 0x627,
0x628, 0x64a, 0x646, 0x648, 0x633, 0x20, 0x627, 0x64a, 0x64a, 0x631, 0x632, 0x6a9,
0x67c, 0x627, 0x645, 0x627, 0x631, 0x6a9, 0x627, 0x6a9, 0x648, 0x631, 0x689, 0x648,
0x628, 0x627, 0x62c, 0x648, 0x62c, 0x648, 0x64a, 0x644, 0x627, 0x631, 0x64a, 0x648,
0x62c, 0x627, 0x645, 0x6cc, 0x646, 0x689, 0x648, 0x632, 0x627, 0x631, 0x64a, 0x648,
0x20, 0x6ab, 0x64a, 0x644, 0x64a, 0x6ab, 0x648, 0x633, 0x633, 0x627, 0x644, 0x67c,
0x627, 0x633, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x627, 0x646, 0x633, 0x627, 0x646,
0x20, 0x644, 0x648, 0x6cc, 0x633, 0x67c, 0x64a, 0x6a9, 0x648, 0x648, 0x645, 0x627,
0x648, 0x634, 0x648, 0x627, 0x6cc, 0x627, 0x622, 0x631, 0x648, 0x628, 0x627, 0x627,
0x633, 0x648, 0x646, 0x633, 0x64a, 0x648, 0x646, 0x627, 0x62a, 0x6cc, 0x6a9, 0x648,
0x6a9, 0x646, 0x628, 0x647, 0x6cc, 0x627, 0x628, 0x647, 0x6cc, 0x627, 0x20, 0x628,
0x6cc, 0x646, 0x689, 0x631, 0x627, 0x633, 0x628, 0x644, 0x6cc, 0x632, 0x628, 0x644,
0x627, 0x646, 0x6a9, 0x2d, 0x633, 0x627, 0x628, 0x644, 0x648, 0x646, 0x628, 0x648,
0x627, 0x20, 0x648, 0x6cc, 0x633, 0x67c, 0x627, 0x628, 0x648, 0x6ab, 0x67c, 0x627,
0x628, 0x648, 0x632, 0x6a9, 0x6cc, 0x645, 0x628, 0x631, 0x62c, 0x20, 0x628, 0x64a,
0x6a9, 0x645, 0x67e, 0x648, 0x20, 0x6ab, 0x631, 0x689, 0x6cc, 0x6a9, 0x64a, 0x646,
0x6a9, 0x646, 0x6a9, 0x627, 0x631, 0x627, 0x6a9, 0x627, 0x633, 0x6a9, 0x6cc, 0x6cc,
0x646, 0x6a9, 0x6cc, 0x645, 0x646, 0x634, 0x6cc, 0x6a9, 0x627, 0x6ab, 0x648, 0x686,
0x6be, 0x648, 0x627, 0x6be, 0x648, 0x627, 0x633, 0x6cc, 0x648, 0x62f, 0x627, 0x62f,
0x20, 0x62c, 0x6cc, 0x648, 0x631, 0x6cc, 0x632, 0x6a9, 0x648, 0x633, 0x62a, 0x627,
0x631, 0x6cc, 0x6a9, 0x627, 0x6a9, 0x631, 0x633, 0x67c, 0x648, 0x646, 0x6a9, 0x648,
0x6cc, 0x627, 0x628, 0x627, 0x6a9, 0x648, 0x631, 0x627, 0x6a9, 0x627, 0x624, 0x689,
0x646, 0x645, 0x627, 0x631, 0x6a9, 0x634, 0x627, 0x646, 0x62f, 0x627, 0x633, 0x646,
0x20, 0x6a9, 0x631, 0x6cc, 0x6a9, 0x689, 0x646, 0x648, 0x631, 0x689, 0x627, 0x6cc,
0x67c, 0x631, 0x648, 0x67c, 0x62f, 0x648, 0x645, 0x6cc, 0x646, 0x6cc, 0x6a9, 0x627,
0x627, 0x6cc, 0x689, 0x645, 0x648, 0x646, 0x67c, 0x646, 0x627, 0x6cc, 0x6cc, 0x631,
0x648, 0x646, 0x67e, 0x627, 0x64a, 0x644, 0x20, 0x633, 0x644, 0x648, 0x627, 0x62f,
0x648, 0x631, 0x641, 0x648, 0x631, 0x67c, 0x20, 0x646, 0x6cc, 0x644, 0x633, 0x646,
0x641, 0x648, 0x631, 0x62a, 0x6cc, 0x644, 0x632, 0x627, 0x6ab, 0x6cc, 0x633, 0x633,
0x20, 0x628, 0x64a, 0x6cc, 0x6af, 0x648, 0x632, 0x20, 0x628, 0x64a, 0x644, 0x648,
0x6cc, 0x20, 0x62a, 0x631, 0x6a9, 0x6ab, 0x631, 0x646, 0x627, 0x62f, 0x627, 0x6ab,
0x627, 0x644, 0x689, 0x6cc, 0x67e, 0x6ab, 0x648, 0x627, 0x62a, 0x645, 0x627, 0x644,
0x627, 0x6ab, 0x648, 0x64a, 0x627, 0x6a9, 0x644, 0x6ab, 0x6cc, 0x627, 0x646, 0x627,
0x647, 0x6cc, 0x644, 0x641, 0x6a9, 0x633, 0x647, 0x648, 0x627, 0x646, 0x627, 0x647,
0x631, 0x645, 0x648, 0x633, 0x6cc, 0x644, 0x648, 0x627, 0x646, 0x689, 0x64a, 0x627,
0x646, 0x627, 0x20, 0x67e, 0x648, 0x644, 0x64a, 0x633, 0x646, 0x648, 0x6a9, 0x633,
0x20, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x645, 0x648, 0x631, 0x64a,
0x646, 0x6ab, 0x648, 0x20, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x67e,
0x64a, 0x67c, 0x633, 0x628, 0x631, 0x6ab, 0x60c, 0x20, 0x627, 0x646, 0x689, 0x64a,
0x627, 0x646, 0x627, 0x67c, 0x644, 0x20, 0x633, 0x67c, 0x64a, 0x60c, 0x20, 0x627,
0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x648, 0x6cc, 0x60c, 0x20,
0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x648, 0x64a, 0x646, 0x633, 0x64a,
0x646, 0x633, 0x60c, 0x20, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x648,
0x64a, 0x646, 0x627, 0x645, 0x6a9, 0x20, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646,
0x627, 0x627, 0x646, 0x648, 0x6a9, 0x627, 0x642, 0x644, 0x6cc, 0x67c, 0x62c, 0x645,
0x627, 0x6cc, 0x6a9, 0x647, 0x62c, 0x648, 0x646, 0x648, 0x644, 0x648, 0x626, 0x633,
0x20, 0x648, 0x6cc, 0x644, 0x645, 0x648, 0x646, 0x67c, 0x64a, 0x633, 0x64a, 0x644,
0x648, 0x60c, 0x20, 0x6a9, 0x648, 0x646, 0x67c, 0x6a9, 0x6cc, 0x6a9, 0x644, 0x6cc,
0x646, 0x689, 0x6cc, 0x632, 0x62c, 0x644, 0x6cc, 0x645, 0x627, 0x644, 0x627, 0x633,
0x20, 0x627, 0x6cc, 0x646, 0x62c, 0x644, 0x633, 0x644, 0x648, 0x648, 0x631, 0x20,
0x67e, 0x631, 0x646, 0x633, 0x20, 0x6a9, 0x648, 0x627, 0x631, 0x67c, 0x631, 0x645,
0x627, 0x633, 0x64a, 0x648, 0x645, 0x646, 0x6ab, 0x648, 0x627, 0x645, 0x646, 0x627,
0x648, 0x633, 0x645, 0x64a, 0x631, 0x64a, 0x6ab, 0x627, 0x67c, 0x645, 0x627, 0x631,
0x67c, 0x6cc, 0x646, 0x6cc, 0x6a9, 0x645, 0x64a, 0x67c, 0x627, 0x645, 0x648, 0x631,
0x648, 0x633, 0x645, 0x632, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x645, 0x6cc, 0x646,
0x648, 0x645, 0x6cc, 0x646, 0x645, 0x64a, 0x631, 0x64a, 0x689, 0x627, 0x645, 0x6cc,
0x62a, 0x644, 0x627, 0x6a9, 0x627, 0x67c, 0x644, 0x627, 0x645, 0x6a9, 0x633, 0x64a,
0x6a9, 0x648, 0x20, 0x69a, 0x627, 0x631, 0x645, 0x64a, 0x6a9, 0x648, 0x64a, 0x644,
0x627, 0x646, 0x645, 0x648, 0x646, 0x6a9, 0x67c, 0x648, 0x646, 0x645, 0x646, 0x67c,
0x631, 0x631, 0x64a, 0x645, 0x648, 0x646, 0x67c, 0x20, 0x648, 0x689, 0x6cc, 0x648,
0x645, 0x627, 0x646, 0x67c, 0x6cc, 0x633, 0x6cc, 0x631, 0x62a, 0x646, 0x6cc, 0x633,
0x627, 0x648, 0x646, 0x6cc, 0x648, 0x6cc, 0x627, 0x631, 0x6a9, 0x646, 0x648, 0x631,
0x648, 0x646, 0x647, 0x627, 0x628, 0x64a, 0x648, 0x644, 0x627, 0x60c, 0x20, 0x634,
0x645, 0x627, 0x644, 0x64a, 0x20, 0x689, 0x627, 0x6a9, 0x648, 0x67c, 0x627, 0x645,
0x631, 0x6a9, 0x632, 0x60c, 0x20, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x20, 0x689,
0x627, 0x6a9, 0x648, 0x67c, 0x627, 0x646, 0x648, 0x6cc, 0x20, 0x633, 0x644, 0x6cc,
0x645, 0x60c, 0x20, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x20, 0x62f, 0x627, 0x6a9,
0x648, 0x62a, 0x627, 0x646, 0x648, 0x648, 0x6a9, 0x627, 0x648, 0x62c, 0x646, 0x6ab,
0x627, 0x67e, 0x627, 0x631, 0x627, 0x645, 0x627, 0x631, 0x628, 0x648, 0x641, 0x6cc,
0x646, 0x6a9, 0x633, 0x67e, 0x648, 0x631, 0x67c, 0x20, 0x627, 0x6cc, 0x648, 0x20,
0x2d, 0x20, 0x67e, 0x631, 0x646, 0x633, 0x62f, 0x20, 0x627, 0x633, 0x67e, 0x627,
0x646, 0x6cc, 0x627, 0x20, 0x628, 0x646, 0x62f, 0x631, 0x67e, 0x648, 0x631, 0x67c,
0x648, 0x20, 0x648, 0x6cc, 0x644, 0x647, 0x648, 0x67e, 0x648, 0x631, 0x62a, 0x648,
0x20, 0x631, 0x6cc, 0x6a9, 0x648, 0x67e, 0x646, 0x67c, 0x627, 0x20, 0x622, 0x631,
0x6cc, 0x646, 0x627, 0x633, 0x631, 0x64a, 0x646, 0x6a9, 0x646, 0x20, 0x627, 0x646,
0x644, 0x67c, 0x631, 0x6cc, 0x633, 0x6cc, 0x641, 0x64a, 0x631, 0x6cc, 0x6ab, 0x6cc,
0x646, 0x627, 0x631, 0x64a, 0x633, 0x627, 0x644, 0x648, 0x67c, 0x631, 0x6cc, 0x648,
0x20, 0x628, 0x631, 0x627, 0x646, 0x6a9, 0x648, 0x633, 0x646, 0x627, 0x62a, 0x631,
0x645, 0x633, 0x646, 0x62a, 0x64a, 0x627, 0x6ab, 0x648, 0x633, 0x646, 0x62a, 0x648,
0x20, 0x689, 0x648, 0x645, 0x6cc, 0x646, 0x6ab, 0x648, 0x633, 0x627, 0x648, 0x20,
0x67e, 0x627, 0x648, 0x644, 0x648, 0x627, 0x67c, 0x648, 0x6a9, 0x648, 0x67c, 0x648,
0x631, 0x645, 0x6cc, 0x67c, 0x633, 0x6cc, 0x67c, 0x6a9, 0x627, 0x633, 0x6cc, 0x646,
0x67c, 0x20, 0x628, 0x627, 0x631, 0x67c, 0x6cc, 0x644, 0x6cc, 0x645, 0x633, 0x6cc,
0x646, 0x67c, 0x20, 0x62c, 0x627, 0x646, 0x632, 0x633, 0x6cc, 0x646, 0x67c, 0x20,
0x6a9, 0x67c, 0x633, 0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x644, 0x648, 0x633, 0x6cc,
0x627, 0x633, 0x64a, 0x646, 0x67c, 0x20, 0x62a, 0x647, 0x627, 0x645, 0x633, 0x633,
0x6d0, 0x646, 0x67c, 0x20, 0x648, 0x6cc, 0x633, 0x646, 0x67c, 0x633, 0x648, 0x64a,
0x641, 0x67c, 0x20, 0x6a9, 0x631, 0x646, 0x67c, 0x67c, 0x6ab, 0x633, 0x6cc, 0x6ab,
0x627, 0x644, 0x67e, 0x627, 0x62a, 0x62c, 0x648, 0x627, 0x646, 0x627, 0x67c, 0x648,
0x631, 0x646, 0x67c, 0x648, 0x67c, 0x648, 0x631, 0x62a, 0x648, 0x644, 0x627, 0x648,
0x6cc, 0x646, 0x6a9, 0x648, 0x648, 0x631, 0x648, 0x627, 0x64a, 0x67c, 0x20, 0x647,
0x627, 0x631, 0x633, 0x648, 0x6cc, 0x646, 0x67e, 0x6cc, 0x6ab, 0x64a, 0x627, 0x6a9,
0x648, 0x67c, 0x67c, 0x6a9, 0x6cc, 0x633, 0x64a, 0x689, 0x64a, 0x648, 0x633, 0x689,
0x648, 0x645, 0x648, 0x646, 0x67c, 0x20, 0x689, 0x64a, 0x20, 0x627, 0x631, 0x648,
0x644, 0x645, 0x6a9, 0x648, 0x627, 0x631, 0x64a, 0x645, 0x627, 0x648, 0x633, 0x646,
0x645, 0x6a9, 0x645, 0x631, 0x689, 0x648, 0x67e, 0x627, 0x644, 0x645, 0x631, 0x631,
0x648, 0x62f, 0x631, 0x627, 0x633, 0x6cc, 0x648, 0x627, 0x67c, 0x631, 0x648, 0x644,
0x648, 0x627, 0x633, 0x62a, 0x648, 0x6a9, 0x644, 0x627, 0x646, 0x6ab, 0x64a, 0x631,
0x628, 0x64a, 0x646, 0x627, 0x689, 0x646, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x6cc,
0x627, 0x646, 0x627, 0x689, 0x64a, 0x631, 0x627, 0x6a9, 0x67c, 0x627, 0x648, 0x627,
0x6a9, 0x62a, 0x648, 0x628, 0x627, 0x634, 0x63a, 0x20, 0x622, 0x628, 0x627, 0x62f,
0x627, 0x6d0, 0x67c, 0x631, 0x627, 0x648, 0x628, 0x62d, 0x631, 0x6cc, 0x646, 0x628,
0x646, 0x6a9, 0x627, 0x6a9, 0x628, 0x6cc, 0x631, 0x648, 0x62a, 0x628, 0x634, 0x6a9,
0x6cc, 0x6a9, 0x628, 0x631, 0x648, 0x6cc, 0x646, 0x64a, 0x6a9, 0x648, 0x644, 0x645,
0x628, 0x648, 0x689, 0x647, 0x627, 0x6a9, 0x647, 0x62f, 0x644, 0x64a, 0x62f, 0x648,
0x628, 0x6cc, 0x62f, 0x648, 0x634, 0x646, 0x628, 0x64a, 0x641, 0x627, 0x645, 0x627,
0x6ab, 0x633, 0x62a, 0x627, 0x63a, 0x632, 0x647, 0x647, 0x628, 0x631, 0x648, 0x646,
0x647, 0x648, 0x20, 0x686, 0x64a, 0x20, 0x645, 0x646, 0x20, 0x69a, 0x627, 0x631,
0x647, 0x627, 0x646, 0x6ab, 0x20, 0x6a9, 0x627, 0x646, 0x6ab, 0x647, 0x627, 0x648,
0x689, 0x627, 0x631, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x62c, 0x627, 0x64a, 0x627,
0x67e, 0x648, 0x631, 0x627, 0x64a, 0x631, 0x648, 0x634, 0x644, 0x645, 0x6a9, 0x627,
0x628, 0x644, 0x6a9, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x6a9, 0x627, 0x6a9, 0x631,
0x627, 0x686, 0x64a, 0x6a9, 0x67c, 0x645, 0x646, 0x689, 0x648, 0x62e, 0x646, 0x62f,
0x64a, 0x6ab, 0x627, 0x6a9, 0x648, 0x644, 0x6a9, 0x62a, 0x647, 0x6a9, 0x631, 0x64a,
0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x6a9, 0x6a9, 0x648, 0x644, 0x627,
0x644, 0x645, 0x67e, 0x648, 0x631, 0x6a9, 0x648, 0x686, 0x646, 0x6ab, 0x6a9, 0x648,
0x6d0, 0x62a, 0x645, 0x6a9, 0x627, 0x648, 0x645, 0x6ab, 0x62f, 0x627, 0x646, 0x645,
0x6a9, 0x627, 0x633, 0x627, 0x631, 0x645, 0x646, 0x6cc, 0x644, 0x627, 0x646, 0x6cc,
0x6a9, 0x648, 0x633, 0x6cc, 0x627, 0x646, 0x648, 0x648, 0x6a9, 0x648, 0x632, 0x646,
0x6cc, 0x67c, 0x6a9, 0x646, 0x648, 0x648, 0x633, 0x64a, 0x628, 0x631, 0x633, 0x6a9,
0x627, 0x648, 0x631, 0x644, 0x67e, 0x646, 0x648, 0x645, 0x20, 0x67e, 0x646, 0x67e,
0x648, 0x646, 0x67c, 0x6cc, 0x646, 0x6cc, 0x6a9, 0x67e, 0x64a, 0x627, 0x646, 0x6ab,
0x20, 0x64a, 0x627, 0x646, 0x6ab, 0x6a9, 0x648, 0x633, 0x62a, 0x627, 0x646, 0x6d0,
0x642, 0x64a, 0x632, 0x64a, 0x20, 0x644, 0x648, 0x631, 0x689, 0x627, 0x633, 0x62e,
0x627, 0x644, 0x64a, 0x646, 0x634, 0x646, 0x6af, 0x6be, 0x627, 0x626, 0x6cc, 0x633,
0x6cc, 0x646, 0x6ab, 0x627, 0x67e, 0x648, 0x631, 0x633, 0x631, 0x64a, 0x689, 0x646,
0x64a, 0x6a9, 0x648, 0x644, 0x64a, 0x645, 0x633, 0x6a9, 0x62a, 0x627, 0x64a, 0x67e,
0x64a, 0x62a, 0x627, 0x634, 0x642, 0x646, 0x62f, 0x62a, 0x628, 0x644, 0x6cc, 0x633,
0x64a, 0x62a, 0x647, 0x631, 0x627, 0x646, 0x62a, 0x647, 0x64a, 0x645, 0x641, 0x648,
0x67c, 0x648, 0x6a9, 0x6cc, 0x648, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628,
0x627, 0x67c, 0x631, 0x627, 0x648, 0x631, 0x648, 0x645, 0x642, 0x64a, 0x627, 0x648,
0x633, 0x62a, 0x646, 0x64a, 0x631, 0x627, 0x648, 0x64a, 0x646, 0x67c, 0x64a, 0x646,
0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x648, 0x633, 0x62a, 0x627, 0x6a9, 0x64a, 0x627,
0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x6cc, 0x627, 0x646, 0x6ab, 0x648, 0x646, 0x64a,
0x6a9, 0x627, 0x67c, 0x64a, 0x631, 0x646, 0x628, 0x631, 0x6ab, 0x64a, 0x631, 0x64a,
0x648, 0x627, 0x646, 0x627, 0x64a, 0x632, 0x648, 0x631, 0x633, 0x6a9, 0x646, 0x627,
0x631, 0x64a, 0x6a9, 0x64a, 0x67e, 0x20, 0x648, 0x631, 0x689, 0x645, 0x62f, 0x64a,
0x631, 0x627, 0x631, 0x64a, 0x6a9, 0x62c, 0x627, 0x648, 0x64a, 0x6a9, 0x633, 0x648,
0x64a, 0x644, 0x64a, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x633, 0x6cc,
0x646, 0x67c, 0x20, 0x647, 0x6cc, 0x644, 0x6cc, 0x646, 0x627, 0x633, 0x67c, 0x646,
0x644, 0x64a, 0x627, 0x689, 0x6cc, 0x644, 0x627, 0x6cc, 0x689, 0x628, 0x631, 0x6cc,
0x633, 0x628, 0x646, 0x628, 0x631, 0x648, 0x6a9, 0x646, 0x20, 0x647, 0x644, 0x689,
0x627, 0x631, 0x648, 0x646, 0x627, 0x64a, 0x648, 0x6a9, 0x644, 0x627, 0x647, 0x648,
0x628, 0x627, 0x631, 0x67c, 0x644, 0x6cc, 0x646, 0x689, 0x627, 0x645, 0x6cc, 0x646,
0x644, 0x627, 0x631, 0x689, 0x20, 0x647, 0x648, 0x64a, 0x645, 0x6cc, 0x644, 0x628,
0x648, 0x631, 0x646, 0x67e, 0x631, 0x62a, 0x633, 0x689, 0x646, 0x64a, 0x627, 0x645,
0x633, 0x62a, 0x631, 0x62f, 0x627, 0x645, 0x627, 0x646, 0x62f, 0x648, 0x631, 0x627,
0x627, 0x633, 0x62a, 0x631, 0x627, 0x20, 0x62e, 0x627, 0x646, 0x627, 0x64a, 0x62a,
0x646, 0x632, 0x628, 0x644, 0x63a, 0x627, 0x62f, 0x628, 0x631, 0x644, 0x646, 0x628,
0x631, 0x627, 0x67c, 0x633, 0x644, 0x627, 0x648, 0x627, 0x628, 0x631, 0x648, 0x633,
0x644, 0x632, 0x628, 0x62e, 0x627, 0x631, 0x633, 0x62a, 0x628, 0x62f, 0x627, 0x67e,
0x633, 0x67c, 0x628, 0x648, 0x633, 0x64a, 0x646, 0x62c, 0x646, 0x686, 0x6cc, 0x633,
0x6cc, 0x646, 0x648, 0x6a9, 0x648, 0x67e, 0x646, 0x20, 0x647, 0x64a, 0x6ab, 0x646,
0x689, 0x628, 0x644, 0x646, 0x62c, 0x628, 0x644, 0x20, 0x627, 0x644, 0x637, 0x627,
0x631, 0x642, 0x6ab, 0x631, 0x646, 0x633, 0x64a, 0x647, 0x6cc, 0x644, 0x633, 0x646,
0x6a9, 0x6cc, 0x62f, 0x20, 0x622, 0x626, 0x644, 0x20, 0x622, 0x641, 0x20, 0x645,
0x6cc, 0x646, 0x627, 0x633, 0x62a, 0x646, 0x628, 0x648, 0x644, 0x6a9, 0x64a, 0x644,
0x646, 0x64a, 0x646, 0x6ab, 0x631, 0x627, 0x689, 0x6a9, 0x6cc, 0x631, 0x648, 0x641,
0x6a9, 0x6cc, 0x641, 0x644, 0x6cc, 0x633, 0x628, 0x648, 0x646, 0x644, 0x648, 0x628,
0x62c, 0x627, 0x646, 0x627, 0x644, 0x648, 0x6ab, 0x632, 0x627, 0x645, 0x628, 0x648,
0x631, 0x6ab, 0x645, 0x6cc, 0x689, 0x631, 0x6cc, 0x689, 0x645, 0x627, 0x644, 0x67c,
0x627, 0x645, 0x64a, 0x631, 0x64a, 0x647, 0x627, 0x645, 0x645, 0x646, 0x633, 0x6a9,
0x67e, 0x627, 0x631, 0x6cc, 0x633, 0x67e, 0x648, 0x689, 0x6ab, 0x648, 0x631, 0x64a,
0x6a9, 0x627, 0x67e, 0x631, 0x627, 0x6ab, 0x633, 0x627, 0x646, 0x20, 0x645, 0x627,
0x631, 0x6cc, 0x646, 0x648, 0x633, 0x64a, 0x631, 0x627, 0x62c, 0x64a, 0x648, 0x627,
0x633, 0x631, 0x627, 0x62a, 0x641, 0x633, 0x6cc, 0x645, 0x641, 0x631, 0x648, 0x67e,
0x648, 0x644, 0x633, 0x6a9, 0x67e, 0x648, 0x6ab, 0x635, 0x648, 0x641, 0x6cc, 0x627,
0x627, 0x633, 0x62a, 0x648, 0x644, 0x648, 0x645, 0x627, 0x644, 0x64a, 0x627, 0x646,
0x648, 0x633, 0x6a9, 0x648, 0x627, 0x689, 0x648, 0x632, 0x648, 0x6cc, 0x67c, 0x6cc,
0x6a9, 0x627, 0x646, 0x648, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x64a, 0x644, 0x646,
0x64a, 0x648, 0x633, 0x648, 0x627, 0x644, 0x6ab, 0x648, 0x6ab, 0x631, 0x627, 0x689,
0x632, 0x6ab, 0x631, 0x628, 0x632, 0x631, 0x6cc, 0x686, 0x627, 0x646, 0x62a, 0x627,
0x646, 0x646, 0x627, 0x631, 0x64a, 0x648, 0x648, 0x686, 0x627, 0x6ab, 0x648, 0x633,
0x6a9, 0x631, 0x64a, 0x633, 0x645, 0x633, 0x6a9, 0x648, 0x6a9, 0x648, 0x632, 0x6a9,
0x631, 0x6ab, 0x648, 0x644, 0x64a, 0x646, 0x645, 0x627, 0x644, 0x62f, 0x64a, 0x67e,
0x645, 0x627, 0x631, 0x64a, 0x634, 0x64a, 0x633, 0x645, 0x6cc, 0x67c, 0x648, 0x62a,
0x631, 0x64a, 0x20, 0x64a, 0x648, 0x646, 0x64a, 0x646, 0x627, 0x67e, 0x6cc, 0x627,
0x627, 0x6a9, 0x644, 0x646, 0x62f, 0x628, 0x648, 0x6ab, 0x646, 0x20, 0x648, 0x6cc,
0x644, 0x686, 0x627, 0x62a, 0x627, 0x645, 0x686, 0x648, 0x6a9, 0x627, 0x6cc, 0x633,
0x67c, 0x631, 0x639, 0x641, 0x627, 0x62a, 0x641, 0x648, 0x6a9, 0x627, 0x641, 0x648,
0x641, 0x62c, 0x64a, 0x6ab, 0x627, 0x644, 0x67e, 0x6ab, 0x648, 0x633, 0x6ab, 0x64a,
0x645, 0x628, 0x64a, 0x631, 0x6ab, 0x648, 0x627, 0x689, 0x644, 0x6a9, 0x64a, 0x646,
0x627, 0x644, 0x647, 0x6cc, 0x646, 0x648, 0x644, 0x648, 0x644, 0x648, 0x627, 0x646,
0x689, 0x631, 0x628, 0x631, 0x64a, 0x6a9, 0x64a, 0x631, 0x64a, 0x67c, 0x645, 0x627,
0x67c, 0x64a, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x64a, 0x6a9, 0x648, 0x627, 0x62c,
0x644, 0x64a, 0x646, 0x645, 0x62c, 0x648, 0x631, 0x648, 0x645, 0x627, 0x631, 0x6a9,
0x64a, 0x633, 0x627, 0x633, 0x645, 0x64a, 0x689, 0x648, 0x6cc, 0x646, 0x627, 0x6cc,
0x631, 0x648, 0x646, 0x6cc, 0x648, 0x648, 0x646, 0x648, 0x631, 0x641, 0x6a9, 0x67e,
0x64a, 0x6ab, 0x648, 0x20, 0x67e, 0x64a, 0x6ab, 0x648, 0x67e, 0x644, 0x627, 0x648,
0x67e, 0x64a, 0x67c, 0x64a, 0x6a9, 0x64a, 0x631, 0x646, 0x67e, 0x648, 0x646, 0x67e,
0x64a, 0x67e, 0x648, 0x631, 0x67c, 0x20, 0x645, 0x648, 0x631, 0x633, 0x628, 0x6cc,
0x631, 0x627, 0x631, 0x648, 0x67c, 0x648, 0x646, 0x6ab, 0x627, 0x633, 0x64a, 0x67e,
0x627, 0x646, 0x67c, 0x647, 0x64a, 0x67c, 0x64a, 0x67c, 0x648, 0x646, 0x6ab, 0x627,
0x62a, 0x627, 0x67e, 0x648, 0x648, 0x627, 0x644, 0x633, 0x647, 0x631, 0x627, 0x631,
0x6d2, 0x62f, 0x20, 0x6a9, 0x645, 0x62a, 0x631, 0x20, 0x634, 0x647, 0x632, 0x627,
0x62f, 0x647, 0x20, 0x62f, 0x631, 0x6d2, 0x20, 0x645, 0x6cc, 0x627, 0x634, 0x62a,
0x646, 0x6cd, 0x622, 0x628, 0x6cc, 0x62c, 0x627, 0x646, 0x622, 0x62f, 0x6cc, 0x633,
0x20, 0x622, 0x628, 0x627, 0x628, 0x627, 0x627, 0x644, 0x62c, 0x632, 0x6cc, 0x631,
0x647, 0x627, 0x633, 0x645, 0x631, 0x647, 0x628, 0x627, 0x645, 0x627, 0x6a9, 0x648,
0x628, 0x627, 0x646, 0x6af, 0x6cc, 0x628, 0x6cc, 0x633, 0x627, 0x626, 0x648, 0x628,
0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x631, 0x647, 0x628, 0x631, 0x627, 0x632, 0x648,
0x6cc, 0x644, 0x6a9, 0x627, 0x632, 0x627, 0x628, 0x644, 0x627, 0x646, 0x6a9, 0x627,
0x633, 0x628, 0x62a, 0x647, 0x62f, 0x627, 0x6a9, 0x627, 0x631, 0x62c, 0x6cc, 0x628,
0x648, 0x62a, 0x6cc, 0x62f, 0x648, 0x622, 0x644, 0x627, 0x627, 0x644, 0x639, 0x6cc,
0x648, 0x646, 0x641, 0x631, 0x6cc, 0x200c, 0x62a, 0x627, 0x648, 0x646, 0x647, 0x631,
0x627, 0x631, 0x647, 0x698, 0x648, 0x647, 0x627, 0x646, 0x633, 0x628, 0x648, 0x631,
0x6af, 0x6a9, 0x627, 0x645, 0x67e, 0x627, 0x644, 0x627, 0x62e, 0x627, 0x631, 0x637,
0x648, 0x645, 0x6a9, 0x6cc, 0x6af, 0x627, 0x644, 0x6cc, 0x6a9, 0x6cc, 0x646, 0x634,
0x627, 0x633, 0x627, 0x644, 0x6cc, 0x628, 0x631, 0x648, 0x6cc, 0x644, 0x644, 0x648,
0x645, 0x647, 0x644, 0x648, 0x628, 0x648, 0x645, 0x628, 0x627, 0x634, 0x6cc, 0x644,
0x648, 0x632, 0x627, 0x6a9, 0x627, 0x645, 0x627, 0x67e, 0x648, 0x62a, 0x648, 0x645,
0x627, 0x633, 0x631, 0x648, 0x645, 0x628, 0x627, 0x628, 0x627, 0x646, 0x647, 0x645,
0x648, 0x6af, 0x627, 0x62f, 0x6cc, 0x634, 0x648, 0x645, 0x648, 0x646, 0x631, 0x648,
0x648, 0x6cc, 0x627, 0x646, 0x627, 0x6cc, 0x631, 0x648, 0x628, 0x6cc, 0x627, 0x646,
0x62c, 0x627, 0x645, 0x646, 0x627, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x646, 0x648,
0x622, 0x6a9, 0x634, 0x648, 0x62a, 0x627, 0x648, 0x622, 0x6af, 0x627, 0x62f, 0x648,
0x6af, 0x648, 0x67e, 0x648, 0x631, 0x62a, 0x648, 0x20, 0x646, 0x648, 0x648, 0x648,
0x633, 0x627, 0x626, 0x648, 0x62a, 0x648, 0x645, 0x647, 0x648, 0x6cc, 0x646, 0x62f,
0x647, 0x648, 0x6a9, 0x627, 0x6cc, 0x62f, 0x6a9, 0x627, 0x646, 0x6a9, 0x648, 0x631,
0x6cc, 0x62c, 0x622, 0x646, 0x6af, 0x648, 0x626, 0x6cc, 0x644, 0x627, 0x622, 0x646,
0x62a, 0x6cc, 0x6af, 0x648, 0x627, 0x622, 0x631, 0x627, 0x6af, 0x648, 0x627, 0x6cc,
0x646, 0x627, 0x628, 0x648, 0x626, 0x646, 0x648, 0x633, 0x200c, 0x622, 0x6cc, 0x631,
0x633, 0x6a9, 0x627, 0x62a, 0x627, 0x645, 0x627, 0x631, 0x6a9, 0x627, 0x6a9, 0x648,
0x631, 0x62f, 0x648, 0x648, 0x627, 0x62e, 0x648, 0x62e, 0x648, 0x6cc, 0x644, 0x627,
0x631, 0x6cc, 0x648, 0x62e, 0x627, 0x645, 0x646, 0x62f, 0x648, 0x633, 0x627, 0x631,
0x6cc, 0x648, 0x6af, 0x627, 0x644, 0x6af, 0x648, 0x633, 0x633, 0x646, 0x200c, 0x62e,
0x648, 0x627, 0x646, 0x633, 0x646, 0x200c, 0x644, 0x648, 0x626, 0x6cc, 0x633, 0x62a,
0x648, 0x6a9, 0x648, 0x645, 0x646, 0x627, 0x631, 0x648, 0x628, 0x627, 0x622, 0x633,
0x648, 0x646, 0x633, 0x6cc, 0x648, 0x646, 0x627, 0x62a, 0x6a9, 0x648, 0x6a9, 0x627,
0x646, 0x628, 0x627, 0x6cc, 0x627, 0x628, 0x627, 0x647, 0x6cc, 0x627, 0x20, 0x628,
0x627, 0x646, 0x62f, 0x631, 0x627, 0x633, 0x628, 0x644, 0x627, 0x646, 0x2010, 0x633,
0x627, 0x628, 0x644, 0x648, 0x646, 0x628, 0x648, 0x626, 0x627, 0x648, 0x6cc, 0x633,
0x62a, 0x627, 0x628, 0x648, 0x6cc, 0x633, 0x6cc, 0x6a9, 0x645, 0x628, 0x631, 0x6cc,
0x62c, 0x200c, 0x628, 0x6cc, 0x6a9, 0x645, 0x67e, 0x648, 0x20, 0x6af, 0x631, 0x627,
0x646, 0x62f, 0x647, 0x6a9, 0x627, 0x646, 0x6a9, 0x648, 0x646, 0x634, 0x6cc, 0x6a9,
0x627, 0x6af, 0x648, 0x686, 0x6cc, 0x648, 0x627, 0x648, 0x627, 0x633, 0x6cc, 0x648,
0x62f, 0x627, 0x62f, 0x20, 0x62e, 0x648, 0x627, 0x631, 0x632, 0x6a9, 0x627, 0x633,
0x62a, 0x627, 0x631, 0x6cc, 0x6a9, 0x627, 0x6a9, 0x631, 0x633, 0x62a, 0x648, 0x646,
0x6a9, 0x648, 0x6cc, 0x627, 0x648, 0x627, 0x6a9, 0x648, 0x631, 0x627, 0x633, 0x627,
0x626, 0x648, 0x62f, 0x627, 0x646, 0x645, 0x627, 0x631, 0x6a9, 0x633, 0x200c, 0x647,
0x627, 0x648, 0x646, 0x62f, 0x627, 0x648, 0x633, 0x646, 0x20, 0x6a9, 0x631, 0x6cc,
0x6a9, 0x62f, 0x646, 0x648, 0x631, 0x62f, 0x6cc, 0x62a, 0x631, 0x648, 0x6cc, 0x62a,
0x627, 0x62f, 0x645, 0x648, 0x646, 0x62a, 0x648, 0x646, 0x627, 0x6cc, 0x631, 0x648,
0x646, 0x67e, 0x647, 0x627, 0x644, 0x633, 0x627, 0x644, 0x648, 0x627, 0x62f, 0x648,
0x631, 0x641, 0x648, 0x631, 0x62a, 0x20, 0x646, 0x644, 0x633, 0x648, 0x646, 0x641,
0x648, 0x631, 0x62a, 0x627, 0x644, 0x632, 0x627, 0x6af, 0x644, 0x6cc, 0x633, 0x200c,
0x628, 0x6cc, 0x6af, 0x648, 0x633, 0x200c, 0x628, 0x6cc, 0x6af, 0x631, 0x627, 0x646,
0x62f, 0x20, 0x62a, 0x648, 0x631, 0x6a9, 0x6af, 0x631, 0x646, 0x627, 0x62f, 0x627,
0x6af, 0x648, 0x627, 0x62f, 0x644, 0x648, 0x67e, 0x6af, 0x648, 0x627, 0x62a, 0x645,
0x627, 0x644, 0x627, 0x6af, 0x648, 0x627, 0x6cc, 0x627, 0x6a9, 0x6cc, 0x644, 0x6af,
0x648, 0x6cc, 0x627, 0x646, 0x647, 0x644, 0x6cc, 0x641, 0x6a9, 0x633, 0x647, 0x627,
0x648, 0x627, 0x646, 0x627, 0x627, 0x631, 0x645, 0x648, 0x633, 0x6cc, 0x648, 0x627,
0x6cc, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627, 0x67e, 0x648, 0x644, 0x6cc, 0x633,
0x646, 0x627, 0x6a9, 0x633, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc, 0x627,
0x646, 0x627, 0x645, 0x627, 0x631, 0x646, 0x6af, 0x648, 0x60c, 0x20, 0x627, 0x6cc,
0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627, 0x67e, 0x6cc, 0x62a, 0x631, 0x632, 0x628,
0x631, 0x6af, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627,
0x62a, 0x644, 0x200c, 0x633, 0x6cc, 0x62a, 0x6cc, 0x60c, 0x20, 0x627, 0x6cc, 0x646,
0x62f, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x648, 0x6cc, 0x60c, 0x20, 0x627,
0x6cc, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x646, 0x633, 0x646,
0x633, 0x60c, 0x20, 0x627, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc,
0x646, 0x627, 0x645, 0x627, 0x6a9, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc,
0x627, 0x646, 0x627, 0x627, 0x6cc, 0x646, 0x648, 0x648, 0x6cc, 0x6a9, 0x627, 0x6cc,
0x6a9, 0x644, 0x648, 0x626, 0x62a, 0x62c, 0x627, 0x645, 0x627, 0x626, 0x6cc, 0x6a9,
0x627, 0x644, 0x648, 0x626, 0x6cc, 0x632, 0x648, 0x6cc, 0x644, 0x645, 0x627, 0x646,
0x62a, 0x6cc, 0x633, 0x644, 0x648, 0x60c, 0x20, 0x6a9, 0x646, 0x62a, 0x627, 0x6a9,
0x6cc, 0x6a9, 0x631, 0x627, 0x644, 0x646, 0x62f, 0x6cc, 0x6a9, 0x644, 0x627, 0x67e,
0x627, 0x632, 0x644, 0x648, 0x633, 0x200c, 0x622, 0x646, 0x62c, 0x644, 0x633, 0x628,
0x62e, 0x634, 0x20, 0x634, 0x627, 0x647, 0x632, 0x627, 0x62f, 0x647, 0x200c, 0x200c,
0x646, 0x634, 0x6cc, 0x646, 0x20, 0x67e, 0x627, 0x6cc, 0x6cc, 0x646, 0x645, 0x627,
0x633, 0x6cc, 0x648, 0x645, 0x627, 0x646, 0x627, 0x6af, 0x648, 0x627, 0x645, 0x627,
0x631, 0x6cc, 0x6af, 0x627, 0x62a, 0x645, 0x627, 0x631, 0x62a, 0x6cc, 0x646, 0x6cc,
0x6a9, 0x645, 0x646, 0x627, 0x645, 0x6cc, 0x646, 0x6cc, 0x645, 0x631, 0x6cc, 0x62f,
0x627, 0x645, 0x62a, 0x627, 0x644, 0x627, 0x6a9, 0x627, 0x62a, 0x644, 0x627, 0x645,
0x6a9, 0x632, 0x6cc, 0x6a9, 0x648, 0x633, 0x6cc, 0x62a, 0x6cc, 0x645, 0x6cc, 0x6a9,
0x644, 0x648, 0x646, 0x645, 0x627, 0x646, 0x6a9, 0x62a, 0x648, 0x646, 0x645, 0x648,
0x646, 0x62a, 0x631, 0x6cc, 0x645, 0x648, 0x646, 0x62a, 0x647, 0x200c, 0x648, 0x6cc,
0x62f, 0x626, 0x648, 0x645, 0x648, 0x646, 0x62a, 0x633, 0x631, 0x627, 0x62a, 0x646,
0x627, 0x633, 0x627, 0x626, 0x648, 0x646, 0x6cc, 0x648, 0x6cc, 0x648, 0x631, 0x6a9,
0x646, 0x648, 0x631, 0x648, 0x646, 0x6cc, 0x627, 0x628, 0x6cc, 0x648, 0x644, 0x627,
0x60c, 0x20, 0x62f, 0x627, 0x6a9, 0x648, 0x62a, 0x627, 0x6cc, 0x20, 0x634, 0x645,
0x627, 0x644, 0x6cc, 0x633, 0x646, 0x62a, 0x631, 0x60c, 0x20, 0x62f, 0x627, 0x6a9,
0x648, 0x62a, 0x627, 0x6cc, 0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc, 0x646, 0x6cc,
0x648, 0x633, 0x627, 0x644, 0x645, 0x60c, 0x20, 0x62f, 0x627, 0x6a9, 0x648, 0x62a,
0x627, 0x6cc, 0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc, 0x627, 0x648, 0x62c, 0x6cc,
0x646, 0x627, 0x6af, 0x627, 0x67e, 0x627, 0x631, 0x627, 0x645, 0x627, 0x631, 0x6cc,
0x628, 0x648, 0x67e, 0x648, 0x631, 0x62a, 0x648, 0x67e, 0x631, 0x646, 0x633, 0x67e,
0x648, 0x631, 0x62a, 0x200c, 0x622, 0x648, 0x627, 0x633, 0x67e, 0x6cc, 0x646, 0x67e,
0x648, 0x631, 0x62a, 0x648, 0x648, 0x644, 0x6cc, 0x648, 0x67e, 0x648, 0x631, 0x62a,
0x648, 0x631, 0x6cc, 0x6a9, 0x648, 0x67e, 0x648, 0x646, 0x62a, 0x627, 0x20, 0x622,
0x631, 0x646, 0x627, 0x633, 0x62e, 0x644, 0x6cc, 0x62c, 0x6a9, 0x20, 0x631, 0x646,
0x6a9, 0x6cc, 0x646, 0x631, 0x6cc, 0x633, 0x6cc, 0x641, 0x6cc, 0x631, 0x62c, 0x627,
0x6cc, 0x646, 0x627, 0x631, 0x632, 0x648, 0x644, 0x648, 0x62a, 0x631, 0x6cc, 0x648,
0x628, 0x631, 0x627, 0x646, 0x6a9, 0x648, 0x633, 0x646, 0x62a, 0x627, 0x631, 0x645,
0x633, 0x627, 0x646, 0x62a, 0x6cc, 0x627, 0x6af, 0x648, 0x633, 0x627, 0x646, 0x62a,
0x648, 0x20, 0x62f, 0x648, 0x645, 0x6cc, 0x646, 0x6af, 0x648, 0x633, 0x627, 0x626,
0x648, 0x67e, 0x627, 0x626, 0x648, 0x644, 0x648, 0x627, 0x633, 0x6a9, 0x648, 0x631,
0x633, 0x628, 0x6cc, 0x633, 0x648, 0x646, 0x62f, 0x633, 0x6cc, 0x62a, 0x6a9, 0x627,
0x633, 0x646, 0x62a, 0x20, 0x628, 0x627, 0x631, 0x62a, 0x644, 0x645, 0x6cc, 0x633,
0x646, 0x62a, 0x20, 0x62c, 0x627, 0x646, 0x633, 0x646, 0x62a, 0x20, 0x6a9, 0x6cc,
0x62a, 0x633, 0x633, 0x646, 0x62a, 0x20, 0x644, 0x648, 0x633, 0x6cc, 0x627, 0x633,
0x646, 0x62a, 0x20, 0x62a, 0x648, 0x645, 0x627, 0x633, 0x633, 0x646, 0x62a, 0x20,
0x648, 0x6cc, 0x646, 0x633, 0x646, 0x62a, 0x633, 0x648, 0x6cc, 0x641, 0x62a, 0x200c,
0x6a9, 0x627, 0x631, 0x646, 0x62a, 0x62a, 0x6af, 0x648, 0x633, 0x6cc, 0x6af, 0x627,
0x644, 0x67e, 0x627, 0x62a, 0x6cc, 0x62e, 0x648, 0x627, 0x646, 0x627, 0x62a, 0x648,
0x631, 0x646, 0x62a, 0x648, 0x648, 0x646, 0x6a9, 0x648, 0x648, 0x631, 0x648, 0x627,
0x6cc, 0x62a, 0x200c, 0x647, 0x648, 0x631, 0x633, 0x648, 0x6cc, 0x646, 0x6cc, 0x67e,
0x6af, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x627, 0x62a, 0x62f, 0x6cc, 0x648, 0x6cc,
0x633, 0x62f, 0x648, 0x645, 0x648, 0x646, 0x20, 0x62f, 0x648, 0x631, 0x648, 0x6cc,
0x644, 0x645, 0x6a9, 0x648, 0x627, 0x631, 0x6cc, 0x645, 0x6a9, 0x200c, 0x645, 0x648,
0x631, 0x62f, 0x648, 0x631, 0x648, 0x62a, 0x631, 0x627, 0x634, 0x648, 0x648, 0x627,
0x644, 0x627, 0x646, 0x6af, 0x200c, 0x6cc, 0x631, 0x628, 0x6cc, 0x646, 0x622, 0x644,
0x645, 0x627, 0x62a, 0x6cc, 0x639, 0x64e, 0x645, 0x627, 0x646, 0x622, 0x646, 0x627,
0x62f, 0x6cc, 0x631, 0x622, 0x642, 0x62a, 0x627, 0x648, 0x622, 0x642, 0x62a, 0x648,
0x628, 0x647, 0x639, 0x634, 0x642, 0x200c, 0x622, 0x628, 0x627, 0x62f, 0x622, 0x62a,
0x6cc, 0x631, 0x627, 0x648, 0x628, 0x627, 0x646, 0x6a9, 0x648, 0x6a9, 0x628, 0x627,
0x631, 0x646, 0x627, 0x626, 0x648, 0x644, 0x628, 0x6cc, 0x634, 0x6a9, 0x6a9, 0x628,
0x631, 0x648, 0x646, 0x626, 0x6cc, 0x686, 0x6cc, 0x62a, 0x627, 0x6a9, 0x644, 0x645,
0x628, 0x648, 0x62f, 0x6cc, 0x644, 0x6cc, 0x62f, 0x628, 0x6cc, 0x62f, 0x648, 0x634,
0x646, 0x628, 0x647, 0x641, 0x627, 0x645, 0x627, 0x6af, 0x648, 0x633, 0x62a, 0x627,
0x627, 0x644, 0x62e, 0x644, 0x6cc, 0x644, 0x647, 0x648, 0x634, 0x6cc, 0x200c, 0x645,
0x6cc, 0x646, 0x200c, 0x633, 0x6cc, 0x62a, 0x6cc, 0x647, 0x646, 0x6af, 0x200c, 0x6a9,
0x646, 0x6af, 0x62e, 0x648, 0x648, 0x62f, 0x627, 0x6cc, 0x631, 0x6a9, 0x648, 0x62a,
0x633, 0x6a9, 0x62c, 0x627, 0x6a9, 0x627, 0x631, 0x62a, 0x627, 0x62c, 0x627, 0x6cc,
0x627, 0x67e, 0x648, 0x631, 0x627, 0x627, 0x648, 0x631, 0x634, 0x644, 0x6cc, 0x645,
0x6a9, 0x631, 0x627, 0x686, 0x6cc, 0x6a9, 0x627, 0x62a, 0x645, 0x627, 0x646, 0x62f,
0x648, 0x62e, 0x627, 0x646, 0x62f, 0x6cc, 0x6af, 0x627, 0x6a9, 0x644, 0x6a9, 0x62a,
0x647, 0x6a9, 0x631, 0x627, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9,
0x6a9, 0x648, 0x627, 0x644, 0x627, 0x644, 0x627, 0x645, 0x67e, 0x648, 0x631, 0x6a9,
0x648, 0x686, 0x6cc, 0x646, 0x6af, 0x6a9, 0x648, 0x6cc, 0x62a, 0x645, 0x627, 0x6a9,
0x627, 0x626, 0x648, 0x645, 0x627, 0x6a9, 0x627, 0x633, 0x627, 0x631, 0x645, 0x627,
0x646, 0x6cc, 0x644, 0x646, 0x6cc, 0x6a9, 0x648, 0x632, 0x6cc, 0x627, 0x646, 0x648,
0x648, 0x648, 0x6a9, 0x648, 0x632, 0x646, 0x62a, 0x633, 0x6a9, 0x646, 0x648, 0x648,
0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x6cc, 0x633, 0x6a9, 0x627, 0x648, 0x631, 0x627,
0x644, 0x67e, 0x646, 0x648, 0x645, 0x200c, 0x67e, 0x646, 0x67e, 0x648, 0x646, 0x62a,
0x6cc, 0x627, 0x646, 0x627, 0x6a9, 0x67e, 0x6cc, 0x648, 0x646, 0x6af, 0x200c, 0x6cc,
0x627, 0x646, 0x6af, 0x642, 0x648, 0x633, 0x62a, 0x627, 0x646, 0x627, 0x6cc, 0x642,
0x6cc, 0x632, 0x6cc, 0x644, 0x200c, 0x627, 0x648, 0x631, 0x62f, 0x627, 0x633, 0x627,
0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x633, 0x626, 0x648, 0x644, 0x634, 0x627, 0x646,
0x6af, 0x647, 0x627, 0x6cc, 0x633, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631, 0x627,
0x633, 0x631, 0x62f, 0x646, 0x6a9, 0x648, 0x644, 0x6cc, 0x645, 0x633, 0x6a9, 0x62a,
0x627, 0x6cc, 0x67e, 0x647, 0x62a, 0x627, 0x634, 0x6a9, 0x646, 0x62f, 0x62a, 0x641,
0x644, 0x6cc, 0x633, 0x62a, 0x6cc, 0x645, 0x641, 0x648, 0x62a, 0x648, 0x6a9, 0x6cc,
0x648, 0x62a, 0x648, 0x645, 0x633, 0x6a9, 0x627, 0x648, 0x644, 0x627, 0x646, 0x200c,
0x628, 0x627, 0x62a, 0x648, 0x631, 0x627, 0x631, 0x648, 0x645, 0x686, 0x6cc, 0x627,
0x648, 0x633, 0x62a, 0x20, 0x646, 0x631, 0x627, 0x648, 0x6cc, 0x646, 0x62a, 0x6cc,
0x627, 0x646, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x200c, 0x648, 0x633, 0x62a, 0x648,
0x6a9, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x6cc, 0x627, 0x646, 0x6af,
0x648, 0x646, 0x6cc, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x631, 0x6af,
0x627, 0x6cc, 0x631, 0x648, 0x627, 0x646, 0x622, 0x632, 0x648, 0x631, 0x642, 0x646,
0x627, 0x631, 0x6cc, 0x6a9, 0x6cc, 0x67e, 0x200c, 0x648, 0x631, 0x62f, 0x645, 0x627,
0x62f, 0x6cc, 0x631, 0x627, 0x631, 0x6cc, 0x6a9, 0x6cc, 0x627, 0x648, 0x6cc, 0x6a9,
0x62c, 0x648, 0x631, 0x62c, 0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x646, 0x648, 0x628,
0x6cc, 0x633, 0x646, 0x62a, 0x20, 0x647, 0x644, 0x646, 0x627, 0x627, 0x633, 0x62a,
0x627, 0x646, 0x644, 0x6cc, 0x622, 0x62f, 0x644, 0x627, 0x6cc, 0x62f, 0x628, 0x631,
0x6cc, 0x633, 0x628, 0x6cc, 0x646, 0x628, 0x631, 0x648, 0x6a9, 0x646, 0x200c, 0x647,
0x6cc, 0x644, 0x62f, 0x627, 0x631, 0x648, 0x6cc, 0x646, 0x627, 0x648, 0x6a9, 0x644,
0x627, 0x644, 0x6cc, 0x646, 0x62f, 0x645, 0x646, 0x644, 0x631, 0x62f, 0x647, 0x627,
0x648, 0x645, 0x644, 0x628, 0x648, 0x631, 0x646, 0x633, 0x6cc, 0x62f, 0x646, 0x6cc,
0x622, 0x645, 0x633, 0x62a, 0x631, 0x62f, 0x627, 0x645, 0x622, 0x646, 0x62f, 0x648,
0x631, 0x627, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x62e, 0x627, 0x646, 0x622, 0x62a,
0x646, 0x628, 0x644, 0x6af, 0x631, 0x627, 0x62f, 0x628, 0x631, 0x644, 0x6cc, 0x646,
0x628, 0x631, 0x627, 0x62a, 0x6cc, 0x633, 0x644, 0x627, 0x648, 0x627, 0x628, 0x631,
0x648, 0x6a9, 0x633, 0x644, 0x628, 0x648, 0x62f, 0x627, 0x67e, 0x633, 0x62a, 0x628,
0x627, 0x632, 0x646, 0x6af, 0x646, 0x6a9, 0x6cc, 0x634, 0x6cc, 0x646, 0x627, 0x648,
0x6a9, 0x67e, 0x646, 0x647, 0x627, 0x6af, 0x62f, 0x648, 0x628, 0x644, 0x6cc, 0x646,
0x62c, 0x628, 0x644, 0x200c, 0x627, 0x644, 0x637, 0x627, 0x631, 0x642, 0x6af, 0x631,
0x646, 0x632, 0x6cc, 0x647, 0x644, 0x633, 0x6cc, 0x646, 0x6a9, 0x6cc, 0x62c, 0x632,
0x6cc, 0x631, 0x647, 0x654, 0x20, 0x645, 0x646, 0x627, 0x633, 0x62a, 0x627, 0x646,
0x628, 0x648, 0x644, 0x62c, 0x631, 0x632, 0x6cc, 0x6a9, 0x627, 0x644, 0x6cc, 0x646,
0x6cc, 0x646, 0x6af, 0x631, 0x627, 0x62f, 0x644, 0x6cc, 0x648, 0x628, 0x644, 0x6cc,
0x627, 0x646, 0x627, 0x644, 0x648, 0x6a9, 0x632, 0x627, 0x645, 0x628, 0x648, 0x631,
0x6af, 0x645, 0x627, 0x62f, 0x631, 0x6cc, 0x62f, 0x645, 0x627, 0x631, 0x6cc, 0x647,
0x200c, 0x647, 0x627, 0x645, 0x646, 0x645, 0x6cc, 0x646, 0x633, 0x6a9, 0x67e, 0x627,
0x62f, 0x6af, 0x627, 0x631, 0x6cc, 0x62a, 0x633, 0x627, 0x633, 0x627, 0x646, 0x200c,
0x645, 0x627, 0x631, 0x6cc, 0x646, 0x648, 0x633, 0x627, 0x631, 0x627, 0x6cc, 0x648,
0x648, 0x633, 0x6cc, 0x645, 0x641, 0x631, 0x648, 0x67e, 0x644, 0x627, 0x633, 0x6a9,
0x648, 0x67e, 0x6cc, 0x647, 0x635, 0x648, 0x641, 0x6cc, 0x647, 0x627, 0x633, 0x62a,
0x6a9, 0x647, 0x644, 0x645, 0x62a, 0x627, 0x644, 0x6cc, 0x646, 0x62a, 0x6cc, 0x631,
0x627, 0x646, 0x627, 0x627, 0x648, 0x644, 0x6cc, 0x627, 0x646, 0x648, 0x641, 0x633,
0x6a9, 0x641, 0x627, 0x62f, 0x648, 0x62a, 0x633, 0x648, 0x627, 0x62a, 0x6cc, 0x6a9,
0x627, 0x646, 0x648, 0x6cc, 0x644, 0x646, 0x6cc, 0x648, 0x633, 0x648, 0x644, 0x6af,
0x627, 0x6af, 0x631, 0x627, 0x62f, 0x648, 0x631, 0x634, 0x648, 0x632, 0x627, 0x6af,
0x631, 0x628, 0x632, 0x648, 0x631, 0x6cc, 0x62e, 0x622, 0x646, 0x62a, 0x627, 0x646,
0x627, 0x646, 0x627, 0x631, 0x6cc, 0x648, 0x648, 0x634, 0x627, 0x6af, 0x648, 0x633,
0x6a9, 0x631, 0x6cc, 0x633, 0x645, 0x633, 0x6a9, 0x631, 0x6af, 0x648, 0x644, 0x646,
0x645, 0x627, 0x647, 0x647, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x648, 0x645, 0x648,
0x631, 0x6cc, 0x633, 0x645, 0x627, 0x6cc, 0x648, 0x62a, 0x631, 0x626, 0x648, 0x646,
0x6cc, 0x648, 0x646, 0x622, 0x67e, 0x6cc, 0x627, 0x627, 0x648, 0x6a9, 0x644, 0x646,
0x62f, 0x628, 0x648, 0x6af, 0x646, 0x648, 0x6cc, 0x644, 0x686, 0x62a, 0x200c, 0x647,
0x627, 0x645, 0x627, 0x6cc, 0x633, 0x62a, 0x631, 0x627, 0x641, 0x627, 0x62a, 0x647,
0x641, 0x627, 0x6a9, 0x627, 0x626, 0x648, 0x641, 0x648, 0x641, 0x6cc, 0x62c, 0x6cc,
0x641, 0x648, 0x646, 0x627, 0x641, 0x648, 0x62a, 0x6cc, 0x6af, 0x627, 0x645, 0x628,
0x6cc, 0x631, 0x6af, 0x648, 0x627, 0x62f, 0x627, 0x644, 0x200c, 0x6a9, 0x627, 0x646,
0x627, 0x644, 0x627, 0x646, 0x62f, 0x631, 0x628, 0x631, 0x6cc, 0x6a9, 0x631, 0x6cc,
0x62a, 0x6cc, 0x645, 0x627, 0x62a, 0x6cc, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x6cc,
0x6a9, 0x648, 0x627, 0x62c, 0x6cc, 0x644, 0x6cc, 0x646, 0x645, 0x627, 0x631, 0x6a9,
0x648, 0x632, 0x647, 0x645, 0x6cc, 0x62f, 0x648, 0x6cc, 0x646, 0x627, 0x626, 0x648,
0x631, 0x648, 0x646, 0x6cc, 0x648, 0x626, 0x647, 0x646, 0x648, 0x631, 0x641, 0x648,
0x644, 0x6a9, 0x646, 0x648, 0x645, 0x626, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x67e,
0x627, 0x6af, 0x648, 0x67e, 0x627, 0x644, 0x627, 0x626, 0x648, 0x67e, 0x6cc, 0x62a,
0x200c, 0x6a9, 0x631, 0x646, 0x67e, 0x627, 0x646, 0x67e, 0x6cc, 0x67e, 0x648, 0x631,
0x62a, 0x200c, 0x645, 0x648, 0x631, 0x632, 0x628, 0x6cc, 0x633, 0x627, 0x6cc, 0x67e,
0x627, 0x646, 0x62a, 0x627, 0x647, 0x6cc, 0x62a, 0x6cc, 0x62a, 0x648, 0x646, 0x6af,
0x627, 0x62a, 0x627, 0x67e, 0x648, 0x648, 0x627, 0x644, 0x6cc, 0x633, 0x41, 0x62,
0x69, 0x64, 0x17c, 0x61, 0x6e, 0x42, 0x61, 0x6e, 0x64, 0x17c, 0x75, 0x6c,
0x42, 0x75, 0x17c, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x44, 0x17c, 0x69,
0x62, 0x75, 0x74, 0x69, 0x41, 0x6c, 0x2d, 0x55, 0x6a, 0x75, 0x6e, 0x44,
0x17c, 0x75, 0x62, 0x61, 0x4b, 0x69, 0x6e, 0x73, 0x7a, 0x61, 0x73, 0x61,
0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x73, 0x7a, 0x75, 0x4e, 0x64, 0x17c,
0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x61, 0x77, 0x61, 0x6b, 0x73, 0x7a,
0x75, 0x74, 0x54, 0x72, 0x79, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4b, 0x61,
0x6a, 0x6d, 0x61, 0x6e, 0x79, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x72, 0x79,
0x6b, 0x61, 0x53, 0x61, 0x6c, 0x77, 0x61, 0x64, 0x6f, 0x72, 0x47, 0x77,
0x61, 0x64, 0x65, 0x6c, 0x75, 0x70, 0x61, 0x47, 0x75, 0x6a, 0x61, 0x6e,
0x61, 0x48, 0x61, 0x77, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x74, 0x79,
0x6e, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28,
0x6d, 0x69, 0x61, 0x73, 0x74, 0x6f, 0x29, 0x4e, 0x6f, 0x77, 0x79, 0x20,
0x4a, 0x6f, 0x72, 0x6b, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x50, 0xf3, 0x142, 0x6e, 0x6f,
0x63, 0x6e, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x50, 0xf3, 0x142, 0x6e, 0x6f, 0x63,
0x6e, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x50, 0xf3, 0x142, 0x6e,
0x6f, 0x63, 0x6e, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x79, 0x6b,
0x6f, 0x41, 0x142, 0x6d, 0x61, 0x74, 0x79, 0x41, 0x6b, 0x74, 0x69, 0x75,
0x62, 0x69, 0x144, 0x73, 0x6b, 0x41, 0x73, 0x7a, 0x63, 0x68, 0x61, 0x62,
0x61, 0x64, 0x42, 0x61, 0x72, 0x6e, 0x61, 0x75, 0x142, 0x42, 0x69, 0x73,
0x7a, 0x6b, 0x65, 0x6b, 0x43, 0x7a, 0x79, 0x74, 0x61, 0x44, 0x61, 0x6d,
0x61, 0x73, 0x7a, 0x65, 0x6b, 0x44, 0x75, 0x73, 0x7a, 0x61, 0x6e, 0x62,
0x65, 0x4b, 0x6f, 0x62, 0x64, 0x6f, 0x44, 0x17c, 0x61, 0x6b, 0x61, 0x72,
0x74, 0x61, 0x4a, 0x65, 0x72, 0x6f, 0x7a, 0x6f, 0x6c, 0x69, 0x6d, 0x61,
0x4b, 0x61, 0x6d, 0x63, 0x7a, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72,
0x61, 0x63, 0x7a, 0x69, 0x4b, 0x75, 0x77, 0x65, 0x6a, 0x74, 0x4e, 0x69,
0x6b, 0x6f, 0x7a, 0x6a, 0x61, 0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x75, 0x17a,
0x6e, 0x69, 0x65, 0x63, 0x6b, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x4b, 0x75, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x6a,
0x4b, 0x79, 0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64, 0x61, 0x53, 0x7a, 0x61,
0x6e, 0x67, 0x68, 0x61, 0x6a, 0x53, 0x72, 0x69, 0x65, 0x64, 0x6e, 0x69,
0x65, 0x6b, 0x6f, 0x142, 0x79, 0x6d, 0x73, 0x6b, 0x54, 0x61, 0x73, 0x7a,
0x6b, 0x69, 0x65, 0x6e, 0x74, 0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x55, 0x72, 0x75, 0x6d, 0x63, 0x7a, 0x69, 0x55, 0x73,
0x74, 0x2d, 0x4e, 0x69, 0x65, 0x72, 0x61, 0x57, 0x69, 0x65, 0x6e, 0x74,
0x69, 0x61, 0x6e, 0x57, 0x142, 0x61, 0x64, 0x79, 0x77, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x79, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x45, 0x72, 0x79, 0x77, 0x61, 0x144, 0x41, 0x7a, 0x6f,
0x72, 0x79, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x4b, 0x61, 0x6e, 0x61,
0x72, 0x79, 0x6a, 0x73, 0x6b, 0x69, 0x65, 0x52, 0x65, 0x70, 0x75, 0x62,
0x6c, 0x69, 0x6b, 0x61, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x6f, 0x6e, 0x65,
0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x79, 0x6c, 0x105, 0x64, 0x6b, 0x61,
0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x4f, 0x77, 0x63, 0x7a, 0x65, 0x4d,
0x61, 0x64, 0x65, 0x72, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x50, 0x6f, 0x142, 0x75, 0x64, 0x6e, 0x69, 0x6f, 0x77, 0x61, 0x15a,
0x77, 0x69, 0x119, 0x74, 0x61, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61,
0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0x68, 0x61, 0x144, 0x41, 0x74, 0x65,
0x6e, 0x79, 0x42, 0x72, 0x61, 0x74, 0x79, 0x73, 0x142, 0x61, 0x77, 0x61,
0x42, 0x72, 0x75, 0x6b, 0x73, 0x65, 0x6c, 0x61, 0x42, 0x75, 0x6b, 0x61,
0x72, 0x65, 0x73, 0x7a, 0x74, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x73,
0x7a, 0x74, 0x4b, 0x69, 0x73, 0x7a, 0x79, 0x6e, 0x69, 0xf3, 0x77, 0x57,
0x79, 0x73, 0x70, 0x61, 0x20, 0x4d, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6d,
0x62, 0x75, 0x142, 0x4b, 0x69, 0x6a, 0xf3, 0x77, 0x4c, 0x69, 0x7a, 0x62,
0x6f, 0x6e, 0x61, 0x4c, 0x75, 0x62, 0x6c, 0x61, 0x6e, 0x61, 0x4c, 0x6f,
0x6e, 0x64, 0x79, 0x6e, 0x4d, 0x61, 0x64, 0x72, 0x79, 0x74, 0x4d, 0x69,
0x144, 0x73, 0x6b, 0x50, 0x61, 0x72, 0x79, 0x17c, 0x52, 0x7a, 0x79, 0x6d,
0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x77, 0x6f, 0x53, 0x61, 0x72, 0x61,
0x74, 0xf3, 0x77, 0x53, 0x79, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f,
0x6c, 0x53, 0x7a, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x57, 0x61,
0x74, 0x79, 0x6b, 0x61, 0x6e, 0x57, 0x69, 0x65, 0x64, 0x65, 0x144, 0x57,
0x69, 0x6c, 0x6e, 0x6f, 0x57, 0x6f, 0x142, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x5a, 0x61, 0x67, 0x72, 0x7a, 0x65, 0x62, 0x5a, 0x75, 0x72, 0x79,
0x63, 0x68, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x79,
0x77, 0x61, 0x43, 0x7a, 0x61, 0x67, 0x6f, 0x73, 0x57, 0x79, 0x73, 0x70,
0x61, 0x20, 0x42, 0x6f, 0x17c, 0x65, 0x67, 0x6f, 0x20, 0x4e, 0x61, 0x72,
0x6f, 0x64, 0x7a, 0x65, 0x6e, 0x69, 0x61, 0x57, 0x79, 0x73, 0x70, 0x79,
0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x77, 0x65, 0x57, 0x79, 0x73,
0x70, 0x79, 0x20, 0x4b, 0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e,
0x61, 0x4d, 0x61, 0x6a, 0x6f, 0x74, 0x74, 0x61, 0x57, 0x79, 0x73, 0x70,
0x61, 0x20, 0x42, 0x6f, 0x75, 0x67, 0x61, 0x69, 0x6e, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x2019, 0x61, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69,
0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x61, 0x46, 0x69, 0x64,
0x17c, 0x69, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x61, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x79, 0x43,
0x6f, 0x6e, 0x61, 0x63, 0x72, 0x69, 0x4a, 0x6f, 0x61, 0x6e, 0x65, 0x73,
0x62, 0x75, 0x72, 0x67, 0x6f, 0x43, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x4d,
0x6f, 0x67, 0x61, 0x64, 0xed, 0x73, 0x63, 0x69, 0x6f, 0x4e, 0x61, 0x69,
0x72, 0xf3, 0x62, 0x69, 0x41, 0x6e, 0x74, 0xed, 0x67, 0x75, 0x61, 0x54,
0x75, 0x63, 0x75, 0x6d, 0xe3, 0x41, 0x73, 0x73, 0x75, 0x6e, 0xe7, 0xe3,
0x6f, 0x47, 0x75, 0x61, 0x69, 0x61, 0x71, 0x75, 0x69, 0x6c, 0x47, 0x75,
0x69, 0x61, 0x6e, 0x61, 0x43, 0x69, 0x64, 0x61, 0x64, 0x65, 0x20, 0x64,
0x6f, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x4d, 0x6f, 0x6e, 0x74,
0x65, 0x76, 0x69, 0x64, 0xe9, 0x75, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6e, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x53, 0xe3, 0x6f, 0x20, 0x42, 0x61,
0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x75, 0x53, 0xe3, 0x6f, 0x20,
0x43, 0x72, 0x69, 0x73, 0x74, 0xf3, 0x76, 0xe3, 0x6f, 0x53, 0x61, 0x6e,
0x74, 0x61, 0x20, 0x4c, 0xfa, 0x63, 0x69, 0x61, 0x53, 0xe3, 0x6f, 0x20,
0x56, 0x69, 0x63, 0x65, 0x6e, 0x74, 0x65, 0x41, 0x6d, 0xe3, 0x41, 0x73,
0x67, 0x61, 0x62, 0x61, 0x74, 0x65, 0x42, 0x65, 0x69, 0x72, 0x75, 0x74,
0x65, 0x44, 0x75, 0x63, 0x68, 0x61, 0x6d, 0x62, 0x65, 0x43, 0x69, 0x64,
0x61, 0x64, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68,
0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x4a, 0x61, 0x63, 0x61, 0x72, 0x74,
0x61, 0x4a, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0xe9, 0x6d, 0x43, 0x61,
0x6c, 0x63, 0x75, 0x74, 0xe1, 0x4e, 0x69, 0x63, 0xf3, 0x73, 0x69, 0x61,
0x52, 0x69, 0x61, 0x64, 0x65, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e,
0x61, 0x58, 0x61, 0x6e, 0x67, 0x61, 0x69, 0x54, 0x65, 0x65, 0x72, 0xe3,
0x54, 0xf3, 0x71, 0x75, 0x69, 0x6f, 0x52, 0x61, 0x6e, 0x67, 0x75, 0x6d,
0x45, 0x63, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67,
0x6f, 0x43, 0x61, 0x6e, 0xe1, 0x72, 0x69, 0x61, 0x73, 0x49, 0x6c, 0x68,
0x61, 0x73, 0x20, 0x46, 0x61, 0x72, 0x6f, 0xe9, 0x47, 0x65, 0xf3, 0x72,
0x67, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x6c, 0x41, 0x6d,
0x73, 0x74, 0x65, 0x72, 0x64, 0xe3, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63,
0xe3, 0x42, 0x65, 0x72, 0x6c, 0x69, 0x6d, 0x42, 0x75, 0x63, 0x61, 0x72,
0x65, 0x73, 0x74, 0x65, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74,
0x65, 0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x71, 0x75, 0x65, 0x49, 0x6c,
0x68, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4d, 0xf4, 0x6e,
0x61, 0x63, 0x6f, 0x53, 0xf3, 0x66, 0x69, 0x61, 0x56, 0x61, 0x72, 0x73,
0xf3, 0x76, 0x69, 0x61, 0x5a, 0x75, 0x72, 0x69, 0x71, 0x75, 0x65, 0x4d,
0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6f, 0x52, 0x65, 0x75, 0x6e, 0x69,
0xe3, 0x6f, 0x43, 0x68, 0x61, 0x74, 0x6e, 0x61, 0x6d, 0x49, 0x6c, 0x68,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61, 0x54,
0x61, 0x72, 0x61, 0x75, 0x61, 0x41, 0x64, 0x69, 0x73, 0x2d, 0x41, 0x62,
0x65, 0x62, 0x61, 0x44, 0x61, 0x63, 0x61, 0x72, 0x4a, 0x69, 0x62, 0x75,
0x74, 0x69, 0x43, 0x61, 0x6d, 0x70, 0x61, 0x6c, 0x61, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x41, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x6e, 0x61, 0x43, 0x61, 0x69, 0x6d, 0xe3, 0x6f, 0x43, 0x75,
0x72, 0x61, 0xe7, 0x61, 0x75, 0x44, 0x6f, 0x6d, 0xed, 0x6e, 0x69, 0x63,
0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x76, 0x69, 0x64, 0x65, 0x75, 0x4d,
0x6f, 0x6e, 0x73, 0x65, 0x72, 0x72, 0x61, 0x74, 0x65, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x49, 0x6f, 0x72, 0x71, 0x75, 0x65, 0x50, 0x6f, 0x72, 0x74,
0x2d, 0x61, 0x75, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x63, 0x65, 0x50, 0x6f,
0x72, 0x74, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x70, 0x61, 0x6e,
0x68, 0x61, 0x41, 0x64, 0x65, 0x6d, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x42, 0x61, 0x67, 0x64, 0x61, 0x64, 0x65, 0x42, 0x61, 0x72, 0xe9, 0x6d,
0x42, 0x61, 0x6e, 0x67, 0x75, 0x65, 0x63, 0x6f, 0x71, 0x75, 0x65, 0x43,
0x61, 0x72, 0x61, 0x63, 0x68, 0x69, 0x43, 0x61, 0x74, 0x6d, 0x61, 0x6e,
0x64, 0x75, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x54, 0x65, 0x65, 0x72, 0xe3,
0x6f, 0x52, 0x65, 0x69, 0x71, 0x75, 0x69, 0x61, 0x76, 0x69, 0x71, 0x75,
0x65, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d, 0x65, 0x73, 0x74, 0x65,
0x72, 0x64, 0xe3, 0x6f, 0x42, 0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e,
0x43, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x61, 0x48, 0x65, 0x6c,
0x73, 0xed, 0x6e, 0x71, 0x75, 0x69, 0x61, 0x43, 0x61, 0x6c, 0x69, 0x6e,
0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x76, 0x6f, 0x53, 0xe3, 0x6f, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x68,
0x6f, 0x54, 0x61, 0x6c, 0x69, 0x6d, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x4e, 0x61, 0x74, 0x61, 0x6c, 0x49, 0x6c, 0x68, 0x61, 0x73,
0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4d, 0x61, 0x68, 0x65, 0x4d, 0x61,
0x75, 0x72, 0xed, 0x63, 0x69, 0x61, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64,
0x61, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61, 0x45, 0x66, 0x61, 0x74,
0x65, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x54, 0x61, 0x72, 0x61, 0x77, 0x61, 0xa05, 0xa2c, 0xa40,
0xa26, 0xa1c, 0xa3e, 0xa28, 0xa05, 0xa71, 0xa15, 0xa30, 0xa3e, 0xa10, 0xa21, 0xa3f,
0xa38, 0x20, 0xa05, 0xa2c, 0xa3e, 0xa2c, 0xa3e, 0xa05, 0xa32, 0xa1c, 0xa40, 0xa05,
0xa30, 0xa38, 0xa05, 0xa38, 0xa2e, 0xa3e, 0xa30, 0xa3e, 0xa2c, 0xa2e, 0xa47, 0xa15,
0xa4b, 0xa2c, 0xa3e, 0xa02, 0xa17, 0xa41, 0xa08, 0xa2c, 0xa3e, 0xa02, 0xa1c, 0xa41,
0xa32, 0xa2c, 0xa3f, 0xa38, 0xa3e, 0xa09, 0xa2c, 0xa32, 0xa70, 0xa1f, 0xa3e, 0xa07,
0xa30, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1c, 0xa3c, 0xa3e, 0xa35, 0xa3f, 0xa32, 0xa47,
0xa2c, 0xa41, 0xa1c, 0xa41, 0xa70, 0xa2c, 0xa41, 0xa30, 0xa3e, 0xa15, 0xa48, 0xa30,
0xa4b, 0xa15, 0xa3e, 0xa38, 0xa3e, 0xa2c, 0xa32, 0xa3e, 0xa02, 0xa15, 0xa3e, 0xa38,
0xa40, 0xa0a, 0xa1f, 0xa3e, 0xa15, 0xa4b, 0xa28, 0xa47, 0xa15, 0xa30, 0xa40, 0xa21,
0xa15, 0xa3e, 0xa30, 0xa26, 0xa3e, 0xa30, 0x20, 0xa0f, 0xa38, 0x20, 0xa38, 0xa32,
0xa3e, 0xa2e, 0xa1c, 0xa3f, 0xa2c, 0xa42, 0xa24, 0xa40, 0xa21, 0xa4c, 0xa06, 0xa32,
0xa3e, 0xa05, 0xa32, 0x20, 0xa05, 0xa2f, 0xa42, 0xa28, 0xa2b, 0xa30, 0xa40, 0xa1f,
0xa3e, 0xa09, 0xa28, 0xa17, 0xa3e, 0xa2c, 0xa4b, 0xa30, 0xa4b, 0xa28, 0xa39, 0xa30,
0xa3e, 0xa30, 0xa47, 0xa1c, 0xa4b, 0xa39, 0xa28, 0xa38, 0xa2c, 0xa30, 0xa17, 0xa1c,
0xa42, 0xa2c, 0xa3e, 0xa15, 0xa70, 0xa2a, 0xa3e, 0xa32, 0xa3e, 0xa16, 0xa3e, 0xa30,
0xa1f, 0xa4c, 0xa2e, 0xa15, 0xa3f, 0xa17, 0xa3e, 0xa32, 0xa40, 0xa15, 0xa3f, 0xa70,
0xa38, 0xa3c, 0xa3e, 0xa38, 0xa3e, 0xa32, 0xa3e, 0xa17, 0xa4b, 0xa38, 0xa32, 0xa3f,
0xa2c, 0xa30, 0xa35, 0xa3f, 0xa32, 0xa47, 0xa32, 0xa4b, 0xa2e, 0xa32, 0xa41, 0xa06,
0xa02, 0xa21, 0xa3e, 0xa32, 0xa41, 0xa2c, 0xa41, 0xa2e, 0xa2c, 0xa3e, 0xa38, 0xa3c,
0xa40, 0xa32, 0xa41, 0xa38, 0xa3e, 0xa15, 0xa3e, 0xa2e, 0xa3e, 0xa32, 0xa3e, 0xa2c,
0xa4b, 0xa2e, 0xa3e, 0xa2a, 0xa41, 0xa1f, 0xa4b, 0xa2e, 0xa38, 0xa47, 0xa30, 0xa42,
0xa0f, 0xa2e, 0xa2c, 0xa3e, 0xa2c, 0xa3e, 0xa28, 0xa47, 0xa2e, 0xa4b, 0xa17, 0xa3e,
0xa26, 0xa3f, 0xa38, 0xa3c, 0xa42, 0xa2e, 0xa4b, 0xa28, 0xa30, 0xa4b, 0xa35, 0xa40,
0xa06, 0xa28, 0xa48, 0xa30, 0xa4b, 0xa2c, 0xa40, 0xa10, 0xa02, 0xa1c, 0xa3e, 0xa2e,
0xa47, 0xa28, 0xa3e, 0xa28, 0xa3f, 0xa06, 0xa2e, 0xa47, 0xa28, 0xa4c, 0xa06, 0xa15,
0xa38, 0xa3c, 0xa3e, 0xa1f, 0xa09, 0xa06, 0xa17, 0xa3e, 0xa21, 0xa42, 0xa17, 0xa42,
0xa2a, 0xa4b, 0xa30, 0xa1f, 0xa4b, 0x2d, 0xa28, 0xa4b, 0xa35, 0xa4b, 0xa38, 0xa3e,
0xa13, 0x20, 0xa1f, 0xa4b, 0xa2e, 0xa24, 0xa4d, 0xa30, 0xa3f, 0xa2a, 0xa4b, 0xa32,
0xa40, 0xa1f, 0xa41, 0xa28, 0xa3f, 0xa38, 0xa35, 0xa3f, 0xa70, 0xa21, 0xa39, 0xa4b,
0xa07, 0xa15, 0xa0f, 0xa21, 0xa15, 0xa10, 0xa02, 0xa15, 0xa30, 0xa47, 0xa1c, 0xa05,
0xa70, 0xa17, 0xa41, 0xa07, 0xa32, 0xa3e, 0xa10, 0xa02, 0xa1f, 0xa40, 0xa17, 0xa41,
0xa06, 0xa06, 0xa30, 0xa17, 0xa41, 0xa06, 0xa07, 0xa28, 0xa3e, 0xa2c, 0xa42, 0xa08,
0xa28, 0xa38, 0x20, 0xa06, 0xa07, 0xa30, 0xa38, 0xa15, 0xa48, 0xa1f, 0xa3e, 0xa2e,
0xa3e, 0xa30, 0xa15, 0xa3e, 0xa15, 0xa4b, 0xa30, 0xa21, 0xa4b, 0xa2c, 0xa3e, 0xa1c,
0xa42, 0xa1c, 0xa41, 0xa0f, 0xa32, 0xa3e, 0x20, 0xa30, 0xa3f, 0xa09, 0xa1c, 0xa3e,
0xa2e, 0xa48, 0xa02, 0xa21, 0xa4b, 0xa1c, 0xa3c, 0xa3e, 0xa30, 0xa3f, 0xa13, 0x20,
0xa17, 0xa48, 0xa32, 0xa47, 0xa17, 0xa4b, 0xa38, 0xa38, 0xa3e, 0xa32, 0xa1f, 0xa3e,
0xa38, 0xa47, 0xa28, 0x20, 0xa1c, 0xa41, 0xa06, 0xa28, 0xa38, 0xa47, 0xa28, 0x20,
0xa32, 0xa42, 0xa08, 0xa38, 0xa1f, 0xa41, 0xa15, 0xa41, 0xa2e, 0xa28, 0xa09, 0xa38,
0xa3c, 0xa35, 0xa3e, 0xa07, 0xa06, 0xa05, 0xa30, 0xa42, 0xa2c, 0xa3e, 0xa10, 0xa38,
0xa41, 0xa70, 0xa15, 0xa40, 0xa05, 0xa28, 0xa10, 0xa1f, 0xa40, 0xa15, 0xa4b, 0xa15,
0xa28, 0xa2c, 0xa3e, 0xa39, 0xa40, 0xa06, 0xa2c, 0xa3e, 0xa39, 0xa40, 0xa06, 0x20,
0xa2c, 0xa3e, 0xa02, 0xa26, 0xa47, 0xa30, 0xa38, 0xa2c, 0xa3e, 0xa30, 0xa2c, 0xa3e,
0xa21, 0xa4b, 0xa38, 0xa2c, 0xa47, 0xa32, 0xa47, 0xa2e, 0xa2c, 0xa47, 0xa32, 0xa40,
0xa1c, 0xa3c, 0xa2c, 0xa32, 0xa48, 0xa02, 0xa15, 0x2d, 0xa38, 0xa48, 0xa2c, 0xa32,
0xa3e, 0xa28, 0xa2c, 0xa4b, 0xa06, 0x20, 0xa35, 0xa3f, 0xa38, 0xa1f, 0xa3e, 0xa2c,
0xa4b, 0xa17, 0xa4b, 0xa1f, 0xa3e, 0xa2c, 0xa4b, 0xa07, 0xa38, 0xa15, 0xa48, 0xa2e,
0xa2c, 0xa4d, 0xa30, 0xa3f, 0xa1c, 0x20, 0xa2c, 0xa47, 0xa15, 0xa48, 0xa02, 0xa2a,
0xa4b, 0x20, 0xa17, 0xa4d, 0xa30, 0xa3e, 0xa02, 0xa21, 0xa47, 0xa15, 0xa48, 0xa28,
0xa15, 0xa41, 0xa28, 0xa15, 0xa48, 0xa30, 0xa3e, 0xa15, 0xa3e, 0xa38, 0xa15, 0xa47,
0xa2f, 0xa47, 0xa28, 0xa47, 0xa15, 0xa47, 0xa2e, 0xa48, 0xa28, 0xa38, 0xa3c, 0xa3f,
0xa15, 0xa3e, 0xa17, 0xa4b, 0xa1a, 0xa3f, 0xa39, 0xa41, 0xa06, 0xa39, 0xa41, 0xa06,
0xa38, 0xa3f, 0xa09, 0xa21, 0xa3e, 0xa21, 0x20, 0xa39, 0xa41, 0xa06, 0xa30, 0xa47,
0xa1c, 0xa3c, 0xa15, 0xa4b, 0xa38, 0xa1f, 0xa3e, 0x20, 0xa30, 0xa40, 0xa15, 0xa3e,
0xa15, 0xa4d, 0xa30, 0xa48, 0xa38, 0xa1f, 0xa28, 0xa15, 0xa2f, 0xa41, 0xa0f, 0xa2c,
0xa3e, 0xa15, 0xa41, 0xa30, 0xa3e, 0xa15, 0xa3e, 0xa13, 0xa21, 0xa48, 0xa28, 0xa2e,
0xa3e, 0xa30, 0xa15, 0xa38, 0xa3c, 0xa4c, 0xa28, 0xa21, 0xa4c, 0xa05, 0xa38, 0xa28,
0xa21, 0xa4c, 0xa05, 0xa38, 0xa28, 0x20, 0xa15, 0xa4d, 0xa30, 0xa40, 0xa15, 0xa21,
0xa47, 0xa28, 0xa35, 0xa30, 0xa21, 0xa3f, 0xa1f, 0xa30, 0xa4b, 0xa07, 0xa1f, 0xa21,
0xa4b, 0xa2e, 0xa40, 0xa28, 0xa3f, 0xa15, 0xa3e, 0xa10, 0xa21, 0xa2e, 0xa4b, 0xa02,
0xa1f, 0xa28, 0xa2f, 0xa41, 0xa30, 0xa42, 0xa28, 0xa40, 0xa2a, 0xa47, 0xa05, 0xa32,
0x20, 0xa38, 0xa32, 0xa35, 0xa3e, 0xa21, 0xa4b, 0xa30, 0xa2b, 0xa4b, 0xa30, 0xa1f,
0x20, 0xa28, 0xa48, 0xa32, 0xa38, 0xa28, 0xa2b, 0xa4b, 0xa30, 0xa1f, 0xa3e, 0xa32,
0xa47, 0xa1c, 0xa3c, 0xa3e, 0xa17, 0xa32, 0xa47, 0xa38, 0x20, 0xa2c, 0xa47, 0xa17,
0xa42, 0xa1c, 0xa3c, 0x20, 0xa2c, 0xa47, 0xa17, 0xa30, 0xa3e, 0xa02, 0xa21, 0x20,
0xa24, 0xa41, 0xa30, 0xa15, 0xa17, 0xa4d, 0xa30, 0xa47, 0xa28, 0xa3e, 0xa21, 0xa3e,
0xa17, 0xa41, 0xa06, 0xa21, 0xa47, 0xa32, 0xa4b, 0xa2a, 0xa17, 0xa41, 0xa06, 0xa1f,
0xa47, 0xa2e, 0xa3e, 0xa32, 0xa3e, 0xa17, 0xa41, 0xa06, 0xa07, 0xa15, 0xa35, 0xa3f,
0xa32, 0xa17, 0xa41, 0xa06, 0xa28, 0xa3e, 0xa39, 0xa48, 0xa32, 0xa40, 0xa2b, 0xa48,
0xa15, 0xa38, 0xa39, 0xa35, 0xa3e, 0xa28, 0xa3e, 0xa39, 0xa30, 0xa2e, 0xa4b, 0xa38,
0xa3f, 0xa71, 0xa32, 0xa4b, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa2a,
0xa4b, 0xa32, 0xa3f, 0xa38, 0xa28, 0xa4b, 0xa15, 0xa38, 0x2c, 0x20, 0xa07, 0xa70,
0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa2e, 0xa3e, 0xa30, 0xa47, 0xa02, 0xa17, 0xa4b,
0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa2a, 0xa40, 0xa1f,
0xa30, 0xa38, 0xa2c, 0xa30, 0xa17, 0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06,
0xa28, 0xa3e, 0xa1f, 0xa48, 0xa32, 0x20, 0xa38, 0xa3f, 0xa1f, 0xa40, 0x2c, 0x20,
0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa35, 0xa47, 0xa35, 0xa47, 0x2c,
0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa35, 0xa3f, 0xa70, 0xa38,
0xa47, 0xa28, 0xa47, 0xa38, 0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28,
0xa3e, 0xa35, 0xa3f, 0xa28, 0xa2e, 0xa48, 0xa15, 0x2c, 0x20, 0xa07, 0xa70, 0xa21,
0xa40, 0xa06, 0xa28, 0xa3e, 0xa07, 0xa28, 0xa41, 0xa35, 0xa3f, 0xa15, 0xa07, 0xa15,
0xa3e, 0xa32, 0xa41, 0xa08, 0xa1f, 0xa1c, 0xa2e, 0xa3e, 0xa07, 0xa15, 0xa3e, 0xa1c,
0xa2f, 0xa42, 0xa28, 0xa4b, 0xa32, 0xa41, 0xa08, 0xa38, 0xa35, 0xa3f, 0xa32, 0xa47,
0xa2e, 0xa4b, 0xa02, 0xa1f, 0xa40, 0xa38, 0xa48, 0xa32, 0xa4b, 0x2c, 0x20, 0xa15,
0xa48, 0xa02, 0xa1f, 0xa15, 0xa40, 0xa15, 0xa4d, 0xa30, 0xa3e, 0xa32, 0xa47, 0xa02,
0xa26, 0xa3f, 0xa1c, 0xa15, 0xa32, 0xa3e, 0x20, 0xa2a, 0xa3e, 0xa1c, 0xa3c, 0xa32,
0xa40, 0xa2e, 0xa3e, 0xa32, 0xa3e, 0xa38, 0x20, 0xa10, 0xa02, 0xa1c, 0xa32, 0xa38,
0xa32, 0xa4b, 0xa05, 0xa30, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa3f, 0xa70, 0xa38, 0x2019,
0xa38, 0x20, 0xa15, 0xa35, 0xa3e, 0xa30, 0xa1f, 0xa30, 0xa2e, 0xa48, 0xa38, 0xa40,
0xa13, 0xa2e, 0xa28, 0xa3e, 0xa17, 0xa41, 0xa06, 0xa2e, 0xa28, 0xa4c, 0xa38, 0xa2e,
0xa48, 0xa30, 0xa40, 0xa17, 0xa4b, 0xa1f, 0xa2e, 0xa3e, 0xa30, 0xa1f, 0xa40, 0xa28,
0xa3f, 0xa15, 0xa2e, 0xa3e, 0xa1f, 0xa3e, 0xa2e, 0xa4b, 0xa30, 0xa38, 0xa2e, 0xa1c,
0xa3c, 0xa47, 0xa24, 0xa32, 0xa3e, 0xa28, 0xa2e, 0xa48, 0xa28, 0xa4b, 0xa2e, 0xa3f,
0xa28, 0xa40, 0xa2e, 0xa47, 0xa30, 0xa3f, 0xa21, 0xa3e, 0xa2e, 0xa47, 0xa1f, 0xa32,
0xa3e, 0xa15, 0xa3e, 0xa1f, 0xa32, 0xa3e, 0xa2e, 0xa48, 0xa15, 0xa38, 0xa40, 0xa15,
0xa4b, 0x20, 0xa38, 0xa3f, 0xa1f, 0xa40, 0xa2e, 0xa3f, 0xa15, 0xa47, 0xa32, 0xa28,
0xa2e, 0xa4b, 0xa02, 0xa15, 0xa1f, 0xa28, 0xa2e, 0xa4b, 0xa28, 0xa1f, 0xa47, 0xa30,
0xa40, 0xa2e, 0xa4b, 0xa02, 0xa1f, 0xa47, 0xa35, 0xa40, 0xa21, 0xa40, 0xa13, 0xa2e,
0xa4b, 0xa02, 0xa1f, 0xa38, 0xa47, 0xa30, 0xa3e, 0xa24, 0xa28, 0xa3e, 0xa38, 0xa3e,
0xa13, 0xa28, 0xa3f, 0xa0a, 0x20, 0xa2f, 0xa3e, 0xa30, 0xa15, 0xa28, 0xa4c, 0xa30,
0xa4b, 0xa28, 0xa39, 0xa3e, 0xa2c, 0xa3f, 0xa09, 0xa32, 0xa3e, 0x2c, 0x20, 0xa09,
0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa21, 0xa15, 0xa4b, 0xa1f, 0xa3e, 0xa38, 0xa47,
0xa02, 0xa1f, 0xa30, 0x2c, 0x20, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa21,
0xa15, 0xa4b, 0xa1f, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0x20, 0xa38, 0xa32, 0xa47, 0xa2e,
0x2c, 0x20, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa21, 0xa15, 0xa4b, 0xa1f,
0xa3e, 0xa28, 0xa42, 0xa15, 0xa13, 0xa1c, 0xa40, 0xa28, 0xa3e, 0xa17, 0xa3e, 0xa2a,
0xa28, 0xa3e, 0xa2e, 0xa3e, 0xa2a, 0xa48, 0xa30, 0xa3e, 0xa2e, 0xa30, 0xa40, 0xa2c,
0xa4b, 0xa2b, 0xa3f, 0xa28, 0xa3f, 0xa15, 0xa38, 0xa2a, 0xa4b, 0xa30, 0xa1f, 0x2d,
0xa14, 0x2d, 0xa2a, 0xa4d, 0xa30, 0xa3f, 0xa70, 0xa38, 0xa2a, 0xa4b, 0xa30, 0xa1f,
0x20, 0xa14, 0xa2b, 0x20, 0xa38, 0xa2a, 0xa47, 0xa28, 0xa2a, 0xa4b, 0xa30, 0xa1f,
0xa4b, 0x20, 0xa35, 0xa47, 0xa32, 0xa4d, 0xa39, 0xa4b, 0xa2a, 0xa3f, 0xa0a, 0xa30,
0xa1f, 0xa4b, 0x20, 0xa30, 0xa40, 0xa15, 0xa4b, 0xa2a, 0xa70, 0xa1f, 0xa3e, 0x20,
0xa05, 0xa30, 0xa47, 0xa28, 0xa38, 0xa30, 0xa48, 0xa02, 0xa15, 0xa3f, 0xa28, 0x20,
0xa07, 0xa28, 0xa32, 0xa48, 0xa1f, 0xa30, 0xa47, 0xa38, 0xa40, 0xa2b, 0xa47, 0xa30,
0xa48, 0xa1c, 0xa40, 0xa28, 0xa3e, 0xa30, 0xa48, 0xa1c, 0xa3c, 0xa4b, 0xa32, 0xa3f,
0xa0a, 0xa1f, 0xa30, 0xa40, 0xa13, 0x20, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa02, 0xa15,
0xa4b, 0xa38, 0xa47, 0xa02, 0xa1f, 0xa3e, 0xa30, 0xa2e, 0xa38, 0xa47, 0xa02, 0xa1f,
0xa40, 0xa06, 0xa17, 0xa4b, 0xa38, 0xa47, 0xa02, 0xa1f, 0xa4b, 0x20, 0xa21, 0xa4b,
0xa2e, 0xa3f, 0xa70, 0xa17, 0xa4b, 0xa38, 0xa3e, 0xa13, 0x20, 0xa2a, 0xa4c, 0xa32,
0xa4b, 0xa07, 0xa71, 0xa1f, 0xa4b, 0xa15, 0xa4b, 0xa30, 0xa1f, 0xa42, 0xa30, 0xa2e,
0xa40, 0xa1f, 0xa38, 0xa3f, 0xa1f, 0xa15, 0xa3e, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20,
0xa2c, 0xa3e, 0xa30, 0xa25, 0xa47, 0xa32, 0xa47, 0xa2e, 0xa40, 0xa38, 0xa47, 0xa02,
0xa1f, 0x20, 0xa1c, 0xa4c, 0xa39, 0xa28, 0xa38, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20,
0xa15, 0xa3f, 0xa1f, 0xa38, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa32, 0xa41, 0xa38,
0xa40, 0xa06, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa25, 0xa4b, 0xa2e, 0xa38, 0xa38,
0xa47, 0xa02, 0xa1f, 0x20, 0xa35, 0xa3f, 0xa28, 0xa38, 0xa47, 0xa02, 0xa1f, 0xa38,
0xa35, 0xa3f, 0xa2b, 0xa1f, 0x20, 0xa15, 0xa30, 0xa70, 0xa1f, 0xa1f, 0xa47, 0xa17,
0xa41, 0xa38, 0xa40, 0xa17, 0xa32, 0xa2a, 0xa3e, 0xa25, 0xa41, 0xa32, 0xa47, 0xa1f,
0xa3f, 0xa1c, 0xa42, 0xa06, 0xa28, 0xa3e, 0xa1f, 0xa4b, 0xa30, 0xa3e, 0xa02, 0xa1f,
0xa4b, 0xa1f, 0xa4b, 0xa30, 0xa1f, 0xa4b, 0xa32, 0xa3e, 0xa35, 0xa48, 0xa28, 0xa15,
0xa42, 0xa35, 0xa30, 0xa35, 0xa3e, 0xa08, 0xa1f, 0xa39, 0xa4c, 0xa30, 0xa38, 0xa35,
0xa3f, 0xa28, 0xa40, 0xa2a, 0xa48, 0xa17, 0xa2f, 0xa15, 0xa41, 0xa24, 0xa24, 0xa21,
0xa47, 0xa35, 0xa3f, 0xa38, 0xa21, 0xa3f, 0xa09, 0xa2e, 0xa4b, 0xa02, 0xa1f, 0x20,
0xa21, 0xa3f, 0xa09, 0xa30, 0xa35, 0xa3f, 0xa32, 0xa47, 0xa2e, 0xa48, 0xa15, 0xa15,
0xa35, 0xa48, 0xa30, 0xa40, 0xa2e, 0xa4c, 0xa38, 0xa28, 0xa2e, 0xa48, 0xa15, 0xa2e,
0xa41, 0xa30, 0xa21, 0xa4b, 0xa2a, 0xa3e, 0xa2e, 0xa30, 0xa30, 0xa4b, 0xa25, 0xa47,
0xa30, 0xa3e, 0xa38, 0xa35, 0xa4b, 0xa2f, 0xa3e, 0xa1f, 0xa30, 0xa4b, 0xa32, 0xa35,
0xa4b, 0xa38, 0xa1f, 0xa4b, 0xa15, 0xa32, 0xa4b, 0xa02, 0xa17, 0xa08, 0xa05, 0xa30,
0xa2c, 0xa3e, 0xa07, 0xa28, 0xa05, 0xa21, 0xa47, 0xa28, 0xa05, 0xa32, 0xa2e, 0xa47,
0xa1f, 0xa40, 0xa05, 0xa2e, 0xa3e, 0xa28, 0xa10, 0xa28, 0xa3e, 0xa21, 0xa3e, 0xa07,
0xa30, 0xa05, 0xa15, 0xa24, 0xa4c, 0xa05, 0xa15, 0xa24, 0xa4b, 0xa2c, 0xa47, 0xa05,
0xa38, 0xa3c, 0xa17, 0xa3e, 0xa2c, 0xa3e, 0xa1f, 0xa0f, 0xa24, 0xa47, 0xa30, 0xa3e,
0xa13, 0xa2c, 0xa17, 0xa26, 0xa3e, 0xa26, 0xa2c, 0xa39, 0xa3f, 0xa30, 0xa40, 0xa28,
0xa2c, 0xa3e, 0xa15, 0xa42, 0xa2c, 0xa48, 0xa02, 0xa15, 0xa3e, 0xa15, 0xa2c, 0xa30,
0xa28, 0xa4c, 0xa32, 0xa2c, 0xa48, 0xa30, 0xa42, 0xa24, 0xa2c, 0xa3f, 0xa38, 0xa3c,
0xa15, 0xa47, 0xa15, 0xa2c, 0xa30, 0xa42, 0xa28, 0xa47, 0xa08, 0xa1a, 0xa3f, 0xa24,
0xa3e, 0xa15, 0xa4b, 0xa32, 0xa70, 0xa2c, 0xa4b, 0xa21, 0xa48, 0xa2e, 0xa38, 0xa15,
0xa38, 0xa22, 0xa3e, 0xa15, 0xa3e, 0xa21, 0xa3f, 0xa32, 0xa40, 0xa26, 0xa41, 0xa2c,
0xa08, 0xa26, 0xa41, 0xa38, 0xa3c, 0xa3e, 0xa02, 0xa2c, 0xa47, 0xa2b, 0xa3e, 0xa2e,
0xa3e, 0xa17, 0xa41, 0xa38, 0xa1f, 0xa3e, 0xa17, 0xa3e, 0xa1c, 0xa3c, 0xa3e, 0xa39,
0xa47, 0xa2c, 0xa30, 0xa3e, 0xa28, 0xa39, 0xa4b, 0x20, 0xa1a, 0xa40, 0x20, 0xa2e,
0xa3f, 0xa28, 0xa4d, 0xa39, 0x20, 0xa38, 0xa3f, 0xa1f, 0xa40, 0xa39, 0xa3e, 0xa02,
0xa17, 0x20, 0xa15, 0xa3e, 0xa02, 0xa17, 0xa39, 0xa4b, 0xa35, 0xa21, 0xa07, 0xa30,
0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0xa1c, 0xa15, 0xa3e, 0xa30, 0xa24, 0xa3e, 0xa1c,
0xa2f, 0xa3e, 0xa2a, 0xa41, 0xa30, 0xa3e, 0xa1c, 0xa47, 0xa30, 0xa42, 0xa38, 0xa32,
0xa2e, 0xa15, 0xa3e, 0xa2c, 0xa41, 0xa32, 0xa15, 0xa2e, 0xa1a, 0xa1f, 0xa15, 0xa3e,
0xa15, 0xa30, 0xa3e, 0xa1a, 0xa40, 0xa15, 0xa3e, 0xa20, 0xa2e, 0xa3e, 0xa02, 0xa21,
0xa42, 0xa16, 0xa3e, 0xa28, 0xa21, 0xa3f, 0xa17, 0xa3e, 0xa15, 0xa4b, 0xa32, 0xa15,
0xa3e, 0xa24, 0xa3e, 0xa15, 0xa30, 0xa48, 0xa38, 0xa28, 0xa47, 0xa1c, 0xa30, 0xa38,
0xa15, 0xa41, 0xa06, 0xa32, 0xa3e, 0xa32, 0xa70, 0xa2a, 0xa41, 0xa30, 0xa15, 0xa41,
0xa1a, 0xa3f, 0xa70, 0xa17, 0xa15, 0xa41, 0xa35, 0xa48, 0xa24, 0xa2e, 0xa15, 0xa3e,
0xa09, 0xa2e, 0xa48, 0xa17, 0xa47, 0xa21, 0xa28, 0xa2e, 0xa15, 0xa38, 0xa3e, 0xa30,
0xa2e, 0xa28, 0xa40, 0xa32, 0xa3e, 0xa2e, 0xa38, 0xa15, 0xa1f, 0xa28, 0xa3f, 0xa15,
0xa4b, 0xa38, 0xa40, 0xa06, 0xa28, 0xa4b, 0xa35, 0xa4b, 0xa15, 0xa41, 0xa1c, 0xa3c,
0xa28, 0xa47, 0xa1f, 0xa38, 0xa15, 0xa28, 0xa4b, 0xa35, 0xa4b, 0xa38, 0xa3f, 0xa2c,
0xa40, 0xa30, 0xa38, 0xa15, 0xa13, 0xa2e, 0xa38, 0xa15, 0xa13, 0xa30, 0xa32, 0xa2b,
0xa28, 0xa4b, 0xa2e, 0x20, 0xa2a, 0xa47, 0xa28, 0xa39, 0xa2a, 0xa4c, 0xa02, 0xa1f,
0xa40, 0xa06, 0xa28, 0xa3e, 0xa15, 0xa2a, 0xa2f, 0xa4b, 0xa02, 0xa17, 0xa2f, 0xa3e,
0xa02, 0xa17, 0xa15, 0xa24, 0xa30, 0xa15, 0xa4b, 0xa38, 0xa24, 0xa3e, 0xa28, 0xa3e,
0xa0f, 0xa15, 0xa3f, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa4b, 0xa30, 0xa21, 0xa3e, 0xa30,
0xa3f, 0xa06, 0xa27, 0xa38, 0xa16, 0xa32, 0xa40, 0xa28, 0xa38, 0xa2e, 0xa30, 0xa15,
0xa70, 0xa26, 0xa38, 0xa3f, 0xa09, 0xa32, 0xa38, 0xa3c, 0xa70, 0xa18, 0xa3e, 0xa08,
0xa38, 0xa3f, 0xa70, 0xa17, 0xa3e, 0xa2a, 0xa41, 0xa30, 0xa38, 0xa30, 0xa3f, 0xa21,
0xa28, 0xa40, 0xa15, 0xa4b, 0xa32, 0xa3f, 0xa38, 0xa15, 0xa24, 0xa48, 0xa2a, 0xa08,
0xa24, 0xa3e, 0xa38, 0xa3c, 0xa15, 0xa70, 0xa26, 0xa1f, 0xa2c, 0xa3f, 0xa32, 0xa3f,
0xa38, 0xa40, 0xa24, 0xa47, 0xa39, 0xa30, 0xa3e, 0xa28, 0xa25, 0xa3f, 0xa70, 0xa2b,
0xa42, 0xa1f, 0xa4b, 0xa15, 0xa40, 0xa13, 0xa1f, 0xa4b, 0xa2e, 0xa38, 0xa15, 0xa09,
0xa32, 0xa3e, 0xa28, 0xa2c, 0xa3e, 0xa24, 0xa30, 0xa0a, 0xa30, 0xa42, 0xa2e, 0xa15,
0xa40, 0xa09, 0xa38, 0xa24, 0x2d, 0xa28, 0xa47, 0xa30, 0xa3e, 0xa35, 0xa3e, 0xa0f,
0xa28, 0xa1f, 0xa3f, 0xa06, 0xa28, 0xa47, 0xa35, 0xa32, 0xa3e, 0xa26, 0xa40, 0xa35,
0xa4b, 0xa38, 0xa24, 0xa15, 0xa2f, 0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0xa30, 0xa70,
0xa17, 0xa42, 0xa28, 0xa2f, 0xa15, 0xa47, 0xa24, 0xa30, 0xa3f, 0xa28, 0xa2c, 0xa30,
0xa17, 0xa2f, 0xa47, 0xa30, 0xa47, 0xa35, 0xa28, 0xa05, 0xa1c, 0xa4b, 0xa30, 0xa47,
0xa38, 0xa2c, 0xa30, 0xa2e, 0xa42, 0xa21, 0xa3e, 0xa15, 0xa47, 0xa28, 0xa47, 0xa30,
0xa40, 0xa15, 0xa47, 0xa2a, 0x20, 0xa35, 0xa30, 0xa21, 0xa2b, 0xa48, 0xa30, 0xa4b,
0xa2e, 0xa21, 0xa40, 0xa05, 0xa30, 0xa3e, 0xa30, 0xa47, 0xa15, 0xa1c, 0xa3e, 0xa35,
0xa3f, 0xa15, 0xa26, 0xa71, 0xa16, 0xa23, 0xa40, 0x20, 0xa1c, 0xa3e, 0xa30, 0xa1c,
0xa40, 0xa06, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa39, 0xa47, 0xa32, 0xa47, 0xa28,
0xa3e, 0xa38, 0xa1f, 0xa47, 0xa28, 0xa32, 0xa40, 0xa10, 0xa21, 0xa40, 0xa32, 0xa47,
0xa21, 0xa2c, 0xa4d, 0xa30, 0xa3f, 0xa38, 0xa2c, 0xa47, 0xa28, 0xa2c, 0xa4d, 0xa30,
0xa4b, 0xa15, 0xa28, 0x20, 0xa39, 0xa3f, 0xa32, 0xa21, 0xa3e, 0xa30, 0xa35, 0xa3f,
0xa28, 0xa2f, 0xa42, 0xa15, 0xa32, 0xa3e, 0xa39, 0xa4b, 0xa2c, 0xa3e, 0xa30, 0xa1f,
0xa32, 0xa3f, 0xa70, 0xa21, 0xa47, 0xa2e, 0xa28, 0xa32, 0xa4c, 0xa30, 0xa21, 0x20,
0xa39, 0xa4b, 0xa35, 0xa47, 0xa2e, 0xa48, 0xa32, 0xa2c, 0xa4b, 0xa30, 0xa28, 0xa2a,
0xa30, 0xa25, 0xa38, 0xa3f, 0xa21, 0xa28, 0xa40, 0xa10, 0xa2e, 0xa38, 0xa1f, 0xa30,
0xa21, 0xa2e, 0xa05, 0xa70, 0xa21, 0xa4b, 0xa30, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d,
0xa30, 0xa3e, 0xa16, 0xa3e, 0xa28, 0xa0f, 0xa25, 0xa28, 0xa38, 0xa2c, 0xa48, 0xa32,
0xa17, 0xa4d, 0xa30, 0xa47, 0xa21, 0xa2c, 0xa30, 0xa32, 0xa3f, 0xa28, 0xa2c, 0xa4d,
0xa30, 0xa3e, 0xa1f, 0xa3f, 0xa38, 0xa32, 0xa3e, 0xa35, 0xa3e, 0xa2c, 0xa30, 0xa71,
0xa38, 0xa32, 0xa1c, 0xa2c, 0xa42, 0xa15, 0xa3e, 0xa30, 0xa48, 0xa38, 0xa1f, 0xa2c,
0xa41, 0xa21, 0xa3e, 0xa2a, 0xa48, 0xa38, 0xa1f, 0xa2c, 0xa41, 0xa38, 0xa3f, 0xa70,
0xa1c, 0xa47, 0xa28, 0xa1a, 0xa3f, 0xa38, 0xa3f, 0xa28, 0xa4c, 0xa15, 0xa4b, 0xa2a,
0xa28, 0xa39, 0xa47, 0xa17, 0xa28, 0xa21, 0xa2c, 0xa32, 0xa3f, 0xa28, 0xa1c, 0xa3f,
0xa2c, 0xa30, 0xa3e, 0xa32, 0xa1f, 0xa30, 0xa17, 0xa30, 0xa28, 0xa1c, 0xa40, 0xa39,
0xa47, 0xa32, 0xa38, 0xa3f, 0xa70, 0xa15, 0xa40, 0xa06, 0xa07, 0xa32, 0x20, 0xa06,
0xa2b, 0x20, 0xa2e, 0xa48, 0xa28, 0xa07, 0xa38, 0xa24, 0xa3e, 0xa02, 0xa2c, 0xa41,
0xa32, 0xa1c, 0xa30, 0xa38, 0xa40, 0xa15, 0xa32, 0xa40, 0xa28, 0xa3f, 0xa70, 0xa17,
0xa4d, 0xa30, 0xa47, 0xa21, 0xa15, 0xa40, 0xa30, 0xa4b, 0xa35, 0xa15, 0xa40, 0xa35,
0xa32, 0xa3f, 0xa38, 0xa2c, 0xa28, 0xa32, 0xa1c, 0xa41, 0xa2c, 0xa32, 0xa1c, 0xa3e,
0xa28, 0xa3e, 0xa32, 0xa70, 0xa21, 0xa28, 0xa32, 0xa15, 0xa1c, 0xa3c, 0xa2e, 0xa2c,
0xa30, 0xa17, 0xa2e, 0xa48, 0xa21, 0xa4d, 0xa30, 0xa3f, 0xa21, 0xa2e, 0xa3e, 0xa32,
0xa1f, 0xa3e, 0xa2e, 0xa3e, 0xa30, 0xa40, 0xa0f, 0xa39, 0xa3e, 0xa2e, 0xa47, 0xa28,
0xa2e, 0xa3f, 0xa70, 0xa38, 0xa15, 0xa2e, 0xa4b, 0xa28, 0xa3e, 0xa15, 0xa4b, 0xa2e,
0xa3e, 0xa38, 0xa15, 0xa4b, 0xa13, 0xa38, 0xa32, 0xa4b, 0xa2a, 0xa48, 0xa30, 0xa3f,
0xa38, 0xa2a, 0xa4b, 0xa21, 0xa17, 0xa4b, 0xa30, 0xa40, 0xa15, 0xa3e, 0xa2a, 0xa4d,
0xa30, 0xa3e, 0xa17, 0xa30, 0xa3f, 0xa17, 0xa3e, 0xa38, 0xa48, 0xa28, 0x20, 0xa2e,
0xa30, 0xa40, 0xa28, 0xa4b, 0xa38, 0xa3e, 0xa30, 0xa3e, 0xa1c, 0xa47, 0xa35, 0xa4b,
0xa38, 0xa48, 0xa30, 0xa3e, 0xa1f, 0xa4b, 0xa35, 0xa38, 0xa3f, 0xa2e, 0xa2b, 0xa30,
0xa4b, 0xa2a, 0xa4b, 0xa32, 0xa38, 0xa15, 0xa4b, 0xa2a, 0xa1c, 0xa47, 0xa38, 0xa4b,
0xa2b, 0xa40, 0xa06, 0xa38, 0xa1f, 0xa3e, 0xa15, 0xa39, 0xa4b, 0xa2e, 0xa1f, 0xa71,
0xa32, 0xa3f, 0xa28, 0xa24, 0xa3f, 0xa30, 0xa3e, 0xa28, 0xa47, 0xa2f, 0xa41, 0xa32,
0xa3f, 0xa06, 0xa28, 0xa4b, 0xa38, 0xa15, 0xa35, 0xa3e, 0xa21, 0xa41, 0xa1c, 0xa3c,
0xa35, 0xa48, 0xa1f, 0xa3f, 0xa15, 0xa28, 0xa35, 0xa3f, 0xa06, 0xa28, 0xa3e, 0xa35,
0xa3f, 0xa32, 0xa28, 0xa3f, 0xa05, 0xa38, 0xa35, 0xa4b, 0xa32, 0xa17, 0xa4b, 0xa17,
0xa4d, 0xa30, 0xa47, 0xa21, 0xa35, 0xa3e, 0xa30, 0xa38, 0xa3e, 0xa05, 0xa1c, 0xa3c,
0xa17, 0xa30, 0xa47, 0xa2c, 0xa1c, 0xa3f, 0xa0a, 0xa30, 0xa3f, 0xa16, 0xa05, 0xa70,
0xa1f, 0xa3e, 0xa28, 0xa28, 0xa47, 0xa30, 0xa3f, 0xa35, 0xa4b, 0xa1a, 0xa3e, 0xa17,
0xa4b, 0xa38, 0xa15, 0xa4d, 0xa30, 0xa3f, 0xa38, 0xa2e, 0xa38, 0xa15, 0xa4b, 0xa15,
0xa4b, 0xa1c, 0xa3c, 0xa15, 0xa4b, 0xa2e, 0xa4b, 0xa30, 0xa4b, 0xa15, 0xa30, 0xa17,
0xa2f, 0xa42, 0xa32, 0xa47, 0xa28, 0xa2e, 0xa3e, 0xa39, 0xa47, 0xa2e, 0xa3e, 0xa32,
0xa26, 0xa40, 0xa35, 0xa2e, 0xa4c, 0xa30, 0xa3f, 0xa38, 0xa3c, 0xa38, 0xa2e, 0xa3e,
0xa2f, 0xa4b, 0xa1f, 0xa40, 0xa30, 0xa3f, 0xa2f, 0xa42, 0xa28, 0xa40, 0xa05, 0xa28,
0xa10, 0xa2a, 0xa40, 0xa06, 0xa06, 0xa15, 0xa32, 0xa48, 0xa02, 0xa21, 0xa2c, 0xa4b,
0xa17, 0xa28, 0xa35, 0xa3f, 0xa32, 0xa47, 0xa1a, 0xa48, 0xa25, 0xa2e, 0xa1a, 0xa42,
0xa15, 0xa08, 0xa38, 0xa1f, 0xa30, 0xa07, 0xa2b, 0xa47, 0xa1f, 0xa2b, 0xa15, 0xa3e,
0xa09, 0xa2b, 0xa4b, 0xa2b, 0xa3c, 0xa3f, 0xa1c, 0xa40, 0xa2b, 0xa41, 0xa28, 0xa3e,
0xa2b, 0xa41, 0xa1f, 0xa40, 0xa17, 0xa32, 0xa2a, 0xa47, 0xa17, 0xa4b, 0xa38, 0xa17,
0xa48, 0xa02, 0xa2c, 0xa40, 0xa05, 0xa30, 0xa17, 0xa41, 0xa06, 0xa21, 0xa3e, 0xa15,
0xa47, 0xa28, 0xa32, 0xa39, 0xa4b, 0xa28, 0xa4b, 0xa32, 0xa41, 0xa32, 0xa42, 0xa0f,
0xa02, 0xa21, 0xa30, 0xa2c, 0xa30, 0xa40, 0xa15, 0xa3f, 0xa30, 0xa3f, 0xa24, 0xa3f,
0xa2e, 0xa24, 0xa40, 0xa15, 0xa4b, 0xa38, 0xa4d, 0xa30, 0xa3e, 0xa0f, 0xa15, 0xa35,
0xa3e, 0xa1c, 0xa3e, 0xa32, 0xa40, 0xa28, 0xa2e, 0xa47, 0xa1c, 0xa41, 0xa30, 0xa4b,
0xa2e, 0xa3e, 0xa30, 0xa15, 0xa3f, 0xa38, 0xa3e, 0xa38, 0xa2e, 0xa3f, 0xa21, 0xa35,
0xa47, 0xa28, 0xa3e, 0xa09, 0xa30, 0xa42, 0xa28, 0xa3f, 0xa2f, 0xa42, 0xa28, 0xa4b,
0xa30, 0xa2b, 0xa4c, 0xa15, 0xa28, 0xa4c, 0xa2e, 0xa3f, 0xa06, 0xa2a, 0xa3e, 0xa17,
0xa4b, 0x20, 0xa2a, 0xa3e, 0xa17, 0xa4b, 0xa2a, 0xa32, 0xa3e, 0xa09, 0xa2a, 0xa3f,
0xa1f, 0xa15, 0xa48, 0xa30, 0xa28, 0xa2a, 0xa4b, 0xa39, 0xa28, 0xa2a, 0xa47, 0xa08,
0xa2a, 0xa4b, 0xa30, 0xa1f, 0x20, 0xa2e, 0xa4b, 0xa30, 0xa47, 0xa38, 0xa2c, 0xa40,
0xa30, 0xa3e, 0xa30, 0xa4b, 0xa1f, 0xa4b, 0xa02, 0xa17, 0xa3e, 0xa38, 0xa48, 0xa2a,
0xa3e, 0xa28, 0xa24, 0xa39, 0xa3f, 0xa24, 0xa40, 0xa1f, 0xa3e, 0xa30, 0xa3e, 0xa35,
0xa3e, 0xa1f, 0xa4b, 0xa02, 0xa17, 0xa3e, 0xa1f, 0xa3e, 0xa2a, 0xa42, 0xa35, 0xa47,
0xa15, 0xa35, 0xa3e, 0xa32, 0xa3f, 0xa38, 0x41, 0x62, 0x69, 0x79, 0xe1, 0x6e,
0x45, 0x6c, 0x20, 0x43, 0x61, 0x69, 0x72, 0x6f, 0x4a, 0x6f, 0x68, 0x61,
0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x54, 0x75, 0x6e,
0x65, 0x7a, 0x43, 0x75, 0x72, 0x61, 0x7a, 0x61, 0x6f, 0x43, 0x69, 0x75,
0x64, 0x61, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x6f, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x45, 0x73, 0x70, 0x61,
0xf1, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c,
0x6f, 0x6d, 0xe9, 0x53, 0x61, 0x6e, 0x20, 0x4a, 0x75, 0x61, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61,
0x53, 0x61, 0x6e, 0x20, 0x43, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x62, 0x61,
0x6c, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x73,
0x59, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61, 0x50, 0x69, 0x6f, 0x6e, 0x79,
0x61, 0x6e, 0x67, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0xe1, 0x69, 0x53,
0x68, 0x61, 0x6e, 0x67, 0xe1, 0x69, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x42, 0x72, 0x75,
0x73, 0x65, 0x6c, 0x61, 0x73, 0x45, 0x6c, 0x20, 0x56, 0x61, 0x74, 0x69,
0x63, 0x61, 0x6e, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64,
0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x64, 0x41, 0x219, 0x67, 0x61, 0x62, 0x61, 0x74, 0x41, 0x74, 0xee, 0x72,
0x61, 0x75, 0x42, 0x69, 0x219, 0x6b, 0x65, 0x6b, 0x44, 0x75, 0x219, 0x61,
0x6e, 0x62, 0x65, 0x48, 0x6f, 0x20, 0x218, 0x69, 0x20, 0x4d, 0x69, 0x6e,
0x49, 0x72, 0x6b, 0x75, 0x21b, 0x6b, 0x49, 0x65, 0x72, 0x75, 0x73, 0x61,
0x6c, 0x69, 0x6d, 0x4b, 0x61, 0x6d, 0x63, 0x69, 0x61, 0x74, 0x6b, 0x61,
0x4b, 0x75, 0x77, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75,
0x7a, 0x6e, 0x65, 0x21b, 0x6b, 0x50, 0x68, 0x65, 0x6e, 0x69, 0x61, 0x6e,
0x54, 0x61, 0x219, 0x6b, 0x65, 0x6e, 0x74, 0x49, 0x61, 0x6b, 0x75, 0x21b,
0x6b, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x65, 0x43, 0x61, 0x70, 0x75, 0x6c,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75, 0x64, 0x53, 0x66, 0x2e, 0x20,
0x45, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x73,
0x74, 0x61, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4d, 0x61, 0x6e,
0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x21b, 0x61, 0x56, 0x61, 0x72,
0x219, 0x6f, 0x76, 0x69, 0x61, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x50, 0x61, 0x219, 0x74, 0x65, 0x6c, 0x75, 0x69, 0x4d, 0x61, 0x72, 0x63,
0x68, 0x69, 0x7a, 0x65, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x41, 0x6c, 0x61, 0x73, 0x63,
0x61, 0x54, 0x68, 0x65, 0x20, 0x56, 0x61, 0x6c, 0x6c, 0x65, 0x79, 0x53,
0x61, 0x6e, 0x20, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x20, 0x4a, 0x75, 0x6a, 0x75, 0x79, 0x53, 0x61, 0x6e, 0x20,
0x4a, 0x75, 0x61, 0x6e, 0x2c, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x6e, 0x6a, 0x65, 0x73, 0x74,
0x61, 0x64, 0x42, 0x65, 0x6c, 0x6d, 0x6f, 0x70, 0x61, 0x6e, 0x49, 0x6e,
0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0x43,
0x6f, 0x63, 0x6b, 0x62, 0x75, 0x72, 0x6e, 0x20, 0x54, 0x6f, 0x77, 0x6e,
0x42, 0x61, 0x73, 0x73, 0x65, 0x2d, 0x54, 0x65, 0x72, 0x72, 0x65, 0x43,
0x69, 0x74, 0x61, 0x64, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x6f, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x42, 0x72, 0x61, 0x64, 0x65, 0x73, 0x4e, 0x6f, 0x72, 0x74,
0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x28, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x29, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x28, 0x4e, 0x65, 0x77, 0x20,
0x53, 0x61, 0x6c, 0x65, 0x6d, 0x29, 0x52, 0x6f, 0x61, 0x64, 0x20, 0x54,
0x6f, 0x77, 0x6e, 0x4d, 0x61, 0x63, 0x20, 0x4d, 0x75, 0x72, 0x64, 0x6f,
0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x65, 0x72, 0x69, 0x20,
0x42, 0x65, 0x67, 0x61, 0x77, 0x61, 0x6e, 0x43, 0x69, 0x74, 0x61, 0x64,
0x20, 0x64, 0x61, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d,
0x69, 0x6e, 0x68, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x73, 0x49, 0x6e, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x61, 0x73,
0x49, 0x6e, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6f, 0x65,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x61, 0x6c, 0x20,
0x53, 0x69, 0x64, 0x53, 0x6f, 0x6e, 0x74, 0x67, 0x2019, 0x65, 0x6c, 0x65,
0x6e, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x6c,
0x65, 0x79, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x65, 0x74, 0x65,
0x72, 0x20, 0x50, 0x6f, 0x72, 0x74, 0x44, 0x6f, 0x75, 0x67, 0x6c, 0x61,
0x73, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x69, 0x65,
0x72, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x75, 0x54, 0x75, 0x72, 0x69, 0x74,
0x67, 0x46, 0x6c, 0x79, 0x69, 0x6e, 0x67, 0x20, 0x46, 0x69, 0x73, 0x68,
0x20, 0x43, 0x6f, 0x76, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x61, 0x73, 0x57,
0x65, 0x6e, 0x6f, 0x49, 0x6e, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x61, 0x20,
0x50, 0x61, 0x73, 0x63, 0x61, 0x52, 0x69, 0x6b, 0x69, 0x74, 0x65, 0x61,
0x48, 0x6f, 0x6e, 0x69, 0x61, 0x72, 0x61, 0x48, 0x61, 0x67, 0xe5, 0x74,
0xf1, 0x61, 0x54, 0x6f, 0x66, 0x6f, 0x6c, 0x59, 0x61, 0x72, 0x65, 0x6e,
0x41, 0x6c, 0x6f, 0x66, 0x69, 0x4b, 0x69, 0x6e, 0x67, 0x73, 0x74, 0x6f,
0x6e, 0x4d, 0x65, 0x6c, 0x65, 0x6b, 0x6f, 0x6b, 0x50, 0x61, 0x6c, 0x69,
0x6b, 0x69, 0x72, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x54, 0x61, 0x72,
0x61, 0x77, 0x61, 0x4e, 0x75, 0x6b, 0x75, 0x2bb, 0x61, 0x6c, 0x6f, 0x66,
0x61, 0x4d, 0x61, 0x74, 0x101, 0x2bb, 0x75, 0x74, 0x75, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x411,
0x43e, 0x439, 0x43b, 0x430, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x430, 0x44f, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x426, 0x435, 0x43d,
0x442, 0x440, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e, 0x2d, 0x421,
0x435, 0x439, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x430, 0x44f, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41f, 0x435,
0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d,
0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x43e, 0x2d,
0x432, 0x20, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x410, 0x437, 0x43e,
0x440, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x411, 0x435,
0x440, 0x43c, 0x443, 0x434, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432,
0x430, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e,
0x2d, 0x432, 0x430, 0x424, 0x430, 0x440, 0x435, 0x440, 0x441, 0x43a, 0x438, 0x435,
0x20, 0x43e, 0x2d, 0x432, 0x430, 0x42e, 0x436, 0x43d, 0x430, 0x44f, 0x20, 0x413,
0x435, 0x43e, 0x440, 0x433, 0x438, 0x44f, 0x43e, 0x2d, 0x432, 0x20, 0x421, 0x432,
0x44f, 0x442, 0x43e, 0x439, 0x20, 0x415, 0x43b, 0x435, 0x43d, 0x44b, 0x411, 0x44e,
0x437, 0x438, 0x43d, 0x433, 0x435, 0x43d, 0x2d, 0x43d, 0x430, 0x2d, 0x412, 0x435,
0x440, 0x445, 0x43d, 0x435, 0x43c, 0x2d, 0x420, 0x435, 0x439, 0x43d, 0x435, 0x43e,
0x2d, 0x432, 0x20, 0x41c, 0x44d, 0x43d, 0x43e, 0x2d, 0x432, 0x20, 0x420, 0x43e,
0x436, 0x434, 0x435, 0x441, 0x442, 0x432, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441,
0x43e, 0x432, 0x44b, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41a, 0x43e, 0x43c,
0x43e, 0x440, 0x44b, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x44b, 0x43e,
0x2d, 0x432, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x413, 0x430, 0x43b, 0x430,
0x43f, 0x430, 0x433, 0x43e, 0x441, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d,
0x432, 0x430, 0x43e, 0x2d, 0x432, 0x430, 0x20, 0x413, 0x430, 0x43c, 0x431, 0x44c,
0x435, 0x43e, 0x2d, 0x432, 0x20, 0x42d, 0x43d, 0x434, 0x435, 0x440, 0x431, 0x435,
0x440, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x441, 0x43a, 0x438, 0x435,
0x20, 0x43e, 0x2d, 0x432, 0x430, 0x43e, 0x2d, 0x432, 0x430, 0x20, 0x41c, 0x438,
0x434, 0x443, 0x44d, 0x439, 0x410, 0x43d, 0x430, 0x430, 0x434, 0x44b, 0x440, 0x410,
0x441, 0x445, 0x430, 0x431, 0x430, 0x430, 0x442, 0x411, 0x430, 0x433, 0x434, 0x430,
0x430, 0x434, 0x411, 0x430, 0x43a, 0x443, 0x443, 0x427, 0x438, 0x442, 0x430, 0x430,
0x414, 0x430, 0x43c, 0x430, 0x430, 0x441, 0x414, 0x443, 0x431, 0x430, 0x430, 0x439,
0x425, 0x43e, 0x20, 0x428, 0x438, 0x20, 0x41c, 0x438, 0x43d, 0x44c, 0x418, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x430, 0x439, 0x41a, 0x430, 0x431, 0x443, 0x443,
0x43b, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x430, 0x442, 0x43a, 0x430, 0x425, 0x430,
0x430, 0x43d, 0x434, 0x44b, 0x433, 0x430, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x44f, 0x440, 0x441, 0x43a, 0x430, 0x439, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430,
0x430, 0x43d, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441,
0x43a, 0x430, 0x439, 0x41e, 0x43c, 0x441, 0x43a, 0x430, 0x439, 0x421, 0x430, 0x445,
0x430, 0x43b, 0x438, 0x438, 0x43d, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x43a, 0x430,
0x430, 0x43d, 0x41e, 0x440, 0x442, 0x43e, 0x20, 0x425, 0x430, 0x43b, 0x44b, 0x43c,
0x430, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x430, 0x442, 0x430, 0x440,
0x423, 0x440, 0x443, 0x43c, 0x447, 0x443, 0x423, 0x443, 0x441, 0x20, 0x41d, 0x44c,
0x430, 0x440, 0x430, 0x414, 0x44c, 0x43e, 0x43a, 0x443, 0x443, 0x441, 0x43a, 0x430,
0x439, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x443,
0x440, 0x410, 0x430, 0x441, 0x442, 0x440, 0x430, 0x445, 0x430, 0x43d, 0x421, 0x442,
0x430, 0x43c, 0x431, 0x443, 0x443, 0x43b, 0x41c, 0x43e, 0x441, 0x43a, 0x443, 0x431,
0x430, 0x421, 0x430, 0x43c, 0x430, 0x430, 0x440, 0x430, 0x423, 0x43b, 0x44c, 0x44f,
0x43d, 0x43e, 0x432, 0x441, 0x43a, 0x430, 0x439, 0x1c5f, 0x2e, 0x1c75, 0x1c64, 0x1c70,
0x1c61, 0x1c5f, 0x1c71, 0x1c5f, 0x1c70, 0x1c5f, 0x1c60, 0x1c5f, 0x1c71, 0x1c60, 0x1c77, 0x1c5a,
0x1c68, 0x1c6e, 0x1c61, 0x1c7d, 0x1c5f, 0x1c71, 0x1c5c, 0x1c69, 0x1c6d, 0x1c5e, 0x1c5f, 0x1c5f,
0x1c71, 0x1c74, 0x1c64, 0x1c5c, 0x1c69, 0x1c63, 0x1c5f, 0x1c5f, 0x1c68, 0x1c69, 0x1c75, 0x1c5f,
0x1c5f, 0x1c74, 0x1c64, 0x1c60, 0x1c5a, 0x1c60, 0x1c5f, 0x1c71, 0x1c75, 0x1c5f, 0x1c66, 0x1c64,
0x1c6d, 0x1c5f, 0x20, 0x1c75, 0x1c5f, 0x1c71, 0x1c70, 0x1c6e, 0x1c68, 0x1c5f, 0x1c65, 0x1c75,
0x1c5f, 0x1c68, 0x1c75, 0x1c5f, 0x1c70, 0x1c5a, 0x1c65, 0x1c75, 0x1c6e, 0x1c5e, 0x1c64, 0x1c61,
0x1c6e, 0x1c75, 0x1c5e, 0x1c5f, 0x1c71, 0x1c60, 0x1c7c, 0x1c65, 0x1c5f, 0x1c75, 0x1c5e, 0x1c5a,
0x1c71, 0x1c75, 0x1c5a, 0x1c6d, 0x1c64, 0x1c65, 0x1c60, 0x1c6e, 0x1c62, 0x1c75, 0x1c7d, 0x1c68,
0x1c6e, 0x1c61, 0x1c7d, 0x20, 0x1c75, 0x1c6e, 0x1c60, 0x1c5f, 0x1c71, 0x1c60, 0x1c69, 0x1c71,
0x1c60, 0x1c5a, 0x1c62, 0x1c5f, 0x1c71, 0x1c6a, 0x1c64, 0x1c60, 0x1c5f, 0x1c5c, 0x1c5a, 0x1c6a,
0x1c77, 0x1c64, 0x1c66, 0x1c69, 0x1c63, 0x1c5f, 0x1c66, 0x1c69, 0x1c63, 0x1c5f, 0x1c60, 0x1c5a,
0x1c65, 0x1c74, 0x1c5f, 0x20, 0x1c68, 0x1c64, 0x1c60, 0x1c5f, 0x1c60, 0x1c68, 0x1c6e, 0x1c65,
0x1c74, 0x1c5a, 0x1c71, 0x1c60, 0x1c6d, 0x1c69, 0x1c68, 0x1c5f, 0x1c60, 0x1c5f, 0x1c63, 0x1c70,
0x1c5f, 0x1c71, 0x1c62, 0x1c5f, 0x1c68, 0x1c60, 0x1c65, 0x1c5f, 0x1c63, 0x1c71, 0x1c70, 0x1c5f,
0x1c63, 0x1c65, 0x1c5a, 0x1c71, 0x1c70, 0x1c5f, 0x1c63, 0x1c65, 0x1c5a, 0x1c71, 0x20, 0x1c60,
0x1c68, 0x1c64, 0x1c60, 0x1c70, 0x1c6e, 0x1c71, 0x1c63, 0x1c6e, 0x1c68, 0x1c70, 0x1c6e, 0x1c74,
0x1c68, 0x1c5a, 0x1c6d, 0x1c6e, 0x1c74, 0x1c70, 0x1c5a, 0x1c62, 0x1c64, 0x1c71, 0x1c64, 0x1c60,
0x1c5f, 0x1c6e, 0x1c70, 0x1c62, 0x1c5a, 0x1c71, 0x1c74, 0x1c5a, 0x1c71, 0x1c6e, 0x1c5e, 0x20,
0x1c65, 0x1c5f, 0x1c5e, 0x1c63, 0x1c6e, 0x1c70, 0x1c5a, 0x1c68, 0x1c6f, 0x1c77, 0x1c5a, 0x1c74,
0x20, 0x1c71, 0x1c6e, 0x1c5e, 0x1c65, 0x1c5a, 0x1c71, 0x1c5c, 0x1c5e, 0x1c6e, 0x1c65, 0x20,
0x1c75, 0x1c6e, 0x1c5c, 0x1c69, 0x1c65, 0x20, 0x1c75, 0x1c6e, 0x1c5c, 0x1c7d, 0x1c68, 0x1c5f,
0x1c71, 0x1c70, 0x20, 0x1c74, 0x1c69, 0x1c68, 0x1c60, 0x1c5c, 0x1c7d, 0x1c68, 0x1c6e, 0x1c71,
0x1c5f, 0x1c70, 0x1c5f, 0x1c5c, 0x1c69, 0x1c63, 0x1c5f, 0x1c70, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c6f,
0x1c5c, 0x1c69, 0x1c63, 0x1c5f, 0x1c74, 0x1c6e, 0x1c62, 0x1c5f, 0x1c5e, 0x1c5f, 0x1c66, 0x1c5f,
0x1c5e, 0x1c64, 0x1c6f, 0x1c77, 0x1c5f, 0x1c60, 0x1c65, 0x1c66, 0x1c5f, 0x1c63, 0x1c5f, 0x1c71,
0x1c5f, 0x1c66, 0x1c5f, 0x1c79, 0x1c68, 0x1c62, 0x1c5a, 0x1c65, 0x1c64, 0x1c5e, 0x1c5a, 0x1c64,
0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c6f, 0x1c5a, 0x1c5e, 0x1c64, 0x1c65,
0x1c71, 0x1c5a, 0x1c60, 0x1c65, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f,
0x1c71, 0x1c5f, 0x1c62, 0x1c6e, 0x1c68, 0x1c6e, 0x1c5d, 0x1c5c, 0x1c5a, 0x2c, 0x20, 0x1c64,
0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c6f, 0x1c64, 0x1c74, 0x1c5a, 0x1c68,
0x1c65, 0x1c75, 0x1c5a, 0x1c68, 0x1c5c, 0x1c7d, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64,
0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c74, 0x1c5a, 0x1c5e, 0x20, 0x1c65, 0x1c64, 0x1c74, 0x1c6d,
0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c75, 0x1c77,
0x1c6e, 0x1c75, 0x1c77, 0x1c5f, 0x1c6d, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d,
0x1c5f, 0x1c71, 0x1c5f, 0x1c75, 0x1c77, 0x1c64, 0x1c71, 0x1c65, 0x1c6e, 0x1c71, 0x1c65, 0x2c,
0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c63, 0x1c64, 0x1c71,
0x1c5f, 0x1c62, 0x1c5f, 0x1c60, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f,
0x1c71, 0x1c5f, 0x1c64, 0x1c71, 0x1c69, 0x1c63, 0x1c64, 0x1c60, 0x1c64, 0x1c60, 0x1c5f, 0x1c5e,
0x1c69, 0x1c63, 0x1c64, 0x1c5b, 0x1c61, 0x1c5f, 0x1c62, 0x1c5f, 0x1c6d, 0x1c60, 0x1c5f, 0x1c61,
0x1c69, 0x1c71, 0x1c6e, 0x1c5f, 0x1c79, 0x1c63, 0x1c5e, 0x1c69, 0x1c63, 0x1c64, 0x1c65, 0x1c63,
0x1c64, 0x1c5e, 0x1c6e, 0x1c62, 0x1c5a, 0x1c71, 0x1c74, 0x1c64, 0x1c65, 0x1c6e, 0x1c5e, 0x1c5a,
0x2c, 0x20, 0x1c60, 0x1c6e, 0x1c71, 0x1c74, 0x1c69, 0x1c60, 0x1c6d, 0x1c60, 0x1c68, 0x1c5f,
0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x1c64, 0x1c61, 0x1c7d, 0x1c60, 0x1c5a, 0x1c5e, 0x1c6e, 0x1c65,
0x20, 0x1c6e, 0x1c67, 0x1c61, 0x1c6e, 0x1c5e, 0x1c6e, 0x1c65, 0x1c5e, 0x1c5a, 0x1c63, 0x1c5f,
0x1c79, 0x1c68, 0x20, 0x1c6f, 0x1c68, 0x1c64, 0x1c71, 0x1c65, 0x20, 0x1c60, 0x1c63, 0x1c5f,
0x1c74, 0x1c5f, 0x1c79, 0x1c68, 0x1c62, 0x1c5f, 0x1c71, 0x1c5f, 0x1c5c, 0x1c69, 0x1c63, 0x1c5f,
0x1c62, 0x1c5f, 0x1c68, 0x1c64, 0x1c5c, 0x1c5a, 0x1c74, 0x1c62, 0x1c5f, 0x1c68, 0x1c74, 0x1c64,
0x1c71, 0x1c60, 0x1c62, 0x1c5f, 0x1c74, 0x1c5f, 0x1c62, 0x1c5a, 0x1c68, 0x1c5a, 0x1c65, 0x1c62,
0x1c5f, 0x1c61, 0x1c5f, 0x1c74, 0x1c5e, 0x1c5f, 0x1c71, 0x1c62, 0x1c6e, 0x1c71, 0x1c5a, 0x1c62,
0x1c64, 0x1c71, 0x1c6e, 0x1c62, 0x1c6e, 0x1c68, 0x1c64, 0x1c70, 0x1c5f, 0x1c62, 0x1c6e, 0x1c74,
0x1c5e, 0x1c5f, 0x1c60, 0x1c5f, 0x1c74, 0x1c5e, 0x1c5f, 0x1c62, 0x1c6e, 0x1c60, 0x1c65, 0x1c64,
0x1c60, 0x1c5a, 0x20, 0x1c65, 0x1c5a, 0x1c66, 0x1c5a, 0x1c68, 0x1c62, 0x1c64, 0x1c60, 0x1c63,
0x1c6e, 0x1c5e, 0x1c5a, 0x1c71, 0x1c62, 0x1c5a, 0x1c71, 0x1c60, 0x1c74, 0x1c5a, 0x1c71, 0x1c62,
0x1c5a, 0x1c71, 0x1c74, 0x1c6e, 0x1c68, 0x1c6e, 0x1c6d, 0x1c62, 0x1c5a, 0x1c71, 0x1c74, 0x1c65,
0x1c6e, 0x1c68, 0x1c5f, 0x1c74, 0x1c71, 0x1c5f, 0x1c65, 0x1c5f, 0x1c63, 0x1c71, 0x1c64, 0x1c63,
0x20, 0x1c6d, 0x1c5a, 0x1c68, 0x1c60, 0x1c71, 0x1c5a, 0x1c62, 0x1c6e, 0x1c75, 0x1c6e, 0x1c63,
0x1c5e, 0x1c5f, 0x1c66, 0x2c, 0x20, 0x1c6e, 0x1c5b, 0x1c5a, 0x1c62, 0x20, 0x1c70, 0x1c5f,
0x1c60, 0x1c5a, 0x1c74, 0x1c5f, 0x1c65, 0x1c6e, 0x1c71, 0x1c74, 0x1c5a, 0x1c68, 0x2c, 0x20,
0x1c6e, 0x1c5b, 0x1c5a, 0x1c62, 0x20, 0x1c70, 0x1c5f, 0x1c60, 0x1c5a, 0x1c74, 0x1c5f, 0x1c71,
0x1c64, 0x1c63, 0x20, 0x1c65, 0x1c6e, 0x1c5e, 0x1c6e, 0x1c62, 0x2c, 0x20, 0x1c6e, 0x1c5b,
0x1c5a, 0x1c62, 0x20, 0x1c70, 0x1c5f, 0x1c60, 0x1c5a, 0x1c74, 0x1c5f, 0x1c71, 0x1c69, 0x1c60,
0x1c5a, 0x1c61, 0x1c64, 0x1c71, 0x1c5f, 0x1c5c, 0x1c5f, 0x1c6f, 0x1c5f, 0x1c71, 0x1c5f, 0x1c62,
0x1c5f, 0x1c6f, 0x1c77, 0x1c64, 0x1c71, 0x1c64, 0x1c60, 0x1c65, 0x1c6f, 0x1c5a, 0x1c68, 0x1c74,
0x20, 0x1c5f, 0x1c63, 0x20, 0x1c6f, 0x1c68, 0x1c64, 0x1c71, 0x1c65, 0x1c6f, 0x1c5a, 0x1c68,
0x1c74, 0x20, 0x1c5a, 0x1c6f, 0x1c77, 0x20, 0x1c65, 0x1c6f, 0x1c6e, 0x1c71, 0x1c6f, 0x1c6d,
0x1c69, 0x1c68, 0x1c74, 0x1c5a, 0x20, 0x1c68, 0x1c64, 0x1c60, 0x1c5a, 0x1c68, 0x1c5f, 0x1c71,
0x1c60, 0x1c64, 0x1c71, 0x20, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c74, 0x1c68, 0x1c6e, 0x1c5c,
0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c68, 0x1c64, 0x1c65, 0x1c5a, 0x1c5e, 0x1c6d, 0x1c69,
0x1c74, 0x1c65, 0x1c5f, 0x1c71, 0x1c74, 0x1c5a, 0x20, 0x1c70, 0x1c5a, 0x1c62, 0x1c64, 0x1c71,
0x1c5c, 0x1c5a, 0x1c64, 0x1c5b, 0x1c5a, 0x1c60, 0x1c5a, 0x1c5b, 0x1c5a, 0x1c68, 0x1c62, 0x1c64,
0x1c5b, 0x1c65, 0x1c64, 0x1c5b, 0x1c60, 0x1c5f, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20,
0x1c75, 0x1c5f, 0x1c68, 0x1c5b, 0x1c77, 0x1c6e, 0x1c5e, 0x1c62, 0x1c64, 0x1c6e, 0x1c65, 0x20,
0x1c74, 0x1c64, 0x20, 0x1c61, 0x1c5a, 0x1c71, 0x1c65, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64,
0x20, 0x1c60, 0x1c64, 0x1c74, 0x1c65, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c5e,
0x1c69, 0x1c65, 0x1c64, 0x1c6d, 0x1c5f, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c5b,
0x1c77, 0x1c5a, 0x1c62, 0x1c5f, 0x1c65, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c63,
0x1c64, 0x1c71, 0x1c65, 0x1c6e, 0x1c74, 0x1c65, 0x1c63, 0x1c64, 0x1c6f, 0x1c77, 0x1c74, 0x20,
0x1c60, 0x1c5a, 0x1c68, 0x1c6e, 0x1c71, 0x1c74, 0x1c5b, 0x1c6e, 0x1c5c, 0x1c69, 0x1c6a, 0x1c64,
0x1c5c, 0x1c5f, 0x1c5e, 0x1c6f, 0x1c5f, 0x1c5b, 0x1c77, 0x1c69, 0x1c5e, 0x1c6e, 0x1c74, 0x1c64,
0x1c61, 0x1c69, 0x1c63, 0x1c5f, 0x1c71, 0x1c5f, 0x1c74, 0x1c5a, 0x1c68, 0x1c5a, 0x1c71, 0x1c74,
0x1c5a, 0x1c74, 0x1c5a, 0x1c68, 0x1c74, 0x1c5a, 0x1c5e, 0x1c5f, 0x1c63, 0x1c5f, 0x1c71, 0x1c60,
0x1c5a, 0x1c63, 0x1c5f, 0x1c79, 0x1c68, 0x1c63, 0x1c5f, 0x1c6d, 0x1c64, 0x1c74, 0x1c66, 0x1c5a,
0x1c68, 0x1c65, 0x1c63, 0x1c64, 0x1c71, 0x1c64, 0x1c6f, 0x1c6e, 0x1c5c, 0x1c7d, 0x1c6d, 0x1c5f,
0x1c60, 0x1c69, 0x1c5b, 0x1c5f, 0x1c5b, 0x1c75, 0x1c5f, 0x1c79, 0x1c68, 0x1c62, 0x1c69, 0x1c71,
0x1c70, 0x1c5f, 0x53, 0x75, 0x20, 0x43, 0xe0, 0x69, 0x72, 0x75, 0x4d, 0x6f,
0x67, 0x61, 0x64, 0xec, 0x73, 0x63, 0x69, 0x75, 0x50, 0x6f, 0x72, 0x74,
0x75, 0x2d, 0x4e, 0x6f, 0x75, 0x54, 0x72, 0xec, 0x70, 0x6f, 0x6c, 0x69,
0x54, 0xf9, 0x6e, 0x69, 0x73, 0x69, 0x53, 0x2019, 0x41, 0x76, 0x61, 0x6e,
0x61, 0x54, 0x7a, 0x69, 0x74, 0x61, 0x64, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x4d, 0xe8, 0x73, 0x73, 0x69, 0x63, 0x75, 0x4e, 0x6f,
0x61, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x65,
0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x4e, 0x65, 0x77, 0x20,
0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64,
0x50, 0xe0, 0x6e, 0x61, 0x6d, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x20,
0x64, 0x65, 0x20, 0x49, 0x73, 0x70, 0x61, 0x67, 0x6e, 0x61, 0x53, 0x61,
0x6e, 0x74, 0x75, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d,
0x65, 0x75, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x47, 0x69, 0x75, 0x61,
0x6e, 0x6e, 0x65, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x43, 0x72, 0x69,
0x73, 0x74, 0x6f, 0x6c, 0x75, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c,
0x75, 0x67, 0x68, 0x69, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x54,
0x6f, 0x6d, 0x6d, 0x61, 0x73, 0x75, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20,
0x56, 0x69, 0x73, 0x73, 0x65, 0x6e, 0x74, 0x65, 0x54, 0x7a, 0x69, 0x74,
0x61, 0x64, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68,
0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x43, 0x61, 0x74, 0xe0, 0x72, 0x52,
0x69, 0x79, 0xe0, 0x64, 0x53, 0x65, 0xf9, 0x6c, 0x54, 0x65, 0x68, 0x65,
0x72, 0xe0, 0x6e, 0x55, 0x6c, 0xe0, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x41, 0x7a, 0x6f, 0x72, 0x72, 0x61, 0x73, 0xcc, 0x73, 0x75, 0x6c,
0x61, 0x73, 0x20, 0x43, 0x61, 0x6e, 0xe0, 0x72, 0x69, 0x61, 0x73, 0x43,
0x61, 0x62, 0x75, 0x20, 0x42, 0x69, 0x72, 0x64, 0x65, 0xcc, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x46, 0xf8, 0x72, 0x6f, 0x79, 0x61, 0x72, 0x47,
0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x53, 0x75, 0x64, 0x53, 0x61, 0x6e, 0x74, 0x2019, 0x45, 0x6c, 0x65,
0x6e, 0x65, 0x42, 0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x75, 0x42, 0x65,
0x72, 0x6c, 0x69, 0x6e, 0x75, 0x42, 0xf9, 0x63, 0x61, 0x72, 0x65, 0x73,
0x74, 0x42, 0xf9, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74, 0x44, 0x75, 0x62,
0x6c, 0x69, 0x6e, 0x75, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x6e, 0xcc, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c,
0x4d, 0xf2, 0x6e, 0x61, 0x63, 0x75, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20,
0x4d, 0x61, 0x72, 0x69, 0x6e, 0x75, 0x49, 0x73, 0x74, 0x6f, 0x63, 0x6f,
0x6c, 0x6d, 0x61, 0x54, 0x7a, 0x69, 0x74, 0x61, 0x64, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61, 0x6e,
0x75, 0x56, 0x61, 0x72, 0x73, 0xe0, 0x76, 0x69, 0x61, 0x5a, 0x61, 0x67,
0xe0, 0x62, 0x72, 0x69, 0x61, 0x5a, 0x75, 0x72, 0x69, 0x67, 0x75, 0x4e,
0x61, 0x74, 0x69, 0x76, 0x69, 0x64, 0x61, 0x64, 0x65, 0x4d, 0x61, 0x75,
0x72, 0xec, 0x74, 0x7a, 0x69, 0x75, 0x73, 0x4d, 0x61, 0x69, 0x6f, 0x74,
0x61, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x47, 0x75, 0xe0,
0x6d, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x73, 0x61, 0x73, 0x411, 0x443,
0x435, 0x43d, 0x43e, 0x441, 0x20, 0x410, 0x458, 0x440, 0x435, 0x441, 0x41a, 0x43e,
0x440, 0x430, 0x43b, 0x20, 0x425, 0x430, 0x440, 0x431, 0x443, 0x440, 0x41a, 0x430,
0x458, 0x43c, 0x430, 0x43d, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x41b, 0x443, 0x438, 0x432, 0x438, 0x43b, 0x435, 0x41b, 0x43e, 0x443,
0x435, 0x440, 0x20, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x438, 0x437, 0x20, 0x41a,
0x432, 0x43e, 0x440, 0x442, 0x435, 0x440, 0x413, 0x43e, 0x442, 0x445, 0x430, 0x431,
0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x20, 0x420, 0x438, 0x43a, 0x43e, 0x421, 0x430,
0x43d, 0x442, 0x458, 0x430, 0x433, 0x43e, 0x421, 0x43a, 0x43e, 0x440, 0x435, 0x437,
0x431, 0x438, 0x441, 0x443, 0x43d, 0x434, 0x425, 0x43e, 0x43d, 0x433, 0x43a, 0x43e,
0x43d, 0x433, 0x423, 0x440, 0x443, 0x43c, 0x45b, 0x438, 0x424, 0x430, 0x440, 0x441,
0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x421, 0x432, 0x435,
0x442, 0x430, 0x20, 0x408, 0x435, 0x43b, 0x435, 0x43d, 0x430, 0x410, 0x434, 0x435,
0x43b, 0x435, 0x458, 0x434, 0x410, 0x441, 0x442, 0x440, 0x430, 0x43a, 0x430, 0x43d,
0x411, 0x438, 0x441, 0x438, 0x43d, 0x433, 0x435, 0x43d, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x43e, 0x20, 0x41c, 0x430, 0x43d, 0x412, 0x438, 0x43b, 0x45a, 0x443, 0x441,
0x412, 0x438, 0x432, 0x438, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430,
0x43d, 0x430, 0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x441, 0x2c, 0x20, 0x418,
0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x411, 0x458, 0x443, 0x43b, 0x430,
0x2c, 0x20, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x430, 0x440, 0x2c,
0x20, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430,
0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x421, 0x430, 0x43b,
0x435, 0x43c, 0x2c, 0x20, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x421, 0x432, 0x438, 0x444, 0x442,
0x20, 0x41a, 0x430, 0x440, 0x435, 0x43d, 0x442, 0x414, 0x438, 0x43c, 0x43e, 0x43d,
0x20, 0x434, 0x2019, 0x418, 0x440, 0x432, 0x438, 0x43b, 0x41b, 0x43e, 0x43d, 0x433,
0x458, 0x438, 0x440, 0x41d, 0x438, 0x458, 0x443, 0x435, 0x42, 0x61, 0x6d, 0x61,
0x6b, 0x6f, 0x42, 0x61, 0x6e, 0x67, 0x75, 0x69, 0x42, 0x61, 0x6e, 0x17e,
0x75, 0x6c, 0x42, 0x69, 0x73, 0x61, 0x6f, 0x42, 0x75, 0x64, 0x17e, 0x75,
0x6d, 0x62, 0x75, 0x72, 0x61, 0x4b, 0x61, 0x6d, 0x70, 0x61, 0x6c, 0x61,
0x4c, 0x75, 0x61, 0x6e, 0x64, 0x61, 0x4c, 0x75, 0x73, 0x61, 0x6b, 0x61,
0x4d, 0x61, 0x6c, 0x61, 0x62, 0x6f, 0x4d, 0x61, 0x70, 0x75, 0x74, 0x6f,
0x4d, 0x62, 0x61, 0x62, 0x61, 0x6e, 0x65, 0x4e, 0x69, 0x6a, 0x61, 0x6d,
0x65, 0x6a, 0x45, 0x6e, 0x6b, 0x6f, 0x72, 0x69, 0x64, 0x17e, 0x42, 0x75,
0x65, 0x6e, 0x6f, 0x73, 0x20, 0x41, 0x6a, 0x72, 0x65, 0x73, 0x17d, 0x75,
0x17e, 0x75, 0x69, 0x53, 0x61, 0x6e, 0x20, 0x48, 0x75, 0x61, 0x6e, 0x53,
0x61, 0x6e, 0x20, 0x4c, 0x75, 0x69, 0x55, 0x161, 0x75, 0x61, 0x69, 0x6a,
0x61, 0x41, 0x72, 0x75, 0x62, 0x61, 0x4b, 0x6f, 0x72, 0x61, 0x6c, 0x20,
0x48, 0x61, 0x72, 0x62, 0x75, 0x72, 0x42, 0x61, 0x69, 0x6a, 0x61, 0x20,
0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x69,
0x7a, 0x65, 0x42, 0x6c, 0x61, 0x6e, 0x6b, 0x2d, 0x53, 0x65, 0x6a, 0x62,
0x6c, 0x6f, 0x6e, 0x42, 0x6f, 0x61, 0x20, 0x56, 0x69, 0x73, 0x74, 0x61,
0x42, 0x6f, 0x67, 0x6f, 0x74, 0x61, 0x42, 0x6f, 0x6a, 0x7a, 0x69, 0x4b,
0x65, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x17e, 0x20, 0x42, 0x65, 0x6a, 0x4b,
0x61, 0x6a, 0x6d, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x10c, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0x53, 0x69, 0x75,
0x64, 0x61, 0x64, 0x20, 0x48, 0x75, 0x61, 0x72, 0x65, 0x7a, 0x44, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68, 0x61, 0x67, 0x65, 0x6e, 0x45,
0x69, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x46, 0x6f, 0x72, 0x74, 0x61,
0x6c, 0x65, 0x7a, 0x61, 0x47, 0x6c, 0x65, 0x6a, 0x73, 0x20, 0x42, 0x65,
0x6a, 0x47, 0x75, 0x73, 0x20, 0x42, 0x65, 0x6a, 0x47, 0x72, 0x61, 0x6e,
0x64, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x47, 0x72, 0x65, 0x6e, 0x61, 0x64,
0x61, 0x48, 0x65, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6a, 0x6f, 0x49,
0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73,
0x4e, 0x6f, 0x6b, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61,
0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x50, 0x69, 0x74, 0x65, 0x72,
0x73, 0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a,
0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x20, 0x53, 0x69, 0x74, 0x69, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x76,
0x69, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x56,
0x69, 0x6e, 0x73, 0x65, 0x6e, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x6a, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x49, 0x6b, 0x76,
0x61, 0x6c, 0x75, 0x69, 0x74, 0x17d, 0x75, 0x6e, 0x6f, 0x4c, 0x75, 0x69,
0x76, 0x69, 0x6c, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x10d, 0x65, 0x6c, 0x6f,
0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x61, 0x6b, 0x69, 0x4b, 0x72, 0x61,
0x6c, 0x65, 0x6e, 0x64, 0x61, 0x6a, 0x6b, 0x4c, 0x61, 0x20, 0x50, 0x61,
0x7a, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x111, 0x65, 0x6c, 0x65, 0x73,
0x4c, 0x6f, 0x75, 0x65, 0x72, 0x20, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x69,
0x7a, 0x20, 0x4b, 0x76, 0x6f, 0x72, 0x74, 0x65, 0x72, 0x4d, 0x61, 0x7a,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x4d, 0x65, 0x74, 0x6c, 0x61, 0x6b, 0x61,
0x74, 0x6c, 0x61, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x53,
0x69, 0x74, 0x69, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x76, 0x69, 0x64, 0x65,
0x6f, 0x4e, 0x6a, 0x75, 0x6a, 0x6f, 0x72, 0x6b, 0x42, 0x6a, 0x75, 0x6c,
0x61, 0x2c, 0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x61, 0x72,
0x2c, 0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x47, 0x6f, 0x74, 0x68,
0x61, 0x62, 0x50, 0x61, 0x6e, 0x61, 0x6d, 0x61, 0x50, 0x61, 0x72, 0x61,
0x6d, 0x61, 0x72, 0x69, 0x62, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f,
0x2d, 0x50, 0x72, 0x65, 0x6e, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x6f,
0x76, 0x20, 0x53, 0x70, 0x65, 0x6a, 0x6e, 0x50, 0x6f, 0x72, 0x74, 0x6f,
0x20, 0x56, 0x65, 0x6c, 0x6a, 0x6f, 0x52, 0x65, 0x73, 0x69, 0x66, 0x65,
0x52, 0x65, 0x64, 0x17e, 0x61, 0x6a, 0x6e, 0x61, 0x52, 0x65, 0x7a, 0x6f,
0x6c, 0x75, 0x74, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x65, 0x6d, 0x53,
0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f,
0x53, 0x61, 0x6f, 0x20, 0x50, 0x61, 0x6f, 0x6c, 0x6f, 0x53, 0x69, 0x74,
0x6b, 0x61, 0x53, 0x76, 0x2e, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x6a, 0x61,
0x53, 0x65, 0x6e, 0x74, 0x20, 0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74,
0x54, 0x6f, 0x72, 0x6f, 0x6e, 0x74, 0x6f, 0x54, 0x6f, 0x72, 0x74, 0x6f,
0x6c, 0x61, 0x56, 0x61, 0x6a, 0x74, 0x68, 0x6f, 0x72, 0x73, 0x44, 0x69,
0x6d, 0x6f, 0x6e, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x76, 0x69, 0x6c, 0x4d,
0x61, 0x6b, 0x6d, 0x75, 0x72, 0x64, 0x6f, 0x4c, 0x6f, 0x6e, 0x67, 0x6a,
0x69, 0x72, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x75, 0x73, 0x74, 0x61, 0x4a,
0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x69, 0x6d, 0x4b, 0x75, 0x61, 0x6c,
0x61, 0x20, 0x4c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x4d, 0x61, 0x6e, 0x69,
0x6c, 0x61, 0x50, 0x6f, 0x6e, 0x74, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x6b,
0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x6b, 0x6f, 0x6c, 0x69, 0x6d,
0x73, 0x6b, 0x55, 0x72, 0x75, 0x6d, 0x107, 0x69, 0x55, 0x73, 0x74, 0x2d,
0x4e, 0x65, 0x72, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x73, 0x6b, 0x61,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x5a, 0x65, 0x6c, 0x65, 0x6e,
0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x61, 0x46, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x4d, 0x61, 0x64, 0x65, 0x69, 0x72, 0x61, 0x4a, 0x75, 0x17e,
0x6e, 0x61, 0x20, 0x44, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61,
0x53, 0x76, 0x65, 0x74, 0x61, 0x20, 0x4a, 0x65, 0x6c, 0x65, 0x6e, 0x61,
0x41, 0x64, 0x65, 0x6c, 0x65, 0x6a, 0x64, 0x42, 0x72, 0x69, 0x7a, 0x62,
0x65, 0x6a, 0x6e, 0x49, 0x75, 0x6b, 0x6c, 0x61, 0x42, 0x72, 0x61, 0x74,
0x69, 0x73, 0x6c, 0x61, 0x76, 0x61, 0x42, 0x69, 0x73, 0x69, 0x6e, 0x67,
0x65, 0x6e, 0x44, 0x17e, 0x65, 0x72, 0x7a, 0x69, 0x4c, 0x6a, 0x75, 0x62,
0x6c, 0x6a, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x6c, 0x74, 0x61, 0x4f, 0x73,
0x6c, 0x6f, 0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x63, 0x61, 0x52,
0x69, 0x67, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x61, 0x6e,
0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x6a,
0x65, 0x76, 0x6f, 0x56, 0x69, 0x6c, 0x6e, 0x6a, 0x75, 0x73, 0x41, 0x6e,
0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x76, 0x6f, 0x42, 0x75,
0x67, 0x61, 0x6e, 0x76, 0x69, 0x6c, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e,
0x6a, 0x65, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x46, 0x61, 0x6b,
0x61, 0x6f, 0x66, 0x6f, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x6a, 0x65, 0x4b,
0x6f, 0x161, 0x72, 0x65, 0x4b, 0x76, 0x61, 0x64, 0x17e, 0x61, 0x6c, 0x65,
0x6a, 0x69, 0x6e, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4e, 0x69, 0x6a, 0x75,
0x65, 0x50, 0x61, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x67, 0x6f, 0x50, 0x6f,
0x72, 0x74, 0x20, 0x4d, 0x6f, 0x72, 0x7a, 0x62, 0x69, 0x52, 0x61, 0x72,
0x6f, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x6a, 0x70, 0x61, 0x6e,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x56, 0x65, 0x76,
0x61, 0x6a, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61,
0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x4c, 0x75, 0x69, 0x76, 0x69, 0x6c,
0x65, 0x42, 0x69, 0x6a, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x53, 0x65, 0x76,
0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43,
0x65, 0x6e, 0x74, 0x61, 0x72, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72,
0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x6f, 0x76,
0x69, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x65, 0x76,
0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50,
0x6f, 0x72, 0x74, 0x20, 0x6f, 0x66, 0x20, 0x53, 0x70, 0x65, 0x6a, 0x6e,
0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x6f, 0x52, 0x65,
0x67, 0x69, 0x6e, 0x61, 0x53, 0x6b, 0x6f, 0x72, 0x65, 0x7a, 0x62, 0x69,
0x73, 0x75, 0x6e, 0x64, 0x53, 0x76, 0x2e, 0x20, 0x42, 0x61, 0x72, 0x74,
0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x6a, 0x53, 0x76, 0x2e, 0x20, 0x44, 0x17e,
0x6f, 0x6e, 0x53, 0x76, 0x2e, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x44, 0x69,
0x6d, 0x6f, 0x6e, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x4d,
0x65, 0x6b, 0x76, 0x6f, 0x72, 0x69, 0x4c, 0x6f, 0x6e, 0x67, 0x6a, 0x65,
0x72, 0x62, 0x6a, 0x65, 0x6e, 0x4e, 0x69, 0x75, 0x65, 0x43, 0x61, 0x73,
0x61, 0x62, 0x62, 0x6c, 0x61, 0x6e, 0x63, 0x61, 0x43, 0xe8, 0x75, 0x74,
0x61, 0x47, 0x69, 0x62, 0x62, 0x75, 0x74, 0x69, 0x4d, 0x75, 0x67, 0x61,
0x64, 0x69, 0x73, 0x63, 0x69, 0x75, 0x4e, 0x61, 0x69, 0x72, 0x6f, 0x62,
0x62, 0x69, 0x54, 0x72, 0xec, 0x70, 0x75, 0x6c, 0x69, 0x43, 0xf2, 0x72,
0x64, 0x75, 0x62, 0x61, 0x42, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x43, 0x61,
0x6d, 0x70, 0x75, 0x20, 0x52, 0x61, 0x6e, 0x6e, 0x69, 0x43, 0x75, 0x73,
0x74, 0x61, 0x20, 0x52, 0x69, 0x63, 0x61, 0x44, 0x69, 0x74, 0x72, 0x6f,
0x69, 0x74, 0x44, 0x75, 0x6d, 0xec, 0x6e, 0x69, 0x63, 0x61, 0x43, 0x69,
0x74, 0xe0, 0x20, 0x64, 0xfb, 0x20, 0x4d, 0xe8, 0x73, 0x73, 0x69, 0x63,
0x75, 0x4d, 0x75, 0x6e, 0x74, 0x69, 0x72, 0x72, 0x65, 0x69, 0x4d, 0x75,
0x6e, 0x7a, 0x69, 0x72, 0x72, 0x61, 0x74, 0x75, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x4a, 0x6f, 0x72, 0x63, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x69, 0x20,
0x54, 0x72, 0x61, 0x6d, 0x75, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x75, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x69, 0x20, 0x54, 0x72, 0x61, 0x6d, 0x75, 0x6e, 0x74, 0x61,
0x6e, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54,
0x72, 0x61, 0x6d, 0x75, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x50, 0x6f, 0x72,
0x74, 0x75, 0x20, 0x64, 0x69, 0x20, 0x53, 0x70, 0x61, 0x67, 0x6e, 0x61,
0x50, 0x6f, 0x72, 0x74, 0x75, 0x20, 0x52, 0x69, 0x63, 0x75, 0x52, 0x69,
0x67, 0x67, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x75,
0x6c, 0x75, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x63, 0xec,
0x61, 0x41, 0x6d, 0x6d, 0xe0, 0x6e, 0x43, 0x75, 0x6c, 0x75, 0x6d, 0x6d,
0x75, 0x47, 0x61, 0x7a, 0x7a, 0x61, 0x47, 0x69, 0x72, 0x75, 0x73, 0x61,
0x6c, 0x65, 0x6d, 0x6d, 0x69, 0x4b, 0x75, 0x72, 0x6b, 0x61, 0x74, 0x61,
0x4d, 0x61, 0x73, 0x63, 0x61, 0x74, 0x69, 0x4e, 0x69, 0x63, 0x75, 0x73,
0xec, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x63, 0x61, 0x6e, 0x6e, 0x61,
0x53, 0x69, 0x75, 0x6c, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72,
0x69, 0x56, 0x6c, 0x61, 0x64, 0xec, 0x76, 0x75, 0x73, 0x74, 0x6f, 0x6b,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x69, 0x72, 0x69, 0x6d, 0x6d, 0x75, 0x72,
0x67, 0x75, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x69, 0x42, 0x69, 0x72,
0x6d, 0x75, 0x64, 0x61, 0x43, 0x61, 0x70, 0x75, 0x20, 0x56, 0x69, 0x72,
0x64, 0x69, 0x47, 0x69, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x69,
0x20, 0x73, 0x63, 0x69, 0x72, 0x6f, 0x63, 0x63, 0x75, 0x53, 0x61, 0x6e,
0x74, 0x2019, 0xc8, 0x6c, 0x69, 0x6e, 0x61, 0x41, 0x64, 0x69, 0x6c, 0x61,
0x69, 0x64, 0x69, 0x42, 0x69, 0x72, 0x6c, 0x69, 0x6e, 0x75, 0x42, 0x72,
0x75, 0x73, 0x73, 0x65, 0x6c, 0x69, 0x44, 0x75, 0x62, 0x62, 0x6c, 0x69,
0x6e, 0x75, 0x47, 0x69, 0x62, 0x62, 0x69, 0x72, 0x74, 0x65, 0x72, 0x72,
0x61, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61,
0x6e, 0x4c, 0xf2, 0x6e, 0x6e, 0x69, 0x72, 0x61, 0x4c, 0x75, 0x73, 0x73,
0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x75, 0x4d, 0x61, 0x75, 0x74, 0x61,
0x50, 0x61, 0x72, 0x69, 0x67, 0x67, 0x69, 0x53, 0x61, 0x72, 0x61, 0x6a,
0x65, 0x76, 0x75, 0x53, 0x74, 0x75, 0x63, 0x63, 0x6f, 0x72, 0x6d, 0x61,
0x56, 0xf2, 0x6c, 0x67, 0x75, 0x67, 0x72, 0x61, 0x64, 0x5a, 0x61, 0x67,
0x61, 0x62, 0x62, 0x72, 0x69, 0x61, 0x4e, 0x61, 0x74, 0x61, 0x6c, 0x69,
0x4d, 0x61, 0x72, 0x64, 0x69, 0x76, 0x69, 0x47, 0x61, 0x6c, 0xe0, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x4b, 0x14d, 0x6e, 0x61, 0x6b, 0x72, 0x79, 0x4d,
0x79, 0x6e, 0x64, 0x6f, 0x7a, 0x61, 0x41, 0x73, 0x75, 0x144, 0x63, 0x69,
0x6f, 0x6e, 0x4b, 0x61, 0x6a, 0x79, 0x6e, 0x6e, 0x61, 0x44, 0x14d, 0x6d,
0x69, 0x6e, 0x69, 0x6b, 0x61, 0x47, 0x72, 0x79, 0x6e, 0x61, 0x64, 0x61,
0x47, 0x77, 0x61, 0x74, 0x79, 0x6d, 0x61, 0x6c, 0x61, 0x4d, 0x61, 0x63,
0x65, 0x69, 0x14d, 0x4d, 0x79, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x65, 0x65,
0x4d, 0x14d, 0x6e, 0x74, 0x73, 0x65, 0x72, 0x72, 0x61, 0x74, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x50, 0x14d, 0x142, 0x6e, 0x6f, 0x63,
0x6e, 0x6f, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e,
0x74, 0x65, 0x72, 0x2c, 0x20, 0x50, 0x14d, 0x142, 0x6e, 0x6f, 0x63, 0x6e,
0x6f, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20,
0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x50, 0x14d, 0x142, 0x6e, 0x6f,
0x63, 0x6e, 0x6f, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c, 0x79,
0x6d, 0x79, 0x4c, 0x6f, 0x6e, 0x67, 0x79, 0x65, 0x61, 0x72, 0x62, 0x79,
0x79, 0x6e, 0x41, 0x64, 0x79, 0x6e, 0x4b, 0x6f, 0x6c, 0x14d, 0x6d, 0x62,
0x6f, 0x48, 0x65, 0x62, 0x72, 0x14d, 0x6e, 0x48, 0x14d, 0x6e, 0x67, 0x6b,
0x14d, 0x6e, 0x67, 0xd4, 0x6d, 0x73, 0x6b, 0xd4, 0x72, 0x61, 0x6c, 0x50,
0x6a, 0x14d, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x54, 0x61, 0x73, 0x7a,
0x6b, 0x79, 0x6e, 0x74, 0x54, 0x14d, 0x6d, 0x73, 0x6b, 0x57, 0x69, 0x79,
0x6e, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x4b,
0x61, 0x6e, 0x61, 0x72, 0x79, 0x6a, 0x73, 0x6b, 0x65, 0x52, 0x65, 0x70,
0x75, 0x62, 0x6c, 0x69, 0x6b, 0x61, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x14d,
0x6e, 0x65, 0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d, 0x6e,
0x64, 0x6b, 0x61, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0xd4, 0x77, 0x63,
0x7a, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x50, 0x6f,
0x142, 0x65, 0x64, 0x6e, 0x69, 0x6f, 0x77, 0x6f, 0x15a, 0x77, 0x69, 0x79,
0x6e, 0x74, 0x6f, 0x20, 0x48, 0x65, 0x6c, 0x79, 0x6e, 0x61, 0x41, 0x74,
0x79, 0x6e, 0x79, 0x4b, 0x69, 0x73, 0x7a, 0x79, 0x6e, 0x69, 0x14d, 0x77,
0x4b, 0x6f, 0x70, 0x79, 0x6e, 0x68, 0x61, 0x67, 0x61, 0x4b, 0x69, 0x6a,
0x14d, 0x77, 0x4c, 0x69, 0x7a, 0x62, 0x14d, 0x6e, 0x61, 0x4c, 0x14d, 0x6e,
0x64, 0x79, 0x6e, 0x4c, 0x75, 0x6b, 0x73, 0x79, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x4d, 0x14d, 0x6e, 0x61, 0x6b, 0x6f, 0xd4, 0x73, 0x6c, 0x6f, 0x53,
0x61, 0x72, 0x61, 0x74, 0x14d, 0x77, 0x57, 0x69, 0x65, 0x64, 0x79, 0x144,
0x47, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61,
0x4b, 0x14d, 0x6d, 0x6f, 0x72, 0x79, 0x4b, 0x65, 0x72, 0x67, 0x75, 0x65,
0x6c, 0x65, 0x6e, 0x6f, 0x77, 0x65, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79,
0x42, 0x6f, 0x75, 0x67, 0x61, 0x69, 0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x2019, 0x6f, 0x77, 0x61, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x57, 0x79,
0x73, 0x70, 0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f,
0x63, 0x6e, 0x6f, 0x627, 0x628, 0x64a, 0x20, 0x62c, 0x627, 0x646, 0x627, 0x64a,
0x6aa, 0x631, 0x627, 0x627, 0x64a, 0x68a, 0x64a, 0x633, 0x20, 0x627, 0x628, 0x628,
0x627, 0x627, 0x644, 0x62c, 0x64a, 0x631, 0x632, 0x627, 0x632, 0x645, 0x627, 0x631,
0x627, 0x628, 0x627, 0x645, 0x627, 0x6aa, 0x648, 0x628, 0x646, 0x6af, 0x64a, 0x628,
0x64a, 0x646, 0x62c, 0x627, 0x644, 0x628, 0x633, 0x627, 0x626, 0x648, 0x628, 0x644,
0x646, 0x67d, 0x627, 0x626, 0x631, 0x628, 0x648, 0x62c, 0x645, 0x628, 0x648, 0x631,
0x627, 0x642, 0x627, 0x626, 0x631, 0x627, 0x6aa, 0x627, 0x633, 0x627, 0x628, 0x644,
0x627, 0x646, 0x6aa, 0x627, 0x633, 0x64a, 0x648, 0x67d, 0x627, 0x6aa, 0x648, 0x646,
0x627, 0x6aa, 0x631, 0x64a, 0x68a, 0x627, 0x6aa, 0x627, 0x631, 0x627, 0x644, 0x20,
0x627, 0x64a, 0x648, 0x646, 0x641, 0x631, 0x64a, 0x20, 0x67d, 0x627, 0x626, 0x648,
0x646, 0x6af, 0x64a, 0x628, 0x648, 0x631, 0x648, 0x646, 0x62c, 0x648, 0x647, 0x627,
0x646, 0x633, 0x628, 0x631, 0x6af, 0x6aa, 0x645, 0x67e, 0x627, 0x644, 0x627, 0x6aa,
0x64a, 0x6af, 0x627, 0x644, 0x64a, 0x6aa, 0x646, 0x634, 0x627, 0x633, 0x627, 0x644,
0x6cc, 0x628, 0x631, 0x648, 0x6cc, 0x644, 0x6cc, 0x644, 0x648, 0x627, 0x646, 0x68a,
0x627, 0x644, 0x648, 0x628, 0x645, 0x628, 0x627, 0x634, 0x6cc, 0x644, 0x648, 0x633,
0x627, 0x6aa, 0x627, 0x645, 0x644, 0x627, 0x628, 0x648, 0x645, 0x628, 0x627, 0x628,
0x64a, 0x646, 0x645, 0x648, 0x63a, 0x627, 0x62f, 0x6cc, 0x634, 0x648, 0x646, 0x648,
0x627, 0x6aa, 0x634, 0x648, 0x637, 0x622, 0x626, 0x648, 0x6af, 0x648, 0x20, 0x68a,
0x627, 0x626, 0x648, 0x6af, 0x648, 0x67e, 0x648, 0x631, 0x67d, 0x648, 0x20, 0x646,
0x648, 0x648, 0x633, 0x627, 0x626, 0x648, 0x20, 0x67d, 0x648, 0x645, 0x67d, 0x631,
0x67e, 0x648, 0x644, 0x64a, 0x62a, 0x64a, 0x648, 0x646, 0x633, 0x648, 0x646, 0x68a,
0x647, 0x648, 0x6aa, 0x627, 0x62f, 0x6aa, 0x627, 0x64a, 0x646, 0x6aa, 0x631, 0x64a,
0x62c, 0x627, 0x646, 0x6af, 0x648, 0x64a, 0x644, 0x627, 0x627, 0x64a, 0x646, 0x67d,
0x64a, 0x6af, 0x648, 0x627, 0x627, 0x631, 0x627, 0x6af, 0x648, 0x6cc, 0x627, 0x646,
0x627, 0x628, 0x64a, 0x648, 0x646, 0x648, 0x633, 0x20, 0x627, 0x64a, 0x626, 0x631,
0x633, 0x6aa, 0x64a, 0x67d, 0x627, 0x20, 0x645, 0x627, 0x631, 0x6aa, 0x627, 0x6aa,
0x627, 0x631, 0x68a, 0x648, 0x628, 0x627, 0x6aa, 0x648, 0x6aa, 0x648, 0x626, 0x64a,
0x644, 0x627, 0x20, 0x631, 0x6cc, 0x626, 0x648, 0x62c, 0x627, 0x645, 0x64a, 0x646,
0x68a, 0x648, 0x632, 0x627, 0x631, 0x6cc, 0x648, 0x20, 0x6af, 0x627, 0x644, 0x6cc,
0x6af, 0x648, 0x633, 0x633, 0x627, 0x644, 0x67d, 0x627, 0x633, 0x627, 0x646, 0x20,
0x62c, 0x648, 0x622, 0x646, 0x633, 0x627, 0x646, 0x20, 0x644, 0x648, 0x626, 0x6cc,
0x633, 0x67d, 0x648, 0x6aa, 0x648, 0x645, 0x64a, 0x646, 0x627, 0x648, 0x634, 0x648,
0x622, 0x626, 0x6cc, 0x627, 0x627, 0x633, 0x627, 0x646, 0x633, 0x6cc, 0x648, 0x646,
0x627, 0x67d, 0x64a, 0x6aa, 0x648, 0x6aa, 0x646, 0x628, 0x627, 0x62d, 0x64a, 0x647,
0x628, 0x627, 0x647, 0x6cc, 0x627, 0x20, 0x628, 0x6cc, 0x646, 0x62f, 0x631, 0x627,
0x633, 0x628, 0x627, 0x631, 0x628, 0x627, 0x68a, 0x648, 0x633, 0x628, 0x6cc, 0x644,
0x645, 0x628, 0x6cc, 0x644, 0x6cc, 0x632, 0x628, 0x644, 0x627, 0x646, 0x6aa, 0x20,
0x633, 0x628, 0x644, 0x648, 0x646, 0x628, 0x627, 0x626, 0x648, 0x20, 0x648, 0x633,
0x62a, 0x627, 0x628, 0x6af, 0x648, 0x67d, 0x627, 0x628, 0x648, 0x626, 0x633, 0x6aa,
0x64a, 0x645, 0x628, 0x631, 0x62c, 0x20, 0x628, 0x64a, 0x6aa, 0x64a, 0x645, 0x67e,
0x648, 0x20, 0x6af, 0x631, 0x627, 0x646, 0x68a, 0x64a, 0x6aa, 0x627, 0x646, 0x6aa,
0x648, 0x646, 0x6aa, 0x631, 0x627, 0x6aa, 0x633, 0x6aa, 0x627, 0x626, 0x64a, 0x646,
0x633, 0x64a, 0x627, 0x645, 0x646, 0x634, 0x6aa, 0x627, 0x6af, 0x648, 0x686, 0x64a,
0x20, 0x647, 0x648, 0x627, 0x20, 0x647, 0x648, 0x627, 0x633, 0x648, 0x68a, 0x64a,
0x68a, 0x20, 0x62c, 0x648, 0x631, 0x64a, 0x632, 0x6aa, 0x648, 0x633, 0x67d, 0x627,
0x20, 0x631, 0x64a, 0x6aa, 0x627, 0x6aa, 0x631, 0x64a, 0x633, 0x67d, 0x646, 0x6aa,
0x648, 0x626, 0x64a, 0x627, 0x628, 0x627, 0x6aa, 0x64a, 0x648, 0x631, 0x627, 0x633,
0x627, 0x626, 0x648, 0x68a, 0x64a, 0x646, 0x645, 0x627, 0x631, 0x6aa, 0x20, 0x634,
0x648, 0x646, 0x68a, 0x627, 0x648, 0x633, 0x646, 0x68a, 0x627, 0x648, 0x633, 0x646,
0x20, 0x6aa, 0x631, 0x64a, 0x6aa, 0x68a, 0x64a, 0x646, 0x648, 0x631, 0x68a, 0x64a,
0x67d, 0x631, 0x627, 0x626, 0x64a, 0x67d, 0x68a, 0x648, 0x645, 0x64a, 0x646, 0x64a,
0x6aa, 0x627, 0x627, 0x64a, 0x68a, 0x645, 0x648, 0x646, 0x67d, 0x646, 0x627, 0x64a,
0x631, 0x648, 0x646, 0x64a, 0x67e, 0x627, 0x64a, 0x644, 0x20, 0x633, 0x644, 0x648,
0x627, 0x68a, 0x648, 0x631, 0x641, 0x648, 0x631, 0x67d, 0x20, 0x646, 0x64a, 0x644,
0x633, 0x646, 0x641, 0x648, 0x631, 0x67d, 0x627, 0x644, 0x64a, 0x632, 0x627, 0x6af,
0x644, 0x64a, 0x633, 0x20, 0x628, 0x64a, 0x6af, 0x631, 0x627, 0x646, 0x68a, 0x20,
0x62a, 0x631, 0x6aa, 0x6af, 0x631, 0x64a, 0x646, 0x627, 0x68a, 0x627, 0x6af, 0x648,
0x627, 0x68a, 0x64a, 0x644, 0x648, 0x67e, 0x6af, 0x648, 0x626, 0x67d, 0x64a, 0x20,
0x645, 0x627, 0x644, 0x627, 0x6af, 0x648, 0x627, 0x64a, 0x627, 0x6aa, 0x64a, 0x644,
0x6af, 0x64a, 0x627, 0x646, 0x627, 0x647, 0x64a, 0x644, 0x64a, 0x20, 0x641, 0x64a,
0x6aa, 0x633, 0x647, 0x631, 0x645, 0x648, 0x633, 0x644, 0x648, 0x627, 0x646, 0x68a,
0x64a, 0x627, 0x646, 0x627, 0x20, 0x67e, 0x648, 0x644, 0x633, 0x646, 0x627, 0x6aa,
0x633, 0x60c, 0x20, 0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x645, 0x631,
0x64a, 0x646, 0x6af, 0x648, 0x60c, 0x20, 0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646,
0x627, 0x67e, 0x64a, 0x67d, 0x631, 0x633, 0x628, 0x631, 0x6af, 0x60c, 0x20, 0x627,
0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x67d, 0x64a, 0x644, 0x20, 0x633, 0x67d,
0x64a, 0x60c, 0x20, 0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x648, 0x64a,
0x648, 0x64a, 0x60c, 0x20, 0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x648,
0x646, 0x633, 0x6cc, 0x646, 0x6cc, 0x632, 0x60c, 0x20, 0x627, 0x646, 0x68a, 0x6cc,
0x627, 0x646, 0x627, 0x648, 0x64a, 0x646, 0x627, 0x645, 0x64a, 0x6aa, 0x60c, 0x20,
0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x627, 0x646, 0x648, 0x648, 0x6aa,
0x627, 0x6aa, 0x627, 0x644, 0x648, 0x626, 0x67d, 0x62c, 0x645, 0x627, 0x626, 0x6aa,
0x627, 0x62c, 0x648, 0x646, 0x626, 0x648, 0x644, 0x648, 0x626, 0x64a, 0x20, 0x648,
0x64a, 0x644, 0x645, 0x648, 0x646, 0x67d, 0x64a, 0x633, 0x64a, 0x644, 0x648, 0x60c,
0x20, 0x6aa, 0x64a, 0x646, 0x67d, 0x6aa, 0x64a, 0x6aa, 0x631, 0x627, 0x644, 0x64a,
0x646, 0x68a, 0x6aa, 0x644, 0x627, 0x20, 0x67e, 0x632, 0x644, 0x627, 0x633, 0x20,
0x627, 0x64a, 0x646, 0x62c, 0x644, 0x633, 0x644, 0x648, 0x626, 0x631, 0x20, 0x67e,
0x631, 0x646, 0x633, 0x632, 0x20, 0x6aa, 0x648, 0x627, 0x631, 0x67d, 0x631, 0x645,
0x6cc, 0x633, 0x6cc, 0x626, 0x648, 0x645, 0x646, 0x627, 0x626, 0x648, 0x633, 0x645,
0x64a, 0x631, 0x64a, 0x6af, 0x648, 0x67d, 0x645, 0x627, 0x631, 0x67d, 0x64a, 0x646,
0x6aa, 0x645, 0x62a, 0x627, 0x645, 0x648, 0x631, 0x648, 0x633, 0x645, 0x632, 0x627,
0x67d, 0x644, 0x627, 0x646, 0x645, 0x6cc, 0x646, 0x648, 0x645, 0x6cc, 0x646, 0x64a,
0x645, 0x64a, 0x631, 0x64a, 0x68a, 0x627, 0x645, 0x64a, 0x67d, 0x644, 0x627, 0x6aa,
0x627, 0x67d, 0x627, 0x644, 0x627, 0x645, 0x64a, 0x6aa, 0x633, 0x64a, 0x6aa, 0x648,
0x20, 0x634, 0x647, 0x631, 0x645, 0x6cc, 0x6aa, 0x648, 0x626, 0x6cc, 0x644, 0x648,
0x646, 0x645, 0x627, 0x646, 0x6aa, 0x67d, 0x646, 0x645, 0x627, 0x646, 0x67d, 0x64a,
0x631, 0x64a, 0x645, 0x648, 0x646, 0x67d, 0x64a, 0x20, 0x648, 0x64a, 0x68a, 0x64a,
0x648, 0x645, 0x627, 0x646, 0x67d, 0x633, 0x631, 0x64a, 0x67d, 0x646, 0x64a, 0x648,
0x64a, 0x627, 0x631, 0x6aa, 0x646, 0x648, 0x631, 0x627, 0x646, 0x647, 0x64a, 0x627,
0x628, 0x64a, 0x648, 0x644, 0x627, 0x647, 0x60c, 0x20, 0x627, 0x62a, 0x631, 0x20,
0x68a, 0x6aa, 0x648, 0x67d, 0x627, 0x633, 0x64a, 0x646, 0x67d, 0x631, 0x60c, 0x20,
0x627, 0x62a, 0x631, 0x20, 0x68a, 0x6aa, 0x648, 0x67d, 0x627, 0x646, 0x64a, 0x648,
0x20, 0x633, 0x64a, 0x644, 0x645, 0x60c, 0x20, 0x627, 0x62a, 0x631, 0x20, 0x68a,
0x6aa, 0x648, 0x67d, 0x627, 0x646, 0x64a, 0x648, 0x6aa, 0x627, 0x648, 0x6aa, 0x64a,
0x646, 0x627, 0x6af, 0x627, 0x67e, 0x646, 0x627, 0x645, 0x627, 0x67e, 0x64a, 0x631,
0x627, 0x645, 0x64a, 0x631, 0x64a, 0x628, 0x648, 0x641, 0x648, 0x646, 0x64a, 0x6aa,
0x633, 0x67e, 0x648, 0x631, 0x67d, 0x20, 0x627, 0x648, 0x20, 0x67e, 0x631, 0x646,
0x633, 0x627, 0x633, 0x67e, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x67b, 0x64a,
0x67d, 0x67e, 0x648, 0x631, 0x67d, 0x648, 0x20, 0x648, 0x64a, 0x644, 0x647, 0x648,
0x67e, 0x648, 0x631, 0x67d, 0x648, 0x20, 0x631, 0x64a, 0x6aa, 0x648, 0x67e, 0x646,
0x67d, 0x627, 0x20, 0x627, 0x631, 0x64a, 0x646, 0x627, 0x633, 0x631, 0x64a, 0x646,
0x6aa, 0x646, 0x20, 0x627, 0x646, 0x644, 0x64a, 0x67d, 0x647, 0x64a, 0x633, 0x64a,
0x641, 0x64a, 0x631, 0x6cc, 0x62c, 0x6cc, 0x646, 0x627, 0x631, 0x64a, 0x632, 0x648,
0x644, 0x64a, 0x648, 0x67d, 0x631, 0x64a, 0x648, 0x20, 0x628, 0x631, 0x627, 0x646,
0x6aa, 0x648, 0x633, 0x646, 0x67d, 0x627, 0x631, 0x64a, 0x645, 0x633, 0x64a, 0x646,
0x67d, 0x64a, 0x627, 0x6af, 0x648, 0x633, 0x64a, 0x646, 0x67d, 0x648, 0x20, 0x68a,
0x648, 0x645, 0x64a, 0x646, 0x6af, 0x648, 0x633, 0x627, 0x626, 0x648, 0x20, 0x67e,
0x648, 0x644, 0x648, 0x627, 0x67d, 0x648, 0x6aa, 0x648, 0x631, 0x67d, 0x648, 0x645,
0x627, 0x626, 0x67d, 0x633, 0x67d, 0x6aa, 0x627, 0x633, 0x64a, 0x646, 0x67d, 0x20,
0x628, 0x631, 0x67f, 0x644, 0x64a, 0x645, 0x64a, 0x633, 0x64a, 0x646, 0x67d, 0x20,
0x62c, 0x627, 0x646, 0x632, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x6aa, 0x67d, 0x633,
0x633, 0x64a, 0x646, 0x67d, 0x20, 0x644, 0x648, 0x633, 0x64a, 0x627, 0x633, 0x64a,
0x646, 0x67d, 0x20, 0x67f, 0x627, 0x645, 0x633, 0x633, 0x64a, 0x646, 0x67d, 0x20,
0x648, 0x646, 0x633, 0x64a, 0x646, 0x67d, 0x633, 0x648, 0x626, 0x641, 0x67d, 0x20,
0x6aa, 0x631, 0x646, 0x67d, 0x67d, 0x64a, 0x6af, 0x648, 0x633, 0x64a, 0x6af, 0x644,
0x67e, 0x627, 0x67f, 0x648, 0x644, 0x64a, 0x62a, 0x64a, 0x62c, 0x648, 0x627, 0x646,
0x627, 0x67d, 0x648, 0x631, 0x646, 0x67d, 0x648, 0x67d, 0x648, 0x631, 0x67d, 0x648,
0x644, 0x627, 0x648, 0x64a, 0x646, 0x6aa, 0x648, 0x648, 0x631, 0x648, 0x627, 0x626,
0x64a, 0x67d, 0x20, 0x647, 0x627, 0x626, 0x648, 0x633, 0x648, 0x646, 0x64a, 0x20,
0x67e, 0x64a, 0x6af, 0x64a, 0x627, 0x6aa, 0x648, 0x62a, 0x627, 0x62a, 0x6aa, 0x64a,
0x633, 0x64a, 0x68a, 0x64a, 0x648, 0x633, 0x68a, 0x648, 0x645, 0x627, 0x646, 0x67d,
0x20, 0x62f, 0x631, 0x648, 0x64a, 0x626, 0x644, 0x645, 0x6aa, 0x648, 0x627, 0x626,
0x631, 0x64a, 0x645, 0x648, 0x633, 0x646, 0x645, 0x64a, 0x6aa, 0x20, 0x645, 0x631,
0x68a, 0x648, 0x67e, 0x627, 0x645, 0x631, 0x631, 0x648, 0x68c, 0x631, 0x627, 0x633,
0x64a, 0x648, 0x648, 0x627, 0x67d, 0x631, 0x648, 0x644, 0x648, 0x648, 0x633, 0x62a,
0x648, 0x6aa, 0x644, 0x627, 0x646, 0x6af, 0x20, 0x627, 0x626, 0x64a, 0x631, 0x628,
0x646, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x64a, 0x627, 0x646, 0x64a, 0x62f, 0x631,
0x627, 0x6aa, 0x67d, 0x627, 0x626, 0x648, 0x627, 0x64a, 0x6aa, 0x67d, 0x648, 0x628,
0x622, 0x634, 0x6af, 0x627, 0x628, 0x627, 0x62f, 0x622, 0x62a, 0x64a, 0x631, 0x627,
0x626, 0x648, 0x628, 0x626, 0x646, 0x6aa, 0x627, 0x6aa, 0x628, 0x631, 0x646, 0x644,
0x628, 0x634, 0x6aa, 0x64a, 0x6aa, 0x628, 0x631, 0x648, 0x646, 0x627, 0x626, 0x64a,
0x6aa, 0x648, 0x644, 0x645, 0x628, 0x648, 0x68d, 0x627, 0x6aa, 0x627, 0x62f, 0x628,
0x626, 0x64a, 0x647, 0x648, 0x686, 0x64a, 0x20, 0x645, 0x646, 0x647, 0x627, 0x646,
0x6af, 0x20, 0x6aa, 0x627, 0x646, 0x6af, 0x647, 0x648, 0x648, 0x68a, 0x627, 0x631,
0x6aa, 0x62a, 0x633, 0x6aa, 0x62c, 0x6aa, 0x627, 0x631, 0x62a, 0x627, 0x62c, 0x64a,
0x627, 0x67e, 0x648, 0x631, 0x627, 0x64a, 0x648, 0x631, 0x648, 0x634, 0x644, 0x645,
0x6aa, 0x627, 0x628, 0x644, 0x6aa, 0x645, 0x686, 0x627, 0x633, 0x6aa, 0x64a, 0x6aa,
0x631, 0x627, 0x686, 0x64a, 0x6a9, 0x67d, 0x645, 0x646, 0x68a, 0x648, 0x6a9, 0x646,
0x62f, 0x64a, 0x627, 0x6af, 0x627, 0x6aa, 0x644, 0x6aa, 0x62a, 0x627, 0x6aa, 0x631,
0x64a, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x6aa, 0x6aa, 0x648, 0x627,
0x644, 0x627, 0x644, 0x645, 0x67e, 0x648, 0x631, 0x6a9, 0x648, 0x686, 0x646, 0x6af,
0x6aa, 0x648, 0x64a, 0x62a, 0x645, 0x6aa, 0x627, 0x626, 0x648, 0x645, 0x6af, 0x627,
0x62f, 0x627, 0x646, 0x645, 0x64a, 0x6aa, 0x627, 0x633, 0x631, 0x645, 0x646, 0x64a,
0x644, 0x627, 0x646, 0x64a, 0x6aa, 0x648, 0x633, 0x64a, 0x627, 0x646, 0x648, 0x648,
0x6aa, 0x632, 0x646, 0x64a, 0x62a, 0x633, 0x6aa, 0x646, 0x648, 0x648, 0x627, 0x633,
0x628, 0x626, 0x64a, 0x631, 0x633, 0x6aa, 0x627, 0x648, 0x645, 0x633, 0x6aa, 0x632,
0x628, 0x627, 0x646, 0x64a, 0x641, 0x646, 0x627, 0x645, 0x20, 0x67e, 0x64a, 0x646,
0x647, 0x67e, 0x648, 0x646, 0x67d, 0x64a, 0x627, 0x646, 0x6aa, 0x634, 0x64a, 0x627,
0x646, 0x6af, 0x20, 0x64a, 0x627, 0x646, 0x6af, 0x6aa, 0x648, 0x67d, 0x627, 0x646,
0x633, 0x64a, 0x6aa, 0x64a, 0x632, 0x644, 0x648, 0x631, 0x68a, 0x627, 0x634, 0x62e,
0x627, 0x644, 0x64a, 0x646, 0x633, 0x64a, 0x648, 0x644, 0x634, 0x646, 0x6af, 0x647,
0x627, 0x626, 0x64a, 0x633, 0x631, 0x64a, 0x62f, 0x64a, 0x6aa, 0x648, 0x644, 0x64a,
0x645, 0x633, 0x6aa, 0x62a, 0x627, 0x626, 0x67e, 0x64a, 0x62a, 0x627, 0x634, 0x642,
0x646, 0x62a, 0x67f, 0x645, 0x641, 0x648, 0x67d, 0x648, 0x6aa, 0x64a, 0x648, 0x62a,
0x645, 0x633, 0x6aa, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x62a,
0x631, 0x64a, 0x648, 0x631, 0x645, 0x6aa, 0x64a, 0x627, 0x633, 0x67d, 0x20, 0x646,
0x64a, 0x631, 0x627, 0x648, 0x64a, 0x646, 0x67d, 0x64a, 0x627, 0x646, 0x648, 0x644,
0x627, 0x62f, 0x648, 0x648, 0x633, 0x62a, 0x648, 0x6aa, 0x64a, 0x627, 0x6aa, 0x62a,
0x633, 0x6aa, 0x631, 0x646, 0x6af, 0x648, 0x646, 0x6cc, 0x6aa, 0x627, 0x67d, 0x631,
0x646, 0x628, 0x631, 0x6af, 0x627, 0x632, 0x648, 0x631, 0x632, 0x6aa, 0x64a, 0x646,
0x631, 0x64a, 0x6aa, 0x64a, 0x67e, 0x20, 0x648, 0x631, 0x68a, 0x64a, 0x641, 0x64a,
0x631, 0x648, 0x645, 0x627, 0x68a, 0x64a, 0x631, 0x627, 0x631, 0x64a, 0x6aa, 0x64a,
0x648, 0x6aa, 0x68f, 0x6a9, 0x6bb, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627,
0x633, 0x64a, 0x646, 0x67d, 0x20, 0x647, 0x64a, 0x644, 0x64a, 0x646, 0x627, 0x627,
0x633, 0x67d, 0x64a, 0x646, 0x644, 0x64a, 0x627, 0x64a, 0x68a, 0x64a, 0x644, 0x64a,
0x68a, 0x628, 0x631, 0x633, 0x628, 0x64a, 0x646, 0x628, 0x631, 0x648, 0x6aa, 0x646,
0x20, 0x647, 0x644, 0x68a, 0x627, 0x631, 0x648, 0x646, 0x64a, 0x648, 0x6aa, 0x644,
0x627, 0x647, 0x648, 0x628, 0x627, 0x631, 0x67d, 0x644, 0x646, 0x68a, 0x645, 0x64a,
0x646, 0x644, 0x627, 0x631, 0x68a, 0x20, 0x647, 0x627, 0x626, 0x648, 0x67e, 0x631,
0x67f, 0x633, 0x68a, 0x646, 0x64a, 0x627, 0x64a, 0x645, 0x633, 0x67d, 0x631, 0x68a,
0x64a, 0x645, 0x627, 0x64a, 0x67f, 0x646, 0x632, 0x628, 0x631, 0x627, 0x62a, 0x64a,
0x633, 0x644, 0x648, 0x627, 0x628, 0x631, 0x633, 0x644, 0x632, 0x628, 0x62e, 0x627,
0x631, 0x64a, 0x633, 0x67d, 0x628, 0x62f, 0x627, 0x67e, 0x64a, 0x633, 0x67d, 0x628,
0x632, 0x64a, 0x62c, 0x64a, 0x646, 0x686, 0x633, 0x64a, 0x646, 0x627, 0x626, 0x648,
0x6aa, 0x648, 0x67e, 0x646, 0x20, 0x647, 0x64a, 0x6af, 0x646, 0x68a, 0x628, 0x644,
0x646, 0x62c, 0x628, 0x631, 0x627, 0x644, 0x67d, 0x631, 0x6af, 0x631, 0x646, 0x632,
0x64a, 0x647, 0x64a, 0x644, 0x633, 0x646, 0x6aa, 0x64a, 0x622, 0x626, 0x64a, 0x632,
0x644, 0x20, 0x622, 0x641, 0x20, 0x645, 0x64a, 0x646, 0x62c, 0x631, 0x633, 0x64a,
0x6aa, 0x644, 0x64a, 0x646, 0x646, 0x20, 0x6af, 0x631, 0x627, 0x68a, 0x6aa, 0x64a,
0x631, 0x648, 0x641, 0x6aa, 0x650, 0x64a, 0x648, 0x644, 0x633, 0x628, 0x646, 0x644,
0x628, 0x644, 0x64a, 0x627, 0x646, 0x627, 0x644, 0x646, 0x68a, 0x646, 0x644, 0x6af,
0x632, 0x645, 0x628, 0x631, 0x6af, 0x645, 0x64a, 0x68a, 0x631, 0x68a, 0x645, 0x627,
0x644, 0x67d, 0x627, 0x645, 0x646, 0x633, 0x6aa, 0x645, 0x648, 0x646, 0x627, 0x6aa,
0x648, 0x645, 0x627, 0x633, 0x6aa, 0x648, 0x67e, 0x626, 0x631, 0x633, 0x67e, 0x648,
0x68a, 0x6af, 0x648, 0x631, 0x633, 0x64a, 0x627, 0x633, 0x64a, 0x646, 0x20, 0x645,
0x631, 0x64a, 0x646, 0x648, 0x633, 0x631, 0x627, 0x62c, 0x6cc, 0x648, 0x648, 0x633,
0x631, 0x627, 0x62a, 0x648, 0x633, 0x645, 0x641, 0x631, 0x648, 0x67e, 0x648, 0x644,
0x627, 0x633, 0x6aa, 0x648, 0x67e, 0x64a, 0x627, 0x633, 0x67d, 0x627, 0x6aa, 0x20,
0x647, 0x648, 0x645, 0x67d, 0x627, 0x644, 0x646, 0x62a, 0x631, 0x627, 0x646, 0x64a,
0x627, 0x644, 0x64a, 0x627, 0x646, 0x648, 0x633, 0x6aa, 0x648, 0x68a, 0x648, 0x632,
0x648, 0x626, 0x67d, 0x64a, 0x6aa, 0x646, 0x648, 0x64a, 0x644, 0x646, 0x64a, 0x633,
0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x627, 0x646, 0x62a,
0x627, 0x646, 0x627, 0x646, 0x627, 0x631, 0x64a, 0x648, 0x648, 0x6aa, 0x631, 0x633,
0x645, 0x633, 0x6aa, 0x648, 0x6aa, 0x648, 0x633, 0x6aa, 0x648, 0x645, 0x648, 0x631,
0x648, 0x6aa, 0x631, 0x6af, 0x64a, 0x648, 0x644, 0x64a, 0x646, 0x645, 0x648, 0x631,
0x64a, 0x634, 0x633, 0x645, 0x64a, 0x627, 0x67d, 0x64a, 0x622, 0x6aa, 0x644, 0x64a,
0x646, 0x68a, 0x628, 0x648, 0x6af, 0x64a, 0x646, 0x20, 0x648, 0x64a, 0x644, 0x64a,
0x627, 0x686, 0x626, 0x67f, 0x645, 0x686, 0x64a, 0x648, 0x6aa, 0x627, 0x64a, 0x633,
0x67d, 0x631, 0x627, 0x641, 0x627, 0x62a, 0x64a, 0x641, 0x6aa, 0x627, 0x626, 0x648,
0x641, 0x648, 0x641, 0x646, 0x627, 0x641, 0x648, 0x67d, 0x64a, 0x6af, 0x627, 0x644,
0x627, 0x67e, 0x627, 0x6af, 0x648, 0x632, 0x6af, 0x64a, 0x645, 0x628, 0x64a, 0x626,
0x631, 0x6af, 0x648, 0x627, 0x68a, 0x644, 0x20, 0x6aa, 0x64a, 0x646, 0x627, 0x644,
0x647, 0x648, 0x646, 0x648, 0x20, 0x644, 0x648, 0x644, 0x648, 0x627, 0x64a, 0x646,
0x68a, 0x631, 0x628, 0x631, 0x64a, 0x6aa, 0x631, 0x64a, 0x67d, 0x645, 0x67d, 0x64a,
0x6aa, 0x648, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x6aa, 0x648, 0x627, 0x62c, 0x644,
0x64a, 0x646, 0x645, 0x631, 0x6aa, 0x64a, 0x648, 0x633, 0x633, 0x645, 0x68a, 0x648,
0x64a, 0x646, 0x627, 0x626, 0x631, 0x648, 0x646, 0x64a, 0x648, 0x648, 0x64a, 0x646,
0x627, 0x631, 0x20, 0x641, 0x648, 0x6aa, 0x646, 0x627, 0x626, 0x648, 0x645, 0x64a,
0x627, 0x67e, 0x627, 0x6af, 0x648, 0x20, 0x67e, 0x627, 0x6af, 0x648, 0x67e, 0x644,
0x627, 0x626, 0x648, 0x67e, 0x67d, 0x6aa, 0x64a, 0x631, 0x646, 0x67e, 0x648, 0x646,
0x67e, 0x6cc, 0x626, 0x64a, 0x67e, 0x648, 0x631, 0x67d, 0x20, 0x645, 0x648, 0x631,
0x633, 0x628, 0x64a, 0x631, 0x627, 0x631, 0x648, 0x67d, 0x648, 0x646, 0x6af, 0x627,
0x633, 0x64a, 0x67e, 0x646, 0x62a, 0x627, 0x647, 0x67d, 0x64a, 0x62a, 0x631, 0x627,
0x648, 0x627, 0x67d, 0x648, 0x646, 0x6af, 0x627, 0x67d, 0x627, 0x67e, 0x648, 0x648,
0x64a, 0x6aa, 0x648, 0x64a, 0x644, 0x633, 0xd85, 0xdb6, 0xdd2, 0xda2, 0xdcf, 0xdb1,
0xdca, 0xd85, 0xd9a, 0xdca, 0xdbb, 0xdcf, 0xd85, 0xda9, 0xdd2, 0xdc3, 0xdca, 0x20,
0xd85, 0xdb6, 0xdcf, 0xdb6, 0xdcf, 0xd87, 0xdbd, 0xdca, 0xda2, 0xdd2, 0xdba, 0xdbb,
0xdca, 0xdc3, 0xdca, 0xd85, 0xdc3, 0xdca, 0xdb8, 0xdcf, 0xdbb, 0xdcf, 0xdb6, 0xdb8,
0xd9a, 0xddc, 0xdb6, 0xdb1, 0xdca, 0xd9c, 0xdd4, 0xdba, 0xdd2, 0xdb6, 0xdb1, 0xdca,
0xda2, 0xdd4, 0xdbd, 0xdca, 0xdb6, 0xdd2, 0xdc3, 0xddd, 0xdb6, 0xdca, 0xdbd, 0xdb1,
0xdca, 0xda7, 0xdba, 0xdbb, 0xdca, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdcf, 0xdc0,
0xdd2, 0xdbd, 0xdca, 0xdb6, 0xdd4, 0xda2, 0xdd4, 0xdb8, 0xdca, 0xdb6, 0xdd4, 0xdbb,
0xdcf, 0xd9a, 0xdba, 0xdd2, 0xdbb, 0xddd, 0xd9a, 0xdd0, 0xdc3, 0xdb6, 0xdca, 0xdbd,
0xdb1, 0xdca, 0xd9a, 0xdcf, 0xdc3, 0xdd9, 0xdba, 0xdd4, 0xda7, 0xdcf, 0xd9a, 0xddc,
0xdb1, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xda9, 0xd9a, 0xdbb, 0xdca, 0xdaf, 0xdcf,
0xdbb, 0xdca, 0x20, 0xd91, 0xdc3, 0xdca, 0x20, 0xdc3, 0xdbd, 0xdcf, 0xdb8, 0xdca,
0xda2, 0xdd2, 0xdb6, 0xdd4, 0xda7, 0xdd2, 0xdaf, 0xdde, 0xd85, 0xdbd, 0xdcf, 0xd91,
0xdbd, 0xdca, 0x20, 0xd86, 0xdba, 0xdd4, 0xdb1, 0xdca, 0xdc6, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xda7, 0xdc0, 0xdd4, 0xdb1, 0xdca, 0xd9c, 0xdd0, 0xdb6, 0xdbb, 0xddd, 0xdb1,
0xdca, 0xdc4, 0xdbb, 0xdcf, 0xdbb, 0xdda, 0xda2, 0xddc, 0xdc4, 0xdb1, 0xdca, 0xdb1,
0xdc3, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0xda2, 0xdd4, 0xdb6, 0xdcf, 0xd9a,
0xdb8, 0xdca, 0xdb4, 0xdcf, 0xdbd, 0xdcf, 0xd9a, 0xdcf, 0xdbb, 0xdca, 0xda7, 0xddc,
0xdb8, 0xdca, 0xd9a, 0xdd2, 0xd9c, 0xdcf, 0xdbd, 0xdd2, 0xd9a, 0xdd2, 0xdb1, 0xdca,
0xdc1, 0xdcf, 0xdc3, 0xdcf, 0xdbd, 0xdcf, 0xd9c, 0xddd, 0xdc3, 0xdca, 0xdbd, 0xdd2,
0xdb6, 0xdca, 0xdbb, 0xdc0, 0xdd2, 0xdbd, 0xdca, 0xdbd, 0xddd, 0xdb8, 0xdca, 0xdbd,
0xdd4, 0xdc0, 0xdb1, 0xdca, 0xda9, 0xdcf, 0xdbd, 0xdd4, 0xdb6, 0xdd4, 0xdb8, 0xdca,
0xdb6, 0xdcf, 0xdc1, 0xdd2, 0xdbd, 0xdd4, 0xdc3, 0xdcf, 0xd9a, 0xdcf, 0xdb8, 0xdbd,
0xdb6, 0xddd, 0xdb8, 0xdb4, 0xdd4, 0xda7, 0xddd, 0xdb8, 0xdc3, 0xdda, 0xdbb, 0xdd4,
0xdb8, 0xdca, 0xdb6, 0xdb6, 0xdcf, 0xdb1, 0xdda, 0xdb8, 0xddc, 0xd9c, 0xda9, 0xdd2,
0xdc2, 0xdd4, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xdbb, 0xddc, 0xdc0, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdba, 0xdd2, 0xdbb, 0xddd, 0xdb6, 0xdd2, 0xdaf, 0xdca, 0xda2, 0xdb8, 0xdd9,
0xdb1, 0xdcf, 0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb8, 0xdda, 0xdb1, 0xdd4, 0xd86, 0xd9a,
0xdca, 0xda0, 0xddc, 0xda7, 0xdca, 0xd94, 0xd86, 0xd9c, 0xda9, 0xddc, 0xd9c, 0xddc,
0xdb4, 0xddc, 0xdbb, 0xdca, 0xdad, 0xddd, 0x2d, 0xdb1, 0xddc, 0xdc0, 0xddd, 0xdc3,
0xdcf, 0xd95, 0x20, 0xdad, 0xddd, 0xdb8, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdd2,
0xdb4, 0xddc, 0xdbd, 0xdd2, 0xda7, 0xdd2, 0xdba, 0xdd4, 0xdb1, 0xdd2, 0xdc3, 0xdca,
0xdc0, 0xdd2, 0xdb1, 0xdca, 0xda9, 0xdca, 0xdc4, 0xddc, 0xd91, 0xd9a, 0xdca, 0xd87,
0xda9, 0xd9a, 0xdca, 0xd87, 0xdb1, 0xd9a, 0xddc, 0xdbb, 0xdcf, 0xda2, 0xdca, 0xd87,
0xdb1, 0xdca, 0xd9c, 0xdd4, 0xdba, 0xdd2, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdb1, 0xdca,
0xda7, 0xdd2, 0xd9c, 0xdd4, 0xdc0, 0xdcf, 0xd87, 0xdbb, 0xd9c, 0xdd4, 0xd85, 0xdba,
0xdd2, 0xdb1, 0xdcf, 0xdb6, 0xdd4, 0xdc0, 0xdb1, 0xddd, 0xdc3, 0xdca, 0x20, 0xd85,
0xdba, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xda7, 0xdb8, 0xdbb, 0xdca, 0xd9a, 0xdcf,
0xd9a, 0xddc, 0xdbb, 0xdca, 0xdaf, 0xddc, 0xdb6, 0xdcf, 0xda2, 0xdd4, 0xda2, 0xdd4,
0xdba, 0xdd2, 0xdbd, 0xdcf, 0x20, 0xdbb, 0xdd2, 0xda2, 0xddd, 0xdbd, 0xdcf, 0xdb8,
0xdd9, 0xdb1, 0xdca, 0xda9, 0xddd, 0xdc3, 0xdcf, 0xdbb, 0xdd2, 0xdba, 0xddd, 0x20,
0xd9c, 0xdbd, 0xdd9, 0xd9c, 0xddc, 0xdc3, 0xdca, 0xdc3, 0xddc, 0xdbd, 0xdca, 0xda7,
0xdcf, 0xdc3, 0xdd0, 0xdb1, 0xdca, 0x20, 0xda2, 0xdd4, 0xdc0, 0xdcf, 0xdb1, 0xdca,
0xdc3, 0xdd0, 0xdb1, 0xdca, 0x20, 0xdbd, 0xdd4, 0xdc0, 0xdd2, 0xdc3, 0xdca, 0xda7,
0xdd4, 0xd9a, 0xdd4, 0xdb8, 0xdb1, 0xdca, 0xd8b, 0xdc2, 0xdd4, 0xd85, 0xdba, 0xdd2,
0xdba, 0xdcf, 0xd85, 0xdbb, 0xdd6, 0xdb6, 0xdcf, 0xd85, 0xdc3, 0xdd4, 0xdb1, 0xdca,
0xdc3, 0xdd2, 0xdba, 0xddc, 0xdb1, 0xdca, 0xd87, 0xda7, 0xdd2, 0xd9a, 0xddc, 0xd9a,
0xdb1, 0xdca, 0xdb6, 0xdc4, 0xdd2, 0xdba, 0xdcf, 0xdb6, 0xdc4, 0xdd2, 0xdba, 0xdcf,
0x20, 0xdb6, 0xdb1, 0xdca, 0xdaf, 0xdd9, 0xdbb, 0xdcf, 0xdc3, 0xdca, 0xdb6, 0xdcf,
0xdb6, 0xda9, 0xddd, 0xdc3, 0xdca, 0xdb6, 0xdd9, 0xdbd, 0xdd9, 0xdb8, 0xdca, 0xdb6,
0xdd9, 0xdbd, 0xdd3, 0xdc3, 0xdca, 0xdb6, 0xdca, 0xdbd, 0xdd0, 0xdb1, 0xdca, 0xd9a,
0xdca, 0x2d, 0xdc3, 0xdd0, 0xdb6, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdb6, 0xddc, 0xd86,
0x20, 0xdc0, 0xdd2, 0xdc3, 0xdca, 0xda7, 0xdcf, 0xdb6, 0xddc, 0xd9c, 0xddc, 0xda7,
0xdcf, 0xdb6, 0xddc, 0xdba, 0xdd2, 0xdc3, 0xdd2, 0xd9a, 0xdda, 0xdb8, 0xdca, 0xdb6,
0xdca, 0x200d, 0xdbb, 0xdd2, 0xda2, 0xdca, 0x20, 0xdb6, 0xddc, 0xd9a, 0xdca, 0xd9a,
0xd9a, 0xdb8, 0xdca, 0xdb4, 0xddd, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd1, 0xdb1,
0xdca, 0xda9, 0xdca, 0xd9a, 0xdd0, 0xdb1, 0xdca, 0xd9a, 0xdd4, 0xdb1, 0xdca, 0xd9a,
0xdbb, 0xd9a, 0xdc3, 0xdca, 0xd9a, 0xdda, 0xdba, 0xdd9, 0xdb1, 0xdca, 0xd9a, 0xdda,
0xdb8, 0xdb1, 0xdca, 0xda0, 0xdd2, 0xd9a, 0xdcf, 0xd9c, 0xddd, 0xda0, 0xdd2, 0xdc4,
0xdd4, 0xd86, 0xdc4, 0xdd4, 0xd86, 0xdc3, 0xdd2, 0xdba, 0xdd4, 0xda9, 0xdcf, 0xda9,
0xdca, 0x20, 0xda2, 0xdd4, 0xdc0, 0xdcf, 0xdbb, 0xdd9, 0xdc3, 0xdca, 0xd9a, 0xddc,
0xdc3, 0xdca, 0xda7, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdd9,
0xdc3, 0xdca, 0xda7, 0xdb1, 0xdca, 0xd9a, 0xdd4, 0xdba, 0xdd2, 0xd85, 0xdb6, 0xdcf,
0xd9a, 0xdd4, 0xdbb, 0xd9a, 0xdcf, 0xdc0, 0xddd, 0xda9, 0xdd9, 0xdb1, 0xdca, 0xdb8,
0xdcf, 0xdbb, 0xdca, 0xd9a, 0xdca, 0xdc1, 0xdc0, 0xdca, 0xdb1, 0xdca, 0xda9, 0xddd,
0xdc3, 0xdb1, 0xdca, 0xda9, 0xddd, 0xdc3, 0xdb1, 0xdca, 0x20, 0xd9a, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xd9a, 0xdca, 0xda9, 0xdd9, 0xdb1, 0xdca, 0xdc0, 0xdbb, 0xdca, 0xda9,
0xdd9, 0xdad, 0xdca, 0xdbb, 0xdd4, 0xdc0, 0xdcf, 0xda9, 0xddc, 0xdb8, 0xdd2, 0xdb1,
0xdd2, 0xd9a, 0xdcf, 0xd91, 0xda9, 0xdca, 0xdb8, 0xdb1, 0xdca, 0xda7, 0xdb1, 0xdca,
0xd91, 0xdba, 0xdd2, 0xdbb, 0xdd4, 0xdb1, 0xdda, 0xdb4, 0xdca, 0xd91, 0xdbd, 0xdca,
0xdc3, 0xdd0, 0xdbd, 0xdca, 0xdc0, 0xdaf, 0xddd, 0xdbb, 0xdca, 0xdc6, 0xddd, 0xda7,
0xdca, 0x20, 0xdb1, 0xdd9, 0xdbd, 0xdca, 0xdc3, 0xdb1, 0xdca, 0xdc6, 0xddc, 0xdbb,
0xdca, 0xda7, 0xdbd, 0xdda, 0xdc3, 0xdcf, 0xd9c, 0xdca, 0xdbd, 0xdda, 0xdc3, 0xdca,
0x20, 0xdb6, 0xddc, 0xd9a, 0xdca, 0xd9a, 0xd9c, 0xdd6, 0xdc3, 0xdca, 0x20, 0xdb6,
0xddc, 0xd9a, 0xdca, 0xd9a, 0xd9c, 0xdca, 0xdbb, 0xdd1, 0xdb1, 0xdca, 0xda9, 0xdca,
0x20, 0xda7, 0xdbb, 0xdca, 0xd9a, 0xdca, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd0, 0xdb1,
0xda9, 0xdcf, 0xd9c, 0xdca, 0xdc0, 0xdcf, 0xda9, 0xdca, 0xdbd, 0xddd, 0xdb4, 0xdca,
0xd9c, 0xddd, 0xdad, 0xdb8, 0xdcf, 0xdbd, 0xdcf, 0xdc0, 0xd9c, 0xdd4, 0xd85, 0xdba,
0xdcf, 0xd9a, 0xdd4, 0xdba, 0xdd2, 0xdbd, 0xdca, 0xd9c, 0xdba, 0xdb1, 0xdcf, 0xdc0,
0xdc4, 0xdbd, 0xdd2, 0xdc6, 0xdd0, 0xd9a, 0xdca, 0xdc3, 0xdca, 0xdc4, 0xdc0, 0xdcf,
0xdb1, 0xdcf, 0xdc4, 0xdd9, 0xdb8, 0xddc, 0xdc3, 0xdd2, 0xdbd, 0xddd, 0xd89, 0xdb1,
0xdca, 0xda9, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdb4, 0xddc, 0xdbd, 0xdd2, 0xdc3, 0xdca,
0xdb1, 0xddc, 0xd9a, 0xdca, 0xdc3, 0xdca, 0x2c, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf,
0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xdb8, 0xdbb, 0xdd9, 0xdb1, 0xdca, 0xd9c, 0xddd,
0x2c, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xdb4,
0xdd3, 0xda7, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0x2c,
0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xda7, 0xdd9,
0xdbd, 0xdca, 0x20, 0xdb1, 0xd9c, 0xdbb, 0xdba, 0x2c, 0x20, 0xd89, 0xdb1, 0xdca,
0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xdc0, 0xdd9, 0xdc0, 0xdda, 0x2c, 0x20,
0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xdc0, 0xdd2, 0xdb1,
0xdca, 0xdc3, 0xdb1, 0xdc3, 0xdca, 0x2c, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xdcf, 0xdc0, 0xdd2, 0xdb1, 0xdb8, 0xdd0, 0xd9a, 0xdca, 0x2c,
0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xd89, 0xdb1,
0xdd4, 0xdc0, 0xdd2, 0xd9a, 0xdca, 0xd89, 0xd9a, 0xdcf, 0xdbd, 0xdd4, 0xdba, 0xdd2,
0xda7, 0xdca, 0xda2, 0xdd0, 0xdb8, 0xdd9, 0xdba, 0xdd2, 0xd9a, 0xdcf, 0xda2, 0xdd6,
0xdb1, 0xddd, 0xdbd, 0xdd4, 0xdc0, 0xdd2, 0xdc3, 0xdca, 0xdc0, 0xdd2, 0xdbd, 0xdca,
0xdb8, 0xddc, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdc3, 0xdd9, 0xdbd, 0xddd, 0x2c, 0x20,
0xd9a, 0xdd9, 0xdb1, 0xdca, 0xda7, 0xd9a, 0xdd2, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdbd,
0xdd9, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xda2, 0xdca, 0xd9a, 0xdca, 0xdbd, 0xdcf, 0x20,
0xdb4, 0xdc3, 0xdca, 0xdbd, 0xdd3, 0xdb8, 0xdcf, 0xdbd, 0xddc, 0xdc3, 0xdca, 0x20,
0xd87, 0xdb1, 0xdca, 0xda2, 0xdbd, 0xdd3, 0xdc3, 0xdca, 0xdbd, 0xddd, 0xdc0, 0xdbb,
0xdca, 0x20, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb1, 0xdca, 0xdc3, 0xdc3, 0xdca,
0x20, 0xd9a, 0xdca, 0xdc0, 0xdcf, 0xda7, 0xdbb, 0xdca, 0xdb8, 0xdc3, 0xdda, 0xdba,
0xdd2, 0xdba, 0xddd, 0xdb8, 0xdb1, 0xd9c, 0xdd4, 0xd86, 0xdb8, 0xdd0, 0xdb1, 0xddd,
0xdc3, 0xdca, 0xdb8, 0xdd0, 0xdbb, 0xdd2, 0xd9c, 0xddc, 0xda7, 0xdca, 0xdb8, 0xdcf,
0xda7, 0xdd2, 0xdb1, 0xdd2, 0xd9a, 0xdca, 0xdb8, 0xda7, 0xdb8, 0xddc, 0xdbb, 0xddc,
0xdc3, 0xdca, 0xdb8, 0xdc3, 0xda7, 0xdca, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0xdb8, 0xdd9,
0xdb1, 0xddc, 0xdb8, 0xdd2, 0xdb1, 0xdd3, 0xdb8, 0xdd9, 0xdbb, 0xdd2, 0xda9, 0xdcf,
0xdb8, 0xdd9, 0xda7, 0xdca, 0xdbd, 0xd9a, 0xda7, 0xdca, 0xdbd, 0xdcf, 0xdb8, 0xdd9,
0xd9a, 0xdca, 0xdc3, 0xdd2, 0xd9a, 0xddd, 0x20, 0xdb1, 0xd9c, 0xdbb, 0xdba, 0xdb8,
0xdd2, 0xd9a, 0xdd2, 0xdba, 0xdd4, 0xdbd, 0xdb1, 0xdca, 0xdb8, 0xddc, 0xdb1, 0xdca,
0xd9a, 0xdca, 0xda7, 0xdb1, 0xdca, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xda7, 0xdd9, 0xdbb,
0xdda, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xda7, 0xdd9, 0xdc0, 0xdd2, 0xda9, 0xdd2, 0xdba,
0xddd, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xda7, 0xdca, 0xdc3, 0xdda, 0xdbb, 0xdcf, 0xda7,
0xdca, 0xdb1, 0xdc3, 0xdca, 0xdc3, 0xddd, 0xdb1, 0xdd2, 0xdc0, 0xdca, 0xdba, 0xddd,
0xd9a, 0xdca, 0xdb1, 0xddd, 0xdb8, 0xdca, 0xdb1, 0xddc, 0xdbb, 0xddc, 0xdb1, 0xdca,
0xdc4, 0xdcf, 0xdb6, 0xdd2, 0xdba, 0xdd4, 0xdbd, 0xdcf, 0x2c, 0x20, 0xd8b, 0xdad,
0xdd4, 0xdbb, 0xdd4, 0x20, 0xda9, 0xd9a, 0xddd, 0xda7, 0xdcf, 0xdb8, 0xdb0, 0xdca,
0x200d, 0xdba, 0xdb8, 0x2c, 0x20, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xda9,
0xd9a, 0xddd, 0xda7, 0xdcf, 0xdb1, 0xdc0, 0x20, 0xdc3, 0xdbd, 0xdd9, 0xdb8, 0xdca,
0xdb8, 0x2c, 0x20, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xda9, 0xd9a, 0xddd,
0xda7, 0xdcf, 0xdb1, 0xdd6, 0xd9a, 0xdca, 0xd94, 0xda2, 0xdd2, 0xdb1, 0xd9c, 0xdcf,
0xdb4, 0xdd0, 0xdb1, 0xdb8, 0xdcf, 0xdb4, 0xdd0, 0xdbb, 0xdcf, 0xdb8, 0xdbb, 0xdd2,
0xdb6, 0xddd, 0xdc6, 0xdd3, 0xdb1, 0xdd2, 0xd9a, 0xdca, 0xdc3, 0xdca, 0xdb4, 0xddc,
0xdbb, 0xdca, 0xda7, 0xdca, 0x2d, 0xd95, 0x2d, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2,
0xdb1, 0xdca, 0xdc3, 0xdca, 0xdb4, 0xddc, 0xdbb, 0xdca, 0xda7, 0xdca, 0x20, 0xd94,
0xdc6, 0xdca, 0x20, 0xdc3, 0xdca, 0xdb4, 0xdd9, 0xdba, 0xdd2, 0xdb1, 0xdca, 0xdb4,
0xddc, 0xdbb, 0xdca, 0xdad, 0xddd, 0x20, 0xdc0, 0xdd9, 0xdbd, 0xdca, 0xdc4, 0xddd,
0xdb4, 0xdd4, 0xdc0, 0xda7, 0xddc, 0xdbb, 0xdd2, 0xd9a, 0xddd, 0xdc0, 0xdb4, 0xdd4,
0xdb1, 0xdca, 0xda7, 0xdcf, 0x20, 0xd87, 0xdbb, 0xdd2, 0xdb1, 0xdcf, 0xdc3, 0xdca,
0xdbb, 0xdd0, 0xdb1, 0xdca, 0xd9a, 0xdd2, 0xdb1, 0xdca, 0x20, 0xdb4, 0xdd2, 0xdc0,
0xdd2, 0xdc3, 0xdd4, 0xdb8, 0xdbb, 0xdd9, 0xdc3, 0xdd2, 0xdc6, 0xdca, 0xdbb, 0xdd9,
0xda2, 0xdd2, 0xdb1, 0xdcf, 0xdbb, 0xdd9, 0xdc3, 0xddc, 0xdbd, 0xdd4, 0xda7, 0xdca,
0xdbb, 0xdd2, 0xdba, 0xddd, 0x20, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdb1, 0xdca, 0xd9a,
0xddd, 0xdc3, 0xdb1, 0xdca, 0xda7, 0xdbb, 0xdda, 0xdb8, 0xdca, 0xdc3, 0xdb1, 0xdca,
0xdad, 0xdd2, 0xdba, 0xdcf, 0xd9c, 0xddd, 0xdc3, 0xdd0, 0xdb1, 0xdca, 0xda7, 0xddd,
0x20, 0xda9, 0xddc, 0xdb8, 0xdd2, 0xdb1, 0xdca, 0xd9c, 0xddd, 0xdc3, 0xdcf, 0xdc0,
0xddd, 0x20, 0xdb4, 0xddd, 0xdbd, 0xddc, 0xd89, 0xda7, 0xddc, 0xd9a, 0xddc, 0xdbb,
0xdca, 0xda7, 0xddd, 0xdb8, 0xdd2, 0xda7, 0xdca, 0xdc3, 0xdd2, 0xda7, 0xdca, 0xd9a,
0xdcf, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xdb6, 0xdbb, 0xdca, 0xdad, 0xdbd,
0xdd9, 0xdb8, 0xdd2, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xda2, 0xddd, 0xdb1,
0xdca, 0xd9c, 0xdda, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xd9a, 0xdd2, 0xda7,
0xdca, 0xdc3, 0xdca, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xdbd, 0xdd4, 0xdc3,
0xdd2, 0xdba, 0xdcf, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xdad, 0xddd, 0xdb8,
0xdc3, 0xdca, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc0, 0xdd2, 0xdb1, 0xdca,
0xda7, 0xdca, 0xdc3, 0xdb1, 0xdca, 0xdc3, 0xdca, 0xdc0, 0xdd2, 0xdc6, 0xdca, 0xda7,
0xdca, 0x20, 0xd9a, 0xdbb, 0xdb1, 0xdca, 0xda7, 0xdca, 0xda7, 0xdd9, 0xd9c, 0xdd4,
0xdc3, 0xdd2, 0xd9c, 0xdbd, 0xdca, 0xdb4, 0xdcf, 0xdad, 0xdd4, 0xdbd, 0xdda, 0xda7,
0xdd2, 0xda2, 0xdd4, 0xd86, 0xdb1, 0xdcf, 0xda7, 0xddc, 0xdbb, 0xdb1, 0xdca, 0xda7,
0xddd, 0xda7, 0xddc, 0xdbb, 0xdca, 0xda7, 0xddd, 0xdbd, 0xdcf, 0xdc0, 0xdd0, 0xdb1,
0xdca, 0xd9a, 0xdd4, 0xdc0, 0xdbb, 0xdca, 0xdc0, 0xdba, 0xdd2, 0xda7, 0xdca, 0xdc4,
0xddd, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xdc0, 0xdd2, 0xdb1, 0xdd2, 0xdb4, 0xdd9, 0xd9c,
0xdca, 0xdba, 0xd9a, 0xdd4, 0xda7, 0xda7, 0xdca, 0xd9a, 0xdd0, 0xdc3, 0xdda, 0xda9,
0xdda, 0xdc0, 0xdd2, 0xdc3, 0xdca, 0xda9, 0xdd4, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xda7,
0xdca, 0x20, 0xda9, 0xdd4, 0x2018, 0xdbb, 0xdca, 0xdc0, 0xdd2, 0xdbd, 0xdca, 0xdb8,
0xd9a, 0xdca, 0xd9a, 0xdd4, 0xdc0, 0xdbb, 0xdd2, 0xdb8, 0xdc0, 0xdca, 0xdc3, 0xdb1,
0xdca, 0xdb8, 0xdd0, 0xd9a, 0xdca, 0xdb8, 0xdd4, 0xdbb, 0xdca, 0xda9, 0xddd, 0xdb4,
0xdd0, 0xdbd, 0xdca, 0xdb8, 0xdbb, 0xdca, 0xdbb, 0xddc, 0xdad, 0xdd9, 0xdbb, 0xdcf,
0xdc3, 0xdd2, 0xdba, 0xddd, 0xdc0, 0xdcf, 0xda7, 0xdca, 0x200d, 0xdbb, 0xddd, 0xdbd,
0xdca, 0xdc0, 0xddc, 0xdc3, 0xdca, 0xda7, 0xddc, 0xd9a, 0xdca, 0xdbd, 0xddd, 0xdb1,
0xdca, 0xd9c, 0xdca, 0xd89, 0xdba, 0xdbb, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdd9, 0xdb1,
0xdca, 0xd92, 0xda9, 0xdca, 0xdb1, 0xdca, 0xd85, 0xdbd, 0xdca, 0xdb8, 0xda7, 0xdd2,
0xd85, 0xdb8, 0xdca, 0xdb8, 0xdcf, 0xdb1, 0xdca, 0xd87, 0xdb1, 0xdcf, 0xdaf, 0xdd2,
0xdba, 0xdca, 0xdbb, 0xdca, 0xd85, 0xd9a, 0xdca, 0xda7, 0xddd, 0xd85, 0xd9a, 0xdca,
0xda7, 0xddd, 0xdb6, 0xdca, 0xd85, 0xdc1, 0xdca, 0xd9c, 0xdb6, 0xdd0, 0xda7, 0xdca,
0xd87, 0xda7, 0xdd2, 0xdbb, 0xdc0, 0xdd4, 0xdb6, 0xdd0, 0xd9c, 0xdca, 0xda9, 0xdd1,
0xda9, 0xdca, 0xdb6, 0xdc4, 0xdbb, 0xdda, 0xdb1, 0xdca, 0xdb6, 0xdd0, 0xd82, 0xd9a,
0xddc, 0xd9a, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xdb1, 0xdde, 0xdbd, 0xdca, 0xdb6, 0xdd3,
0xdbb, 0xdd4, 0xda7, 0xdca, 0xdb6, 0xdd2, 0xdc1, 0xdca, 0xd9a, 0xdd9, 0xd9a, 0xdca,
0xdb6, 0xdf2, 0xdb1, 0xdcf, 0xdba, 0xdd2, 0xda0, 0xdd2, 0xda7, 0xdcf, 0xd9a, 0xddc,
0xdc5, 0xdb9, 0xda9, 0xdd0, 0xdb8, 0xdc3, 0xdca, 0xd9a, 0xdc3, 0xdca, 0xda9, 0xd9a,
0xdcf, 0xda9, 0xdd2, 0xdbd, 0xdd2, 0xda9, 0xdd4, 0xdb6, 0xdcf, 0xdba, 0xdd2, 0xda9,
0xdd4, 0xdc1, 0xdb1, 0xdca, 0xdb6, 0xdda, 0xdc6, 0xdd0, 0xdb8, 0xd9c, 0xdd4, 0xdc3,
0xdca, 0xda7, 0xdcf, 0xd9c, 0xdcf, 0xdc3, 0xdcf, 0xdc4, 0xdd9, 0xdb6, 0xdca, 0xdbb,
0xddd, 0xdb1, 0xdca, 0xdc4, 0xddd, 0xda0, 0xdd2, 0x20, 0xdb8, 0xdd2, 0xd82, 0x20,
0xdb1, 0xd9c, 0xdbb, 0xdba, 0xdc4, 0xddc, 0xd82, 0xd9a, 0xddc, 0xd82, 0xdc4, 0xddc,
0xdc0, 0xdca, 0xda9, 0xdca, 0xd89, 0xdbb, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca,
0xd9a, 0xdca, 0xda2, 0xd9a, 0xdbb, 0xdca, 0xdad, 0xdcf, 0xda2, 0xdba, 0xdb4, 0xdd4,
0xdbb, 0xda2, 0xdd9, 0xdbb, 0xdd4, 0xdc3, 0xdbd, 0xdb8, 0xd9a, 0xdcf, 0xdb6, 0xdd4,
0xdbd, 0xdca, 0xd9a, 0xdb8, 0xdca, 0xda0, 0xda7, 0xdca, 0xd9a, 0xdcf, 0xd9a, 0xdbb,
0xda0, 0xdca, 0xda0, 0xdd2, 0xda0, 0xdd2, 0xd9a, 0xdad, 0xdca, 0xdb8, 0xdb1, 0xdca,
0xda9, 0xdd4, 0xd9b, 0xdb1, 0xdca, 0xda9, 0xdd2, 0xd9c, 0xdcf, 0xd9a, 0xdbd, 0xdca,
0xd9a, 0xda7, 0xdcf, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdca, 0xdb1, 0xddd, 0xdba,
0xdcf, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xd9a, 0xdca, 0xdc0, 0xdcf, 0xdbd,
0xdcf, 0xdbd, 0xdb8, 0xdca, 0xdb4, 0xdd6, 0xdbb, 0xdca, 0xd9a, 0xdd4, 0xda0, 0xdd2,
0xd82, 0xd9a, 0xdd4, 0xdc0, 0xdda, 0xda7, 0xdca, 0xdb8, 0xdd0, 0xd9a, 0xdcf, 0xdc0,
0xdd4, 0xdb8, 0xd9c, 0xda9, 0xdb1, 0xdca, 0xdb8, 0xd9a, 0xdca, 0xdc3, 0xdca, 0xdc3,
0xdcf, 0xdbb, 0xdca, 0xdb8, 0xdd0, 0xdb1, 0xdd2, 0xdbd, 0xdcf, 0xdb8, 0xdc3, 0xdca,
0xd9a, 0xda7, 0xdca, 0xdb1, 0xdd2, 0xd9a, 0xddd, 0xdc3, 0xdd2, 0xdba, 0xdcf, 0xdb1,
0xddc, 0xdc0, 0xddc, 0xd9a, 0xdd4, 0xdc3, 0xdca, 0xdb1, 0xdda, 0xda7, 0xdca, 0xdc3,
0xdca, 0xd9a, 0xdca, 0xdb1, 0xddc, 0xdc0, 0xddc, 0xdc3, 0xdd2, 0xdb6, 0xdbb, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0xd94, 0xdb8, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xd94,
0xdbb, 0xdcf, 0xdbd, 0xdca, 0xdb1, 0xddc, 0xdb8, 0xdca, 0x20, 0xdb4, 0xdd9, 0xdb1,
0xdca, 0xdb4, 0xddc, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdba, 0xdb1, 0xd9a, 0xdca, 0xdb4,
0xdca, 0xdba, 0xddd, 0xdb1, 0xdca, 0xd9c, 0xdca, 0xdba, 0xdd0, 0xdb1, 0xdca, 0xd9c,
0xdca, 0xd9a, 0xda7, 0xdcf, 0xdbb, 0xdca, 0xd9a, 0xddc, 0xdc3, 0xdca, 0xdad, 0xdcf,
0xdb1, 0xdda, 0xd9a, 0xdca, 0xdba, 0xdd2, 0xdc3, 0xdd2, 0xdbd, 0xddd, 0xdbb, 0xdca,
0xda9, 0xdcf, 0xdbb, 0xdd2, 0xdba, 0xdcf, 0xdaf, 0xdca, 0xdc3, 0xd9a, 0xdca, 0xdc4,
0xdbd, 0xdd2, 0xdb1, 0xdca, 0xdc3, 0xdcf, 0xdb8, 0xdcf, 0xdbb, 0xdca, 0xd9a, 0xdb1,
0xdca, 0xda9, 0xdca, 0xdc3, 0xddd, 0xdbd, 0xdca, 0xdc2, 0xdd0, 0xdb1, 0xdca, 0xdc4,
0xdba, 0xdd2, 0xdc3, 0xdd2, 0xd82, 0xd9c, 0xdb4, 0xdca, 0xdb4, 0xdd6, 0xdbb, 0xdd4,
0xdc0, 0xdc1, 0xdca, 0x200d, 0xdbb, 0xdd9, 0xda9, 0xdca, 0xdb1, 0xdd9, 0xd9a, 0xddd,
0xdbd, 0xdd2, 0xdb8, 0xdca, 0xdc3, 0xd9a, 0xdca, 0xdad, 0xdcf, 0xdba, 0xdd2, 0xdb4,
0xdd9, 0xda7, 0xdc2, 0xdca, 0xd9a, 0xdd9, 0xdb1, 0xdca, 0xda7, 0xdca, 0xda7, 0xdca,
0xdb6, 0xdd2, 0xdbd, 0xdd2, 0xdc3, 0xdd2, 0xda7, 0xdd9, 0xdc4, 0xdd9, 0xdbb, 0xdcf,
0xdb1, 0xdca, 0xdad, 0xdd2, 0xdb8, 0xdca, 0xdb4, 0xdd4, 0xda7, 0xddd, 0xd9a, 0xdd2,
0xdba, 0xddd, 0xda7, 0xddc, 0xdb8, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xd8b, 0xdbd,
0xdcf, 0xdb1, 0xdca, 0xdb6, 0xdcf, 0xda7, 0xdbb, 0xdca, 0xd8b, 0xdbb, 0xdd4, 0xdb8,
0xdca, 0xd9a, 0xdd2, 0xd8b, 0xdc3, 0xdca, 0xda7, 0xdca, 0x2d, 0xdb1, 0xdd9, 0xdbb,
0xdcf, 0xdc0, 0xdd2, 0xdba, 0xdd0, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdba, 0xdb1, 0xdca,
0xdc0, 0xdca, 0xdbd, 0xda9, 0xdd2, 0xdc0, 0xddc, 0xdc3, 0xdca, 0xda7, 0xddc, 0xd9a,
0xdca, 0xdba, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xdbb, 0xdd0,
0xd82, 0xd9c, 0xdd4, 0xdb1, 0xdca, 0xdba, 0xdd9, 0xd9a, 0xda7, 0xdbb, 0xdd2, 0xdb1,
0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0xdba, 0xdd9, 0xdbb, 0xdd9, 0xdc0, 0xdb1,
0xdca, 0xd87, 0xdc3, 0xddc, 0xdbb, 0xdda, 0xdb6, 0xdbb, 0xdca, 0xdb8, 0xdd2, 0xdba,
0xdd4, 0xda9, 0xdcf, 0xd9a, 0xdd0, 0xdb1, 0xdbb, 0xdd2, 0xd9a, 0xdda, 0xdb4, 0xdca,
0x20, 0xdc0, 0xdbb, 0xdca, 0xda9, 0xdca, 0xdc6, 0xdcf, 0xdbb, 0xddd, 0xdb8, 0xda9,
0xdd9, 0xdba, 0xdd2, 0xdbb, 0xdcf, 0xdbb, 0xdd9, 0xdba, 0xdd2, 0xd9a, 0xdca, 0xda2,
0xdcf, 0xdc0, 0xdd2, 0xd9a, 0xdca, 0xdaf, 0xd9a, 0xdd4, 0xdab, 0xdd4, 0x20, 0xda2,
0xddd, 0xdbb, 0xdca, 0xda2, 0xdd2, 0xdba, 0xdcf, 0xdc0, 0xdc1, 0xdcf, 0x2e, 0x20,
0xdc4, 0xdd9, 0xdbd, 0xdda, 0xdb1, 0xdcf, 0xdc3, 0xdca, 0xda7, 0xdd0, 0xdb1, 0xdca,
0xdbd, 0xdd2, 0xd87, 0xda9, 0xdd2, 0xdbd, 0xdda, 0xda9, 0xdca, 0xdb6, 0xdca, 0x200d,
0xdbb, 0xdd2, 0xdc3, 0xdca, 0xdb6, 0xdda, 0xdb1, 0xdca, 0xdb6, 0xdca, 0x200d, 0xdbb,
0xddd, 0xd9a, 0xdb1, 0xdca, 0x20, 0xdc4, 0xdd2, 0xdbd, 0xdca, 0xda9, 0xdcf, 0xdc0,
0xdd2, 0xdb1, 0xdca, 0xd89, 0xdba, 0xdd4, 0xd9a, 0xdca, 0xdbd, 0xdcf, 0xdc4, 0xddd,
0xdb6, 0xdcf, 0xdbb, 0xdca, 0xda7, 0xdca, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0xda9, 0xdda,
0xdb8, 0xdb1, 0xdca, 0xdbd, 0xddd, 0xdbb, 0xdca, 0xda9, 0xdca, 0x20, 0xdc4, 0xddd,
0xdc0, 0xdca, 0xdb8, 0xdd9, 0xdbd, 0xdca, 0xdb6, 0xddd, 0xdbb, 0xdca, 0xdb1, 0xdca,
0xdb4, 0xdbb, 0xdca, 0xdad, 0xdca, 0xdc3, 0xdd2, 0xda9, 0xdca, 0xdb1, 0xdd2, 0xd87,
0xdb8, 0xdca, 0xdc3, 0xdca, 0xda7, 0xdbb, 0xdca, 0xda9, 0xdd1, 0xdb8, 0xdca, 0xd87,
0xdb1, 0xdca, 0xda9, 0xddd, 0xdbb, 0xdcf, 0xdc0, 0xd85, 0xdc3, 0xdca, 0xda7, 0xdca,
0x200d, 0xdbb, 0xd9b, 0xdcf, 0xdb1, 0xdca, 0xd87, 0xdad, 0xdd0, 0xdb1, 0xdca, 0xdc3,
0xdca, 0xdb6, 0xdd9, 0xdbd, 0xdca, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdda, 0xda9, 0xdca,
0xda9, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0xdb6, 0xdca, 0x200d,
0xdbb, 0xda7, 0xdd2, 0xdc3, 0xdca, 0xdbd, 0xdc0, 0xdcf, 0xdb6, 0xdca, 0x200d, 0xdbb,
0xdc3, 0xdbd, 0xdca, 0xdc3, 0xdca, 0xdb6, 0xdd4, 0xd9a, 0xdcf, 0xdbb, 0xdd9, 0xdc3,
0xdca, 0xda7, 0xdca, 0xdb6, 0xdd4, 0xda9, 0xdcf, 0xdb4, 0xdd9, 0xdc3, 0xdca, 0xda7,
0xdca, 0xdb6, 0xdd4, 0xdc3, 0xdd2, 0xdb1, 0xdca, 0xda2, 0xdd9, 0xdb1, 0xdca, 0xda0,
0xdd2, 0xdc3, 0xdd2, 0xdb1, 0xddd, 0xd9a, 0xddd, 0xdb4, 0xdb1, 0xdca, 0xdc4, 0xdda,
0xd9c, 0xdb1, 0xdca, 0xda9, 0xdb6, 0xdca, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0xda2, 0xdd2,
0xdb6, 0xdca, 0xdbb, 0xddd, 0xdbd, 0xdca, 0xda7, 0xdcf, 0xd9c, 0xdd4, 0xdc0, 0xdb1,
0xdca, 0xdc3, 0xdda, 0xdc4, 0xdd9, 0xdbd, 0xdca, 0xdc3, 0xdd2, 0xdb1, 0xdca, 0xd9a,
0xdd2, 0xd85, 0xdba, 0xdd2, 0xdbd, 0xdca, 0x20, 0xd94, 0xdc6, 0xdca, 0x20, 0xdb8,
0xdd1, 0xdb1, 0xdca, 0xd89, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0xdca, 0xdb6, 0xdd4,
0xdbd, 0xdca, 0xda2, 0xdbb, 0xdca, 0xdc3, 0xdd2, 0xd9a, 0xdbd, 0xdd2, 0xdb1, 0xdd2,
0xdb1, 0xdca, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd1, 0xda9, 0xdca, 0xd9a, 0xdd2, 0xdbb,
0xddc, 0xdc0, 0xdca, 0xd9a, 0xdd2, 0xdc0, 0xdca, 0xdbd, 0xdd2, 0xdc3, 0xdca, 0xdb6,
0xdb1, 0xdca, 0xdbd, 0xdca, 0xda2, 0xdd4, 0xdb6, 0xdd4, 0xdbd, 0xdca, 0xda2, 0xdb1,
0xdcf, 0xdbd, 0xdb1, 0xdca, 0xda9, 0xdb1, 0xdca, 0xdbd, 0xd9a, 0xdca, 0xdc3, 0xdd0,
0xdb8, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0xdb8, 0xdd0, 0xda9, 0xdca, 0xdbb,
0xdd2, 0xda9, 0xdca, 0xdb8, 0xddd, 0xdbd, 0xdca, 0xda7, 0xdcf, 0xdb8, 0xdbb, 0xdd2,
0xdba, 0xdd9, 0xdc4, 0xdb8, 0xdca, 0xdb1, 0xdca, 0xdb8, 0xdd2, 0xdb1, 0xdca, 0xdc3,
0xdca, 0xd9a, 0xdca, 0xdb8, 0xddc, 0xdb1, 0xdd0, 0xd9a, 0xddd, 0xdb8, 0xddc, 0xdc3,
0xdca, 0xd9a, 0xdc0, 0xdca, 0xd94, 0xdc3, 0xdca, 0xdbd, 0xddd, 0xdb4, 0xdd0, 0xdbb,
0xdd2, 0xdc3, 0xdca, 0xdb4, 0xddc, 0xda9, 0xdca, 0xd9c, 0xddc, 0xdbb, 0xdd2, 0xd9a,
0xdcf, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd9, 0xd9c, 0xdca, 0xdbb, 0xdd2, 0xd9c, 0xdcf,
0xdbb, 0xddd, 0xdb8, 0xdba, 0xdc3, 0xdb8, 0xdbb, 0xdcf, 0xdc3, 0xdd0, 0xdb1, 0xdca,
0x20, 0xdb8, 0xdd0, 0xdbb, 0xdd2, 0xdb1, 0xddd, 0xdc3, 0xdbb, 0xdcf, 0xda2, 0xdd9,
0xdc0, 0xddd, 0xdc3, 0xdbb, 0xda7, 0xddc, 0xdc0, 0xdca, 0xdc3, 0xdd2, 0xdb8, 0xdca,
0xdc6, 0xdd9, 0xdbb, 0xddd, 0xdb4, 0xddc, 0xdbd, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xddc,
0xdb4, 0xdca, 0xda2, 0xdda, 0xdc3, 0xddc, 0xdc6, 0xdd2, 0xdba, 0xdcf, 0xdc3, 0xdca,
0xda7, 0xddc, 0xd9a, 0xdca, 0xdc4, 0xddd, 0xdb8, 0xdca, 0xda7, 0xdbd, 0xdd2, 0xdb1,
0xdca, 0xda7, 0xdd2, 0xdbb, 0xdcf, 0xdb1, 0xdda, 0xd8b, 0xdbd, 0xdca, 0xdba, 0xdb1,
0xddc, 0xdc0, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xdc0, 0xda9, 0xdd4, 0xdc3, 0xdca,
0xdc0, 0xdad, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0xdc0, 0xdc0, 0xdd2, 0xdba, 0xdb1,
0xdcf, 0xdc0, 0xdd2, 0xdbd, 0xdca, 0xdb1, 0xdd2, 0xdba, 0xdd4, 0xdc3, 0xdca, 0xdc0,
0xddc, 0xdbd, 0xdca, 0xd9c, 0xddc, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd1, 0xda9, 0xdca,
0xdc0, 0xdbb, 0xdca, 0xdc3, 0xddd, 0xdc3, 0xd9c, 0xdca, 0xdbb, 0xdd9, 0xdb6, 0xdca,
0xdc3, 0xdd6, 0xdbb, 0xdd2, 0xda0, 0xdca, 0xd87, 0xdb1, 0xdca, 0xda7, 0xdb1, 0xdcf,
0xdb1, 0xdbb, 0xdd2, 0xdc0, 0xddd, 0xda0, 0xdcf, 0xd9c, 0xddc, 0xdc3, 0xdca, 0xd9a,
0xdca, 0x200d, 0xdbb, 0xdd2, 0xdc3, 0xdca, 0xdb8, 0xdc3, 0xdca, 0xd9a, 0xddc, 0xd9a,
0xddc, 0xdc3, 0xdca, 0xd9a, 0xddc, 0xdb8, 0xddc, 0xdbb, 0xddd, 0xd9a, 0xdd9, 0xdbb,
0xdca, 0xd9c, 0xdd4, 0xd91, 0xdbd, 0xdd9, 0xdb1, 0xdca, 0xdb8, 0xdcf, 0xdc4, 0xdda,
0xdb8, 0xdcf, 0xdbd, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdba, 0xdd2, 0xdb1, 0xdb8, 0xdd4,
0xdbb, 0xdd4, 0xdc3, 0xdd2, 0xdba, 0xdb8, 0xdba, 0xddd, 0xda7, 0xdd2, 0xdbb, 0xdd2,
0xdba, 0xdd4, 0xdb1, 0xdd2, 0xdba, 0xdb1, 0xdca, 0xd87, 0xdb4, 0xdd2, 0xdba, 0xdcf,
0xd95, 0xd9a, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xda9, 0xdca, 0xdb6, 0xddd, 0xd9c, 0xdb1,
0xdca, 0xdc0, 0xdd2, 0xdbd, 0xdcf, 0xda0, 0xdd0, 0xdad, 0xdb8, 0xdca, 0xda0, 0xdd6,
0xd9a, 0xdca, 0xd8a, 0xdc3, 0xdca, 0xda7, 0xdbb, 0xdca, 0xd91, 0xdc6, 0xdcf, 0xdad,
0xdca, 0xdc6, 0xdcf, 0xd9a, 0xdcf, 0xd95, 0xdc6, 0xddd, 0xdc6, 0xdd3, 0xda2, 0xdd2,
0xdc6, 0xdd4, 0xdb1, 0xdc6, 0xdd4, 0xda7, 0xdd2, 0xd9c, 0xdbd, 0xdb4, 0xd9c, 0xddd,
0xdc3, 0xdca, 0xd9c, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdd9, 0xdbb, 0xdca, 0xd9c,
0xdd4, 0xdc0, 0xdcf, 0xda9, 0xdbd, 0xdca, 0xd9a, 0xdd0, 0xdb1, 0xdbd, 0xdca, 0xd9c,
0xdd4, 0xdc0, 0xdcf, 0xdb8, 0xdca, 0xdc4, 0xddc, 0xdb1, 0xddc, 0xdbd, 0xdd4, 0xdbd,
0xdd4, 0xd91, 0xdb1, 0xdca, 0xda9, 0xdbb, 0xdca, 0xdb6, 0xdbb, 0xdd2, 0xd9a, 0xdd2,
0xdbb, 0xdd2, 0xdb8, 0xda7, 0xdd2, 0xd9a, 0xddc, 0xdc3, 0xdca, 0xdbb, 0xdda, 0xd9a,
0xdca, 0xdc0, 0xdcf, 0xda2, 0xdbd, 0xdd9, 0xdba, 0xdd2, 0xdb1, 0xdca, 0xdb8, 0xda2,
0xdd4, 0xdbb, 0xddd, 0xdb8, 0xdcf, 0xd9a, 0xdd4, 0xdc0, 0xdd9, 0xdc3, 0xdcf, 0xdc3,
0xdca, 0xdb8, 0xdd2, 0xda9, 0xdca, 0xdc0, 0xdda, 0xdb1, 0xdc0, 0xdd4, 0xdbb, 0xdd4,
0xdb1, 0xddd, 0xdc6, 0xddd, 0xd9a, 0xdca, 0xdb1, 0xddd, 0xdb8, 0xdd2, 0xdba, 0xdcf,
0xdb4, 0xd9c, 0xddd, 0x20, 0xdb4, 0xd9c, 0xddd, 0xdb4, 0xdbd, 0xdcf, 0xdc0, 0xdd4,
0xdb4, 0xdd2, 0xda7, 0xdca, 0xd9a, 0xdd9, 0xdba, 0xdcf, 0xdb1, 0xdca, 0xdb4, 0xddc,
0xdb1, 0xdca, 0xdb4, 0xdd9, 0xdba, 0xdd2, 0xdb4, 0xddc, 0xdbb, 0xdca, 0xda7, 0xdca,
0x20, 0xdb8, 0xddc, 0xdbb, 0xdd9, 0xdc3, 0xdca, 0xdb6, 0xdd2, 0xdbb, 0xdbb, 0xddd,
0xda7, 0xddd, 0xdb1, 0xdca, 0xd9c, 0xdcf, 0xdc3, 0xdba, 0xdd2, 0xdb4, 0xdcf, 0xdb1,
0xdca, 0xda7, 0xdc4, 0xdd2, 0xdad, 0xdd2, 0xda7, 0xdbb, 0xdcf, 0xdc0, 0xdcf, 0xda7,
0xddc, 0xdb1, 0xdca, 0xd9c, 0xda7, 0xdcf, 0xdb4, 0xdd4, 0xdc0, 0xdda, 0xd9a, 0xdca,
0xdc0, 0xdd0, 0xdbd, 0xdd2, 0xdc3, 0xdca, 0x45, 0x6c, 0x2d, 0x41, 0x61, 0x69,
0xfa, 0x6e, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x161, 0x6f, 0x53, 0x76,
0xe4, 0x74, 0xfd, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x161, 0x4b, 0x61, 0x6a,
0x6d, 0x61, 0x6e, 0x69, 0x65, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x79, 0x53, 0x61, 0x6c, 0x76, 0xe1, 0x64, 0x6f, 0x72, 0x42, 0x65, 0x75,
0x6c, 0x61, 0x68, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0xe1,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65,
0x72, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0xe1, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0xe1, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x76, 0xe4, 0x74, 0xfd, 0x20,
0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x6a, 0x53, 0x76,
0xe4, 0x74, 0xe1, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x76, 0x2e,
0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x161, 0x53, 0x76, 0x2e, 0x20, 0x56, 0x69,
0x6e, 0x63, 0x65, 0x6e, 0x74, 0x160, 0xf3, 0x77, 0x61, 0x41, 0x6c, 0x6d,
0x61, 0x2011, 0x41, 0x74, 0x61, 0x48, 0x6f, 0x10d, 0x69, 0x6d, 0x69, 0x6e,
0x6f, 0x76, 0x6f, 0x20, 0x4d, 0x65, 0x73, 0x74, 0x6f, 0x43, 0x68, 0x6f,
0x76, 0x64, 0x4b, 0x61, 0x72, 0xe1, 0x10d, 0x69, 0x4b, 0xe1, 0x74, 0x68,
0x6d, 0x61, 0x6e, 0x64, 0x75, 0x4e, 0x69, 0x6b, 0xf3, 0x7a, 0x69, 0x61,
0x55, 0x72, 0x61, 0x13e, 0x73, 0x6b, 0x50, 0x68, 0x6e, 0x6f, 0x6d, 0x20,
0x50, 0xe9, 0x6e, 0x68, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74, 0x61,
0x72, 0x55, 0x73, 0x165, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x4b, 0x61, 0x6e,
0xe1, 0x72, 0x73, 0x6b, 0x65, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x79, 0x4a, 0x75, 0x17e, 0x6e, 0xe1, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x53, 0x76, 0xe4, 0x74, 0xe1, 0x20, 0x48, 0x65, 0x6c, 0x65,
0x6e, 0x61, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0x68, 0xe1, 0x6e, 0x41,
0x74, 0xe9, 0x6e, 0x79, 0x42, 0x65, 0x6c, 0x65, 0x68, 0x72, 0x61, 0x64,
0x4b, 0x69, 0x161, 0x69, 0x148, 0x6f, 0x76, 0x13d, 0x75, 0x62, 0x13e, 0x61,
0x6e, 0x61, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x73, 0x6b,
0x6f, 0x50, 0x61, 0x72, 0xed, 0x17e, 0x52, 0xed, 0x6d, 0x55, 0x13e, 0x6a,
0x61, 0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x56, 0x69, 0x65, 0x64, 0x65, 0x148,
0x5a, 0xe1, 0x68, 0x72, 0x65, 0x62, 0x56, 0x69, 0x61, 0x6e, 0x6f, 0x10d,
0x6e, 0xfd, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x4b, 0x65, 0x72,
0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e, 0x79, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x79, 0x4d, 0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x75, 0x73, 0x56,
0x65, 0x13e, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xfd, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20,
0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x53, 0x65,
0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53,
0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x53, 0x76, 0x2e, 0x20, 0x4b, 0x72, 0x69, 0x161, 0x74, 0x6f, 0x66,
0x53, 0x76, 0x2e, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x17e, 0x53, 0x76, 0x2e,
0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x63, 0x69, 0x6a, 0x48, 0x6f,
0x161, 0x69, 0x6d, 0x69, 0x6e, 0x68, 0x4d, 0x75, 0x161, 0x6b, 0x61, 0x74,
0x4b, 0x69, 0x7a, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x4b, 0x61, 0x6e, 0x61,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x5a, 0x65,
0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6f, 0x74,
0x6f, 0x6b, 0x69, 0x46, 0x65, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6f, 0x74,
0x6f, 0x6b, 0x69, 0x4a, 0x75, 0x17e, 0x6e, 0x61, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x53, 0x76, 0x2e, 0x20, 0x48, 0x65, 0x6c, 0x65,
0x6e, 0x61, 0x42, 0x72, 0x75, 0x73, 0x65, 0x6c, 0x6a, 0x4b, 0xf6, 0x62,
0x65, 0x6e, 0x68, 0x61, 0x76, 0x6e, 0x44, 0x75, 0x6e, 0x61, 0x6a, 0x42,
0x6f, 0x17e, 0x69, 0x10d, 0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b,
0x69, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0x69,
0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x41, 0x62, 0x69, 0x64, 0x6a, 0x61, 0x61,
0x6e, 0x41, 0x6c, 0x6a, 0x65, 0x65, 0x72, 0x69, 0x73, 0x42, 0x61, 0x6d,
0x61, 0x61, 0x6b, 0x6f, 0x42, 0x61, 0x61, 0x67, 0x69, 0x42, 0x69, 0x73,
0x61, 0x61, 0x77, 0x42, 0x61, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x72, 0x65,
0x42, 0x61, 0x72, 0x61, 0x73, 0x61, 0x66, 0x69, 0x6c, 0x51, 0x61, 0x61,
0x68, 0x69, 0x72, 0x61, 0x4b, 0x61, 0x73, 0x61, 0x62, 0x61, 0x6c, 0x61,
0x61, 0x6e, 0x6b, 0x61, 0x43, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x69, 0x44,
0x61, 0x72, 0x65, 0x73, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x4a, 0x69, 0x62,
0x75, 0x75, 0x74, 0x69, 0x44, 0x6f, 0x75, 0x61, 0x61, 0x6c, 0x61, 0x45,
0x6c, 0x20, 0x43, 0x65, 0x79, 0x75, 0x75, 0x6e, 0x46, 0x69, 0x72, 0x69,
0x69, 0x74, 0x61, 0x77, 0x6e, 0x47, 0x61, 0x62, 0x72, 0x6f, 0x6f, 0x6e,
0x48, 0x61, 0x72, 0x61, 0x61, 0x72, 0x65, 0x4a, 0x6f, 0x68, 0x61, 0x6e,
0x73, 0x62, 0x61, 0x61, 0x67, 0x4b, 0x61, 0x6d, 0x62, 0x61, 0x61, 0x6c,
0x61, 0x4b, 0x68, 0x61, 0x72, 0x74, 0x75, 0x75, 0x6d, 0x4c, 0x61, 0x61,
0x67, 0x6f, 0x6f, 0x73, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x66, 0x69, 0x6c,
0x4c, 0x6f, 0x6f, 0x6d, 0x4c, 0x75, 0x77, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62, 0x61, 0x61, 0x73, 0x68, 0x69, 0x4c,
0x75, 0x73, 0x61, 0x61, 0x6b, 0x61, 0x4d, 0x61, 0x62, 0x75, 0x75, 0x74,
0x6f, 0x4d, 0x61, 0x73, 0x65, 0x65, 0x72, 0x6f, 0x4d, 0x61, 0x62, 0x61,
0x62, 0x61, 0x61, 0x6e, 0x65, 0x4d, 0x6f, 0x6e, 0x72, 0x6f, 0x66, 0x69,
0x79, 0x61, 0x4e, 0x61, 0x79, 0x72, 0x6f, 0x6f, 0x62, 0x69, 0x49, 0x6e,
0x6a, 0x61, 0x6d, 0x69, 0x6e, 0x61, 0x4e, 0x6f, 0x6f, 0x6b, 0x6a, 0x6f,
0x74, 0x42, 0x6f, 0x6f, 0x72, 0x74, 0x6f, 0x2d, 0x4e, 0x6f, 0x6f, 0x66,
0x6f, 0x53, 0x61, 0x77, 0x20, 0x54, 0x6f, 0x6f, 0x6d, 0x54, 0x69, 0x72,
0x69, 0x62, 0x6f, 0x6f, 0x6c, 0x69, 0x54, 0x75, 0x75, 0x6e, 0x69, 0x73,
0x57, 0x69, 0x6e, 0x64, 0x68, 0x6f, 0x6f, 0x6b, 0x41, 0x6e, 0x6a, 0x6f,
0x72, 0x61, 0x67, 0x65, 0x41, 0x6e, 0x67, 0x75, 0x77, 0x69, 0x6c, 0x61,
0x41, 0x6e, 0x74, 0x69, 0x67, 0x75, 0x77, 0x61, 0x41, 0x72, 0x61, 0x67,
0x75, 0x77, 0x61, 0x79, 0x6e, 0x61, 0x42, 0x75, 0x77, 0x65, 0x6e, 0x6f,
0x73, 0x20, 0x41, 0x79, 0x72, 0x69, 0x73, 0x4b, 0x61, 0x74, 0x61, 0x6d,
0x61, 0x61, 0x72, 0x6b, 0x61, 0x4b, 0x6f, 0x72, 0x64, 0x6f, 0x6f, 0x62,
0x61, 0x4c, 0x61, 0x20, 0x52, 0x69, 0x79, 0x6f, 0x6a, 0x61, 0x4d, 0x65,
0x65, 0x6e, 0x64, 0x6f, 0x6f, 0x73, 0x61, 0x52, 0x69, 0x79, 0x6f, 0x20,
0x4a, 0x61, 0x6c, 0x65, 0x6a, 0x6f, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x4a,
0x75, 0x77, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x6e, 0x20, 0x4c, 0x75, 0x77,
0x69, 0x73, 0x54, 0x75, 0x6b, 0x75, 0x75, 0x6d, 0x61, 0x61, 0x6e, 0x55,
0x73, 0x68, 0x75, 0x61, 0x61, 0x79, 0x41, 0x73, 0x75, 0x6e, 0x6b, 0x69,
0x79, 0x6f, 0x6e, 0x41, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x61, 0x6e,
0x42, 0x61, 0x61, 0x68, 0x69, 0x79, 0x61, 0x42, 0x61, 0x68, 0x69, 0x79,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x42, 0x65,
0x6c, 0x69, 0x69, 0x73, 0x65, 0x42, 0x61, 0x6c, 0x61, 0x6e, 0x6b, 0x2d,
0x53, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x42, 0x6f, 0x77, 0x20, 0x46, 0x69,
0x73, 0x74, 0x61, 0x4b, 0x61, 0x6d, 0x62, 0x69, 0x72, 0x69, 0x69, 0x6a,
0x20, 0x42, 0x61, 0x61, 0x79, 0x4b, 0x61, 0x61, 0x6d, 0x62, 0x6f, 0x20,
0x47, 0x61, 0x72, 0x61, 0x6e, 0x64, 0x69, 0x4b, 0x61, 0x61, 0x6e, 0x6b,
0x75, 0x75, 0x6e, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x61, 0x73, 0x4b,
0x61, 0x79, 0x65, 0x65, 0x6e, 0x4b, 0x65, 0x79, 0x6d, 0x61, 0x61, 0x6e,
0x4a, 0x69, 0x6b, 0x61, 0x61, 0x67, 0x6f, 0x4a, 0x69, 0x77, 0x61, 0x61,
0x68, 0x75, 0x75, 0x4d, 0x61, 0x67, 0x61, 0x61, 0x6c, 0x61, 0x64, 0x61,
0x20, 0x4a, 0x75, 0x61, 0x72, 0x65, 0x7a, 0x4b, 0x6f, 0x73, 0x74, 0x61,
0x20, 0x52, 0x69, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x65, 0x73, 0x74,
0x6f, 0x6f, 0x6e, 0x4b, 0x75, 0x79, 0x61, 0x61, 0x62, 0x61, 0x4b, 0x75,
0x72, 0x61, 0x6b, 0x6f, 0x6f, 0x77, 0x44, 0x61, 0x61, 0x6e, 0x6d, 0x61,
0x61, 0x6b, 0x73, 0x68, 0x61, 0x61, 0x6e, 0x44, 0x6f, 0x6f, 0x73, 0x61,
0x6e, 0x44, 0x6f, 0x6f, 0x73, 0x61, 0x6e, 0x20, 0x4b, 0x69, 0x72, 0x65,
0x65, 0x6b, 0x44, 0x65, 0x6e, 0x66, 0x61, 0x72, 0x44, 0x65, 0x74, 0x6f,
0x72, 0x6f, 0x79, 0x74, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x69, 0x6b,
0x61, 0x49, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x65, 0x62, 0x45, 0x6c, 0x20,
0x53, 0x61, 0x6c, 0x66, 0x61, 0x64, 0x6f, 0x6f, 0x72, 0x46, 0x6f, 0x6f,
0x74, 0x20, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x46, 0x6f, 0x6f, 0x74,
0x61, 0x6c, 0x65, 0x73, 0x61, 0x47, 0x61, 0x6c, 0x65, 0x79, 0x73, 0x20,
0x42, 0x61, 0x61, 0x79, 0x47, 0x75, 0x75, 0x73, 0x20, 0x42, 0x61, 0x61,
0x79, 0x47, 0x61, 0x72, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x47, 0x61, 0x72, 0x65, 0x6e, 0x61, 0x61, 0x64, 0x61, 0x47, 0x75, 0x77,
0x61, 0x64, 0x65, 0x6c, 0x75, 0x75, 0x62, 0x47, 0x75, 0x77, 0x61, 0x74,
0x65, 0x6d, 0x61, 0x61, 0x6c, 0x61, 0x47, 0x75, 0x77, 0x61, 0x79, 0x61,
0x71, 0x75, 0x77, 0x69, 0x6c, 0x47, 0x75, 0x79, 0x61, 0x61, 0x6e, 0x61,
0x48, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x48, 0x61, 0x66,
0x61, 0x61, 0x6e, 0x61, 0x48, 0x61, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c,
0x6f, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x62, 0x6f,
0x6c, 0x69, 0x73, 0x4e, 0x6f, 0x6b, 0x69, 0x73, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x65,
0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61,
0x6e, 0x61, 0x42, 0x65, 0x74, 0x65, 0x73, 0x62, 0x61, 0x61, 0x67, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x54, 0x65,
0x6c, 0x20, 0x53, 0x69, 0x74, 0x69, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x79, 0x61, 0x61, 0x6e, 0x61, 0x46, 0x65, 0x65, 0x66, 0x61, 0x61, 0x79,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x46,
0x69, 0x6e, 0x73, 0x65, 0x65, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0x61, 0x61,
0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61,
0x6e, 0x61, 0x49, 0x6e, 0x75, 0x75, 0x66, 0x69, 0x6b, 0x49, 0x71, 0x61,
0x61, 0x6c, 0x75, 0x75, 0x74, 0x4a, 0x75, 0x6e, 0x69, 0x79, 0x75, 0x75,
0x4c, 0x75, 0x75, 0x73, 0x66, 0x69, 0x6c, 0x65, 0x4d, 0x6f, 0x6e, 0x74,
0x69, 0x73, 0x65, 0x65, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74,
0x61, 0x6b, 0x69, 0x4b, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x65, 0x6e, 0x64,
0x69, 0x6b, 0x4c, 0x61, 0x61, 0x20, 0x42, 0x61, 0x61, 0x73, 0x4c, 0x69,
0x69, 0x6d, 0x61, 0x4c, 0x6f, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x6a, 0x61,
0x6c, 0x69, 0x73, 0x4c, 0x6f, 0x6f, 0x77, 0x61, 0x20, 0x42, 0x69, 0x72,
0x69, 0x6e, 0x73, 0x65, 0x73, 0x20, 0x4b, 0x75, 0x77, 0x61, 0x61, 0x74,
0x61, 0x4d, 0x61, 0x61, 0x73, 0x69, 0x69, 0x79, 0x6f, 0x4d, 0x61, 0x6e,
0x61, 0x61, 0x67, 0x75, 0x77, 0x61, 0x4d, 0x61, 0x6e, 0x61, 0x61, 0x75,
0x75, 0x73, 0x4d, 0x61, 0x61, 0x72, 0x69, 0x67, 0x6f, 0x74, 0x4d, 0x61,
0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x69, 0x6b, 0x75, 0x79, 0x75, 0x75,
0x4d, 0x61, 0x7a, 0x61, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x4d, 0x65, 0x6e,
0x6f, 0x6f, 0x6d, 0x69, 0x6e, 0x65, 0x65, 0x4d, 0x65, 0x72, 0x69, 0x69,
0x64, 0x61, 0x4d, 0x65, 0x74, 0x6c, 0x61, 0x61, 0x6b, 0x61, 0x74, 0x6c,
0x61, 0x4d, 0x69, 0x69, 0x71, 0x75, 0x75, 0x6c, 0x6f, 0x6e, 0x4d, 0x6f,
0x6f, 0x6e, 0x6b, 0x74, 0x6f, 0x6f, 0x6e, 0x4d, 0x6f, 0x6f, 0x6e, 0x74,
0x65, 0x65, 0x72, 0x65, 0x79, 0x4d, 0x6f, 0x6f, 0x6e, 0x74, 0x61, 0x66,
0x69, 0x69, 0x64, 0x69, 0x79, 0x6f, 0x4d, 0x6f, 0x6f, 0x6e, 0x74, 0x73,
0x65, 0x72, 0x61, 0x61, 0x74, 0x4e, 0x61, 0x73, 0x61, 0x61, 0x77, 0x4e,
0x69, 0x79, 0x75, 0x75, 0x20, 0x59, 0x6f, 0x6f, 0x6b, 0x42, 0x69, 0x79,
0x75, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f,
0x79, 0x69, 0x67, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x6f, 0x74, 0x61,
0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x2c, 0x20, 0x57,
0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x6f, 0x74, 0x61, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x53, 0x61,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79,
0x69, 0x67, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x6f, 0x74, 0x61, 0x4f,
0x6a, 0x69, 0x6e, 0x61, 0x61, 0x67, 0x61, 0x42, 0x61, 0x6e, 0x61, 0x61,
0x6d, 0x61, 0x46, 0x6f, 0x6f, 0x6e, 0x69, 0x6b, 0x69, 0x73, 0x42, 0x6f,
0x6f, 0x72, 0x74, 0x2d, 0x61, 0x77, 0x2d, 0x42, 0x69, 0x72, 0x69, 0x69,
0x6e, 0x73, 0x42, 0x6f, 0x6f, 0x72, 0x74, 0x20, 0x6f, 0x66, 0x20, 0x49,
0x73, 0x62, 0x61, 0x79, 0x6e, 0x42, 0x6f, 0x6f, 0x72, 0x74, 0x61, 0x20,
0x52, 0x69, 0x69, 0x6b, 0x6f, 0x42, 0x75, 0x6e, 0x74, 0x61, 0x20, 0x41,
0x72, 0x69, 0x6e, 0x61, 0x61, 0x73, 0x52, 0x61, 0x61, 0x6e, 0x6b, 0x69,
0x6e, 0x20, 0x49, 0x6e, 0x6c, 0x65, 0x65, 0x74, 0x52, 0x65, 0x63, 0x65,
0x79, 0x66, 0x52, 0x65, 0x6a, 0x69, 0x69, 0x6e, 0x61, 0x52, 0x65, 0x73,
0x6f, 0x6c, 0x75, 0x75, 0x74, 0x52, 0x69, 0x79, 0x6f, 0x20, 0x42, 0x61,
0x72, 0x61, 0x61, 0x6e, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72,
0x65, 0x65, 0x6d, 0x53, 0x61, 0x6e, 0x74, 0x69, 0x79, 0x61, 0x61, 0x67,
0x6f, 0x53, 0x61, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44, 0x6f, 0x6d, 0x69,
0x6e, 0x67, 0x6f, 0x53, 0x61, 0x61, 0x77, 0x20, 0x42, 0x6f, 0x6f, 0x6c,
0x6f, 0x49, 0x74, 0x6f, 0x71, 0x6f, 0x72, 0x74, 0x6f, 0x6f, 0x6d, 0x69,
0x69, 0x74, 0x53, 0x69, 0x69, 0x74, 0x6b, 0x61, 0x53, 0x74, 0x2e, 0x20,
0x42, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6c, 0x65, 0x6d, 0x69, 0x53, 0x74,
0x2e, 0x20, 0x4a, 0x6f, 0x6f, 0x6e, 0x53, 0x74, 0x2e, 0x20, 0x4b, 0x69,
0x74, 0x69, 0x73, 0x53, 0x74, 0x2e, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x79,
0x61, 0x53, 0x74, 0x2e, 0x20, 0x54, 0x6f, 0x6f, 0x6d, 0x61, 0x73, 0x53,
0x74, 0x2e, 0x20, 0x46, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x49, 0x73,
0x77, 0x69, 0x66, 0x74, 0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e, 0x74, 0x54,
0x65, 0x67, 0x75, 0x73, 0x69, 0x67, 0x61, 0x6c, 0x62, 0x61, 0x54, 0x75,
0x75, 0x6c, 0x54, 0x69, 0x6a, 0x75, 0x77, 0x61, 0x61, 0x6e, 0x61, 0x54,
0x6f, 0x72, 0x74, 0x6f, 0x6f, 0x6c, 0x61, 0x46, 0x61, 0x6e, 0x6b, 0x75,
0x75, 0x66, 0x61, 0x72, 0x57, 0x61, 0x79, 0x74, 0x68, 0x6f, 0x72, 0x61,
0x73, 0x57, 0x69, 0x6e, 0x69, 0x62, 0x65, 0x67, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x61, 0x61, 0x74, 0x4b, 0x61, 0x79, 0x73, 0x65, 0x65, 0x44, 0x61,
0x66, 0x69, 0x73, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019,
0x75, 0x72, 0x66, 0x69, 0x6c, 0x65, 0x4d, 0x61, 0x6b, 0x71, 0x75, 0x77,
0x61, 0x72, 0x69, 0x79, 0x42, 0x61, 0x61, 0x6d, 0x61, 0x72, 0x53, 0x69,
0x79, 0x6f, 0x77, 0x61, 0x54, 0x6f, 0x72, 0x6f, 0x6f, 0x6c, 0x46, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x4c, 0x6f, 0x6e, 0x6a, 0x69, 0x72, 0x62, 0x79,
0x65, 0x65, 0x6e, 0x43, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x6d, 0x61,
0x61, 0x6e, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x79, 0x72, 0x41, 0x6b, 0x74,
0x61, 0x77, 0x41, 0x74, 0x69, 0x79, 0x72, 0x61, 0x77, 0x42, 0x61, 0x71,
0x64, 0x61, 0x61, 0x64, 0x42, 0x61, 0x78, 0x72, 0x65, 0x79, 0x6e, 0x42,
0x61, 0x6e, 0x67, 0x6b, 0x6f, 0x6f, 0x6b, 0x42, 0x61, 0x72, 0x6e, 0x61,
0x61, 0x75, 0x75, 0x6c, 0x42, 0x65, 0x79, 0x72, 0x75, 0x75, 0x64, 0x42,
0x75, 0x72, 0x75, 0x6e, 0x65, 0x79, 0x4a, 0x69, 0x69, 0x74, 0x61, 0x44,
0x69, 0x6d, 0x69, 0x73, 0x68, 0x69, 0x71, 0x44, 0x68, 0x61, 0x61, 0x6b,
0x61, 0x51, 0x61, 0x73, 0x61, 0x48, 0x6f, 0x6f, 0x20, 0x4a, 0x69, 0x20,
0x4d, 0x69, 0x68, 0x20, 0x53, 0x69, 0x74, 0x69, 0x48, 0x6f, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x66, 0x75, 0x64,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x69, 0x6b, 0x4a, 0x61, 0x6b, 0x61,
0x61, 0x72, 0x74, 0x61, 0x4a, 0x61, 0x79, 0x61, 0x62, 0x75, 0x72, 0x61,
0x4a, 0x65, 0x65, 0x72, 0x75, 0x73, 0x61, 0x61, 0x6c, 0x65, 0x6d, 0x4b,
0x61, 0x61, 0x62, 0x75, 0x75, 0x6c, 0x4b, 0x61, 0x6d, 0x6b, 0x61, 0x74,
0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x61, 0x6a, 0x69, 0x4b, 0x68, 0x61,
0x6e, 0x64, 0x69, 0x79, 0x67, 0x61, 0x4b, 0x6f, 0x6c, 0x6b, 0x61, 0x61,
0x74, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x61, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x20, 0x4c, 0x61, 0x6d,
0x62, 0x75, 0x75, 0x72, 0x4b, 0x75, 0x6a, 0x69, 0x6e, 0x67, 0x4d, 0x61,
0x6b, 0x6f, 0x77, 0x4d, 0x61, 0x67, 0x65, 0x64, 0x61, 0x6e, 0x4d, 0x61,
0x6e, 0x69, 0x69, 0x6c, 0x61, 0x4e, 0x6f, 0x66, 0x6f, 0x6b, 0x75, 0x73,
0x6e, 0x65, 0x74, 0x73, 0x69, 0x6b, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x42, 0x65, 0x6e, 0x6f, 0x6d, 0x20,
0x42, 0x65, 0x6e, 0x42, 0x6f, 0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61,
0x6b, 0x42, 0x6f, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x51,
0x61, 0x64, 0x64, 0x61, 0x72, 0x51, 0x69, 0x79, 0x73, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x52, 0x69, 0x79, 0x61, 0x61, 0x64, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x6b, 0x61, 0x61, 0x6e, 0x53, 0x68, 0x61, 0x6e, 0x67, 0x68, 0x61,
0x61, 0x79, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x62, 0x75, 0x75, 0x72, 0x53,
0x61, 0x72, 0x65, 0x64, 0x6e, 0x6f, 0x6b, 0x6f, 0x6c, 0x65, 0x79, 0x6d,
0x69, 0x73, 0x6b, 0x54, 0x65, 0x79, 0x62, 0x65, 0x79, 0x54, 0x6f, 0x73,
0x68, 0x6b, 0x65, 0x65, 0x6e, 0x74, 0x54, 0x69, 0x62, 0x69, 0x6c, 0x69,
0x73, 0x69, 0x54, 0x65, 0x68, 0x72, 0x61, 0x61, 0x6e, 0x46, 0x69, 0x79,
0x61, 0x61, 0x6e, 0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x46, 0x61, 0x6c,
0x61, 0x64, 0x69, 0x66, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x61, 0x67, 0x59, 0x65,
0x72, 0x65, 0x66, 0x61, 0x6e, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x73, 0x42,
0x61, 0x72, 0x6d, 0x75, 0x75, 0x64, 0x61, 0x4b, 0x65, 0x79, 0x62, 0x20,
0x46, 0x61, 0x61, 0x72, 0x64, 0x65, 0x46, 0x61, 0x72, 0x6f, 0x77, 0x52,
0x65, 0x79, 0x6b, 0x6a, 0x61, 0x66, 0x69, 0x6b, 0x53, 0x6f, 0x77, 0x74,
0x20, 0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x6c, 0x65, 0x79, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x79,
0x64, 0x65, 0x42, 0x69, 0x72, 0x69, 0x73, 0x62, 0x61, 0x6e, 0x42, 0x6f,
0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x48, 0x75, 0x62,
0x61, 0x61, 0x72, 0x74, 0x4c, 0x6f, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x4d,
0x65, 0x6c, 0x62, 0x6f, 0x6f, 0x6e, 0x42, 0x65, 0x72, 0x74, 0x41, 0x6d,
0x73, 0x74, 0x65, 0x72, 0x64, 0x61, 0x61, 0x6d, 0x41, 0x6e, 0x64, 0x6f,
0x6f, 0x72, 0x61, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x6b, 0x68, 0x61,
0x61, 0x6e, 0x41, 0x74, 0x65, 0x6e, 0x73, 0x42, 0x65, 0x6c, 0x67, 0x61,
0x72, 0x61, 0x79, 0x64, 0x68, 0x42, 0x61, 0x72, 0x6c, 0x69, 0x69, 0x6e,
0x42, 0x61, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61, 0x66, 0x61, 0x42,
0x61, 0x72, 0x61, 0x73, 0x61, 0x6c, 0x69, 0x73, 0x42, 0x75, 0x6a, 0x61,
0x72, 0x65, 0x73, 0x74, 0x42, 0x75, 0x64, 0x61, 0x62, 0x65, 0x73, 0x74,
0x42, 0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x65, 0x6e, 0x4a, 0x69, 0x73,
0x69, 0x6e, 0x61, 0x61, 0x77, 0x4b, 0x6f, 0x62, 0x65, 0x6e, 0x68, 0x61,
0x61, 0x67, 0x61, 0x6e, 0x44, 0x68, 0x61, 0x61, 0x62, 0x6c, 0x69, 0x6e,
0x47, 0x65, 0x65, 0x72, 0x6e, 0x69, 0x73, 0x69, 0x48, 0x65, 0x6c, 0x65,
0x6e, 0x69, 0x73, 0x6b, 0x69, 0x41, 0x79, 0x6c, 0x65, 0x20, 0x6f, 0x66,
0x20, 0x4d, 0x61, 0x61, 0x6e, 0x49, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x75,
0x75, 0x6c, 0x4a, 0x61, 0x61, 0x72, 0x73, 0x65, 0x79, 0x4b, 0x69, 0x72,
0x6f, 0x6f, 0x66, 0x4b, 0x69, 0x79, 0x65, 0x65, 0x66, 0x4c, 0x75, 0x62,
0x61, 0x6c, 0x61, 0x61, 0x6e, 0x61, 0x4c, 0x61, 0x6e, 0x64, 0x61, 0x6e,
0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x65, 0x72, 0x67, 0x4d, 0x61,
0x64, 0x72, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x61, 0x72, 0x69, 0x68, 0x61,
0x61, 0x6d, 0x4d, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x4d, 0x6f, 0x73, 0x6b,
0x6f, 0x77, 0x42, 0x61, 0x61, 0x72, 0x69, 0x69, 0x73, 0x42, 0x6f, 0x64,
0x67, 0x6f, 0x72, 0x69, 0x6b, 0x61, 0x42, 0x61, 0x72, 0x61, 0x61, 0x67,
0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x69, 0x6e, 0x6f, 0x53,
0x61, 0x72, 0x61, 0x79, 0x65, 0x65, 0x66, 0x6f, 0x53, 0x61, 0x72, 0x61,
0x74, 0x6f, 0x6f, 0x66, 0x53, 0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x62,
0x6f, 0x6c, 0x49, 0x73, 0x6b, 0x6f, 0x6f, 0x62, 0x6a, 0x65, 0x49, 0x73,
0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6f, 0x6d, 0x54, 0x61, 0x61, 0x6c, 0x69,
0x6e, 0x54, 0x69, 0x72, 0x61, 0x61, 0x6e, 0x65, 0x55, 0x6c, 0x79, 0x61,
0x6e, 0x6f, 0x66, 0x69, 0x73, 0x6b, 0x46, 0x61, 0x64, 0x75, 0x75, 0x73,
0x46, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x46, 0x69, 0x79, 0x65,
0x65, 0x6e, 0x61, 0x46, 0x69, 0x6e, 0x69, 0x79, 0x75, 0x75, 0x73, 0x46,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x61, 0x71, 0x72,
0x69, 0x62, 0x53, 0x75, 0x75, 0x72, 0x69, 0x6b, 0x68, 0x41, 0x6e, 0x74,
0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x66, 0x6f, 0x4a, 0x61, 0x67,
0x6f, 0x73, 0x4b, 0x69, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x4b,
0x65, 0x72, 0x67, 0x61, 0x6c, 0x65, 0x6e, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x66, 0x69, 0x73, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x68, 0x69, 0x79, 0x61,
0x61, 0x73, 0x4d, 0x61, 0x79, 0x6f, 0x6f, 0x74, 0x65, 0x52, 0x69, 0x79,
0x75, 0x75, 0x6e, 0x69, 0x79, 0x6f, 0x6e, 0x4f, 0x77, 0x6b, 0x6c, 0x61,
0x61, 0x6e, 0x42, 0x6f, 0x6f, 0x67, 0x61, 0x79, 0x6e, 0x66, 0x69, 0x6c,
0x4a, 0x61, 0x74, 0x61, 0x6d, 0x4a, 0x75, 0x75, 0x6b, 0x49, 0x69, 0x73,
0x74, 0x61, 0x72, 0x66, 0x61, 0x6b, 0x6f, 0x66, 0x6f, 0x47, 0x61, 0x6c,
0x61, 0x62, 0x61, 0x67, 0x6f, 0x73, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x79,
0x72, 0x43, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c,
0x47, 0x75, 0x77, 0x61, 0x6d, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d,
0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x73, 0x72, 0x69, 0x69, 0x4b, 0x75,
0x77, 0x61, 0x6a, 0x61, 0x6c, 0x65, 0x79, 0x6e, 0x4d, 0x61, 0x6a, 0x72,
0x6f, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x77, 0x65, 0x73, 0x61, 0x73, 0x4e,
0x61, 0x77, 0x72, 0x6f, 0x6f, 0x4e, 0x6f, 0x6f, 0x72, 0x66, 0x65, 0x6b,
0x4e, 0x6f, 0x6f, 0x6d, 0x69, 0x79, 0x61, 0x42, 0x61, 0x67, 0x6f, 0x20,
0x42, 0x61, 0x67, 0x6f, 0x42, 0x61, 0x6c, 0x61, 0x77, 0x42, 0x69, 0x74,
0x6b, 0x61, 0x79, 0x72, 0x6e, 0x42, 0x6f, 0x6e, 0x62, 0x65, 0x79, 0x42,
0x6f, 0x6f, 0x72, 0x74, 0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x69,
0x53, 0x65, 0x79, 0x62, 0x61, 0x6e, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74,
0x61, 0x62, 0x75, 0x57, 0x61, 0x6c, 0x69, 0x73, 0x42, 0x69, 0x73, 0xe1,
0x75, 0x44, 0x61, 0x72, 0x20, 0x65, 0x73, 0x2d, 0x53, 0x61, 0x6c, 0x61,
0x6d, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x4c,
0x6f, 0x73, 0x20, 0xc1, 0x6e, 0x67, 0x65, 0x6c, 0x65, 0x73, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0xf3, 0x6e, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x53, 0x61, 0x6e,
0x20, 0x43, 0x72, 0x69, 0x73, 0x74, 0xf3, 0x62, 0x61, 0x6c, 0x44, 0x75,
0x62, 0xe1, 0x69, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x48, 0x6f,
0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x4a, 0x65, 0x72,
0x75, 0x73, 0x61, 0x6c, 0xe9, 0x6e, 0x53, 0x68, 0x61, 0x6e, 0x67, 0x68,
0xe1, 0x69, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x55, 0x6c, 0xe1, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x59, 0x61, 0x6e, 0x67, 0xf3, 0x6e,
0x20, 0x28, 0x52, 0x61, 0x6e, 0x67, 0xfa, 0x6e, 0x29, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x49, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6f, 0x65, 0x4e, 0x61, 0x76,
0x69, 0x64, 0x61, 0x64, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x53, 0x61, 0x69, 0x70, 0xe1, 0x6e,
0x43, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x79, 0x46, 0x75, 0x65, 0x72, 0x74,
0x65, 0x20, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x4e, 0x61, 0x73, 0xe1,
0x75, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x73,
0x4b, 0x68, 0x61, 0x6e, 0x64, 0x79, 0x67, 0x61, 0x49, 0x73, 0x6c, 0x61,
0x73, 0x20, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x61, 0x73, 0x47, 0x75,
0x65, 0x72, 0x6e, 0x73, 0x65, 0x79, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x57,
0x61, 0x6b, 0x65, 0x42, 0x75, 0x79, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61,
0x52, 0x69, 0x6f, 0x20, 0x42, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0xfd, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0xe9, 0x59, 0x69,
0x62, 0x75, 0x74, 0xed, 0x4e, 0x61, 0x73, 0x73, 0x61, 0x75, 0x41, 0x73,
0x74, 0x72, 0x61, 0x6b, 0x68, 0x61, 0x6e, 0x55, 0x6c, 0x79, 0x61, 0x6e,
0x6f, 0x76, 0x73, 0x6b, 0x4a, 0x69, 0x6a, 0x69, 0x20, 0x6c, 0x61, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x20, 0x4b, 0x75, 0x73, 0x69, 0x6e, 0x69, 0x42, 0x61, 0x62, 0x61,
0x64, 0x6f, 0x73, 0x6b, 0x75, 0x72, 0x61, 0x6b, 0x61, 0x6f, 0x45, 0x6c,
0x73, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x61,
0x73, 0x6b, 0x61, 0x7a, 0x69, 0x6e, 0x69, 0x43, 0x65, 0x6e, 0x74, 0x65,
0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x61,
0x73, 0x6b, 0x61, 0x7a, 0x69, 0x6e, 0x69, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x4b, 0x61, 0x73, 0x6b, 0x61, 0x7a, 0x69, 0x6e, 0x69, 0x42, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x69, 0x20, 0x79, 0x61, 0x20, 0x55, 0x68, 0x69,
0x73, 0x70, 0x61, 0x6e, 0x69, 0x61, 0x50, 0x77, 0x65, 0x74, 0x6f, 0x72,
0x69, 0x6b, 0x6f, 0x4a, 0x69, 0x6a, 0x69, 0x20, 0x6c, 0x61, 0x20, 0x48,
0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x6f, 0x4b, 0x72, 0x69, 0x73, 0x6d, 0x61,
0x73, 0x69, 0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0xf6, 0x61, 0x72, 0x6e,
0x61, 0x54, 0x6a, 0x69, 0x74, 0x61, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69,
0x20, 0x4d, 0x69, 0x6e, 0x68, 0x2d, 0x73, 0x74, 0x61, 0x64, 0x65, 0x6e,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x72, 0x6e, 0x61, 0x4b, 0x61, 0x6e, 0x61,
0x72, 0x69, 0x65, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x54, 0x6f, 0x72, 0x73,
0x68, 0x61, 0x6d, 0x6e, 0x53, 0x79, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x65, 0x6e, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x20, 0x48, 0x65, 0x6c,
0x65, 0x6e, 0x61, 0x4b, 0xf6, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x6d, 0x6e,
0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x6e, 0x4a, 0x75, 0x6c,
0xf6, 0x6e, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0xf6, 0x61, 0x72, 0x6e, 0x61,
0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x65, 0x72, 0x6e, 0x61, 0x4b, 0x65, 0x72,
0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x50, 0xe5, 0x73,
0x6b, 0xf6, 0x6e, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0xf6, 0x61,
0x72, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0xf6, 0x61, 0x72, 0x6e, 0x61, 0x4d, 0x69, 0x64, 0x77, 0x61, 0x79, 0xf6,
0x61, 0x72, 0x6e, 0x61, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0xf6, 0x61, 0x72, 0x6e, 0x61, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0xf6,
0x6e, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6c, 0x65,
0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61,
0x64, 0x74, 0x57, 0x6f, 0x73, 0x63, 0x68, 0x74, 0x6f, 0x6b, 0x4d, 0x75,
0x73, 0x63, 0x68, 0x6b, 0x61, 0x74, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42,
0x61, 0x61, 0x74, 0x6f, 0x72, 0x53, 0xfc, 0xfc, 0x64, 0x2d, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x65, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x73,
0x63, 0x68, 0x74, 0x4b, 0x6f, 0x70, 0x65, 0x68, 0x61, 0x67, 0x65, 0x57,
0x69, 0x65, 0x6e, 0x61, 0x63, 0x68, 0x74, 0x73, 0x2d, 0x49, 0x6e, 0x73,
0x6c, 0x65, 0x4f, 0x73, 0x63, 0x68, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73,
0x6c, 0x65, 0x710, 0x712, 0x715, 0x713, 0x722, 0x710, 0x71f, 0x72a, 0x710, 0x710,
0x715, 0x71d, 0x723, 0x20, 0x710, 0x712, 0x712, 0x710, 0x713, 0x719, 0x710, 0x710,
0x72a, 0x710, 0x723, 0x721, 0x710, 0x72a, 0x710, 0x712, 0x710, 0x721, 0x710, 0x71f,
0x718, 0x712, 0x710, 0x722, 0x713, 0x710, 0x718, 0x71d, 0x712, 0x710, 0x722, 0x713,
0x718, 0x720, 0x712, 0x71d, 0x723, 0x710, 0x718, 0x712, 0x720, 0x722, 0x72c, 0x71d,
0x72a, 0x712, 0x72a, 0x710, 0x719, 0x710, 0x712, 0x71d, 0x720, 0x712, 0x718, 0x713,
0x718, 0x721, 0x712, 0x718, 0x72a, 0x710, 0x729, 0x710, 0x717, 0x716, 0x308, 0x717,
0x71f, 0x710, 0x723, 0x710, 0x712, 0x720, 0x722, 0x71f, 0x710, 0x723, 0x712, 0x72c,
0x710, 0x71f, 0x718, 0x722, 0x710, 0x71f, 0x72a, 0x71d, 0x715, 0x710, 0x71f, 0x710,
0x72a, 0x715, 0x710, 0x72a, 0x20, 0x72b, 0x720, 0x721, 0x710, 0x713, 0x71d, 0x712,
0x718, 0x72c, 0x71d, 0x715, 0x718, 0x710, 0x720, 0x710, 0x710, 0x720, 0x20, 0x725,
0x71d, 0x718, 0x722, 0x726, 0x72a, 0x71d, 0x72c, 0x710, 0x718, 0x722, 0x713, 0x712,
0x718, 0x72a, 0x718, 0x722, 0x717, 0x72a, 0x710, 0x72a, 0x71d, 0x71d, 0x718, 0x717,
0x722, 0x71d, 0x723, 0x712, 0x718, 0x72a, 0x713, 0x713, 0x718, 0x712, 0x710, 0x71f,
0x710, 0x721, 0x726, 0x710, 0x720, 0x710, 0x71a, 0x72a, 0x71b, 0x718, 0x721, 0x71f,
0x71d, 0x713, 0x710, 0x720, 0x71d, 0x71f, 0x71d, 0x722, 0x72b, 0x710, 0x723, 0x710,
0x720, 0x710, 0x713, 0x718, 0x723, 0x720, 0x71d, 0x712, 0x72a, 0x71d, 0x712, 0x71d,
0x720, 0x720, 0x718, 0x721, 0x71d, 0x720, 0x718, 0x710, 0x722, 0x715, 0x710, 0x720,
0x718, 0x712, 0x718, 0x721, 0x712, 0x72b, 0x71d, 0x720, 0x718, 0x723, 0x710, 0x71f,
0x710, 0x721, 0x710, 0x720, 0x710, 0x712, 0x718, 0x721, 0x710, 0x726, 0x718, 0x72c,
0x718, 0x721, 0x710, 0x723, 0x71d, 0x72a, 0x718, 0x721, 0x712, 0x710, 0x712, 0x710,
0x722, 0x71d, 0x721, 0x718, 0x729, 0x715, 0x71d, 0x72b, 0x718, 0x721, 0x718, 0x722,
0x72a, 0x718, 0x712, 0x71d, 0x710, 0x722, 0x71d, 0x72a, 0x718, 0x712, 0x71d, 0x722,
0x713, 0x721, 0x71d, 0x722, 0x710, 0x722, 0x71d, 0x710, 0x721, 0x71d, 0x722, 0x718,
0x710, 0x71f, 0x72b, 0x718, 0x72c, 0x710, 0x718, 0x710, 0x713, 0x710, 0x715, 0x710,
0x718, 0x713, 0x710, 0x718, 0x726, 0x718, 0x72a, 0x72c, 0x718, 0x2d, 0x722, 0x718,
0x712, 0x718, 0x723, 0x710, 0x718, 0x20, 0x72c, 0x718, 0x721, 0x71d, 0x71b, 0x72a,
0x71d, 0x726, 0x718, 0x720, 0x71d, 0x723, 0x72c, 0x718, 0x722, 0x723, 0x718, 0x71d,
0x722, 0x715, 0x717, 0x718, 0x71f, 0x710, 0x715, 0x710, 0x71f, 0x710, 0x722, 0x71f,
0x718, 0x72a, 0x713, 0x710, 0x722, 0x713, 0x718, 0x71d, 0x710, 0x710, 0x722, 0x72c,
0x71d, 0x713, 0x718, 0x710, 0x710, 0x72a, 0x710, 0x713, 0x710, 0x718, 0x71d, 0x722,
0x710, 0x712, 0x718, 0x710, 0x71d, 0x722, 0x718, 0x723, 0x20, 0x725, 0x71d, 0x72a,
0x723, 0x71f, 0x710, 0x72c, 0x710, 0x721, 0x72a, 0x71f, 0x710, 0x71f, 0x718, 0x72a,
0x715, 0x718, 0x712, 0x710, 0x713, 0x718, 0x713, 0x718, 0x71d, 0x720, 0x710, 0x20,
0x72a, 0x71d, 0x718, 0x717, 0x710, 0x721, 0x722, 0x715, 0x718, 0x719, 0x710, 0x72a,
0x71d, 0x718, 0x20, 0x713, 0x71d, 0x713, 0x718, 0x723, 0x723, 0x720, 0x72c, 0x710,
0x723, 0x710, 0x722, 0x20, 0x718, 0x710, 0x722, 0x723, 0x710, 0x722, 0x20, 0x720,
0x718, 0x71d, 0x723, 0x72c, 0x718, 0x71f, 0x718, 0x721, 0x710, 0x722, 0x710, 0x718,
0x72b, 0x718, 0x710, 0x71d, 0x710, 0x710, 0x72a, 0x718, 0x712, 0x710, 0x710, 0x723,
0x718, 0x722, 0x71f, 0x723, 0x71d, 0x718, 0x722, 0x710, 0x72c, 0x71d, 0x71f, 0x718,
0x71f, 0x710, 0x722, 0x712, 0x710, 0x717, 0x71d, 0x710, 0x712, 0x710, 0x717, 0x71d,
0x710, 0x20, 0x712, 0x710, 0x722, 0x71d, 0x72a, 0x723, 0x712, 0x710, 0x72a, 0x712,
0x715, 0x718, 0x723, 0x712, 0x71d, 0x720, 0x71d, 0x721, 0x712, 0x71d, 0x720, 0x71d,
0x719, 0x712, 0x720, 0x710, 0x722, 0x71f, 0x2d, 0x723, 0x710, 0x712, 0x720, 0x718,
0x722, 0x712, 0x718, 0x725, 0x20, 0x712, 0x71d, 0x723, 0x72c, 0x710, 0x712, 0x718,
0x713, 0x718, 0x72c, 0x710, 0x712, 0x718, 0x71d, 0x719, 0x71d, 0x71f, 0x721, 0x712,
0x72a, 0x713, 0x20, 0x712, 0x710, 0x71d, 0x71f, 0x710, 0x721, 0x726, 0x718, 0x20,
0x713, 0x72a, 0x722, 0x715, 0x71d, 0x71f, 0x710, 0x722, 0x71f, 0x718, 0x722, 0x71f,
0x710, 0x72a, 0x710, 0x71f, 0x723, 0x71f, 0x710, 0x71d, 0x710, 0x722, 0x71f, 0x710,
0x71d, 0x721, 0x71d, 0x722, 0x72b, 0x71f, 0x713, 0x718, 0x71f, 0x71d, 0x718, 0x710,
0x718, 0x710, 0x723, 0x71d, 0x718, 0x715, 0x710, 0x715, 0x20, 0x710, 0x718, 0x72a,
0x71d, 0x719, 0x71f, 0x718, 0x723, 0x72c, 0x710, 0x20, 0x72a, 0x71d, 0x71f, 0x710,
0x71f, 0x72a, 0x71d, 0x723, 0x72c, 0x718, 0x722, 0x71f, 0x718, 0x71d, 0x710, 0x712,
0x710, 0x71f, 0x718, 0x72a, 0x710, 0x71f, 0x710, 0x718, 0x715, 0x710, 0x722, 0x721,
0x72a, 0x71f, 0x72b, 0x712, 0x71d, 0x722, 0x715, 0x710, 0x723, 0x718, 0x722, 0x715,
0x710, 0x723, 0x718, 0x722, 0x20, 0x71f, 0x72a, 0x71d, 0x71f, 0x715, 0x722, 0x712,
0x72a, 0x715, 0x71d, 0x72c, 0x72a, 0x718, 0x71d, 0x72c, 0x715, 0x718, 0x721, 0x71d,
0x722, 0x71d, 0x71f, 0x710, 0x710, 0x715, 0x721, 0x718, 0x722, 0x72c, 0x718, 0x722,
0x710, 0x71d, 0x72a, 0x718, 0x722, 0x71d, 0x726, 0x71d, 0x710, 0x720, 0x20, 0x723,
0x720, 0x712, 0x710, 0x715, 0x718, 0x72a, 0x726, 0x718, 0x72a, 0x72c, 0x20, 0x722,
0x71d, 0x720, 0x723, 0x718, 0x722, 0x726, 0x718, 0x72a, 0x72c, 0x710, 0x720, 0x71d,
0x719, 0x710, 0x713, 0x720, 0x71d, 0x723, 0x20, 0x712, 0x710, 0x71d, 0x713, 0x718,
0x723, 0x20, 0x712, 0x710, 0x71d, 0x713, 0x72a, 0x710, 0x722, 0x715, 0x20, 0x72c,
0x718, 0x72a, 0x71f, 0x713, 0x72a, 0x71d, 0x722, 0x715, 0x710, 0x713, 0x718, 0x710,
0x715, 0x710, 0x720, 0x718, 0x726, 0x71d, 0x713, 0x718, 0x710, 0x72c, 0x721, 0x710,
0x720, 0x710, 0x713, 0x718, 0x710, 0x71d, 0x710, 0x729, 0x718, 0x71d, 0x720, 0x713,
0x718, 0x71d, 0x710, 0x722, 0x710, 0x717, 0x720, 0x71d, 0x726, 0x710, 0x71f, 0x723,
0x717, 0x710, 0x712, 0x710, 0x722, 0x710, 0x717, 0x71d, 0x72a, 0x721, 0x718, 0x723,
0x71d, 0x710, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x726, 0x718, 0x720,
0x71d, 0x723, 0x722, 0x718, 0x71f, 0x723, 0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d,
0x710, 0x722, 0x710, 0x721, 0x72a, 0x71d, 0x722, 0x713, 0x718, 0x60c, 0x20, 0x710,
0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x726, 0x71d, 0x72c, 0x71d, 0x72a, 0x723,
0x712, 0x71d, 0x72a, 0x713, 0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722,
0x710, 0x721, 0x722, 0x715, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c, 0x71d, 0x720,
0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x712, 0x71d, 0x712,
0x710, 0x71d, 0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x712,
0x71d, 0x722, 0x723, 0x71d, 0x722, 0x71d, 0x723, 0x60c, 0x20, 0x710, 0x722, 0x715,
0x71d, 0x710, 0x722, 0x710, 0x718, 0x71d, 0x722, 0x710, 0x721, 0x710, 0x71f, 0x60c,
0x20, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x710, 0x722, 0x718, 0x712,
0x71d, 0x71f, 0x710, 0x71d, 0x729, 0x720, 0x718, 0x71d, 0x72c, 0x713, 0x721, 0x710,
0x71d, 0x71f, 0x710, 0x713, 0x718, 0x722, 0x718, 0x720, 0x718, 0x71d, 0x723, 0x712,
0x71d, 0x720, 0x721, 0x718, 0x722, 0x72c, 0x710, 0x71f, 0x71d, 0x720, 0x718, 0x60c,
0x20, 0x71f, 0x71d, 0x722, 0x72c, 0x710, 0x71f, 0x71d, 0x71f, 0x72a, 0x720, 0x71d,
0x722, 0x713, 0x729, 0x720, 0x710, 0x20, 0x726, 0x710, 0x719, 0x720, 0x71d, 0x721,
0x710, 0x720, 0x718, 0x723, 0x20, 0x710, 0x722, 0x713, 0x720, 0x718, 0x723, 0x72a,
0x718, 0x712, 0x742, 0x725, 0x710, 0x20, 0x715, 0x72b, 0x720, 0x71d, 0x71b, 0x710,
0x20, 0x72c, 0x71a, 0x72c, 0x71d, 0x710, 0x721, 0x710, 0x723, 0x71d, 0x710, 0x718,
0x721, 0x710, 0x722, 0x710, 0x713, 0x718, 0x710, 0x721, 0x710, 0x722, 0x710, 0x718,
0x723, 0x721, 0x72a, 0x71d, 0x713, 0x718, 0x72c, 0x721, 0x710, 0x72a, 0x72c, 0x71d,
0x722, 0x71d, 0x729, 0x721, 0x710, 0x72c, 0x710, 0x721, 0x718, 0x72a, 0x718, 0x723,
0x721, 0x719, 0x71b, 0x720, 0x710, 0x722, 0x721, 0x722, 0x718, 0x721, 0x71d, 0x722,
0x71d, 0x721, 0x72a, 0x71d, 0x715, 0x710, 0x721, 0x71b, 0x720, 0x71f, 0x710, 0x72c,
0x720, 0x710, 0x721, 0x715, 0x71d, 0x722, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x71f,
0x723, 0x71d, 0x71f, 0x718, 0x721, 0x729, 0x718, 0x720, 0x718, 0x722, 0x721, 0x718,
0x722, 0x71f, 0x72c, 0x718, 0x722, 0x721, 0x718, 0x722, 0x71b, 0x72a, 0x710, 0x71d,
0x721, 0x718, 0x722, 0x72c, 0x71d, 0x712, 0x71d, 0x715, 0x71d, 0x718, 0x721, 0x718,
0x722, 0x72c, 0x723, 0x71d, 0x72a, 0x710, 0x72c, 0x722, 0x710, 0x723, 0x710, 0x718,
0x722, 0x71d, 0x718, 0x20, 0x71d, 0x718, 0x72a, 0x71f, 0x722, 0x718, 0x72a, 0x718,
0x722, 0x717, 0x710, 0x712, 0x71d, 0x718, 0x720, 0x710, 0x60c, 0x20, 0x715, 0x710,
0x71f, 0x718, 0x72c, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710,
0x723, 0x71d, 0x722, 0x72c, 0x72a, 0x60c, 0x20, 0x715, 0x710, 0x71f, 0x718, 0x72c,
0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x722, 0x71d, 0x718,
0x20, 0x723, 0x710, 0x720, 0x71d, 0x721, 0x60c, 0x200c, 0x20, 0x715, 0x710, 0x71f,
0x718, 0x72c, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x718,
0x713, 0x71d, 0x722, 0x710, 0x713, 0x710, 0x726, 0x722, 0x721, 0x710, 0x726, 0x710,
0x72a, 0x710, 0x721, 0x710, 0x72a, 0x71d, 0x712, 0x718, 0x726, 0x71d, 0x722, 0x71d,
0x71f, 0x723, 0x726, 0x718, 0x72a, 0x72c, 0x20, 0x710, 0x718, 0x20, 0x726, 0x72a,
0x71d, 0x722, 0x723, 0x726, 0x718, 0x72a, 0x72c, 0x20, 0x715, 0x710, 0x723, 0x726,
0x722, 0x71d, 0x710, 0x726, 0x718, 0x72a, 0x72c, 0x718, 0x20, 0x712, 0x71d, 0x720,
0x717, 0x718, 0x726, 0x718, 0x72a, 0x72c, 0x718, 0x20, 0x72a, 0x71d, 0x71f, 0x718,
0x726, 0x718, 0x722, 0x72c, 0x710, 0x20, 0x725, 0x72a, 0x71d, 0x722, 0x723, 0x72a,
0x710, 0x722, 0x71f, 0x722, 0x20, 0x710, 0x722, 0x720, 0x71d, 0x72c, 0x72a, 0x71d,
0x723, 0x71d, 0x726, 0x71d, 0x72a, 0x71d, 0x713, 0x71d, 0x722, 0x710, 0x72a, 0x71d,
0x723, 0x718, 0x720, 0x718, 0x72c, 0x72a, 0x71d, 0x718, 0x20, 0x712, 0x72a, 0x722,
0x71f, 0x718, 0x723, 0x710, 0x722, 0x72c, 0x710, 0x72a, 0x721, 0x723, 0x710, 0x722,
0x72c, 0x71d, 0x710, 0x713, 0x718, 0x723, 0x722, 0x72c, 0x718, 0x20, 0x715, 0x718,
0x721, 0x71d, 0x722, 0x713, 0x718, 0x723, 0x710, 0x718, 0x20, 0x726, 0x710, 0x718,
0x720, 0x718, 0x710, 0x71d, 0x71b, 0x718, 0x729, 0x718, 0x72a, 0x721, 0x71d, 0x72c,
0x723, 0x71d, 0x71b, 0x71f, 0x710, 0x721, 0x72a, 0x20, 0x712, 0x72a, 0x20, 0x72c,
0x718, 0x720, 0x721, 0x71d, 0x721, 0x72a, 0x20, 0x71d, 0x718, 0x71a, 0x722, 0x722,
0x723, 0x710, 0x722, 0x72c, 0x20, 0x71f, 0x72c, 0x723, 0x721, 0x72a, 0x72c, 0x71d,
0x20, 0x720, 0x718, 0x72b, 0x710, 0x721, 0x72a, 0x20, 0x72c, 0x710, 0x718, 0x721,
0x710, 0x721, 0x72a, 0x20, 0x712, 0x722, 0x723, 0x722, 0x72c, 0x722, 0x715, 0x718,
0x72a, 0x72c, 0x710, 0x72c, 0x71d, 0x713, 0x718, 0x723, 0x71d, 0x713, 0x710, 0x720,
0x726, 0x710, 0x72c, 0x71d, 0x710, 0x718, 0x710, 0x722, 0x710, 0x72c, 0x718, 0x72a,
0x718, 0x722, 0x72c, 0x718, 0x72c, 0x718, 0x72a, 0x72c, 0x718, 0x720, 0x710, 0x712,
0x722, 0x71f, 0x718, 0x712, 0x71d, 0x72a, 0x723, 0x718, 0x723, 0x72c, 0x710, 0x20,
0x71a, 0x718, 0x72a, 0x72c, 0x710, 0x718, 0x71d, 0x722, 0x71d, 0x726, 0x713, 0x71d,
0x729, 0x718, 0x72c, 0x710, 0x72c, 0x71f, 0x710, 0x71d, 0x723, 0x71d, 0x715, 0x712,
0x71d, 0x723, 0x715, 0x718, 0x721, 0x718, 0x722, 0x72c, 0x20, 0x715, 0x710, 0x718,
0x72a, 0x712, 0x71d, 0x720, 0x721, 0x710, 0x71f, 0x710, 0x718, 0x72a, 0x71d, 0x721,
0x710, 0x718, 0x723, 0x718, 0x722, 0x721, 0x71f, 0x721, 0x718, 0x72a, 0x715, 0x718,
0x726, 0x710, 0x720, 0x721, 0x71d, 0x72a, 0x72a, 0x718, 0x72c, 0x71d, 0x72a, 0x710,
0x723, 0x71d, 0x718, 0x710, 0x72c, 0x72a, 0x718, 0x720, 0x712, 0x718, 0x723, 0x72c,
0x718, 0x71f, 0x720, 0x718, 0x722, 0x713, 0x71d, 0x725, 0x72a, 0x712, 0x71d, 0x71d,
0x722, 0x725, 0x715, 0x722, 0x710, 0x720, 0x721, 0x710, 0x72c, 0x71d, 0x725, 0x721,
0x710, 0x722, 0x710, 0x722, 0x710, 0x715, 0x71d, 0x72a, 0x710, 0x71f, 0x72c, 0x710,
0x718, 0x710, 0x71f, 0x72c, 0x718, 0x712, 0x725, 0x72b, 0x729, 0x710, 0x712, 0x710,
0x715, 0x710, 0x72c, 0x71d, 0x72a, 0x718, 0x712, 0x713, 0x715, 0x715, 0x712, 0x71a,
0x72a, 0x71d, 0x722, 0x712, 0x710, 0x722, 0x71f, 0x718, 0x71f, 0x712, 0x710, 0x72a,
0x722, 0x710, 0x718, 0x720, 0x712, 0x71d, 0x72a, 0x718, 0x72c, 0x712, 0x71d, 0x72b,
0x71f, 0x71d, 0x71f, 0x712, 0x72a, 0x718, 0x722, 0x710, 0x71d, 0x72c, 0x72b, 0x71d,
0x72c, 0x710, 0x71f, 0x718, 0x720, 0x718, 0x721, 0x712, 0x718, 0x715, 0x72a, 0x721,
0x723, 0x718, 0x729, 0x715, 0x71f, 0x710, 0x715, 0x71d, 0x720, 0x71d, 0x715, 0x718,
0x712, 0x710, 0x71d, 0x715, 0x718, 0x72b, 0x710, 0x722, 0x712, 0x71d, 0x726, 0x710,
0x721, 0x710, 0x713, 0x718, 0x723, 0x72c, 0x710, 0x725, 0x719, 0x710, 0x71a, 0x712,
0x72a, 0x718, 0x722, 0x721, 0x715, 0x71d, 0x722, 0x72c, 0x710, 0x20, 0x715, 0x717,
0x718, 0x20, 0x71f, 0x71d, 0x20, 0x721, 0x71d, 0x722, 0x717, 0x718, 0x722, 0x713,
0x20, 0x71f, 0x718, 0x722, 0x713, 0x717, 0x718, 0x712, 0x715, 0x710, 0x71d, 0x72a,
0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x713, 0x710, 0x71f, 0x710, 0x72a, 0x72c, 0x710,
0x713, 0x710, 0x71d, 0x726, 0x718, 0x72a, 0x710, 0x710, 0x718, 0x72a, 0x72b, 0x720,
0x721, 0x71f, 0x710, 0x712, 0x718, 0x720, 0x71f, 0x710, 0x721, 0x72c, 0x72b, 0x710,
0x72c, 0x71f, 0x710, 0x71f, 0x72a, 0x710, 0x71f, 0x71d, 0x71f, 0x710, 0x72c, 0x721,
0x710, 0x722, 0x715, 0x718, 0x71a, 0x710, 0x722, 0x715, 0x71d, 0x713, 0x710, 0x71f,
0x718, 0x720, 0x71f, 0x72c, 0x710, 0x71f, 0x72a, 0x710, 0x723, 0x722, 0x718, 0x71d,
0x72a, 0x723, 0x71f, 0x71f, 0x718, 0x710, 0x720, 0x710, 0x20, 0x720, 0x718, 0x721,
0x726, 0x718, 0x72a, 0x71f, 0x718, 0x72b, 0x71d, 0x722, 0x713, 0x71f, 0x718, 0x71d,
0x72c, 0x721, 0x710, 0x713, 0x710, 0x715, 0x710, 0x722, 0x721, 0x710, 0x71f, 0x710,
0x723, 0x710, 0x72a, 0x721, 0x710, 0x722, 0x71d, 0x720, 0x710, 0x721, 0x723, 0x729,
0x71b, 0x722, 0x71d, 0x729, 0x718, 0x723, 0x71d, 0x710, 0x722, 0x718, 0x712, 0x718,
0x71f, 0x718, 0x719, 0x722, 0x71d, 0x72c, 0x723, 0x71f, 0x722, 0x718, 0x712, 0x718,
0x723, 0x71d, 0x712, 0x72a, 0x723, 0x71f, 0x710, 0x718, 0x721, 0x723, 0x71f, 0x710,
0x718, 0x72a, 0x710, 0x720, 0x726, 0x722, 0x718, 0x721, 0x20, 0x726, 0x722, 0x726,
0x718, 0x722, 0x72c, 0x71d, 0x710, 0x722, 0x710, 0x71f, 0x726, 0x71d, 0x718, 0x722,
0x713, 0x71d, 0x722, 0x713, 0x729, 0x71b, 0x72a, 0x71f, 0x718, 0x723, 0x72c, 0x710,
0x722, 0x710, 0x71d, 0x71f, 0x71d, 0x719, 0x71d, 0x720, 0x718, 0x72a, 0x715, 0x710,
0x72a, 0x71d, 0x710, 0x715, 0x723, 0x710, 0x71a, 0x710, 0x720, 0x71d, 0x722, 0x723,
0x721, 0x72a, 0x71f, 0x722, 0x715, 0x723, 0x710, 0x718, 0x720, 0x72b, 0x722, 0x713,
0x717, 0x710, 0x71d, 0x71d, 0x723, 0x71d, 0x722, 0x713, 0x710, 0x726, 0x718, 0x72a,
0x723, 0x72a, 0x71d, 0x715, 0x722, 0x71d, 0x71f, 0x718, 0x720, 0x71d, 0x721, 0x723,
0x71f, 0x72c, 0x710, 0x71d, 0x726, 0x710, 0x71d, 0x72c, 0x72b, 0x71f, 0x71d, 0x722,
0x72c, 0x72c, 0x726, 0x720, 0x71d, 0x723, 0x72c, 0x717, 0x72a, 0x722, 0x72c, 0x71d,
0x721, 0x726, 0x718, 0x71b, 0x718, 0x71f, 0x71d, 0x718, 0x72c, 0x718, 0x721, 0x723,
0x71f, 0x710, 0x718, 0x720, 0x710, 0x722, 0x712, 0x710, 0x72c, 0x718, 0x72a, 0x710,
0x718, 0x72a, 0x718, 0x721, 0x71f, 0x71d, 0x710, 0x718, 0x723, 0x72c, 0x2d, 0x722,
0x71d, 0x72a, 0x710, 0x712, 0x71d, 0x710, 0x722, 0x72c, 0x71d, 0x710, 0x722, 0x712,
0x720, 0x710, 0x715, 0x71d, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f, 0x71d, 0x710,
0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x71d, 0x722, 0x713, 0x718, 0x722, 0x71d, 0x71f,
0x710, 0x72c, 0x71d, 0x72a, 0x722, 0x712, 0x71d, 0x72a, 0x713, 0x71d, 0x72a, 0x712,
0x710, 0x722, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x719, 0x718,
0x72a, 0x712, 0x71d, 0x72a, 0x721, 0x71d, 0x718, 0x715, 0x710, 0x71f, 0x710, 0x722,
0x72a, 0x71d, 0x71f, 0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a, 0x715, 0x71d, 0x726,
0x710, 0x72a, 0x718, 0x721, 0x715, 0x710, 0x71d, 0x72a, 0x710, 0x72a, 0x710, 0x71d,
0x71f, 0x712, 0x71d, 0x71f, 0x72c, 0x721, 0x71d, 0x722, 0x20, 0x713, 0x718, 0x72a,
0x713, 0x71d, 0x710, 0x721, 0x72a, 0x72c, 0x71d, 0x20, 0x717, 0x71d, 0x720, 0x71d,
0x722, 0x710, 0x723, 0x72c, 0x710, 0x722, 0x720, 0x71d, 0x712, 0x72a, 0x71d, 0x723,
0x712, 0x710, 0x722, 0x712, 0x72a, 0x718, 0x71f, 0x71d, 0x722, 0x20, 0x717, 0x71d,
0x720, 0x715, 0x72a, 0x718, 0x71d, 0x722, 0x710, 0x718, 0x71f, 0x720, 0x710, 0x717,
0x718, 0x712, 0x72a, 0x72c, 0x720, 0x71d, 0x722, 0x715, 0x721, 0x710, 0x722, 0x720,
0x718, 0x72a, 0x715, 0x20, 0x717, 0x710, 0x718, 0x721, 0x71d, 0x720, 0x712, 0x718,
0x72a, 0x722, 0x726, 0x71d, 0x72a, 0x72c, 0x723, 0x71d, 0x715, 0x722, 0x71d, 0x710,
0x721, 0x723, 0x72c, 0x72a, 0x715, 0x721, 0x710, 0x722, 0x715, 0x718, 0x72a, 0x710,
0x710, 0x723, 0x72c, 0x72a, 0x710, 0x71a, 0x722, 0x710, 0x72c, 0x722, 0x718, 0x723,
0x712, 0x720, 0x713, 0x72a, 0x715, 0x712, 0x72a, 0x720, 0x71d, 0x722, 0x712, 0x72a,
0x72c, 0x71d, 0x723, 0x720, 0x712, 0x710, 0x200f, 0x712, 0x72a, 0x718, 0x71f, 0x723,
0x720, 0x712, 0x718, 0x729, 0x718, 0x72a, 0x72b, 0x71b, 0x712, 0x718, 0x715, 0x726,
0x72b, 0x71b, 0x712, 0x718, 0x723, 0x71d, 0x722, 0x713, 0x722, 0x71f, 0x71d, 0x723,
0x71d, 0x722, 0x710, 0x718, 0x71f, 0x718, 0x726, 0x722, 0x717, 0x710, 0x713, 0x722,
0x715, 0x712, 0x720, 0x722, 0x713, 0x712, 0x72a, 0x720, 0x71b, 0x72a, 0x713, 0x718,
0x72a, 0x722, 0x719, 0x71d, 0x717, 0x720, 0x723, 0x722, 0x71f, 0x71d, 0x713, 0x719,
0x72a, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x710, 0x722, 0x710, 0x723, 0x71b, 0x722,
0x712, 0x718, 0x720, 0x713, 0x71d, 0x72a, 0x719, 0x71d, 0x71f, 0x710, 0x720, 0x71d,
0x722, 0x71d, 0x722, 0x713, 0x72a, 0x710, 0x715, 0x71f, 0x71d, 0x72a, 0x718, 0x712,
0x71f, 0x71d, 0x71d, 0x712, 0x720, 0x72b, 0x712, 0x718, 0x722, 0x710, 0x720, 0x71d,
0x718, 0x712, 0x720, 0x71d, 0x710, 0x722, 0x710, 0x720, 0x718, 0x722, 0x715, 0x718,
0x722, 0x720, 0x718, 0x71f, 0x723, 0x721, 0x712, 0x718, 0x72a, 0x713, 0x721, 0x715,
0x72a, 0x71d, 0x715, 0x721, 0x71d, 0x720, 0x71b, 0x710, 0x721, 0x710, 0x72a, 0x71d,
0x717, 0x710, 0x721, 0x721, 0x71d, 0x722, 0x723, 0x71f, 0x721, 0x718, 0x722, 0x710,
0x71f, 0x718, 0x721, 0x718, 0x723, 0x71f, 0x718, 0x710, 0x718, 0x723, 0x720, 0x718,
0x726, 0x710, 0x72a, 0x71d, 0x723, 0x726, 0x718, 0x715, 0x713, 0x718, 0x72a, 0x71d,
0x71f, 0x710, 0x726, 0x72a, 0x710, 0x713, 0x72a, 0x71d, 0x713, 0x710, 0x72a, 0x718,
0x721, 0x71d, 0x723, 0x722, 0x20, 0x721, 0x72a, 0x71d, 0x722, 0x718, 0x723, 0x72a,
0x710, 0x71d, 0x71d, 0x712, 0x718, 0x723, 0x72a, 0x710, 0x72c, 0x718, 0x712, 0x723,
0x721, 0x726, 0x72a, 0x718, 0x726, 0x720, 0x723, 0x729, 0x718, 0x726, 0x71d, 0x710,
0x723, 0x718, 0x726, 0x71d, 0x710, 0x723, 0x72c, 0x718, 0x71f, 0x717, 0x718, 0x720,
0x721, 0x72c, 0x710, 0x720, 0x71d, 0x722, 0x72c, 0x71d, 0x72a, 0x710, 0x722, 0x710,
0x718, 0x720, 0x71d, 0x722, 0x718, 0x712, 0x723, 0x71f, 0x712, 0x715, 0x718, 0x719,
0x718, 0x710, 0x72c, 0x71d, 0x729, 0x710, 0x722, 0x712, 0x71d, 0x71d, 0x722, 0x710,
0x712, 0x720, 0x722, 0x718, 0x723, 0x712, 0x718, 0x720, 0x713, 0x718, 0x713, 0x72a,
0x710, 0x715, 0x718, 0x710, 0x72a, 0x723, 0x718, 0x719, 0x710, 0x713, 0x72a, 0x712,
0x719, 0x71d, 0x718, 0x72a, 0x71a, 0x710, 0x722, 0x72c, 0x710, 0x722, 0x710, 0x722,
0x72a, 0x71d, 0x712, 0x718, 0x72c, 0x72b, 0x713, 0x718, 0x723, 0x71f, 0x72a, 0x71d,
0x723, 0x72c, 0x721, 0x723, 0x71f, 0x718, 0x71f, 0x718, 0x723, 0x71f, 0x718, 0x721,
0x718, 0x72a, 0x718, 0x71f, 0x71d, 0x72a, 0x713, 0x718, 0x720, 0x71d, 0x722, 0x721,
0x710, 0x717, 0x71d, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x721, 0x710, 0x720,
0x715, 0x71d, 0x712, 0x71d, 0x72c, 0x710, 0x721, 0x718, 0x72a, 0x71d, 0x72b, 0x718,
0x723, 0x721, 0x710, 0x71d, 0x718, 0x72c, 0x72a, 0x71d, 0x718, 0x722, 0x71d, 0x718,
0x722, 0x710, 0x726, 0x71d, 0x710, 0x710, 0x71f, 0x720, 0x710, 0x722, 0x715, 0x712,
0x718, 0x713, 0x710, 0x71d, 0x722, 0x712, 0x71d, 0x720, 0x72c, 0x72b, 0x710, 0x72c,
0x721, 0x72c, 0x72b, 0x718, 0x71f, 0x726, 0x728, 0x71a, 0x710, 0x71d, 0x726, 0x710,
0x72c, 0x726, 0x710, 0x71f, 0x710, 0x718, 0x726, 0x718, 0x726, 0x71d, 0x713, 0x71d,
0x726, 0x718, 0x722, 0x710, 0x726, 0x718, 0x72c, 0x71d, 0x713, 0x710, 0x720, 0x710,
0x726, 0x713, 0x718, 0x723, 0x713, 0x721, 0x712, 0x71d, 0x72a, 0x713, 0x718, 0x710,
0x715, 0x710, 0x720, 0x71f, 0x710, 0x722, 0x710, 0x720, 0x71f, 0x710, 0x722, 0x72c,
0x718, 0x722, 0x71f, 0x71d, 0x72a, 0x71d, 0x721, 0x710, 0x72c, 0x71d, 0x71f, 0x718,
0x723, 0x72a, 0x710, 0x71d, 0x71f, 0x718, 0x710, 0x713, 0x710, 0x720, 0x71d, 0x722,
0x721, 0x710, 0x713, 0x718, 0x72a, 0x718, 0x721, 0x710, 0x72a, 0x71f, 0x710, 0x718,
0x723, 0x710, 0x723, 0x721, 0x71d, 0x715, 0x718, 0x710, 0x71d, 0x722, 0x710, 0x718,
0x72a, 0x718, 0x722, 0x71d, 0x718, 0x71d, 0x722, 0x718, 0x72a, 0x726, 0x720, 0x71f,
0x722, 0x718, 0x721, 0x71d, 0x710, 0x726, 0x710, 0x713, 0x718, 0x20, 0x726, 0x710,
0x713, 0x718, 0x726, 0x720, 0x710, 0x718, 0x726, 0x71d, 0x72c, 0x71f, 0x710, 0x71d,
0x72a, 0x722, 0x726, 0x718, 0x717, 0x722, 0x726, 0x710, 0x71d, 0x726, 0x718, 0x72a,
0x72c, 0x20, 0x721, 0x718, 0x72a, 0x71d, 0x723, 0x712, 0x710, 0x71d, 0x72a, 0x710,
0x72a, 0x718, 0x72c, 0x718, 0x722, 0x713, 0x710, 0x723, 0x710, 0x71d, 0x726, 0x710,
0x722, 0x72c, 0x710, 0x717, 0x71d, 0x72c, 0x71d, 0x72c, 0x710, 0x72a, 0x710, 0x718,
0x710, 0x72c, 0x718, 0x722, 0x713, 0x710, 0x72c, 0x710, 0x726, 0x718, 0x718, 0x710,
0x71d, 0x71f, 0x718, 0x71d, 0x720, 0x71d, 0x723, 0x410, 0x431, 0x438, 0x434, 0x4b7,
0x43e, 0x43d, 0x410, 0x43b, 0x4b7, 0x430, 0x437, 0x43e, 0x438, 0x440, 0x411, 0x430,
0x43d, 0x4b7, 0x443, 0x43b, 0x411, 0x443, 0x4b7, 0x443, 0x43c, 0x431, 0x443, 0x440,
0x430, 0x49a, 0x43e, 0x4b3, 0x438, 0x440, 0x430, 0x414, 0x43e, 0x440, 0x443, 0x441,
0x441, 0x430, 0x43b, 0x43e, 0x43c, 0x4b6, 0x438, 0x431, 0x443, 0x442, 0x438, 0x42d,
0x43b, 0x20, 0x410, 0x438, 0x443, 0x43d, 0x4b6, 0x443, 0x431, 0x430, 0x41d, 0x4b7,
0x430, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x43d, 0x43a, 0x43e, 0x440, 0x438, 0x447,
0x410, 0x43d, 0x433, 0x438, 0x43b, 0x44f, 0x4b6, 0x443, 0x4b7, 0x443, 0x439, 0x411,
0x430, 0x445, 0x438, 0x44f, 0x411, 0x430, 0x445, 0x438, 0x44f, 0x20, 0x434, 0x435,
0x20, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x411, 0x43e, 0x439,
0x437, 0x41a, 0x435, 0x43c, 0x431, 0x440, 0x438, 0x4b7, 0x20, 0x411, 0x44d, 0x439,
0x421, 0x44e, 0x434, 0x430, 0x434, 0x20, 0x425, 0x443, 0x430, 0x440, 0x435, 0x441,
0x413, 0x43b, 0x44d, 0x439, 0x441, 0x20, 0x411, 0x44d, 0x439, 0x413, 0x443, 0x441,
0x20, 0x411, 0x44d, 0x439, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x20, 0x422, 0x435,
0x440, 0x43a, 0x4b2, 0x430, 0x432, 0x430, 0x43d, 0x430, 0x41f, 0x435, 0x442, 0x435,
0x440, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430,
0x43d, 0x430, 0x412, 0x435, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x418, 0x43d, 0x434,
0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x441, 0x2c,
0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x4b6, 0x443, 0x43d, 0x43e,
0x41a, 0x440, 0x430, 0x43b, 0x435, 0x43d, 0x434, 0x439, 0x43a, 0x41b, 0x43e, 0x441,
0x2d, 0x410, 0x43d, 0x4b7, 0x435, 0x43b, 0x435, 0x441, 0x41a, 0x432, 0x430, 0x440,
0x442, 0x430, 0x43b, 0x438, 0x20, 0x41f, 0x43e, 0x451, 0x43d, 0x438, 0x20, 0x41f,
0x440, 0x438, 0x43d, 0x441, 0x41d, 0x44e, 0x2d, 0x419, 0x43e, 0x440, 0x43a, 0x411,
0x435, 0x439, 0x43b, 0x430, 0x2c, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430,
0x438, 0x20, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x4e3, 0x421, 0x435, 0x43d, 0x442,
0x440, 0x2c, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x438, 0x20, 0x428,
0x438, 0x43c, 0x43e, 0x43b, 0x4e3, 0x41d, 0x44e, 0x20, 0x421, 0x430, 0x43b, 0x435,
0x43c, 0x2c, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x438, 0x20, 0x428,
0x438, 0x43c, 0x43e, 0x43b, 0x4e3, 0x41f, 0x43e, 0x440, 0x442, 0x438, 0x20, 0x418,
0x441, 0x43f, 0x430, 0x43d, 0x438, 0x44f, 0x41f, 0x43e, 0x440, 0x442, 0x443, 0x20,
0x412, 0x435, 0x43b, 0x445, 0x43e, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x411, 0x430,
0x440, 0x442, 0x435, 0x43b, 0x435, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x20,
0x4b6, 0x43e, 0x43d, 0x441, 0x421, 0x432, 0x438, 0x444, 0x442, 0x2d, 0x41a, 0x430,
0x440, 0x440, 0x435, 0x43d, 0x442, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434,
0x2019, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x43b, 0x41c, 0x430, 0x43a, 0x43a, 0x443,
0x430, 0x440, 0x438, 0x41c, 0x430, 0x432, 0x441, 0x43e, 0x43d, 0x41c, 0x430, 0x43a,
0x43c, 0x435, 0x440, 0x434, 0x43e, 0x41b, 0x43e, 0x43d, 0x433, 0x439, 0x435, 0x440,
0x431, 0x451, 0x43d, 0x410, 0x434, 0x430, 0x43d, 0x410, 0x43b, 0x43c, 0x430, 0x430,
0x442, 0x43e, 0x410, 0x43c, 0x43c, 0x43e, 0x43d, 0x410, 0x448, 0x445, 0x43e, 0x431,
0x43e, 0x434, 0x411, 0x430, 0x433, 0x434, 0x43e, 0x434, 0x411, 0x430, 0x4b3, 0x440,
0x430, 0x439, 0x43d, 0x411, 0x43e, 0x43a, 0x443, 0x414, 0x438, 0x43c, 0x438, 0x448,
0x49b, 0x492, 0x430, 0x437, 0x437, 0x430, 0x4b6, 0x430, 0x43a, 0x430, 0x440, 0x442,
0x430, 0x4b6, 0x430, 0x44f, 0x43f, 0x443, 0x440, 0x430, 0x41a, 0x43e, 0x431, 0x443,
0x43b, 0x41a, 0x443, 0x432, 0x430, 0x439, 0x442, 0x41d, 0x43e, 0x432, 0x43e, 0x43a,
0x443, 0x437, 0x43d, 0x435, 0x442, 0x441, 0x43a, 0x49a, 0x430, 0x442, 0x430, 0x440,
0x49a, 0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x420, 0x438, 0x451,
0x437, 0x422, 0x43e, 0x448, 0x43a, 0x435, 0x43d, 0x442, 0x422, 0x435, 0x4b3, 0x440,
0x43e, 0x43d, 0x422, 0x438, 0x43c, 0x444, 0x443, 0x412, 0x44c, 0x435, 0x43d, 0x442,
0x44f, 0x43d, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20,
0x410, 0x437, 0x43e, 0x440, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x438, 0x44f, 0x4b6,
0x43e, 0x440, 0x4b7, 0x438, 0x44f, 0x438, 0x20, 0x4b6, 0x430, 0x43d, 0x443, 0x431,
0x4e3, 0x42d, 0x443, 0x43a, 0x43b, 0x430, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430,
0x438, 0x20, 0x41c, 0x430, 0x43d, 0x4b6, 0x435, 0x440, 0x441, 0x438, 0x41f, 0x43e,
0x434, 0x433, 0x43e, 0x440, 0x438, 0x442, 0x441, 0x430, 0x421, 0x442, 0x43e, 0x43a,
0x433, 0x43e, 0x43b, 0x43c, 0x421, 0x44e, 0x440, 0x438, 0x445, 0x41a, 0x440, 0x438,
0x441, 0x442, 0x43c, 0x430, 0x441, 0x41a, 0x435, 0x440, 0x433, 0x443, 0x43b, 0x435,
0x43d, 0x411, 0x443, 0x433, 0x435, 0x43d, 0x432, 0x438, 0x43b, 0x43b, 0x424, 0x438,
0x4b7, 0x438, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x440, 0x41c, 0x438, 0x434, 0x432,
0x435, 0x439, 0x412, 0x435, 0x439, 0x43a, 0xb85, 0xbaa, 0xbbf, 0xb9f, 0xbcd, 0xb9c,
0xbbe, 0xba9, 0xbcd, 0xb85, 0xb95, 0xbcd, 0xbb0, 0xbbe, 0xb85, 0xb9f, 0xbbf, 0xbb8,
0xbcd, 0x20, 0xb85, 0xbaa, 0xbbe, 0xbaa, 0xbbe, 0xb85, 0xbb2, 0xbcd, 0xb9c, 0xbbf,
0xbaf, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb85, 0xbb8, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbbe,
0xbaa, 0xbae, 0xbbe, 0xb95, 0xbcb, 0xbaa, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbc1, 0xbaf,
0xbc0, 0xbaa, 0xb9e, 0xbcd, 0xb9a, 0xbc1, 0xbb2, 0xbcd, 0xbaa, 0xbbf, 0xbb8, 0xbbe,
0xbb5, 0xbcd, 0xbaa, 0xbbf, 0xbb3, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbc8, 0xbaf, 0xbb0,
0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbe, 0xbb8, 0xbbe, 0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2,
0xbbf, 0xbaa, 0xbc1, 0xb9c, 0xbc1, 0xbae, 0xbcd, 0xbaa, 0xbc1, 0xbb0, 0xbbe, 0xb95,
0xbc6, 0xbaf, 0xbcd, 0xbb0, 0xbcb, 0xb95, 0xbbe, 0xbb8, 0xbbe, 0xbaa, 0xbbf, 0xbb3,
0xbbe, 0xba9, 0xbcd, 0xb95, 0xbbe, 0xb9a, 0xbbf, 0xbaf, 0xbc2, 0xb9f, 0xbcd, 0xb9f,
0xbbe, 0xb95, 0xbcb, 0xba9, 0xb95, 0xbcd, 0xbb0, 0xbc7, 0xb9f, 0xbbe, 0xb95, 0xbb0,
0xbcd, 0xba4, 0xbbe, 0xbb0, 0xbcd, 0x20, 0xb8e, 0xbb8, 0xbcd, 0x20, 0xb9a, 0xbb2,
0xbbe, 0xbae, 0xbcd, 0xb9c, 0xbbf, 0xbaa, 0xbcc, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0xba4,
0xbb5, 0xbc1, 0xbb2, 0xbbe, 0xb8e, 0xbb2, 0xbcd, 0x20, 0xb86, 0xbaf, 0xbc1, 0xba9,
0xbcd, 0xb83, 0xbaa, 0xbcd, 0xbb0, 0xbc0, 0xb9f, 0xbb5, 0xbc1, 0xba9, 0xbcd, 0xb95,
0xbaa, 0xbcb, 0xbb0, 0xbcb, 0xba9, 0xbcd, 0xbb9, 0xbb0, 0xbbe, 0xbb0, 0xbc7, 0xb9c,
0xbcb, 0xbb9, 0xba9, 0xbcd, 0xba9, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xb95,
0xbcd, 0xb9c, 0xbc1, 0xbaa, 0xbbe, 0xb95, 0xbae, 0xbcd, 0xbaa, 0xbbe, 0xbb2, 0xbbe,
0xb95, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcb, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xb95, 0xbb2,
0xbbf, 0xb95, 0xbbf, 0xba9, 0xbcd, 0xbb7, 0xb9a, 0xbbe, 0xbb2, 0xbbe, 0xb95, 0xbcb,
0xbb8, 0xbcd, 0xbb2, 0xbbf, 0xbaa, 0xbcd, 0xbb0, 0xbc7, 0xbb5, 0xbbf, 0xbb2, 0xbcd,
0xbb2, 0xbc7, 0xbb2, 0xbcb, 0xbae, 0xbcd, 0xbb2, 0xbc1, 0xbb5, 0xbbe, 0xba9, 0xbcd,
0xb9f, 0xbbe, 0xbb2, 0xbc1, 0xbaa, 0xbc1, 0xbae, 0xbcd, 0xbaa, 0xbbe, 0xbb7, 0xbbf,
0xbb2, 0xbc1, 0xb9a, 0xbbe, 0xb95, 0xbbe, 0xbae, 0xbbe, 0xbb2, 0xbaa, 0xbcb, 0xbae,
0xbaa, 0xbc1, 0xb9f, 0xbcb, 0xbae, 0xb9a, 0xbc7, 0xbb0, 0xbc1, 0xbaa, 0xbaa, 0xbbe,
0xba9, 0xbcd, 0xbae, 0xbca, 0xb95, 0xbbe, 0xba4, 0xbbf, 0xbb7, 0xbc1, 0xbae, 0xbbe,
0xba9, 0xbcd, 0xbb0, 0xbcb, 0xbb5, 0xbbf, 0xbaf, 0xbbe, 0xba8, 0xbc8, 0xbb0, 0xbcb,
0xbaa, 0xbbf, 0xb9c, 0xbae, 0xbc7, 0xba9, 0xbbe, 0xba8, 0xbbf, 0xbaf, 0xbbe, 0xbae,
0xbc7, 0xba8, 0xbcb, 0xbb5, 0xbbe, 0xb95, 0xbcd, 0xb9a, 0xbcb, 0xb9f, 0xbcd, 0xb85,
0xbb5, 0xbc1, 0xb95, 0xb9f, 0xbb5, 0xbcd, 0xb95, 0xbc1, 0xbaa, 0xbcb, 0xbb0, 0xbcd,
0xb9f, 0xbcd, 0xb9f, 0xbcb, 0x2d, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbbe, 0xbb5,
0xbcb, 0x20, 0xb9f, 0xbcb, 0xbae, 0xbc7, 0xba4, 0xbcd, 0xbb0, 0xbbf, 0xbaa, 0xbcb,
0xbb2, 0xbbf, 0xb9f, 0xbc1, 0xba9, 0xbbf, 0xbb8, 0xbcd, 0xbb5, 0xbc8, 0xba3, 0xbcd,
0xb9f, 0xbcd, 0xbb9, 0xbcb, 0xb95, 0xbcd, 0xb85, 0xb9f, 0xb95, 0xbcd, 0xb85, 0xb99,
0xbcd, 0xb95, 0xbcb, 0xbb0, 0xbc7, 0xb9c, 0xbcd, 0xb85, 0xb99, 0xbcd, 0xb95, 0xbc1,
0xbaf, 0xbc1, 0xbb2, 0xbbe, 0xb86, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbc1, 0xbb5,
0xbbe, 0xb85, 0xbb0, 0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbaf, 0xbcd, 0xba9, 0xbbe, 0xbaa,
0xbcd, 0xbaf, 0xbc2, 0xba9, 0xbcb, 0xbb8, 0xbcd, 0x20, 0xb8f, 0xbb0, 0xbcd, 0xbb8,
0xbcd, 0xb95, 0xb9f, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbbe, 0xb95, 0xbbe, 0xbb0,
0xbcd, 0xb9f, 0xbcb, 0xbaa, 0xbbe, 0xb9c, 0xbc2, 0xb9c, 0xbc1, 0xbaf, 0xbcd, 0xbb2,
0xbbe, 0x20, 0xbb0, 0xbbf, 0xbaf, 0xbcb, 0xb9c, 0xbbe, 0xbae, 0xbc6, 0xba9, 0xbcd,
0xb9f, 0xbcb, 0xbb8, 0xbbe, 0xbb0, 0xbbf, 0xbaf, 0xbcb, 0x20, 0xb95, 0xbc7, 0xbb2,
0xbc6, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb9a, 0xbbe, 0xbb2, 0xbcd, 0xb9f, 0xbbe, 0xb9a,
0xbbe, 0xba9, 0xbcd, 0x20, 0xb9c, 0xbc1, 0xbb5, 0xbbe, 0xba9, 0xbcd, 0xb9a, 0xbbe,
0xba9, 0xbcd, 0x20, 0xbb2, 0xbc2, 0xbaf, 0xbbf, 0xbb8, 0xbcd, 0xb9f, 0xbc1, 0xb95,
0xbc1, 0xbae, 0xba9, 0xbcd, 0xb89, 0xbb7, 0xbc1, 0xbb5, 0xbbe, 0xbaf, 0xbbe, 0xb85,
0xbb0, 0xbc2, 0xbaa, 0xbbe, 0xb85, 0xbb8, 0xba9, 0xbcd, 0xb9a, 0xbbf, 0xbaf, 0xba9,
0xbcd, 0xb85, 0xb9f, 0xbbf, 0xb95, 0xbcb, 0xb95, 0xba9, 0xbcd, 0xbaa, 0xbbe, 0xbb9,
0xbbf, 0xbaf, 0xbbe, 0xbaa, 0xbb9, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xbaa, 0xba8, 0xbcd,
0xba4, 0xbc7, 0xbb0, 0xbbe, 0xbb8, 0xbcd, 0xbaa, 0xbbe, 0xbb0, 0xbcd, 0xbaa, 0xb9f,
0xbbe, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbb2, 0xbc6, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb2,
0xbbf, 0xbb8, 0xbcd, 0xbaa, 0xbcd, 0xbb2, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbcd, 0x2d,
0xb9a, 0xbaa, 0xbcd, 0xbb2, 0xbbe, 0xba9, 0xbcd, 0xbaa, 0xbcb, 0xbb5, 0xbbe, 0x20,
0xbb5, 0xbbf, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xbaa, 0xbcb, 0xb95, 0xbcb, 0xb9f, 0xbbe,
0xbaa, 0xbcb, 0xbaf, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbc7, 0xbae, 0xbcd, 0xbaa, 0xbbf,
0xbb0, 0xbbf, 0xb9f, 0xbcd, 0xb9c, 0xbcd, 0x20, 0xbb5, 0xbb3, 0xbc8, 0xb95, 0xbc1,
0xb9f, 0xbbe, 0xb95, 0xbc7, 0xbae, 0xbcd, 0xbaa, 0xbcb, 0x20, 0xb95, 0xbbf, 0xbb0,
0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbc7, 0xb95, 0xba9, 0xbcd, 0xb95, 0xbc1, 0xba9, 0xbcd,
0xb95, 0xbb0, 0xb95, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbc6, 0xbaf, 0xbcd, 0xba9, 0xbcd,
0xb95, 0xbc7, 0xbae, 0xba9, 0xbcd, 0xb9a, 0xbbf, 0xb95, 0xbbe, 0xb95, 0xbcb, 0xb9a,
0xbc1, 0xbb5, 0xbbe, 0xbb5, 0xbbe, 0xb9a, 0xbbf, 0xbaf, 0xbc1, 0xb9f, 0xbbe, 0xb9f,
0xbcd, 0x20, 0xbb5, 0xbbe, 0xbb0, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb9f,
0xbbe, 0x20, 0xbb0, 0xbbf, 0xb95, 0xbbe, 0xb95, 0xbcd, 0xbb0, 0xbc6, 0xbb8, 0xbcd,
0xb9f, 0xbbe, 0xba9, 0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbbe, 0xbaa, 0xbc7, 0xb95, 0xbcd,
0xbaf, 0xbc2, 0xbb0, 0xbbe, 0xb95, 0xbcb, 0xb9f, 0xbc6, 0xba9, 0xbcd, 0xbae, 0xbbe,
0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbb7, 0xbbe, 0xbb5, 0xbcd, 0xba9, 0xbcd, 0xb9f, 0xbbe,
0xbb5, 0xbcd, 0xb9a, 0xba9, 0xbcd, 0xb9f, 0xbbe, 0xbb5, 0xbcd, 0xb9a, 0xba9, 0xbcd,
0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xb95, 0xbcd, 0xb9f, 0xbc6, 0xba9, 0xbcd, 0xbb5,
0xbb0, 0xbcd, 0xb9f, 0xbc6, 0xb9f, 0xbcd, 0xbb0, 0xbbe, 0xbaf, 0xbcd, 0xb9f, 0xbcd,
0xb9f, 0xbca, 0xbae, 0xbbf, 0xba9, 0xbbf, 0xb95, 0xbbe, 0xb8e, 0xb9f, 0xbcd, 0xbae,
0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbbe, 0xba9, 0xbcd, 0xb88, 0xbb0, 0xbc1, 0xba9, 0xbc6,
0xbaa, 0xbc7, 0xb8e, 0xbb2, 0xbcd, 0x20, 0xb9a, 0xbbe, 0xbb2, 0xbcd, 0xbb5, 0xb9f,
0xbcb, 0xbb0, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xba8,
0xbc6, 0xbb2, 0xbcd, 0xb9a, 0xba9, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f,
0xbbe, 0xbb2, 0xbc6, 0xb9a, 0xbbe, 0xb95, 0xbbf, 0xbb2, 0xbc7, 0xbb8, 0xbcd, 0x20,
0xbb5, 0xbb3, 0xbc8, 0xb95, 0xbc1, 0xb9f, 0xbbe, 0xb95, 0xbc2, 0xbb8, 0xbcd, 0x20,
0xbaa, 0xbc7, 0xb95, 0xbbf, 0xbb0, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xb9f,
0xbb0, 0xbcd, 0xb95, 0xbcd, 0xb95, 0xbbf, 0xbb0, 0xba9, 0xb9f, 0xbbe, 0xb95, 0xbb5,
0xbc1, 0xb9f, 0xbc7, 0xbb2, 0xbc2, 0xbaa, 0xbcd, 0xb95, 0xbb5, 0xbc1, 0xba4, 0xbae,
0xbbe, 0xbb2, 0xbbe, 0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbaf, 0xb95, 0xbbf, 0xbb2, 0xbcd,
0xb95, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbb9, 0xbb2, 0xbbf, 0xb83, 0xbaa, 0xbc7, 0xb95,
0xbcd, 0xbb8, 0xbcd, 0xbb9, 0xbb5, 0xbbe, 0xba9, 0xbbe, 0xbb9, 0xbc6, 0xbb0, 0xbcd,
0xbae, 0xbcb, 0xb9a, 0xbbf, 0xbb2, 0xbcb, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf,
0xbbe, 0xba9, 0xbbe, 0xbaa, 0xbca, 0xbb2, 0xbbf, 0xbb8, 0xbcd, 0xba8, 0xbbe, 0xb95,
0xbcd, 0xbb8, 0xbcd, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe,
0xba9, 0xbbe, 0xbae, 0xbb0, 0xbc6, 0xba9, 0xbcd, 0xb95, 0xbcb, 0x2c, 0x20, 0xb87,
0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbaa, 0xbc0, 0xb9f, 0xbcd,
0xb9f, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0x2c,
0x20, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xb9f, 0xbc6,
0xbb2, 0xbcd, 0x20, 0xb9a, 0xbbf, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0x2c, 0x20, 0xb87,
0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbb5, 0xbc7, 0xbb5, 0xbc7,
0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbb5,
0xbbf, 0xba9, 0xbcd, 0xb9a, 0xbc6, 0xba9, 0xbc7, 0xbb8, 0xbcd, 0x2c, 0x20, 0xb87,
0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbb5, 0xbbf, 0xba9, 0xbbe,
0xbae, 0xbc7, 0xb95, 0xbcd, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf,
0xbbe, 0xba9, 0xbbe, 0xb87, 0xba9, 0xbc1, 0xbb5, 0xbbf, 0xb95, 0xbcd, 0xb87, 0xb95,
0xbbe, 0xbb2, 0xbc2, 0xbaf, 0xbbf, 0xba4, 0xbcd, 0xb9c, 0xbae, 0xbc8, 0xb95, 0xbcd,
0xb95, 0xbbe, 0xb9c, 0xbc1, 0xba9, 0xbbf, 0xbaf, 0xbc2, 0xbb2, 0xbc2, 0xbaf, 0xbbf,
0xbb8, 0xbcd, 0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbc7, 0xbae, 0xbbe, 0xba9, 0xbcd,
0xb9f, 0xbbf, 0xb9a, 0xbc6, 0xbb2, 0xbcd, 0xbb2, 0xbcb, 0x2c, 0x20, 0xb95, 0xbc6,
0xba9, 0xbcd, 0xb9f, 0xb95, 0xbbf, 0xb95, 0xbbf, 0xbb0, 0xbc6, 0xbb2, 0xba9, 0xbcd,
0xb9f, 0xbbf, 0xb9c, 0xbcd, 0xbb2, 0xbbe, 0x20, 0xbaa, 0xbbe, 0xbb8, 0xbcd, 0xbb2,
0xbbf, 0xbae, 0xbbe, 0xbb2, 0xbbe, 0xbb8, 0xbcd, 0x20, 0xb8f, 0xb9e, 0xbcd, 0xb9a,
0xbb2, 0xbcd, 0xbb8, 0xbcd, 0xbb2, 0xbcb, 0xbaf, 0xbb0, 0xbcd, 0x20, 0xbaa, 0xbbf,
0xbb0, 0xbbf, 0xba9, 0xbcd, 0xbb8, 0xbb8, 0xbcd, 0x20, 0xb95, 0xbc1, 0xbb5, 0xbbe,
0xbb0, 0xbcd, 0xb9f, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xbae, 0xbc7, 0xb9a, 0xbbf, 0xbaf,
0xbcb, 0xbae, 0xbbe, 0xba9, 0xbbe, 0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbae, 0xba9, 0xbbe,
0xbb8, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbbf, 0xb95, 0xbbe, 0xb9f, 0xbcd, 0xbae, 0xbbe,
0xbb0, 0xbcd, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbf, 0xb95, 0xbcd, 0xbae, 0xb9f,
0xbae, 0xbcb, 0xbb0, 0xbbe, 0xbb8, 0xbcd, 0xbae, 0xbb8, 0xb9f, 0xbcd, 0xbb2, 0xba9,
0xbcd, 0xbae, 0xbc6, 0xba9, 0xbcb, 0xbae, 0xbbf, 0xba9, 0xbbf, 0xbae, 0xbc6, 0xbb0,
0xbbf, 0xb9f, 0xbbe, 0xbae, 0xbc6, 0xb9f, 0xbcd, 0xbb2, 0xb95, 0xb9f, 0xbcd, 0xbb2,
0xbbe, 0xbae, 0xbc6, 0xb95, 0xbcd, 0xbb8, 0xbbf, 0xb95, 0xbcb, 0x20, 0xba8, 0xb95,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbf, 0xb95, 0xbcd, 0xbaf, 0xbc2, 0xbb2, 0xba9, 0xbcd,
0xbae, 0xbbe, 0xb99, 0xbcd, 0xb9f, 0xbbe, 0xba9, 0xbcd, 0xbae, 0xbbe, 0xba9, 0xbcd,
0xb9f, 0xbc6, 0xbb0, 0xbcd, 0xbb0, 0xbc7, 0xbae, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbc7,
0xbb5, 0xbc0, 0xb9f, 0xbbf, 0xbaf, 0xbcb, 0xbae, 0xbbe, 0xba9, 0xbcd, 0xb9a, 0xbc6,
0xbb0, 0xbc7, 0xb9f, 0xbcd, 0xba8, 0xb9a, 0xbb5, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2,
0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xba8, 0xbcb, 0xbae, 0xbcd, 0xba8, 0xbcb,
0xbb0, 0xba9, 0xbcd, 0xbb9, 0xbbe, 0xbaa, 0xbc6, 0xbb5, 0xbc1, 0xbb2, 0xbbe, 0x2c,
0x20, 0xbb5, 0xb9f, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb9f, 0xb95, 0xbcb, 0xb9f,
0xbcd, 0xb9f, 0xbbe, 0xb9a, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0x2c, 0x20,
0xbb5, 0xb9f, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb9f, 0xb95, 0xbcb, 0xb9f, 0xbbe,
0xba8, 0xbbf, 0xbaf, 0xbc2, 0x20, 0xb9a, 0xbb2, 0xbc7, 0xbae, 0xbcd, 0x2c, 0x20,
0xbb5, 0xb9f, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb9f, 0xb95, 0xbcb, 0xb9f, 0xbbe,
0xba8, 0xbc2, 0xb95, 0xbcd, 0xb92, 0xb9c, 0xbbf, 0xba9, 0xb95, 0xbbe, 0xbaa, 0xba9,
0xbbe, 0xbae, 0xbbe, 0xbaa, 0xbb0, 0xbae, 0xbb0, 0xbbf, 0xbaa, 0xbcb, 0xb83, 0xbaa,
0xbcb, 0xba9, 0xbbf, 0xb95, 0xbcd, 0xbb8, 0xbcd, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f,
0xbcd, 0x2d, 0xb85, 0xbb5, 0xbcd, 0x2d, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xba9, 0xbcd,
0xbb8, 0xbcd, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xb86, 0xb83, 0xbaa,
0xbcd, 0x20, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xbaa, 0xbcb,
0xbb0, 0xbcd, 0xb9f, 0xbcd, 0xb9f, 0xbcb, 0x20, 0xbb5, 0xbc6, 0xbb2, 0xbcd, 0xbb9,
0xbcb, 0xbaa, 0xbbf, 0xbaf, 0xbc2, 0xbb0, 0xbcd, 0xb9f, 0xbcb, 0x20, 0xbb0, 0xbbf,
0xb95, 0xbcb, 0xbaa, 0xbc1, 0xba9, 0xbcd, 0xb9f, 0xbbe, 0x20, 0xb85, 0xbb0, 0xbc0,
0xba9, 0xbbe, 0xbb8, 0xbcd, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0xb95, 0xbbf, 0xba9, 0xbcd,
0x20, 0xb87, 0xba9, 0xbcd, 0xbb2, 0xbc6, 0xb9f, 0xbcd, 0xbb0, 0xbc6, 0xbb8, 0xbbf,
0xb83, 0xbaa, 0xbbf, 0xbb0, 0xbc6, 0xb9c, 0xbbf, 0xba9, 0xbbe, 0xbb0, 0xbc6, 0xb9a,
0xbb2, 0xbc2, 0xb9f, 0xbcd, 0xbb0, 0xbbf, 0xbaf, 0xbcb, 0x20, 0xbaa, 0xbbf, 0xbb0,
0xbbe, 0xba9, 0xbcd, 0xb95, 0xbcb, 0xb9a, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbb0, 0xbc6,
0xbae, 0xbcd, 0xb9a, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xb95, 0xbcb,
0xb9a, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbcb, 0x20, 0xb9f, 0xbcb, 0xbae, 0xbbf, 0xb99,
0xbcd, 0xb95, 0xbcb, 0xb9a, 0xbbe, 0xbb5, 0xbcb, 0x20, 0xbaa, 0xbb5, 0xbc1, 0xbb2,
0xbcb, 0xb87, 0xb9f, 0xbcb, 0xb95, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcb, 0xbb0, 0xbcd,
0xbae, 0xbbf, 0xb9f, 0xbcd, 0xb9a, 0xbbf, 0xb9f, 0xbcd, 0xb95, 0xbbe, 0xb9a, 0xbc6,
0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbaa, 0xbbe, 0xbb0, 0xbcd, 0xba4,
0xbc7, 0xbb2, 0xbc6, 0xbae, 0xbbf, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f,
0xbcd, 0x20, 0xb9c, 0xbbe, 0xba9, 0xbcd, 0xbb8, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf,
0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xb95, 0xbc0, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb9a,
0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb2, 0xbc2, 0xb9a, 0xbbf,
0xbaf, 0xbbe, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xba4,
0xbbe, 0xbae, 0xbb8, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd,
0x20, 0xbb5, 0xbbf, 0xba9, 0xbcd, 0xb9a, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0xbb8,
0xbcd, 0xbb5, 0xbbf, 0xb83, 0xbaa, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xb95, 0xbb0, 0xba3,
0xbcd, 0xb9f, 0xbcd, 0xba4, 0xbc6, 0xb95, 0xbc1, 0xb9a, 0xbbf, 0xb95, 0xbb2, 0xbcd,
0xbaa, 0xbbe, 0xba4, 0xbc1, 0xbb2, 0xbc7, 0xb9f, 0xbbf, 0xb9c, 0xbc1, 0xbb5, 0xbbe,
0xba9, 0xbbe, 0xb9f, 0xbca, 0xbb0, 0xbca, 0xba9, 0xbcd, 0xb9f, 0xbcb, 0xb9f, 0xbcb,
0xbb0, 0xbcd, 0xb9f, 0xbcb, 0xbb2, 0xbbe, 0xbb5, 0xbbe, 0xba9, 0xbcd, 0xb95, 0xbc2,
0xbb5, 0xbb0, 0xbcd, 0xbb5, 0xbca, 0xbaf, 0xbbf, 0xb9f, 0xbcd, 0xbb9, 0xbbe, 0xbb0,
0xbcd, 0xbb8, 0xbcd, 0xbb5, 0xbbf, 0xba9, 0xbcd, 0xba9, 0xbbf, 0xbaa, 0xbc6, 0xb95,
0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xb9f, 0xbcd, 0xb95, 0xbc7, 0xbb8, 0xbbf, 0xb9f,
0xbc7, 0xbb5, 0xbbf, 0xbb8, 0xbcd, 0xb9f, 0xbc1, 0xbae, 0xbcb, 0xba3, 0xbcd, 0xb9f,
0xbcd, 0x2d, 0xb9f, 0xbbf, 0x20, 0xb89, 0xbb0, 0xbcd, 0xbb5, 0xbbf, 0xbb2, 0xbcd,
0xbb2, 0xbc7, 0xbae, 0xbbe, 0xb95, 0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xbb0, 0xbbf,
0xbae, 0xbbe, 0xb9a, 0xba9, 0xbcd, 0xbae, 0xbc6, 0xb95, 0xbcd, 0xbae, 0xbb0, 0xbcd,
0xb9f, 0xbcb, 0xbaa, 0xbbe, 0xbb2, 0xbcd, 0xbae, 0xbb0, 0xbcd, 0xbb0, 0xbcb, 0xba4,
0xbc7, 0xbb0, 0xbbe, 0xbb8, 0xbcd, 0xbaf, 0xbcb, 0xbb5, 0xbbe, 0xb9f, 0xbcd, 0xbb0,
0xbcb, 0xbb2, 0xbcd, 0xbb5, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbcb, 0xb95, 0xbcd, 0xbb2,
0xbbe, 0xb99, 0xbcd, 0xb87, 0xbaf, 0xbb0, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xba9, 0xbcd,
0xb8f, 0xb9f, 0xba9, 0xbcd, 0xb85, 0xbb2, 0xbcd, 0xbae, 0xbbe, 0xba4, 0xbbf, 0xb85,
0xbae, 0xbcd, 0xbae, 0xbbe, 0xba9, 0xbcd, 0xb85, 0xba9, 0xb9f, 0xbc0, 0xbb0, 0xbcd,
0xb85, 0xb95, 0xbcd, 0xba4, 0xbb5, 0xbcd, 0xb85, 0xb95, 0xbcd, 0xb9f, 0xbcb, 0xbaa,
0xbcd, 0xb85, 0xbb7, 0xbcd, 0xb95, 0xbbe, 0xbaa, 0xbbe, 0xba4, 0xbcd, 0xb85, 0xb9f,
0xbbf, 0xbb0, 0xbbe, 0xbaa, 0xbbe, 0xb95, 0xbcd, 0xba4, 0xbbe, 0xba4, 0xbcd, 0xbaa,
0xbb9, 0xbcd, 0xbb0, 0xbc8, 0xba9, 0xbcd, 0xbaa, 0xbbe, 0xb95, 0xbcd, 0xb95, 0xbc2,
0xbaa, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb95, 0xbcd, 0xbaa, 0xbbe, 0xbb0, 0xbcd,
0xba9, 0xbbe, 0xbb2, 0xbcd, 0xbaa, 0xbc6, 0xbaf, 0xbcd, 0xbb0, 0xbc2, 0xb9f, 0xbcd,
0xbaa, 0xbbf, 0xbb7, 0xbcd, 0xb95, 0xbc6, 0xb95, 0xbcd, 0xbaa, 0xbc1, 0xbb0, 0xbc1,
0xba9, 0xbc7, 0xb9a, 0xbbf, 0xb9f, 0xbbe, 0xb95, 0xbca, 0xbb4, 0xbc1, 0xbae, 0xbcd,
0xbaa, 0xbc1, 0xb9f, 0xbae, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbb8, 0xbcd, 0xb9f, 0xbbe,
0xb95, 0xbcd, 0xb95, 0xbbe, 0xb9f, 0xbbf, 0xbb2, 0xbbf, 0xba4, 0xbc1, 0xbaa, 0xbbe,
0xbaf, 0xbcd, 0xba4, 0xbc1, 0xbb7, 0xba9, 0xbcd, 0xbaa, 0xbc7, 0xb83, 0xbaa, 0xbbe,
0xbae, 0xb95, 0xbc1, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xbb9, 0xbc6, 0xbaa, 0xbcd, 0xbb0,
0xbbe, 0xba9, 0xbcd, 0xbb9, 0xbcb, 0x20, 0xb9a, 0xbbf, 0x20, 0xbae, 0xbbf, 0xba9,
0xbcd, 0x20, 0xb9a, 0xbbf, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0xbb9, 0xbbe, 0xb99, 0xbcd,
0xb95, 0xbbe, 0xb99, 0xbcd, 0xbb9, 0xbcb, 0xbb5, 0xbcd, 0xba4, 0xbcd, 0xb87, 0xbb0,
0xbcd, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xb9c, 0xb95, 0xbbe,
0xbb0, 0xbcd, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xb9c, 0xbc6, 0xbaf, 0xbaa, 0xbc2, 0xbb0,
0xbbe, 0xb9c, 0xbc6, 0xbb0, 0xbc1, 0xb9a, 0xbb2, 0xbc7, 0xbae, 0xbcd, 0xb95, 0xbbe,
0xbaa, 0xbc2, 0xbb2, 0xbcd, 0xb95, 0xbbe, 0xbae, 0xbcd, 0xb9a, 0xb9f, 0xbcd, 0xb95,
0xbbe, 0xb95, 0xbb0, 0xbbe, 0xb9a, 0xbcd, 0xb9a, 0xbbf, 0xb95, 0xbbe, 0xba4, 0xbcd,
0xbae, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbc1, 0xb95, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbbf,
0xb95, 0xbbe, 0xb95, 0xbca, 0xbb2, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xb95,
0xbbf, 0xbb0, 0xbbe, 0xbb8, 0xbcd, 0xba9, 0xbcb, 0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xb95,
0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0xbb2, 0xbbe, 0xbb2, 0xbae, 0xbcd, 0xbaa, 0xbc2,
0xbb0, 0xbcd, 0xb95, 0xbc1, 0xb9a, 0xbbf, 0xb99, 0xbcd, 0xb95, 0xbc1, 0xbb5, 0xbc8,
0xba4, 0xbcd, 0xbae, 0xb95, 0xbbe, 0xbb5, 0xbc1, 0xbae, 0xb95, 0xba4, 0xba9, 0xbcd,
0xbae, 0xb95, 0xbcd, 0xb95, 0xbb8, 0xbb0, 0xbcd, 0xbae, 0xba3, 0xbbf, 0xbb2, 0xbbe,
0xbae, 0xbb8, 0xbcd, 0xb95, 0xb9f, 0xbcd, 0xba8, 0xbbf, 0xb95, 0xbcb, 0xb9a, 0xbbf,
0xbaf, 0xbbe, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb95, 0xbc1, 0xbb8, 0xbcd, 0xba8, 0xbc6,
0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbc0,
0xbaa, 0xbbf, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xb93, 0xbae, 0xbcd, 0xbb8,
0xbcd, 0xb95, 0xbcd, 0xb93, 0xbb0, 0xbb2, 0xbcd, 0xb83, 0xbaa, 0xbcd, 0xba9, 0xbcb,
0xbae, 0xbcd, 0x20, 0xbaa, 0xbc6, 0xba9, 0xbcd, 0xbaa, 0xbcb, 0xba9, 0xbcd, 0xb9f,
0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xb95, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0xb99,
0xbcd, 0xbaf, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbb0, 0xbcd,
0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xba9, 0xbc7, 0xb95, 0xbbf, 0xbb8, 0xbbf,
0xbb2, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbbe, 0xbb0, 0xbbf, 0xbaf, 0xbbe, 0xba4, 0xbcd,
0xb9a, 0xb95, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0xb9a, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xb95,
0xba3, 0xbcd, 0xb9f, 0xbcd, 0xb9a, 0xbbf, 0xbaf, 0xbcb, 0xbb2, 0xbcd, 0xbb7, 0xbbe,
0xb99, 0xbcd, 0xb95, 0xbbe, 0xbaf, 0xbcd, 0xb9a, 0xbbf, 0xb99, 0xbcd, 0xb95, 0xbaa,
0xbcd, 0xbaa, 0xbc2, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xbb0, 0xbc6, 0xb9f, 0xbcd, 0xba8,
0xbbf, 0xb95, 0xbcb, 0xbb2, 0xbbf, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xba4,
0xbbe, 0xbaf, 0xbcd, 0xbaa, 0xbc7, 0xba4, 0xbbe, 0xbb7, 0xbcd, 0xb95, 0xba3, 0xbcd,
0xb9f, 0xbcd, 0xba4, 0xbcd, 0xbaa, 0xbbf, 0xbb2, 0xbbf, 0xb9a, 0xbbf, 0xb9f, 0xbc6,
0xbb9, 0xbcd, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0xba4, 0xbbf, 0xbae, 0xbcd, 0xbaa, 0xbc1,
0xb9f, 0xbcb, 0xb95, 0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbcb, 0xb9f, 0xbbe, 0xbae, 0xbcd,
0xbb8, 0xbcd, 0xb95, 0xbcd, 0xb89, 0xbb2, 0xbbe, 0xba9, 0xbcd, 0xbaa, 0xbbe, 0xb9f,
0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xb89, 0xbb0, 0xbc1, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xb89,
0xbb8, 0xbcd, 0xb9f, 0xbcd, 0x2d, 0xba8, 0xbc7, 0xbb0, 0xbbe, 0xbb5, 0xbbf, 0xbaf,
0xba9, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0xbb5, 0xbbf, 0xbb3, 0xbbe, 0xb9f,
0xbbf, 0xbb5, 0xbca, 0xbb8, 0xbcd, 0xba4, 0xbcb, 0xb95, 0xbcd, 0xbaf, 0xb95, 0xbc1,
0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xbb0, 0xb99, 0xbcd, 0xb95, 0xbc2, 0xba9,
0xbcd, 0xbaf, 0xbc6, 0xb95, 0xbbe, 0xb9f, 0xbbf, 0xbb0, 0xbbf, 0xba9, 0xbcd, 0xbaa,
0xbb0, 0xbcd, 0xb95, 0xbcd, 0xb8f, 0xbb0, 0xbc7, 0xbb5, 0xba9, 0xbcd, 0xb85, 0xb9a,
0xbcb, 0xbb0, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xbae, 0xbc1, 0xb9f, 0xbbe,
0xb95, 0xbc7, 0xba9, 0xbb0, 0xbbf, 0xb95, 0xbc7, 0xbaa, 0xbcd, 0x20, 0xbb5, 0xbc6,
0xbb0, 0xbcd, 0xb9f, 0xbc7, 0xb83, 0xbaa, 0xbb0, 0xbcb, 0xbae, 0xb9f, 0xbbf, 0xbb0,
0xbbe, 0xbb0, 0xbc7, 0xb95, 0xbcd, 0xb9c, 0xbbe, 0xbb5, 0xbbf, 0xb95, 0xbcd, 0xba4,
0xbc6, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb9c, 0xbbe, 0xbb0, 0xbcd, 0xb9c, 0xbbf,
0xbaf, 0xbbe, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb9,
0xbc6, 0xbb2, 0xbc6, 0xba9, 0xbbe, 0xbb8, 0xbcd, 0xb9f, 0xbc7, 0xba9, 0xbcd, 0xbb2,
0xbbf, 0xb85, 0xb9f, 0xbbf, 0xbb2, 0xbc6, 0xbaf, 0xbcd, 0xb9f, 0xbcd, 0xbaa, 0xbbf,
0xbb0, 0xbbf, 0xbb8, 0xbcd, 0xbaa, 0xbc7, 0xba9, 0xbcd, 0xbaa, 0xbc1, 0xbb0, 0xbcb,
0xb95, 0xbcd, 0xb95, 0xba9, 0xbcd, 0x20, 0xbb9, 0xbbf, 0xbb2, 0xbcd, 0xb9f, 0xbbe,
0xbb0, 0xbcd, 0xbb5, 0xbbf, 0xba9, 0xbcd, 0xbaf, 0xbc2, 0xb95, 0xbcd, 0xbb2, 0xbbe,
0xbb9, 0xbcb, 0xbaa, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0xb9f,
0xbc6, 0xbae, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb9,
0xbcb, 0xbb5, 0xbc7, 0xbae, 0xbc6, 0xbb2, 0xbcd, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xba9,
0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xba4, 0xbcd, 0xb9a, 0xbbf, 0xb9f, 0xbcd, 0xba9,
0xbbf, 0xb86, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb9f, 0xbcd, 0xbb0, 0xb9f, 0xbbe, 0xbae,
0xbcd, 0xb85, 0xba3, 0xbcd, 0xb9f, 0xbcb, 0xbb0, 0xbbe, 0xb85, 0xbb8, 0xbcd, 0xb9f,
0xbcd, 0xbb0, 0xbbe, 0xb95, 0xbbe, 0xba9, 0xbcd, 0xb8f, 0xba4, 0xba9, 0xbcd, 0xbb8,
0xbcd, 0xbaa, 0xbc6, 0xbb2, 0xbcd, 0xb95, 0xbbf, 0xbb0, 0xbc7, 0xb9f, 0xbcd, 0xbaa,
0xbc6, 0xbb0, 0xbcd, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xb9f, 0xbbf,
0xbb8, 0xbcd, 0xbb2, 0xbbe, 0xbb5, 0xbbe, 0xbaa, 0xbc1, 0xbb0, 0xbc2, 0xb9a, 0xbb2,
0xbcd, 0xbb8, 0xbcd, 0xbaa, 0xbc1, 0xb9a, 0xbbe, 0xbb0, 0xbc6, 0xbb8, 0xbcd, 0xb9f,
0xbcd, 0xbaa, 0xbc1, 0xb9f, 0xbbe, 0xbaa, 0xbc6, 0xbb8, 0xbcd, 0xb9f, 0xbcd, 0xbaa,
0xb9a, 0xbbf, 0xb99, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9a, 0xbbf, 0xb9a, 0xbbf,
0xba9, 0xbb5, 0xbcd, 0xb95, 0xbcb, 0xbaa, 0xba9, 0xbcd, 0xbb9, 0xbc7, 0xb95, 0xba9,
0xbcd, 0xb9f, 0xbaa, 0xbcd, 0xbb3, 0xbbf, 0xba9, 0xbcd, 0xb9c, 0xbbf, 0xbaa, 0xbcd,
0xbb0, 0xbbe, 0xbb2, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xb95, 0xbb0, 0xbcd, 0xba9, 0xbb8,
0xbc7, 0xbb9, 0xbc6, 0xbb2, 0xbcd, 0xb9a, 0xbbf, 0xb99, 0xbcd, 0xb95, 0xbbf, 0xb90,
0xbb2, 0xbcd, 0x20, 0xb86, 0xb83, 0xbaa, 0xbcd, 0x20, 0xbae, 0xbc7, 0xba9, 0xbcd,
0xb87, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0xbaa, 0xbc1, 0xbb2, 0xbcd, 0xb9c,
0xbc6, 0xbb0, 0xbcd, 0xb9a, 0xbbf, 0xb95, 0xbb2, 0xbbf, 0xba9, 0xbbf, 0xba9, 0xbcd,
0xb95, 0xbbf, 0xbb0, 0xbbe, 0xb9f, 0xbcd, 0xb95, 0xbbf, 0xbb0, 0xbcb, 0xbb5, 0xbcd,
0xb95, 0xbc0, 0xbb5, 0xbcd, 0xbb2, 0xbbf, 0xbb8, 0xbcd, 0xbaa, 0xba9, 0xbcd, 0xb9c,
0xbc1, 0xbaa, 0xbcd, 0xbb2, 0xbbe, 0xba9, 0xbbe, 0xbb2, 0xba3, 0xbcd, 0xb9f, 0xba9,
0xbcd, 0xbb2, 0xb95, 0xbcd, 0xb9a, 0xbae, 0xbcd, 0xbaa, 0xbb0, 0xbcd, 0xb95, 0xbcd,
0xbae, 0xbc7, 0xb9f, 0xbcd, 0xbb0, 0xbbf, 0xb9f, 0xbcd, 0xbae, 0xbbe, 0xbb2, 0xbcd,
0xb9f, 0xbbe, 0xbae, 0xbb0, 0xbbf, 0xbb9, 0xbae, 0xbcd, 0xbae, 0xbbf, 0xba9, 0xbcd,
0xbb8, 0xbcd, 0xb95, 0xbcd, 0xbae, 0xbca, 0xba9, 0xbbe, 0xb95, 0xbcd, 0xb95, 0xbcb,
0xbae, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0xb93, 0xbb8, 0xbcd, 0xbb2, 0xbcb, 0xbaa,
0xbbe, 0xbb0, 0xbc0, 0xbb8, 0xbcd, 0xbaa, 0xbcb, 0xb9f, 0xbcd, 0xb95, 0xbcb, 0xbb0,
0xbbf, 0xb95, 0xbbe, 0xbaa, 0xbcd, 0xbb0, 0xbbe, 0xb95, 0xbcd, 0xbb0, 0xbcb, 0xbae,
0xbcd, 0xb9a, 0xbae, 0xbbe, 0xbb0, 0xbbe, 0xb9a, 0xbbe, 0xba9, 0xbcd, 0x20, 0xbae,
0xbb0, 0xbbf, 0xba9, 0xbcb, 0xb9a, 0xbb0, 0xbaf, 0xbc7, 0xbb5, 0xbcb, 0xb9a, 0xbb0,
0xb9f, 0xbcb, 0xbb5, 0xbcd, 0xb9a, 0xbbf, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbc6, 0xbb0,
0xbcb, 0xbaa, 0xbcb, 0xbb2, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0xbaa, 0xbcd, 0xb9c,
0xbc7, 0xb9a, 0xbcb, 0xb83, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0xbb8, 0xbcd, 0xb9f, 0xbbe,
0xb95, 0xbcd, 0xbb9, 0xbcb, 0xbae, 0xbcd, 0xb9f, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0xba4,
0xbbf, 0xbb0, 0xbbe, 0xba9, 0xbc7, 0xb89, 0xbb2, 0xbcd, 0xbaf, 0xbbe, 0xba9, 0xbcb,
0xbb8, 0xbcd, 0xb95, 0xbcd, 0xbb5, 0xba4, 0xbc1, 0xbb8, 0xbcd, 0xbb5, 0xbbe, 0xb9f,
0xbbf, 0xb95, 0xba9, 0xbcd, 0xbb5, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0xba9, 0xbbe, 0xbb5,
0xbbf, 0xbb2, 0xbcd, 0xba9, 0xbbf, 0xbaf, 0xbb8, 0xbcd, 0xbb5, 0xbcb, 0xbb2, 0xbcd,
0xb95, 0xbcb, 0xb95, 0xbbf, 0xbb0, 0xbbe, 0xb9f, 0xbcd, 0xbb5, 0xbbe, 0xbb0, 0xbcd,
0xbb8, 0xbbe, 0xbb8, 0xb95, 0xbcd, 0xbb0, 0xbc6, 0xbaa, 0xbcd, 0xb9c, 0xbc2, 0xbb0,
0xbbf, 0xb9a, 0xbcd, 0xb86, 0xba3, 0xbcd, 0xb9f, 0xba9, 0xba8, 0xbb0, 0xbbf, 0xbb5,
0xbcb, 0xb9a, 0xbbe, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb95, 0xbbf, 0xbb1, 0xbbf, 0xbb8,
0xbcd, 0xba4, 0xbc1, 0xbae, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0xb95, 0xbcb, 0xbb8, 0xbcd,
0xb95, 0xbca, 0xbae, 0xbb0, 0xbcb, 0xb95, 0xbc6, 0xbb0, 0xbcd, 0xbaf, 0xbc2, 0xbb2,
0xbc6, 0xba9, 0xbcd, 0xbae, 0xbbe, 0xbb9, 0xbc7, 0xbae, 0xbbe, 0xbb2, 0xba4, 0xbcd,
0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0xbae, 0xbca, 0xbb0, 0xbbf, 0xbb7,
0xbbf, 0xbaf, 0xbb8, 0xbcd, 0xbae, 0xbaf, 0xbcb, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0xbb0,
0xbc0, 0xbaf, 0xbc2, 0xba9, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0xb85, 0xbaa, 0xbbf, 0xbaf,
0xbbe, 0xb86, 0xb95, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0xbaa, 0xbcb,
0xb95, 0xbc6, 0xbaf, 0xbcd, 0xba9, 0xbcd, 0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbc7,
0xb9a, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbae, 0xbcd, 0xb9a, 0xbc1, 0xb95, 0xbcd, 0xb88,
0xbb8, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xb88, 0xb83, 0xbaa, 0xbc7, 0xb9f, 0xbcd, 0xb83,
0xbaa, 0xb95, 0xbbe, 0xb83, 0xbaa, 0xbcb, 0xb83, 0xbaa, 0xbbf, 0xb9c, 0xbbf, 0xb83,
0xbaa, 0xbc1, 0xba9, 0xbbe, 0xb83, 0xbaa, 0xbc1, 0xb9f, 0xbbf, 0xb95, 0xbb2, 0xbaa,
0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb95, 0xbc7, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbb0,
0xbcd, 0xb95, 0xbcd, 0xbb5, 0xbbe, 0xb9f, 0xbbe, 0xbb2, 0xbcd, 0xb95, 0xbc7, 0xba9,
0xbb2, 0xbcd, 0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbae, 0xbcd, 0xbb9, 0xbcb, 0xba9, 0xbcb,
0xbb2, 0xbc2, 0xbb2, 0xbc2, 0xb8e, 0xba3, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xbaa, 0xbb0,
0xbbf, 0xb95, 0xbbf, 0xbb0, 0xbbf, 0xb9f, 0xbbf, 0xbae, 0xbbe, 0xb9f, 0xbcd, 0xb9f,
0xbbf, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xbb0, 0xbc7, 0xb95, 0xbcd, 0xbb5, 0xbbe, 0xb9c,
0xbbe, 0xbb2, 0xbc0, 0xbaf, 0xba9, 0xbcd, 0xbae, 0xb9c, 0xbc1, 0xbb0, 0xbcb, 0xbae,
0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xb9a, 0xbbe, 0xbb8, 0xbcd, 0xbae,
0xbbf, 0xb9f, 0xbcd, 0xbb5, 0xbc7, 0xba8, 0xbb5, 0xbcd, 0xbb0, 0xbc2, 0xba8, 0xbbf,
0xbaf, 0xbc1, 0xba8, 0xbbe, 0xbb0, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xb95, 0xbcd, 0xba8,
0xbcb, 0xbb5, 0xbc1, 0xbae, 0xbbf, 0xbaf, 0xbbe, 0xbaa, 0xbc7, 0xb95, 0xbcb, 0x20,
0xbaa, 0xbc7, 0xb95, 0xbcb, 0xbaa, 0xbbe, 0xbb2, 0xbb5, 0xbcd, 0xbaa, 0xbbf, 0xb9f,
0xbcd, 0xb95, 0xbc6, 0xbaf, 0xbcd, 0xbb0, 0xbcd, 0xba9, 0xbcd, 0xb83, 0xbaa, 0xbcb,
0xba9, 0xbcd, 0xbaa, 0xbc6, 0xbaf, 0xbcd, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd,
0x20, 0xbae, 0xbcb, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xbaa, 0xbc8, 0xbb0, 0xbb0, 0xbcb,
0xb9f, 0xbcb, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb9a, 0xbc8, 0xbaa, 0xba9, 0xbcd, 0xba4,
0xbb9, 0xbbf, 0xba4, 0xbbf, 0xba4, 0xbb0, 0xbbe, 0xbb5, 0xbbe, 0xb9f, 0xbcb, 0xba9,
0xbcd, 0xb95, 0xb9f, 0xbaa, 0xbcd, 0xbaa, 0xbc1, 0xbb5, 0xbc7, 0xb95, 0xbcd, 0xbb5,
0xbbe, 0xbb2, 0xbbf, 0xbb8, 0xbcd, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61,
0x6c, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6e, 0x6b, 0x72, 0x69, 0x6a, 0x69,
0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20,
0x47, 0x72, 0x61, 0x64, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c,
0x61, 0x6e, 0x67, 0x20, 0x4a, 0x69, 0x63, 0x69, 0x61, 0x6b, 0x4a, 0x69,
0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x61,
0x6e, 0x62, 0x6f, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61,
0x6e, 0x67, 0x20, 0x49, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x6e, 0x61, 0x70,
0x6f, 0x72, 0x69, 0x73, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c,
0x61, 0x6e, 0x67, 0x20, 0x52, 0x6f, 0x73, 0x61, 0x6e, 0x63, 0x69, 0x4a,
0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x4e,
0x69, 0x75, 0x79, 0x75, 0x65, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61,
0x6c, 0x61, 0x6e, 0x67, 0x20, 0x42, 0x6f, 0x6e, 0x68, 0x75, 0x61, 0x6e,
0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20,
0x50, 0x75, 0x72, 0x61, 0x6e, 0x6b, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x53, 0x6e, 0x70, 0x61, 0x75, 0x72,
0x6f, 0x77, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e,
0x67, 0x20, 0x48, 0x6f, 0x6e, 0x6f, 0x72, 0x75, 0x72, 0x75, 0x418, 0x434,
0x438, 0x497, 0x430, 0x43d, 0x411, 0x430, 0x43d, 0x497, 0x443, 0x43b, 0x411, 0x4e9,
0x497, 0x4af, 0x43c, 0x431, 0x443, 0x440, 0x430, 0x41a, 0x430, 0x4bb, 0x438, 0x440,
0x4d9, 0x496, 0x443, 0x431, 0x430, 0x4ba, 0x430, 0x440, 0x442, 0x443, 0x43c, 0x41d,
0x497, 0x430, 0x43c, 0x435, 0x43d, 0x41a, 0x435, 0x43c, 0x431, 0x440, 0x438, 0x434,
0x436, 0x20, 0x411, 0x44d, 0x439, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x20,
0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x413, 0x43b, 0x435,
0x439, 0x441, 0x20, 0x411, 0x44d, 0x439, 0x41a, 0x430, 0x437, 0x20, 0x431, 0x443,
0x445, 0x442, 0x430, 0x441, 0x44b, 0x422, 0x435, 0x43b, 0x43b, 0x44c, 0x2d, 0x421,
0x438, 0x442, 0x438, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430,
0x41b, 0x443, 0x438, 0x441, 0x432, 0x438, 0x43b, 0x44c, 0x422, 0x4af, 0x431, 0x4d9,
0x43d, 0x433, 0x435, 0x20, 0x41a, 0x435, 0x43d, 0x4d9, 0x437, 0x20, 0x43a, 0x432,
0x430, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x41d, 0x43e, 0x43c, 0x435, 0x411, 0x44c,
0x44e, 0x43b, 0x430, 0x2c, 0x20, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20,
0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x4ae, 0x437, 0x4d9, 0x43a, 0x2c, 0x20,
0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442,
0x430, 0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x2c, 0x20,
0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442,
0x430, 0x418, 0x437, 0x433, 0x435, 0x20, 0x412, 0x430, 0x440, 0x444, 0x43e, 0x43b,
0x43e, 0x43c, 0x435, 0x439, 0x418, 0x437, 0x433, 0x435, 0x20, 0x41b, 0x44e, 0x441,
0x438, 0x44f, 0x41b, 0x43e, 0x43d, 0x433, 0x438, 0x435, 0x440, 0x431, 0x438, 0x435,
0x43d, 0x4d8, 0x43c, 0x43c, 0x430, 0x43d, 0x411, 0x4d9, 0x439, 0x440, 0x4af, 0x442,
0x414, 0x4d9, 0x43c, 0x4d9, 0x448, 0x43a, 0x44a, 0x4ba, 0x435, 0x432, 0x440, 0x43e,
0x43d, 0x496, 0x430, 0x43a, 0x430, 0x440, 0x442, 0x430, 0x496, 0x4d9, 0x44f, 0x43f,
0x443, 0x440, 0x430, 0x421, 0x4d9, 0x43c, 0x4d9, 0x440, 0x43a, 0x430, 0x43d, 0x434,
0x422, 0x4d9, 0x4bb, 0x440, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x443,
0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x411, 0x435, 0x440, 0x43c,
0x443, 0x434, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b,
0x41a, 0x430, 0x43d, 0x430, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b,
0x430, 0x440, 0x44b, 0x424, 0x430, 0x440, 0x435, 0x440, 0x20, 0x443, 0x442, 0x440,
0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x41a, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a,
0x20, 0x414, 0x436, 0x43e, 0x440, 0x434, 0x436, 0x438, 0x44f, 0x418, 0x437, 0x433,
0x435, 0x20, 0x415, 0x43b, 0x435, 0x43d, 0x430, 0x4d8, 0x441, 0x442, 0x435, 0x440,
0x445, 0x430, 0x43d, 0x41c, 0x44d, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432,
0x44b, 0x41c, 0x4d9, 0x441, 0x43a, 0x4d9, 0x4af, 0x420, 0x430, 0x448, 0x442, 0x443,
0x430, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x20, 0x443, 0x442, 0x440,
0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430,
0x433, 0x43e, 0x441, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440,
0x44b, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x44d, 0x41c, 0x430, 0x440, 0x43a, 0x438,
0x437, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0xc05,
0xc2c, 0xc3f, 0xc21, 0xc4d, 0xc1c, 0xc3e, 0xc28, 0xc4d, 0xc05, 0xc15, 0xc4d, 0xc30,
0xc3e, 0xc2f, 0xc3e, 0xc21, 0xc3f, 0xc38, 0xc4d, 0x20, 0xc05, 0xc2c, 0xc3e, 0xc2c,
0xc3e, 0xc05, 0xc32, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc05,
0xc38, 0xc4d, 0xc2e, 0xc3e, 0xc30, 0xc3e, 0xc2c, 0xc3e, 0xc2e, 0xc3e, 0xc15, 0xc4b,
0xc2c, 0xc3e, 0xc02, 0xc17, 0xc41, 0xc2f, 0xc4d, 0xc2c, 0xc02, 0xc1c, 0xc42, 0xc32,
0xc4d, 0xc2c, 0xc3f, 0xc38, 0xc4d, 0xc38, 0xc3e, 0xc35, 0xc4d, 0xc2c, 0xc4d, 0xc32,
0xc3e, 0xc28, 0xc4d, 0xc1f, 0xc48, 0xc30, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc3e, 0xc1c,
0xc3e, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0xc2c, 0xc41, 0xc1c, 0xc2e, 0xc4d,
0xc2c, 0xc41, 0xc30, 0xc3e, 0xc15, 0xc48, 0xc30, 0xc4b, 0xc15, 0xc3e, 0xc38, 0xc3e,
0xc2c, 0xc4d, 0xc32, 0xc3e, 0xc02, 0xc15, 0xc3e, 0xc38, 0xc4d, 0xc2f, 0xc42, 0xc1f,
0xc3e, 0xc15, 0xc4b, 0xc28, 0xc3e, 0xc15, 0xc4d, 0xc30, 0xc40, 0xc21, 0xc15, 0xc3e,
0xc30, 0xc4d, 0xc26, 0xc3e, 0xc30, 0xc4d, 0x20, 0xc0e, 0xc38, 0xc4d, 0x20, 0xc38,
0xc32, 0xc3e, 0xc2e, 0xc4d, 0xc21, 0xc3f, 0xc1c, 0xc4d, 0xc2c, 0xc4c, 0xc1f, 0xc3f,
0xc21, 0xc4c, 0xc32, 0xc3e, 0xc0e, 0xc32, 0xc4d, 0x20, 0xc0e, 0xc2f, 0xc41, 0xc28,
0xc4d, 0xc2b, 0xc4d, 0xc30, 0xc40, 0xc1f, 0xc4c, 0xc28, 0xc4d, 0xc17, 0xc2c, 0xc4b,
0xc30, 0xc4b, 0xc28, 0xc4d, 0xc39, 0xc30, 0xc3e, 0xc30, 0xc47, 0xc1c, 0xc4a, 0xc39,
0xc46, 0xc28, 0xc4d, 0xc38, 0xc4d, 0x200c, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc1c,
0xc41, 0xc2c, 0xc3e, 0xc15, 0xc02, 0xc2a, 0xc3e, 0xc32, 0xc3e, 0xc16, 0xc3e, 0xc30,
0xc4d, 0xc1f, 0xc4b, 0xc2e, 0xc4d, 0xc15, 0xc40, 0xc17, 0xc32, 0xc3f, 0xc15, 0xc3f,
0xc28, 0xc4d, 0xc37, 0xc3e, 0xc38, 0xc3e, 0xc32, 0xc3e, 0xc17, 0xc4b, 0xc38, 0xc4d,
0xc32, 0xc46, 0xc2c, 0xc30, 0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0xc32,
0xc4b, 0xc2e, 0xc4d, 0xc32, 0xc41, 0xc35, 0xc3e, 0xc02, 0xc21, 0xc3e, 0xc32, 0xc41,
0xc2c, 0xc02, 0xc2c, 0xc3e, 0xc37, 0xc3f, 0xc32, 0xc41, 0xc38, 0xc3e, 0xc15, 0xc3e,
0xc2e, 0xc32, 0xc3e, 0xc2c, 0xc4b, 0xc2e, 0xc3e, 0xc2a, 0xc41, 0xc1f, 0xc4b, 0xc2e,
0xc38, 0xc47, 0xc30, 0xc41, 0xc2c, 0xc3e, 0xc2c, 0xc46, 0xc28, 0xc4d, 0xc2e, 0xc4b,
0xc17, 0xc3e, 0xc26, 0xc3f, 0xc37, 0xc41, 0xc2e, 0xc4b, 0xc28, 0xc4d, 0xc30, 0xc4b,
0xc35, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc48, 0xc30, 0xc4b, 0xc2c, 0xc40, 0xc21, 0xc4d,
0xc1c, 0xc3e, 0xc2e, 0xc46, 0xc28, 0xc3e, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0xc2e, 0xc47,
0xc28, 0xc4d, 0xc35, 0xc3e, 0xc15, 0xc4d, 0xc37, 0xc4b, 0xc1f, 0xc4d, 0xc14, 0xc17,
0xc3e, 0xc21, 0xc4c, 0xc17, 0xc4b, 0xc35, 0xc4d, 0xc2a, 0xc4b, 0xc30, 0xc4d, 0xc1f,
0xc4b, 0x2d, 0xc28, 0xc4b, 0xc35, 0xc4b, 0xc38, 0xc3e, 0xc35, 0xc4b, 0x20, 0xc1f,
0xc4b, 0xc2e, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc3f, 0xc2a, 0xc4b, 0xc32, 0xc3f, 0xc1f,
0xc4d, 0xc2f, 0xc41, 0xc28, 0xc3f, 0xc38, 0xc4d, 0xc35, 0xc3f, 0xc02, 0xc21, 0xc4d,
0xc39, 0xc4b, 0xc15, 0xc4d, 0xc05, 0xc21, 0xc3e, 0xc15, 0xc4d, 0xc2f, 0xc3e, 0xc02,
0xc15, 0xc30, 0xc47, 0xc1c, 0xc4d, 0xc06, 0xc02, 0xc17, 0xc4d, 0xc35, 0xc3f, 0xc32,
0xc4d, 0xc32, 0xc3e, 0xc06, 0xc02, 0xc1f, 0xc3f, 0xc17, 0xc4d, 0xc35, 0xc3e, 0xc05,
0xc30, 0xc3e, 0xc17, 0xc4d, 0xc35, 0xc47, 0xc2f, 0xc40, 0xc28, 0xc3e, 0xc2c, 0xc4d,
0xc2f, 0xc42, 0xc28, 0xc4b, 0xc38, 0xc4d, 0x20, 0xc0e, 0xc2f, 0xc3f, 0xc30, 0xc4d,
0xc38, 0xc4d, 0xc15, 0xc1f, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc15, 0xc3e, 0xc15, 0xc4b,
0xc30, 0xc4d, 0xc21, 0xc4b, 0xc2c, 0xc3e, 0xc1c, 0xc41, 0xc1c, 0xc41, 0xc2f, 0xc4d,
0xc32, 0xc3e, 0x20, 0xc30, 0xc3f, 0xc2f, 0xc4b, 0xc1c, 0xc2e, 0xc46, 0xc02, 0xc21,
0xc4b, 0xc1c, 0xc3e, 0xc30, 0xc3f, 0xc2f, 0xc4b, 0x20, 0xc17, 0xc32, 0xc4d, 0xc32,
0xc47, 0xc17, 0xc4b, 0xc38, 0xc4d, 0xc38, 0xc3e, 0xc32, 0xc4d, 0xc1f, 0xc3e, 0xc36,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc1c, 0xc4d, 0xc2f, 0xc42, 0xc28, 0xc4d, 0xc36, 0xc3e,
0xc28, 0xc4d, 0x20, 0xc32, 0xc42, 0xc2f, 0xc3f, 0xc38, 0xc4d, 0xc1f, 0xc41, 0xc15,
0xc41, 0xc2e, 0xc28, 0xc4d, 0xc09, 0xc37, 0xc4d, 0xc2f, 0xc42, 0xc2f, 0xc05, 0xc30,
0xc41, 0xc2c, 0xc3e, 0xc05, 0xc38, 0xc41, 0xc28, 0xc4d, 0xc38, 0xc3f, 0xc2f, 0xc4b,
0xc28, 0xc4d, 0xc05, 0xc1f, 0xc3f, 0xc15, 0xc4b, 0xc15, 0xc28, 0xc4d, 0xc2c, 0xc39,
0xc3f, 0xc2f, 0xc2c, 0xc39, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2c, 0xc02, 0xc21, 0xc30,
0xc3e, 0xc38, 0xc4d, 0xc2c, 0xc3e, 0xc30, 0xc4d, 0xc2c, 0xc21, 0xc4b, 0xc38, 0xc4d,
0xc2c, 0xc46, 0xc32, 0xc46, 0xc2e, 0xc4d, 0xc2c, 0xc46, 0xc32, 0xc40, 0xc1c, 0xc4d,
0xc2c, 0xc4d, 0xc32, 0xc3e, 0xc02, 0xc15, 0xc4d, 0x2d, 0xc38, 0xc2c, 0xc4d, 0xc32,
0xc4b, 0xc28, 0xc4d, 0xc2c, 0xc4b, 0xc35, 0xc3e, 0x20, 0xc35, 0xc3f, 0xc38, 0xc4d,
0xc1f, 0xc3e, 0xc2c, 0xc17, 0xc4b, 0xc1f, 0xc3e, 0xc2c, 0xc4a, 0xc2f, 0xc3f, 0xc38,
0xc40, 0xc15, 0xc47, 0xc02, 0xc2c, 0xc4d, 0xc30, 0xc3f, 0xc21, 0xc4d, 0xc1c, 0xc4d,
0x20, 0xc2c, 0xc47, 0xc15, 0xc3e, 0xc02, 0xc2a, 0xc4b, 0x20, 0xc17, 0xc4d, 0xc30,
0xc3e, 0xc02, 0xc21, 0xc4d, 0xc15, 0xc28, 0xc4d, 0x200c, 0xc15, 0xc42, 0xc28, 0xc4d,
0xc15, 0xc3e, 0xc30, 0xc3e, 0xc15, 0xc38, 0xc4d, 0xc15, 0xc2f, 0xc47, 0xc28, 0xc4d,
0xc15, 0xc47, 0xc2e, 0xc3e, 0xc28, 0xc4d, 0xc1a, 0xc3f, 0xc15, 0xc3e, 0xc17, 0xc4b,
0xc1a, 0xc41, 0xc35, 0xc3e, 0xc35, 0xc3e, 0xc38, 0xc3f, 0xc2f, 0xc41, 0xc26, 0xc3e,
0xc26, 0xc4d, 0x20, 0xc39, 0xc4d, 0xc35, 0xc3e, 0xc30, 0xc46, 0xc1c, 0xc4d, 0xc15,
0xc4b, 0xc38, 0xc4d, 0xc1f, 0xc3e, 0x20, 0xc30, 0xc3f, 0xc15, 0xc3e, 0xc15, 0xc4d,
0xc30, 0xc46, 0xc38, 0xc4d, 0xc1f, 0xc28, 0xc4d, 0xc15, 0xc41, 0xc2f, 0xc2c, 0xc3e,
0xc15, 0xc41, 0xc30, 0xc3e, 0xc15, 0xc35, 0xc4b, 0xc21, 0xc46, 0xc28, 0xc4d, 0xc2e,
0xc3e, 0xc30, 0xc4d, 0xc15, 0xc4d, 0x200c, 0xc37, 0xc3e, 0xc28, 0xc4d, 0xc21, 0xc3e,
0xc38, 0xc28, 0xc4d, 0xc21, 0xc3e, 0xc38, 0xc28, 0xc4d, 0x20, 0xc15, 0xc4d, 0xc30,
0xc40, 0xc15, 0xc4d, 0xc21, 0xc46, 0xc28, 0xc4d, 0xc35, 0xc46, 0xc30, 0xc4d, 0xc21,
0xc3f, 0xc1f, 0xc4d, 0xc30, 0xc4b, 0xc2f, 0xc3f, 0xc1f, 0xc4d, 0xc21, 0xc4a, 0xc2e,
0xc3f, 0xc28, 0xc3f, 0xc15, 0xc3e, 0xc0e, 0xc21, 0xc4d, 0xc2e, 0xc4b, 0xc02, 0xc1f,
0xc28, 0xc4d, 0xc07, 0xc30, 0xc41, 0xc28, 0xc46, 0xc2a, 0xc4d, 0xc0e, 0xc32, 0xc4d,
0x20, 0xc38, 0xc3e, 0xc32, 0xc4d, 0xc35, 0xc21, 0xc4b, 0xc30, 0xc4d, 0xc2b, 0xc4b,
0xc30, 0xc4d, 0xc1f, 0xc4d, 0x20, 0xc28, 0xc46, 0xc32, 0xc4d, 0xc38, 0xc28, 0xc4d,
0xc2b, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc32, 0xc47, 0xc1c, 0xc3e, 0xc17, 0xc4d, 0xc32,
0xc47, 0xc38, 0xc4d, 0x20, 0xc2c, 0xc47, 0xc17, 0xc42, 0xc38, 0xc4d, 0x20, 0xc2c,
0xc47, 0xc17, 0xc4d, 0xc30, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc1f, 0xc30, 0xc4d,
0xc15, 0xc4d, 0xc17, 0xc4d, 0xc30, 0xc46, 0xc28, 0xc21, 0xc3e, 0xc17, 0xc4d, 0xc35,
0xc3e, 0xc21, 0xc46, 0xc32, 0xc4b, 0xc2a, 0xc4d, 0xc17, 0xc4d, 0xc35, 0xc3e, 0xc1f,
0xc3f, 0xc2e, 0xc3e, 0xc32, 0xc3e, 0xc17, 0xc2f, 0xc3e, 0xc15, 0xc4d, 0xc35, 0xc3f,
0xc32, 0xc4d, 0xc17, 0xc2f, 0xc3e, 0xc28, 0xc3e, 0xc39, 0xc3e, 0xc32, 0xc3f, 0xc2b,
0xc3e, 0xc15, 0xc4d, 0xc38, 0xc4d, 0xc39, 0xc35, 0xc3e, 0xc28, 0xc3e, 0xc39, 0xc46,
0xc30, 0xc4d, 0xc2e, 0xc4b, 0xc38, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc4b, 0xc07, 0xc02,
0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc2a, 0xc4b, 0xc32, 0xc3f, 0xc38, 0xc4d, 0xc28,
0xc4b, 0xc15, 0xc4d, 0xc38, 0xc4d, 0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f,
0xc3e, 0xc28, 0xc2e, 0xc30, 0xc46, 0xc02, 0xc17, 0xc4b, 0x2c, 0x20, 0xc07, 0xc02,
0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc2a, 0xc40, 0xc1f, 0xc30, 0xc4d, 0xc38, 0xc4d,
0x200c, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f,
0xc2f, 0xc3e, 0xc28, 0xc1f, 0xc46, 0xc32, 0xc4d, 0x20, 0xc28, 0xc17, 0xc30, 0xc02,
0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc35, 0xc46, 0xc35,
0xc2f, 0xc4d, 0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc35,
0xc3f, 0xc28, 0xc4d, 0x200c, 0xc38, 0xc46, 0xc28, 0xc4d, 0xc28, 0xc46, 0xc38, 0xc4d,
0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc35, 0xc3f, 0xc28,
0xc3e, 0xc2e, 0xc3e, 0xc15, 0xc4d, 0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f,
0xc3e, 0xc28, 0xc07, 0xc28, 0xc41, 0xc35, 0xc3f, 0xc15, 0xc4d, 0xc07, 0xc15, 0xc4d,
0xc35, 0xc3e, 0xc32, 0xc3f, 0xc1f, 0xc4d, 0xc1c, 0xc2e, 0xc48, 0xc15, 0xc3e, 0xc1c,
0xc42, 0xc28, 0xc4b, 0xc32, 0xc42, 0xc2f, 0xc3f, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc2e,
0xc4b, 0xc02, 0xc1f, 0xc3f, 0xc38, 0xc46, 0xc32, 0xc4d, 0xc32, 0xc4b, 0x2c, 0x20,
0xc15, 0xc46, 0xc28, 0xc4d, 0x200c, 0xc1f, 0xc41, 0xc15, 0xc4d, 0xc15, 0xc40, 0xc15,
0xc4d, 0xc30, 0xc32, 0xc46, 0xc02, 0xc21, 0xc4d, 0xc1c, 0xc3f, 0xc15, 0xc4d, 0xc32,
0xc3e, 0x20, 0xc2a, 0xc3e, 0xc1c, 0xc4d, 0xc32, 0xc3f, 0xc2e, 0xc3e, 0xc32, 0xc3e,
0xc38, 0xc4d, 0x20, 0xc0f, 0xc02, 0xc1c, 0xc32, 0xc4d, 0xc38, 0xc4d, 0xc32, 0xc4b,
0xc2f, 0xc30, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3f, 0xc28, 0xc4d, 0xc38, 0xc4d,
0x20, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc30, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0xc2e, 0xc3e,
0xc38, 0xc3f, 0xc2f, 0xc4b, 0xc2e, 0xc28, 0xc3e, 0xc17, 0xc41, 0xc35, 0xc3e, 0xc2e,
0xc28, 0xc3e, 0xc38, 0xc4d, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc17, 0xc4b, 0xc1f, 0xc4d,
0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc1f, 0xc3f, 0xc28, 0xc40, 0xc15, 0xc4d, 0xc2e, 0xc3e,
0xc1f, 0xc2e, 0xc4a, 0xc30, 0xc4b, 0xc38, 0xc4d, 0xc2e, 0xc3e, 0xc38, 0xc1f, 0xc4d,
0x200c, 0xc32, 0xc3e, 0xc28, 0xc4d, 0xc2e, 0xc46, 0xc28, 0xc4b, 0xc2e, 0xc3f, 0xc28,
0xc40, 0xc2e, 0xc46, 0xc30, 0xc3f, 0xc21, 0xc3e, 0xc2e, 0xc46, 0xc1f, 0xc4d, 0xc32,
0xc15, 0xc1f, 0xc4d, 0xc32, 0xc2e, 0xc46, 0xc15, 0xc4d, 0xc38, 0xc3f, 0xc15, 0xc4b,
0x20, 0xc28, 0xc17, 0xc30, 0xc02, 0xc2e, 0xc3f, 0xc15, 0xc46, 0xc32, 0xc3e, 0xc28,
0xc4d, 0xc2e, 0xc4b, 0xc28, 0xc4d, 0x200c, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc28, 0xc4d,
0xc2e, 0xc4b, 0xc02, 0xc1f, 0xc46, 0xc30, 0xc4d, 0xc30, 0xc47, 0xc2e, 0xc4b, 0xc02,
0xc1f, 0xc46, 0xc35, 0xc40, 0xc21, 0xc3f, 0xc2f, 0xc4b, 0xc2e, 0xc3e, 0xc02, 0xc1f,
0xc4d, 0xc38, 0xc46, 0xc30, 0xc3e, 0xc1f, 0xc4d, 0xc28, 0xc3e, 0xc38, 0xc4d, 0xc38,
0xc3e, 0xc35, 0xc4d, 0xc28, 0xc4d, 0xc2f, 0xc42, 0xc2f, 0xc3e, 0xc30, 0xc4d, 0xc15,
0xc4d, 0xc28, 0xc4b, 0xc2e, 0xc4d, 0xc28, 0xc30, 0xc4b, 0xc28, 0xc4d, 0xc39, 0xc3e,
0xc2c, 0xc4d, 0xc2f, 0xc41, 0xc32, 0xc3e, 0x2c, 0x20, 0xc09, 0xc24, 0xc4d, 0xc24,
0xc30, 0x20, 0xc21, 0xc15, 0xc4b, 0xc1f, 0xc38, 0xc46, 0xc02, 0xc1f, 0xc30, 0xc4d,
0x2c, 0x20, 0xc09, 0xc24, 0xc4d, 0xc24, 0xc30, 0x20, 0xc21, 0xc15, 0xc4b, 0xc1f,
0xc3e, 0xc28, 0xc4d, 0xc2f, 0xc42, 0x20, 0xc38, 0xc32, 0xc47, 0xc2e, 0xc4d, 0x2c,
0x20, 0xc09, 0xc24, 0xc4d, 0xc24, 0xc30, 0x20, 0xc21, 0xc15, 0xc4b, 0xc1f, 0xc28,
0xc42, 0xc15, 0xc4d, 0xc12, 0xc1c, 0xc3f, 0xc28, 0xc17, 0xc2a, 0xc28, 0xc3e, 0xc2e,
0xc3e, 0xc2a, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc2c, 0xc4b, 0xc2b, 0xc3f,
0xc28, 0xc3f, 0xc15, 0xc4d, 0xc38, 0xc4d, 0xc2a, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4d,
0x2d, 0xc05, 0xc35, 0xc4d, 0x2d, 0xc2a, 0xc4d, 0xc30, 0xc3f, 0xc28, 0xc4d, 0xc38,
0xc4d, 0xc2a, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4d, 0x20, 0xc06, 0xc2b, 0xc4d, 0x20,
0xc38, 0xc4d, 0xc2a, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0xc2a, 0xc4b, 0xc30, 0xc4d,
0xc1f, 0xc4b, 0x20, 0xc35, 0xc46, 0xc32, 0xc4d, 0xc39, 0xc4b, 0xc2a, 0xc4d, 0xc2f,
0xc42, 0xc30, 0xc4d, 0xc1f, 0xc4b, 0x20, 0xc30, 0xc3f, 0xc15, 0xc4b, 0xc2a, 0xc41,
0xc02, 0xc1f, 0xc3e, 0x20, 0xc05, 0xc30, 0xc40, 0xc28, 0xc38, 0xc4d, 0xc30, 0xc28,
0xc4d, 0x200c, 0xc15, 0xc3f, 0xc28, 0xc4d, 0x20, 0xc07, 0xc28, 0xc4d, 0x200c, 0xc32,
0xc46, 0xc1f, 0xc4d, 0xc30, 0xc46, 0xc38, 0xc3f, 0xc2b, 0xc40, 0xc30, 0xc46, 0xc1c,
0xc40, 0xc28, 0xc3e, 0xc30, 0xc3f, 0xc1c, 0xc32, 0xc4d, 0xc2f, 0xc42, 0xc1f, 0xc4d,
0xc30, 0xc3f, 0xc2f, 0xc4b, 0x20, 0xc2c, 0xc4d, 0xc30, 0xc3e, 0xc02, 0xc15, 0xc4b,
0xc38, 0xc3e, 0xc02, 0xc1f, 0xc30, 0xc46, 0xc2e, 0xc4d, 0xc36, 0xc3e, 0xc02, 0xc1f,
0xc3f, 0xc2f, 0xc3e, 0xc17, 0xc4b, 0xc36, 0xc3e, 0xc02, 0xc1f, 0xc4b, 0x20, 0xc21,
0xc4b, 0xc2e, 0xc3f, 0xc02, 0xc17, 0xc4b, 0xc38, 0xc3e, 0xc35, 0xc4b, 0x20, 0xc2a,
0xc3e, 0xc32, 0xc4b, 0xc07, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0xc15, 0xc4b, 0xc30, 0xc4d,
0xc1f, 0xc42, 0xc30, 0xc4d, 0xc2e, 0xc3f, 0xc1f, 0xc4d, 0xc38, 0xc3f, 0xc1f, 0xc4d,
0xc15, 0xc3e, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc2c, 0xc30,
0xc4d, 0xc24, 0xc46, 0xc32, 0xc46, 0xc2e, 0xc40, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02,
0xc1f, 0xc4d, 0x20, 0xc1c, 0xc3e, 0xc28, 0xc4d, 0xc38, 0xc4d, 0xc38, 0xc46, 0xc2f,
0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc15, 0xc3f, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc38,
0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc32, 0xc42, 0xc38, 0xc3f, 0xc2f,
0xc3e, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc25, 0xc3e, 0xc2e,
0xc38, 0xc4d, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc35, 0xc3f,
0xc28, 0xc4d, 0xc38, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc35, 0xc3f, 0xc2b,
0xc4d, 0xc1f, 0xc4d, 0x20, 0xc15, 0xc30, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc24, 0xc46,
0xc17, 0xc41, 0xc38, 0xc3f, 0xc17, 0xc32, 0xc4d, 0xc2a, 0xc3e, 0xc25, 0xc41, 0xc32,
0xc47, 0xc1f, 0xc3f, 0xc1c, 0xc41, 0xc35, 0xc3e, 0xc28, 0xc3e, 0xc1f, 0xc4a, 0xc30,
0xc02, 0xc1f, 0xc4b, 0xc1f, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4b, 0xc32, 0xc3e, 0xc35,
0xc3e, 0xc28, 0xc4d, 0xc15, 0xc42, 0xc35, 0xc30, 0xc4d, 0xc35, 0xc48, 0xc1f, 0xc4d,
0x200c, 0xc39, 0xc3e, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc35, 0xc3f, 0xc28, 0xc4d, 0xc28,
0xc3f, 0xc2a, 0xc46, 0xc17, 0xc4d, 0xc2f, 0xc15, 0xc41, 0xc1f, 0xc3e, 0xc1f, 0xc4d,
0xc15, 0xc47, 0xc38, 0xc40, 0xc21, 0xc46, 0xc35, 0xc3f, 0xc38, 0xc4d, 0xc21, 0xc4d,
0xc2f, 0xc42, 0xc2e, 0xc3e, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc21, 0xc3f, 0x2019, 0xc09,
0xc30, 0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0xc2e, 0xc15, 0xc3e, 0xc30,
0xc40, 0xc2e, 0xc3e, 0xc38, 0xc28, 0xc4d, 0xc2e, 0xc46, 0xc15, 0xc4d, 0x200c, 0xc2e,
0xc41, 0xc30, 0xc4d, 0xc21, 0xc4b, 0xc2a, 0xc3e, 0xc32, 0xc4d, 0xc2e, 0xc30, 0xc4d,
0xc30, 0xc4a, 0xc24, 0xc47, 0xc30, 0xc3e, 0xc38, 0xc4d, 0xc2f, 0xc4b, 0xc35, 0xc3e,
0xc1f, 0xc4d, 0xc30, 0xc4b, 0xc32, 0xc4d, 0xc35, 0xc4b, 0xc38, 0xc4d, 0xc1f, 0xc4b,
0xc15, 0xc4d, 0xc32, 0xc3e, 0xc02, 0xc17, 0xc4d, 0x200c, 0xc2f, 0xc3f, 0xc2f, 0xc30,
0xc4d, 0x200c, 0xc2c, 0xc48, 0xc2f, 0xc28, 0xc4d, 0xc0e, 0xc21, 0xc46, 0xc28, 0xc4d,
0xc06, 0xc32, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0xc05, 0xc2e, 0xc4d, 0xc2e, 0xc28,
0xc4d, 0xc05, 0xc28, 0xc21, 0xc48, 0xc30, 0xc4d, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc3e,
0xc35, 0xc4d, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc4d, 0xc2f, 0xc3e, 0xc37,
0xc4d, 0xc17, 0xc3e, 0xc2c, 0xc3e, 0xc1f, 0xc4d, 0xc06, 0xc1f, 0xc3f, 0xc30, 0xc3e,
0xc2c, 0xc3e, 0xc17, 0xc4d, 0xc26, 0xc3e, 0xc26, 0xc4d, 0xc2c, 0xc39, 0xc4d, 0xc30,
0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0xc2c, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc15, 0xc3e,
0xc15, 0xc4d, 0xc2c, 0xc3e, 0xc30, 0xc4d, 0xc28, 0xc3e, 0xc32, 0xc4d, 0xc2c, 0xc40,
0xc30, 0xc1f, 0xc4d, 0xc2c, 0xc3f, 0xc37, 0xc4d, 0xc15, 0xc46, 0xc15, 0xc4d, 0xc2c,
0xc4d, 0xc30, 0xc42, 0xc28, 0xc48, 0xc1a, 0xc3f, 0xc24, 0xc3e, 0xc15, 0xc4a, 0xc32,
0xc02, 0xc2c, 0xc4b, 0xc21, 0xc2e, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc38, 0xc4d, 0xc22,
0xc3e, 0xc15, 0xc3e, 0xc21, 0xc3f, 0xc32, 0xc3f, 0xc26, 0xc41, 0xc2c, 0xc3e, 0xc2f,
0xc3f, 0xc21, 0xc41, 0xc37, 0xc28, 0xc4d, 0xc2c, 0xc40, 0xc2b, 0xc3e, 0xc2e, 0xc17,
0xc41, 0xc38, 0xc4d, 0xc1f, 0xc3e, 0xc17, 0xc3e, 0xc1c, 0xc3e, 0xc39, 0xc46, 0xc2c,
0xc4d, 0xc30, 0xc3e, 0xc28, 0xc4d, 0xc39, 0xc4b, 0x20, 0xc1a, 0xc3f, 0x20, 0xc2e,
0xc3f, 0xc28, 0xc4d, 0xc39, 0xc4d, 0x20, 0xc28, 0xc17, 0xc30, 0xc02, 0xc39, 0xc3e,
0xc02, 0xc15, 0xc3e, 0xc02, 0xc17, 0xc4d, 0xc39, 0xc4b, 0xc35, 0xc4d, 0xc21, 0xc4d,
0xc07, 0xc30, 0xc4d, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc1c,
0xc15, 0xc3e, 0xc30, 0xc4d, 0xc24, 0xc3e, 0xc1c, 0xc2f, 0xc2a, 0xc41, 0xc30, 0xc1c,
0xc30, 0xc42, 0xc38, 0xc32, 0xc47, 0xc02, 0xc15, 0xc3e, 0xc2c, 0xc41, 0xc32, 0xc4d,
0xc15, 0xc2e, 0xc4d, 0x200c, 0xc1a, 0xc24, 0xc4d, 0xc15, 0xc3e, 0xc15, 0xc30, 0xc3e,
0xc1a, 0xc40, 0xc16, 0xc3e, 0xc1f, 0xc4d, 0xc2e, 0xc02, 0xc21, 0xc41, 0xc15, 0xc02,
0xc26, 0xc4d, 0xc2f, 0xc3e, 0xc17, 0xc15, 0xc4b, 0xc32, 0xc4d, 0x200c, 0xc15, 0xc24,
0xc3e, 0xc15, 0xc4d, 0xc30, 0xc38, 0xc28, 0xc4b, 0xc2f, 0xc3e, 0xc30, 0xc4d, 0xc38,
0xc4d, 0xc15, 0xc4d, 0xc15, 0xc4c, 0xc32, 0xc3e, 0xc32, 0xc02, 0xc2a, 0xc42, 0xc30,
0xc4d, 0xc15, 0xc41, 0xc1a, 0xc3f, 0xc02, 0xc17, 0xc4d, 0xc15, 0xc41, 0xc35, 0xc48,
0xc1f, 0xc4d, 0xc2e, 0xc15, 0xc3e, 0xc35, 0xc4d, 0xc2e, 0xc17, 0xc21, 0xc3e, 0xc28,
0xc4d, 0xc2e, 0xc15, 0xc3e, 0xc38, 0xc4d, 0xc38, 0xc30, 0xc4d, 0xc2e, 0xc28, 0xc40,
0xc32, 0xc3e, 0xc2e, 0xc38, 0xc4d, 0xc15, 0xc1f, 0xc4d, 0xc28, 0xc3f, 0xc15, 0xc4b,
0xc38, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc4a, 0xc35, 0xc4a, 0xc15, 0xc41, 0xc1c, 0xc4d,
0x200c, 0xc28, 0xc46, 0xc1f, 0xc4d, 0x200c, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc28, 0xc35,
0xc4b, 0xc38, 0xc3f, 0xc2c, 0xc3f, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc13,
0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc13, 0xc30, 0xc32, 0xc4d, 0xc28, 0xc4b,
0xc2e, 0xc4d, 0x200c, 0xc2a, 0xc46, 0xc28, 0xc4d, 0xc39, 0xc4d, 0xc2a, 0xc4a, 0xc28,
0xc4d, 0xc1f, 0xc3f, 0xc2f, 0xc28, 0xc3e, 0xc15, 0xc4d, 0xc2a, 0xc4d, 0xc2f, 0xc4b,
0xc02, 0xc17, 0xc3e, 0xc02, 0xc17, 0xc4d, 0xc16, 0xc24, 0xc3e, 0xc30, 0xc4d, 0xc15,
0xc4b, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc47, 0xc15, 0xc4d, 0xc35, 0xc3f, 0xc1c,
0xc3f, 0xc32, 0xc4b, 0xc30, 0xc4d, 0xc21, 0xc3e, 0xc30, 0xc3f, 0xc2f, 0xc3e, 0xc27,
0xc4d, 0xc38, 0xc16, 0xc3e, 0xc32, 0xc3f, 0xc28, 0xc4d, 0xc38, 0xc2e, 0xc30, 0xc4d,
0xc15, 0xc3e, 0xc02, 0xc21, 0xc4d, 0xc38, 0xc3f, 0xc2f, 0xc4b, 0xc32, 0xc4d, 0xc37,
0xc3e, 0xc02, 0xc18, 0xc48, 0xc38, 0xc3f, 0xc02, 0xc17, 0xc2a, 0xc42, 0xc30, 0xc4d,
0xc38, 0xc4d, 0xc30, 0xc46, 0xc21, 0xc4d, 0xc28, 0xc46, 0xc15, 0xc4a, 0xc32, 0xc3f,
0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc24, 0xc48, 0xc2a, 0xc40, 0xc24, 0xc3e,
0xc37, 0xc4d, 0xc15, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc1f, 0xc3f, 0xc2c, 0xc3f, 0xc32,
0xc3f, 0xc38, 0xc3f, 0xc1f, 0xc46, 0xc39, 0xc4d, 0xc30, 0xc3e, 0xc28, 0xc4d, 0xc25,
0xc3f, 0xc02, 0xc2b, 0xc41, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0xc2f, 0xc4b, 0xc1f, 0xc3e,
0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc09, 0xc32, 0xc3e, 0xc28, 0xc4d, 0xc2c,
0xc3e, 0xc1f, 0xc30, 0xc4d, 0xc09, 0xc30, 0xc41, 0xc2e, 0xc4d, 0x200c, 0xc15, 0xc40,
0xc05, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0x2d, 0xc28, 0xc46, 0xc30, 0xc3e, 0xc35, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0xc1f, 0xc48, 0xc28, 0xc4d, 0xc35, 0xc4d, 0xc32, 0xc3e, 0xc21,
0xc3f, 0xc35, 0xc4b, 0xc38, 0xc4d, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0xc2f, 0xc15, 0xc41,
0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc17, 0xc28, 0xc4d,
0xc2f, 0xc46, 0xc15, 0xc1f, 0xc46, 0xc30, 0xc3f, 0xc02, 0xc2c, 0xc30, 0xc4d, 0xc17,
0xc4d, 0xc2f, 0xc46, 0xc30, 0xc46, 0xc35, 0xc28, 0xc4d, 0xc05, 0xc1c, 0xc4b, 0xc30,
0xc4d, 0xc38, 0xc4d, 0xc2c, 0xc46, 0xc30, 0xc4d, 0xc2e, 0xc41, 0xc21, 0xc3e, 0xc15,
0xc46, 0xc28, 0xc30, 0xc40, 0xc15, 0xc47, 0xc2a, 0xc4d, 0x20, 0xc35, 0xc46, 0xc30,
0xc4d, 0xc21, 0xc46, 0xc2b, 0xc3e, 0xc30, 0xc4b, 0xc2e, 0xc26, 0xc48, 0xc30, 0xc3e,
0xc30, 0xc46, 0xc15, 0xc4d, 0xc1c, 0xc3e, 0xc35, 0xc3f, 0xc15, 0xc4d, 0xc26, 0xc15,
0xc4d, 0xc37, 0xc3f, 0xc23, 0x20, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f,
0xc3e, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc39, 0xc46, 0xc32,
0xc46, 0xc28, 0xc3e, 0xc38, 0xc4d, 0xc1f, 0xc3e, 0xc28, 0xc4d, 0xc32, 0xc40, 0xc05,
0xc21, 0xc46, 0xc32, 0xc48, 0xc21, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc3f, 0xc38, 0xc4d,
0x200c, 0xc2c, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc4b, 0xc15,
0xc46, 0xc28, 0xc4d, 0x20, 0xc39, 0xc3f, 0xc32, 0xc4d, 0xc21, 0xc3e, 0xc30, 0xc4d,
0xc35, 0xc3f, 0xc28, 0xc4d, 0xc2f, 0xc41, 0xc15, 0xc4d, 0xc32, 0xc3e, 0xc39, 0xc4b,
0xc2c, 0xc30, 0xc4d, 0xc1f, 0xc4d, 0xc32, 0xc3f, 0xc02, 0xc21, 0xc46, 0xc2e, 0xc3e,
0xc28, 0xc4d, 0xc32, 0xc3e, 0xc30, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc39, 0xc4c, 0xc2e,
0xc46, 0xc32, 0xc4d, 0xc2c, 0xc4b, 0xc30, 0xc4d, 0xc28, 0xc4d, 0xc2a, 0xc46, 0xc30,
0xc4d, 0xc24, 0xc4d, 0xc38, 0xc3f, 0xc21, 0xc4d, 0xc28, 0xc40, 0xc06, 0xc2e, 0xc4d,
0x200c, 0xc38, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0x200c, 0xc21, 0xc3e, 0xc2e, 0xc4d, 0xc05,
0xc02, 0xc21, 0xc4b, 0xc30, 0xc3e, 0xc05, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc16,
0xc3e, 0xc28, 0xc4d, 0xc0f, 0xc25, 0xc46, 0xc28, 0xc4d, 0xc38, 0xc4d, 0xc2c, 0xc46,
0xc32, 0xc4d, 0x200c, 0xc17, 0xc4d, 0xc30, 0xc47, 0xc21, 0xc4d, 0xc2c, 0xc46, 0xc30,
0xc4d, 0xc32, 0xc3f, 0xc28, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc3e, 0xc1f, 0xc3f, 0xc38,
0xc4d, 0xc32, 0xc3e, 0xc35, 0xc3e, 0xc2c, 0xc4d, 0xc30, 0xc38, 0xc4d, 0xc38, 0xc46,
0xc32, 0xc4d, 0xc38, 0xc4d, 0xc2c, 0xc41, 0xc15, 0xc3e, 0xc30, 0xc46, 0xc38, 0xc4d,
0xc1f, 0xc4d, 0xc2c, 0xc41, 0xc21, 0xc3e, 0xc2a, 0xc46, 0xc38, 0xc4d, 0xc1f, 0xc4d,
0xc2c, 0xc38, 0xc3f, 0xc02, 0xc1c, 0xc28, 0xc4d, 0xc1a, 0xc3f, 0xc38, 0xc3f, 0xc28,
0xc3e, 0xc35, 0xc4d, 0xc15, 0xc4b, 0xc2a, 0xc46, 0xc28, 0xc4d, 0xc39, 0xc3e, 0xc17,
0xc28, 0xc4d, 0xc21, 0xc2c, 0xc4d, 0xc32, 0xc3f, 0xc28, 0xc4d, 0xc1c, 0xc3f, 0xc2c,
0xc4d, 0xc30, 0xc3e, 0xc32, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc35, 0xc46,
0xc30, 0xc4d, 0xc28, 0xc4d, 0xc38, 0xc47, 0xc39, 0xc46, 0xc32, 0xc4d, 0xc38, 0xc3f,
0xc02, 0xc15, 0xc3f, 0xc10, 0xc32, 0xc4d, 0x20, 0xc06, 0xc2b, 0xc4d, 0x20, 0xc2e,
0xc47, 0xc28, 0xc4d, 0xc07, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc02, 0xc2c, 0xc41, 0xc32,
0xc4d, 0xc1c, 0xc46, 0xc30, 0xc4d, 0xc38, 0xc40, 0xc15, 0xc32, 0xc3f, 0xc28, 0xc3f,
0xc28, 0xc4d, 0x200c, 0xc17, 0xc4d, 0xc30, 0xc26, 0xc4d, 0xc15, 0xc3f, 0xc30, 0xc4b,
0xc35, 0xc4d, 0xc15, 0xc40, 0xc35, 0xc4d, 0xc32, 0xc3f, 0xc38, 0xc4d, 0xc2c, 0xc28,
0xc4d, 0xc32, 0xc4d, 0xc2f, 0xc42, 0xc2c, 0xc4d, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc28,
0xc3e, 0xc32, 0xc02, 0xc21, 0xc28, 0xc4d, 0xc32, 0xc15, 0xc4d, 0xc38, 0xc46, 0xc02,
0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc2e, 0xc3e, 0xc21, 0xc4d, 0xc30, 0xc3f, 0xc21,
0xc4d, 0xc2e, 0xc3e, 0xc32, 0xc4d, 0xc1f, 0xc3e, 0xc2e, 0xc3e, 0xc30, 0xc40, 0xc2f,
0xc41, 0xc39, 0xc2e, 0xc4d, 0xc2e, 0xc3f, 0xc28, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d,
0xc2e, 0xc4a, 0xc28, 0xc3e, 0xc15, 0xc4b, 0xc2e, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc4b,
0xc13, 0xc38, 0xc4d, 0xc32, 0xc4b, 0xc2a, 0xc4d, 0xc2f, 0xc3e, 0xc30, 0xc3f, 0xc38,
0xc4d, 0xc2a, 0xc4b, 0xc21, 0xc4d, 0xc17, 0xc4b, 0xc30, 0xc3f, 0xc15, 0xc3e, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc17, 0xc3e, 0xc30, 0xc4b, 0xc2e,
0xc4d, 0xc38, 0xc2e, 0xc3e, 0xc30, 0xc36, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2e, 0xc3e,
0xc30, 0xc3f, 0xc28, 0xc4b, 0xc38, 0xc30, 0xc3e, 0xc1c, 0xc4b, 0xc35, 0xc4b, 0xc38,
0xc30, 0xc3e, 0xc1f, 0xc35, 0xc4d, 0xc38, 0xc3f, 0xc2e, 0xc4d, 0x200c, 0xc2b, 0xc46,
0xc30, 0xc4b, 0xc2a, 0xc4b, 0xc32, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4b, 0xc2a, 0xc4d,
0x200c, 0xc2f, 0xc47, 0xc38, 0xc4b, 0xc2b, 0xc3f, 0xc2f, 0xc3e, 0xc38, 0xc4d, 0xc1f,
0xc3e, 0xc15, 0xc4d, 0xc39, 0xc4b, 0xc2e, 0xc4d, 0xc24, 0xc3e, 0xc32, 0xc4d, 0xc32,
0xc3f, 0xc28, 0xc4d, 0xc1f, 0xc3f, 0xc30, 0xc47, 0xc28, 0xc4d, 0xc09, 0xc32, 0xc4d,
0xc2f, 0xc28, 0xc4b, 0xc35, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc21,
0xc41, 0xc1c, 0xc4d, 0xc35, 0xc3e, 0xc1f, 0xc3f, 0xc15, 0xc28, 0xc4d, 0xc35, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0xc28, 0xc3e, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc28, 0xc3f, 0xc2f,
0xc38, 0xc4d, 0xc35, 0xc4b, 0xc32, 0xc4d, 0xc17, 0xc4b, 0xc17, 0xc4d, 0xc30, 0xc3e,
0xc21, 0xc4d, 0xc35, 0xc3e, 0xc30, 0xc4d, 0xc37, 0xc3e, 0xc1c, 0xc3e, 0xc17, 0xc4d,
0xc30, 0xc46, 0xc2c, 0xc4d, 0xc1c, 0xc4d, 0xc2f, 0xc42, 0xc30, 0xc3f, 0xc1a, 0xc4d,
0xc05, 0xc02, 0xc1f, 0xc3e, 0xc28, 0xc3e, 0xc28, 0xc3e, 0xc30, 0xc3f, 0xc35, 0xc4b,
0xc1a, 0xc3e, 0xc17, 0xc4b, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc30, 0xc3f, 0xc38, 0xc4d,
0xc2e, 0xc38, 0xc4d, 0xc15, 0xc4b, 0xc15, 0xc4b, 0xc38, 0xc4d, 0xc15, 0xc4a, 0xc2e,
0xc4b, 0xc30, 0xc4b, 0xc15, 0xc46, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc2f, 0xc42, 0xc32,
0xc46, 0xc28, 0xc4d, 0xc2e, 0xc3e, 0xc39, 0xc46, 0xc2e, 0xc3e, 0xc32, 0xc4d, 0xc26,
0xc40, 0xc35, 0xc41, 0xc32, 0xc41, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc37, 0xc38, 0xc4d,
0xc2e, 0xc2f, 0xc4b, 0xc1f, 0xc4d, 0xc30, 0xc40, 0xc2f, 0xc42, 0xc28, 0xc3f, 0xc2f,
0xc28, 0xc4d, 0xc0f, 0xc2a, 0xc3f, 0xc2f, 0xc3e, 0xc06, 0xc15, 0xc4d, 0xc32, 0xc3e,
0xc02, 0xc21, 0xc4d, 0xc2c, 0xc4a, 0xc17, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0x200c,
0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0xc1a, 0xc3e, 0xc25, 0xc2e, 0xc4d, 0xc1a,
0xc41, 0xc15, 0xc4d, 0xc08, 0xc38, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0xc07, 0xc2b, 0xc47,
0xc1f, 0xc4d, 0xc2b, 0xc3e, 0xc15, 0xc4b, 0xc2b, 0xc4b, 0xc2b, 0xc40, 0xc1c, 0xc40,
0xc2b, 0xc41, 0xc28, 0xc3e, 0xc2b, 0xc41, 0xc1f, 0xc3f, 0xc17, 0xc3e, 0xc32, 0xc3e,
0xc2a, 0xc3e, 0xc17, 0xc4b, 0xc38, 0xc4d, 0xc17, 0xc3e, 0xc02, 0xc2c, 0xc3f, 0xc2f,
0xc47, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc35, 0xc3e, 0xc21, 0xc32, 0xc4d, 0xc15, 0xc46,
0xc28, 0xc3e, 0xc32, 0xc4d, 0xc17, 0xc4d, 0xc35, 0xc3e, 0xc2e, 0xc4d, 0xc39, 0xc4b,
0xc28, 0xc4b, 0xc32, 0xc41, 0xc32, 0xc41, 0xc0e, 0xc02, 0xc21, 0xc30, 0xc4d, 0xc2c,
0xc46, 0xc30, 0xc40, 0xc15, 0xc3f, 0xc30, 0xc40, 0xc1f, 0xc3f, 0xc2e, 0xc3e, 0xc1f,
0xc3f, 0xc15, 0xc4b, 0xc38, 0xc4d, 0xc30, 0xc47, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc1c,
0xc3e, 0xc32, 0xc48, 0xc28, 0xc4d, 0xc2e, 0xc1c, 0xc41, 0xc30, 0xc4b, 0xc2e, 0xc3e,
0xc30, 0xc4d, 0xc15, 0xc4d, 0xc35, 0xc47, 0xc38, 0xc3e, 0xc38, 0xc4d, 0xc2e, 0xc3f,
0xc21, 0xc4d, 0xc35, 0xc47, 0xc28, 0xc4c, 0xc30, 0xc41, 0xc28, 0xc3f, 0xc2f, 0xc42,
0xc28, 0xc4b, 0xc30, 0xc4d, 0xc2b, 0xc4b, 0xc15, 0xc4d, 0xc28, 0xc4c, 0xc2e, 0xc3f,
0xc2f, 0xc3e, 0xc2a, 0xc3e, 0xc17, 0xc4b, 0x20, 0xc2a, 0xc3e, 0xc17, 0xc4b, 0xc2a,
0xc3e, 0xc32, 0xc3e, 0xc35, 0xc4d, 0xc2a, 0xc3f, 0xc1f, 0xc4d, 0x200c, 0xc15, 0xc48,
0xc30, 0xc4d, 0xc28, 0xc4d, 0xc2a, 0xc4b, 0xc28, 0xc4d, 0x200c, 0xc2a, 0xc48, 0xc2a,
0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4d, 0x20, 0xc2e, 0xc4b, 0xc30, 0xc46, 0xc38, 0xc4d,
0xc2c, 0xc47, 0xc30, 0xc30, 0xc4b, 0xc1f, 0xc4b, 0xc02, 0xc17, 0xc3e, 0xc38, 0xc3e,
0xc2f, 0xc4d, 0xc2a, 0xc3e, 0xc28, 0xc4d, 0xc24, 0xc39, 0xc3f, 0xc24, 0xc3f, 0xc1f,
0xc30, 0xc3e, 0xc35, 0xc3e, 0xc1f, 0xc4b, 0xc02, 0xc17, 0xc3e, 0xc1f, 0xc3e, 0xc2a,
0xc41, 0xc35, 0xc47, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc32, 0xc4d, 0xc32, 0xc3f, 0xc38,
0xc4d, 0xe2d, 0xe32, 0xe1a, 0xe35, 0xe08, 0xe32, 0xe19, 0xe2d, 0xe31, 0xe01, 0xe01,
0xe23, 0xe32, 0xe41, 0xe2d, 0xe14, 0xe14, 0xe34, 0xe2a, 0xe2d, 0xe32, 0xe1a, 0xe32,
0xe1a, 0xe32, 0xe41, 0xe2d, 0xe25, 0xe40, 0xe08, 0xe35, 0xe22, 0xe23, 0xe4c, 0xe41,
0xe2d, 0xe2a, 0xe21, 0xe32, 0xe23, 0xe32, 0xe1a, 0xe32, 0xe21, 0xe32, 0xe42, 0xe01,
0xe1a, 0xe31, 0xe07, 0xe01, 0xe35, 0xe1a, 0xe31, 0xe19, 0xe08, 0xe39, 0xe25, 0xe1a,
0xe34, 0xe2a, 0xe40, 0xe0b, 0xe32, 0xe41, 0xe1a, 0xe25, 0xe19, 0xe44, 0xe17, 0xe23,
0xe4c, 0xe1a, 0xe23, 0xe32, 0xe0b, 0xe0b, 0xe32, 0xe27, 0xe34, 0xe25, 0xe1a, 0xe39,
0xe08, 0xe38, 0xe21, 0xe1a, 0xe39, 0xe23, 0xe32, 0xe44, 0xe04, 0xe42, 0xe23, 0xe04,
0xe32, 0xe2a, 0xe0b, 0xe32, 0xe1a, 0xe25, 0xe32, 0xe07, 0xe01, 0xe49, 0xe32, 0xe40,
0xe0b, 0xe27, 0xe15, 0xe32, 0xe42, 0xe01, 0xe19, 0xe32, 0xe01, 0xe23, 0xe35, 0xe14,
0xe32, 0xe01, 0xe32, 0xe23, 0xe4c, 0xe14, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe2d, 0xe2a,
0xe0b, 0xe32, 0xe25, 0xe32, 0xe21, 0xe08, 0xe34, 0xe1a, 0xe39, 0xe15, 0xe35, 0xe14,
0xe39, 0xe2d, 0xe32, 0xe25, 0xe32, 0xe40, 0xe2d, 0xe25, 0xe44, 0xe2d, 0xe22, 0xe4c,
0xe2d, 0xe38, 0xe07, 0xe1f, 0xe23, 0xe35, 0xe17, 0xe32, 0xe27, 0xe19, 0xe4c, 0xe01,
0xe32, 0xe42, 0xe1a, 0xe42, 0xe23, 0xe40, 0xe19, 0xe2e, 0xe32, 0xe23, 0xe32, 0xe40,
0xe23, 0xe42, 0xe08, 0xe2e, 0xe31, 0xe19, 0xe40, 0xe19, 0xe2a, 0xe40, 0xe1a, 0xe2d,
0xe23, 0xe4c, 0xe01, 0xe08, 0xe39, 0xe1a, 0xe32, 0xe04, 0xe31, 0xe21, 0xe1e, 0xe32,
0xe25, 0xe32, 0xe04, 0xe32, 0xe23, 0xe4c, 0xe17, 0xe39, 0xe21, 0xe04, 0xe34, 0xe01,
0xe32, 0xe25, 0xe35, 0xe01, 0xe34, 0xe19, 0xe0a, 0xe32, 0xe0b, 0xe32, 0xe25, 0xe32,
0xe01, 0xe2d, 0xe2a, 0xe25, 0xe35, 0xe40, 0xe1a, 0xe23, 0xe2d, 0xe27, 0xe34, 0xe25,
0xe42, 0xe25, 0xe40, 0xe21, 0xe25, 0xe39, 0xe2d, 0xe31, 0xe19, 0xe14, 0xe32, 0xe25,
0xe39, 0xe1a, 0xe31, 0xe21, 0xe1a, 0xe32, 0xe0a, 0xe34, 0xe25, 0xe39, 0xe0b, 0xe32,
0xe01, 0xe32, 0xe21, 0xe32, 0xe25, 0xe32, 0xe42, 0xe1a, 0xe21, 0xe32, 0xe1b, 0xe39,
0xe42, 0xe15, 0xe21, 0xe32, 0xe40, 0xe0b, 0xe23, 0xe39, 0xe2d, 0xe31, 0xe21, 0xe1a,
0xe32, 0xe1a, 0xe32, 0xe40, 0xe19, 0xe42, 0xe21, 0xe01, 0xe32, 0xe14, 0xe34, 0xe0a,
0xe39, 0xe21, 0xe31, 0xe19, 0xe42, 0xe23, 0xe40, 0xe27, 0xe35, 0xe22, 0xe44, 0xe19,
0xe42, 0xe23, 0xe40, 0xe1a, 0xe35, 0xe22, 0xe40, 0xe2d, 0xe47, 0xe19, 0xe08, 0xe32,
0xe40, 0xe21, 0xe19, 0xe32, 0xe19, 0xe35, 0xe2d, 0xe32, 0xe40, 0xe21, 0xe22, 0xe4c,
0xe19, 0xe39, 0xe41, 0xe2d, 0xe01, 0xe0a, 0xe2d, 0xe15, 0xe27, 0xe32, 0xe01, 0xe32,
0xe14, 0xe39, 0xe01, 0xe39, 0xe1b, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe15, 0x2d, 0xe42,
0xe19, 0xe42, 0xe27, 0xe40, 0xe0b, 0xe32, 0xe15, 0xe39, 0xe40, 0xe21, 0xe15, 0xe23,
0xe35, 0xe42, 0xe1b, 0xe25, 0xe35, 0xe15, 0xe39, 0xe19, 0xe34, 0xe2a, 0xe27, 0xe34,
0xe19, 0xe14, 0xe4c, 0xe2e, 0xe38, 0xe01, 0xe40, 0xe2d, 0xe14, 0xe31, 0xe01, 0xe41,
0xe2d, 0xe07, 0xe40, 0xe04, 0xe2d, 0xe40, 0xe23, 0xe08, 0xe41, 0xe2d, 0xe07, 0xe01,
0xe34, 0xe25, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe19, 0xe15, 0xe34, 0xe01, 0xe32, 0xe2d,
0xe32, 0xe23, 0xe32, 0xe01, 0xe31, 0xe27, 0xe19, 0xe32, 0xe1a, 0xe31, 0xe27, 0xe42,
0xe19, 0xe2a, 0xe44, 0xe2d, 0xe40, 0xe23, 0xe2a, 0xe01, 0xe32, 0xe15, 0xe32, 0xe21,
0xe32, 0xe23, 0xe4c, 0xe01, 0xe32, 0xe04, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe14, 0xe1a,
0xe32, 0xe08, 0xe39, 0xe08, 0xe34, 0xe27, 0xe25, 0xe32, 0xe23, 0xe34, 0xe42, 0xe2d,
0xe08, 0xe32, 0xe40, 0xe21, 0xe19, 0xe14, 0xe39, 0xe0b, 0xe32, 0xe23, 0xe34, 0xe42,
0xe2d, 0xe01, 0xe32, 0xe25, 0xe40, 0xe25, 0xe01, 0xe2d, 0xe2a, 0xe0b, 0xe31, 0xe25,
0xe15, 0xe32, 0xe0b, 0xe32, 0xe19, 0xe2e, 0xe27, 0xe19, 0xe0b, 0xe31, 0xe19, 0xe25,
0xe39, 0xe2d, 0xe34, 0xe2a, 0xe17, 0xe39, 0xe04, 0xe39, 0xe41, 0xe21, 0xe19, 0xe2d,
0xe39, 0xe0a, 0xe39, 0xe40, 0xe2d, 0xe35, 0xe22, 0xe2d, 0xe32, 0xe23, 0xe39, 0xe1a,
0xe32, 0xe2d, 0xe30, 0xe0b, 0xe38, 0xe19, 0xe0b, 0xe34, 0xe2d, 0xe2d, 0xe07, 0xe04,
0xe2d, 0xe23, 0xe31, 0xe25, 0xe2e, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe1a, 0xe2d, 0xe23,
0xe4c, 0xe1a, 0xe32, 0xe40, 0xe22, 0xe35, 0xe22, 0xe1a, 0xe32, 0xe40, 0xe2d, 0xe35,
0xe22, 0xe1a, 0xe31, 0xe19, 0xe40, 0xe14, 0xe23, 0xe31, 0xe2a, 0xe1a, 0xe32, 0xe23,
0xe4c, 0xe40, 0xe1a, 0xe42, 0xe14, 0xe2a, 0xe40, 0xe1a, 0xe40, 0xe25, 0xe07, 0xe40,
0xe1a, 0xe25, 0xe35, 0xe0b, 0xe1a, 0xe25, 0xe31, 0xe07, 0xe04, 0xe4c, 0x2d, 0xe0b,
0xe32, 0xe1a, 0xe25, 0xe2d, 0xe19, 0xe1a, 0xe31, 0xe27, 0xe27, 0xe35, 0xe0a, 0xe15,
0xe32, 0xe42, 0xe1a, 0xe42, 0xe01, 0xe15, 0xe32, 0xe1a, 0xe2d, 0xe22, 0xe0b, 0xe35,
0xe2d, 0xe48, 0xe32, 0xe27, 0xe41, 0xe04, 0xe21, 0xe1a, 0xe23, 0xe34, 0xe14, 0xe08,
0xe4c, 0xe01, 0xe31, 0xe21, 0xe1b, 0xe39, 0xe01, 0xe23, 0xe31, 0xe19, 0xe14, 0xe35,
0xe41, 0xe04, 0xe19, 0xe04, 0xe38, 0xe19, 0xe04, 0xe32, 0xe23, 0xe32, 0xe04, 0xe31,
0xe2a, 0xe01, 0xe32, 0xe41, 0xe22, 0xe19, 0xe40, 0xe04, 0xe22, 0xe4c, 0xe41, 0xe21,
0xe19, 0xe0a, 0xe34, 0xe04, 0xe32, 0xe42, 0xe01, 0xe0a, 0xe35, 0xe27, 0xe32, 0xe27,
0xe32, 0xe0b, 0xe34, 0xe27, 0xe14, 0xe31, 0xe14, 0xe2e, 0xe31, 0xe27, 0xe40, 0xe23,
0xe0b, 0xe04, 0xe2d, 0xe2a, 0xe15, 0xe32, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe04,
0xe23, 0xe2a, 0xe15, 0xe31, 0xe19, 0xe01, 0xe38, 0xe22, 0xe32, 0xe1a, 0xe32, 0xe04,
0xe39, 0xe23, 0xe32, 0xe40, 0xe0b, 0xe32, 0xe14, 0xe32, 0xe19, 0xe21, 0xe32, 0xe23,
0xe4c, 0xe01, 0xe2a, 0xe2e, 0xe32, 0xe27, 0xe19, 0xe4c, 0xe14, 0xe2d, 0xe27, 0xe4c,
0xe2a, 0xe31, 0xe19, 0xe14, 0xe2d, 0xe27, 0xe4c, 0xe2a, 0xe31, 0xe19, 0x20, 0xe04,
0xe23, 0xe35, 0xe01, 0xe40, 0xe14, 0xe19, 0xe40, 0xe27, 0xe2d, 0xe23, 0xe4c, 0xe14,
0xe35, 0xe17, 0xe23, 0xe2d, 0xe22, 0xe15, 0xe4c, 0xe42, 0xe14, 0xe21, 0xe34, 0xe19,
0xe34, 0xe01, 0xe32, 0xe40, 0xe2d, 0xe14, 0xe21, 0xe31, 0xe19, 0xe15, 0xe31, 0xe19,
0xe40, 0xe2d, 0xe23, 0xe39, 0xe40, 0xe19, 0xe40, 0xe1b, 0xe40, 0xe2d, 0xe25, 0xe0b,
0xe31, 0xe25, 0xe27, 0xe32, 0xe14, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d, 0xe23, 0xe4c,
0xe15, 0xe40, 0xe19, 0xe25, 0xe2a, 0xe31, 0xe19, 0xe1f, 0xe2d, 0xe23, 0xe4c, 0xe15,
0xe32, 0xe40, 0xe25, 0xe0b, 0xe32, 0xe40, 0xe01, 0xe25, 0xe0b, 0xe40, 0xe1a, 0xe22,
0xe4c, 0xe01, 0xe39, 0xe2a, 0xe40, 0xe1a, 0xe22, 0xe4c, 0xe41, 0xe01, 0xe23, 0xe19,
0xe14, 0xe4c, 0xe40, 0xe15, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe01, 0xe23, 0xe19,
0xe32, 0xe14, 0xe32, 0xe01, 0xe27, 0xe32, 0xe40, 0xe14, 0xe2d, 0xe25, 0xe39, 0xe1b,
0xe01, 0xe31, 0xe27, 0xe40, 0xe15, 0xe21, 0xe32, 0xe25, 0xe32, 0xe01, 0xe31, 0xe27,
0xe22, 0xe32, 0xe01, 0xe34, 0xe25, 0xe01, 0xe32, 0xe22, 0xe2d, 0xe32, 0xe19, 0xe32,
0xe41, 0xe2e, 0xe25, 0xe34, 0xe41, 0xe1f, 0xe01, 0xe0b, 0xe4c, 0xe2e, 0xe32, 0xe27,
0xe32, 0xe19, 0xe32, 0xe40, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe21, 0xe0b, 0xe35, 0xe42,
0xe22, 0xe2d, 0xe34, 0xe19, 0xe40, 0xe14, 0xe35, 0xe22, 0xe41, 0xe19, 0xe42, 0xe1e,
0xe25, 0xe34, 0xe2a, 0xe19, 0xe2d, 0xe01, 0xe0b, 0xe4c, 0x2c, 0x20, 0xe2d, 0xe34,
0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19, 0xe32, 0xe21, 0xe32, 0xe40, 0xe23, 0xe07,
0xe42, 0xe01, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19,
0xe32, 0xe1b, 0xe35, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe2a, 0xe40, 0xe1a, 0xe34,
0xe23, 0xe4c, 0xe01, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d,
0xe19, 0xe32, 0xe40, 0xe17, 0xe25, 0xe25, 0xe4c, 0xe0b, 0xe34, 0xe15, 0xe35, 0x2c,
0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19, 0xe32, 0xe27, 0xe35,
0xe40, 0xe27, 0xe22, 0xe4c, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41,
0xe2d, 0xe19, 0xe32, 0xe27, 0xe34, 0xe19, 0xe40, 0xe0b, 0xe19, 0xe40, 0xe19, 0xe2a,
0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19, 0xe32, 0xe27,
0xe34, 0xe19, 0xe32, 0xe41, 0xe21, 0xe04, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14,
0xe35, 0xe41, 0xe2d, 0xe19, 0xe32, 0xe2d, 0xe34, 0xe19, 0xe39, 0xe27, 0xe34, 0xe01,
0xe2d, 0xe35, 0xe01, 0xe27, 0xe32, 0xe25, 0xe34, 0xe15, 0xe08, 0xe32, 0xe40, 0xe21,
0xe01, 0xe32, 0xe08, 0xe39, 0xe42, 0xe19, 0xe25, 0xe39, 0xe2a, 0xe4c, 0xe27, 0xe34,
0xe25, 0xe25, 0xe4c, 0xe21, 0xe2d, 0xe19, 0xe15, 0xe34, 0xe40, 0xe0b, 0xe25, 0xe42,
0xe25, 0x2c, 0x20, 0xe40, 0xe04, 0xe19, 0xe15, 0xe31, 0xe01, 0xe01, 0xe35, 0xe04,
0xe23, 0xe32, 0xe40, 0xe25, 0xe19, 0xe14, 0xe34, 0xe08, 0xe04, 0xe4c, 0xe25, 0xe32,
0xe1b, 0xe32, 0xe0b, 0xe25, 0xe34, 0xe21, 0xe32, 0xe25, 0xe2d, 0xe2a, 0xe41, 0xe2d,
0xe07, 0xe40, 0xe08, 0xe25, 0xe34, 0xe2a, 0xe42, 0xe25, 0xe40, 0xe27, 0xe2d, 0xe23,
0xe4c, 0xe1e, 0xe23, 0xe34, 0xe19, 0xe0b, 0xe4c, 0x20, 0xe04, 0xe27, 0xe2d, 0xe40,
0xe15, 0xe2d, 0xe23, 0xe4c, 0xe21, 0xe32, 0xe40, 0xe0b, 0xe42, 0xe2d, 0xe21, 0xe32,
0xe19, 0xe32, 0xe01, 0xe31, 0xe27, 0xe21, 0xe32, 0xe40, 0xe19, 0xe32, 0xe2a, 0xe4c,
0xe21, 0xe32, 0xe23, 0xe34, 0xe42, 0xe01, 0xe15, 0xe4c, 0xe21, 0xe32, 0xe23, 0xe4c,
0xe15, 0xe34, 0xe19, 0xe35, 0xe01, 0xe21, 0xe32, 0xe15, 0xe32, 0xe42, 0xe21, 0xe23,
0xe2d, 0xe2a, 0xe21, 0xe32, 0xe0b, 0xe32, 0xe17, 0xe25, 0xe32, 0xe19, 0xe40, 0xe21,
0xe42, 0xe19, 0xe21, 0xe34, 0xe19, 0xe35, 0xe40, 0xe21, 0xe23, 0xe35, 0xe14, 0xe32,
0xe40, 0xe21, 0xe17, 0xe25, 0xe32, 0xe01, 0xe32, 0xe15, 0xe25, 0xe30, 0xe40, 0xe21,
0xe47, 0xe01, 0xe0b, 0xe34, 0xe42, 0xe01, 0xe0b, 0xe34, 0xe15, 0xe35, 0xe21, 0xe35,
0xe40, 0xe01, 0xe2d, 0xe25, 0xe07, 0xe21, 0xe2d, 0xe07, 0xe15, 0xe31, 0xe19, 0xe21,
0xe2d, 0xe19, 0xe40, 0xe15, 0xe23, 0xe4c, 0xe40, 0xe23, 0xe22, 0xe4c, 0xe21, 0xe2d,
0xe19, 0xe40, 0xe15, 0xe27, 0xe34, 0xe40, 0xe14, 0xe42, 0xe2d, 0xe21, 0xe2d, 0xe19,
0xe40, 0xe0b, 0xe2d, 0xe23, 0xe4c, 0xe23, 0xe31, 0xe15, 0xe41, 0xe19, 0xe2a, 0xe0b,
0xe2d, 0xe19, 0xe34, 0xe27, 0xe22, 0xe2d, 0xe23, 0xe4c, 0xe01, 0xe19, 0xe2d, 0xe21,
0xe42, 0xe19, 0xe23, 0xe2d, 0xe19, 0xe2e, 0xe32, 0xe42, 0xe1a, 0xe25, 0xe32, 0xe2b,
0xe4c, 0x2c, 0x20, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe17, 0xe14, 0xe32, 0xe42, 0xe04,
0xe15, 0xe32, 0xe40, 0xe0b, 0xe19, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0x2c, 0x20,
0xe19, 0xe2d, 0xe23, 0xe4c, 0xe17, 0xe14, 0xe32, 0xe42, 0xe04, 0xe15, 0xe32, 0xe19,
0xe34, 0xe27, 0xe40, 0xe0b, 0xe40, 0xe25, 0xe21, 0x2c, 0x20, 0xe19, 0xe2d, 0xe23,
0xe4c, 0xe17, 0xe14, 0xe32, 0xe42, 0xe04, 0xe15, 0xe32, 0xe01, 0xe2d, 0xe14, 0xe41,
0xe18, 0xe1a, 0xe42, 0xe2d, 0xe08, 0xe34, 0xe19, 0xe32, 0xe01, 0xe32, 0xe1b, 0xe32,
0xe19, 0xe32, 0xe21, 0xe32, 0xe1b, 0xe32, 0xe23, 0xe32, 0xe21, 0xe32, 0xe23, 0xe34,
0xe42, 0xe1a, 0xe1f, 0xe34, 0xe19, 0xe34, 0xe01, 0xe0b, 0xe4c, 0xe1b, 0xe2d, 0xe23,
0xe4c, 0xe42, 0xe15, 0xe41, 0xe1b, 0xe23, 0xe07, 0xe0b, 0xe4c, 0xe1e, 0xe2d, 0xe23,
0xe4c, 0xe17, 0xe2d, 0xe2d, 0xe1f, 0xe2a, 0xe40, 0xe1b, 0xe19, 0xe1b, 0xe2d, 0xe23,
0xe4c, 0xe15, 0xe39, 0xe40, 0xe27, 0xe25, 0xe42, 0xe22, 0xe40, 0xe1b, 0xe2d, 0xe42,
0xe15, 0xe23, 0xe34, 0xe42, 0xe01, 0xe1b, 0xe38, 0xe19, 0xe15, 0xe32, 0xe2d, 0xe32,
0xe40, 0xe23, 0xe19, 0xe31, 0xe2a, 0xe41, 0xe23, 0xe07, 0xe01, 0xe34, 0xe19, 0xe2d,
0xe34, 0xe19, 0xe40, 0xe25, 0xe47, 0xe15, 0xe40, 0xe23, 0xe0b, 0xe35, 0xe40, 0xe1f,
0xe23, 0xe34, 0xe44, 0xe08, 0xe19, 0xe32, 0xe40, 0xe23, 0xe42, 0xe0b, 0xe25, 0xe39,
0xe17, 0xe23, 0xe35, 0xe42, 0xe2d, 0xe1a, 0xe23, 0xe31, 0xe07, 0xe42, 0xe01, 0xe0b,
0xe31, 0xe19, 0xe15, 0xe32, 0xe40, 0xe23, 0xe21, 0xe0b, 0xe31, 0xe19, 0xe15, 0xe34,
0xe2d, 0xe32, 0xe42, 0xe01, 0xe0b, 0xe32, 0xe19, 0xe42, 0xe15, 0x20, 0xe42, 0xe14,
0xe21, 0xe34, 0xe07, 0xe42, 0xe01, 0xe40, 0xe0b, 0xe32, 0xe40, 0xe1b, 0xe32, 0xe25,
0xe39, 0xe2a, 0xe01, 0xe2d, 0xe40, 0xe23, 0xe2a, 0xe44, 0xe1a, 0xe0b, 0xe31, 0xe19,
0xe14, 0xe4c, 0xe0b, 0xe34, 0xe15, 0xe01, 0xe32, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c,
0xe1a, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe18, 0xe40, 0xe25, 0xe21, 0xe35, 0xe40, 0xe0b,
0xe19, 0xe15, 0xe4c, 0xe08, 0xe2d, 0xe19, 0xe2a, 0xe4c, 0xe40, 0xe0b, 0xe19, 0xe15,
0xe4c, 0xe04, 0xe34, 0xe15, 0xe2a, 0xe4c, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe25,
0xe39, 0xe40, 0xe0b, 0xe35, 0xe22, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe42, 0xe18,
0xe21, 0xe31, 0xe2a, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe27, 0xe34, 0xe19, 0xe40,
0xe0b, 0xe19, 0xe15, 0xe4c, 0xe2a, 0xe27, 0xe34, 0xe1f, 0xe15, 0xe4c, 0xe40, 0xe04,
0xe2d, 0xe23, 0xe4c, 0xe40, 0xe23, 0xe19, 0xe15, 0xe4c, 0xe40, 0xe15, 0xe01, 0xe39,
0xe0b, 0xe34, 0xe01, 0xe31, 0xe25, 0xe1b, 0xe32, 0xe17, 0xe39, 0xe40, 0xe25, 0xe17,
0xe34, 0xe2e, 0xe31, 0xe27, 0xe19, 0xe32, 0xe42, 0xe17, 0xe23, 0xe2d, 0xe19, 0xe42,
0xe15, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe15, 0xe25, 0xe32, 0xe41, 0xe27, 0xe19,
0xe04, 0xe39, 0xe40, 0xe27, 0xe2d, 0xe23, 0xe4c, 0xe44, 0xe27, 0xe15, 0xe4c, 0xe2e,
0xe2d, 0xe23, 0xe4c, 0xe2a, 0xe27, 0xe34, 0xe19, 0xe19, 0xe34, 0xe40, 0xe1e, 0xe01,
0xe22, 0xe32, 0xe01, 0xe39, 0xe17, 0xe31, 0xe15, 0xe40, 0xe04, 0xe0b, 0xe35, 0xe22,
0xe4c, 0xe40, 0xe14, 0xe27, 0xe34, 0xe2a, 0xe14, 0xe39, 0xe21, 0xe2d, 0xe07, 0xe15,
0xe4c, 0xe14, 0xe39, 0xe23, 0xe4c, 0xe27, 0xe34, 0xe25, 0xe25, 0xe4c, 0xe41, 0xe21,
0xe04, 0xe04, 0xe27, 0xe2d, 0xe23, 0xe35, 0xe21, 0xe2d, 0xe27, 0xe4c, 0xe2a, 0xe31,
0xe19, 0xe41, 0xe21, 0xe04, 0xe21, 0xe31, 0xe27, 0xe42, 0xe14, 0xe1e, 0xe32, 0xe25,
0xe4c, 0xe40, 0xe21, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe23, 0xe18, 0xe35, 0xe23, 0xe32,
0xe44, 0xe0b, 0xe42, 0xe22, 0xe27, 0xe32, 0xe42, 0xe17, 0xe23, 0xe25, 0xe27, 0xe2d,
0xe2a, 0xe15, 0xe2d, 0xe04, 0xe25, 0xe2d, 0xe07, 0xe40, 0xe22, 0xe35, 0xe22, 0xe23,
0xe4c, 0xe40, 0xe1a, 0xe35, 0xe22, 0xe19, 0xe40, 0xe2d, 0xe40, 0xe14, 0xe19, 0xe2d,
0xe31, 0xe25, 0xe21, 0xe32, 0xe15, 0xe35, 0xe2d, 0xe31, 0xe21, 0xe21, 0xe32, 0xe19,
0xe2d, 0xe32, 0xe19, 0xe32, 0xe14, 0xe35, 0xe23, 0xe4c, 0xe2d, 0xe31, 0xe04, 0xe15,
0xe32, 0xe2d, 0xe39, 0xe2d, 0xe31, 0xe04, 0xe42, 0xe17, 0xe1a, 0xe35, 0xe2d, 0xe32,
0xe0a, 0xe01, 0xe32, 0xe1a, 0xe31, 0xe15, 0xe2d, 0xe17, 0xe35, 0xe23, 0xe32, 0xe27,
0xe41, 0xe1a, 0xe01, 0xe41, 0xe14, 0xe14, 0xe1a, 0xe32, 0xe2b, 0xe4c, 0xe40, 0xe23,
0xe19, 0xe1a, 0xe32, 0xe01, 0xe39, 0xe01, 0xe23, 0xe38, 0xe07, 0xe40, 0xe17, 0xe1e,
0xe1a, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe19, 0xe32, 0xe27, 0xe4c, 0xe40, 0xe1a, 0xe23,
0xe38, 0xe15, 0xe1a, 0xe34, 0xe0a, 0xe40, 0xe04, 0xe01, 0xe1a, 0xe23, 0xe39, 0xe44,
0xe19, 0xe0a, 0xe34, 0xe15, 0xe32, 0xe42, 0xe04, 0xe25, 0xe31, 0xe21, 0xe42, 0xe1a,
0xe14, 0xe32, 0xe21, 0xe31, 0xe2a, 0xe01, 0xe31, 0xe2a, 0xe14, 0xe34, 0xe25, 0xe35,
0xe14, 0xe39, 0xe44, 0xe1a, 0xe14, 0xe39, 0xe0a, 0xe32, 0xe19, 0xe40, 0xe1a, 0xe41,
0xe1f, 0xe21, 0xe32, 0xe01, 0xe38, 0xe2a, 0xe15, 0xe32, 0xe01, 0xe32, 0xe0b, 0xe32,
0xe40, 0xe2e, 0xe1a, 0xe23, 0xe2d, 0xe19, 0xe19, 0xe04, 0xe23, 0xe42, 0xe2e, 0xe08,
0xe34, 0xe21, 0xe34, 0xe19, 0xe2b, 0xe4c, 0xe2e, 0xe48, 0xe2d, 0xe07, 0xe01, 0xe07,
0xe2e, 0xe2d, 0xe1f, 0xe14, 0xe4c, 0xe2d, 0xe35, 0xe23, 0xe4c, 0xe04, 0xe38, 0xe15,
0xe2a, 0xe04, 0xe4c, 0xe08, 0xe32, 0xe01, 0xe32, 0xe23, 0xe4c, 0xe15, 0xe32, 0xe08,
0xe32, 0xe22, 0xe32, 0xe1b, 0xe38, 0xe23, 0xe30, 0xe40, 0xe22, 0xe23, 0xe39, 0xe0b,
0xe32, 0xe40, 0xe25, 0xe21, 0xe04, 0xe32, 0xe1a, 0xe39, 0xe25, 0xe04, 0xe32, 0xe21,
0xe0a, 0xe31, 0xe15, 0xe01, 0xe32, 0xe01, 0xe32, 0xe23, 0xe32, 0xe08, 0xe35, 0xe01,
0xe32, 0xe15, 0xe21, 0xe31, 0xe19, 0xe14, 0xe38, 0xe2e, 0xe31, 0xe19, 0xe14, 0xe37,
0xe22, 0xe01, 0xe32, 0xe42, 0xe01, 0xe25, 0xe01, 0xe32, 0xe15, 0xe32, 0xe04, 0xe23,
0xe31, 0xe2a, 0xe42, 0xe19, 0xe22, 0xe32, 0xe23, 0xe4c, 0xe2a, 0xe01, 0xe4c, 0xe01,
0xe31, 0xe27, 0xe25, 0xe32, 0xe25, 0xe31, 0xe21, 0xe40, 0xe1b, 0xe2d, 0xe23, 0xe4c,
0xe01, 0xe39, 0xe0a, 0xe34, 0xe07, 0xe04, 0xe39, 0xe40, 0xe27, 0xe15, 0xe21, 0xe32,
0xe40, 0xe01, 0xe4a, 0xe32, 0xe21, 0xe32, 0xe01, 0xe32, 0xe14, 0xe32, 0xe19, 0xe21,
0xe32, 0xe01, 0xe31, 0xe2a, 0xe0b, 0xe32, 0xe23, 0xe4c, 0xe21, 0xe30, 0xe19, 0xe34,
0xe25, 0xe32, 0xe21, 0xe31, 0xe2a, 0xe01, 0xe31, 0xe15, 0xe19, 0xe34, 0xe42, 0xe04,
0xe40, 0xe0b, 0xe35, 0xe22, 0xe42, 0xe19, 0xe42, 0xe27, 0xe04, 0xe38, 0xe0b, 0xe40,
0xe19, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe42, 0xe19, 0xe42, 0xe27, 0xe0b, 0xe34, 0xe1a,
0xe34, 0xe23, 0xe4c, 0xe2a, 0xe01, 0xe4c, 0xe42, 0xe2d, 0xe21, 0xe2a, 0xe01, 0xe4c,
0xe2d, 0xe2d, 0xe23, 0xe31, 0xe25, 0xe1e, 0xe19, 0xe21, 0xe40, 0xe1b, 0xe0d, 0xe1e,
0xe2d, 0xe19, 0xe40, 0xe17, 0xe35, 0xe22, 0xe19, 0xe31, 0xe01, 0xe40, 0xe1b, 0xe35,
0xe22, 0xe07, 0xe22, 0xe32, 0xe07, 0xe01, 0xe32, 0xe15, 0xe32, 0xe23, 0xe4c, 0xe04,
0xe2d, 0xe2a, 0xe15, 0xe32, 0xe40, 0xe19, 0xe22, 0xe4c, 0xe44, 0xe04, 0xe0b, 0xe35,
0xe25, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe32, 0xe23, 0xe34, 0xe22, 0xe32, 0xe23, 0xe4c,
0xe14, 0xe0b, 0xe32, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19, 0xe0b, 0xe32, 0xe21, 0xe32,
0xe23, 0xe4c, 0xe01, 0xe32, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe0b, 0xe35, 0xe48, 0xe22,
0xe07, 0xe44, 0xe2e, 0xe49, 0xe2a, 0xe34, 0xe07, 0xe04, 0xe42, 0xe1b, 0xe23, 0xe4c,
0xe0b, 0xe40, 0xe23, 0xe14, 0xe40, 0xe19, 0xe04, 0xe42, 0xe04, 0xe25, 0xe34, 0xe21,
0xe2a, 0xe01, 0xe4c, 0xe44, 0xe17, 0xe40, 0xe1b, 0xe17, 0xe32, 0xe0a, 0xe40, 0xe04,
0xe19, 0xe15, 0xe4c, 0xe17, 0xe1a, 0xe34, 0xe25, 0xe34, 0xe0b, 0xe34, 0xe40, 0xe15,
0xe2b, 0xe30, 0xe23, 0xe32, 0xe19, 0xe17, 0xe34, 0xe21, 0xe1e, 0xe39, 0xe42, 0xe15,
0xe40, 0xe01, 0xe35, 0xe22, 0xe27, 0xe15, 0xe2d, 0xe21, 0xe2a, 0xe04, 0xe4c, 0xe2d,
0xe39, 0xe25, 0xe32, 0xe19, 0xe1a, 0xe32, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe2d, 0xe38,
0xe23, 0xe38, 0xe21, 0xe0a, 0xe35, 0xe2d, 0xe38, 0xe2a, 0xe15, 0xe4c, 0xe40, 0xe19,
0xe23, 0xe32, 0xe40, 0xe27, 0xe35, 0xe22, 0xe07, 0xe08, 0xe31, 0xe19, 0xe17, 0xe19,
0xe4c, 0xe27, 0xe25, 0xe32, 0xe14, 0xe34, 0xe42, 0xe27, 0xe2a, 0xe15, 0xe47, 0xe2d,
0xe01, 0xe22, 0xe32, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe22, 0xe48, 0xe32,
0xe07, 0xe01, 0xe38, 0xe49, 0xe07, 0xe22, 0xe35, 0xe04, 0xe32, 0xe40, 0xe15, 0xe2d,
0xe23, 0xe34, 0xe19, 0xe40, 0xe1a, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe22, 0xe40,
0xe23, 0xe27, 0xe32, 0xe19, 0xe2d, 0xe30, 0xe42, 0xe0b, 0xe23, 0xe4c, 0xe2a, 0xe40,
0xe1a, 0xe2d, 0xe23, 0xe4c, 0xe21, 0xe34, 0xe27, 0xe14, 0xe32, 0xe04, 0xe30, 0xe40,
0xe19, 0xe23, 0xe35, 0xe40, 0xe04, 0xe1b, 0xe40, 0xe27, 0xe34, 0xe23, 0xe4c, 0xe14,
0xe41, 0xe1f, 0xe42, 0xe23, 0xe21, 0xe32, 0xe40, 0xe14, 0xe23, 0xe32, 0xe40, 0xe23,
0xe04, 0xe22, 0xe32, 0xe27, 0xe34, 0xe01, 0xe40, 0xe0b, 0xe32, 0xe17, 0xe4c, 0x20,
0xe08, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe35, 0xe22, 0xe40, 0xe0b, 0xe19, 0xe15,
0xe4c, 0xe40, 0xe2e, 0xe40, 0xe25, 0xe19, 0xe32, 0xe2a, 0xe41, 0xe15, 0xe19, 0xe25,
0xe35, 0xe22, 0xe4c, 0xe41, 0xe2d, 0xe14, 0xe34, 0xe40, 0xe25, 0xe14, 0xe1a, 0xe23,
0xe34, 0xe2a, 0xe40, 0xe1a, 0xe19, 0xe42, 0xe1a, 0xe23, 0xe01, 0xe40, 0xe04, 0xe19,
0xe2e, 0xe34, 0xe25, 0xe25, 0xe4c, 0xe14, 0xe32, 0xe23, 0xe4c, 0xe27, 0xe34, 0xe19,
0xe22, 0xe39, 0xe04, 0xe25, 0xe32, 0xe42, 0xe2e, 0xe1a, 0xe32, 0xe23, 0xe4c, 0xe15,
0xe25, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe21, 0xe19, 0xe25, 0xe2d, 0xe23, 0xe4c,
0xe14, 0xe42, 0xe2e, 0xe27, 0xe4c, 0xe40, 0xe21, 0xe25, 0xe40, 0xe1a, 0xe34, 0xe23,
0xe4c, 0xe19, 0xe40, 0xe1e, 0xe34, 0xe23, 0xe4c, 0xe17, 0xe0b, 0xe34, 0xe14, 0xe19,
0xe35, 0xe22, 0xe4c, 0xe2d, 0xe31, 0xe21, 0xe2a, 0xe40, 0xe15, 0xe2d, 0xe14, 0xe31,
0xe21, 0xe2d, 0xe31, 0xe19, 0xe14, 0xe2d, 0xe23, 0xe4c, 0xe23, 0xe32, 0xe41, 0xe2d,
0xe2a, 0xe15, 0xe23, 0xe32, 0xe04, 0xe32, 0xe19, 0xe40, 0xe2d, 0xe40, 0xe18, 0xe19,
0xe2a, 0xe4c, 0xe40, 0xe1a, 0xe25, 0xe40, 0xe01, 0xe23, 0xe14, 0xe40, 0xe1a, 0xe2d,
0xe23, 0xe4c, 0xe25, 0xe34, 0xe19, 0xe1a, 0xe23, 0xe32, 0xe15, 0xe34, 0xe2a, 0xe25,
0xe32, 0xe27, 0xe32, 0xe1a, 0xe23, 0xe31, 0xe2a, 0xe40, 0xe0b, 0xe25, 0xe2a, 0xe4c,
0xe1a, 0xe39, 0xe04, 0xe32, 0xe40, 0xe23, 0xe2a, 0xe1a, 0xe39, 0xe14, 0xe32, 0xe40,
0xe1b, 0xe2a, 0xe1a, 0xe38, 0xe2a, 0xe0b, 0xe34, 0xe07, 0xe40, 0xe07, 0xe47, 0xe19,
0xe04, 0xe35, 0xe0a, 0xe35, 0xe40, 0xe19, 0xe32, 0xe42, 0xe04, 0xe40, 0xe1b, 0xe19,
0xe40, 0xe2e, 0xe40, 0xe01, 0xe19, 0xe14, 0xe31, 0xe1a, 0xe1a, 0xe25, 0xe34, 0xe19,
0xe22, 0xe34, 0xe1a, 0xe23, 0xe2d, 0xe25, 0xe15, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe01,
0xe34, 0xe23, 0xe4c, 0xe19, 0xe0b, 0xe35, 0xe22, 0xe4c, 0xe40, 0xe2e, 0xe25, 0xe0b,
0xe34, 0xe07, 0xe01, 0xe34, 0xe40, 0xe01, 0xe32, 0xe30, 0xe41, 0xe21, 0xe19, 0xe2d,
0xe34, 0xe2a, 0xe15, 0xe31, 0xe19, 0xe1a, 0xe39, 0xe25, 0xe40, 0xe08, 0xe2d, 0xe23,
0xe4c, 0xe0b, 0xe35, 0xe22, 0xe4c, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19, 0xe34, 0xe07,
0xe01, 0xe23, 0xe32, 0xe14, 0xe04, 0xe34, 0xe23, 0xe2d, 0xe1f, 0xe40, 0xe04, 0xe35,
0xe22, 0xe1f, 0xe25, 0xe34, 0xe2a, 0xe1a, 0xe2d, 0xe19, 0xe25, 0xe39, 0xe1a, 0xe25,
0xe34, 0xe22, 0xe32, 0xe19, 0xe32, 0xe25, 0xe2d, 0xe19, 0xe14, 0xe2d, 0xe19, 0xe25,
0xe31, 0xe01, 0xe40, 0xe0b, 0xe21, 0xe40, 0xe1a, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe21,
0xe32, 0xe14, 0xe23, 0xe34, 0xe14, 0xe21, 0xe2d, 0xe25, 0xe15, 0xe32, 0xe21, 0xe32,
0xe23, 0xe35, 0xe2e, 0xe32, 0xe21, 0xe19, 0xe4c, 0xe21, 0xe34, 0xe19, 0xe2a, 0xe01,
0xe4c, 0xe42, 0xe21, 0xe19, 0xe32, 0xe42, 0xe01, 0xe21, 0xe2d, 0xe2a, 0xe42, 0xe01,
0xe2d, 0xe2d, 0xe2a, 0xe42, 0xe25, 0xe1b, 0xe32, 0xe23, 0xe35, 0xe2a, 0xe1e, 0xe2d,
0xe14, 0xe01, 0xe2d, 0xe23, 0xe35, 0xe15, 0xe0b, 0xe32, 0xe1b, 0xe23, 0xe32, 0xe01,
0xe0b, 0xe32, 0xe21, 0xe32, 0xe23, 0xe32, 0xe0b, 0xe32, 0xe19, 0xe21, 0xe32, 0xe23,
0xe35, 0xe42, 0xe19, 0xe0b, 0xe32, 0xe23, 0xe32, 0xe40, 0xe22, 0xe42, 0xe27, 0xe0b,
0xe32, 0xe23, 0xe32, 0xe17, 0xe2d, 0xe1f, 0xe0b, 0xe34, 0xe21, 0xe40, 0xe1f, 0xe2d,
0xe42, 0xe23, 0xe42, 0xe1b, 0xe25, 0xe2a, 0xe42, 0xe01, 0xe40, 0xe1b, 0xe35, 0xe22,
0xe42, 0xe0b, 0xe40, 0xe1f, 0xe35, 0xe22, 0xe2a, 0xe15, 0xe2d, 0xe01, 0xe42, 0xe2e,
0xe25, 0xe4c, 0xe21, 0xe17, 0xe32, 0xe25, 0xe25, 0xe34, 0xe19, 0xe19, 0xe4c, 0xe15,
0xe34, 0xe23, 0xe32, 0xe19, 0xe32, 0xe2d, 0xe30, 0xe25, 0xe34, 0xe22, 0xe32, 0xe19,
0xe2d, 0xe1f, 0xe27, 0xe32, 0xe14, 0xe38, 0xe0b, 0xe27, 0xe32, 0xe15, 0xe34, 0xe01,
0xe31, 0xe19, 0xe40, 0xe27, 0xe35, 0xe22, 0xe19, 0xe19, 0xe32, 0xe27, 0xe34, 0xe25,
0xe19, 0xe35, 0xe2d, 0xe38, 0xe2a, 0xe27, 0xe2d, 0xe25, 0xe42, 0xe01, 0xe01, 0xe23,
0xe32, 0xe14, 0xe27, 0xe2d, 0xe23, 0xe4c, 0xe0b, 0xe2d, 0xe0b, 0xe32, 0xe40, 0xe01,
0xe23, 0xe47, 0xe1a, 0xe0b, 0xe39, 0xe23, 0xe34, 0xe04, 0xe2d, 0xe31, 0xe19, 0xe15,
0xe32, 0xe19, 0xe32, 0xe19, 0xe32, 0xe23, 0xe34, 0xe42, 0xe27, 0xe0a, 0xe32, 0xe01,
0xe31, 0xe2a, 0xe04, 0xe23, 0xe34, 0xe2a, 0xe15, 0xe4c, 0xe21, 0xe32, 0xe2a, 0xe42,
0xe04, 0xe42, 0xe04, 0xe2a, 0xe42, 0xe04, 0xe42, 0xe21, 0xe42, 0xe23, 0xe41, 0xe01,
0xe23, 0xe4c, 0xe40, 0xe01, 0xe2d, 0xe25, 0xe2d, 0xe07, 0xe21, 0xe32, 0xe40, 0xe2d,
0xe21, 0xe31, 0xe25, 0xe14, 0xe35, 0xe1f, 0xe2a, 0xe4c, 0xe21, 0xe2d, 0xe23, 0xe34,
0xe40, 0xe0a, 0xe35, 0xe22, 0xe2a, 0xe21, 0xe32, 0xe42, 0xe22, 0xe40, 0xe15, 0xe40,
0xe23, 0xe2d, 0xe39, 0xe19, 0xe35, 0xe22, 0xe07, 0xe2d, 0xe32, 0xe1b, 0xe35, 0xe2d,
0xe32, 0xe42, 0xe2d, 0xe04, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe1a, 0xe39, 0xe40,
0xe01, 0xe19, 0xe27, 0xe34, 0xe25, 0xe25, 0xe4c, 0xe41, 0xe0a, 0xe17, 0xe31, 0xe21,
0xe17, 0xe23, 0xe31, 0xe01, 0xe2d, 0xe35, 0xe2a, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c,
0xe40, 0xe2d, 0xe1f, 0xe32, 0xe40, 0xe15, 0xe1f, 0xe32, 0xe40, 0xe04, 0xe32, 0xe42,
0xe1f, 0xe1f, 0xe34, 0xe08, 0xe34, 0xe1f, 0xe39, 0xe19, 0xe30, 0xe1f, 0xe39, 0xe15,
0xe35, 0xe01, 0xe32, 0xe25, 0xe32, 0xe1b, 0xe32, 0xe42, 0xe01, 0xe2a, 0xe41, 0xe01,
0xe21, 0xe40, 0xe1a, 0xe35, 0xe22, 0xe23, 0xe4c, 0xe01, 0xe31, 0xe27, 0xe14, 0xe31,
0xe25, 0xe04, 0xe32, 0xe19, 0xe31, 0xe25, 0xe01, 0xe27, 0xe21, 0xe42, 0xe2e, 0xe42,
0xe19, 0xe25, 0xe39, 0xe25, 0xe39, 0xe40, 0xe2d, 0xe19, 0xe40, 0xe14, 0xe2d, 0xe23,
0xe4c, 0xe40, 0xe1a, 0xe2d, 0xe23, 0xe35, 0xe04, 0xe34, 0xe23, 0xe34, 0xe17, 0xe34,
0xe21, 0xe32, 0xe15, 0xe35, 0xe04, 0xe2d, 0xe2a, 0xe44, 0xe23, 0xe04, 0xe27, 0xe32,
0xe08, 0xe32, 0xe40, 0xe25, 0xe19, 0xe21, 0xe32, 0xe08, 0xe39, 0xe42, 0xe23, 0xe21,
0xe32, 0xe23, 0xe4c, 0xe40, 0xe04, 0xe0b, 0xe31, 0xe2a, 0xe21, 0xe34, 0xe14, 0xe40,
0xe27, 0xe22, 0xe4c, 0xe19, 0xe32, 0xe2d, 0xe39, 0xe23, 0xe39, 0xe19, 0xe35, 0xe2d,
0xe39, 0xe40, 0xe2d, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01,
0xe19, 0xe39, 0xe40, 0xe21, 0xe2d, 0xe32, 0xe1b, 0xe32, 0xe42, 0xe01, 0x20, 0xe1b,
0xe32, 0xe42, 0xe01, 0xe1b, 0xe32, 0xe40, 0xe25, 0xe32, 0xe1e, 0xe34, 0xe15, 0xe41,
0xe04, 0xe23, 0xe4c, 0xe19, 0xe42, 0xe1b, 0xe19, 0xe32, 0xe40, 0xe1b, 0xe1e, 0xe2d,
0xe23, 0xe4c, 0xe15, 0xe21, 0xe2d, 0xe23, 0xe4c, 0xe2a, 0xe1a, 0xe35, 0xe23, 0xe32,
0xe42, 0xe23, 0xe15, 0xe2d, 0xe07, 0xe01, 0xe32, 0xe44, 0xe0b, 0xe1b, 0xe31, 0xe19,
0xe15, 0xe32, 0xe2e, 0xe35, 0xe15, 0xe35, 0xe15, 0xe32, 0xe23, 0xe30, 0xe27, 0xe32,
0xe15, 0xe2d, 0xe07, 0xe01, 0xe32, 0xe15, 0xe32, 0xe1b, 0xe39, 0xe40, 0xe27, 0xe01,
0xe27, 0xe32, 0xe25, 0xe25, 0xe34, 0xe2a, 0x12a3, 0x1262, 0x1303, 0x1295, 0x12a3, 0x12ad,
0x122b, 0x12a3, 0x12f2, 0x1235, 0x20, 0x12a3, 0x1260, 0x1263, 0x12a3, 0x120d, 0x1300, 0x122d,
0x1235, 0x12a3, 0x1235, 0x1218, 0x122b, 0x1263, 0x1295, 0x130a, 0x1262, 0x1233, 0x12cd, 0x1261,
0x1301, 0x121d, 0x1261, 0x122b, 0x1234, 0x12cd, 0x1273, 0x12ae, 0x1293, 0x12ad, 0x122a, 0x12f3,
0x122d, 0x20, 0x12a4, 0x1235, 0x20, 0x1233, 0x120b, 0x121d, 0x12f1, 0x12cb, 0x120b, 0x12a4,
0x120d, 0x20, 0x12a3, 0x12e9, 0x1295, 0x1203, 0x122b, 0x1228, 0x120a, 0x1265, 0x1228, 0x126a,
0x120d, 0x120e, 0x1218, 0x1209, 0x12cb, 0x1295, 0x12f3, 0x121b, 0x120b, 0x1266, 0x121b, 0x1351,
0x1276, 0x121b, 0x1230, 0x1229, 0x121d, 0x1263, 0x1263, 0x1290, 0x121e, 0x1253, 0x12f5, 0x123e,
0x121e, 0x1295, 0x122e, 0x126d, 0x12eb, 0x1295, 0x1303, 0x1218, 0x1293, 0x1295, 0x12eb, 0x1218,
0x12ed, 0x1295, 0x12cb, 0x12ad, 0x123e, 0x1275, 0x12cb, 0x130b, 0x12f1, 0x1309, 0x1356, 0x122d,
0x1276, 0x20, 0x1296, 0x126e, 0x1233, 0x12a6, 0x20, 0x1276, 0x1218, 0x1271, 0x1292, 0x1235,
0x12a3, 0x12f3, 0x12ad, 0x12a3, 0x1295, 0x12ae, 0x1228, 0x1305, 0x12a3, 0x1295, 0x1312, 0x120b,
0x12a3, 0x1295, 0x1272, 0x1313, 0x12a3, 0x122b, 0x1313, 0x12ed, 0x1293, 0x1265, 0x12c8, 0x1296,
0x1235, 0x20, 0x12a3, 0x12ed, 0x1228, 0x1235, 0x1201, 0x1201, 0x12ed, 0x120b, 0x20, 0x122d,
0x12ee, 0x1203, 0x1218, 0x1295, 0x12f6, 0x12db, 0x122d, 0x12ee, 0x20, 0x130b, 0x12e8, 0x130e,
0x1235, 0x1233, 0x1295, 0x20, 0x1205, 0x12cb, 0x1295, 0x1233, 0x1295, 0x20, 0x120d, 0x12ca,
0x1235, 0x12a1, 0x123d, 0x12cb, 0x12eb, 0x12a3, 0x1229, 0x1263, 0x12a3, 0x1231, 0x1295, 0x1235,
0x12ee, 0x1295, 0x12a3, 0x1272, 0x12ae, 0x12ab, 0x1295, 0x1263, 0x1205, 0x12eb, 0x1263, 0x12a5,
0x12eb, 0x20, 0x12f0, 0x20, 0x1263, 0x1295, 0x12f0, 0x122b, 0x1235, 0x1263, 0x122d, 0x1263,
0x12f6, 0x1235, 0x1260, 0x1208, 0x121d, 0x1260, 0x120a, 0x12dd, 0x1265, 0x120b, 0x1295, 0x12ad,
0x2d, 0x1233, 0x1265, 0x120e, 0x1295, 0x1266, 0x12cb, 0x20, 0x126a, 0x1235, 0x1273, 0x1266,
0x12ed, 0x12da, 0x12ab, 0x121d, 0x1265, 0x122a, 0x1305, 0x20, 0x1260, 0x12ed, 0x12ab, 0x121d,
0x1356, 0x20, 0x130d, 0x122b, 0x1295, 0x12f0, 0x12ab, 0x12e8, 0x1295, 0x127a, 0x12cb, 0x12cb,
0x1232, 0x12e9, 0x12f3, 0x12f5, 0x20, 0x1301, 0x12cb, 0x1228, 0x12dd, 0x12ae, 0x1235, 0x1273,
0x20, 0x122a, 0x12ab, 0x12a9, 0x12eb, 0x1263, 0x12a9, 0x122b, 0x1233, 0x12cd, 0x12f3, 0x12cd,
0x1230, 0x1295, 0x20, 0x12ad, 0x122a, 0x12ad, 0x12f0, 0x1295, 0x1268, 0x122d, 0x12f6, 0x121a,
0x1292, 0x12ab, 0x12a4, 0x12ed, 0x1229, 0x1294, 0x1352, 0x12a4, 0x120d, 0x20, 0x1233, 0x120d,
0x126b, 0x12f6, 0x122d, 0x134e, 0x122d, 0x1275, 0x20, 0x1290, 0x120d, 0x1230, 0x1295, 0x134e,
0x122d, 0x1273, 0x1208, 0x12db, 0x130d, 0x120c, 0x1235, 0x20, 0x1260, 0x12ed, 0x1309, 0x12dd,
0x20, 0x1260, 0x12ed, 0x130d, 0x122b, 0x1295, 0x12f5, 0x20, 0x1271, 0x122d, 0x12ad, 0x130d,
0x1228, 0x1293, 0x12f3, 0x1313, 0x12f0, 0x1209, 0x1355, 0x1313, 0x12eb, 0x12aa, 0x120d, 0x12a4,
0x122d, 0x121e, 0x1235, 0x12ee, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x1356, 0x120a, 0x1235,
0x1296, 0x12ad, 0x1235, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x121b, 0x1228,
0x1295, 0x130e, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x1352, 0x1270, 0x122d,
0x1235, 0x1260, 0x122d, 0x130d, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x1270,
0x120d, 0x20, 0x1232, 0x1272, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x126a,
0x126a, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x126a, 0x1295, 0x1230, 0x1295,
0x1235, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x12ca, 0x1293, 0x121b, 0x12ad,
0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x12a2, 0x1243, 0x120d, 0x12ca, 0x1275,
0x1301, 0x1290, 0x12cd, 0x120d, 0x12ca, 0x126a, 0x120d, 0x121e, 0x1295, 0x1272, 0x1278, 0x120e,
0x1363, 0x20, 0x12a8, 0x1295, 0x1273, 0x12aa, 0x12ad, 0x122b, 0x1208, 0x1295, 0x12f3, 0x12ed,
0x12ad, 0x120e, 0x1235, 0x20, 0x12a3, 0x1295, 0x1300, 0x1208, 0x1235, 0x1208, 0x12c8, 0x122d,
0x20, 0x1355, 0x122a, 0x1295, 0x1230, 0x1235, 0x20, 0x12b3, 0x122d, 0x1270, 0x122d, 0x121b,
0x1230, 0x12ee, 0x121b, 0x1293, 0x1313, 0x121b, 0x1293, 0x12cd, 0x1235, 0x121b, 0x122a, 0x130e,
0x1275, 0x1218, 0x122a, 0x12f3, 0x12a8, 0x1270, 0x121b, 0x20, 0x121c, 0x12ad, 0x1232, 0x12ae,
0x121a, 0x12a8, 0x120e, 0x1295, 0x121e, 0x1295, 0x12ad, 0x1270, 0x1295, 0x121e, 0x1295, 0x1270,
0x1228, 0x12ed, 0x121e, 0x1295, 0x1270, 0x126a, 0x12f0, 0x12ee, 0x121e, 0x1295, 0x1275, 0x1230,
0x122b, 0x1275, 0x1292, 0x12cd, 0x20, 0x12ee, 0x122d, 0x12ad, 0x1290, 0x12cd, 0x121d, 0x1296,
0x122e, 0x1295, 0x12eb, 0x1265, 0x12e9, 0x120b, 0x1363, 0x20, 0x1230, 0x121c, 0x1295, 0x20,
0x12f3, 0x12ae, 0x1273, 0x1230, 0x1295, 0x1270, 0x122d, 0x1363, 0x20, 0x1230, 0x121c, 0x1295,
0x20, 0x12f3, 0x12ae, 0x1273, 0x1292, 0x12cd, 0x20, 0x1233, 0x1208, 0x121d, 0x1363, 0x20,
0x1230, 0x121c, 0x1295, 0x20, 0x12f3, 0x12ae, 0x1273, 0x1291, 0x12ad, 0x12a6, 0x1302, 0x1293,
0x130b, 0x1356, 0x122d, 0x1275, 0x2d, 0x12a6, 0x2d, 0x1355, 0x122a, 0x1295, 0x1235, 0x1356,
0x122d, 0x1275, 0x20, 0x12a6, 0x134d, 0x20, 0x1235, 0x1354, 0x1295, 0x1356, 0x122d, 0x1276,
0x20, 0x1268, 0x120d, 0x12ee, 0x1356, 0x122d, 0x1276, 0x20, 0x122a, 0x12ae, 0x1351, 0x1295,
0x1273, 0x20, 0x12a3, 0x1228, 0x1293, 0x1235, 0x122b, 0x1295, 0x12aa, 0x1295, 0x20, 0x12a2,
0x1295, 0x1208, 0x1275, 0x1228, 0x1232, 0x1348, 0x1228, 0x12de, 0x1209, 0x1275, 0x122d, 0x12ee,
0x20, 0x1265, 0x122b, 0x1295, 0x12ae, 0x1233, 0x1295, 0x1273, 0x1228, 0x121d, 0x1233, 0x1295,
0x1275, 0x12eb, 0x130e, 0x1233, 0x12a6, 0x20, 0x1353, 0x12cd, 0x120e, 0x12a2, 0x1276, 0x1246,
0x122d, 0x1276, 0x121a, 0x1275, 0x1245, 0x12f1, 0x1235, 0x20, 0x1263, 0x122d, 0x1270, 0x1208,
0x121a, 0x1245, 0x12f1, 0x1235, 0x20, 0x12ee, 0x1203, 0x1295, 0x1235, 0x1245, 0x12f5, 0x1235,
0x1272, 0x20, 0x1209, 0x1235, 0x12eb, 0x1235, 0x12ca, 0x134d, 0x1275, 0x20, 0x12ab, 0x1228,
0x1295, 0x1275, 0x1270, 0x1309, 0x1232, 0x130b, 0x120d, 0x1353, 0x12d9, 0x120d, 0x1272, 0x1305,
0x12cb, 0x1293, 0x126b, 0x1295, 0x12a9, 0x1268, 0x122d, 0x12cb, 0x12ed, 0x1275, 0x1206, 0x12dd,
0x12ca, 0x1292, 0x1350, 0x130d, 0x12a8, 0x12ed, 0x12da, 0x12f0, 0x126a, 0x1235, 0x12f1, 0x121e,
0x1295, 0x1275, 0x20, 0x12f2, 0x12a1, 0x122d, 0x126a, 0x120d, 0x121b, 0x12b3, 0x122a, 0x121b,
0x12cd, 0x1230, 0x1295, 0x121b, 0x12ad, 0x1219, 0x122d, 0x12f6, 0x122e, 0x12d8, 0x122b, 0x120e,
0x1295, 0x130d, 0x12e8, 0x122d, 0x1263, 0x12e8, 0x1295, 0x12d3, 0x12f0, 0x1295, 0x12a3, 0x120d,
0x121b, 0x1272, 0x12d3, 0x121b, 0x1295, 0x12a3, 0x1293, 0x12f2, 0x122d, 0x12a3, 0x12ad, 0x1273,
0x12cd, 0x12a3, 0x12ad, 0x1276, 0x1260, 0x12a3, 0x123d, 0x130b, 0x1263, 0x1275, 0x12a3, 0x1272,
0x122b, 0x12cd, 0x1263, 0x1255, 0x12f3, 0x12f5, 0x1263, 0x1215, 0x122c, 0x1295, 0x1263, 0x1295,
0x130d, 0x12ae, 0x12ad, 0x1263, 0x122d, 0x1293, 0x12cd, 0x120d, 0x1260, 0x12ed, 0x1229, 0x1275,
0x1265, 0x1229, 0x1290, 0x12ed, 0x12f1, 0x123b, 0x1295, 0x1260, 0x1253, 0x12db, 0x12a8, 0x1270,
0x121b, 0x20, 0x1206, 0x20, 0x127a, 0x20, 0x121a, 0x1295, 0x12b3, 0x120b, 0x20, 0x1209,
0x121d, 0x1351, 0x122d, 0x121b, 0x12ab, 0x12cd, 0x1292, 0x12ae, 0x1235, 0x12eb, 0x1296, 0x126e,
0x12a9, 0x12dd, 0x1290, 0x1275, 0x1235, 0x12ad, 0x1296, 0x126e, 0x1232, 0x1262, 0x122d, 0x1235,
0x12ad, 0x1355, 0x1296, 0x121d, 0x20, 0x1350, 0x1295, 0x1356, 0x1295, 0x1275, 0x12eb, 0x1293,
0x12ad, 0x1250, 0x1320, 0x122d, 0x12aa, 0x12da, 0x120e, 0x122d, 0x12f3, 0x122d, 0x12eb, 0x12f5,
0x1233, 0x12ab, 0x120a, 0x1295, 0x1236, 0x12cd, 0x120d, 0x123b, 0x1295, 0x130d, 0x1203, 0x12ed,
0x1235, 0x122c, 0x12f5, 0x1290, 0x12ae, 0x120a, 0x121d, 0x1235, 0x12ad, 0x1273, 0x12ed, 0x1350,
0x12ed, 0x1273, 0x123d, 0x12a8, 0x1295, 0x1275, 0x1275, 0x1262, 0x120a, 0x1232, 0x1270, 0x1205,
0x122b, 0x1295, 0x1276, 0x12ad, 0x12ee, 0x12a1, 0x120b, 0x1295, 0x20, 0x1263, 0x1276, 0x122d,
0x126d, 0x12e8, 0x1295, 0x1275, 0x12eb, 0x1295, 0x12e8, 0x1228, 0x126b, 0x1295, 0x12a3, 0x12de,
0x1228, 0x1235, 0x1260, 0x122d, 0x1219, 0x12f3, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d, 0x12f0,
0x12f0, 0x1234, 0x1273, 0x1275, 0x20, 0x134b, 0x122e, 0x121b, 0x12f0, 0x12ed, 0x122b, 0x1228,
0x12ed, 0x12ad, 0x12eb, 0x126a, 0x12ad, 0x12f0, 0x1261, 0x1265, 0x20, 0x1306, 0x122d, 0x1305,
0x12eb, 0x1245, 0x12f5, 0x1235, 0x1272, 0x20, 0x1204, 0x1208, 0x1293, 0x1235, 0x1273, 0x1295,
0x120a, 0x12a3, 0x12f0, 0x1208, 0x12ed, 0x12f5, 0x1265, 0x122a, 0x12dd, 0x1264, 0x1295, 0x12e9,
0x12ad, 0x120b, 0x120e, 0x122d, 0x12f5, 0x20, 0x1203, 0x12cd, 0x1218, 0x120d, 0x1260, 0x122d,
0x1295, 0x12a3, 0x121d, 0x1235, 0x1270, 0x122d, 0x12f3, 0x121d, 0x12a3, 0x1295, 0x12f6, 0x122b,
0x12a3, 0x1235, 0x1275, 0x122b, 0x12ab, 0x1295, 0x12a3, 0x1274, 0x1295, 0x1235, 0x1260, 0x120d,
0x130d, 0x122c, 0x12f5, 0x1265, 0x122b, 0x1235, 0x120d, 0x1235, 0x1261, 0x12ab, 0x1228, 0x1235,
0x1275, 0x1261, 0x12f3, 0x1350, 0x1235, 0x1275, 0x12aa, 0x123a, 0x1293, 0x12cd, 0x1308, 0x122d,
0x1295, 0x12da, 0x12a3, 0x12ed, 0x120d, 0x20, 0x12a6, 0x134d, 0x20, 0x121b, 0x1295, 0x1300,
0x122d, 0x12da, 0x12ad, 0x12e8, 0x126d, 0x120d, 0x12e9, 0x1265, 0x120d, 0x12eb, 0x1293, 0x120e,
0x1295, 0x12f0, 0x1295, 0x121b, 0x122a, 0x1203, 0x121d, 0x1295, 0x1356, 0x12f5, 0x130e, 0x122a,
0x133b, 0x1233, 0x122b, 0x12e8, 0x126e, 0x1236, 0x134d, 0x12eb, 0x1272, 0x122b, 0x1293, 0x126d,
0x12e8, 0x1293, 0x126a, 0x120d, 0x1295, 0x12e8, 0x1235, 0x126e, 0x120d, 0x130e, 0x130d, 0x122b,
0x12f5, 0x12cb, 0x122d, 0x1233, 0x12cd, 0x12db, 0x130d, 0x1228, 0x1265, 0x12a3, 0x1295, 0x1273,
0x1293, 0x1293, 0x122a, 0x126e, 0x12ad, 0x122a, 0x1235, 0x121b, 0x1235, 0x12a8, 0x122d, 0x1309,
0x1208, 0x1295, 0x121b, 0x120d, 0x12f2, 0x126d, 0x1235, 0x121b, 0x12cd, 0x122a, 0x1238, 0x1235,
0x121b, 0x12ee, 0x1275, 0x122d, 0x12e9, 0x1295, 0x12e8, 0x1295, 0x12a3, 0x1355, 0x12eb, 0x1261,
0x1308, 0x1295, 0x126a, 0x120d, 0x12f0, 0x1234, 0x1275, 0x20, 0x134b, 0x1232, 0x12ab, 0x12a4,
0x134b, 0x1274, 0x1349, 0x1293, 0x1349, 0x1272, 0x130b, 0x121d, 0x1265, 0x12e8, 0x122d, 0x1313,
0x12f3, 0x120d, 0x12ab, 0x1293, 0x120d, 0x1313, 0x121d, 0x12a4, 0x1295, 0x12f0, 0x122d, 0x1260,
0x122a, 0x121b, 0x1301, 0x122e, 0x121b, 0x122d, 0x12ac, 0x1233, 0x1235, 0x1291, 0x1218, 0x12eb,
0x1352, 0x1275, 0x12a8, 0x122d, 0x1295, 0x1356, 0x1295, 0x1350, 0x12ed, 0x1356, 0x122d, 0x1275,
0x20, 0x121e, 0x122d, 0x1235, 0x1262, 0x1233, 0x12ed, 0x1353, 0x1295, 0x12cc, 0x12ad, 0x4e,
0x69, 0x75, 0x20, 0x2bb, 0x49, 0x6f, 0x6b, 0x65, 0x48, 0x65, 0x70, 0x65,
0x6c, 0x6f, 0x6e, 0x69, 0x53, 0x65, 0x6c, 0x16b, 0x73, 0x61, 0x6c, 0x65,
0x6d, 0x61, 0x41, 0x74, 0x65, 0x6c, 0x61, 0x69, 0x74, 0x65, 0x50, 0x65,
0x6c, 0x69, 0x73, 0x69, 0x70, 0x61, 0x6e, 0x65, 0x4d, 0x65, 0x6c, 0x69,
0x70, 0x6f, 0x61, 0x6e, 0x65, 0x53, 0x65, 0x6e, 0x113, 0x2bb, 0x41, 0x74,
0x65, 0x6e, 0x69, 0x73, 0x69, 0x4c, 0x6f, 0x6e, 0x69, 0x74, 0x6f, 0x6e,
0x69, 0x4d, 0x6f, 0x73, 0x69, 0x6b, 0x6f, 0x75, 0x50, 0x61, 0x6c, 0x65,
0x73, 0x69, 0x4c, 0x6f, 0x6d, 0x61, 0x2bb, 0x41, 0x6f, 0x6b, 0x61, 0x6c,
0x61, 0x6e, 0x69, 0x50, 0x75, 0x6b, 0x61, 0x6e, 0x69, 0x76, 0x69, 0x6c,
0x61, 0x54, 0x16b, 0x6b, 0x65, 0x4c, 0x61, 0x70, 0x61, 0x6e, 0x75, 0x69,
0x46, 0x69, 0x73, 0x69, 0x4b, 0x61, 0x6d, 0x69, 0x70, 0x69, 0x113, 0x4b,
0x75, 0x61, 0x74, 0x101, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x69, 0x4b, 0x75,
0x61, 0x6d, 0x69, 0x2bb, 0x45, 0x6e, 0x69, 0x74, 0x69, 0x70, 0x75, 0x6c,
0x12b, 0x4b, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x6d, 0x61, 0x73, 0x69, 0x4b,
0x6f, 0x73, 0x69, 0x6c, 0x61, 0x65, 0x4b, 0x75, 0x61, 0x73, 0x61, 0x6c,
0x65, 0x6e, 0x69, 0x4d, 0x61, 0x73, 0x75, 0x6c, 0x6f, 0x4d, 0x61, 0x6c,
0x69, 0x6b, 0x75, 0x65, 0x73, 0x61, 0x4d, 0x69, 0x74, 0x69, 0x75, 0x61,
0x69, 0x4e, 0x61, 0x75, 0x6c, 0x75, 0x4e, 0x69, 0x75, 0x113, 0x4e, 0x6f,
0x61, 0x66, 0x14d, 0x6b, 0x69, 0x50, 0x61, 0x6e, 0x67, 0x6f, 0x70, 0x61,
0x6e, 0x67, 0x6f, 0x50, 0x69, 0x74, 0x69, 0x6b, 0x65, 0x6e, 0x69, 0x50,
0x6f, 0x6e, 0x61, 0x70, 0x113, 0x54, 0x61, 0x75, 0x6c, 0x61, 0x6e, 0x67,
0x61, 0x20, 0x4d, 0x6f, 0x6c, 0x65, 0x73, 0x69, 0x70, 0x69, 0x4c, 0x61,
0x6c, 0x6f, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x2bb, 0x69, 0x70,
0x61, 0x6e, 0x69, 0x54, 0x61, 0x68, 0x69, 0x73, 0x69, 0x54, 0x61, 0x6c,
0x61, 0x76, 0x61, 0x55, 0x65, 0x6b, 0x65, 0x2bb, 0x55, 0x76, 0x65, 0x61,
0x43, 0x65, 0x7a, 0x61, 0x79, 0x69, 0x72, 0x4b, 0x61, 0x68, 0x69, 0x72,
0x65, 0x44, 0x61, 0x72, 0xfc, 0x73, 0x73, 0x65, 0x6c, 0x61, 0x6d, 0x4c,
0x61, 0x79, 0x75, 0x6e, 0x43, 0x75, 0x62, 0x61, 0x4d, 0x6f, 0x67, 0x61,
0x64, 0x69, 0x15f, 0x75, 0x54, 0x75, 0x6e, 0x75, 0x73, 0x42, 0x65, 0x75,
0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x2c,
0x20, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20,
0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x41, 0x6b, 0x74, 0x61, 0x76, 0x41, 0x15f, 0x6b, 0x61, 0x62, 0x61, 0x74,
0x41, 0x74, 0x131, 0x72, 0x61, 0x76, 0x42, 0x61, 0x11f, 0x64, 0x61, 0x74,
0x42, 0x61, 0x68, 0x72, 0x65, 0x79, 0x6e, 0x42, 0x61, 0x6b, 0xfc, 0x44,
0x75, 0x15f, 0x61, 0x6e, 0x62, 0x65, 0x47, 0x61, 0x7a, 0x69, 0x6d, 0x61,
0x11f, 0x75, 0x73, 0x61, 0x47, 0x61, 0x7a, 0x7a, 0x65, 0x45, 0x6c, 0x20,
0x48, 0x61, 0x6c, 0x69, 0x6c, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20,
0x4d, 0x69, 0x6e, 0x68, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x69, 0x4b, 0x75,
0x64, 0xfc, 0x73, 0x4b, 0x61, 0x6c, 0x6b, 0xfc, 0x74, 0x61, 0x4b, 0x75,
0x76, 0x65, 0x79, 0x74, 0x4c, 0x65, 0x66, 0x6b, 0x6f, 0x15f, 0x61, 0x4b,
0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x54, 0x61, 0x68, 0x72,
0x61, 0x6e, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72,
0x41, 0x7a, 0x6f, 0x72, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x4b, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x61, 0x20, 0x41, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x131, 0x4d, 0x61, 0x64, 0x65, 0x69, 0x72, 0x61, 0x20, 0x41,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x47, 0xfc, 0x6e, 0x65, 0x79, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x42, 0x72, 0xfc, 0x6b, 0x73,
0x65, 0x6c, 0x42, 0xfc, 0x6b, 0x72, 0x65, 0x15f, 0x42, 0x75, 0x64, 0x61,
0x70, 0x65, 0x15f, 0x74, 0x65, 0x4b, 0x69, 0x15f, 0x69, 0x6e, 0x65, 0x76,
0x43, 0x65, 0x62, 0x65, 0x6c, 0x69, 0x74, 0x61, 0x72, 0x131, 0x6b, 0x4d,
0x61, 0x6e, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x4c, 0xfc, 0x6b, 0x73,
0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0xdc, 0x73, 0x6b, 0xfc, 0x70, 0x56,
0x61, 0x72, 0x15f, 0x6f, 0x76, 0x61, 0x5a, 0xfc, 0x72, 0x69, 0x68, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x6c, 0x65, 0x72, 0x50, 0x61, 0x73, 0x6b,
0x61, 0x6c, 0x79, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x4d, 0x61,
0x72, 0x6b, 0x69, 0x7a, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x41, 0x62, 0x69, 0x6a, 0x61, 0x6e, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0x61,
0xfd, 0x72, 0x42, 0x72, 0x61, 0x7a, 0x7a, 0x61, 0x77, 0x69, 0x6c, 0x45,
0x6c, 0x2d, 0x41, 0xfd, 0x75, 0x6e, 0xdd, 0x6f, 0x68, 0x61, 0x6e, 0x6e,
0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x77,
0x69, 0x6c, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x15f, 0x6f, 0x4d, 0x6f,
0x6e, 0x72, 0x6f, 0x77, 0x69, 0xfd, 0x61, 0x4e, 0x61, 0xfd, 0x72, 0x6f,
0x62, 0x69, 0x4a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x69, 0x61, 0x6d,
0x65, 0xfd, 0x4e, 0x75, 0x61, 0x6b, 0x15f, 0x6f, 0x74, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x2d, 0x4e, 0x6f, 0x77, 0x6f, 0x53, 0x61, 0x6e, 0x2d, 0x54,
0x6f, 0x6d, 0x65, 0x41, 0x64, 0x61, 0x6b, 0x20, 0x61, 0x64, 0x61, 0x73,
0x79, 0x41, 0x6e, 0x67, 0x69, 0x6c, 0xfd, 0x61, 0x41, 0x72, 0x61, 0x67,
0x75, 0x61, 0xfd, 0x6e, 0x61, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x2d,
0x41, 0xfd, 0x72, 0x65, 0x73, 0x4b, 0x6f, 0x72, 0x64, 0x6f, 0x77, 0x61,
0x17d, 0x75, 0x17e, 0x75, 0xfd, 0x4c, 0x61, 0x2d, 0x52, 0x69, 0x6f, 0x68,
0x61, 0x53, 0x61, 0x6e, 0x2d, 0x4c, 0x75, 0x69, 0x73, 0x55, 0x15f, 0x75,
0x61, 0xfd, 0x61, 0x41, 0x73, 0x75, 0x6e, 0x73, 0xfd, 0x6f, 0x6e, 0x42,
0x61, 0x69, 0xfd, 0x61, 0x42, 0x61, 0x69, 0xfd, 0x61, 0x2d, 0x64, 0x65,
0x2d, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x42, 0x6f, 0x61,
0x2d, 0x57, 0x69, 0x73, 0x74, 0x61, 0x42, 0x6f, 0xfd, 0x73, 0x65, 0x4b,
0x65, 0x6d, 0x62, 0x72, 0x69, 0x6a, 0x2d, 0x42, 0x65, 0xfd, 0x4b, 0x61,
0x6d, 0x70, 0x75, 0x2d, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x69, 0x4b, 0x61,
0xfd, 0x65, 0x6e, 0x6e, 0x61, 0x4b, 0x61, 0xfd, 0x6d, 0x61, 0x6e, 0x20,
0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x53, 0xfd, 0x75, 0x64, 0x61,
0x64, 0x2d, 0x48, 0x75, 0x61, 0x72, 0x65, 0x73, 0x4b, 0x6f, 0x73, 0x74,
0x61, 0x2d, 0x52, 0x69, 0x6b, 0x61, 0x4b, 0x75, 0xfd, 0x61, 0x62, 0x61,
0x4b, 0xfd, 0x75, 0x72, 0x61, 0x73, 0x61, 0x6f, 0x44, 0x65, 0x6e, 0x6d,
0x61, 0x72, 0x6b, 0x73, 0x68, 0x61, 0x77, 0x6e, 0x44, 0x6f, 0x75, 0x73,
0x6f, 0x6e, 0x2d, 0x4b, 0x72, 0x69, 0x6b, 0x44, 0x65, 0x6e, 0x77, 0x65,
0x72, 0x44, 0x65, 0x74, 0x72, 0x6f, 0xfd, 0x74, 0x45, 0xfd, 0x72, 0x75,
0x6e, 0x65, 0x70, 0x65, 0x47, 0x6c, 0x65, 0xfd, 0x73, 0x2d, 0x42, 0x65,
0xfd, 0x47, 0x75, 0x73, 0x2d, 0x42, 0x65, 0xfd, 0x47, 0x72, 0x61, 0x6e,
0x64, 0x2d, 0x54, 0x65, 0x72, 0x6b, 0x47, 0x75, 0xfd, 0x61, 0x6b, 0x69,
0x6c, 0x47, 0x61, 0xfd, 0x61, 0x6e, 0x61, 0x47, 0x61, 0x6c, 0x69, 0x66,
0x61, 0x6b, 0x73, 0x47, 0x61, 0x77, 0x61, 0x6e, 0x61, 0x45, 0x72, 0x6d,
0x6f, 0x73, 0x69, 0x6c, 0x6f, 0x4e, 0x6f, 0x6b, 0x73, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x77, 0x65, 0xfd, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0x73,
0x65, 0x6e, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x49, 0x6e, 0x75, 0x77, 0x69, 0x6b, 0xdd, 0x61, 0x6d, 0x61, 0xfd, 0x6b,
0x61, 0x4c, 0x75, 0x69, 0x73, 0x77, 0x69, 0x6c, 0x6c, 0x4d, 0x6f, 0x6e,
0x74, 0x69, 0x73, 0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e,
0x74, 0x75, 0x6b, 0x69, 0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x65,
0xfd, 0x6b, 0x4c, 0x61, 0x2d, 0x50, 0x61, 0x73, 0x4c, 0x6f, 0x73, 0x2d,
0x41, 0x6e, 0x6a, 0x65, 0x6c, 0x65, 0x73, 0x4c, 0x6f, 0x77, 0x65, 0x72,
0x2d, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x73, 0x2d, 0x4b, 0x75, 0x6f,
0x72, 0x74, 0x65, 0x72, 0x4d, 0x61, 0x73, 0x65, 0xfd, 0x6f, 0x4d, 0x6f,
0x6e, 0x74, 0x65, 0x72, 0x72, 0x65, 0xfd, 0x4d, 0x6f, 0x6e, 0x74, 0x65,
0x77, 0x69, 0x64, 0x65, 0x6f, 0x4e, 0xfd, 0x75, 0x2d, 0xdd, 0x6f, 0x72,
0x6b, 0x42, 0x6f, 0xfd, 0x6c, 0x61, 0x2c, 0x20, 0x44, 0x65, 0x6d, 0x69,
0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x2c, 0x20, 0x44, 0x65, 0x6d, 0x69,
0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x4e, 0xfd, 0x75, 0x2d, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20,
0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f, 0x66,
0x2d, 0x53, 0x70, 0x65, 0xfd, 0x6e, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x2d,
0x57, 0x65, 0x6c, 0xfd, 0x75, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x2d,
0x52, 0x69, 0x6b, 0x6f, 0x52, 0x65, 0x6a, 0x61, 0xfd, 0x6e, 0x61, 0x52,
0x65, 0x7a, 0x6f, 0x6c, 0xfd, 0x75, 0x74, 0x52, 0x69, 0x75, 0x2d, 0x42,
0x72, 0x61, 0x6e, 0x6b, 0x75, 0x53, 0x61, 0x6e, 0x74, 0xfd, 0x61, 0x67,
0x6f, 0x53, 0x61, 0x6e, 0x2d, 0x50, 0x61, 0x75, 0x6c, 0x75, 0x49, 0x6c,
0x6c, 0x6f, 0x6b, 0x6b, 0x6f, 0x72, 0x74, 0x6f, 0x6f, 0x72, 0x6d, 0x69,
0x75, 0x74, 0x53, 0x65, 0x6e, 0x74, 0x2d, 0x4a, 0x6f, 0x6e, 0x73, 0x53,
0x65, 0x6e, 0x74, 0x2d, 0x4b, 0x69, 0x74, 0x73, 0x53, 0x65, 0x6e, 0x74,
0x2d, 0x4c, 0xfd, 0x75, 0x73, 0x69, 0xfd, 0x61, 0x53, 0x65, 0x6e, 0x74,
0x2d, 0x54, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x2d, 0x57,
0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x53, 0x77, 0x69, 0x66, 0x74, 0x2d,
0x4b, 0x61, 0x72, 0x65, 0x6e, 0x74, 0x57, 0x61, 0x6e, 0x6b, 0x75, 0x77,
0x65, 0x72, 0x57, 0x61, 0xfd, 0x74, 0x68, 0x6f, 0x72, 0x73, 0xdd, 0x61,
0x6b, 0x75, 0x74, 0x61, 0x74, 0x4b, 0x65, 0xfd, 0x73, 0x69, 0x44, 0x65,
0xfd, 0x77, 0x69, 0x73, 0x44, 0xfd, 0x75, 0x6d, 0x6f, 0x6e, 0x2d, 0x64,
0x2d, 0xdd, 0x75, 0x72, 0x77, 0x69, 0x6c, 0x4d, 0x61, 0x6b, 0x6b, 0x75,
0x6f, 0x72, 0x69, 0x4d, 0x61, 0x6b, 0x2d, 0x4d, 0x65, 0x72, 0x64, 0x6f,
0x53, 0xfd, 0x6f, 0x77, 0x61, 0x4c, 0x6f, 0x6e, 0x67, 0x69, 0x72, 0x42,
0x61, 0x67, 0x64, 0x61, 0x74, 0x42, 0x61, 0x68, 0x72, 0x65, 0xfd, 0x6e,
0x42, 0x65, 0xfd, 0x72, 0x75, 0x74, 0x42, 0x72, 0x75, 0x6e, 0x65, 0xfd,
0x44, 0x75, 0x62, 0x61, 0xfd, 0x48, 0x65, 0x77, 0x72, 0x6f, 0x6e, 0x48,
0x6f, 0x15f, 0x69, 0x6d, 0x69, 0x6e, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e,
0x67, 0x48, 0x6f, 0x77, 0x64, 0x4a, 0x61, 0xfd, 0x61, 0x70, 0x75, 0x72,
0x61, 0x49, 0xfd, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x69, 0x6d, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0xfd, 0x61, 0x72, 0x73, 0x6b, 0x4b, 0x75,
0x77, 0x65, 0xfd, 0x74, 0x4e, 0x69, 0x6b, 0x6f, 0x73, 0x69, 0xfd, 0x61,
0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x65, 0x73, 0x6b, 0x50,
0x68, 0x65, 0x6e, 0xfd, 0x61, 0x6e, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e,
0x61, 0xfd, 0x47, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x45,
0x72, 0x2d, 0x52, 0x69, 0xfd, 0x61, 0x64, 0x15e, 0x61, 0x6e, 0x68, 0x61,
0xfd, 0x54, 0x61, 0xfd, 0x62, 0x65, 0xfd, 0x54, 0xe4, 0x68, 0x72, 0x61,
0x6e, 0x57, 0xfd, 0x65, 0x6e, 0x74, 0xfd, 0x61, 0x6e, 0xdd, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0xdd, 0x61, 0x6e, 0x67, 0x6f, 0x6e, 0xdd, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xdd,
0x65, 0x72, 0x65, 0x77, 0x61, 0x6e, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x61,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x20,
0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x4b, 0x61, 0x62, 0x6f, 0x2d,
0x57, 0x65, 0x72, 0x64, 0x65, 0x46, 0x61, 0x72, 0x65, 0x72, 0x20, 0x61,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x4d, 0x61, 0x64, 0x65, 0xfd, 0x72,
0x61, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x52, 0x65, 0xfd,
0x6b, 0xfd, 0x61, 0x77, 0x69, 0x6b, 0x47, 0xfc, 0x6e, 0x6f, 0x72, 0x74,
0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xfd, 0x61, 0x4b, 0x65,
0x72, 0x61, 0x6d, 0x61, 0x74, 0x6c, 0x79, 0x20, 0xdd, 0x65, 0x6c, 0x65,
0x6e, 0x61, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0xdd, 0x75, 0x6b, 0x6c,
0x61, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x61, 0x75, 0x53, 0x69, 0x64,
0x6e, 0x65, 0xfd, 0x41, 0x66, 0x69, 0x6e, 0x79, 0x42, 0x72, 0x61, 0x74,
0x69, 0x73, 0x6c, 0x61, 0x77, 0x61, 0x42, 0x72, 0xfd, 0x75, 0x73, 0x73,
0x65, 0x6c, 0x42, 0x75, 0x68, 0x61, 0x72, 0x65, 0x73, 0x74, 0x42, 0xfd,
0x75, 0x7a, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x4b, 0x69, 0x15f, 0x69, 0x6e,
0xfd, 0x6f, 0x77, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x67, 0x61, 0x67, 0x65,
0x6e, 0x4d, 0x65, 0x6e, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x4a, 0x65,
0x72, 0x73, 0x69, 0x4b, 0x69, 0xfd, 0x65, 0x77, 0x4c, 0xfd, 0x75, 0x62,
0x6c, 0xfd, 0x61, 0x6e, 0x61, 0x4c, 0xfd, 0x75, 0x6b, 0x73, 0x65, 0x6d,
0x62, 0x75, 0x72, 0x67, 0x4d, 0x61, 0x72, 0x69, 0xfd, 0x65, 0x68, 0x61,
0x6d, 0x6e, 0x50, 0x61, 0x72, 0x69, 0x17e, 0x50, 0x6f, 0x64, 0x67, 0x6f,
0x72, 0x69, 0x73, 0x61, 0x53, 0x61, 0x72, 0x61, 0xfd, 0x65, 0x77, 0x6f,
0x53, 0x6b, 0x6f, 0x70, 0xfd, 0x65, 0x53, 0x6f, 0x66, 0x69, 0xfd, 0x61,
0x53, 0x74, 0x6f, 0x6b, 0x67, 0x6f, 0x6c, 0x6d, 0x55, 0x6c, 0xfd, 0x61,
0x6e, 0x6f, 0x77, 0x73, 0x6b, 0x57, 0x61, 0x64, 0x75, 0x7a, 0x57, 0x61,
0x74, 0x69, 0x6b, 0x61, 0x6e, 0x57, 0x65, 0x6e, 0x61, 0x57, 0x69, 0x6c,
0x6e, 0xfd, 0x75, 0x73, 0x57, 0x61, 0x72, 0x15f, 0x61, 0x77, 0x61, 0x53,
0xfd, 0x75, 0x72, 0x69, 0x68, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e,
0x61, 0x72, 0x69, 0x77, 0x75, 0x52, 0x6f, 0x17e, 0x64, 0x65, 0x73, 0x74,
0x77, 0x6f, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x20, 0x61, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x79, 0x4d, 0x61, 0xfd, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x77, 0x6c, 0x65, 0x72, 0x4d, 0x61, 0x77, 0x72, 0x69, 0x6b, 0x69, 0xfd,
0x4d, 0x61, 0xfd, 0x6f, 0x74, 0x74, 0x61, 0x52, 0x65, 0xfd, 0x75, 0x6e,
0xfd, 0x6f, 0x6e, 0x42, 0x75, 0x67, 0x65, 0x6e, 0x77, 0x69, 0x6c, 0xc7,
0x61, 0x74, 0x65, 0x6d, 0x50, 0x61, 0x73, 0x68, 0x61, 0x20, 0x61, 0x64,
0x61, 0x73, 0x79, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x47, 0x61, 0x6d, 0x62,
0xfd, 0x65, 0x47, 0x77, 0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61,
0x6c, 0x47, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0xfd, 0x65, 0x4b, 0x77, 0x61, 0x6a, 0x61, 0x6c, 0x65, 0xfd,
0x6e, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x61, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x79, 0x4d, 0x69, 0x64, 0x75, 0x65, 0xfd, 0x50, 0x6f, 0x72,
0x74, 0x2d, 0x4d, 0x6f, 0x72, 0x73, 0x62, 0x69, 0x53, 0x61, 0xfd, 0x70,
0x61, 0x6e, 0x57, 0x65, 0xfd, 0x6b, 0x411, 0x430, 0x43d, 0x491, 0x456, 0x411,
0x440, 0x430, 0x437, 0x437, 0x430, 0x432, 0x456, 0x43b, 0x44c, 0x41a, 0x430, 0x457,
0x440, 0x41a, 0x43e, 0x43d, 0x430, 0x43a, 0x440, 0x456, 0x414, 0x436, 0x456, 0x431,
0x443, 0x442, 0x456, 0x415, 0x43b, 0x44c, 0x2d, 0x410, 0x44e, 0x43d, 0x424, 0x440,
0x456, 0x442, 0x430, 0x443, 0x43d, 0x490, 0x430, 0x431, 0x43e, 0x440, 0x43e, 0x43d,
0x435, 0x419, 0x43e, 0x433, 0x430, 0x43d, 0x43d, 0x435, 0x441, 0x431, 0x443, 0x440,
0x491, 0x41b, 0x430, 0x491, 0x43e, 0x441, 0x41b, 0x456, 0x431, 0x440, 0x435, 0x432,
0x456, 0x43b, 0x44c, 0x41b, 0x443, 0x431, 0x443, 0x43c, 0x431, 0x430, 0x448, 0x456,
0x41c, 0x43e, 0x491, 0x430, 0x434, 0x456, 0x448, 0x43e, 0x41c, 0x43e, 0x43d, 0x440,
0x43e, 0x432, 0x456, 0x44f, 0x41d, 0x456, 0x430, 0x43c, 0x435, 0x439, 0x41d, 0x443,
0x430, 0x43a, 0x448, 0x43e, 0x442, 0x442, 0x423, 0x430, 0x491, 0x430, 0x434, 0x443,
0x491, 0x443, 0x422, 0x440, 0x438, 0x43f, 0x43e, 0x43b, 0x456, 0x412, 0x456, 0x43d,
0x434, 0x433, 0x443, 0x43a, 0x410, 0x43d, 0x491, 0x456, 0x43b, 0x44c, 0x44f, 0x410,
0x43d, 0x442, 0x438, 0x491, 0x443, 0x430, 0x410, 0x440, 0x430, 0x491, 0x443, 0x430,
0x456, 0x43d, 0x430, 0x411, 0x443, 0x435, 0x43d, 0x43e, 0x441, 0x2d, 0x410, 0x439,
0x440, 0x435, 0x441, 0x41b, 0x430, 0x2d, 0x420, 0x456, 0x43e, 0x445, 0x430, 0x420,
0x456, 0x43e, 0x2d, 0x490, 0x430, 0x43b, 0x44c, 0x454, 0x491, 0x43e, 0x441, 0x421,
0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x457, 0x441, 0x410, 0x441, 0x443, 0x43d, 0x441,
0x44c, 0x439, 0x43e, 0x43d, 0x410, 0x442, 0x456, 0x43a, 0x43e, 0x43a, 0x430, 0x43d,
0x411, 0x430, 0x457, 0x44f, 0x20, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430,
0x441, 0x411, 0x43e, 0x491, 0x43e, 0x442, 0x430, 0x41a, 0x435, 0x439, 0x43c, 0x431,
0x440, 0x456, 0x434, 0x436, 0x2d, 0x411, 0x435, 0x439, 0x41a, 0x430, 0x43c, 0x43f,
0x443, 0x2d, 0x490, 0x440, 0x430, 0x43d, 0x434, 0x456, 0x41a, 0x430, 0x454, 0x43d,
0x43d, 0x430, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x43e, 0x432, 0x456, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x427, 0x456, 0x43a, 0x430, 0x491,
0x43e, 0x427, 0x456, 0x443, 0x430, 0x443, 0x430, 0x421, 0x2019, 0x44e, 0x434, 0x430,
0x434, 0x2d, 0x425, 0x443, 0x430, 0x440, 0x435, 0x441, 0x41a, 0x43e, 0x441, 0x442,
0x430, 0x2d, 0x420, 0x456, 0x43a, 0x430, 0x414, 0x43e, 0x443, 0x441, 0x43e, 0x43d,
0x2d, 0x41a, 0x440, 0x456, 0x43a, 0x414, 0x43e, 0x43c, 0x456, 0x43d, 0x456, 0x43a,
0x430, 0x490, 0x43b, 0x435, 0x439, 0x441, 0x2d, 0x411, 0x435, 0x439, 0x490, 0x443,
0x441, 0x2d, 0x411, 0x435, 0x439, 0x490, 0x440, 0x430, 0x43d, 0x434, 0x2d, 0x422,
0x435, 0x440, 0x43a, 0x490, 0x440, 0x435, 0x43d, 0x430, 0x434, 0x430, 0x490, 0x432,
0x430, 0x434, 0x435, 0x43b, 0x443, 0x43f, 0x430, 0x490, 0x432, 0x430, 0x442, 0x435,
0x43c, 0x430, 0x43b, 0x430, 0x490, 0x443, 0x430, 0x44f, 0x43a, 0x456, 0x43b, 0x44c,
0x490, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x415, 0x440, 0x43c, 0x43e, 0x441, 0x456,
0x43b, 0x44c, 0x439, 0x43e, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x43f,
0x43e, 0x43b, 0x456, 0x441, 0x41d, 0x43e, 0x43a, 0x441, 0x2c, 0x20, 0x406, 0x43d,
0x434, 0x456, 0x430, 0x43d, 0x430, 0x41c, 0x430, 0x440, 0x435, 0x43d, 0x433, 0x43e,
0x2c, 0x20, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x41f, 0x456, 0x442,
0x435, 0x440, 0x441, 0x431, 0x435, 0x440, 0x433, 0x2c, 0x20, 0x406, 0x43d, 0x434,
0x456, 0x430, 0x43d, 0x430, 0x422, 0x435, 0x43b, 0x43b, 0x2d, 0x421, 0x456, 0x442,
0x456, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x412, 0x456,
0x432, 0x435, 0x439, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430,
0x412, 0x456, 0x43d, 0x441, 0x435, 0x43d, 0x441, 0x2c, 0x20, 0x406, 0x43d, 0x434,
0x456, 0x430, 0x43d, 0x430, 0x412, 0x456, 0x43d, 0x430, 0x43c, 0x430, 0x43a, 0x2c,
0x20, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x406, 0x43a, 0x430, 0x43b,
0x443, 0x457, 0x442, 0x41b, 0x443, 0x457, 0x441, 0x432, 0x456, 0x43b, 0x43b, 0x41c,
0x43e, 0x43d, 0x442, 0x456, 0x441, 0x435, 0x43b, 0x43b, 0x43e, 0x2c, 0x20, 0x41a,
0x435, 0x43d, 0x442, 0x443, 0x43a, 0x43a, 0x456, 0x41b, 0x43e, 0x443, 0x435, 0x440,
0x2d, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x435, 0x441, 0x2d, 0x41a, 0x432, 0x43e,
0x442, 0x435, 0x440, 0x41c, 0x430, 0x43d, 0x430, 0x491, 0x443, 0x430, 0x41c, 0x430,
0x440, 0x456, 0x491, 0x43e, 0x41c, 0x430, 0x440, 0x442, 0x456, 0x43d, 0x456, 0x43a,
0x430, 0x41c, 0x435, 0x440, 0x456, 0x434, 0x430, 0x41c, 0x435, 0x445, 0x456, 0x43a,
0x43e, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x432, 0x456, 0x434, 0x435, 0x43e, 0x411,
0x2bc, 0x44e, 0x43b, 0x430, 0x2c, 0x20, 0x41f, 0x456, 0x432, 0x43d, 0x456, 0x447,
0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x421, 0x435, 0x43d,
0x442, 0x440, 0x2c, 0x20, 0x41f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x430,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e, 0x2d, 0x421,
0x430, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x41f, 0x456, 0x432, 0x43d, 0x456, 0x447,
0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41e, 0x445, 0x456,
0x43d, 0x430, 0x491, 0x430, 0x41f, 0x443, 0x435, 0x440, 0x442, 0x43e, 0x2d, 0x420,
0x456, 0x43a, 0x43e, 0x420, 0x435, 0x43d, 0x43a, 0x456, 0x43d, 0x2d, 0x406, 0x43d,
0x43b, 0x435, 0x442, 0x420, 0x435, 0x441, 0x456, 0x444, 0x456, 0x420, 0x456, 0x43e,
0x2d, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x44c,
0x44f, 0x491, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x43e, 0x2d, 0x414, 0x43e, 0x43c,
0x456, 0x43d, 0x491, 0x43e, 0x406, 0x442, 0x442, 0x43e, 0x43a, 0x43a, 0x43e, 0x440,
0x442, 0x43e, 0x43e, 0x440, 0x43c, 0x456, 0x443, 0x442, 0x421, 0x435, 0x43d, 0x2d,
0x411, 0x430, 0x440, 0x442, 0x435, 0x43b, 0x435, 0x43c, 0x456, 0x421, 0x435, 0x43d,
0x442, 0x2d, 0x41a, 0x456, 0x442, 0x442, 0x441, 0x421, 0x432, 0x456, 0x444, 0x442,
0x2d, 0x41a, 0x430, 0x440, 0x440, 0x435, 0x43d, 0x442, 0x422, 0x435, 0x491, 0x443,
0x441, 0x456, 0x491, 0x430, 0x43b, 0x44c, 0x43f, 0x430, 0x422, 0x456, 0x445, 0x443,
0x430, 0x43d, 0x430, 0x412, 0x430, 0x439, 0x442, 0x433, 0x43e, 0x440, 0x441, 0x412,
0x456, 0x43d, 0x43d, 0x456, 0x43f, 0x435, 0x491, 0x41a, 0x435, 0x439, 0x441, 0x456,
0x414, 0x435, 0x432, 0x456, 0x441, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434,
0x2bc, 0x42e, 0x440, 0x432, 0x456, 0x43b, 0x44c, 0x41c, 0x430, 0x43a, 0x43a, 0x443,
0x43e, 0x440, 0x456, 0x421, 0x44c, 0x43e, 0x432, 0x430, 0x41b, 0x43e, 0x43d, 0x433,
0x2bc, 0x457, 0x440, 0x414, 0x456, 0x43b, 0x456, 0x425, 0x43e, 0x448, 0x438, 0x43c,
0x456, 0x43d, 0x406, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x404, 0x440,
0x443, 0x441, 0x430, 0x43b, 0x438, 0x43c, 0x41a, 0x430, 0x440, 0x430, 0x447, 0x456,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x44c, 0x43a, 0x41d,
0x456, 0x43a, 0x43e, 0x441, 0x456, 0x44f, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x443,
0x437, 0x43d, 0x435, 0x446, 0x44c, 0x43a, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x41e, 0x43c, 0x441, 0x44c, 0x43a, 0x41f,
0x43e, 0x43d, 0x442, 0x456, 0x430, 0x43d, 0x430, 0x43a, 0x415, 0x440, 0x2d, 0x420,
0x456, 0x44f, 0x434, 0x421, 0x435, 0x440, 0x435, 0x434, 0x43d, 0x44c, 0x43e, 0x43a,
0x43e, 0x43b, 0x438, 0x43c, 0x441, 0x44c, 0x43a, 0x422, 0x430, 0x439, 0x431, 0x435,
0x439, 0x422, 0x43e, 0x43a, 0x456, 0x43e, 0x422, 0x43e, 0x43c, 0x441, 0x44c, 0x43a,
0x423, 0x440, 0x443, 0x43c, 0x447, 0x456, 0x412, 0x2bc, 0x454, 0x43d, 0x442, 0x44c,
0x44f, 0x43d, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x404, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x404, 0x440, 0x435,
0x432, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x44c, 0x43a, 0x456, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x430, 0x43d, 0x430, 0x440,
0x441, 0x44c, 0x43a, 0x456, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x424, 0x430, 0x440, 0x435, 0x440, 0x441, 0x44c, 0x43a, 0x456, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x420, 0x435, 0x439, 0x43a, 0x2bc, 0x44f, 0x432,
0x456, 0x43a, 0x41f, 0x456, 0x432, 0x434, 0x435, 0x43d, 0x43d, 0x430, 0x20, 0x414,
0x436, 0x43e, 0x440, 0x434, 0x436, 0x456, 0x44f, 0x41e, 0x441, 0x442, 0x440, 0x456,
0x432, 0x20, 0x421, 0x432, 0x44f, 0x442, 0x43e, 0x457, 0x20, 0x404, 0x43b, 0x435,
0x43d, 0x438, 0x421, 0x442, 0x435, 0x43d, 0x43b, 0x456, 0x410, 0x434, 0x435, 0x43b,
0x430, 0x457, 0x434, 0x430, 0x411, 0x440, 0x456, 0x441, 0x431, 0x435, 0x43d, 0x411,
0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x2d, 0x425, 0x456, 0x43b, 0x43b, 0x415, 0x432,
0x43a, 0x43b, 0x430, 0x413, 0x43e, 0x431, 0x430, 0x440, 0x442, 0x41b, 0x456, 0x43d,
0x434, 0x435, 0x43c, 0x430, 0x43d, 0x410, 0x444, 0x456, 0x43d, 0x438, 0x41a, 0x438,
0x448, 0x438, 0x43d, 0x456, 0x432, 0x413, 0x435, 0x43b, 0x44c, 0x441, 0x456, 0x43d,
0x43a, 0x456, 0x41e, 0x441, 0x442, 0x440, 0x456, 0x432, 0x20, 0x41c, 0x435, 0x43d,
0x414, 0x436, 0x435, 0x440, 0x441, 0x456, 0x41a, 0x456, 0x440, 0x43e, 0x432, 0x41a,
0x438, 0x457, 0x432, 0x41c, 0x430, 0x440, 0x456, 0x454, 0x433, 0x430, 0x43c, 0x43d,
0x41c, 0x456, 0x43d, 0x441, 0x44c, 0x43a, 0x41f, 0x43e, 0x434, 0x433, 0x43e, 0x440,
0x438, 0x446, 0x44f, 0x421, 0x430, 0x440, 0x430, 0x454, 0x432, 0x43e, 0x421, 0x456,
0x43c, 0x444, 0x435, 0x440, 0x43e, 0x43f, 0x43e, 0x43b, 0x44c, 0x421, 0x43a, 0x43e,
0x43f, 0x2bc, 0x454, 0x421, 0x43e, 0x444, 0x456, 0x44f, 0x422, 0x430, 0x43b, 0x43b,
0x456, 0x43d, 0x43d, 0x423, 0x43b, 0x44c, 0x44f, 0x43d, 0x456, 0x432, 0x441, 0x44c,
0x43a, 0x412, 0x456, 0x434, 0x435, 0x43d, 0x44c, 0x426, 0x44e, 0x440, 0x456, 0x445,
0x410, 0x43d, 0x442, 0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x456, 0x432, 0x443,
0x427, 0x430, 0x491, 0x43e, 0x441, 0x41e, 0x441, 0x442, 0x440, 0x456, 0x432, 0x20,
0x420, 0x456, 0x437, 0x434, 0x432, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e,
0x432, 0x456, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x435,
0x440, 0x491, 0x435, 0x43b, 0x435, 0x43d, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x456,
0x432, 0x438, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x456, 0x439, 0x420, 0x435,
0x44e, 0x43d, 0x44c, 0x439, 0x43e, 0x43d, 0x410, 0x43f, 0x456, 0x430, 0x411, 0x443,
0x491, 0x435, 0x43d, 0x432, 0x456, 0x43b, 0x44c, 0x41e, 0x441, 0x442, 0x440, 0x456,
0x432, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x424, 0x456, 0x434, 0x436, 0x456,
0x424, 0x443, 0x43d, 0x430, 0x444, 0x443, 0x442, 0x456, 0x490, 0x430, 0x43b, 0x430,
0x43f, 0x430, 0x491, 0x43e, 0x441, 0x413, 0x430, 0x43c, 0x431, 0x2bc, 0x454, 0x415,
0x43d, 0x434, 0x435, 0x440, 0x431, 0x435, 0x440, 0x456, 0x41a, 0x456, 0x440, 0x456,
0x442, 0x456, 0x43c, 0x430, 0x442, 0x456, 0x41c, 0x430, 0x440, 0x43a, 0x456, 0x437,
0x44c, 0x43a, 0x456, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41c,
0x456, 0x434, 0x432, 0x435, 0x439, 0x41d, 0x456, 0x443, 0x435, 0x41f, 0x456, 0x442,
0x43a, 0x435, 0x440, 0x43d, 0x422, 0x430, 0x457, 0x442, 0x456, 0x423, 0x43e, 0x43b,
0x43b, 0x456, 0x441, 0x44, 0x17a, 0x69, 0x62, 0x75, 0x74, 0x69, 0x4b, 0x61,
0x69, 0x6d, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79,
0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x20, 0x6d, 0x11b, 0x73, 0x74, 0x6f,
0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x53, 0x65, 0x77, 0x6a,
0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43,
0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x53, 0x65, 0x77, 0x6a, 0x65,
0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65,
0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x65, 0x77,
0x6a, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x61, 0x2f, 0x57,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20,
0x4d, 0x69, 0x6e, 0x68, 0x6f, 0x77, 0x65, 0x20, 0x6d, 0x11b, 0x73, 0x74,
0x6f, 0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x6a, 0x65, 0x63,
0x6b, 0x4a, 0x65, 0x72, 0x6a, 0x65, 0x77, 0x61, 0x6e, 0x4b, 0x61, 0x6e,
0x61, 0x72, 0x69, 0x73, 0x6b, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x46,
0xe4, 0x72, 0xf6, 0x73, 0x6b, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x4a,
0x75, 0x17e, 0x6e, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73,
0x6b, 0x61, 0x42, 0x11b, 0x142, 0x6f, 0x68, 0x72, 0xf3, 0x64, 0x4b, 0x69,
0x161, 0x69, 0x6e, 0x6a, 0x6f, 0x77, 0x48, 0x6f, 0x64, 0x6f, 0x77, 0x6e,
0x61, 0x20, 0x6b, 0x75, 0x70, 0x61, 0x4a, 0x75, 0x74, 0x72, 0x6f, 0x77,
0x6e, 0x61, 0x20, 0x6b, 0x75, 0x70, 0x61, 0x46, 0x69, 0x64, 0x17a, 0x69,
0x639, 0x627, 0x628, 0x62f, 0x62c, 0x627, 0x646, 0x627, 0x6a9, 0x651, 0x631, 0x627,
0x627, 0x644, 0x62c, 0x6cc, 0x626, 0x631, 0x633, 0x628, 0x646, 0x6af, 0x648, 0x626,
0x6cc, 0x628, 0x646, 0x62c, 0x648, 0x644, 0x628, 0x644, 0x6cc, 0x646, 0x679, 0x627,
0x626, 0x631, 0x628, 0x631, 0x627, 0x632, 0x627, 0x648, 0x6cc, 0x644, 0x6d2, 0x628,
0x62c, 0x645, 0x628, 0x631, 0x627, 0x642, 0x627, 0x6c1, 0x631, 0x6c1, 0x6a9, 0x6cc,
0x633, 0x627, 0x20, 0x628, 0x644, 0x627, 0x646, 0x6a9, 0x627, 0x633, 0x6cc, 0x648,
0x679, 0x627, 0x6a9, 0x648, 0x646, 0x6a9, 0x631, 0x6cc, 0x688, 0x6a9, 0x627, 0x631,
0x62c, 0x628, 0x648, 0x62a, 0x6cc, 0x688, 0x648, 0x622, 0x644, 0x627, 0x641, 0x631,
0x6cc, 0x20, 0x679, 0x627, 0x624, 0x646, 0x6af, 0x628, 0x631, 0x648, 0x646, 0x6c1,
0x631, 0x627, 0x631, 0x6d2, 0x6a9, 0x6cc, 0x645, 0x67e, 0x627, 0x644, 0x627, 0x6a9,
0x6af, 0x627, 0x644, 0x6cc, 0x6a9, 0x646, 0x634, 0x627, 0x633, 0x627, 0x644, 0x628,
0x631, 0x6d2, 0x20, 0x648, 0x6cc, 0x644, 0x6d2, 0x644, 0x648, 0x627, 0x646, 0x688,
0x627, 0x644, 0x6cc, 0x648, 0x633, 0x627, 0x6a9, 0x627, 0x645, 0x67e, 0x648, 0x679,
0x648, 0x645, 0x633, 0x6cc, 0x631, 0x648, 0x645, 0x628, 0x627, 0x628, 0x6cc, 0x646,
0x627, 0x6cc, 0x646, 0x62c, 0x627, 0x645, 0x6cc, 0x646, 0x627, 0x646, 0x648, 0x627,
0x6a9, 0x634, 0x648, 0x637, 0x627, 0x624, 0x6af, 0x627, 0x688, 0x624, 0x6af, 0x648,
0x648, 0x633, 0x627, 0x624, 0x20, 0x679, 0x648, 0x645, 0x679, 0x631, 0x6cc, 0x67e,
0x648, 0x644, 0x6cc, 0x62a, 0x6cc, 0x648, 0x646, 0x633, 0x648, 0x646, 0x688, 0x6c1,
0x648, 0x6a9, 0x627, 0x646, 0x6af, 0x648, 0x6cc, 0x644, 0x627, 0x627, 0x646, 0x679,
0x6cc, 0x6af, 0x648, 0x627, 0x628, 0x6cc, 0x648, 0x646, 0x633, 0x20, 0x622, 0x626,
0x631, 0x633, 0x6a9, 0x648, 0x631, 0x688, 0x648, 0x628, 0x627, 0x62c, 0x648, 0x62c,
0x648, 0x626, 0x6cc, 0x645, 0x6cc, 0x646, 0x688, 0x648, 0x632, 0x627, 0x633, 0x627,
0x646, 0x20, 0x644, 0x648, 0x626, 0x633, 0x679, 0x648, 0x6a9, 0x648, 0x645, 0x6cc,
0x646, 0x627, 0x633, 0x646, 0x633, 0x6cc, 0x626, 0x646, 0x627, 0x679, 0x6cc, 0x6a9,
0x648, 0x6a9, 0x646, 0x628, 0x6c1, 0x6cc, 0x627, 0x20, 0x628, 0x646, 0x62f, 0x631,
0x627, 0x632, 0x628, 0x6cc, 0x644, 0x6cc, 0x645, 0x628, 0x6cc, 0x644, 0x627, 0x626,
0x632, 0x628, 0x644, 0x627, 0x646, 0x6a9, 0x20, 0x633, 0x628, 0x644, 0x648, 0x646,
0x628, 0x648, 0x622, 0x20, 0x648, 0x633, 0x679, 0x627, 0x628, 0x6af, 0x648, 0x679,
0x627, 0x628, 0x648, 0x627, 0x626, 0x633, 0x6a9, 0x6cc, 0x645, 0x628, 0x631, 0x62c,
0x20, 0x6a9, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc, 0x6a9, 0x6cc, 0x645,
0x67e, 0x648, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x688, 0x6a9, 0x646, 0x6a9, 0x6cc,
0x648, 0x646, 0x6a9, 0x631, 0x627, 0x6a9, 0x627, 0x633, 0x6a9, 0x627, 0x626, 0x6cc,
0x646, 0x6a9, 0x6cc, 0x645, 0x6cc, 0x646, 0x634, 0x6a9, 0x627, 0x6af, 0x648, 0x686,
0x6cc, 0x6c1, 0x648, 0x622, 0x6c1, 0x648, 0x622, 0x6a9, 0x648, 0x633, 0x679, 0x627,
0x20, 0x631, 0x6cc, 0x6a9, 0x627, 0x6a9, 0x631, 0x6cc, 0x633, 0x679, 0x648, 0x646,
0x6a9, 0x648, 0x626, 0x6cc, 0x627, 0x628, 0x627, 0x6a9, 0x6cc, 0x648, 0x631, 0x627,
0x6a9, 0x627, 0x624, 0x688, 0x627, 0x624, 0x633, 0x646, 0x688, 0x627, 0x624, 0x633,
0x646, 0x20, 0x6a9, 0x631, 0x6cc, 0x6a9, 0x688, 0x6cc, 0x646, 0x648, 0x631, 0x688,
0x6cc, 0x679, 0x631, 0x627, 0x626, 0x679, 0x688, 0x648, 0x645, 0x646, 0x6cc, 0x6a9,
0x627, 0x627, 0x6cc, 0x688, 0x645, 0x648, 0x646, 0x679, 0x646, 0x627, 0x6cc, 0x631,
0x648, 0x646, 0x6cc, 0x67e, 0x627, 0x644, 0x20, 0x633, 0x644, 0x648, 0x627, 0x688,
0x648, 0x631, 0x641, 0x648, 0x631, 0x679, 0x627, 0x644, 0x6cc, 0x632, 0x627, 0x6af,
0x644, 0x6cc, 0x633, 0x20, 0x6a9, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc,
0x6af, 0x648, 0x633, 0x20, 0x6a9, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc,
0x639, 0x638, 0x6cc, 0x645, 0x20, 0x62a, 0x631, 0x6a9, 0x63a, 0x631, 0x646, 0x627,
0x637, 0x6c1, 0x6af, 0x648, 0x627, 0x688, 0x6cc, 0x644, 0x648, 0x67e, 0x6af, 0x648,
0x627, 0x679, 0x6d2, 0x20, 0x645, 0x627, 0x644, 0x627, 0x6af, 0x648, 0x622, 0x6cc,
0x627, 0x6a9, 0x648, 0x626, 0x644, 0x6af, 0x6cc, 0x627, 0x646, 0x627, 0x6c1, 0x6cc,
0x644, 0x6cc, 0x641, 0x6cc, 0x6a9, 0x633, 0x6c1, 0x648, 0x627, 0x646, 0x627, 0x6c1,
0x631, 0x645, 0x648, 0x633, 0x6cc, 0x644, 0x648, 0x627, 0x646, 0x688, 0x6cc, 0x627,
0x646, 0x627, 0x67e, 0x648, 0x644, 0x633, 0x6a9, 0x646, 0x648, 0x6a9, 0x633, 0x60c,
0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x645, 0x631, 0x6cc, 0x646,
0x6af, 0x648, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x67e,
0x6cc, 0x679, 0x631, 0x632, 0x628, 0x631, 0x6af, 0x60c, 0x20, 0x627, 0x646, 0x688,
0x6cc, 0x627, 0x646, 0x627, 0x679, 0x6cc, 0x644, 0x20, 0x633, 0x679, 0x6cc, 0x60c,
0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x648, 0x6d2,
0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x646, 0x633,
0x6cc, 0x646, 0x6cc, 0x632, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646,
0x627, 0x648, 0x6cc, 0x646, 0x627, 0x645, 0x6cc, 0x6a9, 0x60c, 0x20, 0x627, 0x646,
0x688, 0x6cc, 0x627, 0x646, 0x627, 0x627, 0x646, 0x648, 0x648, 0x650, 0x6a9, 0x627,
0x6cc, 0x6a9, 0x627, 0x644, 0x648, 0x626, 0x679, 0x62c, 0x645, 0x627, 0x626, 0x6cc,
0x6a9, 0x627, 0x62c, 0x648, 0x646, 0x6cc, 0x626, 0x648, 0x644, 0x648, 0x626, 0x633,
0x20, 0x648, 0x6cc, 0x644, 0x6d2, 0x645, 0x648, 0x646, 0x679, 0x6cc, 0x633, 0x6cc,
0x644, 0x648, 0x60c, 0x20, 0x6a9, 0x6cc, 0x646, 0x679, 0x648, 0x6a9, 0x6cc, 0x6a9,
0x631, 0x627, 0x644, 0x6cc, 0x646, 0x688, 0x6cc, 0x62c, 0x6a9, 0x644, 0x648, 0x648,
0x631, 0x20, 0x67e, 0x631, 0x646, 0x633, 0x633, 0x20, 0x6a9, 0x648, 0x627, 0x631,
0x679, 0x631, 0x645, 0x646, 0x627, 0x6af, 0x648, 0x622, 0x645, 0x646, 0x627, 0x624,
0x633, 0x645, 0x627, 0x631, 0x679, 0x6cc, 0x646, 0x6a9, 0x645, 0x6cc, 0x632, 0x679,
0x644, 0x627, 0x646, 0x645, 0x6cc, 0x646, 0x648, 0x645, 0x6cc, 0x646, 0x6cc, 0x645,
0x6cc, 0x631, 0x6cc, 0x688, 0x627, 0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x648,
0x20, 0x633, 0x679, 0x6cc, 0x645, 0x6cc, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646,
0x645, 0x648, 0x646, 0x6a9, 0x679, 0x646, 0x645, 0x648, 0x646, 0x679, 0x6cc, 0x631,
0x6cc, 0x645, 0x648, 0x646, 0x679, 0x6cc, 0x20, 0x648, 0x6cc, 0x688, 0x6cc, 0x648,
0x645, 0x648, 0x646, 0x679, 0x633, 0x6cc, 0x631, 0x627, 0x679, 0x646, 0x6cc, 0x648,
0x20, 0x6cc, 0x627, 0x631, 0x6a9, 0x628, 0x6cc, 0x648, 0x644, 0x627, 0x6c1, 0x60c,
0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688, 0x6a9, 0x648, 0x679, 0x627,
0x648, 0x633, 0x637, 0x60c, 0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688,
0x6a9, 0x648, 0x679, 0x627, 0x646, 0x6cc, 0x648, 0x20, 0x633, 0x644, 0x6cc, 0x645,
0x60c, 0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688, 0x6a9, 0x648, 0x679,
0x627, 0x67e, 0x646, 0x627, 0x645, 0x6c1, 0x67e, 0x631, 0x627, 0x645, 0x627, 0x631,
0x6cc, 0x628, 0x648, 0x67e, 0x648, 0x631, 0x679, 0x20, 0x627, 0x648, 0x20, 0x67e,
0x631, 0x646, 0x633, 0x67e, 0x648, 0x631, 0x679, 0x20, 0x622, 0x641, 0x20, 0x627,
0x633, 0x67e, 0x6cc, 0x646, 0x67e, 0x648, 0x631, 0x679, 0x648, 0x20, 0x648, 0x6cc,
0x644, 0x6c1, 0x648, 0x67e, 0x6cc, 0x648, 0x631, 0x679, 0x648, 0x20, 0x631, 0x6cc,
0x6a9, 0x648, 0x67e, 0x646, 0x679, 0x627, 0x20, 0x627, 0x631, 0x6cc, 0x646, 0x627,
0x633, 0x631, 0x6cc, 0x646, 0x6a9, 0x646, 0x20, 0x627, 0x646, 0x644, 0x6cc, 0x679,
0x631, 0x6cc, 0x633, 0x627, 0x626, 0x641, 0x631, 0x6cc, 0x632, 0x648, 0x644, 0x6cc,
0x648, 0x679, 0x631, 0x6cc, 0x626, 0x648, 0x20, 0x628, 0x631, 0x6cc, 0x646, 0x6a9,
0x648, 0x633, 0x646, 0x679, 0x627, 0x631, 0x6cc, 0x646, 0x633, 0x646, 0x679, 0x6cc,
0x627, 0x6af, 0x648, 0x633, 0x627, 0x646, 0x62a, 0x648, 0x20, 0x688, 0x648, 0x645,
0x646, 0x6af, 0x648, 0x627, 0x633, 0x6a9, 0x648, 0x631, 0x632, 0x20, 0x628, 0x627,
0x626, 0x6cc, 0x20, 0x633, 0x646, 0x688, 0x633, 0x6cc, 0x679, 0x6a9, 0x627, 0x633,
0x6cc, 0x646, 0x679, 0x20, 0x628, 0x631, 0x62a, 0x6be, 0x6cc, 0x644, 0x645, 0x6cc,
0x633, 0x6cc, 0x646, 0x679, 0x20, 0x62c, 0x627, 0x646, 0x632, 0x633, 0x6cc, 0x646,
0x679, 0x20, 0x6a9, 0x679, 0x633, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x644, 0x648,
0x633, 0x6cc, 0x627, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x62a, 0x6be, 0x627, 0x645,
0x633, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x648, 0x646, 0x633, 0x646, 0x679, 0x633,
0x648, 0x650, 0x641, 0x679, 0x20, 0x6a9, 0x631, 0x646, 0x679, 0x679, 0x6cc, 0x6af,
0x648, 0x633, 0x6cc, 0x6af, 0x627, 0x644, 0x67e, 0x6d2, 0x62a, 0x6be, 0x648, 0x644,
0x648, 0x62a, 0x6cc, 0x62c, 0x648, 0x622, 0x646, 0x627, 0x679, 0x648, 0x631, 0x646,
0x679, 0x648, 0x679, 0x648, 0x631, 0x679, 0x648, 0x644, 0x627, 0x648, 0x6c1, 0x627,
0x626, 0x679, 0x20, 0x6c1, 0x627, 0x631, 0x633, 0x648, 0x646, 0x651, 0x6cc, 0x67e,
0x6cc, 0x6af, 0x6cc, 0x6a9, 0x648, 0x679, 0x6cc, 0x679, 0x688, 0x6cc, 0x648, 0x633,
0x688, 0x648, 0x645, 0x648, 0x646, 0x679, 0x20, 0x688, 0x6cc, 0x20, 0x627, 0x631,
0x648, 0x6cc, 0x644, 0x6d2, 0x645, 0x6cc, 0x6a9, 0x648, 0x627, 0x631, 0x6cc, 0x645,
0x627, 0x624, 0x633, 0x646, 0x645, 0x6cc, 0x6a9, 0x20, 0x645, 0x631, 0x688, 0x648,
0x67e, 0x644, 0x645, 0x6cc, 0x631, 0x679, 0x631, 0x648, 0x644, 0x644, 0x627, 0x646,
0x6af, 0x20, 0x627, 0x6cc, 0x626, 0x631, 0x20, 0x628, 0x6cc, 0x646, 0x627, 0x644,
0x645, 0x627, 0x679, 0x6cc, 0x627, 0x645, 0x651, 0x627, 0x646, 0x627, 0x646, 0x6cc,
0x62f, 0x631, 0x627, 0x6a9, 0x62a, 0x627, 0x624, 0x627, 0x6a9, 0x679, 0x648, 0x628,
0x627, 0x634, 0x63a, 0x628, 0x62a, 0x622, 0x62a, 0x6cc, 0x631, 0x627, 0x624, 0x628,
0x631, 0x646, 0x627, 0x644, 0x688, 0x6be, 0x627, 0x6a9, 0x6c1, 0x688, 0x644, 0x6cc,
0x62f, 0x628, 0x626, 0x6cc, 0x62f, 0x648, 0x634, 0x627, 0x646, 0x628, 0x6d2, 0x641,
0x627, 0x645, 0x627, 0x6af, 0x648, 0x633, 0x679, 0x627, 0x6c1, 0x648, 0x20, 0x686,
0x6cc, 0x20, 0x645, 0x646, 0x6c1, 0x20, 0x633, 0x679, 0x6cc, 0x6c1, 0x648, 0x648,
0x627, 0x631, 0x688, 0x627, 0x631, 0x6a9, 0x62a, 0x633, 0x6a9, 0x62c, 0x6a9, 0x627,
0x631, 0x62a, 0x6c1, 0x62c, 0x6d2, 0x20, 0x67e, 0x648, 0x631, 0x6c1, 0x6a9, 0x6cc,
0x645, 0x686, 0x679, 0x6a9, 0x627, 0x6a9, 0x627, 0x679, 0x6be, 0x645, 0x646, 0x688,
0x648, 0x62e, 0x646, 0x62f, 0x6cc, 0x6af, 0x627, 0x6a9, 0x648, 0x644, 0x6a9, 0x627,
0x62a, 0x627, 0x6a9, 0x631, 0x6cc, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633,
0x6a9, 0x6a9, 0x648, 0x627, 0x644, 0x627, 0x20, 0x644, 0x645, 0x67e, 0x648, 0x631,
0x6a9, 0x6cc, 0x648, 0x686, 0x646, 0x6af, 0x645, 0x6a9, 0x627, 0x624, 0x645, 0x6cc,
0x6af, 0x6cc, 0x62f, 0x646, 0x645, 0x6a9, 0x627, 0x633, 0x631, 0x646, 0x6a9, 0x648,
0x633, 0x6cc, 0x627, 0x646, 0x648, 0x648, 0x648, 0x6a9, 0x6cc, 0x648, 0x632, 0x646,
0x6cc, 0x633, 0x6a9, 0x67e, 0x648, 0x646, 0x679, 0x6cc, 0x627, 0x646, 0x6a9, 0x67e,
0x6cc, 0x648, 0x646, 0x6af, 0x20, 0x6cc, 0x627, 0x646, 0x6af, 0x6a9, 0x648, 0x633,
0x62a, 0x627, 0x646, 0x6d2, 0x6a9, 0x6cc, 0x632, 0x6cc, 0x644, 0x648, 0x631, 0x688,
0x627, 0x633, 0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x633, 0x6cc, 0x626, 0x648, 0x644,
0x633, 0x631, 0x6c1, 0x62f, 0x646, 0x6cc, 0x6a9, 0x648, 0x644, 0x6cc, 0x645, 0x633,
0x6a9, 0x62a, 0x627, 0x626, 0x67e, 0x6d2, 0x637, 0x628, 0x644, 0x6cc, 0x633, 0x6cc,
0x62a, 0x6c1, 0x631, 0x627, 0x646, 0x62a, 0x6be, 0x645, 0x67e, 0x648, 0x679, 0x627,
0x645, 0x633, 0x6a9, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x62a,
0x627, 0x631, 0x6cc, 0x648, 0x631, 0x648, 0x645, 0x6a9, 0x6cc, 0x627, 0x648, 0x633,
0x62a, 0x2d, 0x646, 0x6cc, 0x631, 0x627, 0x648, 0x6cc, 0x646, 0x679, 0x6cc, 0x627,
0x646, 0x627, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x648, 0x633, 0x62a,
0x6a9, 0x6cc, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x6cc, 0x6a9, 0x627, 0x679, 0x6cc,
0x631, 0x650, 0x646, 0x628, 0x631, 0x6af, 0x6cc, 0x631, 0x6cc, 0x648, 0x627, 0x646,
0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631,
0x688, 0x6cc, 0x645, 0x688, 0x6cc, 0x626, 0x631, 0x627, 0x631, 0x6cc, 0x6a9, 0x62c,
0x627, 0x648, 0x6a9, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x6c1, 0x6cc, 0x644, 0x6cc,
0x646, 0x627, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x644, 0x6d2, 0x627, 0x6cc, 0x688,
0x6cc, 0x644, 0x6cc, 0x688, 0x628, 0x631, 0x648, 0x6a9, 0x646, 0x20, 0x6c1, 0x650,
0x644, 0x688, 0x627, 0x631, 0x648, 0x646, 0x627, 0x6cc, 0x648, 0x6a9, 0x644, 0x627,
0x6c1, 0x648, 0x628, 0x627, 0x631, 0x679, 0x644, 0x650, 0x646, 0x688, 0x645, 0x6cc,
0x646, 0x644, 0x627, 0x631, 0x688, 0x20, 0x6c1, 0x648, 0x648, 0x6d2, 0x67e, 0x631,
0x62a, 0x6be, 0x633, 0x688, 0x646, 0x6cc, 0x627, 0x6cc, 0x645, 0x633, 0x679, 0x631,
0x688, 0x645, 0x627, 0x646, 0x688, 0x648, 0x631, 0x627, 0x627, 0x633, 0x62a, 0x631,
0x627, 0x62e, 0x627, 0x646, 0x627, 0x6cc, 0x62a, 0x6be, 0x646, 0x632, 0x628, 0x631,
0x6cc, 0x679, 0x650, 0x633, 0x644, 0x627, 0x648, 0x627, 0x628, 0x62e, 0x627, 0x631,
0x633, 0x679, 0x628, 0x688, 0x627, 0x67e, 0x633, 0x679, 0x628, 0x632, 0x646, 0x62c,
0x646, 0x686, 0x6cc, 0x633, 0x6cc, 0x646, 0x627, 0x624, 0x688, 0x628, 0x644, 0x646,
0x6c1, 0x6cc, 0x644, 0x633, 0x646, 0x6a9, 0x6cc, 0x6a9, 0x627, 0x644, 0x64a, 0x646,
0x64a, 0x646, 0x63a, 0x631, 0x627, 0x62f, 0x644, 0x6af, 0x698, 0x645, 0x628, 0x631,
0x6af, 0x645, 0x6cc, 0x688, 0x631, 0x688, 0x645, 0x6cc, 0x631, 0x6cc, 0x6c1, 0x627,
0x645, 0x67e, 0x6cc, 0x631, 0x633, 0x67e, 0x648, 0x688, 0x6af, 0x648, 0x631, 0x633,
0x6cc, 0x627, 0x633, 0x627, 0x646, 0x20, 0x645, 0x627, 0x631, 0x6cc, 0x627, 0x646,
0x648, 0x633, 0x6cc, 0x631, 0x627, 0x679, 0x648, 0x627, 0x633, 0x6a9, 0x648, 0x67e,
0x62c, 0x6d2, 0x635, 0x648, 0x641, 0x6cc, 0x6c1, 0x627, 0x633, 0x679, 0x627, 0x6a9,
0x20, 0x6c1, 0x648, 0x645, 0x679, 0x627, 0x644, 0x646, 0x679, 0x6cc, 0x631, 0x627,
0x646, 0x6cc, 0x627, 0x644, 0x6cc, 0x627, 0x646, 0x648, 0x633, 0x6a9, 0x648, 0x6cc,
0x688, 0x648, 0x632, 0x648, 0x627, 0x679, 0x6cc, 0x6a9, 0x646, 0x648, 0x650, 0x644,
0x646, 0x6cc, 0x626, 0x633, 0x632, 0x6cc, 0x6af, 0x631, 0x6cc, 0x628, 0x632, 0x6cc,
0x648, 0x631, 0x62e, 0x627, 0x646, 0x679, 0x627, 0x646, 0x627, 0x646, 0x627, 0x631,
0x6cc, 0x648, 0x648, 0x6a9, 0x631, 0x633, 0x645, 0x633, 0x6a9, 0x631, 0x6af, 0x6cc,
0x648, 0x644, 0x6cc, 0x646, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x67e, 0x645, 0x627,
0x631, 0x6cc, 0x634, 0x633, 0x645, 0x627, 0x6cc, 0x648, 0x679, 0x631, 0x6cc, 0x20,
0x6cc, 0x648, 0x646, 0x6cc, 0x646, 0x628, 0x648, 0x6af, 0x6cc, 0x646, 0x648, 0x644,
0x6d2, 0x686, 0x6cc, 0x62a, 0x6be, 0x645, 0x686, 0x6cc, 0x648, 0x6a9, 0x627, 0x6cc,
0x641, 0x6cc, 0x679, 0x641, 0x6a9, 0x627, 0x624, 0x641, 0x648, 0x641, 0x62c, 0x6cc,
0x641, 0x6cc, 0x648, 0x646, 0x627, 0x641, 0x6cc, 0x648, 0x679, 0x6cc, 0x6af, 0x6cc,
0x644, 0x627, 0x67e, 0x6cc, 0x6af, 0x648, 0x633, 0x6af, 0x627, 0x645, 0x628, 0x6cc,
0x626, 0x631, 0x6af, 0x648, 0x627, 0x688, 0x644, 0x20, 0x6a9, 0x6cc, 0x646, 0x627,
0x644, 0x6c1, 0x648, 0x646, 0x648, 0x644, 0x648, 0x644, 0x648, 0x627, 0x6cc, 0x646,
0x688, 0x631, 0x628, 0x631, 0x6cc, 0x6a9, 0x631, 0x6cc, 0x62a, 0x6cc, 0x645, 0x627,
0x679, 0x6cc, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x645, 0x627, 0x631,
0x6a9, 0x6cc, 0x633, 0x627, 0x633, 0x645, 0x688, 0x648, 0x6d2, 0x646, 0x627, 0x624,
0x631, 0x648, 0x646, 0x624, 0x645, 0x6cc, 0x627, 0x67e, 0x644, 0x627, 0x624, 0x67e,
0x679, 0x6a9, 0x627, 0x626, 0x631, 0x646, 0x67e, 0x648, 0x646, 0x67e, 0x6cc, 0x626,
0x6cc, 0x67e, 0x648, 0x631, 0x679, 0x20, 0x645, 0x648, 0x631, 0x6cc, 0x633, 0x628,
0x6cc, 0x633, 0x627, 0x626, 0x67e, 0x6cc, 0x646, 0x62a, 0x627, 0x6c1, 0x6cc, 0x62a,
0x6cc, 0x679, 0x631, 0x627, 0x648, 0x627, 0x679, 0x648, 0x646, 0x6af, 0x627, 0x679,
0x627, 0x67e, 0x648, 0x648, 0x644, 0x651, 0x6cc, 0x633, 0x6a9, 0x6cc, 0x645, 0x628,
0x631, 0x62c, 0x20, 0x628, 0x6d2, 0x6af, 0x644, 0x6cc, 0x633, 0x20, 0x628, 0x6d2,
0x6af, 0x648, 0x633, 0x20, 0x628, 0x6d2, 0x6af, 0x631, 0x6cc, 0x646, 0x688, 0x20,
0x62a, 0x631, 0x6a9, 0x627, 0x62a, 0x62a, 0x648, 0x642, 0x642, 0x648, 0x631, 0x62a,
0x648, 0x631, 0x645, 0x6cc, 0x62a, 0x62a, 0x6be, 0x648, 0x644, 0x6d2, 0x628, 0x648,
0x688, 0x627, 0x67e, 0x6cc, 0x633, 0x679, 0x633, 0x627, 0x646, 0x2d, 0x62a, 0x648,
0x645, 0x6d0, 0x626, 0x627, 0x633, 0x6c7, 0x646, 0x633, 0x649, 0x64a, 0x648, 0x646,
0x643, 0x6c7, 0x631, 0x627, 0x633, 0x648, 0x633, 0x627, 0x64a, 0x646, 0x649, 0x62a,
0x2d, 0x628, 0x627, 0x631, 0x62a, 0x6be, 0x6d0, 0x644, 0x6d5, 0x645, 0x649, 0x64a,
0x62f, 0x6c7, 0x645, 0x648, 0x646, 0x62a, 0x20, 0x62f, 0x6c7, 0x631, 0x6cb, 0x649,
0x644, 0x644, 0x6d0, 0x62e, 0x648, 0x686, 0x649, 0x645, 0x649, 0x646, 0x20, 0x634,
0x6d5, 0x6be, 0x649, 0x631, 0x649, 0x631, 0x6d0, 0x626, 0x648, 0x646, 0x649, 0x64a,
0x648, 0x646, 0x4a, 0x61, 0x7a, 0x6f, 0x69, 0x72, 0x42, 0x6c, 0x61, 0x6e,
0x74, 0x61, 0x79, 0x72, 0x51, 0x6f, 0x68, 0x69, 0x72, 0x61, 0x44, 0x6f,
0x72, 0x2d, 0x75, 0x73, 0x2d, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x41, 0x6c,
0x2d, 0x41, 0x79, 0x75, 0x6e, 0x58, 0x61, 0x72, 0x61, 0x72, 0x65, 0x59,
0x6f, 0x78, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x4d,
0x6f, 0x67, 0x61, 0x64, 0x69, 0x73, 0x68, 0x6f, 0x4d, 0x6f, 0x6e, 0x72,
0x6f, 0x76, 0x69, 0x79, 0x61, 0x4e, 0x61, 0x79, 0x72, 0x6f, 0x62, 0x69,
0x56, 0x69, 0x6e, 0x64, 0x78, 0x75, 0x6b, 0x41, 0x64, 0x61, 0x6b, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x41, 0x6e, 0x6b, 0x6f, 0x72, 0x69, 0x6a,
0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x2d, 0x41, 0x79, 0x72, 0x65, 0x73,
0x4c, 0x61, 0x2d, 0x52, 0x69, 0x6f, 0x78, 0x61, 0x52, 0x69, 0x6f, 0x2d,
0x47, 0x61, 0x6c, 0x79, 0x65, 0x67, 0x6f, 0x73, 0x53, 0x61, 0x6e, 0x2d,
0x58, 0x75, 0x61, 0x6e, 0x55, 0x73, 0x68, 0x75, 0x61, 0x79, 0x61, 0x41,
0x73, 0x75, 0x6e, 0x73, 0x6f, 0x6e, 0x4b, 0x6f, 0x72, 0x61, 0x6c, 0x2d,
0x58, 0x61, 0x72, 0x62, 0x6f, 0x72, 0x42, 0x61, 0x69, 0x79, 0x61, 0x42,
0x61, 0x68, 0x69, 0x79, 0x61, 0x2d, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x61, 0x73, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x6a, 0x2d, 0x42, 0x65,
0x79, 0x4b, 0x61, 0x79, 0x65, 0x6e, 0x6e, 0x61, 0x4b, 0x61, 0x79, 0x6d,
0x61, 0x6e, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x43,
0x68, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0x4b, 0x79, 0x75, 0x72, 0x61, 0x73,
0x61, 0x6f, 0x44, 0x65, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x78, 0x61,
0x76, 0x6e, 0x45, 0x79, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x47, 0x6c,
0x65, 0x79, 0x73, 0x2d, 0x42, 0x65, 0x79, 0x47, 0x75, 0x73, 0x2d, 0x42,
0x65, 0x79, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6b, 0x69, 0x6c, 0x47, 0x61,
0x76, 0x61, 0x6e, 0x61, 0x45, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6c,
0x6f, 0x50, 0x69, 0x74, 0x65, 0x72, 0x73, 0x62, 0x65, 0x72, 0x67, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x76, 0x65,
0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69,
0x6e, 0x73, 0x65, 0x6e, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x56, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4a, 0x75, 0x6e, 0x6f, 0x4c, 0x75,
0x69, 0x73, 0x76, 0x69, 0x6c, 0x6c, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73,
0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b,
0x6b, 0x69, 0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x65, 0x79, 0x6b,
0x4c, 0x6f, 0x75, 0x65, 0x72, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x65,
0x73, 0x2d, 0x4b, 0x75, 0x6f, 0x72, 0x74, 0x65, 0x72, 0x4e, 0x79, 0x75,
0x2d, 0x59, 0x6f, 0x72, 0x6b, 0x42, 0x6f, 0x79, 0x6c, 0x61, 0x2c, 0x20,
0x53, 0x68, 0x69, 0x6d, 0x6f, 0x6c, 0x69, 0x79, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x2c, 0x20, 0x53,
0x68, 0x69, 0x6d, 0x6f, 0x6c, 0x69, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x4e, 0x79, 0x75, 0x2d, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c,
0x20, 0x53, 0x68, 0x69, 0x6d, 0x6f, 0x6c, 0x69, 0x79, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x47, 0x6f, 0x74, 0x78, 0x6f, 0x62, 0x4f, 0x78,
0x69, 0x6e, 0x61, 0x67, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f, 0x66,
0x2d, 0x53, 0x70, 0x65, 0x79, 0x6e, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x2d,
0x56, 0x65, 0x6c, 0x79, 0x75, 0x52, 0x65, 0x6a, 0x61, 0x79, 0x6e, 0x61,
0x53, 0x61, 0x6e, 0x74, 0x79, 0x61, 0x67, 0x6f, 0x49, 0x74, 0x74, 0x6f,
0x6b, 0x6b, 0x6f, 0x72, 0x74, 0x6f, 0x6f, 0x72, 0x6d, 0x69, 0x75, 0x74,
0x53, 0x65, 0x6e, 0x74, 0x2d, 0x4c, 0x79, 0x75, 0x73, 0x69, 0x79, 0x61,
0x53, 0x65, 0x6e, 0x74, 0x2d, 0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74,
0x53, 0x76, 0x69, 0x66, 0x74, 0x2d, 0x4b, 0x61, 0x72, 0x72, 0x65, 0x6e,
0x74, 0x55, 0x61, 0x79, 0x74, 0x78, 0x6f, 0x72, 0x73, 0x56, 0x69, 0x6e,
0x6e, 0x69, 0x70, 0x65, 0x67, 0x44, 0x79, 0x75, 0x6d, 0x6f, 0x6e, 0x2d,
0x64, 0x2019, 0x59, 0x75, 0x72, 0x76, 0x69, 0x6c, 0x4c, 0x6f, 0x6e, 0x67,
0x79, 0x69, 0x72, 0x41, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x6d, 0x6f, 0x6e,
0x4f, 0x71, 0x74, 0x6f, 0x76, 0x4f, 0x71, 0x74, 0x6f, 0x2018, 0x62, 0x61,
0x41, 0x73, 0x68, 0x78, 0x6f, 0x62, 0x6f, 0x64, 0x42, 0x61, 0x67, 0x2018,
0x64, 0x6f, 0x64, 0x42, 0x61, 0x68, 0x72, 0x61, 0x79, 0x6e, 0x42, 0x6f,
0x6b, 0x75, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x68, 0x71, 0x47, 0x2018, 0x61,
0x7a, 0x6f, 0x58, 0x65, 0x76, 0x72, 0x6f, 0x6e, 0x58, 0x6f, 0x73, 0x68,
0x69, 0x6d, 0x69, 0x6e, 0x58, 0x6f, 0x76, 0x64, 0x4a, 0x61, 0x79, 0x70,
0x75, 0x72, 0x51, 0x75, 0x64, 0x64, 0x75, 0x73, 0x51, 0x6f, 0x62, 0x75,
0x6c, 0x58, 0x61, 0x6e, 0x64, 0x69, 0x67, 0x61, 0x51, 0x75, 0x76, 0x61,
0x79, 0x74, 0x51, 0x6f, 0x2bb, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x51,
0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x2018, 0x72, 0x64, 0x61, 0x41, 0x72, 0x2d,
0x52, 0x69, 0x79, 0x6f, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x71, 0x61,
0x6e, 0x64, 0x53, 0x68, 0x61, 0x6e, 0x78, 0x61, 0x79, 0x54, 0x61, 0x79,
0x70, 0x65, 0x79, 0x54, 0x65, 0x68, 0x72, 0x6f, 0x6e, 0x55, 0x72, 0x75,
0x6d, 0x63, 0x68, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f,
0x6c, 0x6c, 0x61, 0x72, 0x69, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x61,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4b, 0x61, 0x6e,
0x61, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4b,
0x61, 0x62, 0x6f, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x46, 0x61, 0x72,
0x65, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4d,
0x61, 0x64, 0x65, 0x79, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69,
0x4a, 0x61, 0x6e, 0x75, 0x62, 0x69, 0x79, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x79, 0x61, 0x4d, 0x75, 0x71, 0x61, 0x64, 0x64, 0x61, 0x73,
0x20, 0x59, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x69, 0x42, 0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x2d, 0x58, 0x69, 0x6c, 0x6c,
0x45, 0x76, 0x6b, 0x6c, 0x61, 0x58, 0x6f, 0x62, 0x61, 0x72, 0x74, 0x4c,
0x6f, 0x72, 0x64, 0x2d, 0x58, 0x61, 0x75, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x69, 0x42, 0x72, 0x79, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x42, 0x75, 0x64,
0x61, 0x70, 0x65, 0x73, 0x68, 0x74, 0x42, 0x79, 0x75, 0x7a, 0x69, 0x6e,
0x67, 0x65, 0x6e, 0x4b, 0x69, 0x73, 0x68, 0x69, 0x6e, 0x79, 0x6f, 0x76,
0x58, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x4d, 0x65, 0x6e, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x4c, 0x79, 0x75, 0x6b, 0x73, 0x65, 0x6d,
0x62, 0x75, 0x72, 0x67, 0x4d, 0x61, 0x72, 0x69, 0x79, 0x65, 0x78, 0x61,
0x6d, 0x6e, 0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x74, 0x73, 0x61,
0x56, 0x65, 0x6e, 0x61, 0x56, 0x61, 0x72, 0x73, 0x68, 0x61, 0x76, 0x61,
0x53, 0x79, 0x75, 0x72, 0x69, 0x78, 0x52, 0x6f, 0x6a, 0x64, 0x65, 0x73,
0x74, 0x76, 0x6f, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x4b, 0x6f, 0x6b,
0x6f, 0x73, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4b,
0x6f, 0x6d, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72,
0x69, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x79,
0x4d, 0x61, 0x79, 0x6f, 0x72, 0x6b, 0x61, 0x52, 0x65, 0x79, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x43, 0x68, 0x61, 0x74, 0x65, 0x6d, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x69, 0x54, 0x72, 0x75, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x69, 0x47, 0x61, 0x6d, 0x62, 0x79, 0x65, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x69, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x65, 0x72, 0x69,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4d, 0x69, 0x64,
0x76, 0x65, 0x79, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69,
0x50, 0x6f, 0x6e, 0x70, 0x65, 0x69, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69,
0x54, 0x61, 0x69, 0x74, 0x69, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x55,
0x65, 0x79, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x4c, 0x6f, 0x77,
0x65, 0x72, 0x20, 0x50, 0x72, 0x69, 0x6e, 0x63, 0x65, 0x2bc, 0x73, 0x20,
0x51, 0x75, 0x61, 0x72, 0x74, 0x65, 0x72, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x42, 0x1eaf, 0x63, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x42, 0x1eaf, 0x63,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x42, 0x1eaf, 0x63, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x54, 0x50, 0x20, 0x48, 0x1ed3, 0x20, 0x43, 0x68,
0xed, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x48, 0x1ed3, 0x6e, 0x67, 0x20, 0x4b,
0xf4, 0x6e, 0x67, 0x4d, 0x61, 0x20, 0x43, 0x61, 0x6f, 0x42, 0xec, 0x6e,
0x68, 0x20, 0x4e, 0x68, 0x1b0, 0x1ee1, 0x6e, 0x67, 0x54, 0x68, 0x1b0, 0x1ee3,
0x6e, 0x67, 0x20, 0x48, 0x1ea3, 0x69, 0x110, 0xe0, 0x69, 0x20, 0x42, 0x1eaf,
0x63, 0x56, 0x69, 0xea, 0x6e, 0x67, 0x20, 0x43, 0x68, 0x103, 0x6e, 0x4e,
0x61, 0x6d, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x110, 0x1ea3,
0x6f, 0x20, 0x4d, 0x61, 0x6e, 0x4d, 0xe1, 0x74, 0x2d, 0x78, 0x63, 0x1a1,
0x2d, 0x76, 0x61, 0x44, 0x161, 0x69, 0x62, 0x75, 0x74, 0x69, 0x4b, 0x61,
0x69, 0x6d, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6c, 0x61, 0x48, 0x61, 0x77,
0x61, 0x6e, 0x6e, 0x61, 0x52, 0x69, 0x6b, 0x6a, 0x61, 0x77, 0x69, 0x6b,
0x53, 0x69, 0x64, 0x6e, 0x65, 0x69, 0x41, 0x6d, 0x161, 0x74, 0x65, 0x72,
0x64, 0x61, 0x6d, 0x4b, 0x6f, 0x6e, 0x161, 0x74, 0x61, 0x6e, 0x74, 0x69,
0x6e, 0x6f, 0x70, 0x65, 0x6c, 0x4b, 0xf6, 0x6e, 0x69, 0x67, 0x73, 0x62,
0xe4, 0x72, 0x67, 0x4c, 0x61, 0x69, 0x62, 0x61, 0x10d, 0x52, 0x65, 0x76,
0x61, 0x6c, 0x57, 0x61, 0x72, 0x161, 0x61, 0x75, 0x5a, 0xfc, 0x72, 0x69,
0x10d, 0x42, 0x61, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61,
0x73, 0x42, 0x61, 0x65, 0x20, 0x43, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x64,
0x67, 0x65, 0x44, 0x69, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x65, 0x6c, 0x6c,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x44, 0x69, 0x6e,
0x61, 0x73, 0x20, 0x4d, 0x65, 0x63, 0x73, 0x69, 0x63, 0x6f, 0x45, 0x66,
0x72, 0x6f, 0x67, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64,
0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x2c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x44, 0x69, 0x6e, 0x61, 0x73,
0x20, 0x48, 0xf4, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68,
0x4a, 0x65, 0x72, 0x77, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x59, 0x72, 0x20,
0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x44, 0x65, 0x64,
0x77, 0x79, 0x64, 0x64, 0x46, 0x66, 0x61, 0x72, 0x6f, 0x44, 0x65, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x42, 0x72, 0x77, 0x73, 0x65,
0x6c, 0x42, 0x77, 0x63, 0x61, 0x72, 0xe9, 0x73, 0x74, 0x44, 0x75, 0x6c,
0x79, 0x6e, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x47, 0x61, 0x72,
0x6e, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4d, 0x61, 0x6e, 0x61, 0x77, 0x43,
0x61, 0x65, 0x72, 0x67, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x69, 0x6e,
0x4c, 0x6c, 0x75, 0x6e, 0x64, 0x61, 0x69, 0x6e, 0x4c, 0x77, 0x63, 0x73,
0x65, 0x6d, 0x62, 0x77, 0x72, 0x67, 0x52, 0x68, 0x75, 0x66, 0x61, 0x69,
0x6e, 0x59, 0x20, 0x46, 0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x46, 0x69,
0x65, 0x6e, 0x6e, 0x61, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x4e,
0x61, 0x64, 0x6f, 0x6c, 0x69, 0x67, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79,
0x20, 0x50, 0x61, 0x73, 0x67, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x647, 0x627,
0x642, 0x62a, 0x627, 0x648, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x647, 0x639, 0x634,
0x642, 0x20, 0x622, 0x628, 0x627, 0x62a, 0x628, 0x6cc, 0x634, 0x6a9, 0x6cc, 0x6a9,
0x6a9, 0x648, 0x644, 0x648, 0x645, 0x628, 0x648, 0x688, 0x627, 0x6a9, 0x627, 0x6cc,
0x648, 0x631, 0x634, 0x644, 0x6cc, 0x645, 0x6a9, 0x679, 0x645, 0x646, 0x688, 0x648,
0x6a9, 0x648, 0x6cc, 0x679, 0x642, 0x6cc, 0x632, 0x6cc, 0x644, 0x648, 0x631, 0x62f,
0x627, 0x62a, 0x627, 0x634, 0x6a9, 0x6cc, 0x646, 0x62a, 0x62a, 0x6cc, 0x628, 0x644,
0x6cc, 0x633, 0x6cc, 0x679, 0x6cc, 0x645, 0x67e, 0x648, 0x627, 0x648, 0x631, 0x645,
0x686, 0x6cc, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x641, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x6f, 0x2d, 0x73, 0x74, 0x61, 0x64, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x4e, 0x6f, 0x61, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e,
0x6f, 0x61, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f,
0x61, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50, 0xf4,
0x72, 0x74, 0x6f, 0x20, 0x56, 0x65, 0x6c, 0x68, 0x6f, 0x4b, 0x61, 0x6e,
0x61, 0x72, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x45, 0x69, 0x6c, 0x61, 0x6e,
0x6e, 0x65, 0x6e, 0x53, 0xfb, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x50, 0x72, 0x61, 0x61, 0x63, 0x68, 0x46, 0x61, 0x74, 0x69,
0x6b, 0x61, 0x61, 0x6e, 0x73, 0x74, 0xea, 0x64, 0x4b, 0x72, 0x79, 0x73,
0x74, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x6e, 0x65, 0x6e, 0x50, 0x65, 0x61, 0x73, 0x6b,
0x65, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x6e, 0x65, 0x6e, 0x45, 0x6e, 0x64,
0x65, 0x72, 0x62, 0x75, 0x72, 0x79, 0x2d, 0x65, 0x69, 0x6c, 0xe2, 0x6e,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x6e, 0x65, 0x6e, 0x5d0, 0x5b7, 0x5d1, 0x5d9, 0x5d3, 0x5d6, 0x5e9,
0x5d0, 0x5b7, 0x5df, 0x5d0, 0x5b7, 0x5e1, 0x5de, 0x5d0, 0x5b7, 0x5e8, 0x5d0, 0x5b7,
0x5d8, 0x5d5, 0x5e0, 0x5d9, 0x5e1, 0x5d5, 0x5d5, 0x5d9, 0x5e0, 0x5d8, 0x5d4, 0x5d5,
0x5e7, 0x5d4, 0x5d0, 0x20, 0x5d8, 0x5e9, 0x5d9, 0x20, 0x5de, 0x5d9, 0x5df, 0x20,
0x5e9, 0x5d8, 0x5d0, 0x5b8, 0x5d8, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x64, 0xe1,
0x6b, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x6e, 0x6b, 0x1ecd, 0x301, 0x72,
0xe9, 0xe8, 0x6a, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x6e, 0x67, 0xfa,
0xed, 0x6c, 0xe0, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x6e, 0x74, 0xed, 0x67,
0xf9, 0xe0, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x72, 0xfa, 0x62, 0xe1, 0xec,
0x6c, 0xfa, 0x20, 0xe0, 0x74, 0x69, 0x6b, 0x1ecd, 0x6b, 0xe0, 0x6e, 0xec,
0x6c, 0xfa, 0x20, 0x42, 0xe1, 0x68, 0xec, 0x20, 0x42, 0xe1, 0x6e, 0x64,
0x1eb9, 0x301, 0x72, 0xe1, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0xe1,
0x62, 0xe1, 0x64, 0x1ecd, 0x301, 0x1ecd, 0x300, 0x73, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x42, 0x1eb9, 0x300, 0x6c, 0xed, 0xec, 0x73, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x42, 0x6c, 0x61, 0x6e, 0x6b, 0x20, 0x53, 0x61, 0x62, 0x75, 0x6c,
0x1ecd, 0x301, 0x1ecd, 0x300, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x1ecd,
0x301, 0x69, 0x73, 0xe8, 0xec, 0x6c, 0xfa, 0x20, 0xf2, 0x6b, 0x75, 0x6e,
0x20, 0x6b, 0x61, 0x6d, 0x62, 0xed, 0x72, 0xed, 0xec, 0x6a, 0xec, 0xec,
0x6c, 0xfa, 0x20, 0x4b, 0x61, 0x6e, 0x63, 0x75, 0x6e, 0x69, 0x6c, 0xfa,
0x20, 0x4b, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0xec, 0x6c, 0xfa, 0x20, 0x43,
0x68, 0x69, 0x63, 0x61, 0x67, 0x6f, 0xec, 0x6c, 0xfa, 0x20, 0x1e62, 0x69,
0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4b, 0x6f,
0x73, 0x74, 0x61, 0x20, 0x52, 0xed, 0x6b, 0xe0, 0xec, 0x6c, 0xfa, 0x20,
0x4b, 0xed, 0x72, 0x1eb9, 0x73, 0x74, 0x1ecd, 0x301, 0x1ecd, 0x300, 0x6e, 0xf9,
0xec, 0x6c, 0xfa, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x1e63, 0x61, 0x6f, 0xec,
0x6c, 0xfa, 0x20, 0x42, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68,
0x61, 0x6e, 0xec, 0x6c, 0xfa, 0x20, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0xec, 0x6c, 0xfa, 0x20, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x43,
0x72, 0x65, 0x65, 0x6b, 0xec, 0x6c, 0xfa, 0x20, 0x44, 0x65, 0x6e, 0x76,
0x65, 0x72, 0xec, 0x6c, 0xfa, 0x20, 0x44, 0x65, 0x74, 0x72, 0x6f, 0x69,
0x74, 0xec, 0x6c, 0xfa, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b,
0x61, 0xec, 0x6c, 0xfa, 0x20, 0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x74, 0x6f,
0x6e, 0xec, 0x6c, 0xfa, 0x20, 0x45, 0x6c, 0x20, 0x53, 0x61, 0x76, 0x61,
0x64, 0x6f, 0x72, 0xec, 0x6c, 0xfa, 0x20, 0x46, 0x6f, 0x72, 0x74, 0x20,
0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0xec, 0x6c, 0xfa, 0x20, 0x6f, 0x6d,
0x69, 0x20, 0x47, 0x6c, 0x61, 0x63, 0x65, 0xec, 0x6c, 0xfa, 0x20, 0x6f,
0x6d, 0x69, 0x20, 0x47, 0x6f, 0x6f, 0x73, 0xf9, 0xec, 0x6c, 0xfa, 0x20,
0x47, 0x72, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x75, 0x72, 0x6b, 0xec, 0x6c,
0xfa, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x61, 0x64, 0x61, 0xec, 0x6c, 0xfa,
0x20, 0x47, 0x75, 0x61, 0x64, 0x65, 0x6c, 0x6f, 0x75, 0x70, 0x65, 0xec,
0x6c, 0xfa, 0x20, 0x47, 0x75, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61,
0xec, 0x6c, 0xfa, 0x20, 0x48, 0x61, 0x6c, 0x69, 0x66, 0xe1, 0x73, 0xec,
0xec, 0x6c, 0xfa, 0x20, 0x48, 0x61, 0x76, 0x61, 0x6e, 0x61, 0xec, 0x6c,
0xfa, 0x20, 0x48, 0x65, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6f, 0xec,
0x6c, 0xfa, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x70, 0x6f,
0x6c, 0x69, 0x73, 0xec, 0x6c, 0xfa, 0x20, 0x6e, 0x1ecd, 0x301, 0x73, 0xec,
0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x72, 0x1eb9, 0x301, 0x6e, 0x67, 0x6f,
0xec, 0x6c, 0xfa, 0x20, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75,
0x72, 0x67, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43,
0x69, 0x74, 0x79, 0xec, 0x6c, 0xfa, 0x20, 0x56, 0x65, 0x76, 0x61, 0x79,
0xec, 0x6c, 0xfa, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e, 0x65,
0x73, 0x20, 0x6e, 0xed, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0xec, 0x6c, 0xfa, 0x20, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x20,
0x6e, 0xed, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0xec, 0x6c,
0xfa, 0x20, 0x49, 0x6e, 0x75, 0x76, 0x69, 0x6b, 0xec, 0x6c, 0xfa, 0x20,
0x49, 0x71, 0x61, 0x6c, 0x75, 0x69, 0x74, 0xec, 0x6c, 0xfa, 0x20, 0x4a,
0x61, 0x6d, 0x61, 0x69, 0x6b, 0xe0, 0xec, 0x6c, 0xfa, 0x20, 0x4a, 0x75,
0x6e, 0x75, 0xec, 0x6c, 0xfa, 0x20, 0x4c, 0xfa, 0xed, 0x66, 0xed, 0xec,
0x6c, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73,
0x1eb9, 0x301, 0x6c, 0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4b, 0xed, 0x72, 0x61,
0x6c, 0x1eb9, 0x301, 0x6e, 0x64, 0xe1, 0x69, 0x6b, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x67, 0x65, 0x6c, 0x65, 0x73,
0xec, 0x6c, 0xfa, 0x20, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x20, 0x50, 0x72,
0x69, 0x6e, 0x63, 0x65, 0x2019, 0x73, 0x20, 0x51, 0x75, 0x61, 0x72, 0x74,
0x65, 0x72, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x6e, 0x61, 0x67, 0x75,
0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x67, 0x1ecd, 0x301,
0x1ecd, 0x300, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0xe1, 0x74, 0xed,
0x6e, 0xed, 0x6b, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x74, 0x61,
0x6d, 0x6f, 0x72, 0x6f, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61,
0x73, 0x61, 0x74, 0x69, 0x61, 0x6e, 0x69, 0xec, 0x6c, 0xfa, 0x20, 0x4d,
0x65, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d,
0x65, 0x72, 0x69, 0x64, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x65, 0x74,
0x69, 0x6c, 0x61, 0x6b, 0x61, 0x74, 0x69, 0x6c, 0x61, 0xec, 0x6c, 0xfa,
0x20, 0x4d, 0x1eb9, 0x301, 0x73, 0xed, 0x6b, 0xf2, 0xec, 0x6c, 0xfa, 0x20,
0x4d, 0x69, 0x6b, 0x75, 0x6c, 0x1ecd, 0x301, 0x6e, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x6f, 0x6e, 0x69, 0xec, 0x6c, 0xfa, 0x20,
0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x69, 0xec, 0x6c, 0xfa, 0x20, 0x4d,
0x6f, 0x6e, 0x73, 0x65, 0x72, 0x61, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x4e, 0x61, 0x73, 0x61, 0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x59, 0x6f, 0x72, 0x6b, 0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x6f, 0x6d,
0x69, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x65, 0x75, 0x6c, 0xe0, 0x20, 0x6e,
0xed, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x20, 0x6e, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x75, 0x75,
0x6b, 0xec, 0x6c, 0xfa, 0x20, 0x4f, 0x6a, 0x69, 0x6e, 0x61, 0x67, 0x61,
0xec, 0x6c, 0xfa, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x6d, 0x61, 0xec, 0x6c,
0xfa, 0x20, 0x46, 0xed, 0x6e, 0xed, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x50, 0x6f, 0x72, 0x74, 0x2d, 0x61, 0x75, 0x2d, 0x50, 0x72, 0x69, 0x6e,
0x63, 0x65, 0xec, 0x6c, 0xfa, 0x20, 0x65, 0x74, 0xed, 0x20, 0x6f, 0x6d,
0x69, 0x20, 0x53, 0xed, 0x70, 0xe9, 0xe8, 0x6e, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x6f,
0xec, 0x6c, 0xfa, 0x20, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x49,
0x6e, 0x6c, 0x65, 0x74, 0xec, 0x6c, 0xfa, 0x20, 0x52, 0x65, 0x67, 0x69,
0x6e, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x52, 0x65, 0x73, 0x6f, 0x6c, 0xfa,
0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20,
0x44, 0x6f, 0x6d, 0x69, 0x67, 0x6f, 0xec, 0x6c, 0xfa, 0x20, 0x49, 0x74,
0x6f, 0x6b, 0x6f, 0x74, 0x6f, 0x6d, 0x69, 0x74, 0x69, 0xec, 0x6c, 0xfa,
0x20, 0x53, 0x69, 0x6b, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x61, 0x74,
0x69, 0x6c, 0x65, 0x6d, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x74, 0x20,
0x4a, 0x1ecd, 0x301, 0x1ecd, 0x300, 0x6e, 0xf9, 0xec, 0x6c, 0xfa, 0x20, 0x53,
0x74, 0x20, 0x6b, 0x69, 0x74, 0x69, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x53, 0x74, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x61, 0xec, 0x6c, 0xfa, 0x20,
0x53, 0x74, 0x20, 0x54, 0x1ecd, 0x301, 0x6d, 0xe0, 0x73, 0xec, 0xec, 0x6c,
0xfa, 0x20, 0x46, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0xec, 0xec, 0x6c,
0xfa, 0x20, 0x53, 0xfa, 0x66, 0xed, 0x74, 0xf9, 0x20, 0x4b, 0x1ecd, 0x300,
0x72, 0x65, 0x6e, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0x65, 0x67,
0x75, 0x73, 0x69, 0x67, 0x61, 0x70, 0xe0, 0xec, 0x6c, 0xfa, 0x20, 0x54,
0x75, 0x6c, 0xe8, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0xed, 0x6a, 0xfa, 0x61,
0x6e, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0x6f, 0x72, 0x6f, 0x6e, 0x74,
0x6f, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0x6f, 0x74, 0x6f, 0x6c, 0x61, 0xec,
0x6c, 0xfa, 0x20, 0x56, 0x61, 0x6e, 0x6b, 0x75, 0x76, 0x61, 0xec, 0x6c,
0xfa, 0x20, 0x57, 0x68, 0x69, 0x74, 0x65, 0x68, 0x6f, 0x73, 0xec, 0xec,
0x6c, 0xfa, 0x20, 0x57, 0x69, 0x6e, 0x69, 0x70, 0x65, 0x67, 0xec, 0xec,
0x6c, 0xfa, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x49, 0x6c, 0x75,
0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68,
0xec, 0x6c, 0xfa, 0x20, 0x42, 0x1eb9, 0x300, 0x6d, 0xfa, 0x64, 0xe0, 0xec,
0x6c, 0xfa, 0x20, 0x41, 0x6e, 0x6b, 0x254, 0x301, 0x72, 0xe9, 0xe8, 0x6a,
0xec, 0xec, 0x6c, 0xfa, 0x20, 0xe0, 0x74, 0x69, 0x6b, 0x254, 0x6b, 0xe0,
0x6e, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0xe1, 0x68, 0xec, 0x20, 0x42, 0xe1,
0x6e, 0x64, 0x25b, 0x301, 0x72, 0xe1, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x42, 0xe1, 0x62, 0xe1, 0x64, 0x254, 0x301, 0x254, 0x300, 0x73, 0xec, 0xec,
0x6c, 0xfa, 0x20, 0x42, 0x25b, 0x300, 0x6c, 0xed, 0xec, 0x73, 0xec, 0xec,
0x6c, 0xfa, 0x20, 0x42, 0x6c, 0x61, 0x6e, 0x6b, 0x20, 0x53, 0x61, 0x62,
0x75, 0x6c, 0x254, 0x301, 0x254, 0x300, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x42, 0x254, 0x301, 0x69, 0x73, 0xe8, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x68,
0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4b,
0xed, 0x72, 0x25b, 0x73, 0x74, 0x254, 0x301, 0x254, 0x300, 0x6e, 0xf9, 0xec,
0x6c, 0xfa, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0x68, 0x61, 0x6f, 0xec,
0x6c, 0xfa, 0x20, 0x6e, 0x254, 0x301, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x4d, 0x61, 0x72, 0x25b, 0x301, 0x6e, 0x67, 0x6f, 0xec, 0x6c, 0xfa, 0x20,
0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73, 0x25b, 0x301, 0x6c, 0xf2, 0xec, 0x6c,
0xfa, 0x20, 0x4b, 0xed, 0x72, 0x61, 0x6c, 0x25b, 0x301, 0x6e, 0x64, 0xe1,
0x69, 0x6b, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x67,
0x254, 0x301, 0x254, 0x300, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x25b,
0x301, 0x73, 0xed, 0x6b, 0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x69, 0x6b,
0x75, 0x6c, 0x254, 0x301, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x74,
0x20, 0x4a, 0x254, 0x301, 0x254, 0x300, 0x6e, 0xf9, 0xec, 0x6c, 0xfa, 0x20,
0x53, 0x74, 0x20, 0x54, 0x254, 0x301, 0x6d, 0xe0, 0x73, 0xec, 0xec, 0x6c,
0xfa, 0x20, 0x53, 0xfa, 0x66, 0xed, 0x74, 0xf9, 0x20, 0x4b, 0x254, 0x300,
0x72, 0x65, 0x6e, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x25b, 0x300,
0x6d, 0xfa, 0x64, 0xe0, 0x69, 0x2d, 0x41, 0x62, 0x69, 0x64, 0x6a, 0x61,
0x6e, 0x69, 0x2d, 0x41, 0x63, 0x63, 0x72, 0x61, 0x69, 0x2d, 0x41, 0x64,
0x64, 0x69, 0x73, 0x20, 0x41, 0x62, 0x61, 0x62, 0x61, 0x69, 0x2d, 0x41,
0x6c, 0x67, 0x69, 0x65, 0x72, 0x73, 0x69, 0x2d, 0x41, 0x73, 0x6d, 0x61,
0x72, 0x61, 0x69, 0x2d, 0x42, 0x61, 0x6d, 0x61, 0x6b, 0x6f, 0x69, 0x2d,
0x42, 0x61, 0x6e, 0x67, 0x75, 0x69, 0x69, 0x2d, 0x42, 0x61, 0x6e, 0x6a,
0x75, 0x6c, 0x69, 0x2d, 0x42, 0x69, 0x73, 0x73, 0x61, 0x75, 0x69, 0x2d,
0x42, 0x6c, 0x61, 0x6e, 0x74, 0x79, 0x72, 0x65, 0x69, 0x2d, 0x42, 0x72,
0x61, 0x7a, 0x7a, 0x61, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x69, 0x2d, 0x42,
0x75, 0x6a, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x69, 0x2d, 0x43, 0x61,
0x69, 0x72, 0x6f, 0x69, 0x2d, 0x43, 0x61, 0x73, 0x61, 0x62, 0x6c, 0x61,
0x6e, 0x63, 0x61, 0x69, 0x2d, 0x43, 0x65, 0x75, 0x74, 0x61, 0x69, 0x2d,
0x43, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x79, 0x69, 0x2d, 0x44, 0x61, 0x6b,
0x61, 0x72, 0x69, 0x2d, 0x44, 0x61, 0x72, 0x20, 0x65, 0x73, 0x20, 0x53,
0x61, 0x6c, 0x61, 0x61, 0x6d, 0x69, 0x2d, 0x44, 0x6a, 0x69, 0x62, 0x6f,
0x75, 0x74, 0x69, 0x69, 0x2d, 0x44, 0x6f, 0x75, 0x61, 0x6c, 0x61, 0x69,
0x2d, 0x45, 0x6c, 0x20, 0x41, 0x61, 0x69, 0x75, 0x6e, 0x69, 0x2d, 0x46,
0x72, 0x65, 0x65, 0x74, 0x6f, 0x77, 0x6e, 0x69, 0x2d, 0x47, 0x61, 0x62,
0x6f, 0x72, 0x6f, 0x6e, 0x65, 0x69, 0x2d, 0x48, 0x61, 0x72, 0x61, 0x72,
0x65, 0x69, 0x2d, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62,
0x75, 0x72, 0x67, 0x69, 0x4a, 0x75, 0x62, 0x61, 0x69, 0x2d, 0x4b, 0x61,
0x6d, 0x70, 0x61, 0x6c, 0x61, 0x69, 0x2d, 0x4b, 0x68, 0x61, 0x72, 0x74,
0x6f, 0x75, 0x6d, 0x69, 0x2d, 0x4b, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x69,
0x2d, 0x4b, 0x69, 0x6e, 0x73, 0x68, 0x61, 0x73, 0x61, 0x69, 0x2d, 0x4c,
0x61, 0x67, 0x6f, 0x73, 0x69, 0x2d, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x69, 0x2d, 0x4c, 0x6f, 0x6d, 0x65, 0x69, 0x2d,
0x4c, 0x75, 0x61, 0x6e, 0x64, 0x61, 0x69, 0x2d, 0x4c, 0x75, 0x62, 0x75,
0x6d, 0x62, 0x61, 0x73, 0x68, 0x69, 0x69, 0x4c, 0x75, 0x73, 0x61, 0x6b,
0x61, 0x69, 0x4d, 0x61, 0x6c, 0x61, 0x62, 0x6f, 0x69, 0x4d, 0x61, 0x70,
0x75, 0x74, 0x6f, 0x69, 0x4d, 0x61, 0x73, 0x65, 0x72, 0x75, 0x69, 0x4d,
0x62, 0x61, 0x62, 0x61, 0x6e, 0x65, 0x69, 0x2d, 0x4d, 0x6f, 0x67, 0x61,
0x64, 0x69, 0x73, 0x68, 0x75, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x72, 0x6f,
0x76, 0x69, 0x61, 0x69, 0x2d, 0x4e, 0x61, 0x69, 0x72, 0x6f, 0x62, 0x69,
0x69, 0x2d, 0x4e, 0x64, 0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x69, 0x2d,
0x4e, 0x69, 0x61, 0x6d, 0x65, 0x79, 0x69, 0x2d, 0x4e, 0x6f, 0x75, 0x61,
0x6b, 0x63, 0x68, 0x6f, 0x74, 0x74, 0x69, 0x2d, 0x4f, 0x75, 0x61, 0x67,
0x61, 0x64, 0x6f, 0x75, 0x67, 0x6f, 0x75, 0x69, 0x2d, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x69, 0x2d, 0x53, 0xe3, 0x6f,
0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x69, 0x2d, 0x54, 0x72, 0x69, 0x70, 0x6f,
0x6c, 0x69, 0x69, 0x2d, 0x54, 0x75, 0x6e, 0x69, 0x73, 0x69, 0x2d, 0x57,
0x69, 0x6e, 0x64, 0x68, 0x6f, 0x65, 0x6b, 0x69, 0x2d, 0x41, 0x64, 0x61,
0x6b, 0x69, 0x2d, 0x41, 0x6e, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x67, 0x65,
0x69, 0x2d, 0x41, 0x6e, 0x67, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x69, 0x2d,
0x41, 0x6e, 0x74, 0x69, 0x67, 0x75, 0x61, 0x69, 0x2d, 0x41, 0x72, 0x61,
0x67, 0x75, 0x61, 0x69, 0x6e, 0x61, 0x69, 0x2d, 0x42, 0x75, 0x65, 0x6e,
0x6f, 0x73, 0x20, 0x41, 0x69, 0x72, 0x65, 0x73, 0x69, 0x2d, 0x43, 0x61,
0x74, 0x61, 0x6d, 0x61, 0x72, 0x63, 0x61, 0x69, 0x2d, 0x43, 0x6f, 0x72,
0x64, 0x6f, 0x62, 0x61, 0x69, 0x2d, 0x4a, 0x75, 0x6a, 0x75, 0x79, 0x69,
0x2d, 0x4c, 0x61, 0x20, 0x52, 0x69, 0x6f, 0x6a, 0x61, 0x69, 0x2d, 0x4d,
0x65, 0x6e, 0x64, 0x6f, 0x7a, 0x61, 0x69, 0x2d, 0x52, 0x69, 0x6f, 0x20,
0x47, 0x61, 0x6c, 0x6c, 0x65, 0x67, 0x6f, 0x73, 0x69, 0x2d, 0x53, 0x61,
0x6c, 0x74, 0x61, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x20, 0x4a, 0x75, 0x61,
0x6e, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x20, 0x4c, 0x75, 0x69, 0x73, 0x69,
0x2d, 0x54, 0x75, 0x63, 0x75, 0x6d, 0x61, 0x6e, 0x69, 0x2d, 0x55, 0x73,
0x68, 0x75, 0x61, 0x69, 0x61, 0x69, 0x2d, 0x41, 0x72, 0x75, 0x62, 0x61,
0x69, 0x2d, 0x41, 0x73, 0x75, 0x6e, 0x63, 0x69, 0xf3, 0x6e, 0x69, 0x2d,
0x41, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6e, 0x69, 0x2d, 0x42, 0x61,
0x68, 0x69, 0x61, 0x69, 0x2d, 0x42, 0x61, 0x68, 0x69, 0x61, 0x20, 0x42,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x69, 0x2d, 0x42, 0x61, 0x72,
0x62, 0x61, 0x64, 0x6f, 0x73, 0x69, 0x2d, 0x42, 0x65, 0x6c, 0x65, 0x6d,
0x69, 0x2d, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0x65, 0x69, 0x2d, 0x42, 0x6c,
0x61, 0x6e, 0x63, 0x2d, 0x53, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x69, 0x2d,
0x42, 0x6f, 0x61, 0x20, 0x56, 0x69, 0x73, 0x74, 0x61, 0x69, 0x2d, 0x42,
0x6f, 0x67, 0x6f, 0x74, 0x61, 0x69, 0x2d, 0x42, 0x6f, 0x69, 0x73, 0x65,
0x69, 0x2d, 0x43, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x20,
0x42, 0x61, 0x79, 0x69, 0x2d, 0x43, 0x61, 0x6d, 0x70, 0x6f, 0x20, 0x47,
0x72, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x2d, 0x43, 0x61, 0x6e, 0x63, 0x75,
0x6e, 0x69, 0x2d, 0x43, 0x61, 0x72, 0x61, 0x63, 0x61, 0x73, 0x69, 0x2d,
0x43, 0x61, 0x79, 0x65, 0x6e, 0x6e, 0x65, 0x69, 0x2d, 0x43, 0x61, 0x79,
0x6d, 0x61, 0x6e, 0x69, 0x2d, 0x43, 0x68, 0x69, 0x63, 0x61, 0x67, 0x6f,
0x69, 0x2d, 0x43, 0x68, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x69,
0x2d, 0x43, 0x6f, 0x73, 0x74, 0x61, 0x20, 0x52, 0x69, 0x63, 0x61, 0x69,
0x2d, 0x43, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x6e, 0x69, 0x2d, 0x43, 0x75,
0x69, 0x61, 0x62, 0x61, 0x69, 0x2d, 0x43, 0x75, 0x72, 0x61, 0xe7, 0x61,
0x6f, 0x69, 0x2d, 0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68,
0x61, 0x76, 0x6e, 0x69, 0x2d, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x69,
0x2d, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x43, 0x72, 0x65, 0x65,
0x6b, 0x69, 0x2d, 0x44, 0x65, 0x6e, 0x76, 0x65, 0x72, 0x69, 0x2d, 0x44,
0x65, 0x74, 0x72, 0x6f, 0x69, 0x74, 0x69, 0x2d, 0x44, 0x6f, 0x6d, 0x69,
0x6e, 0x69, 0x63, 0x61, 0x69, 0x2d, 0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x74,
0x6f, 0x6e, 0x69, 0x2d, 0x45, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65,
0x69, 0x2d, 0x45, 0x6c, 0x20, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f,
0x72, 0x69, 0x2d, 0x46, 0x6f, 0x72, 0x74, 0x20, 0x4e, 0x65, 0x6c, 0x73,
0x6f, 0x6e, 0x69, 0x2d, 0x46, 0x6f, 0x72, 0x74, 0x61, 0x6c, 0x65, 0x7a,
0x61, 0x69, 0x2d, 0x47, 0x6c, 0x61, 0x63, 0x65, 0x20, 0x42, 0x61, 0x79,
0x69, 0x2d, 0x47, 0x6f, 0x6f, 0x73, 0x65, 0x20, 0x42, 0x61, 0x79, 0x69,
0x2d, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x69,
0x2d, 0x47, 0x72, 0x65, 0x6e, 0x61, 0x64, 0x61, 0x69, 0x2d, 0x47, 0x75,
0x61, 0x64, 0x65, 0x6c, 0x6f, 0x75, 0x70, 0x65, 0x69, 0x2d, 0x47, 0x75,
0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x69, 0x2d, 0x47, 0x75, 0x61,
0x79, 0x61, 0x71, 0x75, 0x69, 0x6c, 0x69, 0x2d, 0x47, 0x75, 0x79, 0x61,
0x6e, 0x61, 0x69, 0x2d, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x78, 0x69,
0x2d, 0x48, 0x61, 0x76, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x48, 0x65, 0x72,
0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6c, 0x6f, 0x69, 0x2d, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x69, 0x2d, 0x4b,
0x6e, 0x6f, 0x78, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x69, 0x2d, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x50, 0x65, 0x74, 0x65,
0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x69, 0x2d, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69,
0x74, 0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x69,
0x2d, 0x56, 0x65, 0x76, 0x61, 0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x69, 0x2d, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e,
0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x69,
0x2d, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x49, 0x6e, 0x75, 0x76, 0x69,
0x6b, 0x69, 0x2d, 0x49, 0x71, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x69, 0x2d,
0x4a, 0x61, 0x6d, 0x61, 0x69, 0x63, 0x61, 0x69, 0x2d, 0x4a, 0x75, 0x6e,
0x65, 0x61, 0x75, 0x69, 0x2d, 0x4c, 0x6f, 0x75, 0x69, 0x73, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x63, 0x65,
0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x63, 0x6b,
0x79, 0x69, 0x2d, 0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x6a,
0x6b, 0x69, 0x2d, 0x4c, 0x61, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x2d, 0x4c,
0x69, 0x6d, 0x61, 0x69, 0x2d, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x67,
0x65, 0x6c, 0x65, 0x73, 0x69, 0x2d, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x20,
0x50, 0x72, 0x69, 0x6e, 0x63, 0x65, 0x2019, 0x73, 0x20, 0x51, 0x75, 0x61,
0x72, 0x74, 0x65, 0x72, 0x69, 0x2d, 0x4d, 0x61, 0x63, 0x65, 0x69, 0x6f,
0x69, 0x2d, 0x4d, 0x61, 0x6e, 0x61, 0x67, 0x75, 0x61, 0x69, 0x2d, 0x4d,
0x61, 0x6e, 0x61, 0x75, 0x73, 0x69, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x67,
0x6f, 0x74, 0x69, 0x2d, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x71,
0x75, 0x65, 0x69, 0x2d, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x72, 0x6f,
0x73, 0x69, 0x2d, 0x4d, 0x61, 0x7a, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x69,
0x2d, 0x4d, 0x65, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x65, 0x65, 0x69, 0x2d,
0x4d, 0x65, 0x72, 0x69, 0x64, 0x61, 0x69, 0x2d, 0x4d, 0x65, 0x74, 0x6c,
0x61, 0x6b, 0x61, 0x74, 0x6c, 0x61, 0x69, 0x2d, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x6f, 0x20, 0x43, 0x69, 0x74, 0x79, 0x69, 0x2d, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x63, 0x74,
0x6f, 0x6e, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x72, 0x65,
0x79, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x76, 0x69, 0x64, 0x65,
0x6f, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x74, 0x73, 0x65, 0x72, 0x72, 0x61,
0x74, 0x69, 0x2d, 0x4e, 0x61, 0x73, 0x73, 0x61, 0x75, 0x69, 0x2d, 0x4e,
0x65, 0x77, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x69, 0x2d, 0x4e, 0x6f, 0x6d,
0x65, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x69, 0x2d,
0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x69, 0x2d, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x69, 0x2d, 0x4e, 0x65, 0x77, 0x20,
0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x69, 0x2d, 0x4e, 0x75, 0x75,
0x6b, 0x69, 0x2d, 0x4f, 0x6a, 0x69, 0x6e, 0x61, 0x67, 0x61, 0x69, 0x2d,
0x50, 0x61, 0x6e, 0x61, 0x6d, 0x61, 0x69, 0x2d, 0x50, 0x61, 0x72, 0x61,
0x6d, 0x61, 0x72, 0x69, 0x62, 0x6f, 0x69, 0x2d, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0x69, 0x2d, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x61, 0x75,
0x2d, 0x50, 0x72, 0x69, 0x6e, 0x63, 0x65, 0x69, 0x2d, 0x50, 0x6f, 0x72,
0x74, 0x20, 0x6f, 0x66, 0x20, 0x53, 0x70, 0x61, 0x69, 0x6e, 0x69, 0x2d,
0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x56, 0x65, 0x6c, 0x68, 0x6f, 0x69,
0x2d, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x63, 0x6f,
0x69, 0x2d, 0x50, 0x75, 0x6e, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x6e,
0x61, 0x73, 0x69, 0x2d, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x49,
0x6e, 0x6c, 0x65, 0x74, 0x69, 0x2d, 0x52, 0x65, 0x63, 0x69, 0x66, 0x65,
0x69, 0x2d, 0x52, 0x65, 0x67, 0x69, 0x6e, 0x61, 0x69, 0x2d, 0x52, 0x65,
0x73, 0x6f, 0x6c, 0x75, 0x74, 0x65, 0x69, 0x2d, 0x52, 0x69, 0x6f, 0x20,
0x42, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x74,
0x61, 0x72, 0x65, 0x6d, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x74, 0x69, 0x61,
0x67, 0x6f, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44, 0x6f,
0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x69, 0x2d, 0x53, 0x61, 0x6f, 0x20, 0x50,
0x61, 0x75, 0x6c, 0x6f, 0x69, 0x2d, 0x49, 0x74, 0x74, 0x6f, 0x71, 0x71,
0x6f, 0x72, 0x74, 0x6f, 0x6f, 0x72, 0x6d, 0x69, 0x69, 0x74, 0x69, 0x2d,
0x53, 0x69, 0x74, 0x6b, 0x61, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x42,
0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x69, 0x2d, 0x53,
0x74, 0x2e, 0x20, 0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73, 0x69, 0x2d, 0x53,
0x74, 0x2e, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73, 0x49, 0x2d, 0x53, 0x74,
0x2e, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x69, 0x2d, 0x53, 0x74, 0x2e,
0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x69, 0x2d, 0x53, 0x74, 0x2e,
0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x69, 0x2d, 0x53, 0x77,
0x69, 0x66, 0x74, 0x20, 0x43, 0x75, 0x72, 0x72, 0x65, 0x6e, 0x74, 0x69,
0x2d, 0x54, 0x65, 0x67, 0x75, 0x63, 0x69, 0x67, 0x61, 0x6c, 0x70, 0x61,
0x69, 0x2d, 0x54, 0x68, 0x75, 0x6c, 0x65, 0x69, 0x2d, 0x54, 0x69, 0x6a,
0x75, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x54, 0x6f, 0x72, 0x6f, 0x6e, 0x74,
0x6f, 0x69, 0x2d, 0x54, 0x6f, 0x72, 0x74, 0x6f, 0x6c, 0x61, 0x69, 0x2d,
0x56, 0x61, 0x6e, 0x63, 0x6f, 0x75, 0x76, 0x65, 0x72, 0x69, 0x2d, 0x57,
0x68, 0x69, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x73, 0x65, 0x69, 0x2d, 0x57,
0x69, 0x6e, 0x6e, 0x69, 0x70, 0x65, 0x67, 0x69, 0x2d, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x61, 0x74, 0x69, 0x2d, 0x43, 0x61, 0x73, 0x65, 0x79, 0x69,
0x2d, 0x44, 0x61, 0x76, 0x69, 0x73, 0x69, 0x2d, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x69, 0x2d, 0x4d, 0x61, 0x63, 0x71, 0x75, 0x61, 0x72, 0x69, 0x65, 0x69,
0x2d, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x69, 0x2d, 0x4d, 0x63, 0x4d,
0x75, 0x72, 0x64, 0x6f, 0x69, 0x2d, 0x50, 0x61, 0x6c, 0x6d, 0x65, 0x72,
0x69, 0x2d, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x69, 0x2d, 0x53,
0x79, 0x6f, 0x77, 0x61, 0x69, 0x2d, 0x54, 0x72, 0x6f, 0x6c, 0x6c, 0x69,
0x2d, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x2d, 0x4c, 0x6f, 0x6e,
0x67, 0x79, 0x65, 0x61, 0x72, 0x62, 0x79, 0x65, 0x6e, 0x69, 0x2d, 0x41,
0x64, 0x65, 0x6e, 0x69, 0x2d, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x69,
0x2d, 0x41, 0x6d, 0x6d, 0x61, 0x6e, 0x69, 0x2d, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x69, 0x2d, 0x41, 0x71, 0x74, 0x61, 0x75, 0x69, 0x2d, 0x41,
0x71, 0x74, 0x6f, 0x62, 0x65, 0x69, 0x2d, 0x41, 0x73, 0x68, 0x67, 0x61,
0x62, 0x61, 0x74, 0x69, 0x2d, 0x42, 0x61, 0x67, 0x68, 0x64, 0x61, 0x64,
0x69, 0x2d, 0x42, 0x61, 0x68, 0x72, 0x61, 0x69, 0x6e, 0x69, 0x2d, 0x42,
0x61, 0x6b, 0x75, 0x69, 0x2d, 0x42, 0x61, 0x6e, 0x67, 0x6b, 0x6f, 0x6b,
0x69, 0x2d, 0x42, 0x61, 0x72, 0x6e, 0x61, 0x75, 0x6c, 0x69, 0x2d, 0x42,
0x65, 0x69, 0x72, 0x75, 0x74, 0x69, 0x2d, 0x42, 0x69, 0x73, 0x68, 0x6b,
0x65, 0x6b, 0x69, 0x2d, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x69, 0x2d,
0x43, 0x68, 0x69, 0x74, 0x61, 0x69, 0x2d, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x6f, 0x69, 0x2d, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x63, 0x75, 0x73,
0x69, 0x2d, 0x44, 0x68, 0x61, 0x6b, 0x61, 0x69, 0x2d, 0x44, 0x69, 0x6c,
0x69, 0x69, 0x2d, 0x44, 0x75, 0x62, 0x61, 0x69, 0x69, 0x2d, 0x44, 0x75,
0x73, 0x68, 0x61, 0x6e, 0x62, 0x65, 0x69, 0x47, 0x61, 0x7a, 0x61, 0x69,
0x2d, 0x48, 0x65, 0x62, 0x72, 0x6f, 0x6e, 0x69, 0x2d, 0x48, 0x6f, 0x20,
0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x20, 0x43, 0x69, 0x74,
0x79, 0x69, 0x2d, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x69, 0x2d, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x2d, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x2d, 0x4a, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61,
0x69, 0x2d, 0x4a, 0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61, 0x69, 0x2d,
0x4a, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x69, 0x2d, 0x4b,
0x61, 0x62, 0x75, 0x6c, 0x69, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x6b, 0x61, 0x69, 0x2d, 0x4b, 0x61, 0x72, 0x61, 0x63, 0x68, 0x69,
0x69, 0x2d, 0x4b, 0x61, 0x74, 0x68, 0x6d, 0x61, 0x6e, 0x64, 0x75, 0x69,
0x2d, 0x4b, 0x68, 0x61, 0x6e, 0x64, 0x79, 0x67, 0x61, 0x69, 0x2d, 0x4b,
0x6f, 0x6c, 0x6b, 0x61, 0x74, 0x61, 0x69, 0x2d, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x2d, 0x4b, 0x75, 0x61,
0x6c, 0x61, 0x20, 0x4c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x69, 0x2d, 0x4b,
0x75, 0x63, 0x68, 0x69, 0x6e, 0x67, 0x69, 0x2d, 0x4b, 0x75, 0x77, 0x61,
0x69, 0x74, 0x69, 0x2d, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x69, 0x2d, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x2d, 0x4d, 0x61, 0x6b, 0x61,
0x73, 0x73, 0x61, 0x72, 0x69, 0x2d, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x61,
0x69, 0x2d, 0x4d, 0x75, 0x73, 0x63, 0x61, 0x74, 0x69, 0x2d, 0x4e, 0x69,
0x63, 0x6f, 0x73, 0x69, 0x61, 0x69, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x6b,
0x75, 0x7a, 0x6e, 0x65, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x2d, 0x4f, 0x6d,
0x73, 0x6b, 0x69, 0x2d, 0x4f, 0x72, 0x61, 0x6c, 0x69, 0x2d, 0x50, 0x68,
0x6e, 0x6f, 0x6d, 0x20, 0x50, 0x65, 0x6e, 0x68, 0x69, 0x2d, 0x50, 0x6f,
0x6e, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x6b, 0x69, 0x2d, 0x50, 0x79, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x69, 0x2d, 0x51, 0x61, 0x74, 0x61,
0x72, 0x49, 0x2d, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x69,
0x2d, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x69, 0x2d,
0x52, 0x69, 0x79, 0x61, 0x64, 0x68, 0x69, 0x2d, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x69, 0x2d, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b,
0x61, 0x6e, 0x64, 0x69, 0x2d, 0x53, 0x65, 0x6f, 0x75, 0x6c, 0x69, 0x2d,
0x53, 0x68, 0x61, 0x6e, 0x67, 0x68, 0x61, 0x69, 0x69, 0x2d, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x69, 0x2d, 0x53, 0x72, 0x65,
0x64, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x79, 0x6d, 0x73, 0x6b, 0x69, 0x2d,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x69, 0x2d, 0x54, 0x61, 0x73, 0x68,
0x6b, 0x65, 0x6e, 0x74, 0x69, 0x2d, 0x54, 0x62, 0x69, 0x6c, 0x69, 0x73,
0x69, 0x69, 0x2d, 0x54, 0x65, 0x68, 0x72, 0x61, 0x6e, 0x69, 0x2d, 0x54,
0x68, 0x69, 0x6d, 0x70, 0x68, 0x75, 0x69, 0x2d, 0x54, 0x6f, 0x6b, 0x79,
0x6f, 0x69, 0x2d, 0x54, 0x6f, 0x6d, 0x73, 0x6b, 0x69, 0x2d, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x69, 0x2d, 0x55,
0x72, 0x75, 0x6d, 0x71, 0x69, 0x69, 0x2d, 0x55, 0x73, 0x74, 0x2d, 0x4e,
0x65, 0x72, 0x61, 0x69, 0x2d, 0x56, 0x69, 0x65, 0x6e, 0x74, 0x69, 0x61,
0x6e, 0x65, 0x69, 0x2d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x69, 0x2d, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x2d, 0x52, 0x61, 0x6e, 0x67, 0x6f, 0x6f, 0x6e, 0x69, 0x2d, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x69, 0x2d, 0x59, 0x65, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x69, 0x2d, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x69, 0x2d, 0x42, 0x65, 0x72, 0x6d, 0x75,
0x64, 0x61, 0x69, 0x2d, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x69, 0x2d,
0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x69, 0x2d,
0x46, 0x61, 0x72, 0x6f, 0x65, 0x69, 0x2d, 0x4d, 0x61, 0x64, 0x65, 0x69,
0x72, 0x61, 0x69, 0x2d, 0x52, 0x65, 0x79, 0x6b, 0x6a, 0x61, 0x76, 0x69,
0x6b, 0x69, 0x2d, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x48, 0x65,
0x6c, 0x65, 0x6e, 0x61, 0x69, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x6c, 0x65,
0x79, 0x69, 0x2d, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x69, 0x64, 0x65, 0x69,
0x2d, 0x42, 0x72, 0x69, 0x73, 0x62, 0x61, 0x6e, 0x65, 0x69, 0x2d, 0x42,
0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x6c, 0x69, 0x2d,
0x44, 0x61, 0x72, 0x77, 0x69, 0x6e, 0x69, 0x2d, 0x45, 0x75, 0x63, 0x6c,
0x61, 0x69, 0x2d, 0x48, 0x6f, 0x62, 0x61, 0x72, 0x74, 0x69, 0x2d, 0x4c,
0x69, 0x6e, 0x64, 0x65, 0x6d, 0x61, 0x6e, 0x69, 0x2d, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x69, 0x2d, 0x4d, 0x65, 0x6c, 0x62,
0x6f, 0x75, 0x72, 0x6e, 0x65, 0x69, 0x2d, 0x50, 0x65, 0x72, 0x74, 0x68,
0x69, 0x2d, 0x53, 0x79, 0x64, 0x6e, 0x65, 0x79, 0x69, 0x2d, 0x41, 0x6d,
0x73, 0x74, 0x65, 0x72, 0x64, 0x61, 0x6d, 0x69, 0x2d, 0x41, 0x6e, 0x64,
0x6f, 0x72, 0x72, 0x61, 0x69, 0x2d, 0x41, 0x73, 0x74, 0x72, 0x61, 0x6b,
0x68, 0x61, 0x6e, 0x69, 0x2d, 0x41, 0x74, 0x68, 0x65, 0x6e, 0x73, 0x69,
0x2d, 0x42, 0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x65, 0x69, 0x2d, 0x42,
0x65, 0x72, 0x6c, 0x69, 0x6e, 0x69, 0x2d, 0x42, 0x72, 0x61, 0x74, 0x69,
0x73, 0x6c, 0x61, 0x76, 0x61, 0x69, 0x2d, 0x42, 0x72, 0x75, 0x73, 0x73,
0x65, 0x6c, 0x73, 0x69, 0x2d, 0x42, 0x75, 0x63, 0x68, 0x61, 0x72, 0x65,
0x73, 0x74, 0x69, 0x2d, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74,
0x69, 0x2d, 0x42, 0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x69, 0x2d,
0x43, 0x68, 0x69, 0x73, 0x69, 0x6e, 0x61, 0x75, 0x69, 0x2d, 0x43, 0x6f,
0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x65, 0x6e, 0x69, 0x2d, 0x44, 0x75,
0x62, 0x6c, 0x69, 0x6e, 0x69, 0x2d, 0x47, 0x69, 0x62, 0x72, 0x61, 0x6c,
0x74, 0x61, 0x72, 0x69, 0x2d, 0x47, 0x75, 0x65, 0x72, 0x6e, 0x73, 0x65,
0x79, 0x69, 0x2d, 0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x69,
0x2d, 0x49, 0x73, 0x6c, 0x65, 0x20, 0x6f, 0x66, 0x20, 0x4d, 0x61, 0x6e,
0x69, 0x2d, 0x49, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x69, 0x2d,
0x4a, 0x65, 0x72, 0x73, 0x65, 0x79, 0x69, 0x2d, 0x4b, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x69, 0x2d, 0x4b, 0x69, 0x72,
0x6f, 0x76, 0x69, 0x2d, 0x4b, 0x69, 0x65, 0x76, 0x69, 0x2d, 0x4c, 0x69,
0x73, 0x62, 0x6f, 0x6e, 0x69, 0x2d, 0x4c, 0x6a, 0x75, 0x62, 0x6c, 0x6a,
0x61, 0x6e, 0x61, 0x69, 0x2d, 0x4c, 0x6f, 0x6e, 0x64, 0x6f, 0x6e, 0x69,
0x2d, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x69,
0x2d, 0x4d, 0x61, 0x64, 0x72, 0x69, 0x64, 0x69, 0x2d, 0x4d, 0x61, 0x6c,
0x74, 0x61, 0x69, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x65, 0x68, 0x61, 0x6d,
0x6e, 0x69, 0x2d, 0x4d, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x2d, 0x4d, 0x6f,
0x6e, 0x61, 0x63, 0x6f, 0x69, 0x2d, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x69, 0x2d, 0x4f, 0x73, 0x6c, 0x6f, 0x69, 0x2d, 0x50, 0x61, 0x72, 0x69,
0x73, 0x69, 0x2d, 0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x63, 0x61,
0x69, 0x2d, 0x50, 0x72, 0x61, 0x67, 0x75, 0x65, 0x69, 0x2d, 0x52, 0x69,
0x67, 0x61, 0x69, 0x2d, 0x52, 0x6f, 0x6d, 0x65, 0x69, 0x2d, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61,
0x72, 0x69, 0x6e, 0x6f, 0x69, 0x2d, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65,
0x76, 0x6f, 0x69, 0x2d, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76, 0x69,
0x2d, 0x53, 0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x69,
0x2d, 0x53, 0x6b, 0x6f, 0x70, 0x6a, 0x65, 0x69, 0x2d, 0x53, 0x6f, 0x66,
0x69, 0x61, 0x69, 0x2d, 0x53, 0x74, 0x6f, 0x63, 0x6b, 0x68, 0x6f, 0x6c,
0x6d, 0x69, 0x2d, 0x54, 0x61, 0x6c, 0x6c, 0x69, 0x6e, 0x6e, 0x69, 0x2d,
0x54, 0x69, 0x72, 0x61, 0x6e, 0x65, 0x69, 0x2d, 0x55, 0x6c, 0x79, 0x61,
0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x69, 0x2d, 0x56, 0x61, 0x64, 0x75, 0x7a,
0x69, 0x2d, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x69, 0x2d, 0x56,
0x69, 0x65, 0x6e, 0x6e, 0x61, 0x69, 0x2d, 0x56, 0x69, 0x6c, 0x6e, 0x69,
0x75, 0x73, 0x69, 0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x69, 0x2d, 0x57, 0x61, 0x72, 0x73, 0x61, 0x77, 0x69, 0x2d, 0x5a,
0x61, 0x67, 0x72, 0x65, 0x62, 0x69, 0x2d, 0x5a, 0x75, 0x72, 0x69, 0x63,
0x68, 0x69, 0x2d, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72,
0x69, 0x76, 0x6f, 0x69, 0x2d, 0x43, 0x68, 0x61, 0x67, 0x6f, 0x73, 0x55,
0x6b, 0x68, 0x69, 0x73, 0x69, 0x6d, 0x75, 0x73, 0x69, 0x69, 0x2d, 0x43,
0x6f, 0x63, 0x6f, 0x73, 0x69, 0x2d, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x6f,
0x69, 0x2d, 0x4b, 0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e, 0x69,
0x4d, 0x61, 0x68, 0x65, 0x69, 0x2d, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x65, 0x73, 0x69, 0x2d, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x69, 0x2d, 0x4d, 0x61, 0x79, 0x6f, 0x74, 0x74, 0x65, 0x69, 0x2d,
0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x2d, 0x41, 0x70, 0x69,
0x61, 0x69, 0x2d, 0x41, 0x75, 0x63, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x2d, 0x42, 0x6f, 0x75, 0x67, 0x61, 0x69, 0x6e, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x69, 0x2d, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x2d,
0x43, 0x68, 0x75, 0x75, 0x6b, 0x69, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x69, 0x2d, 0x45, 0x66, 0x61, 0x74, 0x65, 0x69, 0x2d, 0x46, 0x61,
0x6b, 0x61, 0x6f, 0x66, 0x6f, 0x69, 0x2d, 0x46, 0x69, 0x6a, 0x69, 0x69,
0x2d, 0x46, 0x75, 0x6e, 0x61, 0x66, 0x75, 0x74, 0x69, 0x69, 0x2d, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x69, 0x2d, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x64, 0x61,
0x6c, 0x63, 0x61, 0x6e, 0x61, 0x6c, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x6d,
0x69, 0x2d, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x69, 0x2d,
0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x75, 0x72, 0x79, 0x69, 0x2d, 0x4b,
0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x69, 0x2d, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x69, 0x2d, 0x4b, 0x77, 0x61, 0x6a, 0x61,
0x6c, 0x65, 0x69, 0x6e, 0x69, 0x2d, 0x4d, 0x61, 0x6a, 0x75, 0x72, 0x6f,
0x69, 0x2d, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x69,
0x2d, 0x4d, 0x69, 0x64, 0x77, 0x61, 0x79, 0x69, 0x2d, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x69, 0x2d, 0x4e, 0x69, 0x75, 0x65, 0x69, 0x2d, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x2d, 0x4e, 0x6f, 0x75, 0x6d, 0x65,
0x61, 0x69, 0x2d, 0x50, 0x61, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x67, 0x6f,
0x69, 0x2d, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x69, 0x2d, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x69, 0x2d, 0x50, 0x6f, 0x68, 0x6e, 0x70,
0x65, 0x69, 0x69, 0x2d, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x4d, 0x6f, 0x72,
0x65, 0x73, 0x62, 0x79, 0x69, 0x2d, 0x52, 0x61, 0x72, 0x6f, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x69, 0x2d, 0x53, 0x61, 0x69, 0x70, 0x61, 0x6e, 0x69,
0x2d, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x69, 0x2d, 0x54, 0x61, 0x72,
0x61, 0x77, 0x61, 0x69, 0x2d, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0x61,
0x70, 0x75, 0x69, 0x2d, 0x57, 0x61, 0x6b, 0x65, 0x69, 0x2d, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x41, 0x6d, 0x69, 0x6e, 0x6a, 0x61, 0x6e, 0x41,
0x6e, 0x69, 0x73, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x6d, 0x61, 0x41, 0x72,
0x6a, 0xe9, 0x72, 0x41, 0x73, 0x69, 0x6d, 0x1ef9, 0x72, 0x61, 0x4d, 0x61,
0x6d, 0x1ef9, 0x6b, 0x6f, 0x4d, 0x61, 0x67, 0x69, 0x4d, 0x61, 0x6e, 0x6a,
0x75, 0x72, 0x4d, 0x69, 0x73, 0x61, 0x76, 0x4d, 0x72, 0x61, 0x6e, 0x74,
0x79, 0x72, 0x65, 0x4d, 0x72, 0x61, 0x6a, 0x61, 0x76, 0x69, 0x72, 0x65,
0x4d, 0x75, 0x6a, 0x169, 0x6d, 0x75, 0x72, 0x61, 0x128, 0x6e, 0x6b, 0x75,
0x70, 0x72, 0x69, 0x53, 0x65, 0x76, 0x75, 0x74, 0x61, 0x4b, 0x6f, 0x6e,
0x1ef9, 0x6b, 0x72, 0x69, 0x4e, 0x61, 0x6b, 0x61, 0x72, 0x4e, 0x61, 0x72,
0x20, 0x45, 0x6e, 0x68, 0x20, 0x53, 0x61, 0x72, 0x1ef9, 0x6d, 0x4e, 0x68,
0x69, 0x6d, 0x75, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x61, 0x72, 0x61, 0x45,
0x72, 0x20, 0x41, 0x2019, 0x61, 0x6a, 0x75, 0x6e, 0x1ebc, 0x6d, 0xe3, 0x2d,
0x66, 0x72, 0x69, 0x47, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x6e, 0x129, 0x4a,
0x75, 0x76, 0x1ef9, 0x6e, 0x1ebd, 0x6e, 0x68, 0x6d, 0x75, 0x72, 0x67, 0x75,
0x4a, 0x75, 0x6d, 0x61, 0x4b, 0xe3, 0x6d, 0x70, 0x61, 0x72, 0x61, 0x4b,
0x61, 0x72, 0x74, 0x169, 0x6d, 0x4b, 0x69, 0x67, 0x61, 0x72, 0x69, 0x4b,
0x129, 0x73, 0x61, 0x6a, 0x61, 0x52, 0x61, 0x67, 0x6f, 0x52, 0x69, 0x6d,
0x72, 0x65, 0x76, 0x69, 0x72, 0x69, 0x52, 0xf3, 0x6d, 0x1ebd, 0x52, 0x75,
0x76, 0x1ef9, 0x6e, 0x61, 0x52, 0x75, 0x6d, 0x75, 0x6d, 0x61, 0x73, 0x69,
0x52, 0x75, 0x73, 0x61, 0x6b, 0x61, 0x4d, 0x1ef9, 0x72, 0x61, 0x6d, 0x6f,
0x4d, 0x1ef9, 0x70, 0x75, 0x74, 0x75, 0x4d, 0x1ef9, 0x73, 0x65, 0x72, 0x75,
0x4d, 0x129, 0x6d, 0x61, 0x6d, 0x61, 0x6e, 0x1ebd, 0x4d, 0x169, 0x67, 0x61,
0x6e, 0x69, 0x6e, 0x68, 0x73, 0x75, 0x4d, 0xe3, 0x6e, 0x72, 0xf3, 0x76,
0x69, 0x6a, 0x61, 0x4e, 0x1ef9, 0x6a, 0x72, 0xf3, 0x6d, 0x69, 0x4e, 0x69,
0x2019, 0x6e, 0x69, 0x6a, 0x61, 0x6d, 0x1ebd, 0x6e, 0x1ef9, 0x4e, 0x129, 0x61,
0x6d, 0x1ebd, 0x6a, 0x4e, 0xe3, 0x76, 0x61, 0x67, 0x73, 0xf3, 0x74, 0x69,
0x4f, 0x76, 0x61, 0x67, 0x61, 0x6e, 0x6f, 0x67, 0x6f, 0x76, 0x50, 0x6f,
0x72, 0x74, 0x6f, 0x20, 0x54, 0xe3, 0x67, 0x53, 0x1ef9, 0x20, 0x54, 0x75,
0x6d, 0x1ebd, 0x54, 0x72, 0x69, 0x70, 0x6f, 0x72, 0x69, 0x54, 0x75, 0x6e,
0x129, 0x6e, 0x68, 0x56, 0x129, 0x6e, 0x6e, 0x6f, 0xe9, 0x6b, 0x69, 0x41,
0x6e, 0x61, 0x6b, 0x69, 0x1ef8, 0x6b, 0x6f, 0x72, 0x61, 0x67, 0x69, 0x1ef8,
0x67, 0x69, 0x72, 0x61, 0x1ef8, 0x74, 0x69, 0x67, 0x75, 0x76, 0x61, 0x41,
0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x6e, 0x1ef9, 0x4d, 0x75, 0x65, 0x6e,
0x169, 0x73, 0x61, 0x69, 0x72, 0x69, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x1ef9,
0x72, 0x6b, 0x61, 0x4b, 0xf3, 0x72, 0x6e, 0x6f, 0x6d, 0x61, 0x4a, 0x75,
0x6a, 0x75, 0x6a, 0x52, 0x61, 0x20, 0x52, 0x69, 0x6f, 0x68, 0x61, 0x4d,
0x1ebd, 0x6e, 0x6f, 0x73, 0x61, 0x47, 0x61, 0x72, 0x65, 0x67, 0x6f, 0x20,
0x47, 0x6f, 0x6a, 0x53, 0x61, 0x72, 0x74, 0x61, 0x53, 0x1ef9, 0x20, 0x4a,
0x75, 0x76, 0x1ef9, 0x53, 0x1ef9, 0x20, 0x52, 0x75, 0x6a, 0x54, 0x75, 0x6b,
0x75, 0x6d, 0x1ef9, 0x55, 0x73, 0x75, 0x61, 0x6a, 0x61, 0x41, 0x72, 0x75,
0x6d, 0x61, 0x41, 0x73, 0x169, 0x67, 0x73, 0x1ef9, 0x41, 0x74, 0x69, 0x6b,
0x6f, 0x6b, 0x1ef9, 0x6e, 0x4d, 0x61, 0x6a, 0x69, 0x61, 0x4d, 0x61, 0x6a,
0x69, 0x61, 0x20, 0x4d, 0xe1, 0x6e, 0x6e, 0x65, 0x72, 0x61, 0x20, 0x41,
0x67, 0x4d, 0x61, 0x72, 0x6d, 0x61, 0x6e, 0x75, 0x4d, 0x65, 0x72, 0x1ebd,
0x6a, 0x4d, 0x65, 0x72, 0x69, 0x73, 0x65, 0x53, 0x61, 0x6d, 0x72, 0xe3,
0x6e, 0x20, 0x4b, 0x75, 0x70, 0x72, 0x69, 0x56, 0x65, 0x6a, 0x61, 0x20,
0x48, 0xe1, 0x4d, 0x6f, 0x67, 0x6f, 0x74, 0x61, 0x4d, 0x6f, 0x6a, 0x73,
0x65, 0x4b, 0x1ebd, 0x6d, 0x72, 0x69, 0x6e, 0x6a, 0x65, 0x20, 0x4d, 0x65,
0x6a, 0x52, 0x65, 0x20, 0x4d, 0xe1, 0x67, 0x4b, 0x1ef9, 0x6b, 0x169, 0x6e,
0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x6e, 0x68, 0x4b, 0x61, 0x6a, 0x65,
0x6e, 0x1ef9, 0x4b, 0x61, 0x6a, 0x6d, 0x1ef9, 0x6e, 0x53, 0x69, 0x6b, 0x61,
0x67, 0x75, 0x53, 0x69, 0x68, 0x75, 0x76, 0x61, 0x68, 0x75, 0x76, 0x61,
0x50, 0xe3, 0x6e, 0x129, 0x20, 0x54, 0x169, 0x20, 0x4d, 0xe1, 0x67, 0x4b,
0x72, 0xe9, 0x73, 0x69, 0x74, 0xe3, 0x6e, 0x4b, 0x75, 0x6a, 0x61, 0x6d,
0x61, 0x4b, 0x75, 0x72, 0x61, 0x73, 0x61, 0x76, 0x6f, 0x4e, 0xe1, 0x6e,
0x6d, 0x1ef9, 0x72, 0x6b, 0x69, 0x73, 0x61, 0x76, 0x79, 0x6e, 0x4e, 0x61,
0x76, 0x73, 0xe3, 0x6e, 0x4e, 0x61, 0x75, 0x73, 0xe3, 0x6e, 0x20, 0x4b,
0x72, 0x69, 0x6b, 0x69, 0x4e, 0x65, 0x6e, 0x76, 0x65, 0x72, 0x4e, 0x65,
0x74, 0x6f, 0x72, 0xf3, 0x6a, 0x74, 0x69, 0x4e, 0x6f, 0x6d, 0x129, 0x6e,
0x129, 0x6b, 0x61, 0x45, 0x6e, 0x69, 0x6d, 0xe3, 0x6e, 0x74, 0xe3, 0x45,
0x6a, 0x72, 0x75, 0x6e, 0x1ebd, 0x70, 0xe9, 0xc9, 0x72, 0x20, 0x53, 0x61,
0x72, 0x76, 0x61, 0x6e, 0x6f, 0x72, 0x46, 0xf3, 0x72, 0x74, 0x69, 0x20,
0x4e, 0x1ebd, 0x72, 0x73, 0x75, 0x46, 0x6f, 0x72, 0x74, 0x61, 0x72, 0x65,
0x6a, 0x61, 0x47, 0x72, 0x61, 0x73, 0x65, 0x20, 0x4d, 0x65, 0x6a, 0x47,
0x1ef9, 0x73, 0x6f, 0x20, 0x4d, 0x65, 0x6a, 0x54, 0x75, 0x72, 0x6b, 0x69,
0x20, 0x4d, 0xe1, 0x67, 0x47, 0x72, 0x61, 0x6e, 0x1ef9, 0x6e, 0x61, 0x47,
0x75, 0x76, 0x61, 0x6e, 0x61, 0x72, 0x75, 0x70, 0x65, 0x47, 0x75, 0x72,
0x61, 0x74, 0x65, 0x6d, 0x1ef9, 0x72, 0x61, 0x47, 0x75, 0x76, 0x61, 0x6a,
0x61, 0x6b, 0x69, 0x72, 0x47, 0x75, 0x6a, 0xe1, 0x6e, 0x1ef9, 0x48, 0x61,
0x72, 0x69, 0x66, 0x61, 0x67, 0x48, 0x61, 0x76, 0xe1, 0x6e, 0x1ef9, 0x48,
0xe9, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x72, 0x6f, 0x128, 0x6e, 0x69, 0x6a,
0x61, 0x6e, 0x1ef9, 0x70, 0x6f, 0x72, 0x69, 0x6e, 0x68, 0x4b, 0x69, 0x6e,
0xe3, 0x67, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x4d,
0x1ef9, 0x72, 0x1ebd, 0x67, 0x67, 0x75, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a,
0x1ef9, 0x6e, 0x1ef9, 0x50, 0xe9, 0x74, 0x65, 0x72, 0x6d, 0x75, 0x72, 0x67,
0x69, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x54, 0x65,
0x72, 0x20, 0x73, 0x69, 0x74, 0x69, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a,
0x1ef9, 0x6e, 0x1ef9, 0x56, 0x65, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x128, 0x6e,
0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x56, 0x129, 0x73, 0x65, 0x6e, 0x1ebd, 0x6e,
0x68, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x56, 0x69,
0x6e, 0x1ef9, 0x6d, 0x1ef9, 0x6b, 0x69, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a,
0x1ef9, 0x6e, 0x1ef9, 0x49, 0x6e, 0x169, 0x76, 0x69, 0x6b, 0x69, 0x49, 0x6b,
0x61, 0x72, 0x75, 0x6a, 0x69, 0x6e, 0x4a, 0x61, 0x6d, 0x1ef9, 0x6a, 0x6b,
0x1ef9, 0x4a, 0x75, 0x6e, 0x1ef9, 0x76, 0x6f, 0x52, 0x75, 0x69, 0x6e, 0x68,
0x76, 0x69, 0x72, 0x69, 0x4d, 0xe3, 0x6e, 0x74, 0x69, 0x73, 0xe9, 0x72,
0x75, 0x2c, 0x20, 0x4b, 0x1ebd, 0x74, 0xe1, 0x6b, 0x69, 0x4b, 0x72, 0x61,
0x72, 0x1ebd, 0x6e, 0x69, 0x6b, 0x65, 0x52, 0x61, 0x20, 0x50, 0x61, 0x73,
0x69, 0x52, 0x69, 0x6d, 0x1ef9, 0x52, 0x6f, 0x6e, 0x68, 0x20, 0x1ef8, 0x6e,
0x6a, 0x69, 0x72, 0x69, 0x52, 0x6f, 0x76, 0x65, 0x72, 0x20, 0x50, 0x72,
0x129, 0x6e, 0x73, 0x69, 0x20, 0x4b, 0x75, 0x76, 0x61, 0x72, 0x74, 0xe9,
0x72, 0x4d, 0x1ef9, 0x73, 0x65, 0x6a, 0xf3, 0x4d, 0x1ef9, 0x6e, 0x1ef9, 0x67,
0x75, 0x76, 0x61, 0x4d, 0x1ef9, 0x6e, 0x1ef9, 0x76, 0x6f, 0x4d, 0x1ef9, 0x72,
0x69, 0x67, 0xf3, 0x74, 0x69, 0x4d, 0x1ef9, 0x72, 0x74, 0x69, 0x6e, 0x129,
0x6b, 0x61, 0x4d, 0xe3, 0x72, 0x6f, 0x2d, 0x61, 0x67, 0x20, 0x54, 0xe3,
0x6e, 0x68, 0x4d, 0x1ef9, 0x6a, 0x61, 0x74, 0x72, 0x1ef9, 0x6e, 0x4d, 0x1ebd,
0x6e, 0xe3, 0x6d, 0x129, 0x6e, 0x129, 0x4d, 0x1ebd, 0x72, 0x69, 0x6e, 0x61,
0x4d, 0x1ebd, 0x74, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x72, 0x61,
0x1ebc, 0x6d, 0xe3, 0x20, 0x74, 0x1ef9, 0x20, 0x4d, 0x1ebd, 0x73, 0x69, 0x6b,
0x6f, 0x4d, 0x129, 0x6b, 0x65, 0x72, 0x6f, 0x6e, 0x4d, 0xe3, 0x67, 0x74,
0xe3, 0x6e, 0x4d, 0xe3, 0x74, 0x65, 0x68, 0x65, 0x6a, 0x4d, 0xe3, 0x74,
0x69, 0x76, 0x69, 0x6e, 0xe9, 0x76, 0x4d, 0xe3, 0x6e, 0x73, 0x65, 0x72,
0xe1, 0x74, 0x69, 0x4e, 0x1ef9, 0x73, 0x61, 0x76, 0x59, 0x6a, 0xf3, 0x72,
0x6b, 0x69, 0x20, 0x54, 0xe3, 0x67, 0x4a, 0x79, 0x6a, 0x79, 0x46, 0x65,
0x72, 0x6e, 0x1ef9, 0x6e, 0x75, 0x20, 0x4e, 0xe3, 0x72, 0xe3, 0x6a, 0x61,
0x2d, 0x74, 0xe1, 0x4d, 0x65, 0x75, 0x72, 0x61, 0x2c, 0x20, 0x4e, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4e, 0xe3, 0x72, 0x74, 0x69, 0x53, 0x1ebd,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x4e, 0xe3, 0x72, 0x74, 0x69, 0x4e, 0x129, 0x75, 0x20, 0x53, 0x61,
0x72, 0x65, 0x6e, 0x2c, 0x20, 0x4e, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20,
0x4e, 0xe3, 0x72, 0x74, 0x69, 0x4e, 0x169, 0x67, 0x4f, 0x6a, 0x69, 0x6e,
0x1ef9, 0x67, 0x61, 0x50, 0x61, 0x6e, 0x1ef9, 0x6d, 0x1ef9, 0x50, 0x61, 0x72,
0x61, 0x6d, 0x1ef9, 0x72, 0x69, 0x6d, 0x75, 0x46, 0x75, 0x65, 0x6e, 0x129,
0x6e, 0x68, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x20, 0x50, 0x72, 0x129, 0x73,
0x69, 0x70, 0x69, 0x50, 0xf3, 0x72, 0x74, 0x6f, 0x66, 0x69, 0x20, 0x49,
0x6e, 0x68, 0x70, 0x61, 0x6a, 0x69, 0x6e, 0x50, 0x6f, 0x72, 0x74, 0x6f,
0x20, 0x4b, 0xf3, 0x66, 0x61, 0x50, 0x169, 0x74, 0x61, 0x20, 0x41, 0x72,
0x65, 0x6e, 0x1ef9, 0x52, 0x1ef9, 0x67, 0x6b, 0x129, 0x67, 0x20, 0x128, 0x72,
0xe9, 0x6e, 0x52, 0x69, 0x73, 0x69, 0x66, 0x69, 0x52, 0x69, 0x6a, 0x69,
0x6e, 0x1ef9, 0x52, 0x69, 0x6a, 0x6f, 0x72, 0x75, 0x74, 0x65, 0x47, 0x6f,
0x6a, 0x20, 0x4b, 0x75, 0x70, 0x72, 0x69, 0x53, 0x1ef9, 0x74, 0x61, 0x72,
0x1ebd, 0x6a, 0x53, 0x1ef9, 0x74, 0x69, 0x6a, 0x61, 0x67, 0x75, 0x53, 0x1ef9,
0x74, 0x75, 0x20, 0x4e, 0x75, 0x6d, 0x129, 0x67, 0x67, 0x75, 0x53, 0x1ef9,
0x20, 0x50, 0x61, 0x76, 0x6f, 0x72, 0x75, 0x49, 0x74, 0x6f, 0x67, 0x6b,
0x6f, 0x72, 0x74, 0x75, 0x72, 0x6d, 0x129, 0x6e, 0x74, 0x69, 0x53, 0x69,
0x6e, 0x6b, 0x61, 0x53, 0x1ef9, 0x20, 0x4d, 0x61, 0x72, 0x74, 0x6f, 0x72,
0x6f, 0x6d, 0x1ebd, 0x53, 0x1ef9, 0x6e, 0x20, 0x4a, 0x6f, 0x6e, 0x68, 0x53,
0x1ef9, 0x20, 0x4b, 0x72, 0x69, 0x74, 0xf3, 0x76, 0x1ef9, 0x53, 0x1ef9, 0x74,
0x61, 0x20, 0x52, 0x75, 0x73, 0x61, 0x53, 0x1ef9, 0x20, 0x54, 0x6f, 0x6d,
0x61, 0x6a, 0x53, 0x1ef9, 0x20, 0x56, 0x69, 0x73, 0x1ebd, 0x74, 0x69, 0x53,
0x75, 0x76, 0x69, 0x66, 0x69, 0x74, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x1ebd,
0x74, 0x69, 0x54, 0x65, 0x67, 0x75, 0x73, 0x69, 0x67, 0x61, 0x72, 0x70,
0x61, 0x54, 0x75, 0x72, 0x65, 0x54, 0x69, 0x76, 0x1ef9, 0x6e, 0x1ef9, 0x54,
0x6f, 0x72, 0xe3, 0x74, 0x6f, 0x54, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x61,
0x56, 0x1ef9, 0x67, 0x6b, 0x75, 0x76, 0x65, 0x72, 0x4b, 0xe3, 0x76, 0xe3,
0x72, 0x75, 0x20, 0x4b, 0x75, 0x70, 0x72, 0x69, 0x56, 0x129, 0x6e, 0x129,
0x70, 0xe9, 0x67, 0x59, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x74, 0x69,
0x4b, 0x61, 0x73, 0x65, 0x6a, 0x4e, 0x61, 0x76, 0x69, 0x73, 0x69, 0x4d,
0x1ef9, 0x6b, 0x69, 0x6b, 0x75, 0x61, 0x72, 0x69, 0x4d, 0x1ef9, 0x75, 0x73,
0xe3, 0x6e, 0x4d, 0xe9, 0x67, 0x2d, 0x4d, 0x169, 0x72, 0x6e, 0x6f, 0x50,
0x61, 0x72, 0x6d, 0x1ebd, 0x72, 0x54, 0x6f, 0x72, 0xf3, 0x72, 0x56, 0x6f,
0x6e, 0x68, 0x74, 0xf3, 0x6b, 0x69, 0x52, 0x169, 0x67, 0x69, 0x6a, 0xe9,
0x20, 0x41, 0x72, 0x6d, 0x79, 0x6a, 0x1ebd, 0x6e, 0x41, 0x6e, 0x65, 0x6e,
0x41, 0x72, 0x6d, 0x1ef9, 0x74, 0x69, 0x41, 0x6d, 0x1ef9, 0x41, 0x6e, 0x1ef9,
0x6e, 0x68, 0x69, 0x72, 0x41, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x41, 0x6b,
0x69, 0x74, 0x6f, 0x6d, 0x65, 0x41, 0x6e, 0x68, 0x67, 0x61, 0x6d, 0x61,
0x74, 0x69, 0x41, 0x74, 0x69, 0x72, 0x61, 0x76, 0x4d, 0x61, 0x67, 0x69,
0x6e, 0x61, 0x4d, 0x61, 0x72, 0x1ebd, 0x6a, 0x4d, 0x79, 0x67, 0x6b, 0xf3,
0x6b, 0x69, 0x4d, 0x61, 0x72, 0x6e, 0x1ef9, 0x75, 0x72, 0x4d, 0x65, 0x6a,
0x72, 0x75, 0x74, 0x69, 0x4d, 0x69, 0x73, 0x69, 0x6b, 0xe9, 0x6b, 0x69,
0x4d, 0x72, 0x75, 0x6e, 0x1ebd, 0x6a, 0x53, 0x69, 0x74, 0x61, 0x4b, 0x6f,
0x72, 0xe3, 0x6d, 0x6d, 0x75, 0x4e, 0x61, 0x6d, 0x1ef9, 0x6e, 0x68, 0x6b,
0x6f, 0x4e, 0x69, 0x72, 0x69, 0x4e, 0x75, 0x6d, 0x61, 0x6a, 0x4e, 0x75,
0x73, 0xe1, 0x67, 0x6d, 0x65, 0x46, 0x61, 0x6d, 0x1ef9, 0x67, 0x75, 0x6a,
0x74, 0x61, 0x47, 0x61, 0x6a, 0x61, 0x48, 0x65, 0x6d, 0x72, 0x6f, 0x6e,
0x48, 0xe3, 0x67, 0x20, 0x4b, 0xe3, 0x67, 0x48, 0x6f, 0x76, 0x69, 0x6e,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x4a, 0x61,
0x69, 0x61, 0x70, 0x75, 0x72, 0x61, 0x4a, 0x65, 0x72, 0x75, 0x6a, 0x61,
0x72, 0x1ebd, 0x6a, 0x4b, 0x61, 0x6d, 0x75, 0x72, 0x4b, 0x1ef9, 0x6d, 0x73,
0x61, 0x6e, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x69, 0x4b, 0x61,
0x74, 0x69, 0x6d, 0x1ef9, 0x6e, 0x6e, 0x75, 0x4b, 0x1ef9, 0x6e, 0x79, 0x67,
0x61, 0x4b, 0x61, 0x72, 0x6b, 0x75, 0x74, 0x61, 0x4b, 0x72, 0x61, 0x6e,
0x68, 0x6e, 0xe3, 0x6a, 0x61, 0x72, 0x6b, 0x69, 0x4b, 0x75, 0x76, 0x61,
0x72, 0x61, 0x20, 0x52, 0x169, 0x70, 0x75, 0x72, 0x4b, 0x75, 0x73, 0x129,
0x67, 0x4b, 0x75, 0x76, 0x61, 0x6a, 0x74, 0x69, 0x4d, 0x1ef9, 0x6b, 0x61,
0x75, 0x4d, 0x1ef9, 0x67, 0x61, 0x6e, 0x61, 0x6e, 0x4d, 0x1ef9, 0x6e, 0x129,
0x72, 0x61, 0x4e, 0x129, 0x6b, 0xf3, 0x6a, 0x69, 0x61, 0x4b, 0x75, 0x6e,
0x68, 0x6e, 0x1ebd, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x54, 0xe3,
0x67, 0x53, 0x69, 0x6d, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x54,
0xe3, 0x67, 0x4f, 0x6d, 0x129, 0x6e, 0x68, 0x6b, 0x69, 0x4f, 0x72, 0x61,
0x72, 0x46, 0x79, 0x6e, 0xe3, 0x67, 0x20, 0x50, 0x1ebd, 0x6a, 0x50, 0xe3,
0x6e, 0x74, 0x69, 0x1ef9, 0x6e, 0x1ef9, 0x6b, 0x69, 0x50, 0x69, 0xe3, 0x67,
0x2d, 0x6a, 0x1ef9, 0x67, 0x4b, 0x6f, 0x6a, 0x74, 0x61, 0x6e, 0x1ef9, 0x6a,
0x4b, 0x79, 0x6a, 0x79, 0x72, 0x6f, 0x72, 0x6e, 0x61, 0x52, 0x69, 0x6a,
0x61, 0x6e, 0x65, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x129, 0x6e, 0x1ef9, 0x53,
0x61, 0x6d, 0x1ef9, 0x72, 0x6b, 0x1ef9, 0x6e, 0x6e, 0x61, 0x53, 0x65, 0x2019,
0x75, 0x72, 0x53, 0x1ef9, 0x67, 0x67, 0x61, 0x6a, 0x53, 0x129, 0x67, 0x61,
0x70, 0x75, 0x72, 0x61, 0x53, 0x69, 0x72, 0x65, 0x6e, 0x1ebd, 0x6b, 0x6f,
0x72, 0x79, 0x6d, 0x73, 0x69, 0x6b, 0x69, 0x54, 0x61, 0x6e, 0x68, 0x6b,
0x1ebd, 0x6e, 0x74, 0x69, 0x54, 0x69, 0x6d, 0x69, 0x72, 0x69, 0x6a, 0x69,
0x54, 0x65, 0x2019, 0x65, 0x72, 0x1ef9, 0x54, 0x129, 0x6d, 0x66, 0x75, 0x54,
0xf3, 0x6b, 0x69, 0x6a, 0x6f, 0x54, 0x6f, 0x6d, 0x129, 0x6e, 0x68, 0x6b,
0x69, 0x55, 0x72, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x72, 0x55,
0x72, 0x169, 0x6d, 0x6b, 0x69, 0x55, 0x6e, 0x68, 0x74, 0x69, 0x2d, 0x4e,
0x1ebd, 0x72, 0x61, 0x56, 0x69, 0x6a, 0x1ebd, 0x74, 0x69, 0x6a, 0x1ef9, 0x6e,
0x129, 0x55, 0x72, 0x61, 0x6e, 0x69, 0x76, 0xf3, 0x6e, 0x68, 0x74, 0xf3,
0x6b, 0x69, 0x59, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b,
0x69, 0x52, 0x61, 0x6e, 0x67, 0x169, 0x6d, 0x45, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x129, 0x6e, 0x6d, 0x75, 0x72, 0x67, 0x75, 0x4a, 0x65, 0x72, 0x65,
0x76, 0x1ef9, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x6e, 0x68, 0x4d, 0x65, 0x72,
0x6d, 0x169, 0x6e, 0x61, 0x4b, 0x61, 0x6e, 0x1ef9, 0x72, 0x69, 0x6a, 0x61,
0x20, 0x41, 0x67, 0x50, 0x75, 0x20, 0x54, 0xe1, 0x6e, 0x68, 0x52, 0x65,
0x6a, 0x6b, 0x69, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x69, 0x4a, 0x69, 0xf3,
0x72, 0x6a, 0x61, 0x20, 0x74, 0x1ef9, 0x20, 0x53, 0x75, 0x72, 0x53, 0x1ef9,
0x74, 0x61, 0x20, 0x45, 0x72, 0x65, 0x6e, 0x1ef9, 0x53, 0x69, 0x74, 0x1ef9,
0x72, 0x65, 0x6a, 0x41, 0x6e, 0x65, 0x72, 0x61, 0x6a, 0x6e, 0x69, 0x4d,
0x72, 0x69, 0x6e, 0x68, 0x6d, 0x61, 0x6e, 0x1ebd, 0x4d, 0x72, 0x75, 0x6b,
0x65, 0x6e, 0x20, 0x48, 0x69, 0x72, 0x4e, 0x61, 0x72, 0x76, 0x129, 0x6e,
0x45, 0x75, 0x6b, 0x72, 0x61, 0x48, 0x6f, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x52, 0x129, 0x6e, 0x65, 0x72, 0x6d, 0x1ef9, 0x6e, 0x52, 0xf3, 0x72, 0x6e,
0x69, 0x20, 0x48, 0x6f, 0x76, 0x65, 0x4d, 0x1ebd, 0x72, 0x6d, 0x75, 0x72,
0x6e, 0x129, 0x50, 0xe9, 0x72, 0x74, 0x69, 0x53, 0x69, 0x6e, 0x69, 0x6e,
0x65, 0x6a, 0x41, 0x6d, 0x129, 0x6e, 0x68, 0x74, 0x65, 0x72, 0x6e, 0xe1,
0x1ef8, 0x6e, 0x6f, 0x68, 0x61, 0x41, 0x6e, 0x68, 0x74, 0x61, 0x6b, 0x1ef9,
0x41, 0x74, 0x65, 0x6e, 0x1ef9, 0x4d, 0x65, 0x72, 0x67, 0x72, 0x61, 0x6e,
0x6f, 0x4d, 0x65, 0x72, 0x2d, 0x72, 0x129, 0x67, 0x4d, 0x72, 0x61, 0x74,
0x69, 0x6e, 0x68, 0x72, 0x61, 0x76, 0x61, 0x4d, 0x72, 0x75, 0x73, 0xe9,
0x72, 0x61, 0x4d, 0x75, 0x6b, 0x61, 0x72, 0xe9, 0x6e, 0x68, 0x74, 0x69,
0x4d, 0x75, 0x6e, 0x61, 0x70, 0x65, 0x6e, 0x68, 0x74, 0x65, 0x4d, 0x79,
0x6a, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x53, 0x69, 0x6a, 0x69, 0x6e, 0x1ef9,
0x76, 0x6f, 0x4b, 0x6f, 0x70, 0x1ebd, 0x6e, 0x68, 0x1ef9, 0x67, 0x65, 0x4e,
0x75, 0x6d, 0x72, 0x69, 0x6e, 0x4a, 0x69, 0x6d, 0x72, 0x61, 0x74, 0x61,
0x72, 0x47, 0xe9, 0x72, 0x6e, 0x129, 0x73, 0x65, 0x6a, 0x48, 0xe9, 0x72,
0x73, 0x129, 0x67, 0x6b, 0x65, 0x168, 0x6e, 0x2d, 0x67, 0x72, 0xe9, 0x20,
0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x49, 0x6e, 0x68, 0x74,
0x61, 0x6d, 0x75, 0x72, 0x4a, 0xe9, 0x72, 0x73, 0x65, 0x6a, 0x4b, 0x61,
0x72, 0x69, 0x6e, 0x129, 0x6e, 0x67, 0x72, 0x61, 0x6e, 0x6f, 0x4b, 0x69,
0x72, 0xf3, 0x76, 0x69, 0x4b, 0x69, 0x6a, 0xe9, 0x76, 0x69, 0x52, 0x69,
0x6e, 0x68, 0x6d, 0x6f, 0x76, 0x61, 0x52, 0x69, 0x6d, 0x72, 0x69, 0x1ef9,
0x6e, 0x1ef9, 0x52, 0xe3, 0x6e, 0x65, 0x72, 0x65, 0x52, 0x75, 0x73, 0x1ebd,
0x67, 0x6d, 0x75, 0x72, 0x67, 0x6f, 0x4d, 0x1ef9, 0x6e, 0x69, 0x72, 0x69,
0x4d, 0x61, 0x72, 0x74, 0x61, 0x4d, 0x1ef9, 0x72, 0x69, 0x65, 0x68, 0x1ef9,
0x6d, 0x4d, 0x129, 0x67, 0x73, 0x69, 0x6b, 0x69, 0x4d, 0xe3, 0x6e, 0x1ef9,
0x6b, 0x6f, 0x4d, 0xe3, 0x6e, 0x68, 0x6b, 0x6f, 0x76, 0xd3, 0x73, 0x69,
0x72, 0x6f, 0x50, 0x61, 0x72, 0x69, 0x6e, 0x68, 0x50, 0x6f, 0x6e, 0x67,
0xf3, 0x72, 0x69, 0x6b, 0x61, 0x52, 0x6f, 0x6d, 0x1ef9, 0x53, 0x61, 0x6d,
0x1ef9, 0x72, 0x61, 0x53, 0x1ef9, 0x20, 0x4d, 0x1ef9, 0x72, 0x129, 0x6e, 0x169,
0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0x6f, 0x2f, 0x53, 0x61, 0x72,
0x61, 0x6a, 0x65, 0x76, 0x75, 0x53, 0x61, 0x72, 0x61, 0x74, 0xf3, 0x76,
0x69, 0x53, 0x129, 0x67, 0x66, 0x69, 0x72, 0x69, 0x70, 0x6f, 0x72, 0x53,
0x69, 0x6b, 0x6f, 0x70, 0x69, 0x6a, 0x65, 0x53, 0xf3, 0x66, 0x69, 0x6a,
0x61, 0x45, 0x6e, 0x68, 0x74, 0x75, 0x6b, 0x6f, 0x72, 0x6d, 0x169, 0x54,
0x61, 0x72, 0x129, 0x6e, 0x54, 0x69, 0x72, 0x1ef9, 0x6e, 0x1ef9, 0x55, 0x72,
0x69, 0x6a, 0x61, 0x6e, 0xe3, 0x76, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x56,
0x61, 0x6e, 0x75, 0x6a, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x1ef9, 0x6e, 0x169,
0x56, 0x69, 0x65, 0x6e, 0x1ef9, 0x56, 0x69, 0x72, 0x6e, 0x129, 0x6a, 0x75,
0x73, 0x69, 0x56, 0x6f, 0x72, 0x67, 0x75, 0x67, 0x72, 0x61, 0x6e, 0x6f,
0x56, 0x61, 0x72, 0x73, 0xf3, 0x76, 0x69, 0x6a, 0x61, 0x4a, 0x61, 0x67,
0x72, 0xe9, 0x6d, 0x65, 0x4a, 0x75, 0x72, 0x69, 0x6b, 0x65, 0x1ef8, 0x74,
0x61, 0x6e, 0x1ef9, 0x6e, 0x1ef9, 0x72, 0x69, 0x76, 0x6f, 0x53, 0x61, 0x67,
0x6f, 0x4b, 0x72, 0x69, 0x6e, 0x68, 0x74, 0x69, 0x6d, 0x1ef9, 0x4b, 0xf3,
0x6b, 0x6f, 0x20, 0x41, 0x67, 0x4b, 0x6f, 0x6d, 0xe3, 0x72, 0x65, 0x4b,
0x69, 0x72, 0x67, 0xe9, 0x72, 0x65, 0x6e, 0x4d, 0x1ef9, 0x68, 0xe9, 0x4d,
0x1ef9, 0x72, 0x6e, 0x69, 0x76, 0x61, 0x4d, 0xe3, 0x72, 0x69, 0x73, 0x69,
0x76, 0x4d, 0x1ef9, 0x6a, 0xf3, 0x74, 0x69, 0x52, 0x169, 0x6e, 0x6a, 0x169,
0x76, 0xd3, 0x67, 0x2d, 0x72, 0x1ef9, 0x6e, 0x4d, 0x75, 0x67, 0x1ef9, 0x6e,
0x76, 0x69, 0x72, 0x65, 0x53, 0x61, 0x74, 0x69, 0x6e, 0x1ef9, 0x6d, 0x50,
0x61, 0x73, 0x69, 0x6b, 0x75, 0x76, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d,
0x76, 0x1ebd, 0x73, 0x6f, 0x46, 0x61, 0x6b, 0x61, 0x6f, 0x66, 0x75, 0x46,
0x75, 0x6e, 0x1ef9, 0x66, 0x75, 0x74, 0x69, 0x47, 0x61, 0x72, 0x61, 0x20,
0x50, 0x61, 0x67, 0x6f, 0x47, 0x1ef9, 0x6d, 0x6d, 0x69, 0xe9, 0x72, 0x47,
0x75, 0x76, 0x61, 0x6e, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x1ef9, 0x72, 0x47,
0x75, 0x76, 0x1ef9, 0x6d, 0x48, 0x6f, 0x6e, 0xe3, 0x72, 0x75, 0x72, 0x75,
0x1ebc, 0x6e, 0x6e, 0x65, 0x72, 0x6d, 0x75, 0x72, 0x69, 0x4b, 0x69, 0x72,
0x69, 0x74, 0x69, 0x6d, 0x1ef9, 0x74, 0x69, 0x4b, 0x6f, 0x73, 0x69, 0x72,
0x61, 0x6a, 0x65, 0x4b, 0x75, 0x76, 0x61, 0x6a, 0x61, 0x72, 0x1ebd, 0x6a,
0x4d, 0x1ef9, 0x6a, 0x75, 0x72, 0x6f, 0x4d, 0x1ef9, 0x72, 0x6b, 0x65, 0x6a,
0x61, 0x20, 0x46, 0x61, 0x67, 0x4d, 0x129, 0x6e, 0x6e, 0x75, 0x76, 0x65,
0x6a, 0x4e, 0x1ef9, 0x76, 0x75, 0x72, 0x75, 0x4e, 0x129, 0x76, 0x65, 0x4e,
0xe3, 0x72, 0x66, 0x6f, 0x6b, 0x69, 0x4e, 0xe3, 0x75, 0x6d, 0x1ebd, 0x61,
0x50, 0x61, 0x72, 0x61, 0x76, 0x50, 0x69, 0x74, 0x69, 0x6b, 0x61, 0x69,
0x72, 0x50, 0xf3, 0x72, 0x20, 0x4d, 0xe3, 0x72, 0x65, 0x6e, 0x68, 0x6d,
0x69, 0x52, 0x61, 0x72, 0xe3, 0x74, 0xe3, 0x67, 0x61, 0x53, 0x61, 0x6a,
0x70, 0x1ef9, 0x54, 0x61, 0x6a, 0x74, 0x69, 0x54, 0x61, 0x72, 0x61, 0x75,
0x76, 0x61, 0x54, 0xe3, 0x6e, 0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x56,
0x61, 0x6b, 0x69, 0x56, 0x61, 0x72, 0x69, 0x6e, 0x68, 0x41, 0x62, 0x69,
0x64, 0x69, 0x79, 0xe3, 0x41, 0x6b, 0x61, 0x72, 0x61, 0x41, 0x64, 0x69,
0x73, 0x69, 0x20, 0x41, 0x62, 0x65, 0x62, 0x61, 0x41, 0x67, 0x65, 0x75,
0x41, 0x73, 0x69, 0x6d, 0x61, 0x72, 0x61, 0x42, 0xe3, 0x6b, 0x69, 0x42,
0xe3, 0x6a, 0xfa, 0x42, 0x61, 0x72, 0xe3, 0x74, 0x69, 0x72, 0x65, 0x42,
0x61, 0x72, 0x61, 0x7a, 0x61, 0x77, 0x69, 0x72, 0x69, 0x42, 0x75, 0x79,
0x169, 0x62, 0x75, 0x72, 0x61, 0x4b, 0x61, 0x69, 0x72, 0x75, 0x55, 0x6b,
0x61, 0x20, 0x4d, 0x75, 0x72, 0x75, 0x74, 0x129, 0x67, 0x61, 0x4b, 0x75,
0x6e, 0x61, 0x6b, 0x69, 0x72, 0x69, 0x4b, 0x61, 0x74, 0x75, 0x61, 0x77,
0x61, 0x20, 0x72, 0x75, 0x6b, 0x61, 0x44, 0x69, 0x6a, 0x69, 0x62, 0x75,
0x74, 0x69, 0x44, 0x75, 0x77, 0x61, 0x6c, 0x61, 0x45, 0x75, 0x20, 0x41,
0x79, 0x169, 0x46, 0x69, 0x72, 0x65, 0x74, 0x169, 0x47, 0x61, 0x62, 0x75,
0x72, 0x75, 0x6e, 0x69, 0x4a, 0x75, 0x61, 0x6e, 0x65, 0x73, 0x69, 0x62,
0x75, 0x67, 0x75, 0x59, 0x75, 0x62, 0x61, 0x4b, 0xe3, 0x70, 0x61, 0x72,
0x61, 0x4b, 0x61, 0x74, 0x169, 0x4b, 0x129, 0x78, 0x61, 0x73, 0x61, 0x49,
0x70, 0x61, 0x77, 0x61, 0x69, 0x74, 0x61, 0x52, 0x69, 0x62, 0x65, 0x72,
0x65, 0x76, 0x69, 0x6c, 0x69, 0x52, 0x75, 0x6d, 0xe9, 0x52, 0x75, 0xe3,
0x64, 0x61, 0x4c, 0x75, 0x62, 0x169, 0x62, 0x61, 0x78, 0x69, 0x168, 0x62,
0x61, 0x62, 0x61, 0x6e, 0x69, 0x4d, 0x75, 0x67, 0x61, 0x64, 0x69, 0x73,
0x69, 0x6b, 0x75, 0x4d, 0x169, 0x72, 0x75, 0x77, 0x69, 0x79, 0x61, 0x4e,
0x61, 0x69, 0x72, 0x75, 0x62, 0x69, 0x4e, 0x2019, 0x20, 0x44, 0x69, 0x79,
0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x6f, 0x77, 0x61, 0x6b, 0x69, 0x78,
0x75, 0x74, 0x69, 0x55, 0x77, 0x61, 0x67, 0x61, 0x64, 0x75, 0x67, 0x75,
0x49, 0x67, 0x61, 0x72, 0x61, 0x70, 0x61, 0x77, 0x61, 0x20, 0x50, 0x69,
0x73, 0x61, 0x73, 0xfa, 0x53, 0xe3, 0x20, 0x54, 0x75, 0x6d, 0xe9, 0x54,
0x69, 0x72, 0x69, 0x70, 0x75, 0x72, 0x69, 0x54, 0x75, 0x6e, 0x69, 0x2d,
0x69, 0x74, 0x61, 0x57, 0x129, 0x64, 0x75, 0x65, 0x6b, 0x69, 0xc3, 0x6b,
0x75, 0x72, 0x61, 0x67, 0x65, 0xc3, 0x67, 0x69, 0x72, 0x61, 0xc3, 0x74,
0x69, 0x67, 0x75, 0x77, 0x61, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x69, 0x6e, 0x61, 0x42, 0x75, 0x77, 0x65, 0x6e, 0x75, 0x73, 0x61, 0x69,
0x72, 0x69, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x6b, 0x61, 0x4b, 0xf3,
0x64, 0x75, 0x62, 0x61, 0x59, 0x75, 0x79, 0x75, 0x69, 0x52, 0x61, 0x20,
0x52, 0x69, 0x6f, 0x79, 0x61, 0x4d, 0x1ebd, 0x64, 0x75, 0x73, 0x61, 0x50,
0x61, 0x72, 0x61, 0x6e, 0xe3, 0x20, 0x47, 0x61, 0x72, 0x65, 0x67, 0x75,
0x2d, 0x69, 0x74, 0x61, 0x53, 0x61, 0x75, 0x74, 0x61, 0x53, 0xe3, 0x20,
0x59, 0x75, 0x77, 0xe3, 0x53, 0xe3, 0x20, 0x52, 0x75, 0x69, 0x54, 0x75,
0x6b, 0x75, 0x6d, 0xe3, 0x55, 0x78, 0x75, 0x77, 0x61, 0x79, 0x61, 0x41,
0x73, 0x169, 0x73, 0x69, 0x79, 0xf5, 0x41, 0x74, 0x69, 0x6b, 0x6f, 0x6b,
0xe3, 0x42, 0x61, 0xed, 0x79, 0x61, 0x42, 0xe3, 0x64, 0x65, 0x72, 0x61,
0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x75, 0x61, 0x72, 0x61, 0x42, 0x61,
0x72, 0x61, 0x62, 0x61, 0x64, 0x75, 0x42, 0x65, 0x6c, 0x1ebd, 0x42, 0x65,
0x72, 0x69, 0x73, 0x65, 0x49, 0x77, 0x69, 0x6b, 0x75, 0xed, 0x20, 0x4d,
0x75, 0x72, 0x75, 0x74, 0x129, 0x67, 0x61, 0x42, 0x75, 0x77, 0x61, 0x20,
0x56, 0x69, 0x73, 0x74, 0x61, 0x42, 0x75, 0x67, 0x75, 0x74, 0xe1, 0x42,
0x75, 0x77, 0x69, 0x73, 0xe9, 0x4b, 0x1ebd, 0x62, 0x69, 0x72, 0x69, 0x79,
0x69, 0x20, 0x4b, 0x75, 0x61, 0x72, 0x61, 0x4b, 0xe3, 0x70, 0x75, 0x20,
0x57, 0x61, 0x73, 0x75, 0x4b, 0xe3, 0x6b, 0x169, 0x4b, 0x61, 0x79, 0x65,
0x6e, 0x61, 0x4b, 0x61, 0x69, 0x6d, 0xe3, 0x53, 0x69, 0x76, 0x61, 0x76,
0x61, 0x4b, 0x75, 0x70, 0xe9, 0x20, 0x52, 0x69, 0x6b, 0x61, 0x4b, 0x65,
0x72, 0x65, 0x73, 0x74, 0x169, 0x44, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x6b, 0x61, 0x73, 0xe3, 0x76, 0x44, 0x61, 0x75, 0x73, 0x169, 0x44,
0x61, 0x75, 0x73, 0x169, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x6b, 0x69, 0x44,
0x1ebd, 0x77, 0x65, 0x72, 0x44, 0x75, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x61,
0x45, 0x64, 0x69, 0x6d, 0x169, 0x74, 0x169, 0x45, 0x75, 0x20, 0x53, 0x61,
0x77, 0x61, 0x64, 0x75, 0x46, 0x75, 0x74, 0x69, 0x20, 0x4e, 0x65, 0x75,
0x73, 0x169, 0x46, 0x75, 0x74, 0x61, 0x72, 0x65, 0x73, 0x61, 0x59, 0x72,
0x75, 0x73, 0xe3, 0x67, 0x75, 0x73, 0x75, 0x2d, 0x61, 0x74, 0xe3, 0x20,
0x4b, 0x75, 0x61, 0x72, 0x61, 0x49, 0x70, 0x65, 0x6b, 0x61, 0x77, 0x61,
0x73, 0x75, 0x20, 0x4b, 0x75, 0x61, 0x72, 0x61, 0x57, 0x61, 0x73, 0xfa,
0x20, 0x54, 0x75, 0x72, 0x69, 0x6b, 0x75, 0x47, 0x61, 0x72, 0x61, 0x6e,
0x61, 0x64, 0x61, 0x47, 0x75, 0x61, 0x64, 0x61, 0x72, 0x75, 0x70, 0x69,
0x47, 0x75, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x72, 0x61, 0x47, 0x75, 0x61,
0x79, 0x61, 0x6b, 0x69, 0x75, 0x47, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x48,
0x65, 0x6d, 0x75, 0x73, 0x69, 0x72, 0x75, 0x128, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x70, 0x75, 0x72, 0x69, 0x4b, 0x6e, 0x6f, 0x78, 0x2c, 0x20, 0x128,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x1ebd, 0x67, 0x75, 0x2c,
0x20, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50, 0x69, 0x74, 0x69, 0x72,
0x69, 0x73, 0x69, 0x62, 0x75, 0x67, 0x69, 0x2c, 0x20, 0x128, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x54, 0x61, 0x77, 0x61,
0x2d, 0x77, 0x61, 0x73, 0xfa, 0x2c, 0x20, 0x128, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x57, 0x65, 0x77, 0x61, 0x69, 0x2c, 0x20, 0x128, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x57, 0x129, 0x73, 0x65, 0x6e, 0x65, 0x2d, 0x69, 0x74, 0x61,
0x2c, 0x20, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0xe3,
0x6d, 0x61, 0x6b, 0x69, 0x2c, 0x20, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x49, 0x6e, 0x75, 0x77, 0x69, 0x6b, 0x69, 0x49, 0x6b, 0x61, 0x72, 0x75,
0x69, 0x74, 0x69, 0x59, 0x61, 0x6d, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x75,
0x6e, 0x65, 0x61, 0x75, 0x4c, 0x75, 0x77, 0x69, 0x73, 0x69, 0x76, 0x69,
0x72, 0x69, 0x4d, 0x169, 0x74, 0x69, 0x73, 0x65, 0x72, 0x75, 0x2c, 0x20,
0x4b, 0x1ebd, 0x74, 0x75, 0x6b, 0x69, 0x4b, 0x61, 0x72, 0x61, 0x72, 0x1ebd,
0x64, 0x69, 0x6b, 0x69, 0x52, 0x69, 0x6d, 0x61, 0x50, 0x69, 0x72, 0x129,
0x73, 0x69, 0x70, 0x69, 0x20, 0x51, 0x75, 0x61, 0x72, 0x74, 0x65, 0x72,
0x20, 0x55, 0x69, 0x72, 0x70, 0x65, 0x77, 0x61, 0x72, 0x61, 0x4d, 0x61,
0x73, 0x65, 0x79, 0xf3, 0x4d, 0x61, 0x6e, 0xe1, 0x67, 0x75, 0x77, 0x61,
0x4d, 0x61, 0x72, 0x69, 0x67, 0x75, 0x74, 0x69, 0x4d, 0x61, 0x74, 0x69,
0x6e, 0x69, 0x6b, 0x61, 0x4d, 0x6f, 0x72, 0x6f, 0x69, 0x74, 0x61, 0x79,
0x75, 0x6b, 0xe1, 0x4d, 0x61, 0x73, 0x61, 0x74, 0x61, 0x72, 0xe3, 0x4d,
0x65, 0x74, 0x69, 0x72, 0x61, 0x6b, 0x61, 0x74, 0x69, 0x72, 0x61, 0x1ebc,
0x6d, 0xe3, 0x20, 0x74, 0x1ef9, 0x20, 0x4d, 0x1ebd, 0x73, 0x69, 0x6b, 0x75,
0x4d, 0x69, 0x6b, 0x69, 0x72, 0x169, 0x4d, 0x169, 0x6b, 0x69, 0x74, 0x169,
0x4d, 0xf5, 0x74, 0x65, 0x72, 0x65, 0x69, 0x4d, 0x169, 0x74, 0x69, 0x77,
0x69, 0x64, 0x65, 0x77, 0x75, 0x4d, 0xf5, 0x74, 0x69, 0x73, 0x65, 0x72,
0x61, 0x74, 0x69, 0x59, 0x75, 0x72, 0x6b, 0x69, 0x20, 0x50, 0x69, 0x73,
0x61, 0x73, 0xfa, 0x52, 0xe9, 0x72, 0x61, 0x46, 0x65, 0x6e, 0xe3, 0x64,
0x75, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x79, 0xe3, 0x42, 0x65, 0x75, 0x72,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4e, 0x75, 0x74,
0x69, 0x20, 0x73, 0x75, 0xed, 0x53, 0x1ebd, 0x74, 0x65, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4e, 0x75, 0x74, 0x69, 0x20, 0x73,
0x75, 0xed, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x53, 0x61, 0x72,
0x1ebd, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4e, 0x75,
0x74, 0x69, 0x20, 0x73, 0x75, 0xed, 0x4e, 0x75, 0x6b, 0x69, 0x4f, 0x79,
0x69, 0x6e, 0x61, 0x67, 0x61, 0x50, 0x61, 0x6e, 0x61, 0x6d, 0xe3, 0x50,
0x75, 0x77, 0x65, 0x6e, 0x69, 0x6b, 0x69, 0x73, 0x69, 0x49, 0x67, 0x61,
0x72, 0x61, 0x70, 0x61, 0x77, 0x61, 0x20, 0x50, 0x69, 0x72, 0x129, 0x73,
0x69, 0x70, 0x69, 0x45, 0x73, 0x69, 0x70, 0xe3, 0x79, 0x61, 0x20, 0x49,
0x67, 0x61, 0x72, 0x61, 0x70, 0x61, 0x77, 0x61, 0x49, 0x67, 0x61, 0x72,
0x61, 0x70, 0x61, 0x77, 0x61, 0x20, 0x4b, 0x75, 0x78, 0x69, 0x6d, 0x61,
0x77, 0x61, 0x72, 0x61, 0x49, 0x67, 0x61, 0x72, 0x61, 0x70, 0x61, 0x77,
0x61, 0x20, 0x52, 0x69, 0x6b, 0x75, 0x50, 0x169, 0x74, 0x61, 0x20, 0x41,
0x72, 0x65, 0x6e, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x52, 0xe3, 0x6b, 0x129,
0x20, 0x55, 0x69, 0x6b, 0x65, 0x73, 0x61, 0x77, 0x61, 0x52, 0x65, 0x6a,
0x129, 0x6e, 0x1ef9, 0x52, 0x65, 0x73, 0x6f, 0x72, 0x75, 0x74, 0x69, 0x50,
0x61, 0x72, 0x61, 0x6e, 0xe3, 0x20, 0x4d, 0x75, 0x72, 0x75, 0x74, 0x129,
0x67, 0x61, 0x53, 0xe3, 0x74, 0x61, 0x72, 0x1ebd, 0x53, 0xe3, 0x74, 0x69,
0x79, 0x61, 0x67, 0x75, 0x53, 0xe3, 0x74, 0x75, 0x20, 0x44, 0x75, 0x6d,
0x129, 0x67, 0x75, 0x53, 0xe3, 0x20, 0x50, 0x61, 0x75, 0x72, 0x75, 0x49,
0x74, 0x75, 0x6b, 0x75, 0x74, 0x75, 0x6d, 0x69, 0x74, 0x69, 0x53, 0x69,
0x74, 0x69, 0x6b, 0x61, 0x53, 0xe3, 0x20, 0x42, 0x61, 0x74, 0x75, 0x6c,
0x75, 0x6d, 0x65, 0x75, 0x53, 0xe3, 0x74, 0x69, 0x20, 0x4a, 0x75, 0x6e,
0x69, 0x53, 0xe3, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x75, 0x77, 0xe3,
0x75, 0x53, 0xe3, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x79, 0x61,
0x53, 0xe3, 0x74, 0x69, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x53, 0xe3, 0x20,
0x57, 0x69, 0x73, 0x1ebd, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x6e, 0xe3,
0x20, 0x50, 0x69, 0x72, 0xe3, 0x74, 0xe3, 0x54, 0x65, 0x67, 0x75, 0x73,
0x69, 0x67, 0x61, 0x75, 0x70, 0x61, 0x54, 0x69, 0x78, 0x75, 0x72, 0x69,
0x54, 0x69, 0x6a, 0x75, 0x76, 0x1ef9, 0x6e, 0x1ef9, 0x54, 0x75, 0x72, 0x169,
0x74, 0x75, 0x54, 0x75, 0x74, 0x75, 0x72, 0x61, 0x56, 0xe3, 0x6b, 0x75,
0x77, 0x65, 0x72, 0x69, 0x4b, 0x61, 0x77, 0x61, 0x72, 0x75, 0x20, 0x4d,
0x75, 0x72, 0x75, 0x74, 0x129, 0x67, 0x61, 0x57, 0x69, 0x6e, 0x129, 0x70,
0x65, 0x67, 0x69, 0x4b, 0x61, 0x73, 0x65, 0x69, 0x44, 0x61, 0x77, 0x69,
0x4d, 0x61, 0x6b, 0x69, 0x6b, 0x75, 0x61, 0x72, 0x69, 0x4d, 0x61, 0x75,
0x73, 0x169, 0x4d, 0x61, 0x6b, 0x69, 0x4d, 0x75, 0x64, 0x75, 0x50, 0x61,
0x75, 0x6d, 0x65, 0x72, 0x65, 0x57, 0x6f, 0x73, 0x69, 0x74, 0x75, 0x6b,
0x69, 0x4c, 0x6f, 0x6e, 0x67, 0x79, 0x65, 0x61, 0x72, 0x20, 0x54, 0x61,
0x77, 0x61, 0x2d, 0x77, 0x61, 0x73, 0xfa, 0x41, 0x64, 0x1ebd, 0x41, 0x75,
0x6d, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x69, 0x74, 0x75, 0x62, 0x69, 0x41,
0x73, 0x69, 0x67, 0x61, 0x62, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x6e,
0x61, 0x42, 0x61, 0x72, 0x1ebd, 0x69, 0x4d, 0xe1, 0x67, 0x2d, 0x6b, 0xf3,
0x6b, 0x69, 0x42, 0x69, 0x73, 0x69, 0x6b, 0x65, 0x6b, 0x69, 0x42, 0x75,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x58, 0x69, 0x74, 0x61, 0x4b, 0x75, 0x72,
0x169, 0x62, 0x75, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x44,
0x69, 0x72, 0x69, 0x44, 0x75, 0x78, 0xe3, 0x62, 0x69, 0x46, 0x61, 0x6d,
0x61, 0x67, 0x75, 0x69, 0x74, 0x61, 0x48, 0x65, 0x62, 0x75, 0x72, 0x169,
0x48, 0x169, 0x67, 0x20, 0x4b, 0x169, 0x67, 0x48, 0x6f, 0x77, 0x69, 0x64,
0x69, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x4a,
0x61, 0x6b, 0x61, 0x74, 0x61, 0x59, 0x61, 0x79, 0x61, 0x70, 0x75, 0x72,
0x61, 0x59, 0x65, 0x72, 0x75, 0x73, 0x61, 0x72, 0x1ebd, 0x69, 0x4b, 0xe3,
0x78, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x78, 0x69,
0x4b, 0x61, 0x74, 0x69, 0x6d, 0xe3, 0x64, 0x75, 0x4b, 0xe3, 0x64, 0x69,
0x67, 0x61, 0x4b, 0x61, 0x75, 0x6b, 0x75, 0x74, 0xe1, 0x4b, 0x61, 0x72,
0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x4b, 0x75, 0x61, 0x72, 0x61, 0x20, 0x52, 0x169, 0x70, 0x75, 0x72,
0x75, 0x4b, 0x75, 0x78, 0x129, 0x67, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64,
0xe3, 0x4d, 0x61, 0x73, 0x69, 0x6b, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x73,
0x69, 0x6e, 0x65, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x50, 0x69,
0x73, 0x61, 0x73, 0xfa, 0x53, 0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x69,
0x6b, 0x69, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x4f, 0x6d, 0x69,
0x73, 0x69, 0x6b, 0x69, 0x52, 0x61, 0x75, 0x72, 0x65, 0x50, 0x69, 0x6e,
0xf5, 0x20, 0x50, 0x1ebd, 0x69, 0x50, 0x169, 0x74, 0x69, 0x61, 0x6e, 0x61,
0x6b, 0x61, 0x50, 0x69, 0x169, 0x67, 0x69, 0xe3, 0x67, 0x69, 0x4b, 0x61,
0x74, 0x61, 0x72, 0x69, 0x4b, 0x6f, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x61,
0x69, 0x4b, 0x69, 0x73, 0x69, 0x72, 0x6f, 0x64, 0x61, 0x53, 0x61, 0x6b,
0x61, 0x72, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x6b, 0xe3, 0x64,
0x61, 0x58, 0xe3, 0x67, 0x61, 0x69, 0x58, 0x65, 0x72, 0x65, 0x64, 0x69,
0x6e, 0x65, 0x6b, 0x6f, 0x72, 0x69, 0x6d, 0x69, 0x7a, 0x69, 0x6b, 0x69,
0x54, 0x61, 0x73, 0x69, 0x6b, 0x1ebd, 0x74, 0x69, 0x54, 0x65, 0x79, 0x65,
0x72, 0xe3, 0x54, 0x129, 0x70, 0x75, 0x54, 0x6f, 0x6b, 0x69, 0x79, 0x75,
0x54, 0x6f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x55, 0x72, 0xe3, 0x20,
0x42, 0x61, 0x74, 0x75, 0x72, 0x75, 0x55, 0x72, 0x169, 0x6b, 0x69, 0x55,
0x73, 0x69, 0x74, 0x69, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x56, 0x69, 0x1ebd,
0x74, 0x69, 0x61, 0x6e, 0x65, 0x57, 0x61, 0x72, 0x61, 0x64, 0x69, 0x77,
0x6f, 0x73, 0x69, 0x74, 0x75, 0x6b, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x69, 0x73, 0x69, 0x6b, 0x69, 0x52, 0xe3, 0x67, 0x169, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x129, 0x62, 0x75, 0x67, 0x75, 0x59, 0x65, 0x72, 0x65,
0x77, 0xe3, 0x41, 0x73, 0x6f, 0x72, 0x69, 0x2d, 0x69, 0x74, 0x61, 0x42,
0x65, 0x6d, 0x75, 0x64, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x79,
0x61, 0x2d, 0x69, 0x74, 0x61, 0x4b, 0x61, 0x62, 0x75, 0x20, 0x53, 0x75,
0x69, 0x6b, 0x69, 0x72, 0x69, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61,
0x2d, 0x69, 0x74, 0x61, 0x20, 0x46, 0x61, 0x72, 0x75, 0x77, 0x65, 0x52,
0x65, 0x69, 0x6b, 0x69, 0x79, 0x61, 0x77, 0x69, 0x6b, 0x69, 0x47, 0x65,
0x75, 0x67, 0x69, 0x79, 0x61, 0x20, 0x53, 0x75, 0x20, 0x73, 0x75, 0xed,
0x53, 0xe3, 0x74, 0x61, 0x20, 0x45, 0x72, 0x1ebd, 0x6e, 0x61, 0x49, 0x73,
0x69, 0x74, 0xe3, 0x72, 0x65, 0x69, 0x41, 0x64, 0x65, 0x72, 0x61, 0x69,
0x64, 0x69, 0x42, 0x69, 0x72, 0x69, 0x62, 0x61, 0x6e, 0x69, 0x49, 0x77,
0x69, 0x74, 0x65, 0x72, 0x61, 0x20, 0x4d, 0x75, 0x70, 0x65, 0x6e, 0x61,
0xed, 0x72, 0x61, 0x44, 0x61, 0x72, 0x69, 0x77, 0x129, 0x45, 0x75, 0x6b,
0x61, 0x72, 0x61, 0x48, 0x6f, 0x62, 0x61, 0x74, 0x69, 0x52, 0x129, 0x64,
0x65, 0x6d, 0xe3, 0x4c, 0x75, 0x72, 0x75, 0x64, 0x69, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x4d, 0x65, 0x75, 0x62, 0xfa, 0x6e, 0x69, 0x50, 0x65, 0x72,
0x69, 0x74, 0x69, 0x53, 0x69, 0x64, 0x69, 0x6e, 0x65, 0x69, 0x41, 0x6d,
0x69, 0x74, 0x65, 0x72, 0x69, 0x64, 0xe3, 0xc3, 0x64, 0x75, 0x72, 0x61,
0x41, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6b, 0xe3, 0x41, 0x74, 0x65,
0x6e, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x42, 0x65, 0x75, 0x67, 0x61, 0x72,
0x61, 0x64, 0x75, 0x42, 0x61, 0x72, 0x61, 0x74, 0x69, 0x73, 0x69, 0x6c,
0x61, 0x77, 0x61, 0x42, 0x75, 0x72, 0x75, 0x78, 0x65, 0x72, 0x61, 0x42,
0x75, 0x6b, 0x61, 0x72, 0x65, 0x74, 0x69, 0x42, 0x75, 0x64, 0x61, 0x70,
0x65, 0x74, 0x69, 0x42, 0x75, 0x73, 0x129, 0x67, 0x65, 0x6e, 0x69, 0x58,
0x69, 0x73, 0x69, 0x6e, 0x61, 0x75, 0x4b, 0x6f, 0x70, 0x1ebd, 0x79, 0x61,
0x67, 0x69, 0x44, 0x75, 0x62, 0x69, 0x72, 0x129, 0x47, 0x69, 0x62, 0x61,
0x72, 0x61, 0x75, 0x74, 0xe1, 0x47, 0x75, 0x77, 0x1ebd, 0x6e, 0x69, 0x73,
0x65, 0x69, 0x48, 0x65, 0x75, 0x73, 0x129, 0x6b, 0x65, 0x4d, 0xe3, 0x20,
0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x4a, 0x65, 0x72, 0x73, 0x65,
0x69, 0x4b, 0x61, 0x72, 0x69, 0x6e, 0x129, 0x67, 0x61, 0x72, 0x61, 0x64,
0x75, 0x4b, 0x69, 0x72, 0x6f, 0x77, 0x69, 0x4b, 0x69, 0x79, 0x65, 0x77,
0x65, 0x52, 0x69, 0x75, 0x62, 0x69, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x52,
0x75, 0x78, 0x1ebd, 0x62, 0x75, 0x67, 0x75, 0x4d, 0x61, 0x72, 0x69, 0x1ebd,
0x79, 0xe3, 0x4d, 0x129, 0x73, 0x69, 0x6b, 0x69, 0x55, 0x73, 0x69, 0x72,
0x75, 0x50, 0x75, 0x64, 0x69, 0x67, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x53,
0xe3, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x75, 0x53, 0x61, 0x72, 0x61,
0x79, 0x65, 0x77, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x74, 0x75, 0x77, 0x6f,
0x53, 0x129, 0x77, 0x65, 0x72, 0x75, 0x70, 0x75, 0x49, 0x73, 0x69, 0x6b,
0x75, 0x70, 0x69, 0x79, 0x65, 0x53, 0x75, 0x66, 0x69, 0x79, 0x61, 0x55,
0x6c, 0x69, 0x61, 0x6e, 0x75, 0x77, 0x69, 0x6b, 0x69, 0x56, 0x61, 0x64,
0x75, 0x73, 0x69, 0x57, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x57,
0x69, 0x79, 0x65, 0x6e, 0x61, 0x56, 0x69, 0x77, 0x69, 0x75, 0x2d, 0x69,
0x74, 0x61, 0x57, 0x75, 0x67, 0x6f, 0x67, 0x61, 0x72, 0x61, 0x64, 0x75,
0x57, 0x61, 0x72, 0x69, 0x73, 0xf3, 0x77, 0x69, 0x79, 0x61, 0x53, 0x61,
0x67, 0x61, 0x72, 0x65, 0x62, 0x69, 0x5a, 0x75, 0x72, 0x69, 0x6b, 0x69,
0xc3, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x75, 0x58, 0x61,
0x67, 0x75, 0x2d, 0x69, 0x74, 0x61, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69,
0x6d, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x4b, 0x75, 0x6b, 0x75, 0x2d, 0x69,
0x74, 0x61, 0x4b, 0x75, 0x6d, 0x75, 0x72, 0x65, 0x2d, 0x69, 0x74, 0x61,
0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c, 0x1ebd, 0x4d, 0x61, 0xe9, 0x4d, 0x61,
0x75, 0x64, 0x69, 0x77, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x73, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x79, 0x75, 0x74, 0x69,
0x59, 0x75, 0x6d, 0x75, 0x61, 0x74, 0x69, 0x72, 0x69, 0x73, 0x61, 0x77,
0x61, 0x41, 0x70, 0x69, 0x79, 0x61, 0xd3, 0x6b, 0x69, 0x72, 0xe3, 0x64,
0x69, 0x42, 0x75, 0x67, 0x61, 0x129, 0x77, 0x69, 0x72, 0x69, 0x58, 0x61,
0x74, 0x69, 0x6e, 0xe3, 0x50, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x77, 0x61,
0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x45, 0x66, 0x61, 0x74,
0xe9, 0x46, 0x61, 0x6b, 0x61, 0x75, 0x66, 0x75, 0x47, 0x61, 0x72, 0x61,
0x70, 0x61, 0x67, 0x75, 0x2d, 0x69, 0x74, 0x61, 0x47, 0xe3, 0x62, 0x69,
0x65, 0x72, 0x65, 0x47, 0x75, 0x61, 0x64, 0x61, 0x75, 0x6b, 0x61, 0x6e,
0x61, 0x75, 0x47, 0x75, 0x77, 0xe3, 0x48, 0x75, 0x6e, 0x75, 0x72, 0x75,
0x72, 0x75, 0x1ebc, 0x64, 0x65, 0x62, 0x75, 0x72, 0x69, 0x4b, 0x75, 0x73,
0x69, 0x72, 0x61, 0x69, 0x4b, 0x75, 0x61, 0x79, 0x61, 0x72, 0x1ebd, 0x69,
0x4d, 0x61, 0x79, 0x75, 0x72, 0x75, 0x4d, 0x61, 0x6b, 0x65, 0x73, 0x61,
0x2d, 0x69, 0x74, 0x61, 0x4d, 0x69, 0x64, 0x69, 0x77, 0x65, 0x69, 0x4e,
0x69, 0x77, 0x65, 0x4e, 0x75, 0x72, 0x75, 0x66, 0x75, 0x6b, 0x69, 0x4e,
0x75, 0x6d, 0x65, 0x79, 0x61, 0x50, 0x61, 0x67, 0x75, 0x20, 0x50, 0x61,
0x67, 0x75, 0x50, 0x61, 0x72, 0x61, 0x75, 0x49, 0x67, 0x61, 0x72, 0x61,
0x70, 0x61, 0x77, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x79,
0x52, 0x61, 0x72, 0x75, 0x74, 0x169, 0x67, 0x61, 0x53, 0x61, 0x69, 0x70,
0xe3, 0x54, 0x169, 0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x57, 0x61, 0x72,
0x69, 0x2d, 0x69, 0x74, 0x61, 0x53, 0xe3, 0x20, 0x4b, 0x69, 0x72, 0x69,
0x74, 0x75, 0x77, 0x61, 0x6e, 0x41, 0x6e, 0x74, 0x69, 0x69, 0x67, 0x75,
0x61, 0x41, 0x72, 0x75, 0x75, 0x62, 0x61, 0x42, 0x61, 0x72, 0x62, 0x61,
0x61, 0x64, 0x6f, 0x73, 0x44, 0x6f, 0x6d, 0x69, 0x69, 0x6e, 0x69, 0x6b,
0x61, 0x47, 0x72, 0x65, 0x6e, 0x61, 0x61, 0x64, 0x61, 0x47, 0x75, 0x61,
0x74, 0x65, 0x6d, 0x61, 0x61, 0x6c, 0x61, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x20, 0x53, 0x74, 0x65, 0x65, 0x64, 0x42, 0x65, 0x75, 0x6c,
0x61, 0x68, 0x2c, 0x20, 0x4e, 0x75, 0x75, 0x72, 0x64, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0x4e, 0x75, 0x75, 0x72, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e,
0x75, 0x75, 0x72, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x41,
0x73, 0x63, 0x68, 0x67, 0x61, 0x62, 0x61, 0x74, 0x48, 0x6f, 0x20, 0x43,
0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x20, 0x53, 0x74, 0x65, 0x65,
0x64, 0x4b, 0x75, 0x74, 0x73, 0x63, 0x68, 0x69, 0x6e, 0x67, 0x4e, 0x6f,
0x77, 0x6f, 0x6b, 0x75, 0x73, 0x6e, 0x65, 0x74, 0x73, 0x6b, 0x4b, 0x61,
0x74, 0x61, 0x61, 0x72, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x75,
0x72, 0x54, 0x61, 0x69, 0x70, 0x65, 0x65, 0x55, 0x72, 0x75, 0x6d, 0x74,
0x73, 0x63, 0x68, 0x69, 0x57, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e,
0x65, 0x52, 0x61, 0x6e, 0x67, 0x75, 0x75, 0x6e, 0x41, 0x74, 0x73, 0x6f,
0x6f, 0x72, 0x65, 0x6e, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x75, 0x64, 0x61,
0x4b, 0x61, 0x6e, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x4b, 0x61, 0x70, 0x77,
0x65, 0x72, 0x64, 0x65, 0x6e, 0x46, 0xe4, 0x72, 0xf6, 0x65, 0x72, 0x6e,
0x53, 0xfc, 0xfc, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e,
0x41, 0x74, 0x68, 0x65, 0x65, 0x6e, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68,
0x75, 0x75, 0x77, 0x65, 0x6e, 0x47, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74,
0x61, 0x61, 0x72, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x4d, 0x61, 0x6e,
0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x6f, 0x72, 0x69, 0x67, 0x52,
0x69, 0x69, 0x67, 0x61, 0x57, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x74, 0xe4, 0xe4, 0x64, 0x57, 0x69, 0x69, 0x6e, 0x4a, 0x75, 0x6c,
0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20,
0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x4d, 0x61, 0x6c, 0x65, 0x64,
0x69, 0x69, 0x77, 0x65, 0x6e, 0x50, 0x75, 0x61, 0x73, 0x6b, 0x20, 0x45,
0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0xc1, 0x62, 0x65, 0x64, 0x6a,
0xe1, 0x6e, 0x41, 0x6b, 0x6b, 0x72, 0xe1, 0x41, 0x64, 0x64, 0x69, 0x73,
0x20, 0x41, 0x62, 0x61, 0x62, 0xe1, 0x41, 0x73, 0x6d, 0x61, 0x72, 0xe1,
0x42, 0x61, 0x6d, 0x61, 0x6b, 0xf3, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x72, 0x61, 0x79, 0x42, 0x72, 0x61, 0x7a, 0x7a, 0x61, 0x77, 0x69, 0x6c,
0x61, 0x79, 0x42, 0x75, 0x6a, 0x6f, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x4b,
0xe1, 0x65, 0x72, 0xf3, 0x4b, 0x61, 0x73, 0x61, 0x62, 0x6c, 0x61, 0x6e,
0x6b, 0xe1, 0x4b, 0x79, 0x75, 0x74, 0xe1, 0x44, 0xe1, 0x72, 0x20, 0x65,
0x73, 0x20, 0x73, 0x61, 0x6c, 0xe1, 0x6d, 0x44, 0x6a, 0x65, 0x62, 0xf3,
0x74, 0x69, 0x44, 0x6f, 0x75, 0xe1, 0x6c, 0xe1, 0x45, 0x6c, 0x20, 0xc1,
0x69, 0x75, 0x6e, 0x50, 0x65, 0x72, 0x69, 0x74, 0x68, 0xe1, 0xf3, 0x6e,
0x47, 0x61, 0x62, 0x6f, 0x72, 0x6f, 0x6e, 0xe9, 0x48, 0x61, 0x72, 0xe1,
0x72, 0xe9, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x61,
0x72, 0x67, 0x4a, 0x75, 0x62, 0xe1, 0x4b, 0x65, 0x6e, 0x73, 0x68, 0xe1,
0x73, 0x61, 0x4c, 0x61, 0x67, 0xf3, 0x73, 0x4c, 0x69, 0x62, 0x72, 0x65,
0x77, 0x69, 0x6c, 0x61, 0x79, 0x4c, 0xf3, 0x6d, 0xe9, 0x4c, 0x75, 0x61,
0x6e, 0x64, 0xe1, 0x4c, 0x75, 0x73, 0xe1, 0x6b, 0xe1, 0x4d, 0x61, 0x6c,
0xe1, 0x62, 0xf3, 0x4d, 0x61, 0x70, 0x75, 0x74, 0xf3, 0x4d, 0x61, 0x73,
0xe9, 0x72, 0xf3, 0x4d, 0x62, 0x61, 0x62, 0x61, 0x6e, 0xe9, 0x4d, 0xf3,
0x67, 0xe1, 0x64, 0xe9, 0x73, 0x68, 0xf3, 0x4d, 0x6f, 0x6e, 0x72, 0xf3,
0x77, 0x69, 0xe1, 0x4e, 0xe9, 0x72, 0x75, 0x62, 0x69, 0x4e, 0x64, 0x6a,
0x61, 0x6d, 0x65, 0x6e, 0xe1, 0x4e, 0x69, 0x61, 0x6d, 0xe9, 0x79, 0x50,
0x6f, 0x72, 0x74, 0xf3, 0x2d, 0x4e, 0xf3, 0x77, 0xf3, 0x53, 0xe1, 0xf3,
0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x54, 0x72, 0x69, 0x70, 0xf3, 0x6c, 0x69,
0x57, 0x69, 0x6e, 0x64, 0x6f, 0x65, 0x6b, 0x41, 0x6e, 0x63, 0x68, 0x6f,
0x72, 0x61, 0x67, 0xe9, 0x41, 0x6e, 0x67, 0x75, 0x69, 0x6c, 0x6c, 0xe1,
0x41, 0x6e, 0x74, 0x69, 0x67, 0x75, 0xe1, 0x43, 0x68, 0x61, 0x74, 0x61,
0x6d, 0x61, 0x72, 0x6b, 0xe1, 0x4b, 0x6f, 0x72, 0x74, 0x6f, 0x62, 0x61,
0x4a, 0x6f, 0x6a, 0x75, 0x69, 0x4d, 0x65, 0x6e, 0x64, 0x6f, 0x7a, 0xe1,
0x55, 0x73, 0x68, 0x77, 0xe1, 0x69, 0xe1, 0x41, 0x72, 0x75, 0x62, 0xe1,
0x41, 0x73, 0x75, 0x6e, 0x63, 0x68, 0x69, 0xf3, 0x6e, 0x41, 0x74, 0x69,
0x6b, 0xf3, 0x6b, 0x61, 0x6e, 0x42, 0x61, 0x68, 0xed, 0x61, 0x20, 0x64,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0xe1, 0x73, 0x42, 0x6c,
0x61, 0x6e, 0x63, 0x68, 0x2d, 0x53, 0x61, 0x62, 0x6c, 0xf3, 0x6e, 0x42,
0xf3, 0xe1, 0x20, 0x57, 0x69, 0x73, 0x74, 0xe1, 0x4b, 0x61, 0x6d, 0x62,
0x72, 0x65, 0x6a, 0x20, 0x42, 0x61, 0x79, 0x43, 0x68, 0x61, 0x6d, 0x70,
0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x65, 0x43, 0x68, 0x61, 0x6e,
0x63, 0x68, 0x75, 0x6e, 0x43, 0x68, 0x61, 0x72, 0x61, 0x6b, 0xe1, 0x73,
0x43, 0x68, 0x61, 0x79, 0x65, 0x6e, 0x6e, 0x53, 0x68, 0x65, 0x6b, 0xe1,
0x67, 0xf3, 0x43, 0x68, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0xe1, 0x43,
0x68, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x4a, 0x75, 0xe1, 0x72, 0x65,
0x7a, 0x4b, 0xf3, 0x73, 0x74, 0xe1, 0x20, 0x52, 0x69, 0x6b, 0xe1, 0x4b,
0x72, 0x65, 0x73, 0x74, 0xf3, 0x6e, 0x4b, 0x77, 0x61, 0x69, 0x62, 0xe1,
0x43, 0x68, 0x6f, 0x72, 0xe1, 0x63, 0x68, 0xe1, 0xf3, 0x44, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68, 0x61, 0x77, 0x6e, 0x44, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x20, 0x4b, 0x72, 0xe9, 0x6b, 0x44, 0x65, 0x6e, 0x77,
0xe9, 0x72, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0xe1, 0x49, 0x72,
0x75, 0x6e, 0xe9, 0x70, 0xe9, 0x45, 0x6c, 0x20, 0x53, 0x61, 0x6c, 0x77,
0x61, 0x64, 0xf3, 0x72, 0x50, 0x6f, 0x72, 0x74, 0x61, 0x6c, 0x65, 0x7a,
0xe1, 0x47, 0x6c, 0x61, 0x79, 0x73, 0x20, 0x42, 0x61, 0x79, 0x47, 0x72,
0x65, 0x6e, 0x61, 0x64, 0xe1, 0x47, 0x77, 0xe1, 0x64, 0x65, 0x6c, 0xf3,
0x70, 0x47, 0x77, 0xe1, 0x74, 0xe9, 0x6d, 0xe1, 0x6c, 0xe1, 0x47, 0x77,
0xe1, 0x79, 0x61, 0x6b, 0x69, 0x6c, 0x47, 0x75, 0x79, 0xe1, 0x6e, 0x61,
0x48, 0x61, 0x77, 0xe1, 0x6e, 0xe1, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e,
0xe1, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4b, 0x6e, 0x6f, 0x6b, 0x73, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x4d, 0x61, 0x72, 0x65,
0x6e, 0x67, 0xf3, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x50,
0x65, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x53,
0x69, 0x74, 0x68, 0x69, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e,
0xe1, 0x57, 0x65, 0x77, 0x61, 0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0xe1, 0x6e, 0xe1, 0x57, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x6e, 0x65, 0x73,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x57, 0x69, 0x6e,
0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e,
0xe1, 0x49, 0x6b, 0xe1, 0x6c, 0x77, 0x69, 0x74, 0x4a, 0x61, 0x6d, 0xe1,
0x65, 0x6b, 0xe1, 0x4a, 0x75, 0x6e, 0x69, 0x75, 0x4c, 0x75, 0x69, 0x73,
0x77, 0x69, 0x6c, 0x6c, 0x61, 0x79, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0xf3, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75,
0x6b, 0x69, 0x4c, 0xe1, 0x20, 0x50, 0x61, 0x7a, 0x4c, 0x69, 0x6d, 0xe1,
0x4c, 0xe1, 0x73, 0x20, 0x45, 0x6e, 0x6a, 0x65, 0x6c, 0x65, 0x73, 0x4d,
0x61, 0x63, 0x68, 0x69, 0xf3, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0xf3,
0x20, 0x53, 0x68, 0x61, 0x68, 0x72, 0x4d, 0x65, 0x6b, 0x77, 0x65, 0x6c,
0x6f, 0x6e, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x72, 0x61, 0x79, 0x4d,
0x6f, 0x6e, 0x74, 0x65, 0x77, 0x69, 0x64, 0x69, 0xf3, 0x4e, 0x79, 0x75,
0x20, 0x59, 0xe1, 0x72, 0x6b, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0xe1,
0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4e, 0xe1, 0x72, 0x74,
0x20, 0x44, 0x61, 0x6b, 0xf3, 0x74, 0xe1, 0x53, 0x61, 0x6e, 0x74, 0x68,
0x61, 0x72, 0x2c, 0x20, 0x4e, 0xe1, 0x72, 0x74, 0x20, 0x44, 0x61, 0x6b,
0xf3, 0x74, 0xe1, 0x4e, 0x79, 0x75, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d,
0x2c, 0x20, 0x4e, 0xe1, 0x72, 0x74, 0x20, 0x44, 0x61, 0x6b, 0xf3, 0x74,
0xe1, 0x50, 0x61, 0x6e, 0xe1, 0x6d, 0xe1, 0x50, 0x61, 0x72, 0x61, 0x6d,
0x61, 0x72, 0x69, 0x62, 0xf3, 0x50, 0xf3, 0x65, 0x6e, 0x69, 0x6b, 0x73,
0x50, 0xf3, 0x72, 0x74, 0x2d, 0x61, 0x75, 0x2d, 0x50, 0x72, 0x65, 0x6e,
0x73, 0x50, 0xf3, 0x72, 0x74, 0x20, 0x45, 0x73, 0x70, 0x69, 0x6e, 0x50,
0x6f, 0x72, 0x74, 0xf3, 0x20, 0x77, 0x65, 0x6c, 0x68, 0xf3, 0x50, 0x75,
0x65, 0x72, 0x74, 0xf3, 0x20, 0x52, 0x69, 0x6b, 0xf3, 0x52, 0x65, 0x63,
0x68, 0x69, 0x70, 0xe9, 0x52, 0x65, 0x73, 0xf3, 0x6c, 0x75, 0x74, 0x52,
0x69, 0xf3, 0x20, 0x42, 0x72, 0x61, 0x6e, 0x6b, 0xf3, 0x53, 0x61, 0x6e,
0x74, 0x69, 0x61, 0x67, 0xf3, 0x53, 0x61, 0x6e, 0x74, 0xf3, 0x20, 0x44,
0x6f, 0x6d, 0x69, 0x6e, 0x67, 0xf3, 0x53, 0xe1, 0xf3, 0x20, 0x50, 0x61,
0x75, 0x6c, 0xf3, 0x49, 0x74, 0x74, 0x6f, 0x6b, 0x6b, 0x6f, 0x72, 0x74,
0x75, 0x72, 0x6d, 0x69, 0x74, 0x53, 0x74, 0x2e, 0x20, 0x4c, 0x75, 0x73,
0x69, 0xe1, 0x53, 0x74, 0x2e, 0x20, 0x54, 0xf3, 0x6d, 0x61, 0x73, 0x53,
0x74, 0x2e, 0x20, 0x57, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x53, 0x77,
0x69, 0x66, 0x74, 0x20, 0x4b, 0x61, 0x72, 0x72, 0x61, 0x6e, 0x74, 0x54,
0x65, 0x67, 0x75, 0x63, 0x68, 0x69, 0x67, 0x61, 0x6c, 0x70, 0xe1, 0x54,
0x75, 0x6c, 0xe9, 0x54, 0x69, 0x6a, 0x75, 0xe1, 0x6e, 0x61, 0x54, 0x6f,
0x72, 0x6f, 0x6e, 0x74, 0xf3, 0x54, 0x6f, 0x72, 0x74, 0x6f, 0x6c, 0xe1,
0x57, 0x61, 0x6e, 0x6b, 0x75, 0x77, 0x61, 0x72, 0x4b, 0xe1, 0x73, 0xe9,
0x44, 0x61, 0x77, 0x69, 0x73, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20,
0x55, 0x72, 0x77, 0x69, 0x6c, 0x61, 0x4d, 0x61, 0x6b, 0x77, 0xe1, 0x65,
0x72, 0x53, 0x79, 0xf3, 0x77, 0xe1, 0x41, 0x6b, 0x74, 0x75, 0x62, 0xe9,
0x42, 0x61, 0x68, 0x72, 0x65, 0x6e, 0x42, 0x65, 0x6e, 0x67, 0x6b, 0xe1,
0x6b, 0x42, 0xe9, 0x72, 0x75, 0x74, 0x42, 0x72, 0x75, 0x6e, 0xe1, 0x69,
0x4b, 0xf3, 0x6c, 0x61, 0x6d, 0x62, 0xf3, 0x44, 0x61, 0x6d, 0xe1, 0x73,
0x6b, 0x61, 0x73, 0x44, 0x65, 0x68, 0x6c, 0x69, 0x44, 0x61, 0x62, 0x61,
0x69, 0x44, 0x6f, 0x73, 0x68, 0x61, 0x6d, 0x62, 0x65, 0x68, 0x50, 0x61,
0x6d, 0x61, 0x67, 0x75, 0x73, 0x74, 0xe1, 0x48, 0xf3, 0x20, 0x43, 0x68,
0x69, 0x20, 0x4d, 0x65, 0x6e, 0x6e, 0x48, 0xe1, 0x6e, 0x67, 0x20, 0x4b,
0xe1, 0x6e, 0x67, 0x48, 0xf3, 0x77, 0x64, 0x4a, 0x61, 0x6b, 0xe1, 0x72,
0x74, 0xe1, 0x55, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x69, 0x6d, 0x4b, 0xe1,
0x62, 0x6f, 0x6c, 0x4b, 0x77, 0xe1, 0x6c, 0xe1, 0x20, 0x4c, 0x61, 0x6d,
0x70, 0x75, 0x72, 0x4b, 0x77, 0x61, 0x79, 0x74, 0x4d, 0x61, 0x6b, 0xe1,
0xf3, 0x4e, 0x69, 0x6b, 0xf3, 0x73, 0x68, 0x69, 0xe1, 0x4f, 0x72, 0xe1,
0x6c, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0xe1, 0x6e, 0x67, 0x47, 0x61,
0x74, 0x61, 0x72, 0x4b, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x4b,
0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0xe1, 0x52, 0x79, 0xe1, 0x7a,
0x53, 0x65, 0xf3, 0x6c, 0x53, 0x65, 0x6e, 0x67, 0xe1, 0x70, 0x75, 0x72,
0x53, 0x72, 0x65, 0x64, 0x6e, 0x65, 0x6b, 0xf3, 0x6c, 0x69, 0x6d, 0x73,
0x6b, 0x54, 0xe1, 0x69, 0x70, 0x69, 0x54, 0xe1, 0x73, 0x68, 0x6b, 0x61,
0x6e, 0x64, 0x54, 0x65, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x54, 0x65,
0x68, 0x72, 0xe1, 0x6e, 0x54, 0xf3, 0x6b, 0x79, 0xf3, 0x57, 0x69, 0x65,
0x6e, 0x74, 0x69, 0xe1, 0x6e, 0xe9, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77,
0xf3, 0x73, 0x74, 0x6f, 0x6b, 0x59, 0x65, 0x72, 0x65, 0x77, 0xe1, 0x6e,
0x4b, 0x61, 0x6e, 0x61, 0x72, 0x61, 0x79, 0x4b, 0x61, 0x70, 0x20, 0x57,
0x61, 0x72, 0x64, 0xe9, 0x52, 0x65, 0x79, 0x6b, 0x6a, 0x61, 0x77, 0x69,
0x6b, 0x53, 0x68, 0x65, 0x6d, 0xe1, 0x6c, 0x69, 0x20, 0x4a, 0xe1, 0x72,
0x6a, 0x69, 0xe1, 0x53, 0x74, 0x2e, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e,
0xe1, 0x53, 0x74, 0xe1, 0x6e, 0x6c, 0xe9, 0x41, 0x64, 0x69, 0x6c, 0xe9,
0x64, 0x59, 0x75, 0x6b, 0x6c, 0xe1, 0x4d, 0x65, 0x6c, 0x62, 0x6f, 0x72,
0x6e, 0x53, 0x65, 0x64, 0x68, 0x6e, 0x69, 0xc1, 0x74, 0x65, 0x6e, 0x42,
0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61, 0x77, 0xe1, 0x42, 0x75, 0x6b,
0x68, 0xe1, 0x72, 0x65, 0x73, 0x74, 0x42, 0x75, 0x64, 0xe1, 0x70, 0x65,
0x73, 0x74, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0xe1, 0x67, 0x65, 0x6e,
0x47, 0x75, 0x65, 0x72, 0x6e, 0x73, 0xe9, 0x79, 0x45, 0x73, 0x74, 0x61,
0x6d, 0x62, 0xf3, 0x6c, 0x4a, 0x65, 0x72, 0x73, 0xe9, 0x79, 0x4b, 0x69,
0x72, 0xf3, 0x77, 0x4c, 0x6a, 0x75, 0x62, 0x6c, 0x6a, 0x61, 0x6e, 0xe1,
0x4c, 0x6f, 0x67, 0x7a, 0x65, 0x6d, 0x62, 0x6f, 0x72, 0x67, 0x4d, 0x6f,
0x6e, 0xe1, 0x6b, 0xf3, 0x4d, 0xe1, 0x73, 0x6b, 0xf3, 0xd3, 0x73, 0x6c,
0xf3, 0x50, 0x61, 0x79, 0x72, 0x65, 0x73, 0x50, 0x6f, 0x64, 0x67, 0x6f,
0x72, 0x69, 0x6b, 0xe1, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69,
0x6e, 0xf3, 0x53, 0x61, 0x72, 0x61, 0x6a, 0xe9, 0x77, 0xf3, 0x53, 0x69,
0x6d, 0x70, 0x65, 0x72, 0xf3, 0x70, 0xf3, 0x6c, 0x4b, 0x6f, 0x70, 0x6a,
0xe9, 0x53, 0x6f, 0x66, 0x69, 0xe1, 0x45, 0x73, 0x74, 0x68, 0xe1, 0x6b,
0x68, 0x6f, 0x6c, 0x6d, 0x54, 0x69, 0x72, 0xe1, 0x6e, 0x65, 0x55, 0x6c,
0x79, 0x61, 0x6e, 0xf3, 0x77, 0x73, 0x6b, 0x57, 0x69, 0x65, 0x6e, 0x6e,
0xe1, 0x57, 0x69, 0x6c, 0x6e, 0x69, 0x75, 0x73, 0x57, 0xe1, 0x72, 0x73,
0xe1, 0x5a, 0x79, 0x75, 0x72, 0x65, 0x6b, 0x68, 0x41, 0x6e, 0x74, 0x61,
0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x77, 0xf3, 0x43, 0x68, 0x61, 0x67,
0xf3, 0x73, 0x4b, 0x72, 0x65, 0x73, 0x6d, 0x65, 0x73, 0x4b, 0x75, 0x6b,
0x75, 0x73, 0x4b, 0x6f, 0x6d, 0xf3, 0x72, 0xf3, 0x4b, 0x61, 0x72, 0x67,
0x77, 0x65, 0x6c, 0x65, 0x6e, 0x4d, 0xe1, 0x6c, 0x64, 0x69, 0x70, 0x4d,
0x75, 0x72, 0x69, 0x73, 0x68, 0x61, 0x73, 0x4d, 0x61, 0x79, 0xf3, 0x74,
0x61, 0x79, 0x52, 0xe9, 0x79, 0x75, 0x6e, 0x69, 0x61, 0x6e, 0x41, 0x70,
0x69, 0xe1, 0x41, 0x77, 0x6b, 0x6c, 0xe1, 0x6e, 0x64, 0x42, 0xf3, 0x67,
0xe1, 0x69, 0x6e, 0x77, 0x69, 0x6c, 0x61, 0x79, 0x49, 0x73, 0x74, 0x68,
0x61, 0x72, 0x45, 0x66, 0xe1, 0x74, 0x69, 0x46, 0x61, 0x6b, 0x61, 0xf3,
0x66, 0xf3, 0x4b, 0xf3, 0x73, 0x72, 0x61, 0x69, 0x4d, 0x61, 0x72, 0x6b,
0x65, 0x73, 0xe1, 0x73, 0x4e, 0x75, 0x72, 0x70, 0x6f, 0x6c, 0x6b, 0x50,
0x69, 0x74, 0x6b, 0x61, 0x72, 0x69, 0x6e, 0x41, 0x72, 0x67, 0xea, 0xc7,
0xe9, 0x75, 0x74, 0x61, 0x4e, 0x61, 0x69, 0x72, 0xf6, 0x62, 0x69, 0x54,
0x75, 0x6e, 0x65, 0x78, 0x69, 0x43, 0xf2, 0x72, 0x64, 0x6f, 0x76, 0x61,
0x43, 0x61, 0x69, 0x65, 0xf1, 0x61, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c,
0x75, 0x70, 0x70, 0x61, 0xc7, 0x69, 0x74, 0x74, 0xe6, 0x20, 0x64, 0x6f,
0x20, 0x4d, 0x65, 0x73, 0x63, 0x69, 0x63, 0x6f, 0x53, 0x61, 0x6e, 0x20,
0x50, 0x6f, 0x75, 0x6c, 0x6f, 0x53, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72,
0x74, 0x6f, 0x6d, 0xea, 0x53, 0x61, 0x6e, 0x20, 0x43, 0x72, 0x69, 0x73,
0x74, 0x6f, 0x66, 0x66, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c,
0x75, 0xe7, 0x69, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x56, 0x69, 0xe7, 0x65,
0x6e, 0x73, 0x6f, 0x53, 0x63, 0x69, 0x6e, 0x67, 0x61, 0x70, 0xf4, 0x43,
0x61, 0x6e, 0xe4, 0x69, 0x65, 0x43, 0x61, 0x70, 0x70, 0x6f, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x73, 0x75, 0x64, 0x42, 0x65, 0x6c, 0x67, 0x72, 0x61,
0x64, 0x64, 0x6f, 0x44, 0x6f, 0x62, 0x6c, 0x69, 0x6e, 0x47, 0x69, 0x62,
0x65, 0x72, 0x74, 0xe2, 0x49, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x6e, 0x4c, 0x69, 0x73, 0x62, 0x6f, 0xf1, 0x61, 0x4c, 0x75,
0x62, 0x69, 0x61, 0xf1, 0x61, 0x4d, 0x6f, 0x6e, 0x65, 0x67, 0x6f, 0xd2,
0x73, 0x6c, 0x6f, 0x52, 0x6f, 0x6d, 0x6d, 0x61, 0x54, 0x69, 0x72, 0x61,
0xf1, 0x61, 0x43, 0x6f, 0x6d, 0xf6, 0x72, 0x65, 0x4d, 0x61, 0x69, 0xf2,
0x74, 0x74, 0x61, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x69, 0x78, 0x69,
0x6a9, 0x6cc, 0x679, 0x627, 0x645, 0x627, 0x631, 0x633, 0x6cc, 0x6a9, 0x6cc, 0x645,
0x628, 0x631, 0x62c, 0x20, 0x633, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc,
0x6a9, 0x631, 0x627, 0x633, 0x6cc, 0x633, 0x633, 0x6cc, 0x626, 0x6cc, 0x646, 0x6af,
0x644, 0x6cc, 0x633, 0x20, 0x633, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc,
0x6af, 0x648, 0x633, 0x20, 0x633, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc,
0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x6cc, 0x646, 0x6af, 0x633, 0x6cc, 0x628,
0x644, 0x633, 0x6cc, 0x679, 0x6be, 0x645, 0x646, 0x688, 0x648, 0x6a9, 0x648, 0x644,
0x633, 0x6cc, 0x62a, 0x627, 0x645, 0x633, 0x6cc, 0x624, 0x627, 0x688, 0x6cc, 0x644,
0x627, 0x626, 0x688, 0x41, 0x62, 0x69, 0x6a, 0x61, 0x14b, 0x41, 0x256, 0x69,
0x73, 0x20, 0x41, 0x62, 0x65, 0x62, 0x61, 0x41, 0x6c, 0x6a, 0x25b, 0x25b,
0x72, 0x41, 0x73, 0x6d, 0x61, 0x72, 0x61, 0x61, 0x42, 0x61, 0x6d, 0x61,
0x6b, 0x6f, 0x6f, 0x42, 0x61, 0x14b, 0x67, 0x69, 0x69, 0x42, 0x69, 0x73,
0x6f, 0x6f, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x69, 0x72, 0x42, 0x72,
0x61, 0x73, 0x61, 0x66, 0x69, 0x6c, 0x42, 0x75, 0x6a, 0x75, 0x6d, 0x62,
0x75, 0x72, 0x61, 0x61, 0x4b, 0x61, 0x69, 0x72, 0x6f, 0x6f, 0x4b, 0x61,
0x73, 0x61, 0x62, 0x6c, 0x61, 0x14b, 0x6b, 0x61, 0x61, 0x53, 0x65, 0x79,
0x75, 0x74, 0x61, 0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x69, 0x69, 0x189,
0x61, 0x6b, 0x61, 0x61, 0x72, 0x189, 0x61, 0x72, 0x25b, 0x73, 0x61, 0x6c,
0x61, 0x61, 0x6d, 0x4a, 0x69, 0x62, 0x75, 0x74, 0x69, 0x69, 0x189, 0x75,
0x77, 0x61, 0x6c, 0x61, 0x61, 0x190, 0x6c, 0x20, 0x41, 0x79, 0x75, 0x6e,
0x46, 0x72, 0x69, 0x69, 0x74, 0x61, 0x28a, 0x6e, 0x48, 0x61, 0x72, 0x61,
0x72, 0x65, 0x65, 0x59, 0x6f, 0x68, 0x61, 0x6e, 0x25b, 0x73, 0x62, 0x75,
0x75, 0x72, 0x4a, 0x75, 0x62, 0x61, 0x61, 0x4b, 0x61, 0x6d, 0x70, 0x61,
0x6c, 0x61, 0x61, 0x4b, 0x61, 0x72, 0x74, 0x75, 0x75, 0x6d, 0x4b, 0x69,
0x6e, 0x73, 0x68, 0x61, 0x73, 0x61, 0x61, 0x4c, 0x69, 0x62, 0x72, 0x1dd,
0x66, 0x69, 0x6c, 0x4c, 0x28a, 0x6d, 0x25b, 0x4c, 0x75, 0x77, 0x61, 0x6e,
0x256, 0x61, 0x61, 0x4c, 0x75, 0x73, 0x61, 0x6b, 0x61, 0x61, 0x4d, 0x61,
0x6c, 0x61, 0x62, 0x6f, 0x6f, 0x4d, 0x61, 0x70, 0x75, 0x74, 0x6f, 0x6f,
0x4d, 0x61, 0x73, 0x65, 0x72, 0x75, 0x75, 0x4d, 0x62, 0x61, 0x62, 0x61,
0x6e, 0x65, 0x65, 0x4d, 0x6f, 0x67, 0x61, 0x256, 0x69, 0x73, 0x68, 0x75,
0x75, 0x4d, 0x6f, 0x6e, 0x72, 0x6f, 0x66, 0x69, 0x79, 0x61, 0x61, 0x4e,
0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x61, 0x4e, 0x79, 0x25b, 0x6d, 0x25b,
0x4e, 0x75, 0x77, 0x61, 0x6b, 0x63, 0x254, 0x74, 0x57, 0x61, 0x67, 0x61,
0x256, 0x75, 0x67, 0x75, 0x50, 0x254, 0x72, 0x74, 0x6f, 0x20, 0x4e, 0x6f,
0x66, 0x6f, 0x6f, 0x53, 0x61, 0x6f, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x6b,
0x61, 0x6d, 0x70, 0xe1, 0x54, 0x69, 0x6e, 0x69, 0x69, 0x73, 0x57, 0x69,
0x6e, 0x256, 0x68, 0x75, 0x6b, 0x41, 0x256, 0x61, 0x6b, 0x41, 0x14b, 0x6b,
0x254, 0x72, 0x61, 0x61, 0x6a, 0x269, 0x41, 0x14b, 0x67, 0x75, 0x77, 0x69,
0x6c, 0x61, 0x61, 0x41, 0x6e, 0x74, 0x69, 0x67, 0x75, 0x77, 0x61, 0x61,
0x41, 0x72, 0x61, 0x67, 0x75, 0x77, 0x65, 0x65, 0x6e, 0x61, 0x42, 0x75,
0x77, 0x65, 0x6e, 0x254, 0x73, 0x20, 0x41, 0x69, 0x72, 0x25b, 0x73, 0x4b,
0x61, 0x74, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x61, 0x4b, 0x254, 0x72,
0x256, 0x6f, 0x62, 0x61, 0x61, 0x48, 0x75, 0x68, 0x75, 0x79, 0x69, 0x4c,
0x61, 0x20, 0x52, 0x69, 0x79, 0x254, 0x68, 0x61, 0x4d, 0x25b, 0x6e, 0x256,
0x254, 0x73, 0x61, 0x61, 0x52, 0x69, 0x79, 0x6f, 0x20, 0x47, 0x61, 0x6c,
0x65, 0x67, 0x254, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x48, 0x75, 0x77, 0x61,
0x6e, 0x55, 0x73, 0x68, 0x75, 0x77, 0x65, 0x79, 0x61, 0x61, 0x41, 0x72,
0x75, 0x62, 0x61, 0x61, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x69, 0x79, 0x254,
0x6e, 0x42, 0x61, 0x68, 0x69, 0x79, 0x61, 0x61, 0x42, 0x61, 0x68, 0x69,
0x69, 0x79, 0x61, 0x20, 0x256, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x256, 0x65,
0x72, 0x61, 0x61, 0x73, 0x269, 0x42, 0x61, 0x72, 0x62, 0x61, 0x256, 0x254,
0x254, 0x73, 0x269, 0x42, 0x65, 0x6c, 0x25b, 0x6d, 0x42, 0x6c, 0x61, 0x14b,
0x2d, 0x53, 0x61, 0x62, 0x6c, 0x254, 0x14b, 0x42, 0x6f, 0x77, 0x61, 0x20,
0x46, 0x69, 0x73, 0x74, 0x61, 0x61, 0x42, 0x254, 0x77, 0x61, 0x61, 0x73,
0x269, 0x4b, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x69, 0x6a, 0x269, 0x20, 0x42,
0x61, 0x269, 0x43, 0x61, 0x6d, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e,
0x256, 0x65, 0x65, 0x4b, 0x61, 0x14b, 0x6b, 0x75, 0x6e, 0x4b, 0x61, 0x79,
0x25b, 0x25b, 0x6e, 0x4b, 0x61, 0x79, 0x69, 0x6d, 0x61, 0x61, 0x6e, 0x53,
0x68, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0x6f, 0x53, 0x68, 0x69, 0x77, 0x61,
0x77, 0x61, 0x61, 0x53, 0x69, 0x77, 0x75, 0x256, 0x61, 0x61, 0x256, 0x20,
0x48, 0x75, 0x77, 0x61, 0x72, 0x25b, 0x73, 0x4b, 0x254, 0x73, 0x74, 0x61,
0x20, 0x52, 0x69, 0x6b, 0x61, 0x61, 0x4b, 0x72, 0x25b, 0x73, 0x74, 0x254,
0x6e, 0x4b, 0x75, 0x79, 0x61, 0x62, 0x61, 0x61, 0x4b, 0x75, 0x72, 0x61,
0x73, 0x61, 0x61, 0x77, 0x75, 0x189, 0x61, 0x6e, 0x1dd, 0x6d, 0x61, 0x72,
0x6b, 0x269, 0x68, 0x61, 0x28a, 0x6e, 0x189, 0x254, 0x254, 0x73, 0x1dd, 0x6e,
0x189, 0x254, 0x254, 0x73, 0x1dd, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x69, 0x6b,
0x189, 0x25b, 0x6e, 0x66, 0x61, 0x189, 0x69, 0x74, 0x72, 0x254, 0x254, 0x269,
0x189, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0x61, 0x190, 0x256, 0x6d,
0x254, 0x6e, 0x74, 0x254, 0x6e, 0x190, 0x269, 0x72, 0x75, 0x6e, 0x65, 0x70,
0x65, 0x65, 0x190, 0x6c, 0x20, 0x53, 0x61, 0x6c, 0x66, 0x61, 0x256, 0x254,
0x254, 0x72, 0x46, 0x254, 0x254, 0x72, 0x20, 0x4e, 0x25b, 0x6c, 0x73, 0x254,
0x6e, 0x46, 0x254, 0x72, 0x74, 0x61, 0x6c, 0x25b, 0x73, 0x61, 0x61, 0x47,
0x6c, 0x61, 0x61, 0x73, 0x20, 0x42, 0x61, 0x269, 0x47, 0x75, 0x75, 0x73,
0x20, 0x42, 0x61, 0x269, 0x47, 0x72, 0x61, 0x6e, 0x256, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x47, 0x72, 0x65, 0x6e, 0x61, 0x61, 0x256, 0x61, 0x47, 0x75,
0x77, 0x61, 0x256, 0x65, 0x6c, 0x75, 0x75, 0x70, 0x75, 0x47, 0x75, 0x77,
0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x61, 0x47, 0x75, 0x77, 0x61,
0x79, 0x61, 0x6b, 0x69, 0x6c, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x61,
0x48, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x61, 0x73, 0x48, 0x61, 0x66, 0x61,
0x6e, 0x61, 0x61, 0x190, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6f, 0x6f,
0x196, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c,
0x69, 0x4e, 0x254, 0x6b, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69, 0x79, 0x61,
0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x25b, 0x14b, 0x67, 0x6f, 0x2c, 0x20,
0x196, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x50, 0x65, 0x74,
0x25b, 0x72, 0x73, 0x62, 0x75, 0x75, 0x72, 0x2c, 0x20, 0x196, 0x6e, 0x256,
0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x54, 0x25b, 0x6c, 0x20, 0x53, 0x69,
0x74, 0x69, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x61, 0x6e,
0x61, 0x46, 0x65, 0x66, 0x25b, 0x269, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69,
0x79, 0x61, 0x61, 0x6e, 0x61, 0x46, 0x25b, 0x14b, 0x73, 0x25b, 0x6e, 0x2c,
0x20, 0x196, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x57, 0x69,
0x6e, 0x61, 0x6d, 0x61, 0x61, 0x6b, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69,
0x79, 0x61, 0x61, 0x6e, 0x61, 0x49, 0x6e, 0x75, 0x66, 0x69, 0x6b, 0x49,
0x6b, 0x61, 0x6c, 0x75, 0x77, 0x69, 0x69, 0x74, 0x4a, 0x61, 0x6d, 0x61,
0x269, 0x6b, 0x61, 0x4a, 0x69, 0x6e, 0x6f, 0x6f, 0x4c, 0x75, 0x77, 0x69,
0x66, 0x69, 0x69, 0x6c, 0x4d, 0x254, 0x6e, 0x74, 0x69, 0x73, 0x68, 0x25b,
0x25b, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x25b, 0x6e, 0x74, 0x61, 0x6b, 0x69,
0x4b, 0x72, 0x61, 0x6c, 0x25b, 0x6e, 0x256, 0x61, 0x269, 0x6b, 0x4c, 0x69,
0x6d, 0x61, 0x61, 0x4c, 0x254, 0x73, 0x20, 0x41, 0x6e, 0x6a, 0x25b, 0x6c,
0x25b, 0x25b, 0x73, 0x4c, 0x6f, 0x77, 0x25b, 0x25b, 0x20, 0x50, 0x72, 0x25b,
0x14b, 0x73, 0x20, 0x4b, 0x77, 0x61, 0x74, 0x61, 0x4d, 0x61, 0x73, 0x25b,
0x269, 0x79, 0x6f, 0x6f, 0x4d, 0x61, 0x6e, 0x61, 0x28a, 0x73, 0x4d, 0x61,
0x72, 0x69, 0x67, 0x6f, 0x6f, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69,
0x69, 0x6b, 0x69, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x6f,
0x73, 0x4d, 0x61, 0x73, 0x61, 0x74, 0x1dd, 0x6c, 0x61, 0x6e, 0x4d, 0x65,
0x72, 0x69, 0x69, 0x256, 0x61, 0x4d, 0x65, 0x74, 0x65, 0x6c, 0x61, 0x6b,
0x61, 0x74, 0x65, 0x6c, 0x61, 0x61, 0x4d, 0x25b, 0x73, 0x69, 0x6b, 0x6f,
0x20, 0x53, 0x69, 0x74, 0x69, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x254, 0x14b,
0x4d, 0x254, 0x14b, 0x74, 0x254, 0x6e, 0x4d, 0x254, 0x6e, 0x74, 0x25b, 0x72,
0x25b, 0x269, 0x4d, 0x254, 0x6e, 0x74, 0x65, 0x66, 0x69, 0x256, 0x65, 0x79,
0x6f, 0x6f, 0x4d, 0x254, 0x6e, 0x73, 0x25b, 0x72, 0x61, 0x61, 0x74, 0x269,
0x4e, 0x69, 0x75, 0x20, 0x59, 0x254, 0x72, 0x6b, 0x4e, 0x6f, 0x72, 0x254,
0x6e, 0x79, 0x61, 0x42, 0x69, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x4e, 0x254,
0x254, 0x72, 0x20, 0x189, 0x61, 0x6b, 0x6f, 0x6f, 0x74, 0x61, 0x53, 0x25b,
0x6e, 0x74, 0x61, 0x2c, 0x20, 0x4e, 0x254, 0x254, 0x72, 0x20, 0x189, 0x61,
0x6b, 0x6f, 0x6f, 0x74, 0x61, 0x4e, 0x69, 0x75, 0x20, 0x53, 0x61, 0x6c,
0x25b, 0x6d, 0x2c, 0x20, 0x4e, 0x254, 0x254, 0x72, 0x20, 0x189, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x50, 0x61, 0x6e, 0x61, 0x6d, 0x61, 0x61, 0x50, 0x61,
0x72, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x6f, 0x6f, 0x46, 0x69, 0x6e,
0x69, 0x69, 0x73, 0x50, 0x254, 0x254, 0x72, 0x20, 0x6f, 0x20, 0x50, 0x72,
0x25b, 0x14b, 0x73, 0x50, 0x254, 0x254, 0x72, 0x20, 0x254, 0x66, 0x20, 0x53,
0x70, 0x25b, 0x25b, 0x6e, 0x50, 0x254, 0x72, 0x74, 0x6f, 0x20, 0x46, 0x65,
0x6c, 0x6f, 0x6f, 0x50, 0x254, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b,
0x6f, 0x6f, 0x52, 0x61, 0x14b, 0x6b, 0x269, 0x6e, 0x20, 0x196, 0x6e, 0x6c,
0x25b, 0x25b, 0x74, 0x269, 0x52, 0x65, 0x67, 0x69, 0x69, 0x6e, 0x61, 0x52,
0x25b, 0x73, 0x6f, 0x6c, 0x75, 0x75, 0x74, 0x52, 0x69, 0x79, 0x6f, 0x20,
0x42, 0x72, 0x61, 0x14b, 0x6b, 0x6f, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61,
0x72, 0x25b, 0x6d, 0x53, 0x61, 0x6e, 0x74, 0x69, 0x79, 0x61, 0x67, 0x6f,
0x6f, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x189, 0x6f, 0x6d, 0x269, 0x14b,
0x67, 0x6f, 0x53, 0x61, 0x6f, 0x20, 0x50, 0x61, 0x75, 0x6c, 0x6f, 0x6f,
0x49, 0x74, 0x6f, 0x6b, 0x254, 0x72, 0x74, 0x6f, 0x6f, 0x6d, 0x69, 0x69,
0x74, 0x69, 0x53, 0x69, 0x74, 0x6b, 0x61, 0x61, 0x53, 0x25b, 0x14b, 0x2d,
0x42, 0x61, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x61, 0x79, 0x254, 0x53, 0x25b,
0x14b, 0x20, 0x4a, 0x254, 0x254, 0x6e, 0x73, 0x53, 0x25b, 0x14b, 0x20, 0x4b,
0x69, 0x74, 0x73, 0x53, 0x25b, 0x14b, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x79,
0x61, 0x53, 0x25b, 0x14b, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x61, 0x73, 0x269,
0x53, 0x25b, 0x14b, 0x20, 0x46, 0x269, 0x6e, 0x73, 0x61, 0x14b, 0x53, 0x1dd,
0x77, 0x269, 0x66, 0x74, 0x269, 0x20, 0x4b, 0x1dd, 0x72, 0x25b, 0x25b, 0x6e,
0x74, 0x269, 0x54, 0x75, 0x75, 0x6c, 0x65, 0x54, 0x69, 0x68, 0x75, 0x77,
0x61, 0x6e, 0x61, 0x54, 0x6f, 0x72, 0x254, 0x6e, 0x74, 0x6f, 0x6f, 0x54,
0x254, 0x72, 0x74, 0x254, 0x6c, 0x61, 0x61, 0x46, 0x25b, 0x14b, 0x6b, 0x75,
0x75, 0x66, 0x61, 0x57, 0x61, 0x269, 0x74, 0x68, 0x254, 0x254, 0x73, 0x57,
0x69, 0x6e, 0x69, 0x70, 0x25b, 0x25b, 0x67, 0x4b, 0x65, 0x65, 0x73, 0x69,
0x189, 0x65, 0x66, 0x69, 0x73, 0x189, 0x69, 0x6d, 0x254, 0x6e, 0x20, 0x189,
0x69, 0x72, 0x66, 0x69, 0x6c, 0x4d, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x69,
0x4d, 0x254, 0x73, 0x254, 0x6e, 0x4d, 0x25b, 0x6b, 0x4d, 0x254, 0x72, 0x256,
0x6f, 0x6f, 0x50, 0x61, 0x6c, 0x6d, 0x25b, 0x25b, 0x72, 0x52, 0x6f, 0x74,
0x65, 0x72, 0x61, 0x61, 0x53, 0x69, 0x79, 0x6f, 0x77, 0x61, 0x61, 0x54,
0x72, 0x254, 0x6c, 0x46, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x4c, 0x254, 0x14b,
0x79, 0x69, 0x69, 0x72, 0x62, 0x69, 0x79, 0x25b, 0x6e, 0x41, 0x256, 0x25b,
0x6e, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x69, 0x41, 0x6d, 0x61, 0x61,
0x6e, 0x41, 0x6e, 0x61, 0x256, 0x69, 0x69, 0x72, 0x41, 0x6b, 0x74, 0x61,
0x28a, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x65, 0x41, 0x73, 0x68, 0x67,
0x61, 0x62, 0x61, 0x61, 0x74, 0x41, 0x74, 0x69, 0x72, 0x61, 0x28a, 0x42,
0x61, 0x67, 0x256, 0x61, 0x61, 0x256, 0x42, 0x61, 0x72, 0x25b, 0x25b, 0x6e,
0x42, 0x61, 0x14b, 0x6b, 0x254, 0x6b, 0x42, 0x61, 0x72, 0x6e, 0x61, 0x77,
0x75, 0x6c, 0x42, 0x65, 0x69, 0x72, 0x75, 0x75, 0x74, 0x42, 0x69, 0x73,
0x68, 0x6b, 0x25b, 0x6b, 0x42, 0x72, 0x75, 0x6e, 0x25b, 0x269, 0x53, 0x68,
0x69, 0x74, 0x61, 0x61, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x6f, 0x6f,
0x189, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x254, 0x254, 0x73, 0x269, 0x189, 0x69,
0x6c, 0x69, 0x69, 0x189, 0x75, 0x62, 0x61, 0x269, 0x189, 0x75, 0x73, 0x68,
0x61, 0x6e, 0x62, 0x65, 0x65, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x75, 0x73,
0x74, 0x61, 0x61, 0x47, 0x61, 0x73, 0x61, 0x61, 0x48, 0x65, 0x62, 0x72,
0x254, 0x6e, 0x48, 0x6f, 0x20, 0x43, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x20,
0x6b, 0x61, 0x4d, 0x70, 0xe1, 0x48, 0x254, 0x14b, 0x20, 0x4b, 0x254, 0x14b,
0x48, 0x254, 0x66, 0x256, 0x1dd, 0x4a, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x61,
0x4a, 0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61, 0x61, 0x59, 0x65, 0x72,
0x75, 0x73, 0x61, 0x6c, 0x25b, 0x6d, 0x4b, 0x61, 0x6d, 0x73, 0x68, 0x61,
0x74, 0x6b, 0x61, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x63, 0x269, 0x4b, 0x61,
0x74, 0x6d, 0x61, 0x6e, 0x256, 0x75, 0x75, 0x4b, 0x61, 0x6e, 0x256, 0x69,
0x67, 0x61, 0x61, 0x4b, 0x254, 0x6c, 0x6b, 0x61, 0x74, 0x61, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x6b, 0x4b, 0x75, 0x77, 0x61,
0x6c, 0x61, 0x20, 0x4c, 0x75, 0x6d, 0x70, 0x75, 0x75, 0x72, 0x4b, 0x75,
0x63, 0x69, 0x14b, 0x4b, 0x6f, 0x77, 0x65, 0x65, 0x74, 0x69, 0x4d, 0x61,
0x6b, 0x61, 0x77, 0x6f, 0x6f, 0x4d, 0x61, 0x67, 0x61, 0x256, 0x61, 0x6e,
0x4d, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x61, 0x72, 0x4d, 0x61, 0x6e, 0x69,
0x6c, 0x61, 0x61, 0x4d, 0x75, 0x73, 0x6b, 0x61, 0x61, 0x74, 0x4e, 0x6f,
0x66, 0x6f, 0x6b, 0x75, 0x73, 0x6e, 0x25b, 0x6b, 0x4e, 0x6f, 0x66, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x6b, 0x186, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd,
0x4f, 0x72, 0x61, 0x61, 0x6c, 0x4e, 0x254, 0x6d, 0x20, 0x50, 0x25b, 0x6e,
0x50, 0x254, 0x6e, 0x74, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x6b, 0x50, 0x269,
0x79, 0x254, 0x14b, 0x79, 0x61, 0x14b, 0x4b, 0x254, 0x73, 0x74, 0x61, 0x6e,
0x61, 0x269, 0x4b, 0x69, 0x73, 0x69, 0x6c, 0x254, 0x72, 0x256, 0x61, 0x61,
0x52, 0x69, 0x79, 0x61, 0x61, 0x256, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b,
0x61, 0x6e, 0x256, 0x53, 0x65, 0x77, 0x75, 0x6c, 0x53, 0x68, 0x61, 0x14b,
0x67, 0x61, 0x269, 0x53, 0x69, 0x14b, 0x67, 0x61, 0x70, 0x75, 0x75, 0x72,
0x53, 0x72, 0x25b, 0x256, 0x6e, 0x25b, 0x6b, 0x254, 0x6c, 0x69, 0x6d, 0x54,
0x61, 0x269, 0x70, 0x65, 0x69, 0x54, 0x61, 0x73, 0x68, 0x6b, 0x25b, 0x6e,
0x74, 0x54, 0x69, 0x62, 0x6c, 0x69, 0x73, 0x69, 0x69, 0x54, 0x65, 0x65,
0x72, 0x61, 0x61, 0x6e, 0x54, 0x69, 0x6d, 0x66, 0x75, 0x54, 0x6f, 0x6b,
0x69, 0x79, 0x6f, 0x6f, 0x54, 0x254, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd, 0x55,
0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x254, 0x254, 0x72, 0x55, 0x72, 0x75,
0x6d, 0x6b, 0x69, 0x69, 0x55, 0x73, 0x74, 0x2d, 0x4e, 0x65, 0x72, 0x61,
0x61, 0x46, 0x69, 0x79, 0x65, 0x6e, 0x74, 0x69, 0x79, 0x61, 0x6e, 0x46,
0x6c, 0x61, 0x256, 0x69, 0x66, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x59, 0x61,
0x14b, 0x67, 0x254, 0x6e, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x75, 0x59, 0x65, 0x72, 0x65, 0x66, 0x61, 0x61, 0x6e,
0x41, 0x73, 0x254, 0x254, 0x72, 0x42, 0x25b, 0x72, 0x6d, 0x75, 0x256, 0x61,
0x61, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x69, 0x4b, 0x61, 0x70, 0x66,
0x25b, 0x25b, 0x72, 0x46, 0x61, 0x72, 0x6f, 0x69, 0x4d, 0x61, 0x256, 0x65,
0x69, 0x72, 0x61, 0x61, 0x52, 0x25b, 0x269, 0x6b, 0x79, 0x61, 0x66, 0x69,
0x6b, 0x4a, 0x254, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x20, 0x67, 0x28a, 0x6e,
0x79, 0x269, 0x73, 0x6f, 0x6e, 0x6f, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x6a, 0x61, 0x53, 0x25b, 0x14b, 0x20, 0x45, 0x6c, 0x65, 0x6e, 0x61, 0x61,
0x53, 0x74, 0x61, 0x6e, 0x6c, 0x25b, 0x269, 0x41, 0x256, 0x65, 0x6c, 0x25b,
0x25b, 0x256, 0x42, 0x72, 0x69, 0x73, 0x62, 0x25b, 0x6e, 0x42, 0x72, 0x6f,
0x6b, 0x1dd, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x189, 0x61, 0x72, 0x66, 0x69,
0x6e, 0x55, 0x6b, 0x6c, 0x61, 0x61, 0x48, 0x6f, 0x62, 0x61, 0x61, 0x4c,
0x69, 0x6e, 0x256, 0x65, 0x6d, 0x61, 0x6e, 0x4c, 0x254, 0x72, 0x256, 0x20,
0x48, 0x6f, 0x6f, 0x4d, 0x25b, 0x6c, 0x62, 0x254, 0x72, 0x6e, 0x50, 0x25b,
0x72, 0x74, 0x53, 0x69, 0x256, 0x6e, 0x65, 0x65, 0x41, 0x6d, 0x73, 0x74,
0x25b, 0x72, 0x256, 0x61, 0x6d, 0x41, 0x6e, 0x256, 0x254, 0x72, 0x61, 0x61,
0x41, 0x73, 0x74, 0x72, 0x61, 0x6b, 0x61, 0x61, 0x6e, 0x41, 0x74, 0x25b,
0x6e, 0x42, 0x25b, 0x6c, 0x67, 0x72, 0x61, 0x61, 0x256, 0x42, 0x25b, 0x72,
0x6c, 0x25b, 0x14b, 0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61, 0x66,
0x61, 0x42, 0x72, 0x69, 0x73, 0x25b, 0x6c, 0x42, 0x75, 0x6b, 0x61, 0x72,
0x25b, 0x73, 0x42, 0x75, 0x256, 0x61, 0x70, 0x25b, 0x73, 0x42, 0x75, 0x73,
0x69, 0x14b, 0x67, 0x25b, 0x6e, 0x53, 0x68, 0x69, 0x73, 0x69, 0x6e, 0x6f,
0x6f, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x61, 0x61, 0x67, 0x189, 0x254, 0x62,
0x6c, 0x25b, 0x14b, 0x47, 0x1dd, 0x72, 0x1dd, 0x6e, 0x73, 0x25b, 0x269, 0x48,
0x25b, 0x6c, 0x73, 0x69, 0x14b, 0x6b, 0x69, 0x69, 0x4d, 0x61, 0x6e, 0x20,
0x6b, 0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61,
0x49, 0x73, 0x74, 0x61, 0x6d, 0x62, 0x75, 0x75, 0x6c, 0x4b, 0x61, 0x6c,
0x69, 0x6e, 0x69, 0x14b, 0x67, 0x72, 0x61, 0x61, 0x256, 0x4b, 0x69, 0x72,
0x254, 0x254, 0x66, 0x4b, 0x69, 0x79, 0x25b, 0x66, 0x59, 0x75, 0x62, 0x1dd,
0x6c, 0x79, 0x61, 0x6e, 0x61, 0x4c, 0x254, 0x6e, 0x256, 0x254, 0x6e, 0x4c,
0x75, 0x73, 0x25b, 0x6d, 0x62, 0x75, 0x75, 0x72, 0x4d, 0x61, 0x256, 0x72,
0x69, 0x69, 0x256, 0x4d, 0x61, 0x72, 0x69, 0x79, 0x65, 0x68, 0x61, 0x6d,
0x4d, 0x269, 0x6e, 0x73, 0x4d, 0x6f, 0x6e, 0x61, 0x6b, 0x6f, 0x6f, 0x4d,
0x6f, 0x73, 0x6b, 0x75, 0x75, 0x186, 0x73, 0x6c, 0x6f, 0x6f, 0x50, 0x6f,
0x256, 0x67, 0x6f, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x52, 0x69, 0x67, 0x61,
0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x61, 0x53, 0x61, 0x6e, 0x20,
0x4d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x79,
0x65, 0x66, 0x6f, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x74, 0x254, 0x254, 0x66,
0x53, 0x6b, 0x254, 0x70, 0x79, 0x25b, 0x53, 0x74, 0x254, 0x6b, 0x68, 0x254,
0x6c, 0x6d, 0x54, 0x69, 0x72, 0x61, 0x6e, 0x61, 0x61, 0x55, 0x6c, 0x79,
0x61, 0x6e, 0x254, 0x254, 0x66, 0x46, 0x61, 0x256, 0x75, 0x73, 0x46, 0x61,
0x74, 0x69, 0x6b, 0x61, 0x14b, 0x46, 0x69, 0x79, 0x25b, 0x6e, 0x61, 0x46,
0x69, 0x6c, 0x6e, 0x69, 0x79, 0x75, 0x73, 0x46, 0x254, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x61, 0x256, 0x57, 0x61, 0x72, 0x73, 0x61, 0x28a, 0x53,
0x61, 0x67, 0x72, 0x25b, 0x62, 0x53, 0x75, 0x72, 0x69, 0x69, 0x6b, 0x41,
0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x66, 0x6f, 0x6f,
0x53, 0x68, 0x61, 0x67, 0x254, 0x73, 0x4e, 0x6f, 0x77, 0x25b, 0x6c, 0x4b,
0x6f, 0x6b, 0x6f, 0x6f, 0x73, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x6f, 0x6f,
0x4b, 0x25b, 0x72, 0x67, 0x65, 0x6c, 0x25b, 0x25b, 0x6e, 0x4d, 0x61, 0x68,
0x65, 0x65, 0x4d, 0x61, 0x6c, 0x256, 0x69, 0x69, 0x66, 0x75, 0x49, 0x6d,
0x6f, 0x72, 0x69, 0x73, 0x4d, 0x61, 0x79, 0x254, 0x254, 0x74, 0x269, 0x52,
0x65, 0x65, 0x6e, 0x69, 0x79, 0x254, 0x14b, 0x186, 0x254, 0x6b, 0x6c, 0x61,
0x6e, 0x256, 0x42, 0x75, 0x67, 0x25b, 0x25b, 0x14b, 0x66, 0x69, 0x6c, 0x53,
0x68, 0x61, 0x74, 0x61, 0x6d, 0x43, 0x75, 0x75, 0x6b, 0x45, 0x66, 0x61,
0x74, 0x65, 0x65, 0x46, 0x61, 0x6b, 0x61, 0x77, 0x6f, 0x66, 0x6f, 0x6f,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x65, 0x65, 0x47, 0x75, 0x77, 0x61,
0x256, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x4b, 0x61, 0x6e, 0x74,
0x254, 0x6e, 0x4b, 0x69, 0x72, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x69, 0x4b,
0x254, 0x73, 0x72, 0x25b, 0x25b, 0x4b, 0x77, 0x61, 0x6a, 0x61, 0x6c, 0x25b,
0x25b, 0x6e, 0x4d, 0x61, 0x6a, 0x75, 0x72, 0x6f, 0x6f, 0x4d, 0x69, 0x256,
0x77, 0x65, 0x65, 0x4e, 0x61, 0x77, 0x75, 0x72, 0x75, 0x4e, 0x69, 0x77,
0x75, 0x79, 0x65, 0x4e, 0x254, 0x72, 0x66, 0x6f, 0x6f, 0x6b, 0x50, 0x61,
0x67, 0x6f, 0x20, 0x50, 0x61, 0x67, 0x6f, 0x6f, 0x50, 0x61, 0x6c, 0x61,
0x77, 0x6f, 0x6f, 0x50, 0x269, 0x74, 0x6b, 0x25b, 0x25b, 0x6e, 0x50, 0x6f,
0x6f, 0x6e, 0x70, 0x65, 0x65, 0x50, 0x254, 0x254, 0x72, 0x20, 0x4d, 0x254,
0x72, 0x25b, 0x73, 0x62, 0x69, 0x69, 0x52, 0x61, 0x72, 0x6f, 0x74, 0x254,
0x14b, 0x67, 0x61, 0x53, 0x61, 0x269, 0x70, 0x61, 0x6e, 0x54, 0x61, 0x72,
0x61, 0x77, 0x61, 0x61, 0x54, 0x254, 0x14b, 0x67, 0x61, 0x74, 0x61, 0x70,
0x75, 0x75, 0x57, 0x65, 0x65, 0x6b, 0x905, 0x932, 0x20, 0x906, 0x907, 0x92f,
0x942, 0x928, 0x3c, 0x92e, 0x91c, 0x93c, 0x93e, 0x91f, 0x932, 0x93e, 0x928, 0x921,
0x92c, 0x908, 0x915, 0x93e, 0x92c, 0x932, 0x926, 0x916, 0x923, 0x940, 0x20, 0x91c,
0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x90f, 0x92e, 0x94d, 0x938, 0x94d,
0x91f, 0x930, 0x921, 0x92e, 0x92a, 0x948, 0x930, 0x938, 0x915, 0x94d, 0x930, 0x93f,
0x938, 0x92e, 0x93f, 0x938, 0x915, 0x948, 0x902, 0x91f, 0x928, 0x92a, 0x93e, 0x917,
0x94b, 0x20, 0x917, 0x902, 0x917, 0x94b, 0x41, 0x6c, 0x6a, 0x65, 0x72, 0x69,
0x41, 0x7a, 0x6d, 0x61, 0x72, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x68, 0xec,
0x42, 0x69, 0x73, 0xe0, 0x6f, 0x42, 0x75, 0x6a, 0x75, 0x6e, 0x62, 0x75,
0x72, 0x61, 0x43, 0x61, 0x7a, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x63, 0x61,
0x4b, 0x61, 0x6e, 0x70, 0x61, 0x6c, 0x61, 0x4c, 0x75, 0x62, 0x75, 0x6e,
0x62, 0x61, 0x73, 0x68, 0x69, 0x4d, 0x61, 0x7a, 0x65, 0x72, 0x75, 0x4e,
0x62, 0x61, 0x62, 0x61, 0x6e, 0x65, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69,
0x73, 0x6f, 0x55, 0x61, 0x67, 0x61, 0x64, 0x75, 0x67, 0xf9, 0xcc, 0x7a,
0x6f, 0x6c, 0x61, 0x20, 0x53, 0x2e, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x7a,
0x6f, 0x54, 0xf9, 0x6e, 0x65, 0x7a, 0x69, 0x43, 0xf2, 0x72, 0x64, 0x6f,
0x62, 0x61, 0x52, 0x69, 0x6f, 0x20, 0x47, 0xe0, 0x6c, 0x65, 0x67, 0x6f,
0x73, 0x42, 0x61, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x61, 0x73, 0x43, 0x61, 0x6e, 0x70, 0x6f, 0x20, 0x47,
0x72, 0x61, 0x6e, 0x64, 0x65, 0x43, 0x75, 0x69, 0x61, 0x62, 0xe0, 0x44,
0xe8, 0x6e, 0x76, 0x65, 0x72, 0x44, 0x6f, 0x6d, 0xe9, 0x6e, 0x65, 0x67,
0x61, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0xe0, 0x70, 0x6f, 0x6c, 0x69,
0x73, 0x4b, 0x6e, 0x6f, 0x78, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x29, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x20, 0x28,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x29, 0x50, 0x65, 0x74, 0x65,
0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x29, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74,
0x79, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x29, 0x56,
0x65, 0x76, 0x61, 0x79, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x29, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x20,
0x28, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x29, 0x57, 0x69, 0x6e,
0x61, 0x6d, 0x61, 0x63, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x29, 0x4a, 0x61, 0x6d, 0xe0, 0x65, 0x67, 0x61, 0x4d, 0x6f, 0x6e,
0x74, 0x69, 0x63, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x28, 0x4b, 0x65, 0x6e,
0x74, 0x75, 0x63, 0x6b, 0x79, 0x29, 0x4c, 0x6f, 0x73, 0x20, 0xc0, 0x6e,
0x67, 0x65, 0x6c, 0x65, 0x73, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69,
0x67, 0x61, 0x53, 0x69, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x4d, 0xe8, 0x73, 0x65, 0x67, 0x6f, 0x4d, 0x69, 0x63, 0x68, 0x65, 0x6c,
0x6f, 0x6e, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x20, 0x28, 0x4e, 0x6f,
0x72, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x29, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x20, 0x28, 0x4e, 0x6f, 0x72, 0x64, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x29, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x20, 0x28, 0x4e, 0x6f, 0x72, 0x64, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x29, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x50,
0x72, 0xec, 0x6e, 0x73, 0x69, 0x70, 0x65, 0x50, 0x6f, 0x72, 0x74, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x53, 0x70, 0x61, 0x67, 0x6e, 0x61, 0x53, 0x61,
0x6e, 0x20, 0x50, 0x6f, 0x6c, 0x6f, 0x53, 0x2e, 0x20, 0x42, 0x61, 0x72,
0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0xe8, 0x6f, 0x53, 0x2e, 0x20, 0x4a, 0x6f,
0x61, 0x6e, 0x69, 0x53, 0x2e, 0x20, 0x43, 0x72, 0x69, 0x73, 0x74, 0x6f,
0x66, 0x65, 0x72, 0x53, 0x2e, 0x20, 0x4c, 0x75, 0x73, 0xec, 0x61, 0x53,
0x2e, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x73, 0x6f, 0xcc, 0x7a,
0x6f, 0x6c, 0x61, 0x20, 0x4d, 0x61, 0x63, 0x71, 0x75, 0x61, 0x72, 0x69,
0x65, 0x41, 0x6e, 0xe0, 0x64, 0x79, 0x72, 0x41, 0x6b, 0x74, 0xe0, 0x75,
0x41, 0x7a, 0x67, 0x61, 0x62, 0x61, 0x64, 0x41, 0x74, 0x79, 0x72, 0xe0,
0x75, 0x43, 0x6f, 0x6c, 0x6f, 0x6e, 0x62, 0x6f, 0x44, 0x75, 0x62, 0xe0,
0x69, 0x44, 0x75, 0x73, 0x61, 0x6e, 0x62, 0xe9, 0x4b, 0x61, 0x74, 0x6d,
0x61, 0x6e, 0x64, 0xf9, 0x48, 0xe0, 0x6e, 0x64, 0x69, 0x67, 0x61, 0x4b,
0x72, 0x61, 0x7a, 0x6e, 0x61, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4b, 0x75,
0x61, 0x6c, 0x61, 0x20, 0x4c, 0x75, 0x6e, 0x70, 0x75, 0x72, 0x4d, 0x61,
0x63, 0xe0, 0x6f, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x69,
0x65, 0x74, 0x73, 0x6b, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0xe0, 0x69,
0x53, 0x68, 0x61, 0x6e, 0x67, 0x68, 0xe0, 0x69, 0x5a, 0x72, 0xe9, 0x64,
0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x69, 0x6d, 0x73, 0x6b, 0x54, 0x69, 0x6e,
0x70, 0x75, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0xe0, 0x74, 0x6f, 0x72,
0x55, 0x73, 0x74, 0x2d, 0x47, 0x6e, 0x65, 0x72, 0x61, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4a,
0xe8, 0x72, 0x65, 0x76, 0x61, 0x6e, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43,
0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x43, 0x61, 0x6f, 0x20, 0x56, 0x65,
0x72, 0x64, 0x6f, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0xe0, 0x72,
0x6f, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4d, 0x61, 0x64, 0xe8,
0x69, 0x72, 0x61, 0x52, 0x65, 0x6b, 0x69, 0x61, 0x76, 0x69, 0x6b, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x73, 0x75, 0x64, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x53, 0x2e, 0x20,
0x45, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x64, 0x65, 0x6c, 0xe0, 0x69, 0x64,
0x65, 0x42, 0x72, 0x69, 0x7a, 0x62, 0x61, 0x6e, 0x65, 0xcc, 0x7a, 0x6f,
0x6c, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0xc0, 0x6d, 0x73, 0x74, 0x65, 0x72, 0x64, 0x61, 0x6d, 0x42, 0x72, 0x61,
0x74, 0x69, 0x7a, 0x6c, 0x61, 0x76, 0x61, 0x4a, 0x69, 0x62, 0x69, 0x6c,
0x74, 0x65, 0x72, 0x61, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x75, 0x62, 0x6c, 0x69, 0x61, 0x6e, 0x61,
0x4c, 0x75, 0x73, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4f, 0x7a,
0x6c, 0x6f, 0x50, 0x61, 0x72, 0x69, 0x6a, 0x69, 0x53, 0x69, 0x6e, 0x66,
0x65, 0x72, 0xf2, 0x70, 0x6f, 0x6c, 0x69, 0x53, 0x74, 0x6f, 0x63, 0x6f,
0x6c, 0x6d, 0x61, 0x55, 0x6c, 0x69, 0xe0, 0x6e, 0x6f, 0x73, 0x6b, 0x56,
0x61, 0x74, 0x65, 0x67, 0x61, 0x6e, 0x43, 0x69, 0x61, 0x67, 0x6f, 0x73,
0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x64,
0x61, 0x6c, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x63,
0x6f, 0x73, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6d, 0x6f,
0x72, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x73, 0x69, 0x6f, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d,
0x61, 0x69, 0x6f, 0x74, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x52,
0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20,
0x42, 0x6f, 0x75, 0x67, 0x61, 0x69, 0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x69, 0x61, 0x74, 0x65, 0x6d,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63,
0x75, 0x61, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x45, 0x66, 0x61, 0x74,
0x65, 0x41, 0x74, 0x6f, 0x6c, 0x6f, 0x20, 0x46, 0x61, 0x6b, 0x61, 0x6f,
0x66, 0x6f, 0x41, 0x74, 0x6f, 0x6c, 0x6f, 0x20, 0x46, 0x75, 0x6e, 0x61,
0x66, 0x75, 0x74, 0x69, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x47,
0x75, 0x61, 0x64, 0x61, 0x6c, 0x63, 0x61, 0x6e, 0x61, 0x6c, 0x41, 0x74,
0x6f, 0x6c, 0x6f, 0x20, 0x43, 0x61, 0x6e, 0x74, 0x6f, 0x6e, 0x41, 0x74,
0x6f, 0x6c, 0x6f, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61,
0x74, 0x69, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x41, 0x74, 0x6f, 0x6c, 0x6f, 0x20, 0x4b, 0x77, 0x61, 0x6a,
0x61, 0x6c, 0x65, 0x69, 0x6e, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d,
0x61, 0x72, 0x63, 0x68, 0x65, 0x7a, 0x69, 0x41, 0x74, 0x6f, 0x6c, 0x6f,
0x20, 0x4d, 0x69, 0x64, 0x77, 0x61, 0x79, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20,
0x4e, 0x69, 0x75, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x50, 0x61, 0x6c, 0xe0, 0x75, 0xcc, 0x7a,
0x6f, 0x6c, 0x61, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x70, 0xe8, 0x69,
0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62,
0x79, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x52, 0x61, 0x72, 0x6f, 0x74,
0x6f, 0x6e, 0x67, 0x61, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x53, 0x61,
0x69, 0x70, 0xe0, 0x6e, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x54, 0x61,
0x69, 0x74, 0x69, 0x41, 0x74, 0x6f, 0x6c, 0x6c, 0x6f, 0x20, 0x54, 0x61,
0x72, 0x61, 0x77, 0x61, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x41, 0x74, 0x6f, 0x6c, 0x6f,
0x20, 0x57, 0x61, 0x6b, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x61, 0x62, 0x69, 0x64, 0x6a, 0x61, 0x6e,
0x65, 0x63, 0x72, 0x61, 0x61, 0x1e0d, 0x69, 0x73, 0x20, 0x61, 0x62, 0x61,
0x62, 0x61, 0x61, 0x6c, 0x6a, 0x12b, 0x79, 0x72, 0x73, 0x61, 0x73, 0x6d,
0x61, 0x72, 0x61, 0x62, 0x61, 0x6d, 0x61, 0x6b, 0x6f, 0x62, 0x61, 0x6e,
0x67, 0x75, 0x69, 0x62, 0x61, 0x6e, 0x6a, 0x75, 0x6c, 0x62, 0x69, 0x73,
0x73, 0x61, 0x75, 0x62, 0x6c, 0x61, 0x6e, 0x1e6d, 0x61, 0x65, 0x72, 0x62,
0x72, 0x61, 0x6a, 0x61, 0x76, 0x69, 0x6c, 0x6c, 0x62, 0x75, 0x6a, 0x75,
0x6d, 0x62, 0x75, 0x72, 0x61, 0x6b, 0x61, 0x69, 0x72, 0x6f, 0x6b, 0x61,
0x73, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x61, 0x73, 0x65, 0x75, 0x1e6d,
0x61, 0x6b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x12b, 0x1e0d, 0x6b, 0x61, 0x72,
0x64, 0x61, 0x72, 0x20, 0x65, 0x73, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x61,
0x6d, 0x64, 0x75, 0x61, 0x6c, 0x61, 0x65, 0x6c, 0x20, 0x61, 0x61, 0x69,
0x79, 0x75, 0x6e, 0x70, 0x72, 0x12b, 0x1e6d, 0x61, 0x75, 0x6e, 0x67, 0x61,
0x62, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x72, 0x61, 0x72, 0x65, 0x6a, 0x6f,
0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x6a, 0x75,
0x62, 0x61, 0x6b, 0x6d, 0x70, 0x61, 0x6c, 0x61, 0x6b, 0x61, 0x72, 0x1e6d,
0x6f, 0x75, 0x6d, 0x6b, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x6b, 0x69, 0x6e,
0x73, 0x61, 0x73, 0x61, 0x6c, 0x61, 0x67, 0x6f, 0x73, 0x6c, 0x69, 0x62,
0x72, 0x65, 0x76, 0x69, 0x6c, 0x6c, 0x6c, 0x75, 0x61, 0x6e, 0x1e0d, 0x61,
0x6c, 0x75, 0x62, 0x75, 0x6d, 0x62, 0x61, 0x73, 0x69, 0x6c, 0x75, 0x73,
0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6c, 0x61, 0x62, 0x6f, 0x6d, 0x61, 0x70,
0x75, 0x1e6d, 0x75, 0x6d, 0x61, 0x73, 0x65, 0x72, 0x75, 0x6d, 0x62, 0x61,
0x62, 0x61, 0x6e, 0x65, 0x6d, 0x6f, 0x67, 0x61, 0x1e0d, 0x69, 0x73, 0x75,
0x6d, 0x6f, 0x6e, 0x72, 0x6f, 0x76, 0x69, 0x61, 0x6e, 0x61, 0x69, 0x72,
0x6f, 0x62, 0x69, 0x6e, 0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x6e, 0x69,
0x79, 0x61, 0x6d, 0x65, 0x6e, 0x75, 0x65, 0x6b, 0x73, 0x61, 0x76, 0x61,
0x67, 0x64, 0x75, 0x67, 0x75, 0x70, 0x6f, 0x72, 0x1e6d, 0x6f, 0x2d, 0x1e47,
0x6f, 0x76, 0x6f, 0x73, 0x61, 0x6f, 0x20, 0x1e6d, 0x6f, 0x6d, 0x74, 0x72,
0x69, 0x70, 0x6f, 0x6c, 0x69, 0x1e6d, 0x75, 0x6e, 0x69, 0x73, 0x76, 0x69,
0x6e, 0x64, 0x68, 0x75, 0x6b, 0x61, 0x1e0d, 0x61, 0x6b, 0x61, 0x6e, 0x6b,
0x6f, 0x72, 0x61, 0x6a, 0x65, 0x6e, 0x67, 0x75, 0x69, 0x6c, 0x6c, 0x61,
0x65, 0x6e, 0x1e6d, 0x69, 0x67, 0x75, 0x61, 0x61, 0x72, 0x61, 0x67, 0x76,
0x61, 0x69, 0x6e, 0x61, 0x62, 0x75, 0x65, 0x6e, 0x6f, 0x75, 0x73, 0x20,
0x61, 0x69, 0x72, 0x65, 0x73, 0x6b, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x72,
0x6b, 0x61, 0x6b, 0x6f, 0x72, 0x1e0d, 0x61, 0x62, 0x61, 0x68, 0x75, 0x68,
0x75, 0x65, 0x6c, 0x61, 0x20, 0x72, 0x69, 0x6f, 0x6a, 0x61, 0x6d, 0x65,
0x6e, 0x1e0d, 0x6f, 0x6a, 0x61, 0x72, 0x69, 0x79, 0x6f, 0x20, 0x67, 0x61,
0x6c, 0x6c, 0x65, 0x67, 0x6f, 0x73, 0x73, 0x61, 0x6c, 0x1e6d, 0x61, 0x73,
0x61, 0x6e, 0x20, 0x68, 0x75, 0x61, 0x6e, 0x73, 0x61, 0x6e, 0x20, 0x6c,
0x75, 0x69, 0x73, 0x1e6d, 0x75, 0x6b, 0x16b, 0x6d, 0x6e, 0x75, 0x73, 0x76,
0x61, 0x69, 0x79, 0x61, 0x61, 0x72, 0x75, 0x62, 0x61, 0x65, 0x73, 0x75,
0x6e, 0x73, 0x69, 0x6f, 0x6e, 0x65, 0x1e6d, 0x69, 0x6b, 0x6f, 0x6b, 0x65,
0x6e, 0x62, 0x61, 0x68, 0x69, 0x61, 0x62, 0x61, 0x69, 0x61, 0x20, 0x62,
0x61, 0x6e, 0x1e0d, 0x65, 0x72, 0x61, 0x73, 0x62, 0x61, 0x72, 0x62, 0x61,
0x1e0d, 0x6f, 0x73, 0x62, 0x65, 0x6c, 0x65, 0x6d, 0x62, 0x65, 0x6c, 0x69,
0x6a, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x2d, 0x73, 0x61, 0x62, 0x6c, 0x6f,
0x6e, 0x62, 0x6f, 0x61, 0x20, 0x76, 0x69, 0x73, 0x74, 0x61, 0x62, 0x6f,
0x67, 0x6f, 0x1e6d, 0x61, 0x62, 0x6f, 0x69, 0x73, 0x12b, 0x6b, 0x65, 0x6d,
0x62, 0x72, 0x69, 0x6a, 0x20, 0x62, 0x65, 0x6b, 0x61, 0x6d, 0x70, 0x6f,
0x20, 0x67, 0x72, 0x61, 0x6e, 0x1e0d, 0x65, 0x6b, 0x61, 0x6e, 0x6b, 0x75,
0x6e, 0x6b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x73, 0x6b, 0x65, 0x79, 0x65,
0x6e, 0x6b, 0x65, 0x69, 0x6d, 0x61, 0x6e, 0x63, 0x69, 0x6b, 0x61, 0x67,
0x6f, 0x63, 0x68, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x6b, 0x6f,
0x73, 0x1e6d, 0x61, 0x20, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x72, 0x65, 0x73,
0x1e6d, 0x6f, 0x6e, 0x6b, 0x75, 0x61, 0x61, 0x62, 0x61, 0x6b, 0x79, 0x75,
0x72, 0x61, 0x73, 0x6f, 0x1e0d, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73,
0x61, 0x76, 0x6e, 0x1e0d, 0x61, 0x6f, 0x73, 0x6e, 0x1e0d, 0x61, 0x6f, 0x73,
0x6f, 0x6e, 0x20, 0x6b, 0x72, 0x12b, 0x6b, 0x1e0d, 0x65, 0x6e, 0x76, 0x65,
0x72, 0x1e0d, 0x65, 0x1e6d, 0x72, 0x6f, 0x69, 0x1e6d, 0x1e0d, 0x6f, 0x6d, 0x69,
0x6e, 0x69, 0x6b, 0x61, 0x65, 0x1e0d, 0x6d, 0x6f, 0x6e, 0x1e6d, 0x6f, 0x6e,
0x65, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x65, 0x6c, 0x20, 0x73,
0x61, 0x6c, 0x76, 0x61, 0x1e0d, 0x6f, 0x72, 0x70, 0x68, 0x6f, 0x72, 0x74,
0x20, 0x6e, 0x65, 0x6c, 0x73, 0x6e, 0x70, 0x6f, 0x72, 0x74, 0x61, 0x6c,
0x65, 0x6a, 0x61, 0x67, 0x6c, 0x61, 0x73, 0x20, 0x62, 0x61, 0x79, 0x67,
0x6f, 0x6f, 0x73, 0x20, 0x62, 0x65, 0x67, 0x72, 0x61, 0x6e, 0x1e0d, 0x20,
0x1e6d, 0x75, 0x72, 0x6b, 0x67, 0x72, 0x65, 0x6e, 0x61, 0x1e0d, 0x61, 0x67,
0x75, 0x61, 0x1e0d, 0x65, 0x6c, 0x6f, 0x75, 0x70, 0x65, 0x67, 0x75, 0x61,
0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x67, 0x75, 0x61, 0x6a, 0x61, 0x6b,
0x69, 0x6c, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x68, 0x65, 0x6c, 0x69,
0x70, 0x61, 0x6b, 0x73, 0x68, 0x61, 0x76, 0x61, 0x6e, 0x61, 0x69, 0x6e,
0x1e0d, 0x69, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x6e, 0x6f,
0x6b, 0x73, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61,
0x6d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d,
0x69, 0x79, 0x61, 0x6e, 0x61, 0x70, 0x69, 0x1e6d, 0x74, 0x72, 0x73, 0x62,
0x65, 0x72, 0x67, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e,
0x61, 0x1e6d, 0x65, 0x6c, 0x6c, 0x20, 0x73, 0x69, 0x74, 0x79, 0x2c, 0x20,
0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x76, 0x69, 0x76, 0x69,
0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x76, 0x69,
0x6e, 0x63, 0x65, 0x6e, 0x73, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79,
0x61, 0x6e, 0x61, 0x76, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20,
0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x69, 0x6e, 0x16b, 0x76,
0x69, 0x6b, 0x69, 0x6b, 0x61, 0x6c, 0x75, 0x69, 0x1e6d, 0x6a, 0x61, 0x6d,
0x61, 0x69, 0x6b, 0x61, 0x6a, 0x75, 0x6e, 0x6f, 0x76, 0x6c, 0x6f, 0x75,
0x69, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x6d, 0x6f, 0x6e, 0x1e6d, 0x69, 0x73,
0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x6b, 0x65, 0x6e, 0x1e6d, 0x75, 0x6b,
0x12b, 0x6b, 0x72, 0x65, 0x6c, 0x65, 0x6e, 0x1e0d, 0x65, 0x69, 0x6b, 0x6c,
0x61, 0x20, 0x70, 0x61, 0x6a, 0x6c, 0x6f, 0x73, 0x20, 0x61, 0x6e, 0x6a,
0x65, 0x6c, 0x65, 0x73, 0x6c, 0x6f, 0x76, 0x65, 0x72, 0x20, 0x70, 0x72,
0x69, 0x6e, 0x63, 0x2019, 0x73, 0x20, 0x6b, 0x75, 0x76, 0x61, 0x1e6d, 0x61,
0x72, 0x6d, 0x61, 0x73, 0x12b, 0x6f, 0x6d, 0x61, 0x6e, 0x61, 0x67, 0x75,
0x61, 0x6d, 0x61, 0x6e, 0x61, 0x75, 0x73, 0x6d, 0x61, 0x72, 0x69, 0x67,
0x6f, 0x1e6d, 0x6d, 0x61, 0x72, 0x1e6d, 0x69, 0x6e, 0x69, 0x6b, 0x6d, 0x61,
0x1e6d, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x73, 0x6d, 0x61, 0x73, 0x61, 0x1e6d,
0x6c, 0x61, 0x6e, 0x6d, 0x65, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x12b, 0x6d,
0x65, 0x72, 0x69, 0x1e0d, 0x61, 0x6d, 0x65, 0x1e6d, 0x6c, 0x61, 0x6b, 0x61,
0x1e6d, 0x6c, 0x61, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x63, 0x6f, 0x20, 0x73,
0x69, 0x1e6d, 0x79, 0x6d, 0x69, 0x6b, 0x76, 0x69, 0x6c, 0xf5, 0x6d, 0x6f,
0x6e, 0x6b, 0x1e6d, 0x6f, 0x6e, 0x6d, 0x6f, 0x6e, 0x1e6d, 0x65, 0x72, 0x72,
0x65, 0x6d, 0x6f, 0x6e, 0x1e6d, 0x65, 0x76, 0x69, 0x64, 0x69, 0x6f, 0x6d,
0x6f, 0x6e, 0x1e6d, 0x73, 0x65, 0x72, 0x72, 0x65, 0x1e6d, 0x6e, 0x61, 0x73,
0x61, 0x75, 0x6e, 0x69, 0x79, 0x75, 0x20, 0x79, 0x6f, 0x72, 0x6b, 0x6e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x62, 0x69, 0x79, 0x75, 0x6c, 0x61,
0x2c, 0x20, 0x75, 0x74, 0x74, 0x61, 0x72, 0x20, 0x1e0d, 0x61, 0x6b, 0x6f,
0x1e6d, 0x61, 0x6d, 0x61, 0x1e0d, 0x69, 0x6e, 0x12b, 0x20, 0x75, 0x74, 0x74,
0x61, 0x72, 0x20, 0x1e0d, 0x61, 0x6b, 0x6f, 0x1e6d, 0x61, 0x6e, 0x65, 0x75,
0x20, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x75, 0x74, 0x74, 0x61,
0x72, 0x20, 0x1e0d, 0x61, 0x6b, 0x6f, 0x1e6d, 0x61, 0x6e, 0x16b, 0x6b, 0x6f,
0x6a, 0x69, 0x6e, 0x61, 0x67, 0x61, 0x70, 0x61, 0x6e, 0x61, 0x6d, 0x61,
0x70, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x6f, 0x70, 0x69,
0x6e, 0x69, 0x6b, 0x73, 0x70, 0x6f, 0x72, 0x74, 0x2d, 0x61, 0x75, 0x2d,
0x70, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x70, 0x6f, 0x72, 0x1e6d, 0x20, 0x61,
0x70, 0x20, 0x73, 0x70, 0x61, 0x69, 0x6e, 0x70, 0x6f, 0x72, 0x1e6d, 0x70,
0x20, 0x76, 0x65, 0x6c, 0x68, 0x6f, 0x70, 0x75, 0x65, 0x72, 0x74, 0x6f,
0x20, 0x72, 0x69, 0x6b, 0x6f, 0x70, 0x75, 0x1e47, 0x1e6d, 0x61, 0x20, 0x65,
0x72, 0x65, 0x6e, 0x61, 0x73, 0x72, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20,
0x69, 0x6e, 0x6c, 0x65, 0x64, 0x1e6d, 0x72, 0x65, 0x73, 0x69, 0x70, 0x69,
0x72, 0x65, 0x6a, 0x69, 0x6e, 0x61, 0x72, 0x65, 0x6a, 0x61, 0x6c, 0x79,
0x75, 0x1e6d, 0x72, 0x69, 0x6f, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x6b, 0x6f,
0x73, 0x61, 0x74, 0x61, 0x72, 0x69, 0x73, 0x61, 0x6e, 0x1e6d, 0x69, 0x61,
0x67, 0x6f, 0x73, 0x65, 0x6e, 0x74, 0x6f, 0x20, 0x1e0d, 0x6f, 0x6d, 0x69,
0x6e, 0x67, 0x6f, 0x73, 0x61, 0x6f, 0x20, 0x70, 0x61, 0x75, 0x6c, 0x6f,
0x69, 0x1e6d, 0x6f, 0x6b, 0x6f, 0x1e6d, 0x6f, 0x72, 0x6d, 0x69, 0x1e6d, 0x73,
0x69, 0x74, 0x6b, 0x61, 0x73, 0x65, 0x6e, 0x1e6d, 0x20, 0x62, 0x61, 0x72,
0x74, 0x65, 0x6c, 0x65, 0x6d, 0x69, 0x73, 0x65, 0x6e, 0x1e6d, 0x20, 0x6a,
0x6f, 0x6e, 0x73, 0x73, 0x65, 0x6e, 0x74, 0x20, 0x6b, 0x69, 0x1e6d, 0x1e6d,
0x73, 0x73, 0x65, 0x6e, 0x74, 0x20, 0x6c, 0x75, 0x73, 0x69, 0x61, 0x73,
0x65, 0x6e, 0x1e6d, 0x20, 0x74, 0x6f, 0x6d, 0x61, 0x73, 0x73, 0x65, 0x6e,
0x1e6d, 0x20, 0x76, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x1e6d, 0x73, 0x76, 0x69,
0x70, 0x1e6d, 0x20, 0x6b, 0x75, 0x72, 0x72, 0x65, 0x6e, 0x1e6d, 0x74, 0x65,
0x67, 0x75, 0x74, 0x75, 0x6c, 0x65, 0x74, 0x69, 0x68, 0x76, 0x61, 0x6e,
0x61, 0x1e6d, 0x6f, 0x72, 0x6f, 0x6e, 0x1e6d, 0x6f, 0x1e6d, 0x6f, 0x72, 0x1e6d,
0x6f, 0x6c, 0x61, 0x76, 0x61, 0x6e, 0x6b, 0x16b, 0x76, 0x65, 0x72, 0x76,
0x68, 0x61, 0x69, 0x1e6d, 0x68, 0x6f, 0x72, 0x73, 0x76, 0x69, 0x6e, 0x6e,
0x69, 0x70, 0x65, 0x67, 0x79, 0x61, 0x6b, 0x75, 0x1e6d, 0x61, 0x1e6d, 0x6b,
0x65, 0x73, 0x65, 0x65, 0x1e0d, 0x65, 0x76, 0x69, 0x73, 0x1e0d, 0x79, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64, 0x65, 0x20, 0x75, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x6d, 0x65, 0x6b, 0x76, 0x61, 0x72, 0x69, 0x6d, 0x61, 0x76,
0x73, 0x6f, 0x6e, 0x6d, 0x65, 0x6b, 0x20, 0x6d, 0x75, 0x72, 0x1e0d, 0x6f,
0x70, 0x61, 0x6c, 0x6d, 0x65, 0x72, 0x72, 0x6f, 0x74, 0x65, 0x72, 0x61,
0x73, 0x79, 0x6f, 0x76, 0x61, 0x1e6d, 0x72, 0x6f, 0x6c, 0x6c, 0x76, 0x6f,
0x73, 0x1e6d, 0x6f, 0x6b, 0x6c, 0x6e, 0x67, 0x79, 0x61, 0x72, 0x62, 0x79,
0x65, 0x6e, 0x65, 0x1e0d, 0x65, 0x6e, 0x61, 0x6c, 0x6d, 0x61, 0x1e6d, 0x79,
0x61, 0x6d, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x1e0d, 0x69, 0x72, 0x61,
0x6b, 0x74, 0x61, 0x75, 0x61, 0x61, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x61,
0x73, 0x67, 0x61, 0x62, 0x61, 0x1e6d, 0x61, 0x74, 0x61, 0x72, 0x61, 0x75,
0x62, 0x61, 0x67, 0x64, 0x61, 0x64, 0x62, 0x61, 0x68, 0x72, 0x61, 0x69,
0x6e, 0x62, 0x61, 0x6b, 0x75, 0x62, 0x61, 0x6e, 0x67, 0x6b, 0x6f, 0x6b,
0x62, 0x61, 0x72, 0x6e, 0x61, 0x75, 0x6c, 0x62, 0x65, 0x69, 0x72, 0x75,
0x74, 0x62, 0x69, 0x73, 0x6b, 0x65, 0x6b, 0x62, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x63, 0x69, 0x74, 0x61, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x6f,
0x1e0d, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x75, 0x73, 0x1e0d, 0x69, 0x6c, 0x69,
0x64, 0x75, 0x62, 0x61, 0x69, 0x64, 0x75, 0x73, 0x61, 0x6d, 0x62, 0x65,
0x70, 0x61, 0x6d, 0x61, 0x67, 0x75, 0x73, 0x74, 0x61, 0x67, 0x61, 0x6a,
0x61, 0x68, 0x65, 0x62, 0x72, 0x6f, 0x6e, 0x68, 0x6f, 0x20, 0x63, 0x69,
0x20, 0x6d, 0x69, 0x6e, 0x68, 0x20, 0x73, 0x69, 0x74, 0x69, 0x68, 0x6f,
0x6e, 0x67, 0x20, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x72, 0x6b, 0x75, 0x1e6d,
0x73, 0x6b, 0x6a, 0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61, 0x6a, 0x65,
0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x6b, 0x61, 0x62, 0x75, 0x6c,
0x6b, 0x61, 0x6d, 0x63, 0x61, 0x1e6d, 0x6b, 0x61, 0x6b, 0x61, 0x72, 0x61,
0x63, 0x12b, 0x6b, 0x61, 0x1e6d, 0x6d, 0x61, 0x6e, 0x1e0d, 0x75, 0x6b, 0x61,
0x6e, 0x64, 0x69, 0x67, 0x61, 0x6b, 0x6f, 0x6c, 0x6b, 0x61, 0x74, 0x61,
0x6b, 0x72, 0x61, 0x73, 0x6e, 0x65, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x6b,
0x75, 0x61, 0x6c, 0x61, 0x20, 0x6c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x6b,
0x75, 0x63, 0x69, 0x6e, 0x67, 0x6b, 0x75, 0x76, 0x61, 0x69, 0x74, 0x6d,
0x6b, 0x61, 0x6f, 0x6d, 0x65, 0x67, 0x61, 0x1e0d, 0x61, 0x6e, 0x6d, 0x61,
0x6b, 0x61, 0x73, 0x61, 0x72, 0x6d, 0x61, 0x6e, 0x12b, 0x6c, 0x61, 0x6d,
0x61, 0x73, 0x6b, 0x61, 0x74, 0x6e, 0x69, 0x6b, 0x6f, 0x73, 0x69, 0x79,
0x61, 0x6e, 0x65, 0x76, 0x6f, 0x6b, 0x75, 0x6a, 0x6e, 0x65, 0x1e6d, 0x73,
0x6b, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x70, 0x6e, 0x6f, 0x6d, 0x20, 0x70, 0x65, 0x6e, 0x68, 0x70, 0x6f, 0x6e,
0x1e6d, 0x69, 0x61, 0x6e, 0x61, 0x6b, 0x70, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x6b, 0x6f, 0x73, 0x1e6d, 0x61, 0x6e, 0x65, 0x6b, 0x69,
0x6a, 0x75, 0x6f, 0x72, 0x64, 0x61, 0x72, 0x69, 0x79, 0x61, 0x64, 0x73,
0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x6b,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x6f, 0x6c, 0x73, 0x65, 0x6e, 0x67, 0x61,
0x69, 0x73, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x73, 0x72, 0x65,
0x1e0d, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x79, 0x6d, 0x73, 0x6b, 0x74, 0x61,
0x69, 0x70, 0x65, 0x69, 0x74, 0x61, 0x73, 0x6b, 0x65, 0x6e, 0x1e6d, 0x74,
0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x74, 0x65, 0x68, 0x72, 0x61, 0x6e,
0x74, 0x69, 0x6d, 0x70, 0x68, 0x75, 0x1e6d, 0x6f, 0x6b, 0x79, 0x6f, 0x1e6d,
0x6f, 0x6d, 0x73, 0x6b, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x1e6d, 0x61,
0x72, 0x75, 0x72, 0x75, 0x6d, 0x63, 0x69, 0x79, 0x75, 0x73, 0x74, 0x2d,
0x6e, 0x65, 0x72, 0x61, 0x76, 0x69, 0x65, 0x6e, 0x1e6d, 0x69, 0x61, 0x61,
0x6e, 0x76, 0x6c, 0x61, 0x1e0d, 0x69, 0x76, 0x6f, 0x73, 0x1e6d, 0x6f, 0x6b,
0x79, 0x61, 0x6b, 0x75, 0x1e6d, 0x73, 0x6b, 0x79, 0x61, 0x6e, 0x67, 0x6f,
0x6e, 0x79, 0x69, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x79, 0x65, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x61, 0x6a, 0x6f,
0x72, 0x65, 0x73, 0x62, 0x65, 0x72, 0x6d, 0x16b, 0x64, 0x61, 0x6b, 0x65,
0x6e, 0x65, 0x72, 0x69, 0x6b, 0x65, 0x70, 0x20, 0x76, 0x65, 0x72, 0x1e0d,
0x65, 0x70, 0x65, 0x72, 0x6f, 0x6d, 0x61, 0x1e0d, 0x69, 0x65, 0x72, 0x61,
0x72, 0x65, 0x79, 0x6b, 0x79, 0x61, 0x76, 0x69, 0x6b, 0x64, 0x6b, 0x69,
0x1e47, 0x20, 0x6a, 0x6f, 0x72, 0x6a, 0x69, 0x61, 0x73, 0x65, 0x6e, 0x74,
0x20, 0x68, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x73, 0x1e6d, 0x61, 0x6e, 0x6c,
0x69, 0x61, 0x65, 0x64, 0x69, 0x6c, 0x65, 0x69, 0x64, 0x62, 0x72, 0x69,
0x73, 0x62, 0x61, 0x6e, 0x65, 0x62, 0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x20,
0x68, 0x69, 0x6c, 0x6c, 0x1e0d, 0x61, 0x72, 0x76, 0x69, 0x6e, 0x79, 0x75,
0x6b, 0x6c, 0x61, 0x68, 0x6f, 0x62, 0x61, 0x72, 0x1e6d, 0x6c, 0x69, 0x6e,
0x1e0d, 0x65, 0x72, 0x6d, 0x61, 0x6e, 0x6c, 0x6f, 0x72, 0x1e0d, 0x20, 0x68,
0x6f, 0x76, 0x65, 0x6d, 0x65, 0x6c, 0x62, 0x6f, 0x72, 0x6e, 0x65, 0x70,
0x65, 0x72, 0x74, 0x73, 0x79, 0x1e0d, 0x6e, 0x12b, 0x61, 0x6d, 0x73, 0x1e6d,
0x65, 0x72, 0x1e0d, 0x61, 0x6d, 0x61, 0x61, 0x1e47, 0x1e0d, 0x6f, 0x72, 0x61,
0x61, 0x61, 0x73, 0x1e6d, 0x72, 0x61, 0x68, 0x61, 0x6e, 0x65, 0x74, 0x65,
0x6e, 0x73, 0x62, 0x65, 0x6c, 0x67, 0x72, 0x61, 0x1e0d, 0x65, 0x62, 0x65,
0x72, 0x6c, 0x69, 0x6e, 0x62, 0x72, 0x61, 0x1e6d, 0x69, 0x73, 0x6c, 0x61,
0x76, 0x61, 0x62, 0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x73, 0x62, 0x75,
0x6b, 0x61, 0x72, 0x65, 0x73, 0x74, 0x62, 0x75, 0x1e0d, 0x61, 0x70, 0x65,
0x73, 0x74, 0x62, 0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x6b, 0x69,
0x73, 0x69, 0x6e, 0x61, 0x75, 0x6b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61,
0x67, 0x65, 0x6e, 0x1e0d, 0x62, 0x6c, 0x69, 0x6e, 0x6a, 0x69, 0x62, 0x72,
0x61, 0x6c, 0x1e6d, 0x61, 0x72, 0x67, 0x65, 0x72, 0x6e, 0x73, 0x69, 0x68,
0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x61, 0x61, 0x69, 0x6c, 0x20,
0x61, 0x70, 0x20, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x62,
0x75, 0x6c, 0x6a, 0x65, 0x72, 0x73, 0x69, 0x6b, 0x61, 0x6c, 0x69, 0x6e,
0x69, 0x6e, 0x67, 0x72, 0x61, 0x1e0d, 0x6b, 0x69, 0x72, 0x6f, 0x76, 0x6b,
0x69, 0x79, 0x76, 0x6c, 0x69, 0x73, 0x62, 0x6f, 0x6e, 0x6c, 0x79, 0x75,
0x62, 0x6c, 0x79, 0x61, 0x6e, 0x61, 0x6c, 0x6e, 0x1e0d, 0x6e, 0x6c, 0x6b,
0x73, 0x65, 0x6d, 0x62, 0x72, 0x67, 0x6d, 0x65, 0x1e0d, 0x72, 0x69, 0x1e0d,
0x6d, 0x61, 0x6c, 0x1e6d, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x65, 0x68, 0x61,
0x6d, 0x6d, 0x69, 0x6e, 0x73, 0x6b, 0x6d, 0x6f, 0x6e, 0x61, 0x6b, 0x6f,
0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x6f, 0x73, 0x6c, 0x6f, 0x70, 0x61, 0x72,
0x69, 0x73, 0x70, 0x1e0d, 0x67, 0x6f, 0x72, 0x69, 0x74, 0x73, 0x61, 0x70,
0x72, 0x61, 0x67, 0x72, 0x69, 0x67, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x73, 0x61, 0x6e, 0x20, 0x6d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x73,
0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0x6f, 0x73, 0x61, 0x72, 0x61, 0x1e6d,
0x6f, 0x76, 0x73, 0x69, 0x6d, 0x70, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c,
0x73, 0x6b, 0x6f, 0x70, 0x69, 0x73, 0x6f, 0x70, 0x69, 0x79, 0x61, 0x73,
0x1e6d, 0x6f, 0x6b, 0x68, 0x6f, 0x6d, 0x74, 0x61, 0x6c, 0x69, 0x6e, 0x1e6d,
0x69, 0x72, 0x61, 0x6e, 0x65, 0x75, 0x6c, 0x79, 0x61, 0x6e, 0x6f, 0x76,
0x73, 0x6b, 0x76, 0x61, 0x1e0d, 0x75, 0x6a, 0x76, 0x61, 0x1e6d, 0x69, 0x6b,
0x61, 0x6e, 0x76, 0x69, 0x65, 0x6e, 0x6e, 0x61, 0x76, 0x69, 0x6c, 0x6e,
0x69, 0x75, 0x73, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x1e0d,
0x76, 0x61, 0x72, 0x73, 0x61, 0x74, 0x65, 0x67, 0x75, 0x73, 0x69, 0x67,
0x61, 0x6c, 0x70, 0x61, 0x6a, 0x75, 0x72, 0x69, 0x63, 0x1e6d, 0x61, 0x6e,
0x61, 0x6e, 0x61, 0x72, 0x69, 0x76, 0x63, 0x68, 0x61, 0x67, 0x6f, 0x73,
0x6b, 0x72, 0x69, 0x73, 0x1e6d, 0x6d, 0x61, 0x73, 0x6b, 0x6f, 0x6b, 0x6f,
0x73, 0x6b, 0x6f, 0x6d, 0x6f, 0x72, 0x6f, 0x6b, 0x65, 0x72, 0x67, 0x75,
0x65, 0x6c, 0x65, 0x6e, 0x6d, 0x61, 0x68, 0x65, 0x6d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x65, 0x73, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x75,
0x73, 0x6d, 0x61, 0x79, 0x6f, 0x1e6d, 0x72, 0x65, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x61, 0x70, 0x69, 0x61, 0x101, 0x6b, 0x6c, 0x61, 0x6e, 0x1e0d, 0x62,
0x6f, 0x75, 0x67, 0x61, 0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x63, 0x65, 0x1e6d,
0x61, 0x6d, 0x63, 0x16b, 0x6b, 0x12b, 0x73, 0x74, 0x65, 0x72, 0x69, 0x70,
0x65, 0x1e6d, 0x65, 0x70, 0x61, 0x6b, 0x61, 0x6f, 0x70, 0x6f, 0x70, 0x69,
0x6a, 0x69, 0x70, 0x75, 0x6e, 0x61, 0x70, 0x75, 0x74, 0x69, 0x67, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x67, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x67, 0x75, 0x61, 0x1e0d, 0x61, 0x6c, 0x6b, 0x65, 0x6e, 0x61,
0x6c, 0x6b, 0x61, 0x6e, 0x1e6d, 0x6f, 0x6e, 0x6b, 0x69, 0x72, 0x69, 0x74,
0x69, 0x6d, 0x61, 0x74, 0x69, 0x6b, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6b,
0x76, 0x61, 0x6a, 0x61, 0x6c, 0x65, 0x69, 0x6e, 0x6d, 0x61, 0x6a, 0x75,
0x72, 0x6f, 0x6d, 0x61, 0x72, 0x6b, 0x69, 0x73, 0x61, 0x73, 0x6d, 0x69,
0x1e0d, 0x76, 0x61, 0x79, 0x6e, 0x61, 0x75, 0x72, 0x75, 0x6e, 0x69, 0x75,
0x65, 0x6e, 0x6f, 0x72, 0x70, 0x6f, 0x6b, 0x6e, 0x75, 0x6d, 0x69, 0x65,
0x70, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x70, 0x61, 0x6e, 0x67, 0x6f, 0x70,
0x61, 0x6c, 0x61, 0x75, 0x70, 0x69, 0x1e6d, 0x6b, 0x65, 0x72, 0x6e, 0x70,
0x6f, 0x6e, 0x70, 0x65, 0x69, 0x70, 0x6f, 0x72, 0x1e6d, 0x20, 0x6d, 0x6f,
0x72, 0x65, 0x73, 0x62, 0x69, 0x72, 0x61, 0x72, 0x6f, 0x74, 0x6f, 0x6e,
0x67, 0x61, 0x73, 0x61, 0x69, 0x70, 0x61, 0x6e, 0x1e6d, 0x61, 0x68, 0x69,
0x1e6d, 0x69, 0x74, 0x61, 0x72, 0x61, 0x76, 0x61, 0x1e6d, 0x6f, 0x6e, 0x67,
0x61, 0x1e6d, 0x61, 0x70, 0x75, 0x76, 0x65, 0x6b, 0x76, 0x61, 0x6c, 0x6c,
0x69, 0x73, 0x92c, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x91c, 0x941, 0x932, 0x92a,
0x93c, 0x94d, 0x930, 0x940, 0x91f, 0x93e, 0x909, 0x928, 0x915, 0x93e, 0x930, 0x94d,
0x924, 0x941, 0x92e, 0x915, 0x93f, 0x902, 0x938, 0x93e, 0x938, 0x93e, 0x932, 0x941,
0x92c, 0x941, 0x92e, 0x92c, 0x93e, 0x938, 0x940, 0x92e, 0x94d, 0x2d, 0x92c, 0x93e,
0x92c, 0x93e, 0x928, 0x947, 0x92e, 0x94b, 0x917, 0x93e, 0x926, 0x93f, 0x938, 0x941,
0x928, 0x93e, 0x907, 0x930, 0x94b, 0x92c, 0x93f, 0x928, 0x941, 0x906, 0x915, 0x91a,
0x949, 0x91f, 0x938, 0x94d, 0x92f, 0x93e, 0x928, 0x20, 0x939, 0x94d, 0x935, 0x93e,
0x928, 0x938, 0x94d, 0x92f, 0x93e, 0x928, 0x20, 0x932, 0x942, 0x924, 0x93f, 0x909,
0x938, 0x941, 0x906, 0x907, 0x92f, 0x93e, 0x90f, 0x938, 0x928, 0x938, 0x93f, 0x92f,
0x949, 0x928, 0x92c, 0x94d, 0x932, 0x93e, 0x902, 0x2d, 0x938, 0x947, 0x92c, 0x932,
0x94b, 0x928, 0x915, 0x947, 0x92e, 0x94d, 0x92c, 0x94d, 0x930, 0x93f, 0x91c, 0x20,
0x92c, 0x947, 0x915, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92a, 0x94b, 0x20, 0x917,
0x94d, 0x930, 0x93e, 0x902, 0x921, 0x947, 0x915, 0x94d, 0x92f, 0x93e, 0x928, 0x915,
0x941, 0x928, 0x938, 0x93f, 0x915, 0x93e, 0x917, 0x94b, 0x921, 0x947, 0x928, 0x92e,
0x93e, 0x930, 0x94d, 0x915, 0x938, 0x949, 0x928, 0x924, 0x93f, 0x930, 0x941, 0x928,
0x947, 0x92a, 0x947, 0x92a, 0x93c, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x928, 0x947,
0x932, 0x94d, 0x938, 0x928, 0x92a, 0x93c, 0x94b, 0x930, 0x94d, 0x91f, 0x93e, 0x932,
0x947, 0x91c, 0x93c, 0x93e, 0x939, 0x947, 0x932, 0x93f, 0x92a, 0x947, 0x915, 0x94d,
0x938, 0x907, 0x915, 0x93e, 0x932, 0x941, 0x924, 0x93f, 0x91f, 0x915, 0x94d, 0x930,
0x93e, 0x932, 0x947, 0x928, 0x94d, 0x921, 0x93f, 0x91c, 0x93f, 0x915, 0x92e, 0x928,
0x94b, 0x938, 0x92e, 0x947, 0x930, 0x93f, 0x917, 0x94b, 0x91f, 0x92c, 0x94d, 0x92f,
0x942, 0x932, 0x93e, 0x2c, 0x20, 0x909, 0x924, 0x949, 0x930, 0x949, 0x20, 0x921,
0x915, 0x94b, 0x91f, 0x93e, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x909,
0x924, 0x949, 0x930, 0x949, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x928, 0x94d,
0x92f, 0x942, 0x20, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x909, 0x924,
0x949, 0x930, 0x949, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x913, 0x915, 0x93e,
0x91c, 0x940, 0x928, 0x93e, 0x917, 0x93e, 0x92a, 0x93c, 0x940, 0x928, 0x93f, 0x915,
0x94d, 0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x911, 0x92a, 0x93c, 0x20,
0x938, 0x94d, 0x92a, 0x947, 0x928, 0x930, 0x947, 0x938, 0x93e, 0x907, 0x92a, 0x93c,
0x938, 0x947, 0x928, 0x94d, 0x91f, 0x93e, 0x930, 0x947, 0x92e, 0x938, 0x947, 0x902,
0x924, 0x93f, 0x906, 0x917, 0x94b, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92c, 0x93e,
0x930, 0x94d, 0x924, 0x947, 0x932, 0x947, 0x92e, 0x93f, 0x938, 0x947, 0x902, 0x91f,
0x20, 0x924, 0x949, 0x92e, 0x938, 0x938, 0x94d, 0x935, 0x93f, 0x92a, 0x93c, 0x94d,
0x91f, 0x20, 0x915, 0x930, 0x902, 0x91f, 0x935, 0x94d, 0x92f, 0x93e, 0x928, 0x94d,
0x915, 0x942, 0x935, 0x930, 0x92e, 0x94d, 0x92f, 0x93e, 0x915, 0x92e, 0x941, 0x930,
0x94d, 0x921, 0x94b, 0x932, 0x949, 0x928, 0x94d, 0x917, 0x92f, 0x930, 0x92c, 0x94d,
0x92f, 0x947, 0x928, 0x905, 0x938, 0x94d, 0x917, 0x93e, 0x92c, 0x93e, 0x924, 0x92c,
0x94d, 0x92f, 0x93e, 0x902, 0x917, 0x915, 0x949, 0x915, 0x92c, 0x93f, 0x938, 0x94d,
0x915, 0x947, 0x915, 0x92c, 0x94d, 0x930, 0x942, 0x928, 0x947, 0x924, 0x93f, 0x924,
0x94d, 0x938, 0x93f, 0x924, 0x93e, 0x926, 0x941, 0x92c, 0x924, 0x93f, 0x926, 0x941,
0x938, 0x93e, 0x902, 0x92c, 0x947, 0x92a, 0x93c, 0x93e, 0x92e, 0x93e, 0x917, 0x941,
0x938, 0x94d, 0x924, 0x93e, 0x92f, 0x947, 0x930, 0x941, 0x938, 0x932, 0x947, 0x92e,
0x915, 0x93e, 0x921, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x928, 0x949, 0x92e, 0x20,
0x92a, 0x947, 0x928, 0x94d, 0x939, 0x915, 0x93f, 0x91c, 0x93f, 0x932, 0x949, 0x930,
0x94d, 0x921, 0x93e, 0x938, 0x93e, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x938, 0x902,
0x918, 0x93e, 0x924, 0x93f, 0x924, 0x93e, 0x924, 0x93f, 0x92a, 0x947, 0x924, 0x93f,
0x924, 0x93e, 0x938, 0x915, 0x902, 0x924, 0x91f, 0x94d, 0x2d, 0x92c, 0x93f, 0x932,
0x93f, 0x938, 0x93f, 0x92f, 0x93e, 0x902, 0x917, 0x949, 0x928, 0x915, 0x94d, 0x92f,
0x93e, 0x928, 0x947, 0x930, 0x940, 0x92a, 0x94d, 0x92f, 0x93e, 0x930, 0x94b, 0x926,
0x93e, 0x945, 0x915, 0x93f, 0x923, 0x93e, 0x945, 0x20, 0x91c, 0x93e, 0x945, 0x930,
0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x938, 0x94d, 0x91f, 0x94d, 0x92f, 0x93e, 0x928,
0x932, 0x940, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x93e, 0x915, 0x93e, 0x928,
0x90f, 0x924, 0x947, 0x928, 0x94d, 0x938, 0x906, 0x907, 0x932, 0x20, 0x911, 0x92a,
0x94d, 0x20, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x938, 0x93f, 0x92e, 0x94d, 0x92a,
0x93c, 0x947, 0x930, 0x94b, 0x92a, 0x94b, 0x932, 0x938, 0x94d, 0x915, 0x94b, 0x92a,
0x94d, 0x92f, 0x947, 0x938, 0x94b, 0x92a, 0x93c, 0x93f, 0x92f, 0x93e, 0x91c, 0x93c,
0x94d, 0x92f, 0x942, 0x930, 0x93f, 0x915, 0x93c, 0x92e, 0x949, 0x930, 0x940, 0x938,
0x938, 0x911, 0x915, 0x932, 0x947, 0x902, 0x921, 0x91a, 0x94d, 0x92f, 0x93e, 0x925,
0x92e, 0x924, 0x93f, 0x938, 0x94d, 0x91f, 0x930, 0x90f, 0x92a, 0x93c, 0x947, 0x91f,
0x92a, 0x93c, 0x93e, 0x915, 0x93e, 0x913, 0x92a, 0x93c, 0x94b, 0x92a, 0x93c, 0x93f,
0x91c, 0x940, 0x92a, 0x93c, 0x94d, 0x92f, 0x942, 0x928, 0x93e, 0x92a, 0x93c, 0x941,
0x91f, 0x940, 0x917, 0x94d, 0x92f, 0x93e, 0x92e, 0x92c, 0x93f, 0x92f, 0x930, 0x915,
0x947, 0x902, 0x91f, 0x928, 0x928, 0x93e, 0x945, 0x909, 0x930, 0x941, 0x928, 0x949,
0x930, 0x92a, 0x93c, 0x949, 0x915, 0x928, 0x949, 0x92e, 0x93f, 0x92f, 0x93e, 0x92a,
0x94b, 0x928, 0x92a, 0x947, 0x924, 0x93f, 0xb0f, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb2c,
0xb4d, 0xb30, 0xb3e, 0xb1c, 0xb3e, 0xb71, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb2c,
0xb41, 0xb1c, 0xb42, 0xb2e, 0xb4d, 0xb2c, 0xb41, 0xb30, 0xb3e, 0xb15, 0xb28, 0xb3e,
0xb15, 0xb4d, 0xb30, 0xb3f, 0xb1c, 0xb3f, 0xb2c, 0xb1f, 0xb3f, 0xb2a, 0xb4d, 0xb30,
0xb3f, 0xb1f, 0xb3e, 0xb09, 0xb28, 0xb4d, 0x200c, 0xb17, 0xb3e, 0xb2c, 0xb30, 0xb4d,
0xb23, 0xb4d, 0xb23, 0xb1c, 0xb39, 0xb3e, 0xb28, 0xb4d, 0xb38, 0xb2c, 0xb30, 0xb4d,
0xb17, 0xb15, 0xb30, 0xb1f, 0xb09, 0xb2e, 0xb4d, 0x200c, 0xb32, 0xb3e, 0xb17, 0xb38,
0xb4d, 0x200c, 0xb32, 0xb3f, 0xb2c, 0xb4d, 0xb30, 0xb47, 0xb71, 0xb3f, 0xb32, 0xb4d,
0xb32, 0xb47, 0xb32, 0xb2e, 0xb4d, 0x200c, 0xb2e, 0xb4d, 0x2d, 0xb2c, 0xb3e, 0xb2c,
0xb3e, 0xb28, 0xb47, 0xb2e, 0xb4b, 0xb17, 0xb3e, 0xb21, 0xb3f, 0xb38, 0xb41, 0xb2e,
0xb28, 0xb30, 0xb4b, 0xb2c, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb07, 0xb30, 0xb2c, 0xb3f,
0xb28, 0xb41, 0xb06, 0xb15, 0xb1a, 0xb1f, 0xb05, 0xb17, 0xb3e, 0xb21, 0xb17, 0xb41,
0xb2a, 0xb30, 0xb4d, 0xb1f, 0x2d, 0xb28, 0xb71, 0xb38, 0xb3e, 0xb05, 0x20, 0xb1f,
0xb2e, 0xb47, 0xb24, 0xb4d, 0xb30, 0xb3f, 0xb2a, 0xb32, 0xb3f, 0xb71, 0xb3f, 0xb23,
0xb4d, 0xb21, 0xb39, 0xb15, 0xb4d, 0xb2c, 0xb41, 0xb0f, 0xb28, 0xb38, 0xb4d, 0x20,
0xb06, 0xb07, 0xb30, 0xb47, 0xb38, 0xb4d, 0xb15, 0xb3e, 0xb3c, 0xb1f, 0xb3e, 0xb2e,
0xb3e, 0xb15, 0xb3e, 0xb01, 0xb15, 0xb4b, 0xb21, 0xb4b, 0xb2c, 0xb3e, 0xb1c, 0xb41,
0xb1c, 0xb4b, 0xb0f, 0xb32, 0xb3e, 0x20, 0xb30, 0xb3f, 0xb4d, 0xb05, 0xb1c, 0xb3e,
0xb2e, 0xb47, 0xb23, 0xb4d, 0xb21, 0xb1c, 0xb3e, 0xb30, 0xb3f, 0xb5f, 0xb4b, 0x20,
0xb17, 0xb3e, 0xb32, 0xb47, 0xb17, 0xb4b, 0xb38, 0xb38, 0xb3e, 0xb32, 0xb4d, 0xb1f,
0xb3e, 0xb38, 0xb3e, 0xb5f, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb71, 0xb3e, 0xb28, 0xb4d,
0xb38, 0xb5f, 0xb3e, 0xb28, 0x20, 0xb32, 0xb41, 0xb07, 0xb38, 0xb1f, 0xb4b, 0xb15,
0xb41, 0xb2e, 0xb28, 0xb09, 0xb38, 0xb41, 0xb06, 0xb07, 0xb5f, 0xb3e, 0xb05, 0xb30,
0xb41, 0xb2c, 0xb3e, 0xb2c, 0xb3e, 0xb30, 0xb2c, 0xb3e, 0xb21, 0xb38, 0xb2c, 0xb4d,
0xb32, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x2d, 0xb38, 0xb3e, 0xb2c, 0xb32, 0xb28, 0xb4d,
0xb2c, 0xb4b, 0xb06, 0x20, 0xb71, 0xb3f, 0xb38, 0xb4d, 0xb1f, 0xb3e, 0xb2c, 0xb17,
0xb1f, 0xb3e, 0xb2c, 0xb07, 0xb38, 0xb47, 0xb15, 0xb3e, 0xb2e, 0xb4d, 0xb2a, 0x20,
0xb17, 0xb4d, 0xb30, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb47, 0xb38, 0xb3f, 0xb15, 0xb3e,
0xb17, 0xb15, 0xb37, 0xb4d, 0xb1f, 0xb3e, 0x20, 0xb30, 0xb3f, 0xb15, 0xb3e, 0xb15,
0xb4d, 0xb30, 0xb47, 0xb38, 0xb4d, 0x200d, 0xb1f, 0xb28, 0xb21, 0xb47, 0xb28, 0xb71,
0xb3f, 0xb30, 0xb4d, 0xb21, 0xb47, 0xb1f, 0xb4d, 0xb30, 0xb07, 0xb1f, 0xb4d, 0xb21,
0xb2e, 0xb3f, 0xb28, 0xb3f, 0xb15, 0xb3e, 0xb0f, 0xb21, 0xb4d, 0x200d, 0xb2e, 0xb28,
0xb1f, 0xb28, 0xb4d, 0xb0f, 0xb32, 0xb4d, 0x20, 0xb38, 0xb3e, 0xb32, 0xb71, 0xb3e,
0xb21, 0xb4b, 0xb30, 0xb4d, 0xb2a, 0xb30, 0xb4d, 0xb1f, 0x20, 0xb28, 0xb47, 0xb32,
0xb38, 0xb28, 0xb4d, 0xb2a, 0xb30, 0xb4d, 0xb1f, 0xb32, 0xb47, 0xb1c, 0xb3e, 0xb17,
0xb41, 0xb06, 0xb21, 0xb47, 0xb32, 0xb09, 0xb2a, 0xb47, 0xb17, 0xb41, 0xb06, 0xb24,
0xb47, 0xb2e, 0xb3e, 0xb32, 0xb3e, 0xb39, 0xb3e, 0xb71, 0xb28, 0xb3e, 0xb39, 0xb47,
0xb30, 0xb2e, 0xb38, 0xb3f, 0xb32, 0xb4b, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06,
0xb28, 0xb3e, 0xb2a, 0xb32, 0xb3f, 0xb38, 0xb4d, 0xb2e, 0xb3e, 0xb30, 0xb47, 0xb28,
0xb17, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb71,
0xb47, 0xb71, 0xb3e, 0xb5f, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06,
0xb28, 0xb3e, 0xb07, 0xb28, 0xb41, 0xb71, 0xb3f, 0xb15, 0xb4d, 0xb32, 0xb09, 0xb07,
0xb38, 0xb71, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb2e, 0xb23, 0xb4d, 0xb1f, 0xb3f,
0xb38, 0xb47, 0xb32, 0x2c, 0x20, 0xb15, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb09, 0xb15,
0xb3f, 0xb15, 0xb47, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb32, 0xb47, 0xb23, 0xb4d, 0xb21,
0xb3f, 0xb1c, 0xb3f, 0xb15, 0xb2e, 0xb3e, 0xb38, 0xb3f, 0xb05, 0xb2e, 0xb3e, 0xb30,
0xb3f, 0xb17, 0xb1f, 0xb4d, 0xb2e, 0xb3e, 0xb1f, 0xb3e, 0xb2e, 0xb30, 0xb38, 0xb4d,
0xb2e, 0xb3e, 0xb1c, 0xb3e, 0xb1f, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb2e, 0xb3f, 0xb15,
0xb4d, 0xb35, 0xb47, 0xb32, 0xb28, 0xb4d, 0xb2e, 0xb3e, 0xb28, 0xb15, 0xb1f, 0xb28,
0xb4d, 0xb2e, 0xb28, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb0f, 0xb2e, 0xb23, 0xb4d, 0xb1f,
0xb47, 0xb2d, 0xb3f, 0xb21, 0xb3f, 0xb05, 0xb2e, 0xb28, 0xb1f, 0xb38, 0xb47, 0xb30,
0xb30, 0xb3e, 0xb1f, 0xb4d, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb5f, 0xb30, 0xb4d,
0xb15, 0xb4d, 0xb28, 0xb30, 0xb39, 0xb4d, 0xb28, 0xb2c, 0xb47, 0xb09, 0xb32, 0xb3e,
0xb39, 0x2c, 0x20, 0xb09, 0xb24, 0xb4d, 0xb24, 0xb30, 0x20, 0xb21, 0xb3e, 0xb15,
0xb1f, 0xb3e, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0x2c, 0x20, 0xb09,
0xb24, 0xb4d, 0xb24, 0xb30, 0x20, 0xb21, 0xb3e, 0xb15, 0xb1f, 0xb3e, 0xb28, 0xb4d,
0xb5f, 0xb41, 0x20, 0xb38, 0xb3e, 0xb32, 0xb47, 0xb2e, 0xb4d, 0x2c, 0x20, 0xb09,
0xb24, 0xb4d, 0xb24, 0xb30, 0x20, 0xb21, 0xb3e, 0xb15, 0xb1f, 0xb3e, 0xb05, 0xb1c,
0xb3f, 0xb28, 0xb3e, 0xb17, 0xb3e, 0xb2a, 0xb07, 0xb28, 0xb3f, 0xb15, 0xb4d, 0xb38,
0xb2a, 0xb30, 0xb4d, 0xb1f, 0x2d, 0xb0f, 0xb5f, 0xb41, 0x2d, 0xb2a, 0xb4d, 0xb30,
0xb3f, 0xb28, 0xb4d, 0x200d, 0xb38, 0xb2a, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x20, 0xb05,
0xb2b, 0xb4d, 0x20, 0xb38, 0xb4d, 0xb2a, 0xb47, 0xb28, 0xb4d, 0xb2a, 0xb41, 0xb0f,
0xb30, 0xb4d, 0xb24, 0x20, 0xb30, 0xb3f, 0xb15, 0xb30, 0xb47, 0xb38, 0xb3f, 0xb2a,
0xb3f, 0xb30, 0xb3f, 0xb5f, 0x20, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb19, 0xb4d, 0xb15,
0xb38, 0xb3e, 0xb28, 0xb4d, 0xb24, 0xb30, 0xb47, 0xb2e, 0xb4d, 0xb38, 0xb3e, 0xb23,
0xb4d, 0xb1f, 0x20, 0xb21, 0xb2e, 0xb3f, 0xb19, 0xb4d, 0xb17, 0xb07, 0xb1f, 0xb4d,
0xb1f, 0xb15, 0xb4d, 0xb35, 0xb30, 0xb1f, 0xb30, 0xb2e, 0xb3f, 0xb1f, 0xb4d, 0xb38,
0xb47, 0xb23, 0xb4d, 0xb1f, 0x20, 0xb2c, 0xb3e, 0xb30, 0xb4d, 0xb24, 0xb47, 0xb32,
0xb47, 0xb2e, 0xb3f, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x20, 0xb15, 0xb3f,
0xb1f, 0xb4d, 0x200d, 0xb38, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20,
0xb25, 0xb2e, 0xb3e, 0xb38, 0xb4d, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e,
0x20, 0xb71, 0xb3f, 0xb28, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0xb24, 0xb41,
0xb32, 0xb47, 0xb1f, 0xb30, 0xb23, 0xb4d, 0xb1f, 0xb1f, 0xb30, 0xb1f, 0xb32, 0xb3e,
0xb71, 0xb3e, 0xb19, 0xb4d, 0xb15, 0xb41, 0xb71, 0xb30, 0xb4d, 0xb15, 0xb47, 0xb38,
0xb3f, 0xb21, 0xb47, 0xb71, 0xb3f, 0xb38, 0xb4d, 0xb21, 0xb4d, 0xb5f, 0xb41, 0xb2e,
0xb3e, 0xb23, 0xb4d, 0xb1f, 0x20, 0xb21, 0xb3f, 0x20, 0xb09, 0xb30, 0xb71, 0xb3f,
0xb32, 0xb47, 0xb2e, 0xb15, 0xb4d, 0xb71, 0xb3e, 0xb30, 0xb3f, 0xb2e, 0xb3e, 0xb01,
0xb38, 0xb28, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb15, 0xb2e, 0xb41, 0xb30, 0xb4d, 0xb21,
0xb2a, 0xb3e, 0xb01, 0xb2e, 0xb30, 0xb30, 0xb4b, 0xb24, 0xb47, 0xb30, 0xb3e, 0xb1f,
0xb4b, 0xb32, 0xb4d, 0xb71, 0xb4b, 0xb38, 0xb4d, 0xb24, 0xb15, 0xb4b, 0xb33, 0xb19,
0xb4d, 0xb16, 0xb5f, 0xb3e, 0xb30, 0xb2c, 0xb47, 0xb28, 0xb06, 0xb15, 0xb4d, 0xb1f,
0xb2c, 0xb47, 0xb06, 0xb38, 0xb4d, 0x200d, 0xb17, 0xb3e, 0xb2c, 0xb1f, 0xb4d, 0xb2c,
0xb39, 0xb3e, 0xb30, 0xb47, 0xb28, 0xb2c, 0xb3f, 0xb38, 0xb15, 0xb47, 0xb15, 0xb4d,
0x200c, 0xb26, 0xb41, 0xb38, 0xb3e, 0xb28, 0xb2c, 0xb47, 0xb2a, 0xb3e, 0xb2e, 0xb3e,
0xb17, 0xb41, 0xb38, 0xb4d, 0xb1f, 0xb3e, 0xb39, 0x20, 0xb1a, 0xb3f, 0x20, 0xb2e,
0xb3f, 0xb28, 0xb4d, 0x200c, 0x20, 0xb38, 0xb3f, 0xb1f, 0xb3f, 0xb39, 0xb4b, 0xb71,
0xb21, 0xb4d, 0x200c, 0xb1c, 0xb3e, 0xb15, 0xb30, 0xb4d, 0xb24, 0xb3e, 0xb15, 0xb2c,
0xb41, 0xb32, 0xb4d, 0xb15, 0xb3e, 0xb1f, 0xb2e, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb41,
0xb15, 0xb3e, 0xb28, 0xb21, 0xb4d, 0xb5f, 0xb3e, 0xb17, 0xb3e, 0xb15, 0xb4d, 0xb30,
0xb3e, 0xb38, 0xb28, 0xb5f, 0xb3e, 0xb30, 0xb38, 0xb4d, 0xb15, 0xb28, 0xb3f, 0xb15,
0xb38, 0xb3f, 0xb06, 0xb28, 0xb71, 0xb15, 0xb41, 0xb1c, 0xb28, 0xb47, 0xb1f, 0xb38,
0xb4d, 0xb15, 0xb28, 0xb71, 0xb38, 0xb3f, 0xb2c, 0xb3f, 0xb30, 0xb38, 0xb4d, 0xb15,
0xb05, 0xb30, 0xb3e, 0xb32, 0xb4d, 0x200c, 0xb2a, 0xb28, 0xb2e, 0xb4d, 0x200c, 0x20,
0xb2a, 0xb47, 0xb28, 0xb39, 0xb2a, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb06, 0xb28, 0xb3e,
0xb15, 0xb4d, 0x200c, 0xb15, 0xb37, 0xb4d, 0xb1f, 0xb28, 0xb47, 0xb15, 0xb40, 0xb1c,
0xb3f, 0xb32, 0xb30, 0xb4d, 0xb21, 0xb3e, 0xb38, 0xb15, 0xb3e, 0xb32, 0xb3f, 0xb28,
0xb4d, 0xb38, 0xb3f, 0xb05, 0xb32, 0xb38, 0xb02, 0xb17, 0xb3e, 0xb07, 0xb38, 0xb4d,
0xb30, 0xb47, 0xb21, 0xb28, 0xb47, 0xb15, 0xb32, 0xb5f, 0xb2e, 0xb38, 0xb4d, 0xb15,
0xb24, 0xb3e, 0xb38, 0xb15, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb24, 0xb3f, 0xb2e, 0xb4d,
0xb2a, 0xb41, 0xb1f, 0xb15, 0xb3f, 0xb05, 0xb1f, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb5f,
0xb41, 0xb38, 0xb4d, 0x200d, 0xb1f, 0x2d, 0xb28, 0xb47, 0xb30, 0xb3e, 0xb71, 0xb3f,
0xb0f, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb0f, 0xb28, 0xb4d, 0x200c, 0xb71, 0xb4d, 0xb32,
0xb3e, 0xb21, 0xb3f, 0xb71, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0xb5f, 0xb47,
0xb30, 0xb47, 0xb2c, 0xb3e, 0xb28, 0xb4d, 0xb06, 0xb1c, 0xb30, 0xb47, 0xb38, 0xb4d,
0xb2c, 0xb30, 0xb2e, 0xb41, 0xb21, 0xb3e, 0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c, 0x20,
0xb71, 0xb30, 0xb4d, 0xb26, 0xb47, 0xb2a, 0xb30, 0xb0f, 0xb30, 0xb47, 0xb15, 0xb4d,
0xb5f, 0xb3e, 0xb2c, 0xb3f, 0xb15, 0xb38, 0xb4d, 0x200d, 0xb1f, 0xb3e, 0xb32, 0xb3f,
0xb28, 0xb0f, 0xb21, 0xb3f, 0xb32, 0xb47, 0xb21, 0xb4d, 0xb2c, 0xb4d, 0xb30, 0xb3f,
0xb38, 0xb2c, 0xb28, 0xb4d, 0xb2c, 0xb4d, 0xb30, 0xb4b, 0xb15, 0xb28, 0x20, 0xb39,
0xb3f, 0xb32, 0xb21, 0xb3e, 0xb30, 0xb4d, 0xb71, 0xb3f, 0xb28, 0xb4d, 0xb32, 0xb3f,
0xb23, 0xb4d, 0xb21, 0xb47, 0xb2e, 0xb3e, 0xb28, 0xb32, 0xb30, 0xb4d, 0x200d, 0xb21,
0x20, 0xb39, 0xb3e, 0xb71, 0xb47, 0xb2e, 0xb47, 0xb32, 0xb2c, 0xb4b, 0xb28, 0xb01,
0xb2a, 0xb30, 0xb4d, 0xb24, 0xb06, 0xb2e, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47,
0xb21, 0xb3e, 0xb2e, 0xb4d, 0xb06, 0xb23, 0xb4d, 0xb21, 0xb30, 0xb3e, 0xb06, 0xb38,
0xb4d, 0x200d, 0xb1f, 0xb30, 0xb3e, 0xb15, 0xb3e, 0xb28, 0xb0f, 0xb24, 0xb47, 0xb28,
0xb4d, 0xb38, 0xb2c, 0xb47, 0xb32, 0xb17, 0xb4d, 0xb30, 0xb47, 0xb21, 0xb47, 0xb2c,
0xb4d, 0xb30, 0xb3e, 0xb1f, 0xb3f, 0xb38, 0xb32, 0xb3e, 0xb71, 0xb3e, 0xb2c, 0xb4d,
0xb30, 0xb41, 0xb38, 0xb3f, 0xb32, 0xb4d, 0x200d, 0xb38, 0xb2c, 0xb41, 0xb1a, 0xb3e,
0xb30, 0xb47, 0xb38, 0xb4d, 0xb1f, 0xb2c, 0xb41, 0xb21, 0xb3e, 0xb2a, 0xb47, 0xb38,
0xb4d, 0xb1f, 0xb1a, 0xb3f, 0xb38, 0xb3f, 0xb28, 0xb3e, 0xb09, 0xb15, 0xb2a, 0xb47,
0xb28, 0xb39, 0xb3e, 0xb17, 0xb47, 0xb28, 0xb4d, 0xb06, 0xb07, 0xb32, 0xb4d, 0x20,
0xb05, 0xb2a, 0xb4d, 0x20, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb28, 0xb4d, 0xb15, 0xb3f,
0xb30, 0xb71, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb5f, 0xb3e, 0xb39, 0xb47, 0xb2e, 0xb2e,
0xb28, 0xb3e, 0xb15, 0xb05, 0xb38, 0xb32, 0xb4b, 0xb2a, 0xb21, 0xb17, 0xb30, 0xb3f,
0xb15, 0xb3e, 0xb30, 0xb2e, 0xb4d, 0xb38, 0xb3e, 0xb30, 0xb3e, 0xb1c, 0xb47, 0xb2c,
0xb38, 0xb3e, 0xb30, 0xb3e, 0xb1f, 0xb71, 0xb4d, 0xb38, 0xb3f, 0xb2e, 0xb2b, 0xb47,
0xb30, 0xb2a, 0xb32, 0xb4d, 0xb38, 0xb4d, 0xb15, 0xb2a, 0xb5f, 0xb47, 0xb38, 0xb2a,
0xb3f, 0xb5f, 0xb3e, 0xb38, 0xb4d, 0xb1f, 0xb15, 0xb4d, 0x20, 0xb39, 0xb2e, 0xb4d,
0x200c, 0xb1f, 0xb3e, 0xb07, 0xb30, 0xb47, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb32, 0xb5f,
0xb3e, 0xb28, 0xb71, 0xb38, 0xb4d, 0xb15, 0xb2c, 0xb3e, 0xb21, 0xb41, 0xb1c, 0xb71,
0xb3e, 0xb1f, 0xb3f, 0xb15, 0xb3e, 0xb28, 0xb4d, 0xb71, 0xb3f, 0xb0f, 0xb28, 0xb3e,
0xb71, 0xb3f, 0xb32, 0xb28, 0xb3f, 0xb09, 0xb38, 0xb4d, 0xb71, 0xb32, 0xb17, 0xb17,
0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d, 0xb16, 0xb4d, 0xb30, 0xb40, 0xb38, 0xb4d, 0x200d,
0xb1f, 0x20, 0xb2e, 0xb3e, 0xb38, 0xb15, 0xb15, 0xb38, 0xb4d, 0x200c, 0xb15, 0xb2e,
0xb30, 0xb15, 0xb47, 0xb30, 0xb17, 0xb41, 0xb32, 0xb47, 0xb28, 0xb2e, 0xb30, 0xb3f,
0xb38, 0xb38, 0xb4d, 0xb2e, 0xb3e, 0xb5f, 0xb1f, 0xb47, 0xb2c, 0xb17, 0xb47, 0xb28,
0xb4d, 0x200c, 0xb71, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb1a, 0xb3e, 0xb24, 0xb3e,
0xb2e, 0xb4d, 0x200c, 0xb2a, 0xb15, 0xb3e, 0xb05, 0xb2a, 0xb2a, 0xb3f, 0xb1c, 0xb3f,
0xb17, 0xb3e, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb17, 0xb38, 0xb15, 0xb47, 0xb23, 0xb4d,
0xb1f, 0xb28, 0xb28, 0xb30, 0xb2a, 0xb15, 0xb4d, 0x200c, 0xb28, 0xb09, 0xb2e, 0xb3f,
0xb5f, 0xb2a, 0xb39, 0xb28, 0xb2a, 0xb47, 0xb07, 0xb2a, 0xb30, 0xb4d, 0xb1f, 0xb4d,
0x200c, 0x20, 0xb2e, 0xb30, 0xb47, 0xb38, 0xb2c, 0xb3f, 0xb30, 0xb3e, 0xb30, 0xb1f,
0xb19, 0xb4d, 0xb17, 0xb3e, 0xb1f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb1f, 0xb3e, 0xb2a,
0xb41, 0xc0e, 0xc02, 0xc17, 0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc3e, 0x3c,
0x65, 0x78, 0x65, 0x6d, 0x70, 0x6c, 0x61, 0x72, 0x43, 0x69, 0x74, 0x79,
0x3e, 0xc05, 0xc21, 0xc46, 0xc32, 0xc48, 0xc21, 0xc4d, 0x3c, 0x2f, 0x65, 0x78,
0x65, 0x6d, 0x70, 0x6c, 0x61, 0x72, 0x43, 0x69, 0x74, 0x79, 0x3e, 0xc15,
0xc3e, 0xc02, 0xc1f, 0xc28, 0xc4d, 0x140a, 0x144e, 0x1426, 0x146f, 0x1472, 0x1423, 0x1401,
0x141f, 0x14aa, 0x1423, 0x1450, 0x1423, 0x1403, 0x14c4, 0x1431, 0x1420, 0x1403, 0x1473, 0x14eb,
0x1405, 0x1403, 0x141f, 0x14f4, 0x1422, 0x1473, 0x141f, 0x1489, 0x1418, 0x1423, 0x1411, 0x14c2,
0x142f, 0x1420
};

static inline constexpr char16_t shortZoneNameTable[] = {
0x55, 0x54, 0x43, 0x48, 0x53, 0x54, 0x48, 0x44, 0x54, 0x41f, 0x422, 0x41a,
0x412, 0x20, 0x28, 0x55, 0x54, 0x43, 0x29, 0x413, 0x430, 0x432, 0x430, 0x439,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x432, 0x430, 0x439, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x2205, 0x2205, 0x2205, 0x49, 0x53, 0x54, 0x42, 0x53, 0x54, 0x54,
0x55, 0x48, 0x54, 0xd83a, 0xdd11, 0xd83a, 0xdd16, 0xd83a, 0xdd0a, 0xd83a, 0xdd11, 0xd83a,
0xdd16, 0xd83a, 0xdd16, 0xd83a, 0xdd11, 0xd83a, 0xdd15, 0xd83a, 0xdd16, 0x54, 0x53, 0xc8,
0x20, 0x28, 0xc8, 0x69, 0x72, 0x69, 0x6e, 0x6e, 0x29, 0x54, 0x53, 0x42,
0x90f, 0x91a, 0x90f, 0x938, 0x91f, 0x940, 0x41, 0x43, 0xc9, 0x41, 0x53, 0x42,
0x53, 0x47, 0x48, 0x54, 0x43, 0x55, 0x4f, 0x49, 0x48, 0x4f, 0x4c, 0x48,
0x645, 0x20, 0x639, 0x20, 0x648, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x55, 0x54, 0x43, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75,
0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x6f,
0x6c, 0x75, 0x6c, 0x75, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x54, 0x4d, 0x54, 0x48, 0x54, 0x54, 0x48, 0x54, 0x4c, 0x47, 0x69,
0x1edd, 0x20, 0x48, 0x53, 0x54, 0x189, 0x41, 0x4b
};

static inline constexpr char16_t longZoneNameTable[] = {
0x43, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x65, 0x64, 0x20,
0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x49, 0x72, 0x69, 0x73, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x72, 0x69,
0x74, 0x69, 0x73, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x410, 0x438, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x20,
0x410, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x410,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43d, 0x433, 0x43b, 0x438, 0x430, 0x20, 0x410,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x430, 0x43c, 0x4ad,
0x430, 0x47, 0x65, 0x6b, 0x6f, 0xf6, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x65,
0x72, 0x64, 0x65, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x65,
0x6c, 0x65, 0x20, 0x74, 0x79, 0x64, 0x49, 0x65, 0x72, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x42, 0x72, 0x69, 0x74, 0x73, 0x65, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x41, 0x6d, 0x61, 0x6e, 0x73, 0x61, 0x6e, 0x20, 0x4b,
0x254, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x25b, 0x64, 0x65, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x49, 0x72, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x66, 0x6f, 0x254,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x49, 0x6e, 0x67, 0x72, 0x65, 0x73, 0x69, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4f, 0x72, 0x61, 0x20, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x65, 0x20, 0x65, 0x20, 0x6b,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x75, 0x61, 0x72, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x72, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x62, 0x72, 0x69,
0x74, 0x61, 0x6e, 0x69, 0x6b, 0x65, 0x12e8, 0x1270, 0x1240, 0x1290, 0x1263, 0x1260,
0x1228, 0x20, 0x1201, 0x1208, 0x1308, 0x1265, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0,
0x12ed, 0x122a, 0x123d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1265, 0x122a, 0x1275, 0x123d, 0x20,
0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x639,
0x627, 0x644, 0x645, 0x64a, 0x20, 0x627, 0x644, 0x645, 0x646, 0x633, 0x642, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x64a, 0x631, 0x644, 0x646, 0x62f, 0x627,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x628, 0x631, 0x64a, 0x637, 0x627, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x74, 0x69, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x75,
0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f,
0x72, 0x64, 0x65, 0x6e, 0x61, 0x64, 0x6f, 0x540, 0x561, 0x574, 0x561, 0x577,
0x56d, 0x561, 0x580, 0x570, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56f, 0x578, 0x578,
0x580, 0x564, 0x56b, 0x576, 0x561, 0x581, 0x57e, 0x561, 0x56e, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x57c, 0x56c, 0x561, 0x576, 0x564, 0x561,
0x56f, 0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x580, 0x56b, 0x57f,
0x561, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x9b8,
0x9ae, 0x9a8, 0x9cd, 0x9ac, 0x9bf, 0x9a4, 0x20, 0x9b8, 0x9be, 0x9f0, 0x9cd, 0x9ac,
0x99c, 0x9a8, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x987, 0x9f0,
0x9bf, 0x99a, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac,
0x9cd, 0x9f0, 0x9bf, 0x99f, 0x9bf, 0x99b, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x61, 0x64, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0x63, 0x61, 0x4b,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x73, 0x69, 0x79, 0x61, 0x20,
0x65, 0x64, 0x69, 0x6c, 0x6d, 0x69, 0x15f, 0x20, 0xfc, 0x6d, 0x75, 0x6d,
0x64, 0xfc, 0x6e, 0x79, 0x61, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x130,
0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x72, 0x69, 0x74, 0x61, 0x6e,
0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x9b8, 0x9cd, 0x9a5, 0x9be, 0x9a8, 0x9be, 0x982, 0x995, 0x9bf, 0x9a4, 0x20,
0x986, 0x9a8, 0x9cd, 0x9a4, 0x9b0, 0x9cd, 0x99c, 0x9be, 0x9a4, 0x9bf, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x987, 0x9b0, 0x9bf, 0x9b6, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9b0, 0x9bf, 0x99f,
0x9bf, 0x9b6, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x75, 0x6e, 0x69, 0x62, 0x65, 0x72, 0x74, 0x73, 0x61, 0x6c, 0x20,
0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x75, 0x61, 0x49,
0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x4c, 0x6f, 0x6e, 0x64, 0x72, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x423, 0x43d, 0x456,
0x432, 0x435, 0x440, 0x441, 0x430, 0x43b, 0x44c, 0x43d, 0x44b, 0x20, 0x43a, 0x430,
0x430, 0x440, 0x434, 0x44b, 0x43d, 0x430, 0x432, 0x430, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x406, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x411, 0x440, 0x44b, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x917, 0x94b, 0x930,
0x94b, 0x92c, 0x939, 0x94b, 0x928, 0x93e, 0x92f, 0x20, 0x92e, 0x941, 0x932, 0x941,
0x917, 0x928, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x906, 0x908, 0x930, 0x940, 0x936,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92c,
0x94d, 0x930, 0x940, 0x91f, 0x940, 0x936, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x4b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x69, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x73, 0x76, 0x6a, 0x65, 0x74, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72,
0x69, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41a, 0x43e,
0x43e, 0x440, 0x434, 0x438, 0x43d, 0x438, 0x441, 0x430, 0x43d, 0x43e, 0x20, 0x443,
0x43d, 0x438, 0x432, 0x435, 0x440, 0x437, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x61, 0x6d, 0x7a, 0x65, 0x72, 0x20, 0x68,
0x6f, 0x6c, 0x6c, 0x76, 0x65, 0x64, 0x65, 0x6c, 0x20, 0x6b, 0x65, 0x6e,
0x75, 0x72, 0x7a, 0x68, 0x69, 0x65, 0x74, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x77, 0x65, 0x72, 0x7a,
0x68, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x42, 0x72, 0x65, 0x69, 0x7a, 0x68, 0x2d, 0x56, 0x65, 0x75, 0x72, 0x41a,
0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x438, 0x440, 0x430, 0x43d, 0x43e, 0x20,
0x443, 0x43d, 0x438, 0x432, 0x435, 0x440, 0x441, 0x430, 0x43b, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x100a,
0x103e, 0x102d, 0x1011, 0x102c, 0x1038, 0x101e, 0x100a, 0x1037, 0x103a, 0x20, 0x1000, 0x1019,
0x1039, 0x1018, 0x102c, 0x1037, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x101b, 0x1005,
0x103a, 0x101b, 0x103e, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x103c, 0x102d, 0x1010, 0x102d, 0x1014, 0x103a, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x5354, 0x8abf, 0x4e16, 0x754c, 0x6642, 0x9593, 0x611b, 0x723e, 0x862d, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x82f1, 0x570b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x534f, 0x8c03, 0x4e16,
0x754c, 0x65f6, 0x95f4, 0x7231, 0x5c14, 0x5170, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x82f1, 0x56fd,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x54, 0x65, 0x6d, 0x70, 0x73, 0x20, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72,
0x64, 0x69, 0x6e, 0x61, 0x74, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe0, 0x6e,
0x69, 0x63, 0x61, 0xd804, 0xdd18, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd34,
0xd804, 0xdd17, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2d, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804,
0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x418, 0x440, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x43d,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x13a2, 0x13a9, 0x13e0, 0x13f1, 0x20, 0x13c2, 0x13a6, 0x13d3, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a8, 0x13b5, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c8, 0x13d7, 0x13cf, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x4e16, 0x754c, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x432, 0x441, 0x435, 0x43c, 0x456, 0x301, 0x440, 0x43d, 0x43e, 0x435, 0x20,
0x441, 0x477, 0x433, 0x445, 0x440, 0x43e, 0x43d, 0x457, 0x437, 0x438, 0x301, 0x440,
0x43e, 0x432, 0x430, 0x43d, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x41f, 0x4d7, 0x442, 0x4d7, 0x43c, 0x20, 0x442, 0x4d7, 0x43d, 0x447,
0x435, 0x440, 0x438, 0x20, 0x43a, 0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x430,
0x446, 0x438, 0x43b, 0x435, 0x43d, 0x435, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b, 0x4a, 0x72,
0x75, 0xdf, 0x62, 0x72, 0x65, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x6a, 0x65,
0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x69, 0x63, 0x6b, 0x54, 0x65, 0x6d, 0x70, 0x75, 0x20, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x65, 0x20, 0x63, 0x75, 0x75,
0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x75, 0x6b, 0x6f, 0x6f, 0x72, 0x64,
0x69, 0x6e, 0x69, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x73, 0x76, 0x6a, 0x65,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x6f, 0x76, 0x61, 0x6e, 0xfd,
0x20, 0x73, 0x76, 0x11b, 0x74, 0x6f, 0x76, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x69, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x62, 0x72, 0x69, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f, 0x6f, 0x72,
0x64, 0x69, 0x6e, 0x65, 0x72, 0x65, 0x74, 0x20, 0x75, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x72, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x42, 0x72,
0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x924, 0x93e, 0x932, 0x92e, 0x947, 0x932, 0x940, 0x20, 0x906,
0x932, 0x92e, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x906, 0x92f, 0x930, 0x93f,
0x936, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x92c,
0x94d, 0x930, 0x93f, 0x91f, 0x93f, 0x936, 0x20, 0x917, 0x930, 0x94d, 0x92e, 0x93f,
0x92f, 0x947, 0x902, 0x20, 0x926, 0x93e, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x67,
0x65, 0x63, 0x6f, 0xf6, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x65, 0x72, 0x64,
0x65, 0x20, 0x77, 0x65, 0x72, 0x65, 0x6c, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x49, 0x65, 0x72, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x72, 0x69, 0x74, 0x73,
0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0xf68,
0xf71, 0xf0b, 0xf61, 0xf62, 0xf72, 0xf64, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb2,
0xf71, 0xf72, 0xf0b, 0xf4a, 0xf72, 0xf64, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x20, 0x6b, 0x75, 0x6e, 0x6f, 0x72, 0x64, 0x69, 0x67, 0x69,
0x74, 0x61, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x62, 0x72, 0x69,
0x74, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x4b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x65,
0x72, 0x69, 0x74, 0x75, 0x64, 0x20, 0x6d, 0x61, 0x61, 0x69, 0x6c, 0x6d,
0x61, 0x61, 0x65, 0x67, 0x49, 0x69, 0x72, 0x69, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x42, 0x72, 0x69, 0x74, 0x69, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x58, 0x65, 0x78, 0x65, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69,
0x20, 0x6d, 0x65, 0x49, 0x72, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x68, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x73, 0x6b, 0x69,
0x70, 0x61, 0xf0, 0x20, 0x68, 0x65, 0x69, 0x6d, 0x73, 0x74, 0xed, 0xf0,
0xcd, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x42, 0x72, 0x65, 0x74, 0x73, 0x6b, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x49, 0x72, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x42, 0x72, 0x69, 0x74, 0x61, 0x69,
0x6e, 0x55, 0x54, 0x43, 0x2d, 0x79, 0x6c, 0x65, 0x69, 0x73, 0x61, 0x69,
0x6b, 0x61, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x72, 0x69, 0x74, 0x61,
0x6e, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x74, 0x65, 0x6d, 0x70, 0x73, 0x20, 0x75, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x65, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x6f,
0x6e, 0x6e, 0xe9, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x69, 0x73,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x71, 0x75, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x69, 0x73, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x71, 0x75,
0x65, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd46,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd46, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xc0, 0x6d, 0x20, 0x55, 0x69,
0x6c, 0x65, 0x2d, 0x63, 0x68, 0x6f, 0x69, 0x74, 0x63, 0x68, 0x65, 0x61,
0x6e, 0x6e, 0x20, 0x43, 0x6f, 0x2d, 0xf2, 0x72, 0x64, 0x61, 0x6e, 0x61,
0x69, 0x63, 0x68, 0x74, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x2d, 0xc8, 0x69, 0x72, 0x65, 0x61, 0x6e, 0x6e,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x42, 0x68, 0x72, 0x65, 0x61, 0x74, 0x61, 0x69, 0x6e,
0x6e, 0x42, 0x65, 0x20, 0x4e, 0x69, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x20,
0x4e, 0x69, 0x20, 0x59, 0x254, 0x254, 0x20, 0x4a, 0x65, 0x14b, 0x20, 0x46,
0x25b, 0x25b, 0x20, 0x4b, 0x25b, 0x74, 0x73, 0x75, 0x254, 0x20, 0x4e, 0x69,
0x69, 0x42, 0x65, 0x20, 0x4e, 0x69, 0x20, 0x49, 0x72, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x62, 0x69, 0x69, 0x20, 0x4b, 0x25b, 0x74, 0x73, 0x75, 0x254,
0x20, 0x4e, 0x69, 0x69, 0x42, 0x65, 0x20, 0x4e, 0x69, 0x20, 0x42, 0x72,
0x69, 0x74, 0x61, 0x69, 0x6e, 0x62, 0x69, 0x69, 0x20, 0x4b, 0x25b, 0x74,
0x73, 0x75, 0x254, 0x20, 0x4e, 0x69, 0x69, 0x20, 0x59, 0x25b, 0x20, 0x4c,
0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c,
0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x64, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x62, 0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0x63, 0x61, 0x10db, 0x10e1, 0x10dd,
0x10e4, 0x10da, 0x10d8, 0x10dd, 0x20, 0x10d9, 0x10dd, 0x10dd, 0x10e0, 0x10d3, 0x10d8, 0x10dc,
0x10d8, 0x10e0, 0x10d4, 0x10d1, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8,
0x10e0, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d1, 0x10e0, 0x10d8, 0x10e2, 0x10d0, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x4b,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x69, 0x65, 0x72, 0x74, 0x65, 0x20,
0x57, 0x65, 0x6c, 0x74, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x72, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x3a3, 0x3c5,
0x3bd, 0x3c4, 0x3bf, 0x3bd, 0x3b9, 0x3c3, 0x3bc, 0x3ad, 0x3bd, 0x3b7, 0x20, 0x3a0,
0x3b1, 0x3b3, 0x3ba, 0x3cc, 0x3c3, 0x3bc, 0x3b9, 0x3b1, 0x20, 0x38f, 0x3c1, 0x3b1,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x399, 0x3c1, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3c1,
0x3b5, 0x3c4, 0x3b1, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0xab8, 0xa82, 0xa95, 0xab2, 0xabf,
0xaa4, 0x20, 0xaaf, 0xac1, 0xaa8, 0xabf, 0xab5, 0xab0, 0xacd, 0xab8, 0xab2, 0x20,
0xab8, 0xaae, 0xaaf, 0xa86, 0xa88, 0xab0, 0xabf, 0xab6, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xacd, 0xab0, 0xabf, 0xa9f, 0xabf, 0xab6,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0x48, 0x61, 0x64, 0x65, 0x77, 0x61, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x20, 0x6e, 0x61, 0x20, 0x44, 0x75, 0x6e, 0x69, 0x79, 0x61, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x72, 0x69, 0x73, 0x68, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x42, 0x69, 0x72, 0x74, 0x61, 0x6e, 0x69, 0x79,
0x61, 0x5d6, 0x5de, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e0, 0x5d9, 0x5d1, 0x5e8, 0x5e1,
0x5dc, 0x5d9, 0x20, 0x5de, 0x5ea, 0x5d5, 0x5d0, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5dc, 0x5e0, 0x5d3, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5d1, 0x5e8, 0x5d9, 0x5d8,
0x5e0, 0x5d9, 0x5d4, 0x938, 0x92e, 0x928, 0x94d, 0x935, 0x93f, 0x924, 0x20, 0x935,
0x948, 0x936, 0x94d, 0x935, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x906, 0x907,
0x930, 0x93f, 0x936, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x92c, 0x94d, 0x930, 0x93f, 0x91f, 0x93f, 0x936, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0xe1, 0x6c, 0x74, 0x20, 0x76,
0x69, 0x6c, 0xe1, 0x67, 0x69, 0x64, 0x151, 0xed, 0x72, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x72, 0x69, 0x74, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x53, 0x61, 0x6d,
0x72, 0xe6, 0x6d, 0x64, 0x75, 0x72, 0x20, 0x61, 0x6c, 0xfe, 0x6a, 0xf3,
0xf0, 0x6c, 0x65, 0x67, 0x75, 0x72, 0x20, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0xcd,
0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x42, 0x72, 0x65, 0x74, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x4e, 0x68, 0x61, 0x7a, 0x69, 0x20, 0x4f, 0x67, 0x65,
0x20, 0x1ee4, 0x77, 0x61, 0x20, 0x4e, 0x69, 0x69, 0x6c, 0x65, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x49,
0x72, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b,
0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x42, 0x72, 0x69, 0x74, 0x61,
0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x54, 0x65, 0x72, 0x6b, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x61, 0x73, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x49,
0x6e, 0x67, 0x67, 0x72, 0x69, 0x73, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72,
0x73, 0x61, 0x6c, 0x20, 0x54, 0x65, 0x6d, 0x70, 0x6f, 0x72, 0x65, 0x20,
0x43, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x69, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61,
0x6e, 0x6e, 0x69, 0x63, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61,
0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x63, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x73, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x62, 0x72, 0x69, 0x74, 0x61,
0x6e, 0x69, 0x63, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x41, 0x6d, 0x20, 0x55, 0x69, 0x6c,
0xed, 0x6f, 0x63, 0x68, 0x20, 0x4c, 0xe1, 0x72, 0x6e, 0x61, 0x63, 0x68,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0xe9, 0x61, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0xc9, 0x69, 0x72, 0x65,
0x61, 0x6e, 0x6e, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x72, 0x65, 0x61, 0x74,
0x61, 0x69, 0x6e, 0x65, 0x54, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x63, 0x6f,
0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x6f, 0x20, 0x75, 0x6e, 0x69,
0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49,
0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x52, 0x65, 0x67,
0x6e, 0x6f, 0x20, 0x55, 0x6e, 0x69, 0x74, 0x6f, 0x5354, 0x5b9a, 0x4e16, 0x754c,
0x6642, 0x30a2, 0x30a4, 0x30eb, 0x30e9, 0x30f3, 0x30c9, 0x6a19, 0x6e96, 0x6642, 0x82f1, 0x56fd,
0x590f, 0x6642, 0x9593, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x6e, 0x69,
0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x4b, 0x61, 0x6b, 0x6f, 0x6f,
0x72, 0x64, 0x68, 0x69, 0x6e, 0x61, 0x73, 0x69, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x49, 0x6e, 0x67, 0x67, 0x72,
0x69, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72,
0x73, 0x61, 0x6c, 0x20, 0x4b, 0x6f, 0x72, 0x64, 0x65, 0x6e, 0x61, 0x64,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20,
0x64, 0x69, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x42,
0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0x6b, 0x75, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x67, 0x72, 0x61, 0x263, 0x6c, 0x61, 0x6e, 0x20, 0x69, 0x6d,
0x79, 0x75, 0x64, 0x64, 0x73, 0x65, 0x6e, 0x41, 0x67, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x49, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x42, 0x72, 0x69, 0x74,
0x61, 0x6e, 0x79, 0x61, 0xcb8, 0xc82, 0xc98, 0xc9f, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcbe, 0xcb0, 0xccd, 0xcb5, 0xca4, 0xccd, 0xcb0, 0xcbf, 0xc95, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc90, 0xcb0, 0xcbf, 0xcb7, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xccd, 0xcb0, 0xcbf, 0xc9f,
0xcbf, 0xcb7, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0x6a9, 0x648, 0x622, 0x631, 0x688, 0x646, 0x6cc, 0x679, 0x688, 0x20,
0x6cc, 0x648, 0x646, 0x6cc, 0x648, 0x631, 0x633, 0x644, 0x20, 0x648, 0x64e, 0x6a9,
0x6be, 0x627, 0x64e, 0x6cc, 0x631, 0x650, 0x634, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631, 0x637, 0x672, 0x646,
0x648, 0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x915, 0x94b, 0x911, 0x930, 0x94d, 0x921, 0x928, 0x948, 0x91f, 0x93f,
0x921, 0x20, 0x92f, 0x942, 0x928, 0x935, 0x930, 0x94d, 0x938, 0x932, 0x20, 0x935,
0x916, 0x414, 0x4af, 0x43d, 0x438, 0x435, 0x436, 0x4af, 0x437, 0x456, 0x43b, 0x456,
0x43a, 0x20, 0x4af, 0x439, 0x43b, 0x435, 0x441, 0x442, 0x456, 0x440, 0x456, 0x43b,
0x433, 0x435, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x418, 0x440, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4b0,
0x43b, 0x44b, 0x431, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x62f,
0x6c7, 0x646, 0x64a, 0x6d5, 0x62c, 0x6c7, 0x632, 0x649, 0x644, 0x649, 0x643, 0x20,
0x6c7, 0x64a, 0x644, 0x6d5, 0x633, 0x62a, 0x649, 0x631, 0x649, 0x644, 0x6af, 0x6d5,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x64a, 0x631, 0x644, 0x627, 0x646,
0x62f, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c7, 0x644, 0x649,
0x628, 0x631, 0x64a, 0x62a, 0x627, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x179f, 0x1780, 0x179b, 0x178a, 0x17c2, 0x179b, 0x1798, 0x17b6, 0x1793, 0x1780, 0x17b6,
0x179a, 0x179f, 0x1798, 0x17d2, 0x179a, 0x17bd, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17c0, 0x179a, 0x17a1, 0x1784, 0x17cb, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1785, 0x1780, 0x17d2, 0x179a, 0x1797, 0x1796, 0x200b, 0x17a2, 0x1784,
0x17cb, 0x1782, 0x17d2, 0x179b, 0x17c1, 0x179f, 0x938, 0x92e, 0x928, 0x94d, 0x935, 0x93f,
0x924, 0x20, 0x935, 0x948, 0x936, 0x94d, 0x935, 0x93f, 0x915, 0x20, 0x935, 0x947,
0x933, 0x906, 0x92f, 0x930, 0x93f, 0x936, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x93f, 0x91f,
0x93f, 0x936, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0xd611, 0xc815, 0x20, 0xc138, 0xacc4, 0xc2dc, 0xc544, 0xc77c, 0xb79c, 0xb4dc,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc601, 0xad6d, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x65, 0x72, 0x64, 0xfb,
0x6e, 0xee, 0x20, 0x79, 0x61, 0x20, 0x48, 0x65, 0x76, 0x64, 0x65, 0x6d,
0xee, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xce, 0x72, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x42, 0x72, 0xee, 0x74, 0x61,
0x6e, 0x79, 0x61, 0x79, 0xea, 0x411, 0x438, 0x440, 0x434, 0x438, 0x43a, 0x442,
0x4af, 0x4af, 0x20, 0x434, 0x4af, 0x439, 0x43d, 0x4e9, 0x43b, 0x4af, 0x43a, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeaa, 0xeb2, 0xe81, 0xebb, 0xe99,
0xec0, 0xe8a, 0xeb5, 0xe87, 0xe9e, 0xeb4, 0xe81, 0xeb1, 0xe94, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2,
0xe99, 0x200b, 0xec4, 0xead, 0x200b, 0xea3, 0xeb4, 0x200b, 0xe8a, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead,
0xe99, 0x200b, 0xead, 0xeb1, 0xe87, 0x200b, 0xe81, 0xeb4, 0xe94, 0x55, 0x6e, 0x69,
0x76, 0x65, 0x72, 0x73, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x6b, 0x6f,
0x6f, 0x72, 0x64, 0x69, 0x6e, 0x113, 0x74, 0x61, 0x69, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x12a, 0x72, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c,
0x69, 0x65, 0x6c, 0x62, 0x72, 0x69, 0x74, 0x101, 0x6e, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x70, 0x61, 0x73, 0x61, 0x75, 0x6c, 0x69, 0x6f, 0x20,
0x73, 0x75, 0x64, 0x65, 0x72, 0x69, 0x6e, 0x74, 0x61, 0x73, 0x69, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x69, 0x72, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69,
0x6e, 0x11b, 0x72, 0x6f, 0x77, 0x61, 0x6e, 0x79, 0x20, 0x73, 0x77, 0x11b,
0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x49, 0x72, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x42, 0x72, 0x69, 0x74, 0x65, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x41a, 0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x438, 0x440, 0x430, 0x43d, 0x43e,
0x20, 0x443, 0x43d, 0x438, 0x432, 0x435, 0x440, 0x437, 0x430, 0x43b, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0xd15, 0xd4b, 0xd7c, 0xd21, 0xd3f, 0xd28, 0xd47, 0xd31, 0xd4d, 0xd31, 0xd21,
0xd4d, 0x20, 0xd2f, 0xd42, 0xd23, 0xd3f, 0xd35, 0xd47, 0xd34, 0xd4d, 0x200c, 0xd38,
0xd7d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd10, 0xd31, 0xd3f, 0xd37, 0xd4d, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4d, 0xd30, 0xd3f, 0xd1f, 0xd4d, 0xd1f,
0xd40, 0xd37, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20,
0x53, 0x65, 0x6c, 0x61, 0x72, 0x61, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x49, 0x72, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x42, 0x72, 0x69,
0x74, 0x69, 0x73, 0x68, 0x995, 0x9cb, 0x993, 0x9b0, 0x9cd, 0x9a6, 0x9bf, 0x9a8,
0x9c7, 0x99f, 0x9c7, 0x9a6, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9a8, 0x9bf, 0x9ad, 0x9b0,
0x9cd, 0x9b8, 0x9c7, 0x9b2, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x57, 0x101, 0x20,
0x41, 0x6f, 0x6e, 0x75, 0x69, 0x20, 0x4b, 0x14d, 0x74, 0x75, 0x69, 0x74,
0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x69, 0x72, 0x69, 0x68, 0x69, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x50, 0x69, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x52, 0x61, 0x75,
0x6d, 0x61, 0x74, 0x69, 0x906, 0x92f, 0x930, 0x93f, 0x936, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x93f,
0x91f, 0x93f, 0x936, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x41e, 0x43b, 0x43e, 0x43d, 0x20, 0x443, 0x43b, 0x441, 0x44b,
0x43d, 0x20, 0x437, 0x43e, 0x445, 0x438, 0x446, 0x443, 0x443, 0x43b, 0x430, 0x43b,
0x442, 0x442, 0x430, 0x439, 0x20, 0x446, 0x430, 0x433, 0x418, 0x440, 0x43b, 0x430,
0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x1823,
0x182f, 0x1820, 0x1828, 0x20, 0x1823, 0x182f, 0x1823, 0x1830, 0x202f, 0x1824, 0x1828, 0x20,
0x1835, 0x1823, 0x182c, 0x1822, 0x1834, 0x1821, 0x182d, 0x1823, 0x182f, 0x1823, 0x182f, 0x1833,
0x1820, 0x1833, 0x1820, 0x1822, 0x20, 0x1834, 0x1820, 0x182d, 0x938, 0x92e, 0x928, 0x94d,
0x935, 0x93f, 0x924, 0x20, 0x935, 0x93f, 0x936, 0x94d, 0x935, 0x20, 0x938, 0x92e,
0x92f, 0x92c, 0x947, 0x932, 0x93e, 0x92f, 0x924, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x41, 0x72, 0xe9, 0x6e, 0x6a, 0x6d, 0x1eb9, 0x6e, 0x74, 0x20, 0x1ecd,
0x66, 0x20, 0x44, 0x69, 0x20, 0x48, 0xf3, 0x6c, 0x20, 0x57, 0x1ecd, 0x6c,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0xed, 0x72, 0xed, 0x73, 0x68,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42,
0x72, 0xed, 0x74, 0xed, 0x73, 0x68, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x7d6, 0x7ca,
0x7ef, 0x7d3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7eb, 0x20,
0x7d8, 0x7cf, 0x7f2, 0x7ec, 0x7d6, 0x7df, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7e3,
0x7cd, 0x7f2, 0x7cc, 0x7d9, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d3, 0x7d9, 0x7cc, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x72, 0x65, 0x6a,
0x75, 0x76, 0x76, 0x6f, 0x6e, 0x20, 0x6f, 0x6b, 0x74, 0x61, 0x73, 0x61,
0x161, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x6c, 0xe1, 0x6e, 0x64,
0x64, 0x61, 0x6c, 0x61, 0x161, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x42, 0x72, 0x69, 0x68, 0x74, 0x61, 0x6c, 0x61, 0x161,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x6b,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x72, 0x74, 0x20, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x69, 0x72,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x62, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x62, 0x72, 0x69, 0x74, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x6f, 0x72, 0x61, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x6f, 0x6e, 0x61,
0x64, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x75, 0x20,
0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x64, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019,
0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0xb38, 0xb2e, 0xb28, 0xb4d, 0xb71,
0xb3f, 0xb24, 0x20, 0xb38, 0xb3e, 0xb30, 0xb4d, 0xb2c, 0xb1c, 0xb28, 0xb40, 0xb28,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb07, 0xb30, 0xb3f, 0xb36, 0xb4d, 0x200c, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3f,
0xb1f, 0xb3f, 0xb36, 0xb4d, 0x200c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x64, 0x69, 0x6c, 0x2d,
0x41, 0x64, 0x64, 0x75, 0x6e, 0x79, 0x61, 0x61, 0x20, 0x51, 0x69, 0x6e,
0x64, 0x61, 0x61, 0x2019, 0x65, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x41, 0x61, 0x79, 0x69, 0x72, 0x69, 0x69, 0x73,
0x68, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x42, 0x69, 0x72, 0x69, 0x74, 0x69, 0x73, 0x68,
0x647, 0x645, 0x63a, 0x696, 0x649, 0x20, 0x646, 0x693, 0x6cc, 0x648, 0x627, 0x644,
0x20, 0x648, 0x62e, 0x62a, 0x622, 0x64a, 0x631, 0x634, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x631, 0x64a, 0x62a, 0x627,
0x646, 0x648, 0x64a, 0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x632, 0x645, 0x627, 0x646, 0x20, 0x647, 0x645, 0x627, 0x647, 0x646,
0x6af, 0x20, 0x62c, 0x647, 0x627, 0x646, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x6cc, 0x631, 0x644, 0x646, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x628,
0x631, 0x6cc, 0x62a, 0x627, 0x646, 0x6cc, 0x627, 0x75, 0x6e, 0x69, 0x77, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x79, 0x6e, 0x6f, 0x77, 0x61, 0x6e, 0x79,
0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x42, 0x72, 0x79,
0x74, 0x79, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x43,
0x6f, 0x6f, 0x72, 0x64, 0x65, 0x6e, 0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0xea, 0x73, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x42, 0x72, 0x69, 0x74, 0xe2, 0x6e, 0x69, 0x63, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x72, 0x64, 0x65, 0x6e, 0x61, 0x64,
0x61, 0x20, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x42, 0x72, 0x69, 0x74, 0xe2, 0x6e, 0x69, 0x63, 0x61, 0xa15, 0xa4b,
0xa14, 0xa30, 0xa21, 0xa40, 0xa28, 0xa47, 0xa1f, 0xa47, 0xa21, 0x20, 0xa35, 0xa3f,
0xa06, 0xa2a, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa07, 0xa30, 0xa3f,
0xa38, 0xa3c, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2c, 0xa4d, 0xa30, 0xa3f, 0xa1f, 0xa3f, 0xa38, 0xa3c, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0x54, 0x69, 0x71, 0x73, 0x69, 0x6d, 0x75, 0x79, 0x75, 0x6e, 0x74, 0x69,
0x6e, 0x20, 0x54, 0x75, 0x70, 0x61, 0x63, 0x68, 0x69, 0x73, 0x71, 0x61,
0x20, 0x48, 0x6f, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x42, 0x72, 0x69, 0x74,
0xe1, 0x6e, 0x69, 0x63, 0x6f, 0x54, 0x69, 0x6d, 0x70, 0x75, 0x6c, 0x20,
0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f,
0x6f, 0x72, 0x64, 0x6f, 0x6e, 0x61, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69,
0x63, 0x103, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0xe0, 0x74, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61,
0x64, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x63, 0x412,
0x441, 0x435, 0x43c, 0x438, 0x440, 0x43d, 0x43e, 0x435, 0x20, 0x43a, 0x43e, 0x43e,
0x440, 0x434, 0x438, 0x43d, 0x438, 0x440, 0x43e, 0x432, 0x430, 0x43d, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x440, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x435,
0x43b, 0x438, 0x43a, 0x43e, 0x431, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x938, 0x92e, 0x928, 0x94d, 0x935, 0x93f, 0x924, 0x903, 0x20, 0x935,
0x948, 0x936, 0x94d, 0x935, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x3a, 0x1c60,
0x1c5a, 0x1c68, 0x1c70, 0x1c64, 0x1c71, 0x1c6e, 0x1c74, 0x1c6e, 0x1c70, 0x20, 0x1c6d, 0x1c69,
0x1c71, 0x1c64, 0x1c63, 0x1c5f, 0x1c68, 0x1c65, 0x1c5f, 0x1c5e, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x54, 0x65, 0x6d, 0x70, 0x75, 0x73, 0x20, 0x63, 0x6f, 0x6f, 0x72,
0x64, 0x69, 0x6e, 0x61, 0x64, 0x75, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x73, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x69,
0x61, 0x6c, 0x65, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe0, 0x6e, 0x6e, 0x69,
0x63, 0x61, 0x41a, 0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x438, 0x441, 0x430,
0x43d, 0x43e, 0x20, 0x443, 0x43d, 0x438, 0x432, 0x435, 0x440, 0x437, 0x430, 0x43b,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x441, 0x43a,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430,
0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x441, 0x43a, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d,
0x438, 0x458, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x4b, 0x6f, 0x6f, 0x72, 0x64, 0x69,
0x6e, 0x69, 0x73, 0x61, 0x6e, 0x6f, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x7a, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x49, 0x72, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x42, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x6a, 0x61,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x69, 0x73, 0x61, 0x6e, 0x6f, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72,
0x7a, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49,
0x72, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x72,
0x69, 0x74, 0x61, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x72, 0x61,
0x20, 0x43, 0x75, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x61, 0x20, 0x55,
0x6e, 0x69, 0x76, 0x69, 0x72, 0x73, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x49, 0x72, 0x6c, 0x61,
0x6e, 0x6e, 0x69, 0x73, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x42, 0x72, 0x69, 0x74, 0xe0, 0x6e, 0x6e, 0x69,
0x63, 0x61, 0x75, 0x6e, 0x69, 0x77, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x6e,
0x79, 0x20, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x79, 0x6e, 0x6f, 0x77, 0x61,
0x6e, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x49, 0x72, 0x6c, 0x61, 0x6e,
0x64, 0x79, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x42, 0x72, 0x79, 0x74, 0x79, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x6af, 0x68f, 0x64a, 0x644, 0x20, 0x62f, 0x646, 0x64a, 0x627,
0x648, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x626, 0x631, 0x634, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x628, 0x631, 0x637, 0x627, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x917, 0x926, 0x93f, 0x952, 0x92f, 0x932, 0x20, 0x906, 0x932, 0x92e, 0x940,
0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0xdc3, 0xdb8, 0xd9a, 0xd9a, 0xdca, 0xdc2,
0x20, 0xdc3, 0xdcf, 0xdbb, 0xdca, 0xdc0, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd85, 0xdba, 0xdbb, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xdca, 0x200d, 0xdbb,
0xdd2, 0xdad, 0xdcf, 0xdb1, 0xdca, 0x200d, 0xdba, 0x20, 0xd9c, 0xdd2, 0xdb8, 0xdca,
0xdc4, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x6b, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x6f, 0x76, 0x61, 0x6e, 0xfd, 0x20, 0x73, 0x76,
0x65, 0x74, 0x6f, 0x76, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0xed, 0x72, 0x73,
0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x69, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x7a, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x6b, 0x6f,
0x6f, 0x72, 0x64, 0x69, 0x6e, 0x69, 0x72, 0x61, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72,
0x69, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x49, 0x73, 0x6b, 0x75, 0x2d, 0x78, 0x69, 0x72, 0x61,
0x6e, 0x20, 0x65, 0x65, 0x20, 0x43, 0x61, 0x61, 0x6c, 0x61, 0x6d, 0x6b,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41,
0x79, 0x72, 0x69, 0x73, 0x68, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x42, 0x69, 0x72, 0x69, 0x74, 0x69, 0x73, 0x68, 0x6b, 0x61,
0x74, 0x69, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x61, 0x64, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe1,
0x6e, 0x69, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x4d, 0x66, 0x75, 0x6d, 0x6f, 0x20, 0x77, 0x61,
0x20, 0x6b, 0x75, 0x72, 0x61, 0x74, 0x69, 0x62, 0x75, 0x20, 0x73, 0x61,
0x61, 0x20, 0x75, 0x6c, 0x69, 0x6d, 0x77, 0x65, 0x6e, 0x67, 0x75, 0x6e,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x79, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x69, 0x6e, 0x67, 0x65, 0x72, 0x65, 0x7a, 0x61, 0x53, 0x61, 0x61, 0x20,
0x79, 0x61, 0x20, 0x44, 0x75, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x6f, 0x72,
0x64, 0x69, 0x6e, 0x65, 0x72, 0x61, 0x64, 0x20, 0x75, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x65, 0x6c, 0x6c, 0x20, 0x74, 0x69, 0x64, 0x69, 0x72,
0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x62, 0x72, 0x69, 0x74, 0x74, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x725, 0x715,
0x722, 0x718, 0x72c, 0x710, 0x20, 0x72c, 0x712, 0x720, 0x71d, 0x72c, 0x710, 0x20,
0x721, 0x71b, 0x71f, 0x718, 0x723, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x71d, 0x72a, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x712, 0x72a, 0x71d,
0x71b, 0x722, 0x71d, 0x710, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b7, 0x430,
0x4b3, 0x43e, 0x43d, 0x438, 0x438, 0x20, 0x4b3, 0x430, 0x43c, 0x43e, 0x4b3, 0x430,
0x43d, 0x433, 0x441, 0x43e, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x418,
0x440, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20,
0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f, 0xb92, 0xbb0, 0xbc1, 0xb99,
0xbcd, 0xb95, 0xbbf, 0xba3, 0xbc8, 0xba8, 0xbcd, 0xba4, 0x20, 0xb9a, 0xbb0, 0xbcd,
0xbb5, 0xba4, 0xbc7, 0xb9a, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb90, 0xbb0,
0xbbf, 0xbb7, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb9f, 0xbcd, 0xb9f,
0xbbf, 0xbb7, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0x411, 0x4e9, 0x442, 0x435, 0x43d, 0x434, 0x4e9, 0x43d, 0x44c, 0x44f,
0x20, 0x43a, 0x438, 0x43b, 0x435, 0x448, 0x442, 0x435, 0x440, 0x435, 0x43b, 0x433,
0x4d9, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x438,
0x442, 0x430, 0x43d, 0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0xc38, 0xc2e, 0xc28, 0xc4d, 0xc35, 0xc2f,
0x20, 0xc38, 0xc3e, 0xc30, 0xc4d, 0xc35, 0xc1c, 0xc28, 0xc40, 0xc28, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc10, 0xc30, 0xc3f, 0xc37, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c,
0xc4d, 0xc30, 0xc3f, 0xc1f, 0xc40, 0xc37, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2a, 0xe32,
0xe01, 0xe25, 0xe40, 0xe0a, 0xe34, 0xe07, 0xe1e, 0xe34, 0xe01, 0xe31, 0xe14, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe44, 0xe2d,
0xe23, 0xe4c, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe31, 0xe07, 0xe01, 0xe24, 0xe29,
0x12dd, 0x1270, 0x1233, 0x1290, 0x12e8, 0x20, 0x12a3, 0x12f5, 0x121b, 0x1233, 0x12ca, 0x20,
0x130d, 0x12dc, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a0, 0x12ed,
0x122a, 0x123d, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272,
0x20, 0x1265, 0x122a, 0x1323, 0x1295, 0x12eb, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x65, 0x6d, 0x101, 0x6d, 0x61, 0x6e, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x70, 0x69, 0x6c, 0x69, 0x74, 0x101, 0x6e, 0x69, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x45,
0x15f, 0x20, 0x47, 0xfc, 0x64, 0xfc, 0x6d, 0x6c, 0xfc, 0x20, 0x45, 0x76,
0x72, 0x65, 0x6e, 0x73, 0x65, 0x6c, 0x20, 0x5a, 0x61, 0x6d, 0x61, 0x6e,
0x130, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x6e,
0x67, 0x69, 0x6c, 0x74, 0x65, 0x72, 0x65, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x74, 0x67, 0x61, 0x15f, 0x64, 0x79,
0x72, 0x79, 0x6c, 0xfd, 0x61, 0x6e, 0x20, 0xe4, 0x68, 0x6c, 0x69, 0x75,
0x6d, 0x75, 0x6d, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x65, 0xfd,
0x69, 0x6b, 0x20, 0x42, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69, 0xfd, 0x61,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x437, 0x430, 0x20, 0x432, 0x441, 0x435, 0x441, 0x432, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x43a, 0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x43e,
0x432, 0x430, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x432, 0x20, 0x406, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x456,
0x457, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x412, 0x435, 0x43b, 0x438, 0x43a,
0x456, 0x439, 0x20, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x456, 0x457, 0x6b,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x6f, 0x77, 0x61, 0x6e, 0x79, 0x20,
0x73, 0x77, 0x11b, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x49,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6a9, 0x648, 0x622, 0x631,
0x688, 0x6cc, 0x646, 0x6cc, 0x679, 0x688, 0x20, 0x6cc, 0x648, 0x646, 0x6cc, 0x648,
0x631, 0x633, 0x644, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x626, 0x631, 0x634,
0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x628, 0x631, 0x679, 0x634, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x626, 0x649, 0x631, 0x6d0, 0x644, 0x627, 0x646, 0x62f, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x6d5, 0x646, 0x6af, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x4b, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x61, 0x6c, 0x69, 0x20, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x76, 0x61, 0x71, 0x74,
0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72, 0x69,
0x74, 0x61, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x47, 0x69, 0x1edd, 0x20,
0x50, 0x68, 0x1ed1, 0x69, 0x20, 0x68, 0x1ee3, 0x70, 0x20, 0x51, 0x75, 0x1ed1,
0x63, 0x20, 0x74, 0x1ebf, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x41, 0x69, 0x2d, 0x6c, 0x65, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x6e, 0x68, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x43, 0x79, 0x66, 0x66, 0x72, 0x65, 0x64, 0x6e,
0x69, 0x6f, 0x6c, 0x20, 0x43, 0x79, 0x64, 0x6c, 0x79, 0x6e, 0x6f, 0x6c,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x49, 0x77, 0x65, 0x72, 0x64, 0x64, 0x6f, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x50, 0x72, 0x79, 0x64,
0x61, 0x69, 0x6e, 0x6b, 0x6f, 0xf6, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x61,
0x72, 0x72, 0x65, 0x20, 0x77, 0x72, 0xe2, 0x6c, 0x64, 0x74, 0x69, 0x69,
0x64, 0x49, 0x65, 0x72, 0x73, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x42, 0x72, 0x69, 0x74, 0x73, 0x65, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x43, 0x55,
0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x69, 0x6e, 0x69,
0x77, 0x65, 0x6c, 0x73, 0x65, 0x6c, 0x20, 0x79, 0x75, 0xf1, 0x20, 0x62,
0x6f, 0x6f, 0x6c, 0x65, 0x29, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x49,
0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x47, 0x72, 0x61, 0x6e, 0x64,
0x65, 0x20, 0x42, 0x72, 0x65, 0x74, 0x61, 0x67, 0x6e, 0x65, 0xc0, 0x70,
0x61, 0x70, 0x1ecd, 0x300, 0x20, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x67, 0x62, 0xe1, 0x79, 0xe9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x69, 0x72, 0x69, 0x69,
0x73, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0,
0x20, 0x47, 0x1eb9, 0x1eb9, 0x73, 0x69, 0xc0, 0x70, 0x61, 0x70, 0x254, 0x300,
0x20, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x67, 0x62, 0xe1, 0x79,
0xe9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x41, 0x69, 0x72, 0x69, 0x69, 0x73, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x47, 0x25b, 0x25b,
0x73, 0x69, 0x69, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x6f, 0x6d, 0x68, 0x6c, 0x61, 0x62, 0x61, 0x20, 0x65, 0x73, 0x69,
0x64, 0x69, 0x64, 0x69, 0x79, 0x65, 0x6c, 0x77, 0x65, 0x69, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x49, 0x72, 0x69, 0x73, 0x68, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x69, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73,
0x68, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x47,
0x61, 0x2d, 0x6b, 0x61, 0x72, 0x20, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x1ebd,
0x6e, 0x68, 0x2d, 0x6b, 0x72, 0xe9, 0x6e, 0x2d, 0x6a, 0x61, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x49, 0x72, 0x1ef9, 0x6e, 0x61, 0xd3, 0x72,
0x61, 0x20, 0x4d, 0x72, 0x69, 0x74, 0x1ef9, 0x6e, 0x129, 0x6b, 0x75, 0x20,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x4d, 0x169, 0x64, 0x69, 0x20, 0x74, 0x75, 0x72, 0x75, 0x73,
0x75, 0x73, 0x61, 0x77, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x64, 0x65, 0x6e,
0x61, 0x64, 0x75, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x20, 0x49, 0x72, 0xe3, 0x64, 0x65,
0x69, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x42, 0x69, 0x72,
0x69, 0x74, 0xe3, 0x6e, 0x69, 0x6b, 0x75, 0x55, 0x66, 0x73, 0x74, 0x65,
0x6d, 0x65, 0x74, 0x20, 0x57, 0xe4, 0xe4, 0x6c, 0x74, 0x74, 0x69, 0x64,
0x6a, 0x49, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42, 0x72, 0x69, 0x74,
0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x48, 0x61, 0x6d, 0x64, 0xe1, 0x72, 0xe9, 0x6e, 0x20, 0x4a, 0x61,
0x68, 0xe1, 0x6e, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x65, 0x72,
0x69, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x42, 0x61, 0x72, 0x74, 0xe1, 0x6e, 0x69, 0xe1, 0x69, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73,
0x61, 0x6c, 0x65, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x6f,
0x75, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x2019, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e,
0x6e, 0x69, 0x63, 0x61, 0x189, 0x75, 0x6c, 0x69, 0x6e, 0x79, 0x61, 0x20,
0x61, 0x14b, 0x75, 0x6e, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x256,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x47, 0x61, 0x67, 0x25b, 0x25b, 0x73, 0x68, 0x269, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x938, 0x92e, 0x928,
0x94d, 0x935, 0x93f, 0x924, 0x20, 0x926, 0x941, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x906, 0x907, 0x930, 0x93f, 0x936, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92c,
0x94d, 0x930, 0x93f, 0x91f, 0x93f, 0x936, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x4f, 0x72, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61,
0x6c, 0x65, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x65, 0x6e, 0x61, 0x64,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x7a, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x62, 0x72, 0x69, 0x74,
0xe0, 0x6e, 0x65, 0x67, 0x61, 0x73, 0x6d, 0x61, 0x6e, 0x69, 0x20, 0x6a,
0x61, 0x67, 0x61, 0x20, 0x70, 0x72, 0x75, 0x74, 0x69, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x61, 0x61, 0x69, 0x72, 0x69, 0x73, 0x20, 0x6d, 0x61, 0x6e,
0x61, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x72, 0x69, 0x1e6d,
0x69, 0x73, 0x20, 0x6b, 0x61, 0x72, 0xe3, 0x20, 0x6d, 0x61, 0x73, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0xb38, 0xb2e, 0xb3e, 0xb28, 0xb3f, 0x20, 0xb1c,
0xb3e, 0xb17, 0xb3e, 0x20, 0xb2a, 0xb4d, 0xb30, 0xb41, 0xb24, 0xb3f, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb06, 0xb07, 0xb30, 0xb3f, 0xb38, 0xb4d, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb4d,
0xb30, 0xb3f, 0xb1f, 0xb3f, 0xb36, 0xb4d, 0x200c, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e,
0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e
};

static inline constexpr char16_t shortMetaZoneNameTable[] = {
0x41, 0x4b, 0x54, 0x41, 0x4b, 0x53, 0x54, 0x41, 0x4b, 0x44, 0x54, 0x43,
0x54, 0x43, 0x53, 0x54, 0x43, 0x44, 0x54, 0x45, 0x54, 0x45, 0x53, 0x54,
0x45, 0x44, 0x54, 0x4d, 0x54, 0x4d, 0x53, 0x54, 0x4d, 0x44, 0x54, 0x50,
0x54, 0x50, 0x53, 0x54, 0x50, 0x44, 0x54, 0x41, 0x54, 0x41, 0x53, 0x54,
0x41, 0x44, 0x54, 0x47, 0x4d, 0x54, 0x48, 0x41, 0x54, 0x48, 0x41, 0x53,
0x54, 0x48, 0x41, 0x44, 0x54, 0x43, 0x41, 0x54, 0x45, 0x41, 0x54, 0x53,
0x41, 0x53, 0x54, 0x57, 0x41, 0x54, 0x57, 0x41, 0x53, 0x54, 0x47, 0x53,
0x54, 0x9ad, 0x9be, 0x2e, 0x20, 0x9b8, 0x2e, 0x43, 0x45, 0x54, 0x43, 0x45,
0x53, 0x54, 0x45, 0x45, 0x54, 0x45, 0x45, 0x53, 0x54, 0x57, 0x45, 0x54,
0x57, 0x45, 0x53, 0x54, 0x49, 0x53, 0x54, 0x4d, 0x59, 0x54, 0x53, 0x47,
0x54, 0x41, 0x43, 0x54, 0x20, 0x41, 0x63, 0x72, 0x65, 0x41, 0x43, 0x53,
0x54, 0x20, 0x41, 0x63, 0x72, 0x65, 0x41, 0x46, 0x54, 0x41, 0x4c, 0x4d,
0x54, 0x41, 0x4c, 0x4d, 0x54, 0x20, 0x44, 0x53, 0x54, 0x41, 0x4d, 0x54,
0x41, 0x4d, 0x53, 0x54, 0x41, 0x4e, 0x41, 0x54, 0x41, 0x4e, 0x41, 0x53,
0x54, 0x53, 0x53, 0x54, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x41,
0x70, 0x69, 0x61, 0x53, 0x53, 0x54, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x41, 0x70, 0x69, 0x61, 0x20, 0x44, 0x53, 0x54, 0x41, 0x4c, 0x4d,
0x54, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x41, 0x4c, 0x4d, 0x54, 0x20,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x44, 0x53, 0x54, 0x41, 0x51, 0x54,
0x54, 0x41, 0x51, 0x54, 0x54, 0x20, 0x44, 0x53, 0x54, 0x41, 0x53, 0x54,
0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x41, 0x44, 0x54, 0x20, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x41, 0x52, 0x54, 0x41, 0x52, 0x53, 0x54,
0x57, 0x41, 0x52, 0x54, 0x57, 0x41, 0x52, 0x53, 0x54, 0x41, 0x4d, 0x54,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x41, 0x4d, 0x53, 0x54,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x41, 0x43, 0x44, 0x54,
0x41, 0x43, 0x57, 0x53, 0x54, 0x41, 0x43, 0x57, 0x44, 0x54, 0x41, 0x45,
0x53, 0x54, 0x41, 0x45, 0x44, 0x54, 0x41, 0x57, 0x53, 0x54, 0x41, 0x57,
0x44, 0x54, 0x41, 0x5a, 0x54, 0x41, 0x5a, 0x53, 0x54, 0x41, 0x5a, 0x4f,
0x54, 0x41, 0x5a, 0x4f, 0x53, 0x54, 0x42, 0x53, 0x54, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x42, 0x44, 0x54, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x42, 0x54,
0x54, 0x42, 0x4f, 0x54, 0x42, 0x52, 0x54, 0x42, 0x52, 0x53, 0x54, 0x42,
0x4e, 0x54, 0x43, 0x56, 0x54, 0x43, 0x56, 0x53, 0x54, 0x43, 0x41, 0x53,
0x54, 0x43, 0x68, 0x53, 0x54, 0x43, 0x48, 0x41, 0x53, 0x54, 0x43, 0x48,
0x41, 0x44, 0x54, 0x43, 0x4c, 0x54, 0x43, 0x4c, 0x53, 0x54, 0x43, 0x53,
0x54, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x43, 0x53, 0x54, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x20, 0x44, 0x53, 0x54, 0x43, 0x58, 0x54, 0x43,
0x43, 0x54, 0x43, 0x4f, 0x54, 0x43, 0x4f, 0x53, 0x54, 0x43, 0x4b, 0x54,
0x43, 0x4b, 0x54, 0x20, 0x44, 0x53, 0x54, 0x43, 0x53, 0x54, 0x20, 0x43,
0x75, 0x62, 0x61, 0x43, 0x44, 0x54, 0x20, 0x43, 0x75, 0x62, 0x61, 0x44,
0x41, 0x56, 0x54, 0x44, 0x44, 0x55, 0x54, 0x54, 0x4c, 0x54, 0x45, 0x41,
0x53, 0x54, 0x45, 0x41, 0x53, 0x53, 0x54, 0x45, 0x43, 0x54, 0x20, 0x45,
0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x46, 0x45, 0x45, 0x54, 0x46, 0x4b,
0x54, 0x46, 0x4b, 0x53, 0x54, 0x46, 0x4a, 0x54, 0x46, 0x4a, 0x54, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x47, 0x46, 0x54, 0x54, 0x41, 0x41,
0x46, 0x47, 0x41, 0x4c, 0x54, 0x47, 0x41, 0x4d, 0x54, 0x47, 0x45, 0x54,
0x47, 0x45, 0x53, 0x54, 0x47, 0x49, 0x54, 0x47, 0x4e, 0x53, 0x54, 0x47,
0x4e, 0x53, 0x53, 0x54, 0x45, 0x47, 0x54, 0x45, 0x47, 0x53, 0x54, 0x57,
0x47, 0x54, 0x57, 0x47, 0x53, 0x54, 0x47, 0x53, 0x54, 0x20, 0x47, 0x75,
0x61, 0x6d, 0x47, 0x53, 0x54, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x47, 0x59,
0x54, 0x48, 0x53, 0x54, 0x48, 0x44, 0x54, 0x48, 0x4b, 0x54, 0x48, 0x4b,
0x54, 0x20, 0x44, 0x53, 0x54, 0x48, 0x4f, 0x56, 0x54, 0x48, 0x4f, 0x56,
0x53, 0x54, 0x49, 0x53, 0x54, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x49, 0x4f, 0x54, 0x49, 0x43, 0x54, 0x43, 0x49, 0x54, 0x45, 0x49, 0x54,
0x57, 0x49, 0x54, 0x49, 0x52, 0x53, 0x54, 0x49, 0x52, 0x53, 0x54, 0x20,
0x44, 0x53, 0x54, 0x49, 0x52, 0x4b, 0x54, 0x49, 0x52, 0x4b, 0x53, 0x54,
0x49, 0x53, 0x54, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x49, 0x44,
0x54, 0x4a, 0x53, 0x54, 0x4a, 0x53, 0x54, 0x20, 0x44, 0x53, 0x54, 0x50,
0x45, 0x54, 0x54, 0x50, 0x45, 0x54, 0x53, 0x54, 0x50, 0x45, 0x54, 0x44,
0x54, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x445, 0x447, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab,
0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x445, 0x447, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x445, 0x447, 0x4b, 0x53, 0x54, 0x20,
0x44, 0x53, 0x54, 0x4b, 0x4f, 0x53, 0x54, 0x4b, 0x52, 0x41, 0x54, 0x4b,
0x52, 0x41, 0x53, 0x54, 0x4b, 0x47, 0x54, 0x53, 0x4c, 0x53, 0x54, 0x4c,
0x49, 0x4e, 0x54, 0x4c, 0x48, 0x53, 0x54, 0x4c, 0x48, 0x53, 0x54, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x4d, 0x53, 0x54, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x4d, 0x53, 0x54, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x20, 0x44, 0x53, 0x54, 0x4d, 0x41, 0x47, 0x54, 0x4d, 0x41, 0x47, 0x53,
0x54, 0x4d, 0x56, 0x54, 0x4d, 0x41, 0x52, 0x54, 0x4d, 0x48, 0x54, 0x4d,
0x55, 0x54, 0x4d, 0x55, 0x53, 0x54, 0x4d, 0x41, 0x57, 0x54, 0x50, 0x53,
0x54, 0x4d, 0x50, 0x44, 0x54, 0x4d, 0x55, 0x4c, 0x41, 0x54, 0x55, 0x4c,
0x41, 0x53, 0x54, 0x4d, 0x53, 0x4b, 0x41c, 0x421, 0x41a, 0x4d, 0x53, 0x44,
0x4d, 0x4d, 0x54, 0x4e, 0x52, 0x54, 0x4e, 0x50, 0x54, 0x4e, 0x43, 0x54,
0x4e, 0x44, 0x54, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4e, 0x5a, 0x53, 0x54, 0x4e, 0x5a, 0x44,
0x54, 0x4e, 0x55, 0x54, 0x4e, 0x46, 0x54, 0x4e, 0x46, 0x44, 0x54, 0x46,
0x4e, 0x54, 0x46, 0x4e, 0x53, 0x54, 0x43, 0x68, 0x53, 0x54, 0x20, 0x4e,
0x4d, 0x49, 0x4e, 0x4f, 0x56, 0x54, 0x4e, 0x4f, 0x56, 0x53, 0x54, 0x4f,
0x4d, 0x53, 0x54, 0x4f, 0x4d, 0x53, 0x53, 0x54, 0x50, 0x4b, 0x54, 0x50,
0x4b, 0x54, 0x20, 0x44, 0x53, 0x54, 0x50, 0x57, 0x54, 0x50, 0x47, 0x54,
0x50, 0x59, 0x54, 0x50, 0x59, 0x53, 0x54, 0x45, 0x44, 0x54, 0x20, 0x50,
0x65, 0x72, 0x75, 0x50, 0x53, 0x54, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x65, 0x50, 0x53, 0x54, 0x20, 0x50, 0x68, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x20, 0x44, 0x53, 0x54, 0x50,
0x48, 0x4f, 0x54, 0x50, 0x4d, 0x53, 0x54, 0x50, 0x4d, 0x44, 0x54, 0x50,
0x53, 0x54, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x50,
0x4f, 0x4e, 0x54, 0x50, 0x59, 0x54, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x51, 0x59, 0x5a, 0x54, 0x51, 0x59, 0x5a, 0x54, 0x20, 0x44, 0x53, 0x54,
0x52, 0x45, 0x54, 0x52, 0x4f, 0x54, 0x54, 0x53, 0x41, 0x4b, 0x54, 0x53,
0x41, 0x4b, 0x53, 0x54, 0x53, 0x41, 0x4d, 0x54, 0x53, 0x41, 0x4d, 0x53,
0x54, 0x53, 0x53, 0x54, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x44,
0x53, 0x54, 0x53, 0x43, 0x54, 0x53, 0x42, 0x54, 0x53, 0x52, 0x54, 0x53,
0x59, 0x4f, 0x54, 0x54, 0x41, 0x48, 0x54, 0x54, 0x57, 0x54, 0x54, 0x57,
0x54, 0x20, 0x44, 0x53, 0x54, 0x54, 0x4a, 0x54, 0x54, 0x4b, 0x54, 0x54,
0x4f, 0x54, 0x54, 0x4f, 0x53, 0x54, 0x43, 0x48, 0x55, 0x54, 0x54, 0x4d,
0x54, 0x20, 0x44, 0x53, 0x54, 0x54, 0x56, 0x54, 0x55, 0x59, 0x54, 0x55,
0x59, 0x53, 0x54, 0x55, 0x5a, 0x54, 0x55, 0x5a, 0x54, 0x20, 0x44, 0x53,
0x54, 0x56, 0x55, 0x54, 0x56, 0x55, 0x54, 0x20, 0x44, 0x53, 0x54, 0x56,
0x6e, 0x7a, 0x6c, 0x54, 0x56, 0x4c, 0x41, 0x54, 0x56, 0x4c, 0x41, 0x53,
0x54, 0x56, 0x4f, 0x4c, 0x54, 0x56, 0x4f, 0x4c, 0x53, 0x54, 0x56, 0x4f,
0x53, 0x54, 0x57, 0x41, 0x4b, 0x54, 0x57, 0x46, 0x54, 0x59, 0x41, 0x4b,
0x54, 0x59, 0x41, 0x4b, 0x53, 0x54, 0x59, 0x45, 0x4b, 0x54, 0x59, 0x45,
0x4b, 0x53, 0x54, 0x4d, 0x45, 0x5a, 0x4d, 0x45, 0x53, 0x5a, 0x4f, 0x45,
0x5a, 0x4f, 0x45, 0x53, 0x5a, 0x57, 0x45, 0x5a, 0x57, 0x45, 0x53, 0x5a,
0x53, 0x45, 0x10c, 0x53, 0x45, 0x4c, 0x10c, 0x2205, 0x2205, 0x2205, 0x41, 0x43,
0x57, 0x54, 0x41, 0x45, 0x54, 0x47, 0x75, 0x6c, 0x66, 0x20, 0x53, 0x54,
0x4c, 0x48, 0x54, 0x4c, 0x48, 0x44, 0x54, 0x4e, 0x5a, 0x54, 0x47, 0x75,
0x6c, 0x66, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x49, 0x54, 0x41, 0x57,
0x49, 0x42, 0x49, 0x52, 0x44, 0x54, 0x50, 0x4d, 0x54, 0x56, 0x45, 0x54,
0x48, 0x4b, 0x53, 0x54, 0x43, 0x48, 0x41, 0x54, 0x48, 0x41, 0x4b, 0x48,
0x4e, 0x41, 0x4b, 0x48, 0x45, 0x41, 0x4b, 0x48, 0x43, 0x48, 0x4e, 0x43,
0x48, 0x45, 0x43, 0x48, 0x4e, 0x45, 0x48, 0x45, 0x45, 0x48, 0x52, 0x48,
0x4e, 0x52, 0x48, 0x45, 0x52, 0x48, 0x50, 0x48, 0x4e, 0x50, 0x48, 0x45,
0x50, 0x48, 0x43, 0x55, 0x48, 0x4e, 0x43, 0x55, 0x48, 0x45, 0x43, 0x55,
0x45, 0x45, 0x44, 0x54, 0x57, 0x45, 0x44, 0x54, 0x48, 0x45, 0x47, 0x48,
0x4e, 0x45, 0x47, 0x48, 0x45, 0x45, 0x47, 0x48, 0x4f, 0x47, 0x48, 0x4e,
0x4f, 0x47, 0x48, 0x45, 0x4f, 0x47, 0x48, 0x48, 0x41, 0x48, 0x4e, 0x48,
0x41, 0x48, 0x45, 0x48, 0x41, 0x48, 0x50, 0x4d, 0x58, 0x48, 0x4e, 0x50,
0x4d, 0x58, 0x48, 0x45, 0x50, 0x4d, 0x58, 0x48, 0x54, 0x4e, 0x48, 0x4e,
0x54, 0x4e, 0x48, 0x45, 0x54, 0x4e, 0x48, 0x41, 0x43, 0x48, 0x41, 0x45,
0x48, 0x41, 0x52, 0x48, 0x41, 0x50, 0x43, 0x54, 0x20, 0x28, 0x4b, 0x75,
0x62, 0x61, 0x29, 0x43, 0x53, 0x54, 0x20, 0x28, 0x4b, 0x75, 0x62, 0x61,
0x29, 0x43, 0x44, 0x54, 0x20, 0x28, 0x4b, 0x75, 0x62, 0x61, 0x29, 0x45,
0x47, 0x44, 0x54, 0x41, 0x41, 0x43, 0x41, 0x43, 0x41, 0x43, 0x41, 0x53,
0x41, 0x43, 0x4a, 0x44, 0x54, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x649, 0x627,
0x644, 0x645, 0x627, 0x62a, 0x649, 0x20, 0x642, 0x627, 0x644, 0x627, 0x633, 0x649,
0x627, 0x642, 0x62a, 0x627, 0x6cb, 0x627, 0x642, 0x62a, 0x627, 0x6cb, 0x20, 0x642,
0x627, 0x644, 0x627, 0x633, 0x649, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x6d5, 0x627,
0x642, 0x62a, 0x648, 0x628, 0x6d5, 0x20, 0x642, 0x627, 0x644, 0x627, 0x633, 0x649,
0x642, 0x627, 0x632, 0x627, 0x642, 0x20, 0x6d5, 0x644, 0x649, 0x634, 0x649, 0x639,
0x649, 0x634, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642, 0x20, 0x6d5, 0x644, 0x649,
0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642, 0x20,
0x6d5, 0x644, 0x649, 0x642, 0x649, 0x631, 0x639, 0x649, 0x632, 0x633, 0x62a, 0x627,
0x646, 0x642, 0x649, 0x632, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x627, 0x642, 0x649,
0x632, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x627, 0x20, 0x642, 0x627, 0x644, 0x627,
0x633, 0x649, 0x53, 0x53, 0x41, 0x4b, 0x53, 0x48, 0x41, 0x4b, 0x53, 0x4e,
0x53, 0x53, 0x4e, 0x53, 0x48, 0x4e, 0x53, 0x53, 0x52, 0x53, 0x48, 0x52,
0x53, 0xc7, 0x53, 0x53, 0xc7, 0x53, 0x48, 0xc7, 0x53, 0x50, 0x53, 0x53,
0x50, 0x53, 0x48, 0x50, 0x53, 0x53, 0x4b, 0x53, 0x48, 0x4b, 0x53, 0x47,
0x52, 0x53, 0x53, 0x47, 0x52, 0x53, 0x48, 0x47, 0x52, 0x53, 0x47, 0x52,
0x4f, 0x53, 0x53, 0x47, 0x52, 0x4f, 0x53, 0x48, 0x47, 0x52, 0x4f, 0x53,
0x48, 0x41, 0x4c, 0x53, 0x53, 0x48, 0x41, 0x53, 0x48, 0x48, 0x41, 0x53,
0x4e, 0x46, 0x53, 0x53, 0x4e, 0x46, 0x53, 0x48, 0x4e, 0x46, 0x50, 0x45,
0x53, 0x54, 0x4f, 0x41, 0x4b, 0x4f, 0x49, 0x41, 0x4b, 0x4f, 0x4c, 0x41,
0x4b, 0x4f, 0x49, 0x54, 0x4f, 0x4f, 0x4f, 0x49, 0x4f, 0x4f, 0x4c, 0x4f,
0x4f, 0x4d, 0x50, 0x4f, 0x49, 0x4d, 0x50, 0x4f, 0x4c, 0x4d, 0x50, 0x4f,
0x50, 0x4f, 0x49, 0x50, 0x4f, 0x4c, 0x50, 0x4f, 0x4d, 0x47, 0x43, 0x75,
0x54, 0x43, 0x75, 0x53, 0x54, 0x43, 0x75, 0x44, 0x54, 0x4a, 0x43, 0x41,
0x53, 0x4a, 0x43, 0x41, 0x4a, 0x43, 0x41, 0x4b, 0x52, 0x4a, 0x54, 0x41,
0x53, 0x4a, 0x54, 0x41, 0x4a, 0x54, 0x41, 0x4b, 0x52, 0x4a, 0x59, 0x41,
0x53, 0x4a, 0x59, 0x41, 0x4a, 0x48, 0x59, 0x41, 0x53, 0x4a, 0x41, 0x44,
0x4a, 0x41, 0x44, 0x4b, 0x52, 0x4a, 0x59, 0x54, 0x53, 0x4a, 0x59, 0x54,
0x4a, 0x59, 0x54, 0x4b, 0x52, 0x4a, 0x43, 0x4b, 0x52, 0x4a, 0x54, 0x4b,
0x52, 0x4a, 0x53, 0x4b, 0x52, 0x4a, 0x51, 0x47, 0x47, 0x6b
};

static inline constexpr char16_t longMetaZoneNameTable[] = {
0x41, 0x63, 0x72, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x63, 0x72,
0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x66, 0x67, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x6f,
0x75, 0x74, 0x68, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x75,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x41, 0x70, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x70,
0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x61, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x71,
0x74, 0x61, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x61,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x61, 0x75, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x6f, 0x62,
0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57,
0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42,
0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x61, 0x73, 0x65, 0x79, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d,
0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x48,
0x61, 0x6c, 0x66, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x75, 0x62, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x44, 0x61, 0x76, 0x69, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x63, 0x75, 0x61,
0x64, 0x6f, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x75,
0x72, 0x74, 0x68, 0x65, 0x72, 0x2d, 0x65, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x69, 0x6a, 0x69,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68,
0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72, 0x6e, 0x20, 0x26, 0x20,
0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63,
0x68, 0x20, 0x4d, 0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73,
0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65,
0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73,
0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47,
0x75, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x75, 0x6c, 0x66, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47,
0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48,
0x6f, 0x76, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x4f,
0x63, 0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x6e, 0x64,
0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4c, 0x61, 0x6e, 0x6b,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d,
0x61, 0x63, 0x61, 0x6f, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x63,
0x61, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x73,
0x63, 0x6f, 0x77, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x77, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x79,
0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x61,
0x75, 0x72, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77,
0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x69, 0x75, 0x65, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e,
0x6f, 0x72, 0x74, 0x68, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x72,
0x75, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x50, 0x65, 0x72, 0x75, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x65, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x65, 0x79,
0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53,
0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x6f, 0x75, 0x74, 0x68,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x61, 0x6b, 0x65, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x26, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x75, 0x6b,
0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x410, 0x43a, 0x440, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x440, 0x438, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x43a, 0x440, 0x438, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x444, 0x433,
0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x410, 0x433, 0x4d9, 0x4ad, 0x430,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41c,
0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x410, 0x43b, 0x430, 0x434, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41c, 0x440, 0x430,
0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9,
0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430,
0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43b, 0x438, 0x430, 0x441,
0x43a, 0x430, 0x410, 0x43b, 0x438, 0x430, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x410, 0x43b, 0x438, 0x430, 0x441, 0x43a, 0x430, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442,
0x430, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41c,
0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20,
0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x410, 0x448, 0x44c, 0x445, 0x430, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x20, 0x28, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x410, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x448, 0x44c, 0x445,
0x430, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x20, 0x28, 0x410,
0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x29, 0x410, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x448, 0x44c, 0x445, 0x430, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x20, 0x28, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x410, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x4ad, 0x44b,
0x43d, 0x447, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x20, 0x410, 0x43d, 0x430, 0x434,
0x44b, 0x440, 0x20, 0x430, 0x4bf, 0x44b, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x43f, 0x438, 0x430, 0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410,
0x43a, 0x442, 0x430, 0x443, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a,
0x442, 0x430, 0x443, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x421, 0x430, 0x443, 0x434, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x440,
0x430, 0x431, 0x441, 0x4ad, 0x430, 0x43d, 0x421, 0x430, 0x443, 0x434, 0x442, 0x4d9,
0x438, 0x20, 0x410, 0x440, 0x430, 0x431, 0x441, 0x4ad, 0x430, 0x43d, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x421, 0x430, 0x443, 0x434, 0x442, 0x4d9, 0x438, 0x20,
0x410, 0x440, 0x430, 0x431, 0x441, 0x4ad, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9,
0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x415, 0x440, 0x43c, 0x430,
0x43d, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x415, 0x440, 0x43c, 0x430, 0x43d, 0x442,
0x4d9, 0x44b, 0x43b, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x415, 0x440,
0x43c, 0x430, 0x43d, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x410, 0x433, 0x4d9,
0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433,
0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9,
0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x43c, 0x440, 0x430, 0x4ad, 0x430, 0x448,
0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x43c, 0x440,
0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c,
0x20, 0x43c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b,
0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430,
0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x4ad,
0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x438, 0x45f,
0x44c, 0x430, 0x43d, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x438, 0x45f, 0x44c,
0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x437, 0x435, 0x440,
0x431, 0x430, 0x438, 0x45f, 0x44c, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410,
0x437, 0x43e, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430,
0x49b, 0x4d9, 0x430, 0x410, 0x437, 0x43e, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x410, 0x437, 0x43e, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d,
0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411, 0x443,
0x442, 0x430, 0x43d, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x430, 0x411, 0x440,
0x430, 0x437, 0x438, 0x43b, 0x438, 0x430, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411, 0x440, 0x430, 0x437,
0x438, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411, 0x440, 0x443, 0x43d,
0x435, 0x438, 0x2d, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441, 0x430, 0x43b, 0x430,
0x43c, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x41a,
0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434,
0x435, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x435, 0x438, 0x441, 0x438, 0x427, 0x430,
0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x427, 0x430,
0x442, 0x435, 0x43c, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x427, 0x430, 0x442,
0x435, 0x43c, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x427, 0x438, 0x43b, 0x438, 0x427, 0x438,
0x43b, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x427, 0x438, 0x43b, 0x438,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x438, 0x442, 0x430, 0x438, 0x41a, 0x438, 0x442,
0x430, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x438, 0x442, 0x430,
0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x49a,
0x44c, 0x44b, 0x440, 0x441, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41a, 0x43e,
0x43b, 0x443, 0x43c, 0x431, 0x438, 0x430, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431,
0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x443, 0x43a, 0x20,
0x438, 0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430,
0x49b, 0x4d9, 0x430, 0x41a, 0x443, 0x43a, 0x20, 0x438, 0x434, 0x433, 0x44c, 0x44b,
0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x443, 0x43a, 0x20, 0x438, 0x434, 0x433, 0x44c,
0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c,
0x20, 0x43f, 0x43e, 0x43b, 0x443, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x443, 0x431, 0x430, 0x41a,
0x443, 0x431, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x443, 0x431,
0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x414, 0x435, 0x438, 0x432, 0x438, 0x441, 0x414,
0x438, 0x443, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x418, 0x443, 0x440, 0x432,
0x438, 0x43b, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x430, 0x434, 0x2d, 0x445,
0x430, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x430, 0x434, 0x2d, 0x445, 0x430,
0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x410, 0x433,
0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433,
0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430,
0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9,
0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9,
0x430, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x424, 0x438, 0x45f, 0x438, 0x424, 0x438, 0x45f, 0x438, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x438, 0x45f, 0x438, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x44b, 0x437, 0x442, 0x4d9, 0x438, 0x20,
0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x44b,
0x437, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43b, 0x430, 0x434, 0x430, 0x442, 0x4d9,
0x435, 0x438, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x43a,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x440, 0x4b5, 0x430, 0x43a, 0x44b, 0x440, 0x430,
0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x49b, 0x4d9, 0x430, 0x413, 0x430, 0x43b, 0x430,
0x43f, 0x430, 0x433, 0x43e, 0x441, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d,
0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x413, 0x430, 0x43c, 0x431, 0x435, 0x49a, 0x44b,
0x440, 0x4ad, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x49a, 0x44b, 0x440, 0x4ad, 0x442,
0x4d9, 0x44b, 0x43b, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x49a, 0x44b,
0x440, 0x4ad, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x430,
0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x20, 0x413, 0x438, 0x43b, 0x431,
0x435, 0x440, 0x442, 0x430, 0x410, 0x433, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447,
0x20, 0x418, 0x431, 0x436, 0x44c, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x443, 0x20,
0x410, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x438, 0x430, 0x41c, 0x440,
0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430,
0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x413,
0x443, 0x430, 0x43c, 0x410, 0x45f, 0x44c, 0x430, 0x43c, 0x442, 0x4d9, 0x20, 0x430,
0x4e1, 0x44b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x43b, 0x430, 0x413, 0x430,
0x438, 0x430, 0x43d, 0x430, 0x4b2, 0x430, 0x432, 0x430, 0x438, 0x2d, 0x430, 0x43b,
0x435, 0x443, 0x442, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x4b2, 0x430, 0x432, 0x430, 0x438, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x4b2, 0x430, 0x432, 0x430, 0x438,
0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x4b2, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x4b2, 0x43e, 0x43d, 0x43a, 0x43e,
0x43d, 0x433, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x4b2, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x425, 0x43e, 0x432, 0x434, 0x425,
0x43e, 0x432, 0x434, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x425, 0x43e, 0x432,
0x434, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x418, 0x43d,
0x434, 0x438, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x442, 0x430, 0x438, 0x410, 0x433,
0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x418, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437,
0x438, 0x430, 0x418, 0x440, 0x430, 0x43d, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x418, 0x440, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x437, 0x440, 0x430, 0x438,
0x43b, 0x44c, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x418, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x430, 0x418, 0x430, 0x43f,
0x43e, 0x43d, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x430,
0x43f, 0x43e, 0x43d, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x435, 0x442,
0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a,
0x430, 0x43c, 0x447, 0x430, 0x442, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x41f, 0x435,
0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d,
0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430,
0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430,
0x442, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440,
0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x49e,
0x430, 0x437, 0x430, 0x445, 0x441, 0x4ad, 0x430, 0x43d, 0x41c, 0x440, 0x430, 0x4ad,
0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x49e, 0x430,
0x437, 0x430, 0x445, 0x441, 0x4ad, 0x430, 0x43d, 0x41a, 0x43e, 0x440, 0x435, 0x438,
0x430, 0x41a, 0x43e, 0x440, 0x435, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41a, 0x43e, 0x440, 0x435, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a,
0x43e, 0x441, 0x440, 0x430, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x438,
0x430, 0x440, 0x441, 0x43a, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x438, 0x430,
0x440, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x438, 0x430, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41a, 0x438, 0x440, 0x433, 0x438, 0x437, 0x438, 0x430, 0x428, 0x440, 0x438, 0x2d,
0x41b, 0x430, 0x43d, 0x43a, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9,
0x430, 0x20, 0x41b, 0x430, 0x438, 0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x4b2,
0x430, 0x443, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x4b2, 0x430, 0x443, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x4b2, 0x430, 0x443,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x41c, 0x430, 0x43a,
0x430, 0x43e, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x43a, 0x430,
0x43e, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41c, 0x430, 0x43b, 0x430, 0x438, 0x437, 0x438, 0x430, 0x41c, 0x430, 0x43b, 0x44c,
0x434, 0x438, 0x432, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41c, 0x430, 0x440,
0x448, 0x430, 0x43b, 0x43b, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x434, 0x433, 0x44c,
0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41c,
0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a,
0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x43a, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d,
0x410, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x43c, 0x435, 0x43a,
0x441, 0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442,
0x43e, 0x440, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430,
0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x438,
0x430, 0x43d, 0x43c, 0x430, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x41d, 0x435, 0x43f,
0x430, 0x43b, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x430, 0x20,
0x4bf, 0x44b, 0x446, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x430,
0x20, 0x4bf, 0x44b, 0x446, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x430,
0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x430, 0x20, 0x4bf, 0x44b, 0x446, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x20,
0x4bf, 0x44b, 0x446, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x20,
0x4bf, 0x44b, 0x446, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x430, 0x20, 0x4bf, 0x44b, 0x446, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41d, 0x438, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d,
0x434, 0x41d, 0x438, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d,
0x434, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41d, 0x438, 0x443, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41d,
0x438, 0x443, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x435,
0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e,
0x440, 0x43e, 0x43d, 0x438, 0x430, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x438, 0x430,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d,
0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x438,
0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x41c, 0x430, 0x440, 0x438, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x41e, 0x43c, 0x441, 0x43a, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x41f, 0x430, 0x43a,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f,
0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f,
0x430, 0x43b, 0x430, 0x443, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x2014, 0x20,
0x413, 0x432, 0x438, 0x43d, 0x435, 0x438, 0x430, 0x20, 0x4bf, 0x44b, 0x446, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x438, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x435, 0x440,
0x443, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f,
0x435, 0x440, 0x443, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x438, 0x43b, 0x438, 0x43f,
0x43f, 0x438, 0x43d, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438,
0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9,
0x430, 0x20, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x421, 0x435, 0x43d, 0x2d,
0x41f, 0x438, 0x435, 0x440, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x438, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x438, 0x435, 0x440, 0x438, 0x20,
0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x438, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x438, 0x435, 0x440, 0x438, 0x20,
0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x438, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x41f, 0x43e, 0x43d, 0x43f, 0x435,
0x438, 0x41f, 0x445, 0x435, 0x43d, 0x438, 0x430, 0x43d, 0x41a, 0x44b, 0x437, 0x44b,
0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2a, 0x41a, 0x44b, 0x437, 0x44b, 0x43b, 0x43e,
0x440, 0x434, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x2a, 0x41a, 0x44b,
0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x2a,
0x420, 0x435, 0x438, 0x443, 0x43d, 0x43e, 0x43d, 0x420, 0x43e, 0x442, 0x435, 0x440,
0x430, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421, 0x430, 0x445,
0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x20, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x4bf, 0x44b, 0x421, 0x430,
0x43c, 0x430, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421,
0x430, 0x43c, 0x430, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421, 0x430,
0x43c, 0x43e, 0x430, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421,
0x435, 0x438, 0x448, 0x435, 0x43b, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x433,
0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430,
0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x421, 0x43e, 0x43b, 0x43e,
0x43c, 0x43e, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x433, 0x44c, 0x44b,
0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x410, 0x43b,
0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433,
0x438, 0x430, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x421, 0x438, 0x43e,
0x432, 0x430, 0x422, 0x430, 0x438, 0x442, 0x438, 0x422, 0x430, 0x438, 0x432, 0x430,
0x43d, 0x422, 0x430, 0x438, 0x432, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x422, 0x430, 0x438, 0x432, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x422,
0x430, 0x45f, 0x44c, 0x44b, 0x49b, 0x441, 0x4ad, 0x430, 0x43d, 0x422, 0x43e, 0x43a,
0x435, 0x43b, 0x430, 0x443, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x422, 0x43e, 0x43d,
0x433, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x422, 0x43e, 0x43d, 0x433,
0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x422, 0x440, 0x443, 0x43a, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x422, 0x443, 0x432, 0x430, 0x43b,
0x443, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x438, 0x423, 0x440, 0x443, 0x433,
0x432, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x423, 0x437, 0x431, 0x435,
0x49b, 0x44c, 0x438, 0x441, 0x4ad, 0x430, 0x43d, 0x423, 0x437, 0x431, 0x435, 0x49b,
0x44c, 0x438, 0x441, 0x4ad, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x423, 0x437, 0x431, 0x435, 0x49b, 0x44c, 0x438, 0x441, 0x4ad, 0x430, 0x43d, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x412,
0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x412, 0x435,
0x43d, 0x435, 0x441, 0x443, 0x435, 0x43b, 0x430, 0x412, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x412,
0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x410, 0x43c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440,
0x430, 0x423, 0x435, 0x438, 0x43a, 0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x438,
0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x435, 0x438, 0x418, 0x430, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x418, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x415, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x418, 0x443, 0x43a, 0x43e, 0x43d, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x2d,
0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x75, 0x69, 0x64, 0x2d, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79,
0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41, 0x6d,
0x61, 0x73, 0x6f, 0x6e, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x41, 0x6d, 0x61,
0x73, 0x6f, 0x6e, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x65,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f,
0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20,
0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x61, 0x67,
0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x6f, 0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x74, 0x79,
0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x6f, 0x6f, 0x73, 0x74, 0x65,
0x6c, 0x69, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x6f,
0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x64, 0x61, 0x67,
0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x62, 0x65, 0x72, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64,
0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x62, 0x65, 0x72, 0x67, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x62, 0x65, 0x72, 0x67, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74,
0x79, 0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73, 0x65, 0x20,
0x74, 0x79, 0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73, 0x65, 0x20, 0x64,
0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x2d, 0x74, 0x79, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x74, 0x79, 0x64,
0x41, 0x70, 0x69, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x64, 0x61,
0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69,
0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69,
0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x65, 0x73, 0x65,
0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64,
0x57, 0x65, 0x73, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x69, 0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0xeb, 0x2d, 0x74, 0x79,
0x64, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79,
0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x65, 0x73, 0x65, 0x20,
0x74, 0x79, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x65, 0x73,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x65, 0x73, 0x65,
0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x53, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x53, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x73, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x77, 0x65,
0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x53,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x77, 0x65, 0x73, 0x74,
0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x2d, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73,
0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4f,
0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64,
0x4f, 0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4f, 0x6f,
0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c,
0x69, 0x67, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69,
0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x74, 0x65, 0x6c,
0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73,
0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41,
0x73, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x2d, 0x74,
0x79, 0x64, 0x41, 0x73, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x6a, 0x61,
0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x41, 0x73, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x6a, 0x61,
0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x41, 0x73,
0x6f, 0x72, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x41, 0x73, 0x6f, 0x72, 0x65,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x6a, 0x2d, 0x74, 0x79, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x6a, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x6a, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x42,
0x68, 0x6f, 0x65, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x42, 0x6f,
0x6c, 0x69, 0x76, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x42,
0x72, 0x6f, 0x65, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x2d, 0x74, 0x79, 0x64, 0x4b, 0x61, 0x61,
0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x4b,
0x61, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4b, 0x61,
0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x74, 0x79,
0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74,
0x79, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x2d, 0x74, 0x79, 0x64, 0x43,
0x68, 0x69, 0x6c, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x2d, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x2d, 0x74, 0x79, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x65, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0xeb, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x2d, 0x74, 0x79, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x2d, 0x68, 0x61, 0x6c, 0x66, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x4b, 0x75, 0x62, 0x61, 0x2d, 0x74, 0x79,
0x64, 0x4b, 0x75, 0x62, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4b, 0x75, 0x62, 0x61, 0x2d, 0x64,
0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x44, 0x61, 0x76, 0x69,
0x73, 0x2d, 0x74, 0x79, 0x64, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x74, 0x79,
0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x2d, 0x74,
0x79, 0x64, 0x50, 0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x74, 0x79, 0x64, 0x50, 0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x50, 0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x45, 0x63,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x56, 0x65, 0x72, 0x64, 0x65, 0x72, 0x2d, 0x6f,
0x6f, 0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20,
0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x2d, 0x74, 0x79,
0x64, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x46, 0x69, 0x64, 0x6a, 0x69,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x46, 0x72, 0x61,
0x6e, 0x73, 0x2d, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x2d, 0x74, 0x79,
0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x53, 0x75, 0x69, 0x64,
0x65, 0x72, 0x2d, 0x20, 0x65, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72,
0x6b, 0x74, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x47, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x2d, 0x74, 0x79, 0x64, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d, 0x74, 0x79, 0x64, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xeb,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d,
0x74, 0x79, 0x64, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x2d, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73,
0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4f,
0x6f, 0x73, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73,
0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74,
0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x50, 0x65, 0x72, 0x73, 0x69, 0x65, 0x73, 0x65, 0x20, 0x47,
0x6f, 0x6c, 0x66, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x6f, 0x65, 0x74, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x65,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f,
0x65, 0x74, 0x65, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x74, 0x79,
0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x48, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x48,
0x6f, 0x76, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x49, 0x6e, 0x64, 0x69, 0xeb, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x49,
0x6e, 0x64, 0x69, 0x65, 0x73, 0x65, 0x20, 0x4f, 0x73, 0x65, 0x61, 0x61,
0x6e, 0x2d, 0x74, 0x79, 0x64, 0x49, 0x6e, 0x64, 0x6f, 0x73, 0x6a, 0x69,
0x6e, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x65,
0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x57,
0x65, 0x73, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0xeb,
0x2d, 0x74, 0x79, 0x64, 0x49, 0x72, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64,
0x49, 0x72, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x49, 0x72, 0x61, 0x6e, 0x2d, 0x64, 0x61,
0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65,
0x74, 0x73, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65,
0x74, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x2d, 0x74, 0x79, 0x64, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d, 0x64, 0x61, 0x67,
0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x2d,
0x74, 0x79, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x2d, 0x74, 0x79, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x2d,
0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79,
0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74,
0x79, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x64, 0x61,
0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x2d, 0x74, 0x79, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4c, 0x69, 0x6e, 0x65, 0x2d, 0x65, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67,
0x74, 0x79, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x74,
0x79, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x4d, 0x61, 0x6c, 0x65, 0x69, 0x73, 0x69, 0xeb, 0x2d, 0x74,
0x79, 0x64, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x76, 0x65, 0x2d, 0x74,
0x79, 0x64, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x2d,
0x74, 0x79, 0x64, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x74, 0x79, 0x64, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x2d, 0x74, 0x79,
0x64, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74,
0x79, 0x64, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73, 0x65, 0x20, 0x64, 0x61,
0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x74, 0x79, 0x64, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75,
0x2d, 0x74, 0x79, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4d,
0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x4d, 0x69, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x2d, 0x74, 0x79,
0x64, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0xeb, 0x2d, 0x74, 0x79,
0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d, 0x53, 0x65, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d,
0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x69, 0x65, 0x75,
0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x64, 0x61, 0x67,
0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d,
0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x69, 0x75,
0x65, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x79,
0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4f, 0x6d, 0x73,
0x6b, 0x2d, 0x74, 0x79, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4f, 0x6d,
0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79,
0x64, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x61,
0x70, 0x6f, 0x65, 0x61, 0x2d, 0x4e, 0x69, 0x65, 0x75, 0x2d, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x50, 0x65,
0x72, 0x75, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x65, 0x72, 0x75, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x50,
0x65, 0x72, 0x75, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x79, 0x6e, 0x73, 0x65, 0x20, 0x74,
0x79, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x79, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x79, 0x6e, 0x73, 0x65, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x46, 0x65, 0x6e, 0x69,
0x6b, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79,
0x64, 0x53, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x2d, 0x74, 0x79, 0x64, 0x53, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x53, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x2d, 0x74, 0x79,
0x64, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x2d, 0x74, 0x79, 0x64,
0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d,
0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67,
0x74, 0x79, 0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65,
0x2d, 0x74, 0x79, 0x64, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x65,
0x72, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x65, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x75, 0x69,
0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xeb, 0x2d, 0x74, 0x79,
0x64, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x2d, 0x74, 0x79,
0x64, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x2d, 0x74, 0x79, 0x64, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x2d, 0x74, 0x79, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x64, 0x61, 0x67, 0x6c,
0x69, 0x67, 0x74, 0x79, 0x64, 0x54, 0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x2d, 0x74, 0x79, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x2d, 0x74, 0x79, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79,
0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x2d, 0x74, 0x79, 0x64, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x2d, 0x74, 0x79, 0x64, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x2d,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4f, 0x65, 0x73, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4f,
0x65, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4f,
0x65, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x2d, 0x74, 0x79, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x65, 0x6c, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x6c, 0x61, 0x64, 0x69,
0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x6c,
0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x6c,
0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x57,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x61, 0x6b,
0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x6e, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x4a, 0x61, 0x6b, 0x6f,
0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x4a, 0x61, 0x6b, 0x6f,
0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73,
0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d,
0x74, 0x79, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x74, 0x79, 0x64, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x46, 0x69, 0x6e,
0x69, 0x6d, 0x66, 0x69, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x41,
0x6e, 0x61, 0x61, 0x66, 0x6f, 0x254, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64,
0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x41, 0x74, 0x254, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65, 0x25b,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65,
0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x66, 0x69, 0x6e, 0x69,
0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x66,
0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x53, 0x75, 0x73,
0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x66, 0x69,
0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x41, 0x77, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x42, 0x65, 0x70, 0x254, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x42, 0x65, 0x70, 0x254, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42, 0x65, 0x70, 0x254, 0x20, 0x41,
0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x70, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x70, 0x69, 0x61, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41,
0x70, 0x69, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64,
0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41,
0x67, 0x79, 0x25b, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x41, 0x67, 0x79, 0x25b, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41,
0x67, 0x79, 0x25b, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x67, 0x79, 0x25b, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x41, 0x67, 0x79, 0x25b, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x67, 0x79, 0x25b,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x61,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x61,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x61, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x53, 0x75, 0x73,
0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20,
0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20,
0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x186,
0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0x66, 0x69, 0x6e,
0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61,
0x20, 0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20,
0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x186,
0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0x66, 0x69, 0x6e,
0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65,
0x25b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20, 0x4d,
0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x41, 0x74,
0x254, 0x65, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20, 0x41,
0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x186, 0x73,
0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65,
0x25b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20, 0x41,
0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20,
0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x186,
0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65,
0x65, 0x25b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x186, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x61, 0x20,
0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x41, 0x73, 0x61, 0x62, 0x65, 0x67, 0x79, 0x61,
0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x73, 0x61, 0x62, 0x65, 0x67,
0x79, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x41, 0x73, 0x61, 0x62, 0x65, 0x67, 0x79, 0x61,
0x6e, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x25b, 0x68, 0x79, 0x65, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x25b, 0x68, 0x79, 0x65,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x25b, 0x68, 0x79, 0x65,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42, 0x75,
0x74, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x65, 0x70,
0x76, 0x25b, 0x64, 0x65, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x65, 0x70,
0x76, 0x25b, 0x64, 0x65, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x65, 0x70, 0x76, 0x25b, 0x64, 0x65,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79,
0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x61, 0x74, 0x61, 0x6d,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x61, 0x74, 0x61, 0x6d, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4b, 0x79, 0x61, 0x74, 0x61, 0x6d, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x69, 0x6c, 0x69, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4b, 0x79, 0x69, 0x6c, 0x69, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x69, 0x6c,
0x69, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b,
0x79, 0x61, 0x65, 0x6e, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79,
0x61, 0x65, 0x6e, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x61, 0x65, 0x6e, 0x61, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x42, 0x75, 0x72,
0x6f, 0x6e, 0x79, 0x61, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x20, 0x41,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4b, 0x75, 0x6b, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x75, 0x6b, 0x20, 0x41, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x4b, 0x75, 0x6b, 0x20, 0x41, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x46, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4b, 0x75, 0x62, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x44, 0x61, 0x76, 0x69, 0x73,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x54, 0x69, 0x6d, 0x254, 0x254, 0x20, 0x41, 0x70, 0x75, 0x65,
0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x45, 0x61, 0x73, 0x74, 0x61,
0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x45, 0x61, 0x73, 0x74, 0x61, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x45, 0x61, 0x73, 0x74, 0x61, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59,
0x69, 0x6b, 0x75, 0x77, 0x65, 0x64, 0x254, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20, 0x4d, 0x66, 0x69, 0x6e, 0x69,
0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75,
0x72, 0x6f, 0x70, 0x75, 0x20, 0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66,
0x69, 0x6e, 0x69, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20, 0x4d,
0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69, 0x6e, 0x69, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70,
0x75, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20, 0x41, 0x70, 0x75, 0x65,
0x65, 0x25b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20, 0x41, 0x70,
0x75, 0x65, 0x65, 0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20, 0x41, 0x70, 0x75,
0x65, 0x65, 0x25b, 0x20, 0x4e, 0x6f, 0x68, 0x6f, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x75, 0x20, 0x41, 0x74, 0x254,
0x65, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x6f,
0x70, 0x75, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x53, 0x75,
0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75,
0x72, 0x6f, 0x70, 0x75, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x254, 0x6b,
0x6d, 0x61, 0x6e, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x46, 0x254, 0x6b, 0x6d, 0x61, 0x6e, 0x20, 0x41, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x254, 0x6b, 0x6d, 0x61, 0x6e, 0x20,
0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x46, 0x69, 0x67, 0x79, 0x69, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x46, 0x69, 0x67, 0x79, 0x69, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x69, 0x67, 0x79,
0x69, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46,
0x72, 0x25b, 0x6e, 0x6b, 0x79, 0x65, 0x20, 0x47, 0x61, 0x79, 0x61, 0x6e,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x72, 0x25b, 0x6e, 0x6b, 0x79,
0x65, 0x20, 0x41, 0x6e, 0x61, 0x61, 0x66, 0x6f, 0x254, 0x20, 0x6e, 0x65,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x61, 0x74, 0x69, 0x6b, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x254, 0x73, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x47, 0x79, 0x254, 0x67, 0x79, 0x65, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x47, 0x79, 0x254, 0x67, 0x79, 0x65, 0x61, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x47, 0x79, 0x254, 0x67, 0x79, 0x65, 0x61, 0x20, 0x41, 0x77, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x65, 0x65, 0x62, 0x25b, 0x74, 0x20,
0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x4d, 0x65, 0x61,
0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x74, 0x254, 0x65,
0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x41, 0x74, 0x254, 0x65, 0x25b, 0x20, 0x53, 0x75,
0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x74, 0x254, 0x65,
0x25b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47,
0x254, 0x254, 0x66, 0x6f, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x48, 0x254, 0x6e, 0x6b, 0x254, 0x6e,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x48, 0x254, 0x6e, 0x6b, 0x254, 0x6e, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x48, 0x254, 0x6e, 0x6b, 0x254, 0x6e, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x41,
0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x49, 0x6e, 0x64, 0x254, 0x6b, 0x79, 0x69, 0x6e, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x6e, 0x64, 0x254, 0x6e, 0x65, 0x68,
0x79, 0x69, 0x61, 0x20, 0x4d, 0x66, 0x69, 0x6e, 0x69, 0x6d, 0x66, 0x69,
0x6e, 0x69, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x6e, 0x64, 0x254, 0x6e,
0x65, 0x68, 0x79, 0x69, 0x61, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x6e, 0x64, 0x254, 0x6e, 0x65, 0x68,
0x79, 0x69, 0x61, 0x20, 0x41, 0x74, 0x254, 0x65, 0x65, 0x25b, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x49, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x41,
0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x79, 0x61, 0x70, 0x61, 0x6e,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x47, 0x79, 0x61, 0x70, 0x61, 0x6e, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x47, 0x79, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x41, 0x70, 0x75, 0x65, 0x65, 0x25b, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x41, 0x74, 0x254, 0x65, 0x25b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0x65, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x25b, 0x25b,
0x67, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4c,
0x61, 0x69, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4c, 0x254, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4c, 0x254, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4c, 0x254, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61, 0x6c, 0x65, 0x68,
0x79, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x65, 0x73, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61, 0x6b,
0x65, 0x73, 0x61, 0x73, 0x65, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61,
0x68, 0x79, 0x61, 0x61, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x4d, 0x254, 0x72, 0x69, 0x68, 0x79, 0x69, 0x254,
0x73, 0x6f, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x254, 0x72, 0x69, 0x68,
0x79, 0x69, 0x254, 0x73, 0x6f, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x254, 0x72, 0x69, 0x68, 0x79,
0x69, 0x254, 0x73, 0x6f, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4d, 0x254, 0x254, 0x73, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x4d, 0x25b, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x25b, 0x6b,
0x73, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x4d, 0x25b, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x59, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74,
0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x59, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x254, 0x73,
0x6b, 0x6f, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4d, 0x254, 0x73, 0x6b, 0x6f,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x4d, 0x254, 0x73, 0x6b, 0x6f, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x4d, 0x61, 0x79, 0x61, 0x61, 0x6d, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4e, 0x25b, 0x70, 0x61, 0x6c, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x46, 0x6f,
0x66, 0x6f, 0x72, 0x254, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x46, 0x6f, 0x66, 0x6f, 0x72,
0x254, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x46, 0x6f, 0x66, 0x6f, 0x72, 0x254, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46,
0x6f, 0x66, 0x6f, 0x72, 0x254, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x5a, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0x6f, 0x66, 0x6f, 0x72, 0x254, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0x6f, 0x66, 0x6f, 0x72,
0x254, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4e, 0x254, 0x254, 0x66, 0x254, 0x6b, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e, 0x254, 0x254, 0x66, 0x254, 0x6b,
0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e, 0x254, 0x254, 0x66,
0x254, 0x6b, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x41,
0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4f, 0x6d, 0x73,
0x6b, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4f,
0x6d, 0x73, 0x6b, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x41, 0x77, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x47, 0x69, 0x6e,
0x69, 0x20, 0x46, 0x6f, 0x66, 0x6f, 0x72, 0x254, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x65, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x65, 0x20, 0x53,
0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x65, 0x20, 0x41, 0x77, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x65, 0x72, 0x75, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x50, 0x65, 0x72, 0x75, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64,
0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50, 0x65, 0x72, 0x75, 0x20,
0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x46, 0x69,
0x6e, 0x69, 0x73, 0x65, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50,
0x69, 0x74, 0x6b, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x75,
0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53,
0x65, 0x79, 0x68, 0x79, 0x25b, 0x6c, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x254, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64,
0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53, 0x6f, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x20, 0x41, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x47, 0x79, 0x254, 0x254, 0x67, 0x79, 0x69, 0x61, 0x20, 0x41,
0x6e, 0x61, 0x61, 0x66, 0x6f, 0x254, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x61, 0x67, 0x79, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54,
0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42,
0x65, 0x72, 0x25b, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4b, 0x79, 0x75, 0x75, 0x6b, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x54, 0x25b, 0x6b, 0x6d, 0x25b, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x25b, 0x6b, 0x6d,
0x25b, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x73, 0x75,
0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x25b, 0x6b, 0x6d,
0x25b, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x41, 0x77, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x75, 0x67, 0x77, 0x61, 0x65,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75, 0x72, 0x75, 0x67, 0x77, 0x61,
0x65, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x59, 0x75, 0x72, 0x75, 0x67, 0x77, 0x61, 0x65, 0x20, 0x41,
0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x55, 0x73, 0x62, 0x25b,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x55,
0x73, 0x62, 0x25b, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x75,
0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x55, 0x73,
0x62, 0x25b, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65,
0x72, 0x25b, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x41, 0x77,
0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x65, 0x6e, 0x65, 0x7a,
0x75, 0x65, 0x6c, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x53, 0x75,
0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x41, 0x77, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20,
0x42, 0x65, 0x72, 0x25b, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x57, 0x61, 0x6c, 0x6c,
0x69, 0x73, 0x20, 0x6e, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72,
0x25b, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x41, 0x77, 0x69,
0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x25b, 0x6b, 0x61, 0x74, 0x25b,
0x72, 0x69, 0x6e, 0x62, 0x25b, 0x67, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59,
0x25b, 0x6b, 0x61, 0x74, 0x25b, 0x72, 0x69, 0x6e, 0x62, 0x25b, 0x67, 0x20,
0x53, 0x75, 0x73, 0x75, 0x64, 0x75, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b,
0x59, 0x25b, 0x6b, 0x61, 0x74, 0x25b, 0x72, 0x69, 0x6e, 0x62, 0x25b, 0x67,
0x20, 0x41, 0x77, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x59, 0x75,
0x6b, 0x254, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x25b, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x45, 0x6a, 0x6b, 0x72, 0x69, 0x74, 0x20, 0x5b, 0x41, 0x6b,
0x6f, 0x5d, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x45, 0x6a, 0x6b, 0x72, 0x69, 0x74,
0x20, 0x5b, 0x41, 0x6b, 0x6f, 0x5d, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x45, 0x6a, 0x6b, 0x72, 0x69,
0x74, 0x20, 0x5b, 0x41, 0x6b, 0x6f, 0x5d, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0xeb, 0x73, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0xeb, 0x73,
0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0xeb, 0x73, 0x20, 0x4a, 0x75, 0x67, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0xeb, 0x73, 0x20, 0x50,
0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0xeb, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0xeb,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x48, 0x42, 0x41, 0x2d,
0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x53, 0x48, 0x42, 0x41, 0x2d, 0x73, 0xeb, 0x20, 0x51,
0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x53, 0x48, 0x42, 0x41,
0x2d, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x48, 0x42, 0x41, 0x2d, 0x73,
0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x53, 0x48, 0x42, 0x41, 0x2d, 0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e,
0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x53, 0x48, 0x42, 0x41, 0x2d, 0x73, 0xeb,
0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76,
0x65, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20,
0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x7a, 0x69, 0x74, 0x20, 0x4d, 0x61,
0x6c, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69,
0x74, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x7a,
0x69, 0x74, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72,
0x65, 0x7a, 0x69, 0x74, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x72, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72,
0x65, 0x76, 0x65, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75, 0x74, 0x20,
0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72,
0x65, 0x76, 0x65, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75, 0x74, 0x20,
0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76, 0x65,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74,
0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75, 0x74, 0x20, 0x74, 0xeb, 0x20,
0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41,
0x70, 0x69, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6b, 0x74,
0x61, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x6b, 0x74,
0x6f, 0x62, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f,
0x62, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x61, 0x72, 0x61, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x61, 0x72,
0x61, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x61, 0x72, 0x61, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x72, 0x67,
0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72,
0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73, 0x20,
0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73, 0x20,
0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0xeb, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64,
0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x6f,
0x2d, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x6f, 0x2d,
0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x51,
0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x6f, 0x2d, 0x50, 0x65, 0x72, 0xeb,
0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20,
0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x4c, 0x69,
0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0xeb, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69,
0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e,
0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64,
0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x78, 0x68, 0x61, 0x6e, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x78,
0x68, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x6a, 0x78, 0x68, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x42, 0x75, 0x74, 0x61,
0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x42, 0x6f, 0x6c,
0x69, 0x76, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x42,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x42, 0x72,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x2d, 0x44, 0x75, 0x72, 0x61,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x4b, 0x65, 0x70, 0x69, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x47, 0x6a,
0x65, 0x6c, 0x62, 0xeb, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x65, 0x70,
0x69, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x47, 0x6a, 0x65, 0x6c, 0x62, 0xeb,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x4b, 0x65, 0x70, 0x69, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x47,
0x6a, 0x65, 0x6c, 0x62, 0xeb, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x4b, 0x65, 0x6a, 0x73, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x4b, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x74,
0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x69,
0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x6c, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x4b, 0x69, 0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x4b, 0x69, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x6e,
0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x4b, 0x69, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74, 0x20, 0x74,
0xeb, 0x20, 0x4b, 0x72, 0x69, 0x73, 0x68, 0x74, 0x6c, 0x69, 0x6e, 0x64,
0x6a, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73,
0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x73,
0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a,
0x76, 0x65, 0x20, 0x4b, 0x75, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73,
0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4b, 0x75, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73,
0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4b, 0x75, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x4b, 0x75, 0x62, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x4b, 0x75, 0x62, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x75, 0x62, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x44, 0x65, 0x6a, 0x76, 0x69, 0x73, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x69, 0x74,
0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74, 0x20, 0x74, 0xeb,
0x20, 0x50, 0x61, 0x73, 0x68, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74, 0x20, 0x74, 0xeb, 0x20,
0x50, 0x61, 0x73, 0x68, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75,
0x6c, 0x6c, 0x69, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x73, 0x68,
0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x45, 0x6b, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x51, 0x65, 0x6e, 0x64,
0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f,
0x70, 0xeb, 0x73, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x51, 0x65, 0x6e,
0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x45,
0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb,
0x73, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x45, 0x76,
0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70,
0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x4c, 0x61, 0x72, 0x67, 0xeb, 0x74,
0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x50, 0x65,
0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72,
0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72,
0x6f, 0x70, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69,
0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73,
0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76,
0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x49,
0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x46, 0x69, 0x78,
0x68, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x46, 0x69, 0x78, 0x68, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x46, 0x69, 0x78, 0x68, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x47, 0x75, 0x61, 0x6a, 0x61, 0x6e, 0xeb, 0x73, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x63, 0x65, 0x7a, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76, 0x65,
0x20, 0x4a, 0x75, 0x67, 0x6f, 0x72, 0x65, 0x20, 0x64, 0x68, 0x65, 0x20,
0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x65, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x63, 0x65, 0x7a, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x6a, 0x65,
0x6f, 0x72, 0x67, 0x6a, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x47,
0x6a, 0x65, 0x6f, 0x72, 0x67, 0x6a, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x47, 0x6a,
0x65, 0x6f, 0x72, 0x67, 0x6a, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x47,
0x72, 0x69, 0x6e, 0x75, 0x69, 0xe7, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73,
0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x4c,
0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x72, 0xeb, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64,
0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x47, 0x72, 0xeb,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb,
0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x47, 0x72, 0xeb, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e,
0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x47, 0x75, 0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x47, 0x6a, 0x69, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x48, 0x61, 0x75, 0x61,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x48, 0x61,
0x75, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x48, 0x61, 0x75,
0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x2d, 0x4b, 0x6f,
0x6e, 0x67, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x4f, 0x71, 0x65, 0x61, 0x6e, 0x69, 0x74, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x73,
0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x73, 0xeb, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4a,
0x61, 0x70, 0x6f, 0x6e, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x6b, 0xeb,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0xe7,
0x61, 0x74, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0xe7, 0x61, 0x74, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x20, 0x50, 0x65, 0x72, 0xeb,
0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x6b, 0x6f, 0x72, 0x65, 0x61,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x69, 0x73,
0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61,
0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75,
0x6a, 0x76, 0x65, 0x20, 0x53, 0x70, 0x6f, 0x72, 0x61, 0x64, 0x69, 0x6b,
0xeb, 0x20, 0x45, 0x6b, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c,
0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d,
0x48, 0x6f, 0x75, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x2d, 0x48, 0x6f, 0x75, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x48, 0x6f, 0x75, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x61,
0x6b, 0x61, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x6a,
0x7a, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4d, 0x61, 0x72,
0x6b, 0x65, 0x7a, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73,
0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x73, 0x6f, 0x6e, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74,
0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75,
0x74, 0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74,
0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75,
0x74, 0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x65,
0x76, 0x65, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x65,
0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75, 0x74, 0x20, 0x74,
0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x55, 0x6c, 0x61,
0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0xeb, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x6f, 0x73,
0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0xeb,
0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0xeb, 0x20, 0x73, 0xeb, 0x20,
0x52, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73,
0xeb, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x73, 0xeb,
0x20, 0x52, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75,
0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x74, 0x20, 0x5b, 0x54, 0x6f,
0x6b, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x5d, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65,
0x6e, 0x64, 0x69, 0x74, 0x20, 0x5b, 0x54, 0x6f, 0x6b, 0xeb, 0x73, 0x20,
0x73, 0xeb, 0x20, 0x52, 0x65, 0x5d, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4e, 0x6a, 0x75, 0x66, 0x61,
0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x74, 0x20, 0x5b, 0x54,
0x6f, 0x6b, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x5d, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x73, 0xeb, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69,
0x74, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c,
0x69, 0x74, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0xeb, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0xeb, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65,
0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0xeb, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x74,
0xeb, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0xeb, 0x73, 0x20, 0x73,
0xeb, 0x20, 0x56, 0x65, 0x72, 0x69, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x73, 0xeb, 0x20, 0x73, 0xeb,
0x20, 0x52, 0x65, 0x2d, 0x50, 0x61, 0x70, 0x75, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x73, 0xeb, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x50, 0x65,
0x72, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x76, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x46,
0x65, 0x6e, 0x69, 0x6b, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53,
0x68, 0xeb, 0x6e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x20, 0x64, 0x68, 0x65,
0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x20, 0x64, 0x68,
0x65, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x53, 0x68,
0xeb, 0x6e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x20, 0x64, 0x68, 0x65, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x50, 0x65, 0x6e, 0x69, 0x61, 0x6e,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x7a, 0x69,
0x6c, 0x6f, 0x72, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x69,
0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x7a,
0x69, 0x6c, 0x6f, 0x72, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x52, 0x6f, 0x64, 0x68, 0x65, 0x72, 0xeb, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c,
0x69, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0xeb, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x65, 0x6a,
0x73, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76,
0x65, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x58, 0x68, 0x6f, 0x72, 0x78, 0x68, 0x61, 0x73, 0x20,
0x74, 0xeb, 0x20, 0x4a, 0x75, 0x67, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x6a, 0x6f, 0x75, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x54, 0x61, 0x78, 0x68, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0xeb, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0xc7, 0x75, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x75, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74, 0x20, 0x55,
0x65, 0x6a, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x55, 0x6f, 0x6c,
0x6c, 0x69, 0x73, 0x69, 0x74, 0x20, 0x64, 0x68, 0x65, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x75, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x75, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x69,
0x74, 0x12e8, 0x12a0, 0x134d, 0x130b, 0x1292, 0x1235, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20, 0x12a0, 0x134d, 0x122a,
0x12ab, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x12a0,
0x134d, 0x122a, 0x12ab, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12f0, 0x1261, 0x1265, 0x20,
0x12a0, 0x134d, 0x122a, 0x12ab, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12a0, 0x134d, 0x122a, 0x12ab, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12a0, 0x134d, 0x122a,
0x12ab, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d,
0x12d5, 0x122b, 0x1265, 0x20, 0x12a0, 0x134d, 0x122a, 0x12ab, 0x20, 0x12ad, 0x1228, 0x121d,
0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x120b, 0x1235, 0x12ab, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x120b, 0x1235,
0x12ab, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x120b, 0x1235, 0x12ab, 0x20, 0x12e8,
0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x12a0, 0x121b, 0x12de, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x121b, 0x12de, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a0, 0x121b, 0x12de, 0x1295, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1230, 0x121c, 0x1295, 0x20, 0x12a0,
0x121c, 0x122a, 0x12ab, 0x20, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1230, 0x121c, 0x1295,
0x20, 0x12a0, 0x121c, 0x122a, 0x12ab, 0x20, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20, 0x12e8, 0x1240,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x121d,
0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x121d, 0x1235,
0x122b, 0x1243, 0x12ca, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1270, 0x122b, 0x122b, 0x20, 0x12e8, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1270, 0x122b, 0x122b,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1270, 0x122b, 0x122b, 0x20, 0x12e8, 0x1240, 0x1295,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1353, 0x1235,
0x134a, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x1353, 0x1235, 0x134a, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1353, 0x1235, 0x134a,
0x12ad, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1293, 0x12f5, 0x12ed, 0x122d, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1293, 0x12f2, 0x12ed,
0x122d, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a0, 0x1293, 0x12f5, 0x12ed, 0x122d, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x12e8, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1352, 0x12eb,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x1352, 0x12eb, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x1352, 0x12eb, 0x20, 0x12e8, 0x1240,
0x1295, 0x20, 0x130a, 0x12dc, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12d3, 0x1228, 0x1262,
0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12d3, 0x1228, 0x1262, 0x12eb, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12d3, 0x1228, 0x1262, 0x12eb,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x12a0, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x20, 0x12e8, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x122d, 0x1300, 0x1295,
0x1272, 0x1293, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293,
0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12a0, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x20, 0x121d, 0x1235,
0x122b, 0x1243, 0x12ca, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0, 0x122d, 0x1300, 0x1295,
0x1272, 0x1293, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20,
0x12a0, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x122d, 0x1218,
0x1292, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x122d, 0x1218, 0x1292, 0x12eb,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x122d,
0x1218, 0x1292, 0x12eb, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12a0, 0x1275, 0x120b, 0x1295, 0x1272, 0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1275, 0x120b, 0x1295, 0x1272,
0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1275, 0x120b, 0x1295, 0x1272, 0x12ad,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20, 0x12a0, 0x12cd,
0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20,
0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd,
0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20,
0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1218, 0x12ab,
0x12a8, 0x1208, 0x129b, 0x20, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275,
0x122b, 0x120a, 0x12eb, 0x20, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20, 0x121d, 0x1235,
0x122b, 0x1243, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b,
0x120a, 0x12eb, 0x20, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20, 0x121d, 0x1235,
0x122b, 0x1245, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0,
0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a,
0x12eb, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263,
0x12ca, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a0, 0x12cd,
0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20,
0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235,
0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12e8,
0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x12a0, 0x12d8, 0x122d, 0x1263, 0x1303, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12a0, 0x12d8, 0x122d, 0x1263, 0x1303, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12d8, 0x122d, 0x1263, 0x1303, 0x1295, 0x20, 0x12ad,
0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12de, 0x1228, 0x1235,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12de, 0x1228, 0x1235, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12de, 0x1228, 0x1235, 0x20,
0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1263, 0x1295, 0x130d,
0x120b, 0x12f4, 0x123d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1263, 0x1295, 0x130d, 0x120b,
0x12f4, 0x123d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1263, 0x1295, 0x130d, 0x120b, 0x12f4, 0x123d, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1261, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1266, 0x120a, 0x126a, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1265, 0x122b, 0x12da,
0x120b, 0x12ca, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x1265, 0x122b, 0x1232, 0x120a, 0x12eb, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1265, 0x122b,
0x12da, 0x120b, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1265, 0x1229, 0x1294, 0x12ed, 0x20, 0x12f3, 0x1229,
0x1233, 0x120b, 0x121d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ac, 0x1355, 0x20, 0x1268,
0x122d, 0x12f4, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d,
0x12f4, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ac,
0x1355, 0x20, 0x1268, 0x122d, 0x12f4, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x127b, 0x121e, 0x122e, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x127b, 0x1273, 0x121d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x127b, 0x1273, 0x121d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x127b, 0x1273, 0x121d, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x127a, 0x120a, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x127a, 0x120a, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x127a, 0x120a, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x127b, 0x12ed, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x127b, 0x12ed, 0x1293,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x127b, 0x12ed,
0x1293, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1308, 0x1293, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x12ae, 0x12ae, 0x1235, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12ae, 0x120e, 0x121d, 0x1262, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12ae, 0x120e, 0x121d, 0x1262, 0x12eb, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x120e, 0x121d, 0x1262, 0x12eb, 0x20, 0x12ad, 0x1228,
0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a9, 0x12ad, 0x20, 0x12f0, 0x1234,
0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a9, 0x12ad, 0x20, 0x12f0, 0x1234,
0x1276, 0x127d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12a9, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x130d, 0x121b, 0x123d, 0x20,
0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12a9, 0x1263, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a9, 0x1263, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a9, 0x1263, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d,
0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12f4, 0x126a, 0x1235, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12f1, 0x121e, 0x1295, 0x1275, 0x2d, 0x12f1, 0x122d, 0x126a, 0x120d,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x1272, 0x121e,
0x122d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x1235, 0x1270, 0x122d, 0x20, 0x12f0,
0x1234, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x1235, 0x1270, 0x122d, 0x20,
0x12f0, 0x1234, 0x1275, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12a2, 0x1235, 0x1270, 0x122d, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x12ad, 0x1228,
0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x12b3, 0x12f6, 0x122d, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20, 0x12a0,
0x12cd, 0x122e, 0x1353, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208,
0x129b, 0x12cd, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20,
0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a0,
0x12cd, 0x122e, 0x1353, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353, 0x20,
0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1229, 0x1245, 0x20,
0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x12e8, 0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0, 0x12cd, 0x122e,
0x1353, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20,
0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353,
0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134b, 0x120d,
0x12ad, 0x120b, 0x1295, 0x12f5, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x134b, 0x120d, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x20, 0x12f0, 0x1234, 0x1276,
0x127d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134b,
0x120d, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x12ad,
0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134a, 0x1302, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x134a, 0x1302, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x134a, 0x1302, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1348, 0x1228, 0x1295, 0x1233, 0x12ed, 0x20, 0x1309, 0x12eb, 0x1293,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1348, 0x1228, 0x1295, 0x1233, 0x12ed, 0x20, 0x12f0,
0x1261, 0x1263, 0x12ca, 0x20, 0x12a5, 0x1293, 0x20, 0x12a0, 0x1295, 0x1273, 0x122d, 0x12ad,
0x1272, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x130b, 0x120b, 0x1353, 0x130e, 0x1235,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x130b, 0x121d, 0x1262, 0x12e8, 0x122d, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1302, 0x12ee, 0x122d, 0x1302, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1302, 0x12ee, 0x122d, 0x1302, 0x12eb, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1302, 0x12ee, 0x122d, 0x1302, 0x12eb, 0x20, 0x12ad, 0x1228,
0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1302, 0x120d, 0x1260, 0x122d, 0x1275,
0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x130d, 0x122a, 0x1295,
0x12ca, 0x127d, 0x20, 0x121b, 0x12d5, 0x12a8, 0x120b, 0x12ca, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x130d, 0x122a,
0x1295, 0x120b, 0x1295, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295,
0x12f5, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d,
0x12d5, 0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b,
0x1295, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x20,
0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1263, 0x1205, 0x1228,
0x1230, 0x120b, 0x1324, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1309, 0x12eb, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1203, 0x12cb, 0x12ed,
0x20, 0x12a0, 0x120c, 0x12a1, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1203, 0x12cb, 0x12ed, 0x20, 0x12a0, 0x120c, 0x12a1, 0x1275,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1203, 0x12cb, 0x12ed, 0x20, 0x12a0, 0x120c, 0x12a1, 0x1275,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295, 0x130d, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295, 0x130d, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1206, 0x1295, 0x130d, 0x20,
0x12ae, 0x1295, 0x130d, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1206, 0x126d, 0x12f5, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1206, 0x126d, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1206,
0x126d, 0x12f5, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1205, 0x1295, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1205, 0x1295, 0x12f5, 0x20, 0x12cd, 0x1245,
0x12eb, 0x1296, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x1295, 0x12f6, 0x127b,
0x12ed, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b,
0x12cd, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1294, 0x12e2, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1294, 0x12e2,
0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20,
0x12a2, 0x1295, 0x12f6, 0x1294, 0x12e2, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2,
0x122b, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x122b, 0x1295, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x122b, 0x1295, 0x20,
0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x12e8,
0x1260, 0x130b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x12e8, 0x1240, 0x1295,
0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1303, 0x1353,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1303, 0x1353, 0x1295, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1303, 0x1353, 0x1295, 0x20, 0x12e8,
0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12ab, 0x121d, 0x127b, 0x1275, 0x12ab, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1354, 0x1275, 0x122e, 0x1353, 0x126d, 0x120e, 0x1235, 0x12aa,
0x20, 0x2d, 0x20, 0x12ab, 0x121d, 0x127b, 0x1275, 0x1235, 0x12aa, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1354, 0x1275, 0x122e, 0x1353,
0x126d, 0x120e, 0x1235, 0x12aa, 0x20, 0x2d, 0x20, 0x12ab, 0x121d, 0x127b, 0x1275, 0x1235,
0x12aa, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12ab, 0x12db, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x12ab, 0x12db, 0x12aa, 0x1235, 0x1273,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12ab,
0x12db, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x122a,
0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x122a, 0x12eb, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x122a, 0x12eb, 0x20, 0x12e8,
0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12ae, 0x1235, 0x122b, 0x12a4, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ad, 0x122b, 0x1235,
0x1296, 0x12eb, 0x122d, 0x1235, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x12ad, 0x122b, 0x1235, 0x1296, 0x12ed, 0x12a0, 0x122d, 0x1235,
0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12ad, 0x122b, 0x1235, 0x1296, 0x12eb, 0x122d, 0x1235,
0x12ad, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x12aa, 0x122d, 0x130a, 0x1235, 0x1273, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x120b, 0x12ed, 0x1295, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x120e, 0x122d, 0x12f5, 0x20, 0x1206, 0x12cc, 0x20, 0x12e8,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x120e, 0x122d,
0x12f5, 0x20, 0x1206, 0x12cc, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x120e, 0x122d, 0x12f5,
0x20, 0x1206, 0x12cc, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121b, 0x130b, 0x12f3, 0x1295, 0x20, 0x12e8,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121b, 0x130b,
0x12f3, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121b, 0x130b, 0x12f3, 0x1295, 0x20, 0x1260,
0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x121b, 0x120c, 0x12ed, 0x12e2, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x120d,
0x12f2, 0x126d, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x122d, 0x12b4, 0x1233,
0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x122d, 0x123b, 0x120d, 0x20, 0x12f0,
0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x12cd, 0x122a, 0x123a,
0x12e8, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x12cd, 0x122a, 0x123a, 0x12e8,
0x1235, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b,
0x12cd, 0x122a, 0x123a, 0x12e8, 0x1235, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x121b, 0x12cd, 0x1230, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x121c, 0x12ad, 0x1232, 0x12ae, 0x20, 0x1353, 0x1232, 0x134a, 0x12ad, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121c, 0x12ad, 0x1232, 0x12ae,
0x20, 0x1353, 0x1232, 0x134a, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121c, 0x12ad, 0x1232,
0x12ae, 0x20, 0x1353, 0x1232, 0x134a, 0x12ad, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a1, 0x120b, 0x1295,
0x20, 0x1263, 0x1276, 0x122d, 0x20, 0x130a, 0x12dc, 0x12e8, 0x12a1, 0x120b, 0x1295, 0x20,
0x1263, 0x1276, 0x122d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a1, 0x120b, 0x1295, 0x20, 0x1263,
0x1276, 0x122d, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121e, 0x1235, 0x12ae, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121e, 0x1235, 0x12ae, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x121e, 0x1235, 0x12ae, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121a, 0x12eb, 0x1295, 0x121b,
0x122d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1293, 0x12cd, 0x1229, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x1294, 0x1353, 0x120d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1292, 0x12cd,
0x20, 0x12ab, 0x120c, 0x12f6, 0x1292, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1292,
0x12cd, 0x20, 0x12ab, 0x120c, 0x12f6, 0x1292, 0x12eb, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1292, 0x12cd, 0x20, 0x12ab, 0x120c, 0x12f6, 0x1292,
0x12eb, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1292,
0x12cd, 0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1292,
0x12cd, 0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20,
0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5, 0x20, 0x12e8,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1292, 0x12cd,
0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5, 0x20, 0x12e8, 0x1240,
0x1295, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x1292, 0x12e9, 0x12cc, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1296, 0x122d, 0x134e,
0x120d, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1276,
0x127d, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1296, 0x122e, 0x1295, 0x1203, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1348, 0x122d, 0x1293, 0x1295,
0x12f6, 0x20, 0x12f2, 0x20, 0x1296, 0x122e, 0x1295, 0x127b, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x1348, 0x122d, 0x1293, 0x1295, 0x12f6, 0x20, 0x12f2, 0x20, 0x1296, 0x122e, 0x1295, 0x1203,
0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1296, 0x126e, 0x1232, 0x1265, 0x122a, 0x1235, 0x12ad, 0x20,
0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1296,
0x126e, 0x1232, 0x1262, 0x122d, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1296,
0x126e, 0x1232, 0x1265, 0x122a, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230,
0x12a0, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a6, 0x121d, 0x1235,
0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x12a6, 0x121d, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a6, 0x121d, 0x1235,
0x12ad, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1353, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x1353, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x12ad,
0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x120b, 0x12cd, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x1357, 0x20, 0x1292, 0x12cd, 0x20, 0x130a, 0x1292,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x122b, 0x1313, 0x12ed, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x1353, 0x122b, 0x1313, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x122b, 0x1313, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d,
0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1354, 0x1229, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1354, 0x1229, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1354, 0x1229, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x134a, 0x120a, 0x1352, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134a, 0x120a,
0x1352, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x134a, 0x120a, 0x1352, 0x1295, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x134a, 0x1292, 0x12ad, 0x1235, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20,
0x1230, 0x12d3, 0x1275, 0x1245, 0x12f1, 0x1235, 0x20, 0x12e8, 0x1352, 0x12ec, 0x122d, 0x20,
0x12a5, 0x1293, 0x20, 0x121a, 0x12b4, 0x120e, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x1245,
0x12f1, 0x1235, 0x20, 0x12e8, 0x1352, 0x12ec, 0x122d, 0x20, 0x12a5, 0x1293, 0x20, 0x121a,
0x12b4, 0x120e, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x1245, 0x12f1, 0x1235, 0x20, 0x12e8, 0x1352, 0x12ec, 0x122d, 0x20, 0x12a5, 0x1293, 0x20,
0x121a, 0x12b4, 0x120e, 0x1295, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1352, 0x1275, 0x12ab, 0x12ed, 0x122d, 0x1295,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1356, 0x1293, 0x1354, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1355, 0x12ee, 0x1295, 0x130d, 0x12eb, 0x1295, 0x130d, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x122c, 0x12e9, 0x1292, 0x12e8, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x122e,
0x1274, 0x122b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1233, 0x12ad, 0x1203, 0x120a, 0x1295,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233,
0x12ad, 0x1203, 0x120a, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x12ad, 0x1203, 0x120a,
0x1295, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x121b, 0x122b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x121b, 0x122b, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x1233, 0x121b, 0x122b, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x121e, 0x12cb, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1233, 0x121e, 0x12cb, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1233, 0x121e, 0x12cb, 0x20, 0x12e8, 0x1260, 0x130b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1234, 0x1238, 0x120d, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1232, 0x1295, 0x130b, 0x1352, 0x122d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1230, 0x1208, 0x121e, 0x1295, 0x20, 0x12f0, 0x1234, 0x1276,
0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12f0, 0x1261, 0x1265, 0x20, 0x1302, 0x12ee,
0x122d, 0x1302, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1231, 0x122a, 0x1293, 0x121d,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1232, 0x12ee, 0x12cb, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1273, 0x1202, 0x1272, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1273, 0x12ed, 0x1354,
0x12ed, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1273, 0x12ed, 0x1354, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1273, 0x12ed, 0x1354, 0x12ed,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x1273, 0x1302, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1276, 0x12ac, 0x120b, 0x12cd, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1276, 0x1295,
0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1276, 0x1295, 0x130b, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1276, 0x1295, 0x130b, 0x20, 0x12ad,
0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1279, 0x12ad, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1271, 0x122d, 0x12ad, 0x1218, 0x1292, 0x1235, 0x1273, 0x1295, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1271, 0x122d, 0x12ad, 0x1218, 0x1292, 0x1235, 0x1273, 0x1295,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1271, 0x122d,
0x12ad, 0x1218, 0x1292, 0x1235, 0x1273, 0x1295, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1271, 0x126b, 0x1209, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12a1, 0x122b, 0x1313, 0x12ed, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a1, 0x122b, 0x1313,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a1,
0x122b, 0x1313, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12a1, 0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12a1, 0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a1, 0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273,
0x1295, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126b,
0x1297, 0x1271, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126b, 0x1297, 0x1271, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126b, 0x1297, 0x1271, 0x20,
0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126c, 0x1294, 0x12dd,
0x12cc, 0x120b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126d, 0x120b, 0x12f2, 0x126e, 0x1235,
0x1276, 0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x126a, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x126d, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x12e8, 0x1260,
0x130b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x126e, 0x120d, 0x130e, 0x122b, 0x12f5, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x126e, 0x120d, 0x130e, 0x122b, 0x12f5, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x126b, 0x120d, 0x130e, 0x122b, 0x12f5, 0x20, 0x12e8, 0x1260, 0x130b,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x126e,
0x1235, 0x1276, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12cc, 0x12ad, 0x20, 0x12f0,
0x1234, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12cb, 0x120a, 0x1235, 0x20, 0x12a5,
0x1293, 0x20, 0x1349, 0x1271, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12eb, 0x12a9, 0x1275,
0x1235, 0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12eb, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12eb, 0x12a9,
0x1275, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12e8, 0x12ab, 0x1270, 0x122a, 0x1295, 0x1260,
0x122d, 0x130d, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x12e8, 0x12ab, 0x1270, 0x122a, 0x1295, 0x1260, 0x122d, 0x130d, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x12e8, 0x12ab, 0x1270, 0x122a, 0x1295, 0x1260, 0x122d, 0x130d, 0x20, 0x12e8, 0x1260,
0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12e9, 0x12ae, 0x1295, 0x20, 0x130a, 0x12dc, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x623, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x641, 0x631, 0x64a, 0x642,
0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20,
0x623, 0x641, 0x631, 0x64a, 0x642, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x62c, 0x646, 0x648, 0x628, 0x20, 0x623, 0x641, 0x631, 0x64a, 0x642, 0x64a,
0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623,
0x641, 0x631, 0x64a, 0x642, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x623, 0x641, 0x631, 0x64a, 0x642, 0x64a, 0x627, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x623, 0x641, 0x631, 0x64a, 0x642, 0x64a, 0x627, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x623, 0x644, 0x627, 0x633, 0x643, 0x627, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x20, 0x644, 0x623, 0x644,
0x627, 0x633, 0x643, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x644,
0x627, 0x633, 0x643, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x645, 0x627, 0x632, 0x648,
0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x645, 0x627,
0x632, 0x648, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x645, 0x627, 0x632, 0x648, 0x646,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x631, 0x643, 0x632, 0x64a, 0x20, 0x644,
0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627, 0x644, 0x634, 0x645, 0x627,
0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x20, 0x627, 0x644, 0x645, 0x631, 0x643, 0x632,
0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627, 0x644,
0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x627, 0x644, 0x645,
0x631, 0x643, 0x632, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627,
0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x634, 0x631, 0x642, 0x64a, 0x20,
0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627, 0x644, 0x634, 0x645,
0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x20, 0x627, 0x644, 0x634, 0x631, 0x642,
0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627, 0x644,
0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x627, 0x644, 0x634,
0x631, 0x642, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20,
0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x62c, 0x628, 0x644, 0x64a, 0x20, 0x644,
0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627, 0x644, 0x634, 0x645, 0x627,
0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627,
0x644, 0x62c, 0x628, 0x644, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627, 0x644, 0x634,
0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x62c, 0x628, 0x644, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627,
0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644, 0x647, 0x627,
0x62f, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d,
0x64a, 0x637, 0x20, 0x627, 0x644, 0x647, 0x627, 0x62f, 0x64a, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644, 0x647, 0x627, 0x62f, 0x64a, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x623, 0x646, 0x627, 0x62f, 0x64a, 0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x623, 0x646, 0x627, 0x62f, 0x64a, 0x631, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x20, 0x644, 0x623, 0x646, 0x627, 0x62f, 0x64a, 0x631, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x622, 0x628, 0x64a, 0x627, 0x627, 0x644, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x20,
0x644, 0x622, 0x628, 0x64a, 0x627, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x644, 0x623, 0x628, 0x64a,
0x627, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x639,
0x631, 0x628, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627,
0x644, 0x639, 0x631, 0x628, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x639, 0x631,
0x628, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646, 0x62a, 0x64a, 0x646,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646,
0x62a, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646, 0x62a, 0x64a,
0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646,
0x62a, 0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646, 0x62a, 0x64a, 0x646, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a,
0x631, 0x628, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646, 0x62a, 0x64a, 0x646,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x623, 0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x623, 0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x631,
0x645, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x637, 0x644, 0x633,
0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x20, 0x627, 0x644, 0x623, 0x637, 0x644, 0x633, 0x64a, 0x627,
0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x20, 0x627, 0x644, 0x623, 0x637, 0x644, 0x633, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637,
0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633,
0x637, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a,
0x631, 0x628, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a,
0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x623,
0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x634, 0x631, 0x642, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a,
0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644,
0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631,
0x628, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623,
0x630, 0x631, 0x628, 0x64a, 0x62c, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x623, 0x630, 0x631, 0x628, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x630,
0x631, 0x628, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x632, 0x648, 0x631, 0x633,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x632, 0x648, 0x631, 0x633, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x623, 0x632, 0x648, 0x631, 0x633, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x646, 0x63a, 0x644, 0x627, 0x62f,
0x64a, 0x634, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x646, 0x63a, 0x644,
0x627, 0x62f, 0x64a, 0x634, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x646, 0x63a, 0x644, 0x627, 0x62f, 0x64a,
0x634, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x628, 0x648, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x628, 0x648, 0x644, 0x64a, 0x641, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a, 0x627, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a, 0x627,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x631,
0x648, 0x646, 0x627, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x631, 0x623, 0x633, 0x20, 0x627, 0x644, 0x623, 0x62e, 0x636, 0x631, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x623, 0x633, 0x20, 0x627, 0x644,
0x623, 0x62e, 0x636, 0x631, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x623, 0x633, 0x20, 0x627,
0x644, 0x623, 0x62e, 0x636, 0x631, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x627, 0x645, 0x648, 0x631,
0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x627, 0x62a, 0x627,
0x645, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x627, 0x62a, 0x627,
0x645, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62a, 0x634, 0x627, 0x62a, 0x627, 0x645, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x64a,
0x644, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x64a, 0x644,
0x64a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62a, 0x634, 0x64a, 0x644, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x646,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x627,
0x644, 0x643, 0x631, 0x64a, 0x633, 0x645, 0x627, 0x633, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x643, 0x648, 0x643, 0x648, 0x633, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x644, 0x648, 0x645, 0x628, 0x64a,
0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x644, 0x648, 0x645,
0x628, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x644, 0x648, 0x645, 0x628, 0x64a, 0x627,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x62c, 0x632, 0x631, 0x20, 0x643, 0x648, 0x648, 0x643, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x643, 0x648, 0x643, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c,
0x632, 0x631, 0x20, 0x643, 0x648, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x628, 0x627, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648,
0x628, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62f, 0x627, 0x641, 0x64a, 0x632, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62f, 0x64a, 0x20, 0x645, 0x648, 0x646, 0x62a, 0x20, 0x62f, 0x648,
0x20, 0x631, 0x648, 0x641, 0x64a, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x62a, 0x64a, 0x645, 0x648, 0x631, 0x20, 0x627, 0x644, 0x634, 0x631, 0x642, 0x64a,
0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631, 0x629,
0x20, 0x627, 0x633, 0x62a, 0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c,
0x632, 0x64a, 0x631, 0x629, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632,
0x64a, 0x631, 0x629, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x625,
0x643, 0x648, 0x627, 0x62f, 0x648, 0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x648, 0x633, 0x637, 0x20, 0x623, 0x648, 0x631, 0x648, 0x628, 0x627, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x648, 0x631, 0x648,
0x628, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x648, 0x631, 0x648, 0x628,
0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x623, 0x648, 0x631, 0x648, 0x628, 0x627,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x623, 0x648,
0x631, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x623, 0x648, 0x631,
0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x627, 0x644,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x648, 0x631, 0x648,
0x628, 0x64a, 0x20, 0x28, 0x623, 0x643, 0x62b, 0x631, 0x20, 0x634, 0x631, 0x642,
0x64b, 0x627, 0x29, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x623, 0x648, 0x631, 0x648, 0x628, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x648, 0x631, 0x648, 0x628, 0x627, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x623, 0x648, 0x631, 0x648, 0x628, 0x627, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c,
0x632, 0x631, 0x20, 0x641, 0x648, 0x643, 0x644, 0x627, 0x646, 0x62f, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x641, 0x648, 0x643, 0x644,
0x627, 0x646, 0x62f, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x641, 0x648, 0x643, 0x644,
0x627, 0x646, 0x62f, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x641, 0x64a, 0x62c, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x641, 0x64a, 0x62c, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x64a, 0x62c, 0x64a, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x648, 0x64a, 0x627, 0x646, 0x627, 0x20, 0x627, 0x644, 0x641, 0x631, 0x646,
0x633, 0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645,
0x642, 0x627, 0x637, 0x639, 0x627, 0x62a, 0x20, 0x627, 0x644, 0x641, 0x631, 0x646,
0x633, 0x64a, 0x629, 0x20, 0x627, 0x644, 0x62c, 0x646, 0x648, 0x628, 0x64a, 0x629,
0x20, 0x648, 0x627, 0x644, 0x623, 0x646, 0x62a, 0x627, 0x631, 0x62a, 0x64a, 0x643,
0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x644, 0x627, 0x628,
0x627, 0x63a, 0x648, 0x633, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x627,
0x645, 0x628, 0x64a, 0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x648,
0x631, 0x62c, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x648,
0x631, 0x62c, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x62c, 0x632, 0x631, 0x20, 0x62c, 0x64a, 0x644, 0x628, 0x631, 0x62a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x62a, 0x634, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x63a, 0x631, 0x64a, 0x646,
0x644, 0x627, 0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631,
0x642, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634,
0x631, 0x642, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644, 0x627, 0x646, 0x62f,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x63a, 0x631,
0x64a, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x63a,
0x631, 0x64a, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x648, 0x627, 0x645,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x62e, 0x644, 0x64a, 0x62c,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x64a, 0x627, 0x646, 0x627, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x627, 0x648, 0x627, 0x64a, 0x20, 0x623,
0x644, 0x648, 0x62a, 0x64a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x647, 0x627, 0x648, 0x627, 0x64a, 0x20, 0x623, 0x644, 0x648, 0x62a, 0x64a, 0x627,
0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x647, 0x627, 0x648, 0x627, 0x64a, 0x20, 0x623, 0x644, 0x648, 0x62a,
0x64a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x646, 0x63a, 0x20, 0x643, 0x648, 0x646,
0x63a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x646, 0x63a, 0x20,
0x643, 0x648, 0x646, 0x63a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x646, 0x63a, 0x20, 0x643, 0x648,
0x646, 0x63a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x647, 0x648, 0x641, 0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x647, 0x648, 0x641, 0x62f, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x641, 0x62f, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627,
0x644, 0x647, 0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644, 0x647, 0x646, 0x62f, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x647, 0x646, 0x62f, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x646, 0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x648, 0x633, 0x637, 0x20, 0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a, 0x633, 0x64a,
0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x625,
0x646, 0x62f, 0x648, 0x646, 0x64a, 0x633, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a,
0x633, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x64a, 0x631,
0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x64a, 0x631, 0x627,
0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x625, 0x64a, 0x631, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x631, 0x643, 0x648,
0x62a, 0x633, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x631, 0x643,
0x648, 0x62a, 0x633, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x631, 0x643, 0x648, 0x62a, 0x633, 0x643,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x625, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x644, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x625, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x644, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x633,
0x631, 0x627, 0x626, 0x64a, 0x644, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x64a, 0x627, 0x628, 0x627,
0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x64a, 0x627, 0x628,
0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x64a, 0x627, 0x628, 0x627, 0x646, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643,
0x627, 0x645, 0x634, 0x627, 0x62a, 0x643, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x628, 0x64a, 0x62a, 0x631, 0x648, 0x628, 0x627, 0x641, 0x644, 0x648, 0x641,
0x633, 0x643, 0x2d, 0x643, 0x627, 0x645, 0x62a, 0x634, 0x627, 0x62a, 0x633, 0x643,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x62a, 0x631, 0x648,
0x628, 0x627, 0x641, 0x644, 0x648, 0x641, 0x633, 0x643, 0x2d, 0x643, 0x627, 0x645,
0x62a, 0x634, 0x627, 0x62a, 0x633, 0x643, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x627, 0x632, 0x627,
0x62e, 0x633, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634,
0x631, 0x642, 0x20, 0x643, 0x627, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x643, 0x627,
0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x643, 0x648, 0x631, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x643, 0x648, 0x631, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x631, 0x64a, 0x627, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x643, 0x648, 0x633, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643,
0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627,
0x631, 0x633, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x627, 0x644,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x20, 0x644, 0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633,
0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x642, 0x64a, 0x631, 0x63a, 0x64a,
0x632, 0x633, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c,
0x632, 0x631, 0x20, 0x644, 0x627, 0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x644, 0x648, 0x631, 0x62f, 0x20, 0x647, 0x627, 0x648, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x644, 0x648, 0x631, 0x62f, 0x20, 0x647, 0x627, 0x648, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x644, 0x644, 0x648,
0x631, 0x62f, 0x20, 0x647, 0x627, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x645, 0x627, 0x63a, 0x627, 0x62f, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x645, 0x627, 0x63a, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x627, 0x63a,
0x627, 0x62f, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x627, 0x644, 0x64a, 0x632, 0x64a, 0x627,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x627, 0x644,
0x645, 0x627, 0x644, 0x62f, 0x64a, 0x641, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x645, 0x627, 0x631, 0x643, 0x64a, 0x633, 0x627, 0x633, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x645, 0x627, 0x631, 0x634, 0x627, 0x644,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x631, 0x64a, 0x634, 0x64a,
0x648, 0x633, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x631, 0x64a,
0x634, 0x64a, 0x648, 0x633, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x631, 0x64a, 0x634, 0x64a, 0x648,
0x633, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x645, 0x627, 0x648, 0x633, 0x648, 0x646, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644, 0x647,
0x627, 0x62f, 0x64a, 0x20, 0x644, 0x644, 0x645, 0x643, 0x633, 0x64a, 0x643, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20,
0x627, 0x644, 0x647, 0x627, 0x62f, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x20, 0x644, 0x644, 0x645, 0x643, 0x633, 0x64a, 0x643, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644,
0x647, 0x627, 0x62f, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20,
0x644, 0x644, 0x645, 0x643, 0x633, 0x64a, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x623, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x648, 0x631,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x644, 0x627, 0x646, 0x20,
0x628, 0x627, 0x62a, 0x648, 0x631, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x644, 0x627, 0x646, 0x20,
0x628, 0x627, 0x62a, 0x648, 0x631, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x633, 0x643, 0x648, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x633, 0x643, 0x648, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645,
0x648, 0x633, 0x643, 0x648, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x64a, 0x627, 0x646, 0x645, 0x627, 0x631,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x627, 0x648, 0x631, 0x648, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x628, 0x627, 0x644, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x643, 0x627, 0x644, 0x64a, 0x62f, 0x648, 0x646, 0x64a,
0x627, 0x20, 0x627, 0x644, 0x62c, 0x62f, 0x64a, 0x62f, 0x629, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x643, 0x627, 0x644, 0x64a, 0x62f, 0x648, 0x646, 0x64a, 0x627,
0x20, 0x627, 0x644, 0x62c, 0x62f, 0x64a, 0x62f, 0x629, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x627, 0x644,
0x64a, 0x62f, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x62c, 0x62f, 0x64a,
0x62f, 0x629, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644, 0x646, 0x62f, 0x627,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644,
0x646, 0x62f, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644, 0x646, 0x62f,
0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x646, 0x64a, 0x648, 0x641, 0x627, 0x648, 0x646, 0x62f, 0x644, 0x627,
0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648, 0x641,
0x627, 0x648, 0x646, 0x62f, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648,
0x641, 0x627, 0x648, 0x646, 0x62f, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a,
0x648, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631,
0x629, 0x20, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20, 0x646, 0x648, 0x631,
0x641, 0x648, 0x644, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20, 0x646,
0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x64a, 0x631, 0x646, 0x627,
0x646, 0x62f, 0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646,
0x647, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x631, 0x646, 0x627,
0x646, 0x62f, 0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646,
0x647, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x641, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x648, 0x20, 0x62f,
0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632,
0x631, 0x20, 0x645, 0x627, 0x631, 0x64a, 0x627, 0x646, 0x627, 0x20, 0x627, 0x644,
0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x646, 0x648, 0x641, 0x648, 0x633, 0x64a, 0x628, 0x64a, 0x631, 0x633, 0x643, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x648, 0x641, 0x648, 0x633, 0x64a, 0x628,
0x64a, 0x631, 0x633, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x648, 0x641, 0x648, 0x633, 0x64a, 0x628,
0x64a, 0x631, 0x633, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x645, 0x633, 0x643, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x645, 0x633, 0x643, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648,
0x645, 0x633, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x643, 0x633, 0x62a, 0x627, 0x646, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x643, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x628, 0x627, 0x643, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x644,
0x627, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x628, 0x648,
0x627, 0x20, 0x63a, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x62c, 0x62f,
0x64a, 0x62f, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x631,
0x627, 0x63a, 0x648, 0x627, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628,
0x627, 0x631, 0x627, 0x63a, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x631, 0x627,
0x63a, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x631, 0x648, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x631, 0x648, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x631, 0x648,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x641, 0x64a, 0x644, 0x628, 0x64a, 0x646, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x641, 0x64a, 0x644, 0x628, 0x64a, 0x646, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x627, 0x644, 0x641, 0x64a, 0x644, 0x628, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631,
0x20, 0x641, 0x64a, 0x646, 0x643, 0x633, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x633, 0x627, 0x646, 0x62a, 0x20, 0x628, 0x64a, 0x64a, 0x631, 0x20, 0x648, 0x645,
0x64a, 0x643, 0x648, 0x644, 0x648, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x633, 0x627, 0x646, 0x62a, 0x20, 0x628, 0x64a, 0x64a, 0x631, 0x20, 0x648, 0x645,
0x64a, 0x643, 0x648, 0x644, 0x648, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x646, 0x62a, 0x20,
0x628, 0x64a, 0x64a, 0x631, 0x20, 0x648, 0x645, 0x64a, 0x643, 0x648, 0x644, 0x648,
0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x628, 0x64a, 0x62a, 0x643, 0x64a, 0x631, 0x646, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x628, 0x648, 0x646, 0x627, 0x628, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x648, 0x646, 0x63a, 0x20, 0x64a, 0x627, 0x646,
0x63a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x631, 0x648, 0x64a, 0x646, 0x64a,
0x648, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x631, 0x648, 0x62b, 0x64a,
0x631, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x62e, 0x627,
0x644, 0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x62e,
0x627, 0x644, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x62e, 0x627, 0x644, 0x64a, 0x646,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x633, 0x645, 0x627, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x633, 0x645, 0x627, 0x631, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x645, 0x648, 0x627, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x645, 0x648, 0x627, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633,
0x627, 0x645, 0x648, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x64a, 0x634, 0x644, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x633, 0x646, 0x63a, 0x627, 0x641, 0x648, 0x631, 0x629, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x633, 0x644, 0x64a,
0x645, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x646, 0x648,
0x628, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x633, 0x648, 0x631, 0x64a, 0x646, 0x627, 0x645, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x633, 0x627, 0x64a, 0x648, 0x648, 0x627, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62a, 0x627, 0x647, 0x64a, 0x62a, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62a, 0x627, 0x64a, 0x628, 0x64a, 0x647, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62a, 0x627, 0x64a, 0x628, 0x64a, 0x647, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x627,
0x64a, 0x628, 0x64a, 0x647, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x637, 0x627, 0x62c, 0x643, 0x633, 0x62a, 0x627,
0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648, 0x643, 0x64a, 0x644,
0x627, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648, 0x646, 0x63a,
0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648, 0x646, 0x63a, 0x627,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x62a, 0x648, 0x646, 0x63a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x648, 0x643, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x631, 0x643, 0x645, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x631, 0x643, 0x645,
0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x631, 0x643, 0x645, 0x627,
0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648, 0x641, 0x627, 0x644, 0x648,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x631, 0x648, 0x63a, 0x648,
0x627, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x631, 0x648,
0x63a, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x631, 0x648, 0x63a, 0x648, 0x627,
0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x623, 0x648, 0x632, 0x628, 0x643, 0x633, 0x62a, 0x627, 0x646, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x632, 0x628, 0x643, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x623, 0x648, 0x632, 0x628, 0x643, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x641, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x641, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x627,
0x646, 0x648, 0x627, 0x62a, 0x648, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x646, 0x632, 0x648, 0x64a, 0x644,
0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x644, 0x627, 0x62f, 0x64a,
0x641, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x641, 0x644, 0x627, 0x62f, 0x64a, 0x641, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x641, 0x644, 0x627, 0x62f, 0x64a, 0x641, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x641, 0x648, 0x644, 0x63a, 0x648, 0x63a, 0x631, 0x627, 0x62f, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x641, 0x648, 0x644, 0x63a, 0x648, 0x63a, 0x631, 0x627, 0x62f,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x641, 0x648, 0x644, 0x63a, 0x648, 0x63a, 0x631, 0x627, 0x62f, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641,
0x648, 0x633, 0x62a, 0x648, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c,
0x632, 0x64a, 0x631, 0x629, 0x20, 0x648, 0x64a, 0x643, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x648, 0x627, 0x644, 0x64a, 0x633, 0x20, 0x648, 0x20, 0x641, 0x648,
0x62a, 0x648, 0x646, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x627,
0x643, 0x648, 0x62a, 0x633, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a,
0x627, 0x643, 0x648, 0x62a, 0x633, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x627, 0x643, 0x648, 0x62a,
0x633, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x64a, 0x643, 0x627, 0x62a, 0x631, 0x64a, 0x646, 0x628, 0x648,
0x631, 0x63a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x643, 0x627, 0x62a,
0x631, 0x64a, 0x646, 0x628, 0x648, 0x631, 0x63a, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x643, 0x627, 0x62a,
0x631, 0x64a, 0x646, 0x628, 0x648, 0x631, 0x63a, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x648, 0x643, 0x648,
0x646, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64,
0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64,
0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f,
0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f,
0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6d,
0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x6f, 0x6e,
0x74, 0x61, 0x6e, 0x79, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x6f, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x6f, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x6f, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x73, 0x74, 0x65,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x75, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x75, 0x65, 0x73,
0x74, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x75, 0x65, 0x73, 0x74, 0x65,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x531, 0x586, 0x572, 0x561, 0x576, 0x57d, 0x57f, 0x561,
0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565,
0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x586,
0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x531,
0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x540, 0x561, 0x580, 0x561, 0x57e, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561,
0x576, 0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575,
0x561, 0x576, 0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576,
0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x56c, 0x575, 0x561, 0x57d, 0x56f, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x56c, 0x575, 0x561, 0x57d,
0x56f, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x56c, 0x575,
0x561, 0x57d, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x574, 0x561, 0x566, 0x578, 0x576, 0x575, 0x561, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x531, 0x574, 0x561, 0x566, 0x578, 0x576, 0x575, 0x561,
0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x574, 0x561, 0x566, 0x578, 0x576,
0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f,
0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580,
0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576,
0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f,
0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561,
0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c,
0x575, 0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561, 0x575,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575,
0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53c, 0x565, 0x57c, 0x576, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x20, 0x28, 0x531,
0x544, 0x546, 0x29, 0x53c, 0x565, 0x57c, 0x576, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x20, 0x28, 0x531, 0x544, 0x546, 0x29, 0x53c, 0x565,
0x57c, 0x576, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x20,
0x28, 0x531, 0x544, 0x546, 0x29, 0x53d, 0x561, 0x572, 0x561, 0x572, 0x585, 0x57e,
0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53d, 0x561, 0x572, 0x561, 0x572, 0x585, 0x57e,
0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53d, 0x561, 0x572, 0x561, 0x572, 0x585, 0x57e, 0x56f, 0x56b, 0x561,
0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x57a, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x57a, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x57a, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d,
0x561, 0x578, 0x582, 0x564, 0x575, 0x561, 0x576, 0x20, 0x531, 0x580, 0x561, 0x562,
0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54d, 0x561, 0x578, 0x582, 0x564, 0x575, 0x561, 0x576, 0x20, 0x531, 0x580, 0x561,
0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561,
0x578, 0x582, 0x564, 0x575, 0x561, 0x576, 0x20, 0x531, 0x580, 0x561, 0x562, 0x56b,
0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x563, 0x565,
0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x580, 0x563, 0x565, 0x576, 0x57f, 0x56b, 0x576, 0x561,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x576, 0x561, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x563, 0x565, 0x576,
0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x531, 0x580, 0x563, 0x565,
0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20,
0x531, 0x580, 0x563, 0x565, 0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b, 0x20,
0x57d, 0x57f, 0x576, 0x561, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576,
0x20, 0x531, 0x580, 0x563, 0x565, 0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x561, 0x575, 0x561, 0x57d, 0x57f, 0x561,
0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x561,
0x575, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x540, 0x561, 0x575, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x57f, 0x56c, 0x561, 0x576, 0x57f, 0x575, 0x561, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x57f, 0x56c, 0x561, 0x576,
0x57f, 0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x57f, 0x56c,
0x561, 0x576, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531,
0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578,
0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561,
0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565,
0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x57e,
0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561,
0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575,
0x56b, 0x20, 0x561, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578,
0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561,
0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x580, 0x587, 0x574, 0x57f, 0x575,
0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580,
0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580,
0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x580, 0x587, 0x574, 0x57f,
0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565,
0x56c, 0x575, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c,
0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d,
0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x531, 0x57e,
0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x531,
0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575,
0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f,
0x575, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b,
0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x564, 0x580, 0x562,
0x565, 0x57b, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x564, 0x580, 0x562, 0x565, 0x57b, 0x561, 0x576, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x564, 0x580, 0x562, 0x565, 0x57b, 0x561, 0x576, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x566, 0x578, 0x580, 0x575, 0x561, 0x576,
0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x566, 0x578, 0x580, 0x575, 0x561, 0x576,
0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x566, 0x578, 0x580, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572,
0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532,
0x561, 0x576, 0x563, 0x56c, 0x561, 0x564, 0x565, 0x577, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x561, 0x576, 0x563, 0x56c, 0x561, 0x564,
0x565, 0x577, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x561, 0x576, 0x563,
0x56c, 0x561, 0x564, 0x565, 0x577, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532,
0x578, 0x582, 0x569, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x532, 0x578, 0x56c, 0x56b, 0x57e, 0x56b, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x580, 0x561, 0x566, 0x56b,
0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x532, 0x580, 0x561, 0x566, 0x56b, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x532, 0x580, 0x561, 0x566, 0x56b, 0x56c, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x580, 0x578, 0x582, 0x576,
0x565, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x561, 0x562, 0x578, 0x20, 0x54e, 0x565, 0x580, 0x564, 0x565, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x561, 0x562, 0x578, 0x20, 0x54e,
0x565, 0x580, 0x564, 0x565, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x561,
0x562, 0x578, 0x20, 0x54e, 0x565, 0x580, 0x564, 0x565, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x549, 0x561, 0x574, 0x578, 0x57c, 0x578, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x561, 0x569, 0x565, 0x574, 0x20,
0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x549, 0x561, 0x569, 0x565, 0x574, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x561, 0x569, 0x565, 0x574, 0x20, 0x56f,
0x572, 0x566, 0x578, 0x582, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x56b, 0x56c,
0x56b, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x56b,
0x56c, 0x56b, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x56b, 0x56c, 0x56b,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x56b, 0x576, 0x561, 0x57d, 0x57f,
0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549,
0x56b, 0x576, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x549, 0x56b, 0x576, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54d, 0x578, 0x582, 0x580, 0x562, 0x20, 0x53e, 0x576, 0x576,
0x564, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x56f, 0x578, 0x57d, 0x575,
0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x56c, 0x578, 0x582,
0x574, 0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53f, 0x578, 0x56c, 0x578, 0x582, 0x574, 0x562, 0x56b, 0x561, 0x575,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x56c, 0x578, 0x582, 0x574,
0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578,
0x582, 0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x56f,
0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566,
0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56f, 0x56b, 0x57d, 0x561, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53f, 0x578, 0x582, 0x562, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x562, 0x561, 0x575, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x562, 0x561, 0x575, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x534, 0x565, 0x575, 0x57e, 0x56b, 0x57d, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x534, 0x575, 0x578, 0x582,
0x574, 0x578, 0x576, 0x20, 0x564, 0x2019, 0x545, 0x578, 0x582, 0x580, 0x57e, 0x56b,
0x56c, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x539, 0x56b, 0x574, 0x578, 0x580,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x536, 0x561, 0x57f,
0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x536, 0x561, 0x57f, 0x56f, 0x56b, 0x20, 0x56f, 0x572,
0x566, 0x578, 0x582, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x536, 0x561, 0x57f, 0x56f,
0x56b, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x537, 0x56f, 0x57e, 0x561, 0x564, 0x578, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561,
0x56f, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578, 0x57a, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f,
0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578,
0x57a, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576,
0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580,
0x578, 0x57a, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578, 0x57a,
0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578,
0x57a, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587,
0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578, 0x57a, 0x561,
0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x56b, 0x576, 0x57d, 0x56f,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587,
0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578, 0x57a, 0x561,
0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578, 0x57a,
0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574,
0x57f, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578, 0x57a, 0x561, 0x575,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x578, 0x56c, 0x584, 0x56c, 0x565,
0x576, 0x564, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565,
0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x578,
0x56c, 0x584, 0x56c, 0x565, 0x576, 0x564, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572,
0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556,
0x578, 0x56c, 0x584, 0x56c, 0x565, 0x576, 0x564, 0x575, 0x561, 0x576, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x556, 0x56b, 0x57b, 0x56b, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x556, 0x56b, 0x57b, 0x56b, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556,
0x56b, 0x57b, 0x56b, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x580, 0x561,
0x576, 0x57d, 0x56b, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x533, 0x57e, 0x56b, 0x561,
0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x556, 0x580, 0x561, 0x576, 0x57d, 0x56b, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x570,
0x561, 0x580, 0x561, 0x57e, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x587, 0x20, 0x561,
0x576, 0x57f, 0x561, 0x580, 0x56f, 0x57f, 0x56b, 0x564, 0x575, 0x561, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x533, 0x561, 0x56c, 0x561, 0x57a,
0x561, 0x563, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b,
0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x533, 0x561, 0x574, 0x562, 0x575, 0x565, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576,
0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e,
0x580, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54e, 0x580, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54e, 0x580, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54b, 0x56b, 0x56c, 0x562, 0x565, 0x580, 0x569,
0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x533, 0x580, 0x56b, 0x576, 0x57e, 0x56b,
0x579, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x533, 0x580, 0x565, 0x576, 0x56c,
0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20,
0x533, 0x580, 0x565, 0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561,
0x576, 0x20, 0x533, 0x580, 0x565, 0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f,
0x575, 0x561, 0x576, 0x20, 0x533, 0x580, 0x565, 0x576, 0x56c, 0x561, 0x576, 0x564,
0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x533, 0x580, 0x565,
0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x533,
0x580, 0x565, 0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x580, 0x57d, 0x56b, 0x581, 0x20, 0x56e,
0x578, 0x581, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x533, 0x561, 0x575, 0x561,
0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x540, 0x561, 0x57e, 0x561, 0x575, 0x561, 0x576, 0x2d, 0x561, 0x56c, 0x565, 0x578,
0x582, 0x569, 0x575, 0x561, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x540, 0x561, 0x57e, 0x561, 0x575, 0x561, 0x576, 0x2d, 0x561, 0x56c, 0x565,
0x578, 0x582, 0x569, 0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540,
0x561, 0x57e, 0x561, 0x575, 0x561, 0x576, 0x2d, 0x561, 0x56c, 0x565, 0x578, 0x582,
0x569, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x578, 0x576,
0x56f, 0x578, 0x576, 0x563, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x540, 0x578, 0x576, 0x56f, 0x578, 0x576, 0x563, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x540, 0x578, 0x576, 0x56f, 0x578, 0x576, 0x563, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x540, 0x578, 0x57e, 0x564, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x540, 0x578, 0x57e, 0x564, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x540, 0x578, 0x57e, 0x564, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x540, 0x576, 0x564, 0x56f, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x540, 0x576, 0x564, 0x56f, 0x561, 0x56f, 0x561, 0x576, 0x20,
0x585, 0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x576, 0x564, 0x56f, 0x561, 0x579, 0x56b,
0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576,
0x20, 0x53b, 0x576, 0x564, 0x578, 0x576, 0x565, 0x566, 0x56b, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565,
0x56c, 0x575, 0x561, 0x576, 0x20, 0x53b, 0x576, 0x564, 0x578, 0x576, 0x565, 0x566,
0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x53b, 0x576, 0x564,
0x578, 0x576, 0x565, 0x566, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x561, 0x576, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x53b, 0x580, 0x561, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x53b, 0x580, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x56f, 0x578, 0x582, 0x57f,
0x57d, 0x56f, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x56f, 0x578,
0x582, 0x57f, 0x57d, 0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x57d,
0x580, 0x561, 0x575, 0x565, 0x56c, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53b, 0x57d, 0x580, 0x561, 0x575, 0x565, 0x56c, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x53b, 0x57d, 0x580, 0x561, 0x575, 0x565, 0x56c, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x543, 0x561, 0x57a, 0x578, 0x576, 0x56b, 0x561, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x543, 0x561, 0x57a,
0x578, 0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x543,
0x561, 0x57a, 0x578, 0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x542, 0x561, 0x566, 0x561, 0x56d, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c,
0x575, 0x561, 0x576, 0x20, 0x542, 0x561, 0x566, 0x561, 0x56d, 0x57d, 0x57f, 0x561,
0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x542, 0x561, 0x566, 0x561, 0x56d,
0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x53f, 0x578, 0x580, 0x565, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x580, 0x565, 0x561, 0x575, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x580, 0x565, 0x561, 0x575, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x57d, 0x580, 0x561, 0x565, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x580, 0x561, 0x57d,
0x576, 0x578, 0x575, 0x561, 0x580, 0x57d, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53f, 0x580, 0x561, 0x57d, 0x576, 0x578, 0x575, 0x561,
0x580, 0x57d, 0x56f, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x580, 0x561,
0x57d, 0x576, 0x578, 0x575, 0x561, 0x580, 0x57d, 0x56f, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x542, 0x580, 0x572, 0x566, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53c, 0x561, 0x575, 0x576, 0x20,
0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53c, 0x578, 0x580, 0x564, 0x20, 0x540, 0x561, 0x578,
0x582, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53c, 0x578,
0x580, 0x564, 0x20, 0x540, 0x561, 0x578, 0x582, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x53c, 0x578, 0x580, 0x564, 0x20, 0x540, 0x561, 0x578, 0x582, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x563, 0x561, 0x564, 0x561, 0x576, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x563, 0x561,
0x564, 0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x563,
0x561, 0x564, 0x561, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561,
0x56c, 0x561, 0x575, 0x566, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x56c, 0x564, 0x56b, 0x57e, 0x576, 0x565,
0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561,
0x580, 0x56f, 0x56b, 0x566, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b,
0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x544, 0x561, 0x580, 0x577, 0x561, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572,
0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x544, 0x561, 0x57e, 0x580, 0x56b, 0x56f, 0x56b, 0x578, 0x57d, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x57e, 0x580,
0x56b, 0x56f, 0x56b, 0x578, 0x57d, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544,
0x561, 0x57e, 0x580, 0x56b, 0x56f, 0x56b, 0x578, 0x57d, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x544, 0x578, 0x578, 0x582, 0x57d, 0x578, 0x576, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x565, 0x584, 0x57d, 0x56b, 0x56f,
0x561, 0x575, 0x56b, 0x20, 0x56d, 0x561, 0x572, 0x561, 0x572, 0x585, 0x57e, 0x56f,
0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x544, 0x565, 0x584, 0x57d, 0x56b, 0x56f, 0x561, 0x575,
0x56b, 0x20, 0x56d, 0x561, 0x572, 0x561, 0x572, 0x585, 0x57e, 0x56f, 0x56b, 0x561,
0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544,
0x565, 0x584, 0x57d, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56d, 0x561, 0x572,
0x561, 0x572, 0x585, 0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561,
0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x56c, 0x561, 0x576, 0x20,
0x532, 0x561, 0x57f, 0x578, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x548, 0x582, 0x56c, 0x561, 0x576, 0x20, 0x532, 0x561, 0x57f, 0x578,
0x580, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x56c, 0x561, 0x576,
0x20, 0x532, 0x561, 0x57f, 0x578, 0x580, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x544, 0x578, 0x57d, 0x56f, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x544, 0x578, 0x57d, 0x56f, 0x57e, 0x561, 0x575, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x578, 0x57d, 0x56f, 0x57e, 0x561, 0x575,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x575, 0x561, 0x576, 0x574, 0x561,
0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x561,
0x578, 0x582, 0x580, 0x578, 0x582, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x546, 0x565, 0x57a, 0x561, 0x56c, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x20, 0x53f, 0x561, 0x56c, 0x565,
0x564, 0x578, 0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x20, 0x53f, 0x561, 0x56c, 0x565, 0x564,
0x578, 0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546,
0x578, 0x580, 0x20, 0x53f, 0x561, 0x56c, 0x565, 0x564, 0x578, 0x576, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x20, 0x536,
0x565, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x20, 0x536, 0x565, 0x56c,
0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x546, 0x578, 0x580, 0x20, 0x536, 0x565, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x575, 0x578, 0x582, 0x586,
0x561, 0x578, 0x582, 0x576, 0x564, 0x56c, 0x565, 0x576, 0x564, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x575, 0x578, 0x582, 0x586, 0x561,
0x578, 0x582, 0x576, 0x564, 0x56c, 0x565, 0x576, 0x564, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x546, 0x575, 0x578, 0x582, 0x586, 0x561, 0x578, 0x582, 0x576, 0x564,
0x56c, 0x565, 0x576, 0x564, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x56b,
0x578, 0x582, 0x565, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x546, 0x578, 0x580, 0x586, 0x578, 0x56c, 0x56f, 0x20, 0x56f, 0x572, 0x566,
0x578, 0x582, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578,
0x580, 0x586, 0x578, 0x56c, 0x56f, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x586, 0x578, 0x56c, 0x56f, 0x20,
0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x565,
0x580, 0x576, 0x561, 0x576, 0x564, 0x578, 0x582, 0x20, 0x564, 0x56b, 0x20, 0x546,
0x578, 0x580, 0x578, 0x576, 0x575, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x556, 0x565, 0x580, 0x576, 0x561, 0x576, 0x564, 0x578,
0x582, 0x20, 0x564, 0x56b, 0x20, 0x546, 0x578, 0x580, 0x578, 0x576, 0x575, 0x561,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x565, 0x580, 0x576, 0x561,
0x576, 0x564, 0x578, 0x582, 0x20, 0x564, 0x56b, 0x20, 0x546, 0x578, 0x580, 0x578,
0x576, 0x575, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578,
0x57e, 0x578, 0x57d, 0x56b, 0x562, 0x56b, 0x580, 0x57d, 0x56f, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x57e, 0x578, 0x57d, 0x56b,
0x562, 0x56b, 0x580, 0x57d, 0x56f, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546,
0x578, 0x57e, 0x578, 0x57d, 0x56b, 0x562, 0x56b, 0x580, 0x57d, 0x56f, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x555, 0x574, 0x57d, 0x56f, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x555, 0x574, 0x57d, 0x56f, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x555, 0x574, 0x57d, 0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54a, 0x561, 0x56f, 0x56b, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x56f, 0x56b, 0x57d, 0x57f,
0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x56f, 0x56b,
0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561,
0x56c, 0x561, 0x578, 0x582, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54a, 0x561, 0x57a, 0x578, 0x582, 0x561, 0x20, 0x546, 0x578, 0x580,
0x20, 0x533, 0x57e, 0x56b, 0x576, 0x565, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x580, 0x561, 0x563, 0x57e, 0x561,
0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561,
0x580, 0x561, 0x563, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54a, 0x561, 0x580, 0x561, 0x563, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54a, 0x565, 0x580, 0x578, 0x582, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54a, 0x565, 0x580, 0x578, 0x582, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54a, 0x565, 0x580, 0x578, 0x582, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x556, 0x56b, 0x56c, 0x56b, 0x57a, 0x56b, 0x576, 0x576, 0x565, 0x580,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x56b, 0x56c,
0x56b, 0x57a, 0x56b, 0x576, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x556, 0x56b, 0x56c, 0x56b, 0x57a, 0x56b, 0x576, 0x576, 0x565, 0x580, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x56b, 0x576, 0x56b, 0x584, 0x57d, 0x20,
0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54d, 0x565, 0x576, 0x20, 0x54a, 0x56b, 0x565, 0x57c,
0x20, 0x587, 0x20, 0x544, 0x56b, 0x584, 0x565, 0x56c, 0x578, 0x576, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x565, 0x576, 0x20, 0x54a,
0x56b, 0x565, 0x57c, 0x20, 0x587, 0x20, 0x544, 0x56b, 0x584, 0x565, 0x56c, 0x578,
0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x565, 0x576, 0x20, 0x54a,
0x56b, 0x565, 0x57c, 0x20, 0x587, 0x20, 0x544, 0x56b, 0x584, 0x565, 0x56c, 0x578,
0x576, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x56b, 0x57f, 0x56f, 0x565,
0x57c, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a,
0x578, 0x576, 0x561, 0x57a, 0x565, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x553, 0x56d, 0x565, 0x576, 0x575,
0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54c,
0x565, 0x575, 0x578, 0x582, 0x576, 0x56b, 0x578, 0x576, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54c, 0x578, 0x57f, 0x565, 0x580, 0x561, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561, 0x56d,
0x561, 0x56c, 0x56b, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54d, 0x561, 0x56d, 0x561, 0x56c, 0x56b, 0x576, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54d, 0x561, 0x56d, 0x561, 0x56c, 0x56b, 0x576, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54d, 0x561, 0x574, 0x578, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561, 0x574, 0x578, 0x561, 0x575,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561, 0x574, 0x578, 0x561, 0x575,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x565, 0x575, 0x577, 0x565, 0x56c,
0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x56b, 0x576, 0x563,
0x561, 0x57a, 0x578, 0x582, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54d, 0x578, 0x572, 0x578, 0x574, 0x578, 0x576, 0x56b, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x540, 0x561, 0x580, 0x561, 0x57e, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x54b, 0x578, 0x580, 0x57b, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x578, 0x582, 0x580, 0x56b, 0x576, 0x561,
0x574, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x575,
0x578, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x539, 0x561, 0x56b, 0x569, 0x56b, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x539, 0x561, 0x575, 0x57a, 0x565, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539, 0x561, 0x575, 0x57a, 0x565, 0x575,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539, 0x561, 0x575, 0x57a, 0x565, 0x575,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54f, 0x561, 0x57b, 0x56b, 0x56f, 0x57d,
0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54f, 0x578, 0x56f, 0x565, 0x56c, 0x561, 0x578, 0x582, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54f, 0x578, 0x576, 0x563, 0x561, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54f, 0x578, 0x576,
0x563, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54f, 0x578, 0x576,
0x563, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54f, 0x580, 0x578,
0x582, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539,
0x578, 0x582, 0x580, 0x584, 0x574, 0x565, 0x576, 0x57d, 0x57f, 0x561, 0x576, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539, 0x578, 0x582, 0x580,
0x584, 0x574, 0x565, 0x576, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x539, 0x578, 0x582, 0x580, 0x584, 0x574, 0x565, 0x576, 0x57d, 0x57f,
0x561, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54f, 0x578, 0x582, 0x57e,
0x561, 0x56c, 0x578, 0x582, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x548, 0x582, 0x580, 0x578, 0x582, 0x563, 0x57e, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x580, 0x578,
0x582, 0x563, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548,
0x582, 0x580, 0x578, 0x582, 0x563, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x548, 0x582, 0x566, 0x562, 0x565, 0x56f, 0x57d, 0x57f, 0x561, 0x576,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x566,
0x562, 0x565, 0x56f, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x548, 0x582, 0x566, 0x562, 0x565, 0x56f, 0x57d, 0x57f, 0x561, 0x576, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x561, 0x576, 0x578, 0x582, 0x561, 0x57f,
0x578, 0x582, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54e, 0x561, 0x576, 0x578, 0x582, 0x561, 0x57f, 0x578, 0x582, 0x575, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54e, 0x561, 0x576, 0x578, 0x582, 0x561, 0x57f, 0x578,
0x582, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x565, 0x576, 0x565,
0x57d, 0x578, 0x582, 0x565, 0x56c, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54e, 0x56c, 0x561, 0x564, 0x56b, 0x57e, 0x578, 0x57d,
0x57f, 0x578, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54e, 0x56c, 0x561, 0x564, 0x56b, 0x57e, 0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x56c, 0x561, 0x564, 0x56b, 0x57e, 0x578,
0x57d, 0x57f, 0x578, 0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x578,
0x56c, 0x563, 0x578, 0x563, 0x580, 0x561, 0x564, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54e, 0x578, 0x56c, 0x563, 0x578, 0x563, 0x580, 0x561,
0x564, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x578, 0x56c, 0x563, 0x578,
0x563, 0x580, 0x561, 0x564, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x578,
0x57d, 0x57f, 0x578, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x548, 0x582, 0x565, 0x575, 0x584, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x578, 0x56c,
0x56b, 0x57d, 0x20, 0x587, 0x20, 0x556, 0x578, 0x582, 0x57f, 0x578, 0x582, 0x576,
0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x545,
0x561, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x545, 0x561, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x545, 0x561, 0x56f, 0x578, 0x582, 0x57f,
0x57d, 0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x535, 0x56f, 0x561, 0x57f,
0x565, 0x580, 0x56b, 0x576, 0x562, 0x578, 0x582, 0x580, 0x563, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x535, 0x56f, 0x561, 0x57f, 0x565, 0x580,
0x56b, 0x576, 0x562, 0x578, 0x582, 0x580, 0x563, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x535, 0x56f, 0x561, 0x57f, 0x565, 0x580, 0x56b, 0x576, 0x562, 0x578, 0x582,
0x580, 0x563, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x545, 0x578, 0x582, 0x56f,
0x578, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x986,
0x9ab, 0x997, 0x9be, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x986, 0x9ab, 0x9cd,
0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2,
0x9ac, 0x20, 0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x986, 0x9ab,
0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986, 0x9ab,
0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995,
0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995,
0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9be, 0x9b8,
0x9cd, 0x995, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9be,
0x9b8, 0x9cd, 0x995, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x9b2, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x9f0, 0x20, 0x9a1,
0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9ae,
0x9be, 0x99c, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9ae, 0x9be,
0x99c, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995,
0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20,
0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8,
0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae,
0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6,
0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986,
0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9be,
0x99a, 0x9cd, 0x9af, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4,
0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa,
0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7,
0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd,
0x9af, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf,
0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9f0, 0x9cd, 0x9ac, 0x9cd, 0x9ac, 0x9a4, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0,
0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9f0,
0x9cd, 0x9ac, 0x9cd, 0x9ac, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986,
0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9f0, 0x9cd, 0x9ac,
0x9cd, 0x9ac, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987,
0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20,
0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0,
0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4,
0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0,
0x20, 0x9aa, 0x9cd, 0x9f0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20,
0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2,
0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9aa, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9aa, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9ac, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9ac, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9ac,
0x9c0, 0x9af, 0x9bc, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf,
0x9a8, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x99c,
0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd,
0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x986, 0x9f0, 0x9cd,
0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20,
0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6,
0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7,
0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x986, 0x9f0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x986, 0x9f0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99f, 0x9b2, 0x9be, 0x9a3, 0x9cd,
0x99f, 0x9bf, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99f, 0x9b2, 0x9be,
0x9a3, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x99f, 0x9b2, 0x9be, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20,
0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9a7, 0x9cd, 0x9af, 0x20, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7,
0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd,
0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8,
0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd,
0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8,
0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9be, 0x9b6, 0x9cd,
0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7,
0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa,
0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7,
0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6,
0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be,
0x9a4, 0x9cd, 0x9af, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x985,
0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7,
0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a,
0x9cd, 0x9af, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985,
0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x9a1, 0x9c7, 0x9b2,
0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd,
0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0,
0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9f0, 0x20, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f,
0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9be,
0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9c7, 0x9f0, 0x9ac,
0x9be, 0x987, 0x99c, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x99c, 0x9c7, 0x9f0, 0x9ac, 0x9be, 0x987, 0x99c, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9c7, 0x9f0, 0x9ac,
0x9be, 0x987, 0x99c, 0x9be, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x99c, 0x9cb, 0x9f0, 0x9c7, 0x99b, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x99c, 0x9cb, 0x9f0, 0x9c7, 0x99b, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9cb, 0x9f0, 0x9c7, 0x99b, 0x9f0, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x982, 0x9b2, 0x9be, 0x9a6,
0x9c7, 0x9b6, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ac, 0x9be, 0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9c1, 0x99f, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ac, 0x9b2, 0x9bf, 0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x99c, 0x9bf, 0x9b2, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9f0,
0x9be, 0x99c, 0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x99c, 0x9bf,
0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ac, 0x9cd, 0x9f0, 0x9c1, 0x9a8, 0x9c7, 0x987, 0x20, 0x9a1, 0x9be, 0x9f0, 0x9c1,
0x99b, 0x9be, 0x9b2, 0x9c7, 0x9ae, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9f0, 0x9cd, 0x9a6, 0x9c7, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9f0, 0x9cd, 0x9a6,
0x9c7, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9f0, 0x9cd, 0x9a6, 0x9c7, 0x9f0, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9be, 0x9ae, 0x9cb, 0x9f0, 0x9cb, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9be, 0x9a5, 0x9be,
0x9ae, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9be, 0x9a5, 0x9be, 0x9ae,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9be,
0x9a5, 0x9be, 0x9ae, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c0, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99a, 0x9c0, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99a, 0x9c0, 0x9a8, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x996, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x99f,
0x9ae, 0x9be, 0x99b, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x995, 0x9cb, 0x99b, 0x20, 0x9a6, 0x9cd, 0x9ac,
0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9b2, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9b2, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9b2, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd,
0x99c, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd,
0x99c, 0x9f0, 0x20, 0x985, 0x9f0, 0x9cd, 0x9a7, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7,
0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a1, 0x9c7, 0x9ad,
0x9bf, 0x99b, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a1, 0x9c1, 0x9ae, 0x9cb,
0x9a3, 0x9cd, 0x99f, 0x2d, 0x9a1, 0x9bf, 0x20, 0x986, 0x9f0, 0x9cd, 0x9ad, 0x9bf,
0x9b2, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9ac, 0x20, 0x9a4,
0x9bf, 0x9ae, 0x9cb, 0x9f0, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b7,
0x9cd, 0x99f, 0x9be, 0x9f0, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x20, 0x9a6,
0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x20, 0x9a6, 0x9cd, 0x9ac,
0x9c0, 0x9aa, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x995, 0x9c1,
0x9f1, 0x9c7, 0x9a1, 0x9f0, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7,
0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0,
0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa,
0x9c0, 0x9af, 0x9bc, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0,
0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd,
0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a,
0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x997, 0x9cd, 0x9f0, 0x2d, 0x9aa, 0x9cd,
0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a,
0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be,
0x9a4, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6,
0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa,
0x9c0, 0x9af, 0x9bc, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995, 0x9b2,
0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1,
0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995, 0x9b2,
0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1,
0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ab, 0x995, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac,
0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ab, 0x9bf, 0x99c, 0x9bf, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ab, 0x9cd, 0x9f0, 0x9be, 0x9a8, 0x9cd, 0x9b8, 0x20, 0x997, 0x9af, 0x9bc,
0x9be, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995, 0x9cd,
0x9b7, 0x9bf, 0x9a3, 0x20, 0x9ab, 0x9cd, 0x9f0, 0x9be, 0x9a8, 0x9cd, 0x9b8, 0x20,
0x986, 0x9f0, 0x9c1, 0x20, 0x98f, 0x9a3, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x9cd, 0x995,
0x99f, 0x9bf, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9be, 0x9b2, 0x9be,
0x9aa, 0x9be, 0x997, 0x9cb, 0x99b, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997,
0x9c7, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9f0,
0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x997, 0x9bf, 0x9b2, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x20, 0x9a6, 0x9cd,
0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x989, 0x987, 0x99a, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9ac, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9ac, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3,
0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9ac, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3,
0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3,
0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x9b2,
0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x989, 0x9aa, 0x9b8, 0x9be, 0x997, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9be, 0x9af, 0x9bc,
0x9be, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x9f1,
0x9be, 0x987, 0x2d, 0x98f, 0x9b2, 0x9bf, 0x989, 0x9b6, 0x9cd, 0x9ac, 0x9a8, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x9f1, 0x9be, 0x987, 0x2d, 0x98f,
0x9b2, 0x9bf, 0x989, 0x9b6, 0x9cd, 0x9ac, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x9f1, 0x9be, 0x987, 0x2d, 0x98f,
0x9b2, 0x9bf, 0x989, 0x9b6, 0x9cd, 0x9ac, 0x9a8, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2,
0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982, 0x20, 0x995,
0x999, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982, 0x20, 0x995, 0x999,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982,
0x20, 0x995, 0x999, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9cb,
0x9ad, 0x9cd, 0x200c, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9cb,
0x9ad, 0x9cd, 0x200c, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9cd, 0x200c, 0x9a1, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9f0, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9f0, 0x9a4, 0x20,
0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x99a, 0x9be, 0x987, 0x9a8,
0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20,
0x987, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x9a8, 0x9c7, 0x99a, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd,
0x9af, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x9a8, 0x9c7, 0x99a, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd,
0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x9a8,
0x9c7, 0x99a, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x9f0, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0,
0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x9f0, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0, 0x9cd, 0x995, 0x9c1, 0x99f, 0x9b8,
0x9cd, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0, 0x9cd, 0x995,
0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0, 0x9cd, 0x995, 0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c, 0x9f0, 0x9be, 0x987,
0x9b2, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c, 0x9f0, 0x9be, 0x987,
0x9b2, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987,
0x99c, 0x9f0, 0x9be, 0x987, 0x9b2, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987,
0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be, 0x9a8, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be, 0x9a8, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be,
0x9a8, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9be, 0x99c, 0x9be, 0x996, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9ac, 0x20, 0x995, 0x9be,
0x99c, 0x9be, 0x996, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x995, 0x9be, 0x99c,
0x9be, 0x996, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9cb, 0x9f0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9cb, 0x9f0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9f0, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x99b, 0x9f0, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9f0, 0x9be, 0x99b, 0x9a8, 0x9cb, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9cd, 0x9f0, 0x9be, 0x99b, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9cd, 0x9f0, 0x9be, 0x99b, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be,
0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9bf, 0x9f0, 0x9cd, 0x997, 0x9bf, 0x99c, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9be, 0x987, 0x9a8, 0x20, 0x9a6, 0x9cd,
0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b2, 0x9f0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9f0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9f0,
0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x997, 0x9be, 0x9a6,
0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x997, 0x9be,
0x9a6, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9be, 0x997, 0x9be, 0x9a6, 0x9be, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9b2, 0x9af, 0x9bc, 0x9c7, 0x99a, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9b2, 0x9a6,
0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be,
0x9f0, 0x9cd, 0x995, 0x99b, 0x9c7, 0x99b, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ae, 0x9be, 0x9f0, 0x9cd, 0x9b6, 0x9cd, 0x9ac, 0x9be, 0x9b2, 0x20, 0x9a6, 0x9cd,
0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ae, 0x9f0, 0x9bf, 0x99b, 0x9be, 0x99b, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ae, 0x9f0, 0x9bf, 0x99b, 0x9be, 0x99b, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9f0, 0x9bf, 0x99b, 0x9be, 0x99b,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x989, 0x99b, 0x9a8,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf,
0x995, 0x9cb, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995,
0x9cb, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9c7, 0x995, 0x9cd,
0x9b8, 0x9bf, 0x995, 0x9cb, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9b6, 0x9be, 0x9a8,
0x9cd, 0x9a4, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x99f, 0x9be, 0x9f0, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x99f,
0x9be, 0x9f0, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x99f, 0x9be, 0x9f0, 0x9f0, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9cb, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9cb, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9cb, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9ae,
0x9be, 0x9f0, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9be, 0x989, 0x9f0,
0x9c1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9c7, 0x9aa, 0x9be, 0x9b2,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20, 0x995, 0x9c7,
0x9b2, 0x9bf, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20, 0x995, 0x9c7, 0x9b2, 0x9bf, 0x9a1,
0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20, 0x995, 0x9c7, 0x9b2, 0x9bf,
0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf, 0x9b2,
0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1,
0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9a8, 0x9bf, 0x989, 0x9ab, 0x9be, 0x989, 0x9a3, 0x9cd, 0x9a1, 0x9b2, 0x9c7,
0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989,
0x9ab, 0x9be, 0x989, 0x9a3, 0x9cd, 0x9a1, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989,
0x9ab, 0x9be, 0x989, 0x9a3, 0x9cd, 0x9a1, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0,
0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9c1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8,
0x2019, 0x9f0, 0x9ab, 0x2019, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x2019, 0x9f0, 0x9ab, 0x2019, 0x995, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x9cd, 0x9af,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x2019, 0x9f0, 0x9ab, 0x2019, 0x995, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ab, 0x9be, 0x9f0, 0x9cd, 0x9a8, 0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x20, 0x9a1,
0x9c7, 0x20, 0x9a8, 0x9cb, 0x9f0, 0x9cb, 0x9a8, 0x9cd, 0x200c, 0x9b9, 0x9be, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be, 0x9f0, 0x9cd, 0x9a8, 0x9be, 0x9a3,
0x9cd, 0x9a1, 0x9cb, 0x20, 0x9a1, 0x9c7, 0x20, 0x9a8, 0x9cb, 0x9f0, 0x9cb, 0x9a8,
0x9cd, 0x200c, 0x9b9, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ab, 0x9be, 0x9f0, 0x9cd, 0x9a8, 0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9cb,
0x20, 0x9a1, 0x9c7, 0x20, 0x9a8, 0x9cb, 0x9f0, 0x9cb, 0x9a8, 0x9cd, 0x200c, 0x9b9,
0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9ad, 0x9cb, 0x99b,
0x9bf, 0x9ac, 0x9bf, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9a8, 0x9ad, 0x9cb, 0x99b, 0x9bf, 0x9ac, 0x9bf, 0x9f0, 0x9cd, 0x9b8,
0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a8, 0x9ad, 0x9cb, 0x99b, 0x9bf, 0x9ac, 0x9bf, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8,
0x9cd, 0x995, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x995,
0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8,
0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9be, 0x9b2, 0x9be, 0x989, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be,
0x9aa, 0x9c1, 0x9f1, 0x9be, 0x20, 0x9a8, 0x9bf, 0x989, 0x20, 0x997, 0x9bf, 0x9a8,
0x9bf, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9f0, 0x9be, 0x997,
0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9f0,
0x9be, 0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9f0, 0x9be, 0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9f0, 0x9c1, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9f0, 0x9c1, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9f0, 0x9c1, 0x9f0, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987,
0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa,
0x9be, 0x987, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8, 0x9f0, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9a8, 0x9bf, 0x995, 0x9cd, 0x9b8, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9aa,
0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9f0, 0x9c7, 0x20, 0x986, 0x9f0, 0x9c1, 0x20, 0x9ae,
0x9bf, 0x995, 0x9bf, 0x989, 0x9f1, 0x9c7, 0x9b2, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9aa, 0x9bf, 0x9af,
0x9bc, 0x9c7, 0x9f0, 0x9c7, 0x20, 0x986, 0x9f0, 0x9c1, 0x20, 0x9ae, 0x9bf, 0x995,
0x9bf, 0x989, 0x9f1, 0x9c7, 0x9b2, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9aa,
0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9f0, 0x9c7, 0x20, 0x986, 0x9f0, 0x9c1, 0x20, 0x9ae,
0x9bf, 0x995, 0x9bf, 0x989, 0x9f1, 0x9c7, 0x9b2, 0x9a8, 0x9f0, 0x20, 0x9a1, 0x9c7,
0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x99f,
0x995, 0x9c7, 0x987, 0x9f0, 0x9cd, 0x9a3, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9cb, 0x9a8, 0x9be, 0x9aa, 0x9c7, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9cd, 0x9af, 0x982, 0x9af, 0x9bc, 0x9be, 0x982, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9f0, 0x9bf, 0x987, 0x989, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9f0, 0x9cb, 0x9a5, 0x9c7, 0x9f0, 0x9be, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf,
0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b,
0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99b, 0x9be, 0x9ae, 0x9cb, 0x9f1, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99b, 0x9be, 0x9ae, 0x9cb, 0x9f1, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be, 0x9ae, 0x9cb, 0x9f1, 0x9be, 0x9f0,
0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99b, 0x9bf, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x99b, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99b, 0x9bf, 0x982, 0x997, 0x9be, 0x9aa, 0x9c1, 0x9f0, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9cb, 0x9b2, 0x9cb, 0x9ae,
0x9cb, 0x9a8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd,
0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf,
0x9a3, 0x20, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9c1, 0x9f0, 0x9bf, 0x9a8, 0x9be, 0x9ae, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9cd, 0x9ac, 0x9cb, 0x9f1, 0x9be, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x9b9, 0x9bf, 0x9a4, 0x9bf, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9be, 0x987, 0x9aa,
0x9c7, 0x987, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x99c, 0x9bf, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4,
0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9cb, 0x995, 0x9c7,
0x9b2, 0x9be, 0x989, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x982, 0x997,
0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x982, 0x997, 0x9be, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x982, 0x997,
0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9cd, 0x99a, 0x9c1,
0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9f0, 0x9cd, 0x995,
0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9f0, 0x9cd, 0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf,
0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9f0, 0x9cd, 0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf,
0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99f, 0x9c1, 0x9ad, 0x9be, 0x9b2, 0x9be, 0x989, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x989, 0x9f0, 0x9c1, 0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x989, 0x9f0, 0x9c1, 0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9f0, 0x9c1, 0x997, 0x9c1,
0x9f1, 0x9c7, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x99c, 0x9ac,
0x9c7, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be,
0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989,
0x99c, 0x9ac, 0x9c7, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9a8, 0x9be, 0x99f, 0x9c1, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9a8, 0x9be, 0x99f, 0x9c1, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9a8,
0x9be, 0x99f, 0x9c1, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9c7,
0x9a8, 0x9bf, 0x99c, 0x9c1, 0x9f1, 0x9c7, 0x9b2, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a1, 0x9bf, 0x9ad, 0x9cb, 0x9b7, 0x9cd,
0x99f, 0x9cb, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2,
0x9be, 0x9a1, 0x9bf, 0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x9cb, 0x995, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be,
0x9a1, 0x9bf, 0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x9cb, 0x995, 0x9f0, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cb, 0x9b2, 0x9cd, 0x997, 0x9cb, 0x997, 0x9cd,
0x9f0, 0x9be, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cb, 0x9b2,
0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9f0, 0x9be, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cb, 0x9b2, 0x9cd, 0x997, 0x9cb,
0x997, 0x9cd, 0x9f0, 0x9be, 0x9a1, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9f1, 0x9c7, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9f1, 0x9be, 0x9b2, 0x9bf, 0x99b, 0x20, 0x986, 0x9f0, 0x9c1,
0x20, 0x9ab, 0x9c1, 0x99f, 0x9c1, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x9a4, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x9a4, 0x9cd,
0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x9a4, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x99f,
0x9c7, 0x9f0, 0x9bf, 0x9a3, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x99f, 0x9c7, 0x9f0, 0x9bf,
0x9a3, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x99f, 0x9c7, 0x9f0,
0x9bf, 0x9a3, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9af, 0x9bc, 0x9c1, 0x995, 0x9cb, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x63, 0x72, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x2019, 0x41, 0x63, 0x72, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x53, 0x75, 0x64, 0xe1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019,
0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f,
0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0xc1, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x74,
0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x65, 0x73, 0x20, 0x6e, 0x6f, 0x72,
0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x6d, 0x6f, 0x6e,
0x74, 0x61, 0xf1, 0x65, 0x73, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1,
0x65, 0x73, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x75, 0x20,
0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed,
0x66, 0x69, 0x63, 0x75, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x6e, 0x6f,
0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x75,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019,
0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0x61, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41,
0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74,
0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72,
0x61, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x61, 0x62,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x78, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x78, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x78,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019,
0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019,
0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019,
0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74,
0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c,
0xe1, 0x6e, 0x74, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65,
0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73,
0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73,
0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x78, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c,
0x65, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x78, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x78, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x78, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x42, 0x75, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x61, 0x62, 0x75, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x75, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x75, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74,
0x6d, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f,
0x63, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c,
0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x6d, 0x65, 0x64,
0x69, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x61, 0x76, 0x69,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73,
0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45,
0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x72, 0x65, 0x6d, 0x75, 0x20,
0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x78,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x78, 0x69, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x78, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4c, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x73, 0x75, 0x72, 0x20, 0x79, 0x20,
0x6c, 0x2019, 0x61, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69, 0x63, 0x75, 0x20,
0x66, 0x72, 0x61, 0x6e, 0x63, 0xe9, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x58,
0x65, 0x6f, 0x72, 0x78, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x58, 0x65,
0x6f, 0x72, 0x78, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f,
0x72, 0x78, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x6d, 0x65,
0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x47, 0x6f, 0x6c, 0x66, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x1e24,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x1e24, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x1e24, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4f, 0x63, 0xe9, 0x61, 0x6e, 0x75, 0x20, 0xcd,
0x6e, 0x64, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x72, 0xe1, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x2019, 0x49, 0x72, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x49,
0x72, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x58, 0x61, 0x70, 0xf3,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x58, 0x61, 0x70, 0xf3, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x58, 0x61, 0x70, 0xf3, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0xe1, 0x6e,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x69,
0x72, 0x67, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c,
0x6c, 0x65, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x63, 0xe1, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x63, 0xe1, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63,
0xe1, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x65,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78,
0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78,
0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78,
0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x55, 0x6c,
0xe1, 0x6e, 0x20, 0x42, 0xe1, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019,
0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0xe1, 0x74, 0x6f, 0x72, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x2019, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0xe1, 0x74, 0x6f, 0x72,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xfa, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xfa, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72,
0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x75, 0x65, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61,
0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe1,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x70, 0xfa, 0x61, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0xe1, 0x69, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0xe1, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0xe1,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65,
0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xfa,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x75, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65,
0x73, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x79, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0xf3, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65,
0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x78, 0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x78, 0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x78, 0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x65,
0x73, 0x20, 0x53, 0x65, 0x69, 0x78, 0x65, 0x6c, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49,
0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xf3,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f,
0x72, 0x78, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x79, 0x6f, 0x77, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x54, 0x61, 0x78, 0x69, 0x71, 0x75, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0xe1, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0xe1, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe1, 0x69, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x55, 0x7a,
0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0xe1, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49,
0x73, 0x6c, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20,
0x79, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x75, 0x18f,
0x66, 0x71, 0x61, 0x6e, 0x131, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e,
0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x43, 0x259, 0x6e, 0x75, 0x62, 0x69, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72,
0x62, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x6c, 0x79, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x6c, 0x79, 0x61,
0x73, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x6c, 0x79, 0x61, 0x73, 0x6b,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x4d, 0x259, 0x72, 0x6b, 0x259,
0x7a, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x4d,
0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20,
0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x15e, 0x259, 0x72, 0x71,
0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x15e, 0x259,
0x72, 0x71, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x15e, 0x259, 0x72,
0x71, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61,
0x6c, 0x69, 0x20, 0x44, 0x61, 0x11f, 0x6c, 0x131, 0x71, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e,
0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x44, 0x61, 0x11f, 0x6c, 0x131, 0x71,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e,
0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x44, 0x61, 0x11f, 0x6c, 0x131, 0x71,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x61, 0x6b,
0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f,
0x6b, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c,
0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x61,
0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x70, 0x69, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x70, 0x69, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x41, 0x70, 0x69, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x18f, 0x72, 0x259, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x18f, 0x72, 0x259, 0x62, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x18f, 0x72, 0x259, 0x62, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259,
0x72, 0x62, 0x69, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x45, 0x72, 0x6d, 0x259, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x45, 0x72, 0x6d, 0x259, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x45, 0x72, 0x6d, 0x259, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20,
0x56, 0x61, 0x78, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b,
0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b,
0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69,
0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72,
0x6b, 0x259, 0x7a, 0x69, 0x20, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41,
0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x51,
0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a,
0x69, 0x20, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x76,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72,
0x71, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41,
0x7a, 0x259, 0x72, 0x62, 0x61, 0x79, 0x63, 0x61, 0x6e, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x41, 0x7a, 0x259, 0x72, 0x62, 0x61, 0x79, 0x63, 0x61,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x41, 0x7a, 0x259, 0x72, 0x62, 0x61, 0x79, 0x63,
0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x41, 0x7a, 0x6f, 0x72, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x7a,
0x6f, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x61, 0x6e, 0x71, 0x6c,
0x61, 0x64, 0x65, 0x15f, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x61,
0x6e, 0x71, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x61,
0x6e, 0x71, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x79, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20, 0x76, 0x61,
0x78, 0x74, 0x131, 0x4b, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x61, 0x70, 0x65,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0xc7, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0xc7, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x69, 0x6c, 0x69, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0xc7, 0x69, 0x6c, 0x69, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7,
0x69, 0x6c, 0x69, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0xc7, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x69,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0xc7, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x69, 0x6c, 0x61, 0x64, 0x20, 0x41,
0x64, 0x61, 0x73, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f,
0x6b, 0x6f, 0x73, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4b, 0x75, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x75, 0x6b, 0x20,
0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x75,
0x6b, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x59, 0x61,
0x72, 0x131, 0x6d, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b,
0x75, 0x62, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x44, 0x65, 0x76, 0x69,
0x73, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x44, 0xfc, 0x6d, 0x6f, 0x6e,
0x2d, 0x64, 0x2019, 0xdc, 0x72, 0x76, 0x69, 0x6c, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x54, 0x69, 0x6d, 0x6f,
0x72, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x73, 0x78, 0x61,
0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73,
0x131, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x45,
0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b,
0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69,
0x20, 0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259,
0x72, 0x71, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x259, 0x6e, 0x61,
0x72, 0x20, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62,
0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41,
0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20,
0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x41, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x6f, 0x6c, 0x6b,
0x6c, 0x65, 0x6e, 0x64, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x69,
0x63, 0x69, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x69, 0x63, 0x69,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x46, 0x69, 0x63, 0x69, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x131, 0x7a,
0x20, 0x51, 0x76, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x131, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x131, 0x7a, 0x20, 0x43,
0x259, 0x6e, 0x75, 0x62, 0x69, 0x20, 0x76, 0x259, 0x20, 0x41, 0x6e, 0x74,
0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x51, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x71, 0x6f, 0x73, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x51, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x47, 0x75, 0x72, 0x63, 0xfc, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x47, 0x75, 0x72, 0x63, 0xfc,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x47, 0x75, 0x72, 0x63, 0xfc,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x41, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51,
0x72, 0x69, 0x6e, 0x76, 0x69, 0xe7, 0x20, 0x4f, 0x72, 0x74, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x51,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x51, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x51, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x51, 0x72, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x51, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259,
0x72, 0x62, 0x69, 0x20, 0x51, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4b, 0xf6, 0x72, 0x66, 0x259, 0x7a, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x51, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x48, 0x61, 0x76, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x61, 0x76, 0x61, 0x79, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x61, 0x76, 0x61,
0x79, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x6f, 0x6e, 0x71, 0x20, 0x4b, 0x6f,
0x6e, 0x71, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x6f, 0x6e, 0x71,
0x20, 0x4b, 0x6f, 0x6e, 0x71, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x6f, 0x6e, 0x71,
0x20, 0x4b, 0x6f, 0x6e, 0x71, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x6f, 0x76, 0x64,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x69,
0x6e, 0x64, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x48, 0x69, 0x6e, 0x64, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e, 0x131,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x69, 0x6e, 0x64, 0xe7, 0x69,
0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259,
0x7a, 0x69, 0x20, 0x130, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79,
0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69,
0x20, 0x130, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x130,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x130, 0x72, 0x61, 0x6e,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x130, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x130,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x130, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x130,
0x73, 0x72, 0x61, 0x69, 0x6c, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x59, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x79,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x59, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x79, 0x61,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x61,
0x7a, 0x61, 0x78, 0x131, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x78,
0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x51, 0x61, 0x7a, 0x61,
0x78, 0x131, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x51, 0x61, 0x7a, 0x61, 0x78, 0x131,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f,
0x72, 0x65, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f,
0x72, 0x65, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f, 0x72, 0x65, 0x79,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b,
0x6f, 0x72, 0x73, 0x65, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x51, 0x131, 0x72, 0x11f, 0x131, 0x7a, 0x131, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4c, 0x61, 0x79, 0x6e, 0x20,
0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x61, 0x75, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x61, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x61, 0x75, 0x20, 0x59,
0x61, 0x79, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x71, 0x61,
0x64, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x71,
0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x71, 0x61, 0x64,
0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x4d, 0x61, 0x6c, 0x61, 0x79, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4d, 0x61, 0x72, 0x6b, 0x65, 0x73, 0x61, 0x73, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x72, 0x15f, 0x61, 0x6c, 0x20,
0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x20, 0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x20, 0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20,
0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x55, 0x6c, 0x61,
0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x59,
0x61, 0x79, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x61, 0x75, 0x72,
0x75, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6e,
0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65,
0x6e, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x5a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59,
0x65, 0x6e, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e,
0x79, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x79, 0x75, 0x66, 0x61, 0x75, 0x6e,
0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x79, 0x75, 0x66,
0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x41,
0x64, 0x61, 0x73, 0x131, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4f, 0x6d,
0x73, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4f, 0x6d, 0x73, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x51, 0x76,
0x69, 0x6e, 0x65, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50,
0x61, 0x72, 0x61, 0x71, 0x76, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x50, 0x61, 0x72, 0x61, 0x71, 0x76, 0x61, 0x79, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x50, 0x61, 0x72, 0x61, 0x71, 0x76, 0x61, 0x79, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x65, 0x72, 0x75, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x50, 0x65, 0x72, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50,
0x65, 0x72, 0x75, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x65, 0x6e, 0x69,
0x6b, 0x73, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4d, 0xfc, 0x71, 0x259, 0x64, 0x64, 0x259, 0x73,
0x20, 0x50, 0x79, 0x65, 0x72, 0x20, 0x76, 0x259, 0x20, 0x4d, 0x69, 0x6b,
0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0xfc,
0x71, 0x259, 0x64, 0x64, 0x259, 0x73, 0x20, 0x50, 0x79, 0x65, 0x72, 0x20,
0x76, 0x259, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4d, 0xfc, 0x71, 0x259, 0x64, 0x64, 0x259, 0x73, 0x20, 0x50, 0x79,
0x65, 0x72, 0x20, 0x76, 0x259, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f,
0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50,
0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x50, 0x78, 0x65, 0x6e, 0x79, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x52, 0x65, 0x79, 0x75, 0x6e, 0x79, 0x6f, 0x6e, 0x52, 0x6f, 0x74,
0x65, 0x72, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x78,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x61,
0x78, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x78, 0x61,
0x6c, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x76, 0x61, 0x78, 0x74,
0x131, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x20, 0x79, 0x61, 0x79, 0x20, 0x76, 0x61, 0x78,
0x74, 0x131, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x53, 0x65, 0x79, 0x15f, 0x65, 0x6c, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x69, 0x6e, 0x71,
0x61, 0x70, 0x75, 0x72, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x6f,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72,
0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x43, 0x259, 0x6e, 0x75, 0x62,
0x69, 0x20, 0x43, 0x6f, 0x72, 0x63, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x53, 0x79, 0x6f, 0x76, 0x61, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x54, 0x61, 0x79, 0x62, 0x65, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x54, 0x61, 0x79, 0x62, 0x65, 0x79, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x54, 0x61, 0x79, 0x62, 0x65, 0x79, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x54, 0x61, 0x63, 0x69, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0x6f, 0x6e,
0x71, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0x6f, 0x6e, 0x71,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x54, 0x6f, 0x6e, 0x71, 0x61, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x75, 0x75, 0x6b, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x259, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54,
0xfc, 0x72, 0x6b, 0x6d, 0x259, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x259, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x55, 0x72, 0x75, 0x71, 0x76, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x55, 0x72, 0x75, 0x71, 0x76, 0x61, 0x79, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x55,
0x72, 0x75, 0x71, 0x76, 0x61, 0x79, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0xd6, 0x7a, 0x62, 0x259, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xd6, 0x7a, 0x62, 0x259,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xd6, 0x7a, 0x62,
0x259, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x56, 0x61, 0x75, 0x6e, 0x61, 0x74, 0x75, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x65, 0x6e,
0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x56, 0x6f, 0x6c, 0x71, 0x6f, 0x71, 0x72, 0x61,
0x64, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6f, 0x6c, 0x71, 0x6f,
0x71, 0x72, 0x61, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6f, 0x6c, 0x71, 0x6f,
0x71, 0x72, 0x61, 0x64, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x55, 0x65, 0x79, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x55, 0x6f, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x76, 0x259, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x71, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x71, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x71, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x98f,
0x995, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x995, 0x9b0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x995, 0x9b0, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9ab, 0x997, 0x9be, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20,
0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995,
0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3,
0x20, 0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae,
0x20, 0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986, 0x9ab, 0x9cd, 0x9b0,
0x9bf, 0x995, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986, 0x9ab, 0x9cd, 0x9b0,
0x9bf, 0x995, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9be,
0x9b8, 0x9cd, 0x995, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9be,
0x9b8, 0x9cd, 0x995, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x9b2, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x20, 0x9a6, 0x9bf,
0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2,
0x9cd, 0x9ae, 0x9be, 0x99f, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2,
0x9cd, 0x9ae, 0x9be, 0x99f, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9cd, 0x9ae, 0x9be, 0x99f, 0x9bf, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0,
0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x9be, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x9be,
0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x9be, 0x99e, 0x9cd,
0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b0, 0x9cd, 0x9ac, 0x9a4, 0x9cd,
0x9af, 0x20, 0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b0, 0x9cd, 0x9ac, 0x9a4, 0x9cd, 0x9af, 0x20, 0x985,
0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b0, 0x9cd, 0x9ac, 0x9a4, 0x9cd, 0x9af,
0x20, 0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9a6, 0x9bf, 0x9a8,
0x9c7, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9b0, 0x9b6, 0x9be,
0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9b0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20,
0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x985,
0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9b0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4,
0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20,
0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9a6, 0x9bf, 0x9a8, 0x9c7,
0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a8, 0x9a6, 0x9cd, 0x9af, 0x9cd,
0x9b0, 0x9cd, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a8, 0x9a6, 0x9cd, 0x9af,
0x9cd, 0x9b0, 0x9cd, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x985, 0x9a8, 0x9a6, 0x9cd, 0x9af, 0x9cd, 0x9b0, 0x9cd, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x985, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x985, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x20, 0x9a6, 0x9bf, 0x9a8, 0x9c7, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x995, 0x99f, 0x9be, 0x993, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x995, 0x99f,
0x9be, 0x993, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x995, 0x99f, 0x9be, 0x993, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x995, 0x99f,
0x9cb, 0x9ac, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x995, 0x99f, 0x9cb, 0x9ac,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x995,
0x99f, 0x9cb, 0x9ac, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9ac, 0x9bf, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9ac, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9ac, 0x9bf, 0x20, 0x9a6,
0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9bf,
0x9a8, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf,
0x20, 0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf,
0x20, 0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9b0,
0x20, 0x9aa, 0x9cd, 0x9b0, 0x9ae, 0x9be, 0x9a3, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x986, 0x9b0, 0x9cd, 0x99c,
0x9c7, 0x9a8, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9b0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x9b0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x985, 0x9a4, 0x9b2, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x9bf, 0x995,
0x9c7, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a4, 0x9b2, 0x9be, 0x9a8,
0x9cd, 0x9a4, 0x9bf, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x985, 0x9a4, 0x9b2, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x9bf, 0x995, 0x20,
0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x985,
0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0,
0x9af, 0x9bc, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985,
0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x995,
0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a6, 0x9bf,
0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8,
0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x995, 0x9c7,
0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9b6, 0x9cd,
0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd,
0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x995, 0x9c7, 0x9a8,
0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9b6, 0x9cd, 0x99a,
0x9bf, 0x9ae, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc,
0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be,
0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd,
0x9ac, 0x20, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0,
0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd,
0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9c2, 0x9b0,
0x9cd, 0x9ac, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x985,
0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f,
0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9b6, 0x9cd, 0x99a,
0x9bf, 0x9ae, 0x9bf, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9be, 0x9b0, 0x9ac, 0x9be, 0x987, 0x99c,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9be, 0x9b0, 0x9ac,
0x9be, 0x987, 0x99c, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9be, 0x9b0, 0x9ac, 0x9be, 0x987, 0x99c, 0x9be,
0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x99c, 0x9cb, 0x9b0, 0x9c7,
0x9b8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x99c, 0x9cb, 0x9b0, 0x9c7, 0x9b8,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x99c,
0x9cb, 0x9b0, 0x9c7, 0x9b8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be,
0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac,
0x9be, 0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7,
0x9b6, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9c1, 0x99f, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cb, 0x9b2, 0x9bf, 0x9ad, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x9b8,
0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac,
0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9b0, 0x9be,
0x9b8, 0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ac, 0x9cd, 0x9b0, 0x9c1, 0x9a8, 0x9c7, 0x987, 0x20, 0x9a6, 0x9be, 0x9b0,
0x9c1, 0x9b8, 0x9b8, 0x9be, 0x9b2, 0x9be, 0x9ae, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9b0, 0x9cd, 0x9a6, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9aa,
0x20, 0x9ad, 0x9be, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99a, 0x9be, 0x9ae, 0x9c7, 0x9b0, 0x9cb, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9cd, 0x9af, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9cd, 0x9af, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9cd, 0x9af,
0x9be, 0x9a5, 0x9be, 0x9ae, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99a, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c0,
0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a,
0x9c0, 0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9b0, 0x9bf, 0x9b8, 0x9ae, 0x9be, 0x9b8, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb,
0x995, 0x9cb, 0x9b8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e,
0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9b2, 0x9cb, 0x9ae,
0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9cb, 0x9b2, 0x9cb, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9b2, 0x9cb,
0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1,
0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c1, 0x995, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c1, 0x995, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x985,
0x9b0, 0x9cd, 0x9a7, 0x9c7, 0x995, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9bf, 0x989, 0x9ac, 0x9be, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf,
0x989, 0x9ac, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2,
0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a1, 0x9c7, 0x9ad, 0x9bf, 0x9b8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a1, 0x9c1, 0x9ae, 0x9a8, 0x9cd, 0x99f, 0x2d,
0x9a6, 0x9cd, 0x9af, 0x2019, 0x989, 0x9b0, 0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x99f, 0x9bf, 0x9ae,
0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9b0,
0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987,
0x9b8, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b8, 0x9cd,
0x99f, 0x9be, 0x9b0, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x987, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x9a1, 0x9b0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9b0,
0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7,
0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd,
0x9af, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x987, 0x989,
0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af,
0x9bc, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9b0, 0x9be, 0x9a8,
0x9cd, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20,
0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x987, 0x989, 0x9b0, 0x9cb,
0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd,
0x99a, 0x9bf, 0x9ae, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6,
0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af,
0x9bc, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995, 0x9b2, 0x9cd, 0x9af,
0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1,
0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995, 0x9b2, 0x9cd,
0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa,
0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ab, 0x995, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ab, 0x9b0, 0x9be, 0x9b8, 0x9bf, 0x20, 0x997, 0x9be, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9b0, 0x9be, 0x9b8, 0x9bf,
0x20, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x98f, 0x9ac, 0x982, 0x20,
0x986, 0x9a8, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x9cd, 0x995, 0x99f, 0x9bf, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9be, 0x9b2, 0x9be, 0x9aa, 0x9be, 0x997, 0x9cb,
0x9b8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9cd, 0x9af, 0x9be, 0x9ae, 0x9cd,
0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c,
0x9b0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99c, 0x9b0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9b0, 0x9cd, 0x99c, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9bf, 0x9b2, 0x9ac,
0x9be, 0x9b0, 0x9cd, 0x99f, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1,
0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9a8, 0x9bf, 0x99a, 0x20, 0x9ae, 0x9bf, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae,
0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2,
0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2, 0x9cd,
0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a,
0x9bf, 0x9ae, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be,
0x9a8, 0x9cd, 0x9a1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9c1, 0x9af,
0x9bc, 0x9be, 0x9ae, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x989, 0x9aa, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc,
0x9be, 0x987, 0x2d, 0x986, 0x9b2, 0x9c7, 0x989, 0x9a4, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9be, 0x987, 0x2d, 0x986, 0x9b2, 0x9c7,
0x989, 0x9a4, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9be, 0x987, 0x2d, 0x986, 0x9b2, 0x9c7, 0x989,
0x9a4, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b9, 0x982, 0x20, 0x995, 0x982, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9b9, 0x982, 0x20, 0x995, 0x982, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982, 0x20, 0x995, 0x982, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9,
0x9cb, 0x9ad, 0x9a1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9a1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad,
0x9be, 0x9b0, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9b0, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be,
0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x99a, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20,
0x987, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x9a8, 0x9c7, 0x9b6, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x987,
0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x9a8, 0x9c7, 0x9b6, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9c0, 0x20,
0x987, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x9a8, 0x9c7, 0x9b6, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x987, 0x9b0, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0, 0x9be, 0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac,
0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0, 0x995,
0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0,
0x995, 0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0, 0x995, 0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c, 0x9b0, 0x9be, 0x9af, 0x9bc,
0x9c7, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c, 0x9b0, 0x9be, 0x9af,
0x9bc, 0x9c7, 0x9b2, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x987, 0x99c, 0x9b0, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9b2, 0x20, 0x9a6, 0x9bf,
0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be,
0x9aa, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be,
0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c,
0x9be, 0x9aa, 0x9be, 0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9aa,
0x9cd, 0x9af, 0x9be, 0x9ad, 0x9b2, 0x9b8, 0x9cd, 0x995, 0x2d, 0x20, 0x995, 0x9cd,
0x9af, 0x9be, 0x9ae, 0x99a, 0x9cd, 0x9af, 0x9be, 0x99f, 0x9b8, 0x9cd, 0x995, 0x9bf,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9aa,
0x9cd, 0x9af, 0x9be, 0x9ad, 0x9b2, 0x9b8, 0x9cd, 0x995, 0x2d, 0x20, 0x995, 0x9cd,
0x9af, 0x9be, 0x9ae, 0x99a, 0x9cd, 0x9af, 0x9be, 0x99f, 0x9b8, 0x9cd, 0x995, 0x9bf,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x9a4,
0x9cd, 0x9b0, 0x9c7, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9ad, 0x9b2, 0x9b8, 0x9cd, 0x995,
0x2d, 0x20, 0x995, 0x9cd, 0x9af, 0x9be, 0x9ae, 0x99a, 0x9cd, 0x9af, 0x9be, 0x99f,
0x9b8, 0x9cd, 0x995, 0x9bf, 0x20, 0x997, 0x9c3, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9be, 0x99c, 0x9be,
0x996, 0x9be, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x995, 0x9be, 0x99c, 0x9be, 0x996, 0x9be,
0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6,
0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x995, 0x9be, 0x99c, 0x9be, 0x996, 0x9be, 0x9b8,
0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9b0,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb,
0x9b0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9b0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8,
0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9cb, 0x9b8, 0x9b0, 0x9c7, 0x987, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd,
0x9b8, 0x9cd, 0x995, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9b0,
0x9be, 0x9b8, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995,
0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd, 0x9b8,
0x9cd, 0x995, 0x9bf, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x9b0,
0x997, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9b2, 0x999, 0x9cd, 0x995, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9be,
0x987, 0x9a8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd,
0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9b0, 0x9cd, 0x9a1, 0x20, 0x9b9,
0x9be, 0x993, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9b0,
0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9ae, 0x9b8, 0x9af, 0x9bc, 0x9b2, 0x9b0, 0x9cd, 0x9a1, 0x20,
0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2,
0x9cb, 0x995, 0x20, 0x9ae, 0x9b8, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x995, 0x9be, 0x993,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x995, 0x9be, 0x993, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x995, 0x9be, 0x993,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x997, 0x9be, 0x9a1, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x997, 0x9be,
0x9a1, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x997, 0x9be, 0x9a1, 0x9be, 0x9a8, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9b2, 0x9af, 0x9bc, 0x9c7, 0x9b6, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9b2, 0x9a6,
0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9b0,
0x9cd, 0x995, 0x9c7, 0x9b8, 0x9be, 0x9b8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9be, 0x9b0, 0x9cd, 0x9b6, 0x9be, 0x9b2, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b0,
0x9bf, 0x9b6, 0x9be, 0x9b8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b0, 0x9bf,
0x9b6, 0x9be, 0x9b8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9b0, 0x9bf, 0x9b6, 0x9be, 0x9b8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9b8, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9c7, 0x995,
0x9cd, 0x9b8, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9cd, 0x9b0, 0x9b6, 0x9be,
0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8,
0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9cd, 0x9b0, 0x9b6, 0x9be, 0x9a8, 0x9cd,
0x9a4, 0x20, 0x9ae, 0x9b9, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9c7, 0x995,
0x9cd, 0x9b8, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9cd, 0x9b0, 0x9b6, 0x9be,
0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x20, 0x9ac, 0x9be, 0x9a4, 0x9cb,
0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x20, 0x9ac,
0x9be, 0x9a4, 0x9cb, 0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x20, 0x9ac, 0x9be, 0x9a4, 0x9cb, 0x9b0,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9cb, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9cb, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9cb,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9af, 0x9bc, 0x9be, 0x9a8,
0x9ae, 0x9be, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9be, 0x989, 0x9b0,
0x9c1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9c7, 0x9aa, 0x9be, 0x9b2, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20, 0x995, 0x9cd, 0x9af, 0x9be,
0x9b2, 0x9c7, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20, 0x995, 0x9cd, 0x9af, 0x9be, 0x9b2, 0x9c7,
0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20, 0x995, 0x9cd, 0x9af,
0x9be, 0x9b2, 0x9c7, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf, 0x9b2, 0x9cd, 0x9af,
0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989,
0x99c, 0x9bf, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf,
0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be,
0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x9ab,
0x9be, 0x989, 0x9a8, 0x9cd, 0x9a1, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x9ab, 0x9be, 0x989, 0x9a8,
0x9cd, 0x9a1, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x9ab, 0x9be,
0x989, 0x9a8, 0x9cd, 0x9a1, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20,
0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a8, 0x9bf, 0x989, 0x987, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9b0, 0x9ab,
0x9cb, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9a8, 0x9b0, 0x9ab, 0x9cb, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9b0,
0x9ab, 0x9cb, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9a6, 0x9bf,
0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be,
0x9b0, 0x9cd, 0x9a8, 0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x20, 0x9a1, 0x9bf, 0x20,
0x9a8, 0x9cb, 0x9b0, 0x9cb, 0x9a8, 0x9b9, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ab, 0x9be, 0x9b0, 0x9cd, 0x9a8, 0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x20, 0x9a1,
0x9bf, 0x20, 0x9a8, 0x9cb, 0x9b0, 0x9cb, 0x9a8, 0x9b9, 0x9be, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be, 0x9b0, 0x9cd, 0x9a8,
0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x20, 0x9a1, 0x9bf, 0x20, 0x9a8, 0x9cb, 0x9b0,
0x9cb, 0x9a8, 0x9b9, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4,
0x9cd, 0x9a4, 0x9b0, 0x20, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x9a8, 0x20, 0x9a6, 0x9cd,
0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x9b8, 0x9bf, 0x9ac, 0x9bf, 0x9b0, 0x9cd, 0x9b8,
0x9cd, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x9b8,
0x9bf, 0x9ac, 0x9bf, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x9b8, 0x9bf,
0x9ac, 0x9bf, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993,
0x9ae, 0x9b8, 0x9cd, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be,
0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x989, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9be, 0x9aa, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x20, 0x9a8, 0x9bf, 0x989, 0x20, 0x997,
0x9bf, 0x9a8, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9af, 0x9be,
0x9b0, 0x9be, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9af,
0x9be, 0x9b0, 0x9be, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9b0, 0x9c1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c7, 0x9b0, 0x9c1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9c7, 0x9b0, 0x9c1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab,
0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be,
0x987, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9cb, 0x9a8, 0x9bf,
0x995, 0x9cd, 0x9b8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e,
0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f,
0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x20, 0x993, 0x20, 0x9ae, 0x9bf,
0x995, 0x9c7, 0x9b2, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9c7, 0x9a8,
0x9cd, 0x99f, 0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x20, 0x993, 0x20,
0x9ae, 0x9bf, 0x995, 0x9c7, 0x9b2, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x20, 0x9aa, 0x9bf,
0x9af, 0x9bc, 0x9c7, 0x9b0, 0x20, 0x993, 0x20, 0x9ae, 0x9bf, 0x995, 0x9c7, 0x9b2,
0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9bf, 0x99f, 0x995, 0x9c7, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cb, 0x9a8, 0x9be, 0x9aa, 0x9c7,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x982, 0x987, 0x9af,
0x9bc, 0x9be, 0x982, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x99c, 0x9bf,
0x9b2, 0x9cb, 0x9b0, 0x9cd, 0x9a1, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9bf, 0x99c, 0x9bf, 0x9b2, 0x9cb, 0x9b0, 0x9cd, 0x9a1, 0x9be, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x99c, 0x9bf, 0x9b2, 0x9cb,
0x9b0, 0x9cd, 0x9a1, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b0, 0x9bf, 0x987, 0x989,
0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b0, 0x9a5,
0x9c7, 0x9b0, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x996, 0x9be,
0x9b2, 0x9bf, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x996, 0x9be,
0x9b2, 0x9bf, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b8, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9be, 0x20, 0x997,
0x9c3, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9cb, 0x9af,
0x9bc, 0x9be, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9c7, 0x9b6, 0x9c7, 0x9b2, 0x9b8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b8, 0x9bf, 0x999, 0x9cd, 0x997, 0x9be, 0x9aa, 0x9c1, 0x9b0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9b2, 0x9cb,
0x9ae, 0x9a8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd,
0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3,
0x20, 0x99c, 0x9b0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b8, 0x9c1, 0x9b0, 0x9bf, 0x9a8, 0x9be, 0x9ae, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b8, 0x9be, 0x9af, 0x9bc, 0x9cb, 0x993, 0x9af, 0x9bc, 0x9be, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x9b9, 0x9bf, 0x9a4, 0x9bf, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9a4, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987,
0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9a4, 0x9be, 0x99c, 0x9be, 0x996, 0x9be, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9cb, 0x995, 0x9c7, 0x9b2, 0x9be, 0x989,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9cb, 0x999, 0x9cd, 0x997, 0x9be, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9cb, 0x999, 0x9cd, 0x997, 0x9be, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9cb, 0x999, 0x9cd,
0x997, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c1, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9b0, 0x9cd, 0x995, 0x9ae, 0x9c7, 0x9a8,
0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4,
0x9c1, 0x9b0, 0x9cd, 0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be,
0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4,
0x9c1, 0x9b0, 0x9cd, 0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be,
0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9c1, 0x9ad, 0x9be, 0x9b2,
0x9c1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b0, 0x9c1, 0x997, 0x9c1, 0x9af,
0x9bc, 0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b0, 0x9c1, 0x997, 0x9c1,
0x9af, 0x9bc, 0x9c7, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x989, 0x9b0, 0x9c1, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995,
0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995, 0x9bf, 0x9b8, 0x9cd,
0x9a4, 0x9be, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9a8,
0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a4, 0x9c1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad,
0x9be, 0x9a8, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a4, 0x9c1, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9a8, 0x9c1, 0x9af, 0x9bc,
0x9be, 0x9a4, 0x9c1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9c7, 0x9a8,
0x9c7, 0x99c, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x9b2, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a6, 0x9bf, 0x9ad, 0x9b8, 0x9cd, 0x9a4, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a6, 0x9bf, 0x9ad,
0x9b8, 0x9cd, 0x9a4, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a6, 0x9bf, 0x9ad, 0x9b8, 0x9cd, 0x9a4,
0x995, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9b2, 0x997, 0x9cb, 0x997,
0x9cd, 0x9b0, 0x9be, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9b2, 0x997,
0x9cb, 0x997, 0x9cd, 0x9b0, 0x9be, 0x9a1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9b2, 0x997, 0x9cb, 0x997, 0x9cd, 0x9b0, 0x9be,
0x9a1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9b8, 0x99f, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9af, 0x9bc, 0x9c7, 0x995, 0x20, 0x9a6, 0x9cd,
0x9ac, 0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9af, 0x9bc, 0x9be,
0x9b2, 0x9bf, 0x9b8, 0x20, 0x98f, 0x9ac, 0x982, 0x20, 0x9ab, 0x9c1, 0x99f, 0x9c1,
0x9a8, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9af, 0x9bc, 0x9be, 0x995,
0x9c1, 0x99f, 0x9be, 0x9b8, 0x9cd, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987,
0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x9b8, 0x9cd, 0x995, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9af, 0x9bc, 0x9be,
0x995, 0x9c1, 0x99f, 0x9be, 0x9b8, 0x9cd, 0x995, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x987, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x9a4, 0x9c7, 0x9b0, 0x9bf, 0x9a8,
0x9ac, 0x9c1, 0x9b0, 0x9cd, 0x997, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9af,
0x9bc, 0x9c7, 0x995, 0x9be, 0x9a4, 0x9c7, 0x9b0, 0x9bf, 0x9a8, 0x9ac, 0x9c1, 0x9b0,
0x9cd, 0x997, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x9a4, 0x9c7, 0x9b0, 0x9bf, 0x9a8, 0x9ac,
0x9c1, 0x9b0, 0x9cd, 0x997, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x989,
0x995, 0x9cb, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x41, 0x63, 0x72, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x63, 0x72, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x63, 0x72, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6b, 0x6f, 0x20, 0x68, 0x65, 0x67, 0x6f, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64,
0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e,
0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61,
0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20,
0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x70, 0x61,
0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20,
0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70,
0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f,
0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x70, 0x61, 0x72,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65,
0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6d,
0x65, 0x6e, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x69,
0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x70,
0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f,
0x20, 0x6d, 0x65, 0x6e, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x6f, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x50, 0x61,
0x7a, 0x69, 0x66, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x6f, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x72,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x70, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x70, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x41, 0x70, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6b, 0x74,
0x61, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6b,
0x74, 0x61, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x6b, 0x74,
0x61, 0x75, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6d, 0x65, 0x6e,
0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6d,
0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6b, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x6f,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69,
0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72,
0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69,
0x2d, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x2d,
0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x2d, 0x6d, 0x65, 0x6e,
0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69,
0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65,
0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64,
0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x65,
0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x65, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x42,
0x68, 0x75, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x42, 0x72, 0x75, 0x6e,
0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61,
0x6d, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x61, 0x62,
0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x43, 0x61, 0x62, 0x6f, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x61, 0x73, 0x65, 0x79,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x67, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x67, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54,
0x78, 0x69, 0x6c, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x54, 0x78, 0x69, 0x6c, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x54,
0x78, 0x69, 0x6c, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x78, 0x69, 0x6e, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x78, 0x69, 0x6e, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x54, 0x78, 0x69, 0x6e, 0x61, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x20, 0x75,
0x68, 0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74,
0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x4b, 0x6f, 0x6c, 0x6f, 0x6e, 0x62, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4b, 0x6f, 0x6c, 0x6f, 0x6e, 0x62, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x6f, 0x6c, 0x6f, 0x6e, 0x62,
0x69, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x75, 0x68, 0x61,
0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74,
0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b,
0x75, 0x62, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b,
0x75, 0x62, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x75, 0x62,
0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x44, 0x61, 0x76, 0x69, 0x73, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x72, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x7a, 0x6b, 0x6f, 0x20,
0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x50, 0x61, 0x7a, 0x6b, 0x6f, 0x20, 0x75, 0x68, 0x61, 0x72,
0x74, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x50, 0x61, 0x7a, 0x6b,
0x6f, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x6b,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20,
0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f,
0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20,
0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x65,
0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x20, 0x75, 0x72, 0x72, 0x75, 0x6e,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62,
0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e,
0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65,
0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74,
0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x75, 0x68, 0x61,
0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x75, 0x68, 0x61,
0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46, 0x69, 0x6a, 0x69,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46, 0x69, 0x6a, 0x69,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x46, 0x69, 0x6a, 0x69, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x74,
0x73, 0x65, 0x73, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x46, 0x72, 0x61, 0x6e, 0x74, 0x7a, 0x69, 0x61, 0x72, 0x65, 0x6e, 0x20,
0x6c, 0x75, 0x72, 0x72, 0x61, 0x6c, 0x64, 0x65, 0x20, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x74, 0x61, 0x20, 0x61, 0x6e, 0x74,
0x61, 0x72, 0x74, 0x69, 0x6b, 0x6f, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x74, 0x65, 0x67, 0x69, 0x61, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x72,
0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x75, 0x68, 0x61, 0x72,
0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x6b, 0x6f,
0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x61, 0x72,
0x65, 0x6e, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b,
0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64,
0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65,
0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x47, 0x75, 0x61, 0x6d, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x6f, 0x6c, 0x6b, 0x6f, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x72, 0x20, 0x75,
0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x72, 0x20, 0x75, 0x68, 0x61, 0x72,
0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x72, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x68, 0x6f, 0x76,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x68, 0x6f, 0x76,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x7a, 0x65, 0x61, 0x6e, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x6f, 0x74, 0x78, 0x69, 0x6e, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69,
0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x72, 0x61,
0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x72, 0x61,
0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x72, 0x61, 0x6e, 0x67,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x67,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4a, 0x61, 0x70, 0x6f, 0x6e,
0x69, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x67, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65,
0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x69, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x65,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4c, 0x61, 0x6e,
0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4c, 0x69,
0x6e, 0x65, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x67, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d,
0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x74, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x72, 0x6b,
0x65, 0x73, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x55, 0x68,
0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x7a, 0x69, 0x6f, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x7a, 0x69, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x7a, 0x69, 0x6f, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x65,
0x78, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66,
0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d,
0x65, 0x78, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69,
0x66, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4d, 0x65,
0x78, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66,
0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x72,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x6f, 0x73,
0x6b, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x6f,
0x73, 0x6b, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4d, 0x6f, 0x73,
0x6b, 0x75, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x72,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72,
0x72, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x72,
0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x42, 0x65, 0x72,
0x72, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x5a, 0x65,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x42, 0x65, 0x72, 0x72, 0x69,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x20, 0x42, 0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x65,
0x72, 0x6e, 0x75, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x54, 0x65, 0x72, 0x6e, 0x75, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x54, 0x65, 0x72, 0x6e, 0x75, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x69, 0x75, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65,
0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74,
0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x61, 0x6e, 0x61, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65,
0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x65,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x47, 0x69, 0x6e, 0x65,
0x61, 0x20, 0x42, 0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x69, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x65, 0x72, 0x75, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x65, 0x72, 0x75, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x50, 0x65, 0x72, 0x75, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65,
0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65,
0x74, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x75, 0x6e, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x74, 0x61, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x75, 0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x75, 0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x6f,
0x6e, 0x61, 0x70, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x50, 0x69, 0x6f, 0x6e, 0x67, 0x69, 0x61, 0x6e, 0x67, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x67, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x67, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x55, 0x68, 0x61, 0x72,
0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x48, 0x65, 0x67, 0x6f, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x61, 0x64,
0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x57, 0x61, 0x6b, 0x65,
0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x74,
0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x74, 0x65, 0x67, 0x69, 0x61, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x59, 0x75,
0x6b, 0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x410,
0x444, 0x433, 0x430, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x426, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x430, 0x430, 0x444, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x430,
0x444, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x41f, 0x430, 0x45e, 0x434, 0x43d, 0x451, 0x432, 0x430, 0x430, 0x444, 0x440,
0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x417,
0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x430, 0x444, 0x440, 0x44b, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e,
0x434, 0x43d, 0x435, 0x430, 0x444, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b,
0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x430,
0x444, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x410,
0x43b, 0x44f, 0x441, 0x43a, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x43b, 0x44f, 0x441,
0x43a, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x456, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c,
0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446, 0x44d,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440,
0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446, 0x44d, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e,
0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x44c, 0x43d, 0x44b, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435,
0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x45e, 0x441, 0x445,
0x43e, 0x434, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d,
0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x45e, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x456, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c,
0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x45e, 0x441,
0x445, 0x43e, 0x434, 0x43d, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430,
0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x433,
0x43e, 0x440, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d,
0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x433, 0x43e, 0x440, 0x43d, 0x44b, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440,
0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x433, 0x43e, 0x440, 0x43d,
0x44b, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x426,
0x456, 0x445, 0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x447, 0x430, 0x441, 0x426, 0x456, 0x445, 0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x426, 0x456, 0x445, 0x430, 0x430, 0x43a,
0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x410, 0x43f, 0x456, 0x456,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x20, 0x410, 0x43f, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x43f, 0x456, 0x456, 0x427, 0x430, 0x441,
0x20, 0x421, 0x430, 0x443, 0x434, 0x430, 0x45e, 0x441, 0x43a, 0x430, 0x439, 0x20,
0x410, 0x440, 0x430, 0x432, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x421, 0x430, 0x443,
0x434, 0x430, 0x45e, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x410, 0x440, 0x430, 0x432,
0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x421, 0x430, 0x443, 0x434, 0x430, 0x45e, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x410,
0x440, 0x430, 0x432, 0x456, 0x456, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x446, 0x456,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x446, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x446, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x417,
0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x446, 0x456, 0x43d, 0x44b, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x417, 0x430, 0x445, 0x43e,
0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x446, 0x456,
0x43d, 0x44b, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x446, 0x456, 0x43d, 0x44b, 0x427, 0x430, 0x441, 0x20, 0x410, 0x440,
0x43c, 0x435, 0x43d, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x440, 0x43c, 0x435,
0x43d, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x456, 0x456, 0x410, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x44b, 0x447, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x44b, 0x447, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x410,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b, 0x447, 0x43d, 0x44b, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x446,
0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x439, 0x20, 0x410,
0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x446,
0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x439, 0x20, 0x410,
0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x430, 0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x456, 0x456, 0x426, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x430, 0x2d, 0x437, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x446, 0x44d,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x2d, 0x437, 0x430, 0x445,
0x43e, 0x434, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x45e, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x2d, 0x437,
0x430, 0x445, 0x43e, 0x434, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410,
0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20,
0x443, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x443, 0x441, 0x445,
0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x443, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20, 0x437,
0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x437, 0x430, 0x445, 0x43e,
0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x437, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20, 0x410, 0x437,
0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x20, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d,
0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410,
0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x430, 0x427,
0x430, 0x441, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x456, 0x445, 0x20,
0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410,
0x437, 0x43e, 0x440, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440,
0x430, 0x432, 0x43e, 0x45e, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430,
0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441, 0x20, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x44d, 0x448, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x44d, 0x448, 0x41b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x44d, 0x448, 0x427, 0x430, 0x441, 0x20, 0x411, 0x443, 0x442, 0x430, 0x43d,
0x430, 0x411, 0x430, 0x43b, 0x456, 0x432, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20,
0x447, 0x430, 0x441, 0x411, 0x440, 0x430, 0x437, 0x456, 0x43b, 0x44c, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x411, 0x440, 0x430, 0x437, 0x456, 0x43b, 0x44c,
0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x411, 0x440, 0x430, 0x437, 0x456, 0x43b,
0x44c, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x44f,
0x427, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x431, 0x430, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x44d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b,
0x20, 0x447, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x431, 0x430, 0x2d, 0x412, 0x435,
0x440, 0x434, 0x44d, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x41a, 0x430, 0x431, 0x430, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x44d, 0x427,
0x430, 0x441, 0x20, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x430, 0x427, 0x430, 0x441,
0x20, 0x427, 0x430, 0x442, 0x44d, 0x43c, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x427, 0x430,
0x442, 0x44d, 0x43c, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x427, 0x430, 0x442, 0x44d, 0x43c, 0x430, 0x427, 0x44b, 0x43b, 0x456,
0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x44b, 0x43b, 0x456,
0x439, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x427, 0x44b, 0x43b, 0x456, 0x439,
0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x427, 0x430, 0x441, 0x20, 0x41a, 0x456, 0x442, 0x430, 0x44f, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x20, 0x41a, 0x456, 0x442, 0x430, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x41a, 0x456, 0x442, 0x430, 0x44f, 0x427, 0x430, 0x441,
0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430, 0x20, 0x41a, 0x430,
0x43b, 0x44f, 0x434, 0x427, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x43a, 0x43e, 0x441,
0x430, 0x432, 0x44b, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e,
0x45e, 0x41a, 0x430, 0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x41a, 0x430, 0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439,
0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41a, 0x430, 0x43b, 0x443, 0x43c, 0x431,
0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430,
0x432, 0x43e, 0x45e, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x430,
0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x41a, 0x443, 0x43a, 0x430,
0x41f, 0x430, 0x45e, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x41a, 0x443,
0x43a, 0x430, 0x427, 0x430, 0x441, 0x20, 0x41a, 0x443, 0x431, 0x44b, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x20, 0x41a, 0x443, 0x431, 0x44b, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x41a, 0x443, 0x431, 0x44b, 0x427, 0x430, 0x441, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x414, 0x44d, 0x439, 0x432, 0x456,
0x441, 0x427, 0x430, 0x441, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456,
0x20, 0x414, 0x437, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x414, 0x437, 0x44e, 0x440,
0x432, 0x456, 0x43b, 0x44c, 0x427, 0x430, 0x441, 0x20, 0x423, 0x441, 0x445, 0x43e,
0x434, 0x43d, 0x44f, 0x433, 0x430, 0x20, 0x422, 0x44b, 0x43c, 0x43e, 0x440, 0x430,
0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430,
0x20, 0x412, 0x44f, 0x43b, 0x456, 0x43a, 0x430, 0x434, 0x43d, 0x44f, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430, 0x20, 0x412, 0x44f,
0x43b, 0x456, 0x43a, 0x430, 0x434, 0x43d, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432,
0x430, 0x20, 0x412, 0x44f, 0x43b, 0x456, 0x43a, 0x430, 0x434, 0x43d, 0x44f, 0x42d,
0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x426, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x435,
0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x426, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x435,
0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x426, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x435, 0x45e,
0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d,
0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20,
0x447, 0x430, 0x441, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e,
0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x423,
0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435,
0x439, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x414, 0x430, 0x43b, 0x451, 0x43a, 0x430, 0x45e, 0x441, 0x445, 0x43e,
0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435,
0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440,
0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x417, 0x430,
0x445, 0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439,
0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x427, 0x430, 0x441, 0x20, 0x424, 0x430, 0x43b, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432,
0x43e, 0x45e, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b,
0x20, 0x447, 0x430, 0x441, 0x20, 0x424, 0x430, 0x43b, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432,
0x43e, 0x45e, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x424, 0x430, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x445,
0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441,
0x20, 0x424, 0x456, 0x434, 0x436, 0x44b, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x424, 0x456, 0x434,
0x436, 0x44b, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x424, 0x456, 0x434, 0x436, 0x44b, 0x427, 0x430, 0x441, 0x20, 0x424, 0x440, 0x430,
0x43d, 0x446, 0x443, 0x437, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x413, 0x432, 0x456,
0x44f, 0x43d, 0x44b, 0x427, 0x430, 0x441, 0x20, 0x424, 0x440, 0x430, 0x43d, 0x446,
0x443, 0x437, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x43f, 0x430, 0x45e, 0x434, 0x43d,
0x451, 0x432, 0x430, 0x2d, 0x430, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x44b,
0x447, 0x43d, 0x430, 0x439, 0x20, 0x442, 0x44d, 0x440, 0x44b, 0x442, 0x43e, 0x440,
0x44b, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b,
0x20, 0x447, 0x430, 0x441, 0x20, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433,
0x43e, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432,
0x43e, 0x45e, 0x427, 0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432,
0x43e, 0x45e, 0x20, 0x413, 0x430, 0x43c, 0x431, 0x2019, 0x435, 0x413, 0x440, 0x443,
0x437, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x413, 0x440,
0x443, 0x437, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x413, 0x440,
0x443, 0x437, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x430, 0x441, 0x442,
0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x413, 0x456, 0x43b, 0x431, 0x435, 0x440,
0x442, 0x430, 0x427, 0x430, 0x441, 0x20, 0x43f, 0x430, 0x20, 0x413, 0x440, 0x44b,
0x43d, 0x432, 0x456, 0x447, 0x44b, 0x427, 0x430, 0x441, 0x20, 0x423, 0x441, 0x445,
0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x413, 0x440, 0x44d, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x44b, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x423, 0x441, 0x445, 0x43e, 0x434,
0x43d, 0x44f, 0x439, 0x20, 0x413, 0x440, 0x44d, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x44b, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x413, 0x440, 0x44d,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456, 0x427, 0x430, 0x441, 0x20, 0x417,
0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x413, 0x440, 0x44d, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x417, 0x430, 0x445,
0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x413, 0x440, 0x44d, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x44b, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x413,
0x440, 0x44d, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456, 0x427, 0x430, 0x441,
0x20, 0x41f, 0x435, 0x440, 0x441, 0x456, 0x434, 0x441, 0x43a, 0x430, 0x433, 0x430,
0x20, 0x437, 0x430, 0x43b, 0x456, 0x432, 0x430, 0x427, 0x430, 0x441, 0x20, 0x413,
0x430, 0x44f, 0x43d, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x430,
0x2d, 0x410, 0x43b, 0x435, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441,
0x413, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x430, 0x2d, 0x410, 0x43b, 0x435,
0x443, 0x446, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x413, 0x430, 0x432, 0x430, 0x439,
0x441, 0x43a, 0x430, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x446, 0x43a, 0x456, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441,
0x20, 0x413, 0x430, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x430, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x413, 0x430, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x430, 0x41b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x413, 0x430, 0x43d, 0x43a, 0x43e, 0x43d,
0x433, 0x430, 0x427, 0x430, 0x441, 0x20, 0x425, 0x43e, 0x45e, 0x434, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x425, 0x43e, 0x45e, 0x434, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x425, 0x43e, 0x45e, 0x434, 0x430, 0x427, 0x430,
0x441, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x456, 0x427, 0x430, 0x441, 0x20, 0x406,
0x43d, 0x434, 0x44b, 0x439, 0x441, 0x43a, 0x430, 0x433, 0x430, 0x20, 0x430, 0x43a,
0x456, 0x44f, 0x43d, 0x430, 0x406, 0x43d, 0x434, 0x430, 0x43a, 0x456, 0x442, 0x430,
0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x426, 0x44d, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x456, 0x43d, 0x434, 0x430, 0x43d, 0x435,
0x437, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x423, 0x441,
0x445, 0x43e, 0x434, 0x43d, 0x435, 0x456, 0x43d, 0x434, 0x430, 0x43d, 0x435, 0x437,
0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445,
0x43e, 0x434, 0x43d, 0x435, 0x456, 0x43d, 0x434, 0x430, 0x43d, 0x435, 0x437, 0x456,
0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x406, 0x440, 0x430, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x406, 0x440, 0x430, 0x43d, 0x441,
0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x406, 0x440, 0x430, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x406, 0x440,
0x43a, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x406, 0x440, 0x43a,
0x443, 0x446, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x406, 0x440, 0x43a, 0x443, 0x446,
0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x406, 0x437, 0x440, 0x430, 0x456, 0x43b, 0x44c, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x406, 0x437, 0x440, 0x430, 0x456, 0x43b, 0x44c, 0x441, 0x43a, 0x456,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x406, 0x437, 0x440, 0x430, 0x456, 0x43b, 0x44c, 0x441, 0x43a,
0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427,
0x430, 0x441, 0x20, 0x42f, 0x43f, 0x43e, 0x43d, 0x456, 0x456, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x42f, 0x43f, 0x43e, 0x43d, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x42f, 0x43f, 0x43e, 0x43d, 0x456, 0x456, 0x41a, 0x430,
0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x43a, 0x430, 0x437,
0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x43a, 0x430, 0x437, 0x430,
0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441,
0x427, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x440, 0x44d, 0x456, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x41a, 0x430, 0x440, 0x44d, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x41a, 0x430, 0x440, 0x44d, 0x456, 0x427, 0x430, 0x441, 0x20,
0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x41a, 0x443, 0x441,
0x430, 0x456, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x430, 0x44f, 0x440, 0x441,
0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x430,
0x44f, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x430, 0x44f, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41a, 0x44b, 0x440,
0x433, 0x44b, 0x437, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x427, 0x430, 0x441, 0x20,
0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x41b, 0x430, 0x439,
0x43d, 0x427, 0x430, 0x441, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430,
0x443, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443,
0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41b, 0x43e,
0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41c, 0x430, 0x43b,
0x430, 0x439, 0x437, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20, 0x41c, 0x430, 0x43b,
0x44c, 0x434, 0x44b, 0x432, 0x430, 0x45e, 0x427, 0x430, 0x441, 0x20, 0x41c, 0x430,
0x440, 0x43a, 0x456, 0x437, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442,
0x440, 0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441, 0x20, 0x41c, 0x430, 0x440,
0x448, 0x430, 0x43b, 0x430, 0x432, 0x44b, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440,
0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441, 0x20, 0x41c, 0x430, 0x45e, 0x440,
0x44b, 0x43a, 0x456, 0x44f, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41c, 0x430, 0x45e, 0x440, 0x44b,
0x43a, 0x456, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x41c, 0x430, 0x45e, 0x440, 0x44b, 0x43a, 0x456, 0x44f, 0x427, 0x430, 0x441,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x41c, 0x43e, 0x45e,
0x441, 0x430, 0x43d, 0x41c, 0x435, 0x43a, 0x441, 0x456, 0x43a, 0x430, 0x43d, 0x441,
0x43a, 0x456, 0x20, 0x446, 0x456, 0x445, 0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41c, 0x435, 0x43a, 0x441, 0x456,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446, 0x456, 0x445, 0x430, 0x430,
0x43a, 0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41c, 0x435, 0x43a,
0x441, 0x456, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446, 0x456, 0x445,
0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x423,
0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x430, 0x440, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x430, 0x440, 0x430,
0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x423, 0x43b,
0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x430, 0x440, 0x430, 0x41c, 0x430, 0x441,
0x43a, 0x43e, 0x45e, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41c, 0x430,
0x441, 0x43a, 0x43e, 0x45e, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41c, 0x430,
0x441, 0x43a, 0x43e, 0x45e, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41c, 0x2019, 0x44f,
0x43d, 0x43c, 0x44b, 0x427, 0x430, 0x441, 0x20, 0x41d, 0x430, 0x443, 0x440, 0x443,
0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x44c, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x427, 0x430, 0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41d,
0x43e, 0x432, 0x430, 0x439, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e,
0x43d, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x439,
0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x44b, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x44b, 0x456, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x45e, 0x43d, 0x434, 0x43b, 0x435,
0x43d, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41d, 0x44c, 0x44e,
0x444, 0x430, 0x45e, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x45e, 0x43d, 0x434, 0x43b,
0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41d, 0x456, 0x443, 0x44d,
0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430,
0x20, 0x41d, 0x43e, 0x440, 0x444, 0x430, 0x43b, 0x43a, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x432,
0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430, 0x20, 0x41d, 0x43e, 0x440, 0x444,
0x430, 0x43b, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430, 0x20, 0x41d, 0x43e,
0x440, 0x444, 0x430, 0x43b, 0x43a, 0x427, 0x430, 0x441, 0x20, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x437, 0x456, 0x2d, 0x41d, 0x430,
0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x424, 0x435, 0x440, 0x43d,
0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x437, 0x456, 0x2d, 0x41d, 0x430, 0x440,
0x43e, 0x43d, 0x44c, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434,
0x437, 0x456, 0x2d, 0x41d, 0x430, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x41d, 0x43e,
0x432, 0x430, 0x441, 0x456, 0x431, 0x456, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x41d, 0x43e, 0x432, 0x430, 0x441, 0x456, 0x431, 0x456, 0x440, 0x441,
0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x41d, 0x43e, 0x432, 0x430, 0x441, 0x456, 0x431,
0x456, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x41e, 0x43c, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441,
0x41e, 0x43c, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41e, 0x43c, 0x441, 0x43a,
0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41f,
0x430, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x41f, 0x430, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b,
0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x427, 0x430, 0x441, 0x20, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x427, 0x430,
0x441, 0x20, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x2d, 0x41d, 0x43e, 0x432, 0x430,
0x439, 0x20, 0x413, 0x432, 0x456, 0x43d, 0x435, 0x456, 0x427, 0x430, 0x441, 0x20,
0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x44f, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430,
0x44f, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x424, 0x456, 0x43b,
0x456, 0x43f, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x424,
0x456, 0x43b, 0x456, 0x43f, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x424, 0x456, 0x43b, 0x456, 0x43f, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20,
0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x424, 0x435, 0x43d,
0x456, 0x43a, 0x441, 0x427, 0x430, 0x441, 0x20, 0x421, 0x435, 0x43d, 0x2d, 0x41f,
0x2019, 0x435, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x2019, 0x435, 0x440,
0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x421, 0x435, 0x43d, 0x2d, 0x41f,
0x2019, 0x435, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432,
0x430, 0x20, 0x41f, 0x456, 0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x427, 0x430, 0x441,
0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430, 0x20, 0x41f, 0x430,
0x43d, 0x43f, 0x435, 0x456, 0x41f, 0x445, 0x435, 0x43d, 0x44c, 0x44f, 0x43d, 0x441,
0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x420, 0x44d,
0x44e, 0x43d, 0x44c, 0x451, 0x43d, 0x430, 0x427, 0x430, 0x441, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x420, 0x43e, 0x442, 0x44d, 0x440, 0x430,
0x421, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427,
0x430, 0x441, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x421,
0x430, 0x43c, 0x43e, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x427, 0x430, 0x441, 0x20, 0x421,
0x435, 0x439, 0x448, 0x44d, 0x43b, 0x44c, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430,
0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x421, 0x456, 0x43d, 0x433, 0x430,
0x43f, 0x443, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430,
0x441, 0x20, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x43e, 0x43d, 0x430, 0x432, 0x44b,
0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430,
0x441, 0x20, 0x41f, 0x430, 0x45e, 0x434, 0x43d, 0x451, 0x432, 0x430, 0x439, 0x20,
0x413, 0x435, 0x43e, 0x440, 0x433, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20, 0x421,
0x443, 0x440, 0x44b, 0x43d, 0x430, 0x43c, 0x430, 0x427, 0x430, 0x441, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x421, 0x451, 0x432, 0x430, 0x427,
0x430, 0x441, 0x20, 0x422, 0x430, 0x456, 0x446, 0x456, 0x427, 0x430, 0x441, 0x20,
0x422, 0x430, 0x439, 0x431, 0x44d, 0x44f, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x422, 0x430, 0x439,
0x431, 0x44d, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x44f, 0x427, 0x430, 0x441, 0x20, 0x422,
0x430, 0x434, 0x436, 0x44b, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x427,
0x430, 0x441, 0x20, 0x422, 0x430, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x427, 0x430,
0x441, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x422, 0x43e,
0x43d, 0x433, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x427, 0x430, 0x441, 0x20, 0x422, 0x440,
0x443, 0x43a, 0x430, 0x427, 0x430, 0x441, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x435, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x422,
0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x430,
0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x422, 0x443,
0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x427,
0x430, 0x441, 0x20, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x423,
0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20,
0x423, 0x437, 0x431, 0x435, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d,
0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x423,
0x437, 0x431, 0x435, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x427, 0x430,
0x441, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443,
0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x44c, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x423, 0x43b, 0x430, 0x434, 0x437, 0x456, 0x432, 0x430,
0x441, 0x442, 0x43e, 0x446, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x423, 0x43b,
0x430, 0x434, 0x437, 0x456, 0x432, 0x430, 0x441, 0x442, 0x43e, 0x446, 0x43a, 0x456,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x423, 0x43b, 0x430, 0x434, 0x437, 0x456, 0x432, 0x430, 0x441,
0x442, 0x43e, 0x446, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x412, 0x430, 0x43b, 0x433, 0x430, 0x433, 0x440, 0x430, 0x434,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x412, 0x430, 0x43b, 0x433, 0x430,
0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x412, 0x430,
0x43b, 0x433, 0x430, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x412, 0x430, 0x441, 0x442,
0x43e, 0x43a, 0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430,
0x432, 0x430, 0x20, 0x423, 0x44d, 0x439, 0x43a, 0x427, 0x430, 0x441, 0x20, 0x430,
0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x423, 0x43e, 0x43b, 0x456,
0x441, 0x20, 0x456, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x42f, 0x43a,
0x443, 0x446, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x42f, 0x43a, 0x443, 0x446,
0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x42f, 0x43a, 0x443, 0x446, 0x43a, 0x456, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x415, 0x43a, 0x430,
0x446, 0x44f, 0x440, 0x44b, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x415, 0x43a, 0x430, 0x446, 0x44f, 0x440, 0x44b, 0x43d,
0x431, 0x443, 0x440, 0x433, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x415, 0x43a,
0x430, 0x446, 0x44f, 0x440, 0x44b, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x441, 0x43a,
0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427,
0x430, 0x441, 0x20, 0x42e, 0x43a, 0x430, 0x43d, 0x430, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x90f, 0x915, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x906, 0x915, 0x930, 0x20,
0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x906, 0x915, 0x930, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x906, 0x92b, 0x917, 0x93e, 0x928, 0x940, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x906, 0x92b, 0x94d,
0x930, 0x93f, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e,
0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x916,
0x94b, 0x932, 0x93e, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x928,
0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915,
0x93e, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20,
0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e,
0x92c, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x906, 0x932,
0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x932,
0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x947, 0x928, 0x94d,
0x921, 0x93e, 0x930, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x906, 0x932, 0x93e, 0x938,
0x94d, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x905, 0x932, 0x92e, 0x93e, 0x91f, 0x940, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x905, 0x932, 0x92e, 0x93e, 0x91f, 0x940, 0x20, 0x938, 0x94d, 0x91f,
0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x905,
0x932, 0x92e, 0x93e, 0x91f, 0x940, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x906, 0x92e, 0x93e, 0x91c, 0x928, 0x20, 0x938, 0x92e, 0x906, 0x92e,
0x93e, 0x91c, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x931, 0x93f, 0x20,
0x938, 0x92e, 0x906, 0x92e, 0x93e, 0x91c, 0x928, 0x20, 0x926, 0x948, 0x91c, 0x94d,
0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906,
0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f, 0x930,
0x941, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20,
0x92e, 0x93e, 0x928, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x93e, 0x928,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906,
0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x93e, 0x928,
0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939,
0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20,
0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939, 0x93e,
0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x938,
0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x93e, 0x928,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906,
0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x939, 0x93e, 0x91c,
0x94b, 0x92e, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e,
0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f,
0x20, 0x939, 0x93e, 0x91c, 0x94b, 0x92e, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e,
0x939, 0x93e, 0x20, 0x906, 0x92e, 0x94b, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f,
0x20, 0x939, 0x93e, 0x91c, 0x94b, 0x92e, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939,
0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20,
0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x938, 0x93e,
0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f,
0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20,
0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x92a, 0x947,
0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x906, 0x928, 0x93e, 0x926, 0x940, 0x930, 0x20, 0x938, 0x92e,
0x905, 0x928, 0x93e, 0x926, 0x940, 0x930, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902,
0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x905, 0x928, 0x93e,
0x926, 0x940, 0x930, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e,
0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x906, 0x92a, 0x93f, 0x92f,
0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x93e, 0x928, 0x20, 0x938, 0x92e,
0x905, 0x915, 0x94d, 0x91f, 0x93e, 0x90a, 0x20, 0x938, 0x92e, 0x905, 0x915, 0x93c,
0x94d, 0x91f, 0x93e, 0x90a, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930,
0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x905, 0x915, 0x93c, 0x94d, 0x91f,
0x93e, 0x90a, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x906,
0x915, 0x94d, 0x924, 0x2019, 0x92c, 0x20, 0x938, 0x92e, 0x905, 0x915, 0x93c, 0x94d,
0x91f, 0x94b, 0x92c, 0x947, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930,
0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x905, 0x915, 0x93c, 0x94d, 0x91f,
0x94b, 0x92c, 0x947, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e,
0x906, 0x930, 0x93e, 0x92c, 0x940, 0x92f, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x906,
0x930, 0x93e, 0x92c, 0x940, 0x92f, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x930, 0x93e, 0x92c, 0x940, 0x92f,
0x93e, 0x928, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f, 0x93f, 0x928, 0x93e, 0x20,
0x938, 0x92e, 0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f, 0x93f, 0x928,
0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f, 0x93f, 0x928, 0x93e, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x938, 0x94b,
0x928, 0x93e, 0x92c, 0x20, 0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f,
0x93f, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20,
0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f, 0x93f, 0x928, 0x93e, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b,
0x928, 0x93e, 0x92c, 0x20, 0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f,
0x93f, 0x928, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20,
0x938, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x930,
0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d,
0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x906, 0x91f, 0x932, 0x93e, 0x928, 0x94d,
0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x906, 0x91f, 0x932, 0x93e, 0x928, 0x94d,
0x91f, 0x93f, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x906, 0x91f, 0x932, 0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20,
0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x93f, 0x930,
0x941, 0x20, 0x905, 0x2019, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x905, 0x2019, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f, 0x930, 0x941,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905,
0x2019, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x94b, 0x928,
0x93e, 0x92c, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f,
0x930, 0x941, 0x20, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x93e, 0x930, 0x93f, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20,
0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x93e, 0x930,
0x93f, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938,
0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x905, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x905,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f,
0x20, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x938,
0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x93e, 0x928,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x93e,
0x930, 0x93f, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x94b, 0x928, 0x93e, 0x92c,
0x93e, 0x930, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x928, 0x93f, 0x20, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x93e, 0x930, 0x93f,
0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x91c,
0x93e, 0x930, 0x92c, 0x93e, 0x908, 0x91c, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x906,
0x91c, 0x93e, 0x930, 0x92c, 0x93e, 0x908, 0x91c, 0x93e, 0x928, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x91c, 0x93e, 0x930,
0x92c, 0x93e, 0x908, 0x91c, 0x93e, 0x928, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x906, 0x91c, 0x2019, 0x930, 0x94d, 0x938, 0x928,
0x93f, 0x20, 0x938, 0x92e, 0x906, 0x91c, 0x2019, 0x930, 0x94d, 0x938, 0x928, 0x93f,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906,
0x91c, 0x2019, 0x930, 0x94d, 0x938, 0x928, 0x93f, 0x20, 0x926, 0x948, 0x91c, 0x94d,
0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92c, 0x93e, 0x902, 0x932, 0x93e, 0x926,
0x947, 0x936, 0x20, 0x938, 0x92e, 0x92c, 0x93e, 0x902, 0x932, 0x93e, 0x926, 0x947,
0x936, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x92c, 0x93e, 0x902, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92d, 0x941, 0x91f, 0x93e, 0x928,
0x20, 0x938, 0x92e, 0x92c, 0x932, 0x93f, 0x92d, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92c,
0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x948,
0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92c, 0x94d, 0x930, 0x941,
0x928, 0x947, 0x908, 0x20, 0x926, 0x930, 0x20, 0x909, 0x938, 0x20, 0x938, 0x932,
0x93e, 0x92e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x92d, 0x947, 0x930,
0x94d, 0x926, 0x947, 0x20, 0x938, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x92d, 0x947,
0x930, 0x94d, 0x926, 0x947, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x92d, 0x947, 0x930, 0x94d, 0x926,
0x947, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x91a, 0x93e, 0x92e, 0x2019, 0x930, 0x94d, 0x930, 0x2019, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a, 0x947, 0x925, 0x93e, 0x92e,
0x20, 0x938, 0x92e, 0x91a, 0x947, 0x925, 0x93e, 0x92e, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a, 0x947, 0x925, 0x93e, 0x92e,
0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a, 0x940,
0x932, 0x947, 0x20, 0x938, 0x92e, 0x91a, 0x940, 0x932, 0x947, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a, 0x940, 0x932, 0x947,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x91a,
0x93e, 0x908, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x91a, 0x93e, 0x908, 0x928, 0x93e,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a,
0x93e, 0x908, 0x928, 0x93e, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x916, 0x94d, 0x930, 0x940, 0x938, 0x94d, 0x91f, 0x92e, 0x93e, 0x938,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x915, 0x915, 0x2019,
0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f,
0x20, 0x938, 0x92e, 0x915, 0x932, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x915, 0x932, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x932, 0x92e,
0x94d, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x941, 0x915,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x941,
0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f,
0x20, 0x916, 0x93e, 0x935, 0x938, 0x947, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x915, 0x93f, 0x909, 0x92c, 0x93e, 0x20, 0x938,
0x92e, 0x915, 0x93f, 0x909, 0x92c, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x93f, 0x909, 0x92c, 0x93e, 0x20, 0x938,
0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x921, 0x947, 0x92d, 0x93f,
0x938, 0x20, 0x938, 0x92e, 0x921, 0x941, 0x92e, 0x928, 0x94d, 0x924, 0x2d, 0x921,
0x940, 0x906, 0x930, 0x94d, 0x92d, 0x940, 0x932, 0x20, 0x938, 0x92e, 0x908, 0x938,
0x94d, 0x91f, 0x20, 0x91f, 0x940, 0x92e, 0x94b, 0x930, 0x20, 0x938, 0x94d, 0x91f,
0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x938,
0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e,
0x930, 0x93f, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e,
0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x90f, 0x915, 0x941,
0x935, 0x93e, 0x921, 0x930, 0x20, 0x938, 0x92e, 0x92e, 0x93f, 0x930, 0x941, 0x20,
0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x93f,
0x930, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x93f, 0x930,
0x941, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x926, 0x948,
0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c,
0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928,
0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930,
0x93f, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c,
0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928,
0x93f, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x917, 0x94b, 0x91c, 0x93e, 0x928, 0x938, 0x93f, 0x928, 0x2d, 0x938, 0x93e, 0x928,
0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a,
0x928, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x92f,
0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928,
0x93e, 0x92c, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928,
0x93e, 0x92c, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92b, 0x915, 0x932,
0x947, 0x923, 0x94d, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b,
0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x92b, 0x915, 0x932, 0x947, 0x923, 0x94d,
0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92b,
0x915, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a,
0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x92b, 0x940, 0x91c, 0x940, 0x20, 0x938, 0x92e, 0x92b,
0x940, 0x91c, 0x940, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x92b, 0x940, 0x91c, 0x940, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92b, 0x94d, 0x930, 0x947, 0x928, 0x94d, 0x91a,
0x20, 0x917, 0x941, 0x908, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92b,
0x94d, 0x930, 0x947, 0x928, 0x94d, 0x91a, 0x20, 0x916, 0x94b, 0x932, 0x93e, 0x92f,
0x93e, 0x930, 0x93f, 0x20, 0x906, 0x930, 0x94b, 0x20, 0x90f, 0x928, 0x94d, 0x91f,
0x93e, 0x930, 0x94d, 0x915, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x917, 0x93e,
0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x2019, 0x938, 0x20, 0x938, 0x92e, 0x917, 0x93e,
0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x947, 0x930, 0x20, 0x938, 0x92e, 0x91c, 0x930,
0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x91c, 0x930, 0x94d, 0x91c,
0x93f, 0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x948,
0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x917, 0x940, 0x932, 0x92c,
0x93e, 0x930, 0x94d, 0x91f, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b,
0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x917, 0x94d, 0x930, 0x93f, 0x928, 0x935,
0x940, 0x91a, 0x20, 0x917, 0x947, 0x91c, 0x947, 0x930, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928,
0x91c, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x923, 0x94d,
0x921, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x938, 0x93e, 0x928, 0x921, 0x91c, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x938,
0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947,
0x923, 0x94d, 0x921, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x938, 0x947, 0x928, 0x93e, 0x92c, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x917, 0x941, 0x906, 0x92e, 0x20, 0x938, 0x94d,
0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e,
0x917, 0x93e, 0x932, 0x94d, 0x92b, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x938,
0x92e, 0x939, 0x93e, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f, 0x941,
0x924, 0x93f, 0x92f, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x939, 0x93e, 0x935, 0x93e,
0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f, 0x941, 0x924, 0x93f, 0x92f, 0x93e, 0x928,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x939,
0x93e, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f, 0x941, 0x924, 0x93f,
0x92f, 0x93e, 0x928, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x939, 0x902, 0x915, 0x902, 0x20, 0x938, 0x92e, 0x939, 0x902, 0x915, 0x902,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x939,
0x902, 0x915, 0x902, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20,
0x938, 0x92e, 0x939, 0x2019, 0x92d, 0x94d, 0x926, 0x20, 0x938, 0x92e, 0x939, 0x2019,
0x92d, 0x94d, 0x926, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x939, 0x2019, 0x92d, 0x94d, 0x926, 0x20, 0x926, 0x948, 0x91c, 0x94d,
0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92d, 0x93e, 0x930, 0x924, 0x93e, 0x930,
0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x92d, 0x93e, 0x930, 0x924, 0x20, 0x932, 0x948, 0x925, 0x94b, 0x92e, 0x93e, 0x92f,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x908, 0x902, 0x921, 0x94b, 0x20, 0x91a,
0x907, 0x928, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d,
0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x908, 0x902, 0x921, 0x94b, 0x928, 0x940,
0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930,
0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x930,
0x94d, 0x928, 0x20, 0x908, 0x902, 0x921, 0x94b, 0x928, 0x940, 0x936, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20,
0x91f, 0x93e, 0x908, 0x92e, 0x935, 0x947, 0x938, 0x94d, 0x91f, 0x930, 0x94d, 0x928,
0x20, 0x908, 0x902, 0x921, 0x94b, 0x928, 0x940, 0x936, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x907, 0x930, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x907, 0x930, 0x93e,
0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x907, 0x930, 0x93e, 0x928, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x938, 0x92e, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x908, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x902, 0x20, 0x938, 0x92e, 0x907, 0x91c,
0x94d, 0x930, 0x93e, 0x908, 0x932, 0x20, 0x938, 0x92e, 0x907, 0x91c, 0x94d, 0x930,
0x93e, 0x908, 0x932, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x907, 0x91c, 0x94d, 0x930, 0x93e, 0x908, 0x932, 0x20, 0x938, 0x93e,
0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928,
0x20, 0x938, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928,
0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x947,
0x924, 0x94d, 0x930, 0x2019, 0x92a, 0x93e, 0x92d, 0x94d, 0x932, 0x2019, 0x92d, 0x94d,
0x938, 0x94d, 0x915, 0x2d, 0x915, 0x93e, 0x92e, 0x91a, 0x93e, 0x924, 0x94d, 0x938,
0x94d, 0x915, 0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x947, 0x924, 0x94d, 0x930, 0x94b,
0x92a, 0x93e, 0x935, 0x932, 0x94b, 0x938, 0x94d, 0x915, 0x20, 0x915, 0x93e, 0x92e,
0x91a, 0x91f, 0x915, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930,
0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x92a, 0x947, 0x924, 0x94d, 0x930,
0x94b, 0x92a, 0x93e, 0x935, 0x932, 0x94b, 0x938, 0x94d, 0x915, 0x20, 0x915, 0x93e,
0x92e, 0x91a, 0x91f, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x20, 0x915, 0x93e, 0x91c, 0x93e,
0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928,
0x93e, 0x92c, 0x20, 0x915, 0x93e, 0x91c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x915, 0x2019, 0x930, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x92e, 0x915, 0x2019, 0x930, 0x93f, 0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x2019, 0x930, 0x93f, 0x92f, 0x93e,
0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x938,
0x94d, 0x930, 0x947, 0x20, 0x938, 0x92e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d,
0x928, 0x2019, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e,
0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x2019, 0x92f, 0x93e, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x2019, 0x92f, 0x93e,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x902, 0x20, 0x938, 0x92e, 0x915, 0x940, 0x930, 0x94d, 0x917, 0x940, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x932, 0x902, 0x915, 0x93e, 0x20,
0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x932, 0x93e, 0x908, 0x928, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a,
0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x932, 0x930, 0x94d, 0x921,
0x20, 0x939, 0x93e, 0x935, 0x93f, 0x20, 0x938, 0x92e, 0x932, 0x930, 0x94d, 0x921,
0x20, 0x939, 0x93e, 0x935, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x932, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935,
0x93f, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92e,
0x93e, 0x915, 0x93e, 0x90a, 0x20, 0x938, 0x92e, 0x92e, 0x93e, 0x915, 0x93e, 0x90a,
0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f,
0x93e, 0x908, 0x92e, 0x92e, 0x93e, 0x915, 0x93e, 0x90a, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928,
0x20, 0x938, 0x92e, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x93e, 0x917,
0x93e, 0x926, 0x93e, 0x928, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902,
0x20, 0x938, 0x92e, 0x92e, 0x932, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908,
0x92e, 0x92e, 0x93e, 0x932, 0x926, 0x940, 0x92d, 0x94d, 0x938, 0x20, 0x938, 0x92e,
0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x947, 0x938, 0x93e, 0x938, 0x20, 0x938, 0x92e,
0x92e, 0x93e, 0x930, 0x94d, 0x936, 0x947, 0x932, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x2019, 0x930,
0x93f, 0x936, 0x93f, 0x92f, 0x93e, 0x938, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x92e,
0x2019, 0x930, 0x93f, 0x936, 0x93f, 0x92f, 0x93e, 0x938, 0x928, 0x93f, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x2019, 0x930,
0x93f, 0x936, 0x93f, 0x92f, 0x93e, 0x938, 0x928, 0x93f, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92e, 0x94c, 0x938, 0x928, 0x20,
0x938, 0x92e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x2019, 0x928, 0x93f,
0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92e,
0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x2019, 0x928, 0x93f, 0x20, 0x92a, 0x947,
0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x2019,
0x928, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938,
0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x909, 0x932, 0x93e, 0x928,
0x92c, 0x93e, 0x924, 0x93e, 0x930, 0x20, 0x938, 0x92e, 0x909, 0x932, 0x93e, 0x928,
0x92c, 0x93e, 0x924, 0x93e, 0x930, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x93e, 0x924, 0x93e,
0x930, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x92e, 0x938, 0x94d, 0x915, 0x94c, 0x20, 0x938, 0x92e, 0x92e, 0x938, 0x94d, 0x915,
0x94c, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x92e, 0x938, 0x94d, 0x915, 0x94c, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x92e, 0x93e, 0x930,
0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f,
0x93e, 0x908, 0x92e, 0x928, 0x93e, 0x90a, 0x930, 0x941, 0x20, 0x938, 0x92e, 0x928,
0x947, 0x92a, 0x93e, 0x932, 0x20, 0x938, 0x92e, 0x917, 0x94b, 0x926, 0x93e, 0x928,
0x20, 0x915, 0x947, 0x932, 0x947, 0x921, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x92e, 0x917, 0x94b, 0x926, 0x93e, 0x928, 0x20, 0x915, 0x947, 0x932, 0x947, 0x921,
0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x917, 0x94b, 0x926, 0x93e, 0x928, 0x20, 0x915, 0x947, 0x932,
0x947, 0x921, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x928, 0x93f, 0x909, 0x91c, 0x93f, 0x932, 0x947,
0x923, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x928, 0x93f, 0x909, 0x91c, 0x93f, 0x932,
0x947, 0x923, 0x94d, 0x921, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x928, 0x93f, 0x909, 0x91c, 0x93f, 0x932, 0x947, 0x923, 0x94d,
0x921, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x928,
0x93f, 0x909, 0x92b, 0x93e, 0x909, 0x928, 0x94d, 0x926, 0x932, 0x947, 0x928, 0x94d,
0x926, 0x20, 0x938, 0x92e, 0x928, 0x93f, 0x909, 0x92b, 0x93e, 0x909, 0x928, 0x94d,
0x926, 0x932, 0x947, 0x928, 0x94d, 0x926, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x928, 0x93f, 0x909, 0x92b, 0x93e, 0x909, 0x928,
0x94d, 0x926, 0x932, 0x947, 0x928, 0x94d, 0x926, 0x20, 0x938, 0x93e, 0x928, 0x93e,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x928, 0x940, 0x90a, 0x90f, 0x20, 0x938, 0x92e,
0x928, 0x930, 0x92b, 0x2019, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20,
0x938, 0x92e, 0x928, 0x930, 0x92b, 0x2019, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x928, 0x930, 0x92b, 0x2019, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20,
0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92b, 0x947, 0x930,
0x928, 0x93e, 0x928, 0x94d, 0x926, 0x2019, 0x20, 0x926, 0x947, 0x20, 0x928, 0x930,
0x2019, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x938, 0x92e, 0x92b, 0x947, 0x930, 0x928,
0x93e, 0x928, 0x94d, 0x926, 0x2019, 0x20, 0x926, 0x947, 0x20, 0x928, 0x930, 0x2019,
0x928, 0x94d, 0x939, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x92b, 0x947, 0x930, 0x928, 0x93e, 0x928, 0x94d, 0x926, 0x2019,
0x20, 0x926, 0x947, 0x20, 0x928, 0x930, 0x2019, 0x928, 0x94d, 0x939, 0x93e, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x928, 0x949,
0x930, 0x94d, 0x925, 0x20, 0x92e, 0x93e, 0x930, 0x93f, 0x906, 0x928, 0x93e, 0x20,
0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x928, 0x92d, 0x2019, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x938, 0x92e, 0x928, 0x92d, 0x2019, 0x938, 0x93f, 0x92c, 0x93f,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x928, 0x92d, 0x2019, 0x938, 0x93f, 0x92c, 0x93f, 0x930,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902,
0x20, 0x938, 0x92e, 0x905, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e,
0x905, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92a, 0x93e,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92a, 0x93e,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20,
0x938, 0x92e, 0x92a, 0x93e, 0x932, 0x93e, 0x909, 0x20, 0x938, 0x92e, 0x92a, 0x93e,
0x92a, 0x941, 0x906, 0x20, 0x928, 0x93f, 0x909, 0x20, 0x917, 0x93f, 0x928, 0x940,
0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x941, 0x935, 0x947, 0x20,
0x938, 0x92e, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x941, 0x935, 0x947, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x930,
0x93e, 0x917, 0x941, 0x935, 0x947, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x938, 0x92e, 0x92a,
0x947, 0x930, 0x941, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92b, 0x93f, 0x932, 0x940, 0x92a, 0x940, 0x928,
0x94d, 0x938, 0x20, 0x938, 0x92e, 0x92b, 0x940, 0x932, 0x940, 0x92a, 0x940, 0x928,
0x94d, 0x938, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x92b, 0x940, 0x932, 0x940, 0x92a, 0x940, 0x928,
0x94d, 0x938, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x92b,
0x928, 0x940, 0x915, 0x94d, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b,
0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x948, 0x928, 0x94d, 0x91f,
0x20, 0x92a, 0x93f, 0x92f, 0x947, 0x930, 0x20, 0x906, 0x930, 0x94b, 0x20, 0x92e,
0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x928, 0x20, 0x938, 0x92e, 0x938, 0x948,
0x928, 0x94d, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x947, 0x930, 0x20, 0x906, 0x930,
0x94b, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x928, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x948, 0x928,
0x94d, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x947, 0x930, 0x20, 0x906, 0x930, 0x94b,
0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x928, 0x20, 0x938, 0x93e,
0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x940, 0x91f, 0x915, 0x948,
0x930, 0x94d, 0x928, 0x20, 0x938, 0x92e, 0x92a, 0x928, 0x93e, 0x92a, 0x947, 0x20,
0x938, 0x92e, 0x92a, 0x94d, 0x92f, 0x902, 0x92f, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x915, 0x93c, 0x940, 0x91c, 0x93c, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x915, 0x93c, 0x940, 0x91c, 0x93c, 0x940, 0x932,
0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921,
0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x915, 0x93c, 0x940, 0x91c,
0x93c, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x930, 0x93f, 0x92f, 0x942, 0x928, 0x93f, 0x92f,
0x928, 0x20, 0x938, 0x92e, 0x930, 0x925, 0x947, 0x930, 0x93e, 0x20, 0x938, 0x92e,
0x938, 0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x938, 0x92e, 0x938, 0x93e,
0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x938, 0x92e,
0x93e, 0x930, 0x93e, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x938, 0x92e, 0x93e, 0x930,
0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20,
0x91f, 0x93e, 0x908, 0x92e, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x938, 0x93e, 0x92e, 0x2019, 0x935, 0x93e,
0x20, 0x938, 0x92e, 0x938, 0x93e, 0x92e, 0x2019, 0x935, 0x93e, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x92e, 0x2019,
0x935, 0x93e, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x938, 0x947, 0x936, 0x947, 0x932, 0x94d, 0x938, 0x20, 0x938, 0x92e, 0x938, 0x940,
0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902,
0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x938, 0x932, 0x2019,
0x92e, 0x928, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928,
0x93f, 0x20, 0x938, 0x92e, 0x916, 0x94b, 0x932, 0x93e, 0x20, 0x91c, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x938, 0x941, 0x930, 0x940, 0x928,
0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x938, 0x940, 0x905, 0x935, 0x93e, 0x20, 0x938,
0x92e, 0x91f, 0x93e, 0x939, 0x93f, 0x91f, 0x940, 0x20, 0x938, 0x92e, 0x924, 0x93e,
0x908, 0x92a, 0x947, 0x907, 0x20, 0x938, 0x92e, 0x924, 0x93e, 0x908, 0x92a, 0x947,
0x907, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x924, 0x93e, 0x908, 0x92a, 0x947, 0x907, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x924, 0x93e, 0x91c, 0x93f, 0x915, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x91f, 0x915, 0x947, 0x932, 0x94c, 0x20,
0x938, 0x92e, 0x91f, 0x902, 0x917, 0x93e, 0x20, 0x938, 0x92e, 0x91f, 0x902, 0x917,
0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x91f, 0x902, 0x917, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902,
0x20, 0x938, 0x92e, 0x91a, 0x942, 0x915, 0x20, 0x938, 0x92e, 0x924, 0x941, 0x930,
0x94d, 0x915, 0x92e, 0x947, 0x928, 0x940, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x940, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x940,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x924, 0x941, 0x92d, 0x93e, 0x932, 0x941, 0x20, 0x938,
0x92e, 0x909, 0x930, 0x941, 0x917, 0x941, 0x935, 0x947, 0x20, 0x938, 0x92e, 0x909,
0x930, 0x941, 0x917, 0x941, 0x935, 0x947, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x941, 0x935, 0x947,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x909,
0x91c, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938,
0x92e, 0x909, 0x91c, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x909,
0x91c, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x935, 0x93e, 0x928,
0x941, 0x906, 0x91f, 0x941, 0x20, 0x938, 0x92e, 0x935, 0x93e, 0x928, 0x941, 0x906,
0x91f, 0x941, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x935, 0x93e, 0x928, 0x941, 0x906, 0x91f, 0x941, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92d, 0x947, 0x928, 0x947, 0x91c,
0x941, 0x935, 0x947, 0x932, 0x93e, 0x20, 0x938, 0x92e, 0x92d, 0x94d, 0x932, 0x93e,
0x926, 0x93f, 0x92d, 0x938, 0x94d, 0x924, 0x2019, 0x915, 0x20, 0x938, 0x92e, 0x92d,
0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x938, 0x94d, 0x924, 0x2019, 0x915, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92d, 0x94d,
0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x938, 0x94d, 0x924, 0x2019, 0x915, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92d, 0x932, 0x94d,
0x917, 0x2019, 0x917, 0x94d, 0x930, 0x947, 0x926, 0x20, 0x938, 0x92e, 0x92d, 0x932,
0x94d, 0x917, 0x2019, 0x917, 0x94d, 0x930, 0x947, 0x926, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92d, 0x932, 0x94d, 0x917, 0x2019,
0x917, 0x94d, 0x930, 0x947, 0x926, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x935, 0x948, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x20, 0x938, 0x92e, 0x935, 0x93e, 0x932, 0x940, 0x938, 0x20, 0x906, 0x930,
0x94b, 0x20, 0x92b, 0x941, 0x924, 0x941, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902,
0x20, 0x938, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x940, 0x928,
0x93e, 0x92c, 0x941, 0x930, 0x94d, 0x917, 0x20, 0x938, 0x92e, 0x92f, 0x947, 0x915,
0x93e, 0x924, 0x947, 0x930, 0x940, 0x928, 0x93e, 0x92c, 0x941, 0x930, 0x94d, 0x917,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92f,
0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x940, 0x928, 0x93e, 0x92c, 0x941, 0x930,
0x94d, 0x917, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938,
0x92e, 0x92f, 0x942, 0x915, 0x928, 0x20, 0x938, 0x92e, 0x41, 0x63, 0x72, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x66, 0x67, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f,
0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c,
0x6a, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41,
0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x75, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a,
0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76,
0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c,
0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a,
0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76,
0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6e,
0x61, 0x64, 0x69, 0x72, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6e,
0x61, 0x64, 0x69, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6e, 0x61,
0x64, 0x69, 0x72, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72,
0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x6e, 0x61, 0x41, 0x70, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x70, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x76, 0x74, 0x61,
0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x76, 0x74, 0x61,
0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x76, 0x74, 0x61, 0x75,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75,
0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e,
0x61, 0x41, 0x6b, 0x76, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x41, 0x6b, 0x76, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x41, 0x6b, 0x76, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x72,
0x61, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x61, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20,
0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a,
0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a,
0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42,
0x75, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f,
0x72, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x6d, 0x6f,
0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x61, 0x74, 0x61, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x10c, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x65, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x42, 0x6f, 0x17e, 0x69, 0x107, 0x6e, 0x6f, 0x6d,
0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x56, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69,
0x6d, 0x61, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4b, 0x75,
0x6b, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x69,
0x6d, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20,
0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f, 0x73, 0x74, 0x72,
0x76, 0x69, 0x6d, 0x61, 0x50, 0x6f, 0x6c, 0x75, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x75,
0x62, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x63, 0x65, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x56, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f,
0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x65,
0x6f, 0x73, 0x74, 0x72, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x65,
0x6f, 0x73, 0x74, 0x72, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x65, 0x6f,
0x73, 0x74, 0x72, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x45, 0x6b,
0x76, 0x61, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x44, 0x61, 0x6c, 0x65, 0x6b, 0x6f, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x75, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x69, 0x64, 0x17e, 0x69,
0x6a, 0x75, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x6f, 0x67,
0x76, 0x61, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x6f,
0x6a, 0x20, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x6a, 0x20, 0x54, 0x65, 0x72,
0x69, 0x74, 0x6f, 0x72, 0x69, 0x6a, 0x69, 0x20, 0x69, 0x20, 0x41, 0x6e,
0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x75, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x6a, 0x65, 0x72, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72,
0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x69,
0x6d, 0x61, 0x47, 0x72, 0x69, 0x6e, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e,
0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x75, 0x61, 0x6d, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x5a, 0x61, 0x6c, 0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x76, 0x61, 0x6a, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48,
0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x107, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x107, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x107, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x6d, 0x20, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x75, 0x49,
0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x6e, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61, 0x65,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d,
0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x6e, 0x61, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x6b, 0x61, 0x7a, 0x61,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b,
0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x75, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73,
0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x4c, 0x61, 0x6a, 0x6e, 0x56,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73,
0x74, 0x72, 0x76, 0x75, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x61,
0x75, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x61, 0x75, 0x4c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x75, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x61, 0x75, 0x4d,
0x61, 0x6b, 0x61, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61,
0x6b, 0x61, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6b, 0x61,
0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d,
0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x6e, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x4d, 0x61, 0x72,
0x6b, 0x69, 0x7a, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x4d, 0x61, 0x72, 0x161, 0x61, 0x6c, 0x6f, 0x76, 0x69, 0x6d,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x63, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x6c,
0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73,
0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4d, 0x69, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x75, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e,
0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e,
0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65,
0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e,
0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x4e,
0x69, 0x75, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x161, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a,
0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a,
0x61, 0x4c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76,
0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x53, 0x65, 0x76,
0x65, 0x72, 0x6e, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6a, 0x61, 0x6e,
0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4f, 0x6d,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4f,
0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4f,
0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x75, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x56, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x70, 0x75,
0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x6a, 0x20, 0x47, 0x76, 0x69, 0x6e,
0x65, 0x6a, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61,
0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50,
0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x46,
0x69, 0x6e, 0x69, 0x6b, 0x73, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61,
0x20, 0x53, 0x76, 0x65, 0x74, 0x69, 0x20, 0x50, 0x65, 0x74, 0x61, 0x72,
0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x69, 0x6d, 0x61, 0x20, 0x53, 0x76, 0x65, 0x74, 0x69, 0x20, 0x50,
0x65, 0x74, 0x61, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x6f, 0x6e, 0x4c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x69, 0x6d, 0x61, 0x20, 0x53, 0x76, 0x65, 0x74, 0x69, 0x20, 0x50,
0x65, 0x74, 0x61, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x6f, 0x6e, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x50, 0x69,
0x74, 0x6b, 0x65, 0x72, 0x6e, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x50,
0x6f, 0x6e, 0x61, 0x70, 0x65, 0x50, 0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61,
0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61,
0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x6e, 0x61, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65,
0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x53, 0x61, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d,
0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x6e, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x65, 0x6a, 0x161, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x6f, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x6d, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x69, 0x6d, 0x61, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x64, 0x17e,
0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x63, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x54, 0x61, 0x68, 0x69,
0x107, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54,
0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74,
0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x75, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x75, 0x10d, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x65, 0x6e,
0x65, 0x63, 0x75, 0x65, 0x6c, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x10d, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65,
0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x56, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75,
0x20, 0x56, 0x65, 0x6a, 0x6b, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61,
0x20, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x410, 0x43a, 0x440, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x43a, 0x440, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x43a, 0x440, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x410, 0x432, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x443, 0x436, 0x43d,
0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d,
0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x430, 0x444,
0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x410, 0x459, 0x430, 0x441, 0x43a, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x459, 0x430, 0x441, 0x43a, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x459, 0x430, 0x441, 0x43a,
0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x438, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x438, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430,
0x437, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x43b, 0x430, 0x43d, 0x438,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x446,
0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x43f, 0x438,
0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43f,
0x438, 0x458, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43f,
0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x410, 0x43a, 0x432, 0x430, 0x442, 0x430, 0x443, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x43a, 0x432, 0x430, 0x442, 0x430, 0x443, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x43a, 0x432, 0x430, 0x442, 0x430, 0x443, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x43a, 0x443,
0x442, 0x43e, 0x431, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43a,
0x443, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43a,
0x443, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430,
0x431, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410,
0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x442, 0x43b,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410,
0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x443, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x437, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x438,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x437, 0x435, 0x440, 0x431,
0x435, 0x458, 0x45f, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435,
0x458, 0x45f, 0x430, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434,
0x435, 0x448, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x458, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411,
0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x411,
0x440, 0x443, 0x43d, 0x435, 0x458, 0x20, 0x414, 0x430, 0x440, 0x443, 0x441, 0x430,
0x43b, 0x430, 0x43c, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417,
0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e,
0x440, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x43e, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x427, 0x430,
0x43c, 0x43e, 0x440, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x427, 0x438, 0x43b, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41a, 0x438, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x43d, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x43d, 0x430, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x411, 0x43e, 0x436,
0x438, 0x45b, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43a, 0x43e, 0x441,
0x20, 0x28, 0x41a, 0x435, 0x43b, 0x438, 0x43d, 0x433, 0x29, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438,
0x458, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x432, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443,
0x43a, 0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x43f, 0x43e, 0x43b, 0x443, 0x2d, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x414, 0x435, 0x458, 0x432, 0x438, 0x441, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x414, 0x438, 0x43c, 0x43e, 0x43d,
0x20, 0x434, 0x2019, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x2d, 0x41b, 0x435,
0x441, 0x442, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423,
0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a,
0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x415, 0x43a, 0x432, 0x430, 0x434,
0x43e, 0x440, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440,
0x435, 0x434, 0x45a, 0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434,
0x45a, 0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x435,
0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e,
0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432,
0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x414, 0x430, 0x459, 0x435, 0x20, 0x438, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x424, 0x43e, 0x43b, 0x43a,
0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x43e,
0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424,
0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x424, 0x438, 0x45f, 0x438, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443,
0x441, 0x43a, 0x430, 0x20, 0x413, 0x432, 0x430, 0x458, 0x430, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x440, 0x430, 0x43d, 0x446,
0x443, 0x441, 0x43a, 0x43e, 0x20, 0x458, 0x443, 0x436, 0x43d, 0x43e, 0x20, 0x438,
0x20, 0x430, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x430, 0x43b, 0x430,
0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x458, 0x435, 0x440, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437,
0x438, 0x458, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x440,
0x443, 0x437, 0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x413, 0x440, 0x438, 0x43d, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441,
0x440, 0x435, 0x434, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x413, 0x443, 0x430, 0x43c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43b, 0x438, 0x432, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458, 0x441,
0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458,
0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458,
0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x425,
0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d,
0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x433,
0x43a, 0x43e, 0x43d, 0x448, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x43d, 0x434, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x43d,
0x434, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418,
0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e,
0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437,
0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x440, 0x43a, 0x443, 0x446,
0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a,
0x443, 0x446, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440,
0x43a, 0x443, 0x446, 0x43a, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430,
0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x408, 0x430, 0x43f, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x408, 0x430, 0x43f, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x408, 0x430, 0x43f, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x435,
0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x43e,
0x2d, 0x43a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432,
0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x2d, 0x43a, 0x430, 0x43c, 0x447, 0x430,
0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442,
0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x2d,
0x43a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x2d, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x43a, 0x430, 0x437, 0x430, 0x445,
0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e,
0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41a, 0x43e, 0x448, 0x440, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x458, 0x430, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441,
0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x428,
0x440, 0x438, 0x20, 0x41b, 0x430, 0x43d, 0x43a, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41b, 0x438, 0x43d, 0x435, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440,
0x434, 0x20, 0x425, 0x43e, 0x432, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c,
0x430, 0x43a, 0x430, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430,
0x43a, 0x430, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43a, 0x430,
0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x430, 0x43b,
0x435, 0x437, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x438, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b,
0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446,
0x438, 0x458, 0x443, 0x441, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438,
0x458, 0x443, 0x441, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x41c, 0x43e, 0x441, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447,
0x43a, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x423,
0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430,
0x442, 0x43e, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x43b,
0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x432,
0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441,
0x43a, 0x432, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x43e,
0x441, 0x43a, 0x432, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x41c, 0x438, 0x458, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x430, 0x443, 0x440, 0x443,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x435, 0x43f, 0x430,
0x43b, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430,
0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x43e, 0x432, 0x438,
0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x40a, 0x443, 0x444, 0x430, 0x443,
0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d,
0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x40a, 0x443, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x438, 0x443, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b,
0x43a, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x432, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434,
0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e,
0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x41c, 0x430, 0x440, 0x438, 0x458, 0x430,
0x43d, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431,
0x438, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f,
0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x430, 0x43b,
0x430, 0x443, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430,
0x43f, 0x443, 0x430, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x413, 0x432, 0x438,
0x43d, 0x435, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x458, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438,
0x43f, 0x438, 0x43d, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20,
0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435,
0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458,
0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x438,
0x442, 0x43a, 0x435, 0x440, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430, 0x43d, 0x433,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x437, 0x438,
0x43b, 0x43e, 0x440, 0x434, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41a, 0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x420, 0x435, 0x438,
0x43d, 0x438, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x421, 0x435,
0x458, 0x448, 0x435, 0x43b, 0x438, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x441,
0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x443, 0x436, 0x43d, 0x430, 0x20, 0x40f,
0x43e, 0x440, 0x45f, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x428, 0x43e, 0x432, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x445, 0x438, 0x442, 0x438, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435,
0x458, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458,
0x43f, 0x435, 0x458, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x430,
0x458, 0x43f, 0x435, 0x458, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x422, 0x430, 0x45f, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43a,
0x435, 0x43b, 0x430, 0x443, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x422, 0x440, 0x443, 0x43a, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a,
0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x422, 0x443, 0x432, 0x430, 0x43b,
0x443, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x458, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x423, 0x437, 0x431, 0x435,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430,
0x442, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d,
0x443, 0x430, 0x442, 0x443, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x412, 0x435, 0x43d, 0x435, 0x446, 0x443, 0x435, 0x43b, 0x430,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434,
0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x412, 0x435, 0x458, 0x43a, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43b,
0x438, 0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x433, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x65, 0x75, 0x72, 0x20, 0x41, 0x66,
0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x65, 0x75, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61,
0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x75, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x65, 0x75, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4,
0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43,
0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61,
0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75,
0x72, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x65, 0x75, 0x72, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x65, 0x75, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x61, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x43,
0x2bc, 0x68, 0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68,
0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x72, 0x65, 0x69, 0x7a, 0x65,
0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61,
0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72,
0x65, 0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x4d, 0x65, 0x6e, 0x65, 0x7a,
0x69, 0x6f, 0xf9, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x4d, 0x65, 0x6e, 0x65, 0x7a, 0x69,
0x6f, 0xf9, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61,
0x72, 0x20, 0x4d, 0x65, 0x6e, 0x65, 0x7a, 0x69, 0x6f, 0xf9, 0x65, 0x75,
0x72, 0x20, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x62, 0x61, 0x73, 0x6b, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61,
0x6e, 0x20, 0x48, 0x61, 0x62, 0x61, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x62, 0x61,
0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x2bc, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2bc, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2bc,
0x65, 0x75, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x70, 0x69, 0x61,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x70, 0x69,
0x61, 0x65, 0x75, 0x72, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20,
0x41, 0x72, 0x63, 0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41,
0x72, 0x63, 0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x63, 0x2bc, 0x68,
0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x41, 0x72,
0x63, 0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x72,
0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x63,
0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x72, 0x20,
0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x63, 0x2bc, 0x68, 0x61, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f,
0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x65, 0x6c,
0x65, 0x75, 0x72, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61,
0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x4b,
0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4,
0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72,
0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e,
0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x41, 0x6f, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x41, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x72,
0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x41, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72,
0x20, 0x41, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61,
0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6f,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43,
0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x41, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4,
0x67, 0x65, 0x75, 0x72, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x64, 0x6a, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64,
0x6a, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x7a,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x61, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x7a, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x65, 0x75,
0x72, 0x20, 0x42, 0x68, 0x6f, 0x75, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20,
0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x42, 0x72, 0x61, 0x73,
0xed, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72,
0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75,
0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x65, 0x75, 0x72, 0x20, 0x61, 0x72,
0x20, 0x43, 0x2bc, 0x68, 0x61, 0x62, 0x2d, 0x47, 0x6c, 0x61, 0x73, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61,
0x72, 0x20, 0x43, 0x2bc, 0x68, 0x61, 0x62, 0x2d, 0x47, 0x6c, 0x61, 0x73,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20,
0x43, 0x2bc, 0x68, 0x61, 0x62, 0x2d, 0x47, 0x6c, 0x61, 0x73, 0x65, 0x75,
0x72, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x65, 0x75,
0x72, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x65, 0x75, 0x72, 0x20,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x65, 0x75, 0x72, 0x20, 0x53, 0x69, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x69, 0x6e, 0x61,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x69, 0x6e,
0x61, 0x65, 0x75, 0x72, 0x20, 0x45, 0x6e, 0x65, 0x7a, 0x20, 0x43, 0x68,
0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x65, 0x75, 0x72, 0x20, 0x49,
0x6e, 0x69, 0x7a, 0x69, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x7a, 0x65, 0x75,
0x72, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x6e, 0x69, 0x7a,
0x69, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x75,
0x62, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x65, 0x75, 0x72, 0x20, 0x44,
0x61, 0x76, 0x69, 0x73, 0x65, 0x75, 0x72, 0x20, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x65, 0x75, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x61, 0x72,
0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x45, 0x6e,
0x65, 0x7a, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x6e, 0x65, 0x7a, 0x20,
0x50, 0x61, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x45, 0x6e, 0x65, 0x7a, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x65, 0x75,
0x72, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x65, 0x75, 0x72,
0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x4b, 0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x72, 0x65, 0x69,
0x7a, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x65, 0x75, 0x72, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74,
0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x72, 0x20,
0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x72, 0x20,
0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x61, 0x6c,
0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x65, 0x75, 0x72, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc,
0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72,
0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x6e, 0x69,
0x7a, 0x69, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x6e, 0x69, 0x7a,
0x69, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x75,
0x72, 0x20, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x46, 0x69, 0x64, 0x6a, 0x69,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x46, 0x69, 0x64,
0x6a, 0x69, 0x65, 0x75, 0x72, 0x20, 0x47, 0x77, 0x69, 0x61, 0x6e, 0x61,
0x20, 0x63, 0x2bc, 0x68, 0x61, 0x6c, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x44,
0x6f, 0x75, 0x61, 0x72, 0x6f, 0xf9, 0x20, 0x61, 0x6f, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x46, 0x72, 0x61, 0xf1, 0x73, 0x20, 0x68, 0x61, 0x67,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x61, 0x65,
0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x47, 0x61, 0x6c,
0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e,
0x69, 0x7a, 0x69, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x65,
0x75, 0x72, 0x20, 0x4a, 0x6f, 0x72, 0x6a, 0x69, 0x61, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4a, 0x6f, 0x72,
0x6a, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x4a, 0x6f, 0x72, 0x6a, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e,
0x69, 0x7a, 0x69, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x41,
0x6d, 0x7a, 0x65, 0x72, 0x20, 0x6b, 0x65, 0x69, 0x74, 0x61, 0x74, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x28, 0x41,
0x4b, 0x47, 0x29, 0x65, 0x75, 0x72, 0x20, 0x47, 0x72, 0x65, 0x75, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65,
0x72, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x47, 0x72, 0x65, 0x75, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x61,
0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x47, 0x72, 0x65, 0x75, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75,
0x72, 0x20, 0x47, 0x72, 0x65, 0x75, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x47,
0x72, 0x65, 0x75, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x61, 0x72, 0x20,
0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x47, 0x72, 0x65, 0x75, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e,
0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x50, 0x6c, 0x65,
0x67, 0x2d, 0x6d, 0x6f, 0x72, 0x20, 0x41, 0x72, 0x61, 0x62, 0x2d, 0x68,
0x61, 0x2d, 0x50, 0x65, 0x72, 0x73, 0x65, 0x75, 0x72, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x20, 0x68, 0x61, 0x67, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6c,
0x65, 0x6f, 0x75, 0x74, 0x65, 0x64, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x20, 0x68, 0x61, 0x67, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6c, 0x65, 0x6f,
0x75, 0x74, 0x65, 0x64, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x20, 0x68, 0x61, 0x67, 0x20,
0x61, 0x6e, 0x20, 0x41, 0x6c, 0x65, 0x6f, 0x75, 0x74, 0x65, 0x64, 0x65,
0x75, 0x72, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x65, 0x75, 0x72, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x65, 0x75, 0x72,
0x20, 0x4d, 0x65, 0x75, 0x72, 0x76, 0x6f, 0x72, 0x20, 0x49, 0x6e, 0x64,
0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x65, 0x7a, 0x2d,
0x53, 0x69, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x72, 0x65, 0x69,
0x7a, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61,
0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x61,
0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75,
0x72, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x75, 0x72,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x61, 0x72,
0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x61, 0x72, 0x20, 0x43,
0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x65,
0x75, 0x72, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x65, 0x75, 0x72,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x4b,
0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x53, 0x72, 0x69, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x65, 0x75,
0x72, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x65, 0x75, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x75, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x65, 0x75,
0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20,
0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20,
0x61, 0x72, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x7a, 0x65,
0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x4d, 0x61, 0x72,
0x6b, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x65, 0x75, 0x72,
0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x6f, 0x72, 0x69,
0x73, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x62, 0x61, 0x73, 0x6b,
0x20, 0x6d, 0x65, 0x63, 0x2bc, 0x68, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e,
0x20, 0x48, 0x61, 0x62, 0x61, 0x73, 0x6b, 0x20, 0x6d, 0x65, 0x63, 0x2bc,
0x68, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x62, 0x61, 0x73, 0x6b, 0x20,
0x6d, 0x65, 0x63, 0x2bc, 0x68, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x65, 0x75, 0x72, 0x20, 0x4d,
0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x6f, 0x73, 0x6b,
0x6f, 0x76, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61,
0x72, 0x65, 0x75, 0x72, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x65, 0x75,
0x72, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4e, 0x65, 0x76,
0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x75, 0x72, 0x20, 0x4e,
0x69, 0x75, 0x65, 0x65, 0x75, 0x72, 0x20, 0x45, 0x6e, 0x65, 0x7a, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x6e, 0x65, 0x7a, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x45, 0x6e, 0x65, 0x7a, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e,
0x61, 0x20, 0x61, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x7a, 0x68, 0x65, 0x75,
0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x75, 0x72,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75,
0x72, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x65, 0x75, 0x72, 0x20, 0x50,
0x61, 0x70, 0x6f, 0x75, 0x61, 0x2d, 0x47, 0x69, 0x6e, 0x65, 0x61, 0x2d,
0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72, 0x20, 0x50, 0x65,
0x72, 0x6f, 0x75, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x50, 0x65, 0x72, 0x6f, 0x75, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x65, 0x72, 0x6f, 0x75, 0x65, 0x75,
0x72, 0x20, 0x61, 0x72, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x61, 0x72, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61,
0x72, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x7a, 0x65,
0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x65, 0x75, 0x72, 0x20, 0x53, 0x61, 0x6e, 0x74,
0x2d, 0x50, 0xea, 0x72, 0x2d, 0x68, 0x61, 0x2d, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x50, 0xea, 0x72, 0x2d,
0x68, 0x61, 0x2d, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6e, 0x74, 0x2d,
0x50, 0xea, 0x72, 0x2d, 0x68, 0x61, 0x2d, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65,
0x69, 0x65, 0x75, 0x72, 0x20, 0x50, 0x2bc, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x65, 0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65,
0xfc, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x52, 0x6f, 0x74,
0x68, 0x65, 0x72, 0x61, 0x65, 0x75, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x65, 0x75, 0x72,
0x20, 0x53, 0x65, 0x63, 0x68, 0x65, 0x6c, 0x65, 0x7a, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x6f, 0x75, 0x72, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e,
0x69, 0x7a, 0x69, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x61,
0x72, 0x20, 0x53, 0x75, 0x65, 0x75, 0x72, 0x20, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x65, 0x75, 0x72, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61,
0x65, 0x75, 0x72, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x65, 0x75,
0x72, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x65, 0x75, 0x72, 0x20, 0x54, 0x61, 0x64,
0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x65, 0x75, 0x72, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x65,
0x75, 0x72, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x65, 0x75, 0x72, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x65, 0x75, 0x72, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72, 0x20, 0x4f, 0x75, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4f, 0x75, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x4f, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x65, 0x75, 0x72, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65,
0x6c, 0x61, 0x65, 0x75, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x65, 0x75, 0x72, 0x20, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x75, 0x72, 0x20, 0x57, 0x61, 0x6c,
0x6c, 0x69, 0x73, 0x20, 0x68, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x65, 0x75, 0x72, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65,
0x75, 0x72, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x6f, 0x75, 0x72, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x410, 0x444, 0x433, 0x430, 0x43d, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x430, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x42e, 0x436, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430,
0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x2013, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430,
0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x438, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x437,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430,
0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x442, 0x438, 0x445, 0x43e,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x442, 0x438, 0x445,
0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x442, 0x438, 0x445,
0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x20, 0x2013,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x20,
0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x44f,
0x410, 0x43f, 0x438, 0x44f, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x43f, 0x438, 0x44f, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x440, 0x430, 0x431, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x440, 0x430, 0x431, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x440, 0x430, 0x431, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x436, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x430, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x430, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x430, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x43c, 0x435,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x43c,
0x435, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x430, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x430, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e,
0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x2013, 0x20,
0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x2013, 0x20, 0x437,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x2013, 0x20, 0x437, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430,
0x439, 0x434, 0x436, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a,
0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x2013, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x2013, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x443, 0x442, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x43e, 0x43b,
0x438, 0x432, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440,
0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441, 0x430,
0x43b, 0x430, 0x43c, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434,
0x435, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20,
0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412,
0x435, 0x440, 0x434, 0x435, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x44a, 0x43c, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x44a, 0x43c, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x44a, 0x43c, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b,
0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427,
0x438, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x427, 0x438, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41a, 0x438, 0x442, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x442, 0x430, 0x439, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x442, 0x430, 0x439, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x20, 0x420, 0x43e, 0x436, 0x434, 0x435, 0x441, 0x442, 0x432,
0x43e, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e,
0x43b, 0x443, 0x43c, 0x431, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x439, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x438, 0x20, 0x41a, 0x443, 0x43a, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x438, 0x20, 0x41a, 0x443, 0x43a, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x41a, 0x443, 0x43a,
0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431,
0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x443, 0x431, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41a, 0x443, 0x431, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x414, 0x435, 0x439, 0x432, 0x438, 0x441, 0x414, 0x44e, 0x43c,
0x43e, 0x43d, 0x20, 0x414, 0x44e, 0x440, 0x432, 0x438, 0x43b, 0x418, 0x437, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x442, 0x438, 0x43c, 0x43e, 0x440, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x435, 0x43b, 0x438, 0x43a, 0x434,
0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x412, 0x435, 0x43b, 0x438, 0x43a, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x412, 0x435, 0x43b, 0x438, 0x43a, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20, 0x2013, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e,
0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x414, 0x430, 0x43b, 0x435,
0x447, 0x43d, 0x43e, 0x20, 0x438, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435,
0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432,
0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440,
0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f,
0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x2013, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a,
0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x2013, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x434, 0x436, 0x438, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x434,
0x436, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424,
0x438, 0x434, 0x436, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x424, 0x440, 0x435, 0x43d, 0x441, 0x43a, 0x430, 0x20, 0x413,
0x432, 0x438, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x435, 0x43d, 0x441, 0x43a, 0x438,
0x20, 0x44e, 0x436, 0x43d, 0x438, 0x20, 0x438, 0x20, 0x430, 0x43d, 0x442, 0x430,
0x440, 0x43a, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x438, 0x20, 0x442, 0x435,
0x440, 0x438, 0x442, 0x43e, 0x440, 0x438, 0x438, 0x413, 0x430, 0x43b, 0x430, 0x43f,
0x430, 0x433, 0x43e, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x413, 0x430, 0x43c, 0x431, 0x438, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443,
0x437, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413,
0x440, 0x443, 0x437, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x413,
0x438, 0x43b, 0x431, 0x44a, 0x440, 0x442, 0x421, 0x440, 0x435, 0x434, 0x43d, 0x43e,
0x20, 0x433, 0x440, 0x438, 0x43d, 0x443, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x433,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x433,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x433, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x433,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x433,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x433, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x441, 0x438, 0x439, 0x441, 0x43a,
0x438, 0x20, 0x437, 0x430, 0x43b, 0x438, 0x432, 0x413, 0x430, 0x44f, 0x43d, 0x430,
0x425, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435,
0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425,
0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443,
0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432,
0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d,
0x43a, 0x43e, 0x43d, 0x433, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x425, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432,
0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e,
0x432, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e,
0x432, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x418, 0x43d, 0x434, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x43d, 0x434, 0x438, 0x439, 0x441, 0x43a, 0x438, 0x20, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x442, 0x430,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x438, 0x43d, 0x434, 0x43e, 0x43d,
0x435, 0x437, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x438, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x438, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42f, 0x43f, 0x43e,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42f, 0x43f,
0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42f, 0x43f,
0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441,
0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430,
0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430,
0x442, 0x441, 0x43a, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442,
0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a,
0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x20, 0x2013, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441,
0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x43a, 0x430, 0x437, 0x430, 0x445,
0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x448, 0x440, 0x430,
0x439, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x44f, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x437, 0x441, 0x442,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x415,
0x43a, 0x432, 0x430, 0x442, 0x43e, 0x440, 0x438, 0x430, 0x43b, 0x43d, 0x438, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41b, 0x43e, 0x440, 0x434, 0x20,
0x425, 0x430, 0x443, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x20,
0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425,
0x430, 0x443, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440, 0x43a, 0x438,
0x437, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41c,
0x430, 0x440, 0x448, 0x430, 0x43b, 0x43e, 0x432, 0x438, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x439,
0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x439, 0x20, 0x2013, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x439, 0x20,
0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x443, 0x441,
0x44a, 0x43d, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x442, 0x438, 0x445, 0x43e,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d,
0x431, 0x430, 0x442, 0x43e, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442, 0x43e, 0x440, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x431,
0x430, 0x442, 0x43e, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x43e,
0x432, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x438, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e,
0x43a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x437, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41d, 0x43e, 0x432, 0x43e, 0x437, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x437, 0x435,
0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d,
0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x44e,
0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b,
0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434,
0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44f, 0x424, 0x435, 0x440, 0x43d,
0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e,
0x43d, 0x44f, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x44f, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20,
0x41d, 0x43e, 0x432, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435, 0x44f, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x424, 0x435, 0x43d, 0x438, 0x43a,
0x441, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x438, 0x435, 0x440, 0x20, 0x438, 0x20,
0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x421, 0x435, 0x43d, 0x20, 0x41f,
0x438, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20,
0x41f, 0x438, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b,
0x43e, 0x43d, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f,
0x445, 0x435, 0x43d, 0x44f, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x420, 0x435, 0x44e, 0x43d, 0x438, 0x43e, 0x43d, 0x421, 0x430, 0x445,
0x430, 0x43b, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c,
0x430, 0x440, 0x430, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430,
0x43c, 0x430, 0x440, 0x430, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435,
0x439, 0x448, 0x435, 0x43b, 0x438, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443,
0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x43e,
0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x43e, 0x432, 0x438, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x42e, 0x436, 0x43d, 0x430, 0x20, 0x414, 0x436, 0x43e,
0x440, 0x434, 0x436, 0x438, 0x44f, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x438,
0x442, 0x44f, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x422, 0x430, 0x439, 0x43f, 0x435, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x20, 0x2013,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x20, 0x2013,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x434, 0x436, 0x438,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x2013, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x2013, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x443, 0x44e, 0x43a, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435,
0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d,
0x443, 0x430, 0x442, 0x443, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412,
0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x412, 0x435, 0x43d, 0x435, 0x446, 0x443, 0x435, 0x43b, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434,
0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412,
0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20, 0x423, 0x435, 0x439,
0x43a, 0x423, 0x43e, 0x43b, 0x438, 0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442,
0x443, 0x43d, 0x430, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x415, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x415, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x42e, 0x43a, 0x43e, 0x43d, 0x1021, 0x102c, 0x1016, 0x1002, 0x1014, 0x103a, 0x1014,
0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1021, 0x102c, 0x1016, 0x101b, 0x102d, 0x1000, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1021,
0x102c, 0x1016, 0x101b, 0x102d, 0x1000, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1021, 0x102c, 0x1016, 0x101b, 0x102d, 0x1000, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a,
0x1021, 0x102c, 0x1016, 0x101b, 0x102d, 0x1000, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1021, 0x102c, 0x1016, 0x101b, 0x102d,
0x1000, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1021, 0x102c, 0x1016, 0x101b, 0x102d,
0x1000, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x102c, 0x1005, 0x1000, 0x102c, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x102c, 0x1005, 0x1000, 0x102c, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x101c, 0x102c, 0x1005, 0x1000, 0x102c, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x1019, 0x1031, 0x1007, 0x102f, 0x1036, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x1019, 0x1031, 0x1007, 0x102f, 0x1036, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1019, 0x1031, 0x1007,
0x102f, 0x1036, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004,
0x103a, 0x1038, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a,
0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f,
0x1004, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b,
0x103e, 0x1031, 0x1037, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1015, 0x102d, 0x102f, 0x1004,
0x103a, 0x1038, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a,
0x1010, 0x1014, 0x103a, 0x1038, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x1031,
0x102c, 0x1004, 0x103a, 0x1010, 0x1014, 0x103a, 0x1038, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1010,
0x1014, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x1005,
0x102d, 0x1016, 0x102d, 0x1010, 0x103a, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015,
0x1005, 0x102d, 0x1016, 0x102d, 0x1010, 0x103a, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x1005, 0x102d, 0x1016, 0x102d, 0x1010, 0x103a,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1015, 0x102e, 0x101a, 0x102c,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1015, 0x102e, 0x101a, 0x102c,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x1015, 0x102e, 0x101a, 0x102c, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x101b, 0x1031,
0x1017, 0x103b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x101b,
0x1031, 0x1017, 0x103b, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x101b, 0x1031, 0x1017, 0x103b, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x102c, 0x1002, 0x103b, 0x1004, 0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x1002, 0x103b, 0x1004,
0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x1002, 0x103b, 0x1004,
0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031,
0x102c, 0x1000, 0x103a, 0x20, 0x1021, 0x102c, 0x1002, 0x103b, 0x1004, 0x103a, 0x1010, 0x102e,
0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20, 0x1021, 0x102c, 0x1002, 0x103b, 0x1004, 0x103a,
0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a,
0x20, 0x1021, 0x102c, 0x1002, 0x103b, 0x1004, 0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c,
0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x1019, 0x1031, 0x1038, 0x1014, 0x102e, 0x1038,
0x101a, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c,
0x1019, 0x1031, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x1019,
0x1031, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1010, 0x1039, 0x1010, 0x101c, 0x1014, 0x103a, 0x1010, 0x1005, 0x103a, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1010, 0x1039, 0x1010, 0x101c, 0x1014, 0x103a, 0x1010,
0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x1010, 0x1039, 0x1010, 0x101c, 0x1014, 0x103a, 0x1010, 0x1005, 0x103a,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1029, 0x1005, 0x1010, 0x103c,
0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f,
0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1029, 0x1005,
0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015,
0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c,
0x103b, 0x20, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x101e, 0x103c, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b,
0x102c, 0x1038, 0x20, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1021, 0x101c, 0x101a,
0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x101e, 0x103c, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x102c,
0x1038, 0x20, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a,
0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101e, 0x103c, 0x1005, 0x1010, 0x103c, 0x1031,
0x1038, 0x101c, 0x103b, 0x102c, 0x1038, 0x20, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a,
0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038,
0x101c, 0x103b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e,
0x1031, 0x1037, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b,
0x103e, 0x1031, 0x1037, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1029, 0x1005, 0x1010, 0x103c,
0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c,
0x103b, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x1007, 0x102c, 0x1018, 0x102d, 0x102f, 0x1004, 0x103a, 0x1002, 0x103b, 0x1014,
0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1007, 0x102c, 0x1018,
0x102d, 0x102f, 0x1004, 0x103a, 0x1002, 0x103b, 0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1007, 0x102c, 0x1018,
0x102d, 0x102f, 0x1004, 0x103a, 0x1002, 0x103b, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1031, 0x1007, 0x102d, 0x102f, 0x1038, 0x101b, 0x102e, 0x1038, 0x1005, 0x103a, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1031, 0x1007, 0x102d, 0x102f, 0x1038, 0x101b,
0x102e, 0x1038, 0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1031, 0x1007, 0x102d, 0x102f, 0x1038, 0x101b, 0x102e,
0x1038, 0x1005, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x1004, 0x103a, 0x1039, 0x1002, 0x101c,
0x102c, 0x1038, 0x1012, 0x1031, 0x1037, 0x101b, 0x103e, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1018, 0x1004, 0x103a, 0x1039, 0x1002, 0x101c, 0x102c, 0x1038, 0x1012,
0x1031, 0x1037, 0x101b, 0x103e, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x1004, 0x103a, 0x1039, 0x1002, 0x101c, 0x102c,
0x1038, 0x1012, 0x1031, 0x1037, 0x101b, 0x103e, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x1030,
0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x102d,
0x102f, 0x101c, 0x102e, 0x1038, 0x1018, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x101b, 0x102c, 0x1007, 0x102e, 0x1038, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x101b, 0x102c, 0x1007, 0x102e, 0x1038,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1018, 0x101b, 0x102c, 0x1007, 0x102e, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x101b, 0x1030,
0x1014, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x1010, 0x103a, 0x1017, 0x102c,
0x1012, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x1010,
0x103a, 0x1017, 0x102c, 0x1012, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x1010, 0x103a, 0x1017, 0x102c, 0x1012,
0x102e, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102c, 0x1019, 0x102d, 0x102f, 0x101b, 0x102d,
0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102c, 0x1038,
0x101e, 0x1019, 0x103a, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102c,
0x1038, 0x101e, 0x1019, 0x103a, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102c, 0x1038, 0x101e, 0x1019, 0x103a, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1001, 0x103b, 0x102e, 0x101c, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1001, 0x103b, 0x102e, 0x101c, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102e, 0x101c, 0x102e, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1010, 0x101b, 0x102f, 0x1010, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1010, 0x101b, 0x102f, 0x1010, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x101b, 0x102f, 0x1010, 0x103a,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1001, 0x101b, 0x1005, 0x103a, 0x1005, 0x1019, 0x1010, 0x103a, 0x1000,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1000, 0x102d, 0x102f, 0x1000, 0x102d, 0x102f, 0x1038, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a,
0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x101c,
0x1036, 0x1018, 0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1000, 0x102d, 0x102f, 0x101c, 0x1036, 0x1018, 0x102e, 0x101a, 0x102c, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f,
0x101c, 0x1036, 0x1018, 0x102e, 0x101a, 0x102c, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103d, 0x1010,
0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103d, 0x1010, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014,
0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103d, 0x1010, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014,
0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103b, 0x1030, 0x1038, 0x1018,
0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103b, 0x1030,
0x1038, 0x1018, 0x102c, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103b, 0x1030, 0x1038, 0x1018, 0x102c, 0x1038, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1012, 0x1031, 0x1038, 0x1017, 0x1005,
0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1012, 0x1030, 0x1038, 0x1019,
0x1031, 0x102c, 0x1037, 0x1010, 0x103a, 0x20, 0x1012, 0x102b, 0x101b, 0x103a, 0x1017, 0x102e,
0x1038, 0x101c, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b,
0x103e, 0x1031, 0x1037, 0x1010, 0x102e, 0x1019, 0x1031, 0x102c, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x102e, 0x1005, 0x1010, 0x102c, 0x1000, 0x103b, 0x103d, 0x1014,
0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e, 0x1005,
0x1010, 0x102c, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e, 0x1005, 0x1010,
0x102c, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e,
0x1000, 0x103d, 0x1031, 0x1012, 0x1031, 0x102b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d,
0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025,
0x101b, 0x1031, 0x102c, 0x1015, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004,
0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015,
0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031,
0x1037, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x101b, 0x103e, 0x1031, 0x1037, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1016, 0x103b, 0x102c, 0x1038, 0x20, 0x1025, 0x101b,
0x1031, 0x102c, 0x1015, 0x1012, 0x1031, 0x101e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000,
0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a,
0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x1031, 0x102c, 0x1037,
0x1000, 0x101c, 0x1014, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x1031, 0x102c, 0x1037, 0x1000,
0x101c, 0x1014, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016,
0x1031, 0x102c, 0x1037, 0x1000, 0x101c, 0x1014, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a,
0x1038, 0x1005, 0x102f, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102e, 0x1002, 0x103b, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102e, 0x1002, 0x103b, 0x102e, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016,
0x102e, 0x1002, 0x103b, 0x102e, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x103c, 0x1004, 0x103a, 0x101e,
0x1005, 0x103a, 0x20, 0x1002, 0x102e, 0x1021, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x103c, 0x1004, 0x103a, 0x101e, 0x1005,
0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038,
0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x20, 0x1021, 0x1014, 0x1039, 0x1010, 0x102c, 0x1010,
0x102d, 0x1010, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x102b,
0x101c, 0x102c, 0x1038, 0x1015, 0x102b, 0x1002, 0x102d, 0x102f, 0x1038, 0x1005, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x1019, 0x103a, 0x1018, 0x102e, 0x101a,
0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x1031, 0x102c,
0x103a, 0x1002, 0x103b, 0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1002, 0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b, 0x102e, 0x101a, 0x102c, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002,
0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b, 0x102e, 0x101a, 0x102c, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1002, 0x102e, 0x101c, 0x103a, 0x1018, 0x1010, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a,
0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x101b,
0x1004, 0x103a, 0x1038, 0x1014, 0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1002,
0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1002, 0x101b, 0x1004, 0x103a,
0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1002, 0x101b,
0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20, 0x1002, 0x101b, 0x1004,
0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20, 0x1002, 0x101b, 0x1004, 0x103a,
0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20,
0x1002, 0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x1004, 0x103a, 0x101c, 0x101a, 0x103a, 0x1000, 0x103d,
0x1031, 0x1037, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x102d, 0x102f,
0x1004, 0x103a, 0x101a, 0x102c, 0x1014, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x101f, 0x102c, 0x101d, 0x102d, 0x102f, 0x1004, 0x103a, 0x101a, 0x102e, 0x20, 0x1021,
0x101a, 0x103a, 0x101c, 0x1030, 0x1038, 0x101b, 0x103e, 0x1014, 0x103a, 0x1038, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x102c, 0x101d, 0x102d, 0x102f, 0x1004, 0x103a,
0x101a, 0x102e, 0x20, 0x1021, 0x101a, 0x103a, 0x101c, 0x1030, 0x1038, 0x101b, 0x103e, 0x1014,
0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x101f, 0x102c, 0x101d, 0x102d, 0x102f, 0x1004, 0x103a, 0x101a, 0x102e, 0x20,
0x1021, 0x101a, 0x103a, 0x101c, 0x1030, 0x1038, 0x101b, 0x103e, 0x1014, 0x103a, 0x1038, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a,
0x1000, 0x1031, 0x102c, 0x1004, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x1000, 0x1031, 0x102c, 0x1004, 0x103a, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x1031,
0x102c, 0x1004, 0x103a, 0x1000, 0x1031, 0x102c, 0x1004, 0x103a, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f,
0x102d, 0x102f, 0x1038, 0x1017, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x101f, 0x102d, 0x102f, 0x1038, 0x1017, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x102d, 0x102f, 0x1038, 0x1017, 0x103a,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x102d, 0x1014, 0x1039, 0x1012, 0x102d, 0x101a, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102d,
0x1014, 0x1039, 0x1012, 0x102d, 0x101a, 0x101e, 0x1019, 0x102f, 0x1012, 0x1039, 0x1012, 0x101b,
0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1004, 0x103a, 0x1012,
0x102d, 0x102f, 0x1001, 0x103b, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1014, 0x102c, 0x1038,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015,
0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102d, 0x102f,
0x1014, 0x102e, 0x1038, 0x101b, 0x103e, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a,
0x1038, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102d, 0x102f, 0x1014, 0x102e, 0x1038, 0x101b,
0x103e, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014,
0x1031, 0x102c, 0x1000, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021,
0x1004, 0x103a, 0x1012, 0x102d, 0x102f, 0x1014, 0x102e, 0x1038, 0x101b, 0x103e, 0x102c, 0x1038,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e, 0x101b, 0x1014, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e, 0x101b, 0x1014, 0x103a,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x102e, 0x101b, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e, 0x101b, 0x1030,
0x1001, 0x1030, 0x1010, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x102e, 0x101b, 0x1030, 0x1001, 0x1030, 0x1010, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e, 0x101b, 0x1030, 0x1001,
0x1030, 0x1010, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1005, 0x1039, 0x1005, 0x101b, 0x1031,
0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1005, 0x1039, 0x1005,
0x101b, 0x1031, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x1005, 0x1039, 0x1005, 0x101b, 0x1031, 0x1038, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1002, 0x103b, 0x1015, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1002, 0x103b, 0x1015, 0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x1015, 0x1014, 0x103a, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1000, 0x102c, 0x1007, 0x1000, 0x103a, 0x1005, 0x1010, 0x1014, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1000,
0x102c, 0x1007, 0x1000, 0x103a, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1000, 0x102c, 0x1007,
0x1000, 0x103a, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1000, 0x102d, 0x102f, 0x101b, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x101b, 0x102e, 0x1038, 0x101a,
0x102c, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x101b, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1001, 0x102d, 0x102f, 0x1005, 0x103a, 0x101b, 0x102d, 0x102f, 0x1004, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x101b, 0x102c, 0x1037, 0x1005,
0x103a, 0x1014, 0x102d, 0x102f, 0x101b, 0x102c, 0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1001, 0x101b, 0x102c, 0x1037, 0x1005, 0x103a, 0x1014, 0x102d, 0x102f,
0x101b, 0x102c, 0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x101b, 0x102c, 0x1037, 0x1005, 0x103a, 0x1014, 0x102d,
0x102f, 0x101b, 0x102c, 0x1005, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102c, 0x1002, 0x103b,
0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x101e, 0x102e, 0x101b, 0x102d, 0x101c, 0x1004, 0x103a, 0x1039, 0x1000, 0x102c, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101c, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038,
0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x101c, 0x1031, 0x102c, 0x1037, 0x1012, 0x103a, 0x101f, 0x1031, 0x102c,
0x1004, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101c, 0x1031, 0x102c,
0x1037, 0x1012, 0x103a, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101c, 0x1031, 0x102c, 0x1037,
0x1012, 0x103a, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1000,
0x102c, 0x1021, 0x102d, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019,
0x1000, 0x102c, 0x1021, 0x102d, 0x102f, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1000, 0x102c, 0x1021, 0x102d, 0x102f, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1019, 0x102c, 0x1002, 0x102b, 0x1012, 0x1014, 0x103a, 0x1038, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x102c, 0x1002, 0x102b, 0x1012, 0x1014, 0x103a,
0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1019, 0x102c, 0x1002, 0x102b, 0x1012, 0x1014, 0x103a, 0x1038, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1019, 0x101c, 0x1031, 0x1038, 0x101b, 0x103e, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a, 0x101c, 0x1012, 0x102d, 0x102f, 0x1000,
0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x102c, 0x1001, 0x1031,
0x1038, 0x1021, 0x1015, 0x103a, 0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1019, 0x102c, 0x101b, 0x103e, 0x101a, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a,
0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031,
0x102c, 0x101b, 0x1005, 0x103a, 0x101b, 0x103e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1019, 0x1031, 0x102c, 0x101b, 0x1005, 0x103a, 0x101b, 0x103e, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c,
0x101b, 0x1005, 0x103a, 0x101b, 0x103e, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a,
0x1005, 0x103d, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019,
0x1000, 0x1039, 0x1000, 0x1006, 0x102e, 0x1000, 0x1014, 0x103a, 0x20, 0x1015, 0x1005, 0x102d,
0x1016, 0x102d, 0x1010, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019,
0x1000, 0x1039, 0x1000, 0x1006, 0x102e, 0x1000, 0x1014, 0x103a, 0x20, 0x1015, 0x1005, 0x102d,
0x1016, 0x102d, 0x1010, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1000, 0x1039, 0x1000, 0x1005, 0x102e, 0x1000, 0x1014,
0x103a, 0x20, 0x1015, 0x1005, 0x102d, 0x1016, 0x102d, 0x1010, 0x103a, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101c, 0x1014, 0x103a, 0x1018, 0x102c, 0x1010,
0x1031, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101c, 0x1014,
0x103a, 0x1018, 0x102c, 0x1010, 0x1031, 0x102c, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101c, 0x1014, 0x103a, 0x1018, 0x102c,
0x1010, 0x1031, 0x102c, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a, 0x1005, 0x1000,
0x102d, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c,
0x103a, 0x1005, 0x1000, 0x102d, 0x102f, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a, 0x1005, 0x1000, 0x102d,
0x102f, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x103c, 0x1014, 0x103a, 0x1019, 0x102c, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102c, 0x1025, 0x1030, 0x101b, 0x1030, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102e, 0x1015, 0x1031, 0x102b, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1000, 0x101a,
0x103a, 0x101c, 0x102e, 0x1012, 0x102d, 0x102f, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c,
0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038,
0x1000, 0x102c, 0x101c, 0x102e, 0x1012, 0x102d, 0x102f, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a,
0x102c, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1000, 0x102c, 0x101c, 0x102e, 0x1012, 0x102d,
0x102f, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014,
0x101a, 0x1030, 0x1038, 0x1007, 0x102e, 0x101c, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1007, 0x102e, 0x101c, 0x1014, 0x103a,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1014, 0x101a, 0x1030, 0x1038, 0x1007, 0x102e, 0x101c, 0x1014, 0x103a, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1014, 0x101a, 0x1030, 0x1038, 0x1016, 0x1031, 0x102c, 0x1004, 0x103a, 0x101c, 0x1014, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1016,
0x1031, 0x102c, 0x1004, 0x103a, 0x101c, 0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1016,
0x1031, 0x102c, 0x1004, 0x103a, 0x101c, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1014, 0x102e, 0x1025, 0x1030, 0x1021, 0x1031, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x1031, 0x102c, 0x1016, 0x1031, 0x102c, 0x1037, 0x1001,
0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1014, 0x1031, 0x102c, 0x1016, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x1000,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x1031, 0x102c, 0x1016, 0x1031, 0x102c, 0x1037,
0x1001, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102c, 0x1014, 0x1014, 0x103a, 0x1012, 0x102d, 0x102f,
0x1038, 0x1012, 0x102e, 0x1014, 0x102d, 0x102f, 0x1038, 0x101b, 0x102d, 0x102f, 0x1038, 0x100a,
0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102c, 0x1014, 0x1014,
0x103a, 0x1012, 0x102d, 0x102f, 0x1038, 0x1012, 0x102e, 0x1014, 0x102d, 0x102f, 0x1038, 0x101b,
0x102d, 0x102f, 0x1038, 0x100a, 0x102c, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102c, 0x1014, 0x1014, 0x103a, 0x1012, 0x102d,
0x102f, 0x1038, 0x1012, 0x102e, 0x1014, 0x102d, 0x102f, 0x1038, 0x101b, 0x102d, 0x102f, 0x1038,
0x100a, 0x102c, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102d, 0x102f, 0x1017, 0x102d, 0x102f, 0x1005,
0x1032, 0x1018, 0x102e, 0x1021, 0x1032, 0x101a, 0x103a, 0x1005, 0x103a, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102d, 0x102f, 0x1017, 0x102d, 0x102f, 0x1005, 0x1032,
0x1018, 0x102e, 0x1021, 0x1032, 0x101a, 0x103a, 0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102d, 0x102f, 0x1017,
0x102d, 0x102f, 0x1005, 0x1032, 0x1018, 0x102e, 0x1021, 0x1032, 0x101a, 0x103a, 0x1005, 0x103a,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x103d, 0x1019, 0x103a, 0x1038, 0x1005, 0x103a, 0x1001, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x103d, 0x1019, 0x103a, 0x1038,
0x1005, 0x103a, 0x1001, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x103d, 0x1019, 0x103a, 0x1038, 0x1005, 0x103a, 0x1001,
0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014,
0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1000, 0x1005,
0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010,
0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x101c, 0x102c, 0x1021, 0x102d, 0x102f, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1015, 0x1030, 0x1021, 0x102c,
0x1014, 0x101a, 0x1030, 0x1038, 0x1002, 0x102e, 0x1014, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x101b, 0x102c, 0x1002, 0x103d, 0x1031, 0x1038, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x101b, 0x102c, 0x1002, 0x103d,
0x1031, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1015, 0x102b, 0x101b, 0x102c, 0x1002, 0x103d, 0x1031, 0x1038, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1015, 0x102e, 0x101b, 0x1030, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1015, 0x102e, 0x101b, 0x1030, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102e, 0x101b, 0x1030, 0x1038, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1016, 0x102d, 0x101c, 0x1005, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102d, 0x101c, 0x1005, 0x103a,
0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102d, 0x101c, 0x1005, 0x103a, 0x1015, 0x102d,
0x102f, 0x1004, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102e, 0x1038, 0x1014, 0x1005, 0x103a,
0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1015, 0x102e, 0x1021, 0x1032,
0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x1019, 0x102e, 0x1001, 0x103d, 0x102e, 0x101c, 0x103d,
0x1014, 0x103a, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037,
0x103a, 0x1015, 0x102e, 0x1021, 0x1032, 0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x1019, 0x102e,
0x1001, 0x103d, 0x102e, 0x101c, 0x103d, 0x1014, 0x103a, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1015,
0x102e, 0x1021, 0x1032, 0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x20, 0x1019, 0x102e, 0x1001,
0x103d, 0x102e, 0x101c, 0x103d, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1015, 0x102b, 0x1010, 0x103a, 0x1000, 0x101a, 0x103a, 0x101b, 0x1004, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102d, 0x102f, 0x1014, 0x102c,
0x1016, 0x1032, 0x1021, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015,
0x103c, 0x102f, 0x1036, 0x101a, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x101b, 0x102e, 0x101a, 0x1030, 0x1014, 0x102e, 0x101a, 0x1036, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101b, 0x102d, 0x102f, 0x101e, 0x102e, 0x101b, 0x102c,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x102c, 0x1001, 0x102b, 0x101c,
0x1004, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x102c, 0x1001,
0x102b, 0x101c, 0x1004, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x102c, 0x1001, 0x102b, 0x101c, 0x1004, 0x103a, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1006, 0x1019, 0x102d, 0x102f, 0x1021, 0x102c, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1006, 0x1019, 0x102d, 0x102f, 0x1021, 0x102c, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x1019, 0x102d,
0x102f, 0x1038, 0x1021, 0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x1031, 0x1038, 0x101b,
0x103e, 0x1032, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1005, 0x1004, 0x103a,
0x1039, 0x1000, 0x102c, 0x1015, 0x1030, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1006, 0x1031, 0x102c, 0x103a, 0x101c, 0x1019, 0x103d, 0x1014, 0x103a, 0x1000, 0x103b, 0x103d,
0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1002, 0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b,
0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1005, 0x1030,
0x1038, 0x101b, 0x102e, 0x1014, 0x102c, 0x1019, 0x103a, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x101b, 0x103e, 0x102d, 0x102f, 0x101d, 0x102b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1010, 0x101f, 0x102e, 0x1010, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1011, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1031, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1011, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1031, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1011,
0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1031, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x102c, 0x1002,
0x103b, 0x1005, 0x103a, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x102d, 0x102f, 0x1000, 0x102e, 0x101c, 0x102c,
0x1025, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x103d, 0x1014, 0x103a,
0x1002, 0x102b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x103d, 0x1014,
0x103a, 0x1002, 0x102b, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1010, 0x103d, 0x1014, 0x103a, 0x1002, 0x102b, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1001, 0x103b, 0x102f, 0x1001, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1010, 0x102c, 0x1037, 0x1001, 0x103a, 0x1019, 0x1004, 0x103a, 0x1014, 0x1005, 0x1039, 0x1005,
0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x102c,
0x1037, 0x1001, 0x103a, 0x1019, 0x1004, 0x103a, 0x1014, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014,
0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1010, 0x102c, 0x1037, 0x1001, 0x103a, 0x1019, 0x1004, 0x103a, 0x1014, 0x1005, 0x1039,
0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x1030, 0x1017, 0x102c, 0x1038,
0x101c, 0x1030, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101b, 0x102f,
0x1002, 0x103d, 0x1031, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025,
0x101b, 0x102f, 0x1002, 0x103d, 0x1031, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101b, 0x102f, 0x1002, 0x103d, 0x1031,
0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x1007, 0x1018, 0x1000, 0x103a, 0x1000, 0x1005, 0x1039,
0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025,
0x1007, 0x1018, 0x1000, 0x103a, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025,
0x1007, 0x1018, 0x1000, 0x103a, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1017, 0x1014, 0x103d, 0x102c, 0x1038, 0x1010, 0x1030, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x1014, 0x103d, 0x102c, 0x1038, 0x1010, 0x1030, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017,
0x1014, 0x103d, 0x102c, 0x1038, 0x1010, 0x1030, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x1004, 0x103a,
0x1014, 0x102e, 0x1007, 0x103d, 0x1032, 0x101c, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1017, 0x101c, 0x102c, 0x1012, 0x102e, 0x1017, 0x1031, 0x102c, 0x1037,
0x1005, 0x1010, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1017, 0x101c, 0x102c, 0x1012, 0x102e, 0x1017, 0x1031, 0x102c, 0x1037, 0x1005,
0x1010, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x101c, 0x102c, 0x1012, 0x102e, 0x1017,
0x1031, 0x102c, 0x1037, 0x1005, 0x1010, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1017, 0x102d, 0x102f, 0x101c, 0x103a, 0x1002, 0x102d, 0x102f, 0x1002, 0x101b, 0x1000,
0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x102d, 0x102f, 0x101c,
0x103a, 0x1002, 0x102d, 0x102f, 0x1002, 0x101b, 0x1000, 0x103a, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x102d, 0x102f, 0x1002,
0x102d, 0x102f, 0x1002, 0x101b, 0x1000, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x102d, 0x102f,
0x1005, 0x103a, 0x1010, 0x102d, 0x102f, 0x1001, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x101d, 0x102d, 0x1010, 0x103a, 0x1001, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014,
0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101d, 0x1031, 0x102b,
0x101c, 0x102e, 0x1005, 0x103a, 0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x20, 0x1016, 0x1030,
0x1000, 0x103b, 0x1030, 0x1014, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x101a, 0x1030, 0x1001, 0x1030, 0x1038, 0x1010, 0x103a, 0x1005, 0x103a, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x101a, 0x1030, 0x1001, 0x1030, 0x1038, 0x1010, 0x103a, 0x1005,
0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x101a, 0x1030, 0x1001, 0x1030, 0x1038, 0x1010, 0x103a, 0x1005, 0x103a, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x101b, 0x101a, 0x103a, 0x1001, 0x102b, 0x1038, 0x1010, 0x102e, 0x101b, 0x1004, 0x103a,
0x1018, 0x102c, 0x1038, 0x1001, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x101b, 0x101a, 0x103a, 0x1001, 0x102b, 0x1038, 0x1010, 0x102e, 0x101b, 0x1004, 0x103a, 0x1018,
0x102c, 0x1038, 0x1001, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x101b, 0x101a, 0x103a, 0x1001, 0x102b, 0x1038, 0x1010, 0x102e,
0x101b, 0x1004, 0x103a, 0x1018, 0x102c, 0x1001, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101a, 0x1030,
0x1000, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x827e, 0x514b, 0x6642, 0x9593, 0x827e, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x827e, 0x514b,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x5bcc, 0x6c57, 0x6642, 0x9593, 0x4e2d, 0x975e, 0x6642,
0x9593, 0x6771, 0x975e, 0x6642, 0x9593, 0x5357, 0x975e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x897f,
0x975e, 0x6642, 0x9593, 0x897f, 0x975e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x897f, 0x975e, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x963f, 0x62c9, 0x65af, 0x52a0, 0x6642, 0x9593, 0x963f, 0x62c9, 0x65af,
0x52a0, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x62c9, 0x65af, 0x52a0, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x963f, 0x62c9, 0x6728, 0x5716, 0x6642, 0x9593, 0x963f, 0x62c9, 0x6728, 0x5716, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x963f, 0x62c9, 0x6728, 0x5716, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e9e,
0x99ac, 0x905c, 0x6642, 0x9593, 0x4e9e, 0x99ac, 0x905c, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e9e,
0x99ac, 0x905c, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e2d, 0x90e8, 0x6642, 0x9593, 0x4e2d, 0x90e8,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6771, 0x90e8,
0x6642, 0x9593, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6771, 0x90e8, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x5c71, 0x5340, 0x6642, 0x9593, 0x5c71, 0x5340, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x5c71, 0x5340, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x592a,
0x5e73, 0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x592a, 0x5e73, 0x6d0b, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x963f, 0x7d0d, 0x5fb7, 0x723e, 0x6642, 0x9593, 0x963f, 0x90a3, 0x5e95, 0x6cb3, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x963f, 0x90a3, 0x5e95, 0x6cb3, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f,
0x76ae, 0x4e9e, 0x6642, 0x9593, 0x963f, 0x76ae, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f,
0x76ae, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x514b, 0x9676, 0x6642, 0x9593, 0x963f,
0x514b, 0x9676, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x514b, 0x9676, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x6642, 0x9593, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f,
0x62c9, 0x4f2f, 0x6642, 0x9593, 0x963f, 0x62c9, 0x4f2f, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f,
0x62c9, 0x4f2f, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7, 0x6642, 0x9593, 0x963f,
0x6839, 0x5ef7, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x963f, 0x6839, 0x5ef7, 0x897f, 0x90e8, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7, 0x897f,
0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7, 0x897f, 0x90e8, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x4e9e, 0x7f8e, 0x5c3c, 0x4e9e, 0x6642, 0x9593, 0x4e9e, 0x7f8e, 0x5c3c, 0x4e9e,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e9e, 0x7f8e, 0x5c3c, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x5927, 0x897f, 0x6d0b, 0x6642, 0x9593, 0x5927, 0x897f, 0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x5927, 0x897f, 0x6d0b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x6642,
0x9593, 0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x4e2d,
0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x6642, 0x9593,
0x6fb3, 0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x4e2d,
0x897f, 0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x6771, 0x90e8, 0x6642, 0x9593,
0x6fb3, 0x6d32, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x6771, 0x90e8,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x897f, 0x90e8, 0x6642, 0x9593, 0x6fb3, 0x6d32,
0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x897f, 0x90e8, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7136, 0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7136,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7136, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x4e9e, 0x901f, 0x723e, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x4e9e, 0x901f, 0x723e, 0x7fa4, 0x5cf6,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e9e, 0x901f, 0x723e, 0x7fa4, 0x5cf6, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x5b5f, 0x52a0, 0x62c9, 0x6642, 0x9593, 0x5b5f, 0x52a0, 0x62c9, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x5b5f, 0x52a0, 0x62c9, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e0d, 0x4e39, 0x6642, 0x9593,
0x73bb, 0x5229, 0x7dad, 0x4e9e, 0x6642, 0x9593, 0x5df4, 0x897f, 0x5229, 0x4e9e, 0x6642, 0x9593,
0x5df4, 0x897f, 0x5229, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5df4, 0x897f, 0x5229, 0x4e9e,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x6c76, 0x840a, 0x6642, 0x9593, 0x7dad, 0x5fb7, 0x89d2, 0x6642,
0x9593, 0x7dad, 0x5fb7, 0x89d2, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x7dad, 0x5fb7, 0x89d2, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x51f1, 0x897f, 0x7ad9, 0x6642, 0x9593, 0x67e5, 0x83ab, 0x6d1b, 0x6642,
0x9593, 0x67e5, 0x5766, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x67e5, 0x5766, 0x7fa4, 0x5cf6, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x67e5, 0x5766, 0x7fa4, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x667a,
0x5229, 0x6642, 0x9593, 0x667a, 0x5229, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x667a, 0x5229, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x4e2d, 0x570b, 0x6642, 0x9593, 0x4e2d, 0x570b, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x4e2d, 0x570b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x8056, 0x8a95, 0x5cf6, 0x6642, 0x9593,
0x79d1, 0x79d1, 0x65af, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x54e5, 0x502b, 0x6bd4, 0x4e9e, 0x6642,
0x9593, 0x54e5, 0x502b, 0x6bd4, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x54e5, 0x502b, 0x6bd4,
0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5eab, 0x514b, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x5eab,
0x514b, 0x7fa4, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5eab, 0x514b, 0x7fa4, 0x5cf6, 0x534a,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x53e4, 0x5df4, 0x6642, 0x9593, 0x53e4, 0x5df4, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x53e4, 0x5df4, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6234, 0x7dad, 0x65af, 0x6642,
0x9593, 0x675c, 0x8499, 0x675c, 0x6bd4, 0x723e, 0x6642, 0x9593, 0x6771, 0x5e1d, 0x6c76, 0x6642,
0x9593, 0x5fa9, 0x6d3b, 0x7bc0, 0x5cf6, 0x6642, 0x9593, 0x5fa9, 0x6d3b, 0x7bc0, 0x5cf6, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x5fa9, 0x6d3b, 0x7bc0, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5384,
0x74dc, 0x591a, 0x6642, 0x9593, 0x4e2d, 0x6b50, 0x6642, 0x9593, 0x4e2d, 0x6b50, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x4e2d, 0x6b50, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6771, 0x6b50, 0x6642, 0x9593,
0x6771, 0x6b50, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6771, 0x6b50, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x6b50, 0x6d32, 0x9060, 0x6771, 0x6642, 0x9593, 0x897f, 0x6b50, 0x6642, 0x9593, 0x897f, 0x6b50,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x897f, 0x6b50, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x798f, 0x514b,
0x862d, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x798f, 0x514b, 0x862d, 0x7fa4, 0x5cf6, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x798f, 0x514b, 0x862d, 0x7fa4, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6590,
0x6fdf, 0x6642, 0x9593, 0x6590, 0x6fdf, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6590, 0x6fdf, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x6cd5, 0x5c6c, 0x572d, 0x4e9e, 0x90a3, 0x6642, 0x9593, 0x6cd5, 0x570b,
0x5357, 0x65b9, 0x53ca, 0x5357, 0x6975, 0x6642, 0x9593, 0x52a0, 0x62c9, 0x5df4, 0x54e5, 0x7fa4,
0x5cf6, 0x6642, 0x9593, 0x7518, 0x6bd4, 0x723e, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x55ac, 0x6cbb,
0x4e9e, 0x6642, 0x9593, 0x55ac, 0x6cbb, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x55ac, 0x6cbb,
0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5409, 0x723e, 0x4f2f, 0x7279, 0x7fa4, 0x5cf6, 0x6642,
0x9593, 0x683c, 0x6797, 0x5a01, 0x6cbb, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x683c, 0x9675, 0x862d,
0x6771, 0x90e8, 0x6642, 0x9593, 0x683c, 0x9675, 0x862d, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x683c, 0x9675, 0x862d, 0x6771, 0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x683c, 0x9675,
0x862d, 0x897f, 0x90e8, 0x6642, 0x9593, 0x683c, 0x9675, 0x862d, 0x897f, 0x90e8, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x683c, 0x9675, 0x862d, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x95dc,
0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6ce2, 0x65af, 0x7063, 0x6d77, 0x57df, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x84cb, 0x4e9e, 0x90a3, 0x6642, 0x9593, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f,
0x7559, 0x7533, 0x6642, 0x9593, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f, 0x7559, 0x7533, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f, 0x7559, 0x7533, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x9999, 0x6e2f, 0x6642, 0x9593, 0x9999, 0x6e2f, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x9999, 0x6e2f, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x79d1, 0x5e03, 0x591a, 0x6642, 0x9593, 0x79d1,
0x5e03, 0x591a, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x79d1, 0x5e03, 0x591a, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x5370, 0x5ea6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5370, 0x5ea6, 0x6d0b, 0x6642, 0x9593,
0x5370, 0x5ea6, 0x652f, 0x90a3, 0x6642, 0x9593, 0x5370, 0x5c3c, 0x4e2d, 0x90e8, 0x6642, 0x9593,
0x5370, 0x5c3c, 0x6771, 0x90e8, 0x6642, 0x9593, 0x5370, 0x5c3c, 0x897f, 0x90e8, 0x6642, 0x9593,
0x4f0a, 0x6717, 0x6642, 0x9593, 0x4f0a, 0x6717, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4f0a, 0x6717,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x4f0a, 0x723e, 0x5eab, 0x6b21, 0x514b, 0x6642, 0x9593, 0x4f0a,
0x723e, 0x5eab, 0x6b21, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4f0a, 0x723e, 0x5eab, 0x6b21,
0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4ee5, 0x8272, 0x5217, 0x6642, 0x9593, 0x4ee5, 0x8272,
0x5217, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4ee5, 0x8272, 0x5217, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x65e5, 0x672c, 0x6642, 0x9593, 0x65e5, 0x672c, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x65e5, 0x672c,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x5f7c, 0x5f97, 0x7f85, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af,
0x514b, 0x6642, 0x9593, 0x5f7c, 0x5f97, 0x7f85, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x5f7c, 0x5f97, 0x7f85, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af,
0x514b, 0x65e5, 0x5149, 0x7bc0, 0x7d04, 0x6642, 0x9593, 0x54c8, 0x85a9, 0x514b, 0x6642, 0x9593,
0x6771, 0x54c8, 0x85a9, 0x514b, 0x6642, 0x9593, 0x897f, 0x54c8, 0x85a9, 0x514b, 0x6642, 0x9593,
0x97d3, 0x570b, 0x6642, 0x9593, 0x97d3, 0x570b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x97d3, 0x570b,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x79d1, 0x65af, 0x745e, 0x6642, 0x9593, 0x514b, 0x62c9, 0x65af,
0x8afe, 0x4e9e, 0x723e, 0x65af, 0x514b, 0x6642, 0x9593, 0x514b, 0x62c9, 0x65af, 0x8afe, 0x4e9e,
0x723e, 0x65af, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x514b, 0x62c9, 0x65af, 0x8afe, 0x4e9e,
0x723e, 0x65af, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5409, 0x723e, 0x5409, 0x65af, 0x6642,
0x9593, 0x862d, 0x5361, 0x6642, 0x9593, 0x840a, 0x6069, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x8c6a,
0x52f3, 0x7235, 0x5cf6, 0x6642, 0x9593, 0x8c6a, 0x52f3, 0x7235, 0x5cf6, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x8c6a, 0x52f3, 0x7235, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x9580, 0x6642,
0x9593, 0x6fb3, 0x9580, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x9580, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x99ac, 0x52a0, 0x4e39, 0x6642, 0x9593, 0x99ac, 0x52a0, 0x4e39, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x99ac, 0x52a0, 0x4e39, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x99ac, 0x4f86, 0x897f, 0x4e9e,
0x6642, 0x9593, 0x99ac, 0x723e, 0x5730, 0x592b, 0x6642, 0x9593, 0x99ac, 0x53ef, 0x85a9, 0x65af,
0x6642, 0x9593, 0x99ac, 0x7d39, 0x723e, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x6a21, 0x91cc, 0x897f,
0x65af, 0x6642, 0x9593, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6a21,
0x91cc, 0x897f, 0x65af, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x83ab, 0x68ee, 0x6642, 0x9593, 0x58a8,
0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x58a8, 0x897f, 0x54e5, 0x592a, 0x5e73,
0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x58a8, 0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x70cf, 0x862d, 0x5df4, 0x6258, 0x6642, 0x9593, 0x70cf, 0x862d, 0x5df4,
0x6258, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x70cf, 0x862d, 0x5df4, 0x6258, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x83ab, 0x65af, 0x79d1, 0x6642, 0x9593, 0x83ab, 0x65af, 0x79d1, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x83ab, 0x65af, 0x79d1, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7dec, 0x7538, 0x6642, 0x9593,
0x8afe, 0x9b6f, 0x6642, 0x9593, 0x5c3c, 0x6cca, 0x723e, 0x6642, 0x9593, 0x65b0, 0x5580, 0x91cc,
0x591a, 0x5c3c, 0x4e9e, 0x6642, 0x9593, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9e, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9e, 0x7fa4, 0x5cf6, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x7d10, 0x897f, 0x862d, 0x6642, 0x9593, 0x7d10, 0x897f, 0x862d, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x7d10, 0x897f, 0x862d, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7d10, 0x82ac,
0x862d, 0x6642, 0x9593, 0x7d10, 0x82ac, 0x862d, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x7d10, 0x82ac,
0x862d, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7d10, 0x57c3, 0x5cf6, 0x6642, 0x9593, 0x8afe, 0x798f,
0x514b, 0x5cf6, 0x6642, 0x9593, 0x8afe, 0x798f, 0x514b, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x8afe, 0x798f, 0x514b, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x8cbb, 0x723e, 0x5357, 0x591a,
0x20, 0x2d, 0x20, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x6642, 0x9593, 0x8cbb, 0x723e,
0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x8cbb, 0x723e, 0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea, 0x8afe, 0x7f85,
0x5c3c, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5317, 0x99ac, 0x91cc, 0x4e9e, 0x7d0d, 0x7fa4,
0x5cf6, 0x6642, 0x9593, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9e, 0x6642, 0x9593, 0x65b0, 0x897f,
0x4f2f, 0x5229, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9e,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x9102, 0x6728, 0x65af, 0x514b, 0x6642, 0x9593, 0x9102, 0x6728,
0x65af, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x9102, 0x6728, 0x65af, 0x514b, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x5df4, 0x57fa, 0x65af, 0x5766, 0x6642, 0x9593, 0x5df4, 0x57fa, 0x65af, 0x5766,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x5df4, 0x57fa, 0x65af, 0x5766, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x5e1b, 0x7409, 0x6642, 0x9593, 0x5df4, 0x5e03, 0x4e9e, 0x7d10, 0x5e7e, 0x5167, 0x4e9e, 0x6642,
0x9593, 0x5df4, 0x62c9, 0x572d, 0x6642, 0x9593, 0x5df4, 0x62c9, 0x572d, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x5df4, 0x62c9, 0x572d, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x79d8, 0x9b6f, 0x6642, 0x9593,
0x79d8, 0x9b6f, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x79d8, 0x9b6f, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x83f2, 0x5f8b, 0x8cd3, 0x6642, 0x9593, 0x83f2, 0x5f8b, 0x8cd3, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x83f2, 0x5f8b, 0x8cd3, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x9cf3, 0x51f0, 0x7fa4, 0x5cf6, 0x6642,
0x9593, 0x8056, 0x76ae, 0x57c3, 0x723e, 0x548c, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6, 0x6642,
0x9593, 0x8056, 0x76ae, 0x57c3, 0x723e, 0x548c, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x8056, 0x76ae, 0x57c3, 0x723e, 0x548c, 0x5bc6, 0x514b, 0x9686, 0x7fa4,
0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x76ae, 0x7279, 0x80af, 0x6642, 0x9593, 0x6ce2, 0x7d0d,
0x4f69, 0x6642, 0x9593, 0x5e73, 0x58e4, 0x6642, 0x9593, 0x514b, 0x5b5c, 0x52d2, 0x5967, 0x723e,
0x9054, 0x6642, 0x9593, 0x514b, 0x5b5c, 0x52d2, 0x5967, 0x723e, 0x9054, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x514b, 0x5b5c, 0x52d2, 0x5967, 0x723e, 0x9054, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7559,
0x5c3c, 0x65fa, 0x6642, 0x9593, 0x7f85, 0x745f, 0x62c9, 0x6642, 0x9593, 0x5eab, 0x9801, 0x5cf6,
0x6642, 0x9593, 0x5eab, 0x9801, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5eab, 0x9801, 0x5cf6,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x85a9, 0x99ac, 0x62c9, 0x6642, 0x9593, 0x85a9, 0x99ac, 0x62c9,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x85a9, 0x99ac, 0x62c9, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x85a9,
0x6469, 0x4e9e, 0x6642, 0x9593, 0x85a9, 0x6469, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x85a9,
0x6469, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x585e, 0x5e2d, 0x723e, 0x6642, 0x9593, 0x65b0,
0x52a0, 0x5761, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x7d22, 0x7f85, 0x9580, 0x7fa4, 0x5cf6, 0x6642,
0x9593, 0x5357, 0x55ac, 0x6cbb, 0x4e9e, 0x6642, 0x9593, 0x8607, 0x5229, 0x5357, 0x6642, 0x9593,
0x662d, 0x548c, 0x57fa, 0x5730, 0x6642, 0x9593, 0x5927, 0x6eaa, 0x5730, 0x6642, 0x9593, 0x53f0,
0x5317, 0x6642, 0x9593, 0x53f0, 0x5317, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x53f0, 0x5317, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x5854, 0x5409, 0x514b, 0x6642, 0x9593, 0x6258, 0x514b, 0x52de, 0x7fa4,
0x5cf6, 0x6642, 0x9593, 0x6771, 0x52a0, 0x6642, 0x9593, 0x6771, 0x52a0, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x6771, 0x52a0, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x695a, 0x514b, 0x5cf6, 0x6642, 0x9593,
0x571f, 0x5eab, 0x66fc, 0x6642, 0x9593, 0x571f, 0x5eab, 0x66fc, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x571f, 0x5eab, 0x66fc, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5410, 0x74e6, 0x9b6f, 0x6642, 0x9593,
0x70cf, 0x62c9, 0x572d, 0x6642, 0x9593, 0x70cf, 0x62c9, 0x572d, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x70cf, 0x62c9, 0x572d, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x70cf, 0x8332, 0x5225, 0x514b, 0x6642,
0x9593, 0x70cf, 0x8332, 0x5225, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x70cf, 0x8332, 0x5225,
0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x842c, 0x90a3, 0x675c, 0x6642, 0x9593, 0x842c, 0x90a3,
0x675c, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x842c, 0x90a3, 0x675c, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x59d4, 0x5167, 0x745e, 0x62c9, 0x6642, 0x9593, 0x6d77, 0x53c3, 0x5d34, 0x6642, 0x9593, 0x6d77,
0x53c3, 0x5d34, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6d77, 0x53c3, 0x5d34, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x4f0f, 0x723e, 0x52a0, 0x683c, 0x52d2, 0x6642, 0x9593, 0x4f0f, 0x723e, 0x52a0, 0x683c,
0x52d2, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4f0f, 0x723e, 0x52a0, 0x683c, 0x52d2, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x6c83, 0x65af, 0x6258, 0x514b, 0x6642, 0x9593, 0x5a01, 0x514b, 0x5cf6, 0x6642,
0x9593, 0x74e6, 0x5229, 0x65af, 0x548c, 0x5bcc, 0x5716, 0x7d0d, 0x7fa4, 0x5cf6, 0x6642, 0x9593,
0x96c5, 0x5eab, 0x6b21, 0x514b, 0x6642, 0x9593, 0x96c5, 0x5eab, 0x6b21, 0x514b, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x96c5, 0x5eab, 0x6b21, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x8449, 0x5361,
0x6377, 0x7433, 0x5821, 0x6642, 0x9593, 0x8449, 0x5361, 0x6377, 0x7433, 0x5821, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x8449, 0x5361, 0x6377, 0x7433, 0x5821, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x80b2,
0x7a7a, 0x6642, 0x9593, 0x827e, 0x514b, 0x65f6, 0x95f4, 0x827e, 0x514b, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x827e, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x5bcc, 0x6c57, 0x65f6, 0x95f4,
0x4e2d, 0x975e, 0x65f6, 0x95f4, 0x4e1c, 0x975e, 0x65f6, 0x95f4, 0x5357, 0x975e, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x897f, 0x975e, 0x65f6, 0x95f4, 0x897f, 0x975e, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x897f, 0x975e, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x65af, 0x52a0, 0x65f6, 0x95f4,
0x963f, 0x62c9, 0x65af, 0x52a0, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x65af, 0x52a0,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x6728, 0x56fe, 0x65f6, 0x95f4, 0x963f, 0x62c9,
0x6728, 0x56fe, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x6728, 0x56fe, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x4e9a, 0x9a6c, 0x900a, 0x65f6, 0x95f4, 0x4e9a, 0x9a6c, 0x900a, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x4e9a, 0x9a6c, 0x900a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e2d, 0x90e8, 0x65f6,
0x95f4, 0x4e2d, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e1c,
0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5c71, 0x533a, 0x65f6, 0x95f4, 0x5c71, 0x533a, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x5c71, 0x533a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x592a, 0x5e73, 0x6d0b,
0x65f6, 0x95f4, 0x592a, 0x5e73, 0x6d0b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x592a, 0x5e73, 0x6d0b,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x7eb3, 0x5fb7, 0x5c14, 0x65f6, 0x95f4, 0x963f, 0x90a3,
0x5e95, 0x6cb3, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x90a3, 0x5e95, 0x6cb3, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x963f, 0x76ae, 0x4e9a, 0x65f6, 0x95f4, 0x963f, 0x76ae, 0x4e9a, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x963f, 0x76ae, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x9676,
0x65f6, 0x95f4, 0x963f, 0x514b, 0x9676, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x9676,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x65f6, 0x95f4, 0x963f, 0x514b,
0x6258, 0x6bd4, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x963f, 0x62c9, 0x4f2f, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x4f2f, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x963f, 0x62c9, 0x4f2f, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x6839, 0x5ef7,
0x65f6, 0x95f4, 0x963f, 0x6839, 0x5ef7, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x6839, 0x5ef7,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x6839, 0x5ef7, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x963f,
0x6839, 0x5ef7, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x6839, 0x5ef7, 0x897f,
0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e9a, 0x7f8e, 0x5c3c, 0x4e9a, 0x65f6, 0x95f4, 0x4e9a,
0x7f8e, 0x5c3c, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e9a, 0x7f8e, 0x5c3c, 0x4e9a, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x5927, 0x897f, 0x6d0b, 0x65f6, 0x95f4, 0x5927, 0x897f, 0x6d0b, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x5927, 0x897f, 0x6d0b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x6d32,
0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x897f,
0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x6fb3, 0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e1c,
0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3,
0x6d32, 0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x897f, 0x90e8, 0x65f6,
0x95f4, 0x6fb3, 0x6d32, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x897f,
0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e9a, 0x585e, 0x62dc, 0x7136, 0x65f6, 0x95f4, 0x4e9a,
0x585e, 0x62dc, 0x7136, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e9a, 0x585e, 0x62dc, 0x7136, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x4e9a, 0x901f, 0x5c14, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x4e9a, 0x901f,
0x5c14, 0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e9a, 0x901f, 0x5c14, 0x7fa4, 0x5c9b,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5b5f, 0x52a0, 0x62c9, 0x65f6, 0x95f4, 0x5b5f, 0x52a0, 0x62c9,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5b5f, 0x52a0, 0x62c9, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e0d,
0x4e39, 0x65f6, 0x95f4, 0x73bb, 0x5229, 0x7ef4, 0x4e9a, 0x65f6, 0x95f4, 0x5df4, 0x897f, 0x5229,
0x4e9a, 0x65f6, 0x95f4, 0x5df4, 0x897f, 0x5229, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5df4,
0x897f, 0x5229, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6c76, 0x83b1, 0x65f6, 0x95f4, 0x7ef4,
0x5fb7, 0x89d2, 0x65f6, 0x95f4, 0x7ef4, 0x5fb7, 0x89d2, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x7ef4,
0x5fb7, 0x89d2, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x51ef, 0x897f, 0x7ad9, 0x65f6, 0x95f4, 0x67e5,
0x83ab, 0x6d1b, 0x65f6, 0x95f4, 0x67e5, 0x5766, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x67e5, 0x5766,
0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x67e5, 0x5766, 0x7fa4, 0x5c9b, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x667a, 0x5229, 0x65f6, 0x95f4, 0x667a, 0x5229, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x667a, 0x5229, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e2d, 0x56fd, 0x65f6, 0x95f4, 0x4e2d, 0x56fd,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e2d, 0x56fd, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5723, 0x8bde,
0x5c9b, 0x65f6, 0x95f4, 0x79d1, 0x79d1, 0x65af, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x54e5, 0x4f26,
0x6bd4, 0x4e9a, 0x65f6, 0x95f4, 0x54e5, 0x4f26, 0x6bd4, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x54e5, 0x4f26, 0x6bd4, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5e93, 0x514b, 0x7fa4, 0x5c9b,
0x65f6, 0x95f4, 0x5e93, 0x514b, 0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5e93, 0x514b,
0x7fa4, 0x5c9b, 0x534a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x53e4, 0x5df4, 0x65f6, 0x95f4, 0x53e4,
0x5df4, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x53e4, 0x5df4, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6234,
0x7ef4, 0x65af, 0x65f6, 0x95f4, 0x675c, 0x8499, 0x675c, 0x6bd4, 0x5c14, 0x65f6, 0x95f4, 0x4e1c,
0x5e1d, 0x6c76, 0x65f6, 0x95f4, 0x590d, 0x6d3b, 0x8282, 0x5c9b, 0x65f6, 0x95f4, 0x590d, 0x6d3b,
0x8282, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x590d, 0x6d3b, 0x8282, 0x5c9b, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x5384, 0x74dc, 0x591a, 0x65f6, 0x95f4, 0x4e2d, 0x6b27, 0x65f6, 0x95f4, 0x4e2d,
0x6b27, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e2d, 0x6b27, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e1c,
0x6b27, 0x65f6, 0x95f4, 0x4e1c, 0x6b27, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e1c, 0x6b27, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x6b27, 0x6d32, 0x8fdc, 0x4e1c, 0x65f6, 0x95f4, 0x897f, 0x6b27, 0x65f6,
0x95f4, 0x897f, 0x6b27, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x897f, 0x6b27, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x798f, 0x514b, 0x5170, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x798f, 0x514b, 0x5170, 0x7fa4,
0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x798f, 0x514b, 0x5170, 0x7fa4, 0x5c9b, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x6590, 0x6d4e, 0x65f6, 0x95f4, 0x6590, 0x6d4e, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x6590, 0x6d4e, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6cd5, 0x5c5e, 0x572d, 0x4e9a, 0x90a3, 0x65f6,
0x95f4, 0x6cd5, 0x56fd, 0x5357, 0x65b9, 0x53ca, 0x5357, 0x6781, 0x65f6, 0x95f4, 0x52a0, 0x62c9,
0x5df4, 0x54e5, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x7518, 0x6bd4, 0x5c14, 0x7fa4, 0x5c9b, 0x65f6,
0x95f4, 0x4e54, 0x6cbb, 0x4e9a, 0x65f6, 0x95f4, 0x4e54, 0x6cbb, 0x4e9a, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x4e54, 0x6cbb, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5409, 0x5c14, 0x4f2f, 0x7279,
0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x683c, 0x6797, 0x5a01, 0x6cbb, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x683c, 0x9675, 0x5170, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x4e1c, 0x90e8,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x683c, 0x9675, 0x5170, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x897f,
0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x897f, 0x90e8, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x5173, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6ce2, 0x65af, 0x6e7e, 0x6d77,
0x57df, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x76d6, 0x4e9a, 0x90a3, 0x65f6, 0x95f4, 0x590f, 0x5a01,
0x5937, 0x2d, 0x963f, 0x7559, 0x7533, 0x65f6, 0x95f4, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f,
0x7559, 0x7533, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f, 0x7559,
0x7533, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x9999, 0x6e2f, 0x65f6, 0x95f4, 0x9999, 0x6e2f, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x9999, 0x6e2f, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x79d1, 0x5e03, 0x591a,
0x65f6, 0x95f4, 0x79d1, 0x5e03, 0x591a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x79d1, 0x5e03, 0x591a,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5370, 0x5ea6,
0x6d0b, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x652f, 0x90a3, 0x65f6, 0x95f4, 0x5370, 0x5c3c, 0x4e2d,
0x90e8, 0x65f6, 0x95f4, 0x5370, 0x5c3c, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x5370, 0x5c3c, 0x897f,
0x90e8, 0x65f6, 0x95f4, 0x4f0a, 0x6717, 0x65f6, 0x95f4, 0x4f0a, 0x6717, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x4f0a, 0x6717, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x6b21, 0x514b,
0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x6b21, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f0a,
0x5c14, 0x5e93, 0x6b21, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4ee5, 0x8272, 0x5217, 0x65f6,
0x95f4, 0x4ee5, 0x8272, 0x5217, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4ee5, 0x8272, 0x5217, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x65e5, 0x672c, 0x65f6, 0x95f4, 0x65e5, 0x672c, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x65e5, 0x672c, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4, 0x752b,
0x6d1b, 0x592b, 0x65af, 0x514b, 0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4, 0x752b, 0x6d1b,
0x592b, 0x65af, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4, 0x752b,
0x6d1b, 0x592b, 0x65af, 0x514b, 0x65e5, 0x5149, 0x8282, 0x7ea6, 0x65f6, 0x95f4, 0x54c8, 0x8428,
0x514b, 0x65f6, 0x95f4, 0x4e1c, 0x54c8, 0x8428, 0x514b, 0x65f6, 0x95f4, 0x897f, 0x54c8, 0x8428,
0x514b, 0x65f6, 0x95f4, 0x97e9, 0x56fd, 0x65f6, 0x95f4, 0x97e9, 0x56fd, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x97e9, 0x56fd, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x79d1, 0x65af, 0x745e, 0x65f6, 0x95f4,
0x514b, 0x62c9, 0x65af, 0x8bfa, 0x4e9a, 0x5c14, 0x65af, 0x514b, 0x65f6, 0x95f4, 0x514b, 0x62c9,
0x65af, 0x8bfa, 0x4e9a, 0x5c14, 0x65af, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x514b, 0x62c9,
0x65af, 0x8bfa, 0x4e9a, 0x5c14, 0x65af, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5409, 0x5c14,
0x5409, 0x65af, 0x65f6, 0x95f4, 0x5170, 0x5361, 0x65f6, 0x95f4, 0x83b1, 0x6069, 0x7fa4, 0x5c9b,
0x65f6, 0x95f4, 0x8c6a, 0x52cb, 0x7235, 0x5c9b, 0x65f6, 0x95f4, 0x8c6a, 0x52cb, 0x7235, 0x5c9b,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8c6a, 0x52cb, 0x7235, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x6fb3, 0x95e8, 0x65f6, 0x95f4, 0x6fb3, 0x95e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x95e8,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x9a6c, 0x52a0, 0x4e39, 0x65f6, 0x95f4, 0x9a6c, 0x52a0, 0x4e39,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x9a6c, 0x52a0, 0x4e39, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x9a6c,
0x6765, 0x897f, 0x4e9a, 0x65f6, 0x95f4, 0x9a6c, 0x5c14, 0x5730, 0x592b, 0x65f6, 0x95f4, 0x9a6c,
0x53ef, 0x8428, 0x65af, 0x65f6, 0x95f4, 0x9a6c, 0x7ecd, 0x5c14, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4,
0x6a21, 0x91cc, 0x897f, 0x65af, 0x65f6, 0x95f4, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x83ab, 0x68ee,
0x65f6, 0x95f4, 0x58a8, 0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x65f6, 0x95f4, 0x58a8, 0x897f,
0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x58a8, 0x897f, 0x54e5, 0x592a,
0x5e73, 0x6d0b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e4c, 0x5170, 0x5df4, 0x6258, 0x65f6, 0x95f4,
0x4e4c, 0x5170, 0x5df4, 0x6258, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e4c, 0x5170, 0x5df4, 0x6258,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x83ab, 0x65af, 0x79d1, 0x65f6, 0x95f4, 0x83ab, 0x65af, 0x79d1,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x83ab, 0x65af, 0x79d1, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7f05,
0x7538, 0x65f6, 0x95f4, 0x8bfa, 0x9c81, 0x65f6, 0x95f4, 0x5c3c, 0x6cca, 0x5c14, 0x65f6, 0x95f4,
0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9a, 0x65f6, 0x95f4, 0x65b0, 0x5580, 0x91cc, 0x591a,
0x5c3c, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9a,
0x7fa4, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7ebd, 0x897f, 0x5170, 0x65f6, 0x95f4, 0x7ebd,
0x897f, 0x5170, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x7ebd, 0x897f, 0x5170, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x7ebd, 0x82ac, 0x5170, 0x65f6, 0x95f4, 0x7ebd, 0x82ac, 0x5170, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x7ebd, 0x82ac, 0x5170, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7ebd, 0x57c3, 0x5c9b, 0x65f6,
0x95f4, 0x8bfa, 0x798f, 0x514b, 0x5c9b, 0x65f6, 0x95f4, 0x8bfa, 0x798f, 0x514b, 0x5c9b, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x8bfa, 0x798f, 0x514b, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x8d39,
0x5c14, 0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea, 0x8bfa, 0x7f57, 0x5c3c, 0x4e9a, 0x65f6,
0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea, 0x8bfa, 0x7f57, 0x5c3c,
0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x20, 0x2d, 0x20,
0x8fea, 0x8bfa, 0x7f57, 0x5c3c, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5317, 0x9a6c, 0x91cc,
0x4e9a, 0x7eb3, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9a, 0x65f6,
0x95f4, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x65b0, 0x897f,
0x4f2f, 0x5229, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x9102, 0x6728, 0x65af, 0x514b, 0x65f6,
0x95f4, 0x9102, 0x6728, 0x65af, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x9102, 0x6728, 0x65af,
0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5df4, 0x57fa, 0x65af, 0x5766, 0x65f6, 0x95f4, 0x5df4,
0x57fa, 0x65af, 0x5766, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5df4, 0x57fa, 0x65af, 0x5766, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x5e1b, 0x7409, 0x65f6, 0x95f4, 0x5df4, 0x5e03, 0x4e9a, 0x7ebd, 0x51e0,
0x5185, 0x4e9a, 0x65f6, 0x95f4, 0x5df4, 0x62c9, 0x572d, 0x65f6, 0x95f4, 0x5df4, 0x62c9, 0x572d,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5df4, 0x62c9, 0x572d, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x79d8,
0x9c81, 0x65f6, 0x95f4, 0x79d8, 0x9c81, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x79d8, 0x9c81, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x83f2, 0x5f8b, 0x5bbe, 0x65f6, 0x95f4, 0x83f2, 0x5f8b, 0x5bbe, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x83f2, 0x5f8b, 0x5bbe, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x51e4, 0x51f0,
0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5723, 0x76ae, 0x57c3, 0x5c14, 0x548c, 0x5bc6, 0x514b, 0x9686,
0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5723, 0x76ae, 0x57c3, 0x5c14, 0x548c, 0x5bc6, 0x514b, 0x9686,
0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5723, 0x76ae, 0x57c3, 0x5c14, 0x548c, 0x5bc6,
0x514b, 0x9686, 0x7fa4, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x76ae, 0x7279, 0x80af, 0x65f6,
0x95f4, 0x6ce2, 0x7eb3, 0x4f69, 0x65f6, 0x95f4, 0x5e73, 0x58e4, 0x65f6, 0x95f4, 0x514b, 0x5b5c,
0x52d2, 0x5965, 0x5c14, 0x8fbe, 0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x52d2, 0x5965, 0x5c14, 0x8fbe,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x52d2, 0x5965, 0x5c14, 0x8fbe, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x7559, 0x5c3c, 0x65fa, 0x65f6, 0x95f4, 0x7f57, 0x745f, 0x62c9, 0x65f6, 0x95f4,
0x5e93, 0x9875, 0x5c9b, 0x65f6, 0x95f4, 0x5e93, 0x9875, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x5e93, 0x9875, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x8428, 0x9a6c, 0x62c9, 0x65f6, 0x95f4,
0x8428, 0x9a6c, 0x62c9, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8428, 0x9a6c, 0x62c9, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x8428, 0x6469, 0x4e9a, 0x65f6, 0x95f4, 0x8428, 0x6469, 0x4e9a, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x8428, 0x6469, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x585e, 0x5e2d, 0x5c14,
0x65f6, 0x95f4, 0x65b0, 0x52a0, 0x5761, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x7d22, 0x7f57, 0x95e8,
0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5357, 0x4e54, 0x6cbb, 0x4e9a, 0x65f6, 0x95f4, 0x82cf, 0x5229,
0x5357, 0x65f6, 0x95f4, 0x662d, 0x548c, 0x57fa, 0x5730, 0x65f6, 0x95f4, 0x5927, 0x6eaa, 0x5730,
0x65f6, 0x95f4, 0x53f0, 0x5317, 0x65f6, 0x95f4, 0x53f0, 0x5317, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x53f0, 0x5317, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5854, 0x5409, 0x514b, 0x65f6, 0x95f4, 0x6258,
0x514b, 0x52b3, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x4e1c, 0x52a0, 0x65f6, 0x95f4, 0x4e1c, 0x52a0,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e1c, 0x52a0, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x695a, 0x514b,
0x5c9b, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5410, 0x74e6,
0x9c81, 0x65f6, 0x95f4, 0x4e4c, 0x62c9, 0x572d, 0x65f6, 0x95f4, 0x4e4c, 0x62c9, 0x572d, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x4e4c, 0x62c9, 0x572d, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e4c, 0x5179,
0x522b, 0x514b, 0x65f6, 0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x4e4c, 0x5179, 0x522b, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e07, 0x90a3, 0x675c, 0x65f6,
0x95f4, 0x4e07, 0x90a3, 0x675c, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e07, 0x90a3, 0x675c, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x59d4, 0x5185, 0x745e, 0x62c9, 0x65f6, 0x95f4, 0x6d77, 0x53c2, 0x5d34,
0x65f6, 0x95f4, 0x6d77, 0x53c2, 0x5d34, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6d77, 0x53c2, 0x5d34,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4f0f, 0x5c14, 0x52a0, 0x683c, 0x52d2, 0x65f6, 0x95f4, 0x4f0f,
0x5c14, 0x52a0, 0x683c, 0x52d2, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f0f, 0x5c14, 0x52a0, 0x683c,
0x52d2, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6c83, 0x65af, 0x6258, 0x514b, 0x65f6, 0x95f4, 0x5a01,
0x514b, 0x5c9b, 0x65f6, 0x95f4, 0x74e6, 0x5229, 0x65af, 0x548c, 0x5bcc, 0x56fe, 0x7eb3, 0x7fa4,
0x5c9b, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x6b21, 0x514b, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x6b21,
0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x6b21, 0x514b, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x53f6, 0x5361, 0x6377, 0x7433, 0x5821, 0x65f6, 0x95f4, 0x53f6, 0x5361, 0x6377, 0x7433,
0x5821, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x53f6, 0x5361, 0x6377, 0x7433, 0x5821, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x80b2, 0x7a7a, 0x65f6, 0x95f4, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69,
0x6f, 0x6e, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41,
0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d,
0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x75, 0x6e, 0x74,
0x61, 0x6e, 0x79, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x61, 0x6e, 0x79,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x6d, 0x75, 0x6e, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20, 0x64, 0x2019, 0x41, 0x6d,
0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20,
0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x6e, 0xe0, 0x64, 0x69, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x70,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0xe0, 0x72, 0x61, 0x62, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0xe0, 0x72, 0x61,
0x62, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0xe0, 0x72, 0x61, 0x62, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x65, 0x73, 0x74, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x65,
0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f,
0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x2019, 0x41, 0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41,
0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x63,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0,
0x6e, 0x74, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74,
0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x2d,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x65, 0x2d, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x64, 0x6a, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f,
0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x6f, 0x6c, 0xed, 0x76,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72,
0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72,
0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x20, 0x56,
0x65, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x20, 0x56,
0x65, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x58, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x58,
0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x58, 0x69, 0x6c, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x58, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x58,
0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x58, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x69,
0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65,
0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6d, 0x69, 0x67, 0x20, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x71, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x78, 0x74, 0x72, 0x65, 0x6d,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e,
0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e,
0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x75, 0x61, 0x69, 0x61, 0x6e, 0x61,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x74, 0xe0, 0x72, 0x74, 0x69, 0x64,
0x61, 0x20, 0x69, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x54,
0x65, 0x72, 0x72, 0x65, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x20, 0x41, 0x6e, 0x74, 0xe0, 0x72, 0x74, 0x69, 0x71, 0x75,
0x65, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0xf2, 0x72,
0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0xf2,
0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0xf2, 0x72,
0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0,
0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73,
0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0,
0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x65, 0x73, 0x74, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x65, 0x73, 0x74,
0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e,
0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x6f,
0x6c, 0x66, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xcd, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x63, 0x65, 0xe0, 0x20,
0xcd, 0x6e, 0x64, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x78, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x73, 0x74,
0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x65,
0x73, 0x74, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x49, 0x72, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x49, 0x72, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72,
0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4a, 0x61, 0x70, 0xf3, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x61, 0x6d, 0x74, 0x78, 0x61, 0x74, 0x6b, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d,
0x74, 0x78, 0x61, 0x74, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d, 0x74, 0x78, 0x61, 0x74,
0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x73, 0x74, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f,
0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x43, 0x6f, 0x72, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72,
0x65, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x69, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x7a, 0x73, 0x74, 0x61,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63,
0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x6c, 0xe0, 0x69, 0x73, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20,
0x64, 0x65, 0x20, 0x4d, 0xe8, 0x78, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20, 0x64,
0x65, 0x20, 0x4d, 0xe8, 0x78, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20, 0x64, 0x65, 0x20, 0x4d,
0xe8, 0x78, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x2019, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0xf2, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e,
0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65,
0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69,
0x6e, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x50,
0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x79, 0x6f, 0x6e,
0x67, 0x79, 0x61, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0xed, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0xed, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c,
0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xf3, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x53, 0x75, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0xed, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x61, 0x64, 0x6a,
0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x61, 0x6e, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56,
0x61, 0x6e, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e,
0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x6e, 0x65, 0xe7, 0x75, 0x65, 0x6c, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0x69, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x49, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x49, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x45, 0x61, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x6f, 0x75, 0x74,
0x68, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70,
0x69, 0x6b, 0x6f, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b,
0x6f, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x70, 0x69,
0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x70,
0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41,
0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57,
0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x61,
0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73,
0x61, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x72, 0x69,
0x73, 0x74, 0x6d, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x4b, 0x61, 0x74, 0x75, 0x6e, 0x67, 0x61, 0x6e,
0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69,
0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62, 0x61,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x44, 0x61, 0x76,
0x69, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61,
0x73, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x63, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x46, 0x75, 0x72, 0x74, 0x68, 0x65, 0x72, 0x2d,
0x65, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x72,
0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x72, 0x65, 0x6e, 0x63,
0x68, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72, 0x6e, 0x20, 0x75,
0x67, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69,
0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x4d,
0x65, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45,
0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61,
0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73,
0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x4f, 0x63, 0x65,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69,
0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x7a,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69,
0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x6f, 0x77, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50,
0x61, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x73, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x74, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x74, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67,
0x79, 0x61, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x6d, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0x26, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x67,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73,
0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x67, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x69, 0x6e, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x75,
0x6b, 0x6f, 0x6e, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd09,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804,
0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1c, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804,
0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1c,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd18, 0xd804,
0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804,
0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd07, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd18,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd11, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd11,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd11,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804,
0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804,
0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804,
0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd22, 0xd804, 0xdd33,
0xd804, 0xdd26, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26, 0x20, 0xd804, 0xdd18,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28,
0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd27, 0xd804, 0xdd31, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18,
0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd27, 0xd804, 0xdd31, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28,
0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd27, 0xd804, 0xdd31,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804,
0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804,
0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd20, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd18, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28,
0xd804, 0xdd20, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804,
0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f,
0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd20, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804,
0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c,
0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804,
0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804,
0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd27,
0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27,
0x20, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804,
0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804,
0xdd2d, 0xd804, 0xdd0e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2d, 0xd804, 0xdd0e, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2d, 0xd804, 0xdd0e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c,
0xd804, 0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804,
0xdd01, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1d, 0xd804, 0xdd01, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd01, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd2c,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804,
0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804,
0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd18,
0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd23,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2c,
0xd804, 0xdd1b, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd18, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804,
0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd17, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd17, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd17, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804,
0xdd23, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804,
0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd25,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd20,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804,
0xdd20, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28,
0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2a,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804,
0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd2c,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd1d, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd13, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd34, 0x2d, 0xd804, 0xdd18, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd27, 0x20, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804,
0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd11, 0xd804, 0xdd28,
0xd804, 0xdd1f, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28,
0xd804, 0xdd18, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804,
0xdd2c, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804,
0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804,
0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804,
0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e,
0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a,
0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09,
0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e,
0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22,
0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd2c,
0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c,
0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1c, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c,
0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd25, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1c, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd03, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804,
0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd23, 0xd804, 0xdd1b,
0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804,
0xdd20, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18,
0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd11, 0xd804,
0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804,
0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804,
0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd0f, 0xd804, 0xdd2c, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f,
0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20,
0xd804, 0xdd2d, 0x2d, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd2d, 0x2d, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2c,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804,
0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd01,
0x20, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804,
0xdd01, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd2e,
0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804,
0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804,
0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26,
0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd29, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd2c,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804,
0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd2c,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804,
0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e,
0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd22, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804,
0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd23,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd22,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd1b, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd1b, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804,
0xdd1b, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x2d,
0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0c,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804,
0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x2d, 0xd804, 0xdd07, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c,
0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x2d, 0xd804,
0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804,
0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd0e,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd16, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd0e, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd01, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd2d, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804,
0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd26, 0xd804, 0xdd24, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26, 0xd804, 0xdd24, 0xd804,
0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd26, 0xd804, 0xdd24, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804,
0xdd2e, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd13, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09,
0xd804, 0xdd13, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd09, 0xd804, 0xdd13, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd23,
0xd804, 0xdd34, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804,
0xdd1b, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28,
0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd26, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804,
0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26,
0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804,
0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd23, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd16, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd2e, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22,
0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd23, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804,
0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd07,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd2e,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804,
0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03,
0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1c, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd1c, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1c,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd26,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd26,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd26, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804,
0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804,
0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804,
0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd07, 0xd804,
0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd23, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804,
0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd2a,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2a,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804,
0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804,
0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd2d, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1b,
0xd804, 0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd1a,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2c,
0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd0e,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804,
0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd17, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd08, 0xd804,
0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd08,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd08,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f,
0xd804, 0xdd22, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f,
0xd804, 0xdd2e, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd20,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd01, 0xd804,
0xdd09, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804,
0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804,
0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd0e, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd20, 0xd804, 0xdd2e, 0xd804, 0xdd24, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd16, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16,
0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd16, 0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd16, 0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0x20,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd0e, 0xd804,
0xdd08, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804,
0xdd09, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c,
0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2a, 0xd804, 0xdd1e, 0xd804, 0xdd23, 0xd804, 0xdd2a,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804,
0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22,
0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20,
0xd804, 0xdd2c, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804,
0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804,
0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804,
0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e,
0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e,
0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd20,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804,
0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33,
0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d,
0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2e,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e,
0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804,
0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804,
0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804,
0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd24, 0xd804, 0xdd23,
0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804,
0xdd03, 0x20, 0xd804, 0xdd1c, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd2a, 0xd804, 0xdd1a,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd07,
0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd09, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0x41e, 0x432, 0x445, 0x4c0, 0x430, 0x43d, 0x42e, 0x43a, 0x43a,
0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41c,
0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x41a, 0x44a, 0x438, 0x43b, 0x431, 0x430, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435,
0x43d, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435,
0x43d, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x43b,
0x44f, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x43b, 0x44f, 0x441,
0x43a, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b,
0x435, 0x43d, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41c, 0x430,
0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431,
0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x41b, 0x430, 0x44c, 0x43c, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x20, 0x28, 0x410, 0x426, 0x428, 0x29, 0x41b, 0x430, 0x44c, 0x43c,
0x43d, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x20, 0x28, 0x410, 0x426, 0x428,
0x29, 0x41b, 0x430, 0x44c, 0x43c, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x20, 0x28,
0x410, 0x426, 0x428, 0x29, 0x422, 0x438, 0x439, 0x43d, 0x430, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x438, 0x439,
0x43d, 0x430, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x422, 0x438, 0x439, 0x43d, 0x430, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x430, 0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x445, 0x430, 0x43d, 0x20, 0x410, 0x43f, 0x438, 0x430, 0x2c,
0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x20, 0x410, 0x43f, 0x438,
0x430, 0x2c, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x20, 0x410, 0x43f, 0x438,
0x430, 0x2c, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x421, 0x430, 0x4c0, 0x443,
0x434, 0x438, 0x439, 0x43d, 0x20, 0x4c0, 0x430, 0x44c, 0x440, 0x431, 0x438, 0x439,
0x447, 0x43e, 0x44c, 0x421, 0x430, 0x4c0, 0x443, 0x434, 0x438, 0x439, 0x43d, 0x20,
0x4c0, 0x430, 0x44c, 0x440, 0x431, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x43d, 0x430,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x421, 0x430, 0x4c0, 0x443, 0x434, 0x438, 0x439, 0x43d,
0x20, 0x4c0, 0x430, 0x44c, 0x440, 0x431, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x43d,
0x430, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435,
0x43d, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x41c,
0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430,
0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42d, 0x440, 0x43c, 0x430, 0x43b,
0x43e, 0x439, 0x447, 0x43e, 0x44c, 0x42d, 0x440, 0x43c, 0x430, 0x43b, 0x43e, 0x439,
0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42d, 0x440, 0x43c, 0x430, 0x43b,
0x43e, 0x439, 0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x438, 0x43a, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x442, 0x43b,
0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x43d, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a,
0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a,
0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x43c, 0x430, 0x43b,
0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a,
0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x2c, 0x20, 0x43c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x43c, 0x430,
0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x437, 0x435, 0x440,
0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410,
0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x410, 0x437, 0x43e, 0x440, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440,
0x435, 0x43d, 0x430, 0x448, 0x410, 0x437, 0x43e, 0x440, 0x430, 0x43d, 0x20, 0x433,
0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x410, 0x437, 0x43e, 0x440, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440,
0x435, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x411, 0x430, 0x43d, 0x433, 0x43b,
0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x2d, 0x414, 0x430, 0x440,
0x443, 0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x41a, 0x430, 0x431, 0x43e, 0x2d,
0x412, 0x435, 0x440, 0x434, 0x435, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x430, 0x431,
0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x427, 0x430, 0x442,
0x435, 0x43c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x427, 0x438, 0x43b,
0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x426, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x426, 0x438, 0x439,
0x447, 0x43e, 0x44c, 0x43d, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x426, 0x438,
0x439, 0x447, 0x43e, 0x44c, 0x43d, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x4c0, 0x438, 0x439,
0x441, 0x430, 0x20, 0x43f, 0x430, 0x439, 0x445, 0x430, 0x43c, 0x430, 0x440, 0x20,
0x432, 0x438, 0x43d, 0x447, 0x443, 0x20, 0x434, 0x435, 0x43d, 0x20, 0x433, 0x4c0,
0x430, 0x439, 0x440, 0x435, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x438, 0x439, 0x43d,
0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x43e, 0x43b, 0x443, 0x43c,
0x431, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41a, 0x443, 0x43a, 0x430, 0x43d, 0x2c, 0x20, 0x433,
0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41a, 0x443, 0x43a, 0x430, 0x43d, 0x2c, 0x20,
0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x443,
0x43a, 0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x443, 0x431,
0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e,
0x440, 0x432, 0x438, 0x43b, 0x44c, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b,
0x435, 0x43d, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x41c, 0x430, 0x440, 0x445,
0x438, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x41c, 0x430, 0x440,
0x445, 0x438, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x41c, 0x430, 0x440, 0x445, 0x438, 0x43d, 0x20, 0x433, 0x4c0, 0x430,
0x439, 0x440, 0x435, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x42e, 0x43a, 0x43a, 0x44a, 0x435,
0x440, 0x430, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b,
0x435, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x41c, 0x430, 0x43b,
0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b,
0x435, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a,
0x430, 0x43b, 0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x435, 0x440,
0x430, 0x20, 0x430, 0x2c, 0x20, 0x41c, 0x438, 0x43d, 0x441, 0x43a, 0x435, 0x440,
0x430, 0x20, 0x430, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431,
0x443, 0x437, 0x435, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x41c,
0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431,
0x443, 0x437, 0x435, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x430, 0x43d, 0x20,
0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x424, 0x43e, 0x43b,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439,
0x440, 0x435, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x43e, 0x43b,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439,
0x440, 0x435, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x438, 0x434, 0x436, 0x438,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x424, 0x438, 0x434, 0x436, 0x438, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424,
0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x438, 0x439, 0x43d, 0x20, 0x413, 0x432,
0x438, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x438,
0x439, 0x43d, 0x20, 0x43a, 0x44a, 0x438, 0x43b, 0x431, 0x430, 0x20, 0x430, 0x2c,
0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x434, 0x430, 0x43d,
0x20, 0x430, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430,
0x433, 0x43e, 0x441, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435,
0x43d, 0x430, 0x448, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x413, 0x443, 0x44c,
0x440, 0x436, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x413, 0x443, 0x44c, 0x440, 0x436,
0x438, 0x439, 0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x443, 0x44c,
0x440, 0x436, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x438, 0x43b,
0x431, 0x435, 0x440, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d,
0x430, 0x448, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x438, 0x446, 0x430,
0x20, 0x44e, 0x43a, 0x44a, 0x430, 0x440, 0x430, 0x20, 0x445, 0x430, 0x43d, 0x41c,
0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430,
0x43b, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b,
0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x41c, 0x430, 0x43b,
0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x4c0, 0x430, 0x436, 0x430, 0x440, 0x438,
0x439, 0x43d, 0x20, 0x430, 0x439, 0x43c, 0x430, 0x413, 0x430, 0x439, 0x430, 0x43d,
0x430, 0x413, 0x430, 0x432, 0x430, 0x439, 0x43d, 0x2d, 0x430, 0x43b, 0x435, 0x443,
0x442, 0x438, 0x439, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x430, 0x432, 0x430,
0x439, 0x43d, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x413, 0x430, 0x432, 0x430, 0x439, 0x43d, 0x2d, 0x430, 0x43b, 0x435,
0x443, 0x442, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d,
0x433, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x425, 0x43e, 0x432,
0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x425,
0x4c0, 0x438, 0x43d, 0x434, 0x438, 0x418, 0x43d, 0x434, 0x438, 0x43d, 0x20, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20,
0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x438, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20,
0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x413, 0x4c0, 0x430, 0x436,
0x430, 0x440, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x413, 0x4c0, 0x430, 0x436, 0x430,
0x440, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x4c0,
0x430, 0x436, 0x430, 0x440, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x418,
0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x418, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x418, 0x437, 0x440, 0x430, 0x438,
0x43b, 0x44c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b,
0x44c, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x42f, 0x43f, 0x43e, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42f,
0x43f, 0x43e, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b,
0x435, 0x43d, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d,
0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x41a, 0x430,
0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x41a, 0x43e, 0x440, 0x435, 0x439,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a,
0x43e, 0x441, 0x440, 0x430, 0x44d, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f,
0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41b, 0x430, 0x439, 0x43d, 0x2c,
0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41b, 0x43e, 0x440, 0x434, 0x2d,
0x425, 0x430, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x2d,
0x425, 0x430, 0x443, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x430, 0x448, 0x41c, 0x430,
0x440, 0x43a, 0x438, 0x437, 0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d,
0x430, 0x448, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x430, 0x43d, 0x20, 0x2c,
0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x43a, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a,
0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x422, 0x438, 0x439, 0x43d, 0x430, 0x20, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x430, 0x43d, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x438, 0x439, 0x43d, 0x430, 0x20, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438,
0x43a, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x438, 0x439, 0x43d, 0x430, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x41c, 0x435, 0x43a, 0x441,
0x438, 0x43a, 0x430, 0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430,
0x442, 0x43e, 0x440, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423, 0x43b, 0x430, 0x43d, 0x2d,
0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x43e, 0x441, 0x43a, 0x432,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c, 0x430, 0x41a, 0x435, 0x440, 0x43b, 0x430,
0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x41a, 0x435, 0x440,
0x43b, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41a, 0x435, 0x440, 0x43b, 0x430, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x435, 0x440, 0x43b, 0x430,
0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x41a, 0x435, 0x440, 0x43b,
0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41a, 0x435, 0x440, 0x43b, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41d, 0x44c, 0x44e, 0x444,
0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41d, 0x438,
0x443, 0x44d, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x435,
0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e,
0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443,
0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x2013,
0x20, 0x41a, 0x435, 0x440, 0x43b, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435,
0x439, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f,
0x435, 0x440, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f, 0x435, 0x440, 0x443, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x430, 0x448, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x430, 0x448, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x2c, 0x20, 0x433, 0x4c0, 0x2d,
0x43d, 0x430, 0x448, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20,
0x430, 0x2c, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x430,
0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x430, 0x2c, 0x20,
0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x430, 0x2c,
0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x430, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x41f, 0x43e, 0x43d, 0x430, 0x43f,
0x435, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x420, 0x435, 0x44e,
0x43d, 0x44c, 0x43e, 0x43d, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x421, 0x430, 0x43c,
0x43e, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x430, 0x43d,
0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x421, 0x43e,
0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d,
0x43d, 0x430, 0x448, 0x41a, 0x44a, 0x438, 0x43b, 0x431, 0x430, 0x20, 0x413, 0x435,
0x43e, 0x440, 0x433, 0x438, 0x422, 0x430, 0x438, 0x442, 0x438, 0x2c, 0x20, 0x433,
0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x422, 0x430, 0x439, 0x432, 0x430, 0x43d, 0x44c,
0x422, 0x430, 0x439, 0x432, 0x430, 0x43d, 0x44c, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422,
0x430, 0x439, 0x432, 0x430, 0x43d, 0x44c, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x43e, 0x43d, 0x433,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x427, 0x443, 0x443, 0x43a, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438,
0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423, 0x437,
0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x430,
0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x412, 0x435,
0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x412, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423,
0x44d, 0x439, 0x43a, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435, 0x423, 0x43e, 0x43b,
0x43b, 0x438, 0x441, 0x20, 0x430, 0x2c, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d,
0x430, 0x20, 0x430, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x415, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440,
0x433, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x13a0, 0x13eb, 0x13a8, 0x13c2, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd,
0x13db, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20,
0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e7,
0x13a6, 0x13be, 0x13ee, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5,
0x13ac, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed,
0x13d5, 0x13b5, 0x13ac, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b3, 0x13cd, 0x13a6, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b3, 0x13cd, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b3, 0x13cd, 0x13a6,
0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13ba, 0x13cc, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13ba,
0x13cc, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13ba, 0x13cc, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13f0, 0x13df, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7,
0x13dc, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20,
0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13aa, 0x13af,
0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13d3, 0x13b8,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13d3, 0x13b8, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13d3, 0x13b8,
0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5,
0x13b5, 0x13ac, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c8, 0x13a0, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c8, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c8, 0x13a0, 0x20, 0x13aa, 0x13af,
0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b4, 0x13c8,
0x13ef, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b4, 0x13c8, 0x13ef, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13b4, 0x13c8, 0x13ef, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13e5, 0x13c2,
0x13d8, 0x13c2, 0x13a0, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13e5, 0x13c2, 0x13d8,
0x13c2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac,
0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5,
0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b5, 0x13bb, 0x13c2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13b5, 0x13bb, 0x13c2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b5, 0x13bb, 0x13c2, 0x13a0,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8,
0x13ac, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7,
0x13a7, 0x13b8, 0x13ac, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7,
0x13dc, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13aa, 0x13af,
0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df,
0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7,
0x13dc, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1,
0x13b3, 0x13d7, 0x13dc, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac,
0x20, 0x13d7, 0x13dc, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13d7, 0x13a7, 0x13b8, 0x13ac,
0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13b3, 0x13d7,
0x13dc, 0x20, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13b3, 0x13d7,
0x13dc, 0x20, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13aa, 0x13af,
0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13cf, 0x13b5,
0x13c6, 0x13cc, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13cf, 0x13b5,
0x13c6, 0x13cc, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13cf, 0x13b5, 0x13c6, 0x13cc, 0x13c2, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d0, 0x13b4, 0x13cf, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d0, 0x13b4, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d0, 0x13b4, 0x13cf,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13c2, 0x13a6,
0x13b5, 0x13d5, 0x13cd, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13c2, 0x13a6,
0x13b5, 0x13d5, 0x13cd, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13c2, 0x13a6, 0x13b5, 0x13d5, 0x13cd, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ca, 0x13d4, 0x13c2, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13c9, 0x13b5, 0x13eb, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13c6, 0x13cf, 0x13b5, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6,
0x13cf, 0x13b5, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13cf, 0x13b5, 0x13ef, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ca, 0x13be, 0x13a2, 0x20, 0x13d3, 0x13b7, 0x13cc, 0x13b3,
0x13bb, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ac, 0x13be, 0x13d5, 0x13be,
0x20, 0x13a2, 0x13e4, 0x13f3, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a2, 0x13ac, 0x13be, 0x13d5, 0x13be, 0x20, 0x13a2, 0x13e4, 0x13f3, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2,
0x13ac, 0x13be, 0x13d5, 0x13be, 0x20, 0x13a2, 0x13e4, 0x13f3, 0x13cd, 0x13d7, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13bc, 0x13b6, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13dd,
0x13bb, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13dd, 0x13bb, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13dd,
0x13bb, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13e5, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13b5, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5,
0x13b5, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d3, 0x13b6,
0x13c2, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d3, 0x13b6,
0x13c2, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d3, 0x13b6, 0x13c2, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13aa,
0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d3, 0x13c2,
0x13cd, 0x13d3, 0x13f2, 0x13af, 0x13b2, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13aa, 0x13cd, 0x20, 0x13da, 0x13a6, 0x13da,
0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b8, 0x13bb, 0x13c8,
0x13a2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b8, 0x13bb, 0x13c8,
0x13a2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13aa, 0x13b8, 0x13bb, 0x13c8, 0x13a2, 0x13a0, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d3, 0x13cd, 0x13d3, 0x13f4, 0x13b2, 0x13cd,
0x13a9, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a0, 0x13d3, 0x13cd, 0x13d3, 0x13f4, 0x13b2, 0x13cd, 0x13a9, 0x20, 0x13da, 0x13a6,
0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d3, 0x13cd, 0x13d3, 0x13f4, 0x13b2, 0x13cd, 0x13a9, 0x20,
0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ab, 0x13c6, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ab, 0x13c6, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ab, 0x13c6, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d5, 0x13eb, 0x13cf, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13da, 0x13bc, 0x13be, 0x13d8, 0x2d, 0x13d7, 0x2019, 0x13a4, 0x13b5,
0x13eb, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac,
0x20, 0x13d8, 0x13bc, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13cc,
0x13d5, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13cc, 0x13d5, 0x13b4, 0x13af, 0x13cc, 0x13c5,
0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13cc, 0x13d5, 0x13b4, 0x13af, 0x13cc,
0x13c5, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13c6, 0x13d9, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0,
0x13df, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3,
0x13c8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20,
0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7,
0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20,
0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5,
0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac,
0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13a9, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13a9, 0x20, 0x13da, 0x13a6, 0x13da,
0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13e9, 0x13a9, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13eb, 0x13e5, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13eb, 0x13e5, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13eb, 0x13e5, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13a6, 0x13b8, 0x20, 0x13c8, 0x13ef, 0x13be,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13a6, 0x13b8, 0x13e5, 0x20,
0x13a4, 0x13a6, 0x13c3, 0x13ee, 0x20, 0x26, 0x20, 0x13a4, 0x13c1, 0x13cd, 0x13d3, 0x13b6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13c6, 0x20, 0x13d3, 0x13a6, 0x13cf,
0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a6, 0x13bb, 0x13c7, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13a0,
0x13e5, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13a0, 0x13e5, 0x13a2,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13e3, 0x13a0, 0x13e5, 0x13a2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a9, 0x13b5, 0x13c7, 0x13d8, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13a2, 0x13f3, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13a2,
0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13a0, 0x13b5, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7,
0x13b8, 0x13ac, 0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13b5, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac,
0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db,
0x13f1, 0x20, 0x13a0, 0x13b5, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20,
0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13b5, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a2, 0x13e4,
0x13cd, 0x13db, 0x13f1, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a1, 0x13c9, 0x13c4, 0x13b8, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a6, 0x13ef, 0x13be, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ad, 0x13e9, 0x13f1, 0x2d, 0x13a0, 0x13b5, 0x13f3, 0x13cf, 0x13a0, 0x13c2,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ad, 0x13e9, 0x13f1, 0x2d, 0x13a0, 0x13b5,
0x13f3, 0x13cf, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ad, 0x13e9, 0x13f1, 0x2d, 0x13a0, 0x13b5, 0x13f3, 0x13cf,
0x13a0, 0x13c2, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13b0, 0x13c2, 0x13a9, 0x20, 0x13aa, 0x13c2, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13c2, 0x13a9, 0x20, 0x13aa, 0x13c2, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13c2,
0x13a9, 0x20, 0x13aa, 0x13c2, 0x13a9, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13b0, 0x13e9, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b0,
0x13e9, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13b0, 0x13e9, 0x13d7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a2, 0x13c2, 0x13d7, 0x13a2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13f4, 0x13eb, 0x13ef,
0x20, 0x13a0, 0x13ba, 0x13c9, 0x13af, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2,
0x13c2, 0x13d9, 0x13d3, 0x13b6, 0x13c2, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a2, 0x13c2, 0x13d9, 0x13c2, 0x13cd, 0x13ef,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7,
0x13dc, 0x20, 0x13a2, 0x13c2, 0x13d9, 0x13c2, 0x13cd, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a2, 0x13c2,
0x13d9, 0x13c2, 0x13cd, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13b3,
0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13b3, 0x13c2, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13b3,
0x13c2, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a2, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2,
0x13ab, 0x13e5, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13cf, 0x13b5, 0x13f1, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a2, 0x13cf, 0x13b5, 0x13f1, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13cf, 0x13b5, 0x13f1, 0x20, 0x13aa,
0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a9, 0x13e3,
0x13e9, 0x13c2, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13e9, 0x13c2,
0x13cf, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13e3, 0x13e9, 0x13c2, 0x13cf, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a7, 0x13ce, 0x13a7, 0x13cd, 0x13d5, 0x13c2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13a7, 0x13ce,
0x13a7, 0x13cd, 0x13d5, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5,
0x13b5, 0x13ac, 0x20, 0x13a7, 0x13ce, 0x13a7, 0x13cd, 0x13d5, 0x13c2, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b5, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13aa, 0x13b5, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b5, 0x13a0, 0x13c2, 0x20, 0x13aa, 0x13af,
0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13cd, 0x13b4,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13dd, 0x13cd, 0x13c3, 0x13ef, 0x13cd, 0x13a7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13dd, 0x13cd, 0x13c3, 0x13ef, 0x13cd, 0x13a7,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13dd, 0x13cd, 0x13c3, 0x13ef, 0x13cd, 0x13a7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a9, 0x13b5, 0x13e3, 0x13a2, 0x13cd, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13cd, 0x13d3, 0x13c5, 0x13c5, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db,
0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13ac, 0x13eb, 0x13f3, 0x13af,
0x20, 0x13ad, 0x13eb, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13ac, 0x13eb,
0x13f3, 0x13af, 0x20, 0x13ad, 0x13eb, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13ac, 0x13eb, 0x13f3, 0x13af, 0x20, 0x13ad,
0x13eb, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13b9, 0x13a6, 0x13d3, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9,
0x13a6, 0x13d3, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13a6, 0x13d3, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13b4, 0x13cf, 0x13a2, 0x13a0, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13b5, 0x13d7, 0x13eb, 0x13cd, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13b9, 0x13b5, 0x13a8, 0x13cc, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13b9, 0x13cc, 0x13b5, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13bc, 0x13b5, 0x13cf, 0x13a5, 0x13cd, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13bc, 0x13b5, 0x13cf, 0x13a5, 0x13cd, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13bc, 0x13b5, 0x13cf, 0x13a5,
0x13cd, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13cc,
0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13cd, 0x13c6, 0x13c2,
0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13c2, 0x13cd, 0x13c6, 0x13c2, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13cd,
0x13c6, 0x13c2, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2,
0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13b3, 0x13c2, 0x20, 0x13c6,
0x13d9, 0x13b8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13b3, 0x13c2, 0x20,
0x13c6, 0x13d9, 0x13b8, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13b3, 0x13c2, 0x20, 0x13c6, 0x13d9, 0x13b8, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13cd, 0x13a6, 0x13eb, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13cd, 0x13a6, 0x13eb, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13cd, 0x13a6,
0x13eb, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13ef,
0x13b9, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13be, 0x13b7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c1, 0x13c6, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a2, 0x13e4, 0x20, 0x13a7, 0x13b5, 0x13d9, 0x13c2, 0x13a0, 0x13c2, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13a7, 0x13b5, 0x13d9, 0x13c2, 0x13a0,
0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a2, 0x13e4, 0x20, 0x13a7, 0x13b5, 0x13d9, 0x13c2, 0x13a0, 0x13c2, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13cf, 0x13a2,
0x13b4, 0x13c2, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20,
0x13cf, 0x13a2, 0x13b4, 0x13c2, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13cf, 0x13a2, 0x13b4, 0x13c2,
0x13d7, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a2, 0x13e4, 0x13a4, 0x13c2, 0x13e9, 0x13db, 0x13d3, 0x13a6, 0x13d9, 0x13af, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x13a4, 0x13c2, 0x13e9, 0x13db, 0x13d3,
0x13a6, 0x13d9, 0x13af, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x13a4, 0x13c2, 0x13e9, 0x13db, 0x13d3, 0x13a6, 0x13d9,
0x13af, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13c2, 0x13f3, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c3, 0x13b5, 0x13ec,
0x13b5, 0x13a9, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c3, 0x13b5, 0x13ec, 0x13b5, 0x13a9, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db,
0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13c3, 0x13b5, 0x13ec, 0x13b5, 0x13a9, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ea, 0x13be, 0x13c5,
0x13d9, 0x20, 0x13d5, 0x20, 0x13c3, 0x13b6, 0x13be, 0x13ad, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ea, 0x13be, 0x13c5, 0x13d9, 0x20, 0x13d5, 0x20, 0x13c3, 0x13b6, 0x13be,
0x13ad, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13ea, 0x13be, 0x13c5, 0x13d9, 0x20, 0x13d5, 0x20, 0x13c3, 0x13b6, 0x13be, 0x13ad,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c3, 0x13ec, 0x13cf,
0x13c8, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c3, 0x13ec, 0x13cf,
0x13c8, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13c3, 0x13ec, 0x13cf, 0x13c8, 0x13cd, 0x13a7, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13bb, 0x13cd, 0x13a7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13bb, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13bb, 0x13cd, 0x13a7,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13a9, 0x13cd,
0x13d6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13a9, 0x13cd, 0x13d6,
0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13c6, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13b7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6,
0x13c7, 0x20, 0x13a2, 0x13e4, 0x20, 0x13a9, 0x13a2, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c6, 0x13b5, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6,
0x13b5, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c6, 0x13b5, 0x13c7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c7, 0x13b7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c7, 0x13b7,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13c7, 0x13b7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13c2, 0x13c8, 0x13b5, 0x13a9, 0x13c3, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13c2, 0x13c8, 0x13b5, 0x13a9, 0x13c3, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13c8, 0x13b5, 0x13a9, 0x13c3, 0x20,
0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e7, 0x13b4, 0x13af, 0x13cc,
0x13c5, 0x13af, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13c8, 0x13f0, 0x20, 0x26, 0x20,
0x13bb, 0x13c7, 0x13b6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13d3,
0x13c5, 0x13d8, 0x20, 0x13c8, 0x13f0, 0x20, 0x26, 0x20, 0x13bb, 0x13c7, 0x13b6, 0x13c2,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13c8, 0x13f0, 0x20, 0x26, 0x20, 0x13bb, 0x13c7,
0x13b6, 0x13c2, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c8, 0x13a7, 0x13b5, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13c9, 0x13be, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c8, 0x13ef, 0x13c2,
0x13a9, 0x13f0, 0x13c2, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b4, 0x13f3,
0x13c2, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b3, 0x13de, 0x13b3,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc, 0x13a7, 0x13b5, 0x13c2, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc, 0x13a7, 0x13b5, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc, 0x13a7, 0x13b5, 0x13c2,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc, 0x13bc, 0x13a0,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc, 0x13bc, 0x13a0, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc, 0x13bc, 0x13a0,
0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13ce, 0x13e4, 0x13b4, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cf, 0x13c2,
0x13a6, 0x13c9, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13d0, 0x13b6, 0x13b9, 0x13c2, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db,
0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e7, 0x13a6, 0x13be, 0x13ee, 0x20,
0x13e3, 0x13a0, 0x13e5, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d2, 0x13b5,
0x13be, 0x13bb, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cf, 0x13f2, 0x13e9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4, 0x13af, 0x13d8, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13d4, 0x13f1, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4,
0x13f1, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13d4, 0x13f1, 0x13c7, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4, 0x13e5, 0x13a9, 0x13cd, 0x13d5, 0x13c2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d9, 0x13a8, 0x13b3, 0x13a4, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13d9, 0x13be, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13d9, 0x13be, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13d9, 0x13be, 0x13a6, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13e7, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13db,
0x13b5, 0x13a9, 0x13ba, 0x13c2, 0x13cd, 0x13d4, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13db, 0x13b5, 0x13a9, 0x13ba, 0x13c2, 0x13cd, 0x13d4, 0x13c2, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13db, 0x13b5, 0x13a9,
0x13ba, 0x13c2, 0x13cd, 0x13d4, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13da, 0x13e9, 0x13b7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f3,
0x13b7, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f3, 0x13b7, 0x13c7, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f3,
0x13b7, 0x13c7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4,
0x13cd, 0x13c7, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a4, 0x13cd, 0x13c7, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13cd, 0x13c7, 0x13a9, 0x13cd,
0x13d6, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9,
0x13c4, 0x13e9, 0x13da, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13c4, 0x13e9,
0x13da, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13e9, 0x13c4, 0x13e9, 0x13da, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ea, 0x13c1, 0x13d1, 0x13ea, 0x13b3, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13ed, 0x13b3, 0x13d7, 0x13ec, 0x13cd, 0x13d9, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ed, 0x13b3, 0x13d7, 0x13ec, 0x13cd, 0x13d9, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13b3, 0x13d7,
0x13ec, 0x13cd, 0x13d9, 0x13a9, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13ec, 0x13b6, 0x13aa, 0x13dd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13ec, 0x13b6, 0x13aa, 0x13dd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ec, 0x13b6, 0x13aa, 0x13dd, 0x13d7, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ec, 0x13cd, 0x13d9, 0x13a7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13f0, 0x13e8, 0x20, 0x13a4, 0x13a6, 0x13da,
0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13b5, 0x13cd, 0x20,
0x13a0, 0x13b4, 0x20, 0x13ca, 0x13da, 0x13be, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13ef, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ef, 0x13ab,
0x13e5, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ef, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13f0, 0x13a7, 0x13d6, 0x13b5, 0x13c2, 0x13ca, 0x13a6, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f0, 0x13a7, 0x13d6, 0x13b5, 0x13c2, 0x13ca, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f0,
0x13a7, 0x13d6, 0x13b5, 0x13c2, 0x13ca, 0x13a6, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13f3, 0x13a7, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x963f, 0x514b, 0x91cc, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x91cc, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x963f, 0x514b, 0x91cc, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e2d, 0x90e8, 0x975e, 0x6d32, 0x65f6,
0x95f4, 0x4e1c, 0x90e8, 0x975e, 0x6d32, 0x65f6, 0x95f4, 0x897f, 0x90e8, 0x975e, 0x6d32, 0x65f6,
0x95f4, 0x897f, 0x90e8, 0x975e, 0x6d32, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x897f, 0x90e8, 0x975e,
0x6d32, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x5317,
0x7f8e, 0x4e2d, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e2d, 0x90e8, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e1c,
0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x5317, 0x7f8e, 0x5c71, 0x533a, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x5c71, 0x533a, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x5c71, 0x533a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5317,
0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x963f, 0x7eb3, 0x5fb7, 0x5c14, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x7eb3, 0x5fb7, 0x5c14,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x5957, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x5957,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x5957, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f,
0x514b, 0x6258, 0x522b, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258, 0x522b, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x963f, 0x514b, 0x6258, 0x522b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229,
0x4e9a, 0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x90e8, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x5927,
0x5229, 0x4e9a, 0x4e2d, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229,
0x4e9a, 0x4e2d, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a,
0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e1c, 0x90e8, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a,
0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x897f, 0x90e8,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x585e, 0x62dc, 0x7586, 0x65f6, 0x95f4, 0x963f, 0x585e,
0x62dc, 0x7586, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x585e, 0x62dc, 0x7586, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x73bb, 0x5229, 0x7ef4, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6587, 0x83b1,
0x8fbe, 0x9c81, 0x8428, 0x5170, 0x65f6, 0x95f4, 0x4f5b, 0x5f97, 0x89d2, 0x65f6, 0x95f4, 0x4f5b,
0x5f97, 0x89d2, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f5b, 0x5f97, 0x89d2, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x51ef, 0x897f, 0x65f6, 0x95f4, 0x67e5, 0x83ab, 0x7f57, 0x65f6, 0x95f4, 0x67e5, 0x5854,
0x59c6, 0x65f6, 0x95f4, 0x67e5, 0x5854, 0x59c6, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x67e5, 0x5854,
0x59c6, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5e93, 0x514b, 0x7fa4, 0x5c9b, 0x4ef2, 0x590f, 0x65f6,
0x95f4, 0x8fea, 0x8499, 0xb7, 0x8fea, 0x7ef4, 0x5c14, 0x65f6, 0x95f4, 0x5384, 0x74dc, 0x591a,
0x5c14, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6b27, 0x6d32, 0x6781, 0x4e1c, 0x65f6, 0x95f4, 0x6cd5,
0x5c5e, 0x572d, 0x4e9a, 0x90a3, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6cd5, 0x5c5e, 0x5357, 0x65b9,
0x548c, 0x5357, 0x6781, 0x9886, 0x5730, 0x65f6, 0x95f4, 0x79d1, 0x9686, 0x7fa4, 0x5c9b, 0x65f6,
0x95f4, 0x7518, 0x6bd4, 0x5c14, 0x65f6, 0x95f4, 0x683c, 0x9c81, 0x5409, 0x4e9a, 0x65f6, 0x95f4,
0x683c, 0x9c81, 0x5409, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9c81, 0x5409, 0x4e9a,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x683c, 0x6797, 0x5c3c, 0x6cbb, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x683c, 0x9675, 0x5170, 0x5c9b, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x5c9b,
0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x5c9b, 0x4e1c, 0x90e8,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x5c9b, 0x897f, 0x90e8, 0x65f6, 0x95f4,
0x683c, 0x9675, 0x5170, 0x5c9b, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675,
0x5170, 0x5c9b, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5173, 0x5c9b, 0x65f6, 0x95f4,
0x6d77, 0x6e7e, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x65f6, 0x95f4, 0x4e2d, 0x5357,
0x534a, 0x5c9b, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x5c3c, 0x897f, 0x4e9a, 0x4e2d, 0x90e8, 0x65f6,
0x95f4, 0x5370, 0x5ea6, 0x5c3c, 0x897f, 0x4e9a, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x5370, 0x5ea6,
0x5c3c, 0x897f, 0x4e9a, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x8328, 0x514b,
0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x8328, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f0a,
0x5c14, 0x5e93, 0x8328, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4,
0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x2d, 0x582a, 0x5bdf, 0x52a0, 0x65f6, 0x95f4, 0x5f7c,
0x5f97, 0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x2d, 0x582a, 0x5bdf, 0x52a0,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af,
0x514b, 0x2d, 0x582a, 0x5bdf, 0x52a0, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x54c8, 0x8428, 0x514b,
0x65af, 0x5766, 0x65f6, 0x95f4, 0x54c8, 0x8428, 0x514b, 0x65af, 0x5766, 0x4e1c, 0x90e8, 0x65f6,
0x95f4, 0x54c8, 0x8428, 0x514b, 0x65af, 0x5766, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x79d1, 0x65af,
0x96f7, 0x65f6, 0x95f4, 0x5409, 0x5c14, 0x5409, 0x65af, 0x65af, 0x5766, 0x65f6, 0x95f4, 0x9a6c,
0x5c14, 0x4ee3, 0x592b, 0x65f6, 0x95f4, 0x9a6c, 0x514b, 0x8428, 0x65af, 0x7fa4, 0x5c9b, 0x65f6,
0x95f4, 0x6bdb, 0x91cc, 0x6c42, 0x65af, 0x65f6, 0x95f4, 0x6bdb, 0x91cc, 0x6c42, 0x65af, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x6bdb, 0x91cc, 0x6c42, 0x65af, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7459,
0x9c81, 0x65f6, 0x95f4, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9a, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x65b0, 0x897f, 0x5170, 0x65f6, 0x95f4, 0x65b0, 0x897f, 0x5170, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x65b0, 0x897f, 0x5170, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7ebd, 0x57c3, 0x65f6, 0x95f4,
0x8d39, 0x5c14, 0x5357, 0x591a, 0x2d, 0x8fea, 0x8bfa, 0x7f57, 0x5c3c, 0x4e9a, 0x5c9b, 0x65f6,
0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x2d, 0x8fea, 0x8bfa, 0x7f57, 0x5c3c, 0x4e9a, 0x5c9b,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x2d, 0x8fea, 0x8bfa, 0x7f57,
0x5c3c, 0x4e9a, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5e15, 0x52b3, 0x65f6, 0x95f4, 0x5df4,
0x5e03, 0x4e9a, 0x65b0, 0x51e0, 0x5185, 0x4e9a, 0x65f6, 0x95f4, 0x83f2, 0x5c3c, 0x514b, 0x65af,
0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x76ae, 0x7279, 0x51ef, 0x6069, 0x65f6, 0x95f4, 0x514b, 0x5b5c,
0x6d1b, 0x5c14, 0x8fbe, 0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x6d1b, 0x5c14, 0x8fbe, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x6d1b, 0x5c14, 0x8fbe, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7559,
0x5c3c, 0x6c6a, 0x65f6, 0x95f4, 0x585e, 0x820c, 0x5c14, 0x65f6, 0x95f4, 0x6240, 0x7f57, 0x95e8,
0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5357, 0x4e54, 0x6cbb, 0x4e9a, 0x5c9b, 0x65f6, 0x95f4, 0x82cf,
0x91cc, 0x5357, 0x65f6, 0x95f4, 0x662d, 0x548c, 0x65f6, 0x95f4, 0x5854, 0x5e0c, 0x63d0, 0x5c9b,
0x65f6, 0x95f4, 0x5854, 0x5409, 0x514b, 0x65af, 0x5766, 0x65f6, 0x95f4, 0x6258, 0x514b, 0x52b3,
0x65f6, 0x95f4, 0x6c64, 0x52a0, 0x65f6, 0x95f4, 0x6c64, 0x52a0, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x6c64, 0x52a0, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x695a, 0x514b, 0x65f6, 0x95f4, 0x571f, 0x5e93,
0x66fc, 0x65af, 0x5766, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x65af, 0x5766, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x65af, 0x5766, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x56fe,
0x74e6, 0x5362, 0x65f6, 0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x65af, 0x5766, 0x65f6, 0x95f4,
0x4e4c, 0x5179, 0x522b, 0x514b, 0x65af, 0x5766, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e4c, 0x5179,
0x522b, 0x514b, 0x65af, 0x5766, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x74e6, 0x52aa, 0x963f, 0x56fe,
0x65f6, 0x95f4, 0x74e6, 0x52aa, 0x963f, 0x56fe, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x74e6, 0x52aa,
0x963f, 0x56fe, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x74e6, 0x5229, 0x65af, 0x548c, 0x5bcc, 0x56fe,
0x7eb3, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x8328, 0x514b, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x8328,
0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x8328, 0x514b, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x5357, 0x975e, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x4e2d, 0x90e8, 0x6642, 0x9593, 0x5317,
0x7f8e, 0x4e2d, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x4e2d, 0x90e8, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x6771, 0x90e8, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x6771,
0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x6771, 0x90e8, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x5317, 0x7f8e, 0x5c71, 0x5340, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x5c71, 0x5340, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x5c71, 0x5340, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5317,
0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x4e9e, 0x585e, 0x62dc, 0x7586, 0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7586, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7586, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4f5b, 0x5f97,
0x89d2, 0x6642, 0x9593, 0x4f5b, 0x5f97, 0x89d2, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4f5b, 0x5f97,
0x89d2, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x53ef, 0x53ef, 0x65af, 0x7fa4, 0x5cf6, 0x6642, 0x9593,
0x8fea, 0x8499, 0x8fea, 0x7dad, 0x723e, 0x6642, 0x9593, 0x5384, 0x74dc, 0x591a, 0x723e, 0x6642,
0x9593, 0x52a0, 0x62c9, 0x5e15, 0x6208, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x683c, 0x9b6f, 0x5409,
0x4e9e, 0x6642, 0x9593, 0x683c, 0x9b6f, 0x5409, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x683c,
0x9b6f, 0x5409, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6ce2, 0x65af, 0x7063, 0x6d77, 0x57df,
0x6642, 0x9593, 0x5370, 0x5ea6, 0x6642, 0x9593, 0x4f0a, 0x723e, 0x5eab, 0x8328, 0x514b, 0x6642,
0x9593, 0x4f0a, 0x723e, 0x5eab, 0x8328, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4f0a, 0x723e,
0x5eab, 0x8328, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x79d1, 0x65af, 0x96f7, 0x6642, 0x9593,
0x99ac, 0x723e, 0x4ee3, 0x592b, 0x6642, 0x9593, 0x99ac, 0x514b, 0x85a9, 0x65af, 0x6642, 0x9593,
0x6bdb, 0x91cc, 0x88d8, 0x65af, 0x6642, 0x9593, 0x6bdb, 0x91cc, 0x88d8, 0x65af, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x6bdb, 0x91cc, 0x88d8, 0x65af, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7459, 0x9b6f,
0x6642, 0x9593, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x8cbb, 0x723e, 0x5357, 0x591a, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x6642, 0x9593, 0x8cbb,
0x723e, 0x5357, 0x591a, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x8cbb, 0x723e, 0x5357, 0x591a, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x5df4, 0x5e03, 0x4e9e, 0x65b0, 0x757f, 0x5167, 0x4e9e, 0x6642, 0x9593, 0x76ae, 0x7279,
0x5eb7, 0x6642, 0x9593, 0x585e, 0x820c, 0x723e, 0x6642, 0x9593, 0x65b0, 0x52a0, 0x5761, 0x6642,
0x9593, 0x6240, 0x7f85, 0x9580, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x8607, 0x91cc, 0x5357, 0x6642,
0x9593, 0x6e6f, 0x52a0, 0x6642, 0x9593, 0x6e6f, 0x52a0, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6e6f,
0x52a0, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5716, 0x74e6, 0x76e7, 0x6642, 0x9593, 0x74e6, 0x52aa,
0x963f, 0x5716, 0x6642, 0x9593, 0x74e6, 0x52aa, 0x963f, 0x5716, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x74e6, 0x52aa, 0x963f, 0x5716, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x96c5, 0x5eab, 0x8328, 0x514b,
0x6642, 0x9593, 0x96c5, 0x5eab, 0x8328, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x96c5, 0x5eab,
0x8328, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e2d, 0x5357, 0x534a, 0x5cf6, 0x6642, 0x9593,
0x8056, 0x76ae, 0x57c3, 0x8207, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x8056,
0x76ae, 0x57c3, 0x8207, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x8056, 0x76ae, 0x57c3, 0x8207, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x80b2, 0x7a7a, 0x5730, 0x5340, 0x6642, 0x9593, 0x441, 0x440, 0x435, 0x434, 0x43d,
0x435, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x441, 0x440, 0x435,
0x434, 0x43d, 0x435, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x441, 0x440, 0x435, 0x434, 0x43d,
0x435, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430,
0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430,
0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x43c,
0x435, 0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20,
0x43d, 0x430, 0x433, 0x43e, 0x301, 0x440, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430,
0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43d, 0x430, 0x433, 0x43e, 0x301,
0x440, 0x43d, 0x435, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x43c, 0x435, 0x440,
0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43d, 0x430,
0x433, 0x43e, 0x301, 0x440, 0x43d, 0x435, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x442, 0x438,
0x445, 0x43e, 0x47b, 0x43a, 0x435, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x442, 0x438, 0x445, 0x43e, 0x47b,
0x43a, 0x435, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438,
0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x442, 0x438, 0x445, 0x43e, 0x47b, 0x43a, 0x435, 0x430, 0x301, 0x43d, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x456, 0x301, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x430, 0x486, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x456, 0x301,
0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x456, 0x301, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x441, 0x440, 0x435, 0x434, 0x43d, 0x435, 0x454, 0x475, 0x440,
0x461, 0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x441, 0x440, 0x435, 0x434, 0x43d, 0x435, 0x454, 0x475,
0x440, 0x461, 0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437,
0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x441, 0x440, 0x435, 0x434, 0x43d, 0x435, 0x454, 0x475, 0x440, 0x461, 0x43f,
0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461, 0x43f, 0x435,
0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x454, 0x475, 0x440,
0x461, 0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438,
0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461,
0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0x20, 0x432, 0x44a, 0x20, 0x43a, 0x430, 0x43b,
0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x301, 0x434, 0x463, 0x20, 0x438,
0x486, 0x20, 0x43c, 0x438, 0x301, 0x43d, 0x441, 0x43a, 0x463, 0x437, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461, 0x43f, 0x435, 0x301, 0x439,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x437,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461, 0x43f, 0x435,
0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x437, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461, 0x43f, 0x435, 0x301, 0x439,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x441, 0x440, 0x435, 0x301, 0x434,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x20, 0x43f,
0x43e, 0x20, 0x433, 0x440, 0x456, 0x301, 0x43d, 0xa64b, 0x438, 0x447, 0xa64b, 0x438,
0x486, 0x440, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0x438, 0x486, 0x440, 0x43a, 0xa64b, 0x301, 0x442,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x438, 0x486, 0x440, 0x43a, 0xa64b,
0x301, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x301, 0x447, 0x43d, 0x44b, 0x439, 0x20, 0x43a, 0x430, 0x437, 0x430,
0x445, 0x441, 0x442, 0x430, 0x301, 0x43d, 0x44a, 0x437, 0x430, 0x301, 0x43f, 0x430,
0x434, 0x43d, 0x44b, 0x439, 0x20, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442,
0x430, 0x301, 0x43d, 0x44a, 0x43a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x467, 0x301,
0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x43a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x467, 0x301, 0x440, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x43a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x467, 0x301,
0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43a, 0x438, 0x440, 0x433,
0x438, 0x301, 0x437, 0x457, 0x430, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x301,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x301, 0x43d, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x301,
0x432, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x301, 0x432, 0x441, 0x43a, 0x43e, 0x435, 0x20,
0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x301, 0x432, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x301, 0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x301, 0x440, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x301, 0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x47b,
0x486, 0x301, 0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x47b, 0x486, 0x301, 0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437,
0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x47b, 0x486, 0x301, 0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463,
0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x301, 0x43d, 0x463, 0x437, 0x438, 0x301, 0x43c, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x20, 0x43d, 0x430,
0x20, 0x441, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x301, 0x43d, 0x463, 0x43b, 0x463,
0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x20, 0x43d, 0x430, 0x20, 0x441, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x301, 0x43d,
0x463, 0x432, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x301,
0x446, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432,
0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x301, 0x446, 0x43a,
0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e,
0x441, 0x442, 0x43e, 0x301, 0x446, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x301, 0x434, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x301, 0x434, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437,
0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x301, 0x434, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0xa657, 0x486, 0x43a, 0xa64b, 0x301, 0x442,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0xa657,
0x486, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438,
0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0xa657, 0x486, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b,
0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x454, 0x486, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x457, 0x43d, 0x431, 0xa64b,
0x301, 0x440, 0x436, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x454, 0x486, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x457, 0x43d, 0x431,
0xa64b, 0x301, 0x440, 0x436, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301,
0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x454,
0x486, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x457, 0x43d, 0x431, 0xa64b, 0x301, 0x440,
0x436, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x410, 0x43a, 0x440, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43a, 0x440, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43a, 0x440, 0x438, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x444, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7,
0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x4d1,
0x43d, 0x442, 0x4d1, 0x440, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43b, 0x44f, 0x441, 0x43a,
0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43b, 0x44f, 0x441,
0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43b, 0x44f, 0x441,
0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x44b, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x44b,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x44b,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d,
0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43c, 0x430, 0x437,
0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435,
0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432,
0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432,
0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x443, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x20, 0x28, 0x4aa, 0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20,
0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x442, 0x443, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x20, 0x28, 0x4aa, 0x443, 0x440, 0x4ab, 0x4d7, 0x440,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x4aa, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x442, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x20, 0x28, 0x4aa, 0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x41b, 0x4d1, 0x43f, 0x43a, 0x4d1,
0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41b, 0x4d1, 0x43f, 0x43a, 0x4d1, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x4d1, 0x43f, 0x43a, 0x4d1, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x430, 0x434,
0x44b, 0x440, 0x44c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d,
0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43f, 0x438,
0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43f, 0x438, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43a, 0x442, 0x43e,
0x431, 0x435, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43a, 0x442,
0x43e, 0x431, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43a, 0x442,
0x43e, 0x431, 0x435, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x430, 0x43f, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x430, 0x43f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x430, 0x43f, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d,
0x4d1, 0x4ab, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438,
0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x20, 0x430, 0x43d, 0x4d1, 0x4ab, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x20, 0x430, 0x43d, 0x4d1, 0x4ab, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422,
0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20,
0x430, 0x43d, 0x4d1, 0x4ab, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b,
0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432,
0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7,
0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436,
0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x435,
0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430,
0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x443, 0x442, 0x440,
0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7,
0x441, 0x435, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x43e,
0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x411, 0x443, 0x442,
0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x411, 0x43e, 0x43b,
0x438, 0x432, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x411, 0x440,
0x430, 0x437, 0x438, 0x43b, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x411,
0x440, 0x443, 0x43d, 0x435, 0x439, 0x2d, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441,
0x430, 0x43b, 0x430, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x430, 0x431, 0x43e,
0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x435, 0x439,
0x441, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427, 0x430, 0x43c,
0x43e, 0x440, 0x440, 0x43e, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427,
0x430, 0x442, 0x435, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427,
0x430, 0x442, 0x435, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427, 0x430,
0x442, 0x435, 0x43c, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x438,
0x442, 0x430, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x438,
0x442, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x438, 0x442,
0x430, 0x439, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x420, 0x430, 0x448, 0x442, 0x430, 0x432, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x443, 0x442, 0x440, 0x430,
0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41a, 0x443, 0x43a, 0x4d1, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432,
0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x443, 0x43a, 0x4d1, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441,
0x435, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x443, 0x43a, 0x4d1,
0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x443, 0x431, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x414, 0x435, 0x439, 0x432, 0x438, 0x441,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x414, 0x44e, 0x43c, 0x43e, 0x43d,
0x2d, 0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x44c, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445,
0x4d1, 0x4ab, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41c, 0x4d1, 0x43d, 0x43a, 0x443, 0x43d, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41c, 0x4d1, 0x43d, 0x43a, 0x443, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432,
0x4d7, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x4d1, 0x43d, 0x43a,
0x443, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x43d, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x422, 0x4d7, 0x43f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x43d, 0x4ab, 0x435,
0x442, 0x2d, 0x445, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x424, 0x438, 0x434, 0x436, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x424, 0x438, 0x434, 0x436, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x424, 0x438, 0x434, 0x436, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x440, 0x430, 0x43d,
0x446, 0x438, 0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x438, 0x20, 0x41a,
0x4d1, 0x43d, 0x442, 0x4d1, 0x440, 0x43f, 0x430, 0x20, 0x410, 0x43d, 0x442, 0x430,
0x440, 0x43a, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x435, 0x440, 0x440, 0x438,
0x442, 0x43e, 0x440, 0x438, 0x439, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e,
0x441, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x443, 0x437, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x443, 0x437, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x443, 0x437, 0x438, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x4d1, 0x43d, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x43f,
0x430, 0x20, 0x432, 0x4d1, 0x442, 0x430, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432,
0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x413, 0x440, 0x438, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x413,
0x440, 0x438, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab,
0x20, 0x413, 0x440, 0x438, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d,
0x4d1, 0x4ab, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x413, 0x443, 0x430, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f,
0x435, 0x440, 0x441, 0x438, 0x20, 0x437, 0x430, 0x43b, 0x438, 0x432, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x432, 0x430, 0x439, 0x438,
0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x413, 0x430, 0x432, 0x430, 0x439, 0x438, 0x20, 0x410, 0x43b, 0x435, 0x443,
0x442, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x432, 0x430, 0x439,
0x438, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x43e,
0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425,
0x43e, 0x432, 0x434, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x43e,
0x432, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x43e, 0x432, 0x434,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x418, 0x43d, 0x434, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x418, 0x43d, 0x434, 0x438, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x4d7, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x43d, 0x434, 0x43e,
0x43a, 0x438, 0x442, 0x430, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x422, 0x4d7, 0x43f, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b,
0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1,
0x4ab, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418,
0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x437,
0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x42f,
0x43f, 0x43e, 0x43d, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x42f,
0x43f, 0x43e, 0x43d, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x42f, 0x43f,
0x43e, 0x43d, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f,
0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447,
0x430, 0x442, 0x43a, 0x103, 0x440, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432,
0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x43a, 0x103, 0x440,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x435, 0x442, 0x440, 0x43e,
0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c,
0x447, 0x430, 0x442, 0x43a, 0x103, 0x440, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x430,
0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab,
0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x41a, 0x430,
0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x43e, 0x441,
0x440, 0x430, 0x435, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x438, 0x440, 0x433,
0x438, 0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x430,
0x43d, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x430,
0x439, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x43e, 0x440, 0x434, 0x2d,
0x425, 0x430, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x43e,
0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c,
0x430, 0x43a, 0x430, 0x43e, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c,
0x430, 0x43a, 0x430, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430,
0x43a, 0x430, 0x43e, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430,
0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x43b, 0x430, 0x439,
0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x43b,
0x44c, 0x434, 0x438, 0x432, 0x441, 0x435, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x430, 0x441, 0x20, 0x443,
0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x20, 0x443,
0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a,
0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x432, 0x440,
0x438, 0x43a, 0x438, 0x439, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x43e, 0x443, 0x441, 0x43e,
0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x435, 0x43a, 0x441,
0x438, 0x43a, 0x430, 0x20, 0x41b, 0x4d1, 0x43f, 0x43a, 0x4d1, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x435,
0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x41b, 0x4d1, 0x43f, 0x43a, 0x4d1, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c,
0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x41b, 0x4d1, 0x43f, 0x43a, 0x4d1,
0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x43b, 0x430,
0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411,
0x430, 0x442, 0x43e, 0x440, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x443, 0x441, 0x43a, 0x430,
0x432, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x443, 0x441, 0x43a,
0x430, 0x432, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x443, 0x441, 0x43a,
0x430, 0x432, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c, 0x430, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434,
0x43e, 0x43d, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d,
0x4d7, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7,
0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443,
0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x438, 0x443,
0x44d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x43e, 0x440, 0x444,
0x43e, 0x43b, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430,
0x440, 0x438, 0x430, 0x43d, 0x441, 0x435, 0x43d, 0x20, 0x4ab, 0x443, 0x440, 0x4ab,
0x4d7, 0x440, 0x442, 0x438, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441,
0x435, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x43e, 0x432,
0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440,
0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41e, 0x43c,
0x441, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41e, 0x43c, 0x441,
0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41e, 0x43c, 0x441, 0x43a, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20,
0x2014, 0x20, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435,
0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41f, 0x435, 0x440, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f,
0x435, 0x440, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x435, 0x440,
0x443, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d,
0x441, 0x435, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x438,
0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x441, 0x435, 0x43c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x441,
0x435, 0x43c, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c,
0x435, 0x440, 0x20, 0x442, 0x430, 0x442, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435,
0x43b, 0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x435,
0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x442, 0x430, 0x442, 0x430, 0x20,
0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x442, 0x430,
0x442, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434,
0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x44b, 0x437, 0x44b,
0x43b, 0x43e, 0x440, 0x434, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x420,
0x435, 0x44e, 0x43d, 0x44c, 0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430,
0x43c, 0x43e, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430,
0x43c, 0x43e, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430, 0x43c,
0x43e, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x43e, 0x43b, 0x43e, 0x43c,
0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x4d1, 0x43d,
0x442, 0x4d1, 0x440, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x451, 0x432, 0x430, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x438, 0x442, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x439, 0x43f, 0x435,
0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x439, 0x43f, 0x435,
0x439, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x434, 0x436, 0x438, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x43e,
0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x440, 0x443, 0x43a, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435,
0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x443,
0x432, 0x430, 0x43b, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423,
0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x437, 0x431,
0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412,
0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e,
0x43a, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x423, 0x44d, 0x439, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423,
0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x442, 0x430, 0x442, 0x430, 0x20, 0x424,
0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x415, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x415,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x42e, 0x43a, 0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x66,
0x66, 0x72, 0x65, 0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x69, 0x67, 0x67, 0x4f, 0xdf, 0x2d, 0x41, 0x66, 0x66, 0x72,
0x65, 0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a,
0x69, 0x67, 0x67, 0x53, 0xf6, 0x64, 0x2d, 0x41, 0x66, 0x66, 0x72, 0x65,
0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x69,
0x67, 0x67, 0x57, 0xe4, 0xdf, 0x2d, 0x41, 0x66, 0x66, 0x72, 0x65, 0x6b,
0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x69, 0x67,
0x67, 0x4a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65,
0x20, 0x57, 0xe4, 0xdf, 0x2d, 0x41, 0x66, 0x66, 0x72, 0x65, 0x6b, 0x61,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x69, 0x67, 0x67,
0x57, 0xe4, 0xdf, 0x2d, 0x41, 0x66, 0x66, 0x72, 0x65, 0x6b, 0x61, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x69, 0x67, 0x67, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x20, 0x69, 0x65, 0x72, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x64, 0x65,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x20, 0x69, 0x65, 0x72, 0x20, 0x6a,
0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65, 0x20, 0x5a,
0x69, 0x63, 0x6b, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x20,
0x69, 0x65, 0x72, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69,
0x63, 0x6b, 0x4b, 0x61, 0x70, 0x76, 0xe4, 0x72, 0x64, 0x65, 0x6a, 0x61,
0x61, 0x6e, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x69, 0x67, 0x67, 0x4a,
0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65, 0x20, 0x4b,
0x61, 0x70, 0x76, 0xe4, 0x72, 0x64, 0x65, 0x6a, 0x61, 0x61, 0x6e, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x69, 0x67, 0x67, 0x4b, 0x61, 0x70, 0x76,
0xe4, 0x72, 0x64, 0x65, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x65,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x67, 0x67, 0x4d,
0x65, 0x64, 0x64, 0x65, 0x6c, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70,
0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x4d,
0x65, 0x64, 0x64, 0x65, 0x6c, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70,
0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x6a, 0x65, 0x77, 0xf6, 0x68,
0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x4d,
0x65, 0x64, 0x64, 0x65, 0x6c, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70,
0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x69, 0x63, 0x6b, 0x4f, 0xdf, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x5a, 0x69, 0x63,
0x6b, 0x4f, 0xdf, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70, 0x61, 0x20,
0x73, 0x69, 0x6e, 0x67, 0x20, 0x6a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c,
0x69, 0x6a, 0x6a, 0x65, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x4f, 0xdf, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b, 0x57,
0x65, 0xdf, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70, 0x61, 0x20, 0x73,
0x69, 0x6e, 0x67, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x57, 0x65, 0xdf, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67,
0x20, 0x6a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65,
0x20, 0x5a, 0x69, 0x63, 0x6b, 0x57, 0x65, 0xdf, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x74, 0x2d, 0x5a, 0x69, 0x63, 0x6b, 0x64,
0x65, 0x6d, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x4f, 0x7a, 0x65, 0x6a, 0x61, 0x6e, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20,
0x5a, 0x69, 0x63, 0x6b, 0x5a, 0x69, 0x67, 0x67, 0x20, 0x76, 0x75, 0x6e,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4a, 0x65,
0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65, 0x20, 0x5a, 0x69,
0x67, 0x67, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69,
0x67, 0x67, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x5a, 0x69, 0x67, 0x67, 0x20, 0x76, 0x75, 0x6e,
0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x5a, 0x69, 0x67, 0x67,
0x20, 0x76, 0x75, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65, 0x69, 0x73,
0x63, 0x68, 0xe4, 0x6c, 0x6c, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x6d, 0x65,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x41, 0x63, 0x72, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x61, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65,
0x64, 0x6e, 0x6a, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x61, 0x66,
0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x61, 0x6c, 0x6a, 0x61, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x6a, 0x61, 0x161, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x6a, 0x61, 0x161, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x74, 0x6d, 0x61, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x74,
0x6d, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x61, 0x6c, 0x74, 0x6d, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72,
0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x69,
0x161, 0x6e, 0x6a, 0x65, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6e, 0x61, 0x64, 0x69,
0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x61, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x41, 0x70, 0x69, 0x6a, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x41, 0x70, 0x69, 0x6a, 0x65, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x41, 0x70, 0x69, 0x6a,
0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61,
0x64, 0x61, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x41, 0x6b, 0x74,
0x61, 0x75, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x41,
0x6b, 0x74, 0x61, 0x75, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61,
0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72,
0x61, 0x64, 0x61, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x61, 0x72,
0x61, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x61, 0x72, 0x61, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x61, 0x72, 0x61, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x72, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x6e,
0x6a, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64,
0x6e, 0x6a, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64,
0x6e, 0x6a, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a, 0x65,
0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a, 0x65,
0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a, 0x65, 0x20,
0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64,
0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a, 0x6f, 0x72, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a, 0x6f,
0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x75,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x62, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x72, 0x61, 0x7a,
0x69, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72,
0x74, 0x73, 0x6b, 0x6f, 0x67, 0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x65, 0x6e,
0x6f, 0x72, 0x74, 0x73, 0x6b, 0x6f, 0x67, 0x20, 0x6f, 0x74, 0x6f, 0x10d,
0x6a, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72,
0x74, 0x73, 0x6b, 0x6f, 0x67, 0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a, 0x61,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x61, 0x73, 0x65,
0x79, 0x6a, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x61,
0x10d, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10d, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10d, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x69, 0x6e,
0x65, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b,
0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x42, 0x6f, 0x17e, 0x69, 0x107, 0x6e, 0x6f,
0x67, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0x69, 0x68,
0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0x69,
0x68, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0x69, 0x68, 0x20, 0x4f,
0x74, 0x6f, 0x6b, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0x69, 0x20,
0x4f, 0x74, 0x6f, 0x63, 0x69, 0x2c, 0x20, 0x70, 0x6f, 0x6c, 0x75, 0x73,
0x61, 0x74, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6d, 0x61, 0x6b, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x75, 0x62,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x61, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x69,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e,
0x6a, 0x65, 0x67, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x65,
0x67, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x55, 0x73,
0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x65, 0x67, 0x20, 0x6f, 0x74, 0x6f, 0x6b,
0x61, 0x65, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x6e,
0x6a, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a,
0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x64, 0x61, 0x6c, 0x65, 0x6b, 0x6f, 0x69, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x20, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x66, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x66,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x46, 0x69, 0x64,
0x17e, 0x69, 0x6a, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x46,
0x69, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x46, 0x69, 0x64,
0x17e, 0x69, 0x6a, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x65, 0x20, 0x47, 0x69,
0x6a, 0x61, 0x6e, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x69, 0x68, 0x20, 0x4a,
0x75, 0x17e, 0x6e, 0x69, 0x68, 0x20, 0x54, 0x65, 0x72, 0x69, 0x74, 0x6f,
0x72, 0x69, 0x6a, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x61, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x61, 0x67, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x67, 0x72, 0x75, 0x7a, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x67,
0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x6f, 0x76, 0x69, 0x68, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61,
0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x7a, 0x61, 0x6c, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x67, 0x75, 0x61, 0x6d, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x6c,
0x6a, 0x65, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c,
0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x68, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x68, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x68, 0x6f,
0x76, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x69, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x67, 0x20, 0x6f,
0x63, 0x65, 0x61, 0x6e, 0x61, 0x69, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e,
0x65, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x72, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69,
0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x61, 0x70, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x61,
0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x6b,
0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x6f, 0x72, 0x65, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6c, 0x61, 0x6e, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x45, 0x6b, 0x76, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b,
0x69, 0x68, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6f, 0x74, 0x6f, 0x6b,
0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6d,
0x61, 0x6b, 0x61, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x6d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x6d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x6c, 0x65, 0x7a, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x72, 0x6b, 0x69, 0x17e, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x161, 0x61,
0x6c, 0x6f, 0x76, 0x69, 0x68, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x63, 0x69, 0x6a, 0x75, 0x73, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x61,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75,
0x73, 0x61, 0x6d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x65, 0x6b, 0x73, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x65, 0x6b,
0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x65,
0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x6c,
0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x6f,
0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x6a,
0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x61, 0x6e, 0x65, 0x70, 0x61, 0x6c, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x65, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a,
0x65, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x6f,
0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4e, 0x69, 0x75,
0x65, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4f, 0x74,
0x6f, 0x6b, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4f, 0x74,
0x6f, 0x6b, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f,
0x6d, 0x61, 0x72, 0x69, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x68, 0x20,
0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f,
0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x50, 0x61, 0x70,
0x75, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x47, 0x76, 0x69, 0x6e,
0x65, 0x6a, 0x65, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x61,
0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70,
0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x66, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4f,
0x74, 0x6f, 0x6b, 0x61, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x7a, 0x61, 0x20, 0x53,
0x76, 0x65, 0x74, 0x69, 0x20, 0x50, 0x65, 0x74, 0x61, 0x72, 0x20, 0x69,
0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x76, 0x65, 0x74, 0x69, 0x20,
0x50, 0x65, 0x74, 0x61, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x76, 0x65,
0x74, 0x69, 0x20, 0x50, 0x65, 0x74, 0x61, 0x72, 0x20, 0x69, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x70, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x70, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x6a, 0x6f, 0x6e, 0x67,
0x6a, 0x61, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72,
0x61, 0x64, 0x61, 0x20, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64,
0x61, 0x20, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x65, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x4b, 0x69, 0x7a, 0x69,
0x6c, 0x6f, 0x72, 0x64, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x61, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x65,
0x73, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73,
0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x73, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d, 0x61, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73,
0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x73, 0x65, 0x6a, 0x161, 0x65, 0x6c, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x53,
0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f, 0x76, 0x69, 0x68, 0x20, 0x4f,
0x74, 0x6f, 0x6b, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x4a, 0x75, 0x17e, 0x6e, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x6a, 0x65, 0x73, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x65, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x6a,
0x61, 0x74, 0x61, 0x6a, 0x70, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x61, 0x6a, 0x70, 0x65, 0x161, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x61, 0x6a, 0x70,
0x65, 0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x61, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x65,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x65, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x61, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75,
0x61, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x72, 0x75, 0x67, 0x76,
0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x76, 0x65, 0x6e, 0x65, 0x7a,
0x75, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x10d, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6a, 0x75, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x63, 0x72, 0x65, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x72, 0x65, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x66,
0x67, 0x68, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x74, 0x159, 0x65, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61,
0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x69,
0x68, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f,
0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1,
0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6a,
0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6a,
0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6a, 0x61,
0x161, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe1,
0x6c, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0xe1, 0x6c, 0x6e, 0xed, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe1, 0x6c, 0x6e, 0xed, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65,
0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x76,
0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0xed, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0xed,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x70, 0x69, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x70, 0x69, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x70, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6b, 0x74,
0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6b, 0x74,
0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x61,
0x75, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x74, 0x159, 0x65, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x7a, 0xe1,
0x70, 0x61, 0x64, 0x6e, 0xed, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65,
0x64, 0x6f, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e, 0xed, 0x20, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x74, 0x159, 0x65, 0x64, 0x6f, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e, 0xed,
0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0xe1, 0x7a,
0x65, 0x72, 0x62, 0xe1, 0x6a, 0x64, 0x17e, 0xe1, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0xe1, 0x7a, 0x65, 0x72, 0x62, 0xe1, 0x6a, 0x64,
0x17e, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0xe1, 0x7a, 0x65,
0x72, 0x62, 0xe1, 0x6a, 0x64, 0x17e, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x6f,
0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x6f, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x6f, 0x72, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x62,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x161, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x161,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0xe9, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x62, 0x68, 0xfa, 0x74, 0xe1, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x62, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x75,
0x6e, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61,
0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x68, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x63, 0x65, 0x63, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x56, 0xe1,
0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72,
0x6f, 0x76, 0x61, 0x10d, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73,
0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x16f, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f,
0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76,
0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x6b,
0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b,
0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x75,
0x62, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x44, 0x61, 0x76, 0x69,
0x73, 0x6f, 0x76, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65,
0x10d, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x20,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x20, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x61, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f,
0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f,
0x10d, 0x6e, 0xed, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f,
0x10d, 0x6e, 0xed, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x61, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x56,
0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x68, 0x6f,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x65, 0x6b, 0x76, 0xe1,
0x64, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74,
0x159, 0x65, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x65, 0x76,
0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159,
0x65, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd,
0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x64,
0xe1, 0x6c, 0x6e, 0x11b, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1,
0x70, 0x61, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x66, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x72, 0x61, 0x6e,
0x63, 0x6f, 0x75, 0x7a, 0x73, 0x6b, 0x6f, 0x67, 0x75, 0x79, 0x61, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x63, 0x6f, 0x75, 0x7a, 0x73, 0x6b, 0xfd, 0x63, 0x68,
0x20, 0x6a, 0x69, 0x17e, 0x6e, 0xed, 0x63, 0x68, 0x20, 0x61, 0x20, 0x61,
0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x63, 0x6b, 0xfd, 0x63, 0x68,
0x20, 0xfa, 0x7a, 0x65, 0x6d, 0xed, 0x67, 0x61, 0x6c, 0x61, 0x70, 0xe1,
0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x72,
0x75, 0x7a, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x67,
0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x67,
0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x67, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x159, 0x65, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f,
0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67,
0x72, 0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1,
0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x47, 0x75, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x20, 0x50, 0x65, 0x72, 0x73, 0x6b, 0xe9, 0x68, 0x6f, 0x20,
0x7a, 0xe1, 0x6c, 0x69, 0x76, 0x75, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d,
0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x68,
0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x69, 0x63, 0x6b, 0x6f,
0x6f, 0x63, 0x65, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x69, 0x6e, 0x64, 0x6f, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f,
0x6e, 0xe9, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0xed, 0x72, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0xed,
0x72, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0xed, 0x72, 0xe1,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x7a, 0x72, 0x61, 0x65,
0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x7a, 0x72, 0x61,
0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x7a, 0x72, 0x61,
0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x6a, 0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b,
0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x70, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x6f, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe1, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe1, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f,
0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe1, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f,
0x73, 0x72, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x72, 0xed, 0x6c, 0x61, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x52, 0x6f,
0x76, 0x6e, 0xed, 0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x16f, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x61, 0x20, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x48,
0x6f, 0x77, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x61, 0x20, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x48, 0x6f, 0x77, 0x61,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x61, 0x20, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20,
0x48, 0x6f, 0x77, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x63, 0x61, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x4d, 0x61, 0x63, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x6c, 0x61,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x6c, 0x65,
0x64, 0x69, 0x76, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61,
0x72, 0x6b, 0xe9, 0x7a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f,
0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f,
0x6d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x6f, 0x76, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x6d,
0x65, 0x78, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x6b, 0xfd, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6c, 0xe1, 0x6e, 0x62,
0xe1, 0x74, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75,
0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x75, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x6d, 0x6f, 0x73, 0x6b, 0x65, 0x76, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x65, 0x76, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x65, 0x76, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x61,
0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x70,
0xe1, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76,
0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f,
0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f,
0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0xe9, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x69, 0x75, 0x65, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x6f, 0x75,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0xed, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x6f, 0x75, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0xed, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x6f,
0x75, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0xed, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x6d, 0x61,
0x72, 0x69, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x6f, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6f, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6f, 0x6d, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x70, 0xe1, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x70, 0xe1, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x70, 0xe1, 0x6b, 0x69, 0x73, 0x74, 0xe1,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x70, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x61, 0x70, 0x75, 0x79, 0x2d,
0x4e, 0x6f, 0x76, 0xe9, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x79, 0x70,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x6c,
0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66,
0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x66, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20,
0x46, 0xe9, 0x6e, 0x69, 0x78, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x70, 0x69, 0x65, 0x72, 0x72, 0x65,
0x2d, 0x6d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x70, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x6d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x70, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x6d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x70, 0x6f, 0x6e, 0x61, 0x70, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x63, 0x68, 0x6a, 0x6f, 0x6e,
0x67, 0x6a, 0x61, 0x6e, 0x67, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x72, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x74, 0x68,
0x65, 0x72, 0x6f, 0x76, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63,
0x65, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x160, 0x61,
0x6c, 0x61, 0x6d, 0x6f, 0x75, 0x6e, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x10d, 0x61, 0x73, 0x20, 0x4a,
0x69, 0x17e, 0x6e, 0xed, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65,
0x73, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63,
0x65, 0x20, 0x160, 0xf3, 0x77, 0x61, 0x74, 0x61, 0x68, 0x69, 0x74, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x63, 0x68, 0x61, 0x6a, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x63, 0x68,
0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x74,
0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x74, 0xe1, 0x64, 0x17e,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f,
0x6e, 0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6e,
0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6e, 0x17e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x63, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x74,
0x75, 0x76, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a,
0x62, 0x65, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a, 0x62,
0x65, 0x63, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a, 0x62, 0x65, 0x63,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x63, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x63, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x63, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x20,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x10d, 0x61,
0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x11b,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x11b, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65, 0x6b, 0x61, 0x74,
0x11b, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x79, 0x75, 0x6b, 0x6f,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x63, 0x72, 0x65,
0x2d, 0x74, 0x69, 0x64, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x53, 0x79, 0x64, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x56, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x74, 0x69,
0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d,
0x74, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x74,
0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x75,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x75,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x2d, 0x74, 0x69, 0x64, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x2d, 0x74, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x41, 0x70, 0x69, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x70,
0x69, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74,
0x61, 0x75, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x2d, 0x74, 0x69,
0x64, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x2d,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x72, 0x61,
0x62, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x41, 0x72, 0x61, 0x62,
0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x56,
0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x74,
0x69, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x63, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x6c, 0x69, 0x67, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x6c, 0x69,
0x67, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x6c, 0x69, 0x67, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0xd8, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x6a,
0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x41,
0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41,
0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x72, 0x6e,
0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x2d, 0x74, 0x69,
0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x2d, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x2d,
0x74, 0x69, 0x64, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72,
0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x2d, 0x74, 0x69, 0x64, 0x4b,
0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x74, 0x69, 0x64,
0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x70, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x2d, 0x74,
0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x74, 0x69,
0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4b,
0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4b,
0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4a,
0x75, 0x6c, 0x65, 0xf8, 0x65, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0xf8, 0x65, 0x72,
0x6e, 0x65, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x43, 0x6f, 0x6f,
0x6b, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x43, 0x6f,
0x6f, 0x6b, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x65, 0x72,
0x6e, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x43, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x43,
0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x43, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x44, 0x61, 0x76,
0x69, 0x73, 0x2d, 0x74, 0x69, 0x64, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x74,
0x69, 0x64, 0xd8, 0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x2d, 0x74,
0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x65, 0x6e, 0x2d, 0x74,
0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x65, 0x6e, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b,
0x65, 0xf8, 0x65, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72,
0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xd8, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x46, 0x6a, 0x65, 0x72, 0x6e, 0xf8,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe6, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x65, 0x72,
0x6e, 0x65, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x65, 0x72,
0x6e, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x46, 0x69, 0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x46, 0x69, 0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x46, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x2d, 0x74, 0x69, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x53, 0x79, 0x64, 0x6c, 0x69, 0x67, 0x65, 0x20, 0x6f, 0x67, 0x20,
0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x6b, 0x65, 0x20,
0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x65, 0x72, 0x2d,
0x74, 0x69, 0x64, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x2d, 0x74, 0x69, 0x64, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d,
0x74, 0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0xf8, 0x65,
0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x47, 0x4d, 0x54, 0xd8, 0x73,
0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0xd8, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x56, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x67,
0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x67,
0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x47, 0x75, 0x61, 0x6d, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x47, 0x6f, 0x6c,
0x66, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x65, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x2d, 0x74, 0x69,
0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d,
0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x74, 0x69, 0x64, 0x48,
0x6f, 0x76, 0x64, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x6e, 0x64, 0x69, 0x73,
0x6b, 0x65, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69,
0x6e, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x74, 0x69, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x6b, 0x61, 0x73,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x56, 0x65, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x64, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x4c, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x74,
0x69, 0x64, 0x4c, 0x69, 0x6e, 0x6a, 0x65, 0xf8, 0x65, 0x72, 0x6e, 0x65,
0x2d, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x2d, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x6c, 0x61, 0x79,
0x73, 0x69, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d,
0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x2d, 0x74, 0x69, 0x64, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x74, 0x69,
0x64, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x2d, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x4d,
0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x79, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x2d, 0x74, 0x69, 0x64, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x2d,
0x74, 0x69, 0x64, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x4e, 0x79, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x4e, 0x79,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x79, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x2d, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x7a,
0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x4e, 0x65, 0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x65,
0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x4e, 0x69, 0x75, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d,
0x74, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x4e, 0x6f, 0x72, 0x64, 0x6d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x65,
0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x64, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x64,
0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x2d, 0x74, 0x69, 0x64, 0x50, 0x61, 0x70, 0x75, 0x61,
0x20, 0x4e, 0x79, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x2d, 0x74,
0x69, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x76, 0x69, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x76, 0x69, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x50, 0x65, 0x72, 0x75, 0x76, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0xf8, 0x65, 0x6e,
0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x2d, 0x20, 0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x20, 0x6f,
0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x20, 0x6f, 0x67,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x50, 0x6f, 0x6e, 0x61, 0x70,
0x65, 0x2d, 0x74, 0x69, 0x64, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61,
0x6e, 0x67, 0x2d, 0x74, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x52, 0x65, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x52, 0x6f, 0x74, 0x68,
0x65, 0x72, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72,
0x65, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x6f, 0x75,
0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x2d, 0x74,
0x69, 0x64, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x2d, 0x74, 0x69,
0x64, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x2d, 0x74, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x2d, 0x74, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x54, 0x61, 0x64, 0x73, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x2d,
0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d, 0x74, 0x69, 0x64,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x2d, 0x74, 0x69,
0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x55, 0x73,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x55, 0x73,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x74, 0x69, 0x64, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x56, 0x65, 0x6e, 0x65, 0x7a,
0x75, 0x65, 0x6c, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74,
0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x74, 0x69, 0x64, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x69, 0x64, 0x57, 0x61, 0x6b,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x69, 0x64,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x6f, 0x67, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x74, 0x69, 0x64, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x2d, 0x74, 0x69, 0x64, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x915, 0x947, 0x902, 0x926, 0x930,
0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940,
0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x915, 0x947, 0x902, 0x926, 0x930, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915,
0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x915, 0x947, 0x902,
0x926, 0x930, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x92a, 0x942,
0x930, 0x94d, 0x935, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x938,
0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e,
0x930, 0x940, 0x915, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940,
0x20, 0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x938, 0x92e, 0x93e,
0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940,
0x915, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x92e, 0x93e, 0x909, 0x902,
0x91f, 0x947, 0x928, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928,
0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e,
0x902, 0x924, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930,
0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x938, 0x92e,
0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930,
0x940, 0x915, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92a,
0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x905,
0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f,
0x93f, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x93e, 0x902, 0x915, 0x947, 0x902, 0x926, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x94b, 0x92a, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x915, 0x947, 0x902, 0x926,
0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x915, 0x947, 0x902, 0x926, 0x930,
0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x917, 0x930, 0x94d,
0x92e, 0x93f, 0x92f, 0x947, 0x902, 0x20, 0x926, 0x93e, 0x20, 0x938, 0x92e, 0x93e,
0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930,
0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940,
0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x917, 0x930, 0x94d, 0x92e,
0x93f, 0x92f, 0x947, 0x902, 0x20, 0x926, 0x93e, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x92a, 0x91a, 0x94d, 0x91b, 0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x92a, 0x91a, 0x94d, 0x91b, 0x92e, 0x940,
0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x93e, 0x902, 0x92a, 0x91a, 0x94d, 0x91b, 0x92e, 0x940, 0x20,
0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x917, 0x930, 0x94d, 0x92e, 0x93f,
0x92f, 0x947, 0x902, 0x20, 0x926, 0x93e, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x41,
0x63, 0x72, 0x65, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x63, 0x72, 0x65,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x66, 0x67, 0x68, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x5a, 0x75, 0x69, 0x64, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65,
0x73, 0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x57,
0x65, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41,
0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64,
0x41, 0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6c,
0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4d,
0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x6f, 0x75,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x74,
0x69, 0x6a, 0x64, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x71, 0x74,
0x61, 0x75, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6a, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57,
0x65, 0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x65,
0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x6d, 0x65,
0x65, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e,
0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65,
0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x77, 0x65,
0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4f,
0x6f, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65,
0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x6e, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x42,
0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x68, 0x75, 0x74, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x6f, 0x6c,
0x69, 0x76, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c,
0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x72, 0x61, 0x7a,
0x69, 0x6c, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x76,
0x65, 0x72, 0x64, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x4b, 0x61, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x43, 0x61, 0x73, 0x65, 0x79, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x65, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x65,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x65, 0x6e,
0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x69,
0x6e, 0x65, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x43, 0x6f, 0x63, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43,
0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x68, 0x61, 0x6c, 0x76, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x75,
0x62, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43,
0x75, 0x62, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x75, 0x62,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x44, 0x61, 0x76, 0x69, 0x73, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f,
0x73, 0x74, 0x2d, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x61,
0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x50,
0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x45, 0x63, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69,
0x64, 0x64, 0x65, 0x6e, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f,
0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x56, 0x65, 0x72, 0x64, 0x65, 0x72, 0x2d, 0x6f, 0x6f, 0x73, 0x74, 0x65,
0x6c, 0x69, 0x6a, 0x6b, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x57, 0x65, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x69, 0x6a, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x46, 0x72, 0x61, 0x6e, 0x73, 0x2d, 0x47, 0x75, 0x79, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x7a, 0x75, 0x69, 0x64, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65,
0x20, 0x65, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f,
0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x57, 0x65, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x75, 0x61, 0x6d,
0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x6f, 0x6c, 0x66, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x6f, 0x76,
0x64, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x48, 0x6f, 0x76, 0x64, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4f,
0x63, 0x65, 0x61, 0x61, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x72, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74,
0x69, 0x6a, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x73, 0x72, 0x61, 0xeb,
0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x49,
0x73, 0x72, 0x61, 0xeb, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x2d, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73,
0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x61, 0x7a, 0x61, 0x63,
0x68, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74,
0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x63,
0x68, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x7a, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4c,
0x61, 0x6e, 0x6b, 0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4c, 0x69, 0x6e,
0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d,
0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d,
0x61, 0x63, 0x61, 0x75, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x6c, 0x65, 0x69, 0x73,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x74, 0x69, 0x6a, 0x64,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x6f, 0x73,
0x6b, 0x6f, 0x75, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x65, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75,
0x77, 0x2d, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x77,
0x2d, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x77, 0x2d, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x77,
0x2d, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x77, 0x2d, 0x5a, 0x65, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75,
0x77, 0x2d, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x4e, 0x69, 0x75, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x74,
0x69, 0x6a, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x65, 0x6c,
0x69, 0x6a, 0x6b, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6d, 0x73, 0x6b,
0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4f,
0x6d, 0x73, 0x6b, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x42, 0x65, 0x6c, 0x61, 0x75, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x50, 0x61, 0x70, 0x6f, 0x65, 0x61, 0x2d, 0x4e, 0x69,
0x65, 0x75, 0x77, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x65, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65,
0x69, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x51, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x51, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x51, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x6f, 0x72, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x5a, 0x75, 0x69, 0x64, 0x2d, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x61, 0x6d, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d, 0x74,
0x69, 0x6a, 0x64, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x61, 0x64, 0x7a, 0x6a, 0x69,
0x65, 0x6b, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65,
0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x65,
0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4f,
0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x56,
0x65, 0x6e, 0x65, 0x7a, 0x6f, 0x6c, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0x65, 0x6e, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74,
0x73, 0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65,
0x74, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73,
0x6b, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x65, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x65, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0xf68,
0xf55, 0xf0b, 0xf42, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf66, 0xf84, 0xf4f, 0xf71, 0xf53,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55,
0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40,
0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55,
0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40,
0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf63, 0xfb7, 0xf7c, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62, 0xf72, 0xf0b,
0xf40, 0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56,
0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62, 0xf72,
0xf0b, 0xf40, 0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74,
0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62,
0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1,
0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf71,
0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf63, 0xf66, 0xf72, 0xf0b, 0xf40, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf63, 0xf66, 0xf72, 0xf0b,
0xf40, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf63, 0xf66, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf49,
0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b, 0xf58, 0xf0b, 0xf5b, 0xf71, 0xf7c, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b, 0xf58, 0xf0b, 0xf5b,
0xf71, 0xf7c, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b, 0xf58, 0xf0b, 0xf5b, 0xf71,
0xf7c, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b,
0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf51, 0xf56, 0xf74, 0xf66,
0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72,
0xf0b, 0xf40, 0xf0b, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b,
0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1,
0xf7c, 0xf42, 0xf66, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68,
0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf64, 0xf62, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b,
0xf40, 0xf0b, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b,
0xf40, 0xf0b, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf49,
0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62,
0xf72, 0xf0b, 0xf40, 0xf0b, 0xf58, 0xf60, 0xf74, 0xf0b, 0xf4a, 0xf7a, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b,
0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf58, 0xf60, 0xf74, 0xf0b,
0xf4a, 0xf7a, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58,
0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf58, 0xf60, 0xf74, 0xf0b, 0xf4a,
0xf7a, 0xf53, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b,
0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf54, 0xf7a, 0xf0b, 0xf66,
0xf72, 0xf0b, 0xf55, 0xf72, 0xf42, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b,
0xf40, 0xf0b, 0xf54, 0xf7a, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf55, 0xf72, 0xf42, 0xf0b,
0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72,
0xf0b, 0xf40, 0xf0b, 0xf54, 0xf7a, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf55, 0xf72, 0xf42,
0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf62, 0xf7a, 0xf0b, 0xf56, 0xfb7, 0xf72,
0xf0b, 0xf61, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b,
0xf62, 0xf7a, 0xf0b, 0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61, 0xf53, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68,
0xf0b, 0xf62, 0xf7a, 0xf0b, 0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61, 0xf53, 0xf0b, 0xf66,
0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62,
0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a,
0xf72, 0xf0b, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b,
0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53,
0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf62,
0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53,
0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf62, 0xf0b,
0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71,
0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf68, 0xf62, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf53,
0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf53,
0xf72, 0xf0b, 0xf61, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf4a, 0xf0b, 0xf63,
0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf40, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf68, 0xf7a, 0xf4a, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf40,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf68, 0xf7a, 0xf4a, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72,
0xf40, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1,
0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a,
0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b,
0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72,
0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c,
0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63,
0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2,
0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf74, 0xf66,
0xf0b, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68,
0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b,
0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66,
0xf0b, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68,
0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b,
0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf53, 0xf74, 0xf56, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a,
0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf49, 0xf72, 0xf53,
0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c,
0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63,
0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c,
0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63,
0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1,
0xf7c, 0xf42, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71,
0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61,
0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf5b, 0xf62, 0xf0b, 0xf56, 0xfb7, 0xf60,
0xf72, 0xf0b, 0xf47, 0xf71, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf68, 0xf0b, 0xf5b, 0xf62, 0xf0b, 0xf56, 0xfb7, 0xf60, 0xf72, 0xf0b, 0xf47, 0xf71,
0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf5b, 0xf62, 0xf0b, 0xf56, 0xfb7, 0xf60, 0xf72,
0xf0b, 0xf47, 0xf71, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b, 0xf5b,
0xf7c, 0xf62, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a,
0xf0b, 0xf5b, 0xf7c, 0xf62, 0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b, 0xf5b, 0xf7c,
0xf62, 0xf66, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf44, 0xf92, 0xf0b, 0xf63, 0xf0b,
0xf51, 0xf7a, 0xf64, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf44,
0xf92, 0xf0b, 0xf63, 0xf0b, 0xf51, 0xf7a, 0xf64, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf44, 0xf92,
0xf0b, 0xf63, 0xf0b, 0xf51, 0xf7a, 0xf64, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf60, 0xf56,
0xfb2, 0xf74, 0xf42, 0xf0b, 0xf61, 0xf74, 0xf63, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf56, 0xfb7, 0xf7c, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf56, 0xf72, 0xf0b,
0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb2, 0xf0b, 0xf5b,
0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf56, 0xfb2, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b,
0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf56, 0xfb2, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b,
0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7a, 0xf54, 0xf0b, 0xf56, 0xf71, 0xf4c, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7a, 0xf54, 0xf0b, 0xf56, 0xf71, 0xf4c,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf40, 0xf7a, 0xf54, 0xf0b, 0xf56, 0xf71, 0xf4c, 0xf0b, 0xf56, 0xfb1,
0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf45, 0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf45, 0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf45, 0xf72, 0xf0b, 0xf63,
0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf92, 0xfb1, 0xf0b, 0xf53, 0xf42, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf92, 0xfb1, 0xf0b, 0xf53, 0xf42,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf62, 0xf92, 0xfb1, 0xf0b, 0xf53, 0xf42, 0xf0b, 0xf49, 0xf72, 0xf53,
0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf41, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf66, 0xf9f, 0xf0b, 0xf58, 0xf7a, 0xf66, 0xf0b,
0xf58, 0xf5a, 0xf7c, 0xf0b, 0xf42, 0xfb3, 0xf72, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c, 0xf0b, 0xf63, 0xf7c, 0xf58, 0xf0b, 0xf56, 0xfb7,
0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c,
0xf0b, 0xf63, 0xf7c, 0xf58, 0xf0b, 0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf40, 0xf7c, 0xf0b, 0xf63, 0xf7c, 0xf58, 0xf0b, 0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61,
0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf56, 0xfb7, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf56,
0xfb7, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf40, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf49,
0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf68, 0xf72, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf62, 0xf0b, 0xf68, 0xf71,
0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf68, 0xf72, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf62, 0xf0b, 0xf68, 0xf71,
0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66,
0xf72, 0xf0b, 0xf4a, 0xf62, 0xf0b, 0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a,
0xf53, 0xf4c, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b, 0xf40, 0xf74, 0xf0b,
0xf4c, 0xf7d, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74,
0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62,
0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61,
0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56,
0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b,
0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62,
0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c,
0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64,
0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62,
0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1,
0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a,
0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c,
0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55,
0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54,
0xf7a, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b,
0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf55, 0xf63, 0xf40, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b,
0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf66, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf63, 0xf40, 0xf0b, 0xf63, 0xf7a, 0xf53,
0xf4c, 0xf0b, 0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf66,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf55, 0xf63, 0xf40, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf68,
0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf66, 0xf0b, 0xf56, 0xfb1,
0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf55, 0xf72, 0xf62, 0xf7a, 0xf53, 0xf45, 0xf0b, 0xf42, 0xf72, 0xf0b, 0xf61,
0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf42, 0xf0b, 0xf63,
0xf0b, 0xf54, 0xf0b, 0xf42, 0xf7c, 0xf66, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf47, 0xf7d, 0xf0b, 0xf47, 0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf47, 0xf7d, 0xf0b, 0xf47, 0xf71, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf7d, 0xf0b,
0xf47, 0xf71, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b,
0xf5d, 0xf72, 0xf46, 0xf0b, 0xf63, 0xf74, 0xf0b, 0xf61, 0xf7c, 0xf51, 0xf0b, 0xf54,
0xf60, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b,
0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64,
0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf72,
0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55,
0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf63,
0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55,
0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf63,
0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74,
0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf72,
0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b,
0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf42, 0xf71, 0xf63, 0xf55,
0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf42, 0xf74, 0xf0b, 0xf61,
0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf67, 0xf0b, 0xf5d,
0xf60, 0xf72, 0xf0b, 0x2d, 0xf68, 0xf7a, 0xf0b, 0xf63, 0xf72, 0xf60, 0xf74, 0xf0b,
0xf64, 0xf7a, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf67, 0xf0b,
0xf5d, 0xf60, 0xf72, 0xf0b, 0x2d, 0xf68, 0xf7a, 0xf0b, 0xf63, 0xf72, 0xf60, 0xf74,
0xf0b, 0xf64, 0xf7a, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf67, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b,
0x2d, 0xf68, 0xf7a, 0xf0b, 0xf63, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf64, 0xf7a, 0xf53,
0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf92, 0xfb1, 0xf0b, 0xf42, 0xf62, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf92, 0xfb1, 0xf0b, 0xf42, 0xf62, 0xf0b,
0xf42, 0xfb1, 0xf72, 0xf0b, 0xf62, 0xf92, 0xfb1, 0xf0b, 0xf58, 0xf5a, 0xf7c, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf53, 0xf0b, 0xf4c, 0xf7c,
0xf0b, 0xf45, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66,
0xf0b, 0xf68, 0xf72, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b, 0xf53, 0xf7a, 0xf0b, 0xf64,
0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62,
0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf72, 0xf53, 0xf0b, 0xf4c,
0xf7c, 0xf0b, 0xf53, 0xf7a, 0xf0b, 0xf64, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf68, 0xf72, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b, 0xf53, 0xf7a, 0xf0b,
0xf64, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68,
0xf72, 0xf0b, 0xf62, 0xf71, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf68, 0xf72, 0xf0b, 0xf62, 0xf71, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf0b, 0xf62,
0xf71, 0xf53, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf40, 0xf74, 0xf59,
0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf40,
0xf74, 0xf59, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf40, 0xf74, 0xf59, 0xf72,
0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66, 0xf0b, 0xf62, 0xf7a, 0xf63, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66, 0xf0b, 0xf62, 0xf7a,
0xf63, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66, 0xf0b, 0xf62, 0xf7a, 0xf63, 0xf0b, 0xf49,
0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf47, 0xf0b, 0xf54, 0xf71, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf47, 0xf0b, 0xf54, 0xf71, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf0b, 0xf54,
0xf71, 0xf53, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b,
0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c, 0xf0b, 0xf62,
0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf61,
0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf62, 0xf66, 0xf72, 0xf0b, 0xf53, 0xf7c, 0xf0b,
0xf61, 0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf40, 0xf62, 0xf66, 0xf72, 0xf0b, 0xf53, 0xf7c, 0xf0b, 0xf61, 0xf62, 0xf66, 0xf40,
0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf40, 0xf62, 0xf66, 0xf72, 0xf0b, 0xf53, 0xf7c, 0xf0b, 0xf61,
0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf0b, 0xf42, 0xfb7,
0xf0b, 0xf51, 0xf71, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58,
0xf0b, 0xf42, 0xfb7, 0xf0b, 0xf51, 0xf71, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf0b, 0xf42,
0xfb7, 0xf0b, 0xf51, 0xf71, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51,
0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf71, 0xf63,
0xf0b, 0xf51, 0xf72, 0xf56, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf58, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf64, 0xf71, 0xf66, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf64, 0xf71,
0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf64, 0xf71, 0xf66,
0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7d, 0xf66, 0xf0b, 0xf40, 0xf7c, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7d, 0xf66, 0xf0b, 0xf40, 0xf7c, 0xf0b,
0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf58, 0xf7d, 0xf66, 0xf0b, 0xf40, 0xf7c, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62,
0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53,
0xf7a, 0xf0b, 0xf54, 0xf71, 0xf63, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf53, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf72, 0xf60, 0xf74, 0xf0b,
0xf5b, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf72, 0xf60,
0xf74, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf49, 0xf72,
0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf53, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf55, 0xf60, 0xf74, 0xf53, 0xf4c, 0xf0b,
0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53,
0xf72, 0xf60, 0xf74, 0xf0b, 0xf55, 0xf60, 0xf74, 0xf53, 0xf4c, 0xf0b, 0xf63, 0xf7a,
0xf53, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf55, 0xf60, 0xf74,
0xf53, 0xf4c, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b,
0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55,
0xf62, 0xf0b, 0xf53, 0xf7a, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b, 0x20, 0xf4c, 0xf72,
0xf0b, 0x20, 0xf53, 0xf7c, 0xf0b, 0xf62, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf67, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf62, 0xf0b, 0xf53, 0xf7a, 0xf53,
0xf0b, 0xf4c, 0xf7c, 0xf0b, 0x20, 0xf4c, 0xf72, 0xf0b, 0x20, 0xf53, 0xf7c, 0xf0b,
0xf62, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf67, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf62, 0xf0b, 0xf53,
0xf7a, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b, 0x20, 0xf4c, 0xf72, 0xf0b, 0x20, 0xf53,
0xf7c, 0xf0b, 0xf62, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf67, 0xf0b, 0xf56, 0xfb1, 0xf71,
0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf53, 0xf7c, 0xf0b, 0xf56, 0xf7c, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf56, 0xf72, 0xf62,
0xf66, 0xf40, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf7c,
0xf0b, 0xf56, 0xf7c, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf56, 0xf72, 0xf62, 0xf66, 0xf40,
0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf53, 0xf7c, 0xf0b, 0xf56, 0xf7c, 0xf0b, 0xf66, 0xf72, 0xf0b,
0xf56, 0xf72, 0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7c,
0xf58, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68,
0xf7c, 0xf58, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7c, 0xf58, 0xf66, 0xf40,
0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf40, 0xf72, 0xf0b, 0xf66, 0xf84,
0xf4f, 0xf71, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b,
0xf40, 0xf72, 0xf0b, 0xf66, 0xf84, 0xf4f, 0xf71, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b,
0xf40, 0xf72, 0xf0b, 0xf66, 0xf84, 0xf4f, 0xf71, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71,
0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf54, 0xf0b, 0xf62, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62, 0xf0b, 0xf42, 0xf74, 0xf0b,
0xf5d, 0xf60, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62, 0xf0b, 0xf42, 0xf74, 0xf0b,
0xf5d, 0xf60, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62, 0xf74, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62, 0xf74, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf54, 0xf0b, 0xf62, 0xf74, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf60, 0xf72, 0xf0b,
0xf62, 0xf72, 0xf0b, 0xf51, 0xf44, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf40, 0xf72, 0xf0b,
0xf63, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54,
0xf60, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf51, 0xf44, 0xf0b, 0xf58, 0xf72, 0xf0b,
0xf40, 0xf72, 0xf0b, 0xf63, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf60, 0xf72,
0xf0b, 0xf62, 0xf72, 0xf0b, 0xf51, 0xf44, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf40, 0xf72,
0xf0b, 0xf63, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2,
0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf72, 0xf0b,
0xf61, 0xf74, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf61, 0xf71, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66, 0xf0b, 0xf41, 0xf0b, 0xf63, 0xf72, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66, 0xf0b, 0xf41, 0xf0b, 0xf63, 0xf72,
0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf66, 0xf0b, 0xf41, 0xf0b, 0xf63, 0xf72, 0xf53, 0xf0b, 0xf56,
0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf66, 0xf7a, 0xf0b, 0xf64, 0xf71, 0xf63, 0xf66, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66, 0xf74, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf53, 0xf71,
0xf58, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf74, 0xf0b, 0xf62,
0xf74, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf74, 0xf0b, 0xf42, 0xf74,
0xf0b, 0xf5d, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf74,
0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71,
0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf56, 0xf7a, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf5b, 0xf74, 0xf0b, 0xf5d, 0xf7a, 0xf0b,
0xf63, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf0b, 0xf63, 0xf0b,
0xf4c, 0xf72, 0xf0b, 0xf56, 0xf7c, 0xf66, 0xf72, 0xf0b, 0xf4f, 0xf7c, 0xf40, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf0b, 0xf63, 0xf0b, 0xf4c, 0xf72,
0xf0b, 0xf56, 0xf7c, 0xf66, 0xf72, 0xf0b, 0xf4f, 0xf7c, 0xf40, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56,
0xf0b, 0xf63, 0xf0b, 0xf4c, 0xf72, 0xf0b, 0xf56, 0xf7c, 0xf66, 0xf72, 0xf0b, 0xf4f,
0xf7c, 0xf40, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf71, 0xf7c, 0xf63, 0xf0b, 0xf42,
0xf7c, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf7a, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf56, 0xf71, 0xf7c, 0xf63, 0xf0b, 0xf42, 0xf7c, 0xf0b, 0xf42, 0xf72,
0xf62, 0xf7a, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf71, 0xf7c, 0xf63, 0xf0b, 0xf42, 0xf7c,
0xf0b, 0xf42, 0xf72, 0xf62, 0xf7a, 0xf4c, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf0b,
0xf40, 0xf74, 0xf59, 0xf72, 0xf40, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf61, 0xf0b, 0xf40, 0xf74, 0xf59, 0xf72, 0xf40, 0xf72, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61,
0xf0b, 0xf40, 0xf74, 0xf59, 0xf72, 0xf40, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62,
0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61,
0xf7a, 0xf0b, 0xf40, 0xf0b, 0xf4f, 0xf7a, 0xf0b, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf56,
0xf62, 0xf42, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf7a, 0xf0b,
0xf40, 0xf0b, 0xf4f, 0xf7a, 0xf0b, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf56, 0xf62, 0xf42,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf61, 0xf7a, 0xf0b, 0xf40, 0xf0b, 0xf4f, 0xf7a, 0xf0b, 0xf62, 0xf72,
0xf53, 0xf0b, 0xf56, 0xf62, 0xf42, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51,
0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xd801, 0xdc0a, 0xd801,
0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc45, 0xd801, 0xdc3f, 0xd801, 0xdc32, 0x20, 0xd801, 0xdc13,
0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc0a, 0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc45,
0xd801, 0xdc3f, 0xd801, 0xdc32, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801,
0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13,
0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc0a, 0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc45,
0xd801, 0xdc3f, 0xd801, 0xdc32, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801,
0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc1d,
0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc49, 0xd801, 0xdc32, 0xd801, 0xdc4a,
0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc1d, 0xd801, 0xdc2f, 0xd801,
0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc49, 0xd801, 0xdc32, 0xd801, 0xdc4a, 0x20, 0xd801, 0xdc1d,
0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49,
0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc1d, 0xd801,
0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc49, 0xd801, 0xdc32, 0xd801, 0xdc4a, 0x20,
0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801,
0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc00, 0xd801, 0xdc45, 0xd801, 0xdc3b, 0xd801,
0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b,
0xd801, 0xdc00, 0xd801, 0xdc45, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc4c,
0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801,
0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b,
0xd801, 0xdc00, 0xd801, 0xdc45, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc4c,
0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20,
0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc23, 0xd801, 0xdc35, 0xd801, 0xdc4c,
0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801,
0xdc4b, 0xd801, 0xdc23, 0xd801, 0xdc35, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801,
0xdc4c, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c,
0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801,
0xdc4b, 0xd801, 0xdc23, 0xd801, 0xdc35, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801,
0xdc4c, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b,
0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc11, 0xd801, 0xdc32, 0xd801,
0xdc45, 0xd801, 0xdc2e, 0xd801, 0xdc41, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc13,
0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc11, 0xd801, 0xdc32, 0xd801, 0xdc45, 0xd801, 0xdc2e,
0xd801, 0xdc41, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801,
0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20,
0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc11, 0xd801, 0xdc32, 0xd801, 0xdc45,
0xd801, 0xdc2e, 0xd801, 0xdc41, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc14, 0xd801,
0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34,
0xd801, 0xdc4b, 0xd801, 0xdc08, 0xd801, 0xdc3b, 0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc4c,
0xd801, 0xdc3b, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801,
0xdc4b, 0xd801, 0xdc08, 0xd801, 0xdc3b, 0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801,
0xdc3b, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30,
0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801,
0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc08, 0xd801, 0xdc3b, 0xd801, 0xdc4a, 0xd801,
0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc14,
0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801,
0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc10, 0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc17,
0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801,
0xdc10, 0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc17, 0xd801, 0xdc31, 0xd801, 0xdc4d,
0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801,
0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b,
0xd801, 0xdc10, 0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc17, 0xd801, 0xdc31, 0xd801,
0xdc4d, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b,
0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc24, 0xd801, 0xdc2d, 0xd801,
0xdc41, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc4a, 0xd801, 0xdc32, 0xd801,
0xdc4c, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc24,
0xd801, 0xdc2d, 0xd801, 0xdc41, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc4a,
0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801,
0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20,
0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc24, 0xd801, 0xdc2d, 0xd801, 0xdc41,
0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc4a, 0xd801, 0xdc32, 0xd801, 0xdc4c,
0xd801, 0xdc3c, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801,
0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0x53, 0x74, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x74, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6f,
0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76,
0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68,
0x65, 0x72, 0x6e, 0x20, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x6e, 0x74, 0x61,
0x72, 0x63, 0x74, 0x69, 0x63, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x72,
0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x6e, 0x64,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4b, 0x68, 0x6f, 0x76, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x68,
0x6f, 0x76, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x410, 0x444,
0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x43d, 0x44c, 0x20,
0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x410,
0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d,
0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43e, 0x43d,
0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x435, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x435, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x435, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x411, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x411, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x43d,
0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x411, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x43a,
0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x427, 0x438, 0x43b,
0x438, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x427, 0x438, 0x43b, 0x438, 0x43d,
0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x427, 0x438, 0x43b, 0x438, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x438, 0x442, 0x430, 0x435, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x438, 0x442, 0x430, 0x435, 0x43d, 0x44c,
0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430,
0x41a, 0x438, 0x442, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x443, 0x431, 0x430, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x41a, 0x443, 0x431, 0x430, 0x43d, 0x44c, 0x20, 0x441,
0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x443,
0x431, 0x430, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x418, 0x43d, 0x434, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x441,
0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x418, 0x440,
0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x418, 0x440, 0x430,
0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x418, 0x440, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20,
0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x42f, 0x43f,
0x43e, 0x43d, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x42f, 0x43f,
0x43e, 0x43d, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f,
0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x43e, 0x43d,
0x44c, 0x20, 0x447, 0x438, 0x43b, 0x438, 0x441, 0x435, 0x43c, 0x430, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430,
0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x447, 0x438, 0x432, 0x430, 0x43b, 0x433, 0x43e,
0x43c, 0x430, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x43e, 0x440, 0x435,
0x430, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x43e, 0x440, 0x435, 0x430,
0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x41a, 0x43e, 0x440, 0x435, 0x430, 0x43d, 0x44c, 0x20, 0x43a, 0x438,
0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41c, 0x43e, 0x441, 0x43a,
0x43e, 0x432, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41c, 0x43e, 0x441,
0x43a, 0x43e, 0x432, 0x43e, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432,
0x43e, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x41e, 0x434, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41e, 0x434, 0x20, 0x417, 0x435,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430,
0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41e, 0x434, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x43a, 0x438,
0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41e, 0x43c, 0x441, 0x43a,
0x43e, 0x435, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41e, 0x43c, 0x441, 0x43a,
0x43e, 0x435, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x43d, 0x44c,
0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c, 0x20,
0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x43a, 0x438,
0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x43d, 0x44d, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430,
0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x44d, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x435,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430,
0x435, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c,
0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x423,
0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x43e, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x61, 0x66, 0x67, 0x61, 0x6e, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x73, 0x75, 0x64, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x6f, 0x6e,
0x74, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x6f,
0x6e, 0x74, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70,
0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x3a, 0x20, 0x41, 0x70, 0x69, 0x6f, 0x41, 0x70, 0x69, 0x6f, 0x20,
0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x29, 0x41, 0x70, 0x69, 0x6f, 0x20, 0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x61, 0x72, 0x61, 0x62,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x61, 0x62, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x61, 0x72, 0x61, 0x62, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x2d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x2d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x2d, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61,
0x72, 0x6d, 0x65, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x61,
0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x61, 0x16d, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x6b,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x6b,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x11d,
0x61, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x6a, 0x11d, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x6a, 0x11d, 0x61, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x3a, 0x20, 0x41, 0x63, 0x6f, 0x72, 0x6f, 0x6a, 0x41, 0x63, 0x6f,
0x72, 0x6f, 0x6a, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x29, 0x41, 0x63, 0x6f, 0x72, 0x6f, 0x6a, 0x20,
0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x29, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x15d, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x15d, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x62, 0x75, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x62, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x6a, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c,
0x6a, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x6a, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x62,
0x72, 0x75, 0x6e, 0x65, 0x6a, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6b, 0x61, 0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6b, 0x61, 0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6b, 0x61, 0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x61,
0x6d, 0x6f, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x109, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x109, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x69, 0x6c, 0x69, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x109, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x69,
0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x109, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x72, 0x69, 0x73, 0x74,
0x6e, 0x61, 0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x6b, 0x6f, 0x73, 0x69, 0x6e, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x75, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x75, 0x6b, 0x69, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x75, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x4b, 0x75, 0x62,
0x6f, 0x4b, 0x75, 0x62, 0x6f, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x4b, 0x75, 0x62, 0x6f, 0x20,
0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x29, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x44, 0x61, 0x76,
0x69, 0x73, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x2d, 0x74, 0x69, 0x6d, 0x6f,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x73, 0x6b,
0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x70, 0x61, 0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61,
0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x65, 0x6b, 0x76,
0x61, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x16d, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x16d, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x65, 0x6b,
0x73, 0x74, 0x72, 0x65, 0x6d, 0x2d, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66,
0x69, 0x11d, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66, 0x69,
0x11d, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x66, 0x69, 0x11d, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x3a, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x61, 0x20, 0x47,
0x75, 0x6a, 0x61, 0x6e, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20,
0x46, 0x72, 0x61, 0x6e, 0x63, 0x61, 0x6a, 0x20, 0x53, 0x75, 0x64, 0x61,
0x6a, 0x20, 0x54, 0x65, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x6f, 0x6a,
0x67, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x6b, 0x61, 0x72, 0x74, 0x76, 0x65, 0x6c, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x61, 0x72, 0x74, 0x76, 0x65,
0x6c, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6b, 0x61, 0x72, 0x74, 0x76, 0x65, 0x6c, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x67,
0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72,
0x73, 0x61, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x6b,
0x75, 0x6e, 0x6f, 0x72, 0x64, 0x69, 0x67, 0x69, 0x74, 0x61, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x67, 0x72,
0x6f, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x61, 0x72, 0x61, 0x62, 0x67, 0x6f, 0x6c, 0x66, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x67, 0x75, 0x6a,
0x61, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x3a, 0x20, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x6f, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x6f, 0x6a, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x6f,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x6f, 0x6a, 0x20, 0x28, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x48, 0x61,
0x76, 0x61, 0x6a, 0x6f, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x6f, 0x6a,
0x20, 0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x29, 0x68, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x68, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x68, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x125, 0x6f,
0x76, 0x64, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x125, 0x6f, 0x76,
0x64, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x125, 0x6f, 0x76, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x68, 0x69, 0x6e, 0x64,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x68, 0x69, 0x6e, 0x64, 0x6f, 0x63, 0x65, 0x61, 0x6e, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x68, 0x69, 0x6e, 0x64, 0x6f, 0x109, 0x69,
0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x2d,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x2d,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x69, 0x72, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x69, 0x72, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69, 0x72, 0x61, 0x6e, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6a, 0x61, 0x70, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6a, 0x61, 0x70, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x125, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x2d, 0x6b, 0x61, 0x7a, 0x61,
0x125, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x125, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x72,
0x65, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b,
0x69, 0x72, 0x67, 0x69, 0x7a, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x4c, 0x69, 0x6e, 0x69, 0x61,
0x6a, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x6a, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x3a, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x28,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x28, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29,
0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x6c, 0x61, 0x6a,
0x7a, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61,
0x72, 0x6b, 0x69, 0x7a, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x72, 0x15d, 0x61, 0x6c, 0x69, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61,
0x16d, 0x72, 0x69, 0x63, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6d, 0x61, 0x16d, 0x72, 0x69, 0x63, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x16d, 0x72,
0x69, 0x63, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20,
0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69,
0x6b, 0x61, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x61,
0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x6b, 0x61, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74,
0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x6f, 0x73, 0x6b,
0x76, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x62, 0x69, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6e, 0x61, 0x75, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6e, 0x65, 0x70, 0x61, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6e, 0x6f, 0x76, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x7a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6e, 0x6f, 0x76, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f,
0x76, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x3a, 0x20, 0x4e, 0x6f, 0x76, 0x6c, 0x61, 0x6e, 0x64, 0x6f,
0x4e, 0x6f, 0x76, 0x6c, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x28, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x4e, 0x6f,
0x76, 0x6c, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x28, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x6e, 0x69, 0x75,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x29, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20,
0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x29, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6d, 0x73,
0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6d, 0x73, 0x6b,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6f, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x6c, 0x61, 0x16d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20,
0x50, 0x61, 0x70, 0x75, 0x6f, 0x2d, 0x4e, 0x6f, 0x76, 0x2d, 0x47, 0x76,
0x69, 0x6e, 0x65, 0x6f, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x72, 0x61, 0x67,
0x76, 0x61, 0x6a, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x70, 0x65, 0x72, 0x75, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x70, 0x65, 0x72, 0x75, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x65, 0x72, 0x75, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66, 0x65, 0x6e, 0x69, 0x6b,
0x73, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x53, 0x61, 0x6e, 0x6b,
0x74, 0x61, 0x20, 0x50, 0x69, 0x65, 0x72, 0x6f, 0x20, 0x6b, 0x61, 0x6a,
0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x53, 0x61, 0x6e,
0x6b, 0x74, 0x61, 0x20, 0x50, 0x69, 0x65, 0x72, 0x6f, 0x20, 0x6b, 0x61,
0x6a, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x20, 0x28,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29,
0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x50, 0x69, 0x65, 0x72, 0x6f,
0x20, 0x6b, 0x61, 0x6a, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e,
0x6f, 0x20, 0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x29, 0x70, 0x69, 0x74, 0x6b, 0x61, 0x72, 0x6e, 0x69,
0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x70, 0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x52,
0x65, 0x75, 0x6e, 0x69, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20,
0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x73, 0x61, 0x125, 0x61, 0x6c,
0x65, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x73, 0x61, 0x125,
0x61, 0x6c, 0x65, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x73, 0x61, 0x125, 0x61, 0x6c, 0x65, 0x6e,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x73, 0x65, 0x6a, 0x15d, 0x65, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x73, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x6f, 0x6a, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x53, 0x75, 0x64,
0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x6f, 0x73, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x53, 0x68, 0x6f, 0x77, 0x61, 0x74, 0x61,
0x68, 0x69, 0x74, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74,
0x61, 0x6a, 0x70, 0x65, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74,
0x61, 0x6a, 0x70, 0x65, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x61, 0x6a, 0x70, 0x65, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x74, 0x61, 0x11d, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x74, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x108, 0x75, 0x75, 0x6b, 0x74, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x75, 0x76, 0x61, 0x6c, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75, 0x72, 0x75, 0x67, 0x76,
0x61, 0x6a, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x75, 0x7a, 0x62, 0x65, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x75, 0x7a, 0x62, 0x65, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x3a, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x76, 0x65, 0x6a, 0x6b,
0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x3a, 0x20, 0x56, 0x61, 0x6c, 0x69, 0x73,
0x6f, 0x20, 0x6b, 0x61, 0x6a, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x6f,
0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x75, 0x6b, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x41, 0x63, 0x72,
0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x63, 0x72, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x63, 0x72,
0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x66, 0x67,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67,
0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x41, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x41, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xf5, 0x75, 0x6e, 0x61,
0x2d, 0x41, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e,
0x65, 0x2d, 0x41, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61, 0x65,
0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0xf5, 0x20,
0x61, 0x65, 0x67, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0xf5, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0xf5, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x65, 0x20, 0x61, 0x65, 0x67,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x41, 0x6d, 0x65, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d,
0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b,
0x2d, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x72, 0x61, 0x6e, 0x6e,
0x69, 0x6b, 0x75, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x72, 0x61,
0x6e, 0x6e, 0x69, 0x6b, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x72, 0x61, 0x6e, 0x6e,
0x69, 0x6b, 0x75, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4d,
0xe4, 0x65, 0x73, 0x74, 0x69, 0x6b, 0x75, 0x76, 0xf6, 0xf6, 0x6e, 0x64,
0x69, 0x20, 0x61, 0x65, 0x67, 0x4d, 0xe4, 0x65, 0x73, 0x74, 0x69, 0x6b,
0x75, 0x76, 0xf6, 0xf6, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4d, 0xe4, 0x65, 0x73, 0x74,
0x69, 0x6b, 0x75, 0x76, 0xf6, 0xf6, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65, 0x20,
0x6f, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x56,
0x61, 0x69, 0x6b, 0x73, 0x65, 0x20, 0x6f, 0x6f, 0x6b, 0x65, 0x61, 0x6e,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65, 0x20, 0x6f, 0x6f, 0x6b, 0x65,
0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41,
0x6e, 0x61, 0x64, 0xf5, 0x72, 0x69, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6e,
0x61, 0x64, 0xf5, 0x72, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x6e, 0x61, 0x64, 0xf5, 0x72, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x70, 0x69, 0x61,
0x20, 0x61, 0x65, 0x67, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x70, 0x69, 0x61,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6b, 0x74, 0x61,
0x75, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x6b,
0x74, 0x61, 0x75, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41,
0x6b, 0x74, 0xf6, 0x62, 0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6b, 0x74,
0xf6, 0x62, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x41, 0x6b, 0x74, 0xf6, 0x62, 0x65, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x72, 0x61, 0x61, 0x62, 0x69, 0x61,
0x20, 0x61, 0x65, 0x67, 0x41, 0x72, 0x61, 0x61, 0x62, 0x69, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41,
0x72, 0x61, 0x61, 0x62, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x61, 0x65, 0x67, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67,
0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x61,
0x65, 0x67, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x72,
0x6d, 0x65, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x61, 0x65,
0x67, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b,
0x65, 0x73, 0x6b, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73,
0x6b, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x2d,
0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4b, 0x65, 0x73, 0x6b,
0x2d, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x4c, 0xe4,
0xe4, 0x6e, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x49,
0x64, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x65,
0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69,
0x64, 0x17e, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x41, 0x73,
0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x61, 0x6e, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41,
0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x61, 0x6e, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x73, 0x73, 0x6f,
0x6f, 0x72, 0x69, 0x64, 0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x73, 0x73,
0x6f, 0x6f, 0x72, 0x69, 0x64, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x73, 0x73, 0x6f, 0x6f, 0x72,
0x69, 0x64, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x20, 0x61,
0x65, 0x67, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x68, 0x75, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x42, 0x6f, 0x6c, 0x69, 0x69,
0x76, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x42, 0x72, 0x61, 0x73, 0x69,
0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x42, 0x72, 0x61, 0x73,
0x69, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69, 0x6c,
0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x20, 0x61, 0x65, 0x67, 0x52, 0x6f, 0x68, 0x65,
0x6e, 0x65, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20,
0x61, 0x65, 0x67, 0x52, 0x6f, 0x68, 0x65, 0x6e, 0x65, 0x65, 0x6d, 0x65,
0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x52, 0x6f, 0x68, 0x65, 0x6e, 0x65,
0x65, 0x6d, 0x65, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x43, 0x61, 0x73, 0x65, 0x79, 0x20, 0x61,
0x65, 0x67, 0x54, 0x161, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x20, 0x61, 0x65, 0x67, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x54, 0x161,
0x69, 0x69, 0x6c, 0x69, 0x20, 0x61, 0x65, 0x67, 0x54, 0x161, 0x69, 0x69,
0x6c, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x54, 0x161, 0x69, 0x69, 0x6c, 0x69, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x48, 0x69, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x65,
0x67, 0x48, 0x69, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x48, 0x69, 0x69, 0x6e, 0x61, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4a, 0xf5, 0x75, 0x6c, 0x75,
0x73, 0x61, 0x61, 0x72, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x6f, 0x6f,
0x6b, 0x6f, 0x73, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65,
0x67, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x61, 0x65,
0x67, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74,
0x65, 0x20, 0x61, 0x65, 0x67, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x73,
0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x73,
0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x6f, 0x73, 0x61, 0x6c, 0x69, 0x6e,
0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b, 0x75, 0x75,
0x62, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b,
0x75, 0x75, 0x62, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x44, 0x61, 0x76, 0x69, 0x73, 0x65, 0x20, 0x61, 0x65, 0x67, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x2019, 0x69, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4c, 0x69,
0x68, 0x61, 0x76, 0xf5, 0x74, 0x74, 0x65, 0x73, 0x61, 0x61, 0x72, 0x65,
0x20, 0x61, 0x65, 0x67, 0x4c, 0x69, 0x68, 0x61, 0x76, 0xf5, 0x74, 0x74,
0x65, 0x73, 0x61, 0x61, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4c, 0x69, 0x68, 0x61, 0x76, 0xf5,
0x74, 0x74, 0x65, 0x73, 0x61, 0x61, 0x72, 0x65, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69,
0x20, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x6f, 0x70, 0x61, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64,
0x69, 0x20, 0x6a, 0x61, 0x20, 0x56, 0x61, 0x6c, 0x67, 0x65, 0x76, 0x65,
0x6e, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c,
0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70,
0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74,
0x65, 0x20, 0x61, 0x65, 0x67, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74,
0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x46, 0x69, 0x64,
0x17e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x46,
0x69, 0x64, 0x17e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x50, 0x72, 0x61, 0x6e, 0x74, 0x73, 0x75, 0x73, 0x65, 0x20, 0x47, 0x75,
0x61, 0x6a, 0x61, 0x61, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x50, 0x72,
0x61, 0x6e, 0x74, 0x73, 0x75, 0x73, 0x65, 0x20, 0x41, 0x6e, 0x74, 0x61,
0x72, 0x6b, 0x74, 0x69, 0x6c, 0x69, 0x73, 0x74, 0x65, 0x20, 0x6a, 0x61,
0x20, 0x4c, 0xf5, 0x75, 0x6e, 0x61, 0x61, 0x6c, 0x61, 0x64, 0x65, 0x20,
0x61, 0x65, 0x67, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x65, 0x20, 0x61, 0x65, 0x67, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x2019, 0x20, 0x61, 0x65, 0x67, 0x47, 0x72, 0x75, 0x75, 0x73, 0x69, 0x61,
0x20, 0x61, 0x65, 0x67, 0x47, 0x72, 0x75, 0x75, 0x73, 0x69, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x47,
0x72, 0x75, 0x75, 0x73, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x69, 0x20, 0x73,
0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x69, 0x20, 0x61, 0x65, 0x67, 0x47,
0x72, 0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x61, 0x65, 0x67,
0x47, 0x72, 0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x47, 0x72, 0xf6,
0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x47, 0x72, 0xf6, 0xf6, 0x6e, 0x69,
0x6d, 0x61, 0x61, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x47,
0x72, 0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d,
0x47, 0x72, 0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x47,
0x72, 0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x61, 0x65, 0x67,
0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x47, 0x72, 0xf6, 0xf6, 0x6e, 0x69,
0x6d, 0x61, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x47, 0x72, 0xf6,
0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x47, 0x75, 0x61, 0x6d, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x50, 0xe4, 0x72, 0x73, 0x69,
0x61, 0x20, 0x6c, 0x61, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61,
0x20, 0x61, 0x65, 0x67, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x75, 0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x75, 0x64, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x75,
0x64, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x48, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x61, 0x65, 0x67, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x48, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x48, 0x6f,
0x76, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x61, 0x65,
0x67, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x6f, 0x6b, 0x65, 0x61,
0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x49, 0x6e, 0x64, 0x6f, 0x68, 0x69,
0x69, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x69, 0x61, 0x20, 0x61,
0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x65, 0x73, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e,
0x65, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x69, 0x61,
0x20, 0x61, 0x65, 0x67, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x61,
0x65, 0x67, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x72, 0x61, 0x61,
0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x49, 0x69,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x20, 0x61, 0x65, 0x67, 0x49, 0x69,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x69, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4a, 0x61,
0x61, 0x70, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x61,
0x70, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x74, 0x161, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b,
0x61, 0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x61, 0x6d, 0x74,
0x161, 0x61, 0x74, 0x6b, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x4b, 0x61, 0x73, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x4b, 0x61, 0x73, 0x61, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4,
0x6e, 0x65, 0x2d, 0x4b, 0x61, 0x73, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x61,
0x65, 0x67, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b, 0xf5, 0x72, 0x67, 0xf5, 0x7a,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x53, 0x72, 0x69,
0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0x69,
0x6e, 0x65, 0x2019, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20,
0x61, 0x65, 0x67, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x2019, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x2019, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2019, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x4d, 0x61, 0x63, 0x61, 0x75, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x63,
0x61, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x20,
0x61, 0x65, 0x67, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x4d, 0x61, 0x6c, 0x61, 0x69, 0x73, 0x69, 0x61, 0x20,
0x61, 0x65, 0x67, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x69, 0x76, 0x69, 0x20,
0x61, 0x65, 0x67, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x69, 0x73, 0x61, 0x61,
0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x69, 0x20, 0x53, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20,
0x61, 0x65, 0x67, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x65, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x65, 0x68, 0x68,
0x69, 0x6b, 0x6f, 0x20, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65, 0x20, 0x6f,
0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x65,
0x68, 0x68, 0x69, 0x6b, 0x6f, 0x20, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65,
0x20, 0x6f, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4d, 0x65, 0x68, 0x68,
0x69, 0x6b, 0x6f, 0x20, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65, 0x20, 0x6f,
0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61,
0x72, 0x69, 0x20, 0x61, 0x65, 0x67, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x61, 0x65, 0x67,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x69, 0x72, 0x6d,
0x61, 0x20, 0x61, 0x65, 0x67, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x61,
0x65, 0x67, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x69, 0x20, 0x61, 0x65, 0x67,
0x55, 0x75, 0x73, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73, 0x2d, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73, 0x2d,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73, 0x2d, 0x4d, 0x65,
0x72, 0x65, 0x6d, 0x61, 0x61, 0x20, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73,
0x2d, 0x4d, 0x65, 0x72, 0x65, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73, 0x2d,
0x4d, 0x65, 0x72, 0x65, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x61,
0x65, 0x67, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x20, 0x73,
0x61, 0x61, 0x72, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x65,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x20, 0x61, 0x65, 0x67, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x50, 0xf5, 0x68, 0x6a, 0x61, 0x2d, 0x4d,
0x61, 0x72, 0x69, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x61, 0x65, 0x67, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4f,
0x6d, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4f, 0x6d, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x61, 0x65, 0x67, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x61,
0x65, 0x67, 0x50, 0x61, 0x61, 0x70, 0x75, 0x61, 0x20, 0x55, 0x75, 0x73,
0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x61, 0x65, 0x67, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x61, 0x65, 0x67, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x50,
0x65, 0x72, 0x75, 0x75, 0x20, 0x61, 0x65, 0x67, 0x50, 0x65, 0x72, 0x75,
0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x50, 0x65, 0x72, 0x75, 0x75, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x20,
0x61, 0x65, 0x67, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x46, 0xf6, 0xf6, 0x6e, 0x69, 0x6b, 0x73,
0x69, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2019,
0x69, 0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2019, 0x69, 0x20, 0x6a, 0x61, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2019, 0x69, 0x20,
0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x50, 0x6f, 0x68,
0x6e, 0x70, 0x65, 0x69, 0x20, 0x61, 0x65, 0x67, 0x50, 0x79, 0x6f, 0x6e,
0x67, 0x79, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b, 0xf5,
0x7a, 0xf5, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4b,
0xf5, 0x7a, 0xf5, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0xf5, 0x7a, 0xf5,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x20, 0x61, 0x65,
0x67, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x61, 0x65, 0x67,
0x53, 0x61, 0x68, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x61, 0x65,
0x67, 0x53, 0x61, 0x68, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61,
0x68, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x61, 0x65,
0x67, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x61, 0x65, 0x67, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x53, 0x65, 0x69, 0x161, 0x65, 0x6c, 0x6c, 0x69, 0x20, 0x61, 0x65, 0x67,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61, 0x61,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x69, 0x20, 0x53, 0x61, 0x61, 0x72, 0x74,
0x65, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xf5, 0x75, 0x6e, 0x61, 0x2d, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0x61, 0x65, 0x67, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x20, 0x61, 0x65, 0x67, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x20, 0x61, 0x65, 0x67, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20,
0x61, 0x65, 0x67, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x54,
0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x61, 0x65, 0x67, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x61,
0x65, 0x67, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x61, 0x65, 0x67, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x69, 0x20,
0x61, 0x65, 0x67, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x54, 0xfc, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x54, 0xfc, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75,
0x20, 0x61, 0x65, 0x67, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20,
0x61, 0x65, 0x67, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x61, 0x65, 0x67, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x61, 0x65, 0x67, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x56, 0x65, 0x6e, 0x65, 0x7a,
0x75, 0x65, 0x6c, 0x61, 0x20, 0x61, 0x65, 0x67, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x61, 0x65,
0x67, 0x57, 0x61, 0x6b, 0x65, 0x2019, 0x69, 0x20, 0x61, 0x65, 0x67, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x65, 0x20, 0x6a, 0x61, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x20, 0x61,
0x65, 0x67, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67,
0x45, 0x6b, 0x65, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d,
0x65, 0x45, 0x6b, 0x65, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x45, 0x6b, 0x65, 0x72,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x66, 0x67, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x67, 0x61, 0x6d, 0x65, 0x57, 0x65,
0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x69, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b,
0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x54, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65,
0x54, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x69, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6b, 0x65,
0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f, 0x75,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d,
0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6b, 0x65, 0x6c,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41,
0x6e, 0x61, 0x64, 0x69, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x6d, 0x65, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x41,
0x6e, 0x61, 0x64, 0x69, 0x72, 0x20, 0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x70,
0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x41, 0x70, 0x69, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x70, 0x69,
0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69,
0x6d, 0x65, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x64, 0x7a, 0x6f, 0x6d,
0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x6d, 0x65, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69,
0x6d, 0x65, 0x41, 0x6b, 0x74, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72,
0x61, 0x62, 0x69, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b,
0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x194, 0x65, 0x74, 0x6f, 0x256, 0x6f, 0x192, 0x65, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x194, 0x65, 0x74, 0x6f, 0x256, 0x6f, 0x192,
0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x194, 0x65, 0x74, 0x6f, 0x256, 0x6f, 0x192, 0x65,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65,
0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x263, 0x65, 0x74, 0x6f, 0x256, 0x6f, 0x66, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a,
0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65,
0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x68, 0x75, 0x74, 0x61,
0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42,
0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d,
0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61,
0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x72, 0x69, 0x73,
0x74, 0x6d, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x63,
0x6f, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x44, 0x61, 0x76,
0x69, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73,
0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254,
0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x194, 0x65,
0x64, 0x7a, 0x65, 0x192, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x194, 0x65, 0x64,
0x7a, 0x65, 0x192, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69,
0x6d, 0x65, 0x194, 0x65, 0x64, 0x7a, 0x65, 0x192, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x57, 0x65, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65,
0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b,
0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x46, 0x69, 0x6a, 0x69, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254,
0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46,
0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x65,
0x72, 0x6e, 0x20, 0x26, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74,
0x69, 0x63, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61,
0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x75, 0x6c, 0x66,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48,
0x6f, 0x76, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f,
0x76, 0x64, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x4f, 0x63, 0x65, 0x61,
0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x74, 0x73, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256,
0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x74, 0x73, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x14b, 0x6b, 0x65, 0x6b,
0x65, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4c,
0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x4d, 0x61, 0x6b, 0x61,
0x75, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e,
0x79, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x14b, 0x6b,
0x65, 0x6b, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61,
0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x77, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x77, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b,
0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e,
0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65,
0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e,
0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x4e, 0x69, 0x75, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x64, 0x7a, 0x6f, 0x6d,
0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4f, 0x6d, 0x73,
0x6b, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x70, 0x75,
0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x65, 0x72,
0x75, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50,
0x65, 0x72, 0x75, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x65, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x69, 0x6e, 0x65, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254,
0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x74, 0x2e, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x74, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61,
0x6e, 0x67, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x254, 0x72, 0x64, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x254,
0x72, 0x64, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f,
0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x254,
0x72, 0x64, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x52, 0x65,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256,
0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x20, 0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65,
0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x6b,
0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x65, 0x6e, 0x65, 0x7a,
0x75, 0x65, 0x6c, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6f, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x26, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed,
0xf0, 0x4d, 0x69, 0xf0, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74,
0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x75, 0xf0, 0x75, 0x72, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74,
0x75, 0x72, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75,
0x72, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20,
0x74, 0xed, 0xf0, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x41,
0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x61,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61,
0x69, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61,
0x69, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x70, 0x69, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x41, 0x70, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73,
0x6b, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x41,
0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74,
0x75, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75,
0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x6d, 0x69, 0xf0, 0x20,
0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x6d, 0x69, 0xf0, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x6d, 0x69, 0xf0, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x41, 0x76,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x6d,
0x69, 0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x41, 0x76, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73, 0x74,
0x75, 0x72, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x65, 0x79,
0x73, 0x74, 0x75, 0x72, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x65, 0x79, 0x73, 0x74, 0x75, 0x72,
0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x65, 0x79, 0x73,
0x74, 0x75, 0x72, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41,
0x73, 0x65, 0x72, 0x62, 0x61, 0x64, 0x6a, 0x61, 0x6e, 0x20, 0x74, 0xed,
0xf0, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x64, 0x6a, 0x61, 0x6e, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x73,
0x65, 0x72, 0x62, 0x61, 0x64, 0x6a, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x72,
0x6e, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x7a, 0x6f, 0x72, 0x75,
0x72, 0x6e, 0x61, 0x72, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x72, 0x6e, 0x61, 0x72,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x6a, 0x20, 0x74, 0xed, 0xf0,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x6a, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x6a, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x74,
0xed, 0xf0, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72,
0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20, 0x74, 0xed, 0xf0, 0x47,
0x72, 0xf8, 0x6e, 0x68, 0xf8, 0x76, 0x64, 0x61, 0x6f, 0x79, 0x67, 0x67,
0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x72, 0xf8, 0x6e, 0x68,
0xf8, 0x76, 0x64, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x72,
0xf8, 0x6e, 0x68, 0xf8, 0x76, 0x64, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a,
0x61, 0x72, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x4b, 0x69, 0x6c, 0x69, 0x20, 0x74, 0xed,
0xf0, 0x4b, 0x69, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x4b, 0x69, 0x6c, 0x69, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4b, 0x69, 0x6e, 0x61, 0x20, 0x74,
0xed, 0xf0, 0x4b, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4a, 0xf3, 0x6c, 0x61, 0x6f,
0x79, 0x67, 0x67, 0x6a, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0,
0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61,
0x72, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x6f, 0x79,
0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x6f, 0x79, 0x67,
0x67, 0x6a, 0x61, 0x72, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x43, 0x75, 0x62, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x75,
0x62, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x43, 0x75, 0x62, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x44, 0x61, 0x76, 0x69, 0x73, 0x20, 0x74, 0xed, 0xf0,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74,
0x75, 0x72, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x50,
0xe1, 0x73, 0x6b, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x69, 0x6e, 0x20, 0x74,
0xed, 0xf0, 0x50, 0xe1, 0x73, 0x6b, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x69,
0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x50, 0xe1, 0x73, 0x6b, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x69, 0x6e, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x45, 0x6b, 0x76,
0x61, 0x64, 0x6f, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x69, 0xf0, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x69, 0xf0,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x69, 0xf0, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x65, 0x76,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x6c, 0x6f, 0x6e, 0x67, 0x72, 0x69, 0x20, 0x45, 0x79, 0x73, 0x74,
0x75, 0x72, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x65, 0x76,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6f, 0x79,
0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61,
0x72, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6f, 0x79, 0x67,
0x67, 0x6a, 0x61, 0x72, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x69,
0x6a, 0x69, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x47,
0x75, 0x6a, 0x61, 0x6e, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x72, 0x6f,
0x6e, 0x73, 0x6b, 0x75, 0x20, 0x73, 0x75, 0x6e, 0x6e, 0x61, 0x72, 0x75,
0x20, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0xf8, 0x6b, 0x69, 0x20, 0x6f, 0x67,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x20, 0x74,
0xed, 0xf0, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20,
0x74, 0xed, 0xf0, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x20, 0x74,
0xed, 0xf0, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x79, 0x67, 0x67, 0x6a,
0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0,
0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e, 0x6c,
0x65, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73,
0x74, 0x75, 0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x65, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e,
0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x67,
0x72, 0xf8, 0x6e, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0xed,
0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e,
0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20,
0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x47, 0x75, 0x6c, 0x66,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x6f, 0x76,
0x64, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x68, 0x61, 0x76, 0x20, 0x74, 0xed, 0xf0, 0x49,
0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4d,
0x69, 0xf0, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x72, 0x61, 0x6e,
0x20, 0x74, 0xed, 0xf0, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x72, 0x61, 0x6e, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0xcd, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x20, 0x74, 0xed, 0xf0, 0xcd, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0xcd, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75,
0x72, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x4b, 0x61,
0x73, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x74, 0xed,
0xf0, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x20, 0x74, 0xed, 0xf0, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4b,
0x69, 0x72, 0x67, 0x69, 0x73, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4c,
0x69, 0x6e, 0x65, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74,
0xed, 0xf0, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20,
0x74, 0xed, 0xf0, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x6c, 0x61, 0x69, 0x73, 0x69, 0x61,
0x20, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x6f, 0x79,
0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61,
0x72, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0xf3, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0xf3, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x4d, 0xf3, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x20, 0x74, 0xed, 0xf0, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x74, 0xed, 0xf0, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x20, 0x74, 0xed, 0xf0, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4d, 0x6f, 0x73,
0x6b, 0x76, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20,
0x28, 0x42, 0x75, 0x72, 0x6d, 0x61, 0x29, 0x20, 0x74, 0xed, 0xf0, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0xf3, 0x6e, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x73, 0xe6, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x73, 0xe6, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0xfd,
0x73, 0xe6, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x74, 0xed,
0xf0, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x79, 0x67, 0x67,
0x6a, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x6f, 0x79, 0x67, 0x67, 0x6a, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f,
0x79, 0x67, 0x67, 0x6a, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x74, 0xed, 0xf0, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4f, 0x6d,
0x73, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4f, 0x6d, 0x73,
0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x70,
0x75, 0x61, 0x20, 0x4e, 0xfd, 0x67, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20,
0x74, 0xed, 0xf0, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x20,
0x74, 0xed, 0xf0, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x50, 0x65, 0x72, 0x75, 0x20, 0x74, 0xed, 0xf0,
0x50, 0x65, 0x72, 0x75, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x50, 0x65, 0x72, 0x75, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x73, 0x6f,
0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x74, 0xed, 0xf0, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x74, 0xed, 0xf0, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74,
0xed, 0xf0, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x74, 0xed, 0xf0,
0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x74, 0xed, 0xf0,
0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x53, 0x65, 0x79, 0x73, 0x6b, 0x65, 0x6c, 0x6c,
0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x53,
0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61,
0x72, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x75, 0xf0, 0x75, 0x72, 0x67, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72,
0x20, 0x74, 0xed, 0xf0, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20,
0x74, 0xed, 0xf0, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x54, 0x61, 0x64, 0x73, 0x6a, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed,
0xf0, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x20, 0x74, 0xed, 0xf0, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x69, 0x20, 0x74, 0xed, 0xf0, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x69, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x55, 0x73, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x74, 0xed, 0xf0,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x56,
0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20,
0x74, 0xed, 0xf0, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x74, 0xed, 0xf0, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x74, 0xed, 0xf0,
0x57, 0x61, 0x6b, 0x65, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x20, 0x74, 0xed,
0xf0, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6f, 0x67, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61,
0x72, 0x20, 0x74, 0xed, 0xf0, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x74, 0xed, 0xf0, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x20, 0x74, 0xed, 0xf0, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x66, 0x72, 0x69,
0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69,
0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x66, 0x72, 0x69,
0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x6f, 0x67, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72,
0x61, 0x6e, 0x67, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72,
0x61, 0x6e, 0x67, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72,
0x61, 0x6e, 0x67, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x53,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20, 0x48,
0x69, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20, 0x48,
0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20,
0x48, 0x69, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x42, 0x75, 0x6e, 0x64, 0x6f, 0x6b, 0x20, 0x73, 0x61, 0x20, 0x48, 0x69,
0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x75,
0x6e, 0x64, 0x6f, 0x6b, 0x20, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6c, 0x61,
0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x75, 0x6e, 0x64,
0x6f, 0x6b, 0x20, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x67, 0x61,
0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69,
0x6b, 0x6f, 0x20, 0x73, 0x61, 0x20, 0x48, 0x49, 0x6c, 0x61, 0x67, 0x61,
0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69,
0x6b, 0x6f, 0x20, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x67, 0x61,
0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69,
0x6b, 0x6f, 0x20, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x67, 0x61,
0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73,
0x61, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61,
0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20,
0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72,
0x61, 0x6e, 0x67, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x6f,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x6b, 0x6f, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x6f, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e,
0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61,
0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x6e, 0x67, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e,
0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x20, 0x6e,
0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x20, 0x6e, 0x67, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20,
0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73,
0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4b, 0x61,
0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e,
0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e,
0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75,
0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61,
0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61,
0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x6a, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4b, 0x61, 0x6c, 0x61, 0x68, 0x61, 0x74, 0x69, 0x20, 0x6e, 0x67,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62,
0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62,
0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73,
0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x47,
0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c,
0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c,
0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x53, 0x69, 0x6c,
0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x6e,
0x61, 0x6b, 0x61, 0x2d, 0x73, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e,
0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e,
0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e,
0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e,
0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x6d, 0x75, 0x67, 0x61,
0x6e, 0x67, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x20, 0x61, 0x74,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x6b, 0x6f, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67,
0x61, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e,
0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e,
0x67, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e,
0x67, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75,
0x72, 0x61, 0x6e, 0x67, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72,
0x61, 0x6e, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x79, 0x72, 0x67,
0x79, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20,
0x73, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f, 0x20, 0x6e, 0x67,
0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f, 0x20,
0x6e, 0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b,
0x6f, 0x20, 0x6e, 0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e,
0x67, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20,
0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x50, 0x65, 0x72, 0x75, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20,
0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x67, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e,
0x67, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x41, 0x63, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x41, 0x63, 0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x63, 0x72, 0x65, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x66, 0x67, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x45,
0x74, 0x65, 0x6c, 0xe4, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c,
0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0x6b, 0x69, 0x6e, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c,
0x74, 0x61, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0x6b, 0x69, 0x6e, 0x65,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0x6b, 0x69, 0x6e, 0x65, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79,
0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x69, 0x74, 0xe4,
0x69, 0x6e, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64,
0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x69, 0x74,
0xe4, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76,
0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x69, 0x74, 0xe4, 0x69, 0x6e,
0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x61, 0x6c, 0x6c, 0x69, 0x6f, 0x76, 0x75, 0x6f, 0x72, 0x74, 0x65, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6c, 0x6c, 0x69, 0x6f, 0x76,
0x75, 0x6f, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6c, 0x6c, 0x69,
0x6f, 0x76, 0x75, 0x6f, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61,
0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e,
0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68,
0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x54,
0x79, 0x79, 0x6e, 0x65, 0x6e, 0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x59,
0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20,
0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e, 0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x70, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x70, 0x69,
0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x70, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x71, 0x74, 0x61, 0x77, 0x2019, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x71, 0x74, 0x61, 0x77, 0x2019, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x41, 0x71, 0x74, 0x61, 0x77, 0x2019, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x75, 0x64, 0x69, 0x2d, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53,
0x61, 0x75, 0x64, 0x69, 0x2d, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x53, 0x61, 0x75, 0x64, 0x69, 0x2d, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e,
0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e,
0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4c,
0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x69, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e,
0x73, 0x69, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e,
0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6e, 0x61,
0x64, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x6e,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61,
0x6e, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b,
0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x74, 0x69, 0x73, 0x65, 0x6e, 0x20, 0x4b,
0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x74,
0x69, 0x73, 0x65, 0x6e, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4,
0x6e, 0x74, 0x69, 0x73, 0x65, 0x6e, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x69,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61,
0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64,
0x17e, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x7a,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x61,
0x73, 0x65, 0x79, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x161, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x4b, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x69, 0x69, 0x6e,
0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4a,
0x6f, 0x75, 0x6c, 0x75, 0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x6f, 0x6b, 0x6f, 0x73, 0x73, 0x61, 0x61,
0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x61, 0x61,
0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x6f, 0x6f,
0x6b, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x43,
0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x75, 0x75,
0x62, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x75, 0x75, 0x62,
0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x44, 0x61, 0x76, 0x69, 0x73, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x54, 0x69, 0x6d, 0x6f,
0x72, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0xe4, 0xe4, 0x73,
0x69, 0xe4, 0x69, 0x73, 0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x50, 0xe4, 0xe4, 0x73, 0x69, 0xe4, 0x69, 0x73, 0x73,
0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x50, 0xe4, 0xe4, 0x73, 0x69, 0xe4,
0x69, 0x73, 0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x6f, 0x70, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74,
0xe4, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x69, 0x73,
0x65, 0x6d, 0x6d, 0xe4, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e,
0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x69,
0x64, 0x17e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x69, 0x64,
0x17e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x52, 0x61, 0x6e, 0x73, 0x6b,
0x61, 0x6e, 0x20, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x52, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x6e, 0x20,
0x65, 0x74, 0x65, 0x6c, 0xe4, 0x69, 0x73, 0x74, 0x65, 0x6e, 0x20, 0x6a,
0x61, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x74,
0x65, 0x6e, 0x20, 0x61, 0x6c, 0x75, 0x65, 0x69, 0x64, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f,
0x73, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x61, 0x61, 0x72,
0x74, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x47,
0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x47, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x49, 0x74, 0xe4, 0x2d, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x47, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x47, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61,
0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x47,
0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x75, 0x61, 0x6d, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x65, 0x6d,
0x69, 0x69, 0x72, 0x69, 0x6b, 0x75, 0x6e, 0x74, 0x69, 0x65, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x48, 0x61, 0x76, 0x61, 0x69, 0x6a, 0x69, 0x6e, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x74, 0x69, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x48,
0x61, 0x76, 0x61, 0x69, 0x6a, 0x69, 0x6e, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x74, 0x69, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x61, 0x76, 0x61, 0x69, 0x6a,
0x69, 0x6e, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x74, 0x69, 0x65, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x48, 0x6f, 0x76, 0x64, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x49, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x49, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6c,
0x74, 0x61, 0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x49, 0x74, 0xe4, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69,
0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x72,
0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61,
0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x61, 0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b, 0x61, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x4b, 0x61, 0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b, 0x61, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49,
0x74, 0xe4, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69,
0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x69, 0x72, 0x67,
0x69, 0x73, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x72,
0x69, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4c, 0x69, 0x6e, 0x65, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x63, 0x61,
0x6f, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x61,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69,
0x69, 0x76, 0x69, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x73, 0x61, 0x61, 0x72, 0x74,
0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x6b, 0x73, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x6b, 0x73, 0x65, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x6b, 0x73, 0x65, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x77, 0x73,
0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x6f, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e,
0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65,
0x6e, 0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e, 0x6d, 0x65,
0x72, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61,
0x72, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x61, 0x75, 0x72,
0x75, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65, 0x6e,
0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65, 0x6e, 0x2d, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64,
0x65, 0x6e, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75,
0x64, 0x65, 0x6e, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x6e, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65, 0x6e, 0x2d,
0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75,
0x64, 0x65, 0x6e, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x6e, 0x69,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x69, 0x75, 0x65, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x61,
0x61, 0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x6f, 0x68, 0x6a,
0x6f, 0x69, 0x73, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x61, 0x6e, 0x69,
0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4f, 0x6d, 0x73, 0x6b, 0x69,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d,
0x55, 0x75, 0x64, 0x65, 0x6e, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x50, 0x65, 0x72, 0x75, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x65,
0x72, 0x75, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x50, 0x65, 0x72, 0x75, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x6e, 0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x6e, 0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x6e, 0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x6f, 0x68, 0x6e, 0x70,
0x65, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x6a, 0x6f, 0x6e,
0x67, 0x6a, 0x61, 0x6e, 0x67, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x52, 0xe9, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x69,
0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x45, 0x74, 0x65, 0x6c, 0xe4, 0x2d, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56,
0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x6b,
0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x69, 0x6e, 0x20, 0x6a, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x63, 0x72, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x63, 0x72, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x63, 0x72, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66,
0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75,
0x65, 0x20, 0x6d, 0xe9, 0x72, 0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72,
0x69, 0x71, 0x75, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75,
0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71,
0x75, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73,
0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6c,
0x6d, 0x61, 0x20, 0x41, 0x74, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6c,
0x6d, 0x61, 0x20, 0x41, 0x74, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61,
0x20, 0x41, 0x74, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d,
0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73,
0x74, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69,
0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x73, 0x74, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72,
0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73,
0x74, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69,
0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0x52, 0x6f, 0x63, 0x68, 0x65, 0x75, 0x73, 0x65, 0x73, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x68, 0x65, 0x75, 0x73,
0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x68, 0x65, 0x75,
0x73, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71,
0x75, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72,
0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61,
0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0x41, 0x70, 0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0x41, 0x6b, 0x74, 0x61, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019,
0x41, 0x6b, 0x74, 0x61, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x41, 0x6b, 0x74, 0x61,
0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x71,
0x74, 0xf6, 0x62, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0xf6,
0x62, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61,
0x62, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72,
0x61, 0x62, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61,
0x62, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75,
0x65, 0x73, 0x74, 0x20, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0xe9,
0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x71, 0x75, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x71,
0x75, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x71, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65,
0x2d, 0x6f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f, 0x75, 0x65, 0x73, 0x74,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75,
0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f,
0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0xef, 0x64, 0x6a, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0xef, 0x64, 0x6a, 0x61, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0xef, 0x64, 0x6a, 0x61, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7,
0x6f, 0x72, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7,
0x6f, 0x72, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72,
0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42, 0x68,
0x6f, 0x75, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x75, 0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x75, 0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x75, 0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c,
0x65, 0x73, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x75, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee,
0x6c, 0x65, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c,
0x65, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee,
0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe2, 0x71,
0x75, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc9, 0x71, 0x75, 0x61,
0x74, 0x65, 0x75, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75,
0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73,
0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x75, 0x69,
0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65,
0x73, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x75, 0x69, 0x6e, 0x65, 0x73, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x6f,
0x75, 0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x69, 0x64, 0x6a,
0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20,
0x46, 0x69, 0x64, 0x6a, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65,
0x73, 0x20, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x65, 0x20, 0x66, 0x72, 0x61, 0x6e, 0xe7, 0x61, 0x69, 0x73, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x54, 0x65, 0x72,
0x72, 0x65, 0x73, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x73, 0x20, 0x65, 0x74, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74,
0x69, 0x71, 0x75, 0x65, 0x73, 0x20, 0x66, 0x72, 0x61, 0x6e, 0xe7, 0x61,
0x69, 0x73, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73,
0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6d, 0x6f, 0x79, 0x65, 0x6e, 0x6e, 0x65, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74,
0x20, 0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20,
0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x75, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73,
0x74, 0x20, 0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65,
0x73, 0x74, 0x20, 0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73,
0x74, 0x20, 0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47,
0x75, 0x61, 0x6d, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20,
0x47, 0x6f, 0x6c, 0x66, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x75, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x20, 0x2d, 0x20,
0x41, 0x6c, 0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x20, 0x2d, 0x20,
0x41, 0x6c, 0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x20, 0x2d, 0x20, 0x41, 0x6c,
0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e,
0x64, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x4f, 0x63, 0xe9, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x65,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64,
0x6f, 0x63, 0x68, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x75, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x65, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x65, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73,
0x74, 0x20, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x65, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49,
0x72, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x2019, 0x49, 0x72, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0x49, 0x72, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x2019, 0x49, 0x72, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x49,
0x72, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x2019, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72,
0x61, 0xeb, 0x6c, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20,
0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73,
0x74, 0x20, 0x64, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x75, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x72,
0xe9, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f,
0x72, 0xe9, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0xe9, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0xef, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xef, 0x61, 0x72,
0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xef,
0x61, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75,
0x20, 0x4b, 0x69, 0x72, 0x67, 0x68, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x61,
0x6e, 0x6b, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73,
0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x4c, 0x69, 0x67, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61,
0x6f, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4d,
0x61, 0x6c, 0x61, 0x69, 0x73, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x69, 0x73, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x63, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x77, 0x73, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x75, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x20,
0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0x4f, 0x75, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x2019, 0x4f, 0x75, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x2019, 0x4f, 0x75, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x4d,
0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x75, 0x20, 0x4e, 0xe9, 0x70, 0x61, 0x6c, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f,
0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x43, 0x61, 0x6c, 0xe9, 0x64,
0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x43, 0x61, 0x6c,
0xe9, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x75,
0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x43, 0x61, 0x6c, 0xe9, 0x64, 0x6f,
0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d,
0x5a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d,
0x5a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0xe9,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x4e, 0x65, 0x75, 0x76,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x4e, 0x65, 0x75, 0x76, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72,
0x65, 0x2d, 0x4e, 0x65, 0x75, 0x76, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0xee, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x20, 0x64, 0x75, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x73, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x6f, 0x73, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x50, 0x61, 0x70,
0x6f, 0x75, 0x61, 0x73, 0x69, 0x65, 0x2d, 0x4e, 0x6f, 0x75, 0x76, 0x65,
0x6c, 0x6c, 0x65, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0xe9, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20,
0x50, 0xe9, 0x72, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0xe9,
0x72, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0xe9, 0x72, 0x6f, 0x75, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x50, 0x68, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x73, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e,
0x69, 0x78, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x50, 0x69,
0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e,
0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x20, 0x52, 0xe9, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x75, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47,
0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x20, 0x64, 0x75, 0x20, 0x53, 0x75,
0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x54, 0x61, 0x64, 0x6a, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0xe9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73,
0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x4f, 0x75, 0x7a, 0x62, 0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x7a, 0x62,
0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x4f, 0x75, 0x7a, 0x62, 0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x57,
0x61, 0x6b, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x65, 0x74, 0x2d, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75,
0x72, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x63, 0x72, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71,
0x75, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71,
0x75, 0x65, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20,
0x64, 0x75, 0x20, 0x53, 0x75, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66,
0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f,
0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d,
0x61, 0x20, 0x41, 0x74, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x52, 0x6f,
0x63, 0x68, 0x65, 0x75, 0x73, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41,
0x70, 0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6b, 0x74, 0x61, 0x6f,
0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f,
0x75, 0x65, 0x73, 0x74, 0x20, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x71, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f, 0x75, 0x65, 0x73, 0x74,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73,
0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f,
0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0xef, 0x64, 0x6a, 0x61, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42,
0x72, 0x75, 0x6e, 0xe9, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43, 0x61,
0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x6f, 0x75, 0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x73,
0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x79, 0x61,
0x6e, 0x65, 0x20, 0x66, 0x72, 0x61, 0x6e, 0xe7, 0x61, 0x69, 0x73, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64,
0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20,
0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x48, 0x61, 0x77, 0x61,
0xef, 0x2d, 0x41, 0x6c, 0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e,
0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x2d,
0x41, 0x6c, 0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x2d, 0x41, 0x6c,
0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x63, 0xe9, 0x61,
0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x65, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019,
0x49, 0x72, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x6f,
0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61,
0xeb, 0x6c, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f,
0x72, 0xe9, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0xef, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x4f, 0x75,
0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x43, 0x61,
0x6c, 0xe9, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d,
0x5a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x54, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x4e, 0x65, 0x75, 0x76, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x6f, 0x75,
0xe9, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019,
0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50,
0xe9, 0x72, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x50, 0x68,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x7a,
0x62, 0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x6f, 0x75,
0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe2, 0x6c, 0x4f,
0x72, 0x65, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0xe2,
0x6c, 0x4f, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0xe2,
0x6c, 0x4f, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x6f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0xe2, 0x6c,
0x4f, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x6f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x6f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x68, 0x65, 0x4f, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x68,
0x65, 0x4f, 0x72, 0x65, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x68, 0x65, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44,
0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44,
0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd2c, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a,
0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35,
0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd27,
0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd25,
0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd22,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd25,
0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25,
0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e,
0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e,
0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd24,
0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd46, 0xd83a, 0xdd2d,
0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46,
0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a,
0xdd46, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44,
0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd26,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a,
0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a,
0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a,
0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a,
0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a,
0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a,
0xdd2e, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a,
0xdd2e, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20,
0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a,
0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a,
0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c,
0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a,
0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd34,
0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a,
0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a,
0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24,
0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24,
0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2d,
0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a,
0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd26, 0xd83a, 0xdd2e,
0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x2d,
0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a,
0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd25, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a,
0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a,
0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2e,
0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11,
0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e,
0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11,
0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a,
0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd37,
0xd83a, 0xdd46, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a,
0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd01, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0xd83a, 0xdd35, 0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3e, 0xd83a,
0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a,
0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25,
0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a,
0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a,
0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a,
0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd23, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a,
0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0x2d,
0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0x2d, 0xd83a, 0xdd0a, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd2b, 0xd83a, 0xdd34,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0x20, 0x26, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd26,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd33,
0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd31, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a,
0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd2c, 0xd83a, 0xdd35, 0xd83a,
0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd25, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd3b,
0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd00,
0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0x20, 0xd83a,
0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0xd83a,
0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a,
0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd33,
0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2d,
0xd83a, 0xdd28, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd28,
0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2d,
0xd83a, 0xdd28, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a,
0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a,
0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd27, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27,
0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd35,
0x2d, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0x2d, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd31,
0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23,
0xd83a, 0xdd35, 0x2d, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2b, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a,
0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22,
0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd2b,
0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a,
0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a,
0xdd2e, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46,
0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd01, 0xd83a,
0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2b,
0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd28, 0xd83a,
0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a,
0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22,
0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a,
0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31,
0x2d, 0xd83a, 0xdd1f, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd1f, 0xd83a, 0xdd2b,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd1f, 0xd83a, 0xdd2b, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd0a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd31, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33,
0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0xd83a,
0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd05, 0xd83a,
0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24,
0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2b, 0xd83a,
0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0x20, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3d, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a,
0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0x20, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a,
0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3d, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x20, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd3d, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10,
0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd2d,
0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd25, 0xd83a,
0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a,
0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd25, 0xd83a,
0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06,
0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd06,
0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22,
0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a,
0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a,
0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a,
0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d,
0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd3c, 0x2e,
0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd34, 0x20, 0x26, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2b,
0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd3c, 0x2e, 0x20, 0xd83a, 0xdd06,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0x20, 0x26,
0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd3c, 0x2e, 0x20, 0xd83a,
0xdd06, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0x20,
0x26, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd24,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a,
0xdd3c, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd24,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd24,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd08, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd08, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a,
0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a,
0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a,
0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a,
0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2c, 0x2d, 0xd83a, 0xdd14, 0xd83a,
0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a,
0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a,
0xd83a, 0xdd22, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd31, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a,
0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35,
0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a,
0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd33,
0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd27,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd35,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd41, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a,
0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd13, 0xd83a,
0xdd41, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd13, 0xd83a,
0xdd41, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd31,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1c,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a,
0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd24,
0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1c,
0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27,
0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a,
0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd27,
0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a,
0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd0f, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd0f, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd27, 0x20, 0x26, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd3c,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a,
0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a,
0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd12, 0xd83a,
0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd2b, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32,
0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd2b,
0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd35,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd35,
0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xc0, 0x6d, 0x20, 0x41, 0x63, 0x72,
0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x63, 0x72, 0x65,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x41, 0x63, 0x72, 0x65, 0xc0, 0x6d, 0x20, 0x41, 0x66,
0x67, 0x68, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0xc0, 0x6d,
0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x66, 0x72,
0x61, 0x67, 0x61, 0xc0, 0x6d, 0x20, 0x41, 0x66, 0x72, 0x61, 0x67, 0x61,
0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x41, 0x66,
0x72, 0x61, 0x67, 0x61, 0x20, 0x61, 0x20, 0x44, 0x65, 0x61, 0x73, 0xc0,
0x6d, 0x20, 0x41, 0x66, 0x72, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e, 0x20,
0x49, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x66,
0x72, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x54,
0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x41, 0x66, 0x72, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e, 0x20,
0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0xc0, 0x6d,
0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0xc0, 0x6d, 0x20, 0x41, 0x6d, 0x61, 0x73,
0x6f, 0x69, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x6d,
0x61, 0x73, 0x6f, 0x69, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x6d, 0x61, 0x73,
0x6f, 0x69, 0x6e, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61,
0x6e, 0x20, 0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68,
0x61, 0x6e, 0x20, 0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67,
0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20,
0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0xc0, 0x6d,
0x20, 0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x20,
0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x20,
0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x69, 0x6d,
0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45,
0x61, 0x72, 0xc0, 0x6d, 0x20, 0x4d, 0x6f, 0x6e, 0x61, 0x64, 0x68, 0x20,
0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x6f, 0x6e, 0x61, 0x64, 0x68, 0x20,
0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x4d, 0x6f, 0x6e, 0x61, 0x64, 0x68, 0x20, 0x41, 0x69, 0x6d, 0x65,
0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0xc0, 0x6d, 0x20, 0x61, 0x2019, 0x20,
0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x68, 0xe8, 0x69, 0x6d,
0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43,
0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x68, 0xe8, 0x69, 0x6d, 0x68,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e,
0x20, 0x53, 0x68, 0xe8, 0x69, 0x6d, 0x68, 0xc0, 0x6d, 0x20, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0xc0, 0x6d, 0x20, 0x41, 0x70, 0x69, 0x61, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x70, 0x69, 0x61, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41,
0x70, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0xc0, 0x6d, 0x20, 0x41, 0x71,
0x74, 0x6f, 0x62, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41,
0x71, 0x74, 0x6f, 0x62, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0xc0, 0x6d, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x63, 0x68,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61,
0x63, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x63, 0x68,
0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d, 0x41, 0x72, 0x67, 0x61,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d, 0x41, 0x72, 0x67, 0x61, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d,
0x41, 0x72, 0x67, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x65, 0xc0, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d, 0x41, 0x72, 0x67, 0x61, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x65, 0x20, 0x53, 0x69, 0x61, 0x72, 0x61, 0x69, 0x63,
0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x2d, 0x41, 0x72, 0x67, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x65, 0x20,
0x53, 0x69, 0x61, 0x72, 0x61, 0x69, 0x63, 0x68, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x2d, 0x41, 0x72, 0x67, 0x61, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x65, 0x20, 0x53, 0x69, 0x61, 0x72, 0x61, 0x69, 0x63, 0x68, 0xc0,
0x6d, 0x20, 0x41, 0x69, 0x72, 0x6d, 0x65, 0x69, 0x6e, 0x69, 0x61, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x69, 0x72, 0x6d, 0x65, 0x69,
0x6e, 0x69, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x69, 0x72, 0x6d, 0x65, 0x69,
0x6e, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75,
0x61, 0x69, 0x6e, 0x20, 0x53, 0x68, 0x69, 0x61, 0x72, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69,
0x6e, 0x20, 0x53, 0x68, 0x69, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x61, 0x2019,
0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x68, 0x69, 0x61,
0x72, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20,
0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20,
0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x73, 0x74, 0x72,
0xe0, 0x69, 0x6c, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64,
0x68, 0x61, 0x6e, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69,
0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41,
0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20,
0x49, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61,
0x6e, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20,
0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x41, 0x73, 0x74,
0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61,
0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x73, 0x74, 0x72,
0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61,
0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x41, 0x73,
0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49,
0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x73, 0x74,
0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61,
0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69,
0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x41,
0x73, 0x61, 0x72, 0x62, 0x61, 0x69, 0x64, 0x65, 0xe0, 0x69, 0x6e, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x73, 0x61, 0x72, 0x62, 0x61,
0x69, 0x64, 0x65, 0xe0, 0x69, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x73, 0x61,
0x72, 0x62, 0x61, 0x69, 0x64, 0x65, 0xe0, 0x69, 0x6e, 0xc0, 0x6d, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x41, 0x73, 0x6f, 0x72, 0x61, 0x63, 0x68, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61,
0x6e, 0x61, 0x6e, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x61, 0x63, 0x68, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e,
0x61, 0x6e, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x61, 0x63, 0x68, 0xc0, 0x6d,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x61, 0x69, 0x73, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x61, 0x69, 0x73, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x61, 0x69, 0x73, 0xc0, 0x6d, 0x20, 0x42, 0x75, 0x74, 0xe0,
0x69, 0x6e, 0xc0, 0x6d, 0x20, 0x42, 0x6f, 0x69, 0x6c, 0x69, 0x62, 0x68,
0x69, 0x61, 0xc0, 0x6d, 0x20, 0x42, 0x68, 0x72, 0x61, 0x73, 0xed, 0x6c,
0x69, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x42, 0x68, 0x72,
0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x42, 0x68, 0x72,
0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x42, 0x68, 0x72,
0xf9, 0x6e, 0x61, 0x69, 0x67, 0x68, 0x20, 0x44, 0xe0, 0x72, 0x20, 0x61,
0x73, 0x2d, 0x53, 0x61, 0x6c, 0xe0, 0x6d, 0xc0, 0x6d, 0x20, 0x61, 0x2019,
0x20, 0x43, 0x68, 0x69, 0x70, 0x20, 0x55, 0x61, 0x69, 0x6e, 0x65, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x69,
0x70, 0x20, 0x55, 0x61, 0x69, 0x6e, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x61, 0x2019,
0x20, 0x43, 0x68, 0x69, 0x70, 0x20, 0x55, 0x61, 0x69, 0x6e, 0x65, 0xc0,
0x6d, 0x20, 0x43, 0x68, 0x61, 0x73, 0x65, 0x79, 0xc0, 0x6d, 0x20, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0xc0, 0x6d, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53,
0x69, 0x6c, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x53, 0x69, 0x6c, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53,
0x69, 0x6c, 0x65, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0xec, 0x6e,
0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53,
0xec, 0x6e, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0xec, 0x6e,
0x65, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x6e,
0x61, 0x20, 0x4e, 0x6f, 0x6c, 0x6c, 0x61, 0x69, 0x67, 0xc0, 0x6d, 0x20,
0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x43, 0x68, 0x6f,
0x63, 0x6f, 0x73, 0xc0, 0x6d, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x69, 0x6d,
0x62, 0x69, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x43, 0x6f,
0x6c, 0x6f, 0x69, 0x6d, 0x62, 0x69, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x6f,
0x6c, 0x6f, 0x69, 0x6d, 0x62, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e,
0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4c, 0x65, 0x74, 0x68, 0x2d,
0x74, 0x68, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xc0, 0x6d, 0x20, 0x43, 0xf9, 0x62, 0x61,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x43, 0xf9, 0x62, 0x61, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x43, 0xf9, 0x62, 0x61, 0xc0, 0x6d, 0x20, 0x44, 0x68, 0x61,
0x76, 0x69, 0x73, 0xc0, 0x6d, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0xc0, 0x6d,
0x20, 0x54, 0x68, 0xec, 0x6f, 0x6d, 0x6f, 0x72, 0x20, 0x61, 0x6e, 0x20,
0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e,
0x20, 0x6e, 0x61, 0x20, 0x43, 0xe0, 0x69, 0x73, 0x67, 0x65, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20,
0x6e, 0x61, 0x20, 0x43, 0xe0, 0x69, 0x73, 0x67, 0x65, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x43, 0xe0,
0x69, 0x73, 0x67, 0x65, 0xc0, 0x6d, 0x20, 0x45, 0x61, 0x63, 0x75, 0x61,
0x64, 0x6f, 0x69, 0x72, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68,
0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d,
0x45, 0xf2, 0x72, 0x70, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x52,
0x6f, 0x69, 0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20,
0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0xc0,
0x6d, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d, 0x45,
0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69,
0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20,
0x45, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69,
0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20,
0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69,
0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x6e, 0x61, 0x73,
0x20, 0x66, 0x68, 0x61, 0x69, 0x64, 0x65, 0x20, 0x65, 0x61, 0x72, 0xc0,
0x6d, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d, 0x45,
0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69,
0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20,
0x49, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69,
0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20,
0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x46, 0xe0, 0x63, 0x6c, 0x61,
0x6e, 0x6e, 0x61, 0x63, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x46, 0xe0, 0x63, 0x6c, 0x61, 0x6e, 0x6e, 0x61, 0x63, 0x68, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e,
0x61, 0x6e, 0x20, 0x46, 0xe0, 0x63, 0x6c, 0x61, 0x6e, 0x6e, 0x61, 0x63,
0x68, 0xc0, 0x6d, 0x20, 0x46, 0xec, 0x64, 0x69, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x46, 0xec, 0x64, 0x69, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x46, 0xec,
0x64, 0x69, 0xc0, 0x6d, 0x20, 0x47, 0x75, 0x69, 0x64, 0x68, 0x65, 0xe0,
0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x69, 0x6e, 0x67,
0x65, 0xc0, 0x6d, 0x20, 0x44, 0x65, 0x61, 0x73, 0x61, 0x63, 0x68, 0x20,
0x61, 0x67, 0x75, 0x73, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x61,
0x69, 0x67, 0x65, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x72,
0x61, 0x69, 0x6e, 0x67, 0x65, 0xc0, 0x6d, 0x20, 0x47, 0x68, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0xc0, 0x6d, 0x20, 0x47, 0x68, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43,
0x61, 0x69, 0x72, 0x74, 0x62, 0x68, 0x65, 0x69, 0x6c, 0x65, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x69, 0x72,
0x74, 0x62, 0x68, 0x65, 0x69, 0x6c, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x43, 0x61, 0x69, 0x72, 0x74, 0x62, 0x68, 0x65, 0x69, 0x6c, 0x65,
0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20,
0x47, 0x68, 0x69, 0x6c, 0x65, 0x61, 0x62, 0x61, 0x69, 0x72, 0x74, 0xc0,
0x6d, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61,
0x69, 0x6e, 0x6e, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e,
0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e,
0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20,
0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x20, 0x61,
0x6e, 0x20, 0x45, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e,
0x6e, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e,
0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x20,
0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69,
0x6e, 0x6e, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e,
0x6e, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x47,
0x75, 0x61, 0x6d, 0xc0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x61,
0x6d, 0x61, 0x69, 0x73, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45,
0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x2019, 0x69, 0x20, 0x2019, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x61, 0x63, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61,
0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x2019, 0x69, 0x20, 0x2019, 0x73, 0x20, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x61, 0x63, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x6e,
0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x2019, 0x69, 0x20, 0x2019, 0x73, 0x20, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x61, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xc0, 0x6d, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x48, 0x6f, 0x76, 0x64, 0xc0, 0x6d,
0x20, 0x6e, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x6e, 0x73, 0x65, 0x61, 0x63,
0x68, 0x61, 0x6e, 0xc0, 0x6d, 0x20, 0x43, 0x75, 0x61, 0x6e, 0x20, 0x6e,
0x61, 0x6e, 0x20, 0x49, 0x6e, 0x6e, 0x73, 0x65, 0x61, 0x63, 0x68, 0x61,
0x6e, 0xc0, 0x6d, 0x20, 0x53, 0xec, 0x6e, 0x2d, 0x49, 0x6e, 0x6e, 0x73,
0x65, 0x61, 0x6e, 0x61, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61,
0x64, 0x68, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x6e,
0x64, 0x2d, 0x49, 0x6e, 0x6e, 0x73, 0x65, 0xc0, 0x6d, 0x20, 0x6e, 0x61,
0x6e, 0x20, 0x49, 0x6e, 0x6e, 0x64, 0x2d, 0x49, 0x6e, 0x6e, 0x73, 0x65,
0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x6e, 0x61,
0x6e, 0x20, 0x49, 0x6e, 0x6e, 0x64, 0x2d, 0x49, 0x6e, 0x6e, 0x73, 0x65,
0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x49, 0x6f,
0x72, 0xe0, 0x69, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x49,
0x6f, 0x72, 0xe0, 0x69, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x6f, 0x72, 0xe0,
0x69, 0x6e, 0xc0, 0x6d, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0xc0, 0x6d, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c, 0xc0, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x61, 0x70, 0x61, 0x69, 0x6e, 0x65,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65,
0x61, 0x70, 0x61, 0x69, 0x6e, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20,
0x53, 0x65, 0x61, 0x70, 0x61, 0x69, 0x6e, 0x65, 0xc0, 0x6d, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0xc0, 0x6d, 0x20,
0x43, 0x61, 0x73, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0xc0,
0x6d, 0x20, 0x43, 0x61, 0x73, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe0, 0x69,
0x6e, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x43,
0x61, 0x73, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x20, 0x61,
0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x72, 0xe8, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x43, 0x68,
0x6f, 0x69, 0x72, 0xe8, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x72, 0xe8, 0x61, 0xc0, 0x6d, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0xc0, 0x6d, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0xc0, 0x6d,
0x20, 0x43, 0xec, 0x6f, 0x72, 0x67, 0x61, 0x73, 0x74, 0x61, 0x69, 0x6e,
0xc0, 0x6d, 0x20, 0x4c, 0x61, 0x6e, 0x63, 0x61, 0xc0, 0x6d, 0x20, 0x45,
0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x54, 0x65, 0x72, 0x61,
0x69, 0x6e, 0x61, 0xc0, 0x6d, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0xc0, 0x6d, 0x20, 0x4d, 0x61,
0x63, 0xe0, 0x74, 0x68, 0x75, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x4d, 0x61, 0x63, 0xe0, 0x74, 0x68, 0x75, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x61,
0x63, 0xe0, 0x74, 0x68, 0x75, 0xc0, 0x6d, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0xc0, 0x6d, 0x20, 0x4d, 0x68, 0x61, 0x6c, 0x61, 0x69,
0x64, 0x68, 0x73, 0x65, 0x61, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20,
0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x61,
0x6c, 0x61, 0x64, 0x61, 0x69, 0x62, 0x68, 0xc0, 0x6d, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x2019, 0x20, 0x4d, 0x68,
0x61, 0x72, 0x63, 0x61, 0x69, 0x73, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x6f, 0x69, 0x72,
0x69, 0x73, 0x65, 0x61, 0x73, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x4d, 0x68, 0x6f, 0x69, 0x72, 0x69, 0x73, 0x65, 0x61, 0x73, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e,
0x61, 0x6e, 0x20, 0x4d, 0x68, 0x6f, 0x69, 0x72, 0x69, 0x73, 0x65, 0x61,
0x73, 0xc0, 0x6d, 0x20, 0x4d, 0x68, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0xc0,
0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20,
0x53, 0x68, 0xe8, 0x69, 0x6d, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x61, 0x67,
0x73, 0x61, 0x67, 0x6f, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x61,
0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x68, 0xe8,
0x69, 0x6d, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x61, 0x67, 0x73, 0x61, 0x67,
0x6f, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69,
0x6e, 0x20, 0x53, 0x68, 0xe8, 0x69, 0x6d, 0x68, 0x20, 0x4d, 0x68, 0x65,
0x61, 0x67, 0x73, 0x61, 0x67, 0x6f, 0xc0, 0x6d, 0x20, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0xc0, 0x6d, 0x20, 0x4d, 0x68, 0x6f, 0x73, 0x67, 0x6f, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x68, 0x6f, 0x73, 0x67, 0x6f, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x4d, 0x68, 0x6f, 0x73, 0x67, 0x6f, 0xc0, 0x6d, 0x20, 0x4d,
0x69, 0xe0, 0x6e, 0x6d, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x4e, 0x61, 0x62,
0x68, 0x72, 0x75, 0xc0, 0x6d, 0x20, 0x4e, 0x65, 0x61, 0x70, 0xe0, 0x6c,
0xc0, 0x6d, 0x20, 0x43, 0x68, 0x61, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x6e,
0x6e, 0x20, 0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x43, 0x68, 0x61, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x6e,
0x6e, 0x20, 0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43,
0x68, 0x61, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x6e, 0x6e, 0x20, 0x4e, 0x75,
0x61, 0x69, 0x64, 0x68, 0xc0, 0x6d, 0x20, 0x53, 0x68, 0x65, 0x61, 0x6c,
0x61, 0x69, 0x6e, 0x6e, 0x20, 0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x53, 0x68, 0x65, 0x61, 0x6c, 0x61,
0x69, 0x6e, 0x6e, 0x20, 0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x53, 0x68, 0x65, 0x61, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x20, 0x4e,
0x75, 0x61, 0x69, 0x64, 0x68, 0xc0, 0x6d, 0x20, 0x54, 0x61, 0x6c, 0x61,
0x6d, 0x68, 0x20, 0x61, 0x6e, 0x20, 0xc8, 0x69, 0x73, 0x67, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x54, 0x61, 0x6c, 0x61, 0x6d, 0x68, 0x20,
0x61, 0x6e, 0x20, 0xc8, 0x69, 0x73, 0x67, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54, 0x61,
0x6c, 0x61, 0x6d, 0x68, 0x20, 0x61, 0x6e, 0x20, 0xc8, 0x69, 0x73, 0x67,
0xc0, 0x6d, 0x20, 0x4e, 0x69, 0x75, 0x65, 0xc0, 0x6d, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x69,
0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x45, 0x69, 0x6c, 0x65, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0xc0, 0x6d, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61,
0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x69, 0x72, 0x69, 0x61, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x61, 0x20, 0x54, 0x75, 0x61, 0x74, 0x68, 0xc0, 0x6d,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xc0, 0x6d, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0xc0, 0x6d, 0x20,
0x50, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x50, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x69,
0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x50, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x69,
0x6e, 0xc0, 0x6d, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x62, 0x68, 0xc0, 0x6d,
0x20, 0x47, 0x69, 0x6e, 0x69, 0x20, 0x4e, 0x75, 0x61, 0x69, 0x64, 0x68,
0x20, 0x50, 0x61, 0x70, 0x75, 0x74, 0x68, 0x61, 0x69, 0x63, 0x68, 0x65,
0xc0, 0x6d, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x64,
0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x69, 0x64, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x69, 0x64, 0x68, 0xc0, 0x6d, 0x20, 0x50, 0x65,
0x61, 0x72, 0xf9, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x50, 0x65,
0x61, 0x72, 0xf9, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x65, 0x61, 0x72, 0xf9, 0xc0,
0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x61, 0x63, 0x68,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45,
0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x65, 0x61, 0x63, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x6e, 0x20,
0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x65, 0x61, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69,
0x78, 0xc0, 0x6d, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x61, 0x67, 0x75, 0x73, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x61, 0x67, 0x75, 0x73, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x61, 0x67, 0x75, 0x73, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0, 0x6d, 0x20, 0x50,
0x65, 0x69, 0x74, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0xe0, 0x69, 0x72,
0x6e, 0xc0, 0x6d, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0xc0,
0x6d, 0x20, 0x50, 0x79, 0x65, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67,
0xc0, 0x6d, 0x20, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x51, 0x131, 0x7a, 0x131, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x51, 0x131, 0x7a, 0x131, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0xc0, 0x6d, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69,
0x6f, 0x6e, 0xc0, 0x6d, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61,
0xc0, 0x6d, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0xc0, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0xc0, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x74, 0x68, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x74, 0x68, 0x61, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x74, 0x68, 0x61, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e,
0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x53, 0x68,
0x65, 0x69, 0x73, 0x65, 0x61, 0x6c, 0x6c, 0xc0, 0x6d, 0x20, 0x53, 0x69,
0x6e, 0x67, 0x65, 0x61, 0x70, 0xf2, 0x72, 0xc0, 0x6d, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x53, 0x68, 0x6f, 0x6c, 0x61,
0x69, 0x6d, 0x68, 0xc0, 0x6d, 0x20, 0x53, 0x65, 0xf2, 0x72, 0x73, 0x65,
0x61, 0x20, 0x61, 0x20, 0x44, 0x65, 0x61, 0x73, 0xc0, 0x6d, 0x20, 0x53,
0x75, 0x72, 0x61, 0x6e, 0x61, 0x69, 0x6d, 0xc0, 0x6d, 0x20, 0x53, 0x68,
0x79, 0x6f, 0x77, 0x61, 0xc0, 0x6d, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0xc0, 0x6d, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0xc0, 0x6d, 0x20, 0x54,
0x61, 0x69, 0x64, 0x69, 0x67, 0x65, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e,
0xc0, 0x6d, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0xc0, 0x6d,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0xc0, 0x6d, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0xc0, 0x6d,
0x20, 0x54, 0x75, 0x72, 0x63, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe0,
0x69, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x54, 0x75, 0x72,
0x63, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x54, 0x75, 0x72, 0x63, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe0,
0x69, 0x6e, 0xc0, 0x6d, 0x20, 0x54, 0x75, 0x62, 0x68, 0x61, 0x6c, 0x75,
0xc0, 0x6d, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x64, 0x68,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x69, 0x64, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x69, 0x64, 0x68, 0xc0, 0x6d, 0x20, 0x55, 0x73, 0x62, 0x61, 0x67,
0x61, 0x73, 0x74, 0xe0, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x55, 0x73, 0x62, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x6e, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x55, 0x73, 0x62, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x6e, 0xc0,
0x6d, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0xc0, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x42, 0x68, 0x65, 0x69, 0x6e, 0x69, 0x73, 0x65, 0x61,
0x6c, 0x61, 0xc0, 0x6d, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x54, 0xec,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0xc0, 0x6d, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0xc0, 0x6d, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x57,
0x61, 0x6b, 0x65, 0xc0, 0x6d, 0x20, 0x55, 0x61, 0x6c, 0x6c, 0x61, 0x73,
0x20, 0x61, 0x67, 0x75, 0x73, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0xc0, 0x6d, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xc0, 0x6d,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0xc0, 0x6d, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x65, 0x14b, 0x67, 0x62, 0x25b, 0x20,
0x42, 0x65, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x6f, 0x6b,
0x61, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x53, 0x6f, 0x75, 0x74, 0x68,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x65, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b,
0x20, 0x42, 0x65, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x41, 0x6e,
0x61, 0x69, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20,
0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b, 0x20,
0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20,
0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20,
0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65,
0x69, 0x61, 0x14b, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x42, 0x65,
0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65,
0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x54,
0x65, 0x14b, 0x67, 0x62, 0x25b, 0x62, 0x69, 0x69, 0x20, 0x41, 0x62, 0x65,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x65, 0x14b, 0x67,
0x62, 0x25b, 0x62, 0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x20, 0x59, 0x25b,
0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x65, 0x14b, 0x67, 0x62, 0x25b,
0x62, 0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c,
0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62,
0x25b, 0x62, 0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x62,
0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b,
0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x62, 0x69,
0x69, 0x20, 0x41, 0x62, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74,
0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x47, 0x254, 0x6a, 0x69, 0x61, 0x6e, 0x254, 0x62,
0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x47, 0x254, 0x6a, 0x69, 0x61, 0x6e, 0x254, 0x62, 0x69, 0x69,
0x20, 0x41, 0x62, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20,
0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x47, 0x254, 0x6a, 0x69, 0x61, 0x6e, 0x254, 0x62, 0x69, 0x69, 0x20,
0x41, 0x62, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20,
0x42, 0x65, 0x69, 0x61, 0x14b, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74,
0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4b, 0x61, 0x70, 0x65, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x20, 0x42, 0x65, 0x4b, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20,
0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4b, 0x61, 0x70,
0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x42, 0x65, 0x20, 0x59,
0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61,
0x14b, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20,
0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4b, 0x75, 0x62,
0x61, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73,
0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6e, 0x74, 0x61, 0x72,
0x6b, 0x74, 0x69, 0x6b, 0x20, 0x4b, 0x25b, 0x20, 0x57, 0x75, 0x6f, 0x79,
0x69, 0x67, 0x62, 0x25b, 0x62, 0x69, 0x69, 0x20, 0x4e, 0x69, 0x20, 0x57,
0x69, 0x65, 0x254, 0x20, 0x46, 0x72, 0x25b, 0x6e, 0x74, 0x73, 0x69, 0x20,
0x42, 0x65, 0x42, 0x65, 0x74, 0x73, 0x254, 0x254, 0x6d, 0x254, 0x20, 0x6e,
0x69, 0x20, 0x6b, 0x61, 0x20, 0x14b, 0x25b, 0x6c, 0x25b, 0x20, 0x6b, 0x6f,
0x6d, 0x65, 0x20, 0x6e, 0x254, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x20, 0x42,
0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42,
0x6f, 0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b,
0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x6f, 0x6b, 0xe3,
0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61,
0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x6e, 0x61, 0x69, 0x67,
0x62, 0x25b, 0x20, 0x42, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x41, 0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b, 0x20, 0x42,
0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69,
0x61, 0x14b, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x41, 0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x20, 0x59,
0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61,
0x14b, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x20, 0x42, 0x65, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x42, 0x65, 0x20,
0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74,
0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x20, 0x14a, 0x73, 0x68, 0x254, 0x20, 0x4c, 0x25b, 0x20, 0x42,
0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x42,
0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x42,
0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69,
0x61, 0x14b, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20,
0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20,
0x42, 0x65, 0x69, 0x61, 0x14b, 0x4d, 0x65, 0x7a, 0x69, 0x6b, 0x6f, 0x20,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x4d, 0x65,
0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x4d, 0x65, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b,
0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c,
0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x53, 0x74,
0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x4b, 0x25b, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x53, 0x74,
0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x4b, 0x25b, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x20, 0x59,
0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x4b, 0x25b,
0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x20,
0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69,
0x61, 0x14b, 0x53, 0x65, 0x79, 0x73, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x42, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x63, 0x72, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4d, 0x65, 0x72,
0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2c, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2c, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2c, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6c, 0x65, 0x73, 0x74, 0x65, 0x2c,
0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x6c, 0x65,
0x73, 0x74, 0x65, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x6f, 0x20, 0x6c, 0x65, 0x73, 0x74, 0x65, 0x2c, 0x20, 0x41, 0x6d, 0xe9,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x6d, 0x6f, 0x6e,
0x74, 0x61, 0xf1, 0x61, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x2c, 0x20,
0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x6d, 0x6f, 0x6e,
0x74, 0x61, 0xf1, 0x61, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x2c, 0x20, 0x41,
0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed,
0x66, 0x69, 0x63, 0x6f, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x69, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e,
0x61, 0x64, 0x69, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0xe1, 0x72, 0x61, 0x62, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0xe1, 0x72,
0x61, 0x62, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0xe1, 0x72, 0x61, 0x62, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x78, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72,
0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63,
0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x63, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x41, 0x63, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x63, 0x65, 0x72, 0x62, 0x61,
0x69, 0x78, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x73,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x73,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x73,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x42, 0x75, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75,
0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x63, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c,
0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61,
0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x6d, 0x65, 0x64, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20,
0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x43, 0x75, 0x62, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x75, 0x61,
0x64, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x65, 0x78, 0x74, 0x72, 0x65, 0x6d, 0x6f, 0x20, 0x6c, 0x65,
0x73, 0x74, 0x65, 0x20, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69,
0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x78, 0x69, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x78, 0x69, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x46, 0x69, 0x78, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x47, 0xfc, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x63, 0x65, 0x73, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x69, 0x73, 0x20, 0x65, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72,
0x74, 0x69, 0x63, 0x61, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65,
0x73, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20,
0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f, 0x72,
0x78, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f, 0x72,
0x78, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f, 0x72,
0x78, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20,
0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x72,
0x69, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x6f, 0x6c,
0x66, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x2d, 0x69, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x69, 0x6c, 0x6c, 0x61, 0x73,
0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x69, 0x6c,
0x6c, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x4f, 0x63, 0xe9, 0x61, 0x6e, 0x6f, 0x20, 0xcd, 0x6e, 0x64,
0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x72, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x58, 0x61, 0x70, 0xf3,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x58, 0x61, 0x70, 0xf3, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x6f, 0x20, 0x58, 0x61, 0x70, 0xf3, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0xe1, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f,
0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0xe1, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72,
0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69,
0x7a, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x64, 0x61, 0x20,
0x4c, 0x69, 0xf1, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x6c, 0x61, 0x69, 0x73, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73,
0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61,
0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x6e, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed,
0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e,
0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x71,
0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0xfa, 0x61,
0x2d, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x65, 0x72, 0xfa, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0xfa,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20,
0x46, 0xe9, 0x6e, 0x69, 0x78, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x65, 0x74, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x74, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x65, 0x74, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0xf3, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0xed, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0xed, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0xed, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x72, 0x69, 0x6f, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x53,
0x61, 0x6c, 0x6f, 0x6d, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x69, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x53, 0x75, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0xed, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x78, 0x69, 0x71,
0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65,
0x6c, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c,
0x69, 0x73, 0x20, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d,
0x62, 0x75, 0x72, 0x67, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x10d0, 0x10d5, 0x10e6, 0x10d0, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20,
0x10d0, 0x10e4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0,
0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4,
0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10db,
0x10ee, 0x10e0, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7,
0x20, 0x10d0, 0x10e4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4, 0x10e0,
0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d0, 0x10da, 0x10d0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10da, 0x10d0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0,
0x10da, 0x10d0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10db, 0x10d0, 0x10d6, 0x10dd,
0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10db, 0x10d0, 0x10d6,
0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10db, 0x10d0,
0x10d6, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da,
0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20,
0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc,
0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9,
0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0,
0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da,
0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7,
0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d0, 0x10e6,
0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0,
0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d0, 0x10e6, 0x10db, 0x10dd,
0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4,
0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da,
0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10db, 0x10d0, 0x10e3, 0x10dc, 0x10d7, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0,
0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10db, 0x10d0, 0x10e3, 0x10dc,
0x10d7, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3,
0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9,
0x10d8, 0x10e1, 0x20, 0x10db, 0x10d0, 0x10e3, 0x10dc, 0x10d7, 0x10d8, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db,
0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0,
0x10d8, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db,
0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0,
0x10d8, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db,
0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0,
0x10d8, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10de,
0x10d8, 0x10d0, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10de, 0x10d8, 0x10d0, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10de, 0x10d8, 0x10d0, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0,
0x10d0, 0x10d1, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0,
0x10d0, 0x10d1, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0,
0x10d0, 0x10d1, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc,
0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d2,
0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0,
0x10e0, 0x10d2, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0,
0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc,
0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1,
0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc, 0x10e2,
0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc, 0x10e2, 0x10d8,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10dd, 0x10db, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10dd, 0x10db, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10dd, 0x10db, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d0, 0x10e2, 0x10da, 0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0,
0x10e2, 0x10da, 0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10dd, 0x10d9,
0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e2, 0x10da,
0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3,
0x10e0, 0x10d8, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3,
0x10e0, 0x10d8, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0,
0x10da, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da,
0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3,
0x10e0, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da,
0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0,
0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6, 0x10d4, 0x10e0, 0x10d1, 0x10d0, 0x10d8, 0x10ef,
0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6, 0x10d4, 0x10e0,
0x10d1, 0x10d0, 0x10d8, 0x10ef, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10d6, 0x10d4, 0x10e0, 0x10d1, 0x10d0, 0x10d8, 0x10ef, 0x10d0, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d0, 0x10d6, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb,
0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6,
0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4,
0x10d1, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6, 0x10dd, 0x10e0, 0x10d8,
0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10da, 0x10d0, 0x10d3, 0x10d4, 0x10e8, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10da, 0x10d0, 0x10d3, 0x10d4,
0x10e8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10da,
0x10d0, 0x10d3, 0x10d4, 0x10e8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e3, 0x10e2, 0x10d0, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10dd, 0x10da, 0x10d8, 0x10d5, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10d0, 0x10d6, 0x10d8, 0x10da,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10d0, 0x10d6, 0x10d8,
0x10da, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10d0, 0x10d6,
0x10d8, 0x10da, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10e3, 0x10dc, 0x10d4, 0x10d8,
0x2d, 0x10d3, 0x10d0, 0x10e0, 0x10e3, 0x10e1, 0x10d0, 0x10da, 0x10d0, 0x10db, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10d0, 0x10d1, 0x10dd, 0x2d, 0x10d5, 0x10d4, 0x10e0,
0x10d3, 0x10d4, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10d0, 0x10d1, 0x10dd, 0x2d,
0x10d5, 0x10d4, 0x10e0, 0x10d3, 0x10d4, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10d0,
0x10d1, 0x10dd, 0x2d, 0x10d5, 0x10d4, 0x10e0, 0x10d3, 0x10d4, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0,
0x10db, 0x10dd, 0x10e0, 0x10dd, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0, 0x10e2,
0x10d4, 0x10db, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0, 0x10e2, 0x10d4,
0x10db, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0, 0x10e2, 0x10d4, 0x10db,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d8, 0x10da, 0x10d4, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e9, 0x10d8, 0x10da, 0x10d4, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d8, 0x10da,
0x10d4, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d8, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e9, 0x10d8, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e9, 0x10d8, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10d0, 0x10e0, 0x10e2, 0x10e7, 0x10d4, 0x10da, 0x10d8, 0x10e8,
0x10dd, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e5, 0x10dd, 0x10e5, 0x10dd, 0x10e1, 0x10d8, 0x10e1,
0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10da, 0x10e3, 0x10db, 0x10d1, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10da, 0x10e3, 0x10db, 0x10d1, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10da, 0x10e3, 0x10db, 0x10d1, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc,
0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9,
0x10e3, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4,
0x10d1, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20,
0x10dc, 0x10d0, 0x10ee, 0x10d4, 0x10d5, 0x10e0, 0x10d0, 0x10d3, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d1,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d1, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d4, 0x10d5,
0x10d8, 0x10e1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10e3, 0x10db, 0x10dd,
0x10dc, 0x2d, 0x10d3, 0x10e3, 0x10e0, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7,
0x20, 0x10e2, 0x10d8, 0x10db, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10e6, 0x10d3, 0x10d2, 0x10dd, 0x10db, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc,
0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10d3,
0x10d2, 0x10dd, 0x10db, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10d3, 0x10d2, 0x10dd, 0x10db,
0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d4, 0x10d9, 0x10d5, 0x10d0, 0x10d3, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20,
0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea,
0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d4, 0x10d5,
0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc,
0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd,
0x10de, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7,
0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4,
0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e8, 0x10dd, 0x10e0, 0x10d4, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4,
0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5,
0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1,
0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4,
0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10dd,
0x10da, 0x10d9, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc,
0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4,
0x10dd, 0x10da, 0x10d9, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3,
0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8,
0x10ef, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10ef, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10ef, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0,
0x10e4, 0x10e0, 0x10d0, 0x10dc, 0x10d2, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d2, 0x10d5,
0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10e0, 0x10d0,
0x10dc, 0x10d2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10e1, 0x10d0, 0x10db, 0x10ee, 0x10e0, 0x10d4,
0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d0, 0x10dc, 0x10e2, 0x10d0, 0x10e0,
0x10e5, 0x10e2, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d2, 0x10d0,
0x10da, 0x10d0, 0x10de, 0x10d0, 0x10d2, 0x10dd, 0x10e1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d2, 0x10d0, 0x10db, 0x10d1, 0x10d8, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10e5, 0x10d0, 0x10e0, 0x10d7, 0x10d5, 0x10d4, 0x10da, 0x10dd,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10e5, 0x10d0, 0x10e0, 0x10d7, 0x10d5,
0x10d4, 0x10da, 0x10dd, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10e5, 0x10d0,
0x10e0, 0x10d7, 0x10d5, 0x10d4, 0x10da, 0x10dd, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d2, 0x10d8, 0x10da, 0x10d1,
0x10d4, 0x10e0, 0x10e2, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da,
0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d2, 0x10e0, 0x10d8, 0x10dc,
0x10d5, 0x10d8, 0x10e9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10d0, 0x10e8, 0x10e3, 0x10d0, 0x10da,
0x10dd, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1,
0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0,
0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db,
0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc,
0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc,
0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc,
0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc,
0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10de, 0x10d0, 0x10e0, 0x10e1, 0x10d4, 0x10d7,
0x10d8, 0x10e1, 0x20, 0x10e7, 0x10e3, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d2, 0x10d0, 0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0,
0x10d0, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d0, 0x10da,
0x10d4, 0x10e3, 0x10e2, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10d0, 0x10d5,
0x10d0, 0x10d8, 0x10e1, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d0, 0x10da, 0x10d4, 0x10e3,
0x10e2, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10d0, 0x10d5, 0x10d0, 0x10d8,
0x10e1, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d0, 0x10da, 0x10d4, 0x10e3, 0x10e2, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10dc, 0x10d9, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10dc, 0x10d9, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10dc, 0x10d9, 0x10dd, 0x10dc, 0x10d2, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10d5, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10f0, 0x10dd, 0x10d5, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10dd,
0x10d5, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10d4, 0x10d7, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10d4, 0x10d7, 0x10d8,
0x10e1, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3,
0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10e9, 0x10d8, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0,
0x10d8, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10dc, 0x10d4, 0x10d6, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10dc, 0x10d4, 0x10d6, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4,
0x10d7, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10dc, 0x10d4, 0x10d6, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10d0, 0x10e0, 0x10e2, 0x10e7, 0x10d4, 0x10da, 0x10d8, 0x10d8, 0x10e0, 0x10d9,
0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e0,
0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8,
0x10e0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e1, 0x10e0,
0x10d0, 0x10d4, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e1, 0x10e0,
0x10d0, 0x10d4, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e1, 0x10e0,
0x10d0, 0x10d4, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10de, 0x10dd, 0x10dc, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10de, 0x10dd, 0x10dc, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10de, 0x10dd, 0x10dc, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e7, 0x10d0, 0x10d6, 0x10d0, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10e7, 0x10d0, 0x10d6, 0x10d0, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7,
0x20, 0x10e7, 0x10d0, 0x10d6, 0x10d0, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10e0, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d9, 0x10dd, 0x10e0, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd,
0x10e0, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10e1, 0x10e0, 0x10d4, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e0, 0x10d0, 0x10e1, 0x10dc, 0x10dd, 0x10d8, 0x10d0, 0x10e0,
0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e0, 0x10d0, 0x10e1,
0x10dc, 0x10dd, 0x10d8, 0x10d0, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d9, 0x10e0, 0x10d0, 0x10e1, 0x10dc, 0x10dd, 0x10d8, 0x10d0, 0x10e0, 0x10e1, 0x10d9,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e7, 0x10d8, 0x10e0, 0x10d2, 0x10d8, 0x10d6, 0x10d4, 0x10d7, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10da, 0x10d0, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20,
0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10da, 0x10dd, 0x10e0, 0x10d3, 0x2d, 0x10f0, 0x10d0, 0x10e3, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10da, 0x10dd, 0x10e0, 0x10d3, 0x2d, 0x10f0, 0x10d0, 0x10e3, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10da, 0x10dd, 0x10e0, 0x10d3, 0x2d, 0x10f0, 0x10d0, 0x10e3,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10d0, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10d0, 0x10dc, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10da, 0x10d0, 0x10d8, 0x10d6, 0x10d8, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10da, 0x10d3, 0x10d8, 0x10d5, 0x10d4, 0x10d1, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10e0, 0x10d9, 0x10d8, 0x10d6, 0x10d8,
0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10e0, 0x10e8, 0x10d0, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d5, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d5, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d5, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10db, 0x10dd, 0x10e3, 0x10e1, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10db, 0x10d4, 0x10e5, 0x10e1, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc,
0x10d0, 0x10e0, 0x10d8, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d4, 0x10e5, 0x10e1, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d4, 0x10e5, 0x10e1, 0x10d8, 0x10d9,
0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x20, 0x10dd, 0x10d9,
0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10da, 0x10d0, 0x10dc, 0x2d, 0x10d1,
0x10d0, 0x10e2, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10da,
0x10d0, 0x10dc, 0x2d, 0x10d1, 0x10d0, 0x10e2, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e3, 0x10da, 0x10d0, 0x10dc, 0x2d, 0x10d1, 0x10d0, 0x10e2, 0x10dd, 0x10e0,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd, 0x10d5, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd, 0x10d5, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd, 0x10d5, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10db, 0x10d8, 0x10d0, 0x10dc, 0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10dc, 0x10d0, 0x10e3, 0x10e0, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc,
0x10d4, 0x10de, 0x10d0, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10ee,
0x10d0, 0x10da, 0x10d8, 0x20, 0x10d9, 0x10d0, 0x10da, 0x10d4, 0x10d3, 0x10dd, 0x10dc, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20,
0x10d9, 0x10d0, 0x10da, 0x10d4, 0x10d3, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20, 0x10d9, 0x10d0, 0x10da, 0x10d4,
0x10d3, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8,
0x20, 0x10d6, 0x10d4, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20, 0x10d6, 0x10d4, 0x10da, 0x10d0,
0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0,
0x10da, 0x10d8, 0x20, 0x10d6, 0x10d4, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10dc, 0x10d8, 0x10e3, 0x10e4, 0x10d0, 0x10e3, 0x10dc, 0x10d3, 0x10da, 0x10d4, 0x10dc,
0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10e3, 0x10e4, 0x10d0,
0x10e3, 0x10dc, 0x10d3, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10dc, 0x10d8, 0x10e3, 0x10e4, 0x10d0, 0x10e3, 0x10dc, 0x10d3, 0x10da, 0x10d4, 0x10dc,
0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10e3, 0x10d4, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10dc, 0x10dd, 0x10e0, 0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc,
0x10dd, 0x10e0, 0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc,
0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10dd, 0x10e0,
0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d4, 0x10e0, 0x10dc, 0x10d0, 0x10dc, 0x10d3, 0x10dd,
0x2d, 0x10d3, 0x10d4, 0x2d, 0x10dc, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d4, 0x10e0, 0x10dc, 0x10d0, 0x10dc, 0x10d3, 0x10dd,
0x2d, 0x10d3, 0x10d4, 0x2d, 0x10dc, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d4, 0x10e0, 0x10dc, 0x10d0, 0x10dc, 0x10d3, 0x10dd,
0x2d, 0x10d3, 0x10d4, 0x2d, 0x10dc, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10dc, 0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d8, 0x10e0, 0x10e1, 0x10d9,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d8,
0x10d1, 0x10d8, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc,
0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d8, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10dd, 0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dd,
0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dd, 0x10db, 0x10e1,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10d9, 0x10d8, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10d9, 0x10d8, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10d9, 0x10d8,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10da, 0x10d0, 0x10e3,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10de, 0x10e3, 0x10d0, 0x2d, 0x10d0,
0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20, 0x10d2, 0x10d5, 0x10d8, 0x10dc, 0x10d4, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10e0, 0x10d0, 0x10d2, 0x10d5, 0x10d0, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10e0, 0x10d0, 0x10d2, 0x10d5, 0x10d0,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10e0, 0x10d0, 0x10d2, 0x10d5,
0x10d0, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d4, 0x10e0, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10de, 0x10d4, 0x10e0, 0x10e3, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d4,
0x10e0, 0x10e3, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10da, 0x10d8, 0x10de, 0x10d8, 0x10dc, 0x10d4,
0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10da, 0x10d8, 0x10de,
0x10d8, 0x10dc, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8,
0x10da, 0x10d8, 0x10de, 0x10d8, 0x10dc, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d4,
0x10dc, 0x10d8, 0x10e5, 0x10e1, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3,
0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d4, 0x10dc,
0x2d, 0x10de, 0x10d8, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10d0, 0x20, 0x10db,
0x10d8, 0x10d9, 0x10d4, 0x10da, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10de, 0x10d8, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10d0, 0x20, 0x10db, 0x10d8, 0x10d9, 0x10d4, 0x10da, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10de, 0x10d8, 0x10d4, 0x10e0, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10d0, 0x20, 0x10db, 0x10d8, 0x10d9, 0x10d4, 0x10da, 0x10dd, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e2, 0x10d9, 0x10d4, 0x10e0, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10dd, 0x10dc, 0x10d0, 0x10de, 0x10d4, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10ee, 0x10d4, 0x10dc, 0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e0, 0x10d4, 0x10d8, 0x10e3, 0x10dc, 0x10d8, 0x10dd, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e0, 0x10dd, 0x10d7, 0x10d4, 0x10e0, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10ee, 0x10d0, 0x10da,
0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10db, 0x10dd, 0x10d0, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10db, 0x10dd, 0x10d0, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e1, 0x10d0, 0x10db, 0x10dd, 0x10d0, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d4, 0x10d8, 0x10e8,
0x10d4, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4,
0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d8, 0x10dc, 0x10d2, 0x10d0,
0x10de, 0x10e3, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10dd, 0x10da,
0x10dd, 0x10db, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3,
0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10db,
0x10ee, 0x10e0, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10d4, 0x10dd, 0x10e0, 0x10d2, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10e3, 0x10e0, 0x10d8, 0x10dc, 0x10d0, 0x10db,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d8, 0x10dd, 0x10d5, 0x10d0, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10d0, 0x10d8, 0x10e2, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e2, 0x10d0, 0x10d8, 0x10d1, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e2, 0x10d0, 0x10d8, 0x10d1, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e2, 0x10d0, 0x10d8, 0x10d1, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10d0, 0x10ef, 0x10d8,
0x10d9, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10dd, 0x10d9,
0x10d4, 0x10da, 0x10d0, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10dd, 0x10dc,
0x10d2, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e9, 0x10e3, 0x10e3, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d7, 0x10e3,
0x10e0, 0x10e5, 0x10db, 0x10d4, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d7, 0x10e3, 0x10e0, 0x10e5, 0x10db, 0x10d4, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d7, 0x10e3, 0x10e0, 0x10e5, 0x10db, 0x10d4, 0x10dc, 0x10d4,
0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10e3, 0x10d5, 0x10d0, 0x10da, 0x10e3, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10e0, 0x10e3, 0x10d2, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10e0, 0x10e3, 0x10d2, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10e0, 0x10e3, 0x10d2, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e3, 0x10d6, 0x10d1, 0x10d4, 0x10d9, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e3, 0x10d6, 0x10d1, 0x10d4, 0x10d9, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e3, 0x10d6, 0x10d1, 0x10d4, 0x10d9, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d5, 0x10d0, 0x10dc, 0x10e3, 0x10d0, 0x10e2, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d5, 0x10d0, 0x10dc, 0x10e3, 0x10d0, 0x10e2, 0x10e3, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d5, 0x10d0, 0x10dc, 0x10e3, 0x10d0, 0x10e2, 0x10e3, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10d4, 0x10dc,
0x10d4, 0x10e1, 0x10e3, 0x10d4, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5,
0x10da, 0x10d0, 0x10d3, 0x10d8, 0x10d5, 0x10dd, 0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10da, 0x10d0, 0x10d3, 0x10d8, 0x10d5, 0x10dd, 0x10e1,
0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10da, 0x10d0,
0x10d3, 0x10d8, 0x10d5, 0x10dd, 0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5,
0x10dd, 0x10da, 0x10d2, 0x10dd, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d5, 0x10dd, 0x10da, 0x10d2, 0x10dd, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10dd, 0x10da, 0x10d2, 0x10dd, 0x10d2, 0x10e0,
0x10d0, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10d4, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d5, 0x10dd, 0x10da, 0x10d8, 0x10e1, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x20, 0x10e4, 0x10e3,
0x10e2, 0x10e3, 0x10dc, 0x10d0, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10d9,
0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0,
0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8,
0x10d0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d4, 0x10d9, 0x10d0,
0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10dc, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d4, 0x10d9, 0x10d0, 0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10dc, 0x10d1,
0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d4, 0x10d9, 0x10d0,
0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10dc, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d8, 0x10e3, 0x10d9, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x41,
0x63, 0x72, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x63, 0x72, 0x65,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41,
0x63, 0x72, 0x65, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65,
0x69, 0x74, 0x53, 0xfc, 0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65,
0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65,
0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x61, 0x73, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4f, 0x73, 0x74,
0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x4f, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65,
0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4f, 0x73, 0x74, 0x6b, 0xfc, 0x73,
0x74, 0x65, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x73, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x52, 0x6f,
0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e,
0x73, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61,
0x69, 0x6e, 0x73, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x57, 0x65, 0x73, 0x74,
0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74,
0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b,
0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x5a,
0x65, 0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x70,
0x69, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x71,
0x74, 0xf6, 0x62, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x71, 0x74,
0xf6, 0x62, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x61, 0x62, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x61,
0x62, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73,
0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x2f, 0x57, 0x65,
0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a,
0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72, 0x62,
0x61, 0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72, 0x62, 0x65,
0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41,
0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x63, 0x68, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x63, 0x68, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c,
0x69, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x42, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x2d, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x61, 0x62, 0x6f, 0x2d,
0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x61,
0x62, 0x6f, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x61, 0x62, 0x6f, 0x2d,
0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x43, 0x61, 0x73, 0x65, 0x79, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65,
0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x57, 0x65, 0x69, 0x68, 0x6e, 0x61, 0x63, 0x68, 0x74,
0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x75, 0x62,
0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x4b, 0x75, 0x62, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x75, 0x62,
0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x44, 0x61, 0x76, 0x69, 0x73, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x4f, 0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x73, 0x65, 0x6c, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x45,
0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65,
0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x69, 0x74,
0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x4f, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x61,
0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x65, 0x72, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73, 0x65,
0x6c, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x46, 0x69, 0x64, 0x73, 0x63,
0x68, 0x69, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x46, 0x72, 0x61, 0x6e, 0x7a, 0xf6, 0x73, 0x69, 0x73, 0x63, 0x68,
0x2d, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x46, 0x72, 0x61, 0x6e, 0x7a, 0xf6, 0x73, 0x69, 0x73, 0x63, 0x68,
0x65, 0x2d, 0x53, 0xfc, 0x64, 0x2d, 0x75, 0x6e, 0x64, 0x2d, 0x41, 0x6e,
0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x67, 0x65, 0x62, 0x69, 0x65,
0x74, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x2d, 0x49, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x4d, 0x69, 0x74, 0x74, 0x6c, 0x65, 0x72, 0x65, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x47, 0x75, 0x61, 0x6d, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x47, 0x6f, 0x6c, 0x66, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f,
0x6e, 0x67, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x6f, 0x77,
0x64, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x68,
0x6f, 0x77, 0x64, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x6e, 0x64,
0x69, 0x73, 0x63, 0x68, 0x65, 0x72, 0x2d, 0x4f, 0x7a, 0x65, 0x61, 0x6e,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69,
0x6e, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a,
0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x49, 0x72, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65,
0x69, 0x74, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x72,
0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65,
0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a,
0x65, 0x69, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x61, 0x6d,
0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4b,
0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x61, 0x73,
0x61, 0x63, 0x68, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x4f, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x63, 0x68, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74,
0x6b, 0x61, 0x73, 0x61, 0x63, 0x68, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x69, 0x72,
0x67, 0x69, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x53, 0x72, 0x69, 0x2d, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x4c, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x69, 0x6e, 0x73,
0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4c, 0x6f,
0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48,
0x6f, 0x77, 0x65, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x4d, 0x61, 0x63, 0x61, 0x75, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x2d, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x6c, 0x61,
0x79, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b,
0x7a, 0x65, 0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4d,
0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75,
0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61,
0x75, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x4e, 0x65, 0x75, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x65, 0x75, 0x6b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x65,
0x75, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x4e, 0x65, 0x75, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x4e, 0x65, 0x75, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x4e, 0x65, 0x75, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x65, 0x75,
0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x4e, 0x65, 0x75, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e,
0x65, 0x75, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x69, 0x75,
0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d,
0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0xf6, 0x72,
0x64, 0x6c, 0x69, 0x63, 0x68, 0x65, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x61,
0x6e, 0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x77, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65,
0x69, 0x74, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x65, 0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x72,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e, 0x65, 0x75,
0x67, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x50, 0x65, 0x72,
0x75, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x69, 0x6e, 0x73,
0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x74, 0x2e, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x75, 0x6e, 0x64, 0x2d, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x53, 0x74, 0x2e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x75,
0x6e, 0x64, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x74,
0x2e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x75, 0x6e, 0x64,
0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x50, 0x6a, 0xf6, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x51, 0x75, 0x79, 0x73, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x51, 0x75, 0x79, 0x73, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x51, 0x79, 0x73, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x61, 0x63, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x75, 0x72, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53,
0xfc, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65,
0x68, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x54, 0x61, 0x64, 0x73, 0x63, 0x68, 0x69, 0x6b, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x55, 0x73, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57,
0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x72,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x57,
0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x72,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x57,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x72, 0x20, 0x5a,
0x65, 0x69, 0x74, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x72,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x57,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x61,
0x6b, 0x65, 0x2d, 0x49, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x75, 0x6e, 0x64, 0x2d,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x65, 0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x69, 0x6e, 0x73,
0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x38f, 0x3c1, 0x3b1, 0x20,
0x391, 0x3c6, 0x3b3, 0x3b1, 0x3bd, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391,
0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c6,
0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3cc, 0x3c4, 0x3b9, 0x3b1,
0x3c2, 0x20, 0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c6, 0x3c1,
0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bb, 0x3ac, 0x3c3, 0x3ba, 0x3b1, 0x3c2, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3bb, 0x3ac, 0x3c3, 0x3ba, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bb, 0x3ac, 0x3c3, 0x3ba, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bc, 0x3b1, 0x3b6, 0x3bf, 0x3bd, 0x3af,
0x3bf, 0x3c5, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bc, 0x3b1, 0x3b6, 0x3bf, 0x3bd, 0x3af, 0x3bf,
0x3c5, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3bc, 0x3b1, 0x3b6, 0x3bf, 0x3bd, 0x3af, 0x3bf, 0x3c5, 0x39a, 0x3b5, 0x3bd,
0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc,
0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3c7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba,
0x3ae, 0x20, 0x3b8, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9,
0x3ba, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9,
0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x391,
0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3c7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392,
0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae,
0x20, 0x3b8, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x39f, 0x3c1, 0x3b5, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x39f, 0x3c1, 0x3b5, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3c7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20,
0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x39f, 0x3c1, 0x3b5, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3b8, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9,
0x3c1, 0x3b7, 0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1,
0x3b7, 0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1, 0x3b7, 0x3bd, 0x3b9, 0x3ba,
0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3bd, 0x3c4, 0x3af,
0x3c1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3bd, 0x3c4, 0x3af, 0x3c1, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1,
0x3bd, 0x3c4, 0x3af, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c0, 0x3af, 0x3b1,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x391, 0x3c0, 0x3af, 0x3b1, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c0, 0x3af, 0x3b1, 0x391, 0x3c1, 0x3b1,
0x3b2, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x391, 0x3c1, 0x3b1, 0x3b2,
0x3b9, 0x3ba, 0x3ae, 0x20, 0x3c7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x391, 0x3c1, 0x3b1, 0x3b2, 0x3b9, 0x3ba, 0x3ae,
0x20, 0x3b8, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae,
0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x391, 0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c1,
0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5,
0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4,
0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391,
0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x391, 0x3c1, 0x3bc, 0x3b5, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391,
0x3c1, 0x3bc, 0x3b5, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c1, 0x3bc, 0x3b5, 0x3bd, 0x3af,
0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c4, 0x3bb, 0x3b1, 0x3bd, 0x3c4,
0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c4, 0x3bb, 0x3b1, 0x3bd, 0x3c4,
0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x391, 0x3c4, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3b9, 0x3ba, 0x3bf,
0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1,
0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5,
0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4,
0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5,
0x3bd, 0x3c4, 0x3c1, 0x3bf, 0x3b4, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3bf, 0x3b4, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a,
0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3bf, 0x3b4, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd,
0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3,
0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x394,
0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1,
0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4,
0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b6,
0x3b5, 0x3c1, 0x3bc, 0x3c0, 0x3b1, 0x3ca, 0x3c4, 0x3b6, 0x3ac, 0x3bd, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3b6, 0x3b5, 0x3c1, 0x3bc, 0x3c0, 0x3b1, 0x3ca, 0x3c4, 0x3b6, 0x3ac, 0x3bd,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391,
0x3b6, 0x3b5, 0x3c1, 0x3bc, 0x3c0, 0x3b1, 0x3ca, 0x3c4, 0x3b6, 0x3ac, 0x3bd, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3b6, 0x3bf, 0x3c1, 0x3ce, 0x3bd, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391,
0x3b6, 0x3bf, 0x3c1, 0x3ce, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b6, 0x3bf, 0x3c1, 0x3ce, 0x3bd, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4,
0x3ad, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3bb, 0x3b1,
0x3bd, 0x3c4, 0x3ad, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3bb, 0x3b1, 0x3bd,
0x3c4, 0x3ad, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3bf, 0x3c5, 0x3c4,
0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x392, 0x3bf, 0x3bb, 0x3b9, 0x3b2, 0x3af,
0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3b6, 0x3af,
0x3bb, 0x3b9, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3b6, 0x3af,
0x3bb, 0x3b9, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3b6, 0x3af, 0x3bb, 0x3b9, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3bf, 0x3c5, 0x3bd, 0x3ad,
0x3b9, 0x20, 0x39d, 0x3c4, 0x3b1, 0x3c1, 0x3bf, 0x3c5, 0x3c3, 0x3b1, 0x3bb, 0x3ac,
0x3bc, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3c1, 0x3ac, 0x3c3, 0x3b9, 0x3bd, 0x3bf,
0x3c5, 0x20, 0x391, 0x3ba, 0x3c1, 0x3c9, 0x3c4, 0x3b7, 0x3c1, 0x3af, 0x3bf, 0x3c5,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a0, 0x3c1, 0x3ac, 0x3c3, 0x3b9, 0x3bd, 0x3bf, 0x3c5, 0x20, 0x391,
0x3ba, 0x3c1, 0x3c9, 0x3c4, 0x3b7, 0x3c1, 0x3af, 0x3bf, 0x3c5, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3c1, 0x3ac, 0x3c3,
0x3b9, 0x3bd, 0x3bf, 0x3c5, 0x20, 0x391, 0x3ba, 0x3c1, 0x3c9, 0x3c4, 0x3b7, 0x3c1,
0x3af, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3, 0x3b1, 0x3bc, 0x3cc,
0x3c1, 0x3bf, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3, 0x3ac, 0x3c4, 0x3b1, 0x3bc,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a4, 0x3c3, 0x3ac, 0x3c4, 0x3b1, 0x3bc, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3, 0x3ac, 0x3c4, 0x3b1,
0x3bc, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3b9, 0x3bb, 0x3ae, 0x3c2, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a7, 0x3b9, 0x3bb, 0x3ae, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3b9, 0x3bb, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39a, 0x3af, 0x3bd, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3af, 0x3bd, 0x3b1,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39a, 0x3af, 0x3bd, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3,
0x3bf, 0x3c5, 0x20, 0x3a7, 0x3c1, 0x3b9, 0x3c3, 0x3c4, 0x3bf, 0x3c5, 0x3b3, 0x3ad,
0x3bd, 0x3bd, 0x3c9, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9,
0x3bd, 0x20, 0x39a, 0x3cc, 0x3ba, 0x3bf, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a,
0x3bf, 0x3bb, 0x3bf, 0x3bc, 0x3b2, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf,
0x3bb, 0x3bf, 0x3bc, 0x3b2, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3bb, 0x3bf, 0x3bc, 0x3b2,
0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd,
0x20, 0x39a, 0x3bf, 0x3c5, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd,
0x20, 0x39a, 0x3bf, 0x3c5, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x39a, 0x3bf,
0x3c5, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3cd, 0x3b2, 0x3b1, 0x3c2,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x39a, 0x3bf, 0x3cd, 0x3b2, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3cd, 0x3b2, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3c4, 0x3ad, 0x3b9, 0x3b2, 0x3b9, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3c4, 0x3b9, 0x3bc, 0x3cc, 0x3bd, 0x20, 0x3bd,
0x3c4, 0x2019, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3b2, 0x3af, 0x3bb, 0x38f, 0x3c1, 0x3b1,
0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20,
0x3a4, 0x3b9, 0x3bc, 0x3cc, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3,
0x3bf, 0x3c5, 0x20, 0x3a0, 0x3ac, 0x3c3, 0x3c7, 0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae,
0x3c3, 0x3bf, 0x3c5, 0x20, 0x3a0, 0x3ac, 0x3c3, 0x3c7, 0x3b1, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3bf,
0x3c5, 0x20, 0x3a0, 0x3ac, 0x3c3, 0x3c7, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399,
0x3c3, 0x3b7, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395,
0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4,
0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5,
0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1,
0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce,
0x3c0, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd,
0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1,
0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3c0, 0x3b5, 0x3c1, 0x3b1,
0x3b9, 0x3c4, 0x3ad, 0x3c1, 0x3c9, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb,
0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4,
0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394,
0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0,
0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20,
0x3a6, 0x3cc, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3,
0x3c9, 0x3bd, 0x20, 0x3a6, 0x3cc, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3,
0x3c9, 0x3bd, 0x20, 0x3a6, 0x3cc, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a6, 0x3af, 0x3c4, 0x3b6, 0x3b9, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3af, 0x3c4,
0x3b6, 0x3b9, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a6, 0x3af, 0x3c4, 0x3b6, 0x3b9, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b1,
0x3bb, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x393, 0x3bf, 0x3c5, 0x3b9, 0x3ac,
0x3bd, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b1, 0x3bb, 0x3bb, 0x3b9,
0x3ba, 0x3bf, 0x3cd, 0x20, 0x39d, 0x3cc, 0x3c4, 0x3bf, 0x3c5, 0x20, 0x3ba, 0x3b1,
0x3b9, 0x20, 0x391, 0x3bd, 0x3c4, 0x3b1, 0x3c1, 0x3ba, 0x3c4, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3ba, 0x3b1, 0x3bb, 0x3ac, 0x3c0, 0x3b1,
0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3ba, 0x3ac, 0x3bc,
0x3c0, 0x3b9, 0x3b5, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1,
0x3b3, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af,
0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x393, 0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x393, 0x3ba, 0x3af, 0x3bb, 0x3bc,
0x3c0, 0x3b5, 0x3c1, 0x3c4, 0x39c, 0x3ad, 0x3c3, 0x3b7, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x393, 0x3ba, 0x3c1, 0x3af, 0x3bd, 0x3bf, 0x3c5, 0x3b9, 0x3c4, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1,
0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x393,
0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x393, 0x3c1,
0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394,
0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb,
0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3ac, 0x3bc, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39a, 0x3cc, 0x3bb, 0x3c0, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20,
0x393, 0x3bf, 0x3c5, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20,
0x3a7, 0x3b1, 0x3b2, 0x3ac, 0x3b7, 0x3c2, 0x2d, 0x391, 0x3bb, 0x3b5, 0x3bf, 0x3cd,
0x3c4, 0x3b9, 0x3c9, 0x3bd, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a7, 0x3b1, 0x3b2, 0x3ac, 0x3b7, 0x3c2, 0x2d, 0x391, 0x3bb, 0x3b5, 0x3bf, 0x3cd,
0x3c4, 0x3b9, 0x3c9, 0x3bd, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3b1, 0x3b2,
0x3ac, 0x3b7, 0x3c2, 0x2d, 0x391, 0x3bb, 0x3b5, 0x3bf, 0x3cd, 0x3c4, 0x3b9, 0x3c9,
0x3bd, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a7,
0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x20, 0x39a, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a7, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x20, 0x39a, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3bf,
0x3bd, 0x3b3, 0x3ba, 0x20, 0x39a, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x38f, 0x3c1, 0x3b1,
0x20, 0x3a7, 0x3bf, 0x3b2, 0x3bd, 0x3c4, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3bf, 0x3b2, 0x3bd,
0x3c4, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a7, 0x3bf, 0x3b2, 0x3bd, 0x3c4, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3bd, 0x3b4,
0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3bd, 0x3b4, 0x3b9, 0x3ba,
0x3bf, 0x3cd, 0x20, 0x3a9, 0x3ba, 0x3b5, 0x3b1, 0x3bd, 0x3bf, 0x3cd, 0x38f, 0x3c1,
0x3b1, 0x20, 0x399, 0x3bd, 0x3b4, 0x3bf, 0x3ba, 0x3af, 0x3bd, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x399, 0x3bd, 0x3b4, 0x3bf, 0x3bd, 0x3b7, 0x3c3, 0x3af, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x399, 0x3bd, 0x3b4, 0x3bf, 0x3bd, 0x3b7, 0x3c3, 0x3af, 0x3b1, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x399, 0x3bd, 0x3b4, 0x3bf, 0x3bd, 0x3b7, 0x3c3, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x399, 0x3c1, 0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1, 0x3ac, 0x3bd,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399,
0x3c1, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1, 0x3ba, 0x3bf, 0x3cd,
0x3c4, 0x3c3, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3,
0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x399, 0x3c1, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20,
0x399, 0x3c3, 0x3c1, 0x3b1, 0x3ae, 0x3bb, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c3, 0x3c1, 0x3b1,
0x3ae, 0x3bb, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x399, 0x3c3, 0x3c1, 0x3b1, 0x3ae, 0x3bb, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399,
0x3b1, 0x3c0, 0x3c9, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3b1, 0x3c0,
0x3c9, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3b1, 0x3c0, 0x3c9, 0x3bd, 0x3af, 0x3b1, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3, 0x3ac, 0x3c4, 0x3ba,
0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b5, 0x3c4, 0x3c1, 0x3bf, 0x3c0, 0x3b1, 0x3b2, 0x3bb,
0x3cc, 0x3b2, 0x3c3, 0x3ba, 0x2d, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3, 0x3ac, 0x3c4,
0x3c3, 0x3ba, 0x3b9, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a0, 0x3b5, 0x3c4, 0x3c1, 0x3bf, 0x3c0, 0x3b1, 0x3b2, 0x3bb, 0x3cc,
0x3b2, 0x3c3, 0x3ba, 0x2d, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3, 0x3ac, 0x3c4, 0x3c3,
0x3ba, 0x3b9, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b1, 0x3b6, 0x3b1, 0x3ba, 0x3c3,
0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf,
0x3bb, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20, 0x39a, 0x3b1, 0x3b6, 0x3b1, 0x3ba, 0x3c3,
0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba,
0x3bf, 0x3cd, 0x20, 0x39a, 0x3b1, 0x3b6, 0x3b1, 0x3ba, 0x3c3, 0x3c4, 0x3ac, 0x3bd,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3c1, 0x3ad, 0x3b1, 0x3c2, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39a, 0x3bf, 0x3c1, 0x3ad, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3c1, 0x3ad, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39a, 0x3cc, 0x3c3, 0x3c1, 0x3b1, 0x3ca, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39a, 0x3c1, 0x3b1, 0x3c3, 0x3bd, 0x3bf, 0x3b3, 0x3b9, 0x3ac, 0x3c1, 0x3c3,
0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39a, 0x3c1, 0x3b1, 0x3c3, 0x3bd, 0x3bf, 0x3b3, 0x3b9, 0x3ac,
0x3c1, 0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x39a, 0x3c1, 0x3b1, 0x3c3, 0x3bd, 0x3bf, 0x3b3, 0x3b9, 0x3ac, 0x3c1,
0x3c3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b9, 0x3c1, 0x3b3, 0x3b9, 0x3c3,
0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd,
0x20, 0x39b, 0x3ac, 0x3b9, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39b, 0x3bf, 0x3c1,
0x3bd, 0x3c4, 0x20, 0x3a7, 0x3ac, 0x3bf, 0x3c5, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39b, 0x3bf, 0x3c1,
0x3bd, 0x3c4, 0x20, 0x3a7, 0x3ac, 0x3bf, 0x3c5, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39b, 0x3bf, 0x3c1, 0x3bd, 0x3c4, 0x20,
0x3a7, 0x3ac, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3ba, 0x3ac,
0x3bf, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3ba, 0x3ac, 0x3bf, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3ba, 0x3ac, 0x3bf,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3ac,
0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3ac, 0x3bd,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c,
0x3b1, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20,
0x39c, 0x3b1, 0x3bb, 0x3b1, 0x3b9, 0x3c3, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39c, 0x3b1, 0x3bb, 0x3b4, 0x3af, 0x3b2, 0x3c9, 0x3bd, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39c, 0x3b1, 0x3c1, 0x3ba, 0x3ad, 0x3b6, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x39c, 0x3ac, 0x3c1, 0x3c3, 0x3b1,
0x3bb, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3c5, 0x3c1, 0x3af, 0x3ba, 0x3b9,
0x3bf, 0x3c5, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3c5, 0x3c1, 0x3af, 0x3ba, 0x3b9, 0x3bf,
0x3c5, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39c, 0x3b1, 0x3c5, 0x3c1, 0x3af, 0x3ba, 0x3b9, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39c, 0x3cc, 0x3c3, 0x3bf, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9,
0x3c1, 0x3b7, 0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20, 0x39c, 0x3b5, 0x3be, 0x3b9,
0x3ba, 0x3bf, 0x3cd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1, 0x3b7, 0x3bd, 0x3b9, 0x3ba,
0x3bf, 0x3cd, 0x20, 0x39c, 0x3b5, 0x3be, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1,
0x3b7, 0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20, 0x39c, 0x3b5, 0x3be, 0x3b9, 0x3ba,
0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3bb, 0x3ac, 0x3bd, 0x20,
0x39c, 0x3c0, 0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3bb, 0x3ac,
0x3bd, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3bb, 0x3ac, 0x3bd,
0x20, 0x39c, 0x3c0, 0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c,
0x3cc, 0x3c3, 0x3c7, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3cc, 0x3c3, 0x3c7, 0x3b1,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39c, 0x3cc, 0x3c3, 0x3c7, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b9,
0x3b1, 0x3bd, 0x3bc, 0x3ac, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3b1, 0x3bf,
0x3cd, 0x3c1, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3b5, 0x3c0, 0x3ac,
0x3bb, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x39a, 0x3b1,
0x3bb, 0x3b7, 0x3b4, 0x3bf, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad,
0x3b1, 0x3c2, 0x20, 0x39a, 0x3b1, 0x3bb, 0x3b7, 0x3b4, 0x3bf, 0x3bd, 0x3af, 0x3b1,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x39a, 0x3b1, 0x3bb, 0x3b7, 0x3b4, 0x3bf, 0x3bd,
0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20,
0x396, 0x3b7, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ad, 0x3b1, 0x3c2, 0x20, 0x396, 0x3b7, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x396, 0x3b7, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af,
0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x393,
0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x393, 0x3b7, 0x3c2,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ad, 0x3b1, 0x3c2, 0x20, 0x393, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3b9, 0x3bf, 0x3cd, 0x3b5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3bf,
0x3c5, 0x20, 0x39d, 0x3cc, 0x3c1, 0x3c6, 0x3bf, 0x3bb, 0x3ba, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20, 0x39d, 0x3cc, 0x3c1, 0x3c6, 0x3bf, 0x3bb, 0x3ba,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20, 0x39d, 0x3cc, 0x3c1, 0x3c6, 0x3bf, 0x3bb, 0x3ba,
0x38f, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3b5, 0x3c1, 0x3bd, 0x3ac, 0x3c1, 0x3bd, 0x3c4,
0x3bf, 0x20, 0x3bd, 0x3c4, 0x3b5, 0x20, 0x39d, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b9,
0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x3a6, 0x3b5, 0x3c1, 0x3bd, 0x3ac, 0x3c1, 0x3bd, 0x3c4, 0x3bf,
0x20, 0x3bd, 0x3c4, 0x3b5, 0x20, 0x39d, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b9, 0x3b1,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a6,
0x3b5, 0x3c1, 0x3bd, 0x3ac, 0x3c1, 0x3bd, 0x3c4, 0x3bf, 0x20, 0x3bd, 0x3c4, 0x3b5,
0x20, 0x39d, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b9, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20,
0x39d, 0x3b7, 0x3c3, 0x3b9, 0x3ce, 0x3bd, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9,
0x3b5, 0x3c2, 0x20, 0x39c, 0x3b1, 0x3c1, 0x3b9, 0x3ac, 0x3bd, 0x3b5, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39d, 0x3bf, 0x3b2, 0x3bf, 0x3c3, 0x3b9, 0x3bc, 0x3c0, 0x3af,
0x3c1, 0x3c3, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3bf, 0x3b2, 0x3bf, 0x3c3, 0x3b9, 0x3bc,
0x3c0, 0x3af, 0x3c1, 0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3bf, 0x3b2, 0x3bf, 0x3c3, 0x3b9, 0x3bc, 0x3c0,
0x3af, 0x3c1, 0x3c3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3bc, 0x3c3, 0x3ba,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x39f, 0x3bc, 0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3bc, 0x3c3, 0x3ba, 0x38f, 0x3c1, 0x3b1,
0x20, 0x3a0, 0x3b1, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0,
0x3b1, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3ba, 0x3b9, 0x3c3, 0x3c4,
0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3bb, 0x3ac, 0x3bf, 0x3c5,
0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3b1, 0x3c2, 0x20,
0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x393, 0x3bf, 0x3c5, 0x3b9, 0x3bd, 0x3ad, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3c1, 0x3b1, 0x3b3, 0x3bf, 0x3c5,
0x3ac, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3c1, 0x3b1, 0x3b3, 0x3bf, 0x3c5,
0x3ac, 0x3b7, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a0, 0x3b1, 0x3c1, 0x3b1, 0x3b3, 0x3bf, 0x3c5, 0x3ac, 0x3b7, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b5, 0x3c1, 0x3bf, 0x3cd, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0,
0x3b5, 0x3c1, 0x3bf, 0x3cd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b5, 0x3c1, 0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20,
0x3a6, 0x3b9, 0x3bb, 0x3b9, 0x3c0, 0x3c0, 0x3b9, 0x3bd, 0x3ce, 0x3bd, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a6, 0x3b9, 0x3bb, 0x3b9, 0x3c0, 0x3c0, 0x3b9, 0x3bd, 0x3ce, 0x3bd, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3b9, 0x3bb,
0x3b9, 0x3c0, 0x3c0, 0x3b9, 0x3bd, 0x3ce, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x3a6, 0x3bf, 0x3af, 0x3bd, 0x3b9, 0x3be, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b5, 0x3bd, 0x20, 0x3a0, 0x3b9, 0x3b5, 0x3c1, 0x20,
0x3ba, 0x3b1, 0x3b9, 0x20, 0x39c, 0x3b9, 0x3ba, 0x3b5, 0x3bb, 0x3cc, 0x3bd, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a3, 0x3b5, 0x3bd, 0x20, 0x3a0, 0x3b9, 0x3b5, 0x3c1, 0x20, 0x3ba, 0x3b1,
0x3b9, 0x20, 0x39c, 0x3b9, 0x3ba, 0x3b5, 0x3bb, 0x3cc, 0x3bd, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b5, 0x3bd, 0x20,
0x3a0, 0x3b9, 0x3b5, 0x3c1, 0x20, 0x3ba, 0x3b1, 0x3b9, 0x20, 0x39c, 0x3b9, 0x3ba,
0x3b5, 0x3bb, 0x3cc, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3af, 0x3c4, 0x3ba,
0x3b5, 0x3c1, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3bf, 0x3bd, 0x3ac, 0x3c0,
0x3b5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b9, 0x3bf, 0x3bd, 0x3b3, 0x3b9, 0x3ac,
0x3bd, 0x3b3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a1, 0x3b5, 0x3ca, 0x3bd, 0x3b9,
0x3cc, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a1, 0x3cc, 0x3b8, 0x3b5, 0x3c1, 0x3b1,
0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b1, 0x3c7, 0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3b7,
0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b1, 0x3c7, 0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3b7, 0x3c2,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3,
0x3b1, 0x3c7, 0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20,
0x3a3, 0x3ac, 0x3bc, 0x3b1, 0x3c1, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3ac, 0x3bc,
0x3b1, 0x3c1, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b1, 0x3bc, 0x3ac, 0x3c1, 0x3b1, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a3, 0x3b1, 0x3bc, 0x3cc, 0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b1, 0x3bc,
0x3cc, 0x3b1, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a3, 0x3b1, 0x3bc, 0x3cc, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b5,
0x3cb, 0x3c7, 0x3b5, 0x3bb, 0x3bb, 0x3ce, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3,
0x3b9, 0x3b3, 0x3ba, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3c1, 0x3b7, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x3a3, 0x3bf, 0x3bb, 0x3bf,
0x3bc, 0x3ce, 0x3bd, 0x3c4, 0x3bf, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3cc,
0x3c4, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3bf, 0x3c5, 0x3c1, 0x3b9, 0x3bd, 0x3ac,
0x3bc, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3af, 0x3bf, 0x3b2, 0x3b1, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a4, 0x3b1, 0x3ca, 0x3c4, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20,
0x3a4, 0x3b1, 0x3ca, 0x3c0, 0x3ad, 0x3b9, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3b1, 0x3ca, 0x3c0,
0x3ad, 0x3b9, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a4, 0x3b1, 0x3ca, 0x3c0, 0x3ad, 0x3b9, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4,
0x3b1, 0x3c4, 0x3b6, 0x3b9, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a4, 0x3bf, 0x3ba, 0x3b5, 0x3bb, 0x3ac, 0x3bf, 0x3c5, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a4, 0x3cc, 0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3cc,
0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x3a4, 0x3cc, 0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x38f, 0x3c1, 0x3b1,
0x20, 0x3a4, 0x3c3, 0x3bf, 0x3c5, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3bf,
0x3c5, 0x3c1, 0x3ba, 0x3bc, 0x3b5, 0x3bd, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a4, 0x3bf, 0x3c5, 0x3c1, 0x3ba, 0x3bc, 0x3b5, 0x3bd, 0x3b9, 0x3c3, 0x3c4,
0x3ac, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a4, 0x3bf, 0x3c5, 0x3c1, 0x3ba, 0x3bc, 0x3b5, 0x3bd, 0x3b9, 0x3c3, 0x3c4,
0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3bf, 0x3c5, 0x3b2, 0x3b1, 0x3bb,
0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3bf, 0x3c5, 0x3b3,
0x3bf, 0x3c5, 0x3ac, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3bf, 0x3c5,
0x3b3, 0x3bf, 0x3c5, 0x3ac, 0x3b7, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3bf, 0x3c5, 0x3b3, 0x3bf,
0x3c5, 0x3ac, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3b6, 0x3bc,
0x3c0, 0x3b5, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5,
0x3b6, 0x3bc, 0x3c0, 0x3b5, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3b6,
0x3bc, 0x3c0, 0x3b5, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1,
0x20, 0x392, 0x3b1, 0x3bd, 0x3bf, 0x3c5, 0x3ac, 0x3c4, 0x3bf, 0x3c5, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x392, 0x3b1, 0x3bd, 0x3bf, 0x3c5, 0x3ac, 0x3c4, 0x3bf, 0x3c5, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3b1, 0x3bd, 0x3bf,
0x3c5, 0x3ac, 0x3c4, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x392, 0x3b5, 0x3bd,
0x3b5, 0x3b6, 0x3bf, 0x3c5, 0x3ad, 0x3bb, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20,
0x392, 0x3bb, 0x3b1, 0x3b4, 0x3b9, 0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc, 0x3ba, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x392, 0x3bb, 0x3b1, 0x3b4, 0x3b9, 0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc, 0x3ba,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392,
0x3bb, 0x3b1, 0x3b4, 0x3b9, 0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc, 0x3ba, 0x38f, 0x3c1,
0x3b1, 0x20, 0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba, 0x3bf, 0x3b3, 0x3ba, 0x3c1, 0x3b1,
0x3bd, 0x3c4, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba, 0x3bf, 0x3b3, 0x3ba,
0x3c1, 0x3b1, 0x3bd, 0x3c4, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba, 0x3bf, 0x3b3, 0x3ba, 0x3c1,
0x3b1, 0x3bd, 0x3c4, 0x38f, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c3, 0x3c4, 0x3bf,
0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20, 0x393,
0x3bf, 0x3c5, 0x3ad, 0x3b9, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3ac,
0x3bb, 0x3b9, 0x3c2, 0x20, 0x3ba, 0x3b1, 0x3b9, 0x20, 0x3a6, 0x3bf, 0x3c5, 0x3c4,
0x3bf, 0x3cd, 0x3bd, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b9, 0x3b1, 0x3ba,
0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b9, 0x3b1, 0x3ba, 0x3bf,
0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x393, 0x3b9, 0x3b1, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba,
0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b9, 0x3ba, 0x3b1, 0x3c4, 0x3b5, 0x3c1, 0x3af,
0x3bd, 0x3bc, 0x3c0, 0x3bf, 0x3c5, 0x3c1, 0x3b3, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b9,
0x3ba, 0x3b1, 0x3c4, 0x3b5, 0x3c1, 0x3af, 0x3bd, 0x3bc, 0x3c0, 0x3bf, 0x3c5, 0x3c1,
0x3b3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x391, 0x3b9, 0x3ba, 0x3b1, 0x3c4, 0x3b5, 0x3c1, 0x3af, 0x3bd, 0x3bc, 0x3c0,
0x3bf, 0x3c5, 0x3c1, 0x3b3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b9, 0x3bf,
0x3cd, 0x3ba, 0x3bf, 0x3bd, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20,
0xf3, 0x72, 0x61, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x20, 0xf3,
0x72, 0x61, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20,
0xf3, 0x72, 0x61, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61,
0x20, 0xf3, 0x72, 0x61, 0xa8f, 0xa95, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f,
0xa95, 0xab0, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4,
0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa95, 0xab0, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xaab, 0xa98, 0xabe, 0xaa8,
0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xaa7,
0xacd, 0xaaf, 0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95, 0xabe, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xa86, 0xaab, 0xacd, 0xab0,
0xabf, 0xa95, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa6, 0xa95, 0xacd, 0xab7, 0xabf,
0xaa3, 0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95, 0xabe, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae,
0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf,
0xa95, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa,
0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95,
0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xa85, 0xab2, 0xabe, 0xab8, 0xacd, 0xa95, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xa85, 0xab2, 0xabe, 0xab8, 0xacd, 0xa95, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab2, 0xabe, 0xab8, 0xacd, 0xa95, 0xabe, 0x20,
0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab2, 0xacd, 0xaae,
0xabe, 0xa9f, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab2, 0xacd, 0xaae, 0xabe,
0xa9f, 0xac0, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4,
0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab2, 0xacd, 0xaae, 0xabe, 0xa9f, 0xac0, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f,
0xaae, 0xac7, 0xa9d, 0xacb, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xaae, 0xac7,
0xa9d, 0xacb, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa8f, 0xaae, 0xac7, 0xa9d, 0xacb, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20,
0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd,
0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd,
0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xa95,
0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85,
0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd, 0xaa6,
0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf,
0xa95, 0xaa8, 0x20, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xab8, 0xaae,
0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf,
0xa95, 0xaa8, 0x20, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20,
0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac2, 0xab0, 0xacd,
0xab5, 0xac0, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89,
0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8,
0x20, 0xaae, 0xabe, 0xa89, 0xaa8, 0xacd, 0xa9f, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95,
0xaa8, 0x20, 0xaae, 0xabe, 0xa89, 0xaa8, 0xacd, 0xa9f, 0xaa8, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20,
0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xaae, 0xabe, 0xa89, 0xaa8,
0xacd, 0xa9f, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95,
0xaa8, 0x20, 0xaaa, 0xac7, 0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf,
0xa95, 0xaa8, 0x20, 0xaaa, 0xac7, 0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0,
0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac7, 0xab8,
0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xa85, 0xaa8, 0xabe, 0xaa6, 0xabf, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85,
0xaa8, 0xabe, 0xaa1, 0xabf, 0xab0, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3,
0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xaa8, 0xabe, 0xaa1, 0xabf,
0xab0, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xa8f, 0xaaa, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xaaa,
0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa8f, 0xaaa, 0xabf, 0xaaf, 0xabe, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8,
0xaae, 0xaaf, 0xa85, 0xa95, 0xacd, 0xaa4, 0xabe, 0xa89, 0x20, 0xab8, 0xaae, 0xaaf,
0xa85, 0xa95, 0xacd, 0xaa4, 0xabe, 0xa89, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe,
0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xa95, 0xacd, 0xaa4,
0xabe, 0xa89, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xa8d, 0xa95, 0xacd, 0xa9f, 0xacb, 0xaac, 0x20, 0xab8, 0xaae, 0xaaf,
0xa8d, 0xa95, 0xacd, 0xa9f, 0xacb, 0xaac, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe,
0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8d, 0xa95, 0xacd, 0xa9f,
0xacb, 0xaac, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xa85, 0xab0, 0xac7, 0xaac, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae,
0xaaf, 0xa85, 0xab0, 0xac7, 0xaac, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab0, 0xac7, 0xaac, 0xabf, 0xaaf, 0xaa8,
0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0, 0xacd,
0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xa86, 0xab0, 0xacd, 0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0, 0xacd, 0xa9c,
0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf,
0xaae, 0xac0, 0x20, 0xa86, 0xab0, 0xacd, 0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0,
0xaa8, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae,
0xac0, 0x20, 0xa86, 0xab0, 0xacd, 0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8,
0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6,
0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xa86, 0xab0, 0xacd, 0xa9c, 0xac7, 0xaa8,
0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0, 0xacd, 0xaae, 0xac7, 0xaa8, 0xabf,
0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0, 0xacd, 0xaae, 0xac7, 0xaa8,
0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa86, 0xab0, 0xacd, 0xaae, 0xac7, 0xaa8, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa9f, 0xab2,
0xabe, 0xaa8, 0xacd, 0xa9f, 0xabf, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa9f,
0xab2, 0xabe, 0xaa8, 0xacd, 0xa9f, 0xabf, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa9f, 0xab2, 0xabe, 0xaa8, 0xacd, 0xa9f, 0xabf,
0xa95, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac7,
0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xa91, 0xab8, 0xacd, 0xa9f,
0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa91,
0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95,
0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7,
0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0,
0xac0, 0xaaf, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa91,
0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95,
0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaaa, 0xab6, 0xacd,
0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa91, 0xab8, 0xacd, 0xa9f,
0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd,
0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae,
0xac0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93, 0xab8,
0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95, 0xac7,
0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaaa, 0xab6, 0xacd, 0xa9a,
0xabf, 0xaae, 0xac0, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0xaaf, 0x20, 0xa91, 0xab8, 0xacd, 0xa9f,
0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa91,
0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaaa,
0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0xaaf, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xa93, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf,
0xaaf, 0xaa8, 0x20, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0xaaf, 0x20, 0xaa6,
0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf,
0xaae, 0xac0, 0x20, 0xa91, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf,
0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa91, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0,
0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae,
0xac0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93, 0xab8,
0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaaa, 0xab6,
0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8,
0xaae, 0xaaf, 0xa85, 0xa9d, 0xab0, 0xaac, 0xac8, 0xa9c, 0xabe, 0xaa8, 0x20, 0xab8,
0xaae, 0xaaf, 0xa85, 0xa9d, 0xab0, 0xaac, 0xac8, 0xa9c, 0xabe, 0xaa8, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xa9d, 0xab0, 0xaac, 0xac8,
0xa9c, 0xabe, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xa8f, 0xa9d, 0xacb, 0xab0, 0xacd, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xa8f, 0xa9d, 0xacb, 0xab0, 0xacd, 0xab8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa9d, 0xacb, 0xab0, 0xacd, 0xab8, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xabe,
0xa82, 0xa97, 0xacd, 0xab2, 0xabe, 0xaa6, 0xac7, 0xab6, 0x20, 0xab8, 0xaae, 0xaaf,
0xaac, 0xabe, 0xa82, 0xa97, 0xacd, 0xab2, 0xabe, 0xaa6, 0xac7, 0xab6, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xabe, 0xa82, 0xa97, 0xacd,
0xab2, 0xabe, 0xaa6, 0xac7, 0xab6, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaad, 0xac2, 0xa9f, 0xabe, 0xaa8, 0x20, 0xab8,
0xaae, 0xaaf, 0xaac, 0xacb, 0xab2, 0xabf, 0xab5, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8,
0xaae, 0xaaf, 0xaac, 0xacd, 0xab0, 0xabe, 0xa9d, 0xabf, 0xab2, 0xabf, 0xaaf, 0xabe,
0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xacd, 0xab0, 0xabe, 0xa9d, 0xabf, 0xab2, 0xabf,
0xaaf, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac,
0xacd, 0xab0, 0xabe, 0xa9d, 0xabf, 0xab2, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xacd, 0xab0,
0xac1, 0xaa8, 0xac7, 0xa87, 0x20, 0xaa6, 0xab0, 0xac1, 0xab8, 0xab2, 0xabe, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac5, 0xaaa, 0x20, 0xab5, 0xab0, 0xacd, 0xaa1,
0xac7, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac5, 0xaaa, 0x20, 0xab5, 0xab0, 0xacd,
0xaa1, 0xac7, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95,
0xac5, 0xaaa, 0x20, 0xab5, 0xab0, 0xacd, 0xaa1, 0xac7, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac7, 0xaae, 0xacb,
0xab0, 0xacb, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a,
0xac7, 0xaa4, 0xab9, 0xabe, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xac7, 0xaa4,
0xab9, 0xabe, 0xaae, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9a, 0xac7, 0xaa4, 0xab9, 0xabe, 0xaae, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20,
0xab8, 0xaae, 0xaaf, 0xa9a, 0xabf, 0xab2, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a,
0xabf, 0xab2, 0xac0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9a, 0xabf, 0xab2, 0xac0, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xac0, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a,
0xac0, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a,
0xac0, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95,
0xacd, 0xab0, 0xabf, 0xab8, 0xaae, 0xab8, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xa95, 0xacb, 0xa9d, 0x20,
0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xa95, 0xacb, 0xab2, 0xaae, 0xacd, 0xaac, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8,
0xaae, 0xaaf, 0xa95, 0xacb, 0xab2, 0xaae, 0xacd, 0xaac, 0xabf, 0xaaf, 0xabe, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab2, 0xaae,
0xacd, 0xaac, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac1, 0xa95, 0x20, 0xa86, 0xa87, 0xab2,
0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac1,
0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac1, 0xa95, 0x20,
0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xa85, 0xab0,
0xacd, 0xaa7, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xaac, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xa95, 0xacd, 0xaaf, 0xac1, 0xaac, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xaac, 0xabe, 0x20, 0xaa6, 0xabf,
0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa1, 0xac7, 0xab5, 0xabf, 0xab8, 0x20,
0xab8, 0xaae, 0xaaf, 0xaa1, 0xacd, 0xaaf, 0xac1, 0xaae, 0xacb, 0xaa8, 0xacd, 0xa9f,
0x2d, 0xaa1, 0xac0, 0x2018, 0xa89, 0xab0, 0xacd, 0xab5, 0xabf, 0xab2, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xaa4, 0xabf, 0xaae, 0xacb,
0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xab8, 0xacd, 0xa9f, 0xab0, 0x20, 0xa86,
0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xab8,
0xacd, 0xa9f, 0xab0, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xab8, 0xacd, 0xa9f,
0xab0, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa95, 0xacd,
0xab5, 0xabe, 0xaa1, 0xacb, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xaa7, 0xacd,
0xaaf, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8,
0xaae, 0xaaf, 0xaae, 0xaa7, 0xacd, 0xaaf, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa,
0xabf, 0xaaf, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xaae, 0xaa7, 0xacd, 0xaaf, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf,
0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb,
0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd,
0xab5, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd,
0xab5, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8,
0xac1, 0xaa6, 0xac2, 0xab0, 0x2d, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20,
0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb,
0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a,
0xabf, 0xaae, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd,
0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf,
0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xaab, 0xac9, 0xa95, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa86, 0xa87,
0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab,
0xac9, 0xa95, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa86, 0xa87, 0xab2, 0xac7,
0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xaab, 0xac9, 0xa95, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa86,
0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xac0, 0xa9c, 0xac0,
0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xac0, 0xa9c, 0xac0, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xac0, 0xa9c, 0xac0, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xacd, 0xab0,
0xac7, 0xa82, 0xa9a, 0x20, 0xa97, 0xaaf, 0xabe, 0xaa8, 0xabe, 0x20, 0xab8, 0xaae,
0xaaf, 0xaab, 0xacd, 0xab0, 0xac7, 0xa82, 0xa9a, 0x20, 0xab8, 0xaa7, 0xab0, 0xacd,
0xaa8, 0x20, 0xa85, 0xaa8, 0xac7, 0x20, 0xa8f, 0xaa8, 0xacd, 0xa9f, 0xabe, 0xab0,
0xacd, 0xa95, 0xa9f, 0xabf, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa97, 0xabe, 0xab2,
0xabe, 0xaaa, 0xabe, 0xa97, 0xacb, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa97, 0xac7,
0xaae, 0xacd, 0xaac, 0xabf, 0xaaf, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xacd,
0xaaf, 0xacb, 0xab0, 0xacd, 0xa9c, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9c, 0xacd, 0xaaf, 0xacb, 0xab0, 0xacd, 0xa9c, 0xabf, 0xaaf, 0xabe, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xacd, 0xaaf, 0xacb, 0xab0,
0xacd, 0xa9c, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa97, 0xabf, 0xab2, 0xaac, 0xab0, 0xacd, 0xa9f,
0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8,
0xaae, 0xaaf, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab5, 0xabf, 0xa9a, 0x20, 0xaae,
0xaa7, 0xacd, 0xaaf, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd,
0xab5, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd,
0xa9a, 0xabf, 0xaae, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a,
0xabf, 0xaae, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd,
0xaa1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xa97, 0xacd, 0xab5, 0xabe, 0xaae, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe,
0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa97, 0xab2, 0xacd, 0xaab,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xab5, 0xabe,
0xa88, 0x2d, 0xa8f, 0xab2, 0xacd, 0xaaf, 0xac1, 0xab6, 0xabf, 0xa85, 0xaa8, 0x20,
0xab8, 0xaae, 0xaaf, 0xab9, 0xab5, 0xabe, 0xa87, 0x2d, 0xa8f, 0xab2, 0xacd, 0xaaf,
0xac1, 0xab6, 0xabf, 0xa85, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xab9, 0xab5, 0xabe, 0xa87, 0x2d, 0xa8f, 0xab2, 0xacd, 0xaaf, 0xac1,
0xab6, 0xabf, 0xa85, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xab9, 0xacb, 0xa82, 0xa97, 0x20, 0xa95, 0xacb, 0xa82, 0xa97, 0x20, 0xab8,
0xaae, 0xaaf, 0xab9, 0xacb, 0xa82, 0xa97, 0x20, 0xa95, 0xacb, 0xa82, 0xa97, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xacb, 0xa82, 0xa97,
0x20, 0xa95, 0xacb, 0xa82, 0xa97, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xacb, 0xab5, 0xacd, 0xaa1, 0x20, 0xab8,
0xaae, 0xaaf, 0xab9, 0xacb, 0xab5, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xacb, 0xab5, 0xacd, 0xaa1, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaad, 0xabe, 0xab0,
0xaa4, 0xac0, 0xaaf, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xaad, 0xabe, 0xab0, 0xaa4, 0xac0, 0xaaf, 0x20, 0xaae, 0xab9, 0xabe, 0xab8, 0xabe,
0xa97, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xacb, 0xa9a,
0xabe, 0xa87, 0xaa8, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xaa7, 0xacd, 0xaaf,
0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xacb, 0xaa8, 0xac7, 0xab6, 0xabf, 0xaaf, 0xabe,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0xaaf, 0x20,
0xa87, 0xaa8, 0xacd, 0xaa1, 0xacb, 0xaa8, 0xac7, 0xab6, 0xabf, 0xaaf, 0xabe, 0x20,
0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xa87,
0xaa8, 0xacd, 0xaa1, 0xacb, 0xaa8, 0xac7, 0xab6, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8,
0xaae, 0xaaf, 0xa88, 0xab0, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa88, 0xab0,
0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa88,
0xab0, 0xabe, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa87, 0xab0, 0xacd, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xa87, 0xab0, 0xacd, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xab0, 0xacd,
0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xa9d, 0xab0, 0xabe, 0xa87,
0xab2, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xa9d, 0xab0, 0xabe, 0xa87, 0xab2, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xa9d, 0xab0, 0xabe,
0xa87, 0xab2, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c,
0xabe, 0xaaa, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xabe, 0xaaa, 0xabe,
0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xabe,
0xaaa, 0xabe, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaa, 0xac7, 0xa9f, 0xacd, 0xab0, 0xacb, 0xaaa, 0xac7, 0xab5, 0xab2, 0xacb, 0xab5,
0xacd, 0xab8, 0xacd, 0xa95, 0x2d, 0xa95, 0xabe, 0xaae, 0xa9a, 0xaa4, 0xab8, 0xacd,
0xa95, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xa9f, 0xacd, 0xab0, 0xacb,
0xaaa, 0xac7, 0xab5, 0xab2, 0xacb, 0xab5, 0xacd, 0xab8, 0xacd, 0xa95, 0x2d, 0xa95,
0xabe, 0xaae, 0xa9a, 0xaa4, 0xab8, 0xacd, 0xa95, 0xac0, 0x20, 0xaaa, 0xacd, 0xab0,
0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7,
0xa9f, 0xacd, 0xab0, 0xacb, 0xaaa, 0xac7, 0xab5, 0xab2, 0xacb, 0xab5, 0xacd, 0xab8,
0xacd, 0xa95, 0x2d, 0xa95, 0xabe, 0xaae, 0xa9a, 0xaa4, 0xab8, 0xacd, 0xa95, 0xac0,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xa95, 0xa9d, 0xabe, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xa95, 0xa9d, 0xabe, 0xa95,
0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6,
0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa95, 0xa9d, 0xabe, 0xa95, 0xabf, 0xab8, 0xacd,
0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab0, 0xabf, 0xaaf,
0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab0, 0xabf, 0xaaf, 0xaa8, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab0, 0xabf,
0xaaf, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95,
0xacb, 0xab8, 0xab0, 0xac7, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xab0, 0xac7,
0xab8, 0xacd, 0xaa8, 0xacb, 0xaaf, 0xabe, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xab0, 0xac7, 0xab8, 0xacd, 0xaa8, 0xacb, 0xaaf,
0xabe, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xab0, 0xac7, 0xab8, 0xacd, 0xaa8, 0xacb, 0xaaf,
0xabe, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xabf, 0xab0, 0xacd, 0xa97, 0xabf,
0xa9d, 0xacd, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab2,
0xa82, 0xa95, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xab2, 0xabe, 0xa87, 0xaa8, 0x20,
0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0x20, 0xab9, 0xacb, 0xab5, 0x20, 0xab8,
0xaae, 0xaaf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0x20, 0xab9, 0xacb, 0xab5, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab2, 0xacb, 0xab0, 0xacd,
0xaa1, 0x20, 0xab9, 0xacb, 0xab5, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8,
0xaae, 0xaaf, 0xaae, 0xa95, 0xabe, 0xa89, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa95,
0xabe, 0xa89, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4,
0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa95, 0xabe, 0xa89, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa97, 0xabe, 0xaa1,
0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa97, 0xabe, 0xaa1, 0xaa8, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa97, 0xabe, 0xaa1, 0xaa8,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xaae, 0xab2, 0xac7, 0xab6, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae,
0xabe, 0xab2, 0xaa6, 0xac0, 0xab5, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xabe, 0xab0,
0xacd, 0xa95, 0xacd, 0xaaf, 0xac2, 0xab8, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae,
0xabe, 0xab0, 0xacd, 0xab6, 0xab2, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd,
0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab0, 0xabf, 0xab6,
0xabf, 0xaaf, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab0, 0xabf, 0xab6,
0xabf, 0xaaf, 0xab8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xaae, 0xacb, 0xab0, 0xabf, 0xab6, 0xabf, 0xaaf, 0xab8, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab8, 0xaa8,
0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xac7, 0xa95, 0xacd, 0xab8, 0xabf, 0xa95, 0xaa8,
0x20, 0xaaa, 0xac7, 0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xaae, 0xac7, 0xa95, 0xacd, 0xab8, 0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac7, 0xab8,
0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xaae, 0xac7, 0xa95, 0xacd, 0xab8, 0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac7,
0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8,
0xaae, 0xaaf, 0xa89, 0xab2, 0xabe, 0xaa8, 0x20, 0xaac, 0xabe, 0xa9f, 0xacb, 0xab0,
0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xab2, 0xabe, 0xaa8, 0x20, 0xaac, 0xabe, 0xa9f,
0xacb, 0xab0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89,
0xab2, 0xabe, 0xaa8, 0x20, 0xaac, 0xa9f, 0xacb, 0xab0, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab8, 0xacd,
0xa95, 0xacb, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab8, 0xacd, 0xa95, 0xacb,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab8,
0xacd, 0xa95, 0xacb, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xaae, 0xacd, 0xaaf, 0xabe, 0xaa8, 0xaae, 0xabe, 0xab0, 0x20,
0xab8, 0xaae, 0xaaf, 0xaa8, 0xacc, 0xab0, 0xac1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8,
0xac7, 0xaaa, 0xabe, 0xab3, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1,
0x20, 0xab8, 0xac7, 0xab2, 0xac7, 0xaa1, 0xacb, 0xaa8, 0xabf, 0xaaf, 0xabe, 0x20,
0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0x20, 0xab8, 0xac7, 0xab2, 0xac7,
0xaa1, 0xacb, 0xaa8, 0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0x20, 0xab8, 0xac7, 0xab2, 0xac7,
0xaa1, 0xacb, 0xaa8, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0xa9d, 0xac0,
0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf,
0xac1, 0xa9d, 0xac0, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0xa9d, 0xac0, 0xab2,
0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac2, 0xaab, 0xabe, 0xa89, 0xaa8, 0xacd, 0xaa1, 0xab2,
0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac2,
0xaab, 0xabe, 0xa89, 0xaa8, 0xacd, 0xaa1, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac2,
0xaab, 0xabe, 0xa89, 0xaa8, 0xacd, 0xaa1, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20,
0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xac0, 0xaaf, 0xac1,
0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacb, 0xab0, 0xaab, 0xac9, 0xa95, 0x20, 0xa86,
0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacb,
0xab0, 0xaab, 0xac9, 0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacb, 0xab0,
0xaab, 0xac9, 0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20,
0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xab0, 0xacd, 0xaa8,
0xabe, 0xaa8, 0xacd, 0xaa1, 0xacb, 0x20, 0xaa1, 0xac0, 0x20, 0xaa8, 0xacb, 0xab0,
0xacb, 0xaa8, 0xacd, 0xab9, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xab0, 0xacd,
0xaa8, 0xabe, 0xaa8, 0xacd, 0xaa1, 0xacb, 0x20, 0xaa1, 0xac0, 0x20, 0xaa8, 0xacb,
0xab0, 0xacb, 0xaa8, 0xacd, 0xab9, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xaab, 0xab0, 0xacd, 0xaa8, 0xabe, 0xaa8, 0xacd, 0xaa1, 0xacb,
0x20, 0xaa6, 0xac7, 0x20, 0xaa8, 0xacb, 0xab0, 0xacb, 0xab9, 0xabe, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4,
0xacd, 0xaa4, 0xab0, 0x20, 0xaae, 0xabe, 0xab0, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe,
0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0xaa8, 0xacb,
0xab5, 0xab8, 0xabf, 0xaac, 0xabf, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xaa8, 0xacb, 0xab5, 0xab8, 0xabf, 0xaac, 0xabf, 0xab0, 0xacd, 0xab8,
0xacd, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8,
0xacb, 0xab5, 0xab8, 0xabf, 0xaac, 0xabf, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93,
0xaae, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93, 0xaae, 0xacd,
0xab8, 0xacd, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa93, 0xaae, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xabe, 0xa95, 0xabf, 0xab8, 0xacd,
0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xabe, 0xa95, 0xabf, 0xab8,
0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xabe, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab2,
0xabe, 0xa89, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xabe, 0xaaa, 0xac1, 0xa86, 0x20,
0xaa8, 0xacd, 0xaaf, 0xac1, 0x20, 0xa97, 0xabf, 0xaa8, 0xac0, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xac7, 0xab0, 0xabe, 0xa97, 0xacd, 0xab5, 0xac7, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xac7, 0xab0, 0xabe, 0xa97, 0xacd, 0xab5, 0xac7, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xab0, 0xabe, 0xa97, 0xacd,
0xab5, 0xac7, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaa, 0xac7, 0xab0, 0xac1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7,
0xab0, 0xac1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa,
0xac7, 0xab0, 0xac1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xaab, 0xabf, 0xab2, 0xabf, 0xaaa, 0xabe, 0xa87, 0xaa8, 0x20,
0xab8, 0xaae, 0xaaf, 0xaab, 0xabf, 0xab2, 0xabf, 0xaaa, 0xabe, 0xa87, 0xaa8, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xabf, 0xab2, 0xabf,
0xaaa, 0xabe, 0xa87, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xacb, 0xaa8, 0xabf, 0xa95, 0xacd, 0xab8, 0x20,
0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xaaa, 0xac0, 0xa8f, 0xab0, 0xac0, 0x20,
0xa85, 0xaa8, 0xac7, 0x20, 0xaae, 0xabf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xab2, 0xacb,
0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xaaa, 0xac0,
0xa8f, 0xab0, 0xac0, 0x20, 0xa85, 0xaa8, 0xac7, 0x20, 0xaae, 0xabf, 0xa95, 0xacd,
0xaaf, 0xac1, 0xab2, 0xacb, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xaaa, 0xac0, 0xa8f, 0xab0, 0xac0,
0x20, 0xa85, 0xaa8, 0xac7, 0x20, 0xaae, 0xabf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xab2,
0xacb, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa,
0xabf, 0xa9f, 0xa95, 0xac7, 0xaaf, 0xab0, 0xacd, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaa, 0xacb, 0xaa8, 0xaaa, 0xac7, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xacd, 0xaaf,
0xacb, 0xa82, 0xa97, 0xaaf, 0xabe, 0xa82, 0xa97, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95,
0xabf, 0xa9d, 0xabf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0xabe, 0x20, 0xab8, 0xaae,
0xaaf, 0xa95, 0xabf, 0xa9d, 0xabf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0xabe, 0x20,
0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae,
0xaaf, 0xa95, 0xabf, 0xa9d, 0xabf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0xabe, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab0,
0xac0, 0xaaf, 0xac1, 0xaa8, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab0,
0xacb, 0xaa5, 0xac7, 0xab0, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xa96, 0xabe,
0xab2, 0xabf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xa96, 0xabe, 0xab2, 0xabf,
0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xa96,
0xabe, 0xab2, 0xabf, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xaae, 0xabe, 0xab0, 0xabe, 0x20, 0xab8, 0xaae,
0xaaf, 0xab8, 0xaae, 0xabe, 0xab0, 0xabe, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe,
0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xaae, 0xabe, 0xab0,
0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xab8, 0xaae, 0xacb, 0xa86, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xaae, 0xacb,
0xa86, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xaae,
0xacb, 0xa86, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xab8, 0xac7, 0xab6, 0xab2, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xab8, 0xabf, 0xa82, 0xa97, 0xabe, 0xaaa, 0xac1, 0xab0, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xacb, 0xab2, 0xacb, 0xaae, 0xaa8, 0x20,
0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xaa6, 0xa95, 0xacd, 0xab7, 0xabf, 0xaa3, 0x20, 0xa9c, 0xacd, 0xaaf, 0xacb,
0xab0, 0xacd, 0xa9c, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xac1,
0xab0, 0xac0, 0xaa8, 0xabe, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xacd, 0xaaf,
0xacb, 0xab5, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xab9, 0xabf, 0xaa4,
0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xa87, 0xaaa, 0xac7, 0xa87, 0x20,
0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xa87, 0xaaa, 0xac7, 0xa87, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xa87, 0xaaa, 0xac7, 0xa87,
0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xa9c,
0xac0, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9f, 0xacb, 0xa95, 0xab2, 0xabe, 0xa89, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9f, 0xacb,
0xa82, 0xa97, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9f, 0xacb, 0xa82, 0xa97, 0xabe,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9f, 0xacb, 0xa82,
0xa97, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xa9a, 0xac1, 0xa89, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xac1,
0xab0, 0xacd, 0xa95, 0xaae, 0xac7, 0xaa8, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8,
0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xac1, 0xab0, 0xacd, 0xa95, 0xaae, 0xac7, 0xaa8,
0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xaa4, 0xac1, 0xab0, 0xacd, 0xa95, 0xaae, 0xac7, 0xaa8, 0xabf,
0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9f, 0xab5, 0xabe, 0xab2, 0xac2, 0x20, 0xab8,
0xaae, 0xaaf, 0xa89, 0xab0, 0xac1, 0xa97, 0xacd, 0xab5, 0xac7, 0x20, 0xab8, 0xaae,
0xaaf, 0xa89, 0xab0, 0xac1, 0xa97, 0xacd, 0xab5, 0xac7, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xab0, 0xac1, 0xa97, 0xacd, 0xab5, 0xac7,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xa89, 0xa9d, 0xacd, 0xaac, 0xac7, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8,
0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xa9d, 0xacd, 0xaac, 0xac7, 0xa95, 0xabf, 0xab8,
0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa89, 0xa9d, 0xacd, 0xaac, 0xac7, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe,
0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xab5, 0xaa8, 0xabe, 0xaa4, 0xac2, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xaa8,
0xabe, 0xaa4, 0xac2, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xab5, 0xaa8, 0xabe, 0xaa4, 0xac2, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xac7, 0xaa8, 0xac7, 0xa9d, 0xac1, 0xa8f,
0xab2, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xacd, 0xab2, 0xabe, 0xaa1, 0xabf,
0xab5, 0xacb, 0xab8, 0xacd, 0xa9f, 0xacb, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5,
0xacd, 0xab2, 0xabe, 0xaa1, 0xabf, 0xab5, 0xacb, 0xab8, 0xacd, 0xa9f, 0xacb, 0xa95,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xacd, 0xab2,
0xabe, 0xaa1, 0xabf, 0xab5, 0xacb, 0xab8, 0xacd, 0xa9f, 0xacb, 0xa95, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xacb,
0xab2, 0xacd, 0xa97, 0xacb, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1, 0x20, 0xab8, 0xaae,
0xaaf, 0xab5, 0xacb, 0xab2, 0xacd, 0xa97, 0xacb, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xacb, 0xab2,
0xacd, 0xa97, 0xacb, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xac5, 0xa95, 0x20,
0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5,
0xac9, 0xab2, 0xabf, 0xab8, 0x20, 0xa85, 0xaa8, 0xac7, 0x20, 0xaab, 0xacd, 0xaaf,
0xac1, 0xa9a, 0xac1, 0xaa8, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xabe, 0xa95,
0xac1, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xabe,
0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xabe, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xacd,
0xa95, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaf, 0xac7, 0xa95, 0xabe, 0xa9f, 0xac7, 0xab0, 0xabf, 0xaa8, 0xaac, 0xab0,
0xacd, 0xa97, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xac7, 0xa95, 0xabe, 0xa9f, 0xac7,
0xab0, 0xabf, 0xaa8, 0xaac, 0xab0, 0xacd, 0xa97, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xac7, 0xa95, 0xabe, 0xa9f, 0xac7, 0xab0, 0xabf,
0xaa8, 0xaac, 0xab0, 0xacd, 0xa97, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xac1, 0xa95, 0xacb, 0xaa8, 0x20, 0xab8,
0xaae, 0xaaf, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x66,
0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x69, 0x72, 0x6b, 0x61, 0x20,
0x74, 0x61, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68,
0x69, 0x6e, 0x20, 0x41, 0x66, 0x69, 0x72, 0x6b, 0x61, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x41, 0x66, 0x69, 0x72, 0x6b, 0x61, 0x20, 0x74, 0x61,
0x20, 0x4b, 0x75, 0x64, 0x75, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x66, 0x69, 0x72, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x20, 0x59,
0x61, 0x6d, 0x6d, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x66,
0x69, 0x72, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x66, 0x69, 0x72, 0x6b,
0x61, 0x20, 0x74, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x64, 0x61, 0x6b, 0x65, 0x20,
0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61, 0x72, 0x65, 0x77,
0x61, 0x20, 0x74, 0x61, 0x20, 0x74, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61,
0x72, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41,
0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69,
0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61,
0x6e, 0x61, 0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x72, 0x65, 0x77,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x74, 0x61, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73,
0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x20, 0x64,
0x61, 0x6b, 0x65, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x61, 0x20,
0x47, 0x61, 0x62, 0x61, 0x73, 0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41,
0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72,
0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54,
0x73, 0x61, 0x75, 0x6e, 0x69, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x65,
0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x6b, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x74, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x74, 0x73, 0x61, 0x75, 0x6e,
0x69, 0x20, 0x61, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x63, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x54, 0x73, 0x61, 0x75, 0x6e, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x65, 0x77,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x6b, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41,
0x6d, 0x75, 0x72, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x70, 0x69, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41,
0x70, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x65, 0x62, 0x69,
0x79, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x65, 0x62,
0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x65, 0x62, 0x69,
0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d,
0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d,
0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x6d,
0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x2c, 0x20,
0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x63, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x56, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b,
0x61, 0x6e, 0x61, 0x64, 0x61, 0x2c, 0x20, 0x50, 0x75, 0x65, 0x72, 0x74,
0x6f, 0x20, 0x52, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x56, 0x69,
0x72, 0x67, 0x69, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x2c, 0x20,
0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x63, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x56, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41,
0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41,
0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20,
0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41,
0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54,
0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74,
0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x6c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69,
0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x44, 0x61, 0x69,
0x64, 0x61, 0x69, 0x74, 0x61, 0x63, 0x63, 0x65, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x7a, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76,
0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x6c, 0x69, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x6c, 0x69, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x6c, 0x69,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x61,
0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x53, 0x69, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x53, 0x69, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73,
0x74, 0x6d, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x52, 0x61, 0x62, 0x69, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x20, 0x6e, 0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4b, 0x75,
0x62, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x61,
0x76, 0x69, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x45,
0x61, 0x73, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x54,
0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x61, 0x20, 0x6c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x49, 0x64,
0x61, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x61,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x54, 0x75, 0x72, 0x61,
0x69, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x62, 0x61,
0x7a, 0x61, 0x72, 0x61, 0x20, 0x61, 0x20, 0x6c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x20, 0x61, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x20, 0x67, 0x61,
0x62, 0x61, 0x73, 0x49, 0x64, 0x61, 0x20, 0x6c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x20, 0x61, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x20, 0x67, 0x61,
0x62, 0x61, 0x73, 0x47, 0x61, 0x62, 0x61, 0x73, 0x20, 0x61, 0x20, 0x6c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x20,
0x64, 0x61, 0x20, 0x62, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69,
0x6e, 0x20, 0x54, 0x75, 0x72, 0x61, 0x69, 0x20, 0x6d, 0x61, 0x69, 0x20,
0x4e, 0x69, 0x73, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x74,
0x61, 0x20, 0x79, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x74,
0x75, 0x72, 0x61, 0x69, 0x49, 0x64, 0x61, 0x20, 0x6c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x20, 0x74, 0x61, 0x20, 0x79, 0x61, 0x6d, 0x6d, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x49, 0x64, 0x61, 0x20,
0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x74, 0x61, 0x20, 0x79, 0x61,
0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69,
0x20, 0x64, 0x61, 0x20, 0x62, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x46, 0x72,
0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x75, 0x64, 0x61, 0x6e,
0x63, 0x69, 0x6e, 0x20, 0x46, 0x61, 0x72, 0x61, 0x6e, 0x73, 0x61, 0x20,
0x64, 0x61, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x61, 0x20, 0x49, 0x6e, 0x67, 0x69,
0x6c, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61,
0x62, 0x61, 0x73, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61,
0x62, 0x61, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x61, 0x62, 0x61,
0x73, 0x68, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61,
0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61,
0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x75, 0x6c, 0x66,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x75, 0x79, 0x61,
0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x6e, 0x61, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x65,
0x6b, 0x75, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68,
0x69, 0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x74, 0x61, 0x20,
0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69, 0x6e, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x73, 0x72, 0x61, 0x2bc, 0x69,
0x6c, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x73, 0x72, 0x61,
0x2bc, 0x69, 0x6c, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x48, 0x61, 0x73, 0x6b, 0x65, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x2bc, 0x69, 0x6c, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x6c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x48, 0x61, 0x73, 0x6b, 0x65,
0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69,
0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b,
0x6f, 0x72, 0x69, 0x79, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b,
0x6f, 0x72, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4b, 0x6f, 0x72,
0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x7a,
0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x56, 0x6f, 0x74, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x65, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x65, 0x6b, 0x7a, 0x69, 0x6b, 0x6f,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x4d, 0x65, 0x6b, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x65, 0x6b,
0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a,
0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x20, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x20, 0x74, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20,
0x72, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61,
0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x65,
0x72, 0x75, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x72, 0x75,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x69, 0x6e, 0x65, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x68,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x64, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50,
0x6f, 0x6e, 0x61, 0x70, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x6f, 0x74,
0x68, 0x65, 0x72, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x56, 0x6f, 0x74, 0x65, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x75, 0x64, 0x61, 0x6e, 0x63,
0x69, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x68, 0x69, 0x74, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x75,
0x75, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x64, 0x61, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e4, 0x5d2, 0x5e0, 0x5d9, 0x5e1, 0x5d8, 0x5df,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5e4,
0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8,
0x5d7, 0x20, 0x5d0, 0x5e4, 0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d3, 0x5e8, 0x5d5, 0x5dd, 0x20, 0x5d0, 0x5e4, 0x5e8, 0x5d9, 0x5e7, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e4,
0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8,
0x5d1, 0x20, 0x5d0, 0x5e4, 0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1,
0x20, 0x5d0, 0x5e4, 0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5dc, 0x5e1, 0x5e7, 0x5d4, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5dc, 0x5e1, 0x5e7, 0x5d4, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5dc, 0x5e1,
0x5e7, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5dc, 0x5de, 0x5d8, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5dc, 0x5de, 0x5d8, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5dc, 0x5de, 0x5d8, 0x5d9, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5de, 0x5d6, 0x5d5,
0x5e0, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5de, 0x5d6, 0x5d5, 0x5e0,
0x5e1, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5de, 0x5d6, 0x5d5, 0x5e0, 0x5e1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0,
0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db,
0x5d6, 0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20,
0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d7, 0x5d5, 0x5e3, 0x20, 0x5d4, 0x5de, 0x5d6,
0x5e8, 0x5d7, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d7, 0x5d5, 0x5e3,
0x20, 0x5d4, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d7, 0x5d5, 0x5e3, 0x20,
0x5d4, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x20, 0x5d4, 0x5d4,
0x5e8, 0x5d9, 0x5dd, 0x20, 0x5d1, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x20, 0x5d4, 0x5d4, 0x5e8, 0x5d9,
0x5dd, 0x20, 0x5d1, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d6, 0x5d5, 0x5e8,
0x20, 0x5d4, 0x5d4, 0x5e8, 0x5d9, 0x5dd, 0x20, 0x5d1, 0x5d0, 0x5e8, 0x5d4, 0x5f4,
0x5d1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4,
0x5d1, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e0,
0x5d3, 0x5d9, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8, 0x5d2, 0x5d9, 0x5dc,
0x20, 0x5d0, 0x5e0, 0x5d3, 0x5d9, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7,
0x5d9, 0x5e5, 0x20, 0x5d0, 0x5e0, 0x5d3, 0x5d9, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5e4, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e4,
0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5e4, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e7, 0x5d8, 0x5d0, 0x5d5, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5e7, 0x5d8, 0x5d0, 0x5d5, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e7, 0x5d8, 0x5d0,
0x5d5, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5e7, 0x5d8, 0x5d5, 0x5d1, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5d5, 0x5e7, 0x5d8, 0x5d5, 0x5d1, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e7, 0x5d8, 0x5d5, 0x5d1,
0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d7, 0x5e6, 0x5d9, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e2, 0x5e8, 0x5d1, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5e6, 0x5d9, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20,
0x5e2, 0x5e8, 0x5d1, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d7, 0x5e6, 0x5d9, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e2,
0x5e8, 0x5d1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e8,
0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8,
0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de,
0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0,
0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5e8, 0x5de, 0x5e0, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5e8, 0x5de, 0x5e0, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e8, 0x5de, 0x5e0, 0x5d9, 0x5d4, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0,
0x5d5, 0x5e7, 0x5d9, 0x5d9, 0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d4, 0x5d0, 0x5d8, 0x5dc,
0x5e0, 0x5d8, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x5e7,
0x5d9, 0x5d9, 0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d4, 0x5d0, 0x5d8, 0x5dc, 0x5e0, 0x5d8,
0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d4, 0x5d0, 0x5d5, 0x5e7, 0x5d9, 0x5d9, 0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d4,
0x5d0, 0x5d8, 0x5dc, 0x5e0, 0x5d8, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5d5,
0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de,
0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9,
0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5de, 0x5e8, 0x5db, 0x5d6, 0x2d, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5,
0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de,
0x5e8, 0x5db, 0x5d6, 0x2d, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1,
0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x2d, 0x5de, 0x5e2,
0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6,
0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d5, 0x5e1,
0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d5,
0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5,
0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de,
0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9,
0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5d6, 0x5e8, 0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5f3, 0x5df, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5d6, 0x5e8, 0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5f3, 0x5df, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5d6, 0x5e8, 0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5f3, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd,
0x20, 0x5d4, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x5d9, 0x5d9, 0x5dd, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d0, 0x5d6, 0x5d5,
0x5e8, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d0,
0x5d6, 0x5d5, 0x5e8, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5e0, 0x5d2, 0x5dc, 0x5d3, 0x5e9, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5e0, 0x5d2, 0x5dc, 0x5d3, 0x5e9, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5e0,
0x5d2, 0x5dc, 0x5d3, 0x5e9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d1, 0x5d4, 0x5d5, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d1, 0x5d5, 0x5dc, 0x5d9, 0x5d1, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d1, 0x5e8, 0x5d6, 0x5d9, 0x5dc, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d1, 0x5e8, 0x5d6, 0x5d9, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5e8, 0x5d6, 0x5d9,
0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d1, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d9, 0x20, 0x5d3, 0x5d0, 0x5e8,
0x5d5, 0x5e1, 0x5dc, 0x5d0, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5db, 0x5e3,
0x20, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5db, 0x5e3,
0x20, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5db, 0x5e3, 0x20, 0x5d5, 0x5e8, 0x5d3, 0x5d4,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6,
0x5f3, 0x5d0, 0x5de, 0x5d5, 0x5e8, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6,
0x5f3, 0x5d8, 0x5d4, 0x5d0, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3,
0x5d8, 0x5d4, 0x5d0, 0x5dd, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d8, 0x5d4, 0x5d0, 0x5dd, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d9,
0x5dc, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d9, 0x5dc, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e6, 0x5f3, 0x5d9, 0x5dc, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d9, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e1, 0x5d9, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e1, 0x5d9, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5db, 0x5e8, 0x5d9,
0x5e1, 0x5d8, 0x5de, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9,
0x20, 0x5e7, 0x5d5, 0x5e7, 0x5d5, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7,
0x5d5, 0x5dc, 0x5d5, 0x5de, 0x5d1, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e7, 0x5d5, 0x5dc, 0x5d5, 0x5de, 0x5d1, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5dc, 0x5d5,
0x5de, 0x5d1, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e7, 0x5d5, 0x5e7, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9,
0x5d9, 0x20, 0x5e7, 0x5d5, 0x5e7, 0x20, 0x28, 0x5de, 0x5d7, 0x5e6, 0x5d9, 0x5ea,
0x20, 0x5d4, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7,
0x5d5, 0x5d1, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5d1, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e7, 0x5d5, 0x5d1, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5d9, 0x5e1, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d3, 0x5d5, 0x5de, 0x5d5, 0x5df, 0x20, 0x5d3, 0x5f3, 0x5d0, 0x5d5,
0x5e8, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6,
0x5e8, 0x5d7, 0x20, 0x5d8, 0x5d9, 0x5de, 0x5d5, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5e4, 0x5e1, 0x5d7, 0x5d0, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5e4, 0x5e1, 0x5d7, 0x5d0, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9,
0x20, 0x5d4, 0x5e4, 0x5e1, 0x5d7, 0x5d0, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e7, 0x5d5, 0x5d5, 0x5d3, 0x5d5, 0x5e8,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5d9,
0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db,
0x5d6, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6,
0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0,
0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6,
0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8,
0x5d7, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d9, 0x5e0, 0x5e1, 0x5e7,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d9,
0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8,
0x5d1, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1,
0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e4, 0x5d5,
0x5e7, 0x5dc, 0x5e0, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9,
0x20, 0x5e4, 0x5d5, 0x5e7, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e4,
0x5d5, 0x5e7, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5d9, 0x5d2, 0x5f3, 0x5d9, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e4, 0x5d9, 0x5d2, 0x5f3, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5d9, 0x5d2, 0x5f3, 0x5d9,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2,
0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e8, 0x5e4, 0x5ea, 0x5d9, 0x5ea,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5e8, 0x5e6, 0x5d5, 0x5ea, 0x20,
0x5d4, 0x5d3, 0x5e8, 0x5d5, 0x5de, 0x5d9, 0x5d5, 0x5ea, 0x20, 0x5d5, 0x5d4, 0x5d0,
0x5e0, 0x5d8, 0x5d0, 0x5e8, 0x5e7, 0x5d8, 0x5d9, 0x5d5, 0x5ea, 0x20, 0x5e9, 0x5dc,
0x20, 0x5e6, 0x5e8, 0x5e4, 0x5ea, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9,
0x5d9, 0x20, 0x5d2, 0x5dc, 0x5d0, 0x5e4, 0x5d2, 0x5d5, 0x5e1, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5d2, 0x5de, 0x5d1, 0x5d9, 0x5d9, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5d0, 0x5d5, 0x5e8, 0x5d2, 0x5d9, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5d0, 0x5d5, 0x5e8, 0x5d2, 0x5d9, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d2, 0x5d0, 0x5d5, 0x5e8, 0x5d2, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5d2, 0x5d9,
0x5dc, 0x5d1, 0x5e8, 0x5d8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5e8, 0x5d9,
0x5e0, 0x5d9, 0x5e5, 0x5f3, 0x200f, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6,
0x5e8, 0x5d7, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc, 0x5e0, 0x5d3, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0,
0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0,
0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc,
0x5e0, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20,
0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20,
0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5d5, 0x5d0, 0x5dd, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5d3, 0x5d9, 0x5e0, 0x5d5, 0x5ea, 0x20, 0x5d4, 0x5de,
0x5e4, 0x5e8, 0x5e5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9,
0x5dd, 0x20, 0x5d4, 0x5d0, 0x5dc, 0x5d0, 0x5d5, 0x5d8, 0x5d9, 0x5d9, 0x5dd, 0x20,
0x5d4, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0,
0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d0, 0x5dc, 0x5d0, 0x5d5, 0x5d8, 0x5d9, 0x5d9,
0x5dd, 0x20, 0x5d4, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd,
0x20, 0x5d4, 0x5d0, 0x5dc, 0x5d0, 0x5d5, 0x5d8, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4,
0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d4, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5, 0x5e0, 0x5d2,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5,
0x5e0, 0x5d2, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d4, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5, 0x5e0, 0x5d2, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5d5,
0x5d1, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5d5, 0x5d1, 0x5d3, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7,
0x5d5, 0x5d1, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d4, 0x5d5, 0x5d3, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4,
0x5d0, 0x5d5, 0x5e7, 0x5d9, 0x5d9, 0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d4, 0x5d4, 0x5d5,
0x5d3, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d5, 0x5d3, 0x5d5, 0x2d,
0x5e1, 0x5d9, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6,
0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d5, 0x5e0, 0x5d6, 0x5d9, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3,
0x5d5, 0x5e0, 0x5d6, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2,
0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d5, 0x5e0, 0x5d6, 0x5d9, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d0, 0x5df, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d0, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d0,
0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5d9, 0x5e8, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5d9, 0x5e8, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e8,
0x5e7, 0x5d5, 0x5e1, 0x5d8, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e9, 0x5e8, 0x5d0, 0x5dc, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d9, 0x5e9, 0x5e8, 0x5d0, 0x5dc, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e9, 0x5e8, 0x5d0, 0x5dc,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9,
0x5e4, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e4, 0x5df, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e4,
0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e4, 0x5d8, 0x5e8, 0x5d5, 0x5e4, 0x5d1, 0x5dc, 0x5d5, 0x5d1, 0x5e1, 0x5e7, 0x2d,
0x5e7, 0x5de, 0x5e6, 0x5f3, 0x5d8, 0x5e1, 0x5e7, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e8, 0x5d2, 0x5d9, 0x5dc, 0x20, 0x5e4, 0x5d8, 0x5e8, 0x5d5, 0x5e4, 0x5d1,
0x5dc, 0x5d5, 0x5d1, 0x5e1, 0x5e7, 0x2d, 0x5e7, 0x5de, 0x5e6, 0x5f3, 0x5d8, 0x5e1,
0x5e7, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5e4,
0x5d8, 0x5e8, 0x5d5, 0x5e4, 0x5d1, 0x5dc, 0x5d5, 0x5d1, 0x5e1, 0x5e7, 0x2d, 0x5e7,
0x5de, 0x5e6, 0x5f3, 0x5d8, 0x5e1, 0x5e7, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e7, 0x5d6, 0x5d7, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de,
0x5d6, 0x5e8, 0x5d7, 0x20, 0x5e7, 0x5d6, 0x5d7, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5e7, 0x5d6, 0x5d7, 0x5e1,
0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5e8, 0x5d9, 0x5d0,
0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5e8, 0x5d9, 0x5d0, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e7, 0x5d5, 0x5e8, 0x5d9, 0x5d0, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5e1, 0x5e8, 0x5d0, 0x5d4, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5e8, 0x5e1, 0x5e0, 0x5d5, 0x5d9, 0x5d0, 0x5e8,
0x5e1, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5e8, 0x5e1, 0x5e0, 0x5d5,
0x5d9, 0x5d0, 0x5e8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5e8, 0x5e1, 0x5e0, 0x5d5, 0x5d9, 0x5d0,
0x5e8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e7, 0x5d9, 0x5e8, 0x5d2, 0x5d9, 0x5d6, 0x5e1, 0x5d8, 0x5df, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5dc, 0x5d9, 0x5d9, 0x5df,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5dc, 0x5d5, 0x5e8,
0x5d3, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9,
0x20, 0x5d4, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9,
0x20, 0x5d4, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e7, 0x5d0,
0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x20, 0x5de,
0x5e7, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5, 0x20,
0x5de, 0x5e7, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d2, 0x5d3,
0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d2, 0x5d3, 0x5df, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d2,
0x5d3, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5dc, 0x5d6, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4,
0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5de, 0x5dc, 0x5d3, 0x5d9, 0x5d1, 0x5d9,
0x5d9, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5de,
0x5e8, 0x5e7, 0x5d9, 0x5d6, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9,
0x20, 0x5de, 0x5e8, 0x5e9, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d0,
0x5d5, 0x5e8, 0x5d9, 0x5e6, 0x5d9, 0x5d5, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5de, 0x5d0, 0x5d5, 0x5e8, 0x5d9, 0x5e6, 0x5d9, 0x5d5, 0x5e1, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d0, 0x5d5,
0x5e8, 0x5d9, 0x5e6, 0x5d9, 0x5d5, 0x5e1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d0, 0x5d5, 0x5e1, 0x5d5, 0x5df, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5de, 0x5e7, 0x5e1,
0x5d9, 0x5e7, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1,
0x20, 0x5de, 0x5e7, 0x5e1, 0x5d9, 0x5e7, 0x5d5, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20,
0x5de, 0x5e7, 0x5e1, 0x5d9, 0x5e7, 0x5d5, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5dc, 0x5d0, 0x5df, 0x20, 0x5d1,
0x5d0, 0x5d8, 0x5d5, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5dc,
0x5d0, 0x5df, 0x20, 0x5d1, 0x5d0, 0x5d8, 0x5d5, 0x5e8, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5dc, 0x5d0,
0x5df, 0x20, 0x5d1, 0x5d0, 0x5d8, 0x5d5, 0x5e8, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d1, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d1, 0x5d4, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de,
0x5d5, 0x5e1, 0x5e7, 0x5d1, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d9, 0x5d0, 0x5e0, 0x5de, 0x5e8, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e0, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e0, 0x5e4, 0x5d0, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5dc,
0x5d3, 0x5d5, 0x5e0, 0x5d9, 0x5d4, 0x20, 0x5d4, 0x5d7, 0x5d3, 0x5e9, 0x5d4, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5dc, 0x5d3, 0x5d5, 0x5e0, 0x5d9, 0x5d4, 0x20,
0x5d4, 0x5d7, 0x5d3, 0x5e9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5dc, 0x5d3, 0x5d5, 0x5e0, 0x5d9, 0x5d4,
0x20, 0x5d4, 0x5d7, 0x5d3, 0x5e9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x20, 0x5d6, 0x5d9, 0x5dc,
0x5e0, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x20, 0x5d6,
0x5d9, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x20, 0x5d6, 0x5d9, 0x5dc, 0x5e0,
0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e0, 0x5d9, 0x5d5, 0x5e4, 0x5d0, 0x5d5, 0x5e0, 0x5d3, 0x5dc, 0x5e0, 0x5d3, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x5e4, 0x5d0, 0x5d5, 0x5e0, 0x5d3,
0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x5e4, 0x5d0, 0x5d5, 0x5e0, 0x5d3, 0x5dc,
0x5e0, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e0, 0x5d9, 0x5d5, 0x5d0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4,
0x5d0, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e7, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e7,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e7, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5e0,
0x5e0, 0x5d3, 0x5d5, 0x20, 0x5d3, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5d5, 0x5e0,
0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5e0, 0x5e0, 0x5d3,
0x5d5, 0x20, 0x5d3, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e4, 0x5e8, 0x5e0, 0x5e0, 0x5d3, 0x5d5, 0x20, 0x5d3, 0x5d9, 0x20, 0x5e0, 0x5d5,
0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5de, 0x5e8, 0x5d9, 0x5d0,
0x5e0, 0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e4, 0x5d5, 0x5e0, 0x5d9, 0x5d9, 0x5dd, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e1, 0x5d9, 0x5d1, 0x5d9,
0x5e8, 0x5e1, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d5, 0x5d1, 0x5d5,
0x5e1, 0x5d9, 0x5d1, 0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e1,
0x5d9, 0x5d1, 0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5de, 0x5e1, 0x5e7, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5de, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5de, 0x5e1,
0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e4, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4,
0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5dc,
0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e4, 0x5d5, 0x5d0, 0x5d4,
0x20, 0x5d2, 0x5d9, 0x5e0, 0x5d9, 0x5d0, 0x5d4, 0x20, 0x5d4, 0x5d7, 0x5d3, 0x5e9,
0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d2, 0x5d5, 0x5d5, 0x5d0,
0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d2, 0x5d5, 0x5d5, 0x5d0,
0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e4, 0x5e8, 0x5d2, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d5, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d5, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d5, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5e4, 0x5d9, 0x5dc,
0x5d9, 0x5e4, 0x5d9, 0x5e0, 0x5d9, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4,
0x5e4, 0x5d9, 0x5dc, 0x5d9, 0x5e4, 0x5d9, 0x5e0, 0x5d9, 0x5dd, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5e4, 0x5d9,
0x5dc, 0x5d9, 0x5e4, 0x5d9, 0x5e0, 0x5d9, 0x5dd, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e4, 0x5d9,
0x5e0, 0x5d9, 0x5e7, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5e0, 0x5d8,
0x20, 0x5e4, 0x5d9, 0x5d9, 0x5e8, 0x20, 0x5d5, 0x5de, 0x5d9, 0x5e7, 0x5dc, 0x5d5,
0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5e4, 0x5d9,
0x5d9, 0x5e8, 0x20, 0x5d5, 0x5de, 0x5d9, 0x5e7, 0x5dc, 0x5d5, 0x5df, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5e0,
0x5d8, 0x20, 0x5e4, 0x5d9, 0x5d9, 0x5e8, 0x20, 0x5d5, 0x5de, 0x5d9, 0x5e7, 0x5dc,
0x5d5, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e4, 0x5d9, 0x5d8, 0x5e7, 0x5e8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e4, 0x5d5, 0x5e0, 0x5d0, 0x5e4, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4,
0x5d9, 0x5d5, 0x5e0, 0x5d2, 0x5d9, 0x5d0, 0x5e0, 0x5d2, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e7, 0x5d9, 0x5d6, 0x5d9, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5d6, 0x5d9, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e7, 0x5d9, 0x5d6, 0x5d9, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8, 0x5d0, 0x5d5, 0x5e0,
0x5d9, 0x5d5, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8, 0x5d5, 0x5ea, 0x5f3,
0x5e8, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d7, 0x5dc, 0x5d9, 0x5df,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d7, 0x5dc, 0x5d9, 0x5df, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d7,
0x5dc, 0x5d9, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8,
0x5d2, 0x5d9, 0x5dc, 0x20, 0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e1, 0x5de, 0x5d5, 0x5d0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e1, 0x5de, 0x5d5, 0x5d0, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5de, 0x5d5, 0x5d0, 0x5d4, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9,
0x20, 0x5e1, 0x5d9, 0x5d9, 0x5e9, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1,
0x5d9, 0x5e0, 0x5d2, 0x5e4, 0x5d5, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5d9, 0x5d9, 0x20, 0x5e9, 0x5dc, 0x5de, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d3, 0x5e8, 0x5d5, 0x5dd, 0x20, 0x5d2, 0x5f3, 0x5d5, 0x5e8, 0x5d2, 0x5f3, 0x5d9,
0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d5, 0x5e8, 0x5d9, 0x5e0, 0x5d0,
0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d9, 0x5d9, 0x5d5, 0x5d5, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d4, 0x5d9, 0x5d8, 0x5d9, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d8, 0x5d0, 0x5d9, 0x5e4, 0x5d9, 0x5d9, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d8, 0x5d0, 0x5d9, 0x5e4, 0x5d9, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d0, 0x5d9, 0x5e4,
0x5d9, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d8, 0x5d2, 0x5f3, 0x5d9, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e7, 0x5dc, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d8, 0x5d5, 0x5e0, 0x5d2, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d8, 0x5d5, 0x5e0, 0x5d2, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e0, 0x5d2, 0x5d4, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d5,
0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e8, 0x5e7, 0x5de, 0x5e0,
0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e8,
0x5e7, 0x5de, 0x5e0, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e8, 0x5e7, 0x5de,
0x5e0, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5d1, 0x5d0, 0x5dc, 0x5d5, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5d2, 0x5d5, 0x5d5, 0x5d0, 0x5d9,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5d2, 0x5d5, 0x5d5,
0x5d0, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5d2, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5,
0x5d6, 0x5d1, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5d5, 0x5d6, 0x5d1, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5d6,
0x5d1, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5e0, 0x5d5, 0x5d0, 0x5d8, 0x5d5, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5e0, 0x5d5, 0x5d0, 0x5d8, 0x5d5, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5e0,
0x5d5, 0x5d0, 0x5d8, 0x5d5, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d5, 0x5e0, 0x5e6, 0x5d5, 0x5d0, 0x5dc, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5e1, 0x5d8, 0x5d5,
0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5, 0x5d5,
0x5e1, 0x5d8, 0x5d5, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5e1, 0x5d8,
0x5d5, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d5, 0x5d5, 0x5dc, 0x5d2, 0x5d5, 0x5d2, 0x5e8, 0x5d3, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d5, 0x5d5, 0x5dc, 0x5d2, 0x5d5, 0x5d2, 0x5e8, 0x5d3, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5d5,
0x5dc, 0x5d2, 0x5d5, 0x5d2, 0x5e8, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5d5, 0x5e1, 0x5d8, 0x5d5, 0x5e7, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5d5, 0x5d9, 0x5d9, 0x5e7,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5d5, 0x5d0, 0x5dc, 0x5d9, 0x5e1, 0x20,
0x5d5, 0x5e4, 0x5d5, 0x5d8, 0x5d5, 0x5e0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d9, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9,
0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e7,
0x5d8, 0x5e8, 0x5d9, 0x5e0, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d9, 0x5e7, 0x5d8, 0x5e8, 0x5d9, 0x5e0, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9,
0x5e7, 0x5d8, 0x5e8, 0x5d9, 0x5e0, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5d5, 0x5e7, 0x5d5,
0x5df, 0x905, 0x92b, 0x93c, 0x917, 0x93e, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x92b,
0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942,
0x930, 0x94d, 0x935, 0x940, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20,
0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20,
0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930,
0x940, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930,
0x940, 0x915, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x93e, 0x938,
0x94d, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x93e, 0x938, 0x94d,
0x200d, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x905, 0x932, 0x93e, 0x938, 0x94d, 0x200d, 0x915, 0x93e, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x92e, 0x947, 0x91c, 0x93c,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x92e, 0x947, 0x91c, 0x93c, 0x928, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x92e, 0x947, 0x91c,
0x93c, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930,
0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x915, 0x947,
0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924,
0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940,
0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940,
0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x915, 0x947, 0x902,
0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905,
0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935,
0x940, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x942, 0x930, 0x94d,
0x935, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x940, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930,
0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92e, 0x93e,
0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20,
0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905,
0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x909, 0x902, 0x91f,
0x947, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930,
0x93f, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20,
0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e,
0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902,
0x924, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924,
0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940,
0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x928, 0x93e, 0x921, 0x940,
0x92f, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x928, 0x93e, 0x921, 0x940, 0x92f,
0x930, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x928,
0x93e, 0x921, 0x940, 0x92f, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92a,
0x93f, 0x906, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92a, 0x93f, 0x906, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930,
0x92c, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x92c, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x92c, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x91c, 0x947,
0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d,
0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940,
0x928, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940,
0x928, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x906, 0x930, 0x94d, 0x92e, 0x947,
0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x906, 0x930, 0x94d, 0x92e,
0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91f, 0x932,
0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x911, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x938, 0x92e,
0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947,
0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d,
0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926,
0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d,
0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x908, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x930,
0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c,
0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938,
0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x94b,
0x930, 0x947, 0x938, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x93e, 0x902, 0x917,
0x94d, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x93e,
0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e,
0x926, 0x947, 0x936, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x942, 0x91f, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94b, 0x932, 0x940, 0x935, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x940,
0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93e,
0x91c, 0x93c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x940, 0x932,
0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x942,
0x928, 0x947, 0x908, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938, 0x94d, 0x938, 0x932,
0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d,
0x921, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d,
0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947,
0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91a,
0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x91a, 0x948, 0x925, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x948,
0x925, 0x92e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91a,
0x948, 0x925, 0x92e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938,
0x92e, 0x92f, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93f,
0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91a,
0x93f, 0x932, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x91a, 0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x91a, 0x940, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b,
0x915, 0x94b, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942,
0x939, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b,
0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939,
0x20, 0x938, 0x92e, 0x92f, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938,
0x92e, 0x942, 0x939, 0x20, 0x905, 0x930, 0x94d, 0x926, 0x94d, 0x927, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x921, 0x947, 0x935,
0x93f, 0x938, 0x20, 0x938, 0x92e, 0x92f, 0x921, 0x94d, 0x92f, 0x942, 0x92e, 0x94b,
0x902, 0x91f, 0x20, 0x921, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x935, 0x93f, 0x932,
0x947, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20,
0x924, 0x93f, 0x92e, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x938, 0x94d,
0x91f, 0x930, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x92f,
0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x938, 0x94d, 0x91f,
0x930, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x907, 0x915, 0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f,
0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f,
0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930,
0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92e, 0x927, 0x94d, 0x200d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x940, 0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x200d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d,
0x935, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x917, 0x94d, 0x930,
0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x942,
0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x942,
0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x200d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b,
0x93c, 0x949, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x949,
0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938,
0x92e, 0x942, 0x939, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x92b, 0x93c, 0x949, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b,
0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f, 0x91c,
0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c,
0x93f, 0x91c, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x94d, 0x930,
0x947, 0x902, 0x91a, 0x20, 0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x940, 0x20, 0x92b, 0x93c,
0x94d, 0x930, 0x93e, 0x902, 0x938, 0x20, 0x914, 0x930, 0x20, 0x905, 0x902, 0x91f,
0x93e, 0x930, 0x94d, 0x915, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x917,
0x948, 0x932, 0x93e, 0x92a, 0x947, 0x917, 0x94b, 0x938, 0x20, 0x915, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x917, 0x948, 0x902, 0x92c, 0x93f, 0x92f, 0x930, 0x20, 0x938,
0x92e, 0x92f, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x949, 0x930, 0x94d, 0x91c,
0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x917, 0x93f, 0x932, 0x94d,
0x92c, 0x930, 0x94d, 0x91f, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e,
0x942, 0x939, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948,
0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x932, 0x948, 0x902, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x916, 0x93e, 0x921,
0x93c, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939,
0x935, 0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f,
0x942, 0x936, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x939, 0x935, 0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x939,
0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x938, 0x92e, 0x92f,
0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915,
0x93e, 0x901, 0x917, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x935, 0x94d,
0x921, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x935, 0x94d, 0x921,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x93e, 0x930, 0x924, 0x940, 0x92f, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93f, 0x902, 0x926,
0x20, 0x92e, 0x939, 0x93e, 0x938, 0x93e, 0x917, 0x930, 0x20, 0x938, 0x92e, 0x92f,
0x907, 0x902, 0x921, 0x94b, 0x91a, 0x93e, 0x907, 0x928, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947,
0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d,
0x935, 0x940, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x908, 0x930, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x908,
0x930, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x908, 0x930, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x938, 0x92e, 0x92f, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x938,
0x92e, 0x92f, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907,
0x932, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f,
0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x93e, 0x92a,
0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91c,
0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x947, 0x935,
0x932, 0x93e, 0x938, 0x94d, 0x915, 0x2d, 0x915, 0x948, 0x92e, 0x91a, 0x93e, 0x924,
0x94d, 0x938, 0x915, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x91f, 0x94d,
0x930, 0x94b, 0x92a, 0x947, 0x935, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x2d, 0x915,
0x948, 0x92e, 0x91a, 0x93e, 0x924, 0x94d, 0x938, 0x915, 0x940, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b,
0x92a, 0x947, 0x935, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x2d, 0x915, 0x948, 0x92e,
0x91a, 0x93e, 0x924, 0x94d, 0x938, 0x915, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x91c, 0x93c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x915, 0x91c, 0x93c, 0x93e,
0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x915, 0x91c, 0x93c, 0x93e, 0x916, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f, 0x92f,
0x93e, 0x908, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e,
0x908, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b,
0x930, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x938, 0x930, 0x93e, 0x90f, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930,
0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930,
0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x93f, 0x930, 0x94d, 0x917, 0x93f, 0x938,
0x94d, 0x200d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x932, 0x93e, 0x907,
0x928, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20,
0x938, 0x92e, 0x92f, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935,
0x947, 0x20, 0x938, 0x92e, 0x92f, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939,
0x94b, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x917,
0x93e, 0x926, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x917, 0x93e,
0x926, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92e, 0x932, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92e,
0x93e, 0x932, 0x926, 0x940, 0x935, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930,
0x94d, 0x915, 0x947, 0x938, 0x938, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930,
0x94d, 0x936, 0x932, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942,
0x939, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20,
0x938, 0x92e, 0x92f, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x935, 0x94d, 0x938, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20,
0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x938, 0x92e, 0x92f, 0x92e,
0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936,
0x93e, 0x902, 0x924, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x936, 0x93e, 0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x938, 0x92e, 0x92f, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930,
0x20, 0x938, 0x92e, 0x92f, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b,
0x930, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x932,
0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x949,
0x938, 0x94d, 0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92e, 0x94d, 0x92f, 0x93e, 0x902, 0x92e, 0x93e, 0x930, 0x20, 0x938, 0x92e, 0x92f,
0x928, 0x94c, 0x930, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x947, 0x92a, 0x93e,
0x932, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x948,
0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928,
0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928,
0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942,
0x91c, 0x93c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x928,
0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c,
0x93c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93c, 0x93e,
0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x928,
0x94d, 0x92f, 0x942, 0x92b, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902,
0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d,
0x92f, 0x942, 0x92b, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x928,
0x940, 0x92f, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x949, 0x930, 0x92b, 0x93c,
0x949, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x92f,
0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x949,
0x930, 0x92b, 0x93c, 0x949, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c,
0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947,
0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x92b, 0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b,
0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x930,
0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20,
0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f,
0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x913, 0x92e,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x913, 0x92e, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x913,
0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x932, 0x93e, 0x909, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x92a, 0x941, 0x906, 0x20, 0x928, 0x94d, 0x92f,
0x942, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x948,
0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x948,
0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x947, 0x930, 0x942, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c,
0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c,
0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x940, 0x928, 0x93f, 0x915, 0x94d, 0x938,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938,
0x92e, 0x92f, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947,
0x20, 0x914, 0x930, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f,
0x90f, 0x930, 0x947, 0x20, 0x914, 0x930, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935,
0x947, 0x932, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20,
0x914, 0x930, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x93f, 0x91f, 0x915, 0x948, 0x930, 0x94d, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x94b, 0x928, 0x93e, 0x92a, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94d, 0x92f,
0x94b, 0x902, 0x917, 0x92f, 0x93e, 0x902, 0x917, 0x20, 0x938, 0x92e, 0x92f, 0x930,
0x940, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x930,
0x94b, 0x925, 0x947, 0x930, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x916, 0x93e,
0x932, 0x93f, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x916, 0x93e, 0x932, 0x93f,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x916,
0x93e, 0x932, 0x93f, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x92e, 0x93e,
0x930, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x92e, 0x93e, 0x930,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x938,
0x92e, 0x92f, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x936, 0x947, 0x932, 0x94d,
0x938, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x941,
0x930, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x928, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938, 0x92e,
0x92f, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x940, 0x20, 0x91c, 0x949, 0x930,
0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x942, 0x930,
0x940, 0x928, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x94d, 0x92f, 0x94b,
0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x939, 0x93f, 0x924, 0x940,
0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x938, 0x92e,
0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x91c, 0x93f, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x94b,
0x915, 0x947, 0x932, 0x93e, 0x90a, 0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x94b, 0x902,
0x917, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x94b, 0x902, 0x917,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x941, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x941, 0x930, 0x94d, 0x915,
0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e,
0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x924, 0x941, 0x935, 0x93e, 0x932, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x909,
0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x930,
0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x93c, 0x94d, 0x92c,
0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935,
0x928, 0x941, 0x906, 0x924, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x928, 0x941,
0x906, 0x924, 0x942, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x935, 0x928, 0x941, 0x906, 0x924, 0x942, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935,
0x947, 0x928, 0x947, 0x91c, 0x93c, 0x941, 0x90f, 0x932, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924,
0x94b, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f,
0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b,
0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x94b,
0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x938, 0x92e,
0x92f, 0x935, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x94b, 0x932,
0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x935, 0x947, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938,
0x92e, 0x92f, 0x935, 0x93e, 0x932, 0x93f, 0x938, 0x20, 0x914, 0x930, 0x20, 0x92b,
0x93c, 0x94d, 0x92f, 0x942, 0x91a, 0x942, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915, 0x93e,
0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x938, 0x92e,
0x92f, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930,
0x94d, 0x917, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92f,
0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x941, 0x915, 0x949, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x69, 0x64,
0x151, 0x41, 0x63, 0x72, 0x65, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64,
0x151, 0x41, 0x63, 0x72, 0x65, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x61, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x7a, 0x74,
0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70,
0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x64, 0xe9, 0x6c, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e,
0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6e, 0x79, 0x75,
0x67, 0x61, 0x74, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67,
0x61, 0x74, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x6c, 0x61, 0x73,
0x7a, 0x6b, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x6c, 0x61, 0x73,
0x7a, 0x6b, 0x61, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151,
0x61, 0x6c, 0x61, 0x73, 0x7a, 0x6b, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x69, 0x64, 0x151, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x7a,
0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x6d,
0x61, 0x7a, 0xf3, 0x6e, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61,
0x6d, 0x61, 0x7a, 0xf3, 0x6e, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x6d, 0x61, 0x7a, 0xf3, 0x6e, 0x69,
0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x73, 0x151, 0x20, 0xe1, 0x6c, 0x6c, 0x61,
0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b,
0xf6, 0x7a, 0xe9, 0x70, 0x73, 0x151, 0x20, 0xe1, 0x6c, 0x6c, 0x61, 0x6d,
0x6f, 0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69,
0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x73, 0x151, 0x20, 0xe1, 0x6c,
0x6c, 0x61, 0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74,
0x69, 0x20, 0xe1, 0x6c, 0x6c, 0x61, 0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x69, 0x20,
0xe1, 0x6c, 0x6c, 0x61, 0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20,
0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74,
0x69, 0x20, 0xe1, 0x6c, 0x6c, 0x61, 0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68,
0x65, 0x67, 0x79, 0x76, 0x69, 0x64, 0xe9, 0x6b, 0x69, 0x20, 0x69, 0x64,
0x151, 0x68, 0x65, 0x67, 0x79, 0x76, 0x69, 0x64, 0xe9, 0x6b, 0x69, 0x20,
0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x68, 0x65, 0x67, 0x79, 0x76,
0x69, 0x64, 0xe9, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x63, 0x73, 0x65, 0x6e, 0x64, 0x65, 0x73, 0x2d, 0xf3,
0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x73, 0x65,
0x6e, 0x64, 0x65, 0x73, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20,
0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x63, 0x73, 0x65, 0x6e, 0x64,
0x65, 0x73, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x41, 0x6e, 0x61, 0x64, 0x69,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x41, 0x6e, 0x61, 0x64, 0xed, 0x72,
0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x41, 0x6e, 0x61,
0x64, 0xed, 0x72, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x61, 0x70, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61,
0x70, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x61, 0x70, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x41, 0x71, 0x74, 0x61, 0x75, 0x69, 0x20, 0x69,
0x64, 0x151, 0x41, 0x71, 0x74, 0x61, 0x75, 0x69, 0x20, 0x7a, 0xf3, 0x6e,
0x61, 0x69, 0x64, 0x151, 0x41, 0x71, 0x74, 0x61, 0x75, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x69, 0x20, 0x69, 0x64, 0x151, 0x41, 0x71, 0x74, 0x6f, 0x62,
0x65, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x41, 0x71,
0x74, 0x6f, 0x62, 0x65, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x61, 0x72, 0x61, 0x62, 0x20, 0x69, 0x64, 0x151, 0x61,
0x72, 0x61, 0x62, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x61, 0x72, 0x61, 0x62, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61,
0x74, 0x2d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6e, 0x79, 0x75, 0x67,
0x61, 0x74, 0x2d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79,
0x75, 0x67, 0x61, 0x74, 0x2d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed,
0x6e, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0xf6, 0x72, 0x6d, 0xe9, 0x6e, 0x79, 0x6f, 0x72, 0x73, 0x7a, 0xe1,
0x67, 0x69, 0x20, 0x69, 0x64, 0x151, 0xf6, 0x72, 0x6d, 0xe9, 0x6e, 0x79,
0x6f, 0x72, 0x73, 0x7a, 0xe1, 0x67, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0xf6, 0x72, 0x6d, 0xe9, 0x6e, 0x79, 0x6f, 0x72,
0x73, 0x7a, 0xe1, 0x67, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x2d, 0xf3,
0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20,
0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x61, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70,
0x2d, 0x61, 0x75, 0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x61, 0x75,
0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d,
0x61, 0x75, 0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a,
0xe9, 0x70, 0x2d, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x61, 0x75,
0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x6e, 0x79, 0x75, 0x67, 0x61,
0x74, 0x2d, 0x61, 0x75, 0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6,
0x7a, 0xe9, 0x70, 0x2d, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x61,
0x75, 0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65,
0x74, 0x2d, 0x61, 0x75, 0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x61,
0x75, 0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74,
0x2d, 0x61, 0x75, 0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a, 0x73, 0xe1, 0x6e, 0x69, 0x20,
0x69, 0x64, 0x151, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a,
0x73, 0xe1, 0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a, 0x73, 0xe1,
0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x61, 0x7a, 0x6f, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e,
0x61, 0x61, 0x7a, 0x6f, 0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x61, 0x7a, 0x6f, 0x72, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62,
0x75, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x6f, 0x6c,
0xed, 0x76, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x62, 0x72, 0x61, 0x7a, 0xed, 0x6c, 0x69, 0x61, 0x69, 0x20,
0x69, 0x64, 0x151, 0x62, 0x72, 0x61, 0x7a, 0xed, 0x6c, 0x69, 0x61, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x72, 0x61,
0x7a, 0xed, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x2d, 0x69, 0x20,
0x69, 0x64, 0x151, 0x7a, 0xf6, 0x6c, 0x64, 0x2d, 0x66, 0x6f, 0x6b, 0x69,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69,
0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x7a, 0xf6, 0x6c, 0x64, 0x2d, 0x66,
0x6f, 0x6b, 0x69, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf6,
0x6c, 0x64, 0x2d, 0x66, 0x6f, 0x6b, 0x69, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x63, 0x61, 0x73, 0x65, 0x79, 0x2d, 0x69, 0x20, 0x69,
0x64, 0x151, 0x63, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0xf3, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x68, 0x69, 0x6c, 0x65,
0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x63, 0x68, 0x69,
0x6c, 0x65, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x63, 0x68, 0x69, 0x6c, 0x65, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6b, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x61, 0x72, 0xe1, 0x63, 0x73,
0x6f, 0x6e, 0x79, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0xf3, 0x6b, 0x75, 0x73, 0x7a, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x6f, 0x6f, 0x6b, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151,
0x63, 0x6f, 0x6f, 0x6b, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65,
0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63,
0x6f, 0x6f, 0x6b, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x69, 0x20, 0x66, 0xe9, 0x6c, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x75, 0x62, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151,
0x7a, 0xf3, 0x6e, 0x61, 0x6b, 0x75, 0x62, 0x61, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x75, 0x62, 0x61, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x64, 0x61, 0x76,
0x69, 0x73, 0x69, 0x20, 0x69, 0x64, 0x151, 0x64, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x74,
0x69, 0x6d, 0x6f, 0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x68, 0xfa, 0x73, 0x76, 0xe9, 0x74, 0x2d, 0x73, 0x7a, 0x69,
0x67, 0x65, 0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61,
0x68, 0xfa, 0x73, 0x76, 0xe9, 0x74, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68,
0xfa, 0x73, 0x76, 0xe9, 0x74, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x65,
0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x65, 0x75,
0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e,
0x61, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b,
0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65,
0x6c, 0x65, 0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69, 0x20,
0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6b, 0x65, 0x6c, 0x65, 0x74,
0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x65,
0x75, 0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6d, 0x69, 0x6e, 0x73, 0x7a, 0x6b, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x65, 0x75,
0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e,
0x61, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3,
0x70, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70,
0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x7a, 0x69,
0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x65, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x69, 0x64, 0x7a, 0x73, 0x69, 0x20,
0x69, 0x64, 0x151, 0x66, 0x69, 0x64, 0x7a, 0x73, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x69, 0x64, 0x7a, 0x73, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x72,
0x61, 0x6e, 0x63, 0x69, 0x61, 0x2d, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x61,
0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x61,
0x20, 0x64, 0xe9, 0x6c, 0x69, 0x20, 0xe9, 0x73, 0x20, 0x61, 0x6e, 0x74,
0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x7a, 0x69, 0x20, 0x69, 0x64, 0x151,
0x67, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x72, 0xfa, 0x7a, 0x69,
0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x72, 0xfa, 0x7a, 0x69, 0x61,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x72,
0xfa, 0x7a, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x67, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151,
0x67, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x69, 0x20, 0x6b,
0xf6, 0x7a, 0xe9, 0x70, 0x69, 0x64, 0x151, 0x2c, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x67,
0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x69, 0x64, 0x151,
0x7a, 0xf3, 0x6e, 0x61, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d,
0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x69, 0x64,
0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d,
0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74,
0x2d, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x47, 0x75, 0x61, 0x6d,
0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0xf6, 0x62, 0xf6,
0x6c, 0x62, 0x65, 0x6c, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a,
0xf3, 0x6e, 0x61, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x61, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x69, 0x64,
0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x6f, 0x76, 0x64, 0x69, 0x20,
0x69, 0x64, 0x151, 0x68, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x6e, 0x64, 0x69,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69,
0x6e, 0x64, 0x69, 0x61, 0x69, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69,
0x20, 0x69, 0x64, 0x151, 0x69, 0x6e, 0x64, 0x6f, 0x6b, 0xed, 0x6e, 0x61,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x7a, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0xe9, 0x7a, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75,
0x67, 0x61, 0x74, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x7a, 0x69,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69,
0x72, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x72, 0xe1, 0x6e,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x72,
0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69,
0x64, 0x151, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a,
0x61, 0x70, 0xe1, 0x6e, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61, 0x70, 0xe1,
0x6e, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61,
0x70, 0xe1, 0x6e, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x7a, 0x6b, 0x2d, 0x6b, 0x61, 0x6d, 0x63, 0x73, 0x61, 0x74, 0x6b,
0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x7a, 0x6b, 0x2d, 0x6b, 0x61, 0x6d,
0x63, 0x73, 0x61, 0x74, 0x6b, 0x61, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61,
0x69, 0x64, 0x151, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x7a, 0x6b, 0x2d, 0x6b, 0x61, 0x6d, 0x63, 0x73, 0x61,
0x74, 0x6b, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x7a, 0x74, 0xe1, 0x6e,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x6b,
0x61, 0x7a, 0x61, 0x68, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x6b, 0x61, 0x7a,
0x61, 0x68, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6b, 0x6f, 0x72, 0x65, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x6f,
0x72, 0x65, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6b, 0x72, 0x61, 0x73, 0x7a, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x72,
0x61, 0x73, 0x7a, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x7a, 0x6b, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x72, 0x61,
0x73, 0x7a, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x7a, 0x6b, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x69, 0x72,
0x67, 0x69, 0x7a, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69,
0x64, 0x151, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151,
0x73, 0x6f, 0x72, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x69, 0x20, 0x69, 0x64, 0x151, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x69,
0x64, 0x151, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x4d, 0x61, 0x63, 0x61, 0x75,
0x69, 0x20, 0x69, 0x64, 0x151, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x69, 0x20,
0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x4d, 0x61, 0x63, 0x61, 0x75,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d,
0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61,
0x6c, 0x61, 0x6a, 0x7a, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d,
0x61, 0x6c, 0x64, 0xed, 0x76, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x72, 0x71, 0x75,
0x69, 0x73, 0x65, 0x73, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65,
0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6d, 0x65, 0x78, 0x69, 0x6b, 0xf3, 0x69, 0x20, 0x63, 0x73, 0x65, 0x6e,
0x64, 0x65, 0x73, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6d, 0x65, 0x78, 0x69, 0x6b, 0xf3, 0x69, 0x20, 0x63, 0x73,
0x65, 0x6e, 0x64, 0x65, 0x73, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69,
0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x6d, 0x65, 0x78, 0x69,
0x6b, 0xf3, 0x69, 0x20, 0x63, 0x73, 0x65, 0x6e, 0x64, 0x65, 0x73, 0x2d,
0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x75, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x6f,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x75, 0x6c, 0xe1, 0x6e, 0x62, 0xe1,
0x74, 0x6f, 0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x75, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x6f, 0x72, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x6f, 0x73,
0x7a, 0x6b, 0x76, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x6f, 0x73,
0x7a, 0x6b, 0x76, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6d, 0x6f, 0x73, 0x7a, 0x6b, 0x76, 0x61, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x69, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x61, 0x75, 0x72,
0x75, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x65, 0x70, 0xe1, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0xf3, 0x6e, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d,
0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x7a, 0xe9, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x7a,
0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa,
0x6a, 0x2d, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20,
0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0xfa,
0x6a, 0x2d, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x69, 0x75,
0x65, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x2d, 0x69, 0x20, 0x69, 0x64, 0x151, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0xc9, 0x73,
0x7a, 0x61, 0x6b, 0x2d, 0x6d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x7a, 0x69, 0x62, 0x69, 0x72, 0x73,
0x7a, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f, 0x76, 0x6f, 0x73,
0x7a, 0x69, 0x62, 0x69, 0x72, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x7a,
0x69, 0x62, 0x69, 0x72, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6f, 0x6d, 0x73, 0x7a, 0x6b, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6f, 0x6d, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6f, 0x6d, 0x73, 0x7a, 0x6b,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70,
0x61, 0x6b, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64,
0x151, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x61, 0x6b, 0x69,
0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x70, 0x61, 0x6c, 0x61, 0x75, 0x69, 0x20, 0x69,
0x64, 0x151, 0x70, 0xe1, 0x70, 0x75, 0x61, 0x20, 0xfa, 0x6a, 0x2d, 0x67,
0x75, 0x69, 0x6e, 0x65, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x70, 0x65, 0x72, 0x75, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x65, 0x72,
0x75, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70,
0x65, 0x72, 0x75, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x66, 0xfc, 0x6c, 0xf6, 0x70, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0xfc, 0x6c,
0xf6, 0x70, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0xfc, 0x6c,
0xf6, 0x70, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x68,
0x6f, 0x65, 0x6e, 0x69, 0x78, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0xe9, 0x73, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x2d, 0x69, 0x20, 0x69, 0x64, 0x151, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0xe9, 0x73, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x69, 0x20, 0x7a, 0xf3, 0x6e,
0x61, 0x69, 0x64, 0x151, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0xe9, 0x73, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x70, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69,
0x64, 0x151, 0x70, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x73, 0x7a, 0x69,
0x67, 0x65, 0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x68, 0x65, 0x6e,
0x6a, 0x61, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x51, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x51, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x69, 0x20, 0x7a, 0xf3, 0x6e,
0x61, 0x69, 0x64, 0x151, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x72, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151,
0x72, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151,
0x73, 0x7a, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x69, 0x64,
0x151, 0x73, 0x7a, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x7a, 0x61, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x69, 0x20, 0x69,
0x64, 0x151, 0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x69, 0x20, 0x7a,
0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x73, 0x7a, 0x61, 0x6d, 0x6f, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73,
0x7a, 0x61, 0x6d, 0x6f, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x73, 0x7a, 0x61, 0x6d, 0x6f, 0x61, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x65, 0x79, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x7a, 0x69, 0x6e, 0x67,
0x61, 0x70, 0xfa, 0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x6f, 0x6e, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x64,
0xe9, 0x6c, 0x69, 0x2d, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x73, 0x7a, 0x75, 0x72, 0x69, 0x6e, 0xe1, 0x6d,
0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x79, 0x6f, 0x77, 0x61, 0x69, 0x20,
0x69, 0x64, 0x151, 0x74, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x69, 0x64,
0x151, 0x74, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74,
0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x74, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0xe1, 0x64, 0x7a, 0x73, 0x69,
0x6b, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151,
0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x69, 0x20, 0x69, 0x64, 0x151,
0x74, 0x6f, 0x6e, 0x67, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x72, 0x75, 0x6b, 0x69, 0x20, 0x69,
0x64, 0x151, 0x74, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x7a,
0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0xfc, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0xfc, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x69, 0x20, 0x69, 0x64, 0x151, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x69, 0x20, 0x69, 0x64, 0x151, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x75,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0xfc, 0x7a, 0x62, 0x65, 0x67, 0x69, 0x73,
0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfc, 0x7a, 0x62,
0x65, 0x67, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfc, 0x7a, 0x62, 0x65, 0x67, 0x69,
0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x69,
0x20, 0x69, 0x64, 0x151, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x76, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61,
0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x6c, 0x61, 0x67, 0x79, 0x69, 0x76,
0x6f, 0x73, 0x7a, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76,
0x6c, 0x61, 0x67, 0x79, 0x69, 0x76, 0x6f, 0x73, 0x7a, 0x74, 0x6f, 0x6b,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x6c,
0x61, 0x67, 0x79, 0x69, 0x76, 0x6f, 0x73, 0x7a, 0x74, 0x6f, 0x6b, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x64, 0x69, 0x20, 0x69, 0x64, 0x151,
0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x64, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0xe1, 0x64, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x77, 0x61, 0x6b, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0xe9, 0x73, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x2d,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x7a,
0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x7a, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x65, 0x6b, 0x61, 0x74,
0x79, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x79, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x79, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x79, 0x75, 0x6b, 0x6f, 0x6e, 0x69, 0x20, 0x69, 0x64,
0x151, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x74,
0xed, 0x6d, 0x69, 0x4d, 0x69, 0xf0, 0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b,
0x75, 0x74, 0xed, 0x6d, 0x69, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d,
0x41, 0x66, 0x72, 0xed, 0x6b, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x75,
0xf0, 0x75, 0x72, 0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x75, 0x74, 0xed,
0x6d, 0x69, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x41, 0x66, 0x72,
0xed, 0x6b, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74,
0x75, 0x72, 0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x75, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73,
0x74, 0x75, 0x72, 0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x75, 0x54, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x41, 0x6d, 0x61, 0x73, 0xf3, 0x6e, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41,
0x6d, 0x61, 0x73, 0xf3, 0x6e, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x41, 0x6d, 0x61, 0x73, 0xf3, 0x6e, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e,
0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x6d, 0x69, 0xf0, 0x68,
0x6c, 0x75, 0x74, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0xed,
0x6b, 0x6a, 0x61, 0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b, 0x61,
0x6e, 0x61, 0x64, 0x61, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x6d, 0x69, 0xf0, 0x68, 0x6c, 0x75, 0x74,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0xed, 0x6b, 0x6a, 0x61,
0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64,
0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x6d, 0x69, 0xf0, 0x68, 0x6c, 0x75, 0x74, 0x61, 0x20, 0x42, 0x61,
0x6e, 0x64, 0x61, 0x72, 0xed, 0x6b, 0x6a, 0x61, 0x6e, 0x6e, 0x61, 0x20,
0x6f, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x54, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x61, 0x75, 0x73, 0x74, 0x75, 0x72, 0x68, 0x6c,
0x75, 0x74, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0xed, 0x6b,
0x6a, 0x61, 0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b, 0x61, 0x6e,
0x61, 0x64, 0x61, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x61, 0x75, 0x73, 0x74, 0x75, 0x72, 0x68, 0x6c,
0x75, 0x74, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0xed, 0x6b,
0x6a, 0x61, 0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b, 0x61, 0x6e,
0x61, 0x64, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x61, 0x75, 0x73, 0x74, 0x75, 0x72, 0x68, 0x6c, 0x75,
0x74, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0xed, 0x6b, 0x6a,
0x61, 0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x61,
0x64, 0x61, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x6c, 0x65,
0x74, 0x74, 0x61, 0x66, 0x6a, 0xf6, 0x6c, 0x6c, 0x75, 0x6d, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b,
0x6c, 0x65, 0x74, 0x74, 0x61, 0x66, 0x6a, 0xf6, 0x6c, 0x6c, 0x75, 0x6d,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x4b, 0x6c, 0x65, 0x74, 0x74, 0x61, 0x66, 0x6a, 0xf6, 0x6c, 0x6c, 0x75,
0x6d, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4b, 0x79, 0x72, 0x72,
0x61, 0x68, 0x61, 0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68, 0x61, 0x66, 0x73, 0x73, 0x76,
0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68, 0x61,
0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x54, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x70, 0xed, 0x61, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41,
0x70, 0xed, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x41, 0x70, 0xed, 0x61, 0x41, 0x72, 0x61, 0x62, 0xed,
0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x72, 0x61, 0x62, 0xed, 0x75,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x41, 0x72, 0x61, 0x62, 0xed, 0x75, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0xed, 0x6e, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0xed, 0x6e, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed,
0x6e, 0x75, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0xed, 0x6e, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56,
0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0xed, 0x6e, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x75, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0xed, 0x75, 0x54, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x68,
0x61, 0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x68, 0x61, 0x66, 0x73, 0x73, 0x76,
0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73,
0x68, 0x61, 0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x69, 0xf0, 0x2d, 0xc1, 0x73,
0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x69, 0xf0, 0x2d, 0xc1,
0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x69, 0xf0, 0x2d, 0xc1,
0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x68,
0x6c, 0x75, 0x74, 0x61, 0x20, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed,
0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x68,
0x6c, 0x75, 0x74, 0x61, 0x20, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed,
0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x68, 0x6c,
0x75, 0x74, 0x61, 0x20, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75,
0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x75, 0x73, 0x74, 0x75,
0x72, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41,
0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c,
0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0xc1, 0x73, 0x74,
0x72, 0x61, 0x6c, 0xed, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61,
0x6c, 0xed, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0xc1,
0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75,
0x72, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x41, 0x73,
0x65, 0x72, 0x62, 0x61, 0xed, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0xed, 0x64, 0x73,
0x6a, 0x61, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0xed, 0x64, 0x73,
0x6a, 0x61, 0x6e, 0x41, 0x73, 0xf3, 0x72, 0x65, 0x79, 0x6a, 0x61, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x41, 0x73, 0xf3, 0x72, 0x65, 0x79, 0x6a, 0x75,
0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x41, 0x73, 0xf3, 0x72, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x73, 0x2d, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x73,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x73, 0x42, 0xfa,
0x74, 0x61, 0x6e, 0x74, 0xed, 0x6d, 0x69, 0x42, 0xf3, 0x6c, 0x69, 0x76,
0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0xed, 0x75, 0x42,
0x72, 0xfa, 0x6e, 0x65, 0x69, 0x74, 0xed, 0x6d, 0x69, 0x47, 0x72, 0xe6,
0x6e, 0x68, 0xf6, 0x66, 0xf0, 0x61, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x47, 0x72, 0xe6, 0x6e, 0x68, 0xf6, 0x66, 0xf0, 0x61,
0x65, 0x79, 0x6a, 0x75, 0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x47, 0x72, 0xe6, 0x6e, 0x68, 0xf6, 0x66,
0xf0, 0x61, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x2d, 0x73, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0xed, 0x6c, 0x65, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x53, 0xed, 0x6c, 0x65, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x53, 0xed, 0x6c, 0x65, 0x4b, 0xed,
0x6e, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0xed, 0x6e, 0x61, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b,
0xed, 0x6e, 0x61, 0x4a, 0xf3, 0x6c, 0x61, 0x65, 0x79, 0x6a, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x4b, 0xf3, 0x6b, 0x6f, 0x73, 0x65, 0x79, 0x6a,
0x61, 0x74, 0xed, 0x6d, 0x69, 0x4b, 0xf3, 0x6c, 0x75, 0x6d, 0x62, 0xed,
0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0xf3, 0x6c, 0x75, 0x6d, 0x62,
0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x4b, 0xf3, 0x6c, 0x75, 0x6d, 0x62, 0xed, 0x75, 0x43, 0x6f,
0x6f, 0x6b, 0x73, 0x2d, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x6d,
0x48, 0xe1, 0x6c, 0x66, 0x73, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x2d, 0x65, 0x79,
0x6a, 0x75, 0x6d, 0x4b, 0xfa, 0x62, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x4b, 0xfa, 0x62, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x4b, 0xfa, 0x62, 0x75, 0x44, 0x61, 0x76, 0x69,
0x73, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x54, 0xed, 0x6d, 0x6f, 0x72, 0x2d, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x50,
0xe1, 0x73, 0x6b, 0x61, 0x65, 0x79, 0x6a, 0x75, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x50, 0xe1, 0x73, 0x6b, 0x61, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x50, 0xe1,
0x73, 0x6b, 0x61, 0x65, 0x79, 0x6a, 0x75, 0x45, 0x6b, 0x76, 0x61, 0x64,
0x6f, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x4d, 0x69, 0xf0, 0x2d, 0x45, 0x76,
0x72, 0xf3, 0x70, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x69, 0xf0,
0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x69, 0xf0, 0x2d, 0x45,
0x76, 0x72, 0xf3, 0x70, 0x75, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d,
0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41,
0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70,
0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0x4b, 0x61, 0x6c, 0xed, 0x6e, 0xed, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x56,
0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75,
0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d,
0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72,
0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x79,
0x6a, 0x75, 0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x79, 0x6a, 0x75, 0x6d, 0x46, 0xed, 0x64, 0x6a, 0xed, 0x65, 0x79,
0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x46, 0xed, 0x64, 0x6a, 0xed,
0x65, 0x79, 0x6a, 0x75, 0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x46, 0xed, 0x64, 0x6a, 0xed, 0x65, 0x79,
0x6a, 0x75, 0x6d, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x46, 0x72,
0xf6, 0x6e, 0x73, 0x6b, 0x75, 0x20, 0x47, 0x76, 0xe6, 0x6a, 0x61, 0x6e,
0x61, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x66, 0x72, 0xf6, 0x6e,
0x73, 0x6b, 0x75, 0x20, 0x73, 0x75, 0xf0, 0x75, 0x72, 0x68, 0x61, 0x66,
0x73, 0x73, 0x76, 0xe6, 0xf0, 0x75, 0x6d, 0x20, 0x6f, 0x67, 0x20, 0x53,
0x75, 0xf0, 0x75, 0x72, 0x73, 0x6b, 0x61, 0x75, 0x74, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x47, 0x65,
0x6f, 0x72, 0x67, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0xed, 0x75,
0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x2d, 0x73, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x47,
0x72, 0xe6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x47, 0x72, 0xe6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d,
0x47, 0x72, 0xe6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x56, 0x65, 0x73,
0x74, 0x75, 0x72, 0x2d, 0x47, 0x72, 0xe6, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72,
0x2d, 0x47, 0x72, 0xe6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x56, 0x65,
0x73, 0x74, 0x75, 0x72, 0x2d, 0x47, 0x72, 0xe6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0x76, 0x69, 0xf0, 0x20, 0x50, 0x65, 0x72, 0x73, 0x61, 0x66, 0x6c,
0xf3, 0x61, 0x47, 0x76, 0xe6, 0x6a, 0x61, 0x6e, 0x61, 0x74, 0xed, 0x6d,
0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x48, 0x61, 0x76, 0x61,
0xed, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0xfa, 0x74, 0x61, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x48, 0x61, 0x76, 0x61, 0xed, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65,
0xfa, 0x74, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x48, 0x61, 0x76, 0x61, 0xed, 0x20, 0x6f, 0x67, 0x20,
0x41, 0x6c, 0x65, 0xfa, 0x74, 0x61, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x49, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x49, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x68, 0x61, 0x66, 0x73, 0x74, 0xed, 0x6d,
0x69, 0x49, 0x6e, 0x64, 0xf3, 0x6b, 0xed, 0x6e, 0x61, 0x74, 0xed, 0x6d,
0x69, 0x4d, 0x69, 0xf0, 0x2d, 0x49, 0x6e, 0x64, 0xf3, 0x6e, 0x65, 0x73,
0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72,
0x2d, 0x49, 0x6e, 0x64, 0xf3, 0x6e, 0x65, 0x73, 0xed, 0x75, 0x74, 0xed,
0x6d, 0x69, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x49, 0x6e, 0x64,
0xf3, 0x6e, 0x65, 0x73, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0xcd, 0x72,
0x61, 0x6e, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xcd, 0x72, 0x61, 0x6e,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0xcd, 0x72, 0x61, 0x6e, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xcd, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xcd, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0xcd, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x61, 0x73,
0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x4b, 0x61, 0x73, 0x61,
0x6b, 0x73, 0x74, 0x61, 0x6e, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x4b, 0x61, 0x73, 0x61, 0x6b,
0x73, 0x74, 0x61, 0x6e, 0x4b, 0xf3, 0x72, 0x65, 0x75, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x4b, 0xf3, 0x72, 0x65, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0xf3, 0x72, 0x65, 0x75,
0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x4c, 0xed, 0x6e, 0x75, 0x65, 0x79,
0x6a, 0x61, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d,
0x65, 0x79, 0x6a, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x54, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x4d, 0x61, 0x6c, 0x61, 0x73, 0xed, 0x75, 0x74, 0xed, 0x6d,
0x69, 0x4d, 0x61, 0x6c, 0x64, 0xed, 0x76, 0x65, 0x79, 0x6a, 0x61, 0x2d,
0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4d,
0x61, 0x72, 0x6b, 0x67, 0x72, 0x65, 0x69, 0x66, 0x61, 0x66, 0x72, 0xfa,
0x72, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x2d, 0x65, 0x79,
0x6a, 0x75, 0x6d, 0x4d, 0xe1, 0x72, 0x69, 0x74, 0xed, 0x75, 0x73, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x4d, 0xe1, 0x72, 0x69, 0x74, 0xed, 0x75, 0x73,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x4d, 0xe1, 0x72, 0x69, 0x74, 0xed, 0x75, 0x73, 0x4d, 0x61, 0x77, 0x73,
0x6f, 0x6e, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x4b, 0x79, 0x72, 0x72, 0x61,
0x68, 0x61, 0x66, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d,
0x65, 0x78, 0xed, 0x6b, 0xf3, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x65, 0x78, 0xed, 0x6b, 0xf3,
0x20, 0xe1, 0x20, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68, 0x61, 0x66, 0x73,
0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x65, 0x78, 0xed, 0x6b,
0xf3, 0x20, 0xe1, 0x20, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68, 0x61, 0x66,
0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x54, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0xda, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0xda, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0xda, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4d, 0x6f,
0x73, 0x6b, 0x76, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x6f, 0x73,
0x6b, 0x76, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x75, 0x4d, 0x6a, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x4e, 0xe1, 0x72,
0xfa, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0xfd,
0x6a, 0x75, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0xed, 0x75,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x4e, 0xfd, 0x6a, 0x75, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0xf3,
0x6e, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x4e, 0xfd, 0x6a, 0x75, 0x2d, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0xf3, 0x6e, 0xed, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x4e, 0xfd, 0x6a, 0x61, 0x2d, 0x53, 0x6a, 0xe1, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x4e, 0xfd, 0x6a, 0x61, 0x2d, 0x53, 0x6a, 0xe1, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x4e, 0xfd, 0x6a, 0x61, 0x2d, 0x53, 0x6a, 0xe1, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e,
0xfd, 0x66, 0x75, 0x6e, 0x64, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x4e, 0xfd, 0x66, 0x75, 0x6e, 0x64, 0x6e, 0x61, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x4e, 0xfd, 0x66, 0x75, 0x6e, 0x64, 0x6e, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x4e, 0x69, 0x75, 0x65, 0x2d, 0x74, 0xed, 0x6d, 0x69,
0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x75, 0x6d, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x54, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x50, 0x61, 0x6c, 0xe1, 0x74,
0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x50, 0x61,
0x70, 0xfa, 0x61, 0x20, 0x4e, 0xfd, 0x6a, 0x75, 0x2d, 0x47, 0xed, 0x6e,
0x65, 0x75, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0xe6, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0xe6, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x76, 0xe6, 0x50, 0x65, 0x72, 0xfa, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x50, 0x65, 0x72, 0xfa, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x73, 0x65, 0x79, 0x6a, 0x75,
0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x73, 0x65, 0x79, 0x6a, 0x75,
0x6d, 0x46, 0xf6, 0x6e, 0x69, 0x78, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed,
0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53, 0x61, 0x6e,
0x6b, 0x74, 0x69, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6f,
0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53,
0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x2d, 0x74, 0xed,
0x6d, 0x69, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x74, 0xed, 0x6d,
0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x6a, 0x6f, 0x6e,
0x67, 0x6a, 0x61, 0x6e, 0x67, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x2d, 0x74, 0xed, 0x6d, 0x69, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61,
0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x53, 0x61, 0x6d, 0xf3, 0x61, 0x2d, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x53, 0x61, 0x6d, 0xf3, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53, 0x61, 0x6d, 0xf3, 0x61,
0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x2d, 0x65,
0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0xfa, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x61, 0x6c, 0xf3, 0x6d,
0x6f, 0x6e, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x75, 0xf0, 0x75, 0x72, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0xed, 0x75,
0x74, 0xed, 0x6d, 0x69, 0x53, 0xfa, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d, 0x74, 0xed, 0x6d,
0x69, 0x54, 0x61, 0x68, 0xed, 0x74, 0xed, 0x2d, 0x74, 0xed, 0x6d, 0x69,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x54, 0x61, 0x64, 0x73, 0x6a, 0xed, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xf3, 0x6b, 0x65, 0x6c, 0xe1, 0x2d,
0x74, 0xed, 0x6d, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xfa,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x54, 0xfa, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x54, 0xfa, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x54, 0xfa, 0x76, 0x61, 0x6c, 0xfa, 0x74, 0xed,
0x6d, 0x69, 0xda, 0x72, 0xfa, 0x67, 0x76, 0xe6, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0xda, 0x72, 0xfa, 0x67, 0x76, 0xe6, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xda, 0x72, 0xfa, 0x67, 0x76,
0xe6, 0xda, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d,
0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0xda, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0xda, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x56, 0x61, 0x6e, 0xfa, 0x61, 0x74, 0xfa, 0x2d, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x56, 0x61, 0x6e, 0xfa, 0x61, 0x74, 0xfa, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x56, 0x61, 0x6e,
0xfa, 0x61, 0x74, 0xfa, 0x56, 0x65, 0x6e, 0x65, 0x73, 0xfa, 0x65, 0x6c,
0x61, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x54, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x65, 0x79, 0x6a, 0x75,
0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x2d, 0x20, 0x6f, 0x67, 0x20, 0x46, 0xfa, 0x74, 0xfa, 0x6e, 0x61,
0x65, 0x79, 0x6a, 0x75, 0x6d, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x54, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x72, 0x67, 0x53, 0x75, 0x6d, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x54, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x4a, 0xfa, 0x6b, 0x6f, 0x6e, 0x4f, 0x67, 0x65,
0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x67, 0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x41, 0x66,
0x72, 0x1ecb, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67,
0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41,
0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x66, 0x72, 0x1ecb, 0x6b,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x4d, 0x67, 0x62,
0x61, 0x64, 0x61, 0x20, 0x55, 0x67, 0x77, 0x75, 0x20, 0x41, 0x66, 0x72,
0x1ecb, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x66, 0x72, 0x1ecb, 0x6b, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x66,
0x72, 0x1ecb, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61,
0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x66, 0x72, 0x1ecb, 0x6b, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b,
0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61,
0x72, 0x61, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61,
0x6e, 0x77, 0x1ee5, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x4f, 0x67, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x55, 0x67,
0x77, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x55, 0x67,
0x77, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x55, 0x67, 0x77, 0x75, 0x4f,
0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61,
0x72, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61,
0x72, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4f, 0x67,
0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x68, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x41, 0x72, 0x61, 0x62, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x68, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x4f, 0x67, 0x65, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc,
0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x4f, 0x67,
0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x68, 0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61,
0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x74, 0x69,
0x74, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x74,
0x69, 0x74, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x74, 0x69, 0x74,
0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77,
0x1ee5, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61,
0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68,
0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc,
0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc,
0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64,
0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x4f, 0x67, 0x65, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e,
0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61,
0x6d, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d,
0x1ecd, 0x6b, 0x1ee5, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x67, 0x65,
0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x45, 0x6b, 0x65, 0x72, 0x65, 0x73, 0x69, 0x6d, 0x65,
0x73, 0x69, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65,
0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x43, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x43, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65,
0x20, 0x1ecc, 0x6b, 0x61, 0x72, 0x61, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d,
0x1ecd, 0x6b, 0x1ee5, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43,
0x75, 0x62, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x43, 0x75, 0x62, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x4f, 0x67, 0x65,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x54, 0x69, 0x6d, 0x6f,
0x72, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61,
0x6e, 0x77, 0x1ee5, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77,
0x1ee5, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x4f, 0x67, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x45, 0x74, 0x69, 0x74,
0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61,
0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79,
0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61,
0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d,
0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e,
0x77, 0x1ee5, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64,
0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61,
0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d,
0x1ecd, 0x6b, 0x1ee5, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x46, 0x69, 0x6a, 0x69,
0x4f, 0x67, 0x65, 0x20, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x47,
0x75, 0x69, 0x61, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x46, 0x72, 0x65,
0x6e, 0x63, 0x68, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72, 0x6e,
0x20, 0x26, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63,
0x4f, 0x67, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f,
0x73, 0x4f, 0x67, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x4f, 0x67, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6d, 0x77, 0x69, 0x63, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77,
0x1ee5, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61,
0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b,
0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc,
0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x4f, 0x67,
0x65, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f,
0x67, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x73, 0x69, 0x6d, 0x69, 0x72, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5,
0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb,
0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x68, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x4f, 0x67, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61,
0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x68, 0x65, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20,
0x4b, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67,
0x65, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x67, 0x65,
0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x6c,
0x61, 0x79, 0x73, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x65, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f,
0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x67, 0x65,
0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x77, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x77, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61,
0x72, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4f, 0x67,
0x65, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x4f, 0x67, 0x65, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67,
0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65,
0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x67, 0x65,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x4f, 0x67,
0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x67,
0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x67, 0x65,
0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x50, 0x65,
0x72, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x69, 0x6e, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65,
0x4f, 0x67, 0x65, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20,
0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x67, 0x65,
0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x4f, 0x67, 0x65,
0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4f, 0x67, 0x65, 0x20,
0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x67,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c,
0x65, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4f, 0x67,
0x65, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x53, 0x6f, 0x75,
0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x4f, 0x67,
0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x54,
0x61, 0x68, 0x69, 0x74, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x68, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x67,
0x65, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x67, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x4f,
0x67, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x4f, 0x67, 0x65, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f,
0x67, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x4f, 0x67, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x4f, 0x67, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x67, 0x65,
0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x56, 0x65, 0x6e,
0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x4f, 0x67, 0x65, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0x26, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d,
0x1ecd, 0x6b, 0x1ee5, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f,
0x67, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x67, 0x65, 0x20, 0x59, 0x75,
0x6b, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x63, 0x72,
0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x63, 0x72, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x41, 0x63, 0x72, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x74,
0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x65, 0x67, 0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50,
0x65, 0x67, 0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x50, 0x65, 0x67, 0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41,
0x71, 0x74, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x71,
0x74, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x71, 0x74,
0x6f, 0x62, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41,
0x72, 0x61, 0x62, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x72, 0x61,
0x62, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x42, 0x61,
0x67, 0x69, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x42, 0x61,
0x67, 0x69, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x42, 0x61, 0x67, 0x69, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x72,
0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x65, 0x6e,
0x67, 0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65, 0x6e,
0x67, 0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x69, 0x6d,
0x75, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x42, 0x61, 0x72, 0x61,
0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72,
0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54,
0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x43, 0x69, 0x6c, 0x65,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x54, 0x69, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x54, 0x69, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20,
0x4e, 0x61, 0x74, 0x61, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x63,
0x6f, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x70, 0x2e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x65, 0x70,
0x2e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b, 0x65, 0x70, 0x2e, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x75,
0x62, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x50, 0x61, 0x73, 0x6b,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x50, 0x61,
0x73, 0x6b, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x75,
0x6c, 0x61, 0x75, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x45, 0x6b, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x20, 0x4a, 0x61, 0x75, 0x68, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x42, 0x61,
0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x45, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x50, 0x72, 0x61, 0x6e,
0x63, 0x69, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x57, 0x69, 0x6c,
0x61, 0x79, 0x61, 0x68, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e,
0x20, 0x64, 0x61, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x6b, 0x61, 0x20, 0x50, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x73, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x2e,
0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x47, 0x75, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x65, 0x6c, 0x75, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61,
0x6d, 0x75, 0x64, 0x65, 0x72, 0x61, 0x20, 0x48, 0x69, 0x6e, 0x64, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63,
0x68, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4a,
0x65, 0x70, 0x61, 0x6e, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e,
0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e,
0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x2e, 0x20, 0x4c, 0x69, 0x6e,
0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x61, 0x6b, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x6b, 0x61, 0x75,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x64, 0x65,
0x77, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x70, 0x2e, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x61, 0x77, 0x73, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42,
0x61, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42,
0x61, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x75, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72,
0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x2e, 0x20, 0x4d,
0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x55, 0x74, 0x61, 0x72, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x70, 0x75,
0x61, 0x20, 0x4e, 0x75, 0x67, 0x69, 0x6e, 0x69, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x65,
0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x65, 0x72, 0x75,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75,
0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69,
0x78, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x6e, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64,
0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x6e, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x52, 0x65,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65,
0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61,
0x75, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54,
0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68, 0x75,
0x75, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x57, 0x61, 0x6b,
0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0x64, 0x61, 0x6e, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x59, 0x75,
0x6b, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75,
0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x57, 0x65,
0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x69, 0x61, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x69, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x69,
0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x61, 0x72, 0x61, 0x62, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x72, 0x61, 0x62,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x61, 0x72, 0x61, 0x62, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x61, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x6f, 0x2d, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x2d,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x6f, 0x2d, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x68,
0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x65, 0x69, 0x64, 0x7a, 0x68, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x68, 0x61, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x68, 0x75,
0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72,
0x72, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x45, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x63, 0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x63, 0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x68, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x75, 0x6c,
0x74, 0x72, 0x61, 0x2d, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63,
0x65, 0x73, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x66, 0x72, 0x61, 0x6e,
0x63, 0x65, 0x73, 0x65, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x6f,
0x6e, 0x61, 0x6c, 0x20, 0x65, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x63,
0x74, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x6d, 0x65, 0x64, 0x69, 0x65, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x6f, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43,
0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x69, 0x72, 0x67, 0x68, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4c, 0x69, 0x6e, 0x65,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79,
0x73, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x65, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65,
0x72, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x68, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x61, 0x73, 0x20, 0x50, 0x68, 0x65, 0x6e, 0x69, 0x63, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69,
0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68,
0x65, 0x6c, 0x6c, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20,
0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x53, 0x75, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x6e, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x6e, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c,
0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x6d, 0x6f, 0x6e,
0x74, 0x61, 0x6e, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x61, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x61, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73,
0x74, 0x6d, 0x61, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x43,
0x6f, 0x63, 0x6f, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61,
0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x63, 0x75, 0x62, 0x61, 0x6e, 0x20,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x63, 0x75, 0x62, 0x61, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x63, 0x75, 0x62, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x6e, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x6f, 0x73,
0x74, 0x2d, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x6f, 0x73, 0x74, 0x2d, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x6f, 0x73, 0x74, 0x2d, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x77, 0x65, 0x73,
0x74, 0x2d, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x77, 0x65, 0x73, 0x74, 0x2d, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x77, 0x65, 0x73, 0x74,
0x2d, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x65, 0x20, 0x6d,
0x65, 0x64, 0x69, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x65, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x65, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x73,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77,
0x65, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x6d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61,
0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x20, 0x59, 0x75, 0x6b,
0x6f, 0x6e, 0x41, 0x6d, 0x20, 0x41, 0x63, 0x72, 0x65, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x41, 0x63, 0x72, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x63, 0x72, 0x65, 0x41, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x61, 0x73,
0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x41, 0x66, 0x72, 0x61, 0x69, 0x63, 0x65, 0x20, 0x4c, 0xe1, 0x69, 0x72,
0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x66, 0x72, 0x61, 0x69, 0x63, 0x65, 0x41,
0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x66, 0x72, 0x61, 0x69, 0x63,
0x65, 0x20, 0x54, 0x68, 0x65, 0x61, 0x73, 0x41, 0x6d, 0x20, 0x49, 0x61,
0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x66,
0x72, 0x61, 0x69, 0x63, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x49, 0x61, 0x72,
0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x66, 0x72,
0x61, 0x69, 0x63, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x66, 0x72, 0x61, 0x69, 0x63, 0x65,
0x41, 0x6d, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x63, 0x61, 0x41, 0x6d, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x6d, 0x61, 0x73, 0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x6d, 0x61, 0x73, 0xf3, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x6d, 0x61, 0x73, 0xf3, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x4c, 0xe1, 0x72, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d,
0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75,
0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4c, 0xe1, 0x72, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65,
0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4c, 0xe1,
0x72, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69,
0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41,
0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x61, 0x63,
0x68, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20,
0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f,
0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x61, 0x63, 0x68, 0x20, 0x4d,
0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75,
0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61,
0x72, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63,
0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d,
0x20, 0x53, 0x6c, 0xe9, 0x69, 0x62, 0x68, 0x74, 0x65, 0x20, 0x4d, 0x68,
0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61,
0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x6c, 0xe9, 0x69, 0x62,
0x68, 0x74, 0x65, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65,
0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x6c,
0xe9, 0x69, 0x62, 0x68, 0x74, 0x65, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72,
0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68,
0x41, 0x6d, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e,
0x20, 0x43, 0x68, 0x69, 0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x61,
0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69,
0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69,
0x6e, 0x20, 0x43, 0x68, 0x69, 0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x41, 0x6d,
0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x41, 0x70, 0x69,
0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x41, 0x71, 0x74,
0x61, 0x75, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x41, 0x71, 0x74, 0x61, 0x75, 0x41, 0x6d, 0x20, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x72, 0x61, 0x69, 0x62, 0x65, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x72, 0x61, 0x69, 0x62, 0x65, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x72, 0x61, 0x69, 0x62, 0x65, 0x41, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x74, 0xed,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69,
0x72, 0x67, 0x69, 0x6e, 0x74, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20,
0x68, 0x41, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x74, 0xed, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x61, 0x63, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x74,
0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68,
0x61, 0x72, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69,
0x72, 0x67, 0x69, 0x6e, 0x74, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x61, 0x72,
0x74, 0x68, 0x61, 0x72, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x41, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x74, 0xed, 0x6e, 0x65, 0x41, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x6d, 0xe9, 0x69, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72,
0x6d, 0xe9, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69,
0x72, 0x6d, 0xe9, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x61, 0x6e, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x67, 0x68, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x61,
0x69, 0x67, 0x68, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x61, 0x69, 0x67, 0x68, 0x41, 0x6d, 0x20, 0x4c, 0xe1, 0x72, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x4c, 0xe1, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4c, 0xe1, 0x72,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c,
0x65, 0x41, 0x6d, 0x20, 0x4d, 0x68, 0x65, 0xe1, 0x6e, 0x69, 0x61, 0x72,
0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74,
0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x65,
0xe1, 0x6e, 0x69, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d,
0x68, 0x65, 0xe1, 0x6e, 0x69, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65,
0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c,
0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41,
0x6d, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49,
0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x41, 0x73, 0x61, 0x72, 0x62, 0x61, 0x69, 0x73, 0x65, 0xe1,
0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x73, 0x61, 0x72, 0x62, 0x61, 0x69, 0x73, 0x65, 0xe1, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x61, 0x72, 0x62, 0x61, 0x69,
0x73, 0x65, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20,
0x6e, 0x41, 0x73, 0xf3, 0x72, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20,
0x6e, 0x41, 0x73, 0xf3, 0x72, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x41, 0x73,
0xf3, 0x72, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x69, 0x64, 0xe9, 0x69, 0x73, 0x65, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x69, 0x64, 0xe9,
0x69, 0x73, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x69, 0x64, 0xe9, 0x69, 0x73, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x42, 0xfa, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x42, 0x6f, 0x6c, 0x61, 0x69, 0x76, 0x65, 0x41, 0x6d, 0x20,
0x42, 0x68, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x42, 0x68, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x42,
0x68, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x42,
0x68, 0x72, 0xfa, 0x69, 0x6e, 0xe9, 0x20, 0x44, 0x61, 0x72, 0x61, 0x73,
0x61, 0x6c, 0xe1, 0x6d, 0x41, 0x6d, 0x20, 0x52, 0x69, 0x6e, 0x6e, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x52, 0x69, 0x6e,
0x6e, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x52, 0x69, 0x6e, 0x6e,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x74, 0xe1,
0x69, 0x73, 0x69, 0xfa, 0x6e, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x53, 0x65, 0x61, 0x6d, 0xf3, 0x72, 0x61, 0x63, 0x68,
0x41, 0x6d, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53,
0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x69,
0x6c, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x65, 0x41, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x53, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x53, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53,
0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e,
0x20, 0x6e, 0x61, 0x20, 0x4e, 0x6f, 0x6c, 0x6c, 0x61, 0x67, 0x41, 0x6d,
0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x63,
0x6f, 0x73, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf3,
0x69, 0x6d, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f,
0x6c, 0xf3, 0x69, 0x6d, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6c,
0xf3, 0x69, 0x6d, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1,
0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f,
0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x41,
0x6d, 0x20, 0x4c, 0x65, 0x61, 0x74, 0x68, 0x73, 0x68, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69,
0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x68, 0xfa,
0x62, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x43, 0x68, 0xfa, 0x62, 0x61, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x43, 0x68, 0xfa, 0x62, 0x61, 0x41, 0x6d, 0x20, 0x44, 0x61, 0x76, 0x69,
0x73, 0x41, 0x6d, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x54,
0x68, 0xed, 0x6f, 0x6d, 0xf3, 0x72, 0x20, 0x54, 0x68, 0x6f, 0x69, 0x72,
0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x6e, 0x61,
0x20, 0x43, 0xe1, 0x73, 0x63, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x43, 0xe1, 0x73, 0x63,
0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x6e, 0x61, 0x20,
0x43, 0xe1, 0x73, 0x63, 0x61, 0x41, 0x6d, 0x20, 0x45, 0x61, 0x63, 0x75,
0x61, 0x64, 0xf3, 0x72, 0x41, 0x6d, 0x20, 0x4c, 0xe1, 0x72, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x4c, 0xe1, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70,
0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x4c, 0xe1, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f,
0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65,
0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f,
0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x68, 0x69, 0x61,
0x6e, 0x6f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61,
0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x49, 0x61,
0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f,
0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68,
0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20,
0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x69, 0x6e, 0x20, 0x46, 0x68, 0xe1, 0x63, 0x6c, 0x61, 0x69,
0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1,
0x69, 0x6e, 0x20, 0x46, 0x68, 0xe1, 0x63, 0x6c, 0x61, 0x69, 0x6e, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x46, 0x68,
0xe1, 0x63, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x46,
0x68, 0x69, 0x64, 0x73, 0xed, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x46, 0x68, 0x69,
0x64, 0x73, 0xed, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x46, 0x68, 0x69, 0x64, 0x73, 0xed, 0x41, 0x6d,
0x20, 0x47, 0x68, 0x75, 0xe1, 0x69, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x46,
0x72, 0x61, 0x69, 0x6e, 0x63, 0x65, 0x41, 0x6d, 0x20, 0x46, 0x72, 0x61,
0x6e, 0x63, 0x61, 0x63, 0x68, 0x20, 0x44, 0x68, 0x65, 0x69, 0x73, 0x63,
0x65, 0x61, 0x72, 0x74, 0x20, 0x61, 0x6e, 0x20, 0x44, 0x6f, 0x6d, 0x68,
0x61, 0x69, 0x6e, 0x20, 0x61, 0x67, 0x75, 0x73, 0x20, 0x61, 0x6e, 0x20,
0x41, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x61, 0x69, 0x67, 0x68, 0x41, 0x6d,
0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x6c,
0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x41, 0x6d, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65,
0x6f, 0x69, 0x72, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x53, 0x65, 0x6f, 0x69, 0x72, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x53, 0x65, 0x6f, 0x69, 0x72, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x20,
0x43, 0x68, 0x69, 0x72, 0x65, 0x61, 0x62, 0x61, 0x69, 0x74, 0xed, 0x4d,
0x65, 0xe1, 0x6e, 0x2d, 0x41, 0x6d, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68,
0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e,
0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69,
0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72,
0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69,
0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72,
0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47,
0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61,
0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20,
0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x47, 0x68, 0x75, 0x61, 0x6d, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x4d, 0x75, 0x72, 0x61, 0x73, 0x63, 0x61, 0x69, 0x6c,
0x6c, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x75, 0xe1, 0x69,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x48, 0x61, 0x76, 0xe1, 0xed, 0x2d, 0x41,
0x69, 0x6c, 0x69, 0xfa, 0x69, 0x74, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x48, 0x61,
0x76, 0xe1, 0xed, 0x2d, 0x41, 0x69, 0x6c, 0x69, 0xfa, 0x69, 0x74, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x48, 0x61, 0x76, 0xe1, 0xed, 0x2d, 0x41, 0x69, 0x6c, 0x69, 0xfa, 0x69,
0x74, 0x41, 0x6d, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x43, 0x6f, 0x6e,
0x67, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x43, 0x6f,
0x6e, 0x67, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x43, 0x6f, 0x6e, 0x67,
0x41, 0x6d, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x41, 0x6d, 0x20,
0x61, 0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x69, 0x67, 0x68, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20,
0x68, 0x49, 0x6e, 0x64, 0x2d, 0x53, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x4c, 0xe1, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x49, 0x6e, 0x64, 0x69,
0x6e, 0xe9, 0x69, 0x73, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74,
0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x49, 0x6e, 0x64,
0x69, 0x6e, 0xe9, 0x69, 0x73, 0x65, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x72,
0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x49, 0x6e, 0x64,
0x69, 0x6e, 0xe9, 0x69, 0x73, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20,
0x68, 0x49, 0x61, 0x72, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x49, 0x61, 0x72, 0xe1, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x49, 0x61, 0x72, 0xe1, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x41, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x53, 0x65, 0x61, 0x70, 0xe1, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x61, 0x70, 0xe1, 0x69,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x61, 0x70, 0xe1, 0x69,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x50, 0x68, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x50, 0x68,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x50, 0x68, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x73, 0x61, 0x63,
0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72,
0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x73,
0x61, 0x63, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x49,
0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61,
0x73, 0x61, 0x63, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x43, 0xf3, 0x69, 0x72, 0xe9, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x43, 0xf3, 0x69, 0x72, 0xe9, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20,
0x43, 0xf3, 0x69, 0x72, 0xe9, 0x41, 0x6d, 0x20, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x41, 0x6d, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x43, 0x69, 0x72, 0x67, 0x65, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x53, 0x68, 0x72, 0xed, 0x20, 0x4c, 0x61, 0x6e,
0x63, 0x61, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e,
0x20, 0x6e, 0x61, 0x20, 0x4c, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d, 0x20, 0x4d, 0x68,
0x61, 0x63, 0x61, 0x6f, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x61, 0x63,
0x61, 0x6f, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x4d, 0x68, 0x61, 0x63, 0x61, 0x6f, 0x41, 0x6d, 0x20,
0x4d, 0x68, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x4d, 0x68, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x68, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x4d,
0x61, 0x6c, 0x61, 0x65, 0x69, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x4f,
0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x4d, 0x68, 0x61, 0x69, 0x6c,
0x64, 0xed, 0x76, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f,
0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x61, 0x73,
0x61, 0x63, 0x68, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69,
0x6e, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x41, 0x6d,
0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x68, 0x75, 0x69,
0x72, 0xed, 0x73, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1,
0x6e, 0x20, 0x4d, 0x68, 0x75, 0x69, 0x72, 0xed, 0x73, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x68, 0x75, 0x69, 0x72, 0xed, 0x73,
0x41, 0x6d, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x41, 0x6d, 0x20,
0x4d, 0x65, 0x69, 0x63, 0x73, 0x69, 0x63, 0x65, 0x61, 0x63, 0x68, 0x20,
0x61, 0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x43, 0x68,
0x69, 0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x65, 0x69, 0x63,
0x73, 0x69, 0x63, 0x65, 0x61, 0x63, 0x68, 0x20, 0x61, 0x6e, 0x20, 0x41,
0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0xfa, 0x69, 0x6e,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x4d, 0x65, 0x69, 0x63, 0x73, 0x69, 0x63, 0x65, 0x61, 0x63, 0x68,
0x20, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x43,
0x68, 0x69, 0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20, 0x55, 0x6c, 0xe1, 0x6e,
0x62, 0xe1, 0x74, 0x61, 0x72, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x55, 0x6c, 0xe1,
0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x62,
0xe1, 0x74, 0x61, 0x72, 0x41, 0x6d, 0x20, 0x4d, 0x68, 0x6f, 0x73, 0x63,
0xf3, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x6f, 0x73, 0x63, 0xf3, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x4d, 0x68, 0x6f, 0x73, 0x63, 0xf3, 0x41, 0x6d, 0x20, 0x4d, 0x68, 0x61,
0x65, 0x6e, 0x6d, 0x61, 0x72, 0x41, 0x6d, 0x20, 0x4e, 0xe1, 0x72, 0xfa,
0x41, 0x6d, 0x20, 0x4e, 0x65, 0x69, 0x70, 0x65, 0x61, 0x6c, 0x41, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x43, 0x68, 0x61, 0x6c,
0x61, 0x64, 0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x4e, 0x75, 0x61, 0x2d, 0x43, 0x68, 0x61, 0x6c, 0x61, 0x64, 0xf3,
0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x43,
0x68, 0x61, 0x6c, 0x61, 0x64, 0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x53, 0x68, 0xe9, 0x61, 0x6c,
0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20,
0x4e, 0x75, 0x61, 0x2d, 0x53, 0x68, 0xe9, 0x61, 0x6c, 0x61, 0x69, 0x6e,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x53, 0x68,
0xe9, 0x61, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x54,
0x68, 0x61, 0x6c, 0x61, 0x6d, 0x68, 0x20, 0x61, 0x6e, 0x20, 0xc9, 0x69,
0x73, 0x63, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x54, 0x68, 0x61, 0x6c, 0x61, 0x6d,
0x68, 0x20, 0x61, 0x6e, 0x20, 0xc9, 0x69, 0x73, 0x63, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54, 0x68,
0x61, 0x6c, 0x61, 0x6d, 0x68, 0x20, 0x61, 0x6e, 0x20, 0xc9, 0x69, 0x73,
0x63, 0x41, 0x6d, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x41, 0x6d, 0x20, 0x4f,
0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65,
0xe1, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x41, 0x6d,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0xe1, 0x69, 0x72, 0x69, 0x61, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41,
0x6d, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x63, 0x61, 0x73, 0x74,
0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x50,
0x61, 0x63, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x50, 0x61, 0x63, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x41, 0x6d, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x47, 0x68,
0x75, 0x69, 0x6e, 0x65, 0x20, 0x50, 0x68, 0x61, 0x70, 0x75, 0x61, 0x41,
0x6d, 0x20, 0x50, 0x68, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x50, 0x68, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50,
0x68, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x41, 0x6d, 0x20, 0x50, 0x68,
0x65, 0x69, 0x72, 0x69, 0xfa, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x50, 0x68, 0x65,
0x69, 0x72, 0x69, 0xfa, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x68, 0x65, 0x69, 0x72, 0x69, 0xfa,
0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f, 0x69, 0x6c, 0x65, 0xe1,
0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x65, 0x61, 0x63,
0x68, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x65,
0x61, 0x63, 0x68, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f, 0x69, 0x6c, 0x65,
0xe1, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x65, 0x61,
0x63, 0x68, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e,
0x20, 0x61, 0x6e, 0x20, 0x46, 0x68, 0xe9, 0x69, 0x6e, 0x69, 0x63, 0x73,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41, 0x6d, 0x20, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x41, 0x6d, 0x20, 0x50, 0x68,
0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x20, 0x50, 0x79, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x41, 0x6d, 0x20, 0x51, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x51, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x51, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x41, 0x6d, 0x20, 0x52, 0xe9, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x41, 0x6d, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0x41, 0x6d, 0x20, 0x53, 0x68, 0x61, 0x63, 0x61, 0x69, 0x6c, 0xed,
0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x68, 0x61, 0x63, 0x61, 0x69, 0x6c,
0xed, 0x6e, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x53, 0x68, 0x61, 0x63, 0x61, 0x69, 0x6c, 0xed, 0x6e,
0x41, 0x6d, 0x20, 0x53, 0x68, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x53, 0x68, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x68,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x68, 0x61, 0x6d,
0xf3, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x68, 0x61, 0x6d, 0xf3, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53,
0x68, 0x61, 0x6d, 0xf3, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0xe9,
0x69, 0x73, 0xe9, 0x61, 0x6c, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x68, 0x69,
0x6e, 0x67, 0x65, 0x61, 0x70, 0xf3, 0x72, 0x41, 0x6d, 0x20, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x53, 0x68, 0x6f, 0x6c, 0x61, 0x69,
0x6d, 0x68, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x6f, 0x69,
0x72, 0x73, 0x69, 0x61, 0x20, 0x54, 0x68, 0x65, 0x61, 0x73, 0x41, 0x6d,
0x20, 0x53, 0x68, 0x75, 0x72, 0x61, 0x6e, 0x61, 0x6d, 0x41, 0x6d, 0x20,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x41, 0x6d, 0x20, 0x54, 0x68, 0x61, 0x69,
0x68, 0xed, 0x74, 0xed, 0x41, 0x6d, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20,
0x54, 0xe1, 0x69, 0x64, 0x73, 0xed, 0x63, 0x65, 0x61, 0x73, 0x74, 0xe1,
0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69,
0x6e, 0x20, 0x54, 0xf3, 0x63, 0x61, 0x6c, 0xe1, 0x41, 0x6d, 0x20, 0x54,
0x68, 0x6f, 0x6e, 0x67, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x54, 0x68, 0x6f,
0x6e, 0x67, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x54, 0x68, 0x6f, 0x6e, 0x67, 0x61, 0x41, 0x6d,
0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20,
0x54, 0x75, 0x69, 0x72, 0x63, 0x6d, 0xe9, 0x61, 0x6e, 0x61, 0x73, 0x74,
0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x54,
0x75, 0x69, 0x72, 0x63, 0x6d, 0xe9, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe1,
0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x75, 0x69, 0x72, 0x63,
0x6d, 0xe9, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x54, 0x68, 0xfa, 0x76, 0x61, 0x6c, 0xfa, 0x41, 0x6d, 0x20,
0x55, 0x72, 0x61, 0x67, 0x75, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x55, 0x72,
0x61, 0x67, 0x75, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x55, 0x72, 0x61, 0x67, 0x75, 0x61, 0x41,
0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0xda, 0x69, 0x73, 0x62, 0xe9, 0x69,
0x63, 0x65, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x68, 0xda, 0x69, 0x73, 0x62, 0xe9, 0x69, 0x63,
0x65, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20,
0x68, 0xda, 0x69, 0x73, 0x62, 0xe9, 0x69, 0x63, 0x65, 0x61, 0x73, 0x74,
0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0xfa, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0xfa, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0xfa, 0x41, 0x6d, 0x20,
0x56, 0x65, 0x69, 0x6e, 0x69, 0x73, 0xe9, 0x61, 0x6c, 0x61, 0x41, 0x6d,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x41, 0x6d, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41,
0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x57, 0x61, 0x6b,
0x65, 0x41, 0x6d, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61,
0x67, 0x75, 0x73, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x41, 0x6d,
0x20, 0x49, 0x61, 0x63, 0xfa, 0x74, 0x73, 0x63, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x49, 0x61, 0x63, 0xfa, 0x74, 0x73, 0x63, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x61, 0x63, 0xfa,
0x74, 0x73, 0x63, 0x41, 0x6d, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x41, 0x6d, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x67, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6d,
0x65, 0x72, 0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x6d, 0x61,
0x7a, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x41, 0x6d, 0x61, 0x7a, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55,
0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20,
0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55,
0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61,
0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x20, 0x52, 0x6f,
0x63, 0x63, 0x69, 0x6f, 0x73, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x4d,
0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x20, 0x52, 0x6f, 0x63, 0x63,
0x69, 0x6f, 0x73, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61,
0x67, 0x6e, 0x65, 0x20, 0x52, 0x6f, 0x63, 0x63, 0x69, 0x6f, 0x73, 0x65,
0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x55, 0x53, 0x41,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x6f, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x6f, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x70,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20,
0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x61, 0x72, 0x61, 0x62,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x72, 0x61, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x61, 0x72, 0x61, 0x62, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x63, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x67, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x67, 0x69,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x67, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x42, 0x6f,
0x6c, 0x69, 0x76, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72,
0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x43, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x61, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61,
0x20, 0x43, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x69,
0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49,
0x73, 0x6f, 0x6c, 0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d,
0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20,
0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61,
0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x6c, 0x65,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20,
0x69, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x6d, 0x65, 0x64,
0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69, 0x73, 0x6f,
0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x43, 0x75,
0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x45, 0x73, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x73, 0x6f, 0x6c,
0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x73, 0x6f, 0x6c, 0x61, 0x20, 0x64,
0x69, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x49, 0x73, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73,
0x71, 0x75, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x20, 0x28, 0x4b, 0x61, 0x6c,
0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x29, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c,
0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20,
0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20,
0x46, 0x69, 0x67, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46,
0x69, 0x67, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x67, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x75,
0x69, 0x61, 0x6e, 0x61, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x54,
0x65, 0x72, 0x72, 0x65, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x20, 0x65, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x63,
0x68, 0x65, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65,
0x20, 0x69, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6d, 0x65,
0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x6c, 0x65,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49,
0x73, 0x6f, 0x6c, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65,
0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x4f,
0x63, 0x65, 0x61, 0x6e, 0x6f, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x72,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x6f,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70,
0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x6f,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x69, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x63,
0x6f, 0x72, 0x65, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x63, 0x6f, 0x72, 0x65, 0x61,
0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x63, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x68, 0x69, 0x7a, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x65, 0x20, 0x53, 0x70, 0x6f, 0x72, 0x61, 0x64, 0x69, 0x20, 0x65, 0x71,
0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x63, 0x61,
0x6f, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20,
0x4d, 0x61, 0x63, 0x61, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61,
0x20, 0x4d, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x4d,
0x61, 0x72, 0x63, 0x68, 0x65, 0x73, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x6f, 0x20, 0x28, 0x4d, 0x65, 0x73, 0x73, 0x69, 0x63, 0x6f, 0x29, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f,
0x20, 0x28, 0x4d, 0x65, 0x73, 0x73, 0x69, 0x63, 0x6f, 0x29, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x28, 0x4d,
0x65, 0x73, 0x73, 0x69, 0x63, 0x6f, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x61, 0x20, 0x42, 0x69, 0x72, 0x6d, 0x61, 0x6e, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4e,
0x75, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f,
0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f,
0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x69, 0x75, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73,
0x6f, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20,
0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20,
0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20,
0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65,
0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65,
0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61,
0x20, 0x46, 0x65, 0x6e, 0x69, 0x63, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x79, 0x6f, 0x6e,
0x67, 0x79, 0x61, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x52, 0x69, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x53, 0x65, 0x79, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c,
0x65, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e,
0x61, 0x6d, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x61, 0x67, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a,
0x75, 0x65, 0x6c, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x73, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x69,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20,
0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x69, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x6f, 0x20,
0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x30a2, 0x30af, 0x30ec, 0x6642, 0x9593, 0x30a2, 0x30af,
0x30ec, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30af, 0x30ec, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30d5,
0x30ac, 0x30cb, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x4e2d, 0x592e, 0x30a2, 0x30d5, 0x30ea,
0x30ab, 0x6642, 0x9593, 0x6771, 0x30a2, 0x30d5, 0x30ea, 0x30ab, 0x6642, 0x9593, 0x5357, 0x30a2,
0x30d5, 0x30ea, 0x30ab, 0x6a19, 0x6e96, 0x6642, 0x897f, 0x30a2, 0x30d5, 0x30ea, 0x30ab, 0x6642,
0x9593, 0x897f, 0x30a2, 0x30d5, 0x30ea, 0x30ab, 0x6a19, 0x6e96, 0x6642, 0x897f, 0x30a2, 0x30d5,
0x30ea, 0x30ab, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30e9, 0x30b9, 0x30ab, 0x6642, 0x9593, 0x30a2,
0x30e9, 0x30b9, 0x30ab, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e9, 0x30b9, 0x30ab, 0x590f, 0x6642,
0x9593, 0x30a2, 0x30eb, 0x30c8, 0x30de, 0x30a4, 0x6642, 0x9593, 0x30a2, 0x30eb, 0x30c8, 0x30de,
0x30a4, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30eb, 0x30de, 0x30c8, 0x30a4, 0x590f, 0x6642, 0x9593,
0x30a2, 0x30de, 0x30be, 0x30f3, 0x6642, 0x9593, 0x30a2, 0x30de, 0x30be, 0x30f3, 0x6a19, 0x6e96,
0x6642, 0x30a2, 0x30de, 0x30be, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab,
0x4e2d, 0x90e8, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x4e2d, 0x90e8, 0x6a19, 0x6e96,
0x6642, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x4e2d, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30e1,
0x30ea, 0x30ab, 0x6771, 0x90e8, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x6771, 0x90e8,
0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x6771, 0x90e8, 0x590f, 0x6642, 0x9593,
0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x5c71, 0x5730, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab,
0x5c71, 0x5730, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x5c71, 0x5730, 0x590f,
0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x30a2,
0x30e1, 0x30ea, 0x30ab, 0x592a, 0x5e73, 0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e1, 0x30ea,
0x30ab, 0x592a, 0x5e73, 0x6d0b, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30ca, 0x30c7, 0x30a3, 0x30ea,
0x6642, 0x9593, 0x30a2, 0x30ca, 0x30c7, 0x30a3, 0x30ea, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30ca,
0x30c7, 0x30a3, 0x30ea, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30d4, 0x30a2, 0x6642, 0x9593, 0x30a2,
0x30d4, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30d4, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30a2,
0x30af, 0x30bf, 0x30a6, 0x6642, 0x9593, 0x30a2, 0x30af, 0x30bf, 0x30a6, 0x6a19, 0x6e96, 0x6642,
0x30a2, 0x30af, 0x30bf, 0x30a6, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30af, 0x30c8, 0x30d9, 0x6642,
0x9593, 0x30a2, 0x30af, 0x30c8, 0x30d9, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30af, 0x30c8, 0x30d9,
0x590f, 0x6642, 0x9593, 0x30a2, 0x30e9, 0x30d3, 0x30a2, 0x6642, 0x9593, 0x30a2, 0x30e9, 0x30d3,
0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e9, 0x30d3, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30a2,
0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x6642, 0x9593, 0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1,
0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x590f, 0x6642,
0x9593, 0x897f, 0x90e8, 0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x6642, 0x9593, 0x897f,
0x90e8, 0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x897f, 0x90e8,
0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30eb, 0x30e1,
0x30cb, 0x30a2, 0x6642, 0x9593, 0x30a2, 0x30eb, 0x30e1, 0x30cb, 0x30a2, 0x6a19, 0x6e96, 0x6642,
0x30a2, 0x30eb, 0x30e1, 0x30cb, 0x30a2, 0x590f, 0x6642, 0x9593, 0x5927, 0x897f, 0x6d0b, 0x590f,
0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x90e8, 0x6642,
0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x90e8, 0x6a19, 0x6e96,
0x6642, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x90e8, 0x590f, 0x6642,
0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x897f, 0x90e8, 0x6642,
0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x897f, 0x90e8, 0x6a19,
0x6e96, 0x6642, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x4e2d, 0x897f, 0x90e8,
0x590f, 0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x6771, 0x90e8,
0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x6771, 0x90e8, 0x6a19,
0x6e96, 0x6642, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x6771, 0x90e8, 0x590f,
0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x897f, 0x90e8, 0x6642,
0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x897f, 0x90e8, 0x6a19, 0x6e96,
0x6642, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2, 0x897f, 0x90e8, 0x590f, 0x6642,
0x9593, 0x30a2, 0x30bc, 0x30eb, 0x30d0, 0x30a4, 0x30b8, 0x30e3, 0x30f3, 0x6642, 0x9593, 0x30a2,
0x30bc, 0x30eb, 0x30d0, 0x30a4, 0x30b8, 0x30e3, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30bc,
0x30eb, 0x30d0, 0x30a4, 0x30b8, 0x30e3, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30be, 0x30ec,
0x30b9, 0x6642, 0x9593, 0x30a2, 0x30be, 0x30ec, 0x30b9, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30be,
0x30ec, 0x30b9, 0x590f, 0x6642, 0x9593, 0x30d0, 0x30f3, 0x30b0, 0x30e9, 0x30c7, 0x30b7, 0x30e5,
0x6642, 0x9593, 0x30d0, 0x30f3, 0x30b0, 0x30e9, 0x30c7, 0x30b7, 0x30e5, 0x6a19, 0x6e96, 0x6642,
0x30d0, 0x30f3, 0x30b0, 0x30e9, 0x30c7, 0x30b7, 0x30e5, 0x590f, 0x6642, 0x9593, 0x30d6, 0x30fc,
0x30bf, 0x30f3, 0x6642, 0x9593, 0x30dc, 0x30ea, 0x30d3, 0x30a2, 0x6642, 0x9593, 0x30d6, 0x30e9,
0x30b8, 0x30ea, 0x30a2, 0x6642, 0x9593, 0x30d6, 0x30e9, 0x30b8, 0x30ea, 0x30a2, 0x6a19, 0x6e96,
0x6642, 0x30d6, 0x30e9, 0x30b8, 0x30ea, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30d6, 0x30eb, 0x30cd,
0x30a4, 0x30fb, 0x30c0, 0x30eb, 0x30b5, 0x30e9, 0x30fc, 0x30e0, 0x6642, 0x9593, 0x30ab, 0x30fc,
0x30dc, 0x30d9, 0x30eb, 0x30c7, 0x6642, 0x9593, 0x30ab, 0x30fc, 0x30dc, 0x30d9, 0x30eb, 0x30c7,
0x6a19, 0x6e96, 0x6642, 0x30ab, 0x30fc, 0x30dc, 0x30d9, 0x30eb, 0x30c7, 0x590f, 0x6642, 0x9593,
0x30b1, 0x30a4, 0x30b7, 0x30fc, 0x57fa, 0x5730, 0x6642, 0x9593, 0x30c1, 0x30e3, 0x30e2, 0x30ed,
0x6642, 0x9593, 0x30c1, 0x30e3, 0x30bf, 0x30e0, 0x6642, 0x9593, 0x30c1, 0x30e3, 0x30bf, 0x30e0,
0x6a19, 0x6e96, 0x6642, 0x30c1, 0x30e3, 0x30bf, 0x30e0, 0x590f, 0x6642, 0x9593, 0x30c1, 0x30ea,
0x6642, 0x9593, 0x30c1, 0x30ea, 0x6a19, 0x6e96, 0x6642, 0x30c1, 0x30ea, 0x590f, 0x6642, 0x9593,
0x4e2d, 0x56fd, 0x6642, 0x9593, 0x4e2d, 0x56fd, 0x6a19, 0x6e96, 0x6642, 0x4e2d, 0x56fd, 0x590f,
0x6642, 0x9593, 0x30af, 0x30ea, 0x30b9, 0x30de, 0x30b9, 0x5cf6, 0x6642, 0x9593, 0x30b3, 0x30b3,
0x30b9, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b3, 0x30ed, 0x30f3, 0x30d3, 0x30a2, 0x6642, 0x9593,
0x30b3, 0x30ed, 0x30f3, 0x30d3, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30b3, 0x30ed, 0x30f3, 0x30d3,
0x30a2, 0x590f, 0x6642, 0x9593, 0x30af, 0x30c3, 0x30af, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30af,
0x30c3, 0x30af, 0x8af8, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30af, 0x30c3, 0x30af, 0x8af8, 0x5cf6,
0x590f, 0x6642, 0x9593, 0x30ad, 0x30e5, 0x30fc, 0x30d0, 0x6642, 0x9593, 0x30ad, 0x30e5, 0x30fc,
0x30d0, 0x6a19, 0x6e96, 0x6642, 0x30ad, 0x30e5, 0x30fc, 0x30d0, 0x590f, 0x6642, 0x9593, 0x30c7,
0x30fc, 0x30d3, 0x30b9, 0x57fa, 0x5730, 0x6642, 0x9593, 0x30c7, 0x30e5, 0x30e2, 0x30f3, 0x30fb,
0x30c7, 0x30e5, 0x30eb, 0x30f4, 0x30a3, 0x30eb, 0x57fa, 0x5730, 0x6642, 0x9593, 0x6771, 0x30c6,
0x30a3, 0x30e2, 0x30fc, 0x30eb, 0x6642, 0x9593, 0x30a4, 0x30fc, 0x30b9, 0x30bf, 0x30fc, 0x5cf6,
0x6642, 0x9593, 0x30a4, 0x30fc, 0x30b9, 0x30bf, 0x30fc, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30a4,
0x30fc, 0x30b9, 0x30bf, 0x30fc, 0x5cf6, 0x590f, 0x6642, 0x9593, 0x30a8, 0x30af, 0x30a2, 0x30c9,
0x30eb, 0x6642, 0x9593, 0x4e2d, 0x592e, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6642, 0x9593,
0x4e2d, 0x592e, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6a19, 0x6e96, 0x6642, 0x4e2d, 0x592e,
0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x590f, 0x6642, 0x9593, 0x6771, 0x30e8, 0x30fc, 0x30ed,
0x30c3, 0x30d1, 0x6642, 0x9593, 0x6771, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6a19, 0x6e96,
0x6642, 0x6771, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x590f, 0x6642, 0x9593, 0x6975, 0x6771,
0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6642, 0x9593, 0x897f, 0x30e8, 0x30fc, 0x30ed, 0x30c3,
0x30d1, 0x6642, 0x9593, 0x897f, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6a19, 0x6e96, 0x6642,
0x897f, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x590f, 0x6642, 0x9593, 0x30d5, 0x30a9, 0x30fc,
0x30af, 0x30e9, 0x30f3, 0x30c9, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30d5, 0x30a9, 0x30fc, 0x30af,
0x30e9, 0x30f3, 0x30c9, 0x8af8, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30d5, 0x30a9, 0x30fc, 0x30af,
0x30e9, 0x30f3, 0x30c9, 0x8af8, 0x5cf6, 0x590f, 0x6642, 0x9593, 0x30d5, 0x30a3, 0x30b8, 0x30fc,
0x6642, 0x9593, 0x30d5, 0x30a3, 0x30b8, 0x30fc, 0x6a19, 0x6e96, 0x6642, 0x30d5, 0x30a3, 0x30b8,
0x30fc, 0x590f, 0x6642, 0x9593, 0x4ecf, 0x9818, 0x30ae, 0x30a2, 0x30ca, 0x6642, 0x9593, 0x4ecf,
0x9818, 0x5357, 0x65b9, 0x5357, 0x6975, 0x6642, 0x9593, 0x30ac, 0x30e9, 0x30d1, 0x30b4, 0x30b9,
0x6642, 0x9593, 0x30ac, 0x30f3, 0x30d3, 0x30a8, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b8, 0x30e7,
0x30fc, 0x30b8, 0x30a2, 0x6642, 0x9593, 0x30b8, 0x30e7, 0x30fc, 0x30b8, 0x30a2, 0x6a19, 0x6e96,
0x6642, 0x30b8, 0x30e7, 0x30fc, 0x30b8, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30ae, 0x30eb, 0x30d0,
0x30fc, 0x30c8, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b0, 0x30ea, 0x30cb, 0x30c3, 0x30b8, 0x6a19,
0x6e96, 0x6642, 0x30b0, 0x30ea, 0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x6771, 0x90e8, 0x6642,
0x9593, 0x30b0, 0x30ea, 0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x6771, 0x90e8, 0x6a19, 0x6e96,
0x6642, 0x30b0, 0x30ea, 0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x6771, 0x90e8, 0x590f, 0x6642,
0x9593, 0x30b0, 0x30ea, 0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x897f, 0x90e8, 0x6642, 0x9593,
0x30b0, 0x30ea, 0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642,
0x30b0, 0x30ea, 0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x897f, 0x90e8, 0x590f, 0x6642, 0x9593,
0x30b0, 0x30a2, 0x30e0, 0x6642, 0x9593, 0x6e7e, 0x5cb8, 0x6a19, 0x6e96, 0x6642, 0x30ac, 0x30a4,
0x30a2, 0x30ca, 0x6642, 0x9593, 0x30cf, 0x30ef, 0x30a4, 0x30fb, 0x30a2, 0x30ea, 0x30e5, 0x30fc,
0x30b7, 0x30e3, 0x30f3, 0x6642, 0x9593, 0x30cf, 0x30ef, 0x30a4, 0x30fb, 0x30a2, 0x30ea, 0x30e5,
0x30fc, 0x30b7, 0x30e3, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30cf, 0x30ef, 0x30a4, 0x30fb, 0x30a2,
0x30ea, 0x30e5, 0x30fc, 0x30b7, 0x30e3, 0x30f3, 0x590f, 0x6642, 0x9593, 0x9999, 0x6e2f, 0x590f,
0x6642, 0x9593, 0x30db, 0x30d6, 0x30c9, 0x6642, 0x9593, 0x30db, 0x30d6, 0x30c9, 0x6a19, 0x6e96,
0x6642, 0x30db, 0x30d6, 0x30c9, 0x590f, 0x6642, 0x9593, 0x30a4, 0x30f3, 0x30c9, 0x6a19, 0x6e96,
0x6642, 0x30a4, 0x30f3, 0x30c9, 0x6d0b, 0x6642, 0x9593, 0x30a4, 0x30f3, 0x30c9, 0x30b7, 0x30ca,
0x6642, 0x9593, 0x30a4, 0x30f3, 0x30c9, 0x30cd, 0x30b7, 0x30a2, 0x4e2d, 0x90e8, 0x6642, 0x9593,
0x30a4, 0x30f3, 0x30c9, 0x30cd, 0x30b7, 0x30a2, 0x6771, 0x90e8, 0x6642, 0x9593, 0x30a4, 0x30f3,
0x30c9, 0x30cd, 0x30b7, 0x30a2, 0x897f, 0x90e8, 0x6642, 0x9593, 0x30a4, 0x30e9, 0x30f3, 0x6642,
0x9593, 0x30a4, 0x30e9, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30e9, 0x30f3, 0x590f, 0x6642,
0x9593, 0x30a4, 0x30eb, 0x30af, 0x30fc, 0x30c4, 0x30af, 0x6642, 0x9593, 0x30a4, 0x30eb, 0x30af,
0x30fc, 0x30c4, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30eb, 0x30af, 0x30fc, 0x30c4, 0x30af,
0x590f, 0x6642, 0x9593, 0x30a4, 0x30b9, 0x30e9, 0x30a8, 0x30eb, 0x6642, 0x9593, 0x30a4, 0x30b9,
0x30e9, 0x30a8, 0x30eb, 0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30b9, 0x30e9, 0x30a8, 0x30eb, 0x590f,
0x6642, 0x9593, 0x65e5, 0x672c, 0x590f, 0x6642, 0x9593, 0x30da, 0x30c8, 0x30ed, 0x30d1, 0x30d6,
0x30ed, 0x30d5, 0x30b9, 0x30af, 0x30fb, 0x30ab, 0x30e0, 0x30c1, 0x30e3, 0x30c4, 0x30ad, 0x30fc,
0x6642, 0x9593, 0x30da, 0x30c8, 0x30ed, 0x30d1, 0x30d6, 0x30ed, 0x30d5, 0x30b9, 0x30af, 0x30fb,
0x30ab, 0x30e0, 0x30c1, 0x30e3, 0x30c4, 0x30ad, 0x30fc, 0x6a19, 0x6e96, 0x6642, 0x30da, 0x30c8,
0x30ed, 0x30d1, 0x30d6, 0x30ed, 0x30d5, 0x30b9, 0x30af, 0x30fb, 0x30ab, 0x30e0, 0x30c1, 0x30e3,
0x30c4, 0x30ad, 0x30fc, 0x590f, 0x6642, 0x9593, 0x30ab, 0x30b6, 0x30d5, 0x30b9, 0x30bf, 0x30f3,
0x6642, 0x9593, 0x6771, 0x30ab, 0x30b6, 0x30d5, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x897f,
0x30ab, 0x30b6, 0x30d5, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x97d3, 0x56fd, 0x6642, 0x9593,
0x97d3, 0x56fd, 0x6a19, 0x6e96, 0x6642, 0x97d3, 0x56fd, 0x590f, 0x6642, 0x9593, 0x30b3, 0x30b9,
0x30e9, 0x30a8, 0x6642, 0x9593, 0x30af, 0x30e9, 0x30b9, 0x30ce, 0x30e4, 0x30eb, 0x30b9, 0x30af,
0x6642, 0x9593, 0x30af, 0x30e9, 0x30b9, 0x30ce, 0x30e4, 0x30eb, 0x30b9, 0x30af, 0x6a19, 0x6e96,
0x6642, 0x30af, 0x30e9, 0x30b9, 0x30ce, 0x30e4, 0x30eb, 0x30b9, 0x30af, 0x590f, 0x6642, 0x9593,
0x30ad, 0x30eb, 0x30ae, 0x30b9, 0x6642, 0x9593, 0x30e9, 0x30f3, 0x30ab, 0x6642, 0x9593, 0x30e9,
0x30a4, 0x30f3, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30ed, 0x30fc, 0x30c9, 0x30cf, 0x30a6, 0x6642,
0x9593, 0x30ed, 0x30fc, 0x30c9, 0x30cf, 0x30a6, 0x6a19, 0x6e96, 0x6642, 0x30ed, 0x30fc, 0x30c9,
0x30cf, 0x30a6, 0x590f, 0x6642, 0x9593, 0x30de, 0x30ab, 0x30aa, 0x6642, 0x9593, 0x30de, 0x30ab,
0x30aa, 0x6a19, 0x6e96, 0x6642, 0x30de, 0x30ab, 0x30aa, 0x590f, 0x6642, 0x9593, 0x30de, 0x30ac,
0x30c0, 0x30f3, 0x6642, 0x9593, 0x30de, 0x30ac, 0x30c0, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30de,
0x30ac, 0x30c0, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30de, 0x30ec, 0x30fc, 0x30b7, 0x30a2, 0x6642,
0x9593, 0x30e2, 0x30eb, 0x30c7, 0x30a3, 0x30d6, 0x6642, 0x9593, 0x30de, 0x30eb, 0x30ad, 0x30fc,
0x30ba, 0x6642, 0x9593, 0x30de, 0x30fc, 0x30b7, 0x30e3, 0x30eb, 0x8af8, 0x5cf6, 0x6642, 0x9593,
0x30e2, 0x30fc, 0x30ea, 0x30b7, 0x30e3, 0x30b9, 0x6642, 0x9593, 0x30e2, 0x30fc, 0x30ea, 0x30b7,
0x30e3, 0x30b9, 0x6a19, 0x6e96, 0x6642, 0x30e2, 0x30fc, 0x30ea, 0x30b7, 0x30e3, 0x30b9, 0x590f,
0x6642, 0x9593, 0x30e2, 0x30fc, 0x30bd, 0x30f3, 0x57fa, 0x5730, 0x6642, 0x9593, 0x30e1, 0x30ad,
0x30b7, 0x30b3, 0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x30e1, 0x30ad, 0x30b7, 0x30b3, 0x592a,
0x5e73, 0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x30e1, 0x30ad, 0x30b7, 0x30b3, 0x592a, 0x5e73, 0x6d0b,
0x590f, 0x6642, 0x9593, 0x30a6, 0x30e9, 0x30f3, 0x30d0, 0x30fc, 0x30c8, 0x30eb, 0x6642, 0x9593,
0x30a6, 0x30e9, 0x30f3, 0x30d0, 0x30fc, 0x30c8, 0x30eb, 0x6a19, 0x6e96, 0x6642, 0x30a6, 0x30e9,
0x30f3, 0x30d0, 0x30fc, 0x30c8, 0x30eb, 0x590f, 0x6642, 0x9593, 0x30e2, 0x30b9, 0x30af, 0x30ef,
0x6642, 0x9593, 0x30e2, 0x30b9, 0x30af, 0x30ef, 0x6a19, 0x6e96, 0x6642, 0x30e2, 0x30b9, 0x30af,
0x30ef, 0x590f, 0x6642, 0x9593, 0x30df, 0x30e3, 0x30f3, 0x30de, 0x30fc, 0x6642, 0x9593, 0x30ca,
0x30a6, 0x30eb, 0x6642, 0x9593, 0x30cd, 0x30d1, 0x30fc, 0x30eb, 0x6642, 0x9593, 0x30cb, 0x30e5,
0x30fc, 0x30ab, 0x30ec, 0x30c9, 0x30cb, 0x30a2, 0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30ab,
0x30ec, 0x30c9, 0x30cb, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30cb, 0x30e5, 0x30fc, 0x30ab, 0x30ec,
0x30c9, 0x30cb, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30b8, 0x30fc, 0x30e9,
0x30f3, 0x30c9, 0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30b8, 0x30fc, 0x30e9, 0x30f3, 0x30c9,
0x6a19, 0x6e96, 0x6642, 0x30cb, 0x30e5, 0x30fc, 0x30b8, 0x30fc, 0x30e9, 0x30f3, 0x30c9, 0x590f,
0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30d5, 0x30a1, 0x30f3, 0x30c9, 0x30e9, 0x30f3, 0x30c9,
0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30d5, 0x30a1, 0x30f3, 0x30c9, 0x30e9, 0x30f3, 0x30c9,
0x6a19, 0x6e96, 0x6642, 0x30cb, 0x30e5, 0x30fc, 0x30d5, 0x30a1, 0x30f3, 0x30c9, 0x30e9, 0x30f3,
0x30c9, 0x590f, 0x6642, 0x9593, 0x30cb, 0x30a6, 0x30a8, 0x6642, 0x9593, 0x30ce, 0x30fc, 0x30d5,
0x30a9, 0x30fc, 0x30af, 0x5cf6, 0x6642, 0x9593, 0x30ce, 0x30fc, 0x30d5, 0x30a9, 0x30fc, 0x30af,
0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30ce, 0x30fc, 0x30d5, 0x30a9, 0x30fc, 0x30af, 0x5cf6, 0x590f,
0x6642, 0x9593, 0x30d5, 0x30a7, 0x30eb, 0x30ca, 0x30f3, 0x30c9, 0x30fb, 0x30c7, 0x30fb, 0x30ce,
0x30ed, 0x30fc, 0x30cb, 0x30e3, 0x6642, 0x9593, 0x30d5, 0x30a7, 0x30eb, 0x30ca, 0x30f3, 0x30c9,
0x30fb, 0x30c7, 0x30fb, 0x30ce, 0x30ed, 0x30fc, 0x30cb, 0x30e3, 0x6a19, 0x6e96, 0x6642, 0x30d5,
0x30a7, 0x30eb, 0x30ca, 0x30f3, 0x30c9, 0x30fb, 0x30c7, 0x30fb, 0x30ce, 0x30ed, 0x30fc, 0x30cb,
0x30e3, 0x590f, 0x6642, 0x9593, 0x5317, 0x30de, 0x30ea, 0x30a2, 0x30ca, 0x8af8, 0x5cf6, 0x6642,
0x9593, 0x30ce, 0x30f4, 0x30a9, 0x30b7, 0x30d3, 0x30eb, 0x30b9, 0x30af, 0x6642, 0x9593, 0x30ce,
0x30f4, 0x30a9, 0x30b7, 0x30d3, 0x30eb, 0x30b9, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30ce, 0x30f4,
0x30a9, 0x30b7, 0x30d3, 0x30eb, 0x30b9, 0x30af, 0x590f, 0x6642, 0x9593, 0x30aa, 0x30e0, 0x30b9,
0x30af, 0x6642, 0x9593, 0x30aa, 0x30e0, 0x30b9, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30aa, 0x30e0,
0x30b9, 0x30af, 0x590f, 0x6642, 0x9593, 0x30d1, 0x30ad, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593,
0x30d1, 0x30ad, 0x30b9, 0x30bf, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30d1, 0x30ad, 0x30b9, 0x30bf,
0x30f3, 0x590f, 0x6642, 0x9593, 0x30d1, 0x30e9, 0x30aa, 0x6642, 0x9593, 0x30d1, 0x30d7, 0x30a2,
0x30cb, 0x30e5, 0x30fc, 0x30ae, 0x30cb, 0x30a2, 0x6642, 0x9593, 0x30d1, 0x30e9, 0x30b0, 0x30a2,
0x30a4, 0x6642, 0x9593, 0x30d1, 0x30e9, 0x30b0, 0x30a2, 0x30a4, 0x6a19, 0x6e96, 0x6642, 0x30d1,
0x30e9, 0x30b0, 0x30a2, 0x30a4, 0x590f, 0x6642, 0x9593, 0x30da, 0x30eb, 0x30fc, 0x6642, 0x9593,
0x30da, 0x30eb, 0x30fc, 0x6a19, 0x6e96, 0x6642, 0x30da, 0x30eb, 0x30fc, 0x590f, 0x6642, 0x9593,
0x30d5, 0x30a3, 0x30ea, 0x30d4, 0x30f3, 0x6642, 0x9593, 0x30d5, 0x30a3, 0x30ea, 0x30d4, 0x30f3,
0x6a19, 0x6e96, 0x6642, 0x30d5, 0x30a3, 0x30ea, 0x30d4, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30d5,
0x30a7, 0x30cb, 0x30c3, 0x30af, 0x30b9, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b5, 0x30f3, 0x30d4,
0x30a8, 0x30fc, 0x30eb, 0x5cf6, 0x30fb, 0x30df, 0x30af, 0x30ed, 0x30f3, 0x5cf6, 0x6642, 0x9593,
0x30b5, 0x30f3, 0x30d4, 0x30a8, 0x30fc, 0x30eb, 0x5cf6, 0x30fb, 0x30df, 0x30af, 0x30ed, 0x30f3,
0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30b5, 0x30f3, 0x30d4, 0x30a8, 0x30fc, 0x30eb, 0x5cf6, 0x30fb,
0x30df, 0x30af, 0x30ed, 0x30f3, 0x5cf6, 0x590f, 0x6642, 0x9593, 0x30d4, 0x30c8, 0x30b1, 0x30a2,
0x30f3, 0x6642, 0x9593, 0x30dd, 0x30ca, 0x30da, 0x6642, 0x9593, 0x5e73, 0x58cc, 0x6642, 0x9593,
0x30af, 0x30ba, 0x30ed, 0x30eb, 0x30c0, 0x6642, 0x9593, 0x30af, 0x30ba, 0x30ed, 0x30eb, 0x30c0,
0x6a19, 0x6e96, 0x6642, 0x30af, 0x30ba, 0x30ed, 0x30eb, 0x30c0, 0x590f, 0x6642, 0x9593, 0x30ec,
0x30e6, 0x30cb, 0x30aa, 0x30f3, 0x6642, 0x9593, 0x30ed, 0x30bc, 0x30e9, 0x57fa, 0x5730, 0x6642,
0x9593, 0x30b5, 0x30cf, 0x30ea, 0x30f3, 0x6642, 0x9593, 0x30b5, 0x30cf, 0x30ea, 0x30f3, 0x6a19,
0x6e96, 0x6642, 0x30b5, 0x30cf, 0x30ea, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30b5, 0x30de, 0x30e9,
0x6642, 0x9593, 0x30b5, 0x30de, 0x30e9, 0x6a19, 0x6e96, 0x6642, 0x30b5, 0x30de, 0x30e9, 0x590f,
0x6642, 0x9593, 0x30b5, 0x30e2, 0x30a2, 0x6642, 0x9593, 0x30b5, 0x30e2, 0x30a2, 0x6a19, 0x6e96,
0x6642, 0x30b5, 0x30e2, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30bb, 0x30fc, 0x30b7, 0x30a7, 0x30eb,
0x6642, 0x9593, 0x30b7, 0x30f3, 0x30ac, 0x30dd, 0x30fc, 0x30eb, 0x6a19, 0x6e96, 0x6642, 0x30bd,
0x30ed, 0x30e2, 0x30f3, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b5, 0x30a6, 0x30b9, 0x30b8, 0x30e7,
0x30fc, 0x30b8, 0x30a2, 0x6642, 0x9593, 0x30b9, 0x30ea, 0x30ca, 0x30e0, 0x6642, 0x9593, 0x30bf,
0x30d2, 0x30c1, 0x6642, 0x9593, 0x53f0, 0x5317, 0x590f, 0x6642, 0x9593, 0x30bf, 0x30b8, 0x30ad,
0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x30c8, 0x30b1, 0x30e9, 0x30a6, 0x6642, 0x9593, 0x30c8,
0x30f3, 0x30ac, 0x6642, 0x9593, 0x30c8, 0x30f3, 0x30ac, 0x6a19, 0x6e96, 0x6642, 0x30c8, 0x30f3,
0x30ac, 0x590f, 0x6642, 0x9593, 0x30c1, 0x30e5, 0x30fc, 0x30af, 0x6642, 0x9593, 0x30c8, 0x30eb,
0x30af, 0x30e1, 0x30cb, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x30c8, 0x30eb, 0x30af, 0x30e1,
0x30cb, 0x30b9, 0x30bf, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30c8, 0x30eb, 0x30af, 0x30e1, 0x30cb,
0x30b9, 0x30bf, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30c4, 0x30d0, 0x30eb, 0x6642, 0x9593, 0x30a6,
0x30eb, 0x30b0, 0x30a2, 0x30a4, 0x6642, 0x9593, 0x30a6, 0x30eb, 0x30b0, 0x30a2, 0x30a4, 0x6a19,
0x6e96, 0x6642, 0x30a6, 0x30eb, 0x30b0, 0x30a2, 0x30a4, 0x590f, 0x6642, 0x9593, 0x30a6, 0x30ba,
0x30d9, 0x30ad, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x30a6, 0x30ba, 0x30d9, 0x30ad, 0x30b9,
0x30bf, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30a6, 0x30ba, 0x30d9, 0x30ad, 0x30b9, 0x30bf, 0x30f3,
0x590f, 0x6642, 0x9593, 0x30d0, 0x30cc, 0x30a2, 0x30c4, 0x6642, 0x9593, 0x30d0, 0x30cc, 0x30a2,
0x30c4, 0x6a19, 0x6e96, 0x6642, 0x30d0, 0x30cc, 0x30a2, 0x30c4, 0x590f, 0x6642, 0x9593, 0x30d9,
0x30cd, 0x30ba, 0x30a8, 0x30e9, 0x6642, 0x9593, 0x30a6, 0x30e9, 0x30b8, 0x30aa, 0x30b9, 0x30c8,
0x30af, 0x6642, 0x9593, 0x30a6, 0x30e9, 0x30b8, 0x30aa, 0x30b9, 0x30c8, 0x30af, 0x6a19, 0x6e96,
0x6642, 0x30a6, 0x30e9, 0x30b8, 0x30aa, 0x30b9, 0x30c8, 0x30af, 0x590f, 0x6642, 0x9593, 0x30dc,
0x30eb, 0x30b4, 0x30b0, 0x30e9, 0x30fc, 0x30c9, 0x6642, 0x9593, 0x30dc, 0x30eb, 0x30b4, 0x30b0,
0x30e9, 0x30fc, 0x30c9, 0x6a19, 0x6e96, 0x6642, 0x30dc, 0x30eb, 0x30b4, 0x30b0, 0x30e9, 0x30fc,
0x30c9, 0x590f, 0x6642, 0x9593, 0x30dc, 0x30b9, 0x30c8, 0x30fc, 0x30af, 0x57fa, 0x5730, 0x6642,
0x9593, 0x30a6, 0x30a7, 0x30fc, 0x30af, 0x5cf6, 0x6642, 0x9593, 0x30a6, 0x30a9, 0x30ea, 0x30b9,
0x30fb, 0x30d5, 0x30c4, 0x30ca, 0x6642, 0x9593, 0x30e4, 0x30af, 0x30fc, 0x30c4, 0x30af, 0x6642,
0x9593, 0x30e4, 0x30af, 0x30fc, 0x30c4, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30e4, 0x30af, 0x30fc,
0x30c4, 0x30af, 0x590f, 0x6642, 0x9593, 0x30a8, 0x30ab, 0x30c6, 0x30ea, 0x30f3, 0x30d6, 0x30eb,
0x30b0, 0x6642, 0x9593, 0x30a8, 0x30ab, 0x30c6, 0x30ea, 0x30f3, 0x30d6, 0x30eb, 0x30b0, 0x6a19,
0x6e96, 0x6642, 0x30a8, 0x30ab, 0x30c6, 0x30ea, 0x30f3, 0x30d6, 0x30eb, 0x30b0, 0x590f, 0x6642,
0x9593, 0x30e6, 0x30fc, 0x30b3, 0x30f3, 0x6642, 0x9593, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x57, 0x65, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4b, 0x69,
0x64, 0x75, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65,
0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65,
0x74, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x69, 0x72,
0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x47, 0x69, 0x72, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x47, 0x69, 0x72, 0x69,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x41, 0x70, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41,
0x70, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x61,
0x62, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x72, 0x61, 0x62,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75,
0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c,
0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x73,
0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b,
0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x73, 0x69,
0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x69,
0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x69,
0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x72,
0x75, 0x6e, 0x61, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61,
0x6c, 0x61, 0x6d, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x6e,
0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x43, 0x68, 0x69, 0x6c, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x43,
0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e, 0x61, 0x74, 0x61, 0x6c,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f,
0x61, 0x6e, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x75, 0x62, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x75,
0x62, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x44, 0x61, 0x76, 0x69,
0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x4c,
0x65, 0x73, 0x74, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75,
0x6c, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50,
0x75, 0x6c, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50,
0x75, 0x6c, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x45, 0x6b, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x45, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69,
0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x69, 0x73,
0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69,
0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x65, 0x6e, 0x67,
0x20, 0x4c, 0x75, 0x77, 0x69, 0x68, 0x20, 0x41, 0x64, 0x6f, 0x68, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73,
0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20,
0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61,
0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f,
0x61, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x50, 0x72, 0x61,
0x6e, 0x63, 0x69, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6e,
0x74, 0x61, 0x72, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x6c, 0x61, 0x6e, 0x20,
0x50, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x73, 0x20, 0x73, 0x69, 0x73, 0x69,
0x68, 0x20, 0x4b, 0x69, 0x64, 0x75, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x52, 0x65, 0x72,
0x61, 0x74, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63,
0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x47,
0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x57, 0x65, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x57,
0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x72,
0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4b,
0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x65, 0x6c,
0x75, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x75, 0x79, 0x61,
0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x65, 0x67, 0x6f, 0x72, 0x6f,
0x20, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x73,
0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4a, 0x65, 0x70,
0x61, 0x6e, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e, 0x67, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x69,
0x72, 0x67, 0x69, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x4c,
0x69, 0x6e, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x64, 0x65, 0x77,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75,
0x65, 0x73, 0x61, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x6f, 0x77, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79,
0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79,
0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e,
0x75, 0x67, 0x69, 0x6e, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x72, 0x75,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x50,
0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x6c, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x6c, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6c, 0x61, 0x6e, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x65, 0x69, 0x63, 0x68, 0x65, 0x6c, 0x6c,
0x65, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x20, 0x4b, 0x69, 0x64, 0x75, 0x6c, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x6a, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x6c,
0x6f, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x6e, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x53, 0x75, 0x6c, 0x20, 0x64, 0x69, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0xc1, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64,
0x69, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0xc1, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20,
0x64, 0x69, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72,
0x6f, 0x6e, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72,
0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x61, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61,
0x73, 0xed, 0x66, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61,
0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0xed,
0x66, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0xed,
0x66, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41,
0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x74,
0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41,
0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d,
0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61,
0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x65, 0x6e,
0x74, 0x72, 0x75, 0x2d, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d, 0x4f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72,
0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x61, 0x62, 0x75, 0x20,
0x56, 0x65, 0x72, 0x64, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x61, 0x62, 0x75, 0x20,
0x56, 0x65, 0x72, 0x64, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x62,
0x75, 0x20, 0x56, 0x65, 0x72, 0x64, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61,
0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x75, 0x62, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e,
0x20, 0x64, 0x69, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20,
0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64,
0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x73, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x4d, 0xe9, 0x64, 0x69, 0x75, 0x20,
0x64, 0x69, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65,
0x6c, 0xe1, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe1,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe1,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e,
0x65, 0x6c, 0xe1, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65,
0x6c, 0xe1, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e,
0x65, 0x6c, 0xe1, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x41, 0x76, 0x61, 0xed, 0x20, 0x69, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e,
0x20, 0x64, 0x69, 0x20, 0x41, 0x76, 0x61, 0xed, 0x20, 0x69, 0x20, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x76,
0x61, 0xed, 0x20, 0x69, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66,
0x69, 0x6b, 0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75,
0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64,
0x69, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x50,
0x61, 0x73, 0xed, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x6b, 0x61, 0x6e, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54,
0x65, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x54, 0x65,
0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x54,
0x65, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x75,
0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20,
0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x75, 0x20, 0x69, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x53,
0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x75, 0x20, 0x69, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x41, 0x66, 0x263, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71,
0x74, 0x20, 0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71,
0x74, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71, 0x74, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x1e93, 0x75, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x54, 0x65, 0x66, 0x72, 0x69, 0x71, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d,
0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71,
0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71, 0x74, 0x20, 0x6e, 0x20, 0x75,
0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x1e93, 0x75,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x1e93, 0x75, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x41, 0x6d, 0x61, 0x1e93, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x74, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20,
0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x74, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x4d,
0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x74, 0x6c,
0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d,
0x61, 0x72, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x20,
0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x79, 0x69, 0x64, 0x75, 0x72, 0x61, 0x72, 0x20,
0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x79, 0x69, 0x64, 0x75, 0x72, 0x61, 0x72, 0x20, 0x6e, 0x20, 0x4d, 0x61,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x79, 0x69, 0x64,
0x75, 0x72, 0x61, 0x72, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x6c, 0x77,
0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x61, 0x6d, 0x65, 0x6c, 0x77, 0x69, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x75, 0x6d, 0x65, 0x6c, 0x77, 0x69, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41,
0x70, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x70, 0x79, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x41, 0x70, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x25b, 0x72, 0x61, 0x62, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x25b, 0x72,
0x61, 0x62, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x25b, 0x72, 0x61, 0x62, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x41, 0x72, 0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41,
0x72, 0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41,
0x72, 0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x41, 0x72, 0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6e,
0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41,
0x72, 0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x75, 0x73,
0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6a,
0x75, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d,
0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x72,
0x6d, 0x69, 0x6e, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6d, 0x69,
0x6e, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6d, 0x69, 0x6e,
0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x1e6d, 0x6c, 0x61, 0x73,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x61, 0x1e6d, 0x6c, 0x61, 0x73, 0x61, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x1e6d, 0x6c, 0x61, 0x73, 0x61, 0x6e, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x74,
0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x55,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x74, 0x61, 0x6c, 0x65,
0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d,
0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x74, 0x6c,
0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61,
0x6c, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x74, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74,
0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x20, 0x6e, 0x20, 0x55,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x74,
0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d,
0x61, 0x6c, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d,
0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61,
0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x6e, 0x20, 0x75,
0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x65, 0x72, 0x62, 0x69, 0x1e7,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x65, 0x72, 0x62, 0x69, 0x1e7,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x65, 0x72, 0x62, 0x69,
0x1e7, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x1e93,
0x75, 0x72, 0x69, 0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x75, 0x72, 0x69,
0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x75, 0x72, 0x69, 0x73, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x61, 0x63, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x61, 0x63, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x61, 0x63, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x42,
0x75, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x42,
0x75, 0x6c, 0x69, 0x76, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x42, 0x72, 0x61, 0x1e93, 0x69, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x42,
0x72, 0x61, 0x1e93, 0x69, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x42,
0x72, 0x61, 0x1e93, 0x69, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x61, 0x79, 0x20, 0x44, 0x61, 0x72,
0x20, 0x53, 0x61, 0x6c, 0x61, 0x6d, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x69, 0x72, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4b,
0x61, 0x70, 0x20, 0x56, 0x69, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x61,
0x70, 0x20, 0x56, 0x69, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x43, 0x61, 0x6d, 0x75,
0x72, 0x72, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x43, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x43, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x43, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6c, 0x69, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x43,
0x69, 0x6c, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6c, 0x69, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x43, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69,
0x6e, 0x20, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x73, 0x6d, 0x61, 0x73, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72,
0x69, 0x6e, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x6b, 0x75, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x6c, 0x75, 0x6d, 0x62, 0x79, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x4b, 0x75, 0x6c, 0x75, 0x6d, 0x62, 0x79, 0x61, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x4b, 0x75, 0x6c, 0x75, 0x6d, 0x62, 0x79, 0x61, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e,
0x20, 0x4b, 0x75, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a,
0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x6b, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x4b, 0x75, 0x62, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x62, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x20,
0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d,
0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x49, 0x6b, 0x77, 0x61, 0x1e6d, 0x75,
0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75,
0x66, 0x74, 0x20, 0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x74, 0x61,
0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54,
0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d,
0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x75,
0x72, 0x75, 0x66, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d,
0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20,
0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x73,
0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x6e,
0x69, 0x1e0d, 0x65, 0x6e, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66,
0x74, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74,
0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54,
0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c,
0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x6e, 0x20,
0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x65,
0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69,
0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x47,
0x77, 0x69, 0x79, 0x61, 0x6e, 0x20, 0x74, 0x61, 0x66, 0x72, 0x61, 0x6e,
0x73, 0x69, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x77,
0x61, 0x6b, 0x61, 0x6c, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x1e93, 0x75, 0x6c,
0x20, 0x64, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b,
0x20, 0x6e, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x61, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x75,
0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a,
0x69, 0x72, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x79, 0x65, 0x72,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4a, 0x79, 0x75, 0x72, 0x6a,
0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x4a, 0x79, 0x75, 0x72, 0x6a, 0x79, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x4a, 0x79, 0x75, 0x72, 0x6a, 0x79, 0x61, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72,
0x69, 0x6e, 0x20, 0x4a, 0x69, 0x6c, 0x62, 0x61, 0x72, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x20, 0x6e, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x47,
0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73,
0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x47, 0x75,
0x6c, 0x66, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x48, 0x61, 0x77,
0x61, 0x79, 0x2d, 0x41, 0x6c, 0x69, 0x77, 0x73, 0x79, 0x61, 0x6e, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x69, 0x77,
0x73, 0x79, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61,
0x79, 0x2d, 0x41, 0x6c, 0x69, 0x77, 0x73, 0x79, 0x61, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x48, 0x75, 0x6e, 0x67, 0x20, 0x4b, 0x75,
0x6e, 0x67, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x48, 0x75, 0x6e, 0x67, 0x20, 0x4b, 0x75,
0x6e, 0x67, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x48, 0x75, 0x6e, 0x67, 0x20, 0x4b,
0x75, 0x6e, 0x67, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x48, 0x75,
0x76, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x48, 0x75, 0x76, 0x64, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x48, 0x75, 0x76, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4c, 0x68, 0x65, 0x6e,
0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x67, 0x61, 0x72,
0x61, 0x77, 0x20, 0x41, 0x68, 0x65, 0x6e, 0x64, 0x69, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x75, 0x63, 0x69, 0x6e, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x74, 0x6c, 0x65, 0x6d, 0x6d, 0x61,
0x73, 0x74, 0x20, 0x6e, 0x20, 0x41, 0x6e, 0x64, 0x75, 0x6e, 0x69, 0x73,
0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61,
0x6d, 0x6d, 0x61, 0x72, 0x20, 0x6e, 0x20, 0x41, 0x6e, 0x64, 0x75, 0x6e,
0x69, 0x73, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6d, 0x61, 0x6c, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x6e, 0x64, 0x75, 0x6e,
0x69, 0x73, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x49, 0x7a,
0x72, 0x61, 0x79, 0x69, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x49, 0x7a, 0x72, 0x61,
0x79, 0x69, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x79,
0x69, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4a, 0x61, 0x70,
0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x4a, 0x61, 0x70, 0x75, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x4a, 0x61, 0x70, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x20, 0x6e, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x78, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75,
0x20, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x78, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x72, 0x79,
0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x72, 0x79, 0x61, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x4b, 0x75, 0x72, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x61, 0x67, 0x6e, 0x75,
0x20, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x4b, 0x69, 0x72, 0x69, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69,
0x6e, 0x20, 0x4c, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x4d, 0x61, 0x6c, 0x69, 0x7a, 0x79, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72,
0x69, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e,
0x20, 0x4d, 0x61, 0x72, 0x63, 0x61, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x4d, 0x75, 0x72, 0x69, 0x73, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4d, 0x75,
0x72, 0x69, 0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4d, 0x75, 0x72, 0x69, 0x73,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x75,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x6c, 0x77, 0x69,
0x20, 0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x73, 0x69, 0x6b, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6d,
0x65, 0x6c, 0x77, 0x69, 0x20, 0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x73, 0x69,
0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x6c, 0x77, 0x69,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x4d, 0x69, 0x6b, 0x73, 0x69, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4d, 0x75, 0x1e63, 0x6b, 0x75, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x4d, 0x75, 0x1e63, 0x6b, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4d,
0x75, 0x1e63, 0x6b, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4d,
0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x4e, 0x75, 0x72, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x4e, 0x69, 0x70, 0x61, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x4b, 0x61, 0x6c, 0x69, 0x64, 0x75, 0x6e, 0x79, 0x61, 0x20, 0x54, 0x61,
0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4b, 0x61, 0x6c,
0x69, 0x64, 0x75, 0x6e, 0x79, 0x61, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x79,
0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x64,
0x75, 0x6e, 0x79, 0x61, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75,
0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x5a, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x6d,
0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x61, 0x6c, 0x20, 0x61,
0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x57, 0x61, 0x6b,
0x61, 0x6c, 0x20, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x57, 0x61, 0x6b, 0x61, 0x6c, 0x20, 0x61, 0x6d, 0x61, 0x79,
0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4e, 0x69,
0x79, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67,
0x7a, 0x69, 0x72, 0x74, 0x20, 0x6e, 0x20, 0x4e, 0x75, 0x72, 0x66, 0x75,
0x6c, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x74,
0x20, 0x6e, 0x20, 0x4e, 0x75, 0x72, 0x66, 0x75, 0x6c, 0x6b, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x74, 0x20, 0x6e, 0x20,
0x4e, 0x75, 0x72, 0x66, 0x75, 0x6c, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x46, 0x69, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x4e, 0x75, 0x72, 0x75, 0x6e, 0x68, 0x61, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x46,
0x69, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4e, 0x75,
0x72, 0x75, 0x6e, 0x68, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4e, 0x75, 0x72, 0x75,
0x6e, 0x68, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20,
0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50,
0x61, 0x6c, 0x61, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50,
0x61, 0x70, 0x77, 0x61, 0x20, 0x6e, 0x20, 0x194, 0x69, 0x6e, 0x79, 0x61,
0x20, 0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x79,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x79, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x79, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50, 0x69, 0x72, 0x75, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x50, 0x69, 0x72, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x50, 0x69, 0x72,
0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69,
0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6e, 0x69, 0x6b, 0x73,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50,
0x79, 0x69, 0x72, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b, 0x6c, 0x75, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x79, 0x69, 0x72, 0x20,
0x26, 0x20, 0x4d, 0x69, 0x6b, 0x6c, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x53, 0x61, 0x6e, 0x20, 0x50, 0x79, 0x69, 0x72, 0x20, 0x26, 0x20, 0x4d,
0x69, 0x6b, 0x6c, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x50, 0x69, 0x74, 0x6b, 0x61, 0x72, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x50, 0x79, 0x75, 0x6e, 0x67, 0x79, 0x75, 0x6e, 0x67,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x52, 0x69, 0x79, 0x75, 0x6e,
0x79, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x53, 0x61, 0x6d, 0x77, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6d,
0x77, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x77, 0x61, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x79, 0x63, 0x61, 0x6c,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x53, 0x61, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72,
0x69, 0x6e, 0x20, 0x53, 0x61, 0x6c, 0x75, 0x6d, 0x75, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x4a, 0x79, 0x75, 0x72, 0x6a, 0x79, 0x61,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x1e93, 0x75, 0x6c, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x53, 0x79, 0x75, 0x77, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x61, 0x79, 0x70, 0x61, 0x79,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x54, 0x61, 0x79, 0x70, 0x61, 0x79, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x54, 0x61, 0x79, 0x70, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x1e6c, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x1e6c, 0x75, 0x6b, 0x69, 0x6c,
0x61, 0x77, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x1e6c, 0x75, 0x6e,
0x67, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x1e6c, 0x75, 0x6e, 0x67, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x1e6c, 0x75, 0x6e, 0x67, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x79,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x79, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x79, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x69, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x79, 0x61, 0x74, 0x75, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x79, 0x61, 0x74, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x79, 0x61, 0x74, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x56, 0x69, 0x6e, 0x69, 0x7a, 0x77, 0x69, 0x6c, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41, 0x6d, 0x61,
0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x55, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20,
0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a,
0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x20, 0x61, 0x6b, 0x6b, 0x65, 0x64, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0xc8f, 0xc95, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc8f, 0xc95, 0xcb0, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc8f, 0xc95, 0xcb0, 0xccd, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcab, 0xc98, 0xcbe,
0xca8, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae,
0xca7, 0xccd, 0xcaf, 0x20, 0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc86, 0xcab, 0xccd,
0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca6, 0xc95, 0xccd, 0xcb7,
0xcbf, 0xca3, 0x20, 0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95,
0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20,
0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a,
0xcbf, 0xcae, 0x20, 0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb2, 0xcbe,
0xcb8, 0xccd, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb2, 0xcbe, 0xcb8,
0xccd, 0xc95, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0x200c, 0xc85, 0xcb2, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xcbe,
0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb2, 0xccd,
0xcae, 0xcc7, 0xc9f, 0xcbf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb2, 0xccd, 0xcae,
0xcc7, 0xc9f, 0xcbf, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb2, 0xccd, 0xcae, 0xcc7, 0xc9f, 0xcbf, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcae,
0xcc6, 0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcae, 0xcc6,
0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcae, 0xcc6, 0xc9c, 0xcbe, 0xca8, 0xccd,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89,
0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6,
0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89,
0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6,
0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4,
0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xc95, 0xcc7,
0xc82, 0xca6, 0xccd, 0xcb0, 0xcc0, 0xcaf, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6,
0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0xca6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6,
0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0xca6, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95,
0xca6, 0x20, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0xca6, 0x20, 0xca6, 0xcbf, 0xca8,
0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85,
0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcb0, 0xccd, 0xcb5, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae,
0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcb0, 0xccd, 0xcb5, 0xca4, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95,
0xca6, 0x20, 0xcaa, 0xcb0, 0xccd, 0xcb5, 0xca4, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae,
0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf,
0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20,
0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf,
0xcab, 0xcbf, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85,
0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab,
0xcbf, 0xc95, 0xccd, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xca8, 0xca1, 0xcc0, 0xcb0, 0xccd, 0x200c, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85,
0xca8, 0xca1, 0xcc0, 0xcb0, 0xccd, 0x200c, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xca8, 0xca1, 0xcc0, 0xcb0,
0xccd, 0x200c, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85,
0xcaa, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcaa, 0xcbf, 0xcaf,
0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc85, 0xcaa, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccc, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccc, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccc,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85,
0xc95, 0xccd, 0xc9f, 0xccb, 0xcac, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95,
0xccd, 0xc9f, 0xccb, 0xcac, 0xcc6, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccb, 0xcac,
0xcc6, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xcb0, 0xcc7, 0xcac, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xcb0, 0xcc7, 0xcac, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xcc7,
0xcac, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcc0, 0xca8, 0xcbe,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcc0,
0xca8, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcbf, 0xca8,
0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6,
0xc82, 0xc9f, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd,
0xc9a, 0xcbf, 0xcae, 0x20, 0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcc0,
0xca8, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc85, 0xcb0,
0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xcae, 0xcc7,
0xca8, 0xcbf, 0xcaf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xcae, 0xcc7,
0xca8, 0xcbf, 0xcaf, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xcae, 0xcc7, 0xca8, 0xcbf, 0xcaf,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85,
0xc9f, 0xccd, 0xcb2, 0xcbe, 0xc82, 0xc9f, 0xcbf, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc85, 0xc9f, 0xccd, 0xcb2, 0xcbe, 0xc82, 0xc9f, 0xcbf, 0xc95, 0xccd, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xc9f, 0xccd, 0xcb2, 0xcbe, 0xc82, 0xc9f, 0xcbf, 0xc95, 0xccd, 0x20, 0xca6,
0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd,
0xcb0, 0x20, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf,
0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7,
0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe,
0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcb9, 0xc97, 0xcb2,
0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7,
0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0,
0x20, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86,
0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20,
0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf,
0xcae, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf,
0xcbe, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcaa, 0xcb6,
0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd,
0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8,
0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xcaa,
0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc86,
0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20,
0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a,
0xcbf, 0xcae, 0x20, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf,
0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0,
0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf,
0xcae, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf,
0xcbe, 0xca6, 0x20, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcb9, 0xc97,
0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9c, 0xcb0, 0xccd, 0xcac, 0xcc8,
0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9c, 0xcb0, 0xccd,
0xcac, 0xcc8, 0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9c, 0xcb0, 0xccd, 0xcac,
0xcc8, 0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9c, 0xccb, 0xcb0, 0xcb8, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc85, 0xc9c, 0xccb, 0xcb0, 0xcb8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9c, 0xccb,
0xcb0, 0xcb8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcac, 0xcbe, 0xc82, 0xc97, 0xccd, 0xcb2, 0xcbe, 0xca6, 0xcc7, 0xcb6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xcbe, 0xc82, 0xc97, 0xccd, 0xcb2, 0xcbe, 0xca6,
0xcc7, 0xcb6, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcac, 0xcbe, 0xc82, 0xc97, 0xccd, 0xcb2, 0xcbe, 0xca6, 0xcc7,
0xcb6, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcad, 0xcc2, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xcca,
0xcb2, 0xcbf, 0xcb5, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xccd,
0xcb0, 0xcc6, 0xcb8, 0xcbf, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcac, 0xccd, 0xcb0, 0xcc6, 0xcb8, 0xcbf, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcac, 0xccd, 0xcb0, 0xcc6, 0xcb8, 0xcbf, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xccd, 0xcb0,
0xcc2, 0xca8, 0xcbf, 0x20, 0xca6, 0xcbe, 0xcb0, 0xcc1, 0xcb8, 0xcb2, 0xcc6, 0xcae,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc7, 0xcaa, 0xccd, 0x20, 0xcb5, 0xcb0,
0xccd, 0xca1, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc7, 0xcaa, 0xccd, 0x20,
0xcb5, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc7, 0xcaa, 0xccd, 0x20, 0xcb5,
0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcae, 0xccb, 0xcb0, 0xccb, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xca5, 0xcbe,
0xcae, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xca5, 0xcbe, 0xcae, 0xccd, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc9a, 0xca5, 0xcbe, 0xcae, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc9a, 0xcbf, 0xcb2, 0xcbf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcbf,
0xcb2, 0xcbf, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcbf, 0xcb2, 0xcbf, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc9a, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcc0, 0xca8, 0xcbe, 0x20,
0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcb0, 0xcbf,
0xcb8, 0xccd, 0xcae, 0xcb8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc95, 0xccb, 0xc95, 0xccb, 0xcb8, 0xccd, 0x20, 0xca6, 0xccd,
0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb2,
0xc82, 0xcac, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb2,
0xc82, 0xcac, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb2, 0xc82, 0xcac, 0xcbf,
0xcaf, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc95, 0xcc1, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97,
0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc1, 0xc95, 0xccd, 0x20, 0xca6, 0xccd,
0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc1, 0xc95, 0xccd, 0x20, 0xca6,
0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcae, 0xca7, 0xccd, 0xcaf, 0xc95,
0xcbe, 0xcb2, 0xcc0, 0xca8, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcaf, 0xcc2, 0xcac, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc95, 0xccd, 0xcaf, 0xcc2, 0xcac, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcaf, 0xcc2,
0xcac, 0xcbe, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca1,
0xcc7, 0xcb5, 0xcbf, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca1, 0xcc1, 0xcae,
0xc82, 0xc9f, 0xccd, 0x2d, 0xca1, 0xcbf, 0x20, 0xc89, 0xcb0, 0xccd, 0xcb5, 0xcbf,
0xcb2, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20,
0xc9f, 0xcbf, 0xcae, 0xccb, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc88, 0xcb8,
0xccd, 0xc9f, 0xcb0, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc88, 0xcb8, 0xccd, 0xc9f, 0xcb0, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc88, 0xcb8, 0xccd, 0xc9f, 0xcb0, 0xccd, 0x20, 0xca6, 0xccd,
0xcb5, 0xcc0, 0xcaa, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc88, 0xc95, 0xccd, 0xcb5, 0xcc6, 0xca1, 0xcbe, 0xcb0, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcae, 0xca7, 0xccd, 0xcaf, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb,
0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xca7, 0xccd,
0xcaf, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcae, 0xca7, 0xccd, 0xcaf, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf,
0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa,
0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd,
0xcb5, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf,
0xcaf, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcae, 0xca4, 0xccd, 0xca4, 0xcb7, 0xccd, 0xc9f, 0xcc1, 0x2d, 0xcaa,
0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf,
0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae,
0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcaf, 0xcc1, 0xcb0,
0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf,
0xcae, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbe,
0xc95, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6,
0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbe,
0xc95, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6,
0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbe, 0xc95, 0xccd, 0x200c,
0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0,
0xcaa, 0xc97, 0xcb3, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcab, 0xcbf, 0xc9c, 0xcbf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbf,
0xc9c, 0xcbf, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbf, 0xc9c, 0xcbf, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xccd, 0xcb0, 0xcc6, 0xc82, 0xc9a,
0xccd, 0x20, 0xc97, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca6,
0xc95, 0xccd, 0xcb7, 0xcbf, 0xca3, 0x20, 0xcab, 0xccd, 0xcb0, 0xcc6, 0xc82, 0xc9a,
0xccd, 0x20, 0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xc85, 0xc82, 0xc9f, 0xcbe,
0xcb0, 0xccd, 0xc9f, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc97, 0xcbe,
0xcb2, 0xcbe, 0xcaa, 0xcbe, 0xc97, 0xccb, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc97, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xcac, 0xcbf, 0xcaf, 0xcb0, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc9c, 0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc9c, 0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9c,
0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc97, 0xcbf, 0xcb2, 0xccd, 0xcac, 0xcb0,
0xccd, 0xc9f, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb5, 0xcbf,
0xc9a, 0xccd, 0x20, 0xcb8, 0xcb0, 0xcbe, 0xcb8, 0xcb0, 0xcbf, 0x20, 0xc95, 0xcbe,
0xcb2, 0xcae, 0xcbe, 0xca8, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc97, 0xccd,
0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc97, 0xccd,
0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8,
0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a,
0xcbf, 0xcae, 0x20, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd,
0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd,
0xc9a, 0xcbf, 0xcae, 0x20, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2,
0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf,
0xcae, 0x20, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf,
0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc97, 0xcc1, 0xcb5, 0xcbe, 0xcae, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc97, 0xcb2,
0xccd, 0xcab, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xcbe, 0xcaf, 0xcbf, 0x2d, 0xc85, 0xcb2,
0xccd, 0xcaf, 0xcc1, 0xc9f, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcb9, 0xcb5, 0xcbe, 0xcaf, 0xcbf, 0x2d, 0xc85, 0xcb2, 0xccd, 0xcaf, 0xcc1, 0xc9f,
0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xcbe, 0xcaf, 0xcbf, 0x2d, 0xc85,
0xcb2, 0xccd, 0xcaf, 0xcc1, 0xc9f, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb9, 0xc97,
0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20,
0xc95, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbe, 0xc82,
0xc97, 0xccd, 0x20, 0xc95, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbe, 0xc82,
0xc97, 0xccd, 0x20, 0xc95, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xccd, 0xca1, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xccd, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5,
0xccd, 0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcad, 0xcbe, 0xcb0, 0xca4, 0xcc0, 0xcaf, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbf, 0xc82,
0xca6, 0xcc2, 0x20, 0xcae, 0xcb9, 0xcbe, 0xcb8, 0xcbe, 0xc97, 0xcb0, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc87, 0xc82, 0xca1, 0xcca, 0xc9a, 0xcc8, 0xca8, 0xcbe, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcae, 0xca7, 0xccd, 0xcaf, 0x20, 0xc87, 0xc82, 0xca1, 0xccb, 0xca8,
0xcc7, 0xcb7, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0,
0xccd, 0xcb5, 0x20, 0xc87, 0xc82, 0xca1, 0xccb, 0xca8, 0xcc7, 0xcb7, 0xcbf, 0xcaf,
0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20,
0xc87, 0xc82, 0xca1, 0xccb, 0xca8, 0xcc7, 0xcb7, 0xcbf, 0xcaf, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc87, 0xcb0, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0,
0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb9, 0xc97,
0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xccd, 0x200c, 0xc95, 0xcc1,
0xc9f, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xccd,
0x200c, 0xc95, 0xcc1, 0xc9f, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xccd,
0x200c, 0xc95, 0xcc1, 0xc9f, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb8, 0xccd, 0xcb0, 0xcc7,
0xcb2, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb8, 0xccd, 0xcb0, 0xcc7, 0xcb2,
0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc87, 0xcb8, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xccd, 0x20, 0xcb9, 0xc97,
0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9c, 0xcaa, 0xcbe, 0xca8, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc9c, 0xcaa, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9c, 0xcaa, 0xcbe,
0xca8, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcc6, 0xca4, 0xccd, 0xcb0, 0xccb, 0xcaa, 0xcbe, 0xcb5, 0xccd, 0xcb2, 0xccb, 0xcb8,
0xccd, 0xc95, 0xccd, 0x200c, 0x200c, 0x2d, 0xc95, 0xcae, 0xccd, 0xc9a, 0xca4, 0xccd,
0xcb8, 0xccd, 0x200c, 0xc95, 0xcc0, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6, 0xca4,
0xccd, 0xcb0, 0xccb, 0xcaa, 0xcbe, 0xcb5, 0xccd, 0xcb2, 0xccb, 0xcb8, 0xccd, 0xc95,
0xccd, 0x200c, 0x200c, 0x2d, 0xc95, 0xcae, 0xccd, 0xc9a, 0xca4, 0xccd, 0xcb8, 0xccd,
0x200c, 0xc95, 0xcc0, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6, 0xca4, 0xccd, 0xcb0, 0xccb, 0xcaa, 0xcbe,
0xcb5, 0xccd, 0xcb2, 0xccb, 0xcb8, 0xccd, 0xc95, 0xccd, 0x200c, 0x200c, 0x2d, 0xc95,
0xcae, 0xccd, 0xc9a, 0xca4, 0xccd, 0xcb8, 0xccd, 0x200c, 0xc95, 0xcc0, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xc9d, 0xcbe,
0xc96, 0xccd, 0x200c, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc95, 0xc9d, 0xcbe, 0xc96, 0xccd,
0x200c, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc95, 0xc9d, 0xcbe, 0xc96, 0xccd, 0x200c,
0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca,
0xcb0, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb0,
0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb0, 0xcbf, 0xcaf, 0xca8, 0xccd,
0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb8,
0xcb0, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcb0, 0xcbe, 0xcb8, 0xccd,
0x200c, 0xca8, 0xcca, 0xcaf, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcb0, 0xcbe, 0xcb8, 0xccd, 0x200c, 0xca8, 0xcca,
0xcaf, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcb0,
0xcbe, 0xcb8, 0xccd, 0x200c, 0xca8, 0xcca, 0xcaf, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xccd,
0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc95, 0xcbf, 0xcb0, 0xccd, 0xc97, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2, 0xc82, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcb2, 0xcc8, 0xca8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97,
0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2, 0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20,
0xcb9, 0xccb, 0xcb5, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2, 0xcbe, 0xcb0, 0xccd,
0xca1, 0xccd, 0x20, 0xcb9, 0xccb, 0xcb5, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2, 0xcbe, 0xcb0, 0xccd,
0xca1, 0xccd, 0x20, 0xcb9, 0xccb, 0xcb5, 0xccd, 0x20, 0xcac, 0xcc6, 0xcb3, 0xc97,
0xcbf, 0xca8, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xc95, 0xcbe, 0xcb5, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcae, 0xc95, 0xcbe, 0xcb5, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xc95, 0xcbe,
0xcb5, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcae, 0xc97, 0xca1, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae,
0xc97, 0xca1, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xc97, 0xca1, 0xcbe, 0xca8, 0xccd,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae,
0xcb2, 0xcc7, 0xcb7, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe,
0xcb2, 0xccd, 0xca1, 0xcc0, 0xcb5, 0xccd, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcae, 0xcbe, 0xcb0, 0xccd, 0xc95, 0xccd, 0xcaf, 0xcc1, 0xcb8, 0xcb8, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb0, 0xccd, 0xcb7, 0xcb2, 0xccd, 0x20, 0xca6,
0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe,
0xcb0, 0xcbf, 0xcb7, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb0,
0xcbf, 0xcb7, 0xcb8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb0, 0xcbf, 0xcb7, 0xcb8, 0xccd,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae,
0xcbe, 0xcb5, 0xccd, 0x200c, 0xcb8, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae,
0xcc6, 0xc95, 0xccd, 0xcb8, 0xcbf, 0xc95, 0xca8, 0xccd, 0x20, 0xcaa, 0xcc6, 0xcb8,
0xcbf, 0xcab, 0xcbf, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcc6, 0xc95,
0xccd, 0xcb8, 0xcbf, 0xc95, 0xca8, 0xccd, 0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab,
0xcbf, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcc6, 0xc95, 0xccd, 0xcb8, 0xcbf, 0xc95, 0xca8,
0xccd, 0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xc95, 0xccd, 0x20, 0xcb9,
0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb2, 0xcbe, 0xca8, 0xccd,
0x20, 0xcac, 0xca4, 0xcca, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb2,
0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xca4, 0xcca, 0xcb0, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb2,
0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xca4, 0xcca, 0xcb0, 0xccd, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb8, 0xccd,
0xc95, 0xccb, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccb,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccb, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd,
0xcae, 0xcbe, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xccc, 0xcb0, 0xcc1,
0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcc7, 0xcaa, 0xcbe, 0xcb3, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcb9, 0xcca, 0xcb8, 0x20, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcc6, 0xca1,
0xccb, 0xca8, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcca, 0xcb8,
0x20, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcc6, 0xca1, 0xccb, 0xca8, 0xcbf, 0xcaf,
0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb9, 0xcca, 0xcb8, 0x20, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcc6,
0xca1, 0xccb, 0xca8, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0xcaf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf, 0xcc2, 0xc9c, 0xcbf,
0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8,
0xccd, 0xcaf, 0xcc2, 0xc9c, 0xcbf, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xca8, 0xccd, 0xcaf, 0xcc2, 0xc9c, 0xcbf, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82,
0xca1, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8,
0xccd, 0xcaf, 0xcc2, 0xcab, 0xccc, 0xc82, 0xca1, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf,
0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf, 0xcc2,
0xcab, 0xccc, 0xc82, 0xca1, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1,
0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf, 0xcc2, 0xcab, 0xccc, 0xc82, 0xca1, 0xccd, 0x200c,
0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcbf, 0xcaf, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf,
0xca8, 0xcbe, 0xcb0, 0xccd, 0xcab, 0xccb, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcbe, 0xcb0, 0xccd, 0xcab, 0xccb,
0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcbe, 0xcb0,
0xccd, 0xcab, 0xccb, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcc6,
0xcb0, 0xccd, 0xca8, 0xcbe, 0xc82, 0xca1, 0xccb, 0x20, 0xca1, 0xcc6, 0x20, 0xca8,
0xcca, 0xcb0, 0xcca, 0xca8, 0xccd, 0xcb9, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab,
0xcc6, 0xcb0, 0xccd, 0xca8, 0xcbe, 0xc82, 0xca1, 0xccb, 0x20, 0xca1, 0xcc6, 0x20,
0xca8, 0xcca, 0xcb0, 0xcca, 0xca8, 0xccd, 0xcb9, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcb0, 0xccd,
0xca8, 0xcbe, 0xc82, 0xca1, 0xccb, 0x20, 0xca6, 0xcc6, 0x20, 0xca8, 0xcca, 0xcb0,
0xcca, 0xca8, 0xccd, 0xcb9, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xcae, 0xcb0,
0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97,
0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcca, 0xcb5, 0xcca, 0xcb8, 0xcbf, 0xcac,
0xcbf, 0xcb0, 0xccd, 0x200c, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xca8, 0xcca, 0xcb5, 0xcca, 0xcb8, 0xcbf, 0xcac, 0xcbf, 0xcb0, 0xccd, 0x200c, 0xcb8,
0xccd, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcca, 0xcb5, 0xcca, 0xcb8, 0xcbf, 0xcac, 0xcbf,
0xcb0, 0xccd, 0x200c, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc92, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc92, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccd,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc92, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcbe, 0xc95, 0xcbf, 0xcb8,
0xccd, 0xca4, 0xcbe, 0xca8, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcbe, 0xc95, 0xcbf,
0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcbe, 0xc95, 0xcbf, 0xcb8, 0xccd,
0xca4, 0xcbe, 0xca8, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcaa, 0xcbe, 0xcb2, 0xcbe, 0xcb5, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcaa, 0xcaa, 0xcc1, 0xcb5, 0xcbe, 0x20, 0xca8, 0xccd, 0xcaf, 0xcc2, 0x20, 0xc97,
0xcbf, 0xca8, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb0, 0xcbe,
0xc97, 0xccd, 0xcb5, 0xcc7, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb0, 0xcbe, 0xc97,
0xccd, 0xcb5, 0xcc7, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb0, 0xcbe, 0xc97, 0xccd, 0xcb5, 0xcc7, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6,
0xcb0, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6, 0xcb0, 0xcc1, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcc6, 0xcb0, 0xcc1, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcab, 0xcbf, 0xcb2, 0xcbf, 0xcab, 0xcc8, 0xca8, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcab, 0xcbf, 0xcb2, 0xcbf, 0xcab, 0xcc8, 0xca8, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab,
0xcbf, 0xcb2, 0xcbf, 0xcab, 0xcc8, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbf, 0xca8, 0xcbf, 0xc95, 0xccd,
0xcb8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcaa, 0xcbf, 0xcaf, 0xcb0,
0xccd, 0x20, 0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xcae, 0xcbf, 0xc95, 0xccd,
0xcb5, 0xcc6, 0xcb2, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcc7, 0xc82,
0xc9f, 0xccd, 0x20, 0xcaa, 0xcbf, 0xcaf, 0xcb0, 0xccd, 0x20, 0xcae, 0xca4, 0xccd,
0xca4, 0xcc1, 0x20, 0xcae, 0xcbf, 0xc95, 0xccd, 0xcb5, 0xcc6, 0xcb2, 0xca8, 0xccd,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcaa, 0xcbf, 0xcaf, 0xcb0, 0xccd,
0x20, 0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xcae, 0xcbf, 0xc95, 0xccd, 0xcb5,
0xcc6, 0xcb2, 0xca8, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaa, 0xcbf, 0xc9f, 0xccd, 0x200c, 0xc95, 0xcc8, 0xcb0, 0xccd, 0xca8, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcca, 0xca8, 0xcbe, 0xcaa, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcaa, 0xccd, 0xcaf, 0xcca, 0xc82, 0xc97, 0xccd, 0xcaf, 0xcbe, 0xc82,
0xc97, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccb, 0xc9c, 0xcbf, 0xcb2, 0xccb,
0xcb0, 0xccd, 0xca1, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccb, 0xc9c, 0xcbf,
0xcb2, 0xccb, 0xcb0, 0xccd, 0xca1, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccb, 0xc9c, 0xcbf, 0xcb2,
0xccb, 0xcb0, 0xccd, 0xca1, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb0, 0xcbf, 0xcaf, 0xcc2, 0xca8, 0xcbf, 0xcaf, 0xca8,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb0, 0xcca, 0xca4, 0xcc7, 0xcb0, 0xcbe, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd, 0x200c, 0xcb9, 0xcb2,
0xcbf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xc95,
0xccd, 0x200c, 0xcb9, 0xcb2, 0xcbf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xccd, 0xcaf, 0xcbe,
0xc95, 0xccd, 0x200c, 0xcb9, 0xcb2, 0xcbf, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xcb0, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb8, 0xcae, 0xcb0, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xcb0, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xccb, 0xcb5,
0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xccb, 0xcb5, 0xcbe, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8,
0xcae, 0xccb, 0xcb5, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcc0, 0xcb7, 0xcc6, 0xcb2, 0xccd, 0xcb8, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcbf, 0xc82, 0xc97, 0xcaa, 0xcc1, 0xcb0, 0xccd, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcb8, 0xcbe, 0xcb2, 0xcae, 0xca8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa,
0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca6, 0xc95, 0xccd, 0xcb7, 0xcbf, 0xca3,
0x20, 0xc9c, 0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcb8, 0xcc1, 0xcb0, 0xcbf, 0xca8, 0xcc7, 0xcae, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcb8, 0xccd, 0xcaf, 0xcca, 0xcb5, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4,
0xcbe, 0xcb9, 0xcbf, 0xca4, 0xcbf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc8, 0xcaa,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc8, 0xcaa, 0xcc6, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc8,
0xcaa, 0xcc6, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4,
0xc9d, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc9f, 0xcca, 0xc95, 0xcc6, 0xcb2, 0xcbe, 0xcb5, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc9f, 0xcca, 0xc82, 0xc97, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9f, 0xcca,
0xc82, 0xc97, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc9f, 0xcca, 0xc82, 0xc97, 0xcbe, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcc1, 0xc95, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc1, 0xcb0, 0xccd, 0xc95, 0xccd, 0x200c, 0xcae,
0xcc7, 0xca8, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xca4, 0xcc1, 0xcb0, 0xccd, 0xc95, 0xccd, 0x200c, 0xcae, 0xcc7, 0xca8, 0xcbf,
0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc1, 0xcb0, 0xccd, 0xc95,
0xccd, 0x200c, 0xcae, 0xcc7, 0xca8, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4,
0xcc1, 0xcb5, 0xcbe, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb0, 0xcc1,
0xc97, 0xccd, 0xcb5, 0xcc7, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb0, 0xcc1, 0xc97,
0xccd, 0xcb5, 0xcc7, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb0, 0xcc1, 0xc97, 0xccd, 0xcb5, 0xcc7, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xc9c,
0xccd, 0xcac, 0xcc7, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc89, 0xc9c, 0xccd, 0xcac, 0xcc7, 0xc95, 0xcbf, 0xcb8, 0xccd,
0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xc9c, 0xccd, 0xcac, 0xcc7, 0xc95, 0xcbf,
0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xca8, 0xccc, 0xca4, 0xcc1, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb5, 0xca8, 0xccc, 0xca4, 0xcc1, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xca8, 0xccc, 0xca4,
0xcc1, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcb5, 0xcc6, 0xca8, 0xcbf, 0xc9c, 0xcc1, 0xcb5, 0xcc6, 0xcb2, 0xcbe, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb5, 0xccd, 0xcb2, 0xcbe, 0xca1, 0xcbf, 0xcb5, 0xccb, 0xcb8, 0xccd,
0xc9f, 0xccb, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccd, 0xcb2, 0xcbe,
0xca1, 0xcbf, 0xcb5, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xccb, 0xc95, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5,
0xccd, 0xcb2, 0xcbe, 0xca1, 0xcbf, 0xcb5, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xccb, 0xc95,
0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcb5, 0xccb, 0xcb2, 0xccd, 0xc97, 0xccb, 0xc97, 0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccb, 0xcb2, 0xccd, 0xc97, 0xccb, 0xc97, 0xcbe,
0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccb, 0xcb2, 0xccd, 0xc97, 0xccb, 0xc97,
0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xcca, 0xc95, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xcc7, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcc0,
0xcb8, 0xccd, 0x20, 0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xcab, 0xccd, 0xcaf,
0xcc1, 0xc9f, 0xcc1, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcbe, 0xc95,
0xcc1, 0xc9f, 0xccd, 0xcb8, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcbe,
0xc95, 0xcc1, 0xc9f, 0xccd, 0xcb8, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcbe, 0xc95, 0xcc1,
0xc9f, 0xccd, 0xcb8, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcc7, 0xc95, 0xc9f, 0xcb0, 0xcbf, 0xca8, 0xccd,
0x200c, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcc7,
0xc95, 0xc9f, 0xcb0, 0xcbf, 0xca8, 0xccd, 0x200c, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaf, 0xcc7, 0xc95, 0xcc7, 0xc9f, 0xcb0, 0xcbf, 0xca8, 0xccd, 0x200d, 0xcac,
0xcb0, 0xccd, 0xc97, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcc1, 0xc95, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0x627, 0x620, 0x6a9, 0x631, 0x6d2, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x620, 0x6a9, 0x631, 0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9, 0x631, 0x6d2,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x641, 0x63a, 0x627, 0x646, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x627, 0x641,
0x631, 0x6cc, 0x656, 0x642, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x634, 0x631, 0x642, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x656, 0x642, 0x627,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x646, 0x648, 0x657, 0x628, 0x6cc,
0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x656,
0x642, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x628,
0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x656, 0x642, 0x627, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x656, 0x642, 0x627,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x620, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x620, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620,
0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650,
0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x644, 0x645,
0x620, 0x679, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x644,
0x645, 0x620, 0x679, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x644, 0x645, 0x620, 0x679,
0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x620, 0x645, 0x64e, 0x632, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x620, 0x645, 0x64e, 0x632, 0x64e, 0x646, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x620, 0x645, 0x64e, 0x632, 0x64e, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650,
0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x642,
0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x688, 0x6d2, 0x644, 0x627,
0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627,
0x648, 0x646, 0x679, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x627, 0x648, 0x646, 0x679, 0x6cc, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691,
0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x648, 0x646,
0x679, 0x6cc, 0x646, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6cc, 0x633, 0x650, 0x641, 0x650,
0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6cc, 0x633, 0x650, 0x641,
0x650, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6cc, 0x633, 0x650, 0x641, 0x650, 0x6a9, 0x20,
0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x620, 0x646, 0x64e, 0x691, 0x6cc, 0x656, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x646, 0x64e, 0x691, 0x6cc, 0x656, 0x631,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x620, 0x646, 0x691, 0x6cc, 0x656, 0x631, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9,
0x679, 0x627, 0x624, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9,
0x679, 0x627, 0x624, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9, 0x679, 0x627, 0x624, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x620, 0x6a9, 0x679, 0x648, 0x628, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x620, 0x6a9, 0x679, 0x648, 0x628, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9, 0x679, 0x648,
0x628, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x631, 0x620, 0x628, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x631, 0x620, 0x628, 0x650, 0x6cc, 0x64e, 0x646, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x631, 0x620, 0x628, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x688, 0x6d2,
0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x631, 0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656, 0x646, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631, 0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656,
0x646, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631, 0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656,
0x646, 0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x620,
0x646, 0x679, 0x6cc, 0x656, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x620, 0x646, 0x679,
0x6cc, 0x656, 0x646, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20,
0x627, 0x631, 0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656, 0x646, 0x627, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631,
0x645, 0x6cc, 0x656, 0x646, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x631, 0x645, 0x6cc, 0x656, 0x646, 0x650, 0x6cc, 0x627, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x631, 0x645, 0x6cc, 0x656, 0x646, 0x650, 0x6cc, 0x627, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x679,
0x644, 0x627, 0x646, 0x679, 0x650, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x620, 0x679, 0x644, 0x627, 0x646, 0x679, 0x650, 0x6a9, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x620, 0x679, 0x644, 0x627, 0x646, 0x679, 0x650, 0x6a9, 0x20, 0x688, 0x6d2, 0x644,
0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650,
0x6cc, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679,
0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x631, 0x6a9, 0x632,
0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e,
0x646, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x688, 0x6d2, 0x644, 0x627,
0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633,
0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x631, 0x6a9,
0x632, 0x6cc, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646,
0x20, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x6cc,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646,
0x20, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x6cc,
0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x650, 0x642, 0x6cc, 0x20, 0x622, 0x633,
0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20,
0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc,
0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20,
0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x622, 0x633, 0x679,
0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645,
0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc,
0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc,
0x656, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x654, 0x679, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x632, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627,
0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x632, 0x631, 0x628, 0x627, 0x626,
0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x632, 0x631, 0x628, 0x627, 0x626, 0x6cc,
0x62c, 0x627, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x620, 0x632, 0x648, 0x631, 0x655, 0x633, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x620, 0x632, 0x648, 0x631, 0x655, 0x633, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x620, 0x632, 0x648, 0x631, 0x655, 0x633, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x64e, 0x646, 0x6af, 0x644, 0x627,
0x62f, 0x6cc, 0x634, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x64e, 0x646,
0x6af, 0x644, 0x627, 0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691,
0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x64e, 0x646, 0x6af,
0x644, 0x627, 0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x648, 0x657, 0x679, 0x627, 0x646, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x648, 0x644, 0x650, 0x648, 0x650, 0x6cc,
0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631, 0x620, 0x633, 0x650,
0x644, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631,
0x620, 0x633, 0x650, 0x644, 0x650, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631, 0x620,
0x633, 0x650, 0x644, 0x650, 0x6cc, 0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631, 0x648, 0x657, 0x646, 0x64e,
0x6d2, 0x20, 0x62f, 0x64e, 0x631, 0x648, 0x657, 0x633, 0x64e, 0x644, 0x64e, 0x645,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc, 0x67e, 0x20, 0x624, 0x631,
0x691, 0x648, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc, 0x67e, 0x20,
0x624, 0x631, 0x691, 0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc, 0x67e, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x620, 0x645,
0x648, 0x631, 0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x620, 0x62a, 0x6be, 0x64e, 0x645, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x620, 0x62a, 0x6be, 0x64e, 0x645, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x686, 0x620, 0x62a, 0x6be, 0x64e, 0x645, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686, 0x650, 0x644, 0x6cc, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686, 0x650, 0x644, 0x6cc, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686,
0x650, 0x644, 0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x686, 0x64e, 0x6cc, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x686, 0x64e, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686, 0x64e, 0x6cc,
0x646, 0x627, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x631, 0x633, 0x645, 0x64e, 0x633, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x6a9, 0x648, 0x632, 0x20, 0x627,
0x64e, 0x6cc, 0x644, 0x6cc, 0x646, 0x691, 0x632, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6a9, 0x648, 0x644, 0x648, 0x645, 0x628, 0x650, 0x6cc, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x644, 0x648, 0x645, 0x628, 0x650, 0x6cc,
0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x644, 0x648, 0x645, 0x628, 0x650, 0x6cc, 0x627,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x6a9, 0x64f, 0x6a9, 0x20, 0x627, 0x64e, 0x6cc, 0x644, 0x6cc, 0x646, 0x691, 0x632,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x64f, 0x6a9, 0x20, 0x627, 0x64e,
0x6cc, 0x644, 0x6cc, 0x646, 0x691, 0x632, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691,
0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x64f, 0x6a9, 0x20,
0x627, 0x64e, 0x6cc, 0x644, 0x6cc, 0x646, 0x691, 0x632, 0x20, 0x62d, 0x627, 0x641,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x6a9, 0x6cc, 0x648, 0x657, 0x628, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x6a9, 0x6cc, 0x648, 0x657, 0x628, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691,
0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc, 0x648, 0x657,
0x628, 0x627, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x691, 0x6cc, 0x648, 0x650, 0x633, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x691, 0x645, 0x627, 0x646, 0x679, 0x20, 0x688, 0x6cc,
0x20, 0x627, 0x64f, 0x631, 0x648, 0x6cc, 0x656, 0x644, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x6cc, 0x656, 0x633, 0x679, 0x20, 0x679, 0x6cc, 0x656, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6cc, 0x656, 0x633,
0x679, 0x64e, 0x631, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x6cc, 0x656, 0x633, 0x679, 0x64e, 0x631, 0x20, 0x62c,
0x632, 0x6cc, 0x631, 0x655, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6cc, 0x656, 0x633, 0x679, 0x64e,
0x631, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x6a9, 0x648, 0x6cc,
0x691, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9,
0x632, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x6cc, 0x648, 0x657,
0x631, 0x67e, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x6cc,
0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc,
0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e,
0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x632, 0x6cc, 0x62f, 0x20, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20,
0x6cc, 0x648, 0x631, 0x67e, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a,
0x631, 0x650, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20,
0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650,
0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x650, 0x67e, 0x6cc, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641, 0x627,
0x644, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x62c, 0x632, 0x6cc, 0x631,
0x655, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x627, 0x644, 0x6a9, 0x20, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x627,
0x644, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x62c, 0x632, 0x6cc, 0x631,
0x655, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645,
0x641, 0x6cc, 0x656, 0x62c, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641,
0x6cc, 0x656, 0x62c, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641, 0x6cc, 0x656, 0x62c, 0x6cc, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641,
0x631, 0x620, 0x646, 0x686, 0x20, 0x6af, 0x6cc, 0x648, 0x657, 0x620, 0x646, 0x627,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641, 0x631, 0x6cc, 0x646, 0x686, 0x20,
0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x62a, 0x655, 0x6c1, 0x20, 0x627, 0x646,
0x679, 0x627, 0x631, 0x6a9, 0x679, 0x650, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6af, 0x620, 0x644, 0x620, 0x67e, 0x6cc, 0x6af, 0x648, 0x632, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6af, 0x620, 0x645, 0x628, 0x650, 0x6cc, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x648, 0x631, 0x62c, 0x650, 0x6cc, 0x627,
0x6c1, 0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x648, 0x631,
0x62c, 0x650, 0x6cc, 0x627, 0x6c1, 0x64f, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x648, 0x631,
0x62c, 0x650, 0x6cc, 0x627, 0x6c1, 0x64f, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6af, 0x650, 0x644, 0x628, 0x672,
0x679, 0x20, 0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64f, 0x6a9, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20, 0x648, 0x650, 0x686,
0x20, 0x645, 0x6cc, 0x656, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x634, 0x631, 0x650, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20,
0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x645, 0x634, 0x631, 0x650, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x656, 0x646,
0x20, 0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631,
0x650, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20, 0x644, 0x6cc,
0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x6af,
0x631, 0x6cc, 0x656, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20,
0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc,
0x656, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6af, 0x64f, 0x648,
0x620, 0x645, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6af, 0x64e, 0x644, 0x641,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6af, 0x64f, 0x6cc, 0x64e, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x62d, 0x64e, 0x648, 0x627, 0x6d2, 0x20, 0x627, 0x620, 0x644, 0x6cc,
0x648, 0x657, 0x679, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x62d, 0x64e, 0x648, 0x627, 0x6d2, 0x20, 0x627, 0x620, 0x644, 0x6cc, 0x648,
0x657, 0x679, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691,
0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62d, 0x64e, 0x648, 0x627,
0x6d2, 0x20, 0x627, 0x620, 0x644, 0x6cc, 0x648, 0x657, 0x679, 0x650, 0x6cc, 0x64e,
0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af, 0x20, 0x633,
0x64e, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62d, 0x648, 0x648, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62d, 0x648, 0x648, 0x691, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x62d, 0x648, 0x648, 0x691, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6c1, 0x650, 0x646, 0x62f, 0x648, 0x633, 0x62a, 0x627,
0x646, 0x6c1, 0x650, 0x646, 0x62f, 0x648, 0x633, 0x62a, 0x672, 0x646, 0x620, 0x20,
0x627, 0x648, 0x634, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x650,
0x646, 0x691, 0x648, 0x686, 0x64e, 0x6cc, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x627, 0x650, 0x646, 0x691,
0x648, 0x646, 0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x634, 0x631, 0x650, 0x642, 0x6cc, 0x20, 0x627, 0x650, 0x646, 0x691,
0x648, 0x646, 0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x627, 0x650, 0x646, 0x691,
0x648, 0x646, 0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x650, 0x6cc, 0x656, 0x631, 0x672, 0x646, 0x620, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x650, 0x6cc, 0x656, 0x631, 0x672, 0x646, 0x620, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x650, 0x6cc, 0x656, 0x631, 0x672, 0x646, 0x6cc, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x631,
0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x650, 0x631, 0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x631,
0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x633, 0x631, 0x672, 0x6cc, 0x650,
0x644, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x633, 0x631,
0x672, 0x6cc, 0x650, 0x644, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x633, 0x631, 0x672,
0x6cc, 0x650, 0x644, 0x6cc, 0x20, 0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654,
0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x627, 0x67e, 0x672, 0x646,
0x620, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x627, 0x67e, 0x672, 0x646,
0x620, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x62c, 0x627, 0x67e, 0x672, 0x646, 0x620, 0x20, 0x691, 0x6d2,
0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x6a9, 0x64e, 0x645, 0x686, 0x64e, 0x679, 0x6a9, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6a9, 0x64e, 0x645, 0x686, 0x64e, 0x679, 0x6a9, 0x627, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x6a9, 0x64e, 0x645, 0x686, 0x64e, 0x679, 0x6a9, 0x627, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x642,
0x6cc, 0x20, 0x642, 0x627, 0x632, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679,
0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x642, 0x627, 0x632,
0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648,
0x631, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648,
0x631, 0x650, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x631, 0x650, 0x6cc, 0x627,
0x20, 0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x631, 0x633, 0x64e, 0x6d2, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6a9, 0x631, 0x620, 0x633, 0x646, 0x648, 0x6d2, 0x6cc, 0x627,
0x631, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x631, 0x620,
0x633, 0x646, 0x648, 0x6d2, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9,
0x631, 0x620, 0x633, 0x646, 0x648, 0x6d2, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9,
0x631, 0x63a, 0x632, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x644, 0x64e, 0x646, 0x6a9, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x644,
0x627, 0x6cc, 0x650, 0x654, 0x646, 0x20, 0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64f,
0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x644, 0x639, 0x627, 0x691, 0x20,
0x62d, 0x648, 0x648, 0x6d2, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x644, 0x639,
0x627, 0x691, 0x20, 0x62d, 0x648, 0x648, 0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x644, 0x639, 0x627,
0x691, 0x20, 0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x654, 0x679, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6a9, 0x64e, 0x639, 0x648, 0x657, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6a9, 0x64e, 0x639, 0x648, 0x657, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x64e, 0x6a9, 0x64e, 0x639, 0x648, 0x657, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6af, 0x64e,
0x62f, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6af,
0x64e, 0x62f, 0x64e, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6af, 0x64e, 0x62f, 0x64e,
0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x64e, 0x644, 0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x656, 0x648, 0x655, 0x632,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x631, 0x642, 0x6cc, 0x648,
0x657, 0x633, 0x64e, 0x633, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627,
0x631, 0x634, 0x64e, 0x644, 0x20, 0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64f, 0x6a9,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x648, 0x631, 0x650, 0x634, 0x64e,
0x633, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x648, 0x631, 0x650, 0x634,
0x64e, 0x633, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x648, 0x631, 0x650, 0x634, 0x64e, 0x633, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x627, 0x633, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x6cc,
0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x6cc, 0x641, 0x6a9,
0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646,
0x20, 0x67e, 0x6cc, 0x633, 0x6cc, 0x641, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6a9, 0x633,
0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x6cc, 0x641, 0x6a9, 0x20, 0x688,
0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627,
0x648, 0x644, 0x646, 0x20, 0x628, 0x627, 0x679, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x627, 0x648, 0x644, 0x646, 0x20, 0x628, 0x627, 0x679, 0x631, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627,
0x648, 0x644, 0x646, 0x20, 0x628, 0x627, 0x679, 0x631, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x633, 0x6a9,
0x64e, 0x648, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x633, 0x6a9,
0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x650, 0x6cc, 0x627,
0x646, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x64e,
0x639, 0x648, 0x657, 0x631, 0x648, 0x657, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x646, 0x620, 0x67e, 0x672, 0x644, 0x620, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x646, 0x650, 0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x620, 0x691, 0x648, 0x646, 0x650,
0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x650, 0x648, 0x20,
0x6a9, 0x6cc, 0x644, 0x620, 0x691, 0x648, 0x646, 0x650, 0x6cc, 0x627, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x646, 0x650, 0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x620, 0x691, 0x648, 0x646, 0x650,
0x6cc, 0x633, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x646, 0x650, 0x648, 0x632, 0x650, 0x644, 0x6cc, 0x646, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x650, 0x648, 0x632, 0x650, 0x644, 0x6cc,
0x646, 0x691, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x646, 0x650, 0x648, 0x632, 0x650, 0x644, 0x6cc, 0x646,
0x691, 0x20, 0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x654, 0x679, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x646, 0x6cc, 0x648, 0x20, 0x641, 0x627, 0x624, 0x646, 0x688,
0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc,
0x648, 0x20, 0x641, 0x627, 0x624, 0x646, 0x688, 0x20, 0x644, 0x6cc, 0x646, 0x688,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x646, 0x6cc, 0x648, 0x20, 0x641, 0x627, 0x624, 0x646, 0x688, 0x20, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20,
0x679, 0x627, 0x626, 0x645, 0x646, 0x650, 0x6cc, 0x648, 0x657, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x646, 0x648, 0x631, 0x641, 0x639, 0x627, 0x6a9, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x646, 0x648, 0x631, 0x641, 0x639, 0x627, 0x6a9, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x646, 0x648, 0x631, 0x641, 0x639, 0x627, 0x6a9, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641, 0x631, 0x646, 0x6cc,
0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646,
0x6c1, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x631, 0x646, 0x6cc, 0x646,
0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6c1,
0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x641, 0x631, 0x646, 0x6cc, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc,
0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6c1, 0x627, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x634, 0x64f, 0x645, 0x672, 0x644,
0x6cc, 0x20, 0x645, 0x64e, 0x631, 0x650, 0x6cc, 0x627, 0x646, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x646, 0x6c4, 0x648, 0x6c4, 0x633, 0x650, 0x628, 0x654,
0x631, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x6c4, 0x648,
0x6c4, 0x633, 0x650, 0x628, 0x654, 0x631, 0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x6c4,
0x648, 0x6c4, 0x633, 0x650, 0x628, 0x654, 0x631, 0x633, 0x6a9, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6c4, 0x645,
0x633, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6c4, 0x645, 0x633,
0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x6c4, 0x645, 0x633, 0x6a9, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x627, 0x6a9, 0x650,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x627,
0x6a9, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691,
0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x627, 0x6a9, 0x650,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x64e, 0x644, 0x627, 0x648, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x67e, 0x627, 0x67e, 0x64f, 0x639, 0x627, 0x20, 0x646, 0x6cc,
0x648, 0x657, 0x20, 0x6af, 0x620, 0x646, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x67e, 0x6cc, 0x631, 0x620, 0x6af, 0x648, 0x6d2, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x67e, 0x6cc, 0x631, 0x620, 0x6af, 0x648, 0x6d2, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e,
0x6cc, 0x631, 0x620, 0x6af, 0x648, 0x6d2, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x654, 0x631, 0x648, 0x657, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x654, 0x631, 0x648, 0x657, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x67e, 0x654, 0x631, 0x648, 0x657, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6be, 0x650, 0x644, 0x650, 0x67e, 0x627,
0x6cc, 0x650, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6be, 0x650,
0x644, 0x650, 0x67e, 0x627, 0x6cc, 0x650, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6be, 0x650,
0x644, 0x650, 0x67e, 0x627, 0x6cc, 0x650, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6be, 0x648, 0x646, 0x650,
0x6a9, 0x633, 0x20, 0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64f, 0x6a9, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x64e, 0x6cc,
0x631, 0x6cc, 0x20, 0x645, 0x650, 0x642, 0x6cc, 0x648, 0x657, 0x644, 0x64e, 0x646,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e,
0x64e, 0x6cc, 0x631, 0x6cc, 0x20, 0x645, 0x650, 0x642, 0x6cc, 0x648, 0x657, 0x644,
0x64e, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x64e, 0x6cc,
0x631, 0x6cc, 0x20, 0x645, 0x650, 0x642, 0x6cc, 0x648, 0x657, 0x644, 0x64e, 0x646,
0x20, 0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x67e, 0x650, 0x679, 0x6a9, 0x6cc, 0x631, 0x655, 0x646, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x648, 0x646, 0x6cc, 0x67e, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x646, 0x6af, 0x20, 0x6cc, 0x627, 0x646,
0x6af, 0x20, 0x679, 0x627, 0x626, 0x645, 0x642, 0x650, 0x632, 0x644, 0x648, 0x691,
0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x642, 0x650, 0x632, 0x644, 0x648,
0x691, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x642, 0x650, 0x632, 0x644, 0x648, 0x691, 0x627, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x631,
0x650, 0x6cc, 0x648, 0x657, 0x646, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x631, 0x648, 0x62a, 0x6be, 0x620, 0x631, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x633, 0x64e, 0x6a9, 0x6be, 0x620, 0x644, 0x650, 0x646,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x64e, 0x6a9, 0x6be, 0x620, 0x644,
0x650, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x633, 0x64e, 0x6a9, 0x6be, 0x620, 0x644, 0x650, 0x646,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x633, 0x645, 0x620, 0x631, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633,
0x645, 0x620, 0x631, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x645, 0x620, 0x631, 0x627, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633,
0x620, 0x645, 0x648, 0x622, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x620,
0x645, 0x648, 0x622, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x620, 0x645, 0x648, 0x622, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x6cc,
0x634, 0x620, 0x644, 0x655, 0x632, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633,
0x650, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x657, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x633, 0x648, 0x644, 0x648, 0x645, 0x64e, 0x646, 0x20, 0x62c, 0x62c,
0x6cc, 0x656, 0x631, 0x64e, 0x646, 0x20, 0x6c1, 0x64f, 0x646, 0x62f, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x634, 0x64f, 0x645, 0x672, 0x644, 0x6cc, 0x20, 0x62c,
0x648, 0x631, 0x62c, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x633, 0x64f, 0x631, 0x650, 0x646, 0x627, 0x645, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x633, 0x6cc, 0x648, 0x648, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x679, 0x627, 0x6c1, 0x650, 0x679, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x679, 0x6d2, 0x20, 0x67e, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x679, 0x6d2,
0x20, 0x67e, 0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x679, 0x6d2, 0x20, 0x67e, 0x6d2, 0x20, 0x688, 0x6d2,
0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x627,
0x62c, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x679,
0x648, 0x6a9, 0x620, 0x644, 0x648, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x679,
0x639, 0x627, 0x646, 0x6af, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x679,
0x639, 0x627, 0x646, 0x6af, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x679, 0x639, 0x627, 0x646, 0x6af,
0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x679, 0x654, 0x631, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62a,
0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627,
0x626, 0x645, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x62a, 0x64f, 0x631, 0x6a9, 0x645, 0x620, 0x646, 0x650, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x679, 0x648, 0x657, 0x648, 0x64e, 0x644, 0x648, 0x657, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x657, 0x631, 0x620, 0x6af, 0x648, 0x64e, 0x6d2,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x657, 0x631, 0x620, 0x6af,
0x648, 0x64e, 0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x657, 0x631, 0x620, 0x6af, 0x648,
0x64e, 0x6d2, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x64f, 0x632, 0x628, 0x6cc, 0x6a9, 0x650, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x64f, 0x632, 0x628, 0x6cc,
0x6a9, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x64f, 0x632, 0x628,
0x6cc, 0x6a9, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x64f, 0x6a9, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x64e, 0x646,
0x648, 0x657, 0x627, 0x64e, 0x679, 0x648, 0x657, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x648, 0x64e, 0x646, 0x648, 0x657, 0x627, 0x64e, 0x679, 0x648, 0x657, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x648, 0x64e, 0x646, 0x648, 0x657, 0x627, 0x64e, 0x679, 0x648, 0x657, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648,
0x620, 0x646, 0x620, 0x632, 0x6cc, 0x648, 0x657, 0x644, 0x627, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x648, 0x644, 0x627, 0x691, 0x650, 0x648, 0x648, 0x633, 0x679,
0x648, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x644, 0x627, 0x691,
0x650, 0x648, 0x648, 0x633, 0x679, 0x648, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x644, 0x627,
0x691, 0x650, 0x648, 0x648, 0x633, 0x679, 0x648, 0x6a9, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x648, 0x644, 0x6af,
0x648, 0x6af, 0x631, 0x6cc, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648,
0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x6cc, 0x691, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x648,
0x644, 0x6af, 0x648, 0x6af, 0x631, 0x6cc, 0x691, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x6cc, 0x6a9, 0x20, 0x62c,
0x62c, 0x6cc, 0x656, 0x631, 0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x648, 0x627, 0x644, 0x650, 0x633, 0x20, 0x62a, 0x6c1, 0x655, 0x20, 0x641, 0x6cc,
0x648, 0x657, 0x679, 0x6cc, 0x648, 0x657, 0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6cc, 0x64e, 0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6cc, 0x64e, 0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x6cc, 0x64e, 0x6a9, 0x64f, 0x679, 0x64f, 0x633, 0x6a9, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x620, 0x6a9, 0x64e,
0x679, 0x654, 0x631, 0x650, 0x646, 0x628, 0x654, 0x631, 0x6af, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6cc, 0x620, 0x6a9, 0x64e, 0x679, 0x654, 0x631, 0x650, 0x646,
0x628, 0x654, 0x631, 0x6af, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x620, 0x6a9, 0x64e, 0x679, 0x631,
0x650, 0x646, 0x628, 0x654, 0x631, 0x6af, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x6a9, 0x648, 0x646, 0x20,
0x679, 0x627, 0x626, 0x645, 0x938, 0x947, 0x902, 0x91f, 0x94d, 0x930, 0x932, 0x20,
0x935, 0x916, 0x938, 0x947, 0x902, 0x91f, 0x94d, 0x930, 0x932, 0x20, 0x938, 0x94d,
0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916, 0x938,
0x947, 0x902, 0x91f, 0x94d, 0x930, 0x932, 0x20, 0x921, 0x947, 0x20, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x935, 0x916, 0x92e, 0x936, 0x930, 0x93f, 0x915, 0x940, 0x20,
0x935, 0x916, 0x92e, 0x936, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x938, 0x94d, 0x91f,
0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916, 0x92e, 0x936,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x921, 0x947, 0x20, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x935, 0x916, 0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x935,
0x916, 0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x938, 0x94d, 0x91f,
0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916, 0x92e, 0x93e,
0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x921, 0x947, 0x20, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x935, 0x916, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20,
0x935, 0x916, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x94d,
0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916, 0x92a,
0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x20, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x935, 0x916, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f,
0x915, 0x20, 0x935, 0x916, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915,
0x20, 0x938, 0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20,
0x935, 0x916, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x921,
0x947, 0x20, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x935, 0x916, 0x92e, 0x930, 0x915,
0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20, 0x935, 0x916,
0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940,
0x20, 0x938, 0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20,
0x935, 0x916, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x92a, 0x940, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x935, 0x916, 0x92e, 0x936, 0x930,
0x93f, 0x915, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20, 0x935, 0x916,
0x92e, 0x936, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940,
0x20, 0x938, 0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20,
0x935, 0x916, 0x92e, 0x936, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x92a, 0x940, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x935, 0x916, 0x92e, 0x917, 0x930,
0x940, 0x92c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20, 0x935, 0x916,
0x92e, 0x917, 0x930, 0x940, 0x92c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940,
0x20, 0x938, 0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20,
0x935, 0x916, 0x92e, 0x917, 0x930, 0x940, 0x92c, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x92a, 0x940, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x935, 0x916, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20, 0x913, 0x938, 0x924, 0x20, 0x935, 0x916,
0x410, 0x443, 0x493, 0x430, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x4a3, 0x442, 0x4af,
0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x410,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a,
0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43b, 0x44f, 0x441,
0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43b, 0x44f, 0x441,
0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d,
0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43c, 0x430, 0x437,
0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442,
0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x43e,
0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b, 0x44b,
0x49b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442,
0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x43e, 0x440, 0x442, 0x430,
0x43b, 0x44b, 0x49b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e,
0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x448, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456,
0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x448, 0x44b, 0x493,
0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b,
0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x448, 0x44b, 0x493,
0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b,
0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x442, 0x430, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x442, 0x430, 0x443, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456,
0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x442, 0x430, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20,
0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x422, 0x44b, 0x43d, 0x44b,
0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x422, 0x44b, 0x43d, 0x44b,
0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x49b, 0x20, 0x43c, 0x4b1, 0x445,
0x438, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43f,
0x438, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43f, 0x438,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x421, 0x430, 0x443, 0x434, 0x20, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f, 0x441,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x443, 0x434,
0x20, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f, 0x441, 0x44b, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x421, 0x430, 0x443, 0x434, 0x20, 0x410, 0x440, 0x430, 0x431,
0x438, 0x44f, 0x441, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430,
0x442, 0x44b, 0x441, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b,
0x441, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x43c,
0x435, 0x43d, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410,
0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x410, 0x443, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x43e, 0x440,
0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x2d,
0x431, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x43e, 0x440, 0x442,
0x430, 0x43b, 0x44b, 0x49b, 0x2d, 0x431, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x43e, 0x440, 0x442,
0x430, 0x43b, 0x44b, 0x49b, 0x2d, 0x431, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x410,
0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x448, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x448, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x431,
0x430, 0x442, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410,
0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x431, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x4d8, 0x437, 0x456, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430,
0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4d8, 0x437, 0x456, 0x440,
0x431, 0x430, 0x439, 0x436, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x4d8, 0x437, 0x456, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410,
0x437, 0x43e, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x43e, 0x440, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434,
0x430, 0x440, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x443, 0x442,
0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x43e, 0x43b,
0x438, 0x432, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411,
0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x2d, 0x414, 0x430, 0x440, 0x443,
0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412,
0x435, 0x440, 0x434, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x430,
0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x430, 0x43c,
0x43e, 0x440, 0x440, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x430,
0x442, 0x435, 0x43c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x430,
0x442, 0x435, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x430, 0x442,
0x435, 0x43c, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x427, 0x438, 0x43b, 0x438, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x49a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x49a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x49a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x420, 0x43e,
0x436, 0x434, 0x435, 0x441, 0x442, 0x432, 0x43e, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x44b, 0x43d, 0x44b, 0x4a3, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a,
0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440,
0x44b, 0x43d, 0x44b, 0x4a3, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a,
0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41a, 0x443, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440,
0x44b, 0x43d, 0x44b, 0x4a3, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a,
0x443, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d,
0x44b, 0x4a3, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x43a, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x4a3, 0x20,
0x436, 0x430, 0x440, 0x442, 0x44b, 0x43b, 0x430, 0x439, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431,
0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x414,
0x435, 0x439, 0x432, 0x438, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438,
0x43b, 0x44c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493,
0x44b, 0x441, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x441,
0x445, 0x430, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20,
0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x415, 0x443, 0x440,
0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x440, 0x442,
0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441,
0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x49a, 0x438, 0x44b, 0x440, 0x20, 0x428, 0x44b, 0x493, 0x44b, 0x441,
0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442,
0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x415, 0x443, 0x440,
0x43e, 0x43f, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x434,
0x436, 0x438, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x434,
0x436, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x434, 0x436,
0x438, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x20, 0x413, 0x432,
0x438, 0x430, 0x43d, 0x430, 0x441, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x438, 0x44f, 0x43d, 0x44b, 0x4a3, 0x20,
0x43e, 0x4a3, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x430, 0x439, 0x43c,
0x430, 0x493, 0x44b, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x410, 0x43d, 0x442,
0x430, 0x440, 0x43a, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438,
0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437,
0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437,
0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x4a3, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442,
0x44b, 0x441, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b,
0x441, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f,
0x430, 0x440, 0x441, 0x44b, 0x20, 0x448, 0x44b, 0x493, 0x430, 0x43d, 0x430, 0x493,
0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x439, 0x430, 0x43d,
0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432, 0x430,
0x439, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439, 0x20, 0x436, 0x4d9,
0x43d, 0x435, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413,
0x430, 0x432, 0x430, 0x439, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x410, 0x43b,
0x435, 0x443, 0x442, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x425, 0x43e, 0x432, 0x434, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x425,
0x43e, 0x432, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432,
0x434, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x4ae, 0x43d, 0x434, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x4ae, 0x43d, 0x434, 0x456, 0x20, 0x43c, 0x4b1, 0x445,
0x438, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4ae, 0x43d,
0x434, 0x456, 0x49b, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x418, 0x43d,
0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x418, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440,
0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418,
0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430,
0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x416, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x416, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x49a, 0x430, 0x437,
0x430, 0x49b, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x49a, 0x430, 0x437, 0x430, 0x49b,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411,
0x430, 0x442, 0x44b, 0x441, 0x20, 0x49a, 0x430, 0x437, 0x430, 0x49b, 0x441, 0x442,
0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440,
0x435, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440,
0x435, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435,
0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41a, 0x443, 0x441, 0x430, 0x438, 0x435, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x49a,
0x44b, 0x440, 0x493, 0x44b, 0x437, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41b, 0x430, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430,
0x443, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x44f, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438,
0x439, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x422,
0x44b, 0x43d, 0x44b, 0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x422, 0x44b, 0x43d, 0x44b, 0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a,
0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x422, 0x44b, 0x43d, 0x44b,
0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x4b0, 0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x440, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4b0, 0x43b, 0x430, 0x43d, 0x431, 0x430,
0x442, 0x44b, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4b0, 0x43b, 0x430,
0x43d, 0x431, 0x430, 0x442, 0x44b, 0x440, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9, 0x441, 0x43a, 0x435,
0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9, 0x441, 0x43a,
0x435, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9, 0x441, 0x43a,
0x435, 0x443, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c, 0x430, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e,
0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3,
0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x416, 0x430, 0x4a3, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x430,
0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443,
0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x438, 0x443, 0x44d, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b,
0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438,
0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434,
0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x43e,
0x432, 0x43e, 0x441, 0x456, 0x431, 0x456, 0x440, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x456, 0x431, 0x456, 0x440, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x456, 0x431,
0x456, 0x440, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41e, 0x43c, 0x431, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41e, 0x43c, 0x431, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41e, 0x43c, 0x431, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x4d9, 0x43a, 0x456, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x4d9, 0x43a, 0x456,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x4d9,
0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43b, 0x430, 0x443,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43f, 0x443, 0x430,
0x20, 0x2013, 0x20, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d,
0x435, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f,
0x435, 0x440, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x435,
0x440, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x435, 0x440, 0x443,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440,
0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d,
0x41f, 0x44c, 0x435, 0x440, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x41c, 0x438,
0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421,
0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x436, 0x4d9, 0x43d, 0x435,
0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x438, 0x442,
0x43a, 0x44d, 0x440, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f,
0x43e, 0x43d, 0x43f, 0x435, 0x438, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41f, 0x445, 0x435, 0x43d, 0x44c, 0x44f, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x43e, 0x43d, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x439,
0x448, 0x435, 0x43b, 0x44c, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x438, 0x43d, 0x433,
0x430, 0x43f, 0x443, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e,
0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x4a3, 0x442,
0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x438,
0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x443, 0x440, 0x438,
0x43d, 0x430, 0x43c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x451,
0x432, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x438,
0x442, 0x438, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x439,
0x431, 0x44d, 0x439, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x430,
0x439, 0x431, 0x44d, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x430,
0x439, 0x431, 0x44d, 0x439, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x4d9, 0x436, 0x456, 0x43a, 0x441, 0x442,
0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x43e, 0x43a,
0x435, 0x43b, 0x430, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x43e,
0x43d, 0x433, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x440, 0x443, 0x43a, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x422, 0x4af, 0x440, 0x456, 0x43a, 0x43c, 0x435, 0x43d, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x4af,
0x440, 0x456, 0x43a, 0x43c, 0x435, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x4af, 0x440, 0x456, 0x43a, 0x43c, 0x435, 0x43d,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430,
0x439, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443, 0x433,
0x432, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x439, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4e8, 0x437, 0x431, 0x435, 0x43a, 0x441, 0x442,
0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4e8, 0x437, 0x431,
0x435, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x4e8, 0x437, 0x431, 0x435, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x435, 0x43d,
0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x423, 0x44d, 0x439, 0x43a, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x423,
0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x424,
0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x42e, 0x43a,
0x43e, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x627, 0x6cb, 0x639,
0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x627, 0x641, 0x631,
0x64a, 0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x634, 0x649,
0x639, 0x649, 0x633, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x6ad, 0x62a, 0x6c7, 0x633, 0x62a, 0x649,
0x643, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x627, 0x641, 0x631, 0x64a,
0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a,
0x649, 0x633, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a,
0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x627, 0x641, 0x631,
0x64a, 0x643, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x627, 0x644, 0x627, 0x633, 0x643, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x644, 0x627, 0x633, 0x643, 0x627, 0x20,
0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x644, 0x627, 0x633, 0x643, 0x627, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x627, 0x644, 0x645, 0x627, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x649, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x645, 0x627, 0x632,
0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x645, 0x627,
0x632, 0x648, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x645, 0x627,
0x632, 0x648, 0x646, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649,
0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x648, 0x631,
0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645,
0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x644, 0x62a, 0x6c7, 0x633,
0x62a, 0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x648, 0x644, 0x62a,
0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643,
0x627, 0x20, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20,
0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x634, 0x649, 0x639, 0x649, 0x633,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x648, 0x644, 0x62a, 0x6c7,
0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x634, 0x649, 0x639, 0x649, 0x633,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x648, 0x644, 0x62a, 0x6c7,
0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627,
0x20, 0x62a, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633,
0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645, 0x6d5,
0x631, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x62a, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20,
0x627, 0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639,
0x649, 0x20, 0x62a, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627, 0x645,
0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x62a, 0x649, 0x646, 0x649, 0x642, 0x20,
0x645, 0x6c7, 0x62d, 0x64a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627,
0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x62a, 0x649, 0x646, 0x649, 0x642, 0x20,
0x645, 0x6c7, 0x62d, 0x64a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x633, 0x648, 0x644, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x627,
0x645, 0x6d5, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x62a, 0x649, 0x646, 0x649, 0x642, 0x20, 0x645, 0x6c7, 0x62d, 0x64a, 0x62a,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x67e, 0x64a, 0x627,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x67e, 0x64a, 0x627, 0x20,
0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x67e, 0x64a, 0x627, 0x20, 0x62c, 0x627,
0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x642,
0x62a, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x642,
0x62a, 0x627, 0x6cb, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x642, 0x62a,
0x627, 0x6cb, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x6d5, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x6d5, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x6d5, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633,
0x627, 0x6cb, 0x62f, 0x20, 0x627, 0x631, 0x627, 0x628, 0x64a, 0x627, 0x633, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x6cb, 0x62f, 0x20,
0x627, 0x631, 0x627, 0x628, 0x64a, 0x627, 0x633, 0x649, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x633, 0x627, 0x6cb, 0x62f, 0x20, 0x627, 0x631, 0x627, 0x628, 0x64a,
0x627, 0x633, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x627, 0x631, 0x6af, 0x6d5, 0x646, 0x62a, 0x64a, 0x646,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x631, 0x6af, 0x6d5,
0x646, 0x62a, 0x64a, 0x646, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627,
0x631, 0x6af, 0x6d5, 0x646, 0x62a, 0x64a, 0x646, 0x627, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a,
0x649, 0x633, 0x20, 0x627, 0x631, 0x6af, 0x6d5, 0x646, 0x62a, 0x64a, 0x646, 0x627,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633,
0x20, 0x627, 0x631, 0x6af, 0x6d5, 0x646, 0x62a, 0x64a, 0x646, 0x627, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x627, 0x631,
0x6af, 0x6d5, 0x646, 0x62a, 0x64a, 0x646, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x631, 0x645, 0x6d5,
0x646, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x631,
0x645, 0x6d5, 0x646, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627,
0x631, 0x645, 0x6d5, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x62a, 0x644, 0x627, 0x646,
0x62a, 0x64a, 0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627,
0x62a, 0x644, 0x627, 0x646, 0x62a, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x64a, 0x643, 0x627, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x627, 0x6cb, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x633, 0x62a,
0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x648, 0x631, 0x62a,
0x627, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627,
0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644,
0x64a, 0x627, 0x20, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x2d, 0x628,
0x627, 0x62a, 0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627,
0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x648, 0x631, 0x62a, 0x627,
0x644, 0x649, 0x642, 0x2d, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a,
0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x648, 0x631, 0x62a, 0x627,
0x644, 0x649, 0x642, 0x2d, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x627, 0x6cb,
0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20,
0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x634,
0x649, 0x639, 0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627,
0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x627, 0x6cb, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x628, 0x627,
0x62a, 0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x6cb,
0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639,
0x649, 0x20, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x621, 0x627, 0x632, 0x649, 0x631, 0x628, 0x627, 0x64a, 0x62c, 0x627,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x621, 0x627, 0x632, 0x649,
0x631, 0x628, 0x627, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x621, 0x627, 0x632, 0x649, 0x631, 0x628, 0x627, 0x64a, 0x62c, 0x627, 0x646,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x627, 0x632, 0x648, 0x631, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627,
0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x627, 0x632, 0x648,
0x631, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x627, 0x632, 0x648, 0x631, 0x20, 0x627, 0x631, 0x627,
0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x646, 0x6af, 0x644, 0x627,
0x62f, 0x6d5, 0x634, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627,
0x646, 0x6af, 0x644, 0x627, 0x62f, 0x6d5, 0x634, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x628, 0x627, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x6d5, 0x634, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628,
0x6cb, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628,
0x648, 0x644, 0x64a, 0x6c6, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a, 0x627,
0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644,
0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x628, 0x631, 0x6cb, 0x646, 0x6d5, 0x64a, 0x2d, 0x62f, 0x627,
0x631, 0x6cb, 0x633, 0x633, 0x627, 0x644, 0x627, 0x645, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x643, 0x627, 0x628, 0x648, 0x2d, 0x6c6, 0x6d5, 0x631, 0x62f,
0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x627, 0x628, 0x648,
0x2d, 0x6c6, 0x6d5, 0x631, 0x62f, 0x6d5, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x643, 0x627, 0x628, 0x648, 0x2d, 0x6c6, 0x6d5, 0x631, 0x62f, 0x6d5, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x686,
0x627, 0x645, 0x648, 0x631, 0x631, 0x648, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x686, 0x627, 0x62a, 0x6d5, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x686, 0x627, 0x62a, 0x6d5, 0x645, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x686,
0x627, 0x62a, 0x6d5, 0x645, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x686, 0x64a, 0x644, 0x64a, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x686, 0x64a, 0x644, 0x64a, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x686, 0x64a, 0x644, 0x64a, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x649, 0x62a, 0x627, 0x64a,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x649, 0x62a, 0x627, 0x64a,
0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x649, 0x62a, 0x627, 0x64a, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x643, 0x631, 0x64a, 0x633, 0x62a, 0x645, 0x627, 0x633, 0x20, 0x627, 0x631, 0x627,
0x644, 0x649, 0x646, 0x649, 0x6ad, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x643, 0x648, 0x643, 0x648, 0x633, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627,
0x631, 0x649, 0x646, 0x649, 0x6ad, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x643, 0x648, 0x644, 0x6cb, 0x645, 0x628, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x643, 0x648, 0x644, 0x6cb, 0x645, 0x628, 0x64a, 0x627, 0x20,
0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x648, 0x644, 0x6cb, 0x645, 0x628, 0x64a,
0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x643, 0x6cb, 0x643, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627,
0x631, 0x649, 0x646, 0x649, 0x6ad, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x643, 0x6cb, 0x643, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649,
0x646, 0x649, 0x6ad, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x6cb, 0x643,
0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x646, 0x649, 0x6ad,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x643, 0x6cb, 0x628, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x643, 0x6cb, 0x628, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x6cb,
0x628, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x62f, 0x6d5, 0x64a, 0x6c6, 0x64a, 0x633, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x62f, 0x64a, 0x6cb, 0x645, 0x648, 0x646, 0x2d, 0x62f,
0x64a, 0x6cb, 0x631, 0x6c6, 0x64a, 0x644, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x62a, 0x64a, 0x645, 0x648, 0x631,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627, 0x633, 0x62d, 0x627,
0x20, 0x627, 0x631, 0x627, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x67e, 0x627, 0x633, 0x62d, 0x627, 0x20, 0x627, 0x631, 0x627, 0x644, 0x649,
0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627, 0x633, 0x62d, 0x627, 0x20,
0x627, 0x631, 0x627, 0x644, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6d5, 0x643, 0x6c6, 0x627, 0x62f, 0x648,
0x631, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x631, 0x62a, 0x627,
0x644, 0x649, 0x642, 0x20, 0x6d5, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642,
0x20, 0x6d5, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642, 0x20, 0x6d5, 0x6cb, 0x631,
0x648, 0x67e, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x6d5, 0x6cb,
0x631, 0x648, 0x67e, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x634,
0x649, 0x639, 0x649, 0x633, 0x20, 0x6d5, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20,
0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x6d5,
0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x64a, 0x649, 0x631, 0x20, 0x634,
0x649, 0x639, 0x649, 0x633, 0x20, 0x6d5, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20,
0x6d5, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x6d5, 0x6cb, 0x631, 0x648, 0x67e,
0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633,
0x20, 0x6d5, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x648, 0x644, 0x643,
0x644, 0x6d5, 0x646, 0x62f, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x648, 0x644, 0x643,
0x644, 0x6d5, 0x646, 0x62f, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631,
0x649, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x648, 0x644, 0x643, 0x644,
0x6d5, 0x646, 0x62f, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x641, 0x64a, 0x62c, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x641, 0x64a, 0x62c, 0x64a, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x64a,
0x62c, 0x64a, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x641, 0x631, 0x627, 0x646, 0x62a, 0x633, 0x6cb, 0x632, 0x20,
0x6af, 0x6c6, 0x64a, 0x627, 0x646, 0x627, 0x633, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x641, 0x631, 0x627, 0x646, 0x62a, 0x633, 0x64a, 0x627, 0x646,
0x649, 0x6ad, 0x20, 0x648, 0x6ad, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20,
0x627, 0x64a, 0x645, 0x627, 0x639, 0x649, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20,
0x627, 0x646, 0x62a, 0x627, 0x631, 0x643, 0x62a, 0x64a, 0x643, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af,
0x648, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x627, 0x645,
0x628, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x631, 0x6cb,
0x632, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x631,
0x6cb, 0x632, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x631,
0x6cb, 0x632, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x64a, 0x644, 0x628, 0x6d5, 0x631, 0x62a,
0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x646, 0x649, 0x6ad,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x631, 0x64a, 0x646, 0x6c6,
0x64a, 0x686, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639,
0x649, 0x633, 0x20, 0x6af, 0x631, 0x6d5, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x64a,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649,
0x633, 0x20, 0x6af, 0x631, 0x6d5, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x64a, 0x627,
0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20,
0x6af, 0x631, 0x6d5, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x64a, 0x627, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628,
0x627, 0x62a, 0x649, 0x633, 0x20, 0x6af, 0x631, 0x6d5, 0x646, 0x644, 0x627, 0x646,
0x62f, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627,
0x62a, 0x649, 0x633, 0x20, 0x6af, 0x631, 0x6d5, 0x646, 0x644, 0x627, 0x646, 0x62f,
0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649,
0x633, 0x20, 0x6af, 0x631, 0x6d5, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x64a, 0x627,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x67e, 0x627, 0x631, 0x633, 0x649, 0x20, 0x634, 0x649, 0x639, 0x627, 0x646,
0x627, 0x639, 0x649, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x627, 0x64a,
0x627, 0x646, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x627,
0x6c6, 0x627, 0x64a, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20, 0x627, 0x644, 0x6d5,
0x6cb, 0x62a, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6af, 0x627, 0x6c6, 0x627, 0x64a, 0x20,
0x62c, 0x627, 0x646, 0x6d5, 0x20, 0x627, 0x644, 0x6d5, 0x6cb, 0x62a, 0x20, 0x627,
0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x6af, 0x627, 0x6c6, 0x627, 0x64a, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20,
0x627, 0x644, 0x6d5, 0x6cb, 0x62a, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627,
0x631, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x62d, 0x648, 0x6ad, 0x643, 0x648, 0x6ad, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x62d, 0x648, 0x6ad, 0x643, 0x648, 0x6ad, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x62d, 0x648, 0x6ad, 0x643, 0x648, 0x6ad, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62d,
0x648, 0x6c6, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62d, 0x648,
0x6c6, 0x62f, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62d, 0x648, 0x6c6, 0x62f,
0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x621, 0x6c7, 0x646, 0x62f, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x621, 0x6c7, 0x646, 0x62f, 0x649, 0x20, 0x645, 0x6c7,
0x62d, 0x64a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x621,
0x6c7, 0x646, 0x62f, 0x649, 0x2d, 0x642, 0x649, 0x62a, 0x627, 0x64a, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x631, 0x62a, 0x627, 0x644, 0x649, 0x642,
0x20, 0x64a, 0x646, 0x62f, 0x648, 0x646, 0x6d5, 0x632, 0x64a, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x64a,
0x646, 0x62f, 0x648, 0x646, 0x6d5, 0x632, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x628, 0x627, 0x62a, 0x649, 0x633, 0x20, 0x64a, 0x646, 0x62f,
0x648, 0x646, 0x6d5, 0x632, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x64a, 0x631, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x64a, 0x631, 0x627, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x64a, 0x631,
0x627, 0x646, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x64a, 0x631, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x64a, 0x631, 0x643, 0x6cb, 0x62a, 0x633, 0x643,
0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x64a, 0x631, 0x643, 0x6cb, 0x62a, 0x633,
0x643, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x64a, 0x632, 0x631, 0x627, 0x64a, 0x644, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x64a, 0x632, 0x631, 0x627, 0x64a, 0x644, 0x20, 0x633, 0x62a,
0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x64a, 0x632, 0x631, 0x627, 0x64a, 0x644, 0x20, 0x62c, 0x627,
0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627,
0x67e, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x62c, 0x627, 0x67e, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x62c, 0x627, 0x67e, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x627, 0x632,
0x627, 0x642, 0x20, 0x6d5, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x634, 0x649, 0x639, 0x649, 0x633, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642,
0x20, 0x6d5, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x628,
0x627, 0x62a, 0x649, 0x633, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642, 0x20, 0x6d5,
0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x648, 0x631,
0x6d5, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x648,
0x631, 0x6d5, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x648,
0x631, 0x6d5, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x6cb, 0x633, 0x627, 0x64a, 0x6d5, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x631, 0x627, 0x633, 0x646, 0x648,
0x64a, 0x627, 0x631, 0x633, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x20,
0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a,
0x627, 0x631, 0x633, 0x643, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x649, 0x631, 0x639, 0x649, 0x632, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x644, 0x627,
0x64a, 0x646, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x2d, 0x62d,
0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x644, 0x648, 0x631,
0x62f, 0x2d, 0x62d, 0x627, 0x6cb, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x644,
0x648, 0x631, 0x62f, 0x2d, 0x62d, 0x627, 0x6cb, 0x20, 0x62c, 0x627, 0x632, 0x639,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x6af, 0x627,
0x62f, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627,
0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645,
0x627, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x644, 0x627, 0x64a,
0x632, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627,
0x644, 0x62f, 0x64a, 0x6c6, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x631, 0x643,
0x64a, 0x632, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x631, 0x634, 0x627, 0x644,
0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x6c6, 0x631, 0x64a, 0x643, 0x64a, 0x64a,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x6c6, 0x631, 0x64a,
0x643, 0x64a, 0x64a, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x627, 0x6c6,
0x631, 0x64a, 0x643, 0x64a, 0x64a, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x648, 0x6cb, 0x633, 0x648, 0x646,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x6d5, 0x643, 0x633, 0x64a,
0x643, 0x627, 0x20, 0x62a, 0x649, 0x646, 0x649, 0x642, 0x20, 0x645, 0x6c7, 0x62d,
0x64a, 0x62a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x6d5, 0x643,
0x633, 0x64a, 0x643, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x62a, 0x649, 0x646, 0x649, 0x642, 0x20, 0x645, 0x6c7,
0x62d, 0x64a, 0x62a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x6d5,
0x643, 0x633, 0x64a, 0x643, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x62a, 0x649, 0x646, 0x649, 0x642, 0x20, 0x645, 0x6c7, 0x62d, 0x64a, 0x62a, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c7, 0x644, 0x627, 0x646, 0x628, 0x627,
0x62a, 0x649, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c7, 0x644,
0x627, 0x646, 0x628, 0x627, 0x62a, 0x649, 0x631, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x6c7, 0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x649, 0x631, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645,
0x627, 0x633, 0x643, 0x6d5, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x645, 0x627, 0x633, 0x643, 0x6d5, 0x6cb, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x645, 0x627, 0x633, 0x643, 0x6d5, 0x6cb, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x645, 0x64a, 0x627, 0x646, 0x645,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x627, 0x6cb, 0x631,
0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x6d5, 0x67e, 0x627,
0x644, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627,
0x20, 0x643, 0x627, 0x644, 0x6d5, 0x62f, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x20, 0x643, 0x627,
0x644, 0x6d5, 0x62f, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x20, 0x643, 0x627, 0x644, 0x6d5, 0x62f, 0x648,
0x646, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x20, 0x632, 0x6d5, 0x644,
0x627, 0x646, 0x62f, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x62c, 0x627, 0x6ad, 0x627, 0x20, 0x632, 0x6d5, 0x644, 0x627, 0x646, 0x62f, 0x64a,
0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x20,
0x632, 0x6d5, 0x644, 0x627, 0x646, 0x62f, 0x64a, 0x627, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x64a, 0x6cb,
0x641, 0x627, 0x6cb, 0x646, 0x62f, 0x644, 0x6d5, 0x646, 0x62f, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x646, 0x64a, 0x6cb, 0x641, 0x627, 0x6cb, 0x646, 0x62f,
0x644, 0x6d5, 0x646, 0x62f, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631,
0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x64a,
0x6cb, 0x641, 0x627, 0x6cb, 0x646, 0x62f, 0x644, 0x6d5, 0x646, 0x62f, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646,
0x64a, 0x6cb, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x648,
0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x627, 0x631, 0x627, 0x644, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644,
0x643, 0x20, 0x627, 0x631, 0x627, 0x644, 0x649, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x627, 0x631, 0x627,
0x644, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x641, 0x6d5, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x6cb, 0x2d,
0x62f, 0x64a, 0x2d, 0x646, 0x648, 0x631, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x6d5, 0x631, 0x646, 0x627, 0x646, 0x62f,
0x6cb, 0x2d, 0x62f, 0x64a, 0x2d, 0x646, 0x648, 0x631, 0x648, 0x646, 0x64a, 0x627,
0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x6d5, 0x631, 0x646, 0x627, 0x646,
0x62f, 0x6cb, 0x2d, 0x62f, 0x64a, 0x2d, 0x646, 0x648, 0x631, 0x648, 0x646, 0x64a,
0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x633, 0x649, 0x628, 0x649, 0x631, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627, 0x6ad, 0x627, 0x633, 0x649,
0x628, 0x649, 0x631, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62c, 0x627, 0x6ad,
0x627, 0x633, 0x649, 0x628, 0x649, 0x631, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x645, 0x628, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x645, 0x628, 0x649, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x648, 0x645, 0x628, 0x649, 0x20, 0x62c, 0x627, 0x632,
0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627, 0x643,
0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x67e, 0x627, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x67e, 0x627, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e,
0x627, 0x644, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e,
0x627, 0x67e, 0x6cb, 0x627, 0x20, 0x2013, 0x20, 0x62c, 0x627, 0x6ad, 0x627, 0x20,
0x6af, 0x6c6, 0x64a, 0x646, 0x6d5, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6c6, 0x627, 0x64a, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6c6, 0x627,
0x64a, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af,
0x6c6, 0x627, 0x64a, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x67e, 0x6d5, 0x631, 0x6cb, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x67e, 0x6d5, 0x631, 0x6cb, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x67e, 0x6d5, 0x631, 0x6cb, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x64a, 0x644, 0x64a, 0x67e, 0x64a,
0x646, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x64a, 0x644, 0x64a, 0x67e, 0x64a, 0x646,
0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x633, 0x62a,
0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x641, 0x64a, 0x644, 0x64a, 0x67e, 0x64a, 0x646, 0x20, 0x627,
0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20, 0x62c, 0x627, 0x632, 0x639,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x641, 0x6d5, 0x646, 0x64a,
0x643, 0x633, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x6d5, 0x646, 0x2d, 0x67e, 0x6d5,
0x631, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20, 0x645, 0x64a, 0x643, 0x6d5, 0x644,
0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x6d5, 0x646,
0x2d, 0x67e, 0x6d5, 0x631, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20, 0x645, 0x64a,
0x643, 0x6d5, 0x644, 0x648, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633,
0x6d5, 0x646, 0x2d, 0x67e, 0x6d5, 0x631, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20,
0x645, 0x64a, 0x643, 0x6d5, 0x644, 0x648, 0x646, 0x20, 0x62c, 0x627, 0x632, 0x639,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x64a, 0x62a, 0x643,
0x6d5, 0x631, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e, 0x648,
0x646, 0x67e, 0x6d5, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x67e,
0x62d, 0x6d5, 0x646, 0x64a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x642, 0x649, 0x632, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x649, 0x632, 0x649, 0x644, 0x648, 0x631,
0x62f, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x642, 0x649, 0x632, 0x649,
0x644, 0x648, 0x631, 0x62f, 0x627, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x631, 0x6d5, 0x64a, 0x6cb, 0x646, 0x648,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x631, 0x648, 0x62a, 0x6d5,
0x631, 0x627, 0x20, 0x6cb, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x62d, 0x627,
0x644, 0x64a, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627,
0x62d, 0x627, 0x644, 0x64a, 0x646, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633,
0x627, 0x62d, 0x627, 0x644, 0x64a, 0x646, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x648, 0x627,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x648, 0x627,
0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x648, 0x627, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x633, 0x6d5, 0x64a, 0x634, 0x6d5, 0x644, 0x20, 0x627, 0x631, 0x627, 0x644, 0x62f,
0x627, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x64a,
0x646, 0x6af, 0x627, 0x67e, 0x6cb, 0x631, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x633, 0x648, 0x644, 0x648, 0x645, 0x648, 0x646, 0x20, 0x627, 0x631, 0x627, 0x644,
0x62f, 0x627, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648,
0x6ad, 0x62a, 0x6c7, 0x633, 0x62a, 0x649, 0x643, 0x20, 0x6af, 0x6d5, 0x648, 0x631,
0x6af, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x633, 0x6cb,
0x631, 0x64a, 0x646, 0x627, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x633, 0x6d5, 0x6c6, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a,
0x627, 0x64a, 0x62a, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a,
0x627, 0x64a, 0x628, 0x6d5, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x62a, 0x627, 0x64a, 0x628, 0x6d5, 0x64a, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x62a, 0x627, 0x64a, 0x628, 0x6d5, 0x64a, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x627, 0x62c, 0x649, 0x643,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a,
0x648, 0x643, 0x6d5, 0x644, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f,
0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x62a, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x643, 0x627, 0x632, 0x639, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x631, 0x6cb, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x6c7, 0x631, 0x649, 0x643, 0x645, 0x6d5,
0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x62a, 0x6c7, 0x631, 0x649, 0x643, 0x645, 0x6d5, 0x646, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x6c7, 0x631, 0x649, 0x643, 0x645,
0x6d5, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x62a, 0x6cb, 0x6c6, 0x627, 0x644,
0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6cb, 0x631, 0x6cb, 0x6af,
0x6c6, 0x627, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6cb, 0x631,
0x6cb, 0x6af, 0x6c6, 0x627, 0x64a, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627,
0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6cb,
0x631, 0x6cb, 0x6af, 0x6c6, 0x627, 0x64a, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648, 0x632, 0x628, 0x6d5, 0x643,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x648,
0x632, 0x628, 0x6d5, 0x643, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x648, 0x632, 0x628, 0x6d5, 0x643, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x6c6, 0x627, 0x646, 0x6cb, 0x627, 0x62a, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x6c6, 0x627, 0x646, 0x6cb, 0x627, 0x62a, 0x6cb, 0x20, 0x633, 0x62a,
0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x6c6, 0x627, 0x646, 0x6cb, 0x627, 0x62a, 0x6cb, 0x20, 0x62c,
0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c6,
0x6d5, 0x646, 0x6d5, 0x633, 0x6cb, 0x6d5, 0x644, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x6c6, 0x644, 0x627, 0x62f, 0x64a, 0x6c6, 0x648, 0x633, 0x62a,
0x648, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c6, 0x644, 0x627,
0x62f, 0x64a, 0x6c6, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x20, 0x633, 0x62a, 0x627,
0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x6c6, 0x644, 0x627, 0x62f, 0x64a, 0x6c6, 0x648, 0x633, 0x62a, 0x648,
0x643, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x6c6, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20,
0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c6, 0x648, 0x644, 0x6af, 0x648, 0x6af,
0x631, 0x627, 0x62f, 0x20, 0x633, 0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a,
0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6c6, 0x648, 0x644,
0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6cb, 0x6d5, 0x64a, 0x643, 0x20,
0x627, 0x631, 0x627, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x6cb, 0x648, 0x644, 0x644, 0x64a, 0x633, 0x20, 0x62c, 0x627, 0x646, 0x6d5, 0x20,
0x641, 0x6cb, 0x62a, 0x6cb, 0x646, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x64a, 0x627, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x649, 0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x20, 0x633,
0x62a, 0x627, 0x646, 0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x649, 0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6cb, 0x62a, 0x633, 0x643, 0x20,
0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649,
0x6d5, 0x643, 0x627, 0x62a, 0x6d5, 0x631, 0x64a, 0x646, 0x628, 0x6cb, 0x631, 0x6af,
0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a, 0x649, 0x6d5, 0x643, 0x627, 0x62a, 0x6d5,
0x631, 0x64a, 0x646, 0x628, 0x6cb, 0x631, 0x6af, 0x20, 0x633, 0x62a, 0x627, 0x646,
0x62f, 0x627, 0x631, 0x62a, 0x62a, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649, 0x62a,
0x649, 0x6d5, 0x643, 0x627, 0x62a, 0x6d5, 0x631, 0x64a, 0x646, 0x628, 0x6cb, 0x631,
0x6af, 0x20, 0x62c, 0x627, 0x632, 0x639, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x64a, 0x6cb, 0x643, 0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x649,
0x62a, 0x649, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x17a0, 0x17d2, 0x179c, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1793, 0x17b8, 0x179f, 0x17d2, 0x1790,
0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a,
0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17b6, 0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784,
0x200b, 0x178f, 0x17d2, 0x1794, 0x17bc, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b,
0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x200b, 0x200b,
0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x200b, 0x200b, 0x1793, 0x17b6, 0x179a, 0x178a,
0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a1, 0x17b6, 0x179f, 0x17d2, 0x1780, 0x17b6,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a1, 0x17b6, 0x179f, 0x17d2, 0x1780,
0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2,
0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x17a2, 0x17b6, 0x17a1, 0x17b6, 0x179f,
0x17d2, 0x1780, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17b6, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17bc, 0x1793, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17bc, 0x1793, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c9, 0x17b6,
0x17a0, 0x17d2, 0x179f, 0x17bc, 0x1793, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780,
0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1780,
0x178e, 0x17d2, 0x178f, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x200b, 0x1791, 0x17d2,
0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1780, 0x178e, 0x17d2,
0x178f, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1796, 0x17c1, 0x179b,
0x1790, 0x17d2, 0x1784, 0x17c3, 0x1793, 0x17c5, 0x200b, 0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794,
0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1780, 0x178e, 0x17d2, 0x178f, 0x17b6, 0x179b,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x17a2,
0x17b6, 0x1798, 0x179a, 0x17b7, 0x1780, 0x1781, 0x17b6, 0x1784, 0x1787, 0x17be, 0x1784, 0x1797,
0x17b6, 0x1782, 0x1781, 0x17b6, 0x1784, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x1791, 0x17d2,
0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x1781, 0x17b6,
0x1784, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1781, 0x17b6, 0x1784, 0x1780, 0x17be,
0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1796, 0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3,
0x1793, 0x17c5, 0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a,
0x17b7, 0x1780, 0x1781, 0x17b6, 0x1784, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1781,
0x17b6, 0x1784, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x178f, 0x17c6, 0x1794, 0x1793, 0x17cb, 0x1797, 0x17d2, 0x1793, 0x17c6, 0x1793, 0x17c3, 0x1791,
0x17d2, 0x179c, 0x17b8, 0x1794, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780,
0x200b, 0x200b, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x178f, 0x17c6, 0x1794, 0x1793, 0x17cb, 0x1797, 0x17d2, 0x1793, 0x17c6, 0x1793, 0x17c3,
0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7,
0x1780, 0x200b, 0x200b, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x1793,
0x17c5, 0x178f, 0x17c6, 0x1794, 0x1793, 0x17cb, 0x1797, 0x17d2, 0x1793, 0x17c6, 0x1793, 0x17c3,
0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7,
0x1780, 0x200b, 0x200b, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x17a0,
0x17d2, 0x179c, 0x17b7, 0x1780, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2, 0x178a, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793,
0x17c5, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780,
0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1796,
0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x1793, 0x17c5, 0x1794, 0x17c9, 0x17b6, 0x179f,
0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a,
0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x1794, 0x17d2, 0x1799, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1794, 0x17d2,
0x1799, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790,
0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1794, 0x17d2, 0x1799,
0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179a,
0x17c9, 0x17b6, 0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179a,
0x17c9, 0x17b6, 0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b,
0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179a,
0x17c9, 0x17b6, 0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2,
0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c,
0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1797, 0x17b6,
0x1782, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17b6, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1797, 0x17b6, 0x1782,
0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793,
0x1797, 0x17b6, 0x1782, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1793, 0x17b6, 0x179a,
0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x1793, 0x17b8, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178a, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x1793, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x1793, 0x17b8, 0x1793, 0x17b6,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x178f, 0x17d2, 0x179b, 0x1784,
0x17cb, 0x1791, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x178f,
0x17d2, 0x179b, 0x1784, 0x17cb, 0x1791, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x178f, 0x17d2, 0x179b, 0x1784, 0x17cb, 0x1791, 0x17b7, 0x1780, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a,
0x17b6, 0x179b, 0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b,
0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1796, 0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x200b, 0x200b, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b,
0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x200b, 0x1797, 0x17b6, 0x1782, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x179f,
0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a,
0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1797, 0x17b6, 0x1782, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x200b, 0x1793, 0x17c3, 0x200b, 0x17a2, 0x17bc,
0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2,
0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b,
0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x1797, 0x17b6, 0x1782,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x200b, 0x1793, 0x17c3, 0x200b,
0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1780,
0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b,
0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796,
0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b,
0x17b8, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b,
0x17b8, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179f, 0x17ca, 0x17c2, 0x1794, 0x17c3, 0x17a0,
0x17d2, 0x179f, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179f,
0x17ca, 0x17c2, 0x1794, 0x17c3, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179f, 0x17ca, 0x17c2, 0x1794,
0x17c3, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c,
0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17c1, 0x17a0, 0x17d2, 0x179f, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17c1, 0x17a0, 0x17d2, 0x179f, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a2, 0x17c1, 0x17a0, 0x17d2, 0x179f, 0x179f, 0x1793, 0x17b6, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x1784, 0x17cb, 0x1780, 0x17d2, 0x179b, 0x17b6, 0x178a,
0x17c2, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178a, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x1784, 0x17cb, 0x1780, 0x17d2,
0x179b, 0x17b6, 0x178a, 0x17c2, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1794, 0x1784, 0x17cb, 0x1780, 0x17d2, 0x179b, 0x17b6, 0x178a, 0x17c2, 0x179f,
0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1794, 0x17ca, 0x17bc, 0x178f, 0x1784, 0x17cb, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17bc, 0x179b, 0x17b8, 0x179c,
0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17d2, 0x179a,
0x17b6, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1799,
0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17d2,
0x179a, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x1793, 0x17b6,
0x200b, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1796, 0x17d2, 0x179a, 0x17bb, 0x1799, 0x178e, 0x17c1,
0x178a, 0x17b6, 0x179a, 0x17bc, 0x179f, 0x17b6, 0x17a1, 0x17b9, 0x1798, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x1794, 0x17cb, 0x179c, 0x17c2, 0x179a,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x1794, 0x17cb, 0x179c, 0x17c2, 0x179a,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x1794,
0x17cb, 0x179c, 0x17c2, 0x179a, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780,
0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b6, 0x17c6, 0x1798, 0x17c9,
0x17bc, 0x179a, 0x17c9, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1785, 0x17b6, 0x1790, 0x17b6, 0x17c6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b6,
0x1790, 0x17b6, 0x17c6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b,
0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b6, 0x1790, 0x17b6,
0x17c6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x1793, 0x17c5, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793,
0x17c5, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c,
0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1785, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b7, 0x1793, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x179f,
0x17d2, 0x1798, 0x17b6, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1780, 0x17bc,
0x1780, 0x17bc, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780,
0x17bc, 0x17a1, 0x17bb, 0x17c6, 0x1794, 0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1780, 0x17bc, 0x17a1, 0x17bb, 0x17c6, 0x1794, 0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x17a1, 0x17bb, 0x17c6, 0x1794, 0x17ca, 0x17b8,
0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb,
0x17c6, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1781, 0x17bc, 0x1780, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x1780, 0x17c4, 0x17c7, 0x200b,
0x1781, 0x17bc, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17d2,
0x179a, 0x1787, 0x17bb, 0x17c6, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1781, 0x17bc, 0x1780,
0x1793, 0x17b6, 0x1796, 0x17b6, 0x1780, 0x17cb, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x200b, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1782, 0x17bb, 0x1799, 0x1794,
0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1782, 0x17bb, 0x1799, 0x1794, 0x17b6, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1782, 0x17bb, 0x1799, 0x1794, 0x17b6, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178a, 0x17b6, 0x179c, 0x17b8, 0x179f, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178c, 0x17bb, 0x1799, 0x1798, 0x17c9, 0x17bb,
0x1784, 0x178a, 0x17ba, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x179c, 0x17b8, 0x179b, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x1791, 0x17b8, 0x1798, 0x17d0, 0x179a,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x1793, 0x17c5, 0x1780, 0x17c4, 0x17c7, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1791, 0x17d0,
0x179a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x1793, 0x17c5, 0x1780, 0x17c4, 0x17c7, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1791,
0x17d0, 0x179a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1780, 0x17c4, 0x17c7, 0x17a2,
0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1791, 0x17d0, 0x179a, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc,
0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17c1, 0x1780, 0x17d2, 0x179c, 0x17b6, 0x1791, 0x17d0, 0x179a, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794,
0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6,
0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a,
0x17c9, 0x17bb, 0x1794, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794,
0x200b, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x200b, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x200b,
0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x200b,
0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x200b, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17bb, 0x1784, 0x1794, 0x17bc, 0x1796, 0x17cc, 0x17b6,
0x1793, 0x17c3, 0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9,
0x17bb, 0x1794, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7,
0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c,
0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x1780, 0x17c4, 0x17c7, 0x200b,
0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5,
0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x17a0,
0x17d2, 0x179c, 0x1780, 0x17cb, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x200b, 0x1793, 0x17c5, 0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x200b, 0x1780, 0x17c4,
0x17c7, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x17a1, 0x17c2, 0x1793, 0x1793, 0x17b6,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x1787, 0x17b8, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x1787, 0x17b8, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x1787,
0x17b8, 0x1793, 0x17b6, 0x200b, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2,
0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2,
0x1782, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x1793, 0x200b, 0x1794, 0x17b6, 0x179a, 0x17b6, 0x17c6,
0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1794, 0x17b6, 0x179a, 0x17b6, 0x17c6,
0x1784, 0x1781, 0x17b6, 0x1784, 0x178f, 0x17d2, 0x1794, 0x17bc, 0x1784, 0x20, 0x1793, 0x17b7,
0x1784, 0x1793, 0x17c5, 0x17a2, 0x1784, 0x17cb, 0x178f, 0x17b6, 0x17c6, 0x1784, 0x1791, 0x17b7,
0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x17a1,
0x17b6, 0x1794, 0x17c9, 0x17b6, 0x1780, 0x17bc, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x17c6, 0x1794, 0x17c0, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179f, 0x1780, 0x17a0, 0x17d2, 0x179f, 0x17ca,
0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179f, 0x1780, 0x17a0, 0x17d2,
0x179f, 0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0,
0x17d2, 0x179f, 0x1780, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17b8, 0x1793, 0x17b6, 0x200b, 0x200b,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1780, 0x17b8, 0x1794,
0x17ca, 0x17ba, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1782, 0x17d2, 0x179a,
0x17b8, 0x1793, 0x179c, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b, 0x17c2, 0x1793,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x200b, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x200b, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b,
0x17c2, 0x1793, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8,
0x1793, 0x179b, 0x17c2, 0x1793, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1793,
0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793,
0x179b, 0x17c2, 0x1793, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x17a0,
0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b, 0x17c2, 0x1793, 0x1781, 0x17b6, 0x1784,
0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x1782,
0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b, 0x17c2, 0x1793, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7,
0x1785, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x179b, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17b6, 0x179c, 0x17c3, 0x2d, 0x17a2, 0x17b6, 0x179b,
0x17cb, 0x178a, 0x17d2, 0x1799, 0x17bc, 0x179f, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a0, 0x17b6, 0x179c, 0x17c3, 0x2d, 0x17a2, 0x17b6, 0x179b, 0x17cb, 0x178a,
0x17d2, 0x1799, 0x17bc, 0x179f, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796,
0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0,
0x17b6, 0x179c, 0x17c3, 0x2d, 0x17a2, 0x17b6, 0x179b, 0x17cb, 0x178a, 0x17d2, 0x1799, 0x17bc,
0x179f, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0,
0x17bb, 0x1784, 0x1780, 0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17bb,
0x1784, 0x1780, 0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x17a0,
0x17bb, 0x1784, 0x1780, 0x17bb, 0x1784, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c,
0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a0, 0x17bc, 0x179c, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x17a0, 0x17bc, 0x179c,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17bc, 0x179c, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x1793, 0x17c5, 0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17a0, 0x17b6, 0x179f, 0x1798, 0x17bb, 0x1791, 0x17d2,
0x179a, 0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17bc, 0x1785, 0x17b7, 0x1793, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17bc,
0x178e, 0x17c1, 0x179f, 0x17ca, 0x17b8, 0x200b, 0x200b, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a,
0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a5, 0x178e,
0x17d2, 0x178c, 0x17bc, 0x178e, 0x17c1, 0x179f, 0x17ca, 0x17b8, 0x200b, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17bc, 0x178e, 0x17c1, 0x179f, 0x17ca, 0x17b8, 0x200b,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x17a2, 0x17c0, 0x179a, 0x1782, 0x17bb, 0x178f, 0x179f,
0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x17a2, 0x17c0, 0x179a, 0x1782, 0x17bb, 0x178f,
0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x17a2,
0x17c0, 0x179a, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1793, 0x17b6,
0x179a, 0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x179a, 0x17b6, 0x17a2,
0x17c2, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2,
0x179a, 0x17b6, 0x17a2, 0x17c2, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1,
0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca,
0x17b8, 0x179f, 0x17d2, 0x179a, 0x17b6, 0x17a2, 0x17c2, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1787, 0x1794, 0x17c9, 0x17bb, 0x1793, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1787, 0x1794, 0x17c9, 0x17bb, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x1787,
0x1794, 0x17c9, 0x17bb, 0x1793, 0x1796, 0x17c1, 0x179b, 0x179c, 0x17c1, 0x179b, 0x17b6, 0x1793,
0x17c5, 0x1780, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x1780, 0x17cb, 0x179f, 0x17d2, 0x1790,
0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1780, 0x17b6, 0x17a0, 0x17d2, 0x179f,
0x17b6, 0x1780, 0x17cb, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x200b, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1780, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x1780, 0x17cb, 0x179f, 0x17d2, 0x1790,
0x17b6, 0x1793, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x200b, 0x200b, 0x179b, 0x17b7, 0x1785,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x179a, 0x17c9,
0x17c1, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x179a, 0x17c9, 0x17c1, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x179a, 0x17c9, 0x17c1, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x179f, 0x17d2, 0x179a, 0x17c3, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x178e,
0x17bc, 0x1799, 0x17c9, 0x17b6, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17d2,
0x179a, 0x17b6, 0x178e, 0x17bc, 0x1799, 0x17c9, 0x17b6, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x178e, 0x17bc, 0x1799, 0x17c9,
0x17b6, 0x179f, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2,
0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1780, 0x17c0, 0x17a0, 0x17d2,
0x179f, 0x17ca, 0x17b8, 0x179f, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x17a1,
0x17b6, 0x1789, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a1, 0x178f,
0x17a0, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a1, 0x178f, 0x200b, 0x17a0, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784,
0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a1, 0x178f, 0x17a0, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1780, 0x17b6,
0x178a, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x17a0,
0x17d2, 0x1780, 0x17b6, 0x178a, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1780, 0x17b6, 0x178a, 0x17b6, 0x1793,
0x200b, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x17a1,
0x17c1, 0x179f, 0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1798, 0x17c9, 0x17b6, 0x179b, 0x17cb, 0x178c, 0x17b8, 0x179c, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x1782,
0x17ba, 0x179f, 0x17b6, 0x179f, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1798, 0x17c9, 0x17b6, 0x179f, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1798, 0x17c9, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1793,
0x17c5, 0x200b, 0x1798, 0x17c9, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c5, 0x200b, 0x179f, 0x17b6, 0x1793, 0x17cb,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179f,
0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x200b, 0x1798, 0x17c9, 0x17b7, 0x1780,
0x179f, 0x17ca, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179f,
0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x200b, 0x1798, 0x17c9, 0x17b7, 0x1780,
0x179f, 0x17ca, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b,
0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6,
0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x200b, 0x1798, 0x17c9, 0x17b7,
0x1780, 0x179f, 0x17ca, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17ca, 0x17bc, 0x17a1, 0x17b6, 0x1793, 0x200b, 0x1794, 0x17b6, 0x1791, 0x17bc,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bc, 0x17a1, 0x17b6, 0x1793, 0x200b,
0x1794, 0x17b6, 0x1791, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17ca, 0x17bc, 0x17a1, 0x17b6, 0x1793, 0x200b, 0x1794, 0x17b6, 0x1791, 0x17bc, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17bc, 0x179f, 0x17d2, 0x1782,
0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17bc, 0x179f, 0x17d2, 0x1782, 0x17bc,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17bc, 0x179f, 0x17d2,
0x1782, 0x17bc, 0x200b, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2,
0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17b8,
0x1799, 0x17c9, 0x17b6, 0x1793, 0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x178e, 0x17bc, 0x179a, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x1793, 0x17c5, 0x1793, 0x17c1, 0x1794, 0x17c9, 0x17b6, 0x179b, 0x17cb, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17c2, 0x179b, 0x1780, 0x17b6,
0x17a1, 0x17c1, 0x178a, 0x17bc, 0x1793, 0x17c0, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x178e, 0x17bc, 0x179c,
0x17c9, 0x17c2, 0x179b, 0x1780, 0x17b6, 0x17a1, 0x17c1, 0x178a, 0x17bc, 0x1793, 0x17c0, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17c2, 0x179b,
0x1780, 0x17b6, 0x17a1, 0x17c1, 0x178a, 0x17bc, 0x1793, 0x17c0, 0x1793, 0x17b6, 0x179a, 0x178a,
0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1793, 0x17bc, 0x179c, 0x17c2, 0x179b, 0x179f, 0x17c1, 0x17a1, 0x1784, 0x17cb,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1793, 0x17bc, 0x179c, 0x17c2, 0x179b, 0x179f, 0x17c1,
0x17a1, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b,
0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1793, 0x17bc, 0x179c, 0x17c2,
0x179b, 0x179f, 0x17c1, 0x17a1, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b,
0x1793, 0x17c5, 0x1789, 0x17bc, 0x179c, 0x17a0, 0x17d2, 0x179c, 0x17c4, 0x1793, 0x17a1, 0x17c2,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1789, 0x17bc, 0x179c, 0x17a0,
0x17d2, 0x179c, 0x17c4, 0x1793, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1796, 0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1789,
0x17bc, 0x179c, 0x17a0, 0x17d2, 0x179c, 0x17c4, 0x1793, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x1793, 0x17b8, 0x179c, 0x17c9, 0x17c1, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x178e, 0x17d0,
0x179a, 0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x17d2, 0x178a, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178e,
0x17d0, 0x179a, 0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793,
0x17c5, 0x200b, 0x178e, 0x17d0, 0x179a, 0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x179c, 0x17ca, 0x17c2, 0x178e, 0x17b6,
0x1793, 0x17cb, 0x178a, 0x17bc, 0x200b, 0x178a, 0x17ba, 0x178e, 0x17bc, 0x179a, 0x17c9, 0x17bc,
0x1789, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x179c, 0x17ca, 0x17c2, 0x178e,
0x17b6, 0x1793, 0x17cb, 0x178a, 0x17bc, 0x200b, 0x178a, 0x17ba, 0x178e, 0x17bc, 0x179a, 0x17c9,
0x17bc, 0x1789, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x17a0,
0x17d2, 0x179c, 0x17ca, 0x17c2, 0x178e, 0x17b6, 0x1793, 0x17cb, 0x178a, 0x17bc, 0x200b, 0x178a,
0x17ba, 0x178e, 0x17bc, 0x179a, 0x17c9, 0x17bc, 0x1789, 0x17c9, 0x17b6, 0x1793, 0x17b6, 0x179a,
0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x1794,
0x17ca, 0x17b8, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178e, 0x17bc, 0x179c, 0x17c9,
0x17bc, 0x179f, 0x17ca, 0x17b8, 0x1794, 0x17ca, 0x17b8, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17ca, 0x17b8,
0x1794, 0x17ca, 0x17b8, 0x1780, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b,
0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17bc, 0x1798, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x1798, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x1798, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x1782, 0x17b8, 0x179f,
0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178a,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6,
0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793,
0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793,
0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1795, 0x17b6, 0x17a1,
0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17c9, 0x17b6, 0x1796,
0x17bc, 0x17a2, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x20, 0x1793, 0x17bc, 0x179c, 0x17c2, 0x179b,
0x17a0, 0x17d2, 0x1782, 0x17b8, 0x178e, 0x17c1, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b6,
0x1799, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6,
0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x1794, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1793,
0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17c1, 0x179a, 0x17bc,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17c1, 0x179a, 0x17bc, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17c1, 0x179a, 0x17bc,
0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x179b,
0x17b8, 0x1796, 0x17b8, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c,
0x17b8, 0x179b, 0x17b8, 0x1796, 0x17b8, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x179b, 0x17b8, 0x1796, 0x17b8, 0x1793,
0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b,
0x1795, 0x17bc, 0x1793, 0x17b8, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x200b,
0x1793, 0x17c5, 0x179f, 0x1784, 0x17cb, 0x200b, 0x1796, 0x17d2, 0x1799, 0x17c2, 0x179a, 0x20,
0x1793, 0x17b7, 0x1784, 0x200b, 0x1798, 0x17b8, 0x1782, 0x17bb, 0x1799, 0x17a1, 0x17bb, 0x1784,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x179f, 0x1784, 0x17cb, 0x200b, 0x1796, 0x17d2, 0x1799, 0x17c2,
0x179a, 0x20, 0x1793, 0x17b7, 0x1784, 0x200b, 0x1798, 0x17b8, 0x1782, 0x17bb, 0x1799, 0x17a1,
0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790,
0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x179f, 0x1784, 0x17cb, 0x200b, 0x1796, 0x17d2,
0x1799, 0x17c2, 0x179a, 0x20, 0x1793, 0x17b7, 0x1784, 0x200b, 0x1798, 0x17b8, 0x1782, 0x17bb,
0x1799, 0x17a1, 0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1797, 0x17b8, 0x1783, 0x17ba, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1794, 0x17c9, 0x17bc, 0x178e, 0x17b6, 0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x1793, 0x17c5, 0x1796, 0x17d2, 0x1799, 0x17bb, 0x1784, 0x1799, 0x17c9, 0x17b6, 0x1784, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x179a, 0x17c1, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x17c9,
0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179a, 0x17c9,
0x17bc, 0x1792, 0x17ba, 0x179a, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x179f, 0x17b6, 0x1780, 0x17cb, 0x1781, 0x17b6, 0x179b, 0x17b8, 0x1793, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x179f, 0x17b6, 0x1780, 0x17cb, 0x1781, 0x17b6, 0x179b, 0x17b8,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179f, 0x17b6, 0x1780,
0x17cb, 0x1781, 0x17b6, 0x179b, 0x17b8, 0x1793, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc,
0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x179f, 0x17b6, 0x1798, 0x17d0, 0x179a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x200b, 0x179f,
0x17b6, 0x1798, 0x17d0, 0x179a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x179f, 0x17b6, 0x1798, 0x17d0, 0x179a, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780,
0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179f,
0x17b8, 0x179f, 0x17d2, 0x17a0, 0x17c2, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x179f, 0x17b7, 0x1784, 0x17d2, 0x17a0, 0x1794, 0x17bc, 0x179a, 0x17b8, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x179f,
0x17bc, 0x17a1, 0x17bc, 0x1798, 0x17c9, 0x17bb, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x17a0, 0x17d2, 0x179f, 0x1780, 0x17a0,
0x17d2, 0x179f, 0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x179f, 0x17bc, 0x179a, 0x17b8, 0x178e, 0x17b6, 0x1798, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x179f, 0x17ca, 0x17b8, 0x17a2, 0x17bc, 0x179c, 0x17c9, 0x17b6, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17b6, 0x17a0, 0x17b7, 0x1791,
0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17c3, 0x1794,
0x17c9, 0x17b7, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17c3, 0x1794, 0x17c9, 0x17b7,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784,
0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17c3, 0x1794, 0x17c9, 0x17b7, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x1793, 0x17c5, 0x178f, 0x17b6, 0x1787, 0x17b8, 0x1782, 0x17b8, 0x179f, 0x17d2,
0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f,
0x17bc, 0x1781, 0x17c1, 0x17a1, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x200b, 0x1793,
0x17c5, 0x200b, 0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782,
0x17b6, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17bc, 0x17a2, 0x17bb,
0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17bd, 0x1780,
0x1798, 0x17c9, 0x17c1, 0x1793, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x178f, 0x17bd, 0x1780, 0x1798, 0x17c9, 0x17c1, 0x1793, 0x17b8, 0x179f, 0x17d2,
0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c,
0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17bd, 0x1780,
0x1798, 0x17c9, 0x17c1, 0x1793, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x200b, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1791, 0x17bb, 0x1799, 0x179c, 0x17c9,
0x17b6, 0x179b, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17ca, 0x17bb, 0x1799, 0x179a, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x179a, 0x17bc, 0x17a0, 0x17d2, 0x1782,
0x17b6, 0x1799, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca,
0x17bb, 0x1799, 0x179a, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1793, 0x17b6, 0x200b,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bc, 0x179f, 0x1794, 0x17c1,
0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17ca, 0x17bc, 0x179f, 0x1794, 0x17c1, 0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bc,
0x179f, 0x1794, 0x17c1, 0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1793, 0x17b6,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17b6, 0x1793, 0x17bc, 0x1791,
0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17b6, 0x1793, 0x17bc,
0x1791, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9,
0x17b6, 0x1793, 0x17bc, 0x1791, 0x17bc, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b,
0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x179c, 0x17c9, 0x17c1, 0x178e, 0x17c1, 0x179f, 0x17ca, 0x17bb, 0x1799, 0x17a2, 0x17c1,
0x17a1, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9,
0x17d2, 0x179b, 0x17b6, 0x178c, 0x17b8, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb,
0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17d2, 0x179b, 0x17b6, 0x178c,
0x17b8, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17d2, 0x179b, 0x17b6, 0x178c, 0x17b8,
0x179c, 0x17c9, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1780, 0x17c4, 0x1780,
0x17d2, 0x179a, 0x17b6, 0x178a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17bc,
0x17a0, 0x17d2, 0x1780, 0x17c4, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x178a, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1780, 0x17c4,
0x1780, 0x17d2, 0x179a, 0x17b6, 0x178a, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c,
0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x179c, 0x17d0, 0x179a, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x179c, 0x17c1, 0x1780, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17b6, 0x179b, 0x17b8,
0x179f, 0x20, 0x1793, 0x17b7, 0x1784, 0x17a0, 0x17d2, 0x179c, 0x17ca, 0x17bb, 0x1791, 0x17bc,
0x178e, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1799, 0x17c9,
0x17b6, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1799, 0x17c9, 0x17b6, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7,
0x17cd, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1799, 0x17c9, 0x17b6,
0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b7, 0x1781, 0x17b6, 0x1792, 0x17ba, 0x179a,
0x17b8, 0x1793, 0x1794, 0x17ca, 0x17d0, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17ca, 0x17b7, 0x1781, 0x17b6, 0x1792, 0x17ba, 0x179a, 0x17b8, 0x1793, 0x1794, 0x17ca, 0x17d0,
0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b7,
0x1781, 0x17b6, 0x1792, 0x17ba, 0x179a, 0x17b8, 0x1793, 0x1794, 0x17ca, 0x17d0, 0x1780, 0x1793,
0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1799, 0x17bc, 0x1781, 0x1793, 0x17cb, 0x905,
0x92b, 0x917, 0x93e, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f,
0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20,
0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x926,
0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915,
0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x906, 0x92b, 0x94d,
0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924,
0x902, 0x924, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905,
0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932, 0x93e, 0x938,
0x94d, 0x915, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935,
0x947, 0x933, 0x905, 0x92e, 0x947, 0x91d, 0x94b, 0x928, 0x20, 0x935, 0x947, 0x933,
0x905, 0x92e, 0x947, 0x91d, 0x94b, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x92e, 0x947, 0x91d, 0x94b,
0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d,
0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f,
0x91f, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x935,
0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902,
0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x930, 0x94d, 0x935, 0x924, 0x940, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x930, 0x94d, 0x935, 0x924, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x94d, 0x935, 0x924,
0x940, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94d, 0x930, 0x936,
0x93e, 0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935,
0x947, 0x933, 0x905, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x905,
0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x92c,
0x940, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x92c, 0x940, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930,
0x92c, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947,
0x933, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20,
0x935, 0x947, 0x933, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928,
0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947,
0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d,
0x924, 0x902, 0x924, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f,
0x928, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x905, 0x930,
0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e,
0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d,
0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e, 0x947,
0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x20, 0x935, 0x947, 0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x940, 0x915,
0x20, 0x935, 0x947, 0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x940, 0x915,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x940, 0x915, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x940, 0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d,
0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933,
0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928, 0x20,
0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20,
0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940,
0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x924,
0x902, 0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940,
0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x924,
0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947,
0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x940, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928, 0x20, 0x909, 0x926,
0x947, 0x902, 0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x940, 0x92f, 0x928, 0x20, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924,
0x902, 0x924, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940,
0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x940, 0x92f, 0x928, 0x20, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928,
0x20, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c,
0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c,
0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x905,
0x91d, 0x94b, 0x930, 0x947, 0x938, 0x905, 0x91d, 0x94b, 0x930, 0x947, 0x938, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x905, 0x91d, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932, 0x93e,
0x926, 0x947, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932,
0x93e, 0x926, 0x947, 0x936, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932, 0x93e, 0x926,
0x947, 0x936, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0x92d, 0x942, 0x924, 0x93e, 0x928, 0x92c, 0x94b, 0x932, 0x93f, 0x935,
0x94d, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930,
0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92c,
0x94d, 0x930, 0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d,
0x930, 0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x941,
0x928, 0x947, 0x907, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938, 0x94d, 0x938, 0x932,
0x93e, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930,
0x94d, 0x926, 0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935,
0x930, 0x94d, 0x926, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d,
0x926, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0x915, 0x945, 0x92e, 0x94b, 0x930, 0x93e, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945, 0x925,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945, 0x925, 0x92e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945,
0x925, 0x92e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947,
0x933, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x93f, 0x932,
0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x940, 0x928, 0x20, 0x935, 0x947,
0x933, 0x91a, 0x940, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x940, 0x928, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93f, 0x938,
0x92e, 0x938, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933,
0x915, 0x94b, 0x915, 0x94b, 0x938, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20,
0x935, 0x947, 0x933, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20,
0x935, 0x947, 0x933, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x942, 0x915, 0x20,
0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x915, 0x942, 0x915,
0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x915, 0x942, 0x915, 0x20, 0x906,
0x92f, 0x932, 0x901, 0x921, 0x20, 0x905, 0x930, 0x94d, 0x926, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x92f,
0x941, 0x92c, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x92f, 0x941, 0x92c,
0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935,
0x947, 0x933, 0x915, 0x94d, 0x92f, 0x941, 0x92c, 0x93e, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x921, 0x947, 0x935, 0x94d, 0x939,
0x940, 0x938, 0x20, 0x935, 0x947, 0x933, 0x926, 0x94d, 0x92f, 0x941, 0x92e, 0x93e,
0x901, 0x20, 0x926, 0x942, 0x930, 0x935, 0x940, 0x92f, 0x20, 0x935, 0x947, 0x933,
0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x924, 0x93f, 0x92e, 0x94b, 0x930, 0x20,
0x935, 0x947, 0x933, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x92f, 0x932,
0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20,
0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20,
0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x907, 0x915, 0x94d, 0x935, 0x947, 0x921, 0x94b,
0x930, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x941,
0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927,
0x94d, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f,
0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a,
0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924,
0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926,
0x947, 0x902, 0x924, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x906, 0x928, 0x940, 0x915, 0x2d, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x92f,
0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905,
0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f,
0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924,
0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938,
0x94d, 0x924, 0x902, 0x924, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f,
0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x92b, 0x949, 0x915, 0x932, 0x901, 0x921, 0x20, 0x906, 0x92f, 0x932, 0x901,
0x921, 0x94d, 0x938, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915, 0x932, 0x901,
0x921, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x94d, 0x938, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949,
0x915, 0x932, 0x901, 0x921, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x94d, 0x938,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x92b, 0x93f, 0x91c, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x91c, 0x940,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x92b, 0x93f, 0x91c, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x94d, 0x930, 0x947, 0x902, 0x91a, 0x20,
0x917, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x94d, 0x930,
0x947, 0x928, 0x94d, 0x91a, 0x20, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20,
0x906, 0x928, 0x940, 0x20, 0x905, 0x902, 0x91f, 0x93e, 0x930, 0x94d, 0x915, 0x94d,
0x91f, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933, 0x917, 0x93e, 0x932, 0x93e, 0x92a,
0x93e, 0x917, 0x94b, 0x938, 0x20, 0x935, 0x947, 0x933, 0x917, 0x93e, 0x92e, 0x94d,
0x92c, 0x93f, 0x92f, 0x930, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x949, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x949, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930, 0x94d, 0x91f, 0x20, 0x906, 0x92f,
0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x935, 0x947, 0x933,
0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932,
0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947,
0x902, 0x924, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x905,
0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932,
0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938,
0x94d, 0x924, 0x902, 0x924, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901,
0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x917, 0x932, 0x94d, 0x92b, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932, 0x947,
0x92f, 0x941, 0x936, 0x93f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e,
0x908, 0x2d, 0x905, 0x932, 0x947, 0x92f, 0x941, 0x936, 0x93f, 0x928, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x939,
0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932, 0x947, 0x92f, 0x941, 0x936, 0x93f, 0x928,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x939,
0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x935, 0x947, 0x933,
0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93e,
0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x939, 0x94b, 0x935, 0x94d, 0x939,
0x921, 0x939, 0x94b, 0x935, 0x94d, 0x939, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x939, 0x94b, 0x935, 0x94d,
0x939, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0x92d, 0x93e, 0x930, 0x924, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93f, 0x902,
0x926, 0x20, 0x92e, 0x94d, 0x939, 0x93e, 0x938, 0x93e, 0x917, 0x930, 0x20, 0x935,
0x947, 0x933, 0x907, 0x902, 0x921, 0x94b, 0x91a, 0x940, 0x928, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947,
0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902,
0x924, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x907,
0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x907, 0x930, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x93e, 0x928, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x94d,
0x915, 0x941, 0x91f, 0x938, 0x94d, 0x915, 0x20, 0x935, 0x947, 0x933, 0x908, 0x930,
0x94d, 0x915, 0x941, 0x91f, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x908, 0x930, 0x94d, 0x915,
0x941, 0x91f, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932,
0x20, 0x935, 0x947, 0x933, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x935,
0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x915, 0x91d,
0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x909,
0x926, 0x947, 0x902, 0x924, 0x20, 0x915, 0x91d, 0x93e, 0x915, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924,
0x20, 0x915, 0x91d, 0x93e, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935,
0x947, 0x933, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933,
0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x930, 0x93f, 0x92f,
0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933,
0x915, 0x94b, 0x938, 0x930, 0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930,
0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b,
0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e,
0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x915,
0x93f, 0x930, 0x94d, 0x917, 0x93f, 0x91c, 0x93c, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x935, 0x947, 0x933, 0x932, 0x93e, 0x92f, 0x928, 0x20, 0x906, 0x92f, 0x932,
0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20,
0x939, 0x94b, 0x935, 0x947, 0x20, 0x935, 0x947, 0x933, 0x932, 0x949, 0x930, 0x94d,
0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x932, 0x949, 0x930, 0x94d, 0x921,
0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f,
0x20, 0x935, 0x947, 0x933, 0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x917, 0x93e,
0x926, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x92e, 0x932, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x93e, 0x932, 0x926, 0x940, 0x935, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93f, 0x938, 0x938, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x93e, 0x930, 0x94d, 0x936, 0x932, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921,
0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x930, 0x93f, 0x936, 0x938, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x949, 0x930, 0x93f, 0x936, 0x938, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x930,
0x93f, 0x936, 0x938, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x92e, 0x949, 0x938, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936,
0x93e, 0x902, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x947, 0x915, 0x94d, 0x938,
0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x936, 0x93e, 0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20,
0x935, 0x947, 0x933, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x924, 0x93e, 0x930, 0x20,
0x935, 0x947, 0x933, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x924, 0x93e, 0x930, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x909, 0x932, 0x93e, 0x928, 0x92c, 0x924, 0x93e, 0x930, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x938, 0x94d,
0x915, 0x94b, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x94d, 0x92f, 0x93e, 0x928,
0x92e, 0x93e, 0x930, 0x20, 0x935, 0x947, 0x933, 0x928, 0x930, 0x942, 0x20, 0x935,
0x947, 0x933, 0x928, 0x947, 0x92a, 0x93e, 0x933, 0x20, 0x935, 0x947, 0x933, 0x928,
0x94d, 0x92f, 0x941, 0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x20, 0x915,
0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d,
0x92f, 0x941, 0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x928, 0x94d, 0x92f, 0x941, 0x91d, 0x940, 0x932, 0x945, 0x928, 0x94d, 0x921,
0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x91d, 0x940, 0x932, 0x945,
0x928, 0x94d, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x91d, 0x940, 0x932, 0x945,
0x928, 0x94d, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935,
0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x92b, 0x93e, 0x935, 0x902, 0x921, 0x932,
0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x92b, 0x93e,
0x935, 0x902, 0x921, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x92b,
0x93e, 0x935, 0x902, 0x921, 0x932, 0x901, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x90f, 0x20,
0x935, 0x947, 0x933, 0x928, 0x949, 0x930, 0x92b, 0x949, 0x915, 0x20, 0x906, 0x92f,
0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949, 0x930, 0x92b, 0x949,
0x915, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949, 0x930, 0x92b,
0x949, 0x915, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928,
0x93e, 0x902, 0x921, 0x94b, 0x20, 0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b,
0x928, 0x94d, 0x939, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928,
0x93e, 0x902, 0x921, 0x94b, 0x20, 0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b,
0x928, 0x94d, 0x939, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x921,
0x94b, 0x20, 0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x20, 0x935, 0x947, 0x933, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x935,
0x947, 0x933, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x913, 0x92e, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x932,
0x93e, 0x90a, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93e, 0x92a, 0x941, 0x906, 0x20,
0x928, 0x94d, 0x92f, 0x941, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x930, 0x93e, 0x917, 0x941, 0x906, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x930, 0x93e, 0x917, 0x941, 0x906, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x93e, 0x917, 0x941, 0x906,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x947, 0x930, 0x942, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x947, 0x930, 0x942,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93e, 0x92f,
0x928, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93e, 0x92f,
0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93e, 0x92f, 0x928, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f,
0x928, 0x93f, 0x915, 0x94d, 0x938, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20,
0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930,
0x20, 0x906, 0x928, 0x940, 0x20, 0x92e, 0x93f, 0x915, 0x932, 0x93e, 0x928, 0x20,
0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930,
0x20, 0x906, 0x928, 0x940, 0x20, 0x92e, 0x93f, 0x915, 0x932, 0x93e, 0x928, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933,
0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x20, 0x906, 0x928,
0x940, 0x20, 0x92e, 0x93f, 0x915, 0x932, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93f, 0x91f, 0x915, 0x945,
0x930, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94b, 0x928, 0x947, 0x92a, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x94d, 0x92f, 0x94b, 0x902, 0x917, 0x92f, 0x93e, 0x902,
0x917, 0x20, 0x935, 0x947, 0x933, 0x930, 0x93f, 0x92f, 0x941, 0x928, 0x93f, 0x92f,
0x928, 0x20, 0x935, 0x947, 0x933, 0x930, 0x94b, 0x925, 0x947, 0x930, 0x93e, 0x20,
0x935, 0x947, 0x933, 0x938, 0x916, 0x932, 0x93f, 0x928, 0x20, 0x935, 0x947, 0x933,
0x938, 0x916, 0x932, 0x93f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x938, 0x916, 0x932, 0x93f, 0x928, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x938,
0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e, 0x94b,
0x906, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947, 0x936, 0x947, 0x932, 0x94d,
0x938, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x942,
0x930, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x928, 0x20, 0x906, 0x907, 0x932,
0x901, 0x921, 0x94d, 0x938, 0x20, 0x935, 0x947, 0x933, 0x926, 0x915, 0x94d, 0x937,
0x93f, 0x923, 0x20, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20,
0x935, 0x947, 0x933, 0x938, 0x941, 0x930, 0x93f, 0x928, 0x93e, 0x92e, 0x20, 0x935,
0x947, 0x933, 0x938, 0x94d, 0x92f, 0x94b, 0x935, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x924, 0x93e, 0x939, 0x93f, 0x924, 0x940, 0x20, 0x935, 0x947, 0x933, 0x924, 0x948,
0x92a, 0x947, 0x908, 0x20, 0x935, 0x947, 0x933, 0x924, 0x948, 0x92a, 0x947, 0x908,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x924, 0x948, 0x92a, 0x947, 0x908, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f,
0x91f, 0x20, 0x935, 0x947, 0x933, 0x924, 0x91c, 0x940, 0x915, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x91f, 0x94b, 0x915, 0x932, 0x93e,
0x90a, 0x20, 0x935, 0x947, 0x933, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x91f, 0x94b, 0x902, 0x917,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x91a, 0x941, 0x915, 0x20, 0x935, 0x947, 0x933, 0x924, 0x941, 0x930, 0x94d,
0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x924,
0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x91f, 0x941, 0x935, 0x93e, 0x932, 0x942, 0x20, 0x935, 0x947, 0x933, 0x909,
0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x935, 0x947, 0x933, 0x909, 0x930,
0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935,
0x947, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x91c, 0x93c, 0x94d,
0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x935, 0x928, 0x93e,
0x924, 0x942, 0x20, 0x935, 0x947, 0x933, 0x935, 0x928, 0x93e, 0x924, 0x942, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x935, 0x928, 0x93e, 0x924, 0x942, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x935, 0x947, 0x928, 0x947, 0x91d, 0x941, 0x90f,
0x932, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f,
0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x935, 0x947, 0x933, 0x935,
0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924,
0x94b, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d,
0x930, 0x93e, 0x921, 0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932,
0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x939,
0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x935, 0x948,
0x915, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x935,
0x93e, 0x932, 0x93f, 0x938, 0x20, 0x906, 0x928, 0x940, 0x20, 0x92b, 0x94d, 0x92f,
0x942, 0x91a, 0x942, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x915, 0x941,
0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x915, 0x941,
0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c,
0x930, 0x94d, 0x917, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x947, 0x915, 0x93e, 0x924,
0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x947, 0x915,
0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x941,
0x915, 0x949, 0x928, 0x20, 0x935, 0x947, 0x933, 0xc544, 0xd06c, 0xb808, 0x20, 0xc2dc,
0xac04, 0xc544, 0xd06c, 0xb808, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xd06c, 0xb808, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xd504, 0xac00, 0xb2c8, 0xc2a4, 0xd0c4,
0x20, 0xc2dc, 0xac04, 0xc911, 0xc559, 0xc544, 0xd504, 0xb9ac, 0xce74, 0x20, 0xc2dc, 0xac04,
0xb3d9, 0xc544, 0xd504, 0xb9ac, 0xce74, 0x20, 0xc2dc, 0xac04, 0xb0a8, 0xc544, 0xd504, 0xb9ac,
0xce74, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xc544, 0xd504, 0xb9ac, 0xce74, 0x20, 0xc2dc, 0xac04,
0xc11c, 0xc544, 0xd504, 0xb9ac, 0xce74, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc11c, 0xc544, 0xd504,
0xb9ac, 0xce74, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc54c, 0xb798, 0xc2a4,
0xce74, 0x20, 0xc2dc, 0xac04, 0xc54c, 0xb798, 0xc2a4, 0xce74, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc54c, 0xb798, 0xc2a4, 0xce74, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc54c,
0xb9c8, 0xd2f0, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xc54c, 0xb9c8, 0xd2f0, 0x20,
0xd45c, 0xc900, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc54c, 0xb9c8, 0xd2f0, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb9c8, 0xc874, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb9c8,
0xc874, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb9c8, 0xc874, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xc911, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xbbf8, 0x20,
0xc911, 0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xc911, 0xbd80, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xb3d9, 0xbd80, 0x20, 0xc2dc, 0xac04,
0xbbf8, 0x20, 0xb3d9, 0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xb3d9, 0xbd80,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xc0b0, 0xc9c0, 0x20,
0xc2dc, 0xac04, 0xbbf8, 0x20, 0xc0b0, 0xc545, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20,
0xc0b0, 0xc9c0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xd0dc,
0xd3c9, 0xc591, 0x20, 0xc2dc, 0xac04, 0xbbf8, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xbbf8, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc544, 0xb098, 0xb514, 0xb9ac, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb098, 0xb514,
0xb9ac, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb098, 0xb514, 0xb9ac, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xd53c, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc544, 0xd53c,
0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xd53c, 0xc544, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc545, 0xd0c0, 0xc6b0, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04,
0xc545, 0xd0c0, 0xc6b0, 0x20, 0xd45c, 0xc900, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc545, 0xd0c0,
0xc6b0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc545, 0xd1f4, 0xbca0, 0x20,
0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xc545, 0xd1f4, 0xbca0, 0x20, 0xd45c, 0xc900, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc545, 0xd1f4, 0xbca0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc544, 0xb77c, 0xbe44, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb77c, 0xbe44, 0xc544,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb77c, 0xbe44, 0xc544, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20, 0xc2dc, 0xac04, 0xc544,
0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xd5e8, 0xd2f0,
0xb098, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xd5e8, 0xd2f0,
0xb098, 0x20, 0xc11c, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098,
0x20, 0xc11c, 0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098,
0x20, 0xc11c, 0xbd80, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974,
0xba54, 0xb2c8, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb974, 0xba54, 0xb2c8, 0xc544, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xba54, 0xb2c8, 0xc544, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xb300, 0xc11c, 0xc591, 0x20, 0xc2dc, 0xac04, 0xb300, 0xc11c, 0xc591,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xb300, 0xc11c, 0xc591, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xbd80,
0x20, 0xc2dc, 0xac04, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911,
0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544,
0x20, 0xc911, 0xbd80, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4,
0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xc11c, 0xbd80, 0x20, 0xc2dc, 0xac04,
0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xc11c, 0xbd80, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911,
0xc11c, 0xbd80, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8,
0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xb3d9, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xc624, 0xc2a4,
0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xb3d9, 0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xb3d9, 0xbd80, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544,
0x20, 0xc11c, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac,
0xc544, 0x20, 0xc11c, 0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808,
0xc77c, 0xb9ac, 0xc544, 0x20, 0xc11c, 0xbd80, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc544, 0xc81c, 0xb974, 0xbc14, 0xc774, 0xc794, 0x20, 0xc2dc, 0xac04, 0xc544, 0xc81c,
0xb974, 0xbc14, 0xc774, 0xc794, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xc81c, 0xb974, 0xbc14,
0xc774, 0xc794, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xc870, 0xb808,
0xc2a4, 0x20, 0xc2dc, 0xac04, 0xc544, 0xc870, 0xb808, 0xc2a4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc544, 0xc870, 0xb808, 0xc2a4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbc29,
0xae00, 0xb77c, 0xb370, 0xc2dc, 0x20, 0xc2dc, 0xac04, 0xbc29, 0xae00, 0xb77c, 0xb370, 0xc2dc,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xbc29, 0xae00, 0xb77c, 0xb370, 0xc2dc, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xbd80, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xbcfc, 0xb9ac, 0xbe44,
0xc544, 0x20, 0xc2dc, 0xac04, 0xbe0c, 0xb77c, 0xc9c8, 0xb9ac, 0xc544, 0x20, 0xc2dc, 0xac04,
0xbe0c, 0xb77c, 0xc9c8, 0xb9ac, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbe0c, 0xb77c, 0xc9c8,
0xb9ac, 0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbe0c, 0xb8e8, 0xb098,
0xc774, 0x20, 0xc2dc, 0xac04, 0xce74, 0xbcf4, 0x20, 0xbca0, 0xb974, 0xb370, 0x20, 0xc2dc,
0xac04, 0xce74, 0xbcf4, 0x20, 0xbca0, 0xb974, 0xb370, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xce74,
0xbcf4, 0x20, 0xbca0, 0xb974, 0xb370, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xcf00, 0xc774, 0xc2dc, 0x20, 0xc2dc, 0xac04, 0xcc28, 0xbaa8, 0xb85c, 0x20, 0xc2dc, 0xac04,
0xcc44, 0xd140, 0x20, 0xc2dc, 0xac04, 0xcc44, 0xd140, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcc44,
0xd140, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xce60, 0xb808, 0x20, 0xc2dc,
0xac04, 0xce60, 0xb808, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xce60, 0xb808, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc911, 0xad6d, 0x20, 0xc2dc, 0xac04, 0xc911, 0xad6d, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc911, 0xad6d, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xd06c, 0xb9ac, 0xc2a4, 0xb9c8, 0xc2a4, 0xc12c, 0x20, 0xc2dc, 0xac04, 0xcf54, 0xcf54, 0xc2a4,
0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xcf5c, 0xb86c, 0xbe44, 0xc544, 0x20, 0xc2dc,
0xac04, 0xcf5c, 0xb86c, 0xbe44, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcf5c, 0xb86c, 0xbe44,
0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcfe1, 0x20, 0xc81c, 0xb3c4,
0x20, 0xc2dc, 0xac04, 0xcfe1, 0x20, 0xc81c, 0xb3c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcfe1,
0x20, 0xc81c, 0xb3c4, 0x20, 0xc808, 0xbc18, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xcfe0, 0xbc14, 0x20, 0xc2dc, 0xac04, 0xcfe0, 0xbc14, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xcfe0, 0xbc14, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb370, 0xc774, 0xbe44,
0xc2a4, 0x20, 0xc2dc, 0xac04, 0xb4a4, 0xbabd, 0xb4a4, 0xb974, 0xbe4c, 0x20, 0xc2dc, 0xac04,
0xb3d9, 0xd2f0, 0xbaa8, 0xb974, 0x20, 0xc2dc, 0xac04, 0xc774, 0xc2a4, 0xd130, 0xc12c, 0x20,
0xc2dc, 0xac04, 0xc774, 0xc2a4, 0xd130, 0xc12c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xc2a4,
0xd130, 0xc12c, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc5d0, 0xcf70, 0xb3c4,
0xb974, 0x20, 0xc2dc, 0xac04, 0xc911, 0xbd80, 0xc720, 0xb7fd, 0x20, 0xc2dc, 0xac04, 0xc911,
0xbd80, 0xc720, 0xb7fd, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc911, 0xbd80, 0xc720, 0xb7fd, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb3d9, 0xc720, 0xb7fd, 0x20, 0xc2dc, 0xac04,
0xb3d9, 0xc720, 0xb7fd, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb3d9, 0xc720, 0xb7fd, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xadf9, 0xb3d9, 0xc720, 0xb7fd, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc11c, 0xc720, 0xb7fd, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xc720, 0xb7fd, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc11c, 0xc720, 0xb7fd, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xd3ec, 0xd074, 0xb79c, 0xb4dc, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xd3ec, 0xd074,
0xb79c, 0xb4dc, 0x20, 0xc81c, 0xb3c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd3ec, 0xd074, 0xb79c,
0xb4dc, 0x20, 0xc81c, 0xb3c4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd53c,
0xc9c0, 0x20, 0xc2dc, 0xac04, 0xd53c, 0xc9c0, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd53c, 0xc9c0,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd504, 0xb791, 0xc2a4, 0xb839, 0x20,
0xac00, 0xc774, 0xc544, 0xb098, 0x20, 0xc2dc, 0xac04, 0xd504, 0xb791, 0xc2a4, 0xb839, 0x20,
0xb0a8, 0xbd80, 0x20, 0xc2dd, 0xbbfc, 0xc9c0, 0x20, 0xbc0f, 0x20, 0xb0a8, 0xadf9, 0x20,
0xc2dc, 0xac04, 0xac08, 0xb77c, 0xd30c, 0xace0, 0xc2a4, 0x20, 0xc2dc, 0xac04, 0xac10, 0xbe44,
0xc5d0, 0x20, 0xc2dc, 0xac04, 0xc870, 0xc9c0, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc870, 0xc9c0,
0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc870, 0xc9c0, 0xc544, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xae38, 0xbc84, 0xd2b8, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04,
0xadf8, 0xb9ac, 0xb2c8, 0xce58, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc,
0x20, 0xb3d9, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xb3d9,
0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xb3d9, 0xbd80,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20,
0xc11c, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xc11c, 0xbd80,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xc11c, 0xbd80, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xad0c, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc,
0xac04, 0xac78, 0xd504, 0xb9cc, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd558, 0xc640, 0xc774, 0x20,
0xc54c, 0xb958, 0xc0e8, 0x20, 0xc2dc, 0xac04, 0xd558, 0xc640, 0xc774, 0x20, 0xc54c, 0xb958,
0xc0e8, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd558, 0xc640, 0xc774, 0x20, 0xc54c, 0xb958, 0xc0e8,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd64d, 0xcf69, 0x20, 0xc2dc, 0xac04,
0xd64d, 0xcf69, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd64d, 0xcf69, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xd638, 0xbe0c, 0xb4dc, 0x20, 0xc2dc, 0xac04, 0xd638, 0xbe0c, 0xb4dc,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xd638, 0xbe0c, 0xb4dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc778, 0xb3c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc778, 0xb3c4, 0xc591, 0x20,
0xc2dc, 0xac04, 0xc778, 0xb3c4, 0xcc28, 0xc774, 0xb098, 0x20, 0xc2dc, 0xac04, 0xc911, 0xbd80,
0x20, 0xc778, 0xb3c4, 0xb124, 0xc2dc, 0xc544, 0x20, 0xc2dc, 0xac04, 0xb3d9, 0xbd80, 0x20,
0xc778, 0xb3c4, 0xb124, 0xc2dc, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xbd80, 0x20, 0xc778,
0xb3c4, 0xb124, 0xc2dc, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc774, 0xb780, 0x20, 0xc2dc, 0xac04,
0xc774, 0xb780, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xb780, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc774, 0xb974, 0xcfe0, 0xce20, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xc774,
0xb974, 0xcfe0, 0xce20, 0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xb974, 0xcfe0, 0xce20,
0xd06c, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xc2a4, 0xb77c, 0xc5d8,
0x20, 0xc2dc, 0xac04, 0xc774, 0xc2a4, 0xb77c, 0xc5d8, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774,
0xc2a4, 0xb77c, 0xc5d8, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc77c, 0xbcf8,
0x20, 0xc2dc, 0xac04, 0xc77c, 0xbcf8, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc77c, 0xbcf8, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd398, 0xd2b8, 0xb85c, 0xd30c, 0xbe14, 0xb86d,
0xc2a4, 0xd06c, 0x2d, 0xce84, 0xcc28, 0xce20, 0xd0a4, 0x20, 0xc2dc, 0xac04, 0xd398, 0xd2b8,
0xb85c, 0xd30c, 0xbe14, 0xb86d, 0xc2a4, 0xd06c, 0x2d, 0xce84, 0xcc28, 0xce20, 0xd0a4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xd398, 0xd2b8, 0xb85c, 0xd30c, 0xbe14, 0xb86d, 0xc2a4, 0xd06c, 0x2d,
0xce84, 0xcc28, 0xce20, 0xd0a4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xce74,
0xc790, 0xd750, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xb3d9, 0xbd80, 0x20, 0xce74, 0xc790,
0xd750, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xbd80, 0x20, 0xce74, 0xc790, 0xd750,
0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xb300, 0xd55c, 0xbbfc, 0xad6d, 0x20, 0xc2dc, 0xac04,
0xb300, 0xd55c, 0xbbfc, 0xad6d, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb300, 0xd55c, 0xbbfc, 0xad6d,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcf54, 0xc2a4, 0xb77c, 0xc5d0, 0xc12c,
0x20, 0xc2dc, 0xac04, 0xd06c, 0xb77c, 0xc2a4, 0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c, 0x20,
0xc2dc, 0xac04, 0xd06c, 0xb77c, 0xc2a4, 0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xd06c, 0xb77c, 0xc2a4, 0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd0a4, 0xb974, 0xae30, 0xc2a4, 0xc2a4, 0xd0c4, 0x20,
0xc2dc, 0xac04, 0xb791, 0xce74, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xb77c, 0xc778,
0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xb85c, 0xb4dc, 0x20, 0xd558, 0xc6b0, 0x20,
0xc2dc, 0xac04, 0xb85c, 0xb4dc, 0x20, 0xd558, 0xc6b0, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb85c,
0xb4dc, 0x20, 0xd558, 0xc6b0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb9c8,
0xce74, 0xc624, 0x20, 0xc2dc, 0xac04, 0xb9c8, 0xce74, 0xc624, 0x20, 0xd45c, 0xc900, 0x20,
0xc2dc, 0xac04, 0xb9c8, 0xce74, 0xc624, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xb9c8, 0xac00, 0xb2e8, 0x20, 0xc2dc, 0xac04, 0xb9c8, 0xac00, 0xb2e8, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xb9c8, 0xac00, 0xb2e8, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb9d0,
0xb808, 0xc774, 0xc2dc, 0xc544, 0x20, 0xc2dc, 0xac04, 0xbab0, 0xb514, 0xbe0c, 0x20, 0xc2dc,
0xac04, 0xb9c8, 0xb974, 0xd0a4, 0xc988, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xb9c8,
0xc15c, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xbaa8, 0xb9ac, 0xc154, 0xc2a4, 0x20,
0xc2dc, 0xac04, 0xbaa8, 0xb9ac, 0xc154, 0xc2a4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8, 0xb9ac,
0xc154, 0xc2a4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8, 0xc2a8, 0x20,
0xc2dc, 0xac04, 0xba55, 0xc2dc, 0xcf54, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xc2dc, 0xac04,
0xba55, 0xc2dc, 0xcf54, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xba55,
0xc2dc, 0xcf54, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc6b8, 0xb780, 0xbc14, 0xd1a0, 0xb974, 0x20, 0xc2dc, 0xac04, 0xc6b8, 0xb780, 0xbc14,
0xd1a0, 0xb974, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc6b8, 0xb780, 0xbc14, 0xd1a0, 0xb974, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8, 0xc2a4, 0xd06c, 0xbc14, 0x20, 0xc2dc,
0xac04, 0xbaa8, 0xc2a4, 0xd06c, 0xbc14, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8, 0xc2a4, 0xd06c,
0xbc14, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0xc580, 0xb9c8, 0x20,
0xc2dc, 0xac04, 0xb098, 0xc6b0, 0xb8e8, 0x20, 0xc2dc, 0xac04, 0xb124, 0xd314, 0x20, 0xc2dc,
0xac04, 0xb274, 0xce7c, 0xb808, 0xb3c4, 0xb2c8, 0xc544, 0x20, 0xc2dc, 0xac04, 0xb274, 0xce7c,
0xb808, 0xb3c4, 0xb2c8, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb274, 0xce7c, 0xb808, 0xb3c4,
0xb2c8, 0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb274, 0xc9c8, 0xb79c,
0xb4dc, 0x20, 0xc2dc, 0xac04, 0xb274, 0xc9c8, 0xb79c, 0xb4dc, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xb274, 0xc9c8, 0xb79c, 0xb4dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb274,
0xd380, 0xb4e4, 0xb79c, 0xb4dc, 0x20, 0xc2dc, 0xac04, 0xb274, 0xd380, 0xb4e4, 0xb79c, 0xb4dc,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xb274, 0xd380, 0xb4e4, 0xb79c, 0xb4dc, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xb2c8, 0xc6b0, 0xc5d0, 0x20, 0xc2dc, 0xac04, 0xb178, 0xd37d,
0xc12c, 0x20, 0xc2dc, 0xac04, 0xb178, 0xd37d, 0xc12c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb178,
0xd37d, 0xc12c, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd398, 0xb974, 0xb09c,
0xb3c4, 0x20, 0xb370, 0x20, 0xb178, 0xb85c, 0xb0d0, 0x20, 0xc2dc, 0xac04, 0xd398, 0xb974,
0xb09c, 0xb3c4, 0x20, 0xb370, 0x20, 0xb178, 0xb85c, 0xb0d0, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xd398, 0xb974, 0xb09c, 0xb3c4, 0x20, 0xb370, 0x20, 0xb178, 0xb85c, 0xb0d0, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbd81, 0xb9c8, 0xb9ac, 0xc544, 0xb098, 0x20, 0xc81c,
0xb3c4, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xb178, 0xbcf4, 0xc2dc, 0xbe44, 0xb974,
0xc2a4, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xb178, 0xbcf4, 0xc2dc, 0xbe44, 0xb974, 0xc2a4, 0xd06c,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xb178, 0xbcf4, 0xc2dc, 0xbe44, 0xb974, 0xc2a4, 0xd06c, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc634, 0xc2a4, 0xd06c, 0x20, 0xc2dc, 0xac04,
0xc634, 0xc2a4, 0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc634, 0xc2a4, 0xd06c, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd30c, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04,
0xd30c, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd30c, 0xd0a4, 0xc2a4, 0xd0c4,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd314, 0xb77c, 0xc6b0, 0x20, 0xc2dc,
0xac04, 0xd30c, 0xd478, 0xc544, 0xb274, 0xae30, 0xb2c8, 0x20, 0xc2dc, 0xac04, 0xd30c, 0xb77c,
0xacfc, 0xc774, 0x20, 0xc2dc, 0xac04, 0xd30c, 0xb77c, 0xacfc, 0xc774, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xd30c, 0xb77c, 0xacfc, 0xc774, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xd398, 0xb8e8, 0x20, 0xc2dc, 0xac04, 0xd398, 0xb8e8, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd398,
0xb8e8, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd544, 0xb9ac, 0xd540, 0x20,
0xc2dc, 0xac04, 0xd544, 0xb9ac, 0xd540, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd544, 0xb9ac, 0xd540,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd53c, 0xb2c9, 0xc2a4, 0x20, 0xc81c,
0xb3c4, 0x20, 0xc2dc, 0xac04, 0xc138, 0xc778, 0xd2b8, 0xd53c, 0xc5d0, 0xb974, 0x20, 0xbbf8,
0xd074, 0xb871, 0x20, 0xc2dc, 0xac04, 0xc138, 0xc778, 0xd2b8, 0xd53c, 0xc5d0, 0xb974, 0x20,
0xbbf8, 0xd074, 0xb871, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc138, 0xc778, 0xd2b8, 0xd53c, 0xc5d0,
0xb974, 0x20, 0xbbf8, 0xd074, 0xb871, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xd54f, 0xcf00, 0xc5b8, 0x20, 0xc2dc, 0xac04, 0xd3ec, 0xb098, 0xd398, 0x20, 0xc2dc, 0xac04,
0xd0a4, 0xc9c8, 0xb85c, 0xb974, 0xb2e4, 0x20, 0xc2dc, 0xac04, 0xd0a4, 0xc9c8, 0xb85c, 0xb974,
0xb2e4, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xd0a4, 0xc9c8, 0xb85c, 0xb974, 0xb2e4,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb808, 0xc704, 0xb2c8, 0xc639, 0x20,
0xc2dc, 0xac04, 0xb85c, 0xb370, 0xb77c, 0x20, 0xc2dc, 0xac04, 0xc0ac, 0xd560, 0xb9b0, 0x20,
0xc2dc, 0xac04, 0xc0ac, 0xd560, 0xb9b0, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xd560, 0xb9b0,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xb9c8, 0xb77c, 0x20, 0xc2dc,
0xac04, 0xc0ac, 0xb9c8, 0xb77c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xb9c8, 0xb77c, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xbaa8, 0xc544, 0x20, 0xc2dc, 0xac04,
0xc0ac, 0xbaa8, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xbaa8, 0xc544, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc138, 0xc774, 0xc178, 0x20, 0xc2dc, 0xac04, 0xc2f1,
0xac00, 0xd3ec, 0xb974, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc194, 0xb85c, 0xbaac, 0x20, 0xc81c,
0xb3c4, 0x20, 0xc2dc, 0xac04, 0xc0ac, 0xc6b0, 0xc2a4, 0x20, 0xc870, 0xc9c0, 0xc544, 0x20,
0xc2dc, 0xac04, 0xc218, 0xb9ac, 0xb0a8, 0x20, 0xc2dc, 0xac04, 0xc1fc, 0xc640, 0x20, 0xc2dc,
0xac04, 0xd0c0, 0xd788, 0xd2f0, 0x20, 0xc2dc, 0xac04, 0xb300, 0xb9cc, 0x20, 0xc2dc, 0xac04,
0xb300, 0xb9cc, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb300, 0xb9cc, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xd0c0, 0xc9c0, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xd1a0,
0xcf08, 0xb77c, 0xc6b0, 0x20, 0xc2dc, 0xac04, 0xd1b5, 0xac00, 0x20, 0xc2dc, 0xac04, 0xd1b5,
0xac00, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd1b5, 0xac00, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xcd94, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xd22c, 0xb974, 0xd06c, 0xba54, 0xb2c8,
0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xd22c, 0xb974, 0xd06c, 0xba54, 0xb2c8, 0xc2a4, 0xd0c4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xd22c, 0xb974, 0xd06c, 0xba54, 0xb2c8, 0xc2a4, 0xd0c4, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd22c, 0xbc1c, 0xb8e8, 0x20, 0xc2dc, 0xac04,
0xc6b0, 0xb8e8, 0xacfc, 0xc774, 0x20, 0xc2dc, 0xac04, 0xc6b0, 0xb8e8, 0xacfc, 0xc774, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc6b0, 0xb8e8, 0xacfc, 0xc774, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc6b0, 0xc988, 0xbca0, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xc6b0,
0xc988, 0xbca0, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc6b0, 0xc988, 0xbca0,
0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbc14, 0xb204,
0xc544, 0xd22c, 0x20, 0xc2dc, 0xac04, 0xbc14, 0xb204, 0xc544, 0xd22c, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xbc14, 0xb204, 0xc544, 0xd22c, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xbca0, 0xb124, 0xc218, 0xc5d8, 0xb77c, 0x20, 0xc2dc, 0xac04, 0xbe14, 0xb77c, 0xb514, 0xbcf4,
0xc2a4, 0xd1a0, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xbe14, 0xb77c, 0xb514, 0xbcf4, 0xc2a4, 0xd1a0,
0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbe14, 0xb77c, 0xb514, 0xbcf4, 0xc2a4, 0xd1a0, 0xd06c,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbcfc, 0xace0, 0xadf8, 0xb77c, 0xb4dc,
0x20, 0xc2dc, 0xac04, 0xbcfc, 0xace0, 0xadf8, 0xb77c, 0xb4dc, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xbcfc, 0xace0, 0xadf8, 0xb77c, 0xb4dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xbcf4, 0xc2a4, 0xd1a1, 0x20, 0xc2dc, 0xac04, 0xc6e8, 0xc774, 0xd06c, 0xc12c, 0x20, 0xc2dc,
0xac04, 0xc6d4, 0xb9ac, 0xc2a4, 0xd478, 0xd22c, 0xb098, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc,
0xac04, 0xc57c, 0xcfe0, 0xce20, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xc57c, 0xcfe0, 0xce20, 0xd06c,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc57c, 0xcfe0, 0xce20, 0xd06c, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc608, 0xce74, 0xd14c, 0xb9b0, 0xbd80, 0xb974, 0xd06c, 0x20, 0xc2dc,
0xac04, 0xc608, 0xce74, 0xd14c, 0xb9b0, 0xbd80, 0xb974, 0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc608, 0xce74, 0xd14c, 0xb9b0, 0xbd80, 0xb974, 0xd06c, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc720, 0xcf58, 0x20, 0xc2dc, 0xac04, 0xc870, 0xc120, 0x20, 0xc2dc, 0xac04,
0xc870, 0xc120, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc870, 0xc120, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x66, 0x78,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x41, 0x66, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x4e,
0x61, 0x76, 0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x66,
0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69,
0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x66, 0x72, 0xee,
0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x15f, 0xfb, 0x72, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x41, 0x66, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61,
0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x41, 0x66, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a,
0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x66, 0x72, 0xee, 0x6b,
0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x61, 0x76, 0x65, 0x6e,
0x64, 0xee, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b,
0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x4e, 0x61, 0x76, 0x65, 0x6e, 0x64, 0xee, 0x20, 0x79, 0x61,
0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42,
0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0x65,
0x6e, 0x64, 0xee, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee,
0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74,
0x20, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61,
0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x20, 0x79, 0x61, 0x20, 0x41,
0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b,
0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c,
0x61, 0x74, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b,
0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0xc7, 0xee, 0x79, 0x61, 0x79, 0xee, 0x20, 0x79, 0x61,
0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42,
0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xc7, 0xee, 0x79,
0x61, 0x79, 0xee, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee,
0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0xc7, 0xee, 0x79, 0x61, 0x79, 0xee, 0x20, 0x79, 0x61, 0x20,
0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61,
0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x61, 0x73,
0xee, 0x66, 0xee, 0x6b, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65,
0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x73, 0xee, 0x66, 0xee, 0x6b,
0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61,
0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x50, 0x61, 0x73, 0xee, 0x66, 0xee, 0x6b, 0xea, 0x20, 0x79, 0x61, 0x20,
0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61,
0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x70, 0x69,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x72, 0x65, 0x62, 0x69, 0x73,
0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x72, 0x65,
0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x45,
0x72, 0x65, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0xee, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0xee,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74,
0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x72, 0x6a,
0x61, 0x6e, 0x74, 0xee, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76,
0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74,
0xee, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0xee, 0x6e, 0x61, 0x20,
0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x45, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0xee, 0x6b, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0xee, 0x6b, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0xee, 0x6b, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x79, 0x61, 0x20,
0x4e, 0x61, 0x76, 0xee, 0x6e, 0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61,
0x76, 0xee, 0x6e, 0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x52, 0x6f,
0x6a, 0x61, 0x76, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e,
0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x77, 0x69,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f,
0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41,
0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0x61, 0x20,
0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0x61,
0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61,
0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x79, 0x63, 0x61,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x65, 0x79, 0x63, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x65, 0x79, 0x63, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x42, 0x65, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x42, 0x65, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x42, 0x65, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x15f, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x42, 0xfb, 0x74,
0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x42, 0x6f, 0x6c,
0xee, 0x76, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x42, 0x72, 0x61, 0x73, 0xee, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0xee, 0x6c, 0x79, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0xee, 0x6c, 0x79,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x42, 0x72, 0xfb,
0x6e, 0x65, 0x79, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x65, 0x6c,
0x61, 0x6d, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x43, 0x61, 0x70,
0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x15e, 0xee, 0x6c, 0xee, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x15e, 0xee, 0x6c, 0xee, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x15e, 0xee, 0x6c, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0xc7, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0xc7, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xc7, 0xee, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61,
0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e,
0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0xee, 0x79, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0xee,
0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0xee, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76,
0xea, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x4e, 0xee, 0x76, 0x68, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x75,
0x62, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x75, 0x62,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x44, 0x61, 0x76, 0xee, 0x73,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0xee, 0x6d, 0xfb, 0x72,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x50,
0x61, 0x73, 0x6b, 0x61, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x50, 0x61, 0x73,
0x6b, 0x61, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47,
0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x6c,
0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x6b,
0x77, 0x61, 0x64, 0x6f, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76,
0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70,
0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61,
0x76, 0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x77, 0x72,
0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c,
0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70,
0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61,
0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20,
0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x20, 0x61, 0x20, 0x50,
0xea, 0x15f, 0x74, 0x69, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45,
0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61,
0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70,
0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47,
0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x69, 0x72,
0x61, 0x76, 0xea, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea,
0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x46, 0xee, 0x6a, 0xee, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x46, 0xee, 0x6a, 0xee, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x46, 0xee, 0x6a, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0x75, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x79, 0x61, 0x20,
0x46, 0x72, 0x61, 0x6e, 0x73, 0xee, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0xee, 0x6b, 0x61, 0x20, 0xfb,
0x20, 0x42, 0x61, 0x15f, 0xfb, 0x72, 0xea, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x73, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x47, 0x75, 0x72, 0x63, 0x69, 0x73, 0x74, 0x61,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x75, 0x72, 0x63, 0x69,
0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47, 0x75, 0x72,
0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e,
0x61, 0x76, 0xee, 0x6e, 0xee, 0x20, 0x79, 0x61, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x47, 0x72, 0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x47, 0x72, 0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52, 0x6f,
0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47, 0x72,
0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68,
0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x72,
0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61,
0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x72, 0xee, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x47, 0x72, 0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x4b, 0x65, 0x6e, 0x64, 0x61, 0x76, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x48, 0x6f, 0x76, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x48, 0x6f,
0x76, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x48, 0x69, 0x6e, 0x64,
0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x4f, 0x6b, 0x79, 0x61, 0x6e, 0xfb, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6e,
0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x69, 0x6e, 0x64,
0x69, 0xe7, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61, 0x79, 0x61, 0x20, 0x4e,
0x61, 0x76, 0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f,
0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x45, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61, 0x79, 0x61, 0x20,
0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0xce, 0x72, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xce, 0x72,
0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xce, 0x72, 0x61, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0xce, 0x73, 0x72,
0x61, 0xee, 0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xce, 0x73, 0x72,
0x61, 0xee, 0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xce, 0x73, 0x72, 0x61,
0xee, 0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4a, 0x61, 0x70,
0x6f, 0x6e, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4a,
0x61, 0x70, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0x15f, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x51, 0x61, 0x7a, 0x61, 0x78, 0x69, 0x73, 0x74, 0x61,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x51, 0x61, 0x7a, 0x61,
0x78, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68,
0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x51, 0x61,
0x7a, 0x61, 0x78, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x52, 0x6f,
0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x6f,
0x72, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x6f, 0x72,
0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x51, 0x69, 0x72, 0x78, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea,
0x6e, 0x20, 0x4c, 0xee, 0x6e, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x79, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64,
0xee, 0x76, 0x61, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x4d, 0x61,
0x72, 0x15f, 0x61, 0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x73,
0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x61, 0x73,
0xee, 0x66, 0xee, 0x6b, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0xee, 0x6b,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x73, 0xee,
0x66, 0xee, 0x6b, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0xee, 0x6b, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x50, 0x61, 0x73, 0xee, 0x66, 0xee,
0x6b, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0xee, 0x6b, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0xdb, 0x6c, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xdb, 0x6c, 0x61,
0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xdb,
0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e,
0x61, 0xfb, 0x72, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0x61, 0x20,
0x4e, 0xfb, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0xfb, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0x61,
0x20, 0x4e, 0xfb, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x79, 0x61, 0x20, 0x4e, 0xfb, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x79, 0x61, 0x20,
0x4e, 0xfb, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x79, 0x61, 0x20, 0x4e, 0xfb, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47, 0x69, 0x72,
0x61, 0x76, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x47, 0xee, 0x6e, 0x65, 0x79, 0x61, 0x20, 0x4e,
0xfb, 0x20, 0x79, 0x61, 0x20, 0x50, 0x61, 0x70, 0xfb, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0xfb,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0xfb, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0xfb, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x50, 0x65, 0x72, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50,
0x65, 0x72, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x50, 0x65, 0x72,
0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x46, 0xee, 0x6c,
0xee, 0x70, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x46, 0xee,
0x6c, 0xee, 0x70, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x46, 0xee,
0x6c, 0xee, 0x70, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0xee, 0x6b, 0x73, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0xfb,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0xfb, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0xfb, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x52, 0xe9,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x65, 0x79, 0x15f, 0x65, 0x6c,
0x65, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x53, 0xee, 0x6e, 0x67,
0x61, 0x70, 0xfb, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47,
0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x15f, 0xfb, 0x72,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x75, 0x72, 0xee, 0x6e, 0x61,
0x6d, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x61, 0x68,
0xee, 0x74, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54,
0x61, 0xee, 0x70, 0x65, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x54, 0x61, 0xee, 0x70, 0x65, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x54, 0x61, 0xee, 0x70, 0x65, 0xee, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x54, 0x61, 0x63, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x43, 0x68,
0x75, 0x75, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x69,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x54, 0x69, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x54, 0x69, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x54, 0xfb, 0x76, 0x61, 0x6c, 0xfb, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0xdb, 0x72, 0xfb, 0x67, 0xfb, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xdb, 0x72, 0xfb, 0x67, 0xfb,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xdb, 0x72, 0xfb, 0x67, 0xfb,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4f, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x4f, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x4f, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x56, 0x61, 0x6e,
0xfb, 0x61, 0x74, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x56,
0x61, 0x6e, 0xfb, 0x61, 0x74, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x56, 0x61, 0x6e, 0xfb, 0x61, 0x74, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0xfb, 0x65, 0x6c, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72,
0x61, 0x76, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0xfb,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0xea, 0x410, 0x444, 0x433,
0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443,
0x43a, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x410,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x422, 0x4af, 0x448, 0x442, 0x4af, 0x43a, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442,
0x44b, 0x448, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411,
0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434,
0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x431, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x431, 0x43e, 0x440, 0x431,
0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434,
0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x431, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x2c, 0x20, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x447, 0x44b,
0x433, 0x44b, 0x448, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x447, 0x44b,
0x433, 0x44b, 0x448, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x442, 0x43e,
0x43e, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af,
0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x2c, 0x20, 0x442, 0x43e, 0x43e, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434,
0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x442, 0x43e, 0x43e, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b,
0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20,
0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b, 0x43d,
0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af,
0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410,
0x43f, 0x438, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x430,
0x431, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410,
0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x43c, 0x435,
0x43d, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x438, 0x43a, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e,
0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431,
0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440,
0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x431, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e,
0x440, 0x434, 0x443, 0x43a, 0x20, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e,
0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x447, 0x44b, 0x433, 0x44b,
0x448, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x447, 0x44b, 0x433, 0x44b, 0x448,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x437, 0x435,
0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x435,
0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x437,
0x43e, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410,
0x437, 0x43e, 0x440, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b,
0x430, 0x434, 0x435, 0x448, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x43e, 0x43b, 0x438,
0x432, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438,
0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x430,
0x440, 0x443, 0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x430, 0x43f, 0x435, 0x20, 0x412, 0x435,
0x440, 0x434, 0x435, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41a, 0x430, 0x43f, 0x435, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x41a, 0x430, 0x43f, 0x435, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x430, 0x442, 0x430, 0x43c,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x438, 0x43b,
0x438, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x438,
0x43b, 0x438, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x44b, 0x442,
0x430, 0x439, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x438, 0x441, 0x43c, 0x430, 0x441, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c,
0x431, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a,
0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x443,
0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x443,
0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b,
0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x443, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x436, 0x430, 0x440, 0x44b,
0x43c, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41a, 0x443, 0x431, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x414, 0x44d, 0x432, 0x438, 0x441,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x414, 0x44e, 0x43c,
0x43e, 0x43d, 0x2d, 0x434, 0x2d, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448,
0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x418, 0x441, 0x442, 0x435, 0x440, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x418, 0x441, 0x442, 0x435, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x441, 0x442, 0x435, 0x440, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42d, 0x43a,
0x443, 0x430, 0x434, 0x43e, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x43e, 0x440,
0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x430, 0x43b, 0x438, 0x43d, 0x438,
0x43d, 0x433, 0x440, 0x430, 0x434, 0x20, 0x436, 0x430, 0x43d, 0x430, 0x20, 0x41c,
0x438, 0x43d, 0x441, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x43e, 0x43b, 0x43a,
0x43b, 0x435, 0x43d, 0x434, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440,
0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435,
0x43d, 0x434, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d,
0x44b, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x438, 0x436, 0x438, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x438, 0x436, 0x438, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x424, 0x438, 0x436, 0x438, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x440, 0x430, 0x43d, 0x446,
0x443, 0x437, 0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443,
0x437, 0x20, 0x422, 0x4af, 0x448, 0x442, 0x4af, 0x433, 0x4af, 0x20, 0x436, 0x430,
0x43d, 0x430, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x43a,
0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x430,
0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x435, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438,
0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x440,
0x443, 0x437, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x440, 0x438, 0x43d, 0x432,
0x438, 0x447, 0x20, 0x431, 0x43e, 0x44e, 0x43d, 0x447, 0x430, 0x20, 0x43e, 0x440,
0x442, 0x43e, 0x447, 0x43e, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x427,
0x44b, 0x433, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x443, 0x43b, 0x443, 0x4a3, 0x434, 0x443,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432,
0x430, 0x439, 0x438, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439, 0x438,
0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x430, 0x432,
0x430, 0x439, 0x438, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x425,
0x43e, 0x432, 0x434, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x425, 0x43e, 0x432, 0x434, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x418, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x418, 0x43d, 0x434, 0x438, 0x20, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418, 0x43d,
0x434, 0x43e, 0x43a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443,
0x43a, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b,
0x448, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b,
0x448, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418, 0x440, 0x430, 0x43d,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418, 0x440, 0x430,
0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x43a, 0x4af, 0x43d, 0x434,
0x4af, 0x437, 0x433, 0x4af, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418,
0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418,
0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x43f, 0x43e,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x416, 0x430,
0x43f, 0x43e, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x43f, 0x43e, 0x43d, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41a, 0x430, 0x437, 0x430, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448,
0x20, 0x41a, 0x430, 0x437, 0x430, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x41a, 0x430, 0x437, 0x430, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x440,
0x435, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a,
0x44b, 0x440, 0x433, 0x44b, 0x437, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41b, 0x430, 0x439, 0x43d, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x20,
0x425, 0x430, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41b, 0x43e,
0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430,
0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x43b, 0x430,
0x439, 0x437, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x437, 0x430,
0x441, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430,
0x440, 0x448, 0x430, 0x43b, 0x43b, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x43a, 0x438, 0x439, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438,
0x439, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438,
0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c,
0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447,
0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447, 0x20, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423, 0x43b, 0x430, 0x43d, 0x20,
0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c,
0x439, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20,
0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x416,
0x430, 0x4a3, 0x44b, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438,
0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x417, 0x435,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x44e, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x44d, 0x43d, 0x434, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b,
0x44d, 0x43d, 0x434, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x44d, 0x43d, 0x434, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x438, 0x443, 0x44d,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435,
0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x43e, 0x432,
0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431,
0x438, 0x440, 0x441, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41e, 0x43c, 0x441, 0x43a,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41e, 0x43c, 0x441,
0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41f, 0x430, 0x43b, 0x430, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x2d, 0x416, 0x430, 0x4a3, 0x44b,
0x20, 0x413, 0x432, 0x438, 0x43d, 0x435, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x439, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x439, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434,
0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x44c, 0x435, 0x440, 0x20,
0x436, 0x430, 0x43d, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x435, 0x43d,
0x20, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x436, 0x430, 0x43d, 0x430, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x435, 0x43d,
0x20, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x436, 0x430, 0x43d, 0x430, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x438, 0x442,
0x43a, 0x44d, 0x440, 0x43d, 0x433, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x445, 0x435, 0x43d, 0x44c, 0x44f, 0x43d,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x420, 0x435, 0x44e, 0x43d,
0x438, 0x43e, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x435, 0x439, 0x448,
0x435, 0x43b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421,
0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x448, 0x442,
0x4af, 0x43a, 0x20, 0x416, 0x43e, 0x440, 0x436, 0x438, 0x44f, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430,
0x43c, 0x435, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421,
0x430, 0x43e, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x422, 0x430, 0x438, 0x442, 0x438, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x436,
0x438, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x43e, 0x43d,
0x433, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x427, 0x443, 0x443, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x422, 0x4af, 0x440, 0x43a, 0x43c, 0x4e9, 0x43d, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af,
0x440, 0x43a, 0x43c, 0x4e9, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422,
0x4af, 0x440, 0x43a, 0x43c, 0x4e9, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423, 0x440, 0x443, 0x433,
0x432, 0x430, 0x439, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430,
0x439, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x4e8, 0x437, 0x431, 0x435, 0x43a, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x4e8, 0x437,
0x431, 0x435, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4e8, 0x437, 0x431,
0x435, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x430, 0x43d,
0x443, 0x430, 0x442, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x435, 0x43d, 0x435,
0x441, 0x443, 0x44d, 0x43b, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e,
0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412,
0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433,
0x440, 0x430, 0x434, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423, 0x435, 0x439,
0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423, 0x43e,
0x43b, 0x438, 0x441, 0x20, 0x436, 0x430, 0x43d, 0x430, 0x20, 0x424, 0x443, 0x442,
0x443, 0x43d, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x415, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x42e, 0x43a,
0x43e, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0xec0,
0xea7, 0xea5, 0xeb2, 0xe82, 0xead, 0xe87, 0xead, 0xeb2, 0xec0, 0xe81, 0xea3, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe82,
0xead, 0xe87, 0xead, 0xeb2, 0xec0, 0xe81, 0xea3, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5,
0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe82, 0xead, 0xe87, 0xead, 0xeb2,
0xec0, 0xe81, 0xea3, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xead, 0xeb1, 0xe9f, 0xe81,
0xeb2, 0xe99, 0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xead, 0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2, 0x200b, 0xe81,
0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f,
0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1,
0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead,
0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2, 0x200b, 0xec3, 0xe95, 0xec9,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4,
0x200b, 0xe81, 0xeb2, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b,
0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f,
0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1,
0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xead, 0xeb2,
0x200b, 0xe9f, 0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0,
0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead,
0xeb0, 0xea5, 0xeb1, 0xeaa, 0xe81, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2,
0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb0, 0xea5, 0xeb1, 0xeaa, 0xe81,
0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99,
0xead, 0xeb0, 0xea5, 0xeb1, 0xeaa, 0xe81, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead,
0xecd, 0xea1, 0xeb2, 0xe95, 0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xecd, 0xea1, 0xeb2, 0xe95, 0xeb5, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xead,
0xecd, 0xea1, 0xeb2, 0xe95, 0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xea1,
0xec0, 0xe82, 0xe94, 0xead, 0xeb2, 0xec0, 0xea1, 0xe8a, 0xead, 0xe99, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96,
0xeb2, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xea1, 0x200b, 0xe8a, 0xead, 0xe99, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae,
0xec9, 0xead, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xea1, 0x200b, 0xe8a, 0xead, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7,
0xeb1, 0xe99, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xec0,
0xea7, 0xeb1, 0xe99, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2,
0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99,
0xead, 0xead, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7,
0xeb1, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead, 0xe81, 0xec0,
0xea7, 0xea5, 0xeb2, 0xec1, 0xe96, 0xe9a, 0xe9e, 0xeb9, 0xec0, 0xe82, 0xebb, 0xeb2,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xec1, 0xe96, 0xe9a, 0xe9e, 0xeb9, 0xec0, 0xe82, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5,
0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xec1, 0xe96, 0xe9a, 0xe9e,
0xeb9, 0xec0, 0xe82, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec1, 0xe9b, 0xe8a,
0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95,
0xeb0, 0xe96, 0xeb2, 0xe99, 0xec1, 0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0,
0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xec1, 0xe9b,
0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xead, 0xec0,
0xe9e, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xec0, 0xead, 0xec0, 0xe9e, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81,
0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xeb2, 0xec0, 0xe9e, 0xe8d, 0xec0,
0xea7, 0xea5, 0xeb2, 0xead, 0xeb1, 0xe94, 0xe95, 0xeb2, 0xead, 0xeb9, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb1,
0xe94, 0xe95, 0xeb2, 0xead, 0xeb9, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94,
0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb1, 0xe94, 0xe95, 0xeb2, 0xead, 0xeb9,
0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb1, 0xe94, 0xec2, 0xe95, 0xec0, 0xe9a, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead,
0xeb1, 0xe94, 0xec2, 0xe95, 0xec0, 0xe9a, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0,
0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb1, 0xe94, 0xec2, 0xe95, 0xec0,
0xe9a, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xea3, 0xeb2,
0x200b, 0xe9a, 0xebd, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2,
0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb2, 0x200b, 0xea3,
0xeb2, 0x200b, 0xe9a, 0xebd, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87,
0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xeb2, 0xea3, 0xeb2, 0xe9a, 0xebd, 0xe99, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0x200b, 0xe97,
0xeb4, 0x200b, 0xe99, 0xeb2, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb2, 0x200b,
0xec0, 0xe88, 0xe99, 0x200b, 0xe97, 0xeb4, 0x200b, 0xe99, 0xeb2, 0x200b, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9,
0xead, 0xe99, 0x200b, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0xe99, 0x200b, 0xe97, 0xeb4,
0x200b, 0xe99, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xea7, 0x200b,
0xeaa, 0x200b, 0xec0, 0xe97, 0xeb4, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0xe99,
0x200b, 0xe97, 0xeb4, 0x200b, 0xe99, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7,
0x200b, 0xeaa, 0x200b, 0xec0, 0xe97, 0xeb4, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88,
0xe99, 0x200b, 0xe97, 0xeb4, 0x200b, 0xe99, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xec0,
0xea7, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe97, 0xeb4, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0,
0xe88, 0xe99, 0x200b, 0xe97, 0xeb4, 0x200b, 0xe99, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2,
0xead, 0xeb2, 0xec0, 0xea1, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb2, 0xec0, 0xea1, 0xec0,
0xe99, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9,
0xead, 0xe99, 0xead, 0xeb2, 0xec0, 0xea1, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0xea5,
0xeb2, 0xe82, 0xead, 0xe87, 0xead, 0xeb2, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb4, 0xe81,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xe82, 0xead, 0xe87, 0xead, 0xeb2, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb4, 0xe81, 0xec0,
0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe82, 0xead,
0xe87, 0xead, 0xeb2, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb4, 0xe81, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0xead, 0xead, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xec0,
0xea5, 0xe8d, 0x200b, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xead, 0x200b,
0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xec0, 0xea5, 0xe8d, 0x200b, 0xe81, 0x200b,
0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b,
0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3,
0x200b, 0xec0, 0xea5, 0xe8d, 0x200b, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99,
0x200b, 0xe81, 0xeb2, 0xe87, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99,
0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2,
0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xead, 0xeaa, 0x200b,
0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe81, 0xeb2, 0xe87, 0x200b,
0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd,
0xe87, 0x200b, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd,
0xe99, 0x200b, 0xe81, 0xeb2, 0xe87, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1,
0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xead, 0xead,
0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe95, 0xeb2,
0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0x200b,
0x200b, 0x200b, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd,
0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead,
0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97,
0xec8, 0xebd, 0xe87, 0x200b, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b,
0xea5, 0xebd, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b,
0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xead, 0xeaa,
0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xec0, 0xea5, 0xe8d, 0x200b, 0xe95, 0xeb2, 0x200b,
0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0x200b, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2,
0xe99, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99,
0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8,
0xebd, 0xe87, 0x200b, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5,
0xebd, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95,
0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb1, 0xeaa, 0xec0, 0xe8a, 0xeb5,
0xec4, 0xe9a, 0xe88, 0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb1, 0xeaa, 0xec0, 0xe8a, 0xeb5, 0xec4,
0xe9a, 0xe88, 0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9,
0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb1, 0xeaa, 0xec0, 0xe8a, 0xeb5, 0xec4, 0xe9a,
0xe88, 0xeb1, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b,
0xec2, 0xe8a, 0x200b, 0xec0, 0xea3, 0x200b, 0xeaa, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead,
0xeb2, 0x200b, 0xec2, 0xe8a, 0x200b, 0xec0, 0xea3, 0x200b, 0xeaa, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead,
0xe99, 0xead, 0xeb2, 0x200b, 0xec2, 0xe8a, 0x200b, 0xec0, 0xea3, 0x200b, 0xeaa, 0xec0,
0xea7, 0xea5, 0xeb2, 0x20, 0xe9a, 0xeb1, 0xe87, 0xe81, 0xeb0, 0xea5, 0xeb2, 0xec0,
0xe97, 0xe94, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0x20, 0xe9a, 0xeb1, 0xe87, 0xe81, 0xeb0, 0xea5, 0xeb2, 0xec0, 0xe97,
0xe94, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9,
0xead, 0xe99, 0x20, 0xe9a, 0xeb1, 0xe87, 0xe81, 0xeb0, 0xea5, 0xeb2, 0xec0, 0xe97,
0xe94, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe9e, 0xeb9, 0x200b, 0xe96, 0xeb2,
0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec2, 0xe9a, 0x200b, 0xea5, 0xeb4,
0x200b, 0xec0, 0xea7, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xea1, 0xec0,
0xe82, 0xe94, 0xe9a, 0xea3, 0xeb2, 0xe8a, 0xeb4, 0xec0, 0xea5, 0xe8d, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb2, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe9a,
0xea3, 0xe8a, 0xeb5, 0xec0, 0xea5, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2,
0xea1, 0xec0, 0xe82, 0xe94, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99,
0xe95, 0xeb2, 0xea1, 0xec0, 0xe82, 0xe94, 0xe9a, 0xea3, 0xeb2, 0xe8a, 0xeb5, 0xec0,
0xea5, 0xe8d, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xe9a, 0xea3, 0xeb9, 0x200b,
0xec4, 0xe99, 0xe94, 0xeb2, 0x200b, 0xea3, 0xeb8, 0xeaa, 0x200b, 0xe8a, 0xeb2, 0x200b,
0xea5, 0xeb2, 0xea1, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe84, 0xe9a,
0x200b, 0xec0, 0xea7, 0xeb5, 0xe94, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xec0,
0xe84, 0xe9a, 0x200b, 0xec0, 0xea7, 0xeb5, 0xe94, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b,
0xec0, 0xe84, 0xe9a, 0x200b, 0xec0, 0xea7, 0xeb5, 0xe94, 0xec0, 0xea7, 0xea5, 0xeb2,
0xec0, 0xe84, 0xe8a, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe88, 0xeb2,
0x200b, 0xec2, 0xea1, 0x200b, 0xec2, 0xea3, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xe8a, 0xeb2, 0x200b, 0xe97, 0xeb2, 0xea1, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe8a,
0xeb2, 0x200b, 0xe97, 0xeb2, 0xea1, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95,
0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xe8a, 0xeb2, 0x200b, 0xe97,
0xeb2, 0xea1, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe8a, 0xeb4, 0x200b, 0xea5,
0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xe8a, 0xeb4, 0x200b, 0xea5, 0xeb5, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9,
0xead, 0xe99, 0xe8a, 0xeb4, 0x200b, 0xea5, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xe88, 0xeb5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95,
0xeb0, 0xe96, 0xeb2, 0xe99, 0xe88, 0xeb5, 0xe99, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xe88,
0xeb5, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0,
0x200b, 0xe84, 0x200b, 0xea3, 0xeb4, 0xeaa, 0x200b, 0xea1, 0xeb2, 0xeaa, 0xec0, 0xea7,
0xea5, 0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xec2, 0xe81,
0xec2, 0xe81, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec2, 0xe84, 0xea5, 0xecd, 0xeb2,
0xec0, 0xe9a, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0,
0xe96, 0xeb2, 0xe99, 0xec2, 0xe84, 0xea5, 0xeb3, 0xec0, 0xe9a, 0xe8d, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xec2, 0xe84,
0xea5, 0xecd, 0xeb2, 0xec0, 0xe9a, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeab, 0xea1,
0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xe84, 0xeb8, 0xe81, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xeab, 0xea1, 0xeb9,
0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xe84, 0xeb8, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xec0, 0xe84, 0xeb4, 0xec8, 0xe87, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94,
0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xedd, 0xeb9, 0xec8, 0x200b, 0xec0,
0xe81, 0xeb2, 0xeb0, 0x200b, 0xe84, 0xeb8, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe84,
0xeb4, 0xea7, 0xe9a, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95,
0xeb0, 0xe96, 0xeb2, 0xe99, 0xe82, 0xead, 0xe87, 0xe84, 0xeb4, 0xea7, 0xe9a, 0xeb2,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe84,
0xeb4, 0xea7, 0xe9a, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xe94, 0xea7, 0xeb4,
0xe94, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe94, 0xeb9, 0xea1, 0xead, 0xe87, 0xe94, 0xeb9,
0xea7, 0xeb4, 0xea5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb5, 0xea1, 0xecd, 0xe95,
0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xead, 0xeb5, 0x200b, 0xeaa, 0x200b,
0xec0, 0xe95, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b,
0xead, 0xeb5, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99,
0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xead, 0xeb5, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe95,
0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xead, 0x200b, 0xe81, 0xebb,
0xea7, 0x200b, 0xe94, 0xecd, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea2, 0xeb9,
0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99,
0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0xe81, 0xeb2, 0xe87, 0x200b, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae,
0xec9, 0xead, 0xe99, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe81,
0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2,
0xea3, 0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead,
0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0x200b, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2,
0xea3, 0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead,
0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94,
0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a,
0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3,
0x200b, 0xe9b, 0xebd, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99,
0x200b, 0xead, 0xead, 0xe81, 0x200b, 0xec4, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0,
0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xea2,
0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1,
0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xea2, 0xeb9, 0x200b,
0xec2, 0xea3, 0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b,
0xe95, 0xebb, 0xe81, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xedd, 0xeb9,
0xec8, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc,
0xe81, 0x200b, 0xec1, 0xea5, 0xe99, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xedd, 0xeb9,
0xec8, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc,
0xe81, 0x200b, 0xec1, 0xea5, 0xe99, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xedd, 0xeb9,
0xec8, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc,
0xe81, 0x200b, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe9f, 0xeb4, 0xe88,
0xeb4, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2,
0xe99, 0xe9f, 0xeb4, 0xe88, 0xeb4, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94,
0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe9f, 0xeb4, 0xe88, 0xeb4, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xec0, 0xe9f, 0x200b, 0xea3, 0xe99, 0x200b, 0xe8a, 0xecc, 0x200b,
0xec0, 0xe81, 0xe8d, 0x200b, 0xe99, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe9d, 0xea3,
0xeb1, 0xec8, 0xe87, 0xe95, 0xead, 0xe99, 0xec3, 0xe95, 0xec9, 0x20, 0xec1, 0xea5,
0xeb0, 0x20, 0xec1, 0xead, 0xe99, 0xe95, 0xeb2, 0xe81, 0xe95, 0xeb4, 0xe81, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe81, 0xeb2, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe9b,
0xeb2, 0x200b, 0xe81, 0xead, 0x200b, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec1, 0xe81,
0xea1, 0xec0, 0xe9a, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe88, 0xecd, 0xec0, 0xe88,
0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2,
0xe99, 0xe88, 0xecd, 0xec0, 0xe88, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0,
0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe88, 0xecd, 0xec0, 0xe88, 0xe8d, 0xec0,
0xea7, 0xea5, 0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xe81,
0xeb4, 0xea5, 0xec0, 0xe9a, 0xeb5, 0xe94, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xe81,
0xea3, 0xeb5, 0xe99, 0x200b, 0xea7, 0xeb4, 0x200b, 0xe8a, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead, 0xe81, 0xe82, 0xead, 0xe87,
0xe81, 0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1,
0xe99, 0xead, 0xead, 0xe81, 0xe81, 0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe81,
0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99,
0xead, 0xead, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xea3, 0xeb5, 0xe99, 0xec1,
0xea5, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe95, 0xebb, 0xe81, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe81,
0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99,
0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xead, 0xe99, 0xe97, 0xec8,
0xebd, 0xe87, 0xe81, 0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb2, 0xec0,
0xea7, 0xeb1, 0xe99, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xea7,
0xea1, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe81, 0xeb9, 0x200b, 0xea5, 0x200b,
0xecc, 0xe9f, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe8d, 0xead, 0xeb2, 0xe99,
0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeae, 0xeb2, 0xea7, 0xeb2, 0xe8d, 0x2d, 0xec0,
0xead, 0xea5, 0xeb9, 0xe97, 0xebd, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2,
0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xeae, 0xeb2, 0xea7, 0xeb2, 0xe8d, 0x2d,
0xec0, 0xead, 0xea5, 0xeb9, 0xe97, 0xebd, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95,
0xead, 0xe99, 0xe97, 0xec8, 0xebd, 0xe87, 0xeae, 0xeb2, 0xea7, 0xeb2, 0xe8d, 0x2d,
0xec0, 0xead, 0xea5, 0xeb9, 0xe97, 0xebd, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xeae, 0xead, 0xe87, 0x200b, 0xe81, 0xebb, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99,
0x200b, 0xeae, 0xead, 0xe87, 0x200b, 0xe81, 0xebb, 0xe87, 0x200b, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead,
0xe99, 0x200b, 0xeae, 0xead, 0xe87, 0x200b, 0xe81, 0xebb, 0xe87, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xeae, 0xead, 0xe9a, 0x200b, 0xe94, 0xecc, 0x200b, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96,
0xeb2, 0xe99, 0x200b, 0xeae, 0xead, 0xe9a, 0x200b, 0xe94, 0xecc, 0x200b, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9,
0xead, 0xe99, 0x200b, 0xeae, 0xead, 0xe9a, 0x200b, 0xe94, 0xecc, 0xec0, 0xea7, 0xea5,
0xeb2, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea1, 0xeb0, 0xeab, 0xeb2, 0xeaa, 0xeb0, 0xeab, 0xea1, 0xeb8, 0xe94, 0xead, 0xeb4,
0xe99, 0xec0, 0xe94, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb4, 0xe99, 0xe94,
0xeb9, 0xe88, 0xeb5, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb4,
0xe99, 0x200b, 0xec2, 0xe94, 0x200b, 0xec0, 0xe99, 0x200b, 0xec0, 0xe8a, 0xe8d, 0x200b,
0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb4, 0xe99,
0x200b, 0xec2, 0xe94, 0x200b, 0xec0, 0xe99, 0x200b, 0xec0, 0xe8a, 0xe8d, 0x200b, 0xe95,
0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb4, 0xe99, 0x200b, 0xec2, 0xe94, 0x200b, 0xec0,
0xe99, 0x200b, 0xec0, 0xe8a, 0xe8d, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1,
0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead,
0xeb5, 0x200b, 0xea3, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb5, 0x200b,
0xea3, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99,
0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xead, 0xeb5, 0x200b, 0xea3, 0xeb2, 0x200b,
0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0x200b, 0xeb2, 0xead, 0xeb5, 0x200b, 0xe84, 0xeb8,
0xe94, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb5, 0x200b,
0xe84, 0xeb8, 0xe94, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xead,
0xeb5, 0x200b, 0xe84, 0xeb8, 0xe94, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb4, 0x200b, 0xeaa, 0x200b, 0xea3, 0xeb2, 0x200b,
0xec0, 0xead, 0xea7, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb4, 0x200b, 0xeaa, 0x200b,
0xea3, 0xeb2, 0x200b, 0xec0, 0xead, 0xea7, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2,
0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xeb4, 0xeaa, 0xea3, 0xeb2, 0xec0, 0xead,
0xea7, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe8d, 0xeb5, 0xec8, 0x200b, 0xe9b,
0xeb8, 0xec8, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe8d, 0xeb5, 0xec8, 0x200b,
0xe9b, 0xeb8, 0xec8, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead,
0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xe8d, 0xeb5, 0xec8, 0x200b, 0xe9b,
0xeb8, 0xec8, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe84, 0xeb2, 0xe8a, 0xeb1, 0xe81,
0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe84, 0xeb2,
0x200b, 0xe8a, 0xeb1, 0xe81, 0x200b, 0xeaa, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe95,
0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xe84, 0xeb2, 0x200b, 0xe8a, 0xeb1, 0xe81, 0x200b, 0xeaa,
0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99,
0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xe81, 0xebb, 0xeb2,
0xeab, 0xebc, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xec0, 0xe81, 0xebb, 0xeb2,
0x200b, 0xeab, 0xea5, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead,
0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xec0, 0xe81, 0xebb, 0xeb2, 0x200b,
0xeab, 0xea5, 0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe84, 0xead, 0xeaa, 0xec1, 0xea3,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe84, 0xea3, 0xeb1, 0xeaa, 0x200b, 0xec2,
0xe99, 0x200b, 0xea2, 0xeb2, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2,
0xe99, 0xe84, 0xea3, 0xeb1, 0xeaa, 0x200b, 0xec2, 0xe99, 0x200b, 0xea2, 0xeb2, 0x200b,
0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0,
0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xe84, 0xea3, 0xeb1, 0xeaa,
0x200b, 0xec2, 0xe99, 0x200b, 0xea2, 0xeb2, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0,
0xea7, 0xea5, 0xeb2, 0xec0, 0xe84, 0xe8d, 0xe81, 0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb1, 0xe87, 0xe81, 0xeb2, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xedd, 0xeb9, 0xec8, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b,
0xea5, 0xeb2, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xead, 0xe94,
0x200b, 0xec0, 0xeae, 0xebb, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xea5, 0xead,
0xe94, 0x200b, 0xec0, 0xeae, 0xebb, 0xeb2, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0x200b,
0xeaa, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87,
0x200b, 0xea5, 0xead, 0xe94, 0x200b, 0xec0, 0xeae, 0xebb, 0xeb2, 0x200b, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea1, 0xeb2, 0xec0, 0xe81, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xea1, 0xeb2, 0xec0, 0xe81,
0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9,
0xead, 0xe99, 0xea1, 0xeb2, 0xec0, 0xe81, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2,
0xec0, 0xea1, 0xeb1, 0xe81, 0xe81, 0xeb2, 0xec0, 0xe94, 0xe99, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea1, 0xeb1,
0xe81, 0xe81, 0xeb2, 0xec0, 0xe94, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0,
0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe81, 0xeb2,
0xec0, 0xe94, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0x200b,
0xec0, 0xea5, 0x200b, 0xec0, 0xe8a, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb1,
0xea5, 0xe94, 0xeb5, 0xe9f, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xec0, 0xe84,
0xeb5, 0xe8a, 0xeb1, 0xeaa, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xedd, 0xeb9,
0xec8, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xea1, 0xeb2, 0x200b, 0xec1, 0xe8a,
0xea7, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xea1, 0xebb, 0xeb2, 0x200b,
0xea3, 0xeb4, 0x200b, 0xe97, 0xebd, 0x200b, 0xeaa, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0,
0xea1, 0xebb, 0xeb2, 0x200b, 0xea3, 0xeb4, 0x200b, 0xe97, 0xebd, 0x200b, 0xeaa, 0x200b,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b,
0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xec0, 0xea1, 0xebb, 0xeb2, 0x200b, 0xea3, 0xeb4,
0x200b, 0xe97, 0xebd, 0x200b, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xecd, 0xeaa,
0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec1, 0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4,
0xe81, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe8a, 0xeb4, 0xe81, 0xeb1, 0xe99, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec1, 0xe9b,
0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe8a, 0xeb4, 0xe81,
0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1,
0xe99, 0xec1, 0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0, 0xea1, 0xeb1, 0xe81,
0xe8a, 0xeb4, 0xe81, 0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xead, 0xeb9,
0xea5, 0xeb2, 0xe99, 0xe9a, 0xeb2, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0x20, 0xead, 0xeb9, 0xea5,
0xeb2, 0xe99, 0xe9a, 0xeb2, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5,
0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb9, 0xea5, 0xeb2, 0xe99,
0xe9a, 0xeb2, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1,
0xead, 0x200b, 0xeaa, 0x200b, 0xec2, 0xe84, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xea1, 0xead,
0x200b, 0xeaa, 0x200b, 0xec2, 0xe84, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xea1, 0xead, 0x200b,
0xeaa, 0x200b, 0xec2, 0xe84, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xebd, 0xe99, 0xea1,
0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe99, 0xeb2, 0x200b, 0xead, 0xeb9,
0x200b, 0xea3, 0xeb8, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe99,
0x200b, 0xe9b, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe99, 0xeb4, 0xea7, 0xec1,
0xe84, 0xea5, 0xeb4, 0xec2, 0xe94, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe99, 0xeb4, 0xea7, 0xec1,
0xe84, 0xea5, 0xeb4, 0xec2, 0xe94, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe99, 0xeb4, 0xea7, 0xec1,
0xe84, 0xea5, 0xeb4, 0xec2, 0xe94, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xe99, 0xeb4, 0xea7, 0x200b, 0xe8a, 0xeb5, 0x200b, 0xec1, 0xea5, 0xe99,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0,
0x200b, 0xe96, 0xeb2, 0xe99, 0xe99, 0xeb4, 0xea7, 0x200b, 0xe8a, 0xeb5, 0x200b, 0xec1,
0xea5, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b,
0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xe99, 0xeb4, 0xea7, 0x200b, 0xe8a, 0xeb5, 0x200b,
0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe99, 0xeb4, 0xea7,
0x200b, 0xe9f, 0xeb2, 0xea7, 0x200b, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99,
0x200b, 0xe99, 0xeb4, 0xea7, 0x200b, 0xe9f, 0xeb2, 0xea7, 0x200b, 0xec1, 0xea5, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe99,
0xeb4, 0xea7, 0xe9f, 0xeb2, 0xea7, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe99, 0xeb4, 0xead, 0xeb9, 0xec0, 0xead, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe99, 0xecd, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5,
0xecc, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b,
0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe99,
0xecd, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99,
0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe99, 0xecd, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5,
0xecc, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xe9f, 0xe99, 0xeb1, 0xe99, 0xec2,
0xe94, 0xec0, 0xe94, 0xec2, 0xe99, 0xeae, 0xead, 0xe99, 0xeae, 0xeb2, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe9f,
0xe99, 0xeb1, 0xe99, 0xec2, 0xe94, 0xec0, 0xe94, 0xec2, 0xe99, 0xeae, 0xead, 0xe99,
0xeae, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9,
0xead, 0xe99, 0xec0, 0xe9f, 0xe99, 0xeb1, 0xe99, 0xec2, 0xe94, 0xec0, 0xe94, 0xec2,
0xe99, 0xeae, 0xead, 0xe99, 0xeae, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeab, 0xea1,
0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xea1, 0xeb2, 0xec0, 0xea3, 0xe8d, 0xe99,
0xeb2, 0xec0, 0xeab, 0xe99, 0xeb7, 0xead, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xec2, 0xe99, 0x200b, 0xec2, 0xe9a, 0x200b, 0xe8a, 0xeb4, 0x200b, 0xe9a, 0xeb4, 0x200b,
0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2,
0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec2, 0xe99, 0x200b, 0xec2,
0xe9a, 0x200b, 0xe8a, 0xeb4, 0x200b, 0xe9a, 0xeb4, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b,
0xeae, 0xec9, 0xead, 0xe99, 0xec2, 0xe99, 0x200b, 0xec2, 0xe9a, 0x200b, 0xe8a, 0xeb4,
0x200b, 0xe9a, 0xeb4, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0x200b, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0xead, 0xead, 0xea1, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96,
0xeb2, 0xe99, 0xead, 0xead, 0xea1, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9,
0xead, 0xe99, 0xead, 0xead, 0xea1, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xe81, 0xeb5, 0xeaa, 0xe96, 0xeb2,
0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xe81, 0xeb5, 0xeaa,
0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b,
0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xe81,
0xeb5, 0x200b, 0xeaa, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe9b,
0xeb2, 0xec0, 0xea5, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe9b, 0xeb2, 0xe9b,
0xebb, 0xea7, 0xe81, 0xeb4, 0xe99, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xe9b, 0xeb2, 0x200b, 0xea3, 0xeb2, 0x200b, 0xe81, 0xea7, 0xe8d, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2,
0xe99, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xea3, 0xeb2, 0x200b, 0xe81, 0xea7, 0xe8d, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae,
0xec9, 0xead, 0xe99, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xea3, 0xeb2, 0x200b, 0xe81, 0xea7,
0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe9b, 0x200b, 0xea3, 0xeb9,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0x200b, 0xea1, 0xeb2, 0x200b, 0xe94, 0x200b,
0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe9b, 0x200b, 0xea3, 0xeb9, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae,
0xec9, 0xead, 0xe99, 0x200b, 0xec0, 0xe9b, 0x200b, 0xea3, 0xeb9, 0x200b, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xe9f, 0xeb4, 0x200b, 0xea5, 0xeb4, 0xe9a, 0x200b, 0xe9b,
0xeb4, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b,
0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe9f, 0xeb4, 0x200b, 0xea5, 0xeb4,
0xe9a, 0x200b, 0xe9b, 0xeb4, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xe9f, 0xeb4,
0x200b, 0xea5, 0xeb4, 0xe9a, 0x200b, 0xe9b, 0xeb4, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2,
0xeab, 0xea1, 0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xe9f, 0xeb5, 0xe99, 0xeb4,
0xe81, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xec0, 0xe8a, 0xe99, 0x200b, 0xe9b,
0xeb5, 0x200b, 0xec1, 0xead, 0x20, 0xec1, 0xea5, 0xeb0, 0x200b, 0xea1, 0xeb4, 0x200b,
0xe81, 0xebb, 0xea7, 0x200b, 0xea5, 0xead, 0xe99, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0,
0xe8a, 0xe99, 0x200b, 0xe9b, 0xeb5, 0x200b, 0xec1, 0xead, 0x20, 0xec1, 0xea5, 0xeb0,
0x200b, 0xea1, 0xeb4, 0x200b, 0xe81, 0xebb, 0xea7, 0x200b, 0xea5, 0xead, 0xe99, 0x200b,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd,
0xe87, 0xec0, 0xe8a, 0xe99, 0x200b, 0xe9b, 0xeb5, 0x200b, 0xec1, 0xead, 0x20, 0xec1,
0xea5, 0xeb0, 0x200b, 0xea1, 0xeb4, 0x200b, 0xe81, 0xebb, 0xea7, 0x200b, 0xea5, 0xead,
0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe9e, 0xeb4, 0xe94, 0xec1, 0xe84, 0xea3, 0xecc,
0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec2, 0xe9b, 0xec0, 0xe99, 0xe9a, 0xec0, 0xea7,
0xea5, 0xeb2, 0xe9b, 0xebd, 0xe87, 0xea2, 0xeb2, 0xe87, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe84, 0xeb5, 0xea7, 0xea5, 0xecd, 0xe94, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe84, 0xeb5, 0xea7, 0xea5, 0xecd,
0xe94, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9,
0xead, 0xe99, 0xe84, 0xeb5, 0xea7, 0xea5, 0xecd, 0xe94, 0xeb2, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xec0, 0xea3, 0x200b, 0xead, 0xeb9, 0x200b, 0xe99, 0xeb4, 0x200b,
0xea2, 0xebb, 0xe87, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xec2, 0xea3, 0xe97, 0xeb5,
0xe95, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe8a, 0xeb2, 0x200b, 0xeae,
0xeb2, 0x200b, 0xea5, 0xeb4, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xe8a, 0xeb2, 0x200b,
0xeae, 0xeb2, 0x200b, 0xea5, 0xeb4, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xe8a, 0xeb2,
0x200b, 0xeae, 0xeb2, 0x200b, 0xea5, 0xeb4, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe8a,
0xeb2, 0xea1, 0xebb, 0xea7, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95,
0xeb0, 0xe96, 0xeb2, 0xe99, 0xe8a, 0xeb2, 0xea1, 0xebb, 0xea7, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe8a, 0xeb2, 0xea1,
0xebb, 0xea7, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe8a, 0x200b, 0xec0,
0xe8a, 0x200b, 0xea5, 0x200b, 0xeaa, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xeaa, 0xeb4, 0xe87, 0x200b, 0xe81, 0xeb0, 0x200b, 0xec2, 0xe9b, 0xec0, 0xea7, 0xea5,
0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xec2, 0xe8a, 0xec2,
0xea5, 0xea1, 0xead, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe88, 0xecd, 0xec0, 0xe88,
0xe8d, 0xec3, 0xe95, 0xec9, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe8a, 0xeb8,
0x200b, 0xea3, 0xeb4, 0x200b, 0xe99, 0xeb2, 0xea1, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20,
0xec2, 0xe8a, 0xea7, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe97, 0xeb2, 0xeae, 0xeb4,
0xe95, 0xeb4, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec4, 0xe97, 0x200b, 0xec0,
0xe9b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xec4, 0xe97, 0x200b, 0xec0, 0xe9b, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0x200b,
0xebd, 0xe87, 0x200b, 0xec4, 0xe97, 0x200b, 0xec0, 0xe9b, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe97, 0xeb2, 0xe88, 0xeb4, 0xe81, 0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7,
0xea5, 0xeb2, 0xec2, 0xe95, 0xec0, 0xe81, 0xec0, 0xea5, 0xebb, 0xeb2, 0xec0, 0xea7,
0xea5, 0xeb2, 0xe95, 0xead, 0xe87, 0xe81, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe95, 0xead, 0xe87, 0xe81, 0xeb2,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99,
0xe95, 0xead, 0xe87, 0xe81, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe8a, 0xeb8, 0xe81,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xea7, 0xe81, 0xec0, 0xea1, 0xe99, 0xeb4, 0xeaa,
0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0,
0xe96, 0xeb2, 0xe99, 0xe95, 0xea7, 0xe81, 0xec0, 0xea1, 0xe99, 0xeb4, 0xeaa, 0xe96,
0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9,
0xead, 0xe99, 0xe95, 0xea7, 0xe81, 0xec0, 0xea1, 0xe99, 0xeb4, 0xeaa, 0xe96, 0xeb2,
0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb9, 0xea7, 0xeb2, 0xea5, 0xeb9, 0x200b,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb9, 0x200b, 0xea3, 0xeb9, 0x200b,
0xe81, 0xea7, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xead, 0xeb9, 0x200b, 0xea3,
0xeb9, 0x200b, 0xe81, 0xea7, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xead, 0xeb9,
0x200b, 0xea3, 0xeb9, 0x200b, 0xe81, 0xea7, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead,
0xeb8, 0xeaa, 0xec0, 0xe9a, 0xe81, 0xeb4, 0xe94, 0xeaa, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xead, 0xeb8, 0xeaa, 0xec0, 0xe9a, 0xe81, 0xeb4, 0xe94, 0xeaa, 0xeb0, 0xe96, 0xeb2,
0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead,
0xe99, 0xead, 0xeb8, 0xeaa, 0xec0, 0xe9a, 0xe81, 0xeb4, 0xe94, 0xeaa, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea7, 0xeb2, 0xe99, 0xeb9, 0xead, 0xeb2,
0xe95, 0xeb9, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xea7, 0xeb2, 0xe99, 0xeb9, 0xead, 0xeb2, 0xe95, 0xeb9, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xea7, 0xeb2,
0xe99, 0xeb9, 0xead, 0xeb2, 0xe95, 0xeb9, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xec0, 0xea7, 0x200b, 0xec0, 0xe99, 0x200b, 0xe8a, 0xeb9, 0x200b, 0xec0, 0xead, 0x200b,
0xea5, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb2, 0xe94, 0xeb5, 0xec2, 0xea7,
0xeaa, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95,
0xeb0, 0xe96, 0xeb2, 0xe99, 0xea5, 0xeb2, 0xe94, 0xeb5, 0xec2, 0xea7, 0xeaa, 0xe95,
0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9,
0xead, 0xe99, 0xea5, 0xeb2, 0xe94, 0xeb5, 0xec2, 0xea7, 0xeaa, 0xe95, 0xebb, 0xe81,
0xec0, 0xea7, 0xea5, 0xeb2, 0xec2, 0xea7, 0xec2, 0xe81, 0xe81, 0xea3, 0xeb2, 0xe94,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xec2, 0xea7, 0xec2, 0xe81, 0xe81, 0xea3, 0xeb2, 0xe94, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xec2, 0xea7, 0xec2, 0xe81,
0xe81, 0xea3, 0xeb2, 0xe94, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xea7, 0xead, 0xeaa,
0xec2, 0xe95, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xec0,
0xea7, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea7, 0xeb2, 0xea5, 0xea5, 0xeb4, 0xeaa,
0x20, 0xec1, 0xea5, 0xeb0, 0x20, 0xe9f, 0xeb9, 0xe95, 0xeb9, 0xe99, 0xeb2, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea2, 0xeb2, 0xe81, 0xeb9, 0xe94, 0xeaa, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xea2, 0xeb2, 0xe81,
0xeb9, 0xe94, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae,
0xec9, 0xead, 0xe99, 0xea2, 0xeb2, 0xe81, 0xeb9, 0xe94, 0xeaa, 0xec0, 0xea7, 0xea5,
0xeb2, 0xec0, 0xea2, 0xe81, 0xeb2, 0xec0, 0xe95, 0xea5, 0xeb4, 0xe99, 0xec0, 0xe9a,
0xeb5, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xec0, 0xea2, 0xe81, 0xeb2, 0xec0, 0xe95, 0xea5, 0xeb4, 0xe99, 0xec0,
0xe9a, 0xeb5, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae,
0xec9, 0xead, 0xe99, 0xec0, 0xea2, 0xe81, 0xeb2, 0xec0, 0xe95, 0xea5, 0xeb4, 0xe99,
0xec0, 0xe9a, 0xeb5, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea2, 0xeb9, 0xe84, 0xead,
0xe99, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x101, 0x6c, 0x101, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x101, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x44,
0x69, 0x65, 0x6e, 0x76, 0x69, 0x64, 0x101, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x101, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65,
0x74, 0x75, 0x6d, 0x101, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52,
0x69, 0x65, 0x74, 0x75, 0x6d, 0x101, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x13c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x13c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x13c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x65, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c,
0x61, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74,
0x72, 0x75, 0x6d, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75,
0x73, 0x74, 0x72, 0x75, 0x6d, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72,
0x75, 0x6d, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61, 0x6c, 0x6e, 0x75, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61, 0x6c, 0x6e, 0x75, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61,
0x6c, 0x6e, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f,
0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b,
0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4b, 0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101, 0x6e, 0x61,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x61, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x70, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x70, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x72, 0x101, 0x62, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x70,
0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x72, 0x101, 0x62, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x70, 0x75,
0x73, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72, 0x101, 0x62, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x70, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x12b, 0x6e, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x12b, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x12b,
0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74,
0x75, 0x6d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x12b, 0x6e, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x72, 0x6d, 0x113, 0x6e, 0x69, 0x6a, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72, 0x6d, 0x113, 0x6e, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x72, 0x6d, 0x113, 0x6e, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c,
0x69, 0x6a, 0x61, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c,
0x61, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73,
0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74,
0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74,
0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x72, 0x69, 0x65, 0x74, 0x75,
0x6d, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74,
0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x72, 0x69, 0x65, 0x74, 0x75,
0x6d, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69,
0x73, 0x20, 0x72, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x75, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x61,
0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x75, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75,
0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x61, 0x75,
0x73, 0x74, 0x72, 0x75, 0x6d, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74,
0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x72, 0x69, 0x65, 0x74,
0x75, 0x6d, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73,
0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x72, 0x69, 0x65,
0x74, 0x75, 0x6d, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c,
0x69, 0x6a, 0x61, 0x73, 0x20, 0x72, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x75,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x101,
0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x101, 0x6e, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x75,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x20,
0x73, 0x61, 0x6c, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x20, 0x73,
0x61, 0x6c, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x161, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42,
0x75, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x42, 0x6f, 0x6c, 0x12b, 0x76, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x12b, 0x6c, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x12b,
0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x12b, 0x6c,
0x69, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a,
0x61, 0x73, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x61, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61, 0x62, 0x6f,
0x76, 0x65, 0x72, 0x64, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4b, 0x61, 0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x65, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b,
0x61, 0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x65, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x61, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x65, 0x74, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x65, 0x74, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x10c, 0x65, 0x74, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x10c, 0x12b, 0x6c, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x10c, 0x12b, 0x6c, 0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x12b, 0x6c, 0x65, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x136, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x136, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x136, 0x12b, 0x6e, 0x61, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x5a, 0x69, 0x65, 0x6d, 0x73, 0x76, 0x113, 0x74, 0x6b, 0x75, 0x20,
0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0x75, 0x20, 0x28, 0x4b, 0x12b, 0x6c, 0x69, 0x6e,
0x67, 0x61, 0x29, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x6b, 0x61, 0x20, 0x73, 0x61,
0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x6b, 0x61,
0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x6b, 0x61, 0x20, 0x73,
0x61, 0x6c, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75,
0x62, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x44, 0x65, 0x69, 0x76, 0x69, 0x73, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x44, 0x69, 0x6d, 0x6f, 0x6e,
0x64, 0x69, 0x72, 0x76, 0x69, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x74, 0x69, 0x6d,
0x6f, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x69,
0x65, 0x6c, 0x64, 0x69, 0x65, 0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x69, 0x65, 0x6c, 0x64,
0x69, 0x65, 0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c,
0x69, 0x65, 0x6c, 0x64, 0x69, 0x65, 0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x101, 0x6c, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x65, 0x69,
0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101,
0x6c, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75,
0x73, 0x74, 0x72, 0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75,
0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73,
0x74, 0x72, 0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f,
0x70, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x20, 0x6a, 0x6f,
0x73, 0x6c, 0x61, 0x20, 0x28, 0x46, 0x45, 0x54, 0x29, 0x52, 0x69, 0x65,
0x74, 0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x65, 0x69,
0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d,
0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x6f, 0x6c,
0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x61, 0x20, 0x28, 0x4d, 0x61, 0x6c, 0x76,
0x69, 0x6e, 0x75, 0x29, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x61,
0x20, 0x28, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x75, 0x29, 0x20, 0x73,
0x61, 0x6c, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64,
0x61, 0x20, 0x28, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x75, 0x29, 0x20,
0x73, 0x61, 0x6c, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46,
0x69, 0x64, 0x17e, 0x69, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x47, 0x76, 0x69, 0x101, 0x6e, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x44, 0x69, 0x65, 0x6e, 0x76, 0x69, 0x64, 0x6a, 0x16b,
0x72, 0x75, 0x20, 0x75, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b,
0x74, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x74, 0x65, 0x72, 0x69, 0x74, 0x6f,
0x72, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x47, 0x61, 0x6d, 0x62, 0x6a, 0x113, 0x20, 0x73, 0x61, 0x6c, 0x75,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47, 0x72, 0x75, 0x7a,
0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x61, 0x20,
0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47, 0x72,
0x69, 0x6e, 0x69, 0x10d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d,
0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69,
0x65, 0x74, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x67, 0x72, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x72,
0x73, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x12b, 0x10d, 0x61, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x47, 0x61, 0x6a, 0x101, 0x6e, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x75, 0x2013,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x48, 0x61, 0x76, 0x61, 0x6a, 0x75, 0x2013, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x75, 0x2013, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x6e, 0x6b,
0x6f, 0x6e, 0x67, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e,
0x67, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x48, 0x6f, 0x76, 0x64, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x6e, 0x64, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x73, 0x20,
0x6f, 0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x49, 0x6e, 0x64, 0x6f, 0x137, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x113, 0x7a, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x113, 0x7a, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x113, 0x7a, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x49, 0x72, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x49, 0x72, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x72, 0x101,
0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x113, 0x6c, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x113,
0x6c, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x113, 0x6c, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4a, 0x61, 0x70, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4a, 0x61, 0x70, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a,
0x61, 0x70, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x61, 0x73, 0x2d,
0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x61, 0x73, 0x2d, 0x4b, 0x61, 0x6d, 0x10d,
0x61, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x61, 0x73, 0x2d, 0x4b,
0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b,
0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x6b,
0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x6b, 0x61,
0x7a, 0x61, 0x68, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x61, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0x65, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x69, 0x72,
0x67, 0x69, 0x7a, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4c, 0x61, 0x69, 0x6e, 0x61, 0x73, 0x20, 0x73, 0x61,
0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x6f, 0x72, 0x64,
0x61, 0x20, 0x48, 0x61, 0x76, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x6f, 0x72, 0x64, 0x61, 0x20,
0x48, 0x61, 0x76, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c,
0x6f, 0x72, 0x64, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61, 0x20, 0x73, 0x61,
0x6c, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x6c, 0x61, 0x69, 0x7a, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x6c,
0x64, 0x12b, 0x76, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4d, 0x61, 0x72, 0x137, 0x12b, 0x7a, 0x61, 0x20, 0x73, 0x61, 0x6c,
0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x101, 0x72, 0x161, 0x61,
0x6c, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4d, 0x61, 0x75, 0x72, 0x12b, 0x63, 0x69, 0x6a, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x12b, 0x63, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x12b, 0x63, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4d, 0x6f, 0x73, 0x6f, 0x6e, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x73, 0x20, 0x4b, 0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101,
0x6e, 0x61, 0x20, 0x70, 0x69, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x74, 0x65,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x61, 0x73, 0x20, 0x4b, 0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b,
0x65, 0x101, 0x6e, 0x61, 0x20, 0x70, 0x69, 0x65, 0x6b, 0x72, 0x61, 0x73,
0x74, 0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x73,
0x20, 0x4b, 0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101, 0x6e,
0x61, 0x20, 0x70, 0x69, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x74, 0x65, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x6f, 0x72, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x6f, 0x72, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x73, 0x6b,
0x61, 0x76, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61,
0x73, 0x6b, 0x61, 0x76, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x61,
0x76, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x65, 0x70, 0x101, 0x6c, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x75, 0x6e, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4a, 0x61, 0x75, 0x6e, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x75, 0x6e, 0x6b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4a, 0x61, 0x75, 0x6e, 0x7a, 0x113, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x75, 0x6e, 0x7a, 0x113,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x75, 0x6e, 0x7a,
0x113, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x145, 0x16b, 0x66,
0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x145, 0x16b, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c,
0x65, 0x6e, 0x64, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x145, 0x16b, 0x66, 0x61, 0x75, 0x6e,
0x64, 0x6c, 0x65, 0x6e, 0x64, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x69, 0x75,
0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x61, 0x73, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x61, 0x73, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x61, 0x73, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x64,
0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x146, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20,
0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x146, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x64, 0x69, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x146, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4f, 0x6d, 0x73, 0x6b,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4f, 0x6d, 0x73, 0x6b,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x4a, 0x61, 0x75, 0x6e, 0x67,
0x76, 0x69, 0x6e, 0x65, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61,
0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x72, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x50, 0x65, 0x72, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x72, 0x75, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x12b, 0x6e, 0x75, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x12b, 0x6e, 0x75, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x12b, 0x6e, 0x75, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x113,
0x6e, 0x69, 0x6b, 0x73, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x53, 0x65, 0x6e, 0x70, 0x6a, 0x113, 0x72, 0x61,
0x73, 0x20, 0x75, 0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x65, 0x6e, 0x70,
0x6a, 0x113, 0x72, 0x61, 0x73, 0x20, 0x75, 0x6e, 0x20, 0x4d, 0x69, 0x6b,
0x65, 0x6c, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x65, 0x6e, 0x70, 0x6a,
0x113, 0x72, 0x61, 0x73, 0x20, 0x75, 0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x69, 0x74, 0x6b, 0x113,
0x72, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x6f,
0x6e, 0x61, 0x70, 0x113, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x68,
0x65, 0x6e, 0x6a, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x52, 0x65, 0x69, 0x6e, 0x6a, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x12b, 0x6e,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x68, 0x61,
0x6c, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x12b,
0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53,
0x65, 0x69, 0x161, 0x65, 0x13c, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x16b,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x5a, 0x101, 0x6c,
0x61, 0x6d, 0x61, 0x6e, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x44, 0x69, 0x65, 0x6e, 0x76, 0x69, 0x64, 0x64,
0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x160, 0x6f, 0x76, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x61, 0x69, 0x74, 0x69, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x54, 0x61, 0x69, 0x62, 0x65, 0x69, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x54, 0x61, 0x69, 0x62, 0x65, 0x69, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54,
0x61, 0x69, 0x62, 0x65, 0x69, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x61, 0x64, 0x17e, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x16b, 0x6b, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x101,
0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x72, 0x75, 0x67,
0x76, 0x61, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55,
0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x72, 0x75,
0x67, 0x76, 0x61, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x101,
0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56,
0x65, 0x6e, 0x65, 0x63, 0x75, 0x113, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56,
0x65, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x56, 0x6f, 0x6c, 0x69, 0x73, 0x61, 0x73, 0x20,
0x75, 0x6e, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x75,
0x6b, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e,
0x74, 0xe1, 0x6e, 0x67, 0x6f, 0x20, 0x79, 0x61, 0x20, 0x4c, 0x75, 0x62,
0x75, 0x6d, 0x62, 0x61, 0x73, 0x68, 0x69, 0x4e, 0x74, 0xe1, 0x6e, 0x67,
0x6f, 0x20, 0x79, 0x61, 0x20, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x61, 0x20,
0x79, 0x61, 0x20, 0x190, 0x301, 0x73, 0x69, 0x74, 0x61, 0x4e, 0x74, 0xe1,
0x6e, 0x67, 0x6f, 0x20, 0x79, 0x61, 0x20, 0x41, 0x66, 0x72, 0xed, 0x6b,
0x61, 0x20, 0x79, 0x61, 0x20, 0x53, 0x69, 0x64, 0x69, 0x4e, 0x74, 0xe1,
0x6e, 0x67, 0x6f, 0x20, 0x79, 0x61, 0x20, 0x4c, 0x6f, 0x6e, 0x64, 0x6f,
0x6e, 0x69, 0x4e, 0x74, 0xe1, 0x6e, 0x67, 0x6f, 0x20, 0x79, 0x61, 0x20,
0x53, 0x65, 0x79, 0x73, 0x68, 0x65, 0x6c, 0x41, 0x6b, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x66,
0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117,
0x73, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50,
0x69, 0x65, 0x74, 0x173, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72,
0x173, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61,
0x72, 0x173, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x6c, 0x69, 0x61, 0x73, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x6c, 0x69, 0x61, 0x73, 0x6b, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x41, 0x6c, 0x69, 0x61, 0x73, 0x6b, 0x6f, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x6f, 0x73, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x117, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x117, 0x73, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x117, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69,
0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x6f, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72,
0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20,
0x72, 0x79, 0x74, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160,
0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x6f, 0x73, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69,
0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x6f, 0x73, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69,
0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x6f, 0x73, 0x20, 0x6b, 0x61, 0x6c, 0x6e, 0x173, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6b, 0x61, 0x6c, 0x6e,
0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6b, 0x61, 0x6c, 0x6e,
0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d,
0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79,
0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61,
0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f,
0x73, 0x20, 0x52, 0x61, 0x6d, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x76, 0x61,
0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61,
0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f,
0x73, 0x20, 0x52, 0x61, 0x6d, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x76, 0x61,
0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x117, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x117, 0x73, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x117, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x70,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x70, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x70, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x6b, 0x74, 0x61, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62,
0x117, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x74,
0x6f, 0x62, 0x117, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x117,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61,
0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61,
0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x72, 0x6d, 0x117, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x6d, 0x117, 0x6e, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x6d, 0x117, 0x6e, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e,
0x117, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117,
0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117,
0x73, 0x20, 0x76, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73,
0x20, 0x76, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20, 0x76, 0x61, 0x6b, 0x61, 0x72,
0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74,
0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x64, 0x17e, 0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x7a,
0x6f, 0x72, 0x173, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x173, 0x20, 0x53, 0x61, 0x6c,
0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x173, 0x20, 0x53, 0x61, 0x6c,
0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x161, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x75,
0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42,
0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x72, 0x61, 0x7a,
0x69, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x72, 0x61, 0x7a,
0x69, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x72, 0x75,
0x6e, 0x117, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x17d, 0x61, 0x6c, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x4b, 0x79, 0x161, 0x75,
0x6c, 0x69, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x17d, 0x61,
0x6c, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x4b, 0x79, 0x161, 0x75, 0x6c, 0x69,
0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x17d, 0x61, 0x6c, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x4b,
0x79, 0x161, 0x75, 0x6c, 0x69, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x65, 0x69,
0x73, 0x69, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x61,
0x6d, 0x6f, 0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c,
0x61, 0x74, 0x61, 0x6d, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x10c, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61,
0x6d, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x69, 0x6c, 0x117, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x69, 0x6c, 0x117, 0x73, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x10c, 0x69, 0x6c, 0x117, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x69, 0x6e, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x69,
0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x69, 0x6e, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x61, 0x6c, 0x117, 0x64, 0x173, 0x20,
0x53, 0x61, 0x6c, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x173, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4b, 0x75, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4b, 0x75, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c,
0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4b, 0x75, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173,
0x20, 0x70, 0x75, 0x73, 0x117, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x75, 0x62,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x75, 0x62,
0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x6f, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x44, 0x65, 0x69, 0x76, 0x69, 0x73, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x44, 0x69, 0x75, 0x6d, 0x6f, 0x6e, 0x6f, 0x20, 0x64, 0x2019,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x69, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x65, 0x6c, 0x79,
0x6b, 0x173, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x56, 0x65, 0x6c, 0x79, 0x6b, 0x173, 0x20, 0x73, 0x61,
0x6c, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x65, 0x6c, 0x79, 0x6b, 0x173, 0x20,
0x53, 0x61, 0x6c, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x45, 0x6b, 0x76, 0x61,
0x64, 0x6f, 0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56,
0x69, 0x64, 0x75, 0x72, 0x69, 0x6f, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x69, 0x64,
0x75, 0x72, 0x69, 0x6f, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x56, 0x69, 0x64, 0x75, 0x72, 0x69, 0x6f, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6c, 0x69,
0x6d, 0x173, 0x6a, 0x173, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b,
0x61, 0x72, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x173,
0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x46, 0x69, 0x64, 0x17e, 0x69, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x69, 0x64,
0x17e, 0x69, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x72, 0x61, 0x6e, 0x63, 0x16b,
0x7a, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x47, 0x76, 0x69, 0x61, 0x6e, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x69, 0x65, 0x74,
0x173, 0x20, 0x50, 0x72, 0x61, 0x6e, 0x63, 0x16b, 0x7a, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x69, 0x72, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x6e, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x47, 0x61, 0x6d, 0x62, 0x79, 0x72, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f,
0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x47, 0x72, 0x69, 0x6e, 0x76, 0x69, 0x10d, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x72,
0x79, 0x74, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6b, 0x61, 0x72,
0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6b,
0x61, 0x72, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6b, 0x61, 0x72, 0x173,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x47, 0x75, 0x61, 0x6d, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x50, 0x65, 0x72, 0x73, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x12f, 0x6c, 0x61, 0x6e, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x47, 0x61, 0x6a, 0x61, 0x6e, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x173, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x48, 0x61, 0x76, 0x61, 0x6a, 0x173, 0x2013, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x173, 0x2013, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f,
0x6e, 0x67, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f,
0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x6b,
0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x6e,
0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x49, 0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6e, 0x64,
0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69,
0x6e, 0x117, 0x73, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79,
0x74, 0x173, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b,
0x61, 0x72, 0x173, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4b, 0x61, 0x6d, 0x10d, 0x69, 0x61, 0x74, 0x6b,
0x6f, 0x73, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4b, 0x61, 0x6d, 0x10d, 0x69, 0x61, 0x74, 0x6b, 0x6f, 0x73, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4b, 0x61, 0x6d, 0x10d, 0x69, 0x61, 0x74, 0x6b, 0x6f,
0x73, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x61, 0x7a, 0x61, 0x63,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x63,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4b, 0x6f, 0x72, 0x117, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4b, 0x6f, 0x72, 0x117, 0x6a, 0x6f, 0x73, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4b, 0x6f, 0x72, 0x117, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x6a, 0x117, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4c, 0x61, 0x6e, 0x6b, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4c, 0x61, 0x69, 0x6e, 0x6f, 0x20,
0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4c,
0x6f, 0x72, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4c, 0x6f, 0x72, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x75,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4c, 0x6f, 0x72, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x75, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x6b,
0x61, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x61, 0x69, 0x7a, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61,
0x6c, 0x64, 0x79, 0x76, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x72, 0x161, 0x61,
0x6c, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x61, 0x75,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x63, 0x69, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x69, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d,
0x6f, 0x73, 0x6f, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d,
0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79,
0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d, 0x69, 0x6f, 0x6a,
0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x52, 0x61,
0x6d, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e,
0x79, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x76, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x76, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x76, 0x6f, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4d, 0x69, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69, 0x6f,
0x73, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a,
0x6f, 0x73, 0x69, 0x6f, 0x73, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f,
0x73, 0x69, 0x6f, 0x73, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f,
0x73, 0x69, 0x6f, 0x73, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61,
0x75, 0x6a, 0x6f, 0x73, 0x69, 0x6f, 0x73, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a,
0x6f, 0x73, 0x69, 0x6f, 0x73, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x69, 0x75, 0x66, 0x61,
0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4e, 0x69, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c,
0x65, 0x6e, 0x64, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x69, 0x75, 0x66, 0x61, 0x75,
0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x69,
0x75, 0x6a, 0x117, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x6f, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x61, 0x6e, 0x6f, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50,
0x61, 0x6c, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69,
0x6f, 0x73, 0x20, 0x47, 0x76, 0x69, 0x6e, 0x117, 0x6a, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76,
0x61, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x75, 0x73, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x75, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x50, 0x65, 0x72, 0x75, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x173, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x173,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x6f, 0x20, 0x53,
0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x65,
0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x6f, 0x20, 0x69, 0x72, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x6f, 0x20,
0x69, 0x72, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x6f, 0x20, 0x69,
0x72, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x117, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x63, 0x68, 0x65, 0x6e, 0x6a,
0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x6f,
0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x52, 0x65, 0x75, 0x6e, 0x6a, 0x6f, 0x6e, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x6f, 0x74, 0x65, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x63,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x53, 0x65, 0x69, 0x161, 0x65, 0x6c, 0x69, 0x173, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x16b, 0x72, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6c, 0x69, 0x61,
0x6d, 0x6f, 0x6e, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x50, 0x69, 0x65, 0x74, 0x173, 0x20, 0x44, 0x17e,
0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x69, 0x6f, 0x76, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x61, 0x68, 0x69, 0x10d,
0x69, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x61, 0x69,
0x70, 0x117, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x54, 0x61, 0x69, 0x70, 0x117, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x54, 0x61, 0x69, 0x70, 0x117, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6e,
0x67, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f,
0x6e, 0x67, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x6f, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x10c, 0x75, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x117, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x117, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x117, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61,
0x6a, 0x61, 0x75, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55,
0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55,
0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x65, 0x6e, 0x65, 0x73, 0x75,
0x65, 0x6c, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x65, 0x69, 0x6b, 0x6f,
0x20, 0x53, 0x61, 0x6c, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x56, 0x6f, 0x6c, 0x69, 0x73, 0x6f, 0x20, 0x69, 0x72, 0x20, 0x46,
0x75, 0x74, 0x16b, 0x6e, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a, 0x75, 0x6b, 0x6f, 0x6e,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x74, 0x63, 0x69, 0x6b,
0x61, 0x20, 0x66, 0x6f, 0x20, 0x6c, 0x61, 0x20, 0x6b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x69, 0x61, 0x73, 0x6e, 0x61, 0x6c, 0x63, 0x69, 0x73,
0x74, 0x63, 0x69, 0x6b, 0x61, 0x20, 0x66, 0x6f, 0x20, 0x6c, 0x61, 0x20,
0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x69, 0x61, 0x73, 0x63, 0x69,
0x73, 0x74, 0x63, 0x69, 0x6b, 0x61, 0x20, 0x66, 0x6f, 0x20, 0x6c, 0x61,
0x20, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x69, 0x61, 0x73, 0x74,
0x63, 0x69, 0x6b, 0x61, 0x20, 0x66, 0x6f, 0x20, 0x6c, 0x61, 0x20, 0x67,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72,
0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161,
0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x64, 0x6e, 0x6a, 0x6f, 0x77,
0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61,
0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3,
0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x66, 0x72,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a,
0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x63, 0x6f, 0x6e, 0x61, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x63, 0x6f, 0x6e,
0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x63, 0x6f,
0x6e, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142,
0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63,
0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x79, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e,
0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x70, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63,
0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70,
0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x79, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0xf3, 0x64, 0x7a,
0x61, 0x6a, 0x74, 0x161, 0x6e, 0x79, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f,
0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x67, 0xf3, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142,
0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64,
0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142,
0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x70, 0x69, 0x61,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x70, 0x69, 0x61, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e,
0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f,
0x72, 0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63,
0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x41, 0x72, 0x6d, 0x65, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x41, 0x72, 0x6d, 0x65, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x41, 0x72, 0x6d, 0x65, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x61, 0x77, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53,
0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a,
0x17a, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x70, 0xf3, 0x64, 0x77,
0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x79, 0x20, 0x61, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72,
0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x70, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63,
0x6f, 0x72, 0x6e, 0x79, 0x20, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a,
0x6f, 0x70, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x79,
0x20, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x61, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x61, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x61, 0x77, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77,
0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64,
0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64,
0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61,
0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x41, 0x63, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41,
0x63, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x63, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x161, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x42, 0x68, 0x75, 0x74, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x42, 0x6f, 0x6c, 0x69, 0x77, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20,
0x63, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x4b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x4b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x69, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x43, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43,
0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x69, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x47, 0xf3, 0x64, 0x6f,
0x77, 0x6e, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x63,
0x61, 0x73, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x77, 0x79, 0x63,
0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b,
0x75, 0x70, 0x6f, 0x77, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f,
0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70,
0x6f, 0x77, 0x4b, 0x75, 0x62, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x4b, 0x75, 0x62, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x44, 0x61,
0x76, 0x69, 0x73, 0x20, 0x63, 0x61, 0x73, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x44, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20, 0x63, 0x61,
0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x74,
0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63,
0x61, 0x73, 0x20, 0x4a, 0x61, 0x74, 0x161, 0x6f, 0x77, 0x6e, 0x65, 0x6a,
0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x20, 0x4a, 0x61, 0x74,
0x161, 0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x20, 0x4a, 0x61, 0x74, 0x161, 0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20,
0x6b, 0x75, 0x70, 0x79, 0x45, 0x6b, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a,
0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6e, 0x6f,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6e, 0x6f, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a,
0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a,
0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61,
0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67,
0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3,
0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x46, 0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x46, 0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x46, 0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x6f, 0x6a, 0x73, 0x6b, 0x6f, 0x67, 0x75, 0x79, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20,
0x66, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x6a, 0x73, 0x6b, 0x65, 0x67, 0x6f,
0x20, 0x70, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x64, 0x6e, 0x6a, 0x6f, 0x77,
0x65, 0x67, 0x6f, 0x20, 0x61, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b,
0x74, 0x69, 0x73, 0x6b, 0x65, 0x68, 0x6f, 0x20, 0x74, 0x65, 0x72, 0x69,
0x74, 0x6f, 0x72, 0x69, 0x75, 0x6d, 0x61, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63,
0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x77,
0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x67,
0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f,
0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f,
0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x67,
0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e,
0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72,
0x6e, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x50, 0x65, 0x72, 0x73, 0x69, 0x73,
0x6b, 0x65, 0x67, 0x6f, 0x20, 0x67, 0x6f, 0x6c, 0x66, 0x61, 0x47, 0x75,
0x79, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x73,
0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x6f, 0x6f, 0x63, 0x65, 0x61, 0x6e,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x6f,
0x63, 0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53,
0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61,
0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x6b, 0x69, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e,
0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x49, 0x72, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x49, 0x72, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x49,
0x72, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4a, 0x61, 0x70, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4a, 0x61, 0x70, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x4a, 0x61, 0x70, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x6b, 0x61,
0x7a, 0x61, 0x63, 0x68, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x6b, 0x61,
0x7a, 0x61, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x6b, 0x61,
0x7a, 0x61, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b,
0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b,
0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x6f,
0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x69,
0x6a, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77,
0x63, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70,
0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x20,
0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f,
0x77, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x61, 0x6a, 0x7a, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69,
0x77, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x77, 0x79,
0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x63, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x65, 0x78, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x65, 0x78, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x4d, 0x65, 0x78, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x2d,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x77, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x77, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x6b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f,
0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x65,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4e,
0x6f, 0x77, 0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77,
0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x69, 0x75, 0x65, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x6b, 0x75,
0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x63, 0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4e, 0x6f, 0x77, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e,
0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x70, 0x75, 0x61,
0x2d, 0x4e, 0x6f, 0x77, 0x6f, 0x67, 0x69, 0x6e, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x65,
0x72, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x65, 0x72,
0x75, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75,
0x70, 0x6f, 0x77, 0x53, 0x74, 0x2e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x2d, 0x61, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x74, 0x2e, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x53, 0x74,
0x2e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63,
0x61, 0x73, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f,
0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x50, 0x6f,
0x6e, 0x61, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x6a,
0xf6, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x74, 0x68,
0x65, 0x72, 0x61, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x63, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x75, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x70, 0x6f, 0x142, 0x64, 0x6e, 0x6a, 0x6f, 0x77, 0x6f, 0x67,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x63, 0x61, 0x73, 0x54,
0x61, 0x68, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x54, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x54, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x54, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x61, 0x64, 0x17a, 0x69, 0x6b, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x57, 0x6c, 0x61, 0x64, 0x69,
0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x57, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x57, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63,
0x61, 0x73, 0x20, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x63, 0x61, 0x73,
0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x63, 0x61,
0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x57, 0x61, 0x6c, 0x6c,
0x69, 0x73, 0x20, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x61, 0x73, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54,
0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x53,
0xf6, 0xf6, 0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73,
0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54,
0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73,
0x63, 0x68, 0x20, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x74,
0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x7a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20,
0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68,
0x20, 0x6f, 0x6f, 0x73, 0x74, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x65, 0x74,
0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x6f, 0x6f, 0x73, 0x74, 0x65, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65,
0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x6f, 0x6f, 0x73, 0x74, 0x65,
0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74,
0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x42, 0x61, 0x72, 0x67, 0x74, 0x69,
0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x42, 0x61, 0x72, 0x67,
0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65,
0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x42, 0x61, 0x72, 0x67, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x63, 0x68, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x74,
0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x50, 0x61, 0x7a,
0x69, 0x66, 0x69, 0x6b, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x50,
0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x65, 0x74, 0x41, 0x72, 0x61, 0x61, 0x62, 0x73, 0x63,
0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x41, 0x72, 0x61, 0x61, 0x62, 0x73,
0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74,
0x69, 0x65, 0x74, 0x41, 0x72, 0x61, 0x61, 0x62, 0x73, 0x63, 0x68, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x63, 0x68, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x5a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x5a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x65, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65,
0x73, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69,
0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63,
0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69,
0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74,
0x4f, 0x6f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73, 0x74,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74,
0x4f, 0x6f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65,
0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63,
0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69,
0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x65, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x54, 0x69,
0x65, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74,
0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4f,
0x6f, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63,
0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68,
0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x47, 0x72, 0xf6,
0xf6, 0x6e, 0x77, 0x69, 0x73, 0x63, 0x68, 0x2d, 0x57, 0x65, 0x6c, 0x74,
0x74, 0x69, 0x65, 0x74, 0x49, 0x6e, 0x64, 0x69, 0x65, 0x6e, 0x2d, 0x54,
0x69, 0x65, 0x74, 0x53, 0xf6, 0xf6, 0x64, 0x6f, 0x6f, 0x73, 0x74, 0x61,
0x73, 0x69, 0x61, 0x61, 0x74, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65,
0x74, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x63, 0x68, 0x20,
0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x74, 0x69, 0x65, 0x74,
0x4f, 0x6f, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73,
0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69,
0x65, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d, 0x54, 0x69, 0x65,
0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65,
0x74, 0x4a, 0x61, 0x70, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54,
0x69, 0x65, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65,
0x74, 0x4a, 0x61, 0x70, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4d, 0x6f, 0x73, 0x6b,
0x61, 0x75, 0x2d, 0x54, 0x69, 0x65, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61,
0x75, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69,
0x65, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x66,
0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x4f, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0xfc, 0x64, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x57,
0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73,
0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x49, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x7a,
0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x49, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x49, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x4f, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65,
0x6e, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4f, 0x73,
0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4f,
0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x52, 0x6f, 0x63, 0x6b, 0x79,
0x2d, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x57, 0x65, 0x73,
0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x7a, 0xe4, 0x69, 0x74, 0x4e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74,
0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b, 0xfc,
0x73, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x41, 0x72, 0x61, 0x62, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x41, 0x72, 0x61, 0x62, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x61, 0x62,
0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68,
0x20, 0x5a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x5a, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x2f,
0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x73,
0x63, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x41, 0x73, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x73, 0x63, 0x68,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69,
0x64, 0x73, 0x63, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x63, 0x68, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x42, 0x72,
0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x42,
0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c,
0x69, 0x61, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4b, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4b, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x61,
0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72,
0x72, 0x6f, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x73,
0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65,
0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x43, 0x68, 0x72, 0xeb, 0x73, 0x63, 0x68, 0x74, 0x64, 0x61, 0x67,
0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x6f,
0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65,
0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x75,
0x62, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74,
0x4b, 0x75, 0x62, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x75, 0x62, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x44, 0x61, 0x76, 0x69, 0x73, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4f,
0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4f, 0x75, 0x73, 0x63, 0x68, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65,
0x6c, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x75, 0x73, 0x63, 0x68, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x75, 0x73, 0x63, 0x68, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x72, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x4d, 0xeb, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4d, 0xeb,
0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73,
0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x4d, 0xeb, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68,
0x20, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x69, 0x64, 0x73, 0x63,
0x68, 0x69, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x46, 0x69, 0x64, 0x73, 0x63,
0x68, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x72, 0x61, 0x6e, 0x73,
0xe9, 0x69, 0x73, 0x63, 0x68, 0x2d, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e,
0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x46, 0x72, 0x61, 0x6e, 0x73, 0xe9,
0x69, 0x73, 0x63, 0x68, 0x20, 0x53, 0xfc, 0x64, 0x2d, 0x20, 0x61, 0x6e,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x67, 0x65,
0x62, 0x69, 0x64, 0x64, 0x65, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x2d, 0x49, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x4d, 0xeb, 0x74, 0x74, 0x6c, 0x65, 0x72, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x47, 0x75, 0x61, 0x6d, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x47, 0x6f, 0x6c, 0x66, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x2d, 0x4b,
0x6f, 0x6e, 0x67, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67,
0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x2d, 0x4b, 0x6f, 0x6e,
0x67, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x48, 0x6f, 0x76, 0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x48, 0x6f, 0x76,
0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x48, 0x6f, 0x76, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x49, 0x6e, 0x64, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x49, 0x6e, 0x64, 0x65, 0x73, 0x63, 0x68, 0x65, 0x6e,
0x20, 0x4f, 0x7a, 0x65, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x49,
0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74,
0x4f, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x65, 0x73,
0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x49, 0x72, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x5a, 0xe4, 0x69, 0x74, 0x49, 0x72, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x49,
0x72, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x73,
0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x61, 0x6d,
0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b,
0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74,
0x6b, 0x61, 0x73, 0x61, 0x63, 0x68, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x63,
0x68, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4c, 0x69,
0x6e, 0x6e, 0x65, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77,
0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48,
0x6f, 0x77, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61,
0x6c, 0x61, 0x79, 0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b,
0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x65,
0x78, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x50, 0x61,
0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72,
0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65,
0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x79, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65, 0x73,
0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69,
0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x73, 0xe9, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69,
0x73, 0xe9, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x73, 0xe9, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x66, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x4e, 0x69, 0x75, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d,
0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x77,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e,
0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x6d, 0x73,
0x6b, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x6d, 0x73,
0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x5a, 0xe4, 0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e, 0x65, 0x69, 0x67, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x5a, 0xe4, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x50, 0x65, 0x72,
0x75, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x50, 0x68,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x50,
0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x61, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x65, 0x79, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x2d, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x53, 0xfc, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x79, 0x6f,
0x77, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x54, 0x61, 0x64, 0x73, 0x63, 0x68, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74,
0x55, 0x72, 0x75, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x55,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x55,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x6c,
0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x49, 0x6e, 0x73, 0x65,
0x6c, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x2d, 0x61, 0x2d, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x65, 0x72, 0x67, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x65, 0x72, 0x67, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x65, 0x72, 0x67,
0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x410,
0x43a, 0x440, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x432, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x421,
0x440, 0x435, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x443, 0x436, 0x43d, 0x43e, 0x430, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x430, 0x20, 0x420, 0x435, 0x43f,
0x443, 0x431, 0x43b, 0x438, 0x43a, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43b, 0x458, 0x430, 0x441,
0x43a, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43b,
0x458, 0x430, 0x441, 0x43a, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43b, 0x458,
0x430, 0x441, 0x43a, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x426, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421,
0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20,
0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x418, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x41f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f,
0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f, 0x43b, 0x430, 0x43d,
0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f,
0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f, 0x430, 0x446, 0x438,
0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440,
0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x410, 0x43f, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43f, 0x438,
0x458, 0x430, 0x410, 0x440, 0x430, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x430, 0x440, 0x430, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435,
0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432,
0x43e, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x437, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442,
0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415, 0x440, 0x43c, 0x435,
0x43d, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x415, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415,
0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x410, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x430, 0x20, 0x438, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x430, 0x20, 0x438, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x430, 0x20, 0x438, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x437, 0x435, 0x440, 0x431,
0x435, 0x458, 0x45f, 0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x437, 0x435, 0x440,
0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x437, 0x43e, 0x440,
0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x443, 0x442, 0x430, 0x43d,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x43e, 0x43b,
0x438, 0x432, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x440, 0x430, 0x437,
0x438, 0x43b, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x458, 0x20, 0x414, 0x430, 0x440,
0x443, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434,
0x435, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x430, 0x431,
0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41a,
0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x43e,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x430, 0x442,
0x430, 0x43c, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x430,
0x442, 0x430, 0x43c, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x430, 0x442, 0x430, 0x43c,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x438, 0x43b,
0x435, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x438, 0x43b,
0x435, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430,
0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442,
0x43e, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x438, 0x43b, 0x435, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x438, 0x43d, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x438, 0x43d, 0x430, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432,
0x43e, 0x20, 0x41a, 0x438, 0x43d, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x411, 0x43e, 0x436, 0x438, 0x45c, 0x435, 0x43d, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x438, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x438, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x438,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x443, 0x431, 0x430, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x443, 0x431, 0x430, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e,
0x20, 0x41a, 0x443, 0x431, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x414, 0x435, 0x458, 0x432, 0x438, 0x441, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x414,
0x438, 0x440, 0x432, 0x438, 0x43b, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20, 0x422, 0x438,
0x43c, 0x43e, 0x440, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x412, 0x435, 0x43b, 0x438, 0x433, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x412, 0x435, 0x43b, 0x438, 0x433, 0x434, 0x435, 0x43d, 0x441, 0x43a,
0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412,
0x435, 0x43b, 0x438, 0x433, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x421, 0x440, 0x435,
0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x43d, 0x43e, 0x435,
0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x421, 0x440, 0x435, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440,
0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x430,
0x43b, 0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432,
0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424,
0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a,
0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e,
0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x438,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x438, 0x45f, 0x438, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x438, 0x45f, 0x438, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x424, 0x438, 0x45f, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x441, 0x43a, 0x430, 0x20, 0x413,
0x432, 0x430, 0x458, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443,
0x441, 0x43a, 0x43e, 0x20, 0x458, 0x443, 0x436, 0x43d, 0x43e, 0x20, 0x438, 0x20,
0x430, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413, 0x430, 0x43c, 0x431,
0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413, 0x440,
0x443, 0x437, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413,
0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x43e, 0x432,
0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x421, 0x440, 0x435,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x43f, 0x43e,
0x20, 0x413, 0x440, 0x438, 0x43d, 0x438, 0x447, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20,
0x418, 0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x435,
0x440, 0x441, 0x438, 0x441, 0x43a, 0x438, 0x20, 0x417, 0x430, 0x43b, 0x438, 0x432,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413, 0x432, 0x430,
0x458, 0x430, 0x43d, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x425, 0x430, 0x432, 0x430, 0x438, 0x20, 0x2d, 0x20, 0x410, 0x43b, 0x435,
0x443, 0x442, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x430, 0x432,
0x430, 0x438, 0x20, 0x2d, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a,
0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e,
0x20, 0x425, 0x430, 0x432, 0x430, 0x438, 0x20, 0x2d, 0x20, 0x410, 0x43b, 0x435,
0x443, 0x442, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x43e,
0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x43e, 0x432,
0x434, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x43e, 0x432,
0x434, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x441, 0x43a, 0x438, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438,
0x43d, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x430, 0x20, 0x418, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x20,
0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x430, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458,
0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440,
0x430, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440,
0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442,
0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x430, 0x43d, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20,
0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x408, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x430, 0x43f, 0x43e, 0x43d,
0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x430, 0x43f, 0x43e, 0x43d,
0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x435,
0x43d, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x435, 0x43d, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430,
0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e,
0x440, 0x435, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41a, 0x43e, 0x440, 0x435, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e,
0x440, 0x435, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440,
0x441, 0x43a, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x438, 0x440, 0x433,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41b, 0x438, 0x43d, 0x438, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425,
0x430, 0x443, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442,
0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425,
0x430, 0x443, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x43b, 0x435, 0x437, 0x438, 0x458, 0x430,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x43b,
0x434, 0x438, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x441, 0x430, 0x441, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b,
0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x432, 0x440,
0x438, 0x446, 0x438, 0x443, 0x441, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x443, 0x441, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432,
0x43e, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x443, 0x441, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x43e, 0x441, 0x43e,
0x43d, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x435, 0x43a, 0x441,
0x438, 0x43a, 0x43e, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x43a, 0x43e, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x43f, 0x430,
0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x43e,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x43b, 0x430,
0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e,
0x440, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442,
0x43e, 0x440, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41c, 0x458, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x435, 0x43f, 0x430,
0x43b, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e,
0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458,
0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432,
0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432,
0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41d,
0x43e, 0x432, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b,
0x435, 0x43d, 0x434, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41d, 0x438, 0x443, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x448, 0x43a, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x448, 0x43a, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x448, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x435,
0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e,
0x440, 0x43e, 0x45a, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20,
0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x435,
0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e,
0x440, 0x43e, 0x45a, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e,
0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41e, 0x43c, 0x441, 0x43a, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41e, 0x43c, 0x441, 0x43a, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41e, 0x43c, 0x441, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430,
0x43b, 0x430, 0x443, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x413,
0x432, 0x438, 0x43d, 0x435, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x458, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x458, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41f, 0x435, 0x440, 0x443, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41f, 0x435, 0x440, 0x443, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x435, 0x440, 0x443,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x438, 0x43d, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x442, 0x435, 0x20, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x41f,
0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x430,
0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x43d,
0x442, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a,
0x435, 0x43b, 0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x43d, 0x442,
0x20, 0x41f, 0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435,
0x43b, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x458, 0x43e, 0x43d, 0x433,
0x458, 0x430, 0x43d, 0x433, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x420, 0x435, 0x458, 0x443, 0x43d, 0x438, 0x43e, 0x43d, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430, 0x43a,
0x430, 0x43b, 0x438, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x421, 0x430, 0x43a, 0x430, 0x43b, 0x438, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430,
0x43a, 0x430, 0x43b, 0x438, 0x43d, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430, 0x43c,
0x43e, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430,
0x43c, 0x43e, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x458, 0x448,
0x435, 0x43b, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d,
0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x443, 0x436, 0x43d,
0x430, 0x20, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430,
0x43c, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430,
0x458, 0x43e, 0x432, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x422, 0x430, 0x445, 0x438, 0x442, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x430, 0x45f, 0x438, 0x43a,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x443, 0x443, 0x43a, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435,
0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x440, 0x443, 0x433,
0x432, 0x430, 0x458, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x458, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423,
0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x41b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x435, 0x43d, 0x435, 0x446, 0x443,
0x435, 0x43b, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x43e, 0x442, 0x20, 0x412, 0x435, 0x458, 0x43a, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x430, 0x43b, 0x438, 0x441, 0x20,
0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x430, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x443, 0x43a, 0x43e, 0x43d, 0x905,
0x92b, 0x917, 0x93e, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x92b, 0x94d, 0x930,
0x940, 0x915, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c,
0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x905, 0x92b, 0x94d, 0x930,
0x940, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x94d, 0x930,
0x940, 0x915, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x93e,
0x938, 0x94d, 0x915, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x938, 0x92e, 0x92f, 0x905, 0x92e, 0x947, 0x91c, 0x928, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x905, 0x92e, 0x947, 0x91c, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x92e, 0x947, 0x91c, 0x928, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930, 0x947, 0x92c, 0x93f, 0x92f,
0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930, 0x947, 0x92c, 0x93f, 0x92f,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905,
0x930, 0x947, 0x92c, 0x93f, 0x92f, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902,
0x91f, 0x940, 0x928, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930, 0x94d,
0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f,
0x940, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c,
0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947,
0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928,
0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x930, 0x94d, 0x92e,
0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e,
0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x927, 0x94d,
0x92f, 0x20, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92e, 0x927,
0x94d, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x908, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92e, 0x927, 0x94d,
0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c,
0x940, 0x20, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x942, 0x930, 0x92c, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a,
0x942, 0x930, 0x92c, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c,
0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x91c, 0x930, 0x92c, 0x948,
0x91c, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x90f, 0x91c, 0x94b, 0x930, 0x94d, 0x938,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x90f, 0x91c, 0x94b, 0x930, 0x94d, 0x938, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x90f, 0x91c, 0x94b,
0x930, 0x94d, 0x938, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x902, 0x917, 0x932, 0x93e,
0x926, 0x947, 0x936, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92d, 0x942, 0x91f, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94b,
0x932, 0x940, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c,
0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d,
0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d, 0x930, 0x942, 0x928, 0x947, 0x908,
0x20, 0x926, 0x93e, 0x930, 0x947, 0x938, 0x932, 0x93e, 0x92e, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x92c, 0x930, 0x94d, 0x921, 0x947, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x92c, 0x930, 0x94d, 0x921,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x947,
0x92a, 0x20, 0x92c, 0x930, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x91a, 0x947, 0x925, 0x92e, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x91a, 0x947, 0x925, 0x92e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x91a, 0x947, 0x925, 0x92e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x940, 0x928, 0x940, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x91a, 0x940, 0x928, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x924,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x940, 0x928, 0x940, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94d, 0x930,
0x93f, 0x938, 0x92e, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x915, 0x94b, 0x938, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x932, 0x902, 0x92c,
0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x932, 0x902,
0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94d, 0x92f, 0x942, 0x92c,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94d,
0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x924, 0x93f,
0x92e, 0x94b, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908, 0x938, 0x94d, 0x91f,
0x930, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902,
0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908,
0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x915, 0x94d, 0x935,
0x93e, 0x921, 0x94b, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x930, 0x926,
0x930, 0x2d, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x94d, 0x928, 0x20, 0x92f, 0x942,
0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b,
0x93e, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902,
0x921, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915, 0x932, 0x948, 0x902,
0x921, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915, 0x932, 0x948, 0x902,
0x921, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x94d, 0x930, 0x947, 0x902, 0x91a, 0x20,
0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b,
0x93c, 0x94d, 0x930, 0x93e, 0x902, 0x938, 0x940, 0x938, 0x940, 0x20, 0x926, 0x915,
0x94d, 0x937, 0x93f, 0x923, 0x940, 0x20, 0x915, 0x94d, 0x937, 0x947, 0x924, 0x94d,
0x930, 0x20, 0x906, 0x20, 0x905, 0x902, 0x91f, 0x93e, 0x930, 0x94d, 0x915, 0x91f,
0x93f, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x917, 0x948, 0x932, 0x93e, 0x92a,
0x948, 0x917, 0x94b, 0x938, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91c, 0x93e, 0x930,
0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91c, 0x93e,
0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x91c, 0x93e, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e,
0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930,
0x92c, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921,
0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x917, 0x932, 0x94d,
0x92b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x935,
0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f, 0x942, 0x91f, 0x93f, 0x92f, 0x928,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932,
0x94d, 0x92f, 0x942, 0x91f, 0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932,
0x94d, 0x92f, 0x942, 0x91f, 0x93f, 0x92f, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x93e, 0x902, 0x917, 0x20,
0x915, 0x93e, 0x902, 0x917, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x93e, 0x902,
0x917, 0x20, 0x915, 0x93e, 0x902, 0x917, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x93e, 0x902, 0x917, 0x20, 0x915, 0x93e, 0x902,
0x917, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x94b,
0x92c, 0x94d, 0x926, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x94b, 0x92c, 0x94d,
0x926, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939,
0x94b, 0x92c, 0x94d, 0x926, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x939, 0x93f, 0x902, 0x926, 0x20, 0x92e, 0x939, 0x93e, 0x938, 0x93e, 0x917,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x902, 0x921, 0x94b, 0x91a, 0x93e,
0x907, 0x928, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x907, 0x902,
0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x907, 0x902, 0x921,
0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x908, 0x930, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908, 0x930, 0x93e,
0x928, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908, 0x930,
0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x907, 0x930, 0x915, 0x942, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x930, 0x915, 0x942, 0x924, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907,
0x930, 0x915, 0x942, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x91c, 0x930, 0x93e, 0x92f, 0x932, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x91c, 0x930, 0x93e, 0x92f, 0x932, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x91c, 0x930, 0x93e,
0x92f, 0x932, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c,
0x20, 0x915, 0x91c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x915, 0x91c,
0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915,
0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94d, 0x930,
0x947, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x915, 0x94d, 0x930, 0x947, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x915, 0x94d, 0x930, 0x947, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x915, 0x93f, 0x930, 0x94d, 0x917, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x932, 0x93e, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x92c, 0x947, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x932, 0x93e, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x92c,
0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x932,
0x93e, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x92c, 0x947, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x917, 0x93e,
0x926, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x917, 0x93e, 0x926,
0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92e, 0x932, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x932, 0x926, 0x940, 0x935, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92e, 0x93e, 0x930, 0x940, 0x936, 0x938, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92e, 0x93e, 0x930, 0x940, 0x936, 0x938, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x930, 0x940, 0x936, 0x938, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x948, 0x915, 0x94d,
0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x948, 0x915, 0x94d, 0x938, 0x93f, 0x915,
0x928, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x948, 0x915, 0x94d, 0x938,
0x93f, 0x915, 0x928, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909,
0x932, 0x928, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909,
0x932, 0x928, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x932, 0x928, 0x92c, 0x91f, 0x94b, 0x930, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x938, 0x94d,
0x915, 0x94b, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x938, 0x94d, 0x915,
0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e,
0x93e, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92e, 0x92f, 0x928, 0x92e, 0x93e, 0x930, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x928, 0x947, 0x92a, 0x93e, 0x932, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928,
0x94d, 0x92f, 0x942, 0x20, 0x915, 0x947, 0x932, 0x948, 0x921, 0x94b, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915,
0x947, 0x932, 0x948, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915,
0x947, 0x932, 0x948, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942,
0x91c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928,
0x94d, 0x92f, 0x942, 0x91c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x91c,
0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93e, 0x909,
0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928,
0x94d, 0x92f, 0x942, 0x92b, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d,
0x92f, 0x942, 0x92b, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b,
0x93e, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915, 0x932, 0x948, 0x902, 0x921,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x930, 0x94d, 0x928, 0x947, 0x902, 0x921,
0x94b, 0x20, 0x921, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x930, 0x94d, 0x928, 0x947, 0x902,
0x921, 0x94b, 0x20, 0x921, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d,
0x939, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92b, 0x930, 0x94d, 0x928, 0x947, 0x902, 0x921, 0x94b, 0x20, 0x921, 0x940, 0x20,
0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x93f, 0x92c,
0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92a, 0x928,
0x94b, 0x92c, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94b,
0x92c, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x913, 0x92e, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x913, 0x92e, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x913,
0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a,
0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x930,
0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a,
0x947, 0x930, 0x942, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93f, 0x932, 0x93f, 0x92a,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938,
0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x906, 0x20,
0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947,
0x20, 0x906, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x947,
0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x906, 0x20, 0x92e,
0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x94d, 0x92f, 0x94b,
0x902, 0x917, 0x92f, 0x93e, 0x902, 0x917, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x930,
0x93f, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x938, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938,
0x915, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x938, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20,
0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x938,
0x92e, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x936, 0x947, 0x932, 0x94d,
0x938, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a,
0x941, 0x930, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x91c, 0x93e, 0x930, 0x94d, 0x91c,
0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x941, 0x930, 0x940,
0x928, 0x93e, 0x92e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x93e, 0x907, 0x92a,
0x947, 0x908, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x93e, 0x907, 0x92a, 0x947,
0x908, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924,
0x93e, 0x907, 0x92a, 0x947, 0x908, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947,
0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909,
0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909,
0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x91c, 0x92c, 0x947,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x909, 0x91c, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x91c, 0x92c,
0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x947, 0x928, 0x947, 0x91c, 0x941, 0x90f,
0x932, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d, 0x932, 0x93e, 0x926,
0x93f, 0x92c, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92c, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92c, 0x94b, 0x938, 0x94d, 0x924,
0x94b, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92c, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92c, 0x94b, 0x938, 0x94d, 0x924, 0x94b,
0x915, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94b,
0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92c, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e,
0x926, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c,
0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x20, 0x938,
0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x915, 0x941, 0x924,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938,
0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x91f,
0x948, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x91f, 0x948, 0x930, 0x93f, 0x928, 0x92c, 0x930,
0x94d, 0x917, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92f, 0x947, 0x915, 0x93e, 0x91f, 0x948, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d,
0x917, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x942,
0x915, 0x94b, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0xd0e, 0xd2f, 0xd4d, 0xd15,
0xd4d, 0xd15, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd0e, 0xd2f, 0xd4d, 0xd15,
0xd4d, 0xd15, 0xd7c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd0e, 0xd2f, 0xd4d,
0xd15, 0xd4d, 0xd15, 0xd7c, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd2b, 0xd4d, 0x200c, 0xd17,
0xd3e, 0xd28, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2e, 0xd27, 0xd4d, 0xd2f, 0x20, 0xd06, 0xd2b, 0xd4d, 0xd30, 0xd3f, 0xd15,
0xd4d, 0xd15, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d,
0xd15, 0xd7b, 0x20, 0xd06, 0xd2b, 0xd4d, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd26, 0xd15, 0xd4d, 0xd37, 0xd3f, 0xd23, 0xd3e, 0xd2b,
0xd4d, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd06, 0xd2b,
0xd4d, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a,
0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd06, 0xd2b, 0xd4d,
0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a,
0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd06, 0xd2b, 0xd4d,
0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd32,
0xd3e, 0xd38, 0xd4d, 0x200c, 0xd15, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd32,
0xd3e, 0xd38, 0xd4d, 0xd15, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd32,
0xd3e, 0xd38, 0xd4d, 0x200c, 0xd15, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7d, 0xd2e, 0xd24, 0xd3f,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7d, 0xd2e, 0xd24, 0xd3f, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7d, 0xd2e, 0xd24, 0xd3f, 0x20, 0xd35, 0xd47,
0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd06, 0xd2e, 0xd38, 0xd4b, 0xd7a, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd06, 0xd2e,
0xd38, 0xd4b, 0xd7a, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd06, 0xd2e, 0xd38,
0xd4b, 0xd7a, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15,
0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f,
0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46,
0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd38,
0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d,
0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b,
0x20, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30,
0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15,
0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48,
0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15,
0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd2e, 0xd4c, 0xd23, 0xd4d, 0xd1f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30,
0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2e, 0xd57, 0xd23, 0xd4d, 0xd1f, 0xd7b,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46,
0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2e,
0xd57, 0xd23, 0xd4d, 0xd1f, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15,
0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd2a, 0xd38, 0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35,
0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15,
0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a, 0xd38, 0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05,
0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a, 0xd38, 0xd2b,
0xd3f, 0xd15, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd28, 0xd3e, 0xd21, 0xd3f, 0xd7c, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd28, 0xd3e, 0xd21, 0xd3f, 0xd7c, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd28, 0xd3e, 0xd21, 0xd3f, 0xd7c, 0x20, 0xd35,
0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd05, 0xd2a, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd2a,
0xd3f, 0xd2f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd2a, 0xd3f, 0xd2f,
0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd05, 0xd16, 0xd4d, 0xd24, 0xd4c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd16, 0xd4d, 0xd24, 0xd4c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05,
0xd16, 0xd4d, 0xd24, 0xd4c, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd16, 0xd4d, 0xd24, 0xd4b,
0xd2c, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd16, 0xd4d, 0xd24, 0xd4b,
0xd2c, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd16, 0xd4d, 0xd24,
0xd4b, 0xd2c, 0xd4d, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31, 0xd47, 0xd2c, 0xd4d, 0xd2f,
0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31, 0xd47, 0xd2c, 0xd4d, 0xd2f,
0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31, 0xd47, 0xd2c, 0xd4d,
0xd2f, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40, 0xd28,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40,
0xd28, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d,
0xd31, 0xd40, 0xd28, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e,
0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31,
0xd40, 0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d,
0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40,
0xd28, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d,
0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40,
0xd28, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd2e, 0xd47, 0xd28, 0xd3f,
0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd2e, 0xd47, 0xd28, 0xd3f,
0xd2f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd2e, 0xd47, 0xd28,
0xd3f, 0xd2f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31, 0xd4d, 0xd31, 0xd4d,
0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd05, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d,
0xd31, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31,
0xd4d, 0xd31, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd3f, 0xd15, 0xd4d,
0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd13, 0xd38,
0xd4d, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd13, 0xd38, 0xd4d, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b,
0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f,
0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd21, 0xd47,
0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13,
0xd38, 0xd4d, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38,
0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d,
0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d,
0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b,
0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e,
0xd31, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0xd1f,
0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f,
0xd4d, 0xd30, 0xd7d, 0x20, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31,
0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd13,
0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32,
0xd3f, 0xd2f, 0xd7b, 0x20, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30,
0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b,
0x20, 0xd13, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30,
0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e,
0xd3e, 0xd31, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d,
0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd2a, 0xd1f,
0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48,
0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38, 0xd7c,
0xd2c, 0xd48, 0xd1c, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38,
0xd7c, 0xd2c, 0xd48, 0xd1c, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd38, 0xd7c, 0xd2c, 0xd48, 0xd1c, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd05, 0xd38, 0xd4b, 0xd7c, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd38, 0xd4b, 0xd7c, 0xd38, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd38, 0xd4b, 0xd7c, 0xd38, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c,
0xd02, 0xd17, 0xd4d, 0xd32, 0xd3e, 0xd26, 0xd47, 0xd36, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2c, 0xd02, 0xd17, 0xd4d, 0xd32, 0xd3e, 0xd26, 0xd47, 0xd36, 0xd4d,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd02, 0xd17, 0xd4d, 0xd32, 0xd3e,
0xd26, 0xd47, 0xd36, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2d, 0xd42, 0xd1f,
0xd4d, 0xd1f, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4a, 0xd33,
0xd40, 0xd35, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4d, 0xd30,
0xd38, 0xd40, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4d,
0xd30, 0xd38, 0xd40, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2c, 0xd4d, 0xd30, 0xd38, 0xd40, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2c, 0xd4d, 0xd30, 0xd42, 0xd23, 0xd48, 0x20, 0xd26, 0xd3e, 0xd31, 0xd41,
0xd38, 0xd4d, 0xd38, 0xd32, 0xd3e, 0xd02, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15,
0xd47, 0xd2a, 0xd4d, 0x20, 0xd35, 0xd46, 0xd7c, 0xd26, 0xd46, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd15, 0xd47, 0xd2a, 0xd4d, 0x20, 0xd35, 0xd46, 0xd7c, 0xd26, 0xd46,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd47, 0xd2a, 0xd4d, 0x20, 0xd35,
0xd46, 0xd7c, 0xd26, 0xd46, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd2e, 0xd4b,
0xd31, 0xd4b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd3e, 0xd24, 0xd4d,
0xd24, 0xd02, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd3e, 0xd24, 0xd4d, 0xd24,
0xd02, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd3e, 0xd24, 0xd4d, 0xd24,
0xd02, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd3f, 0xd32, 0xd3f, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd3f, 0xd32, 0xd3f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd1a, 0xd3f, 0xd32, 0xd3f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd48,
0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd48, 0xd28, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd48, 0xd28, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd30, 0xd3f,
0xd38, 0xd4d, 0x200c, 0xd2e, 0xd38, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd15, 0xd4d, 0xd15, 0xd4b,
0xd38, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd33, 0xd02, 0xd2c, 0xd3f, 0xd2f, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd33, 0xd02, 0xd2c, 0xd3f, 0xd2f, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd33, 0xd02, 0xd2c, 0xd3f, 0xd2f,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd41, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20,
0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd15, 0xd41, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40,
0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd41,
0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15,
0xd7e, 0x20, 0xd05, 0xd7c, 0xd26, 0xd4d, 0xd27, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd15, 0xd4d, 0xd2f, 0xd42, 0xd2c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d,
0xd2f, 0xd42, 0xd2c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd2f,
0xd42, 0xd2c, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd21, 0xd47, 0xd35, 0xd3f, 0xd38, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd21, 0xd41, 0xd2e, 0xd4b, 0xd23, 0xd4d, 0xd1f, 0xd4d, 0x20,
0xd21, 0xd3f, 0x20, 0xd09, 0xd7c, 0xd35, 0xd3f, 0xd32, 0xd4d, 0xd32, 0xd46, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd24, 0xd3f, 0xd2e, 0xd4b, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd08, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd7c, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd08, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd7c,
0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd08, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd7c, 0x20, 0xd26, 0xd4d, 0xd35,
0xd40, 0xd2a, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd15, 0xd4d, 0xd35,
0xd21, 0xd4b, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd7b, 0xd1f,
0xd4d, 0xd30, 0xd7d, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d,
0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd2f,
0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15,
0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a,
0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15,
0xd4d, 0xd15, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd47, 0xd2f, 0xd31, 0xd4d,
0xd31, 0xd24, 0xd4d, 0xd24, 0xd46, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d,
0xd2f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d,
0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f,
0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e,
0xd3e, 0xd31, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e,
0xd3e, 0xd31, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c,
0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41,
0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3e, 0xd15, 0xd4d, 0xd15,
0xd4d, 0x200c, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40,
0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3e,
0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd26,
0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2b, 0xd3f, 0xd1c, 0xd3f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd1c,
0xd3f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd1c, 0xd3f, 0x20,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd4d, 0xd30, 0xd1e, 0xd4d, 0xd1a, 0xd4d, 0x20,
0xd17, 0xd2f, 0xd3e, 0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd4d, 0xd30,
0xd1e, 0xd4d, 0xd1a, 0xd4d, 0x20, 0xd38, 0xd24, 0xd47, 0xd7a, 0x2c, 0x20, 0xd05,
0xd28, 0xd4d, 0xd31, 0xd3e, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd15, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17, 0xd3e, 0xd32, 0xd2a, 0xd4d, 0xd2a, 0xd17, 0xd4b,
0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17, 0xd3e, 0xd2e, 0xd4d, 0xd2a,
0xd3f, 0xd2f, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c, 0xd4b, 0xd7c, 0xd1c,
0xd4d, 0xd1c, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c, 0xd4b, 0xd7c,
0xd1c, 0xd4d, 0xd1c, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c,
0xd4b, 0xd7c, 0xd1c, 0xd4d, 0xd1c, 0xd3f, 0xd2f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd17, 0xd3f, 0xd7d, 0xd2c, 0xd47, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0x20, 0xd26,
0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd35, 0xd3f, 0xd1a, 0xd4d, 0xd1a, 0xd4d, 0x20,
0xd2e, 0xd40, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15,
0xd4d, 0xd15, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e, 0xd7b,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d,
0xd15, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e, 0xd7b, 0xd21,
0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d,
0xd15, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e, 0xd7b, 0xd21,
0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e,
0xd3e, 0xd31, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e, 0xd7b,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d,
0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e,
0xd7b, 0xd21, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f,
0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b,
0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17, 0xd4d,
0xd35, 0xd3e, 0xd02, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17, 0xd7e, 0xd2b,
0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd35, 0xd3e, 0xd2f, 0xd4d,
0x2d, 0xd05, 0xd32, 0xd42, 0xd37, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd39, 0xd35, 0xd3e, 0xd2f, 0xd4d, 0x2d, 0xd05, 0xd32, 0xd42, 0xd37, 0xd4d,
0xd2f, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd35, 0xd3e, 0xd2f,
0xd4d, 0x2d, 0xd05, 0xd32, 0xd42, 0xd37, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd21, 0xd47,
0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39,
0xd4b, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd19, 0xd4d, 0xd19, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd39, 0xd4b, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd19, 0xd4d, 0xd19, 0xd4d,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd19, 0xd4d, 0xd15, 0xd4b,
0xd19, 0xd4d, 0xd19, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd21, 0xd4d, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd21, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd2e, 0xd39,
0xd3e, 0xd38, 0xd2e, 0xd41, 0xd26, 0xd4d, 0xd30, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd07, 0xd7b, 0xd21, 0xd4b, 0xd1a, 0xd48, 0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2e, 0xd27, 0xd4d, 0xd2f, 0x20, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4b, 0xd28, 0xd47,
0xd37, 0xd4d, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15,
0xd4d, 0xd15, 0xd7b, 0x20, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4b, 0xd28, 0xd47, 0xd37,
0xd4d, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d,
0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4b, 0xd28, 0xd47,
0xd37, 0xd4d, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd31, 0xd3e, 0xd7b,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd31, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd07, 0xd31, 0xd3e, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48,
0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd7c, 0xd15,
0xd38, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd7c, 0xd15,
0xd38, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd7c,
0xd15, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07,
0xd38, 0xd4d, 0xd30, 0xd3e, 0xd2f, 0xd47, 0xd7d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd07, 0xd38, 0xd4d, 0xd30, 0xd3e, 0xd2f, 0xd47, 0xd7d, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd07, 0xd38, 0xd4d, 0xd30, 0xd3e, 0xd2f, 0xd47, 0xd7d, 0x20, 0xd21,
0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd1c, 0xd2a, 0xd4d, 0xd2a, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c,
0xd2a, 0xd4d, 0xd2a, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c,
0xd2a, 0xd4d, 0xd2a, 0xd3e, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd1f, 0xd4d, 0xd30,
0xd4b, 0xd2a, 0xd3e, 0xd35, 0xd4d, 0x200c, 0xd32, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4d,
0x20, 0xd15, 0xd02, 0xd1a, 0xd3e, 0xd38, 0xd4d, 0xd15, 0xd3f, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2a, 0xd46, 0xd1f, 0xd4d, 0xd30, 0xd4b, 0xd2a, 0xd3e, 0xd35, 0xd4d,
0x200c, 0xd32, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd15, 0xd02, 0xd1a, 0xd3e,
0xd38, 0xd4d, 0xd15, 0xd3f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46,
0xd1f, 0xd4d, 0xd30, 0xd4b, 0xd2a, 0xd3e, 0xd35, 0xd4d, 0x200c, 0xd32, 0xd4b, 0xd38,
0xd4d, 0xd15, 0xd4d, 0x20, 0xd15, 0xd02, 0xd1a, 0xd3e, 0xd38, 0xd4d, 0xd15, 0xd3f,
0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd15, 0xd38, 0xd3e, 0xd16, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e,
0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd15, 0xd38, 0xd3e, 0xd16, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e,
0xd31, 0xd7b, 0x20, 0xd15, 0xd38, 0xd3e, 0xd16, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e,
0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd31, 0xd3f, 0xd2f, 0xd7b,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd31, 0xd3f, 0xd2f, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd31, 0xd3f, 0xd2f, 0xd7b, 0x20,
0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd15, 0xd4a, 0xd38, 0xd4d, 0xd30, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15,
0xd4d, 0xd30, 0xd3e, 0xd38, 0xd4d, 0x200c, 0xd28, 0xd4b, 0xd2f, 0xd3e, 0xd7c, 0xd38,
0xd4d, 0x200c, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd30,
0xd3e, 0xd38, 0xd4d, 0x200c, 0xd28, 0xd4b, 0xd2f, 0xd3e, 0xd7c, 0xd38, 0xd4d, 0x200c,
0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd30, 0xd3e,
0xd38, 0xd4d, 0x200c, 0xd28, 0xd4b, 0xd2f, 0xd3e, 0xd7c, 0xd38, 0xd4d, 0x200c, 0xd15,
0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd7c, 0xd17, 0xd3f, 0xd38,
0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32, 0xd19, 0xd4d,
0xd15, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32, 0xd48, 0xd7b, 0x20, 0xd26, 0xd4d,
0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32,
0xd4b, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd39, 0xd4b, 0xd35, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd32, 0xd4b, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd39, 0xd4b, 0xd35, 0xd4d,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32, 0xd4b, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd39, 0xd4b, 0xd35, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd15, 0xd4c, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2e, 0xd15, 0xd4c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e,
0xd15, 0xd4c, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd17, 0xd3e, 0xd26, 0xd7b, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd17, 0xd3e, 0xd26, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2e, 0xd17, 0xd3e, 0xd26, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2e, 0xd32, 0xd47, 0xd37, 0xd4d, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e,
0xd3e, 0xd32, 0xd3f, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2e, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd38, 0xd38, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd3e, 0xd7c, 0xd37, 0xd7d, 0x20, 0xd26, 0xd4d, 0xd35,
0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd57,
0xd31, 0xd40, 0xd37, 0xd4d, 0xd2f, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2e, 0xd57, 0xd31, 0xd40, 0xd37, 0xd4d, 0xd2f, 0xd38, 0xd4d, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd57, 0xd31, 0xd40, 0xd37, 0xd4d, 0xd2f, 0xd38, 0xd4d,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd3e, 0xd38, 0xd7a, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2e, 0xd46, 0xd15, 0xd4d, 0xd38, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b,
0x20, 0xd2a, 0xd38, 0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2e, 0xd46, 0xd15, 0xd4d, 0x200c, 0xd38, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd2a, 0xd38, 0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2e, 0xd46, 0xd15, 0xd4d, 0xd38, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a,
0xd38, 0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd32, 0xd3e, 0xd7b, 0xd2c,
0xd3e, 0xd24, 0xd4d, 0xd24, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd32,
0xd3e, 0xd7b, 0xd2c, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd7c, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd09, 0xd32, 0xd3e, 0xd7b, 0xd2c, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd7c,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4b, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4b, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4b, 0x20,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd4d, 0xd2f, 0xd3e, 0xd7b, 0xd2e, 0xd3e, 0xd7c,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd57, 0xd31, 0xd41, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd28, 0xd47, 0xd2a, 0xd4d, 0xd2a, 0xd3e, 0xd7e, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0x20, 0xd15, 0xd3e, 0xd32, 0xd3f, 0xd21,
0xd4b, 0xd23, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f,
0xd42, 0x20, 0xd15, 0xd3e, 0xd32, 0xd3f, 0xd21, 0xd4b, 0xd23, 0xd3f, 0xd2f, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0x20, 0xd15, 0xd3e,
0xd32, 0xd3f, 0xd21, 0xd4b, 0xd23, 0xd3f, 0xd2f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd28, 0xd4d, 0xd2f, 0xd42, 0xd38, 0xd3f, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd38, 0xd3f, 0xd32, 0xd3e,
0xd7b, 0xd21, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f,
0xd42, 0xd38, 0xd3f, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32,
0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d,
0xd2f, 0xd42, 0xd2b, 0xd57, 0xd23, 0xd4d, 0xd1f, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42,
0xd2b, 0xd57, 0xd23, 0xd4d, 0xd1f, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd2b,
0xd57, 0xd23, 0xd4d, 0xd1f, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd4d,
0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd2f, 0xd3f, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd28, 0xd4b, 0xd7c, 0x200c, 0xd2b, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20,
0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28,
0xd4b, 0xd7c, 0x200c, 0xd2b, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d,
0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b,
0xd7c, 0x200c, 0xd2b, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35,
0xd40, 0xd2a, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd46, 0xd7c, 0xd23, 0xd3e, 0xd21, 0xd4b,
0x20, 0xd21, 0xd3f, 0x20, 0xd28, 0xd4a, 0xd31, 0xd4b, 0xd7b, 0xd39, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd46, 0xd7c, 0xd23, 0xd3e, 0xd21, 0xd4b, 0x20, 0xd21,
0xd3f, 0x20, 0xd28, 0xd4a, 0xd31, 0xd4b, 0xd7b, 0xd39, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2b, 0xd46, 0xd7c, 0xd23, 0xd3e, 0xd21, 0xd4b, 0x20, 0xd21, 0xd3f,
0x20, 0xd28, 0xd4a, 0xd31, 0xd4b, 0xd7b, 0xd39, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd28, 0xd4b, 0xd7c, 0xd24, 0xd4d, 0xd24, 0xd4d, 0x20, 0xd2e, 0xd31, 0xd3f, 0xd2f,
0xd3e, 0xd28, 0xd3e, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b, 0xd35, 0xd4b, 0xd38, 0xd3f, 0xd2c,
0xd3f, 0xd7c, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd28, 0xd4b, 0xd35, 0xd4b, 0xd38, 0xd3f, 0xd2c, 0xd3f, 0xd7c, 0xd38, 0xd4d, 0xd15,
0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b, 0xd35, 0xd4b, 0xd38,
0xd3f, 0xd2c, 0xd3f, 0xd7c, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd13, 0xd02, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd02, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15,
0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd02, 0xd38, 0xd4d, 0x200c,
0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3e, 0xd15,
0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2a, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd3f,
0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd32,
0xd3e, 0xd35, 0xd41, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3e, 0xd2a, 0xd4d,
0xd2a, 0xd41, 0xd35, 0x20, 0xd28, 0xd4d, 0xd2f, 0xd42, 0x20, 0xd17, 0xd3f, 0xd28,
0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd30, 0xd3e, 0xd17, 0xd4d,
0xd35, 0xd47, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd30, 0xd3e, 0xd17, 0xd4d,
0xd35, 0xd47, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd30, 0xd3e, 0xd17,
0xd4d, 0xd35, 0xd47, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd31, 0xd41,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd31, 0xd41, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd31, 0xd41, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2b, 0xd3f, 0xd32, 0xd3f, 0xd2a, 0xd4d, 0xd2a, 0xd48, 0xd7b, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd32, 0xd3f, 0xd2a, 0xd4d, 0xd2a, 0xd48, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd32, 0xd3f, 0xd2a, 0xd4d, 0xd2a,
0xd48, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd28, 0xd3f, 0xd15,
0xd4d, 0x200c, 0xd38, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd2a,
0xd3f, 0xd2f, 0xd31, 0xd3f, 0x20, 0xd06, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd2e, 0xd3f,
0xd15, 0xd4d, 0xd35, 0xd32, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46,
0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd2a, 0xd3f, 0xd2f, 0xd31, 0xd3f, 0x20, 0xd06,
0xd7b, 0xd21, 0xd4d, 0x20, 0xd2e, 0xd3f, 0xd15, 0xd4d, 0xd35, 0xd32, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20,
0xd2a, 0xd3f, 0xd2f, 0xd31, 0xd3f, 0x20, 0xd06, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd2e,
0xd3f, 0xd15, 0xd4d, 0xd35, 0xd32, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3f, 0xd31, 0xd4d,
0xd31, 0xd4d, 0xd15, 0xd47, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd4a,
0xd28, 0xd3e, 0xd2a, 0xd4d, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a,
0xd4d, 0xd2f, 0xd4b, 0xd02, 0xd17, 0xd4d, 0x200c, 0xd2f, 0xd3e, 0xd02, 0xd17, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd16, 0xd3f, 0xd38, 0xd3f, 0xd32, 0xd4b, 0xd7c,
0xd21, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd16, 0xd3f, 0xd38, 0xd3f, 0xd32, 0xd4b,
0xd7c, 0xd21, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd16, 0xd3f, 0xd38, 0xd3f,
0xd32, 0xd4b, 0xd7c, 0xd21, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd31, 0xd40, 0xd2f, 0xd42, 0xd23,
0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd31, 0xd4b, 0xd25, 0xd46,
0xd31, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0x20,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd2e, 0xd3e, 0xd30, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd38, 0xd2e, 0xd3e, 0xd31, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38,
0xd2e, 0xd3e, 0xd31, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd2e, 0xd4b, 0xd35, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd38, 0xd2e, 0xd4b, 0xd35, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd38, 0xd2e, 0xd4b, 0xd35, 0xd3e, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38,
0xd40, 0xd37, 0xd46, 0xd7d, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38,
0xd3f, 0xd02, 0xd17, 0xd2a, 0xd4d, 0xd2a, 0xd42, 0xd7c, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd38, 0xd4b, 0xd33, 0xd2e, 0xd7b, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40,
0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd26, 0xd15, 0xd4d,
0xd37, 0xd3f, 0xd23, 0x20, 0xd1c, 0xd4b, 0xd7c, 0xd1c, 0xd4d, 0xd1c, 0xd3f, 0xd2f,
0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd41, 0xd30, 0xd3f, 0xd28, 0xd46,
0xd2f, 0xd3f, 0xd02, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd2f, 0xd4b, 0xd35,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd39, 0xd3f, 0xd24, 0xd3f, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd2f, 0xd4d, 0x200c, 0xd2a, 0xd46, 0xd2f,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd2f, 0xd4d, 0x200c, 0xd2a,
0xd46, 0xd2f, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd2f,
0xd4d, 0x200c, 0xd2a, 0xd46, 0xd2f, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd1c, 0xd3f,
0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd1f, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd46, 0xd32, 0xd42, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd1f, 0xd4b, 0xd02, 0xd17, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd1f, 0xd4b, 0xd02, 0xd17, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1f, 0xd4b,
0xd02, 0xd17, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd42, 0xd15, 0xd4d, 0xd15,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd41, 0xd7c, 0xd15, 0xd4d, 0xd15,
0xd4d, 0x200c, 0xd2e, 0xd46, 0xd28, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd41, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c,
0xd2e, 0xd46, 0xd28, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd24, 0xd41, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c, 0xd2e,
0xd46, 0xd28, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd1f, 0xd41, 0xd35, 0xd3e, 0xd32, 0xd41, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd09, 0xd31, 0xd41, 0xd17, 0xd4d, 0xd35, 0xd47, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd09, 0xd31, 0xd41, 0xd17, 0xd4d, 0xd35, 0xd47, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd09, 0xd31, 0xd41, 0xd17, 0xd4d, 0xd35, 0xd47, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd09, 0xd38, 0xd4d, 0x200c, 0xd2c, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38,
0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd38, 0xd4d,
0x200c, 0xd2c, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd38, 0xd4d, 0x200c, 0xd2c, 0xd46,
0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd35, 0xd28, 0xd4d, 0xd35, 0xd3e, 0xd24, 0xd41, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd35, 0xd28, 0xd4d, 0xd35, 0xd3e, 0xd24, 0xd41, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd35, 0xd28, 0xd4d, 0xd35, 0xd3e, 0xd24, 0xd41, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd35, 0xd46, 0xd28, 0xd3f, 0xd38, 0xd4d, 0xd35, 0xd47, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd4d, 0xd32, 0xd3e, 0xd21, 0xd3f, 0xd35,
0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd35, 0xd4d, 0xd32, 0xd3e, 0xd21, 0xd3f, 0xd35, 0xd4b, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd35, 0xd4d, 0xd32, 0xd3e, 0xd21, 0xd3f, 0xd35, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd4b, 0xd15, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd4b, 0xd7e,
0xd17, 0xd4b, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd35, 0xd4b, 0xd7e, 0xd17, 0xd4b, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd4b, 0xd7e, 0xd17, 0xd4b, 0xd17,
0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd47,
0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd3e, 0xd32, 0xd3f, 0xd38, 0xd4d, 0x20, 0xd06,
0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd2b, 0xd4d, 0xd2f, 0xd42, 0xd1a, 0xd4d, 0xd2f,
0xd41, 0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd3e, 0xd15, 0xd38, 0xd4d,
0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd3e,
0xd15, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2f, 0xd3e, 0xd15, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd31,
0xd4d, 0xd31, 0xd30, 0xd3f, 0xd7b, 0xd2c, 0xd7c, 0xd17, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2f, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd30,
0xd3f, 0xd7b, 0xd2c, 0xd7c, 0xd17, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2f, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd30, 0xd3f, 0xd7b,
0xd2c, 0xd7c, 0xd17, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd42, 0xd15,
0xd4d, 0xd15, 0xd4b, 0xd7a, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x63,
0x72, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x73, 0x61,
0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x50, 0x75, 0x73, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x72, 0x67, 0x75, 0x6e,
0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x50, 0x65, 0x72, 0x67, 0x75, 0x6e,
0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x50, 0x65, 0x72, 0x67, 0x75, 0x6e, 0x75,
0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e,
0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e,
0x67, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x54, 0x69,
0x6d, 0x75, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61,
0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67,
0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74,
0x6d, 0x61, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x65, 0x70,
0x61, 0x72, 0x75, 0x68, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75,
0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x43, 0x75, 0x62, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x43, 0x75, 0x62, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x43, 0x75,
0x62, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x75, 0x6c, 0x61,
0x75, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20,
0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61,
0x68, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x45, 0x72, 0x6f, 0x70,
0x61, 0x68, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x54, 0x69, 0x6d,
0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70,
0x61, 0x68, 0x20, 0x63, 0x65, 0x72, 0x75, 0x6b, 0x20, 0x74, 0x69, 0x6d,
0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70,
0x61, 0x68, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x45, 0x72, 0x6f,
0x70, 0x61, 0x68, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x42, 0x61,
0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61,
0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x20, 0x50, 0x65, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x73,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x72, 0x61, 0x6e, 0x63,
0x69, 0x73, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x20, 0x64,
0x61, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x6b, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69,
0x63, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x54,
0x65, 0x6c, 0x75, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4c, 0x61, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x48, 0x69, 0x6e,
0x64, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e,
0x67, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4a, 0x65, 0x70, 0x75, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4a, 0x65, 0x70, 0x75,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67,
0x20, 0x4a, 0x65, 0x70, 0x75, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70,
0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x61, 0x63, 0x61, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x77, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x77, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61,
0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4b, 0x65,
0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61,
0x6e, 0x67, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69,
0x61, 0x6e, 0x61, 0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61,
0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64,
0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x6c, 0x61,
0x75, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x4d, 0x61, 0x73, 0x61, 0x20, 0x59, 0x75, 0x6b,
0x6f, 0x6e, 0x648, 0x642, 0x62a, 0x648, 0x20, 0x628, 0x631, 0x648, 0x646, 0x64a,
0x20, 0x62f, 0x627, 0x631, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x648, 0x642,
0x62a, 0x648, 0x20, 0x6a4, 0x64a, 0x627, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x64a,
0x646, 0x62f, 0x64a, 0x627, 0x648, 0x642, 0x62a, 0x648, 0x20, 0x644, 0x627, 0x674,
0x648, 0x62a, 0x646, 0x20, 0x647, 0x64a, 0x646, 0x62f, 0x64a, 0x648, 0x642, 0x62a,
0x648, 0x20, 0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a, 0x633, 0x64a, 0x627, 0x20,
0x62a, 0x6a0, 0x647, 0x648, 0x642, 0x62a, 0x648, 0x20, 0x625, 0x646, 0x62f, 0x648,
0x646, 0x64a, 0x633, 0x64a, 0x627, 0x20, 0x62a, 0x64a, 0x645, 0x648, 0x631, 0x648,
0x642, 0x62a, 0x648, 0x20, 0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a, 0x633, 0x64a,
0x627, 0x20, 0x628, 0x627, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x62a, 0x648, 0x20,
0x645, 0x644, 0x64a, 0x633, 0x64a, 0x627, 0x126, 0x69, 0x6e, 0x20, 0x10a, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70,
0x65, 0x77, 0x126, 0x69, 0x6e, 0x20, 0x10a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x65, 0x77, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x126, 0x69, 0x6e, 0x20, 0x10a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x20, 0x45, 0x77, 0x72, 0x6f,
0x70, 0x65, 0x77, 0x20, 0x74, 0x61, 0x73, 0x2d, 0x53, 0x61, 0x6a, 0x66,
0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2,
0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985,
0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9c7, 0x9a8, 0x9cd,
0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd,
0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0,
0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9a6, 0x9c7,
0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0,
0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20,
0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae,
0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995,
0x9be, 0x9a8, 0x20, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9b7,
0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be,
0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0,
0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8,
0x20, 0x9a6, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae,
0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995,
0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x989, 0x9a8, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x20,
0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae,
0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x989, 0x9a8, 0x9cd,
0x99f, 0x9c7, 0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0,
0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5,
0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be,
0x989, 0x9a8, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x20, 0x9a6, 0x9c7, 0x9b2, 0x9be, 0x987,
0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20,
0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9c7, 0x9b8,
0x9bf, 0x9ab, 0x9bf, 0x995, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0,
0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20,
0x9aa, 0x9c7, 0x9b8, 0x9bf, 0x9ab, 0x9bf, 0x995, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7,
0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8,
0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be,
0x9a8, 0x20, 0x9aa, 0x9c7, 0x9b8, 0x9bf, 0x9ab, 0x9bf, 0x995, 0x20, 0x9a6, 0x9c7,
0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x985, 0x99f, 0x9b2,
0x9be, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x985,
0x99f, 0x9b2, 0x9be, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x9b7, 0x9cd, 0x99f,
0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae,
0x985, 0x99f, 0x9b2, 0x9be, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x9a6, 0x9c7,
0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9b8, 0x9c7, 0x9a8,
0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb,
0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9b8,
0x9c7, 0x9a8, 0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9af, 0x9bc, 0x9c1,
0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f,
0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae,
0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9af, 0x9bc,
0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9b0, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd,
0x9a8, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd,
0x9a8, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6,
0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8,
0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9b0, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9f1, 0x9c7, 0x9b7,
0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9f1, 0x9c7,
0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb,
0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8,
0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9f1, 0x9c7,
0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb,
0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9b0, 0x20, 0x99f,
0x9be, 0x987, 0x9ae, 0x997, 0x9cd, 0x9b0, 0x9bf, 0x9a8, 0x9f1, 0x9bf, 0x99a, 0x20,
0x9ae, 0x9bf, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x57, 0x101, 0x20, 0x41,
0x77, 0x68, 0x65, 0x6b, 0x65, 0x6e, 0x65, 0x74, 0x101, 0x6e, 0x61, 0x57,
0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x50, 0x75, 0x6b, 0x75, 0x20,
0x6f, 0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x101,
0x20, 0x6f, 0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x72, 0x101, 0x77, 0x68, 0x69, 0x74,
0x69, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x20, 0x6f, 0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x74, 0x6f, 0x6e, 0x67, 0x61,
0x57, 0x101, 0x20, 0x6f, 0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57,
0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20,
0x6f, 0x20, 0x100, 0x77, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x69,
0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x6f, 0x20, 0x100, 0x77, 0x68, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75,
0x72, 0x75, 0x57, 0x101, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57,
0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61,
0x74, 0x65, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x101,
0x20, 0x41, 0x6d, 0x61, 0x68, 0x6f, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x41,
0x6d, 0x61, 0x68, 0x6f, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x6d, 0x61, 0x68, 0x6f,
0x6e, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101,
0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x57, 0x61, 0x65, 0x6e,
0x67, 0x61, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20,
0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x52, 0x101, 0x77,
0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69,
0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x52, 0x101,
0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x75, 0x6e,
0x67, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x20, 0x4d, 0x61, 0x75, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20,
0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x6e, 0x67,
0x61, 0x57, 0x101, 0x20, 0x4b, 0x69, 0x77, 0x61, 0x57, 0x101, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x4b, 0x69, 0x77,
0x61, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x4b,
0x69, 0x77, 0x61, 0x57, 0x101, 0x20, 0x100, 0x70, 0x69, 0x61, 0x57, 0x101,
0x20, 0x100, 0x70, 0x69, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x100, 0x70, 0x69, 0x61, 0x20, 0x41,
0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x101, 0x70,
0x69, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x101, 0x70, 0x69, 0x61, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x41, 0x72, 0x101, 0x70, 0x69, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65,
0x61, 0x57, 0x101, 0x20, 0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x57,
0x101, 0x20, 0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x100, 0x6b,
0x65, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74,
0x69, 0x57, 0x101, 0x20, 0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20,
0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74,
0x65, 0x20, 0x75, 0x72, 0x75, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x20,
0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x100, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x57, 0x101, 0x20, 0x100, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x100, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x52, 0x61, 0x75,
0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x74,
0x69, 0x6b, 0x69, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x74, 0x69, 0x6b, 0x69,
0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x52, 0x61,
0x6e, 0x61, 0x74, 0x69, 0x6b, 0x69, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69,
0x74, 0x65, 0x72, 0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65,
0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72,
0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72, 0x69, 0x61,
0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74,
0x65, 0x61, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65,
0x69, 0x72, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x2d,
0x55, 0x72, 0x75, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72,
0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61,
0x2d, 0x55, 0x72, 0x75, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65,
0x69, 0x72, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x2d,
0x55, 0x72, 0x75, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101,
0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72, 0x69, 0x61,
0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69,
0x74, 0x69, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65,
0x69, 0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x52,
0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65,
0x72, 0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65,
0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x20, 0x41, 0x77, 0x61,
0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72,
0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20,
0x55, 0x72, 0x75, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72,
0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20,
0x55, 0x72, 0x75, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69,
0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x55, 0x72,
0x75, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x41,
0x74, 0x65, 0x70, 0x61, 0x69, 0x68, 0x101, 0x6e, 0x69, 0x61, 0x57, 0x101,
0x20, 0x41, 0x74, 0x65, 0x70, 0x61, 0x69, 0x68, 0x101, 0x6e, 0x69, 0x61,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x41, 0x74, 0x65, 0x70, 0x61, 0x69, 0x68, 0x101, 0x6e, 0x69, 0x61,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x101, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x101, 0x6b, 0x61,
0x72, 0x61, 0x74, 0x113, 0x68, 0x69, 0x57, 0x101, 0x20, 0x50, 0x101, 0x6b,
0x61, 0x72, 0x61, 0x74, 0x113, 0x68, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x101, 0x6b, 0x61,
0x72, 0x61, 0x74, 0x113, 0x68, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x16b, 0x74, 0x61, 0x6e, 0x61, 0x57,
0x101, 0x20, 0x50, 0x6f, 0x72, 0x69, 0x77, 0x69, 0x61, 0x57, 0x101, 0x20,
0x50, 0x61, 0x72, 0x12b, 0x68, 0x69, 0x61, 0x57, 0x101, 0x20, 0x50, 0x61,
0x72, 0x12b, 0x68, 0x69, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x61, 0x72, 0x12b, 0x68, 0x69,
0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20,
0x50, 0x6f, 0x72, 0x6f, 0x6e, 0x61, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75,
0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54,
0x65, 0x20, 0x4b, 0x16b, 0x72, 0x61, 0x65, 0x20, 0x4d, 0x61, 0x74, 0x6f,
0x6d, 0x61, 0x74, 0x6f, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4b, 0x16b,
0x72, 0x61, 0x65, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x6d, 0x61, 0x74, 0x6f,
0x57, 0x101, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x6f,
0x20, 0x54, 0x65, 0x20, 0x4b, 0x16b, 0x72, 0x61, 0x65, 0x20, 0x4d, 0x61,
0x74, 0x6f, 0x6d, 0x61, 0x74, 0x6f, 0x57, 0x101, 0x20, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x52, 0x113, 0x6b, 0x6f, 0x68, 0x75,
0x57, 0x101, 0x20, 0x52, 0x113, 0x6b, 0x6f, 0x68, 0x75, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x52, 0x113,
0x6b, 0x6f, 0x68, 0x75, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57,
0x101, 0x20, 0x48, 0x69, 0x72, 0x69, 0x57, 0x101, 0x20, 0x48, 0x69, 0x72,
0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x48, 0x69, 0x72, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x48, 0x61, 0x69, 0x6e, 0x61, 0x57, 0x101,
0x20, 0x48, 0x61, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x48, 0x61, 0x69, 0x6e, 0x61,
0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x6f, 0x20,
0x54, 0x65, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x4b,
0x69, 0x72, 0x69, 0x68, 0x69, 0x6d, 0x65, 0x74, 0x65, 0x57, 0x101, 0x20,
0x6f, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72,
0x65, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x68, 0x69, 0x57, 0x101, 0x20, 0x4b,
0x6f, 0x72, 0x6f, 0x6d, 0x14d, 0x70, 0x69, 0x61, 0x57, 0x101, 0x20, 0x4b,
0x6f, 0x72, 0x6f, 0x6d, 0x14d, 0x70, 0x69, 0x61, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x6f, 0x72,
0x6f, 0x6d, 0x14d, 0x70, 0x69, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20, 0x41, 0x69,
0x72, 0x61, 0x6e, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20,
0x41, 0x69, 0x72, 0x61, 0x6e, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20,
0x41, 0x69, 0x72, 0x61, 0x6e, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x20, 0x48, 0x61, 0x75, 0x72, 0x75, 0x61, 0x57, 0x101, 0x20,
0x4b, 0x69, 0x75, 0x70, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x4b, 0x69, 0x75, 0x70, 0x61, 0x57,
0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x4b, 0x69, 0x75,
0x70, 0x61, 0x57, 0x101, 0x20, 0x52, 0x113, 0x77, 0x65, 0x74, 0x69, 0x57,
0x101, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54,
0x69, 0x6d, 0x6f, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x52,
0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x6b, 0x69, 0x20,
0x74, 0x65, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x6f,
0x20, 0x41, 0x72, 0x61, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x6b, 0x69,
0x20, 0x74, 0x65, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20,
0x41, 0x72, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65,
0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x41, 0x72, 0x61,
0x6e, 0x67, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x45, 0x6b, 0x75, 0x61, 0x74, 0x6f, 0x61, 0x57, 0x101, 0x20,
0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61,
0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x20, 0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67,
0x61, 0x57, 0x101, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20,
0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61,
0x57, 0x101, 0x20, 0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x52, 0x101, 0x77,
0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x20, 0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x52,
0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x52, 0x61, 0x75,
0x6d, 0x61, 0x74, 0x69, 0x20, 0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x52,
0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x16a, 0x72, 0x6f,
0x70, 0x69, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x72, 0x101, 0x77,
0x68, 0x69, 0x74, 0x69, 0x20, 0x72, 0x61, 0x77, 0x61, 0x57, 0x101, 0x20,
0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x55, 0x72, 0x75, 0x57, 0x101, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x55, 0x72,
0x6f, 0x70, 0x69, 0x20, 0x55, 0x72, 0x75, 0x57, 0x101, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x55, 0x72, 0x6f, 0x70, 0x69, 0x20,
0x55, 0x72, 0x75, 0x57, 0x101, 0x20, 0x6b, 0x69, 0x20, 0x4e, 0x67, 0x101,
0x20, 0x4d, 0x6f, 0x74, 0x75, 0x20, 0x57, 0x68, 0x101, 0x6b, 0x61, 0x6e,
0x61, 0x57, 0x101, 0x20, 0x6b, 0x69, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d,
0x6f, 0x74, 0x75, 0x20, 0x57, 0x68, 0x101, 0x6b, 0x61, 0x6e, 0x61, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x6b, 0x69, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x74, 0x75, 0x20,
0x57, 0x68, 0x101, 0x6b, 0x61, 0x6e, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x12b, 0x74, 0x12b, 0x57,
0x101, 0x20, 0x57, 0x68, 0x12b, 0x74, 0x12b, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x12b, 0x74,
0x12b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20,
0x4b, 0x69, 0x101, 0x6e, 0x61, 0x20, 0x57, 0x12b, 0x77, 0x12b, 0x57, 0x101,
0x20, 0x57, 0x12b, 0x77, 0x12b, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x20, 0x6d, 0x65, 0x20, 0x74, 0x65, 0x20, 0x4b,
0x14d, 0x70, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x6b,
0x69, 0x20, 0x74, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x101,
0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x57, 0x101,
0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x57, 0x101, 0x20, 0x48,
0x14d, 0x72, 0x69, 0x61, 0x57, 0x101, 0x20, 0x48, 0x14d, 0x72, 0x69, 0x61,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x48, 0x14d, 0x72, 0x69, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x70, 0x61, 0x74,
0x69, 0x57, 0x101, 0x20, 0x54, 0x6f, 0x68, 0x61, 0x72, 0x69, 0x74, 0x65,
0x20, 0x4b, 0x69, 0x72, 0x69, 0x77, 0x12b, 0x74, 0x69, 0x57, 0x101, 0x20,
0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69,
0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x72, 0x101, 0x77, 0x68, 0x69,
0x74, 0x69, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x20, 0x6f, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x6b,
0x101, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20,
0x72, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x6f, 0x20, 0x57, 0x68, 0x65, 0x6e,
0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6b, 0x69, 0x20,
0x74, 0x65, 0x20, 0x72, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101,
0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b,
0x69, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57,
0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20,
0x6f, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72, 0x69,
0x6b, 0x69, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75,
0x57, 0x101, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x6f,
0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b,
0x69, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57,
0x101, 0x20, 0x57, 0x68, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x61, 0x69,
0x61, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x101, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x57, 0x101, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x69, 0x70, 0x75,
0x61, 0x57, 0x101, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x69, 0x70, 0x75, 0x61,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x69, 0x70, 0x75, 0x61, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x57, 0x101, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x12a,
0x6e, 0x69, 0x61, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4d,
0x6f, 0x61, 0x6e, 0x61, 0x20, 0x12a, 0x6e, 0x69, 0x61, 0x57, 0x101, 0x20,
0x12a, 0x6e, 0x69, 0x61, 0x68, 0x61, 0x69, 0x6e, 0x61, 0x57, 0x101, 0x20,
0x49, 0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x12b, 0x68, 0x69, 0x61, 0x20, 0x57,
0x61, 0x65, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x49, 0x6e, 0x69, 0x74,
0x6f, 0x6e, 0x12b, 0x68, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65,
0x20, 0x72, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x49,
0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x12b, 0x68, 0x69, 0x61, 0x20, 0x6b, 0x69,
0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x49, 0x72,
0x101, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x49, 0x72, 0x101, 0x6e, 0x61, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x49, 0x72, 0x101, 0x6e, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61,
0x57, 0x101, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x101,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x49, 0x68, 0x61, 0x72, 0x61, 0x69, 0x72, 0x61, 0x57,
0x101, 0x20, 0x49, 0x68, 0x61, 0x72, 0x61, 0x69, 0x72, 0x61, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x49,
0x68, 0x61, 0x72, 0x61, 0x69, 0x72, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74,
0x65, 0x61, 0x57, 0x101, 0x20, 0x48, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x57,
0x101, 0x20, 0x48, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x48, 0x61, 0x70,
0x61, 0x6e, 0x69, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101,
0x20, 0x4b, 0x61, 0x74, 0x61, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x57, 0x101,
0x20, 0x4b, 0x61, 0x74, 0x61, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x20, 0x6b,
0x69, 0x20, 0x74, 0x65, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69,
0x57, 0x101, 0x20, 0x4b, 0x61, 0x74, 0x61, 0x74, 0x101, 0x6e, 0x67, 0x61,
0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x55, 0x72, 0x75, 0x57, 0x101,
0x20, 0x4b, 0x14d, 0x72, 0x65, 0x61, 0x57, 0x101, 0x20, 0x4b, 0x14d, 0x72,
0x65, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x4b, 0x14d, 0x72, 0x65, 0x61, 0x20, 0x41, 0x77, 0x61,
0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0x57, 0x101, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x57, 0x101, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x4b, 0x69, 0x6b, 0x69, 0x74, 0x101, 0x6e, 0x67, 0x61,
0x57, 0x101, 0x20, 0x6f, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x75,
0x74, 0x65, 0x72, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x52, 0x61, 0x69,
0x6e, 0x61, 0x57, 0x101, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x57, 0x101, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20,
0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61,
0x72, 0x65, 0x69, 0x61, 0x57, 0x101, 0x20, 0x4d, 0x101, 0x72, 0x61, 0x74,
0x69, 0x72, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65,
0x73, 0x61, 0x73, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x4e, 0x67, 0x101, 0x20,
0x4d, 0x6f, 0x74, 0x75, 0x20, 0x4d, 0x101, 0x68, 0x61, 0x72, 0x61, 0x57,
0x101, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x68, 0x69, 0x57, 0x101, 0x20, 0x4d,
0x61, 0x72, 0x69, 0x68, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x68, 0x69,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4d,
0x61, 0x77, 0x73, 0x6f, 0x6e, 0x57, 0x101, 0x20, 0x4d, 0x113, 0x68, 0x69,
0x6b, 0x6f, 0x20, 0x4b, 0x69, 0x77, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x4d, 0x113, 0x68, 0x69,
0x6b, 0x6f, 0x20, 0x4b, 0x69, 0x77, 0x61, 0x57, 0x101, 0x20, 0x41, 0x77,
0x61, 0x74, 0x65, 0x61, 0x20, 0x4d, 0x113, 0x68, 0x69, 0x6b, 0x6f, 0x20,
0x4b, 0x69, 0x77, 0x61, 0x57, 0x101, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x57, 0x101, 0x20, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x52, 0x61, 0x75,
0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x6f, 0x68, 0x69, 0x6b,
0x61, 0x75, 0x57, 0x101, 0x20, 0x4d, 0x6f, 0x68, 0x69, 0x6b, 0x61, 0x75,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x4d, 0x6f, 0x68, 0x69, 0x6b, 0x61, 0x75, 0x20, 0x52, 0x61, 0x75,
0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x113, 0x6d, 0x61, 0x57,
0x101, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x4e, 0x65,
0x70, 0x14d, 0x72, 0x61, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75,
0x61, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x6b, 0x69, 0x57, 0x101, 0x20, 0x57,
0x68, 0x65, 0x6e, 0x75, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x6b, 0x69,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x61,
0x6b, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101,
0x20, 0x41, 0x6f, 0x74, 0x65, 0x61, 0x72, 0x6f, 0x61, 0x57, 0x101, 0x20,
0x41, 0x6f, 0x74, 0x65, 0x61, 0x72, 0x6f, 0x61, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x6f, 0x74,
0x65, 0x61, 0x72, 0x6f, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61,
0x57, 0x101, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65,
0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x57, 0x101, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x57, 0x101, 0x20,
0x6f, 0x20, 0x54, 0x65, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65,
0x20, 0x4e, 0x14d, 0x70, 0x6f, 0x6b, 0x65, 0x57, 0x101, 0x20, 0x6f, 0x20,
0x54, 0x65, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x4e,
0x14d, 0x70, 0x6f, 0x6b, 0x65, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4d,
0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x4e, 0x14d, 0x70, 0x6f, 0x6b,
0x65, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x101, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x101,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x57, 0x101, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x74, 0x101, 0x6e, 0x65, 0x57, 0x101,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x74, 0x101, 0x6e, 0x65, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x74, 0x101, 0x6e, 0x65, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x101, 0x72, 0x61, 0x75, 0x57, 0x101,
0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x16b, 0x6b, 0x69, 0x6e,
0x69, 0x57, 0x101, 0x20, 0x50, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x69, 0x57,
0x101, 0x20, 0x50, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x69, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x61,
0x72, 0x61, 0x6b, 0x61, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74,
0x69, 0x57, 0x101, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x101, 0x20, 0x50,
0x65, 0x72, 0x75, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x50, 0x65, 0x72, 0x75, 0x20, 0x52, 0x61, 0x75,
0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x69, 0x72, 0x69, 0x70,
0x12b, 0x6e, 0x69, 0x57, 0x101, 0x20, 0x50, 0x69, 0x72, 0x69, 0x70, 0x12b,
0x6e, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x50, 0x69, 0x72, 0x69, 0x70, 0x12b, 0x6e, 0x69, 0x20,
0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x6f, 0x20,
0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x57, 0x101, 0x20, 0x53, 0x74,
0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6d, 0x65, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x101, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x6f, 0x20, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6d, 0x65,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x101, 0x20,
0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x6f, 0x20, 0x53, 0x74, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6d, 0x65, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x101, 0x20, 0x50, 0x69,
0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x57, 0x101, 0x20, 0x50, 0x6f, 0x6e,
0x61, 0x70, 0x65, 0x57, 0x101, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x57, 0x101, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x57, 0x101, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x57,
0x101, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x101,
0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x48, 0x101, 0x6d, 0x6f, 0x61, 0x57, 0x101,
0x20, 0x48, 0x101, 0x6d, 0x6f, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x48, 0x101, 0x6d, 0x6f, 0x61,
0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x48, 0x65,
0x69, 0x6b, 0x65, 0x72, 0x65, 0x57, 0x101, 0x20, 0x48, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x6f, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d,
0x6f, 0x74, 0x75, 0x20, 0x48, 0x6f, 0x72, 0x6f, 0x6d, 0x6f, 0x6e, 0x61,
0x57, 0x101, 0x20, 0x48, 0x14d, 0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20,
0x74, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x48,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x57, 0x101, 0x20, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x57, 0x101, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69,
0x57, 0x101, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x101, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x54,
0x61, 0x6b, 0x69, 0x72, 0x69, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x57, 0x101,
0x20, 0x54, 0x6f, 0x6b, 0x65, 0x72, 0x61, 0x75, 0x57, 0x101, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x57, 0x101,
0x20, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e, 0x61, 0x74, 0x101, 0x6e,
0x67, 0x61, 0x57, 0x101, 0x20, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e,
0x61, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x54, 0x75, 0x6b, 0x75, 0x6d,
0x61, 0x6e, 0x61, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x20, 0x52, 0x61, 0x75,
0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x54, 0x16b, 0x77, 0x61, 0x72,
0x75, 0x57, 0x101, 0x20, 0x55, 0x72, 0x75, 0x6b, 0x6f, 0x69, 0x57, 0x101,
0x20, 0x55, 0x72, 0x75, 0x6b, 0x6f, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x55, 0x72, 0x75, 0x6b,
0x6f, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101,
0x20, 0x55, 0x68, 0x69, 0x70, 0x65, 0x6b, 0x65, 0x74, 0x101, 0x6e, 0x65,
0x57, 0x101, 0x20, 0x55, 0x68, 0x69, 0x70, 0x65, 0x6b, 0x65, 0x74, 0x101,
0x6e, 0x65, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x55, 0x68, 0x69, 0x70, 0x65, 0x6b, 0x65, 0x74, 0x101,
0x6e, 0x65, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101,
0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x74, 0x16b, 0x57, 0x101, 0x20,
0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x74, 0x16b, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65,
0x6e, 0x75, 0x61, 0x74, 0x16b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74,
0x69, 0x57, 0x101, 0x20, 0x50, 0x65, 0x6e, 0x65, 0x68, 0x16b, 0x65, 0x72,
0x61, 0x57, 0x101, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x57, 0x101, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74,
0x69, 0x57, 0x101, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x57, 0x101, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20,
0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20,
0x4d, 0x6f, 0x74, 0x75, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x57, 0x101, 0x20,
0x57, 0x101, 0x72, 0x69, 0x68, 0x69, 0x20, 0x6d, 0x65, 0x20, 0x57, 0x68,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x57, 0x101, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x101, 0x20,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x90f, 0x915, 0x930, 0x20, 0x935,
0x947, 0x933, 0x90f, 0x915, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x935, 0x947, 0x933, 0x910, 0x915, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x92b, 0x917, 0x93e, 0x923, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d,
0x200d, 0x92f, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x906, 0x92b, 0x94d, 0x930,
0x93f, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x926, 0x915, 0x94d, 0x937, 0x93f,
0x923, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x906, 0x92b, 0x94d,
0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x906, 0x92b,
0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933,
0x940, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905,
0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f,
0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932,
0x94d, 0x92e, 0x93e, 0x91f, 0x940, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932, 0x94d,
0x92e, 0x93e, 0x91f, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935,
0x947, 0x933, 0x905, 0x932, 0x94d, 0x92e, 0x93e, 0x91f, 0x940, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935,
0x947, 0x933, 0x905, 0x200d, 0x945, 0x92e, 0x947, 0x91d, 0x949, 0x928, 0x20, 0x935,
0x947, 0x933, 0x905, 0x200d, 0x945, 0x92e, 0x947, 0x91d, 0x949, 0x928, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945,
0x92e, 0x947, 0x91d, 0x949, 0x928, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933,
0x940, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x938, 0x942, 0x930,
0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94c, 0x930,
0x94d, 0x935, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f,
0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930,
0x94d, 0x935, 0x924, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x94d, 0x935, 0x924, 0x940, 0x92f, 0x20,
0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x945, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x945, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x945, 0x938, 0x93f,
0x92b, 0x93f, 0x915, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930,
0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x90f, 0x928, 0x93e, 0x921, 0x940,
0x92f, 0x930, 0x20, 0x935, 0x947, 0x933, 0x905, 0x928, 0x93e, 0x926, 0x940, 0x930,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x905, 0x928,
0x93e, 0x926, 0x940, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x90f, 0x92a, 0x93f,
0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x90f, 0x92a, 0x93f, 0x92f, 0x93e, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x90f, 0x92a,
0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930,
0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x915, 0x94d,
0x924, 0x93e, 0x909, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x915, 0x94d,
0x924, 0x93e, 0x909, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947,
0x933, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x924, 0x93e, 0x909, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935,
0x947, 0x933, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x20,
0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x947,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x905, 0x200d,
0x945, 0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933,
0x905, 0x930, 0x947, 0x92c, 0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905,
0x930, 0x947, 0x92c, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x947, 0x92c, 0x93f, 0x92f, 0x928,
0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936,
0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f,
0x928, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947,
0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f,
0x93f, 0x928, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930,
0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e,
0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x935, 0x947,
0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91f, 0x932, 0x93e,
0x902, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d,
0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20,
0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20,
0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x2d, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x2d, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x2d, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930,
0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935,
0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x942, 0x930, 0x94d,
0x935, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e,
0x936, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20,
0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x928, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915,
0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91d, 0x930, 0x92c, 0x948, 0x91c,
0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91d, 0x930, 0x92c, 0x948, 0x91c,
0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x905, 0x91d, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945,
0x91d, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945,
0x91d, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x91d, 0x94b, 0x930, 0x947, 0x938,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x92c, 0x93e, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917,
0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933,
0x940, 0x20, 0x935, 0x947, 0x933, 0x92d, 0x942, 0x924, 0x93e, 0x928, 0x20, 0x935,
0x947, 0x933, 0x92c, 0x94d, 0x930, 0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92c,
0x94d, 0x930, 0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930,
0x941, 0x928, 0x947, 0x908, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938, 0x932, 0x93e,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x94d, 0x939,
0x930, 0x94d, 0x921, 0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20,
0x935, 0x94d, 0x939, 0x930, 0x94d, 0x921, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x94d,
0x939, 0x930, 0x94d, 0x921, 0x947, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933,
0x940, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945,
0x925, 0x92e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x91a, 0x945, 0x925, 0x92e, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a,
0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x93f, 0x932,
0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x91a, 0x93f, 0x932, 0x940, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940,
0x20, 0x935, 0x947, 0x933, 0x91a, 0x940, 0x928, 0x940, 0x20, 0x935, 0x947, 0x933,
0x91a, 0x940, 0x928, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x91a, 0x940, 0x928, 0x940, 0x20, 0x938, 0x942, 0x930, 0x94d,
0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x916,
0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x935,
0x947, 0x933, 0x915, 0x949, 0x915, 0x94b, 0x938, 0x20, 0x92c, 0x947, 0x91f, 0x947,
0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915,
0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939,
0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x915, 0x941, 0x915, 0x20, 0x92c,
0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x941, 0x915, 0x20, 0x92c,
0x947, 0x91f, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x915, 0x941, 0x915, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x905,
0x930, 0x94d, 0x927, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e,
0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936,
0x20, 0x935, 0x947, 0x933, 0x921, 0x947, 0x935, 0x94d, 0x939, 0x93f, 0x938, 0x20,
0x935, 0x947, 0x933, 0x921, 0x94d, 0x92f, 0x941, 0x92e, 0x949, 0x928, 0x94d, 0x91f,
0x2d, 0x921, 0x94d, 0x92f, 0x941, 0x930, 0x94d, 0x935, 0x93f, 0x932, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x924, 0x93f, 0x92e, 0x94b,
0x930, 0x20, 0x935, 0x947, 0x933, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x92c,
0x947, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20,
0x92c, 0x947, 0x91f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x92c, 0x947, 0x91f, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x927, 0x94d, 0x200d, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f,
0x928, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x200d, 0x92f, 0x20, 0x92f,
0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x200d, 0x92f, 0x20,
0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d,
0x935, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x92f, 0x941, 0x930,
0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933,
0x940, 0x20, 0x935, 0x947, 0x933, 0x905, 0x917, 0x94d, 0x930, 0x2d, 0x92a, 0x94c,
0x930, 0x94d, 0x935, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x92f, 0x941,
0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20,
0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915, 0x932,
0x902, 0x921, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x92b,
0x949, 0x915, 0x932, 0x902, 0x921, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915,
0x932, 0x902, 0x921, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x91c, 0x940,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92b,
0x93f, 0x91c, 0x940, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x92b, 0x94d, 0x930, 0x947, 0x902, 0x91a, 0x20, 0x926, 0x915,
0x94d, 0x937, 0x93f, 0x923, 0x20, 0x906, 0x923, 0x93f, 0x20, 0x905, 0x902, 0x91f,
0x93e, 0x930, 0x94d, 0x915, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933,
0x917, 0x945, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x20, 0x935, 0x947,
0x933, 0x917, 0x945, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x20, 0x935, 0x947,
0x933, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x949, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940,
0x20, 0x935, 0x947, 0x933, 0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930, 0x94d, 0x91f,
0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x93f, 0x91a, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942, 0x930,
0x94d, 0x935, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942,
0x930, 0x94d, 0x935, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x917, 0x941, 0x906, 0x92e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935,
0x947, 0x933, 0x916, 0x93e, 0x921, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932,
0x942, 0x936, 0x928, 0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d,
0x905, 0x932, 0x942, 0x936, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932, 0x942,
0x936, 0x928, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915,
0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915,
0x93e, 0x901, 0x917, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x939,
0x94b, 0x935, 0x94d, 0x939, 0x94d, 0x921, 0x20, 0x935, 0x947, 0x933, 0x939, 0x94b,
0x935, 0x94d, 0x939, 0x94d, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x939, 0x94b, 0x935, 0x94d, 0x939, 0x94d, 0x921, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92d,
0x93e, 0x930, 0x924, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x939, 0x93f, 0x902, 0x926, 0x20, 0x92e, 0x939, 0x93e,
0x938, 0x93e, 0x917, 0x930, 0x20, 0x935, 0x947, 0x933, 0x907, 0x902, 0x921, 0x94b,
0x91a, 0x93e, 0x92f, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d,
0x200d, 0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f,
0x93e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x93e, 0x924,
0x94d, 0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f,
0x93e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20,
0x935, 0x947, 0x933, 0x907, 0x930, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x907,
0x930, 0x93e, 0x923, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x907, 0x930, 0x93e, 0x923, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f,
0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930,
0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x935, 0x947, 0x933, 0x907,
0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x94d, 0x915, 0x941,
0x924, 0x94d, 0x938, 0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940,
0x20, 0x935, 0x947, 0x933, 0x907, 0x938, 0x94d, 0x930, 0x93e, 0x92f, 0x932, 0x907,
0x938, 0x94d, 0x930, 0x93e, 0x92f, 0x932, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x907, 0x938, 0x94d, 0x930, 0x93e, 0x92f, 0x932,
0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936,
0x20, 0x935, 0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20,
0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935,
0x94d, 0x939, 0x932, 0x94b, 0x938, 0x94d, 0x915, 0x2d, 0x20, 0x915, 0x93e, 0x92e,
0x91a, 0x93e, 0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x940, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935, 0x94d, 0x939, 0x932,
0x94b, 0x938, 0x94d, 0x915, 0x2d, 0x20, 0x915, 0x93e, 0x92e, 0x91a, 0x93e, 0x91f,
0x94d, 0x938, 0x94d, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x935, 0x947, 0x933, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935,
0x94d, 0x939, 0x932, 0x94b, 0x938, 0x94d, 0x915, 0x2d, 0x20, 0x915, 0x93e, 0x92e,
0x91a, 0x93e, 0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x915, 0x91d, 0x93e, 0x915, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x915, 0x91d, 0x93e, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x930,
0x93f, 0x92f, 0x928, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930,
0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x938, 0x94d, 0x930,
0x93e, 0x908, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d,
0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e,
0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x915, 0x94d, 0x938, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x915,
0x93f, 0x930, 0x917, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947,
0x933, 0x932, 0x902, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x932, 0x93e, 0x907,
0x928, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x932, 0x949,
0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x932, 0x949, 0x930, 0x94d, 0x921,
0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a,
0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x915, 0x93e,
0x90a, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x915, 0x93e, 0x90a, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x92e, 0x915, 0x93e, 0x90a, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x935, 0x947, 0x933, 0x92e, 0x945, 0x917, 0x93e, 0x921, 0x928, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x945, 0x917, 0x93e, 0x921, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x945, 0x917, 0x93e, 0x921,
0x928, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x93e, 0x932, 0x926, 0x93f, 0x935, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x93e, 0x930, 0x94d, 0x915, 0x94d, 0x935, 0x947, 0x938, 0x93e, 0x938, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x93e, 0x930, 0x94d, 0x936, 0x932, 0x20, 0x92c, 0x947, 0x91f,
0x947, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20,
0x935, 0x947, 0x933, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x930, 0x940,
0x936, 0x938, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x92a,
0x945, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x947,
0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x92a, 0x945, 0x938, 0x93f, 0x92b,
0x93f, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x92a, 0x945,
0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a,
0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932, 0x93e,
0x928, 0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x935, 0x947, 0x933, 0x909,
0x932, 0x93e, 0x928, 0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932, 0x93e, 0x928,
0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e,
0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933,
0x940, 0x20, 0x935, 0x947, 0x933, 0x928, 0x909, 0x930, 0x941, 0x20, 0x935, 0x947,
0x933, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b,
0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942,
0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d,
0x92f, 0x942, 0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x928, 0x94d, 0x92f, 0x942, 0x91d, 0x940, 0x932, 0x902, 0x921, 0x20, 0x935,
0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942, 0x91d, 0x940, 0x932, 0x902, 0x921, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d,
0x92f, 0x942, 0x91d, 0x940, 0x932, 0x902, 0x921, 0x20, 0x938, 0x942, 0x930, 0x94d,
0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x928,
0x94d, 0x92f, 0x942, 0x20, 0x92b, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x902, 0x921,
0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x92b, 0x93e, 0x909,
0x902, 0x921, 0x932, 0x902, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x92b, 0x93e, 0x909,
0x902, 0x921, 0x932, 0x902, 0x921, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a,
0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949, 0x930,
0x92b, 0x94b, 0x915, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x928,
0x949, 0x930, 0x92b, 0x94b, 0x915, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949, 0x930, 0x92b,
0x94b, 0x915, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f,
0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x930,
0x94d, 0x928, 0x93e, 0x902, 0x921, 0x94b, 0x20, 0x926, 0x940, 0x20, 0x928, 0x94b,
0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x921,
0x94b, 0x20, 0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939,
0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x92e, 0x930, 0x93f, 0x928, 0x93e,
0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94b, 0x935,
0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x935,
0x947, 0x933, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x913, 0x92e, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93e, 0x92a, 0x941,
0x906, 0x20, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x945, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x945, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x945,
0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e,
0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932,
0x93f, 0x92a, 0x93e, 0x907, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932,
0x93f, 0x92a, 0x93e, 0x907, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93e, 0x907, 0x928,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x200d, 0x92b, 0x94b, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x20, 0x92c, 0x947, 0x91f,
0x947, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f,
0x92f, 0x930, 0x20, 0x906, 0x923, 0x93f, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935,
0x947, 0x932, 0x94b, 0x928, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f,
0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x947, 0x20, 0x906, 0x923, 0x93f, 0x20, 0x92e,
0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x94b, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f, 0x20,
0x92a, 0x93f, 0x92f, 0x930, 0x947, 0x20, 0x906, 0x923, 0x93f, 0x20, 0x92e, 0x93f,
0x915, 0x94d, 0x935, 0x947, 0x932, 0x94b, 0x928, 0x20, 0x938, 0x942, 0x930, 0x94d,
0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x93f, 0x91f, 0x915, 0x948, 0x930, 0x94d, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x94b, 0x928, 0x945, 0x92a, 0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x93c, 0x93f,
0x91d, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x915, 0x93c, 0x93f, 0x91d, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x915, 0x93c, 0x93f,
0x91d, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947,
0x933, 0x938, 0x916, 0x932, 0x93f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x938, 0x916, 0x932, 0x93f, 0x928, 0x20, 0x909,
0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x938, 0x92e,
0x93e, 0x930, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e, 0x930, 0x93e,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x938, 0x93e,
0x92e, 0x930, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e, 0x94b,
0x906, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a,
0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93f, 0x902,
0x917, 0x93e, 0x92a, 0x942, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x949, 0x928, 0x20,
0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x924, 0x948, 0x92a, 0x947,
0x908, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x924, 0x948, 0x92a, 0x947, 0x908, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a,
0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x924, 0x93e, 0x91c,
0x93f, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933,
0x91f, 0x94b, 0x915, 0x947, 0x932, 0x93e, 0x90a, 0x20, 0x935, 0x947, 0x933, 0x91f,
0x94b, 0x902, 0x917, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x942, 0x915, 0x20,
0x935, 0x947, 0x933, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x924, 0x941, 0x930,
0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x924, 0x941,
0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x924, 0x941, 0x935, 0x93e, 0x932, 0x942, 0x20, 0x935, 0x947, 0x933, 0x909, 0x930,
0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x909,
0x91d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935,
0x947, 0x933, 0x909, 0x91d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x909, 0x91d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x935,
0x93e, 0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x935, 0x947, 0x933, 0x935, 0x93e,
0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x935, 0x93e, 0x928, 0x941, 0x906, 0x924, 0x941, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x935,
0x94d, 0x939, 0x947, 0x928, 0x947, 0x91d, 0x941, 0x90f, 0x932, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d,
0x924, 0x94b, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d,
0x924, 0x94b, 0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917,
0x94d, 0x930, 0x93e, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917,
0x94d, 0x930, 0x93e, 0x921, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940,
0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x938, 0x94d, 0x91f, 0x949,
0x915, 0x20, 0x935, 0x947, 0x933, 0x935, 0x947, 0x915, 0x20, 0x92c, 0x947, 0x91f,
0x20, 0x935, 0x947, 0x933, 0x935, 0x949, 0x932, 0x93f, 0x938, 0x20, 0x906, 0x923,
0x93f, 0x20, 0x92b, 0x941, 0x91f, 0x941, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x935, 0x947, 0x933,
0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x93e, 0x915, 0x941, 0x924,
0x94d, 0x938, 0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x92f, 0x947, 0x915, 0x924, 0x930, 0x93f, 0x928, 0x92c, 0x930,
0x94d, 0x917, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x947, 0x915, 0x924, 0x930, 0x93f,
0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x92f, 0x947, 0x915, 0x924, 0x930, 0x93f, 0x928, 0x92c,
0x930, 0x94d, 0x917, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x92f, 0x941, 0x915, 0x94b, 0x928, 0x20, 0x935, 0x947, 0x933,
0x410, 0x444, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x4e8, 0x43c, 0x43d, 0x4e9, 0x434, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x438,
0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430,
0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x438, 0x439,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410,
0x43b, 0x44f, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x44b, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9,
0x432, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x423, 0x443, 0x43b, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x443,
0x43b, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x423, 0x443, 0x43b, 0x44b, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x43c, 0x445, 0x43e, 0x43d,
0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41d,
0x43e, 0x43c, 0x445, 0x43e, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x41d, 0x43e, 0x43c, 0x445, 0x43e, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410,
0x43f, 0x438, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410,
0x43f, 0x438, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43f, 0x438, 0x430,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x410, 0x440, 0x430, 0x431, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410,
0x440, 0x430, 0x431, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x430, 0x431, 0x44b, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430,
0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x439, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x439, 0x43d, 0x20, 0x442, 0x4e9, 0x432, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443,
0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x442,
0x4e9, 0x432, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x44d, 0x440,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x439, 0x43d, 0x20, 0x442, 0x4e9, 0x432, 0x20, 0x431, 0x430, 0x440, 0x443,
0x443, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20,
0x437, 0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437,
0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443,
0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x439, 0x43d, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x44d, 0x440,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x439, 0x43d, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x44d,
0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439,
0x436, 0x430, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439,
0x436, 0x430, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x410, 0x437, 0x43e, 0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410,
0x437, 0x43e, 0x440, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x437, 0x43e, 0x440, 0x44b, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x438, 0x439,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x438,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411,
0x443, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x43e, 0x43b,
0x438, 0x432, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x440,
0x430, 0x437, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x411, 0x440, 0x430, 0x437,
0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x430, 0x440,
0x443, 0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435,
0x440, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412,
0x435, 0x440, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x44b,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x44b, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x427, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x427, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x427, 0x438,
0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x425, 0x44f, 0x442, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x425, 0x44f, 0x442, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x425, 0x44f, 0x442, 0x430,
0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x41a, 0x440, 0x438, 0x441, 0x43c, 0x430, 0x441, 0x20, 0x430, 0x440, 0x43b, 0x44b,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x430,
0x440, 0x43b, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x44b,
0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x4af,
0x4af, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434,
0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x4af, 0x4af, 0x43a, 0x438, 0x439,
0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41a,
0x4af, 0x4af, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443,
0x434, 0x44b, 0x43d, 0x20, 0x445, 0x430, 0x433, 0x430, 0x441, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x443, 0x431, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x41a, 0x443, 0x431, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x443, 0x431,
0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x414,
0x44d, 0x432, 0x438, 0x441, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x414,
0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b,
0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20,
0x422, 0x438, 0x43c, 0x43e, 0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417,
0x4af, 0x4af, 0x43d, 0x20, 0x418, 0x441, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x418, 0x441, 0x43b,
0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x418,
0x441, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x44b,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af,
0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43b, 0x441, 0x20, 0x434, 0x43e,
0x440, 0x43d, 0x43e, 0x434, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430,
0x440, 0x443, 0x443, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x438, 0x439, 0x43d, 0x20,
0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x438, 0x439, 0x43d, 0x20,
0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x424, 0x43e, 0x43b,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b,
0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x424, 0x438, 0x436, 0x438, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x424, 0x438, 0x436, 0x438, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x424,
0x438, 0x436, 0x438, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x44b, 0x43d, 0x20,
0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x44b, 0x43d, 0x20, 0x4e8, 0x43c,
0x43d, 0x4e9, 0x434, 0x20, 0x431, 0x430, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440,
0x43a, 0x442, 0x438, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x413,
0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x44b, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x413, 0x4af, 0x440, 0x436, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x413, 0x4af, 0x440, 0x436, 0x438, 0x439, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x413,
0x4af, 0x440, 0x436, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x413, 0x438, 0x43b, 0x44c, 0x431, 0x435, 0x440, 0x442, 0x438,
0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443,
0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x435, 0x440,
0x441, 0x438, 0x439, 0x43d, 0x20, 0x431, 0x443, 0x43b, 0x430, 0x43d, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x425, 0x430, 0x432, 0x430,
0x439, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x425, 0x430, 0x432, 0x430, 0x439, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442,
0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x425, 0x430, 0x432, 0x430, 0x439, 0x2d, 0x410, 0x43b, 0x435,
0x443, 0x442, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e,
0x43d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a,
0x43e, 0x43d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x425, 0x43e, 0x432, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x425, 0x43e, 0x432, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x425, 0x43e, 0x432, 0x434,
0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x42d,
0x43d, 0x44d, 0x442, 0x445, 0x44d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x42d, 0x43d, 0x44d, 0x442, 0x445, 0x44d, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x42d, 0x43d,
0x44d, 0x442, 0x445, 0x44d, 0x433, 0x2d, 0x425, 0x44f, 0x442, 0x430, 0x434, 0x44b,
0x43d, 0x20, 0x445, 0x43e, 0x439, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x422, 0x4e9, 0x432, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437,
0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20,
0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x418, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x418,
0x440, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x418, 0x440, 0x430, 0x43d,
0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x418, 0x440, 0x430, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x42d, 0x440, 0x445, 0x4af, 0x4af, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x42d, 0x440, 0x445, 0x4af, 0x4af, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x42d, 0x440, 0x445, 0x4af, 0x4af, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x418, 0x437, 0x440,
0x430, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x418, 0x437,
0x440, 0x430, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x418, 0x437, 0x440, 0x430,
0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x42f, 0x43f, 0x43e, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x42f,
0x43f, 0x43e, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x42f, 0x43f, 0x43e, 0x43d, 0x44b, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x430, 0x437, 0x430, 0x445,
0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af,
0x43d, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x41a,
0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x421, 0x43e, 0x43b, 0x43e, 0x43d, 0x433, 0x43e, 0x441, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x421, 0x43e, 0x43b, 0x43e, 0x43d, 0x433, 0x43e, 0x441, 0x44b,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x421, 0x43e, 0x43b, 0x43e, 0x43d, 0x433, 0x43e, 0x441, 0x44b, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x43e, 0x441,
0x440, 0x44d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f,
0x440, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x437, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x41b, 0x430, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b,
0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41b, 0x43e, 0x440,
0x434, 0x20, 0x425, 0x43e, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x437, 0x44b, 0x43d, 0x20,
0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x44b, 0x43d, 0x20, 0x430, 0x440,
0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430,
0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41c,
0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430, 0x432,
0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x2d, 0x41d, 0x43e, 0x43c,
0x445, 0x43e, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x2d, 0x41d, 0x43e, 0x43c,
0x445, 0x43e, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41c,
0x435, 0x43a, 0x441, 0x438, 0x43a, 0x2d, 0x41d, 0x43e, 0x43c, 0x445, 0x43e, 0x43d,
0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x423, 0x43b, 0x430, 0x430, 0x43d, 0x431, 0x430, 0x430,
0x442, 0x430, 0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x43b, 0x430,
0x430, 0x43d, 0x431, 0x430, 0x430, 0x442, 0x430, 0x440, 0x44b, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x423,
0x43b, 0x430, 0x430, 0x43d, 0x431, 0x430, 0x430, 0x442, 0x430, 0x440, 0x44b, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x43e, 0x441,
0x43a, 0x432, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x43e,
0x441, 0x43a, 0x432, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c, 0x430, 0x440,
0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x43b, 0x431, 0x44b,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x41a, 0x430,
0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x428, 0x438, 0x43d, 0x44d, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x428, 0x438, 0x43d,
0x44d, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x428,
0x438, 0x43d, 0x44d, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x44c, 0x44e,
0x2d, 0x424, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x44b, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x41d, 0x44c, 0x44e, 0x2d, 0x424, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x44c, 0x44e, 0x2d,
0x424, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x44b, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x438, 0x443, 0x44d,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x440, 0x444,
0x43e, 0x43b, 0x43a, 0x20, 0x430, 0x440, 0x43b, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x430, 0x440, 0x43b,
0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x430,
0x440, 0x43b, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435,
0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20,
0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x430, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434,
0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x430, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x432,
0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440,
0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x41e, 0x43c, 0x441, 0x43a, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x41e, 0x43c, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x41e, 0x43c, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x43b,
0x430, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430,
0x43f, 0x443, 0x430, 0x20, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x413, 0x432, 0x438,
0x43d, 0x435, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x41f, 0x435, 0x440, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x41f, 0x435, 0x440, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x435,
0x440, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x438,
0x439, 0x20, 0x446, 0x430, 0x433, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438,
0x43d, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d,
0x438, 0x439, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x424,
0x435, 0x43d, 0x438, 0x43a, 0x441, 0x20, 0x430, 0x440, 0x43b, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435, 0x440,
0x20, 0x431, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435,
0x440, 0x20, 0x431, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20,
0x431, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x44b, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x438, 0x442, 0x43a,
0x435, 0x440, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x43e,
0x43d, 0x430, 0x43f, 0x435, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x41f, 0x451, 0x43d, 0x44c, 0x44f, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x420,
0x435, 0x44e, 0x43d, 0x438, 0x43e, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x420,
0x43e, 0x442, 0x435, 0x440, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x44b, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x421, 0x435, 0x439, 0x448,
0x435, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434,
0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f,
0x443, 0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x421, 0x43e, 0x43b, 0x43e,
0x43c, 0x43e, 0x43d, 0x44b, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x4e8, 0x43c, 0x43d, 0x4e9, 0x434, 0x20, 0x416,
0x43e, 0x440, 0x436, 0x438, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x44b, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x421, 0x451, 0x432, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x422, 0x430, 0x438, 0x442, 0x438, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x422, 0x430,
0x439, 0x43f, 0x435, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x422, 0x430, 0x436, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x43e, 0x43d, 0x433,
0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x43e, 0x43d,
0x433, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x427, 0x4af, 0x4af, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x423, 0x437, 0x431,
0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x412, 0x430, 0x43d, 0x443,
0x430, 0x442, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x412,
0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x412,
0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x412, 0x435, 0x43d, 0x435, 0x441,
0x443, 0x44d, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433,
0x440, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43e, 0x43b,
0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43e, 0x43b,
0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x44d, 0x43a, 0x438, 0x439, 0x43d,
0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x431, 0x430, 0x20, 0x424,
0x443, 0x442, 0x443, 0x43d, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x42f, 0x43a, 0x443, 0x442, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x42f,
0x43a, 0x443, 0x442, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x42f, 0x43a, 0x443, 0x442, 0x44b, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x415, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d,
0x431, 0x443, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x415, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x42e, 0x43a, 0x43e, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x431, 0x4af, 0x441, 0x1832,
0x1825, 0x182a, 0x20, 0x1834, 0x1820, 0x182d, 0x1833, 0x1825, 0x182a, 0x20, 0x1830, 0x1832,
0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820,
0x182d, 0x1832, 0x1825, 0x182a, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x180e,
0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x1835, 0x1821, 0x182d, 0x1825, 0x1828, 0x20, 0x1821,
0x1837, 0x182d, 0x1821, 0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1834, 0x1820, 0x182d,
0x1835, 0x1821, 0x182d, 0x1825, 0x1828, 0x20, 0x1821, 0x1837, 0x182d, 0x1821, 0x20, 0x180e,
0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820,
0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1835, 0x1821, 0x182d, 0x1825, 0x1828,
0x20, 0x1821, 0x1837, 0x182d, 0x1821, 0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1835,
0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x180e, 0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x1820,
0x182d, 0x1823, 0x182f, 0x1820, 0x20, 0x180e, 0x180e, 0x180e, 0x180e, 0x1822, 0x1822, 0x1828,
0x20, 0x1834, 0x1820, 0x182d, 0x1820, 0x182d, 0x1823, 0x182f, 0x1820, 0x20, 0x180e, 0x180e,
0x180e, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b,
0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1820, 0x182d, 0x1823,
0x182f, 0x1820, 0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1835, 0x1823, 0x1828, 0x20,
0x180e, 0x180e, 0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x1828, 0x1823, 0x182e, 0x1823, 0x182c,
0x1820, 0x1828, 0x20, 0x1833, 0x1820, 0x182f, 0x1820, 0x1822, 0x20, 0x180e, 0x1822, 0x1822,
0x1828, 0x20, 0x1834, 0x1820, 0x182d, 0x1828, 0x1823, 0x182e, 0x1823, 0x182c, 0x1820, 0x1828,
0x20, 0x1833, 0x1820, 0x182f, 0x1820, 0x1822, 0x20, 0x180e, 0x180e, 0x1822, 0x1822, 0x1828,
0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b,
0x20, 0x1834, 0x1820, 0x182d, 0x1828, 0x1823, 0x182e, 0x1823, 0x182c, 0x1820, 0x1828, 0x20,
0x1833, 0x1820, 0x182f, 0x1820, 0x1822, 0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1835,
0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x180e, 0x182a, 0x20, 0x1834, 0x1820, 0x182d, 0x1820,
0x1832, 0x180b, 0x182f, 0x1820, 0x1828, 0x180b, 0x1832, 0x180b, 0x20, 0x180e, 0x1824, 0x1828,
0x20, 0x1834, 0x1820, 0x182d, 0x1820, 0x1832, 0x180b, 0x182f, 0x1820, 0x1828, 0x180b, 0x1832,
0x20, 0x180e, 0x1824, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833,
0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1820, 0x1832, 0x180b, 0x182f,
0x1820, 0x1828, 0x180b, 0x1832, 0x20, 0x180e, 0x1824, 0x1828, 0x20, 0x1835, 0x1823, 0x1828,
0x20, 0x180e, 0x182a, 0x20, 0x1834, 0x1820, 0x182d, 0x1832, 0x1825, 0x182a, 0x20, 0x1821,
0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20,
0x1834, 0x1820, 0x182d, 0x1832, 0x1825, 0x182a, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837,
0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820,
0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1832,
0x1825, 0x182a, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20,
0x1822, 0x1822, 0x1828, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x1824, 0x20,
0x1834, 0x1820, 0x182d, 0x1835, 0x1821, 0x182d, 0x1826, 0x1828, 0x20, 0x1821, 0x1838, 0x1823,
0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1834, 0x1820,
0x182d, 0x1835, 0x1821, 0x182d, 0x1826, 0x1828, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837,
0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820,
0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1835,
0x1821, 0x182d, 0x1826, 0x1828, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b,
0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e,
0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x182a, 0x1820, 0x1837, 0x1820, 0x182d, 0x1823, 0x1828,
0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822,
0x1828, 0x20, 0x1834, 0x1820, 0x182d, 0x182a, 0x1820, 0x1837, 0x1820, 0x182d, 0x1823, 0x1828,
0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822,
0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833,
0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x182a, 0x1820, 0x1837, 0x1820, 0x182d, 0x1823, 0x1828,
0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822,
0x1828, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x1824, 0x20, 0x1834, 0x1820,
0x182d, 0x182d, 0x1837, 0x1822, 0x1828, 0x180b, 0x1838, 0x1822, 0x1834, 0x1822, 0x20, 0x1822,
0x1822, 0x1828, 0x20, 0x1834, 0x1820, 0x182d, 0x905, 0x92b, 0x917, 0x93e, 0x928, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928,
0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f,
0x915, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x938, 0x92e, 0x92f,
0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f,
0x915, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f,
0x915, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f, 0x915,
0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x938, 0x94d, 0x915, 0x93e,
0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x938, 0x94d, 0x915, 0x93e,
0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905,
0x932, 0x938, 0x94d, 0x915, 0x93e, 0x915, 0x94b, 0x20, 0x926, 0x93f, 0x935, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92e, 0x93e, 0x91c, 0x94b, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x90f, 0x92e, 0x93e, 0x91c, 0x94b, 0x928, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92e, 0x93e, 0x91c, 0x94b, 0x928, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930,
0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926, 0x93f, 0x935,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20,
0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93f, 0x92e, 0x93e,
0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93f, 0x92e, 0x93e, 0x932, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93f, 0x92e,
0x93e, 0x932, 0x940, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94d, 0x92f, 0x93e, 0x938,
0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x906, 0x92a,
0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x905, 0x930, 0x92c, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930,
0x92c, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905,
0x930, 0x92c, 0x940, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x905, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x938,
0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d,
0x91c, 0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c,
0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x928,
0x91f, 0x93f, 0x928, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d,
0x91c, 0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x905, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x905, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x92e, 0x947,
0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x91f, 0x94d,
0x932, 0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f,
0x91f, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x91f, 0x94d, 0x932, 0x93e, 0x928,
0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x905,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20,
0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d,
0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x905, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x905,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x930, 0x92c, 0x948,
0x91c, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x930, 0x92c, 0x948,
0x91c, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x90f, 0x91c, 0x94b, 0x930, 0x947, 0x938, 0x94d, 0x20, 0x938, 0x92e, 0x92f,
0x90f, 0x91c, 0x94b, 0x930, 0x947, 0x938, 0x94d, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x91c, 0x94b, 0x930, 0x947, 0x938, 0x94d, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936,
0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947,
0x936, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92c,
0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x92d, 0x941, 0x91f, 0x93e, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92c,
0x94b, 0x932, 0x93f, 0x92d, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92c,
0x94d, 0x930, 0x93e, 0x91c, 0x93f, 0x932, 0x940, 0x92f, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93e,
0x91c, 0x93f, 0x932, 0x940, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92c,
0x94d, 0x930, 0x941, 0x928, 0x93e, 0x907, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938,
0x94d, 0x938, 0x932, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20,
0x92d, 0x930, 0x94d, 0x926, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a,
0x20, 0x92d, 0x930, 0x94d, 0x926, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20, 0x92d, 0x930, 0x94d, 0x926, 0x947,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93e, 0x92e, 0x94b, 0x930, 0x94d, 0x930,
0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93e,
0x925, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93e, 0x925, 0x93e, 0x92e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93e, 0x925,
0x93e, 0x92e, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x91a,
0x940, 0x928, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x94d, 0x930, 0x93f, 0x938, 0x94d, 0x92e, 0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x941,
0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x915, 0x94b, 0x938, 0x20, 0x91f, 0x93e,
0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x92e, 0x94d, 0x92c,
0x93f, 0x92f, 0x93e, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932,
0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x93e, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x92e, 0x94d, 0x92c, 0x93f,
0x92f, 0x93e, 0x932, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x941, 0x915,
0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x941, 0x915,
0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x941, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x906,
0x927, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c,
0x93e, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c,
0x93e, 0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x915, 0x94b, 0x20, 0x926, 0x93f, 0x935,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x921, 0x947, 0x92d, 0x93f, 0x938, 0x20, 0x938,
0x92e, 0x92f, 0x921, 0x941, 0x92e, 0x94b, 0x928, 0x94d, 0x91f, 0x2d, 0x921, 0x940,
0x2018, 0x20, 0x909, 0x930, 0x94d, 0x92d, 0x93f, 0x932, 0x947, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x91f, 0x93f, 0x92e, 0x94b,
0x930, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x91f,
0x93e, 0x92a, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x938, 0x94d, 0x91f, 0x930,
0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x91f, 0x93e, 0x92a, 0x942,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x938, 0x92e, 0x92f,
0x908, 0x915, 0x94d, 0x935, 0x94b, 0x921, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92f, 0x941,
0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947,
0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x947, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92f,
0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a,
0x947, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935,
0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935,
0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x925, 0x92a, 0x2d, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x941, 0x930,
0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932,
0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x932,
0x94d, 0x915, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x91f, 0x93e,
0x92a, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x932, 0x94d, 0x915, 0x932, 0x94d,
0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x932, 0x94d, 0x915, 0x932,
0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93f, 0x91c, 0x940, 0x20, 0x938, 0x92e, 0x92f,
0x92b, 0x93f, 0x91c, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92b, 0x93f, 0x91c, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x94d,
0x930, 0x947, 0x928, 0x94d, 0x91a, 0x20, 0x917, 0x94d, 0x935, 0x93e, 0x928, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x94d, 0x930, 0x947, 0x928, 0x94d, 0x91a, 0x20,
0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x940, 0x20, 0x930, 0x20, 0x905, 0x928,
0x94d, 0x91f, 0x93e, 0x930, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x917, 0x93e, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x94d, 0x20, 0x938,
0x92e, 0x92f, 0x917, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930,
0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x930, 0x94d, 0x91c, 0x93f,
0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x917, 0x93f, 0x932, 0x94d, 0x92c,
0x930, 0x94d, 0x91f, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f,
0x917, 0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x93f, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915,
0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921,
0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932,
0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915,
0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932,
0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x916, 0x93e, 0x921, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x941, 0x91f, 0x93f,
0x92f, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f,
0x932, 0x941, 0x91f, 0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x941, 0x91f,
0x93f, 0x92f, 0x928, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x939, 0x919, 0x915, 0x919, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x919, 0x915, 0x919,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x919, 0x915,
0x919, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x92c, 0x94d, 0x921, 0x20,
0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x92c, 0x94d, 0x921, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x92c, 0x94d, 0x921, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x939, 0x93f, 0x928, 0x94d, 0x926, 0x20, 0x92e, 0x939, 0x93e,
0x938, 0x93e, 0x917, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x928, 0x94d, 0x921,
0x94b, 0x91a, 0x93e, 0x907, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947,
0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x907, 0x928, 0x94d, 0x921,
0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x94b, 0x928,
0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x94b, 0x928, 0x947,
0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x930, 0x93e, 0x928,
0x940, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x930, 0x93e, 0x928, 0x940, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x930, 0x93e, 0x928, 0x940,
0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930, 0x94d,
0x915, 0x941, 0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x908,
0x930, 0x94d, 0x915, 0x941, 0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x91f,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x930,
0x93e, 0x92f, 0x932, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x930, 0x93e, 0x92f,
0x932, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c,
0x930, 0x93e, 0x92f, 0x932, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x93e, 0x91c, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x915, 0x93e, 0x91c, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x915, 0x93e, 0x91c,
0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b,
0x930, 0x93f, 0x92f, 0x93e, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b,
0x930, 0x93f, 0x92f, 0x93e, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x932, 0x940, 0x20,
0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93e,
0x938, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x94d, 0x930, 0x93e, 0x938, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x915,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930,
0x93e, 0x938, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x915, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x93f, 0x930, 0x94d, 0x917, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x932, 0x93e, 0x907, 0x928, 0x20, 0x91f, 0x93e,
0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x932, 0x930, 0x94d, 0x921, 0x20, 0x939,
0x93e, 0x935, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x932, 0x930, 0x94d, 0x921, 0x20,
0x939, 0x93e, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x932, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935, 0x947, 0x20, 0x926,
0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x932, 0x947, 0x938, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x932, 0x94d, 0x926, 0x93f,
0x92d, 0x94d, 0x938, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930, 0x94d, 0x915,
0x93f, 0x938, 0x938, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930, 0x94d, 0x936,
0x932, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x909,
0x930, 0x93f, 0x91f, 0x938, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x909, 0x930, 0x93f,
0x91f, 0x938, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e,
0x909, 0x930, 0x93f, 0x91f, 0x938, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x94d,
0x935, 0x938, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915, 0x94d, 0x938,
0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915,
0x928, 0x20, 0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915, 0x94d,
0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b,
0x93f, 0x915, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x909,
0x932, 0x93e, 0x928, 0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x938, 0x92e,
0x92f, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x932, 0x93e, 0x928,
0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e,
0x938, 0x94d, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x938, 0x94d, 0x915,
0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x938,
0x94d, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x94d, 0x92f, 0x93e,
0x928, 0x92e, 0x93e, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x93e, 0x909, 0x930,
0x941, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x947, 0x92a, 0x93e, 0x932, 0x940, 0x20,
0x938, 0x92e, 0x92f, 0x928, 0x92f, 0x93e, 0x901, 0x20, 0x915, 0x93e, 0x932, 0x947,
0x926, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x92f,
0x93e, 0x901, 0x20, 0x915, 0x93e, 0x932, 0x947, 0x926, 0x94b, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x92f,
0x93e, 0x901, 0x20, 0x915, 0x93e, 0x932, 0x947, 0x926, 0x94b, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93f,
0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x928,
0x94d, 0x92f, 0x942, 0x91c, 0x93f, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f,
0x942, 0x91c, 0x93f, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x926,
0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b,
0x93e, 0x909, 0x928, 0x94d, 0x921, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921,
0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93e,
0x909, 0x928, 0x921, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f,
0x942, 0x92b, 0x93e, 0x909, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921,
0x915, 0x94b, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928,
0x93f, 0x909, 0x90f, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x930, 0x94d, 0x92b,
0x94b, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x915, 0x94b, 0x20, 0x938, 0x92e,
0x92f, 0x928, 0x94b, 0x930, 0x94d, 0x92b, 0x94b, 0x915, 0x20, 0x91f, 0x93e, 0x92a,
0x942, 0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x928, 0x94b, 0x930, 0x94d, 0x92b, 0x94b, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942,
0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x930, 0x94d, 0x928, 0x93e,
0x928, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b,
0x928, 0x94d, 0x939, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x930, 0x94d, 0x928,
0x93e, 0x928, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930,
0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x928, 0x94d, 0x921, 0x94b, 0x20,
0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x92d, 0x94b, 0x938, 0x93f, 0x935, 0x93f,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x92d,
0x94b, 0x938, 0x93f, 0x935, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x92d, 0x94b, 0x938,
0x93f, 0x935, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x940, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x93e, 0x932, 0x93e, 0x909, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x92a,
0x942, 0x935, 0x93e, 0x20, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917, 0x93f, 0x928,
0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x94d, 0x935,
0x947, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x94d, 0x935,
0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e,
0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x947, 0x930, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x930, 0x941, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93f, 0x928, 0x940,
0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93f, 0x928, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93f, 0x932,
0x93f, 0x92a, 0x93f, 0x928, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93f,
0x928, 0x93f, 0x915, 0x94d, 0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938,
0x92e, 0x92f, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930,
0x94d, 0x930, 0x947, 0x20, 0x930, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x92f, 0x941,
0x932, 0x94b, 0x928, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x928,
0x94d, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x94d, 0x930, 0x947, 0x20, 0x930,
0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x92f, 0x941, 0x932, 0x94b, 0x928, 0x915, 0x94b,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x928,
0x94d, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x94d, 0x930, 0x947, 0x20, 0x930,
0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x92f, 0x941, 0x932, 0x94b, 0x928, 0x915, 0x94b,
0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93f, 0x91f,
0x915, 0x948, 0x930, 0x923, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94d, 0x92f, 0x94b,
0x919, 0x92f, 0x93e, 0x919, 0x20, 0x938, 0x92e, 0x92f, 0x930, 0x93f, 0x92f, 0x941,
0x928, 0x93f, 0x92f, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x916, 0x93e,
0x932, 0x93f, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x916, 0x93e, 0x932,
0x93f, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938,
0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938,
0x93e, 0x92e, 0x94b, 0x905, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x92e, 0x94b,
0x905, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e,
0x92e, 0x94b, 0x905, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x938, 0x947, 0x92f, 0x947, 0x91a, 0x947, 0x932, 0x93e, 0x938, 0x94d, 0x20, 0x938,
0x92e, 0x92f, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x94b, 0x932, 0x94b, 0x92e,
0x94b, 0x928, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x926,
0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x941, 0x930, 0x93f, 0x928, 0x93e, 0x92e,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x907, 0x20,
0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x907, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x907,
0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x91c,
0x93f, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x924,
0x94b, 0x915, 0x947, 0x932, 0x93e, 0x909, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x941,
0x909, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e,
0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x941, 0x92d, 0x93e, 0x932, 0x941,
0x20, 0x938, 0x92e, 0x92f, 0x909, 0x930, 0x942, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x94d, 0x92c,
0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x909, 0x91c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x94d,
0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x92d, 0x93e, 0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x938, 0x92e,
0x92f, 0x92d, 0x93e, 0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x93e, 0x928, 0x941, 0x906, 0x924, 0x941,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x947, 0x928, 0x947, 0x91c, 0x94d, 0x92f,
0x941, 0x90f, 0x932, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x94d, 0x932, 0x93e,
0x926, 0x93f, 0x92d, 0x93e, 0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92d, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x93e, 0x938, 0x94d, 0x91f,
0x94b, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92d,
0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x93e, 0x938, 0x94d, 0x91f, 0x94b, 0x915,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x94b, 0x932, 0x94d, 0x917, 0x93e, 0x917,
0x94d, 0x930, 0x93e, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x94b, 0x932, 0x94d,
0x917, 0x93e, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x94b, 0x932, 0x94d, 0x917, 0x93e, 0x917, 0x94d,
0x930, 0x93e, 0x926, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x947, 0x915, 0x20,
0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x93e, 0x932, 0x93f,
0x938, 0x94d, 0x20, 0x930, 0x20, 0x92b, 0x941, 0x91f, 0x941, 0x928, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x92f, 0x93e, 0x915, 0x941, 0x938, 0x94d, 0x91f, 0x20, 0x938,
0x92e, 0x92f, 0x92f, 0x93e, 0x915, 0x941, 0x938, 0x94d, 0x91f, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x93e, 0x915, 0x941, 0x938, 0x94d,
0x91f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915, 0x93e, 0x91f, 0x947,
0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x938, 0x92e, 0x92f, 0x92f,
0x947, 0x915, 0x93e, 0x91f, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915,
0x93e, 0x91f, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92f, 0x941, 0x915, 0x94b, 0x928, 0x915, 0x94b, 0x20, 0x938,
0x92e, 0x92f, 0x41, 0x66, 0x67, 0xe1, 0x6e, 0xed, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0xc1,
0x66, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73,
0x74, 0x20, 0xc1, 0x66, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x53, 0x61, 0xfa, 0x74, 0x20, 0xc1, 0x66, 0x72, 0xed, 0x6b, 0xe1,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57,
0x1eb9, 0x301, 0x73, 0x74, 0x20, 0xc1, 0x66, 0x72, 0xed, 0x6b, 0xe1, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0xc1, 0x66,
0x72, 0xed, 0x6b, 0xe1, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0xc1, 0x66, 0x72,
0xed, 0x6b, 0xe1, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x6c, 0xe1, 0x73, 0x6b,
0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x6c, 0xe1, 0x73, 0x6b, 0xe1,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41,
0x6c, 0xe1, 0x73, 0x6b, 0xe1, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74,
0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1, 0x6d, 0xe1, 0x7a, 0x1ecd, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0xc1, 0x6d, 0xe1, 0x7a, 0x1ecd, 0x6e, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1, 0x6d, 0xe1,
0x7a, 0x1ecd, 0x6e, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20,
0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x4d, 0xed, 0x64,
0xfa, 0x6c, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b,
0xe1, 0x20, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0xc9, 0x72, 0x69, 0xe1,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1,
0x20, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1,
0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9,
0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0xc9, 0x72, 0x69, 0xe1, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301,
0x72, 0xed, 0x6b, 0xe1, 0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0xc9,
0x72, 0x69, 0xe1, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301,
0x72, 0xed, 0x6b, 0xe1, 0x20, 0x4d, 0x61, 0xfa, 0x6e, 0x74, 0x69, 0x6e,
0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd,
0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20,
0x4d, 0x61, 0xfa, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0xc9, 0x72, 0x69, 0xe1,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1,
0x20, 0x4d, 0x61, 0xfa, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0xc9, 0x72, 0x69,
0xe1, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed,
0x6b, 0xe1, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0xed, 0x6b, 0x20, 0xc9,
0x72, 0x69, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74,
0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x50, 0x61,
0x73, 0xed, 0x66, 0xed, 0x6b, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301,
0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x50,
0x61, 0x73, 0xed, 0x66, 0xed, 0x6b, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1,
0x70, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1, 0x70, 0x69, 0x61,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1,
0x70, 0x69, 0x61, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x41, 0x72, 0xe9, 0x62, 0x69, 0xe1, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x41, 0x72, 0xe9, 0x62, 0x69, 0xe1, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x72, 0xe9, 0x62, 0x69,
0xe1, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x6a, 0x1eb9,
0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301,
0x73, 0x74, 0xe1, 0x6e, 0x20, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1,
0x6e, 0x20, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301,
0x73, 0x74, 0xe1, 0x6e, 0x20, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e,
0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x72, 0x6d, 0x1eb9, 0x301, 0x6e, 0x69,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x72, 0x6d, 0x1eb9, 0x301, 0x6e,
0x69, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x41, 0x72, 0x6d, 0x1eb9, 0x301, 0x6e, 0x69, 0x61, 0x20, 0x48, 0x1ecd,
0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0xed, 0x6b, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0xed, 0x6b, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x74, 0x6c,
0xe1, 0x6e, 0x74, 0xed, 0x6b, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0x1ecc,
0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0xed, 0x64,
0xfa, 0x6c, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0xed,
0x64, 0xfa, 0x6c, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20,
0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c,
0x69, 0x61, 0x20, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0x57, 0x1eb9, 0x301,
0x73, 0x74, 0xe1, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20,
0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1,
0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x1ecc, 0x73, 0x74, 0x72, 0xe9,
0x6c, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72,
0xe9, 0x6c, 0x69, 0x61, 0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74,
0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57,
0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x1ecc, 0x73, 0x74, 0x72, 0xe9,
0x6c, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72,
0xe9, 0x6c, 0x69, 0x61, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc,
0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x57, 0x1eb9, 0x301, 0x73,
0x74, 0xe1, 0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x41, 0x7a, 0x1eb9, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x7a, 0x1eb9, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x41, 0x7a, 0x1eb9, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x7a, 0x1ecd, 0x7a, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x41, 0x7a, 0x1ecd, 0x7a, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x7a, 0x1ecd, 0x7a, 0x20, 0x48, 0x1ecd,
0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x1eb9, 0x73, 0x68, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x1eb9,
0x73, 0x68, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x1eb9, 0x73, 0x68, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42,
0x75, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x6f, 0x6c,
0xed, 0x76, 0x69, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42,
0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74,
0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42,
0x72, 0x75, 0x6e, 0x1eb9, 0x69, 0x20, 0x44, 0x61, 0x72, 0xfa, 0x73, 0x61,
0x6c, 0x61, 0x6d, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x65, 0x70, 0x20,
0x56, 0x1eb9, 0x301, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x65, 0x70,
0x20, 0x56, 0x1eb9, 0x301, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4b, 0x65, 0x70, 0x20, 0x56, 0x1eb9, 0x301, 0x64,
0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0x61, 0x6d, 0xf3, 0x72, 0x6f, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68,
0xe1, 0x74, 0x61, 0x6d, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xe1,
0x74, 0x61, 0x6d, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x43, 0x68, 0xe1, 0x74, 0x61, 0x6d, 0x20, 0x44, 0xe9, 0x6c,
0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xed, 0x6c,
0x1eb9, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xed, 0x6c, 0x1eb9, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68,
0xed, 0x6c, 0x1eb9, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0x61, 0xed, 0x6e,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0x61, 0xed, 0x6e, 0x61,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43,
0x68, 0x61, 0xed, 0x6e, 0x61, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x72, 0xed, 0x73, 0x6d, 0xe1, 0x73,
0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4b, 0xf3, 0x6b, 0xf3, 0x73, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x6c, 0xf3, 0x6d, 0x62,
0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x6c, 0xf3, 0x6d,
0x62, 0x69, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4b, 0x6f, 0x6c, 0xf3, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4b, 0xfa, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0xfa, 0x6b, 0x20, 0x41, 0xed,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4b, 0xfa, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x48, 0x61, 0x66, 0x20, 0x48, 0x1ecd, 0x301, 0x74,
0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b,
0xfa, 0x62, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0xfa, 0x62, 0x61,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b,
0xfa, 0x62, 0x61, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x44, 0xe9, 0x76, 0x69, 0x73, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x44, 0x69, 0xfa, 0x6d, 0x1ecd, 0x301, 0x6e, 0x2d, 0x64, 0x2019, 0x55,
0x76, 0x69, 0x6c, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x20,
0x54, 0xed, 0x6d, 0x1ecd, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x61, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74,
0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x1eb8, 0x301, 0x6b, 0x77, 0x75, 0xe1, 0x64,
0x1ecd, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20,
0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed,
0x64, 0xfa, 0x6c, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa,
0x6c, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x48, 0x1ecd, 0x301, 0x74,
0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd,
0x73, 0x74, 0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x54,
0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72,
0x6f, 0x70, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70,
0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x46, 0x61, 0xe1, 0x2d, 0xcd, 0x73, 0x74, 0xe1,
0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f,
0x70, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1,
0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1,
0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x48, 0x1ecd, 0x301, 0x74,
0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46,
0x1ecd, 0x301, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x46, 0x1ecd, 0x301, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x1ecd, 0x301,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed,
0x6a, 0x69, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6a, 0x69, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed,
0x6a, 0x69, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x72, 0x1eb9, 0x301, 0x6e, 0x63,
0x68, 0x20, 0x47, 0x69, 0xe1, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x46, 0x72, 0x1eb9, 0x301, 0x6e, 0x63, 0x68, 0x20, 0x53, 0x61, 0xfa, 0x74,
0x61, 0x6e, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x74, 0xed,
0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0xe1,
0x67, 0x1ecd, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x47, 0xe1, 0x6d, 0x62,
0x69, 0x1eb9, 0x72, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x1ecd, 0x301, 0x6a,
0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x1ecd, 0x301, 0x6a, 0x69,
0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4a, 0x1ecd, 0x301, 0x6a, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x47, 0xed,
0x6c, 0x62, 0x61, 0x74, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x47, 0x72, 0xed, 0x6e, 0x77, 0xed, 0x63,
0x68, 0x20, 0x4d, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x69,
0x73, 0x74, 0x20, 0x47, 0x72, 0xed, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0xcd, 0x69, 0x73, 0x74, 0x20, 0x47, 0x72, 0xed,
0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0xcd, 0x69, 0x73, 0x74, 0x20, 0x47, 0x72, 0xed,
0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301,
0x73, 0x74, 0x20, 0x47, 0x72, 0xed, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0x47, 0x72,
0xed, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0x47,
0x72, 0xed, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74,
0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x47,
0x1ecd, 0x301, 0x6c, 0x66, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x47, 0x61, 0x79, 0xe1, 0x6e, 0x61, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x48, 0x61, 0x77, 0x61, 0x69, 0xed, 0x2d, 0x45, 0x6c, 0xfa,
0x73, 0x68, 0x69, 0xe1, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x48, 0x61,
0x77, 0x61, 0x69, 0xed, 0x2d, 0x45, 0x6c, 0xfa, 0x73, 0x68, 0x69, 0xe1,
0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x48, 0x61, 0x77, 0x61, 0x69, 0xed, 0x2d, 0x45, 0x6c, 0xfa, 0x73, 0x68,
0x69, 0xe1, 0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x48, 0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd, 0x6e, 0x67,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x48, 0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd,
0x6e, 0x67, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x48, 0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd, 0x6e, 0x67, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x48,
0x6f, 0x76, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0xcd, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x20, 0xd3, 0x73, 0x68,
0x1eb9, 0x301, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x6e, 0x64, 0x6f,
0x63, 0x68, 0x61, 0xed, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d,
0xed, 0x64, 0xfa, 0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x1eb9, 0x301,
0x73, 0x68, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x1eb9, 0x301, 0x73, 0x68,
0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x1eb9, 0x301, 0x73, 0x68,
0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x61, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x61, 0x6e, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x49,
0x72, 0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x49,
0x72, 0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x6b, 0xfa, 0x74, 0x73, 0x6b,
0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0xcd, 0x7a, 0x72, 0x1eb9, 0x6c, 0x20, 0x54, 0x61,
0x69, 0x6d, 0xcd, 0x7a, 0x72, 0x1eb9, 0x6c, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x7a, 0x72, 0x1eb9, 0x6c, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed,
0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x61, 0x7a, 0xe9, 0x6b, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x20,
0x4b, 0x61, 0x7a, 0xe9, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0xe9,
0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f,
0x72, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x72, 0x69,
0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4b, 0x6f, 0x72, 0x69, 0x61, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x1ecd, 0x301, 0x73, 0x72, 0x61, 0x1eb9,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x73, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x73, 0x6b, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x1eb9, 0x67, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4c, 0x61, 0xed, 0x6e,
0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4c, 0x1ecd, 0x64, 0x20, 0x48, 0x61, 0xfa, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4c, 0x1ecd, 0x64, 0x20, 0x48, 0x61, 0xfa, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4c, 0x1ecd, 0x64, 0x20, 0x48,
0x61, 0xfa, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4d, 0xe1, 0x67, 0xe1, 0x64, 0x61, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4d, 0xe1, 0x67, 0xe1, 0x64, 0x61, 0x6e, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xe1, 0x67, 0xe1,
0x64, 0x61, 0x6e, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x61, 0x6c, 0xe9, 0x73,
0x68, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x301, 0x64,
0x69, 0x76, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x61, 0x6b, 0x77,
0x1eb9, 0x301, 0x73, 0x61, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xe1,
0x73, 0x68, 0x61, 0x6c, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x72, 0xed, 0x73, 0x68, 0x1ecd,
0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x72, 0xed, 0x73, 0x68,
0x1ecd, 0x73, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4d, 0x1ecd, 0x72, 0xed, 0x73, 0x68, 0x1ecd, 0x73, 0x20, 0x48, 0x1ecd,
0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4d, 0x1ecd, 0x301, 0x73, 0x1ecd, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4d, 0x1eb9, 0x301, 0x6b, 0x73, 0xed, 0x6b, 0xf3, 0x20, 0x50, 0x61, 0x73,
0xed, 0x66, 0xed, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1eb9, 0x301,
0x6b, 0x73, 0xed, 0x6b, 0xf3, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0xed,
0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4d, 0x1eb9, 0x301, 0x6b, 0x73, 0xed, 0x6b, 0xf3, 0x20, 0x50, 0x61, 0x73,
0xed, 0x66, 0xed, 0x6b, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x6e, 0x67, 0xf3, 0x6c, 0x69, 0x61,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x6e, 0x67, 0xf3, 0x6c, 0x69,
0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4d, 0x1ecd, 0x6e, 0x67, 0xf3, 0x6c, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301,
0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4d, 0x1ecd, 0x301, 0x73, 0x6b, 0x6f, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d,
0x1ecd, 0x301, 0x73, 0x6b, 0x6f, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x301, 0x73, 0x6b, 0x6f, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4d, 0x69, 0xe1, 0x6e, 0x6d, 0x61, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4e, 0x61, 0xfa, 0x72, 0x75, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x1eb9, 0x70, 0x1ecd, 0x6c, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa,
0x20, 0x4b, 0x61, 0x6c, 0x1eb9, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x20, 0x4b, 0x61, 0x6c, 0x1eb9, 0x64,
0xf3, 0x6e, 0x69, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x20, 0x4b, 0x61, 0x6c, 0x1eb9, 0x64,
0xf3, 0x6e, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed,
0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x20,
0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x69, 0xfa, 0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x20,
0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed,
0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x66, 0x61, 0xfa,
0x6e, 0x64, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x69, 0xfa, 0x66, 0x61, 0xfa, 0x6e, 0x64, 0x6c, 0xe1, 0x6e, 0x64, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69,
0xfa, 0x66, 0x61, 0xfa, 0x6e, 0x64, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x44,
0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69,
0xfa, 0x1eb9, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x72, 0x66,
0x1ecd, 0x301, 0x6c, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x72, 0x66, 0x1ecd, 0x301, 0x6c,
0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x72, 0x66,
0x1ecd, 0x301, 0x6c, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46,
0x1eb9, 0x72, 0x6e, 0xe1, 0x6e, 0x64, 0xf3, 0x20, 0x64, 0x69, 0x20, 0x4e,
0x1ecd, 0x72, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46,
0x1eb9, 0x72, 0x6e, 0xe1, 0x6e, 0x64, 0xf3, 0x20, 0x64, 0x69, 0x20, 0x4e,
0x1ecd, 0x72, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x1eb9, 0x72, 0x6e, 0xe1, 0x6e, 0x64,
0xf3, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x1ecd, 0x72, 0xf3, 0x6e, 0x69, 0x61,
0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x73, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x73, 0x6b, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x6d, 0x73, 0x6b, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x1ecc, 0x6d, 0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x6d, 0x73, 0x6b, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x50, 0xe1, 0x6b, 0xed, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x50, 0xe1, 0x6b, 0xed, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xe1,
0x6b, 0xed, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0x61,
0x6c, 0xe1, 0x75, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xe1, 0x70, 0x75,
0xe1, 0x20, 0x4e, 0x69, 0xfa, 0x20, 0x47, 0xed, 0x6e, 0x69, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x50, 0xe1, 0x72, 0xe1, 0x67, 0x77, 0x75, 0x65, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x50, 0xe1, 0x72, 0xe1, 0x67, 0x77, 0x75, 0x65,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50,
0xe1, 0x72, 0xe1, 0x67, 0x77, 0x75, 0x65, 0x20, 0x48, 0x1ecd, 0x301, 0x74,
0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50,
0x1eb9, 0x72, 0x75, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0x1eb9, 0x72, 0x75,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50,
0x1eb9, 0x72, 0x75, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6c, 0xed, 0x70,
0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6c, 0xed, 0x70,
0x69, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x46, 0xed, 0x6c, 0xed, 0x70, 0x69, 0x6e, 0x20, 0x48, 0x1ecd, 0x74,
0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46,
0xed, 0x6e, 0x69, 0x6b, 0x73, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x50,
0x69, 0x1eb9, 0x72, 0x20, 0x26, 0x20, 0x4d, 0xed, 0x6b, 0x1eb9, 0x6c, 0x1ecd,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x50,
0x69, 0x1eb9, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x4d, 0xed, 0x6b, 0x1eb9, 0x6c,
0x1ecd, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x1eb9, 0x72, 0x20, 0x61,
0x6e, 0x20, 0x4d, 0xed, 0x6b, 0x1eb9, 0x6c, 0x1ecd, 0x6e, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xed, 0x74,
0x6b, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xf3, 0x6e, 0xe1,
0x70, 0x1eb9, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0x69, 0x1ecd, 0x6e, 0x67,
0x79, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x52, 0x69, 0x79,
0xfa, 0x6e, 0x69, 0x1ecd, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x52, 0x6f,
0x74, 0x1eb9, 0x301, 0x72, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53,
0xe1, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6d, 0x6f, 0xe1, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6d, 0x6f, 0xe1, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6d, 0x6f,
0xe1, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x53, 0x1eb9, 0x301, 0x63, 0x68, 0x1eb9, 0x6c, 0x73, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x1ecd, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x53, 0xf3, 0x6c, 0xf3, 0x6d, 0x1ecd, 0x301, 0x6e, 0x20, 0x41,
0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53,
0x61, 0xfa, 0x74, 0x20, 0x4a, 0x1ecd, 0x301, 0x6a, 0x69, 0x61, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x53, 0xfa, 0x72, 0xed, 0x6e, 0x61, 0x6d, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x53, 0x69, 0xf3, 0x77, 0x61, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x54, 0x61, 0x68, 0xed, 0x74, 0x69, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x54, 0x61, 0x69, 0x70, 0x1eb9, 0x69, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54,
0x61, 0x69, 0x70, 0x1eb9, 0x69, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x54, 0x61, 0x69, 0x70, 0x1eb9, 0x69, 0x20, 0x44,
0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x61,
0x6a, 0xed, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x54, 0x6f, 0x6b, 0x1eb9, 0x6c, 0xe1, 0x75, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x54, 0x1ecd, 0x301, 0x6e, 0x67, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x54, 0x1ecd, 0x301, 0x6e, 0x67, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x1ecd, 0x301, 0x6e, 0x67, 0x61, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x43, 0x68, 0x75, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x54, 0x1ecd, 0x6b, 0x6d, 0x1eb9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x54, 0x1ecd, 0x6b, 0x6d, 0x1eb9, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x54, 0x1ecd, 0x6b, 0x6d, 0x1eb9, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x75, 0x76, 0xe1, 0x6c, 0x75, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x59, 0xfa, 0x72, 0x75, 0x67, 0x77, 0x75, 0x65,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0xfa, 0x72, 0x75, 0x67, 0x77, 0x75,
0x65, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x59, 0xfa, 0x72, 0x75, 0x67, 0x77, 0x75, 0x65, 0x20, 0x48, 0x1ecd, 0x301,
0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x55, 0x7a, 0x62, 0x1eb9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x55, 0x7a, 0x62, 0x1eb9, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x55, 0x7a, 0x62, 0x1eb9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x56, 0x61, 0x6e, 0x75, 0xe1, 0x74, 0x75, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x56, 0x61, 0x6e, 0x75, 0xe1, 0x74, 0x75, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x61, 0x6e, 0x75,
0xe1, 0x74, 0x75, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x56, 0x1eb9, 0x6e, 0x1eb9, 0x7a, 0x75, 0x1eb9, 0x301, 0x6c, 0x61,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x1ecd,
0x73, 0x74, 0x1ecd, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x1ecd, 0x73, 0x74, 0x1ecd, 0x6b, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6f,
0x6c, 0x76, 0xf3, 0x67, 0x72, 0x61, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x56, 0x6f, 0x6c, 0x76, 0xf3, 0x67, 0x72, 0x61, 0x64, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6f, 0x6c, 0x76,
0xf3, 0x67, 0x72, 0x61, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x1ecd, 0x301,
0x73, 0x74, 0x1ecd, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0xe9, 0x6b,
0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x57, 0xe1, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x6e, 0x20, 0x46, 0xfa, 0x74,
0xfa, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0xe9, 0x6b, 0xfa,
0x74, 0x73, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0xe9, 0x6b, 0xfa,
0x74, 0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x59, 0xe9, 0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x20, 0x48, 0x1ecd,
0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x59, 0x1eb9, 0x6b, 0x65, 0x74, 0x1eb9, 0x72, 0xed, 0x6e, 0x62, 0x75,
0x67, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0x1eb9, 0x6b, 0x65, 0x74, 0x1eb9,
0x72, 0xed, 0x6e, 0x62, 0x75, 0x67, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0x1eb9, 0x6b, 0x65, 0x74, 0x1eb9, 0x72,
0xed, 0x6e, 0x62, 0x75, 0x67, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0x75, 0x6b,
0xf3, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x7ca, 0x7dd, 0x7d1, 0x7dc, 0x7ed,
0x7ca, 0x7e3, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7dd, 0x7cc,
0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ca, 0x7f2, 0x7d3, 0x7ca, 0x7f2,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2,
0x7ca, 0x7d3, 0x7d8, 0x7cd, 0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7cc, 0x7f2, 0x7ec, 0x7e0,
0x7ca, 0x7eb, 0x20, 0x7d3, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3, 0x7d8,
0x7cd, 0x7dd, 0x7d8, 0x7cc, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb,
0x20, 0x7e5, 0x7d9, 0x7cf, 0x7ec, 0x7d8, 0x7ce, 0x7ee, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3, 0x7d8, 0x7cd,
0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec,
0x7e0, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3,
0x7d8, 0x7cd, 0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca,
0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7ca, 0x7d9, 0x7cb, 0x7f2,
0x7ca, 0x7df, 0x7ca, 0x7db, 0x7d1, 0x7de, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7df, 0x7ca, 0x7db, 0x7d1, 0x7de, 0x7ca,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7df, 0x7ca, 0x7db,
0x7d1, 0x7de, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7ca, 0x7e1, 0x7ca, 0x7d6, 0x7ed, 0x7cf,
0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca,
0x7e1, 0x7ca, 0x7d6, 0x7ed, 0x7cf, 0x7e3, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7ca, 0x7e1, 0x7ca, 0x7d6, 0x7ed, 0x7cf, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7ca,
0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7d5, 0x7ca, 0x7f2, 0x7d3,
0x7ca, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de,
0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7ee, 0x20, 0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de,
0x7cc, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ca, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7ee, 0x20,
0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7de, 0x7d0,
0x7f0, 0x7d8, 0x7ce, 0x7f0, 0x20, 0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc,
0x7ec, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x7ec, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7f0,
0x20, 0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de,
0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7f0, 0x20, 0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de,
0x7cc, 0x7ec, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7de, 0x7d9, 0x7ce, 0x7ec, 0x7de, 0x7ca, 0x7f2, 0x7ec,
0x7e0, 0x7ca, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de,
0x7d9, 0x7ce, 0x7ec, 0x7de, 0x7ca, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7d9, 0x7ce, 0x7ec, 0x7de, 0x7ca, 0x7f2, 0x7ec,
0x7e0, 0x7ca, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7de, 0x7d0, 0x7f0, 0x7d8, 0x7ce, 0x7f0, 0x20, 0x7ca,
0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7e3, 0x7cc, 0x7eb, 0x20,
0x7d6, 0x7d0, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7d0, 0x7f0, 0x7d8, 0x7ce,
0x7f0, 0x20, 0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7e3,
0x7cc, 0x7eb, 0x20, 0x7d6, 0x7d0, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca,
0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7d0, 0x7f0, 0x7d8,
0x7ce, 0x7f0, 0x20, 0x7ca, 0x7e1, 0x7cb, 0x7d9, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20,
0x7e3, 0x7cc, 0x7eb, 0x20, 0x7d6, 0x7d0, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3,
0x7ca, 0x7f2, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7ca, 0x7d4, 0x7cc, 0x7e6, 0x7ca, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7d4, 0x7cc, 0x7e6, 0x7ca,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d4, 0x7cc, 0x7e6,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d9, 0x7ca, 0x7d3, 0x7ce, 0x7df, 0x7ca,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7d9,
0x7ca, 0x7d3, 0x7ce, 0x7df, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7ca, 0x7d9, 0x7ca, 0x7d3, 0x7ce, 0x7df, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d9,
0x7d1, 0x7d6, 0x7ed, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7ca,
0x7f2, 0x7d5, 0x7cc, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca,
0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7ca, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7f2, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7ca, 0x7d9, 0x7d1, 0x7d6,
0x7ed, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20,
0x7ca, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7f2, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7ca, 0x7d9, 0x7d1, 0x7e1, 0x7cb, 0x7e3, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7d9, 0x7d1, 0x7e1, 0x7cb, 0x7e3,
0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d9, 0x7d1,
0x7e1, 0x7cb, 0x7e3, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2,
0x2d, 0x7e1, 0x7ca, 0x7f2, 0x7de, 0x7ca, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x2d, 0x7e1,
0x7ca, 0x7f2, 0x7de, 0x7ca, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca,
0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x2d, 0x7e1, 0x7ca, 0x7f2, 0x7de, 0x7ca,
0x7f2, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce,
0x7e1, 0x7d8, 0x7ca, 0x7d5, 0x7cd, 0x7e1, 0x7df, 0x7ca, 0x20, 0x7cf, 0x7db, 0x7d1,
0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7cd, 0x7e1, 0x7df, 0x7ca, 0x20, 0x7cf, 0x7db, 0x7d1,
0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7d5, 0x7cd, 0x7e1, 0x7df, 0x7ca, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9,
0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca,
0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x7ec, 0x2d,
0x7d5, 0x7cd, 0x7e1, 0x7df, 0x7ca, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x7ec, 0x2d, 0x7d5, 0x7cd, 0x7e1, 0x7df,
0x7ca, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cf, 0x7db, 0x7d1, 0x7d5,
0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb,
0x7ec, 0x2d, 0x7d5, 0x7cd, 0x7e1, 0x7df, 0x7ca, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7d3, 0x7d0, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df,
0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7d3, 0x7d0, 0x20, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc,
0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce,
0x7e1, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cf, 0x7db,
0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cf,
0x7db, 0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cf, 0x7db,
0x7d1, 0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7ca, 0x7d6, 0x7ed,
0x7cd, 0x7d9, 0x7d1, 0x7d3, 0x7ca, 0x7e6, 0x7d1, 0x7d6, 0x7ca, 0x7f2, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca, 0x7d6, 0x7ed, 0x7cd,
0x7d9, 0x7d1, 0x7d3, 0x7ca, 0x7e6, 0x7d1, 0x7d6, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d6, 0x7ed, 0x7cd, 0x7d9, 0x7d1, 0x7d3,
0x7ca, 0x7e6, 0x7d1, 0x7d6, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ca, 0x7d7, 0x7ed, 0x7d0,
0x7d9, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ca,
0x7d7, 0x7ed, 0x7d0, 0x7d9, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca,
0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7ca, 0x7d7, 0x7ed, 0x7d0, 0x7d9, 0x7db, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d3, 0x7ca, 0x7f2, 0x7dc,
0x7ed, 0x7d1, 0x7df, 0x7ca, 0x7d8, 0x7cd, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d3, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7d1, 0x7df, 0x7ca,
0x7d8, 0x7cd, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d3, 0x7ca,
0x7f2, 0x7dc, 0x7ed, 0x7d1, 0x7df, 0x7ca, 0x7d8, 0x7cd, 0x7db, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d3,
0x7ce, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d3, 0x7cf, 0x7df, 0x7dd, 0x7ed, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d3, 0x7d9, 0x7cb, 0x7d6, 0x7ed, 0x7df,
0x7cc, 0x7df, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7d3, 0x7d9, 0x7cb, 0x7d6, 0x7ed, 0x7df, 0x7cc, 0x7df, 0x7ca, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d3, 0x7d9, 0x7cb, 0x7d6, 0x7ed, 0x7df,
0x7cc, 0x7df, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d3, 0x7d9, 0x7ce, 0x7e3, 0x7cb, 0x7e6,
0x7cc, 0x7eb, 0x20, 0x7d8, 0x7ca, 0x7d9, 0x7ce, 0x7db, 0x7d1, 0x7db, 0x7df, 0x7ca,
0x7e1, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dc, 0x7d9,
0x7cb, 0x7de, 0x7ce, 0x7f2, 0x7eb, 0x2d, 0x7dd, 0x7d9, 0x7cc, 0x7db, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7dc, 0x7d9,
0x7cb, 0x7de, 0x7ce, 0x7f2, 0x7eb, 0x2d, 0x7dd, 0x7d9, 0x7cc, 0x7db, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2,
0x7ca, 0x7d3, 0x7d8, 0x7cd, 0x7dc, 0x7d9, 0x7cb, 0x7de, 0x7ce, 0x7f2, 0x7eb, 0x2d,
0x7dd, 0x7d9, 0x7cc, 0x7db, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7ca, 0x7d9, 0x7cb, 0x7f2, 0x7db, 0x7ed,
0x7ca, 0x7e1, 0x7cf, 0x7d9, 0x7cf, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7d7, 0x7d5, 0x7ca, 0x7e1, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d7, 0x7d5, 0x7ca, 0x7e1, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7d7, 0x7d5, 0x7ca, 0x7e1, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7db, 0x7df, 0x7cc, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7db, 0x7df, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7db, 0x7df, 0x7cc, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7db, 0x7cc, 0x7e3, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7db, 0x7cc, 0x7e3, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7db,
0x7cc, 0x7e3, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7d1, 0x7d9, 0x7cc, 0x7db,
0x7d1, 0x7e1, 0x7ca, 0x7db, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7de, 0x7cf, 0x7db, 0x20, 0x7d5,
0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de,
0x7d0, 0x7df, 0x7d0, 0x7f2, 0x7d3, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7d0, 0x7df, 0x7d0, 0x7f2, 0x7d3, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7d0, 0x7df, 0x7d0, 0x7f2,
0x7d3, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20,
0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7cf, 0x7de, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de, 0x7ce, 0x7f3,
0x7d3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7de, 0x7ce, 0x7f3, 0x7d3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7de, 0x7ce, 0x7f1, 0x7d3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d8, 0x7ca, 0x7dd,
0x7ed, 0x7cc, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7d8, 0x7ce, 0x7e1, 0x7d0, 0x7f2, 0x2d, 0x7d8, 0x7ce, 0x7d9, 0x7d1, 0x7dd, 0x7ed,
0x7cc, 0x7df, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5,
0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7d5, 0x7cc, 0x7e1, 0x7d0, 0x7d9, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7cc, 0x7db, 0x7d1, 0x7df,
0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7cc, 0x7db, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20,
0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cc, 0x7db,
0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7cc,
0x7de, 0x7d1, 0x7e5, 0x7ca, 0x7d8, 0x7d0, 0x7d9, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7cd, 0x7e1, 0x7ca, 0x7eb, 0x20, 0x7cb, 0x7d9,
0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7d5, 0x7cd, 0x7e1, 0x7ca, 0x7eb, 0x20, 0x7cb, 0x7d9, 0x7d0, 0x7d4, 0x7ce,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7cd, 0x7e1, 0x7ca,
0x7eb, 0x20, 0x7cb, 0x7d9, 0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7cb, 0x7d9, 0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7d3, 0x7d0, 0x20, 0x7cb, 0x7d9, 0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7cb,
0x7d9, 0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7de, 0x7ca, 0x7df, 0x7cc, 0x7e3,
0x7cc, 0x7f2, 0x7dc, 0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cb,
0x7d9, 0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cb, 0x7d9, 0x7d0,
0x7d4, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cb, 0x7d9, 0x7d0, 0x7d4, 0x7ce, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7dd, 0x7ca, 0x7df, 0x7de, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5,
0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd,
0x7ca, 0x7df, 0x7de, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7cc, 0x7f2,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7ca, 0x7df, 0x7de, 0x7d1,
0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd, 0x7d6,
0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd,
0x7d6, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7d6,
0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7ca, 0x7ec, 0x7d9, 0x7ca, 0x7f2, 0x7ec,
0x7db, 0x7cc, 0x7ec, 0x20, 0x7dc, 0x7ed, 0x7ce, 0x7e6, 0x7e3, 0x7ca, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d0, 0x7db, 0x7d1, 0x7d5,
0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d8, 0x7ce, 0x7f0, 0x7de, 0x7df, 0x7cf,
0x20, 0x7e3, 0x7cc, 0x7eb, 0x20, 0x7dd, 0x7ca, 0x7ec, 0x7d9, 0x7ca, 0x7f2, 0x7ec,
0x7db, 0x7cc, 0x7eb, 0x20, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d9, 0x7d1, 0x7d5, 0x7cc,
0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7dc, 0x7ed,
0x7df, 0x7ca, 0x7d4, 0x7ca, 0x7dc, 0x7ed, 0x7d0, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7eb, 0x7dc, 0x7ed, 0x7ca, 0x7f2, 0x7d3, 0x7cc,
0x7e6, 0x7cd, 0x7d9, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7d6, 0x7ed, 0x7d0, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d6, 0x7ed, 0x7d0, 0x7d9, 0x7d1, 0x7d6,
0x7ed, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d6, 0x7ed,
0x7d0, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d6, 0x7ed, 0x7cc, 0x7df,
0x7d1, 0x7d3, 0x7cd, 0x7d9, 0x7d5, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dc, 0x7ed, 0x7d9, 0x7cb, 0x7e3, 0x7cd,
0x7d5, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc, 0x7f2, 0x7df,
0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc,
0x7f2, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca,
0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc,
0x7f2, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3,
0x7cb, 0x20, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc, 0x7f2, 0x7df, 0x7ca, 0x7f2, 0x7d8,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7d3, 0x7cb, 0x20, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc, 0x7f2, 0x7df, 0x7ca,
0x7f2, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7d3, 0x7cb, 0x20, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7cc, 0x7f2, 0x7df, 0x7ca,
0x7f2, 0x7d8, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d6, 0x7cc, 0x7d8, 0x7cf, 0x7ee, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e4, 0x7e5, 0x7ca, 0x7e6, 0x20, 0x2d,
0x20, 0x7ca, 0x7df, 0x7cb, 0x7e6, 0x7ce, 0x7d5, 0x7cc, 0x7e6, 0x7ca, 0x7f2, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e4, 0x7e5, 0x7ca,
0x7e6, 0x20, 0x2d, 0x20, 0x7ca, 0x7df, 0x7cb, 0x7e6, 0x7ce, 0x7d5, 0x7cc, 0x7e6,
0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e4, 0x7e5,
0x7ca, 0x7e6, 0x20, 0x2d, 0x20, 0x7ca, 0x7df, 0x7cb, 0x7e6, 0x7ce, 0x7d5, 0x7cc,
0x7e6, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7e4, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x20,
0x7de, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7e4, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x20, 0x7de, 0x7d0, 0x7f2, 0x7dc,
0x7ed, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e4, 0x7d0, 0x7f2, 0x7dc,
0x7ed, 0x20, 0x7de, 0x7d0, 0x7f2, 0x7dc, 0x7ed, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7e4, 0x7cf, 0x7dd, 0x7ed, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7e4, 0x7cf, 0x7dd, 0x7ed, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2,
0x7d5, 0x7ca, 0x7e4, 0x7cf, 0x7dd, 0x7ed, 0x7d8, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7e4, 0x7cc, 0x7f2, 0x7d8, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7cd, 0x7f2, 0x7d8, 0x7ce, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca,
0x7f2, 0x7d8, 0x7ce, 0x7ef, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9,
0x7cb, 0x7f2, 0x7e4, 0x7cc, 0x7f2, 0x7d8, 0x7cf, 0x7eb, 0x20, 0x7db, 0x7cc, 0x7e3,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7cd, 0x7e1,
0x7ca, 0x7eb, 0x20, 0x7cd, 0x7f2, 0x7d8, 0x7cb, 0x7e3, 0x7cf, 0x7db, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7d3, 0x7d0, 0x20, 0x7cd, 0x7f2, 0x7d8, 0x7cb, 0x7e3, 0x7cf, 0x7db, 0x7cc,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7cd, 0x7f2, 0x7d8, 0x7cb, 0x7e3, 0x7cf, 0x7db,
0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7cc,
0x7d9, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7cc, 0x7d9, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7cc, 0x7d9, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7cc, 0x7d9, 0x7d1, 0x7de,
0x7ce, 0x7d5, 0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7cc, 0x7d9, 0x7d1, 0x7de, 0x7ce, 0x7d5, 0x7d1, 0x7db, 0x7de, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cc, 0x7d9, 0x7d1, 0x7de, 0x7ce, 0x7d5,
0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20,
0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7cc, 0x7db, 0x7cc, 0x7ec, 0x7d9, 0x7ca, 0x7ec,
0x7e6, 0x7cc, 0x7df, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7cc, 0x7db, 0x7cc, 0x7ec, 0x7d9, 0x7ca, 0x7ec, 0x7e6, 0x7cc, 0x7df,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cc, 0x7db, 0x7cc,
0x7ec, 0x7d9, 0x7ca, 0x7ec, 0x7e6, 0x7cc, 0x7df, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d6, 0x7ed,
0x7ca, 0x7d4, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d6, 0x7ed, 0x7ca, 0x7d4, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d6, 0x7ed, 0x7ca, 0x7d4, 0x7d0, 0x7f2, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7d6, 0x7ed, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1,
0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7de, 0x7d6, 0x7ed, 0x7ca,
0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20,
0x7de, 0x7d6, 0x7ed, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7d9,
0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de,
0x7cf, 0x7d9, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de,
0x7cf, 0x7d9, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7de, 0x7cf, 0x7db,
0x7d1, 0x7d9, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7de, 0x7d9, 0x7ca, 0x7db, 0x7d1, 0x7e3, 0x7cf, 0x7e6, 0x7cc, 0x7e6, 0x7ca,
0x7d9, 0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7de, 0x7d9, 0x7ca, 0x7db, 0x7d1, 0x7e3, 0x7cf, 0x7e6, 0x7cc, 0x7e6, 0x7ca,
0x7d9, 0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7de,
0x7d9, 0x7ca, 0x7db, 0x7d1, 0x7e3, 0x7cf, 0x7e6, 0x7cc, 0x7e6, 0x7ca, 0x7d9, 0x7d1,
0x7db, 0x7de, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dc, 0x7ed, 0x7cc, 0x7d9, 0x7d1, 0x7dc, 0x7ed, 0x7cc,
0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7cc, 0x7db, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7de,
0x7cd, 0x7ec, 0x7d9, 0x7cd, 0x7f2, 0x7ec, 0x7d8, 0x7cd, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7df, 0x7d0, 0x7d9, 0x7d8, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7df, 0x7d0, 0x7d9, 0x7d8, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7df, 0x7d0, 0x7d9, 0x7d8, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7e1,
0x7dc, 0x7ed, 0x7ca, 0x7d8, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e1, 0x7dc, 0x7ed, 0x7ca, 0x7d8, 0x7ca, 0x7f2, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e1, 0x7dc, 0x7ed, 0x7ca, 0x7d8,
0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20,
0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7e1, 0x7ca, 0x7df, 0x7cd, 0x7d8, 0x7ce, 0x7ef,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e1, 0x7ca, 0x7df,
0x7d1, 0x7d8, 0x7cc, 0x7dd, 0x7ed, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e1, 0x7ca, 0x7d9, 0x7de, 0x7cc, 0x7db,
0x7ca, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e1,
0x7ca, 0x7d9, 0x7d1, 0x7db, 0x7ca, 0x7df, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e1, 0x7cf, 0x7d9, 0x7cc, 0x7db,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7e1, 0x7cf,
0x7d9, 0x7cc, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb,
0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3, 0x7d8, 0x7cd, 0x7e1, 0x7cf, 0x7d9, 0x7cc, 0x7db,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7ca, 0x7d9, 0x7cb, 0x7f2, 0x7e1, 0x7ca, 0x7e5, 0x7d1, 0x7db, 0x7d0, 0x7f2, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d6, 0x7d0, 0x7eb,
0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x20, 0x7e1, 0x7cd, 0x7f2, 0x7de,
0x7db, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d6, 0x7d0, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2, 0x7d3, 0x7ca, 0x7f2,
0x20, 0x7e1, 0x7cd, 0x7f2, 0x7de, 0x7db, 0x7cc, 0x7de, 0x7cc, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d6, 0x7d0, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2,
0x7d3, 0x7ca, 0x7f2, 0x20, 0x7e1, 0x7cd, 0x7f2, 0x7de, 0x7db, 0x7cc, 0x7de, 0x7cc,
0x7ec, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce,
0x7e1, 0x7d8, 0x7ca, 0x7ce, 0x7df, 0x7ca, 0x7f2, 0x7d3, 0x7d5, 0x7d0, 0x7d9, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ce, 0x7df, 0x7ca, 0x7f2,
0x7d3, 0x7d5, 0x7d0, 0x7d9, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7ce,
0x7df, 0x7ca, 0x7f2, 0x7d3, 0x7d5, 0x7d0, 0x7d9, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7e1, 0x7cf, 0x7db, 0x7d1, 0x7de, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e1, 0x7cf, 0x7db, 0x7d1, 0x7de, 0x7ce, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e1, 0x7cf, 0x7db, 0x7d1, 0x7de, 0x7ce,
0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce,
0x7e1, 0x7d8, 0x7ca, 0x7e1, 0x7cc, 0x7e6, 0x7ca, 0x7e3, 0x7d1, 0x7e1, 0x7ca, 0x7d9,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e3, 0x7ca, 0x7e5,
0x7d9, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca,
0x7e3, 0x7cb, 0x7d4, 0x7ca, 0x7df, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7de, 0x7ca, 0x7df, 0x7cc, 0x7d8, 0x7cf, 0x7e3, 0x7cc, 0x7eb, 0x20,
0x7de, 0x7ce, 0x7d8, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7de, 0x7ca, 0x7df, 0x7cc, 0x7d8, 0x7cf, 0x7e3, 0x7cc, 0x7eb, 0x20,
0x7de, 0x7ce, 0x7d8, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca,
0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7de, 0x7ca, 0x7df, 0x7cc, 0x7d8, 0x7cf, 0x7e3, 0x7cc, 0x7eb, 0x20, 0x7de, 0x7ce,
0x7d8, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20,
0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7e3, 0x7cc, 0x7e6, 0x7ce, 0x2d, 0x7d6, 0x7ed,
0x7cb, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7e3, 0x7cc, 0x7e6, 0x7ce, 0x2d, 0x7d6, 0x7ed, 0x7cb, 0x7df, 0x7ca,
0x7f2, 0x7d8, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e3, 0x7cc, 0x7e6,
0x7ce, 0x2d, 0x7d6, 0x7ed, 0x7cb, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd,
0x7ce, 0x7f2, 0x7d8, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7de, 0x7ce, 0x7d8,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd,
0x7ce, 0x7f2, 0x7d8, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7de, 0x7ce, 0x7d8,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7ce, 0x7f2,
0x7d8, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d8, 0x20, 0x7de, 0x7ce, 0x7d8, 0x7ca, 0x7eb,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7d8, 0x7ca, 0x7e3, 0x7cc, 0x7e6, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e3, 0x7d0, 0x7d9, 0x20, 0x7dd, 0x7d0, 0x7df, 0x7de,
0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7e3, 0x7d0, 0x7d9, 0x20, 0x7dd, 0x7d0, 0x7df, 0x7de, 0x20, 0x7d5, 0x7cc,
0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e3, 0x7d0, 0x7d9, 0x20,
0x7dd, 0x7d0, 0x7df, 0x7de, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd, 0x7cd,
0x7d9, 0x7d1, 0x7e3, 0x7ca, 0x7f2, 0x7d8, 0x7cf, 0x7eb, 0x20, 0x7d8, 0x7cb, 0x7ec,
0x20, 0x7e3, 0x7cf, 0x7d9, 0x7cf, 0x7f2, 0x7e4, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7cd, 0x7d9, 0x7d1, 0x7e3, 0x7ca,
0x7f2, 0x7d8, 0x7cf, 0x7eb, 0x20, 0x7d8, 0x7cb, 0x7ec, 0x20, 0x7e3, 0x7cf, 0x7d9,
0x7cf, 0x7f2, 0x7e4, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca,
0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7dd, 0x7cd, 0x7d9, 0x7d1, 0x7e3, 0x7ca, 0x7f2, 0x7d8, 0x7cf, 0x7eb, 0x20, 0x7d8,
0x7cb, 0x7ec, 0x20, 0x7e3, 0x7cf, 0x7d9, 0x7cf, 0x7f2, 0x7e4, 0x7ca, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7e3, 0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d3, 0x7cc, 0x7d9, 0x7d1, 0x7db, 0x7de,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e3, 0x7dd, 0x7ed,
0x7cf, 0x7db, 0x7d3, 0x7cc, 0x7d9, 0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7e3, 0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d3, 0x7cc, 0x7d9, 0x7d1,
0x7db, 0x7de, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5,
0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7cf, 0x7e1, 0x7d1, 0x7db, 0x7de, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7cf, 0x7e1, 0x7d1, 0x7db, 0x7de, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cf, 0x7e1, 0x7d1, 0x7db, 0x7de, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7d4, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d4, 0x7ca, 0x7de, 0x7cc,
0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8,
0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5,
0x7ca, 0x7d4, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7d4, 0x7df, 0x7ca, 0x7d0, 0x7db, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7d4, 0x7ca, 0x7d3, 0x7ce, 0x7e5, 0x7ca, 0x7eb, 0x20, 0x7d6,
0x7cc, 0x7ec, 0x7e3, 0x7cd, 0x7ec, 0x20, 0x7de, 0x7ce, 0x7d8, 0x7ca, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d4, 0x7d9, 0x7ca, 0x7dc,
0x7ed, 0x7ce, 0x7e5, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7d4, 0x7d9, 0x7ca, 0x7dc, 0x7ed, 0x7ce, 0x7e5, 0x7cb, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d4, 0x7d9, 0x7ca, 0x7dc, 0x7ed, 0x7ce,
0x7e5, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20,
0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d4, 0x7cb, 0x7d9, 0x7ce, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d4, 0x7cb, 0x7d9, 0x7ce, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d4, 0x7cb, 0x7d9, 0x7ce, 0x7eb,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7d8, 0x7ca, 0x7dd, 0x7df, 0x7cc, 0x7d4, 0x7cc, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7df, 0x7cc, 0x7d4, 0x7cc, 0x7f2,
0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce,
0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7df, 0x7cc, 0x7d4,
0x7cc, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20,
0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd, 0x7cf, 0x7e3, 0x7cc, 0x7de, 0x7db, 0x20,
0x7d5, 0x7cc, 0x7f2, 0x20, 0x7e0, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7db, 0x7cd, 0x7f2, 0x2d, 0x7d4, 0x7cc, 0x7e6, 0x7cd,
0x7d9, 0x2d, 0x7cb, 0x2d, 0x7e1, 0x7cc, 0x7de, 0x7cb, 0x7df, 0x7d0, 0x7f2, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7db, 0x7cd, 0x7f2,
0x2d, 0x7d4, 0x7cc, 0x7e6, 0x7cd, 0x7d9, 0x2d, 0x7cb, 0x2d, 0x7e1, 0x7cc, 0x7de,
0x7cb, 0x7df, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca,
0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca,
0x7db, 0x7cd, 0x7f2, 0x2d, 0x7d4, 0x7cc, 0x7e6, 0x7cd, 0x7d9, 0x2d, 0x7cb, 0x2d,
0x7e1, 0x7cc, 0x7de, 0x7cb, 0x7df, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d4, 0x7cc,
0x7d5, 0x7d1, 0x7de, 0x7cd, 0x7d9, 0x7e3, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7d4, 0x7cf, 0x7f2, 0x7d4, 0x7cd, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d4, 0x7cc, 0x7e6, 0x7d0, 0x7f2, 0x7dc,
0x7ed, 0x7e6, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7d9, 0x7cb, 0x7e3, 0x7cc, 0x7e6, 0x7d0, 0x7f2, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7d9, 0x7cf, 0x7d5,
0x7cb, 0x7d9, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7db, 0x7ca, 0x7de, 0x7d1, 0x7e4, 0x7ca, 0x7df, 0x7cc, 0x7e3, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7db, 0x7ca, 0x7de, 0x7d1, 0x7e4,
0x7ca, 0x7df, 0x7cc, 0x7e3, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec,
0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7db,
0x7ca, 0x7de, 0x7d1, 0x7e4, 0x7ca, 0x7df, 0x7cc, 0x7e3, 0x20, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7db, 0x7ca,
0x7e1, 0x7cf, 0x7e5, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7db, 0x7ca, 0x7e1, 0x7cf, 0x7e5, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7db, 0x7ca, 0x7e1, 0x7cf, 0x7e5, 0x7ca, 0x7eb, 0x20,
0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8,
0x7ca, 0x7db, 0x7cb, 0x7e6, 0x7cc, 0x7db, 0x7cc, 0x7df, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7db, 0x7cd, 0x7f2, 0x7dc, 0x7ed, 0x7ca,
0x7d4, 0x7ce, 0x7d9, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7db, 0x7df,
0x7cf, 0x7ec, 0x7e1, 0x7e3, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e5, 0x7d9, 0x7cf, 0x7ec, 0x7d8,
0x7ce, 0x7ee, 0x20, 0x7d6, 0x7ed, 0x7d0, 0x7d9, 0x7d1, 0x7d6, 0x7ed, 0x7cc, 0x7df,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7db,
0x7ce, 0x7d9, 0x7cc, 0x7e3, 0x7ca, 0x7e1, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7db, 0x7e6, 0x7cf, 0x7e5, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7ca, 0x7e4, 0x7d5, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7ca, 0x7e6,
0x7cc, 0x7d4, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7d5, 0x7ca, 0x7e6, 0x7cc, 0x7d4, 0x7cb, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca,
0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7ca, 0x7e6, 0x7cc, 0x7d4, 0x7cb, 0x7eb, 0x20, 0x7d5,
0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7ca, 0x7d6, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca,
0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5,
0x7cf, 0x7de, 0x7cb, 0x7df, 0x7cf, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7cf, 0x7f2, 0x7dc, 0x7ed, 0x7ca, 0x7eb, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7cf, 0x7f2, 0x7dc, 0x7ed,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db,
0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7cf, 0x7f2,
0x7dc, 0x7ed, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d7, 0x7ce, 0x7de, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7ce, 0x7d9, 0x7de, 0x7cc, 0x7e1,
0x7cb, 0x7e3, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7d5, 0x7ce, 0x7d9, 0x7de, 0x7cc, 0x7e1,
0x7cb, 0x7e3, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8,
0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d5, 0x7ce, 0x7d9, 0x7de, 0x7d1, 0x7e1, 0x7cb, 0x7e3,
0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7d5, 0x7ce, 0x7dd,
0x7ed, 0x7ca, 0x7df, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d8, 0x7ca, 0x7ce, 0x7f3, 0x7d9, 0x7d1, 0x7dc, 0x7ed, 0x7cb, 0x7e6, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7ce, 0x7f3, 0x7d9,
0x7d1, 0x7dc, 0x7ed, 0x7cb, 0x7e6, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2,
0x7d5, 0x7ca, 0x7ce, 0x7f3, 0x7d9, 0x7d1, 0x7dc, 0x7ed, 0x7cb, 0x7e6, 0x7cc, 0x7eb,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7d8, 0x7ca, 0x7cf, 0x7d6, 0x7ed, 0x7d1, 0x7d3, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1,
0x7d5, 0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7cf, 0x7d6, 0x7ed, 0x7d1, 0x7d3, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5,
0x7ca, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20,
0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7cf, 0x7d6,
0x7ed, 0x7d1, 0x7d3, 0x7ca, 0x7de, 0x7cc, 0x7db, 0x7d1, 0x7d5, 0x7ca, 0x7f2, 0x7eb,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7ca, 0x7e3, 0x7ce, 0x7e5, 0x7ca, 0x7d5, 0x7ce, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7ca,
0x7e3, 0x7ce, 0x7e5, 0x7ca, 0x7d5, 0x7ce, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2,
0x7d5, 0x7ca, 0x7dd, 0x7ed, 0x7ca, 0x7e3, 0x7ce, 0x7e5, 0x7ca, 0x7d5, 0x7ce, 0x7eb,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1,
0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7e3, 0x7cb, 0x7d6, 0x7ed, 0x7ce, 0x7e6, 0x7cb, 0x7df,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd,
0x7ed, 0x7df, 0x7ca, 0x7d8, 0x7d1, 0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7cf,
0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7ed,
0x7df, 0x7ca, 0x7d8, 0x7d1, 0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7cf, 0x7de,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7ed, 0x7df, 0x7ca, 0x7d8,
0x7d1, 0x7dd, 0x7ed, 0x7cf, 0x7db, 0x7d1, 0x7d5, 0x7cf, 0x7de, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd,
0x7ed, 0x7cf, 0x7df, 0x7d1, 0x7dc, 0x7ed, 0x7cf, 0x7dc, 0x7ed, 0x7d9, 0x7ca, 0x7d8,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7cf,
0x7df, 0x7d1, 0x7dc, 0x7ed, 0x7cf, 0x7dc, 0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x20, 0x7d5,
0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2,
0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7dd, 0x7ed, 0x7cf, 0x7df, 0x7d1, 0x7dc, 0x7ed,
0x7cf, 0x7dc, 0x7ed, 0x7d9, 0x7ca, 0x7d8, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7dd, 0x7ed, 0x7d0, 0x7db,
0x7d1, 0x7d5, 0x7d0, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7e5, 0x7ca, 0x7de, 0x20, 0x7d5, 0x7cc, 0x7f2, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e5, 0x7ca, 0x7df, 0x7cc, 0x7db, 0x20, 0x7e3,
0x7cc, 0x7eb, 0x20, 0x7dd, 0x7d5, 0x7ce, 0x7e3, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e6, 0x7ca, 0x7de, 0x7ce, 0x7d5, 0x7d1,
0x7db, 0x7de, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d8,
0x7ca, 0x7e6, 0x7ca, 0x7de, 0x7ce, 0x7d5, 0x7d1, 0x7db, 0x7de, 0x7cc, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca,
0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7e6, 0x7ca, 0x7de, 0x7ce, 0x7d5, 0x7d1,
0x7db, 0x7de, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7e6, 0x7cb, 0x7de, 0x7ca, 0x7d5, 0x7cb,
0x7d9, 0x7cc, 0x7f2, 0x7d3, 0x7ce, 0x7d9, 0x7dc, 0x7ed, 0x20, 0x7d5, 0x7ce, 0x7ec,
0x7e1, 0x7ca, 0x7ec, 0x7d8, 0x7ca, 0x7e6, 0x7cb, 0x7de, 0x7ca, 0x7d5, 0x7cb, 0x7d9,
0x7cc, 0x7f2, 0x7d3, 0x7ce, 0x7d9, 0x7dc, 0x7ed, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2,
0x7d5, 0x7ca, 0x7e6, 0x7cb, 0x7de, 0x7ca, 0x7d5, 0x7cb, 0x7d9, 0x7cc, 0x7f2, 0x7d3,
0x7ce, 0x7d9, 0x7dc, 0x7ed, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x20, 0x7d5, 0x7ce, 0x7e1, 0x7d8, 0x7ca, 0x7e6, 0x7ce, 0x7de, 0x7d0, 0x7f2, 0x7eb,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7d8, 0x7ca, 0x7ec, 0x20, 0x7db, 0x7ce, 0x7e1,
0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x6af, 0x627, 0x65b, 0x62a, 0x20,
0x645, 0x6cc, 0x646, 0x62c, 0x627, 0x65b, 0x6cc, 0x6cc, 0x6af, 0x627, 0x65b, 0x62a,
0x20, 0x645, 0x6cc, 0x646, 0x62c, 0x627, 0x65b, 0x6cc, 0x6cc, 0x20, 0x626, 0x633,
0x62a, 0x627, 0x65b, 0x646, 0x62f, 0x627, 0x65b, 0x631, 0x62f, 0x631, 0x648, 0x659,
0x634, 0x646, 0x627, 0x6cc, 0x6cc, 0x20, 0x646, 0x626, 0x647, 0x627, 0x62f, 0x627,
0x631, 0x20, 0x631, 0x648, 0x659, 0x632, 0x67, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x67, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70,
0xe1, 0x20, 0x64, 0xe1, 0x62, 0xe1, 0x6c, 0x61, 0x161, 0xe1, 0x69, 0x67,
0x69, 0x67, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68,
0x70, 0xe1, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x69, 0xe1, 0x69, 0x67,
0x69, 0x6e, 0x75, 0x6f, 0x72, 0x74, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x68, 0x70, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x6e, 0x75, 0x6f, 0x72,
0x74, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x64,
0xe1, 0x62, 0xe1, 0x6c, 0x61, 0x161, 0xe1, 0x69, 0x67, 0x69, 0x6e, 0x75,
0x6f, 0x72, 0x74, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x6f,
0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x6f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x64, 0xe1, 0x62, 0xe1, 0x6c,
0x61, 0x161, 0xe1, 0x69, 0x67, 0x69, 0x6f, 0x61, 0x72, 0x6a, 0x65, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x20, 0x67, 0x61, 0x73, 0x6b, 0x6b, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2d, 0xe1, 0x69,
0x67, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2d, 0x64, 0xe1, 0x62,
0xe1, 0x6c, 0x61, 0x161, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x6f, 0x73, 0x6b,
0x76, 0x61, 0x2d, 0x67, 0x65, 0x61, 0x73, 0x73, 0x69, 0xe1, 0x69, 0x67,
0x69, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x68, 0x61, 0x6e,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x41, 0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x68,
0x6b, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4c, 0x75, 0x6c, 0x6c, 0x69,
0x2d, 0x41, 0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d,
0x41, 0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x68, 0x6b,
0xe1, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f,
0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x64, 0xe1, 0x62, 0xe1, 0x6c, 0x61,
0x161, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x64,
0xe1, 0x62, 0xe1, 0x6c, 0x61, 0x161, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x20, 0x6e, 0x75,
0x6f, 0x72, 0x74, 0x61, 0x6e, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x20, 0x6e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x6e, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x20, 0x6e, 0x75, 0x6f,
0x72, 0x74, 0x61, 0x6e, 0x64, 0x75, 0x6f, 0x74, 0x74, 0x61, 0x72, 0xe1,
0x69, 0x67, 0x69, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0x64, 0x75, 0x6f, 0x74,
0x74, 0x61, 0x72, 0xe1, 0x69, 0x67, 0x69, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0x64, 0x75, 0x6f, 0x74, 0x74, 0x61, 0x72, 0xe1, 0x69, 0x67, 0x69,
0x4a, 0x61, 0x73, 0x6b, 0x65, 0x73, 0xe1, 0x62, 0x69, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4a, 0x61, 0x73, 0x6b, 0x65, 0x73, 0xe1, 0x62, 0x69, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x73,
0x6b, 0x65, 0x73, 0xe1, 0x62, 0x69, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0xe1, 0x62, 0x69,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72,
0xe1, 0x62, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a,
0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x61, 0x74, 0x6c, 0xe1, 0x6e, 0x74,
0x61, 0x6c, 0x61, 0x161, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x61, 0x74, 0x6c,
0xe1, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x161, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x61, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x61,
0x6c, 0x61, 0x161, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61,
0x73, 0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47,
0x61, 0x73, 0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x61, 0x72, 0x6a, 0x6a, 0x61, 0x62,
0x65, 0x61, 0x6c, 0x69, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73,
0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x20, 0x6f, 0x61, 0x72, 0x6a, 0x6a, 0x61, 0x62, 0x65, 0x61, 0x6c, 0x69,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61,
0x73, 0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x61, 0x72, 0x6a, 0x6a, 0x61, 0x62, 0x65, 0x61, 0x6c,
0x69, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f,
0x72, 0x74, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e,
0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61,
0x72, 0x6a, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f,
0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x17e, 0x61, 0x6e, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69,
0x17e, 0x61, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x17e, 0x61, 0x6e,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x7a, 0x6f, 0x72, 0x61, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x7a, 0x6f, 0x72, 0x61, 0x69, 0x64, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x69, 0x64,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x68,
0x75, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x6f,
0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b,
0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x61, 0x70, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x10c, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4b, 0x69, 0x69, 0x6e, 0x6e, 0xe1, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x4b, 0x69, 0x69, 0x6e, 0x6e, 0xe1, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x69, 0x69, 0x6e, 0x6e,
0xe1, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4a, 0x75, 0x6f, 0x76, 0x6c, 0x61, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x73, 0x75, 0x6c,
0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43,
0x6f, 0x6f, 0x6b, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x75, 0x6c, 0x6c, 0x6f,
0x69, 0x64, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20,
0x67, 0x65, 0x61, 0x73, 0x69, 0x20, 0x62, 0x65, 0x61, 0x6c, 0x6c, 0x65,
0x6d, 0x75, 0x74, 0x74, 0x75, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x75,
0x62, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x75, 0x62, 0x61, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x75, 0x62,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x44, 0x61, 0x76, 0x69, 0x73, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72,
0x74, 0x61, 0x2d, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x42, 0x65, 0x61, 0x73, 0x73, 0xe1, 0x161, 0x73, 0x75, 0x6c,
0x6c, 0x6f, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x65, 0x61, 0x73, 0x73,
0xe1, 0x161, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x65, 0x61, 0x73, 0x73, 0xe1, 0x161,
0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47,
0x61, 0x73, 0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61,
0x73, 0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75,
0x6f, 0x72, 0x74, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0xe1, 0x69, 0x64, 0x64, 0x75,
0x73, 0x2d, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x74, 0x69, 0x20, 0x65, 0x75,
0x72, 0x6f, 0x68, 0x70, 0x61, 0x6c, 0x61, 0x161, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68,
0x70, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x75, 0x6c, 0x6c, 0x75, 0x69, 0x64, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x75,
0x6c, 0x6c, 0x75, 0x69, 0x64, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x75, 0x6c, 0x6c, 0x75, 0x69, 0x64, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x72, 0x61, 0x6e, 0x6b,
0x72, 0x69, 0x69, 0x6b, 0x6b, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x72, 0x61, 0x6e, 0x6b, 0x72,
0x69, 0x69, 0x6b, 0x6b, 0x61, 0x20, 0x6c, 0x75, 0x6c, 0x6c, 0x69, 0x20,
0x26, 0x20, 0x61, 0x6e, 0x74, 0xe1, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61,
0x2d, 0x52, 0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61, 0x6d,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61,
0x2d, 0x52, 0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61, 0x6d,
0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e,
0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x52, 0x75, 0x6f, 0x6e, 0xe1, 0x65,
0x61, 0x74, 0x6e, 0x61, 0x6d, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x52,
0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61, 0x6d, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x52, 0x75,
0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61, 0x6d, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a,
0x65, 0x2d, 0x52, 0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61,
0x6d, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x47, 0x6f, 0x6c, 0x66, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x68, 0x74, 0x61, 0x6c, 0x61, 0x161, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x61, 0x6c, 0x65,
0x75, 0x68, 0x74, 0x61, 0x6c, 0x61, 0x161, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x61, 0x6c, 0x65, 0x75, 0x68, 0x74, 0x61, 0x6c, 0x61, 0x161, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x49, 0x6e, 0x64, 0x69, 0x61, 0xe1, 0x62, 0x69, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x69, 0x6e, 0x6e, 0xe1,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61,
0x72, 0x6a, 0x65, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x61, 0x6e, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x61, 0x6e, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x61, 0x6e, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x49,
0x72, 0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x49,
0x72, 0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x70,
0xe1, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x70, 0xe1,
0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4a, 0x61, 0x70, 0xe1, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d,
0x4b, 0x61, 0x73, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x4b, 0x61, 0x73,
0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4c, 0x69, 0x6e, 0x65,
0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x6c, 0x65, 0x73,
0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x6c, 0x65, 0x64,
0x69, 0x69, 0x76, 0x76, 0x61, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x69, 0x69, 0x64,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x6f, 0x20, 0x4a, 0xe1, 0x73, 0x6b, 0x65, 0x73, 0xe1,
0x62, 0x69, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x20, 0x4a, 0xe1, 0x73, 0x6b, 0x65, 0x73, 0xe1, 0x62, 0x69,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x4a, 0xe1, 0x73, 0x6b, 0x65, 0x73,
0xe1, 0x62, 0x69, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x61,
0x75, 0x72, 0x75, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x53, 0x65, 0x6c, 0xe1,
0x6e, 0x64, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x111, 0x111, 0x61,
0x2d, 0x53, 0x65, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x53,
0x65, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x69,
0x75, 0x65, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50,
0x65, 0x72, 0x75, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69,
0x6e, 0x6e, 0x61, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x6e, 0x61, 0x69, 0x64, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x6e, 0x61, 0x69, 0x64, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x61, 0x69, 0x64, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72,
0x65, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x53,
0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69,
0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4c, 0x75, 0x6c, 0x6c, 0x69, 0x2d,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54,
0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x61, 0x17e,
0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x57, 0x61,
0x6b, 0x65, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6a, 0x61, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61,
0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61,
0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x61, 0x6b, 0x6f, 0x20, 0x50, 0x61,
0x6c, 0x6f, 0x67, 0x61, 0x72, 0x65, 0x20, 0x79, 0x61, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x41, 0x63, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x63, 0x72,
0x65, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x61,
0x66, 0x67, 0x68, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73,
0xf8, 0x72, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x61, 0x6c, 0x61, 0x73, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69,
0x64, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2c, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x61, 0x73, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x74, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x64, 0x65, 0x74, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x74, 0x20, 0x73, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0xf8, 0x73, 0x74, 0x6b, 0x79, 0x73, 0x74, 0x65,
0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0xf8,
0x73, 0x74, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0xf8, 0x73, 0x74, 0x6b, 0x79, 0x73,
0x74, 0x65, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x20, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x73, 0x20, 0x28, 0x55, 0x53, 0x41,
0x29, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x20, 0x4d, 0x6f, 0x75,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x73, 0x20, 0x28, 0x55, 0x53, 0x41, 0x29,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x20, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x73, 0x20, 0x28, 0x55, 0x53, 0x41, 0x29, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0x74, 0x69, 0x6c,
0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0x74,
0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74,
0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x53, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79,
0x73, 0x74, 0x65, 0x6e, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20,
0x28, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x29, 0x20, 0x74, 0x69, 0x64,
0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x29, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x29, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x41,
0x70, 0x69, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x71, 0x74,
0x61, 0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2c, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x6f, 0x62,
0x65, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74,
0x69, 0x64, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x2c, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x61, 0x72, 0x61, 0x62, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69,
0x64, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x76, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x76, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61,
0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x65, 0x72, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x61, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x65, 0x72, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79,
0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x65, 0x72, 0x68, 0x61,
0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74,
0x2d, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73,
0x74, 0x2d, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x2d, 0x73, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0xf8,
0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0xf8, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x76,
0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x61, 0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x73, 0x65, 0x72,
0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x73, 0x65, 0x72,
0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x61, 0x73, 0x6f, 0x72,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x73, 0x6f, 0x72, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x61, 0x73, 0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x62, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x62, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x62, 0x68, 0x75, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x62, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x6b, 0x61, 0x70,
0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x6b, 0x61, 0x70, 0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b, 0x61, 0x70,
0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x43, 0x61, 0x73, 0x65, 0x79, 0x2d,
0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x63, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x63, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x63, 0x68, 0x69, 0x6c,
0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b, 0x69, 0x6e, 0x65,
0x73, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73,
0xf8, 0x79, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0xf8, 0x79, 0x65,
0x6e, 0x65, 0x63, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x63, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x63, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x68, 0x61, 0x6c, 0x76,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x79, 0x65, 0x6e, 0x65,
0x63, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x63,
0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x63, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x44, 0x61,
0x76, 0x69, 0x73, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0xf8, 0x73, 0x74, 0x74,
0x69, 0x6d, 0x6f, 0x72, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0xe5,
0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61, 0x65, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x72, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0xf8, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0xf8, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0xf8, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x66, 0x6a,
0x65, 0x72, 0x6e, 0x2d, 0xf8, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x76, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x76, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x79, 0x65,
0x6e, 0x65, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x66, 0x69,
0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x66, 0x69,
0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x66, 0x69, 0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x44, 0x65, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0xf8, 0x72, 0x74, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69,
0x65, 0x72, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73,
0xf8, 0x79, 0x65, 0x6e, 0x65, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0xf8, 0x79, 0x65,
0x6e, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20,
0x6d, 0x69, 0x64, 0x64, 0x65, 0x6c, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74,
0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0xf8, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0xf8, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x76, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x67, 0x72,
0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x67, 0x72,
0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x47, 0x75, 0x61, 0x6d, 0x2d, 0x74,
0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x50, 0x65, 0x72, 0x73, 0x69, 0x61, 0x62, 0x75, 0x6b,
0x74, 0x61, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x20, 0x6f, 0x67, 0x20,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x65, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x65, 0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x65,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x68, 0x6f, 0x76,
0x64, 0x69, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x61, 0x76, 0x65, 0x74, 0x69, 0x6e,
0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf8,
0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x69, 0x72,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x69, 0x72, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6a, 0x61,
0x70, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x6a, 0x29, 0x20, 0x74, 0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x28, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x6a, 0x29, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b,
0x20, 0x28, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x6a, 0x29, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65,
0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x6b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x2d, 0x74, 0x69,
0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4c, 0x69, 0x6e, 0x6a, 0x65, 0xf8, 0x79, 0x65, 0x6e, 0x65,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0xf8,
0x79, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x2d, 0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x2d, 0xf8, 0x79, 0x61, 0x4d, 0x61, 0x63, 0x61,
0x75, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x4d,
0x61, 0x63, 0x61, 0x75, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x6d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x6d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0x74, 0x69, 0x6c, 0x6c, 0x65,
0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x53, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61,
0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65,
0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x53, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73,
0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x6f,
0x73, 0x6b, 0x76, 0x61, 0x6d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x6e, 0x61, 0x75, 0x72, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x6e, 0x65, 0x70, 0x61, 0x6c, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6e, 0x65, 0x77, 0x7a, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6e, 0x65,
0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6e, 0x65, 0x77, 0x7a,
0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4e, 0x69, 0x75, 0x65, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0xf8, 0x79, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0xf8, 0x79, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x4d, 0x61,
0x72, 0x69, 0x61, 0x6e, 0x65, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x70, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x70, 0x61, 0x6c, 0x61, 0x75, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x70, 0x61, 0x70, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x70, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x70, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x66, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x66, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e,
0x69, 0x78, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65,
0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0x6f,
0x68, 0x6e, 0x70, 0x65, 0x69, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69,
0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2c, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x52, 0xe9,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73,
0x6b, 0x20, 0x28, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x29, 0x20, 0x74,
0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x29, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20,
0x28, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x29, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x73, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x6f, 0x72, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73,
0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x53, 0xf8, 0x72, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x73, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x74, 0x61, 0x68, 0x69, 0x74,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x74, 0x61, 0x64, 0x73, 0x6a, 0x69,
0x6b, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0xf8, 0x79,
0x65, 0x6e, 0x65, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x74,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x75, 0x76, 0x61, 0x6c, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x75, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x75, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x75, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x75, 0x73, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65,
0x6c, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6f,
0x67, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0xf8, 0x79, 0x65, 0x6e,
0x65, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x61, 0x75, 0x73, 0x74, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65,
0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x76,
0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x6c,
0x61, 0x73, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x61, 0x73, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x61, 0x75, 0x73, 0x74, 0x6b,
0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x61, 0x75, 0x73, 0x74, 0x6b, 0x79, 0x73, 0x74, 0x65,
0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x61,
0x75, 0x73, 0x74, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x52,
0x6f, 0x63, 0x6b, 0x79, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x73, 0x20, 0x28, 0x55, 0x53, 0x41, 0x29, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68,
0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c,
0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x69,
0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65,
0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x61, 0x72, 0x61, 0x62, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x65, 0x72,
0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74,
0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x2d, 0x73, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69,
0x64, 0x76, 0x65, 0x73, 0x74, 0x2d, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x61, 0x75, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74,
0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74,
0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x61, 0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x61, 0x73, 0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x6b, 0x61, 0x70, 0x70, 0x76, 0x65, 0x72, 0x64, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x63, 0x68, 0x69,
0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x6f,
0x6b, 0x6f, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x79,
0x61, 0x6e, 0x65, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x79, 0x61,
0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x79, 0x61, 0x6e,
0x65, 0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x61, 0x75, 0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72,
0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50,
0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61, 0x73, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x73,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x61, 0x75, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x66, 0x6a, 0x65, 0x72, 0x6e, 0x2d,
0x61, 0x75, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8,
0x79, 0x61, 0x6e, 0x65, 0x66, 0x69, 0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x44,
0x65, 0x69, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0xf8, 0x72, 0x74, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x61,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0xf8, 0x79,
0x61, 0x6e, 0x65, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x61, 0x75,
0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x65,
0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x6b, 0x69, 0x6e, 0x65,
0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x68, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x68, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x61, 0x75, 0x73, 0x74, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x61, 0x70, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x61, 0x75, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b, 0x6f, 0x72,
0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0xf8, 0x79, 0x61,
0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x2d, 0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0xf8, 0x79, 0x61, 0x6e, 0x65, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68,
0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76,
0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e,
0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b,
0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x61, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6e,
0x79, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x70,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x70, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x66,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65,
0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x61, 0x6d, 0x6f,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0xf8, 0x79,
0x61, 0x6e, 0x65, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x75, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x75, 0x73, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x57, 0x61, 0x6c,
0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6f, 0x67, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x53, 0x75, 0x64, 0x61, 0x66, 0x72, 0x69, 0x63, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xc8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0xc8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc8, 0x73, 0x74,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x6d, 0x6f,
0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41,
0x70, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x61, 0x62,
0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f,
0xe8, 0x73, 0x74, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f,
0xe8, 0x73, 0x74, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74,
0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x65, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x41, 0xe7, 0xf2, 0x72,
0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x41, 0xe7, 0xf2, 0x72, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x41, 0xe7, 0xf2, 0x72, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x61, 0x6c, 0x64, 0x65, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x61, 0x6c,
0x64, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x61,
0x6c, 0x64, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x42, 0x75, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x64, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c,
0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x6d, 0x69, 0x65, 0x6a, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0xf2, 0x72,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x6c, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x63, 0x75, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x71,
0x75, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0xf2, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0xf2, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73,
0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x78,
0x74, 0x72, 0x65, 0x6d, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2,
0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69,
0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61,
0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69,
0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61,
0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65,
0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x61,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x54, 0x65, 0x72, 0x72,
0x65, 0x73, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73,
0x20, 0x65, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x71, 0x75,
0x65, 0x73, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65,
0xf2, 0x72, 0x67, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x61, 0x6c, 0x20,
0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x20, 0x64,
0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8,
0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x74, 0x68, 0x20, 0x47, 0xf2, 0x6c, 0x66, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0xe8, 0x6c,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0xe8, 0x6c, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x2019, 0x49, 0x73, 0x72, 0x61, 0xe8, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70,
0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x61, 0x74, 0x61, 0x6c, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x43, 0x6f, 0x72, 0xe8, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x73, 0x20, 0x45, 0x73, 0x70, 0xf2, 0x72, 0x61, 0x64, 0x65,
0x73, 0x20, 0x45, 0x71, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c,
0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20,
0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x65, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0xf2, 0x76,
0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e,
0xf2, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x2d, 0x4e, 0xf2, 0x76, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x2d,
0x4e, 0xf2, 0x76, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61,
0x2d, 0x4e, 0xf2, 0x76, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x69, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x6f, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e,
0xf2, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0xe8, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x50,
0x65, 0x72, 0xfa, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46,
0xe9, 0x6e, 0x69, 0x78, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x52, 0x65, 0xfc,
0x6e, 0x69, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6a, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6a, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6a, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c,
0x6c, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6c, 0x6c,
0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d,
0x62, 0x75, 0x72, 0x67, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x64, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x75, 0x6f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x73,
0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x73, 0x20,
0x52, 0x6f, 0x63, 0x6f, 0x73, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x73, 0x20, 0x4d,
0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x73, 0x20, 0x52, 0x6f, 0x63,
0x6f, 0x73, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x74, 0x68, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x74, 0x68, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x61, 0x72,
0x61, 0x62, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x67, 0x65, 0x72, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x72,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x72,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0xf2,
0x72, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0xf2,
0x72, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0xf2, 0x72, 0x65,
0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61,
0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x69,
0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x64, 0x61, 0x75,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x69, 0x73, 0x6c,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x65, 0x73,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49,
0x73, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x6d, 0x69, 0x65,
0x6a, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x65, 0x73,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75,
0x62, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d,
0xf2, 0x72, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x72, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20,
0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71,
0x75, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2,
0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x20, 0x65,
0x78, 0x74, 0x72, 0x65, 0x6d, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2,
0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0xf2, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x69, 0x73,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x65, 0x73, 0x20,
0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x73, 0x20,
0x69, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x47,
0x75, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63,
0x65, 0x73, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6e,
0x74, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x20, 0x65, 0x20, 0x54, 0x65,
0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x73, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65,
0x73, 0x69, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67,
0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x65, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x6d, 0x65, 0x72, 0x69,
0x64, 0x69, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x20, 0x4f, 0x63, 0x65,
0x61, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49,
0x73, 0x72, 0x61, 0xe8, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x45, 0x73, 0x70, 0xf2, 0x72,
0x61, 0x64, 0x65, 0x73, 0x20, 0x45, 0x71, 0x75, 0x61, 0x74, 0x6f, 0x72,
0x69, 0x61, 0x6c, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20,
0x49, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65,
0x73, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49,
0x73, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61,
0x63, 0xed, 0x66, 0x69, 0x63, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61,
0x2d, 0x4e, 0xf2, 0x76, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72,
0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x72,
0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x4f, 0x6d,
0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x74,
0x68, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x74, 0x68, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x75, 0x20, 0x64, 0x65, 0x20, 0x50,
0x65, 0x72, 0xfa, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73,
0x20, 0x49, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0x65, 0xfc,
0x6e, 0x69, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a,
0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20,
0x49, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0xf2, 0x72,
0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x53, 0x75, 0x64,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0xb06, 0xb15, 0xb3e, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb15, 0xb3e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2b, 0xb17, 0xb3e, 0xb28, 0xb3f,
0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb27, 0xb4d,
0xb5f, 0x20, 0xb06, 0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb06, 0xb2b, 0xb4d, 0xb30, 0xb3f,
0xb15, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb26, 0xb15, 0xb4d, 0xb37, 0xb3f, 0xb23,
0x20, 0xb06, 0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a,
0xb3f, 0xb2e, 0x20, 0xb06, 0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb2b, 0xb4d,
0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06,
0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f,
0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06,
0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32,
0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb32, 0xb2e, 0xb3e, 0xb1f, 0xb40,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb32, 0xb2e, 0xb3e, 0xb1f, 0xb40, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb32, 0xb2e, 0xb3e, 0xb1f,
0xb40, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb2e, 0xb3e, 0xb1c, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06,
0xb2e, 0xb3e, 0xb1c, 0xb28, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2e, 0xb3e, 0xb1c, 0xb28, 0xb4d, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40,
0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30,
0xb40, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40, 0xb5f, 0x20,
0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb26, 0xb3f,
0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30,
0xb4d, 0xb2c, 0xb24, 0xb4d, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30,
0xb4d, 0xb2c, 0xb24, 0xb4d, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb4d, 0xb2c, 0xb24, 0xb4d,
0xb5f, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb38,
0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb28, 0xb3e, 0xb21, 0xb3f, 0xb30, 0xb4d,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb28, 0xb3e, 0xb21, 0xb3f, 0xb30, 0xb4d, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb28, 0xb3e, 0xb21,
0xb3f, 0xb30, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06,
0xb2a, 0xb3f, 0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e,
0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb24, 0xb4c, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb24, 0xb4c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb24, 0xb4c, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb4d, 0xb24,
0xb4b, 0xb2c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb4d, 0xb24, 0xb4b, 0xb2c,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb4d,
0xb24, 0xb4b, 0xb2c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb2c, 0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb06, 0xb30, 0xb2c, 0xb40, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb2c, 0xb40, 0xb5f, 0x20, 0xb26,
0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30,
0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb30, 0xb4d,
0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47,
0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e,
0x20, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb4d, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb06, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb4d, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb06, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06,
0xb30, 0xb4d, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb06, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1f,
0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb06, 0xb1f, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1f,
0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c,
0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb27, 0xb4d, 0xb5f,
0x20, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f,
0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30,
0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb26, 0xb3f,
0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d,
0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d, 0xb5f,
0x20, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05,
0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27,
0xb4d, 0xb5f, 0x20, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d,
0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d, 0xb5f,
0x20, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e,
0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c,
0x20, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f,
0xb5f, 0x20, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d,
0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20,
0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30,
0xb47, 0xb32, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f,
0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f,
0xb2e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20,
0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32,
0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb47, 0xb30, 0xb2c, 0xb3e,
0xb07, 0xb1c, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb47, 0xb30,
0xb2c, 0xb3e, 0xb07, 0xb1c, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb47, 0xb30, 0xb2c, 0xb3e, 0xb07,
0xb1c, 0xb3e, 0xb28, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb4b, 0xb30, 0xb47, 0xb38, 0xb4d, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb1c, 0xb4b, 0xb30, 0xb47, 0xb38, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb4b, 0xb30, 0xb47, 0xb38, 0xb4d,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb3e, 0xb02, 0xb32, 0xb3e, 0xb26, 0xb47, 0xb36, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb3e, 0xb02, 0xb32, 0xb3e, 0xb26, 0xb47, 0xb36, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c,
0xb3e, 0xb02, 0xb32, 0xb3e, 0xb26, 0xb47, 0xb36, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2d, 0xb41, 0xb1f, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb32, 0xb3f,
0xb2d, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb38,
0xb3f, 0xb32, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3e,
0xb38, 0xb3f, 0xb32, 0xb3f, 0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb3f, 0xb32,
0xb3f, 0xb06, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e,
0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb41, 0xb28, 0xb47, 0xb07,
0x20, 0xb21, 0xb3e, 0xb30, 0xb41, 0xb38, 0xb3e, 0xb32, 0xb3e, 0xb2e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c, 0x20, 0xb2d, 0xb30, 0xb4d, 0xb26,
0xb47, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c, 0x20, 0xb2d,
0xb30, 0xb4d, 0xb26, 0xb47, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c, 0x20, 0xb2d, 0xb30,
0xb4d, 0xb26, 0xb47, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15,
0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3e, 0xb2e, 0xb4b,
0xb30, 0xb4b, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a,
0xb3e, 0xb25, 0xb3e, 0xb2e, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3e,
0xb25, 0xb3e, 0xb2e, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3e, 0xb25, 0xb3e, 0xb2e, 0xb4d, 0x200c,
0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb1a, 0xb3f, 0xb32, 0xb3f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3f, 0xb32, 0xb3f,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb1a, 0xb3f, 0xb32, 0xb3f, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb40, 0xb28, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb1a, 0xb40, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb1a, 0xb40, 0xb28, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb16, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb1f,
0xb2e, 0xb3e, 0xb38, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb15, 0xb4b, 0xb15, 0xb4b, 0xb38, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71,
0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15,
0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb32,
0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb06, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb15, 0xb41, 0xb15, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a,
0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb41, 0xb15, 0xb4d,
0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb41, 0xb15,
0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d,
0xb1c, 0x20, 0xb05, 0xb30, 0xb4d, 0xb26, 0xb4d, 0xb27, 0x20, 0xb17, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15,
0xb4d, 0xb5f, 0xb41, 0xb2c, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb5f,
0xb41, 0xb2c, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb5f, 0xb41, 0xb2c, 0xb3e, 0x20, 0xb26, 0xb3f,
0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb21, 0xb47, 0xb2d,
0xb3f, 0xb38, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb21, 0xb41, 0xb2e, 0xb4b,
0xb23, 0xb4d, 0xb1f, 0x2d, 0xb21, 0xb3f, 0x2018, 0xb09, 0xb30, 0xb2d, 0xb3f, 0xb32,
0xb4d, 0xb32, 0xb47, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c,
0x20, 0xb24, 0xb3f, 0xb2e, 0xb4b, 0xb30, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb07, 0xb37, 0xb4d, 0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb06, 0xb07, 0xb32, 0xb4d,
0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb37, 0xb4d,
0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb06, 0xb07, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23,
0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb07, 0xb37, 0xb4d, 0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb06, 0xb07,
0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb15,
0xb4d, 0xb71, 0xb47, 0xb21, 0xb30, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28,
0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40, 0xb5f, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a,
0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d,
0xb30, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d,
0xb30, 0xb40, 0xb5f, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33,
0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f,
0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e,
0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb30, 0xb2c, 0xb30, 0xb4d, 0xb24, 0xb4d, 0xb24, 0xb40,
0x2d, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20,
0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f,
0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36,
0xb4d, 0xb1a, 0xb3f, 0xb2e, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41,
0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0xb3e,
0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb15, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21,
0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb15, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21,
0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb15, 0xb32, 0xb4d,
0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a,
0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb1c, 0xb3f, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb1c, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb1c, 0xb3f, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2b, 0xb4d, 0xb30, 0xb47, 0xb1e, 0xb4d, 0xb1a, 0x20, 0xb17, 0xb41, 0xb06,
0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb4d, 0xb30, 0xb47, 0xb1e, 0xb4d,
0xb1a, 0x20, 0xb26, 0xb15, 0xb4d, 0xb37, 0xb3f, 0xb23, 0x20, 0xb0f, 0xb2c, 0xb02,
0x20, 0xb06, 0xb23, 0xb4d, 0xb1f, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb3e, 0xb1f, 0xb3f,
0xb15, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb3e, 0xb32, 0xb3e, 0xb2a,
0xb3e, 0xb17, 0xb4b, 0xb38, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb3e, 0xb2e,
0xb4d, 0xb2c, 0xb3f, 0xb5f, 0xb30, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb30,
0xb4d, 0xb1c, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb30, 0xb4d, 0xb1c,
0xb3f, 0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb06, 0x20, 0xb16, 0xb30, 0xb3e,
0xb26, 0xb3f, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb3f, 0xb32, 0xb2c, 0xb30,
0xb4d, 0xb1f, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41,
0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28,
0xb71, 0xb3f, 0xb1a, 0xb4d, 0x20, 0xb2e, 0xb3f, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28,
0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb32,
0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d,
0xb21, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e,
0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21,
0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21,
0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb28, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb32, 0xb4d,
0xb2b, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb17, 0xb41, 0xb0f, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb71,
0xb3e, 0xb07, 0x2d, 0xb06, 0xb32, 0xb47, 0xb09, 0xb1f, 0xb3f, 0xb5f, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb39, 0xb71, 0xb3e, 0xb07, 0x2d, 0xb06, 0xb32, 0xb47, 0xb09, 0xb1f,
0xb3f, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb39, 0xb71, 0xb3e, 0xb07, 0x2d, 0xb06, 0xb32, 0xb47, 0xb09, 0xb1f,
0xb3f, 0xb5f, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb39, 0xb02, 0x20, 0xb15, 0xb02, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39,
0xb02, 0x20, 0xb15, 0xb02, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb39, 0xb02, 0x20, 0xb15, 0xb02, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb4b, 0xb2d,
0xb21, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb4b, 0xb2d, 0xb21, 0xb4d,
0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb4b,
0xb2d, 0xb21, 0xb4d, 0x200c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb30, 0xb24, 0xb40,
0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e,
0xb30, 0xb24, 0x20, 0xb2e, 0xb3e, 0xb39, 0xb3e, 0xb38, 0xb3e, 0xb17, 0xb30, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb4b, 0xb1a, 0xb3e, 0xb07, 0xb28,
0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb07, 0xb23,
0xb4d, 0xb21, 0xb4b, 0xb28, 0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb4b, 0xb28,
0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a,
0xb3f, 0xb2e, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb4b, 0xb28, 0xb47, 0xb38, 0xb3f,
0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb30, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb07, 0xb30, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb07, 0xb30, 0xb3e, 0xb28, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32,
0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb05, 0xb30, 0xb15, 0xb41, 0xb1f,
0xb38, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb30, 0xb4d, 0xb15, 0xb41,
0xb1f, 0xb38, 0xb4d, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb07, 0xb30, 0xb4d, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb4d,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40,
0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb38, 0xb4d, 0xb30, 0xb3e, 0xb0f, 0xb32,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb38, 0xb4d, 0xb30, 0xb3e, 0xb0f, 0xb32, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07,
0xb38, 0xb4d, 0xb30, 0xb3e, 0xb0f, 0xb32, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32,
0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb26,
0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb47,
0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb2a, 0xb3e, 0xb2d, 0xb32, 0xb4b, 0xb2d, 0xb4d, 0xb38,
0xb15, 0x2d, 0xb15, 0xb2e, 0xb1a, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb3f, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb47, 0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb2a, 0xb3e, 0xb2d, 0xb32,
0xb4b, 0xb2d, 0xb4d, 0xb38, 0xb15, 0x2d, 0xb15, 0xb2e, 0xb1a, 0xb1f, 0xb38, 0xb4d,
0xb15, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb47, 0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb2a, 0xb3e, 0xb2d, 0xb32, 0xb4b, 0xb2d, 0xb4d,
0xb38, 0xb15, 0x2d, 0xb15, 0xb2e, 0xb1a, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb3f, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15,
0xb3e, 0xb1c, 0xb3e, 0xb15, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb15, 0xb3e, 0xb1c, 0xb3e,
0xb15, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb15, 0xb3e, 0xb1c, 0xb3e, 0xb15, 0xb38,
0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4b, 0xb30, 0xb3f,
0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4b, 0xb30, 0xb3f, 0xb5f, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4b, 0xb30, 0xb3f, 0xb5f,
0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb15, 0xb4b, 0xb38, 0xb30, 0xb47, 0xb07, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d,
0xb30, 0xb3e, 0xb38, 0xb28, 0xb4b, 0xb5f, 0xb3e, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb28, 0xb4b, 0xb5f, 0xb3e,
0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb28, 0xb4b, 0xb5f,
0xb3e, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb3f, 0xb30, 0xb4d,
0xb17, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb32,
0xb19, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb32, 0xb3e, 0xb07, 0xb28,
0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d,
0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb4b,
0xb71, 0xb47, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39,
0xb4b, 0xb71, 0xb47, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb4b, 0xb71, 0xb47,
0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2e, 0xb3e, 0xb15, 0xb3e, 0xb13, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb15, 0xb3e,
0xb13, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb15,
0xb3e, 0xb13, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb21, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb21, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e,
0xb17, 0xb3e, 0xb21, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb32,
0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb33, 0xb26,
0xb4d, 0xb71, 0xb40, 0xb2a, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb30, 0xb4d,
0xb15, 0xb4d, 0xb71, 0xb47, 0xb38, 0xb3e, 0xb38, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb36, 0xb3e, 0xb32, 0xb4d, 0x200c, 0x20, 0xb26,
0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2e, 0xb4c, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2e, 0xb4c, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb4c, 0xb30,
0xb3f, 0xb38, 0xb38, 0xb4d, 0x200c, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb71, 0xb38, 0xb28, 0xb4d, 0x200c, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb47, 0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb3e, 0xb28,
0xb4d, 0x20, 0xb2a, 0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2e, 0xb47, 0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb3e, 0xb28, 0xb4d,
0x20, 0xb2a, 0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb47, 0xb15,
0xb4d, 0xb38, 0xb3f, 0xb15, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb2a, 0xb3e, 0xb38, 0xb3f,
0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb32, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb2c, 0xb3e,
0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb32, 0xb3e, 0xb28,
0xb4d, 0x200c, 0xb2c, 0xb3e, 0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb32, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb2c,
0xb3e, 0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb38,
0xb4d, 0xb15, 0xb4b, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb4b,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2e, 0xb38, 0xb4d, 0xb15, 0xb4b, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3f, 0xb06, 0xb01,
0xb2e, 0xb3e, 0xb30, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb3e, 0xb09,
0xb30, 0xb41, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb47, 0xb2a, 0xb3e, 0xb33, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb15, 0xb3e, 0xb32, 0xb47,
0xb21, 0xb4b, 0xb28, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f,
0xb41, 0x20, 0xb15, 0xb3e, 0xb32, 0xb47, 0xb21, 0xb4b, 0xb28, 0xb3f, 0xb06, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28,
0xb4d, 0xb5f, 0xb41, 0x20, 0xb15, 0xb3e, 0xb32, 0xb47, 0xb21, 0xb4b, 0xb28, 0xb3f,
0xb06, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c, 0xb3f, 0xb32, 0xb3e,
0xb23, 0xb4d, 0xb21, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c,
0xb3f, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c, 0xb3f,
0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2b, 0xb3e, 0xb09,
0xb23, 0xb4d, 0xb21, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2b, 0xb3e, 0xb09, 0xb23, 0xb4d,
0xb21, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f,
0xb41, 0xb2b, 0xb3e, 0xb09, 0xb23, 0xb4d, 0xb21, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23,
0xb4d, 0xb21, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb28, 0xb3f, 0xb5f, 0xb41, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28,
0xb30, 0xb2b, 0xb4b, 0xb15, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb30, 0xb2b, 0xb4b, 0xb15, 0xb4d, 0x200c, 0x20,
0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb30, 0xb2b, 0xb4b, 0xb15, 0xb4d, 0x200c,
0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb30, 0xb4d,
0xb23, 0xb4d, 0xb23, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4b, 0x20, 0xb21, 0xb3f, 0x20,
0xb28, 0xb4b, 0xb30, 0xb4b, 0xb28, 0xb4d, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2b, 0xb30, 0xb4d, 0xb23, 0xb4d, 0xb23, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4b, 0x20,
0xb21, 0xb3f, 0x20, 0xb28, 0xb4b, 0xb30, 0xb4b, 0xb28, 0xb4d, 0xb28, 0xb3e, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b,
0xb30, 0xb4d, 0xb23, 0xb4d, 0xb23, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4b, 0x20, 0xb21,
0xb3f, 0x20, 0xb28, 0xb4b, 0xb30, 0xb4b, 0xb28, 0xb4d, 0xb28, 0xb3e, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb09, 0xb24, 0xb4d, 0xb24, 0xb30, 0x20, 0xb2e, 0xb47, 0xb30, 0xb3f, 0xb06,
0xb28, 0xb3e, 0x20, 0xb06, 0xb07, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb38, 0xb3f, 0xb2c, 0xb3f,
0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4b, 0xb2d, 0xb4b,
0xb38, 0xb3f, 0xb2c, 0xb3f, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4b, 0xb2d, 0xb4b,
0xb38, 0xb3f, 0xb2c, 0xb3f, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb13,
0xb2e, 0xb38, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb13, 0xb2e, 0xb38, 0xb4d,
0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb13, 0xb2e, 0xb38, 0xb4d, 0xb15, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb15,
0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e,
0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e,
0xb28, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb32, 0xb3e, 0xb09, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb2a, 0xb41, 0xb06, 0x20, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb17,
0xb3f, 0xb28, 0xb3f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb3e, 0xb17,
0xb41, 0xb0f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb3e, 0xb17, 0xb41,
0xb0f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb3e, 0xb17, 0xb41, 0xb0f, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb47, 0xb30, 0xb41, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb47, 0xb30, 0xb41, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb47, 0xb30, 0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15,
0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3e,
0xb07, 0xb28, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb32, 0xb3f,
0xb2a, 0xb3e, 0xb07, 0xb28, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3e,
0xb07, 0xb28, 0xb4d, 0x200c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb28, 0xb3f, 0xb15,
0xb4d, 0xb38, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d,
0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0x2e, 0x20,
0xb2a, 0xb3f, 0xb0f, 0xb30, 0xb47, 0x20, 0xb0f, 0xb2c, 0xb02, 0x20, 0xb2e, 0xb3f,
0xb15, 0xb4d, 0xb5f, 0xb41, 0xb32, 0xb4b, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0x2e, 0x20, 0xb2a, 0xb3f, 0xb0f, 0xb30, 0xb47,
0x20, 0xb0f, 0xb2c, 0xb02, 0x20, 0xb2e, 0xb3f, 0xb15, 0xb4d, 0xb5f, 0xb41, 0xb32,
0xb4b, 0xb28, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0x2e, 0x20, 0xb2a, 0xb3f,
0xb0f, 0xb30, 0xb47, 0x20, 0xb0f, 0xb2c, 0xb02, 0x20, 0xb2e, 0xb3f, 0xb15, 0xb4d,
0xb5f, 0xb41, 0xb32, 0xb4b, 0xb28, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32,
0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3f, 0xb1f, 0xb15, 0xb47, 0xb30,
0xb4d, 0xb28, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb4b, 0xb28, 0xb3e,
0xb2a, 0xb47, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb4b, 0xb5f, 0xb19, 0xb4d, 0xb17,
0xb5f, 0xb3e, 0xb19, 0xb4d, 0xb17, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb3e, 0xb1c,
0xb3f, 0xb32, 0xb4b, 0xb30, 0xb4d, 0xb21, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15,
0xb3e, 0xb1c, 0xb3f, 0xb32, 0xb4b, 0xb30, 0xb4d, 0xb21, 0xb3e, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb3e, 0xb1c, 0xb3f, 0xb32, 0xb4b,
0xb30, 0xb4d, 0xb21, 0xb3e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb30, 0xb3f, 0xb5f, 0xb41, 0xb28, 0xb3f, 0xb05, 0xb28,
0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb30, 0xb4b, 0xb25, 0xb47, 0xb30, 0xb3e,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb16, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb38, 0xb16, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb16,
0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb2e, 0xb3e, 0xb30,
0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb2e, 0xb3e, 0xb30, 0xb3e, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb2e, 0xb3e, 0xb30, 0xb3e,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb38, 0xb3e, 0xb2e, 0xb4b, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb3e, 0xb2e,
0xb4b, 0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38,
0xb3e, 0xb2e, 0xb4b, 0xb06, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb47, 0xb1a, 0xb47, 0xb32, 0xb4d, 0xb32, 0xb47,
0xb38, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb3f, 0xb19, 0xb4d, 0xb17,
0xb3e, 0xb2a, 0xb41, 0xb30, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb38, 0xb4b, 0xb32, 0xb4b, 0xb2e, 0xb28, 0x20, 0xb26, 0xb4d,
0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb26, 0xb15, 0xb4d, 0xb37, 0xb3f, 0xb23, 0x20, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f,
0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb41, 0xb30, 0xb3f, 0xb28, 0xb47, 0xb2e,
0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb4b, 0xb71, 0xb3e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb24, 0xb3e, 0xb39, 0xb3f, 0xb24, 0xb3f, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47, 0xb07, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb3e,
0xb07, 0xb2a, 0xb47, 0xb07, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47, 0xb07, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e,
0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb3e, 0xb1c, 0xb3f, 0xb15,
0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1f, 0xb4b,
0xb15, 0xb47, 0xb32, 0xb3e, 0xb09, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1f, 0xb4b, 0xb19,
0xb4d, 0xb17, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1f, 0xb4b, 0xb19, 0xb4d, 0xb17,
0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1f, 0xb4b,
0xb19, 0xb4d, 0xb17, 0xb3e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb42, 0xb15,
0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb41, 0xb30, 0xb4d, 0xb15, 0xb2e,
0xb47, 0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb24, 0xb41, 0xb30, 0xb4d, 0xb15, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24,
0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb24, 0xb41, 0xb30, 0xb4d, 0xb15, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb38,
0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb24, 0xb41, 0xb2d, 0xb3e, 0xb32, 0xb41, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb09, 0xb30, 0xb41, 0xb17, 0xb41, 0xb0f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09,
0xb30, 0xb41, 0xb17, 0xb41, 0xb0f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb30, 0xb41, 0xb17, 0xb41, 0xb0f, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb09, 0xb1c, 0xb2c, 0xb47, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e,
0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb1c, 0xb2c, 0xb47, 0xb15, 0xb3f, 0xb38,
0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb09, 0xb1c, 0xb2c, 0xb47, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40,
0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb28, 0xb41, 0xb06, 0xb1f, 0xb41,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb28, 0xb41, 0xb06, 0xb1f, 0xb41, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb28, 0xb41,
0xb06, 0xb1f, 0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15,
0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb47, 0xb28, 0xb3f,
0xb1c, 0xb41, 0xb0f, 0xb32, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4d, 0xb32,
0xb3e, 0xb21, 0xb3f, 0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4d, 0xb32, 0xb3e, 0xb21, 0xb3f, 0xb2d, 0xb4b, 0xb37,
0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4d, 0xb32, 0xb3e, 0xb21, 0xb3f, 0xb2d,
0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4b,
0xb32, 0xb17, 0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2d, 0xb4b, 0xb32, 0xb17, 0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2d, 0xb4b, 0xb32, 0xb17, 0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x200c, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb71, 0xb47, 0xb15, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71,
0xb40, 0xb2a, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb71, 0xb3e, 0xb32, 0xb3f, 0xb38, 0xb4d,
0x200c, 0x20, 0xb0f, 0xb2c, 0xb02, 0x20, 0xb2b, 0xb41, 0xb1f, 0xb41, 0xb28, 0xb3e,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb3e, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb3e, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb5f, 0xb3e, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f,
0xb47, 0xb15, 0xb3e, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17,
0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb1f, 0xb47, 0xb30,
0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb1f,
0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb4d, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb5f, 0xb41, 0xb15, 0xb4b, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x66, 0x67, 0x61, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x20,
0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73,
0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x42, 0x61, 0x68, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x20, 0x4b, 0x69, 0x62, 0x62, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x20,
0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x41, 0x6c, 0x61, 0x61, 0x73, 0x6b, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x6c,
0x61, 0x61, 0x73, 0x6b, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x41, 0x6c,
0x61, 0x61, 0x73, 0x6b, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6f, 0x6e, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6f, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6f, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47,
0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69,
0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65,
0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64,
0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x61, 0x68, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4d,
0x61, 0x61, 0x77, 0x6f, 0x6e, 0x74, 0x65, 0x65, 0x6e, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4d, 0x61, 0x61, 0x77,
0x6f, 0x6e, 0x74, 0x65, 0x65, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x4d, 0x61,
0x61, 0x77, 0x6f, 0x6e, 0x74, 0x65, 0x65, 0x6e, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x50, 0x61, 0x61, 0x73, 0x66, 0x69, 0x69,
0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x50, 0x61, 0x61, 0x73, 0x66, 0x69, 0x69, 0x6b, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20,
0x50, 0x61, 0x61, 0x73, 0x66, 0x69, 0x69, 0x6b, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79,
0x79, 0x61, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x72,
0x61, 0x62, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41,
0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x72, 0x6a,
0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41,
0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e,
0x74, 0x69, 0x69, 0x6e, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x61, 0x20,
0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41, 0x72, 0x6a,
0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69,
0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x41, 0x72, 0x6d, 0x61, 0x61, 0x6e, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x72, 0x6d,
0x61, 0x61, 0x6e, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41, 0x72,
0x6d, 0x61, 0x61, 0x6e, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x74, 0x69, 0x6c, 0x61, 0x61, 0x6e,
0x74, 0x69, 0x69, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x69, 0x69, 0x20, 0x41, 0x74, 0x69, 0x6c, 0x61, 0x61, 0x6e, 0x74, 0x69,
0x69, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47,
0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x41, 0x74, 0x69, 0x6c, 0x61, 0x61,
0x6e, 0x74, 0x69, 0x69, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67,
0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74,
0x69, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x47, 0x69,
0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75,
0x79, 0x79, 0x61, 0x61, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64,
0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x44, 0x68, 0x69, 0x68,
0x61, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67,
0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61,
0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61,
0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x44,
0x68, 0x69, 0x68, 0x61, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64,
0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x73,
0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x42,
0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79,
0x61, 0x61, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x73,
0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x20, 0x44,
0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69,
0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x44, 0x68,
0x69, 0x68, 0x61, 0x20, 0x41, 0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x47, 0x75, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x41,
0x77, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x61,
0x6a, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x61,
0x6a, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x61, 0x6a, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x7a, 0x65, 0x65, 0x72,
0x6f, 0x6f, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x41, 0x7a, 0x65, 0x65, 0x72, 0x6f, 0x6f, 0x73, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61,
0x20, 0x41, 0x7a, 0x65, 0x65, 0x72, 0x6f, 0x6f, 0x73, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x61, 0x61, 0x6e, 0x67, 0x69,
0x6c, 0x61, 0x61, 0x64, 0x69, 0x73, 0x68, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x42, 0x61, 0x61, 0x6e, 0x67, 0x69,
0x6c, 0x61, 0x61, 0x64, 0x69, 0x73, 0x68, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x42, 0x61,
0x61, 0x6e, 0x67, 0x69, 0x6c, 0x61, 0x61, 0x64, 0x69, 0x73, 0x68, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x69, 0x68, 0x75,
0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x42, 0x6f, 0x6c, 0x69, 0x69, 0x76, 0x69, 0x79, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x69, 0x72, 0x61,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x42, 0x69, 0x72, 0x61, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x42, 0x69,
0x72, 0x61, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x72, 0x75, 0x75, 0x6e,
0x65, 0x65, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x61, 0x61,
0x6d, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b, 0x65,
0x65, 0x70, 0x20, 0x56, 0x65, 0x65, 0x72, 0x64, 0x65, 0x65, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4b, 0x65, 0x65,
0x70, 0x20, 0x56, 0x65, 0x65, 0x72, 0x64, 0x65, 0x65, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20,
0x4b, 0x65, 0x65, 0x70, 0x20, 0x56, 0x65, 0x65, 0x72, 0x64, 0x65, 0x65,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4b,
0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x6f, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x43, 0x68, 0x69, 0x69, 0x6c, 0x69, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x43, 0x68, 0x69, 0x69, 0x6c, 0x69,
0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x43, 0x68, 0x69, 0x69, 0x6c, 0x69, 0x69, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x43, 0x68, 0x61, 0x61,
0x79, 0x69, 0x6e, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x43, 0x68, 0x61, 0x61, 0x79, 0x69, 0x6e, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75,
0x79, 0x79, 0x61, 0x61, 0x20, 0x43, 0x68, 0x61, 0x61, 0x79, 0x69, 0x6e,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f,
0x64, 0x6f, 0x6c, 0x61, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x73, 0x6d, 0x61,
0x61, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f,
0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20,
0x4b, 0x75, 0x75, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20,
0x4b, 0x75, 0x75, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x42, 0x6f, 0x6e, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x61, 0x6b, 0x6b,
0x61, 0x61, 0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20,
0x4b, 0x75, 0x75, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79,
0x79, 0x61, 0x61, 0x20, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x44, 0x61, 0x61, 0x76, 0x69,
0x69, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x54, 0x69, 0x69, 0x6d, 0x6f, 0x6f, 0x72, 0x20, 0x42, 0x61, 0x68, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64,
0x6f, 0x6c, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c,
0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4f, 0x64,
0x6f, 0x6c, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x6b, 0x6b, 0x75, 0x77, 0x61,
0x61, 0x64, 0x6f, 0x6f, 0x72, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70, 0x61, 0x61,
0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65, 0x65,
0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70, 0x61,
0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c, 0x65,
0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x41, 0x77, 0x75, 0x72,
0x6f, 0x6f, 0x70, 0x70, 0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75,
0x75, 0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f,
0x70, 0x70, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75,
0x72, 0x6f, 0x6f, 0x70, 0x70, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70,
0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70,
0x70, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x20, 0x44, 0x61,
0x62, 0x61, 0x6c, 0x61, 0x74, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70,
0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x41, 0x77, 0x75, 0x72,
0x6f, 0x6f, 0x70, 0x70, 0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x41, 0x77, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x70,
0x61, 0x61, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f,
0x74, 0x61, 0x20, 0x46, 0x61, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x61, 0x6e,
0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x46, 0x61, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4f, 0x64,
0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x46, 0x61, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x46, 0x69, 0x69, 0x6a, 0x69, 0x69, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x46, 0x69, 0x69, 0x6a, 0x69,
0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x46, 0x69, 0x69, 0x6a, 0x69, 0x69, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x46, 0x69, 0x72, 0x65, 0x65,
0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x79, 0x69, 0x6e, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x46, 0x69, 0x72, 0x61,
0x61, 0x6e, 0x73, 0x20, 0x4b, 0x69, 0x62, 0x62, 0x61, 0x61, 0x20, 0x66,
0x69, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47,
0x61, 0x6c, 0x61, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x6f, 0x73, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x61, 0x61, 0x6d, 0x62,
0x69, 0x79, 0x65, 0x65, 0x72, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4a, 0x6f,
0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4a, 0x6f,
0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61,
0x20, 0x47, 0x69, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e,
0x77, 0x69, 0x69, 0x63, 0x68, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75,
0x67, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e,
0x6c, 0x61, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x47, 0x69,
0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x61,
0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e,
0x6c, 0x61, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x69, 0x72, 0x69,
0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x20, 0x44, 0x68, 0x69, 0x68,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69,
0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x64,
0x20, 0x44, 0x68, 0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x47, 0x69,
0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x20, 0x44, 0x68,
0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x69, 0x69, 0x20, 0x47, 0x75, 0x77, 0x61, 0x61, 0x6d, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x47, 0x61, 0x61, 0x6c,
0x66, 0x69, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x47, 0x75, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x48, 0x61, 0x61, 0x77, 0x61, 0x79, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x77, 0x75, 0x74, 0x69, 0x79, 0x61, 0x6e, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x48, 0x61,
0x61, 0x77, 0x61, 0x79, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x77, 0x75, 0x74,
0x69, 0x79, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x48, 0x61, 0x61, 0x77,
0x61, 0x79, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x77, 0x75, 0x74, 0x69, 0x79,
0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x48,
0x6f, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x48, 0x6f, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x48,
0x6f, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x48, 0x6f, 0x6f, 0x76, 0x64,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x48,
0x6f, 0x6f, 0x76, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x48, 0x6f, 0x6f, 0x76, 0x64,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x48,
0x69, 0x6e, 0x64, 0x69, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x61, 0x6e, 0x61, 0x20, 0x48, 0x69,
0x6e, 0x64, 0x69, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6f, 0x43, 0x68, 0x61, 0x61, 0x79, 0x69,
0x6e, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x68, 0x69, 0x79,
0x61, 0x61, 0x20, 0x47, 0x69, 0x64, 0x64, 0x75, 0x75, 0x67, 0x61, 0x6c,
0x65, 0x65, 0x73, 0x73, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6f, 0x6e, 0x65, 0x73, 0x68,
0x69, 0x79, 0x61, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6f,
0x6e, 0x65, 0x65, 0x73, 0x68, 0x69, 0x79, 0x61, 0x61, 0x20, 0x44, 0x68,
0x69, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x69, 0x69, 0x20, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61,
0x61, 0x20, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x73, 0x72, 0x61, 0x61, 0x2019, 0x65, 0x65, 0x6c, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61,
0x61, 0x2019, 0x65, 0x65, 0x6c, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x49, 0x73, 0x72,
0x61, 0x61, 0x2019, 0x65, 0x65, 0x6c, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x4a, 0x61, 0x61, 0x70, 0x70, 0x61, 0x61, 0x6e, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4a, 0x61,
0x61, 0x70, 0x70, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x4a, 0x61,
0x61, 0x70, 0x70, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x4b, 0x61, 0x61, 0x7a, 0x61, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x4b, 0x61, 0x61, 0x7a, 0x61, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x68, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x4b, 0x61, 0x61, 0x7a, 0x61, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x44, 0x68, 0x69, 0x68, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b, 0x6f,
0x6f, 0x72, 0x69, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x69, 0x69, 0x20, 0x4b, 0x6f, 0x6f, 0x72, 0x69, 0x79, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75,
0x79, 0x79, 0x61, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x72, 0x69, 0x79, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b, 0x6f,
0x6f, 0x73, 0x72, 0x65, 0x79, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x4b, 0x69, 0x79, 0x69, 0x72, 0x69, 0x67, 0x69, 0x79, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4c,
0x61, 0x61, 0x6e, 0x6b, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20,
0x4c, 0x69, 0x6e, 0x65, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x4d, 0x61, 0x61, 0x6c, 0x65, 0x65, 0x73, 0x68, 0x69, 0x79, 0x61, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4d, 0x61, 0x61,
0x6c, 0x64, 0x69, 0x69, 0x76, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f,
0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x61, 0x72, 0x73, 0x68,
0x61, 0x61, 0x6c, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x4d, 0x6f, 0x6f, 0x72, 0x69, 0x69, 0x73, 0x68, 0x69, 0x79, 0x65, 0x65,
0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x4d, 0x6f, 0x6f, 0x72, 0x69, 0x73, 0x68, 0x69, 0x79, 0x65, 0x65, 0x73,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e,
0x61, 0x61, 0x20, 0x4d, 0x6f, 0x6f, 0x72, 0x69, 0x73, 0x68, 0x69, 0x79,
0x65, 0x65, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x50, 0x61, 0x61, 0x73, 0x66, 0x69, 0x69, 0x6b, 0x20,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x50, 0x61, 0x61,
0x73, 0x66, 0x69, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x69,
0x6b, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x50, 0x61, 0x61, 0x73,
0x66, 0x69, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20,
0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4d, 0x6f, 0x6f, 0x73,
0x6b, 0x6f, 0x6f, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x4d, 0x6f, 0x6f, 0x73, 0x6b, 0x6f, 0x6f, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20,
0x4d, 0x6f, 0x6f, 0x73, 0x6b, 0x6f, 0x6f, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x4d, 0x61, 0x61, 0x79, 0x69, 0x6e, 0x61, 0x61,
0x6d, 0x61, 0x61, 0x72, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x4e, 0x61, 0x61, 0x77, 0x75, 0x72, 0x75, 0x75, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4e, 0x65, 0x65, 0x70, 0x70, 0x61,
0x61, 0x6c, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4b,
0x61, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x61,
0x20, 0x48, 0x61, 0x61, 0x72, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4b, 0x61, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x61, 0x20, 0x48, 0x61, 0x61, 0x72,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42,
0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4b, 0x61, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6f, 0x6e, 0x69, 0x79, 0x61, 0x61, 0x20, 0x48, 0x61, 0x61, 0x72, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4e, 0x65, 0x77, 0x75, 0x66,
0x61, 0x61, 0x77, 0x6f, 0x6e, 0x64, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4e, 0x65,
0x77, 0x75, 0x66, 0x61, 0x61, 0x77, 0x6f, 0x6e, 0x64, 0x6c, 0x61, 0x61,
0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47,
0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x75, 0x66, 0x61,
0x61, 0x77, 0x6f, 0x6e, 0x64, 0x6c, 0x61, 0x61, 0x6e, 0x64, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79,
0x61, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64,
0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x61, 0x72, 0x69,
0x79, 0x61, 0x61, 0x6e, 0x61, 0x61, 0x20, 0x4b, 0x61, 0x61, 0x62, 0x61,
0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x6b, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x6b, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x6b,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x50, 0x61, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x69, 0x69, 0x20, 0x50, 0x61, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x50, 0x61, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x77, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x50, 0x61, 0x61, 0x70, 0x75, 0x77, 0x61, 0x61,
0x20, 0x47, 0x69, 0x69, 0x6e, 0x69, 0x69, 0x20, 0x48, 0x61, 0x61, 0x72,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x50,
0x61, 0x61, 0x72, 0x61, 0x61, 0x67, 0x75, 0x77, 0x61, 0x61, 0x79, 0x69,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x50,
0x61, 0x61, 0x72, 0x61, 0x61, 0x67, 0x75, 0x77, 0x61, 0x61, 0x79, 0x69,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e,
0x61, 0x61, 0x20, 0x50, 0x61, 0x61, 0x72, 0x61, 0x61, 0x67, 0x75, 0x77,
0x61, 0x61, 0x79, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x50, 0x65, 0x65, 0x72, 0x75, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x50, 0x65, 0x65, 0x72, 0x75, 0x75,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e,
0x61, 0x61, 0x20, 0x50, 0x65, 0x65, 0x72, 0x75, 0x75, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x69, 0x6e, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x69, 0x69, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e,
0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69,
0x6e, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f,
0x64, 0x6f, 0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x46, 0x6f, 0x6f, 0x6e,
0x65, 0x65, 0x6b, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x51, 0x6c, 0x2e, 0x20, 0x50, 0x69, 0x79, 0x65, 0x65, 0x72, 0x65,
0x65, 0x20, 0x66, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x75, 0x79, 0x65, 0x6c,
0x6f, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20,
0x51, 0x6c, 0x2e, 0x20, 0x50, 0x69, 0x79, 0x65, 0x65, 0x72, 0x65, 0x65,
0x20, 0x66, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x75, 0x79, 0x65, 0x6c, 0x6f,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79,
0x79, 0x61, 0x61, 0x20, 0x51, 0x6c, 0x2e, 0x20, 0x50, 0x69, 0x79, 0x65,
0x65, 0x72, 0x65, 0x65, 0x20, 0x66, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x75,
0x79, 0x65, 0x6c, 0x6f, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x50, 0x69, 0x79,
0x6f, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x61, 0x6e, 0x67, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x52, 0x69, 0x69, 0x79, 0x75, 0x75,
0x6e, 0x69, 0x79, 0x65, 0x65, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61,
0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x53, 0x61, 0x61, 0x6d, 0x6f, 0x77,
0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69,
0x20, 0x53, 0x61, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47, 0x75, 0x79, 0x79, 0x61, 0x61,
0x20, 0x53, 0x61, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x53, 0x69, 0x69, 0x73, 0x68, 0x65,
0x65, 0x6c, 0x73, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x61, 0x70, 0x6f, 0x6f, 0x72,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4f, 0x64, 0x6f,
0x6c, 0x6f, 0x6f, 0x74, 0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f,
0x6f, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x4a,
0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x61, 0x20, 0x4b, 0x69, 0x62,
0x62, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x61, 0x6d, 0x65, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x54,
0x61, 0x79, 0x69, 0x70, 0x65, 0x79, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x54, 0x61, 0x79, 0x69, 0x70, 0x65,
0x79, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x47,
0x75, 0x79, 0x79, 0x61, 0x61, 0x20, 0x54, 0x61, 0x79, 0x69, 0x70, 0x65,
0x79, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x54,
0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x54, 0x61, 0x6b, 0x65, 0x6c,
0x61, 0x77, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x65, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x69, 0x69, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x65, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x65, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x59, 0x75, 0x72, 0x61, 0x61, 0x67, 0x61, 0x61, 0x79, 0x69, 0x53, 0x61,
0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x59, 0x75, 0x72,
0x61, 0x61, 0x67, 0x61, 0x61, 0x79, 0x69, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x59, 0x75,
0x72, 0x61, 0x61, 0x67, 0x61, 0x61, 0x79, 0x69, 0x53, 0x61, 0x2019, 0x61,
0x61, 0x74, 0x69, 0x69, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x77, 0x61, 0x74, 0x75,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x77, 0x61, 0x74, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x77, 0x61, 0x74, 0x75, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74,
0x69, 0x69, 0x20, 0x56, 0x65, 0x65, 0x6e, 0x7a, 0x75, 0x77, 0x65, 0x65,
0x6c, 0x61, 0x61, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x69, 0x69, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f,
0x6e, 0x61, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x4f, 0x64, 0x6f, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x53,
0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x57, 0x61, 0x6c, 0x6c,
0x69, 0x73, 0x20, 0x66, 0x69, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69, 0x69,
0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x69,
0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x2019, 0x61, 0x61,
0x74, 0x69, 0x69, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x49, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x69, 0x69, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x53, 0x61, 0x2019, 0x61, 0x61, 0x74, 0x69,
0x69, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53, 0x61, 0x2019,
0x61, 0x61, 0x74, 0x69, 0x69, 0x20, 0x59, 0x75, 0x75, 0x6b, 0x6f, 0x6f,
0x6e, 0x410, 0x441, 0x442, 0x4d5, 0x443, 0x43a, 0x43a, 0x430, 0x433, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x440, 0x4d5, 0x441,
0x442, 0x4d5, 0x433, 0x410, 0x441, 0x442, 0x4d5, 0x443, 0x43a, 0x43a, 0x430, 0x433,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43e, 0x43d, 0x20, 0x440, 0x4d5,
0x441, 0x442, 0x4d5, 0x433, 0x410, 0x441, 0x442, 0x4d5, 0x443, 0x43a, 0x43a, 0x430,
0x433, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20,
0x441, 0x4d5, 0x440, 0x434, 0x44b, 0x433, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441,
0x442, 0x4d5, 0x433, 0x421, 0x43a, 0x4d5, 0x441, 0x4d5, 0x43d, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x440, 0x4d5, 0x441, 0x442,
0x4d5, 0x433, 0x421, 0x43a, 0x4d5, 0x441, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433,
0x421, 0x43a, 0x4d5, 0x441, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x4d5, 0x439, 0x430, 0x433, 0x20, 0x441, 0x4d5, 0x440, 0x434, 0x44b, 0x433, 0x43e,
0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41d, 0x44b, 0x433, 0x44a,
0x443, 0x44b, 0x43b, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5,
0x439, 0x430, 0x433, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41d, 0x44b,
0x433, 0x44a, 0x443, 0x44b, 0x43b, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41d,
0x44b, 0x433, 0x44a, 0x443, 0x44b, 0x43b, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x441, 0x4d5, 0x440, 0x434, 0x44b,
0x433, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x413, 0x443,
0x44b, 0x440, 0x434, 0x437, 0x44b, 0x441, 0x442, 0x43e, 0x43d, 0x44b, 0x20, 0x440,
0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x413, 0x443, 0x44b, 0x440, 0x434, 0x437, 0x44b,
0x441, 0x442, 0x43e, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x413,
0x443, 0x44b, 0x440, 0x434, 0x437, 0x44b, 0x441, 0x442, 0x43e, 0x43d, 0x44b, 0x20,
0x441, 0x4d5, 0x440, 0x434, 0x44b, 0x433, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441,
0x442, 0x4d5, 0x433, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x44b, 0x20,
0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x43c, 0x431, 0x438, 0x441, 0x20, 0x440, 0x4d5,
0x441, 0x442, 0x4d5, 0x433, 0x41c, 0x4d5, 0x441, 0x43a, 0x443, 0x44b, 0x439, 0x44b,
0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41c, 0x4d5, 0x441, 0x43a, 0x443,
0x44b, 0x439, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41c, 0x4d5, 0x441,
0x43a, 0x443, 0x44b, 0x439, 0x44b, 0x20, 0x441, 0x4d5, 0x440, 0x434, 0x44b, 0x433,
0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x4f, 0x73, 0x74, 0x20, 0xc1, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x7a, 0xf3,
0x72, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x20, 0x64, 0x69, 0x20, 0x41, 0x7a, 0xf3, 0x72, 0x65, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x7a, 0xf3, 0x72, 0x65, 0x73,
0x20, 0x54, 0x65, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x64, 0x61, 0x20, 0x64,
0x69, 0x20, 0x5a, 0x6f, 0x6d, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x54, 0x65, 0x6d,
0x70, 0x6f, 0x72, 0x61, 0x64, 0x61, 0x20, 0x64, 0x69, 0x20, 0x5a, 0x6f,
0x6d, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4f, 0x73,
0x74, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x69, 0x20, 0x4f, 0x73,
0x74, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x4f, 0x73, 0x74, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x54, 0x65, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x64, 0x61, 0x20,
0x64, 0x69, 0x20, 0x5a, 0x6f, 0x6d, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x57, 0xe8, 0x73, 0x74, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x20, 0x64, 0x69, 0x20, 0x57, 0xe8, 0x73, 0x74, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x57, 0xe8,
0x73, 0x74, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65,
0x6d, 0x70, 0x6f, 0x72, 0x61, 0x64, 0x61, 0x20, 0x64, 0x69, 0x20, 0x5a,
0x6f, 0x6d, 0x65, 0x72, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x646, 0x681, 0x646, 0x6cc, 0x20, 0x627,
0x641, 0x631, 0x64a, 0x642, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a,
0x681, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x642, 0x627, 0x20, 0x648, 0x62e, 0x62a,
0x62c, 0x646, 0x648, 0x628, 0x64a, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x642, 0x627,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644,
0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x642, 0x627,
0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x627,
0x641, 0x631, 0x6cc, 0x642, 0x627, 0x6cc, 0x64a, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x641, 0x631, 0x6cc,
0x642, 0x627, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x20, 0x644, 0x648,
0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x644, 0x627, 0x633,
0x6a9, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0,
0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x6cc,
0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9,
0x632, 0x64a, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648,
0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x6cc, 0x681,
0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20,
0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648,
0x62e, 0x62a, 0x62f, 0x20, 0x63a, 0x631, 0x647, 0x20, 0x62f, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x63a, 0x631, 0x647, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x63a, 0x631, 0x647, 0x20, 0x62f,
0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e,
0x62a, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x62f, 0x20,
0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a,
0x627, 0x67e, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x67e, 0x6cc, 0x627,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x67e, 0x64a, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631,
0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x639, 0x631, 0x628,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x639, 0x631, 0x628, 0x64a, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x639, 0x631, 0x628, 0x64a,
0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627,
0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67c, 0x627,
0x6cc, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67c, 0x627,
0x6cc, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67c, 0x627, 0x6cc, 0x646, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681,
0x20, 0x627, 0x631, 0x62c, 0x646, 0x67c, 0x627, 0x64a, 0x646, 0x20, 0x648, 0x62e,
0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x62c, 0x646,
0x67c, 0x627, 0x64a, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x631,
0x62c, 0x646, 0x67c, 0x627, 0x64a, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x631, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x631, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631,
0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x62a,
0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627,
0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9,
0x632, 0x64a, 0x20, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20,
0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20,
0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a,
0x20, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681,
0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a,
0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x645, 0x631, 0x6a9,
0x632, 0x64a, 0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x648, 0x62e,
0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x645, 0x631,
0x6a9, 0x632, 0x64a, 0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20,
0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681,
0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a,
0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a,
0x627, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648,
0x64a, 0x20, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648,
0x64a, 0x20, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681,
0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a,
0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x622, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x64a, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x648, 0x64a, 0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681,
0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627,
0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x622, 0x630, 0x631, 0x628,
0x627, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x630, 0x631,
0x628, 0x627, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x630, 0x631, 0x628, 0x627, 0x6cc, 0x6cc,
0x62c, 0x627, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x627, 0x64a, 0x632, 0x648, 0x631, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a,
0x632, 0x648, 0x631, 0x633, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x64a, 0x632, 0x648, 0x631, 0x633, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x646, 0x6af, 0x644, 0x647, 0x20,
0x62f, 0x6d0, 0x634, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x646, 0x6ab, 0x644, 0x627,
0x62f, 0x6cc, 0x634, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x628, 0x646, 0x6ab, 0x644, 0x647, 0x20, 0x62f, 0x64a, 0x634, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x647, 0x648, 0x67c,
0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x648, 0x644, 0x6cc, 0x648, 0x6cc,
0x627, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x631, 0x633, 0x644, 0x6cc, 0x627, 0x20,
0x648, 0x62e, 0x62a, 0x628, 0x631, 0x633, 0x644, 0x6cc, 0x627, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x631, 0x633, 0x644,
0x6cc, 0x627, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x628,
0x631, 0x648, 0x646, 0x627, 0x64a, 0x64a, 0x20, 0x62f, 0x627, 0x631, 0x627, 0x644,
0x633, 0x644, 0x627, 0x645, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x67e, 0x20,
0x648, 0x631, 0x62f, 0x689, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x67e, 0x20,
0x648, 0x631, 0x62f, 0x689, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631, 0x62f, 0x689, 0x20,
0x633, 0x645, 0x631, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x645, 0x627, 0x631, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x686,
0x627, 0x62a, 0x627, 0x645, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x627, 0x62a, 0x627,
0x645, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x686, 0x627, 0x62a, 0x627, 0x645, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0,
0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x686,
0x644, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x644, 0x6cc, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x644, 0x6cc, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x64a, 0x646, 0x20,
0x648, 0x62e, 0x62a, 0x686, 0x6cc, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x686, 0x64a, 0x646, 0x20, 0x62f,
0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e,
0x62a, 0x6a9, 0x631, 0x64a, 0x633, 0x645, 0x633, 0x20, 0x67c, 0x627, 0x67e, 0x648,
0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x6a9, 0x648, 0x632, 0x20, 0x67c, 0x627,
0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648,
0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x644,
0x645, 0x628, 0x6cc, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x627,
0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x6a9, 0x20, 0x67c,
0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x6a9,
0x648, 0x6a9, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648,
0x6a9, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x646,
0x64a, 0x645, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9,
0x64a, 0x648, 0x628, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x648, 0x628,
0x627, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x6a9, 0x6cc, 0x648, 0x628, 0x627, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20,
0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x689, 0x64a, 0x648, 0x633,
0x20, 0x648, 0x62e, 0x62a, 0x689, 0x648, 0x645, 0x648, 0x646, 0x67c, 0x20, 0x689,
0x64a, 0x20, 0x627, 0x631, 0x648, 0x644, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x62a,
0x64a, 0x645, 0x648, 0x631, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a, 0x633, 0x62a,
0x631, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a,
0x633, 0x62a, 0x631, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a, 0x633, 0x62a, 0x631,
0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x6a9, 0x648, 0x627, 0x62f, 0x648, 0x631, 0x20,
0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x627, 0x631, 0x648,
0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x645, 0x631,
0x6a9, 0x632, 0x64a, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632,
0x64a, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x627, 0x648, 0x693,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x627, 0x631,
0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a,
0x681, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681,
0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x627, 0x648, 0x693,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x631, 0x6d0, 0x20, 0x62e, 0x62a, 0x64a,
0x681, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x648, 0x67e,
0x627, 0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a,
0x681, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0, 0x62f,
0x64a, 0x681, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x627,
0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x648, 0x6a9, 0x644, 0x646,
0x689, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x641, 0x648, 0x6a9, 0x644, 0x646, 0x689, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x641, 0x648, 0x6a9, 0x644, 0x646, 0x689, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x62c, 0x6cc, 0x20,
0x648, 0x62e, 0x62a, 0x641, 0x62c, 0x6cc, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x62c, 0x64a, 0x20, 0x62f, 0x20, 0x627,
0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x641, 0x631, 0x627,
0x646, 0x633, 0x648, 0x64a, 0x20, 0x6ab, 0x627, 0x646, 0x627, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x641, 0x631, 0x627, 0x646, 0x633, 0x6d0, 0x20, 0x633, 0x648,
0x6cc, 0x644, 0x20, 0x627, 0x648, 0x20, 0x627, 0x646, 0x67c, 0x627, 0x631, 0x67c,
0x64a, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x6ab, 0x627, 0x644, 0x67e, 0x6ab, 0x648,
0x633, 0x20, 0x648, 0x62e, 0x62a, 0x6ab, 0x64a, 0x645, 0x628, 0x64a, 0x631, 0x20,
0x648, 0x62e, 0x62a, 0x62c, 0x648, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x648, 0x62e,
0x62a, 0x62c, 0x648, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x62c, 0x648, 0x631, 0x62c,
0x64a, 0x627, 0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x62c, 0x644, 0x628, 0x631, 0x67c, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab,
0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x6ab, 0x631, 0x64a, 0x646, 0x648,
0x64a, 0x686, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x62e, 0x62a, 0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646,
0x644, 0x6cc, 0x646, 0x689, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x62e, 0x62a,
0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646, 0x689, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x62e, 0x62a, 0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646,
0x689, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648,
0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646,
0x689, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20,
0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646, 0x689, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc,
0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646, 0x689, 0x20, 0x627,
0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x644, 0x6cc, 0x62c, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x6ab, 0x648, 0x627, 0x646, 0x627, 0x646, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x647,
0x648, 0x627, 0x6cc, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648, 0x62a, 0x6cc, 0x646,
0x20, 0x648, 0x62e, 0x62a, 0x647, 0x648, 0x627, 0x6cc, 0x6cc, 0x20, 0x627, 0x644,
0x6cc, 0x648, 0x62a, 0x6cc, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x62e, 0x62a, 0x647, 0x648, 0x627, 0x6cc, 0x6cc, 0x20, 0x627, 0x644,
0x6cc, 0x648, 0x62a, 0x6cc, 0x646, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631,
0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x646, 0x6ab, 0x20, 0x6a9,
0x627, 0x646, 0x6ab, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x646, 0x6ab, 0x20,
0x6a9, 0x627, 0x646, 0x6ab, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x647, 0x627, 0x646, 0x6ab, 0x20, 0x6a9, 0x627, 0x646, 0x6ab,
0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x648,
0x689, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x648, 0x689, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x648, 0x689,
0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x646, 0x62f,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x647, 0x646, 0x62f, 0x20, 0x633, 0x645, 0x646, 0x62f, 0x631, 0x20, 0x648,
0x62e, 0x62a, 0x627, 0x646, 0x689, 0x648, 0x686, 0x627, 0x6cc, 0x646, 0x647, 0x20,
0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x627, 0x62f, 0x648,
0x646, 0x64a, 0x632, 0x64a, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x646, 0x62f,
0x648, 0x646, 0x6cc, 0x632, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648,
0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x627, 0x646, 0x62f, 0x648, 0x646, 0x6cc, 0x632,
0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a, 0x631, 0x627, 0x646,
0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627,
0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x6a9, 0x648, 0x67c, 0x633,
0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x631, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x6cc, 0x6cc, 0x644, 0x20, 0x648,
0x62e, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x6cc, 0x6cc, 0x644, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x633, 0x631, 0x627,
0x64a, 0x64a, 0x644, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631,
0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x627, 0x67e,
0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x627,
0x67e, 0x627, 0x646, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631,
0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x642, 0x632, 0x627,
0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a,
0x681, 0x20, 0x642, 0x627, 0x632, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648,
0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x642, 0x632, 0x627,
0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x631,
0x64a, 0x627, 0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x631, 0x64a,
0x627, 0x64a, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x6a9, 0x648, 0x631, 0x64a, 0x627, 0x64a, 0x64a, 0x20, 0x62f, 0x20,
0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20,
0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x6a9, 0x631, 0x64a, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x6a9,
0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x631, 0x64a, 0x633, 0x646, 0x648, 0x64a, 0x627,
0x631, 0x633, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x6a9, 0x631, 0x64a, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633,
0x6a9, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x631,
0x63a, 0x6cc, 0x632, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x644,
0x627, 0x64a, 0x646, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648,
0x20, 0x648, 0x62e, 0x62a, 0x644, 0x627, 0x631, 0x689, 0x20, 0x647, 0x648, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x644, 0x627, 0x631, 0x689, 0x20, 0x647, 0x648, 0x64a,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644,
0x627, 0x631, 0x689, 0x20, 0x647, 0x648, 0x64a, 0x20, 0x62f, 0x20, 0x648, 0x631,
0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e,
0x62a, 0x645, 0x64a, 0x6ab, 0x689, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x645,
0x64a, 0x6ab, 0x689, 0x627, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x645, 0x64a, 0x6ab, 0x689, 0x627, 0x646, 0x20, 0x627,
0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x644, 0x627, 0x626, 0x6cc,
0x634, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x644, 0x62f, 0x64a,
0x67e, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x631, 0x6a9, 0x633, 0x633, 0x20,
0x648, 0x62e, 0x62a, 0x645, 0x627, 0x631, 0x634, 0x644, 0x20, 0x67c, 0x627, 0x67e,
0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x631,
0x64a, 0x634, 0x64a, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x631, 0x6cc,
0x634, 0x6cc, 0x633, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x645, 0x627, 0x631, 0x64a, 0x634, 0x64a, 0x633, 0x20, 0x62f, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x648, 0x633,
0x646, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20,
0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x6a9, 0x633,
0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x6a9, 0x633, 0x6cc,
0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x631, 0x6bc, 0x627,
0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x648, 0x644,
0x627, 0x646, 0x20, 0x628, 0x627, 0x67c, 0x631, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x67c, 0x631, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x648, 0x644, 0x627,
0x646, 0x20, 0x628, 0x627, 0x67c, 0x631, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x648, 0x62e, 0x62a,
0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x633, 0x645,
0x631, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x6cc, 0x627, 0x646, 0x645, 0x627, 0x631,
0x20, 0x648, 0x62e, 0x62a, 0x646, 0x627, 0x648, 0x631, 0x648, 0x20, 0x648, 0x62e,
0x62a, 0x646, 0x6cc, 0x67e, 0x627, 0x644, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x6cc,
0x648, 0x20, 0x6a9, 0x627, 0x644, 0x6cc, 0x689, 0x648, 0x646, 0x6cc, 0x627, 0x20,
0x648, 0x62e, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x6a9, 0x627, 0x644, 0x6cc, 0x689,
0x648, 0x646, 0x6cc, 0x627, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x6a9, 0x627, 0x64a, 0x689, 0x648,
0x646, 0x64a, 0x627, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x646, 0x64a, 0x648, 0x632, 0x64a, 0x20, 0x644, 0x6d0, 0x646, 0x689, 0x20, 0x648,
0x62e, 0x62a, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x20, 0x644, 0x6d0, 0x646, 0x689,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x646,
0x64a, 0x648, 0x632, 0x64a, 0x20, 0x644, 0x6d0, 0x646, 0x689, 0x20, 0x62f, 0x20,
0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20,
0x648, 0x62e, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x641, 0x627, 0x648, 0x646, 0x689,
0x644, 0x6d0, 0x646, 0x689, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x646, 0x648,
0x64a, 0x20, 0x641, 0x6cc, 0x644, 0x689, 0x644, 0x6cc, 0x646, 0x689, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x64a, 0x648,
0x20, 0x641, 0x627, 0x648, 0x646, 0x689, 0x644, 0x6d0, 0x646, 0x689, 0x20, 0x62f,
0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647,
0x20, 0x648, 0x62e, 0x62a, 0x646, 0x6cc, 0x6cc, 0x648, 0x20, 0x648, 0x62e, 0x62a,
0x646, 0x648, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20,
0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x646, 0x648, 0x631, 0x641, 0x6a9, 0x627, 0x633,
0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x646, 0x648, 0x631, 0x641, 0x6a9, 0x627,
0x633, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x62f,
0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x648, 0x62e,
0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x62f, 0x64a, 0x20,
0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688,
0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6be, 0x627,
0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x648, 0x648,
0x633, 0x64a, 0x628, 0x631, 0x633, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x648,
0x648, 0x633, 0x64a, 0x628, 0x631, 0x633, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x648, 0x648, 0x633, 0x64a, 0x628,
0x631, 0x633, 0x6a9, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x627, 0x648, 0x645, 0x633, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x648, 0x645,
0x633, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648,
0x62e, 0x62a, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x627, 0x6a9, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x67e, 0x627, 0x644, 0x627, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x627, 0x67e,
0x648, 0x627, 0x20, 0x646, 0x6cc, 0x648, 0x20, 0x6ab, 0x646, 0x6cc, 0x20, 0x648,
0x62e, 0x62a, 0x67e, 0x64a, 0x631, 0x627, 0x6ab, 0x648, 0x627, 0x6cc, 0x20, 0x648,
0x62e, 0x62a, 0x67e, 0x6cc, 0x631, 0x627, 0x6ab, 0x648, 0x627, 0x6cc, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x627, 0x631,
0x627, 0x6ab, 0x648, 0x627, 0x6cc, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x6cc,
0x631, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x64a, 0x646, 0x20, 0x648, 0x62e, 0x62a,
0x641, 0x644, 0x67e, 0x627, 0x64a, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x64a, 0x646, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x641, 0x6cc,
0x646, 0x6a9, 0x633, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e, 0x62a,
0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x67e, 0x6cc, 0x6cc, 0x631, 0x627, 0x20, 0x648,
0x20, 0x645, 0x64a, 0x6a9, 0x644, 0x64a, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x633,
0x6cc, 0x646, 0x67c, 0x20, 0x67e, 0x6cc, 0x6cc, 0x631, 0x627, 0x20, 0x648, 0x20,
0x645, 0x64a, 0x6a9, 0x644, 0x64a, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x67e, 0x6cc,
0x6cc, 0x631, 0x627, 0x20, 0x648, 0x20, 0x645, 0x64a, 0x6a9, 0x644, 0x64a, 0x646,
0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e,
0x62a, 0x67e, 0x6cc, 0x67c, 0x20, 0x6a9, 0x627, 0x631, 0x6cc, 0x646, 0x20, 0x648,
0x62e, 0x62a, 0x67e, 0x648, 0x646, 0x64a, 0x67e, 0x20, 0x648, 0x62e, 0x62a, 0x67e,
0x64a, 0x627, 0x646, 0x6ab, 0x20, 0x64a, 0x627, 0x646, 0x6ab, 0x20, 0x648, 0x62e,
0x62a, 0x631, 0x64a, 0x20, 0x64a, 0x648, 0x646, 0x64a, 0x646, 0x20, 0x648, 0x62e,
0x62a, 0x631, 0x648, 0x62f, 0x631, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x62e,
0x644, 0x64a, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x62e, 0x644, 0x6cc, 0x646,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x633,
0x62e, 0x644, 0x64a, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x633, 0x645, 0x648, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x645, 0x648,
0x627, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x633, 0x645, 0x648, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20,
0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x6cc,
0x686, 0x6cc, 0x644, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x646, 0x6ab, 0x627,
0x67e, 0x648, 0x631, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x633, 0x644, 0x6cc, 0x645, 0x627, 0x646, 0x20, 0x67c, 0x627, 0x67e,
0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x633,
0x648, 0x6cc, 0x644, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x648,
0x62e, 0x62a, 0x633, 0x648, 0x631, 0x6cc, 0x646, 0x627, 0x645, 0x20, 0x648, 0x62e,
0x62a, 0x633, 0x6cc, 0x648, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x67c, 0x647, 0x64a,
0x67c, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627, 0x64a, 0x67e, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x62a, 0x627, 0x64a, 0x67e, 0x64a, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627, 0x64a, 0x67e, 0x64a,
0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627,
0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627, 0x62c, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x648, 0x6a9, 0x6cc, 0x644, 0x627,
0x648, 0x20, 0x648, 0x62e, 0x62a, 0x67c, 0x648, 0x646, 0x6ab, 0x627, 0x20, 0x648,
0x62e, 0x62a, 0x62f, 0x20, 0x67c, 0x648, 0x646, 0x6ab, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67c, 0x648, 0x646, 0x6ab, 0x627,
0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x648, 0x6a9,
0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x648, 0x648, 0x627,
0x644, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x6cc, 0x648, 0x631, 0x648, 0x6ab, 0x648,
0x627, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x6cc, 0x648, 0x631, 0x648, 0x6ab, 0x648,
0x627, 0x6cc, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x6cc, 0x648, 0x631, 0x648, 0x6ab, 0x648, 0x627, 0x6cc, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x648, 0x646, 0x627, 0x62a,
0x648, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x646,
0x648, 0x627, 0x62a, 0x648, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x648, 0x6cc, 0x646, 0x632, 0x648, 0x6cc, 0x644, 0x627, 0x20, 0x648, 0x62e,
0x62a, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x648, 0x633, 0x62a, 0x627, 0x6a9, 0x20,
0x648, 0x62e, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x64a, 0x648, 0x633, 0x62a, 0x627,
0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x648, 0x644, 0x627, 0x62f, 0x64a, 0x648, 0x633, 0x62a, 0x627, 0x6a9, 0x20, 0x627,
0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x627, 0x644, 0x6ab, 0x648,
0x6ab, 0x631, 0x627, 0x62f, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x627, 0x644, 0x6ab,
0x648, 0x6ab, 0x631, 0x627, 0x62f, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x648, 0x627, 0x644, 0x6ab, 0x648, 0x6ab, 0x631, 0x627,
0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x627,
0x633, 0x62a, 0x648, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x64a, 0x6a9, 0x20,
0x62a, 0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x627, 0x644, 0x633,
0x20, 0x627, 0x648, 0x20, 0x641, 0x648, 0x62a, 0x648, 0x646, 0x627, 0x20, 0x648,
0x62e, 0x62a, 0x64a, 0x627, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x20, 0x648, 0x62e,
0x62a, 0x64a, 0x627, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x64a, 0x627, 0x6a9, 0x648, 0x67c,
0x633, 0x6a9, 0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x6cc, 0x6a9, 0x627, 0x67c, 0x6cc, 0x631, 0x6cc, 0x646, 0x628, 0x631, 0x6ab,
0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x64a, 0x627, 0x6a9, 0x64a, 0x67c, 0x631,
0x646, 0x628, 0x631, 0x6ab, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x64a, 0x627, 0x6a9, 0x67c, 0x631, 0x646, 0x628,
0x631, 0x6ab, 0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x6cc, 0x648, 0x6a9, 0x648, 0x646, 0x20, 0x648, 0x62e, 0x62a,
0x62f, 0x20, 0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x62f, 0x20, 0x648,
0x631, 0x681, 0x6d2, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x6a9,
0x648, 0x644, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x631, 0x6bc, 0x627, 0x20,
0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681,
0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20,
0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x63a, 0x631, 0x647, 0x20, 0x62f, 0x20, 0x631,
0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x67e,
0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648,
0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x67e, 0x6cc,
0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a,
0x62f, 0x20, 0x639, 0x631, 0x628, 0x64a, 0x20, 0x648, 0x631, 0x681, 0x67e, 0x627,
0x6bc, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a,
0x6cc, 0x6a9, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681,
0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x633, 0x67c, 0x631, 0x627,
0x644, 0x6cc, 0x627, 0x20, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x62f,
0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x686,
0x64a, 0x646, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681,
0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x648, 0x628, 0x627, 0x20, 0x62f,
0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e,
0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x632, 0x6d2, 0x20, 0x627, 0x631, 0x648,
0x67e, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x644, 0x648, 0x62f, 0x6cc,
0x681, 0x6d2, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x644, 0x648, 0x62f,
0x6cc, 0x681, 0x6d2, 0x20, 0x627, 0x648, 0x631, 0x67e, 0x627, 0x20, 0x62f, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x641, 0x631,
0x627, 0x646, 0x633, 0x6d2, 0x20, 0x633, 0x648, 0x6cc, 0x644, 0x20, 0x627, 0x648,
0x20, 0x627, 0x646, 0x67c, 0x627, 0x631, 0x67c, 0x64a, 0x6a9, 0x20, 0x648, 0x62e,
0x62a, 0x647, 0x648, 0x627, 0x6cc, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648, 0x62a,
0x6cc, 0x646, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20,
0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x62f,
0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627,
0x633, 0x631, 0x627, 0x6cc, 0x6cc, 0x644, 0x648, 0x20, 0x62f, 0x20, 0x648, 0x631,
0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20,
0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648,
0x62e, 0x62a, 0x62f, 0x20, 0x6a9, 0x648, 0x631, 0x6cc, 0x627, 0x20, 0x62f, 0x20,
0x648, 0x631, 0x681, 0x6d2, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20,
0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x6a9, 0x631, 0x69a, 0x6d2, 0x20, 0x67c, 0x627,
0x67c, 0x648, 0x628, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x631, 0x628, 0x20, 0x647,
0x627, 0x648, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x62f, 0x20,
0x631, 0x6bc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x6a9, 0x633, 0x6cc, 0x6a9,
0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x631, 0x6bc, 0x627, 0x20,
0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x646, 0x6cc,
0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x62f, 0x20, 0x648,
0x631, 0x681, 0x6d2, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x646, 0x648, 0x64a, 0x20, 0x641, 0x6cc, 0x644, 0x689, 0x644,
0x6cc, 0x646, 0x689, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2,
0x20, 0x648, 0x62e, 0x62a, 0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x67e, 0x6cc, 0x6cc,
0x631, 0x627, 0x20, 0x648, 0x20, 0x645, 0x64a, 0x6a9, 0x644, 0x64a, 0x646, 0x20,
0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a,
0x62f, 0x20, 0x633, 0x645, 0x648, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681,
0x6d2, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x6a9, 0x648, 0x644,
0x648, 0x642, 0x62a, 0x20, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x622, 0x641,
0x631, 0x6cc, 0x642, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20,
0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x20, 0x627, 0x641, 0x631, 0x6cc,
0x642, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x641,
0x631, 0x6cc, 0x642, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x622, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x622, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x644,
0x645, 0x627, 0x622, 0x62a, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x622, 0x644, 0x645, 0x627, 0x622, 0x62a, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x644,
0x645, 0x627, 0x622, 0x62a, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x645, 0x627,
0x632, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x622, 0x645, 0x627, 0x632, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x645, 0x627, 0x632, 0x648,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x645,
0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x645, 0x631, 0x6cc,
0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x634,
0x631, 0x642, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634, 0x631,
0x642, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x6a9, 0x648, 0x647, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x645, 0x631,
0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x6a9, 0x648, 0x647, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x645, 0x631,
0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x648, 0x647, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x63a,
0x631, 0x628, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x645,
0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x645, 0x631,
0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x646, 0x627, 0x62f, 0x6cc,
0x631, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x646,
0x627, 0x62f, 0x6cc, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x646, 0x627, 0x62f, 0x6cc, 0x631, 0x648,
0x642, 0x62a, 0x20, 0x622, 0x67e, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x67e, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x67e, 0x6cc,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x631, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x639, 0x631, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x639, 0x631, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x622, 0x631, 0x698, 0x627, 0x646, 0x62a, 0x6cc, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x631, 0x698,
0x627, 0x646, 0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x631, 0x698, 0x627, 0x646, 0x62a,
0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x622, 0x631,
0x698, 0x627, 0x646, 0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x622, 0x631, 0x698, 0x627, 0x646,
0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x622, 0x631, 0x698, 0x627,
0x646, 0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x631, 0x645, 0x646,
0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x627, 0x631, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x631,
0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x62a,
0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x622, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x622, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20,
0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x631,
0x6a9, 0x632, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x6cc,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x631, 0x6a9,
0x632, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20,
0x63a, 0x631, 0x628, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x6cc,
0x648, 0x642, 0x62a, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627,
0x6cc, 0x20, 0x634, 0x631, 0x642, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc,
0x20, 0x634, 0x631, 0x642, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644,
0x6cc, 0x627, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x6cc, 0x648, 0x642, 0x62a, 0x20,
0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20, 0x63a, 0x631,
0x628, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627,
0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20, 0x63a, 0x631, 0x628,
0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x6cc, 0x20,
0x63a, 0x631, 0x628, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x645, 0x647, 0x648,
0x631, 0x6cc, 0x20, 0x622, 0x630, 0x631, 0x628, 0x627, 0x6cc, 0x62c, 0x627, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62c, 0x645, 0x647,
0x648, 0x631, 0x6cc, 0x20, 0x622, 0x630, 0x631, 0x628, 0x627, 0x6cc, 0x62c, 0x627,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x62c, 0x645, 0x647, 0x648, 0x631, 0x6cc, 0x20, 0x622, 0x630, 0x631,
0x628, 0x627, 0x6cc, 0x62c, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x632,
0x648, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622,
0x632, 0x648, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x622, 0x632, 0x648, 0x631, 0x648, 0x642, 0x62a, 0x20,
0x628, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x634, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x628, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x634, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x628, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x634, 0x648, 0x642, 0x62a, 0x20, 0x628,
0x648, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x628, 0x648, 0x644, 0x6cc,
0x648, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644,
0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x628,
0x631, 0x627, 0x632, 0x6cc, 0x644, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x628, 0x631, 0x627, 0x632,
0x6cc, 0x644, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x628, 0x631, 0x648, 0x646,
0x626, 0x6cc, 0x20, 0x62f, 0x627, 0x631, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645,
0x648, 0x642, 0x62a, 0x20, 0x6a9, 0x6cc, 0x67e, 0x200c, 0x648, 0x631, 0x62f, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x6cc, 0x67e, 0x200c,
0x648, 0x631, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x6cc, 0x67e, 0x200c, 0x648, 0x631, 0x62f, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x686, 0x627, 0x645, 0x648,
0x631, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x686, 0x62a, 0x200c, 0x647, 0x627, 0x645,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x686, 0x62a, 0x200c,
0x647, 0x627, 0x645, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x686, 0x62a, 0x200c, 0x647, 0x627, 0x645, 0x648, 0x642,
0x62a, 0x20, 0x634, 0x6cc, 0x644, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x634, 0x6cc, 0x644, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634, 0x6cc, 0x644, 0x6cc,
0x648, 0x642, 0x62a, 0x20, 0x686, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x686, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x686, 0x6cc, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x6a9, 0x631,
0x6cc, 0x633, 0x645, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc,
0x631, 0x20, 0x6a9, 0x648, 0x6a9, 0x648, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x6a9,
0x644, 0x645, 0x628, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x6a9, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x644, 0x645,
0x628, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x6a9, 0x648, 0x6a9, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x62c, 0x632,
0x627, 0x6cc, 0x631, 0x20, 0x6a9, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x6a9,
0x648, 0x628, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x6a9, 0x648, 0x628, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x648, 0x628, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x62f, 0x6cc, 0x648, 0x6cc, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x62f, 0x648,
0x645, 0x648, 0x646, 0x20, 0x62f, 0x648, 0x631, 0x648, 0x6cc, 0x644, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x6cc, 0x645, 0x648, 0x631, 0x20, 0x634, 0x631, 0x642, 0x6cc,
0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x627,
0x6cc, 0x633, 0x62a, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x627, 0x6cc, 0x633, 0x62a,
0x631, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x627, 0x6cc, 0x633,
0x62a, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x6a9, 0x648, 0x627, 0x62f, 0x648,
0x631, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x631,
0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645,
0x631, 0x6a9, 0x632, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627,
0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x631, 0x648,
0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x645, 0x6a9, 0x627, 0x646, 0x200c, 0x647, 0x627, 0x6cc, 0x20,
0x62f, 0x6cc, 0x6af, 0x631, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x631, 0x648,
0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x631,
0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x63a, 0x631,
0x628, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62c,
0x632, 0x627, 0x6cc, 0x631, 0x20, 0x641, 0x627, 0x644, 0x6a9, 0x644, 0x646, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62c, 0x632, 0x627,
0x6cc, 0x631, 0x20, 0x641, 0x627, 0x644, 0x6a9, 0x644, 0x646, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x62c,
0x632, 0x627, 0x6cc, 0x631, 0x20, 0x641, 0x627, 0x644, 0x6a9, 0x644, 0x646, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x641, 0x6cc, 0x62c, 0x6cc, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x641, 0x6cc, 0x62c, 0x6cc, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x641, 0x6cc,
0x62c, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x6af, 0x648, 0x6cc, 0x627, 0x646, 0x20,
0x641, 0x631, 0x627, 0x646, 0x633, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x631,
0x632, 0x645, 0x6cc, 0x646, 0x200c, 0x647, 0x627, 0x6cc, 0x20, 0x62c, 0x646, 0x648,
0x628, 0x6cc, 0x20, 0x648, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6af, 0x627, 0x646,
0x20, 0x641, 0x631, 0x627, 0x646, 0x633, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x6af,
0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x633, 0x648, 0x642, 0x62a, 0x20,
0x6af, 0x627, 0x645, 0x628, 0x6cc, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x6af, 0x631,
0x62c, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x6af, 0x631, 0x62c, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6af, 0x631,
0x62c, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627,
0x6cc, 0x631, 0x20, 0x6af, 0x6cc, 0x644, 0x628, 0x631, 0x62a, 0x648, 0x642, 0x62a,
0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x648, 0x6cc, 0x686, 0x648, 0x642, 0x62a, 0x20,
0x634, 0x631, 0x642, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20,
0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20,
0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x63a,
0x631, 0x628, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x6af,
0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x6af,
0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x6af, 0x648, 0x627, 0x645, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x62e, 0x644, 0x6cc, 0x62c, 0x20, 0x641, 0x627, 0x631,
0x633, 0x648, 0x642, 0x62a, 0x20, 0x647, 0x627, 0x648, 0x627, 0x6cc, 0x6cc, 0x2010,
0x627, 0x644, 0x648, 0x634, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x647, 0x627, 0x648, 0x627, 0x6cc, 0x6cc, 0x2010, 0x627, 0x644, 0x648,
0x634, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x647, 0x627, 0x648, 0x627, 0x6cc, 0x6cc, 0x2010, 0x627, 0x644,
0x648, 0x634, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x647, 0x646, 0x6af, 0x200c, 0x6a9,
0x646, 0x6af, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x647,
0x646, 0x6af, 0x200c, 0x6a9, 0x646, 0x6af, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x647, 0x646, 0x6af, 0x200c, 0x6a9,
0x646, 0x6af, 0x648, 0x642, 0x62a, 0x20, 0x62e, 0x648, 0x648, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62e, 0x648, 0x648, 0x62f, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x62e, 0x648, 0x648, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x647, 0x646, 0x62f, 0x648,
0x642, 0x62a, 0x20, 0x627, 0x642, 0x6cc, 0x627, 0x646, 0x648, 0x633, 0x20, 0x647,
0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x647, 0x646, 0x62f, 0x648, 0x686, 0x6cc,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x646,
0x62f, 0x648, 0x646, 0x632, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x634, 0x631, 0x642,
0x20, 0x627, 0x646, 0x62f, 0x648, 0x646, 0x632, 0x6cc, 0x648, 0x642, 0x62a, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x627, 0x646, 0x62f, 0x648, 0x646, 0x632, 0x6cc, 0x648,
0x642, 0x62a, 0x20, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627,
0x6cc, 0x631, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x6cc, 0x631, 0x6a9,
0x648, 0x62a, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x627, 0x6cc, 0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x6cc,
0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x633,
0x631, 0x627, 0x626, 0x6cc, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x633,
0x631, 0x627, 0x626, 0x6cc, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x698, 0x627, 0x67e,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x698, 0x627,
0x67e, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x698, 0x627, 0x67e, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x67e,
0x62a, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x644, 0x648, 0x633, 0x6a9, 0x2010, 0x6a9,
0x627, 0x645, 0x686, 0x627, 0x62a, 0x633, 0x6a9, 0x6cc, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x67e, 0x62a, 0x631, 0x648, 0x67e, 0x627, 0x648,
0x644, 0x648, 0x633, 0x6a9, 0x2010, 0x6a9, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x633,
0x6a9, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x67e, 0x62a, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x644, 0x648,
0x633, 0x6a9, 0x2010, 0x6a9, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x633, 0x6a9, 0x6cc,
0x648, 0x642, 0x62a, 0x20, 0x642, 0x632, 0x627, 0x642, 0x633, 0x62a, 0x627, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x642, 0x632, 0x627, 0x642,
0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20,
0x642, 0x632, 0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20,
0x6a9, 0x631, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x6a9, 0x631, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x631, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x6a9,
0x648, 0x633, 0x631, 0x627, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x6a9, 0x631, 0x627,
0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x631, 0x627, 0x633, 0x646, 0x648, 0x6cc,
0x627, 0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x631, 0x627, 0x633, 0x646, 0x648, 0x6cc,
0x627, 0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x642, 0x631, 0x642, 0x6cc,
0x632, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x644, 0x627, 0x646,
0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20,
0x644, 0x627, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x644, 0x631, 0x62f, 0x647,
0x648, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x644, 0x631,
0x62f, 0x647, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x644, 0x631, 0x62f, 0x647, 0x648, 0x648, 0x642, 0x62a,
0x20, 0x645, 0x627, 0x6a9, 0x627, 0x626, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x627, 0x6a9, 0x627, 0x626, 0x648, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645,
0x627, 0x6a9, 0x627, 0x626, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x627, 0x6af,
0x627, 0x62f, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x645, 0x627, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645, 0x627, 0x6af,
0x627, 0x62f, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x627, 0x644, 0x632,
0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x648, 0x648,
0x642, 0x62a, 0x20, 0x645, 0x627, 0x631, 0x6a9, 0x648, 0x626, 0x632, 0x648, 0x642,
0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x645, 0x627, 0x631, 0x634,
0x627, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x648, 0x631, 0x6cc, 0x633, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x648, 0x631, 0x6cc,
0x633, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x645, 0x648, 0x631, 0x6cc, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x645,
0x627, 0x648, 0x633, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x634, 0x631, 0x642,
0x20, 0x645, 0x6a9, 0x632, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20, 0x645, 0x6a9, 0x632, 0x6cc, 0x6a9,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x634, 0x631, 0x642, 0x20, 0x645, 0x6a9, 0x632, 0x6cc, 0x6a9, 0x648, 0x642,
0x62a, 0x20, 0x627, 0x648, 0x644, 0x627, 0x646, 0x200c, 0x628, 0x627, 0x62a, 0x648,
0x631, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x648,
0x644, 0x627, 0x646, 0x200c, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x648,
0x644, 0x627, 0x646, 0x200c, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x648, 0x642, 0x62a,
0x20, 0x645, 0x633, 0x6a9, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x645, 0x633, 0x6a9, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645, 0x633, 0x6a9, 0x648, 0x648,
0x642, 0x62a, 0x20, 0x645, 0x6cc, 0x627, 0x646, 0x645, 0x627, 0x631, 0x648, 0x642,
0x62a, 0x20, 0x646, 0x627, 0x626, 0x648, 0x631, 0x648, 0x648, 0x642, 0x62a, 0x20,
0x646, 0x67e, 0x627, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x6a9, 0x627, 0x644, 0x62f,
0x648, 0x646, 0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x62f, 0x6cc, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x627, 0x644, 0x62f, 0x648,
0x646, 0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x62f, 0x6cc, 0x62f, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x627,
0x644, 0x62f, 0x648, 0x646, 0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x62f, 0x6cc, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x644, 0x646, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x646, 0x6cc, 0x648,
0x632, 0x6cc, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x644,
0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x646, 0x6cc, 0x648, 0x641, 0x627, 0x646,
0x62f, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x646, 0x6cc, 0x648, 0x641, 0x627, 0x646, 0x62f, 0x644, 0x646, 0x62f, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x646, 0x6cc, 0x648, 0x641, 0x627, 0x646, 0x62f, 0x644, 0x646, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x646, 0x6cc, 0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x62c,
0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644,
0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62c, 0x632,
0x6cc, 0x631, 0x647, 0x654, 0x20, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x6a9,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x646, 0x648, 0x631, 0x641,
0x648, 0x644, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x641, 0x631, 0x646, 0x627, 0x646,
0x62f, 0x648, 0x20, 0x62f, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6cc,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x641, 0x631,
0x646, 0x627, 0x646, 0x62f, 0x648, 0x20, 0x62f, 0x6cc, 0x20, 0x646, 0x648, 0x631,
0x648, 0x646, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x641, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x648,
0x20, 0x62f, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6cc, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x645, 0x627, 0x631,
0x6cc, 0x627, 0x646, 0x627, 0x6cc, 0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc, 0x648,
0x642, 0x62a, 0x20, 0x646, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633,
0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x646, 0x648,
0x648, 0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x646, 0x648, 0x648,
0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x627,
0x648, 0x645, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x648, 0x645, 0x633, 0x6a9,
0x648, 0x642, 0x62a, 0x20, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x67e, 0x627, 0x6a9, 0x633,
0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x67e, 0x627, 0x644, 0x627, 0x626, 0x648, 0x648, 0x642, 0x62a,
0x20, 0x67e, 0x627, 0x67e, 0x648, 0x627, 0x20, 0x6af, 0x6cc, 0x646, 0x647, 0x654,
0x20, 0x646, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x67e, 0x627, 0x631, 0x627, 0x6af,
0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x67e, 0x627, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x67e, 0x627, 0x631,
0x627, 0x6af, 0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x67e, 0x631, 0x648,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x67e, 0x631, 0x648,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x67e, 0x631, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x641, 0x6cc, 0x644, 0x6cc,
0x67e, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x641, 0x6cc, 0x644, 0x6cc, 0x67e, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x641, 0x6cc, 0x644, 0x6cc,
0x67e, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631,
0x20, 0x641, 0x648, 0x646, 0x6cc, 0x6a9, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x633,
0x646, 0x62a, 0x200c, 0x67e, 0x6cc, 0x631, 0x20, 0x648, 0x20, 0x645, 0x6cc, 0x6a9,
0x644, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x633, 0x646, 0x62a, 0x200c, 0x67e, 0x6cc, 0x631, 0x20, 0x648, 0x20, 0x645, 0x6cc,
0x6a9, 0x644, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x633, 0x646, 0x62a, 0x200c, 0x67e, 0x6cc, 0x631,
0x20, 0x648, 0x20, 0x645, 0x6cc, 0x6a9, 0x644, 0x648, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x67e, 0x6cc, 0x62a, 0x6a9, 0x627, 0x6cc, 0x631, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x67e, 0x648, 0x646, 0x627, 0x67e, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x67e,
0x6cc, 0x648, 0x646, 0x6af, 0x200c, 0x6cc, 0x627, 0x646, 0x6af, 0x648, 0x642, 0x62a,
0x20, 0x642, 0x6cc, 0x632, 0x6cc, 0x644, 0x200c, 0x627, 0x648, 0x631, 0x62f, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x642, 0x6cc, 0x632,
0x6cc, 0x644, 0x200c, 0x627, 0x648, 0x631, 0x62f, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x642, 0x6cc, 0x632,
0x6cc, 0x644, 0x200c, 0x627, 0x648, 0x631, 0x62f, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x631, 0x626, 0x648, 0x646, 0x6cc, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x631,
0x648, 0x62a, 0x631, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x627, 0x62e, 0x627,
0x644, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x633, 0x627, 0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x633, 0x627, 0x62e, 0x627,
0x644, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x627, 0x645, 0x627, 0x631,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x633, 0x627,
0x645, 0x627, 0x631, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x633, 0x627, 0x645, 0x648, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x633, 0x627, 0x645, 0x648, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x633,
0x627, 0x645, 0x648, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x6cc, 0x634, 0x644,
0x648, 0x642, 0x62a, 0x20, 0x633, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631, 0x648,
0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x633, 0x644, 0x6cc,
0x645, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x6cc,
0x627, 0x6cc, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x648, 0x642, 0x62a, 0x20,
0x633, 0x648, 0x631, 0x6cc, 0x646, 0x627, 0x645, 0x648, 0x642, 0x62a, 0x20, 0x634,
0x648, 0x648, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x647, 0x6cc, 0x62a,
0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x6cc, 0x67e, 0x647, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62a, 0x627, 0x6cc, 0x67e, 0x647,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x62a, 0x627, 0x6cc, 0x67e, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x62c, 0x6cc, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x648, 0x6a9, 0x644, 0x627, 0x626, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x648,
0x646, 0x6af, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x62a, 0x648, 0x646, 0x6af, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x686, 0x648, 0x626, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633,
0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633, 0x62a, 0x627,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x648, 0x648, 0x627, 0x644, 0x648, 0x648,
0x642, 0x62a, 0x20, 0x627, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x647, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x631, 0x648, 0x6af, 0x648,
0x626, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x627, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x647, 0x648, 0x642,
0x62a, 0x20, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x632, 0x628, 0x6a9, 0x633,
0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x648, 0x627, 0x646, 0x627, 0x62a, 0x648, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x627, 0x646, 0x627, 0x62a,
0x648, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x648, 0x627, 0x646, 0x627, 0x62a, 0x648, 0x648, 0x642, 0x62a, 0x20,
0x648, 0x646, 0x632, 0x648, 0x626, 0x644, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x648,
0x644, 0x627, 0x62f, 0x6cc, 0x200c, 0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x644, 0x627, 0x62f, 0x6cc,
0x200c, 0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648, 0x644, 0x627, 0x62f, 0x6cc,
0x200c, 0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x648, 0x644,
0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x648, 0x642, 0x62a, 0x20,
0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x6cc,
0x631, 0x647, 0x654, 0x20, 0x648, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x648,
0x627, 0x644, 0x6cc, 0x633, 0x20, 0x648, 0x20, 0x641, 0x648, 0x62a, 0x648, 0x646,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6cc, 0x627, 0x6a9,
0x648, 0x62a, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9,
0x648, 0x642, 0x62a, 0x20, 0x6cc, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628,
0x648, 0x631, 0x6af, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x6cc, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631, 0x6af, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x6cc, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631, 0x6af, 0x648,
0x642, 0x62a, 0x20, 0x6cc, 0x648, 0x6a9, 0x627, 0x646, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f,
0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f,
0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x70, 0x6f, 0x142, 0x75, 0x64, 0x6e, 0x69, 0x6f, 0x77,
0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x142, 0x6d, 0x61, 0x63, 0x6b, 0x69,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x142, 0x6d, 0x61, 0x63, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x142, 0x6d, 0x61, 0x63, 0x6b, 0x69,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61,
0x6d, 0x61, 0x7a, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b,
0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79,
0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b,
0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79,
0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b,
0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79,
0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63,
0x7a, 0x6e, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79,
0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70,
0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41,
0x70, 0x69, 0x61, 0x41, 0x70, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x41, 0x70, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x61, 0x75, 0x6b, 0x74, 0x61, 0x75, 0x63, 0x6b, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x61, 0x75, 0x6b, 0x74, 0x61, 0x75, 0x63, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x6b, 0x74, 0x61, 0x75, 0x63, 0x6b,
0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x61, 0x6b, 0x74, 0x69, 0x75, 0x62, 0x69, 0x144, 0x73, 0x6b, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x61, 0x6b, 0x74, 0x69, 0x75, 0x62, 0x69, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6b, 0x74, 0x69,
0x75, 0x62, 0x69, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0xf3, 0x142, 0x77, 0x79, 0x73,
0x65, 0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x50, 0xf3,
0x142, 0x77, 0x79, 0x73, 0x65, 0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73,
0x6b, 0x69, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x50, 0xf3, 0x142,
0x77, 0x79, 0x73, 0x65, 0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b,
0x69, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x79, 0x6e, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x79, 0x63, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x79, 0x63, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x79, 0x63, 0x6b, 0x69, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72,
0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b,
0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b,
0x6f, 0x77, 0x6f, 0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f,
0x77, 0x6f, 0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77,
0x6f, 0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x7a, 0x61,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17c, 0x61, 0x6e, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x65, 0x6a, 0x64, 0x17c, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17c,
0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x79, 0x41, 0x7a, 0x6f, 0x72, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x41, 0x7a, 0x6f, 0x72, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x7a, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x7a, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x7a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61,
0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x77, 0x69,
0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c,
0x69, 0x61, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20,
0x5a, 0x69, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x67, 0x6f, 0x20, 0x50, 0x72,
0x7a, 0x79, 0x6c, 0x105, 0x64, 0x6b, 0x61, 0x57, 0x79, 0x73, 0x70, 0x79,
0x20, 0x5a, 0x69, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x67, 0x6f, 0x20, 0x50,
0x72, 0x7a, 0x79, 0x6c, 0x105, 0x64, 0x6b, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x5a, 0x69, 0x65,
0x6c, 0x6f, 0x6e, 0x65, 0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x79, 0x6c,
0x105, 0x64, 0x6b, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x43, 0x7a,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x43, 0x68, 0x69, 0x6e, 0x79, 0x43, 0x68, 0x69, 0x6e, 0x79, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x43, 0x68, 0x69, 0x6e, 0x79, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x42,
0x6f, 0x17c, 0x65, 0x67, 0x6f, 0x20, 0x4e, 0x61, 0x72, 0x6f, 0x64, 0x7a,
0x65, 0x6e, 0x69, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73,
0x70, 0x79, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x77, 0x65, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x61, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x57, 0x79,
0x73, 0x70, 0x79, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x61, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x4b, 0x75, 0x62, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54,
0x69, 0x6d, 0x6f, 0x72, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20,
0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x61, 0x57,
0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e,
0x6f, 0x63, 0x6e, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x57,
0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e,
0x6f, 0x63, 0x6e, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x45, 0x6b,
0x77, 0x61, 0x64, 0x6f, 0x72, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72,
0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f,
0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64,
0x6b, 0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x64, 0x61, 0x6c, 0x73, 0x7a, 0x79, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x79, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x46, 0x69, 0x64, 0x17c, 0x69, 0x46, 0x69, 0x64, 0x17c, 0x69, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x46, 0x69, 0x64, 0x17c, 0x69, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x47, 0x75, 0x6a, 0x61, 0x6e, 0x61, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x61, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x69, 0x65, 0x20,
0x54, 0x65, 0x72, 0x79, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x20, 0x50, 0x6f,
0x142, 0x75, 0x64, 0x6e, 0x69, 0x6f, 0x77, 0x65, 0x20, 0x69, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x79, 0x63, 0x7a, 0x6e, 0x65, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f,
0x73, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x61, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x47, 0x72, 0x75, 0x7a, 0x6a, 0x61, 0x47, 0x72, 0x75, 0x7a, 0x6a,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x47, 0x72, 0x75, 0x7a,
0x6a, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70,
0x79, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x61, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x75, 0x6e, 0x69, 0x77, 0x65, 0x72, 0x73, 0x61, 0x6c,
0x6e, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x61, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x47, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x61, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x47, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x5a, 0x61, 0x74,
0x6f, 0x6b, 0x61, 0x20, 0x50, 0x65, 0x72, 0x73, 0x6b, 0x61, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x48, 0x61, 0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x79, 0x48, 0x61, 0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x48, 0x61, 0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f,
0x6e, 0x67, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x62, 0x64,
0x6f, 0x4b, 0x6f, 0x62, 0x64, 0x6f, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x4b, 0x6f, 0x62, 0x64, 0x6f, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x69, 0x6e, 0x64, 0x79, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x79, 0x6a,
0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x69, 0x6e, 0x64, 0x6f,
0x63, 0x68, 0x69, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x6a, 0x61, 0x20, 0x15a, 0x72,
0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x6a, 0x61, 0x20, 0x57, 0x73, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x6a, 0x61, 0x20, 0x5a, 0x61, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x49, 0x72, 0x6b, 0x75, 0x63,
0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65,
0x6c, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x61, 0x4a, 0x61, 0x70,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4a,
0x61, 0x70, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x50, 0x69, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x77, 0x142, 0x6f, 0x77,
0x73, 0x6b, 0x20, 0x4b, 0x61, 0x6d, 0x63, 0x7a, 0x61, 0x63, 0x6b, 0x69,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x50, 0x69, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x77, 0x142, 0x6f, 0x77, 0x73, 0x6b, 0x20, 0x4b, 0x61, 0x6d, 0x63,
0x7a, 0x61, 0x63, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x69,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x77, 0x142, 0x6f, 0x77, 0x73, 0x6b,
0x20, 0x4b, 0x61, 0x6d, 0x63, 0x7a, 0x61, 0x63, 0x6b, 0x69, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x57,
0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x5a,
0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x65, 0x7a, 0x6a, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x65, 0x64, 0x69, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x72, 0x6b, 0x69, 0x7a, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57,
0x79, 0x73, 0x70, 0x79, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69,
0x63, 0x7a, 0x6e, 0x79, 0x29, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69,
0x63, 0x7a, 0x6e, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69,
0x63, 0x7a, 0x6e, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x55, 0x142, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d,
0x6f, 0x73, 0x6b, 0x77, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x4e, 0x6f, 0x77, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x6f, 0x77, 0x61,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x4e, 0x6f, 0x77,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x4e, 0x6f, 0x77, 0x61, 0x20, 0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x46, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x69,
0x75, 0x65, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4e, 0x6f, 0x77, 0x6f,
0x73, 0x79, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e, 0x6f, 0x77, 0x61, 0x20,
0x47, 0x77, 0x69, 0x6e, 0x65, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x6a, 0x50, 0x61, 0x72, 0x61, 0x67,
0x77, 0x61, 0x6a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x50, 0x61,
0x72, 0x61, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x50, 0x65, 0x72, 0x75, 0x50, 0x65, 0x72, 0x75, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x50, 0x65, 0x72, 0x75, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x79, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x50, 0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6b, 0x79, 0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64, 0x7a, 0x6b,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6b, 0x79, 0x7a, 0x79, 0x142, 0x6f,
0x72, 0x64, 0x7a, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6b, 0x79,
0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64, 0x7a, 0x6b, 0x69, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x52, 0x65, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x74, 0x68,
0x65, 0x72, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x63, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x53, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x53, 0x65, 0x73, 0x7a, 0x65, 0x6c, 0x65, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x53,
0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x142, 0x75,
0x64, 0x6e, 0x69, 0x6f, 0x77, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53,
0x79, 0x6f, 0x77, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x61, 0x6a, 0x70,
0x65, 0x6a, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x54, 0x61, 0x64, 0x17c, 0x79, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x43,
0x68, 0x75, 0x75, 0x6b, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x55, 0x72, 0x75,
0x67, 0x77, 0x61, 0x6a, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x6a, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61,
0x6a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x57, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x57, 0x142, 0x61, 0x64, 0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x57, 0x142, 0x61, 0x64, 0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x57, 0x142, 0x61, 0x64, 0x79,
0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x57, 0x6f, 0x142, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x6f, 0x142,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x57, 0x6f, 0x142, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4a, 0x61, 0x6b, 0x75,
0x63, 0x6b, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x79, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x79,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x79, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4a, 0x75, 0x6b, 0x6f, 0x6e,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41,
0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x63, 0x72,
0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x63, 0x72,
0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x66, 0x65, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61,
0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53,
0x75, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61,
0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d,
0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x73,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x61, 0x6e, 0x68, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66,
0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72,
0x6d, 0xea, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x72, 0x6d, 0xea, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x41, 0x72, 0x6d, 0xea, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2,
0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41,
0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69, 0x63,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x2d, 0x4f, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x6f, 0x2d, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x6f, 0x2d, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x72, 0x7a, 0x65, 0x69, 0x62, 0x61, 0x69,
0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x72, 0x7a,
0x65, 0x69, 0x62, 0x61, 0x69, 0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x72, 0x7a, 0x65, 0x69, 0x62, 0x61, 0x69,
0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x6f, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65,
0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x75, 0x74, 0xe3, 0x6f,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x42,
0x6f, 0x6c, 0xed, 0x76, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42,
0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61,
0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf4, 0x6d, 0x62, 0x69,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf4, 0x6d,
0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43,
0x6f, 0x6c, 0xf4, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20,
0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4d, 0x65,
0x69, 0x6f, 0x20, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20,
0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x44, 0x61,
0x76, 0x69, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x2d, 0x4c,
0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0xe1, 0x73, 0x63, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x45, 0x71,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x45, 0x78, 0x74,
0x72, 0x65, 0x6d, 0x6f, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c,
0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20,
0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x54, 0x65, 0x72, 0x72,
0x69, 0x74, 0xf3, 0x72, 0x69, 0x6f, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x63, 0x65, 0x73, 0x65, 0x73, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x6c,
0x20, 0x65, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69, 0x64, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47,
0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20,
0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x4d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x6f,
0x6c, 0x66, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x48, 0x61, 0x76, 0x61, 0xed, 0x20, 0x65, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x76, 0x61, 0xed, 0x20, 0x65,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x48, 0x61,
0x76, 0x61, 0xed, 0x20, 0x65, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4f, 0x63,
0x65, 0x61, 0x6e, 0x6f, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe9,
0x73, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x61,
0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x4a, 0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4a,
0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x4a, 0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x61, 0x7a, 0x61, 0x71, 0x75, 0x69,
0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x43, 0x61, 0x7a, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74,
0xe3, 0x6f, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x61,
0x7a, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x69, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x69, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65,
0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x51, 0x75, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73,
0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73,
0x20, 0x64, 0x61, 0x20, 0x4c, 0x69, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x4d, 0x61, 0x6c, 0xe1, 0x73, 0x69, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x61, 0x73, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61,
0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0xed, 0x63, 0x69, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x69, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf4, 0x6e, 0x69,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0xf4, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0xf4, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a,
0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64,
0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65,
0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61,
0x73, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69,
0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61,
0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47,
0x75, 0x69, 0x6e, 0xe9, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x46, 0xea, 0x6e, 0x69, 0x78,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53,
0xe3, 0x6f, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x53, 0xe3, 0x6f, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x65,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20,
0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xe3, 0x6f, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x79, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x52, 0x65, 0x75,
0x6e, 0x69, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63,
0x61, 0x6c, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65, 0x69,
0x63, 0x68, 0x65, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68,
0x61, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xe3, 0x6f, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf3,
0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x61, 0x69,
0x74, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x61, 0x6a, 0x69,
0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x63,
0x6f, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x63, 0x6f, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x54, 0x75, 0x72, 0x63, 0x6f, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x56,
0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x45, 0x63, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x63, 0x72, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x66, 0x65, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x53, 0x75, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f,
0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72,
0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f,
0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x68, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed,
0x66, 0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x74,
0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f,
0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x71,
0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x71, 0x74, 0x61,
0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x71, 0x74,
0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0xe1, 0x62, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41,
0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69, 0x63,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x6f, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x73,
0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x73,
0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x63,
0x68, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x63, 0x68, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x63, 0x68, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x42, 0x75, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0xed, 0x76, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x72, 0x75, 0x6e,
0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61,
0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62,
0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61,
0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x4e, 0x61, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x63, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43,
0x6f, 0x6c, 0xf4, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f,
0x6c, 0xf4, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43,
0x6f, 0x6c, 0xf4, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f,
0x72, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50,
0xe1, 0x73, 0x63, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61,
0x20, 0x64, 0x61, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50,
0xe1, 0x73, 0x63, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x45, 0x71, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x45, 0x78, 0x74, 0x72, 0x65, 0x6d, 0x6f, 0x20, 0x4c,
0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68,
0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e,
0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61,
0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x69, 0x73, 0x20, 0x65,
0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x63, 0x74, 0x69, 0x63, 0x61, 0x73,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20,
0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65,
0xf3, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65,
0xf3, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e,
0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e,
0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75,
0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x76, 0x61, 0x69, 0x20,
0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x48, 0x61, 0x76, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x76, 0x61,
0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xcd, 0x6e, 0x64, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4f, 0x63, 0x65,
0x61, 0x6e, 0x6f, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68,
0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x61, 0x20,
0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x49, 0x72, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4a, 0x61, 0x70, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x4a, 0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x4a, 0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x61, 0x7a, 0x61, 0x71,
0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x43, 0x61, 0x7a, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3,
0x6f, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x61, 0x7a, 0x61, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe3, 0x6f, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43,
0x6f, 0x72, 0x65, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x51, 0x75, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x72,
0x69, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4c, 0x69,
0x6e, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0xe1, 0x73, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75,
0x65, 0x73, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4d,
0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x61,
0x75, 0x72, 0xed, 0x63, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4d,
0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x76, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x69, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf3,
0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf3,
0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2,
0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65,
0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0xea, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x49, 0x6c, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0xe9, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65,
0x72, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20,
0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20, 0x50,
0x65, 0x64, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20,
0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20,
0x53, 0x65, 0x69, 0x63, 0x68, 0x65, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20,
0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x53, 0x75, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x61, 0x69, 0x74, 0x69, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0xe9, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x54, 0x61, 0x6a, 0x69, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x54, 0x75, 0x72, 0x71, 0x75, 0x65, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x71, 0x75, 0x65,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x54, 0x75, 0x72, 0x71, 0x75, 0x65, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x45, 0x63, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45,
0x63, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x41,
0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64,
0x101, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x41,
0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x113, 0x69, 0x6d,
0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b,
0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x113,
0x69, 0x6d, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x44, 0x113,
0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113, 0x72,
0x69, 0x6b, 0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20,
0x6b, 0x65, 0x72, 0x64, 0x101, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61,
0x73, 0x20, 0x67, 0x101, 0x72, 0x62, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x72,
0x64, 0x101, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x67,
0x101, 0x72, 0x62, 0x61, 0x6e, 0x20, 0x7a, 0x113, 0x69, 0x6d, 0x61, 0x73,
0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b,
0x61, 0x73, 0x20, 0x67, 0x101, 0x72, 0x62, 0x61, 0x6e, 0x20, 0x64, 0x61,
0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x50, 0x61,
0x63, 0x12b, 0x66, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113,
0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x50,
0x61, 0x63, 0x12b, 0x66, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d,
0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x113, 0x69, 0x6d, 0x61,
0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x50, 0x61, 0x63, 0x12b, 0x66,
0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b,
0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x41, 0x74, 0x6c, 0x101, 0x6e, 0x74, 0x69, 0x73, 0x6b,
0x61, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x41, 0x74, 0x6c, 0x101, 0x6e,
0x74, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x7a, 0x113, 0x69, 0x6d, 0x61, 0x73,
0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x41, 0x74, 0x6c, 0x101, 0x6e, 0x74,
0x69, 0x73, 0x6b, 0x61, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20,
0x6b, 0x65, 0x72, 0x64, 0x101, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c,
0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x6b,
0x65, 0x72, 0x64, 0x101, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61,
0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x113,
0x69, 0x6d, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d,
0x70, 0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b,
0x65, 0x72, 0x64, 0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61,
0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x113, 0x69,
0x6d, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x44, 0x113, 0x69,
0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70,
0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x57, 0x61, 0x6b, 0x6b, 0x61, 0x72, 0x69, 0x73, 0x6b,
0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x6b,
0x65, 0x72, 0x64, 0x101, 0x57, 0x61, 0x6b, 0x6b, 0x61, 0x72, 0x69, 0x73,
0x6b, 0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20,
0x7a, 0x113, 0x69, 0x6d, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101,
0x57, 0x61, 0x6b, 0x6b, 0x61, 0x72, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20,
0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67,
0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0xa05,
0xa2b, 0xa3c, 0xa17, 0xa3e, 0xa28, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa05, 0xa2b,
0xa30, 0xa40, 0xa15, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30,
0xa2c, 0xa40, 0x20, 0xa05, 0xa2b, 0xa30, 0xa40, 0xa15, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa26, 0xa71, 0xa16, 0xa23, 0xa40, 0x20, 0xa05, 0xa2b, 0xa3c, 0xa30,
0xa40, 0xa15, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa05, 0xa2b, 0xa30, 0xa40,
0xa15, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40,
0x20, 0xa05, 0xa2b, 0xa30, 0xa40, 0xa15, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20,
0xa05, 0xa2b, 0xa30, 0xa40, 0xa15, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa32, 0xa3e,
0xa38, 0xa15, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa32, 0xa3e, 0xa38,
0xa15, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa05, 0xa32, 0xa3e, 0xa38, 0xa15, 0xa3e, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15,
0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa32, 0xa2e, 0xa3e,
0xa1f, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa32, 0xa2e, 0xa3e, 0xa1f,
0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02,
0xa05, 0xa32, 0xa2e, 0xa3e, 0xa1f, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0x2d,
0xa30, 0xa41, 0xa71, 0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa2e, 0xa47,
0xa1c, 0xa3c, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa2e, 0xa47, 0xa1c,
0xa3c, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa05, 0xa2e, 0xa47, 0xa1c, 0xa3c, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71,
0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa15,
0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71,
0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa15,
0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e,
0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20,
0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40,
0x20, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09,
0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20,
0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e,
0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09,
0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20,
0xa2e, 0xa3e, 0xa09, 0xa02, 0xa1f, 0xa47, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40,
0x20, 0xa2e, 0xa3e, 0xa09, 0xa02, 0xa1f, 0xa47, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40,
0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2e, 0xa3e, 0xa09, 0xa02,
0xa1f, 0xa47, 0xa28, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e,
0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05,
0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f,
0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40,
0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15, 0x20, 0xa2a, 0xa4d, 0xa30,
0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa10, 0xa2a, 0xa40,
0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa10, 0xa2a, 0xa40, 0xa06, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa10, 0xa2a, 0xa40,
0xa06, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa05, 0xa15, 0xa24, 0xa3e, 0xa09, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02,
0xa05, 0xa15, 0xa24, 0xa3e, 0xa09, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa15, 0xa24, 0xa3e, 0xa09, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0x2d, 0xa30, 0xa41, 0xa71, 0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02,
0xa05, 0xa15, 0xa24, 0xa4b, 0xa2c, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa15,
0xa24, 0xa4b, 0xa2c, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38, 0xa2e,
0xa3e, 0xa02, 0xa05, 0xa15, 0xa24, 0xa4b, 0xa2c, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0x2d, 0xa30, 0xa41, 0xa71, 0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa30,
0xa2c, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa2c, 0xa40, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30,
0xa2c, 0xa40, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05,
0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71,
0xa1b, 0xa2e, 0xa40, 0x20, 0xa05, 0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa05,
0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20,
0xa05, 0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05,
0xa30, 0xa2e, 0xa40, 0xa28, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05,
0xa30, 0xa2e, 0xa40, 0xa28, 0xa40, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa2e, 0xa40, 0xa28, 0xa40, 0xa06,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa05, 0xa1f, 0xa32, 0xa3e, 0xa02, 0xa1f, 0xa3f, 0xa15, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1f, 0xa32, 0xa3e, 0xa02, 0xa1f, 0xa3f, 0xa15,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05,
0xa1f, 0xa32, 0xa3e, 0xa02, 0xa1f, 0xa3f, 0xa15, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15,
0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa02, 0xa26,
0xa30, 0xa40, 0x20, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06,
0xa08, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47,
0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38,
0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02,
0xa26, 0xa30, 0xa40, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40,
0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa2a, 0xa71,
0xa1b, 0xa2e, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d,
0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30,
0xa40, 0x20, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47,
0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20,
0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38,
0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20,
0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06,
0xa08, 0x20, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47,
0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a,
0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32,
0xa40, 0xa06, 0xa08, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d,
0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06,
0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa2a, 0xa71,
0xa1b, 0xa2e, 0xa40, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa3c, 0xa30, 0xa2c, 0xa3e, 0xa08, 0xa1c,
0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa3c, 0xa30, 0xa2c,
0xa3e, 0xa08, 0xa1c, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa3c, 0xa30, 0xa2c, 0xa3e, 0xa08, 0xa1c,
0xa3e, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa4b, 0xa30, 0xa47, 0xa38, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa4b, 0xa30, 0xa47, 0xa38, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa4b,
0xa30, 0xa47, 0xa38, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26,
0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2c, 0xa70, 0xa17, 0xa32, 0xa3e, 0xa26,
0xa47, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2c, 0xa70, 0xa17, 0xa32,
0xa3e, 0xa26, 0xa47, 0xa38, 0xa3c, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2c, 0xa70, 0xa17, 0xa32, 0xa3e, 0xa26, 0xa47, 0xa38,
0xa3c, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2d, 0xa42, 0xa1f, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2c, 0xa4b, 0xa32, 0xa40, 0xa35, 0xa40, 0xa06, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa40, 0xa06,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1c, 0xa3c, 0xa40,
0xa32, 0xa40, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa40, 0xa06,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2c, 0xa30, 0xa42, 0xa28, 0xa47, 0xa08, 0x20, 0xa26, 0xa3e,
0xa30, 0xa42, 0xa38, 0xa32, 0xa3e, 0xa2e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15,
0xa47, 0xa2a, 0x20, 0xa35, 0xa30, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15,
0xa47, 0xa2a, 0x20, 0xa35, 0xa30, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa2a, 0x20, 0xa35, 0xa30, 0xa21,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa38, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02,
0xa1a, 0xa3e, 0xa2e, 0xa4b, 0xa30, 0xa4b, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa48, 0xa25, 0xa2e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa1a, 0xa48, 0xa25, 0xa2e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa48, 0xa25, 0xa2e, 0x20, 0xa2a, 0xa4d,
0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa3f,
0xa32, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa3f, 0xa32, 0xa40, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa3f,
0xa32, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa40, 0xa28, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa1a, 0xa40, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa1a, 0xa40, 0xa28, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e,
0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4d, 0xa30, 0xa3f, 0xa38,
0xa2e, 0xa38, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa15, 0xa4b, 0xa15, 0xa38, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02,
0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa32, 0xa70, 0xa2c, 0xa40,
0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa32, 0xa70, 0xa2c, 0xa40,
0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa15, 0xa4b, 0xa32, 0xa70, 0xa2c, 0xa40, 0xa06, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa41,
0xa71, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa15, 0xa41, 0xa71, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02,
0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa15, 0xa41, 0xa71, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20,
0xa05, 0xa71, 0xa27, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26,
0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa3f, 0xa0a, 0xa2c, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa3f, 0xa0a, 0xa2c, 0xa3e, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa3f, 0xa0a, 0xa2c,
0xa3e, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa21, 0xa47, 0xa35, 0xa3f, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa21, 0xa3f, 0xa09, 0xa2e, 0xa4b, 0xa02, 0xa1f, 0x20, 0xa21, 0xa3f, 0xa09, 0xa30,
0xa35, 0xa3f, 0xa32, 0xa47, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30,
0xa2c, 0xa40, 0x20, 0xa24, 0xa3f, 0xa2e, 0xa42, 0xa30, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa08, 0xa38, 0xa1f, 0xa30, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa08, 0xa38, 0xa1f, 0xa30, 0x20, 0xa06, 0xa08,
0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa08, 0xa38, 0xa1f, 0xa30, 0x20, 0xa06, 0xa08, 0xa32, 0xa48,
0xa02, 0xa21, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa15, 0xa35, 0xa47, 0xa21, 0xa4b, 0xa30,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa71, 0xa27, 0x20, 0xa2f, 0xa42, 0xa30,
0xa2a, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa71, 0xa27, 0x20, 0xa2f,
0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2e, 0xa71, 0xa27, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30,
0xa2a, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40,
0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2f,
0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20,
0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa4b, 0xa30, 0x2d, 0xa2a,
0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30,
0xa2a, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40,
0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2f,
0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20,
0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3e, 0xa15, 0xa32,
0xa48, 0xa02, 0xa21, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0xa38, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3e, 0xa15, 0xa32, 0xa48, 0xa02, 0xa21,
0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0xa38, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3e, 0xa15, 0xa32,
0xa48, 0xa02, 0xa21, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0xa38, 0x20,
0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3f, 0xa1c, 0xa3c, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2b, 0xa3c, 0xa3f, 0xa1c, 0xa3c, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3f, 0xa1c, 0xa3c, 0xa40,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2b, 0xa4d, 0xa30, 0xa48, 0xa02, 0xa1a, 0x20, 0xa17, 0xa41,
0xa0f, 0xa28, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa4d, 0xa30, 0xa48,
0xa02, 0xa1a, 0x20, 0xa26, 0xa71, 0xa16, 0xa23, 0xa40, 0x20, 0xa05, 0xa24, 0xa47,
0x20, 0xa10, 0xa02, 0xa1f, 0xa3e, 0xa30, 0xa1f, 0xa3f, 0xa15, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa17, 0xa32, 0xa3e, 0xa2a, 0xa3e, 0xa17, 0xa4b, 0xa38, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa17, 0xa48, 0xa02, 0xa2c, 0xa40, 0xa05, 0xa30, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa1c, 0xa3e, 0xa30, 0xa1c, 0xa40, 0xa06, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa1c, 0xa3e, 0xa30, 0xa1c, 0xa40, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1c, 0xa3e, 0xa30, 0xa1c, 0xa40,
0xa06, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa17, 0xa3f, 0xa32, 0xa2c, 0xa30, 0xa1f, 0x20, 0xa06,
0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17, 0xa4d,
0xa30, 0xa40, 0xa28, 0xa35, 0xa3f, 0xa1a, 0x20, 0xa2e, 0xa40, 0xa28, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30,
0xa40, 0xa28, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a,
0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28, 0xa32, 0xa48,
0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28,
0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20,
0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40,
0x20, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30,
0xa40, 0xa28, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa17,
0xa4d, 0xa30, 0xa40, 0xa28, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17,
0xa41, 0xa06, 0xa2e, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa16, 0xa3e, 0xa5c, 0xa40,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17,
0xa41, 0xa2f, 0xa3e, 0xa28, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa35,
0xa3e, 0xa08, 0x2d, 0xa05, 0xa32, 0xa47, 0xa2f, 0xa42, 0xa38, 0xa3c, 0xa3f, 0xa05,
0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa35, 0xa3e, 0xa08, 0x2d, 0xa05,
0xa32, 0xa47, 0xa2f, 0xa42, 0xa38, 0xa3c, 0xa3f, 0xa05, 0xa28, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa35, 0xa3e, 0xa08,
0x2d, 0xa05, 0xa32, 0xa47, 0xa2f, 0xa42, 0xa38, 0xa3c, 0xa3f, 0xa05, 0xa28, 0x20,
0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa39, 0xa3e, 0xa02, 0xa17, 0x20, 0xa15, 0xa3e, 0xa02, 0xa17, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa39, 0xa3e, 0xa02, 0xa17, 0x20, 0xa15, 0xa3e, 0xa02, 0xa17, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa3e,
0xa02, 0xa17, 0x20, 0xa15, 0xa3e, 0xa02, 0xa17, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa4b,
0xa35, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa4b, 0xa35, 0xa21, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa4b,
0xa35, 0xa21, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2d, 0xa3e, 0xa30, 0xa24, 0xa40, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa3f, 0xa70,
0xa26, 0x20, 0xa2e, 0xa39, 0xa3e, 0xa02, 0xa38, 0xa3e, 0xa17, 0xa30, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa07, 0xa70, 0xa21, 0xa4b, 0xa1a, 0xa3e, 0xa08, 0xa28, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa71, 0xa27, 0x20, 0xa07, 0xa70, 0xa21,
0xa4b, 0xa28, 0xa47, 0xa38, 0xa3c, 0xa40, 0xa06, 0xa08, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa07, 0xa70, 0xa21, 0xa4b, 0xa28,
0xa47, 0xa38, 0xa3c, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71,
0xa1b, 0xa2e, 0xa40, 0x20, 0xa07, 0xa70, 0xa21, 0xa4b, 0xa28, 0xa47, 0xa38, 0xa3c,
0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa08, 0xa30, 0xa3e, 0xa28, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa08, 0xa30, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa08, 0xa30, 0xa3e, 0xa28, 0x20,
0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa07, 0xa30, 0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa07, 0xa30, 0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa30, 0xa15, 0xa41, 0xa24, 0xa38,
0xa15, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa1c, 0xa3c, 0xa30, 0xa3e, 0xa08, 0xa32, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa1c, 0xa3c, 0xa30, 0xa3e, 0xa08, 0xa32, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa1c,
0xa3c, 0xa30, 0xa3e, 0xa08, 0xa32, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38,
0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1c, 0xa2a, 0xa3e, 0xa28, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa1c, 0xa2a, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1c, 0xa2a, 0xa3e, 0xa28, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15,
0xa1c, 0xa3c, 0xa3e, 0xa16, 0xa3c, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa15, 0xa1c, 0xa3c, 0xa3e,
0xa16, 0xa3c, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a,
0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa15, 0xa1c, 0xa3c, 0xa3e, 0xa16, 0xa3c, 0xa38,
0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa30, 0xa40,
0xa06, 0xa08, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa30, 0xa40, 0xa06,
0xa08, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa15, 0xa4b, 0xa30, 0xa40, 0xa06, 0xa08, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e,
0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa38, 0xa30, 0xa47,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4d, 0xa30, 0xa3e, 0xa38, 0xa28, 0xa4b,
0xa2f, 0xa3e, 0xa30, 0xa38, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4d,
0xa30, 0xa3e, 0xa38, 0xa28, 0xa4b, 0xa2f, 0xa3e, 0xa30, 0xa38, 0xa15, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4d, 0xa30,
0xa3e, 0xa38, 0xa28, 0xa4b, 0xa2f, 0xa3e, 0xa30, 0xa38, 0xa15, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa15, 0xa3f, 0xa30, 0xa17, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa32, 0xa70, 0xa15, 0xa3e, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa32,
0xa3e, 0xa08, 0xa28, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa32, 0xa4c, 0xa30, 0xa21, 0x20, 0xa39, 0xa4b, 0xa35, 0xa47,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa32, 0xa4c, 0xa30, 0xa21, 0x20, 0xa39, 0xa4b,
0xa35, 0xa47, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa32, 0xa4c, 0xa30, 0xa21, 0x20, 0xa39, 0xa4b, 0xa35, 0xa47, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e,
0xa15, 0xa3e, 0xa09, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa2e, 0xa15, 0xa3e, 0xa09,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa2e,
0xa15, 0xa3e, 0xa09, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0x2d, 0xa30, 0xa41, 0xa71,
0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa2e, 0xa48, 0xa17, 0xa47, 0xa21, 0xa28,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa48, 0xa17, 0xa47, 0xa21, 0xa28, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa48,
0xa17, 0xa47, 0xa21, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20,
0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa32, 0xa47, 0xa38, 0xa3c,
0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e, 0xa32, 0xa26, 0xa40,
0xa35, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e, 0xa30, 0xa15, 0xa3f, 0xa38,
0xa3e, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e, 0xa30, 0xa38, 0xa3c,
0xa32, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2e, 0xa4c, 0xa30, 0xa3f, 0xa38, 0xa3c, 0xa38, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2e, 0xa4c, 0xa30, 0xa3f, 0xa38, 0xa3c, 0xa38, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa4c, 0xa30, 0xa3f, 0xa38,
0xa3c, 0xa38, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa4c, 0xa38, 0xa28, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2e, 0xa48, 0xa15, 0xa38, 0xa40, 0xa15, 0xa28, 0x20, 0xa2a, 0xa48,
0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa48,
0xa15, 0xa38, 0xa40, 0xa15, 0xa28, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f,
0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2e, 0xa48, 0xa15, 0xa38, 0xa40, 0xa15, 0xa28, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f,
0xa2b, 0xa3f, 0xa15, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa32, 0xa28, 0x20, 0xa2c, 0xa1f, 0xa4b, 0xa30,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa32, 0xa28, 0x20, 0xa2c, 0xa1f, 0xa4b,
0xa30, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa09, 0xa32, 0xa28, 0x20, 0xa2c, 0xa1f, 0xa4b, 0xa30, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e,
0xa3e, 0xa38, 0xa15, 0xa4b, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e, 0xa38,
0xa15, 0xa4b, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2e, 0xa3e, 0xa38, 0xa15, 0xa4b, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3f, 0xa06,
0xa02, 0xa2e, 0xa3e, 0xa30, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3e, 0xa09,
0xa30, 0xa42, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa47, 0xa2a, 0xa3e, 0xa32,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0x20, 0xa15, 0xa48, 0xa32,
0xa47, 0xa21, 0xa4b, 0xa28, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28,
0xa3f, 0xa0a, 0x20, 0xa15, 0xa48, 0xa32, 0xa47, 0xa21, 0xa4b, 0xa28, 0xa40, 0xa06,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28,
0xa3f, 0xa0a, 0x20, 0xa15, 0xa48, 0xa32, 0xa47, 0xa21, 0xa4b, 0xa28, 0xa40, 0xa06,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa48, 0xa02,
0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa1c, 0xa3c, 0xa40,
0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa48, 0xa02,
0xa21, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa2b, 0xa3e, 0xa09, 0xa02, 0xa21, 0xa32, 0xa48,
0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa2b, 0xa3e,
0xa09, 0xa02, 0xa21, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa2b, 0xa3e, 0xa09,
0xa02, 0xa21, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e,
0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa40, 0xa0a, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa28, 0xa4b, 0xa30, 0xa2b, 0xa4c, 0xa15, 0x20, 0xa06, 0xa08,
0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa4b, 0xa30,
0xa2b, 0xa4c, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa4b, 0xa30,
0xa2b, 0xa4c, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b,
0xa30, 0xa28, 0xa3e, 0xa02, 0xa21, 0xa4b, 0x20, 0xa21, 0xa47, 0x20, 0xa28, 0xa4b,
0xa30, 0xa4b, 0xa28, 0xa39, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa30,
0xa28, 0xa3e, 0xa02, 0xa21, 0xa4b, 0x20, 0xa21, 0xa47, 0x20, 0xa28, 0xa4b, 0xa30,
0xa4b, 0xa28, 0xa39, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2b, 0xa30, 0xa28, 0xa3e, 0xa02, 0xa21, 0xa4b, 0x20, 0xa21,
0xa47, 0x20, 0xa28, 0xa4b, 0xa30, 0xa4b, 0xa28, 0xa39, 0xa3e, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa2e, 0xa30, 0xa3f, 0xa06, 0xa28, 0xa3e,
0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02,
0xa28, 0xa4c, 0xa35, 0xa4b, 0xa38, 0xa3f, 0xa2c, 0xa40, 0xa30, 0xa38, 0xa15, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa4c, 0xa35, 0xa4b, 0xa38, 0xa3f, 0xa2c, 0xa40,
0xa30, 0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa28, 0xa4c, 0xa35, 0xa4b, 0xa38, 0xa3f, 0xa2c, 0xa40, 0xa30, 0xa38,
0xa15, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa13, 0xa2e, 0xa38, 0xa15, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa13, 0xa2e, 0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa13, 0xa2e, 0xa38, 0xa15, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a,
0xa3e, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2a, 0xa3e, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa3e, 0xa15, 0xa3f, 0xa38,
0xa24, 0xa3e, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26,
0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa32, 0xa3e, 0xa09, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa3e, 0xa2a, 0xa42, 0xa06, 0x20, 0xa28, 0xa3f, 0xa0a,
0x20, 0xa17, 0xa3f, 0xa28, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa48,
0xa30, 0xa3e, 0xa17, 0xa35, 0xa47, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa48,
0xa30, 0xa3e, 0xa17, 0xa35, 0xa47, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa48, 0xa30, 0xa3e, 0xa17, 0xa35, 0xa47, 0x20,
0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2a, 0xa47, 0xa30, 0xa42, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a,
0xa47, 0xa30, 0xa42, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2a, 0xa47, 0xa30, 0xa42, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3f, 0xa32,
0xa3f, 0xa2a, 0xa40, 0xa28, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3f,
0xa32, 0xa3f, 0xa2a, 0xa40, 0xa28, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3f, 0xa32, 0xa3f, 0xa2a, 0xa40, 0xa28,
0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3f, 0xa28, 0xa3f, 0xa15, 0xa38, 0x20, 0xa06,
0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa48,
0xa02, 0xa1f, 0x20, 0xa2a, 0xa40, 0xa05, 0xa30, 0xa47, 0x20, 0xa05, 0xa24, 0xa47,
0x20, 0xa2e, 0xa3f, 0xa15, 0xa47, 0xa32, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa38, 0xa48, 0xa02, 0xa1f, 0x20, 0xa2a, 0xa40, 0xa05, 0xa30, 0xa47, 0x20, 0xa05,
0xa24, 0xa47, 0x20, 0xa2e, 0xa3f, 0xa15, 0xa47, 0xa32, 0xa28, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa48, 0xa02, 0xa1f,
0x20, 0xa2a, 0xa40, 0xa05, 0xa30, 0xa47, 0x20, 0xa05, 0xa24, 0xa47, 0x20, 0xa2e,
0xa3f, 0xa15, 0xa47, 0xa32, 0xa28, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38,
0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa3f, 0xa1f, 0xa15, 0xa48, 0xa30,
0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa4b, 0xa28, 0xa3e, 0xa2a, 0xa47,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa2f, 0xa4b, 0xa02, 0xa17, 0xa2f, 0xa3e,
0xa02, 0xa17, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa3f, 0xa1c, 0xa3c, 0xa32,
0xa4b, 0xa30, 0xa21, 0xa3e, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa15, 0xa3f, 0xa1c,
0xa3c, 0xa32, 0xa4b, 0xa30, 0xa21, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa15, 0xa3f, 0xa1c, 0xa3c, 0xa32, 0xa4b, 0xa30,
0xa21, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0x2d, 0xa30, 0xa41, 0xa71, 0xa24,
0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa30, 0xa3f, 0xa2f, 0xa42, 0xa28, 0xa40, 0xa05,
0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa30, 0xa4b, 0xa25, 0xa47, 0xa30, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa16, 0xa32, 0xa40, 0xa28, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa38, 0xa16, 0xa32, 0xa40, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa16, 0xa32, 0xa40, 0xa28,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa38, 0xa3e, 0xa2e, 0xa4b, 0xa06, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa38, 0xa3e, 0xa2e, 0xa4b, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa3e, 0xa2e, 0xa4b, 0xa06, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38,
0xa47, 0xa38, 0xa3c, 0xa32, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa3f,
0xa70, 0xa17, 0xa3e, 0xa2a, 0xa41, 0xa30, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa4b, 0xa32, 0xa4b, 0xa2e, 0xa28, 0x20,
0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa26, 0xa71, 0xa16, 0xa23, 0xa40, 0x20, 0xa1c, 0xa3e, 0xa30, 0xa1c, 0xa40, 0xa06,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa42, 0xa30, 0xa40, 0xa28, 0xa3e, 0xa2e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa3f, 0xa13, 0xa35, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa24, 0xa3e, 0xa39, 0xa40, 0xa24, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa24, 0xa48, 0xa2a, 0xa08, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24,
0xa48, 0xa2a, 0xa08, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa24, 0xa48, 0xa2a, 0xa08, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e,
0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa3e, 0xa1c, 0xa3f, 0xa15,
0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1f, 0xa4b,
0xa15, 0xa47, 0xa32, 0xa3e, 0xa09, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1f, 0xa4b,
0xa02, 0xa17, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1f, 0xa4b, 0xa02, 0xa17,
0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa1f, 0xa4b, 0xa02, 0xa17, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02,
0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa42, 0xa15, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa41, 0xa30, 0xa15, 0xa2e, 0xa47, 0xa28, 0xa3f,
0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa41, 0xa30,
0xa15, 0xa2e, 0xa47, 0xa28, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa41, 0xa30, 0xa15,
0xa2e, 0xa47, 0xa28, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1f,
0xa41, 0xa35, 0xa3e, 0xa32, 0xa42, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa30,
0xa42, 0xa17, 0xa35, 0xa47, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa30, 0xa42,
0xa17, 0xa35, 0xa47, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa09, 0xa30, 0xa42, 0xa17, 0xa35, 0xa47, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09,
0xa1c, 0xa3c, 0xa2c, 0xa47, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa09, 0xa1c, 0xa3c, 0xa2c, 0xa47, 0xa15, 0xa3f, 0xa38, 0xa24,
0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa09, 0xa1c, 0xa3c, 0xa2c, 0xa47, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa35, 0xa3e, 0xa28, 0xa42, 0xa06, 0xa1f, 0xa42, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa35, 0xa3e, 0xa28, 0xa42, 0xa06, 0xa1f, 0xa42, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa3e, 0xa28,
0xa42, 0xa06, 0xa1f, 0xa42, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20,
0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa48, 0xa28, 0xa47, 0xa1c,
0xa3c, 0xa42, 0xa0f, 0xa32, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa32,
0xa3e, 0xa26, 0xa40, 0xa35, 0xa4b, 0xa38, 0xa24, 0xa15, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa35, 0xa32, 0xa3e, 0xa26, 0xa40, 0xa35, 0xa4b, 0xa38, 0xa24, 0xa15, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa32,
0xa3e, 0xa26, 0xa40, 0xa35, 0xa4b, 0xa38, 0xa24, 0xa15, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35,
0xa4b, 0xa32, 0xa17, 0xa4b, 0xa17, 0xa4d, 0xa30, 0xa47, 0xa21, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa35, 0xa4b, 0xa32, 0xa17, 0xa4b, 0xa17, 0xa4d, 0xa30, 0xa47, 0xa21,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35,
0xa4b, 0xa32, 0xa17, 0xa4b, 0xa17, 0xa4d, 0xa30, 0xa47, 0xa21, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa35, 0xa4b, 0xa38, 0xa1f, 0xa4b, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35,
0xa47, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa35, 0xa3e, 0xa32, 0xa3f, 0xa38, 0x20, 0xa05, 0xa24, 0xa47, 0x20,
0xa2b, 0xa41, 0xa1f, 0xa42, 0xa28, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f,
0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15,
0xa41, 0xa24, 0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2f, 0xa15, 0xa47, 0xa24, 0xa30, 0xa3f, 0xa28, 0xa2c, 0xa30, 0xa17, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15, 0xa47, 0xa24, 0xa30, 0xa3f, 0xa28, 0xa2c, 0xa30,
0xa17, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2f, 0xa15, 0xa47, 0xa24, 0xa30, 0xa3f, 0xa28, 0xa2c, 0xa30, 0xa17, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2f, 0xa42, 0xa15, 0xa4b, 0xa28, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x67, 0x61, 0x6e,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75,
0x64, 0x61, 0x66, 0x72, 0x69, 0x63, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x45, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x45, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0xf1,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4d, 0x6f,
0x6e, 0x74, 0x61, 0xf1, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66,
0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65, 0x20,
0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65, 0x20,
0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65,
0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69,
0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x44, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c,
0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x79, 0xe1,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x79, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x79, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x63, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x43, 0x6f,
0x63, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75,
0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73,
0x63, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45,
0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x6d, 0xe1, 0x73, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69,
0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x46,
0x72, 0x61, 0x6e, 0x63, 0xe9, 0x73, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64,
0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x20, 0x79, 0x20, 0x61, 0x6e, 0x74, 0xe1,
0x72, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x63, 0x65, 0x61,
0x6e, 0x6f, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x49, 0x72, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4a, 0x61, 0x70, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61,
0x70, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70,
0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65,
0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45,
0x73, 0x70, 0xf3, 0x72, 0x61, 0x64, 0x61, 0x73, 0x20, 0x45, 0x63, 0x75,
0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x63, 0x69, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x69, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe1,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75,
0x65, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75,
0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61,
0x6e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x75,
0x65, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x65, 0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65,
0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f,
0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xf3, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72,
0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xf3, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65,
0x64, 0x72, 0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f,
0x68, 0x6e, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x69, 0x6f, 0x6e, 0x79, 0x61, 0x6e, 0x67, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x61, 0x79, 0x69, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20,
0x55, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x63, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x41, 0x63, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x41, 0x63, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c,
0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x69,
0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x69, 0x69, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x69, 0x69, 0x20, 0x4d, 0x65, 0x72, 0x69, 0x64,
0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x66,
0x72, 0x69, 0x63, 0x69, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69,
0x63, 0x69, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x69, 0x69,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72,
0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x75, 0x6c, 0x75, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x75, 0x6c, 0x75,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x75, 0x6c, 0x75,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x103,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20,
0x7a, 0x6f, 0x6e, 0x65, 0x69, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0xee, 0x6e, 0x20, 0x7a, 0x6f, 0x6e, 0x61,
0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x103, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0xee,
0x6e, 0x20, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x7a, 0x6f, 0x6e,
0x65, 0x69, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0xee, 0x6e, 0x20, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0xee, 0x6e, 0x20, 0x7a,
0x6f, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x70, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x71,
0x74, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x7a,
0x6f, 0x6e, 0x65, 0x69, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x4f, 0x72,
0x61, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x41, 0x71, 0x74,
0x6f, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69, 0x20, 0x41,
0x71, 0x74, 0x6f, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x61, 0x72, 0x61,
0x62, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x72, 0x61, 0x62, 0x103, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x72, 0x61, 0x62,
0x103, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x65, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65,
0x69, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0xee, 0x6e, 0x20, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0xee, 0x6e, 0x20,
0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x69, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x69, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x64, 0x6a, 0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x75, 0x6c,
0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64,
0x6a, 0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e,
0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x43, 0x61, 0x70, 0x75, 0x6c, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x61, 0x70, 0x75, 0x6c, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x61, 0x70, 0x75,
0x6c, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f,
0x72, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x43,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x43, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f,
0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x43, 0x75, 0x62, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x75,
0x62, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x61, 0x20, 0x43, 0x75, 0x62, 0x65, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x75, 0x6c, 0x75, 0x69,
0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x50, 0x61,
0x219, 0x74, 0x65, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x50, 0x61, 0x219, 0x74, 0x65, 0x6c,
0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x20, 0x50, 0x61, 0x219, 0x74, 0x65, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x75, 0x6c, 0x75, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20,
0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x69, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20,
0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x20, 0xee, 0x6e, 0x64, 0x65, 0x70,
0x103, 0x72, 0x74, 0x61, 0x74, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x73,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x73, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x65, 0x6c, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65,
0x6c, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x65, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x7a, 0x103,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x65, 0x72, 0x69,
0x74, 0x6f, 0x72, 0x69, 0x69, 0x6c, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x20, 0x219, 0x69, 0x20, 0x41, 0x6e, 0x74, 0x61,
0x72, 0x63, 0x74, 0x69, 0x63, 0x65, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63,
0x65, 0x7a, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f,
0x72, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x68, 0x69, 0x63, 0x68, 0x4f, 0x72,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x69, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x69, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x69, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x69, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x69, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x75,
0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x4f, 0x63, 0x65, 0x61,
0x6e, 0x75, 0x6c, 0x75, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x7a, 0x69, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a,
0x69, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x65, 0x69,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x4a,
0x61, 0x70, 0x6f, 0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x6f,
0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x69, 0x61, 0x21b, 0x6b, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x69, 0x61,
0x21b, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x69, 0x61, 0x21b, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x65, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61,
0x20, 0x43, 0x6f, 0x72, 0x65, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69,
0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0xe2, 0x72, 0x67, 0xe2,
0x7a, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x65, 0x20, 0x4c, 0x69,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x6c, 0x61,
0x79, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x72,
0x63, 0x68, 0x69, 0x7a, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x76, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x65, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x75,
0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x69,
0x69, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x4e, 0x6f, 0x69, 0x69, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x4e, 0x6f, 0x69, 0x69, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x4e, 0x6f, 0x69, 0x69, 0x20, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x4e, 0x6f, 0x69, 0x69, 0x20, 0x5a, 0x65, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x4e, 0x6f, 0x69, 0x69, 0x20,
0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x69, 0x75,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x69,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x65, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x69, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x75,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x61,
0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x75, 0x61, 0x20, 0x47, 0x75, 0x69,
0x6e, 0x65, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x72,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x219, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x219, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x219, 0x69, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x79,
0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x53,
0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x54, 0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x75, 0x75,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x75, 0x6c, 0x75, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x75, 0x6c, 0x75,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x75, 0x6c,
0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a,
0x75, 0x65, 0x6c, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x69, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20,
0x219, 0x69, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x21b, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x59, 0x75, 0x6b,
0x6f, 0x6e, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74,
0x61, 0x64, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x54, 0x65,
0x6d, 0x70, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x54,
0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x54, 0x65, 0x6d,
0x70, 0x20, 0x64, 0x61, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x6f, 0x67, 0x6e,
0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6d, 0x75, 0x6e,
0x74, 0x6f, 0x67, 0x6e, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61,
0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6d, 0x75, 0x6e,
0x74, 0x6f, 0x67, 0x6e, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61,
0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73,
0x74, 0x61, 0x64, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74,
0x61, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x54,
0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73,
0x74, 0x61, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x61, 0x6c, 0x20, 0x56, 0x65,
0x73, 0x74, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x61, 0x6c, 0x20, 0x56,
0x65, 0x73, 0x74, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73,
0x74, 0x61, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x64, 0x61, 0x6c, 0x20, 0x56, 0x65, 0x73, 0x74,
0x54, 0x65, 0x6d, 0x70, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69,
0x63, 0x68, 0x410, 0x43a, 0x440, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x410, 0x43a, 0x440, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a,
0x440, 0x438, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x430, 0x44f, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x412, 0x43e, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x42e, 0x436, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f,
0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43b, 0x44f, 0x441, 0x43a,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43b, 0x44f, 0x441,
0x43a, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410,
0x442, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43b, 0x43c, 0x430,
0x2d, 0x410, 0x442, 0x430, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43c, 0x430, 0x437,
0x43e, 0x43d, 0x43a, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x413, 0x43e, 0x440, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x20, 0x28, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x433, 0x43e, 0x440, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x28, 0x421, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x29, 0x41b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x433, 0x43e, 0x440,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x28, 0x421,
0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x29, 0x422, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422,
0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x438, 0x445, 0x43e, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x440, 0x435, 0x43c,
0x44f, 0x20, 0x43f, 0x43e, 0x20, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44e,
0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43f, 0x438,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43f, 0x438, 0x430,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x442, 0x43e, 0x431,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x442, 0x43e, 0x431,
0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x442, 0x43e, 0x431,
0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x421, 0x430, 0x443, 0x434, 0x43e, 0x432, 0x441, 0x43a, 0x430, 0x44f,
0x20, 0x410, 0x440, 0x430, 0x432, 0x438, 0x44f, 0x421, 0x430, 0x443, 0x434, 0x43e,
0x432, 0x441, 0x43a, 0x430, 0x44f, 0x20, 0x410, 0x440, 0x430, 0x432, 0x438, 0x44f,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x443, 0x434, 0x43e,
0x432, 0x441, 0x43a, 0x430, 0x44f, 0x20, 0x410, 0x440, 0x430, 0x432, 0x438, 0x44f,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430,
0x44f, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x440, 0x43c, 0x435,
0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x440,
0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x442, 0x43b,
0x430, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x426, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x437, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x437,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20,
0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430,
0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430,
0x439, 0x434, 0x436, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d,
0x432, 0x430, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e,
0x2d, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x437,
0x43e, 0x440, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434,
0x435, 0x448, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x411, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435,
0x440, 0x434, 0x435, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x430,
0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x427, 0x430,
0x442, 0x435, 0x43c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x427, 0x430,
0x442, 0x435, 0x43c, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x438,
0x442, 0x430, 0x439, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x438,
0x442, 0x430, 0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x43e, 0x2d, 0x432, 0x20, 0x420, 0x43e, 0x436,
0x434, 0x435, 0x441, 0x442, 0x432, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e,
0x432, 0x44b, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a,
0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x20,
0x41a, 0x443, 0x43a, 0x430, 0x2c, 0x20, 0x43f, 0x43e, 0x43b, 0x443, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x443,
0x431, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x443, 0x431,
0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x44b, 0x439,
0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x41e, 0x2d, 0x432, 0x20, 0x41f, 0x430,
0x441, 0x445, 0x438, 0x41e, 0x2d, 0x432, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41e, 0x2d, 0x432, 0x20, 0x41f,
0x430, 0x441, 0x445, 0x438, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x424, 0x43e, 0x43b,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d,
0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x43e, 0x43b,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d,
0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x424, 0x438, 0x434, 0x436, 0x438, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x424, 0x438, 0x434, 0x436, 0x438, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424,
0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x441, 0x43a, 0x430, 0x44f, 0x20, 0x413,
0x432, 0x438, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437,
0x441, 0x43a, 0x438, 0x435, 0x20, 0x42e, 0x436, 0x43d, 0x44b, 0x435, 0x20, 0x438,
0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x447, 0x435, 0x441,
0x43a, 0x438, 0x435, 0x20, 0x442, 0x435, 0x440, 0x440, 0x438, 0x442, 0x43e, 0x440,
0x438, 0x438, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x441,
0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x413, 0x440, 0x443, 0x437,
0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413, 0x440, 0x443,
0x437, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x43e, 0x2d, 0x432, 0x430, 0x20, 0x413, 0x438,
0x43b, 0x431, 0x435, 0x440, 0x442, 0x430, 0x421, 0x440, 0x435, 0x434, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x43f, 0x43e, 0x20, 0x413,
0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x443, 0x412, 0x43e, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x430, 0x44f, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x435, 0x440,
0x441, 0x438, 0x434, 0x441, 0x43a, 0x438, 0x439, 0x20, 0x437, 0x430, 0x43b, 0x438,
0x432, 0x413, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b,
0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x413, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b,
0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x413, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b,
0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413, 0x43e,
0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x425, 0x43e, 0x432, 0x434, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x418, 0x43d, 0x434, 0x438, 0x439, 0x441, 0x43a, 0x438, 0x439, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x430, 0x44f, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f,
0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x418, 0x43d,
0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x430, 0x44f, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438,
0x44f, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x437, 0x440, 0x430, 0x438,
0x43b, 0x44c, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432,
0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438,
0x439, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432,
0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438,
0x439, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x435, 0x442, 0x440,
0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430,
0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x44b, 0x439, 0x20, 0x41a, 0x430, 0x437, 0x430,
0x445, 0x441, 0x442, 0x430, 0x43d, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x44b,
0x439, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x41a,
0x43e, 0x440, 0x435, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a,
0x43e, 0x440, 0x435, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x44f, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a,
0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a,
0x438, 0x440, 0x433, 0x438, 0x437, 0x438, 0x44f, 0x43e, 0x2d, 0x432, 0x430, 0x20,
0x41b, 0x430, 0x439, 0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41b, 0x43e, 0x440, 0x434, 0x2d,
0x425, 0x430, 0x443, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c,
0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x44b, 0x41c, 0x430, 0x440, 0x43a, 0x438,
0x437, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41c, 0x430,
0x440, 0x448, 0x430, 0x43b, 0x43b, 0x43e, 0x432, 0x44b, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x430, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x43a, 0x438, 0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43c, 0x435, 0x43a, 0x441, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x422, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x438, 0x445,
0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43c,
0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430,
0x442, 0x43e, 0x440, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x41a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x432, 0x430,
0x44f, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x41d, 0x43e,
0x432, 0x430, 0x44f, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x44f,
0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d,
0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443,
0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x440, 0x444,
0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x435, 0x440, 0x43d, 0x430,
0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d,
0x44c, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x44c, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x44b, 0x435, 0x20, 0x41c, 0x430, 0x440, 0x438, 0x430, 0x43d, 0x441, 0x43a, 0x438,
0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430, 0x43a,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430, 0x43f, 0x443, 0x430,
0x20, 0x2013, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x413, 0x432, 0x438,
0x43d, 0x435, 0x44f, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f,
0x438, 0x43d, 0x44b, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x438,
0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x44b, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x43e, 0x2d, 0x432,
0x430, 0x20, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x421, 0x435, 0x43d, 0x2d,
0x41f, 0x44c, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b,
0x43e, 0x43d, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x438,
0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20,
0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a,
0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x2a, 0x41a, 0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434,
0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x2a, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438,
0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x412, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x432, 0x20, 0x421,
0x430, 0x43c, 0x430, 0x440, 0x435, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x43c, 0x430,
0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x44c, 0x441, 0x43a, 0x438, 0x435,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x421, 0x43e, 0x43b, 0x43e,
0x43c, 0x43e, 0x43d, 0x43e, 0x432, 0x44b, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x42e, 0x436, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x435, 0x43e, 0x440,
0x433, 0x438, 0x44f, 0x422, 0x430, 0x439, 0x432, 0x430, 0x43d, 0x44c, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x430, 0x439, 0x432, 0x430, 0x43d, 0x44c,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430,
0x439, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x440, 0x443, 0x433,
0x432, 0x430, 0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x437,
0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x43e, 0x43b, 0x43b, 0x438,
0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x42f, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x42f,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x415, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x433, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x410, 0x440, 0x430, 0x430, 0x431, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x410, 0x440, 0x430, 0x430, 0x431, 0x20, 0x441, 0x4af, 0x440, 0x4af,
0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x430, 0x430, 0x431, 0x20,
0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x42d, 0x440, 0x43c, 0x44d, 0x44d, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x42d,
0x440, 0x43c, 0x44d, 0x44d, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x42d, 0x440, 0x43c, 0x44d, 0x44d, 0x43d, 0x20, 0x441,
0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a,
0x438, 0x438, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x438, 0x438, 0x43d, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x4af, 0x440,
0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x438, 0x438, 0x43d, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x430,
0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x43b,
0x438, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x43b, 0x438, 0x43d, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x4af, 0x440, 0x4af,
0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x43b, 0x438, 0x43d, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x430, 0x439,
0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x495,
0x430, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x495, 0x430, 0x430, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x4af, 0x440,
0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x495, 0x430, 0x430,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441,
0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a,
0x44b, 0x442, 0x430, 0x439, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x44b, 0x442,
0x430, 0x439, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5,
0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x443, 0x440, 0x443, 0x441,
0x443, 0x443, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x443, 0x440, 0x443,
0x441, 0x443, 0x443, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x41a, 0x443, 0x440, 0x443, 0x441, 0x443, 0x443, 0x43d, 0x20,
0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x418, 0x438, 0x43d, 0x434, 0x438, 0x439, 0x44d, 0x20, 0x441, 0x4af, 0x440, 0x4af,
0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x440, 0x430, 0x430, 0x43d, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x440, 0x430, 0x430, 0x43d, 0x20, 0x441, 0x4af,
0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x42b, 0x440, 0x430, 0x430,
0x43d, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x414, 0x44c, 0x43e, 0x43f, 0x43f, 0x443, 0x43e, 0x43d, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x414, 0x44c, 0x43e, 0x43f, 0x43f, 0x443, 0x43e, 0x43d, 0x20,
0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414, 0x44c,
0x43e, 0x43f, 0x43f, 0x443, 0x43e, 0x43d, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5,
0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x43b, 0x438, 0x43d, 0x20,
0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x430, 0x43d, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x495, 0x430, 0x430, 0x20, 0x41a, 0x430, 0x437,
0x430, 0x445, 0x441, 0x442, 0x430, 0x430, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x41a, 0x44d, 0x440, 0x438, 0x44d, 0x439, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a,
0x44d, 0x440, 0x438, 0x44d, 0x439, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x44d, 0x440, 0x438, 0x44d, 0x439, 0x20, 0x441,
0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a,
0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x430, 0x439, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440,
0x441, 0x43a, 0x430, 0x439, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x430, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x44b, 0x440, 0x433, 0x44b, 0x441, 0x442, 0x430,
0x430, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x430, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x430, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x430, 0x43d, 0x20,
0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x430, 0x442, 0x430, 0x440, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x430,
0x442, 0x430, 0x440, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x430, 0x442, 0x430,
0x440, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x41c, 0x43e, 0x441, 0x43a, 0x443, 0x431, 0x430, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x41c, 0x43e, 0x441, 0x43a, 0x443, 0x431, 0x430, 0x20, 0x441, 0x4af,
0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41c, 0x43e, 0x441, 0x43a,
0x443, 0x431, 0x430, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x421, 0x430, 0x4a5, 0x430, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421, 0x430,
0x4a5, 0x430, 0x20, 0x421, 0x44d, 0x439, 0x43b, 0x44d, 0x43d, 0x434, 0x20, 0x441,
0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421, 0x430, 0x4a5,
0x430, 0x20, 0x421, 0x44d, 0x439, 0x43b, 0x44d, 0x43d, 0x434, 0x20, 0x441, 0x430,
0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41d, 0x43e,
0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x430, 0x439, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5,
0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41e, 0x43c, 0x441, 0x43a, 0x430, 0x439,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41e, 0x43c, 0x441, 0x43a, 0x430, 0x439, 0x20,
0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41e, 0x43c,
0x441, 0x43a, 0x430, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x430, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x430, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x430, 0x43d,
0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x438, 0x43d, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x44b, 0x44b, 0x43d, 0x20, 0x441,
0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421, 0x430, 0x445,
0x430, 0x43b, 0x44b, 0x44b, 0x43d, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5,
0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x411, 0x44b,
0x43b, 0x430, 0x434, 0x44c, 0x44b, 0x431, 0x430, 0x441, 0x442, 0x44b, 0x430, 0x43a,
0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x411,
0x44b, 0x43b, 0x430, 0x434, 0x44c, 0x44b, 0x431, 0x430, 0x441, 0x442, 0x44b, 0x430,
0x43a, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x430,
0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414, 0x44c,
0x43e, 0x43a, 0x443, 0x443, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x414, 0x44c, 0x43e, 0x43a, 0x443, 0x443, 0x441, 0x43a, 0x430, 0x439, 0x20,
0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414, 0x44c,
0x43e, 0x43a, 0x443, 0x443, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x441, 0x430, 0x439,
0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x415, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x443, 0x440, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x443,
0x440, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930,
0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x938, 0x92e,
0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930,
0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x906, 0x926,
0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x92e,
0x927, 0x94d, 0x92f, 0x20, 0x905, 0x92f, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f,
0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f,
0x915, 0x93e, 0x3a, 0x20, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20, 0x938, 0x92e,
0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930,
0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20, 0x906,
0x926, 0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20,
0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20, 0x905, 0x92f, 0x93e, 0x92e, 0x20, 0x938,
0x92e, 0x92f, 0x3a, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x936, 0x948, 0x932, 0x20, 0x938, 0x92e,
0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930,
0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x936, 0x948, 0x932, 0x20, 0x906, 0x926, 0x930,
0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930,
0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x936, 0x948,
0x932, 0x20, 0x905, 0x92f, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e,
0x3a, 0x20, 0x938, 0x928, 0x94d, 0x927, 0x93f, 0x92a, 0x94d, 0x930, 0x93f, 0x92f,
0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905,
0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x938, 0x928, 0x94d, 0x927,
0x93f, 0x92a, 0x94d, 0x930, 0x93f, 0x92f, 0x20, 0x906, 0x926, 0x930, 0x94d, 0x936,
0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905,
0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x938, 0x928, 0x94d, 0x927,
0x93f, 0x92a, 0x94d, 0x930, 0x93f, 0x92f, 0x20, 0x905, 0x92f, 0x93e, 0x92e, 0x20,
0x938, 0x92e, 0x92f, 0x903, 0x905, 0x91f, 0x932, 0x93e, 0x923, 0x94d, 0x91f, 0x93f,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x905, 0x91f, 0x932, 0x93e, 0x923, 0x94d,
0x91f, 0x93f, 0x915, 0x20, 0x906, 0x926, 0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e,
0x92f, 0x903, 0x905, 0x91f, 0x932, 0x93e, 0x923, 0x94d, 0x91f, 0x93f, 0x915, 0x20,
0x905, 0x92f, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92e, 0x927, 0x94d,
0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x938, 0x92e,
0x92f, 0x903, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x940, 0x92f, 0x20, 0x906, 0x926, 0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f,
0x903, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x938, 0x92e,
0x92f, 0x903, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x94c, 0x930, 0x94d,
0x935, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x906, 0x926,
0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x94c, 0x930, 0x94d,
0x935, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x93e,
0x936, 0x94d, 0x91a, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x93e, 0x936, 0x94d,
0x91a, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x92f, 0x20, 0x906, 0x926, 0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903,
0x92a, 0x93e, 0x936, 0x94d, 0x91a, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92f, 0x942,
0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x935,
0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x1c5f,
0x1c79, 0x1c5e, 0x1c5f, 0x1c65, 0x1c60, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5f,
0x1c79, 0x1c5e, 0x1c5f, 0x1c65, 0x1c60, 0x1c5f, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71,
0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5f, 0x1c79, 0x1c5e,
0x1c5f, 0x1c65, 0x1c60, 0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c5f, 0x1c62, 0x1c5f, 0x1c61, 0x1c5a, 0x1c71, 0x20, 0x1c5a, 0x1c60,
0x1c74, 0x1c5a, 0x1c5f, 0x1c62, 0x1c5f, 0x1c61, 0x1c5a, 0x1c71, 0x20, 0x1c6e, 0x1c74, 0x1c6e,
0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c5f, 0x1c62,
0x1c5f, 0x1c61, 0x1c5a, 0x1c71, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69, 0x1c5d, 0x20, 0x1c5a,
0x1c60, 0x1c74, 0x1c5a, 0x1c5b, 0x1c5f, 0x1c71, 0x1c5f, 0x1c5e, 0x1c5f, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c5b, 0x1c5f, 0x1c71, 0x1c5f, 0x1c5e, 0x1c5f, 0x20, 0x1c6e, 0x1c65, 0x1c74,
0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5b,
0x1c5f, 0x1c71, 0x1c5f, 0x1c5e, 0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c6e, 0x1c65,
0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a,
0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c,
0x20, 0x1c75, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c5f, 0x1c69, 0x1c71, 0x1c74, 0x1c6e,
0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c5f, 0x1c69, 0x1c71, 0x1c74, 0x1c6e,
0x1c71, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c5f, 0x1c69, 0x1c71, 0x1c74, 0x1c6e, 0x1c71, 0x20,
0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c6e,
0x1c65, 0x1c64, 0x1c6f, 0x1c77, 0x1c64, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f,
0x1c6e, 0x1c65, 0x1c64, 0x1c6f, 0x1c77, 0x1c64, 0x1c60, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e,
0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c6e,
0x1c65, 0x1c64, 0x1c6f, 0x1c77, 0x1c64, 0x1c60, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e, 0x1c71, 0x1c74,
0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c5f, 0x1c79, 0x1c68, 0x1c61,
0x1c6e, 0x1c71, 0x1c74, 0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c6e, 0x1c74, 0x1c6e, 0x1c71, 0x1c70,
0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c5f, 0x1c79, 0x1c68, 0x1c61,
0x1c6e, 0x1c71, 0x1c74, 0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69, 0x1c5d,
0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5f, 0x1c79,
0x1c68, 0x1c61, 0x1c6e, 0x1c71, 0x1c74, 0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c74,
0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e, 0x1c71,
0x1c74, 0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f,
0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20,
0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e, 0x1c71, 0x1c74, 0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c65,
0x1c64, 0x1c5b, 0x1c69, 0x1c5d, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c6e, 0x1c74, 0x1c5e,
0x1c5f, 0x1c71, 0x1c74, 0x1c64, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c74,
0x1c5e, 0x1c5f, 0x1c71, 0x1c74, 0x1c64, 0x1c60, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71,
0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c74, 0x1c5e,
0x1c5f, 0x1c71, 0x1c74, 0x1c64, 0x1c60, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c60, 0x1c6d, 0x1c69, 0x1c75, 0x1c5f, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c60, 0x1c6d, 0x1c69, 0x1c75, 0x1c5f, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a,
0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c60, 0x1c6d, 0x1c69, 0x1c75, 0x1c5f, 0x20,
0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c6e,
0x1c71, 0x1c74, 0x1c68, 0x1c5f, 0x1c5e, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d,
0x1c5f, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c6e, 0x1c71, 0x1c74, 0x1c68,
0x1c5f, 0x1c5e, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20,
0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c65, 0x1c6e, 0x1c71, 0x1c74, 0x1c68, 0x1c5f, 0x1c5e, 0x20, 0x1c69, 0x1c68,
0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c68,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20,
0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f,
0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f,
0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68,
0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c65,
0x1c5f, 0x1c62, 0x1c5f, 0x1c68, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c63, 0x1c6e, 0x1c65,
0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f,
0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c63, 0x1c6e, 0x1c65, 0x1c74, 0x1c5f, 0x1c68,
0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c6e,
0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x1c63, 0x1c6e, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a,
0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c68, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5c, 0x1c68, 0x1c64, 0x1c71, 0x1c63, 0x1c64, 0x1c6a, 0x20,
0x1c62, 0x1c64, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f,
0x1c5d, 0x20, 0x1c5c, 0x1c7d, 0x1c68, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c5d, 0x20, 0x1c5c, 0x1c7d,
0x1c68, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a,
0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c5d, 0x20,
0x1c5c, 0x1c7d, 0x1c68, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c65, 0x1c64,
0x1c5b, 0x1c69, 0x1c5d, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e,
0x20, 0x1c5c, 0x1c7d, 0x1c68, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5c, 0x1c7d, 0x1c68, 0x1c64,
0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5c, 0x1c7d, 0x1c68,
0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69, 0x1c5d,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c66, 0x1c5f, 0x1c63, 0x1c5f, 0x1c6d, 0x1c64, 0x1c7c,
0x1c5f, 0x1c5e, 0x1c6e, 0x1c63, 0x1c74, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c66, 0x1c5f, 0x1c63, 0x1c5f, 0x1c6d, 0x1c64, 0x1c7c, 0x1c5f, 0x1c5e, 0x1c6e,
0x1c63, 0x1c74, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c66, 0x1c5f, 0x1c63, 0x1c5f, 0x1c6d, 0x1c64, 0x1c7c,
0x1c5f, 0x1c5e, 0x1c6e, 0x1c63, 0x1c74, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c65, 0x1c64,
0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c6e, 0x1c60, 0x1c65,
0x1c64, 0x1c60, 0x1c5f, 0x1c71, 0x20, 0x1c6f, 0x1c68, 0x1c5a, 0x1c65, 0x1c5f, 0x1c71, 0x1c5b,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c6e, 0x1c60, 0x1c65, 0x1c64, 0x1c60, 0x1c5f,
0x1c71, 0x20, 0x1c6f, 0x1c68, 0x1c5a, 0x1c65, 0x1c5f, 0x1c71, 0x1c5b, 0x20, 0x1c62, 0x1c5f,
0x1c71, 0x1c5a, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c6e, 0x1c60, 0x1c65,
0x1c64, 0x1c60, 0x1c5f, 0x1c71, 0x20, 0x1c6f, 0x1c68, 0x1c5a, 0x1c65, 0x1c5f, 0x1c71, 0x1c5b,
0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c71,
0x1c64, 0x1c6d, 0x1c69, 0x1c6f, 0x1c77, 0x1c5f, 0x1c69, 0x1c71, 0x1c70, 0x1c5e, 0x1c6e, 0x1c78,
0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c71, 0x1c64, 0x1c6d, 0x1c69, 0x1c6f, 0x1c77,
0x1c5f, 0x1c69, 0x1c71, 0x1c70, 0x1c5e, 0x1c6e, 0x1c78, 0x1c70, 0x20, 0x1c62, 0x1c5f, 0x1c71,
0x1c5a, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c71, 0x1c64, 0x1c6d, 0x1c69, 0x1c6f,
0x1c77, 0x1c5f, 0x1c69, 0x1c71, 0x1c70, 0x1c5e, 0x1c6e, 0x1c78, 0x1c70, 0x20, 0x1c65, 0x1c64,
0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c65, 0x20, 0x1c74,
0x1c64, 0x20, 0x1c6f, 0x1c64, 0x1c6d, 0x1c6e, 0x1c68, 0x1c6e, 0x20, 0x1c5f, 0x1c68, 0x20,
0x1c62, 0x1c64, 0x1c60, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a,
0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c6f, 0x1c64, 0x1c6d, 0x1c6e, 0x1c68, 0x1c6e,
0x20, 0x1c5f, 0x1c68, 0x20, 0x1c62, 0x1c64, 0x1c60, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c71, 0x20,
0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c65,
0x20, 0x1c74, 0x1c64, 0x20, 0x1c6f, 0x1c64, 0x1c6d, 0x1c6e, 0x1c68, 0x1c6e, 0x20, 0x1c5f,
0x1c68, 0x20, 0x1c62, 0x1c64, 0x1c60, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c71, 0x20, 0x1c65, 0x1c64,
0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6d, 0x1c69, 0x1c60, 0x1c5a,
0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x41, 0x63, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x63,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x41, 0x63, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x66, 0x67, 0x68, 0xe0, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6d, 0x65, 0x72, 0x69,
0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f,
0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x74,
0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x41, 0x6d, 0x61, 0x74, 0x7a, 0xf2, 0x6e, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x6d, 0x61, 0x74,
0x7a, 0xf2, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x6d, 0x61,
0x74, 0x7a, 0xf2, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x74, 0x7a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55,
0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55,
0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x65, 0x73, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x73, 0x6f, 0x73,
0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x73,
0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x73, 0x6f, 0x73, 0x20, 0x55, 0x53,
0x41, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x73, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f,
0x73, 0x6f, 0x73, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x61, 0x74, 0x7a, 0xec, 0x66, 0x69,
0x63, 0x75, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x50, 0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20,
0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x61, 0x74, 0x7a,
0xec, 0x66, 0x69, 0x63, 0x75, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20,
0xe0, 0x72, 0x61, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0xe0, 0x72, 0x61, 0x62, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0xe0,
0x72, 0x61, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x74,
0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41,
0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x41, 0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019,
0x41, 0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x63,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x74, 0x6c, 0xe0,
0x6e, 0x74, 0x69, 0x63, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x74, 0x6c,
0xe0, 0x6e, 0x74, 0x69, 0x63, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61,
0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72,
0x75, 0x2d, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72,
0x75, 0x2d, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d, 0x6f,
0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x74,
0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x67, 0x69, 0xe0, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x67, 0x69, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x67, 0x69, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x72, 0x61,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x72, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x72, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x42, 0x68, 0x75,
0x74, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x42, 0x6f, 0x6c, 0xec, 0x76, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xec, 0x6c, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xec, 0x6c, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xec, 0x6c, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x42, 0x72, 0x75, 0x6e,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x43, 0x61, 0x62, 0x75, 0x20, 0x42, 0x69, 0x72, 0x64, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x43, 0x61, 0x62, 0x75, 0x20, 0x42,
0x69, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x43, 0x61, 0x62,
0x75, 0x20, 0x42, 0x69, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x7a, 0x69, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x7a, 0x69, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x75, 0x20, 0x54, 0x7a, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x54, 0x7a, 0x69, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x54, 0x7a, 0x69, 0x6e,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x54, 0x7a, 0x69, 0x6e, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x61, 0x74, 0x69,
0x76, 0x69, 0x64, 0x61, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20,
0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d,
0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf2,
0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc,
0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75,
0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x45, 0x73, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x63,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x7a, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x65, 0x73, 0x74, 0x72, 0x65, 0x6d,
0x75, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x65, 0x20, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x75, 0x20, 0x28, 0x4b, 0x61, 0x6c, 0x69, 0x6e,
0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x74,
0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c,
0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c,
0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6a, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x75, 0x69, 0x61,
0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x74, 0x7a, 0x65, 0x73, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x54, 0x65,
0x72, 0x72, 0x61, 0x73, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x73, 0x20, 0x65, 0x20, 0x61, 0x6e, 0x74, 0xe0, 0x72, 0x74, 0x69,
0x63, 0x61, 0x73, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x74, 0x7a, 0x65, 0x73,
0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73,
0x20, 0x47, 0x61, 0x6c, 0xe0, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x65,
0xf2, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61,
0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64,
0x69, 0x61, 0x6e, 0x75, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0xe0, 0x6d,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x47, 0x75, 0x6c, 0x66,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47,
0x75, 0x79, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x73, 0x20, 0xec, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20,
0xec, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x61, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0xec, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc, 0x6e, 0x64, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x4f, 0x74, 0x7a, 0xe8,
0x61, 0x6e, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x75, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x49, 0x6e, 0x64, 0x6f,
0x74, 0x7a, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73, 0x69, 0x61, 0x20,
0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8,
0x73, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0xe8, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x49, 0x72, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x49, 0x72, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x49,
0x72, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x47,
0x69, 0x61, 0x70, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x47, 0x69, 0x61, 0x70, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x47, 0x69, 0x61, 0x70, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4b, 0x61, 0x7a,
0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0xe0, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4b,
0x61, 0x7a, 0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x74,
0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x63,
0x6f, 0x72, 0x65, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x75, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x68, 0xec, 0x7a, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c,
0x61, 0x6e, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x20, 0x4c, 0xec, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63,
0x61, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0xe8, 0x73, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x73,
0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73,
0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x75, 0x72, 0xec, 0x74, 0x7a, 0x69, 0x75,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x75, 0x72, 0xec, 0x74, 0x7a, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x73, 0x20, 0x4d, 0x61, 0x75, 0x72, 0xec, 0x74, 0x7a, 0x69, 0x75, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50,
0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20, 0x28, 0x4d, 0xe8,
0x73, 0x73, 0x69, 0x63, 0x75, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x50, 0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20,
0x28, 0x4d, 0xe8, 0x73, 0x73, 0x69, 0x63, 0x75, 0x29, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x50, 0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20,
0x28, 0x4d, 0xe8, 0x73, 0x73, 0x69, 0x63, 0x75, 0x29, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe0, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe0, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe0, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0xe0, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61,
0x75, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69,
0x61, 0x20, 0x4e, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x20, 0x4e,
0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0xf2, 0x6e, 0x69, 0x61, 0x20, 0x4e, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x20, 0x4e, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x20, 0x4e, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x61, 0x6e, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x65, 0x74, 0x65, 0x6e,
0x74, 0x72, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0xe0, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x50, 0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x50, 0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x50, 0xe0, 0x70, 0x75, 0x61,
0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x4e, 0x6f, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x65, 0x72, 0xf9,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x65, 0x72, 0xf9,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73,
0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x20, 0x46, 0x65, 0x6e, 0xec, 0x74, 0x7a, 0x69, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73,
0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x53, 0x75, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x79, 0x6f,
0x77, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x61, 0x67,
0xec, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x43, 0x68, 0x75, 0x75, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x55, 0x7a,
0x62, 0xe8, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x55, 0x7a, 0x62, 0xe8, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x55, 0x7a, 0x62, 0xe8, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x56, 0x65, 0x6e,
0x65, 0x74, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc,
0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20,
0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x59, 0x75,
0x6b, 0x6f, 0x6e, 0x410, 0x432, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x408, 0x443, 0x436, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x430,
0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x459, 0x430, 0x441, 0x43a, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x459, 0x430, 0x441, 0x43a, 0x430,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x43b,
0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f,
0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43f,
0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438,
0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458,
0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438,
0x458, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435,
0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458,
0x45f, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x411, 0x443, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411,
0x43e, 0x43b, 0x438, 0x432, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x458, 0x20,
0x414, 0x430, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x443, 0x43c, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441,
0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441,
0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441,
0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430,
0x43c, 0x43e, 0x440, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430,
0x442, 0x430, 0x43c, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442,
0x430, 0x43c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430,
0x43c, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x427, 0x438, 0x43b, 0x435, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x438,
0x43b, 0x435, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41a, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41a, 0x438, 0x43d, 0x435, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41a, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x43e, 0x436, 0x438, 0x45b, 0x43d,
0x43e, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x28, 0x41a, 0x435, 0x43b,
0x438, 0x43d, 0x433, 0x29, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438,
0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x43a,
0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x2c, 0x20, 0x43f, 0x43e, 0x43b, 0x443, 0x2d, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x414, 0x435, 0x458,
0x432, 0x438, 0x441, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x414, 0x438, 0x43c,
0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20,
0x442, 0x438, 0x43c, 0x43e, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423,
0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440, 0x448,
0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e,
0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a,
0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x435, 0x432, 0x440,
0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x440,
0x435, 0x434, 0x45a, 0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f,
0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x434, 0x430, 0x459,
0x435, 0x433, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x43a, 0x430, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435,
0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x424, 0x438, 0x45f, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438,
0x45f, 0x438, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x441, 0x43a, 0x430,
0x20, 0x413, 0x432, 0x430, 0x458, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x458, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438,
0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437,
0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x43f, 0x43e, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x438, 0x447, 0x443, 0x418,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43b, 0x438, 0x432, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458,
0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458, 0x441,
0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458, 0x441, 0x43a, 0x43e,
0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d,
0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425,
0x43e, 0x432, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432,
0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438,
0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x418, 0x440, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440,
0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430,
0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437,
0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43f, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43f,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x430,
0x43f, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f,
0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x2d, 0x43a, 0x430, 0x43c,
0x447, 0x430, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e,
0x2d, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x43e, 0x2d, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440,
0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x43e, 0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41a, 0x43e, 0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x448, 0x440,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458,
0x430, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x20, 0x41b, 0x430, 0x458, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e,
0x432, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430,
0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43b, 0x435, 0x437, 0x438, 0x458, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432,
0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440, 0x43a, 0x438,
0x437, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440, 0x448, 0x430,
0x43b, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438,
0x458, 0x443, 0x441, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443,
0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41c, 0x43e, 0x441, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438, 0x20, 0x41f, 0x430, 0x446, 0x438,
0x444, 0x438, 0x43a, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438, 0x20,
0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438, 0x20, 0x41f, 0x430,
0x446, 0x438, 0x444, 0x438, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411,
0x430, 0x442, 0x43e, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x43b,
0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x438, 0x458, 0x430,
0x43d, 0x43c, 0x430, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x430,
0x443, 0x440, 0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x435, 0x43f,
0x430, 0x43b, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430,
0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430,
0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434,
0x43e, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b,
0x435, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x40a, 0x443, 0x444,
0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x438, 0x443,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x43a, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x432, 0x43e, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432,
0x43e, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434,
0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434,
0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434,
0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441,
0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43b,
0x430, 0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43f, 0x443,
0x430, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435,
0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x458, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430,
0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440,
0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438,
0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424,
0x435, 0x43d, 0x438, 0x43a, 0x441, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458,
0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458,
0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x458, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430, 0x43d, 0x448, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x420, 0x435, 0x438, 0x43d, 0x438,
0x43e, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x420, 0x43e, 0x442, 0x435,
0x440, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445,
0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435,
0x458, 0x448, 0x435, 0x43b, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x441, 0x43a, 0x430, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x408, 0x443, 0x436, 0x43d, 0x430, 0x20, 0x40f, 0x43e, 0x440, 0x45f, 0x438, 0x458,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x443, 0x440, 0x438, 0x43d,
0x430, 0x43c, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x428, 0x43e, 0x432, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x445, 0x438, 0x442, 0x438,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x422, 0x430, 0x45f, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x443,
0x443, 0x43a, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a,
0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a,
0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x432,
0x430, 0x43b, 0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x458, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x440,
0x443, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423,
0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423,
0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x412, 0x435, 0x43d, 0x435, 0x446, 0x443, 0x435, 0x43b, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434,
0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x435, 0x458, 0x43a, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x430,
0x43b, 0x438, 0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408,
0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x433, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x430,
0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x459, 0x430, 0x441,
0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x459,
0x430, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430,
0x437, 0x43e, 0x43d, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435,
0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435,
0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458,
0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447,
0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x43b,
0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430, 0x446,
0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438,
0x458, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408,
0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458,
0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x442, 0x43b,
0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x437,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f,
0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437,
0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410,
0x437, 0x43e, 0x440, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x410, 0x437, 0x43e, 0x440, 0x438, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x458, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x458,
0x20, 0x414, 0x430, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x443, 0x43c, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e,
0x440, 0x442, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435,
0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x435,
0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430,
0x43c, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a,
0x438, 0x43d, 0x435, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41a, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x43e, 0x436, 0x438,
0x45b, 0x43d, 0x43e, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e,
0x432, 0x430, 0x20, 0x28, 0x41a, 0x438, 0x43b, 0x438, 0x43d, 0x433, 0x29, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c,
0x431, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432,
0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x2c, 0x20, 0x43f, 0x43e, 0x43b, 0x443, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431,
0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019,
0x418, 0x440, 0x432, 0x438, 0x43b, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x422, 0x438, 0x43c,
0x43e, 0x440, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x441,
0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440,
0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421,
0x440, 0x435, 0x434, 0x45a, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434,
0x45a, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f,
0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x43e,
0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f,
0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f,
0x438, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x458, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438,
0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x440,
0x443, 0x437, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x438, 0x43b, 0x431,
0x435, 0x440, 0x442, 0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435,
0x434, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x20,
0x43f, 0x43e, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x438, 0x447, 0x443, 0x418, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x425, 0x430, 0x432, 0x430, 0x458, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435,
0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20,
0x41a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x408, 0x430, 0x43f, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x430,
0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430,
0x440, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a,
0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x458, 0x430, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b, 0x438, 0x43d, 0x438,
0x458, 0x441, 0x43a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20,
0x425, 0x43e, 0x432, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430,
0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c,
0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438,
0x458, 0x443, 0x441, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447,
0x43a, 0x438, 0x20, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x43a, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447,
0x43a, 0x438, 0x20, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x43a, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20,
0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441,
0x43a, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x458, 0x430, 0x43d,
0x43c, 0x430, 0x440, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438,
0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e,
0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458,
0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x40a, 0x443, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x438,
0x458, 0x443, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e,
0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b,
0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442,
0x440, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442, 0x440, 0x432,
0x43e, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20,
0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434,
0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430,
0x458, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20, 0x438,
0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440,
0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x458, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430, 0x43d, 0x448,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x420, 0x435,
0x443, 0x43d, 0x438, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421,
0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427,
0x443, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x443,
0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435,
0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x437,
0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443,
0x430, 0x442, 0x443, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442, 0x440,
0x432, 0x43e, 0x20, 0x412, 0x435, 0x458, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x412, 0x430,
0x43b, 0x438, 0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408,
0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41, 0x6b,
0x72, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x72, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x72, 0x65, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x76,
0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f,
0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x2d, 0x61, 0x66,
0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61,
0x73, 0x6b, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d,
0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x6e, 0x61, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e,
0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65,
0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x6b, 0x76, 0x61, 0x74, 0x61, 0x75, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x6b, 0x76, 0x61, 0x74, 0x61, 0x75, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x6b, 0x76, 0x61, 0x74, 0x61, 0x75, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41,
0x6b, 0x75, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x41, 0x6b, 0x75, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x41, 0x6b, 0x75, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x72, 0x61, 0x62,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4a, 0x65, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x6a, 0x61, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x7a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64,
0x17e, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x161, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a,
0x69, 0x6c, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x61, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c,
0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x75, 0x6e,
0x65, 0x6a, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x75, 0x6d,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65,
0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65,
0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73,
0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x10c, 0x69, 0x6c, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x10c, 0x69, 0x6c, 0x65, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x10c, 0x69, 0x6c, 0x65, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69,
0x6e, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69,
0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x6f, 0x17e, 0x69, 0x107,
0x6e, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76,
0x61, 0x20, 0x28, 0x4b, 0x69, 0x6c, 0x69, 0x6e, 0x67, 0x29, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x75,
0x6b, 0x6f, 0x76, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x6b, 0x6f, 0x76,
0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x70, 0x6f,
0x6c, 0x75, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62, 0x61, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x44, 0x65, 0x6a, 0x76, 0x69, 0x73, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x44, 0x69, 0x6d, 0x6f, 0x6e, 0x20, 0x64, 0x2019,
0x49, 0x72, 0x76, 0x69, 0x6c, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73,
0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76,
0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b,
0x72, 0x161, 0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72,
0x161, 0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e,
0x6a, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x65, 0x6d, 0x65, 0x20,
0x64, 0x61, 0x6c, 0x6a, 0x65, 0x67, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x6b,
0x61, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e,
0x69, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75,
0x73, 0x6b, 0x61, 0x20, 0x47, 0x76, 0x61, 0x6a, 0x61, 0x6e, 0x61, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63,
0x75, 0x73, 0x6b, 0x6f, 0x20, 0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x20, 0x69,
0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69,
0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72,
0x75, 0x7a, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x69, 0x10d,
0x75, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x47, 0x72, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69,
0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69,
0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x69, 0x20, 0x47, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x69, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x69, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x5a, 0x61, 0x6c, 0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a,
0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61,
0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61,
0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6f,
0x6b, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x61, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x2d, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61,
0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61,
0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61,
0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75,
0x63, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72,
0x6b, 0x75, 0x63, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x7a, 0x72,
0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f,
0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d,
0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x4b, 0x61,
0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e,
0x6f, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4b, 0x6f, 0x161, 0x72, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x72,
0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x160, 0x72, 0x69, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4c, 0x69, 0x6e, 0x69, 0x6a, 0x73, 0x6b,
0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x76,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x76, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x76, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x65, 0x7a,
0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x69, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x161, 0x61, 0x6c, 0x73, 0x6b,
0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a,
0x75, 0x73, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a,
0x75, 0x73, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6f, 0x6e,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x6b,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x65, 0x6d,
0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75,
0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e,
0x64, 0x6c, 0x65, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x69,
0x6a, 0x75, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f,
0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f, 0x73, 0x74, 0x72, 0x76,
0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6c, 0x61,
0x75, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x70,
0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x76, 0x69, 0x6e,
0x65, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50,
0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67,
0x76, 0x61, 0x6a, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x69, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x69, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69,
0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65,
0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x6e, 0x20, 0x50,
0x6a, 0x65, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f,
0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72,
0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x6f, 0x6e,
0x70, 0x65, 0x6a, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x52,
0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x65,
0x6a, 0x161, 0x65, 0x6c, 0x69, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x75, 0x17e, 0x6e, 0x61, 0x20,
0x44, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x160, 0x6f, 0x76, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61,
0x6a, 0x70, 0x65, 0x6a, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x75, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x72, 0x75,
0x67, 0x76, 0x61, 0x6a, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x56, 0x65, 0x6e, 0x65, 0x63, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x56, 0x65, 0x6a, 0x6b,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x20, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x76, 0x67, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x75, 0x17e, 0x6e, 0x6f,
0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66,
0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61,
0x6e, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c,
0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76,
0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x70, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x70,
0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x70, 0x69,
0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x61,
0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a,
0x65, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65,
0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x41, 0x7a, 0x6f, 0x72, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x7a, 0x6f, 0x72, 0x69, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x7a,
0x6f, 0x72, 0x69, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x161, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x161, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x75,
0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x6f, 0x6c,
0x69, 0x76, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x61, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69,
0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x75, 0x6d, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b,
0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b,
0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b,
0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x61,
0x6d, 0x6f, 0x72, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x61,
0x74, 0x61, 0x6d, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x74,
0x61, 0x6d, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x74, 0x61,
0x6d, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x10c, 0x69, 0x6c, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x10c, 0x69, 0x6c, 0x65, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c,
0x69, 0x6c, 0x65, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x6f, 0x17e,
0x69, 0x107, 0x6e, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x20, 0x28,
0x4b, 0x65, 0x6c, 0x69, 0x6e, 0x67, 0x29, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x6b, 0x6f,
0x76, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x70, 0x6f, 0x6c, 0x75, 0x2d, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b,
0x75, 0x62, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62,
0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x44, 0x65, 0x6a, 0x76, 0x69, 0x73, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x44, 0x69, 0x6d, 0x6f, 0x6e, 0x20, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a,
0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x61, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55,
0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x72,
0x65, 0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61,
0x6e, 0x63, 0x75, 0x73, 0x6b, 0x61, 0x20, 0x47, 0x76, 0x61, 0x6a, 0x61,
0x6e, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e,
0x63, 0x75, 0x73, 0x6b, 0x6f, 0x20, 0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x20,
0x69, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47,
0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x20, 0x70, 0x6f, 0x20, 0x47,
0x72, 0x69, 0x6e, 0x69, 0x10d, 0x75, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e,
0x69, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x69, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x69, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x5a, 0x61, 0x6c, 0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d,
0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c,
0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f,
0x76, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x61, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x49, 0x72, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x72,
0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x7a,
0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b,
0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b,
0x6f, 0x161, 0x72, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x72,
0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x4c, 0x61, 0x6a, 0x6e, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x76, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x76, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x76, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x69,
0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x72,
0x6b, 0x69, 0x7a, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x72,
0x161, 0x61, 0x6c, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x63, 0x69, 0x6a, 0x75, 0x73, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75,
0x73, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6f, 0x6e, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x69,
0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e,
0x65, 0x70, 0x61, 0x6c, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a,
0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76,
0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66,
0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65,
0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x4f, 0x73,
0x74, 0x72, 0x76, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x6f,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x76,
0x69, 0x6e, 0x65, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50,
0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x50, 0x65, 0x72, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50,
0x65, 0x72, 0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72,
0x75, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x69, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x69, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x20, 0x69, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x20, 0x69, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x69, 0x74, 0x6b,
0x65, 0x72, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x6f, 0x6e,
0x70, 0x65, 0x6a, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x6a, 0x6f,
0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x52, 0x65, 0x69, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x6a, 0x161, 0x65,
0x6c, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x6f,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x75, 0x17e,
0x6e, 0x61, 0x20, 0x44, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x160, 0x6f, 0x76, 0x61, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x10c, 0x75, 0x75, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x65, 0x6e, 0x65, 0x63, 0x75,
0x65, 0x6c, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x56, 0x65, 0x6a, 0x6b, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x20,
0x69, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0xa2e7, 0xa0c5, 0xa395, 0xa3e6, 0xa12e, 0xa209, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x67, 0x68, 0xe0, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x69, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x69,
0x76, 0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x4e, 0x75, 0x72,
0x6d, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xc0, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x63, 0x69, 0x72,
0x6f, 0x63, 0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50,
0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xc0, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x75, 0x6e, 0x65,
0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x55,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a,
0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61,
0x7a, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69,
0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x6d,
0x61, 0x7a, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x63,
0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73,
0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x63, 0x69, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x63, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55, 0x72,
0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x6c, 0x69, 0x76,
0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69,
0x67, 0x61, 0x6c, 0x69, 0x20, 0x6c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0xee, 0x20, 0x6d, 0x75, 0x6e,
0x74, 0x61, 0x67, 0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x64, 0xee, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x61,
0x67, 0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0xee, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x61, 0x67, 0x6e,
0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x61, 0x63, 0xec,
0x66, 0x69, 0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x61, 0x63, 0xec, 0x66, 0x69,
0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0xfb, 0x20, 0x50, 0x61, 0x63, 0xec, 0x66, 0x69, 0x63, 0x75,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69,
0x20, 0x41, 0x70, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x55,
0x72, 0x61, 0x20, 0xc0, 0x72, 0x61, 0x62, 0x62, 0x61, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0xc0, 0x72, 0x61, 0x62,
0x62, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0xc0, 0x72, 0x61, 0x62, 0x62, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x69, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x69, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x69, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0x69,
0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20,
0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0x69,
0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20,
0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0x69,
0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64,
0x69, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e,
0x74, 0x69, 0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0,
0x6e, 0x74, 0x69, 0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c,
0xe0, 0x6e, 0x74, 0x69, 0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x69, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x63, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20,
0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73,
0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x69, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x70, 0x75, 0x6e,
0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x69, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74,
0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69,
0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x64, 0x69, 0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x64, 0x69, 0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x64, 0x69, 0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20,
0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x7a, 0x6f, 0x72,
0x72, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72,
0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x69,
0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x42, 0xe0, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x42, 0xe0, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x42, 0xe0, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20,
0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2,
0x20, 0x42, 0x75, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64,
0xfb, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x43, 0x61, 0x70, 0x75, 0x20, 0x56, 0x69, 0x72, 0x64,
0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0x69, 0x20, 0x43, 0x61, 0x70, 0x75, 0x20, 0x56, 0x69, 0x72, 0x64,
0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0x69, 0x20, 0x43, 0x61, 0x70, 0x75, 0x20, 0x56, 0x69, 0x72, 0x64,
0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0x69, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x43, 0x69, 0x6c, 0x69, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xfb,
0x20, 0x43, 0x69, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x43, 0x69, 0x6c, 0x69, 0x55,
0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x55, 0x72,
0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xe2, 0x20,
0x43, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61,
0x20, 0x64, 0x69, 0x20, 0x4e, 0x61, 0x74, 0x61, 0x6c, 0x69, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69,
0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2,
0x20, 0x43, 0x75, 0x6c, 0x75, 0x6d, 0x6d, 0x69, 0x61, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x43,
0x75, 0x6c, 0x75, 0x6d, 0x6d, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x43, 0x75, 0x6c,
0x75, 0x6d, 0x6d, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64,
0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x43,
0x75, 0x62, 0x62, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x43, 0x75, 0x62, 0x62, 0x61, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69,
0x20, 0x43, 0x75, 0x62, 0x62, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x69, 0x76,
0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61,
0x73, 0x63, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61,
0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0x63, 0x61, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61,
0x73, 0x63, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0xc8, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x55, 0x72, 0x61, 0x20, 0x43,
0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x20, 0x45, 0x75, 0x72, 0x75,
0x70, 0x65, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x43, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x20, 0x45,
0x75, 0x72, 0x75, 0x70, 0x65, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69,
0x67, 0x61, 0x6c, 0x69, 0x20, 0x43, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x20, 0x45, 0x75, 0x72, 0x75, 0x70, 0x65, 0x61, 0x55, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x64, 0x69, 0x20, 0x4c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x69,
0x20, 0x4c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20,
0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x69,
0x76, 0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x63, 0x68,
0x69, 0xf9, 0x20, 0x61, 0x20, 0x4c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x75, 0x6e, 0x65, 0x6e,
0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x64, 0x69, 0x20, 0x50, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x69,
0x20, 0x50, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x55, 0x72, 0x61, 0x20, 0x73,
0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc,
0x73, 0x75, 0x6c, 0x69, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x55, 0x72, 0x61, 0x20, 0x64,
0xee, 0x20, 0x46, 0x69, 0x67, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xee, 0x20, 0x46, 0x69, 0x67, 0x69,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0xee, 0x20, 0x46, 0x69, 0x67, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2,
0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x63, 0x69, 0x73, 0x69, 0x55, 0x72, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x63, 0x69, 0x73, 0x69, 0x20, 0x64, 0x69, 0x20, 0x53, 0x63, 0x69, 0x72,
0x6f, 0x63, 0x63, 0x75, 0x20, 0x65, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0x41, 0x6e, 0x74, 0xe0, 0x72, 0x74, 0x69, 0x64, 0x69, 0x55, 0x72, 0x61,
0x20, 0x64, 0xee, 0x20, 0x47, 0x61, 0x6c, 0xe0, 0x70, 0x61, 0x67, 0x6f,
0x73, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x47, 0x69,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x47, 0x69, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0xe2, 0x20, 0x47, 0x69, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x55,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c,
0x69, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x55, 0x72, 0x61,
0x20, 0x4d, 0x69, 0x6e, 0x7a, 0x61, 0x6e, 0x61, 0x20, 0x64, 0x69, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x55, 0x72, 0x61,
0x20, 0x64, 0xe2, 0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e,
0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0xe2, 0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61,
0x6e, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c,
0x61, 0x6e, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20,
0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x61, 0x20,
0x6c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x47,
0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x61, 0x20, 0x6c,
0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61, 0x20,
0x6c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0xe2,
0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x61,
0x20, 0x6c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x55, 0x72,
0x61, 0x20, 0x64, 0xe2, 0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61,
0x6e, 0x6e, 0x69, 0x61, 0x20, 0x70, 0x75, 0x6e, 0x69, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0xe2, 0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e,
0x6e, 0x69, 0x61, 0x20, 0x70, 0x75, 0x6e, 0x69, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0xe2, 0x20, 0x47, 0x72, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x6e, 0x6e,
0x69, 0x61, 0x20, 0x70, 0x75, 0x6e, 0x69, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x55, 0x72, 0x61, 0x20, 0x4e, 0x75, 0x72, 0x6d, 0x61, 0x6c, 0x69, 0x20,
0x64, 0xfb, 0x20, 0x47, 0x75, 0x72, 0x66, 0x75, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0x48, 0x61, 0x77, 0x61, 0x61, 0x69, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x48, 0x61, 0x77, 0x61, 0x61, 0x69, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x48, 0x61, 0x77, 0x61, 0x61, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x55, 0x72,
0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x55, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x55, 0x72, 0x61, 0x20,
0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x55, 0x72, 0x61, 0x20,
0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0xcc, 0x6e, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0x55, 0x63, 0x69, 0x61, 0x6e, 0x75, 0x20, 0x49, 0x6e, 0x6e,
0x69, 0x61, 0x6e, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x49, 0x6e, 0x6e, 0x75, 0x63, 0x69, 0x6e, 0x61, 0x55, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x6e, 0x75, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x20, 0x63, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x6e,
0x75, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x69,
0x76, 0x61, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0x49, 0x6e, 0x6e, 0x75, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0x64, 0x69, 0x20, 0x70, 0x75, 0x6e, 0x65, 0x6e, 0x74, 0x69, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x49,
0x72, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72,
0x74, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x74, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x74, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x49, 0x73, 0x64,
0x72, 0x61, 0x65, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x49, 0x73, 0x64, 0x72, 0x61,
0x65, 0x6c, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0x69, 0x20, 0x49, 0x73, 0x64, 0x72, 0x61, 0x65, 0x6c,
0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x47, 0x69, 0x61, 0x70,
0x70, 0x75, 0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x75,
0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0xfb, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x75, 0x6e, 0x69,
0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x4b, 0x61, 0x7a, 0x7a, 0xe0,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb,
0x20, 0x4b, 0x61, 0x7a, 0x7a, 0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x64, 0x69, 0x20, 0x4c, 0x69, 0x76, 0x61, 0x6e, 0x74, 0x69, 0x55,
0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x4b, 0x61, 0x7a, 0x7a, 0xe0, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x50, 0x75, 0x6e,
0x65, 0x6e, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x43,
0x75, 0x72, 0xec, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x43, 0x75, 0x72, 0xec, 0x61, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xe2,
0x20, 0x43, 0x75, 0x72, 0xec, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0x69, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0x69, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x4b, 0x69,
0x72, 0x67, 0x68, 0xec, 0x7a, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x55,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c,
0x69, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x55, 0x72,
0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x4d, 0x61, 0x67, 0x64, 0x61, 0x6e, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4d,
0x61, 0x67, 0x64, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x67, 0x64, 0x61,
0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x4d, 0x61, 0x6c, 0x69,
0x73, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0xee, 0x20, 0x4d, 0x61,
0x72, 0x64, 0x69, 0x76, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0xee, 0x20,
0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x73, 0x69, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x55, 0x72,
0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x55, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x55, 0x72,
0x61, 0x20, 0x64, 0xfb, 0x20, 0x4d, 0xe8, 0x73, 0x73, 0x69, 0x63, 0x75,
0x20, 0x50, 0x61, 0x63, 0xec, 0x66, 0x69, 0x63, 0x75, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x4d,
0xe8, 0x73, 0x73, 0x69, 0x63, 0x75, 0x20, 0x50, 0x61, 0x63, 0xec, 0x66,
0x69, 0x63, 0x75, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0xfb, 0x20, 0x4d, 0xe8, 0x73, 0x73, 0x69, 0x63, 0x75,
0x20, 0x50, 0x61, 0x63, 0xec, 0x66, 0x69, 0x63, 0x75, 0x55, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x55, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x55, 0x72, 0x61,
0x20, 0x64, 0xfb, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x55, 0x72, 0x61,
0x20, 0x64, 0xe2, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c,
0x69, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xe2, 0x20, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x43, 0x61, 0x6c, 0x69, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x55, 0x72,
0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xe2, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x69, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x61, 0x55, 0x72,
0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xe2, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x6e, 0x69,
0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0xe2, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x69, 0x6c, 0x61,
0x6e, 0x6e, 0x69, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54,
0x69, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x55, 0x72, 0x61, 0x20,
0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x54, 0x69,
0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x54, 0x69, 0x72,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x4e, 0x69, 0x75, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x55, 0x72, 0x61, 0x20,
0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x55, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x55, 0x72,
0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x50,
0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x73,
0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x50, 0xe0, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x50, 0xe0, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0xe2, 0x20, 0x50, 0x61,
0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69,
0x6e, 0xec, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x69, 0x72,
0xf9, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0xfb, 0x20, 0x50, 0x69, 0x72, 0xf9, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xfb, 0x20, 0x50, 0x69, 0x72,
0xf9, 0x55, 0x72, 0x61, 0x20, 0x64, 0xee, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x64, 0xee, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x69, 0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61,
0x6c, 0x69, 0x20, 0x64, 0xee, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019,
0xcc, 0x73, 0x75, 0x6c, 0x69, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69,
0x78, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x2e, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x53, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x20, 0x64, 0x69, 0x20, 0x53, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x6f,
0x6e, 0x61, 0x70, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20,
0x64, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x55,
0x72, 0x61, 0x20, 0x64, 0xee, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x55,
0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0xee,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69,
0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xee, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0xee, 0x20, 0x53, 0x65, 0x79, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x69, 0x55, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x69,
0x20, 0x53, 0x61, 0x6c, 0x75, 0x6d, 0x75, 0x6e, 0x69, 0x55, 0x72, 0x61,
0x20, 0x64, 0xe2, 0x20, 0x47, 0x69, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20,
0x64, 0x69, 0x20, 0x73, 0x63, 0x69, 0x72, 0x6f, 0x63, 0x63, 0x75, 0x55,
0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x79, 0x6f,
0x77, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64,
0x69, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x55, 0x72, 0x61, 0x20,
0x64, 0xfb, 0x20, 0x54, 0x61, 0x67, 0x67, 0xec, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x75, 0x6b,
0x69, 0x6c, 0x61, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x64, 0xfb, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0xfb, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x55, 0x72,
0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20,
0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0xe8, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0xe8, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67,
0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62,
0xe8, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x55, 0x72, 0x61,
0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69,
0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x56, 0x69,
0x6e, 0x69, 0x7a, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0xec, 0x76, 0x75, 0x73, 0x74,
0x6f, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0xec, 0x76, 0x75, 0x73,
0x74, 0x6f, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c,
0x69, 0x20, 0x64, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0xec, 0x76, 0x75,
0x73, 0x74, 0x6f, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0xf2, 0x6c, 0x67, 0x75, 0x67, 0x72, 0x61, 0x64, 0x55, 0x72, 0x61, 0x20,
0x73, 0x75, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x56, 0xf2,
0x6c, 0x67, 0x75, 0x67, 0x72, 0x61, 0x64, 0x55, 0x72, 0x61, 0x20, 0x6c,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x56, 0xf2, 0x6c,
0x67, 0x75, 0x67, 0x72, 0x61, 0x64, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x6c, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x57, 0x61,
0x6b, 0x65, 0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x57, 0x61, 0x6c,
0x6c, 0x69, 0x73, 0x20, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x55, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c, 0x61, 0x72, 0x69,
0x20, 0x64, 0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x55,
0x72, 0x61, 0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x55, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x69, 0x72, 0x69, 0x6d,
0x6d, 0x75, 0x72, 0x67, 0x75, 0x55, 0x72, 0x61, 0x20, 0x73, 0x75, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x64, 0x69, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x69, 0x72, 0x69, 0x6d, 0x6d, 0x75, 0x72, 0x67, 0x75, 0x55, 0x72, 0x61,
0x20, 0x6c, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x20, 0x64, 0x69, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x69, 0x72, 0x69, 0x6d, 0x6d, 0x75, 0x72, 0x67,
0x75, 0x55, 0x72, 0x61, 0x20, 0x64, 0xfb, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77,
0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x142, 0x65, 0x64, 0x6e, 0x69, 0x6f,
0x77, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x61, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61,
0x142, 0x6d, 0x61, 0x63, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x61, 0x142, 0x6d, 0x61, 0x63, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x61, 0x6d, 0x61, 0x7a, 0x14d, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a, 0x14d, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a, 0x14d,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x6f, 0x73,
0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a,
0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b,
0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x67, 0x14d, 0x72, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x67, 0x14d, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x67, 0x14d, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79,
0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x6b, 0x74, 0x61, 0x75, 0x63, 0x6b,
0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x6b, 0x74, 0x61,
0x75, 0x63, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x61, 0x6b, 0x74, 0x69, 0x75, 0x62, 0x69, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6b, 0x74,
0x69, 0x75, 0x62, 0x69, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x50, 0x14d, 0x142, 0x77, 0x79, 0x73, 0x65, 0x70, 0x20,
0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x50, 0x14d, 0x142, 0x77, 0x79,
0x73, 0x65, 0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20,
0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x50, 0x14d, 0x142, 0x77, 0x79,
0x73, 0x65, 0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x41, 0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x41, 0x72,
0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x41, 0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x41, 0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20,
0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x41, 0x72, 0x67,
0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x41, 0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41, 0x72, 0x6d,
0x79, 0x6e, 0x69, 0x6a, 0x6f, 0x41, 0x72, 0x6d, 0x79, 0x6e, 0x69, 0x6a,
0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41, 0x72, 0x6d,
0x79, 0x6e, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x79, 0x63, 0x6b, 0x69, 0x20, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x79, 0x63,
0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x70, 0x6f, 0x73,
0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f,
0x77, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73, 0x74,
0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73, 0x74,
0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x2d, 0x7a, 0x61, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x70,
0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x2d,
0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b,
0x6f, 0x77, 0x6f, 0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17c,
0x61, 0x6e, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17c, 0x61, 0x6e, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41,
0x7a, 0x6f, 0x72, 0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x41, 0x7a, 0x6f, 0x72, 0x79, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x7a, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x7a, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x42, 0x6f, 0x6c, 0x69, 0x77, 0x69, 0x6a, 0x6f, 0x42, 0x72, 0x61,
0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57,
0x79, 0x73, 0x70, 0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x14d, 0x6e, 0x65,
0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d, 0x6e, 0x64, 0x6b,
0x61, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x14d,
0x6e, 0x65, 0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d, 0x6e,
0x64, 0x6b, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57,
0x79, 0x73, 0x70, 0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x14d, 0x6e, 0x65,
0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d, 0x6e, 0x64, 0x6b,
0x61, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x43,
0x68, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x43, 0x68, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x47, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57,
0x79, 0x73, 0x70, 0x79, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x61, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x61, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4b, 0x75, 0x62, 0x61, 0x20,
0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4b, 0x75, 0x62, 0x61, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b,
0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20,
0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x20,
0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x79, 0x73, 0x70, 0x61,
0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x6f,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f,
0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f,
0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73,
0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x64,
0x61, 0x6c, 0x73, 0x7a, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a,
0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x20, 0x28, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x69,
0x64, 0x17c, 0x69, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46,
0x69, 0x64, 0x17c, 0x69, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x47, 0x75, 0x6a, 0x61, 0x6e, 0x61,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x6f, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x65, 0x20, 0x54, 0x65, 0x72, 0x79,
0x74, 0x6f, 0x72, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x142, 0x65, 0x64, 0x6e,
0x69, 0x6f, 0x77, 0x65, 0x20, 0x69, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72,
0x6b, 0x74, 0x79, 0x63, 0x7a, 0x6e, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x79,
0x6a, 0x6f, 0x47, 0x72, 0x75, 0x7a, 0x79, 0x6a, 0x6f, 0x20, 0x28, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x47, 0x72, 0x75, 0x7a, 0x79, 0x6a, 0x6f,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x77, 0x65,
0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x75, 0x6e, 0x69, 0x77, 0x65, 0x72,
0x73, 0x61, 0x6c, 0x6e, 0x65, 0x67, 0x6f, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x75, 0x47, 0x72, 0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f,
0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x47, 0x72,
0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x57, 0x73,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x47, 0x72, 0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79,
0x6a, 0x6f, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x47, 0x72, 0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a,
0x6f, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x47,
0x72, 0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x5a,
0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x47, 0x72, 0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x79, 0x6a, 0x6f, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x48, 0x61, 0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x48, 0x61, 0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x79, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x48, 0x14d, 0x6e, 0x67, 0x6b, 0x14d, 0x6e, 0x67, 0x48,
0x14d, 0x6e, 0x67, 0x6b, 0x14d, 0x6e, 0x67, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x48, 0x14d, 0x6e, 0x67, 0x6b, 0x14d, 0x6e, 0x67, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x4b, 0x6f, 0x62, 0x64, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x4b, 0x6f, 0x62, 0x64, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x69, 0x6e, 0x64,
0x79, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0xd4,
0x63, 0x65, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x79, 0x6a, 0x73, 0x6b,
0x69, 0x69, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x14d, 0x6e, 0x65, 0x7a,
0x79, 0x6a, 0x6f, 0x20, 0x50, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64,
0x6b, 0x6f, 0x77, 0x6f, 0x49, 0x6e, 0x64, 0x14d, 0x6e, 0x65, 0x7a, 0x79,
0x6a, 0x6f, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f,
0x49, 0x6e, 0x64, 0x14d, 0x6e, 0x65, 0x7a, 0x79, 0x6a, 0x6f, 0x20, 0x5a,
0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x49, 0x72, 0x6b, 0x75,
0x63, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x49, 0x72,
0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c,
0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x49, 0x7a, 0x72, 0x61,
0x65, 0x6c, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x4a, 0x61, 0x70, 0x14d, 0x6e, 0x69, 0x6a, 0x6f,
0x4a, 0x61, 0x70, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x4a, 0x61, 0x70, 0x14d, 0x6e, 0x69, 0x6a, 0x6f,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x69, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x77, 0x142, 0x6f, 0x77, 0x73, 0x6b, 0x20, 0x4b,
0x61, 0x6d, 0x63, 0x7a, 0x61, 0x63, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x50, 0x69, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x77, 0x142, 0x6f,
0x77, 0x73, 0x6b, 0x20, 0x4b, 0x61, 0x6d, 0x63, 0x7a, 0x61, 0x63, 0x6b,
0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4d, 0x61, 0x6c, 0x65,
0x7a, 0x79, 0x6a, 0x6f, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63,
0x7a, 0x6e, 0x79, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69,
0x63, 0x7a, 0x6e, 0x79, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x29,
0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x55, 0x142, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61,
0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x4e, 0x6f, 0x77,
0x6f, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x14d, 0x6e, 0x69, 0x6a, 0x6f,
0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x79, 0x6a, 0x6f, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x79, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x46, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x4e, 0x6f, 0x77,
0x6f, 0x20, 0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a,
0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77,
0x6f, 0x20, 0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a,
0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f,
0x77, 0x6f, 0x73, 0x79, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0xd4,
0x6d, 0x73, 0x6b, 0xd4, 0x6d, 0x73, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0xd4, 0x6d, 0x73, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x50, 0x61, 0x70,
0x75, 0x61, 0x2d, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x47, 0x77, 0x69, 0x6e,
0x65, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77,
0x61, 0x6a, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x50, 0x65, 0x72, 0x75, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x50, 0x65, 0x72, 0x75, 0x20, 0x28, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x79, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x46, 0x79, 0x6e, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x50, 0x6a, 0x14d, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6b, 0x79, 0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64,
0x7a, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6b, 0x79, 0x7a,
0x79, 0x142, 0x6f, 0x72, 0x64, 0x7a, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20,
0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x53, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x57,
0x79, 0x73, 0x70, 0x79, 0x20, 0x53, 0x61, 0x6c, 0x14d, 0x6d, 0x14d, 0x6e,
0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x142,
0x65, 0x64, 0x6e, 0x69, 0x6f, 0x77, 0x6f, 0x54, 0x61, 0x6a, 0x70, 0x65,
0x6a, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x54, 0x61, 0x6a,
0x70, 0x65, 0x6a, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x54, 0x14d, 0x6e, 0x67, 0x61, 0x54, 0x14d,
0x6e, 0x67, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x54,
0x14d, 0x6e, 0x67, 0x61, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x79,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x79,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x79, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x6a, 0x20,
0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x55, 0x72, 0x75, 0x67, 0x77,
0x61, 0x6a, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x57, 0x79, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c,
0x61, 0x57, 0x142, 0x61, 0x64, 0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x142, 0x61, 0x64,
0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x6f, 0x142,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x57, 0x6f, 0x142, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x79, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x79, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6a, 0x75, 0x6b, 0x14d, 0x144, 0x73, 0x6b, 0x69,
0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x622, 0x641,
0x631, 0x64a, 0x642, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x648, 0x680, 0x631, 0x20, 0x622, 0x641, 0x631, 0x64a, 0x6aa, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x68f, 0x6a9, 0x6bb, 0x20, 0x622, 0x641, 0x631,
0x64a, 0x6aa, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x622,
0x641, 0x631, 0x64a, 0x642, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x622, 0x641, 0x631, 0x64a, 0x642, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x622, 0x641, 0x631, 0x64a,
0x642, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631,
0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x644, 0x627, 0x633,
0x6aa, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x644, 0x627,
0x633, 0x6aa, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x644, 0x627, 0x633, 0x6aa, 0x627, 0x20,
0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x64a, 0x645, 0x64a, 0x632, 0x648, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x645, 0x64a, 0x632, 0x648, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x64a, 0x645, 0x64a, 0x632, 0x648, 0x646, 0x20, 0x62c, 0x64a,
0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x68f, 0x64a,
0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634,
0x631, 0x642, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x634, 0x631, 0x642, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x647, 0x627, 0x699, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x67e, 0x647, 0x627, 0x699, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x647, 0x627, 0x699, 0x64a, 0x20,
0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x67e, 0x64a, 0x633, 0x64a, 0x641, 0x6aa, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a,
0x633, 0x64a, 0x641, 0x6aa, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x633, 0x64a, 0x641, 0x6aa, 0x20, 0x68f, 0x64a,
0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x67e,
0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x67e, 0x64a,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x67e, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x68f,
0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x639,
0x631, 0x628, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x639,
0x631, 0x628, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x639, 0x631, 0x628, 0x64a, 0x646, 0x20,
0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67d, 0x64a, 0x646, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67d, 0x627,
0x626, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67d, 0x627, 0x626, 0x646, 0x20, 0x62c, 0x64a,
0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x627, 0x631, 0x62c,
0x646, 0x67d, 0x64a, 0x646, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x67d, 0x64a,
0x646, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x627, 0x631,
0x62c, 0x646, 0x67d, 0x64a, 0x646, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648,
0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x622, 0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x622, 0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622,
0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648,
0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x64a, 0x67d, 0x644, 0x627, 0x646, 0x67d, 0x6aa, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x64a, 0x67d, 0x644, 0x627, 0x646, 0x67d, 0x6aa, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x64a, 0x67d, 0x644, 0x627, 0x646, 0x67d, 0x6aa, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x622, 0x633, 0x67d, 0x631, 0x64a,
0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633,
0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x631,
0x6aa, 0x632, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631,
0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x631, 0x6aa, 0x632,
0x64a, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622,
0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d,
0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x631, 0x6aa,
0x632, 0x64a, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x680,
0x631, 0x20, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d,
0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x634, 0x631,
0x642, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x622, 0x633, 0x67d,
0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x68f, 0x64a,
0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x630,
0x631, 0x628, 0x627, 0x626, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x64a, 0x62c, 0x627,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x64a, 0x62c, 0x627,
0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x648, 0x631, 0x632,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x648, 0x631, 0x632,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x632, 0x648, 0x631, 0x632, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x628, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x64a, 0x634, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x628, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x64a, 0x634,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x628, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x64a, 0x634, 0x20, 0x62c,
0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x680, 0x648, 0x67d, 0x627, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x628, 0x648, 0x644, 0x648, 0x64a, 0x627, 0x626, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x628, 0x631, 0x64a, 0x633, 0x64a, 0x644, 0x64a, 0x627,
0x626, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x631, 0x627, 0x633, 0x64a, 0x644,
0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x628, 0x631, 0x627, 0x633, 0x64a, 0x644, 0x64a, 0x627,
0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x631, 0x648, 0x646, 0x627, 0x626,
0x64a, 0x20, 0x62f, 0x627, 0x631, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x67e, 0x20, 0x648, 0x631,
0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x67e, 0x20,
0x648, 0x631, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x67e, 0x20, 0x648, 0x631, 0x68a,
0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x645, 0x648, 0x631, 0x648, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x686, 0x626, 0x67f, 0x645, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x686, 0x626, 0x67f, 0x645, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x626, 0x67f, 0x645, 0x20, 0x62c,
0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x686, 0x644, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x686, 0x644, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x644, 0x64a, 0x20, 0x62c, 0x64a, 0x20,
0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x686, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x686, 0x627, 0x626, 0x646, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x627, 0x626, 0x646, 0x627,
0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x631, 0x633, 0x645, 0x633, 0x20, 0x622, 0x626,
0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x6aa, 0x648, 0x6aa, 0x648, 0x633, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644,
0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648,
0x644, 0x645, 0x628, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x6aa, 0x648, 0x644, 0x645, 0x628, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x644,
0x645, 0x628, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647,
0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648,
0x6aa, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x6aa, 0x20, 0x622, 0x626, 0x64a,
0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x6aa, 0x20, 0x622,
0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x68c, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x648, 0x628, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x648, 0x628, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa,
0x64a, 0x648, 0x628, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x68a, 0x64a, 0x648, 0x633,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x68a, 0x648, 0x645, 0x627, 0x646,
0x67d, 0x20, 0x62f, 0x631, 0x648, 0x64a, 0x626, 0x644, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x648, 0x680, 0x631, 0x20, 0x62a, 0x64a, 0x645, 0x648,
0x631, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x633, 0x67d,
0x631, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x633, 0x67d, 0x631, 0x20, 0x622,
0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x633,
0x67d, 0x631, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x6aa, 0x648, 0x627, 0x68a, 0x648,
0x631, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632,
0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa,
0x632, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x627, 0x648, 0x646,
0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x634, 0x631, 0x642, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e,
0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20,
0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x648, 0x68c, 0x64a, 0x6aa, 0x20, 0x645, 0x634, 0x631, 0x642, 0x64a,
0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648,
0x644, 0x647, 0x647, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x648, 0x644, 0x647, 0x647, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x68f,
0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641,
0x627, 0x6aa, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x622, 0x626, 0x64a, 0x20,
0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641,
0x627, 0x6aa, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x622, 0x626, 0x64a, 0x20,
0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x627, 0x6aa, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x62c, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x641, 0x62c, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x62c, 0x64a, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x627, 0x646, 0x633, 0x64a, 0x633,
0x64a, 0x20, 0x6af, 0x64a, 0x627, 0x646, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x641, 0x631, 0x627, 0x646, 0x633, 0x64a, 0x633, 0x64a, 0x20, 0x68f,
0x627, 0x6a9, 0x6bb, 0x64a, 0x648, 0x20, 0x6fd, 0x20, 0x627, 0x646, 0x67d, 0x627,
0x631, 0x6aa, 0x67d, 0x6aa, 0x20, 0x648, 0x642, 0x62a, 0x6af, 0x627, 0x644, 0x627,
0x67e, 0x627, 0x6af, 0x648, 0x632, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x6af, 0x64a, 0x645, 0x628, 0x64a, 0x626, 0x631, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x62c, 0x627, 0x631, 0x62c, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x62c, 0x627, 0x631, 0x62c, 0x64a, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x62c,
0x627, 0x631, 0x62c, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646,
0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6af,
0x644, 0x628, 0x631, 0x67d, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646,
0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6af, 0x631, 0x64a, 0x646,
0x20, 0x648, 0x686, 0x20, 0x645, 0x64a, 0x646, 0x20, 0x67d, 0x627, 0x626, 0x64a,
0x645, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646, 0x20,
0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x634, 0x631, 0x642, 0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x6af, 0x631,
0x64a, 0x646, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646, 0x20,
0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x63a, 0x631, 0x628, 0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x6af, 0x631,
0x64a, 0x646, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x62e, 0x644, 0x64a, 0x62c, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6af, 0x64a, 0x627, 0x646, 0x627, 0x626, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x647, 0x648, 0x627, 0x626, 0x64a, 0x20, 0x627, 0x644,
0x64a, 0x648, 0x67d, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x647, 0x648, 0x627, 0x626, 0x64a, 0x20, 0x627, 0x644, 0x64a, 0x648, 0x67d, 0x64a,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x647, 0x648, 0x627, 0x626, 0x64a, 0x20, 0x627, 0x644, 0x64a,
0x648, 0x67d, 0x64a, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x627, 0x646, 0x6af,
0x20, 0x6aa, 0x627, 0x646, 0x6af, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x647, 0x627, 0x646, 0x6af, 0x20, 0x6aa, 0x627, 0x646, 0x6af, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x647,
0x627, 0x646, 0x6af, 0x20, 0x6aa, 0x627, 0x646, 0x6af, 0x20, 0x62c, 0x64a, 0x20,
0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x647, 0x648, 0x648, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x647, 0x648, 0x648, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x648, 0x648, 0x68a, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x680, 0x627, 0x631, 0x62a, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x647,
0x646, 0x62f, 0x20, 0x633, 0x645, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x646, 0x68a, 0x648, 0x20, 0x686, 0x627, 0x626, 0x646, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a,
0x20, 0x627, 0x646, 0x68a, 0x648, 0x646, 0x64a, 0x634, 0x64a, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x680, 0x631, 0x20, 0x627, 0x646,
0x68a, 0x648, 0x646, 0x64a, 0x634, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x627, 0x646, 0x68a, 0x648,
0x646, 0x64a, 0x634, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x64a, 0x631, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x64a, 0x631, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x631, 0x627, 0x646,
0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x631, 0x6aa, 0x62a, 0x633, 0x6aa, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x631, 0x6aa, 0x62a, 0x633, 0x6aa, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x631, 0x6aa, 0x62a, 0x633, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f,
0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x633, 0x631, 0x627, 0x626, 0x64a, 0x644, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x644, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x633,
0x631, 0x627, 0x626, 0x64a, 0x644, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x67e,
0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x67e,
0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x642, 0x632, 0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x648, 0x680, 0x631, 0x20, 0x642, 0x632, 0x627, 0x642, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644,
0x647, 0x647, 0x20, 0x642, 0x627, 0x632, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x631, 0x64a, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x631, 0x64a, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x6aa, 0x648, 0x631, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a,
0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648,
0x633, 0x631, 0x627, 0x626, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x6aa, 0x631, 0x6cc, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6aa, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x631, 0x6cc, 0x633, 0x646, 0x648,
0x6cc, 0x627, 0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x631, 0x6cc, 0x633, 0x646,
0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a,
0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x631,
0x63a, 0x632, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x644, 0x627, 0x626, 0x646, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x644, 0x648, 0x631,
0x68a, 0x20, 0x647, 0x648, 0x648, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x644, 0x648, 0x631, 0x68a, 0x20, 0x647, 0x648, 0x648, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x644, 0x648, 0x631, 0x68a, 0x20, 0x647, 0x648, 0x648, 0x64a, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x645, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x6af,
0x627, 0x62f, 0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x644, 0x627, 0x626,
0x64a, 0x634, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x627, 0x644, 0x62f, 0x64a, 0x67e, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x645, 0x631, 0x6aa, 0x64a, 0x648, 0x633, 0x633, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x627, 0x631, 0x634, 0x644, 0x20, 0x622, 0x626, 0x64a, 0x20,
0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x648, 0x631, 0x64a, 0x634, 0x64a, 0x633, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x645, 0x648, 0x631, 0x64a, 0x634, 0x64a, 0x633, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x648,
0x631, 0x64a, 0x634, 0x64a, 0x633, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646,
0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x627, 0x626, 0x648, 0x633, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x645, 0x64a, 0x6aa, 0x633, 0x64a, 0x6aa, 0x646, 0x20, 0x67e, 0x64a, 0x633, 0x64a,
0x641, 0x6aa, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x64a, 0x6aa, 0x633, 0x64a, 0x6aa,
0x646, 0x20, 0x67e, 0x64a, 0x633, 0x64a, 0x641, 0x6aa, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x64a,
0x6aa, 0x633, 0x64a, 0x6aa, 0x646, 0x20, 0x67e, 0x64a, 0x633, 0x64a, 0x641, 0x6aa,
0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627,
0x62a, 0x631, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644,
0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x631, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644,
0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x631, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x645, 0x627, 0x633, 0x6aa, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x645, 0x627, 0x633, 0x6aa, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x627, 0x633, 0x6aa,
0x648, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x64a, 0x627, 0x646, 0x645, 0x627, 0x631,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x627, 0x626, 0x648, 0x631,
0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x67e, 0x627,
0x644, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20,
0x6aa, 0x64a, 0x644, 0x64a, 0x68a, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x6aa, 0x64a, 0x644, 0x64a,
0x68a, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x6aa,
0x64a, 0x644, 0x64a, 0x68a, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20,
0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644,
0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x641, 0x627,
0x626, 0x648, 0x646, 0x68a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x641, 0x627, 0x626, 0x648,
0x646, 0x68a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648,
0x20, 0x641, 0x627, 0x626, 0x648, 0x646, 0x68a, 0x20, 0x644, 0x64a, 0x646, 0x68a,
0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x648, 0x64a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x646, 0x627, 0x631, 0x20, 0x641, 0x648, 0x6aa, 0x20,
0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x646, 0x627, 0x631, 0x20, 0x641, 0x648, 0x6aa, 0x20, 0x622,
0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x627, 0x631,
0x20, 0x641, 0x648, 0x6aa, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646,
0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x68a, 0x648,
0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x68a,
0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x68a, 0x648, 0x20, 0x62f, 0x64a,
0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x62c, 0x64a, 0x20,
0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646,
0x648, 0x648, 0x627, 0x633, 0x628, 0x626, 0x64a, 0x631, 0x633, 0x6aa, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x648, 0x648, 0x627, 0x633, 0x628, 0x626,
0x64a, 0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x648, 0x648, 0x627, 0x633, 0x628,
0x626, 0x64a, 0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x645,
0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x645,
0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x645, 0x633, 0x6aa, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x67e, 0x627, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x67e, 0x627, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x67e, 0x627, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x67e, 0x644, 0x627, 0x626, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x67e, 0x627, 0x67e, 0x648, 0x627, 0x20, 0x646, 0x64a, 0x648, 0x20, 0x6af,
0x646, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x631,
0x627, 0x6af, 0x648, 0x626, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x67e, 0x64a, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a,
0x631, 0x627, 0x6af, 0x648, 0x626, 0x64a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648,
0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x67e, 0x64a, 0x631, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e,
0x64a, 0x631, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x631, 0x648, 0x20, 0x62c, 0x64a,
0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x626, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x626, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x641, 0x644, 0x67e, 0x627, 0x626, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648,
0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x641, 0x648, 0x646, 0x64a, 0x6aa, 0x633, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644,
0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x64a,
0x646, 0x67d, 0x20, 0x67e, 0x64a, 0x626, 0x631, 0x20, 0x6fd, 0x20, 0x645, 0x64a,
0x6aa, 0x644, 0x648, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633,
0x64a, 0x646, 0x67d, 0x20, 0x67e, 0x64a, 0x626, 0x631, 0x20, 0x6fd, 0x20, 0x645,
0x64a, 0x6aa, 0x644, 0x648, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x64a, 0x646, 0x67d, 0x20,
0x67e, 0x64a, 0x626, 0x631, 0x20, 0x6fd, 0x20, 0x645, 0x64a, 0x6aa, 0x644, 0x648,
0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x67d, 0x6aa, 0x64a, 0x631, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x648, 0x646, 0x627, 0x67e, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x627, 0x646, 0x6af,
0x20, 0x64a, 0x627, 0x646, 0x6af, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x631, 0x64a, 0x20, 0x64a, 0x648, 0x646, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x631, 0x648, 0x67f, 0x64a, 0x631, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x633, 0x62e, 0x627, 0x644, 0x64a, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x62e, 0x627, 0x644, 0x64a, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x633, 0x62e, 0x627, 0x644, 0x64a, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f,
0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633,
0x627, 0x645, 0x648, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633,
0x627, 0x645, 0x648, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x627, 0x645, 0x648, 0x627, 0x20,
0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x634, 0x64a, 0x20, 0x634, 0x644, 0x632, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x633, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x633, 0x648, 0x644, 0x648, 0x645, 0x646, 0x20, 0x622, 0x626, 0x64a, 0x20,
0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x68f,
0x6a9, 0x6bb, 0x20, 0x62c, 0x627, 0x631, 0x62c, 0x64a, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x633, 0x648, 0x631, 0x64a, 0x20, 0x646, 0x627, 0x645,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x627, 0x626, 0x64a, 0x648,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x627, 0x647, 0x64a,
0x67d, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x627, 0x626,
0x67e, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x627, 0x626,
0x67e, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x62a, 0x627, 0x626, 0x67e, 0x64a, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x62a, 0x627, 0x62c, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x67d, 0x648, 0x6aa, 0x64a, 0x644, 0x627, 0x626, 0x648,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67d, 0x648, 0x646, 0x6af, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67d, 0x648, 0x646, 0x6af, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x67d, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x686, 0x64a, 0x648, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x62a, 0x631, 0x6aa, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x631, 0x6aa, 0x645, 0x627, 0x646, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x631, 0x6aa, 0x645, 0x627, 0x646, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627,
0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x648, 0x648,
0x627, 0x644, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x648,
0x631, 0x648, 0x6af, 0x627, 0x626, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x64a, 0x648, 0x631, 0x648, 0x6af, 0x627, 0x626, 0x64a, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x64a,
0x648, 0x631, 0x648, 0x6af, 0x627, 0x626, 0x64a, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x632, 0x628, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x628, 0x6aa, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x632, 0x628, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c,
0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x648, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x627, 0x646, 0x648, 0x627, 0x62a,
0x648, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x648, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20, 0x62c,
0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x648, 0x64a, 0x646, 0x632, 0x648, 0x64a, 0x644, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x648, 0x648, 0x633,
0x62a, 0x648, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x644,
0x627, 0x62f, 0x648, 0x648, 0x633, 0x62a, 0x648, 0x6aa, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648,
0x644, 0x627, 0x62f, 0x648, 0x648, 0x633, 0x62a, 0x648, 0x6aa, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631,
0x627, 0x62f, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627,
0x62f, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x648, 0x633, 0x67d, 0x648, 0x6aa, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x64a, 0x6aa, 0x20, 0x622, 0x626,
0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x648, 0x64a, 0x644, 0x633, 0x20, 0x6fd, 0x20, 0x641, 0x62a, 0x648, 0x646,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x627, 0x6aa, 0x62a,
0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x627, 0x6aa,
0x62a, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x627, 0x6aa, 0x62a, 0x633, 0x6aa, 0x20,
0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x64a, 0x6aa, 0x627, 0x67d, 0x64a, 0x631, 0x646, 0x628, 0x631,
0x6af, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x6aa, 0x627, 0x67d,
0x64a, 0x631, 0x646, 0x628, 0x631, 0x6af, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x6aa, 0x627, 0x67d,
0x64a, 0x631, 0x646, 0x628, 0x631, 0x6af, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a,
0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x6aa,
0x648, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x92e, 0x930, 0x915,
0x91c, 0x93c, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92e, 0x930, 0x915, 0x91c,
0x93c, 0x940, 0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915,
0x94d, 0x924, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x926, 0x940, 0x902,
0x952, 0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x913, 0x92d,
0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x913, 0x92d, 0x930, 0x940, 0x20,
0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x913,
0x92d, 0x930, 0x940, 0x20, 0x926, 0x940, 0x902, 0x952, 0x939, 0x20, 0x91c, 0x94b,
0x20, 0x935, 0x915, 0x94d, 0x924, 0x92a, 0x939, 0x93e, 0x921, 0x93c, 0x940, 0x20,
0x935, 0x915, 0x94d, 0x924, 0x92a, 0x939, 0x93e, 0x921, 0x93c, 0x940, 0x20, 0x92e,
0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92a, 0x939,
0x93e, 0x921, 0x93c, 0x940, 0x20, 0x926, 0x940, 0x902, 0x952, 0x939, 0x20, 0x91c,
0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f,
0x915, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f,
0x915, 0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d,
0x924, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x926, 0x940, 0x902,
0x952, 0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x905, 0x91f,
0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x905,
0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x92e, 0x905, 0x92f, 0x93e,
0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x905, 0x91f, 0x932, 0x93e, 0x902,
0x91f, 0x93f, 0x915, 0x20, 0x926, 0x940, 0x902, 0x952, 0x939, 0x20, 0x91c, 0x94b,
0x20, 0x935, 0x915, 0x94d, 0x924, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20,
0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92e,
0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924,
0x941, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x20, 0x909, 0x928, 0x939, 0x93e, 0x930, 0x947, 0x20, 0x91c, 0x94b,
0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x913, 0x92d, 0x930, 0x940, 0x20, 0x92f,
0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x913,
0x92d, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x92e,
0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x913,
0x92d, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x909,
0x928, 0x939, 0x93e, 0x930, 0x947, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d,
0x924, 0x941, 0x909, 0x932, 0x939, 0x902, 0x926, 0x94b, 0x20, 0x92f, 0x942, 0x930,
0x94b, 0x92a, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x909, 0x932, 0x939,
0x902, 0x926, 0x94b, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x92e,
0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x909,
0x932, 0x939, 0x902, 0x926, 0x94b, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x20, 0x909, 0x928, 0x939, 0x93e, 0x930, 0x947, 0x20, 0x91c, 0x94b, 0x20, 0x935,
0x915, 0x94d, 0x924, 0x941, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a,
0x20, 0x92e, 0x940, 0x928, 0x20, 0x935, 0x915, 0x93c, 0x94d, 0x924, 0x941, 0xd87,
0xdc6, 0xdca, 0xd9c, 0xdb1, 0xdd2, 0xdc3, 0xdca, 0xdae, 0xdcf, 0xdb1, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xd85,
0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20,
0xd85, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdaf, 0xd9a, 0xdd4, 0xdab, 0xdd4, 0x20, 0xd85, 0xdb4,
0xdca, 0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd85, 0xdb4, 0xdca, 0x200d,
0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd85, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2,
0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd85, 0xdb4,
0xdca, 0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd87, 0xdbd,
0xdc3, 0xdca, 0xd9a, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdbd,
0xdc3, 0xdca, 0xd9a, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdbd, 0xdc3, 0xdca, 0xd9a, 0xdcf, 0x20, 0xdaf,
0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd87, 0xdb8, 0xdbb, 0xdca, 0xdc3, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd87, 0xdb8, 0xdbb, 0xdca, 0xdc3, 0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdb8, 0xdbb,
0xdca, 0xdc3, 0xdb1, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20,
0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb8, 0xdb0, 0xdca,
0x200d, 0xdba, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4,
0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20,
0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20,
0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb8, 0xdb0, 0xdca,
0x200d, 0xdba, 0xdb8, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20,
0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb1, 0xdd0, 0xd9c,
0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b,
0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1,
0xdd4, 0x20, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad,
0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdaf, 0xdd2,
0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xd9a, 0xdb3, 0xdd4, 0xd9a, 0xdbb, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2,
0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9a, 0xdb3, 0xdd4, 0xd9a, 0xdbb, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad,
0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xd9a, 0xdb3, 0xdd4, 0xd9a, 0xdbb, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86,
0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4,
0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20,
0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2,
0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a,
0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca,
0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdb4, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd85, 0xdb4, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdb4, 0xdd2, 0xdba, 0xdcf,
0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85,
0xdbb, 0xdcf, 0xdb6, 0xdd2, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdbb,
0xdcf, 0xdb6, 0xdd2, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdbb, 0xdcf, 0xdb6, 0xdd2, 0x20, 0xdaf, 0xdc4, 0xdc0,
0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd86, 0xdbb, 0xdca, 0xda2,
0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd86, 0xdbb,
0xdca, 0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb6, 0xda7,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2,
0xdb1, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca,
0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd86, 0xdb8, 0xdda, 0xdb1, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd86, 0xdb8, 0xdda,
0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd86, 0xdb8, 0xdda, 0xdb1, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdad, 0xdca, 0xdbd, 0xdcf, 0xdb1,
0xdca, 0xdad, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85,
0xdad, 0xdca, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0xdad, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdad,
0xdca, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0xdad, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd2,
0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca,
0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd,
0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20,
0xd94, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdb6, 0xda7, 0xdc4,
0xdd2, 0xdbb, 0x20, 0xd94, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd,
0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8,
0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20,
0xd94, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdb6, 0xda7, 0xdc4,
0xdd2, 0xdbb, 0x20, 0xd94, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd,
0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0,
0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca,
0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9f,
0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d,
0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdaf, 0xdc4, 0xdc0,
0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda,
0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdc3, 0xdbb, 0xdca, 0xdb6, 0xdba,
0xdd2, 0xda2, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85,
0xdc3, 0xdbb, 0xdca, 0xdb6, 0xdba, 0xdd2, 0xda2, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdc3,
0xdbb, 0xdca, 0xdb6, 0xdba, 0xdd2, 0xda2, 0xdcf, 0xdb1, 0xdca, 0x20, 0xd9c, 0xdca,
0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd87, 0xdc3, 0xddc, 0xdbb, 0xdca, 0xdc3, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd87, 0xdc3, 0xddc, 0xdbb, 0xdca, 0xdc3, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdc3, 0xddc, 0xdbb,
0xdca, 0xdc3, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xd82, 0xd9c, 0xdbd, 0xdcf, 0xdaf,
0xdda, 0xdc1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xd82, 0xd9c, 0xdbd,
0xdcf, 0xdaf, 0xdda, 0xdc1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xd82, 0xd9c, 0xdbd, 0xdcf, 0xdaf, 0xdda, 0xdc1,
0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf,
0xdbd, 0xdba, 0xdb7, 0xdd4, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb6, 0xddc, 0xdbd, 0xdd2, 0xdc0, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdd3, 0xdbd, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdd3, 0xdbd, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6,
0xdca, 0x200d, 0xdbb, 0xdc3, 0xdd3, 0xdbd, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb6, 0xdd8, 0xdb1, 0xdcf,
0xdba, 0xdd2, 0x20, 0xdaf, 0xdbb, 0xdd4, 0xdc3, 0xdca, 0xdc3, 0xdbd, 0xdcf, 0xdb8,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdda, 0xdb4, 0xdca, 0xdc0,
0xdda, 0xda9, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdda, 0xdb4,
0xdca, 0xdc0, 0xdda, 0xda9, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdda, 0xdb4, 0xdca, 0xdc0, 0xdda, 0xda9,
0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a,
0xdcf, 0xdbd, 0xdba, 0xda0, 0xdb8, 0xddc, 0xdbb, 0xddd, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xda0, 0xdd0, 0xdad, 0xdb8, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xda0, 0xdd0, 0xdad, 0xdb8, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd0, 0xdad, 0xdb8, 0xdca, 0x20,
0xdaf, 0xdd2, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd2,
0xdbd, 0xdd3, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd2, 0xdbd, 0xdd3,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xda0, 0xdd2, 0xdbd, 0xdd3, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xda0, 0xdd3, 0xdb1, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd3, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd3, 0xdb1, 0x20, 0xdaf, 0xdc4,
0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdca, 0x200d,
0xdbb, 0xdd2, 0xdc3, 0xdca, 0xdb8, 0xdc3, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xd9a, 0xddd, 0xdc3,
0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd9a, 0xddc, 0xdbd, 0xddc, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdcf, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdbd, 0xddc, 0xdb8, 0xdca, 0xdb6,
0xdd2, 0xdba, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdbd, 0xddc, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba,
0xdcf, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a,
0xdcf, 0xdbd, 0xdba, 0xd9a, 0xdd4, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd4, 0xd9a, 0xdca, 0x20,
0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd4, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4,
0xdb4, 0xdad, 0xdca, 0x20, 0xdb7, 0xdcf, 0xd9c, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd2,
0xdba, 0xdd4, 0xdb6, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd9a, 0xdd2, 0xdba, 0xdd4, 0xdb6, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd2, 0xdba, 0xdd4,
0xdb6, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd,
0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda9, 0xdcf, 0xdc0, 0xdd2, 0xdc3,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdaf, 0xdd4, 0xdb8, 0xddc, 0xdb1,
0xdca, 0xdad, 0xdca, 0x2d, 0xda9, 0xdca, 0xd8b, 0xdbb, 0xdca, 0xdc0, 0xdd2, 0xdbd,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xda7, 0xdd2, 0xdb8, 0xddd, 0xdbb, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc3, 0xdca, 0xda7, 0xdbb, 0xdca, 0x20, 0xdaf,
0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc3,
0xdca, 0xda7, 0xdbb, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc3,
0xdca, 0xda7, 0xdbb, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba,
0xd89, 0xd9a, 0xdca, 0xdc0, 0xdaf, 0xddd, 0xdbb, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdba, 0xdd4, 0xdbb,
0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0,
0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4,
0xdd3, 0xdba, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20,
0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdad, 0xdc0, 0xdad, 0xdca, 0x2d, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba,
0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3,
0xdba, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4,
0xdd3, 0xdba, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xddd, 0xd9a, 0xdca, 0xdbd, 0xdb1, 0xdca,
0xda9, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdc6, 0xddd, 0xd9a, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xda9, 0xdca, 0x20,
0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xddd, 0xd9a, 0xdca, 0xdbd, 0xdb1, 0xdca,
0xda9, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdc6, 0xdd2,
0xda2, 0xdd2, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xdd2, 0xda2, 0xdd2,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdc6, 0xdd2, 0xda2, 0xdd2, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xd82,
0xdc1, 0x20, 0xd9c, 0xdba, 0xdb1, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb4, 0xdca, 0x200d, 0xdbb, 0xd82, 0xdc1, 0x20, 0xdaf, 0xd9a, 0xdd4, 0xdab, 0xdd4,
0xdaf, 0xdd2, 0xd9c, 0x20, 0xdc3, 0xdc4, 0x20, 0xd87, 0xdb1, 0xdca, 0xda7, 0xdcf,
0xdbb, 0xdca, 0xd9a, 0xdca, 0xda7, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd9c, 0xdbd, 0xdb4, 0xd9c, 0xddc, 0xdc3, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd9c, 0xdd0, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdbb, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda2, 0xddd, 0xdbb, 0xdca, 0xda2, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda2, 0xddd,
0xdbb, 0xdca, 0xda2, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda2, 0xddd, 0xdbb, 0xdca,
0xda2, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9c, 0xdd2, 0xdbd,
0xdca, 0xdb6, 0xdbb, 0xdca, 0xda7, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb1,
0xdd2, 0xda0, 0xdca, 0x20, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb,
0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4,
0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1,
0xdca, 0xdad, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba,
0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1,
0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6,
0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca,
0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca,
0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xd9c, 0xdca,
0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd9c,
0xdbd, 0xdca, 0xdc6, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xdc0,
0xdcf, 0xdba, 0xdd2, 0x2d, 0xd85, 0xdbd, 0xdd9, 0xdba, 0xdd4, 0xdad, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xdc0, 0xdcf,
0xdba, 0xdd2, 0x2d, 0xd85, 0xdbd, 0xdd9, 0xdba, 0xdd4, 0xdad, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdc4, 0xdc0, 0xdcf, 0xdba, 0xdd2, 0x2d, 0xd85, 0xdbd, 0xdd9, 0xdba,
0xdd4, 0xdad, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf,
0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc,
0xd82, 0xd9a, 0xddc, 0xd82, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc,
0xd82, 0xd9a, 0xddc, 0xd82, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc, 0xd82, 0xd9a, 0xddc, 0xd82, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdc4, 0xddc, 0xdc0, 0xdca, 0xda9, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdc4, 0xddc, 0xdc0, 0xdca, 0xda9, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc, 0xdc0, 0xdca, 0xda9,
0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1,
0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2,
0xdba, 0xdb1, 0xdca, 0x20, 0xdc3, 0xdcf, 0xd9c, 0xdbb, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd4, 0xda0, 0xdd3, 0xdb1, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xd89,
0xdb1, 0xdca, 0xdaf, 0xdd4, 0xdb1, 0xdd3, 0xdc3, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4,
0xdd2, 0xdbb, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd4, 0xdb1, 0xdd3, 0xdc3, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd4, 0xdb1, 0xdd3, 0xdc3,
0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89,
0xdbb, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb, 0xdcf,
0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd89, 0xdbb, 0xdcf, 0xdb1, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0x20, 0xd9a,
0xdcf, 0xdbd, 0xdba, 0xd89, 0xdbb, 0xdca, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca,
0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb, 0xdca, 0xd9a,
0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb, 0xdca, 0xd9a, 0xdd4,
0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc1, 0xdca,
0x200d, 0xdbb, 0xdcf, 0xdba, 0xdbd, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a,
0xdc1, 0xdca, 0x200d, 0xdbb, 0xdcf, 0xdba, 0xdbd, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc1, 0xdca, 0x200d, 0xdbb,
0xdcf, 0xdba, 0xdbd, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xda2, 0xdb4, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xda2, 0xdb4, 0xdcf, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda2, 0xdb4, 0xdcf, 0xdb1, 0x20, 0xdaf, 0xdc4,
0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdc3, 0xd9a,
0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1,
0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9a, 0xdc3, 0xd9a, 0xdc3,
0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9a, 0xdc3, 0xd9a, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdbb, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdbb, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdbb, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd9a, 0xddc, 0xdc3, 0xdca, 0xdbb, 0xdda, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd9a, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdca, 0xdb1, 0xddc, 0xdba, 0xdcf, 0xdbb, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdca,
0x200d, 0xdbb, 0xdc3, 0xdca, 0xdb1, 0xddc, 0xdba, 0xdcf, 0xdbb, 0xdca, 0xdc3, 0xdca,
0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdca, 0xdb1, 0xddc, 0xdba, 0xdcf,
0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd2, 0xdbb,
0xdca, 0xd9c, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdc1, 0xdca, 0x200d, 0xdbb, 0xdd3, 0x20, 0xdbd, 0xd82, 0xd9a, 0xdcf,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbd, 0xdba, 0xdd2, 0xdb1, 0xdca, 0x20,
0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbd,
0xddd, 0xdbb, 0xdca, 0xda9, 0xdca, 0x20, 0xdc4, 0xddd, 0xdc0, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbd, 0xddd, 0xdbb, 0xdca, 0xda9, 0xdca, 0x20, 0xdc4,
0xddd, 0xdc0, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdbd, 0xddd, 0xdbb, 0xdca, 0xda9, 0xdca, 0x20, 0xdc4, 0xddd,
0xdc0, 0xdca, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb8, 0xdd9, 0xd9c, 0xda9, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb8, 0xdd9, 0xd9c, 0xda9, 0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd9, 0xd9c, 0xda9, 0xdb1,
0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd0, 0xdbd, 0xdda, 0xdc3, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdcf, 0xdbd, 0xdaf,
0xdd2, 0xdc0, 0xdba, 0xdd2, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb8, 0xdcf, 0xdbb, 0xdca, 0xd9a, 0xdd4, 0xd91, 0xdc3, 0xdcf, 0xdc3, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdcf, 0xdbb, 0xdca, 0xdc2, 0xdbd, 0xdca,
0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb8, 0xdd4, 0xdbb, 0xdd4, 0xdc3, 0xdd2, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb8, 0xdd4, 0xdbb, 0xdd4, 0xdc3, 0xdd2, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd4, 0xdbb, 0xdd4, 0xdc3, 0xdd2,
0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf,
0xdbd, 0xdba, 0xdb8, 0xddc, 0xdc0, 0xdca, 0xdc3, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd9, 0xd9a, 0xdca, 0xdc3, 0xdd2, 0xd9a, 0xddd, 0x20,
0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb8, 0xdd9, 0xd9a, 0xdca, 0xdc3, 0xdd2, 0xd9a, 0xddd, 0x20, 0xdb4,
0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd9, 0xd9a, 0xdca, 0xdc3,
0xdd2, 0xd9a, 0xddd, 0x20, 0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca,
0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdb6, 0xdcf, 0xda7,
0xdbb, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbd, 0xdcf, 0xdb1,
0xdca, 0x20, 0xdb6, 0xdcf, 0xda7, 0xdbb, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbd, 0xdcf, 0xdb1, 0xdca,
0x20, 0xdb6, 0xdcf, 0xda7, 0xdbb, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xddc, 0xdc3,
0xdca, 0xd9a, 0xdc0, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xddc,
0xdc3, 0xdca, 0xd9a, 0xdc0, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xddc, 0xdc3, 0xdca, 0xd9a, 0xdc0, 0xdca,
0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd2, 0xdba, 0xdb1, 0xdca, 0xdb8, 0xdcf, 0xdbb, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdcf, 0xdc0, 0xdd4, 0xdbb, 0xdd4,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdda, 0xdb4, 0xdcf, 0xdbd, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0, 0x20, 0xdc3, 0xdd9, 0xdbd, 0xda9,
0xddc, 0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb1, 0xdc0, 0x20, 0xdc3, 0xdd9, 0xdbd, 0xda9, 0xddc, 0xdb1, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0, 0x20, 0xdc3, 0xdd9, 0xdbd, 0xda9, 0xddc, 0xdb1,
0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2,
0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0, 0xdc3, 0xdd3,
0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0,
0xdc3, 0xdd3, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0, 0xdc3, 0xdd3, 0xdbd, 0xdb1,
0xdca, 0xdad, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb1, 0xdd2, 0xdc0, 0xdca, 0xdc6, 0xdc0, 0xdd4, 0xdb1, 0xdca, 0xdbd, 0xdb1,
0xdca, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd2, 0xdc0, 0xdca,
0xdc6, 0xdc0, 0xdd4, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd2, 0xdc0,
0xdca, 0xdc6, 0xdc0, 0xdd4, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdaf,
0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb1, 0xdd2, 0xdba, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1,
0xddc, 0xdc6, 0xddc, 0xdbd, 0xdca, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc6, 0xddc, 0xdbd,
0xdca, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc6,
0xddc, 0xdbd, 0xdca, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdc6, 0xdbb, 0xdca, 0xdb1, 0xdd0, 0xdb1, 0xdca, 0xda9, 0xddd, 0x20,
0xda9, 0xdd2, 0x20, 0xdb1, 0xddc, 0xdbb, 0xddc, 0xdb1, 0xdca, 0xdc4, 0xdcf, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xdbb, 0xdca, 0xdb1, 0xdd0, 0xdb1, 0xdca,
0xda9, 0xddd, 0x20, 0xda9, 0xdd2, 0x20, 0xdb1, 0xddc, 0xdbb, 0xddc, 0xdb1, 0xdca,
0xdc4, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdc6, 0xdbb, 0xdca, 0xdb1, 0xdd0, 0xdb1, 0xdca, 0xda9, 0xddd, 0x20,
0xda9, 0xdd2, 0x20, 0xdb1, 0xddc, 0xdbb, 0xddc, 0xdb1, 0xdca, 0xdc4, 0xdcf, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd,
0xdba, 0xdb1, 0xddc, 0xdc0, 0xdc3, 0xdd2, 0xdb6, 0xdd2, 0xdbb, 0xdca, 0xdc3, 0xdca,
0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc0, 0xdc3,
0xdd2, 0xdb6, 0xdd2, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc0,
0xdc3, 0xdd2, 0xdb6, 0xdd2, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd94, 0xdb8, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd94, 0xdb8, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd94, 0xdb8, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdcf, 0xd9a, 0xdd2, 0xdc3,
0xdca, 0xdae, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdcf,
0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdae, 0xdcf, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdcf, 0xd9a, 0xdd2, 0xdc3,
0xdca, 0xdae, 0xdcf, 0xdb1, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb4, 0xdbd, 0xdcf, 0xdc0, 0xdd4, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd0, 0xdb4, 0xdd4, 0xdc0, 0xdcf, 0x20,
0xdb1, 0xdd2, 0xdc0, 0xdca, 0x20, 0xd9c, 0xdd2, 0xdb1, 0xdd3, 0xdba, 0xdcf, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd0, 0xdbb, 0xd9c, 0xdd4, 0xdc0, 0xdda,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd0, 0xdbb, 0xd9c, 0xdd4, 0xdc0,
0xdda, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb4, 0xdd0, 0xdbb, 0xd9c, 0xdd4, 0xdc0, 0xdda, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb4, 0xdda,
0xdbb, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdda, 0xdbb, 0xdd4,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb4, 0xdda, 0xdbb, 0xdd4, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb4, 0xdd2, 0xdbd, 0xdd2, 0xdb4, 0xdd3,
0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd2, 0xdbd, 0xdd2, 0xdb4,
0xdd3, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb4, 0xdd2, 0xdbd, 0xdd2, 0xdb4, 0xdd3, 0xdb1, 0x20, 0xd9c, 0xdca,
0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdc6, 0xdd3, 0xdb1, 0xdd2, 0xd9a, 0xdca, 0xdc3, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4,
0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc1, 0xdcf, 0xdb1, 0xdca,
0xdad, 0x20, 0xdb4, 0xdd2, 0xdba, 0xdbb, 0xdda, 0x20, 0xdc3, 0xdc4, 0x20, 0xdb8,
0xdd0, 0xd9a, 0xddd, 0xdbd, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xdb4, 0xdd2, 0xdba, 0xdbb, 0xdda, 0x20,
0xdc3, 0xdc4, 0x20, 0xdb8, 0xdd0, 0xd9a, 0xddd, 0xdbd, 0xdb1, 0xdca, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc1, 0xdcf,
0xdb1, 0xdca, 0xdad, 0x20, 0xdb4, 0xdd2, 0xdba, 0xdbb, 0xdda, 0x20, 0xdc3, 0xdc4,
0x20, 0xdb8, 0xdd0, 0xd9a, 0xddd, 0xdbd, 0xdb1, 0xdca, 0x20, 0xdaf, 0xdd2, 0xdc0,
0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4,
0xdd2, 0xda7, 0xdca, 0xd9a, 0xdd9, 0xdba, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xddc, 0xdb1, 0xdb4, 0xdda, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb4, 0xdca, 0xdba, 0xddc, 0xdb1, 0xdca, 0xd9c, 0xdca, 0xdba, 0xdb1,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbb, 0xdd2, 0xdba, 0xdd4, 0xdb1,
0xdd2, 0xdba, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbb, 0xddc,
0xdad, 0xdd9, 0xdbb, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xd9b,
0xdbd, 0xdd2, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xd9b,
0xdbd, 0xdd2, 0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xd9b, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdc3, 0xdd0, 0xdb8, 0xddd, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdc3, 0xdd0, 0xdb8, 0xddd, 0xdc0, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xdd0, 0xdb8, 0xddd, 0xdc0,
0xdcf, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xdd3, 0xdc2, 0xdd9, 0xdbd, 0xdca, 0xdc3, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xdd2, 0xd82, 0xd9c, 0xdb4, 0xdca,
0xdb4, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xddc,
0xdbd, 0xddc, 0xdb8, 0xdb1, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdaf, 0xd9a, 0xdd4, 0xdab, 0xdd4, 0x20, 0xda2,
0xddd, 0xda2, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3,
0xdd4, 0xdbb, 0xdd2, 0xdb1, 0xdcf, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdc3, 0xdca, 0xdba, 0xddd, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xda7, 0xdcf, 0xdc4, 0xdd2, 0xda7, 0xdd2, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdad, 0xdcf, 0xdba, 0xdd2, 0xdb4, 0xdda, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdad, 0xdcf, 0xdba, 0xdd2, 0xdb4, 0xdda, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdad, 0xdcf, 0xdba, 0xdd2, 0xdb4, 0xdda,
0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xda7, 0xda2, 0xdd2, 0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xddc, 0xd9a, 0xdd9, 0xdbd, 0xdcf, 0xdc0, 0xdd4,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xddc, 0xd82, 0xd9c, 0xdcf, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xddc, 0xd82, 0xd9c, 0xdcf, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xddc,
0xd82, 0xd9c, 0xdcf, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd4, 0xd9a, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xdbb, 0xdca, 0xd9a, 0xdca, 0xdb8, 0xdd9, 0xdb1,
0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xda7, 0xdbb, 0xdca, 0xd9a, 0xdca, 0xdb8, 0xdd9, 0xdb1, 0xdd2, 0xdc3, 0xdca, 0xdad,
0xdcf, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xda7, 0xdbb, 0xdca, 0xd9a, 0xdca, 0xdb8, 0xdd9, 0xdb1, 0xdd2, 0xdc3,
0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xdd4, 0xdc0, 0xdcf, 0xdbd,
0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbb, 0xdd4, 0xd9c, 0xdd4,
0xdc0, 0xdda, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbb, 0xdd4, 0xd9c,
0xdd4, 0xdc0, 0xdda, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbb, 0xdd4, 0xd9c, 0xdd4, 0xdc0, 0xdda, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba,
0xd8b, 0xdc3, 0xdca, 0xdb6, 0xdd9, 0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdc3, 0xdca, 0xdb6, 0xdd9, 0xd9a,
0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdc3, 0xdca, 0xdb6, 0xdd9, 0xd9a,
0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdb1, 0xdd4,
0xd86, 0xda7, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdb1, 0xdd4,
0xd86, 0xda7, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdb1, 0xdd4, 0xd86, 0xda7, 0xdd4, 0x20, 0xd9c, 0xdd2,
0xdb8, 0xdca, 0xdc4, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0,
0xdd9, 0xdb1, 0xdd2, 0xdc3, 0xdd2, 0xdba, 0xdd4, 0xdbd, 0xdcf, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdca, 0xdbd, 0xdaf, 0xdd2, 0xdc0, 0xddd, 0xdc3, 0xdca,
0xda7, 0xddd, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdca,
0xdbd, 0xdaf, 0xdd2, 0xdc0, 0xddd, 0xdc3, 0xdca, 0xda7, 0xddd, 0xd9a, 0xdca, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0,
0xdca, 0xdbd, 0xdaf, 0xdd2, 0xdc0, 0xddd, 0xdc3, 0xdca, 0xda7, 0xddd, 0xd9a, 0xdca,
0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xddc, 0xdbd, 0xdca, 0xd9c, 0xddc, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdcf, 0xda9, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xddc,
0xdbd, 0xdca, 0xd9c, 0xddc, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdcf, 0xda9, 0xdca, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0,
0xddc, 0xdbd, 0xdca, 0xd9c, 0xddc, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdcf, 0xda9, 0xdca,
0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xddc, 0xdc3, 0xdca, 0xda7, 0xddc, 0xd9a, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdda, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd6,
0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdd0, 0xdbd,
0xdd2, 0xdc3, 0xdca, 0x20, 0xdc3, 0xdc4, 0x20, 0xdc6, 0xdd4, 0xda7, 0xdd4, 0xdb1,
0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xd9a, 0xdd4, 0xda7, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xd9a,
0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xd9a, 0xdd4, 0xda7, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xdd9, 0xd9a, 0xda7, 0xdd9,
0xdbb, 0xdd2, 0xdb1, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdba, 0xdd9, 0xd9a, 0xda7, 0xdd9, 0xdbb, 0xdd2, 0xdb1, 0xdca,
0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xdd9, 0xd9a, 0xda7, 0xdd9, 0xdbb, 0xdd2,
0xdb1, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xdd4,
0xd9a, 0xddc, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x61, 0x63,
0x72, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x72,
0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x66, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x61, 0x66, 0x72,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x75, 0x68, 0x6f,
0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x6c, 0x6a, 0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x6c, 0x6a, 0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6d, 0x61, 0x61,
0x74, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6d, 0x61,
0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6d, 0x61,
0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe1, 0x6c, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe1, 0x6c,
0x6e, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0xe1, 0x6c, 0x6e, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65,
0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x76,
0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x74, 0x69, 0x63,
0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x70, 0x69, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x70, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b,
0x74, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b,
0x74, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b, 0x74,
0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62,
0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1,
0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d,
0xe9, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d,
0xe9, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9,
0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x74, 0x72, 0x65, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x7a, 0xe1,
0x70, 0x61, 0x64, 0x6e, 0xfd, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65,
0x64, 0x6f, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e, 0xfd, 0x20, 0x61, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x74, 0x72, 0x65, 0x64, 0x6f, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e, 0xfd,
0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73,
0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64,
0x17e, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x6f,
0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x6f, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x161, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9,
0x161, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0xe9, 0x161, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x68, 0x75, 0x74, 0xe1, 0x6e, 0x73,
0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x6f, 0x6c, 0xed, 0x76, 0x69,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x61, 0x7a,
0xed, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x61,
0x7a, 0xed, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x61,
0x7a, 0xed, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x69,
0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x69, 0x6c, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x69, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20,
0x56, 0x69, 0x61, 0x6e, 0x6f, 0x10d, 0x6e, 0xe9, 0x68, 0x6f, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x10d, 0x61, 0x73, 0x20, 0x4b, 0x6f,
0x6b, 0x6f, 0x73, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x6f, 0x76, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x6f, 0x76, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0xfd,
0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x6f, 0x76, 0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x6f, 0x76, 0x65, 0x6a, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x63, 0x65, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x63, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x10d,
0x61, 0x73, 0x20, 0x56, 0x65, 0x13e, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e,
0xe9, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x20, 0x56, 0x65, 0x13e, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xe9,
0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x56, 0x65, 0x13e, 0x6b,
0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xe9, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x61, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x65, 0x75,
0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74,
0x72, 0x65, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x65, 0x75, 0x72,
0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x75,
0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd,
0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b,
0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65,
0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x75, 0x72,
0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70,
0x61, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3,
0x70, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x72, 0x61, 0x6e,
0x63, 0xfa, 0x7a, 0x73, 0x6b, 0x6f, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0xfa, 0x7a, 0x73, 0x6b, 0x79, 0x63, 0x68, 0x20, 0x6a,
0x75, 0x17e, 0x6e, 0xfd, 0x63, 0x68, 0x20, 0x61, 0x20, 0x61, 0x6e, 0x74,
0x61, 0x72, 0x6b, 0x74, 0x69, 0x63, 0x6b, 0xfd, 0x63, 0x68, 0x20, 0xfa,
0x7a, 0x65, 0x6d, 0xed, 0x67, 0x61, 0x6c, 0x61, 0x70, 0xe1, 0x67, 0x73,
0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed,
0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed,
0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x6f, 0x76, 0x67, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f,
0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f,
0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf3,
0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x67, 0x75, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x20, 0x50, 0x65, 0x72, 0x7a, 0x73, 0x6b, 0xe9, 0x68,
0x6f, 0x20, 0x7a, 0xe1, 0x6c, 0x69, 0x76, 0x75, 0x68, 0x61, 0x76, 0x61,
0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x6f, 0x76, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x6f, 0x76, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69,
0x6e, 0x64, 0x69, 0x63, 0x6b, 0x6f, 0x6f, 0x63, 0x65, 0xe1, 0x6e, 0x73,
0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x6f, 0x10d, 0xed,
0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65,
0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x7a, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x7a, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0xe9, 0x7a, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0xe1,
0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0xe1, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0xe1, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a,
0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61,
0x70, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x10d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61,
0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x6b, 0x61,
0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x6b, 0x61, 0x7a,
0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0xf3, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0xf3, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6b, 0xf3, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x69, 0x72, 0x67,
0x69, 0x7a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0xed,
0x6c, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x52, 0x6f, 0x76, 0x6e, 0xed, 0x6b, 0x6f, 0x76, 0xfd, 0x63,
0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x20, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x20, 0x48, 0x6f, 0x77, 0x61, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61,
0x20, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x48, 0x6f, 0x77, 0x61, 0x6d,
0x61, 0x63, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x61, 0x63, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61,
0x63, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x61, 0x6c, 0x61, 0x6a, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x72, 0x6b, 0xe9, 0x7a, 0x73,
0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x6d, 0x61, 0x75, 0x72,
0xed, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x6f, 0x76, 0x65, 0x6a,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x6d, 0x65, 0x78, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6c,
0x61, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74,
0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6e, 0x61, 0x75, 0x72, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6e, 0x65, 0x70, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x6b,
0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x73,
0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0xe9, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x73, 0xfa,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x69, 0x61, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x73, 0xfa, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x69, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x73,
0xfa, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x69, 0x61, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x6d,
0x61, 0x72, 0x69, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6f, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6f, 0x6d, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x61, 0x70, 0x75, 0x79, 0x2d,
0x4e, 0x6f, 0x76, 0x65, 0x6a, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x79,
0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69,
0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x66, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x70, 0x69, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x6d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x6d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x76, 0xfd,
0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x70,
0x63, 0x68, 0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x6f, 0x76, 0x65, 0x6a, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x63, 0x65, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x160, 0x61, 0x6c, 0x61, 0x6d,
0xfa, 0x6e, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72,
0x6f, 0x76, 0x6f, 0x76, 0x10d, 0x61, 0x73, 0x20, 0x4a, 0x75, 0x17e, 0x6e,
0x65, 0x6a, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x74, 0x63,
0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x61, 0x64,
0x17e, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6e,
0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6e, 0x17e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75,
0x72, 0x75, 0x67, 0x75, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a,
0x62, 0x65, 0x63, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a, 0x62, 0x65,
0x63, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x63, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x63, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x61, 0x66, 0x72, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x161,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x61,
0x66, 0x72, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61,
0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x161, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x61,
0x66, 0x72, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68,
0x6f, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x161, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x6c, 0x6a, 0x61, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6c,
0x6a, 0x61, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6c, 0x6a, 0x61,
0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56,
0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x47, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x47, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x161, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x161,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x161,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x10c, 0x61, 0x73, 0x3a, 0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a,
0x20, 0x41, 0x70, 0x69, 0x61, 0x50, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x72,
0x61, 0x62, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x61,
0x62, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x76,
0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x61, 0x70, 0x76, 0x65,
0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x61, 0x70, 0x76,
0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x6d, 0x6f, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x10c, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x69,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x69, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4b, 0x69, 0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4b, 0x69, 0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x4b, 0x69, 0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x6f, 0x17e, 0x69,
0x10d, 0x6e, 0x6f, 0x6f, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f,
0x76, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0x6f, 0x6f, 0x74, 0x6f, 0x161, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0x6f,
0x6f, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x6f, 0x6f,
0x6b, 0x6f, 0x76, 0x6f, 0x6f, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x73,
0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x10c, 0x61, 0x73, 0x3a, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x10c, 0x61,
0x73, 0x3a, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x56, 0x7a, 0x68, 0x6f, 0x64,
0x6e, 0x6f, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b,
0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x3a, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d,
0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x50, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x56, 0x65, 0x6c, 0x69,
0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b,
0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x72, 0x65,
0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76,
0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64,
0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x44, 0x6f,
0x64, 0x61, 0x74, 0x6e, 0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f,
0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x74,
0x6f, 0x10d, 0x6a, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x74, 0x6f, 0x10d,
0x6a, 0x65, 0x50, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x3a, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a, 0x65, 0x46, 0x69, 0x64,
0x17e, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x69,
0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x69,
0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20,
0x46, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x73, 0x6b, 0x61, 0x20, 0x47, 0x76,
0x61, 0x6a, 0x61, 0x6e, 0x61, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x73,
0x6b, 0x69, 0x20, 0x6a, 0x75, 0x17e, 0x6e, 0x69, 0x20, 0x69, 0x6e, 0x20,
0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x10d, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x161, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x6f, 0x76, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b,
0x69, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x161, 0x6b, 0x69, 0x20,
0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56,
0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68,
0x6f, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e,
0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61,
0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64,
0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x5a, 0x61, 0x6c, 0x69, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x47, 0x76, 0x61, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x6c,
0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x61,
0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x6c, 0x65, 0x75, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x48, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48,
0x6f, 0x76, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x76,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x6f,
0x63, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49,
0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x6f, 0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x4b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20,
0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20,
0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x6a, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x45, 0x6b, 0x76, 0x61, 0x74, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x10c, 0x61,
0x73, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x74, 0x6f, 0x6b,
0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x50,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f,
0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4d,
0x61, 0x72, 0x6b, 0x69, 0x7a, 0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b,
0x69, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x6f, 0x76, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4d, 0x65, 0x68, 0x69, 0x161, 0x6b, 0x69, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4d, 0x65, 0x68, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x65, 0x68,
0x69, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x161,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61,
0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73,
0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6a, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f,
0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f,
0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f,
0x76, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x66, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x69, 0x75, 0x65, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x161, 0x6b, 0x69, 0x20, 0x6f, 0x74,
0x6f, 0x6b, 0x69, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x161, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x50, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x161, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f,
0x6b, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4f, 0x6d, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x6c, 0x61, 0x76, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x70, 0x75, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67,
0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61,
0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50,
0x65, 0x72, 0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50,
0x65, 0x72, 0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65,
0x72, 0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20,
0x4f, 0x74, 0x6f, 0x10d, 0x6a, 0x65, 0x20, 0x46, 0x65, 0x6e, 0x69, 0x6b,
0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x6e, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x69,
0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x50, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x69, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x50, 0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x161, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x52, 0x6f, 0x74, 0x68, 0x65,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53,
0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x53, 0x65, 0x6a, 0x161, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x6f, 0x76, 0x6f, 0x6f, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61,
0x73, 0x3a, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x61,
0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54,
0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54,
0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x61, 0x64, 0x17e,
0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x76, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x6f, 0x76, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x6f, 0x76, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x6f, 0x76, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x20,
0x43, 0x68, 0x75, 0x75, 0x6b, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55,
0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61,
0x73, 0x3a, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x20, 0x57, 0x61, 0x6b, 0x65,
0x10c, 0x61, 0x73, 0x3a, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20,
0x69, 0x6e, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x75, 0x6b, 0x6f,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x61, 0x6b, 0x68,
0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x63, 0x72, 0x65, 0x57, 0x61, 0x6b,
0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x63, 0x72, 0x65,
0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c,
0x6b, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x63, 0x72, 0x65, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x66, 0x67, 0x67, 0x61, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x79, 0x61, 0x64,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x66, 0x75, 0x72, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b,
0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61,
0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68,
0x20, 0x65, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x53, 0x61,
0x61, 0x63, 0x61, 0x64, 0x61, 0x20, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x4b, 0x75, 0x6c, 0x61, 0x79, 0x6c, 0x61, 0x68, 0x61, 0x20,
0x65, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6d, 0x61,
0x73, 0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58,
0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6d,
0x61, 0x73, 0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x57, 0x61,
0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20,
0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d,
0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20,
0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61,
0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41,
0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41,
0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20,
0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d,
0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20,
0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x57, 0x61,
0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x42, 0x75, 0x75, 0x72, 0x61, 0x6c, 0x65, 0x79, 0x64, 0x61, 0x20, 0x65,
0x65, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20,
0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x75, 0x75, 0x72, 0x61, 0x6c,
0x65, 0x79, 0x64, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69,
0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61,
0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x75, 0x75, 0x72,
0x6c, 0x65, 0x79, 0x64, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79,
0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x61, 0x20, 0x65, 0x65, 0x20, 0x57, 0x61, 0x71, 0x6f,
0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20,
0x42, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x57, 0x61, 0x71,
0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44,
0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20,
0x42, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x57, 0x61, 0x71,
0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61,
0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x57,
0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6b,
0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x62, 0x69, 0x79, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x62,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44,
0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20,
0x41, 0x62, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68,
0x20, 0x65, 0x65, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x53, 0x61, 0x61,
0x63, 0x61, 0x64, 0x61, 0x20, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4b, 0x75, 0x6c, 0x61, 0x79, 0x6c, 0x61, 0x68, 0x61, 0x20, 0x41,
0x71, 0x74, 0x61, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68,
0x20, 0x65, 0x65, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x53, 0x61,
0x61, 0x63, 0x61, 0x64, 0x61, 0x20, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x6b, 0x75, 0x6c, 0x61, 0x79, 0x6c, 0x61, 0x68, 0x61, 0x20,
0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x72, 0x61, 0x62, 0x74, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x43, 0x61, 0x72, 0x61, 0x62,
0x74, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68,
0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x43,
0x61, 0x72, 0x61, 0x62, 0x74, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x61,
0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x72, 0x6a,
0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62,
0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74,
0x69, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20,
0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67,
0x61, 0x20, 0x65, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64,
0x6b, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x65, 0x6e, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x65, 0x6e, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x65,
0x65, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20,
0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67,
0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61,
0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69,
0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61,
0x6d, 0x61, 0x68, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20,
0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61,
0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x74,
0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x47,
0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x73, 0x74,
0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d,
0x61, 0x68, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b,
0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61,
0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61,
0x6d, 0x61, 0x68, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64,
0x6b, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72,
0x69, 0x67, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x79, 0x61, 0x64, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20,
0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x41, 0x73,
0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72,
0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61,
0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62,
0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20,
0x65, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61,
0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72,
0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x47, 0x61, 0x6c,
0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x41, 0x73, 0x61, 0x72, 0x62, 0x65, 0x79, 0x6a, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x73,
0x61, 0x72, 0x62, 0x65, 0x79, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x41, 0x73, 0x61, 0x72, 0x62, 0x65, 0x79, 0x6a, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x73, 0x6f,
0x72, 0x65, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x65, 0x64, 0x65,
0x65, 0x73, 0x68, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x65, 0x64, 0x65, 0x65, 0x73, 0x68,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x65, 0x64, 0x65, 0x65, 0x73, 0x68, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x42, 0x75, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x69, 0x66, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72,
0x61, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x61, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20,
0x42, 0x61, 0x72, 0x61, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x75, 0x72, 0x75, 0x6e,
0x65, 0x79, 0x20, 0x44, 0x61, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x61,
0x61, 0x6d, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x65,
0x79, 0x62, 0x20, 0x46, 0x61, 0x61, 0x72, 0x64, 0x65, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x65, 0x79, 0x62, 0x20,
0x46, 0x61, 0x61, 0x72, 0x64, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x4b, 0x65, 0x79, 0x62, 0x20, 0x46, 0x61, 0x61, 0x72, 0x64, 0x65,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x61, 0x6d, 0x6f,
0x72, 0x6f, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x61,
0x61, 0x74, 0x61, 0x6d, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x4a, 0x61, 0x61, 0x74, 0x61, 0x6d, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74,
0x61, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x61, 0x61, 0x74, 0x61, 0x6d, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x69, 0x6c, 0x69, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x69, 0x6c,
0x69, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x69, 0x6c, 0x69,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x68, 0x69, 0x69,
0x6e, 0x61, 0x68, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x53, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x68, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61,
0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x53, 0x68, 0x69, 0x69, 0x6e,
0x61, 0x68, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b,
0x69, 0x72, 0x69, 0x73, 0x6d, 0x61, 0x73, 0x20, 0x41, 0x79, 0x6c, 0x61,
0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x6f,
0x6b, 0x6f, 0x73, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67,
0x61, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69,
0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75,
0x75, 0x6b, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x75, 0x75,
0x6b, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e, 0x75, 0x73, 0x20, 0x58, 0x61,
0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20,
0x4b, 0x75, 0x75, 0x6b, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69,
0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x75,
0x62, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x4b, 0x75, 0x75, 0x62, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65,
0x65, 0x20, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x44, 0x61, 0x66, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x2d,
0x20, 0x64, 0x2019, 0x75, 0x72, 0x66, 0x69, 0x6c, 0x6c, 0x65, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x49, 0x69, 0x73, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x6f, 0x6f, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x49, 0x69, 0x73, 0x74, 0x61, 0x72, 0x20, 0x41, 0x79, 0x6c, 0x61,
0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x49, 0x69, 0x73, 0x74, 0x61, 0x72, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x49, 0x69, 0x73, 0x74,
0x61, 0x72, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x45, 0x6b, 0x75, 0x77, 0x61, 0x64, 0x6f,
0x6f, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61,
0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75,
0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42,
0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x59, 0x75, 0x72,
0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61,
0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72,
0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x69,
0x67, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61,
0x20, 0x59, 0x75, 0x72, 0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75,
0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72,
0x69, 0x67, 0x61, 0x20, 0x46, 0x6f, 0x67, 0x20, 0x65, 0x65, 0x20, 0x59,
0x75, 0x72, 0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x59, 0x75,
0x72, 0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x59,
0x75, 0x72, 0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x47,
0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x59, 0x75, 0x72,
0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x61,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x61, 0x6e, 0x20, 0x41, 0x79, 0x6c, 0x61,
0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x46, 0x61, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x61, 0x6e, 0x20,
0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x46, 0x61, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x61, 0x6e,
0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x65, 0x72, 0x65, 0x6e, 0x6a,
0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x66, 0x75, 0x72, 0x74, 0x61,
0x20, 0x46, 0x61, 0x72, 0x61, 0x6e, 0x73, 0x69, 0x69, 0x73, 0x6b, 0x61,
0x20, 0x26, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x61, 0x72, 0x6b, 0x74, 0x69,
0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c,
0x61, 0x62, 0x61, 0x67, 0x6f, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x72, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69,
0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x69, 0x6c, 0x62, 0x65,
0x65, 0x72, 0x74, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69,
0x6e, 0x77, 0x69, 0x6a, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x47, 0x69,
0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61,
0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61,
0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65,
0x65, 0x64, 0x6b, 0x61, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c,
0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x47,
0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b,
0x61, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x63, 0x61,
0x6e, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47,
0x75, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x75,
0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x61, 0x79, 0x2d, 0x41,
0x6c, 0x75, 0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74,
0x61, 0x20, 0x65, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x61, 0x79, 0x2d,
0x41, 0x6c, 0x75, 0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x48, 0x6f, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6f, 0x6e, 0x67, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x48, 0x6f, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6f, 0x6e,
0x67, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x48, 0x6f, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x48, 0x6f, 0x66, 0x75, 0x64, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x48, 0x6f, 0x66, 0x75, 0x64, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x48, 0x6f, 0x66, 0x75, 0x64, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x48, 0x69, 0x6e,
0x64, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x42, 0x61, 0x64, 0x77, 0x65, 0x79, 0x6e, 0x74, 0x61, 0x20, 0x48, 0x69,
0x6e, 0x64, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x73, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61,
0x6d, 0x61, 0x68, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65,
0x79, 0x73, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x79, 0x73, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c,
0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x65, 0x79, 0x73, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x49, 0x69, 0x72, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x49, 0x69, 0x72, 0x61, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61,
0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x49, 0x69,
0x72, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61,
0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x49, 0x73, 0x72, 0x61, 0x61, 0x69, 0x69, 0x6c, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x61,
0x69, 0x69, 0x6c, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44,
0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20,
0x49, 0x73, 0x72, 0x61, 0x61, 0x69, 0x69, 0x6c, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4a, 0x61, 0x62, 0x61, 0x61, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x61, 0x62, 0x61,
0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68,
0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4a,
0x61, 0x62, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67,
0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75,
0x6c, 0x6b, 0x61, 0x20, 0x65, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x57, 0x61, 0x6b, 0x68, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42,
0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x68,
0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x66, 0x75, 0x72, 0x74, 0x61, 0x20,
0x4b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x75, 0x72, 0x69,
0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x4b, 0x75, 0x75, 0x72, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61,
0x20, 0x65, 0x65, 0x20, 0x4b, 0x75, 0x75, 0x72, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x69,
0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x61, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x69, 0x6b, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x61, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4b, 0x69, 0x79, 0x72, 0x69, 0x67, 0x69, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4c, 0x65, 0x79,
0x6e, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4c, 0x6f, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4c, 0x6f, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65,
0x20, 0x4c, 0x6f, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x57, 0x61, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x65, 0x64, 0x61, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x65,
0x64, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58,
0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x65, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4d, 0x61, 0x6c, 0x65, 0x79, 0x73, 0x68, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x66, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x77, 0x65, 0x73, 0x61, 0x73, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x61, 0x61, 0x72, 0x73, 0x68, 0x61,
0x61, 0x6c, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73,
0x68, 0x69, 0x79, 0x61, 0x61, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x68, 0x69, 0x79,
0x61, 0x61, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58,
0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4d, 0x6f,
0x72, 0x69, 0x73, 0x68, 0x69, 0x79, 0x61, 0x61, 0x73, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x61, 0x64, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x42, 0x61, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x64, 0x61, 0x20,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61,
0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x61, 0x64, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x61, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x61, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20,
0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x61, 0x72,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x6b,
0x6f, 0x77, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4d, 0x61, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x61, 0x72,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e, 0x61, 0x77, 0x72,
0x6f, 0x6f, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e, 0x65,
0x65, 0x62, 0x61, 0x61, 0x6c, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x53, 0x69, 0x2019, 0x6c, 0x61,
0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x53, 0x69, 0x2019, 0x6c, 0x61, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61,
0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4e, 0x69,
0x79, 0x75, 0x75, 0x20, 0x53, 0x69, 0x2019, 0x6c, 0x61, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75,
0x66, 0x6f, 0x6f, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x66,
0x6f, 0x6f, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61,
0x20, 0x65, 0x65, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x66, 0x6f, 0x6f,
0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d,
0x61, 0x61, 0x6c, 0x69, 0x6e, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x61, 0x72,
0x6e, 0x61, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20,
0x65, 0x65, 0x20, 0x46, 0x61, 0x72, 0x6e, 0x61, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x46, 0x61, 0x72, 0x6e, 0x61,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x69, 0x6b,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4e, 0x6f,
0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67,
0x61, 0x20, 0x65, 0x65, 0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20,
0x65, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x42, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x42, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x6c, 0x61, 0x77, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x62, 0x75, 0x77,
0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x47, 0x69, 0x6e, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72,
0x61, 0x67, 0x77, 0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x61,
0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x61,
0x67, 0x77, 0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x42, 0x65, 0x65, 0x72, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x42, 0x65, 0x65, 0x72, 0x75, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x42, 0x65, 0x65, 0x72, 0x75, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x62, 0x69, 0x69,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x62, 0x69, 0x69, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x62, 0x69, 0x69, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x6f, 0x6f, 0x6e, 0x69, 0x6b,
0x69, 0x73, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x42,
0x65, 0x65, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x77,
0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x53,
0x74, 0x2e, 0x20, 0x42, 0x65, 0x65, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d,
0x69, 0x6b, 0x69, 0x77, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74,
0x61, 0x20, 0x65, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x42, 0x65, 0x65,
0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x77, 0x65, 0x6c,
0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x69,
0x74, 0x6b, 0x65, 0x65, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x42, 0x6f, 0x6e, 0x61, 0x62, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x42, 0x6f, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e,
0x67, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x51, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68,
0x20, 0x65, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x53, 0x61, 0x61, 0x63, 0x61, 0x64, 0x61, 0x20, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x61, 0x79, 0x6c, 0x61,
0x68, 0x61, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x52, 0x69, 0x79, 0x75,
0x75, 0x6e, 0x69, 0x79, 0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b,
0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67,
0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6b, 0x61, 0x20, 0x65, 0x65, 0x20, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72,
0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x69, 0x69, 0x73, 0x68,
0x61, 0x6c, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x62, 0x75, 0x75, 0x72, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x6f, 0x77, 0x74, 0x20, 0x4a, 0x6f,
0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x65, 0x79, 0x6d, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x69, 0x79, 0x6f, 0x77, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x65,
0x79, 0x62, 0x65, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x54, 0x65, 0x79, 0x62, 0x65, 0x79, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74,
0x61, 0x20, 0x65, 0x65, 0x20, 0x54, 0x65, 0x79, 0x62, 0x65, 0x79, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x75, 0x75, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x75, 0x75, 0x6b,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x75,
0x66, 0x61, 0x6c, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61,
0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61,
0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x55, 0x72, 0x75,
0x67, 0x77, 0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x55, 0x73, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x61, 0x6e, 0x75,
0x75, 0x74, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x46, 0x61, 0x6e, 0x75, 0x75, 0x74, 0x75, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x46, 0x61, 0x6e, 0x75, 0x75, 0x74, 0x75, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x77, 0x65, 0x65, 0x6c, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x46, 0x61, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x46,
0x61, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x61, 0x64, 0x69,
0x66, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x46, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x46, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x46, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x57, 0x61, 0x79, 0x6b,
0x20, 0x49, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x69, 0x73, 0x20, 0x26,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x79, 0x61, 0x64, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x61, 0x67, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x61, 0x67, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x61, 0x61, 0x67, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41,
0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x63, 0x72,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75, 0x64, 0xe1, 0x66, 0x72,
0x69, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74,
0x61, 0xf1, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x61, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x61, 0xf1, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x4d,
0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f,
0x73, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63,
0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x68, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74,
0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62,
0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e,
0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74,
0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x79,
0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x79, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x79, 0xe1, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0xe9, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0xe9, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x76,
0x69, 0x64, 0x61, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x63, 0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x6d, 0x65, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65,
0x78, 0x74, 0x72, 0x65, 0x6d, 0x6f, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x79, 0x69, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x79, 0x69, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x79, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x61,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69,
0x64, 0x61, 0x20, 0x79, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f,
0x72, 0x69, 0x6f, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x65, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6c, 0xe1,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69,
0x63, 0x68, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0xe1, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0xe1, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x48, 0x61, 0x77, 0xe1, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x63, 0xe9, 0x61, 0x6e, 0x6f, 0x20,
0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0xf3,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x72, 0x69, 0x20,
0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x73, 0x20, 0x45, 0x73, 0x70, 0xf3, 0x72, 0x61, 0x64,
0x61, 0x73, 0x20, 0x45, 0x63, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61,
0x6c, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x63, 0x61, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x63, 0x61, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c,
0xe1, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0xfa, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xfa, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xfa, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x75, 0x65, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75,
0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f,
0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72,
0x61, 0x6e, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x69, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x6f, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0xfa, 0x61, 0x20,
0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x65, 0x72, 0xfa, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20,
0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e,
0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x79, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0xed,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0xed,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c,
0xed, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65, 0x79, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x61, 0x79, 0x69, 0x6b, 0x69, 0x73, 0x74, 0xe1,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x79, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x75, 0x6b, 0xf3,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0xf1, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c,
0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x6d, 0x65, 0x64, 0x69, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45,
0x73, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x6c, 0x65, 0x6a, 0x61, 0x6e, 0x6f, 0x20, 0x65, 0x73,
0x74, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x54, 0x69, 0x65, 0x72,
0x72, 0x61, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x73, 0x20, 0x79, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69, 0x63,
0x61, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20,
0x28, 0x42, 0x69, 0x72, 0x6d, 0x61, 0x6e, 0x69, 0x61, 0x29, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x69,
0x6f, 0x6e, 0x79, 0x61, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x61, 0x76, 0x69, 0x64, 0x61, 0x64, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76,
0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x63, 0xe9, 0x61, 0x6e, 0x6f, 0x20,
0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c,
0x61, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x57, 0xe9, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x57, 0xe9, 0x74, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x57, 0xe9, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x67, 0x75, 0x6e, 0x75, 0x6e,
0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x67, 0x75, 0x6e, 0x75, 0x6e,
0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x61, 0x67, 0x75,
0x6e, 0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73,
0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0xc9, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0xc9, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61,
0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4b,
0x75, 0x73, 0x69, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61,
0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69,
0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d, 0x61,
0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61,
0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61,
0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73,
0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x69, 0x61, 0x6e, 0x67, 0x61, 0x7a, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x70,
0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x61, 0x72,
0x61, 0x62, 0x75, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x61, 0x72, 0x61, 0x62, 0x75, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68,
0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d,
0x77, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d,
0x77, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x79, 0x61,
0x20, 0x4b, 0x61, 0x74, 0x69, 0x20, 0x79, 0x61, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x79, 0x61,
0x20, 0x4b, 0x61, 0x74, 0x69, 0x20, 0x79, 0x61, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x79, 0x61, 0x20,
0x4b, 0x61, 0x74, 0x69, 0x20, 0x79, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0x61,
0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6d,
0x77, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69,
0x6b, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0x61, 0x67,
0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0x61,
0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0x61,
0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x43,
0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x61,
0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61,
0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x55, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x55, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x55, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20,
0x4b, 0x72, 0x69, 0x73, 0x6d, 0x61, 0x73, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79,
0x61, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76,
0x79, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x6e, 0x75, 0x73, 0x75, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69,
0x73, 0x69, 0x77, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x79,
0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x75, 0x62, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x44, 0x61,
0x76, 0x69, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77,
0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61,
0x20, 0x63, 0x68, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x45, 0x6b, 0x77, 0x61, 0x64, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61,
0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c,
0x61, 0x79, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72,
0x69, 0x6b, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61,
0x72, 0x69, 0x6b, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61,
0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69,
0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69,
0x6b, 0x69, 0x20, 0x7a, 0x61, 0x69, 0x64, 0x69, 0x20, 0x6d, 0x77, 0x61,
0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d,
0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20,
0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20,
0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73,
0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x79, 0x61, 0x20, 0x55,
0x66, 0x61, 0x72, 0x61, 0x6e, 0x73, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x75, 0x73, 0x69, 0x6e, 0x69, 0x20, 0x6d, 0x77, 0x61,
0x20, 0x55, 0x66, 0x61, 0x72, 0x61, 0x6e, 0x73, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x6b, 0x74, 0x69, 0x6b, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a,
0x6f, 0x6a, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x6f,
0x6a, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x4a, 0x6f, 0x6a, 0x69, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76,
0x79, 0x61, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69,
0x63, 0x68, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61,
0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x73, 0x68,
0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72,
0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61,
0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x72, 0x69, 0x20, 0x48, 0x69,
0x6e, 0x64, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x20,
0x6d, 0x77, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68,
0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63,
0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63,
0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x61,
0x6e, 0x67, 0x61, 0x7a, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x73, 0x68,
0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x4d,
0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x79, 0x72, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79,
0x61, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61,
0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6c,
0x61, 0x79, 0x73, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77,
0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x72,
0x69, 0x73, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x72,
0x69, 0x73, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x77, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x6d, 0x63, 0x68,
0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63,
0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63,
0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x50, 0x65, 0x72, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x66, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x66, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69,
0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x6e, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x79, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x79, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x61, 0x6e,
0x67, 0x61, 0x7a, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x55, 0x73, 0x68, 0x65, 0x6c, 0x69, 0x73, 0x68, 0x65, 0x6c,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56,
0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x53, 0x6f,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x4b, 0x75, 0x73, 0x69,
0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x6d, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77,
0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20,
0x6e, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41,
0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20,
0x6d, 0x77, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72,
0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69,
0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72,
0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x69, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61,
0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68,
0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b,
0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63,
0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68,
0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b,
0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a,
0x61, 0x62, 0x61, 0x6a, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x7a, 0x61, 0x62, 0x61, 0x6a, 0x61, 0x6e, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41,
0x7a, 0x61, 0x62, 0x61, 0x6a, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x75, 0x74,
0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x42,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61, 0x7a, 0x69,
0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x65, 0x70,
0x75, 0x76, 0x65, 0x64, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x76, 0x65, 0x64, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x65, 0x70, 0x75,
0x76, 0x65, 0x64, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x4e, 0x75, 0x73, 0x75, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77,
0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73,
0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x20, 0x79, 0x61, 0x20,
0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69,
0x6b, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d,
0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4a, 0x69, 0x6f, 0x6a, 0x69, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x4a, 0x69, 0x6f, 0x6a, 0x69, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x69,
0x6f, 0x6a, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x68, 0x75, 0x62, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61,
0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72,
0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x65, 0x73, 0x69,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x61, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x61, 0x72, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x77, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e,
0x65, 0x70, 0x61, 0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4d, 0x70,
0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4d, 0x70, 0x79, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4d, 0x70, 0x79,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x79, 0x75, 0x7a,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x4e, 0x79, 0x75, 0x7a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4e, 0x79, 0x75, 0x7a, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x66, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20,
0x46, 0x69, 0x6e, 0x69, 0x6b, 0x73, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x6f, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a,
0x6f, 0x6a, 0x69, 0x61, 0x20, 0x4b, 0x75, 0x73, 0x69, 0x6e, 0x69, 0x53,
0x61, 0x61, 0x7a, 0x61, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x77, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x76, 0xe4, 0x73, 0x74, 0x62, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73,
0x74, 0x62, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4,
0x73, 0x74, 0x62, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73,
0x79, 0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x74, 0x69, 0x64, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73,
0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2c, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x4b, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x67, 0x61, 0x20, 0x62, 0x65, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x64, 0x4b, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x67, 0x61, 0x20, 0x62, 0x65, 0x72, 0x67, 0x65, 0x6e, 0x2c, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x6c, 0x69, 0x70,
0x70, 0x69, 0x67, 0x61, 0x20, 0x62, 0x65, 0x72, 0x67, 0x65, 0x6e, 0x2c,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76, 0xe4,
0x73, 0x74, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x76, 0xe4, 0x73, 0x74, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x74, 0x69,
0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61, 0x74,
0x69, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2c, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61,
0x75, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6,
0x62, 0x65, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6,
0x62, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73,
0x61, 0x75, 0x64, 0x69, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x73, 0x61, 0x75, 0x64, 0x69, 0x61, 0x72, 0x61, 0x62,
0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x73, 0x61, 0x75, 0x64, 0x69, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0xf6,
0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x61,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x74, 0x69, 0x64, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x61,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76,
0xe4, 0x73, 0x74, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x76, 0xe4, 0x73, 0x74, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0xf6, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76,
0xe4, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76,
0xe4, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a, 0x6a, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a,
0x64, 0x7a, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a,
0x64, 0x7a, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x7a, 0x6f, 0x72, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x61, 0x7a, 0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x7a, 0x6f,
0x72, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x2c, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x74, 0x69,
0x64, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x74, 0x69,
0x64, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2c, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x70,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x43, 0x61, 0x73, 0x65, 0x79, 0x74, 0x69,
0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x74, 0x69, 0x64, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4a,
0x75, 0x6c, 0xf6, 0x6e, 0x73, 0x20, 0x74, 0x69, 0x64, 0x4b, 0x65, 0x65,
0x6c, 0x69, 0x6e, 0x67, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74,
0x69, 0x64, 0x63, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x43,
0x6f, 0x6f, 0x6b, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69,
0x64, 0x43, 0x6f, 0x6f, 0x6b, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x6f, 0x6f,
0x6b, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x74, 0x69,
0x64, 0xf6, 0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b, 0xf6, 0x74, 0x69, 0x64,
0x50, 0xe5, 0x73, 0x6b, 0xf6, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b, 0xf6, 0x6e, 0x2c,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0xf6, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0xf6, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0xf6, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x61,
0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x74, 0x69, 0x64,
0x76, 0xe4, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf6, 0x61, 0x72, 0x6e, 0x61,
0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf6, 0x61, 0x72, 0x6e,
0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x46, 0x69, 0x6a, 0x69, 0x74, 0x69, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x2c,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46, 0x69,
0x6a, 0x69, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x74, 0x69, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x6b,
0x61, 0x20, 0x53, 0x79, 0x64, 0x74, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f,
0x72, 0x69, 0x65, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x47,
0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x74, 0x69, 0x64, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x69, 0x72,
0x69, 0x62, 0x61, 0x74, 0x69, 0x74, 0x69, 0x64, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x67,
0x72, 0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0xf6, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0xf6, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76,
0xe4, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x67, 0x72, 0xf6,
0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x67, 0x72, 0xf6,
0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x47, 0x75, 0x61, 0x6d, 0x74, 0x69, 0x64,
0x50, 0x65, 0x72, 0x73, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x76, 0x69, 0x6b,
0x65, 0x6e, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c,
0x75, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75,
0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x48,
0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x2c, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f,
0x6e, 0x67, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2c, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x43, 0x68, 0x6f, 0x76,
0x64, 0x74, 0x69, 0x64, 0x43, 0x68, 0x6f, 0x76, 0x64, 0x2c, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x68, 0x6f, 0x76,
0x64, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x42, 0x72, 0x69, 0x74, 0x74, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x6f, 0x63, 0x65, 0x61, 0x6e, 0xf6,
0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76,
0xe4, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x74, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x69,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x6d, 0x74, 0x6a, 0x61,
0x74, 0x6b, 0x61, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x6d, 0x74, 0x6a, 0x61,
0x74, 0x6b, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x4b, 0x61, 0x6d, 0x74, 0x6a, 0x61, 0x74, 0x6b, 0x61, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6b, 0x61, 0x7a, 0x61, 0x6b, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74,
0x6b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x6b, 0x61, 0x7a, 0x61, 0x6b,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6b, 0x69, 0x72, 0x67, 0x69, 0x7a,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x53, 0x72, 0x69, 0x20, 0x4c,
0x61, 0x6e, 0x6b, 0x61, 0x74, 0x69, 0x64, 0x4c, 0x69, 0x6e, 0x65, 0xf6,
0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63,
0x61, 0x6f, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d,
0x61, 0x63, 0x61, 0x6f, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2c, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x74,
0x69, 0x64, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x74,
0x69, 0x64, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0xf6, 0x61,
0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x6d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x74,
0x69, 0x6c, 0x6c, 0x61, 0x68, 0x61, 0x76, 0x73, 0x74, 0x69, 0x64, 0x6d,
0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x69,
0x6c, 0x6c, 0x61, 0x68, 0x61, 0x76, 0x73, 0x74, 0x69, 0x64, 0x2c, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6d, 0x65, 0x78,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c,
0x61, 0x68, 0x61, 0x76, 0x73, 0x74, 0x69, 0x64, 0x2c, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2c, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x74,
0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x62, 0x75, 0x72, 0x6d, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x74, 0x69, 0x64, 0x6e, 0x65, 0x70,
0x61, 0x6c, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4e,
0x79, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65,
0x6e, 0x74, 0x69, 0x64, 0x4e, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65, 0x65, 0x6c,
0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a,
0x65, 0x65, 0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65, 0x65, 0x6c,
0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf6, 0x6e, 0x73, 0x20,
0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf6, 0x6e,
0x73, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf6, 0x6e, 0x73, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x74, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x64, 0x6d, 0x61, 0x72,
0x69, 0x61, 0x6e, 0x65, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b,
0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x74, 0x69, 0x64, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e,
0x79, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x73, 0x20, 0x74,
0x69, 0x64, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x75, 0x72, 0x79, 0x74,
0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x74, 0x69,
0x64, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x74, 0x69,
0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x74, 0x69,
0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x74, 0x69, 0x64,
0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2c, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x53, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x74, 0x69, 0x64, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c,
0x6c, 0x65, 0x72, 0x6e, 0x61, 0x74, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x6f, 0x72, 0x65, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69,
0x64, 0x73, 0x79, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x74,
0x69, 0x64, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x74, 0x69, 0x64, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x74, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x74, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2c, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x54, 0x61, 0x64, 0x7a, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x74, 0x69, 0x64, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0x69, 0x64, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x74, 0x69, 0x64, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x74, 0x69, 0x64,
0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x74, 0x69, 0x64, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2c,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2c,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x57, 0x61,
0x6b, 0x65, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6f, 0x63, 0x68, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73,
0x20, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x74, 0x69, 0x64,
0x41, 0x63, 0x72, 0x65, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x41, 0x63, 0x72,
0x65, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x7a, 0x69, 0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x66, 0x67, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0x69, 0x69,
0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x5a, 0x69, 0x69,
0x74, 0x4f, 0x73, 0x63, 0x68, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x53,
0xfc, 0xfc, 0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x69, 0x20, 0x7a, 0x69, 0x69, 0x74, 0x57, 0x65, 0x73, 0x63,
0x68, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68,
0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x69, 0x20, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x7a, 0x69, 0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x5a, 0x69, 0x69, 0x74,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x53, 0x63, 0x68,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x69, 0x74, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x2d, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x5a,
0x69, 0x69, 0x74, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x53, 0x63, 0x68, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69,
0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x4d,
0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69,
0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x7a, 0x69, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65,
0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x4f,
0x73, 0x63, 0x68, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73,
0x63, 0x68, 0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x4f, 0x73, 0x63, 0x68,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69,
0x20, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a,
0x69, 0x69, 0x74, 0x4f, 0x73, 0x63, 0x68, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20,
0x5a, 0x69, 0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x63,
0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x69, 0x74,
0x57, 0x65, 0x73, 0x63, 0x68, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4,
0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x69, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72,
0x20, 0x5a, 0x69, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65,
0x72, 0x20, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x7a, 0x69, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x725,
0x715, 0x722, 0x718, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x71f, 0x71d, 0x72a,
0x725, 0x715, 0x722, 0x718, 0x72c, 0x710, 0x20, 0x72b, 0x72a, 0x72b, 0x71d, 0x72c,
0x710, 0x20, 0x725, 0x715, 0x722, 0x718, 0x72c, 0x710, 0x20, 0x72b, 0x72a, 0x72b,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x71f, 0x71d, 0x72a, 0x725, 0x715,
0x722, 0x718, 0x72c, 0x710, 0x20, 0x729, 0x71d, 0x72c, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x710, 0x71d, 0x71f, 0x71d, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x710, 0x726, 0x713, 0x710, 0x722, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x20, 0x710, 0x726,
0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x715,
0x722, 0x71a, 0x20, 0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x72c, 0x71d, 0x721, 0x722, 0x20, 0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x20, 0x710,
0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a,
0x712, 0x20, 0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x729, 0x71d, 0x72c, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a,
0x712, 0x20, 0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x710, 0x720, 0x710, 0x723, 0x71f, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x710, 0x720, 0x710, 0x723, 0x71f, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x720,
0x710, 0x723, 0x71f, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x720,
0x721, 0x710, 0x72c, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x720, 0x721, 0x710,
0x72c, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x720, 0x721, 0x710, 0x72c, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x721, 0x719, 0x722, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x721, 0x719, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x721,
0x719, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a,
0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713,
0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x721, 0x728, 0x725,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20,
0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a,
0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x71b, 0x718, 0x72a, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713,
0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x71b, 0x718, 0x72a,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20,
0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x71b, 0x718,
0x72a, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710,
0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a,
0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721,
0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x722, 0x710, 0x715, 0x71d,
0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x722, 0x710, 0x715, 0x71d, 0x72a, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x710, 0x722, 0x710, 0x715, 0x71d, 0x72a, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x710, 0x726, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x726, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x726, 0x71d, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x710, 0x71f, 0x72c, 0x710, 0x718, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x710, 0x71f, 0x72c, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71f,
0x72c, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x71f, 0x72c,
0x718, 0x712, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71f, 0x72c, 0x718, 0x712, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x710, 0x71f, 0x72c, 0x718, 0x712, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x710, 0x72a, 0x712, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x72a, 0x712, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x712, 0x71d,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c,
0x71d, 0x722, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c,
0x71d, 0x722, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c,
0x71d, 0x722, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713,
0x722, 0x72c, 0x71d, 0x722, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722,
0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722, 0x710, 0x20, 0x721, 0x725,
0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710,
0x72a, 0x721, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x721,
0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x721, 0x722, 0x71d,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x710, 0x71b, 0x720, 0x722, 0x71b, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713,
0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x710, 0x71b, 0x720,
0x722, 0x71b, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f,
0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x710, 0x71b, 0x720, 0x722, 0x71b, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721,
0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720,
0x71d, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x728,
0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c,
0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x20,
0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c,
0x72a, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d,
0x72c, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x721, 0x725,
0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710,
0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x715, 0x722,
0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c,
0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720,
0x71d, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721,
0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718,
0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d,
0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x719, 0x72a, 0x712,
0x71d, 0x713, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x719, 0x72a, 0x712,
0x71d, 0x713, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x719, 0x72a, 0x712,
0x71d, 0x713, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x719,
0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x719, 0x718, 0x72a, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x710, 0x719, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x722, 0x713, 0x720, 0x710, 0x715, 0x71d, 0x72b, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x712, 0x722, 0x713, 0x720, 0x710, 0x715, 0x71d, 0x72b, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715,
0x712, 0x722, 0x713, 0x720, 0x710, 0x715, 0x71d, 0x72b, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x718, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x718, 0x720, 0x71d, 0x712, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x72a, 0x710, 0x723, 0x71d, 0x720, 0x71d, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x712, 0x72a, 0x710, 0x723, 0x71d, 0x720, 0x71d, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x712, 0x72a, 0x710, 0x723, 0x71d, 0x720, 0x71d, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x72a, 0x718, 0x722, 0x710, 0x71d, 0x20, 0x715,
0x710, 0x72a, 0x718, 0x723, 0x710, 0x720, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x71f, 0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a, 0x715, 0x71d, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x71f, 0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a, 0x715, 0x71d, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x71f, 0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a, 0x715, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71f, 0x710, 0x71d, 0x723, 0x71d, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x72c, 0x72b, 0x710, 0x721, 0x718, 0x72a, 0x718, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x72c, 0x72b, 0x710, 0x72c, 0x721, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x72c, 0x72b, 0x710, 0x72c, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x72c, 0x72b,
0x710, 0x72c, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x72b, 0x71d,
0x720, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c, 0x72b, 0x71d, 0x720, 0x71d, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x72c, 0x72b, 0x71d, 0x720, 0x71d, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x728, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x728, 0x71d, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x728, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x72a, 0x71d, 0x723,
0x72c, 0x721, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d,
0x716, 0x308, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x71f, 0x718, 0x723, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x720, 0x718, 0x721, 0x712, 0x71d, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x720, 0x718, 0x721, 0x712, 0x71d, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x720, 0x718, 0x721, 0x712, 0x71d, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20,
0x715, 0x71f, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x713, 0x719, 0x71d, 0x72a, 0x308,
0x710, 0x20, 0x715, 0x71f, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x713, 0x719, 0x71d,
0x72a, 0x308, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x71f, 0x718, 0x712, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x718,
0x712, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x712, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x715, 0x712, 0x71d, 0x723, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x715, 0x718, 0x721, 0x718, 0x722, 0x72c, 0x20, 0x715, 0x710, 0x718,
0x72a, 0x712, 0x71d, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x715,
0x722, 0x71a, 0x20, 0x72c, 0x71d, 0x721, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x726, 0x728, 0x71a, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x726, 0x728,
0x71a, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20,
0x726, 0x728, 0x71a, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x729,
0x718, 0x710, 0x715, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710,
0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721,
0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a,
0x718, 0x726, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721,
0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718,
0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721,
0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x28, 0x717, 0x721, 0x20, 0x721,
0x715, 0x722, 0x71a, 0x710, 0x29, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710,
0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20,
0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710,
0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308,
0x710, 0x20, 0x715, 0x726, 0x720, 0x71f, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x726, 0x720, 0x71f,
0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308,
0x710, 0x20, 0x715, 0x726, 0x720, 0x71f, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x726, 0x71d, 0x713, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726,
0x71d, 0x713, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x713, 0x71d, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x718, 0x71d, 0x710, 0x722, 0x710, 0x20,
0x726, 0x72a, 0x722, 0x723, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x726, 0x72a, 0x722, 0x723, 0x710, 0x20, 0x72c, 0x71d, 0x721, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x718, 0x710, 0x722, 0x72c, 0x72a, 0x72c, 0x71d, 0x71f, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x710, 0x720, 0x726, 0x710, 0x713,
0x718, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x721, 0x712, 0x71d,
0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x718, 0x72a, 0x713, 0x71d,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x713, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x713, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x713,
0x71d, 0x720, 0x712, 0x72a, 0x72c, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713,
0x72a, 0x71d, 0x722, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x713, 0x72a, 0x71d,
0x722, 0x720, 0x722, 0x715, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x713, 0x72a, 0x71d, 0x722, 0x720, 0x722, 0x715, 0x20, 0x721,
0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x713, 0x72a, 0x71d,
0x722, 0x720, 0x722, 0x715, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x72a, 0x71d, 0x722, 0x720, 0x722,
0x715, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x713, 0x72a, 0x71d, 0x722, 0x720, 0x722, 0x715, 0x20, 0x715, 0x721,
0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x713, 0x72a,
0x71d, 0x722, 0x720, 0x722, 0x715, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x71d,
0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x713, 0x718, 0x710, 0x721, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x721, 0x725, 0x720, 0x722, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x717, 0x710, 0x718, 0x710, 0x71d, 0x71d, 0x20, 0x710, 0x720, 0x718, 0x72b,
0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x717, 0x710, 0x718, 0x710, 0x71d, 0x71d,
0x20, 0x710, 0x720, 0x718, 0x72b, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x717,
0x710, 0x718, 0x710, 0x71d, 0x71d, 0x20, 0x710, 0x720, 0x718, 0x72b, 0x71d, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x717, 0x718, 0x722, 0x713, 0x20, 0x71f,
0x718, 0x722, 0x713, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x717, 0x718, 0x722, 0x713, 0x20,
0x71f, 0x718, 0x722, 0x713, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x717, 0x718, 0x722, 0x713,
0x20, 0x71f, 0x718, 0x722, 0x713, 0x715, 0x717, 0x718, 0x712, 0x715, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x717, 0x718, 0x712, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x717, 0x718,
0x712, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x717, 0x722, 0x715, 0x718, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x729, 0x71d, 0x722, 0x718, 0x723, 0x20,
0x717, 0x722, 0x715, 0x718, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x717, 0x722, 0x715, 0x718, 0x728, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x717, 0x722, 0x715, 0x718, 0x728, 0x71d, 0x722, 0x20, 0x721, 0x728, 0x725,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x717, 0x722, 0x715,
0x718, 0x728, 0x71d, 0x722, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x717, 0x722, 0x715, 0x718, 0x728, 0x71d,
0x722, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x710, 0x71d, 0x72a, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x71f,
0x718, 0x72c, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x71f,
0x718, 0x72c, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x71f,
0x718, 0x72c, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x71d,
0x723, 0x72a, 0x710, 0x71d, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x723,
0x72a, 0x710, 0x71d, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x723, 0x72a,
0x710, 0x71d, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71d, 0x726, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x71d, 0x726, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71d,
0x726, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72c, 0x72a,
0x718, 0x726, 0x712, 0x720, 0x712, 0x723, 0x71f, 0x71d, 0x2d, 0x71f, 0x710, 0x721,
0x71f, 0x72c, 0x723, 0x71f, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72c,
0x72a, 0x718, 0x726, 0x712, 0x720, 0x712, 0x723, 0x71f, 0x71d, 0x2d, 0x71f, 0x710,
0x721, 0x71f, 0x72c, 0x723, 0x71f, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x71d,
0x72c, 0x72a, 0x718, 0x726, 0x712, 0x720, 0x712, 0x723, 0x71f, 0x71d, 0x2d, 0x71f,
0x710, 0x721, 0x71f, 0x72c, 0x723, 0x71f, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x721, 0x715, 0x722, 0x71a, 0x20, 0x71f, 0x719, 0x729, 0x723, 0x72c, 0x710,
0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x20,
0x71f, 0x719, 0x729, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x71f, 0x718, 0x72a, 0x71d, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f,
0x718, 0x72a, 0x71d, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x72a,
0x71d, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x71f, 0x718, 0x723, 0x72a,
0x710, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71f, 0x72a, 0x710, 0x723,
0x722, 0x718, 0x71d, 0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x72a,
0x710, 0x723, 0x722, 0x718, 0x71d, 0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715,
0x71f, 0x72a, 0x710, 0x723, 0x722, 0x718, 0x71d, 0x72a, 0x723, 0x71f, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x729, 0x71d, 0x72a, 0x713, 0x71d, 0x719, 0x723, 0x72c,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x720, 0x710, 0x722, 0x71f,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308,
0x710, 0x20, 0x715, 0x720, 0x710, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x720, 0x718, 0x72a, 0x715, 0x20, 0x717, 0x710, 0x718, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x720, 0x718, 0x72a, 0x715, 0x20, 0x717, 0x710, 0x718, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x720, 0x718, 0x72a, 0x715, 0x20, 0x717, 0x710, 0x718, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x721, 0x710, 0x71f, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x721, 0x710, 0x71f, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x721, 0x710, 0x71f,
0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x710, 0x713, 0x710,
0x715, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x710, 0x713, 0x715, 0x710,
0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x721, 0x710, 0x713, 0x715, 0x710, 0x722, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x720, 0x71d, 0x719, 0x71d, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x721,
0x710, 0x720, 0x715, 0x71d, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x721, 0x710, 0x72a, 0x71f, 0x710, 0x718, 0x723, 0x710, 0x723, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20,
0x715, 0x721, 0x710, 0x72a, 0x72b, 0x710, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x721, 0x718, 0x72a, 0x71d, 0x71b, 0x71d, 0x718, 0x723, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x721, 0x718, 0x72a, 0x71d, 0x71b, 0x71d, 0x718, 0x723, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x721, 0x718, 0x72a, 0x71d, 0x71b, 0x71d, 0x718, 0x723, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x721, 0x710, 0x718, 0x723, 0x718, 0x722, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x721, 0x71f, 0x723, 0x71d, 0x729, 0x718, 0x20, 0x72b, 0x71d,
0x722, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x71f, 0x723, 0x71d,
0x729, 0x718, 0x20, 0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x721, 0x71f, 0x723, 0x71d, 0x729, 0x718, 0x20, 0x72b, 0x71d, 0x722, 0x71d,
0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x720, 0x710,
0x722, 0x712, 0x710, 0x72c, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718,
0x720, 0x710, 0x722, 0x712, 0x710, 0x72c, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715,
0x710, 0x718, 0x720, 0x710, 0x722, 0x712, 0x710, 0x72c, 0x718, 0x72a, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x721, 0x718, 0x723, 0x71f, 0x718, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x721, 0x718, 0x723, 0x71f, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x721, 0x718,
0x723, 0x71f, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x71d, 0x710,
0x722, 0x721, 0x710, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x710,
0x718, 0x72a, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x726,
0x710, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x20,
0x71f, 0x720, 0x71d, 0x715, 0x718, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x722, 0x71d, 0x718, 0x20, 0x71f, 0x720, 0x71d, 0x715, 0x718, 0x722, 0x71d, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x20, 0x71f, 0x720, 0x71d, 0x715,
0x718, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x71d,
0x718, 0x20, 0x719, 0x71d, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x722,
0x71d, 0x718, 0x20, 0x719, 0x71d, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715,
0x722, 0x71d, 0x718, 0x20, 0x719, 0x71d, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x726, 0x710, 0x718, 0x722, 0x720, 0x722,
0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x726, 0x710, 0x718, 0x722,
0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x726, 0x710,
0x718, 0x722, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722,
0x71d, 0x718, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x72a,
0x72c, 0x710, 0x20, 0x715, 0x722, 0x718, 0x72a, 0x726, 0x720, 0x71f, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x715, 0x722, 0x718, 0x72a,
0x726, 0x720, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710,
0x20, 0x715, 0x722, 0x718, 0x72a, 0x726, 0x720, 0x71f, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x726, 0x72a, 0x722, 0x722, 0x715, 0x718, 0x20, 0x715, 0x722, 0x718,
0x72a, 0x718, 0x722, 0x717, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x72a, 0x722,
0x722, 0x715, 0x718, 0x20, 0x715, 0x722, 0x718, 0x72a, 0x718, 0x722, 0x717, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x726, 0x72a, 0x722, 0x722, 0x715, 0x718, 0x20, 0x715,
0x722, 0x718, 0x72a, 0x718, 0x722, 0x717, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x713, 0x719, 0x71d, 0x716, 0x308, 0x710, 0x20, 0x715, 0x721, 0x72a, 0x71d,
0x710, 0x722, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x718, 0x712, 0x718, 0x723, 0x71d, 0x712,
0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x722, 0x718, 0x712, 0x718, 0x723,
0x71d, 0x712, 0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x722, 0x718, 0x712,
0x718, 0x723, 0x71d, 0x712, 0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x718, 0x721, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x718, 0x721, 0x723,
0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x718, 0x721, 0x723, 0x71f, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x726, 0x710, 0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x726, 0x710, 0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x726, 0x710, 0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x726, 0x720, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x726, 0x710, 0x726, 0x718, 0x710, 0x20, 0x713, 0x71d, 0x722, 0x71d, 0x710, 0x20,
0x71a, 0x715, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x72a,
0x713, 0x718, 0x710, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x72a, 0x713, 0x718,
0x710, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x72a, 0x713, 0x718, 0x710, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72a, 0x718, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x726, 0x71d, 0x72a, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x71d,
0x72a, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x720, 0x71d,
0x726, 0x71d, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x720,
0x71d, 0x726, 0x71d, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x71d,
0x720, 0x71d, 0x726, 0x71d, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x722,
0x71d, 0x71f, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x710, 0x722,
0x72c, 0x20, 0x726, 0x71d, 0x725, 0x72a, 0x20, 0x718, 0x721, 0x729, 0x718, 0x720,
0x718, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x723, 0x710, 0x722, 0x72c, 0x20, 0x726,
0x71d, 0x725, 0x72a, 0x20, 0x718, 0x721, 0x729, 0x718, 0x720, 0x718, 0x722, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x723, 0x710, 0x722, 0x72c, 0x20, 0x726, 0x71d, 0x725, 0x72a,
0x20, 0x718, 0x721, 0x729, 0x718, 0x720, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x726, 0x71d, 0x72c, 0x71f, 0x710, 0x71d, 0x72a, 0x722, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x726, 0x718, 0x722, 0x710, 0x726, 0x71d, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x718, 0x722, 0x713, 0x71d, 0x722, 0x713,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71f, 0x71d, 0x719, 0x71d, 0x720, 0x718,
0x72a, 0x715, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x71d, 0x719, 0x71d, 0x720,
0x718, 0x72a, 0x715, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71f, 0x71d, 0x719, 0x71d,
0x720, 0x718, 0x72a, 0x715, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72a,
0x71d, 0x718, 0x722, 0x71d, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x72a, 0x718, 0x72c, 0x71d, 0x72a, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x723, 0x710, 0x71a, 0x710, 0x720, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x723,
0x710, 0x71a, 0x710, 0x720, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x723, 0x710,
0x71a, 0x710, 0x720, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723,
0x721, 0x710, 0x72a, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x723, 0x721, 0x710, 0x72a,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x723, 0x721, 0x710, 0x72a, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x723, 0x721, 0x718, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x723, 0x721, 0x718, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x723, 0x721, 0x718, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x710, 0x71d, 0x72b, 0x71d, 0x720,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x723, 0x71d, 0x722, 0x713, 0x710, 0x726, 0x718, 0x72a,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710,
0x20, 0x715, 0x72b, 0x720, 0x71d, 0x721, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x713, 0x71d, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x20, 0x72c, 0x71d,
0x721, 0x722, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723,
0x718, 0x72a, 0x71d, 0x722, 0x710, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x723, 0x71d, 0x718, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x717,
0x71d, 0x72c, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x710, 0x71d,
0x726, 0x710, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c, 0x710, 0x71d, 0x726, 0x710,
0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x72c, 0x710, 0x71d, 0x726, 0x710, 0x71d, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x710, 0x713, 0x71d, 0x71f, 0x723, 0x72c,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x718, 0x71f, 0x71d,
0x720, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x72c, 0x718, 0x722, 0x713,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x72c, 0x718, 0x722, 0x713, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x72c, 0x718, 0x722, 0x713, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c,
0x72b, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x718, 0x72a,
0x71f, 0x721, 0x722, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c,
0x718, 0x72a, 0x71f, 0x721, 0x722, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x72c, 0x718, 0x72a, 0x71f, 0x721, 0x722, 0x723, 0x72c, 0x710, 0x722, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x718, 0x712, 0x710, 0x720, 0x718, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x713, 0x718, 0x710,
0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x713, 0x718, 0x710,
0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x713, 0x718, 0x710,
0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x719, 0x712, 0x71f,
0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x719, 0x712,
0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x719,
0x712, 0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x710, 0x722, 0x718, 0x710, 0x71b, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x712,
0x710, 0x722, 0x718, 0x710, 0x71b, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x712, 0x710,
0x722, 0x718, 0x710, 0x71b, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x722, 0x719, 0x718, 0x71d, 0x720, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x720, 0x710, 0x715, 0x71d, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x712, 0x720, 0x710, 0x715, 0x71d, 0x712, 0x718, 0x723, 0x72c,
0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x712, 0x720, 0x710, 0x715, 0x71d, 0x712,
0x718, 0x723, 0x72c, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x718, 0x720, 0x713, 0x718, 0x713, 0x72a, 0x710, 0x715, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x712, 0x718, 0x720, 0x713, 0x718, 0x713, 0x72a, 0x710, 0x715, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x712, 0x718, 0x720, 0x713, 0x718, 0x713, 0x72a, 0x710, 0x715, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715,
0x718, 0x710, 0x71d, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x718, 0x71d,
0x720, 0x71d, 0x723, 0x20, 0x718, 0x726, 0x718, 0x72c, 0x718, 0x722, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x71d, 0x710, 0x71f, 0x718, 0x72c, 0x723, 0x71f,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x71d, 0x710, 0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x71d, 0x710, 0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x71d, 0x71f, 0x710, 0x72c, 0x71d, 0x72a, 0x722, 0x712,
0x71d, 0x72a, 0x713, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71d, 0x71f, 0x710, 0x72c, 0x71d,
0x72a, 0x722, 0x712, 0x71d, 0x72a, 0x713, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71d, 0x71f,
0x710, 0x72c, 0x71d, 0x72a, 0x722, 0x712, 0x71d, 0x72a, 0x713, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x71d, 0x718, 0x729, 0x718, 0x722, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x410, 0x444, 0x493, 0x43e, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x49b, 0x43e,
0x438, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x49b, 0x43e, 0x438, 0x20, 0x428,
0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410, 0x444, 0x440,
0x438, 0x49b, 0x43e, 0x438, 0x20, 0x4b6, 0x430, 0x43d, 0x443, 0x431, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x49b, 0x43e, 0x438,
0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410,
0x444, 0x440, 0x438, 0x49b, 0x43e, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x430, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x49b, 0x43e, 0x438,
0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410,
0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440,
0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x410, 0x43b, 0x44f, 0x441, 0x43a,
0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x430, 0x438, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x448, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x43a, 0x4ef, 0x4b3, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x43a, 0x4ef, 0x4b3, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x43a, 0x4ef,
0x4b3, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x423, 0x49b, 0x451, 0x43d,
0x443, 0x441, 0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43c, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x423, 0x49b, 0x451, 0x43d, 0x443, 0x441, 0x438, 0x20, 0x41e, 0x440, 0x43e,
0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x423, 0x49b, 0x451, 0x43d, 0x443, 0x441, 0x438, 0x20, 0x41e,
0x440, 0x43e, 0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x43f, 0x438,
0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410, 0x43f, 0x438, 0x430, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20,
0x410, 0x43f, 0x438, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x440,
0x430, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20,
0x410, 0x440, 0x430, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x410,
0x440, 0x430, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x410, 0x440, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x410, 0x440, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x410, 0x440, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x430, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x43a, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x430, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x430, 0x442, 0x43b,
0x430, 0x43d, 0x442, 0x438, 0x43a, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x438, 0x20, 0x41c,
0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x438, 0x20, 0x41c, 0x430,
0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440,
0x4ef, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x438, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x438, 0x438,
0x20, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x438, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20,
0x492, 0x430, 0x440, 0x431, 0x438, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x438, 0x438, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x430,
0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x438, 0x20, 0x428, 0x430, 0x440, 0x49b, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x438, 0x438, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x448,
0x430, 0x440, 0x49b, 0x438, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x438, 0x20, 0x492,
0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41e, 0x437,
0x430, 0x440, 0x431, 0x43e, 0x439, 0x4b7, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x41e, 0x437, 0x430, 0x440, 0x431, 0x43e, 0x439, 0x4b7, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x41e, 0x437, 0x430, 0x440, 0x431, 0x43e, 0x439, 0x4b7,
0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x410, 0x437, 0x43e, 0x440,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438,
0x438, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x411, 0x443, 0x442, 0x430, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438,
0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x411, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x411, 0x440, 0x430, 0x437,
0x438, 0x43b, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e,
0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x411,
0x440, 0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x43e, 0x440, 0x443, 0x441, 0x441,
0x430, 0x43b, 0x43e, 0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41a, 0x430,
0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x435, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x427, 0x430, 0x43c, 0x43e, 0x440,
0x440, 0x43e, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x427, 0x430, 0x442, 0x430,
0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x427, 0x438, 0x43b, 0x438, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x427, 0x438, 0x43b,
0x438, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x427, 0x438, 0x43b, 0x438, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x427, 0x438, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20,
0x427, 0x438, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437,
0x43e, 0x43d, 0x430, 0x438, 0x20, 0x427, 0x438, 0x43d, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430, 0x438, 0x20, 0x41c, 0x430,
0x432, 0x43b, 0x443, 0x434, 0x438, 0x20, 0x418, 0x441, 0x43e, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e, 0x438,
0x20, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e,
0x438, 0x20, 0x41a, 0x443, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b6, 0x430,
0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x41a, 0x443, 0x43a, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x43d, 0x438, 0x43c, 0x430, 0x438, 0x20, 0x442,
0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x4b6, 0x430,
0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x41a, 0x443, 0x43a, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x41a, 0x443, 0x431, 0x430, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438,
0x438, 0x20, 0x41a, 0x443, 0x431, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41a, 0x443, 0x431, 0x430,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x414, 0x430, 0x432, 0x438, 0x441, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434,
0x2019, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x422, 0x438, 0x43c, 0x43e, 0x440, 0x438, 0x20, 0x428, 0x430, 0x440, 0x49b, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430,
0x438, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20,
0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430, 0x438, 0x20, 0x41f, 0x430, 0x441, 0x445,
0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430,
0x438, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x410, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x41c,
0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x430,
0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x43c, 0x430, 0x440, 0x43a,
0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443,
0x43f, 0x43e, 0x438, 0x438, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e,
0x438, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x448, 0x430,
0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443,
0x43f, 0x43e, 0x438, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x410, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x20,
0x428, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x430,
0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x493, 0x430, 0x440, 0x431,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e,
0x438, 0x438, 0x20, 0x493, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x493, 0x430,
0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x430, 0x437,
0x438, 0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x435, 0x43d, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438,
0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435,
0x43d, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440,
0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x424, 0x438, 0x4b7, 0x438, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x424, 0x438, 0x4b7, 0x438, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x424, 0x438, 0x4b7, 0x438, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x413,
0x432, 0x438, 0x430, 0x43d, 0x430, 0x438, 0x20, 0x424, 0x430, 0x440, 0x43e, 0x43d,
0x441, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b7, 0x430, 0x43d, 0x443,
0x431, 0x4e3, 0x20, 0x432, 0x430, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a,
0x442, 0x438, 0x434, 0x430, 0x438, 0x20, 0x424, 0x430, 0x440, 0x43e, 0x43d, 0x441,
0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x413, 0x430, 0x43b, 0x430, 0x43f,
0x430, 0x433, 0x43e, 0x441, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x413, 0x430,
0x43c, 0x431, 0x438, 0x440, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x413, 0x443,
0x440, 0x4b7, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20,
0x413, 0x443, 0x440, 0x4b7, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x413, 0x443, 0x440, 0x4b7, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3,
0x43e, 0x438, 0x20, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x43c, 0x438, 0x451, 0x43d, 0x430, 0x438, 0x20, 0x413,
0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x438, 0x20,
0x428, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x438, 0x20, 0x428, 0x430,
0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x438, 0x20, 0x428, 0x430, 0x440, 0x49b,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x425, 0x430, 0x43b, 0x438, 0x4b7, 0x438, 0x20, 0x424, 0x43e, 0x440, 0x441,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b2, 0x430, 0x432, 0x430, 0x439, 0x4e3,
0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20,
0x4b2, 0x430, 0x432, 0x430, 0x439, 0x4e3, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x4b2, 0x430, 0x432, 0x430, 0x439, 0x4e3, 0x2d, 0x410, 0x43b,
0x435, 0x443, 0x442, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b2, 0x43e,
0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b2, 0x43e,
0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442,
0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x4b2, 0x43e,
0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x425,
0x43e, 0x432, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b2, 0x43e, 0x432, 0x434,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x430, 0x438, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438,
0x438, 0x20, 0x4b2, 0x438, 0x43d, 0x434, 0x443, 0x441, 0x442, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x443, 0x49b, 0x451, 0x43d, 0x443, 0x441, 0x438,
0x20, 0x4b2, 0x438, 0x43d, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b2,
0x438, 0x43d, 0x434, 0x443, 0x447, 0x438, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x438, 0x20,
0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x448, 0x430, 0x440, 0x49b, 0x438, 0x438, 0x20, 0x418, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x418,
0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x438, 0x20, 0x492, 0x430,
0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x42d, 0x440, 0x43e,
0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x42d, 0x440, 0x43e, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20,
0x42d, 0x440, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x418, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x418, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442,
0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x418, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x418,
0x441, 0x440, 0x43e, 0x438, 0x43b, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x418, 0x441,
0x440, 0x43e, 0x438, 0x43b, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef,
0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x418, 0x441, 0x440, 0x43e, 0x438, 0x43b,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x43e, 0x43f, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x4b6, 0x43e, 0x43f, 0x43e, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x440, 0x430, 0x432, 0x448, 0x430, 0x43d, 0x438, 0x438, 0x20,
0x4b6, 0x43e, 0x43f, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x49a,
0x430, 0x437, 0x43e, 0x49b, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x49a, 0x430, 0x437, 0x43e, 0x49b, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x438, 0x20, 0x428, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x49a, 0x430, 0x437, 0x43e, 0x49b, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x438, 0x20, 0x492, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41a,
0x43e, 0x440, 0x435, 0x44f, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef,
0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f,
0x440, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41a,
0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x49a, 0x438, 0x440, 0x493, 0x438, 0x437, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438,
0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x41b, 0x438, 0x43d, 0x41b, 0x43e, 0x440,
0x434, 0x20, 0x425, 0x43e, 0x443, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x44f,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x441,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430,
0x4b3, 0x43e, 0x438, 0x20, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438,
0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a,
0x438, 0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x43a, 0x438, 0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41c, 0x430, 0x432,
0x441, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x423, 0x49b, 0x451,
0x43d, 0x443, 0x441, 0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43c, 0x20, 0x41c, 0x435,
0x43a, 0x441, 0x438, 0x43a, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x423, 0x49b,
0x451, 0x43d, 0x443, 0x441, 0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43c, 0x20, 0x41c,
0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x423, 0x49b, 0x451, 0x43d,
0x443, 0x441, 0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43c, 0x20, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x43a, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x423, 0x43b,
0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442,
0x43e, 0x440, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41c, 0x43e, 0x441, 0x43a,
0x432, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41c, 0x430, 0x441, 0x43a, 0x430,
0x432, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41c, 0x430, 0x441, 0x43a, 0x430, 0x432,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41c, 0x44f, 0x43d, 0x43c, 0x430, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f,
0x438, 0x20, 0x41d, 0x430, 0x432, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41a, 0x430,
0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x438, 0x20, 0x41d, 0x430, 0x432,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x438, 0x44f, 0x438, 0x20, 0x41d, 0x430, 0x432, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x438, 0x20, 0x41d,
0x430, 0x432, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x438, 0x20, 0x41d, 0x430, 0x432, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x417, 0x435,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x438, 0x20, 0x41d, 0x430, 0x432, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41d, 0x44e, 0x444,
0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41d, 0x44e,
0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x41d, 0x438, 0x443, 0x44d, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430, 0x438, 0x20, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4b7, 0x430, 0x437,
0x438, 0x440, 0x430, 0x438, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x4b7, 0x430, 0x437, 0x438, 0x440, 0x430, 0x438, 0x20, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x445, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x445, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x445, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x41e, 0x43c, 0x441, 0x43a, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41e, 0x43c, 0x441, 0x43a, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x41f, 0x43e, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x41f, 0x43e, 0x43a, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41f, 0x43e, 0x43a, 0x438, 0x441, 0x442,
0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41f, 0x430, 0x43b, 0x430,
0x443, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41f, 0x430, 0x43f, 0x443, 0x430,
0x20, 0x413, 0x432, 0x438, 0x43d, 0x435, 0x44f, 0x438, 0x20, 0x41d, 0x430, 0x432,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x439, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41f, 0x435,
0x440, 0x443, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430,
0x4b3, 0x43e, 0x438, 0x20, 0x424, 0x438, 0x43d, 0x438, 0x43a, 0x441, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x438, 0x435,
0x440, 0x20, 0x432, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x438,
0x435, 0x440, 0x20, 0x432, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d,
0x430, 0x438, 0x20, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x438, 0x435, 0x440,
0x20, 0x432, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x41f, 0x445, 0x435, 0x43d, 0x44f, 0x43d,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x420, 0x435, 0x44e, 0x43d, 0x438, 0x43e,
0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x420, 0x43e, 0x442, 0x435, 0x440,
0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x421, 0x430, 0x43c, 0x43e,
0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x421,
0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x4b6, 0x430, 0x437, 0x438, 0x440, 0x430, 0x4b3, 0x43e, 0x438, 0x20, 0x421,
0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x4b6, 0x43e, 0x440, 0x4b7, 0x438, 0x44f, 0x438, 0x20, 0x4b6, 0x430, 0x43d, 0x443,
0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x421, 0x443, 0x440, 0x438,
0x43d, 0x430, 0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x421, 0x451, 0x432,
0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x422, 0x430, 0x438, 0x442, 0x438,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x439,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x439, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x439, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x422, 0x43e, 0x4b7, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x427, 0x443, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x422, 0x443,
0x440, 0x43a, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430,
0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x438, 0x438, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x430, 0x43d, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x422, 0x443, 0x440, 0x43a,
0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438,
0x438, 0x20, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x4ee, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x4ee, 0x437, 0x431, 0x435, 0x43a, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442, 0x43e,
0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x4ee, 0x437, 0x431,
0x435, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438,
0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e,
0x441, 0x442, 0x43e, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433,
0x440, 0x430, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x442,
0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x412, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x431, 0x435, 0x434, 0x43e, 0x440, 0x438, 0x438, 0x20, 0x4b7, 0x430, 0x437,
0x438, 0x440, 0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x423, 0x43e, 0x43b,
0x43b, 0x438, 0x441, 0x20, 0x432, 0x430, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d,
0x430, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x42f, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4e3, 0x42f,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x415, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440,
0x433, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x42e, 0x43a, 0x43e, 0x43d, 0xb85,
0xb95, 0xbcd, 0xbb0, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb95,
0xbcd, 0xbb0, 0xbc7, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xb95, 0xbcd, 0xbb0, 0xbc7, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xb83, 0xbaa, 0xbcd, 0xb95, 0xbbe, 0xba9, 0xbbf,
0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf,
0xbb0, 0xbbf, 0xb95, 0xbcd, 0xb95, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95,
0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf,
0xbb0, 0xbbf, 0xb95, 0xbcd, 0xb95, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4,
0xbc6, 0xba9, 0xbcd, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb95,
0xbcd, 0xb95, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb86,
0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb95, 0xbcd, 0xb95, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb86, 0xbaa,
0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb95, 0xbcd, 0xb95, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7,
0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf,
0xb95, 0xbcd, 0xb95, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb85, 0xbb2, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb2, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xbb2, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbbe, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca,
0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb2, 0xbcd, 0xbae,
0xbbe, 0xb9f, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb2, 0xbcd,
0xbae, 0xbbe, 0xb9f, 0xbbf, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb85, 0xbb2, 0xbcd, 0xbae, 0xbbe, 0xb9f, 0xbbf, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbae, 0xbc7, 0xb9a, 0xbbe,
0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbae, 0xbc7, 0xb9a,
0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbae, 0xbc7, 0xb9a, 0xbbe, 0xba9, 0xbcd,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf,
0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf,
0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd,
0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95,
0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca,
0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbb5, 0xbc1, 0xba9,
0xbcd, 0xb9f, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xbb5, 0xbc1, 0xba9, 0xbcd, 0xb9f, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbb5,
0xbc1, 0xba9, 0xbcd, 0xb9f, 0xbc8, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca,
0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xb9a, 0xbbf, 0xbaa,
0xbbf, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xb9a, 0xbbf,
0xbaa, 0xbbf, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95,
0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb85, 0xba9, 0xb9f, 0xbc0, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb85, 0xba9, 0xbbe, 0xb9f, 0xbc8, 0xbaf, 0xbb0, 0xbcd, 0x20, 0xba4,
0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xba9, 0xbbe, 0xb9f, 0xbc8,
0xbaf, 0xbb0, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb8f, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb8f, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb8f, 0xbaa, 0xbbf, 0xbaf,
0xbbe, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcc, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcc, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcc, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcb,
0xbaa, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xb9f,
0xbcb, 0xbaa, 0xbc7, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcb, 0xbaa, 0xbc7, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb0, 0xbc7, 0xbaa, 0xbbf, 0xbaf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb0, 0xbc7, 0xbaa, 0xbbf, 0xbaf,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb85, 0xbb0, 0xbc7, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2,
0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb0, 0xbcd,
0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9,
0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf,
0xba9, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20,
0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f,
0xbbf, 0xba9, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f,
0xbbf, 0xba9, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb86, 0xbb0, 0xbcd, 0xbae, 0xbc7, 0xba9, 0xbbf, 0xbaf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb0, 0xbcd, 0xbae, 0xbc7, 0xba9, 0xbbf, 0xbaf,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb86, 0xbb0, 0xbcd, 0xbae, 0xbc7, 0xba9, 0xbbf, 0xbaf, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd,
0xbb2, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xb95,
0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbbf,
0xb95, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb86, 0xbb8,
0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf,
0xba9, 0xbcd, 0x20, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86,
0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20,
0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf,
0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xba4, 0xbcd, 0xba4,
0xbbf, 0xbaf, 0x20, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf,
0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf,
0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xba4, 0xbcd, 0xba4,
0xbbf, 0xbaf, 0x20, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf,
0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf,
0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbae,
0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95,
0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf,
0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb86, 0xbb8,
0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf,
0xba9, 0xbcd, 0x20, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4,
0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2,
0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4,
0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2,
0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4,
0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xbc7, 0xbb1,
0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd,
0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xbc7,
0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2,
0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a, 0xbb0,
0xbcd, 0xbaa, 0xbc8, 0xb9c, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb85, 0xb9a, 0xbb0, 0xbcd, 0xbaa, 0xbc8, 0xb9c, 0xbbe, 0xba9, 0xbcd, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb85, 0xb9a, 0xbb0, 0xbcd, 0xbaa, 0xbc8, 0xb9c, 0xbbe, 0xba9, 0xbcd, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a,
0xbcb, 0xbb0, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a,
0xbcb, 0xbb0, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a, 0xbcb, 0xbb0, 0xbcd, 0xbb8,
0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbb5, 0xb99, 0xbcd, 0xb95, 0xba4, 0xbc7, 0xb9a, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbb5, 0xb99, 0xbcd, 0xb95, 0xba4, 0xbc7, 0xb9a, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xb99,
0xbcd, 0xb95, 0xba4, 0xbc7, 0xb9a, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc2, 0xb9f, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbca, 0xbb2, 0xbbf, 0xbb5, 0xbbf, 0xbaf, 0xbbe,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbc7, 0xb9a, 0xbbf,
0xbb2, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf,
0xbb0, 0xbc7, 0xb9a, 0xbbf, 0xbb2, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf,
0xbb0, 0xbc7, 0xb9a, 0xbbf, 0xbb2, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc1, 0xbb0, 0xbc1, 0xba9,
0xbc7, 0x20, 0xb9f, 0xbb0, 0xbc1, 0xbb8, 0xbcd, 0xbb8, 0xbb2, 0xbbe, 0xbae, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc7, 0xbaa, 0xbcd, 0x20, 0xbb5,
0xbc6, 0xbb0, 0xbcd, 0xb9f, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95,
0xbc7, 0xbaa, 0xbcd, 0x20, 0xbb5, 0xbc6, 0xbb0, 0xbcd, 0xb9f, 0xbc7, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb95, 0xbc7, 0xbaa, 0xbcd, 0x20, 0xbb5, 0xbc6, 0xbb0, 0xbcd, 0xb9f, 0xbc7, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbe,
0xbae, 0xbcb, 0xbb0, 0xbcb, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbae,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xba4, 0xbcd, 0xba4, 0xbbe,
0xbae, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbae, 0xbcd, 0x20,
0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb9a, 0xbbf, 0xbb2, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbf,
0xbb2, 0xbbf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbf, 0xbb2, 0xbbf, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc0, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc0, 0xba9, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc0, 0xba9,
0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb95, 0xbbf, 0xbb1, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbc1, 0xbae, 0xbb8, 0xbcd,
0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95,
0xbcb, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb2, 0xbae, 0xbcd,
0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca,
0xbb2, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb2,
0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc1, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0,
0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95,
0xbc1, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb95, 0xbc1, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3,
0xbcd, 0x20, 0xb85, 0xbb0, 0xbc8, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xbaa, 0xbbe, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xbaa, 0xbbe, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb95, 0xbbf, 0xbaf, 0xbc2, 0xbaa, 0xbbe, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbc7, 0xbb5, 0xbbf, 0xbb8,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbc1, 0xbae, 0xbcb, 0xba3,
0xbcd, 0xb9f, 0xbcd, 0x2d, 0xb9f, 0xbbf, 0x20, 0xb89, 0xbb0, 0xbcd, 0xbb5, 0xbbf,
0xbb2, 0xbcd, 0xbb2, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf,
0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xba4, 0xbbf, 0xbae, 0xbcb, 0xbb0, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88, 0xbb8, 0xbcd, 0xb9f, 0xbb0, 0xbcd,
0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88,
0xbb8, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb88, 0xbb8, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88, 0xb95,
0xbcd, 0xbb5, 0xb9f, 0xbbe, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd,
0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb,
0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95,
0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa,
0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4,
0xbc2, 0xbb0, 0x2d, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb90,
0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20,
0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0,
0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbe, 0xb95, 0xbcd, 0xbb2, 0xbbe,
0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbe, 0xb95, 0xbcd, 0xbb2,
0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3,
0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbe, 0xb95, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd,
0xba4, 0xbc1, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbf,
0xb9c, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbf, 0xb9c,
0xbbf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbf, 0xb9c, 0xbbf, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbf, 0xbb0, 0xb9e,
0xbcd, 0xb9a, 0xbc1, 0x20, 0xb95, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbc6, 0xb9e, 0xbcd, 0xb9a, 0xbc1, 0x20,
0xba4, 0xbc6, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0x26,
0x20, 0xb85, 0xba3, 0xbcd, 0xb9f, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbb2, 0xbaa, 0xb95, 0xbcb, 0xbb8,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc7, 0xbae, 0xbcd, 0xbaa,
0xbbf, 0xbaf, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9c, 0xbbe,
0xbb0, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb9c, 0xbbe, 0xbb0, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9c, 0xbbe,
0xbb0, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb2, 0xbcd, 0xbaa, 0xbb0, 0xbcd,
0xb9f, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb5, 0xbbf,
0xb9a, 0xbcd, 0x20, 0xb9a, 0xbb0, 0xbbe, 0xb9a, 0xbb0, 0xbbf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95,
0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1,
0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4,
0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95,
0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7,
0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2,
0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xbc7, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd,
0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd,
0xb95, 0xbc1, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8,
0xbcd, 0xba4, 0xbc1, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb95, 0xbae, 0xbcd, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbb5, 0xbb3, 0xbc8, 0xb95, 0xbc1, 0xb9f, 0xbbe, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9,
0xbb5, 0xbbe, 0xbaf, 0xbcd, 0x2d, 0xb85, 0xbb2, 0xbc7, 0xbb7, 0xbbf, 0xbaf, 0xba9,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbb5, 0xbbe, 0xbaf, 0xbcd,
0x2d, 0xb85, 0xbb2, 0xbc7, 0xbb7, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9,
0xbb5, 0xbbe, 0xbaf, 0xbcd, 0x2d, 0xb85, 0xbb2, 0xbc7, 0xbb7, 0xbbf, 0xbaf, 0xba9,
0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbb9, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb99, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb99, 0xbcd,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbb9, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb99, 0xbcd, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbcb, 0xbb5,
0xbcd, 0xba4, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbcb, 0xbb5,
0xbcd, 0xba4, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbcb, 0xbb5, 0xbcd, 0xba4, 0xbcd, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xba8,
0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xba8, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0xbaa, 0xbcd, 0x20, 0xbaa, 0xbc6, 0xbb0, 0xbc1, 0xb99, 0xbcd, 0xb95, 0xb9f, 0xbb2,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xba8, 0xbcd, 0xba4, 0xbcb,
0xb9a, 0xbc0, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xb87, 0xba8, 0xbcd, 0xba4, 0xbcb, 0xba9, 0xbc7, 0xb9a,
0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95,
0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb87, 0xba8, 0xbcd, 0xba4,
0xbcb, 0xba9, 0xbc7, 0xb9a, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb87,
0xba8, 0xbcd, 0xba4, 0xbcb, 0xba9, 0xbc7, 0xb9a, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb88, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb88, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88, 0xbb0, 0xbbe,
0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb0, 0xbcd, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd,
0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb0, 0xbcd, 0xb95,
0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb0, 0xbcd,
0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb8, 0xbcd, 0xbb0, 0xbc7,
0xbb2, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb8, 0xbcd, 0xbb0,
0xbc7, 0xbb2, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb8, 0xbcd, 0xbb0, 0xbc7, 0xbb2, 0xbcd,
0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9c, 0xbaa, 0xbcd, 0xbaa, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb9c, 0xbaa, 0xbcd, 0xbaa, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9c,
0xbaa, 0xbcd, 0xbaa, 0xbbe, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xb9f, 0xbcd, 0xbb0,
0xbcb, 0xbaa, 0xbb5, 0xbcd, 0xbb2, 0xbcb, 0xbb5, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd,
0x20, 0xb95, 0xbae, 0xbcd, 0xb9a, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbbf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xb9f, 0xbcd, 0xbb0, 0xbcb, 0xbaa,
0xbb5, 0xbcd, 0xbb2, 0xbcb, 0xbb5, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95,
0xbae, 0xbcd, 0xb9a, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbbf, 0x20, 0xba4, 0xbb0,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xb9f, 0xbcd, 0xbb0, 0xbcb,
0xbaa, 0xbb5, 0xbcd, 0xbb2, 0xbcb, 0xbb5, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20,
0xb95, 0xbae, 0xbcd, 0xb9a, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbbf, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xb9c, 0xb95,
0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xb9c, 0xb95, 0xbb8,
0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xbc7, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xb9c, 0xb95, 0xbb8, 0xbcd, 0xba4,
0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb0,
0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb0, 0xbbf,
0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb0, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2,
0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcb, 0xbb8,
0xbcd, 0xbb0, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcd, 0xbb0,
0xbb8, 0xbcd, 0xba9, 0xbcb, 0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcd, 0xbb0, 0xbb8, 0xbcd, 0xba9,
0xbcb, 0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95,
0xbcd, 0xbb0, 0xbb8, 0xbcd, 0xba9, 0xbcb, 0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbcd,
0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb95, 0xbbf, 0xbb0, 0xbcd, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb2, 0xb99, 0xbcd, 0xb95, 0xbbe,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb2, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba4,
0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbb2, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb9, 0xbcb, 0xbb5, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb2, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20,
0xbb9, 0xbcb, 0xbb5, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb2, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd,
0x20, 0xbb9, 0xbcb, 0xbb5, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xbcd, 0xb95, 0xbbe, 0xbb5,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xbcd, 0xb95, 0xbbe,
0xbb5, 0xbcd, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xb95, 0xbcd, 0xb95, 0xbbe, 0xbb5, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xba4, 0xba9, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xba4, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xb95, 0xba4, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbb2, 0xbc7, 0xbb7, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb2, 0xba4, 0xbcd, 0xba4, 0xbc0, 0xbb5, 0xbc1,
0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb0,
0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xb9a, 0xbbe, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xbb7, 0xbb2, 0xbcd, 0x20, 0xba4,
0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xbca, 0xbb0, 0xbbf, 0xbb7, 0xbbf, 0xbaf, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbca, 0xbb0, 0xbbf, 0xbb7, 0xbbf, 0xbaf, 0xbb8, 0xbcd,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbca, 0xbb0, 0xbbf, 0xbb7, 0xbbf, 0xbaf, 0xbb8, 0xbcd, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe,
0xb9a, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc6, 0xb95,
0xbcd, 0xbb8, 0xbbf, 0xb95, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf,
0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc6, 0xb95, 0xbcd,
0xbb8, 0xbbf, 0xb95, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95,
0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc6, 0xb95, 0xbcd, 0xbb8, 0xbbf, 0xb95, 0xba9, 0xbcd,
0x20, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2,
0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb2, 0xba9,
0xbcd, 0x20, 0xbaa, 0xbbe, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb89, 0xbb2, 0xba9, 0xbcd, 0x20, 0xbaa, 0xbbe, 0xb9f, 0xbb0, 0xbcd, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb89, 0xbb2, 0xba9, 0xbcd, 0x20, 0xbaa, 0xbbe, 0xb9f, 0xbb0, 0xbcd, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe,
0xbb8, 0xbcd, 0xb95, 0xbcb, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe,
0xbb8, 0xbcd, 0xb95, 0xbcb, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbcb,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbcd, 0xbae, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xba8, 0xbb5, 0xbcd, 0xbb0, 0xbc2, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xba8, 0xbc7, 0xbaa, 0xbbe, 0xbb3, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xba8, 0xbbf, 0xbaf, 0xbc2, 0x20, 0xb95, 0xbb2, 0xbbf, 0xb9f, 0xbcb, 0xba9, 0xbbf,
0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2,
0x20, 0xb95, 0xbb2, 0xbbf, 0xb9f, 0xbcb, 0xba9, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xba8, 0xbbf, 0xbaf, 0xbc2, 0x20, 0xb95, 0xbb2, 0xbbf, 0xb9f, 0xbcb, 0xba9, 0xbbf,
0xbaf, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0xb9a, 0xbbf, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4,
0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0xb9a,
0xbbf, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf,
0xbc2, 0xb9a, 0xbbf, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xbaa, 0xb95,
0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf,
0xbaf, 0xbc2, 0xb83, 0xbaa, 0xbb5, 0xbc1, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xbb2, 0xbbe,
0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf,
0xbaf, 0xbc2, 0xb83, 0xbaa, 0xbb5, 0xbc1, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xbb2, 0xbbe,
0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0xb83, 0xbaa,
0xbb5, 0xbc1, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1,
0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8,
0xbbe, 0xbb0, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5,
0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbe, 0xbb0, 0xbcd, 0xb83,
0xbaa, 0xbcb, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8,
0xbbe, 0xbb0, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5,
0xbc1, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xba9, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbcb,
0x20, 0xb9f, 0xbbf, 0x20, 0xba8, 0xbcb, 0xbb0, 0xba9, 0xbcd, 0xbb9, 0xbbe, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xba9, 0xbbe, 0xba9,
0xbcd, 0xb9f, 0xbcb, 0x20, 0xb9f, 0xbbf, 0x20, 0xba8, 0xbcb, 0xbb0, 0xbcb, 0xba9,
0xbcd, 0xbb9, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xba9, 0xbbe, 0xba9,
0xbcd, 0xb9f, 0xbcb, 0x20, 0xb9f, 0xbbf, 0x20, 0xba8, 0xbcb, 0xbb0, 0xbcb, 0xba9,
0xbcd, 0xbb9, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbb5, 0xb9f, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xbae, 0xbb0, 0xbbf,
0xba9, 0xbbe, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbbf, 0xbaa, 0xbbf,
0xbb0, 0xbbf, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xbb8, 0xbcd,
0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbbf, 0xbaa, 0xbbf,
0xbb0, 0xbbf, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb93, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb93, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95,
0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb93, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbe, 0xb95,
0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbaa, 0xbbe, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbaa, 0xbbe, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbe,
0xbb2, 0xbb5, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbaa, 0xbc1,
0xbb5, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0x20, 0xb95, 0xbbf, 0xba9, 0xbbf,
0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbb0, 0xbbe, 0xb95,
0xbc1, 0xbb5, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbb0, 0xbbe,
0xb95, 0xbc1, 0xbb5, 0xbc7, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbb0, 0xbbe, 0xb95, 0xbc1, 0xbb5,
0xbc7, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbaa, 0xbc6, 0xbb0, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6,
0xbb0, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbc1, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb2, 0xbbf, 0xbaa,
0xbcd, 0xbaa, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa,
0xbbf, 0xbb2, 0xbbf, 0xbaa, 0xbcd, 0xbaa, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa,
0xbbf, 0xbb2, 0xbbf, 0xbaa, 0xbcd, 0xbaa, 0xbc8, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb,
0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xba9,
0xbbf, 0xb95, 0xbcd, 0xbb8, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9,
0xbcd, 0xb9f, 0xbcd, 0x20, 0xbaa, 0xbbf, 0xbaf, 0xbb0, 0xbbf, 0x20, 0x26, 0x20,
0xbae, 0xbbf, 0xb95, 0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbbf, 0xbb2, 0xbbe, 0xba9, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd,
0xb9f, 0xbcd, 0x20, 0xbaa, 0xbbf, 0xbaf, 0xbb0, 0xbbf, 0x20, 0x26, 0x20, 0xbae,
0xbbf, 0xb95, 0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbbf, 0xbb2, 0xbbe, 0xba9, 0xbcd, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbaa, 0xbbf,
0xbaf, 0xbb0, 0xbbf, 0x20, 0x26, 0x20, 0xbae, 0xbbf, 0xb95, 0xbcd, 0xb95, 0xbc1,
0xbaf, 0xbbf, 0xbb2, 0xbbe, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xb9f, 0xbcd, 0xb95,
0xbc6, 0xbaf, 0xbcd, 0xbb0, 0xbcd, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbaa, 0xbcb, 0xba9, 0xbbe, 0xbaa, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0xb99, 0xbcd, 0xbaf, 0xbbe, 0xb99, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc8, 0xb9c, 0xbc8, 0xbb2, 0xbcb, 0xbb0,
0xbcd, 0xb9f, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc8, 0xb9c,
0xbc8, 0xbb2, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbbe, 0x20, 0xba4, 0xbb0, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc8, 0xb9c, 0xbc8, 0xbb2, 0xbcb, 0xbb0, 0xbcd,
0xb9f, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbb0, 0xbc0, 0xbaf, 0xbc2, 0xba9, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb0, 0xbcb, 0xba4, 0xbc7, 0xbb0, 0xbbe, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xb95, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xb95, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb9a, 0xb95, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbb0, 0xbbe, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbb0, 0xbbe, 0x20, 0xba4, 0xbb0, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbb0, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbcb, 0xbb5, 0xbbe,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbcb, 0xbb5, 0xbbe, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9a, 0xbae, 0xbcb, 0xbb5, 0xbbe, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc0, 0xb9a, 0xbc6, 0xbb2,
0xbcd, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbf, 0xb99,
0xbcd, 0xb95, 0xbaa, 0xbcd, 0xbaa, 0xbc2, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbe,
0xbb2, 0xbae, 0xba9, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc6, 0xbb1, 0xbcd, 0xb95, 0xbc1,
0x20, 0xb9c, 0xbbe, 0xbb0, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc1, 0xbb0, 0xbbf, 0xba9, 0xbbe, 0xbae, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xbaf, 0xbcb, 0xbb5, 0xbbe, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbb9, 0xbbf, 0xba4, 0xbbf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbbe, 0xbaf, 0xbcd, 0xbaa, 0xbc7, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbbe, 0xbaf, 0xbcd, 0xbaa, 0xbc7, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xba4, 0xbbe, 0xbaf, 0xbcd, 0xbaa, 0xbc7, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xb9c, 0xbbf, 0xb95, 0xbbf,
0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb9f, 0xbcb, 0xb95, 0xbcd, 0xb95, 0xbc6, 0xbb2, 0xbbe, 0xbb5, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbcb, 0xb99, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbcb, 0xb99, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb9f, 0xbcb, 0xb99, 0xbcd, 0xb95, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc1, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc1, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbae, 0xbc6, 0xba9,
0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xba4, 0xbc1, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbae, 0xbc6, 0xba9, 0xbbf, 0xbb8,
0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc1, 0xbb0, 0xbcd, 0xb95,
0xbcd, 0xbae, 0xbc6, 0xba9, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc1,
0xbb5, 0xbbe, 0xbb2, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb0,
0xbc1, 0xb95, 0xbc1, 0xbb5, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89,
0xbb0, 0xbc1, 0xb95, 0xbc1, 0xbb5, 0xbc7, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb0, 0xbc1, 0xb95,
0xbc1, 0xbb5, 0xbc7, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb89, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4,
0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb8, 0xbcd,
0xbaa, 0xbc6, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb89, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9,
0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbb5, 0xba9, 0xbc1, 0xbb5, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbc1, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbb5, 0xba9, 0xbc1, 0xbb5, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbc1,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbb5, 0xba9, 0xbc1, 0xbb5, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbc1, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbc6,
0xba9, 0xbbf, 0xb9a, 0xbc1, 0xbb2, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbb5, 0xbbf, 0xbb3, 0xbbe, 0xb9f, 0xbbf, 0xbb5, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbcb,
0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbbf, 0xbb3, 0xbbe,
0xb9f, 0xbbf, 0xbb5, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbcb, 0xb95, 0xbcd, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbb5, 0xbbf, 0xbb3, 0xbbe, 0xb9f, 0xbbf, 0xbb5, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbcb,
0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbb5, 0xbcb, 0xbb2, 0xbcd, 0xb95, 0xbcb, 0xb95, 0xbcd, 0xbb0, 0xbbe, 0xb9f,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbcb, 0xbb2, 0xbcd, 0xb95,
0xbcb, 0xb95, 0xbcd, 0xbb0, 0xbbe, 0xb9f, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbcb, 0xbb2,
0xbcd, 0xb95, 0xbcb, 0xb95, 0xbcd, 0xbb0, 0xbbe, 0xb9f, 0xbcd, 0x20, 0xb95, 0xbcb,
0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbc7, 0xb95, 0xbcd,
0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5,
0xbbe, 0xbb2, 0xbbf, 0xbb8, 0xbcd, 0x20, 0xbae, 0xbb1, 0xbcd, 0xbb1, 0xbc1, 0xbae,
0xbcd, 0x20, 0xb83, 0xbaa, 0xbcd, 0xbaf, 0xbc2, 0xb9f, 0xbc1, 0xba9, 0xbbe, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd,
0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f,
0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xbcd,
0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbaf, 0xbc7, 0xb95, 0xbbe, 0xb9f, 0xbc6, 0xbb0, 0xbbf, 0xba9,
0xbcd, 0xbaa, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbaf, 0xbc7, 0xb95, 0xbbe, 0xb9f, 0xbc6, 0xbb0, 0xbbf, 0xba9, 0xbcd, 0xbaa, 0xbb0,
0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xbc7, 0xb95, 0xbbe, 0xb9f, 0xbc6, 0xbb0,
0xbbf, 0xba9, 0xbcd, 0xbaa, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xbc2, 0xb95, 0xbcb, 0xba9,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x43, 0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61,
0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x43, 0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61,
0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6e, 0x2d, 0x41, 0x6d,
0x61, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61,
0x74, 0x20, 0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b,
0x61, 0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b,
0x61, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x2d, 0x41,
0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72,
0x61, 0x74, 0x20, 0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x61, 0x2d, 0x41, 0x6d, 0x61, 0x72,
0x69, 0x6b, 0x61, 0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69,
0x6b, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x61, 0x2d, 0x41, 0x6d, 0x61,
0x72, 0x69, 0x6b, 0x61, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x48, 0x69,
0x64, 0x61, 0x77, 0x20, 0x6e, 0x69, 0x71, 0x20, 0x59, 0x61, 0x6d, 0x61,
0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x44, 0x61, 0x79, 0x62, 0x69, 0x6e, 0x79, 0x61, 0x6e, 0x53,
0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x44, 0x61, 0x79, 0x62,
0x69, 0x6e, 0x79, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x41,
0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x44, 0x61, 0x79, 0x62, 0x69,
0x6e, 0x79, 0x61, 0x6e, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74,
0x20, 0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x59, 0x61, 0x79, 0x75, 0x6e, 0x20, 0x54, 0x61, 0x73, 0x69, 0x79,
0x61, 0x6e, 0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x75, 0x6e, 0x20, 0x54, 0x61, 0x73,
0x69, 0x79, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x59, 0x61,
0x79, 0x75, 0x6e, 0x20, 0x54, 0x61, 0x73, 0x69, 0x79, 0x61, 0x6e, 0x20,
0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x20, 0x52, 0x62, 0x61, 0x67,
0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6e, 0x6f,
0x77, 0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x43, 0x6f, 0x6e, 0x6f, 0x77, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x43, 0x6f, 0x6e, 0x6f, 0x77, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72,
0x61, 0x74, 0x20, 0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x6f, 0x77, 0x53, 0x6e, 0x65, 0x67,
0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6e,
0x6f, 0x77, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x6f,
0x77, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x20, 0x52, 0x62,
0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x53, 0x69,
0x6f, 0x77, 0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x53, 0x69, 0x6f, 0x77, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x53, 0x69, 0x6f, 0x77, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61,
0x74, 0x20, 0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x51, 0x75, 0x72, 0x69, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x77,
0x65, 0x79, 0x6a, 0x69, 0x410, 0x43a, 0x440, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x410, 0x43a, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43a, 0x440,
0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x4d8, 0x444, 0x433, 0x430, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4ae, 0x437, 0x4d9, 0x43a, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a,
0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x410, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d,
0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b,
0x448, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9,
0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43c, 0x430, 0x437,
0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c,
0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x4af,
0x437, 0x4d9, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9,
0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x4af, 0x437, 0x4d9, 0x43a,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f,
0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x497, 0x4d9,
0x439, 0x433, 0x435, 0x20, 0x4af, 0x437, 0x4d9, 0x43a, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x43a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433,
0x44b, 0x448, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d,
0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x43a, 0x4e9, 0x43d, 0x447, 0x44b,
0x433, 0x44b, 0x448, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9,
0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x43a, 0x4e9, 0x43d, 0x447, 0x44b,
0x433, 0x44b, 0x448, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9,
0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x442, 0x430, 0x443, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422,
0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x442, 0x430, 0x443,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f,
0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x497, 0x4d9,
0x439, 0x433, 0x435, 0x20, 0x442, 0x430, 0x443, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x422, 0x44b, 0x43d, 0x20, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d,
0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x422, 0x44b, 0x43d, 0x20, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422,
0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x422, 0x44b, 0x43d, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c, 0x20, 0x497, 0x4d9, 0x439,
0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43f, 0x438,
0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43f, 0x438, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x497, 0x4d9, 0x439,
0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x440,
0x4d9, 0x43f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x440,
0x4d9, 0x43f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x440, 0x4d9, 0x43f, 0x20,
0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x20, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x497, 0x4d9, 0x439,
0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x43c,
0x435, 0x43d, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410,
0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440,
0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x430, 0x442, 0x43b,
0x430, 0x43d, 0x442, 0x438, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x430, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x430, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x4ae, 0x437, 0x4d9, 0x43a, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x4af, 0x437, 0x4d9, 0x43a,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x4af, 0x437, 0x4d9, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x20, 0x4af, 0x437, 0x4d9, 0x43a, 0x20, 0x43a, 0x4e9,
0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x4af,
0x437, 0x4d9, 0x43a, 0x20, 0x43a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x4af, 0x437, 0x4d9, 0x43a, 0x20, 0x43a, 0x4e9, 0x43d, 0x431, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b,
0x448, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x497, 0x4d9,
0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9,
0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a,
0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d,
0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x4d8, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x497,
0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4d8, 0x437, 0x435,
0x440, 0x431, 0x430, 0x439, 0x497, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4d8,
0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x497, 0x430, 0x43d, 0x20, 0x497, 0x4d9,
0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x437,
0x43e, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x43e, 0x440, 0x20,
0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x437, 0x43e, 0x443, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b,
0x430, 0x440, 0x44b, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x443, 0x442, 0x430, 0x43d,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x43e, 0x43b, 0x438, 0x432,
0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x430, 0x20, 0x497, 0x4d9, 0x439,
0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x443,
0x43d, 0x435, 0x439, 0x2d, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441, 0x430, 0x43b,
0x430, 0x43c, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x430, 0x431,
0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x435, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x427,
0x430, 0x442, 0x435, 0x43c, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x427, 0x438, 0x43b, 0x438, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x420, 0x430, 0x448, 0x442,
0x443, 0x430, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x443, 0x442,
0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438,
0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438,
0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x443, 0x43a, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b,
0x430, 0x440, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x443,
0x43a, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x43a, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443,
0x43b, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x440, 0x442, 0x430, 0x447, 0x430, 0x20,
0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x41a, 0x443, 0x431, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a,
0x443, 0x431, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20,
0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x414, 0x44d, 0x432, 0x438, 0x441, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438,
0x43b, 0x44c, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d,
0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432,
0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20, 0x443,
0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e,
0x440, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4ae, 0x437, 0x4d9, 0x43a,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x4ae, 0x437, 0x4d9,
0x43a, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x4ae, 0x437, 0x4d9,
0x43a, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x41a, 0x4e9, 0x43d, 0x447,
0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x415, 0x440,
0x430, 0x43a, 0x20, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x433,
0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442,
0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x41a, 0x4e9,
0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a,
0x43b, 0x435, 0x43d, 0x434, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430,
0x440, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b,
0x430, 0x440, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x435, 0x43d, 0x434, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440,
0x44b, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x424, 0x438, 0x434, 0x436, 0x438, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x424, 0x438, 0x434, 0x436, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424,
0x438, 0x434, 0x436, 0x438, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437,
0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x20, 0x43a, 0x4e9,
0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x4bb, 0x4d9, 0x43c, 0x20, 0x410, 0x43d, 0x442,
0x430, 0x440, 0x43a, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20,
0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413,
0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443,
0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413,
0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x20, 0x443, 0x440, 0x442, 0x430, 0x447,
0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447,
0x44b, 0x433, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9,
0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d,
0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b,
0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442,
0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x424, 0x430, 0x440, 0x441, 0x44b, 0x20, 0x43a, 0x443, 0x43b, 0x442,
0x44b, 0x433, 0x44b, 0x43d, 0x44b, 0x4a3, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430,
0x439, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413,
0x430, 0x432, 0x430, 0x439, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439, 0x2d, 0x410,
0x43b, 0x435, 0x443, 0x442, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432, 0x430,
0x439, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x43e,
0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x43e, 0x43d,
0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x425, 0x43e, 0x432, 0x434, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4ba, 0x438, 0x43d, 0x434, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x4ba, 0x438, 0x43d, 0x434, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x44b,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4ba, 0x438, 0x43d, 0x434, 0x2d,
0x43a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x4ae, 0x437, 0x4d9, 0x43a, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437,
0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d,
0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9,
0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d,
0x435, 0x437, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418,
0x440, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x440,
0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418,
0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x441, 0x440, 0x430,
0x438, 0x43b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x441, 0x440,
0x430, 0x438, 0x43b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x441, 0x440, 0x430, 0x438,
0x43b, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x42f, 0x43f, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x42f, 0x43f, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x43f, 0x43e,
0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e,
0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a,
0x438, 0x439, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x435, 0x442,
0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a,
0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432,
0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438,
0x439, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433,
0x44b, 0x448, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430,
0x442, 0x44b, 0x448, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435,
0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435,
0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a,
0x43e, 0x441, 0x440, 0x430, 0x44d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f,
0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x44b, 0x440, 0x433, 0x44b, 0x437,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41b,
0x430, 0x439, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440,
0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434,
0x20, 0x425, 0x430, 0x443, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430,
0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x438,
0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x43b, 0x44c,
0x434, 0x438, 0x432, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440,
0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x440, 0x43a,
0x438, 0x437, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x440, 0x448, 0x430,
0x43b, 0x43b, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x43a, 0x438, 0x439, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430,
0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430,
0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x43e, 0x443, 0x441, 0x43e,
0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441,
0x438, 0x43a, 0x430, 0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x447, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x435,
0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x447, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441,
0x438, 0x43a, 0x430, 0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x447, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442,
0x43e, 0x440, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x43b, 0x430,
0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x423,
0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x497, 0x4d9,
0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9,
0x441, 0x43a, 0x4d9, 0x4af, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c,
0x4d9, 0x441, 0x43a, 0x4d9, 0x4af, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9, 0x441,
0x43a, 0x4d9, 0x4af, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c, 0x430, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x4a3, 0x430, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x42f, 0x4a3, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x4a3, 0x430, 0x20, 0x41a, 0x430,
0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x4a3, 0x430, 0x20,
0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x42f, 0x4a3, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x4a3, 0x430, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e, 0x444,
0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b,
0x435, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x438, 0x443, 0x44d,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x43a, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x44b,
0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438,
0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d,
0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434,
0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x497, 0x4d9,
0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e,
0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41e, 0x43c, 0x441, 0x43a, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43b, 0x430, 0x443,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43f, 0x443, 0x430,
0x20, 0x42f, 0x4a3, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435, 0x44f, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x439, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f,
0x438, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441,
0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435,
0x440, 0x20, 0x4bb, 0x4d9, 0x43c, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d,
0x41f, 0x44c, 0x435, 0x440, 0x20, 0x4bb, 0x4d9, 0x43c, 0x20, 0x41c, 0x438, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d,
0x41f, 0x44c, 0x435, 0x440, 0x20, 0x4bb, 0x4d9, 0x43c, 0x20, 0x41c, 0x438, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x43e, 0x43d, 0x430, 0x43f,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x445, 0x435, 0x43d,
0x44c, 0x44f, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x420, 0x435,
0x44e, 0x43d, 0x44c, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x497, 0x4d9,
0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x430,
0x43c, 0x430, 0x440, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421,
0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c,
0x430, 0x440, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x439, 0x448, 0x435,
0x43b, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f,
0x443, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e,
0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x443, 0x43b, 0x430, 0x440, 0x44b, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a,
0x20, 0x414, 0x436, 0x43e, 0x440, 0x434, 0x436, 0x438, 0x44f, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421, 0x451, 0x432, 0x430, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x438, 0x442, 0x438, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x497,
0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422,
0x430, 0x497, 0x438, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433,
0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x427, 0x443, 0x443, 0x43a,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x440, 0x43a, 0x43c,
0x4d9, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x422, 0x4e9, 0x440, 0x43a, 0x43c, 0x4d9, 0x43d, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x440, 0x43a, 0x43c, 0x4d9, 0x43d, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430,
0x439, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x4ae, 0x437, 0x431, 0x4d9, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4ae, 0x437, 0x431, 0x4d9, 0x43a, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x4ae, 0x437, 0x431, 0x4d9, 0x43a,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430,
0x442, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442,
0x443, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x44d, 0x439, 0x43a, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x4bb, 0x4d9, 0x43c, 0x20,
0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42e, 0x43a, 0x43e, 0x43d, 0x20,
0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0xc0f, 0xc15, 0xc30, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc0f, 0xc15, 0xc30, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc0f, 0xc15,
0xc30, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc06, 0xc2b, 0xc4d, 0xc18, 0xc28, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc30,
0xc32, 0xc4d, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc06, 0xc2b,
0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc26, 0xc15,
0xc4d, 0xc37, 0xc3f, 0xc23, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc06, 0xc2b,
0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36,
0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc06, 0xc2b,
0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc32, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc3e, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc32, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc3e, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc05, 0xc32, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc3e, 0x20, 0xc2a, 0xc17,
0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc32, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc32, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc05, 0xc32, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc2e, 0xc46, 0xc1c,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc2e, 0xc46, 0xc1c,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc2e, 0xc46, 0xc1c, 0xc3e, 0xc28,
0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc27,
0xc4d, 0xc2f, 0xc2e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20,
0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30,
0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2a, 0xc17, 0xc1f,
0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc4c, 0xc02, 0xc1f, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc4c, 0xc02, 0xc1f, 0xc46, 0xc2f, 0xc3f,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc4c, 0xc02, 0xc1f, 0xc46, 0xc2f, 0xc3f,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41,
0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc38, 0xc3f,
0xc2b, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc38, 0xc3f,
0xc2b, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f,
0xc15, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41,
0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc28, 0xc21,
0xc48, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc28, 0xc3e, 0xc28,
0xc4d, 0xc21, 0xc4d, 0xc30, 0xc3f, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc28, 0xc3e, 0xc28,
0xc4d, 0xc21, 0xc4d, 0xc30, 0xc3f, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc0f, 0xc2a, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc0f, 0xc2a, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc0f, 0xc2a,
0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc05, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc1f, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc05, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc1f, 0xc41, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05,
0xc15, 0xc4d, 0xc35, 0xc3e, 0xc1f, 0xc41, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc46,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc46,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc46, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30, 0xc47,
0xc2c, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30,
0xc47, 0xc2c, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30, 0xc47,
0xc2c, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35,
0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc05, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28, 0xc3e, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28,
0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40,
0xc28, 0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc05, 0xc30, 0xc4d, 0xc1c,
0xc46, 0xc02, 0xc1f, 0xc40, 0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a,
0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc05, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02,
0xc1f, 0xc40, 0xc28, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f,
0xc2e, 0x20, 0xc05, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28, 0xc3e,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06,
0xc30, 0xc4d, 0xc2e, 0xc47, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc06, 0xc30, 0xc4d, 0xc2e, 0xc47, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc06, 0xc30, 0xc4d, 0xc2e, 0xc47, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1f, 0xc4d,
0xc32, 0xc3e, 0xc02, 0xc1f, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc05, 0xc1f, 0xc4d, 0xc32, 0xc3e, 0xc02, 0xc1f, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc05, 0xc1f, 0xc4d, 0xc32, 0xc3e, 0xc02, 0xc1f, 0xc3f, 0xc15, 0xc4d, 0x20,
0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30,
0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30,
0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32,
0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20,
0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35,
0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20,
0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc06,
0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20,
0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d,
0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2e, 0xc27, 0xc4d,
0xc2f, 0xc2e, 0x20, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2a, 0xc17,
0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc06,
0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0x20, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc28,
0xc4d, 0x20, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2a, 0xc17, 0xc1f,
0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc06, 0xc38,
0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2a, 0xc17,
0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc30, 0xc4d, 0xc2c, 0xc48, 0xc1c, 0xc3e,
0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc30, 0xc4d, 0xc2c,
0xc48, 0xc1c, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc30, 0xc4d,
0xc2c, 0xc48, 0xc1c, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc4b, 0xc30, 0xc4d, 0xc38, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc4b, 0xc30, 0xc4d, 0xc38, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc4b, 0xc30, 0xc4d, 0xc38, 0xc4d, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc02, 0xc17,
0xc4d, 0xc32, 0xc3e, 0xc26, 0xc47, 0xc36, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2c, 0xc02, 0xc17, 0xc4d, 0xc32, 0xc3e, 0xc26, 0xc47, 0xc36, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2c, 0xc02, 0xc17, 0xc4d, 0xc32, 0xc3e, 0xc26, 0xc47, 0xc36, 0xc4d, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2d, 0xc42,
0xc1f, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc4a, 0xc32,
0xc40, 0xc35, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc4d,
0xc30, 0xc46, 0xc1c, 0xc3f, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2c, 0xc4d, 0xc30, 0xc46, 0xc1c, 0xc3f, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2c, 0xc4d, 0xc30, 0xc46, 0xc1c, 0xc3f, 0xc32, 0xc3f, 0xc2f, 0xc3e,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c,
0xc4d, 0xc30, 0xc42, 0xc28, 0xc47, 0x20, 0xc26, 0xc30, 0xc41, 0xc38, 0xc32, 0xc3e,
0xc2e, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc47, 0xc2a, 0xc4d, 0x20,
0xc35, 0xc46, 0xc30, 0xc4d, 0xc21, 0xc46, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15,
0xc47, 0xc2a, 0xc4d, 0x20, 0xc35, 0xc46, 0xc30, 0xc4d, 0xc21, 0xc46, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc15, 0xc47, 0xc2a, 0xc4d, 0x20, 0xc35, 0xc46, 0xc30, 0xc4d, 0xc21, 0xc46,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a,
0xc3e, 0xc2e, 0xc30, 0xc4d, 0xc30, 0xc4b, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3e, 0xc25,
0xc2e, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3e, 0xc25, 0xc2e, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3e, 0xc25, 0xc2e, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f,
0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3f, 0xc32, 0xc40, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc1a, 0xc3f, 0xc32, 0xc40, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3f, 0xc32, 0xc40, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc48,
0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc48, 0xc28, 0xc3e, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc1a, 0xc48, 0xc28, 0xc3e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20,
0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc15, 0xc4d, 0xc30, 0xc3f, 0xc38, 0xc4d, 0xc2e, 0xc38, 0xc4d, 0x20, 0xc26,
0xc40, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4b, 0xc15, 0xc4b,
0xc38, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc15, 0xc4a, 0xc32, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc15, 0xc4a, 0xc32, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc15, 0xc4a, 0xc32, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc41, 0xc15, 0xc4d,
0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15,
0xc41, 0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc15, 0xc41, 0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc05,
0xc30, 0xc4d, 0xc27, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc15, 0xc4d, 0xc2f, 0xc42, 0xc2c, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc15, 0xc4d, 0xc2f, 0xc42, 0xc2c, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d,
0xc2f, 0xc42, 0xc2c, 0xc3e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46,
0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc21,
0xc47, 0xc35, 0xc3f, 0xc38, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc21, 0xc4d,
0xc2f, 0xc42, 0xc2e, 0xc3e, 0xc02, 0xc1f, 0xc4d, 0x2d, 0xc21, 0xc3f, 0x2019, 0xc09,
0xc30, 0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc24, 0xc48, 0xc2e, 0xc42,
0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc08, 0xc38, 0xc4d, 0xc1f, 0xc30,
0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc08,
0xc38, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc08, 0xc38, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc08,
0xc15, 0xc4d, 0xc35, 0xc21, 0xc3e, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc38, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc30, 0xc32, 0xc4d, 0x20, 0xc2f, 0xc42, 0xc30,
0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38,
0xc46, 0xc02, 0xc1f, 0xc4d, 0xc30, 0xc32, 0xc4d, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b,
0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46, 0xc02, 0xc1f,
0xc4d, 0xc30, 0xc32, 0xc4d, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f,
0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b,
0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42,
0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20,
0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc41, 0xc26, 0xc42,
0xc30, 0x2d, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2f, 0xc42, 0xc30,
0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a,
0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a,
0xc3f, 0xc2e, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2f, 0xc42,
0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3e, 0xc15, 0xc4d, 0x200c, 0xc32,
0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3e, 0xc15, 0xc4d, 0x200c, 0xc32, 0xc4d,
0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2b, 0xc3e, 0xc15, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02,
0xc21, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc1c, 0xc40, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc1c, 0xc40, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b,
0xc3f, 0xc1c, 0xc40, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2b, 0xc4d, 0xc30, 0xc46, 0xc02, 0xc1a, 0xc4d, 0x20, 0xc17, 0xc2f,
0xc3e, 0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc4d, 0xc30, 0xc46,
0xc02, 0xc1a, 0xc4d, 0x20, 0xc26, 0xc15, 0xc4d, 0xc37, 0xc3f, 0xc23, 0x20, 0xc2e,
0xc30, 0xc3f, 0xc2f, 0xc41, 0x20, 0xc05, 0xc02, 0xc1f, 0xc3e, 0xc30, 0xc4d, 0xc15,
0xc3f, 0xc1f, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc3e,
0xc32, 0xc3e, 0xc2a, 0xc3e, 0xc17, 0xc4b, 0xc38, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc17, 0xc3e, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc3e, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc3f, 0xc32,
0xc4d, 0xc2c, 0xc30, 0xc4d, 0xc1f, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d, 0x200c,
0xc35, 0xc3f, 0xc1a, 0xc4d, 0x20, 0xc38, 0xc17, 0xc1f, 0xc41, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc17, 0xc4d, 0xc30,
0xc40, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc17,
0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21,
0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc17,
0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21,
0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc28,
0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc17, 0xc4d, 0xc30,
0xc40, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc17, 0xc4d, 0xc30,
0xc40, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc4d,
0xc35, 0xc3e, 0xc2e, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc32, 0xc4d, 0xc2b, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc39, 0xc35, 0xc3e, 0xc2f, 0xc4d, 0x2d, 0xc05, 0xc32, 0xc4d,
0xc2f, 0xc42, 0xc37, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc39, 0xc35, 0xc3e, 0xc2f, 0xc4d, 0x2d, 0xc05, 0xc32, 0xc4d, 0xc2f, 0xc42, 0xc37,
0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc35, 0xc3e, 0xc2f, 0xc4d,
0x2d, 0xc05, 0xc32, 0xc4d, 0xc2f, 0xc42, 0xc37, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20,
0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30,
0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc3e, 0xc02, 0xc15, 0xc3e, 0xc02,
0xc17, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc3e, 0xc02, 0xc15, 0xc3e,
0xc02, 0xc17, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc3e, 0xc02, 0xc15, 0xc3e, 0xc02,
0xc17, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc39, 0xc4b, 0xc35, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc39, 0xc4b, 0xc35, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc4b, 0xc35,
0xc4d, 0xc21, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2d, 0xc3e, 0xc30, 0xc24, 0xc26, 0xc47, 0xc36, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc39, 0xc3f, 0xc02, 0xc26, 0xc42, 0x20, 0xc2e, 0xc39, 0xc3e, 0x20, 0xc38, 0xc2e,
0xc41, 0xc26, 0xc4d, 0xc30, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc02, 0xc21,
0xc4b, 0xc1a, 0xc48, 0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46,
0xc02, 0xc1f, 0xc4d, 0xc30, 0xc32, 0xc4d, 0x20, 0xc07, 0xc02, 0xc21, 0xc4b, 0xc28,
0xc47, 0xc37, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42,
0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc07, 0xc02, 0xc21, 0xc4b, 0xc28, 0xc47, 0xc37,
0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a,
0xc3f, 0xc2e, 0x20, 0xc07, 0xc02, 0xc21, 0xc4b, 0xc28, 0xc47, 0xc37, 0xc3f, 0xc2f,
0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc3e, 0xc28, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc07, 0xc30, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35,
0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc07, 0xc30, 0xc4d, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc4d, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38,
0xc4d, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc4d, 0xc15, 0xc41, 0xc1f,
0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc1c, 0xc4d, 0xc30, 0xc3e, 0xc2f, 0xc3f, 0xc32,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc1c, 0xc4d, 0xc30, 0xc3e, 0xc2f,
0xc3f, 0xc32, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc1c, 0xc4d, 0xc30, 0xc3e, 0xc2f,
0xc3f, 0xc32, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32,
0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c, 0xc2a,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c, 0xc2a, 0xc3e, 0xc28,
0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c, 0xc2a, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc17,
0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc46, 0xc1f, 0xc4d, 0xc30, 0xc4b, 0xc2a, 0xc3e,
0xc35, 0xc4d, 0xc32, 0xc4b, 0xc35, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x2d, 0xc15,
0xc3e, 0xc2e, 0xc4d, 0xc1b, 0xc3e, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc46, 0xc1f, 0xc4d, 0xc30, 0xc4b, 0xc2a, 0xc3e,
0xc35, 0xc4d, 0xc32, 0xc4b, 0xc35, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x2d, 0xc15,
0xc3e, 0xc2e, 0xc4d, 0xc1b, 0xc3e, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc3f, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2a, 0xc46, 0xc1f, 0xc4d, 0xc30, 0xc4b, 0xc2a, 0xc3e, 0xc35, 0xc4d,
0xc32, 0xc4b, 0xc35, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x2d, 0xc15, 0xc3e, 0xc2e,
0xc4d, 0xc1b, 0xc3e, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc3f, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc1c, 0xc15, 0xc3f,
0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24,
0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc15, 0xc1c, 0xc15, 0xc3f, 0x200c, 0xc38,
0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36,
0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc15, 0xc1c, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc30, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc30, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc30, 0xc3f, 0xc2f, 0xc28,
0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24,
0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4b, 0xc38, 0xc4d,
0xc30, 0xc3e, 0xc2f, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d, 0xc30,
0xc3e, 0xc38, 0xc4d, 0xc28, 0xc4b, 0xc2f, 0xc3e, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d, 0xc30, 0xc3e, 0xc38, 0xc4d,
0xc28, 0xc4b, 0xc2f, 0xc3e, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc15, 0xc4d, 0xc30, 0xc3e, 0xc38, 0xc4d, 0xc28, 0xc4b, 0xc2f, 0xc3e, 0xc30,
0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc3f, 0xc30, 0xc4d, 0xc17, 0xc3f, 0xc38, 0xc4d,
0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc32, 0xc02, 0xc15,
0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc32, 0xc48, 0xc28, 0xc4d, 0x20, 0xc26,
0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc32, 0xc3e, 0xc30,
0xc4d, 0xc21, 0xc4d, 0x20, 0xc39, 0xc4b, 0xc35, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc32, 0xc3e, 0xc30, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc39, 0xc4b, 0xc35, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc32, 0xc3e, 0xc30, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc39, 0xc4b,
0xc35, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2e, 0xc15, 0xc3e, 0xc35, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc15,
0xc3e, 0xc35, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc15, 0xc3e, 0xc35, 0xc4d, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc17,
0xc21, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc17, 0xc21,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc17, 0xc21, 0xc3e, 0xc28, 0xc4d,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e,
0xc32, 0xc47, 0xc37, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e,
0xc3e, 0xc32, 0xc4d, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc15, 0xc4d, 0xc35, 0xc47, 0xc38, 0xc3e, 0xc38,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc37, 0xc32,
0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc37, 0xc38, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc37, 0xc38, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e,
0xc30, 0xc3f, 0xc37, 0xc38, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc38, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2e, 0xc46, 0xc15, 0xc4d, 0xc38, 0xc3f, 0xc15, 0xc28, 0xc4d, 0x20,
0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2e, 0xc46, 0xc15, 0xc4d, 0xc38, 0xc3f, 0xc15, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc38,
0xc3f, 0xc2b, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc46, 0xc15, 0xc4d,
0xc38, 0xc3f, 0xc15, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f, 0xc15,
0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24,
0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc32, 0xc28, 0xc4d,
0x20, 0xc2c, 0xc24, 0xc4b, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09,
0xc32, 0xc28, 0xc4d, 0x20, 0xc2c, 0xc24, 0xc4b, 0xc30, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc09, 0xc32, 0xc28, 0xc4d, 0x20, 0xc2c, 0xc24, 0xc4b, 0xc30, 0xc4d, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc38,
0xc4d, 0xc15, 0xc4b, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc38, 0xc4d,
0xc15, 0xc4b, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc4b, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc2f,
0xc28, 0xc4d, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28,
0xc4c, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc47, 0xc2a, 0xc3e,
0xc32, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0x20,
0xc15, 0xc3e, 0xc32, 0xc46, 0xc21, 0xc4b, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0x20, 0xc15, 0xc3e, 0xc32, 0xc46,
0xc21, 0xc4b, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f,
0xc42, 0x20, 0xc15, 0xc3e, 0xc32, 0xc46, 0xc21, 0xc4b, 0xc28, 0xc3f, 0xc2f, 0xc3e,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28,
0xc4d, 0xc2f, 0xc42, 0xc1c, 0xc3f, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0xc1c, 0xc3f, 0xc32,
0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f,
0xc42, 0xc1c, 0xc3f, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc2a,
0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0xc2b, 0xc4c, 0xc02,
0xc21, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0xc2b, 0xc4c, 0xc02, 0xc21, 0xc4d,
0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28,
0xc4d, 0xc2f, 0xc42, 0xc2b, 0xc4c, 0xc02, 0xc21, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f,
0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46,
0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28,
0xc3f, 0xc2f, 0xc42, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc3e, 0xc30, 0xc4d,
0xc2b, 0xc4b, 0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc28, 0xc3e, 0xc30, 0xc4d, 0xc2b, 0xc4b, 0xc15, 0xc4d, 0x20, 0xc26,
0xc40, 0xc35, 0xc3f, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc3e, 0xc30, 0xc4d, 0xc2b, 0xc4b,
0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc46, 0xc30, 0xc4d, 0xc28, 0xc3e, 0xc02,
0xc21, 0xc4b, 0x20, 0xc21, 0xc3f, 0x20, 0xc28, 0xc4a, 0xc30, 0xc4b, 0xc28, 0xc4d,
0xc39, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc46, 0xc30, 0xc4d, 0xc28,
0xc3e, 0xc02, 0xc21, 0xc4b, 0x20, 0xc21, 0xc3f, 0x20, 0xc28, 0xc4a, 0xc30, 0xc4b,
0xc28, 0xc4d, 0xc39, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc46, 0xc30, 0xc4d, 0xc28,
0xc3e, 0xc02, 0xc21, 0xc4b, 0x20, 0xc21, 0xc3f, 0x20, 0xc28, 0xc4a, 0xc30, 0xc4b,
0xc28, 0xc4d, 0xc39, 0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc09, 0xc24, 0xc4d, 0xc24, 0xc30, 0x20, 0xc2e, 0xc3e, 0xc30,
0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc3e, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4b, 0xc35, 0xc4b, 0xc38, 0xc3f, 0xc2c, 0xc3f,
0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28,
0xc4b, 0xc35, 0xc4b, 0xc38, 0xc3f, 0xc2c, 0xc3f, 0xc30, 0xc4d, 0xc15, 0xc4d, 0xc38,
0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4b, 0xc35, 0xc4b, 0xc38, 0xc3f, 0xc2c, 0xc3f,
0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc13, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc13, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc13, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc3e, 0xc15,
0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2a, 0xc3e, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2a, 0xc3e, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc3e,
0xc32, 0xc3e, 0xc35, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc3e, 0xc2a,
0xc41, 0xc35, 0xc3e, 0x20, 0xc28, 0xc4d, 0xc2f, 0xc42, 0x20, 0xc17, 0xc3f, 0xc28,
0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc30, 0xc3e, 0xc17,
0xc4d, 0xc35, 0xc47, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc30, 0xc3e, 0xc17,
0xc4d, 0xc35, 0xc47, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc30, 0xc3e, 0xc17, 0xc4d, 0xc35,
0xc47, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2a, 0xc46, 0xc30, 0xc42, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc46, 0xc30,
0xc42, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc46, 0xc30, 0xc42, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc32, 0xc3f, 0xc2a,
0xc4d, 0xc2a, 0xc48, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f,
0xc32, 0xc3f, 0xc2a, 0xc4d, 0xc2a, 0xc48, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b,
0xc3f, 0xc32, 0xc3f, 0xc2a, 0xc4d, 0xc2a, 0xc48, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc28, 0xc3f,
0xc15, 0xc4d, 0xc38, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc2a,
0xc3f, 0xc2f, 0xc46, 0xc30, 0xc4d, 0x20, 0xc2e, 0xc30, 0xc3f, 0xc2f, 0xc41, 0x20,
0xc2e, 0xc3f, 0xc15, 0xc4d, 0xc35, 0xc46, 0xc32, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc2a,
0xc3f, 0xc2f, 0xc46, 0xc30, 0xc4d, 0x20, 0xc2e, 0xc30, 0xc3f, 0xc2f, 0xc41, 0x20,
0xc2e, 0xc3f, 0xc15, 0xc4d, 0xc35, 0xc46, 0xc32, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc2a, 0xc3f, 0xc2f,
0xc30, 0xc4d, 0x20, 0xc2e, 0xc30, 0xc3f, 0xc2f, 0xc41, 0x20, 0xc2e, 0xc3f, 0xc15,
0xc4d, 0xc35, 0xc46, 0xc32, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f,
0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2a, 0xc3f, 0xc1f, 0xc4d, 0x200c, 0xc15, 0xc48, 0xc30, 0xc4d, 0xc28,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc4a, 0xc28, 0xc47, 0xc2a, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc4d, 0xc2f, 0xc4b, 0xc02, 0xc17, 0xc3e,
0xc02, 0xc17, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc3f, 0xc1c, 0xc3f,
0xc32, 0xc4b, 0xc30, 0xc4d, 0xc21, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15,
0xc3f, 0xc1c, 0xc3f, 0xc32, 0xc4b, 0xc30, 0xc4d, 0xc21, 0xc3e, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc15, 0xc3f, 0xc1c, 0xc3f, 0xc32, 0xc4b, 0xc30, 0xc4d, 0xc21, 0xc3e, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc30, 0xc40, 0xc2f,
0xc42, 0xc28, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc30,
0xc4a, 0xc24, 0xc47, 0xc30, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc16,
0xc3e, 0xc32, 0xc3f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc16,
0xc3e, 0xc32, 0xc3f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc16, 0xc3e, 0xc32,
0xc3f, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc38, 0xc2e, 0xc3e, 0xc30, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc38, 0xc2e, 0xc3e, 0xc30, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc2e, 0xc3e, 0xc30,
0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc38, 0xc2e, 0xc4b, 0xc35, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc2e,
0xc4b, 0xc35, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc2e, 0xc4b, 0xc35, 0xc3e, 0x20,
0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30,
0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc40, 0xc37, 0xc46, 0xc32, 0xc4d,
0xc38, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc3f, 0xc02, 0xc17, 0xc2a,
0xc42, 0xc30, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc4b, 0xc32, 0xc2e, 0xc28, 0xc4d,
0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc26,
0xc15, 0xc4d, 0xc37, 0xc3f, 0xc23, 0x20, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f,
0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc42, 0xc30, 0xc3f, 0xc28,
0xc3e, 0xc2e, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc4d, 0xc2f, 0xc4b,
0xc35, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc39, 0xc3f, 0xc24, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc48, 0xc2a, 0xc40, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc24, 0xc48, 0xc2a, 0xc40, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc48, 0xc2a,
0xc40, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24,
0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc1c, 0xc3f, 0xc15,
0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc1f, 0xc4b, 0xc15, 0xc46, 0xc32, 0xc3e, 0xc35, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc1f, 0xc3e, 0xc02, 0xc17, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1f,
0xc3e, 0xc02, 0xc17, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1f, 0xc3e, 0xc02, 0xc17, 0xc3e,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a,
0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc41, 0xc30, 0xc4d, 0xc15,
0xc4d, 0x200c, 0xc2e, 0xc46, 0xc28, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc41, 0xc30, 0xc4d, 0xc15, 0xc4d, 0x200c,
0xc2e, 0xc46, 0xc28, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc24, 0xc41, 0xc30, 0xc4d, 0xc15, 0xc4d, 0x200c, 0xc2e, 0xc46, 0xc28, 0xc3f,
0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc41, 0xc35, 0xc3e, 0xc32, 0xc41, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc30, 0xc41, 0xc17, 0xc4d, 0xc35, 0xc47, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc30, 0xc41, 0xc17, 0xc4d, 0xc35, 0xc47, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc09, 0xc30, 0xc41, 0xc17, 0xc4d, 0xc35, 0xc47, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc1c, 0xc4d, 0xc2c,
0xc46, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc09, 0xc1c, 0xc4d, 0xc2c, 0xc46, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc1c, 0xc4d, 0xc2c, 0xc46, 0xc15,
0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc28, 0xc4c, 0xc1f, 0xc41, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc28, 0xc4c, 0xc1f, 0xc41, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc35, 0xc28, 0xc4c, 0xc1f, 0xc41, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc46, 0xc28, 0xc3f, 0xc1c, 0xc41, 0xc32, 0xc3e,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc4d, 0xc32, 0xc3e, 0xc21, 0xc3f, 0xc35,
0xc4b, 0xc38, 0xc4d, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc35, 0xc4d, 0xc32, 0xc3e, 0xc21, 0xc3f, 0xc35, 0xc4b, 0xc38, 0xc4d, 0xc1f, 0xc4b,
0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc4d, 0xc32, 0xc3e, 0xc21, 0xc3f, 0xc35,
0xc4b, 0xc38, 0xc4d, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc4b, 0xc32, 0xc4d, 0xc17, 0xc4b,
0xc17, 0xc4d, 0xc30, 0xc3e, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35,
0xc4b, 0xc32, 0xc4d, 0xc17, 0xc4b, 0xc17, 0xc4d, 0xc30, 0xc3e, 0xc21, 0xc4d, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc35, 0xc4b, 0xc32, 0xc4d, 0xc17, 0xc4b, 0xc17, 0xc4d, 0xc30, 0xc3e,
0xc21, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc35, 0xc47, 0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc35, 0xc3e, 0xc32, 0xc40, 0xc38, 0xc4d, 0x20, 0xc2e, 0xc30,
0xc3f, 0xc2f, 0xc41, 0x20, 0xc2b, 0xc41, 0xc1f, 0xc41, 0xc28, 0xc3e, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc3e, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc3e, 0xc15, 0xc41, 0xc1f, 0xc4d,
0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc3e, 0xc15, 0xc41, 0xc1f,
0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc46, 0xc15, 0xc1f, 0xc46, 0xc30, 0xc3f, 0xc28,
0xc4d, 0x200c, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2f, 0xc46, 0xc15, 0xc1f, 0xc46, 0xc30, 0xc3f, 0xc28, 0xc4d, 0x200c, 0xc2c, 0xc30,
0xc4d, 0xc17, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc46, 0xc15, 0xc1f, 0xc46, 0xc30,
0xc3f, 0xc28, 0xc4d, 0x200c, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc41, 0xc15, 0xc4b,
0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe32, 0xe40, 0xe01, 0xe23, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe40, 0xe01, 0xe23, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe32, 0xe40, 0xe01, 0xe23,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe31, 0xe1f, 0xe01, 0xe32, 0xe19, 0xe34, 0xe2a,
0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34,
0xe01, 0xe32, 0xe01, 0xe25, 0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d,
0xe1f, 0xe23, 0xe34, 0xe01, 0xe32, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d,
0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34, 0xe01, 0xe32,
0xe43, 0xe15, 0xe49, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34,
0xe01, 0xe32, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe2d, 0xe1f, 0xe23,
0xe34, 0xe01, 0xe32, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe41, 0xe2d, 0xe1f,
0xe23, 0xe34, 0xe01, 0xe32, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe30, 0xe41, 0xe25, 0xe2a, 0xe01, 0xe32, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe30, 0xe41,
0xe25, 0xe2a, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41,
0xe2a, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe30, 0xe41, 0xe25, 0xe2a, 0xe01, 0xe32,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe31, 0xe25, 0xe21, 0xe32, 0xe15, 0xe35, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe31,
0xe25, 0xe21, 0xe32, 0xe15, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe31, 0xe25, 0xe21, 0xe32, 0xe15, 0xe35, 0xe40,
0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe21, 0xe30, 0xe0b, 0xe2d, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe2d, 0xe21,
0xe30, 0xe0b, 0xe2d, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe41, 0xe2d, 0xe21, 0xe30, 0xe0b, 0xe2d, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe43, 0xe19, 0xe2d,
0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe15, 0xe2d,
0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34,
0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07,
0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19,
0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27,
0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34,
0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27,
0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34,
0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31,
0xe19, 0xe2d, 0xe2d, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01,
0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe16,
0xe1a, 0xe20, 0xe39, 0xe40, 0xe02, 0xe32, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23,
0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe16, 0xe1a, 0xe20, 0xe39,
0xe40, 0xe02, 0xe32, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32,
0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21,
0xe41, 0xe2a, 0xe07, 0xe41, 0xe16, 0xe1a, 0xe20, 0xe39, 0xe40, 0xe02, 0xe32, 0xe43,
0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37,
0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01,
0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19,
0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40,
0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe41, 0xe1b, 0xe0b, 0xe34,
0xe1f, 0xe34, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32,
0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe30, 0xe19,
0xe32, 0xe14, 0xe35, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe30, 0xe19, 0xe32, 0xe14, 0xe35, 0xe23, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d,
0xe30, 0xe19, 0xe32, 0xe14, 0xe35, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe32, 0xe1b, 0xe35, 0xe2d, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe1b, 0xe35, 0xe2d, 0xe32, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe2d, 0xe32, 0xe1b, 0xe35,
0xe2d, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe31, 0xe04, 0xe15, 0xe32, 0xe2d,
0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe2d, 0xe31, 0xe04, 0xe15, 0xe32, 0xe2d, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe31, 0xe04, 0xe15, 0xe32, 0xe2d,
0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe31, 0xe04, 0xe42, 0xe17, 0xe40, 0xe1a,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d,
0xe31, 0xe04, 0xe42, 0xe17, 0xe40, 0xe1a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe31, 0xe04, 0xe42, 0xe17, 0xe40, 0xe1a,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32, 0xe2b, 0xe23, 0xe31, 0xe1a, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe2b,
0xe23, 0xe31, 0xe1a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a,
0xe07, 0xe2d, 0xe32, 0xe2b, 0xe23, 0xe31, 0xe1a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe32, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe23,
0xe4c, 0xe40, 0xe08, 0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40,
0xe08, 0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe15, 0xe30,
0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe32, 0xe23, 0xe4c,
0xe40, 0xe08, 0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27,
0xe31, 0xe19, 0xe15, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40,
0xe08, 0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31,
0xe19, 0xe15, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe08,
0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32, 0xe23,
0xe4c, 0xe40, 0xe21, 0xe40, 0xe19, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe21,
0xe40, 0xe19, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe21, 0xe40, 0xe19, 0xe35,
0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe15, 0xe41, 0xe25, 0xe19, 0xe15,
0xe34, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe41, 0xe2d, 0xe15, 0xe41, 0xe25, 0xe19, 0xe15, 0xe34, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe41,
0xe2d, 0xe15, 0xe41, 0xe25, 0xe19, 0xe15, 0xe34, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe01, 0xe25,
0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe02,
0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe17, 0xe32,
0xe07, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d,
0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25,
0xe32, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe15,
0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a,
0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27,
0xe31, 0xe19, 0xe15, 0xe01, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe02,
0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe17, 0xe32,
0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe15, 0xe2d, 0xe19, 0xe01,
0xe25, 0xe32, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23,
0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe2a, 0xe40,
0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d,
0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01,
0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35,
0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe17,
0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe02, 0xe2d,
0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35,
0xe22, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30,
0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40,
0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d,
0xe21, 0xe41, 0xe2a, 0xe07, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe15, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40,
0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32, 0xe40, 0xe0b, 0xe2d,
0xe23, 0xe4c, 0xe44, 0xe1a, 0xe08, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe40, 0xe0b, 0xe2d, 0xe23,
0xe4c, 0xe44, 0xe1a, 0xe08, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe32, 0xe40, 0xe0b, 0xe2d, 0xe23, 0xe4c,
0xe44, 0xe1a, 0xe08, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe30, 0xe42,
0xe0b, 0xe23, 0xe4c, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe2d, 0xe30, 0xe42, 0xe0b, 0xe23, 0xe4c, 0xe2a, 0xe40, 0xe27,
0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe30, 0xe42,
0xe0b, 0xe23, 0xe4c, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1a, 0xe31, 0xe07, 0xe01,
0xe25, 0xe32, 0xe40, 0xe17, 0xe28, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe1a, 0xe31, 0xe07, 0xe01, 0xe25, 0xe32, 0xe40, 0xe17,
0xe28, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe1a, 0xe31, 0xe07, 0xe01, 0xe25, 0xe32, 0xe40, 0xe17, 0xe28, 0xe40, 0xe27, 0xe25,
0xe32, 0xe20, 0xe39, 0xe0f, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe1a,
0xe25, 0xe34, 0xe40, 0xe27, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1a, 0xe23,
0xe32, 0xe0b, 0xe34, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe1a, 0xe23, 0xe32, 0xe0b, 0xe34, 0xe40,
0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe1a, 0xe23, 0xe32, 0xe0b, 0xe34, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40,
0xe27, 0xe25, 0xe32, 0xe1a, 0xe23, 0xe39, 0xe44, 0xe19, 0xe14, 0xe32, 0xe23, 0xe38,
0xe2a, 0xe0b, 0xe32, 0xe25, 0xe32, 0xe21, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe04,
0xe1b, 0xe40, 0xe27, 0xe34, 0xe23, 0xe4c, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe04, 0xe1b, 0xe40, 0xe27, 0xe34,
0xe23, 0xe4c, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe40, 0xe04, 0xe1b, 0xe40, 0xe27, 0xe34, 0xe23, 0xe4c, 0xe14, 0xe40,
0xe27, 0xe25, 0xe32, 0xe40, 0xe04, 0xe0b, 0xe35, 0xe22, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe0a, 0xe32, 0xe21, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe23, 0xe40, 0xe27, 0xe25,
0xe32, 0xe41, 0xe0a, 0xe17, 0xe31, 0xe21, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe0a, 0xe17, 0xe31, 0xe21, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe41, 0xe0a, 0xe17, 0xe31,
0xe21, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0a, 0xe34, 0xe25, 0xe35, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe0a, 0xe34, 0xe25, 0xe35,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe0a,
0xe34, 0xe25, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe08, 0xe35, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe08, 0xe35, 0xe19,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe08, 0xe35,
0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32, 0xe30, 0xe04, 0xe23, 0xe34,
0xe2a, 0xe15, 0xe4c, 0xe21, 0xe32, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21,
0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe42, 0xe04, 0xe42, 0xe04, 0xe2a, 0xe40,
0xe27, 0xe25, 0xe32, 0xe42, 0xe04, 0xe25, 0xe2d, 0xe21, 0xe40, 0xe1a, 0xe35, 0xe22,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe42,
0xe04, 0xe25, 0xe2d, 0xe21, 0xe40, 0xe1a, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe42, 0xe04, 0xe25, 0xe2d, 0xe21,
0xe40, 0xe1a, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48,
0xe40, 0xe01, 0xe32, 0xe30, 0xe04, 0xe38, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01,
0xe32, 0xe30, 0xe04, 0xe38, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe23, 0xe36,
0xe48, 0xe07, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2b, 0xe21, 0xe39,
0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe04, 0xe38, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32,
0xe04, 0xe34, 0xe27, 0xe1a, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe04, 0xe34, 0xe27, 0xe1a, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe04, 0xe34,
0xe27, 0xe1a, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe14, 0xe27, 0xe34, 0xe2a,
0xe40, 0xe27, 0xe25, 0xe32, 0xe14, 0xe39, 0xe21, 0xe2d, 0xe07, 0xe15, 0xe4c, 0xe14,
0xe39, 0xe23, 0xe4c, 0xe27, 0xe34, 0xe25, 0xe25, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe15, 0xe34, 0xe21, 0xe2d, 0xe23, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d,
0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32, 0xe30, 0xe2d, 0xe35,
0xe2a, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe01, 0xe32, 0xe30, 0xe2d, 0xe35, 0xe2a,
0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe40, 0xe01, 0xe32, 0xe30, 0xe2d, 0xe35, 0xe2a, 0xe40,
0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe2d, 0xe01, 0xe27,
0xe32, 0xe14, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe22, 0xe38, 0xe42,
0xe23, 0xe1b, 0xe01, 0xe25, 0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe01, 0xe25,
0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe01, 0xe25, 0xe32, 0xe07, 0xe40, 0xe27,
0xe25, 0xe32, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b,
0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe44, 0xe01, 0xe25, 0xe40,
0xe27, 0xe25, 0xe32, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31,
0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15,
0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32,
0xe30, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2b, 0xe21,
0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01, 0xe41,
0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe1f,
0xe2d, 0xe25, 0xe4c, 0xe01, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe1f, 0xe34, 0xe08, 0xe34, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe1f, 0xe34, 0xe08, 0xe34, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe1f, 0xe34, 0xe08, 0xe34, 0xe40,
0xe27, 0xe25, 0xe32, 0xe40, 0xe1f, 0xe23, 0xe19, 0xe0a, 0xe4c, 0xe40, 0xe01, 0xe35,
0xe22, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe1f, 0xe23, 0xe19, 0xe0a,
0xe4c, 0xe40, 0xe0b, 0xe32, 0xe40, 0xe17, 0xe34, 0xe23, 0xe4c, 0xe19, 0xe41, 0xe25,
0xe30, 0xe41, 0xe2d, 0xe19, 0xe15, 0xe32, 0xe23, 0xe4c, 0xe01, 0xe15, 0xe34, 0xe01,
0xe40, 0xe27, 0xe25, 0xe32, 0xe01, 0xe32, 0xe25, 0xe32, 0xe1b, 0xe32, 0xe42, 0xe01,
0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe01, 0xe21, 0xe40, 0xe1a, 0xe35, 0xe22,
0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe08, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe08,
0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe08, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe08, 0xe2d, 0xe23, 0xe4c,
0xe40, 0xe08, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48,
0xe40, 0xe01, 0xe32, 0xe30, 0xe01, 0xe34, 0xe25, 0xe40, 0xe1a, 0xe34, 0xe23, 0xe4c,
0xe15, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe01, 0xe23, 0xe35, 0xe19, 0xe34, 0xe0a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe01, 0xe23,
0xe35, 0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe15,
0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41, 0xe25,
0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c,
0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41, 0xe25,
0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe01, 0xe23, 0xe35,
0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15,
0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe01, 0xe27, 0xe21, 0xe40, 0xe27, 0xe25, 0xe32,
0xe01, 0xe31, 0xe25, 0xe1f, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe01, 0xe32, 0xe22,
0xe2d, 0xe32, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2e, 0xe32, 0xe27, 0xe32,
0xe22, 0x2d, 0xe2d, 0xe30, 0xe25, 0xe39, 0xe40, 0xe0a, 0xe35, 0xe22, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2e, 0xe32,
0xe27, 0xe32, 0xe22, 0x2d, 0xe2d, 0xe30, 0xe25, 0xe39, 0xe40, 0xe0a, 0xe35, 0xe22,
0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe2e,
0xe32, 0xe27, 0xe32, 0xe22, 0x2d, 0xe2d, 0xe30, 0xe25, 0xe39, 0xe40, 0xe0a, 0xe35,
0xe22, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2e, 0xe48, 0xe2d, 0xe07, 0xe01, 0xe07,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2e,
0xe48, 0xe2d, 0xe07, 0xe01, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe2e, 0xe48, 0xe2d, 0xe07, 0xe01, 0xe07, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2e, 0xe2d, 0xe1f, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2e, 0xe2d, 0xe1f, 0xe14, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2e, 0xe2d,
0xe1f, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34, 0xe19, 0xe40, 0xe14,
0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe2b, 0xe32, 0xe2a, 0xe21, 0xe38,
0xe17, 0xe23, 0xe2d, 0xe34, 0xe19, 0xe40, 0xe14, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe34, 0xe19, 0xe42, 0xe14, 0xe08, 0xe35, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe34, 0xe19, 0xe42, 0xe14, 0xe19, 0xe35, 0xe40, 0xe0b, 0xe35, 0xe22,
0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe34, 0xe19, 0xe42, 0xe14, 0xe19, 0xe35, 0xe40, 0xe0b, 0xe35, 0xe22, 0xe1d, 0xe31,
0xe48, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe34, 0xe19, 0xe42, 0xe14, 0xe19, 0xe35, 0xe40, 0xe0b, 0xe35,
0xe22, 0xe1d, 0xe31, 0xe48, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34, 0xe2b, 0xe23, 0xe48, 0xe32, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe34,
0xe2b, 0xe23, 0xe48, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21,
0xe41, 0xe2a, 0xe07, 0xe2d, 0xe34, 0xe2b, 0xe23, 0xe48, 0xe32, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe35, 0xe23, 0xe4c, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d,
0xe35, 0xe23, 0xe4c, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe35, 0xe23, 0xe4c,
0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34,
0xe2a, 0xe23, 0xe32, 0xe40, 0xe2d, 0xe25, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe34, 0xe2a, 0xe23, 0xe32, 0xe40, 0xe2d,
0xe25, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe2d,
0xe34, 0xe2a, 0xe23, 0xe32, 0xe40, 0xe2d, 0xe25, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0d,
0xe35, 0xe48, 0xe1b, 0xe38, 0xe48, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe0d, 0xe35, 0xe48, 0xe1b, 0xe38, 0xe48, 0xe19,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe0d, 0xe35,
0xe48, 0xe1b, 0xe38, 0xe48, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe31, 0xe21,
0xe0a, 0xe31, 0xe15, 0xe04, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe1b, 0xe42,
0xe15, 0xe23, 0xe1b, 0xe31, 0xe1f, 0xe25, 0xe2d, 0xe1f, 0xe2a, 0xe04, 0xe4c, 0x2d,
0xe04, 0xe31, 0xe21, 0xe0a, 0xe31, 0xe15, 0xe2a, 0xe01, 0xe35, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe40, 0xe1b, 0xe42, 0xe15,
0xe23, 0xe1b, 0xe31, 0xe1f, 0xe25, 0xe2d, 0xe1f, 0xe2a, 0xe04, 0xe4c, 0x2d, 0xe04,
0xe31, 0xe21, 0xe0a, 0xe31, 0xe15, 0xe2a, 0xe01, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32,
0xe04, 0xe32, 0xe0b, 0xe31, 0xe04, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe04, 0xe32, 0xe0b, 0xe31, 0xe04, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe15, 0xe30,
0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe32,
0xe0b, 0xe31, 0xe04, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32, 0xe2b, 0xe25, 0xe35,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40,
0xe01, 0xe32, 0xe2b, 0xe25, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21,
0xe41, 0xe2a, 0xe07, 0xe40, 0xe01, 0xe32, 0xe2b, 0xe25, 0xe35, 0xe40, 0xe27, 0xe25,
0xe32, 0xe04, 0xe2d, 0xe2a, 0xe44, 0xe23, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe23,
0xe31, 0xe2a, 0xe42, 0xe19, 0xe22, 0xe32, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe04, 0xe23, 0xe31, 0xe2a,
0xe42, 0xe19, 0xe22, 0xe32, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe04, 0xe23, 0xe31, 0xe2a, 0xe42, 0xe19,
0xe22, 0xe32, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe35, 0xe23,
0xe4c, 0xe01, 0xe35, 0xe0b, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32,
0xe25, 0xe31, 0xe07, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39,
0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe44, 0xe25, 0xe19, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe25, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe42, 0xe2e, 0xe27, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe25, 0xe2d, 0xe23,
0xe4c, 0xe14, 0xe42, 0xe2e, 0xe27, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d,
0xe21, 0xe41, 0xe2a, 0xe07, 0xe25, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe42, 0xe2e, 0xe27,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe40, 0xe01, 0xe4a, 0xe32, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe21, 0xe32,
0xe40, 0xe01, 0xe4a, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe21, 0xe32, 0xe40, 0xe01, 0xe4a, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe01, 0xe32, 0xe14, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe21, 0xe32, 0xe01, 0xe32, 0xe14,
0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe21, 0xe32, 0xe01, 0xe32, 0xe14, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe40, 0xe25, 0xe40, 0xe0b, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe31, 0xe25, 0xe14, 0xe35, 0xe1f, 0xe2a, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe04, 0xe0b, 0xe31, 0xe2a, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe21, 0xe32, 0xe23,
0xe4c, 0xe41, 0xe0a, 0xe25, 0xe25, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe2d,
0xe23, 0xe34, 0xe40, 0xe0a, 0xe35, 0xe22, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe21, 0xe2d, 0xe23, 0xe34, 0xe40, 0xe0a,
0xe35, 0xe22, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe02, 0xe2d, 0xe07, 0xe21, 0xe2d, 0xe23, 0xe34, 0xe40, 0xe0a, 0xe35,
0xe22, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe2d, 0xe27, 0xe4c, 0xe2a, 0xe31,
0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01,
0xe40, 0xe21, 0xe47, 0xe01, 0xe0b, 0xe34, 0xe42, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f,
0xe34, 0xe01, 0xe40, 0xe21, 0xe47, 0xe01, 0xe0b, 0xe34, 0xe42, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe41, 0xe1b, 0xe0b, 0xe34,
0xe1f, 0xe34, 0xe01, 0xe40, 0xe21, 0xe47, 0xe01, 0xe0b, 0xe34, 0xe42, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe39, 0xe25, 0xe32, 0xe19, 0xe1a, 0xe32, 0xe15, 0xe2d,
0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe2d, 0xe39, 0xe25, 0xe32, 0xe19, 0xe1a, 0xe32, 0xe15, 0xe2d, 0xe23, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d,
0xe39, 0xe25, 0xe32, 0xe19, 0xe1a, 0xe32, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe2d, 0xe2a, 0xe42, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe21, 0xe2d, 0xe2a, 0xe42, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe21, 0xe2d,
0xe2a, 0xe42, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1e, 0xe21, 0xe48, 0xe32, 0xe40,
0xe27, 0xe25, 0xe32, 0xe19, 0xe32, 0xe2d, 0xe39, 0xe23, 0xe39, 0xe40, 0xe27, 0xe25,
0xe32, 0xe40, 0xe19, 0xe1b, 0xe32, 0xe25, 0xe40, 0xe27, 0xe25, 0xe32, 0xe19, 0xe34,
0xe27, 0xe41, 0xe04, 0xe25, 0xe34, 0xe42, 0xe14, 0xe40, 0xe19, 0xe35, 0xe22, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe19, 0xe34,
0xe27, 0xe41, 0xe04, 0xe25, 0xe34, 0xe42, 0xe14, 0xe40, 0xe19, 0xe35, 0xe22, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe19, 0xe34,
0xe27, 0xe41, 0xe04, 0xe25, 0xe34, 0xe42, 0xe14, 0xe40, 0xe19, 0xe35, 0xe22, 0xe40,
0xe27, 0xe25, 0xe32, 0xe19, 0xe34, 0xe27, 0xe0b, 0xe35, 0xe41, 0xe25, 0xe19, 0xe14,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe19, 0xe34, 0xe27, 0xe0b, 0xe35, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe19, 0xe34, 0xe27, 0xe0b,
0xe35, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe19, 0xe34,
0xe27, 0xe1f, 0xe31, 0xe19, 0xe14, 0xe4c, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe19, 0xe34,
0xe27, 0xe1f, 0xe31, 0xe19, 0xe14, 0xe4c, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe19, 0xe34, 0xe27,
0xe1f, 0xe31, 0xe19, 0xe14, 0xe4c, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe19, 0xe35, 0xe2d, 0xe39, 0xe40, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32,
0xe40, 0xe01, 0xe32, 0xe30, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d, 0xe25, 0xe4c,
0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe40, 0xe01, 0xe32, 0xe30, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d, 0xe25, 0xe4c,
0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe40,
0xe01, 0xe32, 0xe30, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30,
0xe40, 0xe1f, 0xe2d, 0xe23, 0xe4c, 0xe19, 0xe31, 0xe19, 0xe42, 0xe14, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2b, 0xe21, 0xe39,
0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe40, 0xe1f, 0xe2d, 0xe23, 0xe4c, 0xe19, 0xe31,
0xe19, 0xe42, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe02, 0xe2d, 0xe07, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32,
0xe30, 0xe40, 0xe1f, 0xe2d, 0xe23, 0xe4c, 0xe19, 0xe31, 0xe19, 0xe42, 0xe14, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe21,
0xe32, 0xe40, 0xe23, 0xe35, 0xe22, 0xe19, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d,
0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe19, 0xe42, 0xe27, 0xe0b, 0xe35, 0xe1a, 0xe35,
0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe42, 0xe19, 0xe42, 0xe27, 0xe0b, 0xe35, 0xe1a, 0xe35, 0xe2a, 0xe04,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe42, 0xe19, 0xe42, 0xe27, 0xe0b, 0xe35, 0xe1a, 0xe35, 0xe2a, 0xe04, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe2d, 0xe21, 0xe2a,
0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe2d, 0xe2d, 0xe21, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1b,
0xe32, 0xe01, 0xe35, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe1b, 0xe32, 0xe01, 0xe35, 0xe2a, 0xe16,
0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe1b, 0xe32, 0xe01, 0xe35, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe1b, 0xe32, 0xe40, 0xe25, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1b, 0xe32,
0xe1b, 0xe31, 0xe27, 0xe19, 0xe34, 0xe27, 0xe01, 0xe34, 0xe19, 0xe35, 0xe40, 0xe27,
0xe25, 0xe32, 0xe1b, 0xe32, 0xe23, 0xe32, 0xe01, 0xe27, 0xe31, 0xe22, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe1b, 0xe32, 0xe23,
0xe32, 0xe01, 0xe27, 0xe31, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe1b, 0xe32, 0xe23, 0xe32, 0xe01, 0xe27, 0xe31, 0xe22,
0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe1b, 0xe23, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe1b, 0xe23, 0xe39, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe40, 0xe1b,
0xe23, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1f, 0xe34, 0xe25, 0xe34, 0xe1b, 0xe1b,
0xe34, 0xe19, 0xe2a, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe1f, 0xe34, 0xe25, 0xe34, 0xe1b, 0xe1b, 0xe34, 0xe19, 0xe2a,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe1f, 0xe34, 0xe25, 0xe34, 0xe1b, 0xe1b, 0xe34, 0xe19, 0xe2a, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe1f, 0xe34,
0xe19, 0xe34, 0xe01, 0xe0b, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe0b, 0xe07,
0xe1b, 0xe35, 0xe41, 0xe22, 0xe23, 0xe4c, 0xe41, 0xe25, 0xe30, 0xe21, 0xe35, 0xe40,
0xe01, 0xe2d, 0xe25, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe41, 0xe0b, 0xe07, 0xe1b, 0xe35, 0xe41, 0xe22, 0xe23, 0xe4c,
0xe41, 0xe25, 0xe30, 0xe21, 0xe35, 0xe40, 0xe01, 0xe2d, 0xe25, 0xe07, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe41,
0xe0b, 0xe07, 0xe1b, 0xe35, 0xe41, 0xe22, 0xe23, 0xe4c, 0xe41, 0xe25, 0xe30, 0xe21,
0xe35, 0xe40, 0xe01, 0xe2d, 0xe25, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1e, 0xe34,
0xe15, 0xe41, 0xe04, 0xe23, 0xe4c, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe1b,
0xe19, 0xe32, 0xe40, 0xe1b, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe1b, 0xe35, 0xe22,
0xe07, 0xe22, 0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe37, 0xe22, 0xe0b,
0xe34, 0xe25, 0xe2d, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe04, 0xe37, 0xe22, 0xe0b, 0xe34,
0xe25, 0xe2d, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe04, 0xe37, 0xe22, 0xe0b, 0xe34, 0xe25,
0xe2d, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe23,
0xe2d, 0xe39, 0xe19, 0xe35, 0xe22, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe23,
0xe18, 0xe35, 0xe23, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0b, 0xe32, 0xe04, 0xe32,
0xe25, 0xe34, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe0b, 0xe32, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe0b, 0xe32, 0xe04, 0xe32,
0xe25, 0xe34, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0b, 0xe32, 0xe21, 0xe32, 0xe23,
0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe0b, 0xe32, 0xe21, 0xe32, 0xe23, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe0b, 0xe32, 0xe21, 0xe32, 0xe23, 0xe32, 0xe40,
0xe27, 0xe25, 0xe32, 0xe0b, 0xe32, 0xe21, 0xe31, 0xe27, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe0b, 0xe32, 0xe21, 0xe31, 0xe27,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe0b,
0xe32, 0xe21, 0xe31, 0xe27, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe0b, 0xe40, 0xe0a,
0xe25, 0xe2a, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2a, 0xe34, 0xe07, 0xe04, 0xe42,
0xe1b, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40,
0xe01, 0xe32, 0xe30, 0xe42, 0xe0b, 0xe42, 0xe25, 0xe21, 0xe2d, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe40, 0xe0b, 0xe32, 0xe17, 0xe4c, 0xe08, 0xe2d, 0xe23, 0xe4c, 0xe40,
0xe08, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0b, 0xe39, 0xe23, 0xe34, 0xe19,
0xe32, 0xe40, 0xe21, 0xe40, 0xe27, 0xe25, 0xe32, 0xe44, 0xe0b, 0xe42, 0xe22, 0xe27,
0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe15, 0xe32, 0xe2e, 0xe35, 0xe15, 0xe35, 0xe40,
0xe27, 0xe25, 0xe32, 0xe44, 0xe17, 0xe40, 0xe1b, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe44, 0xe17, 0xe40, 0xe1b, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe44, 0xe17, 0xe40, 0xe1b,
0xe40, 0xe27, 0xe25, 0xe32, 0xe17, 0xe32, 0xe08, 0xe34, 0xe01, 0xe34, 0xe2a, 0xe16,
0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe15, 0xe40, 0xe01, 0xe40, 0xe25,
0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe15, 0xe2d, 0xe07, 0xe01, 0xe32, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe15, 0xe2d, 0xe07,
0xe01, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe15, 0xe2d, 0xe07, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0a, 0xe38,
0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe15, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe40,
0xe21, 0xe19, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe15, 0xe34, 0xe23, 0xe4c, 0xe01,
0xe40, 0xe21, 0xe19, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe40, 0xe15, 0xe34, 0xe23, 0xe4c,
0xe01, 0xe40, 0xe21, 0xe19, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe15, 0xe39, 0xe27, 0xe32, 0xe25, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe38, 0xe23, 0xe38, 0xe01, 0xe27, 0xe31, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe38, 0xe23, 0xe38, 0xe01, 0xe27,
0xe31, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe2d, 0xe38, 0xe23, 0xe38, 0xe01, 0xe27, 0xe31, 0xe22, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe38, 0xe0b, 0xe40, 0xe1a, 0xe01, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d,
0xe38, 0xe0b, 0xe40, 0xe1a, 0xe01, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe38, 0xe0b,
0xe40, 0xe1a, 0xe01, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32,
0xe27, 0xe32, 0xe19, 0xe39, 0xe2d, 0xe32, 0xe15, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe27, 0xe32, 0xe19, 0xe39, 0xe2d,
0xe32, 0xe15, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe27, 0xe32, 0xe19, 0xe39, 0xe2d, 0xe32, 0xe15, 0xe39, 0xe40, 0xe27,
0xe25, 0xe32, 0xe40, 0xe27, 0xe40, 0xe19, 0xe0b, 0xe38, 0xe40, 0xe2d, 0xe25, 0xe32,
0xe40, 0xe27, 0xe25, 0xe32, 0xe27, 0xe25, 0xe32, 0xe14, 0xe35, 0xe27, 0xe2d, 0xe2a,
0xe15, 0xe2d, 0xe04, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe27, 0xe25, 0xe32, 0xe14, 0xe35, 0xe27, 0xe2d, 0xe2a, 0xe15, 0xe2d,
0xe04, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe27, 0xe25, 0xe32, 0xe14, 0xe35, 0xe27, 0xe2d, 0xe2a, 0xe15, 0xe2d, 0xe04, 0xe40,
0xe27, 0xe25, 0xe32, 0xe27, 0xe2d, 0xe25, 0xe42, 0xe01, 0xe01, 0xe23, 0xe32, 0xe14,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe27,
0xe2d, 0xe25, 0xe42, 0xe01, 0xe01, 0xe23, 0xe32, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe27, 0xe2d, 0xe25, 0xe42, 0xe01,
0xe01, 0xe23, 0xe32, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe27, 0xe2d, 0xe2a, 0xe15,
0xe2d, 0xe04, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32, 0xe30, 0xe40, 0xe27,
0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe27, 0xe32, 0xe25, 0xe25, 0xe34, 0xe2a, 0xe41,
0xe25, 0xe30, 0xe1f, 0xe38, 0xe15, 0xe39, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32,
0xe22, 0xe32, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe22, 0xe32, 0xe04, 0xe38, 0xe15,
0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe22, 0xe32, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe40, 0xe22, 0xe04, 0xe32, 0xe40, 0xe15, 0xe23, 0xe34, 0xe19, 0xe1a,
0xe39, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe40, 0xe22, 0xe04, 0xe32, 0xe40, 0xe15, 0xe23, 0xe34, 0xe19,
0xe1a, 0xe39, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe40, 0xe22, 0xe04, 0xe32, 0xe40, 0xe15, 0xe23, 0xe34,
0xe19, 0xe1a, 0xe39, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe22, 0xe39,
0xe04, 0xe2d, 0xe19, 0x130d, 0x12d8, 0x20, 0x12a0, 0x12ad, 0x122a, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a3, 0x12ad, 0x122a, 0x130d,
0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a3, 0x12ad, 0x122a, 0x1293, 0x12ed,
0x20, 0x12a0, 0x134d, 0x130b, 0x1292, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x130d,
0x12dc, 0x20, 0x121b, 0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243,
0x130d, 0x12dc, 0x20, 0x121d, 0x1265, 0x122b, 0x1255, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243,
0x130d, 0x12dc, 0x20, 0x12f0, 0x1261, 0x1265, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x130d,
0x12dc, 0x20, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x121d, 0x12d5,
0x122b, 0x1265, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228,
0x121d, 0x1272, 0x20, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243,
0x130d, 0x12d8, 0x20, 0x12a0, 0x120b, 0x1235, 0x12ab, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a0, 0x120b, 0x1235, 0x12ab, 0x1293, 0x12ed,
0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8, 0x20, 0x12a0, 0x120b, 0x1235,
0x12ab, 0x130d, 0x12dc, 0x20, 0x12a3, 0x121b, 0x12de, 0x1295, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a3, 0x121b, 0x12de, 0x1295, 0x130d,
0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a3, 0x121b, 0x12de, 0x1295, 0x121b,
0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a0, 0x1218, 0x122a, 0x12ab, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x121b,
0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a0, 0x1218, 0x122a, 0x12ab, 0x1293, 0x12ed, 0x20,
0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8, 0x20, 0x121b, 0x12a5, 0x12a8, 0x120b,
0x12ed, 0x20, 0x12a0, 0x1218, 0x122a, 0x12ab, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b,
0x1253, 0x12ca, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x130d, 0x12d8, 0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca, 0x20, 0x12a3, 0x1218,
0x122a, 0x12ab, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8,
0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca, 0x20, 0x12a0, 0x1218, 0x122a, 0x12ab, 0x1293,
0x12ed, 0x20, 0x130e, 0x1266, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x130e, 0x1266, 0x20, 0x12a3, 0x1218, 0x122a,
0x12ab, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8, 0x20,
0x130e, 0x1266, 0x20, 0x12a0, 0x1218, 0x122a, 0x12ab, 0x1293, 0x12ed, 0x20, 0x1353, 0x1235,
0x134a, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x130d, 0x12d8, 0x20, 0x1353, 0x1235, 0x134a, 0x12ad, 0x1293, 0x12ed, 0x20, 0x1218,
0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8, 0x20, 0x1353, 0x1235, 0x134a, 0x12ad, 0x1293,
0x12ed, 0x20, 0x12a0, 0x1352, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x12a0, 0x1352, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x12a0, 0x1352, 0x12eb, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12a0, 0x1228, 0x1265, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a0, 0x1228, 0x1265, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x12a0, 0x1228, 0x1265, 0x20, 0x130d,
0x12d8, 0x130d, 0x12dc, 0x20, 0x12a3, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a3, 0x122d, 0x1300,
0x1295, 0x1272, 0x1293, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a3,
0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x130d, 0x12dc, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263,
0x12ca, 0x20, 0x12a3, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca,
0x20, 0x12a3, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228,
0x121d, 0x1272, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a3, 0x122d, 0x1300,
0x1295, 0x1272, 0x1293, 0x1293, 0x12ed, 0x20, 0x12a0, 0x122d, 0x121c, 0x1292, 0x12eb, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a0, 0x122d,
0x121c, 0x1292, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x12a0, 0x122d, 0x121c, 0x1292, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x12a0, 0x1275, 0x120b, 0x1295, 0x1272, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a0, 0x1275, 0x120b,
0x1295, 0x1272, 0x12ad, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d,
0x12d8, 0x20, 0x12a0, 0x1275, 0x120b, 0x1295, 0x1272, 0x12ad, 0x1293, 0x12ed, 0x20, 0x12a0,
0x12cd, 0x1235, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121b,
0x12d5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a0, 0x12cd,
0x1235, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121b, 0x12d5,
0x12a8, 0x120b, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x12a0, 0x12cd, 0x1235,
0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x12d5, 0x122b,
0x1263, 0x12ca, 0x20, 0x12a0, 0x12cd, 0x1235, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x12a0, 0x12cd, 0x1235, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272,
0x20, 0x12a0, 0x12cd, 0x1235, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca, 0x20, 0x12a3, 0x12cd, 0x1235, 0x1275, 0x122b,
0x120d, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b, 0x1253,
0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a3, 0x12cd, 0x1235, 0x1275, 0x122b,
0x120d, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b, 0x1253,
0x12ca, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x12a3, 0x12cd, 0x1235, 0x1275, 0x122b,
0x120d, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263,
0x12ca, 0x20, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x1218, 0x12d3, 0x120d,
0x1272, 0x20, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12a0, 0x12d8, 0x122d, 0x1263, 0x12e3, 0x1295, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a0, 0x12d8, 0x122d, 0x1263,
0x12e3, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272,
0x20, 0x12a0, 0x12d8, 0x122d, 0x1263, 0x12e3, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x12a0, 0x12de, 0x1228, 0x1235, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a3, 0x12de, 0x1228, 0x1235, 0x1293,
0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a0, 0x12de, 0x1228, 0x1235, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1263, 0x1295, 0x130d, 0x120b, 0x12f2, 0x123d, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1263, 0x1295,
0x130d, 0x120b, 0x12f2, 0x123d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228,
0x121d, 0x1272, 0x20, 0x1263, 0x1295, 0x130d, 0x120b, 0x12f2, 0x123d, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1261, 0x1205, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc,
0x20, 0x1266, 0x120a, 0x126d, 0x12eb, 0x130d, 0x12dc, 0x20, 0x1265, 0x122b, 0x12da, 0x120d,
0x12eb, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20,
0x1265, 0x122b, 0x12da, 0x120d, 0x12eb, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272,
0x20, 0x1265, 0x122b, 0x12da, 0x120d, 0x12eb, 0x1293, 0x12ed, 0x20, 0x1265, 0x1229, 0x1294,
0x20, 0x12f3, 0x1229, 0x1233, 0x120c, 0x121d, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20,
0x12ac, 0x1355, 0x20, 0x1268, 0x122d, 0x12f0, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d, 0x12f0, 0x130d,
0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d,
0x12f0, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x127b, 0x121e, 0x122e,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x127b, 0x1275, 0x1203, 0x121d, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x127b, 0x1275, 0x1203,
0x121d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20,
0x127b, 0x1275, 0x1203, 0x121d, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x127a, 0x120c,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x127a,
0x120c, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x127a, 0x120c, 0x1293,
0x12ed, 0x20, 0x127b, 0x12ed, 0x1293, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x127b, 0x12ed, 0x1293, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x127b, 0x12ed, 0x1293, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x120d, 0x12f0, 0x1275, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x12ae, 0x12ae, 0x1235, 0x20, 0x12f0, 0x1234, 0x1275, 0x20,
0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x12ae, 0x120e, 0x121d, 0x1265, 0x12eb, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12ae, 0x120e, 0x121d,
0x1265, 0x12eb, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12ae, 0x120e,
0x121d, 0x1265, 0x12eb, 0x1293, 0x12ed, 0x20, 0x12a9, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1275,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a9,
0x12ad, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x134d,
0x122d, 0x1242, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a9, 0x12ad, 0x20, 0x12f0,
0x1234, 0x1275, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a9, 0x1263, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20,
0x12a9, 0x1263, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x130d, 0x12d8,
0x20, 0x12a9, 0x1263, 0x1293, 0x12ed, 0x20, 0x12f4, 0x126a, 0x1235, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12f1, 0x121e, 0x2d, 0x12f1, 0x122d, 0x126a, 0x120d, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b, 0x1245, 0x20, 0x1272, 0x121e, 0x122d,
0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x134b, 0x1232,
0x12ab, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20,
0x12f0, 0x1234, 0x1275, 0x20, 0x134b, 0x1232, 0x12ab, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228,
0x121d, 0x1272, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x134b, 0x1232, 0x12ab, 0x130d, 0x12dc,
0x20, 0x12a4, 0x12b3, 0x12f6, 0x122d, 0x1293, 0x12ed, 0x20, 0x121b, 0x12a5, 0x12a8, 0x120b,
0x12ed, 0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x121b, 0x12a5, 0x12a8, 0x120b,
0x12ed, 0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b,
0x1255, 0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x121d, 0x1265, 0x122b, 0x1255,
0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272,
0x20, 0x121d, 0x1265, 0x122b, 0x1255, 0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x1293, 0x12ed,
0x20, 0x122d, 0x1211, 0x1255, 0x2d, 0x121d, 0x1265, 0x122b, 0x1255, 0x20, 0x12a4, 0x12cd,
0x122e, 0x1333, 0x12ca, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x12d5, 0x122b,
0x1263, 0x12ca, 0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x12ca, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca,
0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x12ca, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a4,
0x12cd, 0x122e, 0x1333, 0x12ca, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x12f0, 0x1234,
0x1273, 0x1275, 0x20, 0x134e, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12f0, 0x1234, 0x1273, 0x1275, 0x20,
0x134e, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x130d, 0x12dc, 0x20, 0x12a8, 0x1228, 0x121d, 0x1272,
0x20, 0x12f0, 0x1234, 0x1273, 0x1275, 0x20, 0x134e, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x1293,
0x12ed, 0x20, 0x134a, 0x1302, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x134a, 0x1302, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad,
0x1228, 0x121d, 0x1272, 0x20, 0x134a, 0x1302, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20,
0x1348, 0x1228, 0x1295, 0x1233, 0x12ca, 0x1275, 0x20, 0x130a, 0x12eb, 0x1293, 0x1293, 0x12ed,
0x20, 0x12f0, 0x1261, 0x1263, 0x12ca, 0x1295, 0x20, 0x12a3, 0x1295, 0x1273, 0x122d, 0x1272,
0x12ad, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x130b, 0x120b, 0x1353, 0x130e, 0x1235,
0x1293, 0x12ed, 0x20, 0x130b, 0x121d, 0x1262, 0x12e8, 0x122d, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1306, 0x122d, 0x1302, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1306, 0x122d, 0x1302, 0x12eb, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1306, 0x122d, 0x1302, 0x12eb,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x130a, 0x120d, 0x1260, 0x122d, 0x1275, 0x20,
0x12f0, 0x1234, 0x1273, 0x1275, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265,
0x122b, 0x1253, 0x12ca, 0x20, 0x130d, 0x12d8, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295,
0x12f5, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x121d, 0x1265, 0x122b,
0x1253, 0x12ca, 0x20, 0x130d, 0x12d8, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5,
0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x121d, 0x1265, 0x122b, 0x1253,
0x12ca, 0x20, 0x130d, 0x12d8, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x1293,
0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295,
0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x121d, 0x12d5,
0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1308, 0x120d, 0x134d, 0x20,
0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x1309, 0x12eb, 0x1293, 0x1293, 0x12ed, 0x20, 0x1203,
0x12cb, 0x12ed, 0x2d, 0x12a3, 0x120c, 0x12cd, 0x1272, 0x12eb, 0x1295, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1203, 0x12cb, 0x12ed, 0x2d,
0x12a3, 0x120c, 0x12cd, 0x1272, 0x12eb, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x1203, 0x12cb, 0x12ed, 0x2d, 0x12a3, 0x120c, 0x12cd,
0x1272, 0x12eb, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1206, 0x1295, 0x130d,
0x20, 0x12ae, 0x1295, 0x130d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295, 0x130d, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1206, 0x1295, 0x130d,
0x20, 0x12ae, 0x1295, 0x130d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1206, 0x126d,
0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1206, 0x126d, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x1206, 0x126d, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1205, 0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc,
0x20, 0x1205, 0x1295, 0x12f3, 0x12ca, 0x20, 0x12cd, 0x1245, 0x12eb, 0x1296, 0x1235, 0x1293,
0x12ed, 0x20, 0x12a2, 0x1295, 0x12f6, 0x127b, 0x12ed, 0x1293, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x121b, 0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1290,
0x12e5, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b, 0x1253,
0x12ca, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1290, 0x12e5, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1290,
0x12e5, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a2, 0x122b, 0x1295, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a2, 0x122b,
0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20,
0x12a2, 0x122b, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a2, 0x122d, 0x12a9,
0x1275, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235,
0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a5,
0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3,
0x120d, 0x1272, 0x20, 0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1303, 0x1353, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1303, 0x1353, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x1303, 0x1353, 0x1295, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12ab, 0x12db, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x121d, 0x1265, 0x122b, 0x1245, 0x20, 0x12ab, 0x12db, 0x12aa, 0x1235,
0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x12d5, 0x122b, 0x1265,
0x20, 0x12ab, 0x12db, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x12ae, 0x122a, 0x12eb, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x12ae, 0x122a, 0x12eb, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x12ae, 0x122a, 0x12eb, 0x1295, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ae, 0x122d, 0x1233, 0x12ed, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12ad, 0x122b, 0x1295, 0x1235, 0x1296, 0x12eb, 0x122d, 0x12ad, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12ad, 0x122b,
0x1295, 0x1235, 0x1296, 0x12eb, 0x122d, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12ad, 0x122b, 0x1295, 0x1235, 0x1296, 0x12eb, 0x122d,
0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x122d, 0x1305, 0x1235, 0x1273,
0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x120b, 0x12ed, 0x1295, 0x20, 0x12f0,
0x1234, 0x1273, 0x1275, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x120e, 0x122d, 0x12f5,
0x20, 0x1206, 0x12cd, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f5, 0x1260,
0x129b, 0x20, 0x120e, 0x122d, 0x12f5, 0x20, 0x1206, 0x12cd, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x120e, 0x122d, 0x12f5, 0x20, 0x1206,
0x12cd, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121c, 0x130b, 0x12f3, 0x1295, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x121c, 0x130b,
0x12f3, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272,
0x20, 0x121c, 0x130b, 0x12f3, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121b,
0x120c, 0x12e2, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121e, 0x120d, 0x12f2,
0x126d, 0x1235, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121b, 0x122d, 0x12a9, 0x12cc,
0x1233, 0x1235, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121b, 0x122d, 0x123b, 0x120d,
0x20, 0x12f0, 0x1234, 0x1273, 0x1275, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x121b,
0x12cd, 0x122a, 0x1238, 0x1235, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x130d, 0x12d8, 0x20, 0x121b, 0x12cd, 0x122a, 0x1238, 0x1235, 0x130d, 0x12dc, 0x20, 0x12ad,
0x1228, 0x121d, 0x1272, 0x20, 0x121b, 0x12cd, 0x122a, 0x1238, 0x1235, 0x1293, 0x12ed, 0x20,
0x121b, 0x12cd, 0x1236, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121c, 0x12ad,
0x1232, 0x12ae, 0x20, 0x1353, 0x1235, 0x134a, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x121c, 0x12ad, 0x1232, 0x12ae, 0x20, 0x1353,
0x1235, 0x134a, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d,
0x1272, 0x20, 0x121c, 0x12ad, 0x1232, 0x12ae, 0x20, 0x1353, 0x1235, 0x134a, 0x12ad, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a1, 0x120b, 0x1295, 0x1263, 0x1273, 0x122d, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a1, 0x120b,
0x1295, 0x1263, 0x1273, 0x122d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228,
0x121d, 0x1272, 0x20, 0x12a1, 0x120b, 0x1295, 0x1263, 0x1273, 0x122d, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x121e, 0x1235, 0x12ae, 0x12cd, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x121e, 0x1235, 0x12ae, 0x12cd, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x121e, 0x1235, 0x12ae,
0x12cd, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x121d, 0x12eb, 0x1295, 0x121b, 0x122d,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1293, 0x12cd, 0x1229, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1294, 0x1353, 0x120d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1292, 0x12cd, 0x20, 0x12ab, 0x120c, 0x12f6, 0x1292, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1292, 0x12cd, 0x20, 0x12ab, 0x120c,
0x12f6, 0x1292, 0x12eb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x1292, 0x12cd, 0x20, 0x12ab, 0x120c, 0x12f6, 0x1292, 0x12eb, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1292, 0x12cd,
0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12d3, 0x120d, 0x1272, 0x20, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b,
0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x1292, 0x12cd, 0x134b,
0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x1292, 0x12cc, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1296, 0x122d, 0x134e, 0x120d,
0x12ad, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x20, 0x12f0, 0x1234,
0x1275, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20,
0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x130d, 0x12d8,
0x130d, 0x12dc, 0x20, 0x1348, 0x122d, 0x1293, 0x1295, 0x12f6, 0x20, 0x12f0, 0x20, 0x1296,
0x122e, 0x1295, 0x12eb, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d,
0x12d8, 0x20, 0x1348, 0x122d, 0x1293, 0x1295, 0x12f6, 0x20, 0x12f0, 0x20, 0x1296, 0x122e,
0x1295, 0x12eb, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1348, 0x122d,
0x1293, 0x1295, 0x12f6, 0x20, 0x12f0, 0x20, 0x1296, 0x122e, 0x1295, 0x12eb, 0x1293, 0x12ed,
0x20, 0x1296, 0x126e, 0x1232, 0x122a, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1296, 0x126e, 0x1232, 0x122a, 0x1235, 0x12ad,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1296,
0x126e, 0x1232, 0x122a, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12a6,
0x121d, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x12a6, 0x121d, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a6, 0x121d, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1353, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1353, 0x12aa, 0x1235, 0x1273, 0x1295,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1353,
0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1353, 0x120b,
0x12cd, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1353, 0x1357, 0x20, 0x1292, 0x12cd,
0x20, 0x130a, 0x1292, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x1353, 0x122b, 0x1313,
0x12ed, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20,
0x1353, 0x122b, 0x1313, 0x12ed, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20,
0x1353, 0x122b, 0x1313, 0x12ed, 0x130d, 0x12dc, 0x20, 0x1354, 0x1229, 0x1293, 0x12ed, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x1354, 0x1229, 0x130d, 0x12dc,
0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1354, 0x1229, 0x1293, 0x12ed, 0x20, 0x134a,
0x120a, 0x1352, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x134a, 0x120a, 0x1352, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x134a, 0x120a, 0x1352, 0x1295, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x134a, 0x1292, 0x12ad, 0x1235, 0x20, 0x12f0, 0x1234, 0x1273, 0x1275,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1245, 0x12f1, 0x1235, 0x20, 0x1352, 0x12e8,
0x122d, 0x1295, 0x20, 0x121a, 0x12a8, 0x120e, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1245, 0x12f1, 0x1235, 0x20, 0x1352, 0x12e8,
0x122d, 0x1295, 0x20, 0x121a, 0x12a8, 0x120e, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12d3, 0x120d, 0x1272, 0x20, 0x1245, 0x12f1, 0x1235, 0x20, 0x1352, 0x12e8,
0x122d, 0x1295, 0x20, 0x121a, 0x12a8, 0x120e, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1352, 0x1275, 0x127b, 0x12ed, 0x122d, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1356, 0x1293, 0x1354, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1355, 0x12ee,
0x1295, 0x130d, 0x12eb, 0x1295, 0x130d, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x122d,
0x12e9, 0x1295, 0x12e8, 0x1295, 0x1293, 0x12ed, 0x20, 0x122e, 0x1274, 0x122b, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1233, 0x12ad, 0x1203, 0x120a, 0x1295, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1233, 0x12ad, 0x1203, 0x120a,
0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20,
0x1233, 0x12ad, 0x1203, 0x120a, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1233,
0x121e, 0x12cb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1233, 0x121e, 0x12cb, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3,
0x120d, 0x1272, 0x20, 0x1233, 0x121e, 0x12cb, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20,
0x1232, 0x1238, 0x120d, 0x1235, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1232, 0x130b, 0x1356, 0x122d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1236, 0x120e,
0x121e, 0x1295, 0x20, 0x12f0, 0x1234, 0x1273, 0x1275, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc,
0x20, 0x12f0, 0x1261, 0x1265, 0x20, 0x1306, 0x122d, 0x1305, 0x12eb, 0x130d, 0x12dc, 0x20,
0x1231, 0x122a, 0x1293, 0x121d, 0x1293, 0x12ed, 0x20, 0x1235, 0x12ee, 0x12cb, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1272, 0x1202, 0x1272, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1274, 0x1352, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1274, 0x1352, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12d3,
0x120d, 0x1272, 0x20, 0x1274, 0x1352, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1273,
0x1303, 0x12ad, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1276,
0x12ac, 0x120b, 0x12cd, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1276, 0x1295, 0x130b,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1276,
0x1295, 0x130b, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272,
0x20, 0x1276, 0x1295, 0x130b, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1279, 0x12ad,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1271, 0x122d, 0x12ad, 0x121c, 0x1292, 0x1235,
0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1271, 0x122d, 0x12ad, 0x121c, 0x1292, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1271, 0x122d, 0x12ad, 0x121c,
0x1292, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1271, 0x126b,
0x1209, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x12a1, 0x122b, 0x1313, 0x12ed, 0x1293,
0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x130d, 0x12d8, 0x20, 0x12a1, 0x122b,
0x1313, 0x12ed, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a1, 0x122b,
0x1313, 0x12ed, 0x1293, 0x12ed, 0x20, 0x12a1, 0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273, 0x1295,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12a1,
0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a1, 0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273, 0x1295,
0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x126b, 0x1297, 0x1273, 0x12cd, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x126b, 0x1297, 0x1273,
0x12cd, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20,
0x126b, 0x1297, 0x1273, 0x12cd, 0x20, 0x130d, 0x12d8, 0x130d, 0x12dc, 0x20, 0x126c, 0x1294,
0x12dd, 0x12cc, 0x120b, 0x1293, 0x12ed, 0x20, 0x126d, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276,
0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x126d, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x126d, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276,
0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x126e, 0x120d, 0x130e, 0x130d, 0x122b,
0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x126e, 0x120d, 0x130e, 0x130d, 0x122b, 0x12f5, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20,
0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x126e, 0x120d, 0x130e, 0x130d, 0x122b, 0x12f5, 0x20,
0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12cc, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12cc, 0x120d, 0x1235, 0x1295, 0x20, 0x1349, 0x1277, 0x20, 0x130d,
0x12d8, 0x1293, 0x12ed, 0x20, 0x12eb, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12eb, 0x12a9, 0x1275, 0x1235,
0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20,
0x12eb, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12eb,
0x12ad, 0x1270, 0x122d, 0x1292, 0x1260, 0x122d, 0x130d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12eb, 0x12ad, 0x1270, 0x122d, 0x1292, 0x1260,
0x122d, 0x130d, 0x20, 0x130d, 0x12d8, 0x1293, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272,
0x20, 0x12eb, 0x12ad, 0x1270, 0x122d, 0x1292, 0x1260, 0x122d, 0x130d, 0x20, 0x130d, 0x12d8,
0x1293, 0x12ed, 0x20, 0x12e9, 0x12ae, 0x1295, 0x20, 0x130d, 0x12d8, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x65, 0x6c, 0x12b,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b,
0x65, 0x6c, 0x12b, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6b, 0x65, 0x6c, 0x12b, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x66, 0x69, 0x6b, 0x101, 0x6e, 0x69,
0x73, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d,
0x6c, 0x6f, 0x74, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x61,
0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x69, 0x68,
0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x69, 0x68,
0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x69,
0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x73, 0x69,
0x6b, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x74,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6c, 0x61, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d,
0x61, 0x73, 0x14d, 0x6e, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x61, 0x73, 0x14d, 0x6e, 0x65, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x61,
0x73, 0x14d, 0x6e, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6c, 0x6f, 0x74, 0x6f, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c,
0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20,
0x6c, 0x6f, 0x74, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74,
0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6c, 0x6f, 0x74, 0x6f, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65,
0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x20, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61,
0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x68, 0x61, 0x68,
0x61, 0x6b, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d,
0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x20, 0x6d, 0x6f, 0x2bb, 0x75, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69,
0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6d,
0x6f, 0x2bb, 0x75, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61,
0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6d, 0x6f, 0x2bb,
0x75, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x2bb, 0x61, 0x6e, 0x61, 0x74, 0x69, 0x6c, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x2bb, 0x61, 0x6e, 0x61, 0x74, 0x69, 0x6c, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x2bb, 0x61, 0x6e, 0x61, 0x74, 0x69, 0x6c, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x61, 0x70, 0x69, 0x61, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x61, 0x70, 0x69, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x61, 0x70, 0x69,
0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6b, 0x69, 0x74, 0x61, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x69, 0x74,
0x61, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x6b, 0x69, 0x74, 0x14d, 0x70, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x69, 0x74, 0x14d, 0x70, 0x65,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6b, 0x69, 0x74, 0x14d, 0x70, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x65, 0x70, 0x65, 0x61, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x65, 0x70,
0x65, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x6c, 0x65, 0x70, 0x65, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69,
0x6e, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e,
0x69, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69, 0x6e, 0x61,
0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69,
0x6e, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e,
0x69, 0x74, 0x69, 0x6e, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x101, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x2bb, 0x61, 0x74, 0x61, 0x6c, 0x61, 0x6e, 0x69, 0x74,
0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x20, 0x2bb, 0x61, 0x74, 0x61, 0x6c, 0x61, 0x6e,
0x69, 0x74, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d,
0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x2bb, 0x61, 0x74, 0x61,
0x6c, 0x61, 0x6e, 0x69, 0x74, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65,
0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69,
0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c,
0x6f, 0x74, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61,
0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f,
0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f,
0x74, 0x6f, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69,
0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f,
0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c,
0x113, 0x6c, 0x69, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73,
0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x68, 0x61, 0x68,
0x61, 0x6b, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69,
0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65,
0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f,
0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x68, 0x69,
0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c,
0x69, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x61, 0x70, 0x61,
0x69, 0x73, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x61, 0x70, 0x61, 0x69, 0x73, 0x61, 0x6e,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x73, 0x61, 0x70, 0x61, 0x69, 0x73, 0x61, 0x6e, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x73, 0x6f, 0x6c,
0x65, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x101, 0x73, 0x6f, 0x6c, 0x65, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x73, 0x6f, 0x6c, 0x65,
0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70,
0x101, 0x6e, 0x67, 0x69, 0x6c, 0x101, 0x74, 0x65, 0x73, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x101, 0x6e, 0x67, 0x69,
0x6c, 0x101, 0x74, 0x65, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x70, 0x101, 0x6e, 0x67, 0x69, 0x6c, 0x101, 0x74,
0x65, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x70, 0x16b, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x70, 0x6f, 0x6c, 0x12b, 0x76, 0x69, 0x61, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x101, 0x73,
0x69, 0x6c, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x70, 0x61, 0x6c, 0x101, 0x73, 0x69, 0x6c, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x101, 0x73, 0x69,
0x6c, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70,
0x75, 0x6c, 0x75, 0x6e, 0x65, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6d, 0x75, 0x69, 0x2bb, 0x69, 0x2d, 0x76, 0x113, 0x74,
0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x75,
0x69, 0x2bb, 0x69, 0x2d, 0x76, 0x113, 0x74, 0x65, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x75, 0x69, 0x2bb, 0x69, 0x2d,
0x76, 0x113, 0x74, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6b, 0x65, 0x65, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x6c, 0x6f, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x74, 0x69, 0x68, 0x61,
0x6d, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73,
0x61, 0x74, 0x69, 0x68, 0x61, 0x6d, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x74, 0x69, 0x68, 0x61, 0x6d,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x73, 0x69, 0x61, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x61, 0x69, 0x6e, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x61,
0x69, 0x6e, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6d, 0x6f, 0x74, 0x75, 0x6b, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x6d, 0x61,
0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d,
0x6f, 0x74, 0x75, 0x6b, 0x6f, 0x6b, 0x6f, 0x73, 0x69, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x69,
0x70, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x69, 0x70, 0x69, 0x61, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d,
0x69, 0x70, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x6b, 0x75, 0x6b,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f,
0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x6b, 0x75, 0x6b, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75,
0x6d, 0x6f, 0x74, 0x75, 0x6b, 0x75, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x75, 0x70, 0x61, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x75, 0x70, 0x61,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69,
0x75, 0x70, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x74, 0x61, 0x76, 0x69, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x74, 0x16b, 0x6d, 0x6f, 0x6e, 0x69, 0x2d, 0x74, 0x16b,
0x76, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x74, 0x69, 0x6d, 0x6f, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b,
0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x6e, 0x75, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x6e, 0x75, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x6e,
0x75, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x65, 0x6b, 0x75, 0x65, 0x74, 0x6f, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d,
0x6c, 0x6f, 0x74, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d, 0x6c, 0x6f, 0x74,
0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c,
0x6f, 0x70, 0x65, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f,
0x70, 0x65, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f,
0x70, 0x65, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70,
0x65, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x2d, 0x61, 0x6e, 0x67,
0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65,
0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75,
0x6c, 0x6f, 0x70, 0x65, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75,
0x6c, 0x6f, 0x70, 0x65, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75,
0x6d, 0x6f, 0x74, 0x75, 0x2d, 0x66, 0x6f, 0x6b, 0x75, 0x6c, 0x61, 0x6e,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f,
0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x2d, 0x66, 0x6f, 0x6b, 0x75, 0x6c,
0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x2d, 0x66, 0x6f,
0x6b, 0x75, 0x6c, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x66, 0x69, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x66, 0x69, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x69, 0x73, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x75, 0x69, 0x61, 0x6e,
0x61, 0x2d, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x61, 0x6c, 0x61, 0x6e, 0x69,
0x73, 0x113, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x6e, 0x65, 0x74, 0x101, 0x74, 0x69, 0x6b, 0x61, 0x66, 0x61, 0x6c,
0x61, 0x6e, 0x69, 0x73, 0x113, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x6b, 0x6f, 0x73, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61, 0x6d,
0x69, 0x70, 0x69, 0x113, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x73, 0x65, 0x14d, 0x73, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x73, 0x65, 0x14d, 0x73, 0x69, 0x61, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x65, 0x14d, 0x73,
0x69, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b,
0x69, 0x6c, 0x69, 0x70, 0x61, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x6c, 0x69, 0x6e, 0x69, 0x75, 0x69,
0x73, 0x69, 0x20, 0x6d, 0x101, 0x6c, 0x69, 0x65, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x6d, 0x61,
0x74, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x6d,
0x61, 0x74, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75,
0x61, 0x6d, 0x61, 0x74, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e,
0x75, 0x61, 0x6d, 0x61, 0x74, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66,
0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f,
0x6e, 0x75, 0x61, 0x6d, 0x61, 0x74, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69,
0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x66, 0x6f, 0x6e, 0x75, 0x61, 0x6d, 0x61, 0x74, 0x61, 0x2d, 0x68, 0x69,
0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6b, 0x75, 0x61, 0x6d, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6b, 0x16b, 0x6c, 0x69, 0x66, 0x69, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x61, 0x75, 0x61, 0x69, 0x2bb,
0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x61, 0x75, 0x61, 0x69, 0x2bb, 0x69,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x61, 0x75, 0x61, 0x69, 0x2bb, 0x69,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f, 0x6e, 0x67, 0x69, 0x2d, 0x6b, 0x6f,
0x6e, 0x67, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x68, 0x6f, 0x6e, 0x67, 0x69, 0x2d, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f, 0x6e,
0x67, 0x69, 0x2d, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f, 0x76, 0x69, 0x74, 0x65, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f, 0x76, 0x69,
0x74, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x68, 0x6f, 0x76, 0x69, 0x74, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x69, 0x61, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x61,
0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x6f, 0x73, 0x69,
0x61, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x69, 0x73, 0x69, 0x61,
0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x69, 0x73, 0x69,
0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x6f, 0x6e,
0x69, 0x73, 0x69, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x6c, 0x61,
0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x69, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x6c, 0x61, 0x61, 0x6e, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73,
0x69, 0x61, 0x2d, 0x2bb, 0x12b, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b,
0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x12b, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69,
0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x12b, 0x6b, 0x75, 0x74, 0x69,
0x73, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x69, 0x73, 0x69, 0x6c, 0x65, 0x6c, 0x69, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x73, 0x69, 0x6c, 0x65,
0x6c, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x69, 0x73, 0x69, 0x6c, 0x65, 0x6c, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x61, 0x70, 0x61, 0x6e, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x61,
0x70, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x73, 0x69, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d,
0x70, 0x65, 0x74, 0x65, 0x6c, 0x6f, 0x70, 0x61, 0x76, 0x69, 0x6c, 0x6f,
0x76, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x70, 0x65, 0x74,
0x65, 0x6c, 0x6f, 0x70, 0x61, 0x76, 0x69, 0x6c, 0x6f, 0x76, 0x69, 0x73,
0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x70, 0x65, 0x74, 0x65, 0x6c,
0x6f, 0x70, 0x61, 0x76, 0x69, 0x6c, 0x6f, 0x76, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61,
0x73, 0x61, 0x6b, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x6b, 0x69, 0x74,
0x61, 0x6e, 0x69, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x6b,
0x69, 0x74, 0x61, 0x6e, 0x69, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x14d, 0x6c,
0x65, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b,
0x14d, 0x6c, 0x65, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6b, 0x14d, 0x6c, 0x65, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6b, 0x6f, 0x73, 0x69, 0x6c, 0x61, 0x65, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x6b, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x69, 0x101,
0x73, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6b, 0x61, 0x6c, 0x61, 0x73,
0x69, 0x6e, 0x6f, 0x69, 0x101, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x6b, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x69, 0x101, 0x73,
0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6b, 0x12b, 0x6b, 0x69, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x69,
0x6b, 0x101, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x6c, 0x61, 0x69, 0x6e, 0x65,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74,
0x75, 0x2bb, 0x65, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x65, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74, 0x75, 0x2bb,
0x65, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x65, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74, 0x75, 0x2bb, 0x65,
0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6b, 0x61, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6b, 0x61, 0x75, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6b,
0x61, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6d, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x6e,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b,
0x73, 0x69, 0x61, 0x2d, 0x6d, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b,
0x73, 0x69, 0x61, 0x2d, 0x6d, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6c,
0x65, 0x69, 0x73, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6d, 0x61, 0x6c, 0x61, 0x74, 0x69, 0x76, 0x69, 0x73, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x101, 0x6b,
0x65, 0x73, 0x61, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x6d, 0x61,
0x73, 0x6f, 0x6c, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6d, 0x61, 0x75, 0x6c, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x75, 0x6c,
0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x75, 0x6c, 0x69, 0x74, 0x69, 0x75,
0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d,
0x61, 0x75, 0x73, 0x6f, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6d, 0x65, 0x6b, 0x69, 0x73, 0x69, 0x6b, 0x6f, 0x75,
0x2d, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x65, 0x6b, 0x69, 0x73, 0x69,
0x6b, 0x6f, 0x75, 0x2d, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x65,
0x6b, 0x69, 0x73, 0x69, 0x6b, 0x6f, 0x75, 0x2d, 0x70, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x75, 0x6c, 0x101, 0x6e, 0x69, 0x70, 0x101, 0x74, 0x101, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x6c, 0x101,
0x6e, 0x69, 0x70, 0x101, 0x74, 0x101, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x6c, 0x101, 0x6e, 0x69, 0x70, 0x101,
0x74, 0x101, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6d, 0x6f, 0x73, 0x69, 0x6b, 0x6f, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73,
0x69, 0x61, 0x2d, 0x6d, 0x6f, 0x73, 0x69, 0x6b, 0x6f, 0x75, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x6d, 0x6f, 0x73, 0x69, 0x6b, 0x6f, 0x75, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x65, 0x6d, 0x61, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x61, 0x75, 0x6c, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x65, 0x70,
0x61, 0x6c, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6b, 0x61, 0x6c, 0x65, 0x74, 0x14d, 0x6e, 0x69, 0x61, 0x66, 0x6f, 0x2bb,
0x6f, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b,
0x61, 0x6c, 0x65, 0x74, 0x14d, 0x6e, 0x69, 0x61, 0x66, 0x6f, 0x2bb, 0x6f,
0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b,
0x61, 0x6c, 0x65, 0x74, 0x14d, 0x6e, 0x69, 0x61, 0x66, 0x6f, 0x2bb, 0x6f,
0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x75,
0x2bb, 0x75, 0x73, 0x69, 0x6c, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6e, 0x75, 0x2bb, 0x75, 0x73, 0x69, 0x6c, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x75, 0x2bb,
0x75, 0x73, 0x69, 0x6c, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x2bb, 0x69, 0x6c, 0x6f, 0x66,
0x6f, 0x2bb, 0x6f, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x2bb, 0x69, 0x6c, 0x6f, 0x66, 0x6f,
0x2bb, 0x6f, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x2bb, 0x69, 0x6c, 0x6f, 0x66, 0x6f,
0x2bb, 0x6f, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6e, 0x69, 0x75, 0x113, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6e, 0x6f, 0x61, 0x66, 0x14d, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x6f, 0x61, 0x66, 0x14d, 0x6b, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x6f,
0x61, 0x66, 0x14d, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x66, 0x113, 0x6e, 0x61, 0x6e, 0x69, 0x74, 0x6f, 0x2d, 0x74,
0x65, 0x2d, 0x6e, 0x6f, 0x6c, 0x14d, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x113, 0x6e, 0x61, 0x6e, 0x69,
0x74, 0x6f, 0x2d, 0x74, 0x65, 0x2d, 0x6e, 0x6f, 0x6c, 0x14d, 0x6e, 0x69,
0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66,
0x113, 0x6e, 0x61, 0x6e, 0x69, 0x74, 0x6f, 0x2d, 0x74, 0x65, 0x2d, 0x6e,
0x6f, 0x6c, 0x14d, 0x6e, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x61, 0x74, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x70, 0x12b, 0x73, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x70, 0x12b, 0x73, 0x69, 0x6b, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73,
0x69, 0x61, 0x2d, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x70, 0x12b, 0x73,
0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x6f, 0x6d, 0x69, 0x73, 0x69,
0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x6f, 0x6d, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x6f, 0x6d, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x101,
0x6b, 0x69, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x101, 0x6b, 0x69, 0x73, 0x69, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x70, 0x101, 0x6b, 0x69, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x61,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61,
0x70, 0x75, 0x61, 0x6e, 0x69, 0x75, 0x6b, 0x69, 0x6e, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x61, 0x6b,
0x75, 0x61, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x70, 0x61, 0x6c, 0x61, 0x6b, 0x75, 0x61, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x61, 0x6b, 0x75,
0x61, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70,
0x65, 0x6c, 0x16b, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x70, 0x65, 0x6c, 0x16b, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x70, 0x65, 0x6c, 0x16b, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x61, 0x69, 0x6e, 0x65,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x69, 0x6c,
0x69, 0x70, 0x61, 0x69, 0x6e, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x61, 0x69, 0x6e,
0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f,
0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x66, 0x6f, 0x69, 0x6e, 0x69, 0x6b,
0x69, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x73, 0x101, 0x2d, 0x70, 0x69, 0x65, 0x61, 0x2d, 0x6d, 0x6f, 0x2d, 0x6d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x73, 0x101, 0x2d, 0x70, 0x69, 0x65, 0x61, 0x2d,
0x6d, 0x6f, 0x2d, 0x6d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x101, 0x2d,
0x70, 0x69, 0x65, 0x61, 0x2d, 0x6d, 0x6f, 0x2d, 0x6d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x70, 0x69, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x14d, 0x6e, 0x61, 0x70, 0x113,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x69, 0x6f,
0x6e, 0x67, 0x69, 0x61, 0x6e, 0x67, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x73, 0x69, 0x6c, 0x14d, 0x74, 0x61,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x73,
0x69, 0x6c, 0x14d, 0x74, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6b, 0x69, 0x73, 0x69, 0x6c, 0x14d, 0x74, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x113, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x6f, 0x74, 0x65, 0x6c, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73, 0x61,
0x6b, 0x101, 0x6c, 0x69, 0x6e, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73, 0x61, 0x6b,
0x101, 0x6c, 0x69, 0x6e, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73, 0x61, 0x6b,
0x101, 0x6c, 0x69, 0x6e, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73, 0x61, 0x6d, 0x61,
0x6c, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73, 0x61, 0x6d, 0x61, 0x6c, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73,
0x69, 0x61, 0x2d, 0x73, 0x61, 0x6d, 0x61, 0x6c, 0x61, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x61, 0x2bb, 0x61, 0x6d, 0x6f,
0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x61,
0x2bb, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x68, 0x61, 0x2bb, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d,
0x6f, 0x74, 0x75, 0x2d, 0x73, 0x65, 0x69, 0x73, 0x65, 0x6c, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x6f, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x73, 0x6f, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x73, 0x69, 0x6f, 0x73, 0x69, 0x61, 0x74, 0x6f, 0x6e, 0x67,
0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x75,
0x6c, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x73, 0x69, 0x6f, 0x75, 0x61, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x68, 0x69, 0x73, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x69, 0x70,
0x65, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74,
0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x74, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x73, 0x69, 0x6b, 0x69,
0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x6f, 0x6e, 0x67, 0x61,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x74, 0x16b, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x74, 0x16b, 0x6b, 0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x69, 0x74,
0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x74, 0x16b, 0x6b, 0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x69, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x74, 0x16b, 0x6b, 0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x69, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74,
0x16b, 0x76, 0x61, 0x6c, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x75, 0x6c, 0x75, 0x6b, 0x75, 0x61, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x6c, 0x75, 0x6b,
0x75, 0x61, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x75, 0x6c, 0x75, 0x6b, 0x75, 0x61, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x73, 0x69, 0x70, 0x65,
0x6b, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x73, 0x69, 0x70, 0x65, 0x6b, 0x69, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x75, 0x73, 0x69, 0x70, 0x65, 0x6b, 0x69, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x76, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x76, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x76,
0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x76,
0x61, 0x6c, 0x61, 0x74, 0x69, 0x76, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b,
0x73, 0x69, 0x61, 0x2d, 0x76, 0x61, 0x6c, 0x61, 0x74, 0x69, 0x76, 0x6f,
0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x76, 0x61,
0x6c, 0x61, 0x74, 0x69, 0x76, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73,
0x69, 0x61, 0x2d, 0x76, 0x6f, 0x6c, 0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6c,
0x61, 0x74, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x76, 0x6f, 0x6c, 0x69, 0x6b, 0x6f,
0x6b, 0x61, 0x6c, 0x61, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x76, 0x6f,
0x6c, 0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6c, 0x61, 0x74, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x76, 0x6f, 0x73, 0x69, 0x74,
0x6f, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6d, 0x6f, 0x74, 0x75, 0x75, 0x65, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x76, 0x65, 0x61, 0x20, 0x6d,
0x6f, 0x20, 0x66, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb,
0x69, 0x101, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x2bb, 0x69, 0x101, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b,
0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x69, 0x101, 0x6b, 0x75, 0x74, 0x69, 0x73,
0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x69, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x6c, 0x69, 0x6e, 0x65, 0x70, 0x16b, 0x6b, 0x69, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d,
0x2bb, 0x69, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x6c, 0x69, 0x6e, 0x65, 0x70,
0x16b, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x69, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x6c, 0x69, 0x6e, 0x65, 0x70, 0x16b, 0x6b, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x69, 0x75, 0x6b, 0x6f, 0x6e,
0x69, 0x50, 0x61, 0x6c, 0x6f, 0x67, 0x61, 0x72, 0x65, 0x20, 0x79, 0x61,
0x20, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x79, 0x61, 0x20, 0x6e, 0x67, 0x77,
0x61, 0x67, 0x61, 0x20, 0x6c, 0x65, 0x20, 0x6e, 0x67, 0x77, 0x61, 0x67,
0x61, 0x20, 0x79, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69,
0x63, 0x68, 0x41, 0x63, 0x72, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x41, 0x63, 0x72, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x63, 0x72, 0x65, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x66, 0x67,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47,
0xfc, 0x6e, 0x65, 0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74,
0x131, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x131, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a,
0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d,
0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x6f, 0x11f,
0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x6f, 0x11f,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65,
0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x61,
0x11f, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x61, 0x11f,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x61, 0x11f, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a,
0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a,
0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x6e, 0x61, 0x64, 0x131, 0x72, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x41, 0x6e, 0x61, 0x64, 0x131, 0x72, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x70,
0x69, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x41, 0x6b, 0x74, 0x61, 0x76, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41,
0x6b, 0x74, 0x61, 0x76, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x74, 0x61, 0x76,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6b,
0x74, 0xf6, 0x62, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6b,
0x74, 0xf6, 0x62, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x74, 0xf6, 0x62,
0x65, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41,
0x72, 0x61, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x72, 0x6a,
0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41,
0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x72,
0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x72, 0x6a,
0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42,
0x61, 0x74, 0x131, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x72, 0x6a, 0x61,
0x6e, 0x74, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x45, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x72,
0x74, 0x61, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20,
0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x79, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x130, 0xe7, 0x20, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x76,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x130, 0xe7, 0x20, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x76,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130,
0xe7, 0x20, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41,
0x76, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x79, 0x63, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x79, 0x63, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x79, 0x63, 0x61, 0x6e, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x6f, 0x72,
0x6c, 0x61, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x6f,
0x72, 0x6c, 0x61, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x6c,
0x61, 0x72, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x75,
0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x6f, 0x6c,
0x69, 0x76, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20,
0x44, 0x61, 0x72, 0xfc, 0x2019, 0x73, 0x2d, 0x53, 0x65, 0x6c, 0x61, 0x6d,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x61,
0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43,
0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x61, 0x73, 0x65, 0x79,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72,
0x72, 0x6f, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x15e, 0x69, 0x6c, 0x69, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x15e, 0x69,
0x6c, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x15e, 0x69, 0x6c, 0x69, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0xc7, 0x69, 0x6e, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0xc7, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0xc7, 0x69,
0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x20, 0x41, 0x64, 0x61,
0x73, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x79, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72,
0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x6f, 0x6f, 0x6b, 0x20,
0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x59,
0x61, 0x72, 0x131, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4b, 0xfc, 0x62, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b,
0xfc, 0x62, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0xfc, 0x62, 0x61, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x61, 0x76, 0x69,
0x73, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x54, 0x69,
0x6d, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x73,
0x6b, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x6c, 0x79,
0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61,
0x73, 0x6b, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x45, 0x6b,
0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f,
0x72, 0x74, 0x61, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x76, 0x72,
0x75, 0x70, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x41,
0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76, 0x72, 0x75,
0x70, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75,
0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f,
0x11f, 0x75, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x6c, 0x65, 0x72, 0x69,
0x20, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41,
0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42,
0x61, 0x74, 0x131, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x69,
0x6a, 0x69, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x46, 0x72, 0x61, 0x6e, 0x73, 0x131, 0x7a, 0x20, 0x47, 0x75, 0x79, 0x61,
0x6e, 0x61, 0x73, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x72,
0x61, 0x6e, 0x73, 0x131, 0x7a, 0x20, 0x47, 0xfc, 0x6e, 0x65, 0x79, 0x20,
0x76, 0x65, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47,
0xfc, 0x72, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x47, 0xfc, 0x72, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x47, 0xfc, 0x72, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x20, 0x4f, 0x72, 0x74, 0x61, 0x6c, 0x61, 0x6d, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x47,
0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x47, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x47, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42,
0x61, 0x74, 0x131, 0x20, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x47, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x47, 0x75, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0xf6, 0x72,
0x66, 0x65, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x76, 0x64, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x69, 0x6e,
0x64, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x69, 0x6e,
0x74, 0x20, 0x4f, 0x6b, 0x79, 0x61, 0x6e, 0x75, 0x73, 0x75, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x48, 0x69, 0x6e, 0x64, 0x69, 0xe7, 0x69, 0x6e,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x45,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x45, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x7a, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61,
0x74, 0x131, 0x20, 0x45, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130,
0x72, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x79,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4a, 0x61, 0x70, 0x6f, 0x6e,
0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x79, 0x61,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0xe7, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4b, 0x6f, 0x72, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f,
0x72, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x72, 0x65, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b,
0x131, 0x72, 0x67, 0x131, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x41, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x6b, 0x61, 0x6f,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x6c,
0x65, 0x7a, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x6c, 0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x41, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72,
0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4b, 0x131, 0x79,
0x131, 0x73, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x20, 0x4b, 0x131, 0x79, 0x131, 0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x20, 0x4b, 0x131, 0x79, 0x131, 0x73, 0x131, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x75, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x76, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b,
0x6f, 0x76, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76,
0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d,
0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e,
0x65, 0x70, 0x61, 0x6c, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65,
0x6e, 0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59,
0x65, 0x6e, 0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79,
0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59,
0x65, 0x6e, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61,
0x6e, 0x61, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x70,
0x75, 0x61, 0x20, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x47, 0x69, 0x6e, 0x65,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50,
0x65, 0x72, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x76, 0x65, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x76, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x76, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x131, 0x7a, 0x131, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x131, 0x7a,
0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x131, 0x7a,
0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x65, 0x79,
0x15f, 0x65, 0x6c, 0x6c, 0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53,
0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0xfc, 0x6e, 0x65,
0x79, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x53, 0x68, 0x6f, 0x77, 0x61, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x54, 0x61, 0x63, 0x69, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0xd6, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0xd6, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0xd6, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x64, 0x61, 0x73,
0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0x76, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x77, 0x67, 0x61, 0x6e, 0x79,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65,
0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67,
0x61, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x6f, 0x72, 0x74, 0x61, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62,
0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61,
0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x41, 0x6c, 0xfd, 0x61, 0x73, 0x6b, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x6c, 0xfd, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x41, 0x6c, 0xfd, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x72, 0x6b,
0x65, 0x7a, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72,
0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x67, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a,
0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x67,
0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65,
0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x67, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61,
0x72, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79,
0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x64, 0x61,
0x67, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72,
0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x64, 0x61, 0x67, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69,
0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x64, 0x61, 0x67, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69,
0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0xdd, 0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d,
0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69,
0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0xdd, 0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d,
0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61,
0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0xdd, 0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x41, 0x70, 0x69, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41,
0x70, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x70, 0x69, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x41, 0x72, 0x61, 0x70, 0x20, 0xfd, 0x75, 0x72, 0x74, 0x6c, 0x61, 0x72,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x72, 0x61, 0x70, 0x20,
0xfd, 0x75, 0x72, 0x74, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41,
0x72, 0x61, 0x70, 0x20, 0xfd, 0x75, 0x72, 0x74, 0x6c, 0x61, 0x72, 0x79,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x45, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x77, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x77, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65,
0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a,
0x69, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61,
0x20, 0x67, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41,
0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x67, 0xfc,
0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72,
0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0xfd, 0x61, 0x20, 0x67, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x41,
0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20,
0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x41, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc,
0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e,
0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0xfd, 0x6a, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0xfd, 0x6a, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0xfd, 0x6a, 0x61, 0x6e, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41,
0x7a, 0x6f, 0x72, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x61, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x7a, 0x6f, 0x72,
0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42,
0x6f, 0x6c, 0x69, 0x77, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0xfd,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0xfd,
0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x42, 0x72, 0x75, 0x6e, 0x65, 0xfd, 0x2d, 0x44, 0x61,
0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4b, 0x61, 0x62, 0x6f, 0x2d, 0x57, 0x65, 0x72, 0x64, 0x65,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x61, 0x62, 0x6f, 0x2d, 0x57,
0x65, 0x72, 0x64, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x61, 0x62, 0x6f, 0x2d,
0x57, 0x65, 0x72, 0x64, 0x65, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x61, 0x6d, 0x6f, 0x72,
0x72, 0x6f, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x61, 0x74, 0x65,
0x6d, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x61, 0x74, 0x65, 0x6d,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0xc7, 0x61, 0x74, 0x65, 0x6d, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x69,
0x6c, 0x69, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x69, 0x6c, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0xc7, 0x69, 0x6c, 0x69, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x79, 0x74,
0x61, 0xfd, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x79, 0x74, 0x61,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x48, 0x79, 0x74, 0x61, 0xfd, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x52,
0x6f, 0x17e, 0x64, 0x65, 0x73, 0x74, 0x77, 0x6f, 0x20, 0x61, 0x64, 0x61,
0x73, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0xfd, 0x61,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x75, 0x6b, 0x20, 0x61, 0x64, 0x61,
0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x75,
0x6b, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4b, 0x75, 0x6b, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b,
0x75, 0x62, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x44, 0x65, 0xfd, 0x77, 0x69, 0x73, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x44, 0xfd, 0x75, 0x6d, 0x6f, 0x6e, 0x2d, 0x64, 0x2d, 0xdd, 0x75, 0x72,
0x77, 0x69, 0x6c, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e,
0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x73, 0x68, 0x61, 0x20, 0x61,
0x64, 0x61, 0x73, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61,
0x73, 0x68, 0x61, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x50, 0x61, 0x73, 0x68, 0x61, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x45, 0x6b, 0x77, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0xdd, 0x65,
0x77, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d,
0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69,
0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc,
0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64,
0x6f, 0x67, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20,
0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x7a, 0x61,
0x6b, 0x20, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0xdd,
0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc,
0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61,
0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46,
0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x61, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x6f, 0x6c,
0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72,
0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64,
0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x69,
0x6a, 0x69, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x69, 0x6a, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x72, 0x61,
0x6e, 0x73, 0x75, 0x7a, 0x20, 0x47, 0x77, 0x69, 0x61, 0x6e, 0x61, 0x73,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x72, 0x61, 0x6e, 0x73,
0x75, 0x7a, 0x20, 0x47, 0xfc, 0x6e, 0x6f, 0x72, 0x74, 0x61, 0x20, 0x77,
0x65, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x61,
0x20, 0xfd, 0x65, 0x72, 0x6c, 0x65, 0x72, 0x69, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20,
0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0x61, 0x6d, 0x62, 0xfd, 0x65, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0x72, 0x75, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0x72, 0x75, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0x72, 0x75, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x72, 0x69, 0x6e, 0x77, 0x69,
0xe7, 0x20, 0x6f, 0x72, 0x74, 0x61, 0xe7, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x47,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72,
0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x47, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x47,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x50, 0x61, 0x72, 0x73, 0x20, 0x61, 0xfd, 0x6c, 0x61, 0x67, 0x79, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0x61, 0xfd, 0x61, 0x6e, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0x61, 0x77, 0x61, 0xfd, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x61, 0x77, 0x61, 0xfd,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x61, 0x77,
0x61, 0xfd, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x6f,
0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x6f, 0x6e,
0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x6f, 0x77, 0x64, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x48, 0x6f, 0x77, 0x64, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x48,
0x6f, 0x77, 0x64, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x20, 0x75,
0x6d, 0x6d, 0x61, 0x6e, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x48,
0x69, 0x6e, 0x64, 0x69, 0x68, 0x79, 0x74, 0x61, 0xfd, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0xfd, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x45, 0xfd, 0x72, 0x61, 0x6e, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x45, 0xfd, 0x72, 0x61, 0x6e, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x45, 0xfd, 0x72, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x59, 0x73, 0x72, 0x61, 0xfd, 0x79, 0x6c, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x59, 0x73, 0x72, 0x61, 0xfd, 0x79, 0x6c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x59, 0x73, 0x72, 0x61, 0xfd, 0x79, 0x6c, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd,
0x61, 0x70, 0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0xdd, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0xdd, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x77, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x73, 0x6b, 0x69, 0xfd, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x47, 0x61, 0x7a, 0x61, 0x67, 0x79, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64,
0x6f, 0x67, 0x61, 0x72, 0x20, 0x47, 0x61, 0x7a, 0x61, 0x67, 0x79, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e,
0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x47, 0x61, 0x7a, 0x61, 0x67, 0x79,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f,
0x72, 0x65, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f,
0x72, 0x65, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f, 0x72, 0x65, 0xfd,
0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0xfd, 0x65, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xfd, 0x61,
0x72, 0x73, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0xfd, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0xfd, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4c, 0x61, 0xfd, 0x6e, 0x20, 0x61, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4c,
0x6f, 0x72, 0x64, 0x2d, 0x48, 0x61, 0x75, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x61, 0x75, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x61, 0x75, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x6c, 0x61, 0xfd, 0x7a,
0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x77, 0x6c, 0x65, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x72, 0x15f,
0x61, 0x6c, 0x6c, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x77, 0x72, 0x69, 0x6b, 0x69,
0xfd, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x77, 0x72, 0x69,
0x6b, 0x69, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x77, 0x72, 0x69, 0x6b,
0x69, 0xfd, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4d, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0xdd, 0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0xdd, 0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d,
0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0xdd, 0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61,
0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x6c, 0x61, 0x6e, 0x2d,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x6c, 0x61, 0x6e,
0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x6f, 0x73, 0x6b,
0x77, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x6f, 0x73, 0x6b,
0x77, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4d, 0xfd, 0x61, 0x6e, 0x6d, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54,
0xe4, 0x7a, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4e, 0xfd, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0xfd, 0x75, 0x66, 0x61, 0x75,
0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0xfd, 0x75,
0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e,
0x69, 0x75, 0x65, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x2d, 0x64, 0x69, 0x2d,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x6f, 0x77, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4f, 0x6d, 0x73, 0x6b, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x2d, 0x20, 0x54, 0xe4, 0x7a, 0x65, 0x20,
0x47, 0x77, 0x69, 0x6e, 0x65, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0xfd, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0xfd, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x50, 0x65, 0x72, 0x75, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x65,
0x72, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x65, 0x72, 0x75, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x6c, 0x65, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x65, 0x6e, 0x69, 0x6b,
0x73, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x53, 0x65, 0x6e, 0x2d, 0x50, 0xfd, 0x65, 0x72, 0x20,
0x77, 0x65, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x65,
0x6e, 0x2d, 0x50, 0xfd, 0x65, 0x72, 0x20, 0x77, 0x65, 0x20, 0x4d, 0x69,
0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x65, 0x6e, 0x2d,
0x50, 0xfd, 0x65, 0x72, 0x20, 0x77, 0x65, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x68, 0x65, 0x6e, 0xfd, 0x61,
0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x52, 0x65, 0xfd, 0x75, 0x6e,
0xfd, 0x6f, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x52, 0x6f, 0x74,
0x65, 0x72, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x65, 0xfd, 0x15f, 0x65,
0x6c, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0xfc, 0x6e, 0x6f, 0x72, 0x74, 0x61, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x53, 0xfd, 0x6f, 0x77, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x54, 0x61, 0x69, 0x74, 0x69, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54,
0x61, 0xfd, 0x62, 0x65, 0xfd, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54,
0x61, 0xfd, 0x62, 0x65, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x61, 0xfd, 0x62,
0x65, 0xfd, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x6a, 0x69, 0x67, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7,
0x75, 0x75, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xfc, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x75, 0x77, 0x61,
0x6c, 0x75, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x72, 0x75, 0x67,
0x77, 0x61, 0xfd, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x72, 0x75,
0x67, 0x77, 0x61, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x72, 0x75, 0x67, 0x77,
0x61, 0xfd, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0xd6, 0x7a, 0x62, 0x65, 0x67, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xd6, 0x7a, 0x62, 0x65,
0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xd6, 0x7a, 0x62,
0x65, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x65, 0x6e, 0x65, 0x73, 0x75, 0x65,
0x6c, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x6c, 0x61, 0x64,
0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x65, 0xfd, 0x6b, 0x20, 0x61,
0x64, 0x61, 0x73, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x6f,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x77, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0xdd, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0xdd, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x75, 0x6b, 0x6f, 0x6e, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x410, 0x43a,
0x440, 0x456, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x410, 0x43a, 0x440, 0x456, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x439,
0x447, 0x430, 0x441, 0x3a, 0x20, 0x410, 0x43a, 0x440, 0x456, 0x2c, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x439, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x432, 0x20, 0x410, 0x444, 0x433, 0x430, 0x43d, 0x456, 0x441, 0x442,
0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x434, 0x435, 0x43d, 0x43d, 0x43e,
0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445,
0x456, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e,
0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x410, 0x43b, 0x44f, 0x441, 0x446, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x43b, 0x44f, 0x441, 0x446,
0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x43b, 0x44f, 0x441,
0x446, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x446, 0x456, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x446, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x446, 0x456, 0x437, 0x430, 0x20,
0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x446, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x446, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x438, 0x43c, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441,
0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x433, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x433, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d,
0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x433, 0x456, 0x440, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x442, 0x438,
0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440,
0x447, 0x430, 0x441, 0x3a, 0x20, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x439,
0x447, 0x430, 0x441, 0x3a, 0x20, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x2c,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x439, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x410, 0x43f, 0x456, 0x430, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x410, 0x43f, 0x456,
0x430, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x410, 0x43f, 0x456, 0x430, 0x437,
0x430, 0x20, 0x430, 0x440, 0x430, 0x431, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430, 0x440, 0x430, 0x431,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x430, 0x440, 0x430, 0x431, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x430, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x430, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x430, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x437, 0x430, 0x445,
0x456, 0x434, 0x43d, 0x43e, 0x430, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x437, 0x430, 0x20,
0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x456, 0x440, 0x43c, 0x435, 0x43d, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x432, 0x456, 0x440, 0x43c, 0x435, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x456, 0x440,
0x43c, 0x435, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x447, 0x43d, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447,
0x43d, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x446, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x44c, 0x43d, 0x43e, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20,
0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x437, 0x430, 0x445,
0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x437, 0x430,
0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x430, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e,
0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x441,
0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x441,
0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x437, 0x430,
0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x437,
0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439,
0x434, 0x436, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x430, 0x437, 0x435, 0x440, 0x431, 0x430,
0x439, 0x434, 0x436, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x430, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430,
0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x410, 0x437, 0x43e, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x44c,
0x43a, 0x438, 0x445, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445,
0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441,
0x44c, 0x43a, 0x438, 0x445, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x437, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x411, 0x443, 0x442, 0x430, 0x43d,
0x456, 0x437, 0x430, 0x20, 0x431, 0x43e, 0x43b, 0x456, 0x432, 0x456, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x431, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x431, 0x440,
0x430, 0x437, 0x438, 0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x431, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x44c, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x411, 0x440, 0x443, 0x43d, 0x435,
0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a, 0x430,
0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435,
0x440, 0x434, 0x435, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a, 0x430, 0x431, 0x43e, 0x2d,
0x412, 0x435, 0x440, 0x434, 0x435, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d,
0x438, 0x445, 0x20, 0x41c, 0x430, 0x440, 0x456, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x445, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x430,
0x440, 0x445, 0x456, 0x43f, 0x435, 0x43b, 0x430, 0x437, 0x456, 0x20, 0x427, 0x430,
0x442, 0x435, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x430, 0x440, 0x445, 0x456, 0x43f, 0x435, 0x43b, 0x430, 0x437,
0x456, 0x20, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x430, 0x440, 0x445, 0x456, 0x43f, 0x435, 0x43b, 0x430, 0x437, 0x456,
0x20, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x438, 0x43b,
0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x438, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x438, 0x43b, 0x456, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43a, 0x438, 0x442, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x438, 0x442, 0x430,
0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x43a, 0x438, 0x442, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x420, 0x456,
0x437, 0x434, 0x432, 0x430, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438,
0x445, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430,
0x20, 0x43a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x43a,
0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x43a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a, 0x443, 0x43a, 0x430,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a, 0x443, 0x43a,
0x430, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41a, 0x443, 0x431,
0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x41a, 0x443, 0x431, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x41a, 0x443, 0x431, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x456, 0x457, 0x20,
0x414, 0x435, 0x432, 0x456, 0x441, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x456, 0x457,
0x20, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2bc, 0x42e, 0x440, 0x432,
0x456, 0x43b, 0x44c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x421, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x43c, 0x443, 0x20, 0x422,
0x438, 0x43c, 0x43e, 0x440, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456,
0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x437, 0x430, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41f, 0x430,
0x441, 0x445, 0x438, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x432, 0x20, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x456, 0x437, 0x430,
0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x454,
0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435,
0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x454,
0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434,
0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441,
0x445, 0x456, 0x434, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x434, 0x430, 0x43b,
0x435, 0x43a, 0x43e, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x454,
0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456,
0x434, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f,
0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x454,
0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x44c, 0x43a, 0x438,
0x445, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x424, 0x43e,
0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x44c, 0x43a, 0x438, 0x445, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441,
0x44c, 0x43a, 0x438, 0x445, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20,
0x424, 0x456, 0x434, 0x436, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x424, 0x456, 0x434, 0x436, 0x456, 0x437, 0x430, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x443, 0x20, 0x424, 0x456, 0x434, 0x436, 0x456, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437,
0x44c, 0x43a, 0x43e, 0x457, 0x20, 0x490, 0x432, 0x456, 0x430, 0x43d, 0x438, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x424,
0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x41f,
0x456, 0x432, 0x434, 0x435, 0x43d, 0x43d, 0x438, 0x445, 0x20, 0x456, 0x20, 0x410,
0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x447, 0x43d, 0x438, 0x445, 0x20,
0x442, 0x435, 0x440, 0x438, 0x442, 0x43e, 0x440, 0x456, 0x44f, 0x445, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x490, 0x430, 0x43b, 0x430, 0x43f,
0x430, 0x491, 0x43e, 0x441, 0x443, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456,
0x20, 0x490, 0x430, 0x43c, 0x431, 0x2bc, 0x454, 0x437, 0x430, 0x20, 0x433, 0x440,
0x443, 0x437, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x433, 0x440, 0x443, 0x437, 0x438, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x433, 0x440, 0x443,
0x437, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x413,
0x456, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x430, 0x437, 0x430, 0x20, 0x413, 0x440,
0x438, 0x43d, 0x432, 0x456, 0x447, 0x435, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x445,
0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x490, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456,
0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443,
0x20, 0x490, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x413, 0x443, 0x430, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x41f, 0x435, 0x440, 0x441, 0x44c, 0x43a, 0x43e, 0x457, 0x20,
0x437, 0x430, 0x442, 0x43e, 0x43a, 0x438, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x430, 0x44f, 0x43d, 0x456, 0x437, 0x430,
0x20, 0x433, 0x430, 0x432, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x43e, 0x2d, 0x430,
0x43b, 0x435, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x433, 0x430, 0x432, 0x430, 0x439, 0x441,
0x44c, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x433, 0x430, 0x432, 0x430, 0x439, 0x441,
0x44c, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x413, 0x43e, 0x43d, 0x43a, 0x43e,
0x43d, 0x437, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x437, 0x456, 0x437, 0x430,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x437, 0x456,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x425,
0x43e, 0x432, 0x434, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x443, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x456, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x456, 0x437, 0x430, 0x20, 0x456, 0x43d,
0x434, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432,
0x20, 0x406, 0x43d, 0x434, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x43e, 0x43c, 0x443,
0x20, 0x41e, 0x43a, 0x435, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x406, 0x43d, 0x434, 0x43e, 0x43a, 0x438,
0x442, 0x430, 0x457, 0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x43e, 0x456, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x456,
0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x456, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456,
0x434, 0x43d, 0x43e, 0x456, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x456, 0x439,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x456, 0x440, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x440, 0x430, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x456, 0x440, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x456, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x437, 0x440, 0x430, 0x457,
0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x456, 0x437, 0x440, 0x430, 0x457, 0x43b, 0x44c, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x456, 0x437, 0x440, 0x430, 0x457, 0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x44f, 0x43f, 0x43e, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x44f,
0x43f, 0x43e, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x44f, 0x43f, 0x43e, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x430, 0x43c, 0x447, 0x430, 0x442,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x430,
0x43c, 0x447, 0x430, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41a, 0x430, 0x437,
0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x441, 0x445,
0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x456,
0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41a, 0x430, 0x437, 0x430,
0x445, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x43a, 0x43e, 0x440,
0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x43a, 0x43e, 0x440, 0x435, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a,
0x43e, 0x440, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435,
0x437, 0x430, 0x20, 0x43a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a,
0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20,
0x41a, 0x438, 0x440, 0x433, 0x438, 0x437, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x447,
0x430, 0x441, 0x3a, 0x20, 0x41b, 0x430, 0x43d, 0x43a, 0x430, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x41b, 0x430, 0x439, 0x43d, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430,
0x443, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41b, 0x43e, 0x440,
0x434, 0x2d, 0x425, 0x430, 0x443, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41b, 0x43e, 0x440, 0x434,
0x2d, 0x425, 0x430, 0x443, 0x437, 0x430, 0x20, 0x43c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443,
0x20, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x456, 0x457, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41c, 0x430, 0x43b,
0x44c, 0x434, 0x456, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x456, 0x437,
0x44c, 0x43a, 0x438, 0x445, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x43e, 0x432, 0x438, 0x445,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x456,
0x439, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41c, 0x430, 0x432,
0x440, 0x438, 0x43a, 0x456, 0x439, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41c, 0x430, 0x432, 0x440,
0x438, 0x43a, 0x456, 0x439, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x456, 0x457, 0x20,
0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x437, 0x430, 0x20, 0x442, 0x438, 0x445,
0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41c, 0x435, 0x43a, 0x441,
0x438, 0x446, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x446, 0x456,
0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x442, 0x438,
0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x446, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x432, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e,
0x440, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432,
0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x456,
0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411,
0x430, 0x442, 0x43e, 0x440, 0x456, 0x437, 0x430, 0x20, 0x43c, 0x43e, 0x441, 0x43a,
0x43e, 0x432, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41c,
0x2bc, 0x44f, 0x43d, 0x43c, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456,
0x20, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x456, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41d, 0x43e, 0x432, 0x43e,
0x457, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x456, 0x457, 0x437,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41d, 0x43e, 0x432, 0x43e,
0x457, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x456, 0x457, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x430, 0x445, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x457, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x456, 0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41d, 0x43e, 0x432, 0x456, 0x439, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41d, 0x43e, 0x432, 0x456, 0x439, 0x20,
0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x41d, 0x43e, 0x432, 0x456, 0x439, 0x20, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20,
0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x44c, 0x44e, 0x444,
0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435,
0x43d, 0x434, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x456,
0x443, 0x435, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456,
0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x430, 0x440, 0x445, 0x456, 0x43f, 0x435,
0x43b, 0x430, 0x437, 0x456, 0x20, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x443, 0x2d, 0x434, 0x456, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x430, 0x440, 0x445, 0x456, 0x43f, 0x435, 0x43b, 0x430, 0x437, 0x456, 0x20, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x456, 0x2d, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x430, 0x440, 0x445, 0x456, 0x43f, 0x435, 0x43b, 0x430, 0x437, 0x456, 0x20,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x456, 0x2d,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x437, 0x430, 0x20, 0x43d, 0x43e,
0x432, 0x43e, 0x441, 0x438, 0x431, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43d, 0x43e, 0x432,
0x43e, 0x441, 0x438, 0x431, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43e, 0x43c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43e, 0x43c, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43e, 0x43c,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443,
0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41f, 0x430, 0x43b,
0x430, 0x443, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41f,
0x430, 0x43f, 0x443, 0x430, 0x2d, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x490, 0x432,
0x456, 0x43d, 0x435, 0x44f, 0x437, 0x430, 0x20, 0x43f, 0x430, 0x440, 0x430, 0x491,
0x432, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x43f, 0x430, 0x440, 0x430, 0x491, 0x432, 0x430,
0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x43f, 0x430,
0x440, 0x430, 0x491, 0x432, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x424, 0x456, 0x43b,
0x456, 0x43f, 0x43f, 0x456, 0x43d, 0x430, 0x445, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x424, 0x456, 0x43b, 0x456, 0x43f,
0x43f, 0x456, 0x43d, 0x430, 0x445, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x424, 0x456, 0x43b, 0x456, 0x43f, 0x43f, 0x456, 0x43d, 0x430, 0x445, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x424, 0x435, 0x43d, 0x456, 0x43a,
0x441, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x421, 0x435,
0x43d, 0x2d, 0x41f, 0x2019, 0x454, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445,
0x20, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x2019, 0x454, 0x440, 0x20, 0x456, 0x20,
0x41c, 0x456, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x437, 0x430, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x421,
0x435, 0x43d, 0x2d, 0x41f, 0x2019, 0x454, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456,
0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x20, 0x41f, 0x456, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41f, 0x445,
0x435, 0x43d, 0x44c, 0x44f, 0x43d, 0x456, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x41a,
0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x447, 0x430, 0x441, 0x3a,
0x20, 0x41a, 0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x439, 0x447,
0x430, 0x441, 0x3a, 0x20, 0x41a, 0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434,
0x430, 0x2c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x439, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x439, 0x43e,
0x43d, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x456, 0x457, 0x20, 0x420, 0x43e, 0x442,
0x435, 0x440, 0x430, 0x437, 0x430, 0x20, 0x441, 0x430, 0x445, 0x430, 0x43b, 0x456,
0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x441, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x441, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x441, 0x430, 0x43c, 0x430, 0x440, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441,
0x430, 0x43c, 0x430, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x430, 0x43c, 0x430, 0x440, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456,
0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x437, 0x430, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x421, 0x430,
0x43c, 0x43e, 0x430, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x44c, 0x441, 0x44c,
0x43a, 0x438, 0x445, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x421,
0x456, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x456, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x421, 0x43e, 0x43b, 0x43e,
0x43c, 0x43e, 0x43d, 0x43e, 0x432, 0x438, 0x445, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20,
0x41f, 0x456, 0x432, 0x434, 0x435, 0x43d, 0x43d, 0x430, 0x20, 0x414, 0x436, 0x43e,
0x440, 0x434, 0x436, 0x456, 0x44f, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x456,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x446, 0x456, 0x457, 0x20, 0x421, 0x435, 0x432, 0x430,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x422, 0x430, 0x457, 0x442,
0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20,
0x422, 0x430, 0x439, 0x431, 0x435, 0x457, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422, 0x430, 0x439, 0x431, 0x435, 0x457, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422, 0x430, 0x439, 0x431, 0x435, 0x457, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422, 0x430,
0x434, 0x436, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x422, 0x43e, 0x43a, 0x435, 0x43b,
0x430, 0x443, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445,
0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x422, 0x43e,
0x43d, 0x433, 0x430, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20,
0x427, 0x443, 0x443, 0x43a, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x443, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441,
0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x443, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441,
0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422, 0x443,
0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x422, 0x443, 0x432, 0x430,
0x43b, 0x443, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432,
0x20, 0x423, 0x440, 0x443, 0x491, 0x432, 0x430, 0x457, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x440, 0x443, 0x491, 0x432,
0x430, 0x457, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x440, 0x443, 0x491,
0x432, 0x430, 0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x432, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20,
0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445,
0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443,
0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x445, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x412, 0x435,
0x43d, 0x435, 0x441, 0x443, 0x435, 0x43b, 0x456, 0x437, 0x430, 0x20, 0x432, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x446, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x446, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x446, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x456,
0x457, 0x20, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x456, 0x20, 0x412, 0x435, 0x439, 0x43a, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x20, 0x423, 0x43e, 0x43b, 0x43b, 0x456, 0x441, 0x20,
0x456, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x437, 0x430, 0x20, 0x44f,
0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x44f, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x44f,
0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x454, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x437,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x454, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440,
0x437, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x454, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x437, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x42e, 0x43a, 0x43e, 0x43d, 0x456,
0x61, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x66,
0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66,
0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x6d, 0x61, 0x63, 0x6f, 0x6e, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x6d, 0x61, 0x63, 0x6f, 0x6e, 0x61, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x6d, 0x61, 0x63, 0x6f, 0x6e, 0x61, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65,
0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x77, 0x75,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x79, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x79, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x68, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x68, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x68, 0xf3, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65,
0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0x65, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a,
0x65, 0x64, 0x17a, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f,
0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65,
0x64, 0x17a, 0x6f, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x79, 0x20, 0x61,
0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x79, 0x20, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x7a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x79, 0x20, 0x61, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68,
0x6f, 0x64, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64,
0x17a, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17a, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17a,
0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x63, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x63, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x63, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x161, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x68,
0x75, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x62,
0x6f, 0x6c, 0x69, 0x77, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x62,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x63, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x63, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68,
0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x63, 0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x63, 0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x48, 0x6f, 0x64,
0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x10d,
0x61, 0x73, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x77, 0x79, 0x63,
0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x6b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f,
0x77, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79, 0x63,
0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79,
0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x6b, 0x75, 0x62, 0x61,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x75, 0x62, 0x61, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x75, 0x62, 0x61, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x44, 0x61,
0x76, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x20, 0x64, 0xb4, 0x20, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x4a, 0x75, 0x74,
0x72, 0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x20, 0x4a, 0x75, 0x74, 0x72, 0x6f, 0x77, 0x6e, 0x65, 0x6a,
0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x20, 0x4a, 0x75, 0x74, 0x72, 0x6f, 0x77, 0x6e, 0x65,
0x6a, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x65, 0x6b, 0x77, 0x61, 0x64,
0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a,
0x65, 0x64, 0x17a, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72,
0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77,
0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x66,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17a,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17a,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17a, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x73, 0x6b, 0x6f, 0x67, 0x75,
0x79, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x73, 0x6b, 0x65, 0x68,
0x6f, 0x20, 0x6a, 0x75, 0x17e, 0x6e, 0x65, 0x68, 0x6f, 0x20, 0x61, 0x20,
0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x6b, 0x65, 0x68,
0x6f, 0x20, 0x74, 0x65, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x6a, 0x61,
0x67, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x67, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x10d, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75,
0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0x61, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x50, 0x65, 0x72, 0x73, 0x69, 0x73, 0x6b, 0x65, 0x68, 0x6f,
0x20, 0x67, 0x6f, 0x6c, 0x66, 0x61, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69,
0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69, 0x73, 0x6b,
0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x69, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69,
0x6e, 0x64, 0x69, 0x73, 0x6b, 0x6f, 0x6f, 0x63, 0x65, 0x61, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x6f, 0x63, 0x68,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a,
0x65, 0x64, 0x17a, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x6b, 0x61, 0x7a,
0x61, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x6b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x10d, 0x61, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x69, 0x6a, 0x6f, 0x77,
0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x10d, 0x61, 0x73,
0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48,
0x6f, 0x77, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x6c, 0x61, 0x6a, 0x7a, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x6c, 0x65, 0x64,
0x69, 0x77, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x6d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x65, 0x78, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65,
0x78, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61,
0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x61, 0x75, 0x72, 0x75, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x65, 0x70, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77,
0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f,
0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f,
0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x69, 0x75, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70,
0x79, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x10d, 0x61, 0x73, 0x20, 0x6b,
0x75, 0x70, 0x6f, 0x77, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f,
0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6c,
0x61, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x70,
0x75, 0x61, 0x2d, 0x6e, 0x6f, 0x77, 0x6f, 0x67, 0x69, 0x6e, 0x65, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x72,
0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x66,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x6f, 0x77, 0x79,
0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x10d, 0x61, 0x73, 0x20,
0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20,
0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x61,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77,
0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x10d, 0x61,
0x73, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x77,
0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x70, 0x6f, 0x6e,
0x61, 0x70, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x6a,
0xf6, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x72, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f,
0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f,
0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f, 0x61,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x53,
0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x63, 0x68, 0x20,
0x6b, 0x75, 0x70, 0x6f, 0x77, 0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x67, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x74, 0x61, 0x68, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x61,
0x64, 0x17a, 0x69, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74,
0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x75, 0x75, 0x6b, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x65, 0x6e,
0x65, 0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x57,
0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70,
0x79, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75,
0x70, 0x6f, 0x77, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x6f, 0x77,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x633, 0x637,
0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x6c1, 0x20, 0x679, 0x627, 0x626,
0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642,
0x6c1, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20,
0x627, 0x641, 0x631, 0x6cc, 0x642, 0x6c1, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc,
0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x6c1, 0x20, 0x679, 0x627, 0x626, 0x645,
0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x6c1,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642,
0x6c1, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x644,
0x627, 0x633, 0x6a9, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x644, 0x627,
0x633, 0x6a9, 0x627, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20,
0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627,
0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x645, 0x6cc, 0x632,
0x648, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x646,
0x679, 0x631, 0x644, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x646, 0x679, 0x631,
0x644, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x688, 0x6d2, 0x20,
0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633,
0x679, 0x631, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679,
0x631, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x688,
0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x627, 0x624, 0x646, 0x679, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x627, 0x624, 0x646, 0x679, 0x6cc, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x624, 0x646,
0x679, 0x6cc, 0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20,
0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x679, 0x627,
0x626, 0x645, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x627, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x633,
0x641, 0x6a9, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x646, 0x6cc, 0x62f, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x627, 0x646, 0x6cc, 0x62f, 0x631, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x646, 0x6cc, 0x62f,
0x631, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc,
0x67e, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x67e, 0x6cc,
0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x6cc, 0x67e, 0x6cc, 0x627, 0x20, 0x688, 0x6d2, 0x20, 0x644,
0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x639, 0x631, 0x628, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x639, 0x631, 0x628, 0x20, 0x6a9, 0x627,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x639,
0x631, 0x628, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc,
0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628,
0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627,
0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631,
0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x631, 0x645, 0x6cc, 0x646,
0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x631, 0x645,
0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc,
0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x679, 0x644,
0x627, 0x646, 0x679, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x679, 0x644,
0x627, 0x646, 0x679, 0x6a9, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x679, 0x644, 0x627, 0x646, 0x679,
0x6a9, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x622, 0x633, 0x679, 0x631,
0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679,
0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x633, 0x646, 0x679, 0x631, 0x644, 0x20,
0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x633, 0x646,
0x679, 0x631, 0x644, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20,
0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646,
0x20, 0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x648, 0x6cc, 0x633, 0x679, 0x631,
0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644,
0x6cc, 0x646, 0x20, 0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x648, 0x6cc, 0x633,
0x679, 0x631, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc,
0x646, 0x20, 0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x648, 0x6cc, 0x633, 0x679,
0x631, 0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x622, 0x633,
0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622,
0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x627, 0x6cc, 0x633, 0x679,
0x631, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646,
0x20, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644,
0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x6cc, 0x633, 0x679,
0x631, 0x646, 0x20, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x627, 0x20,
0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x627,
0x20, 0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679,
0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646,
0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626,
0x645, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626,
0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626,
0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x648,
0x631, 0x6cc, 0x633, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x628,
0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x642, 0x62a, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x6be, 0x648, 0x679, 0x627,
0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x648, 0x644, 0x6cc,
0x648, 0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x631,
0x627, 0x632, 0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628,
0x631, 0x627, 0x632, 0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x627, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x631, 0x627,
0x632, 0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x628, 0x631, 0x648, 0x646, 0x626, 0x6cc, 0x20, 0x62f, 0x627, 0x631,
0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9,
0x6cc, 0x67e, 0x20, 0x648, 0x631, 0x688, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631, 0x688, 0x6cc, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x67e,
0x20, 0x648, 0x631, 0x688, 0x6cc, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x686, 0x627, 0x645, 0x648, 0x631, 0x648, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x6cc, 0x62a,
0x6be, 0x645, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x6cc, 0x62a, 0x6be, 0x645,
0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x686, 0x6cc, 0x62a, 0x6be, 0x645, 0x20, 0x688, 0x6d2, 0x20, 0x644,
0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x644, 0x6cc, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x644, 0x6cc, 0x20, 0x6a9, 0x627,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x686,
0x644, 0x6cc, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x6cc,
0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x6cc, 0x646, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x686, 0x6cc, 0x646, 0x6cc, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x631, 0x633, 0x645, 0x633, 0x20, 0x622,
0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648,
0x6a9, 0x648, 0x633, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20,
0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20,
0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642,
0x62a, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x6a9, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646,
0x688, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6a9, 0x20, 0x622, 0x626,
0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6a9, 0x20, 0x622, 0x626, 0x644,
0x6cc, 0x646, 0x688, 0x632, 0x20, 0x646, 0x635, 0x641, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x648, 0x628, 0x627, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x648, 0x628, 0x627, 0x20, 0x627, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6cc,
0x648, 0x628, 0x627, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20,
0x679, 0x627, 0x626, 0x645, 0x688, 0x6cc, 0x648, 0x633, 0x20, 0x679, 0x627, 0x626,
0x645, 0x688, 0x648, 0x645, 0x648, 0x646, 0x679, 0x2d, 0x688, 0x6cc, 0x2019, 0x627,
0x631, 0x648, 0x6cc, 0x644, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x634,
0x631, 0x642, 0x6cc, 0x20, 0x62a, 0x6cc, 0x645, 0x648, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x633,
0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x6a9, 0x648, 0x627,
0x688, 0x648, 0x631, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x633,
0x637, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x648, 0x633, 0x637, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a,
0x648, 0x633, 0x637, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634,
0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634,
0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x628, 0x639, 0x6cc, 0x62f, 0x20, 0x645, 0x634, 0x631,
0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x6cc, 0x20, 0x648, 0x642, 0x62a,
0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641,
0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x632, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x627,
0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646,
0x688, 0x632, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x642, 0x62a, 0x641, 0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688,
0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x641, 0x62c, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645,
0x641, 0x62c, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x641, 0x62c, 0x6cc, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x641, 0x631, 0x6cc, 0x646, 0x686, 0x20, 0x6af, 0x6cc,
0x627, 0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631,
0x6cc, 0x646, 0x686, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x627, 0x648,
0x631, 0x20, 0x627, 0x646, 0x679, 0x627, 0x631, 0x6a9, 0x679, 0x6a9, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x632,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x6af, 0x6cc, 0x645, 0x628, 0x6cc,
0x626, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x627, 0x631, 0x62c, 0x6cc,
0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x631, 0x62c,
0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x644, 0x628, 0x631, 0x679, 0x20,
0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x648, 0x686, 0x20, 0x6a9, 0x627, 0x20, 0x627,
0x635, 0x644, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20,
0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x646,
0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc,
0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20,
0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x646,
0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc,
0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x62e, 0x644, 0x6cc, 0x62c, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a,
0x6c1, 0x648, 0x627, 0x626, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648, 0x679, 0x6cc,
0x626, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648, 0x627, 0x626, 0x6cc,
0x20, 0x627, 0x644, 0x6cc, 0x648, 0x679, 0x6cc, 0x626, 0x646, 0x20, 0x627, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1,
0x648, 0x627, 0x626, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648, 0x679, 0x6cc, 0x626,
0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648, 0x648, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648, 0x648, 0x688, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648,
0x648, 0x688, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1,
0x646, 0x62f, 0x648, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x62d, 0x631,
0x20, 0x6c1, 0x646, 0x62f, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x646, 0x62f,
0x20, 0x686, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x633, 0x637,
0x6cc, 0x20, 0x627, 0x646, 0x688, 0x648, 0x646, 0x6cc, 0x634, 0x6cc, 0x627, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x627, 0x646,
0x688, 0x648, 0x646, 0x6cc, 0x634, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645,
0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x646, 0x688, 0x648, 0x646, 0x6cc,
0x634, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x631, 0x627,
0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x631, 0x627,
0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x688, 0x6d2, 0x20,
0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x631, 0x6a9,
0x62a, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x631, 0x6a9, 0x62a,
0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x631, 0x6a9, 0x62a, 0x633, 0x6a9, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc,
0x644, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x633, 0x631, 0x627,
0x626, 0x6cc, 0x644, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644,
0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626,
0x645, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62c,
0x627, 0x67e, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x688,
0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e,
0x6cc, 0x679, 0x631, 0x648, 0x67e, 0x627, 0x624, 0x644, 0x648, 0x648, 0x633, 0x6a9,
0x2d, 0x6a9, 0x6cc, 0x645, 0x686, 0x633, 0x6a9, 0x6cc, 0x20, 0x679, 0x627, 0x626,
0x645, 0x67e, 0x6cc, 0x679, 0x631, 0x648, 0x67e, 0x627, 0x624, 0x644, 0x648, 0x648,
0x633, 0x6a9, 0x2d, 0x6a9, 0x6cc, 0x645, 0x686, 0x633, 0x6a9, 0x6cc, 0x20, 0x627,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x67e, 0x6cc, 0x679, 0x631, 0x648, 0x67e, 0x627, 0x624, 0x644, 0x648, 0x648, 0x633,
0x6a9, 0x2d, 0x6a9, 0x6cc, 0x645, 0x686, 0x633, 0x6a9, 0x6cc, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x642, 0x627, 0x632, 0x642, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631,
0x642, 0x6cc, 0x20, 0x642, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20,
0x642, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x6a9, 0x648, 0x631, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6a9, 0x648, 0x631, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x631, 0x6cc, 0x627,
0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6a9, 0x648, 0x633, 0x631, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9,
0x631, 0x6cc, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6a9, 0x631, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633,
0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6a9, 0x631, 0x6cc, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633,
0x6a9, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x631,
0x63a, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x644, 0x627, 0x626, 0x646, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632,
0x20, 0x679, 0x627, 0x626, 0x645, 0x644, 0x627, 0x631, 0x688, 0x20, 0x6c1, 0x648,
0x648, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x644, 0x627, 0x631, 0x688, 0x20,
0x6c1, 0x648, 0x648, 0x6d2, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x644, 0x627, 0x631, 0x688, 0x20, 0x6c1,
0x648, 0x648, 0x6d2, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6af, 0x6cc, 0x62f, 0x646, 0x20, 0x679,
0x627, 0x626, 0x645, 0x645, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x627, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x6cc, 0x6af, 0x6cc, 0x62f, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x644, 0x6cc, 0x634, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626,
0x645, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x627, 0x631, 0x6a9, 0x6cc, 0x633, 0x633, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x627, 0x631, 0x634, 0x644, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x631, 0x6cc,
0x634, 0x633, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x631, 0x6cc, 0x634,
0x633, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x627, 0x631, 0x6cc, 0x634, 0x633, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x624, 0x633, 0x646, 0x20, 0x679,
0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e,
0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6a9,
0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x627,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641,
0x6a9, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6cc, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x6cc, 0x62a, 0x648,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x648, 0x644, 0x627, 0x646, 0x20,
0x628, 0x6cc, 0x62a, 0x648, 0x631, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x648, 0x644, 0x627, 0x646, 0x20,
0x628, 0x6cc, 0x62a, 0x648, 0x631, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x679, 0x627, 0x626, 0x645,
0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x633, 0x6a9, 0x648,
0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x627,
0x646, 0x645, 0x627, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x627, 0x624,
0x631, 0x648, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x67e, 0x627, 0x644,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x6cc, 0x648, 0x20, 0x6a9,
0x6cc, 0x644, 0x6cc, 0x688, 0x648, 0x646, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626,
0x645, 0x646, 0x6cc, 0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x6cc, 0x688, 0x648, 0x646,
0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x646, 0x6cc, 0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x6cc, 0x688,
0x648, 0x646, 0x6cc, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20,
0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20,
0x644, 0x6cc, 0x646, 0x688, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679,
0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x648, 0x20, 0x641, 0x627, 0x624,
0x646, 0x688, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x627, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x626,
0x648, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x627, 0x631, 0x641, 0x648, 0x6a9,
0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x646, 0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20,
0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627,
0x626, 0x679, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688,
0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x646, 0x6c1, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688,
0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x646, 0x6c1, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642,
0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20,
0x646, 0x648, 0x631, 0x648, 0x646, 0x6c1, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x646, 0x648, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x631,
0x633, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x648, 0x648, 0x648, 0x633,
0x6cc, 0x628, 0x631, 0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x648, 0x648, 0x648, 0x633, 0x6cc,
0x628, 0x631, 0x633, 0x6a9, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627,
0x648, 0x645, 0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x20, 0x633,
0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x627, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x627, 0x6a9,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x644, 0x627,
0x624, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x627, 0x67e, 0x648, 0x622, 0x20,
0x646, 0x6cc, 0x648, 0x20, 0x6af, 0x646, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645,
0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642,
0x62a, 0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x6cc,
0x631, 0x648, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x644,
0x67e, 0x627, 0x626, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x644, 0x67e,
0x627, 0x626, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x641, 0x644, 0x67e, 0x627, 0x626, 0x646, 0x20, 0x633,
0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x6cc, 0x646, 0x6a9, 0x633,
0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679, 0x627, 0x626,
0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x6cc, 0x626, 0x631, 0x20, 0x627,
0x648, 0x631, 0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x20, 0x679,
0x627, 0x626, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x6cc, 0x626, 0x631,
0x20, 0x627, 0x648, 0x631, 0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646,
0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x6cc, 0x626, 0x631, 0x20,
0x627, 0x648, 0x631, 0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x20,
0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645,
0x67e, 0x679, 0x6a9, 0x627, 0x626, 0x631, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x67e, 0x648, 0x646, 0x627, 0x67e, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e,
0x6cc, 0x627, 0x646, 0x6af, 0x20, 0x6cc, 0x627, 0x646, 0x6af, 0x20, 0x648, 0x642,
0x62a, 0x631, 0x6cc, 0x20, 0x6cc, 0x648, 0x646, 0x6cc, 0x646, 0x20, 0x679, 0x627,
0x626, 0x645, 0x631, 0x648, 0x62a, 0x6be, 0x6cc, 0x631, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x633, 0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x20, 0x679,
0x627, 0x626, 0x645, 0x633, 0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x62e,
0x627, 0x644, 0x6cc, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x633, 0x645, 0x627, 0x631, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633,
0x645, 0x627, 0x631, 0x627, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x645, 0x627, 0x631, 0x627, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x627, 0x645, 0x648,
0x622, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x627, 0x645, 0x648, 0x622, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x633, 0x627, 0x645, 0x648, 0x622, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626,
0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x6cc, 0x634, 0x644, 0x6cc, 0x632,
0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x633, 0x648, 0x644, 0x645, 0x646, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646,
0x688, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x646, 0x648, 0x628, 0x6cc,
0x20, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645,
0x633, 0x648, 0x631, 0x6cc, 0x646, 0x627, 0x645, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x633, 0x6cc, 0x648, 0x648, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645,
0x62a, 0x627, 0x6c1, 0x6cc, 0x62a, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a,
0x627, 0x626, 0x6cc, 0x20, 0x67e, 0x6cc, 0x626, 0x6cc, 0x20, 0x679, 0x627, 0x626,
0x645, 0x62a, 0x627, 0x626, 0x6cc, 0x20, 0x67e, 0x6cc, 0x626, 0x6cc, 0x20, 0x627,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x62a, 0x626, 0x6cc, 0x20, 0x67e, 0x6cc, 0x626, 0x6cc, 0x20, 0x688, 0x6d2, 0x20,
0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x627, 0x62c,
0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x679, 0x648, 0x6a9, 0x6cc, 0x644, 0x627, 0x624, 0x20, 0x679, 0x627, 0x626, 0x645,
0x679, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x679, 0x648,
0x646, 0x6af, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x679, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x648, 0x6a9, 0x20, 0x679, 0x627,
0x626, 0x645, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x627,
0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x627,
0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x679, 0x648, 0x627, 0x644, 0x648, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc,
0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a,
0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x627, 0x646,
0x648, 0x622, 0x679, 0x648, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x627, 0x646,
0x648, 0x622, 0x679, 0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x627, 0x646, 0x648, 0x622, 0x679, 0x648,
0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x6cc, 0x646,
0x632, 0x648, 0x626, 0x6cc, 0x644, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x648, 0x633, 0x62a, 0x6a9,
0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x20, 0x648,
0x648, 0x633, 0x62a, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x20, 0x648,
0x648, 0x633, 0x62a, 0x6a9, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x679,
0x627, 0x626, 0x645, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f,
0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x648, 0x633, 0x679,
0x627, 0x6a9, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x6cc, 0x6a9,
0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x648, 0x627, 0x644, 0x6cc, 0x632, 0x20, 0x627, 0x648, 0x631, 0x20, 0x641, 0x679,
0x648, 0x646, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x648, 0x62a,
0x633, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x648, 0x62a, 0x633,
0x6a9, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x627, 0x679, 0x6cc, 0x631,
0x650, 0x646, 0x628, 0x631, 0x6af, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x6a9,
0x627, 0x679, 0x6cc, 0x631, 0x650, 0x646, 0x628, 0x631, 0x6af, 0x20, 0x627, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc,
0x6a9, 0x627, 0x679, 0x6cc, 0x631, 0x650, 0x646, 0x628, 0x631, 0x6af, 0x20, 0x633,
0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x641, 0x63a, 0x627, 0x646,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x645,
0x6cc, 0x632, 0x648, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x645,
0x6cc, 0x632, 0x648, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646,
0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x639, 0x631, 0x628,
0x20, 0x679, 0x627, 0x626, 0x645, 0x639, 0x631, 0x628, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631,
0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631,
0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc,
0x20, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc,
0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc,
0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626,
0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x630, 0x631,
0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x630, 0x631, 0x628,
0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634,
0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f,
0x6cc, 0x634, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634,
0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x6be, 0x648,
0x679, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x648, 0x644, 0x6cc,
0x648, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x631, 0x627, 0x632,
0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x644, 0x6cc, 0x20, 0x679, 0x627, 0x626,
0x645, 0x686, 0x644, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x644, 0x6cc, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622,
0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc,
0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627,
0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x6a9, 0x648,
0x627, 0x688, 0x648, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x633, 0x637,
0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x641, 0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626,
0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x627,
0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646,
0x688, 0x632, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x641, 0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x641, 0x631, 0x6cc, 0x646, 0x686, 0x20, 0x6af, 0x6cc,
0x627, 0x646, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6af, 0x627, 0x644, 0x627,
0x67e, 0x627, 0x6af, 0x648, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x627,
0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6af, 0x631, 0x6cc, 0x646,
0x20, 0x648, 0x686, 0x20, 0x645, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x62e, 0x644, 0x6cc, 0x62c, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627,
0x6cc, 0x631, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x631,
0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x634, 0x631, 0x642, 0x6cc, 0x20, 0x642, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20,
0x642, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6a9, 0x631, 0x63a, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626,
0x645, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x67e, 0x20, 0x679, 0x627, 0x626, 0x645,
0x646, 0x6cc, 0x67e, 0x627, 0x644, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc,
0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646,
0x648, 0x631, 0x646, 0x6c1, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x631,
0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631,
0x646, 0x6c1, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x679, 0x627, 0x626, 0x645, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2,
0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626,
0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633,
0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x631, 0x648, 0x20,
0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x631,
0x648, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x631, 0x648,
0x62a, 0x6be, 0x6cc, 0x631, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x648,
0x631, 0x6cc, 0x646, 0x627, 0x645, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x631,
0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648, 0x626,
0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648,
0x626, 0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x632, 0x628, 0x6a9,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x632, 0x628,
0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648,
0x6cc, 0x646, 0x632, 0x648, 0x626, 0x6cc, 0x644, 0x627, 0x20, 0x679, 0x627, 0x626,
0x645, 0x648, 0x648, 0x633, 0x679, 0x627, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645,
0x626, 0x627, 0x643, 0x631, 0x6d0, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x643, 0x631, 0x6d0, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x643, 0x631,
0x649, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x649, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7,
0x631, 0x627, 0x20, 0x626, 0x627, 0x641, 0x631, 0x649, 0x642, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x626,
0x627, 0x641, 0x631, 0x649, 0x642, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x62c, 0x6d5, 0x646, 0x6c7, 0x628, 0x649, 0x64a, 0x20, 0x626, 0x627, 0x641, 0x631,
0x649, 0x642, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649,
0x64a, 0x20, 0x626, 0x627, 0x641, 0x631, 0x649, 0x642, 0x627, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x626, 0x627,
0x641, 0x631, 0x649, 0x642, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631,
0x628, 0x649, 0x64a, 0x20, 0x626, 0x627, 0x641, 0x631, 0x649, 0x642, 0x627, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x627, 0x644, 0x64a, 0x627, 0x633, 0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x644, 0x64a, 0x627, 0x633, 0x643, 0x627, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x644, 0x64a, 0x627, 0x633, 0x643, 0x627, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x644, 0x645, 0x6c7, 0x62a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x627, 0x644, 0x645, 0x6c7, 0x62a, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x644, 0x645, 0x6c7, 0x62a, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x645, 0x627, 0x632,
0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x645, 0x627,
0x632, 0x648, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x645, 0x627, 0x632,
0x648, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642,
0x649, 0x633, 0x649, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648,
0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642,
0x649, 0x633, 0x649, 0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20,
0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634,
0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649,
0x633, 0x649, 0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x62a, 0x627, 0x63a, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x62a, 0x627, 0x63a, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x627, 0x63a, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x62a, 0x649, 0x646, 0x686, 0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x649, 0x646, 0x686, 0x20, 0x626, 0x648,
0x643, 0x64a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x649, 0x646, 0x686,
0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x646, 0x627,
0x62f, 0x649, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x646,
0x627, 0x62f, 0x649, 0x631, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x646, 0x627,
0x62f, 0x649, 0x631, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x642, 0x62a, 0x627, 0x64a, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x642, 0x62a, 0x627, 0x64a, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x642, 0x62a, 0x627, 0x64a, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x642,
0x62a, 0x6c6, 0x628, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627,
0x642, 0x62a, 0x6c6, 0x628, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x642,
0x62a, 0x6c6, 0x628, 0x6d5, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x631, 0x6d5, 0x628, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x631, 0x6d5, 0x628, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x6d5, 0x631, 0x6d5, 0x628, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x631, 0x6af, 0x6d0,
0x646, 0x62a, 0x649, 0x646, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649, 0x646, 0x627, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x627, 0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649, 0x646, 0x627, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x626, 0x627, 0x631, 0x6af, 0x6d0,
0x646, 0x62a, 0x649, 0x646, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a,
0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x626, 0x627, 0x631, 0x6af, 0x6d0, 0x646,
0x62a, 0x649, 0x646, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628,
0x649, 0x64a, 0x20, 0x626, 0x627, 0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649, 0x646,
0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x6d5, 0x631, 0x645, 0x6d0, 0x646, 0x649, 0x64a, 0x6d5, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x631, 0x645, 0x6d0, 0x646, 0x649,
0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x631, 0x645, 0x6d0, 0x646,
0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x649,
0x643, 0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x649, 0x643, 0x20,
0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627,
0x62a, 0x644, 0x627, 0x646, 0x62a, 0x649, 0x643, 0x20, 0x626, 0x648, 0x643, 0x64a,
0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649,
0x64a, 0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642,
0x649, 0x633, 0x649, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627,
0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x648,
0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649,
0x64a, 0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642,
0x649, 0x633, 0x649, 0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627,
0x20, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649,
0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a,
0x649, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a,
0x6c7, 0x631, 0x627, 0x20, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642,
0x649, 0x633, 0x649, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a,
0x6c7, 0x631, 0x627, 0x20, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642,
0x649, 0x633, 0x649, 0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20,
0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x634,
0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649,
0x64a, 0x6d5, 0x20, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649,
0x633, 0x649, 0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644,
0x649, 0x64a, 0x6d5, 0x20, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642,
0x649, 0x633, 0x649, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627,
0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x63a, 0x6d5,
0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a,
0x6d5, 0x20, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633,
0x649, 0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x632, 0x6d5, 0x631, 0x628, 0x6d5, 0x64a, 0x62c,
0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x632, 0x6d5,
0x631, 0x628, 0x6d5, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x6d5, 0x632, 0x6d5, 0x631, 0x628, 0x6d5, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x632, 0x648, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627,
0x632, 0x648, 0x631, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x632, 0x648, 0x631,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x628, 0x627, 0x6ad, 0x644, 0x627, 0x62f, 0x649, 0x634, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x628, 0x627, 0x6ad, 0x644, 0x627, 0x62f, 0x649, 0x634, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x628, 0x627, 0x6ad, 0x644, 0x627, 0x62f, 0x649, 0x634, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x628, 0x6c7, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628,
0x648, 0x644, 0x649, 0x6cb, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x628, 0x649, 0x631, 0x627, 0x632, 0x649, 0x644, 0x649, 0x64a, 0x6d5, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x649, 0x631, 0x627, 0x632, 0x649, 0x644,
0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x649, 0x631, 0x627, 0x632,
0x649, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x649, 0x631, 0x6c7, 0x646, 0x649,
0x64a, 0x20, 0x62f, 0x627, 0x631, 0x6c7, 0x633, 0x633, 0x627, 0x644, 0x627, 0x645,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x634, 0x649, 0x644, 0x20,
0x62a, 0x6c7, 0x645, 0x634, 0x6c7, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x64a, 0x6d0, 0x634, 0x649, 0x644, 0x20, 0x62a, 0x6c7, 0x645, 0x634, 0x6c7, 0x642,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x634, 0x649, 0x644, 0x20, 0x62a, 0x6c7,
0x645, 0x634, 0x6c7, 0x642, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x627, 0x633, 0x6d0, 0x64a, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x686, 0x627, 0x645, 0x648, 0x631, 0x631, 0x648, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x686, 0x627, 0x62a, 0x627, 0x645, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x686, 0x627, 0x62a, 0x627, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686,
0x627, 0x62a, 0x627, 0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686, 0x649, 0x644, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x686, 0x649, 0x644, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686,
0x649, 0x644, 0x649, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x62c, 0x6c7, 0x6ad, 0x6af, 0x648, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x62c, 0x6c7, 0x6ad, 0x6af, 0x648, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x62c, 0x6c7, 0x6ad, 0x6af, 0x648, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x631, 0x648, 0x698, 0x62f, 0x6d0, 0x633,
0x62a, 0x6cb, 0x648, 0x20, 0x626, 0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x643, 0x6c7, 0x633, 0x20, 0x626, 0x627,
0x631, 0x649, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648,
0x644, 0x648, 0x645, 0x628, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x643, 0x648, 0x644, 0x648, 0x645, 0x628, 0x649, 0x64a, 0x6d5, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x643, 0x648, 0x644, 0x648, 0x645, 0x628, 0x649, 0x64a, 0x6d5, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x643, 0x6c7, 0x643, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x643, 0x20, 0x626,
0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x643, 0x6c7, 0x643, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631,
0x649, 0x20, 0x64a, 0x6d0, 0x631, 0x649, 0x645, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x628, 0x627,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x628, 0x627, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x643, 0x6c7, 0x628, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62f, 0x627, 0x6cb, 0x649, 0x633,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62f, 0x6c7, 0x645, 0x648, 0x646, 0x62a,
0x2d, 0x62f, 0x6c7, 0x631, 0x6cb, 0x649, 0x644, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x62a, 0x649, 0x645, 0x648,
0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d0, 0x633, 0x62a, 0x6d0,
0x631, 0x20, 0x626, 0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x67e, 0x627, 0x633, 0x643, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x626,
0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d0, 0x633,
0x62a, 0x6d0, 0x631, 0x20, 0x626, 0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x6d0, 0x643, 0x6cb, 0x627, 0x62f, 0x648, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x64a, 0x627, 0x6cb,
0x631, 0x648, 0x67e, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648,
0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e,
0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627,
0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631,
0x642, 0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20,
0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634,
0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e,
0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb,
0x631, 0x648, 0x67e, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5,
0x631, 0x628, 0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x64a,
0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x627, 0x644, 0x643, 0x644,
0x627, 0x646, 0x62f, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x627, 0x644, 0x643, 0x644,
0x627, 0x646, 0x62f, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631,
0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x627, 0x644, 0x643, 0x644, 0x627, 0x646,
0x62f, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x641, 0x649, 0x62c, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649,
0x62c, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649, 0x62c, 0x649, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641,
0x649, 0x631, 0x627, 0x646, 0x633, 0x649, 0x64a, 0x6d5, 0x6af, 0x6d5, 0x20, 0x642,
0x627, 0x631, 0x627, 0x634, 0x644, 0x649, 0x642, 0x20, 0x6af, 0x649, 0x6cb, 0x649,
0x64a, 0x627, 0x646, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649,
0x631, 0x627, 0x646, 0x633, 0x649, 0x64a, 0x6d5, 0x6af, 0x6d5, 0x20, 0x642, 0x627,
0x631, 0x627, 0x634, 0x644, 0x649, 0x642, 0x20, 0x62c, 0x6d5, 0x646, 0x6c7, 0x628,
0x649, 0x64a, 0x20, 0x6cb, 0x6d5, 0x20, 0x626, 0x627, 0x646, 0x62a, 0x627, 0x631,
0x643, 0x62a, 0x649, 0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af,
0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x633, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x6af, 0x627, 0x645, 0x628, 0x649, 0x64a, 0x6d0, 0x631, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x6af, 0x649, 0x631, 0x6c7, 0x632, 0x649, 0x64a, 0x6d5,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af, 0x649, 0x631, 0x6c7, 0x632, 0x649,
0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af, 0x649, 0x631, 0x6c7, 0x632, 0x649,
0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x6af, 0x649, 0x644, 0x628, 0x6d0, 0x631, 0x62a, 0x20, 0x626,
0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x6af, 0x649, 0x631, 0x649, 0x646, 0x6cb, 0x649, 0x686, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x6af,
0x649, 0x631, 0x6d0, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631,
0x6d0, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5,
0x631, 0x642, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646, 0x644, 0x627,
0x646, 0x62f, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x6af, 0x649,
0x631, 0x6d0, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0,
0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631,
0x628, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646, 0x644, 0x627, 0x646,
0x62f, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x6af, 0x6c7, 0x626, 0x627, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af,
0x6c7, 0x644, 0x641, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6be, 0x627, 0x6cb, 0x627, 0x64a,
0x2d, 0x626, 0x627, 0x644, 0x6d0, 0x64a, 0x6c7, 0x62a, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x6be, 0x627, 0x6cb, 0x627, 0x64a, 0x2d, 0x626, 0x627, 0x644, 0x6d0,
0x64a, 0x6c7, 0x62a, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6be, 0x627, 0x6cb, 0x627, 0x64a,
0x2d, 0x626, 0x627, 0x644, 0x6d0, 0x64a, 0x6c7, 0x62a, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x64a, 0x627,
0x6ad, 0x6af, 0x627, 0x6ad, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x64a,
0x627, 0x6ad, 0x6af, 0x627, 0x6ad, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x64a, 0x627,
0x6ad, 0x6af, 0x627, 0x6ad, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62e, 0x648, 0x6cb, 0x62f, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x62e, 0x648, 0x6cb, 0x62f, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62e,
0x648, 0x6cb, 0x62f, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x6be, 0x649, 0x646, 0x62f, 0x649, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6be, 0x649, 0x646, 0x62f, 0x649, 0x20, 0x626,
0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6be,
0x649, 0x646, 0x62f, 0x649, 0x20, 0x686, 0x649, 0x646, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x6be,
0x649, 0x646, 0x62f, 0x648, 0x646, 0x6d0, 0x632, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x6be,
0x649, 0x646, 0x62f, 0x648, 0x646, 0x6d0, 0x632, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x6be,
0x649, 0x646, 0x62f, 0x648, 0x646, 0x6d0, 0x632, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x631, 0x627, 0x646, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x649, 0x631, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x649, 0x631, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x631, 0x643, 0x6c7, 0x62a,
0x633, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x631, 0x643,
0x6c7, 0x62a, 0x633, 0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x631, 0x643,
0x6c7, 0x62a, 0x633, 0x643, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x633, 0x631, 0x627, 0x626, 0x649,
0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649,
0x633, 0x631, 0x627, 0x626, 0x649, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x649, 0x633, 0x631, 0x627, 0x626, 0x649, 0x644, 0x649, 0x64a, 0x6d5,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x64a, 0x627, 0x67e, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x64a, 0x627, 0x67e, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x64a, 0x627, 0x67e, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x67e, 0x6d0, 0x62a, 0x631, 0x648, 0x67e, 0x627, 0x6cb, 0x644, 0x648, 0x6cb, 0x633,
0x643, 0x2d, 0x643, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x643, 0x633, 0x643, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x62a, 0x631, 0x648, 0x67e,
0x627, 0x6cb, 0x644, 0x648, 0x6cb, 0x633, 0x643, 0x2d, 0x643, 0x627, 0x645, 0x686,
0x627, 0x62a, 0x643, 0x633, 0x643, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0,
0x62a, 0x631, 0x648, 0x67e, 0x627, 0x6cb, 0x644, 0x648, 0x6cb, 0x633, 0x643, 0x2d,
0x643, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x643, 0x633, 0x643, 0x649, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634,
0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642, 0x649,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5,
0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642, 0x649, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x631,
0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x631,
0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x631, 0x649, 0x64a,
0x6d5, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x643, 0x648, 0x633, 0x631, 0x627, 0x626, 0x6d0, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x643, 0x649, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627,
0x631, 0x633, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x649, 0x631,
0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x643, 0x649, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633,
0x643, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x642, 0x649, 0x631, 0x63a, 0x649, 0x632, 0x649, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x649, 0x631, 0x649, 0x20,
0x644, 0x627, 0x646, 0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x644,
0x627, 0x64a, 0x646, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x2d,
0x62e, 0x627, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x644, 0x648, 0x631,
0x62f, 0x2d, 0x62e, 0x627, 0x64a, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x644, 0x648, 0x631,
0x62f, 0x2d, 0x62e, 0x627, 0x64a, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x645, 0x6d0, 0x646,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x645, 0x6d0, 0x646,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x645, 0x6d0, 0x646, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645,
0x627, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x645, 0x627, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645,
0x627, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x644, 0x627, 0x64a,
0x634, 0x649, 0x64a, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627,
0x644, 0x62f, 0x649, 0x6cb, 0x6d0, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645,
0x627, 0x631, 0x643, 0x649, 0x632, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645,
0x627, 0x631, 0x634, 0x627, 0x644, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644,
0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x6cb,
0x631, 0x649, 0x62a, 0x649, 0x626, 0x6c7, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x645, 0x627, 0x6cb, 0x631, 0x649, 0x62a, 0x649, 0x626, 0x6c7, 0x633, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x645, 0x627, 0x6cb, 0x631, 0x649, 0x62a, 0x649, 0x626, 0x6c7,
0x633, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x645, 0x627, 0x6cb, 0x633, 0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x645, 0x6d0, 0x643, 0x633, 0x649, 0x643, 0x627, 0x20, 0x62a, 0x649,
0x646, 0x686, 0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x645, 0x6d0, 0x643, 0x633, 0x649, 0x643, 0x627, 0x20, 0x62a,
0x649, 0x646, 0x686, 0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x645, 0x6d0, 0x643, 0x633, 0x649, 0x643, 0x627, 0x20, 0x62a, 0x649,
0x646, 0x686, 0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c7,
0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x6c7, 0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x648, 0x631,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c7, 0x644, 0x627, 0x646, 0x628, 0x627, 0x62a,
0x648, 0x631, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x645, 0x648, 0x633, 0x643, 0x6cb, 0x627, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x645, 0x648, 0x633, 0x643, 0x6cb, 0x627, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x645, 0x648, 0x633, 0x643, 0x6cb, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x649, 0x631, 0x645,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x627, 0x6cb, 0x631, 0x6c7,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x6d0, 0x67e, 0x627, 0x644, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x643, 0x627,
0x644, 0x6d0, 0x62f, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x643, 0x627, 0x644, 0x6d0, 0x62f,
0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x6ad,
0x649, 0x20, 0x643, 0x627, 0x644, 0x6d0, 0x62f, 0x648, 0x646, 0x649, 0x64a, 0x6d5,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x632, 0x6d0, 0x644, 0x627, 0x646, 0x62f,
0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x6ad,
0x649, 0x20, 0x632, 0x6d0, 0x644, 0x627, 0x646, 0x62f, 0x649, 0x64a, 0x6d5, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x632, 0x6d0, 0x644, 0x627,
0x646, 0x62f, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x649, 0x6cb, 0x641, 0x648, 0x626,
0x6c7, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x646, 0x649, 0x6cb, 0x641, 0x648, 0x626, 0x6c7, 0x646, 0x644, 0x627, 0x646, 0x62f,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x646, 0x649, 0x6cb, 0x641, 0x648, 0x626, 0x6c7, 0x646,
0x644, 0x627, 0x646, 0x62f, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x649, 0x64a, 0x6c7, 0x626, 0x6d0, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643,
0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20,
0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x626, 0x627, 0x631,
0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x6d0, 0x631, 0x646, 0x627,
0x646, 0x62f, 0x648, 0x2d, 0x646, 0x648, 0x631, 0x648, 0x646, 0x62e, 0x627, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x6d0, 0x631, 0x646, 0x627, 0x646, 0x62f,
0x648, 0x2d, 0x646, 0x648, 0x631, 0x648, 0x646, 0x62e, 0x627, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x641, 0x6d0, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x648, 0x2d, 0x646, 0x648,
0x631, 0x648, 0x646, 0x62e, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x649, 0x645, 0x627, 0x644, 0x649,
0x64a, 0x20, 0x645, 0x627, 0x631, 0x649, 0x64a, 0x627, 0x646, 0x627, 0x20, 0x626,
0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x646, 0x648, 0x6cb, 0x648, 0x633, 0x649, 0x628, 0x649, 0x631, 0x633,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x648, 0x6cb, 0x648, 0x633,
0x649, 0x628, 0x649, 0x631, 0x633, 0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x648,
0x6cb, 0x648, 0x633, 0x649, 0x628, 0x649, 0x631, 0x633, 0x643, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648,
0x645, 0x633, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x645,
0x633, 0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x645, 0x633, 0x643, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x67e, 0x627, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x67e, 0x627, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x67e, 0x627, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e,
0x627, 0x644, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x627,
0x67e, 0x6c7, 0x626, 0x627, 0x20, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x6af, 0x649,
0x6cb, 0x649, 0x646, 0x6d0, 0x64a, 0x6d5, 0x633, 0x649, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6cb, 0x627, 0x64a,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6cb, 0x627, 0x64a,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x67e, 0x6d0, 0x631, 0x6c7, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e,
0x6d0, 0x631, 0x6c7, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x631, 0x6c7, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x641, 0x649, 0x644, 0x649, 0x67e, 0x67e, 0x649, 0x646, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x641, 0x649, 0x644, 0x649, 0x67e, 0x67e, 0x649, 0x646, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x641, 0x649, 0x644, 0x649, 0x67e, 0x67e, 0x649, 0x646, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641,
0x6d0, 0x646, 0x649, 0x643, 0x633, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644,
0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627, 0x64a,
0x646, 0x649, 0x62a, 0x20, 0x67e, 0x649, 0x626, 0x6d0, 0x631, 0x20, 0x6cb, 0x6d5,
0x20, 0x645, 0x649, 0x643, 0x6d0, 0x644, 0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x633, 0x627, 0x64a, 0x646, 0x649, 0x62a, 0x20, 0x67e, 0x649, 0x626,
0x6d0, 0x631, 0x20, 0x6cb, 0x6d5, 0x20, 0x645, 0x649, 0x643, 0x6d0, 0x644, 0x648,
0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627, 0x64a, 0x646, 0x649, 0x62a, 0x20,
0x67e, 0x649, 0x626, 0x6d0, 0x631, 0x20, 0x6cb, 0x6d5, 0x20, 0x645, 0x649, 0x643,
0x6d0, 0x644, 0x648, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x649, 0x62a, 0x643, 0x627, 0x64a, 0x631,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x648, 0x646, 0x67e, 0x6d0, 0x64a,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x642, 0x649, 0x632, 0x649, 0x644, 0x626,
0x648, 0x631, 0x62f, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x642, 0x649,
0x632, 0x649, 0x644, 0x626, 0x648, 0x631, 0x62f, 0x627, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x642, 0x649, 0x632, 0x649, 0x644, 0x626, 0x648, 0x631, 0x62f, 0x627, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x631,
0x6d0, 0x626, 0x648, 0x646, 0x649, 0x64a, 0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x631, 0x648, 0x62a, 0x6d0, 0x631, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x633, 0x627, 0x62e, 0x627, 0x631, 0x649, 0x646, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x633, 0x627, 0x62e, 0x627, 0x631, 0x649, 0x646, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x633, 0x627, 0x62e, 0x627, 0x631, 0x649, 0x646, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627,
0x645, 0x627, 0x631, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627,
0x645, 0x627, 0x631, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x627,
0x631, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x648, 0x626, 0x627, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x648, 0x626, 0x627, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x633, 0x6d5, 0x645, 0x6d5, 0x631, 0x642, 0x6d5, 0x646, 0x62a, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633,
0x6d0, 0x64a, 0x634, 0x6d0, 0x644, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633,
0x649, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x633, 0x648, 0x644, 0x648, 0x645, 0x648, 0x646, 0x20, 0x626, 0x627, 0x631,
0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x62c, 0x6d5, 0x646, 0x6c7, 0x628, 0x649, 0x64a, 0x20, 0x62c, 0x648, 0x631, 0x62c,
0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x6c7, 0x631,
0x649, 0x646, 0x627, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x648,
0x6cb, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x627, 0x64a, 0x62a,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x6d5, 0x64a, 0x628, 0x6d0,
0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x6d5, 0x64a, 0x628, 0x6d0,
0x64a, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x6d5, 0x64a, 0x628, 0x6d0, 0x64a, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x62a, 0x627, 0x62c, 0x649, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x62a, 0x648, 0x643, 0x6d0, 0x644, 0x627, 0x6cb, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686, 0x6c7, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x62a, 0x6c8, 0x631, 0x643, 0x645, 0x6d5, 0x646, 0x649,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x6c8,
0x631, 0x643, 0x645, 0x6d5, 0x646, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x62a, 0x6c8, 0x631, 0x643, 0x645, 0x6d5, 0x646, 0x649, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x62a, 0x6c7, 0x6cb, 0x627, 0x644, 0x6c7, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x6c7, 0x631, 0x6c7, 0x6af, 0x6cb, 0x627, 0x64a, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c7, 0x631, 0x6c7, 0x6af, 0x6cb, 0x627,
0x64a, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c7, 0x631, 0x6c7, 0x6af, 0x6cb, 0x627,
0x64a, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x6c6, 0x632, 0x628, 0x6d0, 0x643, 0x649, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c6, 0x632, 0x628, 0x6d0,
0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c6,
0x632, 0x628, 0x6d0, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x627,
0x646, 0x6c7, 0x626, 0x627, 0x62a, 0x6c7, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x6cb, 0x627, 0x646, 0x6c7, 0x626, 0x627, 0x62a, 0x6c7, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x6cb, 0x627, 0x646, 0x6c7, 0x626, 0x627, 0x62a, 0x6c7, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x6d0, 0x646,
0x6d0, 0x632, 0x6c7, 0x626, 0x6d0, 0x644, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x6cb, 0x649, 0x644, 0x627, 0x62f, 0x649, 0x6cb, 0x648, 0x633, 0x62a, 0x648,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x649, 0x644, 0x627, 0x62f,
0x649, 0x6cb, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb,
0x649, 0x644, 0x627, 0x62f, 0x649, 0x6cb, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x6cb, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x6cb, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627,
0x62f, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x6cb, 0x6d0, 0x64a, 0x643, 0x20, 0x626, 0x627, 0x631, 0x649, 0x644,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x627, 0x644, 0x644, 0x649,
0x633, 0x20, 0x6cb, 0x6d5, 0x20, 0x641, 0x6c7, 0x62a, 0x6c7, 0x646, 0x627, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6c7, 0x62a, 0x633, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6c7, 0x62a, 0x633,
0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6c7, 0x62a, 0x633, 0x643,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x64a, 0x6d0, 0x643, 0x627, 0x62a, 0x6d0, 0x631, 0x649, 0x646, 0x628, 0x6c7,
0x631, 0x6af, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x643, 0x627,
0x62a, 0x6d0, 0x631, 0x649, 0x646, 0x628, 0x6c7, 0x631, 0x6af, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x64a, 0x6d0, 0x643, 0x627, 0x62a, 0x6d0, 0x631, 0x649, 0x646, 0x628, 0x6c7,
0x631, 0x6af, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x41, 0x66, 0x67, 0x2bb, 0x6f, 0x6e, 0x69, 0x73, 0x74,
0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b,
0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4a, 0x61, 0x6e, 0x75, 0x62, 0x69, 0x79, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69,
0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62,
0x69, 0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x6c, 0x79,
0x61, 0x73, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x6c,
0x79, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x6c, 0x79, 0x61,
0x73, 0x6b, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x6b, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x6b,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x6b, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61,
0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69,
0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61,
0x72, 0x71, 0x69, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69,
0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x54, 0x6f, 0x67, 0x2018, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x20, 0x28, 0x41, 0x51, 0x53, 0x48, 0x29, 0x54, 0x6f, 0x67, 0x2018,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x20, 0x28, 0x41, 0x51, 0x53, 0x48, 0x29, 0x54, 0x6f,
0x67, 0x2018, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x20, 0x28, 0x41, 0x51, 0x53, 0x48, 0x29, 0x54, 0x69, 0x6e,
0x63, 0x68, 0x20, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f, 0x6b, 0x65,
0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20,
0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41,
0x70, 0x69, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x61, 0x75, 0x64, 0x69, 0x79, 0x61, 0x20, 0x41,
0x72, 0x61, 0x62, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x61, 0x75, 0x64, 0x69, 0x79, 0x61, 0x20, 0x41,
0x72, 0x61, 0x62, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x61, 0x75, 0x64, 0x69, 0x79, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62,
0x69, 0x73, 0x74, 0x6f, 0x6e, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69,
0x79, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x6d, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72,
0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41,
0x72, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72,
0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61,
0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b,
0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20,
0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x67,
0x2018, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x67, 0x2018, 0x61, 0x72,
0x62, 0x69, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a,
0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x20, 0x67, 0x2018, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61,
0x72, 0x71, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61,
0x72, 0x71, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69,
0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x47, 0x2018, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x47, 0x2018, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61,
0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4f, 0x7a, 0x61, 0x72, 0x62, 0x61, 0x79, 0x6a, 0x6f,
0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x7a, 0x61, 0x72, 0x62,
0x61, 0x79, 0x6a, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x7a, 0x61, 0x72,
0x62, 0x61, 0x79, 0x6a, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x6f,
0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x75, 0x74,
0x61, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x79,
0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x79, 0x2d, 0x44, 0x6f, 0x72, 0x75, 0x73, 0x73, 0x61,
0x6c, 0x6f, 0x6d, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x61, 0x62,
0x6f, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4b, 0x61, 0x62, 0x6f, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4b, 0x61, 0x62, 0x6f, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43,
0x68, 0x61, 0x74, 0x65, 0x6d, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43,
0x68, 0x61, 0x74, 0x65, 0x6d, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74,
0x65, 0x6d, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x69,
0x6c, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x58, 0x69, 0x74, 0x6f, 0x79, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x58, 0x69, 0x74, 0x6f, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58, 0x69, 0x74,
0x6f, 0x79, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x52, 0x6f, 0x6a, 0x64, 0x65, 0x73, 0x74, 0x76, 0x6f, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x75, 0x6b, 0x20, 0x6f,
0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4b, 0x75, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4b, 0x75, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x79, 0x61, 0x72, 0x69, 0x6d, 0x20, 0x79,
0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x75,
0x62, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x75, 0x62, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x44, 0x65, 0x79, 0x76, 0x69,
0x73, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x44, 0x79, 0x75, 0x6d, 0x6f,
0x6e, 0x2d, 0x64, 0x2019, 0x59, 0x75, 0x72, 0x76, 0x69, 0x6c, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50,
0x61, 0x73, 0x78, 0x61, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a,
0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79,
0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d,
0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x59,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69,
0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x61, 0x6c,
0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x20, 0x76, 0x61, 0x20,
0x4d, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47,
0x2018, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61, 0x72,
0x62, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x47, 0x2018, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x59, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e,
0x64, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x6f, 0x72, 0x6f,
0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69,
0x6a, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x75, 0x7a, 0x20, 0x47, 0x76,
0x69, 0x61, 0x6e, 0x61, 0x73, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x46, 0x72, 0x61, 0x6e, 0x73, 0x75, 0x7a, 0x20, 0x4a, 0x61, 0x6e, 0x75,
0x62, 0x69, 0x79, 0x20, 0x68, 0x75, 0x64, 0x75, 0x64, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x76, 0x61, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x47, 0x61, 0x6d, 0x62, 0x79, 0x65, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x47, 0x72, 0x75, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47,
0x72, 0x75, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x47, 0x72, 0x69, 0x6e, 0x76, 0x69, 0x63, 0x68,
0x20, 0x6f, 0x2018, 0x72, 0x74, 0x61, 0x63, 0x68, 0x61, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x47,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x47, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61,
0x72, 0x62, 0x69, 0x79, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018,
0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61, 0x72,
0x62, 0x69, 0x79, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x46, 0x6f, 0x72, 0x73, 0x20, 0x6b, 0x6f, 0x2018, 0x72,
0x66, 0x61, 0x7a, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x79, 0x61, 0x6e,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x76, 0x61, 0x79,
0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x47, 0x61, 0x76, 0x61, 0x79, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x76, 0x61, 0x79, 0x69, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x79,
0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58, 0x6f,
0x76, 0x64, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58, 0x6f, 0x76, 0x64,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x58, 0x6f, 0x76, 0x64, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x48, 0x69, 0x6e, 0x64, 0x69,
0x73, 0x74, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x48, 0x69, 0x6e, 0x64, 0x20,
0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x48, 0x69, 0x6e, 0x64, 0x69, 0x78, 0x69, 0x74, 0x6f, 0x79, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x45, 0x72, 0x6f, 0x6e, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x45, 0x72, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x45, 0x72,
0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x79, 0x6f, 0x7a,
0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x49, 0x73, 0x72, 0x6f,
0x69, 0x6c, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x49, 0x73, 0x72, 0x6f,
0x69, 0x6c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x49, 0x73, 0x72, 0x6f, 0x69, 0x6c, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59,
0x61, 0x70, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x59, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x59, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a,
0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x51, 0x6f, 0x7a, 0x6f,
0x67, 0x2bb, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x51, 0x6f, 0x7a,
0x6f, 0x67, 0x2bb, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x51, 0x6f,
0x7a, 0x6f, 0x67, 0x2bb, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4b, 0x6f, 0x72, 0x65, 0x79, 0x61, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4b, 0x6f, 0x72, 0x65, 0x79, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4b, 0x6f, 0x72, 0x65, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x51, 0x69, 0x72, 0x67, 0x2bb, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x6f, 0x6e,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4c, 0x61, 0x79, 0x6e, 0x20, 0x6f,
0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x58, 0x61, 0x75, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x58, 0x61, 0x75, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x58, 0x61, 0x75, 0x20, 0x79,
0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x7a, 0x69, 0x79,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x6f,
0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x79, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x79, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x79, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x6f, 0x75,
0x73, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f,
0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x63, 0x68,
0x20, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20,
0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x6c,
0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55,
0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x6f, 0x73,
0x6b, 0x76, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x6f, 0x73,
0x6b, 0x76, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e,
0x65, 0x70, 0x61, 0x6c, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61,
0x6e, 0x67, 0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6e, 0x67,
0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6e, 0x67, 0x69,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61,
0x6e, 0x67, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79,
0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4e, 0x79, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e,
0x64, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x79, 0x75, 0x66, 0x61,
0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x79,
0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x79,
0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x69,
0x75, 0x79, 0x65, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x6d, 0x73,
0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x6f, 0x6b, 0x69, 0x73, 0x74,
0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x6f, 0x6b, 0x69,
0x73, 0x74, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x6f, 0x6b, 0x69, 0x73,
0x74, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x59, 0x61, 0x6e, 0x67,
0x69, 0x20, 0x47, 0x76, 0x69, 0x6e, 0x65, 0x79, 0x61, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x79, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61,
0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x79,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x50, 0x65, 0x72, 0x75, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x65,
0x72, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x65, 0x6e, 0x2d, 0x50, 0x79, 0x65, 0x72, 0x20,
0x76, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x53, 0x65, 0x6e, 0x2d, 0x50, 0x79, 0x65, 0x72,
0x20, 0x76, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x65, 0x6e, 0x2d, 0x50, 0x79, 0x65, 0x72, 0x20, 0x76,
0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x69, 0x74,
0x6b, 0x65, 0x72, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x6f,
0x6e, 0x61, 0x70, 0x65, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x78,
0x65, 0x6e, 0x79, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x52,
0x65, 0x79, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x53, 0x61, 0x78, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x61, 0x78, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x61, 0x78, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x65, 0x79, 0x73, 0x68,
0x65, 0x6c, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75,
0x72, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x6f, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4a, 0x61, 0x6e, 0x75, 0x62, 0x69, 0x79,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x53, 0x79, 0x6f, 0x76, 0x61, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x54, 0x61, 0x69, 0x74, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x54, 0x61, 0x79, 0x76, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x54, 0x61, 0x79, 0x76, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54,
0x61, 0x79, 0x76, 0x61, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x6f, 0x6a, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x79,
0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73,
0x74, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61,
0x79, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x72, 0x75, 0x67, 0x76,
0x61, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x79,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4f, 0x2018, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x2018, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x2018, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x56, 0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x65, 0x79,
0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x55, 0x6f, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x76, 0x61, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x648, 0x642, 0x62a, 0x6cc, 0x410, 0x444, 0x493, 0x43e, 0x43d, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430,
0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438,
0x439, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x416, 0x430, 0x43d, 0x443, 0x431, 0x438, 0x439, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430,
0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440,
0x431, 0x438, 0x439, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x451,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x43b, 0x44f,
0x441, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x43b, 0x44f,
0x441, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430,
0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428,
0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437,
0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x43c, 0x430, 0x440,
0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e,
0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x448, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20,
0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x43e, 0x493, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x43e, 0x493, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x43e, 0x493, 0x20, 0x43a, 0x443, 0x43d,
0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428,
0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x442, 0x438, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e,
0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x442, 0x438, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x438, 0x43d, 0x447, 0x20, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x438, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x430, 0x431,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410,
0x440, 0x430, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410,
0x440, 0x430, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x43a, 0x443, 0x43d,
0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430,
0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431,
0x438, 0x439, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x430, 0x43c, 0x43d, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440,
0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410,
0x440, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x43a, 0x443, 0x43d,
0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c,
0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c,
0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a,
0x430, 0x437, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438,
0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x492, 0x430, 0x440, 0x431, 0x438,
0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x492,
0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b,
0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431,
0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41e, 0x437, 0x430, 0x440,
0x431, 0x430, 0x439, 0x436, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41e, 0x437, 0x430, 0x440, 0x431, 0x430, 0x439, 0x436, 0x43e, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41e, 0x437, 0x430, 0x440, 0x431, 0x430, 0x439, 0x436, 0x43e, 0x43d, 0x20,
0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x437,
0x43e, 0x440, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x437, 0x43e, 0x440,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x44f, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438,
0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x411, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x430, 0x440, 0x443,
0x441, 0x441, 0x430, 0x43b, 0x43e, 0x43c, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435,
0x440, 0x434, 0x435, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41a, 0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x427, 0x430, 0x442, 0x445, 0x430, 0x43c, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x427, 0x430, 0x442, 0x445, 0x430, 0x43c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x427, 0x430, 0x442, 0x445, 0x430, 0x43c, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427, 0x438, 0x43b,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427, 0x438, 0x43b, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x425, 0x438, 0x442, 0x43e, 0x439, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x425, 0x438, 0x442, 0x43e, 0x439, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x425, 0x438, 0x442, 0x43e, 0x439, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x420, 0x43e, 0x436, 0x434,
0x435, 0x441, 0x442, 0x432, 0x43e, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x43e,
0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x451,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x443, 0x43a,
0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41a, 0x443, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b,
0x430, 0x440, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x443, 0x43a, 0x20, 0x43e, 0x440,
0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x44f, 0x440, 0x438, 0x43c, 0x20,
0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x443,
0x431, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x443, 0x431, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x43a, 0x443, 0x43d, 0x434,
0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x414, 0x44d,
0x432, 0x438, 0x441, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x414, 0x443, 0x43c,
0x43e, 0x43d, 0x442, 0x2d, 0x434, 0x2d, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20,
0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f,
0x430, 0x441, 0x445, 0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x20, 0x43e, 0x440,
0x43e, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x20,
0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438,
0x439, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a,
0x430, 0x437, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430,
0x440, 0x49b, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440,
0x49b, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x451,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440,
0x431, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431,
0x438, 0x439, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x43e, 0x43b, 0x44c,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430,
0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x43e, 0x43b, 0x44c,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430,
0x440, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x43e, 0x43b, 0x44c, 0x43a, 0x43b, 0x435,
0x43d, 0x434, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20,
0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x438,
0x436, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x438, 0x436, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x424, 0x438, 0x436, 0x438, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443,
0x437, 0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x441, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x20,
0x436, 0x430, 0x43d, 0x443, 0x431, 0x438, 0x439, 0x20, 0x432, 0x430, 0x20, 0x410,
0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x435,
0x440, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x440, 0x443, 0x437, 0x438,
0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x440, 0x443, 0x437, 0x438,
0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x451,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x438, 0x43b,
0x431, 0x435, 0x440, 0x442, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x440, 0x438, 0x43d, 0x432,
0x438, 0x447, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b,
0x438, 0x439, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438,
0x439, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438,
0x439, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x45e, 0x440, 0x444, 0x430, 0x437,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x430, 0x432, 0x430, 0x439, 0x438,
0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x413, 0x430, 0x432, 0x430, 0x439, 0x438, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x413, 0x430, 0x432, 0x430, 0x439, 0x438, 0x2d, 0x430, 0x43b,
0x435, 0x443, 0x442, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d,
0x433, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x43e, 0x43d, 0x43a, 0x43e,
0x43d, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x425,
0x43e, 0x432, 0x434, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x425, 0x43e, 0x432,
0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x4b2, 0x438, 0x43d, 0x434, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x4b2, 0x438,
0x43d, 0x434, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x4b2, 0x438, 0x43d, 0x434, 0x2d, 0x425, 0x438, 0x442, 0x43e,
0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430,
0x437, 0x438, 0x439, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438,
0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438,
0x439, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20,
0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x42d, 0x440, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x42d, 0x440, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42d, 0x440, 0x43e, 0x43d,
0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x418, 0x441, 0x440, 0x43e,
0x438, 0x43b, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x418, 0x441, 0x440, 0x43e,
0x438, 0x43b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x418, 0x441, 0x440, 0x43e, 0x438, 0x43b, 0x20,
0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f,
0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b,
0x438, 0x439, 0x20, 0x49a, 0x43e, 0x437, 0x43e, 0x493, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438,
0x439, 0x20, 0x49a, 0x43e, 0x437, 0x43e, 0x493, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x441,
0x440, 0x430, 0x435, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x49a,
0x438, 0x440, 0x493, 0x438, 0x437, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41b, 0x430, 0x439, 0x43d, 0x20, 0x43e, 0x440, 0x43e,
0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x435, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x435, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x435, 0x20,
0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x43b, 0x430, 0x439,
0x437, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x43b,
0x44c, 0x434, 0x438, 0x432, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440,
0x41c, 0x430, 0x440, 0x43a, 0x435, 0x437, 0x430, 0x441, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x20, 0x43e, 0x440,
0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x43e, 0x443, 0x432,
0x441, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x43b, 0x430,
0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x43e, 0x441, 0x43a,
0x432, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f,
0x43d, 0x433, 0x438, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438,
0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d, 0x433, 0x438, 0x20,
0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x42f, 0x43d, 0x433, 0x438, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x42f, 0x43d, 0x433, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d, 0x433, 0x438,
0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x42f, 0x43d, 0x433, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x44c,
0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d,
0x434, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41d, 0x438, 0x443, 0x435, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x43e, 0x440,
0x43e, 0x43b, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x43e, 0x440, 0x43e,
0x43b, 0x438, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435,
0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434,
0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41e,
0x43c, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x451,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x43e, 0x43a,
0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f,
0x43e, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x43e,
0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x2d, 0x42f, 0x43d,
0x433, 0x438, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435, 0x44f, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430,
0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f,
0x435, 0x440, 0x443, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x435, 0x440,
0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x438, 0x43b, 0x438, 0x43f,
0x43f, 0x438, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x438, 0x43b, 0x438,
0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x20, 0x43e, 0x440,
0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x432, 0x430,
0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20,
0x432, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x432,
0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x43a, 0x443,
0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x430, 0x43c, 0x43e, 0x430,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x43a, 0x443, 0x43d, 0x434,
0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x435,
0x439, 0x448, 0x435, 0x43b, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x438, 0x43d, 0x433, 0x430,
0x43f, 0x443, 0x440, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x43e, 0x43b,
0x43e, 0x43c, 0x43e, 0x43d, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x416, 0x430, 0x43d, 0x443, 0x431,
0x438, 0x439, 0x20, 0x414, 0x436, 0x43e, 0x440, 0x434, 0x436, 0x438, 0x44f, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x44c, 0x43e, 0x432, 0x430, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x430, 0x438, 0x442, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x43a, 0x443, 0x43d, 0x434,
0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x43e,
0x436, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x43e,
0x43d, 0x433, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x427, 0x443, 0x443, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x430,
0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x443, 0x432, 0x430,
0x43b, 0x443, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x440, 0x443, 0x433,
0x432, 0x430, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x439, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x40e, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x40e, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x40e, 0x437, 0x431, 0x435, 0x43a, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x435, 0x43d, 0x435,
0x441, 0x443, 0x44d, 0x43b, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412,
0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x423, 0x44d, 0x439, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x44d, 0x43b, 0x43b, 0x438,
0x441, 0x20, 0x432, 0x430, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x415, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440,
0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d,
0x431, 0x443, 0x440, 0x433, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x63, 0x72, 0x65, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x63, 0x72,
0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x41, 0x63, 0x72, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x54,
0x72, 0x75, 0x6e, 0x67, 0x20, 0x50, 0x68, 0x69, 0x47, 0x69, 0x1edd, 0x20,
0x110, 0xf4, 0x6e, 0x67, 0x20, 0x50, 0x68, 0x69, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x61, 0x6d, 0x20, 0x50, 0x68,
0x69, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x50, 0x68, 0x69,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0xe2,
0x79, 0x20, 0x50, 0x68, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x50, 0x68, 0x69, 0x47,
0x69, 0x1edd, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x47, 0x69,
0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e,
0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9,
0x61, 0x20, 0x68, 0xe8, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72,
0x75, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20,
0x110, 0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x47,
0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x6d, 0x69,
0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d,
0x69, 0x1ec1, 0x6e, 0x20, 0x6e, 0xfa, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x63,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x6e, 0xfa,
0x69, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20,
0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x6e, 0xfa, 0x69, 0x47, 0x69, 0x1edd, 0x20,
0x54, 0x68, 0xe1, 0x69, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x44, 0x1b0,
0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x54, 0x68, 0xe1, 0x69, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x44,
0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20,
0x68, 0xe8, 0x20, 0x54, 0x68, 0xe1, 0x69, 0x20, 0x42, 0xec, 0x6e, 0x68,
0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x47, 0x69, 0x1edd, 0x20,
0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x70, 0x69, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x70, 0x69, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x70,
0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x71, 0x74,
0x61, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x71,
0x74, 0x6f, 0x62, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62,
0x65, 0x47, 0x69, 0x1edd, 0x20, 0x1ea2, 0x20, 0x52, 0x1ead, 0x70, 0x47, 0x69,
0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x1ea2, 0x20, 0x52, 0x1ead,
0x70, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x1ea2, 0x20, 0x52, 0x1ead, 0x70, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x74, 0xe2, 0x79, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x63,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x74, 0xe2,
0x79, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x6d, 0x69,
0x1ec1, 0x6e, 0x20, 0x74, 0xe2, 0x79, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0x1ea1, 0x69, 0x20, 0x54, 0xe2,
0x79, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x110, 0x1ea1, 0x69, 0x20, 0x54, 0xe2, 0x79,
0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9,
0x61, 0x20, 0x68, 0xe8, 0x20, 0x110, 0x1ea1, 0x69, 0x20, 0x54, 0xe2, 0x79,
0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69,
0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75,
0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d,
0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d,
0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x54, 0xe2,
0x79, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x69, 0x1ec1,
0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x54, 0xe2, 0x79, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e,
0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4,
0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d,
0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69,
0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20,
0x54, 0xe2, 0x79, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x47, 0x69,
0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x47, 0x69,
0x1edd, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x61, 0x70,
0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d,
0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x47,
0x69, 0x1edd, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x51, 0x75, 0x1ed1,
0x63, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54,
0x72, 0x75, 0x6e, 0x67, 0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x54, 0x72, 0x75, 0x6e,
0x67, 0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0x1ea3,
0x6f, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x47,
0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20,
0x43, 0x6f, 0x63, 0x6f, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x75,
0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x51, 0x75, 0x1ea7,
0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x47, 0x69,
0x1edd, 0x20, 0x4e, 0x1eed, 0x61, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x75, 0x62, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x43, 0x75, 0x62, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43,
0x75, 0x62, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73,
0x47, 0x69, 0x1edd, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x110, 0xf4, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x47, 0x69,
0x1edd, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x50, 0x68, 0x1ee5, 0x63, 0x20, 0x53,
0x69, 0x6e, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x50, 0x68, 0x1ee5, 0x63, 0x20, 0x53, 0x69,
0x6e, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x50, 0x68, 0x1ee5, 0x63, 0x20, 0x53, 0x69,
0x6e, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x72, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0xc2,
0x75, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54,
0x72, 0x75, 0x6e, 0x67, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x6d,
0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20,
0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0xc2,
0x75, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x110,
0xf4, 0x6e, 0x67, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9,
0x61, 0x20, 0x68, 0xe8, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0xc2, 0x75,
0x47, 0x69, 0x1edd, 0x20, 0x56, 0x69, 0x1ec5, 0x6e, 0x20, 0x111, 0xf4, 0x6e,
0x67, 0x20, 0x43, 0x68, 0xe2, 0x75, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd,
0x20, 0x54, 0xe2, 0x79, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0xc2, 0x75, 0x47,
0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x54, 0xe2,
0x79, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e,
0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x51,
0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20,
0x48, 0xe8, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x47, 0x69, 0x1edd,
0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x68, 0x75, 0x1ed9,
0x63, 0x20, 0x50, 0x68, 0xe1, 0x70, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x61,
0x6d, 0x20, 0x43, 0x1ef1, 0x63, 0x20, 0x76, 0xe0, 0x20, 0x4e, 0x61, 0x6d,
0x20, 0x4e, 0x1b0, 0x1edb, 0x63, 0x20, 0x50, 0x68, 0xe1, 0x70, 0x47, 0x69,
0x1edd, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x47,
0x69, 0x1edd, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x47, 0x69,
0x1edd, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x72, 0x75, 0x6e,
0x67, 0x20, 0x62, 0xec, 0x6e, 0x68, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e,
0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20,
0x110, 0xf4, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54,
0xe2, 0x79, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x69,
0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20,
0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x56, 0xf9, 0x6e,
0x67, 0x20, 0x56, 0x1ecb, 0x6e, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x47, 0x69, 0x1edd, 0x20, 0x48, 0x1ed3, 0x6e,
0x67, 0x20, 0x4b, 0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x48, 0x1ed3, 0x6e, 0x67, 0x20, 0x4b, 0xf4, 0x6e,
0x67, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x48, 0x1ed3, 0x6e, 0x67, 0x20, 0x4b, 0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d,
0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x1ea4, 0x6e, 0x20, 0x110,
0x1ed9, 0x47, 0x69, 0x1edd, 0x20, 0x1ea4, 0x6e, 0x20, 0x110, 0x1ed9, 0x20, 0x44,
0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0xf4, 0x6e, 0x67,
0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69,
0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69,
0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1,
0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20,
0x48, 0xe8, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x47, 0x69, 0x1edd,
0x20, 0x4e, 0x68, 0x1ead, 0x74, 0x20, 0x42, 0x1ea3, 0x6e, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x68, 0x1ead, 0x74, 0x20,
0x42, 0x1ea3, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x4e, 0x68, 0x1ead, 0x74, 0x20, 0x42, 0x1ea3, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x47,
0x69, 0x1edd, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4,
0x6e, 0x67, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2,
0x79, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x48, 0xe0, 0x6e, 0x20, 0x51, 0x75, 0x1ed1, 0x63,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x48, 0xe0,
0x6e, 0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x48, 0xe0, 0x6e, 0x20, 0x51, 0x75, 0x1ed1,
0x63, 0x47, 0x69, 0x1edd, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x47,
0x69, 0x1edd, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x47, 0x69, 0x1edd,
0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x51,
0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4c, 0x69, 0x6e, 0x65,
0x47, 0x69, 0x1edd, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x61, 0x20, 0x43,
0x61, 0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x4d, 0x61, 0x20, 0x43, 0x61, 0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d, 0x61, 0x20, 0x43, 0x61, 0x6f, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68,
0xe8, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x47, 0x69,
0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x47, 0x69, 0x1edd, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x47, 0x69, 0x1edd,
0x20, 0x54, 0x68, 0xe1, 0x69, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x44,
0x1b0, 0x1a1, 0x6e, 0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x68, 0xe1,
0x69, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67,
0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x4d,
0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x54, 0x68, 0xe1, 0x69, 0x20, 0x42,
0xec, 0x6e, 0x68, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x20, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0x61, 0x74, 0x78, 0x63, 0x1a1, 0x76, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x61, 0x74,
0x78, 0x63, 0x1a1, 0x76, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x4d, 0x61, 0x74, 0x78, 0x63, 0x1a1, 0x76, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x47,
0x69, 0x1edd, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x47, 0x69, 0x1edd, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20,
0x4e, 0x69, 0x75, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0x1ea3, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20,
0x48, 0xe8, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f,
0x20, 0x42, 0x1eaf, 0x63, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x47, 0x69,
0x1edd, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20,
0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x47,
0x69, 0x1edd, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x47, 0x69, 0x1edd,
0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47,
0x75, 0x69, 0x6e, 0x65, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x65,
0x72, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x50, 0x65, 0x72, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20,
0x48, 0xe8, 0x20, 0x50, 0x65, 0x72, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x50,
0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x47,
0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x47, 0x69, 0x1edd, 0x20, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x76, 0xe0,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x76, 0xe0, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x76, 0xe0, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x4e,
0x68, 0x1b0, 0x1ee1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x62, 0x61, 0x6e, 0x20, 0x6e,
0x67, 0xe0, 0x79, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x47,
0x69, 0x1edd, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65,
0x47, 0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f,
0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x4e, 0x61, 0x6d, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x47,
0x69, 0x1edd, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0xe0,
0x69, 0x20, 0x42, 0x1eaf, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x110, 0xe0, 0x69, 0x20, 0x42, 0x1eaf, 0x63, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x110, 0xe0, 0x69,
0x20, 0x42, 0x1eaf, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x61, 0x6a, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x47, 0x69, 0x1edd, 0x20,
0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x47, 0x69, 0x1edd,
0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x47,
0x69, 0x1edd, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x47, 0x69, 0x1edd,
0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x47, 0x69, 0x1edd,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x76, 0xe0, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x47, 0x69, 0x1edd,
0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d,
0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x59,
0x75, 0x6b, 0x6f, 0x6e, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x161,
0x69, 0x20, 0x5a, 0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x161, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a,
0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x161, 0x69, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x74, 0x4d, 0x69, 0x74,
0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69,
0x20, 0x5a, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65,
0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x74, 0x4f, 0x161, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x5a, 0x69, 0x74,
0x4f, 0x161, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x74,
0x4f, 0x161, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x74, 0x57, 0x65,
0x161, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20,
0x5a, 0x69, 0x74, 0x57, 0x65, 0x161, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x69, 0x161, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x7a, 0x69, 0x74, 0x57, 0x65, 0x161, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x69, 0x74, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x66, 0x67,
0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68,
0x20, 0x41, 0x66, 0x66, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x41, 0x66,
0x66, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x44, 0x65, 0x20, 0x41, 0x66,
0x66, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47,
0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x66,
0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65,
0x77, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x66, 0x72, 0x69, 0x63, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72,
0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x66, 0x72, 0x69,
0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68,
0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62,
0x61, 0x72, 0x74, 0x68, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62,
0x61, 0x72, 0x74, 0x68, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x47, 0x6f,
0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x47,
0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x47, 0x6f, 0x67, 0x6c,
0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d, 0x79, 0x6e, 0x79, 0x64, 0x64, 0x6f,
0x65, 0x64, 0x64, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4d, 0x79, 0x6e,
0x79, 0x64, 0x64, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x47, 0x6f, 0x67, 0x6c,
0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4d, 0x79, 0x6e,
0x79, 0x64, 0x64, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x47, 0x6f, 0x67, 0x6c,
0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72,
0x20, 0x54, 0x61, 0x77, 0x65, 0x6c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65,
0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x43, 0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x77, 0x65,
0x6c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x43, 0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x54,
0x61, 0x77, 0x65, 0x6c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x70, 0x69, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x70,
0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x72, 0x61, 0x62,
0x61, 0x69, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x69,
0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x41, 0x72, 0x61, 0x62, 0x61, 0x69, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x79, 0x72, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e, 0x69,
0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x72, 0x69,
0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47,
0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x69,
0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c,
0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e, 0x69,
0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47,
0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x69,
0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x43, 0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x79, 0x72,
0x20, 0x49, 0x77, 0x65, 0x72, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x65,
0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x79, 0x72, 0x20, 0x49, 0x77, 0x65,
0x72, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x43, 0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x79, 0x72,
0x20, 0x49, 0x77, 0x65, 0x72, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68,
0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x41,
0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62,
0x61, 0x72, 0x74, 0x68, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61, 0x6e, 0x6f,
0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c,
0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72,
0x74, 0x68, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e,
0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f,
0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c,
0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77, 0x79, 0x72,
0x61, 0x69, 0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x41,
0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69,
0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77,
0x69, 0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20,
0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c,
0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x73, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x73, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a,
0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x79, 0x72, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x79, 0x72, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x79, 0x72, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x66, 0x69, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c,
0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x42, 0x72,
0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43,
0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x54, 0x73, 0x69, 0x65, 0x69, 0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x54, 0x73,
0x69, 0x65, 0x69, 0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x54, 0x73, 0x69, 0x65, 0x69, 0x6e, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x59, 0x20, 0x4e,
0x61, 0x64, 0x6f, 0x6c, 0x69, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x43, 0x6f, 0x63,
0x6f, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64,
0x64, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x6e, 0x6e, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x59,
0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x69, 0x77, 0x62, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x43, 0x69, 0x77, 0x62, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x43, 0x69, 0x77, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44,
0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79,
0x20, 0x50, 0x61, 0x73, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20,
0x79, 0x20, 0x50, 0x61, 0x73, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x50,
0x61, 0x73, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x45, 0x63, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61,
0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x45, 0x77, 0x72,
0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72,
0x74, 0x68, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62,
0x61, 0x72, 0x74, 0x68, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20,
0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61,
0x69, 0x6e, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69,
0x6e, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x6c,
0x6c, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x45,
0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65,
0x77, 0x69, 0x6e, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c,
0x65, 0x77, 0x69, 0x6e, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x2f, 0x4d,
0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x6e, 0x79,
0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x2f, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x59, 0x6e,
0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x2f, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61,
0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x46, 0x69, 0x6a, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x65, 0x20, 0x46, 0x66, 0x72, 0x65,
0x6e, 0x67, 0x69, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x69,
0x72, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x46, 0x66, 0x72, 0x61, 0x69, 0x6e,
0x63, 0x20, 0x79, 0x6e, 0x20, 0x79, 0x20, 0x44, 0x65, 0x20, 0x61, 0x2019,
0x72, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x67, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77, 0x79,
0x72, 0x61, 0x69, 0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73,
0x20, 0x4c, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69,
0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4c, 0x61,
0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x44,
0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e,
0x79, 0x73, 0x20, 0x4c, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x79, 0x72,
0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4c, 0x61, 0x73, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x47,
0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x79, 0x72, 0x20,
0x59, 0x6e, 0x79, 0x73, 0x20, 0x4c, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65,
0x77, 0x69, 0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20,
0x4c, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x79, 0x20, 0x47, 0x77, 0x6c, 0x66, 0x66,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x2d, 0x54,
0x73, 0x69, 0x65, 0x69, 0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x49, 0x72,
0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x6f, 0x72,
0x65, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x43, 0x6f, 0x72, 0x65, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x74, 0x61, 0x6e,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65,
0x64, 0x64, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x79, 0x72, 0x20, 0x41, 0x72, 0x67, 0x6c, 0x77, 0x79, 0x64, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x79, 0x72, 0x20, 0x41, 0x72,
0x67, 0x6c, 0x77, 0x79, 0x64, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x79, 0x72, 0x20,
0x41, 0x72, 0x67, 0x6c, 0x77, 0x79, 0x64, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d, 0x61,
0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x59, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x69, 0x73, 0x65,
0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f,
0x65, 0x64, 0x64, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x66, 0x69, 0x67, 0x20,
0x4d, 0x65, 0x63, 0x73, 0x69, 0x63, 0x6f, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x66, 0x69, 0x67, 0x20, 0x4d, 0x65, 0x63, 0x73, 0x69, 0x63,
0x6f, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x66, 0x69, 0x67, 0x20, 0x4d, 0x65, 0x63, 0x73,
0x69, 0x63, 0x6f, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x48, 0x61, 0x66, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x66, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x66, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x66, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4e, 0x61,
0x75, 0x72, 0x75, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4e, 0x65, 0x77,
0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4e,
0x65, 0x77, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e,
0x65, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x65, 0x72,
0x69, 0x77, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x50, 0x65, 0x72, 0x69, 0x77, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x50, 0x65, 0x72, 0x69, 0x77,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x6f, 0x68,
0x6e, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x79,
0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x65, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x53, 0x6f,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44,
0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x63, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x79, 0x72, 0x63, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x54, 0x79, 0x72, 0x63, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x48, 0x61, 0x66, 0x20, 0x54, 0x79, 0x72, 0x63, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65,
0x6c, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x48, 0x61, 0x66, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x627, 0x648, 0x6af, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x648, 0x62e, 0x62a, 0x631, 0x648, 0x686, 0x200c, 0x62f, 0x631, 0x627, 0x62a, 0x6cc,
0x646, 0x20, 0x642, 0x632, 0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x648, 0x62e, 0x62a, 0x631, 0x648, 0x686, 0x200c, 0x6a9, 0x67e, 0x62a, 0x6cc, 0x646,
0x20, 0x642, 0x632, 0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648,
0x62e, 0x62a, 0x642, 0x6cc, 0x631, 0x63a, 0x6cc, 0x632, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627, 0x62c, 0x6cc, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x41, 0x63, 0x72, 0x65, 0x2d,
0x74, 0x69, 0x69, 0x64, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x63, 0x72,
0x65, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x41, 0x66, 0x67, 0x68, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x53, 0xfb, 0x64, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73,
0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x57,
0x65, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x41, 0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41,
0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x2d, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x65, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x2d, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x74,
0x69, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61,
0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41,
0x71, 0x74, 0x61, 0x75, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x71, 0x74,
0x61, 0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62,
0x65, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x61, 0x62, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x61, 0x62, 0x79,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x61, 0x62, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x79, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65,
0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x79, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x65,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x74, 0x69, 0x69, 0x64,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x69,
0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x77, 0x65,
0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x77, 0x65, 0x73, 0x74,
0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65,
0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b,
0x65, 0x20, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x45,
0x61, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65,
0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x42, 0x68, 0x75,
0x74, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x42, 0x72, 0x61, 0x7a, 0x79, 0x6c, 0x6a,
0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x42,
0x72, 0x61, 0x7a, 0x79, 0x6c, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x42, 0x72, 0x61, 0x7a, 0x79, 0x6c, 0x6a, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x73, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x76,
0x65, 0x72, 0x64, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x61, 0x61, 0x70,
0x76, 0x65, 0x72, 0x64, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x20, 0x74, 0x69, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69,
0x6c, 0x65, 0x65, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x53, 0x69, 0x6c, 0x65, 0x65, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69,
0x6c, 0x65, 0x65, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x65, 0x65, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x65, 0x65,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x65, 0x65, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4b,
0x72, 0x79, 0x73, 0x74, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x65, 0x69,
0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4b,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65,
0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x43,
0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x43,
0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x65, 0x20, 0x68,
0x61, 0x6c, 0x76, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x4b, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x4b, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x44,
0x61, 0x76, 0x69, 0x73, 0x20, 0x74, 0x69, 0x69, 0x64, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x54,
0x69, 0x6d, 0x6f, 0x72, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x50, 0x65, 0x61, 0x73, 0x6b, 0x65, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x61, 0x73, 0x6b,
0x65, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65,
0x61, 0x73, 0x6b, 0x65, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x45,
0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x69, 0x64,
0x64, 0x65, 0x6e, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x65, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73,
0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x65, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x46, 0x69, 0x65, 0x72, 0x64, 0x65, 0x72, 0x2d, 0x65,
0x61, 0x73, 0x74, 0x6c, 0x69, 0x6b, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65,
0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65,
0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0xe2, 0x6e, 0x65, 0x69, 0x6c, 0xe2, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0xe2, 0x6e, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0xe2, 0x6e, 0x65, 0x69, 0x6c, 0xe2, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x46, 0x69, 0x6a, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x46, 0x69, 0x6a, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x46, 0x69,
0x6a, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x46, 0x72, 0xe2, 0x6e, 0x73, 0x2d, 0x47, 0x75,
0x79, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x46, 0x72, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0xfa, 0x64, 0x6c,
0x69, 0x6b, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72,
0x63, 0x74, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0xe2,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x65, 0x69, 0x6c, 0xe2,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73,
0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x45,
0x61, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57,
0x65, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x47, 0x75, 0x61, 0x6d, 0x65,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x47, 0x6f, 0x6c, 0x66, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x79, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x79, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65,
0x74, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f,
0x6e, 0x67, 0x73, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x74, 0x69, 0x69, 0x64,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x59, 0x6e, 0x64, 0x69,
0x61, 0x61, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x59, 0x6e,
0x64, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x61, 0x6e,
0x2d, 0x74, 0x69, 0x69, 0x64, 0x59, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69,
0x6e, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x59, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x7a, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61,
0x73, 0x74, 0x2d, 0x59, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d,
0x59, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x49, 0x72, 0x6b,
0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x49, 0x72,
0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65,
0x74, 0x73, 0x6b, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x79,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74,
0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x74, 0x69, 0x69, 0x64,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x2d, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x4b, 0x61,
0x7a, 0x61, 0x63, 0x68, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57,
0x65, 0x73, 0x74, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x73, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x7a, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4c, 0x61,
0x6e, 0x6b, 0x61, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4c, 0x69, 0x6e, 0x65,
0x2d, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d,
0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65,
0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x63,
0x61, 0x75, 0x73, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x6c, 0x65, 0x69, 0x73, 0x79,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x65, 0x69, 0x6c, 0xe2,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x74, 0x69, 0x69,
0x64, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x74, 0x69, 0x69, 0x64, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f, 0x73,
0x6b, 0x6f, 0x75, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b,
0x6f, 0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x4e, 0x69, 0x6a, 0x2d, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x4e, 0x69, 0x6a, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x4e, 0x69, 0x6a, 0x2d, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x69, 0x6a, 0x2d, 0x53, 0x65,
0x65, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x4e, 0x69, 0x6a, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x4e, 0x69, 0x6a, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0xe2, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0xe2,
0x6e, 0x73, 0x6b, 0x65, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0xe2, 0x6e, 0x73,
0x6b, 0x65, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x4e, 0x69, 0x75, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x61, 0x72, 0x64, 0x6c, 0x69, 0x6b,
0x65, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d,
0x74, 0x69, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4f, 0x6d, 0x73,
0x6b, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x42, 0x65, 0x6c, 0x61, 0x75, 0x73, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61, 0x70, 0x6f, 0x65, 0x61, 0x2d,
0x4e, 0x69, 0x6a, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x65, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75,
0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6a, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x65, 0x69,
0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d,
0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0x65, 0x69, 0x6c, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x20, 0x74, 0x69,
0x69, 0x64, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x2d,
0x74, 0x69, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x74, 0x69,
0x69, 0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x74,
0x69, 0x69, 0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c,
0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x73, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x53, 0xfb, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x61, 0x6d, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x74, 0x69, 0x69, 0x64,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20,
0x74, 0x69, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x54, 0x61, 0x64, 0x7a, 0x6a, 0x69, 0x65, 0x6b,
0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x2d, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x65, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x46, 0x65, 0x6e, 0x65, 0x7a,
0x6f, 0x6c, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x2d, 0x74, 0x69, 0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x2d, 0x74, 0x69, 0x69, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x61,
0x6b, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65,
0x6e, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x73, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d,
0x74, 0x69, 0x69, 0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d,
0x74, 0x69, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x59, 0x75,
0x6b, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75,
0x65, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20,
0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x6a, 0x61, 0x6e, 0x74, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x41, 0x66,
0x72, 0x69, 0x71, 0x75, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x73,
0x6f, 0x77, 0x77, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65,
0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x64, 0x75, 0x20, 0x73,
0x6f, 0x77, 0x77, 0x75, 0x20, 0x6a, 0x61, 0x6e, 0x74, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20,
0x63, 0x69, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x63, 0x69,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0x75, 0xf1, 0x20, 0x6a, 0x61, 0x67, 0x6c, 0x65, 0x65, 0x6c,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x43, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x73, 0xe0, 0x6e, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x29, 0x43, 0x53, 0x54,
0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x75, 0x20, 0x73, 0xe0, 0x6e, 0x74, 0x61,
0x72, 0x61, 0x61, 0x6c, 0x29, 0x43, 0x44, 0x54, 0x20, 0x28, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x75, 0x20,
0x73, 0xe0, 0x6e, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x45, 0x54, 0x20,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x45,
0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x75, 0x20, 0x70, 0x65, 0x6e,
0x6b, 0x75, 0x29, 0x45, 0x44, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x75, 0x20, 0x70, 0x65,
0x6e, 0x6b, 0x75, 0x29, 0x4d, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x74, 0x75, 0x6e, 0x64, 0x75, 0x29, 0x4d, 0x53, 0x54, 0x20,
0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x75, 0x20, 0x74, 0x75, 0x6e, 0x64, 0x75, 0x29,
0x4d, 0x44, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0xeb, 0x63, 0x63, 0xeb, 0x67, 0x75, 0x20, 0x74, 0x75, 0x6e, 0x64, 0x75,
0x29, 0x50, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x29, 0x50, 0x53, 0x54, 0x20, 0x28,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x29, 0x50, 0x44, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x29, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x70,
0x69, 0x61, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20,
0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61,
0x41, 0x70, 0x69, 0x61, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0xeb, 0x63, 0x63, 0xeb, 0x67, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x61,
0x72, 0x61, 0x61, 0x62, 0x20, 0x79, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69,
0x20, 0x41, 0x72, 0x61, 0x61, 0x62, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x63, 0x69, 0x20, 0x41, 0x72,
0x61, 0x61, 0x62, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x72, 0x73,
0x61, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20,
0x41, 0x72, 0x73, 0x61, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75,
0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x73, 0x6f,
0x77, 0x77, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x65, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62,
0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65,
0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x41, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x61, 0x74,
0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x6b, 0x29, 0x41, 0x53, 0x54, 0x20, 0x28,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29, 0x41,
0x44, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb,
0x63, 0x63, 0xeb, 0x67, 0x75, 0x20, 0x61, 0x74, 0x6c, 0xe0, 0x6e, 0x74,
0x69, 0x6b, 0x29, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x20, 0x62, 0x75, 0x20, 0x64, 0x69,
0x67, 0x67, 0x69, 0x20, 0x62, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x63, 0x69,
0x20, 0x64, 0x69, 0x67, 0x67, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x73, 0x6f,
0x77, 0x77, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d,
0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x64, 0x69, 0x67, 0x67, 0x75,
0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb,
0x63, 0x63, 0xeb, 0x67, 0x20, 0x63, 0x69, 0x20, 0x64, 0x69, 0x67, 0x67,
0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70,
0x65, 0x6e, 0x6b, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b,
0x75, 0x20, 0x62, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20,
0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x70, 0x65, 0x6e, 0x6b,
0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x20, 0x62, 0x75, 0x20, 0x62, 0xeb, 0x74, 0x75, 0x20, 0x53,
0x6f, 0x6f, 0x77, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1,
0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x73, 0x6f, 0x77,
0x77, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67,
0x20, 0x62, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0xef, 0x64, 0x6a, 0x61, 0x6e, 0x20, 0x57, 0x61, 0x78, 0x74, 0x75,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0xef, 0x64, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65,
0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0xef, 0x64, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d,
0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65,
0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x42, 0x68,
0x6f, 0x75, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x42,
0x6f, 0x6c, 0x69, 0x76, 0x69, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x42, 0x72, 0x65, 0x73, 0x69, 0x6c, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x20, 0x74, 0x69, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x74,
0x69, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x43, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x63, 0x69, 0x20, 0x6a, 0x61, 0x6d,
0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x53, 0x69, 0x6c, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69,
0x20, 0x53, 0x69, 0x6c, 0x69, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65,
0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x53, 0x69, 0x6c, 0x69, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20,
0x63, 0x69, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x43, 0x68, 0x69, 0x6e,
0x65, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63,
0xeb, 0x67, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x69, 0x6c, 0x65, 0x20,
0x62, 0x75, 0x20, 0x6e, 0x6f, 0x65, 0x6c, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x69, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69,
0x6e, 0x20, 0x63, 0x69, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x4a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x20,
0x63, 0x69, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x49, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69,
0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x49, 0x6c, 0x65, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x49, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x78,
0x61, 0x61, 0x6a, 0x75, 0x20, 0x65, 0x74, 0x65, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x43, 0x75, 0x62, 0x61, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20,
0x43, 0x75, 0x62, 0x61, 0x43, 0x75, 0x62, 0x61, 0x20, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x69, 0x6c, 0x65, 0x20, 0x62, 0x75, 0x20, 0x50, 0xe2, 0x71,
0x75, 0x65, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1,
0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x49, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73, 0x4a, 0x61,
0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x20, 0x63, 0x69,
0x20, 0x49, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe2, 0x71, 0x75,
0x65, 0x73, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x45, 0x63, 0x75, 0x61,
0x64, 0x6f, 0x72, 0x43, 0x54, 0x45, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x73, 0xe0, 0x6e, 0x74,
0x61, 0x72, 0x61, 0x61, 0x6c, 0x29, 0x43, 0x45, 0x53, 0x54, 0x20, 0x28,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x73,
0xe0, 0x6e, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x29, 0x43, 0x45, 0x53,
0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65,
0x20, 0x77, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x73, 0xe0,
0x6e, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x29, 0x45, 0x45, 0x54, 0x20,
0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70,
0x20, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29, 0x45, 0x45, 0x53,
0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f,
0x70, 0x20, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29, 0x45, 0x45,
0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74,
0x65, 0x20, 0x77, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75,
0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x57, 0x45, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77,
0x75, 0x2d, 0x6a, 0x61, 0x6e, 0x74, 0x57, 0x45, 0x53, 0x54, 0x20, 0x28,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75,
0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x2d, 0x6a, 0x61, 0x6e, 0x74, 0x29,
0x57, 0x45, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x65, 0x74, 0x65, 0x20, 0x77, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70,
0x20, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x2d, 0x6a, 0x61, 0x6e,
0x74, 0x29, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x74,
0x69, 0x6d, 0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69,
0x69, 0x6e, 0x4a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74,
0x65, 0x20, 0x63, 0x69, 0x20, 0x69, 0x6c, 0x65, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x46,
0x69, 0x64, 0x6a, 0x69, 0x4a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20,
0x65, 0x74, 0x65, 0x20, 0x63, 0x69, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x53, 0x75, 0x64, 0x20, 0x61, 0x6b, 0x20,
0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x71, 0x75, 0x65, 0x20,
0x62, 0x75, 0x20, 0x46, 0x72, 0x61, 0x6e, 0xe7, 0x61, 0x69, 0x73, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x67, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x20,
0x77, 0x61, 0x78, 0x74, 0x75, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65,
0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x69, 0x6c, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x47, 0x4d, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x29,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20,
0x63, 0x69, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69,
0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74,
0x65, 0x20, 0x62, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1,
0x20, 0x6a, 0x61, 0x67, 0x6c, 0x65, 0x65, 0x6c, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67,
0x20, 0x62, 0x75, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e,
0x20, 0x63, 0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x4a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65,
0x20, 0x63, 0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x48,
0x6f, 0x76, 0x64, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x49, 0x6e, 0x64, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x67, 0xe9, 0x65, 0x6a, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x65, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e,
0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x45, 0x6e, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x20, 0x62, 0x75, 0x20, 0x64, 0x69, 0x67, 0x67,
0x69, 0x20, 0x62, 0x69, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70, 0x65,
0x6e, 0x6b, 0x75, 0x20, 0x45, 0x6e, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75,
0x20, 0x45, 0x6e, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x49, 0x72, 0x61, 0x6e,
0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d,
0x69, 0x69, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63,
0x63, 0xeb, 0x67, 0x20, 0x63, 0x69, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x6d, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65,
0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d,
0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x6a, 0x61, 0x70, 0x6f, 0x6e, 0x4a, 0x61, 0x70, 0x6f, 0x6e,
0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb,
0x67, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x62, 0x75, 0x20,
0x73, 0x6f, 0x77, 0x77, 0x75, 0x20, 0x6a, 0x61, 0x6e, 0x74, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63,
0x69, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x63, 0x69, 0x20, 0x4b, 0x6f,
0x72, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20,
0x63, 0x69, 0x20, 0x6a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65,
0x74, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4b, 0x69, 0x72, 0x67,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x49,
0x6c, 0x65, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x61, 0x6e, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x2c, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4d, 0x61, 0x6c, 0x61, 0x79,
0x73, 0x69, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x65, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x49, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x63, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1,
0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74,
0x65, 0x20, 0x62, 0x75, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x65,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x20, 0x62, 0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e,
0x20, 0x63, 0x69, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20,
0x62, 0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x63, 0x69,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x71, 0x75, 0x65, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x61, 0x74, 0x61, 0x72, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x74,
0x61, 0x72, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x61, 0x74,
0x61, 0x72, 0x20, 0x74, 0x69, 0x6d, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x77, 0x20, 0x57, 0x61, 0x78, 0x74, 0x75, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x75, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63,
0x69, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x20,
0x62, 0x75, 0x20, 0x62, 0x65, 0x65, 0x73, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x4e, 0x6f, 0x75, 0x76,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4e, 0x6f, 0x75, 0x76,
0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69,
0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c,
0x6c, 0x65, 0x2d, 0x5a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x4e, 0x65,
0x75, 0x76, 0x65, 0x54, 0x65, 0x72, 0x72, 0x65, 0x20, 0x4e, 0x65, 0x75,
0x76, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4e, 0x69, 0x75, 0x65,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x69, 0x6c, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x49,
0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x20, 0x63,
0x69, 0x20, 0x69, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x56, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x73, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x63, 0x69, 0x20, 0x6a,
0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63,
0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e,
0x20, 0x63, 0x69, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x70, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20, 0x77,
0x61, 0x78, 0x74, 0x75, 0x50, 0x65, 0x72, 0x75, 0x20, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x50,
0x65, 0x72, 0x75, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74,
0x65, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x20,
0x77, 0x61, 0x78, 0x74, 0x75, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62,
0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69, 0x20, 0x66,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x4a, 0x61, 0x6d, 0x6f, 0x6e,
0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x20, 0x63, 0x69, 0x20, 0x50, 0x68,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x69, 0x6c, 0x65, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e,
0x69, 0x78, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x61, 0x6b, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x50,
0x6f, 0x6e, 0x61, 0x70, 0x65, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x6e, 0x64, 0x61, 0x6a, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20,
0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e,
0x20, 0x63, 0x69, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x75,
0x72, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x49, 0x6c, 0x65, 0x20, 0x53,
0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x65, 0x20, 0x64, 0x75, 0x20, 0x53, 0x75, 0x64, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x73, 0x79, 0x6f, 0x77, 0x61, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63,
0x69, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x6c, 0x65, 0x65,
0x72, 0x61, 0x6c, 0x75, 0x20, 0x62, 0x69, 0x73, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x74, 0x69, 0x6d,
0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x4a, 0x61, 0x6d, 0x6f,
0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74, 0x65, 0x20, 0x63, 0x69, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78,
0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x79, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65,
0x74, 0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x4f, 0x75, 0x7a, 0x62,
0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x62, 0x75, 0xf1, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x20, 0x63, 0x69,
0x20, 0x4f, 0x75, 0x7a, 0x62, 0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75,
0x20, 0x4f, 0x75, 0x7a, 0x62, 0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x6d, 0x69, 0x69, 0x6e, 0x57,
0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x57, 0x61,
0x78, 0x74, 0x75, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x63, 0x69, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x63,
0x69, 0x20, 0x6a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20, 0x65, 0x74,
0x65, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0x75, 0xf1, 0x20,
0x6d, 0x69, 0x69, 0x6e, 0x4a, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x79, 0x20,
0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x49, 0x6c,
0x65, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x20, 0x57, 0x61, 0x78, 0x74, 0x75, 0x57, 0x61, 0x78, 0x74, 0x75,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x78, 0x74,
0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x62, 0x75, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x59,
0x65, 0x6b, 0x61, 0x74, 0xe9, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72,
0x67, 0x57, 0x61, 0x78, 0x74, 0x75, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0xe1, 0x72, 0xed, 0x6e, 0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x2d, 0x4f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0x4f, 0x6f, 0x72, 0x75, 0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41, 0x6c, 0xe1, 0x73, 0xed, 0x6b,
0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x41, 0x6c, 0xe1, 0x73, 0xed, 0x6b, 0xe0, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x301, 0x20, 0x41,
0x6c, 0xe1, 0x73, 0xed, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x41, 0x6d, 0xe1, 0x73, 0x1ecd, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x41, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x6d,
0xe1, 0x73, 0x1ecd, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x72, 0x75, 0x20, 0x41, 0x6d, 0xe1, 0x73, 0x1ecd, 0x301, 0x6e, 0xec,
0xe0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0xe1, 0x72, 0xed, 0x6e, 0x20,
0x67, 0x62, 0xf9, 0x6e, 0x67, 0x62, 0xf9, 0x6e, 0xe0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x61, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0xe0, 0xe1, 0x72, 0xed,
0x6e, 0x20, 0x67, 0x62, 0xf9, 0x6e, 0x67, 0x62, 0xf9, 0x6e, 0x41, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0xe1, 0x72, 0xed, 0x6e, 0x20, 0x67, 0x62,
0xf9, 0x6e, 0x67, 0x62, 0xf9, 0x6e, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd,
0x6d, 0x1ecd, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xec, 0x68, 0xe0, 0x20,
0xec, 0x6c, 0xe0, 0x20, 0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x41, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0xcc, 0x68, 0x61,
0x20, 0xcc, 0x6c, 0x61, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20,
0xcc, 0x68, 0xe0, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72, 0x75,
0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xf2, 0x6b, 0xe8, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x61, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0xf2, 0x6b,
0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd,
0x6d, 0x1ecd, 0x20, 0x4f, 0x72, 0x69, 0x2d, 0xf2, 0x6b, 0xe8, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x50, 0xe0, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2,
0x20, 0x50, 0xe0, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x79, 0xe1, 0x6c, 0x1eb9, 0x74, 0x61, 0x20,
0x50, 0xe0, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x41, 0x70, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x70, 0x69,
0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd,
0x6d, 0x1ecd, 0x20, 0x41, 0x70, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa,
0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x41,
0x61, 0x67, 0x6f, 0x20, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xec, 0x6e, 0xe0,
0x41, 0x61, 0x67, 0x6f, 0x20, 0xe0, 0x73, 0xec, 0x6b, 0xf2, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x41, 0x61, 0x67, 0x6f,
0x20, 0x53, 0x6f, 0x6d, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x77, 0x1ecd,
0x301, 0x2d, 0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x6a, 0x1eb9, 0x301,
0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49,
0x77, 0x1ecd, 0x301, 0x2d, 0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x6a, 0x1eb9, 0x301, 0x6e, 0x74,
0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x72, 0x75,
0x20, 0x49, 0x77, 0x1ecd, 0x301, 0x2d, 0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0x41, 0x6a, 0x1eb9, 0x301, 0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x74, 0xec, 0x6c, 0xe1, 0x144,
0x74, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xe0,
0x73, 0x69, 0x6b, 0xf2, 0x20, 0xc0, 0x74, 0xec, 0x6c, 0xe1, 0x144, 0x74,
0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x79,
0xe1, 0x6c, 0x1eb9, 0x74, 0x61, 0x20, 0xc0, 0x74, 0xec, 0x6c, 0xe1, 0x144,
0x74, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc0,
0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0xc0,
0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e,
0x67, 0xf9, 0x6e, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72,
0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e, 0x67,
0xf9, 0x6e, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd,
0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9,
0x6e, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x2d, 0x4f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0xcc, 0x6c, 0xe0, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0xcc, 0x6c,
0xe0, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc,
0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd,
0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d,
0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc1, 0x73,
0x1ecd, 0x301, 0x73, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc1, 0x73, 0x1ecd, 0x301, 0x73, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20, 0xc1,
0x73, 0x1ecd, 0x301, 0x73, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x42,
0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x42,
0xf9, 0x72, 0xe0, 0x73, 0xed, 0x6c, 0xed, 0xe0, 0x41, 0x61, 0x67, 0x6f,
0x20, 0xe0, 0x73, 0xec, 0x6b, 0xf2, 0x20, 0x42, 0xf9, 0x72, 0xe0, 0x73,
0xed, 0x6c, 0xed, 0xe0, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x53, 0x6f, 0x6d,
0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4b, 0xe9, 0x70, 0xfa, 0x20, 0x46, 0xe1, 0xe0,
0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x4b, 0xe9, 0x70, 0xfa, 0x20, 0x46, 0xe1, 0xe0,
0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x1eb8, 0x300, 0x72, 0xfa,
0x6e, 0x20, 0x4b, 0xe9, 0x70, 0xfa, 0x20, 0x46, 0xe1, 0xe0, 0x64, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x43, 0x68, 0x61, 0x74, 0x61, 0x6d, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x43, 0x68, 0x61, 0x74, 0x61, 0x6d, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x43, 0x68, 0x61, 0x74,
0x61, 0x6d, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x1e62, 0xed, 0x6c, 0xe8,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x1e62, 0xed, 0x6c, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x72, 0x75, 0x20, 0x1e62, 0xed, 0x6c, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x1e62, 0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x1e62,
0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a,
0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x1e62, 0xe1, 0xed, 0x6e, 0xe0, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9,
0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0xe9,
0x6b, 0xf9, 0x73, 0xf9, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x41, 0x61,
0x67, 0x6f, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x41,
0x61, 0x67, 0x6f, 0x20, 0xe0, 0x73, 0xec, 0x6b, 0xf2, 0x20, 0x4b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x53,
0x6f, 0x6d, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45,
0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x4b, 0x75, 0x75, 0x6b, 0x75,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75,
0x73, 0x75, 0x20, 0x4b, 0x75, 0x75, 0x6b, 0x75, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x49, 0x6c, 0x61, 0x6a, 0x69, 0x20, 0x53, 0x1ecd, 0x6d, 0x61,
0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73,
0x75, 0x20, 0x4b, 0x75, 0x75, 0x6b, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4b, 0xfa, 0x62, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4b, 0xfa, 0x62, 0xe0, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd,
0x20, 0x4b, 0xfa, 0x62, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x44,
0x61, 0x76, 0x69, 0x73, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x20,
0x6f, 0x6f, 0x72, 0xf9, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x41,
0x61, 0x67, 0x6f, 0x20, 0x41, 0x6a, 0x6e, 0x64, 0x65, 0x20, 0x49, 0x62,
0xf9, 0x67, 0x62, 0xe9, 0x20, 0x4f, 0x6d, 0x69, 0x41, 0x61, 0x67, 0x6f,
0x20, 0xe0, 0x73, 0xec, 0x6b, 0xf2, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x61, 0x67, 0x6f,
0x20, 0x53, 0x6f, 0x6d, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x61, 0x67, 0x6f, 0x20,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0xe1, 0x72, 0x69, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe1, 0x72, 0x69, 0x6e,
0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe1, 0x72, 0x69, 0x6e,
0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x68, 0xe0, 0x20, 0xcc, 0x6c,
0xe0, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0x73, 0x69, 0x6b,
0xf2, 0x20, 0xcc, 0x68, 0xe0, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0xcc, 0x68, 0x61,
0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49,
0x77, 0xe1, 0x6a, 0xfa, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72,
0xf9, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0xf3, 0xf2, 0x70, 0xf9, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x20, 0x4f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x20,
0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0xcc,
0x68, 0xe0, 0x20, 0xcc, 0x77, 0x1ecd, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x46, 0xf3, 0x6b, 0xed, 0x6c, 0xe1, 0x6e, 0x64, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x45, 0x74, 0xed, 0x6b, 0x75, 0x6e, 0x20, 0x46, 0xf3, 0x6b, 0xed, 0x6c,
0xe1, 0x6e, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f,
0x72, 0x75, 0x20, 0x45, 0x74, 0xed, 0x6b, 0x75, 0x6e, 0x20, 0x46, 0xf3,
0x6b, 0xed, 0x6c, 0xe1, 0x6e, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x46, 0x69, 0x6a, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x46, 0x69, 0x6a, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0x61, 0x20, 0x46, 0x69,
0x6a, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x47, 0xfa, 0x79, 0xe1,
0x6e, 0xe0, 0x20, 0x46, 0xe1, 0x72, 0xe1, 0x6e, 0x73, 0xe8, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x47, 0xfa, 0x73, 0xf9, 0x20, 0x46, 0xe1, 0x72,
0x61, 0x6e, 0x73, 0xe9, 0x20, 0xe0, 0x74, 0x69, 0x20, 0xc0, 0x6e, 0x74,
0xe1, 0x74, 0xed, 0xec, 0x6b, 0xec, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0x1ecd, 0x6d, 0xe0, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72,
0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x47, 0x69, 0x6c, 0x69, 0x62, 0x61,
0x74, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x20,
0x6f, 0x6f, 0x72, 0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd,
0x300, 0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x6f, 0x6f,
0x72, 0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xec, 0x67, 0x62, 0xe0, 0x20,
0x4f, 0x6f, 0x72, 0x75, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300,
0x20, 0x6f, 0x6f, 0x72, 0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x20, 0x4f,
0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0xcc, 0x67, 0x62, 0x61, 0x20, 0x4f, 0xf2, 0x72,
0x75, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x47, 0x75, 0x6c, 0x66, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x47,
0xfa, 0x79, 0xe0, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd,
0x6d, 0xe0, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x48, 0x6f, 0x66, 0x69, 0x64, 0x69,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x73, 0xed, 0x20, 0x48, 0x6f, 0x66, 0x69, 0x64, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x48, 0x6f, 0x66,
0x69, 0x64, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x45, 0x74, 0xed, 0x6b, 0x75, 0x6e, 0x20, 0xcd,
0x6e, 0x64, 0xed, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47, 0x62, 0xf9, 0x6e, 0x67,
0x62, 0xf9, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x6f,
0x6f, 0x72, 0xf9, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300,
0x20, 0x6f, 0x6f, 0x72, 0xf9, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x72,
0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcd,
0x6b, 0xf3, 0x73, 0xed, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xcd, 0x6b, 0xf3, 0x73,
0xed, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d,
0xe0, 0x20, 0xcd, 0x6b, 0xfa, 0x73, 0xed, 0x6b, 0xec, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x73, 0xed, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x73, 0x69,
0x74, 0xe1, 0xe0, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc,
0x6c, 0xe0, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x4b, 0x61, 0x73,
0x61, 0x73, 0x69, 0x74, 0xe1, 0xe0, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x4b, 0x61, 0x73, 0x61, 0x73, 0x69, 0x74, 0xe1, 0xe0, 0x6e, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4b, 0x6f, 0x72, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2,
0x73, 0xed, 0x20, 0x4b, 0x6f, 0x72, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x4b, 0x6f,
0x72, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4b, 0x6f, 0x73,
0x69, 0x72, 0x61, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x73, 0x69, 0x6b, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x73, 0x69,
0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x73, 0x69,
0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4b, 0x69, 0x72, 0x69,
0x67, 0x69, 0x73, 0x69, 0x74, 0x61, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b,
0x75, 0x73, 0x75, 0x20, 0x4c, 0x61, 0x69, 0x6e, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4d,
0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4d, 0x61, 0x6b, 0x75, 0x65, 0x73, 0x61, 0x73, 0x69,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45,
0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x4d, 0x61, 0x73, 0x61, 0x61,
0x6c, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed,
0x1e63, 0xfa, 0x1e63, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x1e63, 0xfa,
0x1e63, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75,
0x20, 0x4d, 0xe1, 0x72, 0xed, 0x1e63, 0xfa, 0x1e63, 0xec, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec,
0x20, 0x4d, 0x1eb9, 0x301, 0x1e63, 0xed, 0x6b, 0xf2, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe1,
0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0x20, 0x4d, 0x1eb9, 0x301, 0x73,
0xed, 0x6b, 0xf2, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa,
0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x50, 0xe1, 0x73, 0xed, 0x66, 0xed, 0xec,
0x6b, 0xec, 0x20, 0x4d, 0x1eb9, 0x301, 0x73, 0xed, 0x6b, 0xf2, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4d, 0x6f,
0x73, 0x69, 0x6b, 0x6f, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4d, 0x6f, 0x73, 0x69, 0x6b, 0x6f,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x4d,
0x6f, 0x73, 0x69, 0x6b, 0x6f, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc,
0x6c, 0xe0, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x20, 0x54, 0x75, 0x6e, 0x74, 0x75, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4b, 0x61, 0x6c,
0x69, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x75, 0x6e, 0x74, 0x75,
0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0x61, 0x20,
0x4b, 0x61, 0x6c, 0x69, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x75,
0x6e, 0x74, 0x75, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4e,
0x65, 0x77, 0x20, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20,
0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4e, 0x69, 0x75, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9, 0x73,
0xf9, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x45,
0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x46, 0x65, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x61,
0x41, 0x61, 0x67, 0x6f, 0x20, 0xe0, 0x73, 0xec, 0x6b, 0xf2, 0x20, 0x46,
0x65, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x53, 0x6f, 0x6d,
0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x69, 0x73, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x72, 0x69, 0x62,
0x69, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4f, 0x6d, 0x69, 0x73,
0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d,
0xe0, 0x20, 0x4f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0xe1,
0x72, 0xe1, 0x67, 0xfa, 0x77, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x72, 0xe1,
0x67, 0xfa, 0x77, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f,
0x72, 0x75, 0x20, 0x50, 0xe1, 0x72, 0xe1, 0x67, 0xfa, 0x77, 0xe8, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0xe9, 0x72, 0xf9, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50,
0xe9, 0x72, 0xf9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72,
0x75, 0x20, 0x50, 0xe9, 0x72, 0xf9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x6e, 0x6e, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x6e, 0x6e, 0x69,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x6e, 0x6e, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72, 0xe9,
0x6b, 0xf9, 0x73, 0xf9, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0x6f, 0x6e,
0x61, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0x79, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x52, 0x1eb9, 0x301, 0x79, 0xfa, 0x6e, 0xed, 0x1ecd, 0x301, 0x6e, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d,
0x1ecd, 0x6d, 0x1ecd, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0xe8, 0x1e63, 0x1eb9, 0x301, 0x1eb9, 0x300, 0x6c, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x1ecd, 0x6e, 0x20, 0x45, 0x72,
0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x47, 0xfa, 0x73, 0xf9, 0x20,
0x4a, 0x1ecd, 0x301, 0x6a, 0xed, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0xfa, 0x72, 0xed, 0x6e, 0xe1, 0x6d, 0xf9, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b,
0x69, 0x73, 0x69, 0x74, 0x61, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d,
0xe0, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x69, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x54, 0x75, 0x66, 0x61, 0x6c, 0x75, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xda, 0x72, 0xfa, 0x67,
0xfa, 0x77, 0xe8, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x61,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x55, 0x73, 0x69, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x55, 0x73, 0x69, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd,
0x6d, 0xe0, 0x20, 0x55, 0x73, 0x69, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x46, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x46, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20,
0x46, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x41, 0x61, 0x67, 0x6f, 0x20,
0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x46, 0x69, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f, 0x73,
0x69, 0x74, 0x6f, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x46, 0x69, 0x6c, 0x61, 0x64,
0x69, 0x66, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x46, 0x69, 0x6c, 0x61,
0x64, 0x69, 0x66, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x46, 0x6f, 0x6c, 0x69, 0x67, 0x6f, 0x67, 0x69,
0x72, 0x61, 0x64, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x46, 0x6f, 0x6c, 0x69, 0x67, 0x6f,
0x67, 0x69, 0x72, 0x61, 0x64, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x46, 0x6f, 0x6c, 0x69, 0x67, 0x6f, 0x67,
0x69, 0x72, 0x61, 0x64, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x45,
0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9, 0x20, 0x57, 0x61, 0x6b, 0x65, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20,
0x26, 0x20, 0x46, 0x75, 0x74, 0x75, 0x69, 0x6e, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x59, 0x61, 0x74, 0x75, 0x74, 0x73, 0x6b, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x59, 0x61, 0x74, 0x75, 0x74, 0x73, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x59, 0x61, 0x74, 0x75, 0x74, 0x73,
0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x59, 0xfa, 0x6b, 0x1ecd, 0x301, 0x6e, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f,
0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f,
0xf2, 0x72, 0xf9, 0x6e, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b,
0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x41, 0x6c, 0xe1, 0x73, 0xed, 0x6b, 0xe0, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x301, 0x20, 0x41,
0x6c, 0xe1, 0x73, 0xed, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x41, 0x6d, 0xe1, 0x73, 0x254, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x41, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x6d,
0xe1, 0x73, 0x254, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x72, 0x75, 0x20, 0x41, 0x6d, 0xe1, 0x73, 0x254, 0x301, 0x6e, 0xec,
0x41, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0xe1, 0x72, 0xed, 0x6e, 0x20,
0x67, 0x62, 0xf9, 0x6e, 0x67, 0x62, 0xf9, 0x6e, 0x20, 0x6f, 0x6a, 0xfa,
0x6d, 0x254, 0x6d, 0x254, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a,
0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0xcc, 0x68, 0xe0, 0x20, 0xcc, 0x6c,
0xe0, 0x20, 0x4f, 0xf2, 0x72, 0x75, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x4f, 0x72, 0x69,
0x2d, 0xf2, 0x6b, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x79,
0xe1, 0x6c, 0x25b, 0x74, 0x61, 0x20, 0x50, 0xe0, 0x73, 0xed, 0x66, 0xed,
0xec, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x70, 0x69, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x41,
0x70, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254,
0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20,
0x41, 0x6a, 0x25b, 0x6e, 0x74, 0xec, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x49, 0x77, 0x254, 0x301, 0x2d, 0x6f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x41, 0x6a, 0x25b, 0x301, 0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x77, 0x254, 0x301, 0x2d, 0x6f, 0xf2, 0x72,
0xf9, 0x6e, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41,
0x6a, 0x25b, 0x301, 0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x4f, 0x72, 0x75, 0x20, 0x49, 0x77, 0x254, 0x301, 0x2d, 0x6f,
0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x6a, 0x25b, 0x301, 0x6e, 0x74, 0xed,
0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x41, 0x72,
0x61, 0x62, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x79,
0xe1, 0x6c, 0x25b, 0x74, 0x61, 0x20, 0xc0, 0x74, 0xec, 0x6c, 0xe1, 0x144,
0x74, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e,
0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e,
0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0xe0, 0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20,
0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc0, 0xe0,
0x72, 0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0xcc,
0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0xc0, 0xe0, 0x72,
0x69, 0x6e, 0x20, 0x47, 0xf9, 0x6e, 0x67, 0xf9, 0x6e, 0x20, 0xcc, 0x77,
0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x2d,
0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa,
0x6d, 0x254, 0x6d, 0x254, 0x20, 0xcc, 0x6c, 0xe0, 0x2d, 0x4f, 0xf2, 0x72,
0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f,
0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72,
0xf9, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d,
0x254, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc1, 0x73, 0x254, 0x301, 0x73, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc1, 0x73, 0x254,
0x301, 0x73, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72,
0x75, 0x20, 0xc1, 0x73, 0x254, 0x301, 0x73, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4b, 0xe9, 0x70, 0xfa, 0x20,
0x46, 0xe1, 0xe0, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x190,
0x300, 0x72, 0xfa, 0x6e, 0x20, 0x4b, 0xe9, 0x70, 0xfa, 0x20, 0x46, 0xe1,
0xe0, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0x43, 0x68, 0x61, 0x74, 0x61, 0x6d, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x43,
0x68, 0x61, 0x74, 0x61, 0x6d, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0x68, 0xed, 0x6c, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x53, 0x68, 0xed, 0x6c, 0xe8, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x72, 0x75, 0x20, 0x53, 0x68, 0xed,
0x6c, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x68, 0xe1, 0xed,
0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x53, 0x68, 0xe1, 0xed, 0x6e, 0xe0,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d,
0x254, 0x20, 0x53, 0x68, 0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x77, 0x254, 0x6e, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9,
0x73, 0xf9, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x77, 0x254, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75,
0x73, 0x75, 0x20, 0x4b, 0x75, 0x75, 0x6b, 0x75, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xc0, 0x77,
0x254, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x4b,
0x75, 0x75, 0x6b, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x6c,
0x61, 0x6a, 0x69, 0x20, 0x53, 0x254, 0x6d, 0x61, 0x20, 0xc0, 0x77, 0x254,
0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x4b, 0x75,
0x75, 0x6b, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4b, 0xfa, 0x62, 0xe0, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x4b,
0xfa, 0x62, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe1, 0x72,
0x69, 0x6e, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0,
0x20, 0xcc, 0x68, 0x61, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72,
0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x20, 0x4f,
0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0xcc, 0x68,
0xe0, 0x20, 0xcc, 0x77, 0x254, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x45, 0x74, 0xed, 0x6b,
0x75, 0x6e, 0x20, 0x46, 0xf3, 0x6b, 0xed, 0x6c, 0xe1, 0x6e, 0x64, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x46, 0x69, 0x6a, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x254, 0x6d, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0x254, 0x6d, 0xe0, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x254, 0x6e, 0x20, 0x45,
0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x47, 0x69, 0x6c, 0x69, 0x62,
0x61, 0x74, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254,
0x300, 0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x6f, 0x6f,
0x72, 0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x20,
0x6f, 0x6f, 0x72, 0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x20, 0x4f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0xcc, 0x67, 0x62, 0x61, 0x20, 0x4f, 0xf2, 0x72, 0x75,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x47, 0x75, 0x6c, 0x66, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d,
0xe0, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x73, 0xed, 0x20, 0x48, 0x6f, 0x66, 0x69, 0x64, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x48, 0x6f, 0x66, 0x69,
0x64, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x20, 0x6f, 0x6f, 0x72, 0xf9,
0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0xcd, 0x6b, 0xf3, 0x73, 0xed, 0x6b, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0xcd, 0x6b, 0xfa,
0x73, 0xed, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d,
0x254, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0x254, 0x6d, 0xe0, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x73, 0x69, 0x74, 0xe1, 0xe0, 0x6e,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x73, 0xed, 0x20, 0x4b, 0x6f, 0x72, 0x69, 0x61, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20,
0x4b, 0x6f, 0x72, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x69,
0x6e, 0x6f, 0x79, 0x61, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x69,
0x6e, 0x6f, 0x79, 0x61, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x77, 0x254, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75,
0x73, 0x75, 0x20, 0x4c, 0x61, 0x69, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254,
0x6d, 0xe0, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x254, 0x6e, 0x20, 0x45, 0x72,
0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x4d, 0x61, 0x73, 0x61, 0x61, 0x6c,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x73,
0x68, 0xfa, 0x73, 0x68, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x73,
0x68, 0xfa, 0x73, 0x68, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6f, 0x72, 0x75, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x73, 0x68, 0xfa, 0x73,
0x68, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x73, 0xed,
0x66, 0xed, 0xec, 0x6b, 0xec, 0x20, 0x4d, 0x25b, 0x301, 0x73, 0x68, 0xed,
0x6b, 0xf2, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b,
0xec, 0x20, 0x4d, 0x25b, 0x301, 0x73, 0xed, 0x6b, 0xf2, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x50,
0xe1, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0x20, 0x4d, 0x25b, 0x301,
0x73, 0xed, 0x6b, 0xf2, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x73, 0xed, 0x20, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4d, 0x6f, 0x73, 0x69, 0x6b,
0x6f, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20,
0x4d, 0x6f, 0x73, 0x69, 0x6b, 0x6f, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4b, 0x61, 0x6c, 0x69,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x75, 0x6e, 0x74, 0x75, 0x6e,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0x61, 0x20, 0x4b,
0x61, 0x6c, 0x69, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x75, 0x6e,
0x74, 0x75, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x7a, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a,
0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x45,
0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d,
0x254, 0x6d, 0x254, 0x20, 0x45, 0x72, 0xe9, 0x6b, 0xf9, 0x73, 0xf9, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4e, 0x6f, 0x66,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x4e, 0x6f, 0x66, 0x6f,
0x72, 0x69, 0x62, 0x69, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4f, 0x6d,
0x69, 0x73, 0x69, 0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0x254, 0x6d, 0xe0, 0x20, 0x4f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x72, 0xe1,
0x67, 0xfa, 0x77, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe9, 0x72, 0xf9, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x6e, 0x6e, 0x69, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x6e, 0x6e, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x77, 0x254, 0x6e, 0x20, 0x45, 0x72, 0xe9, 0x6b,
0xf9, 0x73, 0xf9, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x52, 0x25b, 0x301, 0x79, 0xfa,
0x6e, 0xed, 0x254, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254,
0x6d, 0xe0, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0xe8, 0x73, 0x68, 0x25b,
0x301, 0x25b, 0x300, 0x6c, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x77,
0x254, 0x6e, 0x20, 0x45, 0x72, 0x65, 0x6b, 0x75, 0x73, 0x75, 0x20, 0x53,
0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x47, 0xfa, 0x73, 0xf9, 0x20, 0x4a, 0x254, 0x301, 0x6a, 0xed, 0xe0, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x73, 0xed, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x69, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xda, 0x72,
0xfa, 0x67, 0xfa, 0x77, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x55, 0x73, 0x69, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x55, 0x73, 0x69, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x46, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d,
0xe0, 0x20, 0x46, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x46,
0x69, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b,
0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20,
0x46, 0x69, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f, 0x73, 0x69, 0x74, 0x6f,
0x6b, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x46, 0x6f, 0x6c, 0x69, 0x67, 0x6f, 0x67, 0x69,
0x72, 0x61, 0x64, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254,
0x6d, 0xe0, 0x20, 0x46, 0x6f, 0x6c, 0x69, 0x67, 0x6f, 0x67, 0x69, 0x72,
0x61, 0x64, 0x69, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x59, 0x61, 0x74, 0x75, 0x74, 0x73, 0x6b,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x59,
0x61, 0x74, 0x75, 0x74, 0x73, 0x6b, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x59, 0xfa, 0x6b, 0x254, 0x301, 0x6e, 0xec, 0x47, 0x77,
0x7a, 0x6c, 0x69, 0x6e, 0x7a, 0x76, 0x65, 0x69, 0x68, 0x63, 0x69, 0x20,
0x42, 0x79, 0x61, 0x75, 0x68, 0x63, 0x75, 0x6e, 0x6a, 0x20, 0x53, 0x69,
0x7a, 0x67, 0x65, 0x6e, 0x68, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x4d, 0x70, 0x75, 0x6d, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x61,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x69, 0x6e, 0x67, 0x69,
0x7a, 0x69, 0x6d, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x4e, 0x74, 0x73, 0x68, 0x6f, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x67,
0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x74, 0x73, 0x68,
0x6f, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73,
0x61, 0x73, 0x65, 0x4e, 0x74, 0x73, 0x68, 0x6f, 0x6e, 0x61, 0x6c, 0x61,
0x6e, 0x67, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79,
0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e,
0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x65, 0x73, 0x69, 0x6a,
0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x73, 0x61, 0x73, 0x65,
0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74,
0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x73, 0x69, 0x6a,
0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e,
0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20,
0x45, 0x61, 0x73, 0x74, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79,
0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f,
0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69,
0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77,
0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20,
0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e,
0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x65,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x70, 0x69,
0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x41, 0x70, 0x69, 0x61, 0x20, 0x65, 0x73, 0x69,
0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x70, 0x69,
0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x65, 0x73,
0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x4e, 0x79, 0x61, 0x6b, 0x61, 0x74, 0x68, 0x6f, 0x20, 0x6e, 0x65, 0x2d,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e,
0x79, 0x61, 0x6b, 0x61, 0x74, 0x68, 0x6f, 0x20, 0x6e, 0x65, 0x2d, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x65, 0x73, 0x69,
0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x4e, 0x79, 0x61, 0x6b, 0x61, 0x74, 0x68, 0x6f, 0x20, 0x6e, 0x65,
0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73,
0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61,
0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x73, 0x61, 0x73, 0x65,
0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x73,
0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e,
0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57,
0x65, 0x73, 0x74, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65,
0x73, 0x74, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61,
0x73, 0x74, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x45, 0x61, 0x73, 0x74, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57, 0x65,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69,
0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a,
0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61,
0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f,
0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x65, 0x73, 0x69, 0x6a,
0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x61,
0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61,
0x6c, 0x61, 0x6d, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x65, 0x73, 0x65, 0x7a, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65,
0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x69,
0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x72, 0x69,
0x73, 0x74, 0x6d, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c,
0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f,
0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x79, 0x69, 0x6e, 0x67, 0x78,
0x65, 0x6e, 0x79, 0x65, 0x20, 0x79, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f,
0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6f, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43,
0x75, 0x62, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x75, 0x62, 0x61, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x43, 0x75, 0x62, 0x61, 0x20, 0x73, 0x61, 0x73,
0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x44, 0x61, 0x76, 0x69,
0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45,
0x61, 0x73, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61,
0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c,
0x65, 0x6b, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x75, 0x72, 0x74, 0x68,
0x65, 0x72, 0x2d, 0x65, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x6a,
0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20,
0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69,
0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x73, 0x61,
0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x69,
0x6a, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x46, 0x69, 0x6a, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x69, 0x6a, 0x69, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61,
0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20,
0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72, 0x6e, 0x20, 0x6e, 0x61, 0x73,
0x65, 0x2d, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45,
0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77,
0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x65, 0x73,
0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x47, 0x75, 0x6c, 0x66, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47,
0x75, 0x79, 0x61, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65,
0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20,
0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x6f, 0x76, 0x64, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x6d, 0x69,
0x73, 0x69, 0x77, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x6f,
0x76, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x48, 0x6f,
0x76, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x49, 0x72, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x72, 0x61,
0x6e, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d,
0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69,
0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69,
0x20, 0x73, 0x61, 0x6b, 0x77, 0x61, 0x2d, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x70,
0x75, 0x6d, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x6e, 0x65, 0x2d,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x4e, 0x74, 0x73, 0x68, 0x6f, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x61,
0x20, 0x6e, 0x65, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c,
0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x20, 0x65, 0x2d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73,
0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e,
0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69,
0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f,
0x62, 0x6f, 0x20, 0x65, 0x2d, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x65, 0x73, 0x69,
0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69,
0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x45, 0x73, 0x69, 0x6d, 0x69, 0x73, 0x69, 0x77, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x77, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x77, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c,
0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x20, 0x65, 0x2d, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x61,
0x75, 0x72, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x61,
0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d,
0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77,
0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c,
0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x61, 0x73,
0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x69, 0x75, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f,
0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c,
0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4f, 0x6d, 0x73,
0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4f, 0x6d, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4f, 0x6d, 0x73, 0x6b, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x65, 0x73,
0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x50, 0x65, 0x72, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x65,
0x72, 0x75, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x50, 0x65, 0x72, 0x75, 0x20, 0x73, 0x61, 0x73, 0x65,
0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x68, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c,
0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x68, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x6e, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x49, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x6e, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61,
0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x6e, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x79, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x52, 0x65, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x52, 0x6f, 0x74, 0x68, 0x65,
0x72, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x65,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x61,
0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x65, 0x79,
0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x6f, 0x72, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x6f, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x6d,
0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c,
0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77,
0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x73, 0x61, 0x73, 0x65,
0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f,
0x62, 0x6f, 0x20, 0x65, 0x2d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x6e, 0x61, 0x73,
0x65, 0x2d, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c,
0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69,
0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x41, 0x6b, 0x72, 0x65, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x6b, 0x72, 0x65, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41,
0x6b, 0x72, 0x65, 0x20, 0x74, 0xe1, 0x41, 0x66, 0x65, 0x67, 0x61, 0x6e,
0x129, 0x74, 0xe3, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x2d, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1,
0x20, 0xf3, 0x72, 0x61, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x52,
0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0x53,
0x75, 0x72, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0xe1,
0x20, 0xf3, 0x72, 0x61, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x52,
0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x41, 0x72,
0x61, 0x6e, 0x68, 0x6b, 0x61, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x72, 0x61, 0x6e, 0x68,
0x6b, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x41, 0x72, 0x61, 0x6e, 0x68, 0x6b, 0x61, 0x20, 0x74,
0xe1, 0x41, 0x72, 0x6d, 0x61, 0x74, 0x79, 0x20, 0x74, 0xe1, 0x20, 0xf3,
0x72, 0x61, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x72, 0x6d,
0x61, 0x74, 0x79, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x61, 0x74, 0x79, 0x20, 0x74,
0xe1, 0x41, 0x6d, 0x1ef9, 0x6a, 0x6f, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0x20,
0xf3, 0x72, 0x61, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x6d,
0x1ef9, 0x6a, 0x6f, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x6d, 0x1ef9, 0x6a, 0x6f, 0x6e,
0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b,
0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x4b, 0x72, 0x129, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x4b, 0x72, 0x129, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x72, 0x129, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x41, 0x6e, 0x1ef9, 0x6e, 0x6e, 0x79, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x6e, 0x1ef9, 0x6e, 0x68, 0x79, 0x72,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x41, 0x6e, 0x1ef9, 0x6e, 0x68, 0x79, 0x72, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x41, 0x70, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x41, 0x67, 0x74, 0x61, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x41, 0x67, 0x74, 0x61, 0x76, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x67, 0x74,
0x61, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x67, 0x74,
0xf3, 0x6d, 0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x41, 0x67, 0x74, 0xf3, 0x6d, 0x65, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x67, 0x74, 0xf3,
0x6d, 0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x61,
0x6d, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x41, 0x72, 0x61, 0x6d, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x72,
0x61, 0x6d, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4f, 0x70, 0xe3, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74,
0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129, 0x6e, 0x1ef9,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74,
0x129, 0x6e, 0x1ef9, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74,
0x129, 0x6e, 0x1ef9, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x72,
0x6a, 0x1ebd, 0x74, 0x129, 0x6e, 0x1ef9, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x1ebd, 0x6e, 0x129,
0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x41, 0x72, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6d,
0x1ebd, 0x6e, 0x129, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x41, 0x74, 0x72, 0x1ef9, 0x74, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x74, 0x72, 0x1ef9, 0x74, 0x69,
0x6b, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x41, 0x74, 0x72, 0x1ef9, 0x74, 0x69, 0x6b, 0x75, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72,
0x61, 0x72, 0x69, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79,
0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41,
0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x4b, 0x75,
0x6a, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f,
0x74, 0x79, 0x61, 0x72, 0x69, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x2d,
0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69,
0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x2d, 0x52, 0xe3, 0x70, 0x75, 0x72,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20,
0x4b, 0x75, 0x6a, 0x75, 0x2d, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61,
0x72, 0x69, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79,
0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x52,
0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41,
0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x52, 0xe3,
0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20,
0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72,
0x61, 0x72, 0x69, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x72, 0x6d, 0x61, 0x6a,
0x6a, 0xe1, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x41, 0x6a, 0x65, 0x72, 0x6d, 0x61, 0x6a, 0x6a, 0xe1, 0x76, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x41, 0x6a, 0x65, 0x72, 0x6d, 0x61, 0x6a, 0x6a, 0xe1, 0x76, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x2d, 0x61, 0x67,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x73,
0x6f, 0x72, 0x2d, 0x61, 0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x73, 0x6f, 0x2d, 0x61, 0x67,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x6e, 0x67, 0x72,
0x61, 0x6e, 0xe9, 0x73, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4d, 0x1ef9, 0x6e, 0x67, 0x72, 0x61, 0x6e, 0xe9, 0x73,
0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x4d, 0x1ef9, 0x6e, 0x67, 0x72, 0x61, 0x6e, 0xe9, 0x73, 0x69,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x75, 0x74, 0x1ef9, 0x76,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x76,
0x69, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x72, 0x61,
0x73, 0x69, 0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4d, 0x72, 0x61, 0x73, 0x69, 0x72, 0x6a, 0x61, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x4d, 0x72, 0x61, 0x73, 0x69, 0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x72, 0x75, 0x6e, 0x1ebd, 0x6a, 0x20, 0x4e, 0x61,
0x72, 0x75, 0x73, 0x61, 0x72, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0x75, 0x20, 0x54, 0xe1, 0x6e, 0x68, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50, 0x75, 0x20, 0x54, 0xe1, 0x6e,
0x68, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x50, 0x75, 0x20, 0x74, 0xe1, 0x6e, 0x68, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x53, 0x61, 0x6d, 0xe3, 0x68, 0x6f, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x61, 0x6e, 0x2d, 0x68, 0x1ef9, 0x6d,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x61,
0x6e, 0x2d, 0x68, 0x1ef9, 0x6d, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x53, 0x61, 0x6e, 0x2d, 0x68, 0x1ef9,
0x6d, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x69, 0x72, 0x65,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x69,
0x72, 0x65, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x53, 0x69, 0x72, 0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x53, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x53, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x53, 0x129, 0x6e,
0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x72, 0x129, 0x74,
0x69, 0x6d, 0x1ef9, 0x6e, 0x68, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd,
0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x6f, 0x6b,
0x6f, 0x6e, 0x68, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0xe3, 0x6d,
0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x4b, 0x6f, 0x72, 0xe3, 0x6d, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x6f,
0x72, 0xe3, 0x6d, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd,
0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x4b, 0x75, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73,
0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76,
0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x75,
0x6d, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x4b, 0x75, 0x6d, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x6d, 0x61, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x4e, 0x61, 0x76, 0x69, 0x6e, 0x68, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4e, 0x75, 0x6d, 0xe3, 0x2d, 0x4e, 0x75,
0x72, 0x76, 0x69, 0x72, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x54, 0x129, 0x6d, 0xe3, 0x72, 0x2d, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x6e, 0x68, 0x6b, 0x75,
0x76, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50, 0x61, 0x6e,
0x68, 0x6b, 0x75, 0x76, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd,
0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x50, 0x61, 0x6e, 0x68, 0x6b, 0x75, 0x76, 0x61, 0x20,
0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x45, 0x6b, 0x75, 0x76, 0x61, 0x6e, 0x6f, 0x72, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20,
0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20,
0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x52, 0xe3, 0x6a,
0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x52, 0xe3, 0x6a,
0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74,
0x1ef9, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x6a, 0xe3, 0x20, 0x68,
0xe1, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3, 0x72, 0x61, 0x20,
0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4f, 0x72, 0xf3,
0x70, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4f, 0x72, 0xf3,
0x70, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x72, 0x76, 0x129, 0x6e, 0x1ef9, 0x20, 0x47,
0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x20, 0x50, 0xe3, 0x20, 0x4d, 0x1ef9, 0x72, 0x76, 0x129, 0x6e, 0x1ef9, 0x20,
0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x72,
0x76, 0x129, 0x6e, 0x1ef9, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73,
0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x46, 0x72, 0x1ef9, 0x73, 0x61, 0x20, 0x47, 0x69, 0x6a, 0x61,
0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x46, 0x72, 0x1ef9,
0x73, 0x61, 0x20, 0x47, 0x61, 0x20, 0x53, 0x75, 0x72, 0x20, 0x6b, 0x61,
0x72, 0x20, 0x1ef8, 0x74, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47, 0x61, 0x72, 0x61, 0x20, 0x50, 0x61,
0x67, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47, 0x1ef9, 0x6d,
0x6d, 0x69, 0x6a, 0x65, 0x72, 0xd3, 0x72, 0x61, 0x20, 0x4a, 0x65, 0xf3,
0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x4a, 0x65, 0xf3, 0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4a, 0x65, 0xf3, 0x72,
0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4a, 0x69, 0x72,
0x6d, 0xe9, 0x72, 0x74, 0x75, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd,
0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x69, 0x72,
0x69, 0x6e, 0x6a, 0x1ef9, 0x6e, 0x169, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x169,
0x76, 0x69, 0x73, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x1ebd, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x52, 0xe3,
0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20,
0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72, 0x1ef9,
0x6e, 0x69, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72, 0x1ef9, 0x6e,
0x69, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72,
0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x1ebd, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x52,
0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x47, 0x75, 0x76, 0x1ef9, 0x6d, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x47, 0x6f, 0x72, 0x66, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x47, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61, 0x2019, 0x69, 0x20, 0x6b, 0x61,
0x72, 0x20, 0x41, 0x72, 0x65, 0x76, 0x74, 0x61, 0x20, 0x47, 0x6f, 0x6a,
0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x48, 0x61, 0x76, 0x61, 0x2019, 0x69, 0x20, 0x6b, 0x61,
0x72, 0x20, 0x41, 0x72, 0x65, 0x76, 0x74, 0x61, 0x20, 0x47, 0x6f, 0x6a,
0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x6b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61, 0x2019, 0x69,
0x20, 0x6b, 0x61, 0x72, 0x20, 0x41, 0x72, 0x65, 0x76, 0x74, 0x61, 0x20,
0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x48, 0xe3, 0x67, 0x20, 0x4b, 0xe3, 0x67, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x48, 0xe3, 0x67, 0x20,
0x4b, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x48, 0xe3, 0x67, 0x20, 0x4b, 0xe3, 0x67, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x48, 0xf3, 0x76, 0x69, 0x6e, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x48, 0xf3, 0x76,
0x69, 0x6e, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x6b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x48, 0xf3, 0x76, 0x69, 0x6e, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x61, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x1ef9, 0x6e, 0x6f,
0x20, 0x128, 0x6e, 0x69, 0x6b, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x128, 0x6e, 0x6f, 0x73, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x128, 0x6e, 0x6f, 0x6e, 0x1ebd, 0x6a, 0x61, 0x20, 0x4b,
0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x128, 0x6e,
0x6f, 0x6e, 0x1ebd, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x128, 0x6e, 0x6f, 0x6e, 0x1ebd, 0x6a,
0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x49, 0x72, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x49, 0x72, 0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x49, 0x72, 0x1ef9, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x6e,
0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x49, 0x73, 0x69, 0x68, 0x61, 0xe9, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x49, 0x73, 0x69,
0x68, 0x61, 0xe9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x49, 0x73, 0x69, 0x68, 0x61, 0xe9, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4a, 0x61, 0x70, 0xe3, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4a, 0x61, 0x70, 0xe3, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x4a, 0x61, 0x70, 0xe3, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0x65, 0x74, 0x72, 0x75, 0x70, 0x61, 0x76, 0x69, 0x72, 0xf3, 0x76, 0x69,
0x6e, 0x68, 0x6b, 0x69, 0x2d, 0x4b, 0x1ef9, 0x6d, 0x73, 0x61, 0x74, 0x69,
0x6e, 0x68, 0x6b, 0x69, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50,
0x65, 0x74, 0x72, 0x75, 0x70, 0x61, 0x76, 0x69, 0x72, 0xf3, 0x76, 0x69,
0x6e, 0x68, 0x6b, 0x69, 0x2d, 0x4b, 0x1ef9, 0x6d, 0x73, 0x61, 0x74, 0x69,
0x6e, 0x68, 0x6b, 0x69, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x75, 0x70, 0x61, 0x76, 0x69, 0x72,
0xf3, 0x76, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x2d, 0x4b, 0x1ef9, 0x6d, 0x73,
0x61, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0xd3, 0x72, 0x61, 0x20, 0x4b,
0x61, 0x6a, 0x61, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x52,
0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b,
0x61, 0x6a, 0x61, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x52,
0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b,
0x6f, 0x72, 0xe9, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4b, 0x6f, 0x72, 0xe9, 0x6a, 0x61, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x6f,
0x72, 0xe9, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x6f, 0x73, 0x69, 0x72, 0x61, 0x6a, 0x65, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x6e, 0x68, 0x6e, 0xe3,
0x6a, 0x61, 0x72, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4b, 0x72, 0x61, 0x6e, 0x68, 0x6e, 0xe3, 0x6a, 0x61,
0x72, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x6e, 0x68, 0x6e, 0xe3, 0x6a,
0x61, 0x72, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b,
0x69, 0x72, 0x67, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x52, 0x1ef9, 0x6e, 0x6b, 0x61, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x56, 0xe3, 0x66, 0x65, 0x20, 0x47, 0x6f, 0x6a,
0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x52, 0xf3, 0x72, 0x6e, 0x69, 0x20, 0x48, 0x6f, 0x76,
0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x52,
0xf3, 0x72, 0x6e, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x65, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x52, 0xf3,
0x72, 0x6e, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x65, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x6b, 0x61, 0x76, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d, 0x1ef9, 0x6b, 0x61, 0x76, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x6b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x4d, 0x1ef9, 0x6b, 0x61, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x4d, 0x1ef9, 0x67, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d, 0x1ef9, 0x67, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x4d, 0x1ef9, 0x67, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x72, 0x61, 0x6a, 0x61, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x20, 0x56, 0x1ebd, 0x73, 0x6f,
0x20, 0x4d, 0x1ef9, 0x72, 0x6e, 0x69, 0x76, 0x61, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x72, 0x6b, 0x65, 0x6a, 0x61, 0x20, 0x46,
0x61, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x72,
0x53, 0x61, 0x72, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0xe3, 0x72, 0x69, 0x73,
0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d,
0xe3, 0x72, 0x69, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0xe3, 0x72, 0x69, 0x73, 0x6f,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x75, 0x73, 0xe3,
0x6e, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ebd, 0x73, 0x69,
0x6b, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d, 0x1ebd, 0x73,
0x69, 0x6b, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x4d, 0x1ebd, 0x73, 0x69, 0x6b, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x55,
0x72, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x55, 0x72, 0x61, 0x6e, 0x20,
0x4d, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x55, 0x72, 0x61, 0x6e, 0x20, 0x4d,
0x61, 0x74, 0x6f, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d,
0xe3, 0x6e, 0x68, 0x6b, 0x6f, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x4d, 0xe3, 0x6e, 0x68, 0x6b, 0x6f, 0x76, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x4d, 0xe3, 0x6e, 0x68, 0x6b, 0x6f, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x4d, 0x129, 0x6a, 0x1ef9, 0x6d, 0x1ef9, 0x72, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x4e, 0x1ef9, 0x76, 0x75, 0x72, 0x75, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4e, 0x1ebd, 0x70, 0x61, 0x72, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e, 0x6f, 0x6e,
0x129, 0x6a, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e, 0x6f, 0x6e,
0x129, 0x6a, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x61, 0x72, 0x65,
0x6e, 0x6f, 0x6e, 0x129, 0x6a, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4a, 0x65, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a,
0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4a, 0x65, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20,
0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x4a, 0x65, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61,
0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47,
0x61, 0x20, 0x74, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x47, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x47, 0x61,
0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4e,
0x129, 0x76, 0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4e, 0xe3,
0x72, 0x66, 0x6f, 0x72, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76,
0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x4e, 0xe3, 0x72, 0x66, 0x6f, 0x72, 0x6b, 0x69, 0x20, 0x47, 0x6f,
0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4e, 0xe3, 0x72, 0x66, 0x6f,
0x72, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x1ef9,
0x6e, 0x75, 0x20, 0x4e, 0xe3, 0x72, 0xe3, 0x6a, 0x61, 0x2d, 0x74, 0xe1,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x1ef9, 0x6e, 0x75, 0x20, 0x4e, 0xe3, 0x72, 0xe3, 0x6a, 0x61,
0x2d, 0x74, 0xe1, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x1ef9, 0x6e, 0x75, 0x20,
0x4e, 0xe3, 0x72, 0xe3, 0x6a, 0x61, 0x2d, 0x74, 0xe1, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x4e, 0xe3, 0x72, 0x74, 0x69, 0x2d, 0x4d, 0x1ef9,
0x72, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76,
0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x69,
0x6d, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x54, 0xe3, 0x67, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x69, 0x6d,
0x69, 0x72, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x53, 0x69, 0x6d, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x69, 0x20,
0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4f, 0x6d,
0x129, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4f, 0x6d, 0x129, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4f,
0x6d, 0x129, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x6e, 0x68,
0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x61,
0x72, 0x61, 0x76, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0x61, 0x70, 0x75, 0x76, 0x61, 0x2d, 0x47, 0x69, 0x6e, 0x1ebd, 0x20, 0x54,
0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x76, 0x61, 0x6a, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x76, 0x61,
0x6a, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x76, 0x61, 0x6a, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x69, 0x72, 0x75, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50, 0x69, 0x72, 0x75,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x50, 0x69, 0x72, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x46, 0x69, 0x72, 0x69, 0x70, 0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x46, 0x69, 0x72, 0x69, 0x70, 0x69,
0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0x46,
0x69, 0x72, 0x69, 0x70, 0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x46, 0x1ebd, 0x6e, 0x129, 0x67, 0x20, 0x47, 0x6f, 0x6a, 0x2d,
0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53,
0x1ef9, 0x20, 0x50, 0x65, 0x6e, 0x72, 0x75, 0x20, 0x6b, 0x61, 0x72, 0x20,
0x4d, 0x129, 0x6b, 0x65, 0x72, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x1ef9, 0x20, 0x50, 0x65, 0x64, 0x72,
0x6f, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x4d, 0x129, 0x6b, 0x65, 0x72, 0x1ef9,
0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x53, 0x1ef9, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x6b,
0x61, 0x72, 0x20, 0x4d, 0x129, 0x6b, 0x65, 0x72, 0x1ef9, 0x76, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x69, 0x6e, 0x6b, 0x61, 0x6a, 0x69,
0x72, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x1ef9, 0x70, 0x65, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x69, 0xe3, 0x67, 0x69, 0x1ef9,
0x6e, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x79, 0x73,
0x79, 0x72, 0x6f, 0x72, 0x6e, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x4b, 0x79, 0x73, 0x79, 0x72, 0x6f, 0x72, 0x6e,
0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x4b, 0x79, 0x73, 0x79, 0x72, 0x6f, 0x72, 0x6e, 0x61, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x48, 0x75, 0x6a, 0xe1, 0x76, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x72,
0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x53, 0x61,
0x6b, 0x61, 0x72, 0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x53, 0x61, 0x6d, 0x1ef9, 0x72, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x61, 0x6d, 0x1ef9, 0x72, 0x61, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x53, 0x61, 0x6d, 0x1ef9, 0x72, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x53, 0x61, 0x6d, 0xe3, 0x76, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x61, 0x6d, 0xe3, 0x76, 0x61, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x53, 0x61, 0x6d, 0xe3, 0x76, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x53, 0x65, 0x6a, 0x73, 0x65, 0x72, 0x65, 0x6e, 0x68, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x129, 0x67, 0x61,
0x70, 0x75, 0x72, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53,
0x61, 0x72, 0x6f, 0x6d, 0x1ef9, 0x76, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76,
0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4a, 0x69,
0xf3, 0x72, 0x6a, 0x61, 0x20, 0x74, 0x1ef9, 0x20, 0x53, 0x75, 0x72, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x1ef9,
0x6d, 0x129, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x69, 0x6f,
0x76, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0x61, 0x6a,
0x74, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0x61, 0x6a,
0x70, 0x65, 0x6a, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x54, 0x61, 0x6a, 0x70,
0x65, 0x6a, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0x61, 0x6a,
0x69, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x72, 0x61, 0x76, 0x75, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x61, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x54, 0xe3, 0x67, 0x61,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x54, 0xe3, 0x67, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x53, 0x75, 0x75, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6f, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6e, 0x68, 0x74,
0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6f, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6e, 0x68, 0x74,
0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6f, 0x6d, 0x1ebd, 0x6e, 0x129,
0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x54, 0x75, 0x76, 0x61, 0x72, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x76, 0x61, 0x6a, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x76, 0x61, 0x6a, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x76, 0x61, 0x6a,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x55, 0x6e, 0x68, 0x6d, 0x65,
0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x55, 0x6e, 0x6e, 0x6d, 0x65, 0x6b, 0x69,
0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x55, 0x6e, 0x68, 0x6d, 0x65, 0x6b,
0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x56, 0x61, 0x6e, 0x169, 0x76, 0x61, 0x74, 0x75, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x56, 0x61, 0x6e, 0x169, 0x76,
0x61, 0x74, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x169, 0x76, 0x61, 0x74, 0x75,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x65, 0x6e, 0x1ebd, 0x6a,
0x75, 0x76, 0xe9, 0x72, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x55, 0x72, 0x61, 0x6e, 0x69, 0x76, 0xf3, 0x6e, 0x68, 0x74, 0xf3, 0x6b,
0x69, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x55, 0x72, 0x61, 0x6e, 0x69, 0x76, 0xf3, 0x6e, 0x68, 0x74, 0xf3, 0x6b,
0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x55, 0x72, 0x61, 0x6e, 0x69, 0x76, 0xf3, 0x6e, 0x68, 0x74,
0xf3, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x6f,
0x72, 0x67, 0x75, 0x67, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x56, 0x6f, 0x72, 0x67, 0x75, 0x67,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x56, 0x6f, 0x72, 0x67, 0x75, 0x67, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x6f,
0x6e, 0x68, 0x74, 0xf3, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x56, 0x65, 0x6a, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76,
0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x61,
0x72, 0x69, 0x6e, 0x68, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x59, 0x6a,
0x61, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x59, 0x6a, 0x61, 0x6b, 0x75,
0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x59, 0x6a, 0x61, 0x6b,
0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x6e, 0x6d, 0x75,
0x72, 0x67, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x6e, 0x6d, 0x75, 0x72,
0x67, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x6e, 0x6d,
0x75, 0x72, 0x67, 0x75, 0x20, 0x74, 0xe1, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x41, 0x63, 0x72, 0x65, 0x20, 0x79, 0x61, 0x72,
0x61, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x20, 0x41, 0x63, 0x72, 0x65, 0x20, 0x79, 0x61,
0x72, 0x61, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x41, 0x63,
0x72, 0x65, 0x20, 0x79, 0x61, 0x72, 0x61, 0x41, 0x66, 0x65, 0x67, 0xe3,
0x6e, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x66, 0x69, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x66, 0x69, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41,
0x66, 0x69, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x75, 0x20, 0x73, 0x75,
0xed, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x66,
0x69, 0x72, 0x69, 0x6b, 0x69, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd,
0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x41, 0x66, 0x69, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x55,
0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x41, 0x66, 0x69, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x55, 0x73,
0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6c, 0x61,
0x73, 0x69, 0x6b, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x45, 0x74, 0xe9, 0x41, 0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x45, 0x74, 0xe9, 0x41, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6d, 0x61, 0x73,
0x75, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x41, 0x6d, 0x61, 0x73, 0x75, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x45, 0x74, 0xe9, 0x41, 0x6d, 0x61, 0x73,
0x75, 0x6e, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0xd3,
0x72, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x20, 0x50,
0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77,
0x61, 0x72, 0x61, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x4c, 0x65, 0x73, 0x69, 0x74, 0x69, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x20, 0x4c, 0x65,
0x73, 0x69, 0x74, 0x69, 0x20, 0x79, 0x61, 0x72, 0x61, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x4c, 0x65, 0x73, 0x69, 0x74, 0x69, 0xd3,
0x72, 0x61, 0x20, 0x4b, 0x72, 0x129, 0x2d, 0x61, 0x67, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x72, 0x129, 0x2d, 0x61,
0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x6b, 0xe3, 0x20, 0xd3, 0x72,
0x61, 0x20, 0x4b, 0x72, 0x129, 0x2d, 0x61, 0x67, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x75, 0x20, 0x74, 0xe1, 0x41, 0x6e, 0x61, 0x64, 0xed, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6e, 0x61, 0x64, 0xed,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x41, 0x6e, 0x61, 0x64, 0xed, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x70, 0x69, 0x79, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x70, 0x69, 0x79, 0x61,
0x20, 0x55, 0x72, 0x61, 0x72, 0x75, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x41, 0x70, 0x69, 0x79, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6b, 0x69, 0x74,
0x61, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x6b, 0x69, 0x74, 0x61, 0x75,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6b, 0x69, 0x74,
0x75, 0x62, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x41, 0x6b, 0x69, 0x74, 0x75, 0x62, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41,
0x6b, 0x69, 0x74, 0x75, 0x62, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x79, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x79,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x79, 0x61,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0x67, 0x1ebd,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x41, 0x72, 0x67, 0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x41, 0x72, 0x67, 0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0x67, 0x1ebd, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72,
0x67, 0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd,
0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x72,
0x67, 0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd,
0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x41, 0x72, 0x69, 0x6d, 0x1ebd, 0x6e, 0x69, 0x79, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0x69, 0x6d,
0x1ebd, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x72, 0x69,
0x6d, 0x1ebd, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x41, 0x74, 0x61, 0x6c, 0xe3, 0x74, 0x69, 0x6b, 0x75, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x74, 0x61, 0x6c,
0xe3, 0x74, 0x69, 0x6b, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x74, 0x61,
0x6c, 0xe3, 0x74, 0x6b, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73,
0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69,
0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74,
0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65,
0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x2d, 0x55, 0x73, 0x69, 0x64, 0x1ebd,
0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61,
0x72, 0x61, 0x2d, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72,
0x61, 0x77, 0x61, 0x72, 0x61, 0x2d, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74,
0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73,
0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x55, 0x72,
0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74,
0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64,
0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61,
0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x75, 0x73, 0x69,
0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69,
0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x73, 0x65, 0x72, 0x69, 0x72, 0x65, 0x74,
0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x41, 0x73, 0x65, 0x72, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x41, 0x73, 0x65, 0x72, 0x69, 0x72, 0x65, 0x74, 0xe3,
0x6d, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x73,
0x6f, 0x72, 0x69, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x73, 0x6f, 0x72, 0x69, 0x2d, 0x69, 0x74,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x73, 0x6f, 0x72, 0x69, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x42, 0xe3,
0x6b, 0x61, 0x72, 0x61, 0x64, 0x65, 0x78, 0x69, 0x20, 0x48, 0x6f, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x42, 0xe3, 0x6b, 0x61, 0x72, 0x61, 0x64,
0x65, 0x78, 0x69, 0x20, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x42, 0xe3, 0x6b, 0x61, 0x72,
0x61, 0x64, 0x65, 0x78, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x42, 0x75, 0x74, 0xe3, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x42, 0x75, 0x72, 0xed, 0x77, 0x69, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0xd3, 0x72, 0x61, 0x20, 0x42,
0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69,
0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xd3, 0x72,
0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x74,
0xe1, 0x42, 0x75, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72,
0x75, 0x73, 0x61, 0x72, 0x61, 0x6d, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4b, 0x61, 0x62, 0x75, 0x20, 0x53, 0x75, 0x69, 0x6b,
0x69, 0x72, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x61, 0x62, 0x75, 0x20, 0x53, 0x75, 0x69, 0x6b, 0x69, 0x72, 0x69,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x62, 0x75, 0x20, 0x53, 0x75, 0x69,
0x6b, 0x69, 0x72, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x58, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x58, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x58, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x58, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x58, 0x69, 0x72, 0x69, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x58, 0x69, 0x72, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x58, 0x69, 0x72, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x58, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x58, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x58, 0x69, 0x6e, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x4b, 0x69, 0x72, 0x69, 0x74,
0x69, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x75,
0x6b, 0x75, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4b, 0x75, 0x72, 0x169, 0x62, 0x69, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x72, 0x169, 0x62,
0x69, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x75, 0x72, 0x169, 0x62, 0x69,
0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x6b, 0x69,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x6b, 0x69,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74,
0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75,
0x62, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b,
0x75, 0x62, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x75, 0x62, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x44, 0x61, 0x77, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x69, 0x6d,
0x75, 0x2d, 0x53, 0x65, 0x6d, 0x75, 0x73, 0x61, 0x77, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x73, 0x69, 0x6b,
0x75, 0x77, 0x61, 0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x73, 0x69,
0x6b, 0x75, 0x77, 0x61, 0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x50, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x77, 0x61,
0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x45, 0x6b, 0x75, 0x61, 0x64, 0xfa, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70,
0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72,
0x75, 0x70, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61,
0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70,
0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72,
0x75, 0x70, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61,
0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70,
0x61, 0x20, 0x4c, 0x65, 0x73, 0x69, 0x74, 0x69, 0x2d, 0x65, 0x74, 0xe9,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72,
0x75, 0x70, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77,
0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd,
0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x45, 0x75,
0x72, 0x75, 0x70, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61,
0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61,
0x77, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d,
0x61, 0x77, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x69, 0x6e,
0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x69, 0x79,
0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x69,
0x79, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x46, 0x69, 0x79, 0x69, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x69, 0x79, 0x61, 0x6e, 0x61,
0x20, 0x46, 0x72, 0xe3, 0x73, 0x61, 0x20, 0x79, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x65, 0x74, 0xe3,
0x6d, 0x61, 0x20, 0x53, 0x75, 0x20, 0x73, 0x75, 0xed, 0x20, 0x46, 0x72,
0xe3, 0x73, 0x61, 0x20, 0x79, 0x61, 0x72, 0x61, 0x20, 0x61, 0x73, 0x75,
0xed, 0x20, 0xc3, 0x74, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x61, 0x72, 0x61, 0x70,
0x61, 0x67, 0x75, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x47, 0xe3, 0x62, 0x69, 0x65, 0x72, 0x65, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x65, 0x75, 0x67,
0x69, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x47, 0x65, 0x75, 0x67, 0x69, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x47,
0x65, 0x75, 0x67, 0x69, 0x79, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20,
0x59, 0x75, 0x62, 0x65, 0x74, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x20, 0x4d, 0x69, 0x72, 0x69, 0x64, 0x69, 0x79, 0x61, 0x6e, 0x75, 0x20,
0x79, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x47, 0x75, 0x72, 0x75, 0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61,
0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x75, 0x72, 0x75,
0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd,
0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x47, 0x75,
0x72, 0x75, 0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x55, 0x72,
0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x47, 0x75, 0x72, 0x75, 0x1ebd, 0x72, 0xe3, 0x64,
0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77,
0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x47, 0x75, 0x72, 0x75, 0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20,
0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x47, 0x75, 0x72, 0x75, 0x1ebd, 0x72, 0xe3, 0x64, 0x69,
0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x75,
0x77, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x69, 0x79, 0x61,
0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x48,
0x61, 0x77, 0x61, 0xed, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4b, 0x61,
0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x75,
0x74, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x48, 0x61, 0x77, 0x61, 0xed, 0x20, 0x61, 0x73, 0x75,
0xed, 0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20,
0x41, 0x72, 0x65, 0x75, 0x74, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x48, 0x61, 0x77, 0x61, 0xed, 0x20, 0x61, 0x73, 0x75, 0xed,
0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x41,
0x72, 0x65, 0x75, 0x74, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x48, 0x169, 0x67, 0x20, 0x4b, 0x169, 0x67,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x48, 0x169, 0x67,
0x20, 0x4b, 0x169, 0x67, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x48, 0x169, 0x67, 0x20,
0x4b, 0x169, 0x67, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x48,
0x6f, 0x77, 0x69, 0x64, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x48, 0x6f, 0x77, 0x69, 0x64, 0x69, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x48, 0x6f, 0x77, 0x69, 0x64, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x128, 0x64, 0x69, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x55, 0x73,
0x65, 0x79, 0x61, 0x6e, 0x75, 0x20, 0x128, 0x64, 0x69, 0x6b, 0x75, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x128, 0x64, 0x6f, 0x78,
0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x128, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69,
0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x128, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x79, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x128, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64,
0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x49, 0x72, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x49, 0x72, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x49,
0x72, 0xe3, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x73,
0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x49, 0x73, 0x69, 0x72, 0x61, 0x79, 0x65, 0x75, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x73, 0x69, 0x72, 0x61, 0x79,
0x65, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x49, 0x73, 0x69, 0x72, 0x61, 0x79,
0x65, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4e, 0x69,
0x70, 0xf5, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4e,
0x69, 0x70, 0xf5, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4e, 0x69, 0x70, 0xf5, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x69, 0x72, 0x65,
0x74, 0xe3, 0x6d, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77,
0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x61, 0x73, 0x61, 0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61,
0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x72,
0x65, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x75, 0x72, 0x65, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x75,
0x72, 0x65, 0x79, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x75, 0x73, 0x69, 0x72, 0x61, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x69, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x69, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4b, 0x69, 0x72, 0x69, 0x67, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x52, 0xe3,
0x6b, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b,
0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x49, 0x6e, 0x129,
0x62, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4c,
0x75, 0x72, 0x75, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4c, 0x75, 0x72, 0x75, 0x64,
0x69, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4c, 0x75,
0x72, 0x75, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x6b, 0x61, 0x75,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe3, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x67, 0x61,
0x64, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe3,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x6c, 0x61,
0x73, 0x69, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d,
0x61, 0x75, 0x64, 0x69, 0x77, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x6b, 0x65, 0x73,
0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69,
0x79, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x79, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x79, 0x75, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x75, 0x73, 0x169, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61,
0x6e, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x55, 0x72, 0xe3, 0x20, 0x42,
0x61, 0x74, 0x75, 0x72, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x55, 0x72, 0xe3, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72, 0x75,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x55, 0x72, 0xe3, 0x20, 0x42, 0x61, 0x74, 0x75,
0x72, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4d,
0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4d, 0x69, 0x79, 0x61, 0x6d, 0xe1, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4e, 0x65, 0x70, 0x61, 0x75, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x72, 0x65,
0x64, 0x169, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73,
0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61,
0x72, 0x65, 0x64, 0x169, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x73,
0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x72, 0x65, 0x64,
0x169, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x65, 0x72, 0xe3,
0x64, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x65, 0x72, 0xe3,
0x64, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x53, 0x65, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20,
0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x49, 0x77, 0xed, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x77, 0xed,
0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x49,
0x77, 0xed, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4e, 0x69, 0x77, 0x65, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x46, 0x65, 0x6e, 0xe3, 0x64, 0x75, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x79,
0xe3, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x65,
0x6e, 0xe3, 0x64, 0x75, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x79, 0xe3, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x46, 0x65, 0x6e, 0xe3, 0x64, 0x75, 0x20, 0x4e, 0x75,
0x72, 0x75, 0x79, 0xe3, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61,
0x72, 0x69, 0x79, 0xe3, 0x6e, 0x61, 0x20, 0x4e, 0x75, 0x74, 0x69, 0x20,
0x73, 0x75, 0xed, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x53, 0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x50,
0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x53, 0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x69, 0x6b, 0x69,
0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53,
0x69, 0x62, 0x69, 0x72, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x50, 0x69,
0x73, 0x61, 0x73, 0xfa, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x4f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x4f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x6b, 0x69, 0x72, 0x65,
0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x50, 0x61, 0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x50, 0x61, 0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d,
0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x72,
0x61, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50,
0x61, 0x70, 0x75, 0x77, 0x61, 0x2d, 0x47, 0x69, 0x6e, 0xe9, 0x20, 0x50,
0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x77, 0x61, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x77, 0x61, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x77, 0x61, 0x69, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x50, 0x65, 0x72, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x50, 0x65, 0x72, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x50, 0x65,
0x72, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x69,
0x72, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x46, 0x69, 0x72, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x46, 0x69, 0x72, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3,
0x2d, 0x69, 0x74, 0x61, 0x20, 0x46, 0x1ebd, 0x6e, 0x69, 0x78, 0x53, 0xe3,
0x20, 0x50, 0x65, 0x64, 0x75, 0x72, 0x75, 0x20, 0x61, 0x73, 0x75, 0xed,
0x20, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0xe3, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0xe3, 0x20, 0x50, 0x65, 0x64, 0x75,
0x72, 0x75, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b, 0x69,
0x72, 0xe3, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53, 0xe3, 0x20, 0x50, 0x65,
0x64, 0x75, 0x72, 0x75, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4d, 0x69,
0x6b, 0x69, 0x72, 0xe3, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x69, 0xfa,
0x67, 0x69, 0xe3, 0x67, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x4b, 0x69, 0x73, 0x69, 0x72, 0x6f, 0x64, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x69, 0x73, 0x69, 0x72,
0x6f, 0x64, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x69, 0x73, 0x69, 0x72,
0x6f, 0x64, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x59,
0x75, 0x6d, 0x75, 0x61, 0x74, 0x69, 0x72, 0x69, 0x73, 0x61, 0x77, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x52, 0x6f, 0x74,
0x65, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x53, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x6e,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x6e,
0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x53, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x53, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x53, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x53, 0x65, 0x69, 0x78, 0x65, 0x72, 0x69, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x129, 0x67, 0x61, 0x70,
0x75, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x70, 0x75, 0xe3,
0x6d, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x53, 0x61, 0x72, 0x75, 0x6d,
0x169, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x65,
0x75, 0x67, 0x69, 0x79, 0x61, 0x20, 0x53, 0x75, 0x20, 0x73, 0x75, 0xed,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x53, 0x69, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x61, 0x69, 0x74, 0x69, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x61,
0x79, 0x69, 0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x6f, 0x6b, 0x65, 0x72,
0x61, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54,
0xf5, 0x67, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x54, 0xf5, 0x67, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x54, 0xf5, 0x67, 0x61,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x43, 0x68, 0x75, 0x75,
0x6b, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x75,
0x72, 0x6b, 0x75, 0x72, 0x61, 0x6e, 0x61, 0x72, 0x65, 0x74, 0xe3, 0x6d,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x75,
0x72, 0x6b, 0x75, 0x72, 0x61, 0x6e, 0x61, 0x72, 0x65, 0x74, 0xe3, 0x6d,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x54, 0x75, 0x72, 0x6b, 0x75, 0x72, 0x61,
0x6e, 0x61, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x54, 0x75, 0x76, 0x61, 0x72, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x55, 0x72, 0x75, 0x67, 0x75,
0x77, 0x61, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x55, 0x72, 0x75, 0x67, 0x75, 0x77, 0x61, 0x69, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x55, 0x72, 0x75, 0x67, 0x75, 0x77, 0x61, 0x69, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x59, 0x169, 0x62, 0x75, 0x65, 0x73, 0x61, 0x72,
0x61, 0x2d, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x59, 0x169, 0x62, 0x75, 0x65, 0x73, 0x61,
0x72, 0x61, 0x2d, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x59, 0x169, 0x62, 0x75, 0x65, 0x73, 0x61, 0x72, 0x61, 0x2d, 0x72,
0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x57, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x57, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x65, 0x6e, 0x1ebd, 0x73,
0x75, 0x65, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x57, 0x61, 0x72, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x69, 0x74,
0x75, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x57, 0x61, 0x72, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x69, 0x74, 0x75,
0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x57, 0x61, 0x72, 0x61, 0x64, 0x69,
0x77, 0x6f, 0x73, 0x69, 0x74, 0x75, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x57, 0x6f, 0x72, 0x77, 0x6f, 0x67, 0x61, 0x72,
0x61, 0x64, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x57, 0x6f, 0x72, 0x77, 0x6f, 0x67, 0x61, 0x72, 0x61, 0x64, 0x75, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x57, 0x6f, 0x72, 0x77, 0x6f, 0x67, 0x61, 0x72, 0x61,
0x64, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x6f,
0x73, 0x69, 0x74, 0x75, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x57, 0x61, 0x72, 0x69, 0x20, 0x61, 0x73,
0x75, 0xed, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x69,
0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x129, 0x62, 0x75, 0x67, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129,
0x62, 0x75, 0x67, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x129, 0x62, 0x75, 0x67, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0xe3, 0x20, 0x50, 0x65, 0x64, 0x75,
0x72, 0x75, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b, 0x69,
0x72, 0x61, 0x6e, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x53, 0xe3, 0x20, 0x50, 0x65, 0x64, 0x75, 0x72, 0x75, 0x20, 0x61, 0x73,
0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0x61, 0x6e, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x53, 0xe3, 0x20, 0x50, 0x65, 0x64, 0x75, 0x72, 0x75, 0x20,
0x61, 0x73, 0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0x61, 0x6e,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x66, 0x67, 0x68,
0x61, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55,
0x61, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x53, 0xfc, 0xfc, 0x64, 0x65, 0x6c, 0x6b, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61,
0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69,
0x6b, 0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b,
0x6f, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74,
0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65, 0x65,
0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20,
0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74,
0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73,
0x74, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x69, 0x66,
0x69, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x6d, 0x65, 0x65, 0x72,
0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x69, 0x66, 0x69,
0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f,
0x20, 0x50, 0x61, 0x73, 0x69, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x70, 0x69, 0x61, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x41, 0x70, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x70,
0x69, 0x61, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x41, 0x72, 0x61, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x41, 0x72, 0x61, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41,
0x72, 0x61, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x61,
0x61, 0x73, 0x74, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69,
0x6e, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61,
0x73, 0x74, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e,
0x69, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x6d,
0x65, 0x65, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41,
0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72,
0x6d, 0x65, 0x65, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b,
0x6f, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f,
0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73,
0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x57, 0x61, 0x61, 0x73, 0x74, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20,
0x57, 0x61, 0x61, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x20, 0x57, 0x61, 0x61, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x61,
0x61, 0x73, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73,
0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73,
0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61,
0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x73, 0x63, 0x68,
0x61, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69,
0x64, 0x73, 0x63, 0x68, 0x61, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x74, 0x73, 0x6f, 0x6f, 0x72, 0x65,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x74, 0x73, 0x6f, 0x6f, 0x72,
0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x41, 0x74, 0x73, 0x6f, 0x6f, 0x72, 0x65, 0x6e,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63,
0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x63, 0x68, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x42, 0x68, 0x75, 0x75, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x42, 0x6f, 0x6c, 0x69, 0x69, 0x77, 0x69, 0x65, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69, 0x6c, 0x69, 0x65,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69,
0x6c, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69,
0x6c, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72,
0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x4b, 0x61, 0x70, 0x77, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4b, 0x61, 0x70, 0x77, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x4b, 0x61, 0x70, 0x77, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x43, 0x68, 0x69, 0x69, 0x6c, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43,
0x68, 0x69, 0x69, 0x6c, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x69, 0x69, 0x6c,
0x65, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x53,
0x63, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53,
0x63, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x63, 0x68, 0x69,
0x69, 0x6e, 0x61, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x4a, 0x75, 0x6c, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x45, 0x69, 0x6c,
0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x45, 0x69,
0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e,
0x20, 0x48, 0x75, 0x61, 0x6c, 0x65, 0x77, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x75,
0x75, 0x62, 0x61, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x44, 0x61, 0x76, 0x69, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74,
0x20, 0x54, 0x69, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x50, 0x75, 0x61, 0x73, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x50, 0x75, 0x61, 0x73, 0x6b, 0x20, 0x45, 0x69,
0x6c, 0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x75, 0x61, 0x73, 0x6b, 0x20, 0x45,
0x69, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x45, 0x6b, 0x77, 0x61, 0x64, 0x6f, 0x6f, 0x72, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4d, 0x61, 0x64, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61,
0x64, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4d, 0x61, 0x64, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x65, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x65, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55,
0x61, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x46, 0x61,
0x61, 0x72, 0x64, 0x65, 0x72, 0x20, 0x55, 0x61, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x57, 0x61, 0x61, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65,
0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57,
0x61, 0x61, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73,
0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x75, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e,
0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x75, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x75, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x75,
0x6e, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x69,
0x64, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x46, 0x72, 0x61, 0x6e, 0x73, 0xf6, 0xf6, 0x73, 0x6b,
0x20, 0x47, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x46, 0x72, 0x61, 0x6e, 0x73, 0xf6, 0xf6, 0x73, 0x6b, 0x20,
0x53, 0xfc, 0xfc, 0x64, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61,
0x72, 0x6b, 0x74, 0x69, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x47, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x4d, 0x61, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73,
0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x75, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x47, 0x6f, 0x6c, 0x66, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x47, 0x75, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68,
0x6f, 0x77, 0x64, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x4f, 0x6f, 0x73, 0x65, 0x61,
0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x49, 0x6e, 0x64, 0x6f, 0x73,
0x63, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x65, 0x73, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x55, 0x61, 0x73, 0x74, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65,
0x73, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61,
0x73, 0x74, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x69,
0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x49, 0x72, 0x61, 0x61, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x49, 0x72, 0x61, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x4a, 0x61, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4a, 0x61, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a,
0x61, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x4b, 0x61, 0x73, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20,
0x4b, 0x61, 0x73, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x4b, 0x61,
0x73, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4b,
0x69, 0x72, 0x67, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e,
0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69,
0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69,
0x69, 0x77, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x45, 0x69, 0x6c, 0x75,
0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x50,
0x61, 0x73, 0x69, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4d, 0x6f,
0x73, 0x6b, 0x61, 0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x6f, 0x73,
0x6b, 0x61, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4d, 0x6a, 0x61,
0x6e, 0x6d, 0x61, 0x61, 0x72, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x61,
0x75, 0x72, 0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x65, 0x70,
0x61, 0x61, 0x6c, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x6b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x6b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x73, 0x69,
0x61, 0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69,
0x73, 0x69, 0x61, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x73,
0x69, 0x61, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x75,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e,
0x64, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e,
0x64, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4e, 0x6f, 0x77,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4f,
0x6d, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4f, 0x6d, 0x73, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x61,
0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x69, 0x67, 0x75, 0x69, 0x6e, 0x65,
0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x50,
0x65, 0x72, 0x75, 0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x65, 0x72,
0x75, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x50, 0x65, 0x72, 0x75, 0x75, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69,
0x78, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x45, 0x69,
0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x6f,
0x6e, 0x61, 0x70, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x6a, 0xf6,
0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x52,
0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53,
0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x61,
0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x53, 0x65, 0x73, 0x63, 0x68, 0x65, 0x6c, 0x65, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x75, 0x72, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6f, 0x6e,
0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0xfc, 0xfc, 0x64, 0x67,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x53, 0x75, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54,
0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x61,
0x69, 0x70, 0x65, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x61, 0x69,
0x70, 0x65, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x61, 0x69, 0x70, 0x65, 0x65, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x54, 0x61, 0x64,
0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x54, 0x75, 0x77, 0x61, 0x61,
0x6c, 0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x55, 0x73, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x57, 0x65, 0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x6c, 0x61, 0x64,
0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x6c, 0x61,
0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61,
0x6e, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x59,
0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x20, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x74, 0x65, 0x6e, 0x70, 0x6f, 0x20, 0x70, 0x69, 0x20,
0x6d, 0x61, 0x20, 0x4b, 0x65, 0x6e, 0x69, 0x47, 0x72, 0x69, 0x6e, 0x69,
0x73, 0x20, 0x4d, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x65, 0x6d, 0x627, 0x64e,
0x644, 0x627, 0x633, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x627, 0x64e, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc,
0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627,
0x64e, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x645, 0x627, 0x632,
0x6c6, 0x646, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x645, 0x627,
0x632, 0x6c6, 0x646, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af,
0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x645, 0x627, 0x632, 0x6c6,
0x646, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x627, 0x645, 0x631,
0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc,
0x627, 0x645, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2,
0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc,
0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc,
0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627,
0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651,
0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f,
0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627,
0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x6a9, 0x6c6, 0x647, 0x633, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x627,
0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x6a9, 0x6c6, 0x647, 0x633, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x627, 0x645, 0x631,
0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af,
0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6a9, 0x6c6, 0x647, 0x633, 0x62a,
0x6af, 0x6ce, 0x646, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x622, 0x631, 0x627, 0x645, 0x632, 0x650, 0x631, 0x6cc, 0x20, 0x627, 0x645, 0x631,
0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631,
0x627, 0x645, 0x632, 0x650, 0x631, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9,
0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631, 0x627, 0x645, 0x632, 0x650, 0x631,
0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x631,
0x62c, 0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x627, 0x631, 0x62c, 0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20,
0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x627, 0x631, 0x62c, 0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631,
0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x62a, 0x6cc,
0x646, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646,
0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x62a, 0x6cc, 0x646, 0x627,
0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627,
0x631, 0x62c, 0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627,
0x645, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627,
0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc,
0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2,
0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x200c,
0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646,
0x6cc, 0x627, 0x645, 0x200c, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627,
0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af,
0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x646, 0x6cc, 0x627, 0x645, 0x200c, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20,
0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631,
0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x64f, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634,
0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6,
0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af,
0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644,
0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x628, 0x631, 0x627, 0x632, 0x6cc,
0x644, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x626,
0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626,
0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20,
0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a,
0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645,
0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631,
0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc,
0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651,
0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f,
0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x62f, 0x6ce, 0x645, 0x62a, 0x631, 0x6cc, 0x20, 0x631, 0x6c6, 0x62f, 0x631, 0x627,
0x62a, 0x6a9, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af,
0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626,
0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc,
0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651,
0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646,
0x646, 0x62f, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6af, 0x631,
0x6cc, 0x646, 0x200c, 0x648, 0x650, 0x686, 0x20, 0x645, 0x6cc, 0x646, 0x20, 0x679,
0x627, 0x626, 0x645, 0x647, 0x626, 0x648, 0x627, 0x6cc, 0x6cc, 0x626, 0x6d2, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x647, 0x626, 0x648, 0x627, 0x6cc, 0x6cc, 0x626, 0x6d2,
0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x647, 0x626, 0x648, 0x627, 0x6cc, 0x6cc, 0x626, 0x6d2, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc,
0x627, 0x645, 0x6cc, 0x20, 0x627, 0x646, 0x62f, 0x6c6, 0x646, 0x6cc, 0x632, 0x6cc,
0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9,
0x6cc, 0x20, 0x627, 0x646, 0x62f, 0x6c6, 0x646, 0x6cc, 0x632, 0x6cc, 0x627, 0x626,
0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x646,
0x62f, 0x6c6, 0x646, 0x6cc, 0x632, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc,
0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627,
0x6cc, 0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x627, 0x6cc, 0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x626, 0x6d2,
0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x627, 0x6cc, 0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x626, 0x6d2,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x6a9, 0x627, 0x632,
0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651,
0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646,
0x646, 0x62f, 0x6cc, 0x20, 0x6a9, 0x627, 0x632, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x6a9, 0x631, 0x627, 0x646, 0x633, 0x646, 0x64f, 0x6cc,
0x627, 0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6a9,
0x631, 0x627, 0x646, 0x633, 0x646, 0x64f, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x626,
0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x6a9, 0x631, 0x627, 0x646, 0x633, 0x646, 0x64f, 0x6cc, 0x627,
0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x644, 0x6c6, 0x631, 0x688, 0x20, 0x647, 0x626,
0x648, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626,
0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x644, 0x6c6, 0x631, 0x688, 0x20, 0x647,
0x626, 0x648, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627,
0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x644, 0x6c6, 0x631, 0x688, 0x20, 0x647, 0x626, 0x648,
0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x645, 0x64e, 0x6af, 0x64e, 0x62f, 0x64e, 0x646, 0x626, 0x6d2, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x645, 0x64e, 0x6af, 0x64e, 0x62f, 0x64e, 0x646, 0x626, 0x6d2, 0x20,
0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x645, 0x64e, 0x6af, 0x64e, 0x62f, 0x64e, 0x646, 0x626, 0x6d2, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631,
0x627, 0x645, 0x632, 0x650, 0x631, 0x6cc, 0x20, 0x645, 0x650, 0x6a9, 0x633, 0x6cc,
0x6a9, 0x6c6, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631, 0x627,
0x645, 0x632, 0x650, 0x631, 0x6cc, 0x20, 0x645, 0x650, 0x6a9, 0x633, 0x6cc, 0x6a9,
0x6c6, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631, 0x627, 0x645, 0x632, 0x650, 0x631,
0x6cc, 0x20, 0x645, 0x650, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x6c6, 0x626, 0x6d2, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x645,
0x627, 0x633, 0x6a9, 0x6c6, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x645,
0x627, 0x633, 0x6a9, 0x6c6, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a,
0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x645, 0x627, 0x633, 0x6a9,
0x6c6, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x648, 0x641, 0x627, 0x648, 0x648, 0x646, 0x688,
0x644, 0x626, 0x6cc, 0x646, 0x688, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x646, 0x6cc, 0x648, 0x641, 0x627, 0x648, 0x648, 0x646, 0x688, 0x644, 0x626, 0x6cc,
0x646, 0x688, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce,
0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x648, 0x641, 0x627, 0x648,
0x648, 0x646, 0x688, 0x644, 0x626, 0x6cc, 0x646, 0x688, 0x626, 0x6d2, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x64f,
0x631, 0x64f, 0x646, 0x647, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x646, 0x64f, 0x631, 0x64f, 0x646, 0x647, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc,
0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646,
0x64f, 0x631, 0x64f, 0x646, 0x647, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6c6, 0x648, 0x6c6,
0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x646, 0x6c6, 0x648, 0x6c6, 0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633,
0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6c6, 0x648, 0x6c6, 0x633, 0x6cc, 0x628,
0x6cc, 0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af,
0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x6c6, 0x645, 0x633, 0x6a9, 0x626,
0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x6c6, 0x645, 0x633, 0x6a9, 0x626,
0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x627, 0x6c6, 0x645, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x648, 0x644,
0x627, 0x62f, 0x6cc, 0x648, 0x64f, 0x633, 0x62a, 0x64f, 0x6a9, 0x626, 0x6d2, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x648, 0x64f, 0x633,
0x62a, 0x64f, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af,
0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x6cc,
0x648, 0x64f, 0x633, 0x62a, 0x64f, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6cc, 0x627, 0x6a9, 0x648,
0x62a, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6cc, 0x627,
0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651,
0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6cc, 0x627, 0x6a9,
0x648, 0x62a, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af,
0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6cc, 0x6ce, 0x6a9, 0x627, 0x62a, 0x631,
0x6cc, 0x646, 0x628, 0x648, 0x631, 0x6af, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x6cc, 0x6ce, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631,
0x6af, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x6cc, 0x6ce, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc,
0x646, 0x628, 0x648, 0x631, 0x6af, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x41, 0x77, 0x67, 0xe1, 0x6e,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0x41, 0x70, 0x72, 0x69,
0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64,
0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x41, 0x70, 0x72, 0x69, 0x6b,
0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61,
0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x41, 0x70, 0x72, 0x69, 0x6b, 0xe1,
0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x41, 0x70,
0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52,
0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x41, 0x70, 0x72, 0x69, 0x6b,
0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x41,
0x70, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x6c, 0xe1, 0x73,
0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x6c, 0xe1,
0x73, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x6c, 0xe1, 0x73, 0x6b, 0xe1, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x41, 0x6d, 0xe1, 0x7a, 0xf3, 0x6e, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x41, 0x6d, 0xe1, 0x7a, 0xf3, 0x6e, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41,
0x6d, 0xe1, 0x7a, 0xf3, 0x6e, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67,
0xe1, 0x68, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69,
0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c,
0x67, 0xe1, 0x68, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20,
0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x41,
0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72,
0xe1, 0x74, 0x6b, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4b, 0xf3, 0x68, 0x65, 0x73, 0x74, 0x61, 0x67, 0x69, 0x20,
0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4b, 0xf3, 0x68, 0x65, 0x73, 0x74, 0x61, 0x67, 0x69, 0x20, 0x41,
0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0xf3, 0x68, 0x65, 0x73,
0x74, 0x61, 0x67, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65, 0x72, 0x69, 0x20, 0x41,
0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65, 0x72, 0x69, 0x20, 0x41, 0x6d, 0x72,
0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65, 0x72,
0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41,
0x70, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x70,
0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x41, 0x70, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x72,
0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x72, 0x61,
0x62, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x41, 0x72, 0x61, 0x62, 0x69, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x72, 0x6a, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x72, 0x6a, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e,
0x64, 0x69, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xe1,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e,
0x64, 0x69, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xe1,
0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x41, 0x72,
0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x72,
0x61, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72,
0x6d, 0x69, 0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0xc1, 0x72, 0x6d, 0x69, 0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0x6d,
0x69, 0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0xe1, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0xc1, 0x73,
0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0xc1, 0x73, 0x74,
0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1,
0x68, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x72, 0xf3, 0x6e,
0x65, 0x6e, 0x64, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69,
0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67,
0xe1, 0x68, 0x69, 0x72, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0xc1,
0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c,
0x67, 0xe1, 0x68, 0x69, 0x72, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20,
0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52,
0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0xc1, 0x73, 0x74,
0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0xc1, 0x73,
0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61,
0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c,
0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69,
0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20,
0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3,
0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c,
0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x7a, 0x65, 0x72, 0x62, 0xe1, 0x69,
0x6a, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x7a,
0x65, 0x72, 0x62, 0xe1, 0x69, 0x6a, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x7a,
0x65, 0x72, 0x62, 0xe1, 0x69, 0x6a, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41,
0x7a, 0xf3, 0x72, 0x65, 0x73, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x41, 0x7a, 0xf3, 0x72, 0x65, 0x73, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x7a, 0xf3, 0x72,
0x65, 0x73, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x61, 0x6c, 0x61,
0x64, 0xe9, 0x73, 0x68, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x42,
0x61, 0x6e, 0x67, 0x61, 0x6c, 0x61, 0x64, 0xe9, 0x73, 0x68, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x42, 0x61, 0x6e, 0x67, 0x61, 0x6c, 0x61, 0x64, 0xe9, 0x73, 0x68, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x42, 0x75, 0x74, 0x68, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x42, 0x6f, 0x6c, 0x69, 0x77, 0x69, 0xe1, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x42, 0x72, 0xe1, 0x7a, 0x69, 0x6c, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x42, 0x72, 0xe1, 0x7a, 0x69, 0x6c,
0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x42, 0x72, 0xe1, 0x7a, 0x69, 0x6c, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x42,
0x72, 0x75, 0x6e, 0xe1, 0x69, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4b, 0x61, 0x70, 0x20, 0x57, 0x61, 0x72, 0x64, 0xe9, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4b, 0x61, 0x70, 0x20, 0x57, 0x61, 0x72, 0x64,
0xe9, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x4b, 0x61, 0x70, 0x20, 0x57, 0x61, 0x72, 0x64, 0xe9,
0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0xf3, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x43, 0x68, 0x61, 0x74, 0x61, 0x6d,
0x20, 0x77, 0x61, 0x68, 0x64, 0x43, 0x68, 0x61, 0x74, 0x61, 0x6d, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x43,
0x68, 0x61, 0x74, 0x61, 0x6d, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x61, 0x79, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x61, 0x79, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x43, 0x68, 0x69, 0x6c, 0x61, 0x79, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x43, 0x68, 0x69,
0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x79, 0x20, 0x72, 0xf3,
0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x72, 0x65, 0x73,
0x6d, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x75, 0x6b, 0x75, 0x73, 0x20, 0x49,
0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69, 0xe1, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69, 0xe1, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4b, 0x75, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x75, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x75, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x6e, 0xe9, 0x6d, 0x2d, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x79,
0x75, 0x62, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x79,
0x75, 0x62, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x79, 0x75, 0x62, 0xe1, 0x61, 0x79,
0x20, 0x72, 0xf3, 0x63, 0x68, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x44, 0x61, 0x77, 0x69, 0x73, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x55, 0x72, 0x77, 0x69, 0x6c,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72,
0xe1, 0x74, 0x6b, 0x69, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x49, 0x73, 0x74, 0x68, 0x61, 0x72, 0x20,
0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x49, 0x73, 0x74, 0x68, 0x61, 0x72, 0x20, 0x49, 0x73, 0x6c, 0xe1,
0x6e, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x49, 0x73, 0x74, 0x68, 0x61, 0x72, 0x20, 0x49,
0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d,
0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x45, 0x6b, 0x77, 0xe1,
0x64, 0xf3, 0x72, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65,
0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68,
0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c,
0x67, 0xe1, 0x68, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x59,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52,
0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x59, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b,
0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0xe9,
0x6d, 0x74, 0x65, 0x72, 0xe9, 0x6e, 0x20, 0x52, 0xf3, 0x64, 0x61, 0x72,
0xe1, 0x74, 0x6b, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65,
0x6e, 0x64, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52,
0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x50, 0x61, 0x6c, 0x6b, 0x6c, 0xe1, 0x6e, 0x64, 0x20,
0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x50, 0x61, 0x6c, 0x6b, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x49, 0x73,
0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x61, 0x6c, 0x6b, 0x6c, 0xe1,
0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x46, 0x69, 0x6a, 0x69, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x46,
0x69, 0x6a, 0x69, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x50, 0x61, 0x72, 0xe1, 0x6e, 0x73, 0x69, 0x20, 0x47, 0x77, 0x69, 0x6e,
0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x5a, 0x65, 0x72, 0x62,
0xe1, 0x72, 0x69, 0x20, 0x50, 0x61, 0x72, 0x61, 0x6e, 0x73, 0x69, 0x20,
0x6f, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0xe1,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x72, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4a, 0xe1, 0x72, 0x6a, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4a, 0xe1, 0x72, 0x6a, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4a, 0xe1, 0x72,
0x6a, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x47, 0x65, 0x6c, 0x62, 0x61, 0x72,
0x74, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x47, 0x72, 0x69, 0x6e, 0x77, 0x65, 0x63, 0x68, 0x20,
0x4d, 0x69, 0x6e, 0x20, 0x57, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61,
0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0xe1,
0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64,
0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c,
0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74,
0x6b, 0x69, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x47, 0x72,
0x69, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x47, 0x72, 0x69,
0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e,
0x64, 0x69, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20,
0x47, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x57, 0x61, 0x68, 0x64,
0x4b, 0x68, 0x61, 0x6c, 0x69, 0x6a, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x47, 0x75, 0x79, 0xe1,
0x6e, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0x61, 0x77,
0xe1, 0x69, 0x2f, 0x41, 0x6c, 0x75, 0x73, 0x68, 0x69, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x48, 0x61, 0x77, 0xe1, 0x69, 0x2f, 0x41, 0x6c,
0x75, 0x73, 0x68, 0x69, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0x61, 0x77, 0xe1, 0x69, 0x2f,
0x41, 0x6c, 0x75, 0x73, 0x68, 0x69, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72,
0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0xe1, 0x6e,
0x67, 0x20, 0x4b, 0xe1, 0x6e, 0x67, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x48, 0xe1, 0x6e, 0x67, 0x20, 0x4b, 0xe1, 0x6e, 0x67, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x48, 0xe1, 0x6e, 0x67, 0x20, 0x4b, 0xe1, 0x6e, 0x67, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x48, 0xf3, 0x77, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48,
0xf3, 0x77, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0xf3, 0x77, 0x64, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x48, 0x65, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0x65, 0x6e, 0x64, 0x69, 0x20,
0x7a, 0x65, 0x72, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0x65,
0x6e, 0x64, 0xf3, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0x45, 0x6e,
0x64, 0x68, 0x6f, 0x6e, 0x69, 0x73, 0x68, 0x69, 0xe1, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52,
0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x45, 0x6e, 0x64,
0x68, 0x6f, 0x6e, 0x69, 0x73, 0x68, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3,
0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x45, 0x6e, 0x64, 0x68, 0x6f, 0x6e,
0x69, 0x73, 0x68, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc9, 0x72, 0xe1, 0x6e, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc9, 0x72, 0xe1, 0x6e, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0xc9, 0x72, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x45, 0x72, 0x6b, 0x75, 0x73, 0x6b, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x45, 0x72, 0x6b, 0x75, 0x73, 0x6b,
0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x45, 0x72, 0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x45,
0x73, 0x72, 0xe1, 0x69, 0x6c, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x45, 0x73, 0x72, 0xe1, 0x69, 0x6c, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x45, 0x73, 0x72, 0xe1,
0x69, 0x6c, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x4a, 0xe1, 0x70, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x4a, 0xe1, 0x70, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4a, 0xe1,
0x70, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4b, 0xe1, 0x7a, 0x61, 0x6b, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64,
0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x4b, 0xe1, 0x7a, 0x65, 0x6b,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e,
0x64, 0x69, 0x20, 0x4b, 0xe1, 0x7a, 0x65, 0x6b, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x4b, 0xf3, 0x72, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x4b, 0xf3, 0x72, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0xf3,
0x72, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4b, 0xf3, 0x73, 0x72, 0x61, 0x69, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0xf3, 0x79,
0xe1, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0xf3, 0x79, 0xe1, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0xf3, 0x79, 0xe1, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b,
0x61, 0x72, 0x67, 0x65, 0x7a, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4c, 0x69, 0x6e, 0xe9, 0x20, 0x49, 0x73,
0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1,
0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x2c, 0x20, 0x4c,
0xe1, 0x64, 0x68, 0x61, 0x77, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x2c, 0x20,
0x4c, 0xe1, 0x64, 0x68, 0x61, 0x77, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x73, 0x74, 0x72,
0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x2c, 0x20, 0x4c, 0xe1, 0x64, 0x68,
0x61, 0x77, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x67, 0x61, 0x64, 0xe1, 0x6e,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x67, 0x61, 0x64,
0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4d, 0x61, 0x6c, 0x69, 0x73, 0x68, 0x69, 0xe1, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x6c, 0x64, 0x69, 0x70, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0x61, 0x72, 0x6b, 0xe9, 0x73,
0xe1, 0x73, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0x75, 0x72, 0x69, 0x74, 0xe1,
0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0x75,
0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0x75, 0x72,
0x69, 0x74, 0xe1, 0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72,
0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72,
0xe1, 0x6d, 0x7a, 0x65, 0x72, 0x69, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0xf3, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0xe1,
0x6d, 0x7a, 0x65, 0x72, 0x69, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0xf3, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65, 0x72, 0x69, 0x20,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0xf3, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x55, 0x6c,
0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x55, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x55, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4d, 0xe1, 0x73, 0x6b, 0xf3, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4d, 0xe1, 0x73, 0x6b, 0xf3, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x73, 0x6b,
0xf3, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0xe1, 0x72, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0xe9, 0x70, 0xe1, 0x6c, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x79, 0x75, 0x20, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4e, 0x79, 0x75, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0xf3,
0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x79, 0x75, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0xf3, 0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72,
0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x69, 0x75,
0x20, 0x5a, 0x69, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4e, 0x69, 0x75, 0x20, 0x5a, 0x69, 0x6c, 0xe1, 0x6e, 0x64,
0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4e, 0x69, 0x75, 0x20, 0x5a, 0x69, 0x6c, 0xe1, 0x6e, 0x64,
0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4e, 0x69, 0x70, 0xe1, 0x6e, 0x64, 0x6c, 0x61, 0x79, 0x6e, 0x64,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x69, 0x70, 0xe1, 0x6e,
0x64, 0x6c, 0x61, 0x79, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x69, 0x70, 0xe1,
0x6e, 0x64, 0x6c, 0x61, 0x79, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x69,
0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x75, 0x72, 0x70,
0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x75, 0x72, 0x70, 0x6f, 0x6c, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x75, 0x72,
0x70, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61,
0x79, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x61, 0x77, 0xe1, 0x73, 0x69,
0x62, 0x69, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e,
0x61, 0x77, 0xe1, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e,
0x61, 0x77, 0xe1, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0xd3, 0x6d, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xd3,
0x6d, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0xd3, 0x6d, 0x73, 0x6b, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x50, 0xe1, 0x6b, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x50, 0xe1, 0x6b, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x50, 0xe1, 0x6b, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x50, 0x61, 0x6c, 0xe1, 0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x50, 0xe1, 0x70, 0x75, 0xe1, 0x20, 0x4e, 0x69, 0x75, 0x20, 0x47, 0x69,
0x6e, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x61,
0x72, 0x61, 0x67, 0xf3, 0x61, 0x79, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0xf3, 0x61, 0x79, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50,
0x61, 0x72, 0x61, 0x67, 0xf3, 0x61, 0x79, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0xe9,
0x72, 0xf3, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0xe9, 0x72,
0xf3, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x50, 0xe9, 0x72, 0xf3, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x65,
0x6c, 0x70, 0x69, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50,
0x65, 0x6c, 0x70, 0x69, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x65, 0x6c, 0x70, 0x69,
0x6e, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x65, 0x6b, 0x73,
0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x53, 0x74, 0x2e, 0x20, 0x50, 0xe9, 0x72, 0x69, 0x20, 0x6f,
0x20, 0x4d, 0x69, 0x6b, 0x77, 0xe9, 0x6c, 0x69, 0x6e, 0x20, 0x77, 0x61,
0x68, 0x64, 0x53, 0x74, 0x2e, 0x20, 0x50, 0xe9, 0x72, 0x69, 0x20, 0x6f,
0x20, 0x4d, 0x69, 0x6b, 0x77, 0xe9, 0x6c, 0x69, 0x6e, 0x20, 0x61, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0x74, 0x2e, 0x20,
0x50, 0xe9, 0x72, 0x69, 0x20, 0x6f, 0x20, 0x4d, 0x69, 0x6b, 0x77, 0xe9,
0x6c, 0x69, 0x6e, 0x20, 0x72, 0xf3, 0x63, 0x68, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x50, 0x69, 0x74, 0x6b, 0x61, 0x72, 0xe9, 0x6e, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0xf3, 0x6e, 0x70, 0x69, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0xe1,
0x6e, 0x67, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xe9, 0x79,
0x75, 0x6e, 0x69, 0x61, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x52, 0x6f, 0x74, 0x68, 0xe9, 0x72, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x53, 0x61, 0x6b, 0x68, 0xe1, 0x6c, 0x69, 0x6e, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0x61, 0x6b, 0x68, 0xe1, 0x6c, 0x69,
0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x53, 0x61, 0x6b, 0x68, 0xe1, 0x6c, 0x69, 0x6e, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x53, 0x61, 0x6d, 0xf3, 0xe1, 0x77, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x53, 0x61, 0x6d, 0xf3, 0xe1, 0x77, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53,
0x61, 0x6d, 0xf3, 0xe1, 0x77, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0xe9, 0x63, 0x68, 0x65, 0x6c,
0x65, 0x73, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0x65, 0x6e,
0x67, 0xe1, 0x70, 0x75, 0x72, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x53, 0x6f, 0x6c, 0x6f, 0x6d,
0xe1, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x5a, 0x65, 0x72, 0x62, 0xe1, 0x72, 0x69, 0x20,
0x4a, 0xe1, 0x72, 0x6a, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x79, 0x6d, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x53, 0x79, 0xf3, 0x77, 0xe1, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x54, 0xe1, 0x69, 0x70, 0x69, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x54, 0xe1, 0x69, 0x70, 0x69, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x54, 0xe1, 0x69, 0x70, 0x69, 0x61, 0x79, 0x20, 0x72, 0xf3, 0x63, 0x68,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54, 0xe1, 0x6a, 0x65, 0x6b, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54,
0x6f, 0x6b, 0x65, 0x6c, 0xe1, 0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x54, 0x6f, 0x6e, 0x67, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x54, 0x6f, 0x6e, 0x67, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54, 0x6f, 0x6e, 0x67,
0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x43, 0x68, 0x75, 0x6b, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x54, 0x6f, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x54, 0x6f,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x54, 0x6f, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x54, 0x75, 0x77, 0xe1, 0x6c, 0x75, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x59, 0x75, 0x72, 0xf3, 0x67, 0xf3, 0x61, 0x79,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x59, 0x75, 0x72, 0xf3, 0x67,
0xf3, 0x61, 0x79, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x59, 0x75, 0x72, 0xf3, 0x67, 0xf3, 0x61,
0x79, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4f, 0x7a, 0x62, 0x65, 0x6b, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4f, 0x7a, 0x62,
0x65, 0x6b, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4f, 0x7a, 0x62,
0x65, 0x6b, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0xe1,
0x6e, 0x75, 0xe1, 0x74, 0x75, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x57, 0xe1, 0x6e, 0x75, 0xe1, 0x74, 0x75, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0xe1, 0x6e,
0x75, 0xe1, 0x74, 0x75, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0x65, 0x6e, 0x65, 0x7a,
0x77, 0xe9, 0x6c, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57,
0x61, 0x6c, 0xe1, 0x64, 0x69, 0x77, 0xe1, 0x73, 0x74, 0xf3, 0x6b, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0x61, 0x6c, 0xe1, 0x64, 0x69,
0x77, 0xe1, 0x73, 0x74, 0xf3, 0x6b, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0x61, 0x6c, 0xe1,
0x64, 0x69, 0x77, 0xe1, 0x73, 0x74, 0xf3, 0x6b, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61,
0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x57, 0x61, 0x79, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0xe1,
0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0x61, 0x6c,
0x6c, 0x69, 0x73, 0x20, 0x6f, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0xe1,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x59, 0xe1, 0x6b, 0x75, 0x73,
0x6b, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x59, 0xe1, 0x6b, 0x75,
0x73, 0x6b, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x59, 0xe1, 0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x59, 0x61, 0x6b, 0xe1, 0x74, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x72,
0x67, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x59, 0x61, 0x6b, 0xe1,
0x74, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x72, 0x67, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x59, 0x61,
0x6b, 0xe1, 0x74, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x72, 0x67, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x59, 0x75, 0x6b, 0xf3, 0x6e, 0x20, 0x77, 0x61, 0x68, 0x64, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x67, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d,
0x65, 0x72, 0x69, 0x64, 0x69, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65,
0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70,
0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e,
0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73,
0x73, 0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d,
0x61, 0x73, 0x73, 0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d,
0x61, 0x73, 0x73, 0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65,
0x7a, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x6f, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x6f, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x67,
0x6e, 0x65, 0x20, 0x41, 0x6c, 0x6c, 0x69, 0x67, 0x67, 0x69, 0xe6, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x20, 0x41,
0x6c, 0x6c, 0x69, 0x67, 0x67, 0x69, 0xe6, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x6e, 0x74,
0x61, 0x67, 0x6e, 0x65, 0x20, 0x41, 0x6c, 0x6c, 0x69, 0x67, 0x67, 0x69,
0xe6, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66,
0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69,
0x66, 0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66,
0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x70, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x41,
0x70, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72,
0x61, 0x62, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0xf1,
0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0xf1, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0xf1,
0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6d,
0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x65, 0x7a,
0x6f, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x2d, 0x70,
0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x6d, 0x65, 0x7a, 0x6f, 0x2d, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x2d, 0x70, 0x6f, 0x6e,
0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x67, 0x69, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x67, 0x69, 0x61, 0x6e, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x67, 0x69, 0x61,
0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61,
0x70, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x43, 0x61, 0x70, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x43, 0x61, 0x70, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x69, 0x6c,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x6f, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20,
0x43, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x43, 0x69, 0xf1, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61, 0x20,
0x43, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x69, 0x73, 0x6f, 0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d,
0x61, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65,
0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x43,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73,
0x6f, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73,
0x6f, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x62, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x62, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x43, 0x75, 0x62, 0x62, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x45, 0x73, 0x74, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x71, 0x75, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x71, 0x75, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75,
0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76,
0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69,
0x6e, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70,
0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e,
0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70,
0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x4d,
0x61, 0x6c, 0x76, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f,
0x65, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73,
0x6f, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0xf1, 0x65, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x67, 0x69, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46,
0x69, 0x67, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x67, 0x69, 0x6f, 0x61, 0x20, 0x64,
0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x66, 0x72, 0x61,
0x6e, 0x73, 0x65, 0x69, 0x73, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0xe6, 0x72, 0x65, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x20, 0x65, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x63,
0x68, 0x65, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x73, 0x65, 0x69, 0x78, 0x69,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65,
0xf2, 0x72, 0x67, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72,
0x67, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d,
0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x6f, 0x61, 0x20, 0x64,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e,
0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f,
0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e,
0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x6f, 0x72, 0x66, 0x6f,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x4f, 0xe7, 0x65, 0x61, 0x6e, 0x6f, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x49, 0x6e, 0x64, 0x6f, 0x63, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f,
0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72,
0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x6f, 0x6e, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20,
0x47, 0x69, 0x61, 0x70, 0x70, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x69, 0x61, 0x70,
0x70, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x43,
0x6f, 0x72, 0x65, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x6f, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4b,
0x69, 0x72, 0x67, 0x68, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x64, 0x61,
0x20, 0x4c, 0x69, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x65,
0x73, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x72, 0x63, 0x68, 0x65, 0x69, 0x78, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7,
0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65, 0x73, 0x63, 0x69, 0x63,
0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69,
0x63, 0x6f, 0x20, 0x6d, 0x65, 0x73, 0x63, 0x69, 0x63, 0x61, 0xf1, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65,
0x73, 0x63, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x2019,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x2019, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x6f, 0x61, 0x20,
0x64, 0x61, 0x20, 0x42, 0x69, 0x72, 0x6d, 0x61, 0x6e, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x6f, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x6f, 0x61, 0x20,
0x64, 0x61, 0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x65, 0x75,
0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61,
0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x65,
0x75, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x61, 0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x61, 0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0xe6, 0x72, 0x61, 0x6e, 0x65, 0x75, 0x76, 0x61, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54,
0xe6, 0x72, 0x61, 0x6e, 0x65, 0x75, 0x76, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x54, 0xe6, 0x72,
0x61, 0x6e, 0x65, 0x75, 0x76, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x69, 0x75, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x69, 0x73, 0x6f, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0xd2, 0x6d, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0xd2,
0x6d, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0xd2, 0x6d, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x47,
0x75, 0x69, 0x6e, 0x65, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72,
0xf9, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72,
0xf9, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0xf1, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0xf1, 0x65, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x50, 0x68,
0x6f, 0x65, 0x6e, 0x69, 0x78, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6e, 0x20, 0x50, 0xea, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50,
0xea, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0xea, 0x20, 0x65, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67,
0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65,
0x72, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68,
0x65, 0x6c, 0x6c, 0x65, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x63, 0x69, 0x6e, 0x67, 0x61, 0x70, 0xf4, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67,
0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x73, 0x75, 0x64, 0x6f, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x61, 0x67, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6f,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x73, 0x73, 0x75,
0x65, 0x6c, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x61, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x645,
0x6cc, 0x632, 0x648, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x645, 0x6cc, 0x632, 0x648, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x639, 0x631, 0x628, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x639, 0x631, 0x628, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627,
0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633,
0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631,
0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x679,
0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627,
0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc,
0x62c, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622, 0x630,
0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622, 0x630,
0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x20, 0x633, 0x6cc, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x628, 0x646, 0x6af, 0x644, 0x6c1,
0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20,
0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x628,
0x6be, 0x648, 0x679, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x628, 0x648, 0x644, 0x6cc, 0x648, 0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x686, 0x644, 0x6cc, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x686, 0x644, 0x6cc, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x686, 0x644, 0x6cc, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x686, 0x6cc, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6a9,
0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648,
0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x633,
0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627,
0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x6a9, 0x648, 0x627,
0x688, 0x648, 0x631, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x648, 0x633,
0x637, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x648, 0x633, 0x637, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633,
0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x648, 0x633, 0x637, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x634,
0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x634,
0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x628, 0x639, 0x6cc, 0x62f, 0x20, 0x645, 0x634, 0x631,
0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641,
0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x632, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641, 0x627,
0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646,
0x688, 0x632, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x641, 0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688,
0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x641, 0x631, 0x6cc, 0x646, 0x686, 0x20, 0x6af, 0x6cc,
0x627, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641, 0x631,
0x6cc, 0x646, 0x686, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x622, 0x6ba,
0x20, 0x627, 0x646, 0x679, 0x627, 0x631, 0x6a9, 0x679, 0x6a9, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x632, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62c, 0x627, 0x631, 0x62c, 0x6cc,
0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x648, 0x686,
0x20, 0x633, 0x6cc, 0x20, 0x627, 0x635, 0x644, 0x20, 0x648, 0x64e, 0x62e, 0x645,
0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a,
0x631, 0x628, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646,
0x688, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62e, 0x644, 0x6cc,
0x62c, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x6cc, 0x646, 0x6af,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x6cc,
0x646, 0x6af, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x6cc, 0x646, 0x6af,
0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x646, 0x62f,
0x648, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x631, 0x627, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x631, 0x627, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x642, 0x632, 0x627, 0x62e, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a,
0x631, 0x628, 0x6cc, 0x20, 0x642, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6a9, 0x631, 0x63a, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x627, 0x644,
0x62f, 0x6cc, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646, 0x6cc,
0x67e, 0x627, 0x644, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646, 0x6cc,
0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646,
0x688, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x646, 0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626,
0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646,
0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x646, 0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641,
0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648,
0x631, 0x646, 0x6c1, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641,
0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648,
0x631, 0x646, 0x6c1, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x627, 0x6a9, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x6cc, 0x631, 0x627,
0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e,
0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x6cc, 0x631,
0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x633,
0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e,
0x6cc, 0x626, 0x631, 0x20, 0x622, 0x6ba, 0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626,
0x648, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20,
0x67e, 0x6cc, 0x626, 0x631, 0x20, 0x622, 0x6ba, 0x20, 0x645, 0x6a9, 0x644, 0x6cc,
0x626, 0x648, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x6cc,
0x626, 0x631, 0x20, 0x622, 0x6ba, 0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648,
0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x67e, 0x6cc, 0x627, 0x646, 0x6af, 0x20, 0x6cc, 0x627, 0x646, 0x6af,
0x20, 0x648, 0x64e, 0x62e, 0x631, 0x648, 0x62a, 0x6be, 0x6cc, 0x631, 0x627, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x633, 0x648, 0x631, 0x6cc, 0x646, 0x627,
0x645, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62a, 0x627, 0x62c, 0x6a9,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62a,
0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6cc, 0x648,
0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6cc,
0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x648, 0x6cc, 0x646, 0x632,
0x648, 0x626, 0x6cc, 0x644, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x648, 0x648, 0x633, 0x679, 0x627, 0x6a9, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47,
0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67,
0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x256, 0x6f,
0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x73, 0x6f, 0x6e, 0x6f,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c,
0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74,
0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69,
0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x47, 0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20,
0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41,
0x6d, 0x61, 0x73, 0x254, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x6d, 0x61, 0x73, 0x254, 0x6e, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61, 0x73, 0x254, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61,
0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b,
0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269,
0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67,
0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20,
0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x6c, 0x269,
0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a,
0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20,
0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x6c,
0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e,
0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x62, 0x28a, 0x6e, 0x28a, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41,
0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269,
0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x62, 0x28a,
0x6e, 0x28a, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61,
0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b,
0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x62, 0x28a, 0x6e, 0x28a,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61,
0x6c, 0x269, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x70, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x6b,
0x61, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20,
0x6b, 0x61, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x70, 0x69, 0x79, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x41, 0x70, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x41, 0x70, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x47, 0x61, 0x6c, 0x61, 0x61, 0x72, 0x69, 0x62, 0x75, 0x74,
0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x47, 0x61, 0x6c, 0x61, 0x61, 0x72, 0x69, 0x62, 0x75,
0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47,
0x61, 0x6c, 0x61, 0x61, 0x72, 0x69, 0x62, 0x75, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x72, 0x6a,
0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69,
0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x72, 0x6a, 0x61,
0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x67, 0x269,
0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41,
0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x67, 0x269, 0x74, 0x65,
0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e,
0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x74, 0x1dd,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x6d, 0x61, 0x6c, 0x269,
0x6b, 0x61, 0x20, 0x6b, 0x61, 0x74, 0x1dd, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61,
0x6c, 0x269, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x74, 0x1dd, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b,
0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x186, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269,
0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x186, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269,
0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x67, 0x269,
0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x186,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67,
0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b,
0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x67,
0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x186, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61,
0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x186, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b,
0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b,
0x61, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x6b, 0x61, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x186, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269,
0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x73, 0x25b, 0x72,
0x62, 0x61, 0x269, 0x6a, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x73, 0x25b, 0x72, 0x62, 0x61,
0x269, 0x6a, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41,
0x73, 0x25b, 0x72, 0x62, 0x61, 0x269, 0x6a, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x73, 0x254, 0x254, 0x72, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41,
0x73, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41,
0x73, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x42, 0x61, 0x14b, 0x67, 0x6c, 0x61, 0x256, 0x25b, 0x25b, 0x73, 0x68,
0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x42, 0x61, 0x14b, 0x67, 0x6c, 0x61, 0x256, 0x25b, 0x25b, 0x73, 0x68,
0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x42, 0x61, 0x14b, 0x67,
0x6c, 0x61, 0x256, 0x25b, 0x25b, 0x73, 0x68, 0x269, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x42, 0x6f,
0x6c, 0x69, 0x66, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x42, 0x72, 0x75, 0x6e, 0x25b, 0x269, 0x20, 0x189, 0x61, 0x72, 0x75,
0x73, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x61, 0x70, 0x66, 0x25b, 0x25b,
0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4b, 0x61, 0x70, 0x66, 0x25b, 0x25b, 0x72, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x4b, 0x61, 0x70, 0x66, 0x25b, 0x25b, 0x72, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x53, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x53, 0x68, 0x61, 0x74, 0x61, 0x6d, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x68, 0x61,
0x74, 0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x53, 0x68,
0x61, 0x74, 0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x53, 0x68, 0x69, 0x6c, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x68, 0x69, 0x6c, 0x69,
0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x53, 0x68, 0x69, 0x6c,
0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x43,
0x61, 0x269, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x43, 0x61, 0x269, 0x6e, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x43, 0x61, 0x269, 0x6e, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4e, 0x6f, 0x77, 0x25b, 0x6c, 0x20,
0x6b, 0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x4b, 0x6f, 0x6b, 0x6f, 0x6f, 0x73, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74,
0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x6f, 0x6c, 0x254,
0x6d, 0x62, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x4b, 0x6f, 0x6c, 0x254, 0x6d, 0x62, 0x69, 0x69,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4b, 0x6f, 0x6c, 0x254, 0x6d,
0x62, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x4b, 0x28a, 0x6b, 0x28a, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b,
0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x28a, 0x6b, 0x28a, 0x20, 0x6b,
0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4b, 0x28a, 0x6b, 0x28a, 0x20,
0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4b, 0x75,
0x62, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x4b, 0x75, 0x62, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x4b, 0x75, 0x62, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x189, 0x65, 0x66, 0x69, 0x73, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x189, 0x69,
0x6d, 0x254, 0x6e, 0x20, 0x189, 0x69, 0x72, 0x66, 0x69, 0x6c, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x69,
0x6d, 0x254, 0x254, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x50, 0x61, 0x61, 0x6b, 0x269, 0x20, 0x6b, 0x61, 0x41, 0x61, 0x74,
0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x61, 0x61, 0x6b,
0x269, 0x20, 0x6b, 0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x50, 0x61, 0x61,
0x6b, 0x269, 0x20, 0x6b, 0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74,
0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x45, 0x6b, 0x75, 0x77, 0x61, 0x256, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72,
0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x63,
0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74,
0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254,
0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x47, 0x61,
0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x61,
0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x66,
0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b,
0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47,
0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67,
0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e,
0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e,
0x61, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47, 0x61,
0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269,
0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x254, 0x6b, 0x6c,
0x61, 0x6e, 0x256, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd,
0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x254, 0x6b, 0x6c, 0x61, 0x6e, 0x256,
0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x254, 0x6b,
0x6c, 0x61, 0x6e, 0x256, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b,
0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x69, 0x6a, 0x69, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x61, 0x20, 0x47, 0x61, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x6a, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x66, 0x254, 0x6e, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x73, 0x6f, 0x6e, 0x6f,
0x6c, 0x61, 0x14b, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x61, 0x74, 0x75, 0x74,
0x61, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x254, 0x73, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x65, 0x65, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4a,
0x254, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4a, 0x254, 0x72, 0x6a, 0x69, 0x79,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4a, 0x254, 0x72, 0x6a,
0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x4a, 0x69, 0x6c, 0x62, 0x25b, 0x25b, 0x72, 0x20, 0x6b, 0x61, 0x42, 0x28a,
0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x72, 0x69,
0x69, 0x6e, 0x77, 0x69, 0x73, 0x68, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61,
0x6e, 0x256, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67, 0x61, 0x6a,
0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67, 0x61,
0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a,
0x62, 0x61, 0x6b, 0x61, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256,
0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67,
0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47, 0x72, 0x69, 0x6e, 0x6c,
0x61, 0x6e, 0x256, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69,
0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x47, 0x75, 0x77, 0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x254, 0x6c, 0x66, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47,
0x75, 0x79, 0x61, 0x6e, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x77, 0x61, 0x79, 0x269, 0x269,
0x20, 0x6e, 0x2019, 0x41, 0x6c, 0x65, 0x77, 0x75, 0x74, 0x69, 0x69, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41,
0x77, 0x61, 0x79, 0x269, 0x269, 0x20, 0x6e, 0x2019, 0x41, 0x6c, 0x65, 0x77,
0x75, 0x74, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41,
0x77, 0x61, 0x79, 0x269, 0x269, 0x20, 0x6e, 0x2019, 0x41, 0x6c, 0x65, 0x77,
0x75, 0x74, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x48, 0x254, 0x14b, 0x20, 0x4b, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x48, 0x254, 0x14b, 0x20,
0x4b, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x48, 0x254,
0x14b, 0x20, 0x4b, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x48, 0x254, 0x66, 0x256, 0x1dd, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x48, 0x254, 0x66, 0x256, 0x1dd,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x48, 0x254, 0x66, 0x256, 0x1dd,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x49, 0x6e, 0x256,
0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x49, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61,
0x74, 0x65, 0x14b, 0x6b, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49, 0x6e, 0x256, 0x69, 0x63, 0x61, 0x269,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x190, 0x6e, 0x256, 0x6f, 0x6e, 0x6f, 0x73, 0x69, 0x69, 0x20,
0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x190, 0x6e, 0x256,
0x6f, 0x6e, 0x6f, 0x73, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61,
0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x190, 0x6e, 0x256, 0x6f, 0x6e, 0x6f, 0x73,
0x69, 0x69, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68,
0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49, 0x72, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49, 0x72,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x49, 0x72, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x59, 0x69, 0x73, 0x68, 0x72, 0x61, 0x25b,
0x6c, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x59, 0x69, 0x73, 0x68, 0x72, 0x61, 0x25b, 0x6c, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x59, 0x69, 0x73, 0x68, 0x72, 0x61, 0x25b, 0x6c,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4a, 0x61, 0x70,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x4a, 0x61, 0x70, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x4a, 0x61, 0x70, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a,
0x62, 0x61, 0x6b, 0x61, 0x4b, 0x61, 0x73, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x4b, 0x61, 0x73, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x6a,
0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x61, 0x73, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c,
0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x4b, 0x6f, 0x72, 0x65, 0x65, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x6f, 0x72,
0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4b, 0x6f, 0x72,
0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4b,
0x254, 0x73, 0x72, 0x25b, 0x25b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x53, 0x69, 0x72, 0x69, 0x20, 0x4c, 0x61, 0x14b, 0x6b, 0x61, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4c,
0x61, 0x269, 0x6e, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd,
0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4c, 0x254, 0x72, 0x256, 0x20, 0x48, 0x6f,
0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4c, 0x254, 0x72, 0x256, 0x20, 0x48, 0x6f, 0x6f, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x4c, 0x254, 0x72, 0x256, 0x20, 0x48, 0x6f, 0x6f,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x61, 0x6b,
0x61, 0x77, 0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x4d, 0x61, 0x6b, 0x61, 0x77, 0x6f, 0x6f, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x61, 0x6b, 0x61, 0x77, 0x6f, 0x6f,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x61, 0x67,
0x61, 0x256, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x4d, 0x61, 0x67, 0x61, 0x256, 0x61, 0x6e, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x256, 0x61,
0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x61,
0x6c, 0x25b, 0x73, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x61, 0x6c, 0x256, 0x69, 0x69, 0x66,
0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4d, 0x61, 0x72, 0x6b, 0x65, 0x73, 0x61, 0x73, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b,
0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49, 0x6d, 0x6f, 0x72, 0x69, 0x73,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x49, 0x6d, 0x6f, 0x72, 0x69, 0x73, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x49, 0x6d, 0x6f, 0x72, 0x69, 0x73, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a,
0x62, 0x61, 0x6b, 0x61, 0x4d, 0x254, 0x73, 0x254, 0x6e, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x25b, 0x73,
0x69, 0x6b, 0x20, 0x6b, 0x61, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4d, 0x25b, 0x73, 0x69, 0x6b, 0x20, 0x6b, 0x61, 0x70, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x4d, 0x25b, 0x73, 0x69, 0x6b, 0x20, 0x6b, 0x61, 0x70, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x254, 0x254, 0x72, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x55,
0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x254,
0x254, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d,
0x6f, 0x73, 0x6b, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x75, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x75,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x69, 0x79,
0x61, 0x6e, 0x6d, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4e, 0x61, 0x77, 0x75, 0x72, 0x75, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4e,
0x65, 0x65, 0x70, 0x61, 0x6c, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x61, 0x6c, 0x65, 0x256, 0x6f, 0x6e,
0x69, 0x69, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x61, 0x6c, 0x65,
0x256, 0x6f, 0x6e, 0x69, 0x69, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4b, 0x61, 0x6c, 0x65, 0x256, 0x6f,
0x6e, 0x69, 0x69, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x256,
0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x256,
0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x61, 0x66, 0x254, 0x6c,
0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x61,
0x28a, 0x6e, 0x256, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x61, 0x66, 0x254, 0x6c,
0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x46, 0x61, 0x28a, 0x6e, 0x256, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x61,
0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46,
0x61, 0x28a, 0x6e, 0x256, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x61, 0x66, 0x254,
0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4e,
0x69, 0x77, 0x75, 0x79, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4e, 0x254, 0x72, 0x66, 0x6f, 0x6f, 0x6b,
0x20, 0x6b, 0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4e, 0x254, 0x72, 0x66, 0x6f, 0x6f, 0x6b, 0x20, 0x6b, 0x61, 0x41,
0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x4e, 0x254, 0x72, 0x66, 0x6f, 0x6f, 0x6b, 0x20,
0x6b, 0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x25b, 0x72,
0x6e, 0x61, 0x6e, 0x256, 0x6f, 0x20, 0x256, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x254, 0x6e, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x46, 0x25b, 0x72, 0x6e, 0x61, 0x6e, 0x256, 0x6f,
0x20, 0x256, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x254, 0x6e, 0x79, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x25b, 0x72, 0x6e, 0x61, 0x6e,
0x256, 0x6f, 0x20, 0x256, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x254, 0x6e, 0x79,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x61,
0x72, 0x69, 0x79, 0x61, 0x6e, 0x20, 0x6b, 0x1dd, 0x62, 0x28a, 0x74, 0x75,
0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79,
0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4e, 0x6f, 0x66, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4e,
0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x6b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x186, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x186, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x186, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a,
0x62, 0x61, 0x6b, 0x61, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x50, 0x61, 0x6c, 0x61,
0x77, 0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x50, 0x61, 0x70, 0x75, 0x61, 0x73, 0x69, 0x69, 0x20,
0x47, 0x69, 0x6e, 0x65, 0x65, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x50, 0x65, 0x72, 0x75, 0x75, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50,
0x65, 0x72, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x50,
0x65, 0x72, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x6f, 0x65, 0x6e, 0x69, 0x69, 0x73,
0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x53, 0x25b, 0x14b, 0x2d, 0x50, 0x65, 0x74, 0x72, 0x254, 0x73,
0x20, 0x6e, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x254, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53,
0x25b, 0x14b, 0x2d, 0x50, 0x65, 0x74, 0x72, 0x254, 0x73, 0x20, 0x6e, 0x61,
0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x53, 0x25b, 0x14b, 0x2d, 0x50, 0x65, 0x74, 0x72, 0x254,
0x73, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x254, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x50, 0x269, 0x74,
0x6b, 0x25b, 0x25b, 0x6e, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b,
0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x50, 0x269, 0x79, 0x254, 0x14b, 0x79, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x52, 0x65, 0x65,
0x6e, 0x69, 0x79, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x61, 0x6b, 0x61, 0x6c,
0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x53, 0x61, 0x6b,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x53, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x61, 0x6d, 0x6f, 0x77,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x53, 0x61, 0x6d, 0x6f,
0x77, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x53,
0x65, 0x73, 0x68, 0x25b, 0x6c, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x69, 0x14b, 0x67, 0x61, 0x70, 0x75,
0x75, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x254, 0x254, 0x6e, 0x20, 0x6b,
0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x4a, 0x254, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79,
0x269, 0x73, 0x6f, 0x6e, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x53, 0x69, 0x79, 0x6f, 0x77, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x54, 0x61, 0x269, 0x70, 0x65, 0x69, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x61, 0x269, 0x70,
0x65, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x54, 0x61, 0x269,
0x70, 0x65, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x28a, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x254, 0x14b, 0x67, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x254,
0x14b, 0x67, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x54, 0x254,
0x14b, 0x67, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x43, 0x75, 0x75, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x54, 0x75, 0x66, 0x61,
0x6c, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x55, 0x72, 0x75, 0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x55, 0x72,
0x75, 0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x55, 0x72, 0x75, 0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x55, 0x73, 0x62, 0x65, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x55, 0x73, 0x62, 0x65, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x55,
0x73, 0x62, 0x65, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x61, 0x6e, 0x75, 0x77,
0x61, 0x74, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x46, 0x61, 0x6e, 0x75, 0x77, 0x61, 0x74, 0x75, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x61, 0x6e, 0x75, 0x77, 0x61,
0x74, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46,
0x65, 0x6e, 0x65, 0x73, 0x75, 0x77, 0x65, 0x6c, 0x61, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x6c,
0x61, 0x256, 0x69, 0x66, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x6c, 0x61,
0x256, 0x69, 0x66, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x46, 0x6c, 0x61, 0x256, 0x69, 0x66, 0x254, 0x73, 0x74,
0x254, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46,
0x254, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x61, 0x256, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x254, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x61, 0x256, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x46, 0x254, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x61, 0x256,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x254, 0x73,
0x74, 0x254, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x57, 0x65, 0x65, 0x6b, 0x20, 0x6b, 0x61, 0x42, 0x28a,
0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x57, 0x61, 0x6c,
0x69, 0x73, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x59,
0x75, 0x6b, 0x254, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x905, 0x92b, 0x93c, 0x917, 0x93e, 0x928, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947,
0x940, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x926, 0x916, 0x923, 0x20, 0x905, 0x92b, 0x93c,
0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20,
0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b,
0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x200d, 0x915,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x200d, 0x915, 0x93e, 0x20, 0x926, 0x947,
0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20,
0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x90f, 0x92e, 0x947, 0x91c, 0x93c, 0x94b, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x90f, 0x92e, 0x947, 0x91c, 0x93c, 0x94b, 0x928, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x90f, 0x92e, 0x947,
0x91c, 0x93c, 0x94b, 0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924,
0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20,
0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x93f,
0x915, 0x940, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20,
0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926,
0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930,
0x93f, 0x915, 0x940, 0x20, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x91f, 0x948,
0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x93f,
0x915, 0x940, 0x20, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x948,
0x932, 0x947, 0x940, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940,
0x20, 0x905, 0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x939, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20,
0x92a, 0x939, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940,
0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x939, 0x93e,
0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940,
0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930,
0x936, 0x93e, 0x902, 0x924, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f,
0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x947, 0x20, 0x926,
0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947,
0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x926, 0x947, 0x20,
0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909,
0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905,
0x930, 0x92c, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x930, 0x92c,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x905, 0x930, 0x92c, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902,
0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x930, 0x94d,
0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d,
0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930,
0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940,
0x928, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x906, 0x930, 0x94d, 0x92e,
0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x906, 0x930, 0x94d,
0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905,
0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x926, 0x947, 0x20, 0x927,
0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c,
0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93f,
0x91a, 0x932, 0x947, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938,
0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x947, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938,
0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x947, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926,
0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947,
0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x200d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902,
0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947,
0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x908, 0x20, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x200d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x948,
0x932, 0x947, 0x940, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x926,
0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947,
0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x905, 0x91c, 0x93c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91c, 0x93c, 0x930, 0x92c, 0x948, 0x91c,
0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x905, 0x91c, 0x93c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91c, 0x93c, 0x94b, 0x930,
0x947, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947,
0x936, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93e, 0x902,
0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93e, 0x902, 0x917,
0x94d, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92d, 0x942, 0x91f, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92c, 0x94b, 0x932, 0x940, 0x935, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x940,
0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c,
0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x94d,
0x930, 0x93e, 0x91c, 0x93c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92c, 0x94d, 0x930, 0x942, 0x928, 0x947, 0x908, 0x20, 0x926,
0x93e, 0x930, 0x942, 0x938, 0x94d, 0x938, 0x932, 0x92e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930,
0x94d, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20, 0x926,
0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x948,
0x925, 0x92e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x948, 0x925,
0x92e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x91a, 0x948, 0x925, 0x92e, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x93f, 0x932, 0x940,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x93f, 0x932, 0x940, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x91a,
0x93f, 0x932, 0x940, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x940, 0x928,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x940, 0x928, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x940,
0x928, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947,
0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938, 0x20,
0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915,
0x94b, 0x915, 0x94b, 0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20,
0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x915, 0x941, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x941,
0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x941, 0x915, 0x20, 0x91f, 0x93e,
0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x915, 0x941, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20,
0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x905, 0x926, 0x94d, 0x927, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x92f, 0x942,
0x92c, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x92f,
0x942, 0x92c, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x926, 0x947,
0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20,
0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x921, 0x947, 0x935, 0x93f, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x921, 0x94d, 0x92f, 0x942, 0x92e, 0x94b, 0x902, 0x91f, 0x20, 0x921, 0x940, 0x20,
0x905, 0x930, 0x94d, 0x935, 0x93f, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x924, 0x93f, 0x92e, 0x94b,
0x930, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x908, 0x938, 0x94d,
0x91f, 0x930, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x91f, 0x93e, 0x92a, 0x942,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926,
0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x907, 0x915, 0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93f, 0x91a, 0x932, 0x947,
0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92c, 0x93f, 0x91a, 0x932, 0x947, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x92c, 0x93f, 0x91a, 0x932, 0x947, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947,
0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x94b, 0x92a, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x917, 0x94d,
0x930, 0x20, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926,
0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x949, 0x915, 0x932, 0x948, 0x902, 0x921,
0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d,
0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b,
0x93c, 0x949, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x941,
0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92b,
0x93c, 0x949, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x941,
0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c,
0x93f, 0x91c, 0x940, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x94d,
0x930, 0x947, 0x902, 0x91a, 0x20, 0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x926, 0x916, 0x923, 0x947, 0x20, 0x92c,
0x916, 0x947, 0x902, 0x20, 0x92b, 0x93c, 0x94d, 0x930, 0x93e, 0x902, 0x938, 0x20,
0x915, 0x928, 0x947, 0x20, 0x905, 0x902, 0x91f, 0x93e, 0x930, 0x94d, 0x915, 0x91f,
0x93f, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x917, 0x948, 0x932,
0x93e, 0x92a, 0x947, 0x917, 0x94b, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x917, 0x948, 0x902, 0x92c, 0x93f, 0x92f, 0x930, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930, 0x94d, 0x91f,
0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d,
0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948,
0x932, 0x947, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902,
0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932,
0x948, 0x902, 0x921, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x916,
0x93e, 0x921, 0x93c, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x939, 0x935, 0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d,
0x92f, 0x942, 0x936, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x939,
0x935, 0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x939,
0x935, 0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20,
0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926,
0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915,
0x93e, 0x901, 0x917, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x939,
0x94b, 0x935, 0x94d, 0x921, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92d, 0x93e,
0x930, 0x924, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x939, 0x93f, 0x902, 0x926, 0x20, 0x92e, 0x939, 0x93e, 0x938,
0x93e, 0x917, 0x930, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x907, 0x902,
0x921, 0x94b, 0x91a, 0x93e, 0x907, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92c, 0x93f, 0x91a, 0x932, 0x947, 0x20, 0x907, 0x902, 0x921, 0x94b,
0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928,
0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x907, 0x902, 0x921, 0x94b,
0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x908, 0x930, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x908, 0x930, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x908, 0x930, 0x93e, 0x928, 0x20, 0x926, 0x947, 0x20,
0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909,
0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x907,
0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907,
0x932, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x907, 0x91c, 0x93c, 0x930,
0x93e, 0x907, 0x932, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x926,
0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947,
0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e,
0x928, 0x947, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c,
0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x20, 0x915, 0x91c,
0x93c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x915, 0x91c,
0x93c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x908,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x926, 0x947, 0x20, 0x927,
0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c,
0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b,
0x938, 0x930, 0x93e, 0x90f, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915,
0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x930,
0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x93f, 0x930,
0x94d, 0x917, 0x93f, 0x938, 0x94d, 0x200d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x932, 0x93e, 0x907, 0x928, 0x20, 0x91f, 0x93e, 0x92a,
0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x932, 0x949, 0x930, 0x94d, 0x921,
0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x932, 0x949,
0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x926, 0x947, 0x20,
0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909,
0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e,
0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x917, 0x93e,
0x926, 0x93e, 0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x932, 0x947,
0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e,
0x93e, 0x932, 0x926, 0x940, 0x935, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x947, 0x938, 0x938, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x930, 0x94d, 0x936, 0x932, 0x20, 0x91f, 0x93e,
0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x930, 0x940,
0x936, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x930,
0x940, 0x936, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x935, 0x94d, 0x938, 0x928, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928,
0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20,
0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x947, 0x915, 0x94d, 0x938,
0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20,
0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926,
0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c,
0x91f, 0x94b, 0x930, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x938, 0x94d, 0x915,
0x94b, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x926, 0x93f, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92e, 0x94d, 0x92f, 0x93e, 0x902, 0x92e, 0x93e, 0x930, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94c, 0x930, 0x942, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x928, 0x947, 0x92a, 0x93e, 0x932, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x948, 0x932, 0x947,
0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b,
0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x948, 0x932,
0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932, 0x948, 0x902, 0x921,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x91c,
0x93c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c,
0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942,
0x92b, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93c, 0x93e,
0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x92b,
0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x947,
0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20,
0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x928, 0x940, 0x92f, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928,
0x949, 0x930, 0x92b, 0x93c, 0x949, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949,
0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949,
0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d,
0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e,
0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x930,
0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20,
0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d,
0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d,
0x939, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92b, 0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921,
0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939,
0x93e, 0x20, 0x926, 0x947, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94b, 0x935, 0x94b, 0x938,
0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x913,
0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a,
0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x93e, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92a, 0x932, 0x93e, 0x909, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92a, 0x93e, 0x92a, 0x941, 0x906, 0x20, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917,
0x93f, 0x928, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948,
0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x930,
0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a,
0x947, 0x930, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x947,
0x930, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92b, 0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92b,
0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x926, 0x940, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92b, 0x93c, 0x940, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x20, 0x91f, 0x93e,
0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x947, 0x902, 0x91f,
0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x915, 0x928, 0x947, 0x20, 0x92e,
0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930,
0x947, 0x20, 0x915, 0x928, 0x947, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947,
0x932, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930,
0x947, 0x20, 0x915, 0x928, 0x947, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947,
0x932, 0x93e, 0x928, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x93f, 0x91f, 0x915, 0x948, 0x930,
0x94d, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x94b, 0x928,
0x93e, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x94d,
0x92f, 0x94b, 0x902, 0x917, 0x92f, 0x93e, 0x902, 0x917, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x930, 0x940, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x930, 0x94b, 0x925, 0x947, 0x930, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x916, 0x93e, 0x932, 0x93f,
0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x916, 0x93e, 0x932,
0x93f, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x938, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x926, 0x940, 0x20,
0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x926, 0x947,
0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20,
0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x938, 0x947, 0x936, 0x947, 0x932, 0x94d, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x928,
0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d,
0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x926,
0x916, 0x923, 0x20, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x942, 0x930, 0x940, 0x928, 0x93e,
0x92e, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x94d, 0x92f,
0x94b, 0x935, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x93e,
0x939, 0x93f, 0x924, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924,
0x93e, 0x907, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924,
0x93e, 0x907, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x926, 0x947,
0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20,
0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x924, 0x93e, 0x91c, 0x93f, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91f, 0x94b, 0x915, 0x947, 0x932,
0x93e, 0x90a, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91f, 0x94b, 0x902,
0x917, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91f, 0x94b, 0x902,
0x917, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x91a, 0x941, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924,
0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x941, 0x930, 0x94d,
0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x924,
0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x941, 0x935, 0x93e,
0x932, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x930, 0x941,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909,
0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935,
0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x91c, 0x93c, 0x94d, 0x92c,
0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935,
0x928, 0x941, 0x906, 0x924, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x935, 0x928, 0x941, 0x906, 0x924, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x928, 0x941, 0x906, 0x924, 0x942,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x947, 0x928, 0x947, 0x91c, 0x93c,
0x941, 0x90f, 0x932, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935,
0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94d, 0x932, 0x93e, 0x926,
0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94d, 0x932, 0x93e,
0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x935, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930,
0x93e, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94b, 0x932,
0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94b, 0x932, 0x94d,
0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x926, 0x940, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x935, 0x947, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x935, 0x93e, 0x932, 0x93f, 0x938, 0x20, 0x915, 0x928,
0x947, 0x20, 0x92b, 0x93c, 0x94d, 0x92f, 0x942, 0x91a, 0x942, 0x928, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x93e,
0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x947,
0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947,
0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924,
0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x926, 0x940, 0x20,
0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92f, 0x941, 0x915, 0x949, 0x928, 0x20, 0x91f, 0x948, 0x92e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65, 0x67, 0x61, 0x20, 0x73,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65, 0x67, 0x61, 0x20,
0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65,
0x67, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65, 0x67, 0x61,
0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20,
0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67,
0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65,
0x67, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x41, 0x6d, 0xe8,
0x72, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x6d,
0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x20, 0x41, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x67,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50,
0x61, 0x73, 0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x50, 0x61, 0x73, 0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x50, 0x61, 0x73, 0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41,
0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6a, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6a,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x65, 0x67, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x65, 0x67, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72,
0x6f, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x6f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x65,
0x6e, 0x74, 0x72, 0x6f, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x42, 0x75, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x42,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64,
0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x69, 0x61, 0x74, 0x65,
0x6d, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x43, 0x69, 0x61, 0x74, 0x65, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x69, 0x61, 0x74, 0x65, 0x6d,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x43, 0x69, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x53, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x53,
0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x64,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f,
0x6c, 0x6f, 0x6e, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x43, 0x6f, 0x6c, 0x6f, 0x6e, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6e, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63,
0x75, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc8, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x73, 0x74, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20,
0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x66, 0x72, 0x61,
0x6e, 0x73, 0x65, 0x7a, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0x54, 0x65, 0x72, 0x65, 0x20, 0x66, 0x72, 0x61, 0x6e,
0x73, 0x65, 0x7a, 0x69, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x73, 0x75,
0x64, 0x20, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6e, 0x74,
0xe0, 0x72, 0x74, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x20, 0x47, 0x61, 0x6c, 0xe0, 0x70, 0x61, 0x67, 0x6f,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4a, 0x65,
0x6f, 0x72, 0x6a, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4a, 0x65,
0x6f, 0x72, 0x6a, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4a, 0x65, 0x6f,
0x72, 0x6a, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6d,
0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x73, 0xe8, 0x61, 0x6e, 0x6f,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x73, 0x69, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61,
0x65, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43,
0x6f, 0x72, 0xe8, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f,
0x72, 0xe8, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x72, 0xe8,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a,
0x6f, 0x6c, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4b, 0x69,
0x72, 0x67, 0x68, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x45, 0x63, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a,
0x6f, 0x6c, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4d, 0x61, 0x6c,
0x65, 0x7a, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x68,
0x65, 0x7a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x73, 0x69, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x73, 0x69, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x2019, 0x6c, 0x20, 0x4d, 0xe8, 0x73, 0x65, 0x67, 0x6f, 0x20, 0x64, 0x65,
0x2019, 0x6c, 0x20, 0x50, 0x61, 0x73, 0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x4d, 0xe8, 0x73, 0x65, 0x67, 0x6f, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x73, 0xec, 0x66, 0x65, 0x67, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x4d, 0xe8, 0x73, 0x65, 0x67, 0x6f, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x73, 0xec, 0x66, 0x65, 0x67, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0xe0, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0xe0, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0xe0, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20,
0x28, 0x42, 0x69, 0x72, 0x6d, 0x61, 0x6e, 0x69, 0x61, 0x29, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x2019, 0x6c, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x54, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x54, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e,
0x69, 0x75, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x50,
0x61, 0x6c, 0xe0, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x47, 0x75, 0x69, 0x6e, 0xe8, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0xe0, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0xe0,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0xe0,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x65,
0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x2019, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x46, 0x65, 0x6e, 0x69, 0x7a, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x6f, 0x20,
0x65, 0x20, 0x4d, 0x69, 0x63, 0x68, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x53, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x6f, 0x20, 0x65, 0x20,
0x4d, 0x69, 0x63, 0x68, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x53, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x63,
0x68, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x70,
0xe8, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c,
0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x53, 0x61, 0x6d, 0xf2, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x53, 0x61, 0x6d, 0xf2, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x53, 0x61,
0x6d, 0xf2, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x53, 0x65, 0x69, 0x73, 0x65, 0x6c, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20,
0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x73, 0x75, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x54, 0x61, 0x69, 0x74, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x69, 0x70, 0xe9, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c,
0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0xe0, 0x75, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c,
0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x2019, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a,
0x6f, 0x6c, 0x65, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75,
0xe0, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75,
0xe0, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe0,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x56, 0x65,
0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x74, 0x6f, 0x6c, 0x6f, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c,
0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x61, 0x61, 0x70, 0x67, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61,
0x64, 0x69, 0x6e, 0x69, 0x20, 0x61, 0x61, 0x70, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x20, 0x68, 0x70,
0x75, 0x20, 0x61, 0x61, 0x70, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x64, 0x6b, 0x69, 0x1e47, 0x20, 0x61, 0x61, 0x70, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x6b,
0x75, 0x1e47, 0x70, 0x75, 0x20, 0x61, 0x61, 0x70, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x20, 0x6b, 0x75,
0x1e47, 0x70, 0x75, 0x20, 0x61, 0x61, 0x70, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x76, 0x65, 0x1e0d, 0x61, 0x20, 0x6b, 0x75, 0x1e47, 0x70, 0x75, 0x20, 0x61,
0x61, 0x70, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x72, 0xe3, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x6d, 0x61,
0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x20, 0x1e0d, 0x65, 0x6c, 0x61, 0x61, 0x69, 0x1e6d,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x6d, 0x61, 0x6a, 0x6f, 0x6e, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x101, 0x6d, 0x61, 0x6a, 0x6f, 0x6e, 0x20, 0x6d,
0x61, 0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101,
0x6d, 0x61, 0x6a, 0x6f, 0x6e, 0x20, 0x6b, 0x61, 0x72, 0xe3, 0x20, 0x6d,
0x61, 0x73, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61, 0x64, 0x69,
0x6e, 0x12b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61, 0x64, 0x69, 0x6e,
0x12b, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6d, 0x61, 0x64, 0x69, 0x6e, 0x12b, 0x20, 0x1e0d, 0x65, 0x6c,
0x61, 0x61, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d,
0x61, 0x68, 0x61, 0x70, 0x75, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x76, 0x65,
0x1e0d, 0x61, 0x68, 0x61, 0x70, 0x75, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x68,
0x61, 0x70, 0x75, 0x20, 0x1e0d, 0x65, 0x6c, 0x61, 0x61, 0x69, 0x1e6d, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x68, 0x61, 0x72, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x68, 0x61, 0x72, 0x6b, 0x61, 0x20, 0x6d, 0x61, 0x6e, 0x61,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61, 0x72, 0x6b,
0x61, 0x20, 0x1e0d, 0x65, 0x6c, 0x61, 0x61, 0x69, 0x1e6d, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x70, 0x65, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x70, 0x65, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x20, 0x6d, 0x61,
0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65,
0x73, 0x69, 0x70, 0x69, 0x6b, 0x20, 0x1e0d, 0x65, 0x6c, 0x61, 0x61, 0x69,
0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x65, 0x70, 0x69, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x65, 0x70, 0x69, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x65, 0x70, 0x69, 0x101,
0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x74, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x101, 0x72, 0x62, 0x69, 0x101, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x101, 0x72, 0x62, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x62, 0x69,
0x101, 0x74, 0x69, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x74, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x101, 0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69,
0x6e, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x101, 0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101,
0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75,
0x70, 0x75, 0x20, 0x101, 0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47,
0x75, 0x70, 0x75, 0x20, 0x101, 0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e,
0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75,
0x20, 0x101, 0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x6b,
0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x101,
0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101,
0x1e6d, 0x6c, 0x101, 0x6e, 0x1e6d, 0x69, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x101, 0x1e6d, 0x6c, 0x101, 0x6e, 0x1e6d, 0x69, 0x6b, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x1e6d, 0x6c,
0x101, 0x6e, 0x1e6d, 0x69, 0x6b, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x1e6d,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20,
0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20,
0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65,
0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101,
0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d,
0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75,
0x1e47, 0x75, 0x70, 0x75, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74,
0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x64, 0x69,
0x6e, 0x101, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70,
0x75, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69,
0x20, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x77, 0x65, 0x1e0d, 0x101,
0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69,
0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x61,
0x70, 0x75, 0x20, 0x101, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101,
0x74, 0x69, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x61, 0x70, 0x16b, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x77,
0x65, 0x1e0d, 0x101, 0x68, 0x61, 0x70, 0x16b, 0x20, 0x64, 0x65, 0x6c, 0x101,
0x69, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101,
0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x16b, 0x20, 0x101, 0x73, 0x74, 0x72, 0x65,
0x6c, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72,
0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x6b,
0x75, 0x1e47, 0x75, 0x70, 0x16b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c,
0x69, 0x101, 0x74, 0x69, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47,
0x75, 0x70, 0x16b, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x101, 0x6a, 0x65, 0x72, 0x62, 0x101, 0x69, 0x6a, 0x101,
0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x6a, 0x65, 0x72, 0x62, 0x101,
0x69, 0x6a, 0x101, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x6a, 0x65, 0x72, 0x62, 0x101, 0x69,
0x6a, 0x101, 0x6e, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x61, 0x6a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x61, 0x6a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6a, 0x6f, 0x72,
0x65, 0x73, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x62, 0x101, 0x6e, 0x67, 0x6c, 0x101, 0x64, 0x65, 0x73, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x62, 0x101, 0x6e, 0x67, 0x6c, 0x101, 0x64, 0x65, 0x73,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x62, 0x101, 0x6e, 0x67, 0x6c, 0x101, 0x64, 0x65, 0x73, 0x20, 0x6b,
0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x75, 0x74,
0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x6f, 0x6c, 0x77, 0x69,
0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x72, 0x101, 0x6a, 0x69, 0x6c,
0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x72, 0x101, 0x6a, 0x69,
0x6c, 0x69, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x62, 0x72, 0x101, 0x6a, 0x69, 0x6c, 0x69, 0x101,
0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62,
0x72, 0x75, 0x6e, 0x65, 0x74, 0x69, 0x20, 0x64, 0x101, 0x72, 0x75, 0x73,
0x101, 0x6c, 0x101, 0x6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x65, 0x70,
0x20, 0x62, 0x1e0d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x65, 0x70, 0x20,
0x62, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6b, 0x65, 0x70, 0x20, 0x62, 0x1e0d, 0x20, 0x6b, 0x101,
0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x101, 0x6d, 0x6f,
0x72, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x63, 0x79, 0x101, 0x74, 0x101, 0x6d, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x63, 0x79, 0x101, 0x74, 0x101, 0x6d, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x79, 0x101, 0x74,
0x101, 0x6d, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x74, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x63, 0x69, 0x6e, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63,
0x69, 0x6c, 0x69, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x63, 0x69, 0x6c, 0x69, 0x20, 0x6b, 0x101, 0x72,
0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x69, 0x6e, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x63, 0x69, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x69, 0x6e, 0x20, 0x64,
0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x72,
0x69, 0x73, 0x6d, 0x101, 0x73, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6b, 0x6f, 0x6b, 0x6f, 0x73, 0x20, 0x64, 0x12b, 0x70, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x101, 0x20, 0x6b,
0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x75, 0x6b,
0x20, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x75, 0x6b,
0x20, 0x64, 0x12b, 0x70, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x75, 0x6b, 0x20, 0x64, 0x12b, 0x70,
0x20, 0x101, 0x64, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6b, 0x75, 0x62, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6b, 0x75, 0x62, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x75, 0x62, 0x101, 0x20, 0x64, 0x65,
0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x64, 0x65, 0x62,
0x69, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e0d, 0x75, 0x6d, 0x6f, 0x6e,
0x1e6d, 0x20, 0x1e0d, 0x69, 0x20, 0x61, 0x72, 0x77, 0x69, 0x6c, 0x113, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x61, 0x70, 0x75,
0x20, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69,
0x73, 0x1e6d, 0x72, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x69, 0x73, 0x1e6d, 0x72, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x73, 0x1e6d,
0x72, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x69, 0x71, 0x75, 0x101, 0x1e0d, 0x6f, 0x72, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x79,
0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x79, 0x75, 0x72, 0x6f,
0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101,
0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6b,
0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d,
0x101, 0x68, 0x70, 0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101,
0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68,
0x70, 0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20, 0x79, 0x75, 0x72,
0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x72, 0x20, 0x77, 0x65, 0x1e0d, 0x101,
0x68, 0x70, 0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x74, 0x69,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x16b, 0x1e47,
0x16b, 0x70, 0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74,
0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x16b,
0x1e47, 0x16b, 0x70, 0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101,
0x74, 0x69, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x16b, 0x1e47, 0x16b, 0x70,
0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20,
0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61,
0x6b, 0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x74,
0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61, 0x6b, 0x6c, 0x79, 0x101,
0x6e, 0x1e0d, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x74, 0x69, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61,
0x6b, 0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x74,
0x69, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x70, 0x69, 0x6a, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69, 0x6a,
0x69, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x70, 0x69, 0x6a, 0x69, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20,
0x67, 0x75, 0x79, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x72,
0x65, 0x6e, 0x63, 0x20, 0x64, 0x61, 0x6b, 0x69, 0x1e47, 0x20, 0x61, 0x1e0d,
0x65, 0x20, 0x61, 0x61, 0x6e, 0x1e6d, 0x101, 0x72, 0x6b, 0x1e6d, 0x69, 0x6b,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x67, 0x101, 0x6c, 0x101, 0x70, 0x6f, 0x67,
0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x67, 0x61, 0x6d, 0x62, 0x69, 0x79,
0x72, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6a, 0x61, 0x72, 0x6a, 0x12b, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6a, 0x61, 0x72, 0x6a, 0x69, 0x101, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6a, 0x61, 0x72, 0x6a, 0x69, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x67, 0x12b, 0x6c, 0x62, 0x72, 0x1e6d, 0x20,
0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x67, 0x72, 0x69, 0x6e,
0x77, 0x69, 0x63, 0x20, 0x6d, 0x12b, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20, 0x67, 0x72, 0x69, 0x6e,
0x6c, 0x101, 0x6e, 0x64, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d,
0x101, 0x68, 0x70, 0x75, 0x20, 0x67, 0x72, 0x69, 0x6e, 0x6c, 0x101, 0x6e,
0x64, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20, 0x67, 0x72,
0x69, 0x6e, 0x6c, 0x101, 0x6e, 0x64, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x61, 0x6b, 0x16b, 0x1e47,
0x16b, 0x70, 0x20, 0x67, 0x72, 0x69, 0x6e, 0x6c, 0x101, 0x6e, 0x1e0d, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x61, 0x6b, 0x16b, 0x1e47, 0x16b,
0x70, 0x20, 0x67, 0x72, 0x69, 0x6e, 0x6c, 0x101, 0x6e, 0x1e0d, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77,
0x65, 0x1e0d, 0x61, 0x6b, 0x16b, 0x1e47, 0x16b, 0x70, 0x20, 0x67, 0x72, 0x69,
0x6e, 0x6c, 0x101, 0x6e, 0x1e0d, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x67, 0x61, 0x6c, 0x70, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x101, 0x77,
0x101, 0x65, 0x2d, 0x20, 0x61, 0x61, 0x6c, 0x16b, 0x73, 0x101, 0x6e, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x68, 0x101, 0x77, 0x101, 0x65, 0x2d, 0x20, 0x61,
0x61, 0x6c, 0x16b, 0x73, 0x101, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x101, 0x77, 0x101, 0x65,
0x2d, 0x20, 0x61, 0x61, 0x6c, 0x16b, 0x73, 0x101, 0x6e, 0x20, 0x1e0d, 0x65,
0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61, 0x6e,
0x67, 0x20, 0x6b, 0x61, 0x6e, 0x67, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68,
0x61, 0x6e, 0x67, 0x20, 0x6b, 0x61, 0x6e, 0x67, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61, 0x6e,
0x67, 0x20, 0x6b, 0x61, 0x6e, 0x67, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x77, 0x6f, 0x1e0d, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x68, 0x77, 0x6f, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x77, 0x6f, 0x1e0d, 0x20,
0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x101,
0x72, 0x61, 0x74, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x62, 0x101, 0x72, 0x61, 0x74, 0x20, 0x6b, 0x101,
0x6a, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x6e, 0x1e0d, 0x6f, 0x63,
0x69, 0x6e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x64, 0x69,
0x6e, 0x69, 0x20, 0x69, 0x6e, 0x1e0d, 0x6e, 0x65, 0x73, 0x69, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20,
0x69, 0x6e, 0x1e0d, 0x6e, 0x65, 0x73, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x16b, 0x1e47, 0x70, 0x16b, 0x20, 0x69,
0x6e, 0x1e0d, 0x6e, 0x65, 0x73, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x69, 0x72, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x72, 0x101,
0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x69, 0x72, 0x101, 0x6e, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69,
0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x12b, 0x72, 0x6b, 0x75, 0x73, 0x74,
0x61, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x12b, 0x72, 0x6b, 0x75, 0x73,
0x74, 0x61, 0x6b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x12b, 0x72, 0x6b, 0x75, 0x73, 0x74, 0x61, 0x6b,
0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69,
0x6a, 0x61, 0x72, 0x101, 0x12b, 0x6c, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69,
0x6a, 0x61, 0x72, 0x101, 0x12b, 0x6c, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x6a, 0x61, 0x72, 0x101,
0x12b, 0x6c, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6a, 0x101, 0x70, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6a, 0x101, 0x70, 0x101, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6a, 0x101, 0x70, 0x101, 0x6e, 0x20,
0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b,
0x101, 0x6a, 0x101, 0x6b, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x61, 0x70, 0x75, 0x20, 0x6b, 0x101,
0x6a, 0x101, 0x6b, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x16b, 0x1e47, 0x70, 0x16b, 0x20, 0x6b, 0x101,
0x6a, 0x101, 0x6b, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6b, 0x6f, 0x72, 0x69, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b,
0x6f, 0x72, 0x69, 0x101, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x72, 0x69, 0x101, 0x6e,
0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6b, 0x61, 0x73, 0x72, 0x101, 0x113, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b,
0x72, 0x101, 0x73, 0x6e, 0x14d, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6b, 0x72, 0x101, 0x73, 0x6e, 0x14d, 0x72, 0x73, 0x6b, 0x61,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6b, 0x72, 0x101, 0x73, 0x6e, 0x14d, 0x72, 0x73, 0x6b, 0x61, 0x20,
0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x69,
0x72, 0x67, 0x69, 0x6a, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6c, 0x101, 0x69, 0x6e, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6c, 0x61, 0x1e5b, 0x20, 0x68, 0x61, 0x77, 0x65, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6c, 0x61, 0x1e5b, 0x20, 0x68, 0x61, 0x77, 0x65, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6c, 0x61, 0x1e5b, 0x20, 0x68, 0x61, 0x77, 0x65, 0x20, 0x1e0d, 0x65, 0x6c,
0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x67, 0x101,
0x64, 0x61, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x67, 0x101,
0x64, 0x61, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x67, 0x101, 0x64, 0x61, 0x6e, 0x20,
0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x74, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d,
0x61, 0x6c, 0x65, 0x73, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d,
0x101, 0x6c, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101,
0x72, 0x6b, 0x73, 0x61, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101,
0x72, 0x73, 0x101, 0x6c, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6d, 0x61, 0x72, 0x12b, 0x73, 0x61, 0x73, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6d, 0x61, 0x72, 0x12b, 0x73, 0x61, 0x73, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61, 0x72,
0x12b, 0x73, 0x61, 0x73, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6d, 0x101, 0x77, 0x6f, 0x73, 0x6e, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x101, 0x6e, 0x20, 0x70,
0x65, 0x73, 0x69, 0x70, 0x69, 0x63, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x101, 0x6e, 0x20, 0x70, 0x65, 0x73, 0x69,
0x70, 0x69, 0x63, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x101, 0x6e,
0x20, 0x70, 0x65, 0x73, 0x69, 0x70, 0x69, 0x63, 0x20, 0x1e0d, 0x65, 0x6c,
0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x16b, 0x6c, 0x101, 0x6e,
0x62, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x16b, 0x6c,
0x101, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x16b, 0x6c, 0x101, 0x6e,
0x62, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x6f, 0x73, 0x6b,
0x6f, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6d, 0x69, 0xf1, 0x79, 0x101, 0x6d, 0x101, 0x72, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6e, 0x101, 0x75, 0x72, 0x16b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e,
0x65, 0x70, 0x101, 0x6c, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b,
0x20, 0x6b, 0x65, 0x6c, 0x65, 0x1e0d, 0x6f, 0x6e, 0x69, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x1e0d,
0x6f, 0x6e, 0x69, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x20, 0x6b, 0x65, 0x6c,
0x65, 0x1e0d, 0x6f, 0x6e, 0x69, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x6a, 0x69, 0x6c, 0x101,
0x6e, 0x1e0d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x6a, 0x69,
0x6c, 0x101, 0x6e, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x6a, 0x69, 0x6c, 0x101,
0x6e, 0x1e0d, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x70, 0x61, 0x75, 0x6e, 0x1e0d, 0x6c, 0x79,
0x101, 0x6e, 0x1e0d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x70,
0x61, 0x75, 0x6e, 0x1e0d, 0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69,
0x16b, 0x70, 0x61, 0x75, 0x6e, 0x1e0d, 0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20,
0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e,
0x69, 0x16b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x6f, 0x72, 0x70, 0x6f,
0x6b, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x6f,
0x72, 0x70, 0x6f, 0x6b, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x6f, 0x72,
0x70, 0x6f, 0x6b, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x1e0d, 0x65, 0x6c, 0x101,
0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61, 0x72, 0x6e, 0x7e,
0x1e0d, 0x6f, 0x20, 0x1e0d, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6f, 0x68, 0x101,
0x7e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61, 0x72, 0x6e, 0x7e, 0x1e0d,
0x6f, 0x20, 0x1e0d, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6f, 0x68, 0x101, 0x7e,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x61, 0x72, 0x6e, 0x7e, 0x1e0d, 0x6f, 0x20, 0x1e0d, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6f, 0x68, 0x101, 0x7e, 0x20, 0x6b, 0x101, 0x72, 0x7e,
0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x61, 0x77, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x73, 0x72, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x61,
0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73, 0x72, 0x6b, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x61,
0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73, 0x72, 0x6b, 0x20, 0x6b, 0x101,
0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6d, 0x73, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6d, 0x73, 0x6b, 0x61, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x61, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x70, 0x101, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x101, 0x6b, 0x69, 0x73, 0x74, 0x101,
0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x70, 0x101, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x6b,
0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x101, 0x6c,
0x101, 0x75, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x101, 0x70, 0x75, 0x101,
0x20, 0x6e, 0x69, 0x75, 0x20, 0x67, 0x75, 0x6e, 0x12b, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x70, 0x101, 0x72, 0x101, 0x67, 0x75, 0x79, 0x113, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x70, 0x101, 0x72, 0x101, 0x67, 0x75, 0x79, 0x113,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x101, 0x72, 0x101, 0x67, 0x75, 0x79, 0x113, 0x20, 0x6b, 0x101,
0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65, 0x72, 0x75,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65, 0x72, 0x75, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65,
0x72, 0x75, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x70, 0x69, 0x6e, 0x69, 0x6b, 0x73, 0x61, 0x20, 0x64,
0x12b, 0x70, 0x61, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x113,
0x6e, 0x1e6d, 0x20, 0x70, 0x69, 0x65, 0x72, 0x113, 0x20, 0x61, 0x1e0d, 0x113,
0x20, 0x6d, 0x65, 0x6b, 0x77, 0x113, 0x6c, 0x101, 0x6e, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x73, 0x113, 0x6e, 0x1e6d, 0x20, 0x70, 0x69, 0x65, 0x72, 0x113,
0x20, 0x61, 0x1e0d, 0x113, 0x20, 0x6d, 0x65, 0x6b, 0x77, 0x113, 0x6c, 0x101,
0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x73, 0x113, 0x6e, 0x1e6d, 0x20, 0x70, 0x69, 0x65, 0x72, 0x113,
0x20, 0x61, 0x1e0d, 0x113, 0x20, 0x6d, 0x65, 0x6b, 0x77, 0x113, 0x6c, 0x101,
0x6e, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x69, 0x1e6d, 0x6b, 0x113, 0x72, 0x61, 0x6e, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x70, 0x6f, 0x6e, 0x101, 0x70, 0x65, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x79, 0x6f, 0x6e, 0x67, 0x61, 0x79, 0x101, 0x6e, 0x67, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x72, 0x12b, 0x16b, 0x6e, 0x12b, 0x61, 0x6e, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x72, 0x6f, 0x74, 0x113, 0x72, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x73, 0x61, 0x6b, 0x101, 0x6c, 0x69, 0x6e, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x73, 0x61, 0x6b, 0x101, 0x6c, 0x69, 0x6e, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73,
0x61, 0x6b, 0x101, 0x6c, 0x69, 0x6e, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x61, 0x101, 0x6d, 0x6f, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x73, 0x61, 0x101, 0x6d, 0x6f, 0x61, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73,
0x61, 0x101, 0x6d, 0x6f, 0x61, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d,
0x73, 0x113, 0x73, 0x65, 0x6c, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73,
0x69, 0x6e, 0x67, 0x101, 0x70, 0x75, 0x72, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x6f, 0x6c, 0x6f,
0x6d, 0x61, 0x6e, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x74, 0x69, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x64, 0x6b, 0x12b, 0x1e47, 0x61, 0x20, 0x6a, 0x61, 0x72,
0x6a, 0x12b, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x75, 0x72, 0x12b,
0x6e, 0x101, 0x6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x61, 0x77, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x74, 0x101, 0x68, 0x69, 0x74, 0x69, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x74, 0x101, 0x69, 0x70, 0x113, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x74, 0x101, 0x69, 0x70, 0x113, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x74, 0x101, 0x69, 0x70,
0x113, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x74, 0x101, 0x6a, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e6d,
0x6f, 0x6b, 0x65, 0x6c, 0x101, 0x75, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e6d,
0x61, 0x6e, 0x67, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e6d, 0x61, 0x6e,
0x67, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x1e6d, 0x61, 0x6e, 0x67, 0x101, 0x20, 0x6b, 0x101, 0x72,
0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x75, 0x6b, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x74, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x6d, 0x61, 0x6e,
0x61, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x74, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x6b, 0x101, 0x72,
0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x74, 0x75, 0x77, 0x61, 0x6c,
0x16b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x75, 0x72, 0x67, 0x75, 0x77, 0x113,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x75, 0x72, 0x67, 0x75, 0x77, 0x113, 0x20,
0x6d, 0x61, 0x6e, 0x61, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x75,
0x72, 0x67, 0x75, 0x77, 0x113, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x75, 0x7a, 0x77, 0x113, 0x6b, 0x69, 0x73, 0x74,
0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x75, 0x7a, 0x77, 0x113, 0x6b,
0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x75, 0x7a, 0x77, 0x113, 0x6b, 0x69,
0x73, 0x74, 0x101, 0x6e, 0x20, 0x6b, 0x101, 0x72, 0xe3, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x77, 0x61, 0x6e, 0x75, 0x101, 0x74, 0x16b, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x77, 0x61, 0x6e, 0x75, 0x101, 0x74, 0x16b, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x61,
0x6e, 0x75, 0x101, 0x74, 0x16b, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x6e, 0x113, 0x7a, 0x75, 0x65, 0x6c,
0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x6c, 0x101, 0x64, 0x69, 0x77,
0x6f, 0x73, 0x74, 0x61, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x6c,
0x101, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x61, 0x6b, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x6c,
0x101, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x61, 0x6b, 0x20, 0x6b, 0x101,
0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x101, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x101, 0x1e0d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x101,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x101, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x101, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x101, 0x1e0d, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x77, 0x61, 0x6b, 0x61, 0x20, 0x64, 0x12b, 0x70,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x61, 0x6c, 0x69, 0x73, 0x20,
0x61, 0x1e0d, 0x113, 0x20, 0x70, 0x75, 0x1e6d, 0x75, 0x6e, 0x101, 0x79, 0x61,
0x6b, 0x75, 0x73, 0x74, 0x75, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x79,
0x61, 0x6b, 0x75, 0x73, 0x74, 0x75, 0x6b, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x79, 0x61, 0x6b, 0x75,
0x73, 0x74, 0x75, 0x6b, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x79, 0x61, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x61, 0x72, 0x67, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x79, 0x61, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x72, 0x67, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x79,
0x61, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x72, 0x67,
0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x906,
0x92a, 0x917, 0x93e, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x906, 0x92a,
0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947,
0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x906, 0x92a, 0x94d,
0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x926, 0x915, 0x94d,
0x937, 0x93f, 0x923, 0x20, 0x906, 0x92a, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x938, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x906, 0x92a, 0x94d, 0x930, 0x93f,
0x915, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x938, 0x94d, 0x91a, 0x93f,
0x92e, 0x20, 0x906, 0x92a, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x938,
0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x906, 0x92a, 0x94d, 0x930, 0x93f, 0x915, 0x93e,
0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x200d, 0x915, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x200d, 0x915, 0x93e, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x92e, 0x947,
0x91c, 0x93c, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x92e, 0x947, 0x91c,
0x93c, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x905, 0x92e, 0x947, 0x91c, 0x93c, 0x928, 0x20, 0x915, 0x93e,
0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902,
0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x926, 0x93f, 0x928,
0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949,
0x92a, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a,
0x941, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x939, 0x949, 0x930, 0x94d, 0x915, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x939, 0x949, 0x930, 0x94d, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902,
0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x949, 0x930, 0x94d, 0x915,
0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92a, 0x947, 0x938, 0x93f, 0x92a, 0x93f, 0x915, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92a, 0x947, 0x938, 0x93f, 0x92a, 0x93f, 0x915, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x947, 0x938,
0x93f, 0x92a, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x930,
0x92c, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x930, 0x92c, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x930,
0x92c, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940,
0x928, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928,
0x93e, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915,
0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902,
0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921,
0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x905, 0x930,
0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921,
0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x905, 0x930,
0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x915, 0x93e, 0x930,
0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x915, 0x93e,
0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905,
0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x926, 0x93f, 0x928,
0x940, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x924, 0x93f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x92e,
0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x926,
0x93f, 0x928, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940,
0x20, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a,
0x941, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x926,
0x93f, 0x928, 0x940, 0x20, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941,
0x923, 0x94d, 0x92a, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x926, 0x93f,
0x928, 0x940, 0x20, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923,
0x94d, 0x92a, 0x941, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a,
0x941, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x935, 0x947, 0x921,
0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x93e,
0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x911, 0x938, 0x94d, 0x200d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x935,
0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921,
0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x911, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x924, 0x93f, 0x20, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941,
0x923, 0x94d, 0x92a, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x935, 0x947, 0x921, 0x93c, 0x93e,
0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91c, 0x930, 0x92c, 0x93e,
0x907, 0x91c, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91c, 0x930,
0x92c, 0x93e, 0x907, 0x91c, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902,
0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91c, 0x930, 0x92c, 0x93e,
0x907, 0x91c, 0x93e, 0x928, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91c, 0x93c, 0x94b,
0x930, 0x947, 0x938, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91c, 0x93c, 0x94b,
0x930, 0x947, 0x938, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20,
0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947,
0x938, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932,
0x93e, 0x926, 0x947, 0x938, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e,
0x926, 0x947, 0x938, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e,
0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x941, 0x91f, 0x93e, 0x928,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x94b, 0x932, 0x940, 0x935, 0x93f, 0x92f,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c,
0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x94d,
0x930, 0x93e, 0x91c, 0x93c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x94d,
0x930, 0x93e, 0x91c, 0x93c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x915, 0x93e,
0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92c, 0x94d, 0x930, 0x942, 0x928, 0x947, 0x924, 0x93f, 0x20, 0x926, 0x93e,
0x930, 0x942, 0x938, 0x94d, 0x938, 0x932, 0x92e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x947,
0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901,
0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x93e,
0x92e, 0x94b, 0x930, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x94d, 0x92f, 0x93e, 0x924, 0x93e, 0x92e,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x94d, 0x92f, 0x93e, 0x924, 0x93e, 0x92e,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x91a, 0x94d, 0x92f, 0x93e, 0x924, 0x93e, 0x92e, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x93f, 0x932, 0x940,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x93f,
0x932, 0x940, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x940, 0x928, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x91a, 0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x940, 0x928, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94d, 0x930,
0x93f, 0x938, 0x92e, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x915, 0x94b, 0x915, 0x94b, 0x938, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94b, 0x932, 0x902, 0x92c,
0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94b, 0x932, 0x902,
0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f,
0x93e, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x941, 0x915, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x20, 0x906, 0x927, 0x93e, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901,
0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94d,
0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94d, 0x92f,
0x942, 0x92c, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x921, 0x947,
0x935, 0x93f, 0x938, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x921, 0x94d, 0x92f, 0x942,
0x92e, 0x94b, 0x902, 0x91f, 0x20, 0x921, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x935,
0x93f, 0x932, 0x947, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c,
0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x924, 0x93f, 0x92e, 0x94b, 0x930,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x908, 0x938, 0x94d,
0x91f, 0x930, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x908, 0x938, 0x94d,
0x91f, 0x930, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x915, 0x93e, 0x930,
0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x907, 0x915, 0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x2d, 0x924, 0x93f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x926,
0x93f, 0x928, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92e, 0x93e, 0x926, 0x93f, 0x928, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x2d, 0x924, 0x93f, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c,
0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a,
0x2d, 0x924, 0x93f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c,
0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a,
0x2d, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949,
0x92a, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f, 0x20,
0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x911, 0x930, 0x949, 0x20, 0x935, 0x947, 0x921, 0x93c, 0x93e,
0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x2d,
0x924, 0x93f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e,
0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x2d, 0x924, 0x93f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921,
0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x92f, 0x941,
0x930, 0x94b, 0x92a, 0x2d, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902,
0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e,
0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x2d, 0x924, 0x93f, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93e, 0x915, 0x20,
0x2d, 0x932, 0x947, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93e, 0x915, 0x20, 0x2d, 0x932, 0x947, 0x902,
0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92e, 0x93e, 0x928, 0x93e,
0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93e, 0x915, 0x20,
0x2d, 0x932, 0x947, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20,
0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92a, 0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92a, 0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902,
0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93c, 0x93f, 0x91c, 0x940,
0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93c, 0x94d, 0x930, 0x947, 0x902, 0x91a, 0x20,
0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x926,
0x949, 0x915, 0x93f, 0x923, 0x949, 0x20, 0x92a, 0x93c, 0x94d, 0x930, 0x93e, 0x902,
0x938, 0x20, 0x911, 0x921, 0x93c, 0x947, 0x20, 0x905, 0x902, 0x91f, 0x93e, 0x930,
0x94d, 0x915, 0x91f, 0x93f, 0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x917, 0x94d,
0x92f, 0x93e, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x20, 0x924, 0x93f,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x917, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92c,
0x940, 0x92f, 0x930, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91c, 0x949, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91c, 0x949, 0x930,
0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f,
0x92f, 0x93e, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930,
0x94d, 0x91f, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x940,
0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20,
0x939, 0x949, 0x92a, 0x941, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947,
0x902, 0x921, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e,
0x20, 0x939, 0x949, 0x92a, 0x941, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932,
0x947, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949,
0x92a, 0x941, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x902, 0x921,
0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941,
0x923, 0x94d, 0x92a, 0x941, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947,
0x902, 0x921, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e,
0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x947, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20,
0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x932, 0x947, 0x902, 0x921, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x917, 0x932, 0x94d, 0x92a,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x939, 0x935, 0x93e, 0x924, 0x93f, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942,
0x938, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x935, 0x93e, 0x924, 0x93f,
0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x938, 0x928, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x935, 0x93e,
0x924, 0x93f, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x938, 0x928, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x93e,
0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x93e,
0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x930, 0x93e,
0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939,
0x94b, 0x935, 0x94d, 0x921, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x939, 0x94b, 0x935,
0x94d, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x915, 0x93e, 0x930,
0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92c, 0x93e, 0x930, 0x924, 0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902,
0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92c, 0x93e, 0x930, 0x949, 0x924,
0x20, 0x915, 0x93e, 0x91c, 0x93e, 0x20, 0x938, 0x93e, 0x92e, 0x941, 0x926, 0x94d,
0x930, 0x940, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x902, 0x921, 0x94b, 0x91a,
0x93e, 0x907, 0x928, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x927, 0x94d,
0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x938, 0x93f, 0x92f, 0x93e,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939,
0x949, 0x92a, 0x941, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x938, 0x93f,
0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e,
0x20, 0x915, 0x941, 0x923, 0x94d, 0x92a, 0x941, 0x20, 0x907, 0x902, 0x921, 0x94b,
0x928, 0x947, 0x938, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907,
0x930, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x930, 0x93e, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x924, 0x93f, 0x930, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x930, 0x94d,
0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x93e,
0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x930, 0x94d, 0x915,
0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901,
0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x91c,
0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x91c,
0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907,
0x932, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91c,
0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x91c,
0x93c, 0x93e, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x939, 0x949, 0x92a, 0x941, 0x20,
0x915, 0x91c, 0x93c, 0x93e, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x947, 0x921, 0x93c, 0x93e, 0x20, 0x915, 0x941, 0x923,
0x94d, 0x92a, 0x941, 0x20, 0x915, 0x91c, 0x93c, 0x93e, 0x915, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94b, 0x930, 0x93f, 0x92f,
0x93e, 0x924, 0x93f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94b, 0x930, 0x93f,
0x92f, 0x93e, 0x924, 0x93f, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x924,
0x93f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x915, 0x94b, 0x938, 0x930, 0x93e, 0x90f, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x94d, 0x930, 0x93e,
0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e,
0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x915, 0x93f, 0x930, 0x94d, 0x917,
0x93f, 0x938, 0x94d, 0x200d, 0x924, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x932, 0x93e, 0x907, 0x928, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935,
0x947, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20,
0x939, 0x94b, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939,
0x94b, 0x935, 0x947, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e,
0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901,
0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x932,
0x947, 0x938, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e,
0x932, 0x926, 0x940, 0x935, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x930,
0x94d, 0x915, 0x947, 0x938, 0x938, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e,
0x930, 0x94d, 0x938, 0x932, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92e, 0x949, 0x930, 0x940, 0x938, 0x938, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x92e, 0x949, 0x930, 0x940, 0x938, 0x938, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x949, 0x930,
0x940, 0x938, 0x938, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e,
0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x93e, 0x935, 0x94d, 0x938,
0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f,
0x915, 0x94b, 0x924, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92a, 0x93f, 0x915,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915,
0x94b, 0x924, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92a, 0x93f, 0x915, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x924, 0x93f, 0x20, 0x92a,
0x947, 0x938, 0x93f, 0x92a, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c,
0x91f, 0x94b, 0x930, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x909, 0x932, 0x93e, 0x928,
0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c,
0x91f, 0x94b, 0x930, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e,
0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x949, 0x938, 0x94d, 0x915,
0x94b, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x915, 0x93e, 0x930, 0x93e,
0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92e,
0x94d, 0x92f, 0x93e, 0x902, 0x92e, 0x93e, 0x930, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x928, 0x949, 0x909, 0x930, 0x941, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x947,
0x92a, 0x93e, 0x932, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942,
0x20, 0x915, 0x947, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x947, 0x932,
0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e,
0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942,
0x20, 0x915, 0x947, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932, 0x947,
0x902, 0x921, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x91c,
0x93c, 0x940, 0x932, 0x947, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902,
0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x91c,
0x93c, 0x940, 0x932, 0x947, 0x902, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x92a, 0x93c,
0x93e, 0x909, 0x902, 0x921, 0x932, 0x947, 0x902, 0x921, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x92a, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932,
0x947, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x92a, 0x93c, 0x93e, 0x909,
0x902, 0x921, 0x932, 0x947, 0x902, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x940, 0x92f, 0x942, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x928, 0x949, 0x930, 0x92a, 0x93c, 0x949, 0x915, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x949, 0x930,
0x92a, 0x93c, 0x949, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928,
0x949, 0x930, 0x92a, 0x93c, 0x949, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20,
0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930,
0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928,
0x94d, 0x939, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930,
0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928,
0x94d, 0x939, 0x93e, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e,
0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x928, 0x94b, 0x935, 0x94b, 0x938,
0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x913, 0x92e, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x913, 0x92e, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x915, 0x93e, 0x930,
0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x915, 0x93e,
0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92a, 0x932, 0x93e, 0x909, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93e,
0x92a, 0x941, 0x906, 0x20, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917, 0x93f, 0x928,
0x940, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d,
0x935, 0x947, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x948, 0x930, 0x93e, 0x917,
0x94d, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947,
0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x915,
0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x92a, 0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x92a, 0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20,
0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x915, 0x93e, 0x930,
0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e,
0x92a, 0x93c, 0x940, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20,
0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x911, 0x921, 0x93c, 0x947, 0x20, 0x92e,
0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20,
0x911, 0x921, 0x93c, 0x947, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932,
0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930,
0x947, 0x20, 0x911, 0x921, 0x93c, 0x947, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935,
0x947, 0x932, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x93f, 0x91f, 0x915, 0x947, 0x930, 0x94d, 0x928,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x94b, 0x928, 0x93e, 0x92a, 0x947, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x92a, 0x94d, 0x92f, 0x94b, 0x902, 0x917, 0x92f, 0x93e,
0x902, 0x917, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x930, 0x940, 0x92f, 0x942, 0x928,
0x93f, 0x92f, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x930, 0x94b, 0x925, 0x947,
0x930, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x915, 0x93e, 0x932, 0x93f,
0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x915, 0x93e, 0x932, 0x93f, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x938, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x915, 0x93e, 0x930, 0x93e,
0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938,
0x92e, 0x94b, 0x906, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x92e, 0x94b, 0x906,
0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x947, 0x938, 0x947, 0x932, 0x94d, 0x938,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x941,
0x930, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x928,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x926,
0x915, 0x94d, 0x937, 0x93f, 0x923, 0x940, 0x20, 0x91c, 0x949, 0x930, 0x94d, 0x91c,
0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x942, 0x930, 0x940,
0x928, 0x93e, 0x92e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x938, 0x94d, 0x92f, 0x94b,
0x935, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x924, 0x93e, 0x939, 0x93f, 0x924,
0x940, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x924, 0x93e,
0x907, 0x92a, 0x947, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x924, 0x93e, 0x91c, 0x93f, 0x915, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91f, 0x94b, 0x915, 0x947, 0x932,
0x93e, 0x90a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91f, 0x94b, 0x902, 0x917, 0x93e,
0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91f,
0x94b, 0x902, 0x917, 0x93e, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x91a, 0x941, 0x915, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x924,
0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e,
0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x924, 0x941, 0x935, 0x93e,
0x932, 0x942, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x909, 0x930, 0x941, 0x917, 0x94d,
0x935, 0x947, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x909, 0x930, 0x941, 0x917, 0x94d,
0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x915,
0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x909, 0x91c, 0x93c, 0x94d,
0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e,
0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x909, 0x91c,
0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x928, 0x941, 0x906, 0x924, 0x942, 0x20, 0x92c, 0x947,
0x932, 0x93e, 0x935, 0x928, 0x941, 0x906, 0x924, 0x942, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x928, 0x941,
0x906, 0x924, 0x942, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e,
0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x928, 0x947, 0x91c,
0x93c, 0x941, 0x90f, 0x932, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x94d,
0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b,
0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f,
0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x915, 0x93e, 0x930, 0x93e,
0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935,
0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930,
0x93e, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x935, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930,
0x93e, 0x921, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901, 0x20, 0x92e, 0x93e, 0x938,
0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x947, 0x915, 0x20, 0x926, 0x94d,
0x935, 0x940, 0x92a, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x935, 0x93e, 0x932, 0x93f,
0x938, 0x20, 0x911, 0x921, 0x93c, 0x947, 0x20, 0x92a, 0x93c, 0x94d, 0x92f, 0x942,
0x91a, 0x942, 0x928, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92f, 0x93e, 0x915,
0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92f,
0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928,
0x93e, 0x902, 0x915, 0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92f, 0x93e, 0x915,
0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x915, 0x93e, 0x930, 0x93e, 0x901,
0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92f, 0x947,
0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20,
0x92c, 0x947, 0x932, 0x93e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f,
0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947,
0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x915, 0x93e, 0x930, 0x93e,
0x901, 0x20, 0x92e, 0x93e, 0x938, 0x93e, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0xb06,
0xb2a, 0xb17, 0xb3e, 0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb06, 0xb2a,
0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47,
0xb21, 0xb3c, 0xb3e, 0x20, 0xb39, 0xb2a, 0xb41, 0x20, 0xb06, 0xb2a, 0xb4d, 0xb30,
0xb3f, 0xb15, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb26, 0xb15, 0xb3f, 0xb23,
0x20, 0xb06, 0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21,
0xb3c, 0xb3e, 0x20, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb06,
0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71,
0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41,
0x20, 0xb06, 0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21,
0xb3c, 0xb3e, 0x20, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb06,
0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb06, 0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb21, 0xb47, 0xb32,
0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb2e, 0xb47,
0xb1c, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb2e, 0xb47, 0xb1c, 0xb28,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb06, 0xb2e, 0xb47, 0xb1c, 0xb28, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e,
0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f,
0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39,
0xb2a, 0xb41, 0xb24, 0xb3f, 0x20, 0xb1c, 0xb3e, 0xb17, 0xb3e, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb1c,
0xb3e, 0xb17, 0xb3e, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39,
0xb2a, 0xb41, 0x20, 0xb1c, 0xb3e, 0xb17, 0xb3e, 0xb24, 0xb3f, 0x20, 0xb21, 0xb47,
0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb30,
0xb4d, 0xb15, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb30, 0xb4d, 0xb15,
0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb39, 0xb30, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e,
0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb38, 0xb3f,
0xb2a, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb38,
0xb3f, 0xb2a, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb38, 0xb3f, 0xb2a, 0xb3f,
0xb15, 0xb4d, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb0f, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb0f, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb0f, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb21, 0xb47,
0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30,
0xb2c, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30,
0xb2c, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30, 0xb2c, 0xb3f, 0xb06, 0xb24,
0xb3f, 0x20, 0xb2e, 0xb47, 0xb26, 0xb3e, 0xb23, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb06, 0xb30, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f,
0xb28, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f,
0xb28, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a,
0xb41, 0x20, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28,
0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20,
0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb06, 0xb30, 0xb4d, 0xb1c,
0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c,
0xb3e, 0x20, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb06, 0xb30,
0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb15, 0xb3e,
0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30, 0xb2e, 0xb47,
0xb28, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30, 0xb2e,
0xb47, 0xb28, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb30, 0xb2e, 0xb47, 0xb28, 0xb3f,
0xb5f, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb06, 0xb1f, 0xb4d, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb06, 0xb1f, 0xb4d, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f,
0xb3f, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb06, 0xb1f, 0xb4d, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb3f,
0xb15, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb05, 0xb38, 0xb4d, 0xb1f,
0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05,
0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20,
0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d,
0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb26, 0xb3f,
0xb28, 0xb3f, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06,
0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb71, 0xb47,
0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f,
0xb06, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb71,
0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f,
0x20, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20, 0xb71, 0xb47, 0xb21, 0xb3c,
0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb21, 0xb47, 0xb32,
0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c,
0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47,
0xb32, 0xb3f, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f,
0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb71, 0xb47, 0xb21,
0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30,
0xb47, 0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e,
0xb39, 0xb2a, 0xb41, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d,
0x200d, 0xb2a, 0xb41, 0x20, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32,
0xb3f, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d, 0xb1f, 0xb4d,
0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb71, 0xb47, 0xb21, 0xb3c,
0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb38, 0xb4d,
0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb71, 0xb47,
0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb21,
0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb1c,
0xb47, 0xb30, 0xb2c, 0xb3e, 0xb07, 0xb1c, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb05, 0xb1c, 0xb47, 0xb30, 0xb2c, 0xb3e, 0xb07, 0xb1c, 0xb3e, 0xb28, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb05, 0xb1c, 0xb47, 0xb30, 0xb2c, 0xb3e, 0xb07, 0xb1c, 0xb3e, 0xb28, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb1c, 0xb30,
0xb47, 0xb38, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb1c, 0xb30, 0xb47, 0xb38,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb05, 0xb1c, 0xb30, 0xb47, 0xb38, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb32, 0xb3e,
0xb26, 0xb47, 0xb38, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb3e, 0xb19,
0xb4d, 0xb17, 0xb32, 0xb3e, 0xb26, 0xb47, 0xb38, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb3e, 0xb19,
0xb4d, 0xb17, 0xb32, 0xb3e, 0xb26, 0xb47, 0xb38, 0xb4d, 0x20, 0xb15, 0xb3e, 0xb30,
0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb41, 0xb1f, 0xb3e, 0xb28,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb32, 0xb71, 0xb3f, 0xb06, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb1c, 0xb3f, 0xb32, 0xb3f, 0xb5f,
0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb1c, 0xb3f,
0xb32, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb1c, 0xb3f, 0xb32, 0xb3f, 0xb5f, 0xb3e,
0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c,
0xb4d, 0xb30, 0xb41, 0xb28, 0xb47, 0xb07, 0x20, 0xb26, 0xb3e, 0xb30, 0xb41, 0xb38,
0xb3e, 0xb32, 0xb3e, 0xb2e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb47, 0xb2a,
0x20, 0xb2c, 0xb30, 0xb4d, 0xb21, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb47,
0xb2a, 0x20, 0xb2c, 0xb30, 0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb47, 0xb2a, 0x20, 0xb2c,
0xb30, 0xb4d, 0xb21, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb1a, 0xb2e, 0xb30, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3e, 0xb24, 0xb3e, 0xb2e, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3e, 0xb24, 0xb3e, 0xb2e, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3e,
0xb24, 0xb3e, 0xb2e, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3f, 0xb32, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb1a, 0xb3f, 0xb32, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3f, 0xb32, 0xb3f, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3f, 0xb28,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3f, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1a, 0xb3f, 0xb28,
0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb15, 0xb4d, 0xb30, 0xb3f, 0xb38, 0xb2e, 0xb3e, 0xb38, 0x20, 0xb26, 0xb3f, 0xb2a,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb15, 0xb38, 0x20, 0xb26, 0xb3f, 0xb2a,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb5f,
0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f,
0xb5f, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb5f, 0xb3e, 0x20,
0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb41,
0xb15, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb41,
0xb15, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb41, 0xb15, 0x20, 0xb26, 0xb3f,
0xb2a, 0x20, 0xb06, 0xb26, 0xb4d, 0xb26, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e,
0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb4d, 0xb71, 0xb41, 0xb2c, 0xb3e,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb4d, 0xb71, 0xb41, 0xb2c, 0xb3e, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb15, 0xb4d, 0xb71, 0xb41, 0xb2c, 0xb3e, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07,
0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb21, 0xb47, 0xb2c, 0xb3f, 0xb38,
0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb21, 0xb4d, 0xb5f, 0xb41, 0xb2e, 0xb3e,
0xb23, 0xb4d, 0xb1f, 0x20, 0xb21, 0xb3f, 0x20, 0xb05, 0xb30, 0xb71, 0xb3f, 0xb32,
0xb47, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39,
0xb2a, 0xb41, 0x20, 0xb24, 0xb3f, 0xb2e, 0xb30, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb07, 0xb38, 0xb4d, 0xb1f, 0xb30, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb07, 0xb38, 0xb4d, 0xb1f, 0xb30, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb07, 0xb38, 0xb4d, 0xb1f, 0xb30, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb15, 0xb3e,
0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb15, 0xb4d, 0xb71,
0xb3e, 0xb21, 0xb30, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f,
0xb28, 0xb3f, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb24, 0xb3f,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20,
0xb5f, 0xb41, 0xb30, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e,
0xb26, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb3f, 0xb06, 0x20,
0xb24, 0xb3f, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb39, 0xb2a, 0xb41, 0x20, 0xb5f,
0xb41, 0xb30, 0xb2a, 0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47,
0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb3f,
0xb06, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41,
0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb15, 0xb3e,
0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb30, 0x20, 0xb71,
0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a,
0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e,
0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a,
0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e,
0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a,
0xb3f, 0xb06, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41,
0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb5f, 0xb41, 0xb30, 0xb2a, 0xb24, 0xb3f,
0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a,
0xb3e, 0xb15, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb15, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21,
0x20, 0xb26, 0xb3f, 0xb2a, 0xb24, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb15, 0xb32, 0xb3e,
0xb23, 0xb4d, 0xb21, 0x20, 0xb26, 0xb3f, 0xb2a, 0xb24, 0xb3f, 0x20, 0xb15, 0xb3e,
0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3f, 0xb1c, 0xb3f,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3f, 0xb1c, 0xb3f, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3f,
0xb1c, 0xb3f, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2a, 0xb4d, 0xb30, 0xb47, 0xb1e, 0xb4d, 0xb1a, 0xb4d, 0x20, 0xb17, 0xb41,
0xb5f, 0xb3e, 0xb28, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb4d, 0xb30,
0xb47, 0xb1e, 0xb4d, 0xb1a, 0x20, 0xb26, 0xb15, 0xb3f, 0xb23, 0x20, 0xb05, 0xb21,
0xb3c, 0xb47, 0x20, 0xb06, 0xb23, 0xb4d, 0xb1f, 0xb3e, 0xb30, 0xb1f, 0xb3f, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb17, 0xb32, 0xb3e, 0xb2a, 0xb17, 0xb38, 0x20,
0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb17, 0xb2e, 0xb4d, 0xb2c, 0xb3f,
0xb5f, 0xb30, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f,
0xb5f, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f,
0xb5f, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb17, 0xb3f, 0xb32, 0xb4d,
0x200d, 0xb2c, 0xb47, 0xb30, 0xb4d, 0xb1f, 0x20, 0xb26, 0xb3f, 0xb2a, 0xb24, 0xb3f,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb71, 0xb3f,
0xb1a, 0x20, 0xb2e, 0xb3f, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47,
0xb21, 0xb3c, 0xb3e, 0x20, 0xb39, 0xb2a, 0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb3f,
0xb28, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71,
0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb39, 0xb2a, 0xb41, 0x20, 0xb17, 0xb4d, 0xb30,
0xb3f, 0xb28, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c,
0xb3e, 0x20, 0xb39, 0xb2a, 0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb32,
0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0x20, 0xb15, 0xb41, 0xb23,
0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb32, 0xb3e,
0xb23, 0xb4d, 0xb21, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c,
0xb3e, 0x20, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb17, 0xb4d,
0xb30, 0xb3f, 0xb28, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21,
0xb3c, 0xb3e, 0x20, 0xb15, 0xb41, 0xb23, 0xb4d, 0x200d, 0xb2a, 0xb41, 0x20, 0xb17,
0xb4d, 0xb30, 0xb3f, 0xb28, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb15, 0xb3e,
0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb17, 0xb32, 0xb4d, 0xb2a,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb39, 0xb3e, 0xb71, 0xb3e, 0xb07, 0x2d, 0xb06, 0xb32, 0xb4d, 0xb5f, 0xb41,
0xb38, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb3e, 0xb71, 0xb3e,
0xb07, 0x2d, 0xb06, 0xb32, 0xb4d, 0xb5f, 0xb41, 0xb38, 0xb3e, 0xb28, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39,
0xb3e, 0xb71, 0xb3e, 0xb07, 0x2d, 0xb06, 0xb32, 0xb4d, 0xb5f, 0xb41, 0xb38, 0xb3e,
0xb28, 0x20, 0xb21, 0xb47, 0x20, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb39, 0xb19, 0xb4d, 0xb17, 0x20, 0xb15, 0xb19, 0xb4d, 0xb17, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb19, 0xb4d, 0xb17, 0x20, 0xb15, 0xb19, 0xb4d,
0xb17, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb39, 0xb19, 0xb4d, 0xb17, 0x20, 0xb15, 0xb19, 0xb4d, 0xb15, 0x20,
0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb71,
0xb21, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb71, 0xb21, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb39, 0xb71,
0xb21, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2c, 0xb3e, 0xb30, 0xb24, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2c, 0xb3e, 0xb30, 0xb24, 0x20, 0xb15, 0xb3e,
0xb1c, 0xb3e, 0x20, 0xb38, 0xb3e, 0xb2e, 0xb41, 0xb26, 0xb4d, 0xb30, 0xb3f, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb1a, 0xb3f, 0xb28, 0xb3e,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb26, 0xb3f, 0xb28, 0xb3f, 0x20,
0xb07, 0xb23, 0xb4d, 0xb21, 0xb28, 0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41, 0x20, 0xb07,
0xb23, 0xb4d, 0xb21, 0xb28, 0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23, 0xb4d, 0xb2a, 0xb41,
0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb28, 0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb07, 0xb30, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb07, 0xb30, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb30, 0xb3e, 0xb28, 0x20, 0xb21, 0xb47,
0x20, 0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07,
0xb30, 0xb15, 0xb41, 0xb38, 0xb4d, 0xb24, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb07, 0xb30, 0xb15, 0xb41, 0xb38, 0xb4d, 0xb24, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb30, 0xb15,
0xb41, 0xb38, 0xb4d, 0xb24, 0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1c, 0xb30, 0xb3e, 0xb07, 0xb32, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb07, 0xb1c, 0xb30, 0xb3e, 0xb07, 0xb32, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1c,
0xb30, 0xb3e, 0xb07, 0xb32, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1c, 0xb3e, 0xb2a, 0xb3e,
0xb28, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb15, 0xb1c, 0xb3e, 0xb15, 0xb3e, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb39, 0xb2a, 0xb41,
0x20, 0xb15, 0xb1c, 0xb3e, 0xb15, 0xb3e, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb47, 0xb21, 0xb3c, 0xb3e, 0xb15, 0xb41, 0xb23,
0xb4d, 0xb2a, 0xb41, 0x20, 0xb15, 0xb1c, 0xb3e, 0xb15, 0xb3e, 0xb38, 0xb4d, 0xb24,
0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb30, 0xb3f, 0xb5f, 0xb3e,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb30, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15,
0xb30, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb38, 0xb30, 0xb3e, 0xb0f, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb4d, 0xb28, 0xb5f, 0xb3e, 0xb30,
0xb4d, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb15, 0xb4d, 0xb30,
0xb3e, 0xb38, 0xb4d, 0xb28, 0xb5f, 0xb3e, 0xb30, 0xb4d, 0xb38, 0xb4d, 0xb15, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb15, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb4d, 0xb28, 0xb3e, 0xb30, 0xb4d, 0xb38, 0xb4d,
0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb15, 0xb3f, 0xb30, 0xb4d, 0xb17, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb32, 0xb3e, 0xb07, 0xb28, 0x20, 0xb26, 0xb3f, 0xb2a,
0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20,
0xb39, 0xb71, 0xb47, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb32, 0xb30, 0xb4d, 0xb21,
0x20, 0xb39, 0xb71, 0xb47, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb71,
0xb47, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0xb4d, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb26, 0xb3e, 0xb28, 0x20, 0xb26, 0xb3f,
0xb2a, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb26, 0xb3e,
0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb26, 0xb3e, 0xb28, 0x20, 0xb15, 0xb3e, 0xb30,
0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb32, 0xb47, 0xb38,
0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e, 0xb32, 0xb21,
0xb4d, 0xb71, 0xb3f, 0xb38, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e,
0xb30, 0xb4d, 0xb15, 0xb38, 0xb38, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb3e,
0xb30, 0xb4d, 0xb38, 0xb3e, 0xb32, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb2e, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2e, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb30, 0xb3f, 0xb38,
0xb38, 0xb4d, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2e, 0xb3e, 0xb71, 0xb38, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e,
0xb47, 0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb24, 0xb3f, 0x20, 0xb2a, 0xb47, 0xb38,
0xb3f, 0xb2a, 0xb3f, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb47, 0xb15,
0xb4d, 0xb38, 0xb3f, 0xb15, 0xb24, 0xb3f, 0x20, 0xb2a, 0xb47, 0xb38, 0xb3f, 0xb2a,
0xb3f, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb2e, 0xb47, 0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb24, 0xb3f,
0x20, 0xb2a, 0xb47, 0xb38, 0xb3f, 0xb2a, 0xb3f, 0xb15, 0x20, 0xb21, 0xb47, 0xb32,
0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb32, 0xb3e, 0xb28,
0x20, 0xb2c, 0xb1f, 0xb30, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb32, 0xb3e,
0xb28, 0x20, 0xb2c, 0xb1f, 0xb30, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb32, 0xb3e, 0xb28, 0x20, 0xb2c,
0xb1f, 0xb30, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2e, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb38,
0xb4d, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb2e, 0xb38, 0xb4d, 0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e,
0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb01, 0xb2e,
0xb3e, 0xb30, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb09, 0xb30, 0xb41, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb47, 0xb2a, 0xb3e, 0xb32, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb15, 0xb47, 0xb32, 0xb47, 0xb21,
0xb28, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb5f,
0xb41, 0x20, 0xb15, 0xb47, 0xb32, 0xb47, 0xb21, 0xb28, 0xb3f, 0xb5f, 0xb3e, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb15, 0xb47, 0xb32, 0xb47, 0xb21, 0xb28, 0xb3f,
0xb5f, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c, 0xb3f, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c, 0xb3f, 0xb32,
0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c, 0xb3f, 0xb32,
0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2a, 0xb3e, 0xb09, 0xb23,
0xb4d, 0xb21, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2a, 0xb3e, 0xb09, 0xb23, 0xb4d, 0xb21, 0xb32, 0xb3e,
0xb23, 0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2a, 0xb3e, 0xb09, 0xb23,
0xb4d, 0xb21, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e,
0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb3f, 0xb5f, 0xb41, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb30, 0xb2a, 0xb3e, 0xb01, 0xb15, 0x20, 0xb26,
0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb30, 0xb2a, 0xb3e, 0xb01,
0xb15, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb30, 0xb2a, 0xb3e, 0xb01, 0xb15,
0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb30, 0xb4d, 0xb28, 0xb3e, 0xb21, 0x20,
0xb21, 0xb47, 0x20, 0xb28, 0xb30, 0xb39, 0xb4d, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2a, 0xb30, 0xb4d, 0xb28, 0xb3e, 0xb21, 0x20, 0xb21, 0xb47, 0x20, 0xb28,
0xb30, 0xb39, 0xb4d, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb30, 0xb4d, 0xb28, 0xb3e, 0xb21,
0x20, 0xb21, 0xb47, 0x20, 0xb28, 0xb30, 0xb39, 0xb4d, 0xb28, 0x20, 0xb15, 0xb3e,
0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb28, 0xb71, 0xb38, 0xb3f,
0xb2c, 0xb3f, 0xb30, 0xb4d, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb28, 0xb71, 0xb38, 0xb3f, 0xb2c, 0xb3f, 0xb30, 0xb4d, 0xb38, 0xb4d, 0xb15, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb28, 0xb71, 0xb38, 0xb3f, 0xb2c, 0xb30, 0xb4d, 0xb38, 0xb4d, 0xb15, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb32, 0xb3e, 0xb05, 0xb2e, 0xb38, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb05, 0xb2e, 0xb38, 0xb4d, 0xb15, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb05, 0xb2e, 0xb38, 0xb4d, 0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e,
0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb15, 0xb3f, 0xb38, 0xb4d,
0xb24, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e,
0xb28, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2a, 0xb32, 0xb3e, 0xb09, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb2a,
0xb41, 0xb5f, 0xb3e, 0x20, 0xb2a, 0xb41, 0xb28, 0xb3f, 0x20, 0xb17, 0xb3f, 0xb28,
0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb30, 0xb3e, 0xb17, 0xb4d,
0xb71, 0xb47, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb30, 0xb3e, 0xb17,
0xb4d, 0xb71, 0xb47, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb30, 0xb3e, 0xb17, 0xb4d, 0xb71, 0xb47,
0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a,
0xb47, 0xb30, 0xb41, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb47, 0xb30, 0xb41,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb2a, 0xb47, 0xb30, 0xb41, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3f, 0xb28, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3f, 0xb28, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2a, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3f, 0xb28, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e,
0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb3f, 0xb28, 0xb3f, 0xb15, 0xb4d,
0xb38, 0x20, 0xb26, 0xb3f, 0xb2a, 0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0x20, 0xb2a, 0xb3f, 0xb0f, 0xb30, 0xb47, 0x20,
0xb05, 0xb21, 0xb3c, 0xb47, 0x20, 0xb2e, 0xb3f, 0xb15, 0xb4d, 0xb71, 0xb47, 0xb32,
0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f,
0x20, 0xb2a, 0xb3f, 0xb0f, 0xb30, 0xb47, 0x20, 0xb05, 0xb21, 0xb3c, 0xb47, 0x20,
0xb2e, 0xb3f, 0xb15, 0xb4d, 0xb71, 0xb47, 0xb32, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb47,
0xb23, 0xb4d, 0xb1f, 0x20, 0xb2a, 0xb3f, 0xb0f, 0xb30, 0xb47, 0x20, 0xb05, 0xb21,
0xb3c, 0xb47, 0x20, 0xb2e, 0xb3f, 0xb15, 0xb4d, 0xb71, 0xb47, 0xb32, 0xb3e, 0xb28,
0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2a, 0xb3f, 0xb1f, 0xb15, 0xb07, 0xb30, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb2a, 0xb28, 0xb3e, 0xb2a, 0xb47, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb2a, 0xb4d,
0xb5f, 0xb19, 0xb4d, 0xb17, 0xb5f, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb30, 0xb3f, 0xb5f, 0xb41, 0xb28, 0xb3f, 0xb5f, 0xb28, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb30, 0xb24, 0xb47, 0xb30, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb38, 0xb15, 0xb3e, 0xb32, 0xb3f, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb38, 0xb15, 0xb3e, 0xb32, 0xb3f, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb15, 0xb3e, 0xb32, 0xb3f,
0xb28, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb38, 0xb3e, 0xb2e, 0xb06, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb3e, 0xb2e,
0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb38, 0xb3e, 0xb2e, 0xb06, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07,
0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb47, 0xb38, 0xb47, 0xb32, 0xb4d,
0xb38, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb3e,
0xb2a, 0xb41, 0xb30, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb38, 0xb32, 0xb2e, 0xb28, 0x20, 0xb26, 0xb3f, 0xb2a,
0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb26, 0xb15, 0xb3f, 0xb23, 0x20,
0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb5f, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb38, 0xb41, 0xb30, 0xb3f, 0xb28, 0xb3e, 0xb2e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb38, 0xb71, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb3e, 0xb39, 0xb3f,
0xb24, 0xb3f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb3e,
0xb07, 0xb2a, 0xb47, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07, 0xb1f, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb24, 0xb3e, 0xb1c, 0xb3f, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24,
0xb3e, 0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1f, 0xb15, 0xb47, 0xb32, 0xb3e,
0xb09, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb1f, 0xb19,
0xb4d, 0xb17, 0xb3e, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb1a, 0xb41, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb41,
0xb30, 0xb4d, 0xb15, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb41, 0xb30, 0xb4d, 0xb15, 0xb2e, 0xb47,
0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb41, 0xb30, 0xb4d,
0xb15, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb24, 0xb41, 0xb71,
0xb3e, 0xb32, 0xb41, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb30, 0xb41, 0xb17,
0xb4d, 0xb71, 0xb47, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb30, 0xb41, 0xb17,
0xb4d, 0xb71, 0xb47, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb30, 0xb41, 0xb17, 0xb4d, 0xb71, 0xb47, 0x20,
0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb1c,
0xb4d, 0xb71, 0xb47, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb09, 0xb1c, 0xb4d, 0xb71, 0xb47, 0xb15, 0xb3f, 0xb38, 0xb4d,
0xb24, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb09, 0xb1c, 0xb4d, 0xb71, 0xb47, 0xb15, 0xb3f, 0xb38,
0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c,
0xb47, 0xb32, 0xb3e, 0xb71, 0xb28, 0xb41, 0xb06, 0xb24, 0xb41, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb28, 0xb41, 0xb06, 0xb24, 0xb41, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb28, 0xb41,
0xb06, 0xb24, 0xb41, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb47, 0xb28, 0xb47, 0xb1c, 0xb41, 0xb0f, 0xb32, 0xb3e, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb4d, 0xb32, 0xb3e, 0xb26, 0xb3f, 0xb71, 0xb38,
0xb4d, 0xb24, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb4d, 0xb32, 0xb3e,
0xb26, 0xb3f, 0xb71, 0xb38, 0xb4d, 0xb24, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb4d, 0xb32, 0xb3e,
0xb26, 0xb3f, 0xb71, 0xb38, 0xb4d, 0xb24, 0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e,
0xb01, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb3e, 0xb32, 0xb17, 0xb17, 0xb4d,
0xb30, 0xb3e, 0xb21, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb3e, 0xb32, 0xb17,
0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb71, 0xb3e, 0xb32, 0xb17, 0xb17, 0xb4d,
0xb30, 0xb3e, 0xb21, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb47, 0xb15, 0x20, 0xb26, 0xb3f, 0xb2a, 0x20, 0xb2c, 0xb47,
0xb32, 0xb3e, 0xb71, 0xb3e, 0xb32, 0xb3f, 0xb38, 0x20, 0xb05, 0xb21, 0xb3c, 0xb47,
0x20, 0xb2a, 0xb41, 0xb1f, 0xb41, 0xb28, 0xb3e, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e,
0xb5f, 0xb3e, 0xb15, 0xb41, 0xb24, 0xb4d, 0xb38, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb5f, 0xb3e, 0xb15, 0xb41, 0xb24, 0xb4d, 0xb38, 0xb15, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb5f, 0xb3e,
0xb15, 0xb41, 0xb24, 0xb4d, 0xb38, 0xb15, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01,
0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb24, 0xb47, 0xb30,
0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e, 0xb5f,
0xb47, 0xb15, 0xb3e, 0xb24, 0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb24, 0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c, 0xb30,
0xb4d, 0xb17, 0x20, 0xb15, 0xb3e, 0xb30, 0xb3e, 0xb01, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0x1420, 0x1551, 0x1401, 0x14c2, 0x1428, 0x20, 0x142f, 0x152d, 0x146f, 0x1426, 0x1456,
0x140f, 0x1423, 0x20, 0x1401, 0x1403, 0x1422, 0x1438, 0x14c2, 0x1420
};

// GENERATED PART ENDS HERE

} // QtTimeZoneLocale

#undef LOCALE_TAGS

QT_END_NAMESPACE

#endif // QTIMEZONELOCALE_DATA_P_H
